(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.2'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      5927,        182]*)
(*NotebookOutlinePosition[      6618,        206]*)
(*  CellTagsIndexPosition[      6574,        202]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["BeginPackage[\"Fractals`\"];", "Input"],

Cell["(* Load required packages *)", "Text"],

Cell["Needs[\"Utilities`FilterOptions`\"]; ", "Input"],

Cell["(* Usage messages for exported symbols *)", "Text"],

Cell[TextData[
"mandelbrotRate::usage = \"mandelbrotRate[cx,cy,lim] computes the escape rate \
\\\nfor c = cx + I cy under the quadratic map z\[Rule]\\!\\(z\\^2\\)+c \
beginning with \\\nz=0.  The maximum number of iterations is given by lim, \
which must be an \\\ninteger.\"; "], "Input"],

Cell[TextData[
"juliaRate::usage = \"juliaRate[zx,zy,cx,cy,lim] computes the escape rate for \
z \\\n= zx + I zy under the quadratic map z\[Rule]\\!\\(z\\^2\\)+c where c = \
cx +I cy \\\nis the complex parameter characterizing the map.  The maximum \
number of \\\niterations is given by lim, which must be an integer.\"; "], 
  "Input"],

Cell[BoxData[
    \(\(DisplayKoch::usage = "\<DisplayKoch[polygon,n,opts] produces a Koch \
curve of \

order n based upon the list of points in polygon. By default, the triangle \

rule is applied in the outward direction assuming that the polygon is defined \
\

counterclockwise.
\n  Options: 
\n    KochType\[Rule]\"sieve\" applies the triangle-building rule in reverse.
\n    ShowIterations\[Rule]True can be used to display intermediate \
iterations.
\n    CloseFigure\[Rule]True connects the last point to the first to close \
the figure.\>";\)\)], "Input"],

Cell[BoxData[
    \(\(ShowIterations::usage = "\<a True|False option for DisplayKoch.\>";\)\
\)], "Input"],

Cell[BoxData[
    \(\(KochType::usage = "\<an option for DisplayKoch: 
\n  \"snowflake\" produces outward triangles 
\n  \"sieve\" produces inward triangles\>";\)\)], "Input"],

Cell[BoxData[
    \(\(CloseFigure::usage = "\<a True|False option for DisplayKoch that can \
be used \

when the list of points is not a closed polygon.\>";\)\)], "Input"],

Cell["Begin[\"`Private`\"];", "Input"],

Cell["(* Definitions *)", "Text"],

Cell["\<\
mandelbrotRate = 
  Compile[{cx, cy, {lim, _Integer}}, 
   Module[{c, z, ct = 0}, c = cx + I*cy; z = c; 
     While[Abs[z] < 2. && ct <= lim, ++ct; z = z*z + c; ]; ct]];\
\>", "Input"],

Cell["\<\
juliaRate = Compile[{zx, zy, cx, cy, {lim, _Integer}}, 
   Module[{c, z, ct = 0}, c = cx + I*cy; z = zx + I*zy; 
     While[Abs[z] < 2. && ct <= lim, ++ct; z = z*z + c; ]; ct]];\
\>", "Input"],

Cell["perp[vector:{x_, y_}] := {y, -x}/Sqrt[x^2 + y^2]", "Input"],

Cell["perp[pt1:{x1_, y1_}, pt2:{x2_, y2_}] := perp[pt1 - pt2]", "Input"],

Cell["\<\
newpt1[pt1:{x1_, y1_}, pt2:{x2_, y2_}] := pt1 + 1/3*(pt2 - pt1)\
\>", "Input"],

Cell["\<\
newpt2[pt1:{x1_, y1_}, pt2:{x2_, y2_}] := 
  1/2*(pt1 + pt2) + 1/Sqrt[12]*perp[pt1, pt2]*Sqrt[(pt2 - pt1) . (pt2 - pt1)]\
\
\>", "Input"],

Cell["\<\
newpt3[pt1:{x1_, y1_}, pt2:{x2_, y2_}] := pt1 + 2/3*(pt2 - pt1)\
\>", "Input"],

Cell["\<\
KochRule = Line[{pt1:{x1_, y1_}, pt2:{x2_, y2_}}] :> 
    {Line[{pt1, newpt1[pt1, pt2]}], 
     Line[{newpt1[pt1, pt2], newpt2[pt1, pt2]}], 
     Line[{newpt2[pt1, pt2], newpt3[pt1, pt2]}], 
     Line[{newpt3[pt1, pt2], pt2}]}; \
\>", "Input"],

Cell["KochSplit[x_] := x /. KochRule", "Input"],

Cell["\<\
DisplayKoch[polygon:{{_, _}..}, n_, opts___Rule] := 
  Module[{curve, type, showIterations, close, NestFunction, optKoch, 
    optPlot}, {type, showIterations, close} = 
     {KochType, ShowIterations, CloseFigure} /. {opts} /. 
      Options[DisplayKoch]; curve = 
     If[type == \"sieve\", Reverse[polygon], polygon]; 
    If[close, AppendTo[curve, First[curve]]]; 
    curve = Line /@ Partition[curve, 2, 1]; 
    NestFunction = If[showIterations, NestList, Nest]; 
    optPlot = FilterOptions[Graphics, opts]; 
    Show[Graphics[NestFunction[KochSplit, curve, n]], optPlot, AspectRatio -> \
Automatic]]\
\>", "Input"],

Cell["(* Options for exported functions *)", "Text"],

Cell["\<\
Options[DisplayKoch] = 
   {ShowIterations -> False, KochType -> \"snowflake\", CloseFigure -> \
False}; \
\>", "Input"],

Cell["(* Error conditions *)", "Text"],

Cell["End[]", "Input"],

Cell["(* Protect exported functions *)", "Text"],

Cell["Protect[juliaRate, mandelbrotRate, DisplayKoch]", "Input"],

Cell["EndPackage[]", "Input"]
},
FrontEndVersion->"5.2 for Microsoft Windows",
ScreenRectangle->{{0, 1400}, {0, 952}},
WindowToolbars->"EditBar",
WindowSize->{1392, 912},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
Magnification->1.5
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 45, 0, 41, "Input"],
Cell[1802, 53, 44, 0, 47, "Text"],
Cell[1849, 55, 54, 0, 41, "Input"],
Cell[1906, 57, 57, 0, 47, "Text"],
Cell[1966, 59, 290, 4, 110, "Input"],
Cell[2259, 65, 337, 5, 110, "Input"],
Cell[2599, 72, 564, 15, 224, "Input"],
Cell[3166, 89, 106, 2, 42, "Input"],
Cell[3275, 93, 175, 3, 94, "Input"],
Cell[3453, 98, 171, 4, 68, "Input"],
Cell[3627, 104, 38, 0, 41, "Input"],
Cell[3668, 106, 33, 0, 47, "Text"],
Cell[3704, 108, 194, 5, 110, "Input"],
Cell[3901, 115, 202, 4, 87, "Input"],
Cell[4106, 121, 65, 0, 41, "Input"],
Cell[4174, 123, 72, 0, 41, "Input"],
Cell[4249, 125, 88, 2, 41, "Input"],
Cell[4340, 129, 147, 4, 64, "Input"],
Cell[4490, 135, 88, 2, 41, "Input"],
Cell[4581, 139, 253, 6, 133, "Input"],
Cell[4837, 147, 47, 0, 41, "Input"],
Cell[4887, 149, 632, 13, 271, "Input"],
Cell[5522, 164, 52, 0, 47, "Text"],
Cell[5577, 166, 130, 4, 64, "Input"],
Cell[5710, 172, 38, 0, 47, "Text"],
Cell[5751, 174, 22, 0, 41, "Input"],
Cell[5776, 176, 48, 0, 47, "Text"],
Cell[5827, 178, 64, 0, 41, "Input"],
Cell[5894, 180, 29, 0, 41, "Input"]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

