(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.2'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[   2016324,      48784]*)
(*NotebookOutlinePosition[   2050269,      50008]*)
(*  CellTagsIndexPosition[   2050225,      50004]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Shooting Method for Bound-State Wave Functions", "Title",
  TextAlignment->Center,
  TextJustification->0],

Cell["\<\
This notebook introduces the shooting method for obtaining wave functions and \
energies for bound states of the Schr\[ODoubleDot]dinger equation.  We \
consider one-dimensional potentials in the body of the notebook and develop \
extensions to three dimensions in the exercises that follow.  We begin with a \
numerical demonstration of how imposition of a boundary condition at infinity \
leads to a discrete spectrum of energy eigenvalues.  Thus, we write a module \
for a one-sided shooting method in which we determine the energy that best \
satisfies a similar boundary condition imposed at a finite radius.  Next we \
develop a more robust two-sided method which integrates the differential \
equation inward from both sides and obtains the energy by matching the \
logarithmic derivative at an interior point.  Our specific example studies \
the energy splitting between even and odd solutions in a symmetric \
double-well potential.  The exercises extend these methods to \
three-dimensional problems.\
\>", "Abstract",
  TextAlignment->Left,
  TextJustification->0],

Cell[CellGroupData[{

Cell["Initialization", "Section"],

Cell[CellGroupData[{

Cell["Defaults and packages", "Subsection"],

Cell[BoxData[{
    \(\(ClearAll["\<Global`*\>"];\)\), "\n", 
    \(\(Off[General::spell, \ General::spell1];\)\), "\n", 
    \(Off[NDSolve::precw]\)}], "Input"],

Cell[BoxData[
    \(\($TextStyle = {FontFamily \[Rule] "\<Times\>", 
          FontSize \[Rule] 12};\)\)], "Input"],

Cell[BoxData[{
    \(\(Needs["\<Utilities`FilterOptions`\>"];\)\), "\n", 
    \(Needs["\<Graphics`\>"]\)}], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell[TextData[{
  "\tA wide variety of boundary-value problems can be solved using the ",
  StyleBox["shooting method",
    FontSlant->"Italic"],
  " in which one formulates a matching condition based upon trial solutions \
of the differential equation and uses root-finding techniques to adjust a \
parameter or eigenvalue such that the matching condition is satisfied.  We \
illustrate this technique by studying the energy eigenvalues for a \
one-dimensional Schr\[ODoubleDot]dinger equation based upon a symmetric \
double-well potential.  The Schr\[ODoubleDot]dinger equation has the form"
}], "Text"],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{"-", 
            FractionBox[
              RowBox[{
                SuperscriptBox["\[Psi]", "\[Prime]\[Prime]",
                  MultilineFunction->None], "[", "x", "]"}], \(2\ m\)]}], " ",
           "+", " ", \(V[x]\ \[Psi][x]\)}], 
        "\[LongEqual]", \(e\ \[Psi][x]\)}], 
      TraditionalForm]], "DisplayFormula"],

Cell[TextData[{
  "where ",
  StyleBox["m",
    FontSlant->"Italic"],
  " is the mass, ",
  Cell[BoxData[
      \(TraditionalForm\`V[x]\)]],
  " is the potential, ",
  Cell[BoxData[
      \(TraditionalForm\`e\)]],
  " is the energy, and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Psi][x]\)]],
  " is a wave function which for bound states must satisfy the boundary \
conditions"
}], "Text"],

Cell[BoxData[{
    \(lim\+\(x \[Rule] \(-\[Infinity]\)\)\[Psi][x] \[Equal] 0\), "\n", 
    \(lim\+\(x \[Rule] \(+\[Infinity]\)\)\[Psi][x] \[Equal] 
      0\)}], "DisplayFormula",
  FontFamily->"Times New Roman"],

Cell[TextData[{
  "and be everywhere continuous and differentiable.  In general these \
conditions can be satisfied only for discrete values of the energy, known as \
",
  StyleBox["eigenvalues",
    FontSlant->"Italic"],
  " or ",
  StyleBox["eigenenergies",
    FontSlant->"Italic"],
  ".  Eigenfunctions should be normalized to unit probability according to"
}], "Text"],

Cell[BoxData[
    \(\[Integral]\_\(-\[Infinity]\)\%\[Infinity]\(\[LeftBracketingBar]\[Psi][
                  x]\[RightBracketingBar]\^2\) \[DifferentialD]x\  = 
      1\)], "DisplayFormula",
  FontFamily->"Times New Roman"],

Cell[TextData[{
  "\tFor the purposes of illustration, we define the harmonic oscillator \
potential and a symmetric double-well potential in one dimension that \
resembles a harmonic oscillator for large ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " but has a bump in the middle.  The size of the bump has no particular \
significance."
}], "Text"],

Cell[BoxData[{
    \(\(ho[x_] := x\^2\/2;\)\), "\[IndentingNewLine]", 
    \(pot[x_] := x\^2 + 5  E\^\(-x\^2\)\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(bottom = FindMinimum[pot[x], {x, 1}];\)\), "\n", 
    \(emin = bottom\[LeftDoubleBracket]1\[RightDoubleBracket]; 
    x0 = x /. bottom\[LeftDoubleBracket]2\[RightDoubleBracket];\), "\n", 
    \({emin, x0}\)}], "Input"],

Cell[BoxData[
    \({2.6094379124341005`, 1.2686362411797578`}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[pot[x], {x, \(-5\), 5}, PlotRange \[Rule] {All, {2, 8}}, 
        Frame \[Rule] True, FrameLabel \[Rule] {"\<x\>", "\<V[x]\>"}, 
        PlotLabel \[Rule] "\<a double-well potential\>"];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0952381 -0.206011 0.103006 [
[.11905 -0.0125 -9.15625 -15.6875 ]
[.11905 -0.0125 9.15625 0 ]
[.30952 -0.0125 -9.15625 -15.6875 ]
[.30952 -0.0125 9.15625 0 ]
[.5 -0.0125 -5 -15.6875 ]
[.5 -0.0125 5 0 ]
[.69048 -0.0125 -5 -15.6875 ]
[.69048 -0.0125 5 0 ]
[.88095 -0.0125 -5 -15.6875 ]
[.88095 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -5.0625 -31.375 ]
[.5 -0.0125 5.0625 -15.6875 ]
[-0.0125 .10301 -10 -7.84375 ]
[-0.0125 .10301 0 7.84375 ]
[-0.0125 .20601 -10 -7.84375 ]
[-0.0125 .20601 0 7.84375 ]
[-0.0125 .30902 -10 -7.84375 ]
[-0.0125 .30902 0 7.84375 ]
[-0.0125 .41202 -10 -7.84375 ]
[-0.0125 .41202 0 7.84375 ]
[-0.0125 .51503 -10 -7.84375 ]
[-0.0125 .51503 0 7.84375 ]
[-0.0125 .61803 -10 -7.84375 ]
[-0.0125 .61803 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -25.6875 -13.3125 ]
[-0.01875 .30902 -10 13.3125 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -58.9375 0 ]
[.5 .63053 58.9375 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.11905 0 m
.11905 .00625 L
s
gsave
.11905 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.30952 0 m
.30952 .00625 L
s
gsave
.30952 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.5 0 m
.5 .00625 L
s
gsave
.5 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.69048 0 m
.69048 .00625 L
s
gsave
.69048 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.88095 0 m
.88095 .00625 L
s
gsave
.88095 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.125 Mabswid
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.07143 0 m
.07143 .00375 L
s
.02381 0 m
.02381 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -66.0625 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
69.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .10301 m
.00625 .10301 L
s
gsave
-0.0125 .10301 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
0 .20601 m
.00625 .20601 L
s
gsave
-0.0125 .20601 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
0 .30902 m
.00625 .30902 L
s
gsave
-0.0125 .30902 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
0 .41202 m
.00625 .41202 L
s
gsave
-0.0125 .41202 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
0 .51503 m
.00625 .51503 L
s
gsave
-0.0125 .51503 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7) show
1.000 setlinewidth
grestore
0 .61803 m
.00625 .61803 L
s
gsave
-0.0125 .61803 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .0824 m
.00375 .0824 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .16481 m
.00375 .16481 L
s
0 .18541 m
.00375 .18541 L
s
0 .22661 m
.00375 .22661 L
s
0 .24721 m
.00375 .24721 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .32962 m
.00375 .32962 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .49443 m
.00375 .49443 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .57683 m
.00375 .57683 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -92.1563 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
74.3125 11.8438 translate
90 rotate
-74.3125 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(V) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
71.625 15.813 moveto
(@) show
75.563 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
81.688 15.813 moveto
(D) show
85.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.11905 .61178 m
.11905 .61803 L
s
.30952 .61178 m
.30952 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.69048 .61178 m
.69048 .61803 L
s
.88095 .61178 m
.88095 .61803 L
s
.125 Mabswid
.16667 .61428 m
.16667 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.2619 .61428 m
.2619 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.7381 .61428 m
.7381 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.07143 .61428 m
.07143 .61803 L
s
.02381 .61428 m
.02381 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.97619 .61428 m
.97619 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 0 m
1 0 L
s
.99375 .10301 m
1 .10301 L
s
.99375 .20601 m
1 .20601 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .41202 m
1 .41202 L
s
.99375 .51503 m
1 .51503 L
s
.125 Mabswid
.99625 .0206 m
1 .0206 L
s
.99625 .0412 m
1 .0412 L
s
.99625 .0618 m
1 .0618 L
s
.99625 .0824 m
1 .0824 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .14421 m
1 .14421 L
s
.99625 .16481 m
1 .16481 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .22661 m
1 .22661 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26781 m
1 .26781 L
s
.99625 .28842 m
1 .28842 L
s
.99625 .32962 m
1 .32962 L
s
.99625 .35022 m
1 .35022 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .39142 m
1 .39142 L
s
.99625 .43262 m
1 .43262 L
s
.99625 .45322 m
1 .45322 L
s
.99625 .47383 m
1 .47383 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .53563 m
1 .53563 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57683 m
1 .57683 L
s
.99625 .59743 m
1 .59743 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
.5 0 m
.5 .61803 L
s
gsave
.5 .63053 -119.938 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(a) show
71.375 15.813 moveto
(double) show
103.938 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
111.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(well) show
135.125 15.813 moveto
(potential) show
176.875 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.23115 .61803 m
.26171 .43981 L
.28158 .33843 L
.30316 .24121 L
.32216 .16893 L
.33296 .13461 L
.34309 .10772 L
.35264 .08759 L
.35756 .0794 L
.36292 .07226 L
.36584 .06919 L
.36855 .06686 L
.37114 .06512 L
.37247 .06441 L
.3739 .06379 L
.37532 .06332 L
.37662 .06301 L
.37786 .06284 L
.37917 .06278 L
.38035 .06283 L
.38145 .06296 L
.38266 .06322 L
.38395 .06361 L
.38517 .06409 L
.38651 .06475 L
.38924 .06648 L
.39176 .06857 L
.39413 .07095 L
.39945 .07775 L
.40519 .08725 L
.41569 .10993 L
.42531 .13578 L
.44584 .20002 L
.45585 .23154 L
.46494 .25769 L
.47378 .27923 L
.47843 .28855 L
.48336 .29668 L
.48587 .30007 L
.48863 .3032 L
.49117 .30549 L
.49349 .3071 L
.49471 .30775 L
.49587 .30824 L
.49709 .30863 L
.49778 .30879 L
.49843 .3089 L
.4996 .30901 L
.50066 .309 L
.50189 .30885 L
Mistroke
.50305 .3086 L
.50421 .30821 L
.50546 .30766 L
.50772 .30632 L
.51039 .30415 L
.51283 .30162 L
.51814 .2944 L
.52316 .28553 L
.54165 .23907 L
.5599 .18148 L
.58003 .12092 L
.58957 .09776 L
.59984 .0788 L
.60551 .07136 L
.60798 .06881 L
.61069 .06654 L
.61326 .06487 L
.61565 .06376 L
.61697 .06332 L
.61817 .06303 L
.61884 .06292 L
.61956 .06283 L
.62086 .06278 L
.6221 .06284 L
.62323 .06299 L
.62453 .06328 L
.62577 .06367 L
.62796 .06463 L
.63035 .06607 L
.63298 .06811 L
.63577 .07081 L
.64078 .07699 L
.65078 .09418 L
.66184 .12011 L
.68116 .18051 L
.70201 .26313 L
.74311 .46596 L
Mfstroke
.74311 .46596 m
.76865 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00j03oool01@2oM0000000001d_`3oool0gia80080000000<004RL0=ooo`3oool0_P3oool0
0>P0oooo00P0oooO09a8M02oool0oooo0?oog`2LB00004RL0=oool00oooo003Y0?ooo`060?oo_`1d
B9`0gooo0;md000007@0_ooo`@3oool00>X0oooo00@0omnL07A8B0000000M;oo`P3oool00>X0oooo
00<0oono07@00018W=l0``3oool00>T0oooo00D0oooO09a80000B9`0W7A809cOo`320?ooo`00j@3o
ool01P2oM000001809cOo`3ooml0W4Qd0;oool40oooo003W0?ooo`090?oog`2LB000000000000000
M;l0okmd0000000007@0_ooo0<00oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo001o0?ooo`040?oog`2LB00004RL0=oo
oc80oooo00<0_g@0000000000000100000000`18W=l0oooo0?ooo`0/0?ooo`050?oOW01800000000
0000M02oool0<P3oool00`2oM0000000000000040000000304RLg`3oool0oooo03@0oooo00@0oooO
09a80000B9`0goooC03oool007l0oooo00@0oooO09a80000B9`0gooo<`3oool00`3OW4P000000000
0003000000030018W03Oool0oooo02/0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo0380
oooo00<0gia80000000000000`0000000`00B9`0gooo0?ooo`0d0?ooo`040?oog`2LB00004RL0=oo
od`0oooo001o0?ooo`040?oog`2LB00004RL0=oooc@0oooo00<0gia804RLg`3oool00P3oool00`3O
W7@0W=oo0?ooo`0[0?ooo`030;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool0<P3oool00`3O
W4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo03@0oooo00@0oooO09a80000B9`0goooC03oool0
07/0oooo00<0_g@00000000000001@0000000`1d_ol0oooo0?ooo`0b0?ooo`030=nLB01d_ol0oooo
02h0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`0c0?ooo`030=nLB01d_ol0oooo
03@0oooo00<0_g@00000000000001@0000000`1d_ol0oooo0?ooo`190?ooo`00K`3oool00`3ooml0
W4P0000000070000000404RLg`3oool0omnL07BLg`80oooo00@0oooO09a80000B9`0gooo9@3oool0
0`3ooml0W4P0000000070000000304RLg`3oool0oooo00@0oooo00<0_gA809cOo`3oool0;@3oool0
0`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo03@0oooo00<0_gA809cOo`3oool0<`3oool0
203ogi`0M9cO0?ooo`3oool0oooO09a80000B9`0goooC03oool007`0oooo00L0gia807Boo`3oool0
oooO09a80000B9`0gooo03H0oooo00<0oooO09a8M02oool0;@3oool00`3ogi`0B00007Boo`030?oo
o`030?noM00004P0W=oo03@0oooo00<0oooO09a8M02oool0=03oool01`3OW4P0M;oo0?ooo`3ooml0
W4P00018W03Oool0C03oool007`0oooo00L0oooO09a8M02oool0oooO09a80000B9`0gooo03L0oooo
00<0okmd001d_`3oool0;03oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo03D0oooo
00<0okmd001d_`3oool0<`3oool01`3ooml0W4Qd0;ooo`3ooml0W4P00018W03Oool0C03oool007d0
oooo00H0omnL04Qd_`3ooml0W4P00018W03Ooolg0?ooo`030?oo_`1d04P0W=oo02`0oooo00<0omnL
04P0001d_ol00`3oool00`3o_g@0001809cOo`0e0?ooo`030?oo_`1d04P0W=oo03@0oooo00H0omnL
04Qd_`3ooml0W4P00018W03Ooom<0?ooo`00OP3oool01@3OW4P0M;oO09a80000B9`0gooo03L0oooo
00<0oono07@00018W=l0;03oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo03D0oooo
00<0oono07@00018W=l0=@3oool01@3OW4P0M;oO09a80000B9`0gooo04`0oooo001n0?ooo`050?oo
g`2LB4P0B4P00018W03Oool0<P3oool00`3o_i`0W=oo0?ooo`020?ooo`030?oOW0180000B9cO02d0
oooo00<0gia80018W03Oool00P3oool00`2oM00007Bo0?ooo`0`0?ooo`030?noW02Lgol0oooo0080
oooo00<0omnL04P00018W=l0=@3oool01@3ooml0W4Q804Q80000B9`0gooo04`0oooo001o0?ooo`04
0?oOW018000004RL0=oooc80oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooobd0oooo
00L0oono07@0001d_ol0oooo0?noM00007@0_ooo0340oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d
0000001d0;ooocH0oooo00@0omnL04P00000B9`0goooC03oool00800oooo00<0gia80018W03Oool0
<`3oool01P3ooml0W4P0000000000000001d0;ooobl0oooo00D0oono07@000000000001d0;ooo`0c
0?ooo`060?oog`2LB00000000000000007@0_ooo>03oool00`3OW4P004RL0=ooo`1<0?ooo`00o`3o
oona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3o
oona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00E@3ooooo000003800000:P3oool005D0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo09D0oooo00<00000
0?ooo`3oool0U@3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`2E0?oo
o`030000003oool0oooo09D0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3o
ool0U@3oool00`000000oooo0?ooo`2E0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000
003oool0oooo09D0oooo00<000000?ooo`3oool0U@3oool00`000000oooo0?ooo`0X0?ooo`00E@3o
ool2000009H0oooo00<000000?ooo`3oool0U03oool2000002X0oooo001E0?ooo`030000003oool0
oooo09D0oooo00<000000?ooo`3oool0U@3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`00
0000oooo0?ooo`2E0?ooo`030000003oool0oooo09D0oooo00<000000?ooo`3oool0:03oool005D0
oooo00<000000?ooo`3oool0U@3oool00`000000oooo0?ooo`2E0?ooo`030000003oool0oooo02P0
oooo001E0?ooo`030000003oool0oooo09D0oooo00<000000?ooo`3oool0U@3oool00`000000oooo
0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`2E0?ooo`030000003oool0oooo09D0oooo00<0
00000?ooo`3oool0:03oool005D0oooo0P00002F0?ooo`030000003oool0oooo09@0oooo0P00000Z
0?ooo`00E@3oool00`000000oooo0?ooo`2E0?ooo`030000003oool0oooo09D0oooo00<000000?oo
o`3oool0:03oool005D0oooo00<000000?ooo`3oool0U@3oool00`000000oooo0?ooo`2E0?ooo`03
0000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo09D0oooo00<000000?ooo`3oool0
U@3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`2E0?ooo`030000003o
ool0oooo09D0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0U@3oool0
0`000000oooo0?ooo`2E0?ooo`030000003oool0oooo02P0oooo001E0?ooo`800000UP3oool00`00
0000oooo0?ooo`2D0?ooo`800000:P3oool005D0oooo00<000000?ooo`3oool0KP3oool600000240
oooo00<000000?ooo`3oool07`3oool600000700oooo00<000000?ooo`3oool0:03oool005D0oooo
00<000000?ooo`3oool0K03oool2000000D0oooo0P00000P0?ooo`030000003oool0oooo01h0oooo
0P0000050?ooo`030000003oool0oooo06d0oooo00<000000?ooo`3oool0:03oool005D0oooo00<0
00000?ooo`3oool0J`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01d0oooo00<0
00000?ooo`3oool07@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo06`0oooo00<0
00000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0J`3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`08
0?ooo`030000003oool0oooo06/0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?oo
o`3oool0JP3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo01`0oooo00<000000?oo
o`3oool06`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo06X0oooo00<000000?oo
o`3oool0:03oool005D0oooo00<000000?ooo`3oool0JP3oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo01/0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo06T0oooo00<000000?ooo`3oool0:03oool005D0oooo0P00001Z0?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0J03oool2000002X0oooo00150?ooo`030?oo_`1d
000000000080000000<0M;oo0?ooo`3oool0203oool00`000000oooo0?ooo`1X0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0J03oool00`000000oooo0?ooo`0X0?ooo`00A@3oool0
0`3ogi`0B00004RLg`020?ooo`030;md0018W=l0oooo00P0oooo00<000000?ooo`3oool0I`3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo06P0oooo00<000000?ooo`3oool0:03oool0
04X0oooo00<0oono07@0B02Lgol0203oool00`000000oooo0?ooo`1W0?ooo`030000003oool0oooo
00l0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0I`3oool00`000000oooo0?ooo`0X0?ooo`00BP3oool00`3ookl0
M00007Boo`080?ooo`030000003oool0oooo06H0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`1W0?ooo`030000003oool0oooo02P0oooo001:0?ooo`030?noM0000000M;oo00P0oooo
00<000000?ooo`3oool0IP3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01P0oooo
00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo06H0oooo
00<000000?ooo`3oool0:03oool004T0oooo00@0omnL04P0000004P0W=oo203oool3000006D0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`1T0?ooo`<00000:P3oool004L0oooo00D0
okmd000000000000001809cOo`090?ooo`030000003oool0oooo06D0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`1V0?ooo`030000003oool0oooo02P0oooo00190?ooo`030;md0000
M;l0oooo00T0oooo00<000000?ooo`3oool0I@3oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0oooo01L0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0D0?ooo`030000003o
ool0oooo06D0oooo00<000000?ooo`3oool0:03oool004X0oooo00<0_g@004RLg`3oool0203oool0
0`000000oooo0?ooo`1T0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0I@3oool0
0`000000oooo0?ooo`0X0?ooo`00A@3oool00`3ooml0W9bo0?ooo`020?ooo`030=nLB00007@0_ooo
00P0oooo00<000000?ooo`3oool0I03oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo
06@0oooo00<000000?ooo`3oool0:03oool004H0oooo00H0okmd04RLg`3oool0oono07@00000M;l9
0?ooo`030000003oool0oooo06<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0F
0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`1T
0?ooo`030000003oool0oooo02P0oooo00170?ooo`050=nLB000000000000000M02oool02@3oool2
000006@0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
oooo01D0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`1S0?ooo`800000:P3oool0
05D0oooo00<000000?ooo`3oool0H`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
06<0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0HP3oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo06<0oooo00<000000?ooo`3oool0:03oool005D0oooo
00<000000?ooo`3oool0HP3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo06<0oooo
00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0HP3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo0680oooo00<000000?ooo`3oool0:03oool005D0oooo00<00000
0?ooo`3oool0H@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool04`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0680oooo00<00000
0?ooo`3oool0:03oool005D0oooo0P00001R0?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool0H@3oool2000002X0oooo001E0?ooo`030000003oool0oooo0640oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool0703oool00`000000oooo0?ooo`1Q0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000
003oool0oooo0640oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo0180oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`1Q0?ooo`030000
003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo0600oooo00<000000?ooo`3oool07@3o
ool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0703o
ool00`000000oooo0?ooo`1Q0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0
oooo0600oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo0180oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`1P0?ooo`030000003oool0
oooo02P0oooo001E0?ooo`030000003oool0oooo0600oooo00<000000?ooo`3oool07P3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool07P3oool00`00
0000oooo0?ooo`1P0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo0600
oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`1P0?ooo`030000003oool0oooo02P0
oooo001E0?ooo`800000H03oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0140oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo05h0oooo
0P00000Z0?ooo`00E@3oool00`000000oooo0?ooo`1O0?ooo`030000003oool0oooo0200oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0200oooo00<0
00000?ooo`3oool0G`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`1O
0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0G`3oool00`000000oooo0?ooo`0X
0?ooo`00E@3oool00`000000oooo0?ooo`1N0?ooo`030000003oool0oooo0240oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0240oooo00<000000?oo
o`3oool0GP3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`1N0?ooo`03
0000003oool0oooo0280oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0?0?ooo`03
0000003oool0oooo0280oooo00<000000?ooo`3oool0GP3oool00`000000oooo0?ooo`0X0?ooo`00
E@3oool00`000000oooo0?ooo`1N0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0
GP3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool2000005l0oooo00<000000?ooo`3oool08`3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool08P3o
ool00`000000oooo0?ooo`1M0?ooo`800000:P3oool004T0oooo00@0oooO09a80000B9`0gooo203o
ool00`000000oooo0?ooo`1M0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0G@3o
ool00`000000oooo0?ooo`0X0?ooo`00B@3oool0103ooml0W4P00018W03Oool80?ooo`030000003o
ool0oooo05d0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0>0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`1M0?ooo`030000003o
ool0oooo02P0oooo00190?ooo`040?oog`2LB00004RL0=ooo`P0oooo00<000000?ooo`3oool0G@3o
ool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo05d0oooo00<000000?ooo`3oool0:03o
ool004D0oooo00<0_g@00000000000001@0000000`1d_ol0oooo0?ooo`050?ooo`030000003oool0
oooo05d0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`1M0?ooo`030000003oool0
oooo02P0oooo00150?ooo`080?oOW01dW=l0oooo0?ooo`3ooml0W4P00018W03Oool80?ooo`030000
003oool0oooo05`0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`1L0?ooo`030000
003oool0oooo02P0oooo00160?ooo`070=nLB01d_ol0oooo0?oog`2LB00004RL0=ooo`080?ooo`<0
0000G03oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo05X0oooo0`00000Z0?ooo`00
AP3oool01`3ooml0W4Qd0;ooo`3ooml0W4P00018W03Oool0203oool00`000000oooo0?ooo`1L0?oo
o`030000003oool0oooo02L0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo02H0oooo00<000000?ooo`3oool0G03oool00`000000oooo0?ooo`0X0?oo
o`00A`3oool01P3ogi`0B7Bo0?oog`2LB00004RL0=ooo`P0oooo00<000000?ooo`3oool0F`3oool0
0`000000oooo0?ooo`0X0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`0W0?ooo`030000003oool0oooo05`0oooo00<000000?ooo`3oool0:03oool0
04P0oooo00D0gia807Bog`2LB00004RL0=ooo`080?ooo`030000003oool0oooo05/0oooo00<00000
0?ooo`3oool0:03oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool0:03oool00`000000oooo0?ooo`1K0?ooo`030000003oool0oooo02P0oooo00180?oo
o`050?oog`2LB4P0B4P00018W03Oool0203oool00`000000oooo0?ooo`1K0?ooo`030000003oool0
oooo02T0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo02P0oooo00<000000?ooo`3oool0F`3oool00`000000oooo0?ooo`0X0?ooo`00B@3oool0103o
gi`0B0000018W03Oool80?ooo`030000003oool0oooo05/0oooo00<000000?ooo`3oool0:@3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0:@3oool0
0`000000oooo0?ooo`1K0?ooo`030000003oool0oooo02P0oooo001:0?ooo`030=nLB000B9`0gooo
00P0oooo0P00001K0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0FP3oool20000
02X0oooo001E0?ooo`030000003oool0oooo05X0oooo00<000000?ooo`3oool0:`3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0:P3oool00`000000
oooo0?ooo`1J0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo05X0oooo
00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`1J0?ooo`030000003oool0oooo02P0oooo
001E0?ooo`030000003oool0oooo05X0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?oo
o`1J0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo05T0oooo00<00000
0?ooo`3oool0;03oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool0:`3oool00`000000oooo0?ooo`1J0?ooo`030000003oool0oooo02P0oooo001E0?oo
o`030000003oool0oooo05T0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`1I0?oo
o`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo05T0oooo00<000000?ooo`3o
ool0;@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool0;@3oool00`000000oooo0?ooo`1I0?ooo`030000003oool0oooo02P0oooo001E0?ooo`800000
FP3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo05P0oooo0P00000Z0?ooo`00E@3o
ool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0F@3o
ool00`000000oooo0?ooo`0X0?ooo`00;P3oool00`3OW7@0W=oo0?ooo`0T0?ooo`030000003oool0
oooo05P0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo00P0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`1H0?ooo`030000003oool0
oooo02P0oooo000^0?ooo`030=nLB000M;l0oooo02@0oooo00<000000?ooo`3oool0F03oool00`00
0000oooo0?ooo`0_0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`0`0?ooo`030000003oool0oooo05P0oooo00<000000?ooo`3oool0:03oool002h0
oooo00D0gia8000000000000000007Boo`0R0?ooo`030000003oool0oooo05P0oooo00<000000?oo
o`3oool0;`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool0<03oool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo02P0oooo000^0?ooo`03
0=nLB0000000000000<0000000<0001d0;ooo`3oool07P3oool00`000000oooo0?ooo`1H0?ooo`03
0000003oool0oooo0300oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo0300oooo00<000000?ooo`3oool0F03oool00`000000oooo0?ooo`0X0?ooo`00
;P3oool01P3OW7@0W=oo0?ooo`3oool0omnL04P000<0000000<004RL0=ooo`3oool06`3oool20000
05P0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`1G0?ooo`800000:P3oool003D0
oooo00<0okmd0000000000000P0000000`00M;l0oooo0?ooo`0H0?ooo`030000003oool0oooo05L0
oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`1G0?ooo`030000003oool0oooo02P0
oooo000h0?ooo`040?noM0000000001d0;oooaT0oooo00<000000?ooo`3oool0E`3oool00`000000
oooo0?ooo`0b0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`0c0?ooo`030000003oool0oooo05L0oooo00<000000?ooo`3oool0:03oool003D0oooo
00@0okmd00000000000007Bo703oool00`000000oooo0?ooo`1G0?ooo`030000003oool0oooo03<0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo03<0
oooo00<000000?ooo`3oool0E`3oool00`000000oooo0?ooo`0X0?ooo`00;P3oool02@3OW7@0W=oo
0?ooo`3oool0omnL04P00000000004RL0=ooo`0N0?ooo`030000003oool0oooo05L0oooo00<00000
0?ooo`3oool0<`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool0=03oool00`000000oooo0?ooo`1G0?ooo`030000003oool0oooo02P0oooo000^0?oo
o`060=nLB000000000000000000007@0_ooo8@3oool00`000000oooo0?ooo`1F0?ooo`030000003o
ool0oooo03D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo03@0oooo00<000000?ooo`3oool0E`3oool00`000000oooo0?ooo`0X0?ooo`00;P3oool0
0`3OW4P0000007Boo`0T0?ooo`800000E`3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0e0?ooo`030000003oool0
oooo05D0oooo0P00000Z0?ooo`00;P3oool00`3OW7@0W=oo0?ooo`0D0?ooo`030?oog`2LB0000000
0080000000<0M;oo0?ooo`3oool0203oool00`000000oooo0?ooo`1F0?ooo`030000003oool0oooo
03H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
03H0oooo00<000000?ooo`3oool0EP3oool00`000000oooo0?ooo`0X0?ooo`00A@3oool00`3ooml0
W4Qd0;ooo`020?ooo`030;md001d_ol0oooo00P0oooo00<000000?ooo`3oool0EP3oool00`000000
oooo0?ooo`0f0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`0f0?ooo`030000003oool0oooo05H0oooo00<000000?ooo`3oool0:03oool004X0oooo
00<0oooO09a8M02oool0203oool00`000000oooo0?ooo`1E0?ooo`030000003oool0oooo03P0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo03L0oooo
00<000000?ooo`3oool0EP3oool00`000000oooo0?ooo`0X0?ooo`00;@3oool00`3ogi`0B0000000
000>000000030000M02oool0oooo00X0oooo00<0_g@007Boo`3oool01`3oool00`000000oooo0?oo
o`1E0?ooo`030000003oool0oooo03T0oooo00D000000?ooo`3oool0oooo000000030?ooo`030000
003oool0oooo03T0oooo00<000000?ooo`3oool0E@3oool00`000000oooo0?ooo`0X0?ooo`00;@3o
ool00`3ogi`0M9cO0?ooo`0>0?ooo`030;mdM02oool0oooo00T0oooo00<0oooO09a80018W=l0203o
ool00`000000oooo0?ooo`1E0?ooo`030000003oool0oooo03X0oooo00@000000?ooo`3oool00000
0P3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo05D0oooo00<000000?ooo`3oool0
:03oool002d0oooo00<0omnL09boo`3oool03P3oool00`3OW9`0_ooo0?ooo`090?ooo`030=nLB000
0000M;oo00P0oooo0`00001E0?ooo`030000003oool0oooo03/0oooo1@00000m0?ooo`030000003o
ool0oooo05<0oooo0`00000Z0?ooo`00;@3oool00`3ogi`0W;oo0?ooo`0>0?ooo`030=nLW02oool0
oooo00D0oooo00L0gmoO0?ooo`3ookl0M0000000000007@0_ooo00P0oooo00<000000?ooo`3oool0
E@3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool0
E@3oool00`000000oooo0?ooo`0X0?ooo`00AP3oool01P3OW4P00000000000000000001809cOo`T0
oooo00<000000?ooo`3oool0E03oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo03d0
oooo00<000000?ooo`3oool0E@3oool00`000000oooo0?ooo`0X0?ooo`00<`3oool00`2oM7@0_ooo
0?ooo`040?ooo`030;mdM02oool0oooo00T0oooo00@0oono07@000000000B9cO2`3oool00`000000
oooo0?ooo`1D0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0?P3oool00`000000
oooo0?ooo`1D0?ooo`030000003oool0oooo02P0oooo000c0?ooo`030;md0018W=l0oooo00<0oooo
00<0gia80000M02oool02`3oool00`2oM000M;oo0?ooo`0;0?ooo`030000003oool0oooo05@0oooo
00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo05@0oooo
00<000000?ooo`3oool0:03oool003<0oooo00T0_g@00000000007@0_ooo0?oog`2LB000B9cO0=nL
W02oool02`3oool01P3ogi`0B000000000000000001809cOo`P0oooo00<000000?ooo`3oool0E03o
ool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0E03o
ool00`000000oooo0?ooo`0X0?ooo`00<`3oool01P2oW9`0gmnL04P000000000001809cOo`l0oooo
00D0_g@0000000000000000004RLg`080?ooo`030000003oool0oooo05@0oooo00<000000?ooo`3o
ool0?P3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo05@0oooo00<000000?ooo`3o
ool0:03oool003<0oooo00<0_gAd0;oOW01800000P000000100004P0W=oo0=nLW02ooolI0?ooo`80
0000E03oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3o
ool0D`3oool2000002X0oooo000c0?ooo`030;md0000B9`0gooo0080oooo00@0_g@00000000007@0
_ooo6@3oool00`000000oooo0?ooo`1C0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3o
ool0?`3oool00`000000oooo0?ooo`1C0?ooo`030000003oool0oooo02P0oooo000c0?ooo`030;md
B02Lgol0oooo00<0oooo00<0okmd0000M02oool06@3oool00`000000oooo0?ooo`1C0?ooo`030000
003oool0oooo03l0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`1C0?ooo`030000
003oool0oooo02P0oooo000c0?ooo`030;nLW03Oool0oooo00@0oooo00<0gibL0;ooo`3oool0603o
ool00`000000oooo0?ooo`1C0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0?`3o
ool00`000000oooo0?ooo`1C0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0
oooo05<0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0
oooo05<0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0DP3oool00`00
0000oooo0?ooo`100?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0D`3oool00`00
0000oooo0?ooo`0X0?ooo`00;@3oool00`3ogi`0W;oo0?ooo`0>0?ooo`030=nLW02oool0oooo01@0
oooo0P00001C0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool0@03oool00`000000
oooo0?ooo`1A0?ooo`800000:P3oool002d0oooo00<0omnL09boo`3oool03P3oool00`3OW9`0_ooo
0?ooo`0D0?ooo`030000003oool0oooo0580oooo00<000000?ooo`3oool0@03oool00`000000oooo
0?ooo`100?ooo`030000003oool0oooo0580oooo00<000000?ooo`3oool0:03oool002d0oooo00<0
omnL07BLg`3oool03P3oool00`2oM7@0_ooo0?ooo`0D0?ooo`030000003oool0oooo0580oooo00<0
00000?ooo`3oool0@03oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo0580oooo00<0
00000?ooo`3oool0:03oool002d0oooo00<0omnL04P0000000003P0000000`0007@0_ooo0?ooo`0D
0?ooo`030000003oool0oooo0580oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`10
0?ooo`030000003oool0oooo0580oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?oo
o`3oool0D@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo0400oooo00<000000?oo
o`3oool0DP3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`1A0?ooo`03
0000003oool0oooo0440oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`1A0?ooo`03
0000003oool0oooo02P0oooo001E0?ooo`800000DP3oool00`000000oooo0?ooo`110?ooo`030000
003oool0oooo0440oooo00<000000?ooo`3oool0D03oool2000002X0oooo001E0?ooo`030000003o
ool0oooo0540oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`110?ooo`030000003o
ool0oooo0540oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0D@3oool0
0`000000oooo0?ooo`110?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0D@3oool0
0`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`1@0?ooo`030000003oool0oooo
0480oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`1@0?ooo`030000003oool0oooo
02P0oooo001E0?ooo`030000003oool0oooo0500oooo00<000000?ooo`3oool0@P3oool00`000000
oooo0?ooo`120?ooo`030000003oool0oooo0500oooo00<000000?ooo`3oool0:03oool005D0oooo
00<000000?ooo`3oool0D03oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo0480oooo
00<000000?ooo`3oool0D03oool00`000000oooo0?ooo`0X0?ooo`00E@3oool200000540oooo00<0
00000?ooo`3oool0@P3oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo04l0oooo0P00
000Z0?ooo`00E@3oool00`000000oooo0?ooo`1@0?ooo`030000003oool0oooo0480oooo00<00000
0?ooo`3oool0@P3oool00`000000oooo0?ooo`1@0?ooo`030000003oool0oooo02P0oooo00170?oo
o`050?oOW018000000000000001d_ol02@3oool00`000000oooo0?ooo`1@0?ooo`030000003oool0
oooo0480oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`1?0?ooo`030000003oool0
oooo02P0oooo00160?ooo`070?oOW0180000B9cO0?ooo`3ookl0M00007Boo`080?ooo`030000003o
ool0oooo04l0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`130?ooo`030000003o
ool0oooo04l0oooo00<000000?ooo`3oool0:03oool004D0oooo00<0oooO09a80000M;l00`3oool0
0`3o_g@004RL0=ooo`070?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0@`3oool0
0`000000oooo0?ooo`130?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0:03oool0
04D0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`070?ooo`030000003oool0oooo
04l0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo
04l0oooo00<000000?ooo`3oool0:03oool004D0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0
001809cOo`070?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0@`3oool00`000000
oooo0?ooo`130?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0:03oool004D0oooo
00<0omnL04P0001d_ol00`3oool00`2oM000001809cOo`070?ooo`<00000C`3oool00`000000oooo
0?ooo`130?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool0C03oool3000002X0oooo
00150?ooo`080?oo_`1d0000000004RLg`3oool0okmd00000000M;l80?ooo`030000003oool0oooo
04h0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo
04h0oooo00<000000?ooo`3oool0:03oool004H0oooo00<0_g@00018W02oW4P00P0000000`00B9`0
gooo0?ooo`070?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0A03oool00`000000
oooo0?ooo`140?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0:03oool004H0oooo
00<0omnL04P0001d_ol0303oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo04@0oooo
00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo02P0oooo
00170?ooo`030=nLB00004P0W=oo00/0oooo00<000000?ooo`3oool0CP3oool00`000000oooo0?oo
o`140?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool0CP3oool00`000000oooo0?oo
o`0X0?ooo`00B03oool00`3o_g@00000001d_`0:0?ooo`030000003oool0oooo04h0oooo00<00000
0?ooo`3oool0A03oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo04d0oooo00<00000
0?ooo`3oool0:03oool004T0oooo00D0oooO09a800000000001d0;ooo`070?ooo`800000CP3oool0
0`000000oooo0?ooo`150?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool0C03oool2
000002X0oooo001E0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0A@3oool00`00
0000oooo0?ooo`150?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0:03oool005D0
oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo04D0
oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo
0?ooo`1=0?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool0AP3oool00`000000oooo
0?ooo`1<0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo04d0oooo00<0
00000?ooo`3oool0A@3oool00`000000oooo0?ooo`160?ooo`030000003oool0oooo04`0oooo00<0
00000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0C03oool00`000000oooo0?ooo`16
0?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool0C03oool00`000000oooo0?ooo`0X
0?ooo`00E@3oool2000004d0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`160?oo
o`030000003oool0oooo04/0oooo0P00000Z0?ooo`00E@3oool00`000000oooo0?ooo`1<0?ooo`03
0000003oool0oooo04H0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`1<0?ooo`03
0000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo04`0oooo00<000000?ooo`3oool0
AP3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo04/0oooo00<000000?ooo`3oool0
:03oool005D0oooo00<000000?ooo`3oool0C03oool00`000000oooo0?ooo`160?ooo`030000003o
ool0oooo04L0oooo00<000000?ooo`3oool0B`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool0
0`000000oooo0?ooo`1<0?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool0A`3oool0
0`000000oooo0?ooo`1;0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo
04/0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo
04/0oooo00<000000?ooo`3oool0:03oool005D0oooo0P00001<0?ooo`030000003oool0oooo04L0
oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`1:0?ooo`800000:P3oool005D0oooo
00<000000?ooo`3oool0B`3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo04P0oooo
00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?oo
o`1;0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0B03oool00`000000oooo0?oo
o`1:0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo04/0oooo00<00000
0?ooo`3oool0A`3oool00`000000oooo0?ooo`180?ooo`030000003oool0oooo04X0oooo00<00000
0?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`180?oo
o`030000003oool0oooo04P0oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`0X0?oo
o`00E@3oool00`000000oooo0?ooo`1:0?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3o
ool0B03oool00`000000oooo0?ooo`1:0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000
003oool0oooo04X0oooo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`180?ooo`030000
003oool0oooo04X0oooo00<000000?ooo`3oool0:03oool005D0oooo0P00001;0?ooo`030000003o
ool0oooo04P0oooo00<000000?ooo`3oool0B@3oool00`000000oooo0?ooo`180?ooo`800000:P3o
ool004P0oooo00<0gia804RLg`3oool02P3oool00`000000oooo0?ooo`1:0?ooo`030000003oool0
oooo04P0oooo00<000000?ooo`3oool0B@3oool00`000000oooo0?ooo`190?ooo`030000003oool0
oooo02P0oooo00180?ooo`030?oOW018B9`0gooo00X0oooo00<000000?ooo`3oool0B@3oool00`00
0000oooo0?ooo`190?ooo`030000003oool0oooo04T0oooo00<000000?ooo`3oool0B@3oool00`00
0000oooo0?ooo`0X0?ooo`00B03oool00`3ooml0W4Q809cOo`0:0?ooo`030000003oool0oooo04T0
oooo00<000000?ooo`3oool0B@3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo04T0
oooo00<000000?ooo`3oool0:03oool004T0oooo00<0gia804RLg`3oool02@3oool00`000000oooo
0?ooo`190?ooo`030000003oool0oooo04T0oooo00<000000?ooo`3oool0B@3oool00`000000oooo
0?ooo`190?ooo`030000003oool0oooo02P0oooo00190?ooo`030?oOW018B9`0gooo00T0oooo00<0
00000?ooo`3oool0B@3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo04T0oooo00<0
00000?ooo`3oool0B@3oool00`000000oooo0?ooo`0X0?ooo`00B@3oool00`3ooml0W4Q809cOo`09
0?ooo`<00000B@3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo04X0oooo00<00000
0?ooo`3oool0AP3oool3000002X0oooo001:0?ooo`030=nLB000M;l0oooo00P0oooo00<000000?oo
o`3oool0B@3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo04X0oooo00<000000?oo
o`3oool0B03oool00`000000oooo0?ooo`0X0?ooo`00BP3oool00`3ookl0M01d0;ooo`080?ooo`03
0000003oool0oooo04P0oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`1:0?ooo`03
0000003oool0oooo04P0oooo00<000000?ooo`3oool0:03oool004D0oooo00<0omoO0=ooo`3oool0
0`3oool00`2oM000M;oo0?ooo`070?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3oool0
BP3oool00`000000oooo0?ooo`1:0?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3oool0
:03oool004D0oooo00<0oono07A8W03Oool00`3oool00`3o_g@007Bo0?ooo`070?ooo`030000003o
ool0oooo04P0oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`1:0?ooo`030000003o
ool0oooo04P0oooo00<000000?ooo`3oool0:03oool004H0oooo00<0_g@00000000000000`000000
0`0007@0_ooo0?ooo`060?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3oool0BP3oool0
0`000000oooo0?ooo`1:0?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3oool0:03oool0
04H0oooo00<0okmd000000000000100000000`1d_ol0oooo0?ooo`050?ooo`800000B@3oool00`00
0000oooo0?ooo`1:0?ooo`030000003oool0oooo04/0oooo00<000000?ooo`3oool0AP3oool20000
02X0oooo001E0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0B`3oool00`000000
oooo0?ooo`1;0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0:03oool005D0oooo
00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`1;0?ooo`030000003oool0oooo04/0oooo
00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?oo
o`170?ooo`030000003oool0oooo04/0oooo00<000000?ooo`3oool0B`3oool00`000000oooo0?oo
o`170?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo04L0oooo00<00000
0?ooo`3oool0B`3oool00`000000oooo0?ooo`1;0?ooo`030000003oool0oooo04L0oooo00<00000
0?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`1;0?oo
o`030000003oool0oooo04`0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`0X0?oo
o`00E@3oool2000004P0oooo00<000000?ooo`3oool0B`3oool00`000000oooo0?ooo`1<0?ooo`03
0000003oool0oooo04D0oooo0P00000Z0?ooo`00E@3oool00`000000oooo0?ooo`160?ooo`030000
003oool0oooo04`0oooo00<000000?ooo`3oool0C03oool00`000000oooo0?ooo`160?ooo`030000
003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool0C03o
ool00`000000oooo0?ooo`1<0?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool0:03o
ool005D0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`1<0?ooo`030000003oool0
oooo04`0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`00
0000oooo0?ooo`160?ooo`030000003oool0oooo04`0oooo00<000000?ooo`3oool0C03oool00`00
0000oooo0?ooo`160?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo04H0
oooo00<000000?ooo`3oool0C03oool00`000000oooo0?ooo`1=0?ooo`030000003oool0oooo04D0
oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0A@3oool00`000000oooo
0?ooo`1=0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0A@3oool00`000000oooo
0?ooo`0X0?ooo`00E@3oool2000004H0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?oo
o`1=0?ooo`030000003oool0oooo04@0oooo0P00000Z0?ooo`00E@3oool00`000000oooo0?ooo`15
0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`15
0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo04D0oooo00<000000?oo
o`3oool0C@3oool00`000000oooo0?ooo`1=0?ooo`030000003oool0oooo04D0oooo00<000000?oo
o`3oool0:03oool005D0oooo00<000000?ooo`3oool0A@3oool00`000000oooo0?ooo`1=0?ooo`03
0000003oool0oooo04d0oooo00<000000?ooo`3oool0A@3oool00`000000oooo0?ooo`0X0?ooo`00
E@3oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0
CP3oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003o
ool0oooo04@0oooo00<000000?ooo`3oool0CP3oool00`000000oooo0?ooo`1>0?ooo`030000003o
ool0oooo04@0oooo00<000000?ooo`3oool0:03oool005D0oooo0P0000150?ooo`030000003oool0
oooo04h0oooo00<000000?ooo`3oool0CP3oool00`000000oooo0?ooo`130?ooo`800000:P3oool0
04L0oooo00D0gia8000000000000000004RLg`090?ooo`030000003oool0oooo04@0oooo00<00000
0?ooo`3oool0CP3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo04@0oooo00<00000
0?ooo`3oool0:03oool004H0oooo00L0gia80000001d_ol0oooo0?oog`2LB000B9cO00P0oooo00<0
00000?ooo`3oool0A03oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo04h0oooo00<0
00000?ooo`3oool0A03oool00`000000oooo0?ooo`0X0?ooo`00A@3oool00`3ooml0W4P0001d_`03
0?ooo`030?noM000B9`0gooo00L0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`1>
0?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`0X
0?ooo`00AP3oool00`2oM00007Bo0?ooo`020?ooo`030;md0000B9`0gooo00L0oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01D0oooo00<000000?oo
o`3oool0=`3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo01D0oooo00<000000?oo
o`3oool07`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo02P0oooo00160?ooo`07
0?oOW01807@0_ooo0?ooo`2oM000000007Boo`080?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool02`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool02`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02P0oooo00170?oo
o`050?oo_`1dB4P0B00000000018W=l02@3ooooo000003800000:P3oool004L0oooo00@0omnL04P0
0000000007Boo`3ooomV0?ooo`00AP3oool01P3ogi`0B0000000M02oool0okmd001d_ol0ooooI@3o
ool004H0oooo00<0_g@00000M02oool00P3oool00`2oM000B9cO0?ooo`3o0?ooof<0oooo00160?oo
o`030;md0000B9`0gooo0080oooo00<0gia8001d_`3oool0o`3ooomS0?ooo`00AP3oool01`3o_g@0
000007Boo`3oool0oono07@0001d_ol0o`3ooomT0?ooo`00A`3oool01@3o_g@00000000000000000
B9cO0?l0ooooI@3oool00?l0oooo103oool01P3ooml0W4P0000000000000001d0;ooojL0oooo003o
0?ooo`H0oooo00<0_g@00018W03Oool0Z03oool00?l0oooo1P3oool00`2oM00004RL0=ooo`2X0?oo
o`00o`3oool60?ooo`030;md0000B9`0gooo0:P0oooo002H0?ooo`080?oog`2LB000000000000000
B9`0W7@000000000M;l50?ooo`080?oog`2LB00000000000000004P0B4P00018W03Oool20?ooo`05
0=nLB000000000000000001d_ol00`3oool02`3ookl0M000000000000000B9bo09a80000B9`0gooo
0?ooo`3ooml0W4P00080000000D0001809cOo`3oool0oooo0=nLB002000000040000M02oool0oooo
0;md0080000000<0001d0;ooo`3oool0403oool00`3ogi`0M9cO0?ooo`020?ooo`030=nLM02Lgol0
oooo00@0oooo00L0_g@0000000000000001d0;ooo`3oool0gia80080000000H0001d09bLB0000000
00000000M02oool60?ooo`030;md0000B4P0B0000080000000<0B9cO0?ooo`3oool00P3oool01@3O
W4P00000000000000000M;oo00<0oooo00@0omnL04P000000000B9cO0P3oool01`2oM00000000000
000007@0_ooo0?ooo`2oM0000P0000000`0007@0_ooO09a80002000000070018W03Oool0omnL04P0
00000000B9cO0=nLB002000000030000M02ooml0W4P00080000000D004RL09ad0000000007Bo0=nL
B002000000030000M02oool0oooo06d0oooo002H0?ooo`090?noM0000000M;oo0?ooo`3ogi`0B000
001d_`3o_kl0_ooo00<0oooo0140oooO09a800000000M;oo0?ooo`3ookl0M0000000000007@0_ooo
0=nLB0000000B9cO0?ooo`3ooml0W4P004RLg`020?ooo`050=nLB00007@0_ooo0?ooo`3OW4P00P00
00002@0007@0_ooo0;md0000B9`0gooo0?ooo`3ogi`0B00007Boo`020?ooo`0;0=nLB00007@0_ooo
0?ooo`2oM0000000001d_`3oool0omnL04P0M02oool0403oool01`2oM000001d0;ooo`3oool0oono
07@00000M;l0103oool03`2oM0000000001d_`3oool0omnL04P0M02oool0oooo0=nLB00007@0_ooo
0?ooo`3OW4P0001d0;ooo`070?ooo`070;md00000000M;oo0?ooo`3ookl0M00004RLg`020?ooo`07
0=nLB0000000B9cO0?ooo`3ooml0W4P004RLg`020?ooo`0D0=nLB00007@0_ooo0=no_`3Oool0_g@0
00000000M;l0oooo0?oOW01807@0_ooo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l20?oo
o`0D0=nLB00007@0_ooo0=no_`3Oool0gia80000M02oool0okmd0000001d_ol0oooo0?oOW0180000
07Bo0?no_`2oool0gia80000M02ooom_0?ooo`00V03oool01`3o_g@0001809cOo`3oool0oooO09a8
0000M;l01@3oool00`3ogi`0B00004RLg`020?ooo`070?oog`2LB00007Bo0?ooo`3ogi`0B000001d
_`030?ooo`0;0?noM00007@0_ooo0?ooo`2oM00004RL0=ooo`3oool0oooO09a80000M;l00P3oool0
0`2oM00004RL0=ooo`020?ooo`0:0?noM00007@0_ooo0?ooo`3OW4P0001d0;ooo`3ogi`0B00004RL
g`<0oooo00<0omnL07BLg`3oool03P3oool02@3ogi`0B00004QdM01dW=l0oooo0?noM00004P0B4Q8
09cOo`020?ooo`030?oOW0180000B9cO00<0oooo00X0omnL07BLg`3oool0gia80000M02oool0oooo
0=nLB00007@0_ooo1`3oool00`2oM00004RL0=ooo`020?ooo`060?noM00007@0_ooo0?oOW0180000
07Bo0`3oool02`3o_g@0001d0;ooo`3oool0gia80018W03Oool0oooo0?oOW0180000B9cO00<0oooo
00X0omnL07BLg`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo0P3oool00`3OW4P004RL0=oo
o`020?ooo`0:0=nLB00007@0_ooo0?noM00004P0W=oo0?ooo`3ooml0W4P0001d_`80oooo00<0gia8
0000M02oool0K`3oool009T0oooo00H0okmd00000000M;l0oooO09a80000M;l50?ooo`030?noM000
0000M;oo0080oooo00L0oooO09a80000M;l0oooo0?noM0000000M;oo00<0oooo00/0okmd0000001d
_ol0oooo0;md0000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`030;md0000B9`0gooo0080oooo
00X0omnL04P0001d_ol0oooo0=nLB00007@0_ooo0?noM00004P0W=oo503oool02@3OW4P0001d0;oo
o`2oM7@0_ooO09a80000M;l0omnL07BLg`020?ooo`030?noM00004P0W=oo00H0oooo00L0gia80000
M02oool0oooo0=nLB00007@0_ooo00L0oooo00<0_g@00018W03Oool00P3oool01P3ogi`0B00007Bo
o`3o_g@0000007Boo`<0oooo00/0okmd0000001d_ol0oooo0=nLB000B9`0gooo0?ooo`3o_g@00018
09cOo`060?ooo`070=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`030=nLB000B9`0gooo
0080oooo00X0gia80000M02oool0oooo0?noM000000007Bo0?oog`2LB00007Bo0P3oool00`3OW4P0
001d0;ooo`1_0?ooo`00V`3oool0103o_g@0000000000000M;l50?ooo`030?noM0000000M;oo0080
oooo00L0oooO09a80000M;l0oooo0?noM0000000M;oo00<0oooo00/0okmd0000001d_ol0oooo0;md
0000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`030;md0000B9`0gooo0080oooo00X0omnL04P0
001d_ol0oooo0=nLB00007@0_ooo0?noM00007@0_ooo1P3oool00`3ooml0W4P0000000070000000B
04RLg`3oool0oooo0?oo_`1d0000B9cO0?ooo`3ogi`0M7Ad04P0B02Lgol0oooo0;mdM02oool0oooo
0?noM00007@0_ooo1P3oool01`3OW4P0001d0;ooo`3oool0gia80000M02oool01`3oool00`2oM000
04RL0=ooo`020?ooo`060?oOW0180000M;oo0?noM0000000M;oo0`3oool02`3o_g@0000007Boo`3o
ool0gia80018W03Oool0oooo0?noM00007@0_ooo00H0oooo00L0gia80018W03Oool0oooo0?oog`2L
B00007Bo0080oooo00<0gia80018W03Oool00P3oool00`3OW4P0001d0;ooo`030?ooo`040?noM000
00000000001d_`80oooo00<0gia80000M02oool0K`3oool009P0oooo00L0oono07@0001d_ol0oooo
0?oog`2LB00007Bo00D0oooo00<0oono07@00018W=l00P3oool01`3ooml0W4P0001d_`3oool0oono
07@0001d_ol00`3oool02`2oM000001809cOo`3oool0_g@00018W03Oool0oooo0?oog`2LB00007Bo
0080oooo00<0_g@00018W03Oool00P3oool02@3o_g@0001809cOo`3oool0gia80000M02oool0omnL
04P000050000000304RLg`3oool0oooo00`0oooo00L0gia80000B02Lgol0oooo0?oog`2LB00007Bo
0080oooo00D0okmd04RLg`3oool0omnL04P000050000000904RLg`3oool0gia80000M02oool0oooo
0=nLB00007@0_ooo00L0oooo00<0_g@00000M02oool00P3oool01P3OW4P0001809cOo`3ookl0M000
07Boo`<0oooo00X0_g@00000B02Lgol0oooo0=nLB000B9`0gooo0?ooo`3ogi`0B0001@0000002@18
W=l0oooo0;md000007@0_ooo0?ooo`3ooml0W4P0001d_`020?ooo`030=nLB000B9`0gooo0080oooo
00X0gia80000M02oool0oono07@0001d_ol0oooo0?oog`2LB00007Bo0P3oool00`3OW4P0001d0;oo
o`1_0?ooo`00V03oool01`3ookl0M00004RLg`3oool0oono07@00000M;l01P3oool01`3o_g@00018
09cOo`3oool0okmd00000000M;l00P3oool01`3OW4P004RL0=ooo`3oool0okmd00000000M;l00P3o
ool01`2oM00004RL0=ooo`3oool0oooO09a80000M;l00P3oool03`2oM00000000018W03Oool0oono
07@00000B9`0gooo0?ooo`3OW4P0001d0;ooo`3oool0_g@004RLg`020?ooo`030=nLB00004P0W=oo
00d0oooo00<0oono07@00000M;l00P3oool00`3ogi`0B00007Boo`020?ooo`060?oog`2LB7@0_ooo
0?ooo`2oM000B9cO0P3oool02`3OW4P0001809cOo`3oool0gia80000M02oool0oooo0=nLB00007@0
_ooo00D0oooo0180oono07@00000000004Q804Q8W03Oool0omnL04P00000B9`0gooo0?ooo`3OW4P0
04RL0=ooo`3oool0okmd00000000M;l20?ooo`030=nLB000B9`0gooo0080oooo00l0_g@004RLg`3o
ool0oooo0=nLB00004P0W=oO09a80000000004Q804P0B02Lgol0oono07@00018W=l00P3oool03`3O
W4P004RL0=ooo`3ooml0W4P00000000007@0_ooo0?oo_`1d0000B9cO0?ooo`3ookl0M000001d_`02
0?ooo`030=nLB00007@0_ooo06l0oooo002I0?ooo`060?oo_`1d000000000000000007@0_ooo1`3o
ool00`3o_g@000000000000200000005001d_`3oool0oooo0?ooo`3o_g@00`0000002`18W=l0oooo
0?oo_`1d000000000018W03Oool0omnL04P00000000007Bo0080oooo00L0_g@00018W03Ogi`0B000
00000000B9`0gooo0080oooo00<0gia80000M02oool00P3oool01@3OW4P000000000000004P0W=oo
00d0oooo00H0oooO09a800000000000004RLg`3o_g@30000000804RLg`3oool0_g@000000000B9`0
gooo0?ooo`3OW4P20000000;0000B02Lgol0oooo0?ooo`3OW4P0001d0;ooo`3oool0gia80000M02o
ool01P3oool0203ooml0W4P00018W03Ogi`0B0000000000007@0_ooo0`3oool01@3o_g@000000000
00000000B9cO0080oooo00P0_g@000000000000000000018W03Oool0oooo0=nLB080000000d00018
09cOo`3oool0oooo0;md0000B9`0gooO09a800000000001d0;ooo`3oool0_g@000<0000000P004RL
0=ooo`2oM000001d0;ooo`3oool0oono07@00080000000L0001d0;ooo`3oool0oooo0=nLB00007@0
_ooo06l0oooo002Y0?ooo`030?oog`2LB00007Bo01@0oooo00<0_g@00018W03Oool01P3oool00`3O
W4P0001d0;ooo`0[0?ooo`070=nLB00007@0_ooo0?ooo`3OW4P0001d0;ooo`0I0?ooo`030;md0000
B9`0gooo01<0oooo00<0_g@00018W03Oool03P3oool00`3OW4P0001d0;ooo`1_0?ooo`00Z@3oool0
0`3ooml0W4P0001d_`0D0?ooo`030;md0000B9`0gooo00H0oooo00<0gia80000M02oool0:`3oool0
1`3OW4P0001d0;ooo`3oool0gia80000M02oool06@3oool00`3ookl0M4RL0=ooo`0C0?ooo`030?oo
_`1dB9`0gooo00h0oooo00<0gia80000M02oool0K`3oool00:T0oooo00<0oooO09a80000M;l0503o
ool00`2oM00004RL0=ooo`060?ooo`030=nLB00007@0_ooo02/0oooo00L0gia80000M02oool0oooo
0=nLB00007@0_ooo01T0oooo00<0oooO09adW03Oool04`3oool00`3ooml0W7BL0=ooo`0>0?ooo`03
0=nLB00007@0_ooo06l0oooo002X0?ooo`040?oo_`1d00000000001d_a80oooo00D0oono07@00000
000004RL0=ooo`040?ooo`050?oog`2LB00000000000M02oool0:@3oool02@3ooml0W4P000000000
07@0_ooO09a800000000001d0;ooo`0d0?ooo`030=nLB00004P0W=oo00L0oooo00D0oooO09a80000
0000001d0;ooo`1_0?ooo`00Z@3oool00`3ooml0W4P0001d_`0C0?ooo`040?oog`2LB00004RL0=oo
o`H0oooo00<0_g@00000M02oool0:`3oool01`2oM000001d0;ooo`3oool0_g@00000M02oool0=03o
ool00`3OW4P0001809cOo`090?ooo`030;md000007@0_ooo06l0oooo003o0?oook40oooo003o0?oo
ok40oooo003o0?oook40oooo003o0?oook40oooo0000\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {-8.21226, 0.374123, \
0.0518628, 0.0479517}}]
}, Closed]],

Cell[TextData[{
  "Below we use ",
  StyleBox["NDSolve", "InlineInput",
    FontWeight->"Bold"],
  " to integrate the differential equation with ",
  Cell[BoxData[
      \(TraditionalForm\`m = 0.5\)]],
  " from ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          StyleBox["xmin",
            FontSlant->"Italic"], "=", \(-5\)}], TraditionalForm]]],
  " toward  large positive ",
  StyleBox["x",
    FontSlant->"Italic"],
  " for a range of energies near one of the eigenvalues."
}], "Text"],

Cell[BoxData[{
    \(\(Clear[f];\)\), "\n", 
    \(f[x_, e_] := 
      Module[{\[Psi], sol, xmatch = 5}, 
        sol = \(NDSolve[{\(-\(\[Psi]\^\[DoublePrime]\)[x]\) + 
                    pot[x] \[Psi][x] \[Equal] 
                  e\ \[Psi][x], \[Psi][\(-xmatch\)] \[Equal] 
                  0, \(\[Psi]\^\[Prime]\)[\(-xmatch\)] \[Equal] 
                  10\^\(-4\)}, \[Psi], {x, \(-xmatch\), 
                xmatch}]\)\[LeftDoubleBracket]1\[RightDoubleBracket]; \
\[IndentingNewLine]\[Psi][x] /. sol]\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(tmp = Table[f[x, e], {e, 4.43, 4.431, 0.0002}];\)\), "\n", 
    \(\(Plot[Evaluate[tmp], {x, \(-5\), 5}];\)\)}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0952381 0.297899 1.79428 [
[.11905 .2854 -9.15625 -15.6875 ]
[.11905 .2854 9.15625 0 ]
[.30952 .2854 -9.15625 -15.6875 ]
[.30952 .2854 9.15625 0 ]
[.69048 .2854 -5 -15.6875 ]
[.69048 .2854 5 0 ]
[.88095 .2854 -5 -15.6875 ]
[.88095 .2854 5 0 ]
[.4875 .02876 -33.3125 -7.84375 ]
[.4875 .02876 0 7.84375 ]
[.4875 .11847 -27.3125 -7.84375 ]
[.4875 .11847 0 7.84375 ]
[.4875 .20818 -33.3125 -7.84375 ]
[.4875 .20818 0 7.84375 ]
[.4875 .38761 -25 -7.84375 ]
[.4875 .38761 0 7.84375 ]
[.4875 .47733 -19 -7.84375 ]
[.4875 .47733 0 7.84375 ]
[.4875 .56704 -25 -7.84375 ]
[.4875 .56704 0 7.84375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.11905 .2979 m
.11905 .30415 L
s
gsave
.11905 .2854 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.30952 .2979 m
.30952 .30415 L
s
gsave
.30952 .2854 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.69048 .2979 m
.69048 .30415 L
s
gsave
.69048 .2854 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.88095 .2979 m
.88095 .30415 L
s
gsave
.88095 .2854 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.125 Mabswid
.16667 .2979 m
.16667 .30165 L
s
.21429 .2979 m
.21429 .30165 L
s
.2619 .2979 m
.2619 .30165 L
s
.35714 .2979 m
.35714 .30165 L
s
.40476 .2979 m
.40476 .30165 L
s
.45238 .2979 m
.45238 .30165 L
s
.54762 .2979 m
.54762 .30165 L
s
.59524 .2979 m
.59524 .30165 L
s
.64286 .2979 m
.64286 .30165 L
s
.7381 .2979 m
.7381 .30165 L
s
.78571 .2979 m
.78571 .30165 L
s
.83333 .2979 m
.83333 .30165 L
s
.07143 .2979 m
.07143 .30165 L
s
.02381 .2979 m
.02381 .30165 L
s
.92857 .2979 m
.92857 .30165 L
s
.97619 .2979 m
.97619 .30165 L
s
.25 Mabswid
0 .2979 m
1 .2979 L
s
.5 .02876 m
.50625 .02876 L
s
gsave
.4875 .02876 -94.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.15) show
1.000 setlinewidth
grestore
.5 .11847 m
.50625 .11847 L
s
gsave
.4875 .11847 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.1) show
1.000 setlinewidth
grestore
.5 .20818 m
.50625 .20818 L
s
gsave
.4875 .20818 -94.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.05) show
1.000 setlinewidth
grestore
.5 .38761 m
.50625 .38761 L
s
gsave
.4875 .38761 -86 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.05) show
1.000 setlinewidth
grestore
.5 .47733 m
.50625 .47733 L
s
gsave
.4875 .47733 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.1) show
1.000 setlinewidth
grestore
.5 .56704 m
.50625 .56704 L
s
gsave
.4875 .56704 -86 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.15) show
1.000 setlinewidth
grestore
.125 Mabswid
.5 .0467 m
.50375 .0467 L
s
.5 .06464 m
.50375 .06464 L
s
.5 .08259 m
.50375 .08259 L
s
.5 .10053 m
.50375 .10053 L
s
.5 .13641 m
.50375 .13641 L
s
.5 .15436 m
.50375 .15436 L
s
.5 .1723 m
.50375 .1723 L
s
.5 .19024 m
.50375 .19024 L
s
.5 .22613 m
.50375 .22613 L
s
.5 .24407 m
.50375 .24407 L
s
.5 .26201 m
.50375 .26201 L
s
.5 .27996 m
.50375 .27996 L
s
.5 .31584 m
.50375 .31584 L
s
.5 .33378 m
.50375 .33378 L
s
.5 .35173 m
.50375 .35173 L
s
.5 .36967 m
.50375 .36967 L
s
.5 .40556 m
.50375 .40556 L
s
.5 .4235 m
.50375 .4235 L
s
.5 .44144 m
.50375 .44144 L
s
.5 .45938 m
.50375 .45938 L
s
.5 .49527 m
.50375 .49527 L
s
.5 .51321 m
.50375 .51321 L
s
.5 .53116 m
.50375 .53116 L
s
.5 .5491 m
.50375 .5491 L
s
.5 .01081 m
.50375 .01081 L
s
.5 .58498 m
.50375 .58498 L
s
.5 .60293 m
.50375 .60293 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .2979 m
.03279 .29792 L
.04262 .29794 L
.04758 .29795 L
.05288 .29797 L
.06244 .29802 L
.06788 .29805 L
.07379 .29809 L
.07899 .29814 L
.08451 .29821 L
.08947 .29828 L
.09494 .29837 L
.10458 .29859 L
.10978 .29875 L
.11472 .29893 L
.12399 .29935 L
.13397 .29998 L
.14487 .30095 L
.15519 .30221 L
.16454 .30373 L
.17534 .30605 L
.18551 .30892 L
.19528 .31244 L
.20559 .31713 L
.22406 .32852 L
.2437 .34562 L
.26204 .36666 L
.30094 .42453 L
.32075 .45554 L
.3312 .47019 L
.3423 .4833 L
.35214 .49208 L
.35484 .49394 L
.35774 .49566 L
.36025 .4969 L
.363 .49798 L
.36422 .49837 L
.36552 .49873 L
.36665 .49898 L
.36787 .4992 L
.36859 .4993 L
.36925 .49938 L
.37055 .49948 L
.37174 .4995 L
.37299 .49948 L
.37408 .4994 L
.37524 .49926 L
.37635 .49908 L
.37735 .49888 L
.37981 .4982 L
Mistroke
.38215 .49734 L
.38719 .49473 L
.39176 .49151 L
.402 .4815 L
.41148 .46914 L
.4204 .45523 L
.459 .38103 L
.50005 .29793 L
.53959 .21792 L
.57762 .14419 L
.58766 .1282 L
.59717 .11555 L
.60242 .10979 L
.60812 .10464 L
.6133 .10101 L
.6158 .09964 L
.61809 .09859 L
.62036 .09777 L
.62274 .09712 L
.62398 .09688 L
.62513 .0967 L
.6264 .09657 L
.6271 .09653 L
.62775 .09651 L
.62891 .09651 L
.62998 .09656 L
.63116 .09667 L
.6324 .09684 L
.63349 .09704 L
.6345 .09727 L
.6368 .09793 L
.63947 .09896 L
.642 .10017 L
.64674 .10308 L
.65741 .11233 L
.66705 .12344 L
.6773 .13744 L
.69571 .16588 L
.73343 .22302 L
.75275 .24624 L
.7627 .25606 L
.7736 .26514 L
.79362 .27769 L
.80454 .28255 L
.81471 .28603 L
.81975 .28741 L
.82521 .28869 L
.83003 .28964 L
.83514 .29046 L
.83974 .29106 L
Mistroke
.8448 .29157 L
.84702 .29174 L
.84941 .29189 L
.85166 .29201 L
.85372 .29208 L
.85501 .29212 L
.85622 .29214 L
.85731 .29215 L
.85849 .29216 L
.85978 .29216 L
.86043 .29215 L
.86114 .29214 L
.8624 .29212 L
.86357 .29209 L
.86476 .29205 L
.86588 .292 L
.86799 .29189 L
.87026 .29174 L
.87272 .29153 L
.87794 .29093 L
.88349 .29003 L
.88822 .28901 L
.89345 .28758 L
.89837 .28586 L
.90355 .28361 L
.91281 .27805 L
.91797 .27384 L
.92278 .26898 L
.92801 .2624 L
.93367 .25335 L
.93915 .24213 L
.9442 .22904 L
.94894 .21373 L
.95414 .19257 L
.96318 .14114 L
.96812 .10218 L
.97274 .05636 L
.97619 .01472 L
Mfstroke
.02381 .2979 m
.03279 .29792 L
.04262 .29794 L
.04758 .29795 L
.05288 .29797 L
.06244 .29802 L
.06788 .29805 L
.07379 .29809 L
.07899 .29814 L
.08451 .29821 L
.08947 .29828 L
.09494 .29837 L
.10458 .29859 L
.10978 .29875 L
.11472 .29893 L
.12399 .29935 L
.13397 .29998 L
.14487 .30095 L
.15519 .30221 L
.16454 .30373 L
.17534 .30605 L
.18551 .30892 L
.19528 .31244 L
.20559 .31713 L
.22406 .32852 L
.2437 .34561 L
.26204 .36666 L
.30094 .42452 L
.32075 .45552 L
.3312 .47017 L
.3423 .48327 L
.35214 .49205 L
.35484 .49391 L
.35774 .49563 L
.36025 .49687 L
.363 .49795 L
.36422 .49834 L
.36552 .4987 L
.36665 .49895 L
.36787 .49917 L
.36859 .49927 L
.36925 .49935 L
.37055 .49944 L
.37174 .49947 L
.37299 .49944 L
.37408 .49936 L
.37524 .49923 L
.37635 .49905 L
.37735 .49884 L
.37981 .49817 L
Mistroke
.38215 .4973 L
.38719 .49469 L
.39176 .49147 L
.402 .48146 L
.41148 .46909 L
.4204 .45519 L
.459 .38098 L
.50005 .29787 L
.53959 .21786 L
.57762 .14412 L
.58766 .12812 L
.59717 .11547 L
.60242 .10972 L
.60812 .10457 L
.6133 .10095 L
.6158 .09957 L
.61809 .09853 L
.62036 .09771 L
.62274 .09706 L
.62398 .09682 L
.62513 .09665 L
.6264 .09652 L
.6271 .09648 L
.62775 .09645 L
.62891 .09646 L
.62998 .09651 L
.63116 .09661 L
.6324 .09679 L
.63349 .09699 L
.6345 .09722 L
.6368 .09788 L
.63947 .09891 L
.642 .10013 L
.64674 .10304 L
.65741 .1123 L
.66705 .12343 L
.6773 .13744 L
.69571 .1659 L
.73343 .2231 L
.75275 .24635 L
.7627 .25619 L
.7736 .26531 L
.79362 .27793 L
.80454 .28286 L
.81471 .28641 L
.82521 .28919 L
.83514 .29111 L
.84405 .29233 L
.84908 .29282 L
.85372 .29317 L
Mistroke
.85616 .29331 L
.85838 .29341 L
.85961 .29346 L
.86095 .2935 L
.86224 .29353 L
.8634 .29356 L
.86457 .29357 L
.86567 .29358 L
.86684 .29358 L
.8681 .29358 L
.8692 .29357 L
.87037 .29355 L
.87137 .29353 L
.87246 .2935 L
.87516 .29341 L
.87761 .29328 L
.88045 .2931 L
.88313 .29288 L
.8879 .29236 L
.89309 .29162 L
.89794 .2907 L
.9025 .28962 L
.90792 .28801 L
.91288 .28613 L
.92299 .28077 L
.9283 .27684 L
.93389 .27158 L
.94345 .2589 L
.94879 .24901 L
.95365 .23766 L
.95819 .22447 L
.9631 .20662 L
.97202 .16134 L
.97619 .13247 L
Mfstroke
.02381 .2979 m
.03279 .29792 L
.04262 .29794 L
.04758 .29795 L
.05288 .29797 L
.06244 .29802 L
.06788 .29805 L
.07379 .29809 L
.07899 .29814 L
.08451 .29821 L
.08947 .29828 L
.09494 .29837 L
.10458 .29859 L
.10978 .29875 L
.11472 .29893 L
.12399 .29935 L
.13397 .29998 L
.14487 .30095 L
.15519 .30221 L
.16454 .30373 L
.17534 .30605 L
.18551 .30892 L
.19528 .31244 L
.20559 .31713 L
.22406 .32852 L
.2437 .34561 L
.26204 .36665 L
.30094 .42451 L
.32075 .45551 L
.3312 .47015 L
.3423 .48325 L
.35214 .49202 L
.35484 .49388 L
.35774 .4956 L
.36025 .49684 L
.363 .49792 L
.36422 .49831 L
.36552 .49867 L
.36665 .49892 L
.36787 .49914 L
.36859 .49924 L
.36925 .49931 L
.37055 .49941 L
.37174 .49944 L
.37299 .49941 L
.37408 .49933 L
.37524 .4992 L
.37635 .49902 L
.37735 .49881 L
.37981 .49813 L
Mistroke
.38215 .49727 L
.38719 .49465 L
.39176 .49143 L
.402 .48142 L
.41148 .46905 L
.4204 .45515 L
.459 .38094 L
.50005 .29781 L
.53959 .21779 L
.57762 .14405 L
.58766 .12805 L
.59717 .1154 L
.60242 .10965 L
.60812 .1045 L
.6133 .10088 L
.6158 .09951 L
.61809 .09847 L
.62036 .09765 L
.62274 .097 L
.62398 .09676 L
.62513 .09659 L
.6264 .09646 L
.6271 .09642 L
.62775 .0964 L
.62891 .0964 L
.62998 .09645 L
.63116 .09656 L
.6324 .09674 L
.63349 .09694 L
.6345 .09717 L
.6368 .09784 L
.63947 .09887 L
.642 .10009 L
.64674 .103 L
.65741 .11228 L
.66705 .12341 L
.6773 .13744 L
.69571 .16592 L
.73343 .22317 L
.75275 .24646 L
.7627 .25632 L
.7736 .26547 L
.79362 .27817 L
.80454 .28316 L
.81471 .28679 L
.82521 .28969 L
.83514 .29175 L
.84405 .29315 L
.84904 .29377 L
.85372 .29426 L
Mistroke
.85866 .29469 L
.86317 .29501 L
.86818 .29528 L
.87098 .2954 L
.87357 .29549 L
.87607 .29556 L
.87735 .29559 L
.87873 .29561 L
.87952 .29563 L
.88025 .29564 L
.88093 .29564 L
.88168 .29565 L
.88298 .29566 L
.88369 .29566 L
.88436 .29566 L
.88499 .29566 L
.88569 .29566 L
.88695 .29565 L
.88817 .29564 L
.88931 .29562 L
.89053 .2956 L
.89188 .29558 L
.89461 .2955 L
.89696 .29542 L
.89953 .2953 L
.90471 .295 L
.90951 .29461 L
.91397 .29414 L
.91925 .29343 L
.92406 .2926 L
.9296 .29139 L
.93483 .28992 L
.94456 .28606 L
.94947 .28334 L
.95486 .27956 L
.95946 .27548 L
.9644 .27 L
.97332 .25611 L
.97619 .2502 L
Mfstroke
.02381 .2979 m
.03279 .29792 L
.04262 .29794 L
.04758 .29795 L
.05288 .29797 L
.06244 .29802 L
.06788 .29805 L
.07379 .29809 L
.07899 .29814 L
.08451 .29821 L
.08947 .29828 L
.09494 .29837 L
.10458 .29859 L
.10978 .29875 L
.11472 .29893 L
.12399 .29935 L
.13397 .29998 L
.14487 .30095 L
.15519 .30221 L
.16454 .30373 L
.17534 .30605 L
.18551 .30892 L
.19528 .31244 L
.20559 .31713 L
.22406 .32851 L
.2437 .34561 L
.26204 .36665 L
.30094 .42449 L
.32075 .45549 L
.3312 .47013 L
.3423 .48323 L
.35214 .492 L
.35484 .49385 L
.35774 .49557 L
.36025 .49681 L
.363 .49789 L
.36422 .49828 L
.36552 .49864 L
.36665 .49889 L
.36787 .49911 L
.36859 .49921 L
.36925 .49928 L
.37055 .49938 L
.37174 .49941 L
.37299 .49938 L
.37408 .4993 L
.37524 .49916 L
.37635 .49898 L
.37735 .49878 L
.37981 .4981 L
Mistroke
.38215 .49723 L
.38719 .49462 L
.39176 .49139 L
.402 .48138 L
.41148 .46901 L
.4204 .4551 L
.459 .38089 L
.50005 .29776 L
.53959 .21772 L
.57762 .14397 L
.58766 .12798 L
.59717 .11533 L
.60242 .10958 L
.60812 .10444 L
.6133 .10082 L
.6158 .09945 L
.61809 .09841 L
.62036 .09759 L
.62274 .09695 L
.62398 .0967 L
.62513 .09653 L
.6264 .09641 L
.6271 .09637 L
.62775 .09635 L
.62891 .09635 L
.62998 .0964 L
.63116 .09651 L
.6324 .09669 L
.63349 .09689 L
.6345 .09712 L
.6368 .09779 L
.63947 .09882 L
.642 .10005 L
.64674 .10297 L
.65741 .11225 L
.66705 .1234 L
.6773 .13744 L
.69571 .16594 L
.73343 .22325 L
.75275 .24657 L
.7627 .25645 L
.7736 .26563 L
.79362 .27841 L
.80454 .28346 L
.81471 .28718 L
.82521 .29018 L
.83514 .2924 L
.85372 .29535 L
.86342 .29649 L
.87371 .29753 L
Mistroke
.89215 .29937 L
.90157 .30053 L
.9117 .30216 L
.91661 .30318 L
.92126 .30434 L
.93001 .3072 L
.93504 .30939 L
.9396 .31182 L
.94452 .31505 L
.94983 .31944 L
.95476 .3246 L
.9593 .3305 L
.96356 .33729 L
.96822 .34651 L
.97619 .36793 L
Mfstroke
.02381 .2979 m
.03279 .29792 L
.04262 .29794 L
.04758 .29795 L
.05288 .29797 L
.06244 .29802 L
.06788 .29805 L
.07379 .29809 L
.07899 .29814 L
.08451 .29821 L
.08947 .29828 L
.09494 .29837 L
.10458 .29859 L
.10978 .29875 L
.11472 .29893 L
.12399 .29935 L
.13397 .29998 L
.14487 .30095 L
.15519 .30221 L
.16454 .30373 L
.17534 .30605 L
.18551 .30892 L
.19528 .31244 L
.20559 .31713 L
.22406 .32851 L
.2437 .3456 L
.26204 .36664 L
.30094 .42448 L
.32075 .45547 L
.3312 .47011 L
.3423 .4832 L
.35214 .49197 L
.35484 .49382 L
.35774 .49554 L
.36025 .49678 L
.363 .49786 L
.36422 .49825 L
.36552 .49861 L
.36665 .49886 L
.36787 .49907 L
.36859 .49917 L
.36925 .49925 L
.37055 .49934 L
.37174 .49937 L
.37299 .49934 L
.37408 .49926 L
.37524 .49913 L
.37635 .49895 L
.37735 .49874 L
.37981 .49806 L
Mistroke
.38215 .4972 L
.38719 .49458 L
.39176 .49136 L
.402 .48134 L
.41148 .46897 L
.4204 .45506 L
.459 .38084 L
.50005 .2977 L
.53959 .21766 L
.57762 .1439 L
.58766 .1279 L
.59717 .11526 L
.60242 .10951 L
.60812 .10437 L
.6133 .10075 L
.6158 .09938 L
.61809 .09835 L
.62036 .09753 L
.62274 .09689 L
.62398 .09665 L
.62513 .09648 L
.6264 .09635 L
.6271 .09631 L
.62775 .09629 L
.62891 .0963 L
.62998 .09635 L
.63116 .09646 L
.6324 .09664 L
.63349 .09684 L
.6345 .09707 L
.6368 .09774 L
.63947 .09878 L
.642 .10001 L
.64674 .10293 L
.65741 .11223 L
.66705 .12339 L
.6773 .13744 L
.69571 .16597 L
.73343 .22333 L
.75275 .24668 L
.7627 .25658 L
.7736 .26579 L
.79362 .27865 L
.80454 .28377 L
.81471 .28756 L
.82521 .29068 L
.83514 .29304 L
.85372 .29644 L
.87201 .29929 L
.88241 .30109 L
Mistroke
.89215 .30317 L
.90172 .30591 L
.90709 .30789 L
.91201 .31008 L
.91716 .31289 L
.92279 .31671 L
.93304 .32645 L
.93836 .33349 L
.94337 .34183 L
.95273 .36348 L
.95798 .38039 L
.96281 .40008 L
.96811 .42757 L
.97371 .4653 L
.97619 .48563 L
Mfstroke
.02381 .2979 m
.03279 .29792 L
.04262 .29794 L
.04758 .29795 L
.05288 .29797 L
.06244 .29802 L
.06788 .29805 L
.07379 .29809 L
.07899 .29814 L
.08451 .29821 L
.08947 .29828 L
.09494 .29837 L
.10458 .29859 L
.10978 .29875 L
.11472 .29893 L
.12399 .29935 L
.13397 .29998 L
.14487 .30095 L
.15519 .30221 L
.16454 .30373 L
.17534 .30605 L
.18551 .30892 L
.19528 .31244 L
.20559 .31713 L
.22406 .32851 L
.2437 .3456 L
.26204 .36664 L
.30094 .42447 L
.32075 .45545 L
.3312 .47009 L
.3423 .48318 L
.35214 .49194 L
.35484 .4938 L
.35774 .49551 L
.36025 .49675 L
.363 .49783 L
.36422 .49822 L
.36552 .49857 L
.36665 .49883 L
.36787 .49904 L
.36859 .49914 L
.36925 .49922 L
.37055 .49931 L
.37174 .49934 L
.37299 .49931 L
.37408 .49923 L
.37524 .49909 L
.37635 .49891 L
.37735 .49871 L
.37981 .49803 L
Mistroke
.38215 .49716 L
.38719 .49454 L
.39176 .49132 L
.402 .4813 L
.41148 .46892 L
.4204 .45502 L
.459 .38079 L
.50005 .29765 L
.53959 .21759 L
.57762 .14383 L
.58766 .12783 L
.59717 .11519 L
.60242 .10944 L
.60812 .1043 L
.6133 .10069 L
.6158 .09932 L
.61809 .09829 L
.62036 .09747 L
.62274 .09683 L
.62398 .09659 L
.62513 .09642 L
.6264 .0963 L
.6271 .09626 L
.62775 .09624 L
.62891 .09624 L
.62998 .0963 L
.63116 .09641 L
.6324 .09659 L
.63349 .09679 L
.6345 .09702 L
.6368 .0977 L
.63947 .09874 L
.642 .09997 L
.64674 .10289 L
.65741 .1122 L
.66705 .12337 L
.6773 .13744 L
.69571 .16599 L
.73343 .2234 L
.75275 .24679 L
.7627 .25672 L
.7736 .26595 L
.79362 .27888 L
.80454 .28407 L
.81471 .28794 L
.83514 .29368 L
.85372 .29753 L
.86381 .29949 L
.8745 .30178 L
.8844 .30437 L
Mistroke
.89367 .30756 L
.90248 .31168 L
.91196 .31791 L
.91736 .32266 L
.92248 .32826 L
.92753 .33509 L
.9321 .34269 L
.9417 .36457 L
.94697 .38121 L
.95192 .40098 L
.96076 .44969 L
.96574 .48748 L
.97032 .53111 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oe`3oool0
0`000000oooo0?ooo`3F0?ooo`00e`3oool00`000000oooo0?ooo`3F0?ooo`00e`3oool00`000000
oooo0?ooo`3F0?ooo`00e`3oool00`000000oooo0?ooo`3F0?ooo`00e`3oool300000=H0oooo003G
0?ooo`030000003oool0oooo0=H0oooo003G0?ooo`030000003oool0oooo0<X0oooo00<000000?oo
o`3oool02@3oool00;40oooo00D0omnL04P000000000001d0;ooo`030?ooo`030?noM0000000M;oo
00<0oooo00T0_g@000000000000000000018W03Oool0oooo0?oog`2LB0000`0000000`1d_ol0oooo
0?ooo`090?ooo`030000003oool0oooo0<X0oooo00<000000?ooo`3oool02@3oool00;00oooo00L0
oono07@0001d_ol0oooo0?noM00007@0_ooo0080oooo00<0okmd0000001d_ol0103oool00`3ogi`0
B00007Boo`030?ooo`030?oog`2LB7@0_ooo0080oooo00<0_g@007Boo`3oool02@3oool00`000000
oooo0?ooo`3:0?ooo`030000003oool0oooo00T0oooo002`0?ooo`030;md0000B9`0gooo0080oooo
00<0_g@0001d_`3oool0203oool00`3ogi`0B00007Boo`080?ooo`030?oog`2LB7@0_ooo00T0oooo
00<000000?ooo`3oool0bP3oool00`000000oooo0?ooo`090?ooo`00[`3oool00`3ookl0M00004RL
g`030?ooo`030=nLB00007@0_ooo00P0oooo00<0omnL04P0001d_ol02@3oool00`2oM000M;oo0?oo
o`080?ooo`030000003oool0oooo0<X0oooo00<000000?ooo`3oool02@3oool00:<0oooo00<0oooO
09a8000000001`0000001@18W=l0oooo0?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0203o
ool00`3ogi`0B00007Boo`080?ooo`030?oog`2LB000B9cO00T0oooo00<000000?ooo`3oool0b@3o
ool00`000000oooo0?ooo`0:0?ooo`00[`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0
W=oo00P0oooo00<0omnL04P0001d_ol0203oool00`3OW4P0000007Boo`090?ooo`@00000b03oool0
0`000000oooo0?ooo`0:0?ooo`00[`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo
00P0oooo00<0omnL04P0001d_ol0103oool01`3Ogml0oooo0?oo_`1d000000000000M02oool02@3o
ool00`000000oooo0?ooo`390?ooo`030000003oool0oooo00X0oooo002_0?ooo`030?oOW0180000
M;oo00<0oooo00<0okmd0000B02Lgol0203oool00`3ogi`0B00007Boo`040?ooo`060=nLB0000000
00000000000004P0W=oo2P3oool00`000000oooo0?ooo`390?ooo`030000003oool0oooo00X0oooo
002_0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool0203oool00`3ogi`0B00007Bo
o`040?ooo`040?oo_`1d0000000004RLg``0oooo00<000000?ooo`3oool0b@3oool00`000000oooo
0?ooo`0:0?ooo`00/03oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00P0oooo00<0
omnL04P0001d_ol01@3oool00`2oM000M;oo0?ooo`0<0?ooo`030000003oool0oooo0<T0oooo00<0
00000?ooo`3oool02P3oool00;00oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00L0oooo
00D0oooO09a800000000000007Boo`050?ooo`060?oOW018000000000000000004P0W=oo2@3oool0
0`000000oooo0?ooo`390?ooo`030000003oool0oooo00X0oooo002a0?ooo`050?oo_`1d00000000
0000M02oool02P3oool00`3o_g@0000007Boo`060?ooo`050;md00000000000000000018W=l02@3o
ool00`000000oooo0?ooo`390?ooo`030000003oool0oooo00X0oooo003G0?ooo`030000003oool0
oooo0<T0oooo00<000000?ooo`3oool02P3oool00=L0oooo0`0000380?ooo`030000003oool0oooo
00/0oooo003G0?ooo`030000003oool0oooo0<P0oooo00<000000?ooo`3oool02`3oool00=L0oooo
00<000000?ooo`3oool0b03oool00`000000oooo0?ooo`0;0?ooo`00e`3oool00`000000oooo0?oo
o`380?ooo`030000003oool0oooo00/0oooo003G0?ooo`030000003oool0oooo0<P0oooo00<00000
0?ooo`3oool02`3oool00=L0oooo00<000000?ooo`3oool0b03oool00`000000oooo0?ooo`0;0?oo
o`00e`3oool00`000000oooo0?ooo`380?ooo`030000003oool0oooo00/0oooo003G0?ooo`<00000
b03oool00`000000oooo0?ooo`0;0?ooo`00e`3oool00`000000oooo0?ooo`380?ooo`030000003o
ool0oooo00/0oooo003G0?ooo`030000003oool0oooo0<L0oooo00<000000?ooo`3oool0303oool0
0=L0oooo00<000000?ooo`3oool0a`3oool00`000000oooo0?ooo`0<0?ooo`00e`3oool00`000000
oooo0?ooo`370?ooo`030000003oool0oooo00`0oooo003G0?ooo`030000003oool0oooo0<L0oooo
00<000000?ooo`3oool0303oool00=L0oooo00<000000?ooo`3oool0a`3oool00`000000oooo0?oo
o`0<0?ooo`00e`3oool00`000000oooo0?ooo`370?ooo`030000003oool0oooo00`0oooo003G0?oo
o`<00000a`3oool00`000000oooo0?ooo`0<0?ooo`00e`3oool00`000000oooo0?ooo`370?ooo`03
0000003oool0oooo00`0oooo003G0?ooo`030000003oool0oooo0<L0oooo00<000000?ooo`3oool0
303oool00=L0oooo00<000000?ooo`3oool0a`3oool00`000000oooo0?ooo`0<0?ooo`00e`3oool0
0`000000oooo0?ooo`360?ooo`030000003oool0oooo00d0oooo003G0?ooo`030000003oool0oooo
0<H0oooo00<000000?ooo`3oool03@3oool00=L0oooo00<000000?ooo`3oool0<P3oool6000008h0
oooo00<000000?ooo`3oool03@3oool00=L0oooo00<000000?ooo`3oool0;`3oool3000000H0oooo
0P00002<0?ooo`030000003oool0oooo00d0oooo003G0?ooo`<00000;P3oool00`000000oooo0?oo
o`090?ooo`030000003oool0oooo08T0oooo00<000000?ooo`3oool03@3oool00=L0oooo00<00000
0?ooo`3oool0;03oool2000000d0oooo0P0000290?ooo`030000003oool0oooo00d0oooo003G0?oo
o`030000003oool0oooo02/0oooo0P00000@0?ooo`030000003oool0oooo08H0oooo00<000000?oo
o`3oool03@3oool00;X0oooo00D0omnL04P000000000001d0;ooo`030?ooo`030?noM0000000M;oo
00<0oooo00H0_g@000000000000000000018W03Oool90?ooo`030000003oool0oooo02X0oooo0P00
000B0?ooo`800000QP3oool00`000000oooo0?ooo`0=0?ooo`00^@3oool01`3ookl0M00007Boo`3o
ool0okmd0000M02oool00P3oool00`3o_g@0000007Boo`040?ooo`030?oOW0180000M;oo00/0oooo
00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo08<0oooo
00<000000?ooo`3oool03@3oool00;T0oooo00<0_g@00018W03Oool00P3oool00`2oM00007Bo0?oo
o`080?ooo`030?oOW0180000M;oo00/0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?oo
o`0F0?ooo`030000003oool0oooo0840oooo00<000000?ooo`3oool03P3oool00;P0oooo00<0oono
07@00018W=l00`3oool00`3OW4P0001d0;ooo`080?ooo`030?oOW0180000M;oo00/0oooo00<00000
0?ooo`3oool09`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo0800oooo00<00000
0?ooo`3oool03P3oool00:`0oooo00<0oooO09a8000000001`0000001@18W=l0oooo0?oOW0180000
M;oo00<0oooo00<0okmd0000B02Lgol0203oool00`3ogi`0B00007Boo`0;0?ooo`030000003oool0
oooo02H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`1o0?ooo`030000003oool0
oooo00h0oooo002h0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0203oool00`3o
gi`0B00007Boo`0;0?ooo`@000009@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
07l0oooo00<000000?ooo`3oool03P3oool00;P0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0
001809cOo`080?ooo`030?oOW0180000M;oo00/0oooo00<000000?ooo`3oool09@3oool00`000000
oooo0?ooo`0L0?ooo`030000003oool0oooo07h0oooo00<000000?ooo`3oool03P3oool00;P0oooo
00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`080?ooo`030?oOW0180000M;oo00/0oooo
00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo07d0oooo
00<000000?ooo`3oool03P3oool00;P0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;oo
o`080?ooo`030?oOW0180000M;oo00/0oooo00<000000?ooo`3oool0903oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo07`0oooo00<000000?ooo`3oool03P3oool00;T0oooo00<0gia8
0018W03Oool00P3oool00`2oM00007Bo0?ooo`080?ooo`030?oOW0180000M;oo00/0oooo00<00000
0?ooo`3oool08`3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo07`0oooo00<00000
0?ooo`3oool03P3oool00;T0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00L0oooo00D0
oooO09a800000000000007Boo`0;0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0
8P3oool00`000000oooo0?ooo`1j0?ooo`030000003oool0oooo00l0oooo002j0?ooo`050?oo_`1d
000000000000M02oool02P3oool00`3o_g@0000007Boo`0;0?ooo`030000003oool0oooo0240oooo
0P00000U0?ooo`030000003oool0oooo07T0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`090?ooo`00e`3oool300000200oooo0P00000V0?ooo`030000003oool0oooo07T0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`090?ooo`00e`3oool00`000000oooo0?ooo`0P
0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0N03oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo00T0oooo003G0?ooo`030000003oool0oooo01l0oooo00<000000?oo
o`3oool0:03oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool02P3oool00=L0oooo00<000000?ooo`3oool07P3oool2000002X0oooo00<000000?ooo`3o
ool0M`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00X0oooo003G0?ooo`030000
003oool0oooo01d0oooo0P00000/0?ooo`030000003oool0oooo07H0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`0:0?ooo`00e`3oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo02`0oooo00<000000?ooo`3oool0M03oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo00X0oooo003G0?ooo`030000003oool0oooo01`0oooo0P00000^0?ooo`030000003oool0
oooo07@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0:0?ooo`00e`3oool00`00
0000oooo0?ooo`0L0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0L`3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo00X0oooo003G0?ooo`<000006`3oool200000340
oooo00<000000?ooo`3oool0LP3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00/0
oooo003G0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0<03oool00`000000oooo
0?ooo`1b0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool02`3oool00=L0oooo00<0
00000?ooo`3oool06P3oool2000003<0oooo00<000000?ooo`3oool0L@3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo00/0oooo003G0?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool0<`3oool00`000000oooo0?ooo`1_0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool02`3oool00=L0oooo00<000000?ooo`3oool06@3oool2000003D0oooo00<000000?oo
o`3oool0K`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00/0oooo003G0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`1^0?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool02`3oool00=L0oooo00<000000?ooo`3oool0
603oool2000003P0oooo00<000000?ooo`3oool0K@3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo00`0oooo003G0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0=`3o
ool00`000000oooo0?ooo`1]0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0303o
ool00=L0oooo0`00000G0?ooo`800000>P3oool00`000000oooo0?ooo`1[0?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool0303oool00=L0oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`0j0?ooo`030000003oool0oooo06X0oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`0<0?ooo`00e`3oool00`000000oooo0?ooo`0F0?ooo`800000?03oool00`000000
oooo0?ooo`1Z0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0303oool00=L0oooo
00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo06T0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0=0?ooo`00e`3oool00`000000oooo0?oo
o`0E0?ooo`800000?P3oool00`000000oooo0?ooo`1Y0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool03@3oool00=L0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0n
0?ooo`030000003oool0oooo06P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0=
0?ooo`00e`3oool00`000000oooo0?ooo`0D0?ooo`800000@@3oool00`000000oooo0?ooo`1V0?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool03@3oool00=L0oooo0`00000D0?ooo`03
0000003oool0oooo0400oooo00<000000?ooo`3oool0IP3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo00d0oooo003G0?ooo`030000003oool0oooo01<0oooo0P0000130?ooo`030000
003oool0oooo06D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0>0?ooo`00e`3o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool0I03o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00h0oooo002a0?ooo`050?oOW0180000
00000000M02oool00`3oool00`3o_g@0000007Boo`030?ooo`050?oOW018000000000000M02oool0
0P3oool00`3ooml0W4P0000000020000000307Boo`3oool0oooo00T0oooo00<000000?ooo`3oool0
4P3oool2000004D0oooo00<000000?ooo`3oool0I03oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo00h0oooo002`0?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`020?ooo`03
0?noM0000000M;oo0080oooo00/0oono07@0001d_ol0oooo0?noM00007@0_ooo0?ooo`3ooml0W4Qd
0;ooo`020?ooo`030;md001d_ol0oooo00T0oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`150?ooo`030000003oool0oooo0680oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`0>0?ooo`00/03oool00`2oM00004RL0=ooo`020?ooo`030;md0000M;l0oooo00H0oooo00<0
_g@00018W03Oool00P3oool00`2oM00007Bo0?ooo`050?ooo`030?oog`2LB7@0_ooo00T0oooo00<0
00000?ooo`3oool04@3oool2000004P0oooo00<000000?ooo`3oool0H@3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo00h0oooo002_0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia8
0000M02oool01@3oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00H0oooo00<0_g@0
07Boo`3oool0203oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo04L0oooo00<00000
0?ooo`3oool0H@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00l0oooo002S0?oo
o`030?oog`2LB000000000L0000000D0B9cO0?ooo`3ogi`0B00007Boo`030?ooo`030?noM00004P0
W=oo00D0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`050?ooo`030?oog`2LB000
B9cO00T0oooo00<000000?ooo`3oool0403oool2000004X0oooo00<000000?ooo`3oool0H03oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo00l0oooo002_0?ooo`030?oOW0180000M;oo
00<0oooo00<0okmd0000B02Lgol01@3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo
00D0oooo00<0gia80000001d_ol02@3oool4000000l0oooo00<000000?ooo`3oool0BP3oool00`00
0000oooo0?ooo`1O0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03`3oool00:l0
oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`050?ooo`030?oOW0180000M;oo00<0
oooo00/0okmd0000B02Lgol0oooo0=oOg`3oool0oono07@000000000001d0;ooo`090?ooo`030000
003oool0oooo00l0oooo0P00001<0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`0?0?ooo`00[`3oool00`3ogi`0B00007Boo`030?ooo`030?noM000
04P0W=oo00D0oooo00<0omnL04P0001d_ol00`3oool01@3o_g@0001809cOo`3oool0gia800<00000
00<0001809cOo`3oool02@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo04`0oooo
00<000000?ooo`3oool0G@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0100oooo
002_0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool01@3oool00`3ookl0M00004RL
g`030?ooo`080=nLB00007@0_ooo0?ooo`3ookl0M00000000018W=l<0?ooo`030000003oool0oooo
00h0oooo0P00001>0?ooo`030000003oool0oooo05d0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0@0?ooo`00/03oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00H0
oooo00<0gia80018W03Oool00P3oool00`2oM00007Bo0?ooo`020?ooo`030;md001d_ol0oooo00`0
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo05`0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0@0?ooo`00/03oool01`3ookl0M000
07Boo`3oool0okmd0000M02oool01`3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool00`3o
ool01P3ogi`0B000000000000000001809cOo`T0oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`1?0?ooo`030000003oool0oooo05X0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`0@0?ooo`00/@3oool01@3ookl0M0000000000007@0_ooo00T0oooo00D0oono07@00000
0000001d0;ooo`050?ooo`050;md00000000000000000018W=l02@3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo0500oooo00<000000?ooo`3oool0FP3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0140oooo003G0?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool0D@3oool00`000000oooo0?ooo`1I0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool04@3oool00=L0oooo0`00000<0?ooo`030000003oool0oooo05<0oooo00<000000?ooo`3o
ool0E`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0140oooo003G0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0E03oool00`000000oooo0?ooo`1F0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool04P3oool00=L0oooo00<000000?ooo`3oool02`3o
ool00`000000oooo0?ooo`1F0?ooo`030000003oool0oooo05D0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`0B0?ooo`00e`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo05L0oooo00<000000?ooo`3oool0D`3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo0180oooo003G0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0F03oool00`00
0000oooo0?ooo`1C0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04`3oool00=L0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`1I0?ooo`030000003oool0oooo0580
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0C0?ooo`00e`3oool00`000000oooo
0?ooo`090?ooo`030000003oool0oooo05/0oooo00<000000?ooo`3oool0D03oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo01<0oooo003G0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool0G03oool00`000000oooo0?ooo`1?0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool0503oool00=L0oooo0`0000080?ooo`030000003oool0oooo05h0oooo00<00000
0?ooo`3oool0CP3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01@0oooo000i0?oo
o`040?oog`2LB00004RL0=ooodX0oooo00<0_g@0000000000000100000000`18W=l0oooo0?ooo`16
0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0?`3oool00`2oM000000000000004
0000000304RLg`3oool0oooo01H0oooo00<000000?ooo`3oool0<`3oool0103ooml0W4P00018W03O
oolE0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool05@3oool003T0oooo00@0oooO
09a80000B9`0goooB`3oool00`3OW4P0000000000003000000030018W03Oool0oooo04H0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`110?ooo`030=nLB0000000000000<0000000<0
04RL0=ooo`3oool05`3oool00`000000oooo0?ooo`0b0?ooo`040?oog`2LB00004RL0=oooaD0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo
000i0?ooo`040?oog`2LB00004RL0=oood`0oooo00<0gia804RLg`3oool00P3oool00`3OW7@0W=oo
0?ooo`160?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0@P3oool00`3OW4P0B9cO
0?ooo`020?ooo`030=nLM02Lgol0oooo01P0oooo00<000000?ooo`3oool0<@3oool0103ooml0W4P0
0018W03OoolD0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`090?ooo`00=@3oool00`2oM0000000000000050000000307Boo`3oool0oooo04X0oooo
00<0gia807Boo`3oool0BP3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo04@0oooo
00<0gia807Boo`3oool07@3oool00`000000oooo0?ooo`0/0?ooo`030;md00000000000000D00000
00<0M;oo0?ooo`3oool04@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00T0oooo
00<000000?ooo`3oool02P3oool002P0oooo00<0oooO09a8000000001`0000001@18W=l0oooo0?oo
o`3ogi`0M9cO0080oooo00@0oooO09a80000B9`0gooo?@3oool00`3ooml0W4P00000000700000003
04RLg`3oool0oooo00@0oooo00<0_gA809cOo`3oool0B@3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo04D0oooo00<0_gA809cOo`3oool07@3oool00`000000oooo0?ooo`0[0?ooo`08
0?oOW01dW=l0oooo0?ooo`3ooml0W4P00018W03OoolC0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`00=P3oool01`3OW4P0M;oo0?ooo`3o
oml0W4P00018W03Oool0CP3oool00`3ooml0W4Qd0;ooo`190?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool0AP3oool00`3ooml0W4Qd0;ooo`0N0?ooo`030000003oool0oooo02/0oooo
00L0gia807Boo`3oool0oooO09a80000B9`0gooo0180oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00/0oooo000f0?ooo`070?oog`2LB7@0_ooo0?oo
g`2LB00004RL0=ooo`1?0?ooo`030?noM000M;l0oooo04P0oooo0`0000050?ooo`030000003oool0
oooo04L0oooo00<0okmd001d_`3oool07P3oool2000002/0oooo00L0oooO09a8M02oool0oooO09a8
0000B9`0gooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00/0oooo000g0?ooo`060?oOW018M;l0oooO09a80000B9`0goooC`3oool00`3ookl0
M01809cOo`180?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0B03oool00`3ookl0
M01809cOo`0P0?ooo`030000003oool0oooo02T0oooo00H0omnL04Qd_`3ooml0W4P00018W03OoolA
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0<
0?ooo`00>03oool01@3OW4P0M;oO09a80000B9`0gooo04l0oooo00<0oono07@00018W=l0B03oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo04P0oooo00<0oono07@00018W=l08@3oool0
0`000000oooo0?ooo`0Y0?ooo`050=nLB01d_ml0W4P00018W03Oool0403oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool03@3oool003P0oooo00D0oooO
09a8B018B00004RL0=ooo`1:0?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l0B03o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo04@0oooo00<0oknL09cOo`3oool00P3o
ool00`3ogi`0B00004RLg`0R0?ooo`800000:@3oool01@3ooml0W4Q804Q80000B9`0gooo00l0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00d0oooo
000i0?ooo`040?oOW018000004RL0=ooodX0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d
0;ooodP0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`140?ooo`080?oog`2LB4P0
W=oo0?ooo`3ookl0M0000000M02ooolS0?ooo`800000:@3oool0103ogi`0B0000018W03Oool>0?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0>0?oo
o`00>P3oool00`3OW4P004RL0=ooo`1;0?ooo`060?oog`2LB00000000000000007@0_oooB@3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo04H0oooo00H0oooO09a80000000000000000
M02ooolV0?ooo`800000:03oool00`3OW4P004RL0=ooo`0=0?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0?0?ooo`00e`3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo07<0oooo0`00000e0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`0@0?ooo`00e`3oool01@000000oooo0?ooo`3o
ool0000007L0oooo1000000b0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0A0?ooo`00e`3oool3000000030?ooo`000000oooo07P0oooo1000000_
0?ooo`030000003oool0oooo0080oooo0P00000<0?ooo`030000003oool0oooo0180oooo003G0?oo
o`040000003oool0oooo000007`0oooo1000000[0?ooo`800000103oool00`000000oooo0?ooo`0:
0?ooo`8000005@3oool00=L0oooo00@000000?ooo`3oool00000OP3oool4000002L0oooo0P000004
0?ooo`800000303oool00`000000oooo0?ooo`0E0?ooo`00e`3oool00`000000oooo000000220?oo
o`H000007P3oool4000000@0oooo0P00000;0?ooo`<00000603oool00=L0oooo00<000000?ooo`00
0000QP3oool6000001H0oooo100000040?ooo`@000002P3oool3000001/0oooo003G0?ooo`800000
R@3ooolJ00000080oooo1P0000090?ooo`D000007P3oool00=L0oooo0P00002<0?oooaT000002@3o
ool6000002<0oooo003G0?ooo`030000003oool0oooo08l0oooo7P00000Y0?ooo`00o`00002`0000
0040oooo000:0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`020?ooo`T000001`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0180
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0@0?ooo`8000004`3oool00`000000
oooo0?ooo`0B0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04P3oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0=0?ooo`<0000000<0oooo0000000000000P0000040?ooo`@000002@3oool00`000000
oooo0?ooo`0B0?ooo`030000003oool0oooo00T0oooo000c0?ooo`030000003oool0oooo00/0oooo
2000000l0?ooo`030000003oool0oooo04h0oooo0P00001A0?ooo`030000003oool0oooo04d0oooo
0`0000020?ooo`@00000103oool5000002D0oooo00190?ooo`@00000R03oool00`000000oooo0000
002S0?ooo`<00000103oool3000000H0oooo0`00000R0?ooo`00C@3oool2000008H0oooo00<00000
0?ooo`000000YP3oool3000000@0oooo0P0000070?ooo`<000007`3oool004l0oooo0`0000220?oo
o`040000003oool0oooo00000:T0oooo0P0000040?ooo`800000203oool2000001d0oooo001B0?oo
o`<00000O`3oool010000000oooo0?ooo`00002[0?ooo`800000103oool2000000P0oooo0P00000K
0?ooo`00E@3oool2000007`0oooo00@000000?ooo`3oool0oooo0`00002[0?ooo`800000103oool2
000000P0oooo0P00000I0?ooo`00E`3oool2000007X0oooo00D000000?ooo`3oool0oooo0000002_
0?ooo`800000103oool00`000000oooo0?ooo`070?ooo`8000005`3oool005T0oooo0P00001g0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool0[`3oool2000000<0oooo00<000000?oo
o`3oool0203oool00`000000oooo0?ooo`0D0?ooo`00F`3oool00`000000oooo0?ooo`1d0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0/@3oool01@000000oooo0?ooo`3oool00000
00X0oooo00<000000?ooo`3oool04`3oool005`0oooo0P00001c0?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool0/P3oool01@000000oooo0?ooo`3oool0000000X0oooo00<000000?oo
o`3oool04P3oool005h0oooo0P00001a0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0/P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3o
ool04@3oool00600oooo00<000000?ooo`3oool0K@3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo0;<0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo0140oooo001Q0?ooo`030000003oool0oooo06`0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`2d0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`0@0?ooo`00HP3oool2000006/0oooo00<000000?ooo`3oool01@3oool3
00000;@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo0100oooo001T0?ooo`030000003oool0oooo06P0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`2e0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`0?0?ooo`00I@3oool00`000000oooo0?ooo`1V0?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool0]P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool03`3oool006H0oooo0P00001V0?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool0]P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool03`3oool006P0oooo00<000000?ooo`3oool0HP3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo0;L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00h0oooo001Y0?ooo`030000003oool0oooo0640oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`2g0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`0>0?ooo`00JP3oool00`000000oooo0?ooo`1O0?oo
o`030000003oool0oooo00P0oooo00<000000?ooo`3oool0^03oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool03@3oool006/0oooo00<000000?ooo`3o
ool0GP3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0;P0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00d0oooo001/0?ooo`030000
003oool0oooo05`0oooo00<000000?ooo`3oool02@3oool300000;T0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00`0oooo001]0?ooo`030000003o
ool0oooo05/0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`2i0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0<0?ooo`00K@3oool0
0`000000oooo0?ooo`1J0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0^P3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02`3oool0
06h0oooo00<000000?ooo`3oool0F@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
0;X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
00/0oooo001_0?ooo`030000003oool0oooo05L0oooo00<000000?ooo`3oool02`3oool00`000000
oooo0?ooo`2k0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`0:0?ooo`00L03oool00`000000oooo0?ooo`1F0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0^`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool02P3oool00740oooo00<000000?ooo`3oool0E03oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo0;/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00T0oooo001b0?ooo`030000003oool0oooo05<0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`2k0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`00LP3oool00`000000oooo0?ooo`1B0?oo
o`030000003oool0oooo00d0oooo0`00002l0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool04P3oool007<0oooo00<000000?ooo`3oool0D@3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo0;`0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0B0?ooo`00
M03oool00`000000oooo0?ooo`0j0?ooo`050?oOW018000000000000M02oool00`3oool00`3o_g@0
000007Boo`030?ooo`050?oOW018000000000000M02oool00P3oool00`000000W4P0000000020000
000307Boo`3oool0oooo00T0oooo00<000000?ooo`3oool0_03oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0180oooo001d0?ooo`030000003oool0oooo03T0oooo00L0oono07@0001d
_ol0oooo0?noM00007@0_ooo0080oooo00<0okmd0000001d_ol00P3oool02`3ookl0M00007Boo`3o
ool0okmd0000M02oool0oooo0000002LB7@0_ooo0080oooo00<0_g@007Boo`3oool02@3oool00`00
0000oooo0?ooo`2l0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool04@3oool007D0
oooo00<000000?ooo`3oool0>03oool00`2oM00004RL0=ooo`020?ooo`030;md0000M;l0oooo00H0
oooo00<0_g@00018W03Oool00P3oool00`2oM00007Bo000000050?ooo`030?oog`2LB7@0_ooo00T0
oooo00<000000?ooo`3oool0_@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0140
oooo001e0?ooo`030000003oool0oooo03L0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d
0;ooo`050?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000000000001P3oool00`2oM000M;oo
0?ooo`080?ooo`030000003oool0oooo0;d0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0A0?ooo`00MP3oool00`000000oooo0?ooo`0f0?ooo`030?oOW0180000M;oo00<0oooo00<0
okmd0000B02Lgol01@3oool00`3ogi`0B00007Boo`030?ooo`030?noM0000000W=oo00D0oooo00<0
oooO09a80018W=l02@3oool00`000000oooo0?ooo`2m0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool04@3oool007L0oooo00<000000?ooo`3oool0=@3oool00`3ogi`0B00007Boo`03
0?ooo`030?noM00004P0W=oo00D0oooo00<0omnL04P0001d_ol00`3oool20000000309cOo`3oool0
oooo00<0oooo00<0gia80000001d_ol02@3oool400000;`0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0@0?ooo`00M`3oool00`000000oooo0?ooo`0e0?ooo`030?oOW0180000M;oo
00<0oooo00<0okmd0000B02Lgol01@3oool00`3ogi`0B00007Boo`030?ooo`0;0000000004P0W=oo
0?ooo`3Ogml0oooo0?oo_`1d000000000000M02oool02@3oool00`000000oooo0?ooo`2n0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0403oool007P0oooo00<000000?ooo`3oool0
=03oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00D0oooo00<0omnL04P0001d_ol0
0P3oool2000000040000B02Lgol0oooo0=nLB0<0000000<0001809cOo`3oool02@3oool00`000000
oooo0?ooo`2n0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0403oool007T0oooo
00<000000?ooo`3oool0<`3oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00D0oooo
00<0oono07@00018W=l00P3oool02@000000gia80000M02oool0oooo0?oo_`1d0000000004RLg`0<
0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0@
0?ooo`00N@3oool00`000000oooo0?ooo`0d0?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d
_`3oool01P3oool00`3OW4P004RL0=ooo`02000000030;md0000M;l0oooo0080oooo00<0_g@007Bo
o`3oool0303oool00`000000oooo0?ooo`2n0?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool03`3oool007X0oooo00<000000?ooo`3oool0<`3oool01`3ookl0M00007Boo`3oool0okmd
0000M02oool01`3oool01`3ookl0M00007Boo`000000okmd0000M02oool00`3oool01P3ogi`0B000
000000000000001809cOo`T0oooo00<000000?ooo`3oool0_`3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00l0oooo001k0?ooo`030000003oool0oooo03<0oooo00D0oono07@00000
0000001d0;ooo`090?ooo`050?oo_`00000000000000M02oool01@3oool01@2oM000000000000000
0000B9cO00T0oooo00<000000?ooo`3oool0_`3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo00l0oooo001k0?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`2o0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03`3oool0
07`0oooo00<000000?ooo`3oool0@03oool2000001H0oooo0`00002o0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool03`3oool007d0oooo00<000000?ooo`3oool0?`3oool00`000000
oooo0?ooo`0E0?ooo`030000003oool0oooo0<00oooo00D000000?ooo`3oool0oooo0000000A0?oo
o`00O@3oool00`000000oooo0?ooo`0n0?ooo`8000005`3oool00`000000oooo0?ooo`300?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool03P3oool007h0oooo00<000000?ooo`3oool0
?@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0<00oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`0>0?ooo`00OP3oool00`000000oooo0?ooo`0l0?ooo`800000603o
ool00`000000oooo0?ooo`300?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03P3o
ool007l0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0
oooo0<00oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0>0?ooo`00P03oool00`00
0000oooo0?ooo`0i0?ooo`8000006@3oool00`000000oooo0?ooo`310?ooo`050000003oool0oooo
0?ooo`000000403oool00800oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo0<40oooo00D000000?ooo`3oool0oooo0000000@0?ooo`00P@3oool00`00
0000oooo0?ooo`0g0?ooo`8000006P3oool300000<40oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0=0?ooo`00PP3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool0`@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00d0
oooo00220?ooo`030000003oool0oooo03D0oooo0P00000K0?ooo`030000003oool0oooo0<40oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0=0?ooo`00P`3oool00`000000oooo0?oo
o`0d0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0`@3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo00d0oooo00240?ooo`030000003oool0oooo0380oooo0P00000L
0?ooo`030000003oool0oooo0<80oooo00D000000?ooo`3oool0oooo0000000?0?ooo`00Q03oool0
0`000000oooo0?ooo`0b0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0`P3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00250?ooo`030000003oool0oooo
0300oooo0P00000M0?ooo`030000003oool0oooo0<80oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0<0?ooo`00Q@3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo01`0
oooo00<000000?ooo`3oool0`P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00`0
oooo00260?ooo`030000003oool0oooo02h0oooo0P00000N0?ooo`<00000`P3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00`0oooo00270?ooo`030000003oool0oooo02d0oooo00<0
00000?ooo`3oool07@3oool00`000000oooo0?ooo`330?ooo`050000003oool0oooo0?ooo`000000
3P3oool008L0oooo00<000000?ooo`3oool0;03oool2000001l0oooo00<000000?ooo`3oool0``3o
ool01@000000oooo0?ooo`3oool0000000h0oooo00280?ooo`030000003oool0oooo02/0oooo00<0
00000?ooo`3oool07P3oool00`000000oooo0?ooo`330?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool02`3oool008T0oooo00<000000?ooo`3oool0:@3oool200000200oooo00<00000
0?ooo`3oool0``3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00/0oooo00290?oo
o`030000003oool0oooo02T0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`330?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool02`3oool008X0oooo00<000000?ooo`3o
ool09`3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0<<0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`0;0?ooo`00R`3oool00`000000oooo0?ooo`0V0?ooo`030000
003oool0oooo0200oooo0`0000330?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
2`3oool008/0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo0<@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0:0?ooo`00S03oool0
0`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0a03oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo00X0oooo002=0?ooo`030000003oool0oooo
0280oooo00<000000?ooo`3oool01@3oool01@3ogi`0B0000000000007@0_ooo00<0oooo00<0okmd
0000001d_ol00`3oool01P2oM000000000000000000004RL0=ooo`T0oooo00<000000?ooo`3oool0
a03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00X0oooo002=0?ooo`030000003o
ool0oooo0240oooo00<000000?ooo`3oool01@3oool01`3ookl0M00007Boo`3oool0okmd0000M02o
ool00P3oool00`3o_g@0000007Boo`040?ooo`030?oOW0180000M;oo00/0oooo00<000000?ooo`3o
ool0a03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00X0oooo002>0?ooo`030000
003oool0oooo01l0oooo00<000000?ooo`3oool01P3oool00`2oM00004RL0=ooo`020?ooo`030;md
0000M;l0oooo00P0oooo00<0omnL04P0001d_ol02`3oool00`000000oooo0?ooo`340?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool02P3oool008l0oooo00<000000?ooo`3oool07P3o
ool00`000000oooo0?ooo`050?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool0203o
ool00`3ogi`0B00007Boo`0;0?ooo`030000003oool0oooo0<@0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`0:0?ooo`00T03oool00`000000oooo0?ooo`0L0?ooo`030000003oool0
oooo00H0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`080?ooo`030?oOW0180000
M;oo00/0oooo00<000000?ooo`3oool0a03oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo00X0oooo002@0?ooo`800000703oool00`000000oooo0?ooo`070?ooo`030?oOW0180000M;oo
00<0oooo00<0okmd0000B02Lgol0203oool00`3ogi`0B00007Boo`0;0?ooo`@00000``3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo00T0oooo002A0?ooo`8000006`3oool00`000000
oooo0?ooo`070?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0203oool00`3ogi`0
B00007Boo`0;0?ooo`030000003oool0oooo0<D0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`090?ooo`00TP3oool2000001T0oooo00<000000?ooo`3oool0203oool00`3ogi`0B000
07Boo`030?ooo`030?noM00004P0W=oo00P0oooo00<0omnL04P0001d_ol02`3oool00`000000oooo
0?ooo`350?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool02@3oool009<0oooo0P00
000G0?ooo`030000003oool0oooo00T0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;oo
o`080?ooo`030?oOW0180000M;oo00/0oooo00<000000?ooo`3oool0a@3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo00T0oooo002D0?ooo`800000503oool2000000d0oooo00<0gia8
0018W03Oool00P3oool00`2oM00007Bo0?ooo`080?ooo`030?oOW0180000M;oo00/0oooo00<00000
0?ooo`3oool0a@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00T0oooo002E0?oo
o`8000004P3oool00`000000oooo0?ooo`0=0?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;oo
o`070?ooo`050?oog`2LB00000000000001d_ol02`3oool00`000000oooo0?ooo`350?ooo`030000
003oool0oooo00h0oooo002G0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03`3o
ool01@3ookl0M0000000000007@0_ooo00X0oooo00<0okmd0000001d_ol02`3oool00`000000oooo
0?ooo`350?ooo`030000003oool0oooo00h0oooo002H0?ooo`800000303oool2000002l0oooo00<0
00000?ooo`3oool0a@3oool00`000000oooo0?ooo`0>0?ooo`00VP3oool2000000P0oooo0P00000a
0?ooo`<00000a@3oool00`000000oooo0?ooo`0>0?ooo`00W03oool900000380oooo00<000000?oo
o`3oool0aP3oool00`000000oooo0?ooo`0=0?ooo`00X03oool00`000000oooo0?ooo`0d0?ooo`03
0000003oool0oooo0<H0oooo00<000000?ooo`3oool03@3oool00=L0oooo00<000000?ooo`3oool0
aP3oool00`000000oooo0?ooo`0=0?ooo`00e`3oool00`000000oooo0?ooo`360?ooo`030000003o
ool0oooo00d0oooo003G0?ooo`030000003oool0oooo0<H0oooo00<000000?ooo`3oool03@3oool0
0=L0oooo00<000000?ooo`3oool0aP3oool00`000000oooo0?ooo`0=0?ooo`00e`3oool300000<H0
oooo00<000000?ooo`3oool03@3oool00=L0oooo00<000000?ooo`3oool0aP3oool00`000000oooo
0?ooo`0=0?ooo`00e`3oool00`000000oooo0?ooo`360?ooo`030000003oool0oooo00d0oooo003G
0?ooo`030000003oool0oooo0<H0oooo00<000000?ooo`3oool03@3oool00=L0oooo00<000000?oo
o`3oool0a`3oool00`000000oooo0?ooo`0<0?ooo`00e`3oool00`000000oooo0?ooo`370?ooo`03
0000003oool0oooo00`0oooo003G0?ooo`030000003oool0oooo0<L0oooo00<000000?ooo`3oool0
303oool00=L0oooo00<000000?ooo`3oool0a`3oool00`000000oooo0?ooo`0<0?ooo`00e`3oool3
00000<L0oooo00<000000?ooo`3oool0303oool00=L0oooo00<000000?ooo`3oool0a`3oool00`00
0000oooo0?ooo`0<0?ooo`00e`3oool00`000000oooo0?ooo`370?ooo`030000003oool0oooo00`0
oooo003G0?ooo`030000003oool0oooo0<L0oooo00<000000?ooo`3oool0303oool00=L0oooo00<0
00000?ooo`3oool0a`3oool00`000000oooo0?ooo`0<0?ooo`00e`3oool00`000000oooo0?ooo`37
0?ooo`030000003oool0oooo00`0oooo003G0?ooo`030000003oool0oooo0<P0oooo00<000000?oo
o`3oool02`3oool00=L0oooo00<000000?ooo`3oool0b03oool00`000000oooo0?ooo`0;0?ooo`00
e`3oool300000<P0oooo00<000000?ooo`3oool02`3oool00=L0oooo00<000000?ooo`3oool0b03o
ool00`000000oooo0?ooo`0;0?ooo`00e`3oool00`000000oooo0?ooo`380?ooo`030000003oool0
oooo00/0oooo002a0?ooo`050?oOW018000000000000M02oool00`3oool00`3o_g@0000007Boo`03
0?ooo`090;md00000000000000000000B9`0gooo0?ooo`3ooml0W4P000<0000000<0M;oo0?ooo`3o
ool02@3oool00`000000oooo0?ooo`380?ooo`030000003oool0oooo00/0oooo002`0?ooo`070?oo
_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`020?ooo`030?noM0000000M;oo00@0oooo00<0omnL04P0
001d_ol00`3oool00`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0oooo00T0oooo00<000000?oo
o`3oool0b03oool00`000000oooo0?ooo`0;0?ooo`00/03oool00`2oM00004RL0=ooo`020?ooo`03
0;md0000M;l0oooo00P0oooo00<0omnL04P0001d_ol0203oool00`3ooml0W4Qd0;ooo`090?ooo`03
0000003oool0oooo0<P0oooo00<000000?ooo`3oool02`3oool00:l0oooo00<0oono07@00018W=l0
0`3oool00`3OW4P0001d0;ooo`080?ooo`030?oOW0180000M;oo00T0oooo00<0_g@007Boo`3oool0
203oool00`000000oooo0?ooo`380?ooo`030000003oool0oooo00/0oooo002_0?ooo`030?oOW018
0000M;oo00<0oooo00<0okmd0000B02Lgol0203oool00`3ogi`0B00007Boo`080?ooo`030?oog`2L
B000B9cO00T0oooo00<000000?ooo`3oool0b03oool00`000000oooo0?ooo`0;0?ooo`00[`3oool0
0`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00P0oooo00<0omnL04P0001d_ol0203oool0
0`3OW4P0000007Boo`090?ooo`@00000b03oool00`000000oooo0?ooo`0:0?ooo`00[`3oool00`3o
gi`0B00007Boo`030?ooo`030?noM00004P0W=oo00P0oooo00<0omnL04P0001d_ol0103oool01`3O
gml0oooo0?oo_`1d000000000000M02oool02@3oool00`000000oooo0?ooo`390?ooo`030000003o
ool0oooo00X0oooo002_0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0203oool0
0`3ogi`0B00007Boo`040?ooo`060=nLB000000000000000000004P0W=oo2P3oool00`000000oooo
0?ooo`390?ooo`030000003oool0oooo00X0oooo002_0?ooo`030?oo_`1d0000B9cO00<0oooo00<0
gia80000M02oool0203oool00`3ogi`0B00007Boo`040?ooo`040?oo_`1d0000000004RLg``0oooo
00<000000?ooo`3oool0b@3oool00`000000oooo0?ooo`0:0?ooo`00/03oool00`3OW4P004RL0=oo
o`020?ooo`030;md0000M;l0oooo00P0oooo00<0omnL04P0001d_ol01@3oool00`2oM000M;oo0?oo
o`0<0?ooo`030000003oool0oooo0<T0oooo00<000000?ooo`3oool02P3oool00;00oooo00L0oono
07@0001d_ol0oooo0?noM00007@0_ooo00L0oooo00D0oooO09a800000000000007Boo`050?ooo`06
0?oOW018000000000000000004P0W=oo2@3oool00`000000oooo0?ooo`390?ooo`030000003oool0
oooo00X0oooo002a0?ooo`050?oo_`1d000000000000M02oool02P3oool00`3o_g@0000007Boo`06
0?ooo`050;md00000000000000000018W=l02@3oool00`000000oooo0?ooo`390?ooo`030000003o
ool0oooo00X0oooo003G0?ooo`<00000b@3oool00`000000oooo0?ooo`0:0?ooo`00e`3oool00`00
0000oooo0?ooo`390?ooo`030000003oool0oooo00X0oooo003G0?ooo`030000003oool0oooo0<T0
oooo00<000000?ooo`3oool02P3oool00=L0oooo00<000000?ooo`3oool0bP3oool00`000000oooo
0?ooo`090?ooo`00e`3oool00`000000oooo0?ooo`3:0?ooo`030000003oool0oooo00T0oooo003G
0?ooo`030000003oool0oooo0<X0oooo00<000000?ooo`3oool02@3oool00=L0oooo00<000000?oo
o`3oool0bP3oool00`000000oooo0?ooo`090?ooo`00e`3oool00`000000oooo0?ooo`3:0?ooo`03
0000003oool0oooo00T0oooo003G0?ooo`<00000bP3oool00`000000oooo0?ooo`090?ooo`00e`3o
ool00`000000oooo0?ooo`3F0?ooo`00e`3oool00`000000oooo0?ooo`3F0?ooo`00e`3oool00`00
0000oooo0?ooo`3F0?ooo`00e`3oool00`000000oooo0?ooo`3F0?ooo`00e`3oool00`000000oooo
0?ooo`3F0?ooo`00e`3oool00`000000oooo0?ooo`3F0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {-5.25985, -0.166029, \
0.0366114, 0.00194329}}]
}, Closed]],

Cell[TextData[{
  "This particular family of solutions possesses one node for moderate ",
  StyleBox["x",
    FontSlant->"Italic"],
  " representing approximations to the lowest odd eigenfunction.  For \
energies near 4.4305 the solutions tend to approach zero for positive ",
  Cell[BoxData[
      FormBox[
        RowBox[{"x", "\[Rule]", 
          RowBox[{"-", 
            StyleBox["xmin",
              FontSlant->"Italic"]}]}], TraditionalForm]]],
  ", but diverge unless the energy is precisely the correct eigenvalue.   \
From this figure we deduce that the eigenvalue is between 4.4304 and 4.4306.  \
A more precise value can be obtained by using ",
  StyleBox["FindRoot", "InlineInput",
    FontWeight->"Bold"],
  " to locate the energy for which ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Psi]", "[", 
            RowBox[{"-", 
              StyleBox["xmin",
                FontSlant->"Italic"]}], "]"}], "\[Equal]", 
          RowBox[{"\[Psi]", "[", 
            StyleBox["xmax",
              FontSlant->"Italic"], "]"}], "\[Equal]", "0"}], 
        TraditionalForm]]],
  "."
}], "Text"],

Cell[TextData[{
  StyleBox["Note:",
    FontColor->RGBColor[1, 0, 0]],
  " recent changes to the ",
  StyleBox["FindRoot", "InlineInput",
    FontWeight->"Bold"],
  " function require the equation to be specified externally with an \
explicitly numeric argument."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(xmatch = 5;\)\), "\n", 
    \(\(bc[
          e_?NumericQ] := \[Psi][
            xmatch] /. \(NDSolve[{\(-\(\[Psi]\^\[DoublePrime]\)[x]\) + 
                    pot[x] \[Psi][x] \[Equal] 
                  e\ \[Psi][x], \[Psi][\(-xmatch\)] \[Equal] 
                  0, \(\[Psi]\^\[Prime]\)[\(-xmatch\)] \[Equal] 
                  10\^\(-4\)}, \[Psi], {x, \(-xmatch\), 
                xmatch}]\)\[LeftDoubleBracket]1\[RightDoubleBracket];\)\), "\
\[IndentingNewLine]", 
    \(evalue = 
      e /. FindRoot[bc[e], {e, 4.43, 4.431}, AccuracyGoal \[Rule] 10, 
          WorkingPrecision \[Rule] 20]\)}], "Input"],

Cell[BoxData[
    \(4.43048102092942212621889792809089758616`20. \)], "Output"]
}, Closed]],

Cell["\<\
Despite the 10 digit accuracy we requested for the boundary condition, the \
solution inevitably diverges if carried too far, either because of the \
remaining numerical error in the eigenvalue or the accumulated integration \
error in the solution to the differential equation.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(xmin = \(-5\); xmax = 7;\), "\n", 
    \(\(\[Psi][
          x_] = \[Psi][
            x] /. \(NDSolve[{\(-\(\[Psi]\^\[DoublePrime]\)[x]\) + 
                    pot[x] \[Psi][x] \[Equal] 
                  evalue\ \[Psi][x], \[Psi][\(-xmatch\)] \[Equal] 
                  0, \(\[Psi]\^\[Prime]\)[\(-xmatch\)] \[Equal] 
                  10\^\(-4\)}, \[Psi], {x, xmin, 
                xmax}]\)\[LeftDoubleBracket]1\[RightDoubleBracket];\)\), "\n\
", 
    \(\(Plot[\[Psi][x], {x, xmin, xmax}];\)\)}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.420635 0.0793651 0.434797 1.50045 [
[.10317 .4223 -9.15625 -15.6875 ]
[.10317 .4223 9.15625 0 ]
[.2619 .4223 -9.15625 -15.6875 ]
[.2619 .4223 9.15625 0 ]
[.57937 .4223 -5 -15.6875 ]
[.57937 .4223 5 0 ]
[.7381 .4223 -5 -15.6875 ]
[.7381 .4223 5 0 ]
[.89683 .4223 -5 -15.6875 ]
[.89683 .4223 5 0 ]
[.40813 .13471 -27.3125 -7.84375 ]
[.40813 .13471 0 7.84375 ]
[.40813 .28475 -27.3125 -7.84375 ]
[.40813 .28475 0 7.84375 ]
[.40813 .58484 -19 -7.84375 ]
[.40813 .58484 0 7.84375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.10317 .4348 m
.10317 .44105 L
s
gsave
.10317 .4223 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.2619 .4348 m
.2619 .44105 L
s
gsave
.2619 .4223 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.57937 .4348 m
.57937 .44105 L
s
gsave
.57937 .4223 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.7381 .4348 m
.7381 .44105 L
s
gsave
.7381 .4223 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.89683 .4348 m
.89683 .44105 L
s
gsave
.89683 .4223 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.125 Mabswid
.14286 .4348 m
.14286 .43855 L
s
.18254 .4348 m
.18254 .43855 L
s
.22222 .4348 m
.22222 .43855 L
s
.30159 .4348 m
.30159 .43855 L
s
.34127 .4348 m
.34127 .43855 L
s
.38095 .4348 m
.38095 .43855 L
s
.46032 .4348 m
.46032 .43855 L
s
.5 .4348 m
.5 .43855 L
s
.53968 .4348 m
.53968 .43855 L
s
.61905 .4348 m
.61905 .43855 L
s
.65873 .4348 m
.65873 .43855 L
s
.69841 .4348 m
.69841 .43855 L
s
.77778 .4348 m
.77778 .43855 L
s
.81746 .4348 m
.81746 .43855 L
s
.85714 .4348 m
.85714 .43855 L
s
.06349 .4348 m
.06349 .43855 L
s
.02381 .4348 m
.02381 .43855 L
s
.93651 .4348 m
.93651 .43855 L
s
.97619 .4348 m
.97619 .43855 L
s
.25 Mabswid
0 .4348 m
1 .4348 L
s
.42063 .13471 m
.42688 .13471 L
s
gsave
.40813 .13471 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
.42063 .28475 m
.42688 .28475 L
s
gsave
.40813 .28475 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.1) show
1.000 setlinewidth
grestore
.42063 .58484 m
.42688 .58484 L
s
gsave
.40813 .58484 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.1) show
1.000 setlinewidth
grestore
.125 Mabswid
.42063 .16472 m
.42438 .16472 L
s
.42063 .19472 m
.42438 .19472 L
s
.42063 .22473 m
.42438 .22473 L
s
.42063 .25474 m
.42438 .25474 L
s
.42063 .31476 m
.42438 .31476 L
s
.42063 .34477 m
.42438 .34477 L
s
.42063 .37478 m
.42438 .37478 L
s
.42063 .40479 m
.42438 .40479 L
s
.42063 .46481 m
.42438 .46481 L
s
.42063 .49481 m
.42438 .49481 L
s
.42063 .52482 m
.42438 .52482 L
s
.42063 .55483 m
.42438 .55483 L
s
.42063 .1047 m
.42438 .1047 L
s
.42063 .07469 m
.42438 .07469 L
s
.42063 .04468 m
.42438 .04468 L
s
.42063 .01467 m
.42438 .01467 L
s
.42063 .61485 m
.42438 .61485 L
s
.25 Mabswid
.42063 0 m
.42063 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .4348 m
.03279 .43481 L
.04262 .43484 L
.04758 .43486 L
.05288 .43488 L
.05789 .4349 L
.06244 .43494 L
.06785 .43498 L
.07283 .43503 L
.0783 .43511 L
.0841 .43521 L
.09385 .43546 L
.09939 .43565 L
.10458 .43587 L
.10954 .43614 L
.11496 .43649 L
.11974 .43687 L
.12483 .43736 L
.13443 .43856 L
.14335 .44011 L
.15364 .44254 L
.16476 .44619 L
.17527 .45086 L
.18493 .45643 L
.20376 .47144 L
.21444 .48269 L
.22442 .49501 L
.26333 .55439 L
.28314 .58286 L
.28857 .5891 L
.29433 .59466 L
.29926 .59841 L
.3019 .60001 L
.30469 .60137 L
.30715 .60228 L
.30855 .60267 L
.30985 .60296 L
.31105 .60315 L
.31214 .60326 L
.31339 .60332 L
.31458 .60331 L
.31587 .60321 L
.31653 .60314 L
.31724 .60303 L
.31851 .60278 L
.3197 .60247 L
.32237 .60153 L
.32524 .60013 L
.33041 .59664 L
.33532 .59221 L
Mistroke
.34454 .58119 L
.36438 .54829 L
.38278 .51177 L
.42196 .43213 L
.4636 .34742 L
.48451 .30751 L
.49367 .29276 L
.49852 .28601 L
.50372 .27974 L
.5088 .27468 L
.51344 .27105 L
.51567 .26966 L
.51809 .26842 L
.52038 .2675 L
.5225 .26688 L
.52364 .26663 L
.52486 .26644 L
.52602 .26633 L
.52707 .26628 L
.52825 .26628 L
.52953 .26637 L
.53018 .26644 L
.53087 .26654 L
.53212 .26678 L
.53344 .26711 L
.53487 .26756 L
.53747 .26862 L
.54232 .27139 L
.54723 .27519 L
.55181 .27956 L
.56223 .292 L
.58329 .32374 L
.6019 .35325 L
.62217 .38136 L
.63318 .39384 L
.64338 .40347 L
.65375 .41144 L
.6635 .41741 L
.67246 .42176 L
.68214 .4254 L
.69288 .42839 L
.70273 .43038 L
.71252 .43179 L
.72163 .43273 L
.72687 .43314 L
.73163 .43345 L
.73682 .43373 L
.74232 .43396 L
.74706 .43413 L
.75229 .43427 L
Mistroke
.76148 .43446 L
.76666 .43453 L
.77222 .4346 L
.77747 .43464 L
.78227 .43468 L
.78704 .4347 L
.79213 .43472 L
.79692 .43474 L
.80127 .43475 L
.80377 .43475 L
.8064 .43476 L
.80929 .43476 L
.81057 .43476 L
.81195 .43476 L
.81324 .43476 L
.81443 .43476 L
.81562 .43476 L
.81674 .43477 L
.81805 .43477 L
.81926 .43477 L
.81989 .43477 L
.82059 .43477 L
.82132 .43476 L
.822 .43476 L
.82318 .43476 L
.82444 .43476 L
.8267 .43476 L
.82939 .43476 L
.83184 .43476 L
.8344 .43475 L
.83717 .43474 L
.8422 .43473 L
.84688 .43471 L
.85183 .43468 L
.85606 .43465 L
.86071 .4346 L
.86593 .43452 L
.87088 .43442 L
.87533 .43429 L
.88017 .43409 L
.88293 .43395 L
.88542 .4338 L
.88811 .4336 L
.89102 .43333 L
.89372 .43304 L
.8962 .43271 L
.90107 .43186 L
.90579 .43069 L
.90843 .42984 L
.91084 .42891 L
Mistroke
.91552 .42653 L
.91994 .42337 L
.92245 .42105 L
.9251 .41806 L
.92986 .4109 L
.9325 .40562 L
.93498 .39959 L
.93791 .39077 L
.94063 .38054 L
.94518 .35761 L
.94758 .34168 L
.95014 .32092 L
.95259 .29671 L
.95489 .26904 L
.95698 .23896 L
.9592 .20103 L
.96345 .10551 L
Mfstroke
.96345 .10551 m
.96679 0 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003o]@3oool0
0`000000oooo0?ooo`3X0?ooo`030000003oool0oooo00d0oooo002e0?ooo`030000003oool0oooo
0>P0oooo00<000000?ooo`3oool03@3oool00;D0oooo00<000000?ooo`3oool0j03oool00`000000
oooo0?ooo`0=0?ooo`00]@3oool00`000000oooo0?ooo`3X0?ooo`030000003oool0oooo00d0oooo
002e0?ooo`030000003oool0oooo0>P0oooo00<000000?ooo`3oool03@3oool00;D0oooo00<00000
0?ooo`3oool0j03oool00`000000oooo0?ooo`0=0?ooo`00]@3oool200000>T0oooo00<000000?oo
o`3oool03@3oool00;D0oooo00<000000?ooo`3oool0j03oool00`000000oooo0?ooo`0=0?ooo`00
]@3oool00`000000oooo0?ooo`3X0?ooo`030000003oool0oooo00d0oooo002e0?ooo`030000003o
ool0oooo0>P0oooo00<000000?ooo`3oool03@3oool00;D0oooo00<000000?ooo`3oool0j03oool0
0`000000oooo0?ooo`0=0?ooo`00]@3oool00`000000oooo0?ooo`3X0?ooo`030000003oool0oooo
00d0oooo002e0?ooo`030000003oool0oooo0>L0oooo00<000000?ooo`3oool03P3oool00;D0oooo
00<000000?ooo`3oool0i`3oool00`000000oooo0?ooo`0>0?ooo`00]@3oool00`000000oooo0?oo
o`3W0?ooo`030000003oool0oooo00h0oooo002e0?ooo`030000003oool0oooo0>L0oooo00<00000
0?ooo`3oool03P3oool00;D0oooo00<000000?ooo`3oool0i`3oool00`000000oooo0?ooo`0>0?oo
o`00]@3oool00`000000oooo0?ooo`3W0?ooo`030000003oool0oooo00h0oooo002e0?ooo`030000
003oool0oooo0>L0oooo00<000000?ooo`3oool03P3oool00;D0oooo0P00003X0?ooo`030000003o
ool0oooo00h0oooo002e0?ooo`030000003oool0oooo0>L0oooo00<000000?ooo`3oool03P3oool0
0;D0oooo00<000000?ooo`3oool0i`3oool00`000000oooo0?ooo`0>0?ooo`00]@3oool00`000000
oooo0?ooo`3W0?ooo`030000003oool0oooo00h0oooo002e0?ooo`030000003oool0oooo0>L0oooo
00<000000?ooo`3oool03P3oool00;D0oooo00<000000?ooo`3oool0i`3oool00`000000oooo0?oo
o`0>0?ooo`00]@3oool00`000000oooo0?ooo`3W0?ooo`030000003oool0oooo00h0oooo002e0?oo
o`030000003oool0oooo0>L0oooo00<000000?ooo`3oool03P3oool00;D0oooo00<000000?ooo`3o
ool0i`3oool00`000000oooo0?ooo`0>0?ooo`00]@3oool00`000000oooo0?ooo`3W0?ooo`030000
003oool0oooo00h0oooo002e0?ooo`030000003oool0oooo0>L0oooo00<000000?ooo`3oool03P3o
ool00;D0oooo00<000000?ooo`3oool0i`3oool00`000000oooo0?ooo`0>0?ooo`00]@3oool00`00
0000oooo0?ooo`3W0?ooo`030000003oool0oooo00h0oooo002e0?ooo`800000j03oool00`000000
oooo0?ooo`0>0?ooo`00]@3oool00`000000oooo0?ooo`3W0?ooo`030000003oool0oooo00h0oooo
002e0?ooo`030000003oool0oooo0>H0oooo00<000000?ooo`3oool03`3oool00;D0oooo00<00000
0?ooo`3oool0iP3oool00`000000oooo0?ooo`0?0?ooo`00]@3oool00`000000oooo0?ooo`3V0?oo
o`030000003oool0oooo00l0oooo002e0?ooo`030000003oool0oooo0>H0oooo00<000000?ooo`3o
ool03`3oool00;D0oooo00<000000?ooo`3oool0iP3oool00`000000oooo0?ooo`0?0?ooo`00]@3o
ool00`000000oooo0?ooo`3V0?ooo`030000003oool0oooo00l0oooo002e0?ooo`030000003oool0
oooo0>H0oooo00<000000?ooo`3oool03`3oool00;D0oooo00<000000?ooo`3oool0iP3oool00`00
0000oooo0?ooo`0?0?ooo`00]@3oool00`000000oooo0?ooo`3V0?ooo`030000003oool0oooo00l0
oooo002e0?ooo`030000003oool0oooo0>H0oooo00<000000?ooo`3oool03`3oool00;D0oooo00<0
00000?ooo`3oool0iP3oool00`000000oooo0?ooo`0?0?ooo`00]@3oool200000>L0oooo00<00000
0?ooo`3oool03`3oool00;D0oooo00<000000?ooo`3oool0iP3oool00`000000oooo0?ooo`0?0?oo
o`00]@3oool00`000000oooo0?ooo`3V0?ooo`030000003oool0oooo00l0oooo002e0?ooo`030000
003oool0oooo0>H0oooo00<000000?ooo`3oool03`3oool00;D0oooo00<000000?ooo`3oool0iP3o
ool00`000000oooo0?ooo`0?0?ooo`00]@3oool00`000000oooo0?ooo`3V0?ooo`030000003oool0
oooo00l0oooo002e0?ooo`030000003oool0oooo0>H0oooo00<000000?ooo`3oool03`3oool00;D0
oooo00<000000?ooo`3oool0iP3oool00`000000oooo0?ooo`0?0?ooo`00V03oool01@3ogi`0B000
0000000007@0_ooo00<0oooo00D0okmd0000001d_ol0oooo0;md00060000000304RLg`3oool0oooo
00L0oooo00<000000?ooo`3oool0iP3oool00`000000oooo0?ooo`0?0?ooo`00U`3oool01`3ookl0
M00007Boo`3oool0okmd0000M02oool00P3oool00`3o_g@0000007Boo`020?ooo`030=nLB0000000
000000<0000000<004RL0=ooo`3oool01`3oool00`000000oooo0?ooo`3V0?ooo`030000003oool0
oooo00l0oooo002G0?ooo`030;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool01`3oool00`3O
W4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo00L0oooo00<000000?ooo`3oool0iP3oool00`00
0000oooo0?ooo`0?0?ooo`00UP3oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00P0
oooo00<0gia807Boo`3oool02`3oool00`000000oooo0?ooo`3V0?ooo`030000003oool0oooo00l0
oooo002:0?ooo`030?oog`2LB000000000L0000000D0B9cO0?ooo`3ogi`0B00007Boo`030?ooo`03
0?noM00004P0W=oo00T0oooo00<0_gA809cOo`3oool02P3oool00`000000oooo0?ooo`3V0?ooo`03
0000003oool0oooo00l0oooo002F0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0
2@3oool00`3ooml0W4Qd0;ooo`0:0?ooo`@00000i@3oool00`000000oooo0?ooo`0?0?ooo`00UP3o
ool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00X0oooo00<0okmd001d_`3oool02@3o
ool00`000000oooo0?ooo`3V0?ooo`030000003oool0oooo00l0oooo002F0?ooo`030?oOW0180000
M;oo00<0oooo00<0okmd0000B02Lgol02P3oool00`3ookl0M01809cOo`090?ooo`030000003oool0
oooo0>H0oooo00<000000?ooo`3oool03`3oool009H0oooo00<0oono07@00018W=l00`3oool00`3O
W4P0001d0;ooo`0:0?ooo`030?oo_`1d0000B9cO00T0oooo00<000000?ooo`3oool0iP3oool00`00
0000oooo0?ooo`0?0?ooo`00U`3oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00D0
oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0B00004RLg`090?ooo`030000003oool0oooo0>H0
oooo00<000000?ooo`3oool03`3oool009L0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo
00H0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`T0oooo00<000000?ooo`3oool0
iP3oool00`000000oooo0?ooo`0?0?ooo`00V03oool01@3ookl0M0000000000007@0_ooo00P0oooo
00H0oooO09a80000000000000000M02oool:0?ooo`030000003oool0oooo0>H0oooo00<000000?oo
o`3oool03`3oool00;D0oooo00<000000?ooo`3oool0iP3oool00`000000oooo0?ooo`0?0?ooo`00
]@3oool00`000000oooo0?ooo`3U0?ooo`030000003oool0oooo0100oooo002e0?ooo`030000003o
ool0oooo0>D0oooo00<000000?ooo`3oool0403oool00;D0oooo00<000000?ooo`3oool0i@3oool0
0`000000oooo0?ooo`0@0?ooo`00]@3oool00`000000oooo0?ooo`3U0?ooo`030000003oool0oooo
0100oooo002e0?ooo`800000iP3oool00`000000oooo0?ooo`0@0?ooo`00]@3oool00`000000oooo
0?ooo`3U0?ooo`030000003oool0oooo0100oooo002e0?ooo`030000003oool0oooo0>D0oooo00<0
00000?ooo`3oool0403oool00;D0oooo00<000000?ooo`3oool0i@3oool00`000000oooo0?ooo`0@
0?ooo`00]@3oool00`000000oooo0?ooo`3U0?ooo`030000003oool0oooo0100oooo002e0?ooo`03
0000003oool0oooo0>D0oooo00<000000?ooo`3oool0403oool00;D0oooo00<000000?ooo`3oool0
i@3oool00`000000oooo0?ooo`0@0?ooo`00]@3oool00`000000oooo0?ooo`3U0?ooo`030000003o
ool0oooo0100oooo002e0?ooo`030000003oool0oooo0>D0oooo00<000000?ooo`3oool0403oool0
0;D0oooo00<000000?ooo`3oool0i@3oool00`000000oooo0?ooo`0@0?ooo`00]@3oool00`000000
oooo0?ooo`3U0?ooo`030000003oool0oooo0100oooo002e0?ooo`030000003oool0oooo0>D0oooo
00<000000?ooo`3oool0403oool00;D0oooo00<000000?ooo`3oool0i@3oool00`000000oooo0?oo
o`0@0?ooo`00]@3oool200000>H0oooo00<000000?ooo`3oool0403oool00;D0oooo00<000000?oo
o`3oool0i@3oool00`000000oooo0?ooo`0@0?ooo`00]@3oool00`000000oooo0?ooo`3U0?ooo`03
0000003oool0oooo0100oooo002e0?ooo`030000003oool0oooo0>D0oooo00<000000?ooo`3oool0
403oool00;D0oooo00<000000?ooo`3oool0i@3oool00`000000oooo0?ooo`0@0?ooo`00]@3oool0
0`000000oooo0?ooo`3U0?ooo`030000003oool0oooo0100oooo002e0?ooo`030000003oool0oooo
0>D0oooo00<000000?ooo`3oool0403oool00;D0oooo00<000000?ooo`3oool0i@3oool00`000000
oooo0?ooo`0@0?ooo`00]@3oool00`000000oooo0?ooo`3U0?ooo`030000003oool0oooo0100oooo
002e0?ooo`030000003oool0oooo0>D0oooo00<000000?ooo`3oool0403oool00;D0oooo00<00000
0?ooo`3oool0i@3oool00`000000oooo0?ooo`0@0?ooo`00]@3oool00`000000oooo0?ooo`3T0?oo
o`030000003oool0oooo0140oooo002e0?ooo`030000003oool0oooo0>@0oooo00<000000?ooo`3o
ool04@3oool00;D0oooo0P00003U0?ooo`030000003oool0oooo0140oooo002e0?ooo`030000003o
ool0oooo0>@0oooo00<000000?ooo`3oool04@3oool00;D0oooo00<000000?ooo`3oool0i03oool0
0`000000oooo0?ooo`0A0?ooo`00]@3oool00`000000oooo0?ooo`3T0?ooo`030000003oool0oooo
0140oooo002e0?ooo`030000003oool0oooo0>@0oooo00<000000?ooo`3oool04@3oool00;D0oooo
00<000000?ooo`3oool0i03oool00`000000oooo0?ooo`0A0?ooo`00]@3oool00`000000oooo0?oo
o`3T0?ooo`030000003oool0oooo0140oooo002e0?ooo`030000003oool0oooo0>@0oooo00<00000
0?ooo`3oool04@3oool00;D0oooo00<000000?ooo`3oool0i03oool00`000000oooo0?ooo`0A0?oo
o`00]@3oool00`000000oooo0?ooo`3T0?ooo`030000003oool0oooo0140oooo002e0?ooo`030000
003oool0oooo0>@0oooo00<000000?ooo`3oool04@3oool00;D0oooo00<000000?ooo`3oool0i03o
ool00`000000oooo0?ooo`0A0?ooo`00]@3oool00`000000oooo0?ooo`3T0?ooo`030000003oool0
oooo0140oooo002e0?ooo`800000i03oool00`000000oooo0?ooo`0B0?ooo`00]@3oool00`000000
oooo0?ooo`3S0?ooo`030000003oool0oooo0180oooo002e0?ooo`030000003oool0oooo0><0oooo
00<000000?ooo`3oool04P3oool00;D0oooo00<000000?ooo`3oool0h`3oool00`000000oooo0?oo
o`0B0?ooo`00]@3oool00`000000oooo0?ooo`3S0?ooo`030000003oool0oooo0180oooo002e0?oo
o`030000003oool0oooo02T0oooo1@00002e0?ooo`030000003oool0oooo0180oooo002e0?ooo`03
0000003oool0oooo02L0oooo0P0000040?ooo`<00000/`3oool00`000000oooo0?ooo`0B0?ooo`00
]@3oool00`000000oooo0?ooo`0U0?ooo`8000002@3oool200000;40oooo00<000000?ooo`3oool0
4P3oool009P0oooo00D0omnL04P000000000001d0;ooo`030?ooo`030?noM0000000M;oo00<0oooo
00H0_g@000000000000000000018W03Oool90?ooo`030000003oool0oooo02@0oooo00<000000?oo
o`3oool02`3oool00`000000oooo0?ooo`2^0?ooo`030000003oool0oooo0180oooo002G0?ooo`07
0?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`020?ooo`030?noM0000000M;oo00@0oooo00<0omnL
04P0001d_ol02`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool0[@3oool00`000000oooo0?ooo`0B0?ooo`00U`3oool00`2oM00004RL0=ooo`020?oo
o`030;md0000M;l0oooo00P0oooo00<0omnL04P0001d_ol02`3oool00`000000oooo0?ooo`0R0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool0[03oool00`000000oooo0?ooo`0B0?oo
o`00UP3oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00P0oooo00<0omnL04P0001d
_ol02`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool0Z`3oool00`000000oooo0?ooo`0B0?ooo`00RP3oool00`3ooml0W4P0000000070000000504RL
g`3oool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`080?ooo`030?oOW0180000M;oo00/0
oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0:T0
oooo00<000000?ooo`3oool04`3oool009H0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@00018
09cOo`080?ooo`030?oOW0180000M;oo00/0oooo1000000N0?ooo`030000003oool0oooo01D0oooo
00<000000?ooo`3oool0Z03oool00`000000oooo0?ooo`0C0?ooo`00UP3oool00`3ogi`0B00007Bo
o`030?ooo`030?noM00004P0W=oo00P0oooo00<0omnL04P0001d_ol02`3oool00`000000oooo0?oo
o`0N0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0Y`3oool00`000000oooo0?oo
o`0C0?ooo`00UP3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00P0oooo00<0omnL
04P0001d_ol02`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool0YP3oool00`000000oooo0?ooo`0C0?ooo`00UP3oool00`3ookl0M00004RLg`030?oo
o`030=nLB00007@0_ooo00P0oooo00<0omnL04P0001d_ol02`3oool00`000000oooo0?ooo`0M0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool0Y@3oool00`000000oooo0?ooo`0C0?oo
o`00U`3oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00P0oooo00<0omnL04P0001d
_ol02`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3o
ool0Y03oool00`000000oooo0?ooo`0C0?ooo`00U`3oool01`3ookl0M00007Boo`3oool0okmd0000
M02oool01`3oool01@3ooml0W4P0000000000000M;oo00/0oooo00<000000?ooo`3oool06`3oool0
0`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0:@0oooo00<000000?ooo`3oool04`3oool0
09P0oooo00D0oono07@000000000001d0;ooo`0:0?ooo`030?noM0000000M;oo00/0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0:<0oooo00<00000
0?ooo`3oool04`3oool00;D0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0P0?oo
o`030000003oool0oooo0:80oooo00<000000?ooo`3oool04`3oool00;D0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0:80oooo00<000000?ooo`3o
ool04`3oool00;D0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0R0?ooo`030000
003oool0oooo0:40oooo00<000000?ooo`3oool04`3oool00;D0oooo00<000000?ooo`3oool0603o
ool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0:40oooo00<000000?ooo`3oool04`3o
ool00;D0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0
oooo09l0oooo00<000000?ooo`3oool0503oool00;D0oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`0V0?ooo`030000003oool0oooo09h0oooo00<000000?ooo`3oool0503oool00;D0
oooo0P00000G0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0WP3oool00`000000
oooo0?ooo`0D0?ooo`00]@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo02P0oooo
00<000000?ooo`3oool0W@3oool00`000000oooo0?ooo`0D0?ooo`00]@3oool00`000000oooo0?oo
o`0E0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0W03oool00`000000oooo0?oo
o`0D0?ooo`00]@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo02X0oooo00<00000
0?ooo`3oool0W03oool00`000000oooo0?ooo`0D0?ooo`00]@3oool00`000000oooo0?ooo`0D0?oo
o`030000003oool0oooo02`0oooo00<000000?ooo`3oool0V`3oool00`000000oooo0?ooo`0D0?oo
o`00]@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3o
ool0VP3oool00`000000oooo0?ooo`0D0?ooo`00]@3oool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo02h0oooo00<000000?ooo`3oool0VP3oool00`000000oooo0?ooo`0D0?ooo`00]@3o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0V@3o
ool00`000000oooo0?ooo`0D0?ooo`00]@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo0300oooo00<000000?ooo`3oool0V03oool00`000000oooo0?ooo`0E0?ooo`00]@3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0U`3oool00`00
0000oooo0?ooo`0E0?ooo`00]@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo03<0
oooo00<000000?ooo`3oool0UP3oool00`000000oooo0?ooo`0E0?ooo`00]@3oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0UP3oool00`000000oooo
0?ooo`0E0?ooo`00]@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo03D0oooo00<0
00000?ooo`3oool0U@3oool00`000000oooo0?ooo`0E0?ooo`00]@3oool200000140oooo00<00000
0?ooo`3oool0=P3oool00`000000oooo0?ooo`2D0?ooo`030000003oool0oooo01D0oooo002e0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`2D0?oo
o`030000003oool0oooo01D0oooo002e0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3o
ool0>03oool00`000000oooo0?ooo`2B0?ooo`030000003oool0oooo01H0oooo002e0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`2B0?ooo`030000
003oool0oooo01H0oooo002e0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0>P3o
ool00`000000oooo0?ooo`2A0?ooo`030000003oool0oooo01H0oooo002e0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`2@0?ooo`030000003oool0
oooo01H0oooo002e0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0?03oool00`00
0000oooo0?ooo`2@0?ooo`030000003oool0oooo01H0oooo002e0?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`2?0?ooo`030000003oool0oooo01H0
oooo002e0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0?`3oool00`000000oooo
0?ooo`2=0?ooo`030000003oool0oooo01L0oooo002e0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool0@@3oool00`000000oooo0?ooo`2<0?ooo`030000003oool0oooo01L0oooo002e
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`2<
0?ooo`030000003oool0oooo01L0oooo002e0?ooo`030000003oool0oooo00X0oooo00<000000?oo
o`3oool0@`3oool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo01L0oooo002e0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`2:0?ooo`03
0000003oool0oooo01L0oooo002e0?ooo`8000002P3oool00`000000oooo0?ooo`160?ooo`030000
003oool0oooo08P0oooo00<000000?ooo`3oool0603oool00;D0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`160?ooo`030000003oool0oooo08P0oooo00<000000?ooo`3oool0603o
ool00;D0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`180?ooo`030000003oool0
oooo08L0oooo00<000000?ooo`3oool0603oool00;D0oooo00<000000?ooo`3oool0203oool00`00
0000oooo0?ooo`190?ooo`030000003oool0oooo08H0oooo00<000000?ooo`3oool0603oool00380
oooo00@0oooO09a80000B9`0gooo?03oool00`2oM0000000000000040000000304RLg`3oool0oooo
03T0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0c0?ooo`030;md000000000000
00@0000000<0B9cO0?ooo`3oool03P3oool00`000000oooo0?ooo`0]0?ooo`040?oog`2LB00004RL
0=oooch0oooo00D0omnL04P000000000000007Boo`0@0?ooo`030000003oool0oooo01T0oooo000b
0?ooo`040?oog`2LB00004RL0=ooocd0oooo00<0gia80000000000000`0000000`00B9`0gooo0?oo
o`0i0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0=03oool00`3OW4P000000000
0003000000030018W03Oool0oooo00l0oooo00<000000?ooo`3oool0;03oool0103ooml0W4P00018
W03Ooolm0?ooo`070?oOW0180000B9cO0?ooo`3ookl0M00007Boo`0?0?ooo`030000003oool0oooo
01T0oooo000b0?ooo`040?oog`2LB00004RL0=oooch0oooo00<0gia804RLg`3oool00P3oool00`3O
W7@0W=oo0?ooo`0i0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0=P3oool00`3O
W4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo00l0oooo00<000000?ooo`3oool0;03oool0103o
oml0W4P00018W03Oooll0?ooo`030?oog`2LB00007Bo00<0oooo00<0okmd0018W03Oool03@3oool0
0`000000oooo0?ooo`0J0?ooo`00;P3oool00`2oM0000000000000050000000307Boo`3oool0oooo
03`0oooo00<0gia807Boo`3oool0?@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
03L0oooo00<0gia807Boo`3oool0503oool00`000000oooo0?ooo`0W0?ooo`030;md000000000000
00D0000000<0M;oo0?ooo`3oool0>@3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo
00d0oooo00<000000?ooo`3oool06P3oool00280oooo00<0oooO09a8000000001`0000001018W=l0
oooo0?oOW01dW=l20?ooo`040?oog`2LB00004RL0=oooc00oooo00<0oooO09a8000000001`000000
0`18W=l0oooo0?ooo`030?ooo`030;mdB02Lgol0oooo03`0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`0i0?ooo`030;mdB02Lgol0oooo01@0oooo00<000000?ooo`3oool09P3oool0
203ogi`0M9cO0?ooo`3oool0oooO09a80000B9`0gooo?03oool00`3ogi`0B00007Boo`030?ooo`03
0?noM00004P0W=oo00d0oooo00<000000?ooo`3oool06P3oool002l0oooo00L0gia807Boo`3oool0
oooO09a80000B9`0gooo0400oooo00<0oooO09a8M02oool0?03oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo03T0oooo00<0oooO09a8M02oool05@3oool00`000000oooo0?ooo`0V0?oo
o`070=nLB01d_ol0oooo0?oog`2LB00004RL0=ooo`0l0?ooo`030?oOW0180000M;oo00<0oooo00<0
_g@00000B02Lgol0303oool00`000000oooo0?ooo`0K0?ooo`00;`3oool01`3ooml0W4Qd0;ooo`3o
oml0W4P00018W03Oool0@@3oool00`3o_g@007Bo0?ooo`0k0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0>`3oool00`3o_g@007Bo0?ooo`0E0?ooo`8000009P3oool01`3ooml0W4Qd
0;ooo`3ooml0W4P00018W03Oool0?03oool0203ookl0M00000000018W=l0oooo0?noM000000007Bo
3@3oool00`000000oooo0?ooo`0K0?ooo`00<03oool01P3ogi`0B7Bo0?oog`2LB00004RL0=oood40
oooo00<0oono07@0B02Lgol0>`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo03/0
oooo00<0oono07@0B02Lgol05`3oool00`000000oooo0?ooo`0T0?ooo`060?oOW018M;l0oooO09a8
0000B9`0gooo?@3oool00`2oM00004RL0;nLB002000000030018W03Oool0oooo00`0oooo00<00000
0?ooo`3oool06`3oool00340oooo00D0gia807Bog`2LB00004RL0=ooo`110?ooo`030?oo_`1d0000
B9cO03/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0l0?ooo`030?oo_`1d0000
B9cO01P0oooo00<000000?ooo`3oool0903oool01@3OW4P0M;oO09a80000B9`0gooo03d0oooo00<0
omnL04P0001d_ol0403oool00`000000oooo0?ooo`0L0?ooo`00<@3oool01@3ooml0W4Q804Q80000
B9`0gooo03`0oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0B00004RLg`0k0?ooo`800000103o
ool00`000000oooo0?ooo`0g0?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l06@3o
ool00`000000oooo0?ooo`0S0?ooo`050?oog`2LB4P0B4P00018W03Oool0?P3oool00`3OW4P00018
09cOo`0?0?ooo`030000003oool0oooo01`0oooo000b0?ooo`040?oOW018000004RL0=oooc`0oooo
00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;oooc/0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`0h0?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02ooolJ0?ooo`03
0000003oool0oooo02<0oooo00@0omnL04P00000B9`0gooo?`3oool00`3o_g@00000001d_`0=0?oo
o`030000003oool0oooo01d0oooo000c0?ooo`030=nLB000B9`0gooo03d0oooo00H0oooO09a80000
000000000000M02oooll0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0>@3oool0
1P3ooml0W4P0000000000000001d0;oooa`0oooo00<000000?ooo`3oool08`3oool00`3OW4P004RL
0=ooo`100?ooo`050?oog`2LB00000000000M02oool02P3oool00`000000oooo0?ooo`0M0?ooo`00
]@3oool01@000000oooo0?ooo`3oool0000005l0oooo00<000000?ooo`3oool0L`3oool00`000000
oooo0?ooo`0N0?ooo`00]@3oool01@000000oooo0?ooo`3oool000000600oooo0P00001c0?ooo`03
0000003oool0oooo01h0oooo002e0?ooo`040000003oool0oooo000006<0oooo0P00001`0?ooo`03
0000003oool0oooo01l0oooo002e0?ooo`040000003oool0oooo000006D0oooo0P00001]0?ooo`03
0000003oool0oooo0200oooo002e0?ooo`030000003oool0000006P0oooo0P00001[0?ooo`030000
003oool0oooo0200oooo002e0?ooo`030000003oool0000006X0oooo0P00001X0?ooo`030000003o
ool0oooo0240oooo002e0?ooo`800000K@3oool400000680oooo0P00000T0?ooo`00]@3oool20000
0740oooo1000001L0?ooo`8000009P3oool00;D0oooo00<000000?ooo`3oool0M03oool8000004l0
oooo1@00000X0?ooo`00]@3oool00`000000oooo0?ooo`1l0?oooa800000<P3oool;000002d0oooo
003o00000;0000000@3oool000X0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo00<0oooo2P0000000`3oool000000?ooo`0?0?ooo`030000003oool0
oooo00l0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0=0?ooo`800000403oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00T0oooo000/0?ooo`030000003oool0oooo
00d0oooo1000000a0?ooo`030000003oool0oooo0400oooo0P0000130?ooo`030000003oool0oooo
0440oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0[0?ooo`00@03oool4000006l0
oooo00<000000?ooo`000000nP3oool004@0oooo0P00001]0?ooo`030000003oool000000?X0oooo
00160?ooo`800000JP3oool010000000oooo0?ooo`00003j0?ooo`00B03oool2000006P0oooo00@0
00000?ooo`3oool00000nP3oool004X0oooo0P00001U0?ooo`050000003oool0oooo0?ooo`000000
nP3oool004`0oooo0P00001S0?ooo`050000003oool0oooo0?ooo`000000nP3oool004h0oooo0P00
001P0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0n03oool00500oooo0P00001N
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0n03oool00580oooo00<000000?oo
o`3oool0FP3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?P0oooo001C0?ooo`80
0000FP3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?P0oooo001E0?ooo`030000
003oool0oooo05H0oooo00<000000?ooo`3oool0103oool200000?T0oooo001F0?ooo`800000EP3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?P0oooo001H0?ooo`030000003oool0
oooo0580oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3h0?ooo`00F@3oool00`00
0000oooo0?ooo`1A0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0n03oool005X0
oooo00<000000?ooo`3oool0D03oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0?P0
oooo001J0?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`3h0?ooo`00F`3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool0n03oool005`0oooo00<000000?ooo`3oool0C03oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo0?P0oooo001M0?ooo`030000003oool0oooo04/0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`3h0?ooo`00G@3oool00`000000oooo0?ooo`1:0?ooo`03
0000003oool0oooo00P0oooo00<000000?ooo`3oool0n03oool005h0oooo00<000000?ooo`3oool0
B@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0?P0oooo001O0?ooo`030000003o
ool0oooo04L0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3h0?ooo`00G`3oool0
0`000000oooo0?ooo`170?ooo`030000003oool0oooo00T0oooo0P00003i0?ooo`00H03oool00`00
0000oooo0?ooo`150?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0n03oool00640
oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?P0
oooo001Q0?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`3h0?ooo`00HP3oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool0n03oool006<0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo0?P0oooo001S0?ooo`030000003oool0oooo0400oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`3h0?ooo`00I03oool00`000000oooo0?ooo`0n0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool0n03oool006D0oooo00<000000?ooo`3oool0
?@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0?P0oooo001U0?ooo`030000003o
ool0oooo03`0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`3h0?ooo`00IP3oool0
0`000000oooo0?ooo`0k0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0n03oool0
06L0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
0?P0oooo001W0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`3h0?ooo`00J03oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo0100oooo
0P00003i0?ooo`00J@3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`3oool0n03oool006X0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo0?P0oooo001Z0?ooo`030000003oool0oooo03@0oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`3h0?ooo`00J`3oool00`000000oooo0?ooo`0b0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool0n03oool006`0oooo00<000000?ooo`3oool0
<@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0?P0oooo001/0?ooo`030000003o
ool0oooo0300oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`3h0?ooo`00K@3oool0
0`000000oooo0?ooo`0_0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0n03oool0
06h0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
0?P0oooo001^0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`3h0?ooo`00K`3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo01D0oooo
00<000000?ooo`3oool0n03oool00700oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?oo
o`0F0?ooo`030000003oool0oooo0?P0oooo001`0?ooo`030000003oool0oooo02T0oooo00<00000
0?ooo`3oool05P3oool00`000000oooo0?ooo`3h0?ooo`00L@3oool00`000000oooo0?ooo`0W0?oo
o`030000003oool0oooo01L0oooo0P00003i0?ooo`00LP3oool00`000000oooo0?ooo`0V0?ooo`03
0000003oool0oooo01L0oooo00<000000?ooo`3oool0n03oool00780oooo00<000000?ooo`3oool0
9@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo0?P0oooo001c0?ooo`030000003o
ool0oooo02<0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`3h0?ooo`00M03oool0
0`000000oooo0?ooo`0R0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0n03oool0
07D0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
0?P0oooo001e0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`3h0?ooo`00MP3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01/0oooo
00<000000?ooo`3oool0n03oool007L0oooo00<000000?ooo`3oool0703oool01`000000oooo0?oO
W018000000000000M02oool00`3oool00`3o_g@0000007Boo`030?ooo`060;md0000000000000000
0000B9`0gooo2@3oool00`000000oooo0?ooo`3h0?ooo`00N03oool00`000000oooo0?ooo`0K0?oo
o`080000003ookl0M00007Boo`3oool0okmd0000M02oool20?ooo`030?noM0000000M;oo00@0oooo
00<0omnL04P0001d_ol02`3oool00`000000oooo0?ooo`3h0?ooo`00N03oool00`000000oooo0?oo
o`0J0?ooo`050000003oool0_g@00018W03Oool00P3oool00`2oM00007Bo0?ooo`080?ooo`030?oO
W0180000M;oo00/0oooo00<000000?ooo`3oool0n03oool007T0oooo00<000000?ooo`3oool06@3o
ool010000000oono07@00018W=l30?ooo`030=nLB00007@0_ooo00P0oooo00<0omnL04P0001d_ol0
2`3oool00`000000oooo0?ooo`3h0?ooo`00NP3oool00`000000oooo0?ooo`0G0?ooo`050000003o
ool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`080?ooo`030?oOW0180000M;oo00/0oooo
00<000000?ooo`3oool0n03oool007/0oooo00<000000?ooo`3oool05@3oool01P000000oooo0?oo
o`3ogi`0B00007Boo`<0oooo00<0okmd0000B02Lgol0203oool00`3ogi`0B00007Boo`0;0?ooo`@0
0000m`3oool007/0oooo00<000000?ooo`3oool0503oool01`000000oooo0?ooo`3oool0omnL04P0
001d_ol00`3oool00`3o_g@0001809cOo`080?ooo`030?oOW0180000M;oo00/0oooo00<000000?oo
o`3oool0n03oool007`0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`020?ooo`03
0?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0203oool00`3ogi`0B00007Boo`0;0?ooo`03
0000003oool0oooo0?P0oooo001m0?ooo`8000004@3oool00`000000oooo0?ooo`030?ooo`030?oo
_`1d0000B9cO00<0oooo00<0gia80000M02oool0203oool00`3ogi`0B00007Boo`0;0?ooo`030000
003oool0oooo0?P0oooo001o0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool01@3o
ool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00P0oooo00<0omnL04P0001d_ol02`3o
ool00`000000oooo0?ooo`3h0?ooo`00P03oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo00H0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00L0oooo00D0oooO09a800000000
000007Boo`0;0?ooo`030000003oool0oooo0?P0oooo00210?ooo`030000003oool0oooo00T0oooo
00<000000?ooo`3oool0203oool01@3ookl0M0000000000007@0_ooo00X0oooo00<0okmd0000001d
_ol02`3oool00`000000oooo0?ooo`3h0?ooo`00PP3oool2000000L0oooo0P00000X0?ooo`030000
003oool0oooo0?P0oooo00240?ooo`L00000:P3oool00`000000oooo0?ooo`3h0?ooo`00]@3oool0
0`000000oooo0?ooo`3h0?ooo`00]@3oool00`000000oooo0?ooo`3h0?ooo`00]@3oool00`000000
oooo0?ooo`3h0?ooo`00]@3oool00`000000oooo0?ooo`3h0?ooo`00]@3oool200000?T0oooo002e
0?ooo`030000003oool0oooo0?P0oooo0000\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {-5.31181, -0.28978, \
0.0439337, 0.00232384}}]
}, Closed]],

Cell[TextData[
" Clearly one must choose the integration range wisely \[LongDash] if the \
range is too narrow, inaccuracy in the initial conditions becomes important; \
if too wide, integration errors accumulate.  "], "Text"],

Cell["\<\
\tSuppose that the potential varies relatively slowly, such that local \
solutions to the Schr\[ODoubleDot]dinger equation can be approximated by\
\>", "Text"],

Cell[BoxData[
    \(\[Psi][x] \[TildeTilde] 
      a\ \[ExponentialE]\^\(\[ImaginaryI]k\ x\) + 
        b\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ k\ x\)\)], "DisplayFormula",\

  FontFamily->"Times New Roman"],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`k = \@\(2 \( m(e - V)\)\)\)]],
  " is the wave number and ",
  StyleBox["a",
    FontSlant->"Italic"],
  " and ",
  StyleBox["b",
    FontSlant->"Italic"],
  " are arbitrary constants.  In the ",
  StyleBox["classically allowed ",
    FontSlant->"Italic"],
  "region, where the energy is greater than the local value of the potential, \
the wave number is real and the convex solutions are oscillatory.  \
Conversely, in the ",
  StyleBox["classically forbidden",
    FontSlant->"Italic"],
  " region, where the energy is smaller than the potential, ",
  Cell[BoxData[
      \(TraditionalForm\`k = \[ImaginaryI]\ \[Kappa]\)]],
  " is imaginary and the concave solution is a mixture of exponentially \
decaying and exponentially growing components.  Eigenfunctions in the \
classically forbidden region are composed of a delicate balance of growing \
and decaying solutions whose coefficients vary as the local wave number \
changes.  In general, numerical integration algorithms encounter trouble when \
small but inevitable numerical errors upset this delicate balance such that \
the exponentially growing solution eventually overwhelms the desired \
eigenfunction.  The classically allowed and forbidden regions for a \
particular eigenenergy are shown below, along with the corresponding \
eigenfunction."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(tp = \(x /. 
              FindRoot[
                pot[x] \[Equal] 
                  evalue, {x, #}] &\) /@ {\(-2\), \(- .5\),  .5, 
            2};\)\), "\n", 
    \(regions = 
      Graphics[{RGBColor[0.5, 0.05, 0.03], 
          Rectangle[{\(-5\), 
              2}, {tp\[LeftDoubleBracket]1\[RightDoubleBracket], 8}], 
          Rectangle[{tp\[LeftDoubleBracket]4\[RightDoubleBracket], 2}, {5, 
              8}], Rectangle[{tp\[LeftDoubleBracket]2\[RightDoubleBracket], 
              2}, {tp\[LeftDoubleBracket]3\[RightDoubleBracket], 8}], 
          RGBColor[0, 0.26, 0.13], 
          Rectangle[{tp\[LeftDoubleBracket]1\[RightDoubleBracket], 
              2}, {tp\[LeftDoubleBracket]2\[RightDoubleBracket], 8}], 
          Rectangle[{tp\[LeftDoubleBracket]3\[RightDoubleBracket], 
              2}, {tp\[LeftDoubleBracket]4\[RightDoubleBracket], 8}]}]; 
    Plot[{pot[x], evalue, evalue + 10  \[Psi][x]}, {x, \(-5\), 5}, 
      PlotRange \[Rule] {All, {2, 8}}, 
      PlotStyle \[Rule] {GrayLevel[0], {GrayLevel[1], Dashing[{0.02, 0.04}]}, 
          RGBColor[0, 0, 1]}, Frame \[Rule] True, 
      FrameLabel \[Rule] {"\<x\>", "\<V[x], \[Psi][x]\>"}, 
      PlotLabel \[Rule] "\<a double-well potential\>", 
      Prolog \[Rule] 
        regions\[LeftDoubleBracket]1\[RightDoubleBracket]];\)}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0952381 -0.206011 0.103006 [
[.11905 -0.0125 -9.15625 -15.6875 ]
[.11905 -0.0125 9.15625 0 ]
[.30952 -0.0125 -9.15625 -15.6875 ]
[.30952 -0.0125 9.15625 0 ]
[.5 -0.0125 -5 -15.6875 ]
[.5 -0.0125 5 0 ]
[.69048 -0.0125 -5 -15.6875 ]
[.69048 -0.0125 5 0 ]
[.88095 -0.0125 -5 -15.6875 ]
[.88095 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -5.0625 -31.375 ]
[.5 -0.0125 5.0625 -15.6875 ]
[-0.0125 .10301 -10 -7.84375 ]
[-0.0125 .10301 0 7.84375 ]
[-0.0125 .20601 -10 -7.84375 ]
[-0.0125 .20601 0 7.84375 ]
[-0.0125 .30902 -10 -7.84375 ]
[-0.0125 .30902 0 7.84375 ]
[-0.0125 .41202 -10 -7.84375 ]
[-0.0125 .41202 0 7.84375 ]
[-0.0125 .51503 -10 -7.84375 ]
[-0.0125 .51503 0 7.84375 ]
[-0.0125 .61803 -10 -7.84375 ]
[-0.0125 .61803 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -25.6875 -27.0313 ]
[-0.01875 .30902 -10 27.0313 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -58.9375 0 ]
[.5 .63053 58.9375 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.11905 0 m
.11905 .00625 L
s
gsave
.11905 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.30952 0 m
.30952 .00625 L
s
gsave
.30952 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.5 0 m
.5 .00625 L
s
gsave
.5 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.69048 0 m
.69048 .00625 L
s
gsave
.69048 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.88095 0 m
.88095 .00625 L
s
gsave
.88095 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.125 Mabswid
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.07143 0 m
.07143 .00375 L
s
.02381 0 m
.02381 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -66.0625 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
69.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .10301 m
.00625 .10301 L
s
gsave
-0.0125 .10301 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
0 .20601 m
.00625 .20601 L
s
gsave
-0.0125 .20601 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
0 .30902 m
.00625 .30902 L
s
gsave
-0.0125 .30902 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
0 .41202 m
.00625 .41202 L
s
gsave
-0.0125 .41202 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
0 .51503 m
.00625 .51503 L
s
gsave
-0.0125 .51503 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7) show
1.000 setlinewidth
grestore
0 .61803 m
.00625 .61803 L
s
gsave
-0.0125 .61803 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .0824 m
.00375 .0824 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .16481 m
.00375 .16481 L
s
0 .18541 m
.00375 .18541 L
s
0 .22661 m
.00375 .22661 L
s
0 .24721 m
.00375 .24721 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .32962 m
.00375 .32962 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .49443 m
.00375 .49443 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .57683 m
.00375 .57683 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -105.875 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
88.0313 11.8438 translate
90 rotate
-88.0313 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(V) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
71.625 15.813 moveto
(@) show
75.563 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
81.688 15.813 moveto
(D) show
85.625 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(,) show
91.625 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(y) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
99.063 15.813 moveto
(@) show
103.000 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
109.125 15.813 moveto
(D) show
113.063 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.11905 .61178 m
.11905 .61803 L
s
.30952 .61178 m
.30952 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.69048 .61178 m
.69048 .61803 L
s
.88095 .61178 m
.88095 .61803 L
s
.125 Mabswid
.16667 .61428 m
.16667 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.2619 .61428 m
.2619 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.7381 .61428 m
.7381 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.07143 .61428 m
.07143 .61803 L
s
.02381 .61428 m
.02381 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.97619 .61428 m
.97619 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 0 m
1 0 L
s
.99375 .10301 m
1 .10301 L
s
.99375 .20601 m
1 .20601 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .41202 m
1 .41202 L
s
.99375 .51503 m
1 .51503 L
s
.125 Mabswid
.99625 .0206 m
1 .0206 L
s
.99625 .0412 m
1 .0412 L
s
.99625 .0618 m
1 .0618 L
s
.99625 .0824 m
1 .0824 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .14421 m
1 .14421 L
s
.99625 .16481 m
1 .16481 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .22661 m
1 .22661 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26781 m
1 .26781 L
s
.99625 .28842 m
1 .28842 L
s
.99625 .32962 m
1 .32962 L
s
.99625 .35022 m
1 .35022 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .39142 m
1 .39142 L
s
.99625 .43262 m
1 .43262 L
s
.99625 .45322 m
1 .45322 L
s
.99625 .47383 m
1 .47383 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .53563 m
1 .53563 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57683 m
1 .57683 L
s
.99625 .59743 m
1 .59743 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
.5 0 m
.5 .61803 L
s
gsave
.5 .63053 -119.938 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(a) show
71.375 15.813 moveto
(double) show
103.938 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
111.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(well) show
135.125 15.813 moveto
(potential) show
176.875 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 .05 .03 r
.02381 0 m
.02381 .61803 L
.30098 .61803 L
.30098 0 L
F
.69902 0 m
.69902 .61803 L
.97619 .61803 L
.97619 0 L
F
.46226 0 m
.46226 .61803 L
.53774 .61803 L
.53774 0 L
F
0 .26 .13 r
.30098 0 m
.30098 .61803 L
.46226 .61803 L
.46226 0 L
F
.53774 0 m
.53774 .61803 L
.69902 .61803 L
.69902 0 L
F
0 g
.5 Mabswid
.23115 .61803 m
.26171 .43981 L
.28158 .33843 L
.30316 .24121 L
.32216 .16893 L
.33296 .13461 L
.34309 .10772 L
.35264 .08759 L
.35756 .0794 L
.36292 .07226 L
.36584 .06919 L
.36855 .06686 L
.37114 .06512 L
.37247 .06441 L
.3739 .06379 L
.37532 .06332 L
.37662 .06301 L
.37786 .06284 L
.37917 .06278 L
.38035 .06283 L
.38145 .06296 L
.38266 .06322 L
.38395 .06361 L
.38517 .06409 L
.38651 .06475 L
.38924 .06648 L
.39176 .06857 L
.39413 .07095 L
.39945 .07775 L
.40519 .08725 L
.41569 .10993 L
.42531 .13578 L
.44584 .20002 L
.45585 .23154 L
.46494 .25769 L
.47378 .27923 L
.47843 .28855 L
.48336 .29668 L
.48587 .30007 L
.48863 .3032 L
.49117 .30549 L
.49349 .3071 L
.49471 .30775 L
.49587 .30824 L
.49709 .30863 L
.49778 .30879 L
.49843 .3089 L
.4996 .30901 L
.50066 .309 L
.50189 .30885 L
Mistroke
.50305 .3086 L
.50421 .30821 L
.50546 .30766 L
.50772 .30632 L
.51039 .30415 L
.51283 .30162 L
.51814 .2944 L
.52316 .28553 L
.54165 .23907 L
.5599 .18148 L
.58003 .12092 L
.58957 .09776 L
.59984 .0788 L
.60551 .07136 L
.60798 .06881 L
.61069 .06654 L
.61326 .06487 L
.61565 .06376 L
.61697 .06332 L
.61817 .06303 L
.61884 .06292 L
.61956 .06283 L
.62086 .06278 L
.6221 .06284 L
.62323 .06299 L
.62453 .06328 L
.62577 .06367 L
.62796 .06463 L
.63035 .06607 L
.63298 .06811 L
.63577 .07081 L
.64078 .07699 L
.65078 .09418 L
.66184 .12011 L
.68116 .18051 L
.70201 .26313 L
.74311 .46596 L
Mfstroke
.74311 .46596 m
.76865 .61803 L
s
1 g
[ .02 .04 ] 0 setdash
.02381 .25035 m
.06244 .25035 L
.10458 .25035 L
.14415 .25035 L
.18221 .25035 L
.22272 .25035 L
.26171 .25035 L
.30316 .25035 L
.34309 .25035 L
.3815 .25035 L
.42237 .25035 L
.46172 .25035 L
.49955 .25035 L
.53984 .25035 L
.57861 .25035 L
.61984 .25035 L
.65954 .25035 L
.69774 .25035 L
.73838 .25035 L
.77751 .25035 L
.81909 .25035 L
.85916 .25035 L
.89771 .25035 L
.93871 .25035 L
.97619 .25035 L
s
0 0 1 r
[ ] 0 setdash
.02381 .25035 m
.03279 .25036 L
.04262 .25038 L
.04758 .25038 L
.05288 .25039 L
.06244 .25042 L
.06788 .25044 L
.07379 .25046 L
.07899 .25049 L
.08451 .25053 L
.08947 .25057 L
.09494 .25062 L
.10458 .25075 L
.10978 .25084 L
.11472 .25094 L
.12399 .25119 L
.13397 .25155 L
.14487 .2521 L
.15519 .25283 L
.16454 .2537 L
.17534 .25503 L
.18551 .25668 L
.19528 .2587 L
.20559 .26139 L
.22406 .26793 L
.2437 .27774 L
.26204 .28982 L
.30094 .32303 L
.32075 .34083 L
.3312 .34923 L
.3423 .35675 L
.35214 .36179 L
.35484 .36286 L
.35774 .36384 L
.36025 .36455 L
.363 .36518 L
.36422 .3654 L
.36552 .3656 L
.36665 .36575 L
.36787 .36587 L
.36859 .36593 L
.36925 .36597 L
.37055 .36603 L
.37174 .36605 L
.37299 .36603 L
.37408 .36598 L
.37524 .36591 L
.37635 .3658 L
.37735 .36568 L
.37981 .3653 L
Mistroke
.38215 .3648 L
.38719 .3633 L
.39176 .36145 L
.402 .3557 L
.41148 .3486 L
.4204 .34062 L
.459 .29801 L
.50005 .25029 L
.53959 .20435 L
.57762 .16201 L
.58766 .15283 L
.59717 .14557 L
.60242 .14227 L
.60812 .13931 L
.6133 .13724 L
.6158 .13645 L
.61809 .13585 L
.62036 .13538 L
.62274 .13501 L
.62398 .13487 L
.62513 .13477 L
.6264 .1347 L
.6271 .13468 L
.62775 .13466 L
.62891 .13467 L
.62998 .1347 L
.63116 .13476 L
.6324 .13486 L
.63349 .13497 L
.6345 .13511 L
.6368 .13549 L
.63947 .13608 L
.642 .13679 L
.64674 .13846 L
.65741 .14379 L
.66705 .15018 L
.6773 .15824 L
.69571 .17459 L
.73343 .20747 L
.75275 .22085 L
.7627 .22651 L
.7736 .23177 L
.79362 .23908 L
.80454 .24196 L
.81471 .24407 L
.82521 .24575 L
.83514 .24697 L
.84405 .24782 L
.85372 .24852 L
.86317 .24903 L
Mistroke
.87357 .24944 L
.87881 .2496 L
.88443 .24974 L
.88971 .24986 L
.89461 .24994 L
.90472 .25008 L
.91006 .25013 L
.91576 .25018 L
.92119 .25021 L
.9263 .25024 L
.93585 .25028 L
.94051 .25029 L
.94562 .2503 L
.95096 .25031 L
.95594 .25032 L
.96061 .25032 L
.96486 .25033 L
.96722 .25033 L
.96977 .25033 L
.97094 .25033 L
.97219 .25033 L
.97334 .25033 L
.97442 .25033 L
.97527 .25033 L
.97619 .25033 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00j03oool01@2oM0000000001d_`3oool0gia80080000000<004RL0=ooo`3oool0_P3oool0
0>P0oooo00P0oooO09a8M02oool0oooo0?oog`2LB00004RL0=oool00oooo003Y0?ooo`060?oo_`1d
B9`0gooo0;md000007@0_ooo`@3oool00>X0oooo00@0omnL07A8B0000000M;oo`P3oool00>X0oooo
00<0oono07@00018W=l0``3oool00>T0oooo00D0oooO09a80000B9`0W7A809cOo`320?ooo`00j@3o
ool01P2oM000001809cOo`3ooml0W4Qd0;oool40oooo003W0?ooo`090?oog`2LB000000000000000
M;l0okmd0000000007@0_ooo0<00oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo001o0?ooo`040?oog`2LB00004RL0=oo
oc80oooo00<0_g@0000000000000100000000`18W=l0oooo0?ooo`0/0?ooo`050?oOW01800000000
0000M02oool0<P3oool00`2oM0000000000000040000000304RLg`3oool0oooo03@0oooo00@0oooO
09a80000B9`0goooC03oool007l0oooo00@0oooO09a80000B9`0gooo<`3oool00`3OW4P000000000
0003000000030018W03Oool0oooo02/0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo0380
oooo00<0gia80000000000000`0000000`00B9`0gooo0?ooo`0d0?ooo`040?oog`2LB00004RL0=oo
od`0oooo001o0?ooo`040?oog`2LB00004RL0=oooc@0oooo00<0gia804RLg`3oool00P3oool00`3O
W7@0W=oo0?ooo`0[0?ooo`030;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool0<P3oool00`3O
W4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo03@0oooo00@0oooO09a80000B9`0goooC03oool0
07/0oooo00<0_g@00000000000001@0000000`1d_ol0oooo0?ooo`0b0?ooo`030=nLB01d_ol0oooo
02h0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`0c0?ooo`030=nLB01d_ol0oooo
03@0oooo00<0_g@00000000000001@0000000`1d_ol0oooo0?ooo`190?ooo`00K`3oool00`3ooml0
W4P0000000070000000404RLg`3oool0omnL07BLg`80oooo00@0oooO09a80000B9`0gooo9@3oool0
0`3ooml0W4P0000000070000000304RLg`3oool0oooo00@0oooo00<0_gA809cOo`3oool0;@3oool0
0`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo03@0oooo00<0_gA809cOo`3oool0<`3oool0
203ogi`0M9cO0?ooo`3oool0oooO09a80000B9`0goooC03oool007`0oooo00L0gia807Boo`3oool0
oooO09a80000B9`0gooo03H0oooo00<0oooO09a8M02oool0;@3oool00`3ogi`0B00007Boo`030?oo
o`030?noM00004P0W=oo03@0oooo00<0oooO09a8M02oool0=03oool01`3OW4P0M;oo0?ooo`3ooml0
W4P00018W03Oool0C03oool007`0oooo00L0oooO09a8M02oool0oooO09a80000B9`0gooo03L0oooo
00<0okmd001d_`3oool0;03oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo03D0oooo
00<0okmd001d_`3oool0<`3oool01`3ooml0W4Qd0;ooo`3ooml0W4P00018W03Oool0C03oool007d0
oooo00H0omnL04Qd_`3ooml0W4P00018W03Ooolg0?ooo`030?oo_`1d04P0W=oo02`0oooo00<0omnL
04P0001d_ol00`3oool00`3o_g@0001809cOo`0e0?ooo`030?oo_`1d04P0W=oo03@0oooo00H0omnL
04Qd_`3ooml0W4P00018W03Ooom<0?ooo`00OP3oool01@3OW4P0M;oO09a80000B9`0gooo03L0oooo
00<0oono07@00018W=l0;03oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo03D0oooo
00<0oono07@00018W=l0=@3oool01@3OW4P0M;oO09a80000B9`0gooo04`0oooo001n0?ooo`050?oo
g`2LB4P0B4P00018W03Oool0<P3oool00`3o_i`0W=oo0?ooo`020?ooo`030?oOW0180000B9cO02d0
oooo00<0gia80018W03Oool00P3oool00`2oM00007Bo0?ooo`0`0?ooo`030?noW02Lgol0oooo0080
oooo00<0omnL04P00018W=l0=@3oool01@3ooml0W4Q804Q80000B9`0gooo04`0oooo001o0?ooo`04
0?oOW018000004RL0=oooc80oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooobd0oooo
00L0oono07@0001d_ol0oooo0?noM00007@0_ooo0340oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d
0000001d0;ooocH0oooo00@0omnL04P00000B9`0goooC03oool00800oooo00<0gia80018W03Oool0
<`3oool01P3ooml0W4P0000000000000001d0;ooobl0oooo00D0oono07@000000000001d0;ooo`0c
0?ooo`060?oog`2LB00000000000000007@0_ooo>03oool00`3OW4P004RL0=ooo`1<0?ooo`00o`3o
oona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3o
oona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00E@3ooooo000003800000:P3oool005D0oooo
00<000000?ooo`3oool0103ooomE07l<1c40048Q5P1o30La00128ED0O``700<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?oooeD0O``7<@00
@R4F07l<1c40048QE@1o30L00`000000oooo0?ooo`040?ooo`030000003oool0oooo02P0oooo001E
0?ooo`030000003oool0oooo00@0ooooE@1o30La00128AH0O``7<@00@R5E07l<1`L0oooo00<00000
0?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0103ooomE07l<1c40048Q5P1o30La0012
8ED0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?oooeD0
O``7<@00@R4F07l<1c40048QE@1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000
003oool0oooo00@0ooooE@1o30La00128AH0O``7<@00@R5E07l<1`L0oooo00<000000?ooo`3oool0
:03oool005D0oooo0P0000050?oooeD0O``7<@00@R4F07l<1c40048QE@1o30L60?ooo`800000:P3o
ool005D0oooo00<000000?ooo`3oool0103ooomE07l<1c40048Q5P1o30La00128ED0O``71`3oool0
0`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?oooeD0O``7<@00@R4F07l<
1c40048QE@1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo00@0
ooooE@1o30La00128AH0O``7<@00@R5E07l<1`L0oooo00<000000?ooo`3oool0:03oool005D0oooo
00<000000?ooo`3oool0103ooomE07l<1c40048Q5P1o30La00128ED0O``71`3oool00`000000oooo
0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?oooeD0O``7<@00@R4F07l<1c40048QE@1o
30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`8000001@3ooomE07l<1c40048Q5P1o30La
00128ED0O``71P3oool2000002X0oooo001E0?ooo`030000003oool0oooo00@0ooooE@1o30La0012
8AH0O``7<@00@R5E07l<1`L0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3o
ool0103ooomE07l<1c40048Q5P1o30La00128ED0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00
E@3oool00`000000oooo0?ooo`040?oooeD0O``7<@00@R4F07l<1c40048QE@1o30L70?ooo`030000
003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo00@0ooooE@1o30La00128AH0O``7<@00
@R5E07l<1`L0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0103ooomE
07l<1c40048Q5P1o30La00128ED0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool20000
00D0ooooE@1o30La00128AH0O``7<@00@R5E07l<1`H0oooo0P00000Z0?ooo`00E@3oool00`000000
oooo0?ooo`040?oooeD0O``75@00@R46000001H0048Q5P1o30LG00128@H000005000@R5E07l<1`L0
oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0103ooomE07l<1a<0048Q
0P00000500128@8000005@00@R4F07l<1aH0048Q0P00000500128@0300000000@R40048Q0140048Q
E@1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo00@0ooooE@1o
30LB00128@0300000000@R40048Q00L0048Q00<0000000128@00@R404P00@R4F07l<1aD0048Q00<0
000000128@00@R401P00@R400`000000048Q00128@0@00128ED0O``71`3oool00`000000oooo0?oo
o`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?oooeD0O``74P00@R400`000000048Q00128@07
00128@0300000000@R40048Q0180048Q5P1o30LD00128@0300000000@R40048Q00P0048Q00<00000
00128@00@R403`00@R5E07l<1`L0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?oo
o`3oool0103ooomE07l<1a40048Q00<0000000128@00@R402@00@R400`000000048Q00128@0A0012
8AH0O``74`00@R400`000000048Q00128@0:00128@0300000000@R40048Q00h0048QE@1o30L70?oo
o`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo00@0ooooE@1o30LA00128@03
00000000@R40048Q00X0048Q00<0000000128@00@R404000@R4F07l<1a80048Q00<0000000128@00
@R403000@R400`000000048Q00128@0=00128ED0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00
E@3oool2000000D0ooooE@1o30L@00128@0300000000@R40048Q00/0048Q00<0000000128@00@R40
4000@R4F07l<1a80048Q00<0000000128@00@R403000@R400`000000048Q00128@0=00128ED0O``7
1P3oool2000002X0oooo00150?ooo`030?oo_`1d000000000080000000<0M;oo0?ooo`3oool0203o
ool00`000000oooo0?ooo`040?oooeD0O``73`00@R400`000000048Q00128@0=00128@0300000000
@R40048Q00l0048Q5P1o30LA00128@0300000000@R40048Q00h0048Q00<0000000128@00@R403000
@R5E07l<1`L0oooo00<000000?ooo`3oool0:03oool004D0oooo00<0omnL04P00018W=l00P3oool0
0`2oM000B9cO0?ooo`080?ooo`030000003oool0oooo00@0ooooE@1o30L>00128@0300000000@R40
048Q00h0048Q00<0000000128@00@R403`00@R4F07l<1a40048Q00<0000000128@00@R403P00@R40
0`000000048Q00128@0<00128ED0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00BP3oool00`3o
okl0M01809cOo`080?ooo`030000003oool0oooo00@0ooooE@1o30L>00128@0300000000@R40048Q
00l0048Q00<0000000128@00@R403P00@R4F07l<1a00048Q00<0000000128@00@R404000@R400`00
0000048Q00128@0;00128ED0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00BP3oool00`3ookl0
M00007Boo`080?ooo`030000003oool0oooo00@0ooooE@1o30L=00128@0300000000@R40048Q0100
048Q00<0000000128@00@R403P00@R4F07l<1a00048Q00<0000000128@00@R404000@R400`000000
048Q00128@0;00128ED0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00BP3oool00`3o_g@00000
07Boo`080?ooo`030000003oool0oooo00@0ooooE@1o30L=00128@0300000000@R40048Q0140048Q
00<0000000128@00@R403@00@R4F07l<1a00048Q00<0000000128@00@R404@00@R400`000000048Q
00128@0:00128ED0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00B@3oool0103ogi`0B0000000
B02Lgol80?ooo`<00000103ooomE07l<1``0048Q00<0000000128@00@R404P00@R400`000000048Q
00128@0=00128AH0O``73`00@R400`000000048Q00128@0B00128@0300000000@R40048Q00X0048Q
E@1o30L50?ooo`<00000:P3oool004L0oooo00D0okmd000000000000001809cOo`090?ooo`030000
003oool0oooo00@0ooooE@1o30L<00128@0300000000@R40048Q01<0048Q00<0000000128@00@R40
3000@R4F07l<1`l0048Q00<0000000128@00@R404P00@R400`000000048Q00128@0:00128ED0O``7
1`3oool00`000000oooo0?ooo`0X0?ooo`00B@3oool00`2oM00007Bo0?ooo`090?ooo`030000003o
ool0oooo00@0ooooE@1o30L<00128@0300000000@R40048Q01<0048Q00<0000000128@00@R403000
@R4F07l<1`h0048Q00<0000000128@00@R405000@R400`000000048Q00128@0900128ED0O``71`3o
ool00`000000oooo0?ooo`0X0?ooo`00BP3oool00`2oM000B9cO0?ooo`080?ooo`030000003oool0
oooo00@0ooooE@1o30L;00128@0300000000@R40048Q01@0048Q00<0000000128@00@R403000@R4F
07l<1`h0048Q00<0000000128@00@R405000@R400`000000048Q00128@0900128ED0O``71`3oool0
0`000000oooo0?ooo`0X0?ooo`00A@3oool00`3ooml0W9bo0?ooo`020?ooo`030=nLB00007@0_ooo
00P0oooo00<000000?ooo`3oool0103ooomE07l<1`/0048Q00<0000000128@00@R405@00@R400`00
0000048Q00128@0;00128AH0O``73@00@R400`000000048Q00128@0F00128@0300000000@R40048Q
00P0048QE@1o30L70?ooo`030000003oool0oooo02P0oooo00160?ooo`060?noM018W=l0oooo0?oo
_`1d000007Bo2@3oool00`000000oooo0?ooo`040?oooeD0O``72P00@R400`000000048Q00128@0F
00128@0300000000@R40048Q00/0048Q5P1o30L=00128@0300000000@R40048Q01H0048Q00<00000
00128@00@R402000@R5E07l<1`L0oooo00<000000?ooo`3oool0:03oool004L0oooo00D0gia80000
00000000001d0;ooo`090?ooo`8000001@3ooomE07l<1`X0048Q00<0000000128@00@R405P00@R40
0`000000048Q00128@0;00128AH0O``73@00@R400`000000048Q00128@0F00128@0300000000@R40
048Q00P0048QE@1o30L60?ooo`800000:P3oool005D0oooo00<000000?ooo`3oool0103ooomE07l<
1`X0048Q00<0000000128@00@R405`00@R400`000000048Q00128@0:00128AH0O``73000@R400`00
0000048Q00128@0H00128@0300000000@R40048Q00L0048QE@1o30L70?ooo`030000003oool0oooo
02P0oooo001E0?ooo`030000003oool0oooo00@0ooooE@1o30L900128@0300000000@R40048Q01P0
048Q00<0000000128@00@R402P00@R4F07l<1``0048Q00<0000000128@00@R406000@R400`000000
048Q00128@0700128ED0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo
0?ooo`040?oooeD0O``72@00@R400`000000048Q00128@0I00128@0300000000@R40048Q00T0048Q
5P1o30L<00128@0300000000@R40048Q00/0048Q1@000?l800128@0300000000@R40048Q00L0048Q
E@1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo00@0ooooE@1o
30L900128@0300000000@R40048Q01T0048Q00<0000000128@00@R402@00@R4F07l<1`/0048Q00<0
000000128@00@R402@00@R440000o`<0048Q0`000?l700128@0300000000@R40048Q00H0048QE@1o
30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo00@0ooooE@1o30L8
00128@0300000000@R40048Q01X0048Q00<0000000128@00@R402@00@R4F07l<1`/0048Q00<00000
00128@00@R402000@R400`000?l0048Q00128@0800128@<0003o1000@R400`000000048Q00128@06
00128ED0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool2000000D0ooooE@1o30L80012
8@0300000000@R40048Q01/0048Q00<0000000128@00@R402000@R4F07l<1`/0048Q00<000000012
8@00@R401P00@R420000o`h0048Q0P000?l200128@0300000000@R40048Q00H0048QE@1o30L60?oo
o`800000:P3oool005D0oooo00<000000?ooo`3oool0103ooomE07l<1`P0048Q00<0000000128@00
@R406`00@R400`000000048Q00128@0800128AH0O``72P00@R400`000000048Q00128@0600128@03
0000o`00@R40048Q0100048Q0P000?l00`00@R40000000128@0600128ED0O``71`3oool00`000000
oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?oooeD0O``72000@R400`000000048Q
00128@0K00128@0300000000@R40048Q00P0048Q5P1o30L:00128@0300000000@R40048Q00@0048Q
0P000?lE00128@030000o`000000048Q00H0048QE@1o30L70?ooo`030000003oool0oooo02P0oooo
001E0?ooo`030000003oool0oooo00@0ooooE@1o30L700128@0300000000@R40048Q01d0048Q00<0
000000128@00@R401`00@R4F07l<1`X0048Q00<0000000128@00@R400`00@R400`000?l0048Q0012
8@0F00128@030000o`00@R40048Q00D0048QE@1o30L70?ooo`030000003oool0oooo02P0oooo001E
0?ooo`030000003oool0oooo00@0ooooE@1o30L700128@0300000000@R40048Q01d0048Q00<00000
00128@00@R401`00@R4F07l<1`X0048Q00<0000000128@00@R400P00@R400`000?l0048Q00128@0H
00128@030000o`00@R40048Q00@0048QE@1o30L70?ooo`030000003oool0oooo02P0oooo001E0?oo
o`030000003oool0oooo00@0ooooE@1o30L700128@0300000000@R40048Q01h0048Q00<000000012
8@00@R401P00@R4F07l<1`T0048Q00<0000000128@00@R400P00@R400`000?l0048Q00128@0I0012
8@03000000000?l0048Q00@0048QE@1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`03
0000003oool0oooo00@0ooooE@1o30L700128@0300000000@R40048Q01h0048Q00<0000000128@00
@R401P00@R4F07l<1`T0048Q00D0000000128@00@R40048Q0000o`0L00128@0300000000@R40003o
00@0048QE@1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`8000001@3ooomE07l<1`H0
048Q00<0000000128@00@R407`00@R400`000000048Q00128@0600128AH0O``72@00@R4010000000
048Q00128@000?lN00128@0300000000@R40003o00<0048QE@1o30L60?ooo`800000:P3oool005D0
oooo00<000000?ooo`3oool0103ooomE07l<1`H0048Q00<0000000128@00@R408000@R400`000000
048Q00128@0500128AH0O``72000@R4010000000048Q00128@000?lO00128@0400000000@R40048Q
0000o`80048QE@1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo
00@0ooooE@1o30L600128@0300000000@R40048Q0200048Q00<0000000128@00@R401@00@R4F07l<
1`P0048Q00<0000000128@000?l08000@R401P000000048Q00128@00@R40003o00128ED0O``71`3o
ool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?oooeD0O``71@00@R40
0`000000048Q00128@0Q00128@0300000000@R40048Q00D0048Q5P1o30L800128@03000000000?l0
048Q0240048Q00D0000000128@00@R40048Q0000o`1E07l<1`L0oooo00<000000?ooo`3oool0:03o
ool005D0oooo00<000000?ooo`3oool0103ooomE07l<1`D0048Q00<0000000128@00@R408P00@R40
0`000000048Q00128@0400128AH0O``71`00@R400`000000003o00128@0R00128@0300000000@R40
048Q0080048Q00<0003o07l<1`1o30L0DP1o30L70?ooo`030000003oool0oooo02P0oooo001E0?oo
o`030000003oool0oooo00@0ooooE@1o30L500128@0300000000@R40048Q0280048Q00<000000012
8@00@R401000@R4F07l<1`L0048Q00<0003o00128@00@R408P00@R400`000000048Q00128@020012
8@0307l<1`000?l0003o0580O``71`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool2000000D0
ooooE@1o30L500128@0300000000@R40048Q02<0048Q00<0000000128@00@R400`00@R4F07l<1`L0
048Q00<0003o00128@00@R408P00@R400`000000048Q00128@0200128@<0O``700<0003o07l<1`1o
30L0C`1o30L60?ooo`800000:P3oool002h0oooo00<0giad09cOo`3oool0603oool0103ooml0W4P0
0018W03Oool80?ooo`030000003oool0oooo00@0ooooE@1o30L400128@0300000000@R40048Q02@0
048Q00<0000000128@00@R400`00@R4F07l<1`H0048Q00<0003o00128@00@R409000@R4010000000
048Q00128@00@R4407l<1`030000o`1o30L0O``704h0O``71`3oool00`000000oooo0?ooo`0X0?oo
o`00;P3oool00`3OW4P007Bo0?ooo`0H0?ooo`040?oog`2LB00004RL0=ooo`P0oooo00<000000?oo
o`3oool0103ooomE07l<1`@0048Q00<0000000128@00@R409000@R400`000000048Q00128@030012
8AH0O``71@00@R400`000?l0000000128@0U00128@0400000000@R40048Q00128@D0O``70P000?m>
07l<1`L0oooo00<000000?ooo`3oool0:03oool002h0oooo00D0gia8000000000000000007Boo`0F
0?ooo`040?oog`2LB00004RL0=ooo`P0oooo00<000000?ooo`3oool0103ooomE07l<1`@0048Q00<0
000000128@00@R409@00@R400`000000048Q00128@0200128AH0O``71000@R400`000?l0048Q0000
000V00128@0400000000@R40048Q00128@L0O``700<0003o07l<1`1o30L0B`1o30L70?ooo`030000
003oool0oooo02P0oooo000^0?ooo`030=nLB0000000000000<0000000<0001d0;ooo`3oool03P3o
ool00`2oM0000000000000050000000307Boo`3oool0oooo00D0oooo00<000000?ooo`3oool0103o
oomE07l<1`@0048Q00<0000000128@00@R409@00@R400`000000048Q00128@0200128AH0O``70`00
@R4010000?l0048Q00128@00000V00128@0400000000@R40048Q00128@P0O``700<0003o07l<1`1o
30L0BP1o30L70?ooo`030000003oool0oooo02P0oooo000^0?ooo`060=nLM02Lgol0oooo0?ooo`3o
gi`0B0000`0000000`00B9`0gooo0?ooo`0;0?ooo`080?oOW01dW=l0oooo0?ooo`3ooml0W4P00018
W03Oool80?ooo`030000003oool0oooo00@0ooooE@1o30L300128@0300000000@R40048Q02H0048Q
00<0000000128@00@R400P00@R4F07l<1`80048Q00@0003o00128@00@R400000:000@R400`000000
048Q00128@0907l<1`80003oBP1o30L70?ooo`030000003oool0oooo02P0oooo000e0?ooo`030?no
M000000000000080000000<007Bo0?ooo`3oool02@3oool01`3OW4P0M;oo0?ooo`3ooml0W4P00018
W03Oool0203oool3000000@0ooooE@1o30L300128@0300000000@R40048Q02L0048Q00@000000012
8@00@R40048Q5P1o30L00`00@R40003o00128@0200128@0300000000@R40048Q02H0048Q00<00000
00128@00@R402`1o30L00`000?l0O``707l<1`1707l<1`D0oooo0`00000Z0?ooo`00>03oool0103o
_g@000000000M02oool:0?ooo`070?oog`2LB7@0_ooo0?oog`2LB00004RL0=ooo`080?ooo`030000
003oool0oooo00@0ooooE@1o30L300128@0300000000@R40048Q02L0048Q00@0000000128@00@R40
048Q5P1o30L00`000?l0048Q00128@0200128@0300000000@R40048Q02H0048Q00<0000000128@00
@R40301o30L00`000?l0O``707l<1`1607l<1`L0oooo00<000000?ooo`3oool0:03oool003D0oooo
00@0okmd00000000000007Bo3P3oool01P3ogi`0B7Bo0?oog`2LB00004RL0=ooo`P0oooo00<00000
0?ooo`3oool0103ooomE07l<1`80048Q00<0000000128@00@R40:000@R4010000000048Q00128@00
@R4E07l<1`030000o`00@R40048Q0080048Q00<0000000128@00@R409`00@R400`000000048Q0012
8@0=07l<1`030000o`1o30L0O``704D0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00;P3oool0
2@3OW7@0W=oo0?ooo`3oool0omnL04P00000000004RL0=ooo`0A0?ooo`050=nLB01d_ml0W4P00018
W03Oool0203oool00`000000oooo0?ooo`040?oooeD0O``70P00@R400`000000048Q00128@0X0012
8@0400000000@R40048Q00128A@0O``700<0003o07l<1`00@R400`00@R400`000000048Q00128@0X
00128@0300000000@R40O``700d0O``70P000?m507l<1`L0oooo00<000000?ooo`3oool0:03oool0
02h0oooo00H0gia800000000000000000000M02ooolD0?ooo`050?oog`2LB4P0B4P00018W03Oool0
203oool00`000000oooo0?ooo`040?oooeD0O``70P00@R400`000000048Q00128@0Y00128@030000
0000@R40048Q01<0O``700<0003o07l<1`1o30L01000@R400`000000048Q00128@0X00128@030000
0000@R40O``700l0O``700<0003o07l<1`1o30L0@P1o30L70?ooo`030000003oool0oooo02P0oooo
000^0?ooo`030=nLB0000000M;oo01P0oooo00@0omnL04P00000B9`0gooo203oool00`000000oooo
0?ooo`040?oooeD0O``70P00@R400`000000048Q00128@0Y00128@0300000000@R40048Q0180O``7
00@0003o07l<1`1o30L0O``70`00@R400`000000048Q00128@0Y00128@0300000000@R40O``70100
O``70P000?m207l<1`L0oooo00<000000?ooo`3oool0:03oool002h0oooo00<0giad09cOo`3oool0
6@3oool00`3OW4P004RL0=ooo`080?ooo`8000001@3ooomE07l<1`0300128@000000048Q02/0048Q
00<0000000128@00@R404@1o30L00`000?l0O``707l<1`0207l<1`<0048Q00<0000000128@00@R40
:@00@R400`000000048Q07l<1`0B07l<1`80003o@01o30L60?ooo`800000:P3oool005D0oooo00<0
00000?ooo`3oool0103ooomE07l<1`0300128@000000048Q02`0048Q00<0000000128@1o30L0401o
30L00`000?l0O``707l<1`0207l<1`<0048Q00<0000000128@00@R40:P00@R400`000000O``707l<
1`0C07l<1`80003o?P1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0
oooo00@0ooooE@1o30L00`00@R40000000128@0/00128@0300000000@R40O``700l0O``700<0003o
07l<1`1o30L00`1o30L300128@0300000000@R40048Q02X0048Q00<0000007l<1`1o30L05@1o30L2
0000oc`0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00;@3oool00`3ogi`0B0000000000>0000
00030000M02oool0oooo01@0oooo00<000000?ooo`3oool0103ooomE07l<1`0300128@000000048Q
02`0048Q00<0000000128@1o30L03P1o30L00`000?l0O``707l<1`0407l<1`80048Q00<000000012
8@00@R40:`00@R400`000000O``707l<1`0G07l<1`<0003o>@1o30L70?ooo`030000003oool0oooo
02P0oooo000]0?ooo`030?oOW01dW=l0oooo00h0oooo00<0_gAd0;ooo`3oool0503oool00`000000
oooo0?ooo`040?oooeD0O``700<0000000128@00@R40;000@R400`000000048Q07l<1`0=07l<1`03
0000o`1o30L0O``700D0O``70P00@R400`000000048Q00128@0[00128@030000001o30L0O``701X0
O``70`000?lf07l<1`L0oooo00<000000?ooo`3oool0:03oool002d0oooo00<0omnL09boo`3oool0
3P3oool00`3OW9`0_ooo0?ooo`0D0?ooo`030000003oool0oooo00@0ooooE@1o30L00`000000048Q
00128@0]00128@030000001o30L0O``700/0O``700<0003o07l<1`1o30L01P1o30L200128@030000
0000@R40048Q02`0048Q00<0000007l<1`1o30L0701o30L40000oc80O``71`3oool00`000000oooo
0?ooo`0X0?ooo`00;@3oool00`3ogi`0W;oo0?ooo`0>0?ooo`030=nLW02oool0oooo01@0oooo00<0
00000?ooo`3oool0103ooomE07l<1`0300000000@R40048Q02d0048Q00<0000007l<1`1o30L02P1o
30L00`000?l0O``707l<1`0707l<1`0300128@000000048Q02h0048Q00<0000007l<1`1o30L0801o
30L90000obT0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool2000000D0ooooE@1o30L0
0`000000048Q00128@0]00128@030000001o30L0O``700T0O``700<0003o07l<1`1o30L0201o30L0
0`00@R40000000128@0^00128@030000001o30L0O``702T0O``78P000?l707l<1`H0oooo0P00000Z
0?ooo`00<`3oool00`2oM7@0_ooo0?ooo`040?ooo`030;mdM02oool0oooo01P0oooo00<000000?oo
o`3oool0103ooolZ0000o`80oooo2`1o30L70?ooo`/0O``71`3oool407l<1`0300000000@R40048Q
00@0048Q203oool;00128@L0oooo2`00@R470?ooo`T0O``700<0003o07l<1`3oool01P3oool307l<
1`0300000000@R40048Q00D0048Q1`3oool<00128@L0oooo2`00@R470?ooo`/0O``71`3oool;07l<
1`L0oooo2`1o30L80?ooo`/0O``71`3oool207l<1`P0003o1P3oool00`000000oooo0?ooo`0X0?oo
o`00<`3oool00`2oM000B9cO0?ooo`030?ooo`030=nLB00007@0_ooo01T0oooo00<000000?ooo`3o
ool0103ooolZ07l<1`T0003o8@1o30L00`000000048Q00128@0_00128@030000001o30L0O``700H0
O``700<0003o07l<1`1o30L02P1o30L00`000000048Q00128@0^00128@0307l<1`000000O``70580
O``71`3oool00`000000oooo0?ooo`0X0?ooo`00<`3oool02@2oM00000000000M02oool0oooO09a8
0018W=l0gibL0;ooo`0I0?ooo`030000003oool0oooo00@0oooo<`1o30L30000oah0O``700<00000
00128@00@R40;`00@R400`000000O``707l<1`0607l<1`030000o`1o30L0O``700T0O``700<00000
00128@00@R40;`00@R400`1o30L0000007l<1`1B07l<1`L0oooo00<000000?ooo`3oool0:03oool0
03<0oooo00H0_ibL0=oOW018000000000000B02LgolL0?ooo`030000003oool0oooo00@0oooo=P1o
30L40000oaX0O``700<0000000128@00@R40;`00@R400`000000O``707l<1`0507l<1`030000o`1o
30L0O``700X0O``700<0000000128@00@R40;`00@R400`1o30L0000007l<1`1B07l<1`L0oooo00<0
00000?ooo`3oool0:03oool003<0oooo00<0_gAd0;oOW01800000P000000100004P0W=oo0=nLW02o
oolI0?ooo`030000003oool0oooo00@0oooo>P1o30L20000oaP0O``700<0000000128@00@R40;`00
@R400`1o30L0000007l<1`0407l<1`030000o`1o30L0O``700/0O``700<0000000128@00@R40;`00
@R400`1o30L0000007l<1`1B07l<1`L0oooo00<000000?ooo`3oool0:03oool003<0oooo00<0_g@0
0018W03Oool00P3oool0102oM00000000000M02ooolI0?ooo`8000001@3oooll07l<1`80003o5@1o
30L00`000000O``700128@0`00128@0307l<1`000000O``700<0O``700<0003o07l<1`1o30L02`1o
30L00`000000O``700128@0`00128@0307l<1`000000O``70580O``71P3oool2000002X0oooo000c
0?ooo`030;mdB02Lgol0oooo00<0oooo00<0okmd0000M02oool06@3oool00`000000oooo0?ooo`04
0?oooch0O``70P000?lC07l<1`030000001o30L0048Q0300048Q0P1o30L010000000O``707l<1`00
0?l>07l<1`030000001o30L0048Q0300048Q0P1o30L00`000000O``707l<1`1@07l<1`L0oooo00<0
00000?ooo`3oool0:03oool003<0oooo00<0_ibL0=ooo`3oool0103oool00`3OW9`0_ooo0?ooo`0H
0?ooo`030000003oool0oooo00@0oooo@01o30L20000oa40O``700<0000007l<1`00@R40<000@R42
07l<1`040000001o30L0O``70000o`d0O``700<0000007l<1`1o30L0<@00@R4207l<1`030000001o
30L0O``70500O``71`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`04
0?oood80O``70P000?l?07l<1`030000001o30L0048Q0300048Q0`1o30L00`000000003o07l<1`0=
07l<1`030000001o30L0O``70340048Q0P1o30L00`000000O``707l<1`1@07l<1`L0oooo00<00000
0?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0103ooom407l<1`80003o3@1o30L00`00
0000O``700128@0`00128@<0O``700<0003o07l<1`1o30L0301o30L010000000O``707l<1`1o30La
00128@80O``700<0000007l<1`1o30L0D01o30L70?ooo`030000003oool0oooo02P0oooo000]0?oo
o`030?oOW02L_ol0oooo00h0oooo00<0gibL0;ooo`3oool0503oool00`000000oooo0?ooo`040?oo
odH0O``70P000?l:07l<1`030000001o30L0O``70340048Q0P1o30L00`000?l0O``70000000=07l<
1`040000001o30L0O``707l<1c40048Q0P1o30L00`000000O``707l<1`1@07l<1`L0oooo00<00000
0?ooo`3oool0:03oool002d0oooo00<0omnL09boo`3oool03P3oool00`3OW9`0_ooo0?ooo`0D0?oo
o`8000001@3ooom807l<1`80003o201o30L00`000000O``707l<1`0a00128@0507l<1`000?l0O``7
07l<1`0000003@1o30L010000000O``707l<1`1o30La00128@<0O``700<0000007l<1`1o30L0C`1o
30L60?ooo`800000:P3oool002d0oooo00<0omnL07BLg`3oool03P3oool00`2oM7@0_ooo0?ooo`04
0?ooo`030?oog`2LB00000000080000000<0M;oo0?ooo`3oool0203oool00`000000oooo0?ooo`04
0?ooodX0O``700<0003o07l<1`1o30L01@1o30L00`000000O``707l<1`0a00128@0307l<1`000?l0
O``70080O``700<0000007l<1`1o30L02@1o30L00`000000O``707l<1`0207l<1c40048Q0`1o30L0
0`000000O``707l<1`1?07l<1`L0oooo00<000000?ooo`3oool0:03oool002d0oooo00<0omnL04P0
000000003P0000000`0007@0_ooo0?ooo`040?ooo`030?oog`2LB7@0_ooo0080oooo00<0_g@007Bo
o`3oool0203oool00`000000oooo0?ooo`040?oood/0O``700<0003o07l<1`1o30L0101o30L00`00
0000O``707l<1`0a00128@030000o`1o30L0O``70080O``700<0000007l<1`1o30L02@1o30L00`00
0000O``707l<1`0207l<1c40048Q0`1o30L00`000000O``707l<1`1?07l<1`L0oooo00<000000?oo
o`3oool0:03oool004X0oooo00<0oooO09a8M02oool0203oool00`000000oooo0?ooo`040?oood`0
O``700<0003o07l<1`1o30L00P1o30L010000000O``707l<1`1o30L`00128@030000o`1o30L0O``7
00@0O``700<0000007l<1`1o30L01`1o30L00`000000O``707l<1`0307l<1c40048Q0`1o30L00`00
0000O``707l<1`1?07l<1`L0oooo00<000000?ooo`3oool0:03oool004/0oooo00<0_g@007Boo`3o
ool01`3oool00`000000oooo0?ooo`040?ooodd0O``700D0003o07l<1`1o30L0O``70000000307l<
1bl0048Q00<0003o00128@1o30L01P1o30L00`000000O``707l<1`0507l<1`030000001o30L0O``7
00@0O``7<@00@R4407l<1`030000001o30L0O``704h0O``71`3oool00`000000oooo0?ooo`0X0?oo
o`00>03oool01P3ooml0W4P00018W03Oool0oooO09bL_``0oooo00<0oooO09a80018W=l0203oool0
0`000000oooo0?ooo`040?ooodh0O``700@0003o07l<1`1o30L000000`1o30L^00128@030000o`00
@R40048Q00P0O``700<0000007l<1`1o30L00`1o30L00`000000O``707l<1`0507l<1c40048Q101o
30L00`000000O``707l<1`1>07l<1`L0oooo00<000000?ooo`3oool0:03oool003P0oooo00H0oono
07@00000B9`0gooo0;mdB02Lgol<0?ooo`030=nLB0000000M;oo00P0oooo0`0000040?ooodl0O``7
0P000?l010000000O``707l<1`1o30L]00128@040000o`00@R40048Q00128@T0O``71@00000807l<
1c40048Q101o30L00`000000O``707l<1`1>07l<1`D0oooo0`00000Z0?ooo`00>@3oool0103OW4P0
00000000001d_ol90?ooo`070=oOg`3oool0oono07@000000000001d0;ooo`080?ooo`030000003o
ool0oooo00@0ooooD@1o30L010000?l0O``707l<1`1o30L/00128@030000o`00@R40048Q0080048Q
5P1o30La00128@@0O``700<0000007l<1`1o30L0CP1o30L70?ooo`030000003oool0oooo02P0oooo
00160?ooo`060=nLB000000000000000000004P0W=oo2@3oool00`000000oooo0?ooo`040?oooe00
O``700<0000007l<1`000?l00P1o30L[00128@030000o`00@R40048Q00<0048Q5P1o30La00128@@0
O``700<0000007l<1`1o30L0CP1o30L70?ooo`030000003oool0oooo02P0oooo00160?ooo`040?oo
_`1d0000000004RLg`/0oooo00<000000?ooo`3oool0103ooom@07l<1`050000001o30L0O``70000
o`1o30L0:P00@R400`000?l0048Q00128@0400128AH0O``7<@00@R4507l<1`030000001o30L0O``7
04d0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00A`3oool00`2oM000M;oo0?ooo`0;0?ooo`03
0000003oool0oooo00@0ooooD01o30L01@000000O``707l<1`1o30L0003o02X0048Q00<0003o0012
8@00@R401000@R4F07l<1c40048Q1@1o30L00`000000O``707l<1`1=07l<1`L0oooo00<000000?oo
o`3oool0:03oool004L0oooo00H0omnL04P00000000000000000B02Lgol80?ooo`030000003oool0
oooo00@0ooooD01o30L00`000000O``707l<1`0207l<1`030000o`00@R40048Q02H0048Q00<0003o
00128@00@R401@00@R4F07l<1c40048Q1@1o30L00`000000O``707l<1`1=07l<1`L0oooo00<00000
0?ooo`3oool0:03oool004P0oooo00D0_g@0000000000000000004RLg`080?ooo`030000003oool0
oooo00@0ooooD01o30L00`000000O``707l<1`0207l<1`0300128@000?l0048Q02D0048Q00<0003o
00128@00@R401P00@R4F07l<1c40048Q1@1o30L00`000000O``707l<1`1=07l<1`L0oooo00<00000
0?ooo`3oool0:03oool005D0oooo0P0000050?ooodl0O``700<0000007l<1`1o30L00`1o30L20012
8@030000o`00@R40048Q0280048Q00<0003o00128@00@R401`00@R4F07l<1c40048Q1@1o30L00`00
0000O``707l<1`1=07l<1`H0oooo0P00000Z0?ooo`00<P3oool01@3ogi`0M9cO0?ooo`3ookl0M000
0080000000<0001d0;ooo`3oool06@3oool00`000000oooo0?ooo`040?ooodl0O``700<0000007l<
1`1o30L00`1o30L300128@80003o8@00@R400`000?l0048Q00128@0800128AH0O``7<@00@R4607l<
1`030000001o30L0O``704`0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00<P3oool00`3ooml0
W4P0000000030000000407Bo_`1d000004RL0=oooaT0oooo00<000000?ooo`3oool0103ooom?07l<
1`030000001o30L0O``700<0O``71@00@R400`000?l0048Q00128@0M00128@030000o`00@R40048Q
00T0048Q5P1o30La00128@H0O``700<0000007l<1`1o30L0C01o30L70?ooo`030000003oool0oooo
02P0oooo000j0?ooo`060;mdB02Lgkl0M00000000000B9`0gooo5@3oool00`000000oooo0?ooo`04
0?ooodl0O``700<0000007l<1`1o30L00`1o30L600128@030000o`00@R40048Q01/0048Q00<0003o
00128@00@R402P00@R4F07l<1c40048Q1P1o30L00`000000O``707l<1`1<07l<1`L0oooo00<00000
0?ooo`3oool0:03oool003P0oooo00D0gia8000000000000001d0;ooo`0H0?ooo`030000003oool0
oooo00@0ooooC`1o30L00`000000O``707l<1`0307l<1`L0048Q00<0003o00128@00@R406000@R42
0000o`d0048Q5P1o30La00128@H0O``700<0000007l<1`1o30L0C01o30L70?ooo`030000003oool0
oooo02P0oooo000d0?ooo`080?noM0000000000000000018W=l0oooo0;mdB02LgolI0?ooo`030000
003oool0oooo00@0ooooCP1o30L00`000000O``707l<1`0407l<1`P0048Q00<0003o00128@00@R40
5P00@R400`000?l0048Q00128@0=00128AH0O``7<@00@R4607l<1`030000001o30L0O``704`0O``7
1`3oool00`000000oooo0?ooo`0X0?ooo`00<03oool01P2oM0000000000000000000001d0;ooo`<0
oooo00<0oono07A8W03Oool06@3oool2000000D0ooooCP1o30L00`000000O``707l<1`0407l<1`T0
048Q00<0003o00128@00@R405000@R400`000?l0048Q00128@0>00128AH0O``7<@00@R4707l<1`03
0000001o30L0O``704/0O``71P3oool2000002X0oooo000^0?ooo`040?oog`2LB0000000001d_`H0
oooo00<0oooO09a80018W=l06P3oool00`000000oooo0?ooo`040?ooodh0O``700<0000007l<1`1o
30L0101o30L:00128@80003o4`00@R400`000?l0048Q00128@0?00128AH0O``7<@00@R4707l<1`03
0000001o30L0O``704/0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00=P3oool0102oM0000000
00000000M;lK0?ooo`030000003oool0oooo00@0ooooCP1o30L00`000000O``707l<1`0407l<1``0
048Q0P000?l?00128@80003o4P00@R4F07l<1c40048Q1`1o30L00`000000O``707l<1`1;07l<1`L0
oooo00<000000?ooo`3oool0:03oool003<0oooo00H0oooO09a80000000000000000B02LgolL0?oo
o`030000003oool0oooo00@0ooooCP1o30L00`000000O``707l<1`0407l<1`h0048Q0P000?l;0012
8@80003o5000@R4F07l<1c40048Q1`1o30L00`000000O``707l<1`1;07l<1`L0oooo00<000000?oo
o`3oool0:03oool00380oooo00@0oooO09a80000B9`0gooo7`3oool00`000000oooo0?ooo`040?oo
odd0O``700<0000007l<1`1o30L01@1o30L@00128@@0003o1000@R430000oaH0048Q5P1o30La0012
8@L0O``700<0000007l<1`1o30L0B`1o30L70?ooo`030000003oool0oooo02P0oooo000b0?ooo`03
0?oOW01dW=l0oooo0200oooo00<000000?ooo`3oool0103ooom=07l<1`030000001o30L0O``700D0
O``74`00@R450000oaT0048Q5P1o30La00128@P0O``700<0000007l<1`1o30L0BP1o30L70?ooo`03
0000003oool0oooo02P0oooo000]0?ooo`030?oOW0180000000000h0000000<0001d0;ooo`3oool0
503oool2000000D0ooooC@1o30L00`000000O``707l<1`0507l<1c40048Q5P1o30La00128@P0O``7
00<0000007l<1`1o30L0BP1o30L60?ooo`800000:P3oool002d0oooo00<0omnL07BLg`3oool03P3o
ool00`2oM7@0_ooo0?ooo`0D0?ooo`030000003oool0oooo00@0ooooC@1o30L00`000000O``707l<
1`0507l<1c40048Q5P1o30La00128@P0O``700<0000007l<1`1o30L0BP1o30L70?ooo`030000003o
ool0oooo02P0oooo000]0?ooo`030?oOW02L_ol0oooo00h0oooo00<0gibL0;ooo`3oool0503oool0
0`000000oooo0?ooo`040?ooodd0O``700<0000007l<1`1o30L01@1o30La00128AH0O``7<@00@R48
07l<1`030000001o30L0O``704X0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00;@3oool00`3o
gi`0W;oo0?ooo`0>0?ooo`030=nLW02oool0oooo01@0oooo00<000000?ooo`3oool0103ooom<07l<
1`030000001o30L0O``700H0O``7<@00@R4F07l<1c40048Q2@1o30L00`000000O``707l<1`1907l<
1`L0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0103ooom<07l<1`03
0000001o30L0O``700H0O``7<@00@R4F07l<1c40048Q2@1o30L00`000000O``707l<1`1907l<1`L0
oooo00<000000?ooo`3oool0:03oool003<0oooo00<0_gAd0;ooo`3oool0103oool00`2oM7@0_ooo
0?ooo`0H0?ooo`030000003oool0oooo00@0ooooC01o30L00`000000O``707l<1`0607l<1c40048Q
5P1o30La00128@T0O``700<0000007l<1`1o30L0B@1o30L70?ooo`030000003oool0oooo02P0oooo
000c0?ooo`030;md0018W=l0oooo00<0oooo00<0gia80000M02oool06@3oool2000000D0ooooC01o
30L00`000000O``707l<1`0607l<1c40048Q5P1o30La00128@T0O``700<0000007l<1`1o30L0B@1o
30L60?ooo`800000:P3oool003<0oooo00T0_g@00000000007@0_ooo0?oog`2LB000B9cO0=nLW02o
ool06@3oool00`000000oooo0?ooo`040?oood`0O``700<0000007l<1`1o30L01P1o30La00128AH0
O``7<@00@R4907l<1`030000001o30L0O``704T0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00
<`3oool01P2oW9`0gmnL04P000000000001809cOo`h0oooo00D0omnL04P000000000000007Boo`09
0?ooo`030000003oool0oooo00@0ooooC01o30L00`000000O``707l<1`0607l<1c40048Q5P1o30La
00128@X0O``700<0000007l<1`1o30L0B01o30L70?ooo`030000003oool0oooo02P0oooo000c0?oo
o`030;mdM02ogi`0B0000080000000@0001809cOo`3OW9`0_ooo2P3oool01`3ogi`0B00004RLg`3o
ool0oono07@0001d_ol0203oool00`000000oooo0?ooo`040?oood/0O``700<0000007l<1`1o30L0
1`1o30La00128AH0O``7<@00@R4:07l<1`030000001o30L0O``704P0O``71`3oool00`000000oooo
0?ooo`0X0?ooo`00<`3oool00`2oM00004RL0=ooo`020?ooo`040;md00000000001d0;ooo`T0oooo
00<0oooO09a80000M;l00`3oool00`3o_g@004RL0=ooo`070?ooo`030000003oool0oooo00@0oooo
B`1o30L00`000000O``707l<1`0707l<1c40048Q5P1o30La00128@X0O``700<0000007l<1`1o30L0
B01o30L70?ooo`030000003oool0oooo02P0oooo000c0?ooo`030;mdB02Lgol0oooo00<0oooo00<0
okmd0000M02oool02@3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00L0oooo00<0
00000?ooo`3oool0103ooom;07l<1`030000001o30L0O``700L0O``7<@00@R4F07l<1c40048Q2P1o
30L00`000000O``707l<1`1807l<1`L0oooo00<000000?ooo`3oool0:03oool003<0oooo00<0_ibL
0=ooo`3oool0103oool00`3OW9`0_ooo0?ooo`080?ooo`030?oOW0180000M;oo00<0oooo00<0okmd
0000B02Lgol01`3oool00`000000oooo0?ooo`040?oood/0O``700<0000007l<1`1o30L01`1o30La
00128AH0O``7<@00@R4:07l<1`030000001o30L0O``704P0O``71`3oool00`000000oooo0?ooo`0X
0?ooo`00A@3oool00`3ogi`0B00007Boo`030?ooo`030;md000004P0W=oo00L0oooo0`0000040?oo
od/0O``700<0000007l<1`1o30L01`1o30La00128AH0O``7<@00@R4;07l<1`030000001o30L0O``7
04L0O``71@3oool3000002X0oooo00150?ooo`080?oo_`1d0000000004RLg`3oool0okmd00000000
M;l80?ooo`030000003oool0oooo00@0ooooBP1o30L00`000000O``707l<1`0807l<1c40048Q5P1o
30La00128@/0O``700<0000007l<1`1o30L0A`1o30L70?ooo`030000003oool0oooo02P0oooo000]
0?ooo`030?oOW02L_ol0oooo00h0oooo00<0gibL0;ooo`3oool01@3oool00`2oM00004RL0;nLB002
000000030018W03Oool0oooo00L0oooo00<000000?ooo`3oool0103ooom:07l<1`030000001o30L0
O``700P0O``7<@00@R4F07l<1c40048Q2`1o30L00`000000O``707l<1`1707l<1`L0oooo00<00000
0?ooo`3oool0:03oool002d0oooo00<0omnL09boo`3oool03P3oool00`3OW9`0_ooo0?ooo`050?oo
o`030?oOW0180000M;oo00`0oooo00<000000?ooo`3oool0103ooom:07l<1`030000001o30L0O``7
00P0O``7<@00@R4F07l<1c40048Q2`1o30L00`000000O``707l<1`1707l<1`L0oooo00<000000?oo
o`3oool0:03oool002d0oooo00<0omnL07BLg`3oool03P3oool00`2oM7@0_ooo0?ooo`060?ooo`03
0=nLB00004P0W=oo00/0oooo00<000000?ooo`3oool0103ooom:07l<1`030000001o30L0O``700P0
O``7<@00@R4F07l<1c40048Q2`1o30L00`000000O``707l<1`1707l<1`L0oooo00<000000?ooo`3o
ool0:03oool002d0oooo00<0omnL04P0000000003P0000000`0007@0_ooo0?ooo`070?ooo`030?no
M000000007Bo00X0oooo00<000000?ooo`3oool0103ooom:07l<1`030000001o30L0O``700P0O``7
<@00@R4F07l<1c40048Q301o30L00`000000O``707l<1`1607l<1`L0oooo00<000000?ooo`3oool0
:03oool004T0oooo00D0oooO09a800000000001d0;ooo`070?ooo`8000001@3ooom907l<1`030000
001o30L0O``700T0O``7<@00@R4F07l<1c40048Q301o30L00`000000O``707l<1`1607l<1`H0oooo
0P00000Z0?ooo`00E@3oool00`000000oooo0?ooo`040?ooodT0O``700<0000007l<1`1o30L02@1o
30La00128AH0O``7<@00@R4<07l<1`030000001o30L0O``704H0O``71`3oool00`000000oooo0?oo
o`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?ooodT0O``700<0000007l<1`1o30L02@1o30La
00128AH0O``7<@00@R4<07l<1`030000001o30L0O``704H0O``71`3oool00`000000oooo0?ooo`0X
0?ooo`00E@3oool00`000000oooo0?ooo`040?ooodT0O``700<0000007l<1`1o30L02@1o30La0012
8AH0O``7<@00@R4=07l<1`030000001o30L0O``704D0O``71`3oool00`000000oooo0?ooo`0X0?oo
o`00E@3oool00`000000oooo0?ooo`040?ooodT0O``700<0000007l<1`1o30L02@1o30La00128AH0
O``7<@00@R4=07l<1`030000001o30L0O``704D0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00
E@3oool00`000000oooo0?ooo`040?ooodP0O``700<0000007l<1`1o30L02P1o30La00128AH0O``7
<@00@R4=07l<1`030000001o30L0O``704D0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00E@3o
ool2000000D0ooooB01o30L00`000000O``707l<1`0:07l<1c40048Q5P1o30La00128@d0O``700<0
000007l<1`1o30L0A@1o30L60?ooo`800000:P3oool005D0oooo00<000000?ooo`3oool0103ooom8
07l<1`030000001o30L0O``700X0O``7<@00@R4F07l<1c40048Q3@1o30L00`000000O``707l<1`15
07l<1`L0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0103ooom807l<
1`030000001o30L0O``700X0O``7<@00@R4F07l<1c40048Q3P1o30L00`000000O``707l<1`1407l<
1`L0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0103ooom807l<1`03
0000001o30L0O``700X0O``7<@00@R4F07l<1c40048Q3P1o30L00`000000O``707l<1`1407l<1`L0
oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0103ooom807l<1`030000
001o30L0O``700X0O``7<@00@R4F07l<1c40048Q3P1o30L00`000000O``707l<1`1407l<1`L0oooo
00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0103ooom707l<1`030000001o
30L0O``700/0O``7<@00@R4F07l<1c40048Q3P1o30L00`000000O``707l<1`1407l<1`L0oooo00<0
00000?ooo`3oool0:03oool005D0oooo0P0000050?ooodL0O``700<0000007l<1`1o30L02`1o30La
00128AH0O``7<@00@R4>07l<1`030000001o30L0O``704@0O``71P3oool2000002X0oooo001E0?oo
o`030000003oool0oooo00@0ooooA`1o30L00`000000O``707l<1`0;07l<1c40048Q5P1o30La0012
8@l0O``700<0000007l<1`1o30L0@`1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`03
0000003oool0oooo00@0ooooA`1o30L00`000000O``707l<1`0;07l<1c40048Q5P1o30La00128@l0
O``700<0000007l<1`1o30L0@`1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000
003oool0oooo00@0ooooA`1o30L00`000000O``707l<1`0;07l<1c40048Q5P1o30La00128@l0O``7
00<0000007l<1`1o30L0@`1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003o
ool0oooo00@0ooooAP1o30L00`000000O``707l<1`0<07l<1c40048Q5P1o30La00128@l0O``700<0
000007l<1`1o30L0@`1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0
oooo00@0ooooAP1o30L00`000000O``707l<1`0<07l<1c40048Q5P1o30La00128@l0O``700<00000
07l<1`1o30L0@`1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo
00@0ooooAP1o30L00`000000O``707l<1`0<07l<1c40048Q5P1o30La00128@l0O``700<0000007l<
1`1o30L0@`1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`8000001@3ooom607l<1`03
0000001o30L0O``700`0O``7<@00@R4F07l<1c40048Q401o30L00`000000O``707l<1`1207l<1`H0
oooo0P00000Z0?ooo`00B03oool00`3OW4P0B9cO0?ooo`0:0?ooo`030000003oool0oooo00@0oooo
AP1o30L00`000000O``707l<1`0<07l<1c40048Q5P1o30La00128A00O``700<0000007l<1`1o30L0
@P1o30L70?ooo`030000003oool0oooo02P0oooo00180?ooo`030?oOW018B9`0gooo00X0oooo00<0
00000?ooo`3oool0103ooom507l<1`030000001o30L0O``700d0O``7<@00@R4F07l<1c40048Q401o
30L00`000000O``707l<1`1207l<1`L0oooo00<000000?ooo`3oool0:03oool004P0oooo00<0oooO
09a8B02Lgol02P3oool00`000000oooo0?ooo`040?ooodD0O``700<0000007l<1`1o30L03@1o30La
00128AH0O``7<@00@R4@07l<1`030000001o30L0O``70480O``71`3oool00`000000oooo0?ooo`0X
0?ooo`00B@3oool00`3OW4P0B9cO0?ooo`090?ooo`030000003oool0oooo00@0ooooA@1o30L00`00
0000O``707l<1`0=07l<1c40048Q5P1o30La00128A00O``700<0000007l<1`1o30L0@P1o30L70?oo
o`030000003oool0oooo02P0oooo00190?ooo`030?oOW018B9`0gooo00T0oooo00<000000?ooo`3o
ool0103ooom507l<1`030000001o30L0O``700d0O``7<@00@R4F07l<1c40048Q401o30L00`000000
O``707l<1`1207l<1`L0oooo00<000000?ooo`3oool0:03oool004T0oooo00<0oooO09a8B02Lgol0
2@3oool3000000@0ooooA@1o30L00`000000O``707l<1`0=07l<1c40048Q5P1o30La00128A40O``7
00<0000007l<1`1o30L0@@1o30L50?ooo`<00000:P3oool004X0oooo00<0gia8001d_`3oool0203o
ool00`000000oooo0?ooo`040?ooodD0O``700<0000007l<1`1o30L03@1o30La00128AH0O``7<@00
@R4A07l<1`030000001o30L0O``70440O``71`3oool00`000000oooo0?ooo`0X0?ooo`00BP3oool0
0`3ookl0M01d0;ooo`080?ooo`030000003oool0oooo00@0ooooA01o30L00`000000O``707l<1`0>
07l<1c40048Q5P1o30La00128A40O``700<0000007l<1`1o30L0@@1o30L70?ooo`030000003oool0
oooo02P0oooo00150?ooo`030?oOg`3Oool0oooo00<0oooo00<0_g@007Boo`3oool01`3oool00`00
0000oooo0?ooo`040?oood@0O``700<0000007l<1`1o30L03P1o30La00128AH0O``7<@00@R4A07l<
1`030000001o30L0O``70440O``71`3oool00`000000oooo0?ooo`0X0?ooo`00A@3oool00`3ookl0
M4RL0=ooo`030?ooo`030?noM000M;l0oooo00L0oooo00<000000?ooo`3oool0103ooom407l<1`03
0000001o30L0O``700h0O``7<@00@R4F07l<1c40048Q4@1o30L00`000000O``707l<1`1107l<1`L0
oooo00<000000?ooo`3oool0:03oool004H0oooo00<0_g@00000000000000`0000000`0007@0_ooo
0?ooo`060?ooo`030000003oool0oooo00@0ooooA01o30L00`000000O``707l<1`0>07l<1c40048Q
5P1o30La00128A40O``700<0000007l<1`1o30L0@@1o30L70?ooo`030000003oool0oooo02P0oooo
00160?ooo`030?noM0000000000000@0000000<0M;oo0?ooo`3oool01@3oool2000000D0ooooA01o
30L00`000000O``707l<1`0>07l<1c40048Q5P1o30La00128A80O``700<0000007l<1`1o30L0@01o
30L60?ooo`800000:P3oool005D0oooo00<000000?ooo`3oool0103ooom307l<1`030000001o30L0
O``700l0O``7<@00@R4F07l<1c40048Q4P1o30L00`000000O``707l<1`1007l<1`L0oooo00<00000
0?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0103ooom307l<1`030000001o30L0O``7
00l0O``7<@00@R4F07l<1c40048Q4P1o30L00`000000O``707l<1`1007l<1`L0oooo00<000000?oo
o`3oool0:03oool005D0oooo00<000000?ooo`3oool0103ooom307l<1`030000001o30L0O``700l0
O``7<@00@R4F07l<1c40048Q4P1o30L00`000000O``707l<1`1007l<1`L0oooo00<000000?ooo`3o
ool0:03oool005D0oooo00<000000?ooo`3oool0103ooom307l<1`030000001o30L0O``700l0O``7
<@00@R4F07l<1c40048Q4P1o30L00`000000O``707l<1`1007l<1`L0oooo00<000000?ooo`3oool0
:03oool005D0oooo00<000000?ooo`3oool0103ooom307l<1`030000001o30L0O``700l0O``7<@00
@R4F07l<1c40048Q4`1o30L00`000000O``707l<1`0o07l<1`L0oooo00<000000?ooo`3oool0:03o
ool005D0oooo0P0000050?oood<0O``700<0000007l<1`1o30L03`1o30La00128AH0O``7<@00@R4C
07l<1`030000001o30L0O``703l0O``71P3oool2000002X0oooo001E0?ooo`030000003oool0oooo
00@0oooo@P1o30L00`000000O``707l<1`0@07l<1c40048Q5P1o30La00128A<0O``700<0000007l<
1`1o30L0?`1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo00@0
oooo@P1o30L00`000000O``707l<1`0@07l<1c40048Q5P1o30La00128A<0O``700<0000007l<1`1o
30L0?`1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo00@0oooo
@P1o30L00`000000O``707l<1`0@07l<1c40048Q5P1o30La00128A<0O``700<0000007l<1`1o30L0
?`1o30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo00@0oooo@P1o
30L00`000000O``707l<1`0@07l<1c40048Q5P1o30La00128A<0O``700<0000007l<1`1o30L0?`1o
30L70?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo00@0oooo@P1o30L0
0`000000O``707l<1`0@07l<1c40048Q5P1o30La00128A@0O``700<0000007l<1`1o30L0?P1o30L7
0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo00@0oooo@@1o30L00`00
0000O``707l<1`0A07l<1c40048Q5P1o30La00128A@0O``700<0000007l<1`1o30L0?P1o30L70?oo
o`030000003oool0oooo02P0oooo001E0?ooo`8000001@3ooom107l<1`030000001o30L0O``70140
O``7<@00@R4F07l<1c40048Q501o30L00`000000O``707l<1`0n07l<1`H0oooo0P00000Z0?ooo`00
E@3oool00`000000oooo0?ooo`040?oood40O``700<0000007l<1`1o30L04@1o30La00128AH0O``7
<@00@R4D07l<1`030000001o30L0O``703h0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00E@3o
ool00`000000oooo0?ooo`040?oood40O``700<0000007l<1`1o30L04@1o30La00128AH0O``7<@00
@R4D07l<1`030000001o30L0O``703h0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool0
0`000000oooo0?ooo`040?oood40O``700<0000007l<1`1o30L04@1o30La00128AH0O``7<@00@R4D
07l<1`030000001o30L0O``703h0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`00
0000oooo0?ooo`040?oood00O``700<0000007l<1`1o30L04P1o30La00128AH0O``7<@00@R4E07l<
1`030000001o30L0O``703d0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000
oooo0?ooo`040?oood00O``700<0000007l<1`1o30L04P1o30La00128AH0O``7<@00@R4E07l<1`03
0000001o30L0O``703d0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool2000000D0oooo
@01o30L00`000000O``707l<1`0B07l<1c40048Q5P1o30La00128AD0O``700<0000007l<1`1o30L0
?@1o30L60?ooo`800000:P3oool004L0oooo00D0gia8000000000000000004RLg`090?ooo`030000
003oool0oooo00@0oooo@01o30L00`000000O``707l<1`0B07l<1c40048Q5P1o30La00128AD0O``7
00<0000007l<1`1o30L0?@1o30L70?ooo`030000003oool0oooo02P0oooo00160?ooo`070=nLB000
0000M;oo0?ooo`3ooml0W4P004RLg`080?ooo`030000003oool0oooo00@0oooo@01o30L00`000000
O``707l<1`0B07l<1c40048Q5P1o30La00128AD0O``700<0000007l<1`1o30L0?@1o30L70?ooo`03
0000003oool0oooo02P0oooo00150?ooo`030?oog`2LB00007Bo00<0oooo00<0okmd0018W03Oool0
1`3oool00`000000oooo0?ooo`040?oood00O``700<0000007l<1`1o30L04P1o30La00128AH0O``7
<@00@R4E07l<1`030000001o30L0O``703d0O``71`3oool00`000000oooo0?ooo`0X0?ooo`00AP3o
ool00`2oM00007Bo0?ooo`020?ooo`030;md0000B9`0gooo00L0oooo00<000000?ooo`3oool0103o
oolo07l<1`030000001o30L0O``701<0O``7<@00@R4F07l<1c40048Q5P1o30L00`000000O``707l<
1`0l07l<1`L0oooo00<000000?ooo`3oool0:03oool004H0oooo00L0omnL04P0M02oool0oooo0;md
00000000M;oo00P0oooo00<000000?ooo`3oool0103ooolo07l<1`030000001o30L0O``701<0O``7
<@00@R4F07l<1c40048Q5P1o30L00`000000O``707l<1`0l07l<1`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0:03oool004L0oooo00D0oono07A8B0180000000004RLg`090?ooo`L00000
?`1o30L00`000000O``707l<1`0C07l<1c40048Q5P1o30La00128AH0O``700<0000007l<1`1o30L0
?01o30L8000002X0oooo00170?ooo`040?oOW01800000000001d_ol0ooooIP3oool004H0oooo00H0
omnL04P0000007@0_ooo0?noM000M;oo0?ooofD0oooo00160?ooo`030;md000007@0_ooo0080oooo
00<0_g@004RLg`3oool0o`3ooomS0?ooo`00AP3oool00`2oM00004RL0=ooo`020?ooo`030=nLB000
M;l0oooo0?l0ooooH`3oool004H0oooo00L0okmd0000001d_ol0oooo0?oo_`1d0000M;oo0?l0oooo
I03oool004L0oooo00D0okmd000000000000000004RLg`3o0?ooofD0oooo003o0?ooo`@0oooo00H0
oooO09a80000000000000000M02ooonW0?ooo`00o`3oool60?ooo`030;md0000B9`0gooo0:P0oooo
003o0?ooo`H0oooo00<0_g@00018W03Oool0Z03oool00?l0oooo1P3oool00`2oM00004RL0=ooo`2X
0?ooo`00V03oool0203ooml0W4P000000000000004RL09ad0000000007Bo1@3oool0203ooml0W4P0
000000000000001804Q80000B9`0gooo0P3oool01@3OW4P00000000000000000M;oo00<0oooo00/0
oono07@000000000000004RL_`2LB00004RL0=ooo`3oool0oooO09a80002000000050000B02Lgol0
oooo0?ooo`3OW4P00P000000100007@0_ooo0?ooo`2oM002000000030000M02oool0oooo0100oooo
00<0omnL07BLg`3oool00P3oool00`3OW7@0W=oo0?ooo`040?ooo`070;md0000000000000000M02o
ool0oooo0=nLB002000000060000M02LW4P000000000000007@0_ooo1P3oool00`2oM00004Q804P0
00020000000304RLg`3oool0oooo0080oooo00D0gia8000000000000000007Boo`030?ooo`040?oO
W0180000000004RLg`80oooo00L0_g@0000000000000001d0;ooo`3oool0_g@00080000000<0001d
0;oog`2LB0000P0000001`00B9`0gooo0?oOW0180000000004RLg`3OW4P00P0000000`0007@0_ooO
09a80002000000050018W02LM0000000001d_`3OW4P00P0000000`0007@0_ooo0?ooo`1]0?ooo`00
V03oool02@3o_g@0000007Boo`3oool0omnL04P00000M;l0okno0;ooo`030?ooo`0A0?oog`2LB000
000007Boo`3oool0oono07@000000000001d0;ooo`3OW4P0000004RLg`3oool0oooO09a80018W=l0
0P3oool01@3OW4P0001d0;ooo`3oool0gia80080000000T0001d0;ooo`2oM00004RL0=ooo`3oool0
omnL04P0001d_ol00P3oool02`3OW4P0001d0;ooo`3oool0_g@000000000M;l0oooo0?oOW01807@0
_ooo0100oooo00L0_g@00000M02oool0oooo0?oo_`1d000007Bo00@0oooo00l0_g@000000000M;l0
oooo0?oOW01807@0_ooo0?ooo`3OW4P0001d0;ooo`3oool0gia80000M02oool01`3oool01`2oM000
000007Boo`3oool0oono07@00018W=l00P3oool01`3OW4P0000004RLg`3oool0oooO09a80018W=l0
0P3oool0503OW4P0001d0;ooo`3O_kl0gooo0;md0000000007Bo0?ooo`3ogi`0B01d0;ooo`3oool0
gia80018W03Oool0oooo0?oog`2LB00007Bo0P3oool0503OW4P0001d0;ooo`3O_kl0gooo0=nLB000
07@0_ooo0?noM0000000M;oo0?ooo`3ogi`0B000001d_`3o_kl0_ooo0=nLB00007@0_oooK`3oool0
09P0oooo00L0okmd0000B02Lgol0oooo0?oog`2LB00007Bo00D0oooo00<0omnL04P00018W=l00P3o
ool01`3ooml0W4P0001d_`3oool0omnL04P00000M;l00`3oool02`3o_g@0001d0;ooo`3oool0_g@0
0018W03Oool0oooo0?oog`2LB00007Bo0080oooo00<0_g@00018W03Oool00P3oool02P3o_g@0001d
0;ooo`3oool0gia80000M02oool0omnL04P00018W=l30?ooo`030?oOW01dW=l0oooo00h0oooo00T0
omnL04P00018M7@0M9cO0?ooo`3o_g@0001804Q8B02Lgol00P3oool00`3ogi`0B00004RLg`030?oo
o`0:0?oOW01dW=l0oooo0=nLB00007@0_ooo0?ooo`3OW4P0001d0;ooo`L0oooo00<0_g@00018W03O
ool00P3oool01P3o_g@0001d0;ooo`3ogi`0B000001d_`<0oooo00/0okmd0000M02oool0oooo0=nL
B000B9`0gooo0?ooo`3ogi`0B00004RLg`030?ooo`0:0?oOW01dW=l0oooo0=nLB000B9`0gooo0?oo
o`3ooml0W4P0001d_`80oooo00<0gia80018W03Oool00P3oool02P3OW4P0001d0;ooo`3o_g@00018
09cOo`3oool0oooO09a80000M;l20?ooo`030=nLB00007@0_ooo06l0oooo002I0?ooo`060?noM000
000007Bo0?oog`2LB00007Bo1@3oool00`3o_g@0000007Boo`020?ooo`070?oog`2LB00007Bo0?oo
o`3o_g@0000007Boo`030?ooo`0;0?noM0000000M;oo0?ooo`2oM00004RL0=ooo`3oool0oooO09a8
0000M;l00P3oool00`2oM00004RL0=ooo`020?ooo`0:0?oOW0180000M;oo0?ooo`3OW4P0001d0;oo
o`3o_g@0001809cOoa@0oooo00T0gia80000M02oool0_gAd0;oog`2LB00007Bo0?oOW01dW=l00P3o
ool00`3o_g@0001809cOo`060?ooo`070=nLB00007@0_ooo0?ooo`3OW4P0001d0;ooo`070?ooo`03
0;md0000B9`0gooo0080oooo00H0omnL04P0001d_ol0okmd0000001d_ol30?ooo`0;0?noM0000000
M;oo0?ooo`3OW4P004RL0=ooo`3oool0okmd0000B02Lgol01P3oool01`3OW4P004RL0=ooo`3oool0
oooO09a80000M;l00P3oool00`3OW4P004RL0=ooo`020?ooo`0:0=nLB00007@0_ooo0?ooo`3o_g@0
0000001d_`3ooml0W4P0001d_`80oooo00<0gia80000M02oool0K`3oool009/0oooo00@0okmd0000
0000000007Bo1@3oool00`3o_g@0000007Boo`020?ooo`070?oog`2LB00007Bo0?ooo`3o_g@00000
07Boo`030?ooo`0;0?noM0000000M;oo0?ooo`2oM00004RL0=ooo`3oool0oooO09a80000M;l00P3o
ool00`2oM00004RL0=ooo`020?ooo`0:0?oOW0180000M;oo0?ooo`3OW4P0001d0;ooo`3o_g@0001d
0;ooo`H0oooo00<0oooO09a8000000001`0000004P18W=l0oooo0?ooo`3ookl0M00004RLg`3oool0
omnL07AdM01804P0W=oo0?ooo`2oM7@0_ooo0?ooo`3o_g@0001d0;ooo`H0oooo00L0gia80000M02o
ool0oooo0=nLB00007@0_ooo00L0oooo00<0_g@00018W03Oool00P3oool01P3ogi`0B00007Boo`3o
_g@0000007Boo`<0oooo00/0okmd0000001d_ol0oooo0=nLB000B9`0gooo0?ooo`3o_g@0001d0;oo
o`060?ooo`070=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`030=nLB000B9`0gooo0080
oooo00<0gia80000M02oool00`3oool0103o_g@0000000000000M;l20?ooo`030=nLB00007@0_ooo
06l0oooo002H0?ooo`070?oo_`1d0000M;oo0?ooo`3ooml0W4P0001d_`050?ooo`030?oo_`1d0000
B9cO0080oooo00L0oooO09a80000M;l0oooo0?oo_`1d0000M;oo00<0oooo00/0_g@00000B02Lgol0
oooo0;md0000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`030;md0000B9`0gooo0080oooo00T0
okmd0000B02Lgol0oooo0=nLB00007@0_ooo0?oOW01800001@0000000`18W=l0oooo0?ooo`0<0?oo
o`070=nLB00004P0W=oo0?ooo`3ooml0W4P0001d_`020?ooo`050?noM018W=l0oooo0?oOW0180000
1@0000002@18W=l0oooo0=nLB00007@0_ooo0?ooo`3OW4P0001d0;ooo`070?ooo`030;md000007@0
_ooo0080oooo00H0gia80000B02Lgol0oono07@0001d_ol30?ooo`0:0;md000004P0W=oo0?ooo`3O
W4P004RL0=ooo`3oool0omnL04P000D0000000T0B9cO0?ooo`2oM000001d0;ooo`3oool0oooO09a8
0000M;l00P3oool00`3OW4P004RL0=ooo`020?ooo`0:0=nLB00007@0_ooo0?oo_`1d0000M;oo0?oo
o`3ooml0W4P0001d_`80oooo00<0gia80000M02oool0K`3oool009P0oooo00L0oono07@00018W=l0
oooo0?oo_`1d000007Bo00H0oooo00L0okmd0000B02Lgol0oooo0?noM000000007Bo0080oooo00L0
gia80018W03Oool0oooo0?noM000000007Bo0080oooo00L0_g@00018W03Oool0oooo0?oog`2LB000
07Bo0080oooo00l0_g@000000000B9`0gooo0?oo_`1d000004RL0=ooo`3oool0gia80000M02oool0
oooo0;md0018W=l00P3oool00`3OW4P0001809cOo`0=0?ooo`030?oo_`1d000007Bo0080oooo00<0
omnL04P0001d_ol00P3oool01P3ooml0W4Qd0;ooo`3oool0_g@004RLg`80oooo00/0gia80000B02L
gol0oooo0=nLB00007@0_ooo0?ooo`3OW4P0001d0;ooo`050?ooo`0B0?oo_`1d000000000018B018
B9`0gooo0?oOW018000004RL0=ooo`3oool0gia80018W03Oool0oooo0?noM000000007Bo0P3oool0
0`3OW4P004RL0=ooo`020?ooo`0?0;md0018W=l0oooo0?ooo`3OW4P0001809cOg`2LB00000000018
B01804P0W=oo0?oo_`1d0000B9cO0080oooo00l0gia80018W03Oool0oooO09a800000000001d0;oo
o`3ookl0M00004RLg`3oool0oono07@00000M;l00P3oool00`3OW4P0001d0;ooo`1_0?ooo`00V@3o
ool01P3ookl0M000000000000000001d0;ooo`L0oooo00<0okmd0000000000000P0000001@00M;l0
oooo0?ooo`3oool0okmd00<0000000/0B9cO0?ooo`3ookl0M00000000000B9`0gooo0?oOW0180000
0000001d_`020?ooo`070;md0000B9`0gmnL04P00000000004RL0=ooo`020?ooo`030=nLB00007@0
_ooo0080oooo00D0gia8000000000000001809cOo`0=0?ooo`060?oog`2LB000000000000018W=l0
okmd0`0000002018W=l0oooo0;md0000000004RL0=ooo`3oool0gia80P0000002`0004P0W=oo0?oo
o`3oool0gia80000M02oool0oooo0=nLB00007@0_ooo00H0oooo00P0oooO09a80000B9`0gmnL04P0
00000000001d0;ooo`<0oooo00D0okmd000000000000000004RLg`020?ooo`080;md000000000000
00000000B9`0gooo0?ooo`3OW4P20000000=0000B02Lgol0oooo0?ooo`2oM00004RL0=oog`2LB000
00000000M02oool0oooo0;md0003000000080018W03Oool0_g@00000M02oool0oooo0?oo_`1d0002
000000070000M02oool0oooo0?ooo`3OW4P0001d0;ooo`1_0?ooo`00Z@3oool00`3ooml0W4P0001d
_`0D0?ooo`030;md0000B9`0gooo00H0oooo00<0gia80000M02oool0:`3oool01`3OW4P0001d0;oo
o`3oool0gia80000M02oool06@3oool00`2oM00004RL0=ooo`0C0?ooo`030;md0000B9`0gooo00h0
oooo00<0gia80000M02oool0K`3oool00:T0oooo00<0oooO09a80000M;l0503oool00`2oM00004RL
0=ooo`060?ooo`030=nLB00007@0_ooo02/0oooo00L0gia80000M02oool0oooo0=nLB00007@0_ooo
01T0oooo00<0oono07A8W03Oool04`3oool00`3ookl0M4RL0=ooo`0>0?ooo`030=nLB00007@0_ooo
06l0oooo002Y0?ooo`030?oog`2LB00007Bo01@0oooo00<0_g@00018W03Oool01P3oool00`3OW4P0
001d0;ooo`0[0?ooo`070=nLB00007@0_ooo0?ooo`3OW4P0001d0;ooo`0I0?ooo`030?oog`2LM9`0
gooo01<0oooo00<0oooO09adW03Oool03P3oool00`3OW4P0001d0;ooo`1_0?ooo`00Z03oool0103o
okl0M00000000000M;lB0?ooo`050?oo_`1d000000000018W03Oool0103oool01@3ooml0W4P00000
000007@0_ooo02T0oooo00T0oooO09a800000000001d0;oog`2LB00000000000M02oool0=03oool0
0`3OW4P0001809cOo`070?ooo`050?oog`2LB00000000000M02oool0K`3oool00:T0oooo00<0oooO
09a80000M;l04`3oool0103ooml0W4P00018W03Oool60?ooo`030;md000007@0_ooo02/0oooo00L0
_g@00000M02oool0oooo0;md000007@0_ooo03@0oooo00<0gia80000B02Lgol02@3oool00`2oM000
001d0;ooo`1_0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {-8.21226, 0.374123, \
0.0518628, 0.0479517}}]
}, Closed]],

Cell["\<\
The classically allowed and forbidden regions are indicated by green or red \
for the energy shown as a white line.  The corresponding eigenfunction, \
shifted to the energy level and with arbitrary normalization, is shown in \
blue.\
\>", "Caption"],

Cell["\<\
\tA more robust version of the shooting method attempts to match\
\>", "Text"],

Cell[BoxData[{
    \(\[Psi]\_1[x\_m] \[Equal] \[Psi]\_2[x\_m]\), "\n", 
    \(\[Psi]\_1\%\[Prime][x\_m] \[Equal] \[Psi]\_2\%\[Prime][
        x\_m]\)}], "DisplayFormula",
  FontFamily->"Times New Roman"],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`\[Psi]\_1\)]],
  " is obtained by integration from ",
  StyleBox["xmin",
    FontSlant->"Italic"],
  " to ",
  StyleBox["xmatch",
    FontSlant->"Italic"],
  " while ",
  Cell[BoxData[
      \(TraditionalForm\`\[Psi]\_2\)]],
  " is integrated from ",
  StyleBox["xmax",
    FontSlant->"Italic"],
  " to ",
  StyleBox["xmatch",
    FontSlant->"Italic"],
  " where ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          StyleBox["xmin",
            FontSlant->"Italic"], "<", 
          StyleBox["xmatch",
            FontSlant->"Italic"], "<", 
          StyleBox["xmax",
            FontSlant->"Italic"]}], TraditionalForm]]],
  ".  Equivalently, if we match the two values of the ",
  StyleBox["logarithmic derivative ",
    FontSlant->"Italic"],
  Cell[BoxData[
      \(TraditionalForm\`\(\[Psi]\^\[Prime]\)[x]/\[Psi][x]\)]],
  " at an appropriately chosen interior match point, it is not necessary to \
normalize the solution during the search for the eigenenergy."
}], "Text"],

Cell["\<\
\tUnfortunately, it is difficult or impossible to formulate rules that can be \
used to choose the starting conditions and search intervals in a general \
automatic procedure that will work for any potential.  Thus, it is necessary \
to tailor the method of solution to the special characteristics of the \
problem at hand, which requires the kind of insight that only experience can \
provide.  In this notebook we examine in some detail the solutions to a \
particular double-well potential that is sufficiently challenging to \
illustrate some of the issues that must be faced yet simple enough to solve \
without more specialized techniques.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Implementation", "Section"],

Cell[CellGroupData[{

Cell["One-sided shooting method", "Subsection"],

Cell[TextData[{
  "The following function solves the eigenvalue problem by integrating the \
Schr\[ODoubleDot]dinger equation from ",
  StyleBox["xmin",
    FontSlant->"Italic"],
  " to ",
  StyleBox["xmax",
    FontSlant->"Italic"],
  " and finding an energy for which ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Psi]", "[", 
            StyleBox["xmax",
              FontSlant->"Italic"], "]"}], "\[Equal]", "0"}], 
        TraditionalForm]]],
  "."
}], "Text"],

Cell[BoxData[
    \(ShootBSWF1[pot_, m_, emin_, emax_, xmin_, xmax_, xmatch_, 
        opts___Rule] := \[IndentingNewLine]Module[{\[Psi], x, y, trial, bc, 
          evalue, e, norm, optsRoot, optsND, 
          opts2}, \[IndentingNewLine]optsRoot = 
          FilterOptions[FindRoot, opts]; 
        optsND = 
          FilterOptions[NDSolve, opts]; \[IndentingNewLine]trial[
            e_] := \(NDSolve[{\(-\(\(y\^\[DoublePrime]\)[x]\/\(2  m\)\)\) + 
                    pot[x] y[x] \[Equal] e\ y[x], 
                y[\(-xmatch\)] \[Equal] 
                  0, \(y\^\[Prime]\)[\(-xmatch\)] \[Equal] 10\^\(-6\)}, 
              y, {x, xmin, xmax}, 
              optsND]\)\[LeftDoubleBracket]1\[RightDoubleBracket]; \
\[IndentingNewLine]bc[e_?NumericQ] := 
          y[xmatch] /. trial[e]; \[IndentingNewLine]evalue = 
          If[emin \[Equal] emax, emin, 
            e /. With[{opts2 = optsRoot}, 
                FindRoot[bc[e] \[Equal] 0, {e, emin, emax}, 
                  opts2]]]; \[IndentingNewLine]\[Psi] = 
          y /. trial[evalue]; \[IndentingNewLine]norm = 
          1/\@NIntegrate[\[Psi][x]\^2, {x, xmin, xmax}]; \
\[IndentingNewLine]{energy \[Rule] evalue, 
          wavefunction \[Rule] \((norm\ \[Psi][#] &)\), 
          normalization \[Rule] norm}]\)], "Input"],

Cell[BoxData[
    \(\(solutions = 
        wavefunction[x] /. 
          Table[ShootBSWF1[ho, 1, n + 0.4, n + 0.6, \(-5\), 5, 5, 
              MaxIterations \[Rule] 50], {n, 0, 3}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[Evaluate[solutions], {x, \(-5\), 5}];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0952381 0.286484 0.421813 [
[.11905 .27398 -9.15625 -15.6875 ]
[.11905 .27398 9.15625 0 ]
[.30952 .27398 -9.15625 -15.6875 ]
[.30952 .27398 9.15625 0 ]
[.69048 .27398 -5 -15.6875 ]
[.69048 .27398 5 0 ]
[.88095 .27398 -5 -15.6875 ]
[.88095 .27398 5 0 ]
[.4875 .0334 -27.3125 -7.84375 ]
[.4875 .0334 0 7.84375 ]
[.4875 .11776 -27.3125 -7.84375 ]
[.4875 .11776 0 7.84375 ]
[.4875 .20212 -27.3125 -7.84375 ]
[.4875 .20212 0 7.84375 ]
[.4875 .37085 -19 -7.84375 ]
[.4875 .37085 0 7.84375 ]
[.4875 .45521 -19 -7.84375 ]
[.4875 .45521 0 7.84375 ]
[.4875 .53957 -19 -7.84375 ]
[.4875 .53957 0 7.84375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.11905 .28648 m
.11905 .29273 L
s
gsave
.11905 .27398 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.30952 .28648 m
.30952 .29273 L
s
gsave
.30952 .27398 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.69048 .28648 m
.69048 .29273 L
s
gsave
.69048 .27398 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.88095 .28648 m
.88095 .29273 L
s
gsave
.88095 .27398 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.125 Mabswid
.16667 .28648 m
.16667 .29023 L
s
.21429 .28648 m
.21429 .29023 L
s
.2619 .28648 m
.2619 .29023 L
s
.35714 .28648 m
.35714 .29023 L
s
.40476 .28648 m
.40476 .29023 L
s
.45238 .28648 m
.45238 .29023 L
s
.54762 .28648 m
.54762 .29023 L
s
.59524 .28648 m
.59524 .29023 L
s
.64286 .28648 m
.64286 .29023 L
s
.7381 .28648 m
.7381 .29023 L
s
.78571 .28648 m
.78571 .29023 L
s
.83333 .28648 m
.83333 .29023 L
s
.07143 .28648 m
.07143 .29023 L
s
.02381 .28648 m
.02381 .29023 L
s
.92857 .28648 m
.92857 .29023 L
s
.97619 .28648 m
.97619 .29023 L
s
.25 Mabswid
0 .28648 m
1 .28648 L
s
.5 .0334 m
.50625 .0334 L
s
gsave
.4875 .0334 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.6) show
1.000 setlinewidth
grestore
.5 .11776 m
.50625 .11776 L
s
gsave
.4875 .11776 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.4) show
1.000 setlinewidth
grestore
.5 .20212 m
.50625 .20212 L
s
gsave
.4875 .20212 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
.5 .37085 m
.50625 .37085 L
s
gsave
.4875 .37085 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
.5 .45521 m
.50625 .45521 L
s
gsave
.4875 .45521 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.4) show
1.000 setlinewidth
grestore
.5 .53957 m
.50625 .53957 L
s
gsave
.4875 .53957 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.6) show
1.000 setlinewidth
grestore
.125 Mabswid
.5 .05449 m
.50375 .05449 L
s
.5 .07558 m
.50375 .07558 L
s
.5 .09667 m
.50375 .09667 L
s
.5 .13885 m
.50375 .13885 L
s
.5 .15994 m
.50375 .15994 L
s
.5 .18103 m
.50375 .18103 L
s
.5 .22321 m
.50375 .22321 L
s
.5 .2443 m
.50375 .2443 L
s
.5 .26539 m
.50375 .26539 L
s
.5 .30757 m
.50375 .30757 L
s
.5 .32867 m
.50375 .32867 L
s
.5 .34976 m
.50375 .34976 L
s
.5 .39194 m
.50375 .39194 L
s
.5 .41303 m
.50375 .41303 L
s
.5 .43412 m
.50375 .43412 L
s
.5 .4763 m
.50375 .4763 L
s
.5 .49739 m
.50375 .49739 L
s
.5 .51848 m
.50375 .51848 L
s
.5 .01231 m
.50375 .01231 L
s
.5 .56066 m
.50375 .56066 L
s
.5 .58175 m
.50375 .58175 L
s
.5 .60284 m
.50375 .60284 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .28648 m
.03279 .28649 L
.04262 .28649 L
.04758 .28649 L
.05288 .28649 L
.06244 .28649 L
.06788 .28649 L
.07379 .2865 L
.07936 .2865 L
.08451 .28651 L
.09425 .28652 L
.09917 .28653 L
.10458 .28654 L
.10978 .28656 L
.11472 .28657 L
.12399 .28661 L
.12917 .28665 L
.13397 .28668 L
.13921 .28673 L
.14487 .28679 L
.15037 .28686 L
.15543 .28694 L
.16539 .28715 L
.17444 .2874 L
.184 .28777 L
.19347 .28827 L
.20364 .28899 L
.21322 .28989 L
.22197 .29095 L
.23188 .29251 L
.24105 .29435 L
.25112 .29691 L
.26181 .30037 L
.27118 .30416 L
.28129 .30916 L
.29956 .32108 L
.31961 .33918 L
.3407 .36471 L
.37881 .42749 L
.41937 .5079 L
.43811 .54301 L
.45842 .57452 L
.46953 .58751 L
.4744 .59208 L
.47977 .59625 L
.48482 .59932 L
.48955 .60142 L
.49216 .60225 L
.49455 .6028 L
.49587 .60302 L
Mistroke
.49656 .60311 L
.4973 .60319 L
.49801 .60325 L
.49866 .60329 L
.49992 .60332 L
.50112 .6033 L
.50225 .60323 L
.50349 .60311 L
.50481 .60292 L
.5062 .60265 L
.50751 .60234 L
.50997 .60159 L
.51439 .59972 L
.5192 .59694 L
.52446 .59304 L
.53003 .58795 L
.54004 .57653 L
.55955 .54707 L
.57784 .51336 L
.61658 .43627 L
.63626 .40033 L
.65777 .36682 L
.67834 .34137 L
.69745 .32343 L
.708 .31567 L
.71794 .30959 L
.73654 .30098 L
.74629 .29767 L
.7566 .29489 L
.76625 .29285 L
.77506 .29138 L
.78452 .29014 L
.79468 .28913 L
.80408 .28842 L
.813 .28791 L
.82264 .2875 L
.83289 .28719 L
.84164 .28699 L
.8513 .28684 L
.85621 .28677 L
.86138 .28672 L
.87062 .28665 L
.87578 .28662 L
.88057 .28659 L
.8858 .28657 L
.89147 .28655 L
.89693 .28654 L
.90198 .28653 L
.9067 .28652 L
.9119 .28651 L
Mistroke
.92092 .2865 L
.92586 .2865 L
.93048 .2865 L
.93545 .28649 L
.94087 .28649 L
.94565 .28649 L
.95075 .28649 L
.96036 .28649 L
.96927 .28649 L
.97619 .28648 L
Mfstroke
.02381 .28648 m
.03279 .28649 L
.04262 .2865 L
.04758 .28651 L
.05288 .28652 L
.06244 .28654 L
.06788 .28655 L
.07379 .28657 L
.07899 .2866 L
.08451 .28663 L
.08947 .28666 L
.09494 .28671 L
.10458 .28682 L
.10978 .2869 L
.11472 .28699 L
.12399 .28721 L
.13397 .28755 L
.13962 .2878 L
.14487 .28808 L
.15543 .28881 L
.16539 .28977 L
.17433 .29091 L
.184 .29253 L
.1935 .29461 L
.20394 .29759 L
.21483 .30165 L
.225 .3065 L
.23545 .31276 L
.24535 .32006 L
.26391 .33791 L
.28382 .36384 L
.30224 .39422 L
.3415 .47319 L
.3612 .51227 L
.36982 .52713 L
.37926 .5408 L
.38381 .54621 L
.38868 .55099 L
.39137 .55316 L
.39382 .55482 L
.39614 .5561 L
.3986 .55714 L
.39988 .55755 L
.40055 .55773 L
.40128 .5579 L
.40256 .55811 L
.40375 .55822 L
.40447 .55825 L
.40525 .55825 L
.40598 .55821 L
Mistroke
.40667 .55815 L
.40796 .55795 L
.40868 .55779 L
.40933 .55762 L
.41184 .55672 L
.41325 .55604 L
.41455 .55529 L
.41946 .55149 L
.42417 .54633 L
.42922 .53913 L
.43832 .52177 L
.44914 .49396 L
.459 .46231 L
.4989 .29168 L
.53728 .12404 L
.5475 .08913 L
.55709 .06206 L
.56709 .0402 L
.5724 .03134 L
.57811 .02395 L
.58052 .0215 L
.58309 .01931 L
.58531 .01777 L
.58771 .01646 L
.58918 .01584 L
.59054 .01539 L
.59183 .01507 L
.59249 .01494 L
.5932 .01484 L
.59445 .01473 L
.59515 .01472 L
.59578 .01472 L
.59702 .01481 L
.59819 .01497 L
.5994 .01523 L
.60074 .0156 L
.60344 .01667 L
.60595 .01801 L
.6083 .01957 L
.61359 .02408 L
.6193 .03037 L
.63932 .06165 L
.66041 .10378 L
.69849 .18005 L
.71971 .21426 L
.73902 .23826 L
.74864 .24775 L
.75908 .25635 L
.77803 .26803 L
.78748 .27227 L
Mistroke
.79788 .27596 L
.80874 .2789 L
.81892 .28097 L
.82931 .28256 L
.83918 .28367 L
.84804 .28442 L
.85771 .28503 L
.86804 .28549 L
.8774 .28579 L
.88259 .28592 L
.8882 .28603 L
.89346 .28612 L
.89837 .28619 L
.90789 .28628 L
.91293 .28632 L
.91833 .28636 L
.92399 .28639 L
.92935 .28641 L
.93413 .28642 L
.9394 .28644 L
.94867 .28645 L
.95375 .28646 L
.95851 .28647 L
.97619 .28648 L
Mfstroke
.02381 .28648 m
.03279 .28652 L
.04262 .28657 L
.04758 .2866 L
.05288 .28663 L
.06244 .28672 L
.06788 .28678 L
.07379 .28687 L
.07899 .28697 L
.08451 .28709 L
.08947 .28723 L
.09494 .28741 L
.10458 .28784 L
.11472 .28847 L
.12399 .28927 L
.13469 .29055 L
.14487 .29223 L
.15445 .29434 L
.16467 .29732 L
.17432 .30097 L
.18306 .30514 L
.20231 .31788 L
.21315 .32765 L
.22313 .33855 L
.24195 .36452 L
.26261 .40106 L
.28376 .44491 L
.30304 .48589 L
.31261 .50449 L
.32302 .52187 L
.32779 .52846 L
.33285 .53431 L
.33717 .53824 L
.33965 .54001 L
.34195 .54133 L
.3443 .54233 L
.34554 .54271 L
.34623 .54288 L
.34686 .54301 L
.34753 .54311 L
.34826 .54319 L
.34958 .54324 L
.35077 .54317 L
.35204 .54299 L
.35314 .54274 L
.35432 .54237 L
.35645 .54144 L
.35898 .53987 L
.36129 .53801 L
.36641 .53235 L
Mistroke
.37199 .52373 L
.37726 .5132 L
.38215 .50137 L
.40271 .43101 L
.42133 .34455 L
.44224 .2372 L
.4514 .19221 L
.46145 .1477 L
.47052 .11384 L
.48023 .08611 L
.4856 .07511 L
.48814 .07107 L
.49049 .068 L
.49285 .0656 L
.49406 .06462 L
.4954 .06375 L
.49657 .06317 L
.49766 .06279 L
.49881 .06253 L
.50005 .06245 L
.50132 .06255 L
.50246 .06282 L
.50378 .06333 L
.50503 .06401 L
.50725 .06568 L
.50965 .06817 L
.5123 .07173 L
.51511 .07639 L
.52013 .08698 L
.53019 .11626 L
.54125 .15903 L
.58164 .35934 L
.60025 .44304 L
.61007 .47852 L
.62051 .50806 L
.62575 .51949 L
.63153 .52947 L
.63406 .53295 L
.63679 .53613 L
.63935 .53856 L
.64173 .54036 L
.64417 .54174 L
.64546 .54228 L
.64684 .54273 L
.64753 .5429 L
.64829 .54305 L
.649 .54314 L
.64965 .5432 L
.6509 .54323 L
.65222 .54314 L
Mistroke
.65336 .54297 L
.65458 .54268 L
.65574 .54232 L
.6568 .54191 L
.65939 .54061 L
.66184 .53899 L
.66688 .53459 L
.67144 .52944 L
.68166 .51451 L
.70006 .47952 L
.74014 .39575 L
.75853 .36376 L
.77871 .33639 L
.79867 .3171 L
.80957 .30939 L
.81974 .30371 L
.8302 .29915 L
.8401 .29582 L
.849 .29351 L
.85866 .29159 L
.86807 .29018 L
.87845 .28903 L
.88367 .28859 L
.88929 .28819 L
.89457 .28788 L
.89947 .28764 L
.90905 .28728 L
.91411 .28713 L
.91952 .287 L
.92521 .28689 L
.93058 .2868 L
.93537 .28674 L
.94063 .28669 L
.94962 .28661 L
.95913 .28656 L
.97619 .28648 L
Mfstroke
.02381 .28648 m
.03279 .28662 L
.04262 .28678 L
.05288 .28701 L
.06244 .28731 L
.06792 .28753 L
.0731 .28778 L
.0828 .2884 L
.09324 .28934 L
.09914 .29003 L
.10458 .29079 L
.11425 .29252 L
.12331 .29468 L
.13324 .29778 L
.14393 .30221 L
.15404 .30767 L
.16506 .31533 L
.1756 .32457 L
.18515 .33479 L
.20483 .36191 L
.2258 .39981 L
.24667 .44422 L
.26587 .4855 L
.27556 .50395 L
.28461 .51826 L
.28955 .52447 L
.29405 .52896 L
.29662 .53097 L
.29906 .53247 L
.3003 .53308 L
.30162 .53361 L
.30231 .53383 L
.30308 .53404 L
.30372 .53419 L
.30442 .53431 L
.30563 .53443 L
.30677 .53443 L
.30746 .53439 L
.30811 .53431 L
.30933 .53407 L
.31063 .53369 L
.31182 .53321 L
.31451 .5317 L
.31665 .53004 L
.31901 .52776 L
.32376 .5216 L
.32896 .51248 L
.33454 .49986 L
.34462 .46974 L
.35475 .43055 L
Mistroke
.3658 .37928 L
.3859 .27267 L
.3969 .21415 L
.40726 .16414 L
.41733 .1238 L
.42175 .10953 L
.42659 .09665 L
.42889 .09164 L
.43134 .08709 L
.43346 .08384 L
.43576 .08105 L
.43711 .07978 L
.43836 .07884 L
.43955 .07817 L
.44082 .0777 L
.44195 .07748 L
.44301 .07745 L
.44418 .07762 L
.44543 .07803 L
.44671 .0787 L
.4479 .07955 L
.45061 .08228 L
.45324 .08601 L
.45605 .09112 L
.46115 .10333 L
.46578 .11752 L
.47578 .15721 L
.485 .20253 L
.50581 .31983 L
.51565 .37387 L
.52621 .42456 L
.53132 .44516 L
.53617 .46178 L
.54049 .47397 L
.54295 .47975 L
.54527 .48439 L
.54765 .48833 L
.54984 .49121 L
.55218 .49347 L
.5534 .49433 L
.5547 .49498 L
.55574 .49532 L
.55686 .4955 L
.55791 .49549 L
.55887 .49534 L
.55999 .49498 L
.56121 .49437 L
.56237 .49359 L
.56343 .4927 L
.56579 .49011 L
Mistroke
.568 .48694 L
.57303 .47716 L
.57793 .46434 L
.58321 .44725 L
.60436 .35231 L
.62445 .24423 L
.6445 .14567 L
.65323 .11087 L
.66266 .08054 L
.66795 .06713 L
.6728 .05711 L
.67745 .0496 L
.68235 .04382 L
.68485 .04171 L
.68622 .04078 L
.68753 .04005 L
.68874 .0395 L
.69005 .03904 L
.69125 .03874 L
.69235 .03857 L
.69366 .0385 L
.69488 .03855 L
.69559 .03864 L
.69623 .03874 L
.69693 .0389 L
.69768 .0391 L
.69908 .03959 L
.70038 .04017 L
.70333 .04193 L
.70596 .04398 L
.70838 .04625 L
.71386 .05261 L
.7238 .06786 L
.74239 .10475 L
.76277 .1493 L
.78182 .18789 L
.80127 .22036 L
.81972 .24364 L
.82944 .25308 L
.84003 .26142 L
.84958 .26741 L
.85857 .2719 L
.86892 .27593 L
.87829 .27871 L
.88839 .28097 L
.89929 .28273 L
.90883 .28385 L
.91928 .28472 L
.92455 .28505 L
.93019 .28535 L
Mistroke
.94036 .28576 L
.94919 .28601 L
.95878 .28621 L
.97619 .28648 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oe`3oool0
0`000000oooo0?ooo`3F0?ooo`00e`3oool00`000000oooo0?ooo`3F0?ooo`00e`3oool00`000000
oooo0?ooo`3F0?ooo`00e`3oool00`000000oooo0?ooo`3F0?ooo`00e`3oool00`000000oooo0?oo
o`3F0?ooo`00e`3oool300000=H0oooo003G0?ooo`030000003oool0oooo02<0oooo1P00002]0?oo
o`00e`3oool00`000000oooo0?ooo`0R0?ooo`8000001@3oool300000:X0oooo003G0?ooo`030000
003oool0oooo0240oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`2W0?ooo`00^P3o
ool01@3ogi`0B0000000000007@0_ooo00<0oooo00<0okmd0000001d_ol00`3oool01@3ogi`0B000
000000000000M;oo00X0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo0:H0oooo002i0?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`020?oo
o`030?noM0000000M;oo0080oooo00L0omnL04P00018W=l0oooo0?oo_`1d0000M;oo00T0oooo00<0
00000?ooo`3oool07`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0:D0oooo002i
0?ooo`030;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool01@3oool00`3ooml0W4P0001d_`03
0?ooo`030?noM000B9`0gooo00P0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo0:@0oooo002h0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000
M02oool01@3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00P0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0:@0oooo002/0?ooo`03
0?oog`2LB000000000L0000000D0B9cO0?ooo`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo
00D0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`080?ooo`030000003oool0oooo
01`0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`2S0?ooo`00^03oool00`3ogi`0
B00007Boo`030?ooo`030?noM00004P0W=oo00D0oooo00<0omnL04P0001d_ol00`3oool00`2oM000
001809cOo`080?ooo`@000006P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0:80
oooo002h0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01@3oool0203ookl0M000
00000018W=l0oooo0?noM000000007Bo2@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool0XP3oool00;P0oooo00<0omnL04P0001d_ol00`3oool00`3o
_g@0001809cOo`060?ooo`030;md0000B9`0_ia80080000000<004RL0=ooo`3oool0203oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool06P3oool40000
08<0oooo002h0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool01P3oool00`3ogi`0
B00007Boo`0=0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool05P3oool00`000000
oooo0?ooo`0G0?ooo`800000103oool00`000000oooo0?ooo`200?ooo`00^@3oool00`3OW4P004RL
0=ooo`020?ooo`030;md0000M;l0oooo00L0oooo00<0gia80000B02Lgol0303oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool05@3oool3000000H0oooo
00<000000?ooo`3oool0O`3oool00;T0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00T0
oooo00<0okmd00000000M;l02`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`080?ooo`800000O`3oool00;X0oooo
00D0oono07@000000000001d0;ooo`0;0?ooo`050?oog`2LB00000000000M02oool0203oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo07`0oooo003G0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool0N`3oool00=L0oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`1j0?ooo`00e`3oool3000001L0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0N@3oool00=L0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`1i0?ooo`00e`3oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo01l0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo07P0oooo003G0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3o
ool0803oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool0N03oool00=@0oooo1P00000F0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo07L0oooo003D0?ooo`040000003o
ool0oooo00000080oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0R0?ooo`030000
003oool0oooo00P0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`1g0?ooo`00d`3o
ool01@000000oooo0?ooo`3oool0000000<0oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`0R0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool05@3oool00`000000oooo
0?ooo`1f0?ooo`00dP3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool05@3oool00`000000oooo0?ooo`1f0?ooo`00d@3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0T
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`1e
0?ooo`00d@3oool00`000000oooo0?ooo`030?ooo`<00000103oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo02H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo07D0oooo002m0?ooo`@000003`3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0V0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`1d0?ooo`00
^`3oool2000000@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0W0?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`1d0?ooo`00^P3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0:03o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0L`3o
ool00;X0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo02T0oooo00D000000?ooo`3oool0oooo0000000M0?ooo`030000003oool0oooo07<0oooo002i
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0Z
0?ooo`050000003oool0oooo0?ooo`0000007P3oool00`000000oooo0?ooo`1b0?ooo`00^03oool0
0`000000oooo0?ooo`080?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0:`3oool0
10000000oooo0?ooo`00000N0?ooo`030000003oool0oooo0780oooo002h0?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0
00000200oooo00<000000?ooo`3oool0L@3oool00;L0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo02d0oooo0P00000P0?ooo`030000003oool0oooo
0740oooo002g0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`070?ooo`<000001`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo02d0
oooo0P00000Q0?ooo`030000003oool0oooo0700oooo002g0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00P0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool0L03oool00;H0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0203oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo02d0oooo0P00000R0?ooo`030000003oool0oooo06l0oooo002f
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`08
0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0^
0?ooo`030000003oool000000240oooo00<000000?ooo`3oool0K`3oool00;H0oooo00T000000?oo
o`3oool0oooo0?oOW018000000000000M02oool00`3oool01@3o_g@0000007Boo`3oool0000000<0
oooo00@0oooO00000000B9`0gooo2@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0000000R0?ooo`030000003oool0oooo
06h0oooo002e0?ooo`0;0000003oool0oooo0?ooo`3ookl0M00007Boo`3oool0okmd0000M02oool0
0P3oool00`3o_g@0000007Boo`020?ooo`070000003oool0oooo0?oog`00000004RL0=ooo`090?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0]0?oo
o`050000003oool0oooo0?ooo`0000008@3oool00`000000oooo0?ooo`1^0?ooo`00]@3oool01`00
0000oooo0?ooo`3oool0_g@00018W03Oool00P3oool00`2oM00007Bo0?ooo`060?ooo`070000003o
ool0oooo0?oog`00000004RL0=ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`0]0?ooo`050000003oool0oooo0?ooo`0000008P3oool00`00
0000oooo0?ooo`1]0?ooo`00]@3oool01P000000oooo0?ooo`3ookl0M00004RLg`<0oooo00<0gia8
0000M02oool01@3oool00`2oM0000000000000050000000307Boo`3oool0oooo00H0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo02h0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo06d0oooo002/0?oo
o`030?oog`2LB000000000L0000000D0B9cO0?ooo`3ogi`0B00007Boo`030?ooo`030?noM00004P0
W=oo00D0oooo00P0omnL07BLg`000000oooo0000002LB00004RL0=ooo`T0oooo00<000000?ooo`3o
ool02P3oool01@000000oooo0?ooo`3oool0000002l0oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo06`0oooo002d0?ooo`070000003oool0oooo0?oo
o`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00H0oooo00L0gia80000003oool0000009a8
0000B9`0gooo00T0oooo100000090?ooo`050000003oool0oooo0?ooo`000000;`3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0K03oool00;@0oooo
00L000000?ooo`3oool0oooo0?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01P3oool01`3o
oml000000000003ooml0W4P00018W03Oool02@3oool00`000000oooo0?ooo`0:0?ooo`050000003o
ool0oooo0?ooo`000000;`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0200oooo
00<000000?ooo`3oool0J`3oool00;<0oooo00<000000?ooo`3oool00P3oool00`3ogi`0B00007Bo
o`030?ooo`030?noM00004P0W=oo00L0oooo00H0omnL0000003ooml0W4P00018W03Oool90?ooo`03
0000003oool0oooo00/0oooo00<000000?ooo`000000<03oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo01l0oooo00<000000?ooo`3oool0J`3oool00;<0oooo00<000000?ooo`3oool0
0P3oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00P0oooo00D0000007Bog`2LB000
04RL0=ooo`090?ooo`030000003oool0oooo00/0oooo00<000000?ooo`000000;`3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0JP3oool00;<0oooo
00<000000?ooo`3oool00`3oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00P0oooo
00D0000009a8B018B00004RL0=ooo`090?ooo`030000003oool0oooo00/0oooo00<000000?ooo`00
0000;`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3o
ool0JP3oool00;<0oooo00<000000?ooo`3oool00`3oool01`3ookl0M00007Boo`3oool0okmd0000
M02oool0203oool2000000040?oOW018000004RL0=ooo`T0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0000000_0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0803oool0
0`000000oooo0?ooo`1Y0?ooo`00/P3oool00`000000oooo0?ooo`050?ooo`050?oo_`1d00000000
0000M02oool02@3oool01P000000oooo0000003OW4P004RL0=ooo`T0oooo00<000000?ooo`3oool0
303oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
7`3oool00`000000oooo0?ooo`1Y0?ooo`00/P3oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0000000`0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0]0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`1X0?ooo`00/P3oool0
0`000000oooo0?ooo`0B0?ooo`040000003oool0oooo000000`0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`1X0?ooo`00/P3oool00`000000oooo0?ooo`0B0?ooo`050000003oool0
oooo0?ooo`0000002`3oool3000000`0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0I`3oool00;40oooo00<00000
0?ooo`3oool04`3oool01@000000oooo0?ooo`3oool0000000/0oooo00<000000?ooo`3oool02`3o
ool00`000000oooo0000000^0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`1W0?ooo`00/@3oool00`000000oooo0?ooo`0C0?ooo`050000003oool0
oooo0?ooo`0000002`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0000002d0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo06H0oooo002a
0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`08
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`000000;@3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0IP3oool00;40oooo00<000000?oo
o`3oool04P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00P0oooo00<000000?oo
o`3oool02`3oool010000000oooo0?ooo`00000/0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool0803oool00`000000oooo0?ooo`1U0?ooo`00/03oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0;0?oo
o`040000003oool0oooo000002`0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0O
0?ooo`030000003oool0oooo06D0oooo002`0?ooo`030000003oool0oooo01<0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00X0oooo00D000000?oo
o`3oool0oooo0000000[0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool07`3oool0
0`000000oooo0?ooo`1U0?ooo`00/03oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0:0?ooo`050000003oool0oooo
0?ooo`000000:`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01l0oooo00<00000
0?ooo`3oool0I03oool00;00oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo00L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo02P0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0O0?oo
o`030000003oool0oooo06@0oooo002_0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`070?ooo`<000002P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo02P0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0O0?ooo`03
0000003oool0oooo06<0oooo002_0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
7`3oool00`000000oooo0?ooo`1S0?ooo`00[`3oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo01l0oooo00<000000?ooo`3oool0HP3oool00:l0oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool04`3oool0
0`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0680oooo002_0?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`1Q0?ooo`00[P3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool09P3oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0H@3oool00:h0oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo02H0oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0600oooo
002^0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool02@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`0V0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`1P0?ooo`00[P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool0903oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01l0oooo00<00000
0?ooo`3oool0G`3oool00:h0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo02@0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0O0?oo
o`030000003oool0oooo05l0oooo002^0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`050?ooo`<000001`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo02@0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0O0?ooo`03
0000003oool0oooo05h0oooo002]0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
7`3oool00`000000oooo0?ooo`1N0?ooo`00[@3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01l0oooo00<000000?ooo`3oool0G@3oool00:d0oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`080?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool06`3oool0
0`000000oooo0?ooo`0O0?ooo`030000003oool0oooo05`0oooo002]0?ooo`030000003oool0oooo
00X0oooo00D0omnL04P000000000001d0;ooo`030?ooo`80000000<0M;oo0?ooo`2oM0001P000000
1018W=l0oooo0?ooo`0000060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`1L0?ooo`00[@3oool00`000000oooo0?ooo`090?ooo`070?oo_`1d0000
M;oo0?ooo`3o_g@0001d0;ooo`020?ooo`80000000@0M;oo0?ooo`3oool0gia81@0000001000B9`0
gooo0?ooo`0000060?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`0R0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool07`3oool00`00
0000oooo0?ooo`1K0?ooo`00[03oool00`000000oooo0?ooo`0:0?ooo`030;md0000B9`0gooo0080
oooo00D0_g@0001d_`3oool0oooo000000050?ooo`030=nLB018W=l0oooo0080oooo00@0giad09cO
o`3oool000001P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool08P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01l0oooo00<00000
0?ooo`3oool0FP3oool00:`0oooo00<000000?ooo`3oool02@3oool00`3ookl0M00004RLg`030?oo
o`040=nLB00007@0_ooo000000L0oooo00<0gia807Boo`3oool01@3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0R
0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`1J
0?ooo`00[03oool00`000000W4P0000000070000000504RLg`3oool0omnL04P0001d_ol00`3oool0
103o_g@0001809cOo`0000080?ooo`030;mdB02Lgol0oooo00@0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0F@3o
ool00:`0oooo00<000000?ooo`3oool02@3oool00`3ogi`0B00007Boo`030?ooo`040?noM00004P0
W=oo000000P0oooo00<0oooO09a8M02oool0103oool00`000000oooo0?ooo`030?ooo`@00000103o
ool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0803o
ool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo05T0oooo002/0?ooo`030000003oool0
oooo00T0oooo00<0omnL04P0001d_ol00`3oool0103o_g@0001809cOo`0000090?ooo`030?noM000
M;l0oooo00<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0P0?ooo`030000003o
ool0oooo01l0oooo00<000000?ooo`3oool0F03oool00:`0oooo00<000000?ooo`3oool02@3oool0
0`3ogi`0B00007Boo`030?ooo`040?noM00004P0W=oo000000T0oooo00<0oono07@0B02Lgol00`3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool02`3o
ool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`1G0?ooo`00Z`3oool00`000000oooo0?ooo`0:0?ooo`030?oo_`1d0000
B9cO00<0oooo00<0gia80000M00000002P3oool00`3ookl0M00004RLg`040?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo0200oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0
oooo05L0oooo002[0?ooo`030000003oool0oooo00/0oooo00<0gia80018W03Oool00P3oool00`2o
M00007Bo000000050?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l0103oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`0O0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool07`3oool00`00
0000oooo0?ooo`1F0?ooo`00Z`3oool00`000000oooo0?ooo`0;0?ooo`080?oo_`1d0000M;oo0?oo
o`3o_g@0001d0;ooo`0000050?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02oool40?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo01h0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0O0?oo
o`030000003oool0oooo05D0oooo002[0?ooo`030000003oool0oooo00`0oooo00L0oono07@00000
0000001d0;ooo`3oool0000000H0oooo00H0oooO09a80000000000000000M02oool50?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0N0?ooo`030000
003oool0oooo05D0oooo002[0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool07P3o
ool00`000000oooo0?ooo`1D0?ooo`00ZP3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0
oooo01l0oooo00<000000?ooo`3oool0D`3oool00:X0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`0A0?ooo`040000003oool0oooo0?ooo`<000000`3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`0O
0?ooo`030000003oool0oooo0580oooo002Z0?ooo`030000003oool0oooo0180oooo00<000000?oo
o`3oool04@3oool01@000000oooo0?ooo`3oool0000000D0oooo00<000000?ooo`3oool03`3oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool07P3oool0
0`000000oooo0?ooo`1B0?ooo`00ZP3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
0140oooo00D000000?ooo`3oool0oooo000000050?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool0703oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo01h0oooo00<00000
0?ooo`3oool0D@3oool00:X0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0A0?oo
o`050000003oool0oooo0?ooo`0000001@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0
oooo01`0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0
oooo0500oooo002Y0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04P3oool01@00
0000oooo0?ooo`3oool0000000D0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0L
0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`1?
0?ooo`00>@3oool0103ooml0W4P00018W03Ooom:0?ooo`030;md00000000000000@0000000<0B9cO
0?ooo`3oool0603oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01<0oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool04`3oool00`2oM0000000000000040000000304RLg`3oool0oooo00l0oooo00<0
00000?ooo`3oool07`3oool00`000000oooo0?ooo`0H0?ooo`040?oog`2LB00004RL0=oooc80oooo
000i0?ooo`040?oog`2LB00004RL0=oood/0oooo00<0gia80000000000000`0000000`00B9`0gooo
0?ooo`0H0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04`3oool010000000oooo
0?ooo`0000040?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool06`3oool00`000000
oooo0?ooo`0D0?ooo`030=nLB0000000000000<0000000<004RL0=ooo`3oool0403oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo01P0oooo00@0oooO09a80000B9`0gooo<P3oool003T0
oooo00@0oooO09a80000B9`0goooC03oool00`3OW4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo
01P0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0C0?ooo`040000003oool0oooo
000000@0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01H0oooo00<0gia804RLg`3oool00P3oool00`3OW7@0W=oo0?ooo`0A0?ooo`030000003oool0
oooo01h0oooo00<000000?ooo`3oool05`3oool0103ooml0W4P00018W03Ooolb0?ooo`00=@3oool0
0`2oM0000000000000050000000307Boo`3oool0oooo04X0oooo00<0gia807Boo`3oool0703oool0
0`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01<0oooo00@000000?ooo`3oool00000103o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool05`3o
ool00`3OW4P0M;oo0?ooo`0F0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool04P3o
ool00`2oM0000000000000050000000307Boo`3oool0oooo02l0oooo000X0?ooo`030?oog`2LB000
000000L0000000D0B9cO0?ooo`3oool0omnL07BLg`020?ooo`040?oog`2LB00004RL0=ooocd0oooo
00<0oooO09a8000000001`0000000`18W=l0oooo0?ooo`040?ooo`030;mdB02Lgol0oooo01X0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00<00000
00<0oooo0000003oool04`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01P0oooo
00<0_gA809cOo`3oool05P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0140oooo
00P0omnL07BLg`3oool0oooo0?oog`2LB00004RL0=oooc80oooo000f0?ooo`070=nLB01d_ol0oooo
0?oog`2LB00004RL0=ooo`1>0?ooo`030?oog`2LB7@0_ooo01X0oooo00<000000?ooo`3oool04P3o
ool00`000000oooo0?ooo`0D0?ooo`040000003oool0oooo000000<0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<0oooO09a8M02oool0
5`3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0140oooo00L0gia807Boo`3oool0
oooO09a80000B9`0gooo0380oooo000f0?ooo`070?oog`2LB7@0_ooo0?oog`2LB00004RL0=ooo`1?
0?ooo`030?noM000M;l0oooo01T0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0E
0?ooo`030000003oool0000000<0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0I
0?ooo`030000003oool0oooo01T0oooo00<0okmd001d_`3oool05`3oool00`000000oooo0?ooo`0N
0?ooo`030000003oool0oooo0100oooo00L0oooO09a8M02oool0oooO09a80000B9`0gooo0380oooo
000g0?ooo`060?oOW018M;l0oooO09a80000B9`0goooC`3oool00`3ookl0M01809cOo`0I0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool05@3oool00`000000oooo000000030?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0J0?ooo`03
0?oo_`1d04P0W=oo01P0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0@0?ooo`06
0?oOW018M;l0oooO09a80000B9`0gooo<P3oool003P0oooo00D0gia807Bog`2LB00004RL0=ooo`1?
0?ooo`030?oo_`1d0000B9cO01T0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0E
0?ooo`030000003oool0000000<0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo01X0oooo00<0oono07@00018W=l06@3oool00`000000oooo0?ooo`0N
0?ooo`030000003oool0oooo0100oooo00D0gia807Bog`2LB00004RL0=ooo`0b0?ooo`00>03oool0
1@3ooml0W4Q804Q80000B9`0gooo04X0oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0B00004RL
g`0H0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool05P3oool00`000000oooo0000
00020?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0E0?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l06P3oool00`000000oooo0?oo
o`0N0?ooo`030000003oool0oooo00l0oooo00D0oooO09a8B018B00004RL0=ooo`0b0?ooo`00>@3o
ool0103ogi`0B0000018W03Ooom:0?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02ooolH
0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool05P3oool00`000000oooo00000002
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0E
0?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02ooolK0?ooo`8000007`3oool00`000000
oooo0?ooo`0?0?ooo`040?oOW018000004RL0=oooc80oooo000j0?ooo`030=nLB000B9`0gooo04/0
oooo00H0oooO09a80000000000000000M02ooolI0?ooo`030000003oool0oooo0180oooo00<00000
0?ooo`3oool05P3oool00`000000oooo000000020?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`0F0?ooo`060?oog`2LB00000000000000007@0_ooo
7P3oool2000001h0oooo00<000000?ooo`3oool03`3oool00`3OW4P004RL0=ooo`0b0?ooo`00Y`3o
ool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01L0oooo0P0000020?ooo`030000003o
ool0oooo01D0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0m0?ooo`8000007@3o
ool00`000000oooo0?ooo`130?ooo`00Y`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo01L0oooo1000000H0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0?`3oool2
000001`0oooo0P0000130?ooo`00YP3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
01P0oooo0P0000000`3oool000000?ooo`0G0?ooo`030000003oool0oooo01H0oooo00<000000?oo
o`3oool0@@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0400oooo002V0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool0603oool2000000030?ooo`000000oooo01L0
oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`120?ooo`800000703oool200000400
oooo002V0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0603oool2000000030?oo
o`000000oooo01P0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`140?ooo`<00000
6`3oool4000003`0oooo002V0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0000000I0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0B03o
ool4000001/0oooo0`00000i0?ooo`00YP3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo01T0oooo0P00000J0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0C03oool2
000001`0oooo0P00000g0?ooo`00Y@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
01X0oooo0P00000J0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0CP3oool80000
01H0oooo2000000_0?ooo`00Y@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01X0
oooo0P00000J0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0EP3oool4000001X0
oooo1000000[0?ooo`00Y@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01X0oooo
0P00000K0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0FP3oool@000000h0oooo
3@00000N0?ooo`00o`00002`00000040oooo000:0?ooo`030000003oool0oooo0180oooo00<00000
0?ooo`3oool02P0000030?ooo`T000001@3oool<000000P0oooo3@00000C0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo0140oooo0P00000B0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100oooo3@0000090?ooo`030000003oool0
oooo00/0oooo3000000?0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool02@3oool0
02`0oooo200000040?ooo`P000002@3oool8000000d0oooo2000000O0?ooo`030000003oool0oooo
01`0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0K0?ooo`8000006`3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0803oool40000
0200oooo100000070?ooo`030000003oool0oooo0380oooo000c0?ooo`<000002P3oool4000000d0
oooo1000000A0?ooo`@000006`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0180
oooo00<000000?ooo`3oool06`3oool2000001/0oooo00<000000?ooo`3oool04P3oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo01T0oooo1`00000P0?ooo`@000002`3oool00`000000
oooo0?ooo`0b0?ooo`00=P3oool2000000`0oooo0P00000?0?ooo`8000004`3oool4000003H0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0K0?ooo`800000703oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo03L0oooo0P00000S0?ooo`@00000A03oool003P0oooo0`00
000;0?ooo`<000003P3oool3000001@0oooo0P00000d0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool06P3oool3000001`0oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`0e0?ooo`<000008P3oool3000004P0oooo000k0?ooo`800000303oool3000000h0oooo1000000B
0?ooo`<00000<@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01X0oooo0P000000
0`3oool000000?ooo`0J0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0<P3oool3
000002<0oooo0P00001;0?ooo`00?@3oool2000000d0oooo0P00000@0?ooo`8000004`3oool30000
02d0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0J0?ooo`80000000<0oooo0000
003oool06P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0300oooo0P00000T0?oo
o`800000C@3oool003l0oooo0P00000=0?ooo`800000403oool2000001@0oooo00<000000?ooo`3o
ool0:P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01/0oooo0P0000000`3oool0
00000?ooo`0J0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0;P3oool2000002@0
oooo0P00001?0?ooo`00@@3oool2000000d0oooo0P00000@0?ooo`030000003oool0oooo0180oooo
0P00000Z0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool06P3oool00`000000oooo
00000002000001`0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0/0?ooo`800000
903oool200000540oooo00130?ooo`8000003@3oool00`000000oooo0?ooo`0>0?ooo`800000503o
ool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool06P3o
ool01@000000oooo0000003oool0000001`0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`0Y0?ooo`<000009@3oool00`000000oooo0?ooo`1A0?ooo`00A@3oool2000000`0oooo0P00
000@0?ooo`030000003oool0oooo0180oooo0P00000W0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool06P3oool01@000000oooo0000003oool0000001`0oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`0X0?ooo`8000009P3oool2000005@0oooo00170?ooo`030000003o
ool0oooo00/0oooo0P00000?0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool08`3o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`0000000P3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool09P3o
ool2000002H0oooo0P00001F0?ooo`00B03oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo00d0oooo0P00000D0?ooo`8000008`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo01X0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0U0?ooo`030000003o
ool0oooo05H0oooo00190?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0C0?ooo`8000008@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo01X0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0U0?ooo`030000003o
ool0oooo05L0oooo001:0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`0J0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool09@3o
ool00`000000oooo0?ooo`1H0?ooo`00B`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0M0?ooo`030000003oool0
oooo0180oooo00<000000?ooo`3oool06P3oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0S0?ooo`030000003o
ool0oooo02D0oooo00<000000?ooo`3oool0F@3oool004`0oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool06`3oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01T0oooo00D000000?ooo`3oool0oooo0000
00030?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0R0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0FP3oool004d0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool06P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01X0oooo00@00000
0?ooo`3oool0oooo0`0000000`3oool000000?ooo`0J0?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool08@3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo05/0oooo001>
0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0J
0?ooo`050000003oool0oooo0?ooo`0000000`3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0U0?ooo`030000003o
ool0oooo05`0oooo001?0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`0J0?ooo`050000003oool0oooo0?ooo`0000000`3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool07P3oool2000002L0oooo00<0
00000?ooo`3oool0G@3oool00500oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0B
0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool00P3oool01@000000oooo0?ooo`3o
ool0000001`0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0N0?ooo`030000003o
ool0oooo02H0oooo00<000000?ooo`3oool0GP3oool00540oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool00P3oool0
1@000000oooo0?ooo`3oool0000001`0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0G`3oool00540oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo02H0oooo00<00000
0?ooo`3oool0H03oool00580oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0?0?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0B0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo02L0oooo00<000000?ooo`3oool0H03oool005<0oooo00<000000?ooo`3o
ool0303oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3o
ool04P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3o
ool0H@3oool005@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0>0?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0B0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0K0?ooo`030000
003oool0oooo02H0oooo00<000000?ooo`3oool0HP3oool005@0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool00`3o
ool300000080oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`1S0?ooo`00E@3oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`080?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool09`3oool0
0`000000oooo0?ooo`1S0?ooo`00EP3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
00l0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`1T0?ooo`00E`3oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0=0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool06@3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool09`3oool00`000000
oooo0?ooo`1U0?ooo`00E`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0180oooo
00<000000?ooo`3oool00`3oool01@3ogi`0B0000000000007@0_ooo00<0oooo00D0okmd0000001d
_ol0oooo0;md00060000000404RLg`3oool0oooo000000H0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`0W0?ooo`030000003oool0oooo06H0oooo001H0?ooo`030000003oool0oooo
00`0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
00`0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`030?ooo`070?oo_`1d0000M;oo
0?ooo`3o_g@0001d0;ooo`020?ooo`030?noM0000000M;oo0080oooo00<0gia80000000000000`00
00001000B9`0gooo0?ooo`0000060?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
9`3oool00`000000oooo0?ooo`1V0?ooo`00F@3oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo0180oooo00<000000?ooo`3oool00`3oool00`2oM00004RL0=ooo`020?ooo`030;md0000
M;l0oooo00L0oooo00<0gia804RLg`3oool00P3oool00`3OW7@0W=oo000000070?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`1W0?ooo`00F@3oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool0
0`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00P0oooo00<0gia807Boo`3oool00`3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool06`3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool09`3oool0
0`000000oooo0?ooo`1X0?ooo`00FP3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
00l0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool00P3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo
00T0oooo00<0_gA809cOo`3oool00P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`1X0?ooo`00FP3oool00`000000
oooo0?ooo`0=0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`3ogi`0
B00007Boo`030?ooo`030?noM00004P0W=oo00T0oooo00<0oooO09a8M02oool00P3oool00`000000
oooo0?ooo`050?ooo`@000000`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo06T0
oooo001K0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`030?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol02P3oool0103o_g@007Bo
0?ooo`0000080?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0:03oool00`000000
oooo0?ooo`1Y0?ooo`00F`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00l0oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0180oooo
00<000000?ooo`3oool00`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00X0oooo
00@0oono07@0B02Lgol00000203oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01/0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo02L0
oooo00<000000?ooo`3oool0JP3oool005`0oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`0B0?ooo`030000003oool0oooo00<0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d
0;ooo`0:0?ooo`040?oo_`1d0000B9cO000000P0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0K0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool04`3oool00`000000
oooo0?ooo`0X0?ooo`030000003oool0oooo06X0oooo001L0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`040?ooo`030=nLB000B9`0gooo0080oooo
00<0_g@0001d_`3oool01@3oool00`3o_i`0W=oo0?ooo`020?ooo`040?oOW0180000B9cO000000P0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo06/0
oooo001M0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`040?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`060?ooo`090?oog`2LB4P0W=oo
0?ooo`3ookl0M0000000M02oool0000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04`3oool00`000000oooo
0?ooo`0W0?ooo`030000003oool0oooo06`0oooo001M0?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`060?ooo`050?oo_`1d000000000000M02oool0
203oool01`3ooml0W4P0000000000000001d0;ooo`0000002@3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0B0?oo
o`030000003oool0oooo02P0oooo00<000000?ooo`3oool0K03oool005h0oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo01D0oooo00D000000?ooo`3o
ool0oooo0000000D0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo01/0oooo00D000000?ooo`3oool0oooo0000000D
0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0K@3oool005h0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01@0oooo00D000000?oo
o`3oool0oooo0000000D0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo01/0oooo00@000000?ooo`3oool00000503o
ool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo06d0oooo001O0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0E0?ooo`040000003oool0
oooo000001@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`070?ooo`<000001@3o
ool00`000000oooo0?ooo`0K0?ooo`040000003oool0oooo000001<0oooo00<000000?ooo`3oool0
:03oool00`000000oooo0?ooo`1^0?ooo`00G`3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool05@3oool00`000000oooo0000000C0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo01X0oooo00@000000?ooo`3oool000004`3oool00`000000oooo0?ooo`0X0?ooo`030000
003oool0oooo06h0oooo001P0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`0E0?ooo`030000003oool0000001<0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0000000B0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0K`3o
ool00600oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo01D0oooo0P00000C0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0203oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo01/0oooo00<000000?ooo`0000004P3oool0
0`000000oooo0?ooo`0X0?ooo`030000003oool0oooo06l0oooo001Q0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool06`3oool200000180oooo00<000000?ooo`3oool0:03oool00`00
0000oooo0?ooo`1`0?ooo`00H@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0503oool2000001<0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool06`3oool200000140
oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`1`0?ooo`00HP3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0503oool00`000000oooo
0000000A0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo01X0oooo0P00000A0?ooo`030000003oool0oooo02P0oooo
00<000000?ooo`3oool0L@3oool00680oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`0000004@3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`0K0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?oo
o`1a0?ooo`00H`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool04`3oool010000000oooo0?ooo`00000@0?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`3oool02@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo0780oooo001S
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo00X0oooo0`0000070?ooo`030000003oool0oooo01X0oooo0P00000?
0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0LP3oool006@0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0140oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool02P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01X0oooo0P00000>0?oo
o`030000003oool0oooo02T0oooo00<000000?ooo`3oool0L`3oool006@0oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01X0oooo00<000000?ooo`00
00003@3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo07<0oooo001U0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0000000`0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`1d0?ooo`00I@3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0603o
ool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0603o
ool010000000oooo0?ooo`00000<0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0
M03oool006H0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo01P0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0Y0?ooo`030000
003oool0oooo07D0oooo001V0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`0H0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0
:P3oool00`000000oooo0?ooo`1e0?ooo`00I`3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool0603oool01@000000oooo0?ooo`3oool0000000T0oooo
00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`1f0?ooo`00I`3oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool05`3oool01@000000oooo0?oo
o`3oool0000000P0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`1f0?ooo`00J03o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0603o
ool00`000000oooo0?ooo`0<0?ooo`<000002@3oool00`000000oooo0?ooo`0F0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0Y0?ooo`030000003o
ool0oooo07L0oooo001X0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo07L0oooo001Y0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
00`0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo
07P0oooo001Y0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0E0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`0Z0?ooo`030000003oool0oooo07P0oooo001Z0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0<0?ooo`050?oOW018000000000000M02o
ool00`3oool00`3o_g@0000007Boo`020?ooo`070000003oool0oooo0?oog`2LB00004RL0=ooo`09
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`1h
0?ooo`00JP3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool02`3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool00P3oool00`3o_g@0000007Bo
o`020?ooo`070000003oool0oooo0?oog`2LB00004RL0=ooo`090?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`1i0?ooo`00JP3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`0=0?ooo`050000003oool0oooo0?ooo`0000003@3oool00`2oM00004RL0=ooo`020?ooo`03
0;md0000M;l0oooo00H0oooo00L000000?ooo`3oool0oooO09a80000B9`0gooo00T0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00D0oooo00@00000
0?ooo`3oool00000;@3oool00`000000oooo0?ooo`1i0?ooo`00J`3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0=
0?ooo`050000003oool0oooo0?ooo`000000303oool00`3ookl0M00004RLg`030?ooo`030=nLB000
07@0_ooo00D0oooo200000000`1d_ol0oooo0?ooo`060?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool04`3oool00`000000oooo0?ooo`050?ooo`040000003oool0oooo000002`0oooo
00<000000?ooo`3oool0NP3oool006/0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool03`3oool00`000000oooo0000
000=0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01@3oool020000000M9cO0?oo
o`3oool0oooO09a80000B9`0gooo2@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool01P3oool00`000000oooo0000000]0?ooo`030000003oool0oooo
07X0oooo001/0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo00l0oooo00<000000?ooo`0000003@3oool00`3ogi`0
B00007Boo`030?ooo`030?noM00004P0W=oo00D0oooo00P000000=nLB01d_ol0oooo0?oog`2LB000
04RL0=ooo`T0oooo1000000;0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0000000/0?ooo`030000003oool0oooo07/0oooo001/0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo0100oooo0P00000=0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01@3oool0
20000000oooO09a8M02oool0oooO09a80000B9`0gooo2@3oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo0140oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0[0?ooo`03
0000003oool0oooo07/0oooo001]0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
303oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00@0oooo00T000000?ooo`3oool0
omnL04Qd_`3ooml0W4P00018W03Oool02@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0
oooo07`0oooo001]0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo0140oooo0P00000=0?ooo`030?oo_`1d0000B9cO
00<0oooo00<0gia80000M02oool0103oool02@000000oooo0?ooo`3oool0gia807Bog`2LB00004RL
0=ooo`090?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`070?ooo`800000;03oool00`000000oooo0?ooo`1l0?ooo`00KP3oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo0100oooo00D000000?ooo`3oool0oooo0000000C0?ooo`800000
3P3oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00@0oooo00T000000?ooo`3oool0
oooo0?oog`2LB4P0B4P00018W03Oool02@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo00l0oooo00<000000?ooo`3oool01`3oool2000002/0oooo00<000000?ooo`3oool0O@3oool0
06h0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0@0?ooo`050000003oool0oooo
0?ooo`0000004P3oool010000000oooo0?ooo`00000=0?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0
001d0;ooo`050?ooo`030000003oool0oooo0080oooo00@0omnL04P00000B9`0gooo2@3oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool01`3oool01000
0000oooo0?ooo`00000Z0?ooo`030000003oool0oooo07d0oooo001_0?ooo`030000003oool0oooo
00`0oooo00<000000?ooo`3oool04@3oool00`000000oooo0000000C0?ooo`040000003oool0oooo
000000h0oooo00D0oono07@000000000001d0;ooo`050?ooo`030000003oool0oooo00@0oooo00<0
gia80018W03Oool02@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool01`3oool010000000oooo0?ooo`00000Y0?ooo`030000003oool0oooo07h0oooo
001_0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0403oool00`000000oooo0000
000C0?ooo`050000003oool0oooo0?ooo`0000005`3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`070?ooo`050000
003oool0oooo0?ooo`000000:@3oool00`000000oooo0?ooo`1n0?ooo`00L03oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo0140oooo0P00000C0?ooo`050000003oool0oooo0?ooo`00
00005`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool0303oool00`000000oooo0?ooo`070?ooo`050000003oool0oooo0?ooo`000000:03oool00`00
0000oooo0?ooo`1o0?ooo`00L03oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02D0
oooo00<000000?ooo`3oool0O`3oool00740oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`0@0?ooo`8000004P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool04@3oool3000000h0oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0903oool00`000000oooo
0?ooo`200?ooo`00L@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00l0oooo0P00
000B0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0903oool00`000000oooo0?oo
o`200?ooo`00LP3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00h0oooo00@00000
0?ooo`3oool00000403oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0280oooo00<0
00000?ooo`3oool0P@3oool00780oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=
0?ooo`040000003oool0oooo00000100oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0R0?ooo`030000003oool0oooo0840oooo001b0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool03@3oool01@000000oooo0?ooo`3oool0000000l0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0880oooo001c0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool03@3oool01@000000oooo0?ooo`3oool0000000h0oooo00<0
00000?ooo`3oool0203oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool04P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool0203oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0880oooo001c
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0O
0?ooo`030000003oool0oooo08<0oooo001d0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool02`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01<0oooo00<000000?oo
o`3oool04`3oool200000080oooo0P0000090?ooo`030000003oool0oooo00X0oooo00<000000?oo
o`3oool07P3oool00`000000oooo0?ooo`230?ooo`00M03oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0oooo01D0oooo0`00000:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool07@3oool00`000000oooo0?ooo`240?ooo`00M@3oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0D0?ooo`<00000
8@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0
Q03oool007D0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`250?ooo`00MP3oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`080?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo08D0oooo001g0?ooo`030000003oool0oooo
00`0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool0QP3oool007L0oooo00<000000?ooo`3oool03@3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool05P3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool06@3oool00`000000
oooo0?ooo`260?ooo`00N03oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo08L0oooo
001h0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?oo
o`0A0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0Q`3oool007T0oooo00<00000
0?ooo`3oool03@3oool010000000oooo0?ooo`00000?0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool07P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01H0oooo00<0
00000?ooo`3oool0R03oool007T0oooo00<000000?ooo`3oool03P3oool00`000000oooo0000000?
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0C
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0R@3oool007X0oooo00<000000?oo
o`3oool03@3oool200000140oooo00D000000?ooo`3oool0oooo0000000E0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0603oool3000001d0oooo00<000000?ooo`3oool04`3oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo08T0oooo001k0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool0403oool00`000000oooo0000000F0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`2:0?ooo`00O03oool00`000000
oooo0?ooo`0;0?ooo`030000003oool000000140oooo00<000000?ooo`0000005`3oool01@000000
oooo0?ooo`3oool0000001X0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0E0?oo
o`030000003oool0oooo0140oooo00<000000?ooo`3oool0R`3oool007d0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0000000B0?ooo`030000003oool0oooo01H0oooo00@000000?ooo`3o
ool000006`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`2;0?ooo`00OP3oool00`000000oooo0?ooo`080?ooo`05
0000003oool0oooo0?ooo`0000004@3oool2000001P0oooo00H000000?oOW000000000000000M02o
ool30?ooo`030?noM0000000M;oo00<0oooo00D0omnL04P000000000000007Boo`0:0?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0>0?ooo`030000
003oool0oooo08`0oooo001o0?ooo`8000001P3oool2000000D0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0000000H0?ooo`80000000D0M;oo0?ooo`3o_g@0001d0;ooo`020?ooo`030?no
M0000000M;oo0080oooo00L0omnL04P00018W=l0oooo0?oo_`1d0000M;oo00T0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool0S@3oool00840oooo1P0000080?ooo`030000003oool0oooo00`0oooo00@000000?ooo`3o
ool000005`3oool00`2oM00000000=ooo`020?ooo`030;md0000M;l0oooo00D0oooo00<0oooO09a8
0000M;l00`3oool00`3o_g@004RL0=ooo`080?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo08d0oooo00230?ooo`80
00002`3oool00`000000oooo0?ooo`090?ooo`800000103oool00`000000oooo0?ooo`0C0?ooo`03
0?oo_`1d0000000000<0oooo00<0gia80000M02oool01@3oool00`3ogi`0B00007Boo`030?ooo`03
0?noM00004P0W=oo00P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0L0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool0SP3oool00940oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01<0oooo00@0omnL0000001d_ol0
00000P3oool00`3o_g@0001809cOo`050?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02L
gol0203oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3o
ool01`3oool200000940oooo002B0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
1P3oool00`000000oooo0?ooo`0B0?ooo`040?oOW0000000M;oo00000080oooo00<0okmd0000B02L
gol01@3oool00`3ogi`0B00007Boo`030?ooo`030;md000004P0W=oo00P0oooo1000000G0?ooo`03
0000003oool0oooo01l0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`2A0?ooo`00
T`3oool7000000X0oooo00<000000?ooo`3oool04@3oool02@000000B00007Boo`3oool000000?oo
o`3o_g@0001809cOo`050?ooo`080?oo_`1d0000000004RLg`3oool0okmd00000000M;l90?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool0803oool7000009@0oooo002T0?ooo`030000
003oool0oooo0100oooo00@000000?oOW0180000M;oo0P3oool010000000okmd0000B02Lgol60?oo
o`030;md0000B9`0_ia80080000000<004RL0=ooo`3oool0203oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo0;`0oooo002U0?ooo`030000003oool0oooo00h0oooo00D000000?ooo`3o
okl0M00004RLg`020?ooo`040000003OW4P0001d0;ooo`H0oooo00<0omnL04P0001d_ol03@3oool0
0`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0;`0oooo002V0?ooo`030000003oool0oooo
00`0oooo00L000000?ooo`3oool0oooo0=nLB000B9`0gooo0080oooo00<00000001d_`3oool01`3o
ool00`3OW4P0001809cOo`0<0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0_@3o
ool00:L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`020?ooo`070?oo_`1d0000
M;oo0?ooo`3o_g@0001d000000090?ooo`030?noM000000007Bo00/0oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`2m0?ooo`00Z03oool2000000T0oooo00<000000?ooo`3oool0103o
ool01P3ookl0M0000000000007@0_ooo000000X0oooo00D0oooO09a800000000001d0;ooo`080?oo
o`030000003oool0oooo01D0oooo00<000000?ooo`3oool0_P3oool00:X0oooo0`0000040?ooo`80
00003@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool0_`3oool00:`0oooo1@00000?0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`2o0?ooo`00`@3oool00`000000oooo0?ooo`0C0?ooo`<000004`3o
ool00`000000oooo0?ooo`300?ooo`00`P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo0180oooo00<000000?ooo`3oool0`@3oool00<80oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo0<80oooo00330?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`320?ooo`00a03oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0``3oool00<@0oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0<@0oooo0035
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`34
0?ooo`00aP3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool0a@3oool00<L0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo0<H0oooo00380?ooo`030000003oool0oooo00`0oooo0`00000<0?ooo`030000
003oool0oooo0<L0oooo00390?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`370?ooo`00bP3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`3oool0b03oool00</0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo0<T0oooo003<0?ooo`030000003oool0oooo00P0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3:0?ooo`00c@3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo00L0oooo0P00003=0?ooo`00cP3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00D0oooo0P00003?0?ooo`00c`3oool3000000D0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`3?0?ooo`00dP3oool2000000<0oooo00@000000?oo
o`3oool0oooo0`00003B0?ooo`00e03oool700000=D0oooo003G0?ooo`030000003oool0oooo0=H0
oooo003G0?ooo`030000003oool0oooo0=H0oooo003G0?ooo`030000003oool0oooo0=H0oooo003G
0?ooo`030000003oool0oooo0=H0oooo003G0?ooo`030000003oool0oooo0=H0oooo003G0?ooo`03
0000003oool0oooo0=H0oooo0000\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {-5.25985, -0.67918, \
0.0366114, 0.00826623}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Interior matching condition", "Subsection"],

Cell[TextData[{
  "The following function compares values of the logarithmic derivative ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Psi]\^\[Prime]\)[x]/\[Psi][x]\)]],
  " at an interior point ",
  StyleBox["xmatch",
    FontSlant->"Italic"],
  " obtained by integrating either from ",
  StyleBox["xmin",
    FontSlant->"Italic"],
  " to ",
  StyleBox["xmatch",
    FontSlant->"Italic"],
  " or from ",
  StyleBox["xmax",
    FontSlant->"Italic"],
  " to ",
  StyleBox["xmatch",
    FontSlant->"Italic"],
  ".  The difference, or ",
  StyleBox["mismatch",
    FontSlant->"Italic"],
  ", between these values can then be used to determine the eigenenergy. "
}], "Text"],

Cell[BoxData[
    \(mismatch[e_, m_, pot_, xmin_, xmax_, xmatch_, 
        opts___Rule] := \[IndentingNewLine]Module[{bc1, bc2, x, 
          y}, \[IndentingNewLine]bc1 = \(y\^\[Prime]\)[xmatch]/
              y[xmatch] /. \((\(NDSolve[{\(-\(\(y\^\[DoublePrime]\)[x]\/\(2  
                                m\)\)\) + pot[x] y[x] \[Equal] e\ y[x], 
                    y[xmin] \[Equal] 0, \(y\^\[Prime]\)[xmin] \[Equal] 
                      10\^\(-6\)}, y, {x, xmin, xmatch}, 
                  opts]\)\[LeftDoubleBracket]1\[RightDoubleBracket])\); \
\[IndentingNewLine]bc2 = \(y\^\[Prime]\)[xmatch]/
              y[xmatch] /. \((\(NDSolve[{\(-\(\(y\^\[DoublePrime]\)[x]\/\(2  
                                m\)\)\) + pot[x] y[x] \[Equal] e\ y[x], 
                    y[xmax] \[Equal] 0, \(y\^\[Prime]\)[xmax] \[Equal] 
                      10\^\(-6\)}, y, {x, xmax, xmatch}, 
                  opts]\)\[LeftDoubleBracket]1\[RightDoubleBracket])\); 
        bc1 - bc2]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Double-sided shooting method", "Subsection"],

Cell[TextData[{
  "The following function integrates from ",
  StyleBox["xmin",
    FontSlant->"Italic"],
  " to ",
  StyleBox["xmatch",
    FontSlant->"Italic"],
  " and then from ",
  StyleBox["xmax",
    FontSlant->"Italic"],
  " to ",
  StyleBox["xmatch",
    FontSlant->"Italic"],
  " and uses ",
  StyleBox["FindRoot", "InlineInput",
    FontWeight->"Bold"],
  StyleBox[" ",
    FontSlant->"Italic"],
  "to determine an energy for which the mismatch between two logarithmic \
derivative vanishes.  The two sides of the solution are then normalized and \
patched together to generate the eigenfunction."
}], "Text"],

Cell[BoxData[
    \(ShootBSWF2[pot_, m_, emin_, emax_, xmin_, xmax_, xmatch_, 
        opts___Rule] := \[IndentingNewLine]Module[{\[Psi], x, y, x0, trial, 
          eq, bc, bc1, bc2, \[Psi]1, \[Psi]2, ratio, evalue, e, norm1, norm2, 
          norm, optsRoot, optsND, opts2}, \[IndentingNewLine]optsRoot = 
          FilterOptions[FindRoot, opts]; 
        optsND = 
          FilterOptions[NDSolve, opts]; \[IndentingNewLine]eq[e_, x_, 
            x0_] := {\(-\(\(y\^\[DoublePrime]\)[x]\/\(2  m\)\)\) + 
                pot[x] y[x] \[Equal] e\ y[x], 
            y[x0] \[Equal] 0, \(y\^\[Prime]\)[x0] \[Equal] 
              10\^\(-6\)}; \[IndentingNewLine]y1[e_, x_] := 
          y[x] /. \(NDSolve[eq[e, x, xmin], y, {x, xmin, xmatch}, 
                optsND]\)\[LeftDoubleBracket]1\[RightDoubleBracket]; \
\[IndentingNewLine]bc1[e_] := 
          D[y1[e, x], x]/y1[e, x] /. x \[Rule] xmatch; \[IndentingNewLine]y2[
            e_, x_] := 
          y[x] /. \(NDSolve[eq[e, x, xmax], y, {x, xmax, xmatch}, 
                optsND]\)\[LeftDoubleBracket]1\[RightDoubleBracket]; \
\[IndentingNewLine]bc2[e_] := 
          D[y2[e, x], x]/y2[e, x] /. x \[Rule] xmatch; \[IndentingNewLine]bc[
            e_?NumericQ] := bc1[e] - bc2[e]; \[IndentingNewLine]evalue = 
          If[emax \[Equal] emin, emin, 
            e /. With[{opts2 = optsRoot}, 
                FindRoot[bc[e], {e, emin, emax}, 
                  opts2]]]; \[IndentingNewLine]\[Psi]1[x_] = 
          y1[evalue, x]; \ \[Psi]2[x_] = 
          y2[evalue, 
            x]; \[IndentingNewLine]ratio = \[Psi]1[xmatch]/\[Psi]2[
              xmatch]; \[IndentingNewLine]norm1 = 
          NIntegrate[\[Psi]1[x]\^2, {x, xmin, 
              xmatch}]; \[IndentingNewLine]norm2 = 
          NIntegrate[\[Psi]2[x]\^2, {x, xmatch, 
              xmax}]; \[IndentingNewLine]norm = 
          1/\@\(norm1\  + \ \(ratio\^2\) norm2\); \[IndentingNewLine]\[Psi][
            x_] = If[x \[LessEqual] xmatch, norm\ \[Psi]1[x], 
            norm\ ratio\ \[Psi]2[x]]; \[IndentingNewLine]{energy \[Rule] 
            evalue, wavefunction \[Rule] \((\[Psi][#] &)\), 
          normalization \[Rule] norm}]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["A plotting function", "Subsection"],

Cell["\<\
The following function plots the potential, eigenenergy, and eigenfunction \
together.\
\>", "Text"],

Cell[BoxData[{
    \(\(Clear[PlotBSWF];\)\), "\n", 
    \(\(PlotBSWF[solution_, xmin_, xmax_, opts___Rule] := 
        Module[{e = energy /. solution, \[Psi], 
            f}, \[IndentingNewLine]\[Psi][x_] = 
            wavefunction[x] /. solution; \[IndentingNewLine]f[x_] = 
            e + If[xmin < x < xmax, \[Psi][x], 
                0]; \[IndentingNewLine]Plot[{pot[x], f[x], e}, {x, xmin, 
              xmax}, opts, 
            PlotStyle \[Rule] {RGBColor[1, 0, 0], Dashing[{0}], 
                Dashing[{0.02, 0.04}]}, Frame \[Rule] True, 
            FrameLabel \[Rule] {"\<x\>", "\<V, \[Psi]\>"}, 
            PlotLabel \[Rule] "\<Wave Function with Energy = \>" <> 
                ToString[N[energy /. solution, 4]]]];\)\)}], "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Test case: harmonic oscillator", "Section"],

Cell[TextData[{
  "The eigenvalue problem for the harmonic oscillator potential ",
  Cell[BoxData[
      \(TraditionalForm\`V[x] = \(1\/2\) k\ x\^2\)]],
  " can be solved symbolically, with exact eigenenergies ",
  Cell[BoxData[
      \(TraditionalForm\`\[CurlyEpsilon]\_n = \((n + 1\/2)\) \[Omega]\)]],
  " where ",
  Cell[BoxData[
      \(TraditionalForm\`\[Omega] = \@\(k/m\)\)]],
  " is the classical oscillation frequency.  Using the double-sided shooting \
method, the energy eigenvalues are determined by matching the logarithmic \
derivative of the wave functions obtained by integration from either side of \
the potential toward the interior.  Although the match point can be chosen \
almost anywhere, for a symmetric potential it is natural to choose the center \
of the potential.  Even solutions have finite wave functions but vanishing \
derivatives, so that the mismatch between logarithmic derivatives exhibits a \
simple zero.  Odd solutions have finite derivatives but vanishing wave \
functions, so that the mismatch exhibits a singularity at the eigenvalue.The \
figure below illustrates the matching condition used by the symmetric \
shooting method.  Note that we must increase the default ",
  StyleBox["AccuracyGoal", "InlineInput",
    FontWeight->"Bold"],
  " to obtain a smooth curve for higher energies."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
        mismatch[e, 1, ho, \(-5\), 5, 0, AccuracyGoal \[Rule] 10], {e, 0, 
          10}, PlotRange \[Rule] {All, {\(-10\), 10}}, 
        PlotPoints \[Rule] 100, Frame \[Rule] True, 
        FrameLabel \[Rule] {"\<energy\>", "\<mismatch\>"}, 
        PlotLabel \[Rule] "\<matching condition for m=1\>"];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0952381 0.309017 0.0309017 [
[.02381 -0.0125 -5 -15.6875 ]
[.02381 -0.0125 5 0 ]
[.21429 -0.0125 -5 -15.6875 ]
[.21429 -0.0125 5 0 ]
[.40476 -0.0125 -5 -15.6875 ]
[.40476 -0.0125 5 0 ]
[.59524 -0.0125 -5 -15.6875 ]
[.59524 -0.0125 5 0 ]
[.78571 -0.0125 -5 -15.6875 ]
[.78571 -0.0125 5 0 ]
[.97619 -0.0125 -8 -15.6875 ]
[.97619 -0.0125 8 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -17.9688 -31.375 ]
[.5 -0.0125 17.9688 -15.6875 ]
[-0.0125 .07725 -27.3125 -7.84375 ]
[-0.0125 .07725 0 7.84375 ]
[-0.0125 .15451 -18.3125 -7.84375 ]
[-0.0125 .15451 0 7.84375 ]
[-0.0125 .23176 -27.3125 -7.84375 ]
[-0.0125 .23176 0 7.84375 ]
[-0.0125 .30902 -10 -7.84375 ]
[-0.0125 .30902 0 7.84375 ]
[-0.0125 .38627 -19 -7.84375 ]
[-0.0125 .38627 0 7.84375 ]
[-0.0125 .46353 -10 -7.84375 ]
[-0.0125 .46353 0 7.84375 ]
[-0.0125 .54078 -19 -7.84375 ]
[-0.0125 .54078 0 7.84375 ]
[-0.0125 .61803 -16 -7.84375 ]
[-0.0125 .61803 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -43 -25.2813 ]
[-0.01875 .30902 -27.3125 25.2813 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -69.8125 0 ]
[.5 .63053 69.8125 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
gsave
.02381 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.21429 0 m
.21429 .00625 L
s
gsave
.21429 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.40476 0 m
.40476 .00625 L
s
gsave
.40476 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.59524 0 m
.59524 .00625 L
s
gsave
.59524 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.78571 0 m
.78571 .00625 L
s
gsave
.78571 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8) show
1.000 setlinewidth
grestore
.97619 0 m
.97619 .00625 L
s
gsave
.97619 -0.0125 -69 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
1.000 setlinewidth
grestore
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.5 0 m
.5 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -78.9688 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(energy) show
94.938 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .07725 m
.00625 .07725 L
s
gsave
-0.0125 .07725 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7.5) show
1.000 setlinewidth
grestore
0 .15451 m
.00625 .15451 L
s
gsave
-0.0125 .15451 -79.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
0 .23176 m
.00625 .23176 L
s
gsave
-0.0125 .23176 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.5) show
1.000 setlinewidth
grestore
0 .30902 m
.00625 .30902 L
s
gsave
-0.0125 .30902 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
0 .38627 m
.00625 .38627 L
s
gsave
-0.0125 .38627 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.5) show
1.000 setlinewidth
grestore
0 .46353 m
.00625 .46353 L
s
gsave
-0.0125 .46353 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
0 .54078 m
.00625 .54078 L
s
gsave
-0.0125 .54078 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7.5) show
1.000 setlinewidth
grestore
0 .61803 m
.00625 .61803 L
s
gsave
-0.0125 .61803 -77 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .01545 m
.00375 .01545 L
s
0 .0309 m
.00375 .0309 L
s
0 .04635 m
.00375 .04635 L
s
0 .0618 m
.00375 .0618 L
s
0 .09271 m
.00375 .09271 L
s
0 .10816 m
.00375 .10816 L
s
0 .12361 m
.00375 .12361 L
s
0 .13906 m
.00375 .13906 L
s
0 .16996 m
.00375 .16996 L
s
0 .18541 m
.00375 .18541 L
s
0 .20086 m
.00375 .20086 L
s
0 .21631 m
.00375 .21631 L
s
0 .24721 m
.00375 .24721 L
s
0 .26266 m
.00375 .26266 L
s
0 .27812 m
.00375 .27812 L
s
0 .29357 m
.00375 .29357 L
s
0 .32447 m
.00375 .32447 L
s
0 .33992 m
.00375 .33992 L
s
0 .35537 m
.00375 .35537 L
s
0 .37082 m
.00375 .37082 L
s
0 .40172 m
.00375 .40172 L
s
0 .41717 m
.00375 .41717 L
s
0 .43262 m
.00375 .43262 L
s
0 .44807 m
.00375 .44807 L
s
0 .47898 m
.00375 .47898 L
s
0 .49443 m
.00375 .49443 L
s
0 .50988 m
.00375 .50988 L
s
0 .52533 m
.00375 .52533 L
s
0 .55623 m
.00375 .55623 L
s
0 .57168 m
.00375 .57168 L
s
0 .58713 m
.00375 .58713 L
s
0 .60258 m
.00375 .60258 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -121.438 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
86.2813 11.8438 translate
90 rotate
-86.2813 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(mismatch) show
109.563 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .61178 m
.02381 .61803 L
s
.21429 .61178 m
.21429 .61803 L
s
.40476 .61178 m
.40476 .61803 L
s
.59524 .61178 m
.59524 .61803 L
s
.78571 .61178 m
.78571 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.07143 .61428 m
.07143 .61803 L
s
.11905 .61428 m
.11905 .61803 L
s
.16667 .61428 m
.16667 .61803 L
s
.2619 .61428 m
.2619 .61803 L
s
.30952 .61428 m
.30952 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.5 .61428 m
.5 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.69048 .61428 m
.69048 .61803 L
s
.7381 .61428 m
.7381 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.88095 .61428 m
.88095 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 0 m
1 0 L
s
.99375 .07725 m
1 .07725 L
s
.99375 .15451 m
1 .15451 L
s
.99375 .23176 m
1 .23176 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .38627 m
1 .38627 L
s
.99375 .46353 m
1 .46353 L
s
.99375 .54078 m
1 .54078 L
s
.125 Mabswid
.99625 .01545 m
1 .01545 L
s
.99625 .0309 m
1 .0309 L
s
.99625 .04635 m
1 .04635 L
s
.99625 .0618 m
1 .0618 L
s
.99625 .09271 m
1 .09271 L
s
.99625 .10816 m
1 .10816 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .13906 m
1 .13906 L
s
.99625 .16996 m
1 .16996 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .20086 m
1 .20086 L
s
.99625 .21631 m
1 .21631 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26266 m
1 .26266 L
s
.99625 .27812 m
1 .27812 L
s
.99625 .29357 m
1 .29357 L
s
.99625 .32447 m
1 .32447 L
s
.99625 .33992 m
1 .33992 L
s
.99625 .35537 m
1 .35537 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .40172 m
1 .40172 L
s
.99625 .41717 m
1 .41717 L
s
.99625 .43262 m
1 .43262 L
s
.99625 .44807 m
1 .44807 L
s
.99625 .47898 m
1 .47898 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .50988 m
1 .50988 L
s
.99625 .52533 m
1 .52533 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57168 m
1 .57168 L
s
.99625 .58713 m
1 .58713 L
s
.99625 .60258 m
1 .60258 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .30902 m
1 .30902 L
s
gsave
.5 .63053 -130.813 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(matching) show
110.375 15.813 moveto
(condition) show
158.563 15.813 moveto
(for) show
175.438 15.813 moveto
(m) show
184.813 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
192.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
198.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .35079 m
.03318 .34408 L
.04339 .3361 L
.05298 .32782 L
.06221 .31897 L
.07203 .30832 L
.08148 .29651 L
.09153 .28167 L
.10121 .26418 L
.11052 .24295 L
.11524 .22981 L
.12043 .21273 L
.12997 .1705 L
.13467 .14139 L
.13974 .09964 L
.14219 .07361 L
.14486 .03885 L
s
.14486 .03885 m
.14723 0 L
s
.16668 0 m
.16668 .61803 L
s
.18872 .61803 m
.19283 .56628 L
.19726 .5261 L
.20217 .49225 L
.20665 .46756 L
.21627 .42697 L
.22552 .39714 L
.23537 .37067 L
.24485 .34791 L
.25492 .32492 L
.26463 .30272 L
.27396 .28022 L
.2839 .25377 L
.29346 .2241 L
.30266 .18918 L
.30778 .1656 L
.31245 .14033 L
.32188 .07245 L
.3265 .02581 L
s
.3265 .02581 m
.32858 0 L
s
.3571 0 m
.3571 .61803 L
s
.38796 .61803 m
.38862 .60954 L
.3933 .56363 L
.39836 .52431 L
.40831 .46576 L
.4179 .42304 L
.42713 .38886 L
.43694 .35655 L
.44639 .32732 L
.45644 .2965 L
.46611 .26549 L
.47542 .23278 L
.48533 .1925 L
.49486 .14466 L
.50403 .08422 L
.50874 .04425 L
s
.50874 .04425 m
.51291 0 L
s
.54777 0 m
.54777 .61803 L
s
.58426 .61803 m
.5856 .60305 L
.59065 .55787 L
.60058 .48968 L
.61014 .43928 L
.61933 .39865 L
.62912 .36005 L
.63854 .32505 L
.64856 .28809 L
.65821 .25085 L
.66749 .21151 L
.67736 .16291 L
.68687 .10484 L
.69601 .031 L
s
.69601 .031 m
.69896 0 L
s
.7383 0 m
.7383 .61803 L
s
.7798 .61803 m
.78214 .59176 L
.7873 .5485 L
.79218 .51353 L
.80149 .45771 L
.81139 .40803 L
.82093 .36546 L
.83009 .32693 L
.83985 .28635 L
.84925 .24576 L
.85923 .19868 L
.86885 .14636 L
.87811 .08498 L
.88327 .04351 L
s
.88327 .04351 m
.88787 0 L
s
.93219 0 m
.93219 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{381.625, 235.875},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40008l0001HB000`400?l00000o`00003oo`3ooooo
0?oooch0oooo003o0?ooool0oooo?P3oool00?l0ooooB@3oool00`3ooml0W4P00000000300000003
04RLg`3oool0gia80080000000<0M;oo0?ooo`3oool0iP3oool00?l0ooooB@3oool00`2oM00004RL
0=ooo`030?ooo`030=nLW02oool0oooo0080oooo00<0_gAd0;ooo`3oool0iP3oool00?l0ooooB@3o
ool00`3ooml0W7BL0=ooo`030?ooo`030?oo_`2LW=l0oooo0080oooo00<0omnL07BLg`3oool0iP3o
ool00?l0ooooBP3oool00`2oM0000000000000030000000704RLg`3oool0oooo0?ooo`3ooml0W4Qd
0;ooo`3V0?ooo`00o`3oool/0?ooo`070;md0000000000000000M02oool0oooo0;md000200000003
0000M02ooml0W4P00080000000@004RL0=ooo`3oool0_g@00P000000100007@0_ooo0?ooo`2oM002
000000050000B02Lgol0oooo0?oo_`1d0000100000000`0007@0_ooo0?ooo`030?ooo`030;md001d
_ol0oooo0>D0oooo003o0?ooob/0oooo00l0_g@000000000M;l0oooo0?oOW01807@0_ooo0?ooo`3O
W4P004RL0=ooo`3oool0oooO09a80000M;l00P3oool02`2oM0000000001d_`3oool0omnL04P0M02o
ool0oooo0=nLB00007@0_ooo00<0oooo00<0_gA809cOo`3oool01`3oool0103ogi`0B0000018W03O
oooU0?ooo`00o`3ooolZ0?ooo`030?oOW0180000B9cO00<0oooo00h0omnL07BLg`3oool0gia80018
W03Oool0oooo0?oog`2LB00007Bo0?ooo`3ogi`0B00004RLg`<0oooo00H0omnL07BLg`3oool0gia8
0000M02oool30?ooo`050?oo_`1d0000000000000000M;l01@3oool0102oM00004RL09ad001d_ooU
0?ooo`00o`3ooolZ0?ooo`030?noM00004P0W=oo00H0oooo00/0gia80018W03Oool0oooo0?oog`2L
B00007Bo0?ooo`3o_g@0001809cOo`060?ooo`030=nLB00007@0_ooo00<0oooo00H0gia80000M02o
ool0oono07@00018W=l30?ooo`050?noM0000000M;oo0?oOW018M;l0i@3oool00?l0oooo:P3oool0
0`3o_g@0001d0;ooo`060?ooo`0;0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd0000
M02oool01P3oool00`3OW4P0001d0;ooo`020?ooo`070?oo_`1d0000B9cO0?ooo`3ooml0W4P0001d
_`020?ooo`070?oog`2LB00004RL0=ooo`3oool0_gA809cOo`3T0?ooo`00o`3ooolZ0?ooo`030?oO
W0180000000000@0000000`0B9cO0?ooo`2oM000001d0;ooo`3oool0oooO09a80000M;l0oooo0?oO
W01800050000000504RLg`3oool0gia80000B02Lgol00P3oool01`3ookl0M00004RLg`3oool0oooO
09a80018W=l00P3oool00`3o_g@0001809cOo`020?ooo`030?noM018W=l0oooo0><0oooo003o0?oo
ob/0oooo00l0_g@004RLg`3oool0oooo0=nLB00004P0W=oO09a80000000004Q804P0B02Lgol0oono
07@00018W=l00P3oool04P2oM000B9cO0?ooo`3oool0gia80000B02Lgml0W4P00000000004P0B7BL
0=noM00007@0_ooo0;md0000M;l0oooo0?noM080000000D004RL0=oo_`1d000004RL0=ooo`020?oo
o`030?oo_`1d07@0_ooo0><0oooo003o0?ooob`0oooo00D0gia8000000000000001809cOo`020?oo
o`070;md0000B9`0gooO09a800000000001d0;ooo`030?ooo`050=nLB000000000000000B02Lgol0
0P3oool0202oM000001d0;oOW0180000001d0;ooo`3ooml0W4P00P000000101d_ol0oooo0?oog`2L
B002000000070018W03Oool0oooo0;md0000000004RL0=ooo`3R0?ooo`00o`3ooooo0?oooch0oooo
003o0?ooool0oooo?P3oool00?l0ooooo`3oooln0?ooo`00o`3ooooo0?oooch0oooo003o0?ooool0
oooo?P3oool00?l0ooooo`3oooln0?ooo`00o`3ooooo0?oooch0oooo003o0?ooool0oooo?P3oool0
0?l0ooooo`3oooln0?ooo`00o`3ooooo0?oooch0oooo003o0?ooool0oooo?P3oool00?l0ooooo`3o
ooln0?ooo`00o`3ooooo0?oooch0oooo003o0?ooool0oooo?P3oool00?l0ooooo`3oooln0?ooo`00
o`3ooooo0?oooch0oooo001]0?ooo`050?oOW018000000000000M02oool0C@3oool00`2oM0000000
000000040000000304RLg`3oool0oooo04h0oooo00@0oooO09a80000B9`0goooC@3oool01@3ogi`0
B000000000000000M;oo04l0oooo00D0gia8000000000000000004RLg`1:0?ooo`060;md00000000
000000000000B9`0gooo0`3oool01@3ogi`0B0000000000007@0_ooo02<0oooo001/0?ooo`070?oo
_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`1=0?ooo`030=nLB0000000000000<0000000<004RL0=oo
o`3oool0CP3oool0103ooml0W4P00018W03Ooom<0?ooo`070?oOW0180000B9cO0?ooo`3ookl0M000
07Boo`1=0?ooo`070=nLB0000000M;oo0?ooo`3ooml0W4P004RLg`1:0?ooo`030?oOW0180000M;oo
00@0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo0280oooo001/0?ooo`030;md0000B9`0
gooo0080oooo00<0_g@0001d_`3oool0C@3oool00`3OW4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0
oooo04h0oooo00@0oooO09a80000B9`0goooB`3oool00`3ooml0W4P0001d_`030?ooo`030?noM000
B9`0gooo04/0oooo00<0oooO09a80000M;l00`3oool00`3o_g@004RL0=ooo`190?ooo`030?oOW018
0000M;oo00@0oooo00<0_g@00018W03Oool00P3oool00`2oM00007Bo0?ooo`0Q0?ooo`00J`3oool0
0`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo04h0oooo00<0gia807Boo`3oool0CP3oool0
0`2oM0000000000000050000000307Boo`3oool0oooo04P0oooo00<0omnL04P0001d_ol00`3oool0
0`3o_g@0001809cOo`1<0?ooo`030;md0000M;l0oooo0080oooo00<0_g@00018W03Oool0B@3oool0
0`3ogi`0B00007Boo`030?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool08@3oool0
06/0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`1?0?ooo`030;mdB02Lgol0oooo
04d0oooo00P0omnL07BLg`3oool0oooo0?oog`2LB00004RL0=oood/0oooo00<0omnL04P0001d_ol0
0`3oool00`3o_g@0001809cOo`1<0?ooo`070?oOW01807@0_ooo0?ooo`2oM000000007Boo`1:0?oo
o`030?oOW0180000M;oo00<0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0Q0?oo
o`00J`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo04l0oooo00<0oooO09a8M02o
ool0CP3oool01`3OW4P0M;oo0?ooo`3ooml0W4P00018W03Oool0B`3oool00`3ogi`0B00007Boo`03
0?ooo`030;md000004P0W=oo04d0oooo00D0oono07A8B0180000000004RLg`1;0?ooo`030?oOW018
0000M;oo00<0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0Q0?ooo`00J`3oool0
0`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo0500oooo00<0okmd001d_`3oool0C@3oool0
1`3ooml0W4Qd0;ooo`3ooml0W4P00018W03Oool0B`3oool0203ookl0M00000000018W=l0oooo0?no
M000000007BoCP3oool0103ogi`0B00000000000M;m<0?ooo`030?oOW0180000M;oo00<0oooo00<0
omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0Q0?ooo`00J`3oool00`3ogi`0B00007Boo`03
0?ooo`030?noM00004P0W=oo0500oooo00<0oono07@0B02Lgol0CP3oool01P3ogi`0B7Bo0?oog`2L
B00004RL0=oood`0oooo00<0_g@00018W02oW4P00P0000000`00B9`0gooo0?ooo`1<0?ooo`060?oO
W0180000001d0;ooo`3o_g@007BoB`3oool00`3ogi`0B00007Boo`030?ooo`030?oOW0180000M;oo
00<0oooo00<0okmd0000B02Lgol08@3oool006/0oooo00<0oono07@00018W=l00`3oool00`3OW4P0
001d0;ooo`1@0?ooo`030?oo_`1d0000B9cO04l0oooo00D0gia807Bog`2LB00004RL0=ooo`1<0?oo
o`030?oOW0180000M;oo0540oooo00<0_g@00000M02oool00P3oool00`2oM000B9cO0?ooo`190?oo
o`030?oOW0180000M;oo00<0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`0Q0?oo
o`00K03oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo04/0oooo00<0oknL09cOo`3o
ool00P3oool00`3ogi`0B00004RLg`1?0?ooo`050?oog`2LB4P0B4P00018W03Oool0C@3oool00`3O
W4P0001809cOo`1@0?ooo`030;md0000B9`0gooo0080oooo00<0gia8001d_`3oool0B@3oool00`3o
gi`0B00007Boo`040?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d_`3oool08@3oool006`0
oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo04`0oooo00P0oooO09a8B02Lgol0oooo0?oo
_`1d0000001d0;oooe00oooo00@0omnL04P00000B9`0goooCP3oool00`3o_g@00000001d_`1?0?oo
o`070?noM0000000M;oo0?ooo`3ookl0M00007Boo`180?ooo`050?oog`2LB00000000000001d_ol0
103oool01`3ookl0M00007Boo`3oool0okmd0000M02oool08P3oool006d0oooo00D0oono07@00000
0000001d0;ooo`1>0?ooo`060?oog`2LB00000000000000007@0_oooDP3oool00`3OW4P004RL0=oo
o`1?0?ooo`050?oog`2LB00000000000M02oool0C@3oool01@3o_g@00000000000000000B9cO04/0
oooo00<0okmd0000001d_ol01@3oool01@3ookl0M0000000000007@0_ooo02<0oooo003o0?ooool0
oooo?P3oool00?l0ooooo`3oooln0?ooo`00o`3ooooo0?oooch0oooo003o0?ooool0oooo?P3oool0
0?l0ooooo`3oooln0?ooo`00o`3ooooo0?oooch0oooo003o0?ooool0oooo?P3oool00?l0ooooo`3o
ooln0?ooo`00o`3ooooo0?oooch0oooo003o0?ooool0oooo?P3oool006D0ooooo`00002i000001l0
oooo001U0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`0B0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`0B0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`0A0?ooo`050000003oool0oooo0?ooo`000000403oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool04P3oool010000000oooo0?ooo`00000A0?oo
o`8000004`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01d0oooo001U0?ooo`03
0000003oool0oooo00P0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0A0?ooo`05
0000003oool0oooo0?ooo`000000403oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool04P3oool010000000oooo0?ooo`00000A0?ooo`8000004`3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo
00P0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000
oooo0?ooo`0n0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0A03oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool03@3oool00`000000
oooo0?ooo`0o0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0?`3oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo
00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo04@0oooo
00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo03h0oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo
001U0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`130?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0n0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`0M0?ooo`00I@3oool3000003h0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`13
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0n
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool06P3oool2000001l0oooo001U0?oo
o`030000003oool0oooo03h0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`130?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo03h0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0n0?oo
o`030000003oool0oooo0140oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?oo
o`00I@3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool0@`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3o
ool0?P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool07@3oool006D0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo04<0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`100?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0?0?ooo`030000
003oool0oooo03h0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0K0?ooo`030000
003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`130?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0@03o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0
oooo0400oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0
oooo00l0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo00<000000?ooo`3oool0?@3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`100?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0?P3oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`<00000?@3oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`100?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0?P3oool00`000000
oooo0?ooo`0?0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0J0?ooo`8000007`3oool006D0oooo00<000000?ooo`3oool0?@3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`100?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0?P3oool00`000000oooo
0?ooo`0?0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo03d0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool0@03oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo03h0oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`0m
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0n
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0B
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo00<000000?oo
o`3oool0?@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0480oooo00<000000?oo
o`3oool02`3oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool0?@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo03d0oooo00<000000?oo
o`3oool04P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`03
0000003oool0oooo03d0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`120?ooo`03
0000003oool0oooo00/0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0>0?ooo`03
0000003oool0oooo03d0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0m0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00
I@3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
@P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
?@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
7@3oool006D0oooo0`00000m0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0@P3o
ool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0?@3o
ool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01X0oooo0P00000O0?ooo`00I@3oool0
0`000000oooo0?ooo`0m0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0@P3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0m0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0?@3oool0
0`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool0
06D0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
0480oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
03d0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
01d0oooo001U0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`120?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0@03oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`0l0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool06`3oool00`000000
oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo00P0oooo
00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo03l0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01/0oooo
00<000000?ooo`3oool07@3oool006D0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0o0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?oo
o`0A0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`<00000?03oool00`000000oooo0?ooo`08
0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0o
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0J
0?ooo`8000007`3oool006D0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`080?oo
o`030000003oool0oooo0440oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0o0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo03`0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo01d0oooo00190?ooo`030=nLB018W=l0oooo00@0oooo00H0okmd0000001d
_ol0oooo0?oog`2LB0030000000307Boo`3oool0oooo00T0oooo00<000000?ooo`3oool0?03oool0
0`000000oooo0?ooo`080?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`0o0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0?03oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool04`3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo00190?ooo`030?oOW018B9`0gooo
00@0oooo00L0okmd0000001d_ol0oooo0?oog`2LB7@0_ooo0080oooo00<0_g@007Boo`3oool02@3o
ool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0@@3o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0?03o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3o
ool004T0oooo00<0oooO09a8B02Lgol03@3oool00`3ooml0W4Qd0;ooo`090?ooo`030000003oool0
oooo03`0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`110?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0
oooo03`0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00BP3oool00`3O
W4P0B9cO0?ooo`0=0?ooo`030;md001d_ol0oooo00P0oooo00<000000?ooo`3oool0?03oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`0o0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0?03oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool04`3oool00`00
0000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo000j0?ooo`030?oog`2LB000000000L0
000000<0B9cO0?ooo`3oool00`3oool00`3ogi`0B4RL0=ooo`0<0?ooo`030?oog`2LB000B9cO00T0
oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0440
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo03`0
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0
oooo001:0?ooo`030?oog`2LB4P0W=oo00`0oooo00<0gia80000001d_ol02@3oool4000003/0oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo03/0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool06@3oool3000001l0oooo001;0?ooo`030=nLB000M;l0oooo00L0oooo00L0
gmoO0?ooo`3ookl0M0000000000007@0_ooo00T0oooo00<000000?ooo`3oool0?03oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool03@3oool00`000000
oooo0?ooo`0n0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0>`3oool00`000000
oooo0?ooo`0B0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001;0?ooo`030?oo_`1d07@0_ooo00L0oooo
00H0gia800000000000000000000B02Lgol:0?ooo`030000003oool0oooo03`0oooo00<000000?oo
o`3oool0203oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool0?P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo03/0oooo00<000000?oo
o`3oool04P3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo01@0oooo00<000000?oo
o`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00AP3oool00`3ogml0gooo0?ooo`030?ooo`03
0;md001d_ol0oooo00H0oooo00@0oono07@000000000B9cO303oool00`000000oooo0?ooo`0l0?oo
o`030000003oool0oooo00P0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo03h0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0k0?oo
o`030000003oool0oooo0180oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`0D0?oo
o`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool004H0oooo00<0oono07A8W03O
ool00`3oool00`3o_g@007Bo0?ooo`070?ooo`030;md001d_ol0oooo00`0oooo00<000000?ooo`3o
ool0?03oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0400oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool0>`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo00170?ooo`030;md
00000000000000<0000000<0001d0;ooo`3oool01P3oool01P3ogi`0B000000000000000001809cO
o`T0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
0400oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
03/0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
01d0oooo00170?ooo`030?noM0000000000000@0000000<0M;oo0?ooo`3oool01P3oool01@2oM000
0000000000000000B9cO00T0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo0400oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0n0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`0B0?oo
o`030000003oool0oooo03/0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo01d0oooo001U0?ooo`<00000>`3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo03l0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0m0?ooo`03
0000003oool0oooo0140oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo03/0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0J0?ooo`80
00007`3oool006D0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo03l0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0m0?ooo`030000
003oool0oooo0140oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo03X0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0K0?ooo`030000
003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0?@3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool04`3o
ool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo03d0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo00<000000?ooo`3oool0>`3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0?ooo`0m0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0>P3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool05@3oool00`00
0000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo03/0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo03X0
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo01D0
oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo
0?ooo`0j0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0?P3oool00`000000oooo
0?ooo`0?0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`0j0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0>P3oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo0`00
000j0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?oo
o`0?0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`0j0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?oo
o`0F0?ooo`030000003oool0oooo01X0oooo0P00000O0?ooo`00I@3oool00`000000oooo0?ooo`0j
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0j
0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0F
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo00<000000?oo
o`3oool0>P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo03h0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo0180oooo00<000000?oo
o`3oool0>@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo03T0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`03
0000003oool0oooo03X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0n0?ooo`03
0000003oool0oooo00l0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo03T0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0i0?ooo`03
0000003oool0oooo01H0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00
I@3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
?P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
>@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
7@3oool006D0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo03d0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0l0?ooo`030000003o
ool0oooo0180oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0D0?ooo`030000003o
ool0oooo03T0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0K0?ooo`030000003o
ool0oooo01d0oooo001U0?ooo`<00000>@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo03d0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0k0?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo03T0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0J0?ooo`8000007`3oool0
06D0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo
03d0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo
01<0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
03P0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
01d0oooo001U0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool02`3oool00`000000
oooo0?ooo`0m0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0>`3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3oool05@3oool00`000000
oooo0?ooo`0h0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool06`3oool00`000000
oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo03/0oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo01D0oooo
00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01/0oooo
00<000000?ooo`3oool07@3oool006D0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`0k0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?oo
o`0E0?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?oo
o`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo03P0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo0140oooo00<00000
0?ooo`3oool0>`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo03P0oooo00<00000
0?ooo`3oool05@3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo01L0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`0h0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo03/0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0h0?oo
o`030000003oool0oooo01D0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo0`00000h0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo03X0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0h0?ooo`03
0000003oool0oooo01D0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01X0oooo0P00000O0?ooo`00I@3oool00`000000oooo0?ooo`0h0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo03X0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0h0?ooo`030000
003oool0oooo01D0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0H0?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool005@0oooo00<0oooO09a8000000000P00
00000`1d_ol0oooo0?ooo`090?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0>P3o
ool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool05P3o
ool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`0M0?ooo`00E03oool00`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0
oooo00T0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo03`0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0
oooo01@0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
oooo03L0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0K0?ooo`030000003oool0
oooo01d0oooo001I0?ooo`030?oog`2LB7@0_ooo00T0oooo00<000000?ooo`3oool0>03oool00`00
0000oooo0?ooo`0<0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`0j0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0=`3oool00`00
0000oooo0?ooo`0F0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001J0?ooo`030;md001d_ol0oooo00P0
oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo03/0
oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo03L0
oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0
oooo00170?ooo`030?oog`2LB000000000L0000000<0B9cO0?ooo`3oool01@3oool00`3ooml0W4P0
04RLg`090?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`0k0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0>@3oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool05P3oool00`000000oooo
0?ooo`0f0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06`3oool00`000000oooo
0?ooo`0M0?ooo`00F@3oool00`3OW4P0000007Boo`090?ooo`@00000=P3oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`0i0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?oo
o`0F0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?oo
o`0I0?ooo`<000007`3oool005D0oooo00L0gmoO0?ooo`3ookl0M0000000000007@0_ooo00T0oooo
00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo03/0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo01D0oooo
00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo03H0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo
001E0?ooo`060=nLB000000000000000000004P0W=oo2P3oool00`000000oooo0?ooo`0g0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo03T0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0f0?ooo`03
0000003oool0oooo01L0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool005D0oooo00@0oono07@000000000
B9cO303oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3o
ool0>P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3o
ool05P3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3o
ool0=@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool07@3oool005H0oooo00<0_g@007Boo`3oool0303oool00`000000oooo0?ooo`0f0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo03P0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0f0?ooo`030000
003oool0oooo01L0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool005H0oooo00H0omnL04P0000000000000
0000B02Lgol90?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`0j0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0>03oool00`000000
oooo0?ooo`0F0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`0e0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06`3oool00`000000
oooo0?ooo`0M0?ooo`00E`3oool01@2oM0000000000000000000B9cO00T0oooo00<000000?ooo`3o
ool0=P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3o
ool04`3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3o
ool0=P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`<00000
=P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
=@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0J0?ooo`8000007`3oool006D0oooo00<000000?ooo`3oool0=P3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool0503o
ool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0=@3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`030000003oool0
oooo03H0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0
oooo01@0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0
oooo03D0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0d0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`00
0000oooo0?ooo`0e0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0>@3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`0e0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0=03oool00`00
0000oooo0?ooo`0K0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool006D0
oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo03T0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo03@0
oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0
oooo001U0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`0i0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0=P3oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`0d0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool06`3oool00`000000oooo
0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo00l0oooo00<0
00000?ooo`3oool0>03oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo03H0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`3oool0<`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool07@3oool006D0oooo0`00000e0?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool0>03oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo03H0oooo00<00000
0?ooo`3oool0603oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool0<`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01X0oooo0P00000O
0?ooo`00I@3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool0>03oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo03D0oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool0<`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool07@3oool006D0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo03P0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0e0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo03<0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0
=@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`0d0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0F0?ooo`030000003o
ool0oooo03D0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0c0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo00<000000?ooo`3oool0<`3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0?ooo`0e0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0<P3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool07@3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`<00000<`3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool05P3oool00`00
0000oooo0?ooo`0d0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0<P3oool00`00
0000oooo0?ooo`0K0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool07@3oool00`00
0000oooo0?ooo`0J0?ooo`8000007`3oool006D0oooo00<000000?ooo`3oool0<`3oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`0d0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0<P3oool00`000000
oooo0?ooo`0K0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool07@3oool00`000000
oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo03<0oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0380oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01h0oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?oo
o`0c0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?oo
o`0G0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`0a0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?oo
o`0N0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo00<00000
0?ooo`3oool0<P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo03H0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo01/0oooo00<00000
0?ooo`3oool0<@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0340oooo00<00000
0?ooo`3oool07P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?oo
o`030000003oool0oooo0380oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0e0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo0340oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0a0?oo
o`030000003oool0oooo01h0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?oo
o`00I@3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool0=@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3o
ool0<@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool07@3oool006D0oooo0`00000b0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
=@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0
<03oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01X0oooo0P00000O0?ooo`00I@3o
ool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0=@3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool0703o
ool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0<03o
ool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3o
ool004H0oooo00<0_g@0000000000000100000002@18W=l0oooo0?ooo`3o_g@0000007Boo`3oool0
oooO09a800030000000307Boo`3oool0oooo00T0oooo00<000000?ooo`3oool0<@3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool06@3oool00`000000
oooo0?ooo`0b0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0<03oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool07`3oool00`000000
oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo00170?ooo`030=nLB0000000000000<00000
00X004RL0=ooo`3oool0okmd0000001d_ol0oooo0?oog`2LB7@0_ooo0P3oool00`2oM000M;oo0?oo
o`090?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`0d0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?oo
o`0L0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool07@3oool00`000000oooo0?oo
o`0`0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`0M0?ooo`00B03oool00`3OW4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo00T0oooo00<0oooO
09a8M02oool02@3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01<0oooo00<00000
0?ooo`3oool0=03oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0380oooo00<00000
0?ooo`3oool0703oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo01d0oooo00<00000
0?ooo`3oool0<03oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01/0oooo00<00000
0?ooo`3oool07@3oool004T0oooo00<0gia807Boo`3oool03P3oool00`2oM000M;oo0?ooo`080?oo
o`030000003oool0oooo0300oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0d0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`0M0?oo
o`030000003oool0oooo02l0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0`0?oo
o`030000003oool0oooo01l0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?oo
o`00>P3oool00`3ooml0W4P0000000070000000304RLg`3oool0oooo00<0oooo00<0_gA809cOo`3o
ool0303oool00`3ooml0W4P004RLg`090?ooo`030000003oool0oooo0300oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool0<@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3o
ool07P3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0M0?ooo`009P3oool00`3ookl0M7Bo0?ooo`040?ooo`030=nL
W02oool0oooo01X0oooo00<0oooO09a8M02oool0303oool00`3OW4P0000007Boo`090?ooo`@00000
;`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0
;`3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0
803oool00`000000oooo0?ooo`0I0?ooo`<000007`3oool002H0oooo00<0okmd0000000000001000
00000`0007@0_ooo0?ooo`0K0?ooo`030?noM000M;l0oooo00L0oooo00L0gmoO0?ooo`3ookl0M000
0000000007@0_ooo00T0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0E0?ooo`03
0000003oool0oooo03<0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0a0?ooo`03
0000003oool0oooo01d0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0N0?ooo`03
0000003oool0oooo02l0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo01d0oooo000V0?ooo`030;md00000000000000@0000000<0001d0;ooo`3oool0
6`3oool00`3ookl0M01809cOo`070?ooo`060=nLB000000000000000000004P0W=oo2P3oool00`00
0000oooo0?ooo`0_0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0<P3oool00`00
0000oooo0?ooo`0K0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool07P3oool00`00
0000oooo0?ooo`0^0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0;`3oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool002L0
oooo00<0_gAd0;ooo`3oool00`3oool00`3OW9`0_ooo0?ooo`0K0?ooo`030?oo_`1d0000B9cO00L0
oooo00@0oono07@000000000B9cO303oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
0300oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo
01l0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo
01/0oooo00<000000?ooo`3oool07@3oool002H0oooo00<0omnL09boo`3oool07@3oool00`3o_i`0
W=oo0?ooo`020?ooo`030?oOW0180000B9cO00P0oooo00<0_g@007Boo`3oool0303oool00`000000
oooo0?ooo`0^0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0<P3oool00`000000
oooo0?ooo`0K0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool07P3oool00`000000
oooo0?ooo`0^0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0;P3oool00`000000
oooo0?ooo`0Q0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool002H0oooo
00<0_g@0001d_`3oool0103oool00`3OW9`0_ooo0?ooo`0F0?ooo`080?oog`2LB4P0W=oo0?ooo`3o
okl0M0000000M02oool80?ooo`060?oOW018000000000000000004P0W=oo2@3oool00`000000oooo
0?ooo`0^0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0<P3oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool07P3oool00`000000oooo
0?ooo`0^0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0;P3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool002H0oooo00<0
gia8000000000000100000000`0007@0_ooo0?ooo`0G0?ooo`060?oog`2LB00000000000000007@0
_ooo2P3oool01@2oM0000000000000000000B9cO00T0oooo00<000000?ooo`3oool0;P3oool00`00
0000oooo0?ooo`0F0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0703oool00`00
0000oooo0?ooo`0_0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0;@3oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo000V0?ooo`030?oog`2LB000000000@0
000000<0001d0;ooo`3oool0=@3oool3000002d0oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`0a0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0;`3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0803oool00`000000
oooo0?ooo`0^0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool06P3oool2000001l0
oooo000W0?ooo`030=nLW02oool0oooo00<0oooo00<0gibL0;ooo`3oool0=@3oool00`000000oooo
0?ooo`0]0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0<@3oool00`000000oooo
0?ooo`0L0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool07`3oool00`000000oooo
0?ooo`0]0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0;P3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool002H0oooo00<0
okmd04RLg`3oool0?03oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool0<03oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02l0oooo00<0
00000?ooo`3oool07`3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo0200oooo00<0
00000?ooo`3oool0;@3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool07@3oool002H0oooo00<0_g@0001d_`3oool0103oool00`3OW9`0_ooo0?ooo`0e
0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0`
0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0P
0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0]
0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M
0?ooo`009P3oool00`3OW4P0000000000004000000030000M02oool0oooo03D0oooo00<000000?oo
o`3oool0:`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0300oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0200oooo00<000000?oo
o`3oool0;03oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo02d0oooo00<000000?oo
o`3oool08P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo000V0?ooo`03
0?oog`2LB000000000@0000000<0001d0;ooo`3oool0=@3oool00`000000oooo0?ooo`0[0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0N0?ooo`03
0000003oool0oooo02h0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0/0?ooo`03
0000003oool0oooo0240oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0R0?ooo`03
0000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool002d0oooo00<0gibL0;ooo`3oool0
=@3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
;`3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0
803oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0
;03oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
7@3oool002H0oooo00<0oono07Ad_`3oool0103oool00`3OW9`0_ooo0?ooo`0e0?ooo`<00000:P3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool07P3o
ool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0;03o
ool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool08`3o
ool00`000000oooo0?ooo`0J0?ooo`8000007`3oool00200oooo00<0oooO09a80000M;l00`3oool0
0`3o_g@0000000000004000000030000M02oool0oooo03D0oooo00<000000?ooo`3oool0:@3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool07`3oool0
0`000000oooo0?ooo`0]0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0:`3oool0
0`000000oooo0?ooo`0R0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool08`3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo000P0?ooo`030?oog`2LB00007Bo
00<0oooo00<0_g@0000000000000100000000`0007@0_ooo0?ooo`0e0?ooo`030000003oool0oooo
02T0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo
01l0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo
02/0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo
02<0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00;@3oool00`3OW9`0
_ooo0?ooo`0e0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0703oool00`000000
oooo0?ooo`0]0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0;03oool00`000000
oooo0?ooo`0R0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool08P3oool00`000000
oooo0?ooo`0/0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool06`3oool00`000000
oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo01`0oooo
00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo02`0oooo
00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo0280oooo
00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo01/0oooo
00<000000?ooo`3oool07@3oool006D0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0803oool00`000000oooo0?oo
o`0/0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?oo
o`0S0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0903oool00`000000oooo0?oo
o`0K0?ooo`030000003oool0oooo01d0oooo000V0?ooo`090?oog`2LB00000000000M02ooml0W4P0
0000000007@0_ooo03H0oooo0`00000W0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3o
ool0;03oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3o
ool08`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3o
ool0:`3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo01X0oooo0P00000O0?ooo`00
9P3oool0203OW7@0W=oo0?oOW018000007Bo0?ooo`3o_g@0B9cO=`3oool00`000000oooo0?ooo`0V
0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0Z
0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0T
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool002H0oooo00T0gibL0;oo
o`3oool0_g@00000B02Lgol0oooo0;mdM02oool0=P3oool00`000000oooo0?ooo`0V0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo02/0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0Z0?ooo`030000
003oool0oooo02<0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0U0?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool002H0oooo00T0omnL04Qd_`3oool0omnL
04P00000M;l0oooo0;nLW03Oool0=P3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo
01l0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo
02/0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo
02@0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo
01/0oooo00<000000?ooo`3oool07@3oool002H0oooo00D0_g@00000000004P0W=oo0=nLB0020000
000307Boo`3oool0oooo03D0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0P0?oo
o`030000003oool0oooo02/0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0Z0?oo
o`030000003oool0oooo02@0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0T0?oo
o`030000003oool0oooo02X0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3o
ool0:P3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3o
ool0903oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0M0?ooo`009P3oool00`3ookl0M7Bo0?ooo`040?ooo`030=nL
W02oool0oooo03D0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo02X0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0Z0?ooo`030000
003oool0oooo02@0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0T0?ooo`030000
003oool0oooo02X0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0K0?ooo`030000
003oool0oooo01d0oooo000V0?ooo`030?noM0000000000000@0000000<0001d0;ooo`3oool0=@3o
ool300000280oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0Z0?ooo`030000003o
ool0oooo02<0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0U0?ooo`030000003o
ool0oooo02P0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0Y0?ooo`030000003o
ool0oooo02H0oooo00<000000?ooo`3oool06P3oool2000001l0oooo000V0?ooo`030;md00000000
000000@0000000<0001d0;ooo`3oool0=@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo02<0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0
oooo02T0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0
oooo02D0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool07@3oool002L0oooo00<0_gAd0;ooo`3oool00`3oool00`3O
W9`0_ooo0?ooo`0V0?ooo`050?oOW018000000000000M02oool02P3oool00`000000oooo0?ooo`0P
0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0T
0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0X
0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0V
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool002H0oooo00<0omnL09bo
o`3oool0;03oool01`3ookl0M00007Boo`3oool0okmd0000M02oool02@3oool00`000000oooo0?oo
o`0P0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?oo
o`0U0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?oo
o`0X0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?oo
o`0V0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool002H0oooo00<0_g@0
001d_`3oool0103oool00`3OW9`0_ooo0?ooo`0U0?ooo`030;md0000B9`0gooo0080oooo00<0_g@0
001d_`3oool0203oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo02D0oooo00<00000
0?ooo`3oool0:03oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo02P0oooo00<00000
0?ooo`3oool09P3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02H0oooo00<00000
0?ooo`3oool0:03oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo01/0oooo00<00000
0?ooo`3oool07@3oool002H0oooo00<0gia8000000000000100000000`0007@0_ooo0?ooo`0T0?oo
o`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool0203oool00`000000oooo0?ooo`0N0?oo
o`030000003oool0oooo02H0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`0V0?oo
o`030000003oool0oooo02P0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0W0?oo
o`030000003oool0oooo02H0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`0W0?oo
o`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool002H0oooo00<0oooO09a80000
0000100000000`0007@0_ooo0?ooo`0T0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02L
gol0203oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3o
ool09`3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3o
ool09`3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3o
ool0:03oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool07@3oool002L0oooo00<0gibL0;ooo`3oool00`3oool00`3OW9`0_ooo0?ooo`0T0?ooo`030?oO
W0180000M;oo00<0oooo00<0okmd0000B02Lgol0203ooooo00000;T000007`3oool002H0oooo00<0
okmd04RLg`3oool0:`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00P0oooo00<0
00000?ooo`3oool0703oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo02H0oooo00<0
00000?ooo`3oool09`3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo02P0oooo00<0
00000?ooo`3oool09P3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02P0oooo00<0
00000?ooo`3oool09`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo000V
0?ooo`030;md0000M;l0oooo00@0oooo00<0gibL0;ooo`3oool0903oool00`3ogi`0B00007Boo`03
0?ooo`030?noM00004P0W=oo00P0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0Y
0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`0V
0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0W
0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo01d0oooo000V0?ooo`030=nLB0000000000000@0000000<0001d0;oo
o`3oool0903oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00P0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02D0oooo00<000000?oo
o`3oool0:03oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo02P0oooo00<000000?oo
o`3oool09P3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02L0oooo00<000000?oo
o`3oool0:03oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo000V0?ooo`03
0?oog`2LB000000000@0000000<0001d0;ooo`3oool09@3oool00`3OW4P004RL0=ooo`020?ooo`03
0;md0000M;l0oooo00P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0[0?ooo`03
0000003oool0oooo02@0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0U0?ooo`03
0000003oool0oooo02T0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0W0?ooo`03
0000003oool0oooo02L0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo01d0oooo000]0?ooo`030=nLW02oool0oooo02D0oooo00L0oono07@0001d_ol0
oooo0?noM00007@0_ooo00T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0/0?oo
o`030000003oool0oooo02@0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0U0?oo
o`030000003oool0oooo02T0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0X0?oo
o`030000003oool0oooo02L0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo01d0oooo001F0?ooo`050?oo_`1d000000000000M02oool02P3oool00`00
0000oooo0?ooo`0G0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool08`3oool00`00
0000oooo0?ooo`0Z0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0:@3oool00`00
0000oooo0?ooo`0U0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool09P3oool00`00
0000oooo0?ooo`0Y0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool002L0
oooo00L0_g@00000B02Lgol0oono07@000000000B9cO03L0oooo0`00000F0?ooo`030000003oool0
oooo02h0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0
oooo02@0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0
oooo02P0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0
oooo01X0oooo0P00000O0?ooo`009P3oool01@3o_g@000000000B02Lgol0_g@00080000000<0001d
0;ooo`3oool0=@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo02l0oooo00<00000
0?ooo`3oool08`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02@0oooo00<00000
0?ooo`3oool0:P3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo02P0oooo00<00000
0?ooo`3oool09P3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo01/0oooo00<00000
0?ooo`3oool07@3oool002H0oooo00T0_gAd0;ooo`3oool0oono07A8W03Oool0omnL04Q8W03Oool0
=P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0
8P3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0
:P3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0
9P3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
7@3oool002H0oooo00P0_gA809cOo`3oool0giad09cOo`3oool0oknL09cOocL0oooo00<000000?oo
o`3oool04P3oool2000003<0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0[0?oo
o`030000003oool0oooo02<0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0T0?oo
o`030000003oool0oooo02T0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0Y0?oo
o`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool002H0oooo00<0gia800000000
0000100000000`00B9`0gooo0?ooo`0e0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool0<`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3o
ool08`3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3o
ool0:@3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0M0?ooo`009P3oool00`3ookl0M00000000004000000030018
W03Oool0oooo03D0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0d0?ooo`030000
003oool0oooo0240oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0S0?ooo`030000
003oool0oooo02/0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0Z0?ooo`030000
003oool0oooo02D0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0K0?ooo`030000
003oool0oooo01d0oooo000/0?ooo`030;mdB02Lgol0oooo03H0oooo0`00000?0?ooo`030000003o
ool0oooo03D0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0/0?ooo`030000003o
ool0oooo0280oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0S0?ooo`030000003o
ool0oooo02X0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0Z0?ooo`030000003o
ool0oooo01X0oooo0P00000O0?ooo`009P3oool00`2oM7@0_ooo0?ooo`0l0?ooo`030000003oool0
oooo00d0oooo0P00000h0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0;@3oool0
0`000000oooo0?ooo`0R0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool08`3oool0
0`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0:P3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo000T0?ooo`030?oo_`1d00000000
00H0000000<0B9cO0?ooo`3oool0=@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
03P0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo
0280oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo
02/0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo
01/0oooo00<000000?ooo`3oool07@3oool002<0oooo00<0okmd0000000000001`0000000`0007@0
_ooo0?ooo`0e0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0>@3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool08P3oool00`000000
oooo0?ooo`0/0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0:`3oool00`000000
oooo0?ooo`0T0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool06`3oool00`000000
oooo0?ooo`0M0?ooo`009P3oool00`2oM7@0_ooo0?ooo`030?ooo`030?oo_`1dB9`0gooo03H0oooo
00<000000?ooo`3oool02@3oool2000003`0oooo00<000000?ooo`3oool07`3oool00`000000oooo
0?ooo`0^0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0;@3oool00`000000oooo
0?ooo`0R0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0903oool00`000000oooo
0?ooo`0[0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool002H0oooo00<0
_gAd0;ooo`3oool00`3oool00`2oM4P0W=oo0?ooo`0f0?ooo`030000003oool0oooo00P0oooo00<0
00000?ooo`3oool0?03oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo02h0oooo00<0
00000?ooo`3oool08@3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo0240oooo00<0
00000?ooo`3oool0;03oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo02/0oooo00<0
00000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`17
0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0/
0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo01d0oooo000W0?ooo`030?oog`2LB00000000080000000<0M;oo0?oo
o`3oool0=P3oool3000004L0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0_0?oo
o`030000003oool0oooo0200oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo02`0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0/0?oo
o`030000003oool0oooo01X0oooo0P00000O0?ooo`009P3oool00`3ookl0M0000000000400000003
04RLg`3oool0oooo03D0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo0300oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0^0?ooo`03
0000003oool0oooo0200oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0S0?ooo`03
0000003oool0oooo02`0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00
9P3oool00`3OW4P0B9cO0?ooo`020?ooo`040?noM0000000001d0;ooocH0oooo00<000000?ooo`3o
ool0A`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3o
ool0803oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3o
ool0;@3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0M0?ooo`009P3oool00`2oM7@0_ooo0?ooo`030?ooo`030=nL
B00007@0_ooo03H0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0M0?ooo`030000
003oool0oooo0300oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0_0?ooo`030000
003oool0oooo0200oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0R0?ooo`030000
003oool0oooo02d0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`009P3o
ool00`3OW4P0B9cO0?ooo`030?ooo`030;md0000M;l0oooo03H0oooo00<000000?ooo`3oool0A`3o
ool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0;@3o
ool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`0M0?ooo`009P3oool01`3ooml0W4P00000M02oool0gia800000018W=l0
>03oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0
<@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0
7`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0
;@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`030000003o
ool0oooo04L0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0b0?ooo`030000003o
ool0oooo01l0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0O0?ooo`030000003o
ool0oooo02h0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0]0?ooo`030000003o
ool0oooo01/0oooo00<000000?ooo`3oool07@3oool00240oooo00<0oknL09cOo`3oool02@3oool0
0`3OW9`0_ooo0?ooo`0e0?ooo`<00000A`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0
oooo0380oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0
oooo01l0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo02h0oooo00<000000?ooo`3oool06P3oool2000001l0oooo000Q0?ooo`030;md000000000000
00T0000000<0001d0;ooo`3oool0=@3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo
01/0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo
0300oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo
0240oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
01d0oooo000P0?ooo`030?oo_`1d0000000000X0000000<0001d0;ooo`3oool05P3oool00`2oM000
0000000000040000000904RLg`3oool0oooo0?noM0000000M;oo0?ooo`3ooml0W4P000<0000000<0
M;oo0?ooo`3oool02@3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool0<`3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0300oooo00<0
00000?ooo`3oool07P3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo0240oooo00<0
00000?ooo`3oool0;P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo000W
0?ooo`030=nLW02oool0oooo00<0oooo00<0gibL0;ooo`3oool05`3oool00`3OW4P0000000000003
0000000:0018W03Oool0oooo0?noM0000000M;oo0?ooo`3ooml0W4Qd0;ooo`80oooo00<0_g@007Bo
o`3oool02@3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool0<`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0340oooo00<000000?oo
o`3oool07P3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo0240oooo00<000000?oo
o`3oool0;P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo000V0?ooo`03
0?noM018W=l0oooo01l0oooo00<0gia804RLg`3oool00P3oool00`3OW7@0W=oo0?ooo`090?ooo`03
0?oog`2LB7@0_ooo00T0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo03@0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0a0?ooo`03
0000003oool0oooo01h0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0P0?ooo`03
0000003oool0oooo02l0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00
9P3oool00`2oM00007Bo0?ooo`040?ooo`030=nLW02oool0oooo01T0oooo00<0gia807Boo`3oool0
3P3oool00`2oM000M;oo0?ooo`080?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0
<@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0
803oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
7@3oool002H0oooo00<0gia8000000000000100000000`0007@0_ooo0?ooo`0J0?ooo`030;mdB02L
gol0oooo00`0oooo00<0oooO09a80018W=l02@3oool00`000000oooo0?ooo`170?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo0340oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0`0?ooo`030000003o
ool0oooo0200oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0K0?ooo`030000003o
ool0oooo01d0oooo000V0?ooo`030?oog`2LB000000000@0000000<0001d0;ooo`3oool06P3oool0
0`3ooml0W4Qd0;ooo`0<0?ooo`030=nLB0000000M;oo00T0oooo100000160?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0
oooo0380oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0
oooo01l0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`0I0?ooo`<000007`3oool0
02d0oooo00<0gibL0;ooo`3oool06`3oool00`3o_g@007Bo0?ooo`070?ooo`070=oOg`3oool0oono
07@000000000001d0;ooo`090?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0603o
ool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0<P3o
ool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3o
ool004/0oooo00<0oono07@0B02Lgol01`3oool01P3OW4P00000000000000000001809cOo`X0oooo
00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo03H0oooo
00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo01d0oooo
00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0300oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00B`3oool00`3ookl0M00004RL
g`070?ooo`040?oo_`1d0000000004RLg``0oooo00<000000?ooo`3oool0A`3oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool06`3oool00`000000oooo
0?ooo`0c0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0<@3oool00`000000oooo
0?ooo`0O0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool06`3oool00`000000oooo
0?ooo`0M0?ooo`00AP3oool00`3o_i`0W=oo0?ooo`020?ooo`030?oOW0180000B9cO00P0oooo00<0
_g@007Boo`3oool0303oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo01H0oooo00<0
00000?ooo`3oool0=`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo03<0oooo00<0
00000?ooo`3oool0703oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01h0oooo00<0
00000?ooo`3oool0<@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo0016
0?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02oool80?ooo`060?oOW018000000000000
000004P0W=oo2@3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo01H0oooo00<00000
0?ooo`3oool0=`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo03<0oooo00<00000
0?ooo`3oool0703oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01h0oooo00<00000
0?ooo`3oool0<@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo00170?oo
o`060?oog`2LB00000000000000007@0_ooo2P3oool01@2oM0000000000000000000B9cO00T0oooo
00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo03L0oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo01`0oooo
00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0340oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool3000004L0oooo00<0
00000?ooo`3oool05@3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool0=03oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0340oooo00<0
00000?ooo`3oool07P3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01X0oooo0P00
000O0?ooo`00I@3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool0>03oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo03@0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo01d0oooo00<00000
0?ooo`3oool0<P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?oo
o`030000003oool0oooo04L0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0h0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo0380oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0b0?oo
o`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo00<000000?ooo`3o
ool0A`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool0<P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`170?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo03D0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0b0?ooo`030000
003oool0oooo01d0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`0K0?ooo`030000
003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0503o
ool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0=@3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0703o
ool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3o
ool006D0oooo0`0000170?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0>@3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0c0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0<`3oool0
0`000000oooo0?ooo`0J0?ooo`8000007`3oool006D0oooo00<000000?ooo`3oool0A`3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`0e0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0<`3oool00`00
0000oooo0?ooo`0L0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool06`3oool00`00
0000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo03H0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo01`0
oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0
oooo001U0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool04`3oool00`000000oooo
0?ooo`0j0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0=P3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`0c0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo00<0
00000?ooo`3oool0A`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo03/0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`3oool0=03oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo03<0oooo00<0
00000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`17
0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0d
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo04L0oooo00<000000?oo
o`3oool04P3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool0=`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo03@0oooo00<000000?oo
o`3oool06`3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool07@3oool006D0oooo0`0000170?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool0>`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool0=03oool00`000000oooo0?ooo`0J0?ooo`8000007`3oool006D0oooo00<000000?ooo`3oool0
A`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0
5`3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
=@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`170?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0G0?ooo`030000003o
ool0oooo03L0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0e0?ooo`030000003o
ool0oooo01/0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0K0?ooo`030000003o
ool0oooo01d0oooo001U0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`0l0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0=`3oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0e0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool0
06D0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo
03`0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
03D0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000
oooo0?ooo`170?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0?@3oool00`000000
oooo0?ooo`0F0?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0e0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0=@3oool00`000000
oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo04L0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo03H0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo01/0oooo
00<000000?ooo`3oool07@3oool006D0oooo0`0000170?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`3oool0?@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo03P0oooo00<0
00000?ooo`3oool05`3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool0=@3oool00`000000oooo0?ooo`0J0?ooo`8000007`3oool006D0oooo00<00000
0?ooo`3oool0A`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo03d0oooo00<00000
0?ooo`3oool05@3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo01L0oooo00<00000
0?ooo`3oool0=P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo03D0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00E03oool00`3ooml0W4P0000000020000
000307Boo`3oool0oooo00T0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0?0?oo
o`030000003oool0oooo03h0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0i0?oo
o`030000003oool0oooo01L0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo03H0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?oo
o`00E03oool00`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0oooo00T0oooo00<000000?ooo`3o
ool0A`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3o
ool05@3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3o
ool0=P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0M0?ooo`00F@3oool00`3ooml0W4Qd0;ooo`090?ooo`030000
003oool0oooo04L0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0n0?ooo`030000
003oool0oooo01D0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0G0?ooo`030000
003oool0oooo03H0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0f0?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool005X0oooo00<0_g@007Boo`3oool0203o
ool00`000000oooo0?ooo`170?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0?P3o
ool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool05P3o
ool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0=P3o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001I0?ooo`030?oog`2LB000
B9cO00T0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo03l0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0j0?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0
oooo03H0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00F@3oool00`3O
W4P0000007Boo`090?ooo`@00000AP3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
03l0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
03H0oooo00<000000?ooo`3oool06@3oool3000001l0oooo001E0?ooo`070=oOg`3oool0oono07@0
00000000001d0;ooo`090?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0o0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0>P3oool0
0`000000oooo0?ooo`0F0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0g0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool0
05D0oooo00H0gia800000000000000000000B02Lgol:0?ooo`030000003oool0oooo04L0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo01@0oooo00<0
00000?ooo`3oool0>P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo03L0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool07@3oool005D0oooo00@0oono07@000000000B9cO303oool00`000000oooo0?oo
o`170?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?oo
o`0h0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?oo
o`0K0?ooo`030000003oool0oooo01d0oooo001F0?ooo`030;md001d_ol0oooo00`0oooo00<00000
0?ooo`3oool0A`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0400oooo00<00000
0?ooo`3oool04`3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool0>03oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo03L0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00EP3oool01P3ogi`0B000000000000000
001809cOo`T0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo0400oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0k0?ooo`030000003o
ool0oooo01D0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo03L0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00E`3oool0
1@2oM0000000000000000000B9cO00T0oooo0`0000170?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool0@03oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo03/0oooo00<0
00000?ooo`3oool05@3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool0=`3oool00`000000oooo0?ooo`0J0?ooo`8000007`3oool006D0oooo00<00000
0?ooo`3oool0A`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0400oooo00<00000
0?ooo`3oool04`3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool0>03oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo03P0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`170?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo03/0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0h0?oo
o`030000003oool0oooo01L0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool0?03oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3o
ool05`3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool07@3oool006D0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo0440oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0l0?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0G0?ooo`030000
003oool0oooo03P0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3o
ool00`000000oooo0?ooo`170?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0@@3o
ool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0503o
ool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0>03o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`030000003oool0
oooo04L0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`110?ooo`030000003oool0
oooo0180oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo03T0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo0`0000170?ooo`030000003oool0oooo
00`0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
03`0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0J0?ooo`8000007`3oool006D0
oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0440
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo03T0
oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo
0?ooo`170?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0@@3oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool04`3oool00`000000oooo
0?ooo`0j0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0>@3oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo04L0oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool0?@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo03X0oooo00<0
00000?ooo`3oool05P3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool07@3oool006D0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0m
0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0F
0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M
0?ooo`00I@3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool0@P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo03d0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo01H0oooo00<000000?oo
o`3oool0>@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`03
0000003oool0oooo04L0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`120?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0oooo03X0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0j0?ooo`03
0000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo0`0000170?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo03h0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0j0?ooo`030000
003oool0oooo01D0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0J0?ooo`800000
7`3oool006D0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo0480oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0n0?ooo`030000003o
ool0oooo01<0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0E0?ooo`030000003o
ool0oooo03X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool0
0`000000oooo0?ooo`170?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0@P3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool04`3oool0
0`000000oooo0?ooo`0j0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0>P3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo
04L0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
03/0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo
01/0oooo00<000000?ooo`3oool07@3oool006D0oooo00<000000?ooo`3oool0A`3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`0o0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0>`3oool00`000000
oooo0?ooo`0E0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool06`3oool00`000000
oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo03l0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo01D0oooo
00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo
001U0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`130?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?oo
o`0B0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0503oool00`000000oooo0?oo
o`0k0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo0`000017
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0k
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`0J
0?ooo`8000007`3oool006D0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo04<0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0o0?oo
o`030000003oool0oooo0180oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`0D0?oo
o`030000003oool0oooo03/0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?oo
o`00B@3oool00`3OW4P0B9cO0?ooo`040?ooo`060?noM0000000M;oo0?ooo`3ooml0W4P00`000000
0`1d_ol0oooo0?ooo`090?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`130?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0?`3oool0
0`000000oooo0?ooo`0B0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`0k0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool0
04T0oooo00<0omnL04Q8W03Oool0103oool01`3o_g@0000007Boo`3oool0oooO09a8M02oool00P3o
ool00`2oM000M;oo0?ooo`090?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`130?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0?`3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0503o
ool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3o
ool004T0oooo00<0oooO09a8B02Lgol03@3oool00`3ooml0W4Qd0;ooo`090?ooo`030000003oool0
oooo04L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`130?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo03`0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0k0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool07@3oool004X0oooo00<0gia804RLg`3oool03@3oool00`2o
M000M;oo0?ooo`080?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`140?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0@03oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`0k0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool004X0
oooo00<0omnL04Q8W03Oool0303oool00`3ooml0W4P004RLg`090?ooo`030000003oool0oooo04L0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo03`0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo01/0
oooo00<000000?ooo`3oool07@3oool004X0oooo00<0oooO09a8B02Lgol0303oool00`3OW4P00000
07Boo`090?ooo`@00000AP3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo04@0oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo0140oooo
00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo03`0oooo
00<000000?ooo`3oool06@3oool3000001l0oooo001;0?ooo`030=nLB000M;l0oooo00L0oooo00L0
gmoO0?ooo`3ookl0M0000000000007@0_ooo00T0oooo00<000000?ooo`3oool0A`3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`100?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0?03oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool06`3oool00`000000
oooo0?ooo`0M0?ooo`00B`3oool00`3ookl0M01d0;ooo`070?ooo`060=nLB0000000000000000000
04P0W=oo2P3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo00T0oooo00<000000?oo
o`3oool0A03oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0400oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo01<0oooo00<000000?oo
o`3oool0?03oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo00160?ooo`03
0?oOg`3Oool0oooo00<0oooo00<0_g@007Boo`3oool01P3oool0103ookl0M00000000018W=l<0?oo
o`030000003oool0oooo04L0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`140?oo
o`030000003oool0oooo00h0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo03`0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0l0?oo
o`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool004H0oooo00<0oono07A8W03O
ool00`3oool00`3o_g@007Bo0?ooo`070?ooo`030;md001d_ol0oooo00`0oooo00<000000?ooo`3o
ool0A`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool0?@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0M0?ooo`00A`3oool00`2oM000000000000003000000030000
M02oool0oooo00H0oooo00H0omnL04P00000000000000000B02Lgol90?ooo`030000003oool0oooo
04L0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
03d0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo
01/0oooo00<000000?ooo`3oool07@3oool004L0oooo00<0okmd000000000000100000000`1d_ol0
oooo0?ooo`060?ooo`050;md00000000000000000018W=l02@3oool3000004L0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool0@@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo03d0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo01X0oooo0P00000O0?oo
o`00I@3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3o
ool0A@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3o
ool0403oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3o
ool0?03oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`030000
003oool0oooo04L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`150?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo03d0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0m0?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo00<000000?ooo`3oool0A`3o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`110?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0?@3o
ool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`170?ooo`030000003oool0
oooo00P0oooo00<000000?ooo`3oool0A@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo0440oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0m0?ooo`030000003oool0
oooo0180oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0
oooo01d0oooo001U0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0203oool00`00
0000oooo0?ooo`150?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0@@3oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`0m0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool006D0
oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo04D0
oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo03d0
oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool3000004L0oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo03h0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo01X0oooo
0P00000O0?ooo`00I@3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo00P0oooo00<0
00000?ooo`3oool0A@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0440oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool0?@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo001U
0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`15
0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0m
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo00<000000?oo
o`3oool0A`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo04D0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool0?P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo03d0oooo00<000000?oo
o`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?ooo`170?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo0480oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0o0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`160?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
@P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
7@3oool006D0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo04H0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`120?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo03h0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool3
000004L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`160?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo03l0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0
oooo01X0oooo0P00000O0?ooo`00I@3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
0480oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
01d0oooo001U0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`160?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0@P3oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0n0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool006D0oooo
00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo04H0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo03h0oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?ooo`00I@3oool00`000000oooo0?oo
o`170?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`100?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?oo
o`0K0?ooo`030000003oool0oooo01d0oooo001U0?ooo`030000003oool0oooo04L0oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`160?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool0@`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0400oooo00<00000
0?ooo`3oool04@3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo01/0oooo00<00000
0?ooo`3oool07@3oool006D0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo04H0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`130?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo03h0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?oo
o`00I@3oool3000004L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`160?ooo`03
0000003oool0oooo00/0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo0400oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0n0?ooo`03
0000003oool0oooo01X0oooo0P00000O0?ooo`00I@3oool00`000000oooo0?ooo`170?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo04<0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`100?ooo`030000
003oool0oooo0100oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0K0?ooo`030000
003oool0oooo01d0oooo001=0?ooo`060;md00000000000000000000B9`0gooo0`3oool01@3ogi`0
B0000000000007@0_ooo00X0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo04L0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`130?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo03l0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0M0?oo
o`00CP3oool00`3ogi`0B00007Boo`040?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`09
0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`17
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0o
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool004h0oooo00<0omnL04P0
001d_ol0103oool00`2oM00004RL0=ooo`020?ooo`030;md0000M;l0oooo00P0oooo00<000000?oo
o`3oool0203oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool0203oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo0100oooo00<000000?oo
o`000000?`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool07@3oool004h0oooo00<0omnL04P0001d_ol00`3oool00`3ookl0M00004RLg`030?ooo`03
0=nLB00007@0_ooo00P0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00P0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo0140oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0000001@0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0B0?ooo`80
00004`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01d0oooo001>0?ooo`030?oO
W0180000M;oo00<0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`080?ooool00000
^@00000O0?ooo`00CP3oool00`3ogi`0B00007Boo`030?ooo`030?oOW0180000M;oo00<0oooo00<0
okmd0000B02Lgol0o`3ooooP0?ooo`00CP3oool00`3ogi`0B00007Boo`030?ooo`030?oOW0180000
M;oo00<0oooo00<0okmd0000B02Lgol0o`3ooooP0?ooo`00CP3oool00`3ogi`0B00007Boo`030?oo
o`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0o`3ooooP0?ooo`00CP3oool00`3ogi`0
B00007Boo`030?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool0o`3ooooP0?ooo`00
CP3oool00`3ogi`0B00007Boo`040?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d_`3oool0
o`3ooooP0?ooo`00C03oool01@3ooml0W4P0000000000000M;oo00@0oooo00L0oono07@0001d_ol0
oooo0?noM00007@0_ooo0?l0ooooh@3oool004h0oooo00<0okmd0000001d_ol01@3oool01@3ookl0
M0000000000007@0_ooo0?l0oooohP3oool00?l0ooooo`3oooln0?ooo`00o`3ooolE0?ooo`030?oo
g`2LB000000000<0000000<0B9cO0?ooo`3oool0o`3ooolP0?ooo`00o`3ooolE0?ooo`030;md0000
B9`0gooo00<0oooo00<0gibL0;ooo`3oool0o`3ooolP0?ooo`00o`3ooolE0?ooo`030?oog`2LM9`0
gooo00<0oooo00<0oono09bLg`3oool0o`3ooolP0?ooo`00o`3ooolF0?ooo`030;md000000000000
00<0000000<0B9cO0?ooo`3oool0o`3ooolO0?ooo`00f`3oool01P3OW4P000000000000004P0W=oO
09a80080000000<0001d0;oog`2LB0000P0000000`00M;l0oooO09a80002000000090018W02LM000
0000001d_`3oool0omnL04P000000000B9cO0080oooo00L0_g@0000000000000001809cOo`3oool0
gia80080000000<0001809cOg`2LB0000P0000001P00M;l0gia8000000000000001d07Ad00800000
00<0001d0;oog`2LB0000P0000000`00B9`0gono07@00004000000030000M02oool0oooo00L0oooo
00D0_g@0000000000000001809cOo`030?ooo`050=nLB000000000000000001d_ol00P3oool01P2o
M00000000000000007@0_ooO09a80080000000@004RL0=ooo`3ooml0W4P00P0000001@0004P0B4P0
0018W03Oool0gia80080000000L0001d0;ooo`3ogi`0B00000000018W=l0gia80080000000@0001d
0;ooo`3oool0gia80`000000101d_ol0oooo0?ooo`2oM002000000030000M02ooml0W4P000800000
00<004RL0=ooo`3oool0103oool00`3ogi`0B000000000020000000407Boo`3oool0oooo0=nLB0<0
000000@0M;oo0?ooo`3oool0_g@00P0000000`0004P0W=oo0?ooo`050?ooo`060=nLB00000000000
0000B02Lgml0W4P00P0000000`0007@0_ooO09a8000200000003001d_`3oool0oooo00`0oooo00H0
_g@000000000000000000018W03OoonF0?ooo`00g03oool00`3OW4P0001d0;ooo`020?ooo`0U0;md
0000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd0000001d_ol0oooo0?oOW018000007Bo0?no
_`2oool0gia80000M02oool0gkno0=ooo`2oM00000000018W03Oool0oooO09a8M02oool0oooo0=nL
B00007@0_ooo0?ooo`3ooml0W4P0001d_`020?ooo`0;0=nLB00007@0_ooo0?ooo`3OW4P004RL0=oo
o`3oool0oooO09a80000M;l00P3oool00`2oM4P0W=oo0?ooo`0;0?ooo`0?0;md0000000004RL0=oo
o`3ooml0W4Qd0;ooo`3oool0gia800000018W=l0oooo0?oog`2LB000B9cO0080oooo02H0gia80018
W03Oool0oooo0?oog`2LB00007Bo0?ooo`3ooml0W4P00000001d_ol0oooo0?oo_`1d000000000000
M02oool0gia80000M02oool0oooo0=nLB00007@0_ooo0=no_`3Oool0gia80000M02oool0oooo0=nL
B0000000B9cO0?ooo`3ooml0W4P004RLg`80oooo00L0gia80018W03Oool0oooo0?oog`2LB00007Bo
00P0oooo00<0oooO09a80000M;l00`3oool01`3OW4P0000004RLg`3oool0oooO09a80018W=l00P3o
ool00`3OW4P0001d0;ooo`080?ooo`030=nLB00007@0_ooo0080oooo00L0_g@00018W03Oool0oooo
0?oog`2LB00007Bo0100oooo00<0omnL04P0001d_ol0V03oool00=`0oooo00<0gia80000M02oool0
0P3oool03`2oM00004RL0=ooo`3oool0oooO09a80000M;l0oooo0?noM00004P0W=oo0?ooo`3ooml0
W4P0001d_`020?ooo`070=nLB000B9`0gooo0?ooo`3ogi`0B000001d_`030?ooo`0:0?oOW02L_ol0
oooo0=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`80oooo00/0gia80000M02oool0oooo0=nLB000
B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`050?oo_`1d0000000000000000M;l0203oool00`3o
gi`0B000001d_`030?ooo`050?oOW02L_ol0omnL04P00000M;l00`3oool03`3o_g@0001d0;ooo`3o
ool0gia80018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3ogi`0B00004RLg`020?ooo`030?oog`2L
B00007Bo0080oooo00L0gia80000M02oool0oooo0=nLB000B9`0gooo0080oooo00H0gia80000M02o
ool0omnL04P00000M;l30?ooo`0;0?noM00007@0_ooo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a8
0000M;l0203oool00`3ooml0W4P0001d_`020?ooo`030?oOW018000007Bo00<0oooo00L0okmd0000
M02oool0oooo0=nLB00007@0_ooo00P0oooo00<0gia80000M02oool00P3oool02P2oM00004RL0=oo
o`3oool0oooO09a80000M;l0oooo0?oog`2LB0080000000304RLg`3oool0oooo0080oooo00<0omnL
04P0001d_ol0V03oool00=`0oooo00<0gia80000M02oool00P3oool01`2oM00004RL0=ooo`3oool0
oooO09a80000M;l00P3oool01P3o_g@00000001d_`3ooml0W4P0001d_`80oooo00L0gia80018W03O
ool0oooo0?noM00004P0W=oo00H0oooo00L0gia80000M02oool0oooo0?oog`2LB00007Bo0080oooo
00/0gia80000M02oool0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`060=nLB000
07@0_ooo0?oo_`1d0000B9cO1`3oool00`3o_g@0001809cOo`050?ooo`030?noM0000000M;oo00<0
oooo00l0okmd0000001d_ol0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd0000
001d_ol00P3oool00`3ooml0W4P0001d_`020?ooo`070=nLB00007@0_ooo0?ooo`3OW4P004RL0=oo
o`020?ooo`060=nLB00007@0_ooo0?noM0000000M;oo0`3oool02`3o_g@0000007Boo`3oool0gia8
0018W03Oool0oooo0?oog`2LB00007Bo00P0oooo00<0oooO09a80000M;l00P3oool00`3o_g@00000
07Boo`030?ooo`070?noM0000000M;oo0?ooo`3OW4P0001d0;ooo`080?ooo`030=nLB00007@0_ooo
0080oooo00L0_g@00018W03Oool0oooo0?oog`2LB00007Bo0100oooo00<0omnL04P0001d_ol0V03o
ool00=`0oooo00<0gia80000M02oool00P3oool01`2oM00004RL0=ooo`3oool0oooO09a80000M;l0
103oool0103o_g@0000000000000M;l20?ooo`070=nLB000B9`0gooo0?ooo`3o_g@0001d0;ooo`06
0?ooo`070=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`020?ooo`0C0=nLB00007@0_ooo0?ooo`3O
W4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?oo_`1d0000B9cO0?ooo`3ooml0W4P0001d_`07
0?ooo`030?noM00007@0_ooo00D0oooo00<0okmd0000001d_ol00`3oool03`3o_g@0000007Boo`3o
ool0gia80018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3o_g@0000007Boo`020?ooo`030?oog`2L
B00007Bo0080oooo00L0gia80000M02oool0oooo0=nLB000B9`0gooo0080oooo00H0gia80000M02o
ool0okmd0000001d_ol30?ooo`0;0?noM0000000M;oo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a8
0000M;l0203oool00`3ooml0W4P0001d_`020?ooo`030?noM0000000M;oo00<0oooo00L0okmd0000
001d_ol0oooo0=nLB00007@0_ooo00P0oooo00<0gia80000M02oool00P3oool01`2oM00004RL0=oo
o`3oool0oooO09a80000M;l0403oool00`3ogi`0B00007Boo`2H0?ooo`00g03oool00`3OW4P00018
09cOo`020?ooo`0?0;md000007@0_ooo0?ooo`3ooml0W4P0001d_`3oool0oono07@0001d_ol0oooo
0?oog`2LB00007Bo0080oooo00L0gia80018W03Oool0oooo0?oOW01804P0W=oo0080oooo00/0okmd
0000B02Lgol0oooo0=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`020?ooo`0C0=nLB00007@0_ooo
0?ooo`2oM000001d0;ooo`3oool0oooO09a80000M;l0oooo0?oo_`1d0000B9cO0?ooo`3ooml0W4P0
04RLg`070?ooo`030?oOW01804P0W=oo0080oooo00H0okmd0000B02Lgol0oono07@0001d_ol30?oo
o`0?0;md000004P0W=oo0?ooo`2oM000001d0;ooo`3oool0oooO09a80000M;l0oooo0?oo_`1d0000
B9cO0080oooo00<0oooO09a80000M;l00P3oool01`3OW4P0001d0;ooo`3oool0gia80018W03Oool0
0P3oool01P3OW4P0001d0;ooo`3ookl0M00007Boo`<0oooo00/0_g@00000B02Lgol0oooo0;md0000
07@0_ooo0?ooo`3ooml0W4P0001d_`080?ooo`030?oog`2LB00007Bo0080oooo00<0oono07@0001d
_ol00`3oool01`2oM000001809cOo`3oool0gia80000B02Lgol0203oool00`3OW4P0001809cOo`02
0?ooo`070;md000007@0_ooo0?ooo`3ooml0W4P0001d_`0@0?ooo`030?oOW0180000M;oo09P0oooo
003J0?ooo`0F0?oog`2LB00000000000B018B4P0W=oo0?oo_`1d000007Ad07@0M02oool0oono07@0
0018W=l0oooo0?oo_`1d0000B9cO0?ooo`3ookl0M000001d_`80oooo00<0gia80018W03Oool00P3o
ool0702oM00004RL0=ooo`3ooml0W4P00000M02oool0oooo0=nLB0000000001809cOo`3ookl0M000
04RLg`3ooml0W4P00000000007@0_ooO09a80000000004Q804P0B02Lgol0oono07@00018W=l20?oo
o`040;md0000M;l0oooo0?noM080000000<004RL0=ooo`3oool01@3oool0602oM00004RL0=ooo`3o
oml0W4P00000M02oool0oooo0=nLB000B9`0gooo0?ooo`3o_g@00000001d_`3ooml0W4P000000000
B4P0B01809cOo`3ookl0M00004RLg`80oooo02H0okmd0000B02Lgol0oooo0?noM000000007Bo0?oo
g`2LB00000000000M02oool0oooo0=nLB000B9`0gooo0?oog`2LB00000000000M02oool0oooo0=nL
B000B9`0gooo0?ooo`3o_g@00000001d_`3ooml0W4P000000000B4P0B01809cOo`3ookl0M00004RL
g`P0oooo00<0oooO09a80000M;l00`3oool03`3OW4P004RL0=ooo`3oool0okmd00000000M;l0oooO
09a800000000001804QdW03O_g@0001d0;ooo`030?ooo`0A0?oog`2LB00000000000B018B4P0W=oo
0?oo_`1d000007Ad07@0M02oool0oono07@00018W=l0oooo0?oog`2LB000200000000`18W=l0oooo
0?ooo`020?ooo`030?oOW0180000M;oo09P0oooo003L0?ooo`0<0;md000007@0_ooO09a800000000
001809cOo`3ookl0M0000000000007@0_ooo0P3oool0203ookl0M000000000000000001d0;ooo`3o
ool0_g@00`0000001000B9`0gooo0?ooo`3OW4P20000000;0000B02Lgol0oooo0?ooo`3OW4P0001d
0;oog`2LB00000000000M02oool00P3oool02`2oM000001d0;ooo`3oool0_g@00018W03Ooml0W4P0
0000000007@0_ooo0080oooo00D0oooO09a800000000000007Boo`0:0?ooo`050=nLB00000000000
0000B02Lgol00`3oool01@3o_g@00000000000000000B9cO00<0oooo00L0_g@00018W03Ooml0W4P0
0000000007@0_ooo00<0oooo00<0okmd0000000000000P0000001`00M;l0oooo0?ooo`2oM000001d
0;ooo`2oM0000`0000001@00B9`0gooo0;md000007@0_ooo0080oooo00D0okmd0000000000000000
04RLg`030?ooo`070;md0000B9`0gooO09a800000000001d0;ooo`070?ooo`030?noM00000000000
0080000000@004RL0=ooo`3oool0okmd0`0000002P18W=l0oooo0?ooo`3oool0_g@00000M02ogi`0
B0000000M02oool50?ooo`0<0;md000007@0_ooO09a800000000001809cOo`3ookl0M00000000000
07@0_ooo403oool00`3ogi`0B00007Boo`2H0?ooo`00lP3oool00`2oM00004RL0=ooo`0:0?ooo`03
0=nLB00007@0_ooo03l0oooo00<0oooO09a80000M;l01P3oool00`2oM00004RL0=ooo`0N0?ooo`03
0?oog`2LB00007Bo03<0oooo00<0omnL04P0001d_ol0V03oool00?80oooo00<0oono07A8W03Oool0
2P3oool00`3OW4P0001d0;ooo`0o0?ooo`030?oog`2LB00007Bo00H0oooo00<0oono07A8W03Oool0
7`3oool00`2oM00007Bo0?ooo`0b0?ooo`030?oOW0180000M;oo09P0oooo003b0?ooo`030?oog`2L
M9`0gooo00X0oooo00<0gia80000M02oool0?`3oool00`3ooml0W4P0001d_`060?ooo`030?oog`2L
M9`0gooo01l0oooo00<0gia8001d_`3oool0<03oool01@3ooml0W4P0000000000000M;oo09P0oooo
003m0?ooo`050?oog`2LB00000000000M02oool01P3oool00`3OW4P0001809cOo`0e0?ooo`040?oo
_`1d00000000001d_`80oooo00<0gia80000B02Lgol01P3oool00`3OW4P0001809cOo`0J0?ooo`07
0?oo_`1d07@0_ooo0?oog`2LB00004RL0=ooo`0^0?ooo`030?noM0000000M;oo09P0oooo003o0?oo
o`030;md000007@0_ooo00H0oooo00<0gia80000B02Lgol0=P3oool00`3ooml0W4P0001d_`020?oo
o`030=nLB00004P0W=oo00H0oooo00<0gia80000B02Lgol06`3oool01@3ookl0M0000000000004P0
W=oo0<X0oooo003o0?ooool0oooo?P3oool00?l0ooooo`3oooln0?ooo`00o`3ooooo0?oooch0oooo
003o0?ooool0oooo?P3oool00001\
\>"],
  ImageRangeCache->{{{0, 571.438}, {352.813, 0}} -> {-2.68707, -13.8747, \
0.0358864, 0.110601}}]
}, Closed]],

Cell[TextData[{
  "Although we matched at the center above, that point is not suitable for ",
  StyleBox["FindRoot", "InlineInput",
    FontWeight->"Bold"],
  " because the singularity for odd solutions makes the secant method \
unstable.  For construction of the wave functions it is better to use a small \
value of ",
  StyleBox["x\[NotEqual] ",
    FontSlant->"Italic"],
  "0 where a wave function node is unlikely.  The lowest four eigenfunctions \
are displayed below using this method and are compared with exact \
wavefunctions.  Note that the exact wave functions employ the parameter ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha] = \@\(m\ k\)\)]],
  " and differ in phase from the numerical wave functions by ",
  Cell[BoxData[
      \(TraditionalForm\`\((-)\)\^n\)]],
  "."
}], "Text"],

Cell[BoxData[
    \(\(\[Psi][n_, \[Alpha]_] = 
        Function[
          x, \(\@\(\[Alpha]\/\(\(2\^n\) \(n!\) \@\[Pi]\)\)\) 
            Exp[\(-\[Alpha]\^2\)\ x\^2/2] 
            HermiteH[n, \[Alpha]\ x]];\)\)], "Input"],

Cell[BoxData[
    \(\(solutions = 
        wavefunction[x] /. 
          Table[ShootBSWF2[ho, 1, n + 0.4, n + 0.6, \(-5\), 5, 0.5, 
              MaxIterations \[Rule] 50], {n, 0, 3}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(DisplayTogether[
        Plot[Evaluate[solutions], {x, \(-5\), 5}, Frame \[Rule] True, 
          FrameLabel \[Rule] {\*"\"\<\!\(\@\(m\\\ k\)\)x\>\"", \*"\"\<\!\(\
\[Psi]\_n\)\>\""}, PlotLabel \[Rule] "\<points: exact, curves: numerical\>"], 
        ListPlot[
          Flatten[Table[{x, \(\((\(-1\))\)\^n\) \(\[Psi][n, 1]\)[
                    x]}, {x, \(-5\), 5, 0.1}, {n, 0, 3}], 1], 
          PlotStyle \[Rule] AbsolutePointSize[2]]];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0952381 0.286484 0.421813 [
[.11905 -0.0125 -9.15625 -15.6875 ]
[.11905 -0.0125 9.15625 0 ]
[.30952 -0.0125 -9.15625 -15.6875 ]
[.30952 -0.0125 9.15625 0 ]
[.5 -0.0125 -5 -15.6875 ]
[.5 -0.0125 5 0 ]
[.69048 -0.0125 -5 -15.6875 ]
[.69048 -0.0125 5 0 ]
[.88095 -0.0125 -5 -15.6875 ]
[.88095 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -20.0938 -32.625 ]
[.5 -0.0125 20.0938 -15.6875 ]
[-0.0125 .0334 -27.3125 -7.84375 ]
[-0.0125 .0334 0 7.84375 ]
[-0.0125 .11776 -27.3125 -7.84375 ]
[-0.0125 .11776 0 7.84375 ]
[-0.0125 .20212 -27.3125 -7.84375 ]
[-0.0125 .20212 0 7.84375 ]
[-0.0125 .28648 -10 -7.84375 ]
[-0.0125 .28648 0 7.84375 ]
[-0.0125 .37085 -19 -7.84375 ]
[-0.0125 .37085 0 7.84375 ]
[-0.0125 .45521 -19 -7.84375 ]
[-0.0125 .45521 0 7.84375 ]
[-0.0125 .53957 -19 -7.84375 ]
[-0.0125 .53957 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -43 -8.0625 ]
[-0.01875 .30902 -27.3125 8.0625 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -77.9688 0 ]
[.5 .63053 77.9688 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.11905 0 m
.11905 .00625 L
s
gsave
.11905 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.30952 0 m
.30952 .00625 L
s
gsave
.30952 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.5 0 m
.5 .00625 L
s
gsave
.5 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.69048 0 m
.69048 .00625 L
s
gsave
.69048 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.88095 0 m
.88095 .00625 L
s
gsave
.88095 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.125 Mabswid
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.07143 0 m
.07143 .00375 L
s
.02381 0 m
.02381 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -81.0938 -36.625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.9375 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 17.063 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
0.000 0.000 0.000 setrgbcolor
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
63.000 9.875 moveto
(\\217) show
72.750 9.875 moveto
(!) show
75.188 9.875 moveto
(!) show
77.625 9.875 moveto
(!) show
80.063 9.875 moveto
(!) show
82.500 9.875 moveto
(!) show
84.938 9.875 moveto
(!) show
87.375 9.875 moveto
(!) show
89.688 9.875 moveto
(!) show
73.375 17.063 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(m) show
85.000 17.063 moveto
(k) show
93.063 17.063 moveto
(x) show
99.188 17.063 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .0334 m
.00625 .0334 L
s
gsave
-0.0125 .0334 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.6) show
1.000 setlinewidth
grestore
0 .11776 m
.00625 .11776 L
s
gsave
-0.0125 .11776 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.4) show
1.000 setlinewidth
grestore
0 .20212 m
.00625 .20212 L
s
gsave
-0.0125 .20212 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
0 .28648 m
.00625 .28648 L
s
gsave
-0.0125 .28648 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
0 .37085 m
.00625 .37085 L
s
gsave
-0.0125 .37085 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
0 .45521 m
.00625 .45521 L
s
gsave
-0.0125 .45521 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.4) show
1.000 setlinewidth
grestore
0 .53957 m
.00625 .53957 L
s
gsave
-0.0125 .53957 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.6) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .05449 m
.00375 .05449 L
s
0 .07558 m
.00375 .07558 L
s
0 .09667 m
.00375 .09667 L
s
0 .13885 m
.00375 .13885 L
s
0 .15994 m
.00375 .15994 L
s
0 .18103 m
.00375 .18103 L
s
0 .22321 m
.00375 .22321 L
s
0 .2443 m
.00375 .2443 L
s
0 .26539 m
.00375 .26539 L
s
0 .30757 m
.00375 .30757 L
s
0 .32867 m
.00375 .32867 L
s
0 .34976 m
.00375 .34976 L
s
0 .39194 m
.00375 .39194 L
s
0 .41303 m
.00375 .41303 L
s
0 .43412 m
.00375 .43412 L
s
0 .4763 m
.00375 .4763 L
s
0 .49739 m
.00375 .49739 L
s
0 .51848 m
.00375 .51848 L
s
0 .01231 m
.00375 .01231 L
s
0 .56066 m
.00375 .56066 L
s
0 .58175 m
.00375 .58175 L
s
0 .60284 m
.00375 .60284 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -104.219 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
69.0625 11.8438 translate
90 rotate
-69.0625 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(y) show
70.438 17.688 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 8.500 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(n) show
75.125 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.11905 .61178 m
.11905 .61803 L
s
.30952 .61178 m
.30952 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.69048 .61178 m
.69048 .61803 L
s
.88095 .61178 m
.88095 .61803 L
s
.125 Mabswid
.16667 .61428 m
.16667 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.2619 .61428 m
.2619 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.7381 .61428 m
.7381 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.07143 .61428 m
.07143 .61803 L
s
.02381 .61428 m
.02381 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.97619 .61428 m
.97619 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .0334 m
1 .0334 L
s
.99375 .11776 m
1 .11776 L
s
.99375 .20212 m
1 .20212 L
s
.99375 .28648 m
1 .28648 L
s
.99375 .37085 m
1 .37085 L
s
.99375 .45521 m
1 .45521 L
s
.99375 .53957 m
1 .53957 L
s
.125 Mabswid
.99625 .05449 m
1 .05449 L
s
.99625 .07558 m
1 .07558 L
s
.99625 .09667 m
1 .09667 L
s
.99625 .13885 m
1 .13885 L
s
.99625 .15994 m
1 .15994 L
s
.99625 .18103 m
1 .18103 L
s
.99625 .22321 m
1 .22321 L
s
.99625 .2443 m
1 .2443 L
s
.99625 .26539 m
1 .26539 L
s
.99625 .30757 m
1 .30757 L
s
.99625 .32867 m
1 .32867 L
s
.99625 .34976 m
1 .34976 L
s
.99625 .39194 m
1 .39194 L
s
.99625 .41303 m
1 .41303 L
s
.99625 .43412 m
1 .43412 L
s
.99625 .4763 m
1 .4763 L
s
.99625 .49739 m
1 .49739 L
s
.99625 .51848 m
1 .51848 L
s
.99625 .01231 m
1 .01231 L
s
.99625 .56066 m
1 .56066 L
s
.99625 .58175 m
1 .58175 L
s
.99625 .60284 m
1 .60284 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .28648 m
1 .28648 L
s
.5 0 m
.5 .61803 L
s
gsave
.5 .63053 -138.969 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(points) show
92.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(:) show
98.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(exact) show
123.813 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(,) show
129.813 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(curves) show
160.938 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(:) show
167.250 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(numerical) show
214.938 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .28648 m
.03279 .28649 L
.04262 .28649 L
.04758 .28649 L
.05288 .28649 L
.06244 .28649 L
.06788 .28649 L
.07379 .2865 L
.07936 .2865 L
.08451 .28651 L
.09425 .28652 L
.09917 .28653 L
.10458 .28654 L
.10978 .28656 L
.11472 .28657 L
.12399 .28661 L
.12917 .28665 L
.13397 .28668 L
.13921 .28673 L
.14487 .28679 L
.15037 .28686 L
.15543 .28694 L
.16539 .28715 L
.17444 .2874 L
.184 .28777 L
.19347 .28827 L
.20364 .28899 L
.21323 .28989 L
.22197 .29095 L
.23188 .29251 L
.24105 .29435 L
.25112 .29691 L
.26181 .30037 L
.27118 .30416 L
.28129 .30916 L
.29956 .32108 L
.31961 .33918 L
.3407 .36471 L
.37881 .42749 L
.41937 .5079 L
.43811 .54301 L
.45842 .57452 L
.46953 .58751 L
.4744 .59208 L
.47977 .59625 L
.48482 .59932 L
.48955 .60142 L
.49216 .60225 L
.49455 .6028 L
.49587 .60302 L
Mistroke
.49656 .60311 L
.4973 .60319 L
.49801 .60325 L
.49866 .60329 L
.49992 .60332 L
.50112 .6033 L
.50225 .60323 L
.50349 .60311 L
.50481 .60292 L
.5062 .60265 L
.50751 .60234 L
.50997 .60159 L
.51439 .59972 L
.5192 .59694 L
.52446 .59304 L
.53003 .58795 L
.54004 .57653 L
.55955 .54707 L
.57784 .51336 L
.61658 .43627 L
.63626 .40033 L
.65777 .36682 L
.67834 .34137 L
.69745 .32343 L
.708 .31567 L
.71794 .30959 L
.73654 .30098 L
.74629 .29767 L
.7566 .29489 L
.76625 .29285 L
.77506 .29138 L
.78452 .29014 L
.79468 .28913 L
.80408 .28842 L
.813 .28791 L
.82264 .2875 L
.83289 .28719 L
.84164 .28699 L
.8513 .28684 L
.85621 .28677 L
.86138 .28672 L
.87062 .28665 L
.87578 .28662 L
.88057 .28659 L
.8858 .28657 L
.89147 .28655 L
.89693 .28654 L
.90198 .28653 L
.9067 .28652 L
.9119 .28651 L
Mistroke
.92092 .2865 L
.92586 .2865 L
.93048 .2865 L
.93545 .28649 L
.94087 .28649 L
.94565 .28649 L
.95075 .28649 L
.96036 .28649 L
.96927 .28649 L
.97619 .28648 L
Mfstroke
.02381 .28648 m
.03279 .28649 L
.04262 .2865 L
.04758 .28651 L
.05288 .28652 L
.06244 .28654 L
.06788 .28655 L
.07379 .28657 L
.07899 .2866 L
.08451 .28663 L
.08947 .28666 L
.09494 .28671 L
.10458 .28682 L
.10978 .2869 L
.11472 .28699 L
.12399 .28721 L
.13397 .28755 L
.13962 .2878 L
.14487 .28808 L
.15543 .28881 L
.16539 .28977 L
.17433 .29091 L
.184 .29253 L
.1935 .29461 L
.20394 .29759 L
.21483 .30165 L
.225 .3065 L
.23545 .31276 L
.24535 .32006 L
.26391 .33791 L
.28382 .36384 L
.30224 .39422 L
.3415 .47319 L
.3612 .51227 L
.36982 .52713 L
.37926 .5408 L
.38381 .54621 L
.38868 .55099 L
.39137 .55316 L
.39382 .55482 L
.39614 .5561 L
.3986 .55714 L
.39988 .55755 L
.40055 .55773 L
.40128 .5579 L
.40256 .55811 L
.40375 .55822 L
.40447 .55825 L
.40525 .55825 L
.40598 .55821 L
Mistroke
.40667 .55815 L
.40796 .55795 L
.40868 .55779 L
.40933 .55762 L
.41184 .55672 L
.41325 .55604 L
.41455 .55529 L
.41946 .55149 L
.42417 .54633 L
.42922 .53913 L
.43832 .52177 L
.44914 .49396 L
.459 .46231 L
.4989 .29168 L
.53728 .12404 L
.5475 .08913 L
.55709 .06206 L
.56709 .0402 L
.5724 .03134 L
.57811 .02395 L
.58052 .0215 L
.58309 .01931 L
.58531 .01777 L
.58771 .01646 L
.58918 .01584 L
.59054 .01539 L
.59183 .01507 L
.59249 .01494 L
.5932 .01484 L
.59445 .01473 L
.59515 .01472 L
.59578 .01472 L
.59702 .01481 L
.59819 .01497 L
.5994 .01523 L
.60074 .0156 L
.60344 .01667 L
.60595 .01801 L
.6083 .01957 L
.61359 .02408 L
.6193 .03037 L
.63932 .06165 L
.66041 .10379 L
.69849 .18005 L
.71971 .21426 L
.73902 .23826 L
.74864 .24775 L
.75908 .25635 L
.77803 .26803 L
.78748 .27227 L
Mistroke
.79788 .27596 L
.80874 .2789 L
.81892 .28097 L
.82931 .28256 L
.83918 .28367 L
.84804 .28442 L
.85771 .28503 L
.86804 .28549 L
.8774 .28579 L
.88259 .28592 L
.8882 .28603 L
.89346 .28612 L
.89837 .28619 L
.90789 .28629 L
.91293 .28632 L
.91833 .28636 L
.92399 .28639 L
.92935 .28641 L
.93413 .28642 L
.9394 .28644 L
.94867 .28645 L
.95375 .28646 L
.95851 .28647 L
.97619 .28648 L
Mfstroke
.02381 .28648 m
.03279 .28652 L
.04262 .28657 L
.04758 .2866 L
.05288 .28663 L
.06244 .28672 L
.06788 .28678 L
.07379 .28687 L
.07899 .28697 L
.08451 .28709 L
.08947 .28723 L
.09494 .28741 L
.10458 .28784 L
.11472 .28847 L
.12399 .28927 L
.13469 .29055 L
.14487 .29223 L
.15445 .29434 L
.16467 .29732 L
.17432 .30097 L
.18306 .30514 L
.20231 .31788 L
.21315 .32765 L
.22313 .33855 L
.24195 .36452 L
.26261 .40106 L
.28376 .44491 L
.30304 .48589 L
.31261 .5045 L
.32302 .52187 L
.32779 .52846 L
.33285 .53431 L
.33717 .53824 L
.33965 .54002 L
.34195 .54133 L
.3443 .54233 L
.34554 .54271 L
.34623 .54288 L
.34686 .54301 L
.34753 .54311 L
.34826 .54319 L
.34958 .54324 L
.35077 .54317 L
.35204 .54299 L
.35314 .54274 L
.35432 .54237 L
.35645 .54144 L
.35898 .53988 L
.36129 .53801 L
.36641 .53235 L
Mistroke
.37199 .52373 L
.37726 .5132 L
.38215 .50137 L
.40271 .43102 L
.42133 .34456 L
.44224 .23721 L
.4514 .19221 L
.46145 .1477 L
.47052 .11384 L
.48023 .08611 L
.4856 .07511 L
.48814 .07107 L
.49049 .068 L
.49285 .0656 L
.49406 .06462 L
.4954 .06375 L
.49657 .06317 L
.49766 .06278 L
.49881 .06253 L
.50005 .06245 L
.50132 .06255 L
.50246 .06282 L
.50378 .06333 L
.50503 .06401 L
.50725 .06568 L
.50965 .06817 L
.5123 .07172 L
.51511 .07638 L
.52013 .08698 L
.53019 .11626 L
.54125 .15902 L
.58164 .35934 L
.60025 .44304 L
.61007 .47852 L
.62051 .50807 L
.62575 .5195 L
.63153 .52947 L
.63406 .53296 L
.63679 .53613 L
.63935 .53856 L
.64173 .54036 L
.64417 .54174 L
.64546 .54229 L
.64684 .54273 L
.64753 .5429 L
.64829 .54305 L
.649 .54315 L
.64965 .5432 L
.6509 .54323 L
.65222 .54314 L
Mistroke
.65336 .54297 L
.65458 .54268 L
.65574 .54232 L
.6568 .54191 L
.65939 .54061 L
.66184 .53899 L
.66688 .53458 L
.67144 .52943 L
.68166 .5145 L
.70006 .47951 L
.74014 .39574 L
.75853 .36376 L
.77871 .33639 L
.79867 .3171 L
.80957 .30939 L
.81974 .30371 L
.8302 .29915 L
.8401 .29582 L
.849 .29351 L
.85866 .29159 L
.86807 .29018 L
.87845 .28904 L
.88367 .28859 L
.88929 .28819 L
.89457 .28788 L
.89947 .28764 L
.90905 .28728 L
.91411 .28713 L
.91952 .287 L
.92521 .28689 L
.93058 .2868 L
.93537 .28674 L
.94063 .28669 L
.94962 .28661 L
.95913 .28656 L
.97619 .28648 L
Mfstroke
.02381 .28648 m
.03279 .28662 L
.04262 .28678 L
.05288 .28701 L
.06244 .28731 L
.06792 .28753 L
.0731 .28778 L
.0828 .2884 L
.09324 .28934 L
.09914 .29003 L
.10458 .29079 L
.11425 .29252 L
.12331 .29468 L
.13324 .29778 L
.14393 .30221 L
.15404 .30767 L
.16506 .31533 L
.1756 .32457 L
.18516 .33479 L
.20483 .36191 L
.2258 .39982 L
.24667 .44423 L
.26587 .48551 L
.27556 .50396 L
.28461 .51827 L
.28955 .52448 L
.29405 .52897 L
.29662 .53098 L
.29906 .53248 L
.3003 .53309 L
.30162 .53362 L
.30231 .53384 L
.30308 .53405 L
.30372 .5342 L
.30442 .53432 L
.30563 .53444 L
.30677 .53445 L
.30746 .5344 L
.30811 .53432 L
.30933 .53409 L
.31063 .5337 L
.31182 .53323 L
.31451 .53171 L
.31665 .53006 L
.31901 .52777 L
.32376 .52161 L
.32896 .5125 L
.33454 .49988 L
.34462 .46975 L
.35475 .43056 L
Mistroke
.3658 .37929 L
.3859 .27268 L
.3969 .21415 L
.40726 .16414 L
.41733 .12379 L
.42175 .10952 L
.42659 .09665 L
.42889 .09164 L
.43134 .08708 L
.43346 .08383 L
.43576 .08104 L
.43711 .07977 L
.43836 .07884 L
.43955 .07816 L
.44082 .07769 L
.44195 .07747 L
.44301 .07744 L
.44418 .07761 L
.44543 .07802 L
.44671 .07869 L
.4479 .07954 L
.45061 .08227 L
.45324 .086 L
.45605 .09111 L
.46115 .10332 L
.46578 .1175 L
.47578 .15719 L
.485 .20252 L
.50581 .31983 L
.51565 .37387 L
.52621 .42456 L
.53132 .44516 L
.53617 .46178 L
.54049 .47398 L
.54295 .47976 L
.54527 .48439 L
.54765 .48834 L
.54984 .49121 L
.55218 .49348 L
.5534 .49434 L
.5547 .49499 L
.55574 .49533 L
.55686 .49551 L
.55791 .49551 L
.55887 .49535 L
.55999 .49499 L
.56121 .49439 L
.56237 .49361 L
.56343 .49271 L
.56579 .49013 L
Mistroke
.568 .48696 L
.57303 .47718 L
.57793 .46436 L
.58321 .44727 L
.60436 .35233 L
.62445 .24422 L
.6445 .14565 L
.65323 .11085 L
.66266 .08053 L
.66795 .06712 L
.6728 .05711 L
.67745 .04961 L
.68235 .04383 L
.68485 .04172 L
.68622 .0408 L
.68753 .04007 L
.68874 .03952 L
.69005 .03906 L
.69125 .03877 L
.69235 .0386 L
.69366 .03853 L
.69488 .03858 L
.69559 .03866 L
.69623 .03877 L
.69693 .03893 L
.69768 .03913 L
.69908 .03962 L
.70038 .04021 L
.70333 .04197 L
.70596 .04402 L
.70838 .04629 L
.71386 .05266 L
.7238 .0679 L
.74239 .10479 L
.76277 .14934 L
.78182 .18791 L
.80127 .22036 L
.81972 .24363 L
.82944 .25307 L
.84003 .26141 L
.84958 .26741 L
.85857 .27191 L
.86892 .27594 L
.87829 .27871 L
.88839 .28097 L
.89929 .28273 L
.90883 .28385 L
.91928 .28472 L
.92455 .28506 L
.93019 .28535 L
Mistroke
.94036 .28575 L
.94919 .28601 L
.95878 .28621 L
.97619 .28648 L
Mfstroke
2 Mabswid
.02381 .28649 Mdot
.02381 .28649 Mdot
.02381 .28653 Mdot
.02381 .28664 Mdot
.03333 .28649 Mdot
.03333 .2865 Mdot
.03333 .28655 Mdot
.03333 .28673 Mdot
.04286 .28649 Mdot
.04286 .28651 Mdot
.04286 .28658 Mdot
.04286 .28686 Mdot
.05238 .28649 Mdot
.05238 .28652 Mdot
.05238 .28664 Mdot
.05238 .28705 Mdot
.0619 .28649 Mdot
.0619 .28654 Mdot
.0619 .28672 Mdot
.0619 .28733 Mdot
.07143 .2865 Mdot
.07143 .28657 Mdot
.07143 .28684 Mdot
.07143 .28772 Mdot
.08095 .2865 Mdot
.08095 .28661 Mdot
.08095 .28701 Mdot
.08095 .28828 Mdot
.09048 .28651 Mdot
.09048 .28667 Mdot
.09048 .28726 Mdot
.09048 .28907 Mdot
.1 .28653 Mdot
.1 .28676 Mdot
.1 .28762 Mdot
.1 .29015 Mdot
.10952 .28656 Mdot
.10952 .2869 Mdot
.10952 .28812 Mdot
.10952 .29162 Mdot
.11905 .28659 Mdot
.11905 .28709 Mdot
.11905 .28881 Mdot
.11905 .2936 Mdot
.12857 .28664 Mdot
.12857 .28735 Mdot
.12857 .28977 Mdot
.12857 .29623 Mdot
.1381 .28672 Mdot
.1381 .28773 Mdot
.1381 .29106 Mdot
.1381 .29965 Mdot
.14762 .28682 Mdot
.14762 .28825 Mdot
.14762 .29278 Mdot
.14762 .30405 Mdot
.15714 .28697 Mdot
.15714 .28896 Mdot
.15714 .29505 Mdot
.15714 .30963 Mdot
.16667 .28718 Mdot
.16667 .28991 Mdot
.16667 .298 Mdot
.16667 .3166 Mdot
.17619 .28746 Mdot
.17619 .29119 Mdot
.17619 .30179 Mdot
.17619 .32513 Mdot
.18571 .28785 Mdot
.18571 .29287 Mdot
.18571 .30659 Mdot
.18571 .33543 Mdot
.19524 .28838 Mdot
.19524 .29505 Mdot
.19524 .31257 Mdot
.19524 .34763 Mdot
.20476 .28908 Mdot
.20476 .29786 Mdot
.20476 .31991 Mdot
.20476 .3618 Mdot
.21429 .29 Mdot
.21429 .30142 Mdot
.21429 .32879 Mdot
.21429 .37793 Mdot
.22381 .29121 Mdot
.22381 .30587 Mdot
.22381 .33937 Mdot
.22381 .39587 Mdot
.23333 .29277 Mdot
.23333 .31138 Mdot
.23333 .35174 Mdot
.23333 .41534 Mdot
.24286 .29476 Mdot
.24286 .31809 Mdot
.24286 .36596 Mdot
.24286 .43588 Mdot
.25238 .29727 Mdot
.25238 .32615 Mdot
.25238 .38199 Mdot
.25238 .45684 Mdot
.2619 .30041 Mdot
.2619 .3357 Mdot
.2619 .39968 Mdot
.2619 .47737 Mdot
.27143 .30427 Mdot
.27143 .34685 Mdot
.27143 .41879 Mdot
.27143 .49645 Mdot
.28095 .30898 Mdot
.28095 .35966 Mdot
.28095 .43888 Mdot
.28095 .51293 Mdot
.29048 .31466 Mdot
.29048 .37414 Mdot
.29048 .4594 Mdot
.29048 .52553 Mdot
.3 .32142 Mdot
.3 .39022 Mdot
.3 .47964 Mdot
.3 .53297 Mdot
.30952 .32936 Mdot
.30952 .40776 Mdot
.30952 .49872 Mdot
.30952 .53405 Mdot
.31905 .3386 Mdot
.31905 .42651 Mdot
.31905 .51568 Mdot
.31905 .52772 Mdot
.32857 .34919 Mdot
.32857 .4461 Mdot
.32857 .52945 Mdot
.32857 .51324 Mdot
.3381 .36118 Mdot
.3381 .46606 Mdot
.3381 .53894 Mdot
.3381 .49029 Mdot
.34762 .37458 Mdot
.34762 .48581 Mdot
.34762 .54312 Mdot
.34762 .459 Mdot
.35714 .38935 Mdot
.35714 .50469 Mdot
.35714 .54105 Mdot
.35714 .4201 Mdot
.36667 .4054 Mdot
.36667 .52192 Mdot
.36667 .53201 Mdot
.36667 .37491 Mdot
.37619 .42258 Mdot
.37619 .5367 Mdot
.37619 .51553 Mdot
.37619 .3253 Mdot
.38571 .4407 Mdot
.38571 .5482 Mdot
.38571 .4915 Mdot
.38571 .27366 Mdot
.39524 .4595 Mdot
.39524 .55563 Mdot
.39524 .46021 Mdot
.39524 .22275 Mdot
.40476 .47865 Mdot
.40476 .55825 Mdot
.40476 .42237 Mdot
.40476 .17554 Mdot
.41429 .49781 Mdot
.41429 .55545 Mdot
.41429 .37913 Mdot
.41429 .13495 Mdot
.42381 .51655 Mdot
.42381 .54678 Mdot
.42381 .33204 Mdot
.42381 .10371 Mdot
.43333 .53447 Mdot
.43333 .53198 Mdot
.43333 .28298 Mdot
.43333 .08403 Mdot
.44286 .55113 Mdot
.44286 .51104 Mdot
.44286 .23409 Mdot
.44286 .07747 Mdot
.45238 .56609 Mdot
.45238 .4842 Mdot
.45238 .18763 Mdot
.45238 .0847 Mdot
.4619 .57896 Mdot
.4619 .45193 Mdot
.4619 .14585 Mdot
.4619 .10547 Mdot
.47143 .58938 Mdot
.47143 .41499 Mdot
.47143 .11086 Mdot
.47143 .13854 Mdot
.48095 .59705 Mdot
.48095 .37432 Mdot
.48095 .08445 Mdot
.48095 .18177 Mdot
.49048 .60174 Mdot
.49048 .33107 Mdot
.49048 .06802 Mdot
.49048 .23224 Mdot
.5 .60332 Mdot
.5 .28648 Mdot
.5 .06245 Mdot
.5 .28648 Mdot
.50952 .60174 Mdot
.50952 .2419 Mdot
.50952 .06802 Mdot
.50952 .34072 Mdot
.51905 .59705 Mdot
.51905 .19864 Mdot
.51905 .08445 Mdot
.51905 .3912 Mdot
.52857 .58938 Mdot
.52857 .15798 Mdot
.52857 .11086 Mdot
.52857 .43443 Mdot
.5381 .57896 Mdot
.5381 .12104 Mdot
.5381 .14585 Mdot
.5381 .4675 Mdot
.54762 .56609 Mdot
.54762 .08877 Mdot
.54762 .18763 Mdot
.54762 .48827 Mdot
.55714 .55113 Mdot
.55714 .06193 Mdot
.55714 .23409 Mdot
.55714 .4955 Mdot
.56667 .53447 Mdot
.56667 .04099 Mdot
.56667 .28298 Mdot
.56667 .48893 Mdot
.57619 .51655 Mdot
.57619 .02619 Mdot
.57619 .33204 Mdot
.57619 .46926 Mdot
.58571 .49781 Mdot
.58571 .01752 Mdot
.58571 .37913 Mdot
.58571 .43802 Mdot
.59524 .47865 Mdot
.59524 .01472 Mdot
.59524 .42237 Mdot
.59524 .39743 Mdot
.60476 .4595 Mdot
.60476 .01734 Mdot
.60476 .46021 Mdot
.60476 .35021 Mdot
.61429 .4407 Mdot
.61429 .02476 Mdot
.61429 .4915 Mdot
.61429 .29931 Mdot
.62381 .42258 Mdot
.62381 .03627 Mdot
.62381 .51553 Mdot
.62381 .24767 Mdot
.63333 .4054 Mdot
.63333 .05105 Mdot
.63333 .53201 Mdot
.63333 .19806 Mdot
.64286 .38935 Mdot
.64286 .06828 Mdot
.64286 .54105 Mdot
.64286 .15286 Mdot
.65238 .37458 Mdot
.65238 .08716 Mdot
.65238 .54312 Mdot
.65238 .11397 Mdot
.6619 .36118 Mdot
.6619 .10691 Mdot
.6619 .53894 Mdot
.6619 .08268 Mdot
.67143 .34919 Mdot
.67143 .12687 Mdot
.67143 .52945 Mdot
.67143 .05972 Mdot
.68095 .3386 Mdot
.68095 .14646 Mdot
.68095 .51568 Mdot
.68095 .04525 Mdot
.69048 .32936 Mdot
.69048 .1652 Mdot
.69048 .49872 Mdot
.69048 .03892 Mdot
.7 .32142 Mdot
.7 .18274 Mdot
.7 .47964 Mdot
.7 .04 Mdot
.70952 .31466 Mdot
.70952 .19883 Mdot
.70952 .4594 Mdot
.70952 .04744 Mdot
.71905 .30898 Mdot
.71905 .21331 Mdot
.71905 .43888 Mdot
.71905 .06004 Mdot
.72857 .30427 Mdot
.72857 .22612 Mdot
.72857 .41879 Mdot
.72857 .07652 Mdot
.7381 .30041 Mdot
.7381 .23727 Mdot
.7381 .39968 Mdot
.7381 .0956 Mdot
.74762 .29727 Mdot
.74762 .24682 Mdot
.74762 .38199 Mdot
.74762 .11613 Mdot
.75714 .29476 Mdot
.75714 .25488 Mdot
.75714 .36596 Mdot
.75714 .13709 Mdot
.76667 .29277 Mdot
.76667 .26159 Mdot
.76667 .35174 Mdot
.76667 .15763 Mdot
.77619 .29121 Mdot
.77619 .2671 Mdot
.77619 .33937 Mdot
.77619 .1771 Mdot
.78571 .29 Mdot
.78571 .27155 Mdot
.78571 .32879 Mdot
.78571 .19504 Mdot
.79524 .28908 Mdot
.79524 .27511 Mdot
.79524 .31991 Mdot
.79524 .21117 Mdot
.80476 .28838 Mdot
.80476 .27792 Mdot
.80476 .31257 Mdot
.80476 .22534 Mdot
.81429 .28785 Mdot
.81429 .2801 Mdot
.81429 .30659 Mdot
.81429 .23753 Mdot
.82381 .28746 Mdot
.82381 .28178 Mdot
.82381 .30179 Mdot
.82381 .24783 Mdot
.83333 .28718 Mdot
.83333 .28305 Mdot
.83333 .298 Mdot
.83333 .25637 Mdot
.84286 .28697 Mdot
.84286 .28401 Mdot
.84286 .29505 Mdot
.84286 .26333 Mdot
.85238 .28682 Mdot
.85238 .28472 Mdot
.85238 .29278 Mdot
.85238 .26891 Mdot
.8619 .28672 Mdot
.8619 .28524 Mdot
.8619 .29106 Mdot
.8619 .27332 Mdot
.87143 .28664 Mdot
.87143 .28561 Mdot
.87143 .28977 Mdot
.87143 .27674 Mdot
.88095 .28659 Mdot
.88095 .28588 Mdot
.88095 .28881 Mdot
.88095 .27937 Mdot
.89048 .28656 Mdot
.89048 .28607 Mdot
.89048 .28812 Mdot
.89048 .28135 Mdot
.9 .28653 Mdot
.9 .28621 Mdot
.9 .28762 Mdot
.9 .28282 Mdot
.90952 .28651 Mdot
.90952 .2863 Mdot
.90952 .28726 Mdot
.90952 .2839 Mdot
.91905 .2865 Mdot
.91905 .28636 Mdot
.91905 .28701 Mdot
.91905 .28469 Mdot
.92857 .2865 Mdot
.92857 .2864 Mdot
.92857 .28684 Mdot
.92857 .28525 Mdot
.9381 .28649 Mdot
.9381 .28643 Mdot
.9381 .28672 Mdot
.9381 .28564 Mdot
.94762 .28649 Mdot
.94762 .28645 Mdot
.94762 .28664 Mdot
.94762 .28592 Mdot
.95714 .28649 Mdot
.95714 .28646 Mdot
.95714 .28658 Mdot
.95714 .28611 Mdot
.96667 .28649 Mdot
.96667 .28647 Mdot
.96667 .28655 Mdot
.96667 .28624 Mdot
.97619 .28649 Mdot
.97619 .28648 Mdot
.97619 .28653 Mdot
.97619 .28632 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{396, 244.5},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40009B0001KR000`400?l00000o`00003oo`3ooooo
0?oooe@0oooo003o0?ooool0ooooE03oool00?l0ooooo`3ooomD0?ooo`00o`3ooooo0?oooe@0oooo
003o0?ooocT0oooo00<0oono0;noo`3oool0o`3ooolH0?ooo`00o`3oooli0?ooo`030=nLB000M;l0
oooo0?l0oooo603oool00?l0oooo>03oool0103ogi`0B0000000M02ooooo0?oooaP0oooo003o0?oo
ocP0oooo00@0_g@0000000000000M;oo1P3oool01P3OW4P000000000000004P0W=oO09a800800000
00<0001d0;oog`2LB0000P0000001@00M;l0oooo0?ooo`3oool0_g@00080000000<0001809cOo`2o
M0000P0000002@0007@0_ooo0?ooo`3oool0_g@000000000M;l0oooo0=nLB002000000030018W03O
ool0oooo0>X0oooo003o0?ooocL0oooo00D0omnL04P0000007@0W9a8001d_`070?ooo`030=nLB000
07@0_ooo0080oooo00L0_g@00018W03Oool0oooo0?oog`2LB00007Bo00D0oooo00L0gia80000M02o
ool0oooo0;md000004P0W=oo00@0oooo00P0oooO09a8M02oool0oooo0?oog`2LB00004RL0=ooon`0
oooo003o0?ooocL0oooo00H0_g@000000018W=l0omnL04P0M02oool60?ooo`030=nLB00007@0_ooo
0080oooo00L0_g@00018W03Oool0oooo0?oog`2LB00007Bo00D0oooo00H0gia80000M02oool0gia8
0000001d_ol60?ooo`060?oo_`1dB9`0gooo0;md000007@0_oook@3oool00?l0oooo=P3oool01`3o
gi`0B0000000M02oool0oooO09a8B02Lgol01P3oool00`3OW4P0001d0;ooo`020?ooo`070;md0000
B9`0gooo0?ooo`3ooml0W4P0001d_`050?ooo`050=nLB00007@0_kmd00000018W=l0203oool0103o
gi`0M4Q80000001d_oo^0?ooo`00o`3ooolf0?ooo`030;md00000000B9cO0080oooo00<0gia804RL
g`3oool01@3oool00`3OW4P0001d0;ooo`020?ooo`070;md0000B9`0gooo0?ooo`3ooml0W4P0001d
_`050?ooo`040=nLB00000000000001d_`T0oooo00<0oono07@00018W=l0k`3oool00?l0oooo<P3o
ool01`3ooml0W4Qd0;ooo`3ogi`0B0000000M02oool00P3oool00`3ogi`0B4RL0=ooo`050?ooo`03
0=nLB00004P0W=oo0080oooo00L0_g@00000M02oool0oooo0?oog`2LB00007Bo00D0oooo00D0gia8
0000M02oool0_g@0001d_`070?ooo`050?oog`2LB00004RL09adB02Lgol0kP3oool00?l0oooo<`3o
ool01@3ookl0M000000000000000B9cO00<0oooo00<0oooO09a8B02Lgol00`3oool03P3ooml0W4P0
0000000004P0B4Q809cOo`3ookl0M000001dM01d07@0_ooo0?oo_`1d0000B9cO1@3oool01`3OW4P0
001d0;ooo`3oool0okmd0018W03Oool01@3oool01P2oM000001809cOo`3ooml0W4Qd0;ooond0oooo
003o0?ooocD0oooo00<0omnL04P0M02oool0103oool00`2oM000B9cO0?ooo`040?ooo`0<0;md0000
07@0_ooO09a800000000001809cOo`3ookl0M0000000000007@0_ooo1@3oool01@3OW4P0001d0;oo
o`3ooml0W4P00080000000D007Bo0?ooo`3oool0oooO09a8000200000005001d_`3o_g@000000000
M02oool0k03oool00?l0oooo?03oool00`3o_g@004RL0=ooo`0E0?ooo`030=nLB00007@0_ooo0?`0
oooo003o0?oooc`0oooo00<0oono07@0B02Lgol05@3oool00`3OW4P0001d0;ooo`3l0?ooo`00o`3o
oolm0?ooo`030;md0018W=l0oooo01@0oooo00<0gia80000M02oool0o03oool00?l0oooo?@3oool0
0`3o_g@007Bo0?ooo`0B0?ooo`050?oog`2LB00000000000M02oool0o03oool00?l0oooo?@3oool0
0`3ookl0M01d0;ooo`0D0?ooo`030;md000007@0_ooo0?`0oooo003o0?oooch0oooo00<0_g@007Bo
o`3oool0o`3ooolC0?ooo`00o`3oooln0?ooo`030?noM000M;l0oooo0?l0oooo4`3oool00?l0oooo
?P3oool00`3ookl0M0000000000M000000030000M02oool0oooo0?80oooo003o0?ooool0ooooE03o
ool00?l0ooooo`3ooomD0?ooo`00o`3ooooo0?oooe@0oooo003o0?ooool0ooooE03oool00?l0oooo
o`3ooomD0?ooo`00o`3ooooo0?oooe@0oooo003o0?ooool0ooooE03oool00?l0ooooo`3ooomD0?oo
o`00o`3ooooo0?oooe@0oooo002T0?ooo`040?oog`2LB00004RL0=ooodl0oooo00<0_g@000000000
0000100000000`18W=l0oooo0?ooo`190?ooo`050?oOW018000000000000M02oool0D03oool00`2o
M0000000000000040000000304RLg`3oool0oooo0540oooo00@0oooO09a80000B9`0goooE03oool0
0:@0oooo00@0oooO09a80000B9`0goooD03oool00`3OW4P0000000000003000000030018W03Oool0
oooo04P0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo0500oooo00<0gia8000000000000
0`0000000`00B9`0gooo0?ooo`1A0?ooo`040?oog`2LB00004RL0=oooe@0oooo002T0?ooo`040?oo
g`2LB00004RL0=oooe40oooo00<0gia804RLg`3oool00P3oool00`3OW7@0W=oo0?ooo`180?ooo`03
0;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool0D03oool00`3OW4P0B9cO0?ooo`020?ooo`03
0=nLM02Lgol0oooo0540oooo00@0oooO09a80000B9`0goooE03oool00:00oooo00<0_g@000000000
00001@0000000`1d_ol0oooo0?ooo`1?0?ooo`030=nLB01d_ol0oooo04/0oooo00<0oono07@00018
W=l00`3oool00`3OW4P0001d0;ooo`1A0?ooo`030=nLB01d_ol0oooo0540oooo00<0_g@000000000
00001@0000000`1d_ol0oooo0?ooo`1A0?ooo`00U03oool00`3ooml0W4P0000000070000000404RL
g`3oool0omnL07BLg`80oooo00@0oooO09a80000B9`0gooo@`3oool00`3ooml0W4P0000000070000
000304RLg`3oool0oooo00<0oooo00<0_gA809cOo`3oool0BP3oool00`3ogi`0B00007Boo`030?oo
o`030?noM00004P0W=oo0580oooo00<0_gA809cOo`3oool0D03oool0203ogi`0M9cO0?ooo`3oool0
oooO09a80000B9`0goooE03oool00:40oooo00L0gia807Boo`3oool0oooO09a80000B9`0gooo05<0
oooo00<0oooO09a8M02oool0BP3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo0580
oooo00<0oooO09a8M02oool0D@3oool01`3OW4P0M;oo0?ooo`3ooml0W4P00018W03Oool0E03oool0
0:40oooo00L0oooO09a8M02oool0oooO09a80000B9`0gooo05@0oooo00<0okmd001d_`3oool0B@3o
ool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo05<0oooo00<0okmd001d_`3oool0D03o
ool01`3ooml0W4Qd0;ooo`3ooml0W4P00018W03Oool0E03oool00:80oooo00H0omnL04Qd_`3ooml0
W4P00018W03OoomD0?ooo`030?oo_`1d04P0W=oo04T0oooo00<0omnL04P0001d_ol00`3oool00`3o
_g@0001809cOo`1C0?ooo`030?oo_`1d04P0W=oo0540oooo00H0omnL04Qd_`3ooml0W4P00018W03O
oomD0?ooo`00X`3oool01@3OW4P0M;oO09a80000B9`0gooo05@0oooo00<0oono07@00018W=l0B@3o
ool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo05<0oooo00<0oono07@00018W=l0DP3o
ool01@3OW4P0M;oO09a80000B9`0gooo05@0oooo002S0?ooo`050?oog`2LB4P0B4P00018W03Oool0
C`3oool00`3o_i`0W=oo0?ooo`020?ooo`030?oOW0180000B9cO04X0oooo00<0gia80018W03Oool0
0P3oool00`2oM00007Bo0?ooo`1>0?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l0
DP3oool01@3ooml0W4Q804Q80000B9`0gooo05@0oooo002T0?ooo`040?oOW018000004RL0=ooodl0
oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooodX0oooo00L0oono07@0001d_ol0oooo
0?noM00007@0_ooo04l0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;oooe<0oooo00@0
omnL04P00000B9`0goooE03oool00:D0oooo00<0gia80018W03Oool0D03oool01P3ooml0W4P00000
00000000001d0;oood`0oooo00D0oono07@000000000001d0;ooo`1A0?ooo`060?oog`2LB0000000
0000000007@0_oooE@3oool00`3OW4P004RL0=ooo`1D0?ooo`00o`3ooooo0?oooe@0oooo003o0?oo
ool0ooooE03oool00?l0ooooo`3ooomD0?ooo`00o`3ooooo0?oooe@0oooo003o0?ooool0ooooE03o
ool00?l0ooooo`3ooomD0?ooo`00o`3ooooo0?oooe@0oooo003o0?ooool0ooooE03oool00?l0oooo
o`3ooomD0?ooo`00o`3ooooo0?oooe@0oooo001X0?ooool00000bP00000Q0?ooo`00J03oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool04`3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool04`3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04`3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04`3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool04`3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo00P0
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0180
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0=03oool00`000000oooo
0?ooo`1D0?ooo`030000003oool0oooo05<0oooo00<000000?ooo`3oool0E03oool00`000000oooo
0?ooo`1D0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool07`3oool006P0oooo00<0
00000?ooo`3oool0h@3oool00`000000oooo0?ooo`3Q0?ooo`030000003oool0oooo01l0oooo001X
0?ooo`030000003oool0oooo0>40oooo00<000000?ooo`3oool0h@3oool00`000000oooo0?ooo`0O
0?ooo`00J03oool300000>40oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`2d0?oo
o`8000008@3oool006P0oooo00<000000?ooo`3oool0h@3oool00`000000oooo0?ooo`0U0?ooo`03
0000003oool0000000@0000000<0oooo0000003oool0/P3oool00`000000oooo0?ooo`0O0?ooo`00
J03oool00`000000oooo0?ooo`3Q0?ooo`030000003oool0oooo02@0oooo0`0000020?ooo`030000
003oool0000000<00000/P3oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?oo
o`3Q0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`0000001`3oool300000;40oooo00<0
00000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0h@3oool00`000000oooo0?ooo`0R
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0[P3oool00`000000oooo0?ooo`0O
0?ooo`00B`3oool01@3ogi`0B0000000000007@0_ooo00<0oooo00<0okmd0000001d_ol00`3oool0
1@3ogi`0B000000000000000M;oo00X0oooo00<000000?ooo`3oool0h@3oool00`000000oooo0?oo
o`0P0?ooo`8000003`3oool200000:h0oooo00<000000?ooo`3oool07`3oool004X0oooo00L0oono
07@0001d_ol0oooo0?noM00007@0_ooo0080oooo00<0okmd0000001d_ol00P3oool01`3ogi`0B000
04RLg`3oool0oono07@0001d_ol02@3oool00`000000oooo0?ooo`3Q0?ooo`030000003oool0oooo
01l0oooo0`00000?0?ooo`<00000[@3oool00`000000oooo0?ooo`0O0?ooo`00BP3oool00`2oM000
04RL0=ooo`020?ooo`030;md0000M;l0oooo00D0oooo00<0oooO09a80000M;l00`3oool00`3o_g@0
04RL0=ooo`080?ooo`030000003oool0oooo0>40oooo00<000000?ooo`3oool07`3oool200000140
oooo0P00002]0?ooo`030000003oool0oooo01l0oooo00190?ooo`030?oo_`1d0000B9cO00<0oooo
00<0gia80000M02oool01@3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00P0oooo
00<000000?ooo`3oool0h@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0180oooo
00<000000?ooo`3oool0ZP3oool00`000000oooo0?ooo`0O0?ooo`00?@3oool00`3ooml0W4P00000
00070000000504RLg`3oool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`050?ooo`030?oO
W0180000M;oo00<0oooo00<0okmd0000B02Lgol0203oool00`000000oooo0?ooo`3Q0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`2Y0?ooo`030000
003oool0oooo01l0oooo00190?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01@3o
ool00`3ogi`0B00007Boo`030?ooo`030;md000004P0W=oo00P0oooo1000003P0?ooo`030000003o
ool0oooo01d0oooo00<000000?ooo`3oool05@3oool200000:P0oooo0`00000Q0?ooo`00B@3oool0
0`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00D0oooo00P0oono07@000000000B9cO0?oo
o`3o_g@00000001d_`T0oooo00<000000?ooo`3oool0h@3oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo01D0oooo0`00000L0?ooo`030000003oool0oooo08X0oooo00<000000?ooo`3o
ool07`3oool004T0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`060?ooo`030;md
0000B9`0_ia80080000000<004RL0=ooo`3oool0203oool00`000000oooo0?ooo`3Q0?ooo`030000
003oool0oooo01`0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0J0?ooo`L00000
Q`3oool00`000000oooo0?ooo`0O0?ooo`00B@3oool00`3ookl0M00004RLg`030?ooo`030=nLB000
07@0_ooo00H0oooo00<0omnL04P0001d_ol03@3oool00`000000oooo0?ooo`3Q0?ooo`030000003o
ool0oooo01/0oooo0`00000I0?ooo`030000003oool0oooo01P0oooo0`0000030?ooo`<00000QP3o
ool00`000000oooo0?ooo`0O0?ooo`00BP3oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0
oooo00L0oooo00<0gia80000B02Lgol0303oool00`000000oooo0?ooo`3Q0?ooo`030000003oool0
oooo01/0oooo0P00000K0?ooo`030000003oool0oooo01D0oooo0P0000070?ooo`<00000Q@3oool0
0`000000oooo0?ooo`0O0?ooo`00BP3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool02@3o
ool00`3o_g@00000001d_`0;0?ooo`030000003oool0oooo0>40oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01@0oooo0`0000090?ooo`030000003o
ool0000008<0oooo00<000000?ooo`3oool07`3oool004/0oooo00D0oono07@000000000001d0;oo
o`0;0?ooo`050?oog`2LB00000000000M02oool0203oool00`000000oooo0?ooo`3Q0?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0C0?ooo`800000
2`3oool300000880oooo00<000000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0h@3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01`0oooo0P00000C0?ooo`030000003o
ool0oooo00`0oooo0P0000220?ooo`030000003oool0oooo01l0oooo001X0?ooo`030000003oool0
oooo0>40oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0M0?ooo`<000004@3oool0
0`000000oooo0?ooo`0?0?ooo`030000003oool0oooo07l0oooo00<000000?ooo`3oool07`3oool0
06P0oooo0`00003Q0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool07P3oool20000
0140oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`1m0?ooo`8000008@3oool006P0
oooo00<000000?ooo`3oool0h@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo0200
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo07h0
oooo00<000000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0h@3oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`0B0?ooo`030000003oool0oooo07d0oooo00<000000?ooo`3oool07`3oool006P0oooo00<0
00000?ooo`3oool0h@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0280oooo00<0
00000?ooo`3oool0303oool300000180oooo0`00001n0?ooo`030000003oool0oooo01l0oooo001X
0?ooo`030000003oool0oooo0=l0oooo1@00000F0?ooo`<00000903oool00`000000oooo0?ooo`0;
0?ooo`800000503oool00`000000oooo0000001m0?ooo`030000003oool0oooo01l0oooo001X0?oo
o`030000003oool0oooo0=d0oooo0`000000103oool000000?ooo`3oool3000001@0oooo00<00000
0?ooo`3oool0903oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo01H0oooo00<00000
0?ooo`3oool0N`3oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`3L0?oo
o`<000000P3oool010000000oooo0?ooo`3oool300000180oooo00<000000?ooo`3oool09@3oool2
000000X0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`1j0?ooo`030000003oool0
oooo01l0oooo001X0?ooo`030000003oool0oooo0=`0oooo0P0000030?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0U0?ooo`<000002@3oool00`00
0000oooo0?ooo`0G0?ooo`030000003oool0oooo07X0oooo00<000000?ooo`3oool07`3oool006P0
oooo00<000000?ooo`3oool0g03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`1i0?ooo`030000003oool0oooo01l0
oooo001X0?ooo`030000003oool0oooo0=/0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0:03oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo01T0oooo0P00001j0?ooo`030000003oool0oooo01l0oooo
001X0?ooo`<00000a`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo02P0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`0I0?ooo`<00000N03oool200000240oooo001X
0?ooo`030000003oool0oooo0<D0oooo1@00000@0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo02T0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo07P0oooo00<00000
0?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0a03oool2000000050?ooo`000000oooo
0?ooo`0000003`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo00<0oooo0P00000N
0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool07`3oool006P0oooo00<000000?oo
o`3oool0``3oool00`000000oooo0?ooo`050?ooo`800000303oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0[0?oo
o`040000003oool0oooo0?ooo`<000007P3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0
oooo01l0oooo001X0?ooo`030000003oool0oooo0<80oooo0`0000060?ooo`<000002P3oool30000
00H0oooo00<000000?ooo`3oool01@3oool3000000`0oooo00<000000?ooo`3oool0:`3oool20000
00<0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`1f0?ooo`030000003oool0oooo
01l0oooo001X0?ooo`030000003oool0oooo0<80oooo0P0000080?ooo`030000003oool0oooo00T0
oooo0P0000070?ooo`030000003oool0oooo00H0oooo0P00000;0?ooo`030000003oool0oooo02`0
oooo0`0000020?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0MP3oool00`000000
oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`310?ooo`030000003oool0oooo00P0oooo
00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool0203oool3000002h0oooo00D000000?ooo`3oool0oooo0000000Q0?ooo`03
0000003oool0oooo07D0oooo00<000000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0
`@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
;P3oool00`000000oooo0000000R0?ooo`030000003oool0oooo07D0oooo00<000000?ooo`3oool0
7`3oool006P0oooo00<000000?ooo`3oool0`03oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0000000S0?ooo`030000003o
ool0oooo07@0oooo00<000000?ooo`3oool07`3oool006P0oooo0`0000300?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00P0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0
oooo0240oooo0`00001d0?ooo`8000008@3oool006P0oooo00<000000?ooo`3oool0`03oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0203oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0<03oool00`00
0000oooo0?ooo`0R0?ooo`800000M@3oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000
oooo0?ooo`2o0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`0a0?ooo`800000903oool00`000000oooo0?ooo`1c0?ooo`030000003oool0oooo01l0
oooo001X0?ooo`030000003oool0oooo0;l0oooo00<000000?ooo`3oool03@3oool2000000D0oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool0<03oool00`000000oooo0000000U0?ooo`030000003oool0oooo0780oooo
00<000000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0_P3oool3000000h0oooo0`00
00030?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo0300oooo00@000000?ooo`3oool00000903oool00`000000oooo
0?ooo`1b0?ooo`030000003oool0oooo01l0oooo001;0?ooo`050?oOW018000000000000M02oool0
0`3oool00`3o_g@0000007Boo`050?ooo`040?oog`2LB00004RL0=ooo`T0oooo00<000000?ooo`3o
ool0_P3oool200000100oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0a0?ooo`03
0000003oool00000008000008`3oool00`000000oooo0?ooo`1b0?ooo`030000003oool0oooo01l0
oooo001:0?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`020?ooo`030?noM0000000M;oo
00D0oooo00@0oooO09a80000B9`0gooo2@3oool00`000000oooo0?ooo`2n0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool00P00000;0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`0`0?ooo`040000003oool0oooo0?ooo`800000903o
ool00`000000oooo0?ooo`1a0?ooo`030000003oool0oooo01l0oooo001:0?ooo`030;md0000B9`0
gooo0080oooo00<0_g@0001d_`3oool02@3oool0103ooml0W4P00018W03Oool90?ooo`030000003o
ool0oooo0;h0oooo00<000000?ooo`3oool0403oool00`000000oooo00000002000000/0oooo00<0
00000?ooo`3oool02@3oool3000000@0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0L@3oool00`000000oooo0?oo
o`0O0?ooo`00B@3oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00D0oooo00<0_g@0
0000000000001@0000000`1d_ol0oooo0?ooo`060?ooo`030000003oool0oooo0;d0oooo00<00000
0?ooo`3oool04@3oool010000000oooo0?ooo`00000<0?ooo`030000003oool0oooo00X0oooo0P00
00040?ooo`030000003oool0oooo02l0oooo0`0000040?ooo`030000003oool0oooo0280oooo00<0
00000?ooo`3oool0L03oool00`000000oooo0?ooo`0O0?ooo`00?@3oool00`3ooml0W4P000000007
0000000504RLg`3oool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`050?ooo`080?oOW01d
W=l0oooo0?ooo`3ooml0W4P00018W03Oool90?ooo`030000003oool0oooo0;d0oooo00<000000?oo
o`3oool04P3oool00`000000oooo0000000<0?ooo`030000003oool0oooo00/0oooo00D000000?oo
o`3oool0oooo0000000b0?ooo`8000001@3oool00`000000oooo0?ooo`0Q0?ooo`<00000L@3oool0
0`000000oooo0?ooo`0O0?ooo`00B@3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo
00H0oooo00L0gia807Boo`3oool0oooO09a80000B9`0gooo00T0oooo1000002l0?ooo`030000003o
ool0oooo0180oooo0P00000=0?ooo`030000003oool0oooo00/0oooo00D000000?ooo`3oool0oooo
0000000b0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool08@3oool2000006l0oooo
0`00000Q0?ooo`00B@3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00H0oooo00L0
oooO09a8M02oool0oooO09a80000B9`0gooo00T0oooo00<000000?ooo`3oool0_@3oool00`000000
oooo0?ooo`0B0?ooo`8000003@3oool00`000000oooo0?ooo`0;0?ooo`050000003oool0oooo0?oo
o`000000<P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0280oooo00<000000?oo
o`3oool0K`3oool00`000000oooo0?ooo`0O0?ooo`00B@3oool00`3ogi`0B00007Boo`030?ooo`03
0?noM00004P0W=oo00L0oooo00H0omnL04Qd_`3ooml0W4P00018W03Oool90?ooo`030000003oool0
oooo0;`0oooo00<000000?ooo`3oool04`3oool2000000d0oooo00<000000?ooo`3oool0303oool0
0`000000oooo0000000200000340oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo06l0oooo00<000000?ooo`3oool07`3oool004T0oooo00<0oono07@0
0018W=l00`3oool00`3OW4P0001d0;ooo`080?ooo`050=nLB01d_ml0W4P00018W03Oool02@3oool0
0`000000oooo0?ooo`2l0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`0<0?ooo`040000003oool0000000000340oooo00<000000?ooo`3oool01`3o
ool2000002<0oooo00<000000?ooo`3oool0KP3oool00`000000oooo0?ooo`0O0?ooo`00BP3oool0
0`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00P0oooo00D0oooO09a8B018B00004RL0=oo
o`090?ooo`030000003oool0oooo0;`0oooo00<000000?ooo`3oool04`3oool2000000d0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0000000b0?ooo`030000003oool0oooo00L0oooo0`00
000R0?ooo`030000003oool0oooo06h0oooo00<000000?ooo`3oool07`3oool004X0oooo00L0oono
07@0001d_ol0oooo0?noM00007@0_ooo00X0oooo00@0omnL04P00000B9`0gooo2@3oool00`000000
oooo0?ooo`2k0?ooo`030000003oool0oooo01@0oooo0P00000=0?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`000000<P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0280
oooo00<000000?ooo`3oool0K@3oool00`000000oooo0?ooo`0O0?ooo`00B`3oool01@3ookl0M000
0000000007@0_ooo00`0oooo00<0gia80018W03Oool02@3oool00`000000oooo0?ooo`2k0?ooo`03
0000003oool0oooo01@0oooo0P00000=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3o
ool0<@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0240oooo00<000000?ooo`3o
ool0K@3oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`2j0?ooo`800000
5P3oool00`000000oooo0000000<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
<03oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0
K03oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`2i0?ooo`<000005@3o
ool010000000oooo0?ooo`00000<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
<03oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0240oooo0P00001]0?ooo`030000
003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo0;X0oooo00<000000?ooo`3oool0503o
ool010000000oooo0?ooo`00000<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
<03oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0240oooo0`00001/0?ooo`030000
003oool0oooo01l0oooo001X0?ooo`<00000^P3oool00`000000oooo0?ooo`0D0?ooo`030000003o
ool00000008000002`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool000000340oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo06X0oooo0P00
000Q0?ooo`00J03oool00`000000oooo0?ooo`2j0?ooo`030000003oool0oooo01@0oooo00@00000
0?ooo`3oool00000303oool00`000000oooo0?ooo`0<0?ooo`030000003oool000000300oooo00<0
00000?ooo`3oool03@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo06/0oooo00<0
00000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0^@3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0<
0?ooo`040000003oool00000000002l0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo06X0oooo00<000000?ooo`3oool07`3oool006P0oooo00<00000
0?ooo`3oool0^@3oool00`000000oooo0?ooo`0C0?ooo`<000000`3oool00`000000oooo0?ooo`09
0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`0000000P00000^0?ooo`030000003oool0
oooo00d0oooo0`00000R0?ooo`030000003oool0oooo06X0oooo00<000000?ooo`3oool07`3oool0
06P0oooo00<000000?ooo`3oool0^@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0<0?ooo`040000003oool0oooo
000002l0oooo00<000000?ooo`3oool03P3oool2000002<0oooo00<000000?ooo`3oool0J@3oool0
0`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`2i0?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
00/0oooo00D000000?ooo`3oool0oooo0000000_0?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool08@3oool00`000000oooo0?ooo`1Y0?ooo`030000003oool0oooo01l0oooo001X0?oo
o`030000003oool0oooo0;P0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo00P0oooo00<000000?ooo`3oool02`3oool01@000000oooo0?ooo`3oool0
000002h0oooo0`00000A0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0J03oool0
0`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`2h0?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
00X0oooo0P0000030?ooo`030000003oool0oooo02`0oooo0P00000B0?ooo`030000003oool0oooo
0240oooo0P00001Y0?ooo`030000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo0;P0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00P0
oooo00<000000?ooo`3oool02@3oool3000000@0oooo00<000000?ooo`3oool0:`3oool00`000000
oooo0?ooo`0B0?ooo`030000003oool0oooo0200oooo0`00001X0?ooo`030000003oool0oooo01l0
oooo001X0?ooo`<00000^03oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0240oooo
00<000000?ooo`3oool0IP3oool200000240oooo001X0?ooo`030000003oool0oooo0;L0oooo00<0
00000?ooo`3oool0503oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00P0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02/0oooo00<0
00000?ooo`3oool04`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo06H0oooo00<0
00000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0]`3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0IP3oool00`000000oooo0?ooo`0O
0?ooo`00J03oool00`000000oooo0?ooo`2g0?ooo`030000003oool0oooo01@0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00X0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo01<0oooo0`00000S0?oo
o`030000003oool0oooo06D0oooo00<000000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3o
ool0]`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool0:@3oool00`000000oooo0?ooo`0D0?ooo`8000008`3oool00`000000oooo0?ooo`1U0?ooo`03
0000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo0;H0oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo06@0oooo00<000000?ooo`3oool0
7`3oool006P0oooo00<000000?ooo`3oool0]P3oool00`000000oooo0?ooo`0D0?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0F0?ooo`030000003o
ool0oooo0240oooo00<000000?ooo`3oool0I03oool00`000000oooo0?ooo`0O0?ooo`00J03oool0
0`000000oooo0?ooo`2e0?ooo`<000005@3oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo02P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo06<0oooo00<000000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0]P3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool09`3oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo0200oooo0`00001T0?ooo`030000003oool0oooo
01l0oooo001X0?ooo`030000003oool0oooo0;D0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`090?ooo`8000001`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0200
oooo0P00001T0?ooo`030000003oool0oooo01l0oooo001X0?ooo`<00000]@3oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo00T0oooo0`0000060?ooo`030000003oool0oooo00P0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo01T0oooo
0P00000R0?ooo`030000003oool0oooo0640oooo0P00000Q0?ooo`00J03oool00`000000oooo0?oo
o`2e0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`0W0?ooo`030000003oool0oooo01T0oooo0`00000R0?ooo`030000003oool0oooo0640oooo00<0
00000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0]@3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`08
0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0H@3oool00`000000oooo0?ooo`0O
0?ooo`00J03oool00`000000oooo0?ooo`2e0?ooo`030000003oool0oooo01<0oooo0`00000;0?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?oo
o`<000009P3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0240oooo00<000000?oo
o`3oool0H03oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`2d0?ooo`03
0000003oool0oooo01@0oooo0P00000=0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3o
ool07@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo05l0oooo00<000000?ooo`3o
ool07`3oool004/0oooo00D0omnL04P000000000001d0;ooo`030?ooo`050?noM0000000M;oo0?oo
o`2oM0001P0000000`18W=l0oooo0?ooo`070?ooo`030000003oool0oooo0;@0oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo02D0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`0Q0?ooo`800000H03oool00`000000oooo0?ooo`0O0?oo
o`00BP3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool00P3oool00`3o_g@0000007Boo`02
0?ooo`030=nLB0000000000000<0000000<004RL0=ooo`3oool01`3oool00`000000oooo0?ooo`2d
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0U
0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0803oool3000005l0oooo00<00000
0?ooo`3oool07`3oool004X0oooo00<0_g@00018W03Oool00P3oool00`2oM00007Bo0?ooo`070?oo
o`030=nLB018W=l0oooo0080oooo00<0giad09cOo`3oool01`3oool00`000000oooo0?ooo`2d0?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo00H0oooo0P00000<0?ooo`030000003oool0oooo02<0oooo00<000000?oo
o`3oool0803oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo05h0oooo00<000000?oo
o`3oool07`3oool004T0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`080?ooo`03
0=nLB01d_ol0oooo00/0oooo00<000000?ooo`3oool0]03oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`<0
0000303oool00`000000oooo0?ooo`0R0?ooo`<00000803oool300000280oooo00<000000?ooo`3o
ool0G@3oool00`000000oooo0?ooo`0O0?ooo`00?@3oool00`3ooml0W4P0000000070000000504RL
g`3oool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`090?ooo`030;mdB02Lgol0oooo00X0
oooo00<000000?ooo`3oool0/`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0P0?ooo`8000008`3oool00`000000
oooo0?ooo`1L0?ooo`030000003oool0oooo01l0oooo00190?ooo`030?oOW0180000M;oo00<0oooo
00<0okmd0000B02Lgol02@3oool00`3ooml0W4Qd0;ooo`0:0?ooo`@00000/P3oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool08`3oool00`000000oooo
0?ooo`0R0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0FP3oool300000240oooo
00190?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol02P3oool00`3o_g@007Bo0?oo
o`090?ooo`030000003oool0oooo0;<0oooo00<000000?ooo`3oool0503oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo05/0oooo00<000000?ooo`3oool07`3oool004T0oooo00<0omnL
04P0001d_ol00`3oool00`3o_g@0001809cOo`0:0?ooo`030?oo_`1d04P0W=oo00T0oooo00<00000
0?ooo`3oool0/`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool08@3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo0200oooo00<00000
0?ooo`3oool0F`3oool00`000000oooo0?ooo`0O0?ooo`00B@3oool00`3ookl0M00004RLg`030?oo
o`030=nLB00007@0_ooo00X0oooo00<0oono07@00018W=l02@3oool00`000000oooo0?ooo`2b0?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo02D0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`1J0?oo
o`030000003oool0oooo01l0oooo001:0?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d_`3o
ool01@3oool00`3o_i`0W=oo0?ooo`020?ooo`030?oOW0180000B9cO00T0oooo00<000000?ooo`3o
ool0/P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo01l0oooo0`00001K0?ooo`03
0000003oool0oooo01l0oooo001:0?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`060?oo
o`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02oool90?ooo`030000003oool0oooo0;80oooo
00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0240oooo
00<000000?ooo`3oool0903oool300000240oooo0P00001K0?ooo`030000003oool0oooo01l0oooo
001;0?ooo`050?oo_`1d000000000000M02oool0203oool01P3ooml0W4P0000000000000001d0;oo
o`X0oooo00<000000?ooo`3oool0/P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0U0?ooo`030000003oool00000
0280oooo00<000000?ooo`3oool0F03oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000
oooo0?ooo`2b0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0803oool00`000000
oooo0?ooo`1H0?ooo`030000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo0;40oooo
0P00000F0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool07`3oool00`000000oooo
0?ooo`0Y0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0E`3oool00`000000oooo
0?ooo`0O0?ooo`00J03oool300000;40oooo0`00000D0?ooo`030000003oool0oooo0180oooo00D0
00000?ooo`3oool0oooo000000060?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
7`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0
E@3oool200000240oooo001X0?ooo`030000003oool0oooo0;40oooo0P00000E0?ooo`030000003o
ool0oooo0180oooo00D000000?ooo`3oool0oooo000000060?ooo`030000003oool0oooo00l0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo01h0oooo
0`00001G0?ooo`030000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo0;40oooo00<0
00000?ooo`3oool0503oool00`000000oooo0?ooo`0B0?ooo`050000003oool0oooo0?ooo`000000
1@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0
:P3oool300000200oooo0P00001G0?ooo`030000003oool0oooo01l0oooo001X0?ooo`030000003o
ool0oooo0;40oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0B0?ooo`050000003o
ool0oooo0?ooo`0000001@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01d0oooo
00<000000?ooo`3oool0;03oool200000280oooo00<000000?ooo`3oool0E03oool00`000000oooo
0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`2a0?ooo`030000003oool0oooo01<0oooo0P00
000C0?ooo`<000000P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool07@3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0200oooo00<0
00000?ooo`3oool0D`3oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`2`
0?ooo`030000003oool0oooo01<0oooo0`00000D0?ooo`050000003oool0oooo0?ooo`0000001@3o
ool00`000000oooo0?ooo`0@0?ooo`<000007P3oool00`000000oooo0?ooo`0_0?ooo`030000003o
ool0oooo01l0oooo00<000000?ooo`3oool0D`3oool00`000000oooo0?ooo`0O0?ooo`00J03oool0
0`000000oooo0?ooo`2`0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0503oool0
10000000oooo0?ooo`0000050?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool07@3o
ool00`000000oooo0?ooo`0_0?ooo`8000008@3oool2000005<0oooo00<000000?ooo`3oool07`3o
ool006P0oooo00<000000?ooo`3oool0/03oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo01@0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0oooo01`0oooo00<000000?ooo`3oool0;`3oool300000200oooo0`00001B0?ooo`030000003o
ool0oooo01l0oooo001X0?ooo`030000003oool0oooo0;00oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`0D0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool04`3o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0340oooo0P00000Q0?ooo`800000DP3o
ool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`2`0?ooo`030000003oool0
oooo01@0oooo00<000000?ooo`3oool0503oool010000000oooo0?ooo`0000030?ooo`<00000503o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo03<0oooo0P00000P0?ooo`030000003o
ool0oooo0500oooo00<000000?ooo`3oool07`3oool006P0oooo0`00002_0?ooo`030000003oool0
oooo01@0oooo00<000000?ooo`3oool05@3oool010000000oooo0?ooo`0000040?ooo`030000003o
ool0oooo01<0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0e0?ooo`030000003o
ool0oooo01h0oooo00<000000?ooo`000000CP3oool200000240oooo001X0?ooo`030000003oool0
oooo0:l0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0E0?ooo`040000003oool0
oooo000000<0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0J0?ooo`<00000=@3o
ool300000200oooo0`00001>0?ooo`030000003oool0oooo01l0oooo001X0?ooo`030000003oool0
oooo0:l0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
000000<0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo03D0oooo00<000000?ooo`000000803oool2000004h0oooo00<000000?ooo`3oool07`3oool0
06P0oooo00<000000?ooo`3oool0[`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`0000000`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool0>@3oool200000200oooo00<000000?ooo`3oool0B`3oool00`00
0000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`2_0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool05P3oool00`000000oooo000000030?ooo`030000003oool0oooo01D0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo01h0
oooo0P00001;0?ooo`030000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo0:h0oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0000000<0oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo03X0oooo
0`00000O0?ooo`<00000BP3oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?oo
o`2^0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool05`3oool00`000000oooo0000
00020?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?oo
o`0k0?ooo`030000003oool0000001l0oooo0`0000190?ooo`030000003oool0oooo01l0oooo001X
0?ooo`030000003oool0oooo0:h0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0G
0?ooo`030000003oool000000080oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo03h0oooo0P00000P0?ooo`030000003oool0000004H0oooo00<00000
0?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0[P3oool00`000000oooo0?ooo`0D0?oo
o`030000003oool0oooo01P0oooo0P0000020?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0n0?ooo`<00000803oool3000004D0oooo00<000000?oo
o`3oool07`3oool006P0oooo0`00002^0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool0603oool200000080oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo0400oooo00D000000?ooo`000000oooo0000000N0?ooo`800000A03oool20000
0240oooo001X0?ooo`030000003oool0oooo0:h0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0H0?ooo`80000000<0oooo0000003oool06@3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo04<0oooo0`00000O0?ooo`<00000@P3oool00`000000oooo0?ooo`0O0?ooo`00
J03oool00`000000oooo0?ooo`2]0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
6@3oool2000000030?ooo`000000oooo01T0oooo00<000000?ooo`3oool05`3oool00`000000oooo
0?ooo`140?ooo`<000000P3oool00`000000oooo0?ooo`0J0?ooo`<00000@@3oool00`000000oooo
0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`2]0?ooo`030000003oool0oooo01@0oooo00<0
00000?ooo`3oool06@3oool2000000030?ooo`000000oooo01X0oooo00<000000?ooo`3oool05P3o
ool00`000000oooo0?ooo`170?ooo`@00000703oool00`000000oooo00000002000003h0oooo00<0
00000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0[03oool3000001D0oooo00<00000
0?ooo`3oool06@3oool2000000030?ooo`000000oooo01X0oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0?ooo`190?ooo`<0000000<0oooo0000003oool06`3oool4000003`0oooo00<00000
0?ooo`3oool07`3oool005T0oooo00D0omnL04P000000000001d0;ooo`0:0?ooo`030000003oool0
oooo0:d0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
000001/0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`1=0?ooo`@0000000<0oooo
0000003oool0603oool00`000000oooo0?ooo`03000003T0oooo00<000000?ooo`3oool07`3oool0
05P0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00T0oooo00<000000?ooo`3oool0[@3o
ool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01X0oooo0P00000L0?ooo`030000003o
ool0oooo01D0oooo00<000000?ooo`3oool0CP3oool00`000000oooo0?ooo`03000000<0oooo00<0
00000?ooo`3oool05P3oool300000080oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?oo
o`0O0?ooo`00F03oool00`2oM00004RL0=ooo`020?ooo`030;md0000M;l0oooo00P0oooo00<00000
0?ooo`3oool0[03oool00`000000oooo0?ooo`0D0?ooo`800000703oool2000001`0oooo0P00000F
0?ooo`030000003oool0oooo0580oooo1`0000020?ooo`050000003oool0oooo0?ooo`0000004@3o
ool00`000000oooo00000003000000<0oooo00D000000?ooo`3oool0oooo0000000/0?ooo`030000
003oool0oooo01l0oooo001G0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool0203o
ool00`000000oooo0?ooo`2/0?ooo`030000003oool0oooo01<0oooo0`00000L0?ooo`800000703o
ool3000001D0oooo00<000000?ooo`3oool0E`3oool:000000<0oooo00D000000?ooo`3oool0oooo
000000030?ooo`030000003oool0oooo00H0oooo00<000000?ooo`000000200000020?ooo`030000
003oool0oooo0080oooo00D000000?ooo`3oool0oooo0000000O0?ooo`030000003oool0oooo01l0
oooo001G0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0203oool00`000000oooo
0?ooo`080?ooo`030000003oool0oooo0080oooo00D000000?ooo`3oool0oooo000000030?ooo`03
0000003oool0oooo0080oooo00D000000?ooo`3oool0oooo000000030?ooo`030000003oool0oooo
0080oooo00D000000?ooo`3oool0oooo000000030?ooo`030000003oool0oooo0080oooo00D00000
0?ooo`3oool0oooo000000030?ooo`030000003oool0oooo0080oooo00D000000?ooo`3oool0oooo
000000030?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0F`3oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo01/0oooo0P00000M0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool0F`3oool010000000oooo0?ooo`3oool?000000<0oooo00D000000?ooo`3o
ool0oooo000000040?ooo`060000003oool0oooo0?ooo`000000oooo3@0000020?ooo`030000003o
ool0oooo0080oooo00D000000?ooo`3oool0oooo000000030?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0O0?ooo`00E`3oool00`3ogi`0B00007Bo
o`030?ooo`030?noM00004P0W=oo00P0ooooo`00003:00000240oooo001G0?ooo`030?oOW0180000
M;oo00<0oooo00<0okmd0000B02Lgol0203oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo0080oooo00D000000?ooo`3oool0oooo000000030?ooo`030000003oool0oooo0080oooo;@00
00020?ooo`030000003oool0oooo0080oooo3`0000020?ooo`050000003oool0oooo0?ooo`000000
B03oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01/0oooo0P00000M0?ooo`030000
003oool0oooo01<0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`020?ooo`030000
003oool000000140000000<0oooo0000003oool00`3oool01@000000oooo0?ooo`3oool0000000<0
oooo00<000000?ooo`3oool00P3oool@000000030?ooo`000000oooo00<0oooo00D000000?ooo`3o
ool0oooo000000030?ooo`030000003oool0oooo0080oooo00D000000?ooo`3oool0oooo00000003
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0O
0?ooo`00E`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00P0oooo00<000000?oo
o`3oool07P3oool01@000000oooo0?ooo`3oool0000000@0oooo00<000000?ooo`3oool01`000003
0?ooo`030000003oool0oooo00L000000`3oool01P000000oooo0?ooo`3oool000000?ooo`T00000
0P3oool01@000000oooo0?ooo`3oool0000000@0oooo00<000000?ooo`0000001`0000030?ooo`03
0000003oool0oooo0400oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0K0?ooo`80
00007P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0480oooo00<000000?ooo`3o
ool0200000020?ooo`050000003oool0oooo0?ooo`000000103oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0000000P000000P3oool00`000000oooo0?ooo`020?ooo`050000003oool0oooo
0?ooo`000000;03oool00`000000oooo0?ooo`0O0?ooo`00:03oool01@3ogi`0M9cO0?ooo`3ookl0
M0000080000000<0001d0;ooo`3oool09@3oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0
_ooo00P0oooo00<000000?ooo`3oool0:`3oool010000000oooo0?ooo`3oool700000080oooo00@0
00000?ooo`3oool0oooo1`0000020?ooo`050000003oool0oooo0?ooo`000000103oool600000080
oooo00<000000?ooo`3oool01P3oool01P000000oooo0?ooo`3oool000000?ooo`D000000P3oool0
0`000000oooo0?ooo`0l0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0703oool2
000001h0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0
000000@0000000<0oooo0000003oool00`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0
000000<000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo03L0oooo00<00000
0?ooo`3oool07`3oool002P0oooo00<0oooO09a8000000000`000000101d_kl0M0000018W03OoolV
0?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d_`3oool0203oool00`000000oooo0?ooo`0d
0?ooo`H000001`3oool600000080oooo00<000000?ooo`3oool01P3oool00`000000oooo00000003
000000<0oooo00<000000?ooo`3oool02P3oool600000080oooo00<000000?ooo`3oool0>03oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01`0oooo0P00000N0?ooo`030000003oool0
oooo0180oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0400000080oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`0400000080oooo00<000000?ooo`3oool0@03oool0
0`000000oooo0?ooo`0O0?ooo`00<03oool01P2oM4P0W=no07@00000000004RL0=ooob80oooo00L0
oono07@0001d_ol0oooo0?noM00007@0_ooo00T0oooo00<000000?ooo`3oool0>03oool5000000P0
oooo1P00000;0?ooo`030000003oool0000000@000003`3oool00`000000oooo0000000400000080
oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01/0
oooo0`00000N0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0=@3oool00`000000
oooo0?ooo`0400000080oooo00<000000?ooo`3oool07P3oool400000080oooo00<000000?ooo`3o
ool0A03oool00`000000oooo0?ooo`0O0?ooo`00;P3oool01@3OW4P000000000000007@0_ooo02H0
oooo00D0oono07@000000000001d0;ooo`0:0?ooo`030000003oool0oooo03/0oooo0`00000;0?oo
o`<000000P3oool00`000000oooo0?ooo`0:0?ooo`<0000000<0oooo0000003oool03`3oool00`00
0000oooo0?ooo`04000003@0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0K0?oo
o`<000007P3oool00`000000oooo0?ooo`0A0?ooo`<00000=@3oool4000000030?ooo`000000oooo
0240oooo00<000000?ooo`0000000P00001;0?ooo`030000003oool0oooo01l0oooo000Z0?ooo`08
0?noM0000000000000000018W=l0oooo0;mdB02Lgolf0?ooo`030000003oool0oooo03`0oooo00H0
00000?ooo`00000000000?ooo`00000:0?ooo`@000003P3oool3000001@0oooo0`0000000`3oool0
00000?ooo`0`0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool06`3oool200000003
0?ooo`000000oooo01d0oooo00<000000?ooo`3oool0403oool2000003<0oooo00<000000?ooo`3o
ool00P00000U0?ooo`<00000CP3oool00`000000oooo0?ooo`0O0?ooo`009P3oool01P2oM0000000
000000000000001d0;ooo`<0oooo00<0oono07A8W03Oool0=P3oool00`000000oooo0?ooo`100?oo
o`<000002`3oool200000080oooo00<000000?ooo`3oool02P3oool00`000000oooo000000020000
01@0oooo0`00000_0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0703oool20000
00030?ooo`000000oooo01d0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0a0?oo
o`@00000903oool00`000000oooo00000002000004l0oooo00<000000?ooo`3oool07`3oool002@0
oooo00@0oooO09a80000000007Bo1P3oool00`3ooml0W4P004RLg`0g0?ooo`030000003oool0oooo
0440oooo0`00000<0?ooo`@000003P3oool3000001@0oooo00H000000?ooo`00000000000?ooo`00
000[0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool06`3oool01@000000oooo0000
003oool0000001h0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0]0?ooo`040000
003oool0oooo0?ooo`8000009@3oool300000580oooo00<000000?ooo`3oool07`3oool002`0oooo
00@0_g@000000000000007Bo>03oool3000004@0oooo0P00000<0?ooo`8000003`3oool01P000000
oooo000000000000oooo000001@0oooo0`00000Z0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool06`3oool01@000000oooo0000003oool0000001h0oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0/0?ooo`D000009P3oool300000580oooo0P00000Q0?ooo`00:@3oool0
1P3ooml0W4P0000000000000001809cOocT0oooo00<000000?ooo`3oool0A03oool3000000d0oooo
0`00000@0?ooo`<00000503oool2000002X0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`0K0?ooo`050000003oool000000?ooo`0000007P3oool00`000000oooo0?ooo`0?0?ooo`03
0000003oool0oooo02d0oooo0`00000U0?ooo`030000003oool0000005H0oooo00<000000?ooo`3o
ool07`3oool002P0oooo00@0oooO09a80000B9`0gooo?03oool00`000000oooo0?ooo`150?ooo`04
0000003oool00000000000`0oooo0`00000@0?ooo`8000005P3oool3000002L0oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`0K0?ooo`050000003oool000000?ooo`0000007`3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo02X0oooo0`00000W0?ooo`<00000E`3oool0
0`000000oooo0?ooo`0O0?ooo`00:03oool00`3ogi`0M9cO0?ooo`030?ooo`030;nLW03Oool0oooo
0080oooo00<0gibL0;ooo`3oool0<P3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo
00X0oooo00@000000?ooo`0000000000403oool00`000000oooo0000000D0?ooo`<000009@3oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01X0oooo00@000000?ooo`3oool000000P3o
ool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0:@3o
ool3000002P0oooo0P00001H0?ooo`030000003oool0oooo01l0oooo000]0?ooo`030?oo_`1d0000
000000<0000000<0001d0;ooo`3oool0<P3oool00`000000oooo0?ooo`180?ooo`<000003`3oool2
000000l0oooo0`00000D0?ooo`8000009@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo01/0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0L0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0:03oool00`000000oooo0000000V0?ooo`030000003o
ool0000005X0oooo00<000000?ooo`3oool07`3oool002h0oooo00<0oknL09cOo`3oool00P3oool0
0`3OW9`0_ooo0?ooo`0b0?ooo`030000003oool0oooo04T0oooo00<000000?ooo`0000003P3oool3
000000l0oooo0P00000F0?ooo`<000008P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo01/0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0L0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool09P3oool2000002P0oooo0`00001K0?ooo`030000003o
ool0oooo01l0oooo000]0?ooo`030?oog`2LM9`0gooo03P0oooo00<000000?ooo`3oool0C03oool0
0`000000oooo0?ooo`0<0?ooo`8000004@3oool00`000000oooo0?ooo`0D0?ooo`<000008@3oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01/0oooo00@000000?ooo`3oool000000P3o
ool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool09@3o
ool3000002T0oooo00<000000?ooo`3oool0FP3oool00`000000oooo0?ooo`0O0?ooo`00;@3oool0
0`3ookl0M4RL0=ooo`030?ooo`030=nLW02oool0oooo0380oooo00<000000?ooo`3oool0C@3oool2
000000h0oooo00<000000?ooo`3oool03`3oool2000001D0oooo0P00000Q0?ooo`030000003oool0
oooo01@0oooo00<000000?ooo`3oool06`3oool010000000oooo0?ooo`0000020?ooo`030000003o
ool0oooo01d0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0U0?ooo`030000003o
ool0000002T0oooo00<000000?ooo`3oool0F`3oool00`000000oooo0?ooo`0O0?ooo`00;P3oool0
0`2oM000000000000002000000030000M02oool0oooo0380oooo00<000000?ooo`3oool0C@3oool3
000000h0oooo0P00000?0?ooo`<000005P3oool00`000000oooo0?ooo`0M0?ooo`<000005@3oool0
0`000000oooo0?ooo`0J0?ooo`050000003oool0oooo0?ooo`0000000`3oool00`000000oooo0?oo
o`0L0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool00`000000oooo0?oo
o`0W0?ooo`030000003oool0000005h0oooo00<000000?ooo`3oool07`3oool003<0oooo00<0gibL
0;ooo`3oool0<P3oool3000004h0oooo0P00000>0?ooo`<000003`3oool2000001L0oooo00<00000
0?ooo`000000703oool2000001D0oooo00<000000?ooo`3oool06`3oool01@000000oooo0?ooo`3o
ool0000000<0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo0280oooo0P00000Y0?ooo`<00000GP3oool200000240oooo001X0?ooo`030000003oool0
oooo0500oooo00<000000?ooo`3oool0303oool200000140oooo00<000000?ooo`3oool05@3oool3
000001/0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0J0?ooo`<000000P3oool0
1@000000oooo0?ooo`3oool0000001h0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0Q0?ooo`<00000:P3oool00`000000oooo0?ooo`1N0?ooo`030000003oool0oooo01l0oooo001X
0?ooo`030000003oool0oooo0540oooo0P00000>0?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`0000005@3oool2000001/0oooo00<000000?ooo`3oool04`3oool3000001`0oooo00D00000
0?ooo`3oool0oooo000000030?ooo`030000003oool0oooo01/0oooo0`00000=0?ooo`030000003o
ool0oooo0240oooo0P00000Z0?ooo`030000003oool0oooo05l0oooo00<000000?ooo`3oool07`3o
ool006P0oooo00<000000?ooo`3oool0D@3oool3000000d0oooo00<000000?ooo`3oool0403oool3
000001H0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool00P3oool01@000000oooo0?ooo`3oool0000001h0oooo0P00
000<0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?oo
o`1P0?ooo`030000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo0580oooo0P00000>
0?ooo`8000004@3oool00`000000oooo0?ooo`0F0?ooo`800000603oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0O
0?ooo`800000:@3oool00`000000oooo0000001S0?ooo`030000003oool0oooo01l0oooo001X0?oo
o`030000003oool0oooo05<0oooo00<000000?ooo`3oool0303oool300000140oooo00<000000?oo
o`3oool05@3oool3000001H0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0L0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool07P3oool3000002P0oooo0`00001T0?oo
o`030000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo05@0oooo00<000000?ooo`3o
ool0303oool200000180oooo00<000000?ooo`3oool05@3oool2000001H0oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0080oooo00@000000?ooo`3o
ool0oooo0`00000M0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool07P3oool20000
02X0oooo00<000000?ooo`3oool0H`3oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000
oooo0?ooo`1E0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0F0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool07@3oool00`000000
oooo0?ooo`0Y0?ooo`030000003oool0oooo06@0oooo00<000000?ooo`3oool07`3oool006P0oooo
00<000000?ooo`3oool0EP3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0100oooo
0P00000G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0M0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`0Z0?ooo`030000003oool0oooo06@0oooo00<000000?ooo`3oool07`3oool006P0oooo00<0
00000?ooo`3oool0E@3oool3000000h0oooo00<000000?ooo`3oool03`3oool3000001L0oooo0P00
000C0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0703oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?oo
o`1U0?ooo`030000003oool0oooo01l0oooo001X0?ooo`<00000EP3oool2000000l0oooo0P00000@
0?ooo`8000005`3oool300000140oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0L
0?ooo`030000003oool0oooo00L0oooo0`00000L0?ooo`<00000:@3oool2000006L0oooo0P00000Q
0?ooo`00J03oool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo00`0oooo0`00000A0?oo
o`030000003oool0oooo01D0oooo0P00000A0?ooo`030000003oool0oooo01@0oooo00<000000?oo
o`3oool06`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool0703oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo01/0oooo0P00000Y0?oo
o`<00000J03oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`1I0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0F0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0Y0?ooo`80
0000J@3oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`1I0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool04@3oool2000001L0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0
J@3oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`1J0?ooo`8000003P3o
ool00`000000oooo0?ooo`0@0?ooo`<000005P3oool00`000000oooo0000000=0?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo01P0oooo0P00000/0?ooo`030000003oool0oooo06T0oooo00<000000?ooo`3oool07`3o
ool004/0oooo00D0omnL04P000000000001d0;ooo`030?ooo`050?noM0000000M;oo0?ooo`2oM000
1P0000000`18W=l0oooo0?ooo`070?ooo`030000003oool0oooo05X0oooo0`00000=0?ooo`030000
003oool0oooo0140oooo00<000000?ooo`3oool05P3oool3000000/0oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
5`3oool3000002/0oooo00<000000?ooo`3oool0JP3oool00`000000oooo0?ooo`0O0?ooo`00BP3o
ool01`3ookl0M00007Boo`3oool0okmd0000M02oool00P3oool00`3o_g@0000007Boo`020?ooo`03
0=nLB0000000000000<0000000<004RL0=ooo`3oool01`3oool00`000000oooo0?ooo`1K0?ooo`03
0000003oool0oooo00d0oooo0P00000B0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3o
ool0J`3oool00`000000oooo0?ooo`0O0?ooo`00BP3oool00`2oM00004RL0=ooo`020?ooo`030;md
0000M;l0oooo00L0oooo00<0gia804RLg`3oool00P3oool00`3OW7@0W=oo0?ooo`070?ooo`030000
003oool0oooo05`0oooo00<000000?ooo`3oool0303oool300000180oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
7@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
:@3oool3000006`0oooo00<000000?ooo`3oool07`3oool004T0oooo00<0oono07@00018W=l00`3o
ool00`3OW4P0001d0;ooo`080?ooo`030=nLB01d_ol0oooo00/0oooo00<000000?ooo`3oool0G03o
ool00`000000oooo0?ooo`0=0?ooo`8000004P3oool00`000000oooo0?ooo`0G0?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0Z0?ooo`800000K@3o
ool00`000000oooo0?ooo`0O0?ooo`00B@3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0
W=oo00T0oooo00<0_gA809cOo`3oool02P3oool00`000000oooo0?ooo`1M0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
oooo00P0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0
oooo06d0oooo00<000000?ooo`3oool07`3oool004T0oooo00<0omnL04P0001d_ol00`3oool00`3o
_g@0001809cOo`090?ooo`030?oog`2LB7@0_ooo00X0oooo1000001L0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool0403oool2000001P0oooo0P0000080?ooo`030000003oool0oooo
01<0oooo00<000000?ooo`3oool06P3oool2000000L0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`0L0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool05@3oool20000
02`0oooo00<000000?ooo`3oool0J`3oool300000240oooo00190?ooo`030?oOW0180000M;oo00<0
oooo00<0okmd0000B02Lgol02P3oool00`3o_g@007Bo0?ooo`090?ooo`030000003oool0oooo05h0
oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0@0?ooo`<000005`3oool3000000H0
oooo0`00000D0?ooo`030000003oool0oooo01X0oooo0`0000060?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01@0
oooo0`00000[0?ooo`030000003oool0oooo06h0oooo00<000000?ooo`3oool07`3oool004T0oooo
00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0:0?ooo`030?oo_`1d04P0W=oo00T0oooo
00<000000?ooo`3oool0GP3oool2000000l0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo00D0oooo0P00000D0?ooo`800000703oool2000000L0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo06h0oooo
00<000000?ooo`3oool07`3oool004T0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;oo
o`0:0?ooo`030?oo_`1d0000B9cO00T0oooo00<000000?ooo`3oool0GP3oool3000000h0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool04P3oool3000001`0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo01`0oooo0`0000030?ooo`030000003oool0oooo01D0oooo00<0
00000?ooo`3oool0:P3oool00`000000oooo0?ooo`1_0?ooo`030000003oool0oooo01l0oooo001:
0?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d_`3oool01@3oool00`3o_i`0W=oo0?ooo`02
0?ooo`030?oOW0180000B9cO00T0oooo00<000000?ooo`3oool0G`3oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0:P3oool200000740oooo00<00000
0?ooo`3oool07`3oool004X0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00H0oooo00P0
oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`T0oooo00<000000?ooo`3oool0H03oool00`00
0000oooo0?ooo`0<0?ooo`<000004P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0Y0?ooo`<00000L@3oool00`00
0000oooo0?ooo`0O0?ooo`00B`3oool01@3ookl0M0000000000007@0_ooo00P0oooo00H0oooO09a8
0000000000000000M02oool:0?ooo`030000003oool0oooo0600oooo00<000000?ooo`3oool03@3o
ool2000001<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool07@3oool01@000000oooo0?ooo`3oool0000001D0oooo
00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`1`0?ooo`030000003oool0oooo01l0oooo
001X0?ooo`030000003oool0oooo0640oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0@0?ooo`800000603oool200000080oooo00<000000?ooo`3oool0503oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0M0?ooo`040000003oool0oooo000001D0oooo0P00000/0?ooo`030000003oool0oooo0740oooo
00<000000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0H@3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo0100oooo0`00000G0?ooo`<0000000<0oooo0000003oool0503o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`0M0?ooo`040000003oool0oooo000001@0oooo0`00000/0?ooo`030000
003oool0oooo0740oooo00<000000?ooo`3oool07`3oool006P0oooo0`00001R0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0G0?ooo`040000003o
ool0oooo000001D0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo00D0oooo0`00000N0?ooo`040000003oool0oooo000001@0oooo0P00000/0?ooo`03
0000003oool0oooo0740oooo0P00000Q0?ooo`00J03oool00`000000oooo0?ooo`1R0?ooo`800000
3`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`000000
5@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
1P3oool2000001h0oooo00@000000?ooo`3oool00000503oool00`000000oooo0?ooo`0[0?ooo`03
0000003oool0oooo0780oooo00<000000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0
HP3oool3000000l0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0G0?ooo`800000
5P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`0M0?ooo`<000004`3oool00`000000oooo0?ooo`0[0?ooo`030000
003oool0oooo07<0oooo00<000000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0H`3o
ool00`000000oooo0?ooo`0>0?ooo`8000004P3oool00`000000oooo0?ooo`0G0?ooo`030000003o
ool0oooo01<0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool07@3oool400000180oooo00<000000?ooo`3oool0:P3o
ool2000007D0oooo00<000000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0I03oool0
0`000000oooo0?ooo`0=0?ooo`<000004@3oool00`000000oooo0?ooo`0G0?ooo`800000503oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`0M0?ooo`<000004P3oool00`000000oooo0?ooo`0Z0?ooo`<00000M@3oool0
0`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`1T0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0F0?ooo`80000000<0oooo0000
003oool04@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00T0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`0M0?ooo`8000004@3oool2000002h0oooo00<000000?oo
o`3oool0M03oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`1U0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool0403oool00`000000oooo0000000E0?ooo`03
0000003oool0oooo00<000004@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo00l0
oooo0`00000]0?ooo`030000003oool0oooo07D0oooo00<000000?ooo`3oool07`3oool006P0oooo
00<000000?ooo`3oool0I@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0140oooo
0`00000D0?ooo`050000003oool0oooo0?ooo`0000004P3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0L0?ooo`80
00004@3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo07D0oooo00<000000?ooo`3o
ool07`3oool006P0oooo00<000000?ooo`3oool0IP3oool00`000000oooo0?ooo`0>0?ooo`030000
003oool0oooo0100oooo0P00000E0?ooo`050000003oool0oooo0?ooo`0000004@3oool00`000000
oooo0?ooo`0K0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0203oool00`000000
oooo0?ooo`0L0?ooo`8000004@3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo07H0
oooo00<000000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0IP3oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04`3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0703oool200000100oooo
00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`1f0?ooo`030000003oool0oooo01l0oooo
001X0?ooo`<00000I`3oool2000000l0oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`0C0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0203oool00`000000oooo0?oo
o`0K0?ooo`030000003oool000000100oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?oo
o`1f0?ooo`8000008@3oool006P0oooo00<000000?ooo`3oool0I`3oool3000000h0oooo0P00000B
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo01T0oooo0`00000;0?ooo`030000003oool0oooo00P0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0000000?0?ooo`030000003oool0oooo02/0oooo0P00001i
0?ooo`030000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo06P0oooo00<000000?oo
o`3oool03@3oool300000140oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo00P0oooo00<000000?ooo`3oool06`3oool010000000oooo0?ooo`00000=
0?ooo`030000003oool0oooo02/0oooo0`00001i0?ooo`030000003oool0oooo01l0oooo001X0?oo
o`030000003oool0oooo06P0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo00l0oooo0`0000060?ooo`800000303oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0J0?oo
o`040000003oool0oooo000000`0oooo0P00000^0?ooo`030000003oool0oooo07P0oooo00<00000
0?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0J@3oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`050?oo
o`<000002P3oool3000001/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`0000000P00000:0?ooo`<00000;@3oool00`00
0000oooo0?ooo`1i0?ooo`030000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo06T0
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0A0?ooo`800000403oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`0;0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool06@3oool01@000000
oooo0?ooo`3oool0000000`0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`1i0?oo
o`030000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo06X0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`0@0?ooo`<000003P3oool00`000000oooo0?ooo`080?ooo`03
0000003oool0oooo00P0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo00P0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`1j0?ooo`03
0000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo06X0oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
303oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0
NP3oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`1[0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0/0?ooo`030000003o
ool0oooo07/0oooo00<000000?ooo`3oool07`3oool006P0oooo0`00001[0?ooo`8000003`3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0NP3oool2
00000240oooo001X0?ooo`030000003oool0oooo06/0oooo0`00000?0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
00T0oooo0`00000H0?ooo`800000103oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
02`0oooo00<000000?ooo`3oool0O03oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000
oooo0?ooo`1/0?ooo`030000003oool0oooo00h0oooo0P00000B0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01/0
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0:0?ooo`800000603oool3000000@0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0/0?ooo`800000OP3oool00`000000
oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`1]0?ooo`030000003oool0oooo00d0oooo
0`00000A0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool3000000D0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool05P3oool2000000D0oooo00<000000?ooo`3oool00`3oool3000002`0oooo
0`00001n0?ooo`030000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo06d0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool0303oool2000000D0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0503oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00<0oooo0P00000^0?ooo`030000003oool0oooo07d0oooo00<0
00000?ooo`3oool07`3oool004/0oooo00D0omnL04P000000000001d0;ooo`030?ooo`030?noM000
0000M;oo00D0oooo00@0oooO09a80000B9`0gooo2@3oool00`000000oooo0?ooo`1^0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool0403oool2000000X0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0
O@3oool00`000000oooo0?ooo`0O0?ooo`00BP3oool01`3ookl0M00007Boo`3oool0okmd0000M02o
ool00P3oool00`3o_g@0000007Boo`050?ooo`040?oog`2LB00004RL0=ooo`T0oooo00<000000?oo
o`3oool0KP3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0100oooo0`0000080?oo
o`030000003oool0oooo00l0oooo00D000000?ooo`3oool0oooo0000000M0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo00H0oooo00D000000?ooo`3oool0oooo0000000_0?ooo`030000003oool0oooo07h0oooo00<0
00000?ooo`3oool07`3oool004X0oooo00<0_g@00018W03Oool00P3oool00`2oM00007Bo0?ooo`09
0?ooo`040?oog`2LB00004RL0=ooo`T0oooo00<000000?ooo`3oool0K`3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`0?0?ooo`050000003oool0oooo0?ooo`000000703oool00`000000oooo0?ooo`0>0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`060?ooo`050000
003oool0oooo0?ooo`000000;`3oool00`000000oooo0?ooo`1n0?ooo`030000003oool0oooo01l0
oooo00190?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool01@3oool00`2oM0000000
000000050000000307Boo`3oool0oooo00H0oooo00<000000?ooo`3oool0K`3oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0000001d0oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool01`3oool010000000oooo
0?ooo`00000_0?ooo`030000003oool0oooo07l0oooo00<000000?ooo`3oool07`3oool004T0oooo
00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`050?ooo`080?oOW01dW=l0oooo0?ooo`3o
oml0W4P00018W03Oool90?ooo`030000003oool0oooo0700oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0403oool0
0`000000oooo0000000L0?ooo`<000003`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo0180oooo00<000000?ooo`3oool01`3oool010000000oooo0?ooo`00000_0?ooo`030000003o
ool0oooo07l0oooo00<000000?ooo`3oool07`3oool004T0oooo00<0omnL04P0001d_ol00`3oool0
0`3o_g@0001809cOo`060?ooo`070=nLB01d_ol0oooo0?oog`2LB00004RL0=ooo`090?ooo`@00000
K`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`0A0?ooo`8000007@3oool00`000000oooo0?ooo`0>0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`080?ooo`800000
;`3oool00`000000oooo0?ooo`1n0?ooo`<000008@3oool004T0oooo00<0omnL04P0001d_ol00`3o
ool00`3o_g@0001809cOo`060?ooo`070?oog`2LB7@0_ooo0?oog`2LB00004RL0=ooo`090?ooo`03
0000003oool0oooo06l0oooo0`00000@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`0B0?ooo`800000703oool00`000000oooo0?ooo`0?0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`080?ooo`80
0000;P3oool200000880oooo00<000000?ooo`3oool07`3oool004T0oooo00<0omnL04P0001d_ol0
0`3oool00`3o_g@0001809cOo`070?ooo`060?oOW018M;l0oooO09a80000B9`0gooo2@3oool00`00
0000oooo0?ooo`1`0?ooo`8000003`3oool300000180oooo00<000000?ooo`3oool00P3oool30000
01<0oooo0`00000K0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo00P0oooo0`00000]0?ooo`<00000PP3oool00`00
0000oooo0?ooo`0O0?ooo`00B@3oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00P0
oooo00D0gia807Bog`2LB00004RL0=ooo`090?ooo`030000003oool0oooo0780oooo00<000000?oo
o`3oool03@3oool200000180oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0B0?oo
o`800000703oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`090?ooo`800000;P3oool00`000000oooo0?ooo`210?oo
o`030000003oool0oooo01l0oooo001:0?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d_`3o
ool0203oool01@3ooml0W4Q804Q80000B9`0gooo00T0oooo00<000000?ooo`3oool0LP3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo0140oooo0P0000030?ooo`030000003oool0oooo
0180oooo00<000000?ooo`0000006`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`080?ooo`030000003oool00000
02d0oooo00<000000?ooo`3oool0PP3oool00`000000oooo0?ooo`0O0?ooo`00BP3oool01`3ookl0
M00007Boo`3oool0okmd0000M02oool02P3oool0103ogi`0B0000018W03Oool90?ooo`030000003o
ool0oooo07<0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0@0?ooo`<0000000<0
oooo0000003oool04`3oool010000000oooo0?ooo`00000J0?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0>0?ooo`8000002P3oool00`000000oooo
0000000]0?ooo`030000003oool0oooo0880oooo00<000000?ooo`3oool07`3oool004/0oooo00D0
oono07@000000000001d0;ooo`0<0?ooo`030=nLB000B9`0gooo00T0oooo00<000000?ooo`3oool0
L`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0140oooo00@000000?ooo`3oool0
0000503oool01@000000oooo0?ooo`3oool0000001T0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0>0?ooo`<000003P3oool3000000T0oooo00D000000?ooo`3oool0oooo0000000[
0?ooo`030000003oool0oooo08<0oooo00<000000?ooo`3oool07`3oool006P0oooo00<000000?oo
o`3oool0M03oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0100oooo00@000000?oo
o`3oool00000503oool01@000000oooo0?ooo`3oool0000001T0oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0203o
ool01@000000oooo0?ooo`3oool0000002/0oooo00<000000?ooo`3oool0P`3oool00`000000oooo
0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`1d0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0000000D0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool05@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`3oool03@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool0:03oool00`000000oooo0?ooo`240?ooo`030000003oool0oooo01l0oooo001X
0?ooo`030000003oool0oooo07D0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0@
0?ooo`800000503oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool0203oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo02P0oooo00<00000
0?ooo`3oool0Q03oool00`000000oooo0?ooo`0O0?ooo`00J03oool3000007@0oooo0`00000?0?oo
o`030000003oool0oooo0140oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo02H0oooo00<000000?ooo`3oool0Q03oool200000240oooo001X0?ooo`03
0000003oool0oooo07D0oooo0P00000>0?ooo`<000004@3oool2000001@0oooo00<000000?ooo`3o
ool00`3oool3000001@0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool01`3oool3000000D0oooo00<000000?ooo`3o
ool09@3oool3000008H0oooo00<000000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0
MP3oool00`000000oooo0?ooo`0=0?ooo`8000004@3oool00`000000oooo0000000C0?ooo`030000
003oool0oooo00@0oooo0P00000D0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00P0oooo0P0000060?ooo`030000
003oool0oooo02D0oooo0P0000270?ooo`030000003oool0oooo01l0oooo001X0?ooo`030000003o
ool0oooo07L0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`040000003o
ool0000000000140oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0A0?ooo`<00000
4`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0
QP3oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`1g0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0300000100oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`0@0?ooo`800000503oool00`000000oooo0?oo
o`0B0?ooo`800000203oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool08`3oool00`000000oooo0?ooo`270?ooo`030000003oool0oooo01l0oooo001X
0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0=
0?ooo`050000003oool0oooo0?ooo`0000004@3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0B0?ooo`<000001P3o
ool3000000T0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0S0?ooo`030000003o
ool0oooo08L0oooo00<000000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0N03oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00/0oooo0`0000030?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool04`3oool2000000H0oooo0P0000090?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`280?ooo`030000003oool0
oooo01l0oooo001X0?ooo`030000003oool0oooo07P0oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0?ooo`0;0?ooo`800000103oool00`000000oooo0?ooo`0<0?ooo`<000002P3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool05@3oool00`00
0000oooo000000020?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0203oool30000
0280oooo00<000000?ooo`3oool0R03oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000
oooo0?ooo`1i0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0=0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool05P3oool5000000X0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo08T0
oooo00<000000?ooo`3oool07`3oool006P0oooo0`00001h0?ooo`<000003`3oool2000000X0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00T0oooo
0`00000>0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool05`3oool00`000000oooo
0?ooo`0:0?ooo`<000002`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo08T0oooo
0P00000Q0?ooo`00J03oool00`000000oooo0?ooo`1i0?ooo`8000003`3oool3000000T0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo0P00
000=0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0903oool2000000d0oooo00<0
00000?ooo`3oool07@3oool3000008/0oooo00<000000?ooo`3oool07`3oool006P0oooo00<00000
0?ooo`3oool0NP3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool02`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo02@0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`0M0?ooo`800000S03oool00`000000oooo0?ooo`0O
0?ooo`00J03oool00`000000oooo0?ooo`1k0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00P0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo01H0oooo00<000000?oo
o`3oool08`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01d0oooo00<000000?oo
o`3oool0R`3oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`1l0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`080?ooo`<0
00002@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool05P3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`2<0?ooo`030000003oool0oooo01l0oooo001X0?ooo`030000
003oool0oooo07`0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00T0oooo0P0000080?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo08`0oooo00<000000?ooo`3oool0
7`3oool006P0oooo00<000000?ooo`3oool0O@3oool2000000l0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`060?ooo`<00000603oool00`000000oooo0?ooo`0R0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`2=0?ooo`030000003o
ool0oooo01l0oooo001X0?ooo`030000003oool0oooo07d0oooo0`00000>0?ooo`040000003oool0
oooo0?ooo`<000003@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0240oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo08d0oooo00<0
00000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0OP3oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo008000003`3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo0180oooo0P0000060?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0803oool2
000001@0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`2>0?ooo`030000003oool0
oooo01l0oooo001X0?ooo`030000003oool0oooo07h0oooo00<000000?ooo`3oool03@3oool30000
00030?ooo`000000oooo00l0oooo00<000000?ooo`3oool00P3oool3000001<0oooo0`0000050?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool07`3oool3000001<0oooo0`00000G0?oo
o`<00000S`3oool00`000000oooo0?ooo`0O0?ooo`00J03oool3000007l0oooo00<000000?ooo`3o
ool03@3oool300000180oooo0P0000020?ooo`8000005@3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0C0?ooo`80
00005`3oool2000008l0oooo0P00000Q0?ooo`00J03oool00`000000oooo0?ooo`200?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool0403oool4000001P0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0
5@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0900oooo00<000000?ooo`3oool0
7`3oool006P0oooo00<000000?ooo`3oool0P03oool00`000000oooo0000000=0?ooo`030000003o
ool000000180oooo00<000000?ooo`000000603oool01@000000oooo0?ooo`3oool0000001`0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool0T03oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?oo
o`210?ooo`<000002`3oool2000000030?ooo`000000oooo0140oooo0P00000J0?ooo`040000003o
ool0oooo000001`0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0G0?ooo`030000
003oool0oooo0180oooo00<000000?ooo`3oool0T@3oool00`000000oooo0?ooo`0O0?ooo`00J03o
ool00`000000oooo0?ooo`220?ooo`8000002P3oool300000080oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0000001d0oooo00<000000?ooo`3oool07P3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo0100oooo0P00002C0?ooo`030000003o
ool0oooo01l0oooo001;0?ooo`050?oOW018000000000000M02oool00`3oool00`3o_g@0000007Bo
o`030?ooo`050?oOW018000000000000001d_ol02P3oool00`000000oooo0?ooo`240?ooo`030000
003oool0000000L0oooo0P0000020?ooo`<00000403oool00`000000oooo0000000J0?ooo`800000
7@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01T0oooo0P00000@0?ooo`<00000
T`3oool00`000000oooo0?ooo`0O0?ooo`00BP3oool01`3ookl0M00007Boo`3oool0okmd0000M02o
ool00P3oool00`3o_g@0000007Boo`020?ooo`070?oOW0180000B9cO0?ooo`3ookl0M00007Boo`09
0?ooo`030000003oool0oooo08D0oooo0`0000030?ooo`030000003oool0000000D0oooo0P00000?
0?ooo`@000006P3oool3000001`0oooo00<000000?ooo`3oool0703oool2000001/0oooo0`00000?
0?ooo`800000U03oool00`000000oooo0?ooo`0O0?ooo`00BP3oool00`2oM00004RL0=ooo`020?oo
o`030;md0000M;l0oooo00D0oooo00<0oooO09a80000M;l00`3oool00`3o_g@004RL0=ooo`080?oo
o`030000003oool0oooo08H0oooo1`0000080?ooo`030000003oool0oooo00`0oooo0P0000020?oo
o`800000603oool3000001`0oooo00<000000?ooo`3oool06`3oool3000001`0oooo0P00000>0?oo
o`030000003oool0oooo09@0oooo00<000000?ooo`3oool07`3oool004T0oooo00<0oono07@00018
W=l00`3oool00`3OW4P0001d0;ooo`050?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02L
gol0203oool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`00
00002P3oool00`000000oooo0?ooo`020?ooo`<000005P3oool00`000000oooo0000000M0?ooo`03
0000003oool0oooo01`0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`080?ooo`<0
0000U`3oool00`000000oooo0?ooo`0O0?ooo`00B@3oool00`3ogi`0B00007Boo`030?ooo`030?no
M00004P0W=oo00D0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`080?ooo`@00000
UP3oool3000000H0oooo00<000000?ooo`0000001P3oool00`000000oooo0?ooo`0E0?ooo`040000
003oool0oooo000001`0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0O0?ooo`03
0000003oool0000000H0oooo0`00002F0?ooo`<000008@3oool004T0oooo00<0omnL04P0001d_ol0
0`3oool00`3o_g@0001809cOo`050?ooo`030?oOW0180000M;oo00<0oooo00<0_g@00000B02Lgol0
203oool00`000000oooo0?ooo`2H0?ooo`8000000P3oool00`000000oooo0?ooo`03000000P0oooo
00<000000?ooo`3oool04`3oool01@000000oooo0?ooo`3oool0000001`0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0P0?ooo`<000000P3oool00`000000oooo0?ooo`02000009T0
oooo00<000000?ooo`3oool07`3oool004T0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@00018
09cOo`050?ooo`080?oo_`1d0000000004RLg`3oool0okmd00000000M;l90?ooo`030000003oool0
oooo09X0oooo1`00000:0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool08P3oool7
000009/0oooo00<000000?ooo`3oool07`3oool004T0oooo00<0omnL04P0001d_ol00`3oool00`3o
_g@0001809cOo`060?ooo`030;md0000B9`0_ia80080000000<004RL0=ooo`3oool0203oool00`00
0000oooo0?ooo`2L0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`0000004@3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`0V0?ooo`030000003oool0oooo09/0oooo00<000000?ooo`3oool07`3oool004T0
oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`060?ooo`030?oOW0180000M;oo00d0
oooo00<000000?ooo`3oool0[03oool3000000l0oooo0`0000050?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`350?ooo`030000003oool0oooo01l0
oooo001:0?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d_`3oool01`3oool00`3OW4P00018
09cOo`0<0?ooo`030000003oool0oooo0:d0oooo0P00000?0?ooo`8000001`3oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo01L0oooo0P0000380?ooo`030000003oool0oooo01l0oooo
001:0?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`090?ooo`030?noM000000007Bo00/0
oooo00<000000?ooo`3oool0[`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00H0
oooo0`00000H0?ooo`030000003oool0oooo01H0oooo0`0000380?ooo`030000003oool0oooo01l0
oooo001;0?ooo`050?oo_`1d000000000000M02oool02`3oool01@3ooml0W4P00000000007@0_ooo
00P0oooo00<000000?ooo`3oool0/03oool3000000L0oooo0`00000:0?ooo`800000603oool00`00
0000oooo0?ooo`0G0?ooo`030000003oool0oooo0<L0oooo00<000000?ooo`3oool07`3oool006P0
oooo00<000000?ooo`3oool0/@3oool300000080oooo00<000000?ooo`3oool00`00000=0?ooo`03
0000003oool0oooo01D0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`380?ooo`03
0000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo0;80oooo00<000000?ooo`000000
100000000`3oool000000?ooo`0=0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`380?ooo`030000003oool0oooo01l0oooo001X0?ooo`<00000]P3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool05@3o
ool00`000000oooo0?ooo`380?ooo`8000008@3oool006P0oooo00<000000?ooo`3oool0b`3oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0bP3oool0
0`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`3;0?ooo`800000503oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo0</0oooo00<000000?ooo`3oool07`3oool006P0
oooo00<000000?ooo`3oool0b`3oool3000001<0oooo00<000000?ooo`3oool04P3oool300000<`0
oooo00<000000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0c03oool2000001<0oooo
00<000000?ooo`3oool04P3oool200000<d0oooo00<000000?ooo`3oool07`3oool006P0oooo00<0
00000?ooo`3oool0c@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool0c@3oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`3>
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3=
0?ooo`030000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo0<l0oooo0P00000@0?oo
o`030000003oool0oooo00l0oooo0P00003@0?ooo`030000003oool0oooo01l0oooo001X0?ooo`03
0000003oool0oooo0<l0oooo0`00000?0?ooo`030000003oool0oooo00h0oooo0`00003@0?ooo`03
0000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo0=00oooo0P00000?0?ooo`030000
003oool0oooo00h0oooo0P00003A0?ooo`030000003oool0oooo01l0oooo001X0?ooo`<00000dP3o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0d03o
ool200000240oooo001X0?ooo`030000003oool0oooo0=<0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0=40oooo00<000000?ooo`3oool07`3oool0
06P0oooo00<000000?ooo`3oool0e03oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`000000e03oool00`000000oooo0?ooo`0O0?ooo`00J03oool00`000000
oooo0?ooo`3C0?ooo`<000002`3oool00`000000oooo0?ooo`090?ooo`<00000e@3oool00`000000
oooo0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`3D0?ooo`030000003oool0000000X0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3D0?ooo`030000003oool0oooo01l0oooo
001X0?ooo`030000003oool0oooo0=L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`090?ooo`030000003oool0oooo0=D0oooo00<000000?ooo`3oool07`3oool006P0oooo00<00000
0?ooo`3oool0f03oool2000000L0oooo00<000000?ooo`3oool01P3oool300000=P0oooo00<00000
0?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0f03oool3000000H0oooo00<000000?oo
o`3oool01@3oool300000=T0oooo00<000000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3o
ool0f@3oool01@000000oooo0000003oool0000000<0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0000000200000=X0oooo00<000000?ooo`3oool07`3oool006P0oooo0`00003L0?ooo`H0
000000<0oooo0000000000000`00003L0?ooo`8000008@3oool006P0oooo00<000000?ooo`3oool0
g@3oool00`000000oooo0?ooo`03000000<0oooo00<000000?ooo`3oool0g03oool00`000000oooo
0?ooo`0O0?ooo`00J03oool00`000000oooo0?ooo`3Q0?ooo`030000003oool0oooo0>40oooo00<0
00000?ooo`3oool07`3oool006P0oooo00<000000?ooo`3oool0h@3oool00`000000oooo0?ooo`3Q
0?ooo`030000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo03@0oooo00<000000?oo
o`3oool0E03oool00`000000oooo0?ooo`1C0?ooo`030000003oool0oooo05@0oooo00<000000?oo
o`3oool0E03oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo01l0oooo001X0?ooo`03
0000003oool0oooo03@0oooo00<000000?ooo`3oool0E03oool00`000000oooo0?ooo`1C0?ooo`03
0000003oool0oooo05@0oooo00<000000?ooo`3oool0E03oool00`000000oooo0?ooo`0c0?ooo`03
0000003oool0oooo01l0oooo001X0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
7`3oool006P0ooooo`00003:00000240oooo003o0?ooool0ooooE03oool00?l0ooooo`3ooomD0?oo
o`00o`3ooooo0?oooe@0oooo003o0?ooool0ooooE03oool00?l0ooooo`3ooomD0?ooo`00o`3ooooo
0?oooe@0oooo003o0?ooool0ooooE03oool00?l0ooooo`3ooomD0?ooo`00f@3oool01P3ooml0W4P0
000000000000001d0;ooool0ooooM03oool00=/0oooo00<0_g@00018W03Oool0F@3oool00`2oM000
04RL0=ooo`3o0?oooaT0oooo003K0?ooo`030;md0000B9`0gooo05X0oooo00<0omnL09boo`3oool0
o`3ooolH0?ooo`00f`3oool00`2oM00004RL0=ooo`1K0?ooo`030;nLW03Oool0oooo0?l0oooo5`3o
ool00=/0oooo00<0_g@00018B01800000P0000000`18W=l0oooo0?ooo`020?ooo`050=nLB0000000
00000000001d_ol00P3oool01@3OW4P000000000000007@0M7@00080000000<0001d0;oog`2LB000
0P0000002@00B9`0gooo0?oOW0180000000004RLg`3ookl0_kno07@000020000000707Boo`3oool0
oooo0?oog`2LB000001d0;ooo`080?ooo`0;0;md0000000000000000M02oool0oooo0;md00000000
07Bo0?ooo`3OW4P00P0000000`00B9`0gooO09a80002000000040018W02LM0000000001d_`80oooo
00D0_g@0000000000000001809cOo`020?ooo`0:0?oOW0180000000004RLg`3oool0oooO09a80000
000004RL0=ooo`H0oooo00D0_g@0000000000000001809cOo`020?ooo`090?oo_`1d000000000000
0018W;l0W4P00018W03Oool0_g@00080000000<0001809cOo`3oool00`3oool00`3ooml0W4Qd0;oo
o`050?ooo`090;md0000000000000000M02oool0oooo0?oo_`2o_kl0M0000080000000D0M;oo0?oo
g`2LB000001d0;ooo`060?ooo`060;md0000000000000000M02ooml0W4P00P0000001000B9`0gooo
0?oo_`1d00020000000504RL_`2LB00004RL0=ooo`3OW4P00P0000000`0004P0W=oO09a800020000
00030000M02ooml0W4P00080000000@007Bo0?ooo`3oool0_g@00P000000100007@0_ooo0?ooo`2o
M002000000040000B02Lgol0oooo0=nLB080000000@0001d0;ooo`3oool0_g@00P0000001@0004P0
W=oo0?ooo`3ooml0W4P00080000000D004RL09ad0000000007Bo0=nLB002000000030000M02oool0
oooo09<0oooo003K0?ooo`070;md00000000M;oo0?ooo`3ookl0M00004RLg`020?ooo`070=nLB000
0000B9cO0?ooo`3ooml0W4P004RLg`020?ooo`0;0=nLB00007@0_ooo0?ooo`3OW4P004RL0=ooo`3o
ool0oooO09a80000M;l00P3oool0403OW4P0001d0;ooo`3O_kl0gono07@0B02Lgol0oooo0?oo_`1d
04P0W=oo0?ooo`3ooml0W4P00000M02oool70?ooo`0Y0;md0000000007Bo0?ooo`3ogi`0B01d0;oo
o`3ooml0W4Qd0;ooo`3oool0oooO09a80000B9`0gooo0?noM0000000M;oo0?ooo`3ogi`0B000001d
_`3o_kl0_ooo0;md0000000004RL0=ooo`3ooml0W4Qd0;ooo`3oool0gia80000M02oool0gkno0=oo
o`3oool0_g@00000001d_ol01P3oool03@2oM00000000018W03Oool0oooO09a8M02oool0oooo0=nL
B00007@0_ooo0?ooo`3OW4P00P0000001@0007@0_ooo0=nLB00007@0_ooo00D0oooo00<0okmd0000
0018W=l0103oool04@2oM0000000001d_`3oool0omnL04P0M02oool0oono07@0B02Lgol0oooo0?oo
_`1d04P0W=oO09a8000007@0_ooo00L0oooo00L0gia80018W03Oool0oooo0?oog`2LB00007Bo0080
oooo00D0gia80000M02oool0oooo0=nLB002000000050000M02oool0gia80000M02oool00P3oool0
1`2oM00004RL0=ooo`3oool0oooO09a80000M;l00P3oool02`2oM0000000001d_`3oool0omnL04P0
M02oool0oooo0=nLB00007@0_ooo00<0oooo01L0gia80000M02oool0oooo0;md0000000004RL0=oo
o`3ooml0W4Qd0;ooo`3o_g@0000007Boo`3oool0omnL04P00000M;l0okno0;ooo`3OW4P0001d0;oo
o`2E0?ooo`00f`3oool00`2oM00004RL0=ooo`020?ooo`060?noM00007@0_ooo0?oOW018000007Bo
0`3oool03`3o_g@0001d0;ooo`3oool0gia80000M02oool0oooo0=nLB000B9`0gooo0?ooo`3ooml0
W4P0001d_`020?ooo`060=nLB000B9`0gooo0?ooo`3ookl0_koo0P3oool00`3o_g@0000007Boo`0;
0?ooo`030?oOW0180000B9cO00<0oooo01D0omnL07BLg`3oool0oono07A8W03Oool0_g@00000M02o
ool0oooo0?noM00004P0W=oo0?ooo`3ooml0W4P0001d_`3oool0omnL04P00000M;l00`3oool01P3o
gi`0W;oo0?ooo`3OW4P004RL0=ooo`/0oooo00<0omnL04P00000M;l00`3oool02P3ogi`0W;oo0?oo
o`2oM00004RL0=ooo`3oool0oooO09a80000M;l20?ooo`030=nLB00007@0_ooo00@0oooo00D0oooO
09a80000B9`0W7Ad0;ooo`020?ooo`030?oOW0180000B9cO00<0oooo00@0omnL07BLg`3ookl0_koo
0P3oool00`3o_g@0000007Boo`0:0?ooo`070=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`020?oo
o`070;md0000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`030=nLB00007@0_ooo0080oooo00/0
_g@00018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3ogi`0B00004RLg`030?ooo`060?oOW01dW=l0
oooo0=nLB00007@0_ooo0`3oool01P3OW4P0001d0;ooo`3ogi`0B000001d_`<0oooo00T0omnL09bo
o`3o_g@0001809cOo`3oool0oooO09a80000M;l00P3oool00`3OW4P0001d0;ooo`2E0?ooo`00f`3o
ool00`2oM00004RL0=ooo`020?ooo`060?oOW0180000M;oo0?noM0000000M;oo0`3oool03`3o_g@0
000007Boo`3oool0gia80000M02oool0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`020?oo
o`030=nLB000B9`0gooo00<0oooo00D0oono07@000000000001d0;ooo`0;0?ooo`030?noM00004P0
W=oo00L0oooo00@0omnL07A8B0000000M;oo0`3oool02P3o_g@00000001d_`3ooml0W4P0001d_`3o
ool0okmd0000B02Lgol60?ooo`030=nLB000B9`0gooo00/0oooo00<0okmd0000B02Lgol01P3oool0
1`2oM00004RL0=ooo`3oool0oooO09a80000M;l00P3oool00`3OW4P0001d0;ooo`040?ooo`050?oO
W0180000M;oo0?noM018W=l00P3oool00`3o_g@0001809cOo`070?ooo`050?oo_`1d000000000000
M02oool02P3oool01`3OW4P004RL0=ooo`3oool0oooO09a80000M;l00P3oool01`2oM00004RL0=oo
o`3oool0oooO09a80000M;l00P3oool00`3OW4P0001d0;ooo`020?ooo`0;0;md0000B9`0gooo0?oo
o`3ooml0W4P0001d_`3oool0okmd0000B02Lgol01P3oool00`3OW4P0001d0;ooo`030?ooo`060=nL
B00007@0_ooo0?noM00004P0W=oo1P3oool01P3o_g@00000001d_`3ooml0W4P0001d_`80oooo00<0
gia80000M02oool0U@3oool00=/0oooo00<0_g@00018W03Oool00P3oool01P3ogi`0B00007Boo`3o
_g@0000007Boo`<0oooo00l0okmd0000001d_ol0oooo0=nLB00007@0_ooo0?ooo`3OW4P004RL0=oo
o`3oool0oooO09a80000M;l00P3oool00`3OW4P004RL0=ooo`020?ooo`040;md00000000000004RL
g`d0oooo00<0okmd0000M02oool01`3oool00`3ookl0M00004RLg`060?ooo`080?noM00000000000
001d_`3oool0okmd0000M02oool60?ooo`030=nLB000B9`0gooo00/0oooo00<0okmd0000M02oool0
1P3oool01`2oM00004RL0=ooo`3oool0oooO09a80000M;l00P3oool00`3OW4P0001d0;ooo`040?oo
o`0:0;md0000B9`0gooo0?oog`2LB7@0_ooo0?ooo`3o_g@0001d0;ooo`H0oooo00@0_g@000000000
0000B9cO303oool01`3OW4P004RL0=ooo`3oool0oooO09a80000M;l00P3oool01`2oM00004RL0=oo
o`3oool0oooO09a80000M;l00P3oool00`3OW4P0001d0;ooo`020?ooo`0;0;md0000B9`0gooo0?oo
o`3ooml0W4P0001d_`3oool0okmd0000M02oool01P3oool00`3OW4P0001d0;ooo`030?ooo`060=nL
B00007@0_ooo0?noM00007@0_ooo203oool0103o_g@0000000000000M;l20?ooo`030=nLB00007@0
_ooo09D0oooo003K0?ooo`030;md000007@0_ooo0080oooo00H0gia80000B02Lgol0oono07@0001d
_ol30?ooo`0?0;md000004P0W=oo0?ooo`3OW4P0001d0;ooo`3oool0_g@00000M02oool0oooo0?oo
g`2LB00007Bo0080oooo00L0gia80018W03Oool0oooo0?oOW0180000B9cO0080oooo00<0_ibL0=oo
o`3oool02P3oool00`3ogi`0B000000000040000000704RLg`3oool0oooO09a80000B9`0W7A809cO
o`020?ooo`0;0?oo_`1d0000M;oo0?ooo`3ooml0W4P0001d_`3oool0omnL04P0B02Lgol00P3oool0
1`3o_g@0001809cOo`3oool0gia80018W03Oool02`3oool00`3ogi`0B01809cOo`020?ooo`0;0?no
M00004P0W=oo0?ooo`2oM00004RL0=ooo`3oool0oooO09a80000M;l00P3oool00`3OW4P0001809cO
o`030?ooo`030?oOW0180000M;oo0080oooo00D0okmd04RLg`3oool0omnL04P000050000000404RL
g`3ogi`0B00004RLg`80oooo00<0_ibL0=ooo`3oool02@3oool01`2oM000001d0;ooo`3oool0oooO
09a80000M;l00P3oool01`2oM00004RL0=ooo`3oool0oooO09a80000M;l00P3oool00`3OW4P00018
09cOo`020?ooo`0:0;md000007@0_ooo0?ooo`3ooml0W4P0001d_`3oool0omnL04P000D0000000D0
B9cO0?ooo`3OW4P0001809cOo`030?ooo`060=nLB00007@0_ooo0?oOW01804P0W=oo0P3oool02P3o
_g@0001809cOo`3ookl0M00007Boo`3oool0oooO09a80000M;l20?ooo`030=nLB00007@0_ooo09D0
oooo003I0?ooo`0O0?oo_`1d000000000018B018B9`0gooo0?oOW018000004RL0=ooo`3oool0gia8
0018W03Oool0oooo0?noM000000007Bo0?oog`2LB00000000000M02ooml0W4P000000000B4P0B018
09cOo`3ookl0M00004RLg`020?ooo`0@0=nLB000B9`0gooo0?ooo`3ookl0M01d0;ooo`3oool0okmd
0018W03Oool0oooo0?oog`2LB000001d0;ooo`L0oooo01H0_g@004RLg`3oool0oooo0=nLB00004P0
W=oo0?ooo`2oM000001809cOo`3ooml0W4Qd0;ooo`3oool0oono07@00018W=l0oooo0?oo_`1d0000
07Bo0P3oool02`2oM00004RL0=ooo`3ooml0W4P00000M02oool0oooo0=nLB000B9`0gooo00`0oooo
01X0_g@00018W03Oool0oooO09a8000007@0_ooo0?ooo`2oM00004RL0=ooo`3oool0oooO09a80000
M;l0oooO09a800000000001804QdW03O_g@0001d0;oog`2LB000001d0;ooo`80oooo00H0oooO09a8
M02oool0oooo0;md0018W=l20?ooo`0=0=nLB00004P0W=oo0?oo_`1d07@0_ooo0?ooo`3o_g@004RL
0=oog`2LB000001d0;ooo`050?ooo`090?oog`2LB00000000018B01804P0W=oo0?oo_`1d0000B9cO
0080oooo01D0_g@00018W03Oool0oooo0?oog`2LB00007Bo0?oog`2LB00000000000B018B4P0W=oo
0?oo_`1d000007Ad07@0M02oool0oono07@00018W=l00P3oool0802oM000B9cO0?ooo`3oool0gia8
0000B02Lgml0W4P00000000004P0B7BL0=noM00004P0B4P00000000007@0_ooo0?ooo`2oM00004RL
0=ooo`3ooml0W4P00000M02oool0oono07@00018W=l0oooo0?oo_`1d000007Bo0P3oool00`3OW4P0
001d0;ooo`2E0?ooo`00fP3oool0203ooml0W4P00018W03Ogi`0B0000000000007@0_ooo0`3oool0
1@3o_g@00000000000000000B9cO00<0oooo00d0_g@00000M02oool0oooo0;md0000B9`0gooO09a8
00000000001d0;ooo`3oool0_g@000<0000000<004RL0=ooo`3o_g@00P0000002000M;l0_ibL0=oo
o`3oool0oooO09a8000007@0_ooo203oool01`3OW4P000000000000004P0W=oo0?oog`2LB0000P00
00002000M;l0okmd0000000007@0_ooo0?ooo`3ookl0M0000P0000000`0007@0_ooo0?ooo`020?oo
o`070=nLB000000000000000B02Lgol0oooo0;md0003000000030018W03Oool0oooo00X0oooo00h0
gia8000000000000001809cOo`3ookl0M00000000000B9`0gooo0?oOW01800000000001d_`80oooo
00H0_g@00000M02ogi`0B0000000B018B002000000090000M02oool0oooO09a80000000004RL0=oo
o`3oool0gia80080000000D0001809cOo`3oool0oooo0?noM00200000006001d_`2oW9`0gooO09a8
000007@0_ooo1`3oool0402oM00004RL0=oog`2LB00000000000M02oool0oono07@00000000004RL
0=ooo`3ogi`0B00000000000M;l20?ooo`0<0;md000007@0_ooO09a800000000001809cOo`3ookl0
M0000000000007@0_ooo0`3oool01@3OW4P000000000000004P0W=oo0080oooo00T0_g@00000M02o
gi`0B0000000M02oool0_g@00000M02oool00P3oool01@3OW4P000000000000004P0W=oo0080oooo
00H0oono07@00000000000000000M02oool20?ooo`030=nLB00007@0_ooo09D0oooo003j0?ooo`03
0;md0000B9`0gooo03@0oooo00<0_g@00018W03Oool0QP3oool00`3OW4P0001d0;ooo`2E0?ooo`00
nP3oool00`3ookl0M4RL0=ooo`0d0?ooo`030?oo_`1dB9`0gooo08H0oooo00<0gia80000M02oool0
U@3oool00?X0oooo00<0oooO09adW03Oool0=03oool00`3ooml0W7BL0=ooo`260?ooo`030=nLB000
07@0_ooo09D0oooo003]0?ooo`030=nLB00004P0W=oo0;H0oooo00<0gia80000B02Lgol03`3oool0
1@3ooml0W4P00000000007@0_ooo09D0oooo003]0?ooo`030=nLB00004P0W=oo0;H0oooo00<0gia8
0000B02Lgol04@3oool00`2oM000001d0;ooo`2E0?ooo`00o`3ooooo0?oooe@0oooo003o0?ooool0
ooooE03oool00?l0ooooo`3ooomD0?ooo`00o`3ooooo0?oooe@0oooo0000\
\>"],
  ImageRangeCache->{{{0, 593}, {365.75, 0}} -> {-7.66472, -0.963189, \
0.0345328, 0.00779691}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Properties of the double-well potential", "Section"],

Cell["\<\
For low energies, the potential plotted below can be described as two wells \
separated by a barrier.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(FilledPlot[{pot[x], 4}, {x, \(-5\), 5}, 
        PlotRange \[Rule] {All, {2, 8}}, Frame \[Rule] True, 
        FrameLabel \[Rule] {"\<x\>", "\<V[x]\>"}, 
        PlotLabel \[Rule] "\<a double-well potential\>"];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 1 0 0.618034 [
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% Start of sub-graphic
p
0 0 1 0.618034 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0952381 -0.206011 0.103006 [
[.11905 -0.0125 -9.15625 -15.6875 ]
[.11905 -0.0125 9.15625 0 ]
[.30952 -0.0125 -9.15625 -15.6875 ]
[.30952 -0.0125 9.15625 0 ]
[.5 -0.0125 -5 -15.6875 ]
[.5 -0.0125 5 0 ]
[.69048 -0.0125 -5 -15.6875 ]
[.69048 -0.0125 5 0 ]
[.88095 -0.0125 -5 -15.6875 ]
[.88095 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -5.0625 -31.375 ]
[.5 -0.0125 5.0625 -15.6875 ]
[-0.0125 .10301 -10 -7.84375 ]
[-0.0125 .10301 0 7.84375 ]
[-0.0125 .20601 -10 -7.84375 ]
[-0.0125 .20601 0 7.84375 ]
[-0.0125 .30902 -10 -7.84375 ]
[-0.0125 .30902 0 7.84375 ]
[-0.0125 .41202 -10 -7.84375 ]
[-0.0125 .41202 0 7.84375 ]
[-0.0125 .51503 -10 -7.84375 ]
[-0.0125 .51503 0 7.84375 ]
[-0.0125 .61803 -10 -7.84375 ]
[-0.0125 .61803 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -25.6875 -13.3125 ]
[-0.01875 .30902 -10 13.3125 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -58.9375 0 ]
[.5 .63053 58.9375 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.11905 0 m
.11905 .00625 L
s
gsave
.11905 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.30952 0 m
.30952 .00625 L
s
gsave
.30952 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.5 0 m
.5 .00625 L
s
gsave
.5 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.69048 0 m
.69048 .00625 L
s
gsave
.69048 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.88095 0 m
.88095 .00625 L
s
gsave
.88095 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.125 Mabswid
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.07143 0 m
.07143 .00375 L
s
.02381 0 m
.02381 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -66.0625 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
69.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .10301 m
.00625 .10301 L
s
gsave
-0.0125 .10301 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
0 .20601 m
.00625 .20601 L
s
gsave
-0.0125 .20601 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
0 .30902 m
.00625 .30902 L
s
gsave
-0.0125 .30902 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
0 .41202 m
.00625 .41202 L
s
gsave
-0.0125 .41202 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
0 .51503 m
.00625 .51503 L
s
gsave
-0.0125 .51503 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7) show
1.000 setlinewidth
grestore
0 .61803 m
.00625 .61803 L
s
gsave
-0.0125 .61803 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .0824 m
.00375 .0824 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .16481 m
.00375 .16481 L
s
0 .18541 m
.00375 .18541 L
s
0 .22661 m
.00375 .22661 L
s
0 .24721 m
.00375 .24721 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .32962 m
.00375 .32962 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .49443 m
.00375 .49443 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .57683 m
.00375 .57683 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -92.1563 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
74.3125 11.8438 translate
90 rotate
-74.3125 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(V) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
71.625 15.813 moveto
(@) show
75.563 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
81.688 15.813 moveto
(D) show
85.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.11905 .61178 m
.11905 .61803 L
s
.30952 .61178 m
.30952 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.69048 .61178 m
.69048 .61803 L
s
.88095 .61178 m
.88095 .61803 L
s
.125 Mabswid
.16667 .61428 m
.16667 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.2619 .61428 m
.2619 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.7381 .61428 m
.7381 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.07143 .61428 m
.07143 .61803 L
s
.02381 .61428 m
.02381 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.97619 .61428 m
.97619 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 0 m
1 0 L
s
.99375 .10301 m
1 .10301 L
s
.99375 .20601 m
1 .20601 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .41202 m
1 .41202 L
s
.99375 .51503 m
1 .51503 L
s
.125 Mabswid
.99625 .0206 m
1 .0206 L
s
.99625 .0412 m
1 .0412 L
s
.99625 .0618 m
1 .0618 L
s
.99625 .0824 m
1 .0824 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .14421 m
1 .14421 L
s
.99625 .16481 m
1 .16481 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .22661 m
1 .22661 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26781 m
1 .26781 L
s
.99625 .28842 m
1 .28842 L
s
.99625 .32962 m
1 .32962 L
s
.99625 .35022 m
1 .35022 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .39142 m
1 .39142 L
s
.99625 .43262 m
1 .43262 L
s
.99625 .45322 m
1 .45322 L
s
.99625 .47383 m
1 .47383 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .53563 m
1 .53563 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57683 m
1 .57683 L
s
.99625 .59743 m
1 .59743 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
.5 0 m
.5 .61803 L
s
gsave
.5 .63053 -119.938 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(a) show
71.375 15.813 moveto
(double) show
103.938 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
111.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(well) show
135.125 15.813 moveto
(potential) show
176.875 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.23115 .61803 m
.26171 .43981 L
.28158 .33843 L
.30316 .24121 L
.32216 .16893 L
.33296 .13461 L
.34309 .10772 L
.35264 .08759 L
.35756 .0794 L
.36292 .07226 L
.36584 .06919 L
.36855 .06686 L
.37114 .06512 L
.37247 .06441 L
.3739 .06379 L
.37532 .06332 L
.37662 .06301 L
.37786 .06284 L
.37917 .06278 L
.38035 .06283 L
.38145 .06296 L
.38266 .06322 L
.38395 .06361 L
.38517 .06409 L
.38651 .06475 L
.38924 .06648 L
.39176 .06857 L
.39413 .07095 L
.39945 .07775 L
.40519 .08725 L
.41569 .10993 L
.42531 .13578 L
.44584 .20002 L
.45585 .23154 L
.46494 .25769 L
.47378 .27923 L
.47843 .28855 L
.48336 .29668 L
.48587 .30007 L
.48863 .3032 L
.49117 .30549 L
.49349 .3071 L
.49471 .30775 L
.49587 .30824 L
.49709 .30863 L
.49778 .30879 L
.49843 .3089 L
.4996 .30901 L
.50066 .309 L
.50189 .30885 L
Mistroke
.50305 .3086 L
.50421 .30821 L
.50546 .30766 L
.50772 .30632 L
.51039 .30415 L
.51283 .30162 L
.51814 .2944 L
.52316 .28553 L
.54165 .23907 L
.5599 .18148 L
.58003 .12092 L
.58957 .09776 L
.59984 .0788 L
.60551 .07136 L
.60798 .06881 L
.61069 .06654 L
.61326 .06487 L
.61565 .06376 L
.61697 .06332 L
.61817 .06303 L
.61884 .06292 L
.61956 .06283 L
.62086 .06278 L
.6221 .06284 L
.62323 .06299 L
.62453 .06328 L
.62577 .06367 L
.62796 .06463 L
.63035 .06607 L
.63298 .06811 L
.63577 .07081 L
.64078 .07699 L
.65078 .09418 L
.66184 .12011 L
.68116 .18051 L
.70201 .26313 L
.74311 .46596 L
Mfstroke
.74311 .46596 m
.76865 .61803 L
s
.02381 .20601 m
.06244 .20601 L
.10458 .20601 L
.14415 .20601 L
.18221 .20601 L
.22272 .20601 L
.26171 .20601 L
.30316 .20601 L
.34309 .20601 L
.3815 .20601 L
.42237 .20601 L
.46172 .20601 L
.49955 .20601 L
.53984 .20601 L
.57861 .20601 L
.61984 .20601 L
.65954 .20601 L
.69774 .20601 L
.73838 .20601 L
.77751 .20601 L
.81909 .20601 L
.85916 .20601 L
.89771 .20601 L
.93871 .20601 L
.97619 .20601 L
s
0 1 1 r
.02381 2.36913 m
.06244 1.96822 L
.10458 1.56964 L
.14415 1.23202 L
.18221 .9409 L
.22272 .66724 L
.26171 .43981 L
.28158 .33843 L
.30316 .24121 L
.32216 .16893 L
.33296 .13461 L
.34309 .10772 L
.35264 .08759 L
.35756 .0794 L
.36292 .07226 L
.36584 .06919 L
.36855 .06686 L
.37114 .06512 L
.37247 .06441 L
.3739 .06379 L
.37532 .06332 L
.37662 .06301 L
.37786 .06284 L
.37917 .06278 L
.38035 .06283 L
.38145 .06296 L
.38266 .06322 L
.38395 .06361 L
.38517 .06409 L
.38651 .06475 L
.38924 .06648 L
.39176 .06857 L
.39413 .07095 L
.39945 .07775 L
.40519 .08725 L
.41569 .10993 L
.42531 .13578 L
.44584 .20002 L
.45585 .23154 L
.46494 .25769 L
.47378 .27923 L
.47843 .28855 L
.48336 .29668 L
.48587 .30007 L
.48863 .3032 L
.49117 .30549 L
.49349 .3071 L
.49471 .30775 L
.49587 .30824 L
.49709 .30863 L
.49778 .30879 L
.49843 .3089 L
.4996 .30901 L
.50066 .309 L
.50189 .30885 L
.50305 .3086 L
.50421 .30821 L
.50546 .30766 L
.50772 .30632 L
.51039 .30415 L
.51283 .30162 L
.51814 .2944 L
.52316 .28553 L
.54165 .23907 L
.5599 .18148 L
.58003 .12092 L
.58957 .09776 L
.59984 .0788 L
.60551 .07136 L
.60798 .06881 L
.61069 .06654 L
.61326 .06487 L
.61565 .06376 L
.61697 .06332 L
.61817 .06303 L
.61884 .06292 L
.61956 .06283 L
.62086 .06278 L
.6221 .06284 L
.62323 .06299 L
.62453 .06328 L
.62577 .06367 L
.62796 .06463 L
.63035 .06607 L
.63298 .06811 L
.63577 .07081 L
.64078 .07699 L
.65078 .09418 L
.66184 .12011 L
.68116 .18051 L
.70201 .26313 L
.74311 .46596 L
.78271 .7017 L
.82078 .96258 L
.86131 1.27651 L
.90032 1.61395 L
.94179 2.0105 L
.97619 2.36913 L
.97619 .20601 L
.93871 .20601 L
.89771 .20601 L
.85916 .20601 L
.81909 .20601 L
.77751 .20601 L
.73838 .20601 L
.69774 .20601 L
.65954 .20601 L
.61984 .20601 L
.57861 .20601 L
.53984 .20601 L
.49955 .20601 L
.46172 .20601 L
.42237 .20601 L
.3815 .20601 L
.34309 .20601 L
.30316 .20601 L
.26171 .20601 L
.22272 .20601 L
.18221 .20601 L
.14415 .20601 L
.10458 .20601 L
.06244 .20601 L
.02381 .20601 L
F
0 g
.23115 .61803 m
.26171 .43981 L
.28158 .33843 L
.30316 .24121 L
.32216 .16893 L
.33296 .13461 L
.34309 .10772 L
.35264 .08759 L
.35756 .0794 L
.36292 .07226 L
.36584 .06919 L
.36855 .06686 L
.37114 .06512 L
.37247 .06441 L
.3739 .06379 L
.37532 .06332 L
.37662 .06301 L
.37786 .06284 L
.37917 .06278 L
.38035 .06283 L
.38145 .06296 L
.38266 .06322 L
.38395 .06361 L
.38517 .06409 L
.38651 .06475 L
.38924 .06648 L
.39176 .06857 L
.39413 .07095 L
.39945 .07775 L
.40519 .08725 L
.41569 .10993 L
.42531 .13578 L
.44584 .20002 L
.45585 .23154 L
.46494 .25769 L
.47378 .27923 L
.47843 .28855 L
.48336 .29668 L
.48587 .30007 L
.48863 .3032 L
.49117 .30549 L
.49349 .3071 L
.49471 .30775 L
.49587 .30824 L
.49709 .30863 L
.49778 .30879 L
.49843 .3089 L
.4996 .30901 L
.50066 .309 L
.50189 .30885 L
Mistroke
.50305 .3086 L
.50421 .30821 L
.50546 .30766 L
.50772 .30632 L
.51039 .30415 L
.51283 .30162 L
.51814 .2944 L
.52316 .28553 L
.54165 .23907 L
.5599 .18148 L
.58003 .12092 L
.58957 .09776 L
.59984 .0788 L
.60551 .07136 L
.60798 .06881 L
.61069 .06654 L
.61326 .06487 L
.61565 .06376 L
.61697 .06332 L
.61817 .06303 L
.61884 .06292 L
.61956 .06283 L
.62086 .06278 L
.6221 .06284 L
.62323 .06299 L
.62453 .06328 L
.62577 .06367 L
.62796 .06463 L
.63035 .06607 L
.63298 .06811 L
.63577 .07081 L
.64078 .07699 L
.65078 .09418 L
.66184 .12011 L
.68116 .18051 L
.70201 .26313 L
.74311 .46596 L
Mfstroke
.74311 .46596 m
.76865 .61803 L
s
.02381 .20601 m
.06244 .20601 L
.10458 .20601 L
.14415 .20601 L
.18221 .20601 L
.22272 .20601 L
.26171 .20601 L
.30316 .20601 L
.34309 .20601 L
.3815 .20601 L
.42237 .20601 L
.46172 .20601 L
.49955 .20601 L
.53984 .20601 L
.57861 .20601 L
.61984 .20601 L
.65954 .20601 L
.69774 .20601 L
.73838 .20601 L
.77751 .20601 L
.81909 .20601 L
.85916 .20601 L
.89771 .20601 L
.93871 .20601 L
.97619 .20601 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0 0 1 0.618034 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0952381 -0.206011 0.103006 [
[.11905 -0.0125 -9.15625 -15.6875 ]
[.11905 -0.0125 9.15625 0 ]
[.30952 -0.0125 -9.15625 -15.6875 ]
[.30952 -0.0125 9.15625 0 ]
[.5 -0.0125 -5 -15.6875 ]
[.5 -0.0125 5 0 ]
[.69048 -0.0125 -5 -15.6875 ]
[.69048 -0.0125 5 0 ]
[.88095 -0.0125 -5 -15.6875 ]
[.88095 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -5.0625 -31.375 ]
[.5 -0.0125 5.0625 -15.6875 ]
[-0.0125 .10301 -10 -7.84375 ]
[-0.0125 .10301 0 7.84375 ]
[-0.0125 .20601 -10 -7.84375 ]
[-0.0125 .20601 0 7.84375 ]
[-0.0125 .30902 -10 -7.84375 ]
[-0.0125 .30902 0 7.84375 ]
[-0.0125 .41202 -10 -7.84375 ]
[-0.0125 .41202 0 7.84375 ]
[-0.0125 .51503 -10 -7.84375 ]
[-0.0125 .51503 0 7.84375 ]
[-0.0125 .61803 -10 -7.84375 ]
[-0.0125 .61803 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -25.6875 -13.3125 ]
[-0.01875 .30902 -10 13.3125 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -58.9375 0 ]
[.5 .63053 58.9375 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.11905 0 m
.11905 .00625 L
s
gsave
.11905 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.30952 0 m
.30952 .00625 L
s
gsave
.30952 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.5 0 m
.5 .00625 L
s
gsave
.5 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.69048 0 m
.69048 .00625 L
s
gsave
.69048 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.88095 0 m
.88095 .00625 L
s
gsave
.88095 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.125 Mabswid
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.07143 0 m
.07143 .00375 L
s
.02381 0 m
.02381 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -66.0625 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
69.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .10301 m
.00625 .10301 L
s
gsave
-0.0125 .10301 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
0 .20601 m
.00625 .20601 L
s
gsave
-0.0125 .20601 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
0 .30902 m
.00625 .30902 L
s
gsave
-0.0125 .30902 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
0 .41202 m
.00625 .41202 L
s
gsave
-0.0125 .41202 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
0 .51503 m
.00625 .51503 L
s
gsave
-0.0125 .51503 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7) show
1.000 setlinewidth
grestore
0 .61803 m
.00625 .61803 L
s
gsave
-0.0125 .61803 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .0824 m
.00375 .0824 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .16481 m
.00375 .16481 L
s
0 .18541 m
.00375 .18541 L
s
0 .22661 m
.00375 .22661 L
s
0 .24721 m
.00375 .24721 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .32962 m
.00375 .32962 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .49443 m
.00375 .49443 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .57683 m
.00375 .57683 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -92.1563 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
74.3125 11.8438 translate
90 rotate
-74.3125 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(V) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
71.625 15.813 moveto
(@) show
75.563 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
%%IncludeResource: font Mathematica2
%%IncludeFont: Mathematica2
/Mathematica2 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
81.688 15.813 moveto
(D) show
85.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.11905 .61178 m
.11905 .61803 L
s
.30952 .61178 m
.30952 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.69048 .61178 m
.69048 .61803 L
s
.88095 .61178 m
.88095 .61803 L
s
.125 Mabswid
.16667 .61428 m
.16667 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.2619 .61428 m
.2619 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.7381 .61428 m
.7381 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.07143 .61428 m
.07143 .61803 L
s
.02381 .61428 m
.02381 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.97619 .61428 m
.97619 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 0 m
1 0 L
s
.99375 .10301 m
1 .10301 L
s
.99375 .20601 m
1 .20601 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .41202 m
1 .41202 L
s
.99375 .51503 m
1 .51503 L
s
.125 Mabswid
.99625 .0206 m
1 .0206 L
s
.99625 .0412 m
1 .0412 L
s
.99625 .0618 m
1 .0618 L
s
.99625 .0824 m
1 .0824 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .14421 m
1 .14421 L
s
.99625 .16481 m
1 .16481 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .22661 m
1 .22661 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26781 m
1 .26781 L
s
.99625 .28842 m
1 .28842 L
s
.99625 .32962 m
1 .32962 L
s
.99625 .35022 m
1 .35022 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .39142 m
1 .39142 L
s
.99625 .43262 m
1 .43262 L
s
.99625 .45322 m
1 .45322 L
s
.99625 .47383 m
1 .47383 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .53563 m
1 .53563 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57683 m
1 .57683 L
s
.99625 .59743 m
1 .59743 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
.5 0 m
.5 .61803 L
s
gsave
.5 .63053 -119.938 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(a) show
71.375 15.813 moveto
(double) show
103.938 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
111.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(well) show
135.125 15.813 moveto
(potential) show
176.875 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00j03oool01@2?=@000000000eS`3oool0a5lD0080000000<001AO0<Coo`3oool0_P3oool0
0>P0oooo00P0ooo405lD=@2?ool0oooo0?ooa01O500001AO0<Cool00oooo003Y0?ooo`060?ooS`0e
55l0a?oo08le000003D0Sooo`@3oool00>X0oooo00@0olAO03DD50000000=Hoo`P3oool00>X0oooo
00<0oon?03D0000DGl@0``3oool00>T0oooo00D0ooo405lD000055l0GcDD05o4o`320?ooo`00j@3o
ool01P2?=@00000D05o4o`3ool@0Ga@e08oool40oooo003W0?ooo`090?ooa01O5000000000000000
=Hl0ohle0000000003D0Sooo0<00oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo001o0?ooo`040?ooa01O500001AO0<Co
oc80oooo00<0ScD0000000000000100000000`0DGl@0oooo0?ooo`0[0?ooo`050?o4G`0D00000000
0000=@2?ool0<`3oool00`2?=@000000000000040000000301AOa03oool0oooo03@0oooo00@0ooo4
05lD000055l0a?ooC03oool007l0oooo00@0ooo405lD000055l0a?oo<`3oool00`34Ga@000000000
000300000003000DG`34ool0oooo02X0oooo00L0oon?03D0000eSol0oooo0?n?=@0003D0Sooo03<0
oooo00<0a5lD0000000000000`0000000`0055l0a?oo0?ooo`0d0?ooo`040?ooa01O500001AO0<Co
od`0oooo001o0?ooo`040?ooa01O500001AO0<Cooc@0oooo00<0a5lD01AOa03oool00P3oool00`34
GcD0GlCo0?ooo`0Z0?ooo`0308le000055l0a?oo0080oooo00<0ScD0000eS`3oool0<`3oool00`34
Ga@055o40?ooo`020?ooo`030<AO=@1Oa?l0oooo03@0oooo00@0ooo405lD000055l0a?ooC03oool0
07/0oooo00<0ScD00000000000001@0000000`0eSol0oooo0?ooo`0b0?ooo`030<AO500eSol0oooo
02d0oooo00<0oon?03D0000DGl@00`3oool00`34Ga@0000e08ooo`0d0?ooo`030<AO500eSol0oooo
03@0oooo00<0ScD00000000000001@0000000`0eSol0oooo0?ooo`190?ooo`00K`3oool00`3ool@0
Ga@0000000070000000401AOa03oool0olAO03EOa080oooo00@0ooo405lD000055l0a?oo9@3oool0
0`3ool@0Ga@0000000070000000301AOa03oool0oooo00@0oooo00<0ScDD05o4o`3oool0;03oool0
0`3oa5l0500003F?o`030?ooo`030?n?=@0001@0GlCo03D0oooo00<0ScDD05o4o`3oool0<`3oool0
203oa5l0=Eo40?ooo`3oool0ooo405lD000055l0a?ooC03oool007`0oooo00L0a5lD03F?o`3oool0
ooo405lD000055l0a?oo03H0oooo00<0ooo405lD=@2?ool0;03oool00`3oa5l0500003F?o`030?oo
o`030?n?=@0001@0GlCo03D0oooo00<0ooo405lD=@2?ool0=03oool01`34Ga@0=Hoo0?ooo`3ool@0
Ga@0000DG`34ool0C03oool007`0oooo00L0ooo405lD=@2?ool0ooo405lD000055l0a?oo03L0oooo
00<0ohle000eS`3oool0:`3oool00`3oa5l0500003F?o`030?ooo`030?n?=@0001@0GlCo03H0oooo
00<0ohle000eS`3oool0<`3oool01`3ool@0Ga@e08ooo`3ool@0Ga@0000DG`34ool0C03oool007d0
oooo00H0olAO01@eS`3ool@0Ga@0000DG`34oolg0?ooo`030?ooS`0e01@0GlCo02/0oooo00<0olAO
01@0000eSol00`3oool00`3oScD0000D05o4o`0f0?ooo`030?ooS`0e01@0GlCo03@0oooo00H0olAO
01@eS`3ool@0Ga@0000DG`34oom<0?ooo`00OP3oool01@34Ga@0=Ho405lD000055l0a?oo03L0oooo
00<0oon?03D0000DGl@0:`3oool00`3oohl0=@0001AOa0030?ooo`030<AO500003D0Sooo03H0oooo
00<0oon?03D0000DGl@0=@3oool01@34Ga@0=Ho405lD000055l0a?oo04`0oooo001n0?ooo`050?oo
a01O51@051@0000DG`34ool0<P3oool00`3oSel0GlCo0?ooo`020?ooo`030?o4G`0D000055o402`0
oooo00<0a5lD000DG`34ool00P3oool00`2?=@0003F?0?ooo`0a0?ooo`030?n?G`1Oa?l0oooo0080
oooo00<0olAO01@0000DGl@0=@3oool01@3ool@0Ga@D01@D000055l0a?oo04`0oooo001o0?ooo`04
0?o4G`0D000001AO0<Cooc80oooo00P0ooo405lD501Oa?l0oooo0?ooS`0e0000000e08ooob`0oooo
00L0oon?03D0000eSol0oooo0?n?=@0003D0Sooo0380oooo00P0ooo405lD501Oa?l0oooo0?ooS`0e
0000000e08ooocH0oooo00@0olAO01@0000055l0a?ooC03oool00800oooo00<0a5lD000DG`34ool0
<`3oool01P3ool@0Ga@0000000000000000e08ooobh0oooo00D0oon?03D000000000000e08ooo`0d
0?ooo`060?ooa01O500000000000000003D0Sooo>03oool00`34Ga@001AO0<Coo`1<0?ooo`00o`3o
oona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3o
oona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00E@3ooooo000003800000:P3oool005D0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo09D0oooo00<00000
0?ooo`3oool0U@3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`2E0?oo
o`030000003oool0oooo09D0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3o
ool0U@3oool00`000000oooo0?ooo`2E0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000
003oool0oooo09D0oooo00<000000?ooo`3oool0U@3oool00`000000oooo0?ooo`0X0?ooo`00E@3o
ool2000009H0oooo00<000000?ooo`3oool0U03oool2000002X0oooo001E0?ooo`030000003oool0
oooo09D0oooo00<000000?ooo`3oool0U@3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`00
0000oooo0?ooo`2E0?ooo`030000003oool0oooo09D0oooo00<000000?ooo`3oool0:03oool005D0
oooo00<000000?ooo`3oool0U@3oool00`000000oooo0?ooo`2E0?ooo`030000003oool0oooo02P0
oooo001E0?ooo`030000003oool0oooo09D0oooo00<000000?ooo`3oool0U@3oool00`000000oooo
0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`2E0?ooo`030000003oool0oooo09D0oooo00<0
00000?ooo`3oool0:03oool005D0oooo0P00002F0?ooo`030000003oool0oooo09@0oooo0P00000Z
0?ooo`00E@3oool00`000000oooo0?ooo`2E0?ooo`030000003oool0oooo09D0oooo00<000000?oo
o`3oool0:03oool005D0oooo00<000000?ooo`3oool0U@3oool00`000000oooo0?ooo`2E0?ooo`03
0000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo09D0oooo00<000000?ooo`3oool0
U@3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`2E0?ooo`030000003o
ool0oooo09D0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0U@3oool0
0`000000oooo0?ooo`2E0?ooo`030000003oool0oooo02P0oooo001E0?ooo`800000UP3oool00`00
0000oooo0?ooo`2D0?ooo`800000:P3oool005D0oooo00<000000?ooo`3oool0KP3oool600000240
oooo00<000000?ooo`3oool07`3oool600000700oooo00<000000?ooo`3oool0:03oool005D0oooo
00<000000?ooo`3oool0K03oool2000000D00?oo0P00000P0?ooo`030000003oool0oooo01h0oooo
0P000005003oo`030000003oool0oooo06d0oooo00<000000?ooo`3oool0:03oool005D0oooo00<0
00000?ooo`3oool0J`3oool00`0000000?oo003oo`07003oo`030000003oool0oooo01d0oooo00<0
00000?ooo`3oool07@3oool00`0000000?oo003oo`06003oo`030000003oool0oooo06`0oooo00<0
00000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0J`3oool00`0000000?oo003oo`07
003oo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0703oool00`0000000?oo003oo`08
003oo`030000003oool0oooo06/0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?oo
o`3oool0JP3oool00`0000000?oo003oo`09003oo`030000003oool0oooo01`0oooo00<000000?oo
o`3oool06`3oool00`0000000?oo003oo`0:003oo`030000003oool0oooo06X0oooo00<000000?oo
o`3oool0:03oool005D0oooo00<000000?ooo`3oool0JP3oool00`0000000?oo003oo`0:003oo`03
0000003oool0oooo01/0oooo00<000000?ooo`3oool06P3oool00`0000000?oo003oo`0<003oo`03
0000003oool0oooo06T0oooo00<000000?ooo`3oool0:03oool005D0oooo0P00001Z0?ooo`030000
0000ool00?oo00/00?oo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0J0?ooo`030000
0000ool00?oo00`00?oo00<000000?ooo`3oool0J03oool2000002X0oooo00150?ooo`030?ooS`0e
000000000080000000<0=Hoo0?ooo`3oool0203oool00`000000oooo0?ooo`1X0?ooo`0300000000
ool00?oo00d00?oo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`0300000000
ool00?oo00h00?oo00<000000?ooo`3oool0J03oool00`000000oooo0?ooo`0X0?ooo`00A@3oool0
0`3oa5l0500001AOa0020?ooo`0308le000DGl@0oooo00P0oooo00<000000?ooo`3oool0I`3oool0
0`0000000?oo003oo`0>003oo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06@3oool0
0`0000000?oo003oo`0>003oo`030000003oool0oooo06P0oooo00<000000?ooo`3oool0:03oool0
04X0oooo00<0oon?03D0501Oa?l0203oool00`000000oooo0?ooo`1W0?ooo`0300000000ool00?oo
00l00?oo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`0300000000ool00?oo
01000?oo00<000000?ooo`3oool0I`3oool00`000000oooo0?ooo`0X0?ooo`00BP3oool00`3oohl0
=@0003F?o`080?ooo`030000003oool0oooo06H0oooo00<00000003oo`00ool04000ool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<00000003oo`00ool04000ool00`000000
oooo0?ooo`1W0?ooo`030000003oool0oooo02P0oooo001:0?ooo`030?n?=@000000=Hoo00P0oooo
00<000000?ooo`3oool0IP3oool00`0000000?oo003oo`0A003oo`030000003oool0oooo01P0oooo
00<000000?ooo`3oool0603oool00`0000000?oo003oo`0A003oo`030000003oool0oooo06H0oooo
00<000000?ooo`3oool0:03oool004T0oooo00@0olAO01@0000001@0GlCo203oool3000006D0oooo
00<00000003oo`00ool04P00ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01L0oooo
00<00000003oo`00ool04P00ool00`000000oooo0?ooo`1T0?ooo`<00000:P3oool004L0oooo00D0
ohle000000000000000D05o4o`090?ooo`030000003oool0oooo06D0oooo00<00000003oo`00ool0
4`00ool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01L0oooo00<00000003oo`00ool0
4P00ool00`000000oooo0?ooo`1V0?ooo`030000003oool0oooo02P0oooo00190?ooo`0308le0000
=Hl0oooo00T0oooo00<000000?ooo`3oool0I@3oool00`0000000?oo003oo`0C003oo`030000003o
ool0oooo01L0oooo00<000000?ooo`3oool05P3oool00`0000000?oo003oo`0D003oo`030000003o
ool0oooo06D0oooo00<000000?ooo`3oool0:03oool004X0oooo00<0ScD001AOa03oool0203oool0
0`000000oooo0?ooo`1T0?ooo`0300000000ool00?oo01@00?oo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`0F0?ooo`0300000000ool00?oo01@00?oo00<000000?ooo`3oool0I@3oool0
0`000000oooo0?ooo`0X0?ooo`00A@3oool00`3ool@0Gen?0?ooo`020?ooo`030<AO500003D0Sooo
00P0oooo00<000000?ooo`3oool0I03oool00`0000000?oo003oo`0E003oo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool05@3oool00`0000000?oo003oo`0F003oo`030000003oool0oooo
06@0oooo00<000000?ooo`3oool0:03oool004H0oooo00H0ohle01AOa03oool0oon?03D00000=Hl9
0?ooo`030000003oool0oooo06<0oooo00<00000003oo`00ool05P00ool00`000000oooo0?ooo`0F
0?ooo`030000003oool0oooo01D0oooo00<00000003oo`00ool05P00ool00`000000oooo0?ooo`1T
0?ooo`030000003oool0oooo02P0oooo00170?ooo`050<AO5000000000000000=@2?ool02@3oool2
000006@0oooo00<00000003oo`00ool05P00ool00`000000oooo0?ooo`0F0?ooo`030000003oool0
oooo01D0oooo00<00000003oo`00ool05P00ool00`000000oooo0?ooo`1S0?ooo`800000:P3oool0
05D0oooo00<000000?ooo`3oool0H`3oool00`0000000?oo003oo`0G003oo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool0503oool00`0000000?oo003oo`0H003oo`030000003oool0oooo
06<0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0HP3oool00`000000
0?oo003oo`0H003oo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0503oool00`000000
0?oo003oo`0H003oo`030000003oool0oooo06<0oooo00<000000?ooo`3oool0:03oool005D0oooo
00<000000?ooo`3oool0HP3oool00`0000000?oo003oo`0I003oo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool0503oool00`0000000?oo003oo`0H003oo`030000003oool0oooo06<0oooo
00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0HP3oool00`0000000?oo003o
o`0I003oo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04`3oool00`0000000?oo003o
o`0J003oo`030000003oool0oooo0680oooo00<000000?ooo`3oool0:03oool005D0oooo00<00000
0?ooo`3oool0H@3oool00`0000000?oo003oo`0J003oo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool04`3oool00`0000000?oo003oo`0J003oo`030000003oool0oooo0680oooo00<00000
0?ooo`3oool0:03oool005D0oooo0P00001R0?ooo`0300000000ool00?oo01/00?oo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0C0?ooo`0300000000ool00?oo01X00?oo00<000000?oo
o`3oool0H@3oool2000002X0oooo001E0?ooo`030000003oool0oooo0640oooo00<00000003oo`00
ool06`00ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0180oooo00<00000003oo`00
ool07000ool00`000000oooo0?ooo`1Q0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000
003oool0oooo0640oooo00<00000003oo`00ool06`00ool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo0180oooo00<00000003oo`00ool07000ool00`000000oooo0?ooo`1Q0?ooo`030000
003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo0600oooo00<00000003oo`00ool07@00
ool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0180oooo00<00000003oo`00ool07000
ool00`000000oooo0?ooo`1Q0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0
oooo0600oooo00<00000003oo`00ool07@00ool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo0180oooo00<00000003oo`00ool07@00ool00`000000oooo0?ooo`1P0?ooo`030000003oool0
oooo02P0oooo001E0?ooo`030000003oool0oooo0600oooo00<00000003oo`00ool07P00ool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<00000003oo`00ool07P00ool00`00
0000oooo0?ooo`1P0?ooo`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo0600
oooo00<00000003oo`00ool07P00ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140
oooo00<00000003oo`00ool07P00ool00`000000oooo0?ooo`1P0?ooo`030000003oool0oooo02P0
oooo001E0?ooo`800000H03oool00`0000000?oo003oo`0O003oo`030000003oool0oooo0140oooo
00<000000?ooo`3oool04@3oool00`0000000?oo003oo`0O003oo`030000003oool0oooo05h0oooo
0P00000Z0?ooo`00E@3oool00`000000oooo0?ooo`1O0?ooo`0300000000ool00?oo02000?oo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`0300000000ool00?oo02000?oo00<0
00000?ooo`3oool0G`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`1O
0?ooo`0300000000ool00?oo02000?oo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@
0?ooo`0300000000ool00?oo02000?oo00<000000?ooo`3oool0G`3oool00`000000oooo0?ooo`0X
0?ooo`00E@3oool00`000000oooo0?ooo`1N0?ooo`0300000000ool00?oo02400?oo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`0@0?ooo`0300000000ool00?oo02400?oo00<000000?oo
o`3oool0GP3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`1N0?ooo`03
00000000ool00?oo02800?oo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0?0?ooo`03
00000000ool00?oo02800?oo00<000000?ooo`3oool0GP3oool00`000000oooo0?ooo`0X0?ooo`00
E@3oool00`000000oooo0?ooo`1N0?ooo`0300000000ool00?oo02800?oo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`0?0?ooo`0300000000ool00?oo02800?oo00<000000?ooo`3oool0
GP3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool2000005l0oooo00<00000003oo`00ool08`00
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00l0oooo00<00000003oo`00ool08P00
ool00`000000oooo0?ooo`1M0?ooo`800000:P3oool004T0oooo00@0ooo405lD000055l0a?oo203o
ool00`000000oooo0?ooo`1M0?ooo`0300000000ool00?oo02@00?oo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`0>0?ooo`0300000000ool00?oo02@00?oo00<000000?ooo`3oool0G@3o
ool00`000000oooo0?ooo`0X0?ooo`00B@3oool0103ool@0Ga@0000DG`34ool80?ooo`030000003o
ool0oooo05d0oooo00<00000003oo`00ool09000ool00`000000oooo0?ooo`0>0?ooo`030000003o
ool0oooo00h0oooo00<00000003oo`00ool09000ool00`000000oooo0?ooo`1M0?ooo`030000003o
ool0oooo02P0oooo00190?ooo`040?ooa01O500001AO0<Coo`P0oooo00<000000?ooo`3oool0G@3o
ool00`0000000?oo003oo`0U003oo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03P3o
ool00`0000000?oo003oo`0T003oo`030000003oool0oooo05d0oooo00<000000?ooo`3oool0:03o
ool004D0oooo00<0ScD00000000000001@0000000`0eSol0oooo0?ooo`050?ooo`030000003oool0
oooo05d0oooo00<00000003oo`00ool09@00ool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo00h0oooo00<00000003oo`00ool09000ool00`000000oooo0?ooo`1M0?ooo`030000003oool0
oooo02P0oooo00150?ooo`080?o4G`0eGl@0oooo0?ooo`3ool@0Ga@0000DG`34ool80?ooo`030000
003oool0oooo05`0oooo00<00000003oo`00ool09P00ool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo00d0oooo00<00000003oo`00ool09P00ool00`000000oooo0?ooo`1L0?ooo`030000
003oool0oooo02P0oooo00160?ooo`070<AO500eSol0oooo0?ooa01O500001AO0<Coo`080?ooo`<0
0000103ooooo000002@00000103oool3000002X0oooo00160?ooo`070?ooa01O53D0Sooo0?ooa01O
500001AO0<Coo`080?ooo`030000003oool0oooo00@0ooooF000ool00`000000oooo0?ooo`0W0?oo
o`0300000000ool00?oo00`00?oo00<00000003oo`00ool03@00ool00`000000oooo0?ooo`0V0?oo
o`0300000000ool00?oo05D00?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00A`3oool01P3oa5l0
53F?0?ooa01O500001AO0<Coo`P0oooo00<000000?ooo`3oool0103ooomG003oo`030000003oool0
oooo02P0oooo00<00000003oo`00ool03000ool00`0000000?oo003oo`0<003oo`030000003oool0
oooo02L0oooo00<00000003oo`00ool0E@00ool70?ooo`030000003oool0oooo02P0oooo00180?oo
o`050<AO500eSl@0Ga@0000DG`34ool0203oool00`000000oooo0?ooo`040?oooeL00?oo00<00000
0?ooo`3oool0:03oool00`0000000?oo003oo`0<003oo`0300000000ool00?oo00`00?oo00<00000
0?ooo`3oool0:03oool00`0000000?oo003oo`1D003oo`L0oooo00<000000?ooo`3oool0:03oool0
04P0oooo00D0ooo405lD500D500001AO0<Coo`080?ooo`030000003oool0oooo00@0ooooE`00ool0
0`000000oooo0?ooo`0Y0?ooo`0300000000ool00?oo00/00?oo00<00000003oo`00ool03000ool0
0`000000oooo0?ooo`0X0?ooo`0300000000ool00?oo05@00?oo1`3oool00`000000oooo0?ooo`0X
0?ooo`00B@3oool0103oa5l05000000DG`34ool80?ooo`030000003oool0oooo00@0ooooE`00ool0
0`000000oooo0?ooo`0Y0?ooo`0300000000ool00?oo00/00?oo00<00000003oo`00ool02`00ool0
0`000000oooo0?ooo`0Y0?ooo`0300000000ool00?oo05@00?oo1`3oool00`000000oooo0?ooo`0X
0?ooo`00BP3oool00`34Ga@001AO0<Coo`080?ooo`8000001@3ooomF003oo`030000003oool0oooo
02X0oooo00<00000003oo`00ool02`00ool00`0000000?oo003oo`0;003oo`030000003oool0oooo
02T0oooo00<00000003oo`00ool0E000ool60?ooo`800000:P3oool005D0oooo00<000000?ooo`3o
ool0103ooomF003oo`030000003oool0oooo02/0oooo00<00000003oo`00ool02P00ool00`000000
0?oo003oo`0;003oo`030000003oool0oooo02X0oooo00<00000003oo`00ool0D`00ool70?ooo`03
0000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo00@0ooooEP00ool00`000000oooo
0?ooo`0[0?ooo`0300000000ool00?oo00X00?oo00<00000003oo`00ool02`00ool00`000000oooo
0?ooo`0Z0?ooo`0300000000ool00?oo05<00?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00E@3o
ool00`000000oooo0?ooo`040?oooeH00?oo00<000000?ooo`3oool0:`3oool00`0000000?oo003o
o`0:003oo`0300000000ool00?oo00X00?oo00<000000?ooo`3oool0:`3oool00`0000000?oo003o
o`1C003oo`L0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0103ooomE
003oo`030000003oool0oooo02`0oooo00<00000003oo`00ool02P00ool00`0000000?oo003oo`0:
003oo`030000003oool0oooo02/0oooo00<00000003oo`00ool0D`00ool70?ooo`030000003oool0
oooo02P0oooo001E0?ooo`030000003oool0oooo00@0ooooE@00ool00`000000oooo0?ooo`0]0?oo
o`0300000000ool00?oo00T00?oo00<00000003oo`00ool02P00ool00`000000oooo0?ooo`0/0?oo
o`0300000000ool00?oo05800?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000
oooo0?ooo`040?oooeD00?oo00<000000?ooo`3oool0;@3oool00`0000000?oo003oo`09003oo`03
00000000ool00?oo00T00?oo00<000000?ooo`3oool0;@3oool00`0000000?oo003oo`1B003oo`L0
oooo00<000000?ooo`3oool0:03oool005D0oooo0P0000050?oooeD00?oo00<000000?ooo`3oool0
;@3oool00`0000000?oo003oo`09003oo`0300000000ool00?oo00T00?oo00<000000?ooo`3oool0
;@3oool00`0000000?oo003oo`1B003oo`H0oooo0P00000Z0?ooo`00E@3oool00`000000oooo0?oo
o`040?oooe@00?oo00<000000?ooo`3oool0;P3oool00`0000000?oo003oo`09003oo`0300000000
ool00?oo00P00?oo00<000000?ooo`3oool0;P3oool00`0000000?oo003oo`1B003oo`L0oooo00<0
00000?ooo`3oool0:03oool002h0oooo00<0a5le05o4o`3oool0903oool00`000000oooo0?ooo`04
0?oooe@00?oo00<000000?ooo`3oool0;`3oool00`0000000?oo003oo`08003oo`0300000000ool0
0?oo00P00?oo00<000000?ooo`3oool0;`3oool00`0000000?oo003oo`1A003oo`L0oooo00<00000
0?ooo`3oool0:03oool002h0oooo00<0a5lD000eS`3oool0903oool00`000000oooo0?ooo`040?oo
oe@00?oo00<000000?ooo`3oool0;`3oool00`0000000?oo003oo`08003oo`0300000000ool00?oo
00L00?oo00<000000?ooo`3oool0<03oool00`0000000?oo003oo`1A003oo`L0oooo00<000000?oo
o`3oool0:03oool002h0oooo00D0a5lD000000000000000003F?o`0R0?ooo`030000003oool0oooo
00@0ooooE000ool00`000000oooo0?ooo`0_0?ooo`0300000000ool00?oo00P00?oo00<00000003o
o`00ool01`00ool00`000000oooo0?ooo`0`0?ooo`0300000000ool00?oo05400?oo1`3oool00`00
0000oooo0?ooo`0X0?ooo`00;P3oool00`34Ga@0000000000003000000030000=@2?ool0oooo01h0
oooo00<000000?ooo`3oool0103ooomD003oo`030000003oool0oooo0300oooo00<00000003oo`00
ool01`00ool00`0000000?oo003oo`07003oo`030000003oool0oooo0300oooo00<00000003oo`00
ool0D@00ool70?ooo`030000003oool0oooo02P0oooo000^0?ooo`060<AO=@1Oa?l0oooo0?ooo`3o
a5l050000`0000000`0055l0a?oo0?ooo`0K0?ooo`8000001@3ooomC003oo`030000003oool0oooo
0340oooo00<00000003oo`00ool01`00ool00`0000000?oo003oo`06003oo`030000003oool0oooo
0340oooo00<00000003oo`00ool0D@00ool60?ooo`800000:P3oool003D0oooo00<0ohle00000000
00000P0000000`00=Hl0oooo0?ooo`0H0?ooo`030000003oool0oooo00@0ooooD`00ool00`000000
oooo0?ooo`0b0?ooo`0300000000ool00?oo00H00?oo00<00000003oo`00ool01P00ool00`000000
oooo0?ooo`0b0?ooo`0300000000ool00?oo05000?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00
>03oool0103oScD000000000=@2?oolI0?ooo`030000003oool0oooo00@0ooooD`00ool00`000000
oooo0?ooo`0b0?ooo`0300000000ool00?oo00H00?oo00<00000003oo`00ool01@00ool00`000000
oooo0?ooo`0c0?ooo`0300000000ool00?oo05000?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00
=@3oool0103oScD0000000000000=HlL0?ooo`030000003oool0oooo00@0ooooD`00ool00`000000
oooo0?ooo`0c0?ooo`0300000000ool00?oo00D00?oo00<00000003oo`00ool01@00ool00`000000
oooo0?ooo`0c0?ooo`0300000000ool00?oo05000?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00
;P3oool02@34GcD0GlCo0?ooo`3oool0olAO01@00000000001AO0<Coo`0N0?ooo`030000003oool0
oooo00@0ooooD`00ool00`000000oooo0?ooo`0c0?ooo`0300000000ool00?oo00D00?oo00<00000
003oo`00ool01000ool00`000000oooo0?ooo`0d0?ooo`0300000000ool00?oo05000?oo1`3oool0
0`000000oooo0?ooo`0X0?ooo`00;P3oool01P34Ga@00000000000000000000e08ooob40oooo00<0
00000?ooo`3oool0103ooomB003oo`030000003oool0oooo03D0oooo00<00000003oo`00ool01000
ool00`0000000?oo003oo`04003oo`030000003oool0oooo03@0oooo00<00000003oo`00ool0D000
ool70?ooo`030000003oool0oooo02P0oooo000^0?ooo`030<AO50000000=Hoo02@0oooo0P000005
0?oooe800?oo00<000000?ooo`3oool0=@3oool00`0000000?oo003oo`04003oo`0300000000ool0
0?oo00<00?oo00<000000?ooo`3oool0=P3oool00`0000000?oo003oo`1?003oo`H0oooo0P00000Z
0?ooo`00;P3oool00`34GcD0GlCo0?ooo`0D0?ooo`030?ooa01O500000000080000000<0=Hoo0?oo
o`3oool0203oool00`000000oooo0?ooo`040?oooe800?oo00<000000?ooo`3oool0=P3oool00`00
00000?oo003oo`03003oo`0300000000ool00?oo00<00?oo00<000000?ooo`3oool0=P3oool00`00
00000?oo003oo`1?003oo`L0oooo00<000000?ooo`3oool0:03oool004D0oooo00<0ooo405lD=@2?
ool00P3oool00`2?=@00=Hoo0?ooo`080?ooo`030000003oool0oooo00@0ooooDP00ool00`000000
oooo0?ooo`0f0?ooo`0300000000ool00?oo00<00?oo00<00000003oo`00ool00P00ool00`000000
oooo0?ooo`0g0?ooo`0300000000ool00?oo04l00?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00
BP3oool00`3ool@0Ga@e08ooo`080?ooo`030000003oool0oooo00@0ooooD@00ool00`000000oooo
0?ooo`0h0?ooo`0300000000ool00?oo00800?oo00D00000003oo`00ool00?oo0000000j0?ooo`03
00000000ool00?oo04l00?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00;@3oool00`3oa5l05000
0000000>000000030000=@2?ool0oooo00X0oooo00<0ScD003F?o`3oool01`3oool00`000000oooo
0?ooo`040?oooe400?oo00<000000?ooo`3oool0>@3oool01@0000000?oo003oo`00ool0000000<0
0?oo00<000000?ooo`3oool0>@3oool00`0000000?oo003oo`1>003oo`L0oooo00<000000?ooo`3o
ool0:03oool002d0oooo00<0olAO03EOa03oool03P3oool00`2?=CD0Sooo0?ooo`090?ooo`030?oo
a01O500055o400P0oooo00<000000?ooo`3oool0103ooomA003oo`030000003oool0oooo03X0oooo
00@00000003oo`00ool000000P00ool00`000000oooo0?ooo`0j0?ooo`0300000000ool00?oo04h0
0?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00;@3oool00`3oa5l0Ghoo0?ooo`0>0?ooo`030<AO
G`2?ool0oooo00T0oooo00<0a5lD0000000eSol0203oool3000000@0ooooD@00ool00`000000oooo
0?ooo`0k0?ooo`D00000?@3oool00`0000000?oo003oo`1>003oo`D0oooo0`00000Z0?ooo`00;@3o
ool00`3oa5l0Ghoo0?ooo`0>0?ooo`030<AOG`2?ool0oooo00D0oooo00L0a<C40?ooo`3oohl0=@00
0000000003D0Sooo00P0oooo00<000000?ooo`3oool0103ooomA003oo`030000003oool0oooo03d0
oooo00<000000?ooo`3oool0?@3oool00`0000000?oo003oo`1>003oo`L0oooo00<000000?ooo`3o
ool0:03oool004H0oooo00H0a5lD00000000000000000000501Oa?l90?ooo`030000003oool0oooo
00@0ooooD000ool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo03d0oooo00<00000003o
o`00ool0CP00ool70?ooo`030000003oool0oooo02P0oooo000c0?ooo`0308le=@2?ool0oooo00@0
oooo00<0ScDe08ooo`3oool02@3oool0103oohl0=@000000000DGl@;0?ooo`030000003oool0oooo
00@0ooooD000ool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo03h0oooo00<00000003o
o`00ool0C@00ool70?ooo`030000003oool0oooo02P0oooo000c0?ooo`0308le000DGl@0oooo00<0
oooo00<0a5lD0000=@2?ool02`3oool00`2?=@00=Hoo0?ooo`0;0?ooo`030000003oool0oooo00@0
ooooD000ool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo03h0oooo00<00000003oo`00
ool0C@00ool70?ooo`030000003oool0oooo02P0oooo000c0?ooo`0908le00000000000e08ooo`3o
ol@0Ga@001AOa034Gel0Sooo00/0oooo00H0olAO01@00000000000000000501Oa?l80?ooo`030000
003oool0oooo00@0ooooD000ool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo03h0oooo
00<00000003oo`00ool0C@00ool70?ooo`030000003oool0oooo02P0oooo000c0?ooo`0608mOG`34
a5l050000000000001@0GlCo3`3oool01@2?=@00000000000000000055o400P0oooo00<000000?oo
o`3oool0103ooom@003oo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0?P3oool00`00
00000?oo003oo`1=003oo`L0oooo00<000000?ooo`3oool0:03oool003<0oooo00<0ScDe08o4G`0D
00000P000000100001@0GlCo0<AOG`2?oolI0?ooo`8000001@3ooom?003oo`030000003oool0oooo
03l0oooo00<000000?ooo`3oool0?P3oool00`0000000?oo003oo`1=003oo`H0oooo0P00000Z0?oo
o`00<`3oool00`2?=@0001AO0<Coo`020?ooo`0408le00000000000e08oooaT0oooo00<000000?oo
o`3oool0103ooom?003oo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0?`3oool00`00
00000?oo003oo`1<003oo`L0oooo00<000000?ooo`3oool0:03oool003<0oooo00<0ScDD05o4o`3o
ool00`3oool00`3oScD0000e08ooo`0I0?ooo`030000003oool0oooo00@0ooooC`00ool00`000000
oooo0?ooo`0o0?ooo`030000003oool0oooo03l0oooo00<00000003oo`00ool0C000ool70?ooo`03
0000003oool0oooo02P0oooo000c0?ooo`0308mOG`34ool0oooo00@0oooo00<0a5mO08ooo`3oool0
603oool00`000000oooo0?ooo`040?ooodl00?oo00<000000?ooo`3oool0?`3oool00`000000oooo
0?ooo`0o0?ooo`0300000000ool00?oo04`00?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00E@3o
ool00`000000oooo0?ooo`040?ooodl00?oo00<000000?ooo`3oool0?`3oool00`000000oooo0?oo
o`0o0?ooo`0300000000ool00?oo04`00?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool0
0`000000oooo0?ooo`040?ooodh00?oo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0o
0?ooo`0300000000ool00?oo04`00?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00;@3oool00`3o
a5l0Ghoo0?ooo`0>0?ooo`030<AOG`2?ool0oooo01@0oooo0P0000050?ooodh00?oo00<000000?oo
o`3oool0@03oool00`000000oooo0?ooo`100?ooo`0300000000ool00?oo04/00?oo1P3oool20000
02X0oooo000]0?ooo`030?o4G`1OSol0oooo00h0oooo00<0a5mO08ooo`3oool0503oool00`000000
oooo0?ooo`040?ooodh00?oo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`100?ooo`03
00000000ool00?oo04/00?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00;@3oool00`3oa5l0=Eo4
0?ooo`0>0?ooo`0308le=@2?ool0oooo01@0oooo00<000000?ooo`3oool0103ooom>003oo`030000
003oool0oooo0400oooo00<000000?ooo`3oool0@03oool00`0000000?oo003oo`1;003oo`L0oooo
00<000000?ooo`3oool0:03oool002d0oooo00<0olAO01@0000000003P0000000`0003D0Sooo0?oo
o`0D0?ooo`030000003oool0oooo00@0ooooCP00ool00`000000oooo0?ooo`100?ooo`030000003o
ool0oooo0400oooo00<00000003oo`00ool0B`00ool70?ooo`030000003oool0oooo02P0oooo001E
0?ooo`030000003oool0oooo00@0ooooC@00ool00`000000oooo0?ooo`110?ooo`030000003oool0
oooo0400oooo00<00000003oo`00ool0B`00ool70?ooo`030000003oool0oooo02P0oooo001E0?oo
o`030000003oool0oooo00@0ooooC@00ool00`000000oooo0?ooo`110?ooo`030000003oool0oooo
0440oooo00<00000003oo`00ool0BP00ool70?ooo`030000003oool0oooo02P0oooo001E0?ooo`80
00001@3ooom=003oo`030000003oool0oooo0440oooo00<000000?ooo`3oool0@@3oool00`000000
0?oo003oo`1:003oo`H0oooo0P00000Z0?ooo`00E@3oool00`000000oooo0?ooo`040?ooodd00?oo
00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`110?ooo`0300000000ool00?oo04X00?oo
1`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?ooodd00?oo00<0
00000?ooo`3oool0@@3oool00`000000oooo0?ooo`110?ooo`0300000000ool00?oo04X00?oo1`3o
ool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?oood`00?oo00<00000
0?ooo`3oool0@P3oool00`000000oooo0?ooo`120?ooo`0300000000ool00?oo04T00?oo1`3oool0
0`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?oood`00?oo00<000000?oo
o`3oool0@P3oool00`000000oooo0?ooo`120?ooo`0300000000ool00?oo04T00?oo1`3oool00`00
0000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?oood`00?oo00<000000?ooo`3o
ool0@P3oool00`000000oooo0?ooo`120?ooo`0300000000ool00?oo04T00?oo1`3oool00`000000
oooo0?ooo`0X0?ooo`00E@3oool2000000D0ooooC000ool00`000000oooo0?ooo`120?ooo`030000
003oool0oooo0480oooo00<00000003oo`00ool0B@00ool60?ooo`800000:P3oool005D0oooo00<0
00000?ooo`3oool0103ooom<003oo`030000003oool0oooo0480oooo00<000000?ooo`3oool0@P3o
ool00`0000000?oo003oo`19003oo`L0oooo00<000000?ooo`3oool0:03oool004L0oooo00D0olAO
01@000000000000003F?o`090?ooo`030000003oool0oooo00@0ooooC000ool00`000000oooo0?oo
o`120?ooo`030000003oool0oooo04<0oooo00<00000003oo`00ool0B000ool70?ooo`030000003o
ool0oooo02P0oooo00160?ooo`070?o4G`0D000055o40?ooo`3oohl0=@0003F?o`080?ooo`030000
003oool0oooo00@0ooooB`00ool00`000000oooo0?ooo`130?ooo`030000003oool0oooo04<0oooo
00<00000003oo`00ool0B000ool70?ooo`030000003oool0oooo02P0oooo00150?ooo`030?ooa01O
500003F?00<0oooo00<0ohle000DG`34ool01`3oool00`000000oooo0?ooo`040?oood/00?oo00<0
00000?ooo`3oool0@`3oool00`000000oooo0?ooo`130?ooo`0300000000ool00?oo04P00?oo1`3o
ool00`000000oooo0?ooo`0X0?ooo`00A@3oool00`3oa5l0500003F?o`030?ooo`030?n?=@0001@0
GlCo00L0oooo00<000000?ooo`3oool0103ooom;003oo`030000003oool0oooo04<0oooo00<00000
0?ooo`3oool0@`3oool00`0000000?oo003oo`18003oo`L0oooo00<000000?ooo`3oool0:03oool0
04D0oooo00<0olAO01@0000eSol00`3oool00`3oScD0000D05o4o`070?ooo`030000003oool0oooo
00@0ooooB`00ool00`000000oooo0?ooo`130?ooo`030000003oool0oooo04<0oooo00<00000003o
o`00ool0B000ool70?ooo`030000003oool0oooo02P0oooo00150?ooo`030?o4G`0D0000=Hoo00<0
oooo00<0ScD00000501Oa?l01`3oool3000000@0ooooB`00ool00`000000oooo0?ooo`130?ooo`03
0000003oool0oooo04@0oooo00<00000003oo`00ool0A`00ool50?ooo`<00000:P3oool004D0oooo
00P0oon?03D00000000055o40?ooo`3oScD00000000eS`P0oooo00<000000?ooo`3oool0103ooom:
003oo`030000003oool0oooo04@0oooo00<000000?ooo`3oool0A03oool00`0000000?oo003oo`17
003oo`L0oooo00<000000?ooo`3oool0:03oool004H0oooo00<0ScD0000DG`2?Ga@00P0000000`00
55l0a?oo0?ooo`070?ooo`030000003oool0oooo00@0ooooBP00ool00`000000oooo0?ooo`140?oo
o`030000003oool0oooo04@0oooo00<00000003oo`00ool0A`00ool70?ooo`030000003oool0oooo
02P0oooo00160?ooo`030?o4G`0D0000=Hoo00`0oooo00<000000?ooo`3oool0103ooom:003oo`03
0000003oool0oooo04@0oooo00<000000?ooo`3oool0A03oool00`0000000?oo003oo`17003oo`L0
oooo00<000000?ooo`3oool0:03oool004L0oooo00<0a5lD0000501Oa?l02`3oool00`000000oooo
0?ooo`040?ooodX00?oo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`140?ooo`030000
0000ool00?oo04L00?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00B03oool00`3oScD00000000e
S`0:0?ooo`030000003oool0oooo00@0ooooBP00ool00`000000oooo0?ooo`140?ooo`030000003o
ool0oooo04D0oooo00<00000003oo`00ool0AP00ool70?ooo`030000003oool0oooo02P0oooo0019
0?ooo`050?ooa01O500000000000=@2?ool01`3oool2000000D0ooooB@00ool00`000000oooo0?oo
o`150?ooo`030000003oool0oooo04D0oooo00<00000003oo`00ool0AP00ool60?ooo`800000:P3o
ool005D0oooo00<000000?ooo`3oool0103ooom9003oo`030000003oool0oooo04D0oooo00<00000
0?ooo`3oool0A@3oool00`0000000?oo003oo`16003oo`L0oooo00<000000?ooo`3oool0:03oool0
05D0oooo00<000000?ooo`3oool0103ooom9003oo`030000003oool0oooo04D0oooo00<000000?oo
o`3oool0A@3oool00`0000000?oo003oo`16003oo`L0oooo00<000000?ooo`3oool0:03oool005D0
oooo00<000000?ooo`3oool0103ooom9003oo`030000003oool0oooo04D0oooo00<000000?ooo`3o
ool0AP3oool00`0000000?oo003oo`15003oo`L0oooo00<000000?ooo`3oool0:03oool005D0oooo
00<000000?ooo`3oool0103ooom9003oo`030000003oool0oooo04D0oooo00<000000?ooo`3oool0
AP3oool00`0000000?oo003oo`15003oo`L0oooo00<000000?ooo`3oool0:03oool005D0oooo00<0
00000?ooo`3oool0103ooom8003oo`030000003oool0oooo04H0oooo00<000000?ooo`3oool0AP3o
ool00`0000000?oo003oo`15003oo`L0oooo00<000000?ooo`3oool0:03oool005D0oooo0P000005
0?ooodP00?oo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`160?ooo`0300000000ool0
0?oo04D00?oo1P3oool2000002X0oooo001E0?ooo`030000003oool0oooo00@0ooooB000ool00`00
0000oooo0?ooo`160?ooo`030000003oool0oooo04H0oooo00<00000003oo`00ool0A@00ool70?oo
o`030000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo00@0ooooB000ool00`000000
oooo0?ooo`160?ooo`030000003oool0oooo04L0oooo00<00000003oo`00ool0A000ool70?ooo`03
0000003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo00@0ooooB000ool00`000000oooo
0?ooo`160?ooo`030000003oool0oooo04L0oooo00<00000003oo`00ool0A000ool70?ooo`030000
003oool0oooo02P0oooo001E0?ooo`030000003oool0oooo00@0ooooB000ool00`000000oooo0?oo
o`160?ooo`030000003oool0oooo04L0oooo00<00000003oo`00ool0A000ool70?ooo`030000003o
ool0oooo02P0oooo001E0?ooo`030000003oool0oooo00@0ooooA`00ool00`000000oooo0?ooo`17
0?ooo`030000003oool0oooo04L0oooo00<00000003oo`00ool0A000ool70?ooo`030000003oool0
oooo02P0oooo001E0?ooo`8000001@3ooom7003oo`030000003oool0oooo04L0oooo00<000000?oo
o`3oool0A`3oool00`0000000?oo003oo`14003oo`H0oooo0P00000Z0?ooo`00E@3oool00`000000
oooo0?ooo`040?ooodL00?oo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`180?ooo`03
00000000ool00?oo04<00?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo
0?ooo`040?ooodL00?oo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`180?ooo`030000
0000ool00?oo04<00?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?oo
o`040?ooodL00?oo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`180?ooo`0300000000
ool00?oo04<00?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`04
0?ooodH00?oo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`180?ooo`0300000000ool0
0?oo04<00?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?oo
odH00?oo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`180?ooo`0300000000ool00?oo
04<00?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?ooodH0
0?oo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`180?ooo`0300000000ool00?oo04<0
0?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool2000000D0ooooAP00ool00`000000oooo
0?ooo`180?ooo`030000003oool0oooo04T0oooo00<00000003oo`00ool0@P00ool60?ooo`800000
:P3oool004P0oooo00<0a5lD01AOa03oool02P3oool00`000000oooo0?ooo`040?ooodH00?oo00<0
00000?ooo`3oool0B03oool00`000000oooo0?ooo`190?ooo`0300000000ool00?oo04800?oo1`3o
ool00`000000oooo0?ooo`0X0?ooo`00B03oool00`3oa5l051AO0<Coo`0:0?ooo`030000003oool0
oooo00@0ooooA@00ool00`000000oooo0?ooo`190?ooo`030000003oool0oooo04T0oooo00<00000
003oo`00ool0@P00ool70?ooo`030000003oool0oooo02P0oooo00180?ooo`030?ooa01O51@0GlCo
00X0oooo00<000000?ooo`3oool0103ooom5003oo`030000003oool0oooo04T0oooo00<000000?oo
o`3oool0B@3oool00`0000000?oo003oo`12003oo`L0oooo00<000000?ooo`3oool0:03oool004T0
oooo00<0a5lD01AOa03oool02@3oool00`000000oooo0?ooo`040?ooodD00?oo00<000000?ooo`3o
ool0B@3oool00`000000oooo0?ooo`190?ooo`0300000000ool00?oo04800?oo1`3oool00`000000
oooo0?ooo`0X0?ooo`00B@3oool00`3oa5l051AO0<Coo`090?ooo`030000003oool0oooo00@0oooo
A@00ool00`000000oooo0?ooo`190?ooo`030000003oool0oooo04T0oooo00<00000003oo`00ool0
@P00ool70?ooo`030000003oool0oooo02P0oooo00190?ooo`030?ooa01O51@0GlCo00T0oooo0`00
00040?ooodD00?oo00<000000?ooo`3oool0B@3oool00`000000oooo0?ooo`1:0?ooo`0300000000
ool00?oo04400?oo1@3oool3000002X0oooo001:0?ooo`030<AO5000=Hl0oooo00P0oooo00<00000
0?ooo`3oool0103ooom5003oo`030000003oool0oooo04T0oooo00<000000?ooo`3oool0BP3oool0
0`0000000?oo003oo`11003oo`L0oooo00<000000?ooo`3oool0:03oool004X0oooo00<0oon?03D0
=@2?ool0203oool00`000000oooo0?ooo`040?oood@00?oo00<000000?ooo`3oool0BP3oool00`00
0000oooo0?ooo`1:0?ooo`0300000000ool00?oo04400?oo1`3oool00`000000oooo0?ooo`0X0?oo
o`00A@3oool00`3oa<@0a?oo0?ooo`030?ooo`0308le000eSol0oooo00L0oooo00<000000?ooo`3o
ool0103ooom4003oo`030000003oool0oooo04X0oooo00<000000?ooo`3oool0BP3oool00`000000
0?oo003oo`11003oo`L0oooo00<000000?ooo`3oool0:03oool004D0oooo00<0oon?03DDG`34ool0
0`3oool00`3oScD003F?0?ooo`070?ooo`030000003oool0oooo00@0ooooA000ool00`000000oooo
0?ooo`1:0?ooo`030000003oool0oooo04X0oooo00<00000003oo`00ool0@@00ool70?ooo`030000
003oool0oooo02P0oooo00160?ooo`0308le00000000000000<0000000<0000e08ooo`3oool01P3o
ool00`000000oooo0?ooo`040?oood@00?oo00<000000?ooo`3oool0BP3oool00`000000oooo0?oo
o`1:0?ooo`0300000000ool00?oo04400?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00AP3oool0
0`3oScD00000000000040000000303F?o`3oool0oooo00D0oooo0P0000050?oood@00?oo00<00000
0?ooo`3oool0BP3oool00`000000oooo0?ooo`1;0?ooo`0300000000ool00?oo04000?oo1P3oool2
000002X0oooo001E0?ooo`030000003oool0oooo00@0oooo@`00ool00`000000oooo0?ooo`1;0?oo
o`030000003oool0oooo04/0oooo00<00000003oo`00ool0@000ool70?ooo`030000003oool0oooo
02P0oooo001E0?ooo`030000003oool0oooo00@0oooo@`00ool00`000000oooo0?ooo`1;0?ooo`03
0000003oool0oooo04/0oooo00<00000003oo`00ool0@000ool70?ooo`030000003oool0oooo02P0
oooo001E0?ooo`030000003oool0oooo00@0oooo@`00ool00`000000oooo0?ooo`1;0?ooo`030000
003oool0oooo04/0oooo00<00000003oo`00ool0@000ool70?ooo`030000003oool0oooo02P0oooo
001E0?ooo`030000003oool0oooo00@0oooo@`00ool00`000000oooo0?ooo`1;0?ooo`030000003o
ool0oooo04/0oooo00<00000003oo`00ool0@000ool70?ooo`030000003oool0oooo02P0oooo001E
0?ooo`030000003oool0oooo00@0oooo@`00ool00`000000oooo0?ooo`1;0?ooo`030000003oool0
oooo04`0oooo00<00000003oo`00ool0?`00ool70?ooo`030000003oool0oooo02P0oooo001E0?oo
o`8000001@3ooom3003oo`030000003oool0oooo04/0oooo00<000000?ooo`3oool0C03oool00`00
00000?oo003oo`0o003oo`H0oooo0P00000Z0?ooo`00E@3oool00`000000oooo0?ooo`040?oood80
0?oo00<000000?ooo`3oool0C03oool00`000000oooo0?ooo`1<0?ooo`0300000000ool00?oo03l0
0?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?oood800?oo
00<000000?ooo`3oool0C03oool00`000000oooo0?ooo`1<0?ooo`0300000000ool00?oo03l00?oo
1`3oool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?oood800?oo00<0
00000?ooo`3oool0C03oool00`000000oooo0?ooo`1<0?ooo`0300000000ool00?oo03l00?oo1`3o
ool00`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?oood800?oo00<00000
0?ooo`3oool0C03oool00`000000oooo0?ooo`1<0?ooo`0300000000ool00?oo03l00?oo1`3oool0
0`000000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?oood800?oo00<000000?oo
o`3oool0C03oool00`000000oooo0?ooo`1=0?ooo`0300000000ool00?oo03h00?oo1`3oool00`00
0000oooo0?ooo`0X0?ooo`00E@3oool00`000000oooo0?ooo`040?oood400?oo00<000000?ooo`3o
ool0C@3oool00`000000oooo0?ooo`1=0?ooo`0300000000ool00?oo03h00?oo1`3oool00`000000
oooo0?ooo`0X0?ooo`00E@3oool2000000D0oooo@@00ool00`000000oooo0?ooo`1=0?ooo`030000
003oool0oooo04d0oooo00<00000003oo`00ool0?P00ool60?ooo`800000:P3oool005D0oooo00<0
00000?ooo`3oool0103ooom1003oo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0C@3o
ool00`0000000?oo003oo`0n003oo`L0oooo00<000000?ooo`3oool0:03oool005D0oooo00<00000
0?ooo`3oool0103ooom1003oo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0C@3oool0
0`0000000?oo003oo`0n003oo`L0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?oo
o`3oool0103ooom1003oo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0C@3oool00`00
00000?oo003oo`0n003oo`L0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3o
ool0103ooom0003oo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0CP3oool00`000000
0?oo003oo`0m003oo`L0oooo00<000000?ooo`3oool0:03oool005D0oooo00<000000?ooo`3oool0
103ooom0003oo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0CP3oool00`0000000?oo
003oo`0m003oo`L0oooo00<000000?ooo`3oool0:03oool005D0oooo0P0000050?oood000?oo00<0
00000?ooo`3oool0CP3oool00`000000oooo0?ooo`1>0?ooo`0300000000ool00?oo03d00?oo1P3o
ool2000002X0oooo00170?ooo`050<AO5000000000000000000DGl@02@3oool00`000000oooo0?oo
o`040?oood000?oo00<000000?ooo`3oool0CP3oool00`000000oooo0?ooo`1>0?ooo`0300000000
ool00?oo03d00?oo1`3oool00`000000oooo0?ooo`0X0?ooo`00AP3oool01`34Ga@0000003F?o`3o
ool0ooo405lD000DGl@0203oool00`000000oooo0?ooo`040?oood000?oo00<000000?ooo`3oool0
CP3oool00`000000oooo0?ooo`1>0?ooo`0300000000ool00?oo03d00?oo1`3oool00`000000oooo
0?ooo`0X0?ooo`00A@3oool00`3ool@0Ga@0000eS`030?ooo`030?n?=@0055l0a?oo00L0oooo00<0
00000?ooo`3oool0103ooom0003oo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0CP3o
ool00`0000000?oo003oo`0m003oo`L0oooo00<000000?ooo`3oool0:03oool004H0oooo00<0ScD0
000eS`3oool00P3oool00`2?=@0001AO0<Coo`070?ooo`030000003oool0oooo00@0oooo7@00ool0
0`0000000?oo003oo`0O003oo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0=`3oool0
0`000000oooo0?ooo`0g0?ooo`030000003oool0oooo01D0oooo00<00000003oo`00ool07`00ool0
0`0000000?oo003oo`0J003oo`L0oooo00<000000?ooo`3oool0:03oool004H0oooo00L0olAO01@0
=@2?ool0oooo08le00000000=Hoo00P0oooo00<000000?ooo`3oool0103oool00`0000000?oo003o
o`0<003oo`0300000000ool00?oo00/00?oo00<00000003oo`00ool03000ool00`0000000?oo003o
o`0;003oo`0300000000ool00?oo00800?oo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo00L0oooo00<00000003oo`00ool00P00ool00`0000000?oo003o
o`0;003oo`0300000000ool00?oo00`00?oo00<00000003oo`00ool02`00ool00`0000000?oo003o
o`0<003oo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0:03oool004L0oooo00D0oon?
03DD500D0000000001AOa0090?ooool00000<P00000Z0?ooo`00A`3oool0103oa5l0500000000000
=Hoo0?ooofH0oooo00160?ooo`060?o4G`0D0000000e08ooo`3oScD003F?o`3ooomU0?ooo`00AP3o
ool00`2?=@00000e08ooo`020?ooo`0308le000DGl@0oooo0?l0ooooH`3oool004H0oooo00<0ScD0
000DG`34ool00P3oool00`34Ga@003F?0?ooo`3o0?ooof<0oooo00160?ooo`070?n?=@000000=Hoo
0?ooo`3oohl0=@0003F?o`3o0?ooof@0oooo00170?ooo`050?n?=@00000000000000000DGl@0o`3o
oomU0?ooo`00o`3oool40?ooo`060?ooa01O500000000000000003D0SoooY`3oool00?l0oooo1P3o
ool00`2?=@0001AO0<Coo`2X0?ooo`00o`3oool60?ooo`0308le000055l0a?oo0:P0oooo003o0?oo
o`H0oooo00<0ScD0000DG`34ool0Z03oool009P0oooo00P0ooo405lD000000000000000DG`1O=@00
0000000eS`D0oooo00P0ooo405lD0000000000000000500D500001AO0<Coo`80oooo00D0a5lD0000
00000000000003F?o`030?ooo`0;0?ooS`0e000000000000000DGhl0Ga@0000DG`34ool0oooo0?oo
a01O50000P0000001@0001@0GlCo0?ooo`3oool0a5lD0080000000@0000e08ooo`3oool0ScD00P00
00000`0003D0Sooo0?ooo`0@0?ooo`030?o4G`0eGl@0oooo0080oooo00<0a5le05o4o`3oool0103o
ool01`2?=@0000000000000003D0Sooo0?ooo`34Ga@00P0000001P0003D0GelD000000000000000e
08ooo`H0oooo00<0ScD0000D500D00000P0000000`0DGl@0oooo0?ooo`020?ooo`050<AO50000000
00000000000eSol00`3oool0103oa5l050000000000DGl@20?ooo`0708le0000000000000000=@2?
ool0oooo08le0002000000030000=@2?ol@0Ga@00080000000L001AO0<Coo`3oa5l050000000000D
Gl@0a5lD0080000000<0000e08ooa01O50000P0000001@0055l0GcD000000000=Hl0a5lD00800000
00<0000e08ooo`3oool0K@3oool009P0oooo00T0ohle0000000eSol0oooo0?o4G`0D000003F?0?n?
S`2?ool00`3oool04@3ool@0Ga@00000000eSol0oooo0?ooS`0e000000000000=@2?ool0a5lD0000
000DGl@0oooo0?ooa01O500055o40080oooo00D0a5lD0000=@2?ool0oooo0<AO5002000000090000
=@2?ool0ScD0000DG`34ool0oooo0?o4G`0D0000=Hoo0080oooo00/0a5lD0000=@2?ool0oooo08le
0000000003F?0?ooo`3oa5l0500e08ooo`0@0?ooo`0708le000003D0Sooo0?ooo`3oohl0=@00000e
S`040?ooo`0?08le0000000003F?0?ooo`3oa5l0500e08ooo`3oool0a5lD0000=@2?ool0oooo0<AO
500003D0Sooo00L0oooo00L0ScD00000000eSol0oooo0?ooS`0e000055o40080oooo00L0a5lD0000
000DGl@0oooo0?ooa01O500055o40080oooo01@0a5lD0000=@2?ool0a8n?0<Coo`2?=@000000000e
S`3oool0olAO01@0=@2?ool0oooo0<AO500055l0a?oo0?ooo`3ool@0Ga@0000eS`80oooo01@0a5lD
0000=@2?ool0a8n?0<Coo`34Ga@0000e08ooo`3oScD0000003F?o`3oool0olAO01@00000=Hl0ohn?
08ooo`34Ga@0000e08ooofl0oooo002H0?ooo`070?n?=@0001@0GlCo0?ooo`3ool@0Ga@0000eS`05
0?ooo`030?o4G`0D000055o40080oooo00L0ooo405lD0000=Hl0oooo0?o4G`0D000003F?00<0oooo
00/0ohle0000=@2?ool0oooo08le000055l0a?oo0?ooo`3ool@0Ga@0000eS`020?ooo`0308le0000
55l0a?oo0080oooo00X0ohle0000=@2?ool0oooo0<AO500003D0Sooo0?o4G`0D000055o40`3oool0
0`3oa5l0=Eo40?ooo`0>0?ooo`090?o4G`0D000053De03EOa03oool0ohle0000500D51@0GlCo0080
oooo00<0olAO01@0000DGl@00`3oool02P3oa5l0=Eo40?ooo`34Ga@0000e08ooo`3oool0a5lD0000
=@2?ool70?ooo`0308le000055l0a?oo0080oooo00H0ohle0000=@2?ool0olAO01@00000=Hl30?oo
o`0;0?n?=@0003D0Sooo0?ooo`34Ga@001AO0<Coo`3oool0olAO01@0000DGl@00`3oool02P3oa5l0
=Eo40?ooo`34Ga@001AO0<Coo`3oool0ooo405lD0000=Hl20?ooo`030<AO500055l0a?oo0080oooo
00X0a5lD0000=@2?ool0ohle0000501Oa?l0oooo0?ooa01O500003F?0P3oool00`34Ga@0000e08oo
o`1_0?ooo`00V@3oool01P3oScD00000000eS`3ool@0Ga@0000eS`D0oooo00<0ohle0000000eSol0
0P3oool01`3ool@0Ga@0000eS`3oool0ohle0000000eSol00`3oool02`3oScD0000003F?o`3oool0
ScD0000DG`34ool0oooo0?ooa01O500003F?0080oooo00<0ScD0000DG`34ool00P3oool02P3oa5l0
500003F?o`3oool0a5lD0000=@2?ool0ohle0000501Oa?lD0?ooo`090<AO500003D0Sooo08le=@2?
ol@0Ga@0000eS`3oa5l0=Eo40080oooo00<0ohle0000501Oa?l01P3oool01`34Ga@0000e08ooo`3o
ool0a5lD0000=@2?ool01`3oool00`2?=@0001AO0<Coo`020?ooo`060?o4G`0D0000=Hoo0?n?=@00
0000=Hoo0`3oool02`3oScD0000003F?o`3oool0a5lD000DG`34ool0oooo0?n?=@0001@0GlCo00H0
oooo00L0a5lD000DG`34ool0oooo0?ooa01O500003F?0080oooo00<0a5lD000DG`34ool00P3oool0
2P34Ga@0000e08ooo`3oool0ohle00000000=Hl0ooo405lD0000=Hl20?ooo`030<AO500003D0Sooo
06l0oooo002K0?ooo`040?n?=@0000000000000eS`D0oooo00<0ohle0000000eSol00P3oool01`3o
ol@0Ga@0000eS`3oool0ohle0000000eSol00`3oool02`3oScD0000003F?o`3oool0ScD0000DG`34
ool0oooo0?ooa01O500003F?0080oooo00<0ScD0000DG`34ool00P3oool02P3oa5l0500003F?o`3o
ool0a5lD0000=@2?ool0ohle0000=@2?ool60?ooo`030?ooa01O5000000000L00000018055o40?oo
o`3oool0oon?03D0000DGl@0oooo0?o4G`0e=CD0500D05o4o`3oool0ScDe08ooo`3oool0ohle0000
=@2?ool60?ooo`070<AO500003D0Sooo0?ooo`34Ga@0000e08ooo`070?ooo`0308le000055l0a?oo
0080oooo00H0olAO01@0000eSol0ohle0000000eSol30?ooo`0;0?n?=@000000=Hoo0?ooo`34Ga@0
01AO0<Coo`3oool0ohle0000=@2?ool01P3oool01`34Ga@001AO0<Coo`3oool0ooo405lD0000=Hl0
0P3oool00`34Ga@001AO0<Coo`020?ooo`030<AO500003D0Sooo00<0oooo00@0ohle000000000000
03F?0P3oool00`34Ga@0000e08ooo`1_0?ooo`00V03oool01`3oohl0=@0003F?o`3oool0ooo405lD
0000=Hl01@3oool00`3oohl0=@0001AOa0020?ooo`070?ooa01O500003F?0?ooo`3oohl0=@0003F?
o`030?ooo`0;08le000001@0GlCo0?ooo`2?=@0001AO0<Coo`3oool0ooo405lD0000=Hl00P3oool0
0`2?=@0001AO0<Coo`020?ooo`090?n?=@0001@0GlCo0?ooo`34Ga@0000e08ooo`3oa5l0500000D0
000000<055o40?ooo`3oool0303oool01`34Ga@0000D05o4o`3oool0ooo405lD0000=Hl00P3oool0
1@3oScD055o40?ooo`3oa5l0500000D0000000T055o40?ooo`34Ga@0000e08ooo`3oool0a5lD0000
=@2?ool01`3oool00`2?=@00000e08ooo`020?ooo`060<AO500001@0GlCo0?ooS`0e0000=Hoo0`3o
ool02P2?=@00000D05o4o`3oool0a5lD000DG`34ool0oooo0?o4G`0D00050000000901AOa03oool0
ScD00000=@2?ool0oooo0?ooa01O500003F?0080oooo00<0a5lD000DG`34ool00P3oool02P34Ga@0
000e08ooo`3oohl0=@0003F?o`3oool0ooo405lD0000=Hl20?ooo`030<AO500003D0Sooo06l0oooo
002H0?ooo`070?ooS`0e000055o40?ooo`3oohl0=@00000eS`060?ooo`070?n?=@0001@0GlCo0?oo
o`3oScD00000000eS`020?ooo`070<AO500055l0a?oo0?ooo`3oScD00000000eS`020?ooo`0708le
000055l0a?oo0?ooo`3ool@0Ga@0000eS`020?ooo`0?08le0000000001AO0<Coo`3oohl0=@00000D
G`34ool0oooo0<AO500003D0Sooo0?ooo`2?=@0055o40080oooo00<0a5lD0000501Oa?l03@3oool0
0`3oohl0=@00000eS`020?ooo`030?o4G`0D0000=Hoo0080oooo00H0ooo405lD=@2?ool0oooo08le
000DGl@20?ooo`0;0<AO500001@0GlCo0?ooo`34Ga@0000e08ooo`3oool0a5lD0000=@2?ool01@3o
ool04P3oohl0=@000000000051@051AO0<Coo`3oa5l05000000DG`34ool0oooo0<AO500055l0a?oo
0?ooo`3oScD00000000eS`80oooo00<0a5lD000DG`34ool00P3oool03`2?=@0055o40?ooo`3oool0
a5lD0000501Oa<@0Ga@00000000051@0500D05o4o`3oohl0=@0001AOa0020?ooo`0?0<AO500055l0
a?oo0?ooa01O500000000000=@2?ool0oon?03D0000DGl@0oooo0?ooS`0e000003F?0080oooo00<0
a5lD0000=@2?ool0K`3oool009T0oooo00H0oon?03D00000000000000000=@2?ool70?ooo`030?n?
=@00000000000080000000D003F?0?ooo`3oool0oooo0?n?=@030000000;01AOa03oool0oon?03D0
0000000001AO0<Coo`3oa5l0500000000000=Hl00P3oool01`2?=@0001AO0<C4G`0D00000000000D
G`34ool00P3oool00`34Ga@0000e08ooo`020?ooo`050<AO5000000000000000501Oa?l03@3oool0
1P3ool@0Ga@000000000000055o40?n?=@<0000000P055o40?ooo`2?=@000000000DG`34ool0oooo
0<AO5080000000/0000D05o4o`3oool0oooo0<AO500003D0Sooo0?ooo`34Ga@0000e08ooo`060?oo
o`080?ooa01O500001AO0<C4G`0D000000000000=@2?ool30?ooo`050?n?=@00000000000000000D
Gl@00P3oool0202?=@00000000000000000001AO0<Coo`3oool0a5lD0P0000003@0001@0GlCo0?oo
o`3oool0ScD0000DG`34ol@0Ga@00000000003D0Sooo0?ooo`2?=@000`000000200055l0a?oo08le
000003D0Sooo0?ooo`3oohl0=@000P0000001`0003D0Sooo0?ooo`3oool0a5lD0000=@2?ool0K`3o
ool00:T0oooo00<0ooo405lD0000=Hl0503oool00`2?=@0001AO0<Coo`060?ooo`030<AO500003D0
Sooo02/0oooo00L0a5lD0000=@2?ool0oooo0<AO500003D0Sooo01T0oooo00<0ScD0000DG`34ool0
4`3oool00`2?=@0001AO0<Coo`0>0?ooo`030<AO500003D0Sooo06l0oooo002Y0?ooo`030?ooa01O
500003F?01@0oooo00<0ScD0000DG`34ool01P3oool00`34Ga@0000e08ooo`0[0?ooo`070<AO5000
03D0Sooo0?ooo`34Ga@0000e08ooo`0I0?ooo`030?ooS`0e55l0a?oo01<0oooo00<0oon?03DDG`34
ool03P3oool00`34Ga@0000e08ooo`1_0?ooo`00Z@3oool00`3ool@0Ga@0000eS`0D0?ooo`0308le
000055l0a?oo00H0oooo00<0a5lD0000=@2?ool0:`3oool01`34Ga@0000e08ooo`3oool0a5lD0000
=@2?ool06@3oool00`3ool@0GcEO0<Coo`0C0?ooo`030?ooa01O=El0a?oo00h0oooo00<0a5lD0000
=@2?ool0K`3oool00:P0oooo00@0oon?03D00000000003F?4P3oool01@3oohl0=@000000000055l0
a?oo00@0oooo00D0ooo405lD00000000000e08ooo`0Y0?ooo`090?ooa01O500000000000=@2?ol@0
Ga@00000000003D0Sooo03@0oooo00<0a5lD0000501Oa?l01`3oool01@3ool@0Ga@00000000003D0
Sooo06l0oooo002Y0?ooo`030?ooa01O500003F?01<0oooo00@0ooo405lD000055l0a?oo1P3oool0
0`2?=@00000e08ooo`0[0?ooo`0708le000003D0Sooo0?ooo`2?=@00000e08ooo`0d0?ooo`030<AO
500001@0GlCo00T0oooo00<0ScD00000=@2?ool0K`3oool00?l0oooo/@3oool00?l0oooo/@3oool0
0?l0oooo/@3oool00?l0oooo/@3oool00001\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {-0.000937647, -5.00002*^-06, \
0.0034868, 0.00564177}, {{0.375, 430.563}, {265.875, 0}} -> {-8.21118, \
0.374123, 0.0518628, 0.0479517}, {{0.375, 430.563}, {265.875, 0}} -> \
{-8.21118, 0.374123, 0.0518628, 0.0479517}}]
}, Closed]],

Cell[TextData[{
  "Suppose that the energy is between the bottom of the well and its central \
maximum, ",
  Cell[BoxData[
      \(TraditionalForm\`2.6 < \[CurlyEpsilon] < 5\)]],
  ".  There will then be two pairs of classical turning points, symmetrically \
placed, where the energy and potential are equal, such that ",
  Cell[BoxData[
      \(TraditionalForm\`\[CurlyEpsilon] \[Equal] V[x\_tp]\)]],
  ".  In the outer regions and the central region where the potential is \
above the energy, the wave function is a mixture of exponentially decaying \
and exponentially growing components, whereas it becomes oscillatory within \
the two wells in the regions where the energy is below the potential.   If \
the energy is near the top of the barrier, the wave function is not strongly \
suppressed within the barrier, but if the energy is very close to the bottom \
of the well, the central barrier then looks very thick and there is very \
little penetration of the wave function into this ",
  StyleBox["classically forbidden",
    FontSlant->"Italic"],
  " region.  We will find that the thick barrier case is numerically very \
challenging.  The effective thickness of the barrier is determined by the \
mass."
}], "Text"],

Cell["\<\
It is useful to construct an equivalent harmonic oscillator that is a good \
approximation near the bottom of a well.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(potseries = Series[pot[x], {x, x0, 2}]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{
      "2.6094379124341005`", 
        "+", \(1.5334400416122662`*^-12\ \((x - 1.2686362411797578`)\)\), 
        "+", \(3.2188758248680687`\ \((x - 1.2686362411797578`)\)\^2\), "+", 
        InterpretationBox[\(O[x - 1.2686362411797578`]\^3\),
          SeriesData[ x, 1.2686362411797578, {}, 0, 3, 1],
          Editable->False]}],
      SeriesData[ 
      x, 1.2686362411797578, {2.6094379124341005, .15334400416122662*^-11, 
        3.2188758248680687}, 0, 3, 1],
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pot1[x_] = Normal[potseries]\)], "Input"],

Cell[BoxData[
    \(\(\(2.6094379124341005`\)\(\[InvisibleSpace]\)\) + 
      1.5334400416122662`*^-12\ \((\(-1.2686362411797578`\) + x)\) + 
      3.2188758248680687`\ \((\(-1.2686362411797578`\) + x)\)\^2\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[{pot[x], pot1[x]}, {x, \(-3\), 3}, 
        PlotStyle \[Rule] {Dashing[{}], Dashing[{0.02, 0.04}]}];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.15873 -0.23684 0.0964016 [
[.02381 .03987 -9.15625 -15.6875 ]
[.02381 .03987 9.15625 0 ]
[.18254 .03987 -9.15625 -15.6875 ]
[.18254 .03987 9.15625 0 ]
[.34127 .03987 -9.15625 -15.6875 ]
[.34127 .03987 9.15625 0 ]
[.65873 .03987 -5 -15.6875 ]
[.65873 .03987 5 0 ]
[.81746 .03987 -5 -15.6875 ]
[.81746 .03987 5 0 ]
[.97619 .03987 -5 -15.6875 ]
[.97619 .03987 5 0 ]
[.4875 .14877 -10 -7.84375 ]
[.4875 .14877 0 7.84375 ]
[.4875 .24517 -10 -7.84375 ]
[.4875 .24517 0 7.84375 ]
[.4875 .34157 -10 -7.84375 ]
[.4875 .34157 0 7.84375 ]
[.4875 .43797 -10 -7.84375 ]
[.4875 .43797 0 7.84375 ]
[.4875 .53437 -10 -7.84375 ]
[.4875 .53437 0 7.84375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .05237 m
.02381 .05862 L
s
gsave
.02381 .03987 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
.18254 .05237 m
.18254 .05862 L
s
gsave
.18254 .03987 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.34127 .05237 m
.34127 .05862 L
s
gsave
.34127 .03987 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.65873 .05237 m
.65873 .05862 L
s
gsave
.65873 .03987 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.81746 .05237 m
.81746 .05862 L
s
gsave
.81746 .03987 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.97619 .05237 m
.97619 .05862 L
s
gsave
.97619 .03987 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
.125 Mabswid
.05556 .05237 m
.05556 .05612 L
s
.0873 .05237 m
.0873 .05612 L
s
.11905 .05237 m
.11905 .05612 L
s
.15079 .05237 m
.15079 .05612 L
s
.21429 .05237 m
.21429 .05612 L
s
.24603 .05237 m
.24603 .05612 L
s
.27778 .05237 m
.27778 .05612 L
s
.30952 .05237 m
.30952 .05612 L
s
.37302 .05237 m
.37302 .05612 L
s
.40476 .05237 m
.40476 .05612 L
s
.43651 .05237 m
.43651 .05612 L
s
.46825 .05237 m
.46825 .05612 L
s
.53175 .05237 m
.53175 .05612 L
s
.56349 .05237 m
.56349 .05612 L
s
.59524 .05237 m
.59524 .05612 L
s
.62698 .05237 m
.62698 .05612 L
s
.69048 .05237 m
.69048 .05612 L
s
.72222 .05237 m
.72222 .05612 L
s
.75397 .05237 m
.75397 .05612 L
s
.78571 .05237 m
.78571 .05612 L
s
.84921 .05237 m
.84921 .05612 L
s
.88095 .05237 m
.88095 .05612 L
s
.9127 .05237 m
.9127 .05612 L
s
.94444 .05237 m
.94444 .05612 L
s
.25 Mabswid
0 .05237 m
1 .05237 L
s
.5 .14877 m
.50625 .14877 L
s
gsave
.4875 .14877 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.5 .24517 m
.50625 .24517 L
s
gsave
.4875 .24517 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
.5 .34157 m
.50625 .34157 L
s
gsave
.4875 .34157 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.5 .43797 m
.50625 .43797 L
s
gsave
.4875 .43797 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7) show
1.000 setlinewidth
grestore
.5 .53437 m
.50625 .53437 L
s
gsave
.4875 .53437 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8) show
1.000 setlinewidth
grestore
.125 Mabswid
.5 .07165 m
.50375 .07165 L
s
.5 .09093 m
.50375 .09093 L
s
.5 .11021 m
.50375 .11021 L
s
.5 .12949 m
.50375 .12949 L
s
.5 .16805 m
.50375 .16805 L
s
.5 .18733 m
.50375 .18733 L
s
.5 .20661 m
.50375 .20661 L
s
.5 .22589 m
.50375 .22589 L
s
.5 .26445 m
.50375 .26445 L
s
.5 .28373 m
.50375 .28373 L
s
.5 .30301 m
.50375 .30301 L
s
.5 .32229 m
.50375 .32229 L
s
.5 .36085 m
.50375 .36085 L
s
.5 .38013 m
.50375 .38013 L
s
.5 .39941 m
.50375 .39941 L
s
.5 .41869 m
.50375 .41869 L
s
.5 .45725 m
.50375 .45725 L
s
.5 .47653 m
.50375 .47653 L
s
.5 .49581 m
.50375 .49581 L
s
.5 .51509 m
.50375 .51509 L
s
.5 .03308 m
.50375 .03308 L
s
.5 .0138 m
.50375 .0138 L
s
.5 .55365 m
.50375 .55365 L
s
.5 .57293 m
.50375 .57293 L
s
.5 .59221 m
.50375 .59221 L
s
.5 .61149 m
.50375 .61149 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02748 .61803 m
.06244 .49594 L
.10458 .36238 L
.14415 .25083 L
.18221 .15833 L
.20342 .1144 L
.22272 .08013 L
.24299 .05093 L
.26171 .03104 L
.271 .02394 L
.27575 .02107 L
.28086 .01857 L
.28352 .0175 L
.28596 .01668 L
.28844 .01599 L
.29075 .01548 L
.29215 .01523 L
.29342 .01505 L
.29483 .01489 L
.2956 .01483 L
.29633 .01478 L
.29698 .01475 L
.29766 .01473 L
.29888 .01472 L
.3002 .01474 L
.30094 .01478 L
.30164 .01483 L
.3029 .01494 L
.30424 .0151 L
.30665 .01551 L
.30923 .0161 L
.31209 .01695 L
.3172 .01897 L
.32199 .02144 L
.33092 .02753 L
.34055 .03615 L
.36059 .06039 L
.38169 .09327 L
.4198 .16117 L
.44105 .19636 L
.46036 .22203 L
.46999 .23169 L
.47507 .2358 L
.48044 .23937 L
.48514 .24181 L
.48763 .24284 L
.49031 .24373 L
.49276 .24437 L
.49501 .24479 L
.49609 .24493 L
Mistroke
.49726 .24505 L
.49837 .24513 L
.4994 .24516 L
.50069 .24516 L
.50134 .24514 L
.50206 .2451 L
.50332 .245 L
.5045 .24486 L
.50589 .24464 L
.50715 .24439 L
.51 .24364 L
.51245 .2428 L
.51512 .24169 L
.51997 .23913 L
.52964 .23201 L
.53858 .22321 L
.55729 .19886 L
.5779 .16521 L
.61816 .09353 L
.63667 .06429 L
.64716 .05009 L
.6569 .03879 L
.66742 .02886 L
.67698 .02205 L
.68224 .01923 L
.68719 .0172 L
.68992 .01633 L
.69242 .0157 L
.69518 .01519 L
.69657 .015 L
.6973 .01492 L
.69809 .01485 L
.69939 .01476 L
.70059 .01472 L
.70189 .01472 L
.70263 .01473 L
.70332 .01476 L
.70456 .01484 L
.70571 .01495 L
.70696 .0151 L
.70828 .0153 L
.71058 .01575 L
.71272 .01629 L
.71758 .01792 L
.72287 .02033 L
.72768 .02309 L
.73848 .0312 L
.74845 .04093 L
.75764 .05172 L
.77843 .082 L
Mistroke
.8178 .15835 L
.85566 .25032 L
.89597 .36402 L
.93476 .48663 L
Mfstroke
.93476 .48663 m
.97252 .61803 L
s
[ .02 .04 ] 0 setdash
.48085 .61803 m
.49955 .51636 L
.53984 .33607 L
.57861 .20032 L
.59832 .1455 L
.61984 .09659 L
.64043 .06046 L
.65954 .03626 L
.67011 .02675 L
.6753 .02309 L
.68007 .0203 L
.68433 .01829 L
.689 .0166 L
.69144 .01593 L
.69371 .01544 L
.6961 .01506 L
.69745 .0149 L
.69867 .0148 L
.69985 .01474 L
.70096 .01472 L
.70217 .01472 L
.70348 .01477 L
.70478 .01486 L
.70543 .01492 L
.70614 .01499 L
.70859 .01536 L
.71088 .01583 L
.71334 .01648 L
.71776 .01802 L
.72335 .02067 L
.72864 .02387 L
.73852 .03171 L
.74824 .04177 L
.75853 .05496 L
.77699 .08513 L
.79656 .12631 L
.81487 .17338 L
.8537 .30048 L
.89497 .47634 L
s
.89497 .47634 m
.92192 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`004`3oool00`3ookl0M000000000020000000307Boo`3oool0oooo03/0oooo00<0_g@00000
00000000100000000`18W=l0oooo0?ooo`0j0?ooo`060;md00000000000000000000B9`0gooo>@3o
ool00`000000oooo0?ooo`0m0?ooo`060;md00000000000000000000B9`0gooo>P3oool00`2oM000
0000000000040000000304RLg`3oool0oooo03T0oooo00<0oono07@0000000000P0000000`1d_ol0
oooo0?ooo`0<0?ooo`004`3oool00`3ogi`0B00004RLg`020?ooo`030;md0018W=l0oooo03`0oooo
00<0gia80000000000000`0000000`00B9`0gooo0?ooo`0k0?ooo`030?oOW0180000M;oo03/0oooo
00<000000?ooo`3oool0?P3oool00`3ogi`0B00007Boo`0m0?ooo`030=nLB0000000000000<00000
00<004RL0=ooo`3oool0>@3oool00`3ogi`0B00004RLg`020?ooo`030;md0018W=l0oooo00`0oooo
000H0?ooo`030?oo_`1d04P0W=oo03d0oooo00<0gia804RLg`3oool00P3oool00`3OW7@0W=oo0?oo
o`0k0?ooo`030?oOW0180000M;oo03/0oooo00<000000?ooo`3oool0?P3oool00`3ogi`0B00007Bo
o`0n0?ooo`030=nLB018W=l0oooo0080oooo00<0giad09cOo`3oool0?P3oool00`3ookl0M01809cO
o`0<0?ooo`00603oool00`3ookl0M00007Boo`0n0?ooo`030=nLB01d_ol0oooo03l0oooo00<0omnL
04P0001d_ol0>`3oool00`000000oooo0?ooo`0n0?ooo`030?oOW0180000M;oo03l0oooo00<0gia8
07Boo`3oool0@P3oool00`3ookl0M00007Boo`0<0?ooo`001`3oool00`3ooml0W4P0000000070000
000304RLg`3oool0oooo00@0oooo00<0okmd0000001d_ol0;P3oool00`3ooml0W4P0000000070000
000304RLg`3oool0oooo00@0oooo00<0_gA809cOo`3oool0;`3oool00`3ooml0W4P0000000070000
000304RLg`3oool0oooo0080oooo00<0omnL04P0001d_ol0>`3oool00`000000oooo0?ooo`0n0?oo
o`030?oOW0180000M;oo0400oooo00<0_gA809cOo`3oool0@@3oool00`3o_g@0000007Boo`0<0?oo
o`005`3oool0103ogi`0B0000000B02Lgolo0?ooo`030?oog`2LB7@0_ooo03h0oooo00<0omnL04P0
001d_ol0>`3oool00`000000oooo0?ooo`0n0?ooo`030?oOW0180000M;oo0400oooo00<0oooO09a8
M02oool0@03oool0103ogi`0B0000000B02Lgol<0?ooo`005@3oool01@3o_g@000000000000004P0
W=oo0440oooo00<0okmd001d_`3oool0?@3oool00`3ogi`0B00007Boo`0k0?ooo`800000?`3oool0
0`3ogi`0B00007Boo`110?ooo`030?noM000M;l0oooo03d0oooo00D0okmd000000000000001809cO
o`0=0?ooo`005`3oool00`2oM00007Bo0?ooo`110?ooo`030?oo_`1d04P0W=oo0240oooo2`00000A
0?ooo`030?oOW0180000M;oo03/0oooo00<000000?ooo`3oool0?P3oool00`3ogi`0B00007Boo`0;
0?ooo`/00000:`3oool00`3ookl0M01809cOo`0o0?ooo`030;md0000M;l0oooo00d0oooo000H0?oo
o`030;md0018W=l0oooo0400oooo00<0oono07@00018W=l07`3oool2000000/0oooo0`00000>0?oo
o`030?oOW0180000M;oo03/0oooo00<000000?ooo`3oool0?P3oool00`3ogi`0B00007Boo`0:0?oo
o`030000003oool0oooo00T0oooo0`00000X0?ooo`030?oo_`1d0000B9cO0400oooo00<0_g@004RL
g`3oool0303oool001<0oooo00<0oooO09bL_`3oool00P3oool00`3OW4P0001d0;ooo`0k0?ooo`03
0?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l07@3oool200000100oooo0P00000<0?ooo`03
0?oOW0180000M;oo03/0oooo00<000000?ooo`3oool0?P3oool00`3ogi`0B00007Boo`080?ooo`80
00003P3oool200000280oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0B00004RLg`0k0?ooo`03
0?oog`2LW;l0oooo0080oooo00<0gia80000M02oool0303oool001@0oooo00H0okmd04RLg`3oool0
oono07@00000M;ll0?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02ooolK0?ooo`800000
503oool00`000000oooo0?ooo`070?ooo`050?oog`2LB00000000000001d_ol0>`3oool00`000000
oooo0?ooo`0l0?ooo`050?oog`2LB00000000000001d_ol01P3oool200000180oooo0P00000P0?oo
o`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02oooll0?ooo`060?noM018W=l0oooo0?oo_`1d
000007Bo3@3oool001D0oooo00D0gia8000000000000001d0;ooo`0m0?ooo`060?oog`2LB0000000
0000000007@0_ooo6`3oool00`000000oooo0?ooo`0E0?ooo`8000002@3oool00`3o_g@0000007Bo
o`0k0?ooo`030000003oool0oooo03h0oooo00<0okmd0000001d_ol01@3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0oooo01h0oooo00H0oooO09a80000000000000000M02oooln0?ooo`05
0=nLB000000000000000M02oool03@3oool007H0oooo0P00000J0?ooo`030000003oool0oooo04@0
oooo00<000000?ooo`3oool0A03oool2000001P0oooo0P00001d0?ooo`00M@3oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool0@`3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo0740oooo001d0?ooo`030000003oool0oooo01d0oooo00<0
00000?ooo`3oool0@P3oool2000004<0oooo00<000000?ooo`3oool0703oool00`000000oooo0?oo
o`1`0?ooo`00L`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0440oooo00<00000
0?ooo`3oool0@@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo06l0oooo001b0?oo
o`030000003oool0oooo0240oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`100?oo
o`030000003oool0oooo0200oooo00<000000?ooo`3oool0KP3oool00740oooo00<000000?ooo`3o
ool08`3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3o
ool08P3oool00`000000oooo0?ooo`1]0?ooo`00L@3oool00`000000oooo0?ooo`0T0?ooo`030000
003oool0oooo03h0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0T0?ooo`030000
003oool0oooo06`0oooo001`0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0?P3o
ool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0J`3o
ool006l0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0
oooo03`0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`1Z0?ooo`00KP3oool00`00
0000oooo0?ooo`0Y0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0>`3oool00`00
0000oooo0?ooo`0Y0?ooo`800000J`3oool000L0ooooo`00002U000000D0oooo000A0?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00P0oooo00<000000?ooo`0000003@3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00X0oooo00@000000?ooo`00000000002P3oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0=0?ooo`00
4@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`080?ooo`030000003oool0000000d0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0:0?ooo`040000003oool00000000000X0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool03@3oool00140oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo02P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0i0?ooo`030000
003oool0oooo03P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0X0?ooo`800000
5P3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo00d0oooo001Z0?ooo`030000003o
ool0oooo0340oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`0g0?ooo`030000003o
ool0oooo0300oooo00<000000?ooo`000000I`3oool006X0oooo00<000000?ooo`3oool0<P3oool0
0`000000oooo0?ooo`0g0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool0<`3oool0
0`000000oooo0?ooo`1U0?ooo`00J@3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo
03L0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo
06@0oooo001X0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool0=P3oool00`000000
oooo0?ooo`0e0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool0H`3oool006P0oooo
00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0e0?ooo`800000=@3oool00`000000oooo
0?ooo`0h0?ooo`030000003oool0oooo0680oooo001W0?ooo`030000003oool0oooo03L0oooo00<0
00000?ooo`3oool0=@3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo03P0oooo00<0
00000?ooo`3oool0HP3oool006H0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0d
0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`1Q
0?ooo`00IP3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo03@0oooo00<000000?oo
o`3oool0<P3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo0600oooo001U0?ooo`03
0000003oool0oooo03/0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0b0?ooo`80
0000?@3oool00`000000oooo0?ooo`1P0?ooo`00I03oool00`000000oooo0?ooo`0m0?ooo`030000
003oool0oooo0380oooo00<000000?ooo`3oool0<@3oool2000003l0oooo00<000000?ooo`3oool0
G`3oool006@0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0b0?ooo`030000003o
ool0oooo0300oooo00<000000?ooo`000000?`3oool00`000000oooo0?ooo`1O0?ooo`00H`3oool0
0`000000oooo0?ooo`0o0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0<03oool2
00000440oooo00<000000?ooo`3oool0GP3oool006<0oooo00<000000?ooo`3oool0@03oool00`00
0000oooo0?ooo`0`0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`000000@@3oool00`00
0000oooo0?ooo`1N0?ooo`00HP3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo0300
oooo0P00000`0?ooo`800000@`3oool00`000000oooo0?ooo`1M0?ooo`00HP3oool00`000000oooo
0?ooo`120?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0;P3oool00`000000oooo
000000120?ooo`800000G`3oool00640oooo00<000000?ooo`3oool0A03oool00`000000oooo0?oo
o`0^0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`000000@`3oool00`000000oooo0000
001N0?ooo`00H03oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo02h0oooo00<00000
0?ooo`3oool0;@3oool00`000000oooo0?ooo`140?ooo`800000GP3oool00600oooo00<000000?oo
o`3oool0AP3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo02`0oooo00<000000?oo
o`3oool0A@3oool00`000000oooo0000001M0?ooo`00G`3oool00`000000oooo0?ooo`170?ooo`03
0000003oool0oooo02d0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`150?ooo`03
0000003oool0000005d0oooo001O0?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3oool0
;03oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`000000
G03oool005h0oooo00<000000?ooo`3oool0B@3oool00`000000oooo0?ooo`0/0?ooo`030000003o
ool0oooo02/0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0000001L0?ooo`00GP3oool0
0`000000oooo0?ooo`1:0?ooo`030000003oool0oooo02/0oooo0P00000[0?ooo`030000003oool0
oooo04T0oooo00<000000?ooo`000000F`3oool005d0oooo00<000000?ooo`3oool0C03oool00`00
0000oooo0?ooo`0Z0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0B@3oool00`00
0000oooo0000001K0?ooo`00G@3oool00`000000oooo0?ooo`1<0?ooo`030000003oool0oooo02X0
oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`1=0?ooo`030000003oool0oooo05P0
oooo001L0?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0:@3oool00`000000oooo
0?ooo`0Y0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0F03oool005`0oooo00<0
00000?ooo`3oool0CP3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo02P0oooo00<0
00000?ooo`3oool0C`3oool00`000000oooo0?ooo`1G0?ooo`00F`3oool00`000000oooo0?ooo`1@
0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`1?
0?ooo`030000003oool0oooo05L0oooo001K0?ooo`030000003oool0oooo0500oooo00<000000?oo
o`3oool0:03oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo0540oooo00<000000?oo
o`3oool0EP3oool005X0oooo00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`0W0?ooo`03
0000003oool0oooo02L0oooo00<000000?ooo`3oool0D@3oool00`000000oooo0?ooo`1F0?ooo`00
FP3oool00`000000oooo0?ooo`1C0?ooo`030000003oool0oooo02H0oooo0P00000W0?ooo`030000
003oool0oooo05<0oooo00<000000?ooo`3oool0E@3oool005T0oooo00<000000?ooo`3oool0E03o
ool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0E03o
ool00`000000oooo0?ooo`1E0?ooo`00F@3oool00`000000oooo0?ooo`1E0?ooo`030000003oool0
oooo02D0oooo00<000000?ooo`3oool09@3oool010000000oooo0?ooo`00001D0?ooo`030000003o
ool0oooo05@0oooo001H0?ooo`030000003oool0oooo05H0oooo00<000000?ooo`3oool0603oool0
103ooml0W4P00018W03Oool90?ooo`030000003oool0oooo02@0oooo00@000000?ooo`3oool00000
E@3oool00`000000oooo0?ooo`1D0?ooo`00F03oool00`000000oooo0?ooo`1G0?ooo`030000003o
ool0oooo01L0oooo00@0oooO09a80000B9`0gooo2@3oool00`000000oooo0?ooo`0T0?ooo`040000
003oool0oooo000005H0oooo00<000000?ooo`3oool0D`3oool005L0oooo00<000000?ooo`3oool0
F03oool00`000000oooo0?ooo`0G0?ooo`040?oog`2LB00004RL0=ooo`T0oooo00<000000?ooo`3o
ool08`3oool010000000oooo0?ooo`00001G0?ooo`030000003oool0oooo05<0oooo001G0?ooo`03
0000003oool0oooo05T0oooo00<000000?ooo`3oool04P3oool00`2oM00000000000000500000003
07Boo`3oool0oooo00H0oooo00<000000?ooo`3oool08`3oool010000000oooo0?ooo`00001H0?oo
o`030000003oool0oooo0580oooo001F0?ooo`030000003oool0oooo05/0oooo00<000000?ooo`3o
ool04@3oool0203ogi`0M9cO0?ooo`3oool0oooO09a80000B9`0gooo2@3oool00`000000oooo0?oo
o`0R0?ooo`050000003oool0oooo0?ooo`000000F03oool00`000000oooo0?ooo`1B0?ooo`00EP3o
ool00`000000oooo0?ooo`1K0?ooo`030000003oool0oooo0180oooo00L0gia807Boo`3oool0oooO
09a80000B9`0gooo00T0oooo0`00000R0?ooo`040000003oool0oooo000005L0oooo00@000000?oo
o`3oool00000D`3oool005D0oooo00<000000?ooo`3oool0G@3oool00`000000oooo0?ooo`0A0?oo
o`070?oog`2LB7@0_ooo0?oog`2LB00004RL0=ooo`090?ooo`030000003oool0oooo0240oooo00D0
00000?ooo`3oool0oooo0000001G0?ooo`040000003oool0oooo000005<0oooo001E0?ooo`030000
003oool0oooo05d0oooo00<000000?ooo`3oool04P3oool01P3ogi`0B7Bo0?oog`2LB00004RL0=oo
o`T0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`1J0?ooo`040000003oool0oooo
00000580oooo001D0?ooo`030000003oool0oooo05l0oooo00<000000?ooo`3oool04P3oool01@3O
W4P0M;oO09a80000B9`0gooo00T0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`1K
0?ooo`040000003oool0oooo00000580oooo001D0?ooo`030000003oool0oooo05l0oooo00<00000
0?ooo`3oool04P3oool01@3ooml0W4Q804Q80000B9`0gooo00T0oooo00<000000?ooo`3oool0803o
ool00`000000oooo0?ooo`1K0?ooo`050000003oool0oooo0?ooo`000000D@3oool005@0oooo00<0
00000?ooo`3oool0H03oool00`000000oooo0?ooo`0B0?ooo`040?oOW018000004RL0=ooo`T0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`1M0?ooo`040000003oool0oooo00000540
oooo001C0?ooo`030000003oool0oooo0680oooo00<000000?ooo`3oool04P3oool00`3OW4P004RL
0=ooo`090?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0G@3oool01@000000oooo
0?ooo`3oool000000500oooo001C0?ooo`030000003oool0oooo0680oooo00<000000?ooo`3oool0
7P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo05l0oooo00@000000?ooo`3oool0
0000D03oool00580oooo00<000000?ooo`3oool0I03oool00`000000oooo0?ooo`0M0?ooo`800000
7P3oool00`000000oooo0?ooo`1P0?ooo`050000003oool0oooo0?ooo`000000C`3oool00580oooo
00<000000?ooo`3oool0I03oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01d0oooo
00<000000?ooo`3oool0I03oool00`000000oooo0?ooo`1=0?ooo`00DP3oool00`000000oooo0?oo
o`1U0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0703oool00`000000oooo0?oo
o`1V0?ooo`030000003oool0oooo04`0oooo001A0?ooo`030000003oool0oooo06L0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo06L0oooo00<00000
0?ooo`3oool0C03oool00540oooo00<000000?ooo`3oool0I`3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo01/0oooo00<000000?ooo`3oool0I`3oool00`000000oooo0?ooo`1<0?oo
o`00D03oool00`000000oooo0?ooo`1Y0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`1Y0?ooo`030000003oool0oooo04/0oooo001@0?ooo`030000
003oool0oooo06T0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo06X0oooo00<000000?ooo`3oool0B`3oool004l0oooo00<000000?ooo`3oool0J`3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0J`3o
ool00`000000oooo0?ooo`1:0?ooo`00C`3oool00`000000oooo0?ooo`1/0?ooo`030000003oool0
oooo01P0oooo0P00000I0?ooo`030000003oool0oooo06`0oooo00<000000?ooo`3oool0BP3oool0
04l0oooo00<000000?ooo`3oool0K03oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`1U0?ooo`030000003oool0oooo
04X0oooo001>0?ooo`030000003oool0oooo06h0oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0IP3oool00`000000
oooo0?ooo`190?ooo`00CP3oool00`000000oooo0?ooo`1^0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo06L0oooo
00<000000?ooo`3oool0B@3oool004d0oooo00<000000?ooo`3oool0L03oool00`000000oooo0?oo
o`0F0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`1X0?ooo`030000003oool0oooo04P0oooo001=0?ooo`030000003oool0oooo0740oooo00<00000
0?ooo`3oool05@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool0J03oool00`000000oooo0?ooo`180?ooo`00C@3oool00`000000oooo0?ooo`1a0?oo
o`030000003oool0oooo01D0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo06T0oooo00<000000?ooo`3oool0A`3oool004`0oooo00<000000?ooo`3o
ool0L`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`1Z0?ooo`030000003oool0oooo04L0oooo001<0?ooo`030000
003oool0oooo07@0oooo00<000000?ooo`3oool04`3oool2000001@0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`1U0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
A`3oool004/0oooo00<000000?ooo`3oool0MP3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo0180oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`1U0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool0AP3oool004/0oooo00<000000?ooo`3oool0MP3oool0
0`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0L@3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo04H0oooo001;0?ooo`030000003oool0oooo
07L0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
07<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`150?ooo`00BP3oool00`000000
oooo0?ooo`1i0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`1d0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0A@3oool004X0oooo
00<000000?ooo`3oool0NP3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00l0oooo
00<000000?ooo`3oool0M03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo04@0oooo
00190?ooo`030000003oool0oooo07/0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo07D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`140?ooo`00B@3oool00`000000oooo0?ooo`1l0?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0A03oool004T0oooo00<000000?ooo`3oool0O@3oool00`000000oooo0?ooo`0=0?oo
o`8000003@3oool00`000000oooo0?ooo`1h0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool0@`3oool004P0oooo00<000000?ooo`3oool0O`3oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo00/0oooo00<000000?ooo`3oool0P03oool00`000000oooo0?ooo`130?ooo`00
B03oool00`000000oooo0?ooo`200?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`220?ooo`030000003oool0oooo0480oooo00170?ooo`030000003o
ool0oooo07h0oooo00<0oooO09a8000000000P0000000`1d_ol0oooo0?ooo`090?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool0P`3oool00`000000oooo0?ooo`120?ooo`00A`3oool0
0`000000oooo0?ooo`1n0?ooo`030?oog`2LB7@0_ooo0080oooo0P00000:0?ooo`030000003oool0
oooo00P0oooo00<000000?ooo`3oool0Q03oool00`000000oooo0?ooo`120?ooo`00AP3oool00`00
0000oooo0?ooo`240?ooo`040?oog`2LB7@00000000000P0oooo00<000000?ooo`3oool01P3oool2
000008P0oooo00<000000?ooo`3oool0@@3oool004H0oooo00<000000?ooo`3oool0Q@3oool0102o
M000M;oo0?ooo`0000070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0R03oool0
0`000000oooo0?ooo`110?ooo`00AP3oool00`000000oooo0?ooo`240?ooo`030?oog`2LB000B9cO
0080oooo0`0000040?ooo`030000003oool0oooo0080oooo0`00002<0?ooo`030000003oool0oooo
0400oooo00150?ooo`030000003oool0oooo08D0oooo00<0gia80000001d_ol01@3oool9000008l0
oooo00<000000?ooo`3oool0@03oool004D0oooo00<000000?ooo`3oool0P@3oool01`3Ogml0oooo
0?oo_`1d000000000000M02oool02@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
07L0oooo00<000000?ooo`3oool0?`3oool004@0oooo00<000000?ooo`3oool0PP3oool01P3OW4P0
0000000000000000001809cOo`X0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`1g
0?ooo`030000003oool0oooo03l0oooo00140?ooo`030000003oool0oooo0880oooo00@0oono07@0
00000000B9cO303oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo07P0oooo00<00000
0?ooo`3oool0?`3oool004@0oooo00<000000?ooo`3oool0P`3oool00`2oM000M;oo0?ooo`0<0?oo
o`030000003oool0oooo01L0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0?ooo`0n0?oo
o`00@`3oool00`000000oooo0?ooo`240?ooo`060?oOW018000000000000000004P0W=oo2@3oool0
0`000000oooo0?ooo`0G0?ooo`030000003oool0oooo07T0oooo00<000000?ooo`3oool0?P3oool0
04<0oooo00<000000?ooo`3oool0Q@3oool01@2oM0000000000000000000B9cO00T0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`1i0?ooo`030000003oool0oooo03h0oooo00130?oo
o`030000003oool0oooo09<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`1k0?oo
o`030000003oool0oooo03d0oooo00120?ooo`030000003oool0oooo09@0oooo0P00000G0?ooo`03
0000003oool0oooo07<0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0m0?ooo`00
@P3oool00`000000oooo0?ooo`2D0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0
L`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo03d0oooo00110?ooo`030000003o
ool0oooo09D0oooo00<000000?ooo`3oool0S@3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo03`0oooo00110?ooo`030000003oool0oooo09D0oooo00<000000?ooo`3oool0S@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo03`0oooo00110?ooo`030000003oool0oooo
09D0oooo00<000000?ooo`3oool0S@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
03/0oooo00100?ooo`030000003oool0oooo09H0oooo00<000000?ooo`3oool0SP3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo03/0oooo00100?ooo`030000003oool0oooo09H0oooo
00<000000?ooo`3oool0SP3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo03/0oooo
00100?ooo`030000003oool0oooo09H0oooo00<000000?ooo`3oool0S`3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo03X0oooo000o0?ooo`030000003oool0oooo09L0oooo0P00002@
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0>P3oool003l0oooo00<000000?oo
o`3oool0U`3oool00`000000oooo0?ooo`2G0?ooo`030000003oool0oooo03X0oooo000n0?ooo`03
0000003oool0oooo09P0oooo00<000000?ooo`3oool0V03oool00`000000oooo0?ooo`0i0?ooo`00
?P3oool00`000000oooo0?ooo`2H0?ooo`030000003oool0oooo09P0oooo00<000000?ooo`3oool0
>@3oool003h0oooo00<000000?ooo`3oool0V03oool00`000000oooo0?ooo`2H0?ooo`030000003o
ool0oooo03T0oooo000m0?ooo`030000003oool0oooo09T0oooo00<000000?ooo`3oool0V@3oool0
0`000000oooo0?ooo`0h0?ooo`00?@3oool00`000000oooo0?ooo`2I0?ooo`030000003oool0oooo
09T0oooo00<000000?ooo`3oool0>03oool003d0oooo00<000000?ooo`3oool0V@3oool00`000000
oooo0?ooo`2I0?ooo`030000003oool0oooo03P0oooo000l0?ooo`030000003oool0oooo09X0oooo
0P00002K0?ooo`030000003oool0oooo03L0oooo000l0?ooo`030000003oool0oooo09X0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`260?ooo`030000003oool0oooo03L0oooo000k
0?ooo`030000003oool0oooo09/0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`26
0?ooo`030000003oool0oooo03L0oooo000k0?ooo`030000003oool0oooo09/0oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`270?ooo`030000003oool0oooo03H0oooo000k0?ooo`03
0000003oool0oooo09/0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`280?ooo`03
0000003oool0oooo03H0oooo000j0?ooo`030000003oool0oooo09`0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`290?ooo`030000003oool0oooo03D0oooo000j0?ooo`030000003o
ool0oooo09`0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`290?ooo`030000003o
ool0oooo03D0oooo000j0?ooo`030000003oool0oooo09`0oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`290?ooo`030000003oool0oooo03D0oooo000i0?ooo`030000003oool0oooo
09d0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`210?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool0=03oool003T0oooo00<000000?ooo`3oool0W@3oool200000100
oooo00<000000?ooo`3oool0P@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo03@0
oooo000i0?ooo`030000003oool0oooo09d0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`210?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0=03oool003P0oooo00<0
00000?ooo`3oool0WP3oool00`000000oooo0?ooo`2D0?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool0<`3oool003P0oooo00<000000?ooo`3oool0S`3oool01@3ogi`0B00000000000
0000M;oo00X0oooo00<000000?ooo`3oool0U03oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo03<0oooo000g0?ooo`030000003oool0oooo08l0oooo00L0omnL04P00018W=l0oooo0?oo
_`1d0000M;oo00T0oooo00<000000?ooo`3oool0U03oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo03<0oooo000g0?ooo`030000003oool0oooo08h0oooo00<0oooO09a80000M;l00`3o
ool00`3o_g@004RL0=ooo`080?ooo`030000003oool0oooo09@0oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`0b0?ooo`00=`3oool00`000000oooo0?ooo`2>0?ooo`030?oOW0180000
M;oo00<0oooo00<0okmd0000B02Lgol0203oool00`000000oooo0?ooo`2E0?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool0<P3oool003H0oooo00<000000?ooo`3oool0S`3oool00`3o
gi`0B00007Boo`030?ooo`030?noM00004P0W=oo00P0oooo00<000000?ooo`3oool0U@3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo0380oooo000f0?ooo`030000003oool0oooo08l0
oooo00<0omnL04P0001d_ol00`3oool00`2oM000001809cOo`080?ooo`<00000X03oool00`000000
oooo0?ooo`0a0?ooo`00=P3oool00`000000oooo0?ooo`2?0?ooo`080?oo_`1d0000000004RLg`3o
ool0okmd00000000M;l90?ooo`030000003oool0oooo0:00oooo00<000000?ooo`3oool0<@3oool0
03D0oooo00<000000?ooo`3oool0T@3oool00`2oM00004RL0;nLB002000000030018W03Oool0oooo
00P0oooo00<000000?ooo`3oool0X@3oool00`000000oooo0?ooo`0`0?ooo`00=@3oool00`000000
oooo0?ooo`2A0?ooo`030?oOW0180000M;oo00d0oooo00<000000?ooo`3oool0X@3oool00`000000
oooo0?ooo`0`0?ooo`00=03oool00`000000oooo0?ooo`2C0?ooo`030=nLB00004P0W=oo00`0oooo
00<000000?ooo`3oool0X@3oool00`000000oooo0?ooo`0`0?ooo`00=03oool00`000000oooo0?oo
o`2D0?ooo`030?noM000000007Bo00/0oooo00<000000?ooo`3oool0XP3oool00`000000oooo0?oo
o`0_0?ooo`00=03oool00`000000oooo0?ooo`2E0?ooo`050?oog`2LB00000000000M02oool0203o
ool00`000000oooo0?ooo`2R0?ooo`030000003oool0oooo02l0oooo000c0?ooo`030000003oool0
oooo0:<0oooo00<000000?ooo`3oool0XP3oool00`000000oooo0?ooo`0_0?ooo`00<`3oool00`00
0000oooo0?ooo`2S0?ooo`800000Y03oool00`000000oooo0?ooo`0^0?ooo`00<`3oool00`000000
oooo0?ooo`2S0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0U@3oool00`000000
oooo0?ooo`0^0?ooo`00<P3oool00`000000oooo0?ooo`2T0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0U@3oool00`000000oooo0?ooo`0^0?ooo`00<P3oool00`000000oooo0?oo
o`2T0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0U`3oool00`000000oooo0?oo
o`0]0?ooo`00<P3oool00`000000oooo0?ooo`2T0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool0U`3oool00`000000oooo0?ooo`0]0?ooo`00<@3oool00`000000oooo0?ooo`2U0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool0U`3oool00`000000oooo0?ooo`0]0?oo
o`00<@3oool00`000000oooo0?ooo`2U0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool0V03oool00`000000oooo0?ooo`0/0?ooo`00<@3oool00`000000oooo0?ooo`2U0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool0V@3oool00`000000oooo0?ooo`0/0?ooo`00<03o
ool00`000000oooo0?ooo`2V0?ooo`8000002P3oool00`000000oooo0?ooo`2=0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool0;03oool00300oooo00<000000?ooo`3oool0YP3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo08d0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0[0?ooo`00<03oool00`000000oooo0?ooo`2V0?ooo`030000003oool0oooo
09T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0[0?ooo`00;`3oool00`000000
oooo0?ooo`2W0?ooo`030000003oool0oooo09X0oooo00<000000?ooo`3oool02@3oool00`000000
oooo0?ooo`0[0?ooo`00;`3oool00`000000oooo0?ooo`2W0?ooo`030000003oool0oooo09X0oooo
00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0[0?ooo`00;`3oool00`000000oooo0?oo
o`2W0?ooo`030000003oool0oooo09X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`0Z0?ooo`00;P3oool00`000000oooo0?ooo`2X0?ooo`030000003oool0oooo09X0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0Z0?ooo`00;P3oool00`000000oooo0?ooo`2X0?oo
o`030000003oool0oooo09/0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0Z0?oo
o`00;P3oool00`000000oooo0?ooo`2X0?ooo`800000W03oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo02T0oooo000^0?ooo`030000003oool0oooo0:P0oooo00<000000?ooo`3oool0
Z03oool00`000000oooo0?ooo`0Y0?ooo`00;@3oool00`000000oooo0?ooo`2Y0?ooo`030000003o
ool0oooo0:P0oooo00<000000?ooo`3oool0:@3oool002d0oooo00<000000?ooo`3oool0Z@3oool0
0`000000oooo0?ooo`2Y0?ooo`030000003oool0oooo02P0oooo000]0?ooo`030000003oool0oooo
0:T0oooo00<000000?ooo`3oool0Z@3oool00`000000oooo0?ooo`0X0?ooo`00;03oool00`000000
oooo0?ooo`2Z0?ooo`030000003oool0oooo0:T0oooo00<000000?ooo`3oool0:03oool002`0oooo
00<000000?ooo`3oool0ZP3oool00`000000oooo0?ooo`2Z0?ooo`030000003oool0oooo02L0oooo
000/0?ooo`030000003oool0oooo0:X0oooo00<000000?ooo`3oool0ZP3oool00`000000oooo0?oo
o`0W0?ooo`00:`3oool00`000000oooo0?ooo`2[0?ooo`800000Z`3oool00`000000oooo0?ooo`0W
0?ooo`00:`3oool00`000000oooo0?ooo`2[0?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool0XP3oool00`000000oooo0?ooo`0V0?ooo`00:`3oool00`000000oooo0?ooo`2[0?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool0XP3oool00`000000oooo0?ooo`0V0?ooo`00
:P3oool00`000000oooo0?ooo`2N0?ooo`030=nLB018W=l0oooo00/0oooo00<000000?ooo`3oool0
1P3oool00`000000oooo0?ooo`2R0?ooo`030000003oool0oooo02H0oooo000Z0?ooo`030000003o
ool0oooo09h0oooo00<0omnL04Q8W03Oool02`3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo0:<0oooo00<000000?ooo`3oool09P3oool002X0oooo00<000000?ooo`3oool0WP3oool0
0`3ooml0W4Q809cOo`0;0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0Y03oool0
0`000000oooo0?ooo`0U0?ooo`00:@3oool00`000000oooo0?ooo`2P0?ooo`030=nLB018W=l0oooo
00X0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`2T0?ooo`030000003oool0oooo
02D0oooo000Y0?ooo`030000003oool0oooo0:00oooo00<0omnL04Q8W03Oool02P3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo0:@0oooo00<000000?ooo`3oool09@3oool002T0oooo
00<000000?ooo`3oool0X03oool00`3ooml0W4Q809cOo`0:0?ooo`<00000103oool00`000000oooo
0?ooo`2H0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0903oool002P0oooo00<0
00000?ooo`3oool0XP3oool00`3OW4P007Bo0?ooo`090?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool0V03oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo02@0oooo000X
0?ooo`030000003oool0oooo0:80oooo00<0oono07@0M02oool02@3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo09P0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0T
0?ooo`00:03oool00`000000oooo0?ooo`2M0?ooo`030?oOg`3Oool0oooo00<0oooo00<0_g@007Bo
o`3oool0203oool00`000000oooo0?ooo`2P0?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool08`3oool002L0oooo00<000000?ooo`3oool0WP3oool00`3ookl0M4RL0=ooo`030?ooo`03
0?noM000M;l0oooo00P0oooo00<000000?ooo`3oool0X03oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo02<0oooo000W0?ooo`030000003oool0oooo09l0oooo00<0_g@0000000000000
0`0000000`0007@0_ooo0?ooo`070?ooo`030000003oool0oooo0:00oooo00<000000?ooo`3oool0
2`3oool00`000000oooo0?ooo`0S0?ooo`009`3oool00`000000oooo0?ooo`2O0?ooo`030?noM000
0000000000@0000000<0M;oo0?ooo`3oool01P3oool00`000000oooo0?ooo`2P0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool08P3oool002H0oooo00<000000?ooo`3oool0/03oool0
0`000000oooo0?ooo`2Q0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool08P3oool0
02H0oooo00<000000?ooo`3oool0/03oool200000:80oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`0R0?ooo`009P3oool00`000000oooo0?ooo`2`0?ooo`030000003oool0oooo0;00
oooo00<000000?ooo`3oool08@3oool002D0oooo00<000000?ooo`3oool0/@3oool00`000000oooo
0?ooo`2`0?ooo`030000003oool0oooo0240oooo000U0?ooo`030000003oool0oooo0;40oooo00<0
00000?ooo`3oool0/03oool00`000000oooo0?ooo`0Q0?ooo`009@3oool00`000000oooo0?ooo`2a
0?ooo`030000003oool0oooo0;00oooo00<000000?ooo`3oool08@3oool002D0oooo00<000000?oo
o`3oool0/@3oool00`000000oooo0?ooo`2a0?ooo`030000003oool0oooo0200oooo000T0?ooo`03
0000003oool0oooo0;80oooo00<000000?ooo`3oool0/@3oool00`000000oooo0?ooo`0P0?ooo`00
903oool00`000000oooo0?ooo`2b0?ooo`030000003oool0oooo0;40oooo00<000000?ooo`3oool0
803oool002@0oooo00<000000?ooo`3oool0/P3oool200000:<0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0O0?ooo`008`3oool00`000000oooo0?ooo`2c0?ooo`030000003oool0
oooo0:80oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0O0?ooo`008`3oool00`00
0000oooo0?ooo`2c0?ooo`040000003oool0oooo00000:40oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`0O0?ooo`008`3oool00`000000oooo0?ooo`2c0?ooo`040000003oool0oooo
00000:80oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0N0?ooo`008P3oool00`00
0000oooo0?ooo`2d0?ooo`030000003oool000000:<0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`0N0?ooo`008P3oool00`000000oooo0?ooo`2d0?ooo`030000003oool000000:<0
oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0N0?ooo`008P3oool00`000000oooo
0?ooo`2d0?ooo`030000003oool000000:<0oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`0M0?ooo`008@3oool00`000000oooo0?ooo`2e0?ooo`030000003oool000000:@0oooo00<0
00000?ooo`3oool03@3oool00`000000oooo0?ooo`0M0?ooo`008@3oool00`000000oooo0?ooo`2e
0?ooo`800000Y@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01d0oooo000Q0?oo
o`030000003oool0oooo0;D0oooo0P00002e0?ooo`030000003oool0oooo01d0oooo000P0?ooo`03
0000003oool0oooo0;H0oooo0P00002f0?ooo`030000003oool0oooo01`0oooo000P0?ooo`030000
003oool0oooo0;H0oooo00<000000?ooo`3oool0]@3oool00`000000oooo0?ooo`0L0?ooo`00803o
ool00`000000oooo0?ooo`2f0?ooo`030000003oool0oooo0;D0oooo00<000000?ooo`3oool0703o
ool00200oooo00<000000?ooo`3oool0]P3oool00`000000oooo0?ooo`2f0?ooo`030000003oool0
oooo01/0oooo000O0?ooo`030000003oool0oooo0;L0oooo00<000000?ooo`3oool0]P3oool00`00
0000oooo0?ooo`0K0?ooo`007`3oool00`000000oooo0?ooo`2g0?ooo`030000003oool0oooo0;H0
oooo00<000000?ooo`3oool06`3oool001l0oooo00<000000?ooo`3oool0]`3oool200000;L0oooo
00<000000?ooo`3oool06`3oool001l0oooo00<000000?ooo`3oool0]`3oool00`000000oooo0?oo
o`2g0?ooo`030000003oool0oooo01X0oooo000N0?ooo`030000003oool0oooo0;P0oooo00<00000
0?ooo`3oool0]`3oool00`000000oooo0?ooo`0J0?ooo`007P3oool00`000000oooo0?ooo`2Y0?oo
o`050=nLB0000000000000000018W=l02P3oool00`000000oooo0?ooo`2g0?ooo`030000003oool0
oooo01X0oooo000N0?ooo`030000003oool0oooo0:P0oooo00L0gia80000001d_ol0oooo0?oog`2L
B000B9cO00T0oooo00<000000?ooo`3oool0^03oool00`000000oooo0?ooo`0I0?ooo`007@3oool0
0`000000oooo0?ooo`2X0?ooo`030?oog`2LB00007Bo00<0oooo00<0okmd0018W03Oool0203oool0
0`000000oooo0?ooo`2h0?ooo`030000003oool0oooo01T0oooo000M0?ooo`030000003oool0oooo
0:T0oooo00<0_g@0001d_`3oool00P3oool00`2oM00004RL0=ooo`080?ooo`030000003oool0oooo
0;P0oooo00<000000?ooo`3oool06@3oool001d0oooo00<000000?ooo`3oool0Z@3oool01`3ogi`0
B01d0;ooo`3oool0_g@00000001d_ol02@3oool00`000000oooo0?ooo`2i0?ooo`030000003oool0
oooo01P0oooo000M0?ooo`030000003oool0oooo0:X0oooo00D0oono07A8B0180000000004RLg`0:
0?ooo`<00000^@3oool00`000000oooo0?ooo`0H0?ooo`00703oool00`000000oooo0?ooo`2[0?oo
o`040?oOW01800000000001d_`/0oooo00<000000?ooo`3oool0Y`3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo01P0oooo000L0?ooo`030000003oool0oooo0:X0oooo00H0omnL04P0
000007@0_ooo0?noM000M;l:0?ooo`030000003oool0oooo0:L0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`0H0?ooo`00703oool00`000000oooo0?ooo`2Z0?ooo`030;md000007@0
_ooo0080oooo00<0_g@004RLg`3oool01P3oool00`000000oooo0000002Y0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool05`3oool001`0oooo00<000000?ooo`3oool0ZP3oool00`2o
M00004RL0=ooo`020?ooo`030=nLB000M;l0oooo00H0oooo00<000000?ooo`000000ZP3oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo01L0oooo000K0?ooo`030000003oool0oooo0:/0
oooo00L0okmd0000001d_ol0oooo0?oo_`1d0000M;oo00H0oooo00@000000?ooo`3oool00000ZP3o
ool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo01L0oooo000K0?ooo`030000003oool0
oooo0:`0oooo00D0okmd000000000000000004RLg`070?ooo`040000003oool0oooo00000:X0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0F0?ooo`006`3oool00`000000oooo0?oo
o`2h0?ooo`040000003oool0oooo00000:X0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0F0?ooo`006P3oool00`000000oooo0?ooo`2i0?ooo`040000003oool0oooo00000:/0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0F0?ooo`006P3oool00`000000oooo0?oo
o`2h0?ooo`040000003oool0oooo0?ooo`800000ZP3oool00`000000oooo0?ooo`0?0?ooo`030000
003oool0oooo01H0oooo000J0?ooo`030000003oool0oooo0;P0oooo00D000000?ooo`3oool0oooo
0000002n0?ooo`030000003oool0oooo01D0oooo000J0?ooo`030000003oool0oooo0;P0oooo00D0
00000?ooo`3oool0oooo0000002n0?ooo`030000003oool0oooo01D0oooo000I0?ooo`030000003o
ool0oooo0;d0oooo00<000000?ooo`3oool0_03oool00`000000oooo0?ooo`0E0?ooo`006@3oool0
0`000000oooo0?ooo`2m0?ooo`030000003oool0oooo0;d0oooo00<000000?ooo`3oool0503oool0
01T0oooo00<000000?ooo`3oool0_@3oool00`000000oooo0?ooo`2m0?ooo`030000003oool0oooo
01@0oooo000H0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool0_@3oool00`000000
oooo0?ooo`0D0?ooo`00603oool00`000000oooo0?ooo`2n0?ooo`030000003oool0oooo0;d0oooo
00<000000?ooo`3oool0503oool001P0oooo00<000000?ooo`3oool0_P3oool200000;l0oooo00<0
00000?ooo`3oool04`3oool001P0oooo00<000000?ooo`3oool0_P3oool00`000000oooo0?ooo`2n
0?ooo`030000003oool0oooo01<0oooo000G0?ooo`030000003oool0oooo0;l0oooo00<000000?oo
o`3oool0_P3oool00`000000oooo0?ooo`0C0?ooo`005`3oool00`000000oooo0?ooo`2o0?ooo`03
0000003oool0oooo0;l0oooo00<000000?ooo`3oool04P3oool001L0oooo00<000000?ooo`3oool0
_`3oool00`000000oooo0?ooo`2o0?ooo`030000003oool0oooo0180oooo000G0?ooo`030000003o
ool0oooo0;l0oooo00<000000?ooo`3oool0_`3oool00`000000oooo0?ooo`0B0?ooo`005P3oool0
0`000000oooo0?ooo`300?ooo`030000003oool0oooo0<00oooo00<000000?ooo`3oool04@3oool0
01H0oooo00<000000?ooo`3oool0`03oool00`000000oooo0?ooo`300?ooo`030000003oool0oooo
0140oooo000F0?ooo`030000003oool0oooo0<00oooo0P0000310?ooo`030000003oool0oooo0140
oooo000E0?ooo`030000003oool0oooo0<40oooo00<000000?ooo`3oool0[03oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo0140oooo000E0?ooo`030000003oool0oooo0;X0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`2/0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool0403oool001D0oooo00<000000?ooo`3oool0^P3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo0:`0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0@
0?ooo`005@3oool00`000000oooo0?ooo`2j0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool0[03oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0100oooo000D0?ooo`03
0000003oool0oooo0;/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2/0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool03`3oool001@0oooo00<000000?ooo`3oool0
^P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0:d0oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0?0?ooo`00503oool00`000000oooo0?ooo`2j0?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0[@3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo00l0oooo000D0?ooo`030000003oool0oooo0;X0oooo00<000000?ooo`3oool01@3oool2
00000:h0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0?0?ooo`004`3oool00`00
0000oooo0?ooo`2k0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0[@3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo00h0oooo000C0?ooo`030000003oool0oooo0;/0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`330?ooo`030000003oool0oooo00h0
oooo0000\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {-3.2672, 2.28801, 0.0225451, \
0.0371217}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(k\_eff = 2  SeriesCoefficient[potseries, 2]\)], "Input"],

Cell[BoxData[
    \(6.437751649736137`\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Omega] = \@\(k\_eff/m\)\)], "Input"],

Cell[BoxData[
    \(2.537272482358987`\ \@\(1\/m\)\)], "Output"]
}, Closed]],

Cell[TextData[
"For small \[Omega] we can approximate low-lying energy levels as follows."], 
  "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[CurlyEpsilon]\_n_ = emin + \((n + 1\/2)\) \[Omega]\)], "Input"],

Cell[BoxData[
    \(\(\(2.6094379124341005`\)\(\[InvisibleSpace]\)\) + 
      2.537272482358987`\ \@\(1\/m\)\ \((1\/2 + n)\)\)], "Output"]
}, Closed]],

Cell[TextData[{
  "Thus, roughly speaking, the thin barrier applies when ",
  Cell[BoxData[
      \(TraditionalForm\`m \[LessEqual] 1\)]],
  " and the thick barrier when ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  ">10."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Narrow barrier", "Section"],

Cell[CellGroupData[{

Cell["Low-lying states", "Subsection"],

Cell[TextData[{
  "Approximating each well by an independent harmonic oscillator, the lowest \
energy levels for ",
  Cell[BoxData[
      \(TraditionalForm\`m = 1\/2\)]],
  " would be "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[\[CurlyEpsilon]\_n /. m \[Rule] 1\/2, {n, 0, 1}]\)], "Input"],

Cell[BoxData[
    \({4.403560490428165`, 7.991805646416295`}\)], "Output"]
}, Closed]],

Cell[TextData[{
  "Because the wave functions for each well overlap within the barrier, the \
even and odd combinations of these wave functions have different energies, \
with the even combination being lower because the kinetic energy contribution \
is smaller.  Thus, the single-well eigenvalues separate into pairs and the \
splitting between the even and odd partners increases with ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " because the higher wave functions overlap more strongly in the central \
region.  The figure below reveals even solutions at ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(e\)\(\[TildeTilde]\)\)\)]],
  "3.89 and 6.55 and odd solutions at ",
  Cell[BoxData[
      \(TraditionalForm\`e \[TildeTilde] 4.43\)]],
  " and 8.13.  "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[mismatch[e, 0.5, pot, \(-5\), 5, 0], {e, 3.5, 10}, 
        PlotRange \[Rule] {All, {\(-10\), 10}}, PlotPoints \[Rule] 100, 
        Frame \[Rule] True, 
        FrameLabel \[Rule] {"\<energy\>", "\<mismatch\>"}, 
        PlotLabel \[Rule] "\<matching condition for m=0.5\>"];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.489011 0.14652 0.309017 0.0309017 [
[.09707 -0.0125 -5 -15.6875 ]
[.09707 -0.0125 5 0 ]
[.24359 -0.0125 -5 -15.6875 ]
[.24359 -0.0125 5 0 ]
[.39011 -0.0125 -5 -15.6875 ]
[.39011 -0.0125 5 0 ]
[.53663 -0.0125 -5 -15.6875 ]
[.53663 -0.0125 5 0 ]
[.68315 -0.0125 -5 -15.6875 ]
[.68315 -0.0125 5 0 ]
[.82967 -0.0125 -5 -15.6875 ]
[.82967 -0.0125 5 0 ]
[.97619 -0.0125 -8 -15.6875 ]
[.97619 -0.0125 8 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -17.9688 -31.375 ]
[.5 -0.0125 17.9688 -15.6875 ]
[-0.0125 .07725 -27.3125 -7.84375 ]
[-0.0125 .07725 0 7.84375 ]
[-0.0125 .15451 -18.3125 -7.84375 ]
[-0.0125 .15451 0 7.84375 ]
[-0.0125 .23176 -27.3125 -7.84375 ]
[-0.0125 .23176 0 7.84375 ]
[-0.0125 .30902 -10 -7.84375 ]
[-0.0125 .30902 0 7.84375 ]
[-0.0125 .38627 -19 -7.84375 ]
[-0.0125 .38627 0 7.84375 ]
[-0.0125 .46353 -10 -7.84375 ]
[-0.0125 .46353 0 7.84375 ]
[-0.0125 .54078 -19 -7.84375 ]
[-0.0125 .54078 0 7.84375 ]
[-0.0125 .61803 -16 -7.84375 ]
[-0.0125 .61803 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -43 -25.2813 ]
[-0.01875 .30902 -27.3125 25.2813 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -74.3125 0 ]
[.5 .63053 74.3125 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.09707 0 m
.09707 .00625 L
s
gsave
.09707 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.24359 0 m
.24359 .00625 L
s
gsave
.24359 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
.39011 0 m
.39011 .00625 L
s
gsave
.39011 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.53663 0 m
.53663 .00625 L
s
gsave
.53663 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7) show
1.000 setlinewidth
grestore
.68315 0 m
.68315 .00625 L
s
gsave
.68315 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8) show
1.000 setlinewidth
grestore
.82967 0 m
.82967 .00625 L
s
gsave
.82967 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(9) show
1.000 setlinewidth
grestore
.97619 0 m
.97619 .00625 L
s
gsave
.97619 -0.0125 -69 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
1.000 setlinewidth
grestore
.125 Mabswid
.12637 0 m
.12637 .00375 L
s
.15568 0 m
.15568 .00375 L
s
.18498 0 m
.18498 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.27289 0 m
.27289 .00375 L
s
.3022 0 m
.3022 .00375 L
s
.3315 0 m
.3315 .00375 L
s
.36081 0 m
.36081 .00375 L
s
.41941 0 m
.41941 .00375 L
s
.44872 0 m
.44872 .00375 L
s
.47802 0 m
.47802 .00375 L
s
.50733 0 m
.50733 .00375 L
s
.56593 0 m
.56593 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62454 0 m
.62454 .00375 L
s
.65385 0 m
.65385 .00375 L
s
.71245 0 m
.71245 .00375 L
s
.74176 0 m
.74176 .00375 L
s
.77106 0 m
.77106 .00375 L
s
.80037 0 m
.80037 .00375 L
s
.85897 0 m
.85897 .00375 L
s
.88828 0 m
.88828 .00375 L
s
.91758 0 m
.91758 .00375 L
s
.94689 0 m
.94689 .00375 L
s
.06777 0 m
.06777 .00375 L
s
.03846 0 m
.03846 .00375 L
s
.00916 0 m
.00916 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -78.9688 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(energy) show
94.938 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .07725 m
.00625 .07725 L
s
gsave
-0.0125 .07725 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7.5) show
1.000 setlinewidth
grestore
0 .15451 m
.00625 .15451 L
s
gsave
-0.0125 .15451 -79.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
0 .23176 m
.00625 .23176 L
s
gsave
-0.0125 .23176 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.5) show
1.000 setlinewidth
grestore
0 .30902 m
.00625 .30902 L
s
gsave
-0.0125 .30902 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
0 .38627 m
.00625 .38627 L
s
gsave
-0.0125 .38627 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.5) show
1.000 setlinewidth
grestore
0 .46353 m
.00625 .46353 L
s
gsave
-0.0125 .46353 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
0 .54078 m
.00625 .54078 L
s
gsave
-0.0125 .54078 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7.5) show
1.000 setlinewidth
grestore
0 .61803 m
.00625 .61803 L
s
gsave
-0.0125 .61803 -77 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .01545 m
.00375 .01545 L
s
0 .0309 m
.00375 .0309 L
s
0 .04635 m
.00375 .04635 L
s
0 .0618 m
.00375 .0618 L
s
0 .09271 m
.00375 .09271 L
s
0 .10816 m
.00375 .10816 L
s
0 .12361 m
.00375 .12361 L
s
0 .13906 m
.00375 .13906 L
s
0 .16996 m
.00375 .16996 L
s
0 .18541 m
.00375 .18541 L
s
0 .20086 m
.00375 .20086 L
s
0 .21631 m
.00375 .21631 L
s
0 .24721 m
.00375 .24721 L
s
0 .26266 m
.00375 .26266 L
s
0 .27812 m
.00375 .27812 L
s
0 .29357 m
.00375 .29357 L
s
0 .32447 m
.00375 .32447 L
s
0 .33992 m
.00375 .33992 L
s
0 .35537 m
.00375 .35537 L
s
0 .37082 m
.00375 .37082 L
s
0 .40172 m
.00375 .40172 L
s
0 .41717 m
.00375 .41717 L
s
0 .43262 m
.00375 .43262 L
s
0 .44807 m
.00375 .44807 L
s
0 .47898 m
.00375 .47898 L
s
0 .49443 m
.00375 .49443 L
s
0 .50988 m
.00375 .50988 L
s
0 .52533 m
.00375 .52533 L
s
0 .55623 m
.00375 .55623 L
s
0 .57168 m
.00375 .57168 L
s
0 .58713 m
.00375 .58713 L
s
0 .60258 m
.00375 .60258 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -121.438 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
86.2813 11.8438 translate
90 rotate
-86.2813 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(mismatch) show
109.563 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.09707 .61178 m
.09707 .61803 L
s
.24359 .61178 m
.24359 .61803 L
s
.39011 .61178 m
.39011 .61803 L
s
.53663 .61178 m
.53663 .61803 L
s
.68315 .61178 m
.68315 .61803 L
s
.82967 .61178 m
.82967 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.12637 .61428 m
.12637 .61803 L
s
.15568 .61428 m
.15568 .61803 L
s
.18498 .61428 m
.18498 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.27289 .61428 m
.27289 .61803 L
s
.3022 .61428 m
.3022 .61803 L
s
.3315 .61428 m
.3315 .61803 L
s
.36081 .61428 m
.36081 .61803 L
s
.41941 .61428 m
.41941 .61803 L
s
.44872 .61428 m
.44872 .61803 L
s
.47802 .61428 m
.47802 .61803 L
s
.50733 .61428 m
.50733 .61803 L
s
.56593 .61428 m
.56593 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.62454 .61428 m
.62454 .61803 L
s
.65385 .61428 m
.65385 .61803 L
s
.71245 .61428 m
.71245 .61803 L
s
.74176 .61428 m
.74176 .61803 L
s
.77106 .61428 m
.77106 .61803 L
s
.80037 .61428 m
.80037 .61803 L
s
.85897 .61428 m
.85897 .61803 L
s
.88828 .61428 m
.88828 .61803 L
s
.91758 .61428 m
.91758 .61803 L
s
.94689 .61428 m
.94689 .61803 L
s
.06777 .61428 m
.06777 .61803 L
s
.03846 .61428 m
.03846 .61803 L
s
.00916 .61428 m
.00916 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 0 m
1 0 L
s
.99375 .07725 m
1 .07725 L
s
.99375 .15451 m
1 .15451 L
s
.99375 .23176 m
1 .23176 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .38627 m
1 .38627 L
s
.99375 .46353 m
1 .46353 L
s
.99375 .54078 m
1 .54078 L
s
.125 Mabswid
.99625 .01545 m
1 .01545 L
s
.99625 .0309 m
1 .0309 L
s
.99625 .04635 m
1 .04635 L
s
.99625 .0618 m
1 .0618 L
s
.99625 .09271 m
1 .09271 L
s
.99625 .10816 m
1 .10816 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .13906 m
1 .13906 L
s
.99625 .16996 m
1 .16996 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .20086 m
1 .20086 L
s
.99625 .21631 m
1 .21631 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26266 m
1 .26266 L
s
.99625 .27812 m
1 .27812 L
s
.99625 .29357 m
1 .29357 L
s
.99625 .32447 m
1 .32447 L
s
.99625 .33992 m
1 .33992 L
s
.99625 .35537 m
1 .35537 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .40172 m
1 .40172 L
s
.99625 .41717 m
1 .41717 L
s
.99625 .43262 m
1 .43262 L
s
.99625 .44807 m
1 .44807 L
s
.99625 .47898 m
1 .47898 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .50988 m
1 .50988 L
s
.99625 .52533 m
1 .52533 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57168 m
1 .57168 L
s
.99625 .58713 m
1 .58713 L
s
.99625 .60258 m
1 .60258 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .30902 m
1 .30902 L
s
gsave
.5 .63053 -135.313 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(matching) show
110.375 15.813 moveto
(condition) show
158.563 15.813 moveto
(for) show
175.438 15.813 moveto
(m) show
184.813 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
192.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
207.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .35054 m
.03318 .34587 L
.04339 .34013 L
.05298 .33399 L
.06221 .32717 L
.07203 .31864 L
.08148 .30874 L
.09153 .29563 L
.10121 .27923 L
.11052 .25795 L
.11533 .24373 L
.12043 .22508 L
.12516 .20309 L
.1296 .17651 L
.13212 .1578 L
.13451 .13668 L
.139 .08435 L
.14132 .0476 L
s
.14132 .0476 m
.14366 0 L
s
.16018 0 m
.16018 .61803 L
s
.18538 .61803 m
.18752 .59817 L
.19211 .56462 L
.19714 .53725 L
.2021 .51642 L
.20661 .50117 L
.21632 .47618 L
.22566 .45858 L
.23559 .44411 L
.24515 .4329 L
.25435 .42385 L
.26414 .41557 L
.27357 .40856 L
.28359 .4019 L
.29324 .39605 L
.30252 .39085 L
.3124 .38565 L
.32191 .38091 L
.33106 .37653 L
.3408 .37203 L
.35017 .3678 L
.36013 .36339 L
.36973 .3592 L
.37896 .35518 L
.38879 .35091 L
.39824 .34677 L
.4083 .34232 L
.41798 .33796 L
.4273 .33369 L
.43721 .32901 L
.44675 .32438 L
.45593 .31977 L
.4657 .31466 L
.47511 .30952 L
.48511 .30378 L
.49474 .29793 L
.504 .29197 L
.51386 .28519 L
.52335 .27817 L
.53343 .27009 L
.54315 .26159 L
.5525 .2526 L
.56245 .242 L
.57203 .23052 L
.58124 .21809 L
.59104 .20287 L
.60048 .1858 L
.61051 .16418 L
.62017 .1388 L
.62947 .1084 L
Mistroke
.63936 .0668 L
.64889 .0126 L
Mfstroke
.64889 .0126 m
.65045 0 L
s
.70121 0 m
.70121 .61803 L
s
.76242 .61803 m
.76312 .61399 L
.77281 .57226 L
.78213 .54095 L
.79205 .51409 L
.8016 .49268 L
.81078 .47513 L
.82055 .45905 L
.82996 .44527 L
.83997 .43215 L
.8496 .42074 L
.85887 .41054 L
.86873 .40052 L
.87823 .39159 L
.88832 .38258 L
.89804 .37433 L
.9074 .36663 L
.91735 .35867 L
.92693 .35125 L
.93615 .34412 L
.94595 .33674 L
.9554 .32968 L
.96543 .3221 L
.9751 .31483 L
.97619 .314 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{361.813, 223.625},
  ImageMargins->{{36, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40008N0001Cb000`400?l00000o`00003oo`3ooooo
0?ooob00oooo003o0?ooool0oooo803oool00?l0ooooo`3ooolP0?ooo`00o`3oooli0?ooo`030?oo
g`2LB000000000<0000000<0B9cO0?ooo`3OW4P00P0000000`1d_ol0oooo0?ooo`3H0?ooo`00o`3o
ooli0?ooo`030;md0000B9`0gooo00<0oooo00<0gibL0;ooo`3oool00P3oool00`2oM7@0_ooo0?oo
o`3H0?ooo`00o`3oooli0?ooo`030?oog`2LM9`0gooo00<0oooo00<0oono09bLg`3oool00P3oool0
0`3ogi`0M9cO0?ooo`3H0?ooo`00o`3ooolj0?ooo`030;md00000000000000<0000000L0B9cO0?oo
o`3oool0oooo0?oog`2LB7@0_ooo0=P0oooo003o0?oooa`0oooo00L0_g@0000000000000001d0;oo
o`3oool0_g@00080000000<0001d0;oog`2LB0000P0000001000B9`0gooo0?ooo`2oM00200000004
0000M02oool0oooo0;md0080000000D0001809cOo`3oool0oono07@00004000000030000M02oool0
oooo00<0oooo00<0_g@007Boo`3oool0e`3oool00?l0oooo6`3oool03`2oM0000000001d_`3oool0
omnL04P0M02oool0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`0;0;md00000000
07Bo0?ooo`3ogi`0B01d0;ooo`3oool0gia80000M02oool00`3oool00`2oM4P0W=oo0?ooo`070?oo
o`040?oOW018000004RL0=ooomL0oooo003o0?oooaX0oooo00<0omnL04P00018W=l00`3oool03P3o
gi`0M9cO0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?oOW0180000B9cO0`3oool0
1P3ogi`0M9cO0?ooo`3OW4P0001d0;ooo`<0oooo00D0oono07@0000000000000001d_`050?ooo`04
0;md0000B9`0W7@007BoomL0oooo003o0?oooaX0oooo00<0okmd0000B02Lgol01P3oool02`3OW4P0
04RL0=ooo`3oool0oooO09a80000M;l0oooo0?noM00004P0W=oo00H0oooo00<0gia80000M02oool0
0`3oool01P3OW4P0001d0;ooo`3ookl0M00004RLg`<0oooo00D0okmd0000001d_ol0omnL04Qd_`3G
0?ooo`00o`3ooolJ0?ooo`030?noM00007@0_ooo00H0oooo00/0gia80018W03Oool0oooo0?oog`2L
B00007Bo0?ooo`3o_g@0001d0;ooo`060?ooo`030=nLB00007@0_ooo0080oooo00L0oono07@00018
W=l0oooo0?oog`2LB00007Bo0080oooo00L0oooO09a80000B9`0gooo0?ooo`2oM4P0W=oo0=H0oooo
003o0?oooaX0oooo00<0omnL04P000000000100000003018W=l0oooo0;md000007@0_ooo0?ooo`3o
oml0W4P0001d_`3oool0omnL04P000D0000000D0B9cO0?ooo`3OW4P0001809cOo`020?ooo`070?oo
_`1d0000B9cO0?ooo`3ooml0W4P004RLg`020?ooo`030?noM00004P0W=oo0080oooo00<0okmd04RL
g`3oool0e@3oool00?l0oooo6`3oool03`2oM000B9cO0?ooo`3oool0gia80000B02Lgml0W4P00000
0000B4P0B01809cOo`3ookl0M00004RLg`020?ooo`0B0;md0018W=l0oooo0?ooo`3OW4P0001809cO
g`2LB00000000000B018M9`0gkmd0000M02oool0_g@0001d_`3oool0okmd0P0000001@00B9`0gono
07@00000B9`0gooo0080oooo00<0oono07@0M02oool0e@3oool00?l0oooo703oool01@3OW4P00000
0000000004P0W=oo0080oooo00L0_g@00018W03Ooml0W4P00000000007@0_ooo00<0oooo00D0gia8
000000000000001809cOo`020?ooo`080;md000007@0_mnL04P0000007@0_ooo0?oog`2LB0020000
000407Boo`3oool0oooO09a80080000000L004RL0=ooo`3oool0_g@000000000B9`0gooo0=@0oooo
003o0?ooool0oooo803oool00?l0ooooo`3ooolP0?ooo`00o`3ooooo0?ooob00oooo003o0?ooool0
oooo803oool00?l0ooooo`3ooolP0?ooo`00o`3ooooo0?ooob00oooo003o0?ooool0oooo803oool0
0?l0ooooo`3ooolP0?ooo`00o`3ooooo0?ooob00oooo003o0?ooool0oooo803oool00?l0ooooo`3o
oolP0?ooo`00o`3ooooo0?ooob00oooo003o0?ooool0oooo803oool00?l0ooooo`3ooolP0?ooo`00
o`3ooooo0?ooob00oooo002<0?ooo`040?oog`2LB00004RL0=oooc@0oooo00<0oooO09a800000000
0P0000000`1d_ol0oooo0?ooo`0f0?ooo`050?oOW018000000000000001d_ol0>03oool00`3OW4P0
B9cO0?ooo`0h0?ooo`050=nLB0000000000000000018W=l0=@3oool01@3ookl0M00000000000B9`0
gooo03D0oooo00H0_g@000000000000000000018W03Oool30?ooo`050?oOW018000000000000M02o
ool08P3oool008`0oooo00@0oooO09a80000B9`0gooo=03oool00`3ooml0W4Qd0;ooo`020?ooo`03
0;md001d_ol0oooo03D0oooo00L0omnL04P00018W=l0oooo0?oo_`1d0000M;oo03L0oooo00<0omnL
04Q8W03Oool0=`3oool01`3OW4P0000007Boo`3oool0oooO09a80018W=l0=`3oool00`3OW4P00000
07Boo`0e0?ooo`030?oOW0180000M;oo00@0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo
0240oooo002<0?ooo`040?oog`2LB00004RL0=ooocT0oooo00<0oooO09a8M02oool0=03oool00`3o
oml0W4P0001d_`030?ooo`030?noM000B9`0gooo03H0oooo00<0oooO09a8B02Lgol0=P3oool00`3o
oml0W4P0001d_`030?ooo`030?noM000B9`0gooo03L0oooo00<0oono07@0001d_ol0=03oool00`3o
gi`0B00007Boo`040?ooo`030;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool0803oool008P0
oooo00<0_g@00000000000001@0000000`1d_ol0oooo0?ooo`0g0?ooo`030;md001d_ol0oooo03<0
oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0g0?ooo`030=nLB018W=l0oooo03H0
oooo00<0_g@0001d_`3oool00P3oool00`2oM00004RL0=ooo`0h0?ooo`030?oOW01804P0W=oo03<0
oooo00<0omnL04P0001d_ol00`3oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo0200
oooo00280?ooo`080?oOW01dW=l0oooo0?ooo`3ooml0W4P00018W03Oooli0?ooo`030?oog`2LB000
B9cO03@0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0g0?ooo`030?oOW018B9`0
gooo03H0oooo00L0omnL04P0M02oool0oooo0;md00000000M;oo03D0oooo00<0oooO09a800000000
0`0000000`00M;l0oooo0?ooo`0a0?ooo`030?oOW0180000M;oo00<0oooo00<0omnL04P0001d_ol0
0`3oool00`3o_g@0001809cOo`0P0?ooo`00R@3oool01`3OW4P0M;oo0?ooo`3ooml0W4P00018W03O
ool0>@3oool00`3OW4P0000007Boo`0d0?ooo`030?oOW0180000M;oo00<0oooo00<0_g@00000B02L
gol0=`3oool00`3ooml0W4Q809cOo`0g0?ooo`050?oo_`1dB4P0B00000000018W=l0=P3oool0202o
M000000007Boo`3oool0oooO09a8000007@0_ooo<P3oool00`3ogi`0B00007Boo`030?ooo`030?oO
W0180000M;oo00<0oooo00<0okmd0000B02Lgol0803oool008T0oooo00L0oooO09a8M02oool0oooO
09a80000B9`0gooo03D0oooo00L0gmoO0?ooo`3ookl0M0000000000007@0_ooo03@0oooo00P0oono
07@000000000B9cO0?ooo`3o_g@00000001d_cT0oooo00<0gia8001d_`3oool0=P3oool0103ogi`0
B00000000000M;lf0?ooo`030?oOW0180000B9cO00<0oooo00<0okmd0000B02Lgol0<P3oool00`3o
gi`0B00007Boo`030?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0803oool008X0
oooo00H0omnL04Qd_`3ooml0W4P00018W03Ooole0?ooo`060=nLB000000000000000000004P0W=oo
=P3oool00`2oM00004RL0;nLB002000000030018W03Oool0oooo03P0oooo00<0oono07@0M02oool0
=@3oool01P3ogi`0B0000000M02oool0okmd001d_cD0oooo00<0omnL04P0001d_ol00`3oool00`3o
_g@0001809cOo`0b0?ooo`030?oOW0180000M;oo00<0oooo00<0omnL04P0001d_ol00`3oool00`3o
_g@0001809cOo`0P0?ooo`00R`3oool01@3OW4P0M;oO09a80000B9`0gooo03D0oooo00@0oono07@0
00000000B9cO>03oool00`3ogi`0B00007Boo`0h0?ooo`030?oOg`3Oool0oooo00<0oooo00<0_g@0
07Boo`3oool0=03oool00`2oM000001d0;ooo`020?ooo`030;md0018W=l0oooo03<0oooo00<0omnL
04P0001d_ol00`3oool00`3OW4P0001d0;ooo`0b0?ooo`030?oOW0180000M;oo00<0oooo00<0oono
07@00018W=l00`3oool00`3OW4P0001d0;ooo`0P0?ooo`00R`3oool01@3ooml0W4Q804Q80000B9`0
gooo03H0oooo00<0_g@007Boo`3oool0>@3oool00`3OW4P0001809cOo`0g0?ooo`030?oo_`1dB9`0
gooo00<0oooo00<0okmd001d_`3oool0=03oool00`2oM00004RL0=ooo`020?ooo`030=nLB000M;l0
oooo03<0oooo00<0oooO09a8001d_ol00P3oool0103ooml0W4P00018W03Ooolb0?ooo`030?oOW018
0000M;oo00@0oooo00<0gia80018W03Oool00P3oool00`2oM00007Bo0?ooo`0P0?ooo`00S03oool0
103ogi`0B0000018W03Ooolf0?ooo`060?oOW018000000000000000004P0W=oo=`3oool00`3o_g@0
0000001d_`0g0?ooo`030;md00000000000000<0000000<0001d0;ooo`3oool0<`3oool01`3o_g@0
000007Boo`3oool0oono07@0001d_ol0=@3oool01`3o_g@0001d0;ooo`3oool0gia80000B02Lgol0
<@3oool01@3ooml0W4P0000000000000M;oo00@0oooo00L0oono07@0001d_ol0oooo0?noM00007@0
_ooo0240oooo002=0?ooo`030=nLB000B9`0gooo03L0oooo00D0_g@0000000000000000004RLg`0h
0?ooo`050?oog`2LB00000000000M02oool0=03oool00`3o_g@00000000000040000000307Boo`3o
ool0oooo03<0oooo00D0okmd000000000000000004RLg`0g0?ooo`050?noM000000000000000B02L
gol0=03oool00`3o_g@0000007Boo`050?ooo`050?oo_`1d000000000000M02oool08P3oool00?l0
ooooo`3ooolP0?ooo`00o`3ooooo0?ooob00oooo003o0?ooool0oooo803oool00?l0ooooo`3ooolP
0?ooo`00o`3ooooo0?ooob00oooo003o0?ooool0oooo803oool00?l0ooooo`3ooolP0?ooo`00o`3o
oooo0?ooob00oooo003o0?ooool0oooo803oool00?l0ooooo`3ooolP0?ooo`00I03ooooo000009`0
00007`3oool006@0oooo00D000000?ooo`3oool0oooo0000000;0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool00P3oool00`000000oooo000000090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00P0
oooo0P00000;0?ooo`030000003oool0oooo00D0oooo00D000000?ooo`3oool0oooo0000000<0?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo01d0oooo001T0?ooo`050000003oool0oooo0?ooo`0000002`3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo0080oooo00<000000?ooo`0000002@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`080?ooo`8000002`3oool00`000000oooo0?ooo`050?ooo`050000003oool0oooo
0?ooo`000000303oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?ooo`0U0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0O0?oo
o`030000003oool0oooo03T0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0[0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0b0?oo
o`030000003oool0oooo03T0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0M0?oo
o`00I03oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool0a@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3o
ool07@3oool006@0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo0<D0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`1g0?ooo`030000
003oool0oooo01d0oooo001T0?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`350?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0M`3o
ool00`000000oooo0?ooo`0M0?ooo`00I03oool3000003P0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`350?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0MP3oool2
000001l0oooo001T0?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`350?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0M`3oool00`00
0000oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool0a03oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo07L0
oooo00<000000?ooo`3oool07@3oool006@0oooo00<000000?ooo`3oool0=`3oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo0<@0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`1g0?ooo`030000003oool0oooo01d0oooo001T0?ooo`030000003oool0oooo03L0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`340?ooo`030000003oool0oooo01@0oooo00<0
00000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?ooo`0g
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0a03oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool006@0oooo0`00000g0?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0a03oool00`000000oooo0?ooo`0D0?oo
o`030000003oool0oooo07H0oooo0P00000O0?ooo`00I03oool00`000000oooo0?ooo`0g0?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool0a03oool00`000000oooo0?ooo`0D0?ooo`03
0000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool006@0oooo00<000000?ooo`3oool0
=`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0<@0oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo01d0oooo001T0?ooo`030000003o
ool0oooo03L0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`340?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`00I03oool0
0`000000oooo0?ooo`0g0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0``3oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool0
06@0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
0<<0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo
01d0oooo001T0?ooo`<00000=`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0<<0
oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`1f0?ooo`8000007`3oool006@0oooo
00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0<<0oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo01d0oooo
001T0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`330?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?oo
o`0M0?ooo`00I03oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool0``3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo07L0oooo00<00000
0?ooo`3oool07@3oool006@0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo0<<0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`1g0?oo
o`030000003oool0oooo01d0oooo001T0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`330?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3o
ool0M`3oool00`000000oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?ooo`0g0?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool0`P3oool00`000000oooo0?ooo`0F0?ooo`030000
003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool006@0oooo0`00000g0?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0`P3oool00`000000oooo0?ooo`0F0?ooo`030000003o
ool0oooo07H0oooo0P00000O0?ooo`00B@3oool00`3OW4P0B9cO0?ooo`040?ooo`060?noM0000000
M;oo0?ooo`3ooml0W4P00`0000000`1d_ol0oooo0?ooo`080?ooo`030000003oool0oooo03L0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`320?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`00B@3oool00`3ogi`0B4RL0=oo
o`040?ooo`070?noM0000000M;oo0?ooo`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0oooo00P0
oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0<80
oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo01d0
oooo00190?ooo`030?oog`2LB4P0W=oo00d0oooo00<0oooO09a8M02oool0203oool00`000000oooo
0?ooo`0f0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0`P3oool00`000000oooo
0?ooo`0F0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool004X0oooo00<0
gia804RLg`3oool03@3oool00`2oM000M;oo0?ooo`070?ooo`030000003oool0oooo03H0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`310?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`00>@3oool00`3ooml0W4P000000007
0000000304RLg`3oool0oooo00@0oooo00<0omnL04Q8W03Oool0303oool00`3ooml0W4P004RLg`08
0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`31
0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M
0?ooo`00BP3oool00`3ooml0W4Q809cOo`0<0?ooo`030=nLB0000000M;oo00P0oooo1000000e0?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool0`@3oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo07D0oooo0`00000O0?ooo`00B`3oool00`3OW4P007Bo0?ooo`070?ooo`07
0=oOg`3oool0oono07@000000000001d0;ooo`080?ooo`030000003oool0oooo03H0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`310?ooo`030000003oool0oooo01L0oooo00<00000
0?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`00B`3oool00`3ookl0M01d0;ooo`070?oo
o`060=nLB000000000000000000004P0W=oo2@3oool00`000000oooo0?ooo`0f0?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool0`03oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo07L0oooo00<000000?ooo`3oool07@3oool004H0oooo00<0omoO0=ooo`3oool00`3oool0
0`2oM000M;oo0?ooo`060?ooo`040?oo_`1d0000000004RLg`/0oooo00<000000?ooo`3oool0=P3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0<00oooo00<000000?ooo`3oool0603o
ool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo01d0oooo00160?ooo`030?oo_`1dB9`0
gooo00<0oooo00<0okmd001d_`3oool01`3oool00`2oM000M;oo0?ooo`0;0?ooo`030000003oool0
oooo03H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`300?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`00A`3oool00`2o
M000000000000003000000030000M02oool0oooo00H0oooo00H0omnL04P00000000000000000B02L
gol80?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`2o0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?oo
o`0M0?ooo`00A`3oool00`3o_g@00000000000040000000307Boo`3oool0oooo00H0oooo00D0_g@0
000000000000000004RLg`080?ooo`<00000=P3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo0;l0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`1f0?ooo`8000007`3o
ool006@0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo0;l0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0
oooo01d0oooo001T0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`2n0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0M`3oool00`00
0000oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool0_P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo07L0
oooo00<000000?ooo`3oool07@3oool006@0oooo00<000000?ooo`3oool0=@3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`1g0?ooo`030000003oool0oooo01d0oooo001T0?ooo`030000003oool0oooo03D0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`2n0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?ooo`0e
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0_@3oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool006@0oooo0`00000e0?oo
o`030000003oool0oooo00L0oooo00<000000?ooo`3oool0_@3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo07H0oooo0P00000O0?ooo`00I03oool00`000000oooo0?ooo`0e0?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool0_@3oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool006@0oooo00<000000?ooo`3oool0
=@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0;`0oooo00<000000?ooo`3oool0
703oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo01d0oooo001T0?ooo`030000003o
ool0oooo03D0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`2l0?ooo`030000003o
ool0oooo01`0oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`00I03oool0
0`000000oooo0?ooo`0e0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0_03oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool0
06@0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
0;`0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo
01d0oooo001T0?ooo`<00000=03oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0;/0
oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`1f0?ooo`8000007`3oool006@0oooo
00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0;/0oooo
00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo01d0oooo
001T0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0203oool00`000000oooo0?oo
o`2k0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?oo
o`0M0?ooo`00I03oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo00P0oooo00<00000
0?ooo`3oool0^`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo07L0oooo00<00000
0?ooo`3oool07@3oool006@0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`080?oo
o`030000003oool0oooo0;X0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`1g0?oo
o`030000003oool0oooo01d0oooo001T0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3o
ool0203oool00`000000oooo0?ooo`2j0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3o
ool0M`3oool00`000000oooo0?ooo`0M0?ooo`00I03oool3000003@0oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`2j0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0
MP3oool2000001l0oooo001T0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`2j0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0M`3o
ool00`000000oooo0?ooo`0M0?ooo`00D`3oool00`3ooml0W4P0000000020000000307Boo`3oool0
oooo00T0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo0;T0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0
oooo01d0oooo001C0?ooo`030?oog`2LB7@0_ooo0080oooo00<0_g@007Boo`3oool02@3oool00`00
0000oooo0?ooo`0d0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0^@3oool00`00
0000oooo0?ooo`0O0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool005P0
oooo00<0oooO09a8M02oool02@3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool0^@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo07L0
oooo00<000000?ooo`3oool07@3oool005T0oooo00<0_g@007Boo`3oool0203oool00`000000oooo
0?ooo`0c0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0^03oool00`000000oooo
0?ooo`0P0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool004L0oooo00<0
oooO09a8000000001`0000000`18W=l0oooo0?ooo`040?ooo`030?oog`2LB000B9cO00T0oooo00<0
00000?ooo`3oool0<`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0;P0oooo00<0
00000?ooo`3oool0803oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo01d0oooo001H
0?ooo`030=nLB0000000M;oo00T0oooo1000000b0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool0]`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo07D0oooo0`00000O
0?ooo`00E03oool01`3Ogml0oooo0?oo_`1d000000000000M02oool02@3oool00`000000oooo0?oo
o`0c0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0]`3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool005@0oooo00H0gia8
00000000000000000000B02Lgol:0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`2g0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0
M`3oool00`000000oooo0?ooo`0M0?ooo`00E03oool0103ookl0M00000000018W=l<0?ooo`030000
003oool0oooo03<0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`2f0?ooo`030000
003oool0oooo0280oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`00E@3o
ool00`2oM000M;oo0?ooo`0<0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`2f0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0M`3o
ool00`000000oooo0?ooo`0M0?ooo`00E@3oool01P3ogi`0B000000000000000001809cOo`T0oooo
00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0;D0oooo
00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo01d0oooo
001F0?ooo`050;md00000000000000000018W=l02@3oool300000380oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`2e0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0
MP3oool2000001l0oooo001T0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`2d0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0M`3o
ool00`000000oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?ooo`0b0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool0]03oool00`000000oooo0?ooo`0T0?ooo`030000003oool0
oooo07L0oooo00<000000?ooo`3oool07@3oool006@0oooo00<000000?ooo`3oool0<P3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo0;<0oooo00<000000?ooo`3oool09@3oool00`00
0000oooo0?ooo`1g0?ooo`030000003oool0oooo01d0oooo001T0?ooo`030000003oool0oooo0380
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`2c0?ooo`030000003oool0oooo02D0
oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`00I03oool00`000000oooo
0?ooo`0b0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0/P3oool00`000000oooo
0?ooo`0V0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool006@0oooo0`00
000b0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0/P3oool00`000000oooo0?oo
o`0V0?ooo`030000003oool0oooo07H0oooo0P00000O0?ooo`00I03oool00`000000oooo0?ooo`0b
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0/@3oool00`000000oooo0?ooo`0W
0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool006@0oooo00<000000?oo
o`3oool0<P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0;40oooo00<000000?oo
o`3oool09`3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo01d0oooo001T0?ooo`03
0000003oool0oooo0340oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`2`0?ooo`03
0000003oool0oooo02P0oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`00
I03oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0
/03oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool0
7@3oool006@0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo0:l0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`1g0?ooo`030000003o
ool0oooo01d0oooo001T0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`2_0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0M`3oool0
0`000000oooo0?ooo`0M0?ooo`00I03oool300000340oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`2^0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0MP3oool20000
01l0oooo001T0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool02`3oool00`000000
oooo0?ooo`2]0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0M`3oool00`000000
oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0[@3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo07L0oooo
00<000000?ooo`3oool07@3oool006@0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo0:`0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?oo
o`1g0?ooo`030000003oool0oooo01d0oooo001T0?ooo`030000003oool0oooo0300oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`2/0?ooo`030000003oool0oooo02`0oooo00<00000
0?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?ooo`0`0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0Z`3oool00`000000oooo0?ooo`0]0?oo
o`030000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool006@0oooo0`00000`0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0Z`3oool00`000000oooo0?ooo`0]0?ooo`03
0000003oool0oooo07H0oooo0P00000O0?ooo`00AP3oool00`2oM0000000000000040000000904RL
g`3oool0oooo0?noM0000000M;oo0?ooo`3ooml0W4P000<0000000<0M;oo0?ooo`3oool0203oool0
0`000000oooo0?ooo`0`0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0ZP3oool0
0`000000oooo0?ooo`0^0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool0
04L0oooo00<0gia80000000000000`0000002P00B9`0gooo0?ooo`3o_g@0000007Boo`3oool0oooO
09a8M02oool20?ooo`030;md001d_ol0oooo00P0oooo00<000000?ooo`3oool0;`3oool00`000000
oooo0?ooo`0=0?ooo`030000003oool0oooo0:T0oooo00<000000?ooo`3oool0;`3oool00`000000
oooo0?ooo`1g0?ooo`030000003oool0oooo01d0oooo00180?ooo`030=nLB018W=l0oooo0080oooo
00<0giad09cOo`3oool02@3oool00`3ooml0W4Qd0;ooo`080?ooo`030000003oool0oooo02l0oooo
00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`2X0?ooo`030000003oool0oooo0300oooo
00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`009P3oool00`3ookl0M7Bo0?oo
o`040?ooo`030=nLW02oool0oooo01T0oooo00<0gia807Boo`3oool03P3oool00`2oM000M;oo0?oo
o`070?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`2X0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?oo
o`0M0?ooo`009P3oool00`3o_g@0000000000004000000030000M02oool0oooo00T0oooo00<0oooO
09a8000000001`0000000`18W=l0oooo0?ooo`040?ooo`030;mdB02Lgol0oooo00`0oooo00<0oooO
09a80018W=l0203oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool0Y`3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo07L0oooo00<00000
0?ooo`3oool07@3oool002H0oooo00<0_g@0000000000000100000000`0007@0_ooo0?ooo`0J0?oo
o`030?oog`2LB7@0_ooo00`0oooo00<0gia80000001d_ol0203oool4000002d0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`2V0?ooo`030000003oool0oooo0380oooo00<000000?oo
o`3oool0M@3oool3000001l0oooo000W0?ooo`030;mdM02oool0oooo00<0oooo00<0gibL0;ooo`3o
ool06`3oool00`3o_g@007Bo0?ooo`070?ooo`070=oOg`3oool0oono07@000000000001d0;ooo`08
0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`2U
0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M
0?ooo`009P3oool00`3ogi`0W;oo0?ooo`0R0?ooo`030?oo_`1d04P0W=oo00L0oooo00H0gia80000
0000000000000000B02Lgol90?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`2T0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0M`3o
ool00`000000oooo0?ooo`0M0?ooo`009P3oool00`2oM00007Bo0?ooo`040?ooo`030=nLW02oool0
oooo01/0oooo00<0oono07@00018W=l01`3oool0103ookl0M00000000018W=l;0?ooo`030000003o
ool0oooo02d0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`2T0?ooo`030000003o
ool0oooo03@0oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`009P3oool0
0`3OW4P0000000000004000000030000M02oool0oooo01H0oooo00<0oknL09cOo`3oool00P3oool0
0`3ogi`0B00004RLg`080?ooo`030;md001d_ol0oooo00/0oooo00<000000?ooo`3oool0;03oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0:<0oooo00<000000?ooo`3oool0=@3oool0
0`000000oooo0?ooo`1g0?ooo`030000003oool0oooo01d0oooo000V0?ooo`030?oog`2LB0000000
00@0000000<0001d0;ooo`3oool05P3oool0203ooml0W4Q809cOo`3oool0oono07@0000007@0_ooo
203oool01P3ogi`0B000000000000000001809cOo`P0oooo00<000000?ooo`3oool0;03oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo0:80oooo00<000000?ooo`3oool0=P3oool00`00
0000oooo0?ooo`1g0?ooo`030000003oool0oooo01d0oooo000W0?ooo`030=nLW02oool0oooo00<0
oooo00<0gibL0;ooo`3oool05`3oool01P3ooml0W4P0000000000000001d0;ooo`X0oooo00D0_g@0
000000000000000004RLg`080?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`2Q0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0M`3o
ool00`000000oooo0?ooo`0M0?ooo`009P3oool00`3o_g@0B9cO0?ooo`0k0?ooo`<00000;03oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0:00oooo00<000000?ooo`3oool0>03oool0
0`000000oooo0?ooo`1f0?ooo`8000007`3oool002H0oooo00<0_g@0001d_`3oool0103oool00`3O
W9`0_ooo0?ooo`0d0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`2O0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool0M`3oool00`00
0000oooo0?ooo`0M0?ooo`009P3oool00`3OW4P0000000000004000000030000M02oool0oooo03@0
oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo09h0
oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo01d0
oooo000V0?ooo`030?oog`2LB000000000@0000000<0001d0;ooo`3oool0=03oool00`000000oooo
0?ooo`0[0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0W@3oool00`000000oooo
0?ooo`0k0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool002d0oooo00<0
gibL0;ooo`3oool0=03oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool0W03oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo07L0oooo00<0
00000?ooo`3oool07@3oool002H0oooo00<0oono07Ad_`3oool0103oool00`3OW9`0_ooo0?ooo`0d
0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`2K
0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M
0?ooo`00803oool00`3ooml0W4P0001d_`030?ooo`030?noM0000000000000@0000000<0001d0;oo
o`3oool0=03oool3000002X0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`2J0?oo
o`030000003oool0oooo03h0oooo00<000000?ooo`3oool0MP3oool2000001l0oooo000P0?ooo`03
0?oog`2LB00007Bo00<0oooo00<0_g@0000000000000100000000`0007@0_ooo0?ooo`0d0?ooo`03
0000003oool0oooo02X0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`2I0?ooo`03
0000003oool0oooo03l0oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`00
;@3oool00`3OW9`0_ooo0?ooo`0d0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`2G0?ooo`800000@P3oool00`000000oooo0?ooo`1g0?ooo`030000
003oool0oooo01d0oooo001T0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool04`3o
ool00`000000oooo0?ooo`2F0?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool0M`3o
ool00`000000oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?ooo`0X0?ooo`030000003oool0
oooo01@0oooo00<000000?ooo`3oool0U@3oool00`000000oooo0?ooo`130?ooo`030000003oool0
oooo07L0oooo00<000000?ooo`3oool07@3oool002H0oooo00T0oooO09a800000000001d0;oog`2L
B00000000000M02oool0=@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool0T`3oool2000004H0oooo00<000000?ooo`3oool0M`3oool00`000000oooo
0?ooo`0M0?ooo`009P3oool0203OW7@0W=oo0?oOW018000007Bo0?ooo`3o_g@0B9cO=P3oool30000
02L0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`2B0?ooo`030000003oool0oooo
04H0oooo00<000000?ooo`3oool0MP3oool2000001l0oooo000V0?ooo`090=nLW02oool0oooo0;md
000004P0W=oo0?ooo`2oM7@0_ooo03D0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?oo
o`0E0?ooo`030000003oool0oooo0940oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?oo
o`1g0?ooo`030000003oool0oooo01d0oooo000V0?ooo`090?oOW018M;l0oooo0?oOW018000007Bo
0?ooo`2oW9`0gooo03D0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0F0?ooo`03
0000003oool0oooo08l0oooo0P00001:0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3o
ool07@3oool002H0oooo00D0_g@00000000004P0W=oo0=nLB0020000000307Boo`3oool0oooo03@0
oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo08h0
oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo01d0
oooo001T0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool05`3oool00`000000oooo
0?ooo`2=0?ooo`030000003oool0oooo04/0oooo00<000000?ooo`3oool0M`3oool00`000000oooo
0?ooo`0M0?ooo`009P3oool00`3ookl0M7Bo0?ooo`040?ooo`030=nLW02oool0oooo03@0oooo00<0
00000?ooo`3oool09@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo08/0oooo0P00
001>0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool002H0oooo00<0okmd
000000000000100000000`0007@0_ooo0?ooo`0d0?ooo`030000003oool0oooo02@0oooo00<00000
0?ooo`3oool0603oool00`000000oooo0?ooo`2:0?ooo`030000003oool0oooo04h0oooo00<00000
0?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`009P3oool00`2oM0000000000000040000
00030000M02oool0oooo03@0oooo0`00000T0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool0R03oool200000540oooo00<000000?ooo`3oool0MP3oool2000001l0oooo000W0?ooo`03
0;mdM02oool0oooo00<0oooo00<0gibL0;ooo`3oool09@3oool01@3ogi`0B0000000000007@0_ooo
00X0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
08H0oooo0P00001C0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool002H0
oooo00<0omnL09boo`3oool0:`3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool02@3oool0
0`000000oooo0?ooo`0R0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0Q@3oool0
0`000000oooo0?ooo`1C0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool07@3oool0
02H0oooo00<0_g@0001d_`3oool0103oool00`3OW9`0_ooo0?ooo`0T0?ooo`030;md0000B9`0gooo
0080oooo00<0_g@0001d_`3oool0203oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo
01/0oooo00<000000?ooo`3oool0P`3oool2000005H0oooo00<000000?ooo`3oool0M`3oool00`00
0000oooo0?ooo`0M0?ooo`009P3oool00`3OW4P0000000000004000000030000M02oool0oooo02<0
oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`080?ooo`030000003oool0oooo0240
oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`220?ooo`030000003oool0oooo05H0
oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`0M0?ooo`009P3oool00`3ooml0W4P0
00000004000000030000M02oool0oooo02<0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@00018
09cOo`080?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`200?ooo`800000F@3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo01d0oooo
000W0?ooo`030=nLW02oool0oooo00<0oooo00<0gibL0;ooo`3oool08`3oool00`3ogi`0B00007Bo
o`030?ooo`030?noM00004P0W=oo00P0ooooo`00002L000001l0oooo000V0?ooo`030?noM018W=l0
oooo02X0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`080?ooo`030000003oool0
oooo01h0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`1l0?ooo`800000G@3oool0
0`000000oooo0?ooo`1g0?ooo`030000003oool0oooo01d0oooo000V0?ooo`030;md0000M;l0oooo
00@0oooo00<0gibL0;ooo`3oool08`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo
00P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo
07X0oooo0P00001O0?ooo`030000003oool0oooo06d0oooo0P0000080?ooo`030000003oool0oooo
01d0oooo000V0?ooo`030=nLB0000000000000@0000000<0001d0;ooo`3oool08`3oool00`3ookl0
M00004RLg`030?ooo`030=nLB00007@0_ooo00P0oooo00<000000?ooo`3oool0703oool00`000000
oooo0?ooo`0P0?ooo`030000003oool0oooo07P0oooo0P00001Q0?ooo`030000003oool0oooo06`0
oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0M0?ooo`009P3oool00`3ooml0W4P0
00000004000000030000M02oool0oooo02@0oooo00<0gia80018W03Oool00P3oool00`2oM00007Bo
0?ooo`080?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`1f0?ooo`800000H`3oool00`000000oooo0?ooo`1Z0?ooo`8000002`3oool00`000000oooo
0?ooo`0M0?ooo`00;@3oool00`3OW9`0_ooo0?ooo`0T0?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0
001d0;ooo`090?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool08P3oool00`000000
oooo0?ooo`1d0?ooo`800000I@3oool00`000000oooo0?ooo`1Y0?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool07@3oool005D0oooo00D0oono07@000000000001d0;ooo`0:0?ooo`<0
0000603oool2000002D0oooo00<000000?ooo`3oool0LP3oool2000006L0oooo00<000000?ooo`3o
ool0J03oool00`000000oooo0?ooo`0;0?ooo`8000007`3oool002L0oooo00L0_g@00000B02Lgol0
oono07@000000000B9cO03H0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0U0?oo
o`030000003oool0oooo0700oooo0P00001Y0?ooo`030000003oool0oooo06L0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`0M0?ooo`009P3oool01@3o_g@000000000B02Lgol0_g@0
0080000000<0001d0;ooo`3oool0=03oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo
02H0oooo00<000000?ooo`3oool0KP3oool2000006/0oooo00<000000?ooo`3oool0IP3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo01d0oooo000V0?ooo`090;mdM02oool0oooo0?oo
_`1dB9`0gooo0?oOW018B9`0gooo03D0oooo00<000000?ooo`3oool0503oool2000002T0oooo00<0
00000?ooo`3oool0K03oool2000006d0oooo00<000000?ooo`3oool0I@3oool00`000000oooo0?oo
o`0?0?ooo`030000003oool0oooo01d0oooo000V0?ooo`080;mdB02Lgol0oooo0=nLM02Lgol0oooo
0?noW02Lgolf0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0:@3oool00`000000
oooo0?ooo`1Z0?ooo`800000K`3oool00`000000oooo0?ooo`1S0?ooo`8000004P3oool00`000000
oooo0?ooo`0M0?ooo`009P3oool00`3OW4P0000000000004000000030018W03Oool0oooo03@0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo06P0oooo
0P00001a0?ooo`030000003oool0oooo0640oooo0P00000D0?ooo`030000003oool0oooo01d0oooo
000V0?ooo`030?oo_`1d0000000000@0000000<004RL0=ooo`3oool0=03oool00`000000oooo0?oo
o`0@0?ooo`800000;@3oool00`000000oooo0?ooo`1W0?ooo`030000003oool0oooo0740oooo00<0
00000?ooo`3oool0H03oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01d0oooo000/
0?ooo`030;mdB02Lgol0oooo03D0oooo0`00000?0?ooo`030000003oool0oooo02d0oooo00<00000
0?ooo`3oool0I@3oool2000007@0oooo00<000000?ooo`3oool0GP3oool2000001H0oooo0P00000O
0?ooo`009P3oool00`2oM7@0_ooo0?ooo`0k0?ooo`030000003oool0oooo00d0oooo0P00000`0?oo
o`030000003oool0oooo06<0oooo0P00001f0?ooo`030000003oool0oooo05d0oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`0M0?ooo`00903oool00`3ookl0M0000000000600000003
04RLg`3oool0oooo03@0oooo00<000000?ooo`3oool02`3oool200000380oooo00<000000?ooo`3o
ool0H@3oool2000007P0oooo00<000000?ooo`3oool0G03oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01d0oooo000S0?ooo`030?noM0000000000000L0000000<0001d0;ooo`3oool0
=03oool00`000000oooo0?ooo`090?ooo`800000=03oool00`000000oooo0?ooo`1N0?ooo`<00000
NP3oool00`000000oooo0?ooo`1K0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0
7@3oool002H0oooo00<0_gAd0;ooo`3oool00`3oool00`3ookl0M4RL0=ooo`0e0?ooo`030000003o
ool0oooo00L0oooo0P00000f0?ooo`030000003oool0oooo05/0oooo0`00001m0?ooo`030000003o
ool0oooo05X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0M0?ooo`009P3oool0
0`2oM7@0_ooo0?ooo`030?ooo`030;mdB02Lgol0oooo03D0oooo00<000000?ooo`3oool0?`3oool0
0`000000oooo0?ooo`1I0?ooo`800000P03oool00`000000oooo0?ooo`1I0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool07@3oool006@0oooo0`00000o0?ooo`030000003oool0oooo
05L0oooo0P0000220?ooo`030000003oool0oooo05L0oooo0P00000M0?ooo`8000007`3oool002L0
oooo00<0oooO09a8000000000P0000000`1d_ol0oooo0?ooo`0e0?ooo`030000003oool0oooo03l0
oooo00<000000?ooo`3oool0E@3oool2000008@0oooo00<000000?ooo`3oool0EP3oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo01d0oooo000V0?ooo`030?oo_`1d0000000000@00000
00<0B9cO0?ooo`3oool0=03oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo0580oooo
0`0000260?ooo`030000003oool0oooo05D0oooo00<000000?ooo`3oool07`3oool00`000000oooo
0?ooo`0M0?ooo`009P3oool00`3OW4P0B9cO0?ooo`020?ooo`040?noM0000000001d0;ooocD0oooo
00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`1?0?ooo`<00000R@3oool00`000000oooo
0?ooo`1C0?ooo`8000008P3oool00`000000oooo0?ooo`0M0?ooo`009P3oool00`2oM7@0_ooo0?oo
o`030?ooo`030=nLB00007@0_ooo03D0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?oo
o`1=0?ooo`800000S03oool00`000000oooo0?ooo`1B0?ooo`030000003oool0oooo0280oooo00<0
00000?ooo`3oool07@3oool002H0oooo00<0gia804RLg`3oool00`3oool00`2oM00007Bo0?ooo`0e
0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0B`3oool2000008h0oooo00<00000
0?ooo`3oool0D@3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo01d0oooo000V0?oo
o`070?oog`2LB000001d0;ooo`3OW4P0000004RLg`0g0?ooo`<00000?`3oool00`000000oooo0?oo
o`190?ooo`800000T03oool00`000000oooo0?ooo`1@0?ooo`030000003oool0oooo02<0oooo0P00
000O0?ooo`00I03oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo04L0oooo0P00002B
0?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0M
0?ooo`008@3oool00`3o_i`0W=oo0?ooo`090?ooo`030=nLW02oool0oooo01H0oooo00<0_g@00000
00000000100000002@18W=l0oooo0?ooo`3o_g@0000007Boo`3oool0oooO09a800030000000307Bo
o`3oool0oooo00P0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`150?ooo`800000
U03oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0
7@3oool00240oooo00<0_g@00000000000002@0000000`0007@0_ooo0?ooo`0G0?ooo`030=nLB000
0000000000<0000000X004RL0=ooo`3oool0okmd0000001d_ol0oooo0?oog`2LB7@0_ooo0P3oool0
0`2oM000M;oo0?ooo`080?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0@`3oool2
000009H0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0
oooo01d0oooo000P0?ooo`030?oo_`1d0000000000X0000000<0001d0;ooo`3oool0603oool00`3O
W4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo00T0oooo00<0oooO09a8M02oool0203oool00`00
0000oooo0?ooo`0o0?ooo`030000003oool0oooo0440oooo0P00002H0?ooo`030000003oool0oooo
04/0oooo0P00000Z0?ooo`030000003oool0oooo01d0oooo000W0?ooo`030=nLW02oool0oooo00<0
oooo00<0gibL0;ooo`3oool06@3oool00`3OW4P0M;oo0?ooo`0>0?ooo`030;md001d_ol0oooo00L0
oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0n0?ooo`<00000VP3oool00`000000
oooo0?ooo`1:0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool07@3oool002H0oooo
00<0okmd04RLg`3oool08@3oool00`2oM4P0W=oo0?ooo`0<0?ooo`030?oog`2LB000B9cO00P0oooo
00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0k0?ooo`<00000W@3oool00`000000oooo
0?ooo`190?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool07@3oool002H0oooo00<0
_g@0001d_`3oool0103oool00`3OW9`0_ooo0?ooo`0J0?ooo`030?oog`2LB7@0_ooo00`0oooo00<0
gia80000001d_ol0203oool4000003h0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?oo
o`2N0?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3oool0:P3oool3000001l0oooo000V
0?ooo`030=nLB0000000000000@0000000<0001d0;ooo`3oool06`3oool00`3o_g@007Bo0?ooo`07
0?ooo`070=oOg`3oool0oono07@000000000001d0;ooo`080?ooo`030000003oool0oooo03l0oooo
00<000000?ooo`3oool0>03oool200000:40oooo00<000000?ooo`3oool0A`3oool00`000000oooo
0?ooo`0]0?ooo`030000003oool0oooo01d0oooo000V0?ooo`030?oog`2LB000000000@0000000<0
001d0;ooo`3oool06`3oool00`3ookl0M01809cOo`070?ooo`060=nLB000000000000000000004P0
W=oo2@3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3o
ool0X@3oool00`000000oooo0?ooo`160?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3o
ool07@3oool002d0oooo00<0gibL0;ooo`3oool06`3oool00`3ookl0M00004RLg`070?ooo`040?oo
_`1d0000000004RLg`/0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0e0?ooo`80
0000Y03oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3o
ool07@3oool004H0oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0B00004RLg`080?ooo`030;md
001d_ol0oooo00/0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0c0?ooo`800000
YP3oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0
7@3oool004H0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`P0oooo00H0omnL04P0
0000000000000000B02Lgol80?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0<@3o
ool200000:P0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`0a0?ooo`030000003o
ool0oooo01d0oooo00170?ooo`060?oog`2LB00000000000000007@0_ooo2P3oool01@2oM0000000
000000000000B9cO00P0oooo0`00000o0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3o
ool0Z03oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo0340oooo0P00000O0?ooo`00
I03oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo02h0oooo0P00002[0?ooo`030000
003oool0oooo0440oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0M0?ooo`00I03o
ool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo02`0oooo0P00002]0?ooo`030000003o
ool0oooo0400oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0M0?ooo`00I03oool0
0`000000oooo0?ooo`0o0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0[@3oool0
0`000000oooo0?ooo`0o0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool07@3oool0
06@0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo
0:h0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo
01d0oooo001T0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0:@3oool00`000000
oooo0?ooo`2_0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool0=`3oool00`000000
oooo0?ooo`0M0?ooo`00I03oool3000003l0oooo00<000000?ooo`3oool0:03oool00`000000oooo
0?ooo`2`0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0=`3oool2000001l0oooo
001T0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?oo
o`2a0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?oo
o`0M0?ooo`00I03oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo02D0oooo0P00002d
0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0M
0?ooo`00I03oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo02@0oooo00<000000?oo
o`3oool0]03oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo03/0oooo00<000000?oo
o`3oool07@3oool006@0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0S0?ooo`03
0000003oool0oooo0;D0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`0l0?ooo`03
0000003oool0oooo01d0oooo001T0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0
8P3oool00`000000oooo0?ooo`2f0?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3oool0
?03oool00`000000oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?ooo`0o0?ooo`030000003o
ool0oooo0240oooo00<000000?ooo`3oool0]`3oool00`000000oooo0?ooo`0g0?ooo`030000003o
ool0oooo03d0oooo00<000000?ooo`3oool07@3oool006@0oooo0`00000o0?ooo`030000003oool0
oooo0200oooo00<000000?ooo`3oool0^03oool00`000000oooo0?ooo`0f0?ooo`030000003oool0
oooo03d0oooo0P00000O0?ooo`00I03oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo
01l0oooo00<000000?ooo`3oool0^@3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo
03l0oooo00<000000?ooo`3oool07@3oool006@0oooo00<000000?ooo`3oool0?`3oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo0;X0oooo00<000000?ooo`3oool0=03oool00`000000
oooo0?ooo`100?ooo`030000003oool0oooo01d0oooo001T0?ooo`030000003oool0oooo03l0oooo
00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`2k0?ooo`030000003oool0oooo03@0oooo
00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?oo
o`0o0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0_03oool00`000000oooo0?oo
o`0c0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool07@3oool006@0oooo00<00000
0?ooo`3oool0?`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0;d0oooo00<00000
0?ooo`3oool0<P3oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo01d0oooo001T0?oo
o`<00000?`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0;d0oooo00<000000?oo
o`3oool0<@3oool00`000000oooo0?ooo`120?ooo`8000007`3oool005<0oooo00<0oooO09a80000
00000P0000000`1d_ol0oooo0?ooo`090?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`2n0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3o
ool0@`3oool00`000000oooo0?ooo`0M0?ooo`00D`3oool00`3ooml0W4Qd0;ooo`020?ooo`030;md
001d_ol0oooo00T0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo0;l0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`140?ooo`030000
003oool0oooo01d0oooo001H0?ooo`030?oog`2LB7@0_ooo00T0oooo00<000000?ooo`3oool0?`3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0;l0oooo00<000000?ooo`3oool0;`3o
ool00`000000oooo0?ooo`150?ooo`030000003oool0oooo01d0oooo001I0?ooo`030;md001d_ol0
oooo00P0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
oooo0<00oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`150?ooo`030000003oool0
oooo01d0oooo001H0?ooo`030?oog`2LB000B9cO00T0oooo00<000000?ooo`3oool0?`3oool00`00
0000oooo0?ooo`0G0?ooo`030000003oool0oooo0<40oooo00<000000?ooo`3oool0;P3oool00`00
0000oooo0?ooo`160?ooo`030000003oool0oooo01d0oooo001H0?ooo`030=nLB0000000M;oo00T0
oooo1000000n0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0`@3oool00`000000
oooo0?ooo`0]0?ooo`030000003oool0oooo04D0oooo0`00000O0?ooo`00E03oool01`3Ogml0oooo
0?oo_`1d000000000000M02oool02@3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool0`P3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo
04L0oooo00<000000?ooo`3oool07@3oool005@0oooo00H0gia800000000000000000000B02Lgol:
0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`32
0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`0M
0?ooo`00E03oool0103ookl0M00000000018W=l<0?ooo`030000003oool0oooo03l0oooo00<00000
0?ooo`3oool05@3oool00`000000oooo0?ooo`330?ooo`030000003oool0oooo02/0oooo00<00000
0?ooo`3oool0B@3oool00`000000oooo0?ooo`0M0?ooo`00E@3oool00`2oM000M;oo0?ooo`0<0?oo
o`030000003oool0oooo03l0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`330?oo
o`030000003oool0oooo02/0oooo00<000000?ooo`3oool0B@3oool00`000000oooo0?ooo`0M0?oo
o`00E@3oool01P3ogi`0B000000000000000001809cOo`T0oooo00<000000?ooo`3oool0?`3oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0<@0oooo00<000000?ooo`3oool0:P3oool0
0`000000oooo0?ooo`1:0?ooo`030000003oool0oooo01d0oooo001F0?ooo`050;md000000000000
00000018W=l02@3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool0a03oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo04/0oooo00<00000
0?ooo`3oool07@3oool006@0oooo0`00000o0?ooo`030000003oool0oooo01@0oooo00<000000?oo
o`3oool0a03oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo04X0oooo0P00000O0?oo
o`00I03oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3o
ool0a@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo04`0oooo00<000000?ooo`3o
ool07@3oool006@0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo0<D0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`1<0?ooo`030000
003oool0oooo01d0oooo001T0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool04P3o
ool00`000000oooo0?ooo`360?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0C@3o
ool00`000000oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?ooo`0o0?ooo`030000003oool0
oooo0180oooo00<000000?ooo`3oool0aP3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0
oooo04d0oooo00<000000?ooo`3oool07@3oool006@0oooo00<000000?ooo`3oool0?`3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo0<H0oooo00<000000?ooo`3oool09P3oool00`00
0000oooo0?ooo`1>0?ooo`030000003oool0oooo01d0oooo001T0?ooo`<00000?`3oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo0<L0oooo00<000000?ooo`3oool09P3oool00`000000
oooo0?ooo`1=0?ooo`8000007`3oool006@0oooo00<000000?ooo`3oool0?`3oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo0<L0oooo00<000000?ooo`3oool09@3oool00`000000oooo
0?ooo`1?0?ooo`030000003oool0oooo01d0oooo001T0?ooo`030000003oool0oooo03l0oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`380?ooo`030000003oool0oooo02D0oooo00<0
00000?ooo`3oool0C`3oool00`000000oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?ooo`0o
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0b03oool00`000000oooo0?ooo`0T
0?ooo`030000003oool0oooo0500oooo00<000000?ooo`3oool07@3oool006@0oooo00<000000?oo
o`3oool0?`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0<P0oooo00<000000?oo
o`3oool0903oool00`000000oooo0?ooo`1@0?ooo`030000003oool0oooo01d0oooo001T0?ooo`03
0000003oool0oooo03l0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`390?ooo`03
0000003oool0oooo02<0oooo00<000000?ooo`3oool0D@3oool00`000000oooo0?ooo`0M0?ooo`00
I03oool3000003l0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`390?ooo`030000
003oool0oooo02<0oooo00<000000?ooo`3oool0D03oool2000001l0oooo001T0?ooo`030000003o
ool0oooo03l0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`390?ooo`030000003o
ool0oooo0280oooo00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`0M0?ooo`00I03oool0
0`000000oooo0?ooo`0o0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0bP3oool0
0`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0580oooo00<000000?ooo`3oool07@3oool0
06@0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
0<X0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`1B0?ooo`030000003oool0oooo
01d0oooo001T0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`3:0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0D`3oool00`000000
oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool0bP3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo05<0oooo
00<000000?ooo`3oool07@3oool006@0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo0</0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`1C0?ooo`030000003oool0oooo01d0oooo001T0?ooo`<00000?`3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo0</0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`1C
0?ooo`8000007`3oool004T0oooo00<0gia804RLg`3oool0103oool01P3o_g@0000007Boo`3oool0
oooO09a800<0000000<0M;oo0?ooo`3oool0203oool00`000000oooo0?ooo`0o0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool0b`3oool00`000000oooo0?ooo`0P0?ooo`030000003o
ool0oooo05@0oooo00<000000?ooo`3oool07@3oool004T0oooo00<0omnL04Q8W03Oool0103oool0
1`3o_g@0000007Boo`3oool0oooO09a8M02oool00P3oool00`2oM000M;oo0?ooo`080?ooo`030000
003oool0oooo03l0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`3;0?ooo`030000
003oool0oooo01l0oooo00<000000?ooo`3oool0E@3oool00`000000oooo0?ooo`0M0?ooo`00B@3o
ool00`3ooml0W4Q809cOo`0=0?ooo`030?oog`2LB7@0_ooo00P0oooo00<000000?ooo`3oool0?`3o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0<`0oooo00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`1E0?ooo`030000003oool0oooo01d0oooo001:0?ooo`030=nLB018W=l0
oooo00d0oooo00<0_g@007Boo`3oool01`3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool0c03oool00`000000oooo0?ooo`0O0?ooo`030000003oool0
oooo05D0oooo00<000000?ooo`3oool07@3oool004X0oooo00<0omnL04Q8W03Oool0303oool00`3o
oml0W4P004RLg`080?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`3<0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0EP3oool00`00
0000oooo0?ooo`0M0?ooo`00BP3oool00`3ooml0W4Q809cOo`0<0?ooo`030=nLB0000000M;oo00P0
oooo1000000n0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0c03oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo05@0oooo0`00000O0?ooo`00B`3oool00`3OW4P007Bo
0?ooo`070?ooo`070=oOg`3oool0oono07@000000000001d0;ooo`080?ooo`030000003oool0oooo
03l0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`3<0?ooo`030000003oool0oooo
01h0oooo00<000000?ooo`3oool0EP3oool00`000000oooo0?ooo`0M0?ooo`00B`3oool00`3ookl0
M01d0;ooo`070?ooo`060=nLB000000000000000000004P0W=oo2@3oool00`000000oooo0?ooo`0o
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0c@3oool00`000000oooo0?ooo`0M
0?ooo`030000003oool0oooo05L0oooo00<000000?ooo`3oool07@3oool004H0oooo00<0omoO0=oo
o`3oool00`3oool00`2oM000M;oo0?ooo`060?ooo`040?oo_`1d0000000004RLg`/0oooo00<00000
0?ooo`3oool0?`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0<d0oooo00<00000
0?ooo`3oool07@3oool00`000000oooo0?ooo`1G0?ooo`030000003oool0oooo01d0oooo00160?oo
o`030?oo_`1dB9`0gooo00<0oooo00<0okmd001d_`3oool01`3oool00`2oM000M;oo0?ooo`0;0?oo
o`030000003oool0oooo03l0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`3=0?oo
o`030000003oool0oooo01d0oooo00<000000?ooo`3oool0E`3oool00`000000oooo0?ooo`0M0?oo
o`00A`3oool00`2oM000000000000003000000030000M02oool0oooo00H0oooo00H0omnL04P00000
000000000000B02Lgol80?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`3=0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0F03oool0
0`000000oooo0?ooo`0M0?ooo`00A`3oool00`3o_g@00000000000040000000307Boo`3oool0oooo
00H0oooo00D0_g@0000000000000000004RLg`080?ooo`<00000?`3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo0<d0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`1G
0?ooo`8000007`3oool006@0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo0<d0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`1H0?oo
o`030000003oool0oooo01d0oooo001T0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`3>0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3o
ool0F03oool00`000000oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?ooo`0o0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool0cP3oool00`000000oooo0?ooo`0K0?ooo`030000
003oool0oooo05T0oooo00<000000?ooo`3oool07@3oool006@0oooo00<000000?ooo`3oool0?`3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0<h0oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`1I0?ooo`030000003oool0oooo01d0oooo001T0?ooo`030000003oool0
oooo03l0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3>0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool0F@3oool00`000000oooo0?ooo`0M0?ooo`00I03oool00`00
0000oooo0?ooo`0o0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0cP3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo05X0oooo00<000000?ooo`3oool07@3oool006@0
oooo0`00000o0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0cP3oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo05T0oooo0P00000O0?ooo`00I03oool00`000000oooo
0?ooo`0o0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0c`3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo05X0oooo00<000000?ooo`3oool07@3oool006@0oooo00<0
00000?ooo`3oool0?`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0<l0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`1J0?ooo`030000003oool0oooo01d0oooo001T
0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3?
0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0F`3oool00`000000oooo0?ooo`0M
0?ooo`00I03oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo00T0oooo00<000000?oo
o`3oool0c`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo05/0oooo00<000000?oo
o`3oool07@3oool006@0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo0<l0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`1K0?ooo`03
0000003oool0oooo01d0oooo001T0?ooo`<00000?`3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo0<l0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`1J0?ooo`800000
7`3oool006@0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo0<l0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`1L0?ooo`030000003o
ool0oooo01d0oooo001T0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0203oool0
0`000000oooo0?ooo`3@0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0G03oool0
0`000000oooo0?ooo`0M0?ooo`00I03oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo
00P0oooo00<000000?ooo`3oool0d03oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
05`0oooo00<000000?ooo`3oool07@3oool006@0oooo00<000000?ooo`3oool0?`3oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo0=00oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`1L0?ooo`030000003oool0oooo01d0oooo001T0?ooo`030000003oool0oooo03l0oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3@0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool0G@3oool00`000000oooo0?ooo`0M0?ooo`00I03oool3000003l0oooo00<0
00000?ooo`3oool0203oool00`000000oooo0?ooo`3@0?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool0G03oool2000001l0oooo001T0?ooo`030000003oool0oooo03l0oooo00<00000
0?ooo`3oool0203oool00`000000oooo0?ooo`3@0?ooo`030000003oool0oooo01L0oooo00<00000
0?ooo`3oool0G@3oool00`000000oooo0?ooo`0M0?ooo`00C03oool01P2oM0000000000000000000
04RL0=ooo`<0oooo00D0omnL04P000000000001d0;ooo`0:0?ooo`030000003oool0oooo03l0oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3@0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool0G@3oool00`000000oooo0?ooo`0M0?ooo`00C@3oool00`3ogi`0B00007Bo
o`040?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`090?ooo`030000003oool0oooo03l0
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3A0?ooo`030000003oool0oooo01H0
oooo00<000000?ooo`3oool0GP3oool00`000000oooo0?ooo`0M0?ooo`00C@3oool00`3ogi`0B000
07Boo`040?ooo`030;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool0203oool00`000000oooo
0?ooo`0U0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool0>@3oool00`000000oooo
0?ooo`0i0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool05P3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`0M0?ooo`00C@3oool00`3ogi`0B00007Boo`030?ooo`030?oo_`1d0000B9cO00<0oooo00<0
gia80000M02oool0203oool01@000000oooo0?ooo`3oool0000000/0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo000000090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`050?ooo`050000003oool0oooo0?ooo`000000303oool00`000000
oooo0?ooo`050?ooo`050000003oool0oooo0?ooo`0000002`3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo01d0oooo001=0?ooo`030?oOW0180000M;oo00<0oooo00<0omnL04P0001d
_ol00`3oool00`3o_g@0001809cOo`080?ooo`050000003oool0oooo0?ooo`0000002`3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0000000T0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00D0oooo00D000000?ooo`3oool0oooo0000000<
0?ooo`030000003oool0oooo00D0oooo00D000000?ooo`3oool0oooo0000000;0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool07@3oool004d0oooo00<0omnL04P0001d_ol00`3oool0
0`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00P0ooooo`00002L000001l0oooo001=0?oo
o`030?oOW0180000M;oo00<0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`3o0?oo
ol<0oooo001=0?ooo`030?oOW0180000M;oo00<0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0
001809cOo`3o0?oool<0oooo001=0?ooo`030?oOW0180000M;oo00<0oooo00<0oono07@00018W=l0
0`3oool00`3OW4P0001d0;ooo`3o0?oool<0oooo001=0?ooo`030?oOW0180000M;oo00@0oooo00<0
gia80018W03Oool00P3oool00`2oM00007Bo0?ooo`3o0?oool<0oooo001;0?ooo`050?oog`2LB000
00000000001d_ol0103oool01`3ookl0M00007Boo`3oool0okmd0000M02oool0o`3oooo40?ooo`00
C@3oool00`3o_g@0000007Boo`050?ooo`050?oo_`1d000000000000M02oool0o`3oooo50?ooo`00
o`3ooooo0?ooob00oooo003n0?ooo`030?oog`2LB000000000<0000000<0B9cO0?ooo`3oool0o`3o
oolH0?ooo`00oP3oool00`2oM00004RL0=ooo`030?ooo`030=nLW02oool0oooo0?l0oooo603oool0
0?h0oooo00<0oooO09adW03Oool00`3oool00`3ookl0W9cO0?ooo`3o0?oooaP0oooo003o0?ooo`03
0;md00000000000000<0000000<0B9cO0?ooo`3oool0o`3ooolG0?ooo`00a@3oool01P3OW4P00000
0000000004P0W=oO09a80080000000<0001d0;oog`2LB0000P0000000`00M;l0oooO09a800020000
00090018W02LM0000000001d_`3oool0omnL04P000000000B9cO0080oooo00L0_g@0000000000000
001809cOo`3oool0gia80080000000<0001809cOg`2LB0000P0000001P00M;l0gia8000000000000
001d07Ad0080000000<0001d0;oog`2LB0000P0000000`00B9`0gono07@00004000000030000M02o
ool0oooo00L0oooo00D0_g@0000000000000001809cOo`030?ooo`050=nLB000000000000000001d
_ol00P3oool01P2oM00000000000000007@0_ooO09a80080000000@004RL0=ooo`3ooml0W4P00P00
00001@0004P0B4P00018W03Oool0gia80080000000L0001d0;ooo`3ogi`0B00000000018W=l0gia8
0080000000@0001d0;ooo`3oool0gia80`000000101d_ol0oooo0?ooo`2oM002000000030000M02o
oml0W4P00080000000<004RL0=ooo`3oool0103oool00`3ogi`0B000000000020000000407Boo`3o
ool0oooo0=nLB0<0000000@0M;oo0?ooo`3oool0_g@00P0000000`0004P0W=oo0?ooo`040?ooo`06
0=nLB000000000000000B02Lgml0W4P00P0000000`0007@0_ooO09a8000200000003001d_`3oool0
oooo00`0oooo00D0omnL04P000000000001d0;ooo`030?ooo`060?noM0000000M;oo0?ooo`3ooml0
W4P00`0000000`1d_ol0oooo0?ooo`210?ooo`00aP3oool00`3OW4P0001d0;ooo`020?ooo`0U0;md
0000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd0000001d_ol0oooo0?oOW018000007Bo0?no
_`2oool0gia80000M02oool0gkno0=ooo`2oM00000000018W03Oool0oooO09a8M02oool0oooo0=nL
B00007@0_ooo0?ooo`3ooml0W4P0001d_`020?ooo`0;0=nLB00007@0_ooo0?ooo`3OW4P004RL0=oo
o`3oool0oooO09a80000M;l00P3oool00`2oM4P0W=oo0?ooo`0;0?ooo`0?0;md0000000004RL0=oo
o`3ooml0W4Qd0;ooo`3oool0gia800000018W=l0oooo0?oog`2LB000B9cO0080oooo02H0gia80018
W03Oool0oooo0?oog`2LB00007Bo0?ooo`3ooml0W4P00000001d_ol0oooo0?oo_`1d000000000000
M02oool0gia80000M02oool0oooo0=nLB00007@0_ooo0=no_`3Oool0gia80000M02oool0oooo0=nL
B0000000B9cO0?ooo`3ooml0W4P004RLg`80oooo00L0gia80018W03Oool0oooo0?oog`2LB00007Bo
00P0oooo00<0oooO09a80000M;l00`3oool01`3OW4P0000004RLg`3oool0oooO09a80018W=l00P3o
ool00`3OW4P0001d0;ooo`070?ooo`030=nLB00007@0_ooo0080oooo00L0_g@00018W03Oool0oooo
0?oog`2LB00007Bo00h0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo0080oooo00L0okmd
0000001d_ol0oooo0?oog`2LB7@0_ooo0080oooo00<0_g@007Boo`3oool0P@3oool00<H0oooo00<0
gia80000M02oool00P3oool03`2oM00004RL0=ooo`3oool0oooO09a80000M;l0oooo0?noM00004P0
W=oo0?ooo`3ooml0W4P0001d_`020?ooo`070=nLB000B9`0gooo0?ooo`3ogi`0B000001d_`030?oo
o`0:0?oOW02L_ol0oooo0=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`80oooo00/0gia80000M02o
ool0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`050?oo_`1d0000000000000000
M;l0203oool00`3ogi`0B000001d_`030?ooo`050?oOW02L_ol0omnL04P00000M;l00`3oool03`3o
_g@0001d0;ooo`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3ogi`0B00004RLg`02
0?ooo`030?oog`2LB00007Bo0080oooo00L0gia80000M02oool0oooo0=nLB000B9`0gooo0080oooo
00H0gia80000M02oool0omnL04P00000M;l30?ooo`0;0?noM00007@0_ooo0?ooo`3OW4P004RL0=oo
o`3oool0oooO09a80000M;l0203oool00`3ooml0W4P0001d_`020?ooo`030?oOW018000007Bo00<0
oooo00L0okmd0000M02oool0oooo0=nLB00007@0_ooo00L0oooo00<0gia80000M02oool00P3oool0
1`2oM00004RL0=ooo`3oool0oooO09a80000M;l00P3oool00`3ooml0W4P0000000070000000504RL
g`3oool0_g@00018W03Oool00P3oool00`2oM00007Bo0?ooo`0:0?ooo`030?oog`2LB7@0_ooo0840
oooo00360?ooo`030=nLB00007@0_ooo0080oooo00L0_g@00018W03Oool0oooo0?oog`2LB00007Bo
0080oooo00H0okmd00000000M;l0oooO09a80000M;l20?ooo`070=nLB000B9`0gooo0?ooo`3o_g@0
001809cOo`060?ooo`070=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`020?ooo`0;0=nLB00007@0
_ooo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l00P3oool01P3OW4P0001d0;ooo`3ookl0
M00004RLg`L0oooo00<0okmd0000B02Lgol01@3oool00`3o_g@0000007Boo`030?ooo`0?0?noM000
0000M;oo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?noM0000000M;oo0080oooo
00<0oooO09a80000M;l00P3oool01`3OW4P0001d0;ooo`3oool0gia80018W03Oool00P3oool01P3O
W4P0001d0;ooo`3o_g@0000007Boo`<0oooo00/0okmd0000001d_ol0oooo0=nLB000B9`0gooo0?oo
o`3ooml0W4P0001d_`080?ooo`030?oog`2LB00007Bo0080oooo00<0okmd0000001d_ol00`3oool0
1`3o_g@0000007Boo`3oool0gia80000M02oool01`3oool00`3OW4P0001d0;ooo`020?ooo`070;md
0000B9`0gooo0?ooo`3ooml0W4P0001d_`0=0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000
M02oool02`3oool00`2oM000M;oo0?ooo`200?ooo`00aP3oool00`3OW4P0001d0;ooo`020?ooo`07
0;md0000B9`0gooo0?ooo`3ooml0W4P0001d_`040?ooo`040?noM00000000000001d_`80oooo00L0
gia80018W03Oool0oooo0?noM00007@0_ooo00H0oooo00L0gia80000M02oool0oooo0?oog`2LB000
07Bo0080oooo01<0gia80000M02oool0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0
oono07@00018W=l0oooo0?oog`2LB00007Bo00L0oooo00<0okmd0000M02oool01@3oool00`3o_g@0
000007Boo`030?ooo`0?0?noM0000000M;oo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0
oooo0?noM0000000M;oo0080oooo00<0oooO09a80000M;l00P3oool01`3OW4P0001d0;ooo`3oool0
gia80018W03Oool00P3oool01P3OW4P0001d0;ooo`3o_g@0000007Boo`<0oooo00/0okmd0000001d
_ol0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`080?ooo`030?oog`2LB00007Bo0080oooo
00<0okmd0000001d_ol00`3oool01`3o_g@0000007Boo`3oool0gia80000M02oool01`3oool00`3O
W4P0001d0;ooo`020?ooo`070;md0000B9`0gooo0?ooo`3ooml0W4P0001d_`0=0?ooo`030?oOW018
0000M;oo00<0oooo00<0okmd0000B02Lgol02P3oool00`3ooml0W4P004RLg`210?ooo`00aP3oool0
0`3OW4P0001809cOo`020?ooo`0?0;md000007@0_ooo0?ooo`3ooml0W4P0001d_`3oool0oono07@0
001d_ol0oooo0?oog`2LB00007Bo0080oooo00L0gia80018W03Oool0oooo0?oOW01804P0W=oo0080
oooo00/0okmd0000B02Lgol0oooo0=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`020?ooo`0C0=nL
B00007@0_ooo0?ooo`2oM000001d0;ooo`3oool0oooO09a80000M;l0oooo0?oo_`1d0000B9cO0?oo
o`3ooml0W4P004RLg`070?ooo`030?oOW01804P0W=oo0080oooo00H0okmd0000B02Lgol0oono07@0
001d_ol30?ooo`0?0;md000004P0W=oo0?ooo`2oM000001d0;ooo`3oool0oooO09a80000M;l0oooo
0?oo_`1d0000B9cO0080oooo00<0oooO09a80000M;l00P3oool01`3OW4P0001d0;ooo`3oool0gia8
0018W03Oool00P3oool01P3OW4P0001d0;ooo`3ookl0M00007Boo`<0oooo00/0_g@00000B02Lgol0
oooo0;md000007@0_ooo0?ooo`3ooml0W4P0001d_`080?ooo`030?oog`2LB00007Bo0080oooo00<0
oono07@0001d_ol00`3oool01`2oM000001809cOo`3oool0gia80000B02Lgol01`3oool00`3OW4P0
001809cOo`020?ooo`070;md000007@0_ooo0?ooo`3ooml0W4P0001d_`0=0?ooo`030?oOW0180000
M;oo00<0oooo00<0okmd0000B02Lgol02P3oool00`3OW4P0000007Boo`210?ooo`00a03oool05P3o
oml0W4P00000000004P0B4Q809cOo`3ookl0M000001dM01d07@0_ooo0?oo_`1d0000B9cO0?ooo`3o
okl0M00004RLg`3oool0oono07@00000M;l20?ooo`030=nLB000B9`0gooo0080oooo01`0_g@00018
W03Oool0oooO09a8000007@0_ooo0?ooo`3OW4P000000000B02Lgol0oono07@00018W=l0oooO09a8
00000000001d0;oog`2LB00000000018B01804P0W=oo0?oo_`1d0000B9cO0P3oool0102oM00007Bo
0?ooo`3o_g@2000000030018W03Oool0oooo00D0oooo01P0_g@00018W03Oool0oooO09a8000007@0
_ooo0?ooo`3OW4P004RL0=ooo`3oool0okmd00000000M;l0oooO09a80000000004Q804P0B02Lgol0
oono07@00018W=l20?ooo`0V0?noM00004P0W=oo0?ooo`3o_g@00000001d_`3ooml0W4P000000000
07@0_ooo0?ooo`3OW4P004RL0=ooo`3ooml0W4P00000000007@0_ooo0?ooo`3OW4P004RL0=ooo`3o
ool0okmd00000000M;l0oooO09a80000000004Q804P0B02Lgol0oono07@00018W=l80?ooo`030?oo
g`2LB00007Bo00<0oooo00l0gia80018W03Oool0oooo0?noM000000007Bo0?oog`2LB00000000000
B018M9`0gkmd0000M02oool00P3oool03P3ooml0W4P00000000004P0B4Q809cOo`3ookl0M000001d
M01d07@0_ooo0?oo_`1d0000B9cO0P3oool00`3ooml0W4P0000000070000000404RLg`3ogi`0B000
07Boo`<0oooo00<0okmd0000B02Lgol01P3oool01`3Ogml0oooo0?oo_`1d000000000000M02oool0
P@3oool00<H0oooo00`0_g@00000M02ooml0W4P00000000004P0W=oo0?oo_`1d000000000000M02o
ool20?ooo`080?oo_`1d000000000000000007@0_ooo0?ooo`2oM003000000040018W03Oool0oooo
0=nLB080000000/0001809cOo`3oool0oooo0=nLB00007@0_ooO09a800000000001d0;ooo`020?oo
o`0;0;md000007@0_ooo0?ooo`2oM00004RL0=oog`2LB00000000000M02oool00P3oool01@3ooml0
W4P0000000000000M;oo00X0oooo00D0gia8000000000000001809cOo`030?ooo`050?noM0000000
000000000018W=l00`3oool01`2oM00004RL0=oog`2LB00000000000M02oool00`3oool00`3o_g@0
00000000000200000007001d_`3oool0oooo0;md000007@0_ooo0;md0003000000050018W03Oool0
_g@00000M02oool00P3oool01@3o_g@00000000000000000B9cO00<0oooo00L0_g@00018W03Ooml0
W4P00000000007@0_ooo00L0oooo00<0okmd0000000000000P0000001000B9`0gooo0?ooo`3o_g@3
0000000:04RLg`3oool0oooo0?ooo`2oM000001d0;oOW0180000001d0;ooo`@0oooo00`0_g@00000
M02ooml0W4P00000000004P0W=oo0?oo_`1d000000000000M02oool=0?ooo`030?oOW0180000M;oo
00<0oooo00<0okmd0000B02Lgol01P3oool01P3OW4P00000000000000000001809cOoh80oooo003L
0?ooo`030;md0000B9`0gooo00X0oooo00<0gia80000M02oool0?`3oool00`3ooml0W4P0001d_`06
0?ooo`030;md0000B9`0gooo01h0oooo00<0oooO09a80000M;l0;`3oool00`3ookl0M00004RLg`03
0?ooo`030=nLB00007@0_ooo00H0oooo00@0oono07@000000000B9cOQ03oool00=`0oooo00<0oono
07A8W03Oool02P3oool00`3OW4P0001d0;ooo`0o0?ooo`030?oog`2LB00007Bo00H0oooo00<0oono
07A8W03Oool07`3oool00`2oM00007Bo0?ooo`0_0?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0
001d_`3oool01`3oool00`2oM000M;oo0?ooo`240?ooo`00g03oool00`3ooml0W7BL0=ooo`0:0?oo
o`030=nLB00007@0_ooo03l0oooo00<0oooO09a80000M;l01P3oool00`3ooml0W7BL0=ooo`0O0?oo
o`030=nLB000M;l0oooo02l0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00P0oooo00H0
omnL04P00000000000000000B02Lgon10?ooo`00i`3oool01@3ooml0W4P00000000007@0_ooo00H0
oooo00<0gia80000B02Lgol0=@3oool0103ookl0M00000000000M;l20?ooo`030=nLB00004P0W=oo
00H0oooo00<0gia80000B02Lgol06P3oool01`3ookl0M01d0;ooo`3ooml0W4P00018W03Oool0;03o
ool01@3ookl0M0000000000007@0_ooo00X0oooo00D0_g@0000000000000000004RLg`210?ooo`00
j@3oool00`2oM000001d0;ooo`060?ooo`030=nLB00004P0W=oo03H0oooo00<0oooO09a80000M;l0
0P3oool00`3OW4P0001809cOo`060?ooo`030=nLB00004P0W=oo01/0oooo00D0oono07@000000000
001809cOo`320?ooo`00o`3ooooo0?ooob00oooo003o0?ooool0oooo803oool00?l0ooooo`3ooolP
0?ooo`00o`3ooooo0?ooob00oooo0000\
\>"],
  ImageRangeCache->{{{0, 541.688}, {334.438, 0}} -> {1.66297, -14.1164, \
0.0249507, 0.118304}}]
}, Closed]],

Cell[TextData[{
  "However, matching at the center does not work particularly well for \
construction of the wave functions, especially for odd solutions.  \
Furthermore, it is also difficult to ensure that the ",
  StyleBox["FindRoot", "InlineInput",
    FontWeight->"Bold"],
  " will settle upon the desired solution.  Therefore, a certain amount of \
trial and error is needed with these algorithms to choose parameters which \
will produce the desired solutions.  The following choices produce the lowest \
four solutions for this case."
}], "Text"],

Cell[BoxData[
    \(\(solutions = {\[IndentingNewLine]ShootBSWF2[pot, 0.5, 3, 4, \(-5\), 5, 
            0.5, MaxIterations \[Rule] 50], 
          ShootBSWF2[pot, 0.5, 4.4, 4.5, \(-5\), 5, 0.5, 
            MaxIterations \[Rule] 50], \[IndentingNewLine]ShootBSWF2[pot, 
            0.5, 6, 7, \(-5\), 5, 0.5, 
            MaxIterations \[Rule] 50], \[IndentingNewLine]ShootBSWF2[pot, 
            0.5, 8, 9, \(-5\), 5, 0.5, 
            MaxIterations \[Rule] 50]};\)\)], "Input"],

Cell[BoxData[{
    \(\(plot1 = 
        PlotBSWF[solutions\[LeftDoubleBracket]1\[RightDoubleBracket], \(-5\), 
          5, PlotRange \[Rule] {All, {2.5, 5.5}}, 
          DisplayFunction \[Rule] Identity];\)\), "\n", 
    \(\(plot2 = 
        PlotBSWF[solutions\[LeftDoubleBracket]2\[RightDoubleBracket], \(-5\), 
          5, PlotRange \[Rule] {All, {2.5, 5.5}}, 
          DisplayFunction \[Rule] Identity];\)\), "\n", 
    \(plot3 = 
      PlotBSWF[solutions\[LeftDoubleBracket]3\[RightDoubleBracket], \(-5\), 
        5, PlotRange \[Rule] {All, {5.5, 7.5}}, 
        DisplayFunction \[Rule] Identity]; 
    plot4 = PlotBSWF[
        solutions\[LeftDoubleBracket]4\[RightDoubleBracket], \(-5\), 5, 
        PlotRange \[Rule] {All, {7.5, 9}}, 
        DisplayFunction \[Rule] Identity];\), "\n", 
    \(\(plots = {{plot1, plot2}, {plot3, plot4}};\)\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[GraphicsArray[plots], 
        DisplayFunction \[Rule] $DisplayFunction];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 0.0147151 0.47619 [
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0238095 0.0147151 0.477324 0.295003 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0952381 -1.69959 0.309017 [
[.11905 -0.0125 -9.15625 -15.6875 ]
[.11905 -0.0125 9.15625 0 ]
[.30952 -0.0125 -9.15625 -15.6875 ]
[.30952 -0.0125 9.15625 0 ]
[.5 -0.0125 -5 -15.6875 ]
[.5 -0.0125 5 0 ]
[.69048 -0.0125 -5 -15.6875 ]
[.69048 -0.0125 5 0 ]
[.88095 -0.0125 -5 -15.6875 ]
[.88095 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -5.0625 -31.375 ]
[.5 -0.0125 5.0625 -15.6875 ]
[-0.0125 .07725 -25 -7.84375 ]
[-0.0125 .07725 0 7.84375 ]
[-0.0125 .15451 -10 -7.84375 ]
[-0.0125 .15451 0 7.84375 ]
[-0.0125 .23176 -25 -7.84375 ]
[-0.0125 .23176 0 7.84375 ]
[-0.0125 .30902 -19 -7.84375 ]
[-0.0125 .30902 0 7.84375 ]
[-0.0125 .38627 -25 -7.84375 ]
[-0.0125 .38627 0 7.84375 ]
[-0.0125 .46353 -10 -7.84375 ]
[-0.0125 .46353 0 7.84375 ]
[-0.0125 .54078 -25 -7.84375 ]
[-0.0125 .54078 0 7.84375 ]
[-0.0125 .61803 -19 -7.84375 ]
[-0.0125 .61803 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -40.6875 -13.0313 ]
[-0.01875 .30902 -25 13.0313 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -95.6875 0 ]
[.5 .63053 95.6875 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.11905 0 m
.11905 .00625 L
s
gsave
.11905 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.30952 0 m
.30952 .00625 L
s
gsave
.30952 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.5 0 m
.5 .00625 L
s
gsave
.5 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.69048 0 m
.69048 .00625 L
s
gsave
.69048 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.88095 0 m
.88095 .00625 L
s
gsave
.88095 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.125 Mabswid
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.07143 0 m
.07143 .00375 L
s
.02381 0 m
.02381 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -66.0625 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
69.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .07725 m
.00625 .07725 L
s
gsave
-0.0125 .07725 -86 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5.75) show
1.000 setlinewidth
grestore
0 .15451 m
.00625 .15451 L
s
gsave
-0.0125 .15451 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
0 .23176 m
.00625 .23176 L
s
gsave
-0.0125 .23176 -86 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6.25) show
1.000 setlinewidth
grestore
0 .30902 m
.00625 .30902 L
s
gsave
-0.0125 .30902 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6.5) show
1.000 setlinewidth
grestore
0 .38627 m
.00625 .38627 L
s
gsave
-0.0125 .38627 -86 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6.75) show
1.000 setlinewidth
grestore
0 .46353 m
.00625 .46353 L
s
gsave
-0.0125 .46353 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7) show
1.000 setlinewidth
grestore
0 .54078 m
.00625 .54078 L
s
gsave
-0.0125 .54078 -86 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7.25) show
1.000 setlinewidth
grestore
0 .61803 m
.00625 .61803 L
s
gsave
-0.0125 .61803 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7.5) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .01545 m
.00375 .01545 L
s
0 .0309 m
.00375 .0309 L
s
0 .04635 m
.00375 .04635 L
s
0 .0618 m
.00375 .0618 L
s
0 .09271 m
.00375 .09271 L
s
0 .10816 m
.00375 .10816 L
s
0 .12361 m
.00375 .12361 L
s
0 .13906 m
.00375 .13906 L
s
0 .16996 m
.00375 .16996 L
s
0 .18541 m
.00375 .18541 L
s
0 .20086 m
.00375 .20086 L
s
0 .21631 m
.00375 .21631 L
s
0 .24721 m
.00375 .24721 L
s
0 .26266 m
.00375 .26266 L
s
0 .27812 m
.00375 .27812 L
s
0 .29357 m
.00375 .29357 L
s
0 .32447 m
.00375 .32447 L
s
0 .33992 m
.00375 .33992 L
s
0 .35537 m
.00375 .35537 L
s
0 .37082 m
.00375 .37082 L
s
0 .40172 m
.00375 .40172 L
s
0 .41717 m
.00375 .41717 L
s
0 .43262 m
.00375 .43262 L
s
0 .44807 m
.00375 .44807 L
s
0 .47898 m
.00375 .47898 L
s
0 .49443 m
.00375 .49443 L
s
0 .50988 m
.00375 .50988 L
s
0 .52533 m
.00375 .52533 L
s
0 .55623 m
.00375 .55623 L
s
0 .57168 m
.00375 .57168 L
s
0 .58713 m
.00375 .58713 L
s
0 .60258 m
.00375 .60258 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -106.875 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
74.0313 11.8438 translate
90 rotate
-74.0313 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(V) show
71.625 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(,) show
77.625 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(y) show
85.063 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.11905 .61178 m
.11905 .61803 L
s
.30952 .61178 m
.30952 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.69048 .61178 m
.69048 .61803 L
s
.88095 .61178 m
.88095 .61803 L
s
.125 Mabswid
.16667 .61428 m
.16667 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.2619 .61428 m
.2619 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.7381 .61428 m
.7381 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.07143 .61428 m
.07143 .61803 L
s
.02381 .61428 m
.02381 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.97619 .61428 m
.97619 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 0 m
1 0 L
s
.99375 .07725 m
1 .07725 L
s
.99375 .15451 m
1 .15451 L
s
.99375 .23176 m
1 .23176 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .38627 m
1 .38627 L
s
.99375 .46353 m
1 .46353 L
s
.99375 .54078 m
1 .54078 L
s
.125 Mabswid
.99625 .01545 m
1 .01545 L
s
.99625 .0309 m
1 .0309 L
s
.99625 .04635 m
1 .04635 L
s
.99625 .0618 m
1 .0618 L
s
.99625 .09271 m
1 .09271 L
s
.99625 .10816 m
1 .10816 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .13906 m
1 .13906 L
s
.99625 .16996 m
1 .16996 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .20086 m
1 .20086 L
s
.99625 .21631 m
1 .21631 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26266 m
1 .26266 L
s
.99625 .27812 m
1 .27812 L
s
.99625 .29357 m
1 .29357 L
s
.99625 .32447 m
1 .32447 L
s
.99625 .33992 m
1 .33992 L
s
.99625 .35537 m
1 .35537 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .40172 m
1 .40172 L
s
.99625 .41717 m
1 .41717 L
s
.99625 .43262 m
1 .43262 L
s
.99625 .44807 m
1 .44807 L
s
.99625 .47898 m
1 .47898 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .50988 m
1 .50988 L
s
.99625 .52533 m
1 .52533 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57168 m
1 .57168 L
s
.99625 .58713 m
1 .58713 L
s
.99625 .60258 m
1 .60258 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
.5 0 m
.5 .61803 L
s
gsave
.5 .63053 -156.688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Wave) show
93.875 15.813 moveto
(Function) show
139.313 15.813 moveto
(with) show
163.563 15.813 moveto
(Energy) show
200.563 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
211.375 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6.58621) show
250.375 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.23998 .61803 m
.26171 .23788 L
s
.26171 .23788 m
.27725 0 L
s
.72174 0 m
.74311 .31632 L
s
.74311 .31632 m
.76 .61803 L
s
0 g
.02381 .33566 m
.03279 .33572 L
.04262 .33581 L
.05288 .33592 L
.06244 .33607 L
.06792 .33619 L
.0731 .33631 L
.0828 .33663 L
.08823 .33686 L
.09324 .33711 L
.10458 .33787 L
.11425 .33878 L
.12331 .33992 L
.13324 .34158 L
.14393 .34395 L
.15404 .34691 L
.16506 .3511 L
.1756 .35621 L
.18515 .36194 L
.20483 .37742 L
.2258 .39974 L
.26587 .45395 L
.28424 .47729 L
.28928 .4826 L
.29468 .48754 L
.29934 .49107 L
.30442 .49402 L
.3072 .49521 L
.30842 .49563 L
.30977 .49602 L
.31096 .4963 L
.31223 .49652 L
.31294 .49661 L
.31358 .49668 L
.31485 .49674 L
.31599 .49674 L
.31706 .49668 L
.31823 .49655 L
.31948 .49633 L
.32079 .49602 L
.32203 .49564 L
.32436 .49472 L
.32691 .49338 L
.32962 .49158 L
.33446 .48734 L
.33968 .48132 L
.34542 .47296 L
.35558 .45382 L
.36487 .43188 L
.38576 .37092 L
Mistroke
.40639 .30416 L
.42552 .2467 L
.44405 .20184 L
.4544 .18258 L
.46377 .16884 L
.46866 .16303 L
.47382 .15789 L
.47828 .15423 L
.48305 .1511 L
.48574 .14969 L
.48819 .14862 L
.49069 .14774 L
.49299 .14712 L
.49429 .14684 L
.49567 .14661 L
.49634 .14653 L
.49706 .14645 L
.49785 .14638 L
.49859 .14634 L
.49923 .14631 L
.49994 .1463 L
.50067 .14631 L
.50135 .14633 L
.50258 .14641 L
.50389 .14655 L
.50512 .14674 L
.50648 .147 L
.50923 .14772 L
.51178 .14861 L
.51415 .14964 L
.51952 .15268 L
.52527 .15707 L
.53581 .16829 L
.54544 .18229 L
.5645 .22094 L
.58525 .27801 L
.60461 .33983 L
.626 .40684 L
.63573 .43341 L
.64623 .45762 L
.65614 .47542 L
.6609 .48208 L
.66523 .48704 L
.66996 .49127 L
.67256 .49307 L
.67502 .49443 L
.67729 .49541 L
.67939 .49607 L
.68049 .49633 L
.68169 .49654 L
Mistroke
.68295 .49669 L
.68413 .49675 L
.68541 .49674 L
.68613 .4967 L
.6868 .49665 L
.68803 .49649 L
.68934 .49624 L
.69063 .49592 L
.69184 .49556 L
.6941 .4947 L
.69925 .49199 L
.70482 .48796 L
.71434 .47885 L
.72478 .46644 L
.74585 .43752 L
.76701 .40868 L
.78629 .38615 L
.80629 .36803 L
.82522 .35577 L
.83488 .35112 L
.84514 .34718 L
.8539 .34452 L
.86357 .34221 L
.87367 .34038 L
.88293 .3391 L
.8931 .33807 L
.90379 .33728 L
.91335 .33679 L
.9186 .33658 L
.92355 .33641 L
.93318 .33616 L
.94192 .336 L
.95059 .33588 L
.95976 .33579 L
.97619 .33566 L
Mfstroke
[ .02 .04 ] 0 setdash
.02381 .33566 m
.06244 .33566 L
.10458 .33566 L
.14415 .33566 L
.18221 .33566 L
.22272 .33566 L
.26171 .33566 L
.30316 .33566 L
.34309 .33566 L
.3815 .33566 L
.42237 .33566 L
.46172 .33566 L
.49955 .33566 L
.53984 .33566 L
.57861 .33566 L
.61984 .33566 L
.65954 .33566 L
.69774 .33566 L
.73838 .33566 L
.77751 .33566 L
.81909 .33566 L
.85916 .33566 L
.89771 .33566 L
.93871 .33566 L
.97619 .33566 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.522676 0.0147151 0.97619 0.295003 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0952381 -3.09017 0.412023 [
[.11905 -0.0125 -9.15625 -15.6875 ]
[.11905 -0.0125 9.15625 0 ]
[.30952 -0.0125 -9.15625 -15.6875 ]
[.30952 -0.0125 9.15625 0 ]
[.5 -0.0125 -5 -15.6875 ]
[.5 -0.0125 5 0 ]
[.69048 -0.0125 -5 -15.6875 ]
[.69048 -0.0125 5 0 ]
[.88095 -0.0125 -5 -15.6875 ]
[.88095 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -5.0625 -31.375 ]
[.5 -0.0125 5.0625 -15.6875 ]
[-0.0125 .0412 -19 -7.84375 ]
[-0.0125 .0412 0 7.84375 ]
[-0.0125 .12361 -19 -7.84375 ]
[-0.0125 .12361 0 7.84375 ]
[-0.0125 .20601 -10 -7.84375 ]
[-0.0125 .20601 0 7.84375 ]
[-0.0125 .28842 -19 -7.84375 ]
[-0.0125 .28842 0 7.84375 ]
[-0.0125 .37082 -19 -7.84375 ]
[-0.0125 .37082 0 7.84375 ]
[-0.0125 .45322 -19 -7.84375 ]
[-0.0125 .45322 0 7.84375 ]
[-0.0125 .53563 -19 -7.84375 ]
[-0.0125 .53563 0 7.84375 ]
[-0.0125 .61803 -10 -7.84375 ]
[-0.0125 .61803 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -34.6875 -13.0313 ]
[-0.01875 .30902 -19 13.0313 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -95.6875 0 ]
[.5 .63053 95.6875 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.11905 0 m
.11905 .00625 L
s
gsave
.11905 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.30952 0 m
.30952 .00625 L
s
gsave
.30952 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.5 0 m
.5 .00625 L
s
gsave
.5 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.69048 0 m
.69048 .00625 L
s
gsave
.69048 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.88095 0 m
.88095 .00625 L
s
gsave
.88095 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.125 Mabswid
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.07143 0 m
.07143 .00375 L
s
.02381 0 m
.02381 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -66.0625 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
69.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .0412 m
.00625 .0412 L
s
gsave
-0.0125 .0412 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7.6) show
1.000 setlinewidth
grestore
0 .12361 m
.00625 .12361 L
s
gsave
-0.0125 .12361 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7.8) show
1.000 setlinewidth
grestore
0 .20601 m
.00625 .20601 L
s
gsave
-0.0125 .20601 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8) show
1.000 setlinewidth
grestore
0 .28842 m
.00625 .28842 L
s
gsave
-0.0125 .28842 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8.2) show
1.000 setlinewidth
grestore
0 .37082 m
.00625 .37082 L
s
gsave
-0.0125 .37082 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8.4) show
1.000 setlinewidth
grestore
0 .45322 m
.00625 .45322 L
s
gsave
-0.0125 .45322 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8.6) show
1.000 setlinewidth
grestore
0 .53563 m
.00625 .53563 L
s
gsave
-0.0125 .53563 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8.8) show
1.000 setlinewidth
grestore
0 .61803 m
.00625 .61803 L
s
gsave
-0.0125 .61803 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(9) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .0618 m
.00375 .0618 L
s
0 .0824 m
.00375 .0824 L
s
0 .10301 m
.00375 .10301 L
s
0 .14421 m
.00375 .14421 L
s
0 .16481 m
.00375 .16481 L
s
0 .18541 m
.00375 .18541 L
s
0 .22661 m
.00375 .22661 L
s
0 .24721 m
.00375 .24721 L
s
0 .26781 m
.00375 .26781 L
s
0 .30902 m
.00375 .30902 L
s
0 .32962 m
.00375 .32962 L
s
0 .35022 m
.00375 .35022 L
s
0 .39142 m
.00375 .39142 L
s
0 .41202 m
.00375 .41202 L
s
0 .43262 m
.00375 .43262 L
s
0 .47383 m
.00375 .47383 L
s
0 .49443 m
.00375 .49443 L
s
0 .51503 m
.00375 .51503 L
s
0 .55623 m
.00375 .55623 L
s
0 .57683 m
.00375 .57683 L
s
0 .59743 m
.00375 .59743 L
s
0 .0206 m
.00375 .0206 L
s
0 0 m
.00375 0 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -100.875 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
74.0313 11.8438 translate
90 rotate
-74.0313 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(V) show
71.625 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(,) show
77.625 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(y) show
85.063 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.11905 .61178 m
.11905 .61803 L
s
.30952 .61178 m
.30952 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.69048 .61178 m
.69048 .61803 L
s
.88095 .61178 m
.88095 .61803 L
s
.125 Mabswid
.16667 .61428 m
.16667 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.2619 .61428 m
.2619 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.7381 .61428 m
.7381 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.07143 .61428 m
.07143 .61803 L
s
.02381 .61428 m
.02381 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.97619 .61428 m
.97619 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .0412 m
1 .0412 L
s
.99375 .12361 m
1 .12361 L
s
.99375 .20601 m
1 .20601 L
s
.99375 .28842 m
1 .28842 L
s
.99375 .37082 m
1 .37082 L
s
.99375 .45322 m
1 .45322 L
s
.99375 .53563 m
1 .53563 L
s
.125 Mabswid
.99625 .0618 m
1 .0618 L
s
.99625 .0824 m
1 .0824 L
s
.99625 .10301 m
1 .10301 L
s
.99625 .14421 m
1 .14421 L
s
.99625 .16481 m
1 .16481 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .22661 m
1 .22661 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26781 m
1 .26781 L
s
.99625 .30902 m
1 .30902 L
s
.99625 .32962 m
1 .32962 L
s
.99625 .35022 m
1 .35022 L
s
.99625 .39142 m
1 .39142 L
s
.99625 .41202 m
1 .41202 L
s
.99625 .43262 m
1 .43262 L
s
.99625 .47383 m
1 .47383 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .51503 m
1 .51503 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57683 m
1 .57683 L
s
.99625 .59743 m
1 .59743 L
s
.99625 .0206 m
1 .0206 L
s
.99625 0 m
1 0 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
.5 0 m
.5 .61803 L
s
gsave
.5 .63053 -156.688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Wave) show
93.875 15.813 moveto
(Function) show
139.313 15.813 moveto
(with) show
163.563 15.813 moveto
(Energy) show
200.563 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
211.375 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8.12439) show
250.375 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.21475 .61803 m
.22272 .40284 L
s
.22272 .40284 m
.23998 0 L
s
.76 0 m
.78271 .54066 L
s
.78271 .54066 m
.78553 .61803 L
s
0 g
.02381 .25726 m
.03279 .2575 L
.04262 .2578 L
.05288 .25821 L
.06244 .25873 L
.0731 .25953 L
.0828 .26056 L
.094 .26223 L
.10458 .26444 L
.11428 .26718 L
.12455 .27102 L
.13333 .27524 L
.14299 .28107 L
.1624 .29736 L
.17331 .30965 L
.18328 .32306 L
.22299 .39531 L
.24302 .43678 L
.25175 .45331 L
.26118 .4687 L
.26646 .47573 L
.27131 .48097 L
.27595 .48471 L
.27829 .48607 L
.27952 .48664 L
.28086 .48714 L
.28159 .48736 L
.28226 .48753 L
.2829 .48766 L
.28359 .48777 L
.2848 .48787 L
.28609 .48785 L
.28682 .48779 L
.28748 .48769 L
.28822 .48755 L
.28899 .48735 L
.29028 .4869 L
.2917 .48626 L
.29424 .48468 L
.29695 .4824 L
.30183 .47665 L
.30716 .46791 L
.31207 .45756 L
.32317 .42616 L
.33296 .38989 L
.34333 .34423 L
.38121 .15647 L
.39119 .11504 L
.40066 .08352 L
.40588 .06998 L
Mistroke
.40864 .06401 L
.41158 .05859 L
.41429 .05444 L
.41675 .05141 L
.41923 .04905 L
.42155 .04747 L
.42272 .04691 L
.42397 .04649 L
.42515 .04624 L
.42622 .04616 L
.42748 .04622 L
.4282 .04633 L
.42885 .04649 L
.43006 .04689 L
.43134 .0475 L
.43256 .04823 L
.43388 .04921 L
.43626 .05142 L
.44167 .05858 L
.44637 .06705 L
.45132 .07804 L
.46016 .10236 L
.49832 .25004 L
.51961 .33954 L
.53893 .40935 L
.54859 .43628 L
.55368 .44759 L
.55905 .45708 L
.56376 .46313 L
.56505 .46441 L
.56643 .46558 L
.56763 .46643 L
.56893 .46718 L
.5702 .46774 L
.57137 .46811 L
.57249 .46831 L
.57369 .46838 L
.57475 .46831 L
.57575 .46812 L
.57685 .46779 L
.57803 .46728 L
.57928 .46657 L
.58042 .46576 L
.58298 .46342 L
.58534 .46058 L
.58758 .45732 L
.5926 .44794 L
.59804 .43464 L
.60901 .39872 L
.61914 .35648 L
Mistroke
.63947 .25613 L
.65851 .16176 L
.67675 .08861 L
.682 .07236 L
.68699 .05917 L
.69182 .04857 L
.69636 .04054 L
.69878 .03703 L
.70139 .03383 L
.70357 .03163 L
.70596 .02969 L
.70855 .02813 L
.70996 .02751 L
.71128 .02709 L
.71252 .02682 L
.71365 .02667 L
.71495 .02663 L
.71618 .02671 L
.71724 .02687 L
.7184 .02715 L
.71962 .02753 L
.72076 .02799 L
.7228 .02903 L
.72499 .03046 L
.72999 .03482 L
.73457 .04005 L
.74439 .05452 L
.75351 .07097 L
.77422 .11337 L
.79297 .15129 L
.81328 .18635 L
.8244 .20197 L
.83464 .21407 L
.84506 .22422 L
.8548 .23193 L
.86462 .23816 L
.87358 .24268 L
.88353 .24661 L
.89422 .24978 L
.9041 .25195 L
.91326 .25345 L
.92331 .25465 L
.934 .25556 L
.94433 .25618 L
.95547 .25666 L
.96599 .25699 L
.97566 .25725 L
.97619 .25726 L
Mfstroke
[ .02 .04 ] 0 setdash
.02381 .25726 m
.06244 .25726 L
.10458 .25726 L
.14415 .25726 L
.18221 .25726 L
.22272 .25726 L
.26171 .25726 L
.30316 .25726 L
.34309 .25726 L
.3815 .25726 L
.42237 .25726 L
.46172 .25726 L
.49955 .25726 L
.53984 .25726 L
.57861 .25726 L
.61984 .25726 L
.65954 .25726 L
.69774 .25726 L
.73838 .25726 L
.77751 .25726 L
.81909 .25726 L
.85916 .25726 L
.89771 .25726 L
.93871 .25726 L
.97619 .25726 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.0238095 0.323031 0.477324 0.603319 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0952381 -0.515028 0.206011 [
[.11905 -0.0125 -9.15625 -15.6875 ]
[.11905 -0.0125 9.15625 0 ]
[.30952 -0.0125 -9.15625 -15.6875 ]
[.30952 -0.0125 9.15625 0 ]
[.5 -0.0125 -5 -15.6875 ]
[.5 -0.0125 5 0 ]
[.69048 -0.0125 -5 -15.6875 ]
[.69048 -0.0125 5 0 ]
[.88095 -0.0125 -5 -15.6875 ]
[.88095 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -5.0625 -31.375 ]
[.5 -0.0125 5.0625 -15.6875 ]
[-0.0125 .10301 -10 -7.84375 ]
[-0.0125 .10301 0 7.84375 ]
[-0.0125 .20601 -19 -7.84375 ]
[-0.0125 .20601 0 7.84375 ]
[-0.0125 .30902 -10 -7.84375 ]
[-0.0125 .30902 0 7.84375 ]
[-0.0125 .41202 -19 -7.84375 ]
[-0.0125 .41202 0 7.84375 ]
[-0.0125 .51503 -10 -7.84375 ]
[-0.0125 .51503 0 7.84375 ]
[-0.0125 .61803 -19 -7.84375 ]
[-0.0125 .61803 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -34.6875 -13.0313 ]
[-0.01875 .30902 -19 13.0313 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -95.6875 0 ]
[.5 .63053 95.6875 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.11905 0 m
.11905 .00625 L
s
gsave
.11905 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.30952 0 m
.30952 .00625 L
s
gsave
.30952 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.5 0 m
.5 .00625 L
s
gsave
.5 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.69048 0 m
.69048 .00625 L
s
gsave
.69048 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.88095 0 m
.88095 .00625 L
s
gsave
.88095 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.125 Mabswid
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.07143 0 m
.07143 .00375 L
s
.02381 0 m
.02381 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -66.0625 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
69.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .10301 m
.00625 .10301 L
s
gsave
-0.0125 .10301 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
0 .20601 m
.00625 .20601 L
s
gsave
-0.0125 .20601 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3.5) show
1.000 setlinewidth
grestore
0 .30902 m
.00625 .30902 L
s
gsave
-0.0125 .30902 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
0 .41202 m
.00625 .41202 L
s
gsave
-0.0125 .41202 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4.5) show
1.000 setlinewidth
grestore
0 .51503 m
.00625 .51503 L
s
gsave
-0.0125 .51503 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
0 .61803 m
.00625 .61803 L
s
gsave
-0.0125 .61803 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5.5) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .0824 m
.00375 .0824 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .16481 m
.00375 .16481 L
s
0 .18541 m
.00375 .18541 L
s
0 .22661 m
.00375 .22661 L
s
0 .24721 m
.00375 .24721 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .32962 m
.00375 .32962 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .49443 m
.00375 .49443 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .57683 m
.00375 .57683 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -100.875 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
74.0313 11.8438 translate
90 rotate
-74.0313 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(V) show
71.625 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(,) show
77.625 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(y) show
85.063 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.11905 .61178 m
.11905 .61803 L
s
.30952 .61178 m
.30952 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.69048 .61178 m
.69048 .61803 L
s
.88095 .61178 m
.88095 .61803 L
s
.125 Mabswid
.16667 .61428 m
.16667 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.2619 .61428 m
.2619 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.7381 .61428 m
.7381 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.07143 .61428 m
.07143 .61803 L
s
.02381 .61428 m
.02381 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.97619 .61428 m
.97619 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 0 m
1 0 L
s
.99375 .10301 m
1 .10301 L
s
.99375 .20601 m
1 .20601 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .41202 m
1 .41202 L
s
.99375 .51503 m
1 .51503 L
s
.125 Mabswid
.99625 .0206 m
1 .0206 L
s
.99625 .0412 m
1 .0412 L
s
.99625 .0618 m
1 .0618 L
s
.99625 .0824 m
1 .0824 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .14421 m
1 .14421 L
s
.99625 .16481 m
1 .16481 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .22661 m
1 .22661 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26781 m
1 .26781 L
s
.99625 .28842 m
1 .28842 L
s
.99625 .32962 m
1 .32962 L
s
.99625 .35022 m
1 .35022 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .39142 m
1 .39142 L
s
.99625 .43262 m
1 .43262 L
s
.99625 .45322 m
1 .45322 L
s
.99625 .47383 m
1 .47383 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .53563 m
1 .53563 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57683 m
1 .57683 L
s
.99625 .59743 m
1 .59743 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
.5 0 m
.5 .61803 L
s
gsave
.5 .63053 -156.688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Wave) show
93.875 15.813 moveto
(Function) show
139.313 15.813 moveto
(with) show
163.563 15.813 moveto
(Energy) show
200.563 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
211.375 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3.89199) show
250.375 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.27725 .61803 m
.28158 .57386 L
.30316 .37942 L
.32216 .23485 L
.33296 .16622 L
.34309 .11243 L
.35264 .07217 L
.35756 .0558 L
.36292 .04152 L
.36584 .03537 L
.36855 .03072 L
.37114 .02723 L
.37247 .02582 L
.3739 .02457 L
.37532 .02363 L
.37662 .02302 L
.37786 .02267 L
.37917 .02255 L
.38035 .02265 L
.38145 .02292 L
.38266 .02343 L
.38395 .02422 L
.38517 .02518 L
.38651 .02649 L
.38924 .02996 L
.39176 .03414 L
.39413 .03889 L
.39945 .05249 L
.40519 .07149 L
.41569 .11686 L
.42531 .16856 L
.44584 .29703 L
.45585 .36008 L
.46494 .41238 L
.47378 .45545 L
.47843 .4741 L
.48336 .49035 L
.48587 .49714 L
.48863 .50339 L
.49117 .50798 L
.49349 .51119 L
.49471 .51249 L
.49587 .51348 L
.49709 .51426 L
.49778 .51458 L
.49843 .5148 L
.4996 .51501 L
.50066 .51499 L
.50189 .5147 L
.50305 .51419 L
Mistroke
.50421 .51342 L
.50546 .51232 L
.50772 .50964 L
.51039 .50528 L
.51283 .50023 L
.51814 .4858 L
.52316 .46805 L
.54165 .37513 L
.5599 .25996 L
.58003 .13883 L
.58957 .09251 L
.59984 .05459 L
.60551 .03971 L
.60798 .03462 L
.61069 .03007 L
.61326 .02673 L
.61565 .02451 L
.61697 .02363 L
.61817 .02306 L
.61884 .02283 L
.61956 .02266 L
.62086 .02255 L
.6221 .02266 L
.62323 .02297 L
.62453 .02355 L
.62577 .02433 L
.62796 .02625 L
.63035 .02913 L
.63298 .03322 L
.63577 .03861 L
.64078 .05097 L
.65078 .08536 L
.66184 .13722 L
.68116 .258 L
.70201 .42325 L
Mfstroke
.70201 .42325 m
.72174 .61803 L
s
0 g
.02381 .28677 m
.03279 .28677 L
.04262 .28678 L
.04758 .28679 L
.05288 .28679 L
.06244 .28681 L
.06788 .28682 L
.07379 .28684 L
.07899 .28685 L
.08451 .28688 L
.08947 .2869 L
.09494 .28694 L
.10458 .28702 L
.10978 .28708 L
.11472 .28715 L
.12399 .28731 L
.13397 .28756 L
.14487 .28793 L
.15519 .28843 L
.16454 .28904 L
.17534 .28997 L
.18551 .29114 L
.19528 .29259 L
.20559 .29455 L
.22406 .29941 L
.23354 .3027 L
.2437 .30693 L
.26204 .31651 L
.3034 .34692 L
.32409 .36466 L
.34324 .38006 L
.36183 .39199 L
.37121 .39633 L
.37656 .39821 L
.38157 .39956 L
.38384 .40005 L
.38627 .40047 L
.38766 .40067 L
.38893 .40083 L
.39013 .40095 L
.3914 .40106 L
.39204 .40111 L
.39273 .40115 L
.39396 .4012 L
.39465 .40122 L
.39539 .40124 L
.39609 .40125 L
.39673 .40124 L
.39796 .40123 L
.39866 .40121 L
Mistroke
.39931 .40118 L
.40049 .40112 L
.40175 .40103 L
.40416 .4008 L
.40676 .40047 L
.41147 .39965 L
.41605 .39861 L
.42026 .39747 L
.44072 .39022 L
.4598 .38289 L
.46923 .3798 L
.47816 .37747 L
.48309 .37649 L
.48569 .37606 L
.48845 .37569 L
.49077 .37543 L
.492 .37532 L
.49332 .37521 L
.49453 .37513 L
.49564 .37508 L
.49671 .37503 L
.49782 .375 L
.49906 .37498 L
.49976 .37497 L
.5004 .37497 L
.50159 .37499 L
.50284 .37502 L
.50396 .37506 L
.50501 .37511 L
.50739 .37527 L
.50997 .37551 L
.51232 .37578 L
.5176 .37661 L
.52212 .37754 L
.52704 .37875 L
.53598 .38144 L
.55534 .38866 L
.56563 .39267 L
.57673 .39656 L
.58187 .39807 L
.58728 .39939 L
.5919 .40027 L
.59452 .40065 L
.59696 .40092 L
.59815 .40102 L
.59943 .40111 L
.60065 .40118 L
.60177 .40122 L
.60278 .40124 L
.6039 .40124 L
Mistroke
.60506 .40123 L
.60615 .4012 L
.60741 .40114 L
.60862 .40106 L
.61088 .40085 L
.61331 .40053 L
.61597 .40008 L
.62129 .39884 L
.6263 .39726 L
.63565 .39328 L
.64573 .38762 L
.65663 .38015 L
.69766 .34604 L
.71809 .32978 L
.73716 .31698 L
.7555 .30729 L
.77515 .29966 L
.78534 .2967 L
.79641 .29414 L
.80687 .29224 L
.81654 .29088 L
.82617 .28982 L
.83521 .28905 L
.84513 .28841 L
.85582 .28791 L
.8654 .28758 L
.87588 .28732 L
.88117 .28722 L
.88681 .28713 L
.89698 .28701 L
.90232 .28696 L
.90723 .28693 L
.91253 .28689 L
.91834 .28687 L
.92814 .28683 L
.93349 .28682 L
.9385 .28681 L
.94334 .2868 L
.94861 .28679 L
.9582 .28678 L
.96754 .28677 L
.97619 .28677 L
Mfstroke
[ .02 .04 ] 0 setdash
.02381 .28677 m
.06244 .28677 L
.10458 .28677 L
.14415 .28677 L
.18221 .28677 L
.22272 .28677 L
.26171 .28677 L
.30316 .28677 L
.34309 .28677 L
.3815 .28677 L
.42237 .28677 L
.46172 .28677 L
.49955 .28677 L
.53984 .28677 L
.57861 .28677 L
.61984 .28677 L
.65954 .28677 L
.69774 .28677 L
.73838 .28677 L
.77751 .28677 L
.81909 .28677 L
.85916 .28677 L
.89771 .28677 L
.93871 .28677 L
.97619 .28677 L
s
MathSubEnd
P
% End of sub-graphic
% Start of sub-graphic
p
0.522676 0.323031 0.97619 0.603319 MathSubStart
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0952381 -0.515028 0.206011 [
[.11905 -0.0125 -9.15625 -15.6875 ]
[.11905 -0.0125 9.15625 0 ]
[.30952 -0.0125 -9.15625 -15.6875 ]
[.30952 -0.0125 9.15625 0 ]
[.5 -0.0125 -5 -15.6875 ]
[.5 -0.0125 5 0 ]
[.69048 -0.0125 -5 -15.6875 ]
[.69048 -0.0125 5 0 ]
[.88095 -0.0125 -5 -15.6875 ]
[.88095 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -5.0625 -31.375 ]
[.5 -0.0125 5.0625 -15.6875 ]
[-0.0125 .10301 -10 -7.84375 ]
[-0.0125 .10301 0 7.84375 ]
[-0.0125 .20601 -19 -7.84375 ]
[-0.0125 .20601 0 7.84375 ]
[-0.0125 .30902 -10 -7.84375 ]
[-0.0125 .30902 0 7.84375 ]
[-0.0125 .41202 -19 -7.84375 ]
[-0.0125 .41202 0 7.84375 ]
[-0.0125 .51503 -10 -7.84375 ]
[-0.0125 .51503 0 7.84375 ]
[-0.0125 .61803 -19 -7.84375 ]
[-0.0125 .61803 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -34.6875 -13.0313 ]
[-0.01875 .30902 -19 13.0313 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -92.6875 0 ]
[.5 .63053 92.6875 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.11905 0 m
.11905 .00625 L
s
gsave
.11905 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.30952 0 m
.30952 .00625 L
s
gsave
.30952 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.5 0 m
.5 .00625 L
s
gsave
.5 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.69048 0 m
.69048 .00625 L
s
gsave
.69048 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.88095 0 m
.88095 .00625 L
s
gsave
.88095 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.125 Mabswid
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.07143 0 m
.07143 .00375 L
s
.02381 0 m
.02381 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -66.0625 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
69.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .10301 m
.00625 .10301 L
s
gsave
-0.0125 .10301 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
0 .20601 m
.00625 .20601 L
s
gsave
-0.0125 .20601 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3.5) show
1.000 setlinewidth
grestore
0 .30902 m
.00625 .30902 L
s
gsave
-0.0125 .30902 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
0 .41202 m
.00625 .41202 L
s
gsave
-0.0125 .41202 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4.5) show
1.000 setlinewidth
grestore
0 .51503 m
.00625 .51503 L
s
gsave
-0.0125 .51503 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
0 .61803 m
.00625 .61803 L
s
gsave
-0.0125 .61803 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5.5) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .0824 m
.00375 .0824 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .16481 m
.00375 .16481 L
s
0 .18541 m
.00375 .18541 L
s
0 .22661 m
.00375 .22661 L
s
0 .24721 m
.00375 .24721 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .32962 m
.00375 .32962 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .49443 m
.00375 .49443 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .57683 m
.00375 .57683 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -100.875 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
74.0313 11.8438 translate
90 rotate
-74.0313 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(V) show
71.625 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(,) show
77.625 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(y) show
85.063 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.11905 .61178 m
.11905 .61803 L
s
.30952 .61178 m
.30952 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.69048 .61178 m
.69048 .61803 L
s
.88095 .61178 m
.88095 .61803 L
s
.125 Mabswid
.16667 .61428 m
.16667 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.2619 .61428 m
.2619 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.7381 .61428 m
.7381 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.07143 .61428 m
.07143 .61803 L
s
.02381 .61428 m
.02381 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.97619 .61428 m
.97619 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 0 m
1 0 L
s
.99375 .10301 m
1 .10301 L
s
.99375 .20601 m
1 .20601 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .41202 m
1 .41202 L
s
.99375 .51503 m
1 .51503 L
s
.125 Mabswid
.99625 .0206 m
1 .0206 L
s
.99625 .0412 m
1 .0412 L
s
.99625 .0618 m
1 .0618 L
s
.99625 .0824 m
1 .0824 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .14421 m
1 .14421 L
s
.99625 .16481 m
1 .16481 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .22661 m
1 .22661 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26781 m
1 .26781 L
s
.99625 .28842 m
1 .28842 L
s
.99625 .32962 m
1 .32962 L
s
.99625 .35022 m
1 .35022 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .39142 m
1 .39142 L
s
.99625 .43262 m
1 .43262 L
s
.99625 .45322 m
1 .45322 L
s
.99625 .47383 m
1 .47383 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .53563 m
1 .53563 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57683 m
1 .57683 L
s
.99625 .59743 m
1 .59743 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
.5 0 m
.5 .61803 L
s
gsave
.5 .63053 -153.688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Wave) show
93.875 15.813 moveto
(Function) show
139.313 15.813 moveto
(with) show
163.563 15.813 moveto
(Energy) show
200.563 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
211.375 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4.4305) show
244.375 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.27725 .61803 m
.28158 .57386 L
.30316 .37942 L
.32216 .23485 L
.33296 .16622 L
.34309 .11243 L
.35264 .07217 L
.35756 .0558 L
.36292 .04152 L
.36584 .03537 L
.36855 .03072 L
.37114 .02723 L
.37247 .02582 L
.3739 .02457 L
.37532 .02363 L
.37662 .02302 L
.37786 .02267 L
.37917 .02255 L
.38035 .02265 L
.38145 .02292 L
.38266 .02343 L
.38395 .02422 L
.38517 .02518 L
.38651 .02649 L
.38924 .02996 L
.39176 .03414 L
.39413 .03889 L
.39945 .05249 L
.40519 .07149 L
.41569 .11686 L
.42531 .16856 L
.44584 .29703 L
.45585 .36008 L
.46494 .41238 L
.47378 .45545 L
.47843 .4741 L
.48336 .49035 L
.48587 .49714 L
.48863 .50339 L
.49117 .50798 L
.49349 .51119 L
.49471 .51249 L
.49587 .51348 L
.49709 .51426 L
.49778 .51458 L
.49843 .5148 L
.4996 .51501 L
.50066 .51499 L
.50189 .5147 L
.50305 .51419 L
Mistroke
.50421 .51342 L
.50546 .51232 L
.50772 .50964 L
.51039 .50528 L
.51283 .50023 L
.51814 .4858 L
.52316 .46805 L
.54165 .37513 L
.5599 .25996 L
.58003 .13883 L
.58957 .09251 L
.59984 .05459 L
.60551 .03971 L
.60798 .03462 L
.61069 .03007 L
.61326 .02673 L
.61565 .02451 L
.61697 .02363 L
.61817 .02306 L
.61884 .02283 L
.61956 .02266 L
.62086 .02255 L
.6221 .02266 L
.62323 .02297 L
.62453 .02355 L
.62577 .02433 L
.62796 .02625 L
.63035 .02913 L
.63298 .03322 L
.63577 .03861 L
.64078 .05097 L
.65078 .08536 L
.66184 .13722 L
.68116 .258 L
.70201 .42325 L
Mfstroke
.70201 .42325 m
.72174 .61803 L
s
0 g
.02381 .39771 m
.03279 .39772 L
.04262 .39773 L
.04758 .39774 L
.05288 .39775 L
.06244 .39778 L
.06788 .3978 L
.07379 .39783 L
.07899 .39786 L
.08451 .3979 L
.08947 .39795 L
.09494 .39801 L
.10458 .39815 L
.10978 .39824 L
.11472 .39836 L
.12399 .39863 L
.13397 .39903 L
.14487 .39964 L
.15519 .40043 L
.16454 .4014 L
.17534 .40287 L
.18551 .40469 L
.19528 .40692 L
.20559 .40989 L
.22406 .4171 L
.2437 .42793 L
.26204 .44126 L
.30094 .47791 L
.32075 .49755 L
.3312 .50682 L
.3423 .51512 L
.35214 .52068 L
.35484 .52186 L
.35774 .52295 L
.36025 .52373 L
.363 .52442 L
.36422 .52467 L
.36552 .52489 L
.36665 .52505 L
.36787 .52519 L
.36859 .52525 L
.36925 .5253 L
.37055 .52536 L
.37174 .52538 L
.37299 .52536 L
.37408 .52531 L
.37524 .52522 L
.37635 .52511 L
.37735 .52498 L
.37981 .52455 L
Mistroke
.38215 .524 L
.38719 .52235 L
.39176 .5203 L
.402 .51396 L
.41148 .50612 L
.4204 .49731 L
.459 .4503 L
.50005 .39764 L
.53959 .34694 L
.57762 .30021 L
.58766 .29008 L
.59717 .28207 L
.60242 .27842 L
.60812 .27516 L
.6133 .27287 L
.6158 .272 L
.61809 .27134 L
.62036 .27082 L
.62274 .27041 L
.62398 .27026 L
.62513 .27015 L
.6264 .27007 L
.6271 .27004 L
.62775 .27003 L
.62891 .27003 L
.62998 .27007 L
.63116 .27014 L
.6324 .27025 L
.63349 .27037 L
.6345 .27052 L
.6368 .27094 L
.63947 .2716 L
.642 .27237 L
.64674 .27422 L
.65741 .2801 L
.66705 .28716 L
.6773 .29605 L
.69571 .3141 L
.73343 .35038 L
.75275 .36514 L
.7627 .3714 L
.7736 .3772 L
.79362 .38527 L
.80454 .38844 L
.81471 .39077 L
.82521 .39263 L
.83514 .39398 L
.84405 .39491 L
.85372 .39568 L
.86317 .39625 L
Mistroke
.87357 .3967 L
.87881 .39688 L
.88443 .39703 L
.88971 .39716 L
.89461 .39725 L
.90472 .3974 L
.91006 .39746 L
.91576 .39751 L
.92119 .39755 L
.9263 .39758 L
.93585 .39763 L
.94051 .39764 L
.94562 .39766 L
.95594 .39768 L
.9655 .39769 L
.97442 .3977 L
.97619 .39771 L
Mfstroke
[ .02 .04 ] 0 setdash
.02381 .39771 m
.06244 .39771 L
.10458 .39771 L
.14415 .39771 L
.18221 .39771 L
.22272 .39771 L
.26171 .39771 L
.30316 .39771 L
.34309 .39771 L
.3815 .39771 L
.42237 .39771 L
.46172 .39771 L
.49955 .39771 L
.53984 .39771 L
.57861 .39771 L
.61984 .39771 L
.65954 .39771 L
.69774 .39771 L
.73838 .39771 L
.77751 .39771 L
.81909 .39771 L
.85916 .39771 L
.89771 .39771 L
.93871 .39771 L
.97619 .39771 L
s
MathSubEnd
P
% End of sub-graphic
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{441.813, 273.063},
  ImageMargins->{{16, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000:F0001VB000`400?l00000o`00003oo`3ooooo
0?oooiP0oooo003o0?ooool0ooooV03oool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo
003o0?ooool0ooooV03oool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0
ooooV03oool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool0
0?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0ooooo`3o
oonH0?ooo`00o`3ooooo0?oooiP0oooo002V0?ooo`050;md0000000007Bo0?ooo`3OW4P00P000000
0`00B9`0gooo0?ooo`3o0?oooc`0oooo00D0_g@000000000M;l0oooo0=nLB002000000030018W03O
ool0oooo0:40oooo002V0?ooo`080?oog`2LB7@0_ooo0?ooo`3ooml0W4P00018W03Ooooo0?oooch0
oooo00P0oooO09a8M02oool0oooo0?oog`2LB00004RL0=oooj<0oooo002W0?ooo`060?oo_`1dB9`0
gooo0;md000007@0_oooo`3ooom00?ooo`060?oo_`1dB9`0gooo0;md000007@0_oooY03oool00:P0
oooo00@0omnL07A8B0000000M;ooo`3ooom20?ooo`040?oOW01dB4P0000007BoojD0oooo002X0?oo
o`030?oo_`1d0000B9cO0?l0oooo@`3oool00`3ookl0M00004RLg`2V0?ooo`00Y`3oool01@3ooml0
W4P00018W02LM4P0W=oo0?l0oooo@@3oool01@3ooml0W4P00018W02LM4P0W=oo0:D0oooo002W0?oo
o`060;md000004P0W=oo0?oog`2LB7@0_oooo`3ooom00?ooo`060;md000004P0W=oo0?oog`2LB7@0
_oooY03oool00:D0oooo00T0oooO09a8000000000000001d_`3o_g@000000000M02oool0o`3ooolm
0?ooo`090?oog`2LB000000000000000M;l0okmd0000000007@0_ooo0:<0oooo003o0?ooool0oooo
V03oool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0
ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0ooooo`3ooonH
0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0ooooo`3ooonH0?ooo`00o`3o
oooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0
oooo003o0?ooool0ooooV03oool006`0oooo00@0oooO09a80000B9`0gooo6P3oool00`2oM0000000
000000040000000304RLg`3oool0oooo01@0oooo00D0omnL04P000000000001d0;ooo`0K0?ooo`03
0;md00000000000000@0000000<0B9cO0?ooo`3oool0703oool0103ooml0W4P00018W03Ooono0?oo
o`040?oog`2LB00004RL0=oooaX0oooo00<0_g@0000000000000100000000`18W=l0oooo0?ooo`0D
0?ooo`050?oOW018000000000000M02oool06`3oool00`2oM0000000000000040000000304RLg`3o
ool0oooo01`0oooo00@0oooO09a80000B9`0goooG`3oool006`0oooo00@0oooO09a80000B9`0gooo
6`3oool00`3OW4P0000000000003000000030018W03Oool0oooo01<0oooo00L0oono07@0001d_ol0
oooo0?noM00007@0_ooo01/0oooo00<0gia80000000000000`0000000`00B9`0gooo0?ooo`0L0?oo
o`040?oog`2LB00004RL0=oookl0oooo00@0oooO09a80000B9`0gooo6`3oool00`3OW4P000000000
0003000000030018W03Oool0oooo01<0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo01/0
oooo00<0gia80000000000000`0000000`00B9`0gooo0?ooo`0L0?ooo`040?oog`2LB00004RL0=oo
oel0oooo001/0?ooo`040?oog`2LB00004RL0=oooa`0oooo00<0gia804RLg`3oool00P3oool00`3O
W7@0W=oo0?ooo`0C0?ooo`030;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool06`3oool00`3O
W4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo01`0oooo00@0oooO09a80000B9`0gooo_`3oool0
103ooml0W4P00018W03OoolL0?ooo`030=nLB018W=l0oooo0080oooo00<0giad09cOo`3oool04`3o
ool00`2oM00004RL0=ooo`020?ooo`030;md0000M;l0oooo01/0oooo00<0gia804RLg`3oool00P3o
ool00`3OW7@0W=oo0?ooo`0L0?ooo`040?oog`2LB00004RL0=oooel0oooo001X0?ooo`030;md0000
0000000000D0000000<0M;oo0?ooo`3oool06P3oool00`3OW4P0M;oo0?ooo`0F0?ooo`030?oo_`1d
0000B9cO00<0oooo00<0gia80000M02oool0703oool00`3OW4P0M;oo0?ooo`0L0?ooo`030;md0000
0000000000D0000000<0M;oo0?ooo`3oool0^03oool00`2oM0000000000000050000000307Boo`3o
ool0oooo01X0oooo00<0gia807Boo`3oool05P3oool00`3ookl0M00004RLg`030?ooo`030=nLB000
07@0_ooo01`0oooo00<0gia807Boo`3oool0703oool00`2oM0000000000000050000000307Boo`3o
ool0oooo05`0oooo001L0?ooo`030?oog`2LB000000000L0000000@0B9cO0?ooo`3ogi`0M9cO0P3o
ool0103ooml0W4P00018W03Oool>0?ooo`030?oog`2LB000000000L0000000<0B9cO0?ooo`3oool0
0`3oool00`2oM4P0W=oo0?ooo`0E0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0
7@3oool00`2oM4P0W=oo0?ooo`0K0?ooo`080?oOW01dW=l0oooo0?ooo`3ooml0W4P00018W03Ooon_
0?ooo`030?oog`2LB000000000L0000000@0B9cO0?ooo`3ogi`0M9cO0P3oool0103ooml0W4P00018
W03Oool>0?ooo`030?oog`2LB000000000L0000000<0B9cO0?ooo`3oool00`3oool00`2oM4P0W=oo
0?ooo`0E0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol07@3oool00`2oM4P0W=oo
0?ooo`0K0?ooo`080?oOW01dW=l0oooo0?ooo`3ooml0W4P00018W03OoomO0?ooo`00J@3oool01`3O
W4P0M;oo0?ooo`3ooml0W4P00018W03Oool07P3oool00`3ooml0W4Qd0;ooo`0E0?ooo`030?oOW018
0000M;oo00<0oooo00<0okmd0000B02Lgol07@3oool00`3ooml0W4Qd0;ooo`0L0?ooo`070=nLB01d
_ol0oooo0?oog`2LB00004RL0=ooo`2l0?ooo`070=nLB01d_ol0oooo0?oog`2LB00004RL0=ooo`0N
0?ooo`030?oog`2LB7@0_ooo01D0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0M
0?ooo`030?oog`2LB7@0_ooo01`0oooo00L0gia807Boo`3oool0oooO09a80000B9`0gooo05l0oooo
001Y0?ooo`070?oog`2LB7@0_ooo0?oog`2LB00004RL0=ooo`0O0?ooo`030?noM000M;l0oooo01@0
oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0N0?ooo`030?noM000M;l0oooo01/0
oooo00L0oooO09a8M02oool0oooO09a80000B9`0gooo0;`0oooo00L0oooO09a8M02oool0oooO09a8
0000B9`0gooo01l0oooo00<0okmd001d_`3oool0503oool00`3ogi`0B00007Boo`030?ooo`030?no
M00004P0W=oo01h0oooo00<0okmd001d_`3oool06`3oool01`3ooml0W4Qd0;ooo`3ooml0W4P00018
W03Oool0G`3oool006X0oooo00H0omnL04Qd_`3ooml0W4P00018W03OoolO0?ooo`030?oo_`1d04P0
W=oo01@0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0N0?ooo`030?oo_`1d04P0
W=oo01`0oooo00H0omnL04Qd_`3ooml0W4P00018W03Ooonm0?ooo`060?oOW018M;l0oooO09a80000
B9`0gooo7`3oool00`3ookl0M01809cOo`0D0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000
B02Lgol07P3oool00`3ookl0M01809cOo`0L0?ooo`060?oOW018M;l0oooO09a80000B9`0goooG`3o
ool006/0oooo00D0gia807Bog`2LB00004RL0=ooo`0O0?ooo`030?oo_`1d0000B9cO01@0oooo00<0
oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`0N0?ooo`030?oo_`1d0000B9cO01d0oooo00D0
gia807Bog`2LB00004RL0=ooo`2n0?ooo`050=nLB01d_ml0W4P00018W03Oool07`3oool00`3ookl0
M00004RLg`0D0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool07P3oool00`3ookl0
M00004RLg`0M0?ooo`050=nLB01d_ml0W4P00018W03Oool0G`3oool006/0oooo00D0oooO09a8B018
B00004RL0=ooo`0J0?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l05@3oool00`3O
W4P004RL0=ooo`020?ooo`030;md0000M;l0oooo01T0oooo00<0oknL09cOo`3oool00P3oool00`3o
gi`0B00004RLg`0M0?ooo`050?oog`2LB4P0B4P00018W03Oool0_P3oool01@3ooml0W4Q804Q80000
B9`0gooo01X0oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0B00004RLg`0E0?ooo`030=nLB000
B9`0gooo0080oooo00<0_g@0001d_`3oool06@3oool00`3o_i`0W=oo0?ooo`020?ooo`030?oOW018
0000B9cO01d0oooo00D0oooO09a8B018B00004RL0=ooo`1O0?ooo`00K03oool0103ogi`0B0000018
W03OoolJ0?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02ooolE0?ooo`070?oo_`1d0000
M;oo0?ooo`3o_g@0001d0;ooo`0J0?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02ooolN
0?ooo`040?oOW018000004RL0=oookl0oooo00@0omnL04P00000B9`0gooo6P3oool0203ooml0W4Q8
09cOo`3oool0oono07@0000007@0_ooo5@3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool0
6P3oool0203ooml0W4Q809cOo`3oool0oono07@0000007@0_ooo7P3oool0103ogi`0B0000018W03O
oomO0?ooo`00K@3oool00`3OW4P004RL0=ooo`0K0?ooo`060?oog`2LB00000000000000007@0_ooo
5`3oool01@3ookl0M0000000000007@0_ooo01`0oooo00H0oooO09a80000000000000000M02ooolP
0?ooo`030=nLB000B9`0gooo0<00oooo00<0gia80018W03Oool06`3oool01P3ooml0W4P000000000
0000001d0;oooaL0oooo00D0oono07@000000000001d0;ooo`0L0?ooo`060?oog`2LB00000000000
000007@0_ooo803oool00`3OW4P004RL0=ooo`1O0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0
ooooV03oool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool0
0?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo001A0?oooc80000000<0o`00000000000000
C00000000`3o000000000000000`00000980oooo:P0000000`3o000000000000001J000000030?l0
00000000000002T00000C03oool00540oooo00D000000?ooo`3oool0oooo000000080?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00H0oooo00@000000?ooo`3oool0o`00103oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool00`3oool0103o0000oooo0?ooo`0000070?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`2@0?oo
o`050000003oool0oooo0?ooo`0000001`3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool01P3oool01@000000oooo0?ooo`3oool0o`0000<0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool01@3oool01@000000oooo0?ooo`3oool0o`0000@0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
1P3oool01@000000oooo0?ooo`3oool0000004`0oooo001A0?ooo`030000003oool0oooo02l0oooo
00<0o`000?ooo`3oool0903oool00`000000oooo0?ooo`0U0?ooo`030?l0003oool0oooo02l0oooo
00<000000?ooo`3oool0M`3oool00`3OW4P0B9cO0?ooo`040?ooo`030?noM0000000M;oo00<0oooo
00D0omnL04P000000000000007Boo`070?ooo`030000003oool0oooo02L0oooo00<0o`000?ooo`3o
ool0;03oool00`000000oooo0?ooo`0[0?ooo`030?l0003oool0oooo02P0oooo00<000000?ooo`3o
ool0BP3oool00540oooo0P00000`0?ooo`030?l0003oool0oooo02@0oooo00<000000?ooo`3oool0
9@3oool00`3o0000oooo0?ooo`0^0?ooo`800000N@3oool00`3ogi`0B4RL0=ooo`040?ooo`030?no
M0000000M;oo0080oooo00L0omnL04P00018W=l0oooo0?oo_`1d0000M;oo00H0oooo00<000000?oo
o`3oool09`3oool00`3o0000oooo0?ooo`0/0?ooo`030000003oool0oooo02/0oooo00<0o`000?oo
o`3oool0:03oool00`000000oooo0?ooo`1:0?ooo`00D@3oool00`000000oooo0?ooo`0_0?ooo`03
0?l0003oool0oooo02@0oooo00<000000?ooo`3oool09@3oool00`3o0000oooo0?ooo`0_0?ooo`03
0000003oool0oooo07L0oooo00<0oooO09a8B02Lgol0203oool00`3ooml0W4P0001d_`030?ooo`03
0?noM000B9`0gooo00D0oooo00<000000?ooo`3oool09`3oool00`3o0000oooo0?ooo`0/0?ooo`03
0000003oool0oooo02/0oooo00<0o`000?ooo`3oool09`3oool2000004`0oooo001A0?ooo`800000
<03oool00`3o0000oooo0?ooo`0T0?ooo`030000003oool0oooo02D0oooo00<0o`000?ooo`3oool0
;P3oool2000007X0oooo00<0gia804RLg`3oool01`3oool00`3ogi`0B00007Boo`030?ooo`030?no
M00004P0W=oo00D0oooo00<000000?ooo`3oool09`3oool00`3o0000oooo0?ooo`0/0?ooo`030000
003oool0oooo0240oooo1@0000050?ooo`030?l0003oool0oooo02P0oooo00<000000?ooo`3oool0
BP3oool00540oooo00<000000?ooo`3oool0;`3oool00`3o0000oooo0?ooo`0T0?ooo`030000003o
ool0oooo02D0oooo00<0o`000?ooo`3oool0;`3oool00`000000oooo0?ooo`1h0?ooo`030?oOW018
B9`0gooo00L0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`050?ooo`030000003o
ool0oooo02L0oooo00<0o`000?ooo`3oool0;03oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool00P3oool00`3o0000oooo0?ooo`0X0?ooo`030000003o
ool0oooo04X0oooo001A0?ooo`030000003oool0oooo02h0oooo00<0o`000?ooo`3oool09@3oool0
0`000000oooo0?ooo`0U0?ooo`030?l0003oool0oooo02l0oooo00<000000?ooo`3oool0N03oool0
0`3ooml0W4Q809cOo`070?ooo`030?oOW0180000M;oo00<0oooo00<0_g@00000B02Lgol01@3oool2
000002P0oooo00<0o`000?ooo`3oool0;03oool00`000000oooo0?ooo`0P0?ooo`030000003oool0
oooo00@0oooo00D000000?ooo`3oool0oooo0?l0000Y0?ooo`800000C03oool00540oooo0P00000_
0?ooo`030?l0003oool0oooo02D0oooo00<000000?ooo`3oool09P3oool00`3o0000oooo0?ooo`0]
0?ooo`800000N`3oool00`3OW4P007Bo0?ooo`060?ooo`080?oo_`1d0000000004RLg`3oool0okmd
00000000M;l60?ooo`030000003oool0oooo02L0oooo00<0o`000?ooo`3oool07P3oool2000000`0
oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`050?ooo`050000003oool0oooo0?oo
o`3o0000:P3oool00`000000oooo0?ooo`1:0?ooo`00;@3oool00`3ooml0W4P00000000200000003
07Boo`3oool0oooo0080oooo00<0okmd0000001d_ol0103oool00`3OW4P0B9cO0?ooo`030?ooo`03
0?oog`2LB00000000080000000<0M;oo0?ooo`3oool01@3oool00`000000oooo0?ooo`0^0?ooo`03
0?l0003oool0oooo02D0oooo00<000000?ooo`3oool09P3oool00`3o0000oooo0?ooo`0^0?ooo`03
0000003oool0oooo07T0oooo00<0oono07@0M02oool01`3oool00`2oM00004RL0;nLB00200000003
0018W03Oool0oooo00D0oooo00<000000?ooo`3oool09`3oool00`3o0000oooo0?ooo`0M0?ooo`04
0000003oool0oooo000000/0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`060?oo
o`040000003oool0oooo0?l002X0oooo00<000000?ooo`3oool0BP3oool002d0oooo00<0oooO09a8
M02oool00P3oool00`2oM000M;oo0?ooo`020?ooo`030?noM0000000M;oo00@0oooo00<0omnL04Q8
W03Oool00`3oool00`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0oooo00D0oooo00<000000?oo
o`3oool0;P3oool00`3o0000oooo0?ooo`0U0?ooo`030000003oool0oooo02H0oooo00<0o`000?oo
o`3oool0;P3oool00`000000oooo0?ooo`1d0?ooo`030?oOg`3Oool0oooo00<0oooo00<0_g@007Bo
o`3oool01P3oool00`3ogi`0B00007Boo`0:0?ooo`030000003oool0oooo02L0oooo00<0o`000?oo
o`3oool0703oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00P0oooo00<000000?oo
o`3oool07P3oool00`000000oooo0?ooo`070?ooo`040000003oool0oooo0?l002X0oooo00<00000
0?ooo`3oool0BP3oool00380oooo00<0oooO09a8M02oool02@3oool00`3ooml0W4Q809cOo`080?oo
o`030?oog`2LB7@0_ooo00D0oooo0P00000_0?ooo`030?l0003oool0oooo02D0oooo00<000000?oo
o`3oool09P3oool00`3o0000oooo0?ooo`0]0?ooo`800000MP3oool00`3ookl0M4RL0=ooo`030?oo
o`030?noM000M;l0oooo00L0oooo00<0gia80000B02Lgol02@3oool00`000000oooo0?ooo`0W0?oo
o`030?l0003oool0oooo01/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`080?oo
o`030000003oool0oooo01h0oooo00<000000?ooo`3oool0203oool00`000000oooo0?l0000Y0?oo
o`800000C03oool003<0oooo00<0_g@007Boo`3oool02@3oool00`3OW4P0B9cO0?ooo`080?ooo`03
0;md001d_ol0oooo00@0oooo00<000000?ooo`3oool0;P3oool00`3o0000oooo0?ooo`0U0?ooo`03
0000003oool0oooo02H0oooo00<0o`000?ooo`3oool0;P3oool00`000000oooo0?ooo`1e0?ooo`03
0;md00000000000000<0000000<0001d0;ooo`3oool01`3oool00`3o_g@00000001d_`080?ooo`03
0000003oool0oooo02H0oooo00<0o`000?ooo`3oool0703oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0o`0002X0oooo00<000000?ooo`3oool0BP3oool00380oooo00<0oooO09a80018W=l0
2P3oool00`3ogi`0B4RL0=ooo`070?ooo`030?oog`2LB000B9cO00D0oooo00<000000?ooo`3oool0
;P3oool00`3o0000oooo0?ooo`0U0?ooo`030000003oool0oooo02H0oooo00<0o`000?ooo`3oool0
;P3oool00`000000oooo0?ooo`1e0?ooo`030?noM0000000000000@0000000<0M;oo0?ooo`3oool0
1`3oool01@3ooml0W4P00000000007@0_ooo00D0oooo00<000000?ooo`3oool09P3oool00`3o0000
oooo0?ooo`0L0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o0000:@3oool00`000000
oooo0?ooo`1:0?ooo`00<P3oool00`3OW4P0000007Boo`0:0?ooo`030?oog`2LB4P0W=oo00L0oooo
00<0gia80000001d_ol01@3oool2000002l0oooo00<0o`000?ooo`3oool09@3oool00`000000oooo
0?ooo`0V0?ooo`030?l0003oool0oooo02d0oooo0P00002B0?ooo`030000003oool0oooo02H0oooo
00<0o`000?ooo`3oool06`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0;0?ooo`030000003o0000oooo02P0oooo
00<000000?ooo`3oool0BP3oool002h0oooo00L0gmoO0?ooo`3ookl0M0000000000007@0_ooo00/0
oooo00<0gia8001d_`3oool00P3oool01`3Ogml0oooo0?oo_`1d000000000000M02oool01@3oool0
0`000000oooo0?ooo`0^0?ooo`030?l0003oool0oooo02D0oooo00<000000?ooo`3oool09P3oool0
0`3o0000oooo0?ooo`0^0?ooo`030000003oool0oooo0900oooo00<000000?ooo`3oool09P3oool0
0`3o0000oooo0?ooo`0K0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo00`0oooo00<000000?l0003oool09`3oool2
000004`0oooo000^0?ooo`060=nLB000000000000000000004P0W=oo303oool00`3ookl0M01d0;oo
o`020?ooo`060=nLB000000000000000000004P0W=oo1P3oool00`000000oooo0?ooo`0^0?ooo`03
0?l0003oool0oooo02D0oooo00<000000?ooo`3oool09P3oool00`3o0000oooo0?ooo`0^0?ooo`03
0000003oool0oooo0900oooo00<000000?ooo`3oool09P3oool00`3o0000oooo0?ooo`0K0?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0L0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`1:0?ooo`00
;P3oool0103ookl0M00000000018W=l90?ooo`030?oOg`3Oool0oooo00<0oooo00P0_g@007Boo`3o
ool0oooo0?oo_`1d0000000004RLg`P0oooo0P00000_0?ooo`030?l0003oool0oooo02D0oooo00<0
00000?ooo`3oool09P3oool00`3o0000oooo0?ooo`0]0?ooo`800000N@3oool00`3OW4P0B9cO0?oo
o`040?ooo`030?noM0000000M;oo00<0oooo00D0gia8000000000000000004RLg`070?ooo`030000
003oool0oooo02H0oooo00<0o`000?ooo`3oool06P3oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo02L0oooo00<000000?ooo`3oool0BP3oool002l0oooo00<0_g@007Boo`3oool02@3o
ool00`3ookl0M4RL0=ooo`030?ooo`030?noM000M;l0oooo0080oooo00<0_g@007Boo`3oool0203o
ool00`000000oooo0?ooo`0^0?ooo`030?l0003oool0oooo02D0oooo00<000000?ooo`3oool09P3o
ool00`3o0000oooo0?ooo`0^0?ooo`030000003oool0oooo07L0oooo00<0omnL04Q8W03Oool0103o
ool00`3o_g@0000007Boo`020?ooo`070=nLB0000000M;oo0?ooo`3ooml0W4P004RLg`060?ooo`03
0000003oool0oooo02H0oooo00<0o`000?ooo`3oool06P3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0=0?ooo`03
0?l000000000oooo02H0oooo0P00001<0?ooo`00;`3oool01P3ogi`0B000000000000000001809cO
o`L0oooo00<0_g@00000000000000`0000001P0007@0_ooo0?ooo`3oool0omnL04P00080000000<0
001809cOo`3oool0103oool2000002l0oooo00<0o`000?ooo`3oool09@3oool00`000000oooo0?oo
o`0V0?ooo`030?l0003oool0oooo02d0oooo0P00001i0?ooo`030?oog`2LB4P0W=oo00P0oooo00<0
oooO09a80000M;l00`3oool00`3o_g@004RL0=ooo`050?ooo`030000003oool0oooo02H0oooo00<0
o`000?ooo`3oool06P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool06`3oool00`000000oooo0?ooo`0>0?ooo`030?l000000000oooo02L0oooo00<0
00000?ooo`3oool0BP3oool00300oooo00D0_g@0000000000000000004RLg`070?ooo`030?noM000
0000000000@0000000D0M;oo0?ooo`3oool0oooo0;md00030000000304RLg`3oool0oooo00<0oooo
00<000000?ooo`3oool0;P3oool00`3o0000oooo0?ooo`0U0?ooo`030000003oool0oooo02H0oooo
00<0o`000?ooo`3oool0;P3oool00`000000oooo0?ooo`1h0?ooo`030=nLB018W=l0oooo00P0oooo
00<0_g@0001d_`3oool00P3oool00`2oM00004RL0=ooo`050?ooo`030000003oool0oooo02H0oooo
00<0o`000?ooo`3oool06@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0>0?ooo`030?l0003oool0000002L0oooo
00<000000?ooo`3oool0BP3oool00540oooo00<000000?ooo`3oool0;@3oool00`3o0000oooo0?oo
o`0V0?ooo`030000003oool0oooo02H0oooo00<0o`000?ooo`3oool0;P3oool00`000000oooo0?oo
o`1h0?ooo`030?oOW018B9`0gooo00P0oooo00L0omnL04P0M02oool0oooo0;md00000000M;oo00H0
oooo00<000000?ooo`3oool09P3oool00`3o0000oooo0?ooo`0I0?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00h0
oooo00<0o`000?ooo`0000009`3oool00`000000oooo0?ooo`1:0?ooo`00D@3oool2000002h0oooo
00<0o`000?ooo`3oool09P3oool00`000000oooo0?ooo`0W0?ooo`030?l0003oool0oooo02`0oooo
0P00001j0?ooo`030?oog`2LB4P0W=oo00T0oooo00D0oono07A8B0180000000004RLg`070?ooo`80
00009`3oool00`3o0000oooo0?ooo`0I0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00h0oooo00@0o`000?ooo`3o
ool000009@3oool2000004`0oooo00150?ooo`050?oOW018000000000000001d_ol01`3oool00`00
0000oooo0?ooo`0]0?ooo`030?l0003oool0oooo02H0oooo00<000000?ooo`3oool09`3oool00`3o
0000oooo0?ooo`0]0?ooo`030000003oool0oooo07T0oooo00<0gia8001d_`3oool0203oool0103o
gi`0B00000000000M;l80?ooo`030000003oool0oooo02H0oooo00<0o`000?ooo`3oool06@3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0?0?ooo`040?l0003oool0oooo000002H0oooo00<000000?ooo`3oool0BP3o
ool004@0oooo00L0omnL04P00018W=l0oooo0?oo_`1d0000M;oo00H0oooo00<000000?ooo`3oool0
;@3oool00`3o0000oooo0?ooo`0V0?ooo`030000003oool0oooo02L0oooo00<0o`000?ooo`3oool0
;@3oool00`000000oooo0?ooo`1i0?ooo`030?oo_`1d07@0_ooo00L0oooo00H0omnL04P0000007@0
_ooo0?noM000M;l70?ooo`030000003oool0oooo02H0oooo00<0o`000?ooo`3oool06@3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`0?0?ooo`050?l0003oool0oooo0?ooo`0000009@3oool00`000000oooo0?ooo`1:
0?ooo`00@`3oool00`3ooml0W4P0001d_`030?ooo`030?noM000B9`0gooo00D0oooo0P00000^0?oo
o`030?l0003oool0oooo02H0oooo00<000000?ooo`3oool09`3oool00`3o0000oooo0?ooo`0/0?oo
o`800000MP3oool00`3ogml0gooo0?ooo`030?ooo`030;md001d_ol0oooo00H0oooo00<0_g@00000
M02oool00P3oool00`2oM000B9cO0?ooo`050?ooo`030000003oool0oooo02H0oooo00<0o`000?oo
o`3oool0603oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0?0?ooo`050?l0003oool0oooo0?ooo`0000009@3oool0
0`000000oooo0?ooo`1:0?ooo`00@`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo
00D0oooo00<000000?ooo`3oool0;@3oool00`3o0000oooo0?ooo`0T0?ooo`H000009P3oool00`3o
0000oooo0?ooo`0]0?ooo`030000003oool0oooo07@0oooo00<0oono07A8W03Oool00`3oool00`3o
_g@007Bo0?ooo`060?ooo`030;md0000B9`0gooo0080oooo00<0gia8001d_`3oool01@3oool00`00
0000oooo0?ooo`0V0?ooo`030?l0003oool0oooo01P0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool03`3oool00`3o
0000oooo0?ooo`020?ooo`030000003oool0oooo0240oooo0P00001<0?ooo`00@`3oool00`3ogi`0
B00007Boo`030?ooo`030?noM00004P0W=oo00D0oooo00<000000?ooo`3oool0;@3oool00`3o0000
oooo0?ooo`0S0?ooo`80000000@0oooo0000003oool0oooo0P00000U0?ooo`030?l0003oool0oooo
02d0oooo00<000000?ooo`3oool0M@3oool00`2oM000000000000003000000030000M02oool0oooo
00D0oooo00L0okmd0000001d_ol0oooo0?oo_`1d0000M;oo00H0oooo00<000000?ooo`3oool09P3o
ool00`3o0000oooo0?ooo`0H0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0100oooo00<0o`000?ooo`3oool00P3o
ool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo04X0oooo00130?ooo`030?oOW0180000
M;oo00<0oooo00<0_g@00000B02Lgol01@3oool2000002h0oooo00<0o`000?ooo`3oool08P3oool0
1@000000oooo0?ooo`3oool0000000@0oooo00<000000?ooo`3oool08P3oool00`3o0000oooo0?oo
o`0/0?ooo`800000M`3oool00`3o_g@00000000000040000000307Boo`3oool0oooo00D0oooo00D0
okmd000000000000000004RLg`070?ooo`030000003oool0oooo02H0oooo00<0o`000?ooo`3oool0
603oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`0@0?ooo`030?l0003oool0oooo00<0oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`1:0?ooo`00@`3oool0203ookl0M00000000018W=l0oooo0?noM000
000007Bo1P3oool00`000000oooo0?ooo`0]0?ooo`030?l0003oool0oooo0240oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0240oooo00<0o`000?oo
o`3oool0;@3oool00`000000oooo0?ooo`2@0?ooo`030000003oool0oooo02H0oooo00<0o`000?oo
o`3oool0603oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`0@0?ooo`030?l0003oool0oooo00<0oooo00<000000?oo
o`3oool0803oool2000004`0oooo00140?ooo`030;md0000B9`0_ia80080000000<004RL0=ooo`3o
ool01@3oool2000002h0oooo00<0o`000?ooo`3oool0803oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool0803oool00`3o0000oooo0?ooo`0/0?ooo`80
0000TP3oool00`000000oooo0?ooo`0V0?ooo`030?l0003oool0oooo01P0oooo00<000000?ooo`3o
ool03P3oool01@000000oooo0?ooo`3oool0000001/0oooo00<000000?ooo`3oool0403oool00`3o
0000oooo0?ooo`040?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0BP3oool004@0
oooo00<0omnL04P0001d_ol02P3oool00`000000oooo0?ooo`0]0?ooo`030?l0003oool0oooo01l0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01l0
oooo00<0o`000?ooo`3oool0;@3oool00`000000oooo0?ooo`2@0?ooo`030000003oool0oooo02H0
oooo00<0o`000?ooo`3oool05`3oool00`000000oooo0?ooo`0?0?ooo`050000003oool0oooo0?oo
o`0000006`3oool00`000000oooo0?ooo`0@0?ooo`030?l0003oool0oooo00D0oooo00<000000?oo
o`3oool07`3oool00`000000oooo0?ooo`1:0?ooo`00A@3oool00`3OW4P0001809cOo`090?ooo`03
0000003oool0oooo02d0oooo00<0o`000?ooo`3oool07`3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool07P3oool00`3o0000oooo0?ooo`0]0?ooo`03
0000003oool0oooo08<0oooo00D0gia8000000000000000004RLg`080?ooo`030000003oool0oooo
02H0oooo00<0o`000?ooo`3oool05`3oool00`000000oooo0?ooo`0?0?ooo`050000003oool0oooo
0?ooo`0000006P3oool00`000000oooo0?ooo`0A0?ooo`030?l0003oool0oooo00D0oooo00<00000
0?ooo`3oool07`3oool00`000000oooo0?ooo`1:0?ooo`00AP3oool00`3o_g@00000001d_`080?oo
o`800000;P3oool00`3o0000oooo0?ooo`0N0?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`0N0?ooo`030?l0003oool0oooo02`0oooo0P0000240?oo
o`070=nLB0000000M;oo0?ooo`3ooml0W4P004RLg`070?ooo`030000003oool0oooo02H0oooo00<0
o`000?ooo`3oool05`3oool00`000000oooo0?ooo`0?0?ooo`050000003oool0oooo0?ooo`000000
6P3oool00`000000oooo0?ooo`0A0?ooo`030?l0003oool0oooo00H0oooo00<000000?ooo`3oool0
7@3oool2000004`0oooo00170?ooo`050?oog`2LB00000000000M02oool01@3oool00`000000oooo
0?ooo`0]0?ooo`030?l0003oool0oooo01d0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo01d0oooo00<0o`000?ooo`3oool0;@3oool00`000000oooo
0?ooo`210?ooo`030?oog`2LB00007Bo00<0oooo00<0okmd0018W03Oool01P3oool00`000000oooo
0?ooo`0V0?ooo`030?l0003oool0oooo01L0oooo00<000000?ooo`3oool0403oool010000000oooo
0?ooo`00000J0?ooo`030000003oool0oooo0140oooo00<0o`000?ooo`3oool01`3oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo04X0oooo001A0?ooo`030000003oool0oooo02`0oooo
00<0o`000?ooo`3oool07P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool07@3oool00`3o0000oooo0?ooo`0]0?ooo`030000003oool0oooo0880oooo
00<0_g@0001d_`3oool00P3oool00`2oM00004RL0=ooo`060?ooo`030000003oool0oooo02H0oooo
00<0o`000?ooo`3oool05`3oool00`000000oooo0?ooo`0@0?ooo`040000003oool0oooo000001X0
oooo00<000000?ooo`3oool04@3oool00`3o0000oooo0?ooo`070?ooo`030000003oool0oooo01d0
oooo00<000000?ooo`3oool0BP3oool002l0oooo00D0omnL04P000000000000007Boo`030?ooo`05
0?noM0000000M;oo0?ooo`2oM0001P0000001018W=l0oooo0?oog`2LB0030000000307Boo`3oool0
oooo00D0oooo0P00000]0?ooo`030?l0003oool0oooo01d0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`080?ooo`030000003oool0oooo01d0oooo00<0o`000?ooo`3oool0:`3oool2
000008@0oooo00L0omnL04P0M02oool0oooo0;md00000000M;oo00L0oooo00<000000?ooo`3oool0
9@3oool00`3o0000oooo0?ooo`0G0?ooo`030000003oool0oooo0140oooo00@000000?ooo`3oool0
00006P3oool00`000000oooo0?ooo`0A0?ooo`030?l0003oool0oooo00P0oooo00<000000?ooo`3o
ool0703oool00`000000oooo0?ooo`1:0?ooo`00;P3oool01`3ogi`0B00004RLg`3oool0oono07@0
001d_ol00P3oool00`3o_g@0000007Boo`020?ooo`030=nLB0000000000000<0000000D004RL0=oo
o`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0oooo00D0oooo00<000000?ooo`3oool0;03oool0
0`3o0000oooo0?ooo`0M0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0203oool0
0`000000oooo0?ooo`0M0?ooo`030?l0003oool0oooo02`0oooo00<000000?ooo`3oool0P`3oool0
1@3ookl0M4Q804P000000000B9cO00P0oooo0P00000V0?ooo`030?l0003oool0oooo01L0oooo00<0
00000?ooo`3oool04@3oool010000000oooo0?ooo`00000J0?ooo`030000003oool0oooo0180oooo
00<0o`000?ooo`3oool01`3oool00`000000oooo0?ooo`0K0?ooo`800000C03oool002d0oooo00<0
oooO09a80000M;l00`3oool00`3o_g@004RL0=ooo`070?ooo`030=nLB018W=l0oooo0080oooo00<0
giad09cOo`3oool0103oool00`3ooml0W4Qd0;ooo`050?ooo`030000003oool0oooo02`0oooo00<0
o`000?ooo`3oool0703oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool0703oool00`3o0000oooo0?ooo`0/0?ooo`030000003oool0oooo08<0oooo00@0
omnL04P00000000007Bo2@3oool00`000000oooo0?ooo`0U0?ooo`030?l0003oool0oooo01L0oooo
00<000000?ooo`3oool04@3oool010000000oooo0?ooo`00000I0?ooo`030000003oool0oooo01<0
oooo00<0o`000?ooo`3oool0203oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo04X0
oooo000]0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0203oool00`3OW4P0M;oo
0?ooo`090?ooo`030;md001d_ol0oooo00@0oooo0P00000]0?ooo`030?l0003oool0oooo01`0oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo01`0oooo
00<0o`000?ooo`3oool0:`3oool2000008@0oooo00H0omnL04P0000007@0_ooo0?noM000M;l80?oo
o`030000003oool0oooo02D0oooo00<0o`000?ooo`3oool05`3oool00`000000oooo0?ooo`0B0?oo
o`030000003oool0000001T0oooo00<000000?ooo`3oool04`3oool00`3o0000oooo0?ooo`090?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool0BP3oool001H0oooo00<0giad09cOo`3o
ool0503oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00T0oooo00<0_gA809cOo`3o
ool01`3oool00`3ooml0W4P004RLg`050?ooo`030000003oool0oooo02`0oooo00<0o`000?ooo`3o
ool06`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool0703oool00`3o0000oooo0?ooo`0/0?ooo`030000003oool0oooo05d0oooo00<0giad09cOo`3o
ool08P3oool00`2oM000001d0;ooo`020?ooo`030;md0018W=l0oooo00H0oooo00<000000?ooo`3o
ool09@3oool00`3o0000oooo0?ooo`0F0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`00
00006@3oool00`000000oooo0?ooo`0C0?ooo`030?l0003oool0oooo00X0oooo00<000000?ooo`3o
ool0603oool2000004`0oooo000F0?ooo`030=nLB000M;l0oooo01@0oooo00<0omnL04P0001d_ol0
0`3oool00`2oM000001809cOo`090?ooo`030?oog`2LB7@0_ooo00L0oooo00<0gia80000001d_ol0
1@3oool2000002d0oooo00<0o`000?ooo`3oool06`3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool06`3oool00`3o0000oooo0?ooo`0[0?ooo`800000
G`3oool00`3OW4P007Bo0?ooo`0R0?ooo`030;md0000B9`0gooo0080oooo00<0gia8001d_`3oool0
1P3oool00`000000oooo0?ooo`0U0?ooo`030?l0003oool0oooo01H0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0000000I0?ooo`030000003oool0oooo01<0oooo00<0o`000?ooo`3oool0
2`3oool2000001T0oooo00<000000?ooo`3oool0BP3oool001H0oooo00D0gia80000000000000000
07Boo`0B0?ooo`080?oo_`1d0000000004RLg`3oool0okmd00000000M;l;0?ooo`030?noM000M;l0
oooo0080oooo00L0gmoO0?ooo`3ookl0M0000000000007@0_ooo00D0oooo00<000000?ooo`3oool0
;03oool00`3o0000oooo0?ooo`0K0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0K0?ooo`030?l0003oool0oooo02`0oooo00<000000?ooo`3oool0
G@3oool01@3OW4P00000000000000000M;oo0200oooo00L0okmd0000001d_ol0oooo0?oo_`1d0000
M;oo00L0oooo00<000000?ooo`3oool09@3oool00`3o0000oooo0?ooo`0F0?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`0000006@3oool00`000000oooo0?ooo`0C0?ooo`030?l0003oool0
oooo00d0oooo0P00000G0?ooo`030000003oool0oooo04X0oooo000F0?ooo`030=nLB00000000000
00<0000000<0001d0;ooo`3oool03`3oool00`2oM00004RL0;nLB002000000030018W03Oool0oooo
00X0oooo00<0oono07@0B02Lgol00P3oool01P3OW4P00000000000000000001809cOo`H0oooo00<0
00000?ooo`3oool0;03oool00`3o0000oooo0?ooo`0J0?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`0K0?ooo`030?l0003oool0oooo02`0oooo00<0
00000?ooo`3oool0G@3oool00`3OW4P0000000000003000000030000M02oool0oooo01d0oooo00D0
okmd000000000000000004RLg`080?ooo`030000003oool0oooo02D0oooo00<0o`000?ooo`3oool0
5P3oool00`000000oooo0?ooo`0D0?ooo`8000006@3oool00`000000oooo0?ooo`0C0?ooo`030?l0
003oool0oooo00l0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`1:0?ooo`005P3o
ool01P3OW7@0W=oo0?ooo`3oool0omnL04P000<0000000<004RL0=ooo`3oool0303oool00`3ogi`0
B00007Boo`0?0?ooo`030?oo_`1d0000B9cO0080oooo00@0oono07@000000000B9cO203oool20000
02d0oooo00<0o`000?ooo`3oool06P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool06P3oool00`3o0000oooo0?ooo`0[0?ooo`800000G`3oool01P3O
W7@0W=oo0?ooo`3oool0omnL04P000<0000000<004RL0=ooo`3oool09`3oool00`000000oooo0?oo
o`0U0?ooo`030?l0003oool0oooo01H0oooo00<000000?ooo`3oool0503oool2000001P0oooo00<0
00000?ooo`3oool0503oool00`3o0000oooo0?ooo`0@0?ooo`8000004`3oool2000004`0oooo000M
0?ooo`030?noM000000000000080000000<007Bo0?ooo`3oool02P3oool00`3OW4P0001809cOo`09
0?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l00`3oool00`2oM000M;oo0?ooo`08
0?ooo`030000003oool0oooo02`0oooo00<0o`000?ooo`3oool06P3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool06P3oool00`3o0000oooo0?ooo`0/
0?ooo`030000003oool0oooo06@0oooo00<0okmd0000000000000P0000000`00M;l0oooo0?ooo`0T
0?ooo`030000003oool0oooo02D0oooo00<0o`000?ooo`3oool05@3oool00`000000oooo0?ooo`0E
0?ooo`800000603oool00`000000oooo0?ooo`0D0?ooo`030?l0003oool0oooo0180oooo1`00000=
0?ooo`030000003oool0oooo04X0oooo000P0?ooo`040?noM0000000001d0;ooo``0oooo00<0okmd
00000000M;l0203oool0203ooml0W4Q809cOo`3oool0oono07@0000007@0_ooo0`3oool01P3ogi`0
B000000000000000001809cOo`D0oooo00<000000?ooo`3oool0;03oool00`3o0000oooo0?ooo`0I
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0I
0?ooo`030?l0003oool0oooo02`0oooo00<000000?ooo`3oool0I`3oool0103o_g@000000000M02o
ool;0?ooo`050=nLB0000000000000000018W=l00`3oool01@3o_g@0000007Boo`3oool0_g@000H0
000000<0B9cO0?ooo`3oool0103oool010000000oooo0?ooo`3oool?000000H0oooo1@0000060?oo
o`@0000000<0o`000?ooo`3oool0103oool4000000H0oooo1@0000020?ooo`040000003oool0oooo
0?ooo`D000001P3oool4000000D0oooo1P0000060?ooo`D000001P3oool4000000H0oooo1@000006
0?ooo`D0000000<0o`000?ooo`3oool00`3oool4000000L0oooo100000060?ooo`/000000`3oool0
0`000000oooo0?ooo`1:0?ooo`007@3oool0103o_g@0000000000000M;l@0?ooo`050?oog`2LB000
00000000M02oool01P3oool01P3ooml0W4P0000000000000001d0;ooo`D0oooo00D0_g@000000000
0000000004RLg`050?ooo`800000;@3oool00`3o0000oooo0?ooo`0I0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0I0?ooo`030?l0003oool0oooo
02/0oooo0P00001V0?ooo`040?noM00000000000001d_`d0oooo00L0gia80000001d_ol0oooo0?oo
g`2LB000B9cO0080oooo00<0okmd0000001d_ol00P3oool00`3OW4P0000000000003000000030018
W03Oool0oooo00@0oooo00<000000?ooo`3oool0303oool4000001D0oooo00<0o`000?ooo`3oool0
5@3oool00`000000oooo0?ooo`0E0?ooo`800000603oool00`000000oooo0?ooo`0D0?ooo`030?l0
003oool0oooo02H0oooo00<000000?ooo`3oool0BP3oool001H0oooo00T0giad09cOo`3oool0oooo
0?oOW018000000000018W03Oool0<P3oool00`000000oooo0?ooo`0/0?ooo`030?l0003oool0oooo
01T0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
01T0oooo00<0o`000?ooo`3oool0;03oool00`000000oooo0?ooo`1M0?ooo`090=nLM02Lgol0oooo
0?ooo`3ogi`0B00000000000B9`0gooo00h0oooo00<0oooO09a80000M;l00`3oool00`3o_g@004RL
0=ooo`070?ooo`030=nLB018W=l0oooo0080oooo00<0giad09cOo`3oool0103oool00`000000oooo
0?ooo`0@0?ooo`@000004@3oool00`3o0000oooo0?ooo`0E0?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0D0?ooo`030?l0003oool0oooo02D0oooo
0P00001<0?ooo`005P3oool01P3OW4P00000000000000000001d0;ooocD0oooo00<000000?ooo`3o
ool0;03oool00`3o0000oooo0?ooo`0H0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`0H0?ooo`030?l0003oool0oooo02`0oooo00<000000?ooo`3o
ool0G@3oool01P3OW4P00000000000000000001d0;oooa80oooo00<0_g@0001d_`3oool00P3oool0
0`2oM00004RL0=ooo`080?ooo`030=nLB01d_ol0oooo00P0oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`0>0?ooo`030?l0003oool0oooo01D0oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01D0oooo00<0o`000?ooo`3oool09P3oool0
0`000000oooo0?ooo`1:0?ooo`005P3oool00`3OW4P0000007Boo`0O0?ooo`050?oOW01800000000
0000001d_ol00`3oool01P3o_g@0000007Boo`3oool0oooO09a800<0000000<0M;oo0?ooo`3oool0
1@3oool2000002d0oooo00<0o`000?ooo`3oool0603oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool06@3oool00`3o0000oooo0?ooo`0Z0?ooo`800000
G`3oool00`3OW4P0000007Boo`0E0?ooo`070?oOW01807@0_ooo0?ooo`2oM000000007Boo`0:0?oo
o`030;mdB02Lgol0oooo00L0oooo00<000000?ooo`3oool05@3oool2000000h0oooo00<0o`000?oo
o`3oool0503oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01D0oooo00<000000?oo
o`3oool05@3oool00`3o0000oooo0?ooo`0V0?ooo`030000003oool0oooo04X0oooo000F0?ooo`03
0=nLM02Lgol0oooo01h0oooo00L0omnL04P00018W=l0oooo0?oo_`1d0000M;oo0080oooo00L0okmd
0000001d_ol0oooo0?oog`2LB7@0_ooo0080oooo00<0_g@007Boo`3oool01@3oool00`000000oooo
0?ooo`0[0?ooo`030?l0003oool0oooo01T0oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo01T0oooo00<0o`000?ooo`3oool0:`3oool00`000000oooo
0?ooo`1M0?ooo`030=nLM02Lgol0oooo01H0oooo00D0oono07A8B0180000000004RLg`0;0?ooo`03
0?oog`2LB7@0_ooo00L0oooo0P00000H0?ooo`030000003oool0oooo00/0oooo00<0o`000?ooo`3o
ool0503oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3o
ool05@3oool00`3o0000oooo0?ooo`0U0?ooo`800000C03oool003H0oooo00<0oooO09a80000M;l0
0`3oool00`3o_g@004RL0=ooo`0:0?ooo`030?oog`2LB7@0_ooo00D0oooo0P00000/0?ooo`030?l0
003oool0oooo01T0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0>0?ooo`030000
003oool0oooo01P0oooo00<0o`000?ooo`3oool0:P3oool2000007P0oooo00@0omnL04P000000000
07Bo3@3oool00`3o_g@007Bo0?ooo`060?ooo`030000003oool0oooo01P0oooo0P00000;0?ooo`03
0?l0003oool0oooo01@0oooo00<000000?ooo`3oool05`3oool2000001H0oooo00<000000?ooo`3o
ool05@3oool00`3o0000oooo0?ooo`0V0?ooo`030000003oool0oooo04X0oooo000f0?ooo`030?oO
W0180000M;oo00<0oooo00<0okmd0000B02Lgol02`3oool00`2oM000M;oo0?ooo`040?ooo`030000
003oool0oooo02/0oooo00<0o`000?ooo`3oool0603oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool0603oool00`3o0000oooo0?ooo`0[0?ooo`030000
003oool0oooo07D0oooo00H0omnL04P0000007@0_ooo0?noM000M;l<0?ooo`030?oo_`1d04P0W=oo
00H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`080?ooo`030?l0003oool0oooo
01@0oooo00<000000?ooo`3oool05`3oool2000001D0oooo00<000000?ooo`3oool05P3oool00`3o
0000oooo0?ooo`0V0?ooo`030000003oool0oooo04X0oooo000P0?ooo`060?oog`2LB00004RL0=oo
o`3ooml0W9bo403oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00X0oooo00<0oooO
09a80018W=l01@3oool00`000000oooo0?ooo`0[0?ooo`030?l0003oool0oooo01P0oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01P0oooo00<0o`00
0?ooo`3oool0:`3oool00`000000oooo0?ooo`1W0?ooo`060?oog`2LB00004RL0=ooo`3ooml0W9bo
203oool00`2oM000001d0;ooo`020?ooo`030;md0018W=l0oooo00X0oooo00<0oono07@00018W=l0
1P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00P0oooo00<0o`000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0H0?ooo`8000005@3oool00`000000oooo0?ooo`0F0?ooo`030?l0
003oool0oooo02H0oooo00<000000?ooo`3oool0BP3oool00200oooo00H0oono07@00000B9`0gooo
0;mdB02Lgol@0?ooo`030?oOW0180000M;oo00<0oooo00<0_g@00000B02Lgol02P3oool00`3OW4P0
000007Boo`050?ooo`800000;03oool00`3o0000oooo0?ooo`0H0?ooo`030000003oool0oooo00d0
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0H0?ooo`030?l0003oool0oooo02X0
oooo0P00001Y0?ooo`060?oo_`1d000004RL0=ooo`2oM4P0W=oo203oool00`2oM00004RL0=ooo`02
0?ooo`030=nLB000M;l0oooo00D0oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0B00004RLg`06
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool01`3oool00`3o0000oooo0?ooo`0C
0?ooo`030000003oool0oooo01P0oooo0P00000E0?ooo`030000003oool0oooo01H0oooo00<0o`00
0?ooo`3oool09@3oool2000004`0oooo000Q0?ooo`040=nLB0000000000007Booa40oooo00P0oono
07@000000000B9cO0?ooo`3o_g@00000001d_`L0oooo00L0gmoO0?ooo`3ookl0M0000000000007@0
_ooo00D0oooo00<000000?ooo`3oool0:`3oool00`3o0000oooo0?ooo`0G0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0G0?ooo`030?l0003oool0
oooo02/0oooo00<000000?ooo`3oool0J03oool0103OW4P000000000001d_ol90?ooo`070?noM000
0000M;oo0?ooo`3ookl0M00007Boo`060?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02o
ool60?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool01P3oool00`3o0000oooo0?oo
o`0C0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`000000503oool00`000000oooo0?oo
o`0F0?ooo`030?l0003oool0oooo02H0oooo00<000000?ooo`3oool0BP3oool003L0oooo00<0_g@0
0018W02oW4P00P0000000`00B9`0gooo0?ooo`060?ooo`060=nLB000000000000000000004P0W=oo
1P3oool00`000000oooo0?ooo`0[0?ooo`030?l0003oool0oooo01L0oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo01L0oooo00<0o`000?ooo`3oool0
:`3oool00`000000oooo0?ooo`1f0?ooo`050?noM0000000000000000018W=l0203oool01P3ooml0
W4P0000000000000001d0;ooo`L0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`06
0?ooo`030?l0003oool0oooo01<0oooo00<000000?ooo`3oool0603oool00`000000oooo0000000D
0?ooo`030000003oool0oooo01H0oooo00<0o`000?ooo`3oool09P3oool00`000000oooo0?ooo`1:
0?ooo`00=`3oool00`3ogi`0B00007Boo`0;0?ooo`040?oo_`1d0000000004RLg`P0oooo0P00000/
0?ooo`030?l0003oool0oooo01L0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo01L0oooo00<0o`000?ooo`3oool0:P3oool200000980oooo00<00000
0?ooo`3oool07@3oool00`000000oooo0?ooo`050?ooo`030?l0003oool0oooo01<0oooo00<00000
0?ooo`3oool0603oool00`000000oooo0000000C0?ooo`030000003oool0oooo01L0oooo00<0o`00
0?ooo`3oool09P3oool00`000000oooo0?ooo`1:0?ooo`00>03oool00`3OW4P0001809cOo`0;0?oo
o`030;md001d_ol0oooo00P0oooo00<000000?ooo`3oool0:`3oool00`3o0000oooo0?ooo`0F0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0G0?oo
o`030?l0003oool0oooo02/0oooo00<000000?ooo`3oool0T03oool00`000000oooo0?ooo`0N0?oo
o`030000003oool0oooo00@0oooo00<0o`000?ooo`3oool04P3oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0000001<0oooo00<000000?ooo`3oool05`3oool00`3o0000oooo0?ooo`0U0?oo
o`800000C03oool003T0oooo00<0okmd00000000M;l02P3oool01P3ogi`0B0000000000000000018
09cOo`D0oooo00@000000?ooo`3oool0oooo4P0000040?ooo`@000001`3oool4000000D0oooo00<0
o`000000000000000`0000060?ooo`D000001P3oool4000000H0oooo1@0000050?ooo`H000001P3o
ool4000000<0oooo00<000000?ooo`3oool01@0000060?ooo`D000001P3oool4000000L0oooo1000
00060?oooaD00000103oool00`000000oooo0?ooo`2@0?ooo`030000003oool0oooo01h0oooo00<0
00000?ooo`3oool00`3oool00`3o0000oooo0?ooo`0C0?ooo`030000003oool0oooo01T0oooo00@0
00000?ooo`3oool000004P3oool00`000000oooo0?ooo`0G0?ooo`030?l0003oool0oooo02H0oooo
00<000000?ooo`3oool0BP3oool003X0oooo00D0oooO09a800000000001d0;ooo`080?ooo`050;md
00000000000000000018W=l01@3oool2000001@0oooo1@00000C0?ooo`030?l0003oool0oooo01H0
oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01H0
oooo00<0o`000?ooo`3oool04@3oool5000001@0oooo0P00001h0?ooo`050=nLB000000000000000
0018W=l00`3oool00`3o_g@0000007Boo`050?ooo`040?oog`2LB00004RL0=ooo`H0oooo00<00000
0?ooo`3oool07`3oool00`000000oooo0?ooo`020?ooo`030?l0003oool0oooo01<0oooo00<00000
0?ooo`3oool06@3oool010000000oooo0?ooo`00000B0?ooo`030000003oool0oooo01P0oooo00<0
o`000?ooo`3oool09@3oool00`000000oooo0?ooo`1:0?ooo`00D@3oool00`000000oooo0?ooo`0H
0?ooo`8000004@3oool00`3o0000oooo0?ooo`0E0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0F0?ooo`030?l0003oool0oooo00l0oooo0P00000J
0?ooo`030000003oool0oooo07D0oooo00L0gia80000001d_ol0oooo0?oog`2LB000B9cO0080oooo
00<0okmd0000001d_ol01@3oool0103ooml0W4P00018W03Oool60?ooo`030000003oool0oooo01l0
oooo00<000000?ooo`3oool00P3oool00`3o0000oooo0?ooo`0C0?ooo`030000003oool0oooo01T0
oooo00@000000?ooo`3oool000004@3oool00`000000oooo0?ooo`0I0?ooo`030?l0003oool0oooo
02@0oooo0P00001<0?ooo`006P3oool01@3ogi`0M9cO0?ooo`3ookl0M0000080000000<0001d0;oo
o`3oool0;@3oool2000001/0oooo0P00000?0?ooo`030?l0003oool0oooo01D0oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01D0oooo00<0o`000?oo
o`3oool02`3oool4000001/0oooo0P00001S0?ooo`050?oOW01dW=l0oooo0?oo_`1d00000P000000
0`0007@0_ooo0?ooo`090?ooo`030?oog`2LB00007Bo00<0oooo00<0okmd0018W03Oool02@3oool0
103ooml0W4P00018W03Oool60?ooo`030000003oool0oooo0200oooo00D000000?ooo`3oool0oooo
0?l0000E0?ooo`030000003oool0oooo01T0oooo00D000000?ooo`3oool0oooo0000000@0?ooo`03
0000003oool0oooo01T0oooo00<0o`000?ooo`3oool09@3oool00`000000oooo0?ooo`1:0?ooo`00
6P3oool00`3ooml0W4P0000000030000000407Bo_`1d000004RL0=ooo`/0oooo00D0omnL04P00000
0000000007Boo`030?ooo`030?noM0000000M;oo00@0oooo00<0gia804RLg`3oool00`3oool00`3o
oml0W4P0000000020000000307Boo`3oool0oooo00D0oooo00<000000?ooo`3oool0703oool20000
00d0oooo00<0o`000?ooo`3oool05@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool05@3oool00`3o0000oooo0?ooo`0:0?ooo`030000003oool0oooo
01h0oooo00<000000?ooo`3oool0H@3oool00`3ooml0W4P0000000030000000407Bo_`1d000004RL
0=ooo`X0oooo00<0_g@0001d_`3oool00P3oool00`2oM00004RL0=ooo`050?ooo`030;md00000000
000000D0000000<0M;oo0?ooo`3oool00`3oool00`000000oooo0?ooo`0P0?ooo`050000003oool0
oooo0?ooo`3o0000503oool00`000000oooo0?ooo`0J0?ooo`050000003oool0oooo0?ooo`000000
403oool00`000000oooo0?ooo`0I0?ooo`030?l0003oool0oooo02D0oooo00<000000?ooo`3oool0
BP3oool00280oooo00H0_gA809cO_`1d000000000018W03Oool60?ooo`070?oOW0180000B9cO0?oo
o`3ookl0M00007Boo`020?ooo`030?noM0000000M;oo00@0oooo00<0omnL04Q8W03Oool00`3oool0
0`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0oooo00D0oooo00<000000?ooo`3oool07P3oool0
0`000000oooo0?ooo`0:0?ooo`030?l0003oool0oooo01@0oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01D0oooo00<0o`000?ooo`3oool02@3oool0
0`000000oooo0?ooo`0O0?ooo`030000003oool0oooo06T0oooo00H0_gA809cO_`1d000000000018
W03Oool60?ooo`070?oOW01807@0_ooo0?ooo`2oM000000007Boo`060?ooo`080?oOW01dW=l0oooo
0?ooo`3ooml0W4P00018W03Oool60?ooo`030000003oool0oooo0240oooo00@000000?ooo`3oool0
o`00503oool00`000000oooo0?ooo`0J0?ooo`050000003oool0oooo0?ooo`000000403oool00`00
0000oooo0?ooo`0I0?ooo`030?l0003oool0oooo02D0oooo00<000000?ooo`3oool0BP3oool00200
oooo00D0gia8000000000000001d0;ooo`080?ooo`030?oog`2LB00007Bo00<0oooo00<0okmd0018
W03Oool0203oool00`3ooml0W4Q809cOo`080?ooo`030?oog`2LB7@0_ooo00D0oooo0P00000P0?oo
o`8000002P3oool00`3o0000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?oo
o`3oool04P3oool00`000000oooo0?ooo`0E0?ooo`030?l0003oool0oooo00L0oooo00<000000?oo
o`3oool07`3oool2000006T0oooo00D0gia8000000000000001d0;ooo`0:0?ooo`050?oo_`1dB4P0
B00000000018W=l0203oool01`3OW4P0M;oo0?ooo`3ooml0W4P00018W03Oool01P3oool200000280
oooo00@000000?ooo`3oool0o`00503oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0J0?ooo`030?l0003oool0oooo
02@0oooo0P00001<0?ooo`00703oool0203o_g@00000000000000000B9cO0?ooo`2oM4P0W=oo2@3o
ool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00T0oooo00<0gia804RLg`3oool0203o
ool00`2oM000M;oo0?ooo`040?ooo`030000003oool0oooo0240oooo0P0000080?ooo`030?l0003o
ool0oooo01@0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo01D0oooo00<0o`000?ooo`3oool01P3oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo06<0oooo00P0okmd000000000000000004RLg`3oool0_gA809cOo`/0oooo00@0omnL04P0
0000000007Bo2@3oool01`3ooml0W4Qd0;ooo`3ooml0W4P00018W03Oool01P3oool00`000000oooo
0?ooo`0R0?ooo`030000003oool0o`0001@0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool06P3oool00`3o0000oooo
0?ooo`0U0?ooo`030000003oool0oooo04X0oooo000H0?ooo`060;md0000000000000000000007@0
_ooo0`3oool00`3ookl0M4RL0=ooo`090?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02L
gol02@3oool00`3ogi`0B4RL0=ooo`070?ooo`030?oog`2LB000B9cO00D0oooo00<000000?ooo`3o
ool08`3oool00`000000oooo0?ooo`040?ooo`030?l0003oool0oooo01@0oooo00<000000?ooo`3o
ool04P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01D0oooo00<0o`000?ooo`3o
ool01@3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo05l0oooo00H0_g@000000000
000000000000M02oool30?ooo`030?oo_`1dB9`0gooo00X0oooo00H0omnL04P0000007@0_ooo0?no
M000M;l90?ooo`060?oOW018M;l0oooO09a80000B9`0gooo1P3oool00`000000oooo0?ooo`0R0?oo
o`030000003oool0o`0001<0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool06P3oool00`3o0000oooo0?ooo`0U0?oo
o`030000003oool0oooo04X0oooo000F0?ooo`040?oog`2LB0000000001d_`H0oooo00<0oooO09a8
0018W=l02P3oool00`3ogi`0B00007Boo`030?ooo`030;md000004P0W=oo00T0oooo00<0oooO09a8
B02Lgol01`3oool00`3OW4P0000007Boo`050?ooo`8000009@3oool00`000000oooo0?ooo`030?oo
o`030?l0003oool0oooo01@0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo01@0oooo00<0o`000?ooo`3oool0103oool00`000000oooo0?ooo`0R0?oo
o`800000G`3oool0103ooml0W4P000000000M;l60?ooo`030?oog`2LB000B9cO00/0oooo00<0_g@0
0000M02oool00P3oool00`2oM000B9cO0?ooo`080?ooo`050=nLB01d_ml0W4P00018W03Oool01P3o
ool00`000000oooo0?ooo`0S0?ooo`030000003o0000oooo0180oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool06P3o
ool00`3o0000oooo0?ooo`0U0?ooo`030000003oool0oooo04X0oooo000N0?ooo`040;md00000000
0000001d_`/0oooo00P0oono07@000000000B9cO0?ooo`3o_g@00000001d_`/0oooo00<0gia8001d
_`3oool00P3oool01`3Ogml0oooo0?oo_`1d000000000000M02oool01@3oool00`000000oooo0?oo
o`0U0?ooo`030000003oool0oooo0080oooo00<0o`000?ooo`3oool0503oool00`000000oooo0?oo
o`0B0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0503oool00`3o0000oooo0?oo
o`030?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0I@3oool0102oM00000000000
0000M;l<0?ooo`030;md0000B9`0gooo0080oooo00<0gia8001d_`3oool0203oool01@3ooml0W4Q8
04Q80000B9`0gooo00H0oooo00<000000?ooo`3oool08`3oool00`000000o`000?ooo`0B0?ooo`03
0000003oool0oooo01/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo01/0oooo00<0o`000?ooo`3oool0903oool2000004`0oooo000K0?ooo`060?oo
g`2LB00000000000000004P0W=oo3@3oool00`2oM00004RL0;nLB002000000030018W03Oool0oooo
00X0oooo00<0oono07@0M02oool00P3oool01P3OW4P00000000000000000001809cOo`H0oooo00<0
00000?ooo`3oool09P3oool01@000000oooo0?ooo`3oool0o`0001D0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01@0oooo00<0o`000?ooo`3oool0
0P3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo0680oooo00H0oooO09a800000000
00000000B02Lgol=0?ooo`070?noM0000000M;oo0?ooo`3ookl0M00007Boo`0:0?ooo`040?oOW018
000004RL0=ooo`H0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo01/0oooo00<0o`000?ooo`3oool09@3oool00`000000oooo0?ooo`1:0?ooo`006P3o
ool0103ooml0W4P00018W03Oool@0?ooo`030?oOW0180000M;oo00X0oooo00<0omoO0=ooo`3oool0
0`3oool0202oM000M;oo0?ooo`3oool0oono07@000000000B9cO203oool2000002P0oooo00@00000
0?ooo`3oool0o`005@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01@0oooo00<0
00000?ooo`3oool04`3oool01@3o0000oooo0?ooo`3oool0000002L0oooo0P00001S0?ooo`040?oo
g`2LB00004RL0=oooa40oooo00D0okmd000000000000000004RLg`0<0?ooo`030=nLB000B9`0gooo
00H0oooo00<000000?ooo`3oool0903oool00`3o000000000?ooo`0@0?ooo`030000003oool0oooo
01`0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
01/0oooo00<0o`000?ooo`3oool09@3oool00`000000oooo0?ooo`1:0?ooo`006P3oool00`3ogi`0
M9cO0?ooo`0B0?ooo`030=nLB00004P0W=oo00T0oooo00<0oono07A8W03Oool00`3oool00`3o_g@0
07Bo0?ooo`020?ooo`030;md001d_ol0oooo00P0oooo00<000000?ooo`3oool09`3oool010000000
oooo0?ooo`3o000D0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`0C0?ooo`040?l0003oool0oooo000002T0oooo00<000000?ooo`3oool0H@3oool0
0`3ogi`0M9cO0?ooo`0/0?ooo`030000003oool0oooo02@0oooo00<0o`000000003oool0403oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0203oool0
0`000000oooo0?ooo`0L0?ooo`030?l0003oool0oooo02@0oooo0P00001<0?ooo`00<03oool00`3o
_g@00000001d_`090?ooo`030;md00000000000000<0000000H0001d0;ooo`3oool0oooo0?oOW018
0002000000030000B02Lgol0oooo00@0oooo0P00000Y0?ooo`030000003oool0o`0001@0oooo00<0
00000?ooo`3oool0503oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01<0oooo00<0
o`000?ooo`000000:@3oool200000980oooo00<000000?ooo`3oool0903oool00`3o0000oooo0000
000@0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo01`0oooo00<0o`000?ooo`3oool09@3oool00`000000oooo0?oo
o`1:0?ooo`00<@3oool01@3ooml0W4P00000000007@0_ooo00H0oooo00<0okmd0000000000001000
00001@1d_ol0oooo0?ooo`3oool0_g@000<0000000<0B9cO0?ooo`3oool00`3oool00`000000oooo
0?ooo`0Y0?ooo`030000003o0000oooo01<0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo0180oooo00<0o`000000003oool0:P3oool00`000000oooo
0?ooo`2@0?ooo`030000003oool0oooo02@0oooo00<0o`000?ooo`0000003`3oool00`000000oooo
0?ooo`0M0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0203oool00`000000oooo
0?ooo`0L0?ooo`030?l0003oool0oooo02D0oooo00<000000?ooo`3oool0BP3oool00540oooo00<0
00000?ooo`3oool0:@3oool00`000000o`000?ooo`0B0?ooo`030000003oool0oooo01D0oooo00<0
00000?ooo`3oool05@3oool00`000000oooo0?ooo`0B0?ooo`030?l000000000oooo02X0oooo00<0
00000?ooo`3oool0MP3oool01@3OW4P00000000000000000B9cO00<0oooo00<0okmd0000001d_ol0
0`3oool01@3ogi`0B000000000000000M;oo00L0oooo00<000000?ooo`3oool0903oool0103o0000
oooo0?ooo`00000>0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo01`0oooo00<0o`000?ooo`3oool09@3oool00`00
0000oooo0?ooo`1:0?ooo`00D@3oool2000002/0oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0E0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0Y0?ooo`800000M`3oool01`3OW4P0000007Boo`3oool0oooO09a80018W=l00P3oool0
0`3o_g@0000007Boo`020?ooo`070?oOW0180000B9cO0?ooo`3ookl0M00007Boo`060?ooo`030000
003oool0oooo02@0oooo00@0o`000?ooo`3oool000003P3oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0M0?ooo`03
0?l0003oool0oooo02@0oooo0P00001<0?ooo`00AP3oool00`3OW4P0B9cO0?ooo`080?ooo`030000
003oool0oooo02X0oooo00<0o`000000003oool04@3oool00`000000oooo0?ooo`0E0?ooo`030000
003oool0oooo01H0oooo00<000000?ooo`3oool0403oool00`000000o`000?ooo`0[0?ooo`030000
003oool0oooo07@0oooo00<0oooO09a80000M;l00`3oool00`3o_g@004RL0=ooo`050?ooo`030?oo
g`2LB00007Bo00<0oooo00<0okmd0018W03Oool01@3oool00`000000oooo0?ooo`0T0?ooo`050?l0
003oool0oooo0?ooo`000000303oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0M0?ooo`030?l0003oool0oooo02D0
oooo00<000000?ooo`3oool0BP3oool004H0oooo00<0omnL04Q8W03Oool0203oool00`000000oooo
0?ooo`0Z0?ooo`030?l0003oool000000100oooo00<000000?ooo`3oool05P3oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o0000;03oool00`000000oooo
0?ooo`1e0?ooo`030;md0000M;l0oooo0080oooo00<0_g@00018W03Oool01@3oool00`3ogi`0B000
07Boo`030?ooo`030?noM00004P0W=oo00D0oooo00<000000?ooo`3oool0903oool01@3o0000oooo
0?ooo`3oool0000000`0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool07P3oool00`3o0000oooo0?ooo`0U0?ooo`03
0000003oool0oooo04X0oooo00160?ooo`030?oog`2LB4P0W=oo00P0oooo0P00000[0?ooo`030?l0
003oool000000100oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0G0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3o0000:`3oool2000007L0oooo00L0omnL04P0M02oool0
oooo0;md00000000M;oo00H0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`050?oo
o`030000003oool0oooo02@0oooo00<0o`000?ooo`3oool00P3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo01h0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo01h0oooo00<0o`000?ooo`3oool09@3oool00`000000oooo0?ooo`1:0?oo
o`00A`3oool00`3OW4P0B9cO0?ooo`070?ooo`030000003oool0oooo02X0oooo00@0o`000?ooo`3o
ool000003`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool0303oool010000000oooo0?ooo`3o000/0?ooo`030000003oool0oooo07H0oooo00D0oono
07A8B0180000000004RLg`070?ooo`030?oOW0180000M;oo00<0oooo00<0_g@00000B02Lgol01@3o
ool2000002D0oooo00<0o`000?ooo`3oool00P3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo01l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo01l0oooo00<0o`000?ooo`3oool0903oool2000004`0oooo00170?ooo`030?oOW018B9`0
gooo00L0oooo00<000000?ooo`3oool0:P3oool01@3o0000oooo0?ooo`3oool0000000d0oooo00<0
00000?ooo`3oool05`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00`0oooo00@0
00000?ooo`3oool0o`00;03oool00`000000oooo0?ooo`1f0?ooo`040?oOW01800000000001d_`P0
oooo00P0oono07@000000000B9cO0?ooo`3o_g@00000001d_`H0oooo00<000000?ooo`3oool0903o
ool00`3o0000oooo0?ooo`030?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool07`3o
ool00`3o0000oooo0?ooo`0U0?ooo`030000003oool0oooo04X0oooo00170?ooo`030?oog`2LB4P0
W=oo00L0oooo0P00000[0?ooo`050?l0003oool0oooo0?ooo`0000003@3oool00`000000oooo0?oo
o`0G0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool02`3oool01@000000oooo0?oo
o`3oool0o`0002/0oooo0P00001g0?ooo`060?oOW0180000001d0;ooo`3o_g@007Bo203oool00`2o
M00004RL0;nLB002000000030018W03Oool0oooo00D0oooo00<000000?ooo`3oool0903oool00`3o
0000oooo0?ooo`030?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0803oool00`00
0000oooo0?ooo`080?ooo`D000008@3oool00`3o0000oooo0?ooo`0U0?ooo`030000003oool0oooo
04X0oooo00180?ooo`030=nLB000M;l0oooo00H0oooo00<000000?ooo`3oool0:P3oool00`3o0000
oooo0?ooo`020?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool00`3oool00`3o0000
oooo0?ooo`0Y0?ooo`030000003oool0oooo07D0oooo00<0_g@00000M02oool00P3oool00`2oM000
B9cO0?ooo`060?ooo`030?oOW0180000M;oo00X0oooo00<000000?ooo`3oool0903oool00`3o0000
oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool08@3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo0240oooo00<0o`000?ooo`3oool09@3oool00`000000
oooo0?ooo`1:0?ooo`00B03oool00`3ookl0M01d0;ooo`060?ooo`800000:P3oool00`3o0000oooo
0?ooo`040?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0103oool00`3o0000oooo
0?ooo`0X0?ooo`800000M`3oool00`2oM00004RL0=ooo`020?ooo`030=nLB000M;l0oooo00L0oooo
00<0gia80000B02Lgol02@3oool00`000000oooo0?ooo`0T0?ooo`030?l0003oool0oooo00D0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo02l0oooo
00<0o`000?ooo`3oool08`3oool2000004`0oooo00130?ooo`030?oOg`3Oool0oooo00<0oooo00<0
_g@007Boo`3oool01@3oool00`000000oooo0?ooo`0Y0?ooo`030?l0003oool0oooo00D0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030?l0003oool0oooo02T0oooo00<0
00000?ooo`3oool0M@3oool01`3o_g@0000007Boo`3oool0oono07@0001d_ol02@3oool00`3o_g@0
0000001d_`080?ooo`030000003oool0oooo02@0oooo00<0o`000?ooo`3oool01P3oool200000080
oooo0P00000S0?ooo`030000003oool0oooo02l0oooo00<0o`000?ooo`3oool0903oool00`000000
oooo0?ooo`1:0?ooo`00@`3oool00`3ookl0M4RL0=ooo`030?ooo`030?noM000M;l0oooo00D0oooo
00<000000?ooo`3oool0:@3oool00`3o0000oooo0?ooo`060?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool01P3oool00`3o0000oooo0?ooo`0Y0?ooo`030000003oool0oooo07H0oooo
00D0okmd000000000000000004RLg`0;0?ooo`050?oog`2LB00000000000M02oool01@3oool00`00
0000oooo0?ooo`0T0?ooo`030?l0003oool0oooo00L0oooo1000000T0?ooo`030000003oool0oooo
02l0oooo00<0o`000?ooo`3oool0903oool00`000000oooo0?ooo`1:0?ooo`00A03oool00`2oM000
000000000003000000030000M02oool0oooo00@0oooo0P00000Z0?ooo`030?l0003oool0oooo00L0
oooo0P000000103oool0000000000000000M0?ooo`030000003oool0oooo01`0oooo0P000000103o
ool000000000000000090?ooo`030?l0003oool0oooo02P0oooo0P00002B0?ooo`030000003oool0
oooo02@0oooo00<0o`000?ooo`3oool0;`3oool00`000000oooo0?ooo`0_0?ooo`030?l0003oool0
oooo02<0oooo0P00001<0?ooo`00A03oool00`3o_g@00000000000040000000307Boo`3oool0oooo
00<0oooo00<000000?ooo`3oool0:@3oool00`3o0000oooo0?ooo`090?ooo`8000007`3oool00`00
0000oooo0?ooo`0N0?ooo`8000002`3oool00`3o0000oooo0?ooo`0Y0?ooo`030000003oool0oooo
0900oooo00<000000?ooo`3oool0903oool00`3o0000oooo0?ooo`0_0?ooo`030000003oool0oooo
02l0oooo00<0o`000?ooo`3oool0903oool00`000000oooo0?ooo`1:0?ooo`00D@3oool00`000000
oooo0?ooo`0Y0?ooo`030?l0003oool0oooo02X0oooo00<000000?ooo`3oool0:`3oool00`3o0000
oooo0?ooo`0Y0?ooo`030000003oool0oooo0900oooo00<000000?ooo`3oool0903oool00`3o0000
oooo0?ooo`0_0?ooo`030000003oool0oooo02l0oooo00<0o`000?ooo`3oool0903oool00`000000
oooo0?ooo`1:0?ooo`00D@3oool2000002X0oooo00<0o`000?ooo`3oool0:P3oool00`000000oooo
0?ooo`0[0?ooo`030?l0003oool0oooo02P0oooo0P00001h0?ooo`050=nLB0000000000000000018
W=l00`3oool00`3o_g@0000007Boo`030?ooo`050=nLB0000000000000000018W=l01`3oool00`00
0000oooo0?ooo`0S0?ooo`030?l0003oool0oooo0300oooo00<000000?ooo`3oool0;`3oool00`3o
0000oooo0?ooo`0T0?ooo`030000003oool0oooo04X0oooo000`0?ooo`030=nLB018W=l0oooo00@0
oooo00D0okmd0000001d_ol0oooo0;md00060000000404RLg`3oool0oooO09a800<0000000<0M;oo
0?ooo`3oool01@3oool00`000000oooo0?ooo`0Y0?ooo`030?l0003oool0oooo02X0oooo00<00000
0?ooo`3oool0:`3oool00`3o0000oooo0?ooo`0Y0?ooo`030000003oool0oooo07D0oooo00L0gia8
0000001d_ol0oooo0?oog`2LB000B9cO0080oooo00<0okmd0000001d_ol00P3oool01`3OW4P00000
07Boo`3oool0oooO09a80018W=l01P3oool00`000000oooo0?ooo`0S0?ooo`030?l0003oool0oooo
0300oooo00<000000?ooo`3oool0;`3oool00`3o0000oooo0?ooo`0S0?ooo`800000C03oool00300
oooo00<0omnL04Q8W03Oool0103oool00`3o_g@0000007Boo`020?ooo`030=nLB0000000000000<0
000000D004RL0=ooo`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0oooo00D0oooo00<000000?oo
o`3oool0:@3oool00`3o0000oooo0?ooo`0Z0?ooo`030000003oool0oooo02/0oooo00<0o`000?oo
o`3oool0:@3oool00`000000oooo0?ooo`1d0?ooo`030?oog`2LB00007Bo00<0oooo00<0okmd0018
W03Oool01@3oool00`3ooml0W4P0001d_`030?ooo`030?noM000B9`0gooo00D0oooo00<000000?oo
o`3oool08`3oool00`3o0000oooo0?ooo`0`0?ooo`030000003oool0oooo02l0oooo00<0o`000?oo
o`3oool0903oool00`000000oooo0?ooo`1:0?ooo`00<03oool00`3ooml0W4Q809cOo`0:0?ooo`03
0=nLB018W=l0oooo0080oooo00<0giad09cOo`3oool0103oool00`3ooml0W4Qd0;ooo`050?ooo`80
0000:P3oool00`3o0000oooo0?ooo`0Z0?ooo`030000003oool0oooo02/0oooo00<0o`000?ooo`3o
ool0:03oool2000007L0oooo00<0_g@0001d_`3oool00P3oool00`2oM00004RL0=ooo`060?ooo`03
0;md0000M;l0oooo0080oooo00<0_g@00018W03Oool01@3oool00`000000oooo0?ooo`0S0?ooo`03
0?l0003oool0oooo0300oooo00<000000?ooo`3oool0;`3oool00`3o0000oooo0?ooo`0T0?ooo`03
0000003oool0oooo04X0oooo000a0?ooo`030=nLB018W=l0oooo00X0oooo00<0gia807Boo`3oool0
2@3oool00`2oM000M;oo0?ooo`040?ooo`030000003oool0oooo02T0oooo00<0o`000?ooo`3oool0
:P3oool00`000000oooo0?ooo`0[0?ooo`030?l0003oool0oooo02T0oooo00<000000?ooo`3oool0
M@3oool01`3ogi`0B01d0;ooo`3oool0_g@00000001d_ol01`3oool01`3ogi`0B01d0;ooo`3oool0
_g@00000001d_ol01P3oool00`000000oooo0?ooo`0S0?ooo`030?l0003oool0oooo0300oooo00<0
00000?ooo`3oool0;`3oool00`3o0000oooo0?ooo`0T0?ooo`030000003oool0oooo04X0oooo000a
0?ooo`030?oOW018B9`0gooo00/0oooo00<0_gA809cOo`3oool01`3oool00`3ooml0W4P004RLg`05
0?ooo`800000:P3oool00`3o0000oooo0?ooo`0Z0?ooo`030000003oool0oooo02/0oooo00<0o`00
0?ooo`3oool0:03oool2000007P0oooo00D0oono07A8B0180000000004RLg`090?ooo`050?oo_`1d
B4P0B00000000018W=l01`3oool2000002@0oooo00<0o`000?ooo`3oool0<03oool00`000000oooo
0?ooo`0_0?ooo`030?l0003oool0oooo02<0oooo0P00001<0?ooo`00<@3oool00`3ooml0W4Q809cO
o`0;0?ooo`030?oog`2LB7@0_ooo00L0oooo00<0gia80000001d_ol01@3oool00`000000oooo0?oo
o`0Y0?ooo`030?l0003oool0oooo02X0oooo00<000000?ooo`3oool0:`3oool00`3o0000oooo0?oo
o`0Y0?ooo`030000003oool0oooo07H0oooo00@0omnL04P00000000007Bo2P3oool0103ogi`0B000
00000000M;l80?ooo`030000003oool0oooo02<0oooo00<0o`000?ooo`3oool0<03oool00`000000
oooo0?ooo`0_0?ooo`030?l0003oool0oooo02@0oooo00<000000?ooo`3oool0BP3oool00380oooo
00<0gia8001d_`3oool02`3oool00`3o_g@007Bo0?ooo`020?ooo`070=oOg`3oool0oono07@00000
0000001d0;ooo`050?ooo`030000003oool0oooo02T0oooo00<0o`000?ooo`3oool0:P3oool00`00
0000oooo0?ooo`0[0?ooo`030?l0003oool0oooo02T0oooo00<000000?ooo`3oool0M@3oool01P3o
gi`0B0000000M02oool0okmd001d_`P0oooo00H0omnL04P0000007@0_ooo0?noM000M;l70?ooo`03
0000003oool0oooo02<0oooo00<0o`000?ooo`3oool0<03oool00`000000oooo0?ooo`0_0?ooo`03
0?l0003oool0oooo02@0oooo00<000000?ooo`3oool0BP3oool00380oooo00<0oono07@0M02oool0
2`3oool00`3ookl0M01809cOo`020?ooo`060=nLB000000000000000000004P0W=oo1P3oool20000
02X0oooo00<0o`000?ooo`3oool0:P3oool00`000000oooo0?ooo`0[0?ooo`030?l0003oool0oooo
02P0oooo0P00001g0?ooo`030;md000007@0_ooo0080oooo00<0_g@004RLg`3oool01P3oool00`2o
M000001d0;ooo`020?ooo`030;md0018W=l0oooo00D0oooo00<000000?ooo`3oool08`3oool00`3o
0000oooo0?ooo`0`0?ooo`030000003oool0oooo02l0oooo00<0o`000?ooo`3oool08`3oool20000
04`0oooo000]0?ooo`030?oOg`3Oool0oooo00<0oooo00<0_g@007Boo`3oool02P3oool00`3ookl0
M00004RLg`020?ooo`040?oo_`1d0000000004RLg`P0oooo00<000000?ooo`3oool0:@3oool00`3o
0000oooo0?ooo`0Z0?ooo`030000003oool0oooo02/0oooo00<0o`000?ooo`3oool0:@3oool00`00
0000oooo0?ooo`1e0?ooo`030;md0000B9`0gooo0080oooo00<0gia8001d_`3oool01P3oool00`2o
M00004RL0=ooo`020?ooo`030=nLB000M;l0oooo00D0oooo00<000000?ooo`3oool08`3oool00`3o
0000oooo0?ooo`0`0?ooo`030000003oool0oooo02l0oooo00<0o`000?ooo`3oool0903oool00`00
0000oooo0?ooo`1:0?ooo`00;@3oool00`3ookl0M4RL0=ooo`030?ooo`030?noM000M;l0oooo00D0
oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0B00004RLg`030?ooo`030;md001d_ol0oooo00P0
oooo00<000000?ooo`3oool0:@3oool00`3o0000oooo0?ooo`0Z0?ooo`030000003oool0oooo02/0
oooo00<0o`000?ooo`3oool0:@3oool00`000000oooo0?ooo`1e0?ooo`070?noM0000000M;oo0?oo
o`3ookl0M00007Boo`070?ooo`070?noM0000000M;oo0?ooo`3ookl0M00007Boo`060?ooo`030000
003oool0oooo02<0oooo00<0o`000?ooo`3oool0<03oool00`000000oooo0?ooo`0_0?ooo`030?l0
003oool0oooo02@0oooo00<000000?ooo`3oool0BP3oool002h0oooo00<0_g@00000000000000`00
00000`0007@0_ooo0?ooo`040?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02oool30?oo
o`060?oOW018000000000000000004P0W=oo1@3oool2000002T0oooo00<0o`000?ooo`3oool0:`3o
ool00`000000oooo0?ooo`0/0?ooo`030?l0003oool0oooo02L0oooo0P00001h0?ooo`050?noM000
0000000000000018W=l02@3oool01@3o_g@00000000000000000B9cO00L0oooo00<000000?ooo`3o
ool08`3oool00`3o0000oooo0?ooo`0`0?ooo`030000003oool0oooo02l0oooo00<0o`000?ooo`3o
ool0903oool00`000000oooo0?ooo`1:0?ooo`00;P3oool00`3o_g@00000000000040000000307Bo
o`3oool0oooo00@0oooo00H0oooO09a80000000000000000M02oool50?ooo`050;md000000000000
00000018W=l01@3oool00`000000oooo0?ooo`0X0?ooo`030?l0003oool0oooo02/0oooo00<00000
0?ooo`3oool0;03oool00`3o0000oooo0?ooo`0X0?ooo`030000003oool0oooo0900oooo00<00000
0?ooo`3oool08`3oool00`3o0000oooo0?ooo`0`0?ooo`030000003oool0oooo02l0oooo00<0o`00
0?ooo`3oool08`3oool2000004`0oooo001A0?ooo`030000003oool0oooo02P0oooo00<0o`000?oo
o`3oool0:`3oool00`000000oooo0?ooo`0/0?ooo`030?l0003oool0oooo02P0oooo00<000000?oo
o`3oool0T03oool00`000000oooo0?ooo`0S0?ooo`030?l0003oool0oooo0300oooo00<000000?oo
o`3oool0<03oool00`3o0000oooo0?ooo`0S0?ooo`030000003oool0oooo04X0oooo000i0?ooo`03
0=nLB018W=l0oooo00@0oooo00H0okmd0000001d_ol0oooo0?oog`2LB0030000000307Boo`3oool0
oooo00D0oooo0P00000Y0?ooo`030?l0003oool0oooo02/0oooo00<000000?ooo`3oool0;03oool0
0`3o0000oooo0?ooo`0W0?ooo`800000P`3oool01@3ookl0M00000000000B9`0gooo00X0oooo00<0
00000?ooo`3oool08`3oool00`3o0000oooo0?ooo`0`0?ooo`030000003oool0oooo0300oooo00<0
o`000?ooo`3oool08`3oool00`000000oooo0?ooo`1:0?ooo`00>@3oool00`3ogi`0B4RL0=ooo`04
0?ooo`070?noM0000000M;oo0?ooo`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0oooo00D0oooo
00<000000?ooo`3oool0:03oool00`3o0000oooo0?ooo`0[0?ooo`030000003oool0oooo02`0oooo
00<0o`000?ooo`3oool0:03oool00`000000oooo0?ooo`240?ooo`030=nLB0000000M;oo00T0oooo
00<000000?ooo`3oool08`3oool00`3o0000oooo0?ooo`0`0?ooo`030000003oool0oooo0300oooo
00<0o`000?ooo`3oool08`3oool00`000000oooo0?ooo`1:0?ooo`00>@3oool00`3ooml0W4Q809cO
o`0=0?ooo`030?oog`2LB7@0_ooo00D0oooo0P00000Y0?ooo`030?l0003oool0oooo02/0oooo00<0
00000?ooo`3oool0;03oool00`3o0000oooo0?ooo`0W0?ooo`800000Q`3oool00`3ookl0M00007Bo
o`080?ooo`030000003oool0oooo02<0oooo00<0o`000?ooo`3oool0<03oool00`000000oooo0?oo
o`0`0?ooo`030?l0003oool0oooo0280oooo0P00001<0?ooo`00>P3oool00`3OW4P0B9cO0?ooo`0=
0?ooo`030;md001d_ol0oooo00@0oooo00<000000?ooo`3oool0:03oool00`3o0000oooo0?ooo`0[
0?ooo`030000003oool0oooo02`0oooo00<0o`000?ooo`3oool0:03oool00`000000oooo0?ooo`26
0?ooo`030?oOW01804P0W=oo00L0oooo00<000000?ooo`3oool08`3oool00`3o0000oooo0?ooo`0`
0?ooo`030000003oool0oooo0300oooo00<0o`000?ooo`3oool08`3oool00`000000oooo0?ooo`1:
0?ooo`00>P3oool00`3ogi`0B4RL0=ooo`0<0?ooo`030?oog`2LB000B9cO00D0oooo00<000000?oo
o`3oool04P3oool00`000000oooo0?ooo`0C0?ooo`030?l0003oool0oooo00T0oooo00<000000?oo
o`3oool07`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo00X0oooo00<0o`000?oo
o`3oool04P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0880oooo00<0oooO09a8
000000000`0000000`00M;l0oooo0?ooo`050?ooo`030000003oool0oooo0180oooo00<000000?oo
o`3oool03P3oool00`3o0000oooo0?ooo`0>0?ooo`030000003oool0oooo01l0oooo00<000000?oo
o`3oool07`3oool00`000000oooo0?ooo`0>0?ooo`030?l0003oool0oooo00h0oooo00<000000?oo
o`3oool04P3oool00`000000oooo0?ooo`1:0?ooo`00>P3oool00`3ooml0W4Q809cOo`0<0?ooo`03
0=nLB0000000M;oo00D0oooo:`0000000`3o000000000000001J000000030?l000000000000002T0
0000Q03oool0202oM000000007Boo`3oool0oooO09a8000007@0_ooo1P3ooolV000000030?l00000
0000000006<0000000<0o`000000000000009000001<0?ooo`00>`3oool00`3OW4P007Bo0?ooo`07
0?ooo`070=oOg`3oool0oono07@000000000001d0;ooo`3o0?ooocd0oooo00<0omnL04P00018W=l0
0`3oool00`3o_g@0001809cOo`3o0?ooo`H0oooo000k0?ooo`030?oo_`1d07@0_ooo00L0oooo00H0
gia800000000000000000000B02Lgooo0?oooch0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0
001809cOo`3o0?ooo`H0oooo000f0?ooo`030?oOg`3Oool0oooo00<0oooo00<0_g@007Boo`3oool0
1P3oool0103ookl0M00000000018W=oo0?oood00oooo00<0omnL04P0001d_ol00`3oool00`3OW4P0
001d0;ooo`3o0?ooo`H0oooo000f0?ooo`030?oo_`1dB9`0gooo00<0oooo00<0okmd001d_`3oool0
1`3oool00`2oM000M;oo0?ooo`3o0?oood00oooo00<0oooO09a8001d_ol00P3oool0103ooml0W4P0
0018W03Ooooo0?ooo`H0oooo000g0?ooo`030;md00000000000000<0000000<0001d0;ooo`3oool0
1P3oool01P3ogi`0B000000000000000001809cOoh/0oooo00<0oooO09a8000000000`0000000`18
W=l0oooo0=nLB0020000000307Boo`3oool0oooo0:@0oooo00L0okmd0000M02oool0oooo0=nLB000
04P0W=oo08`0oooo00<0oooO09a8000000000`0000000`18W=l0oooo0=nLB0020000000307Boo`3o
ool0oooo06`0oooo000g0?ooo`030?noM0000000000000@0000000<0M;oo0?ooo`3oool01P3oool0
1@2oM0000000000000000000B9cO08/0oooo00<0_g@00018W03Oool00`3oool00`3OW9`0_ooo0?oo
o`020?ooo`030;mdM02oool0oooo0:D0oooo00D0okmd000000000000001809cOo`2=0?ooo`030;md
0000B9`0gooo00<0oooo00<0gibL0;ooo`3oool00P3oool00`2oM7@0_ooo0?ooo`1/0?ooo`00e`3o
ool00`3ooml0W7BL0=ooo`030?ooo`030?oo_`2LW=l0oooo0080oooo00<0omnL07BLg`3oool0o`3o
oolh0?ooo`030?oog`2LM9`0gooo00<0oooo00<0oono09bLg`3oool00P3oool00`3ogi`0M9cO0?oo
o`1/0?ooo`00f03oool00`2oM0000000000000030000000704RLg`3oool0oooo0?ooo`3ooml0W4Qd
0;ooo`3o0?ooocT0oooo00<0_g@00000000000000`0000001`18W=l0oooo0?ooo`3oool0oooO09a8
M02oool0K03oool002<0oooo00<0_gA809cOo`3oool00P3oool00`3ooml0W7BL0=ooo`040?ooo`08
0?oog`2LB000000000000000B9`0W7@000000000M;l30?ooo`030?oog`2LB7@0_ooo00D0oooo00D0
_g@0000000000000001d0;ooo`040?ooo`030?oog`2LB00000000080000000<0001809cOo`3oool0
103oool02@3ookl0M000000000000000B9bo09a80000B9`0gooo0;md0002000000030000M02ooml0
W4P00080000000@004RL0=ooo`3oool0_g@00P0000002@0004P0W=oo0?ooo`3oool0omnL04P00000
0000B9cO0=nLB002000000040000M02oool0oooo0=nLB0<0000000@0M;oo0?ooo`3oool0_g@00P00
00000`0007@0_ooO09a80002000000030018W03Oool0oooo00H0oooo00<0omnL07BLg`3oool00P3o
ool00`3OW7@0W=oo0?ooo`020?ooo`0:0=nLB000000000000000M02oool0omnL04P000000000B9cO
0=nLB080000000<0001809cOg`2LB0000P0000000`00M;l0oooo0?ooo`030?ooo`030?oog`2LB000
000000L0000000<007Bo0?ooo`2oM0000P0000000`0007@0_ooO09a80002000000040018W03Oool0
oooo0;md0080000000@0001d0;ooo`3oool0_g@00P0000001@0004P0W=oo0?ooo`3ookl0M00000@0
000000<0001d0;ooo`3oool00`3oool00`2oM000M;oo0?ooo`0I0?ooo`050?oOW018000000000000
001d_ol00`3oool01P3o_g@0000007Boo`3oool0oooO09a800<0000000<0M;oo0?ooo`3oool00`3o
ool01@3OW4P00000000000000000B9cO00@0oooo00D0omnL04P000000000000007Boo`020?ooo`03
0;md00000000000000@0000000D0B9cO0?ooo`3oool0oooo0;md0003000000030018W03Oool0oooo
0300oooo00<0_gA809cOo`3oool00P3oool00`3ooml0W7BL0=ooo`040?ooo`080?oog`2LB0000000
00000000B9`0W7@000000000M;l30?ooo`030?oog`2LB7@0_ooo00D0oooo00D0_g@0000000000000
001d0;ooo`050?ooo`030?oog`2LB00000000080000000<0001809cOo`3oool0103oool02@3ookl0
M000000000000000B9bo09a80000B9`0gooo0;md0002000000030000M02ooml0W4P00080000000@0
04RL0=ooo`3oool0_g@00P0000002@0004P0W=oo0?ooo`3oool0omnL04P000000000B9cO0=nLB002
000000040000M02oool0oooo0=nLB0<0000000@0M;oo0?ooo`3oool0_g@00P0000000`0007@0_ooO
09a80002000000030018W03Oool0oooo00H0oooo00<0omnL07BLg`3oool00P3oool00`3OW7@0W=oo
0?ooo`020?ooo`0:0=nLB000000000000000M02oool0omnL04P000000000B9cO0=nLB080000000<0
001809cOg`2LB0000P0000000`00M;l0oooo0?ooo`020?ooo`030?oog`2LB000000000L0000000<0
07Bo0?ooo`2oM0000P0000000`0007@0_ooO09a80002000000040018W03Oool0oooo0;md00800000
00@0001d0;ooo`3oool0_g@00P0000001@0004P0W=oo0?ooo`3ookl0M00000@0000000<0001d0;oo
o`3oool00`3oool00`2oM000M;oo0?ooo`0I0?ooo`050=nLB0000000000000000018W=l00`3oool0
0`3o_g@0000007Boo`030?ooo`080;md00000000000000000000B9`0gooo0?ooo`2oM00600000003
04RLg`3oool0oooo00<0oooo00L0oooO09a80000B9`0gooo0?ooo`3ookl0M00000<0000000T0M;oo
0?ooo`3oool0oooo0?oo_`1d000000000018W03Oool07@3oool00280oooo00<0oono07@00018W=l0
0`3oool00`3o_g@0001809cOo`040?ooo`090?noM0000000M;oo0?ooo`3ogi`0B000001d_`3o_kl0
_ooo0080oooo00<0okmd00000018W=l0103oool01`2oM0000000001d_`3oool0omnL04P0M02oool0
1@3oool00`3ogi`0B00004RLg`070?ooo`050=nLB00007@0_ooo0?ooo`3OW4P00P0000002@0007@0
_ooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`0H0;md0000000004RL0=ooo`3ooml0
W4Qd0;ooo`3oool0gia80000M02oool0gkno0=ooo`3OW4P0001d0;ooo`3oool0gia800000018W=l0
oooo0?oog`2LB000B9cO0P3oool01`3OW4P004RL0=ooo`3oool0oooO09a80000M;l02@3oool01`2o
M000001d0;ooo`3oool0oono07@00000M;l0103oool0403OW4P0001d0;ooo`3oool0gia80000M02o
ool0gkno0=ooo`3OW4P0001d0;ooo`3oool0oooO09a80000M;l80?ooo`030?oOW0180000B9cO00@0
oooo00/0omnL04P0M02oool0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`0;0;md
0000000007Bo0?ooo`3ogi`0B01d0;ooo`3oool0gia80000M02oool00`3oool00`2oM4P0W=oo0?oo
o`070?ooo`040?oOW018000004RL0=oooaP0oooo00L0omnL04P00018W=l0oooo0?oo_`1d0000M;oo
0080oooo00L0okmd0000001d_ol0oooo0?oog`2LB7@0_ooo0080oooo00<0_g@007Boo`3oool00P3o
ool01`3OW4P0000007Boo`3oool0oooO09a80018W=l00P3oool01`3ogi`0B00004RLg`3oool0oono
07@0001d_ol00P3oool00`3OW4P0000000000003000000030018W03Oool0oooo0080oooo00<0omnL
04P0001d_ol0<P3oool00`3ookl0M00004RLg`030?ooo`030?noM00004P0W=oo00@0oooo00T0okmd
0000001d_ol0oooo0?oOW018000007Bo0?no_`2oool00P3oool00`3o_g@0000004RLg`040?ooo`07
0;md0000000007Bo0?ooo`3ogi`0B01d0;ooo`060?ooo`030?oOW0180000B9cO00L0oooo00D0gia8
0000M02oool0oooo0=nLB002000000090000M02oool0gia80018W03Oool0oooo0?oog`2LB00007Bo
0080oooo01P0_g@000000000B9`0gooo0?oog`2LB7@0_ooo0?ooo`3OW4P0001d0;ooo`3O_kl0gooo
0=nLB00007@0_ooo0?ooo`3OW4P0000004RLg`3oool0oooO09a80018W=l20?ooo`070=nLB000B9`0
gooo0?ooo`3ooml0W4P0001d_`090?ooo`070;md000007@0_ooo0?ooo`3ookl0M000001d_`040?oo
o`0@0=nLB00007@0_ooo0?ooo`3OW4P0001d0;ooo`3O_kl0gooo0=nLB00007@0_ooo0?ooo`3ooml0
W4P0001d_`L0oooo00<0omnL04P00018W=l0103oool02`3ogi`0B01d0;ooo`3oool0gia80018W03O
ool0oooo0?oog`2LB00007Bo0080oooo00/0_g@000000000M;l0oooo0?oOW01807@0_ooo0?ooo`3O
W4P0001d0;ooo`030?ooo`030;mdB02Lgol0oooo00L0oooo00@0omnL04P00000B9`0gooo603oool0
1`3OW4P0000007Boo`3oool0oooO09a80018W=l00P3oool00`3o_g@0000007Boo`040?ooo`030?oO
W0180000M;oo00@0oooo00<0gia80000000000000`0000000`00B9`0gooo0?ooo`030?ooo`080?oo
g`2LB00004RL0=ooo`3oool0omnL04P00018W=l20?ooo`030;md0018W=l0oooo00@0oooo00<0gia8
0000001d_ol0703oool00280oooo00@0okmd00000000B9`0gooo0P3oool00`2oM0000000001d_`04
0?ooo`070?noM00004P0W=oo0?ooo`3ooml0W4P0001d_`030?ooo`050?oog`2LB00004RL09adM02o
ool00P3oool00`3ogi`0B00004RLg`030?ooo`030?oOW01dW=l0oooo00@0oooo00<0oono07@00018
W=l01`3oool01`2oM00004RL0=ooo`3oool0oooO09a80000M;l00P3oool02`3OW4P004RL0=ooo`3o
ool0oooO09a80000M;l0oooo0?oOW018000007Bo00<0oooo00H0omnL09boo`3oool0gia80018W03O
ool20?ooo`060=nLB00007@0_ooo0?oOW018000007Bo0`3oool02`3o_g@0001d0;ooo`3oool0gia8
0018W03Oool0oooo0?oog`2LB00007Bo00P0oooo00T0omnL04P00018M7@0M9cO0?ooo`3o_g@00018
04Q8B02Lgol00`3oool01`3OW4P0001d0;ooo`3oool0gia80018W03Oool00P3oool01`3OW4P0001d
0;ooo`3oool0oooO09a80000M;l0203oool00`3ookl0M00004RLg`050?ooo`0>0?noM01d_ol0oooo
0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0omnL04P00018W=l30?ooo`060?oOW01dW=l0
oooo0=nLB00007@0_ooo0`3oool01@3ookl0M00000000000000007Bo00D0oooo00@0_g@00018W02L
M000M;oo1P3oool00`3ooml0W4P0000000070000000304RLg`3oool0oooo00@0oooo00<0oooO09a8
0000M;l00`3oool00`3o_g@004RL0=ooo`0:0?ooo`070?oog`2LB7@0_ooo0?ooo`3ooml0W4P0001d
_`030?ooo`060?noM000B9`0gooo0?oog`2LB00007Bo0`3oool00`3o_g@004RL0=ooo`020?ooo`03
0=nLB018W=l0oooo0080oooo00<0giad09cOo`3oool00P3oool00`3ogi`0B00007Boo`0b0?ooo`04
0?noM000000004RL0=ooo`80oooo00<0_g@000000000M;l0103oool01`3o_g@0001809cOo`3oool0
oooO09a80000M;l00`3oool01@3ooml0W4P00018W02LM7@0_ooo0080oooo00<0omnL04P00018W=l0
0`3oool00`3ogi`0M9cO0?ooo`050?ooo`030?oo_`1d0000B9cO00L0oooo00L0_g@00018W03Oool0
oooo0?oog`2LB00007Bo0080oooo00/0gia80018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3ogi`0
B000001d_`030?ooo`060?oOW02L_ol0oooo0=nLB000B9`0gooo0P3oool01P3OW4P0001d0;ooo`3o
gi`0B000001d_`<0oooo00/0okmd0000M02oool0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d
_`080?ooo`090?oOW0180000B7Ad07BLg`3oool0okmd0000B018B4P0W=oo00<0oooo00L0gia80000
M02oool0oooo0=nLB000B9`0gooo0080oooo00L0gia80000M02oool0oooo0?oog`2LB00007Bo00L0
oooo00<0oono07@00018W=l01@3oool03P3o_g@0M;oo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a8
0000M;l0oooo0?oOW0180000B9cO0`3oool01P3ogi`0M9cO0?ooo`3OW4P0001d0;ooo`<0oooo00D0
oono07@0000000000000001d_`050?ooo`040;md0000B9`0W7@007Boo`L0oooo00<0oooO09a80000
00001`0000000`18W=l0oooo0?ooo`030?ooo`030?oog`2LB00007Bo00<0oooo00<0okmd0018W03O
ool0203oool00`3ogi`0B00007Boo`050?ooo`030=nLB018W=l0oooo0080oooo00<0giad09cOo`3o
ool00`3oool0103ooml0W4P00018W03Oool60?ooo`030?oo_`1d04P0W=oo00D0oooo00<0oono07@0
001d_ol06`3oool00240oooo00/0oooO09a8000007@0W9ad09cOo`3oool0oono07@00000M9`0W4Qd
0;ooo`040?ooo`060?noM000000007Bo0?oog`2LB00007Bo0`3oool01@3ogi`0B00007Boo`3o_g@0
B9cO0080oooo00<0okmd0000B02Lgol02P3oool00`3ookl0M00004RLg`070?ooo`070;md0000B9`0
gooo0?ooo`3ooml0W4P0001d_`020?ooo`0;0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0
okmd0000B02Lgol01P3oool00`3OW4P004RL0=ooo`020?ooo`060=nLB00007@0_ooo0?noM0000000
M;oo0`3oool02`3o_g@0000007Boo`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo00P0oooo
00T0gia80000M02oool0_gAd0;oog`2LB00007Bo0?oOW01dW=l00`3oool01`3OW4P0001d0;ooo`3o
ool0gia80018W03Oool00P3oool01`3OW4P0001d0;ooo`3oool0oooO09a80000M;l0203oool00`3o
okl0M00004RLg`080?ooo`0;0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd0000B02L
gol01P3oool00`3OW4P0001d0;ooo`030?ooo`060=nLB00007@0_ooo0?oo_`1d0000B9cO0`3oool0
1@3o_g@0000007Boo`3ogi`0B7Bo01L0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cO
o`0;0?ooo`060;md001d_ol0oooo0?ooo`2oM00007Bo0`3oool01P2oM00004RL0=ooo`3ogi`0B000
07Boo`<0oooo00<0okmd0000B02Lgol00`3oool00`3OW4P0M;oo0?ooo`060?ooo`030?oOW0180000
M;oo0340oooo00/0oooO09a8000007@0W9ad09cOo`3oool0oono07@00000M9`0W4Qd0;ooo`040?oo
o`060?noM000000007Bo0?oog`2LB00007Bo0`3oool01@3ogi`0B00007Boo`3o_g@0B9cO0080oooo
00<0okmd0000B02Lgol02`3oool00`3ookl0M00004RLg`070?ooo`070;md0000B9`0gooo0?ooo`3o
oml0W4P0001d_`020?ooo`0;0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd0000B02L
gol01P3oool00`3OW4P004RL0=ooo`020?ooo`060=nLB00007@0_ooo0?noM0000000M;oo0`3oool0
2`3o_g@0000007Boo`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo00P0oooo00T0gia80000
M02oool0_gAd0;oog`2LB00007Bo0?oOW01dW=l00`3oool01`3OW4P0001d0;ooo`3oool0gia80018
W03Oool00P3oool01`3OW4P0001d0;ooo`3oool0oooO09a80000M;l01`3oool00`3ookl0M00004RL
g`080?ooo`0;0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd0000B02Lgol01P3oool0
0`3OW4P0001d0;ooo`030?ooo`060=nLB00007@0_ooo0?oo_`1d0000B9cO0`3oool01@3o_g@00000
07Boo`3ogi`0B7Bo01P0oooo00<0_g@0001d_`3oool00P3oool00`2oM00004RL0=ooo`080?ooo`03
0?oOW0180000M;oo00H0oooo00<0gia807Boo`3oool00`3oool00`2oM00000000000000500000003
07Boo`3oool0oooo00<0oooo00<0oono07@0001d_ol01P3oool00`3ogi`0B01809cOo`0J0?ooo`00
8@3oool02`3ogi`0B00004RLg`3ogi`0B7Bo0?ooo`3OW4P0000007Boo`3o_g@0M;oo00H0oooo00@0
okmd00000000000007Bo0`3oool02P2oM00004RL0=ooo`3ooml0W4Qd0;ooo`3oool0okmd0000M02o
ool:0?ooo`030?oo_`1d0000B9cO0080oooo00<0omnL07BLg`3oool00P3oool01`2oM00004RL0=oo
o`3oool0oooO09a80000M;l00P3oool02`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?no
M00007@0_ooo00H0oooo00<0gia80018W03Oool00P3oool01P3OW4P0001d0;ooo`3o_g@0000007Bo
o`<0oooo00/0okmd0000001d_ol0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`070?ooo`0;
0?oo_`1d0000B9cO0?ooo`3ogi`0M7Ad04P0B02Lgol0oooo0;mdM02oool00P3oool01`3OW4P0001d
0;ooo`3oool0gia80018W03Oool00P3oool01`3OW4P0001d0;ooo`3oool0oooO09a80000M;l0203o
ool00`3ookl0M00004RLg`030?ooo`030?noM01d_ol0oooo0080oooo00/0gia80018W03Oool0oooo
0?oog`2LB00007Bo0?ooo`3o_g@0001d0;ooo`060?ooo`030=nLB00007@0_ooo0080oooo00L0oono
07@00018W=l0oooo0?oog`2LB00007Bo0080oooo00L0oooO09a80000B9`0gooo0?ooo`2oM4P0W=oo
01H0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0:0?ooo`030?oog`2LB000B9cO
0080oooo00/0omnL04P0M02oool0oooo0;md00000000M;oo0?ooo`3ogi`0B00007Boo`030?ooo`03
0?noM00004P0W=oo00@0oooo00<0_gA809cOo`3oool01@3oool00`3ogi`0B00007Boo`0a0?ooo`0;
0?oOW0180000B9cO0?oOW018M;l0oooo0=nLB0000000M;oo0?noM01d_ol01P3oool0103o_g@00000
00000000M;l30?ooo`0:0;md0000B9`0gooo0?oog`2LB7@0_ooo0?ooo`3o_g@0001d0;ooo`/0oooo
00<0oono07@00018W=l00P3oool00`3ogi`0M9cO0?ooo`020?ooo`070;md0000B9`0gooo0?ooo`3o
oml0W4P0001d_`020?ooo`0;0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd0000M02o
ool01P3oool00`3OW4P004RL0=ooo`020?ooo`060=nLB00007@0_ooo0?noM0000000M;oo0`3oool0
2`3o_g@0000007Boo`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo00L0oooo00/0oono07@0
0018W=l0oooo0?oOW01dM7@0B01809cOo`3oool0_gAd0;ooo`020?ooo`070=nLB00007@0_ooo0?oo
o`3OW4P004RL0=ooo`020?ooo`070=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`070?ooo`030?oo
_`1d0000B9cO00<0oooo00<0okmd07Boo`3oool00P3oool02`3OW4P004RL0=ooo`3oool0oooO09a8
0000M;l0oooo0?noM00007@0_ooo00H0oooo00<0gia80000M02oool00P3oool01`3ookl0M00004RL
g`3oool0oooO09a80000M;l00P3oool01`3ooml0W4P00018W03Oool0oooo0;mdB02Lgol05`3oool0
1`3ogi`0B01d0;ooo`3oool0_g@00000001d_ol02@3oool00`3ogi`0B00007Boo`070?ooo`030;md
B02Lgol0oooo0080oooo00P0omnL07BLg`3oool0oooo0?oog`2LB00004RL0=ooo`H0oooo00<0okmd
0000001d_ol00P3oool00`3ooml0W4P00000000300000003001d_`3oool0oooo01P0oooo000Q0?oo
o`0;0=nLB00004P0W=oo0?oog`2LB7@0_ooO09a8000007@0_ooo0?oo_`1dM;l00`3oool01`3ookl0
M00007Boo`3oool0oooO09a80000M;l00P3oool00`3ogi`0B00007Boo`020?ooo`050?noM018W=l0
oooo0?oOW01800001@0000000`18W=l0oooo0?ooo`030?ooo`030?oo_`1d0000B9cO0080oooo00<0
okmd04RLg`3oool00P3oool01`2oM00004RL0=ooo`3oool0oooO09a80000M;l00P3oool02`2oM000
001d0;ooo`3oool0oooO09a80000M;l0oooo0?oOW01804P0W=oo0080oooo00L0okmd0000B02Lgol0
oooo0=nLB000B9`0gooo0080oooo00H0gia80000M02oool0oono07@0001d_ol30?ooo`0;0;md0000
04P0W=oo0?ooo`2oM000001d0;ooo`3oool0oooO09a80000M;l01`3oool01`3OW4P0001809cOo`3o
ool0oooO09a80000M;l00P3oool02`3o_g@0B9cO0?ooo`3oool0gia80000M02oool0oooo0=nLB000
B9`0gooo0080oooo00L0gia80000M02oool0oooo0?oog`2LB00007Bo00P0oooo00<0oono07@00018
W=l00`3oool00`2oM000M;oo0?ooo`020?ooo`0:0;md000007@0_ooo0?ooo`3ooml0W4P0001d_`3o
ool0omnL04P000D0000000D0B9cO0?ooo`3OW4P0001809cOo`020?ooo`070?oo_`1d0000B9cO0?oo
o`3ooml0W4P004RLg`020?ooo`030?noM00004P0W=oo0080oooo00<0okmd04RLg`3oool05@3oool0
0`3ogi`0B00007Boo`030?ooo`030;md000004P0W=oo00X0oooo00<0gia80000001d_ol00`3oool0
1@3ookl0M4Q804P000000000B9cO0080oooo00<0omnL04P0001d_ol00`3oool00`2oM000001809cO
o`040?ooo`030?oog`2LB7@0_ooo00D0oooo00<0omnL04P0001d_ol0<@3oool02`3OW4P0001809cO
o`3ooml0W4Qd0;oog`2LB000001d0;ooo`3ookl0M7Bo00<0oooo00L0oono07@0001d_ol0oooo0?oo
g`2LB00007Bo0080oooo00<0omnL04P0001d_ol00P3oool01@3o_g@0B9cO0?ooo`3ogi`0B00000D0
000000<0B9cO0?ooo`3oool0103oool00`3ookl0M00004RLg`020?ooo`030?noM018W=l0oooo0080
oooo00L0_g@00018W03Oool0oooo0?oog`2LB00007Bo0080oooo00/0_g@00000M02oool0oooo0?oo
g`2LB00007Bo0?ooo`3ogi`0B01809cOo`020?ooo`070?noM00004P0W=oo0?ooo`3OW4P004RL0=oo
o`020?ooo`060=nLB00007@0_ooo0?oo_`1d0000M;oo0`3oool02`2oM000001809cOo`3oool0_g@0
0000M02oool0oooo0?oog`2LB00007Bo00L0oooo00L0gia80000B02Lgol0oooo0?oog`2LB00007Bo
0080oooo00/0okmd04RLg`3oool0oooo0=nLB00007@0_ooo0?ooo`3OW4P004RL0=ooo`020?ooo`07
0=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`070?ooo`030?oo_`1d0000B9cO00<0oooo00<0_g@0
07Boo`3oool00P3oool02P2oM000001d0;ooo`3oool0oooO09a80000M;l0oooo0?oOW01800050000
000504RLg`3oool0gia80000B02Lgol00P3oool01`3ookl0M00004RLg`3oool0oooO09a80018W=l0
0P3oool00`3o_g@0001809cOo`020?ooo`030?noM018W=l0oooo01L0oooo00D0oono07A8B0180000
000004RLg`0:0?ooo`030?oOW0180000M;oo00L0oooo00<0oooO09a8M02oool00`3oool01`3OW4P0
M;oo0?ooo`3ooml0W4P00018W03Oool01@3oool0103ogi`0B0000000B02Lgol20?ooo`080;md0000
0000M;oo0?ooo`3ooml0W4P00000M02ooolI0?ooo`00803oool02@3ookl0M0000018W03Oool0oooo
0?noM01dW9`0B00004RLg`020?ooo`0@0;mdB02Lgol0oooo0?ooo`3ookl0M00004RLg`3oool0oono
07@00000M;l0oooo0?oog`2LB000001d0;ooo`80oooo00H0oooO09a8M02oool0oooo0;md0018W=l2
0?ooo`030=nLB00004P0W=oo00D0oooo00<0oono07@0000000000`0000005018W=l0oooo0?ooo`3o
ool0_g@00018W03Oool0oooo0?oog`2LB00007Bo0?oog`2LB00000000018B01804P0W=oo0?oo_`1d
0000B9cO0P3oool08@2oM00004RL0=ooo`3ooml0W4P00000M02oool0oooo0=nLB000B9`0gooo0?oo
g`2LB00000000000M02oool0oooo0=nLB000B9`0gooo0?ooo`3o_g@00000001d_`3ooml0W4P00000
0000B4P0B01809cOo`3ookl0M00004RLg`060?ooo`030?oo_`1d000007Bo0080oooo00<0omnL04P0
001d_ol00P3oool02`3ooml0W4Qd0;oog`2LB00000000000M02oool0oooo0=nLB000B9`0gooo0080
oooo00L0gia80000000004P0W=oo0?oo_`1d0000B9cO00P0oooo00<0oono07@00000000010000000
2`1d_ol0oooo0?oog`2LB00000000018B01804P0W=oo0?oo_`1d0000B9cO0080oooo0180_g@004RL
g`3oool0oooo0=nLB00004P0W=oO09a800000000001804QdW03O_g@0001d0;ooo`2oM00007Bo0?oo
o`3o_g@2000000050018W03Ookl0M0000018W03Oool00P3oool00`3ookl0M01d0;ooo`040?ooo`03
0?oog`2LB000000000L0000000<0B9cO0?ooo`3oool0103oool0203ookl0M00000000018W=l0oooo
0?noM000000007Bo1`3oool01`3Ogml0oooo0?oo_`1d000000000000M02oool00`3oool0103ogi`0
B00000000000M;l30?ooo`080?oo_`1d0000000004RLg`3oool0okmd00000000M;l60?ooo`030?no
M000M;l0oooo00@0oooo00<0omnL04P0001d_ol0<03oool02@3ookl0M0000018W03Oool0oooo0?no
M01dW9`0B00004RLg`020?ooo`0@0;mdB02Lgol0oooo0?ooo`3ookl0M00004RLg`3oool0oono07@0
0000M;l0oooo0?oog`2LB000001d0;ooo`80oooo00H0oooO09a8M02oool0oooo0;md0018W=l20?oo
o`030=nLB00004P0W=oo00H0oooo00<0oono07@0000000000`0000005018W=l0oooo0?ooo`3oool0
_g@00018W03Oool0oooo0?oog`2LB00007Bo0?oog`2LB00000000018B01804P0W=oo0?oo_`1d0000
B9cO0P3oool08@2oM00004RL0=ooo`3ooml0W4P00000M02oool0oooo0=nLB000B9`0gooo0?oog`2L
B00000000000M02oool0oooo0=nLB000B9`0gooo0?ooo`3o_g@00000001d_`3ooml0W4P000000000
B4P0B01809cOo`3ookl0M00004RLg`060?ooo`030?oo_`1d000007Bo0080oooo00<0omnL04P0001d
_ol00P3oool02`3ooml0W4Qd0;oog`2LB00000000000M02oool0oooo0=nLB000B9`0gooo0080oooo
00L0gia80000000004P0W=oo0?oo_`1d0000B9cO00L0oooo00<0oono07@000000000100000002`1d
_ol0oooo0?oog`2LB00000000018B01804P0W=oo0?oo_`1d0000B9cO0080oooo0180_g@004RLg`3o
ool0oooo0=nLB00004P0W=oO09a800000000001804QdW03O_g@0001d0;ooo`2oM00007Bo0?ooo`3o
_g@2000000050018W03Ookl0M0000018W03Oool00P3oool00`3ookl0M01d0;ooo`050?ooo`030?oo
g`2LB000000000L0000000<0B9cO0?ooo`3oool01@3oool0103ogi`0B00000000000M;l;0?ooo`03
0?oOW0180000M;oo00P0oooo00<0okmd001d_`3oool00P3oool01`3ooml0W4Qd0;ooo`3ooml0W4P0
0018W03Oool00`3oool01@3o_g@000000000000004P0W=oo0080oooo00<0omnL04P00018W=l00`3o
ool00`3o_g@0001809cOo`0I0?ooo`00803oool00`3o_g@0000004RLg`020?ooo`040?oo_`1d0000
001809cOo`80oooo00<0okmd04RLg`3oool00P3oool0203ookl0M000000000000000001d0;ooo`3o
oml0W4P00P0000002@0007@0_ooo0?oog`2LB00000000018W03Oool0oooo0=nLB002000000030000
B02Lgol0oooo00D0oooo00<0oono07@00018W=l00P3oool0303o_g@0B9cO0?ooo`3ookl0M0000000
0000B9`0gooo0?oOW01800000000001d_`80oooo00L0_g@00018W03Ooml0W4P00000000007@0_ooo
00<0oooo00L0gia8000000000000001809cOo`3oool0_g@000<0000000D004RL0=ooo`2oM000001d
0;ooo`020?ooo`050?noM0000000000000000018W=l00`3oool01`2oM00004RL0=oog`2LB0000000
0000M02oool01@3oool01P3ooml0W4P0000000000000B9cO0?noM0<0000000X0B9cO0?ooo`2oM000
00000018W03Oool0_g@00000M02oool0_g@00`0000002@00B9`0gooo0=nLB00007@0_ooO09a80000
0000001d0;ooo`080?ooo`030?oo_`1d0000B9cO00<0oooo00<0_g@007Boo`3oool00P3oool01`2o
M00004RL0=oog`2LB00000000000M02oool00`3oool01@3OW4P000000000000004P0W=oo0080oooo
00P0_g@00000M02ogi`0B0000000M02oool0oooO09a80080000000@0M;oo0?ooo`3ooml0W4P00P00
00001`00B9`0gooo0?ooo`2oM00000000018W03Oool05@3oool00`2oM00004RL0;nLB00200000003
0018W03Oool0oooo00H0oooo00H0gia800000000000000000000B02Lgol30?ooo`060?oOW0180000
001d0;ooo`3o_g@007Bo0`3oool00`2oM00004RL0;nLB002000000030018W03Oool0oooo00D0oooo
00<0oono07@0B02Lgol0103oool00`3ogi`0B00007Boo`0`0?ooo`030?noM0000000B9cO0080oooo
00@0oono07@0000004P0W=oo0P3oool00`3o_g@0B9cO0?ooo`020?ooo`080?oo_`1d000000000000
000007@0_ooo0?oog`2LB002000000090000M02oool0oooO09a80000000004RL0=ooo`3oool0gia8
0080000000<0001809cOo`3oool01P3oool00`3ookl0M00004RLg`020?ooo`0<0?noM018W=l0oooo
0?oo_`1d000000000018W03Oool0omnL04P00000000007Bo0P3oool01`2oM00004RL0=oog`2LB000
00000000M02oool00`3oool01`3OW4P000000000000004P0W=oo0?ooo`2oM0000`0000001@00B9`0
gooo0;md000007@0_ooo0080oooo00D0okmd000000000000000004RLg`030?ooo`070;md0000B9`0
gooO09a800000000001d0;ooo`050?ooo`060?oog`2LB000000000000018W=l0okmd0`0000002P18
W=l0oooo0;md0000000004RL0=ooo`2oM000001d0;ooo`2oM003000000090018W03Oool0gia80000
M02ooml0W4P00000000007@0_ooo00L0oooo00<0oono07@00018W=l00`3oool00`2oM000M;oo0?oo
o`020?ooo`070;md0000B9`0gooO09a800000000001d0;ooo`030?ooo`050=nLB000000000000000
B02Lgol00P3oool0202oM000001d0;oOW0180000001d0;ooo`3ooml0W4P00P000000101d_ol0oooo
0?oog`2LB002000000070018W03Oool0oooo0;md0000000004RL0=ooo`0E0?ooo`060?oOW0180000
001d0;ooo`3o_g@007Bo2P3oool00`3ogi`0B00007Boo`080?ooo`030?oo_`1d04P0W=oo00<0oooo
00H0omnL04Qd_`3ooml0W4P00018W03Oool50?ooo`030;md0000M;l0oooo0080oooo00<0omnL04P0
001d_ol00`3oool00`3o_g@0001809cOo`0I0?ooo`00803oool00`2oM000001d0;ooo`020?ooo`04
0?oo_`1d000004RL0=ooo`80oooo00<0oono07A8W03Oool07`3oool00`3ookl0M00004RLg`020?oo
o`030?oOW01dW=l0oooo01`0oooo00<0_g@00018W03Oool0;P3oool00`2oM00004RL0=ooo`020?oo
o`030=nLB00007@0_ooo00`0oooo00<0oono07@00018W=l00`3oool00`3o_g@0M;oo0?ooo`0o0?oo
o`030?oOW0180000M;oo00/0oooo00@0oono07@000000000B9cO1@3oool00`2oM000001d0;ooo`02
0?ooo`070;md0018W=l0oooo0?ooo`3ogi`0B00007Boo`0:0?ooo`030?oo_`1d0000B9cO00@0oooo
00<0omnL04P0001d_ol0<03oool00`2oM000001d0;ooo`020?ooo`040?oo_`1d000004RL0=ooo`80
oooo00<0oono07A8W03Oool0803oool00`3ookl0M00004RLg`020?ooo`030?oOW01dW=l0oooo01`0
oooo00<0_g@00018W03Oool0;P3oool00`2oM00004RL0=ooo`020?ooo`030=nLB00007@0_ooo00/0
oooo00<0oono07@00018W=l00`3oool00`3o_g@0M;oo0?ooo`0o0?ooo`030;md000007@0_ooo0080
oooo00<0_g@004RLg`3oool0203oool00`3ogi`0B00007Boo`080?ooo`030?oo_`1d0000B9cO00@0
oooo00D0gia807Bog`2LB00004RL0=ooo`060?ooo`070;md0018W=l0oooo0?ooo`3ogi`0B00007Bo
o`030?ooo`030=nLB00007@0_ooo01T0oooo000O0?ooo`030?oOW018000007Bo00<0oooo00<0okmd
00000018W=l0103oool00`3OW7@0W=oo0?ooo`0N0?ooo`030?oo_`1d0000B9cO00@0oooo00<0giad
09cOo`3oool06P3oool00`3ookl0M4RL0=ooo`0^0?ooo`030?oo_`1dB9`0gooo0080oooo00<0gia8
0000M02oool0303oool00`3ookl0M00004RLg`040?ooo`030?oo_`1dM;l0oooo03l0oooo00<0gia8
0000B02Lgol02`3oool00`2oM000M;oo0?ooo`050?ooo`030;md0000B9`0gooo0080oooo00<0gia8
001d_`3oool00P3oool00`3OW4P0001809cOo`040?ooo`030?noW02Lgol0oooo0080oooo00<0omnL
04P00018W=l0103oool00`3ogi`0B00007Boo`0_0?ooo`030?oOW018000007Bo00<0oooo00<0okmd
00000018W=l0103oool00`3OW7@0W=oo0?ooo`0O0?ooo`030?oo_`1d0000B9cO00@0oooo00<0giad
09cOo`3oool06P3oool00`3ookl0M4RL0=ooo`0^0?ooo`030?oo_`1dB9`0gooo0080oooo00<0gia8
0000M02oool02`3oool00`3ookl0M00004RLg`040?ooo`030?oo_`1dM;l0oooo03h0oooo00<0_g@0
0018W03Oool00P3oool00`3OW4P007Bo0?ooo`080?ooo`030?oOW0180000M;oo00<0oooo00<0oknL
09cOo`3oool00P3oool00`3ogi`0B00004RLg`040?ooo`080?oog`2LB4P0B4P00018W03Oool0oooo
0?oog`2LW;l30?ooo`070=nLB00007@0_ooo0?ooo`3ooml0W4P007Boo`020?ooo`040?oog`2LB000
04RL0=oooaT0oooo000O0?ooo`030=nLB0000000B9cO0080oooo00@0oooO09a800000000M;oo103o
ool00`3OW4P007Bo0?ooo`0N0?ooo`030?oOW0180000B9cO00<0oooo00<0oono07@0M02oool06`3o
ool00`3ooml0W7BL0=ooo`0^0?ooo`030?oog`2LM9`0gooo0080oooo00<0gia80000M02oool0303o
ool00`3ogi`0B00004RLg`040?ooo`030=nLB018W=l0oooo0400oooo00<0okmd00000000M;l02P3o
ool01P3ogi`0B000000000000000001809cOo`80oooo00L0okmd0000001d_ol0oooo0?oo_`1d0000
M;oo00@0oooo00<0okmd00000000M;l00`3oool0203ooml0W4Q809cOo`3oool0oono07@0000007@0
_ooo0P3oool01@3ooml0W4P0000000000000M;oo02l0oooo00<0gia800000018W=l00P3oool0103o
oml0W4P00000001d_ol40?ooo`030=nLB000M;l0oooo01l0oooo00<0omnL04P00018W=l00`3oool0
0`3ookl0M01d0;ooo`0K0?ooo`030?oog`2LM9`0gooo02h0oooo00<0oooO09adW03Oool00P3oool0
0`3OW4P0001d0;ooo`0;0?ooo`030?oOW0180000B9cO00@0oooo00<0gia804RLg`3oool0?P3oool0
1`3o_g@0000007Boo`3oool0oono07@0001d_ol01`3oool01@3ooml0W4P0000000000000M;oo00<0
oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`D0oooo00@0omnL04P00000B9`0gooo
0P3oool01P3o_g@0B9cO0?ooo`3ookl0M000001d_`<0oooo00L0okmd0000M02oool0oooo0=nLB000
04P0W=oo01X0oooo000M0?ooo`030?oog`2LB00000000080000000<007Bo0?oOW01800000`000000
1018W=l0oooo0?ooo`2oM002000000030018W03Oool0oooo01X0oooo00<0oooO09a8000000001P00
00000`0007@0_ooo0?ooo`0O0?ooo`030=nLB00004P0W=oo02D0oooo00<0gia80000B02Lgol0103o
ool01@3ooml0W4P00000000007@0_ooo00X0oooo00<0oooO09a8000000001`0000000`18W=l0oooo
0?ooo`100?ooo`050?oog`2LB00000000000M02oool0203oool01@2oM0000000000000000000B9cO
00<0oooo00D0okmd000000000000000004RLg`060?ooo`080?oog`2LB00000000000M02oool0oooo
0?oog`2LB002000000030000M02oool0oooo00@0oooo00<0okmd0000001d_ol0;@3oool00`3ooml0
W4P00000000200000003001d_`3ogi`0B00000<0000000@0B9cO0?ooo`3oool0_g@00P0000000`00
B9`0gooo0?ooo`0K0?ooo`030?oog`2LB000000000H0000000<0001d0;ooo`3oool07`3oool00`3O
W4P0001809cOo`0U0?ooo`030=nLB00004P0W=oo00@0oooo00D0oooO09a800000000001d0;ooo`09
0?ooo`030?oog`2LB000000000L0000000<0B9cO0?ooo`3oool0?P3oool01@3o_g@0000000000000
0000B9cO00X0oooo00<0okmd0000001d_ol0103oool01P3ooml0W4P0000000000000001d0;ooo`L0
oooo00<0gia80018W03Oool00`3oool01@3OW4P000000000000007@0_ooo00@0oooo00D0okmd0000
00000000001809cOo`0K0?ooo`00MP3oool00`3OW4P0001809cOo`0U0?ooo`030=nLB00004P0W=oo
00H0oooo00<0_g@00000M02oool0o`3ooolC0?ooo`030=nLB00004P0W=oo02D0oooo00<0gia80000
B02Lgol01P3oool00`2oM000001d0;ooo`2V0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0oooo
V03oool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0
ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0ooooo`3ooonH
0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0ooooo`3ooonH0?ooo`00o`3o
oooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0
oooo003o0?ooool0ooooV03oool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?oo
ool0ooooV03oool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03o
ool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0oooo
o`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool00:40oooo00D0_g@00000
0000M;l0oooo0=nLB002000000030018W03Oool0oooo0?l0oooo@`3oool01@2oM0000000001d_`3o
ool0gia80080000000<004RL0=ooo`3oool0W`3oool00:40oooo00P0oooO09a8M02oool0oooo0?oo
g`2LB00004RL0=ooool0ooooA@3oool0203ooml0W4Qd0;ooo`3oool0oooO09a80000B9`0goooX@3o
ool00:80oooo00H0oono07A8W03Oool0_g@00000M02ooooo0?ooodL0oooo00H0oono07A8W03Oool0
_g@00000M02ooonR0?ooo`00X`3oool0103ogi`0M4Q80000001d_ooo0?ooodT0oooo00@0omnL07A8
B0000000M;ooX`3oool00:<0oooo00<0oono07@00018W=l0o`3ooom:0?ooo`030?oo_`1d0000B9cO
0:@0oooo002R0?ooo`050?oog`2LB00004RL09adB02Lgol0o`3ooom80?ooo`050?oog`2LB00004RL
09adB02Lgol0X`3oool00:80oooo00H0_g@00000B02Lgol0oooO09a8M02ooooo0?ooodL0oooo00H0
_g@00000B02Lgol0oooO09a8M02ooonR0?ooo`00X03oool02@3ooml0W4P000000000000007Bo0?no
M0000000001d0;ooo`3o0?oood@0oooo00T0oooO09a8000000000000001d_`3o_g@000000000M02o
ool0X@3oool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool0
0?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0ooooo`3o
oonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0ooooo`3ooonH0?ooo`00
o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oo
oiP0oooo003o0?ooool0ooooV03oool00?l0ooooo`3ooonH0?ooo`00J03oool0103ooml0W4P00018
W03OoolJ0?ooo`030;md00000000000000@0000000<0B9cO0?ooo`3oool04`3oool01@3ogi`0B000
0000000007@0_ooo01/0oooo00<0_g@0000000000000100000000`18W=l0oooo0?ooo`0L0?ooo`04
0?oog`2LB00004RL0=ooolL0oooo00@0oooO09a80000B9`0gooo6P3oool00`2oM000000000000004
0000000304RLg`3oool0oooo01<0oooo00D0omnL04P000000000001d0;ooo`0K0?ooo`030;md0000
0000000000@0000000<0B9cO0?ooo`3oool0703oool0103ooml0W4P00018W03OoomM0?ooo`00J03o
ool0103ooml0W4P00018W03OoolK0?ooo`030=nLB0000000000000<0000000<004RL0=ooo`3oool0
4P3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool06`3oool00`3OW4P00000000000030000
00030018W03Oool0oooo01`0oooo00@0oooO09a80000B9`0goooa`3oool0103ooml0W4P00018W03O
oolK0?ooo`030=nLB0000000000000<0000000<004RL0=ooo`3oool04P3oool01`3ookl0M00007Bo
o`3oool0okmd0000M02oool06`3oool00`3OW4P0000000000003000000030018W03Oool0oooo01`0
oooo00@0oooO09a80000B9`0goooG@3oool006P0oooo00@0oooO09a80000B9`0gooo703oool00`3O
W4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo0180oooo00<0_g@00018W03Oool00P3oool00`2o
M00007Bo0?ooo`0K0?ooo`030=nLB018W=l0oooo0080oooo00<0giad09cOo`3oool0703oool0103o
oml0W4P00018W03Oooo70?ooo`040?oog`2LB00004RL0=oooa`0oooo00<0gia804RLg`3oool00P3o
ool00`3OW7@0W=oo0?ooo`0B0?ooo`030;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool06`3o
ool00`3OW4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo01`0oooo00@0oooO09a80000B9`0gooo
G@3oool006@0oooo00<0_g@00000000000001@0000000`1d_ol0oooo0?ooo`0J0?ooo`030=nLB01d
_ol0oooo01D0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`0L0?ooo`030=nLB01d
_ol0oooo01`0oooo00<0_g@00000000000001@0000000`1d_ol0oooo0?ooo`300?ooo`030;md0000
0000000000D0000000<0M;oo0?ooo`3oool06P3oool00`3OW4P0M;oo0?ooo`0E0?ooo`030?oo_`1d
0000B9cO00<0oooo00<0gia80000M02oool0703oool00`3OW4P0M;oo0?ooo`0L0?ooo`030;md0000
0000000000D0000000<0M;oo0?ooo`3oool0FP3oool005L0oooo00<0oooO09a8000000001`000000
1@18W=l0oooo0?ooo`3ogi`0M9cO0080oooo00@0oooO09a80000B9`0gooo3@3oool00`3ooml0W4P0
000000070000000304RLg`3oool0oooo00@0oooo00<0_gA809cOo`3oool0503oool00`3ogi`0B000
07Boo`030?ooo`030?noM00004P0W=oo01d0oooo00<0_gA809cOo`3oool06`3oool0203ogi`0M9cO
0?ooo`3oool0oooO09a80000B9`0gooo]P3oool00`3ooml0W4P0000000070000000504RLg`3oool0
oooo0?oOW01dW=l00P3oool0103ooml0W4P00018W03Oool=0?ooo`030?oog`2LB000000000L00000
00<0B9cO0?ooo`3oool0103oool00`2oM4P0W=oo0?ooo`0D0?ooo`030?oOW0180000M;oo00<0oooo
00<0okmd0000B02Lgol07@3oool00`2oM4P0W=oo0?ooo`0K0?ooo`080?oOW01dW=l0oooo0?ooo`3o
oml0W4P00018W03OoomM0?ooo`00I@3oool01`3OW4P0M;oo0?ooo`3ooml0W4P00018W03Oool07P3o
ool00`3ooml0W4Qd0;ooo`0D0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol07@3o
ool00`3ooml0W4Qd0;ooo`0L0?ooo`070=nLB01d_ol0oooo0?oog`2LB00004RL0=ooo`340?ooo`07
0=nLB01d_ol0oooo0?oog`2LB00004RL0=ooo`0N0?ooo`030?oog`2LB7@0_ooo01@0oooo00<0omnL
04P0001d_ol00`3oool00`3o_g@0001809cOo`0M0?ooo`030?oog`2LB7@0_ooo01`0oooo00L0gia8
07Boo`3oool0oooO09a80000B9`0gooo05d0oooo001U0?ooo`070?oog`2LB7@0_ooo0?oog`2LB000
04RL0=ooo`0O0?ooo`030?noM000M;l0oooo01<0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0
001809cOo`0N0?ooo`030?noM000M;l0oooo01/0oooo00L0oooO09a8M02oool0oooO09a80000B9`0
gooo0<@0oooo00L0oooO09a8M02oool0oooO09a80000B9`0gooo01l0oooo00<0okmd001d_`3oool0
4`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo01h0oooo00<0okmd001d_`3oool0
6`3oool01`3ooml0W4Qd0;ooo`3ooml0W4P00018W03Oool0G@3oool006H0oooo00H0omnL04Qd_`3o
oml0W4P00018W03OoolO0?ooo`030?oo_`1d04P0W=oo01<0oooo00<0omnL04P0001d_ol00`3oool0
0`3o_g@0001809cOo`0N0?ooo`030?oo_`1d04P0W=oo01`0oooo00H0omnL04Qd_`3ooml0W4P00018
W03Oooo50?ooo`060?oOW018M;l0oooO09a80000B9`0gooo7`3oool00`3ookl0M01809cOo`0C0?oo
o`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol07P3oool00`3ookl0M01809cOo`0L0?oo
o`060?oOW018M;l0oooO09a80000B9`0goooG@3oool006L0oooo00D0gia807Bog`2LB00004RL0=oo
o`0O0?ooo`030?oo_`1d0000B9cO01<0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;oo
o`0N0?ooo`030?oo_`1d0000B9cO01d0oooo00D0gia807Bog`2LB00004RL0=ooo`360?ooo`050=nL
B01d_ml0W4P00018W03Oool07`3oool00`3ookl0M00004RLg`0C0?ooo`030?oo_`1d0000B9cO00<0
oooo00<0gia80000M02oool07P3oool00`3ookl0M00004RLg`0M0?ooo`050=nLB01d_ml0W4P00018
W03Oool0G@3oool006L0oooo00D0oooO09a8B018B00004RL0=ooo`0J0?ooo`030?noW02Lgol0oooo
0080oooo00<0omnL04P00018W=l0503oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo
01T0oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0B00004RLg`0M0?ooo`050?oog`2LB4P0B4P0
0018W03Oool0aP3oool01@3ooml0W4Q804Q80000B9`0gooo01X0oooo00<0oknL09cOo`3oool00P3o
ool00`3ogi`0B00004RLg`0D0?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d_`3oool06@3o
ool00`3o_i`0W=oo0?ooo`020?ooo`030?oOW0180000B9cO01d0oooo00D0oooO09a8B018B00004RL
0=ooo`1M0?ooo`00J03oool0103ogi`0B0000018W03OoolJ0?ooo`080?oog`2LB4P0W=oo0?ooo`3o
okl0M0000000M02ooolD0?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`0J0?ooo`080?oo
g`2LB4P0W=oo0?ooo`3ookl0M0000000M02ooolN0?ooo`040?oOW018000004RL0=ooolL0oooo00@0
omnL04P00000B9`0gooo6P3oool0203ooml0W4Q809cOo`3oool0oono07@0000007@0_ooo503oool0
1`3ookl0M00007Boo`3oool0okmd0000M02oool06P3oool0203ooml0W4Q809cOo`3oool0oono07@0
000007@0_ooo7P3oool0103ogi`0B0000018W03OoomM0?ooo`00J@3oool00`3OW4P004RL0=ooo`0K
0?ooo`060?oog`2LB00000000000000007@0_ooo5P3oool01@3ookl0M0000000000007@0_ooo01`0
oooo00H0oooO09a80000000000000000M02ooolP0?ooo`030=nLB000B9`0gooo0<P0oooo00<0gia8
0018W03Oool06`3oool01P3ooml0W4P0000000000000001d0;oooaH0oooo00D0oono07@000000000
001d0;ooo`0L0?ooo`060?oog`2LB00000000000000007@0_ooo803oool00`3OW4P004RL0=ooo`1M
0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0ooooo`3ooonH0?ooo`00o`3o
oooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0
oooo001=0?oook<00000V@3ooonc000004X0oooo001=0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool07`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo00<0
00000?ooo`3oool07`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo09L0oooo00<0
00000?ooo`3oool04P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo00<0
00000?ooo`3oool07`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool0B03oool004d0oooo00<000000?ooo`3oool0EP3oool00`000000oooo0?ooo`1F
0?ooo`030000003oool0oooo09L0oooo00<000000?ooo`3oool0EP3oool00`000000oooo0?ooo`1F
0?ooo`030000003oool0oooo04P0oooo001=0?ooo`030000003oool0oooo05H0oooo00<000000?oo
o`3oool0E@3oool2000009T0oooo00<000000?ooo`3oool0EP3oool00`000000oooo0?ooo`1F0?oo
o`030000003oool0oooo04P0oooo001=0?ooo`030000003oool0oooo03l0oooo0`3o000D0?ooo`03
0000003oool0oooo0180oooo0`3o00110?ooo`030000003oool0oooo09L0oooo00<000000?ooo`3o
ool0?`3oool40?l001<0oooo00<000000?ooo`3oool04P3oool30?l00440oooo00<000000?ooo`3o
ool0B03oool004d0oooo00<000000?ooo`3oool0?P3oool20?l00080oooo00<0o`000?ooo`3oool0
4@3oool00`000000oooo0?ooo`0A0?ooo`040?l0003oool0oooo0?l00440oooo00<000000?ooo`3o
ool0U`3oool00`000000oooo0?ooo`0o0?ooo`040?l0003oool0oooo0?l001<0oooo00<000000?oo
o`3oool04@3oool0103o0000oooo0?ooo`3o00110?ooo`030000003oool0oooo04P0oooo001=0?oo
o`030000003oool0oooo03h0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?ooo`0@0?oo
o`030000003oool0oooo0100oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?ooo`0n0?oo
o`030000003oool0oooo09L0oooo00<000000?ooo`3oool0?P3oool00`3o0000oooo0?ooo`020?oo
o`030?l0003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`3o0000oooo0?ooo`020?oo
o`030?l0003oool0oooo03h0oooo00<000000?ooo`3oool0B03oool004d0oooo00<000000?ooo`3o
ool0?@3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0oooo0100oooo00<000000?ooo`3o
ool0403oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0oooo03d0oooo0P00002I0?ooo`03
0000003oool0oooo03h0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?ooo`0@0?ooo`03
0000003oool0oooo0100oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?ooo`0n0?ooo`03
0000003oool0oooo04P0oooo001=0?ooo`030000003oool0oooo03d0oooo00<0o`000?ooo`3oool0
0`3oool00`3o0000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<0o`000?ooo`3oool0
0P3oool00`3o0000oooo0?ooo`0n0?ooo`030000003oool0oooo09L0oooo00<000000?ooo`3oool0
?P3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0oooo0100oooo00<000000?ooo`3oool0
403oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0oooo03h0oooo00<000000?ooo`3oool0
B03oool004d0oooo00<000000?ooo`3oool0?@3oool00`3o0000oooo0?ooo`040?ooo`030?l0003o
ool0oooo00l0oooo00<000000?ooo`3oool03`3oool00`3o0000oooo0?ooo`040?ooo`030?l0003o
ool0oooo03d0oooo00<000000?ooo`3oool0U`3oool00`000000oooo0?ooo`0m0?ooo`030?l0003o
ool0oooo00@0oooo00<0o`000?ooo`3oool03`3oool00`000000oooo0?ooo`0?0?ooo`030?l0003o
ool0oooo00@0oooo00<0o`000?ooo`3oool0?@3oool00`000000oooo0?ooo`180?ooo`00C@3oool0
0`000000oooo0?ooo`0m0?ooo`030?l0003oool0oooo00@0oooo00<0o`000?ooo`3oool03`3oool0
0`000000oooo0?ooo`0?0?ooo`030?l0003oool0oooo00@0oooo00<0o`000?ooo`3oool0?@3oool0
0`000000oooo0?ooo`2G0?ooo`030000003oool0oooo03d0oooo00<0o`000?ooo`3oool0103oool0
0`3o0000oooo0?ooo`0?0?ooo`030000003oool0oooo00l0oooo00<0o`000?ooo`3oool0103oool0
0`3o0000oooo0?ooo`0m0?ooo`030000003oool0oooo04P0oooo001=0?ooo`030000003oool0oooo
03d0oooo00<0o`000?ooo`3oool0103oool00`3o0000oooo0?ooo`0?0?ooo`030000003oool0oooo
00l0oooo00<0o`000?ooo`3oool01@3oool00`3o0000oooo0?ooo`0k0?ooo`800000V@3oool00`00
0000oooo0?ooo`0m0?ooo`030?l0003oool0oooo00@0oooo00<0o`000?ooo`3oool03`3oool00`00
0000oooo0?ooo`0?0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool0?03oool00`00
0000oooo0?ooo`180?ooo`00C@3oool00`000000oooo0?ooo`0l0?ooo`030?l0003oool0oooo00H0
oooo00<0o`000?ooo`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`030?l0003oool0oooo00H0
oooo00<0o`000?ooo`3oool0?03oool00`000000oooo0?ooo`2G0?ooo`030000003oool0oooo03`0
oooo00<0o`000?ooo`3oool01P3oool00`3o0000oooo0?ooo`0>0?ooo`030000003oool0oooo00h0
oooo00<0o`000?ooo`3oool01P3oool00`3o0000oooo0?ooo`0l0?ooo`030000003oool0oooo04P0
oooo000o0?ooo`030?oo_`1d000000000080000000<0M;oo0?ooo`3oool01P3oool00`000000oooo
0?ooo`0l0?ooo`030?l0003oool0oooo00H0oooo00<0o`000?ooo`3oool03P3oool00`000000oooo
0?ooo`0>0?ooo`030?l0003oool0oooo00H0oooo00<0o`000?ooo`3oool0?03oool00`000000oooo
0?ooo`290?ooo`030?oo_`1d000000000080000000<0M;oo0?ooo`3oool01P3oool00`000000oooo
0?ooo`0l0?ooo`030?l0003oool0oooo00H0oooo00<0o`000?ooo`3oool03P3oool00`000000oooo
0?ooo`0>0?ooo`030?l0003oool0oooo00H0oooo00<0o`000?ooo`3oool0?03oool00`000000oooo
0?ooo`180?ooo`00?`3oool00`3ogi`0B00004RLg`020?ooo`030;md0018W=l0oooo00H0oooo00<0
00000?ooo`3oool0?03oool00`3o0000oooo0?ooo`060?ooo`030?l0003oool0oooo00h0oooo00<0
00000?ooo`3oool03P3oool00`3o0000oooo0?ooo`070?ooo`030?l0003oool0oooo03X0oooo0P00
002;0?ooo`030?oOW0180000B9cO0080oooo00<0_g@004RLg`3oool01P3oool00`000000oooo0?oo
o`0l0?ooo`030?l0003oool0oooo00H0oooo00<0o`000?ooo`3oool03P3oool00`000000oooo0?oo
o`0>0?ooo`030?l0003oool0oooo00L0oooo00<0o`000?ooo`3oool0>`3oool00`000000oooo0?oo
o`180?ooo`00A03oool00`3ookl0M01809cOo`060?ooo`030000003oool0oooo03/0oooo00<0o`00
0?ooo`3oool01`3oool00`3o0000oooo0?ooo`0>0?ooo`030000003oool0oooo00d0oooo00<0o`00
0?ooo`3oool0203oool00`3o0000oooo0?ooo`0k0?ooo`030000003oool0oooo08h0oooo00<0oono
07@0B02Lgol01P3oool00`000000oooo0?ooo`0k0?ooo`030?l0003oool0oooo00L0oooo00<0o`00
0?ooo`3oool03P3oool00`000000oooo0?ooo`0=0?ooo`030?l0003oool0oooo00P0oooo00<0o`00
0?ooo`3oool0>`3oool00`000000oooo0?ooo`180?ooo`00A03oool00`3ookl0M00007Boo`060?oo
o`030000003oool0oooo03/0oooo00<0o`000?ooo`3oool0203oool00`3o0000oooo0?ooo`0=0?oo
o`030000003oool0oooo00d0oooo00<0o`000?ooo`3oool0203oool00`3o0000oooo0?ooo`0k0?oo
o`030000003oool0oooo08h0oooo00<0oono07@0001d_ol01P3oool00`000000oooo0?ooo`0k0?oo
o`030?l0003oool0oooo00P0oooo00<0o`000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?oo
o`030?l0003oool0oooo00P0oooo00<0o`000?ooo`3oool0>`3oool00`000000oooo0?ooo`180?oo
o`00A03oool00`3o_g@0000007Boo`060?ooo`030000003oool0oooo03/0oooo00<0o`000?ooo`3o
ool0203oool00`3o0000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<0o`000?ooo`3o
ool0203oool00`3o0000oooo0?ooo`0k0?ooo`030000003oool0oooo08h0oooo00<0okmd0000001d
_ol01P3oool00`000000oooo0?ooo`0k0?ooo`030?l0003oool0oooo00P0oooo00<0o`000?ooo`3o
ool03@3oool00`000000oooo0?ooo`0=0?ooo`030?l0003oool0oooo00P0oooo00<0o`000?ooo`3o
ool0>`3oool00`000000oooo0?ooo`180?ooo`00@`3oool0103ogi`0B0000000B02Lgol60?ooo`80
0000?03oool00`3o0000oooo0?ooo`080?ooo`030?l0003oool0oooo00d0oooo00<000000?ooo`3o
ool03@3oool00`3o0000oooo0?ooo`090?ooo`030?l0003oool0oooo03T0oooo0P00002?0?ooo`04
0?oOW0180000001809cOo`H0oooo0P00000l0?ooo`030?l0003oool0oooo00P0oooo00<0o`000?oo
o`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030?l0003oool0oooo00T0oooo00<0o`000?oo
o`3oool0>@3oool2000004X0oooo00110?ooo`050?noM000000000000000B02Lgol01`3oool00`00
0000oooo0?ooo`0j0?ooo`030?l0003oool0oooo00T0oooo00<0o`000?ooo`3oool03@3oool00`00
0000oooo0?ooo`0<0?ooo`030?l0003oool0oooo00X0oooo00<0o`000?ooo`3oool0>P3oool00`00
0000oooo0?ooo`2;0?ooo`050?noM000000000000000B02Lgol01`3oool00`000000oooo0?ooo`0j
0?ooo`030?l0003oool0oooo00T0oooo00<0o`000?ooo`3oool03@3oool00`000000oooo0?ooo`0<
0?ooo`030?l0003oool0oooo00X0oooo00<0o`000?ooo`3oool0>P3oool00`000000oooo0?ooo`18
0?ooo`00@`3oool00`2oM00007Bo0?ooo`070?ooo`030000003oool0oooo03X0oooo00<0o`000?oo
o`3oool02P3oool00`3o0000oooo0?ooo`0<0?ooo`030000003oool0oooo00`0oooo00<0o`000?oo
o`3oool02P3oool00`3o0000oooo0?ooo`0j0?ooo`030000003oool0oooo08d0oooo00<0_g@0001d
_`3oool01`3oool00`000000oooo0?ooo`0j0?ooo`030?l0003oool0oooo00X0oooo00<0o`000?oo
o`3oool0303oool00`000000oooo0?ooo`0<0?ooo`030?l0003oool0oooo00X0oooo00<0o`000?oo
o`3oool0>P3oool00`000000oooo0?ooo`180?ooo`00A03oool00`2oM000B9cO0?ooo`060?ooo`03
0000003oool0oooo03X0oooo00<0o`000?ooo`3oool02P3oool00`3o0000oooo0?ooo`0<0?ooo`03
0000003oool0oooo00`0oooo00<0o`000?ooo`3oool02P3oool00`3o0000oooo0?ooo`0j0?ooo`03
0000003oool0oooo08h0oooo00<0_g@004RLg`3oool01P3oool00`000000oooo0?ooo`0j0?ooo`03
0?l0003oool0oooo00X0oooo00<0o`000?ooo`3oool0303oool00`000000oooo0?ooo`0<0?ooo`03
0?l0003oool0oooo00X0oooo00<0o`000?ooo`3oool0>P3oool00`000000oooo0?ooo`180?ooo`00
?`3oool00`3ooml0W9bo0?ooo`020?ooo`030=nLB00007@0_ooo00H0oooo00<000000?ooo`3oool0
>P3oool00`3o0000oooo0?ooo`0:0?ooo`030?l0003oool0oooo00`0oooo00<000000?ooo`3oool0
303oool00`3o0000oooo0?ooo`0;0?ooo`030?l0003oool0oooo03P0oooo0P00002;0?ooo`030?oo
g`2LW;l0oooo0080oooo00<0gia80000M02oool01P3oool00`000000oooo0?ooo`0j0?ooo`030?l0
003oool0oooo00X0oooo00<0o`000?ooo`3oool0303oool00`000000oooo0?ooo`0<0?ooo`030?l0
003oool0oooo00/0oooo00<0o`000?ooo`3oool0>@3oool00`000000oooo0?ooo`180?ooo`00@03o
ool01P3o_g@0B9cO0?ooo`3ookl0M000001d_`L0oooo00<000000?ooo`3oool0>@3oool00`3o0000
oooo0?ooo`0;0?ooo`030?l0003oool0oooo00`0oooo00<000000?ooo`3oool02`3oool00`3o0000
oooo0?ooo`0<0?ooo`030?l0003oool0oooo03T0oooo00<000000?ooo`3oool0RP3oool01P3o_g@0
B9cO0?ooo`3ookl0M000001d_`L0oooo00<000000?ooo`3oool0>@3oool00`3o0000oooo0?ooo`0;
0?ooo`030?l0003oool0oooo00`0oooo00<000000?ooo`3oool02`3oool00`3o0000oooo0?ooo`0<
0?ooo`030?l0003oool0oooo03T0oooo00<000000?ooo`3oool0B03oool00440oooo00D0gia80000
00000000001d0;ooo`070?ooo`030000003oool0oooo03T0oooo00<0o`000?ooo`3oool0303oool0
0`3o0000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<0o`000?ooo`3oool0303oool0
0`3o0000oooo0?ooo`0i0?ooo`030000003oool0oooo08/0oooo00D0gia8000000000000001d0;oo
o`070?ooo`030000003oool0oooo03T0oooo00<0o`000?ooo`3oool0303oool00`3o0000oooo0?oo
o`0;0?ooo`030000003oool0oooo00/0oooo00<0o`000?ooo`3oool0303oool00`3o0000oooo0?oo
o`0i0?ooo`030000003oool0oooo04P0oooo001=0?ooo`030000003oool0oooo03T0oooo00<0o`00
0?ooo`3oool0303oool00`3o0000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<0o`00
0?ooo`3oool0303oool00`3o0000oooo0?ooo`0i0?ooo`030000003oool0oooo09L0oooo00<00000
0?ooo`3oool0>@3oool00`3o0000oooo0?ooo`0<0?ooo`030?l0003oool0oooo00/0oooo00<00000
0?ooo`3oool02`3oool00`3o0000oooo0?ooo`0<0?ooo`030?l0003oool0oooo03T0oooo00<00000
0?ooo`3oool0B03oool004d0oooo00<000000?ooo`3oool0>@3oool00`3o0000oooo0?ooo`0<0?oo
o`030?l0003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`3o0000oooo0?ooo`0<0?oo
o`030?l0003oool0oooo03P0oooo0P00002I0?ooo`030000003oool0oooo03T0oooo00<0o`000?oo
o`3oool0303oool00`3o0000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<0o`000?oo
o`3oool0303oool00`3o0000oooo0?ooo`0i0?ooo`030000003oool0oooo04P0oooo001=0?ooo`03
0000003oool0oooo03P0oooo00<0o`000?ooo`3oool03@3oool00`3o0000oooo0?ooo`0;0?ooo`03
0000003oool0oooo00/0oooo00<0o`000?ooo`3oool0303oool00`3o0000oooo0?ooo`0i0?ooo`03
0000003oool0oooo09L0oooo00<000000?ooo`3oool0>03oool00`3o0000oooo0?ooo`0=0?ooo`03
0?l0003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`3o0000oooo0?ooo`0<0?ooo`03
0?l0003oool0oooo03T0oooo00<000000?ooo`3oool0B03oool004d0oooo00<000000?ooo`3oool0
>03oool00`3o0000oooo0?ooo`0>0?ooo`030?l0003oool0oooo00X0oooo00<000000?ooo`3oool0
2`3oool00`3o0000oooo0?ooo`0=0?ooo`030?l0003oool0oooo03P0oooo00<000000?ooo`3oool0
U`3oool00`000000oooo0?ooo`0h0?ooo`030?l0003oool0oooo00h0oooo00<0o`000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0;0?ooo`030?l0003oool0oooo00d0oooo00<0o`000?ooo`3oool0
>03oool00`000000oooo0?ooo`180?ooo`00C@3oool00`000000oooo0?ooo`0h0?ooo`030?l0003o
ool0oooo00h0oooo00<0o`000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030?l0003o
ool0oooo00h0oooo00<0o`000?ooo`3oool0=`3oool2000009T0oooo00<000000?ooo`3oool0>03o
ool00`3o0000oooo0?ooo`0>0?ooo`030?l0003oool0oooo00X0oooo00<000000?ooo`3oool02P3o
ool00`3o0000oooo0?ooo`0>0?ooo`030?l0003oool0oooo03P0oooo00<000000?ooo`3oool0B03o
ool004d0oooo00<000000?ooo`3oool0>03oool00`3o0000oooo0?ooo`0>0?ooo`030?l0003oool0
oooo00X0oooo00<000000?ooo`3oool02P3oool00`3o0000oooo0?ooo`0>0?ooo`030?l0003oool0
oooo03P0oooo00<000000?ooo`3oool0U`3oool00`000000oooo0?ooo`0h0?ooo`030?l0003oool0
oooo00h0oooo00<0o`000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030?l0003oool0
oooo00h0oooo00<0o`000?ooo`3oool0>03oool00`000000oooo0?ooo`180?ooo`00C@3oool00`00
0000oooo0?ooo`0h0?ooo`030?l0003oool0oooo00h0oooo00<0o`000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0:0?ooo`030?l0003oool0oooo00h0oooo00<0o`000?ooo`3oool0>03oool00`00
0000oooo0?ooo`2G0?ooo`030000003oool0oooo03P0oooo00<0o`000?ooo`3oool03P3oool00`3o
0000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<0o`000?ooo`3oool03P3oool00`3o
0000oooo0?ooo`0h0?ooo`030000003oool0oooo04P0oooo000a0?ooo`030?oo_`1d000000000080
000000<0M;oo0?ooo`3oool00P3oool01P3o_g@0000007Boo`3oool0oooO09a800<0000000<0M;oo
0?ooo`3oool01P3oool00`000000oooo0?ooo`0h0?ooo`030?l0003oool0oooo00h0oooo00<0o`00
0?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030?l0003oool0oooo00h0oooo00<0o`00
0?ooo`3oool0>03oool00`000000oooo0?ooo`1k0?ooo`030?oo_`1d000000000080000000<0M;oo
0?ooo`3oool00P3oool01P3o_g@0000007Boo`3oool0oooO09a800<0000000<0M;oo0?ooo`3oool0
1P3oool00`000000oooo0?ooo`0h0?ooo`030?l0003oool0oooo00h0oooo00<0o`000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0:0?ooo`030?l0003oool0oooo00h0oooo00<0o`000?ooo`3oool0
>03oool00`000000oooo0?ooo`180?ooo`00<@3oool00`3ogi`0B00004RLg`020?ooo`030;md0018
W=l0oooo0080oooo00L0okmd0000001d_ol0oooo0?oog`2LB7@0_ooo0080oooo00<0_g@007Boo`3o
ool01P3oool00`000000oooo0?ooo`0g0?ooo`030?l0003oool0oooo00l0oooo00<0o`000?ooo`3o
ool02P3oool00`000000oooo0?ooo`0:0?ooo`030?l0003oool0oooo00h0oooo00<0o`000?ooo`3o
ool0=`3oool2000007d0oooo00<0omnL04P00018W=l00P3oool00`2oM000B9cO0?ooo`020?ooo`07
0?noM0000000M;oo0?ooo`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0oooo00H0oooo00<00000
0?ooo`3oool0=`3oool00`3o0000oooo0?ooo`0?0?ooo`030?l0003oool0oooo00X0oooo00<00000
0?ooo`3oool02P3oool00`3o0000oooo0?ooo`0>0?ooo`030?l0003oool0oooo03P0oooo00<00000
0?ooo`3oool0B03oool003H0oooo00<0oono07@0B02Lgol02`3oool00`3ooml0W4Qd0;ooo`060?oo
o`030000003oool0oooo03L0oooo00<0o`000?ooo`3oool0403oool00`3o0000oooo0?ooo`090?oo
o`030000003oool0oooo00X0oooo00<0o`000?ooo`3oool03P3oool00`3o0000oooo0?ooo`0h0?oo
o`030000003oool0oooo0800oooo00<0oono07@0B02Lgol02`3oool00`3ooml0W4Qd0;ooo`060?oo
o`030000003oool0oooo03L0oooo00<0o`000?ooo`3oool0403oool00`3o0000oooo0?ooo`090?oo
o`030000003oool0oooo00X0oooo00<0o`000?ooo`3oool03P3oool00`3o0000oooo0?ooo`0h0?oo
o`030000003oool0oooo04P0oooo000f0?ooo`030?oo_`1d0000M;oo00`0oooo00<0_g@007Boo`3o
ool01@3oool00`000000oooo0?ooo`0g0?ooo`030?l0003oool0oooo0100oooo00<0o`000?ooo`3o
ool02@3oool00`000000oooo0?ooo`0:0?ooo`030?l0003oool0oooo00h0oooo00<0o`000?ooo`3o
ool0>03oool00`000000oooo0?ooo`200?ooo`030?oo_`1d0000M;oo00`0oooo00<0_g@007Boo`3o
ool01@3oool00`000000oooo0?ooo`0g0?ooo`030?l0003oool0oooo0100oooo00<0o`000?ooo`3o
ool02@3oool00`000000oooo0?ooo`0:0?ooo`030?l0003oool0oooo00h0oooo00<0o`000?ooo`3o
ool0>03oool00`000000oooo0?ooo`180?ooo`00=P3oool00`3o_g@0000007Boo`0;0?ooo`030?oo
g`2LB000B9cO00H0oooo00<000000?ooo`3oool0=`3oool00`3o0000oooo0?ooo`0@0?ooo`030?l0
003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`3o0000oooo0?ooo`0@0?ooo`030?l0
003oool0oooo03L0oooo00<000000?ooo`3oool0P03oool00`3o_g@0000007Boo`0;0?ooo`030?oo
g`2LB000B9cO00H0oooo00<000000?ooo`3oool0=`3oool00`3o0000oooo0?ooo`0@0?ooo`030?l0
003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`3o0000oooo0?ooo`0@0?ooo`030?l0
003oool0oooo03L0oooo00<000000?ooo`3oool0B03oool003D0oooo00@0omnL04P0000004P0W=oo
2`3oool00`3OW4P0000007Boo`060?ooo`800000>03oool00`3o0000oooo0?ooo`0@0?ooo`030?l0
003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`3o0000oooo0?ooo`0@0?ooo`030?l0
003oool0oooo03H0oooo0P0000210?ooo`040?oOW0180000001809cOo`/0oooo00<0gia80000001d
_ol01P3oool2000003P0oooo00<0o`000?ooo`3oool0403oool00`3o0000oooo0?ooo`090?ooo`03
0000003oool0oooo00T0oooo00<0o`000?ooo`3oool0403oool00`3o0000oooo0?ooo`0f0?ooo`80
0000BP3oool003<0oooo00D0okmd000000000000001809cOo`080?ooo`070=oOg`3oool0oono07@0
00000000001d0;ooo`060?ooo`030000003oool0oooo03L0oooo00<0o`000?ooo`3oool0403oool0
0`3o0000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<0o`000?ooo`3oool0403oool0
0`3o0000oooo0?ooo`0g0?ooo`030000003oool0oooo07d0oooo00D0okmd000000000000001809cO
o`080?ooo`070=oOg`3oool0oono07@000000000001d0;ooo`060?ooo`030000003oool0oooo03L0
oooo00<0o`000?ooo`3oool0403oool00`3o0000oooo0?ooo`090?ooo`030000003oool0oooo00T0
oooo00<0o`000?ooo`3oool0403oool00`3o0000oooo0?ooo`0g0?ooo`030000003oool0oooo04P0
oooo000J0?ooo`030=nLM02Lgol0oooo01P0oooo00<0_g@0001d_`3oool0203oool01P3OW4P00000
000000000000001809cOo`L0oooo00<000000?ooo`3oool0=P3oool00`3o0000oooo0?ooo`0A0?oo
o`030?l0003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`3o0000oooo0?ooo`0@0?oo
o`030?l0003oool0oooo03L0oooo00<000000?ooo`3oool0I03oool00`3OW7@0W=oo0?ooo`0H0?oo
o`030;md0000M;l0oooo00P0oooo00H0gia800000000000000000000B02Lgol70?ooo`030000003o
ool0oooo03H0oooo00<0o`000?ooo`3oool04@3oool00`3o0000oooo0?ooo`090?ooo`030000003o
ool0oooo00T0oooo00<0o`000?ooo`3oool0403oool00`3o0000oooo0?ooo`0g0?ooo`030000003o
ool0oooo04P0oooo000J0?ooo`030=nLB000M;l0oooo01T0oooo00<0_g@004RLg`3oool01`3oool0
103ookl0M00000000018W=l90?ooo`030000003oool0oooo03H0oooo00<0o`000?ooo`3oool04@3o
ool00`3o0000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<0o`000?ooo`3oool0403o
ool00`3o0000oooo0?ooo`0f0?ooo`800000IP3oool00`3OW4P007Bo0?ooo`0I0?ooo`030;md0018
W=l0oooo00L0oooo00@0oono07@000000000B9cO2@3oool00`000000oooo0?ooo`0f0?ooo`030?l0
003oool0oooo0140oooo00<0o`000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030?l0
003oool0oooo0100oooo00<0o`000?ooo`3oool0=`3oool00`000000oooo0?ooo`180?ooo`006P3o
ool01@3OW4P00000000000000000M;oo0180oooo00<0oooO09bL_`3oool00P3oool00`3OW4P0001d
0;ooo`080?ooo`030;md001d_ol0oooo00T0oooo00<000000?ooo`3oool0=P3oool00`3o0000oooo
0?ooo`0A0?ooo`030?l0003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`3o0000oooo
0?ooo`0@0?ooo`030?l0003oool0oooo03L0oooo00<000000?ooo`3oool0I03oool01@3OW4P00000
000000000000M;oo0180oooo00<0oooO09bL_`3oool00P3oool00`3OW4P0001d0;ooo`080?ooo`03
0;md001d_ol0oooo00T0oooo00<000000?ooo`3oool0=P3oool00`3o0000oooo0?ooo`0A0?ooo`03
0?l0003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`3o0000oooo0?ooo`0@0?ooo`03
0?l0003oool0oooo03L0oooo00<000000?ooo`3oool0B03oool001X0oooo00<0gia8000000000000
0`0000000`0007@0_ooo0?ooo`0?0?ooo`060?noM018W=l0oooo0?oo_`1d000007Bo2@3oool01P3o
gi`0B000000000000000001809cOo`H0oooo00<000000?ooo`3oool0=P3oool00`3o0000oooo0?oo
o`0B0?ooo`030?l0003oool0oooo00P0oooo00<000000?ooo`3oool02@3oool00`3o0000oooo0?oo
o`0@0?ooo`030?l0003oool0oooo03L0oooo00<000000?ooo`3oool0I03oool00`3OW4P000000000
0003000000030000M02oool0oooo00l0oooo00H0okmd04RLg`3oool0oono07@00000M;l90?ooo`06
0?oOW018000000000000000004P0W=oo1P3oool00`000000oooo0?ooo`0f0?ooo`030?l0003oool0
oooo0180oooo00<0o`000?ooo`3oool0203oool00`000000oooo0?ooo`090?ooo`030?l0003oool0
oooo0100oooo00<0o`000?ooo`3oool0=`3oool00`000000oooo0?ooo`180?ooo`006P3oool01P3O
W7@0W=oo0?ooo`3oool0omnL04P000<0000000<004RL0=ooo`3oool03@3oool01@3OW4P000000000
000007@0_ooo00X0oooo00D0_g@0000000000000000004RLg`060?ooo`030000003oool0oooo03H0
oooo00<0o`000?ooo`3oool04P3oool00`3o0000oooo0?ooo`080?ooo`030000003oool0oooo00P0
oooo00<0o`000?ooo`3oool04P3oool00`3o0000oooo0?ooo`0f0?ooo`030000003oool0oooo06@0
oooo00H0giad09cOo`3oool0oooo0?oOW0180003000000030018W03Oool0oooo00d0oooo00D0gia8
000000000000001d0;ooo`0:0?ooo`050;md00000000000000000018W=l01P3oool00`000000oooo
0?ooo`0f0?ooo`030?l0003oool0oooo0180oooo00<0o`000?ooo`3oool0203oool00`000000oooo
0?ooo`080?ooo`030?l0003oool0oooo0180oooo00<0o`000?ooo`3oool0=P3oool00`000000oooo
0?ooo`180?ooo`008@3oool00`3o_g@000000000000200000003001d_`3oool0oooo02@0oooo00<0
00000?ooo`3oool0=P3oool00`3o0000oooo0?ooo`0B0?ooo`030?l0003oool0oooo00P0oooo00<0
00000?ooo`3oool0203oool00`3o0000oooo0?ooo`0B0?ooo`030?l0003oool0oooo03D0oooo0P00
001]0?ooo`030?noM000000000000080000000<007Bo0?ooo`3oool0903oool00`000000oooo0?oo
o`0f0?ooo`030?l0003oool0oooo0180oooo00<0o`000?ooo`3oool0203oool00`000000oooo0?oo
o`080?ooo`030?l0003oool0oooo0180oooo00<0o`000?ooo`3oool0=P3oool00`000000oooo0?oo
o`180?ooo`00903oool0103o_g@000000000M02ooolU0?ooo`030000003oool0oooo03H0oooo00<0
o`000?ooo`3oool04P3oool00`3o0000oooo0?ooo`080?ooo`030000003oool0oooo00P0oooo00<0
o`000?ooo`3oool04P3oool00`3o0000oooo0?ooo`0f0?ooo`030000003oool0oooo06h0oooo00@0
okmd0000000007@0_ooo9@3oool00`000000oooo0?ooo`0f0?ooo`030?l0003oool0oooo0180oooo
00<0o`000?ooo`3oool0203oool00`000000oooo0?ooo`080?ooo`030?l0003oool0oooo0180oooo
00<0o`000?ooo`3oool0=P3oool00`000000oooo0?ooo`180?ooo`008@3oool0103o_g@000000000
0000M;lX0?ooo`030000003oool0oooo03H0oooo00<0o`000?ooo`3oool04P3oool00`3o0000oooo
0?ooo`080?ooo`030000003oool0oooo00P0oooo00<0o`000?ooo`3oool04P3oool00`3o0000oooo
0?ooo`0f0?ooo`030000003oool0oooo06/0oooo00@0okmd00000000000007Bo:03oool00`000000
oooo0?ooo`0f0?ooo`030?l0003oool0oooo0180oooo00<0o`000?ooo`3oool0203oool00`000000
oooo0?ooo`080?ooo`030?l0003oool0oooo0180oooo00<0o`000?ooo`3oool0=P3oool00`000000
oooo0?ooo`180?ooo`006P3oool02@3OW7@0W=oo0?ooo`3oool0omnL04P00000000004RL0=ooo`0Z
0?ooo`030000003oool0oooo03D0oooo00<0o`000?ooo`3oool04`3oool00`3o0000oooo0?ooo`08
0?ooo`030000003oool0oooo00P0oooo00<0o`000?ooo`3oool04P3oool00`3o0000oooo0?ooo`0f
0?ooo`030000003oool0oooo06@0oooo00T0giad09cOo`3oool0oooo0?oOW018000000000018W03O
ool0:P3oool00`000000oooo0?ooo`0e0?ooo`030?l0003oool0oooo01<0oooo00<0o`000?ooo`3o
ool0203oool00`000000oooo0?ooo`080?ooo`030?l0003oool0oooo0180oooo00<0o`000?ooo`3o
ool0=P3oool00`000000oooo0?ooo`180?ooo`006P3oool01P3OW4P00000000000000000001d0;oo
obd0oooo00<000000?ooo`3oool0=@3oool00`3o0000oooo0?ooo`0C0?ooo`030?l0003oool0oooo
00P0oooo00<000000?ooo`3oool0203oool00`3o0000oooo0?ooo`0B0?ooo`030?l0003oool0oooo
03D0oooo0P00001V0?ooo`060=nLB000000000000000000007@0_ooo;@3oool00`000000oooo0?oo
o`0e0?ooo`030?l0003oool0oooo01<0oooo00<0o`000?ooo`3oool0203oool00`000000oooo0?oo
o`080?ooo`030?l0003oool0oooo00H0oooo1P0000060?ooo`030?l0003oool0oooo03H0oooo00<0
00000?ooo`3oool0B03oool001X0oooo00<0gia80000001d_ol0<03oool00`000000oooo0?ooo`0e
0?ooo`030?l0003oool0oooo01<0oooo00<0o`000?ooo`3oool0203oool00`000000oooo0?ooo`07
0?ooo`030?l0003oool0oooo01<0oooo00<0o`000?ooo`3oool0=P3oool00`000000oooo0?ooo`1T
0?ooo`030=nLB0000000M;oo0300oooo00<000000?ooo`3oool0=@3oool00`3o0000oooo0?ooo`0C
0?ooo`030?l0003oool0oooo00P0oooo00<000000?ooo`3oool01`3oool00`3o0000oooo0?ooo`05
0?ooo`8000001P3oool00`000000oooo0?ooo`030?ooo`030?l0003oool0oooo03H0oooo00<00000
0?ooo`3oool0B03oool001X0oooo00<0giad09cOo`3oool09P3oool0103ooml0W4P00018W03Oool6
0?ooo`030000003oool0oooo03D0oooo00<0o`000?ooo`3oool0503oool00`3o0000oooo0?ooo`07
0?ooo`030000003oool0oooo00L0oooo00<0o`000?ooo`3oool0503oool00`3o0000oooo0?ooo`0e
0?ooo`030000003oool0oooo06@0oooo00<0giad09cOo`3oool09P3oool0103ooml0W4P00018W03O
ool60?ooo`030000003oool0oooo03D0oooo00<0o`000?ooo`3oool0503oool00`3o0000oooo0?oo
o`070?ooo`030000003oool0oooo00L0oooo00<0o`000?ooo`3oool0103oool00`000000oooo0?oo
o`070?ooo`800000103oool00`3o0000oooo0?ooo`0e0?ooo`030000003oool0oooo04P0oooo0013
0?ooo`040?oog`2LB00004RL0=ooo`H0oooo00@000000?ooo`3oool0oooo6P0000060?ooo`D00000
1P3oool4000000D0oooo00<0o`000000000000000`0000060?ooo`D000001P3oool4000000H0oooo
1@0000050?ooo`030?l000000000000000<000001P3oool5000000H0oooo100000060?ooo`D00000
1P3ooolP00000080oooo0P00002?0?ooo`040?oog`2LB00004RL0=ooo`H0oooo00<000000?ooo`3o
ool0=@3oool00`3o0000oooo0?ooo`0D0?ooo`030?l0003oool0oooo00L0oooo00<000000?ooo`3o
ool01`3oool00`3o0000oooo0?ooo`030?ooo`030000003oool0oooo00X0oooo0P0000020?ooo`03
0?l0003oool0oooo03D0oooo00<000000?ooo`3oool0B03oool004<0oooo00@0oooO09a80000B9`0
gooo1P3oool00`000000oooo0?ooo`0K0?ooo`P000004P3oool00`3o0000oooo0?ooo`0D0?ooo`03
0?l0003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`3o0000oooo0?ooo`0D0?ooo`03
0?l0003oool0oooo0140oooo1@00000O0?ooo`030000003oool0oooo08d0oooo00@0oooO09a80000
B9`0gooo1P3oool00`000000oooo0?ooo`0e0?ooo`030?l0003oool0oooo01@0oooo00<0o`000?oo
o`3oool01`3oool00`000000oooo0?ooo`070?ooo`030?l0003oool0oooo00<0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?l0000g0?ooo`030000003oool0oooo04P0oooo000T0?ooo`06
0?oog`2LB00004RL0=ooo`3ooml0W9bo5@3oool00`2oM0000000000000050000000307Boo`3oool0
oooo00<0oooo00<000000?ooo`3oool08`3oool3000000l0oooo00<0o`000?ooo`3oool0503oool0
0`3o0000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<0o`000?ooo`3oool0503oool0
0`3o0000oooo0?ooo`0=0?ooo`@00000903oool00`000000oooo0?ooo`1^0?ooo`060?oog`2LB000
04RL0=ooo`3ooml0W9bo5@3oool00`2oM0000000000000050000000307Boo`3oool0oooo00<0oooo
00<000000?ooo`3oool0=@3oool00`3o0000oooo0?ooo`0D0?ooo`030?l0003oool0oooo00L0oooo
00<000000?ooo`3oool01`3oool0103o0000oooo0?ooo`3oool200000100oooo00<000000?l0003o
ool0=P3oool00`000000oooo0?ooo`180?ooo`00903oool01P3ookl0M0000018W03Oool0_gA809cO
oaD0oooo00P0omnL07BLg`3oool0oooo0?oog`2LB00004RL0=ooo`H0oooo00<000000?ooo`3oool0
9P3oool00`000000oooo0?ooo`0<0?ooo`030?l0003oool0oooo01@0oooo00<0o`000?ooo`3oool0
1`3oool00`000000oooo0?ooo`060?ooo`030?l0003oool0oooo01D0oooo00<0o`000?ooo`3oool0
2`3oool2000002P0oooo00<000000?ooo`3oool0KP3oool01P3ookl0M0000018W03Oool0_gA809cO
oaD0oooo00P0omnL07BLg`3oool0oooo0?oog`2LB00004RL0=ooo`H0oooo00<000000?ooo`3oool0
=@3oool00`3o0000oooo0?ooo`0D0?ooo`030?l0003oool0oooo00L0oooo00<000000?ooo`3oool0
1P3oool01@3o0000oooo0?ooo`3oool0000001<0oooo00<000000?ooo`3oool0=@3oool00`000000
oooo0?ooo`180?ooo`009@3oool0103OW4P000000000001d_olG0?ooo`070=nLB01d_ol0oooo0?oo
g`2LB00004RL0=ooo`060?ooo`800000:03oool3000000X0oooo00<0o`000?ooo`3oool05@3oool0
0`3o0000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<0o`000?ooo`3oool05@3oool0
0`3o0000oooo0?ooo`090?ooo`800000:@3oool200000740oooo00@0gia8000000000000M;oo5`3o
ool01`3OW4P0M;oo0?ooo`3ooml0W4P00018W03Oool01P3oool2000003D0oooo00<0o`000?ooo`3o
ool05@3oool00`3o0000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00@0o`000?ooo`3o
ool00000503oool00`3o000000000?ooo`0d0?ooo`800000BP3oool00400oooo00L0oooO09a8M02o
ool0oooO09a80000B9`0gooo00H0oooo00<000000?ooo`3oool0:P3oool2000000P0oooo00<0o`00
0?ooo`3oool05P3oool00`3o0000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo00<0o`00
0?ooo`3oool05@3oool00`3o0000oooo0?ooo`060?ooo`<00000;03oool00`000000oooo0?ooo`2:
0?ooo`070?oog`2LB7@0_ooo0?oog`2LB00004RL0=ooo`060?ooo`030000003oool0oooo03@0oooo
00<0o`000?ooo`3oool05P3oool00`3o0000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo
00@0o`000?ooo`3oool00000503oool00`3o0000oooo0000000e0?ooo`030000003oool0oooo04P0
oooo00110?ooo`060?oOW018M;l0oooO09a80000B9`0gooo1P3oool00`000000oooo0?ooo`0/0?oo
o`030000003oool0oooo00D0oooo00<0o`000?ooo`3oool05P3oool00`3o0000oooo0?ooo`060?oo
o`030000003oool0oooo00H0oooo00<0o`000?ooo`3oool05P3oool00`3o0000oooo0?ooo`040?oo
o`030000003oool0oooo02d0oooo00<000000?ooo`3oool0R`3oool01P3ogi`0B7Bo0?oog`2LB000
04RL0=ooo`H0oooo00<000000?ooo`3oool0=03oool00`3o0000oooo0?ooo`0F0?ooo`030?l0003o
ool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`3o0000oooo0000000F0?ooo`030?l0003o
ool0000003@0oooo00<000000?ooo`3oool0B03oool00480oooo00D0gia807Bog`2LB00004RL0=oo
o`060?ooo`030000003oool0oooo02d0oooo0P0000050?ooo`030?l0003oool0oooo01H0oooo00<0
o`000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030?l0003oool0oooo01H0oooo00<0
o`000?ooo`3oool00`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo08`0oooo00D0
gia807Bog`2LB00004RL0=ooo`060?ooo`030000003oool0oooo03@0oooo00<0o`000?ooo`3oool0
5P3oool00`3o0000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo00<0o`000000003oool0
5P3oool0103o0000oooo0?ooo`00000c0?ooo`030000003oool0oooo04P0oooo00120?ooo`050?oo
g`2LB4P0B4P00018W03Oool01P3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo0080
oooo00<0o`000?ooo`3oool05P3oool00`3o0000oooo0?ooo`060?ooo`030000003oool0oooo00H0
oooo00<0o`000?ooo`3oool05P3oool00`3o0000oooo0?ooo`020?ooo`030000003oool0oooo02h0
oooo0P00002>0?ooo`050?oog`2LB4P0B4P00018W03Oool01P3oool00`000000oooo0?ooo`0d0?oo
o`030?l0003oool0oooo01H0oooo00<0o`000?ooo`3oool01P3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo01H0oooo00@0o`000?ooo`3oool0oooo0P00000a0?ooo`030000003oool0
oooo04P0oooo00130?ooo`040?oOW018000004RL0=ooo`H0oooo00<000000?ooo`3oool0<03oool0
1@000000oooo0?ooo`3oool0o`0001P0oooo00<0o`000?ooo`3oool01P3oool00`000000oooo0?oo
o`050?ooo`030?l0003oool0oooo01L0oooo00D0o`000?ooo`3oool0oooo0000000b0?ooo`030000
003oool0oooo08d0oooo00@0omnL04P00000B9`0gooo1P3oool00`000000oooo0?ooo`0d0?ooo`03
0?l0003oool0oooo01H0oooo00<0o`000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`03
0?l000000000oooo01L0oooo00<0o`000?ooo`3oool00`3oool00`000000oooo0?ooo`0^0?ooo`03
0000003oool0oooo04P0oooo00140?ooo`030=nLB000B9`0gooo00H0oooo00<000000?ooo`3oool0
<@3oool2000000030?ooo`3o0000oooo01L0oooo00<0o`000?ooo`3oool01P3oool00`000000oooo
0?ooo`050?ooo`030?l0003oool0oooo01L0oooo00@0o`000?ooo`0000000000<`3oool00`000000
oooo0?ooo`2>0?ooo`030=nLB000B9`0gooo00H0oooo00<000000?ooo`3oool0=03oool00`3o0000
oooo0?ooo`0F0?ooo`030?l0003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`0G0?ooo`030?l0003oool0oooo00@0oooo00<000000?ooo`3oool0;@3oool00`000000
oooo0?ooo`180?ooo`007P3oool01@3ogi`0M9cO0?ooo`3ookl0M0000080000000<0001d0;ooo`3o
ool09@3oool00`000000oooo0?ooo`0c0?ooo`030000003o0000oooo01P0oooo00<0o`000?ooo`3o
ool01@3oool00`000000oooo0?ooo`050?ooo`030?l0003oool0oooo01L0oooo00<0o`000000003o
ool0<`3oool2000006X0oooo00D0omnL07BLg`3oool0oono07@00002000000030000M02oool0oooo
02D0oooo00<000000?ooo`3oool0=03oool00`3o0000oooo0?ooo`0G0?ooo`030?l0003oool0oooo
00D0oooo00<000000?ooo`3oool0103oool00`000000o`000?ooo`0H0?ooo`030?l0003oool0oooo
00D0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`180?ooo`007P3oool00`3ooml0
W4P0000000030000000407Bo_`1d000004RL0=ooobD0oooo00<000000?ooo`3oool0<`3oool00`3o
000000000?ooo`0H0?ooo`030?l0003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`3o
0000oooo0?ooo`0G0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0J03oool00`3o
oml0W4P0000000030000000407Bo_`1d000004RL0=ooobD0oooo00<000000?ooo`3oool0<`3oool0
0`3o0000oooo0?ooo`0H0?ooo`030?l0003oool0oooo00D0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?l0000I0?ooo`030?l0003oool0oooo00H0oooo00<000000?ooo`3oool0:`3oool0
0`000000oooo0?ooo`180?ooo`009P3oool01P2oM4P0W=no07@00000000004RL0=ooob40oooo00<0
00000?ooo`3oool0<`3oool0103o0000oooo00000000000G0?ooo`030?l0003oool0oooo00D0oooo
00<000000?ooo`3oool01@3oool00`3o0000oooo0?ooo`0F0?ooo`030000003o0000oooo03D0oooo
00<000000?ooo`3oool0L03oool01P2oM4P0W=no07@00000000004RL0=ooob40oooo00<000000?oo
o`3oool0<`3oool00`3o0000oooo0?ooo`0H0?ooo`030?l0003oool0oooo00D0oooo00<000000?oo
o`3oool00P3oool010000000oooo0?ooo`3o000I0?ooo`030?l0003oool0oooo00L0oooo00<00000
0?ooo`3oool0:P3oool00`000000oooo0?ooo`180?ooo`00903oool01@3OW4P000000000000007@0
_ooo02@0oooo00<000000?ooo`3oool0<`3oool01@3o0000oooo0?ooo`3oool0000001H0oooo00<0
o`000?ooo`3oool01@3oool00`000000oooo0?ooo`040?ooo`030?l0003oool0oooo01D0oooo0P00
00020?ooo`030?l0003oool0oooo03<0oooo00<000000?ooo`3oool0KP3oool01@3OW4P000000000
000007@0_ooo02@0oooo00<000000?ooo`3oool0<`3oool00`3o0000oooo0?ooo`0H0?ooo`030?l0
003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?l0000K0?ooo`030?l0
003oool0oooo00L0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`180?ooo`00803o
ool0203o_g@00000000000000000B9cO0?ooo`2oM4P0W=oo9@3oool00`000000oooo0?ooo`0c0?oo
o`030?l0003oool0oooo0080oooo00<000000?ooo`3oool04`3oool00`3o0000oooo0?ooo`050?oo
o`030000003oool0oooo00@0oooo00<0o`000?ooo`3oool0503oool00`000000oooo0?ooo`020?oo
o`030?l0003oool0oooo0380oooo0P00001/0?ooo`080?noM0000000000000000018W=l0oooo0;md
B02LgolU0?ooo`030000003oool0oooo03<0oooo00<0o`000?ooo`3oool0603oool00`3o0000oooo
0?ooo`050?ooo`050000003oool0oooo0?ooo`0000000P3oool00`3o0000oooo0?ooo`0I0?ooo`03
0?l0003oool0oooo00P0oooo0P00000Y0?ooo`030000003oool0oooo04P0oooo000L0?ooo`060;md
0000000000000000000007@0_ooo0`3oool00`3ookl0M4RL0=ooo`0U0?ooo`030000003oool0oooo
03<0oooo00<0o`000?ooo`3oool00`3oool00`000000oooo0?ooo`0C0?ooo`030?l0003oool0oooo
00X0000000<0oooo0?l0003oool0503oool00`000000oooo0?ooo`030?ooo`030?l0003oool0oooo
03<0oooo00<000000?ooo`3oool0IP3oool01P2oM0000000000000000000001d0;ooo`<0oooo00<0
oono07A8W03Oool09@3oool00`000000oooo0?ooo`0c0?ooo`030?l0003oool0oooo01T0oooo00<0
o`000?ooo`3oool0103oool010000000oooo0?ooo`0000030?ooo`030?l0003oool0oooo01T0oooo
00<0o`000?ooo`3oool02P3oool2000002L0oooo00<000000?ooo`3oool0B03oool001X0oooo00@0
oooO09a80000000007Bo1P3oool00`3ooml0W4P004RLg`0>0?ooo`040?oog`2LB00004RL0=ooo`80
oooo00H0okmd0000001d_ol0oooo0?oog`2LB0030000000307Boo`3oool0oooo00H0oooo00<00000
0?ooo`3oool0<`3oool00`3o0000oooo0?ooo`040?ooo`030000003oool0oooo0140oooo10000004
0?ooo`030000003oool0oooo00<0oooo0P00000D0?ooo`030000003oool0oooo00@0oooo00<0o`00
0?ooo`3oool0<`3oool00`000000oooo0?ooo`1T0?ooo`040?oog`2LB0000000001d_`H0oooo00<0
oooO09a80018W=l03P3oool0103ooml0W4P00018W03Oool20?ooo`060?noM0000000M;oo0?ooo`3o
oml0W4P00`0000000`1d_ol0oooo0?ooo`060?ooo`030000003oool0oooo03<0oooo00<0o`000?oo
o`3oool06@3oool00`3o0000oooo0?ooo`040?ooo`030000003oool0000000@0oooo00<0o`000?oo
o`3oool06@3oool00`3o0000oooo0?ooo`0<0?ooo`8000009@3oool00`000000oooo0?ooo`180?oo
o`008P3oool0102oM000000000000000M;l?0?ooo`040?oog`2LB00004RL0=ooo`80oooo00L0okmd
0000001d_ol0oooo0?oog`2LB7@0_ooo0080oooo00<0_g@007Boo`3oool01P3oool00`000000oooo
0?ooo`0c0?ooo`030?l0003oool0oooo00D0oooo0P00000>0?ooo`<0000000<0oooo0?l0003oool0
1@3oool00`000000oooo0?ooo`040?ooo`040?l0000000000000000000l0oooo0P0000070?ooo`03
0?l0003oool0oooo03<0oooo00<000000?ooo`3oool0K03oool0102oM000000000000000M;l?0?oo
o`040?oog`2LB00004RL0=ooo`80oooo00L0okmd0000001d_ol0oooo0?oog`2LB7@0_ooo0080oooo
00<0_g@007Boo`3oool01P3oool00`000000oooo0?ooo`0c0?ooo`030?l0003oool0oooo01T0oooo
00<0o`000?ooo`3oool0103oool00`000000oooo000000040?ooo`030?l0003oool0oooo01T0oooo
00<0o`000?ooo`3oool03P3oool300000280oooo00<000000?ooo`3oool0B03oool001l0oooo00H0
oooO09a80000000000000000B02Lgol@0?ooo`040?oog`2LB00004RL0=ooo`/0oooo00<0oooO09a8
M02oool01P3oool00`000000oooo0?ooo`0c0?ooo`030?l0003oool0oooo00L0oooo0`0000090?oo
o`800000103oool00`3o0000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo00<0o`000?oo
o`3oool00P3oool2000000/0oooo0P0000090?ooo`030?l0003oool0oooo0380oooo0P00001[0?oo
o`060?oog`2LB00000000000000004P0W=oo403oool0103ooml0W4P00018W03Oool;0?ooo`030?oo
g`2LB7@0_ooo00H0oooo00<000000?ooo`3oool0<`3oool00`3o0000oooo0?ooo`0I0?ooo`030?l0
003oool0oooo00@0oooo0P0000040?ooo`030?l0003oool0oooo01X0oooo00<0o`000?ooo`3oool0
4@3oool;000001L0oooo00<000000?ooo`3oool0B03oool001h0oooo00@0oooO09a80000B9`0gooo
3`3oool00`2oM0000000000000050000000307Boo`3oool0oooo00T0oooo00<0_g@007Boo`3oool0
1@3oool00`000000oooo0?ooo`0c0?ooo`030?l0003oool0oooo00X0oooo2@0000070?ooo`030?l0
003oool0oooo00<0oooo00<000000?ooo`3oool00`3oool00`3o0000oooo0?ooo`040?ooo`/00000
2`3oool00`3o0000oooo0?ooo`0c0?ooo`030000003oool0oooo06P0oooo00@0oooO09a80000B9`0
gooo3`3oool00`2oM0000000000000050000000307Boo`3oool0oooo00T0oooo00<0_g@007Boo`3o
ool01@3oool010000000oooo0?ooo`3ooolJ000000H0oooo1@0000060?ooo`@000000`3oool0103o
0000oooo0?ooo`3oool4000000H0oooo1@0000060?ooo`@0000000<0o`000?ooo`3oool00`3oool5
000000030?ooo`3o0000oooo00<0oooo1@0000060?ooo`D000001P3oool4000000H0oooo1@000006
0?ooo`D000001P3ooolE000000<0oooo00<000000?ooo`3oool0B03oool001h0oooo00<0omnL07BL
g`3oool0403oool0203ogi`0M9cO0?ooo`3oool0oooO09a80000B9`0gooo2`3oool00`3ooml0W4P0
04RLg`060?ooo`030000003oool0oooo0380oooo00<0o`000?ooo`3oool06`3oool00`3o0000oooo
0?ooo`030?ooo`030000003oool0oooo00<0oooo00<0o`000?ooo`3oool06`3oool00`3o0000oooo
0?ooo`0b0?ooo`030000003oool0oooo06P0oooo00<0omnL07BLg`3oool0403oool0203ogi`0M9cO
0?ooo`3oool0oooO09a80000B9`0gooo2`3oool00`3ooml0W4P004RLg`060?ooo`030000003oool0
oooo01/0oooo1P00000A0?ooo`030?l0003oool0oooo01/0oooo00<0o`000?ooo`3oool00P3oool2
000000D0oooo00<0o`000?ooo`3oool06`3oool00`3o0000oooo0?ooo`0b0?ooo`030000003oool0
oooo04P0oooo000b0?ooo`070=nLB01d_ol0oooo0?oog`2LB00004RL0=ooo`0;0?ooo`030=nLB000
0000M;oo00H0oooo0P00000c0?ooo`030?l0003oool0oooo01/0oooo00<0o`000?ooo`3oool00`3o
ool00`000000oooo0?ooo`030?ooo`030?l0003oool0oooo01/0oooo00<0o`000?ooo`3oool0<@3o
ool2000007h0oooo00L0gia807Boo`3oool0oooO09a80000B9`0gooo00/0oooo00<0gia80000001d
_ol01P3oool200000280oooo0P00000?0?ooo`030?l0003oool0oooo01/0oooo00L0o`000?ooo`3o
ool0oooo0000003oool0000000D0oooo00<0o`000?ooo`3oool06`3oool00`3o0000oooo0?ooo`0a
0?ooo`800000BP3oool00380oooo00L0oooO09a8M02oool0oooO09a80000B9`0gooo00L0oooo00L0
gmoO0?ooo`3ookl0M0000000000007@0_ooo00H0oooo00<000000?ooo`3oool0<P3oool00`3o0000
oooo0?ooo`0K0?ooo`030?l0003oool0oooo00<0oooo00<000000?ooo`3oool00`3oool00`3o0000
oooo0?ooo`0K0?ooo`030?l0003oool0oooo0380oooo00<000000?ooo`3oool0O03oool01`3ooml0
W4Qd0;ooo`3ooml0W4P00018W03Oool01`3oool01`3Ogml0oooo0?oo_`1d000000000000M02oool0
1P3oool00`000000oooo0?ooo`0S0?ooo`<00000303oool00`3o0000oooo0?ooo`0K0?ooo`070?l0
003oool0oooo0?ooo`000000oooo000000050?ooo`030?l0003oool0oooo01/0oooo00<0o`000?oo
o`3oool0<P3oool00`000000oooo0?ooo`180?ooo`00<`3oool01P3ogi`0B7Bo0?oog`2LB00004RL
0=ooo`L0oooo00H0gia800000000000000000000B02Lgol70?ooo`030000003oool0oooo0380oooo
00<0o`000?ooo`3oool06`3oool00`3o0000oooo0?ooo`030?ooo`030000003oool0oooo0080oooo
00<0o`000?ooo`3oool0703oool00`3o0000oooo0?ooo`0b0?ooo`030000003oool0oooo07d0oooo
00H0omnL04Qd_`3ooml0W4P00018W03Oool70?ooo`060=nLB000000000000000000004P0W=oo1`3o
ool00`000000oooo0?ooo`0V0?ooo`8000002P3oool00`3o0000oooo0?ooo`0K0?ooo`040?l0003o
ool0oooo00000080oooo00<000000?ooo`3oool00P3oool00`3o0000oooo0?ooo`0L0?ooo`030?l0
003oool0oooo0380oooo00<000000?ooo`3oool0B03oool003@0oooo00D0gia807Bog`2LB00004RL
0=ooo`070?ooo`040?oo_`1d0000000004RLg`T0oooo00<000000?ooo`3oool0<P3oool00`3o0000
oooo0?ooo`0K0?ooo`030?l0003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool00`3o0000
oooo0?ooo`0L0?ooo`030?l0003oool0oooo0380oooo00<000000?ooo`3oool0OP3oool01@3OW4P0
M;oO09a80000B9`0gooo00L0oooo00@0oono07@000000000B9cO2@3oool00`000000oooo0?ooo`0X
0?ooo`030000003oool0oooo00L0oooo00<0o`000?ooo`3oool06`3oool00`3o0000oooo00000003
0?ooo`030000003oool0oooo0080oooo00<0o`000?ooo`3oool0703oool00`3o0000oooo0?ooo`0b
0?ooo`030000003oool0oooo04P0oooo000d0?ooo`050?oog`2LB4P0B4P00018W03Oool0203oool0
0`2oM000M;oo0?ooo`090?ooo`030000003oool0oooo0380oooo00<0o`000?ooo`3oool06`3oool0
0`3o0000oooo0?ooo`030?ooo`030000003oool0oooo0080oooo00<0o`000?ooo`3oool0703oool0
0`3o0000oooo0?ooo`0a0?ooo`800000P03oool01@3ooml0W4Q804Q80000B9`0gooo00P0oooo00<0
_g@007Boo`3oool02@3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo00H0oooo00<0
o`000?ooo`3oool06`3oool00`3o000000000?ooo`030?ooo`030000003oool0oooo0080oooo00<0
o`000?ooo`3oool0703oool00`3o0000oooo0?ooo`0b0?ooo`030000003oool0oooo04P0oooo000e
0?ooo`040?oOW018000004RL0=ooo`P0oooo00H0omnL04P00000000000000000B02Lgol60?ooo`03
0000003oool0oooo0380oooo00<0o`000?ooo`3oool0703oool00`3o0000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00<0o`000?ooo`3oool0703oool00`3o0000oooo0?ooo`0b0?ooo`03
0000003oool0oooo07l0oooo00@0omnL04P00000B9`0gooo203oool01P3ogi`0B000000000000000
001809cOo`H0oooo00<000000?ooo`3oool0:P3oool2000000H0oooo00<0o`000?ooo`3oool0703o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<0o`000?ooo`3oool0703o
ool00`3o0000oooo0?ooo`0b0?ooo`030000003oool0oooo04P0oooo000f0?ooo`030=nLB000B9`0
gooo00T0oooo00D0_g@0000000000000000004RLg`060?ooo`030000003oool0oooo0380oooo00<0
o`000?ooo`3oool0703oool00`3o0000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<0
o`000?ooo`3oool0703oool00`3o0000oooo0?ooo`0b0?ooo`030000003oool0oooo0800oooo00<0
gia80018W03Oool02@3oool01@2oM0000000000000000000B9cO00H0oooo00<000000?ooo`3oool0
;03oool00`000000oooo0?ooo`030?ooo`030?l0003oool0oooo01/0oooo00<000000?l0003oool0
0`3oool00`000000oooo0?ooo`020?ooo`030?l0003oool0oooo01`0oooo00<0o`000?ooo`3oool0
<P3oool00`000000oooo0?ooo`180?ooo`00C@3oool00`000000oooo0?ooo`0b0?ooo`030?l0003o
ool0oooo01`0oooo00<0o`000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030?l0003o
ool0oooo01`0oooo00<0o`000?ooo`3oool0<P3oool00`000000oooo0?ooo`2G0?ooo`030000003o
ool0oooo02d0oooo00<000000?ooo`3oool00P3oool00`3o0000oooo0?ooo`0J0?ooo`030000003o
ool0o`0000@0oooo00<000000?ooo`3oool00P3oool00`3o0000oooo0?ooo`0M0?ooo`030?l0003o
ool0oooo0340oooo00<000000?ooo`3oool0B03oool004d0oooo00<000000?ooo`3oool0<@3oool0
0`3o0000oooo0?ooo`0M0?ooo`030?l0003oool0oooo0080oooo00D000000?ooo`3oool0oooo0?l0
000P0?ooo`030?l0003oool0oooo0300oooo0P00002I0?ooo`030000003oool0oooo02h0oooo00@0
00000?ooo`3oool0o`00703oool010000000oooo0?ooo`3o00040?ooo`050000003oool0oooo0?oo
o`3o0000803oool00`3o0000oooo0?ooo`0a0?ooo`030000003oool0oooo04P0oooo001=0?ooo`03
0000003oool0oooo0340oooo00<0o`000?ooo`3oool07@3oool00`3o0000oooo0?ooo`020?ooo`05
0000003oool0oooo0?ooo`3o0000803oool00`3o0000oooo0?ooo`0a0?ooo`030000003oool0oooo
09L0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?l0000K0?ooo`050000003oool0oooo
0?ooo`3o0000103oool01@000000oooo0?ooo`3oool0o`000200oooo00<0o`000?ooo`3oool0<@3o
ool00`000000oooo0?ooo`180?ooo`00C@3oool00`000000oooo0?ooo`0a0?ooo`030?l0003oool0
oooo01h0oooo00D0o`000?ooo`3oool0oooo000000030?ooo`030?l0003oool0oooo01h0oooo00<0
o`000?ooo`3oool0<@3oool00`000000oooo0?ooo`2G0?ooo`030000003oool0oooo0300oooo00<0
00000?l0003oool06P3oool00`000000oooo0?ooo`020?ooo`050?l0003oool0oooo0?ooo`000000
0`3oool00`3o0000oooo0?ooo`0N0?ooo`030?l0003oool0oooo0340oooo00<000000?ooo`3oool0
B03oool004d0oooo00<000000?ooo`3oool0<@3oool00`3o0000oooo0?ooo`0N0?ooo`050?l0003o
ool0oooo0?ooo`0000000`3oool00`3o0000oooo0?ooo`0N0?ooo`030?l0003oool0oooo0300oooo
0P00002I0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`030?ooo`050?l0003oool0oooo0?ooo`0000000`3oool00`3o0000oooo0?ooo`0N0?ooo`03
0?l0003oool0oooo0340oooo00<000000?ooo`3oool0B03oool004d0oooo00<000000?ooo`3oool0
<@3oool00`3o0000oooo0?ooo`0N0?ooo`050?l0003oool0oooo0?ooo`0000000P3oool00`3o0000
oooo0?ooo`0O0?ooo`030?l0003oool0oooo0340oooo00<000000?ooo`3oool0U`3oool00`000000
oooo0?ooo`0a0?ooo`030?l000000000oooo01L0oooo00<000000?ooo`3oool0103oool01@3o0000
oooo0?ooo`3oool000000080oooo00<0o`000?ooo`3oool07`3oool00`3o0000oooo0?ooo`0a0?oo
o`030000003oool0oooo04P0oooo001=0?ooo`030000003oool0oooo0340oooo00<0o`000?ooo`3o
ool07`3oool0103o0000oooo0?ooo`0000020?ooo`030?l0003oool0oooo01l0oooo00<0o`000?oo
o`3oool0<@3oool00`000000oooo0?ooo`2G0?ooo`030000003oool0oooo0340oooo00<0o`000?oo
o`0000005P3oool00`000000oooo0?ooo`060?ooo`040?l0003oool0oooo00000080oooo00<0o`00
0?ooo`3oool07`3oool00`3o0000oooo0?ooo`0a0?ooo`030000003oool0oooo04P0oooo000o0?oo
o`030?oog`2LB00000000080000000<0M;oo0?ooo`3oool01P3oool00`000000oooo0?ooo`0a0?oo
o`030?l0003oool0oooo01l0oooo00@0o`000?ooo`3oool000000P3oool00`3o0000oooo0?ooo`0O
0?ooo`030?l0003oool0oooo0340oooo00<000000?ooo`3oool0R@3oool00`3ooml0W4P000000002
0000000307Boo`3oool0oooo00H0oooo00<000000?ooo`3oool0<@3oool0103o0000oooo0?ooo`00
000E0?ooo`030000003oool0oooo00H0oooo00@0o`000?ooo`3oool000000P3oool00`3o0000oooo
0?ooo`0O0?ooo`030?l0003oool0oooo0340oooo00<000000?ooo`3oool0B03oool003l0oooo00<0
oooO09a8M02oool00P3oool00`2oM000M;oo0?ooo`060?ooo`030000003oool0oooo0340oooo00<0
o`000?ooo`3oool07`3oool01P3o0000oooo0?ooo`000000oooo0?l00280oooo00<0o`000?ooo`3o
ool0<03oool2000008/0oooo00<0oooO09a8M02oool00P3oool00`2oM000M;oo0?ooo`060?ooo`03
0000003oool0oooo0340oooo00D0o`000?ooo`3oool0oooo0000000C0?ooo`030000003oool0oooo
00L0oooo00H0o`000?ooo`3oool000000?ooo`3o000R0?ooo`030?l0003oool0oooo0340oooo00<0
00000?ooo`3oool0B03oool004@0oooo00<0oooO09a8M02oool01P3oool00`000000oooo0?ooo`0`
0?ooo`030?l0003oool0oooo0200oooo00H0o`000?ooo`3oool000000?ooo`3o000R0?ooo`030?l0
003oool0oooo0340oooo00<000000?ooo`3oool0SP3oool00`3ooml0W4Qd0;ooo`060?ooo`030000
003oool0oooo0300oooo00<0o`000?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?ooo`030000
003oool0oooo00P0oooo00H0o`000?ooo`3oool000000?ooo`3o000S0?ooo`030?l0003oool0oooo
0300oooo00<000000?ooo`3oool0B03oool004D0oooo00<0_g@007Boo`3oool01@3oool00`000000
oooo0?ooo`0`0?ooo`030?l0003oool0oooo0240oooo00D0o`000?ooo`000000oooo0?l0000R0?oo
o`030?l0003oool0oooo0340oooo00<000000?ooo`3oool0S`3oool00`2oM000M;oo0?ooo`050?oo
o`030000003oool0oooo0300oooo00<0o`000?ooo`3oool0103oool2000000d0oooo0P00000<0?oo
o`050?l0003oool000000?ooo`3o00008`3oool00`3o0000oooo0?ooo`0`0?ooo`030000003oool0
oooo04P0oooo00140?ooo`030?oog`2LB000B9cO00H0oooo00<000000?ooo`3oool0<03oool00`3o
0000oooo0?ooo`0R0?ooo`030?l000000000o`0002<0oooo00<0o`000?ooo`3oool0<@3oool00`00
0000oooo0?ooo`2>0?ooo`030?oog`2LB000B9cO00H0oooo00<000000?ooo`3oool0<03oool00`3o
0000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool03@3oool00`3o
000000000?l0000T0?ooo`030?l0003oool0oooo0300oooo00<000000?ooo`3oool0B03oool004@0
oooo00<0gia80000001d_ol01P3oool200000340oooo00<0o`000?ooo`3oool08P3oool20?l002@0
oooo00<0o`000?ooo`3oool0<03oool200000900oooo00<0gia80000001d_ol01P3oool200000340
oooo00<0o`000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00h0
oooo0`3o000T0?ooo`030?l0003oool0oooo02l0oooo0P00001:0?ooo`00@03oool01`3Ogml0oooo
0?oo_`1d000000000000M02oool01P3oool00`000000oooo0?ooo`0`0?ooo`030?l0003oool0oooo
02<0oooo00<000000?ooo`3oool08`3oool00`3o0000oooo0?ooo`0`0?ooo`030000003oool0oooo
08X0oooo00L0gmoO0?ooo`3ookl0M0000000000007@0_ooo00H0oooo00<000000?ooo`3oool0<03o
ool00`3o0000oooo0?ooo`080?ooo`T000004P3oool00`000000oooo0?ooo`0S0?ooo`030?l0003o
ool0oooo0300oooo00<000000?ooo`3oool0B03oool00400oooo00H0gia800000000000000000000
B02Lgol70?ooo`030000003oool0oooo0300oooo00<0o`000?ooo`3oool08`3oool00`000000oooo
0?ooo`0S0?ooo`030?l0003oool0oooo0300oooo00<000000?ooo`3oool0RP3oool01P3OW4P00000
000000000000001809cOo`L0oooo00<000000?ooo`3oool0<03oool00`3o0000oooo0?ooo`0<0?oo
o`8000005@3oool00`000000oooo0?ooo`0S0?ooo`030?l0003oool0oooo0300oooo00<000000?oo
o`3oool0B03oool00400oooo00@0oono07@000000000B9cO2@3oool00`000000oooo0?ooo`0`0?oo
o`030?l0003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool00`3o0000oooo0?ooo`0_0?oo
o`800000S03oool0103ookl0M00000000018W=l90?ooo`030000003oool0oooo0300oooo00<0o`00
0?ooo`3oool08`3oool00`000000oooo0?ooo`0S0?ooo`030?l0003oool0oooo0300oooo00<00000
0?ooo`3oool0B03oool00440oooo00<0_g@007Boo`3oool02@3oool00`000000oooo0?ooo`0`0?oo
o`030?l0003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool00`3o0000oooo0?ooo`0`0?oo
o`030000003oool0oooo08/0oooo00<0_g@007Boo`3oool02@3oool00`000000oooo0?ooo`0`0?oo
o`030?l0003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool00`3o0000oooo0?ooo`0`0?oo
o`030000003oool0oooo04P0oooo00110?ooo`060?oOW018000000000000000004P0W=oo1P3oool0
0`000000oooo0?ooo`0`0?ooo`030?l0003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool0
0`3o0000oooo0?ooo`0`0?ooo`030000003oool0oooo08/0oooo00H0omnL04P00000000000000000
B02Lgol60?ooo`030000003oool0oooo0300oooo00<0o`000?ooo`3oool08`3oool00`000000oooo
0?ooo`0T0?ooo`030?l0003oool0oooo02l0oooo00<000000?ooo`3oool0B03oool00480oooo00D0
_g@0000000000000000004RLg`060?ooo`030000003oool0oooo02l0oooo00<0o`000?ooo`3oool0
903oool00`000000oooo0?ooo`0S0?ooo`030?l0003oool0oooo0300oooo00<000000?ooo`3oool0
S03oool01@2oM0000000000000000000B9cO00H0oooo00<000000?ooo`3oool0;`3oool00`3o0000
oooo0?ooo`0T0?ooo`030000003oool0oooo02@0oooo00<0o`000?ooo`3oool0;`3oool00`000000
oooo0?ooo`180?ooo`00C@3oool00`000000oooo0?ooo`0_0?ooo`030?l0003oool0oooo02@0oooo
00<000000?ooo`3oool08`3oool00`3o0000oooo0?ooo`0_0?ooo`800000V@3oool00`000000oooo
0?ooo`0_0?ooo`030?l0003oool0oooo02@0oooo00<000000?ooo`3oool0903oool00`3o0000oooo
0?ooo`0_0?ooo`030000003oool0oooo04P0oooo001=0?ooo`030000003oool0oooo02l0oooo00<0
o`000?ooo`3oool0903oool00`000000oooo0?ooo`0S0?ooo`030?l0003oool0oooo0300oooo00<0
00000?ooo`3oool0U`3oool00`000000oooo0?ooo`0_0?ooo`030?l0003oool0oooo02@0oooo00<0
00000?ooo`3oool0903oool00`3o0000oooo0?ooo`0_0?ooo`030000003oool0oooo04P0oooo001=
0?ooo`030000003oool0oooo02l0oooo00<0o`000?ooo`3oool0903oool00`000000oooo0?ooo`0S
0?ooo`030?l0003oool0oooo0300oooo00<000000?ooo`3oool0U`3oool00`000000oooo0?ooo`0_
0?ooo`030?l0003oool0oooo02@0oooo00<000000?ooo`3oool0903oool00`3o0000oooo0?ooo`0_
0?ooo`030000003oool0oooo04P0oooo001=0?ooo`030000003oool0oooo02l0oooo00<0o`000?oo
o`3oool0903oool00`000000oooo0?ooo`0S0?ooo`030?l0003oool0oooo0300oooo00<000000?oo
o`3oool0U`3oool00`000000oooo0?ooo`0_0?ooo`030?l0003oool0oooo02@0oooo00<000000?oo
o`3oool0903oool00`3o0000oooo0?ooo`0_0?ooo`030000003oool0oooo04P0oooo001=0?ooo`03
0000003oool0oooo02l0oooo00<0o`000?ooo`3oool0903oool00`000000oooo0?ooo`0S0?ooo`03
0?l0003oool0oooo02l0oooo0P00002I0?ooo`030000003oool0oooo02l0oooo00<0o`000?ooo`3o
ool0903oool00`000000oooo0?ooo`0T0?ooo`030?l0003oool0oooo02l0oooo00<000000?ooo`3o
ool0B03oool004d0oooo00<000000?ooo`3oool0;`3oool00`3o0000oooo0?ooo`0T0?ooo`030000
003oool0oooo02<0oooo00<0o`000?ooo`3oool0<03oool00`000000oooo0?ooo`2G0?ooo`030000
003oool0oooo02l0oooo00<0o`000?ooo`3oool0903oool00`000000oooo0?ooo`0T0?ooo`030?l0
003oool0oooo02l0oooo00<000000?ooo`3oool0B03oool00340oooo00<0oooO09a8000000000P00
00000`1d_ol0oooo0?ooo`020?ooo`060?noM0000000M;oo0?ooo`3ooml0W4P00`0000000`1d_ol0
oooo0?ooo`060?ooo`030000003oool0oooo02l0oooo00<0o`000?ooo`3oool0903oool00`000000
oooo0?ooo`0T0?ooo`030?l0003oool0oooo02l0oooo00<000000?ooo`3oool0N`3oool00`3ooml0
W4P0000000020000000307Boo`3oool0oooo0080oooo00H0okmd0000001d_ol0oooo0?oog`2LB003
0000000307Boo`3oool0oooo00H0oooo00<000000?ooo`3oool0;`3oool00`3o0000oooo0?ooo`0T
0?ooo`030000003oool0oooo02@0oooo00<0o`000?ooo`3oool0;`3oool00`000000oooo0?ooo`18
0?ooo`00<@3oool00`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0oooo0080oooo00L0okmd0000
001d_ol0oooo0?oog`2LB7@0_ooo0080oooo00<0_g@007Boo`3oool01P3oool00`000000oooo0?oo
o`0^0?ooo`030?l0003oool0oooo02D0oooo00<000000?ooo`3oool0903oool00`3o0000oooo0?oo
o`0^0?ooo`800000O@3oool00`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0oooo0080oooo00L0
okmd0000001d_ol0oooo0?oog`2LB7@0_ooo0080oooo00<0_g@007Boo`3oool01P3oool00`000000
oooo0?ooo`0^0?ooo`030?l0003oool0oooo02D0oooo00<000000?ooo`3oool09@3oool00`3o0000
oooo0?ooo`0^0?ooo`030000003oool0oooo04P0oooo000f0?ooo`030?oog`2LB7@0_ooo00/0oooo
00<0oooO09a8M02oool01P3oool00`000000oooo0?ooo`0^0?ooo`030?l0003oool0oooo02D0oooo
00<000000?ooo`3oool0903oool00`3o0000oooo0?ooo`0_0?ooo`030000003oool0oooo0800oooo
00<0oooO09a8M02oool02`3oool00`3ooml0W4Qd0;ooo`060?ooo`030000003oool0oooo02h0oooo
00<0o`000?ooo`3oool09@3oool00`000000oooo0?ooo`0U0?ooo`030?l0003oool0oooo02h0oooo
00<000000?ooo`3oool0B03oool003L0oooo00<0_g@007Boo`3oool02`3oool00`2oM000M;oo0?oo
o`050?ooo`030000003oool0oooo02h0oooo00<0o`000?ooo`3oool09@3oool00`000000oooo0?oo
o`0T0?ooo`030?l0003oool0oooo02l0oooo00<000000?ooo`3oool0P@3oool00`2oM000M;oo0?oo
o`0;0?ooo`030;md001d_ol0oooo00D0oooo00<000000?ooo`3oool0;P3oool00`3o0000oooo0?oo
o`0U0?ooo`030000003oool0oooo02D0oooo00<0o`000?ooo`3oool0;P3oool00`000000oooo0?oo
o`180?ooo`00=P3oool00`3ooml0W4P004RLg`0;0?ooo`030?oog`2LB000B9cO00H0oooo00D00000
0?ooo`3oool0oooo000000070?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00@000000?ooo`3oool0o`00
1@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
1P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
0P3oool0103o0000oooo0?ooo`0000080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00D000000?ooo`3o
ool0oooo000000220?ooo`030?oog`2LB000B9cO00/0oooo00<0oooO09a80018W=l01P3oool01@00
0000oooo0?ooo`3oool0000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?l00005
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`03
0?ooo`030?l0003oool0000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool01@000000oooo0?ooo`3o
ool0000004X0oooo000f0?ooo`030=nLB0000000M;oo00/0oooo00<0gia80000001d_ol01P3ooola
000000030?l000000000000004`0000000<0o`00000000000000<00000220?ooo`030=nLB0000000
M;oo00/0oooo00<0gia80000001d_ol01P3ooola000000030?l000000000000004d0000000<0o`00
000000000000;`00001:0?ooo`00<P3oool01`3Ogml0oooo0?oo_`1d000000000000M02oool01`3o
ool01`3Ogml0oooo0?oo_`1d000000000000M02oool0o`3ooolh0?ooo`070=oOg`3oool0oono07@0
00000000001d0;ooo`070?ooo`070=oOg`3oool0oono07@000000000001d0;ooo`3o0?ooo`@0oooo
000b0?ooo`060=nLB000000000000000000004P0W=oo203oool01P3OW4P000000000000000000018
09cOool0oooo>@3oool01P3OW4P00000000000000000001809cOo`P0oooo00H0gia8000000000000
00000000B02Lgooo0?ooo`D0oooo000b0?ooo`040?oo_`1d0000000004RLg`X0oooo00@0oono07@0
00000000B9cOo`3ooolk0?ooo`040?oo_`1d0000000004RLg`X0oooo00@0oono07@000000000B9cO
o`3oool70?ooo`00<`3oool00`2oM000M;oo0?ooo`0;0?ooo`030;md001d_ol0oooo0?l0oooo?03o
ool00`2oM000M;oo0?ooo`0;0?ooo`030;md001d_ol0oooo0?l0oooo1`3oool003<0oooo00H0omnL
04P00000000000000000B02Lgol80?ooo`060?oOW018000000000000000004P0W=ooR`3oool00`3o
oml0W4P0000000030000000304RLg`3oool0gia80080000000<0M;oo0?ooo`3oool0W`3oool01P3o
gi`0B000000000000000001809cOo`P0oooo00H0omnL04P00000000000000000B02Lgon@0?ooo`03
0?oog`2LB000000000<0000000<0B9cO0?ooo`3OW4P00P0000000`1d_ol0oooo0?ooo`1U0?ooo`00
=03oool01@2oM0000000000000000000B9cO00T0oooo00D0_g@0000000000000000004RLg`2;0?oo
o`030;md0000B9`0gooo00<0oooo00<0gibL0;ooo`3oool00P3oool00`2oM7@0_ooo0?ooo`2P0?oo
o`050;md00000000000000000018W=l02@3oool01@2oM0000000000000000000B9cO0900oooo00<0
_g@00018W03Oool00`3oool00`3OW9`0_ooo0?ooo`020?ooo`030;mdM02oool0oooo06D0oooo003B
0?ooo`030?oog`2LM9`0gooo00<0oooo00<0oono09bLg`3oool00P3oool00`3ogi`0M9cO0?ooo`3o
0?oood@0oooo00<0oooO09adW03Oool00`3oool00`3ookl0W9cO0?ooo`020?ooo`030?oOW01dW=l0
oooo06D0oooo003C0?ooo`030;md00000000000000<0000000L0B9cO0?ooo`3oool0oooo0?oog`2L
B7@0_ooo0?l0ooooA@3oool00`2oM0000000000000030000000704RLg`3oool0oooo0?ooo`3ooml0
W4Qd0;ooo`1U0?ooo`007P3oool00`2oM4P0W=oo0?ooo`020?ooo`030?oog`2LM9`0gooo00@0oooo
00P0oooO09a80000000000000018W02LM0000000001d_`<0oooo00<0oooO09a8M02oool01@3oool0
1@2oM00000000000000007@0_ooo00D0oooo00<0oooO09a8000000000P0000000`0004P0W=oo0?oo
o`040?ooo`090?oo_`1d0000000000000018W;l0W4P00018W03Oool0_g@00080000000<0001d0;oo
g`2LB0000P0000001000B9`0gooo0?ooo`2oM002000000090000B02Lgol0oooo0?ooo`3ogi`0B000
00000018W=l0gia80080000000@0001d0;ooo`3oool0gia80`000000101d_ol0oooo0?ooo`2oM002
000000030000M02ooml0W4P00080000000<004RL0=ooo`3oool01P3oool00`3ogi`0M9cO0?ooo`02
0?ooo`030=nLM02Lgol0oooo0080oooo00X0gia8000000000000001d0;ooo`3ogi`0B00000000018
W=l0gia80P0000000`0004P0W=oO09a8000200000003001d_`3oool0oooo0080oooo00<0oooO09a8
000000001`0000000`00M;l0oooo0;md0002000000030000M02ooml0W4P00080000000@004RL0=oo
o`3oool0_g@00P000000100007@0_ooo0?ooo`2oM002000000050000B02Lgol0oooo0?oo_`1d0000
100000000`0007@0_ooo0?ooo`030?ooo`030;md001d_ol0oooo01L0oooo00<0oono07@000000000
0P0000000`1d_ol0oooo0?ooo`020?ooo`030?noM0000000M;oo00<0oooo00D0gia8000000000000
000004RLg`020?ooo`050?oo_`1d000000000018W03Oool01P3oool0302oM0000000000000000000
04RL0=ooo`3oool0oono07@00000000004RL0=ooo`@0oooo00D0oono07@00000000004RL0=ooo`10
0?ooo`030;mdB02Lgol0oooo0080oooo00<0oooO09adW03Oool0103oool0203ooml0W4P000000000
000004RL09ad0000000007Bo0`3oool00`3ooml0W4Qd0;ooo`050?ooo`050;md0000000000000000
M02oool0103oool00`3ooml0W4P000000002000000030000B02Lgol0oooo00@0oooo00T0oono07@0
00000000000004RL_`2LB00004RL0=ooo`2oM0000P0000000`0007@0_ooO09a80002000000040018
W03Oool0oooo0;md0080000000T0001809cOo`3oool0oooo0?oOW0180000000004RLg`3OW4P00P00
0000100007@0_ooo0?ooo`3OW4P30000000407Boo`3oool0oooo0;md0080000000<0001d0;oog`2L
B0000P0000000`00B9`0gooo0?ooo`060?ooo`030?oOW01dW=l0oooo0080oooo00<0giad09cOo`3o
ool00P3oool02P3OW4P000000000000007@0_ooo0?oOW0180000000004RLg`3OW4P2000000030000
B02Lgml0W4P00080000000<007Bo0?ooo`3oool00`3oool00`3ooml0W4P00000000700000003001d
_`3oool0_g@00080000000<0001d0;oog`2LB0000P0000001000B9`0gooo0?ooo`2oM00200000004
0000M02oool0oooo0;md0080000000D0001809cOo`3oool0oono07@00004000000030000M02oool0
oooo00<0oooo00<0_g@007Boo`3oool06`3oool0103ooml0W4P00018W03Oool20?ooo`030?noM000
0000M;oo00D0oooo00L0oooO09a80000B9`0gooo0?ooo`3ookl0M00000<0000000<0M;oo0?ooo`3o
ool00`3oool01@3ogi`0B0000000000007@0_ooo0080oooo00<0oooO09a8000000000P0000000`1d
_ol0oooo0?ooo`0L0?ooo`007@3oool00`3ookl0M00004RLg`030?ooo`030?noM00004P0W=oo00@0
oooo00T0okmd0000001d_ol0oooo0?oOW018000007Bo0?no_`2oool00P3oool00`3o_g@0000004RL
g`040?ooo`070;md0000000007Bo0?ooo`3ogi`0B01d0;ooo`060?ooo`030?oOW0180000B9cO00L0
oooo00D0gia80000M02oool0oooo0=nLB002000000090000M02oool0gia80018W03Oool0oooo0?oo
g`2LB00007Bo0080oooo01P0_g@000000000B9`0gooo0?oog`2LB7@0_ooo0?ooo`3OW4P0001d0;oo
o`3O_kl0gooo0=nLB00007@0_ooo0?ooo`3OW4P0000004RLg`3oool0oooO09a80018W=l20?ooo`07
0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`090?ooo`070;md000007@0_ooo0?ooo`3ookl0M000
001d_`040?ooo`0@0=nLB00007@0_ooo0?ooo`3OW4P0001d0;ooo`3O_kl0gooo0=nLB00007@0_ooo
0?ooo`3ooml0W4P0001d_`L0oooo00<0omnL04P00018W=l0103oool02`3ogi`0B01d0;ooo`3oool0
gia80018W03Oool0oooo0?oog`2LB00007Bo0080oooo00/0_g@000000000M;l0oooo0?oOW01807@0
_ooo0?ooo`3OW4P0001d0;ooo`030?ooo`030;mdB02Lgol0oooo00L0oooo00@0omnL04P00000B9`0
gooo5`3oool00`3ogi`0B00004RLg`020?ooo`030;md0018W=l0oooo0080oooo00<0okmd0000001d
_ol00P3oool01`3OW4P0000007Boo`3oool0oooO09a80018W=l0103oool00`3OW4P0000007Boo`06
0?ooo`030?oOW0180000M;oo00H0oooo00<0gia80000001d_ol01P3oool00`3OW4P0000007Boo`0n
0?ooo`030?oo_`1d0000B9cO00<0oooo00<0okmd0000B02Lgol0103oool02@3o_g@0000007Boo`3o
ool0omnL04P00000M;l0okno0;ooo`020?ooo`030?noM0000000B9cO00@0oooo00L0_g@000000000
M;l0oooo0?oOW01807@0_ooo00D0oooo00<0omnL04P00018W=l01`3oool01@3OW4P0001d0;ooo`3o
ool0gia80080000000T0001d0;ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l00P3oool0602o
M00000000018W03Oool0oooO09a8M02oool0oooo0=nLB00007@0_ooo0=no_`3Oool0gia80000M02o
ool0oooo0=nLB0000000B9cO0?ooo`3ooml0W4P004RLg`80oooo00L0gia80018W03Oool0oooo0?oo
g`2LB00007Bo00T0oooo00L0_g@00000M02oool0oooo0?oo_`1d000007Bo00@0oooo0100gia80000
M02oool0oooo0=nLB00007@0_ooo0=no_`3Oool0gia80000M02oool0oooo0?oog`2LB00007Bo203o
ool00`3ogi`0B00004RLg`040?ooo`0;0?oOW01807@0_ooo0?ooo`3OW4P004RL0=ooo`3oool0oooO
09a80000M;l00P3oool02`2oM0000000001d_`3oool0omnL04P0M02oool0oooo0=nLB00007@0_ooo
00<0oooo00<0_gA809cOo`3oool01`3oool0103ogi`0B0000018W03OoolK0?ooo`040?oog`2LB000
04RL0=ooo`80oooo00<0okmd0000001d_ol01@3oool0203ooml0W4P00018W03Oool0oooo0?oOW018
0000B9cO0P3oool00`2oM000B9cO0?ooo`020?ooo`0;0?oo_`1d0000M;oo0?ooo`3o_g@0001d0;oo
o`3oool0oooO09a8M02oool00P3oool00`2oM000M;oo0?ooo`0L0?ooo`007@3oool0103o_g@00000
0018W03Oool20?ooo`030;md0000000007Bo00@0oooo00L0okmd0000B02Lgol0oooo0?oog`2LB000
07Bo00<0oooo00D0oooO09a80000B9`0W7Ad0;ooo`020?ooo`030?oOW0180000B9cO00<0oooo00<0
omnL07BLg`3oool01@3oool00`3ookl0M00004RLg`070?ooo`070;md0000B9`0gooo0?ooo`3ooml0
W4P0001d_`020?ooo`0;0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0omnL04P00000M;l0
0`3oool01P3ogi`0W;oo0?ooo`3OW4P004RL0=ooo`80oooo00H0gia80000M02oool0omnL04P00000
M;l30?ooo`0;0?noM00007@0_ooo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0203oool0
2@3ogi`0B00004QdM01dW=l0oooo0?noM00004P0B4Q809cOo`030?ooo`070=nLB00007@0_ooo0?oo
o`3OW4P004RL0=ooo`020?ooo`070=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`070?ooo`030?oo
_`1d0000B9cO00D0oooo00h0okmd07Boo`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo0?oo
o`3ogi`0B00004RLg`<0oooo00H0omnL07BLg`3oool0gia80000M02oool30?ooo`050?oo_`1d0000
000000000000M;l01@3oool0102oM00004RL09ad001d_ol70?ooo`030?oog`2LB000000000L00000
00<0B9cO0?ooo`3oool0203oool00`3ookl0M01809cOo`060?ooo`030?oog`2LB00007Bo00<0oooo
00<0okmd0018W03Oool0103oool00`3ookl0M00007Boo`050?ooo`030?oOW0180000M;oo00L0oooo
00<0oono07@0001d_ol01P3oool00`3ookl0M00007Boo`0m0?ooo`040?noM000000004RL0=ooo`80
oooo00<0_g@000000000M;l0103oool01`3o_g@0001809cOo`3oool0oooO09a80000M;l00`3oool0
1@3ooml0W4P00018W02LM7@0_ooo0080oooo00<0omnL04P00018W=l00`3oool00`3ogi`0M9cO0?oo
o`040?ooo`030?oo_`1d0000B9cO00L0oooo00L0_g@00018W03Oool0oooo0?oog`2LB00007Bo0080
oooo00/0gia80018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3ogi`0B000001d_`030?ooo`060?oO
W02L_ol0oooo0=nLB000B9`0gooo0P3oool01P3OW4P0001d0;ooo`3ogi`0B000001d_`<0oooo00/0
okmd0000M02oool0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`080?ooo`090?oOW0180000
B7Ad07BLg`3oool0okmd0000B018B4P0W=oo00<0oooo00L0gia80000M02oool0oooo0=nLB000B9`0
gooo0080oooo00L0gia80000M02oool0oooo0?oog`2LB00007Bo00P0oooo00<0oono07@00018W=l0
1@3oool03P3o_g@0M;oo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?oOW0180000
B9cO0`3oool01P3ogi`0M9cO0?ooo`3OW4P0001d0;ooo`<0oooo00D0oono07@0000000000000001d
_`050?ooo`040;md0000B9`0W7@007Boo`H0oooo00<0oooO09a8000000001`0000000`18W=l0oooo
0?ooo`080?ooo`040?oog`2LB00004RL0=ooo`X0oooo00@0oooO09a80000B9`0gooo1P3oool00`3o
okl0M01809cOo`020?ooo`030;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool01@3oool00`3o
oml0W4Qd0;ooo`0L0?ooo`00703oool02`3ooml0W4P00000M02LW7@0W=oo0?ooo`3ookl0M000001d
W02LB7@0_ooo00@0oooo00H0okmd00000000M;l0oooO09a80000M;l30?ooo`050?oOW0180000M;oo
0?noM018W=l00P3oool00`3o_g@0001809cOo`0;0?ooo`030?oo_`1d0000B9cO00L0oooo00L0_g@0
0018W03Oool0oooo0?oog`2LB00007Bo0080oooo00/0gia80018W03Oool0oooo0?oog`2LB00007Bo
0?ooo`3o_g@0001809cOo`060?ooo`030=nLB000B9`0gooo0080oooo00H0gia80000M02oool0okmd
0000001d_ol30?ooo`0;0?noM0000000M;oo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0
203oool02@3OW4P0001d0;ooo`2oM7@0_ooO09a80000M;l0omnL07BLg`030?ooo`070=nLB00007@0
_ooo0?ooo`3OW4P004RL0=ooo`020?ooo`070=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`070?oo
o`030?oo_`1d0000B9cO00P0oooo00/0gia80018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3o_g@0
001809cOo`060?ooo`030=nLB00007@0_ooo00<0oooo00H0gia80000M02oool0oono07@00018W=l3
0?ooo`050?noM0000000M;oo0?oOW018M;l0703oool00`3ookl0M00007Boo`070?ooo`030;md0000
M;l0oooo0080oooo00<0_g@00018W03Oool01@3oool00`3ogi`0B01809cOo`040?ooo`030?oOW018
0000M;oo00P0oooo00<0omnL04P0B02Lgol01P3oool00`3ogi`0B01809cOo`0k0?ooo`0;0?oog`2L
B000001d09bLM02Lgol0oooo0?oo_`1d000007BL09a8M02oool0103oool01P3o_g@00000001d_`3o
oml0W4P0001d_`<0oooo00D0omnL04P0001d_ol0okmd04RLg`020?ooo`030?noM00004P0W=oo00X0
oooo00<0oono07@00018W=l01`3oool01`2oM00004RL0=ooo`3oool0oooO09a80000M;l00P3oool0
2`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?noM00004P0W=oo00H0oooo00<0gia80018
W03Oool00P3oool01P3OW4P0001d0;ooo`3o_g@0000007Boo`<0oooo00/0okmd0000001d_ol0oooo
0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`080?ooo`090=nLB00007@0_ooo0;mdM02ooml0W4P0
001d_`3ogi`0M9cO00<0oooo00L0gia80000M02oool0oooo0=nLB000B9`0gooo0080oooo00L0gia8
0000M02oool0oooo0?oog`2LB00007Bo00P0oooo00<0oono07@00018W=l0203oool02`3OW4P004RL
0=ooo`3oool0oooO09a80000M;l0oooo0?noM00004P0W=oo00H0oooo00<0gia80000M02oool00`3o
ool01P3OW4P0001d0;ooo`3ookl0M00004RLg`<0oooo00D0okmd0000001d_ol0omnL04Qd_`0G0?oo
o`030;md00000000000000D0000000<0M;oo0?ooo`3oool00`3oool00`2oM0000000000000050000
000307Boo`3oool0oooo00<0oooo00L0oono07@0001d_ol0oooo0?oo_`1d0000B9cO00<0oooo00<0
gia80000M02oool01P3oool00`2oM000M;oo0?ooo`0K0?ooo`00703oool02`3ogi`0B00004RLg`3o
gi`0B7Bo0?ooo`3OW4P0000007Boo`3o_g@0M;oo00H0oooo00@0okmd00000000000007Bo0`3oool0
2P2oM00004RL0=ooo`3ooml0W4Qd0;ooo`3oool0okmd0000M02oool;0?ooo`030?oo_`1d0000B9cO
0080oooo00<0omnL07BLg`3oool00P3oool01`2oM00004RL0=ooo`3oool0oooO09a80000M;l00P3o
ool02`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?noM00007@0_ooo00H0oooo00<0gia8
0018W03Oool00P3oool01P3OW4P0001d0;ooo`3o_g@0000007Boo`<0oooo00/0okmd0000001d_ol0
oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`070?ooo`0;0?oo_`1d0000B9cO0?ooo`3ogi`0
M7Ad04P0B02Lgol0oooo0;mdM02oool00P3oool01`3OW4P0001d0;ooo`3oool0gia80018W03Oool0
0P3oool01`3OW4P0001d0;ooo`3oool0oooO09a80000M;l01`3oool00`3ookl0M00004RLg`030?oo
o`030?noM01d_ol0oooo0080oooo00/0gia80018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3o_g@0
001d0;ooo`060?ooo`030=nLB00007@0_ooo0080oooo00L0oono07@00018W=l0oooo0?oog`2LB000
07Bo0080oooo00L0oooO09a80000B9`0gooo0?ooo`2oM4P0W=oo01/0oooo00<0okmd0000001d_ol0
1`3oool01`3ogi`0B01d0;ooo`3oool0_g@00000001d_ol00P3oool00`3ooml0W4P0000000030000
0003001d_`3oool0oooo0080oooo00<0omnL04P0001d_ol0103oool00`3ooml0W4P0000000030000
0005001d_`3oool0oooo0?oog`2LB000100000000`00M;l0oooo0?ooo`0i0?ooo`0;0?oOW0180000
B9cO0?oOW018M;l0oooo0=nLB0000000M;oo0?noM01d_ol01P3oool0103o_g@0000000000000M;l3
0?ooo`0:0;md0000B9`0gooo0?oog`2LB7@0_ooo0?ooo`3o_g@0001d0;ooo`X0oooo00<0oono07@0
0018W=l00P3oool00`3ogi`0M9cO0?ooo`020?ooo`070;md0000B9`0gooo0?ooo`3ooml0W4P0001d
_`020?ooo`0;0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd0000M02oool01P3oool0
0`3OW4P004RL0=ooo`020?ooo`060=nLB00007@0_ooo0?noM0000000M;oo0`3oool02`3o_g@00000
07Boo`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo00L0oooo00/0oono07@00018W=l0oooo
0?oOW01dM7@0B01809cOo`3oool0_gAd0;ooo`020?ooo`070=nLB00007@0_ooo0?ooo`3OW4P004RL
0=ooo`020?ooo`070=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`080?ooo`030?oo_`1d0000B9cO
00<0oooo00<0okmd07Boo`3oool00P3oool02`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo
0?noM00007@0_ooo00H0oooo00<0gia80000M02oool00P3oool01`3ookl0M00004RLg`3oool0oooO
09a80000M;l00P3oool01`3ooml0W4P00018W03Oool0oooo0;mdB02Lgol05P3oool0203ogi`0M9cO
0?ooo`3oool0oooO09a80000B9`0gooo1P3oool0203ogi`0M9cO0?ooo`3oool0oooO09a80000B9`0
gooo1P3oool01`3o_g@0000007Boo`3oool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`05
0?ooo`030?oog`2LB000B9cO01`0oooo000L0?ooo`0;0=nLB00004P0W=oo0?oog`2LB7@0_ooO09a8
000007@0_ooo0?oo_`1dM;l00`3oool01`3ookl0M00007Boo`3oool0oooO09a80000M;l00P3oool0
0`3ogi`0B00007Boo`020?ooo`050?noM018W=l0oooo0?oOW01800001@0000000`18W=l0oooo0?oo
o`040?ooo`030?oo_`1d0000B9cO0080oooo00<0okmd04RLg`3oool00P3oool01`2oM00004RL0=oo
o`3oool0oooO09a80000M;l00P3oool02`2oM000001d0;ooo`3oool0oooO09a80000M;l0oooo0?oO
W01804P0W=oo0080oooo00L0okmd0000B02Lgol0oooo0=nLB000B9`0gooo0080oooo00H0gia80000
M02oool0oono07@0001d_ol30?ooo`0;0;md000004P0W=oo0?ooo`2oM000001d0;ooo`3oool0oooO
09a80000M;l01`3oool01`3OW4P0001809cOo`3oool0oooO09a80000M;l00P3oool02`3o_g@0B9cO
0?ooo`3oool0gia80000M02oool0oooo0=nLB000B9`0gooo0080oooo00L0gia80000M02oool0oooo
0?oog`2LB00007Bo00L0oooo00<0oono07@00018W=l00`3oool00`2oM000M;oo0?ooo`020?ooo`0:
0;md000007@0_ooo0?ooo`3ooml0W4P0001d_`3oool0omnL04P000D0000000D0B9cO0?ooo`3OW4P0
001809cOo`020?ooo`070?oo_`1d0000B9cO0?ooo`3ooml0W4P004RLg`020?ooo`030?noM00004P0
W=oo0080oooo00<0okmd04RLg`3oool06@3oool0103ogi`0B0000000B02Lgol80?ooo`050?oo_`1d
B4P0B00000000018W=l00`3oool0202oM000000007Boo`3oool0oooO09a8000007@0_ooo0`3oool0
0`3ogi`0B00007Boo`040?ooo`0A0;md00000000M;oo0?ooo`3ooml0W4P00000M02oool0oooo0;md
00000000M;oo0?ooo`3ooml0W4P00000M02oool0>P3oool02`3OW4P0001809cOo`3ooml0W4Qd0;oo
g`2LB000001d0;ooo`3ookl0M7Bo00<0oooo00L0oono07@0001d_ol0oooo0?oog`2LB00007Bo0080
oooo00<0omnL04P0001d_ol00P3oool01@3o_g@0B9cO0?ooo`3ogi`0B00000D0000000<0B9cO0?oo
o`3oool00`3oool00`3ookl0M00004RLg`020?ooo`030?noM018W=l0oooo0080oooo00L0_g@00018
W03Oool0oooo0?oog`2LB00007Bo0080oooo00/0_g@00000M02oool0oooo0?oog`2LB00007Bo0?oo
o`3ogi`0B01809cOo`020?ooo`070?noM00004P0W=oo0?ooo`3OW4P004RL0=ooo`020?ooo`060=nL
B00007@0_ooo0?oo_`1d0000M;oo0`3oool02`2oM000001809cOo`3oool0_g@00000M02oool0oooo
0?oog`2LB00007Bo00L0oooo00L0gia80000B02Lgol0oooo0?oog`2LB00007Bo0080oooo00/0okmd
04RLg`3oool0oooo0=nLB00007@0_ooo0?ooo`3OW4P004RL0=ooo`020?ooo`070=nLB00007@0_ooo
0?ooo`3ooml0W4P0001d_`080?ooo`030?oo_`1d0000B9cO00<0oooo00<0_g@007Boo`3oool00P3o
ool02P2oM000001d0;ooo`3oool0oooO09a80000M;l0oooo0?oOW01800050000000504RLg`3oool0
gia80000B02Lgol00P3oool01`3ookl0M00004RLg`3oool0oooO09a80018W=l00P3oool00`3o_g@0
001809cOo`020?ooo`030?noM018W=l0oooo01H0oooo00L0gia807Boo`3oool0oooO09a80000B9`0
gooo00L0oooo00L0gia807Boo`3oool0oooO09a80000B9`0gooo00D0oooo00P0omnL04P0000004P0
W=oo0?ooo`3ogi`0B00007Boo`<0oooo00<0okmd0000B02Lgol01@3oool00`3OW4P0000007Boo`0L
0?ooo`006`3oool02@3ookl0M0000018W03Oool0oooo0?noM01dW9`0B00004RLg`020?ooo`0@0;md
B02Lgol0oooo0?ooo`3ookl0M00004RLg`3oool0oono07@00000M;l0oooo0?oog`2LB000001d0;oo
o`80oooo00H0oooO09a8M02oool0oooo0;md0018W=l20?ooo`030=nLB00004P0W=oo00H0oooo00<0
oono07@0000000000`0000005018W=l0oooo0?ooo`3oool0_g@00018W03Oool0oooo0?oog`2LB000
07Bo0?oog`2LB00000000018B01804P0W=oo0?oo_`1d0000B9cO0P3oool08@2oM00004RL0=ooo`3o
oml0W4P00000M02oool0oooo0=nLB000B9`0gooo0?oog`2LB00000000000M02oool0oooo0=nLB000
B9`0gooo0?ooo`3o_g@00000001d_`3ooml0W4P000000000B4P0B01809cOo`3ookl0M00004RLg`06
0?ooo`030?oo_`1d000007Bo0080oooo00<0omnL04P0001d_ol00P3oool02`3ooml0W4Qd0;oog`2L
B00000000000M02oool0oooo0=nLB000B9`0gooo0080oooo00L0gia80000000004P0W=oo0?oo_`1d
0000B9cO00L0oooo00<0oono07@000000000100000002`1d_ol0oooo0?oog`2LB00000000018B018
04P0W=oo0?oo_`1d0000B9cO0080oooo0180_g@004RLg`3oool0oooo0=nLB00004P0W=oO09a80000
0000001804QdW03O_g@0001d0;ooo`2oM00007Bo0?ooo`3o_g@2000000050018W03Ookl0M0000018
W03Oool00P3oool00`3ookl0M01d0;ooo`050?ooo`030?oog`2LB000000000L0000000<0B9cO0?oo
o`3oool01@3oool01@3o_g@000000000000004P0W=oo00T0oooo00@0omnL04P00000000007Bo0`3o
ool00`3ogi`0B00004RLg`030?ooo`030?noM00004P0W=oo00<0oooo00<0omnL04P0001d_ol00`3o
ool00`3ogi`0B00004RLg`030?ooo`060?noM00004P0W=oo0?oOW0180000B9cO0`3oool00`3o_g@0
001809cOo`0i0?ooo`090?oo_`1d000004RL0=ooo`3oool0okmd07BLW0180000B9cO0080oooo0100
_gA809cOo`3oool0oooo0?oo_`1d0000B9cO0?ooo`3ookl0M000001d_`3oool0oooO09a8000007@0
_ooo0P3oool01P3ooml0W4Qd0;ooo`3oool0_g@004RLg`80oooo00<0gia80000B02Lgol01@3oool0
0`3ookl0M000000000030000000D04RLg`3oool0oooo0?ooo`2oM00004RL0=ooo`3oool0oooO09a8
0000M;l0oooO09a80000000004Q804P0B02Lgol0oono07@00018W=l20?ooo`0Q0;md0000B9`0gooo
0?oog`2LB000001d0;ooo`3oool0gia80018W03Oool0oooO09a800000000001d0;ooo`3oool0gia8
0018W03Oool0oooo0?noM000000007Bo0?oog`2LB00000000018B01804P0W=oo0?oo_`1d0000B9cO
00H0oooo00<0oono07@00000M;l00P3oool00`3ogi`0B00007Boo`020?ooo`0;0?oog`2LB7@0_ooO
09a800000000001d0;ooo`3oool0gia80018W03Oool00P3oool01`3OW4P000000000B02Lgol0oono
07@00018W=l0203oool00`3ookl0M000000000040000000;07Boo`3oool0oooO09a80000000004Q8
04P0B02Lgol0oono07@00018W=l00P3oool04P2oM000B9cO0?ooo`3oool0gia80000B02Lgml0W4P0
0000000004P0B7BL0=noM00007@0_ooo0;md0000M;l0oooo0?noM080000000D004RL0=oo_`1d0000
04RL0=ooo`020?ooo`030?oo_`1d07@0_ooo00@0oooo00<0oooO09a8000000001`0000000`18W=l0
oooo0?ooo`050?ooo`070?oog`2LB7@0_ooo0?oog`2LB00004RL0=ooo`070?ooo`070?oog`2LB7@0
_ooo0?oog`2LB00004RL0=ooo`030?ooo`050?noM000000000000000B02Lgol00P3oool00`3ogi`0
B00007Boo`030?ooo`0;0?noM00004P0W=oo0?ooo`3Ogml0oooo0?oo_`1d000000000000M02oool0
703oool001/0oooo00<0okmd00000018W=l00P3oool0103ookl0M0000000B02Lgol20?ooo`030?no
M018W=l0oooo0080oooo00P0oono07@00000000000000000M02oool0oooO09a80080000000T0001d
0;ooo`3ooml0W4P000000000B9`0gooo0?ooo`3OW4P00P0000000`0004P0W=oo0?ooo`060?ooo`03
0?oo_`1d0000B9cO0080oooo00`0okmd04RLg`3oool0oono07@00000000004RL0=ooo`3ogi`0B000
00000000M;l20?ooo`070;md0000B9`0gooO09a800000000001d0;ooo`030?ooo`070=nLB0000000
00000000B02Lgol0oooo0;md0003000000050018W03Oool0_g@00000M02oool00P3oool01@3o_g@0
0000000000000000B9cO00<0oooo00L0_g@00018W03Ooml0W4P00000000007@0_ooo00D0oooo00H0
oooO09a800000000000004RLg`3o_g@30000000:04RLg`3oool0_g@000000000B9`0gooo0;md0000
07@0_ooo0;md00<0000000T004RL0=ooo`3OW4P0001d0;oog`2LB00000000000M02oool01`3oool0
0`3ookl0M00004RLg`030?ooo`030;md001d_ol0oooo0080oooo00L0_g@00018W03Ooml0W4P00000
000007@0_ooo00<0oooo00D0gia8000000000000001809cOo`020?ooo`080;md000007@0_mnL04P0
000007@0_ooo0?oog`2LB0020000000407Boo`3oool0oooO09a80080000000L004RL0=ooo`3oool0
_g@000000000B9`0gooo01P0oooo00<0_g@0001d_`3oool0203oool01P3ogi`0B0000000M02oool0
okmd001d_`80oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`030?ooo`030?oOW018
0000M;oo00<0oooo00<0omnL04P0001d_ol00`3oool01P3o_g@0001809cOo`3ogi`0B00007Boo`<0
oooo00<0okmd0000B02Lgol0>@3oool00`3o_g@0000004RLg`020?ooo`040?oo_`1d0000001809cO
o`80oooo00<0okmd04RLg`3oool00P3oool0203ookl0M000000000000000001d0;ooo`3ooml0W4P0
0P0000002@0007@0_ooo0?oog`2LB00000000018W03Oool0oooo0=nLB002000000030000B02Lgol0
oooo00D0oooo00<0oono07@00018W=l00P3oool0303o_g@0B9cO0?ooo`3ookl0M00000000000B9`0
gooo0?oOW01800000000001d_`80oooo00L0_g@00018W03Ooml0W4P00000000007@0_ooo00<0oooo
00L0gia8000000000000001809cOo`3oool0_g@000<0000000D004RL0=ooo`2oM000001d0;ooo`02
0?ooo`050?noM0000000000000000018W=l00`3oool01`2oM00004RL0=oog`2LB00000000000M02o
ool01@3oool01P3ooml0W4P0000000000000B9cO0?noM0<0000000X0B9cO0?ooo`2oM00000000018
W03Oool0_g@00000M02oool0_g@00`0000002@00B9`0gooo0=nLB00007@0_ooO09a800000000001d
0;ooo`080?ooo`030?oo_`1d0000B9cO00<0oooo00<0_g@007Boo`3oool00P3oool01`2oM00004RL
0=oog`2LB00000000000M02oool00`3oool01@3OW4P000000000000004P0W=oo0080oooo00P0_g@0
0000M02ogi`0B0000000M02oool0oooO09a80080000000@0M;oo0?ooo`3ooml0W4P00P0000001`00
B9`0gooo0?ooo`2oM00000000018W03Oool05P3oool01P3ogi`0B7Bo0?oog`2LB00004RL0=ooo`P0
oooo00H0omnL04Qd_`3ooml0W4P00018W03Oool50?ooo`030;md0000M;l0oooo0080oooo00<0omnL
04P0001d_ol00`3oool01@3o_g@0001809cOo`3oool0gia800<0000000<0001809cOo`3oool0703o
ool001/0oooo00<0_g@00000M02oool00P3oool0103ookl0M0000018W03Oool20?ooo`030?oo_`1d
B9`0gooo0200oooo00<0oono07@00018W=l00P3oool00`3ogi`0M9cO0?ooo`0L0?ooo`030;md0000
B9`0gooo02h0oooo00<0_g@00018W03Oool00P3oool00`3OW4P0001d0;ooo`0;0?ooo`030?oo_`1d
0000B9cO00<0oooo00<0okmd07Boo`3oool0@`3oool00`2oM000B9cO0?ooo`070?ooo`030;md0000
07@0_ooo0080oooo00H0_g@004RLg`3oool0omnL04P0001d_ol30?ooo`030=nLB00007@0_ooo00<0
oooo00<0omnL04P0001d_ol00`3oool00`3ogi`0B00007Boo`030?ooo`060=nLB00007@0_ooo0?oO
W0180000M;oo0`3oool00`3OW4P0001d0;ooo`0i0?ooo`030;md000007@0_ooo0080oooo00@0oono
07@00000B9`0gooo0P3oool00`3ookl0M4RL0=ooo`0O0?ooo`030?oo_`1d0000B9cO0080oooo00<0
omnL07BLg`3oool0703oool00`2oM00004RL0=ooo`0^0?ooo`030;md0000B9`0gooo0080oooo00<0
gia80000M02oool0303oool00`3ookl0M00004RLg`030?ooo`030?noM01d_ol0oooo0440oooo00D0
gia807Bog`2LB00004RL0=ooo`090?ooo`050=nLB01d_ml0W4P00018W03Oool01P3oool01`2oM000
B9cO0?ooo`3oool0oono07@00018W=l00`3oool0203OW4P0001d0;ooo`3oool0oono07@000000000
B9cO7`3oool001X0oooo00<0omnL04P00000M;l00`3oool00`3o_g@0000004RLg`040?ooo`030=nL
M02Lgol0oooo01l0oooo00<0oono07@00018W=l0103oool00`3OW7@0W=oo0?ooo`0J0?ooo`030?oo
_`1dB9`0gooo02h0oooo00<0oono07A8W03Oool00P3oool00`3OW4P0001d0;ooo`0;0?ooo`030?oo
_`1d0000B9cO00@0oooo00<0oono07Ad_`3oool0?@3oool00`3ooml0W9bo0?ooo`020?ooo`030=nL
B00007@0_ooo00L0oooo00<0_g@00018W03Oool00P3oool01P3OW4P007Bo0?ooo`3ooml0W4P007Bo
o`80oooo00@0oooO09a80000B9`0gooo0`3oool00`3ogi`0B00007Boo`030?ooo`030?oog`2LB000
M;oo0080oooo00L0oooO09a80000B9`0gooo0?oog`2LB000M;oo0080oooo00@0oooO09a80000B9`0
gooo>03oool00`3ogi`0B000001d_`030?ooo`030?noM0000000B9cO00@0oooo00<0giad09cOo`3o
ool07P3oool00`3ookl0M00004RLg`040?ooo`030=nLM02Lgol0oooo01X0oooo00<0oono07A8W03O
ool0;P3oool00`3ookl0M4RL0=ooo`020?ooo`030=nLB00007@0_ooo00`0oooo00<0oono07@00018
W=l0103oool00`3ookl0M7Bo0?ooo`100?ooo`050?oog`2LB4P0B4P00018W03Oool02@3oool0203o
oml0W4Q804Q80000B9`0gooo0?ooo`3ooml0W9bo0`3oool00`3OW4P0001d0;ooo`020?ooo`030=nL
B000B9`0gooo0080oooo00<0_g@0001d_`3oool00P3oool00`2oM000M;oo0?ooo`0O0?ooo`006P3o
ool00`3OW4P0000004RLg`020?ooo`040?oog`2LB000000007Boo`@0oooo00<0gia8001d_`3oool0
7`3oool00`3ogi`0B00004RLg`030?ooo`030?oo_`1d07@0_ooo01/0oooo00<0oooO09adW03Oool0
;P3oool00`3ooml0W7BL0=ooo`020?ooo`030=nLB00007@0_ooo00/0oooo00<0omnL04P00018W=l0
103oool00`3OW4P0B9cO0?ooo`0n0?ooo`060?noM018W=l0oooo0?oo_`1d000007Bo203oool01`3o
_g@0000007Boo`3oool0oono07@0001d_ol00P3oool01`3o_g@0001d0;ooo`3oool0gia80000B02L
gol00P3oool01@3ooml0W4P0000000000000M;oo00@0oooo00L0okmd0000M02oool0oooo0=nLB000
04P0W=oo0080oooo00L0okmd0000M02oool0oooo0=nLB00004P0W=oo03T0oooo00<0gia800000018
W=l00P3oool0103ooml0W4P00000001d_ol40?ooo`030=nLB000M;l0oooo01h0oooo00<0omnL04P0
0018W=l00`3oool00`3ookl0M01d0;ooo`0K0?ooo`030?oog`2LM9`0gooo02h0oooo00<0oooO09ad
W03Oool00P3oool00`3OW4P0001d0;ooo`0<0?ooo`030?oOW0180000B9cO00@0oooo00<0gia804RL
g`3oool0@@3oool0103ogi`0B0000018W03Oool:0?ooo`040?oOW018000004RL0=ooo`80oooo00H0
okmd04RLg`3oool0oono07@00000M;l30?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`03
0?ooo`060?oOW018000000000000000004P0W=oo703oool001P0oooo00<0oooO09a8000000000P00
00000`00M;l0omnL04P000030000000404RLg`3oool0oooo0;md0080000000<004RL0=ooo`3oool0
6`3oool00`3ooml0W4P000000006000000030000M02oool0oooo01l0oooo00<0gia80000B02Lgol0
9@3oool00`3OW4P0001809cOo`040?ooo`050?oog`2LB00000000000M02oool02@3oool00`3ooml0
W4P0000000070000000304RLg`3oool0oooo03h0oooo00D0gia8000000000000001d0;ooo`090?oo
o`050?noM0000000000000000018W=l0103oool01@3o_g@000000000000004P0W=oo00D0oooo00<0
okmd0000001d_ol01@3oool01@3o_g@000000000000004P0W=oo00@0oooo00D0okmd000000000000
001809cOo`0h0?ooo`030?oog`2LB00000000080000000<007Bo0?oOW01800000`0000001018W=l0
oooo0?ooo`2oM002000000030018W03Oool0oooo01X0oooo00<0oooO09a8000000001P0000000`00
07@0_ooo0?ooo`0O0?ooo`030=nLB00004P0W=oo02D0oooo00<0gia80000B02Lgol0103oool01@3o
oml0W4P00000000007@0_ooo00X0oooo00<0oooO09a8000000001`0000000`18W=l0oooo0?ooo`11
0?ooo`030=nLB000B9`0gooo00/0oooo00<0gia80018W03Oool00`3oool01@3OW4P0000000000000
07@0_ooo00@0oooo00D0oono07@000000000001d0;ooo`050?ooo`050;md00000000000000000018
W=l0703oool00780oooo00<0gia80000B02Lgol09@3oool00`3OW4P0001809cOo`060?ooo`030;md
000007@0_ooo0?l0oooo7@3oool00`3OW4P0001809cOo`0U0?ooo`030=nLB00004P0W=oo00H0oooo
00<0_g@00000M02oool0X03oool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?oo
ool0ooooV03oool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03o
ool00?l0ooooo`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0oooo
o`3ooonH0?ooo`00o`3ooooo0?oooiP0oooo003o0?ooool0ooooV03oool00?l0ooooo`3ooonH0?oo
o`00o`3ooooo0?oooiP0oooo0000\
\>"],
  ImageRangeCache->{{{0, 661.688}, {408.563, 0}} -> {-0.0509877, -0.0309082, \
0.00476504, 0.00476504}, {{16, 315.813}, {398.813, 213.5}} -> {-10.0602, \
4.42899, 0.0882839, 0.0272088}, {{345.813, 645.625}, {398.813, 213.5}} -> \
{-29.2068, 6.69675, 0.0882839, 0.0204066}, {{16, 315.813}, {195, 9.6875}} -> \
{-9.79535, -4.65201, 0.0882839, 0.0408133}, {{345.813, 645.625}, {195, \
9.6875}} -> {-29.3392, -4.65201, 0.0882839, 0.0408133}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
        Evaluate[wavefunction[x] /. solutions], {x, \(-5\), 5}];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0952381 0.309019 0.474879 [
[.11905 .29652 -9.15625 -15.6875 ]
[.11905 .29652 9.15625 0 ]
[.30952 .29652 -9.15625 -15.6875 ]
[.30952 .29652 9.15625 0 ]
[.69048 .29652 -5 -15.6875 ]
[.69048 .29652 5 0 ]
[.88095 .29652 -5 -15.6875 ]
[.88095 .29652 5 0 ]
[.4875 .02409 -27.3125 -7.84375 ]
[.4875 .02409 0 7.84375 ]
[.4875 .11907 -27.3125 -7.84375 ]
[.4875 .11907 0 7.84375 ]
[.4875 .21404 -27.3125 -7.84375 ]
[.4875 .21404 0 7.84375 ]
[.4875 .40399 -19 -7.84375 ]
[.4875 .40399 0 7.84375 ]
[.4875 .49897 -19 -7.84375 ]
[.4875 .49897 0 7.84375 ]
[.4875 .59395 -19 -7.84375 ]
[.4875 .59395 0 7.84375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.11905 .30902 m
.11905 .31527 L
s
gsave
.11905 .29652 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.30952 .30902 m
.30952 .31527 L
s
gsave
.30952 .29652 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.69048 .30902 m
.69048 .31527 L
s
gsave
.69048 .29652 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.88095 .30902 m
.88095 .31527 L
s
gsave
.88095 .29652 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.125 Mabswid
.16667 .30902 m
.16667 .31277 L
s
.21429 .30902 m
.21429 .31277 L
s
.2619 .30902 m
.2619 .31277 L
s
.35714 .30902 m
.35714 .31277 L
s
.40476 .30902 m
.40476 .31277 L
s
.45238 .30902 m
.45238 .31277 L
s
.54762 .30902 m
.54762 .31277 L
s
.59524 .30902 m
.59524 .31277 L
s
.64286 .30902 m
.64286 .31277 L
s
.7381 .30902 m
.7381 .31277 L
s
.78571 .30902 m
.78571 .31277 L
s
.83333 .30902 m
.83333 .31277 L
s
.07143 .30902 m
.07143 .31277 L
s
.02381 .30902 m
.02381 .31277 L
s
.92857 .30902 m
.92857 .31277 L
s
.97619 .30902 m
.97619 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.5 .02409 m
.50625 .02409 L
s
gsave
.4875 .02409 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.6) show
1.000 setlinewidth
grestore
.5 .11907 m
.50625 .11907 L
s
gsave
.4875 .11907 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.4) show
1.000 setlinewidth
grestore
.5 .21404 m
.50625 .21404 L
s
gsave
.4875 .21404 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
.5 .40399 m
.50625 .40399 L
s
gsave
.4875 .40399 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
.5 .49897 m
.50625 .49897 L
s
gsave
.4875 .49897 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.4) show
1.000 setlinewidth
grestore
.5 .59395 m
.50625 .59395 L
s
gsave
.4875 .59395 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.6) show
1.000 setlinewidth
grestore
.125 Mabswid
.5 .04784 m
.50375 .04784 L
s
.5 .07158 m
.50375 .07158 L
s
.5 .09532 m
.50375 .09532 L
s
.5 .14281 m
.50375 .14281 L
s
.5 .16656 m
.50375 .16656 L
s
.5 .1903 m
.50375 .1903 L
s
.5 .23779 m
.50375 .23779 L
s
.5 .26153 m
.50375 .26153 L
s
.5 .28528 m
.50375 .28528 L
s
.5 .33276 m
.50375 .33276 L
s
.5 .35651 m
.50375 .35651 L
s
.5 .38025 m
.50375 .38025 L
s
.5 .42774 m
.50375 .42774 L
s
.5 .45148 m
.50375 .45148 L
s
.5 .47523 m
.50375 .47523 L
s
.5 .52271 m
.50375 .52271 L
s
.5 .54646 m
.50375 .54646 L
s
.5 .5702 m
.50375 .5702 L
s
.5 .00035 m
.50375 .00035 L
s
.5 .61769 m
.50375 .61769 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .30902 m
.03279 .30903 L
.04262 .30905 L
.04758 .30906 L
.05288 .30908 L
.06244 .30912 L
.06788 .30914 L
.07379 .30918 L
.07899 .30922 L
.08451 .30928 L
.08947 .30934 L
.09494 .30942 L
.10458 .30961 L
.10978 .30975 L
.11472 .3099 L
.12399 .31028 L
.13397 .31084 L
.14487 .31171 L
.15519 .31286 L
.16454 .31425 L
.17534 .31641 L
.18551 .31911 L
.19528 .32246 L
.20559 .32697 L
.22406 .33817 L
.23354 .34576 L
.2437 .35549 L
.26204 .37757 L
.3034 .44769 L
.32409 .48857 L
.34324 .52408 L
.36183 .55158 L
.37121 .56158 L
.37656 .56591 L
.38157 .56903 L
.38384 .57014 L
.38627 .57112 L
.38766 .57158 L
.38893 .57195 L
.39013 .57223 L
.3914 .57248 L
.39204 .57259 L
.39273 .57268 L
.39396 .57281 L
.39465 .57286 L
.39539 .57289 L
.39609 .57291 L
.39673 .5729 L
.39796 .57286 L
.39866 .57282 L
Mistroke
.39931 .57276 L
.40049 .57262 L
.40175 .57241 L
.40416 .57189 L
.40676 .57112 L
.41147 .56924 L
.41605 .56684 L
.42026 .56421 L
.44072 .54749 L
.4598 .5306 L
.46923 .52348 L
.47816 .51811 L
.48309 .51584 L
.48569 .51486 L
.48845 .51399 L
.49077 .5134 L
.492 .51314 L
.49332 .5129 L
.49453 .51272 L
.49564 .51258 L
.49671 .51248 L
.49782 .51241 L
.49906 .51236 L
.49976 .51235 L
.5004 .51235 L
.50159 .51238 L
.50284 .51245 L
.50396 .51254 L
.50501 .51266 L
.50739 .51302 L
.50997 .51357 L
.51232 .51422 L
.5176 .51613 L
.52212 .51826 L
.52704 .52105 L
.53598 .52726 L
.55534 .54391 L
.56563 .55314 L
.57673 .5621 L
.58187 .56558 L
.58728 .56863 L
.5919 .57065 L
.59452 .57153 L
.59696 .57216 L
.59815 .57239 L
.59943 .5726 L
.60065 .57275 L
.60177 .57284 L
.60278 .57289 L
.6039 .5729 L
Mistroke
.60506 .57287 L
.60615 .5728 L
.60741 .57266 L
.60862 .57248 L
.61088 .57199 L
.61331 .57127 L
.61597 .57023 L
.62129 .56736 L
.6263 .56372 L
.63565 .55455 L
.64573 .5415 L
.65663 .52429 L
.69766 .44564 L
.71809 .40816 L
.73716 .37867 L
.7555 .35632 L
.77515 .33874 L
.78534 .33192 L
.79641 .32601 L
.80687 .32165 L
.81654 .3185 L
.82617 .31606 L
.83521 .31429 L
.84513 .31281 L
.85582 .31165 L
.8654 .31088 L
.87588 .31029 L
.88117 .31006 L
.88681 .30985 L
.89698 .30958 L
.90232 .30947 L
.90723 .30939 L
.91253 .30931 L
.91834 .30925 L
.92814 .30917 L
.93349 .30914 L
.9385 .30911 L
.94334 .30909 L
.94861 .30907 L
.9582 .30905 L
.96754 .30903 L
.97619 .30902 L
Mfstroke
.02381 .30902 m
.03279 .30904 L
.04262 .30908 L
.04758 .3091 L
.05288 .30913 L
.06244 .30919 L
.06788 .30924 L
.07379 .3093 L
.07899 .30937 L
.08451 .30947 L
.08947 .30957 L
.09494 .30971 L
.10458 .31003 L
.10978 .31026 L
.11472 .31052 L
.12399 .31114 L
.13397 .31206 L
.14487 .31347 L
.15519 .31531 L
.16454 .31753 L
.17534 .32092 L
.18551 .32511 L
.19528 .33026 L
.20559 .3371 L
.22406 .35373 L
.2437 .37869 L
.26204 .40941 L
.30094 .4939 L
.32075 .53917 L
.3312 .56055 L
.3423 .57968 L
.35214 .59249 L
.35484 .5952 L
.35774 .59771 L
.36025 .59952 L
.363 .60111 L
.36422 .60168 L
.36552 .60219 L
.36665 .60256 L
.36787 .60288 L
.36859 .60303 L
.36925 .60314 L
.37055 .60328 L
.37174 .60332 L
.37299 .60327 L
.37408 .60316 L
.37524 .60296 L
.37635 .6027 L
.37735 .6024 L
.37981 .60141 L
Mistroke
.38215 .60014 L
.38719 .59633 L
.39176 .59162 L
.402 .577 L
.41148 .55893 L
.4204 .53863 L
.459 .43025 L
.50005 .30886 L
.53959 .19199 L
.57762 .08429 L
.58766 .06093 L
.59717 .04246 L
.60242 .03406 L
.60812 .02654 L
.6133 .02126 L
.6158 .01925 L
.61809 .01773 L
.62036 .01653 L
.62274 .0156 L
.62398 .01524 L
.62513 .01499 L
.6264 .01481 L
.6271 .01475 L
.62775 .01472 L
.62891 .01472 L
.62998 .0148 L
.63116 .01496 L
.6324 .01521 L
.63349 .01551 L
.6345 .01584 L
.6368 .01682 L
.63947 .01833 L
.642 .02011 L
.64674 .02437 L
.65741 .03792 L
.66705 .05419 L
.6773 .07468 L
.69571 .11629 L
.73343 .19994 L
.75275 .23396 L
.7627 .24838 L
.7736 .26175 L
.79362 .28034 L
.80454 .28767 L
.81471 .29303 L
.82521 .29732 L
.83514 .30042 L
.84405 .30257 L
.85372 .30435 L
.86317 .30565 L
Mistroke
.87357 .3067 L
.87881 .30711 L
.88443 .30747 L
.88971 .30775 L
.89461 .30797 L
.90472 .30832 L
.91006 .30846 L
.91576 .30858 L
.92119 .30867 L
.9263 .30874 L
.93585 .30884 L
.94051 .30887 L
.94562 .3089 L
.95594 .30895 L
.9655 .30899 L
.97442 .30901 L
.97619 .30902 L
Mfstroke
.02381 .30902 m
.03279 .30912 L
.04262 .30925 L
.05288 .30943 L
.06244 .30966 L
.06792 .30983 L
.0731 .31003 L
.0828 .31051 L
.08823 .31086 L
.09324 .31125 L
.10458 .31242 L
.11425 .31382 L
.12331 .31557 L
.13324 .31811 L
.14393 .32177 L
.15404 .32631 L
.16506 .33275 L
.1756 .34061 L
.18515 .3494 L
.20483 .3732 L
.2258 .4075 L
.26587 .4908 L
.28424 .52667 L
.28928 .53483 L
.29468 .54242 L
.29934 .54785 L
.30442 .55239 L
.3072 .55421 L
.30842 .55486 L
.30977 .55546 L
.31096 .55588 L
.31223 .55622 L
.31294 .55636 L
.31358 .55646 L
.31485 .55657 L
.31599 .55656 L
.31706 .55647 L
.31823 .55626 L
.31948 .55593 L
.32079 .55545 L
.32203 .55487 L
.32436 .55346 L
.32691 .5514 L
.32962 .54863 L
.33446 .54211 L
.33968 .53287 L
.34542 .52002 L
.35558 .4906 L
.36487 .45689 L
.38576 .36321 L
Mistroke
.40639 .26062 L
.42552 .17231 L
.44405 .10337 L
.4544 .07377 L
.46377 .05266 L
.46866 .04374 L
.47382 .03583 L
.47828 .03021 L
.48305 .02541 L
.48574 .02324 L
.48819 .02159 L
.49069 .02024 L
.49299 .01928 L
.49429 .01886 L
.49567 .01851 L
.49634 .01837 L
.49706 .01825 L
.49785 .01815 L
.49859 .01808 L
.49923 .01804 L
.49994 .01803 L
.50067 .01804 L
.50135 .01808 L
.50258 .0182 L
.50389 .01841 L
.50512 .0187 L
.50648 .0191 L
.50923 .0202 L
.51178 .02157 L
.51415 .02315 L
.51952 .02783 L
.52527 .03458 L
.53581 .05182 L
.54544 .07333 L
.5645 .13272 L
.58525 .22043 L
.60461 .31542 L
.626 .4184 L
.63573 .45924 L
.64623 .49645 L
.65614 .52379 L
.6609 .53403 L
.66523 .54165 L
.66996 .54815 L
.67256 .55092 L
.67502 .55302 L
.67729 .55452 L
.67939 .55554 L
.68049 .55593 L
.68169 .55626 L
Mistroke
.68295 .55648 L
.68413 .55658 L
.68541 .55657 L
.68613 .55651 L
.6868 .55642 L
.68803 .55617 L
.68934 .55579 L
.69063 .5553 L
.69184 .55474 L
.6941 .55343 L
.69925 .54926 L
.70482 .54307 L
.71434 .52907 L
.72478 .51 L
.74585 .46555 L
.76701 .42123 L
.78629 .38661 L
.80629 .35877 L
.82522 .33993 L
.83488 .33278 L
.84514 .32673 L
.8539 .32264 L
.86357 .31909 L
.87367 .31627 L
.88293 .31431 L
.8931 .31272 L
.90379 .31152 L
.91335 .31076 L
.9186 .31043 L
.92355 .31018 L
.93318 .30979 L
.94192 .30954 L
.95059 .30936 L
.95976 .30921 L
.97619 .30902 L
Mfstroke
.02381 .30902 m
.03279 .30929 L
.04262 .30964 L
.05288 .31011 L
.06244 .31071 L
.0731 .31163 L
.0828 .31282 L
.094 .31474 L
.10458 .3173 L
.11428 .32045 L
.12455 .32488 L
.13333 .32973 L
.14299 .33645 L
.1624 .35523 L
.17331 .3694 L
.18328 .38486 L
.22299 .46812 L
.24302 .51592 L
.25175 .53497 L
.26118 .55271 L
.26646 .56082 L
.27131 .56686 L
.27595 .57116 L
.27829 .57273 L
.27952 .57339 L
.28086 .57397 L
.28159 .57422 L
.28226 .57442 L
.2829 .57457 L
.28359 .57469 L
.2848 .57481 L
.28609 .57479 L
.28682 .57472 L
.28748 .5746 L
.28822 .57444 L
.28899 .57421 L
.29028 .57369 L
.2917 .57295 L
.29424 .57114 L
.29695 .5685 L
.30183 .56187 L
.30716 .55181 L
.31207 .53988 L
.32317 .50368 L
.33296 .46188 L
.34333 .40926 L
.38121 .19286 L
.39119 .14511 L
.40066 .10877 L
.40588 .09316 L
Mistroke
.40864 .08629 L
.41158 .08004 L
.41429 .07526 L
.41675 .07176 L
.41923 .06905 L
.42155 .06723 L
.42272 .06658 L
.42397 .06609 L
.42515 .06581 L
.42622 .06571 L
.42748 .06578 L
.4282 .06591 L
.42885 .06609 L
.43006 .06656 L
.43134 .06725 L
.43256 .0681 L
.43388 .06922 L
.43626 .07177 L
.44167 .08003 L
.44637 .08979 L
.45132 .10246 L
.46016 .13049 L
.49832 .30069 L
.51961 .40385 L
.53893 .48431 L
.54859 .51535 L
.55368 .52838 L
.55905 .53932 L
.56376 .5463 L
.56505 .54776 L
.56643 .54911 L
.56763 .5501 L
.56893 .55097 L
.5702 .55161 L
.57137 .55203 L
.57249 .55227 L
.57369 .55235 L
.57475 .55226 L
.57575 .55205 L
.57685 .55166 L
.57803 .55108 L
.57928 .55026 L
.58042 .54933 L
.58298 .54663 L
.58534 .54336 L
.58758 .53959 L
.5926 .52879 L
.59804 .51346 L
.60901 .47206 L
.61914 .42338 L
Mistroke
.63947 .30771 L
.65851 .19895 L
.67675 .11464 L
.682 .09591 L
.68699 .08071 L
.69182 .06849 L
.69636 .05924 L
.69878 .05519 L
.70139 .0515 L
.70357 .04896 L
.70596 .04672 L
.70855 .04493 L
.70996 .04422 L
.71128 .04373 L
.71252 .04342 L
.71365 .04325 L
.71495 .04321 L
.71618 .0433 L
.71724 .04349 L
.7184 .0438 L
.71962 .04424 L
.72076 .04477 L
.7228 .04597 L
.72499 .04761 L
.72999 .05264 L
.73457 .05867 L
.74439 .07535 L
.75351 .0943 L
.77422 .14318 L
.79297 .18688 L
.81328 .22729 L
.8244 .2453 L
.83464 .25924 L
.84506 .27094 L
.8548 .27982 L
.86462 .287 L
.87358 .29221 L
.88353 .29674 L
.89422 .3004 L
.9041 .3029 L
.91326 .30462 L
.92331 .30601 L
.934 .30706 L
.94433 .30777 L
.95547 .30832 L
.96599 .3087 L
.97566 .309 L
.97619 .30902 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3ooona
0?ooo`00e`3oool300000=H0oooo003G0?ooo`030000003oool0oooo0=H0oooo003G0?ooo`030000
003oool0oooo0=H0oooo003G0?ooo`030000003oool0oooo0=H0oooo003G0?ooo`030000003oool0
oooo0=H0oooo002j0?ooo`050?oOW018000000000000M02oool00`3oool00`3o_g@0000007Boo`03
0?ooo`050?oOW018000000000000001d_ol02P3oool00`000000oooo0?ooo`3F0?ooo`00^@3oool0
1`3ookl0M00007Boo`3oool0okmd0000M02oool00P3oool00`3o_g@0000007Boo`020?ooo`070?oO
W0180000B9cO0?ooo`3ookl0M00007Boo`090?ooo`030000003oool0oooo03<0oooo0P00002Q0?oo
o`00^@3oool00`2oM00004RL0=ooo`020?ooo`030;md0000M;l0oooo00D0oooo00<0oooO09a80000
M;l00`3oool00`3o_g@004RL0=ooo`080?ooo`030000003oool0oooo0300oooo0`0000020?ooo`<0
0000WP3oool00;P0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`050?ooo`030?oO
W0180000M;oo00<0oooo00<0okmd0000B02Lgol01@3oool7000002d0oooo0P0000080?ooo`030000
003oool0oooo09/0oooo002/0?ooo`030?oog`2LB000000000L0000000D0B9cO0?ooo`3ogi`0B000
07Boo`030?ooo`030?noM00004P0W=oo00D0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@00018
09cOo`030?ooo`8000000`3oool010000000oooo0?ooo`3oool2000002X0oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`2J0?ooo`00^03oool00`3ogi`0B00007Boo`030?ooo`030?no
M00004P0W=oo00D0oooo00<0omnL04P0001d_ol00`3oool00`2oM000001809cOo`020?ooo`030000
003oool0oooo00<0oooo100000020?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0
2`3oool00`000000oooo0?ooo`2I0?ooo`00^03oool00`3ogi`0B00007Boo`030?ooo`030?noM000
04P0W=oo00D0oooo00P0oono07@000000000B9cO0?ooo`3o_g@00000001d_`80oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02@0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`2H0?ooo`00^03oool00`3ogi`0B00007Boo`030?ooo`03
0?noM00004P0W=oo00H0oooo00<0_g@00018W02oW4P00P0000001000B9`0gooo0?ooo`0000070?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0?0?oo
o`030000003oool0oooo09L0oooo002h0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02o
ool01P3oool00`3ogi`0B00007Boo`040?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool0UP3oool00;T0oooo00<0gia80018W03Oool00P3oool00`2oM00007Bo0?ooo`070?ooo`030=nL
B00004P0W=oo0080oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo01l0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`2E0?ooo`00^@3o
ool01`3ookl0M00007Boo`3oool0okmd0000M02oool02@3oool0103o_g@00000001d_`00000:0?oo
o`030000003oool0oooo00P0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo09@0oooo002j0?ooo`050?oo_`1d000000000000M02oool02`3oool01@3o
oml000000000000007@0_ooo00P0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0L
0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0T`3oool00</0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`2B0?ooo`00bP3oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo0980oooo003:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
4@3oool6000007X0oooo00390?ooo`030000003oool0oooo00/0oooo0`00000;0?ooo`030000003o
ool0oooo01L0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo00@0oooo0P00001h0?ooo`00b@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`1e0?ooo`00b03oool00`00
0000oooo0?ooo`0<0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool05@3oool00`00
0000oooo0?ooo`0L0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0203oool00`00
0000oooo0?ooo`1e0?ooo`00b03oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`1d0?ooo`00b03oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool04`3oool00`000000oooo
0?ooo`0N0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`1c0?ooo`00a`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool04P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool03@3oool00`000000oooo0?ooo`1b0?ooo`00a`3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0P
0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`1a
0?ooo`00aP3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`1a0?ooo`00]P3oool4000000`0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3o
ool08P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool0L03oool00;H0oooo00D000000?ooo`3oool0oooo0000000:0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo02<0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0700
oooo002e0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`0?0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`0T0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`1_0?ooo`00]03oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool0403oool300000100oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`0U0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`1_0?ooo`00/`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool09@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool0KP3oool00;80oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo02H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0E0?oo
o`030000003oool0oooo06h0oooo002a0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool02`3oool00`000000oooo0?ooo`0W0?ooo`050000003oool0oooo0?ooo`000000603oool00`00
0000oooo0?ooo`1]0?ooo`00/@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool0:03oool01@000000oooo0?ooo`3oool0000001P0oooo00<000000?oo
o`3oool0K@3oool00;00oooo00<000000?ooo`3oool0303oool010000000oooo0?ooo`00000D0?oo
o`030000003oool0oooo0180oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0Y0?oo
o`030000003oool0000001X0oooo00<000000?ooo`3oool0K03oool00;00oooo00<000000?ooo`3o
ool0303oool010000000oooo0?ooo`00000D0?ooo`030000003oool0oooo0180oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0000001X0oooo00<000000?oo
o`3oool0K03oool00;00oooo00<000000?ooo`3oool03@3oool2000001D0oooo00<000000?ooo`3o
ool04P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`1[0?ooo`00[`3oool00`000000oooo0?ooo`0>0?ooo`800000
5@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
:`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo06/0oooo002_0?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0000000L0?ooo`030000003o
ool0oooo06X0oooo002_0?ooo`030000003oool0oooo00h0oooo0P00000E0?ooo`<000004`3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo02/0oooo00<000000?ooo`000000703oool0
0`000000oooo0?ooo`1Z0?ooo`00[P3oool00`000000oooo0?ooo`0?0?ooo`8000005@3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0:`3oool00`00
0000oooo0000000M0?ooo`030000003oool0oooo06T0oooo002^0?ooo`030000003oool0oooo00h0
oooo00@000000?ooo`3oool00000503oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool0:P3oool01@000000oooo0?ooo`3oool0000001`0oooo00<00000
0?ooo`3oool0J@3oool00:h0oooo00<000000?ooo`3oool03P3oool010000000oooo0?ooo`00000D
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0[
0?ooo`050000003oool0oooo0?ooo`0000007@3oool00`000000oooo0?ooo`1X0?ooo`00[@3oool0
0`000000oooo0?ooo`0?0?ooo`040000003oool0oooo000001@0oooo00<000000?ooo`3oool0503o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo06P0oooo002]0?ooo`030000003oool0
oooo00X0oooo00D0omnL04P000000000001d000000030?ooo`030?noM0000000M;oo00D0oooo00@0
oooO09a80000B9`0gooo2@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01/0oooo
00<000000?ooo`3oool0I`3oool00:d0oooo00<000000?ooo`3oool02@3oool01`3ookl0M00007Bo
o`3oool0okmd0000002oool00P3oool00`3o_g@0000007Boo`050?ooo`040?oog`2LB00004RL0=oo
o`T0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
02/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
06L0oooo002/0?ooo`030000003oool0oooo00X0oooo00<0_g@00018W03Oool00P3oool00`000000
07Bo0?ooo`020?ooo`030000003oool0oooo00@0oooo00@0oooO09a80000B9`0gooo2@3oool00`00
0000oooo0?ooo`0E0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0:`3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0I`3oool00:`0
oooo00<000000?ooo`3oool02@3oool00`3ookl0M00004RLg`030?ooo`030000000007@0_ooo00<0
oooo00<000000?ooo`2oM0001`0000000`1d_ol0oooo0?ooo`060?ooo`030000003oool0oooo01H0
oooo00@000000?ooo`3oool00000;P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool0IP3oool00:`0oooo00<0000009a8000000001`0000001@18W=l0
oooo0?oOW0180000M;oo0080oooo00@000000?noM00004P0W=oo0`3oool010000000oooo0?oOW01d
W=l20?ooo`040?oog`2LB00004RL0=ooo`T0oooo00<000000?ooo`3oool05P3oool010000000oooo
0?ooo`00000^0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`1V0?ooo`00[03oool00`000000oooo0?ooo`090?ooo`030?oOW0180000M;oo0080oooo
00@000000?noM00004P0W=oo0`3oool02P000000oooo0?ooo`3OW4P0M;oo0?ooo`3ooml0W4P00018
W03Oool90?ooo`@000005@3oool010000000oooo0?ooo`00000^0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`1U0?ooo`00Z`3oool00`000000oooo
0?ooo`0:0?ooo`030?oOW0180000M;oo0080oooo00@000000?noM00004P0W=oo103oool02@000000
oooo0?oog`2LB7@0_ooo0?oog`2LB00004RL0=ooo`090?ooo`030000003oool0oooo01H0oooo00<0
00000?ooo`000000;P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool0I@3oool00:/0oooo00<000000?ooo`3oool02P3oool00`3ogi`0B00007Boo`02
0?ooo`040000003o_g@0001809cOo`@0oooo00T000000?ooo`3oool0omnL04Qd_`3ooml0W4P00018
W03Oool02@3oool00`000000oooo0?ooo`0G0?ooo`800000;P3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo01/0oooo00<000000?ooo`3oool0I03oool00:/0oooo00<000000?ooo`3o
ool02P3oool02@3ookl0M00004RLg`3oool000000?ooo`3OW4P0001d0;ooo`040?ooo`090000003o
ool0oooo0?ooo`3OW4P0M;oO09a80000B9`0gooo00T0oooo00<000000?ooo`3oool05`3oool20000
02h0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
06@0oooo002[0?ooo`030000003oool0oooo00/0oooo00L0gia80018W03Oool000000?ooo`2oM000
07Bo00H0oooo00P000000?ooo`3oool0oooO09a8B018B00004RL0=ooo`T0oooo00<000000?ooo`3o
ool05`3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`1T0?ooo`00ZP3oool00`000000oooo0?ooo`0<0?ooo`070?oo
_`1d0000M;oo0000003o_g@0001d0;ooo`060?ooo`080000003oool0oooo0?ooo`3ogi`0B0000018
W03Oool90?ooo`030000003oool0oooo01L0oooo0P00000]0?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`1S0?ooo`00ZP3oool00`000000oooo0?oo
o`0=0?ooo`050?oo_`00000000000000M02oool01`3oool00`000000oooo0?ooo`020?ooo`030=nL
B000B9`0gooo00T0oooo00<000000?ooo`3oool05`3oool2000002d0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo06<0oooo002Z0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0>0?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`000000;@3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool0HP3oool00:X0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0000000]0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`1R0?ooo`00Z@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0F0?ooo`040000003oool0oooo
000002`0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo0640oooo002Y0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0=0?ooo`<000005@3oool01@000000oooo0?ooo`3oool0000002/0oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0640oooo002Y0?oo
o`030000003oool0oooo00h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo01D0oooo00D000000?ooo`3oool0oooo0000000[0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`1Q0?ooo`00Z@3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`0E0?ooo`050000003oool0oooo0?ooo`000000:`3oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0H03oool00:P0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02P0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0600oooo002X0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo01@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0W0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`1O0?ooo`00
Z03oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
303oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
9`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
G`3oool00:P0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo02L0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo05h0oooo002W0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`0V0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`1N0?ooo`00Y`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool0GP3oool00:L0oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool04P3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool04`3oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo05d0oooo002W0?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0;0?ooo`<000004P3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo05d0oooo002V0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool0203oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo01@0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`1L0?ooo`00YP3oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0E
0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0G03oool00:H0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00X0oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo02<0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo05`0oooo002V0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0S0?ooo`03
0000003oool0oooo01H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`1K0?ooo`00
Y@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
8P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0
F`3oool00:D0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo0280oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo05X0oooo002U0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`0R0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`1J0?ooo`00Y@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool0F@3oool00:@0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`0B0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo05T0oooo002T0?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool0503oool00`000000oooo0?ooo`080?ooo`<000003P3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo05T0oooo002T0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool0503oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool03@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool06@3oool00`000000oooo0?ooo`1H0?ooo`00Y03oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0L
0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0F03oool00:@0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00P0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo01h0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo05L0oooo002S0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`080?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0N0?ooo`03
0000003oool0oooo01d0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`1G0?ooo`00
X`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00<0oooo00D0omnL04P000000000
001d0;ooo`030?ooo`050?noM0000000M;oo0?ooo`2oM0001`0000090?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo
01h0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`1F0?ooo`00X`3oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo0080oooo00L0oono07@0001d_ol0oooo0?noM00007@0
_ooo0080oooo00<0okmd0000001d_ol00P3oool00`3OW4P0000000000004000000030=ooo`3oool0
oooo00H0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo01h0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0
oooo05H0oooo002S0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool00P3oool00`2o
M00004RL0=ooo`020?ooo`030;md0000M;l0oooo00L0oooo00<0gia804RLg`3oool00P3oool00`00
0000W=oo0?ooo`070?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0L0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`1E0?ooo`00X`3oool00`000000oooo0?ooo`0>0?ooo`070000003oool0oooo0?oo
o`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00P0oooo00D0gia807Boo`3oool0oooo0000
00090?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?oo
o`0L0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool06@3oool00`000000oooo0?oo
o`1E0?ooo`00X`3oool00`000000oooo0?ooo`060?ooo`030?oog`2LB000000000L0000000D0B9cO
0?ooo`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00T0oooo00D0_gA809cOo`3oool0oooo
000000080?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`0L0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`1D0?ooo`00XP3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0080oooo00<0
omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`090?ooo`050?oog`2LB7@0_ooo0?ooo`000000
203oool4000000X0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0L0?ooo`030000
003oool0oooo0240oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`1D0?ooo`00XP3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0080oooo00<0omnL04P0001d_ol00`3o
ool00`3o_g@0001809cOo`0:0?ooo`040?noM000M;l0oooo000000P0oooo00<000000?ooo`3oool0
2`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
8P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo05<0oooo002R0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool00P3oool00`3ogi`0B00007Boo`030?ooo`030?noM000
04P0W=oo00X0oooo00@0oono07@0B02Lgol00000203oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0R0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool0D`3oool00:80oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`020?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool02P3o
ool0103ookl0M00004RLg`0000080?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`1B0?ooo`00XP3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo00@0oooo00<0gia80018W03Oool00P3oool00`2oM00007Bo0?ooo`050?ooo`030?noW02L
gol0oooo0080oooo00D0omnL04P00018W=l0oooo000000070?ooo`030000003oool0oooo00T0oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo02@0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`1B0?ooo`00XP3oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo00@0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00H0
oooo00X0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`3oool000001`3oool00`000000oooo
0?ooo`090?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`0U0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0D@3oool00:40oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`050?ooo`050?oo_`1d000000000000M02oool0
203oool01P3ooml0W4P0000000000000001d0;ooo`80oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`0U0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0D@3oool00:40
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0500
oooo002Q0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool06`3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool05`3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`1@0?ooo`00X@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01`0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`3oool0C`3oool00:40oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0L
0?ooo`030000003oool0oooo00@0oooo0`0000070?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool0603oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool0CP3oool00:40oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0L0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo01L0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo04d0oooo002P0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool0703oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`1=0?ooo`00X03oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0[0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool0C03oool00:00oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0;03o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo04/0oooo002P0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
oooo02h0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`1;0?ooo`00X03oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool05@3oool00`00
0000oooo0?ooo`0_0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0BP3oool009l0
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01D0
oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo04T0
oooo002O0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0803oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`190?ooo`00>`3oool0103ooml0W4P00018W03Ooom90?ooo`030;md00000000000000@00000
00<0B9cO0?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0200oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01d0oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0;0?ooo`030;md00000000000000@00000
00<0B9cO0?ooo`3oool0703oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo0140oooo
00@0oooO09a80000B9`0gooo<`3oool003/0oooo00@0oooO09a80000B9`0goooBP3oool00`3OW4P0
000000000003000000030018W03Oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000
oooo0?ooo`0P0?ooo`030000003oool0oooo0080oooo0`0000040?ooo`030000003oool0oooo01d0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0=0?ooo`030=nLB0000000000000<0
000000<004RL0=ooo`3oool07@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo0100
oooo00@0oooO09a80000B9`0gooo<`3oool003/0oooo00@0oooO09a80000B9`0goooB`3oool00`3O
W4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo00d0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0N0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool03P3oool00`3O
W4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo01h0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0?0?ooo`040?oog`2LB00004RL0=oooc<0oooo000g0?ooo`030;md000000000000
00D0000000<0M;oo0?ooo`3oool0B@3oool00`3OW4P0M;oo0?ooo`0A0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool08@3oool01@000000oooo0?ooo`3oool0000000D0oooo00<00000
0?ooo`3oool07`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00l0oooo00<0gia8
07Boo`3oool08`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00X0oooo00<0_g@0
0000000000001@0000000`1d_ol0oooo0?ooo`0`0?ooo`00:P3oool00`3ooml0W4P0000000070000
000504RLg`3oool0oooo0?oOW01dW=l00P3oool0103ooml0W4P00018W03Oooll0?ooo`030?oog`2L
B000000000L0000000<0B9cO0?ooo`3oool0103oool00`2oM4P0W=oo0?ooo`0?0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool08P3oool01@000000oooo0?ooo`3oool0000000D0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo
00<0_gA809cOo`3oool08`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00T0oooo
00P0omnL07BLg`3oool0oooo0?oog`2LB00004RL0=oooc<0oooo000h0?ooo`070=nLB01d_ol0oooo
0?oog`2LB00004RL0=ooo`1=0?ooo`030?oog`2LB7@0_ooo00l0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0R0?ooo`050000003oool0oooo0?ooo`0000001@3oool00`000000oooo
0?ooo`0O0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0403oool00`3ooml0W4Qd
0;ooo`0T0?ooo`8000006@3oool00`000000oooo0?ooo`090?ooo`070=nLB01d_ol0oooo0?oog`2L
B00004RL0=ooo`0c0?ooo`00>03oool01`3ooml0W4Qd0;ooo`3ooml0W4P00018W03Oool0CP3oool0
0`3o_g@007Bo0?ooo`0>0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool08P3oool0
1@000000oooo0?ooo`3oool0000000@0oooo00<000000?ooo`3oool0803oool00`000000oooo0?oo
o`0C0?ooo`030000003oool0oooo0140oooo00<0okmd001d_`3oool09@3oool00`000000oooo0?oo
o`0G0?ooo`030000003oool0oooo00P0oooo00L0oooO09a8M02oool0oooO09a80000B9`0gooo03<0
oooo000i0?ooo`060?oOW018M;l0oooO09a80000B9`0goooCP3oool00`3ookl0M01809cOo`0>0?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool08`3oool010000000oooo0?ooo`000004
0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0B
0?ooo`030?oo_`1d04P0W=oo02H0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`08
0?ooo`060?oOW018M;l0oooO09a80000B9`0gooo<`3oool003X0oooo00D0gia807Bog`2LB00004RL
0=ooo`1>0?ooo`030?oo_`1d0000B9cO00h0oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`0S0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool08@3oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo0180oooo00<0oono07@00018W=l09`3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo00P0oooo00D0gia807Bog`2LB00004RL0=ooo`0c0?oo
o`00>P3oool01@3ooml0W4Q804Q80000B9`0gooo04T0oooo00<0oknL09cOo`3oool00P3oool00`3o
gi`0B00004RLg`0>0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0903oool01000
0000oooo0?ooo`0000030?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`0=0?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l0:03oool0
0`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00L0oooo00D0oooO09a8B018B00004RL0=oo
o`0c0?ooo`00>`3oool0103ogi`0B0000018W03Ooom90?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0
M0000000M02oool=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0903oool01000
0000oooo0?ooo`0000030?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`0=0?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02ooolY0?ooo`80
0000603oool00`000000oooo0?ooo`070?ooo`040?oOW018000004RL0=oooc<0oooo000l0?ooo`03
0=nLB000B9`0gooo04X0oooo00H0oooO09a80000000000000000M02oool>0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`04000002D0oooo00<00000
0?ooo`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`060?oog`2LB00000000000000007@0_ooo
;03oool2000001L0oooo0P0000080?ooo`030=nLB000B9`0gooo03<0oooo002M0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool09@3oool00`000000oooo000000020?ooo`030000003o
ool0oooo02@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`120?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`3oool0>`3oool009d0oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`0U0?ooo`030000003oool000000080oooo00<000000?ooo`3oool0903oool0
0`000000oooo0?ooo`0?0?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool05P3oool2
000003/0oooo002M0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool09P3oool01@00
0000oooo0000003oool0000002L0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`15
0?ooo`800000603oool2000003T0oooo002M0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool09P3oool01@000000oooo0000003oool0000002L0oooo00<000000?ooo`3oool03`3oool0
0`000000oooo0?ooo`170?ooo`030000003oool0oooo01L0oooo0P00000g0?ooo`00W03oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo02L0oooo0P0000000`3oool000000?ooo`0V0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool0B03oool2000001T0oooo0`00000d0?oo
o`00W03oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo02L0oooo0`00000Y0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool0BP3oool4000001P0oooo0P00000b0?ooo`00
W03oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo02L0oooo0`00000Y0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool0CP3oool4000001H0oooo0`00000_0?ooo`00W03o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02P0oooo0`00000Y0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool0D`3oool3000001H0oooo1000000[0?ooo`00W03oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02P0oooo0P00000Z0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool0EP3oool900000140oooo2@00000R0?ooo`00V`3oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0:03oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo05l0oooo:`00000A0?ooo`000P3ooooo00000:`0
00000`3oool000`0oooo00<000000?ooo`3oool0403oool9000000030?ooo`000000000000H00000
103oool010000000oooo0?ooo`3oool=000000<0oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0B0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04@3oool2000000h0
oooo00D000000?ooo`3oool0oooo0000000D0?ooo`030000003oool0oooo0100oooo0P00000C0?oo
o`030000003oool0oooo0180oooo00@000000?ooo`3oool000003P3oool00`000000oooo0000000C
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0A
0?ooo`040000003oool0oooo0?ooo`P00000203oool010000000oooo0?ooo`3oool9000000L0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0;0?ooo`00:03oool5000000@0oooo2000
00090?ooo`T00000>`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool0:@3oool2000002/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0C0?ooo`030000003oool0oooo03X0oooo1000000@0?ooo`@00000<`3oool002d0oooo10000008
0?ooo`@000002@3oool4000000030?ooo`000000000000800000C03oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo02T0oooo0P00000[0?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool0C@3oool400000100oooo1000000g0?ooo`00<@3oool3000000T0oooo0P00000;0?oo
o`8000000`3oool4000004P0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0X0?oo
o`040000003oool00000000002X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`19
0?ooo`@00000403oool4000003/0oooo000d0?ooo`8000002@3oool2000000/0oooo0`0000040?oo
o`@00000@`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo02P0oooo00@000000?oo
o`0000000000:`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo04D0oooo1000000A
0?ooo`<00000?`3oool003H0oooo0P0000090?ooo`800000303oool3000000D0oooo0P0000110?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0:@3oool010000000oooo00000000000[
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0@`3oool2000001<0oooo0P000012
0?ooo`00>03oool2000000T0oooo0P00000=0?ooo`030000003oool0oooo00@0oooo0P00000o0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`020000
02/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`100?ooo`<000004`3oool20000
04@0oooo000j0?ooo`8000002@3oool2000000`0oooo0P0000060?ooo`800000?@3oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool00P00000[0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool0?P3oool2000001@0oooo0P0000160?ooo`00
?03oool00`000000oooo0?ooo`080?ooo`800000303oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo03X0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0X0?ooo`060000
003oool0oooo0000003oool00000:`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
03d0oooo0P00000D0?ooo`800000B03oool003d0oooo0P00000:0?ooo`030000003oool0oooo00X0
oooo0P0000060?ooo`800000>P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02L0
oooo00@000000?ooo`3oool0oooo0`00000[0?ooo`030000003oool0oooo00P0oooo00<000000?oo
o`3oool0?03oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo04P0oooo000o0?ooo`03
0000003oool0oooo00P0oooo00<000000?ooo`3oool02`3oool2000000H0oooo0P00000g0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0:03oool01`000000oooo0?ooo`3oool00000
0?ooo`000000:`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo03X0oooo0P00000E
0?ooo`030000003oool0oooo04T0oooo00100?ooo`030000003oool0oooo00P0oooo0P00000=0?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo02P0oooo00L000000?ooo`3oool0oooo0000003oool0000002/0oooo00<0
00000?ooo`3oool0203oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo01<0oooo0P00
001<0?ooo`00@@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool09`3oool00`000000oooo0?ooo`020?ooo`030000003oool0000002/0oooo00<00000
0?ooo`3oool0203oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo01<0oooo00<00000
0?ooo`3oool0C03oool00480oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo00D0oooo0P00000c0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool09`3oool00`000000oooo0?ooo`020?ooo`040000003oool0oooo000002/0oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo01<0oooo00<00000
0?ooo`3oool0C@3oool004<0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo02L0oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`00000[
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0C
0?ooo`030000003oool0oooo04h0oooo00140?ooo`030000003oool0oooo00T0oooo00<000000?oo
o`3oool02`3oool2000000L0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo02L0oooo00<000000?ooo`3oool00`3oool010000000oooo0?ooo`00000[
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0C
0?ooo`030000003oool0oooo04l0oooo00150?ooo`030000003oool0oooo00T0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo02d0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo00<0oooo00@000000?oo
o`3oool00000:`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo03D0oooo00<00000
0?ooo`3oool04`3oool00`000000oooo0?ooo`1@0?ooo`00AP3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0/0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`030?oo
o`040000003oool0oooo000002`0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0d
0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0D@3oool004L0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool0:`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02H0oooo00<000000?oo
o`3oool0103oool01@000000oooo0?ooo`3oool0000002/0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`0c0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0DP3oool0
04P0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
02H0oooo00<000000?ooo`3oool0103oool3000000030?ooo`000000oooo02X0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo01<0oooo00<000000?oo
o`3oool0D`3oool004T0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo02L0oooo00<000000?ooo`3oool0103oool01@000000oooo0?ooo`3oool00000
02/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo
01<0oooo00<000000?ooo`3oool0E03oool004T0oooo00<000000?ooo`3oool02@3oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0:03oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool01@3oool01@000000
oooo0?ooo`3oool0000002`0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0a0?oo
o`030000003oool0oooo01<0oooo00<000000?ooo`3oool0E@3oool004X0oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool09`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool0E@3oool004/0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo02H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo02T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0_0?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool0EP3oool004`0oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool09@3o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0E`3o
ool004d0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo02H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo02T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0
oooo01@0oooo00<000000?ooo`3oool0E`3oool004d0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`0<0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0903oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo02X0oooo00D000000?ooo`3oool0oooo0000000`
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0F03oool004h0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool08P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02H0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02T0oooo00D000000?oo
o`3oool0oooo0000000_0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0F@3oool0
04h0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
02D0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
02T0oooo00D000000?ooo`3oool0oooo0000000^0?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool0F@3oool004l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo02D0oooo00<000000?ooo`3oool01`3oool3000000<0oooo00<000000?oo
o`3oool0:@3oool01@000000oooo0?ooo`3oool0000002d0oooo00<000000?ooo`3oool05@3oool0
0`000000oooo0?ooo`1J0?ooo`00C`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00`0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0:P3oool010000000oooo0?ooo`00000]0?ooo`030000003oool0
oooo01@0oooo00<000000?ooo`3oool0F`3oool00500oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool07`3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0203oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo02X0oooo00@000000?ooo`3oool00000;03oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo05/0oooo001@0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
01l0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo
00P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0Y0?ooo`030000003oool00000
02d0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`1L0?ooo`00D@3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool09@3oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0:@3oool00`000000
oooo0000000/0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0G@3oool00540oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01<0oooo
00D0omnL04P000000000001d0;ooo`030?ooo`050?noM0000000M;oo0?ooo`2oM0001P0000000`18
W=l0oooo0?ooo`070?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0:@3oool00`00
0000oooo0000000[0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0G@3oool00580
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo01<0
oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo0080oooo00<0okmd0000001d_ol00P3oool0
0`3OW4P0000000000003000000030018W03Oool0oooo00L0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`0Z0?ooo`800000:`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo05h0oooo001B0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`0C0?ooo`030;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool01`3oool01`3O
W4P0B9cO0?ooo`000000oooo0=nLM02Lgol0203oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo02X0oooo0P00000Z0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0GP3o
ool005<0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo0180oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`080?ooo`030=nLB0000000
oooo00/0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0Y0?ooo`800000:@3oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo05l0oooo001C0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0B0?ooo`030?oOW0180000M;oo
00<0oooo00<0okmd0000B02Lgol02@3oool00`000000W=oo0?ooo`0:0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool0G`3oool005@0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0=0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool06@3oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo0180oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0
001809cOo`090?ooo`030000002LB7@0_ooo00X0oooo100000040?ooo`030000003oool0oooo02T0
oooo0P00000X0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0H03oool005@0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo01<0oooo
00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`080?ooo`040000003oool0okmd001d_`X0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0Y0?ooo`800000:03oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo0640oooo001E0?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01P0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0C0?ooo`030?oOW0180000M;oo00<0oooo
00<0okmd0000B02Lgol0203oool01@000000oooo0?oo_`1d04P0W=oo00T0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`0X0?ooo`8000009`3oool00`000000oooo0?ooo`0E0?ooo`03
0000003oool0oooo0640oooo001E0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0
303oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
2`3oool00`000000oooo0?ooo`0C0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool0
203oool01@000000oooo0?oo_`1d0000B9cO00T0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`0X0?ooo`8000009P3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0680
oooo001F0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`0D0?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d_`3oool01@3oool00`3o_i`0W=oo
000000020?ooo`030?oOW0180000B9cO00T0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`0X0?ooo`030000003oool0000002D0oooo00<000000?ooo`3oool05@3oool00`000000oooo
0?ooo`1R0?ooo`00EP3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool05@3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool01P3oool0203ooml0
W4Q80000003oool0oono07@0000007@0_ooo2@3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo02L0oooo00@000000?ooo`3oool00000903oool00`000000oooo0?ooo`0E0?ooo`030000
003oool0oooo06<0oooo001G0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool02`3o
ool00`000000oooo0?ooo`0F0?ooo`050?oo_`1d000000000000M02oool0203oool01P3ooml00000
000000000000001d0;ooo`X0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0V0?oo
o`040000003oool0oooo000002@0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`1S
0?ooo`00E`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool08`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool09P3oool010000000oooo0?ooo`00000S0?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool0I03oool005P0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo02<0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo02H0oooo00@000000?ooo`3oool000008`3oool00`000000oooo0?ooo`0E
0?ooo`030000003oool0oooo06@0oooo001H0?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01<0oooo00<000000?oo
o`3oool02`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`0V0?ooo`050000003oool0oooo0?ooo`0000008@3oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo06D0oooo001I0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo
00h0oooo0`0000080?ooo`030000003oool0oooo02D0oooo00D000000?ooo`3oool0oooo0000000P
0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0I@3oool005T0oooo00<000000?oo
o`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo02<0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo02@0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01D0oooo00<000000?oo
o`3oool0IP3oool005X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo02<0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`080?ooo`03
0000003oool0oooo02@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo01H0oooo00<000000?ooo`3oool0IP3oool005X0oooo00<000000?ooo`3oool0
2`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0
I`3oool005/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo0280oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo02<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0K0?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`3oool0I`3oool005/0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool03`3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0J03oool0
05`0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
0280oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
0280oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool0J03oool005`0oooo00<000000?ooo`3oool02`3oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`0;0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0J@3oool005d0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0240oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0240oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool0J@3oool005d0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool04@3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0G0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0JP3oool005h0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0280oooo00<00000
0?ooo`3oool04@3oool3000000X0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`1Z
0?ooo`00GP3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool07`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01D0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`1[0?ooo`00G`3oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo00/0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo01D0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`1[0?ooo`00
G`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0
7P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`1/0?ooo`00H03oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`1/0?ooo`00H03oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool07@3oool0
0`000000oooo0?ooo`080?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0?ooo`1]0?ooo`00H@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00`0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
00`0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`1]0?ooo`00H@3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0803oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0703oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`1]0?ooo`00H@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0140oooo
00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`1^0?ooo`00HP3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0803oool00`000000oooo0?oo
o`0E0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool05`3oool00`000000oooo0?oo
o`1^0?ooo`00HP3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool0203oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool0803oool00`000000oooo0?ooo`0E0?ooo`<000003@3oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0F
0?ooo`030000003oool0oooo06l0oooo001S0?ooo`030000003oool0oooo00X0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool02P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo01D0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo06l0oooo001S0?ooo`03
0000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`080?ooo`05
0000003oool0oooo0?ooo`000000303oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
00`0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo
0700oooo001T0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`070?ooo`050000003oool0oooo0?ooo`000000303oool00`000000oooo0?ooo`0P0?oo
o`030000003oool0oooo01H0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo0700oooo001T0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3o
ool02`3oool00`000000oooo0?ooo`080?ooo`040000003oool0oooo000000`0oooo00<000000?oo
o`3oool07`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`1a0?ooo`00I03oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool01`3oool010000000oooo0?ooo`00000<0?oo
o`030000003oool0oooo01`0oooo00D0omnL04P00000000000000;ooo`030?ooo`030?noM0000000
M;oo00D0oooo00@0oooO09a80000B9`0gooo2@3oool00`000000oooo0?ooo`0>0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo01L0oooo00<000000?ooo`3oool0L@3oool006D0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00P0oooo0P00000<0?ooo`030000003oool0
oooo01`0oooo00L0oono07@0001d_ol0oooo0000000007@0_ooo0080oooo00<0okmd0000001d_ol0
1@3oool0103ooml0W4P00018W03Oool90?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3o
ool05`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3o
ool05P3oool00`000000oooo0?ooo`1b0?ooo`00I@3oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo01`0oooo00L0_g@00018W03Oool000000?ooo`2oM00007Bo00X0oooo00@0oooO09a8
0000B9`0gooo2@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo01L0oooo00<00000
0?ooo`3oool03P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01L0oooo00<00000
0?ooo`3oool0LP3oool006H0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo00P0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0K0?oo
o`090?oo_`1d0000B9cO0?ooo`000000oooo0=nLB00007@0_ooo00D0oooo00<0_g@0000000000000
1@0000000`1d_ol0oooo0?ooo`060?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
5`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`1c0?ooo`00IP3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool0203oool2000000X0oooo00<000000?ooo`3oool0703o
ool02@3ogi`0B00007Boo`3oool000000?ooo`3o_g@0001809cOo`050?ooo`080?oOW01dW=l0oooo
0?ooo`3ooml0W4P00018W03Oool90?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
5`3oool00`000000oooo0?ooo`1c0?ooo`00I`3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool01P3oool00`000000oooo0000000:0?ooo`030000003o
ool0oooo01`0oooo00@0omnL04P0001d_ol000000P3oool00`3o_g@0001809cOo`060?ooo`070=nL
B01d_ol0oooo0?oog`2LB00004RL0=ooo`090?ooo`@000003`3oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo07@0oooo001W0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3o
ool02`3oool00`000000oooo0?ooo`060?ooo`040000003oool0oooo000000P0oooo00<000000?oo
o`3oool07@3oool0103ogi`0B00007Boo`0000020?ooo`030?noM00004P0W=oo00H0oooo00L0oooO
09a8M02oool0oooO09a80000B9`0gooo00T0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo07@0oooo001W0?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`050?ooo`040000003oool0oooo000000P0oooo
00<000000?ooo`3oool07@3oool0103ogi`0B00007Boo`0000020?ooo`030?noM00004P0W=oo00L0
oooo00H0omnL04Qd_`3ooml0W4P00018W03Oool90?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool05@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool05P3oool00`000000oooo0?ooo`1e0?ooo`00J03oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool01@3oool01@000000oooo0?ooo`3oool0
000000H0oooo00<000000?ooo`3oool07P3oool00`3ookl0M000000000030?ooo`030=nLB00007@0
_ooo00P0oooo00D0gia807Bog`2LB00004RL0=ooo`090?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool05`3oool00`000000oooo0?ooo`1e0?ooo`00J03oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool07`3oool00`3OW4P000000=ooo`02
0?ooo`030;md0000M;l0oooo00P0oooo00D0oooO09a8B018B00004RL0=ooo`090?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`1f0?ooo`00J@3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool07`3oool0
1`000000M00007Boo`3oool0okmd0000M02oool02P3oool0103ogi`0B0000018W03Oool90?ooo`03
0000003oool0oooo0140oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`1f0?ooo`00
J@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
803oool01P000000oono07@000000000001d0;ooo``0oooo00<0gia80018W03Oool01@3oool80000
0140oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`1g0?ooo`00JP3oool00`000000
oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00D000000?ooo`3oool0oooo000000070?oo
o`050000003oool0oooo0?ooo`0000008P3oool00`000000oooo0?ooo`0E0?ooo`800000103oool0
10000000oooo0?ooo`3oool2000000l0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`0D0?ooo`050000003oool0oooo0?ooo`0000006@3oool00`000000oooo0?ooo`1g0?ooo`00JP3o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00/0oooo00@000000?ooo`3oool00000
1`3oool010000000oooo0?ooo`00000R0?ooo`030000003oool0oooo01<0oooo0`0000060?ooo`03
0000003oool0oooo00<0oooo0P00000=0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool05@3oool01@000000oooo0?ooo`3oool0000001P0oooo00<000000?ooo`3oool0N03oool006X0
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0;0?ooo`040000003oool0oooo0000
00P0oooo00<000000?ooo`0000008P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool01@3oool3000000/0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0E0?ooo`040000003oool0oooo000001P0oooo00<000000?ooo`3oool0N@3oool0
06/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0;0?ooo`8000002@3oool00`00
0000oooo0000000R0?ooo`030000003oool0oooo0100oooo0P00000:0?ooo`<00000203oool20000
00T0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool00000
01P0oooo00<000000?ooo`3oool0N@3oool006/0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`0;0?ooo`8000002P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`8000001`3oool00`000000
oooo0?ooo`0?0?ooo`030000003oool0oooo01L0oooo0P00000H0?ooo`030000003oool0oooo07X0
oooo001[0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`080?ooo`8000008@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo07X0oooo
001/0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`080?ooo`800000803oool00`000000oooo0?ooo`0>0?ooo`8000003P3oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`0H0?ooo`8000005`3oool00`000000oooo0?ooo`1k0?ooo`00K03oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`000000203oool010000000oooo0?ooo`00
000O0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0>0?ooo`800000103oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`0000005P3oool00`000000oooo0?ooo`1l0?ooo`00K@3oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`000000203oool01@000000oooo0?ooo`3o
ool0000001h0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo01L0oooo00D000000?ooo`3oool0oooo0000000E0?ooo`030000003oool0oooo07`0oooo
001]0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0203oool010000000oooo0?oo
o`0000060?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool06`3oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04@3oool01@000000oooo
0?ooo`3oool0000000d0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo0140oooo00<000000?ooo`3oool0O@3oool006h0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`060?ooo`050000003oool0oooo0?ooo`0000001P3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool02`3oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo0180oooo0P0000020?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0OP3oool006h0oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`050?ooo`050000003oool0oooo0?ooo`0000001@3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool02@3oool2000001@0oooo00<000000?ooo`3o
ool0503oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo01H0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo07h0oooo001_0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0E0?ooo`03
0000003oool0000000T0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool0O`3oool006l0oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00D000000?ooo`3oool0
oooo000000090?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0203oool00`000000
oooo0?ooo`0E0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`0000001`3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`200?ooo`00L03oool00`000000oooo0?ooo`0?0?ooo`050000003oool0oooo0?ooo`00
00001P3oool00`000000oooo0000000;0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3o
ool0203oool00`000000oooo0?ooo`0F0?ooo`<000005`3oool00`000000oooo000000060?ooo`03
0000003oool0oooo01H0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo0800oooo001`0?ooo`030000003oool0oooo0100oooo00<000000?ooo`000000
1`3oool2000000`0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo01L0oooo00<000000?ooo`3oool0603oool3000000@0oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
P@3oool00740oooo00<000000?ooo`3oool0403oool2000000L0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00H0oooo0P00000J0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`0000000`00000I0?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool01P3oool2000008@0oooo001b0?ooo`030000003oool0oooo00l0oooo0`000004
0?ooo`80000000<0oooo0000003oool03@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool03P3oool2000000@0oooo0P0000260?ooo`00LP3oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo00@000000`3oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo01/0oooo0P00000I0?ooo`030000003oool0oooo0140oooo1@0000270?oo
o`00L`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00<0oooo0P00000N0?ooo`03
0000003oool0oooo01d0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`2N0?ooo`00
M03oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
7P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
W`3oool007D0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo00h0oooo0P00000=0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0X03o
ool007H0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool02@3oool010000000oooo0?ooo`3oool200000280oooo00<0
00000?ooo`3oool0803oool00`000000oooo0?ooo`0>0?ooo`800000X`3oool007L0oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0100oooo0P000009
0?ooo`040000003oool00000000002@0oooo00<000000?ooo`3oool08@3oool2000000`0oooo0P00
002U0?ooo`00N03oool00`000000oooo0?ooo`040?ooo`800000403oool00`000000oooo0?ooo`0B
0?ooo`<000001@3oool3000002H0oooo0`00000S0?ooo`@000001@3oool300000:L0oooo001i0?oo
o`<0000000<0oooo0000000000004`3oool00`000000oooo0?ooo`0D0?ooo`H00000:03oool00`00
0000oooo0?ooo`0W0?ooo`D00000ZP3oool007/0oooo0`00000E0?ooo`030000003oool0oooo01L0
oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`3F0?ooo`00T`3oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0eP3oool009@0oooo00<0
00000?ooo`3oool05@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo0=H0oooo002E
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool02`3oool01@3ogi`0B00000000000
07@0_ooo00<0oooo00<0okmd0000001d_ol00`3oool01@3ogi`0B000000000000000M;oo00X0oooo
00<000000?ooo`3oool0eP3oool009H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`0;0?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`020?ooo`030?noM0000000M;oo0080
oooo00L0omnL04P00018W=l0oooo0?oo_`1d0000M;oo00T0oooo00<000000?ooo`3oool0eP3oool0
09H0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0<0?ooo`030;md0000B9`0gooo
0080oooo00<0_g@0001d_`3oool01@3oool00`3ooml0W4P0001d_`030?ooo`030?noM000B9`0gooo
00P0oooo00<000000?ooo`3oool0eP3oool009L0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`0;0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool01@3oool00`3ogi`0
B00007Boo`030?ooo`030?noM00004P0W=oo00P0oooo00<000000?ooo`3oool0eP3oool009P0oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0<0?ooo`030?oOW0180000M;oo00<0oooo
00<0okmd0000B02Lgol01@3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00P0oooo
00<000000?ooo`3oool0eP3oool009T0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0=0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01@3oool00`3ogi`0B00007Bo
o`030?ooo`030;md000004P0W=oo00P0oooo1000003E0?ooo`00VP3oool2000000/0oooo00<00000
0?ooo`3oool03P3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00D0oooo00P0oono
07@000000000B9cO0?ooo`3o_g@00000001d_`T0oooo00<000000?ooo`3oool0eP3oool009`0oooo
00<000000?ooo`3oool01P3oool200000140oooo00<0omnL04P0001d_ol00`3oool00`3o_g@00018
09cOo`060?ooo`030;md0000B9`0_ia80080000000<004RL0=ooo`3oool0203oool00`000000oooo
0?ooo`3F0?ooo`00W@3oool2000000<0oooo0`00000C0?ooo`030?oo_`1d0000B9cO00<0oooo00<0
gia80000M02oool01P3oool00`3ogi`0B00007Boo`0=0?ooo`030000003oool0oooo0=H0oooo002O
0?ooo`@000005P3oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00L0oooo00<0gia8
0000B02Lgol0303oool00`000000oooo0?ooo`3F0?ooo`00^@3oool01`3ookl0M00007Boo`3oool0
okmd0000M02oool02@3oool00`3o_g@00000001d_`0;0?ooo`030000003oool0oooo0=H0oooo002j
0?ooo`050?oo_`1d000000000000M02oool02`3oool01@3ooml0W4P00000000007@0_ooo00P0oooo
00<000000?ooo`3oool0eP3oool00=L0oooo00<000000?ooo`3oool0eP3oool00=L0oooo00<00000
0?ooo`3oool0eP3oool00=L0oooo00<000000?ooo`3oool0eP3oool00=L0oooo0`00003F0?ooo`00
o`3ooona0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {-5.3208, -0.65827, \
0.0370358, 0.00742761}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Higher energies", "Subsection"],

Cell[TextData[{
  "Higher energies must be integrated over larger ranges to ensure that the \
initial conditions are adequate.  As the energy increases, the central bump \
becomes less important and the spacing between even and odd levels approaches \
the uniform spacing of the simple harmonic oscillator, but now with ",
  Cell[BoxData[
      \(TraditionalForm\`k = 1\)]],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
        mismatch[e, 0.5, pot, \(-7\), 7, 0, MaxSteps \[Rule] 5000], {e, 3, 
          22}, PlotRange \[Rule] {All, {\(-50\), 50}}, 
        PlotPoints \[Rule] 100, Frame \[Rule] True, 
        FrameLabel \[Rule] {"\<energy\>", "\<mismatch\>"}, 
        PlotLabel \[Rule] "\<matching condition for m=0.5\>"];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.126566 0.0501253 0.309017 0.00618034 [
[.12406 -0.0125 -5 -15.6875 ]
[.12406 -0.0125 5 0 ]
[.24937 -0.0125 -9.5 -15.6875 ]
[.24937 -0.0125 9.5 0 ]
[.37469 -0.0125 -8 -15.6875 ]
[.37469 -0.0125 8 0 ]
[.5 -0.0125 -12.5 -15.6875 ]
[.5 -0.0125 12.5 0 ]
[.62531 -0.0125 -8 -15.6875 ]
[.62531 -0.0125 8 0 ]
[.75063 -0.0125 -12.5 -15.6875 ]
[.75063 -0.0125 12.5 0 ]
[.87594 -0.0125 -8 -15.6875 ]
[.87594 -0.0125 8 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -17.9688 -31.375 ]
[.5 -0.0125 17.9688 -15.6875 ]
[-0.0125 .0618 -24.3125 -7.84375 ]
[-0.0125 .0618 0 7.84375 ]
[-0.0125 .18541 -24.3125 -7.84375 ]
[-0.0125 .18541 0 7.84375 ]
[-0.0125 .30902 -10 -7.84375 ]
[-0.0125 .30902 0 7.84375 ]
[-0.0125 .43262 -16 -7.84375 ]
[-0.0125 .43262 0 7.84375 ]
[-0.0125 .55623 -16 -7.84375 ]
[-0.0125 .55623 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -40 -25.2813 ]
[-0.01875 .30902 -24.3125 25.2813 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -74.3125 0 ]
[.5 .63053 74.3125 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.12406 0 m
.12406 .00625 L
s
gsave
.12406 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
.24937 0 m
.24937 .00625 L
s
gsave
.24937 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7.5) show
1.000 setlinewidth
grestore
.37469 0 m
.37469 .00625 L
s
gsave
.37469 -0.0125 -69 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
1.000 setlinewidth
grestore
.5 0 m
.5 .00625 L
s
gsave
.5 -0.0125 -73.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(12.5) show
1.000 setlinewidth
grestore
.62531 0 m
.62531 .00625 L
s
gsave
.62531 -0.0125 -69 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(15) show
1.000 setlinewidth
grestore
.75063 0 m
.75063 .00625 L
s
gsave
.75063 -0.0125 -73.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(17.5) show
1.000 setlinewidth
grestore
.87594 0 m
.87594 .00625 L
s
gsave
.87594 -0.0125 -69 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
.125 Mabswid
.02381 0 m
.02381 .00375 L
s
.04887 0 m
.04887 .00375 L
s
.07393 0 m
.07393 .00375 L
s
.099 0 m
.099 .00375 L
s
.14912 0 m
.14912 .00375 L
s
.17419 0 m
.17419 .00375 L
s
.19925 0 m
.19925 .00375 L
s
.22431 0 m
.22431 .00375 L
s
.27444 0 m
.27444 .00375 L
s
.2995 0 m
.2995 .00375 L
s
.32456 0 m
.32456 .00375 L
s
.34962 0 m
.34962 .00375 L
s
.39975 0 m
.39975 .00375 L
s
.42481 0 m
.42481 .00375 L
s
.44987 0 m
.44987 .00375 L
s
.47494 0 m
.47494 .00375 L
s
.52506 0 m
.52506 .00375 L
s
.55013 0 m
.55013 .00375 L
s
.57519 0 m
.57519 .00375 L
s
.60025 0 m
.60025 .00375 L
s
.65038 0 m
.65038 .00375 L
s
.67544 0 m
.67544 .00375 L
s
.7005 0 m
.7005 .00375 L
s
.72556 0 m
.72556 .00375 L
s
.77569 0 m
.77569 .00375 L
s
.80075 0 m
.80075 .00375 L
s
.82581 0 m
.82581 .00375 L
s
.85088 0 m
.85088 .00375 L
s
.901 0 m
.901 .00375 L
s
.92607 0 m
.92607 .00375 L
s
.95113 0 m
.95113 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -78.9688 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(energy) show
94.938 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .0618 m
.00625 .0618 L
s
gsave
-0.0125 .0618 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
0 .18541 m
.00625 .18541 L
s
gsave
-0.0125 .18541 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
0 .30902 m
.00625 .30902 L
s
gsave
-0.0125 .30902 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
0 .43262 m
.00625 .43262 L
s
gsave
-0.0125 .43262 -77 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
0 .55623 m
.00625 .55623 L
s
gsave
-0.0125 .55623 -77 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .09271 m
.00375 .09271 L
s
0 .12361 m
.00375 .12361 L
s
0 .15451 m
.00375 .15451 L
s
0 .21631 m
.00375 .21631 L
s
0 .24721 m
.00375 .24721 L
s
0 .27812 m
.00375 .27812 L
s
0 .33992 m
.00375 .33992 L
s
0 .37082 m
.00375 .37082 L
s
0 .40172 m
.00375 .40172 L
s
0 .46353 m
.00375 .46353 L
s
0 .49443 m
.00375 .49443 L
s
0 .52533 m
.00375 .52533 L
s
0 .0309 m
.00375 .0309 L
s
0 .58713 m
.00375 .58713 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -118.438 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
86.2813 11.8438 translate
90 rotate
-86.2813 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(mismatch) show
109.563 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.12406 .61178 m
.12406 .61803 L
s
.24937 .61178 m
.24937 .61803 L
s
.37469 .61178 m
.37469 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.62531 .61178 m
.62531 .61803 L
s
.75063 .61178 m
.75063 .61803 L
s
.87594 .61178 m
.87594 .61803 L
s
.125 Mabswid
.02381 .61428 m
.02381 .61803 L
s
.04887 .61428 m
.04887 .61803 L
s
.07393 .61428 m
.07393 .61803 L
s
.099 .61428 m
.099 .61803 L
s
.14912 .61428 m
.14912 .61803 L
s
.17419 .61428 m
.17419 .61803 L
s
.19925 .61428 m
.19925 .61803 L
s
.22431 .61428 m
.22431 .61803 L
s
.27444 .61428 m
.27444 .61803 L
s
.2995 .61428 m
.2995 .61803 L
s
.32456 .61428 m
.32456 .61803 L
s
.34962 .61428 m
.34962 .61803 L
s
.39975 .61428 m
.39975 .61803 L
s
.42481 .61428 m
.42481 .61803 L
s
.44987 .61428 m
.44987 .61803 L
s
.47494 .61428 m
.47494 .61803 L
s
.52506 .61428 m
.52506 .61803 L
s
.55013 .61428 m
.55013 .61803 L
s
.57519 .61428 m
.57519 .61803 L
s
.60025 .61428 m
.60025 .61803 L
s
.65038 .61428 m
.65038 .61803 L
s
.67544 .61428 m
.67544 .61803 L
s
.7005 .61428 m
.7005 .61803 L
s
.72556 .61428 m
.72556 .61803 L
s
.77569 .61428 m
.77569 .61803 L
s
.80075 .61428 m
.80075 .61803 L
s
.82581 .61428 m
.82581 .61803 L
s
.85088 .61428 m
.85088 .61803 L
s
.901 .61428 m
.901 .61803 L
s
.92607 .61428 m
.92607 .61803 L
s
.95113 .61428 m
.95113 .61803 L
s
.97619 .61428 m
.97619 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .0618 m
1 .0618 L
s
.99375 .18541 m
1 .18541 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .43262 m
1 .43262 L
s
.99375 .55623 m
1 .55623 L
s
.125 Mabswid
.99625 .09271 m
1 .09271 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .15451 m
1 .15451 L
s
.99625 .21631 m
1 .21631 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .27812 m
1 .27812 L
s
.99625 .33992 m
1 .33992 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .40172 m
1 .40172 L
s
.99625 .46353 m
1 .46353 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .52533 m
1 .52533 L
s
.99625 .0309 m
1 .0309 L
s
.99625 .58713 m
1 .58713 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .30902 m
1 .30902 L
s
gsave
.5 .63053 -135.313 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(matching) show
110.375 15.813 moveto
(condition) show
158.563 15.813 moveto
(for) show
175.438 15.813 moveto
(m) show
184.813 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
192.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
207.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .32245 m
.03318 .32086 L
.04339 .31872 L
.05298 .3161 L
.06221 .31255 L
.06695 .31002 L
.07203 .30635 L
.07451 .30399 L
.07719 .30077 L
.07973 .29679 L
.08208 .29186 L
.08321 .28885 L
.08442 .28494 L
.08547 .28082 L
.08662 .27525 L
.08795 .26663 L
.08857 .26157 L
.08923 .2551 L
.08988 .24721 L
.09047 .23833 L
.09106 .22704 L
.09162 .21329 L
.0922 .19396 L
.09274 .16859 L
.09306 .14895 L
.09334 .12612 L
.09367 .09099 L
.09398 .04378 L
s
.09398 .04378 m
.09418 0 L
s
.09566 0 m
.09567 .61803 L
s
.09697 .61803 m
.09707 .59631 L
.09735 .5536 L
.09762 .52451 L
.09812 .48557 L
.09837 .47111 L
.09865 .45788 L
.09923 .43673 L
.09984 .42026 L
.10043 .4085 L
.10096 .39995 L
.10212 .38612 L
.10277 .38029 L
.10336 .37584 L
.10451 .36874 L
.10557 .36362 L
.10682 .35883 L
.10816 .35467 L
.1106 .34897 L
.11297 .34488 L
.11517 .3419 L
.12019 .33693 L
.12513 .33346 L
.12963 .33102 L
.1393 .32703 L
.14861 .32404 L
.15851 .3213 L
.16804 .31884 L
.1772 .31651 L
.18696 .31393 L
.19635 .31127 L
.20634 .30809 L
.21596 .30448 L
.22521 .30023 L
.23037 .29736 L
.23506 .29431 L
.24453 .28616 L
.24919 .28059 L
.2516 .2771 L
.25424 .27258 L
.25907 .26168 L
.26171 .2535 L
.26417 .24359 L
.2667 .2299 L
.26795 .22122 L
.2691 .21154 L
.27125 .18723 L
.27239 .16937 L
.2736 .14416 L
.27417 .12887 L
Mistroke
.27478 .10955 L
.27586 .06292 L
.27647 .02655 L
Mfstroke
.27647 .02655 m
.2768 0 L
s
.28076 0 m
.28077 .61803 L
s
.28472 .61803 m
.28505 .59432 L
.28568 .55857 L
.28636 .52937 L
.28758 .4913 L
.28818 .47697 L
.28883 .4639 L
.29 .44485 L
.2914 .42758 L
.29268 .41525 L
.29506 .39808 L
.29728 .38645 L
.2997 .3767 L
.30229 .36862 L
.30747 .35691 L
.31236 .34911 L
.31722 .34319 L
.32169 .33878 L
.33125 .33144 L
.34044 .32594 L
.35022 .32098 L
.35964 .31665 L
.36965 .31224 L
.37929 .30793 L
.38857 .30353 L
.39844 .29832 L
.40794 .29244 L
.41804 .28464 L
.42311 .27977 L
.42777 .27446 L
.43262 .26777 L
.43714 .25999 L
.44171 .24986 L
.44429 .24269 L
.44673 .23456 L
.4491 .22494 L
.45167 .21183 L
.45401 .19642 L
.45618 .17775 L
.4573 .16557 L
.45852 .14989 L
.45957 .1335 L
.46071 .11152 L
.46205 .07814 L
.46267 .05883 L
.46332 .03448 L
.46397 .00539 L
s
.46397 .00539 m
.46407 0 L
s
.47025 0 m
.47025 .61803 L
s
.47654 .61803 m
.47658 .6155 L
.47723 .58725 L
.47781 .5658 L
.47897 .53191 L
.48003 .50777 L
.48128 .48534 L
.48262 .46612 L
.48506 .44006 L
.48742 .4217 L
.48963 .40851 L
.4922 .39636 L
.49464 .38711 L
.49958 .37282 L
.50408 .36316 L
.51338 .3489 L
.51818 .34335 L
.52328 .33831 L
.53244 .33072 L
.5422 .32392 L
.55159 .31806 L
.56157 .31217 L
.57118 .30652 L
.58043 .30085 L
.59028 .29423 L
.59975 .28687 L
.60982 .27727 L
.61487 .27138 L
.61952 .265 L
.62886 .24798 L
.63341 .23635 L
.63603 .2281 L
.63842 .21918 L
.64314 .19585 L
.64574 .17803 L
.64821 .15582 L
.65054 .12739 L
.65172 .10893 L
.65303 .08392 L
.65413 .05748 L
.65533 .0213 L
s
.65533 .0213 m
.65586 0 L
s
.66373 0 m
.66374 .61803 L
s
.67192 .61803 m
.67315 .57706 L
.67434 .54632 L
.67572 .5185 L
.67697 .49824 L
.67933 .46885 L
.68183 .44598 L
.68396 .43083 L
.6863 .41738 L
.69102 .39704 L
.69608 .38144 L
.70122 .36957 L
.70609 .36066 L
.71536 .34761 L
.72523 .33706 L
.73473 .32872 L
.74483 .32094 L
.75455 .31398 L
.76391 .30742 L
.77387 .30024 L
.78346 .29278 L
.79268 .28467 L
.80249 .27434 L
.81194 .26161 L
.81675 .25344 L
.82198 .2426 L
.82701 .22933 L
.83165 .21327 L
.83619 .19195 L
.83877 .17592 L
.84119 .15702 L
.8437 .13153 L
.84609 .0988 L
.84824 .05805 L
.84945 .02754 L
s
.84945 .02754 m
.85033 0 L
s
.85951 0 m
.85952 .61803 L
s
.8693 .61803 m
.86952 .6103 L
.87072 .57692 L
.87182 .55225 L
.87428 .5102 L
.87641 .48373 L
.87873 .46155 L
.88353 .42893 L
.88595 .41682 L
.88862 .40569 L
.89332 .39013 L
.89838 .37721 L
.90836 .35861 L
.91797 .3456 L
.92721 .33556 L
.93705 .32641 L
.94653 .31843 L
.95659 .31033 L
.96629 .30251 L
.97562 .29459 L
.97619 .29409 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{356.75, 220.5},
  ImageMargins->{{36, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40008G0001BR000`400?l00000o`00003oo`3ooooo
0?oooaT0oooo003o0?ooool0oooo6@3oool00?l0ooooo`3ooolI0?ooo`00o`3ooold0?ooo`030?oo
g`2LB000000000<0000000<0B9cO0?ooo`3OW4P00P0000000`1d_ol0oooo0?ooo`3F0?ooo`00o`3o
oold0?ooo`030;md0000B9`0gooo00<0oooo00<0gibL0;ooo`3oool00P3oool00`2oM7@0_ooo0?oo
o`3F0?ooo`00o`3ooold0?ooo`030?oog`2LM9`0gooo00<0oooo00<0oono09bLg`3oool00P3oool0
0`3ogi`0M9cO0?ooo`3F0?ooo`00o`3ooole0?ooo`030;md00000000000000<0000000L0B9cO0?oo
o`3oool0oooo0?oog`2LB7@0_ooo0=H0oooo003o0?oooaL0oooo00L0_g@0000000000000001d0;oo
o`3oool0_g@00080000000<0001d0;oog`2LB0000P0000001000B9`0gooo0?ooo`2oM00200000004
0000M02oool0oooo0;md0080000000D0001809cOo`3oool0oono07@00004000000030000M02oool0
oooo00<0oooo00<0_g@007Boo`3oool0e@3oool00?l0oooo5P3oool03`2oM0000000001d_`3oool0
omnL04P0M02oool0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`0;0;md00000000
07Bo0?ooo`3ogi`0B01d0;ooo`3oool0gia80000M02oool00`3oool00`2oM4P0W=oo0?ooo`070?oo
o`040?oOW018000004RL0=ooomD0oooo003o0?oooaD0oooo00<0omnL04P00018W=l00`3oool03P3o
gi`0M9cO0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?oOW0180000B9cO0`3oool0
1P3ogi`0M9cO0?ooo`3OW4P0001d0;ooo`<0oooo00D0oono07@0000000000000001d_`050?ooo`04
0;md0000B9`0W7@007BoomD0oooo003o0?oooaD0oooo00<0okmd0000B02Lgol01P3oool02`3OW4P0
04RL0=ooo`3oool0oooO09a80000M;l0oooo0?noM00004P0W=oo00H0oooo00<0gia80000M02oool0
0`3oool01P3OW4P0001d0;ooo`3ookl0M00004RLg`<0oooo00D0okmd0000001d_ol0omnL04Qd_`3E
0?ooo`00o`3ooolE0?ooo`030?noM00007@0_ooo00H0oooo00/0gia80018W03Oool0oooo0?oog`2L
B00007Bo0?ooo`3o_g@0001d0;ooo`060?ooo`030=nLB00007@0_ooo0080oooo00L0oono07@00018
W=l0oooo0?oog`2LB00007Bo0080oooo00L0oooO09a80000B9`0gooo0?ooo`2oM4P0W=oo0=@0oooo
003o0?oooaD0oooo00<0omnL04P000000000100000003018W=l0oooo0;md000007@0_ooo0?ooo`3o
oml0W4P0001d_`3oool0omnL04P000D0000000D0B9cO0?ooo`3OW4P0001809cOo`020?ooo`070?oo
_`1d0000B9cO0?ooo`3ooml0W4P004RLg`020?ooo`030?noM00004P0W=oo0080oooo00<0okmd04RL
g`3oool0d`3oool00?l0oooo5P3oool03`2oM000B9cO0?ooo`3oool0gia80000B02Lgml0W4P00000
0000B4P0B01809cOo`3ookl0M00004RLg`020?ooo`0B0;md0018W=l0oooo0?ooo`3OW4P0001809cO
g`2LB00000000000B018M9`0gkmd0000M02oool0_g@0001d_`3oool0okmd0P0000001@00B9`0gono
07@00000B9`0gooo0080oooo00<0oono07@0M02oool0d`3oool00?l0oooo5`3oool01@3OW4P00000
0000000004P0W=oo0080oooo00L0_g@00018W03Ooml0W4P00000000007@0_ooo00<0oooo00D0gia8
000000000000001809cOo`020?ooo`080;md000007@0_mnL04P0000007@0_ooo0?oog`2LB0020000
000407Boo`3oool0oooO09a80080000000L004RL0=ooo`3oool0_g@000000000B9`0gooo0=80oooo
003o0?ooool0oooo6@3oool00?l0ooooo`3ooolI0?ooo`00o`3ooooo0?oooaT0oooo003o0?ooool0
oooo6@3oool00?l0ooooo`3ooolI0?ooo`00o`3ooooo0?oooaT0oooo003o0?ooool0oooo6@3oool0
0?l0ooooo`3ooolI0?ooo`00o`3ooooo0?oooaT0oooo003o0?ooool0oooo6@3oool00?l0ooooo`3o
oolI0?ooo`00o`3ooooo0?oooaT0oooo003o0?ooool0oooo6@3oool00?l0ooooo`3ooolI0?ooo`00
o`3ooooo0?oooaT0oooo002@0?ooo`030?oog`2LB00000000080000000<0M;oo0?ooo`3oool09`3o
ool00`3OW4P0B9cO0?ooo`040?ooo`060?noM0000000M;oo0?ooo`3ooml0W4P00`0000000`1d_ol0
oooo0?ooo`0Q0?ooo`060;md00000000000000000000B9`0gooo0`3oool01@3ogi`0B00000000000
07@0_ooo01d0oooo00P0_g@000000000000000000018W03Oool0oooo0;md00H0000000T0B9cO0?oo
o`3oool0okmd0000001d_ol0oooo0?oog`2LB0000`0000000`1d_ol0oooo0?ooo`0M0?ooo`090;md
00000000000000000000B9`0gooo0?ooo`3ooml0W4P000<0000000<0M;oo0?ooo`3oool07@3oool0
1P2oM000000000000000000004RL0=ooo`@0oooo00<0gia804RLg`3oool0103oool01P3o_g@00000
07Boo`3oool0oooO09a800<0000000<0M;oo0?ooo`3oool06P3oool00`2oM0000000000000040000
000904RLg`3oool0oooo0?ooo`3ogi`0B0000000000007@0_ooo04`0oooo002@0?ooo`030?oog`2L
B7@0_ooo0080oooo00<0_g@007Boo`3oool09`3oool00`3ogi`0B4RL0=ooo`040?ooo`070?noM000
0000M;oo0?ooo`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0oooo0280oooo00<0omnL04P0001d
_ol0103oool01`3ookl0M00007Boo`3oool0okmd0000M02oool07@3oool00`3ogi`0B00007Boo`04
0?ooo`030=nLB0000000000000<0000000X004RL0=ooo`3oool0okmd0000001d_ol0oooo0?oog`2L
B7@0_ooo0P3oool00`2oM000M;oo0?ooo`0N0?ooo`030?oOW0180000M;oo00<0oooo00<0oooO09a8
M02oool00P3oool00`2oM000M;oo0?ooo`0N0?ooo`030?oOW0180000M;oo00H0oooo00<0omnL04Q8
W03Oool0103oool01`3o_g@0000007Boo`3oool0oooO09a8M02oool00P3oool00`2oM000M;oo0?oo
o`0K0?ooo`030=nLB0000000000000<0000000X004RL0=ooo`3oool0oono07@0001d_ol0oooo0?no
M00007@0_oooB`3oool009D0oooo00<0oooO09a8M02oool09`3oool00`3ooml0W4Q809cOo`0=0?oo
o`030?oog`2LB7@0_ooo0280oooo00<0omnL04P0001d_ol0103oool00`2oM00004RL0=ooo`020?oo
o`030;md0000M;l0oooo01`0oooo00<0omnL04P0001d_ol01@3oool00`3OW4P0B9cO0?ooo`020?oo
o`030=nLM02Lgol0oooo00T0oooo00<0oooO09a8M02oool07P3oool00`3ogi`0B00007Boo`080?oo
o`030?oog`2LB7@0_ooo01h0oooo00<0omnL04P0001d_ol01P3oool00`3ooml0W4Q809cOo`0=0?oo
o`030?oog`2LB7@0_ooo01`0oooo00<0gia804RLg`3oool00P3oool01P3OW7@0W=oo0?ooo`2oM000
04RL0=ooo`80oooo00<0_g@0001d_`3oool0BP3oool009H0oooo00<0_g@007Boo`3oool09`3oool0
0`3OW4P0B9cO0?ooo`0=0?ooo`030;md001d_ol0oooo0240oooo00<0omnL04P0001d_ol00`3oool0
0`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo01`0oooo00<0omnL04P0001d_ol01P3oool0
0`3OW4P0M;oo0?ooo`0>0?ooo`030;md001d_ol0oooo01d0oooo00<0omnL04P0001d_ol02@3oool0
0`2oM000M;oo0?ooo`0M0?ooo`030?oOW0180000M;oo00L0oooo00<0gia804RLg`3oool03@3oool0
0`2oM000M;oo0?ooo`0L0?ooo`030=nLB01d_ol0oooo00<0oooo00<0oono07@00018W=l00`3oool0
0`3OW4P0001d0;ooo`1:0?ooo`00U@3oool00`3ooml0W4P004RLg`0X0?ooo`030?oOW018B9`0gooo
00`0oooo00<0oooO09a80018W=l08P3oool00`3ogi`0B00007Boo`030?ooo`030?oOW0180000M;oo
00<0oooo00<0okmd0000B02Lgol0703oool00`3ogi`0B00007Boo`070?ooo`030;mdB02Lgol0oooo
00`0oooo00<0oooO09a80018W=l07P3oool00`3ogi`0B00007Boo`080?ooo`030?oog`2LB000B9cO
01h0oooo00<0omnL04P0001d_ol01`3oool00`3ogi`0B4RL0=ooo`0<0?ooo`030?oog`2LB000B9cO
01h0oooo00<0_gA809cOo`3oool00P3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo
04X0oooo002E0?ooo`030=nLB0000000M;oo02P0oooo00<0oooO09a8B02Lgol0303oool00`3OW4P0
000007Boo`0R0?ooo`030?oOW0180000M;oo00<0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0
001809cOo`0L0?ooo`030?oOW0180000M;oo00L0oooo00<0oooO09a8M02oool0303oool00`3OW4P0
000007Boo`0N0?ooo`030?oOW0180000M;oo00P0oooo00<0gia80000001d_ol07P3oool00`3ogi`0
B00007Boo`070?ooo`030?oog`2LB4P0W=oo00`0oooo00<0gia80000001d_ol07P3oool00`3ooml0
W4Qd0;ooo`020?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0BP3oool00940oooo
00L0gmoO0?ooo`3ookl0M0000000000007@0_ooo02T0oooo00<0gia8001d_`3oool01`3oool01`3O
gml0oooo0?oo_`1d000000000000M02oool08P3oool00`3ogi`0B00007Boo`030?ooo`030?oOW018
0000M;oo00<0oooo00<0okmd0000B02Lgol0703oool00`3ogi`0B00007Boo`080?ooo`030?noM000
M;l0oooo00L0oooo00L0gmoO0?ooo`3ookl0M0000000000007@0_ooo01h0oooo00<0omnL04P0001d
_ol0103oool01`3Ogml0oooo0?oo_`1d000000000000M02oool07P3oool00`3ogi`0B00007Boo`08
0?ooo`030=nLB000M;l0oooo00L0oooo00L0gmoO0?ooo`3ookl0M0000000000007@0_ooo01l0oooo
00L0okmd001d_`3oool0oooo0?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0BP3oool00940
oooo00H0gia800000000000000000000B02LgolZ0?ooo`030?oo_`1d07@0_ooo00L0oooo00H0gia8
00000000000000000000B02LgolS0?ooo`030?oOW0180000M;oo00<0oooo00<0omnL04P0001d_ol0
0`3oool00`3o_g@0001809cOo`0L0?ooo`030?oOW0180000M;oo00P0oooo00<0oono07@0B02Lgol0
1`3oool01P3OW4P00000000000000000001809cOoal0oooo00<0omnL04P0001d_ol0103oool01P3O
W4P00000000000000000001809cOoal0oooo00<0omnL04P0001d_ol0203oool00`3ookl0M01d0;oo
o`070?ooo`060=nLB000000000000000000004P0W=oo803oool01`3ookl0M01809cOo`3oool0omnL
04P0001d_ol00`3oool00`3o_g@0001809cOo`1:0?ooo`00T@3oool0103ookl0M00000000018W=lW
0?ooo`030?oOg`3Oool0oooo00<0oooo00<0_g@007Boo`3oool01P3oool0103ookl0M00000000018
W=lU0?ooo`030?oOW0180000M;oo00<0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;oo
o`0L0?ooo`030?oOW0180000M;oo00P0oooo00<0oono07@00018W=l01`3oool0103ookl0M0000000
0018W=lQ0?ooo`030?oOW0180000M;oo00@0oooo00@0oono07@000000000B9cO8@3oool00`3ogi`0
B00007Boo`030?ooo`030?oOg`3Oool0oooo00<0oooo00<0_g@007Boo`3oool01P3oool0103ookl0
M00000000018W=lR0?ooo`070?oo_`1d0000B9cO0?ooo`3ookl0M00004RLg`030?ooo`030=nLB000
07@0_ooo04X0oooo002B0?ooo`030;md001d_ol0oooo02L0oooo00<0oono07A8W03Oool00`3oool0
0`3o_g@007Bo0?ooo`070?ooo`030;md001d_ol0oooo02D0oooo00<0omnL04P0001d_ol0103oool0
0`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo01`0oooo00<0omnL04P0001d_ol00`3oool0
0`3o_i`0W=oo0?ooo`020?ooo`030?oOW0180000B9cO00P0oooo00<0_g@007Boo`3oool08@3oool0
0`3ogi`0B00007Boo`050?ooo`030;md001d_ol0oooo0240oooo00<0omnL04P0001d_ol00`3oool0
0`3ookl0M4RL0=ooo`030?ooo`030?noM000M;l0oooo00L0oooo00<0_g@007Boo`3oool07@3oool0
0`3o_i`0W=oo0?ooo`020?ooo`030?oOW0180000B9cO0080oooo00<0gia80018W03Oool00P3oool0
0`2oM00007Bo0?ooo`1:0?ooo`00TP3oool01P3ogi`0B000000000000000001809cOobD0oooo00<0
_g@00000000000000`0000000`0007@0_ooo0?ooo`060?ooo`060?oOW018000000000000000004P0
W=oo803oool01@3ooml0W4P0000000000000M;oo00@0oooo00L0oono07@0001d_ol0oooo0?noM000
07@0_ooo01/0oooo00D0oooO09a800000000000007Boo`030?ooo`080?oog`2LB4P0W=oo0?ooo`3o
okl0M0000000M02oool80?ooo`060?oOW018000000000000000004P0W=oo703oool01@3ooml0W4P0
000000000000M;oo00D0oooo00H0omnL04P00000000000000000B02LgolL0?ooo`050?oog`2LB000
00000000001d_ol0103oool00`2oM000000000000003000000030000M02oool0oooo00H0oooo00H0
omnL04P00000000000000000B02LgolJ0?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02o
ool20?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`1;0?ooo`00T`3oool01@2oM0000000
000000000000B9cO02D0oooo00<0okmd000000000000100000000`1d_ol0oooo0?ooo`060?ooo`05
0;md00000000000000000018W=l08P3oool00`3o_g@0000007Boo`050?ooo`050?oo_`1d00000000
0000M02oool07P3oool00`3o_g@0000007Boo`040?ooo`060?oog`2LB00000000000000007@0_ooo
2P3oool01@2oM0000000000000000000B9cO01h0oooo00<0okmd0000001d_ol01P3oool01@2oM000
0000000000000000B9cO01h0oooo00<0okmd0000001d_ol0103oool00`3o_g@00000000000040000
000307Boo`3oool0oooo00H0oooo00D0_g@0000000000000000004RLg`0K0?ooo`060?oog`2LB000
00000000000007@0_ooo103oool01@3ookl0M0000000000007@0_ooo04`0oooo003o0?ooool0oooo
6@3oool00?l0ooooo`3ooolI0?ooo`00o`3ooooo0?oooaT0oooo003o0?ooool0oooo6@3oool00?l0
ooooo`3ooolI0?ooo`00o`3ooooo0?oooaT0oooo003o0?ooool0oooo6@3oool00?l0ooooo`3ooolI
0?ooo`00o`3ooooo0?oooaT0oooo003o0?ooool0oooo6@3oool006<0ooooo`00002E00000200oooo
001S0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`0000002@3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool2000000030?ooo`00
0000oooo00D0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool00`3oool01@000000oooo0000003oool0000000T0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00P0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`0000000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00H0oooo
0P0000020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo01h0oooo001S0?ooo`030000003oool0oooo02<0oooo00<0
00000?ooo`3oool02@3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo00P0oooo00<0
00000?ooo`0000009@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0000000/0oooo00<0
00000?ooo`3oool0<03oool00`000000oooo0?ooo`090?ooo`040000003oool0oooo00000280oooo
00<000000?ooo`3oool09@3oool01@000000oooo0?ooo`3oool0000000H0oooo00<000000?ooo`3o
ool0;`3oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0S0?ooo`030000
003oool0oooo04H0oooo00<000000?ooo`000000B@3oool00`000000oooo0000001:0?ooo`040000
003oool0oooo000004X0oooo00D000000?ooo`3oool0oooo0000000h0?ooo`030000003oool0oooo
01h0oooo001S0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0AP3oool00`000000
oooo000000190?ooo`030000003oool0000004X0oooo00@000000?ooo`3oool00000BP3oool01@00
0000oooo0?ooo`3oool0000003P0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?oo
o`3oool08`3oool00`000000oooo0?ooo`160?ooo`030000003oool0000004T0oooo00<000000?oo
o`000000BP3oool010000000oooo0?ooo`00001:0?ooo`050000003oool0oooo0?ooo`000000>03o
ool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0
oooo04H0oooo00<000000?ooo`000000B@3oool00`000000oooo0000001:0?ooo`040000003oool0
oooo000004X0oooo00D000000?ooo`3oool0oooo0000000h0?ooo`030000003oool0oooo01h0oooo
001S0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0000
00190?ooo`030000003oool0000004X0oooo00@000000?ooo`3oool00000BP3oool01@000000oooo
0?ooo`3oool0000003P0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool0
8`3oool00`000000oooo0?ooo`160?ooo`030000003oool0000004P0oooo00@000000?ooo`3oool0
0000BP3oool010000000oooo0?ooo`00001:0?ooo`050000003oool0oooo0?ooo`000000>03oool0
0`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo
04H0oooo00<000000?ooo`000000B03oool010000000oooo0?ooo`00001:0?ooo`040000003oool0
oooo000004X0oooo00D000000?ooo`3oool0oooo0000000h0?ooo`030000003oool0oooo01h0oooo
001S0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0000
00180?ooo`040000003oool0oooo000004X0oooo00@000000?ooo`3oool00000BP3oool01@000000
oooo0?ooo`3oool0000003P0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3o
ool08`3oool00`000000oooo0?ooo`160?ooo`030000003oool0000004P0oooo00@000000?ooo`3o
ool00000BP3oool010000000oooo0?ooo`00001:0?ooo`050000003oool0oooo0?ooo`000000>03o
ool00`000000oooo0?ooo`0N0?ooo`00H`3oool2000002@0oooo00<000000?ooo`3oool0AP3oool0
0`000000oooo000000180?ooo`040000003oool0oooo000004X0oooo00@000000?ooo`3oool00000
BP3oool01@000000oooo0?ooo`3oool0000003H0oooo0`00000P0?ooo`00H`3oool00`000000oooo
0?ooo`0S0?ooo`030000003oool0oooo04H0oooo00<000000?ooo`000000B03oool010000000oooo
0?ooo`00001:0?ooo`040000003oool0oooo000004X0oooo00D000000?ooo`3oool0oooo0000000h
0?ooo`030000003oool0oooo01h0oooo001S0?ooo`030000003oool0oooo02<0oooo00<000000?oo
o`3oool0AP3oool00`000000oooo000000180?ooo`040000003oool0oooo000004X0oooo00@00000
0?ooo`3oool00000BP3oool01@000000oooo0?ooo`3oool0000003P0oooo00<000000?ooo`3oool0
7P3oool006<0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`160?ooo`030000003o
ool0000004P0oooo00@000000?ooo`3oool00000BP3oool010000000oooo0?ooo`00001:0?ooo`05
0000003oool0oooo0?ooo`000000>03oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000
oooo0?ooo`0S0?ooo`030000003oool0oooo04H0oooo00<000000?ooo`000000B03oool010000000
oooo0?ooo`0000190?ooo`050000003oool0oooo0?ooo`000000BP3oool01@000000oooo0?ooo`3o
ool0000003P0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08`3oool0
0`000000oooo0?ooo`160?ooo`030000003oool0000004P0oooo00@000000?ooo`3oool00000B@3o
ool01@000000oooo0?ooo`3oool0000004X0oooo00D000000?ooo`3oool0oooo0000000h0?ooo`03
0000003oool0oooo01h0oooo001S0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0
AP3oool00`000000oooo000000180?ooo`040000003oool0oooo000004T0oooo00D000000?ooo`3o
ool0oooo0000001:0?ooo`050000003oool0oooo0?ooo`000000>03oool00`000000oooo0?ooo`0N
0?ooo`00H`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo04H0oooo00<000000?oo
o`000000B03oool010000000oooo0?ooo`0000190?ooo`050000003oool0oooo0?ooo`000000BP3o
ool01@000000oooo0?ooo`3oool0000003P0oooo00<000000?ooo`3oool07P3oool004d0oooo00@0
oooO09a80000B9`0gooo0`3oool01@3ogi`0B0000000000007@0_ooo00X0oooo00<000000?ooo`3o
ool08`3oool00`000000oooo0?ooo`160?ooo`030000003oool0000004P0oooo00@000000?ooo`3o
ool00000B@3oool01@000000oooo0?ooo`3oool0000004X0oooo00D000000?ooo`3oool0oooo0000
000h0?ooo`030000003oool0oooo01h0oooo001=0?ooo`040?oog`2LB00004RL0=ooo`80oooo00L0
oono07@0001d_ol0oooo0?noM00007@0_ooo00T0oooo00<000000?ooo`3oool08`3oool00`000000
oooo0?ooo`160?ooo`030000003oool0000004P0oooo00@000000?ooo`3oool00000B@3oool01@00
0000oooo0?ooo`3oool0000004X0oooo00D000000?ooo`3oool0oooo0000000h0?ooo`030000003o
ool0oooo01h0oooo001=0?ooo`040?oog`2LB00004RL0=ooo`80oooo00<0_g@00018W03Oool00P3o
ool00`2oM00007Bo0?ooo`080?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0AP3o
ool00`000000oooo000000180?ooo`040000003oool0oooo000004T0oooo00D000000?ooo`3oool0
oooo0000001:0?ooo`050000003oool0oooo0?ooo`000000>03oool00`000000oooo0?ooo`0N0?oo
o`00B@3oool00`2oM0000000000000050000000407Boo`3ookl0M00004RLg`<0oooo00<0gia80000
M02oool0203oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo04H0oooo00<000000?oo
o`000000B03oool010000000oooo0?ooo`0000190?ooo`050000003oool0oooo0?ooo`000000BP3o
ool01@000000oooo0?ooo`3oool0000003P0oooo00<000000?ooo`3oool07P3oool003`0oooo00<0
oooO09a8000000001`0000001@18W=l0oooo0?ooo`3ogi`0M9cO0080oooo00P0oooO09a80000B9`0
gooo0?ooo`3ogi`0B00007Boo`<0oooo00<0okmd0000B02Lgol0203oool00`000000oooo0?ooo`0S
0?ooo`030000003oool0oooo04H0oooo00<000000?ooo`000000B03oool010000000oooo0?ooo`00
00190?ooo`050000003oool0oooo0?ooo`000000BP3oool01@000000oooo0?ooo`3oool0000003P0
oooo00<000000?ooo`3oool07P3oool004X0oooo00/0gia807Boo`3oool0oooO09a80000B9`0gooo
0?ooo`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00P0oooo0`00000S0?ooo`030000003o
ool0oooo04H0oooo00<000000?ooo`000000B03oool010000000oooo0?ooo`0000190?ooo`050000
003oool0oooo0?ooo`000000BP3oool01@000000oooo0?ooo`3oool0000003D0oooo1000000P0?oo
o`00BP3oool02`3ooml0W4Qd0;ooo`3ooml0W4P00018W03Oool0oooo0?oOW0180000M;oo00<0oooo
00<0okmd0000B02Lgol0203oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo04H0oooo
00<000000?ooo`000000B03oool010000000oooo0?ooo`0000190?ooo`050000003oool0oooo0?oo
o`000000BP3oool01@000000oooo0?ooo`3oool0000003P0oooo00<000000?ooo`3oool07P3oool0
04/0oooo00X0omnL04Qd_`3ooml0W4P00018W03Oool0oooo0?oOW0180000M;oo0`3oool00`3o_g@0
001809cOo`080?ooo`030000003oool0oooo0280oooo0P0000180?ooo`030000003oool0000004P0
oooo00@000000?ooo`3oool00000B@3oool01@000000oooo0?ooo`3oool0000004X0oooo00D00000
0?ooo`3oool0oooo0000000h0?ooo`030000003oool0oooo01h0oooo001<0?ooo`090=nLB01d_ml0
W4P00018W03Oool0oooo0?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool0203oool00`000000
oooo0?ooo`0R0?ooo`800000B03oool00`000000oooo000000180?ooo`040000003oool0oooo0000
04T0oooo00D000000?ooo`3oool0oooo0000001:0?ooo`050000003oool0oooo0?ooo`000000>03o
ool00`000000oooo0?ooo`0N0?ooo`00C03oool01@3ooml0W4Q804Q80000B9`0gooo0080oooo00<0
gia80018W03Oool00P3oool00`2oM00007Bo0?ooo`080?ooo`030000003oool0oooo0280oooo0P00
00180?ooo`030000003oool0000004P0oooo00@000000?ooo`3oool00000B@3oool01@000000oooo
0?ooo`3oool0000004X0oooo00D000000?ooo`3oool0oooo0000000h0?ooo`030000003oool0oooo
01h0oooo001=0?ooo`040?oOW018000004RL0=ooo`80oooo00L0oono07@0001d_ol0oooo0?noM000
07@0_ooo00T0oooo00<000000?ooo`3oool08P3oool2000004P0oooo00<000000?ooo`000000B03o
ool010000000oooo0?ooo`0000190?ooo`050000003oool0oooo0?ooo`000000BP3oool01@000000
oooo0?ooo`3oool0000003P0oooo00<000000?ooo`3oool07P3oool004h0oooo00<0gia80018W03O
ool00`3oool01@3ookl0M0000000000007@0_ooo00X0oooo00<000000?ooo`3oool08P3oool20000
04P0oooo00<000000?ooo`000000B03oool010000000oooo0?ooo`0000190?ooo`050000003oool0
oooo0?ooo`000000BP3oool01@000000oooo0?ooo`3oool0000003P0oooo00<000000?ooo`3oool0
7P3oool006<0oooo00<000000?ooo`3oool08P3oool2000004P0oooo00<000000?ooo`000000B03o
ool010000000oooo0?ooo`0000190?ooo`050000003oool0oooo0?ooo`000000B@3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool07P3oool006<0oooo
00<000000?ooo`3oool08P3oool2000004P0oooo00<000000?ooo`000000B03oool010000000oooo
0?ooo`0000190?ooo`050000003oool0oooo0?ooo`000000B@3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo03H0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3o
ool08P3oool2000004P0oooo00<000000?ooo`000000B03oool010000000oooo0?ooo`0000190?oo
o`050000003oool0oooo0?ooo`000000B@3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo03H0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08P3oool20000
04L0oooo00@000000?ooo`3oool00000B03oool010000000oooo0?ooo`0000190?ooo`050000003o
ool0oooo0?ooo`000000B@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo03H0oooo
00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08P3oool2000004L0oooo00@0
00000?ooo`3oool00000B03oool010000000oooo0?ooo`0000190?ooo`050000003oool0oooo0?oo
o`000000B@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo03H0oooo00<000000?oo
o`3oool07P3oool006<0oooo0P00000S0?ooo`800000A`3oool010000000oooo0?ooo`0000180?oo
o`040000003oool0oooo000004T0oooo00D000000?ooo`3oool0oooo000000190?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool0=03oool300000200oooo001S0?ooo`030000003oool0
oooo0280oooo0P0000170?ooo`040000003oool0oooo000004L0oooo00D000000?ooo`3oool0oooo
000000190?ooo`050000003oool0oooo0?ooo`000000B@3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo03H0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool0
8P3oool2000004L0oooo00@000000?ooo`3oool00000A`3oool01@000000oooo0?ooo`3oool00000
04P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000
oooo0?ooo`0R0?ooo`800000A`3oool010000000oooo0?ooo`0000170?ooo`050000003oool0oooo
0?ooo`000000B03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo04L0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo01h0oooo001S0?oo
o`030000003oool0oooo0280oooo0P0000170?ooo`040000003oool0oooo000004L0oooo00D00000
0?ooo`3oool0oooo000000180?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0A`3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool07P3o
ool006<0oooo00<000000?ooo`3oool08P3oool2000004L0oooo00@000000?ooo`3oool00000A`3o
ool01@000000oooo0?ooo`3oool0000004P0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`170?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0=P3oool00`000000oooo
0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0R0?ooo`800000A`3oool010000000oooo0?oo
o`0000170?ooo`050000003oool0oooo0?ooo`000000B03oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo04L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0f0?ooo`03
0000003oool0oooo01h0oooo001S0?ooo`030000003oool0oooo0280oooo0P0000170?ooo`040000
003oool0oooo000004L0oooo00D000000?ooo`3oool0oooo000000180?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
03H0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08P3oool2000004L0
oooo00@000000?ooo`3oool00000A`3oool01@000000oooo0?ooo`3oool0000004P0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0=P3oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0R0?oo
o`800000A`3oool010000000oooo0?ooo`0000170?ooo`050000003oool0oooo0?ooo`000000B03o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo01h0oooo001S0?ooo`030000003oool0
oooo0280oooo0P0000170?ooo`040000003oool0oooo000004L0oooo00D000000?ooo`3oool0oooo
000000180?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0AP3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool07P3oool006<0oooo00<0
00000?ooo`3oool08P3oool2000004L0oooo00@000000?ooo`3oool00000A`3oool01@000000oooo
0?ooo`3oool0000004P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`160?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0N0?ooo`00
H`3oool00`000000oooo0?ooo`0R0?ooo`800000A`3oool010000000oooo0?ooo`0000170?ooo`05
0000003oool0oooo0?ooo`000000B03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
04H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo
01h0oooo001S0?ooo`8000008`3oool2000004L0oooo00@000000?ooo`3oool00000A`3oool01@00
0000oooo0?ooo`3oool0000004P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`16
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0=03oool300000200oooo001S0?oo
o`030000003oool0oooo0280oooo0P0000170?ooo`040000003oool0oooo000004L0oooo00D00000
0?ooo`3oool0oooo000000180?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0AP3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool07P3o
ool006<0oooo00<000000?ooo`3oool08P3oool2000004L0oooo00@000000?ooo`3oool00000A`3o
ool01@000000oooo0?ooo`3oool0000004P0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`160?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0=P3oool00`000000oooo
0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0R0?ooo`800000A`3oool010000000oooo0?oo
o`0000170?ooo`050000003oool0oooo0?ooo`000000B03oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo04H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0f0?ooo`03
0000003oool0oooo01h0oooo001S0?ooo`030000003oool0oooo0280oooo0P0000170?ooo`040000
003oool0oooo000004L0oooo00D000000?ooo`3oool0oooo000000180?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
03H0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08P3oool2000004L0
oooo00@000000?ooo`3oool00000A`3oool01@000000oooo0?ooo`3oool0000004P0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`160?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0=P3oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0R0?oo
o`800000A`3oool010000000oooo0?ooo`0000170?ooo`050000003oool0oooo0?ooo`000000B03o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo01h0oooo001S0?ooo`030000003oool0
oooo0280oooo0P0000170?ooo`040000003oool0oooo000004L0oooo00D000000?ooo`3oool0oooo
000000170?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0AP3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool07P3oool006<0oooo00<0
00000?ooo`3oool08P3oool2000004L0oooo00@000000?ooo`3oool00000A`3oool01@000000oooo
0?ooo`3oool0000004L0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`150?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0N0?ooo`00
H`3oool00`000000oooo0?ooo`0R0?ooo`800000A`3oool010000000oooo0?ooo`0000170?ooo`05
0000003oool0oooo0?ooo`000000A`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
04D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo
01h0oooo001S0?ooo`030000003oool0oooo0280oooo0P0000170?ooo`040000003oool0oooo0000
04L0oooo00D000000?ooo`3oool0oooo000000170?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0A@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo03H0oooo00<00000
0?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08P3oool2000004L0oooo00@000000?oo
o`3oool00000A`3oool01@000000oooo0?ooo`3oool0000004L0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`150?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0=P3o
ool00`000000oooo0?ooo`0N0?ooo`00H`3oool2000002<0oooo0P0000170?ooo`040000003oool0
oooo000004L0oooo00D000000?ooo`3oool0oooo000000170?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0A@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo03@0oooo
0`00000P0?ooo`00H`3oool00`000000oooo0?ooo`0R0?ooo`800000A`3oool010000000oooo0?oo
o`0000170?ooo`050000003oool0oooo0?ooo`000000A`3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo04D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0f0?ooo`03
0000003oool0oooo01h0oooo001S0?ooo`030000003oool0oooo0280oooo0P0000170?ooo`040000
003oool0oooo000004H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`150?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0A@3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo03H0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool0
8P3oool2000004L0oooo00@000000?ooo`3oool00000AP3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo04D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`150?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0N0?ooo`00
H`3oool00`000000oooo0?ooo`0R0?ooo`800000A`3oool010000000oooo0?ooo`0000160?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0A@3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo04D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0f0?ooo`03
0000003oool0oooo01h0oooo001S0?ooo`030000003oool0oooo0280oooo0P0000170?ooo`040000
003oool0oooo000004H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`150?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo03H0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool0
8P3oool2000004L0oooo00@000000?ooo`3oool00000AP3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo04@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`140?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0N0?ooo`00
H`3oool00`000000oooo0?ooo`0R0?ooo`800000A`3oool010000000oooo0?ooo`0000160?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo04@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0f0?ooo`03
0000003oool0oooo01h0oooo00190?ooo`030;md00000000000000@0000000T0B9cO0?ooo`3oool0
oooo0?oOW018000000000000M02oool02P3oool00`000000oooo0?ooo`0R0?ooo`800000A`3oool0
10000000oooo0?ooo`0000160?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0A03o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo01h0oooo001:0?ooo`030=nLB0000000
000000<0000000X004RL0=ooo`3oool0oono07@0001d_ol0oooo0?noM00007@0_ooo2@3oool00`00
0000oooo0?ooo`0R0?ooo`800000A`3oool010000000oooo0?ooo`0000160?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo04@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0
oooo01h0oooo001;0?ooo`030=nLB018W=l0oooo0080oooo00H0giad09cOo`3oool0_g@00018W03O
ool20?ooo`030;md0000M;l0oooo00P0oooo00<000000?ooo`3oool08P3oool2000004H0oooo00D0
00000?ooo`3oool0oooo000000160?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
A03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo01h0oooo001<0?ooo`030=nLB01d
_ol0oooo00<0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`080?ooo`030000003o
ool0oooo0280oooo0P0000160?ooo`050000003oool0oooo0?ooo`000000AP3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`140?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0=P3oool00`000000oooo
0?ooo`0N0?ooo`00?03oool00`3ooml0W4P0000000070000000304RLg`3oool0oooo00@0oooo00<0
_gA809cOo`3oool00P3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00P0oooo00<0
00000?ooo`3oool08P3oool2000004H0oooo00D000000?ooo`3oool0oooo000000160?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo04<0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0f0?ooo`030000
003oool0oooo01h0oooo001=0?ooo`030?oog`2LB7@0_ooo0080oooo00<0omnL04P0001d_ol00`3o
ool00`3o_g@0001809cOo`080?ooo`<000008P3oool2000004H0oooo00D000000?ooo`3oool0oooo
000000150?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0A03oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`0c0?ooo`@00000803oool004h0oooo00L0okmd001d_`3oool0oooo0?oOW0180000M;oo00<0
oooo00<0okmd0000B02Lgol0203oool00`000000oooo0?ooo`0R0?ooo`800000AP3oool01@000000
oooo0?ooo`3oool0000004D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`130?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo03H0oooo00<000000?ooo`3oool07P3oool004h0oooo00L0oono07@0B02L
gol0oooo0?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0203oool00`000000oooo0?ooo`0R
0?ooo`800000AP3oool01@000000oooo0?ooo`3oool0000004D0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`130?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0@`3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool07P3o
ool004h0oooo00L0oono07@00018W=l0oooo0?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool0
203oool00`000000oooo0?ooo`0R0?ooo`800000AP3oool01@000000oooo0?ooo`3oool0000004D0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo03H0
oooo00<000000?ooo`3oool07P3oool004T0oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0B000
04RLg`020?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d_`3oool0203oool00`000000oooo
0?ooo`0R0?ooo`800000AP3oool01@000000oooo0?ooo`3oool0000004D0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0@`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3o
ool07P3oool004T0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`80oooo00L0oono
07@0001d_ol0oooo0?noM00007@0_ooo00T0oooo00<000000?ooo`3oool08P3oool2000004D0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0480oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo01h0oooo
001:0?ooo`060?oog`2LB00000000000000007@0_ooo103oool01@3ookl0M0000000000007@0_ooo
00X0oooo00<000000?ooo`3oool08P3oool2000004D0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`130?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0@`3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`0f0?ooo`030000003oool0oooo01h0oooo001S0?ooo`030000003oool0oooo0280
oooo0P0000150?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0@P3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`120?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0=P3oool00`000000
oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0R0?ooo`800000A@3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`120?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0@P3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool07P3oool006<0oooo00<0
00000?ooo`3oool08P3oool2000004D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`120?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0203oool00`000000oooo0?oo
o`0f0?ooo`030000003oool0oooo01h0oooo001S0?ooo`030000003oool0oooo0280oooo0P000015
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`11
0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0N
0?ooo`00H`3oool00`000000oooo0?ooo`0R0?ooo`800000A@3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0480oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`110?oo
o`030000003oool0oooo00L0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`080?oo
o`030000003oool0oooo03H0oooo00<000000?ooo`3oool07P3oool006<0oooo0P00000S0?ooo`80
0000A@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0480oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool0@@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo03@0oooo0`00000P0?ooo`00
H`3oool00`000000oooo0?ooo`0R0?ooo`800000A@3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo0440oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`110?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo03H0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo000000150?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0@@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`100?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0=P3o
ool00`000000oooo0?ooo`0N0?ooo`00:@3oool00`3ookl0M7Bo0?ooo`040?ooo`030=nLW02oool0
oooo0300oooo00<000000?ooo`3oool08@3oool00`000000oooo000000150?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo0400oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`100?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0N0?ooo`00:@3oool00`3o
_g@0000000000004000000030000M02oool0oooo0300oooo00<000000?ooo`3oool08@3oool00`00
0000oooo000000140?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0@@3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool0203oool00`00
0000oooo0?ooo`0o0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0=P3oool00`00
0000oooo0?ooo`0N0?ooo`00:@3oool00`2oM000000000000004000000030000M02oool0oooo0300
oooo00<000000?ooo`3oool08@3oool00`000000oooo000000140?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0400
oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0N0?ooo`00:P3oool00`2oM7@0_ooo
0?ooo`030?ooo`030=nLW02oool0oooo0300oooo00<000000?ooo`3oool08@3oool00`000000oooo
000000140?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0@03oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool0203oool00`000000oooo
0?ooo`0o0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0=P3oool00`000000oooo
0?ooo`0N0?ooo`00:@3oool00`3ogi`0W;oo0?ooo`0g0?ooo`030000003oool0oooo0240oooo00<0
00000?ooo`000000A03oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0400oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool0?P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo03H0oooo00<0
00000?ooo`3oool07P3oool002T0oooo00<0_g@0001d_`3oool0103oool00`3OW9`0_ooo0?ooo`0`
0?ooo`030000003oool0oooo0240oooo00<000000?ooo`000000A03oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0o
0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool07P3oool002T0oooo00<0gia80000
00000000100000000`0007@0_ooo0?ooo`0`0?ooo`030000003oool0oooo0240oooo00<000000?oo
o`000000@`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo03l0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo00T0oooo00<000000?oo
o`3oool0?P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo03H0oooo00<000000?oo
o`3oool07P3oool002T0oooo00<0oooO09a800000000100000000`0007@0_ooo0?ooo`0`0?ooo`03
0000003oool0oooo0240oooo00<000000?ooo`000000@`3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo03l0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0n0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo03H0oooo00<000000?ooo`3oool07P3oool002X0oooo00<0gibL0;ooo`3oool0
0`3oool00`3OW9`0_ooo0?ooo`0`0?ooo`030000003oool0oooo0240oooo00<000000?ooo`000000
@`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
?@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool0
7P3oool002T0oooo00<0okmd04RLg`3oool0=`3oool200000280oooo00<000000?ooo`000000@`3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0?03o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo03@0oooo0`00000P0?ooo`00:@3oool0
0`2oM00007Bo0?ooo`040?ooo`030=nLW02oool0oooo0300oooo00<000000?ooo`3oool08@3oool0
0`000000oooo000000120?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0?P3oool0
0`000000oooo0?ooo`080?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`0l0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0=P3oool0
0`000000oooo0?ooo`0N0?ooo`00:@3oool00`3OW4P0000000000004000000030000M02oool0oooo
0300oooo00<000000?ooo`3oool08@3oool00`000000oooo000000120?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
03`0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0N0?ooo`00:@3oool00`3ooml0
W4P000000004000000030000M02oool0oooo0300oooo00<000000?ooo`3oool08@3oool00`000000
oooo000000120?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0?@3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`0k0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0=P3oool00`000000
oooo0?ooo`0N0?ooo`00<03oool00`3OW9`0_ooo0?ooo`0`0?ooo`030000003oool0oooo0240oooo
00<000000?ooo`000000@P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo03d0oooo
00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo03H0oooo
00<000000?ooo`3oool07P3oool002T0oooo00<0oono07Ad_`3oool0103oool00`3OW9`0_ooo0?oo
o`0`0?ooo`030000003oool0oooo0200oooo00@000000?ooo`3oool00000@@3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0k0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0>P3oool00`000000oooo
0?ooo`0?0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool07P3oool002<0oooo00<0
oooO09a80000M;l00`3oool00`3o_g@0000000000004000000030000M02oool0oooo0300oooo00<0
00000?ooo`3oool0803oool010000000oooo0?ooo`0000110?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo03X0oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0N0?ooo`008`3oool00`3ooml0W4P0001d
_`030?ooo`030;md00000000000000@0000000<0001d0;ooo`3oool0<03oool00`000000oooo0?oo
o`0P0?ooo`040000003oool0oooo00000440oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`0k0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0>P3oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`0f0?ooo`030000003oool0oooo01h0oooo000`0?ooo`030=nLW02oool0oooo0300oooo00<0
00000?ooo`3oool0803oool010000000oooo0?ooo`0000100?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo03T0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo0180oooo
00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo0?oo
o`0P0?ooo`040000003oool0oooo00000400oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`0j0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0>03oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`0f0?ooo`030000003oool0oooo01h0oooo001S0?ooo`030000003oool0oooo0200oooo00@0
00000?ooo`3oool00000?`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo03T0oooo
00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo03H0oooo
00<000000?ooo`3oool07P3oool002T0oooo00T0oooO09a800000000001d0;oog`2LB00000000000
M02oool0<@3oool00`000000oooo0?ooo`0P0?ooo`040000003oool0oooo000003l0oooo00<00000
0?ooo`3oool0203oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool0=`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo03D0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo01h0oooo000Y0?oo
o`080=nLM02Lgol0omnL04P00000M;l0oooo0?noM018W=lb0?ooo`030000003oool0oooo0200oooo
00@000000?ooo`3oool00000?P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo03L0
oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo03H0
oooo00<000000?ooo`3oool07P3oool002T0oooo00T0gibL0;ooo`3oool0_g@00000B02Lgol0oooo
0;mdM02oool0<@3oool200000200oooo00D000000?ooo`3oool0oooo0000000n0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo03H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0c0?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`3oool0=03oool300000200oooo000Y0?ooo`090?oOW018M;l0
oooo0?oOW018000007Bo0?ooo`2oW9`0gooo0340oooo00<000000?ooo`3oool07`3oool01@000000
oooo0?ooo`3oool0000003d0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0f0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0=03oool2000001D0oooo00<000000?oo
o`3oool0<`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo03H0oooo00<000000?oo
o`3oool07P3oool002T0oooo00D0_g@00000000004P0W=oo0=nLB0020000000307Boo`3oool0oooo
0300oooo00<000000?ooo`3oool07`3oool01@000000oooo0?ooo`3oool0000003`0oooo00<00000
0?ooo`3oool02`3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo0140oooo00<00000
0?ooo`3oool0<P3oool2000001L0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`0G
0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?oo
o`3oool07`3oool01@000000oooo0?ooo`3oool0000003/0oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`0d0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0<@3oool0
0`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0f0?ooo`030000003oool0oooo01h0oooo000Y0?ooo`030?oo_`1dM;l0oooo
00@0oooo00<0gibL0;ooo`3oool0<03oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
03<0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
03H0oooo00<000000?ooo`3oool07P3oool002T0oooo00<0okmd000000000000100000000`0007@0
_ooo0?ooo`0`0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0f0?ooo`800000403oool00`000000oooo0?ooo`0a0?ooo`8000005P3oool00`000000
oooo0?ooo`0_0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0;P3oool2000001`0
oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0N0?ooo`00:@3oool00`2oM0000000
00000004000000030000M02oool0oooo0300oooo00<000000?ooo`3oool07@3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0`0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0;P3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`0/0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool07P3oool002X0oooo00<0
_gAd0;ooo`3oool00`3oool00`3OW9`0_ooo0?ooo`0Q0?ooo`050?oOW018000000000000M02oool0
2P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
=03oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo02h0oooo0P00000I0?ooo`030000
003oool0oooo02d0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0/0?ooo`030000
003oool0oooo01d0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo01h0oooo000Y0?ooo`030?oOW02L_ol0oooo02L0oooo00L0oono07@0001d_ol0oooo
0?noM00007@0_ooo00T0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo03@0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0/0?ooo`80
00006`3oool00`000000oooo0?ooo`0[0?ooo`8000007P3oool00`000000oooo0?ooo`0[0?ooo`03
0000003oool0oooo01h0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`080?ooo`03
0000003oool0oooo01h0oooo000Y0?ooo`030;md0000M;l0oooo00@0oooo00<0gibL0;ooo`3oool0
803oool00`2oM00004RL0=ooo`020?ooo`030;md0000M;l0oooo00P0oooo00<000000?ooo`3oool0
703oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0380oooo0P00000D0?ooo`030000
003oool0oooo02X0oooo0P00000M0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0
7P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0
:@3oool2000000/0oooo00<000000?ooo`3oool07P3oool002T0oooo00<0gia80000000000001000
00000`0007@0_ooo0?ooo`0O0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool0203o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0<03o
ool2000001H0oooo00<000000?ooo`3oool0:03oool2000001l0oooo00<000000?ooo`3oool0:03o
ool200000240oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0P0?ooo`030000003o
ool0oooo02P0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0N0?ooo`00:@3oool0
0`3ooml0W4P000000004000000030000M02oool0oooo01l0oooo00<0omnL04P0001d_ol00`3oool0
0`3o_g@0001809cOo`080?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`0]0?ooo`<00000603oool00`000000oooo0?ooo`0V0?ooo`8000008@3oool0
0`000000oooo0?ooo`0V0?ooo`8000008`3oool00`000000oooo0?ooo`0W0?ooo`8000008`3oool0
0`000000oooo0?ooo`0W0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool07P3oool0
02X0oooo00<0gibL0;ooo`3oool00`3oool00`3OW9`0_ooo0?ooo`0O0?ooo`030?oOW0180000M;oo
00<0oooo00<0okmd0000B02Lgol0203ooooo000009D00000803oool002T0oooo00<0okmd04RLg`3o
ool09P3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00P0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo02D0oooo1000000O0?ooo`03
0000003oool0oooo02<0oooo0P00000T0?ooo`030000003oool0oooo02<0oooo0P00000V0?ooo`03
0000003oool0oooo02@0oooo0P00000V0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3o
ool03`3oool00`000000oooo0?ooo`0N0?ooo`00:@3oool00`2oM00007Bo0?ooo`040?ooo`030=nL
W02oool0oooo01l0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`080?ooo`030000
003oool0oooo01D0oooo0`00000;0?ooo`030000003oool0oooo0240oooo1000000S0?ooo`030000
003oool0oooo0200oooo0`00000V0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0
9P3oool00`000000oooo0?ooo`0R0?ooo`800000:03oool00`000000oooo0?ooo`0S0?ooo`030000
003oool0oooo0100oooo00<000000?ooo`3oool07P3oool002T0oooo00<0gia80000000000001000
00000`0007@0_ooo0?ooo`0O0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool0203o
ool00`000000oooo0?ooo`0A0?ooo`@000003P3oool00`000000oooo0?ooo`0M0?ooo`@000009`3o
ool00`000000oooo0?ooo`0M0?ooo`<00000:@3oool00`000000oooo0?ooo`0P0?ooo`800000:@3o
ool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool08@3o
ool2000001<0oooo00<000000?ooo`3oool07P3oool002T0oooo00<0oooO09a80000000010000000
0`0007@0_ooo0?ooo`0P0?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d_`3oool0203oool0
0`000000oooo0?ooo`0=0?ooo`@000004P3oool00`000000oooo0?ooo`0I0?ooo`@00000:`3oool0
0`000000oooo0?ooo`0K0?ooo`800000;03oool00`000000oooo0?ooo`0N0?ooo`800000:`3oool0
0`000000oooo0?ooo`0O0?ooo`800000:`3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool07P3oool00300oooo00<0gibL0;ooo`3oool0803oool01`3o
okl0M00007Boo`3oool0okmd0000M02oool02@3oool00`000000oooo0?ooo`090?ooo`@000005P3o
ool00`000000oooo0?ooo`0F0?ooo`<00000;`3oool00`000000oooo0?ooo`0I0?ooo`800000;P3o
ool00`000000oooo0?ooo`0L0?ooo`800000;@3oool00`000000oooo0?ooo`0N0?ooo`030000003o
ool0oooo02/0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0D0?ooo`030000003o
ool0oooo01h0oooo001D0?ooo`050?oo_`1d000000000000M02oool02P3oool00`000000oooo0?oo
o`060?ooo`<000006P3oool00`000000oooo0?ooo`0C0?ooo`<00000<P3oool00`000000oooo0?oo
o`0G0?ooo`800000<03oool00`000000oooo0?ooo`0J0?ooo`800000;`3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool07@3oool2000001L0oooo00<0
00000?ooo`3oool07P3oool002X0oooo00L0_g@00000B02Lgol0oono07@000000000B9cO0380oooo
00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0A0?ooo`800000=@3oool00`000000oooo
0?ooo`0E0?ooo`800000<P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo02l0oooo
00<000000?ooo`3oool06`3oool2000002l0oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo01h0oooo000Y0?ooo`050?noM0000000001809cOo`2oM000
0P0000000`0007@0_ooo0?ooo`0`0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0
3P3oool3000003L0oooo00<000000?ooo`3oool04`3oool2000003@0oooo00<000000?ooo`3oool0
5`3oool200000380oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0_0?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0N0?ooo`00:@3o
ool02@2oM7@0_ooo0?ooo`3ookl0M4RL0=ooo`3ogi`0B4RL0=ooo`0a0?ooo`030000003oool0oooo
02<0oooo00<000000?ooo`3oool02`3oool3000003X0oooo00<000000?ooo`3oool04@3oool20000
03H0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool07P3oool002T0oooo00P0_gA809cOo`3oool0giad09cOo`3oool0
oknL09cOoc80oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`090?ooo`800000?@3o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool05@3o
ool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0<@3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool07P3o
ool002T0oooo00<0gia8000000000000100000000`00B9`0gooo0?ooo`0`0?ooo`030000003oool0
oooo02<0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0m0?ooo`030000003oool0
oooo00h0oooo0P00000i0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0=03oool0
0`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01h0oooo000Y0?ooo`030?oo_`1d00000000
00@0000000<004RL0=ooo`3oool0<03oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
03T0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
01`0oooo00<000000?ooo`3oool07P3oool002l0oooo00<0_gA809cOo`3oool0<@3oool2000002@0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo03H0
oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo01H0
oooo00<000000?ooo`3oool06`3oool300000200oooo000Y0?ooo`030;mdM02oool0oooo03L0oooo
00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0400oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo0140oooo
00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo03D0oooo
00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01h0oooo
000W0?ooo`030?oo_`1d0000000000H0000000<0B9cO0?ooo`3oool0<03oool00`000000oooo0?oo
o`0S0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`0h0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?oo
o`0E0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool07P3oool002H0oooo00<0okmd
0000000000001`0000000`0007@0_ooo0?ooo`0`0?ooo`030000003oool0oooo02<0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool0>`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo03T0oooo00<00000
0?ooo`3oool04P3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool07`3oool00`000000oooo0?ooo`0N0?ooo`00:@3oool00`2oM7@0_ooo0?ooo`030?oo
o`030?oo_`1dB9`0gooo0340oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo0480oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0l0?oo
o`030000003oool0oooo00h0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo03P0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0P0?oo
o`030000003oool0oooo01h0oooo000Y0?ooo`030;mdM02oool0oooo00<0oooo00<0_gA809cOo`3o
ool0<@3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool0@`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool0>@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3o
ool07P3oool006<0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo04<0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0m0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo03T0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo01h0oooo000Z0?ooo`030?oog`2LB00000000080000000<0M;oo0?ooo`3oool0<@3o
ool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0@`3o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0>P3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool07P3o
ool002T0oooo00<0oono07@000000000100000000`18W=l0oooo0?ooo`0`0?ooo`030000003oool0
oooo02<0oooo00D000000?ooo`3oool0oooo000000160?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool0?`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo03`0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`3oool08`3oool00`000000oooo0?ooo`0N0?ooo`00:@3oool00`3OW4P0B9cO0?ooo`02
0?ooo`040?noM0000000001d0;oooc40oooo00<000000?ooo`3oool08`3oool01@000000oooo0?oo
o`3oool0000004H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0o0?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo03`0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0S0?ooo`030000
003oool0oooo01h0oooo000Y0?ooo`030;mdM02oool0oooo00<0oooo00<0gia80000M02oool0<@3o
ool00`000000oooo0?ooo`0S0?ooo`050000003oool0oooo0?ooo`000000AP3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0n0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0?03oool00`000000oooo
0?ooo`0?0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool07P3oool002T0oooo00<0
gia804RLg`3oool00`3oool00`2oM00007Bo0?ooo`0a0?ooo`030000003oool0oooo02<0oooo00D0
00000?ooo`3oool0oooo000000160?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
@03oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0
303oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
903oool00`000000oooo0?ooo`0N0?ooo`00:@3oool01`3ooml0W4P00000M02oool0gia800000018
W=l0<`3oool2000002@0oooo00D000000?ooo`3oool0oooo000000160?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
03l0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool08`3oool300000200oooo001S0?ooo`030000003oool0oooo02<0
oooo00@000000?ooo`3oool00000A`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
0440oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo
00`0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
02D0oooo00<000000?ooo`3oool07P3oool002@0oooo00<0oknL09cOo`3oool02@3oool00`3OW9`0
_ooo0?ooo`0`0?ooo`030000003oool0oooo02<0oooo00@000000?ooo`3oool00000A`3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`0o0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0?P3oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool07P3oool002@0
oooo00<0_g@00000000000002@0000000`0007@0_ooo0?ooo`0`0?ooo`030000003oool0oooo02<0
oooo00@000000?ooo`3oool00000A`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
0440oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
02H0oooo00<000000?ooo`3oool07P3oool002<0oooo00<0oono07@0000000002P0000000`0007@0
_ooo0?ooo`0`0?ooo`030000003oool0oooo02<0oooo00@000000?ooo`3oool00000A`3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0203oool00`00
0000oooo0?ooo`100?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0?`3oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool07P3oool002X0
oooo00<0gibL0;ooo`3oool00`3oool00`3OW9`0_ooo0?ooo`0`0?ooo`030000003oool0oooo02<0
oooo00@000000?ooo`3oool00000A`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
0480oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
02L0oooo00<000000?ooo`3oool07P3oool002T0oooo00<0okmd04RLg`3oool0=`3oool00`000000
oooo0?ooo`0S0?ooo`040000003oool0oooo000004L0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`120?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0@@3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`0W0?ooo`030000003oool0oooo01h0oooo000Y0?ooo`030;md0000M;l0oooo00@0
oooo00<0gibL0;ooo`3oool0<03oool00`000000oooo0?ooo`0S0?ooo`040000003oool0oooo0000
04L0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
0400oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo
01h0oooo000Y0?ooo`030=nLB0000000000000@0000000<0001d0;ooo`3oool0<03oool00`000000
oooo0?ooo`0S0?ooo`040000003oool0oooo000004L0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`120?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0@@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`0X0?ooo`030000003oool0oooo01h0oooo000Y0?ooo`030?oog`2LB000000000@0
000000<0001d0;ooo`3oool0<03oool00`000000oooo0?ooo`0S0?ooo`030000003oool0000004P0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0400
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo01h0
oooo000`0?ooo`030=nLW02oool0oooo0300oooo00<000000?ooo`3oool08`3oool00`000000oooo
000000180?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0@`3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool0203oool00`000000oooo
0?ooo`110?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0:@3oool00`000000oooo
0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0000004P0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool0@P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0440oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo01h0oooo001S
0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`000000B03oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`12
0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool07P3oool006<0oooo0P00000T0?oo
o`030000003oool0000004P0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`130?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo0480oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0X0?oo
o`<00000803oool006<0oooo00<000000?ooo`3oool08`3oool00`000000oooo000000180?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo04<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`120?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0N0?ooo`00
H`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0000004P0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
@`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo01h0oooo001S0?ooo`030000003o
ool0oooo02<0oooo00<000000?ooo`000000B03oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo04@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`130?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo02X0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08`3oool0
0`000000oooo000000180?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0A03oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`130?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0:`3oool0
0`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool00000
04P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
04<0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo
01h0oooo001S0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`000000B03oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`140?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0@`3oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool07P3oool004T0oooo
00<0_g@0000000000000100000002@18W=l0oooo0?ooo`3oool0omnL04P000000000001d0;ooo`0:
0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`000000B03oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`14
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`08
0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool07P3oool004X0oooo00<0gia80000
000000000`0000002P00B9`0gooo0?ooo`3ookl0M00007Boo`3oool0okmd0000M02oool90?ooo`03
0000003oool0oooo02<0oooo00<000000?ooo`000000B03oool01@000000oooo0?ooo`3oool00000
04L0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
02/0oooo00<000000?ooo`3oool07P3oool004/0oooo00<0gia804RLg`3oool00P3oool01P3OW7@0
W=oo0?ooo`2oM00004RL0=ooo`80oooo00<0_g@0001d_`3oool0203oool00`000000oooo0?ooo`0S
0?ooo`030000003oool0000004P0oooo00D000000?ooo`3oool0oooo000000170?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo04@0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0/0?ooo`030000003o
ool0oooo01h0oooo001<0?ooo`030=nLB01d_ol0oooo00<0oooo00<0oono07@00018W=l00`3oool0
0`3OW4P0001d0;ooo`080?ooo`030000003oool0oooo02<0oooo00<000000?ooo`000000B03oool0
1@000000oooo0?ooo`3oool0000004L0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`140?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool07P3oool004d0oooo00<0_gA8
09cOo`3oool00P3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00P0oooo00<00000
0?ooo`3oool08`3oool00`000000oooo000000180?ooo`050000003oool0oooo0?ooo`000000A`3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`140?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0;03o
ool00`000000oooo0?ooo`0N0?ooo`00C@3oool00`3ooml0W4Qd0;ooo`020?ooo`030?oOW0180000
M;oo00<0oooo00<0okmd0000B02Lgol0203oool3000002<0oooo00<000000?ooo`000000B03oool0
1@000000oooo0?ooo`3oool0000004L0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`150?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo02T0oooo1000000P0?ooo`00CP3oool01`3o_g@007Bo0?ooo`3o
ool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`080?ooo`030000003oool0oooo02<0oooo
00<000000?ooo`000000B03oool01@000000oooo0?ooo`3oool0000004L0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0A03oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3o
ool07P3oool004h0oooo00L0oono07@0B02Lgol0oooo0?oOW0180000M;oo00<0oooo00<0okmd0000
B02Lgol0203oool00`000000oooo0?ooo`0S0?ooo`030000003oool0000004P0oooo00D000000?oo
o`3oool0oooo000000170?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0A@3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`0/0?ooo`030000003oool0oooo01h0oooo001>0?ooo`070?oo_`1d0000B9cO
0?ooo`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00P0oooo00<000000?ooo`3oool08`3o
ool00`000000oooo000000180?ooo`050000003oool0oooo0?ooo`000000A`3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`140?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0;@3oool00`000000oooo
0?ooo`0N0?ooo`00B@3oool00`3o_i`0W=oo0?ooo`020?ooo`030?oOW0180000B9cO0080oooo00<0
gia80018W03Oool00P3oool00`2oM00007Bo0?ooo`080?ooo`030000003oool0oooo02<0oooo00<0
00000?ooo`000000B03oool01@000000oooo0?ooo`3oool0000004L0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
A03oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0
7P3oool004T0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`80oooo00L0oono07@0
001d_ol0oooo0?noM00007@0_ooo00T0oooo00<000000?ooo`3oool08`3oool00`000000oooo0000
00180?ooo`050000003oool0oooo0?ooo`000000A`3oool00`000000oooo0?ooo`030?ooo`030000
003oool0oooo04D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`140?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0N0?ooo`00BP3o
ool01P3ooml0W4P0000000000000001d0;ooo`@0oooo00D0oono07@000000000001d0;ooo`0:0?oo
o`030000003oool0oooo02<0oooo0P0000190?ooo`050000003oool0oooo0?ooo`000000A`3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`140?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0;@3oool0
0`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0S0?ooo`800000B@3oool01@00
0000oooo0?ooo`3oool0000004L0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`15
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?oo
o`3oool08`3oool2000004T0oooo00D000000?ooo`3oool0oooo000000170?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool0A@3oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo04@0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0
oooo01h0oooo001S0?ooo`030000003oool0oooo02<0oooo0P0000190?ooo`040000003oool0oooo
000004P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`160?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo02h0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08`3oool20000
04T0oooo00@000000?ooo`3oool00000B03oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo04H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`140?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`00
0000oooo0?ooo`0S0?ooo`800000B@3oool010000000oooo0?ooo`0000180?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo04D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0
oooo01h0oooo001S0?ooo`030000003oool0oooo02<0oooo0P0000190?ooo`040000003oool0oooo
000004P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`160?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0A@3oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo02h0oooo00<000000?ooo`3oool07P3oool006<0oooo0P00000T0?ooo`800000B@3oool01000
0000oooo0?ooo`0000180?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0AP3oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`0/0?ooo`<00000803oool006<0oooo00<000000?ooo`3oool08`3oool20000
04T0oooo00@000000?ooo`3oool00000B03oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo04H0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`150?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`00
0000oooo0?ooo`0S0?ooo`800000B@3oool010000000oooo0?ooo`0000180?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo04D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0
oooo01h0oooo001S0?ooo`030000003oool0oooo02<0oooo0P0000190?ooo`040000003oool0oooo
000004P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`160?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0A@3oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo02h0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08`3oool20000
04T0oooo00@000000?ooo`3oool00000B03oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo04H0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`150?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`00
0000oooo0?ooo`0S0?ooo`800000B@3oool010000000oooo0?ooo`0000180?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo04D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0_0?ooo`030000003oool0
oooo01h0oooo001S0?ooo`030000003oool0oooo02<0oooo0P0000190?ooo`040000003oool0oooo
000004P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`160?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0A@3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo02l0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08`3oool20000
04T0oooo00@000000?ooo`3oool00000B03oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo04H0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`150?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`00
0000oooo0?ooo`0S0?ooo`800000B@3oool010000000oooo0?ooo`0000180?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo04H0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0_0?ooo`030000003oool0
oooo01h0oooo001S0?ooo`030000003oool0oooo02<0oooo0P0000190?ooo`040000003oool0oooo
000004P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`160?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo02l0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08`3oool20000
04T0oooo00@000000?ooo`3oool00000B03oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo04H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`160?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`00
0000oooo0?ooo`0S0?ooo`800000B@3oool010000000oooo0?ooo`0000180?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo04H0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0_0?ooo`030000003oool0
oooo01h0oooo001S0?ooo`800000903oool2000004T0oooo00@000000?ooo`3oool00000B03oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`160?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0;@3oool3
00000200oooo001S0?ooo`030000003oool0oooo02<0oooo0P0000190?ooo`040000003oool0oooo
000004P0oooo00D000000?ooo`3oool0oooo000000190?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0AP3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0300oooo00<0
00000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08`3oool2000004T0oooo00@00000
0?ooo`3oool00000B03oool01@000000oooo0?ooo`3oool0000004T0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`160?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
<03oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0S0?ooo`800000B@3o
ool010000000oooo0?ooo`0000180?ooo`050000003oool0oooo0?ooo`000000B@3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`0`0?ooo`030000003oool0oooo01h0oooo001S0?ooo`030000003oool0oooo02<0oooo
0P0000190?ooo`040000003oool0oooo000004P0oooo00D000000?ooo`3oool0oooo000000190?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo0300oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3o
ool08`3oool2000004T0oooo00@000000?ooo`3oool00000B03oool01@000000oooo0?ooo`3oool0
000004T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`160?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`00
0000oooo0?ooo`0S0?ooo`800000B@3oool010000000oooo0?ooo`0000180?ooo`050000003oool0
oooo0?ooo`000000B@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo04L0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo01h0oooo001S
0?ooo`030000003oool0oooo02<0oooo0P0000190?ooo`030000003oool0000004T0oooo00D00000
0?ooo`3oool0oooo000000190?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0A`3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool07P3o
ool006<0oooo00<000000?ooo`3oool08`3oool2000004T0oooo00<000000?ooo`000000B@3oool0
1@000000oooo0?ooo`3oool0000004T0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`170?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?oo
o`0N0?ooo`00H`3oool00`000000oooo0?ooo`0S0?ooo`800000B@3oool00`000000oooo00000019
0?ooo`050000003oool0oooo0?ooo`000000B@3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo04L0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0`0?ooo`030000003o
ool0oooo01h0oooo001S0?ooo`030000003oool0oooo02<0oooo0P0000190?ooo`030000003oool0
000004T0oooo00D000000?ooo`3oool0oooo000000190?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool0A`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0300oooo00<0
00000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08`3oool2000004T0oooo00<00000
0?ooo`000000B@3oool01@000000oooo0?ooo`3oool0000004T0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`170?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0<03o
ool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0S0?ooo`800000B@3oool0
0`000000oooo000000190?ooo`050000003oool0oooo0?ooo`000000B@3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`0`0?ooo`030000003oool0oooo01h0oooo001S0?ooo`800000903oool2000004T0oooo00<00000
0?ooo`000000B@3oool01@000000oooo0?ooo`3oool0000004T0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`170?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0;P3o
ool300000200oooo001S0?ooo`030000003oool0oooo02<0oooo0P0000190?ooo`030000003oool0
000004T0oooo00D000000?ooo`3oool0oooo000000190?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool0A`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0300oooo00<0
00000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08`3oool2000004T0oooo00<00000
0?ooo`000000B@3oool01@000000oooo0?ooo`3oool0000004T0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`170?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0<@3o
ool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0S0?ooo`800000B@3oool0
0`000000oooo000000190?ooo`050000003oool0oooo0?ooo`000000B@3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0a0?ooo`030000003oool0oooo01h0oooo001S0?ooo`030000003oool0oooo02<0oooo0P000019
0?ooo`030000003oool0000004T0oooo00D000000?ooo`3oool0oooo000000190?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo0340oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08`3oool2
000004T0oooo00<000000?ooo`000000B@3oool01@000000oooo0?ooo`3oool0000004T0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool0<@3oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0S
0?ooo`800000B@3oool00`000000oooo000000190?ooo`050000003oool0oooo0?ooo`000000B@3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01h0oooo001=0?ooo`040?oog`2LB000
04RL0=ooo`<0oooo00D0omnL04P000000000001d0;ooo`0:0?ooo`030000003oool0oooo02<0oooo
0P0000190?ooo`030000003oool0000004T0oooo00D000000?ooo`3oool0oooo000000190?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0340oooo00<000000?ooo`3oool07P3oool004d0oooo00@0oooO09a80000B9`0
gooo0P3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool02@3oool00`000000oooo0?ooo`0S
0?ooo`800000B@3oool00`000000oooo000000190?ooo`050000003oool0oooo0?ooo`000000B@3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01h0oooo001=0?ooo`040?oog`2LB000
04RL0=ooo`80oooo00<0_g@00018W03Oool00P3oool00`2oM00007Bo0?ooo`080?ooo`030000003o
ool0oooo02<0oooo0P0000190?ooo`030000003oool0000004T0oooo00@000000?ooo`3oool00000
BP3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01h0oooo00190?ooo`030;md0000
0000000000D0000000@0M;oo0?oo_`1d0000B9cO0`3oool00`3OW4P0001d0;ooo`080?ooo`030000
003oool0oooo02<0oooo0P0000190?ooo`030000003oool0000004T0oooo00@000000?ooo`3oool0
0000BP3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01h0oooo00190?ooo`0<0?oO
W01dW=l0oooo0?ooo`3ooml0W4P00018W03Oool0oooo0?oOW0180000M;oo0`3oool00`3o_g@00018
09cOo`080?ooo`030000003oool0oooo02<0oooo0P0000190?ooo`030000003oool0000004T0oooo
00@000000?ooo`3oool00000BP3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo04L0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01h0
oooo001:0?ooo`0;0=nLB01d_ol0oooo0?oog`2LB00004RL0=ooo`3oool0omnL04P0001d_ol00`3o
ool00`3o_g@0001809cOo`080?ooo`<000008`3oool2000004T0oooo00<000000?ooo`000000B@3o
ool010000000oooo0?ooo`00001:0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
A`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo02h0oooo1000000P0?ooo`00BP3o
ool02`3ooml0W4Qd0;ooo`3ooml0W4P00018W03Oool0oooo0?oOW0180000M;oo00<0oooo00<0okmd
0000B02Lgol0203oool00`000000oooo0?ooo`0S0?ooo`800000B@3oool00`000000oooo00000019
0?ooo`040000003oool0oooo000004X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`170?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?oo
o`0N0?ooo`00B`3oool02P3ogi`0B7Bo0?oog`2LB00004RL0=ooo`3oool0omnL04P0001d_ol30?oo
o`030?noM00004P0W=oo00P0oooo00<000000?ooo`3oool08`3oool2000004T0oooo00<000000?oo
o`000000B@3oool010000000oooo0?ooo`00001:0?ooo`050000003oool0oooo0?ooo`000000BP3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool07P3o
ool004`0oooo00T0gia807Bog`2LB00004RL0=ooo`3oool0oono07@00018W=l00`3oool00`3OW4P0
001d0;ooo`080?ooo`030000003oool0oooo02<0oooo0P0000190?ooo`030000003oool0000004T0
oooo00@000000?ooo`3oool00000BP3oool01@000000oooo0?ooo`3oool0000004X0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01h0oooo001<0?oo
o`050?oog`2LB4P0B4P00018W03Oool00P3oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0
oooo00P0oooo00<000000?ooo`3oool08`3oool2000004T0oooo00<000000?ooo`000000B@3oool0
10000000oooo0?ooo`00001:0?ooo`050000003oool0oooo0?ooo`000000BP3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool07P3oool004d0oooo00@0
omnL04P00000B9`0gooo0P3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool02@3oool00`00
0000oooo0?ooo`0S0?ooo`800000B@3oool00`000000oooo000000190?ooo`040000003oool0oooo
000004X0oooo00D000000?ooo`3oool0oooo0000001:0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool0<@3oool00`000000oooo0?ooo`0N0?ooo`00CP3oool00`3OW4P004RL0=ooo`03
0?ooo`050?oo_`1d000000000000M02oool02P3oool00`000000oooo0?ooo`0S0?ooo`800000B@3o
ool00`000000oooo000000190?ooo`040000003oool0oooo000004X0oooo00D000000?ooo`3oool0
oooo0000001:0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0<@3oool00`000000
oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0S0?ooo`800000B@3oool00`000000oooo
000000190?ooo`040000003oool0oooo000004X0oooo00D000000?ooo`3oool0oooo0000001:0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`0N0?oo
o`00H`3oool00`000000oooo0?ooo`0S0?ooo`800000B@3oool00`000000oooo000000190?ooo`04
0000003oool0oooo000004X0oooo00D000000?ooo`3oool0oooo0000001:0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`00
0000oooo0?ooo`0S0?ooo`800000B@3oool00`000000oooo000000190?ooo`040000003oool0oooo
000004X0oooo00D000000?ooo`3oool0oooo0000001:0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool0<@3oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0S
0?ooo`800000B@3oool00`000000oooo000000190?ooo`040000003oool0oooo000004X0oooo00D0
00000?ooo`3oool0oooo0000001:0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
<@3oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo0?ooo`0S0?ooo`800000B@3o
ool00`000000oooo000000190?ooo`040000003oool0oooo000004X0oooo00D000000?ooo`3oool0
oooo0000001:0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0<@3oool00`000000
oooo0?ooo`0N0?ooo`00H`3oool2000002@0oooo0P0000190?ooo`030000003oool0000004T0oooo
00@000000?ooo`3oool00000BP3oool01@000000oooo0?ooo`3oool0000004X0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`0_0?ooo`<00000803oool006<0oooo00<000000?ooo`3o
ool08`3oool2000004T0oooo00<000000?ooo`000000B@3oool010000000oooo0?ooo`00001:0?oo
o`050000003oool0oooo0?ooo`000000BP3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0340oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08`3oool20000
04T0oooo00<000000?ooo`000000B@3oool010000000oooo0?ooo`00001:0?ooo`050000003oool0
oooo0?ooo`000000BP3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0340oooo00<0
00000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08`3oool2000004T0oooo00<00000
0?ooo`000000B@3oool010000000oooo0?ooo`00001:0?ooo`050000003oool0oooo0?ooo`000000
BP3oool01@000000oooo0?ooo`3oool0000003@0oooo00<000000?ooo`3oool07P3oool006<0oooo
00<000000?ooo`3oool08`3oool2000004T0oooo00<000000?ooo`000000B@3oool010000000oooo
0?ooo`00001:0?ooo`050000003oool0oooo0?ooo`000000BP3oool01@000000oooo0?ooo`3oool0
000003@0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08`3oool20000
04T0oooo00<000000?ooo`000000B@3oool010000000oooo0?ooo`00001:0?ooo`050000003oool0
oooo0?ooo`000000BP3oool01@000000oooo0?ooo`3oool0000003@0oooo00<000000?ooo`3oool0
7P3oool006<0oooo00<000000?ooo`3oool08`3oool2000004T0oooo00<000000?ooo`000000B@3o
ool010000000oooo0?ooo`00001:0?ooo`050000003oool0oooo0?ooo`000000BP3oool01@000000
oooo0?ooo`3oool0000003@0oooo00<000000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3o
ool08`3oool2000004T0oooo00<000000?ooo`000000B@3oool010000000oooo0?ooo`00001:0?oo
o`050000003oool0oooo0?ooo`000000BP3oool01@000000oooo0?ooo`3oool0000003@0oooo00<0
00000?ooo`3oool07P3oool006<0oooo00<000000?ooo`3oool08`3oool2000004T0oooo0P00001:
0?ooo`040000003oool0oooo000004X0oooo00D000000?ooo`3oool0oooo0000001:0?ooo`050000
003oool0oooo0?ooo`000000=03oool00`000000oooo0?ooo`0N0?ooo`00H`3oool00`000000oooo
0?ooo`0S0?ooo`800000B@3oool2000004X0oooo00@000000?ooo`3oool00000BP3oool01@000000
oooo0?ooo`3oool0000004X0oooo00D000000?ooo`3oool0oooo0000000d0?ooo`030000003oool0
oooo01h0oooo001S0?ooo`030000003oool0oooo02<0oooo0P00000:0?ooo`030000003oool0oooo
02l0oooo00<000000?ooo`3oool02P3oool2000002@0oooo00<000000?ooo`3oool08`3oool01000
0000oooo0?ooo`0000080?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0303oool0
1@000000oooo0?ooo`3oool0000001h0oooo00<000000?ooo`3oool0:@3oool01@000000oooo0?oo
o`3oool000000080oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0N0?ooo`00H`3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`050?ooo`<000002@3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`02000000D0oooo00<0
00000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool01@3oool010000000oooo0000000000080?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0080oooo
00@000000?ooo`3oool0oooo0P0000090?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool0203oool00`000000oooo0?ooo`070?ooo`040000003oool0oooo000000<0oooo00@000000?oo
o`3oool000002@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01h0oooo001S0?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?oo
o`030000003oool0oooo00D0oooo0`0000090?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo008000001@3oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`050?ooo`040000003oool00000000000P0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool00P3oool0
10000000oooo0?ooo`3oool2000000T0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo00L0oooo00@000000?ooo`3oool000000`3oool010000000oooo
0?ooo`0000090?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool07P3oool006<0oooo
o`00002E00000200oooo003o0?ooool0oooo6@3oool00?l0ooooo`3ooolI0?ooo`00o`3ooooo0?oo
oaT0oooo003o0?ooool0oooo6@3oool00?l0ooooo`3ooolI0?ooo`00o`3ooooo0?oooaT0oooo003o
0?ooool0oooo6@3oool00?T0oooo00<0oooO09a8000000000`0000000`18W=l0oooo0?ooo`3o0?oo
oaH0oooo003i0?ooo`030;md0000B9`0gooo00<0oooo00<0gibL0;ooo`3oool0o`3ooolF0?ooo`00
n@3oool00`3ooml0W7BL0=ooo`030?ooo`030?oo_`2LW=l0oooo0?l0oooo5P3oool00?X0oooo00<0
_g@00000000000000`0000000`18W=l0oooo0?ooo`3o0?oooaD0oooo00300?ooo`060=nLB0000000
00000000B02Lgml0W4P00P0000000`0007@0_ooO09a8000200000003001d_`3ooml0W4P000800000
00T004RL09ad0000000007Bo0?ooo`3ogi`0B00000000018W=l00P3oool01`2oM000000000000000
04P0W=oo0?ooo`3OW4P00P0000000`0004P0W=oO09a8000200000006001d_`3OW4P0000000000000
07@0M7@00P0000000`0007@0_ooO09a80002000000030018W03Ookl0M00000@0000000<0001d0;oo
o`3oool01`3oool01@2oM00000000000000004P0W=oo00<0oooo00D0gia8000000000000000007Bo
o`020?ooo`060;md0000000000000000M02ooml0W4P00P0000001000B9`0gooo0?oog`2LB0020000
00050000B018B00004RL0=ooo`3OW4P00P0000001`0007@0_ooo0?oOW0180000000004RLg`3OW4P0
0P000000100007@0_ooo0?ooo`3OW4P30000000407Boo`3oool0oooo0;md0080000000<0001d0;oo
g`2LB0000P0000000`00B9`0gooo0?ooo`040?ooo`030?oOW018000000000080000000@0M;oo0?oo
o`3oool0gia80`000000101d_ol0oooo0?ooo`2oM002000000030000B02Lgol0oooo00@0oooo00H0
gia8000000000000001809cOg`2LB002000000030000M02ooml0W4P00080000000<007Bo0?ooo`3o
ool0303oool01@3ogi`0B0000000000007@0_ooo00<0oooo00H0okmd0000001d_ol0oooo0?oog`2L
B0030000000307Boo`3oool0oooo07l0oooo00310?ooo`030=nLB00007@0_ooo0080oooo02D0_g@0
0018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3o_g@0000007Boo`3oool0omnL04P00000M;l0okno
0;ooo`3OW4P0001d0;ooo`3O_kl0gooo0;md0000000004RL0=ooo`3ooml0W4Qd0;ooo`3oool0gia8
0000M02oool0oooo0?oog`2LB00007Bo0080oooo00/0gia80000M02oool0oooo0=nLB000B9`0gooo
0?ooo`3ooml0W4P0001d_`020?ooo`030;mdB02Lgol0oooo00/0oooo00l0_g@000000000B9`0gooo
0?oog`2LB7@0_ooo0?ooo`3OW4P0000004RLg`3oool0oooO09a80018W=l00P3oool09P3OW4P004RL
0=ooo`3oool0oooO09a80000M;l0oooo0?oog`2LB000000007Boo`3oool0oono07@000000000001d
0;ooo`3OW4P0001d0;ooo`3oool0gia80000M02oool0gkno0=ooo`3OW4P0001d0;ooo`3oool0gia8
00000018W=l0oooo0?oog`2LB000B9cO0P3oool01`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0
203oool00`3ooml0W4P0001d_`030?ooo`070=nLB0000000B9cO0?ooo`3ooml0W4P004RLg`020?oo
o`030=nLB00007@0_ooo00L0oooo00<0gia80000M02oool00P3oool01`2oM00004RL0=ooo`3oool0
oooO09a80000M;l03P3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool00P3oool01`3o_g@0
000007Boo`3oool0oooO09a8M02oool00P3oool00`2oM000M;oo0?ooo`1o0?ooo`00`@3oool00`3O
W4P0001d0;ooo`020?ooo`0?0;md0000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd0000B02L
gol0oooo0?oog`2LB00007Bo0080oooo00L0gia80018W03Oool0oooo0?oOW018000007Bo00<0oooo
00X0omnL09boo`3oool0gia80000M02oool0oooo0?oog`2LB00007Bo0P3oool02`3OW4P0001d0;oo
o`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo0080oooo00D0oono07@0000000000000001d
_`080?ooo`030?oOW018000007Bo00<0oooo00D0omnL09boo`3ogi`0B000001d_`030?ooo`0?0?no
M00007@0_ooo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?oOW0180000B9cO0080
oooo00<0oooO09a80000M;l00P3oool01`3OW4P0001d0;ooo`3oool0gia80018W03Oool00P3oool0
1P3OW4P0001d0;ooo`3ogi`0B000001d_`<0oooo00/0okmd0000M02oool0oooo0=nLB000B9`0gooo
0?ooo`3ooml0W4P0001d_`080?ooo`030?oog`2LB00007Bo0080oooo00<0omnL04P00000M;l00`3o
ool01`3o_g@0001d0;ooo`3oool0gia80000M02oool01`3oool00`3OW4P0001d0;ooo`020?ooo`0:
0;md0000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0oooO09a800P0000000H0B9cO0?ooo`3oool0
_g@00018W03Oool20?ooo`030;md0000M;l0oooo00X0oooo00<0oooO09a8M02oool0O`3oool00<40
oooo00<0gia80000M02oool00P3oool01`2oM00004RL0=ooo`3oool0oooO09a80000M;l00P3oool0
1P3o_g@00000001d_`3ooml0W4P0001d_`80oooo00L0gia80018W03Oool0oooo0?noM00004P0W=oo
00H0oooo00L0gia80000M02oool0oooo0?oog`2LB00007Bo0080oooo00/0gia80000M02oool0oooo
0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`060=nLB00007@0_ooo0?oo_`1d0000B9cO
1`3oool00`3o_g@0001809cOo`050?ooo`030?noM0000000M;oo00<0oooo00l0okmd0000001d_ol0
oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd0000001d_ol00P3oool00`3ooml0
W4P0001d_`020?ooo`070=nLB00007@0_ooo0?ooo`3OW4P004RL0=ooo`020?ooo`060=nLB00007@0
_ooo0?noM0000000M;oo0`3oool02`3o_g@0000007Boo`3oool0gia80018W03Oool0oooo0?oog`2L
B00007Bo00P0oooo00<0oooO09a80000M;l00P3oool00`3o_g@0000007Boo`030?ooo`070?noM000
0000M;oo0?ooo`3OW4P0001d0;ooo`070?ooo`030=nLB00007@0_ooo0080oooo00L0_g@00018W03O
ool0oooo0?oog`2LB00007Bo00d0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`0;
0?ooo`030;md001d_ol0oooo07h0oooo00310?ooo`030=nLB00007@0_ooo0080oooo00L0_g@00018
W03Oool0oooo0?oog`2LB00007Bo00@0oooo00@0okmd00000000000007Bo0P3oool01`3OW4P004RL
0=ooo`3oool0okmd0000M02oool01P3oool01`3OW4P0001d0;ooo`3oool0oooO09a80000M;l00P3o
ool04`3OW4P0001d0;ooo`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3ookl0M000
04RLg`3oool0oooO09a80000M;l01`3oool00`3o_g@0001d0;ooo`050?ooo`030?noM0000000M;oo
00<0oooo00l0okmd0000001d_ol0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd
0000001d_ol00P3oool00`3ooml0W4P0001d_`020?ooo`070=nLB00007@0_ooo0?ooo`3OW4P004RL
0=ooo`020?ooo`060=nLB00007@0_ooo0?noM0000000M;oo0`3oool02`3o_g@0000007Boo`3oool0
gia80018W03Oool0oooo0?oog`2LB00007Bo00P0oooo00<0oooO09a80000M;l00P3oool00`3o_g@0
000007Boo`030?ooo`070?noM0000000M;oo0?ooo`3OW4P0001d0;ooo`070?ooo`030=nLB00007@0
_ooo0080oooo00L0_g@00018W03Oool0oooo0?oog`2LB00007Bo00d0oooo00<0omnL04P0001d_ol0
0`3oool00`3o_g@0001809cOo`0:0?ooo`030?oog`2LB000B9cO07l0oooo00310?ooo`030=nLB000
04P0W=oo0080oooo00l0_g@00000M02oool0oooo0?oog`2LB00007Bo0?ooo`3ookl0M00007Boo`3o
ool0oooO09a80000M;l00P3oool01`3OW4P004RL0=ooo`3oool0omnL04P0B02Lgol00P3oool02`3o
_g@0001809cOo`3oool0gia80000M02oool0oooo0?oog`2LB00007Bo0080oooo01<0gia80000M02o
ool0oooo0;md000007@0_ooo0?ooo`3ooml0W4P0001d_`3oool0oono07@00018W=l0oooo0?oog`2L
B000B9cO00L0oooo00<0omnL04P0B02Lgol00P3oool01P3o_g@0001809cOo`3ookl0M00007Boo`<0
oooo00l0_g@00000B02Lgol0oooo0;md000007@0_ooo0?ooo`3ooml0W4P0001d_`3oool0oono07@0
0018W=l00P3oool00`3ooml0W4P0001d_`020?ooo`070=nLB00007@0_ooo0?ooo`3OW4P004RL0=oo
o`020?ooo`060=nLB00007@0_ooo0?oo_`1d0000M;oo0`3oool02`2oM000001809cOo`3oool0_g@0
0000M02oool0oooo0?oog`2LB00007Bo00P0oooo00<0oooO09a80000M;l00P3oool00`3ookl0M000
07Boo`030?ooo`070;md000004P0W=oo0?ooo`3OW4P0001809cOo`070?ooo`030=nLB00004P0W=oo
0080oooo00L0_g@00000M02oool0oooo0?oog`2LB00007Bo00d0oooo00<0omnL04P0001d_ol00`3o
ool00`3o_g@0001809cOo`0:0?ooo`030=nLB0000000M;oo07l0oooo002o0?ooo`0F0?oog`2LB000
00000000B018B4P0W=oo0?oo_`1d000007Ad07@0M02oool0oono07@00018W=l0oooo0?oo_`1d0000
B9cO0?ooo`3ookl0M000001d_`80oooo00<0gia80018W03Oool00P3oool0702oM00004RL0=ooo`3o
oml0W4P00000M02oool0oooo0=nLB0000000001809cOo`3ookl0M00004RLg`3ooml0W4P000000000
07@0_ooO09a80000000004Q804P0B02Lgol0oono07@00018W=l20?ooo`040;md0000M;l0oooo0?no
M080000000<004RL0=ooo`3oool01@3oool0602oM00004RL0=ooo`3ooml0W4P00000M02oool0oooo
0=nLB000B9`0gooo0?ooo`3o_g@00000001d_`3ooml0W4P000000000B4P0B01809cOo`3ookl0M000
04RLg`80oooo02H0okmd0000B02Lgol0oooo0?noM000000007Bo0?oog`2LB00000000000M02oool0
oooo0=nLB000B9`0gooo0?oog`2LB00000000000M02oool0oooo0=nLB000B9`0gooo0?ooo`3o_g@0
0000001d_`3ooml0W4P000000000B4P0B01809cOo`3ookl0M00004RLg`P0oooo00<0oooO09a80000
M;l00`3oool03`3OW4P004RL0=ooo`3oool0okmd00000000M;l0oooO09a800000000001804QdW03O
_g@0001d0;ooo`020?ooo`0A0?oog`2LB00000000000B018B4P0W=oo0?oo_`1d000007Ad07@0M02o
ool0oono07@00018W=l0oooo0?oog`2LB000200000001@18W=l0oooo0?oOW0180000M;oo00<0oooo
00<0okmd0000B02Lgol01P3oool01`3Ogml0oooo0?oo_`1d000000000000M02oool0O`3oool00<40
oooo00`0_g@00000M02ooml0W4P00000000004P0W=oo0?oo_`1d000000000000M02oool20?ooo`08
0?oo_`1d000000000000000007@0_ooo0?ooo`2oM003000000040018W03Oool0oooo0=nLB0800000
00/0001809cOo`3oool0oooo0=nLB00007@0_ooO09a800000000001d0;ooo`020?ooo`0;0;md0000
07@0_ooo0?ooo`2oM00004RL0=oog`2LB00000000000M02oool00P3oool01@3ooml0W4P000000000
0000M;oo00X0oooo00D0gia8000000000000001809cOo`030?ooo`050?noM0000000000000000018
W=l00`3oool01`2oM00004RL0=oog`2LB00000000000M02oool00`3oool00`3o_g@0000000000002
00000007001d_`3oool0oooo0;md000007@0_ooo0;md0003000000050018W03Oool0_g@00000M02o
ool00P3oool01@3o_g@00000000000000000B9cO00<0oooo00L0_g@00018W03Ooml0W4P000000000
07@0_ooo00L0oooo00<0okmd0000000000000P0000001000B9`0gooo0?ooo`3o_g@30000000:04RL
g`3oool0oooo0?ooo`2oM000001d0;oOW0180000001d0;ooo`@0oooo00`0_g@00000M02ooml0W4P0
0000000004P0W=oo0?oo_`1d000000000000M02oool=0?ooo`030?oOW0180000M;oo00<0oooo00<0
okmd0000B02Lgol01P3oool01P3OW4P00000000000000000001809cOoh00oooo003G0?ooo`030;md
0000B9`0gooo00X0oooo00<0gia80000M02oool0?`3oool00`3ooml0W4P0001d_`060?ooo`030;md
0000B9`0gooo01h0oooo00<0oooO09a80000M;l0;`3oool00`3ookl0M00004RLg`030?ooo`030=nL
B00007@0_ooo00H0oooo00@0oono07@000000000B9cOPP3oool00=L0oooo00<0oono07A8W03Oool0
2P3oool00`3OW4P0001d0;ooo`0o0?ooo`030?oog`2LB00007Bo00H0oooo00<0oono07A8W03Oool0
7`3oool00`2oM00007Bo0?ooo`0_0?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d_`3oool0
1`3oool00`2oM000M;oo0?ooo`220?ooo`00e`3oool00`3ooml0W7BL0=ooo`0:0?ooo`030=nLB000
07@0_ooo03l0oooo00<0oooO09a80000M;l01P3oool00`3ooml0W7BL0=ooo`0O0?ooo`030=nLB000
M;l0oooo02l0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00P0oooo00H0omnL04P00000
000000000000B02Lgomo0?ooo`00hP3oool01@3ooml0W4P00000000007@0_ooo00H0oooo00<0gia8
0000B02Lgol0=@3oool0103ookl0M00000000000M;l20?ooo`030=nLB00004P0W=oo00H0oooo00<0
gia80000B02Lgol06P3oool01`3ookl0M01d0;ooo`3ooml0W4P00018W03Oool0;03oool01@3ookl0
M0000000000007@0_ooo00X0oooo00D0_g@0000000000000000004RLg`1o0?ooo`00i03oool00`2o
M000001d0;ooo`060?ooo`030=nLB00004P0W=oo03H0oooo00<0oooO09a80000M;l00P3oool00`3O
W4P0001809cOo`060?ooo`030=nLB00004P0W=oo01/0oooo00D0oono07@000000000001809cOo`30
0?ooo`00o`3ooooo0?oooaT0oooo003o0?ooool0oooo6@3oool00?l0ooooo`3ooolI0?ooo`00o`3o
oooo0?oooaT0oooo0000\
\>"],
  ImageRangeCache->{{{0, 534.125}, {329.75, 0}} -> {-2.38739, -70.9176, \
0.074252, 0.602217}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(sol = 
        ShootBSWF2[pot, 0.5, 21, 22, \(-7\), 7, 0.5, 
          MaxIterations \[Rule] 50];\)\), "\n", 
    \(\(PlotBSWF[sol, \(-7\), 7, 
        PlotRange \[Rule] {All, {21, 22.3}}];\)\)}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0680272 -9.98363 0.475411 [
[.09184 -0.0125 -9.15625 -15.6875 ]
[.09184 -0.0125 9.15625 0 ]
[.22789 -0.0125 -9.15625 -15.6875 ]
[.22789 -0.0125 9.15625 0 ]
[.36395 -0.0125 -9.15625 -15.6875 ]
[.36395 -0.0125 9.15625 0 ]
[.5 -0.0125 -5 -15.6875 ]
[.5 -0.0125 5 0 ]
[.63605 -0.0125 -5 -15.6875 ]
[.63605 -0.0125 5 0 ]
[.77211 -0.0125 -5 -15.6875 ]
[.77211 -0.0125 5 0 ]
[.90816 -0.0125 -5 -15.6875 ]
[.90816 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -5.0625 -31.375 ]
[.5 -0.0125 5.0625 -15.6875 ]
[-0.0125 .09508 -25 -7.84375 ]
[-0.0125 .09508 0 7.84375 ]
[-0.0125 .19016 -25 -7.84375 ]
[-0.0125 .19016 0 7.84375 ]
[-0.0125 .28525 -25 -7.84375 ]
[-0.0125 .28525 0 7.84375 ]
[-0.0125 .38033 -25 -7.84375 ]
[-0.0125 .38033 0 7.84375 ]
[-0.0125 .47541 -16 -7.84375 ]
[-0.0125 .47541 0 7.84375 ]
[-0.0125 .57049 -25 -7.84375 ]
[-0.0125 .57049 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -40.6875 -13.0313 ]
[-0.01875 .30902 -25 13.0313 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -95.6875 0 ]
[.5 .63053 95.6875 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.09184 0 m
.09184 .00625 L
s
gsave
.09184 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.22789 0 m
.22789 .00625 L
s
gsave
.22789 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.36395 0 m
.36395 .00625 L
s
gsave
.36395 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.5 0 m
.5 .00625 L
s
gsave
.5 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.63605 0 m
.63605 .00625 L
s
gsave
.63605 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.77211 0 m
.77211 .00625 L
s
gsave
.77211 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.90816 0 m
.90816 .00625 L
s
gsave
.90816 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.125 Mabswid
.12585 0 m
.12585 .00375 L
s
.15986 0 m
.15986 .00375 L
s
.19388 0 m
.19388 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.29592 0 m
.29592 .00375 L
s
.32993 0 m
.32993 .00375 L
s
.39796 0 m
.39796 .00375 L
s
.43197 0 m
.43197 .00375 L
s
.46599 0 m
.46599 .00375 L
s
.53401 0 m
.53401 .00375 L
s
.56803 0 m
.56803 .00375 L
s
.60204 0 m
.60204 .00375 L
s
.67007 0 m
.67007 .00375 L
s
.70408 0 m
.70408 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.80612 0 m
.80612 .00375 L
s
.84014 0 m
.84014 .00375 L
s
.87415 0 m
.87415 .00375 L
s
.05782 0 m
.05782 .00375 L
s
.02381 0 m
.02381 .00375 L
s
.94218 0 m
.94218 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -66.0625 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
69.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .09508 m
.00625 .09508 L
s
gsave
-0.0125 .09508 -86 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(21.2) show
1.000 setlinewidth
grestore
0 .19016 m
.00625 .19016 L
s
gsave
-0.0125 .19016 -86 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(21.4) show
1.000 setlinewidth
grestore
0 .28525 m
.00625 .28525 L
s
gsave
-0.0125 .28525 -86 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(21.6) show
1.000 setlinewidth
grestore
0 .38033 m
.00625 .38033 L
s
gsave
-0.0125 .38033 -86 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(21.8) show
1.000 setlinewidth
grestore
0 .47541 m
.00625 .47541 L
s
gsave
-0.0125 .47541 -77 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(22) show
1.000 setlinewidth
grestore
0 .57049 m
.00625 .57049 L
s
gsave
-0.0125 .57049 -86 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(22.2) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .02377 m
.00375 .02377 L
s
0 .04754 m
.00375 .04754 L
s
0 .07131 m
.00375 .07131 L
s
0 .11885 m
.00375 .11885 L
s
0 .14262 m
.00375 .14262 L
s
0 .16639 m
.00375 .16639 L
s
0 .21393 m
.00375 .21393 L
s
0 .23771 m
.00375 .23771 L
s
0 .26148 m
.00375 .26148 L
s
0 .30902 m
.00375 .30902 L
s
0 .33279 m
.00375 .33279 L
s
0 .35656 m
.00375 .35656 L
s
0 .4041 m
.00375 .4041 L
s
0 .42787 m
.00375 .42787 L
s
0 .45164 m
.00375 .45164 L
s
0 .49918 m
.00375 .49918 L
s
0 .52295 m
.00375 .52295 L
s
0 .54672 m
.00375 .54672 L
s
0 .59426 m
.00375 .59426 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -106.875 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
74.0313 11.8438 translate
90 rotate
-74.0313 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(V) show
71.625 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(,) show
77.625 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(y) show
85.063 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.09184 .61178 m
.09184 .61803 L
s
.22789 .61178 m
.22789 .61803 L
s
.36395 .61178 m
.36395 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.63605 .61178 m
.63605 .61803 L
s
.77211 .61178 m
.77211 .61803 L
s
.90816 .61178 m
.90816 .61803 L
s
.125 Mabswid
.12585 .61428 m
.12585 .61803 L
s
.15986 .61428 m
.15986 .61803 L
s
.19388 .61428 m
.19388 .61803 L
s
.2619 .61428 m
.2619 .61803 L
s
.29592 .61428 m
.29592 .61803 L
s
.32993 .61428 m
.32993 .61803 L
s
.39796 .61428 m
.39796 .61803 L
s
.43197 .61428 m
.43197 .61803 L
s
.46599 .61428 m
.46599 .61803 L
s
.53401 .61428 m
.53401 .61803 L
s
.56803 .61428 m
.56803 .61803 L
s
.60204 .61428 m
.60204 .61803 L
s
.67007 .61428 m
.67007 .61803 L
s
.70408 .61428 m
.70408 .61803 L
s
.7381 .61428 m
.7381 .61803 L
s
.80612 .61428 m
.80612 .61803 L
s
.84014 .61428 m
.84014 .61803 L
s
.87415 .61428 m
.87415 .61803 L
s
.05782 .61428 m
.05782 .61803 L
s
.02381 .61428 m
.02381 .61803 L
s
.94218 .61428 m
.94218 .61803 L
s
.97619 .61428 m
.97619 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 0 m
1 0 L
s
.99375 .09508 m
1 .09508 L
s
.99375 .19016 m
1 .19016 L
s
.99375 .28525 m
1 .28525 L
s
.99375 .38033 m
1 .38033 L
s
.99375 .47541 m
1 .47541 L
s
.99375 .57049 m
1 .57049 L
s
.125 Mabswid
.99625 .02377 m
1 .02377 L
s
.99625 .04754 m
1 .04754 L
s
.99625 .07131 m
1 .07131 L
s
.99625 .11885 m
1 .11885 L
s
.99625 .14262 m
1 .14262 L
s
.99625 .16639 m
1 .16639 L
s
.99625 .21393 m
1 .21393 L
s
.99625 .23771 m
1 .23771 L
s
.99625 .26148 m
1 .26148 L
s
.99625 .30902 m
1 .30902 L
s
.99625 .33279 m
1 .33279 L
s
.99625 .35656 m
1 .35656 L
s
.99625 .4041 m
1 .4041 L
s
.99625 .42787 m
1 .42787 L
s
.99625 .45164 m
1 .45164 L
s
.99625 .49918 m
1 .49918 L
s
.99625 .52295 m
1 .52295 L
s
.99625 .54672 m
1 .54672 L
s
.99625 .59426 m
1 .59426 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
.5 0 m
.5 .61803 L
s
gsave
.5 .63053 -156.688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Wave) show
93.875 15.813 moveto
(Function) show
139.313 15.813 moveto
(with) show
163.563 15.813 moveto
(Energy) show
200.563 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
211.375 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(21.6417) show
250.375 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.17893 .61803 m
.18221 .39141 L
s
.18221 .39141 m
.18861 0 L
s
.81127 0 m
.82051 .56954 L
s
.82051 .56954 m
.8212 .61803 L
s
0 g
.02381 .30509 m
.02846 .30512 L
.03279 .30514 L
.03754 .30517 L
.04262 .30522 L
.04795 .30528 L
.053 .30537 L
.05753 .30548 L
.06244 .30563 L
.06751 .30585 L
.07308 .30618 L
.0789 .30667 L
.08435 .3073 L
.08944 .30808 L
.0941 .30902 L
.09947 .31042 L
.10458 .31216 L
.10961 .31434 L
.11417 .31681 L
.11908 .32009 L
.12439 .3245 L
.13385 .33507 L
.14277 .34889 L
.15228 .36852 L
.16271 .3964 L
.18378 .46931 L
.19379 .50572 L
.20329 .53467 L
.20583 .54071 L
.20853 .54606 L
.20993 .54836 L
.21149 .55049 L
.2129 .55201 L
.21353 .55257 L
.21423 .55308 L
.21545 .55372 L
.21673 .55404 L
.21792 .554 L
.21903 .55366 L
.22034 .55286 L
.22153 .55176 L
.2229 .55003 L
.2242 .54793 L
.22659 .54285 L
.22883 .53664 L
.23392 .51712 L
.23888 .49089 L
.24421 .45515 L
.26267 .28978 L
.2728 .19688 L
Mistroke
.27815 .15656 L
.28108 .13854 L
.28384 .12476 L
.28647 .11482 L
.28795 .11068 L
.28866 .1091 L
.28932 .10785 L
.29047 .10619 L
.29173 .10517 L
.29296 .10499 L
.29427 .10569 L
.29552 .10721 L
.29622 .10844 L
.29687 .10982 L
.29933 .11706 L
.30157 .12641 L
.30394 .13909 L
.30885 .17359 L
.31334 .21342 L
.32345 .31945 L
.3323 .40943 L
.33725 .44933 L
.33959 .46416 L
.34175 .47516 L
.3429 .47995 L
.34418 .48425 L
.34545 .48755 L
.34615 .4889 L
.34681 .48988 L
.34801 .49094 L
.34911 .49107 L
.35037 .49021 L
.35157 .4884 L
.35268 .4859 L
.35385 .48236 L
.35595 .47378 L
.35821 .46149 L
.36071 .44446 L
.37139 .341 L
.38142 .22912 L
.3865 .18121 L
.38898 .16219 L
.39124 .14795 L
.39362 .13639 L
.39494 .13167 L
.39617 .12833 L
.39683 .12701 L
.39752 .12595 L
.39877 .12492 L
.39946 .12485 L
Mistroke
.40011 .12509 L
.40082 .12572 L
.40158 .12681 L
.40281 .12944 L
.40395 .13286 L
.4052 .13767 L
.40653 .14398 L
.41124 .17525 L
.41588 .21753 L
.42008 .26259 L
.42945 .36901 L
.43456 .41997 L
.43934 .45672 L
.44184 .47061 L
.44324 .47662 L
.44451 .48092 L
.44519 .48274 L
.44591 .48433 L
.4467 .48564 L
.44743 .48646 L
.44874 .48697 L
.44949 .48671 L
.45019 .4861 L
.45141 .48421 L
.45269 .48109 L
.45379 .47753 L
.45499 .47272 L
.45767 .45861 L
.46016 .44168 L
.47891 .24918 L
.48405 .19764 L
.48686 .17352 L
.48949 .15435 L
.49191 .13997 L
.49413 .12989 L
.49539 .12552 L
.49659 .12233 L
.49785 .11998 L
.49857 .11912 L
.49923 .11863 L
.49993 .11843 L
.50058 .11854 L
.50128 .11898 L
.50203 .11981 L
.50328 .12204 L
.504 .12377 L
.50466 .12568 L
.50704 .13481 L
.50958 .14841 L
.51534 .19215 L
Mistroke
.52059 .24389 L
.53107 .35977 L
.53567 .4064 L
.53812 .42808 L
.54075 .44826 L
.54325 .46396 L
.54554 .47497 L
.54664 .47903 L
.54784 .48254 L
.54898 .48493 L
.55003 .48633 L
.55132 .48697 L
.55253 .48649 L
.55362 .48516 L
.5548 .48277 L
.55609 .47904 L
.55745 .47385 L
.55988 .46147 L
.56236 .44512 L
.56501 .42377 L
.56979 .3772 L
.58066 .25432 L
.58559 .2031 L
.59083 .15984 L
.59356 .14348 L
.59512 .13635 L
.59658 .13117 L
.59723 .12935 L
.59791 .12777 L
.59914 .12578 L
.60045 .12486 L
.6012 .12491 L
.60189 .12533 L
.60315 .12697 L
.60386 .12841 L
.60451 .13005 L
.60572 .13387 L
.60699 .13899 L
.60919 .15044 L
.61162 .16651 L
.61699 .21306 L
.62201 .26622 L
.63241 .38215 L
.63754 .43053 L
.64003 .44986 L
.64229 .46447 L
.6445 .47586 L
.64658 .48376 L
.64773 .48691 L
.64878 .48903 L
Mistroke
.64994 .49052 L
.65116 .49111 L
.65231 .49075 L
.65354 .48939 L
.65471 .48719 L
.65577 .48442 L
.65701 .48026 L
.65837 .47466 L
.66082 .46173 L
.66568 .42689 L
.67024 .38533 L
.68063 .27526 L
.69087 .17586 L
.69602 .13931 L
.69894 .12407 L
.70024 .11866 L
.70165 .11379 L
.70302 .11007 L
.70428 .10753 L
.70549 .10591 L
.70677 .10506 L
.70792 .10504 L
.70899 .10565 L
.71016 .10701 L
.71142 .10927 L
.71276 .11255 L
.71422 .11713 L
.71688 .12804 L
.72186 .1566 L
.73147 .23423 L
.74037 .31928 L
.74933 .40286 L
.75901 .47763 L
.76454 .50974 L
.76696 .52106 L
.76962 .5315 L
.77197 .53902 L
.77448 .54532 L
.77586 .54804 L
.77713 .55007 L
.7784 .55168 L
.7796 .55281 L
.78074 .55355 L
.78198 .55398 L
.78315 .55405 L
.78421 .55384 L
.78546 .55328 L
.78616 .5528 L
.78682 .55227 L
.78929 .54947 L
Mistroke
.79064 .54747 L
.7919 .54528 L
.79427 .54049 L
.79958 .52688 L
.81802 .46262 L
.8364 .39907 L
.84684 .37063 L
.85658 .35007 L
.8662 .33499 L
.8716 .32852 L
.87652 .32368 L
.8817 .31953 L
.88734 .31594 L
.89266 .31329 L
.89759 .31137 L
.90222 .30994 L
.90722 .30873 L
.91193 .30785 L
.91626 .30721 L
.92105 .30667 L
.92629 .30622 L
.93177 .30588 L
.93686 .30566 L
.94191 .30549 L
.94739 .30536 L
.95256 .30528 L
.95732 .30522 L
.96162 .30518 L
.96635 .30515 L
.97591 .30509 L
.97619 .30509 L
Mfstroke
[ .02 .04 ] 0 setdash
.02381 .30509 m
.06244 .30509 L
.10458 .30509 L
.14415 .30509 L
.18221 .30509 L
.22272 .30509 L
.26171 .30509 L
.30316 .30509 L
.34309 .30509 L
.3815 .30509 L
.42237 .30509 L
.46172 .30509 L
.49955 .30509 L
.53984 .30509 L
.57861 .30509 L
.61984 .30509 L
.65954 .30509 L
.69774 .30509 L
.73838 .30509 L
.77751 .30509 L
.81909 .30509 L
.85916 .30509 L
.89771 .30509 L
.93871 .30509 L
.97619 .30509 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00m03oool01@2oM0000000001d_`3oool0gia80080000000<004RL0=ooo`3oool0/P3oool0
0?@0oooo00P0oooO09a8M02oool0oooo0?oog`2LB00004RL0=oook@0oooo003e0?ooo`060?oo_`1d
B9`0gooo0;md000007@0_ooo]@3oool00?H0oooo00@0omnL07A8B0000000M;oo]P3oool00?H0oooo
00<0oono07@00018W=l0]`3oool00?D0oooo00D0oooO09a80000B9`0W7A809cOo`2f0?ooo`00m@3o
ool01P2oM000001809cOo`3ooml0W4Qd0;oookD0oooo003c0?ooo`090?oog`2LB000000000000000
M;l0okmd0000000007@0_ooo0;@0oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo00200?ooo`050?oOW018000000000000
001d_ol09P3oool0103ooml0W4P00018W03OoolR0?ooo`030;md00000000000000@0000000<0B9cO
0?ooo`3oool06`3oool01@3ogi`0B0000000000007@0_ooo0280oooo00<0_g@00000000000001000
00000`18W=l0oooo0?ooo`0S0?ooo`040?oog`2LB00004RL0=ooob@0oooo00D0omnL04P000000000
000007Boo`0i0?ooo`00O`3oool01`3ogi`0B00004RLg`3oool0oono07@0001d_ol09@3oool0103o
oml0W4P00018W03OoolS0?ooo`030=nLB0000000000000<0000000<004RL0=ooo`3oool06P3oool0
1`3ookl0M00007Boo`3oool0okmd0000M02oool08P3oool00`3OW4P0000000000003000000030018
W03Oool0oooo02<0oooo00@0oooO09a80000B9`0gooo8`3oool01`3ogi`0B00004RLg`3oool0oono
07@0001d_ol0>03oool007h0oooo00<0oooO09a80000M;l00`3oool00`3o_g@004RL0=ooo`0T0?oo
o`040?oog`2LB00004RL0=ooob@0oooo00<0gia804RLg`3oool00P3oool00`3OW7@0W=oo0?ooo`0J
0?ooo`030;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool08P3oool00`3OW4P0B9cO0?ooo`02
0?ooo`030=nLM02Lgol0oooo02<0oooo00@0oooO09a80000B9`0gooo8P3oool00`3ooml0W4P0001d
_`030?ooo`030?noM000B9`0gooo03L0oooo001n0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd
0000B02Lgol0803oool00`2oM0000000000000050000000307Boo`3oool0oooo0280oooo00<0gia8
07Boo`3oool07@3oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo02<0oooo00<0gia8
07Boo`3oool08`3oool00`2oM0000000000000050000000307Boo`3oool0oooo01l0oooo00<0omnL
04P0001d_ol00`3oool00`3o_g@0001809cOo`0g0?ooo`00LP3oool00`3ooml0W4P0000000070000
000504RLg`3oool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0D0?ooo`030?oog`2LB000
000000L0000000@0B9cO0?ooo`3ogi`0M9cO0P3oool0103ooml0W4P00018W03OoolE0?ooo`030?oo
g`2LB000000000L0000000<0B9cO0?ooo`3oool0103oool00`2oM4P0W=oo0?ooo`0L0?ooo`030?oO
W0180000M;oo00<0oooo00<0okmd0000B02Lgol0903oool00`2oM4P0W=oo0?ooo`0R0?ooo`080?oO
W01dW=l0oooo0?ooo`3ooml0W4P00018W03OoolR0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd
0000B02Lgol0=`3oool007h0oooo00<0omnL04P0001d_ol00`3oool00`2oM000001809cOo`0Q0?oo
o`070=nLB01d_ol0oooo0?oog`2LB00004RL0=ooo`0V0?ooo`030?oog`2LB7@0_ooo01`0oooo00<0
omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0T0?ooo`030?oog`2LB7@0_ooo02<0oooo00L0
gia807Boo`3oool0oooO09a80000B9`0gooo0280oooo00<0omnL04P0001d_ol00`3oool00`2oM000
001809cOo`0g0?ooo`00OP3oool0203ookl0M00000000018W=l0oooo0?noM000000007Bo8P3oool0
1`3ooml0W4Qd0;ooo`3ooml0W4P00018W03Oool09`3oool00`3o_g@007Bo0?ooo`0K0?ooo`030?oO
W0180000M;oo00<0oooo00<0okmd0000B02Lgol09@3oool00`3o_g@007Bo0?ooo`0R0?ooo`070?oo
g`2LB7@0_ooo0?oog`2LB00004RL0=ooo`0R0?ooo`080?oo_`1d0000000004RLg`3oool0okmd0000
0000M;lh0?ooo`00O`3oool00`2oM00004RL0;nLB002000000030018W03Oool0oooo0280oooo00H0
omnL04Qd_`3ooml0W4P00018W03OoolW0?ooo`030?oo_`1d04P0W=oo01/0oooo00<0omnL04P0001d
_ol00`3oool00`3o_g@0001809cOo`0U0?ooo`030?oo_`1d04P0W=oo02<0oooo00H0omnL04Qd_`3o
oml0W4P00018W03OoolS0?ooo`030;md0000B9`0_ia80080000000<004RL0=ooo`3oool0=`3oool0
07l0oooo00<0omnL04P0001d_ol0:03oool01@3OW4P0M;oO09a80000B9`0gooo02L0oooo00<0oono
07@00018W=l06`3oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo02D0oooo00<0oono
07@00018W=l0903oool01@3OW4P0M;oO09a80000B9`0gooo02<0oooo00<0omnL04P0001d_ol0?03o
ool00800oooo00<0gia80000B02Lgol09`3oool01@3ooml0W4Q804Q80000B9`0gooo0280oooo00<0
oknL09cOo`3oool00P3oool00`3ogi`0B00004RLg`0L0?ooo`030=nLB000B9`0gooo0080oooo00<0
_g@0001d_`3oool0803oool00`3o_i`0W=oo0?ooo`020?ooo`030?oOW0180000B9cO02@0oooo00D0
oooO09a8B018B00004RL0=ooo`0T0?ooo`030=nLB00004P0W=oo03/0oooo00210?ooo`030?noM000
000007Bo02L0oooo00@0omnL04P00000B9`0gooo8P3oool0203ooml0W4Q809cOo`3oool0oono07@0
000007@0_ooo703oool01`3ookl0M00007Boo`3oool0okmd0000M02oool08@3oool0203ooml0W4Q8
09cOo`3oool0oono07@0000007@0_ooo9@3oool0103ogi`0B0000018W03OoolU0?ooo`030?noM000
000007Bo03X0oooo00220?ooo`050?oog`2LB00000000000M02oool09@3oool00`3OW4P004RL0=oo
o`0S0?ooo`060?oog`2LB00000000000000007@0_ooo7P3oool01@3ookl0M0000000000007@0_ooo
02<0oooo00H0oooO09a80000000000000000M02ooolW0?ooo`030=nLB000B9`0gooo02H0oooo00D0
oooO09a800000000001d0;ooo`0g0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00H03ooolj000000030?l00000000000000;X0000000<0o`00000000000000=`00000O0?ooo`00
H03oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1P3oool00`3o000000000?ooo`090?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?l000080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`3o000000000?ooo`090?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?l000080?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0M0?ooo`00H03oool0
0`000000oooo0?ooo`0g0?ooo`030?l0003oool0oooo05/0oooo00<000000?ooo`3oool0G03oool0
0`3o0000oooo0?ooo`0f0?ooo`030000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo
03L0oooo00<0o`000?ooo`3oool0F`3oool00`000000oooo0?ooo`1L0?ooo`030?l0003oool0oooo
03H0oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0=`3oool00`3o0000
oooo0?ooo`1K0?ooo`030000003oool0oooo05`0oooo00<0o`000?ooo`3oool0=P3oool00`000000
oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`0g0?ooo`030?l0003oool0oooo05/0oooo
00<000000?ooo`3oool0G03oool00`3o0000oooo0?ooo`0f0?ooo`030000003oool0oooo01d0oooo
001P0?ooo`030000003oool0oooo03L0oooo00<0o`000?ooo`3oool0F`3oool00`000000oooo0?oo
o`1L0?ooo`030?l0003oool0oooo03H0oooo00<000000?ooo`3oool07@3oool00600oooo0`00000g
0?ooo`030?l0003oool0oooo05/0oooo00<000000?ooo`3oool0G03oool00`3o0000oooo0?ooo`0e
0?ooo`8000007`3oool00600oooo00<000000?ooo`3oool0=`3oool00`3o0000oooo0?ooo`1K0?oo
o`030000003oool0oooo05`0oooo00<0o`000?ooo`3oool0=P3oool00`000000oooo0?ooo`0M0?oo
o`00H03oool00`000000oooo0?ooo`0g0?ooo`030?l0003oool0oooo05/0oooo00<000000?ooo`3o
ool0G03oool00`3o0000oooo0?ooo`0f0?ooo`030000003oool0oooo01d0oooo001P0?ooo`030000
003oool0oooo03L0oooo00<0o`000?ooo`3oool0F`3oool00`000000oooo0?ooo`1L0?ooo`030?l0
003oool0oooo03H0oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0=`3o
ool00`3o0000oooo0?ooo`1K0?ooo`030000003oool0oooo05`0oooo00<0o`000?ooo`3oool0=P3o
ool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`0g0?ooo`030?l0003oool0
oooo05/0oooo00<000000?ooo`3oool0G03oool00`3o0000oooo0?ooo`0f0?ooo`030000003oool0
oooo01d0oooo001P0?ooo`030000003oool0oooo03L0oooo00<0o`000?ooo`3oool0F`3oool00`00
0000oooo0?ooo`1L0?ooo`030?l0003oool0oooo03H0oooo00<000000?ooo`3oool07@3oool00600
oooo0`00000g0?ooo`030?l0003oool0oooo05/0oooo00<000000?ooo`3oool0G03oool00`3o0000
oooo0?ooo`0e0?ooo`8000007`3oool00600oooo00<000000?ooo`3oool0=`3oool00`3o0000oooo
0?ooo`1K0?ooo`030000003oool0oooo05`0oooo00<0o`000?ooo`3oool0=P3oool00`000000oooo
0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`0g0?ooo`030?l0003oool0oooo05/0oooo00<0
00000?ooo`3oool0G03oool00`3o0000oooo0?ooo`0f0?ooo`030000003oool0oooo01d0oooo001P
0?ooo`030000003oool0oooo03L0oooo00<0o`000?ooo`3oool0F`3oool00`000000oooo0?ooo`1L
0?ooo`030?l0003oool0oooo03H0oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?oo
o`3oool0=`3oool00`3o0000oooo0?ooo`1K0?ooo`030000003oool0oooo05`0oooo00<0o`000?oo
o`3oool0=P3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`0g0?ooo`03
0?l0003oool0oooo05/0oooo00<000000?ooo`3oool0G03oool00`3o0000oooo0?ooo`0f0?ooo`03
0000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo03L0oooo00<0o`000?ooo`3oool0
F`3oool00`000000oooo0?ooo`1L0?ooo`030?l0003oool0oooo03H0oooo00<000000?ooo`3oool0
7@3oool00600oooo0`00000g0?ooo`030?l0003oool0oooo05/0oooo00<000000?ooo`3oool0G03o
ool00`3o0000oooo0?ooo`0e0?ooo`8000007`3oool00600oooo00<000000?ooo`3oool0=`3oool0
0`3o0000oooo0?ooo`1K0?ooo`030000003oool0oooo05`0oooo00<0o`000?ooo`3oool0=P3oool0
0`000000oooo0?ooo`0M0?ooo`00>P3oool00`2oM0000000000000040000000504RLg`3oool0oooo
0?ooo`2oM0000`0000002@00B9`0gooo0?ooo`3oool0okmd0000001d_ol0oooo0;md000600000003
04RLg`3oool0oooo00D0oooo00<000000?ooo`3oool0=`3oool00`3o0000oooo0?ooo`1K0?ooo`03
0000003oool0oooo05`0oooo00<0o`000?ooo`3oool0=P3oool00`000000oooo0?ooo`0M0?ooo`00
>`3oool00`3OW4P0000000000003000000030018W03Oool0oooo0080oooo00<0omnL04P0001d_ol0
103oool00`3o_g@0000007Boo`020?ooo`030=nLB0000000000000<0000000<004RL0=ooo`3oool0
1@3oool00`000000oooo0?ooo`0g0?ooo`030?l0003oool0oooo05/0oooo00<000000?ooo`3oool0
G03oool00`3o0000oooo0?ooo`0f0?ooo`030000003oool0oooo01d0oooo000l0?ooo`030=nLB018
W=l0oooo0080oooo00<0giad09cOo`3oool00P3oool00`3ogi`0B00007Boo`0:0?ooo`030=nLB018
W=l0oooo0080oooo00<0giad09cOo`3oool01@3oool00`000000oooo0?ooo`0g0?ooo`030?l0003o
ool0oooo05/0oooo00<000000?ooo`3oool0G03oool00`3o0000oooo0?ooo`0f0?ooo`030000003o
ool0oooo01d0oooo000m0?ooo`030=nLB01d_ol0oooo00H0oooo00<0omnL04P0001d_ol02`3oool0
0`3OW4P0M;oo0?ooo`090?ooo`030000003oool0oooo03L0oooo00<0o`000?ooo`3oool0F`3oool0
0`000000oooo0?ooo`1L0?ooo`030?l0003oool0oooo03H0oooo00<000000?ooo`3oool07@3oool0
03h0oooo00<0_gA809cOo`3oool01@3oool00`3ogi`0B00007Boo`0<0?ooo`030;mdB02Lgol0oooo
00P0oooo00<000000?ooo`3oool0=`3oool00`3o0000oooo0?ooo`1K0?ooo`030000003oool0oooo
05`0oooo00<0o`000?ooo`3oool0=P3oool00`000000oooo0?ooo`0M0?ooo`00?P3oool00`3ooml0
W4Qd0;ooo`050?ooo`030?oOW0180000M;oo00`0oooo00<0oooO09a8M02oool0203oool3000003L0
oooo00<0o`000?ooo`3oool0F`3oool00`000000oooo0?ooo`1M0?ooo`030?l0003oool0oooo03<0
oooo0`00000O0?ooo`00?`3oool00`3o_g@007Bo0?ooo`040?ooo`030?oOW0180000M;oo00d0oooo
00<0okmd001d_`3oool01`3oool00`000000oooo0?ooo`0f0?ooo`030?l0003oool0oooo05`0oooo
00<000000?ooo`3oool0G@3oool00`3o0000oooo0?ooo`0e0?ooo`030000003oool0oooo01d0oooo
000o0?ooo`030?oo_`1d04P0W=oo00@0oooo00<0omnL04P0001d_ol03@3oool00`3ookl0M01809cO
o`070?ooo`030000003oool0oooo03H0oooo00<0o`000?ooo`3oool0G03oool00`000000oooo0?oo
o`1M0?ooo`030?l0003oool0oooo03D0oooo00<000000?ooo`3oool07@3oool003l0oooo00<0oono
07@00018W=l0103oool00`3ogi`0B00007Boo`0=0?ooo`030?oo_`1d0000B9cO00L0oooo00<00000
0?ooo`3oool0=P3oool00`3o0000oooo0?ooo`0M0?ooo`030000003oool0oooo03`0oooo00<00000
0?ooo`3oool0?03oool00`000000oooo0?ooo`0N0?ooo`030?l0003oool0oooo03D0oooo00<00000
0?ooo`3oool07@3oool003X0oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0B00004RLg`040?oo
o`030?oOW0180000M;oo00P0oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0B00004RLg`070?oo
o`030000003oool0oooo03H0oooo00<0o`000?ooo`3oool0703oool3000003d0oooo00<000000?oo
o`3oool0>`3oool3000001l0oooo00<0o`000?ooo`3oool0=@3oool00`000000oooo0?ooo`0M0?oo
o`00>P3oool0203ooml0W4Q809cOo`3oool0oono07@0000007@0_ooo0P3oool01@3ooml0W4P00000
00000000M;oo00P0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`L0oooo00<00000
0?ooo`3oool0=P3oool00`3o0000oooo0?ooo`0L0?ooo`040000003oool0oooo000003`0oooo00<0
00000?ooo`3oool0>`3oool00`000000oooo0000000O0?ooo`030?l0003oool0oooo03D0oooo00<0
00000?ooo`3oool07@3oool003/0oooo00H0oooO09a80000000000000000M02oool50?ooo`030?no
M0000000M;oo00T0oooo00H0oooO09a80000000000000000M02oool80?ooo`030000003oool0oooo
03H0oooo00<0o`000?ooo`3oool06`3oool01@000000oooo0?ooo`3oool0000003`0oooo00<00000
0?ooo`3oool0>`3oool010000000oooo0?ooo`00000N0?ooo`030?l0003oool0oooo03D0oooo00<0
00000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0=P3oool00`3o0000oooo0?ooo`0K
0?ooo`050000003oool0oooo0?ooo`000000?03oool00`000000oooo0?ooo`0j0?ooo`050000003o
ool0oooo0?ooo`0000007P3oool00`3o0000oooo0?ooo`0e0?ooo`030000003oool0oooo01d0oooo
001P0?ooo`<00000=P3oool00`3o0000oooo0?ooo`0K0?ooo`050000003oool0oooo0?ooo`000000
?03oool2000003/0oooo00D000000?ooo`3oool0oooo0000000N0?ooo`030?l0003oool0oooo03@0
oooo0P00000O0?ooo`00H03oool00`000000oooo0?ooo`0f0?ooo`030?l0003oool0oooo01/0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01X0oooo
0`00000M0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0K0?ooo`030?l0003oool0oooo03D0oooo00<000000?ooo`3oool07@3oool00600oooo00<0
00000?ooo`3oool0=P3oool00`3o0000oooo0?ooo`0K0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool06P3oool2000001`0oooo0P0000000`3oool000000?ooo`0J0?ooo`<00000703o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01/0oooo00<0o`000?ooo`3oool0=@3o
ool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`0f0?ooo`030?l0003oool0
oooo01/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0I0?ooo`040000003oool0
oooo000001X0oooo00D000000?ooo`000000oooo0000000K0?ooo`030000003oool0000001`0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0K0?ooo`030?l0003oool0oooo03D0oooo
00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0=P3oool00`3o0000oooo0?oo
o`0J0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool06@3oool010000000oooo0?oo
o`00000J0?ooo`050000003oool000000?ooo`0000006P3oool010000000oooo0?ooo`00000K0?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool06`3oool00`3o0000oooo0?ooo`0e0?oo
o`030000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo03H0oooo00<0o`000?ooo`3o
ool06P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01P0oooo00@000000?ooo`3o
ool000006P3oool01@000000oooo0000003oool0000001X0oooo00@000000?ooo`3oool000006`3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01X0oooo00<0o`000?ooo`3oool0=@3o
ool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`0f0?ooo`030?l0003oool0
oooo01X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0H0?ooo`040000003oool0
oooo000001X0oooo00D000000?ooo`000000oooo0000000J0?ooo`050000003oool0oooo0?ooo`00
00006P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01X0oooo00<0o`000?ooo`3o
ool0=@3oool00`000000oooo0?ooo`0M0?ooo`00H03oool3000003H0oooo00<0o`000?ooo`3oool0
6P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool000000080oooo00<000000?ooo`3oool0
5`3oool01@000000oooo0?ooo`3oool0000001X0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0J0?ooo`030?l0003oool0oooo03@0oooo0P00000O0?ooo`00H03oool00`000000oooo
0?ooo`0f0?ooo`030?l0003oool0oooo01X0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool05P3oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo01L0oooo00D000000?ooo`3oool0oooo0000000J0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool06P3oool00`3o0000oooo0?ooo`0e0?oo
o`030000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo03H0oooo00<0o`000?ooo`3o
ool06@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`0F0?ooo`040000003oool0oooo00000080oooo00<000000?oo
o`3oool05`3oool01@000000oooo0?ooo`3oool0000001T0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`0I0?ooo`030?l0003oool0oooo03D0oooo00<000000?ooo`3oool07@3oool0
0600oooo00<000000?ooo`3oool0=P3oool00`3o0000oooo0?ooo`0I0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
01H0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo01T0oooo00<0o`000?ooo`3oool0=@3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`00
0000oooo0?ooo`0f0?ooo`030?l0003oool0oooo01T0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`0G0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool05P3oool01000
0000oooo0?ooo`0000020?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool06@3oool0
0`3o0000oooo0?ooo`0e0?ooo`030000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo
03H0oooo00<0o`000?ooo`3oool06@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0F0?ooo`040000003oool0oooo
00000080oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0I0?ooo`030?l0003oool0
oooo03D0oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0=P3oool00`3o
0000oooo0?ooo`0I0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool05P3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo01@0oooo00D000000?ooo`3oool0oooo00000003
0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0F
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool06@3oool00`3o0000oooo0?ooo`0e
0?ooo`030000003oool0oooo01d0oooo001P0?ooo`<00000=P3oool00`3o0000oooo0?ooo`0H0?oo
o`030000003oool0oooo00L0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo01@0oooo00D000000?ooo`3oool0oooo000000030?ooo`030000003oool0
oooo01D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool06@3oool00`3o0000oooo0?ooo`0d0?ooo`8000007`3oool0
0600oooo00<000000?ooo`3oool0=P3oool00`3o0000oooo0?ooo`0H0?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
01@0oooo00D000000?ooo`3oool0oooo000000030?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool06@3oool00`3o0000oooo0?ooo`0e0?ooo`030000003oool0oooo01d0oooo000j0?oo
o`030;md00000000000000@0000000D0B9cO0?ooo`3oool0oooo0;md0003000000070018W03Oool0
oooo0?ooo`3o_g@0000007Boo`050?ooo`040?oog`2LB00004RL0=ooo`L0oooo00<000000?ooo`3o
ool0=P3oool00`3o0000oooo0?ooo`0H0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool05@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01@0oooo00D000000?ooo`3o
ool0oooo000000030?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0E0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0603oool00`3o
0000oooo0?ooo`0e0?ooo`030000003oool0oooo01d0oooo000k0?ooo`030=nLB0000000000000<0
000000<004RL0=ooo`3oool00P3oool00`3ogi`0B00007Boo`040?ooo`030?noM0000000M;oo00D0
oooo00@0oooO09a80000B9`0gooo1`3oool00`000000oooo0?ooo`0f0?ooo`030?l0003oool0oooo
01P0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool0503oool01@000000oooo0?ooo`3oool0000000<0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool0203oool00`000000oooo0?ooo`0H0?ooo`030?l0003oool0oooo03D0oooo00<00000
0?ooo`3oool07@3oool003`0oooo00<0gia804RLg`3oool00P3oool00`3OW7@0W=oo0?ooo`020?oo
o`030?oOW0180000M;oo00`0oooo00@0oooO09a80000B9`0gooo1`3oool00`000000oooo0?ooo`0f
0?ooo`030?l0003oool0oooo01P0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0E
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0503oool01@000000oooo0?ooo`3o
ool0000000<0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0H0?ooo`030?l0003o
ool0oooo03D0oooo00<000000?ooo`3oool07@3oool003d0oooo00<0gia807Boo`3oool01P3oool0
0`3ogi`0B00007Boo`080?ooo`030;md00000000000000D0000000<0M;oo0?ooo`3oool0103oool0
0`000000oooo0?ooo`0f0?ooo`030?l0003oool0oooo01P0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0503oool0
1@000000oooo0?ooo`3oool0000000@0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0203oool00`000000oooo0?oo
o`0H0?ooo`030?l0003oool0oooo03D0oooo00<000000?ooo`3oool07@3oool003h0oooo00<0_gA8
09cOo`3oool01@3oool00`3ogi`0B00007Boo`080?ooo`080?oOW01dW=l0oooo0?ooo`3ooml0W4P0
0018W03Oool70?ooo`030000003oool0oooo03H0oooo00<0o`000?ooo`3oool05`3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo01P0oooo00<0o`000?ooo`3oool0=@3oool00`000000
oooo0?ooo`0M0?ooo`00?P3oool00`3ooml0W4Qd0;ooo`050?ooo`030?oOW0180000M;oo00T0oooo
00L0gia807Boo`3oool0oooO09a80000B9`0gooo00L0oooo0`00000f0?ooo`030?l0003oool0oooo
01L0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0H0?ooo`030?l0003oool0oooo
03<0oooo0`00000O0?ooo`00?`3oool00`3o_g@007Bo0?ooo`040?ooo`030?oOW0180000M;oo00T0
oooo00L0oooO09a8M02oool0oooO09a80000B9`0gooo00L0oooo00<000000?ooo`3oool0=P3oool0
0`3o0000oooo0?ooo`0G0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0603oool0
0`3o0000oooo0?ooo`0e0?ooo`030000003oool0oooo01d0oooo000o0?ooo`030?oo_`1d04P0W=oo
00@0oooo00<0omnL04P0001d_ol02P3oool01P3ogi`0B7Bo0?oog`2LB00004RL0=ooo`L0oooo00<0
00000?ooo`3oool0=P3oool00`3o0000oooo0?ooo`0G0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool04`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool0603oool00`3o0000oooo0?ooo`0e0?ooo`030000003oool0oooo01d0oooo000o
0?ooo`030?oo_`1d0000B9cO00@0oooo00<0omnL04P0001d_ol02`3oool01@3OW4P0M;oO09a80000
B9`0gooo00L0oooo00<000000?ooo`3oool0=P3oool00`3o0000oooo0?ooo`0G0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo0180oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool05`3oool00`3o0000oooo0?ooo`0e0?ooo`030000003o
ool0oooo01d0oooo000j0?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l0103oool0
0`3ogi`0B00007Boo`0;0?ooo`050?oog`2LB4P0B4P00018W03Oool01`3oool00`000000oooo0?oo
o`0f0?ooo`030?l0003oool0oooo01L0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?oo
o`0C0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`0G0?ooo`030?l0003oool0oooo03D0oooo00<000000?ooo`3oool07@3oool003X0oooo00P0oooO
09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`80oooo00D0oooO09a800000000000007Boo`0<0?oo
o`040?oOW018000004RL0=ooo`L0oooo00<000000?ooo`3oool0=P3oool00`3o0000oooo0?ooo`0G
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0B
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool05`3oool00`3o0000oooo0?ooo`0e
0?ooo`030000003oool0oooo01d0oooo000k0?ooo`060?oog`2LB00000000000000007@0_ooo1@3o
ool00`3o_g@0000007Boo`0=0?ooo`030=nLB000B9`0gooo00L0oooo00<000000?ooo`3oool0=P3o
ool00`3o0000oooo0?ooo`0G0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool04P3o
ool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool05`3o
ool00`3o0000oooo0?ooo`0e0?ooo`030000003oool0oooo01d0oooo001P0?ooo`<00000=P3oool0
0`3o0000oooo0?ooo`0F0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool05`3oool0
0`3o0000oooo0?ooo`0d0?ooo`8000007`3oool00600oooo00<000000?ooo`3oool0=P3oool00`3o
0000oooo0?ooo`0F0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool05`3oool00`3o
0000oooo0?ooo`0e0?ooo`030000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo03H0
oooo00<0o`000?ooo`3oool05P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01L0
oooo00<0o`000?ooo`3oool0=@3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo
0?ooo`0f0?ooo`030?l0003oool0oooo01H0oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0G0?ooo`030?l0003oool0oooo03D0oooo00<000000?ooo`3oool07@3oool00600oooo00<0
00000?ooo`3oool0=P3oool00`3o0000oooo0?ooo`0F0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool05P3oool00`3o0000oooo0?ooo`0e0?ooo`030000003oool0oooo01d0oooo001P
0?ooo`030000003oool0oooo03H0oooo00<0o`000?ooo`3oool05P3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo01H0oooo00<0o`000?ooo`3oool0=@3oool00`000000oooo0?ooo`0M
0?ooo`00H03oool00`000000oooo0?ooo`0f0?ooo`030?l0003oool0oooo01H0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00P0oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0140oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0F0?ooo`030?l0003oool0oooo03D0oooo00<000000?oo
o`3oool07@3oool00600oooo00<000000?ooo`3oool0=P3oool00`3o0000oooo0?ooo`0F0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`080?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00l0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool05P3oool00`3o0000oooo0?ooo`0e0?ooo`03
0000003oool0oooo01d0oooo001P0?ooo`<00000=P3oool00`3o0000oooo0?ooo`0F0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo0100oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00l0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool05P3oool00`3o0000oooo0?ooo`0d0?ooo`800000
7`3oool00600oooo00<000000?ooo`3oool0=P3oool00`3o0000oooo0?ooo`0E0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool05P3oool00`3o0000oooo0?ooo`0e0?ooo`030000003o
ool0oooo01d0oooo001P0?ooo`030000003oool0oooo03H0oooo00<0o`000?ooo`3oool05@3oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01H0oooo00<0o`000?ooo`3oool0=@3oool0
0`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`0f0?ooo`030?l0003oool0oooo
01D0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0F0?ooo`030?l0003oool0oooo
03D0oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0=P3oool00`3o0000
oooo0?ooo`0E0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`0:0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool02@3oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool05P3oool00`3o0000
oooo0?ooo`0e0?ooo`030000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo03H0oooo
00<0o`000?ooo`3oool05@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00l0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00T0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01H0oooo
00<0o`000?ooo`3oool0=@3oool00`000000oooo0?ooo`0M0?ooo`008P3oool00`3OW7@0W=oo0?oo
o`0k0?ooo`030000003oool0oooo03H0oooo00<0o`000?ooo`3oool05@3oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`0?0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo01D0oooo00<0o`000?ooo`3oool0=@3oool00`000000oooo0?oo
o`0M0?ooo`008P3oool00`3OW4P007Bo0?ooo`0k0?ooo`<00000=P3oool00`3o0000oooo0?ooo`0E
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool05@3oool00`3o0000oooo0?ooo`0d
0?ooo`8000007`3oool00280oooo00D0gia8000000000000000007Boo`0i0?ooo`030000003oool0
oooo03H0oooo00<0o`000?ooo`3oool05@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo00l0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo01D0oooo00<0o`000?ooo`3oool0=@3oool00`000000oooo0?ooo`0M0?ooo`008P3oool00`3O
W4P0000000000003000000030000M02oool0oooo00l0oooo00<0_g@0000000000000100000001@18
W=l0oooo0?ooo`3oool0_g@000<0000000L004RL0=ooo`3oool0oooo0?noM0000000M;oo00<0oooo
00D0omnL04P000000000000007Boo`080?ooo`030000003oool0oooo03H0oooo00<0o`000?ooo`3o
ool05@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool03`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01D0oooo00<0o`000?ooo`3o
ool0=@3oool00`000000oooo0?ooo`0M0?ooo`008P3oool01P3OW7@0W=oo0?ooo`3oool0omnL04P0
00<0000000<004RL0=ooo`3oool03@3oool00`3OW4P0000000000003000000030018W03Oool0oooo
0080oooo00<0omnL04P0001d_ol0103oool00`3o_g@0000007Boo`020?ooo`070?oOW0180000B9cO
0?ooo`3ookl0M00007Boo`070?ooo`030000003oool0oooo03H0oooo00<0o`000?ooo`3oool05@3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01D0oooo00<0o`000?ooo`3oool0=@3o
ool00`000000oooo0?ooo`0M0?ooo`00:@3oool00`3o_g@000000000000200000003001d_`3oool0
oooo00/0oooo00<0gia804RLg`3oool00P3oool00`3OW7@0W=oo0?ooo`020?ooo`030?oOW0180000
M;oo00P0oooo00<0oooO09a80000M;l00`3oool00`3o_g@004RL0=ooo`060?ooo`030000003oool0
oooo03H0oooo00<0o`000?ooo`3oool0503oool00`000000oooo0?ooo`0?0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo01D0oooo00<0o`000?ooo`3oool0=@3oool00`000000oooo0?ooo`0M0?ooo`00;03oool0103o
_g@000000000M02oool=0?ooo`030=nLB01d_ol0oooo00H0oooo00<0omnL04P0001d_ol0203oool0
0`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00H0oooo00<000000?ooo`3oool0=P3oool0
0`3o0000oooo0?ooo`0D0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool05@3oool0
0`3o0000oooo0?ooo`0e0?ooo`030000003oool0oooo01d0oooo000Y0?ooo`040?noM00000000000
001d_a40oooo00<0_gA809cOo`3oool01@3oool00`3ogi`0B00007Boo`080?ooo`030?oOW0180000
M;oo00<0oooo00<0okmd0000B02Lgol01P3oool00`000000oooo0?ooo`0f0?ooo`030?l0003oool0
oooo01@0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0
oooo00l0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0E0?ooo`030?l0003oool0
oooo03D0oooo00<000000?ooo`3oool07@3oool00280oooo00T0giad09cOo`3oool0oooo0?oOW018
000000000018W03Oool04`3oool00`3ooml0W4Qd0;ooo`050?ooo`030?oOW0180000M;oo00P0oooo
00<0omnL04P0001d_ol00`3oool00`2oM000001809cOo`060?ooo`<00000=P3oool00`3o0000oooo
0?ooo`0D0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`050?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0?0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool05P3oool00`3o0000oooo
0?ooo`0b0?ooo`<000007`3oool00280oooo00H0gia800000000000000000000M02ooolG0?ooo`03
0?noM000M;l0oooo00@0oooo00<0omnL04P0001d_ol0203oool0203ookl0M00000000018W=l0oooo
0?noM000000007Bo1`3oool00`000000oooo0?ooo`0f0?ooo`030?l0003oool0oooo01@0oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool03@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool03@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`0F0?ooo`030?l0003oool0oooo03@0oooo00<0
00000?ooo`3oool07@3oool00280oooo00<0gia80000001d_ol06P3oool00`3ookl0M01809cOo`04
0?ooo`030?oOW0180000M;oo00T0oooo00<0_g@00018W02oW4P00P0000000`00B9`0gooo0?ooo`06
0?ooo`030000003oool0oooo03H0oooo00<0o`000?ooo`3oool0503oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo01H0oooo00<0o`000?ooo`3oool0=03oool00`000000oooo0?ooo`0M
0?ooo`008P3oool00`3OW7@0W=oo0?ooo`0J0?ooo`030?oo_`1d0000B9cO00@0oooo00<0omnL04P0
001d_ol02@3oool00`3ogi`0B00007Boo`0;0?ooo`030000003oool0oooo03D0oooo00<0o`000?oo
o`3oool05@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo01H0oooo00<0o`000?oo
o`3oool0=03oool00`000000oooo0?ooo`0M0?ooo`00>P3oool00`3o_i`0W=oo0?ooo`020?ooo`03
0?oOW0180000B9cO00@0oooo00<0omnL04P0001d_ol02P3oool00`3OW4P0001809cOo`0:0?ooo`03
0000003oool0oooo03D0oooo00<0o`000?ooo`3oool05@3oool00`000000oooo0?ooo`0?0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00/0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo01D0oooo00<0o`000?ooo`3oool0=03oool00`000000oooo0?ooo`0M0?ooo`00
>P3oool0203ooml0W4Q809cOo`3oool0oono07@0000007@0_ooo0P3oool01@3ooml0W4P000000000
0000M;oo00/0oooo00<0okmd00000000M;l02@3oool00`000000oooo0?ooo`0e0?ooo`030?l0003o
ool0oooo01D0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0E0?ooo`030?l0003o
ool0oooo03@0oooo00<000000?ooo`3oool07@3oool002`0oooo00H0oooO09a80000B9`0gooo0?oo
g`2LW;l90?ooo`060?oog`2LB00000000000000007@0_ooo1@3oool00`3o_g@0000007Boo`0<0?oo
o`050?oog`2LB00000000000M02oool01P3oool00`000000oooo0?ooo`050?oooa0000000P3oool7
000000/0oooo1`0000050?ooo`030?l0003oool0oooo00<0oooo1`00000;0?ooo`P000002`3oool7
000000P0oooo00<000000?ooo`3oool01`0000050?ooo`030000003oool0oooo00<0oooo1`000002
0?ooo`030000003oool0oooo00D0oooo200000000`3oool000000?ooo`090?ooo`L000002`3oool7
000000T0oooo00<000000?ooo`0000001P00000:0?ooo`P00000303oool7000000/0oooo1`00000;
0?oooa4000001P3oool00`000000oooo0?ooo`0M0?ooo`00;03oool01P3ookl0M0000018W03Oool0
_gA809cOobh0oooo0`00000E0?ooo`H000006P3oool00`3o0000oooo0?ooo`0D0?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool05@3oool00`3o0000oooo0?ooo`0H0?ooo`H000005@3o
ool2000001l0oooo000]0?ooo`040=nLB0000000000007Boobl0oooo00<000000?ooo`3oool06`3o
ool3000001L0oooo00<0o`000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo01D0oooo00<0o`000?ooo`3oool05@3oool3000001`0oooo00<000000?ooo`3oool07@3o
ool00600oooo00<000000?ooo`3oool07P3oool2000001D0oooo00<0o`000?ooo`3oool0503oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01D0oooo00<0o`000?ooo`3oool04P3oool3
000001l0oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0803oool00`00
0000oooo0?ooo`0B0?ooo`030?l0003oool0oooo01@0oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0E0?ooo`030?l0003oool0oooo0140oooo00<000000?ooo`3oool0803oool00`00
0000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`0Q0?ooo`8000004P3oool00`3o0000
oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`0=0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`0=0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool05@3oool00`3o0000
oooo0?ooo`0@0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool07@3oool00600oooo
00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0?0?ooo`030?l0003oool0oooo01@0oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0E0?ooo`030?l0003oool0oooo00l0oooo
00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?oo
o`0T0?ooo`030000003oool0oooo00h0oooo00<0o`000?ooo`3oool0503oool00`000000oooo0?oo
o`0A0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo01D0oooo00<0o`000?ooo`3oool03@3oool2000002D0oooo00<0
00000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0=
0?ooo`030?l0003oool0oooo01@0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0E
0?ooo`030?l0003oool0oooo00`0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0M
0?ooo`009P3oool01@3ogi`0M9cO0?ooo`3ookl0M0000080000000<0001d0;ooo`3oool0<03oool3
000002H0oooo00<000000?ooo`3oool0303oool00`3o0000oooo0?ooo`0D0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo0180oooo00<000000?ooo`3oool0503oool00`3o0000oooo0?ooo`0;0?ooo`030000003oool0
oooo02D0oooo0P00000O0?ooo`009P3oool00`3ooml0W4P0000000030000000407Bo_`1d000004RL
0=oooc00oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`0;0?ooo`030?l0003oool0
oooo01@0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0D0?ooo`030?l0003oool0
oooo00/0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0M0?ooo`00;P3oool01P2o
M4P0W=no07@00000000004RL0=ooob`0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?oo
o`0;0?ooo`030?l0003oool0oooo01<0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`0D0?ooo`030?l0003oool0oooo00X0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?oo
o`0M0?ooo`00;03oool01@3OW4P000000000000007@0_ooo02l0oooo00<000000?ooo`3oool0:03o
ool00`000000oooo0?ooo`0:0?ooo`030?l0003oool0oooo01<0oooo00<000000?ooo`3oool04`3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool04P3o
ool00`000000oooo0?ooo`0D0?ooo`030?l0003oool0oooo00X0oooo00<000000?ooo`3oool09`3o
ool00`000000oooo0?ooo`0M0?ooo`00:03oool0203o_g@00000000000000000B9cO0?ooo`2oM4P0
W=oo<03oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo00T0oooo00<0o`000?ooo`3o
ool04`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool02`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01@0oooo00<0o`000?ooo`3o
ool02@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo01d0oooo000T0?ooo`060;md
0000000000000000000007@0_ooo0`3oool00`3ookl0M4RL0=ooo`0`0?ooo`030000003oool0oooo
02T0oooo00<000000?ooo`3oool02@3oool00`3o0000oooo0?ooo`0C0?ooo`030000003oool0oooo
01<0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool0503oool00`3o0000oooo0?ooo`090?ooo`030000003oool0oooo
02P0oooo00<000000?ooo`3oool07@3oool00280oooo00@0oooO09a80000000007Bo1P3oool00`3o
oml0W4P004RLg`0a0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0203oool00`3o
0000oooo0?ooo`0C0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0503oool00`3o
0000oooo0?ooo`080?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool07@3oool002X0
oooo00@0_g@000000000000007Bo<P3oool3000002X0oooo00<000000?ooo`3oool0203oool00`3o
0000oooo0?ooo`0C0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0503oool00`3o
0000oooo0?ooo`080?ooo`030000003oool0oooo02P0oooo0P00000O0?ooo`009`3oool01P3ooml0
W4P0000000000000001809cOoc<0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`07
0?ooo`030?l0003oool0oooo01<0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`09
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0D
0?ooo`030?l0003oool0oooo00L0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0M
0?ooo`009P3oool0103ooml0W4P00018W03Oool@0?ooo`030;md00000000000000@0000000D0B9cO
0?ooo`3oool0oooo0;md0003000000070018W03Oool0oooo0?ooo`3o_g@0000007Boo`030?ooo`05
0=nLB0000000000000000018W=l0203oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo
00L0oooo00<0o`000?ooo`3oool04`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
00l0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo
01<0oooo00<0o`000?ooo`3oool01`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo
01d0oooo000V0?ooo`030?oOW01dW=l0oooo0180oooo00<0gia80000000000000`0000000`00B9`0
gooo0?ooo`020?ooo`030?oOW0180000M;oo00@0oooo00<0okmd0000001d_ol00P3oool01`3OW4P0
000007Boo`3oool0oooO09a80018W=l01`3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0
oooo00H0oooo00<0o`000?ooo`3oool04`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0
oooo01<0oooo00<0o`000?ooo`3oool01P3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0
oooo01d0oooo000l0?ooo`030=nLB018W=l0oooo0080oooo00<0giad09cOo`3oool00P3oool00`3o
gi`0B00007Boo`080?ooo`030?oog`2LB00007Bo00<0oooo00<0okmd0018W03Oool01P3oool00`00
0000oooo0?ooo`0/0?ooo`030000003oool0oooo00H0oooo00<0o`000?ooo`3oool04`3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo01<0oooo00<0o`000?ooo`3oool01P3oool00`00
0000oooo0?ooo`0[0?ooo`030000003oool0oooo01d0oooo000m0?ooo`030=nLB01d_ol0oooo00H0
oooo00<0omnL04P0001d_ol02@3oool00`2oM00007Bo0?ooo`020?ooo`030;md0000B9`0gooo00H0
oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`050?ooo`030?l0003oool0oooo0180
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0C0?ooo`030?l0003oool0oooo00H0
oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0M0?ooo`00?P3oool00`2oM4P0W=oo
0?ooo`050?ooo`030?oOW0180000M;oo00T0oooo00L0omnL04P0M02oool0oooo0;md00000000M;oo
00L0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`050?ooo`030?l0003oool0oooo
0180oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0C0?ooo`030?l0003oool0oooo
00D0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0M0?ooo`00?P3oool00`3ooml0
W4Qd0;ooo`050?ooo`030?oOW0180000M;oo00X0oooo00D0oono07A8B0180000000004RLg`080?oo
o`<00000;@3oool00`000000oooo0?ooo`050?ooo`030?l0003oool0oooo0180oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool0203oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool0203oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00T0oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`0C0?ooo`030?l0003oool0oooo00D0oooo00<000000?oo
o`3oool0:P3oool3000001l0oooo000o0?ooo`030?noM000M;l0oooo00@0oooo00<0omnL04P0001d
_ol02P3oool0103ogi`0B00000000000M;l90?ooo`030000003oool0oooo02h0oooo00<000000?oo
o`3oool0103oool00`3o0000oooo0?ooo`0B0?ooo`030000003oool0oooo01@0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00P0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00P0oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo01@0oooo00<000000?oo
o`3oool04`3oool00`3o0000oooo0?ooo`040?ooo`030000003oool0oooo02d0oooo00<000000?oo
o`3oool07@3oool003l0oooo00<0oono07@0B02Lgol0103oool00`3ogi`0B00007Boo`090?ooo`06
0?oOW0180000001d0;ooo`3o_g@007Bo203oool00`000000oooo0?ooo`0^0?ooo`030000003oool0
oooo00@0oooo00<0o`000?ooo`3oool04P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo01<0oooo00<0o`000?ooo`3oool0103oool00`000000oooo0?ooo`0]0?ooo`030000003oool0
oooo01d0oooo000o0?ooo`030?oo_`1d0000B9cO00@0oooo00<0omnL04P0001d_ol02@3oool00`2o
M000001d0;ooo`020?ooo`030;md0018W=l0oooo00H0oooo00<000000?ooo`3oool0;P3oool00`00
0000oooo0?ooo`040?ooo`030?l0003oool0oooo0180oooo00<000000?ooo`3oool05@3oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0203oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool05@3oool00`00
0000oooo0?ooo`0B0?ooo`030?l0003oool0oooo00<0oooo00<000000?ooo`3oool0;P3oool00`00
0000oooo0?ooo`0M0?ooo`00>P3oool00`3o_i`0W=oo0?ooo`020?ooo`030?oOW0180000B9cO00@0
oooo00<0omnL04P0001d_ol02@3oool00`2oM00004RL0=ooo`020?ooo`030=nLB000M;l0oooo00H0
oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`030?ooo`030?l0003oool0oooo0180
oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00P0
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0B0?ooo`030?l0003oool0oooo00<0
oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0M0?ooo`00>P3oool0203ooml0W4Q8
09cOo`3oool0oono07@0000007@0_ooo0P3oool01@3ooml0W4P0000000000000M;oo00T0oooo00L0
okmd0000001d_ol0oooo0?oo_`1d0000M;oo00L0oooo00<000000?ooo`3oool0;`3oool00`000000
oooo0?ooo`030?ooo`030?l0003oool0oooo0180oooo00<000000?ooo`3oool05@3oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool05@3oool00`000000
oooo0?ooo`0B0?ooo`030?l0003oool0oooo0080oooo00<000000?ooo`3oool0;`3oool00`000000
oooo0?ooo`0M0?ooo`00>`3oool01P3ooml0W4P0000000000000001d0;ooo`D0oooo00<0okmd0000
001d_ol02P3oool01@3o_g@00000000000000000B9cO00P0oooo00<000000?ooo`3oool0;`3oool0
0`000000oooo0?ooo`030?ooo`030?l0003oool0oooo0180oooo00<000000?ooo`3oool05@3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`080?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool05@3oool0
0`000000oooo0?ooo`0B0?ooo`030?l0003oool0oooo0080oooo00<000000?ooo`3oool0;`3oool0
0`000000oooo0?ooo`0M0?ooo`00H03oool300000300oooo00<000000?ooo`3oool00P3oool00`3o
0000oooo0?ooo`0B0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0203oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool04P3oool00`3o
0000oooo0?ooo`020?ooo`030000003oool0oooo02h0oooo0P00000O0?ooo`00H03oool00`000000
oooo0?ooo`0`0?ooo`030000003oool0oooo0080oooo00<0o`000?ooo`3oool04@3oool00`000000
oooo0?ooo`0F0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool04P3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0203oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`0F0?ooo`030000003oool0oooo0180oooo00D0o`000?ooo`3oool0oooo0000000b0?oo
o`030000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3o
ool00P3oool00`3o0000oooo0?ooo`0A0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool0203oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool04P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3o
ool04P3oool01@3o0000oooo0?ooo`3oool000000380oooo00<000000?ooo`3oool07@3oool00600
oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`020?ooo`030?l0003oool0oooo0140
oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0180
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00P0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0B0?ooo`050?l0003oool0oooo0?oo
o`000000<P3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`0a0?ooo`05
0000003oool0oooo0?ooo`3o00004`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
00P0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
0140oooo00D0o`000?ooo`3oool0oooo0000000b0?ooo`030000003oool0oooo01d0oooo001P0?oo
o`030000003oool0oooo0340oooo00D000000?ooo`3oool0oooo0?l0000C0?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo01L0oooo00<000000?ooo`3oool04@3oool0103o0000oooo0?ooo`00000c0?ooo`030000003o
ool0oooo01d0oooo001P0?ooo`030000003oool0oooo0340oooo00D000000?ooo`3oool0oooo0?l0
000C0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0B0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0203oool00`000000oooo0?oo
o`090?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool04@3oool0103o0000oooo0?oo
o`00000c0?ooo`030000003oool0oooo01d0oooo001P0?ooo`<00000<@3oool01@000000oooo0?oo
o`3oool0o`0001<0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo01<0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0A0?ooo`040?l0
003oool0oooo00000380oooo0P00000O0?ooo`00H03oool00`000000oooo0?ooo`0b0?ooo`040000
003oool0oooo0?l001<0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo01@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0A0?ooo`04
0?l0003oool0oooo000003<0oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3o
ool0<P3oool010000000oooo0?ooo`3o000C0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool04@3oool00`3o0000oooo0000000d0?ooo`030000003oool0oooo01d0oooo001P0?ooo`03
0000003oool0oooo0380oooo00@000000?ooo`3oool0o`004`3oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo0140oooo00<0o`000?ooo`000000=03oool00`000000oooo0?ooo`0M0?oo
o`00H03oool00`000000oooo0?ooo`0c0?ooo`030000003oool0o`000180oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0A0?ooo`030?l0003oool0000003@0oooo00<000000?ooo`3o
ool07@3oool00600oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?l0000B0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool0403oool00`3o0000oooo0000000d0?ooo`030000
003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3o00004P3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0503o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0100oooo00<0o`000000003oool0=03o
ool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`0c0?ooo`030000003oool0
o`000180oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo01D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0@0?ooo`030?l000000000
oooo03@0oooo00<000000?ooo`3oool07@3oool00600oooo0`00000d0?ooo`030000003o0000oooo
0140oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0@0?ooo`030?l000000000oooo
03<0oooo0P00000O0?ooo`00H03oool00`000000oooo0?ooo`0d0?ooo`030000003o0000oooo0140
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01H0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0@0?ooo`030?l000000000oooo03@0
oooo00<000000?ooo`3oool07@3oool004P0oooo00<0_g@0000000000000100000000`18W=l0oooo
0;md00060000000304RLg`3oool0oooo00D0oooo00<000000?ooo`3oool0=03oool00`000000o`00
0?ooo`0A0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0F0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool05P3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0d0?ooo`030000003oool0oooo01d0oooo00190?ooo`030=nLB0000000000000<0000000@0
04RL0=ooo`3oool0gia81@0000000`00B9`0gooo0?ooo`050?ooo`030000003oool0oooo03@0oooo
00<000000?l0003oool04@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0M0?ooo`00BP3oool00`3OW4P0B9cO0?oo
o`020?ooo`060=nLM02Lgol0oooo0?ooo`3OW4P0B9cO0`3oool00`3OW7@0W=oo0?ooo`050?ooo`03
0000003oool0oooo03D0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool05`3oool01@000000oooo0?ooo`3oool00000
00`0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0M0?ooo`00B`3oool00`3OW4P0
M;oo0?ooo`060?ooo`030=nLB01d_ol0oooo00T0oooo00<000000?ooo`3oool0=@3oool00`000000
oooo0?ooo`0?0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0G0?ooo`050000003oool0oooo0?ooo`000000303oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo01`0oooo00<000000?ooo`3oool03P3oool00`000000o`000?ooo`0e0?oo
o`030000003oool0oooo01d0oooo001<0?ooo`030;mdB02Lgol0oooo00H0oooo00<0_gA809cOo`3o
ool0203oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01L0oooo00D000000?ooo`3o
ool0oooo0000000<0?ooo`030000003oool0oooo00/0oooo00D000000?ooo`3oool0oooo0000000I
0?ooo`050000003oool0oooo0?ooo`0000007P3oool00`000000oooo0?ooo`0>0?ooo`030000003o
0000oooo03D0oooo00<000000?ooo`3oool07@3oool004`0oooo00<0oooO09a8M02oool01P3oool0
0`3ooml0W4Qd0;ooo`080?ooo`<00000=@3oool00`3o000000000?ooo`0?0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0G0?ooo`040000003oool0
oooo000000d0oooo00<000000?ooo`3oool02`3oool010000000oooo0?ooo`00000J0?ooo`050000
003oool0oooo0?ooo`0000007P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0o`0003<0
oooo0`00000O0?ooo`00C@3oool00`3o_g@007Bo0?ooo`060?ooo`030?noM000M;l0oooo00L0oooo
00<000000?ooo`3oool0=@3oool00`3o000000000?ooo`0?0?ooo`030000003oool0oooo01`0oooo
00@000000?ooo`3oool000006P3oool010000000oooo0?ooo`00000=0?ooo`030000003oool0oooo
00/0oooo00@000000?ooo`3oool000006P3oool01@000000oooo0?ooo`3oool0000001h0oooo00<0
00000?ooo`3oool03@3oool010000000oooo0?ooo`3o000e0?ooo`030000003oool0oooo01d0oooo
001=0?ooo`030?oo_`1d04P0W=oo00H0oooo00<0oono07@0B02Lgol01`3oool00`000000oooo0?oo
o`0e0?ooo`030?l000000000oooo00l0oooo00<000000?ooo`3oool0703oool010000000oooo0?oo
o`00000J0?ooo`80000000<0oooo0000003oool0303oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0000001/0oooo00@000000?ooo`3oool000007P3oool00`000000oooo0?ooo`0=0?ooo`04
0000003oool0oooo0?l003D0oooo00<000000?ooo`3oool07@3oool004d0oooo00<0oono07@00018
W=l01P3oool00`3ookl0M00004RLg`070?ooo`030000003oool0oooo03D0oooo00<0o`000000003o
ool03P3oool00`000000oooo0?ooo`0M0?ooo`040000003oool0oooo000001/0oooo0`00000=0?oo
o`030000003oool0oooo00`0oooo0`00000K0?ooo`040000003oool00000000001h0oooo00<00000
0?ooo`3oool03@3oool010000000oooo0?ooo`3o000e0?ooo`030000003oool0oooo01d0oooo0018
0?ooo`030?noW02Lgol0oooo0080oooo00H0omnL04P00018W=l0oooo0?noW02Lgol30?ooo`030?oO
W0180000B9cO00L0oooo00<000000?ooo`3oool0=@3oool00`3o0000oooo0000000>0?ooo`030000
003oool0oooo01d0oooo0`00000/0?ooo`030000003oool0oooo02X0oooo0`00000P0?ooo`030000
003oool0oooo00/0oooo00D000000?ooo`3oool0oooo0?l0000e0?ooo`030000003oool0oooo01d0
oooo00180?ooo`0A0?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02oool0oooo0?oog`2LB4P0W=oo
0?ooo`3ookl0M0000000M02oool01`3oool00`000000oooo0?ooo`0e0?ooo`030?l0003oool00000
00h0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo
02/0oooo0P00000P0?ooo`030000003oool0oooo00/0oooo00D000000?ooo`3oool0oooo0?l0000e
0?ooo`030000003oool0oooo01d0oooo00190?ooo`060?oog`2LB00000000000000007@0_ooo0`3o
ool01P3ooml0W4P0000000000000001d0;ooo`P0oooo00<000000?ooo`3oool0=@3oool00`3o0000
oooo0000000>0?ooo`030000003oool0oooo04`0oooo00<000000?ooo`3oool0C@3oool00`000000
oooo0?ooo`0;0?ooo`050000003oool0oooo0?ooo`3o0000=@3oool00`000000oooo0?ooo`0M0?oo
o`00H03oool3000003D0oooo00<0o`000?ooo`0000003@3oool00`000000oooo0?ooo`1=0?ooo`03
0000003oool0oooo04d0oooo00<000000?ooo`3oool02`3oool01@000000oooo0?ooo`3oool0o`00
03@0oooo0P00000O0?ooo`00H03oool00`000000oooo0?ooo`0e0?ooo`040?l0003oool0oooo0000
00`0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool00P3oool00`3o0000oooo0?ooo`0c0?ooo`030000003oool0oooo
01d0oooo001P0?ooo`030000003oool0oooo03@0oooo00D0o`000?ooo`3oool0oooo0000000<0?oo
o`030000003oool0oooo04d0oooo00<000000?ooo`3oool0CP3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo0080oooo00<0o`000?ooo`3oool0<`3oool00`000000oooo0?ooo`0M0?oo
o`00H03oool00`000000oooo0?ooo`0d0?ooo`050?l0003oool0oooo0?ooo`000000303oool00`00
0000oooo0?ooo`1=0?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`020?ooo`030?l0003oool0oooo03<0oooo00<000000?ooo`3oool07@3oool00600
oooo00<000000?ooo`3oool0=03oool00`3o0000oooo0?ooo`020?ooo`030000003oool0oooo00P0
oooo00<000000?ooo`3oool0CP3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo00P0
oooo00<000000?ooo`3oool00`3oool00`3o0000oooo0?ooo`0c0?ooo`030000003oool0oooo01d0
oooo001P0?ooo`030000003oool0oooo03@0oooo00<0o`000?ooo`3oool00P3oool00`000000oooo
0?ooo`080?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0CP3oool00`000000oooo
0?ooo`080?ooo`030000003oool0oooo00<0oooo00<0o`000?ooo`3oool0<`3oool00`000000oooo
0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`0d0?ooo`030?l0003oool0oooo0080oooo00<0
00000?ooo`3oool0203oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo04h0oooo00<0
00000?ooo`3oool0203oool00`000000oooo0?ooo`030?ooo`030?l0003oool0oooo03<0oooo00<0
00000?ooo`3oool07@3oool00600oooo0`00000d0?ooo`030?l0003oool0oooo00<0oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo04h0oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`040?ooo`030?l0003oool0oooo0380oooo0P00000O
0?ooo`00H03oool00`000000oooo0?ooo`0d0?ooo`030?l0003oool0oooo00<0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo04h0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`040?ooo`030?l0003oool0oooo03<0oooo00<000000?oo
o`3oool07@3oool00600oooo00<000000?ooo`3oool0=03oool00`3o0000oooo0?ooo`030?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool0C`3oool00`000000oooo0?ooo`1?0?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool0103oool00`3o0000oooo0?ooo`0c0?ooo`03
0000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo03@0oooo00<0o`000?ooo`3oool0
103oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0
C`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00@0oooo00<0o`000?ooo`3oool0
<`3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`0d0?ooo`030?l0003o
ool0oooo00@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`1?0?ooo`030000003o
ool0oooo04l0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030?l0003o
ool0oooo03<0oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0=03oool0
0`3o0000oooo0?ooo`040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0D03oool0
0`000000oooo0?ooo`1@0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01@3oool0
0`3o0000oooo0?ooo`0c0?ooo`030000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo
03@0oooo00<0o`000?ooo`3oool01@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
0500oooo00<000000?ooo`3oool0D03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
00D0oooo00<0o`000?ooo`3oool0<`3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000
oooo0?ooo`0d0?ooo`030?l0003oool0oooo00H0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`1@0?ooo`030000003oool0oooo0540oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`060?ooo`030?l0003oool0oooo03<0oooo00<000000?ooo`3oool07@3oool00600oooo
0`00000d0?ooo`030?l0003oool0oooo00H0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`1@0?ooo`030000003oool0oooo0540oooo00@000000?ooo`3oool0oooo0P0000080?ooo`03
0?l0003oool0oooo0380oooo0P00000O0?ooo`00H03oool00`000000oooo0?ooo`0d0?ooo`030?l0
003oool0oooo00L0oooo00@000000?ooo`0000000000D`3oool00`000000oooo0?ooo`1A0?ooo`80
000000<0oooo0000003oool02@3oool00`3o0000oooo0?ooo`0c0?ooo`030000003oool0oooo01d0
oooo000j0?ooo`030;md00000000000000@0000000<0B9cO0?ooo`2oM0001P0000002018W=l0oooo
0?ooo`3o_g@0000007Boo`3oool0_g@01P0000000`18W=l0oooo0?ooo`050?ooo`030000003oool0
oooo03@0oooo00<0o`000?ooo`3oool0203oool2000005@0oooo00<000000?ooo`3oool0D`3oool2
000000X0oooo00<0o`000?ooo`3oool0<`3oool00`000000oooo0?ooo`0M0?ooo`00>`3oool00`3O
W4P0000000000003000000040018W03Oool0oooo0=nLB0D0000000H004RL0=ooo`3oool0okmd0000
001d_ol20?ooo`030=nLB0000000000000<0000000<004RL0=ooo`3oool01@3oool00`000000oooo
0?ooo`0d0?ooo`030?l0003oool0oooo05h0oooo00<000000?ooo`3oool0G`3oool00`3o0000oooo
0?ooo`0c0?ooo`030000003oool0oooo01d0oooo000l0?ooo`030=nLB018W=l0oooo0080oooo00H0
giad09cOo`3oool0oooo0=nLB018W=l30?ooo`030=nLM02Lgol0oooo00H0oooo00<0gia804RLg`3o
ool00P3oool00`3OW7@0W=oo0?ooo`050?ooo`030000003oool0oooo03@0oooo00<0o`000?ooo`3o
ool0GP3oool00`000000oooo0?ooo`1O0?ooo`030?l0003oool0oooo03<0oooo00<000000?ooo`3o
ool07@3oool003d0oooo00<0gia807Boo`3oool01P3oool00`3OW4P0M;oo0?ooo`0;0?ooo`030=nL
B01d_ol0oooo00T0oooo00<000000?ooo`3oool0=03oool00`3o0000oooo0?ooo`1N0?ooo`030000
003oool0oooo05l0oooo00<0o`000?ooo`3oool0<`3oool00`000000oooo0?ooo`0M0?ooo`00?P3o
ool00`2oM4P0W=oo0?ooo`060?ooo`030;mdB02Lgol0oooo00/0oooo00<0_gA809cOo`3oool0203o
ool00`000000oooo0?ooo`0d0?ooo`030?l0003oool0oooo05h0oooo00<000000?ooo`3oool0G`3o
ool00`3o0000oooo0?ooo`0c0?ooo`030000003oool0oooo01d0oooo000n0?ooo`030?oog`2LB7@0
_ooo00H0oooo00<0oooO09a8M02oool02`3oool00`3ooml0W4Qd0;ooo`080?ooo`<00000=03oool0
0`3o0000oooo0?ooo`1N0?ooo`030000003oool0oooo05l0oooo00<0o`000?ooo`3oool0<@3oool3
000001l0oooo000o0?ooo`030?noM000M;l0oooo00H0oooo00<0okmd001d_`3oool02`3oool00`3o
_g@007Bo0?ooo`070?ooo`030000003oool0oooo03@0oooo00<0o`000?ooo`3oool0GP3oool00`00
0000oooo0?ooo`1O0?ooo`030?l0003oool0oooo03<0oooo00<000000?ooo`3oool07@3oool003l0
oooo00<0oono07@0B02Lgol01P3oool00`3ookl0M01809cOo`0;0?ooo`030?oo_`1d04P0W=oo00L0
oooo00<000000?ooo`3oool0=03oool00`3o0000oooo0?ooo`1N0?ooo`030000003oool0oooo05l0
oooo00<0o`000?ooo`3oool0<`3oool00`000000oooo0?ooo`0M0?ooo`00?`3oool00`3ookl0M000
04RLg`060?ooo`030?oo_`1d0000B9cO00/0oooo00<0oono07@00018W=l01`3oool00`000000oooo
0?ooo`0d0?ooo`030?l0003oool0oooo05h0oooo00<000000?ooo`3oool0G`3oool00`3o0000oooo
0?ooo`0c0?ooo`030000003oool0oooo01d0oooo000j0?ooo`030?noW02Lgol0oooo0080oooo00H0
omnL04P00018W=l0oooo0?noW02Lgol30?ooo`030?oOW0180000B9cO00H0oooo00<0oknL09cOo`3o
ool00P3oool00`3ogi`0B00004RLg`070?ooo`030000003oool0oooo03@0oooo00<0o`000?ooo`3o
ool0GP3oool00`000000oooo0?ooo`1O0?ooo`030?l0003oool0oooo03<0oooo00<000000?ooo`3o
ool07@3oool003X0oooo0140oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`3oool0oooO09a8
B02Lgol0oooo0?oo_`1d0000001d0;ooo`060?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000
M02oool70?ooo`030000003oool0oooo03@0oooo00<0o`000?ooo`3oool0GP3oool00`000000oooo
0?ooo`1O0?ooo`030?l0003oool0oooo03<0oooo00<000000?ooo`3oool07@3oool003/0oooo00H0
oooO09a80000000000000000M02oool30?ooo`060?oog`2LB00000000000000007@0_ooo203oool0
1P3ooml0W4P0000000000000001d0;ooo`P0oooo00<000000?ooo`3oool0=03oool00`3o0000oooo
0?ooo`1N0?ooo`030000003oool0oooo05l0oooo00<0o`000?ooo`3oool0<`3oool00`000000oooo
0?ooo`0M0?ooo`00H03oool3000003@0oooo00<0o`000?ooo`3oool0GP3oool00`000000oooo0?oo
o`1O0?ooo`030?l0003oool0oooo0380oooo0P00000O0?ooo`00H03oool00`000000oooo0?ooo`0d
0?ooo`030?l0003oool0oooo05h0oooo00<000000?ooo`3oool0G`3oool00`3o0000oooo0?ooo`0c
0?ooo`030000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo03@0oooo00<0o`000?oo
o`3oool0GP3oool00`000000oooo0?ooo`1O0?ooo`030?l0003oool0oooo03<0oooo00<000000?oo
o`3oool07@3oool00600oooo00<000000?ooo`3oool0=03oool00`3o0000oooo0?ooo`1N0?ooo`03
0000003oool0oooo05l0oooo00<0o`000?ooo`3oool0<`3oool00`000000oooo0?ooo`0M0?ooo`00
H03oool00`000000oooo0?ooo`0d0?ooo`030?l0003oool0oooo05h0oooo00<000000?ooo`3oool0
G`3oool00`3o0000oooo0?ooo`0c0?ooo`030000003oool0oooo01d0oooo001P0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool0603oool00`3o0000oooo0?ooo`0<0?ooo`030000003o
ool0oooo02H0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0W0?ooo`030000003o
ool0oooo02H0oooo00<000000?ooo`3oool0303oool00`3o0000oooo0?ooo`0G0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0203oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo00<0oooo00D0o`000?ooo`3oool0oooo0000
000:0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`020?ooo`030?l0003oool0oooo00<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo01d0oooo001P0?ooocL0000000<0o`00000000000000`0000000
0`3o000000000000000d000001l0oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?ooobH0oooo00<0oooO09a80000
00000`0000000`18W=l0oooo0=nLB0020000000307Boo`3oool0oooo07d0oooo003o0?ooobH0oooo
00<0_g@00018W03Oool00`3oool00`3OW9`0_ooo0?ooo`020?ooo`030;mdM02oool0oooo07d0oooo
003o0?ooobH0oooo00<0oooO09adW03Oool00`3oool00`3ookl0W9cO0?ooo`020?ooo`030?oOW01d
W=l0oooo07d0oooo003o0?ooobL0oooo00<0_g@00000000000000`0000001`18W=l0oooo0?ooo`3o
ool0oooO09a8M02oool0O@3oool00740oooo00<0_gA809cOo`3oool00P3oool00`3ooml0W7BL0=oo
o`040?ooo`080?oog`2LB000000000000000B9`0W7@000000000M;l30?ooo`030?oog`2LB7@0_ooo
00D0oooo00D0_g@0000000000000001d0;ooo`040?ooo`030?oog`2LB00000000080000000<00018
09cOo`3oool0103oool02@3ookl0M000000000000000B9bo09a80000B9`0gooo0;md000200000003
0000M02ooml0W4P00080000000@004RL0=ooo`3oool0_g@00P0000002@0004P0W=oo0?ooo`3oool0
omnL04P000000000B9cO0=nLB002000000040000M02oool0oooo0=nLB0<0000000@0M;oo0?ooo`3o
ool0_g@00P0000000`0007@0_ooO09a80002000000030018W03Oool0oooo00H0oooo00<0omnL07BL
g`3oool00P3oool00`3OW7@0W=oo0?ooo`020?ooo`0:0=nLB000000000000000M02oool0omnL04P0
00000000B9cO0=nLB080000000<0001809cOg`2LB0000P0000000`00M;l0oooo0?ooo`030?ooo`03
0?oog`2LB000000000L0000000<007Bo0?ooo`2oM0000P0000000`0007@0_ooO09a8000200000004
0018W03Oool0oooo0;md0080000000@0001d0;ooo`3oool0_g@00P0000001@0004P0W=oo0?ooo`3o
okl0M00000@0000000<0001d0;ooo`3oool00`3oool00`2oM000M;oo0?ooo`0F0?ooo`030;md0000
0000000000@0000000D0B9cO0?ooo`3oool0oooo0;md0003000000070018W03Oool0oooo0?ooo`3o
_g@0000007Boo`030?ooo`050?oOW018000000000000001d_ol01P3oool0103ooml0W4P00018W03O
ool30?ooo`060;md00000000000000000000B9`0gooo103oool00`3OW4P0B9cO0?ooo`0^0?ooo`00
L03oool00`3ookl0M00004RLg`030?ooo`030?noM00004P0W=oo00@0oooo00T0okmd0000001d_ol0
oooo0?oOW018000007Bo0?no_`2oool00P3oool00`3o_g@0000004RLg`040?ooo`070;md00000000
07Bo0?ooo`3ogi`0B01d0;ooo`050?ooo`030?oOW0180000B9cO00L0oooo00D0gia80000M02oool0
oooo0=nLB002000000090000M02oool0gia80018W03Oool0oooo0?oog`2LB00007Bo0080oooo01P0
_g@000000000B9`0gooo0?oog`2LB7@0_ooo0?ooo`3OW4P0001d0;ooo`3O_kl0gooo0=nLB00007@0
_ooo0?ooo`3OW4P0000004RLg`3oool0oooO09a80018W=l20?ooo`070=nLB000B9`0gooo0?ooo`3o
oml0W4P0001d_`090?ooo`070;md000007@0_ooo0?ooo`3ookl0M000001d_`040?ooo`0@0=nLB000
07@0_ooo0?ooo`3OW4P0001d0;ooo`3O_kl0gooo0=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`P0
oooo00<0omnL04P00018W=l0103oool02`3ogi`0B01d0;ooo`3oool0gia80018W03Oool0oooo0?oo
g`2LB00007Bo0080oooo00/0_g@000000000M;l0oooo0?oOW01807@0_ooo0?ooo`3OW4P0001d0;oo
o`030?ooo`030;mdB02Lgol0oooo00L0oooo00@0omnL04P00000B9`0gooo5`3oool00`3OW4P00000
00000003000000030018W03Oool0oooo0080oooo00<0omnL04P0001d_ol0103oool00`3o_g@00000
07Boo`020?ooo`070?oOW0180000B9cO0?ooo`3ookl0M00007Boo`050?ooo`040?oog`2LB00004RL
0=ooo`@0oooo00<0omnL04P0001d_ol01P3oool00`3ogi`0B4RL0=ooo`0^0?ooo`00L03oool0103o
_g@000000018W03Oool20?ooo`030;md0000000007Bo00@0oooo00L0okmd0000B02Lgol0oooo0?oo
g`2LB00007Bo00<0oooo00D0oooO09a80000B9`0W7Ad0;ooo`020?ooo`030?oOW0180000B9cO00<0
oooo00<0omnL07BLg`3oool0103oool00`3ookl0M00004RLg`070?ooo`070;md0000B9`0gooo0?oo
o`3ooml0W4P0001d_`020?ooo`0;0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0omnL04P0
0000M;l00`3oool01P3ogi`0W;oo0?ooo`3OW4P004RL0=ooo`80oooo00H0gia80000M02oool0omnL
04P00000M;l30?ooo`0;0?noM00007@0_ooo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0
203oool02@3ogi`0B00004QdM01dW=l0oooo0?noM00004P0B4Q809cOo`030?ooo`070=nLB00007@0
_ooo0?ooo`3OW4P004RL0=ooo`020?ooo`070=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`080?oo
o`030?oo_`1d0000B9cO00D0oooo00h0okmd07Boo`3oool0gia80018W03Oool0oooo0?oog`2LB000
07Bo0?ooo`3ogi`0B00004RLg`<0oooo00H0omnL07BLg`3oool0gia80000M02oool30?ooo`050?oo
_`1d0000000000000000M;l01@3oool0102oM00004RL09ad001d_ol60?ooo`030?oog`2LB0000000
00L0000000<0B9cO0?ooo`3oool01@3oool00`3OW4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo
0080oooo00<0omnL04P0001d_ol0203oool00`3ooml0W4P0001d_`030?ooo`030?noM000B9`0gooo
00@0oooo00@0oooO09a80000B9`0gooo103oool00`3ogi`0B00007Boo`060?ooo`030?oog`2LB4P0
W=oo02h0oooo001_0?ooo`0;0?oog`2LB000001d09bLM02Lgol0oooo0?oo_`1d000007BL09a8M02o
ool0103oool01P3o_g@00000001d_`3ooml0W4P0001d_`<0oooo00D0omnL04P0001d_ol0okmd04RL
g`020?ooo`030?noM00004P0W=oo00X0oooo00<0oono07@00018W=l01`3oool01`2oM00004RL0=oo
o`3oool0oooO09a80000M;l00P3oool02`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?no
M00004P0W=oo00H0oooo00<0gia80018W03Oool00P3oool01P3OW4P0001d0;ooo`3o_g@0000007Bo
o`<0oooo00/0okmd0000001d_ol0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`080?ooo`09
0=nLB00007@0_ooo0;mdM02ooml0W4P0001d_`3ogi`0M9cO00<0oooo00L0gia80000M02oool0oooo
0=nLB000B9`0gooo0080oooo00L0gia80000M02oool0oooo0?oog`2LB00007Bo00P0oooo00<0oono
07@00018W=l0203oool02`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?noM00004P0W=oo
00H0oooo00<0gia80000M02oool00`3oool01P3OW4P0001d0;ooo`3ookl0M00004RLg`<0oooo00D0
okmd0000001d_ol0omnL04Qd_`0I0?ooo`030=nLB01d_ol0oooo00H0oooo00<0omnL04P0001d_ol0
203oool00`3ogi`0B00007Boo`030?ooo`040?noM00004P0W=oo0;md00L0000000L0M;oo0?ooo`3o
ool0oooo0?oOW0180000M;oo00L0oooo00<0gia804RLg`3oool0;@3oool006l0oooo00/0omnL04P0
0018W=l0omnL04Qd_`3oool0gia80000001d_ol0okmd07Boo`060?ooo`040?noM00000000000001d
_`<0oooo00X0_g@00018W03Oool0oooO09a8M02oool0oooo0?noM00007@0_ooo2P3oool00`3ookl0
M00004RLg`020?ooo`030?oOW01dW=l0oooo0080oooo00L0_g@00018W03Oool0oooo0?oog`2LB000
07Bo0080oooo00/0gia80018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3o_g@0001d0;ooo`060?oo
o`030=nLB000B9`0gooo0080oooo00H0gia80000M02oool0okmd0000001d_ol30?ooo`0;0?noM000
0000M;oo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l01`3oool02`3ookl0M00004RLg`3o
ool0omnL07AdM01804P0W=oo0?ooo`2oM7@0_ooo0080oooo00L0gia80000M02oool0oooo0=nLB000
B9`0gooo0080oooo00L0gia80000M02oool0oooo0?oog`2LB00007Bo00P0oooo00<0oono07@00018
W=l00`3oool00`3o_g@0M;oo0?ooo`020?ooo`0;0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3o
ool0okmd0000M02oool01P3oool00`3OW4P0001d0;ooo`020?ooo`070?oo_`1d0000B9cO0?ooo`3o
oml0W4P0001d_`020?ooo`070?oog`2LB00004RL0=ooo`3oool0_gA809cOo`0I0?ooo`030;mdB02L
gol0oooo00D0oooo00<0omnL04P0001d_ol0203oool00`3ogi`0B00007Boo`030?ooo`050?noM000
04P0W=oo0?oOW01dW=l00P3oool0103ooml0W4P00018W03Oool40?ooo`030?oOW0180000M;oo00L0
oooo00<0omnL04Q8W03Oool0;@3oool006l0oooo00/0gia80000B02Lgol0oooO09a8M02ooml0W4P0
0000M02oool0oono07Ad_`030?ooo`070?oo_`1d0000M;oo0?ooo`3ooml0W4P0001d_`020?ooo`03
0?oOW0180000M;oo0080oooo00D0okmd04RLg`3oool0omnL04P000050000000304RLg`3oool0oooo
00<0oooo00<0oono07@00018W=l00P3oool00`3o_g@0B9cO0?ooo`020?ooo`070;md0000B9`0gooo
0?ooo`3ooml0W4P0001d_`020?ooo`0;0;md000007@0_ooo0?ooo`3ooml0W4P0001d_`3oool0omnL
04P0B02Lgol00P3oool01`3o_g@0001809cOo`3oool0gia80018W03Oool00P3oool01P3OW4P0001d
0;ooo`3ookl0M00007Boo`<0oooo00/0_g@00000B02Lgol0oooo0;md000007@0_ooo0?ooo`3ooml0
W4P0001d_`070?ooo`070=nLB00004P0W=oo0?ooo`3ooml0W4P0001d_`020?ooo`0;0?noM018W=l0
oooo0?ooo`3OW4P0001d0;ooo`3oool0gia80018W03Oool00P3oool01`3OW4P0001d0;ooo`3oool0
oooO09a80000M;l0203oool00`3ookl0M00004RLg`030?ooo`030;md001d_ol0oooo0080oooo00X0
_g@00000M02oool0oooo0?oog`2LB00007Bo0?ooo`3ogi`0B0001@0000001@18W=l0oooo0=nLB000
04P0W=oo0080oooo00L0oono07@00018W=l0oooo0?oog`2LB000B9cO0080oooo00<0okmd0000B02L
gol00P3oool00`3o_g@0B9cO0?ooo`0H0?ooo`030?oog`2LB7@0_ooo00D0oooo00<0omnL04P0001d
_ol0203oool00`3ogi`0B00007Boo`030?ooo`0;0;md000004P0W=oo0?ooo`3OW4P0M;oo0?ooo`3o
oml0W4P00018W03Oool0103oool00`3ogi`0B00007Boo`070?ooo`030?oog`2LB4P0W=oo02d0oooo
001^0?ooo`090?oo_`1d000004RL0=ooo`3oool0okmd07BLW0180000B9cO0080oooo0100_gA809cO
o`3oool0oooo0?oo_`1d0000B9cO0?ooo`3ookl0M000001d_`3oool0oooO09a8000007@0_ooo0P3o
ool01P3ooml0W4Qd0;ooo`3oool0_g@004RLg`80oooo00<0gia80000B02Lgol01@3oool00`3ookl0
M000000000030000000D04RLg`3oool0oooo0?ooo`2oM00004RL0=ooo`3oool0oooO09a80000M;l0
oooO09a80000000004Q804P0B02Lgol0oono07@00018W=l20?ooo`0Q0;md0000B9`0gooo0?oog`2L
B000001d0;ooo`3oool0gia80018W03Oool0oooO09a800000000001d0;ooo`3oool0gia80018W03O
ool0oooo0?noM000000007Bo0?oog`2LB00000000018B01804P0W=oo0?oo_`1d0000B9cO00H0oooo
00<0oono07@00000M;l00P3oool00`3ogi`0B00007Boo`020?ooo`0;0?oog`2LB7@0_ooO09a80000
0000001d0;ooo`3oool0gia80018W03Oool00P3oool01`3OW4P000000000B02Lgol0oono07@00018
W=l0203oool00`3ookl0M000000000040000000;07Boo`3oool0oooO09a80000000004Q804P0B02L
gol0oono07@00018W=l00P3oool04P2oM000B9cO0?ooo`3oool0gia80000B02Lgml0W4P000000000
04P0B7BL0=noM00007@0_ooo0;md0000M;l0oooo0?noM080000000D004RL0=oo_`1d000004RL0=oo
o`020?ooo`030?oo_`1d07@0_ooo00@0oooo00<0oooO09a8000000001`0000000`18W=l0oooo0?oo
o`080?ooo`030?noM000M;l0oooo00@0oooo00<0omnL04P0001d_ol0203oool0203ookl0M0000000
0018W=l0oooo0?noM000000007Bo0P3oool01`3ooml0W4Qd0;ooo`3ooml0W4P00018W03Oool0103o
ool00`3ogi`0B00007Boo`080?ooo`030=nLB000M;l0oooo02`0oooo001^0?ooo`030?noM0000000
B9cO0080oooo00@0oono07@0000004P0W=oo0P3oool00`3o_g@0B9cO0?ooo`020?ooo`080?oo_`1d
000000000000000007@0_ooo0?oog`2LB002000000090000M02oool0oooO09a80000000004RL0=oo
o`3oool0gia80080000000<0001809cOo`3oool01@3oool00`3ookl0M00004RLg`020?ooo`0<0?no
M018W=l0oooo0?oo_`1d000000000018W03Oool0omnL04P00000000007Bo0P3oool01`2oM00004RL
0=oog`2LB00000000000M02oool00`3oool01`3OW4P000000000000004P0W=oo0?ooo`2oM0000`00
00001@00B9`0gooo0;md000007@0_ooo0080oooo00D0okmd000000000000000004RLg`030?ooo`07
0;md0000B9`0gooO09a800000000001d0;ooo`050?ooo`060?oog`2LB000000000000018W=l0okmd
0`0000002P18W=l0oooo0;md0000000004RL0=ooo`2oM000001d0;ooo`2oM003000000090018W03O
ool0gia80000M02ooml0W4P00000000007@0_ooo00P0oooo00<0oono07@00018W=l00`3oool00`2o
M000M;oo0?ooo`020?ooo`070;md0000B9`0gooO09a800000000001d0;ooo`030?ooo`050=nLB000
000000000000B02Lgol00P3oool0202oM000001d0;oOW0180000001d0;ooo`3ooml0W4P00P000000
101d_ol0oooo0?oog`2LB002000000070018W03Oool0oooo0;md0000000004RL0=ooo`0H0?ooo`03
0?oo_`1d04P0W=oo00@0oooo00<0omnL04P0001d_ol02@3oool00`2oM00004RL0;nLB00200000003
0018W03Oool0oooo0080oooo00H0omnL04Qd_`3ooml0W4P00018W03Oool40?ooo`030?oOW0180000
M;oo00P0oooo00<0oono07@0M02oool0;03oool006h0oooo00<0_g@00000M02oool00P3oool0103o
okl0M0000018W03Oool20?ooo`030?oo_`1dB9`0gooo01l0oooo00<0oono07@00018W=l00P3oool0
0`3ogi`0M9cO0?ooo`0L0?ooo`030;md0000B9`0gooo02h0oooo00<0_g@00018W03Oool00P3oool0
0`3OW4P0001d0;ooo`0<0?ooo`030?oo_`1d0000B9cO00<0oooo00<0okmd07Boo`3oool0@P3oool0
0`3ookl0M00004RLg`040?ooo`030?oOW0180000M;oo00T0oooo00<0omnL04P0001d_ol0203oool0
1@3OW4P0M;oO09a80000B9`0gooo00@0oooo00<0omnL04P0001d_ol00`3oool00`3ogml0gooo0?oo
o`030?ooo`030;md001d_ol0oooo02/0oooo001]0?ooo`030?oOW018000007Bo00<0oooo00<0okmd
00000018W=l0103oool00`3OW7@0W=oo0?ooo`0N0?ooo`030?oo_`1d0000B9cO00@0oooo00<0giad
09cOo`3oool06P3oool00`3ookl0M4RL0=ooo`0^0?ooo`030?oo_`1dB9`0gooo0080oooo00<0gia8
0000M02oool0303oool00`3ookl0M00004RLg`040?ooo`030?oo_`1dM;l0oooo03`0oooo00<0oknL
09cOo`3oool00P3oool00`3ogi`0B00004RLg`040?ooo`030?oOW0180000M;oo00X0oooo00<0gia8
0000B02Lgol01`3oool01@3ooml0W4Q804Q80000B9`0gooo00@0oooo00<0omnL04P0001d_ol00`3o
ool00`3ookl0M4RL0=ooo`030?ooo`030?noM000M;l0oooo02/0oooo001]0?ooo`030=nLB0000000
B9cO0080oooo00@0oooO09a800000000M;oo103oool00`3OW4P007Bo0?ooo`0N0?ooo`030?oOW018
0000B9cO00<0oooo00<0oono07@0M02oool06`3oool00`3ooml0W7BL0=ooo`0^0?ooo`030?oog`2L
M9`0gooo0080oooo00<0gia80000M02oool0303oool00`3ogi`0B00004RLg`040?ooo`030=nLB018
W=l0oooo03`0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`80oooo00D0oooO09a8
00000000000007Boo`0;0?ooo`030?noM000000007Bo00L0oooo00@0omnL04P00000B9`0gooo0P3o
ool01@3ooml0W4P0000000000000M;oo00@0oooo00<0_g@00000000000000`0000000`0007@0_ooo
0?ooo`0Z0?ooo`00J`3oool00`3ooml0W4P00000000200000003001d_`3ogi`0B00000<0000000@0
B9cO0?ooo`3oool0_g@00P0000000`00B9`0gooo0?ooo`0J0?ooo`030?oog`2LB000000000H00000
00<0001d0;ooo`3oool07`3oool00`3OW4P0001809cOo`0U0?ooo`030=nLB00004P0W=oo00@0oooo
00D0oooO09a800000000001d0;ooo`0:0?ooo`030?oog`2LB000000000L0000000<0B9cO0?ooo`3o
ool0?03oool01P3ooml0W4P0000000000000001d0;ooo`D0oooo00<0okmd0000001d_ol0303oool0
1@3ooml0W4P00000000007@0_ooo00D0oooo00<0gia80018W03Oool0103oool00`3o_g@0000007Bo
o`040?ooo`030?noM0000000000000@0000000<0M;oo0?ooo`3oool0:@3oool00<@0oooo00<0gia8
0000B02Lgol09@3oool00`3OW4P0001809cOo`060?ooo`030;md000007@0_ooo0;P0oooo003o0?oo
ok40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo0000\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {-12.0417, 20.6477, \
0.0726079, 0.0103896}}]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Thick barrier", "Section"],

Cell[TextData[{
  "Next, consider a thick-barrier case with ",
  Cell[BoxData[
      \(TraditionalForm\`m = 20\)]],
  ".  It is useful to estimate the ground-state energy using the harmonic \
oscillator approximation for an isolated well.  This energy is quite close to \
the bottom of well, so that the barrier appears to be rather thick.  \
Nevertheless, we might expect the true energy to be slightly smaller than \
this estimate because our model potential rises higher than the true \
potential on one side."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[CurlyEpsilon]\_0 /. m \[Rule] 20\)], "Input"],

Cell[BoxData[
    \(2.893113599833817`\)], "Output"]
}, Closed]],

Cell["\<\
Although there is a hint of glitch near 2.88, the zero/pole structure is so \
sharp that it is easily missed unless a very large number of points is used \
by the plotting routine.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
        mismatch[e, 20, pot, \(-3\), 3, 0, MaxSteps \[Rule] 5000], {e, emin, 
          3}, PlotRange \[Rule] {All, {\(-50\), 50}}, PlotPoints \[Rule] 500, 
        Frame \[Rule] True, 
        FrameLabel \[Rule] {"\<energy\>", "\<mismatch\>"}, 
        PlotLabel \[Rule] "\<matching condition for m=20\>"];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-6.33927 2.43849 0.309017 0.00618034 [
[.0008 -0.0125 -9.5 -15.6875 ]
[.0008 -0.0125 9.5 0 ]
[.24464 -0.0125 -9.5 -15.6875 ]
[.24464 -0.0125 9.5 0 ]
[.48849 -0.0125 -9.5 -15.6875 ]
[.48849 -0.0125 9.5 0 ]
[.73234 -0.0125 -9.5 -15.6875 ]
[.73234 -0.0125 9.5 0 ]
[.97619 -0.0125 -5 -15.6875 ]
[.97619 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -17.9688 -31.375 ]
[.5 -0.0125 17.9688 -15.6875 ]
[-0.0125 .0618 -24.3125 -7.84375 ]
[-0.0125 .0618 0 7.84375 ]
[-0.0125 .18541 -24.3125 -7.84375 ]
[-0.0125 .18541 0 7.84375 ]
[-0.0125 .30902 -10 -7.84375 ]
[-0.0125 .30902 0 7.84375 ]
[-0.0125 .43262 -16 -7.84375 ]
[-0.0125 .43262 0 7.84375 ]
[-0.0125 .55623 -16 -7.84375 ]
[-0.0125 .55623 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -40 -25.2813 ]
[-0.01875 .30902 -24.3125 25.2813 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -72.8125 0 ]
[.5 .63053 72.8125 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.0008 0 m
.0008 .00625 L
s
gsave
.0008 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.6) show
1.000 setlinewidth
grestore
.24464 0 m
.24464 .00625 L
s
gsave
.24464 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.7) show
1.000 setlinewidth
grestore
.48849 0 m
.48849 .00625 L
s
gsave
.48849 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.8) show
1.000 setlinewidth
grestore
.73234 0 m
.73234 .00625 L
s
gsave
.73234 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.9) show
1.000 setlinewidth
grestore
.97619 0 m
.97619 .00625 L
s
gsave
.97619 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
.125 Mabswid
.04957 0 m
.04957 .00375 L
s
.09833 0 m
.09833 .00375 L
s
.1471 0 m
.1471 .00375 L
s
.19587 0 m
.19587 .00375 L
s
.29341 0 m
.29341 .00375 L
s
.34218 0 m
.34218 .00375 L
s
.39095 0 m
.39095 .00375 L
s
.43972 0 m
.43972 .00375 L
s
.53726 0 m
.53726 .00375 L
s
.58603 0 m
.58603 .00375 L
s
.6348 0 m
.6348 .00375 L
s
.68357 0 m
.68357 .00375 L
s
.78111 0 m
.78111 .00375 L
s
.82988 0 m
.82988 .00375 L
s
.87865 0 m
.87865 .00375 L
s
.92742 0 m
.92742 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -78.9688 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(energy) show
94.938 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .0618 m
.00625 .0618 L
s
gsave
-0.0125 .0618 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
0 .18541 m
.00625 .18541 L
s
gsave
-0.0125 .18541 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
0 .30902 m
.00625 .30902 L
s
gsave
-0.0125 .30902 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
0 .43262 m
.00625 .43262 L
s
gsave
-0.0125 .43262 -77 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
0 .55623 m
.00625 .55623 L
s
gsave
-0.0125 .55623 -77 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .09271 m
.00375 .09271 L
s
0 .12361 m
.00375 .12361 L
s
0 .15451 m
.00375 .15451 L
s
0 .21631 m
.00375 .21631 L
s
0 .24721 m
.00375 .24721 L
s
0 .27812 m
.00375 .27812 L
s
0 .33992 m
.00375 .33992 L
s
0 .37082 m
.00375 .37082 L
s
0 .40172 m
.00375 .40172 L
s
0 .46353 m
.00375 .46353 L
s
0 .49443 m
.00375 .49443 L
s
0 .52533 m
.00375 .52533 L
s
0 .0309 m
.00375 .0309 L
s
0 .58713 m
.00375 .58713 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -118.438 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
86.2813 11.8438 translate
90 rotate
-86.2813 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(mismatch) show
109.563 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.0008 .61178 m
.0008 .61803 L
s
.24464 .61178 m
.24464 .61803 L
s
.48849 .61178 m
.48849 .61803 L
s
.73234 .61178 m
.73234 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.04957 .61428 m
.04957 .61803 L
s
.09833 .61428 m
.09833 .61803 L
s
.1471 .61428 m
.1471 .61803 L
s
.19587 .61428 m
.19587 .61803 L
s
.29341 .61428 m
.29341 .61803 L
s
.34218 .61428 m
.34218 .61803 L
s
.39095 .61428 m
.39095 .61803 L
s
.43972 .61428 m
.43972 .61803 L
s
.53726 .61428 m
.53726 .61803 L
s
.58603 .61428 m
.58603 .61803 L
s
.6348 .61428 m
.6348 .61803 L
s
.68357 .61428 m
.68357 .61803 L
s
.78111 .61428 m
.78111 .61803 L
s
.82988 .61428 m
.82988 .61803 L
s
.87865 .61428 m
.87865 .61803 L
s
.92742 .61428 m
.92742 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .0618 m
1 .0618 L
s
.99375 .18541 m
1 .18541 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .43262 m
1 .43262 L
s
.99375 .55623 m
1 .55623 L
s
.125 Mabswid
.99625 .09271 m
1 .09271 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .15451 m
1 .15451 L
s
.99625 .21631 m
1 .21631 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .27812 m
1 .27812 L
s
.99625 .33992 m
1 .33992 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .40172 m
1 .40172 L
s
.99625 .46353 m
1 .46353 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .52533 m
1 .52533 L
s
.99625 .0309 m
1 .0309 L
s
.99625 .58713 m
1 .58713 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .30902 m
1 .30902 L
s
gsave
.5 .63053 -133.813 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(matching) show
110.375 15.813 moveto
(condition) show
158.563 15.813 moveto
(for) show
175.438 15.813 moveto
(m) show
184.813 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
192.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
204.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .42935 m
.02567 .42933 L
.02769 .42931 L
.0296 .42929 L
.03143 .42927 L
.03338 .42925 L
.03525 .42923 L
.03724 .42921 L
.03917 .42918 L
.04101 .42917 L
.04298 .42914 L
.04487 .42912 L
.04669 .42911 L
.04863 .42909 L
.05049 .42907 L
.05248 .42904 L
.05439 .42902 L
.05622 .42901 L
.05818 .42898 L
.06006 .42896 L
.06206 .42894 L
.06399 .42892 L
.06584 .4289 L
.06781 .42888 L
.06971 .42886 L
.07154 .42884 L
.07348 .42882 L
.07535 .4288 L
.07734 .42878 L
.07926 .42876 L
.0811 .42874 L
.08306 .42872 L
.08495 .4287 L
.08677 .42868 L
.0887 .42866 L
.09056 .42864 L
.09254 .42862 L
.09445 .4286 L
.09628 .42858 L
.09823 .42856 L
.10011 .42854 L
.10211 .42852 L
.10403 .4285 L
.10588 .42848 L
.10785 .42846 L
.10974 .42844 L
.11156 .42842 L
.1135 .4284 L
.11537 .42838 L
.11736 .42836 L
Mistroke
.11927 .42834 L
.12111 .42832 L
.12307 .4283 L
.12495 .42828 L
.12695 .42826 L
.12888 .42824 L
.13074 .42822 L
.13271 .4282 L
.13462 .42818 L
.13644 .42816 L
.13839 .42814 L
.14026 .42812 L
.14226 .42809 L
.14418 .42807 L
.14602 .42805 L
.14799 .42803 L
.14988 .42801 L
.15169 .42799 L
.15363 .42797 L
.15549 .42795 L
.15748 .42793 L
.15938 .42791 L
.16122 .42789 L
.16317 .42787 L
.16505 .42785 L
.16705 .42783 L
.16898 .42781 L
.17083 .42779 L
.1728 .42777 L
.1747 .42775 L
.17652 .42773 L
.17846 .42771 L
.18033 .42769 L
.18232 .42767 L
.18424 .42765 L
.18608 .42763 L
.18804 .42761 L
.18993 .42759 L
.19174 .42757 L
.19367 .42755 L
.19553 .42753 L
.19751 .42751 L
.19941 .42749 L
.20124 .42747 L
.20319 .42744 L
.20507 .42742 L
.20706 .4274 L
.20899 .42738 L
.21083 .42736 L
.2128 .42734 L
Mistroke
.21469 .42732 L
.21651 .4273 L
.21845 .42728 L
.22032 .42726 L
.2223 .42724 L
.22421 .42722 L
.22605 .4272 L
.22801 .42718 L
.22989 .42716 L
.23189 .42714 L
.23382 .42712 L
.23567 .4271 L
.23765 .42708 L
.23955 .42706 L
.24137 .42704 L
.24332 .42701 L
.24519 .42699 L
.24718 .42697 L
.2491 .42695 L
.25094 .42693 L
.25291 .42691 L
.2548 .42689 L
.25661 .42687 L
.25855 .42685 L
.26041 .42683 L
.26239 .42681 L
.2643 .42679 L
.26613 .42677 L
.26808 .42675 L
.26996 .42673 L
.27196 .42671 L
.27388 .42669 L
.27573 .42667 L
.2777 .42665 L
.2796 .42662 L
.28142 .42661 L
.28336 .42658 L
.28523 .42656 L
.28722 .42654 L
.28913 .42652 L
.29097 .4265 L
.29293 .42648 L
.29481 .42646 L
.29682 .42644 L
.29875 .42642 L
.3006 .4264 L
.30258 .42638 L
.30448 .42636 L
.30631 .42634 L
.30826 .42632 L
Mistroke
.31014 .4263 L
.31213 .42627 L
.31405 .42625 L
.31589 .42623 L
.31786 .42621 L
.31975 .42619 L
.32157 .42617 L
.32351 .42615 L
.32537 .42613 L
.32735 .42611 L
.32926 .42609 L
.3311 .42607 L
.33305 .42605 L
.33494 .42603 L
.33694 .42601 L
.33886 .42598 L
.34072 .42596 L
.34269 .42594 L
.34459 .42592 L
.34641 .4259 L
.34835 .42588 L
.35023 .42586 L
.35221 .42584 L
.35413 .42582 L
.35597 .4258 L
.35793 .42578 L
.35982 .42576 L
.36164 .42574 L
.36357 .42572 L
.36543 .4257 L
.36741 .42568 L
.36931 .42565 L
.37115 .42563 L
.3731 .42561 L
.37497 .42559 L
.37697 .42557 L
.37889 .42555 L
.38074 .42553 L
.38271 .42551 L
.3846 .42549 L
.38642 .42547 L
.38836 .42545 L
.39023 .42543 L
.39221 .42541 L
.39413 .42538 L
.39597 .42536 L
.39792 .42534 L
.39981 .42532 L
.40181 .4253 L
.40374 .42528 L
Mistroke
.40559 .42526 L
.40757 .42524 L
.40947 .42522 L
.4113 .4252 L
.41324 .42518 L
.41512 .42516 L
.41711 .42513 L
.41903 .42511 L
.42087 .42509 L
.42283 .42507 L
.42472 .42505 L
.42654 .42503 L
.42848 .42501 L
.43034 .42499 L
.43232 .42497 L
.43423 .42495 L
.43606 .42493 L
.43802 .42491 L
.4399 .42489 L
.44189 .42486 L
.44382 .42484 L
.44567 .42482 L
.44764 .4248 L
.44954 .42478 L
.45136 .42476 L
.4533 .42474 L
.45517 .42472 L
.45716 .4247 L
.45907 .42468 L
.46092 .42465 L
.46288 .42463 L
.46476 .42461 L
.46658 .42459 L
.46851 .42457 L
.47037 .42455 L
.47234 .42453 L
.47425 .42451 L
.47608 .42449 L
.47802 .42447 L
.4799 .42445 L
.48189 .42442 L
.48382 .4244 L
.48566 .42438 L
.48763 .42436 L
.48952 .42434 L
.49134 .42432 L
.49328 .4243 L
.49515 .42428 L
.49713 .42426 L
.49904 .42424 L
Mistroke
.50088 .42422 L
.50283 .42419 L
.50471 .42417 L
.50671 .42415 L
.50864 .42413 L
.5105 .42411 L
.51247 .42409 L
.51437 .42407 L
.51619 .42405 L
.51814 .42403 L
.52001 .424 L
.522 .42398 L
.52392 .42396 L
.52576 .42394 L
.52772 .42392 L
.52961 .4239 L
.53143 .42388 L
.53336 .42386 L
.53522 .42384 L
.5372 .42382 L
.53911 .42379 L
.54094 .42377 L
.54289 .42375 L
.54477 .42373 L
.54677 .42371 L
.54869 .42369 L
.55054 .42367 L
.55251 .42365 L
.5544 .42362 L
.55623 .4236 L
.55817 .42358 L
.56004 .42356 L
.56202 .42354 L
.56393 .42352 L
.56577 .4235 L
.56773 .42348 L
.56962 .42346 L
.57162 .42343 L
.57355 .42341 L
.57541 .42339 L
.57738 .42337 L
.57928 .42335 L
.58111 .42333 L
.58306 .42331 L
.58493 .42329 L
.58693 .42326 L
.58885 .42324 L
.59069 .42322 L
.59266 .4232 L
.59455 .42318 L
Mistroke
.59637 .42316 L
.5983 .42314 L
.60017 .42312 L
.60215 .42309 L
.60406 .42307 L
.60589 .42305 L
.60785 .42303 L
.60973 .42301 L
.61173 .42299 L
.61365 .42297 L
.6155 .42295 L
.61747 .42292 L
.61937 .4229 L
.6212 .42288 L
.62314 .42286 L
.62501 .42284 L
.627 .42282 L
.62891 .4228 L
.63076 .42278 L
.63272 .42275 L
.6346 .42273 L
.63642 .42271 L
.63835 .42269 L
.64021 .42267 L
.64219 .42265 L
.64409 .42263 L
.64592 .42261 L
.64787 .42258 L
.64975 .42256 L
.65175 .42254 L
.65367 .42252 L
.65552 .4225 L
.65748 .42248 L
.65938 .42245 L
.6612 .42243 L
.66314 .42241 L
.665 .42239 L
.66699 .42237 L
.6689 .42235 L
.67074 .42233 L
.67269 .4223 L
.67458 .42228 L
.67658 .42226 L
.67851 .42224 L
.68036 .42222 L
.68234 .42219 L
.68424 .42217 L
.68606 .42215 L
.68801 .42213 L
.68988 .4221 L
Mistroke
.69187 .42208 L
.69379 .42205 L
.69563 .42202 L
.69657 .42201 L
.6976 .42198 L
.69807 .42197 L
.69857 .42196 L
.69881 .42195 L
.69905 .42193 L
.69949 .42191 L
.69973 .42189 L
.69995 .42186 L
.70007 .42184 L
.7002 .42182 L
.70032 .42178 L
.70038 .42176 L
.70044 .42174 L
.70049 .42171 L
.70054 .42168 L
.70059 .42165 L
.70064 .42161 L
.7007 .42155 L
.70076 .42145 L
.70081 .42133 L
.70087 .42112 L
.70093 .4207 L
.70096 .42022 L
.70099 .4193 L
.70105 .35131 L
.70111 .42475 L
.70116 .42342 L
.70121 .42297 L
.70126 .42272 L
.70132 .42257 L
.70138 .42246 L
.70141 .42242 L
.70145 .42238 L
.70151 .42233 L
.70158 .42228 L
.70164 .42225 L
.7017 .42223 L
.70181 .42219 L
.70194 .42216 L
.70207 .42213 L
.70221 .42211 L
.70235 .4221 L
.70259 .42208 L
.70281 .42206 L
.70331 .42204 L
.70378 .42202 L
Mistroke
.70428 .422 L
.70517 .42198 L
.70699 .42195 L
.70894 .42192 L
.71081 .4219 L
.7128 .42187 L
.71472 .42185 L
.71656 .42183 L
.71852 .4218 L
.72041 .42178 L
.72222 .42176 L
.72416 .42174 L
.72602 .42172 L
.728 .42169 L
.7299 .42167 L
.73173 .42165 L
.73368 .42163 L
.73556 .42161 L
.73756 .42158 L
.73948 .42156 L
.74133 .42154 L
.7433 .42152 L
.74519 .4215 L
.74702 .42148 L
.74896 .42145 L
.75082 .42143 L
.75281 .42141 L
.75472 .42139 L
.75656 .42137 L
.75852 .42135 L
.7604 .42132 L
.7624 .4213 L
.76433 .42128 L
.76619 .42126 L
.76816 .42124 L
.77006 .42122 L
.77189 .42119 L
.77384 .42117 L
.77571 .42115 L
.7777 .42113 L
.77962 .42111 L
.78147 .42109 L
.78343 .42106 L
.78532 .42104 L
.78714 .42102 L
.78908 .421 L
.79094 .42098 L
.79292 .42096 L
.79483 .42093 L
.79666 .42091 L
Mistroke
.79862 .42089 L
.8005 .42087 L
.8025 .42085 L
.80442 .42082 L
.80627 .4208 L
.80824 .42078 L
.81014 .42076 L
.81197 .42074 L
.81391 .42072 L
.81578 .4207 L
.81777 .42067 L
.81968 .42065 L
.82152 .42063 L
.82348 .42061 L
.82537 .42059 L
.82718 .42057 L
.82911 .42054 L
.83097 .42052 L
.83295 .4205 L
.83485 .42048 L
.83668 .42046 L
.83863 .42044 L
.84051 .42041 L
.8425 .42039 L
.84443 .42037 L
.84627 .42035 L
.84824 .42033 L
.85013 .4203 L
.85195 .42028 L
.85389 .42026 L
.85576 .42024 L
.85774 .42022 L
.85965 .4202 L
.86149 .42017 L
.86344 .42015 L
.86533 .42013 L
.86733 .42011 L
.86926 .42009 L
.87111 .42006 L
.87308 .42004 L
.87498 .42002 L
.87681 .42 L
.87876 .41998 L
.88063 .41996 L
.88262 .41993 L
.88453 .41991 L
.88638 .41989 L
.88834 .41987 L
.89023 .41984 L
.89205 .41982 L
Mistroke
.89398 .4198 L
.89584 .41978 L
.89782 .41976 L
.89973 .41974 L
.90156 .41971 L
.90351 .41969 L
.90539 .41967 L
.90739 .41965 L
.90931 .41963 L
.91116 .4196 L
.91313 .41958 L
.91503 .41956 L
.91685 .41954 L
.91879 .41952 L
.92066 .4195 L
.92265 .41947 L
.92456 .41945 L
.9264 .41943 L
.92836 .41941 L
.93024 .41939 L
.93225 .41936 L
.93418 .41934 L
.93603 .41932 L
.93801 .4193 L
.93991 .41927 L
.94174 .41925 L
.94369 .41923 L
.94556 .41921 L
.94756 .41919 L
.94948 .41916 L
.95132 .41914 L
.95329 .41912 L
.95518 .4191 L
.957 .41908 L
.95893 .41905 L
.9608 .41903 L
.96278 .41901 L
.96469 .41899 L
.96653 .41897 L
.96848 .41894 L
.97036 .41892 L
.97236 .4189 L
.97429 .41888 L
.97614 .41885 L
.97619 .41885 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3oool00`3ooml0W4P0000000030000000304RLg`3oool0gia80080
000000<0M;oo0?ooo`3oool0X`3oool00?l0oooo00<0_g@00018W03Oool00`3oool00`3OW9`0_ooo
0?ooo`020?ooo`030;mdM02oool0oooo0:<0oooo003o0?ooo`030?oog`2LM9`0gooo00<0oooo00<0
oono09bLg`3oool00P3oool00`3ogi`0M9cO0?ooo`2S0?ooo`00o`3oool10?ooo`030;md00000000
000000<0000000L0B9cO0?ooo`3oool0oooo0?oog`2LB7@0_ooo0:<0oooo003R0?ooo`070;md0000
000000000000M02oool0oooo0;md0002000000030000M02ooml0W4P00080000000@004RL0=ooo`3o
ool0_g@00P000000100007@0_ooo0?ooo`2oM002000000050000B02Lgol0oooo0?oo_`1d00001000
00000`0007@0_ooo0?ooo`030?ooo`030;md001d_ol0oooo0:80oooo003Q0?ooo`0?0;md00000000
07Bo0?ooo`3ogi`0B01d0;ooo`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo0080oooo00/0
_g@000000000M;l0oooo0?oOW01807@0_ooo0?ooo`3OW4P0001d0;ooo`030?ooo`030;mdB02Lgol0
oooo00L0oooo00@0omnL04P00000B9`0goooXP3oool00>00oooo00<0omnL04P00018W=l00`3oool0
3P3ogi`0M9cO0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?oOW0180000B9cO0`3o
ool01P3ogi`0M9cO0?ooo`3OW4P0001d0;ooo`<0oooo00D0oono07@0000000000000001d_`050?oo
o`040;md0000B9`0W7@007Booj80oooo003P0?ooo`030?noM00004P0W=oo00H0oooo00/0gia80018
W03Oool0oooo0?oog`2LB00007Bo0?ooo`3o_g@0001809cOo`060?ooo`030=nLB00007@0_ooo00<0
oooo00H0gia80000M02oool0oono07@00018W=l30?ooo`050?noM0000000M;oo0?oOW018M;l0XP3o
ool00>00oooo00<0okmd0000M02oool01P3oool02`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0
oooo0?noM00007@0_ooo00H0oooo00<0gia80000M02oool00P3oool01`3ookl0M00004RLg`3oool0
oooO09a80000M;l00P3oool01`3ooml0W4P00018W03Oool0oooo0;mdB02Lgol0X@3oool00>00oooo
00<0omnL04P000000000100000003018W=l0oooo0;md000007@0_ooo0?ooo`3ooml0W4P0001d_`3o
ool0omnL04P000D0000000D0B9cO0?ooo`3OW4P0001809cOo`020?ooo`070?oo_`1d0000B9cO0?oo
o`3ooml0W4P004RLg`020?ooo`030?noM00004P0W=oo0080oooo00<0okmd04RLg`3oool0X03oool0
0>40oooo00l0_g@004RLg`3oool0oooo0=nLB00004P0W=oO09a80000000004Q804P0B02Lgol0oono
07@00018W=l00P3oool04P2oM000B9cO0?ooo`3oool0gia80000B02Lgml0W4P00000000004P0B7BL
0=noM00007@0_ooo0;md0000M;l0oooo0?noM080000000D004RL0=oo_`1d000004RL0=ooo`020?oo
o`030?oo_`1d07@0_ooo0:00oooo003R0?ooo`050=nLB000000000000000B02Lgol00P3oool01`2o
M00004RL0=oog`2LB00000000000M02oool00`3oool01@3OW4P000000000000004P0W=oo0080oooo
00P0_g@00000M02ogi`0B0000000M02oool0oooO09a80080000000@0M;oo0?ooo`3ooml0W4P00P00
00001`00B9`0gooo0?ooo`2oM00000000018W03Oool0W`3oool00?l0oooo/@3oool00?l0oooo/@3o
ool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3o
ool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3o
ool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool005D0oooo00<0
_g@0000000000000100000001P18W=l0oooo0?ooo`3o_g@0000007Boo`<0oooo00D0omnL04P00000
0000000007Boo`0e0?ooo`030;md00000000000000@0000000H0B9cO0?ooo`3oool0okmd0000001d
_ol40?ooo`030=nLB018W=l0oooo03H0oooo00<0_g@0000000000000100000001P18W=l0oooo0?oo
o`3o_g@0000007Boo`<0oooo00D0gia8000000000000000004RLg`0e0?ooo`030;md000000000000
00@0000000`0B9cO0?ooo`3oool0okmd0000001d_ol0oooo0?oo_`1d000000000018W03Oooln0?oo
o`030?oo_`1d000000000080000000<0M;oo0?ooo`3oool0903oool005H0oooo00<0gia800000000
00000`0000001P00B9`0gooo0?ooo`3o_g@0000007Boo`80oooo00L0omnL04P00018W=l0oooo0?oo
_`1d0000M;oo03D0oooo00<0gia80000000000000`0000001P00B9`0gooo0?ooo`3o_g@0000007Bo
o`@0oooo00<0omnL04Q8W03Oool0=`3oool00`3OW4P0000000000003000000060018W03Oool0oooo
0?noM0000000M;oo0P3oool01`3OW4P0000007Boo`3oool0oooO09a80018W=l0=@3oool00`3OW4P0
000000000003000000060018W03Oool0oooo0?noM0000000M;oo103oool00`3OW4P0000007Boo`0m
0?ooo`030?oOW0180000B9cO0080oooo00<0_g@004RLg`3oool0903oool005L0oooo00<0gia804RL
g`3oool00P3oool00`3OW7@0W=oo0?ooo`040?ooo`030?oog`2LB00007Bo00<0oooo00<0okmd0018
W03Oool0=@3oool00`3OW4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo00L0oooo00<0oooO09a8
B02Lgol0>03oool00`3OW4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo00@0oooo00<0oooO09a8
0000M;l00`3oool00`3o_g@004RL0=ooo`0e0?ooo`030=nLB018W=l0oooo0080oooo00<0giad09cO
o`3oool0203oool00`3ookl0M00007Boo`110?ooo`030?oo_`1d04P0W=oo02@0oooo001H0?ooo`03
0=nLB01d_ol0oooo00P0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0f0?ooo`03
0=nLB01d_ol0oooo00`0oooo00<0gia804RLg`3oool0>03oool00`3OW4P0M;oo0?ooo`090?ooo`03
0;md0000M;l0oooo0080oooo00<0_g@00018W03Oool0=P3oool00`3OW4P0M;oo0?ooo`0=0?ooo`03
0?oOW01804P0W=oo0400oooo00<0oono07@0001d_ol0903oool005T0oooo00<0_gA809cOo`3oool0
1`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo03L0oooo00<0_gA809cOo`3oool0
2`3oool00`3ogi`0B4RL0=ooo`0i0?ooo`030;mdB02Lgol0oooo00P0oooo00L0omnL04P0M02oool0
oooo0;md00000000M;oo03P0oooo00<0_gA809cOo`3oool0203oool00`3ooml0W4P0000000030000
0003001d_`3oool0oooo03h0oooo00<0okmd0000001d_ol0903oool005T0oooo00<0oooO09a8M02o
ool01`3oool00`3ogi`0B00007Boo`030?ooo`030;md000004P0W=oo03L0oooo00<0oooO09a8M02o
ool02`3oool00`3ooml0W4Q809cOo`0i0?ooo`030?oog`2LB7@0_ooo00T0oooo00D0oono07A8B018
0000000004RLg`0i0?ooo`030?oog`2LB7@0_ooo00P0oooo00P0_g@00000001d_ol0oooo0?oog`2L
B000001d0;oooch0oooo00@0omnL04P0000004P0W=oo903oool005X0oooo00<0okmd001d_`3oool0
1P3oool0203ookl0M00000000018W=l0oooo0?noM000000007Bo>@3oool00`3o_g@007Bo0?ooo`0;
0?ooo`030=nLB000M;l0oooo03T0oooo00<0okmd001d_`3oool0203oool0103ogi`0B00000000000
M;lk0?ooo`030?noM000M;l0oooo00H0oooo00<0omnL04P00018W=l00`3oool00`3o_g@0001809cO
o`0l0?ooo`050?noM000000000000000B02Lgol09@3oool005X0oooo00<0oono07@0B02Lgol01`3o
ool00`2oM00004RL0;nLB002000000030018W03Oool0oooo03P0oooo00<0oono07@0B02Lgol02`3o
ool00`3ookl0M01d0;ooo`0i0?ooo`030?oo_`1d04P0W=oo00L0oooo00H0omnL04P0000007@0_ooo
0?noM000M;lj0?ooo`030?oo_`1d04P0W=oo00H0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0
001809cOo`0n0?ooo`030;md0000M;l0oooo02D0oooo001J0?ooo`030?oo_`1d0000B9cO00L0oooo
00<0omnL04P0001d_ol0?@3oool00`3ookl0M00004RLg`060?ooo`030?oOg`3Oool0oooo00<0oooo
00<0_g@007Boo`3oool0>03oool00`3ookl0M00004RLg`070?ooo`030;md000007@0_ooo0080oooo
00<0_g@004RLg`3oool0>03oool00`3ookl0M00004RLg`060?ooo`030?oOW0180000M;oo00<0oooo
00<0gia80000M02oool0?`3oool00`2oM000B9cO0?ooo`0T0?ooo`00E@3oool00`3o_i`0W=oo0?oo
o`020?ooo`030?oOW0180000B9cO00P0oooo00<0gia80000B02Lgol0=`3oool00`3o_i`0W=oo0?oo
o`020?ooo`030?oOW0180000B9cO00H0oooo00<0oono07A8W03Oool00`3oool00`3o_g@007Bo0?oo
o`0c0?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l01`3oool00`2oM00004RL0=oo
o`020?ooo`030=nLB000M;l0oooo03<0oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0B00004RL
g`060?ooo`030?oog`2LB000M;oo0080oooo00@0oooO09a80000B9`0gooo>P3oool00`3ooml0W9bo
0?ooo`020?ooo`030=nLB00007@0_ooo02@0oooo001E0?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0
M0000000M02oool90?ooo`030?noM000000007Bo03H0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d
0000001d0;ooo`L0oooo00<0_g@00000000000000`0000000`0007@0_ooo0?ooo`0b0?ooo`080?oo
g`2LB4P0W=oo0?ooo`3ookl0M0000000M02oool70?ooo`070?noM0000000M;oo0?ooo`3ookl0M000
07Boo`0d0?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02oool70?ooo`070?noM00007@0
_ooo0?ooo`3OW4P0001809cOo`0l0?ooo`060?noM018W=l0oooo0?oo_`1d000007Bo9@3oool005H0
oooo00H0oooO09a80000000000000000M02oool;0?ooo`050?oog`2LB00000000000M02oool0=03o
ool01P3ooml0W4P0000000000000001d0;ooo`P0oooo00<0okmd000000000000100000000`1d_ol0
oooo0?ooo`0b0?ooo`060?oog`2LB00000000000000007@0_ooo2@3oool01@3o_g@0000000000000
0000B9cO03H0oooo00H0oooO09a80000000000000000M02oool90?ooo`050?noM000000000000000
B02Lgol0?P3oool01@3OW4P000000000000007@0_ooo02D0oooo003o0?oook40oooo003o0?oook40
oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40
oooo003o0?oook40oooo001P0?ooool00000<P00000O0?ooo`00H03oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0M
0?ooo`00H03oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`3o0?ooobh0
oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000
003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo0?l0oooo;P3oool00`000000oooo0?oo
o`0M0?ooo`00H03oool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool0
0600oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo001P0?ooo`03
0000003oool0oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo
0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool00600oooo0P00003o0?ooobd0oooo0`00
000O0?ooo`00H03oool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool0
0600oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo001P0?ooo`03
0000003oool0oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00B`3oool0103ooml0W4P0
0018W03Oool30?ooo`050?oOW018000000000000M02oool02@3oool00`000000oooo0?ooo`3o0?oo
obh0oooo00<000000?ooo`3oool07@3oool004/0oooo00@0oooO09a80000B9`0gooo0P3oool01`3o
okl0M00007Boo`3oool0okmd0000M02oool0203oool00`000000oooo0?ooo`3o0?ooobh0oooo00<0
00000?ooo`3oool07@3oool004/0oooo00@0oooO09a80000B9`0gooo0P3oool00`2oM00004RL0=oo
o`020?ooo`030;md0000M;l0oooo00L0oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000003o
ool0oooo01d0oooo00170?ooo`030;md00000000000000D0000000@0M;oo0?oo_`1d0000B9cO0`3o
ool00`3OW4P0001d0;ooo`070?ooo`030000003oool0oooo0?l0oooo;P3oool00`000000oooo0?oo
o`0M0?ooo`00>`3oool00`3ooml0W4P0000000070000000404RLg`3oool0omnL07BLg`80oooo00P0
oooO09a80000B9`0gooo0?ooo`3ogi`0B00007Boo`<0oooo00<0okmd0000B02Lgol01`3oool00`00
0000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool004P0oooo00/0gia807Boo`3o
ool0oooO09a80000B9`0gooo0?ooo`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00L0oooo
0`00003o0?ooob`0oooo0`00000O0?ooo`00B03oool02`3ooml0W4Qd0;ooo`3ooml0W4P00018W03O
ool0oooo0?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01`3oool00`000000oooo0?ooo`3o
0?ooobh0oooo00<000000?ooo`3oool07@3oool004T0oooo00X0omnL04Qd_`3ooml0W4P00018W03O
ool0oooo0?oOW0180000M;oo0`3oool00`3o_g@0001809cOo`070?ooo`030000003oool0oooo0?l0
oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00BP3oool02@3OW4P0M;oO09a80000B9`0gooo0?oo
o`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00L0oooo00<000000?ooo`3oool0o`3oool^
0?ooo`030000003oool0oooo01d0oooo001:0?ooo`050?oog`2LB4P0B4P00018W03Oool00P3oool0
0`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00L0oooo00<000000?ooo`3oool0o`3oool^
0?ooo`030000003oool0oooo01d0oooo001;0?ooo`040?oOW018000004RL0=ooo`80oooo00L0oono
07@0001d_ol0oooo0?noM00007@0_ooo00P0oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000
003oool0oooo01d0oooo001<0?ooo`030=nLB000B9`0gooo00<0oooo00D0oono07@000000000001d
0;ooo`090?ooo`030000003oool0oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00H03o
ool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool00600oooo00<00000
0?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo
0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00H03oool200000?l0oooo;@3oool3000001l0
oooo001P0?ooo`030000003oool0oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00H03o
ool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool00600oooo00<00000
0?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo
0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`3o0?ooobh0
oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000
003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo0?l0oooo;P3oool00`000000oooo0?oo
o`0M0?ooo`00H03oool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool0
0600oooo0P00003o0?ooobd0oooo0`00000O0?ooo`00H03oool00`000000oooo0?ooo`3o0?ooobh0
oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000
003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo0?l0oooo;P3oool00`000000oooo0?oo
o`0M0?ooo`00H03oool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool0
0600oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo001P0?ooo`03
0000003oool0oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo
0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0o`3o
ool^0?ooo`030000003oool0oooo01d0oooo001P0?ooo`800000o`3oool]0?ooo`<000007`3oool0
0600oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo001P0?ooo`03
0000003oool0oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo
0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0o`3o
ool^0?ooo`030000003oool0oooo01d0oooo00170?ooo`030;md00000000000000@0000000T0B9cO
0?ooo`3oool0oooo0?oOW018000000000000M02oool02@3oool00`000000oooo0?ooo`3o0?ooobh0
oooo00<000000?ooo`3oool07@3oool004P0oooo00<0gia80000000000000`0000002P00B9`0gooo
0?ooo`3ookl0M00007Boo`3oool0okmd0000M02oool80?ooo`030000003oool0oooo0?l0oooo;P3o
ool00`000000oooo0?ooo`0M0?ooo`00B@3oool00`3OW4P0B9cO0?ooo`020?ooo`060=nLM02Lgol0
oooo0;md0000B9`0gooo0P3oool00`2oM00007Bo0?ooo`070?ooo`030000003oool0oooo0?l0oooo
;P3oool00`000000oooo0?ooo`0M0?ooo`00BP3oool00`3OW4P0M;oo0?ooo`030?ooo`030?oo_`1d
0000B9cO00<0oooo00<0gia80000M02oool01`3oool00`000000oooo0?ooo`3o0?ooobh0oooo00<0
00000?ooo`3oool07@3oool003/0oooo00<0oooO09a8000000001`0000000`18W=l0oooo0?ooo`03
0?ooo`030;mdB02Lgol0oooo0080oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`07
0?ooo`030000003oool0oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00B`3oool00`3o
oml0W4Qd0;ooo`020?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01`3oool30000
0?l0oooo;03oool3000001l0oooo001<0?ooo`070?noM000M;l0oooo0?ooo`3ogi`0B00007Boo`03
0?ooo`030?noM00004P0W=oo00L0oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000003oool0
oooo01d0oooo001<0?ooo`070?oo_`1d04P0W=oo0?ooo`3ogi`0B00007Boo`030?ooo`030?noM000
04P0W=oo00L0oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo000X
0?ooo`030?oo_`1dM;l0oooo00@0oooo00<0gibL0;ooo`3oool06P3oool01`3ookl0M00004RLg`3o
ool0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`070?ooo`030000003oool0oooo0?l0oooo
;P3oool00`000000oooo0?ooo`0M0?ooo`00:03oool00`3o_g@0000000000004000000030000M02o
ool0oooo01D0oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0B00004RLg`020?ooo`030=nLB000
B9`0gooo0080oooo00<0_g@0001d_`3oool01`3oool00`000000oooo0?ooo`3o0?ooobh0oooo00<0
00000?ooo`3oool07@3oool002P0oooo00<0_g@0000000000000100000000`0007@0_ooo0?ooo`0E
0?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02oool20?ooo`070?oo_`1d0000M;oo0?oo
o`3o_g@0001d0;ooo`080?ooo`030000003oool0oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M
0?ooo`00:@3oool00`2oM7@0_ooo0?ooo`030?ooo`030=nLW02oool0oooo01H0oooo00H0oooO09a8
0000000000000000M02oool40?ooo`050?oo_`1d000000000000M02oool02@3oool00`000000oooo
0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool002P0oooo00<0omnL09boo`3oool0=@3o
ool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool002P0oooo00<0_g@0
001d_`3oool0103oool00`3OW9`0_ooo0?ooo`0^0?ooo`030000003oool0oooo0?l0oooo;P3oool0
0`000000oooo0?ooo`0M0?ooo`00:03oool00`3OW4P0000000000004000000030000M02oool0oooo
02h0oooo0P00003o0?ooobd0oooo0`00000O0?ooo`00:03oool00`3ooml0W4P00000000400000003
0000M02oool0oooo02h0oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0
oooo000Y0?ooo`030=nLW02oool0oooo00<0oooo00<0gibL0;ooo`3oool0;P3oool00`000000oooo
0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool002P0oooo00<0okmd04RLg`3oool0=@3o
ool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool002P0oooo00<0_g@0
001d_`3oool0103oool00`3OW9`0_ooo0?ooo`0^0?ooo`030000003oool0oooo0?l0oooo;P3oool0
0`000000oooo0?ooo`0M0?ooo`00:03oool00`3OW4P0000000000004000000030000M02oool0oooo
02h0oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo000X0?ooo`03
0?oog`2LB000000000@0000000<0001d0;ooo`3oool0;P3oool00`000000oooo0?ooo`3o0?ooobh0
oooo00<000000?ooo`3oool07@3oool002l0oooo00<0gibL0;ooo`3oool0;P3oool00`000000oooo
0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool002P0oooo00<0oono07Ad_`3oool0103o
ool00`3OW9`0_ooo0?ooo`0^0?ooo`030000003oool0oooo0?l0oooo;P3oool00`000000oooo0?oo
o`0M0?ooo`008P3oool00`3ooml0W4P0001d_`030?ooo`030?noM0000000000000@0000000<0001d
0;ooo`3oool0;P3oool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool0
0280oooo00<0oooO09a80000M;l00`3oool00`2oM000000000000004000000030000M02oool0oooo
02h0oooo0P00003o0?ooobd0oooo0`00000O0?ooo`00;`3oool00`3OW9`0_ooo0?ooo`0^0?ooo`03
0000003oool0oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo
0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0o`3o
ool^0?ooo`030000003oool0oooo01d0oooo000X0?ooo`090?oog`2LB00000000000M02ooml0W4P0
0000000007@0_ooo02l0oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0
oooo000X0?ooo`080=nLM02Lgol0omnL04P00000M;l0oooo0?noM018W=l`0?ooo`030000003oool0
oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00:03oool02@3OW9`0_ooo0?ooo`2oM000
001809cOo`3oool0_gAd0;ooo`0_0?ooo`030000003oool0oooo0?l0oooo;P3oool00`000000oooo
0?ooo`0M0?ooo`00:03oool02@3ogi`0B7Bo0?ooo`3ogi`0B000001d_`3oool0_ibL0=ooo`0_0?oo
o`030000003oool0oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00:03oool01@2oM000
00000000B02Lgol0gia80080000000<0M;oo0?ooo`3oool0;P3oool00`000000oooo0?ooo`3o0?oo
obh0oooo00<000000?ooo`3oool07@3oool00600oooo0P00003o0?ooobd0oooo0`00000O0?ooo`00
:03oool00`3ookl0M7Bo0?ooo`040?ooo`030=nLW02oool0oooo02h0oooo00<000000?ooo`3oool0
o`3oool^0?ooo`030000003oool0oooo01d0oooo000X0?ooo`030?noM0000000000000@0000000<0
001d0;ooo`3oool0;P3oool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3o
ool002P0oooo00<0_g@0000000000000100000000`0007@0_ooo0?ooo`0^0?ooo`030000003oool0
oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00:@3oool00`2oM7@0_ooo0?ooo`030?oo
o`030=nLW02oool0oooo0200oooo00D0omnL04P000000000001d0;ooo`090?ooo`030000003oool0
oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00:03oool00`3ogi`0W;oo0?ooo`0V0?oo
o`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`080?ooo`030000003oool0oooo0?l0oooo;P3o
ool00`000000oooo0?ooo`0M0?ooo`00:03oool00`2oM00007Bo0?ooo`040?ooo`030=nLW02oool0
oooo01l0oooo00<0_g@00018W03Oool00P3oool00`2oM00007Bo0?ooo`070?ooo`030000003oool0
oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00:03oool00`3OW4P00000000000040000
00030000M02oool0oooo01h0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`070?oo
o`030000003oool0oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00:03oool00`3ooml0
W4P000000004000000030000M02oool0oooo01h0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0
001809cOo`070?ooo`030000003oool0oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00
:@3oool00`3OW9`0_ooo0?ooo`030?ooo`030=nLW02oool0oooo01h0oooo00<0omnL04P0001d_ol0
0`3oool00`3o_g@0001809cOo`070?ooool00000<P00000O0?ooo`00:03oool00`3o_g@0B9cO0?oo
o`0U0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01`3oool00`000000oooo0?oo
o`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool002P0oooo00<0_g@0001d_`3oool0103oool0
0`3OW9`0_ooo0?ooo`0N0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01`3oool0
0`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool002P0oooo00<0gia80000
00000000100000000`0007@0_ooo0?ooo`0N0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000
M02oool01`3oool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool002P0
oooo00<0oooO09a800000000100000000`0007@0_ooo0?ooo`0O0?ooo`030=nLB000B9`0gooo0080
oooo00<0_g@0001d_`3oool01`3oool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3o
ool07@3oool002l0oooo00<0gibL0;ooo`3oool07`3oool01`3ookl0M00007Boo`3oool0okmd0000
M02oool0203oool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool00580
oooo00D0oono07@000000000001d0;ooo`090?ooo`030000003oool0oooo0?l0oooo;P3oool00`00
0000oooo0?ooo`0M0?ooo`00:@3oool01`2oM000001809cOo`3ookl0M00000000018W=l0<03oool0
0`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool002P0oooo00D0okmd0000
000004P0W=oo0;md0002000000030000M02oool0oooo02h0oooo00<000000?ooo`3oool0o`3oool^
0?ooo`030000003oool0oooo01d0oooo000X0?ooo`090;mdM02oool0oooo0?oo_`1dB9`0gooo0?oO
W018B9`0gooo02l0oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo
000X0?ooo`080;mdB02Lgol0oooo0=nLM02Lgol0oooo0?noW02Lgol`0?ooo`800000o`3oool]0?oo
o`<000007`3oool002P0oooo00<0gia8000000000000100000000`00B9`0gooo0?ooo`0^0?ooo`03
0000003oool0oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00:03oool00`3ookl0M000
00000004000000030018W03Oool0oooo02h0oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000
003oool0oooo01d0oooo000^0?ooo`030;mdB02Lgol0oooo02l0oooo00<000000?ooo`3oool0dP3o
ool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo01d0oooo000X0?ooo`030;mdM02oool0
oooo03D0oooo00<000000?ooo`3oool0dP3oool00`000000oooo0?ooo`1H0?ooo`030000003oool0
oooo01d0oooo000V0?ooo`030?oo_`1d0000000000H0000000<0B9cO0?ooo`3oool0;P3oool00`00
0000oooo0?ooo`3B0?ooo`030000003oool0oooo05P0oooo00<000000?ooo`3oool07@3oool002D0
oooo00<0okmd0000000000001`0000000`0007@0_ooo0?ooo`0^0?ooo`030000003oool0oooo0=80
oooo00<000000?ooo`3oool0F03oool00`000000oooo0?ooo`0M0?ooo`00:03oool00`2oM7@0_ooo
0?ooo`030?ooo`030?oo_`1dB9`0gooo02l0oooo00<000000?ooo`3oool0dP3oool00`000000oooo
0?ooo`1H0?ooo`030000003oool0oooo01d0oooo000X0?ooo`030;mdM02oool0oooo00<0oooo00<0
_gA809cOo`3oool0;`3oool00`000000oooo0?ooo`3B0?ooo`030000003oool0oooo05P0oooo00<0
00000?ooo`3oool07@3oool00600oooo0P00003C0?ooo`030000003oool0oooo05H0oooo0`00000O
0?ooo`00:@3oool00`3ooml0W4P0000000020000000307Boo`3oool0oooo02l0oooo00<000000?oo
o`3oool0dP3oool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo01d0oooo000X0?ooo`03
0?oo_`1d0000000000@0000000<0B9cO0?ooo`3oool0;P3oool00`000000oooo0?ooo`3B0?ooo`03
0000003oool0oooo05P0oooo00<000000?ooo`3oool07@3oool002P0oooo00<0gia804RLg`3oool0
0P3oool0103o_g@000000000M02oool_0?ooo`030000003oool0oooo0=80oooo00<000000?ooo`3o
ool0F03oool00`000000oooo0?ooo`0M0?ooo`00:03oool00`2oM7@0_ooo0?ooo`030?ooo`030=nL
B00007@0_ooo02l0oooo00<000000?ooo`3oool0dP3oool00`000000oooo0?ooo`1H0?ooo`030000
003oool0oooo01d0oooo000X0?ooo`030=nLB018W=l0oooo00<0oooo00<0_g@0001d_`3oool0;`3o
ool00`000000oooo0?ooo`3B0?ooo`030000003oool0oooo05P0oooo00<000000?ooo`3oool07@3o
ool002P0oooo00L0oooO09a8000007@0_ooo0=nLB0000000B9cO0340oooo00<000000?ooo`3oool0
dP3oool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo01d0oooo001P0?ooo`030000003o
ool0oooo0=80oooo00<000000?ooo`3oool0F03oool00`000000oooo0?ooo`0M0?ooo`008`3oool0
0`3o_i`0W=oo0?ooo`090?ooo`030=nLW02oool0oooo02h0oooo00<000000?ooo`3oool0dP3oool0
0`000000oooo0?ooo`1H0?ooo`030000003oool0oooo01d0oooo000S0?ooo`030;md000000000000
00T0000000<0001d0;ooo`3oool0;P3oool200000=<0oooo00<000000?ooo`3oool0EP3oool30000
01l0oooo000R0?ooo`030?oo_`1d0000000000X0000000<0001d0;ooo`3oool0;P3oool00`000000
oooo0?ooo`3B0?ooo`030000003oool0oooo05P0oooo00<000000?ooo`3oool07@3oool002T0oooo
00<0gibL0;ooo`3oool00`3oool00`3OW9`0_ooo0?ooo`0^0?ooo`030000003oool0oooo0=80oooo
00<000000?ooo`3oool0F03oool00`000000oooo0?ooo`0M0?ooo`00:03oool00`3o_g@0B9cO0?oo
o`0e0?ooo`030000003oool0oooo0=80oooo00<000000?ooo`3oool0F03oool00`000000oooo0?oo
o`0M0?ooo`00:03oool00`2oM00007Bo0?ooo`040?ooo`030=nLW02oool0oooo02h0oooo00<00000
0?ooo`3oool0dP3oool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo01d0oooo000X0?oo
o`030=nLB0000000000000@0000000<0001d0;ooo`3oool05@3oool00`2oM0000000000000040000
000904RLg`3oool0oooo0?ooo`3ogi`0B0000000000007@0_ooo00T0oooo00<000000?ooo`3oool0
dP3oool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo01d0oooo000X0?ooo`030?oog`2L
B000000000@0000000<0001d0;ooo`3oool05P3oool00`3OW4P00000000000030000000:0018W03O
ool0oooo0?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`P0oooo00<000000?ooo`3oool0dP3oool0
0`000000oooo0?ooo`090?ooodP000001`3oool00`000000oooo0?ooo`0M0?ooo`00;`3oool00`3O
W9`0_ooo0?ooo`0G0?ooo`030=nLB018W=l0oooo0080oooo00H0giad09cOo`3oool0_g@00018W03O
ool20?ooo`030;md0000M;l0oooo00L0oooo00<000000?ooo`3oool0Q03ooomJ000004l0oooo00<0
00000?ooo`3oool07@3oool004X0oooo00<0gia807Boo`3oool00`3oool00`3ookl0M00004RLg`03
0?ooo`030=nLB00007@0_ooo00L0oooo00<000000?ooo`3oool09`3ooomN00000:P0oooo00<00000
0?ooo`3oool07@3oool004/0oooo00<0_gA809cOo`3oool00P3oool00`3ogi`0B00007Boo`030?oo
o`030?noM00004P0W=oo00L0oooo00<000000?ooo`3oool0103ooolT00000?l0oooo1P3oool00`00
0000oooo0?ooo`0M0?ooo`00B`3oool00`3ooml0W4Qd0;ooo`020?ooo`030?oOW0180000M;oo00<0
oooo00<0okmd0000B02Lgol01`3oool300000?l0oooo;03oool3000001l0oooo001<0?ooo`070?no
M000M;l0oooo0?ooo`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00L0oooo00<000000?oo
o`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo001<0?ooo`070?oo_`1d04P0W=oo0?oo
o`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00L0oooo00<000000?ooo`3oool0o`3oool^
0?ooo`030000003oool0oooo01d0oooo001<0?ooo`070?oo_`1d0000B9cO0?ooo`3ookl0M00004RL
g`030?ooo`030=nLB00007@0_ooo00L0oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000003o
ool0oooo01d0oooo00170?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l00P3oool0
0`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00L0oooo00<000000?ooo`3oool0o`3oool^
0?ooo`030000003oool0oooo01d0oooo00170?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000
M02oool20?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`080?ooo`030000003oool0oooo
0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00B03oool01P3ooml0W4P0000000000000001d
0;ooo`@0oooo00D0oono07@000000000001d0;ooo`090?ooo`030000003oool0oooo0?l0oooo;P3o
ool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`3o0?ooobh0oooo00<00000
0?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo
01d0oooo001P0?ooo`800000o`3oool]0?ooo`<000007`3oool00600oooo00<000000?ooo`3oool0
o`3oool^0?ooo`030000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo0?l0oooo;P3o
ool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`3o0?ooobh0oooo00<00000
0?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo
01d0oooo001P0?ooo`030000003oool0oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00
H03oool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool00600oooo00<0
00000?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo001P0?ooo`030000003oool0
oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`3o0?oo
obh0oooo00<000000?ooo`3oool07@3oool00600oooo0P00003o0?ooobd0oooo0`00000O0?ooo`00
H03oool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool00600oooo00<0
00000?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo001P0?ooo`030000003oool0
oooo0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`3o0?oo
obh0oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0o`3oool^0?ooo`03
0000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo0?l0oooo;P3oool00`000000oooo
0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3o
ool00600oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo001P0?oo
o`800000o`3oool]0?ooo`<000007`3oool00600oooo00<000000?ooo`3oool0o`3oool^0?ooo`03
0000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo0?l0oooo;P3oool00`000000oooo
0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3o
ool004/0oooo00@0oooO09a80000B9`0gooo0`3oool01@3ogi`0B0000000000007@0_ooo00T0oooo
00<000000?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo001;0?ooo`040?oog`2L
B00004RL0=ooo`80oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00P0oooo00<000000?oo
o`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo001;0?ooo`040?oog`2LB00004RL0=oo
o`80oooo00<0_g@00018W03Oool00P3oool00`2oM00007Bo0?ooo`070?ooo`030000003oool0oooo
0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00A`3oool00`2oM00000000000000500000004
07Boo`3ookl0M00004RLg`<0oooo00<0gia80000M02oool01`3oool00`000000oooo0?ooo`3o0?oo
obh0oooo00<000000?ooo`3oool07@3oool004L0oooo00`0omnL07BLg`3oool0oooo0?oog`2LB000
04RL0=ooo`3oool0omnL04P0001d_ol30?ooo`030?noM00004P0W=oo00L0oooo00<000000?ooo`3o
ool0o`3oool^0?ooo`030000003oool0oooo01d0oooo00180?ooo`0;0=nLB01d_ol0oooo0?oog`2L
B00004RL0=ooo`3oool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`070?ooo`<00000o`3o
ool/0?ooo`<000007`3oool004P0oooo00/0oooO09a8M02oool0oooO09a80000B9`0gooo0?ooo`3o
gi`0B00007Boo`030?ooo`030?noM00004P0W=oo00L0oooo00<000000?ooo`3oool0o`3oool^0?oo
o`030000003oool0oooo01d0oooo00190?ooo`0:0?oOW018M;l0oooO09a80000B9`0gooo0?ooo`3o
gi`0B00007Boo`<0oooo00<0okmd0000B02Lgol01`3oool00`000000oooo0?ooo`3o0?ooobh0oooo
00<000000?ooo`3oool07@3oool004X0oooo00T0gia807Bog`2LB00004RL0=ooo`3oool0oono07@0
0018W=l00`3oool00`3OW4P0001d0;ooo`070?ooo`030000003oool0oooo0?l0oooo;P3oool00`00
0000oooo0?ooo`0M0?ooo`00BP3oool01@3ooml0W4Q804Q80000B9`0gooo0080oooo00<0gia80018
W03Oool00P3oool00`2oM00007Bo0?ooo`070?ooo`030000003oool0oooo0?l0oooo;P3oool00`00
0000oooo0?ooo`0M0?ooo`00B`3oool0103ogi`0B0000018W03Oool20?ooo`070?oo_`1d0000M;oo
0?ooo`3o_g@0001d0;ooo`080?ooo`030000003oool0oooo0?l0oooo;P3oool00`000000oooo0?oo
o`0M0?ooo`00C03oool00`3OW4P004RL0=ooo`030?ooo`050?oo_`1d000000000000M02oool02@3o
ool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool00600oooo00<00000
0?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo
0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`3o0?ooobh0
oooo00<000000?ooo`3oool07@3oool00600oooo0P00003o0?ooobd0oooo0`00000O0?ooo`00H03o
ool00`000000oooo0?ooo`3o0?ooobh0oooo00<000000?ooo`3oool07@3oool00600oooo00<00000
0?ooo`3oool0o`3oool^0?ooo`030000003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo
0?l0oooo;P3oool00`000000oooo0?ooo`0M0?ooo`00H03oool00`000000oooo0?ooo`3o0?ooobh0
oooo00<000000?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0o`3oool^0?ooo`030000
003oool0oooo01d0oooo001P0?ooo`030000003oool0oooo0?l0oooo;P3oool00`000000oooo0?oo
o`0M0?ooo`00H03oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo04L0oooo00<00000
0?ooo`3oool0A`3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool07@3oool00600oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo00D0oooo00<000000?ooo`3oool07@3oool00600ooooo`00000b000001l0
oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40
oooo003o0?oook40oooo00360?ooo`030?oog`2LB000000000<0000000<0B9cO0?ooo`3oool0h@3o
ool00<H0oooo00<0_g@00018W03Oool00`3oool00`3OW9`0_ooo0?ooo`3Q0?ooo`00aP3oool00`3o
oml0W7BL0=ooo`030?ooo`030?oo_`2LW=l0oooo0>40oooo00370?ooo`030;md00000000000000<0
000000<0B9cO0?ooo`3oool0h03oool008d0oooo00H0gia8000000000000001809cOg`2LB0020000
00030000M02ooml0W4P00080000000<007Bo0?oog`2LB0000P0000002@00B9`0W7@000000000M;l0
oooo0?oOW0180000000004RLg`020?ooo`070;md0000000000000000B02Lgol0oooo0=nLB0020000
00030000B02Lgml0W4P00080000000H007Bo0=nLB000000000000000M01dM002000000030000M02o
oml0W4P00080000000<004RL0=oo_`1d0000100000000`0007@0_ooo0?ooo`080?ooo`050;md0000
000000000000B02Lgol00`3oool01@3OW4P00000000000000000M;oo0080oooo00H0_g@000000000
0000001d0;oog`2LB002000000040018W03Oool0oooO09a80080000000D0001804Q80000B9`0gooo
0=nLB002000000070000M02oool0omnL04P000000000B9cO0=nLB002000000040000M02oool0oooo
0=nLB0<0000000@0M;oo0?ooo`3oool0_g@00P0000000`0007@0_ooO09a80002000000030018W03O
ool0oooo00@0oooo00<0omnL04P0000000000P000000101d_ol0oooo0?ooo`3OW4P30000000407Bo
o`3oool0oooo0;md0080000000<0001809cOo`3oool0103oool01P3OW4P000000000000004P0W=oO
09a80080000000<0001d0;oog`2LB0000P0000000`00M;l0oooo0?ooo`0:0?ooo`030;md00000000
000000@0000000T0B9cO0?ooo`3oool0oooo0?oOW018000000000000M02oool0D03oool008h0oooo
00<0gia80000M02oool00P3oool09@2oM00004RL0=ooo`3oool0oooO09a80000M;l0oooo0?noM000
0000M;oo0?ooo`3ogi`0B000001d_`3o_kl0_ooo0=nLB00007@0_ooo0=no_`3Oool0_g@000000000
B9`0gooo0?oog`2LB7@0_ooo0?ooo`3OW4P0001d0;ooo`3oool0oooO09a80000M;l00P3oool02`3O
W4P0001d0;ooo`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo0080oooo00<0_gA809cOo`3o
ool0303oool03`2oM00000000018W03Oool0oooO09a8M02oool0oooo0=nLB0000000B9cO0?ooo`3o
oml0W4P004RLg`020?ooo`0V0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0oooO09a80000
0000M;oo0?ooo`3ookl0M0000000000007@0_ooo0=nLB00007@0_ooo0?ooo`3OW4P0001d0;ooo`3O
_kl0gooo0=nLB00007@0_ooo0?ooo`3OW4P0000004RLg`3oool0oooO09a80018W=l20?ooo`070=nL
B000B9`0gooo0?ooo`3ooml0W4P0001d_`080?ooo`030?oog`2LB00007Bo00<0oooo00L0gia80000
0018W=l0oooo0?oog`2LB000B9cO0080oooo00<0gia80000M02oool01`3oool00`3OW4P0001d0;oo
o`020?ooo`070;md0000B9`0gooo0?ooo`3ooml0W4P0001d_`0>0?ooo`030=nLB0000000000000<0
000000X004RL0=ooo`3oool0oono07@0001d_ol0oooo0?noM00007@0_oooC`3oool008h0oooo00<0
gia80000M02oool00P3oool03`2oM00004RL0=ooo`3oool0oooO09a80000M;l0oooo0?noM00004P0
W=oo0?ooo`3ooml0W4P0001d_`020?ooo`070=nLB000B9`0gooo0?ooo`3ogi`0B000001d_`030?oo
o`0:0?oOW02L_ol0oooo0=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`80oooo00/0gia80000M02o
ool0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`050?oo_`1d0000000000000000
M;l02@3oool00`3ogi`0B000001d_`030?ooo`050?oOW02L_ol0omnL04P00000M;l00`3oool03`3o
_g@0001d0;ooo`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3ogi`0B00004RLg`02
0?ooo`030?oog`2LB00007Bo0080oooo00L0gia80000M02oool0oooo0=nLB000B9`0gooo0080oooo
00H0gia80000M02oool0omnL04P00000M;l30?ooo`0;0?noM00007@0_ooo0?ooo`3OW4P004RL0=oo
o`3oool0oooO09a80000M;l0203oool00`3ooml0W4P0001d_`020?ooo`030?oOW018000007Bo00<0
oooo00L0okmd0000M02oool0oooo0=nLB00007@0_ooo00L0oooo00<0gia80000M02oool00P3oool0
2P2oM00004RL0=ooo`3oool0oooO09a80000M;l0oooo0?oog`2LB0080000000604RLg`3oool0oooo
0?ooo`3OW4P0B9cO0`3oool01P3OW7@0W=oo0?ooo`2oM00004RL0=ooo`80oooo00<0_g@0001d_`3o
ool0CP3oool008h0oooo00<0gia80000M02oool00P3oool01`2oM00004RL0=ooo`3oool0oooO09a8
0000M;l00P3oool01P3o_g@00000001d_`3ooml0W4P0001d_`80oooo00L0gia80018W03Oool0oooo
0?noM00004P0W=oo00H0oooo00L0gia80000M02oool0oooo0?oog`2LB00007Bo0080oooo00/0gia8
0000M02oool0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`060=nLB00007@0_ooo
0?oo_`1d0000B9cO203oool00`3o_g@0001809cOo`050?ooo`030?noM0000000M;oo00<0oooo00l0
okmd0000001d_ol0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd0000001d_ol0
0P3oool00`3ooml0W4P0001d_`020?ooo`070=nLB00007@0_ooo0?ooo`3OW4P004RL0=ooo`020?oo
o`060=nLB00007@0_ooo0?noM0000000M;oo0`3oool02`3o_g@0000007Boo`3oool0gia80018W03O
ool0oooo0?oog`2LB00007Bo00P0oooo00<0oooO09a80000M;l00P3oool00`3o_g@0000007Boo`03
0?ooo`070?noM0000000M;oo0?ooo`3OW4P0001d0;ooo`070?ooo`030=nLB00007@0_ooo0080oooo
00L0_g@00018W03Oool0oooo0?oog`2LB00007Bo0100oooo00<0gia807Boo`3oool00`3oool00`3o
okl0M00004RLg`030?ooo`030=nLB00007@0_ooo04h0oooo002>0?ooo`030=nLB00007@0_ooo0080
oooo00L0_g@00018W03Oool0oooo0?oog`2LB00007Bo00@0oooo00@0okmd00000000000007Bo0P3o
ool01`3OW4P004RL0=ooo`3oool0okmd0000M02oool01P3oool01`3OW4P0001d0;ooo`3oool0oooO
09a80000M;l00P3oool04`3OW4P0001d0;ooo`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo
0?ooo`3ookl0M00004RLg`3oool0oooO09a80000M;l0203oool00`3o_g@0001d0;ooo`050?ooo`03
0?noM0000000M;oo00<0oooo00l0okmd0000001d_ol0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0
001d_`3oool0okmd0000001d_ol00P3oool00`3ooml0W4P0001d_`020?ooo`070=nLB00007@0_ooo
0?ooo`3OW4P004RL0=ooo`020?ooo`060=nLB00007@0_ooo0?noM0000000M;oo0`3oool02`3o_g@0
000007Boo`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo00P0oooo00<0oooO09a80000M;l0
0P3oool00`3o_g@0000007Boo`030?ooo`070?noM0000000M;oo0?ooo`3OW4P0001d0;ooo`070?oo
o`030=nLB00007@0_ooo0080oooo00L0_g@00018W03Oool0oooo0?oog`2LB00007Bo0140oooo00<0
_gA809cOo`3oool00P3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo04h0oooo002>
0?ooo`030=nLB00004P0W=oo0080oooo00l0_g@00000M02oool0oooo0?oog`2LB00007Bo0?ooo`3o
okl0M00007Boo`3oool0oooO09a80000M;l00P3oool01`3OW4P004RL0=ooo`3oool0omnL04P0B02L
gol00P3oool02`3o_g@0001809cOo`3oool0gia80000M02oool0oooo0?oog`2LB00007Bo0080oooo
01<0gia80000M02oool0oooo0;md000007@0_ooo0?ooo`3ooml0W4P0001d_`3oool0oono07@00018
W=l0oooo0?oog`2LB000B9cO00P0oooo00<0omnL04P0B02Lgol00P3oool01P3o_g@0001809cOo`3o
okl0M00007Boo`<0oooo00l0_g@00000B02Lgol0oooo0;md000007@0_ooo0?ooo`3ooml0W4P0001d
_`3oool0oono07@00018W=l00P3oool00`3ooml0W4P0001d_`020?ooo`070=nLB00007@0_ooo0?oo
o`3OW4P004RL0=ooo`020?ooo`060=nLB00007@0_ooo0?oo_`1d0000M;oo0`3oool02`2oM0000018
09cOo`3oool0_g@00000M02oool0oooo0?oog`2LB00007Bo00P0oooo00<0oooO09a80000M;l00P3o
ool00`3ookl0M00007Boo`030?ooo`070;md000004P0W=oo0?ooo`3OW4P0001809cOo`070?ooo`03
0=nLB00004P0W=oo0080oooo00L0_g@00000M02oool0oooo0?oog`2LB00007Bo0140oooo00<0oooO
09a8M02oool00P3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo04h0oooo002<0?oo
o`0F0?oog`2LB00000000000B018B4P0W=oo0?oo_`1d000007Ad07@0M02oool0oono07@00018W=l0
oooo0?oo_`1d0000B9cO0?ooo`3ookl0M000001d_`80oooo00<0gia80018W03Oool00P3oool0702o
M00004RL0=ooo`3ooml0W4P00000M02oool0oooo0=nLB0000000001809cOo`3ookl0M00004RLg`3o
oml0W4P00000000007@0_ooO09a80000000004Q804P0B02Lgol0oono07@00018W=l20?ooo`040;md
0000M;l0oooo0?noM080000000<004RL0=ooo`3oool01P3oool0602oM00004RL0=ooo`3ooml0W4P0
0000M02oool0oooo0=nLB000B9`0gooo0?ooo`3o_g@00000001d_`3ooml0W4P000000000B4P0B018
09cOo`3ookl0M00004RLg`80oooo02H0okmd0000B02Lgol0oooo0?noM000000007Bo0?oog`2LB000
00000000M02oool0oooo0=nLB000B9`0gooo0?oog`2LB00000000000M02oool0oooo0=nLB000B9`0
gooo0?ooo`3o_g@00000001d_`3ooml0W4P000000000B4P0B01809cOo`3ookl0M00004RLg`P0oooo
00<0oooO09a80000M;l00`3oool03`3OW4P004RL0=ooo`3oool0okmd00000000M;l0oooO09a80000
0000001804QdW03O_g@0001d0;ooo`020?ooo`0A0?oog`2LB00000000000B018B4P0W=oo0?oo_`1d
000007Ad07@0M02oool0oono07@00018W=l0oooo0?oog`2LB000200000000`18W=l0oooo0?ooo`04
0?ooo`070?noM000M;l0oooo0?ooo`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo04h0oooo
002>0?ooo`0<0;md000007@0_ooO09a800000000001809cOo`3ookl0M0000000000007@0_ooo0P3o
ool0203ookl0M000000000000000001d0;ooo`3oool0_g@00`0000001000B9`0gooo0?ooo`3OW4P2
0000000;0000B02Lgol0oooo0?ooo`3OW4P0001d0;oog`2LB00000000000M02oool00P3oool02`2o
M000001d0;ooo`3oool0_g@00018W03Ooml0W4P00000000007@0_ooo0080oooo00D0oooO09a80000
0000000007Boo`0;0?ooo`050=nLB000000000000000B02Lgol00`3oool01@3o_g@0000000000000
0000B9cO00<0oooo00L0_g@00018W03Ooml0W4P00000000007@0_ooo00<0oooo00<0okmd00000000
00000P0000001`00M;l0oooo0?ooo`2oM000001d0;ooo`2oM0000`0000001@00B9`0gooo0;md0000
07@0_ooo0080oooo00D0okmd000000000000000004RLg`030?ooo`070;md0000B9`0gooO09a80000
0000001d0;ooo`070?ooo`030?noM000000000000080000000@004RL0=ooo`3oool0okmd0`000000
2P18W=l0oooo0?ooo`3oool0_g@00000M02ogi`0B0000000M02oool40?ooo`0<0;md000007@0_ooO
09a800000000001809cOo`3ookl0M0000000000007@0_ooo4P3oool01`3ookl0M01809cOo`3oool0
omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`1>0?ooo`00Y03oool00`2oM00004RL0=ooo`0:
0?ooo`030=nLB00007@0_ooo0400oooo00<0oooO09a80000M;l01P3oool00`2oM00004RL0=ooo`0N
0?ooo`030?oog`2LB00007Bo03@0oooo00L0oono07@00018W=l0oooo0?oo_`1d0000B9cO00<0oooo
00<0gia80000M02oool0CP3oool00:@0oooo00<0oono07A8W03Oool02P3oool00`3OW4P0001d0;oo
o`100?ooo`030?oog`2LB00007Bo00H0oooo00<0oono07A8W03Oool07`3oool00`2oM00007Bo0?oo
o`0^0?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l00P3oool00`3OW4P004RL0=oo
o`020?ooo`030;md0000M;l0oooo04h0oooo002T0?ooo`030?oog`2LM9`0gooo00X0oooo00<0gia8
0000M02oool0@03oool00`3ooml0W4P0001d_`060?ooo`030?oog`2LM9`0gooo01l0oooo00<0gia8
001d_`3oool0;P3oool0203ooml0W4Q809cOo`3oool0oono07@0000007@0_ooo0P3oool01`3ookl0
M00007Boo`3oool0okmd0000M02oool0C`3oool00:l0oooo00D0oooO09a800000000001d0;ooo`06
0?ooo`030=nLB00004P0W=oo03H0oooo00@0oono07@00000000007Bo0P3oool00`3OW4P0001809cO
o`060?ooo`030=nLB00004P0W=oo01X0oooo00L0oono07@0M02oool0oooO09a80000B9`0gooo02/0
oooo00H0oooO09a80000000000000000M02oool40?ooo`050?oo_`1d000000000000M02oool0D03o
ool00;40oooo00<0_g@00000M02oool01P3oool00`3OW4P0001809cOo`0g0?ooo`030?oog`2LB000
07Bo0080oooo00<0gia80000B02Lgol01P3oool00`3OW4P0001809cOo`0K0?ooo`050?oo_`1d0000
00000000B02Lgol0R`3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo
/@3oool00001\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {2.46954, -77.0979, \
0.00202556, 0.799197}}]
}, Closed]],

Cell["\<\
One can zoom in upon the glitch, but it is difficult to find because the even \
and odd solutions are so close together.  \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
        mismatch[e, 20, pot, \(-3\), 3, 0, MaxSteps \[Rule] 5000], {e, 
          2.88715, 2.8872}, PlotRange \[Rule] {All, {\(-50\), 50}}, 
        PlotPoints \[Rule] 500, Frame \[Rule] True, 
        FrameLabel \[Rule] {"\<energy\>", "\<mismatch\>"}, 
        PlotLabel \[Rule] "\<matching condition for m=20\>"];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-54993.3 19047.6 0.309017 0.00618034 [
[.02381 -0.0125 -21.5 -15.6875 ]
[.02381 -0.0125 21.5 0 ]
[.21429 -0.0125 -21.5 -15.6875 ]
[.21429 -0.0125 21.5 0 ]
[.40476 -0.0125 -21.5 -15.6875 ]
[.40476 -0.0125 21.5 0 ]
[.59524 -0.0125 -21.5 -15.6875 ]
[.59524 -0.0125 21.5 0 ]
[.78571 -0.0125 -21.5 -15.6875 ]
[.78571 -0.0125 21.5 0 ]
[.97619 -0.0125 -18.5 -15.6875 ]
[.97619 -0.0125 18.5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -17.9688 -31.375 ]
[.5 -0.0125 17.9688 -15.6875 ]
[-0.0125 .0618 -24.3125 -7.84375 ]
[-0.0125 .0618 0 7.84375 ]
[-0.0125 .18541 -24.3125 -7.84375 ]
[-0.0125 .18541 0 7.84375 ]
[-0.0125 .30902 -10 -7.84375 ]
[-0.0125 .30902 0 7.84375 ]
[-0.0125 .43262 -16 -7.84375 ]
[-0.0125 .43262 0 7.84375 ]
[-0.0125 .55623 -16 -7.84375 ]
[-0.0125 .55623 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -40 -25.2813 ]
[-0.01875 .30902 -24.3125 25.2813 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -72.8125 0 ]
[.5 .63053 72.8125 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
gsave
.02381 -0.0125 -82.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.88715) show
1.000 setlinewidth
grestore
.21429 0 m
.21429 .00625 L
s
gsave
.21429 -0.0125 -82.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.88716) show
1.000 setlinewidth
grestore
.40476 0 m
.40476 .00625 L
s
gsave
.40476 -0.0125 -82.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.88717) show
1.000 setlinewidth
grestore
.59524 0 m
.59524 .00625 L
s
gsave
.59524 -0.0125 -82.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.88718) show
1.000 setlinewidth
grestore
.78571 0 m
.78571 .00625 L
s
gsave
.78571 -0.0125 -82.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.88719) show
1.000 setlinewidth
grestore
.97619 0 m
.97619 .00625 L
s
gsave
.97619 -0.0125 -79.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.8872) show
1.000 setlinewidth
grestore
.125 Mabswid
.0619 0 m
.0619 .00375 L
s
.1 0 m
.1 .00375 L
s
.1381 0 m
.1381 .00375 L
s
.17619 0 m
.17619 .00375 L
s
.25238 0 m
.25238 .00375 L
s
.29048 0 m
.29048 .00375 L
s
.32857 0 m
.32857 .00375 L
s
.36667 0 m
.36667 .00375 L
s
.44286 0 m
.44286 .00375 L
s
.48095 0 m
.48095 .00375 L
s
.51905 0 m
.51905 .00375 L
s
.55714 0 m
.55714 .00375 L
s
.63333 0 m
.63333 .00375 L
s
.67143 0 m
.67143 .00375 L
s
.70952 0 m
.70952 .00375 L
s
.74762 0 m
.74762 .00375 L
s
.82381 0 m
.82381 .00375 L
s
.8619 0 m
.8619 .00375 L
s
.9 0 m
.9 .00375 L
s
.9381 0 m
.9381 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -78.9688 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(energy) show
94.938 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .0618 m
.00625 .0618 L
s
gsave
-0.0125 .0618 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
0 .18541 m
.00625 .18541 L
s
gsave
-0.0125 .18541 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
0 .30902 m
.00625 .30902 L
s
gsave
-0.0125 .30902 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
0 .43262 m
.00625 .43262 L
s
gsave
-0.0125 .43262 -77 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
0 .55623 m
.00625 .55623 L
s
gsave
-0.0125 .55623 -77 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .09271 m
.00375 .09271 L
s
0 .12361 m
.00375 .12361 L
s
0 .15451 m
.00375 .15451 L
s
0 .21631 m
.00375 .21631 L
s
0 .24721 m
.00375 .24721 L
s
0 .27812 m
.00375 .27812 L
s
0 .33992 m
.00375 .33992 L
s
0 .37082 m
.00375 .37082 L
s
0 .40172 m
.00375 .40172 L
s
0 .46353 m
.00375 .46353 L
s
0 .49443 m
.00375 .49443 L
s
0 .52533 m
.00375 .52533 L
s
0 .0309 m
.00375 .0309 L
s
0 .58713 m
.00375 .58713 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -118.438 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
86.2813 11.8438 translate
90 rotate
-86.2813 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(mismatch) show
109.563 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .61178 m
.02381 .61803 L
s
.21429 .61178 m
.21429 .61803 L
s
.40476 .61178 m
.40476 .61803 L
s
.59524 .61178 m
.59524 .61803 L
s
.78571 .61178 m
.78571 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.0619 .61428 m
.0619 .61803 L
s
.1 .61428 m
.1 .61803 L
s
.1381 .61428 m
.1381 .61803 L
s
.17619 .61428 m
.17619 .61803 L
s
.25238 .61428 m
.25238 .61803 L
s
.29048 .61428 m
.29048 .61803 L
s
.32857 .61428 m
.32857 .61803 L
s
.36667 .61428 m
.36667 .61803 L
s
.44286 .61428 m
.44286 .61803 L
s
.48095 .61428 m
.48095 .61803 L
s
.51905 .61428 m
.51905 .61803 L
s
.55714 .61428 m
.55714 .61803 L
s
.63333 .61428 m
.63333 .61803 L
s
.67143 .61428 m
.67143 .61803 L
s
.70952 .61428 m
.70952 .61803 L
s
.74762 .61428 m
.74762 .61803 L
s
.82381 .61428 m
.82381 .61803 L
s
.8619 .61428 m
.8619 .61803 L
s
.9 .61428 m
.9 .61803 L
s
.9381 .61428 m
.9381 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .0618 m
1 .0618 L
s
.99375 .18541 m
1 .18541 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .43262 m
1 .43262 L
s
.99375 .55623 m
1 .55623 L
s
.125 Mabswid
.99625 .09271 m
1 .09271 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .15451 m
1 .15451 L
s
.99625 .21631 m
1 .21631 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .27812 m
1 .27812 L
s
.99625 .33992 m
1 .33992 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .40172 m
1 .40172 L
s
.99625 .46353 m
1 .46353 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .52533 m
1 .52533 L
s
.99625 .0309 m
1 .0309 L
s
.99625 .58713 m
1 .58713 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .30902 m
1 .30902 L
s
gsave
.5 .63053 -133.813 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(matching) show
110.375 15.813 moveto
(condition) show
158.563 15.813 moveto
(for) show
175.438 15.813 moveto
(m) show
184.813 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
192.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
204.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .41819 m
.02567 .41817 L
.02769 .41815 L
.0296 .41812 L
.03143 .4181 L
.03338 .41808 L
.03525 .41805 L
.03724 .41803 L
.03917 .418 L
.04101 .41798 L
.04298 .41795 L
.04487 .41793 L
.04669 .4179 L
.04863 .41788 L
.05049 .41785 L
.05248 .41782 L
.05439 .4178 L
.05622 .41777 L
.05818 .41774 L
.06006 .41771 L
.06206 .41768 L
.06399 .41766 L
.06584 .41763 L
.06781 .4176 L
.06971 .41757 L
.07154 .41754 L
.07348 .41751 L
.07535 .41747 L
.07734 .41744 L
.07926 .41741 L
.0811 .41738 L
.08306 .41734 L
.08495 .41731 L
.08677 .41728 L
.0887 .41724 L
.09056 .41721 L
.09254 .41717 L
.09445 .41713 L
.09628 .4171 L
.09823 .41706 L
.10011 .41702 L
.10211 .41698 L
.10403 .41694 L
.10588 .4169 L
.10785 .41686 L
.10974 .41682 L
.11156 .41678 L
.1135 .41674 L
.11537 .4167 L
.11736 .41665 L
Mistroke
.11927 .41661 L
.12111 .41656 L
.12307 .41652 L
.12495 .41647 L
.12695 .41642 L
.12888 .41637 L
.13074 .41632 L
.13271 .41627 L
.13462 .41622 L
.13644 .41617 L
.13839 .41612 L
.14026 .41606 L
.14226 .416 L
.14418 .41595 L
.14602 .41589 L
.14799 .41583 L
.14988 .41577 L
.15169 .41572 L
.15363 .41565 L
.15549 .41559 L
.15748 .41553 L
.15938 .41546 L
.16122 .4154 L
.16317 .41533 L
.16505 .41526 L
.16705 .41518 L
.16898 .41511 L
.17083 .41504 L
.1728 .41496 L
.1747 .41488 L
.17652 .41481 L
.17846 .41472 L
.18033 .41464 L
.18232 .41456 L
.18424 .41447 L
.18608 .41438 L
.18804 .41429 L
.18993 .4142 L
.19174 .41411 L
.19367 .41401 L
.19553 .41391 L
.19751 .4138 L
.19941 .4137 L
.20124 .4136 L
.20319 .41348 L
.20507 .41337 L
.20706 .41325 L
.20899 .41313 L
.21083 .41301 L
.2128 .41288 L
Mistroke
.21469 .41275 L
.21651 .41262 L
.21845 .41248 L
.22032 .41234 L
.2223 .41219 L
.22421 .41204 L
.22605 .41189 L
.22801 .41172 L
.22989 .41156 L
.23189 .41138 L
.23382 .4112 L
.23567 .41102 L
.23765 .41082 L
.23955 .41063 L
.24137 .41043 L
.24332 .41022 L
.24519 .41 L
.24718 .40977 L
.2491 .40953 L
.25094 .40929 L
.25291 .40903 L
.2548 .40877 L
.25661 .4085 L
.25855 .40821 L
.26041 .40792 L
.26239 .40759 L
.2643 .40726 L
.26613 .40693 L
.26808 .40656 L
.26996 .40619 L
.27196 .40578 L
.27388 .40536 L
.27573 .40493 L
.2777 .40445 L
.2796 .40396 L
.28142 .40347 L
.28336 .40291 L
.28523 .40234 L
.28722 .4017 L
.28913 .40104 L
.29097 .40036 L
.29293 .39959 L
.29481 .3988 L
.29682 .39789 L
.29875 .39694 L
.3006 .39595 L
.30258 .39482 L
.30448 .39363 L
.30631 .39238 L
.30826 .39092 L
Mistroke
.31014 .38938 L
.31213 .38756 L
.31405 .38561 L
.31589 .38351 L
.31786 .38099 L
.31975 .37823 L
.32157 .37521 L
.32351 .37149 L
.32537 .36731 L
.32735 .36203 L
.32926 .35588 L
.3311 .34865 L
.33305 .33898 L
.33494 .32692 L
.3359 .3193 L
.33694 .30956 L
.33886 .28555 L
.33982 .26934 L
.34083 .24741 L
.34178 .22042 L
.34266 .18641 L
.34316 .16135 L
.34364 .13214 L
.34406 .09999 L
.34453 .05504 L
Mfstroke
.34453 .05504 m
.34496 0 L
s
.34794 0 m
.34794 .61803 L
s
.35419 .61803 m
.35496 .59585 L
.35593 .57495 L
.35777 .54647 L
.35872 .53557 L
.35973 .52588 L
.36155 .51209 L
.36348 .50093 L
.36534 .49253 L
.36731 .48536 L
.36922 .4797 L
.37105 .47514 L
.373 .47101 L
.37487 .46761 L
.37687 .46447 L
.37879 .46183 L
.38063 .45958 L
.3826 .45745 L
.38449 .45562 L
.38631 .45403 L
.38825 .45249 L
.39012 .45114 L
.3921 .44983 L
.39401 .44868 L
.39585 .44766 L
.3978 .44665 L
.39968 .44576 L
.40168 .44487 L
.40361 .44408 L
.40547 .44337 L
.40744 .44266 L
.40934 .44202 L
.41117 .44144 L
.41311 .44086 L
.41498 .44034 L
.41697 .43981 L
.41889 .43933 L
.42073 .43889 L
.42269 .43845 L
.42458 .43804 L
.4264 .43767 L
.42833 .43729 L
.43019 .43695 L
.43217 .4366 L
.43408 .43627 L
.43591 .43598 L
.43786 .43567 L
.43974 .43539 L
.44174 .43511 L
.44366 .43484 L
Mistroke
.44551 .4346 L
.44748 .43435 L
.44938 .43412 L
.4512 .43391 L
.45314 .43369 L
.45501 .43348 L
.45699 .43327 L
.45891 .43308 L
.46074 .4329 L
.4627 .43271 L
.46459 .43254 L
.46659 .43236 L
.46852 .4322 L
.47038 .43204 L
.47235 .43188 L
.47425 .43173 L
.47608 .43159 L
.47803 .43145 L
.4799 .43132 L
.4819 .43118 L
.48382 .43105 L
.48566 .43093 L
.48763 .4308 L
.48952 .43068 L
.49134 .43057 L
.49327 .43046 L
.49514 .43035 L
.49712 .43024 L
.49903 .43014 L
.50086 .43004 L
.50282 .42994 L
.5047 .42984 L
.5067 .42974 L
.50862 .42965 L
.51048 .42956 L
.51245 .42947 L
.51434 .42939 L
.51617 .42931 L
.51811 .42922 L
.51998 .42914 L
.52197 .42906 L
.52389 .42899 L
.52573 .42891 L
.52769 .42884 L
.52958 .42877 L
.53139 .4287 L
.53332 .42863 L
.53518 .42856 L
.53716 .42849 L
.53906 .42843 L
Mistroke
.5409 .42837 L
.54285 .42831 L
.54472 .42824 L
.54672 .42818 L
.54864 .42812 L
.55049 .42807 L
.55245 .42801 L
.55435 .42795 L
.55617 .4279 L
.55811 .42785 L
.55997 .4278 L
.56196 .42774 L
.56387 .42769 L
.56571 .42764 L
.56766 .42759 L
.56955 .42754 L
.57155 .42749 L
.57348 .42745 L
.57533 .4274 L
.57731 .42736 L
.57921 .42731 L
.58103 .42727 L
.58298 .42723 L
.58485 .42719 L
.58684 .42714 L
.58876 .4271 L
.59061 .42706 L
.59257 .42702 L
.59446 .42698 L
.59628 .42695 L
.59821 .42691 L
.60007 .42687 L
.60205 .42683 L
.60396 .4268 L
.60579 .42676 L
.60775 .42673 L
.60963 .42669 L
.61162 .42666 L
.61355 .42662 L
.6154 .42659 L
.61737 .42656 L
.61926 .42653 L
.62109 .4265 L
.62303 .42646 L
.6249 .42643 L
.62688 .4264 L
.6288 .42637 L
.63064 .42634 L
.6326 .42631 L
.63448 .42629 L
Mistroke
.63649 .42626 L
.63842 .42623 L
.64028 .4262 L
.64225 .42617 L
.64415 .42615 L
.64598 .42612 L
.64793 .42609 L
.64981 .42607 L
.6518 .42604 L
.65372 .42602 L
.65557 .42599 L
.65753 .42597 L
.65943 .42594 L
.66125 .42592 L
.66318 .4259 L
.66505 .42587 L
.66703 .42585 L
.66894 .42582 L
.67078 .4258 L
.67273 .42578 L
.67461 .42576 L
.67661 .42574 L
.67854 .42571 L
.68039 .42569 L
.68237 .42567 L
.68426 .42565 L
.68609 .42563 L
.68803 .42561 L
.68991 .42559 L
.69189 .42557 L
.69381 .42555 L
.69565 .42553 L
.69762 .42551 L
.6995 .42549 L
.70132 .42547 L
.70325 .42545 L
.70511 .42544 L
.70709 .42542 L
.709 .4254 L
.71083 .42538 L
.71278 .42536 L
.71466 .42535 L
.71665 .42533 L
.71858 .42531 L
.72043 .4253 L
.72239 .42528 L
.72429 .42526 L
.72611 .42525 L
.72805 .42523 L
.72992 .42521 L
Mistroke
.7319 .4252 L
.73382 .42518 L
.73565 .42517 L
.73761 .42515 L
.7395 .42513 L
.7415 .42512 L
.74343 .4251 L
.74528 .42509 L
.74726 .42507 L
.74916 .42506 L
.75099 .42504 L
.75294 .42503 L
.75481 .42502 L
.7568 .425 L
.75872 .42499 L
.76056 .42497 L
.76253 .42496 L
.76442 .42495 L
.76624 .42493 L
.76817 .42492 L
.77004 .42491 L
.77202 .42489 L
.77393 .42488 L
.77576 .42487 L
.77771 .42485 L
.77959 .42484 L
.78159 .42483 L
.78352 .42482 L
.78537 .4248 L
.78734 .42479 L
.78924 .42478 L
.79106 .42477 L
.793 .42476 L
.79487 .42474 L
.79686 .42473 L
.79878 .42472 L
.80062 .42471 L
.80258 .4247 L
.80446 .42469 L
.80628 .42468 L
.80821 .42466 L
.81007 .42465 L
.81205 .42464 L
.81395 .42463 L
.81578 .42462 L
.81773 .42461 L
.81961 .4246 L
.8216 .42459 L
.82352 .42458 L
.82537 .42457 L
Mistroke
.82734 .42456 L
.82923 .42455 L
.83105 .42454 L
.83299 .42453 L
.83485 .42452 L
.83684 .42451 L
.83875 .4245 L
.84059 .42449 L
.84254 .42448 L
.84442 .42447 L
.84642 .42446 L
.84835 .42445 L
.85021 .42444 L
.85218 .42443 L
.85408 .42442 L
.85591 .42441 L
.85785 .4244 L
.85972 .4244 L
.86171 .42439 L
.86363 .42438 L
.86548 .42437 L
.86744 .42436 L
.86933 .42435 L
.87114 .42434 L
.87308 .42433 L
.87494 .42433 L
.87692 .42432 L
.87882 .42431 L
.88066 .4243 L
.88261 .42429 L
.88449 .42428 L
.88649 .42428 L
.88841 .42427 L
.89026 .42426 L
.89223 .42425 L
.89412 .42424 L
.89595 .42424 L
.89789 .42423 L
.89976 .42422 L
.90174 .42421 L
.90366 .42421 L
.9055 .4242 L
.90745 .42419 L
.90934 .42418 L
.91134 .42418 L
.91327 .42417 L
.91513 .42416 L
.91711 .42415 L
.91901 .42415 L
.92084 .42414 L
Mistroke
.92278 .42413 L
.92466 .42412 L
.92665 .42412 L
.92857 .42411 L
.93042 .4241 L
.93238 .4241 L
.93427 .42409 L
.93609 .42408 L
.93803 .42408 L
.93989 .42407 L
.94188 .42406 L
.94379 .42406 L
.94562 .42405 L
.94758 .42404 L
.94946 .42404 L
.95146 .42403 L
.95338 .42402 L
.95524 .42402 L
.95721 .42401 L
.95911 .424 L
.96093 .424 L
.96287 .42399 L
.96474 .42399 L
.96673 .42398 L
.96865 .42397 L
.97049 .42397 L
.97245 .42396 L
.97434 .42396 L
.97615 .42395 L
.97619 .42395 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{327.625, 202.5},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007[0001;b000`400?l00000o`00003oo`3oooo/
0?ooo`00o`3oooo/0?ooo`00o`3oooo/0?ooo`00o`3ooolE0?ooo`030?oog`2LB000000000<00000
00<0B9cO0?ooo`3OW4P00P0000000`1d_ol0oooo0?ooo`390?ooo`00o`3ooolE0?ooo`030;md0000
B9`0gooo00<0oooo00<0gibL0;ooo`3oool00P3oool00`2oM7@0_ooo0?ooo`390?ooo`00o`3ooolE
0?ooo`030?oog`2LM9`0gooo00<0oooo00<0oono09bLg`3oool00P3oool00`3ogi`0M9cO0?ooo`39
0?ooo`00o`3ooolF0?ooo`030;md00000000000000<0000000L0B9cO0?ooo`3oool0oooo0?oog`2L
B7@0_ooo0<T0oooo003g0?ooo`070;md0000000000000000M02oool0oooo0;md0002000000030000
M02ooml0W4P00080000000@004RL0=ooo`3oool0_g@00P000000100007@0_ooo0?ooo`2oM0020000
00050000B02Lgol0oooo0?oo_`1d0000100000000`0007@0_ooo0?ooo`030?ooo`030;md001d_ol0
oooo0<P0oooo003f0?ooo`0?0;md0000000007Bo0?ooo`3ogi`0B01d0;ooo`3oool0gia80018W03O
ool0oooo0?oog`2LB00007Bo0080oooo00/0_g@000000000M;l0oooo0?oOW01807@0_ooo0?ooo`3O
W4P0001d0;ooo`030?ooo`030;mdB02Lgol0oooo00L0oooo00@0omnL04P00000B9`0gooob03oool0
0?D0oooo00<0omnL04P00018W=l00`3oool03P3ogi`0M9cO0?ooo`3OW4P004RL0=ooo`3oool0oooO
09a80000M;l0oooo0?oOW0180000B9cO0`3oool01P3ogi`0M9cO0?ooo`3OW4P0001d0;ooo`<0oooo
00D0oono07@0000000000000001d_`050?ooo`040;md0000B9`0W7@007BoolP0oooo003e0?ooo`03
0?noM00004P0W=oo00H0oooo00/0gia80018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3o_g@00018
09cOo`060?ooo`030=nLB00007@0_ooo00<0oooo00H0gia80000M02oool0oono07@00018W=l30?oo
o`050?noM0000000M;oo0?oOW018M;l0b03oool00?D0oooo00<0okmd0000M02oool01P3oool02`3O
W4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?noM00007@0_ooo00H0oooo00<0gia80000M02o
ool00P3oool01`3ookl0M00004RLg`3oool0oooO09a80000M;l00P3oool01`3ooml0W4P00018W03O
ool0oooo0;mdB02Lgol0a`3oool00?D0oooo00<0omnL04P000000000100000003018W=l0oooo0;md
000007@0_ooo0?ooo`3ooml0W4P0001d_`3oool0omnL04P000D0000000D0B9cO0?ooo`3OW4P00018
09cOo`020?ooo`070?oo_`1d0000B9cO0?ooo`3ooml0W4P004RLg`020?ooo`030?noM00004P0W=oo
0080oooo00<0okmd04RLg`3oool0aP3oool00?H0oooo00l0_g@004RLg`3oool0oooo0=nLB00004P0
W=oO09a80000000004Q804P0B02Lgol0oono07@00018W=l00P3oool04P2oM000B9cO0?ooo`3oool0
gia80000B02Lgml0W4P00000000004P0B7BL0=noM00007@0_ooo0;md0000M;l0oooo0?noM0800000
00D004RL0=oo_`1d000004RL0=ooo`020?ooo`030?oo_`1d07@0_ooo0<H0oooo003g0?ooo`050=nL
B000000000000000B02Lgol00P3oool01`2oM00004RL0=oog`2LB00000000000M02oool00`3oool0
1@3OW4P000000000000004P0W=oo0080oooo00P0_g@00000M02ogi`0B0000000M02oool0oooO09a8
0080000000@0M;oo0?ooo`3ooml0W4P00P0000001`00B9`0gooo0?ooo`2oM00000000018W03Oool0
a@3oool00?l0ooook03oool00?l0ooook03oool00?l0ooook03oool00?l0ooook03oool00?l0oooo
k03oool00?l0ooook03oool00?l0ooook03oool00?l0ooook03oool00?l0ooook03oool00?l0oooo
k03oool00?l0ooook03oool00?l0ooook03oool00?l0ooook03oool00?l0ooook03oool00?l0oooo
k03oool004<0oooo00<0_g@0000000000000100000001P18W=l0oooo0?ooo`3o_g@0000007Boo`<0
oooo00D0gia8000000000000000004RLg`040?ooo`050=nLB0000000000000000018W=l01@3oool0
0`3OW4P0B9cO0?ooo`050?ooo`090;md00000000000000000000B9`0gooo0?ooo`3ooml0W4P000<0
000000<0M;oo0?ooo`3oool02`3oool00`2oM0000000000000040000000604RLg`3oool0oooo0?no
M0000000M;oo0`3oool01@3OW4P00000000000000000B9cO00@0oooo00D0gia80000000000000000
04RLg`050?ooo`030=nLB018W=l0oooo00D0oooo00H0_g@000000000000000000018W03Oool30?oo
o`050?oOW018000000000000001d_ol0303oool00`2oM0000000000000040000000604RLg`3oool0
oooo0?noM0000000M;oo0`3oool01@3OW4P00000000000000000B9cO00@0oooo00D0gia800000000
0000000004RLg`050?ooo`030=nLB018W=l0oooo00D0oooo00H0_g@000000000000000000018W03O
ool40?ooo`030=nLB018W=l0oooo00d0oooo00<0_g@0000000000000100000001P18W=l0oooo0?oo
o`3o_g@0000007Boo`<0oooo00D0gia8000000000000000004RLg`040?ooo`050=nLB00000000000
00000018W=l01@3oool00`3OW4P0B9cO0?ooo`050?ooo`060;md00000000000000000000B9`0gooo
0`3oool01@3OW4P00000000000000000B9cO00/0oooo00<0_g@0000000000000100000001P18W=l0
oooo0?ooo`3o_g@0000007Boo`<0oooo00D0gia8000000000000000004RLg`040?ooo`050=nLB000
0000000000000018W=l01@3oool00`3OW4P0B9cO0?ooo`050?ooo`0<0;md00000000000000000000
B9`0gooo0?ooo`3ookl0M00000000000B9`0gooo4`3oool00`2oM0000000000000040000000604RL
g`3oool0oooo0?noM0000000M;oo0`3oool01@3OW4P00000000000000000B9cO00@0oooo00D0gia8
000000000000000004RLg`050?ooo`030=nLB018W=l0oooo00<0oooo00<0_g@00000000000001000
00000`18W=l0oooo0?ooo`0H0?ooo`00A03oool00`3OW4P0000000000003000000060018W03Oool0
oooo0?noM0000000M;oo0P3oool01`3OW4P0000007Boo`3oool0oooO09a80018W=l00P3oool01`3O
W4P0000007Boo`3oool0oooO09a80018W=l0103oool00`3ogi`0B4RL0=ooo`060?ooo`030?oOW018
0000M;oo00<0oooo00<0oooO09a8M02oool00P3oool00`2oM000M;oo0?ooo`0<0?ooo`030=nLB000
0000000000<0000000H004RL0=ooo`3oool0okmd0000001d_ol20?ooo`070=nLB0000000M;oo0?oo
o`3ooml0W4P004RLg`020?ooo`070=nLB0000000M;oo0?ooo`3ooml0W4P004RLg`040?ooo`030?oO
W018B9`0gooo00H0oooo00<0omnL04P0001d_ol0103oool01`3ogi`0B00004RLg`3oool0oono07@0
001d_ol0303oool00`3OW4P0000000000003000000060018W03Oool0oooo0?noM0000000M;oo0P3o
ool01`3OW4P0000007Boo`3oool0oooO09a80018W=l00P3oool01`3OW4P0000007Boo`3oool0oooO
09a80018W=l0103oool00`3ogi`0B4RL0=ooo`060?ooo`030?oOW0180000M;oo00H0oooo00<0omnL
04Q8W03Oool03P3oool00`3OW4P0000000000003000000060018W03Oool0oooo0?noM0000000M;oo
0P3oool01`3OW4P0000007Boo`3oool0oooO09a80018W=l00P3oool01`3OW4P0000007Boo`3oool0
oooO09a80018W=l0103oool00`3ogi`0B4RL0=ooo`060?ooo`030?oOW0180000M;oo00@0oooo00L0
gia80000001d_ol0oooo0?oog`2LB000B9cO00/0oooo00<0gia80000000000000`0000001P00B9`0
gooo0?ooo`3o_g@0000007Boo`80oooo00L0gia80000001d_ol0oooo0?oog`2LB000B9cO0080oooo
00L0gia80000001d_ol0oooo0?oog`2LB000B9cO00@0oooo00<0omnL04Q8W03Oool01P3oool00`3o
gi`0B00007Boo`060?ooo`030=nLB0000000M;oo01<0oooo00<0gia80000000000000`0000001P00
B9`0gooo0?ooo`3o_g@0000007Boo`80oooo00L0gia80000001d_ol0oooo0?oog`2LB000B9cO0080
oooo00L0gia80000001d_ol0oooo0?oog`2LB000B9cO00@0oooo00<0omnL04Q8W03Oool0103oool0
0`3OW4P0000000000003000000030018W03Oool0oooo01P0oooo00150?ooo`030=nLB018W=l0oooo
0080oooo00<0giad09cOo`3oool0103oool00`3ooml0W4P0001d_`030?ooo`060?noM000B9`0gooo
0?oog`2LB00007Bo0`3oool00`3o_g@004RL0=ooo`030?ooo`030?oog`2LB4P0W=oo00H0oooo00<0
omnL04P0001d_ol0203oool00`3ooml0W4Qd0;ooo`0=0?ooo`030=nLB018W=l0oooo0080oooo00<0
giad09cOo`3oool0103oool00`3ooml0W4P0001d_`030?ooo`060?noM000B9`0gooo0?oog`2LB000
07Bo0`3oool00`3o_g@004RL0=ooo`030?ooo`030?oog`2LB4P0W=oo00H0oooo00<0omnL04P0001d
_ol00`3oool00`3ooml0W4P0001d_`030?ooo`030?noM000B9`0gooo00`0oooo00<0gia804RLg`3o
ool00P3oool00`3OW7@0W=oo0?ooo`040?ooo`030?oog`2LB00007Bo00<0oooo00H0okmd0018W03O
ool0oooO09a80000M;l30?ooo`030?noM000B9`0gooo00<0oooo00<0oooO09a8B02Lgol01P3oool0
0`3ogi`0B00007Boo`060?ooo`030?oog`2LB4P0W=oo00l0oooo00<0gia804RLg`3oool00P3oool0
0`3OW7@0W=oo0?ooo`040?ooo`030?oog`2LB00007Bo00<0oooo00H0okmd0018W03Oool0oooO09a8
0000M;l30?ooo`030?noM000B9`0gooo00<0oooo00<0oooO09a8B02Lgol01P3oool00`3ogi`0B000
07Boo`030?ooo`030?oog`2LB00007Bo00<0oooo00<0okmd0018W03Oool02`3oool00`3OW4P0B9cO
0?ooo`020?ooo`030=nLM02Lgol0oooo00@0oooo00<0oooO09a80000M;l00`3oool01P3o_g@004RL
0=ooo`3ooml0W4P0001d_`<0oooo00<0okmd0018W03Oool00`3oool00`3ooml0W4Q809cOo`060?oo
o`030?oOW0180000M;oo00L0oooo00<0oono07@0001d_ol04`3oool00`3OW4P0B9cO0?ooo`020?oo
o`030=nLM02Lgol0oooo00@0oooo00<0oooO09a80000M;l00`3oool01P3o_g@004RL0=ooo`3ooml0
W4P0001d_`<0oooo00<0okmd0018W03Oool00`3oool00`3ooml0W4Q809cOo`050?ooo`030=nLB018
W=l0oooo0080oooo00<0giad09cOo`3oool0603oool004H0oooo00<0gia807Boo`3oool02@3oool0
0`2oM00007Bo0?ooo`020?ooo`060;md0000B9`0gooo0?ooo`2oM00007Bo0`3oool00`2oM00004RL
0=ooo`040?ooo`030=nLB018W=l0oooo00D0oooo00<0omnL04P0001d_ol02@3oool00`2oM000M;oo
0?ooo`0=0?ooo`030=nLB01d_ol0oooo00T0oooo00<0_g@0001d_`3oool00P3oool01P2oM00004RL
0=ooo`3oool0_g@0001d_`<0oooo00<0_g@00018W03Oool0103oool00`3OW4P0B9cO0?ooo`050?oo
o`030?oOW0180000M;oo00<0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0=0?oo
o`030=nLB01d_ol0oooo00T0oooo00<0_g@0001d_`3oool00P3oool01P2oM00004RL0=ooo`3oool0
_g@0001d_`<0oooo00<0_g@00018W03Oool0103oool00`3OW4P0B9cO0?ooo`050?ooo`030?oOW018
0000M;oo00L0oooo00<0gia804RLg`3oool03`3oool00`3OW4P0M;oo0?ooo`090?ooo`030;md0000
M;l0oooo0080oooo00H0_g@00018W03Oool0oooo0;md0000M;l30?ooo`030;md0000B9`0gooo00@0
oooo00<0gia804RLg`3oool01@3oool00`3ogi`0B00007Boo`040?ooo`030;md0000M;l0oooo0080
oooo00<0_g@00018W03Oool0303oool00`3OW4P0M;oo0?ooo`090?ooo`030;md0000M;l0oooo0080
oooo00H0_g@00018W03Oool0oooo0;md0000M;l30?ooo`030;md0000B9`0gooo00@0oooo00<0gia8
04RLg`3oool01@3oool00`3ogi`0B00007Boo`080?ooo`030?oOW01804P0W=oo01<0oooo00<0gia8
07Boo`3oool02@3oool00`2oM00007Bo0?ooo`020?ooo`060;md0000B9`0gooo0?ooo`2oM00007Bo
0`3oool00`2oM00004RL0=ooo`040?ooo`030=nLB018W=l0oooo00D0oooo00<0gia807Boo`3oool0
703oool004L0oooo00<0_gA809cOo`3oool0203oool01`3ogi`0B01d0;ooo`3oool0_g@00000001d
_ol00P3oool01`3ogi`0B01d0;ooo`3oool0_g@00000001d_ol01@3oool00`3ogi`0B4RL0=ooo`05
0?ooo`030?oOW0180000M;oo00P0oooo00<0oooO09a80018W=l03`3oool00`2oM4P0W=oo0?ooo`08
0?ooo`070?oOW01807@0_ooo0?ooo`2oM000000007Boo`020?ooo`070?oOW01807@0_ooo0?ooo`2o
M000000007Boo`050?ooo`030?oOW018B9`0gooo00D0oooo00<0omnL04P0001d_ol00`3oool00`3o
gi`0B00007Boo`030?ooo`030?noM00004P0W=oo00h0oooo00<0_gA809cOo`3oool0203oool01`3o
gi`0B01d0;ooo`3oool0_g@00000001d_ol00P3oool01`3ogi`0B01d0;ooo`3oool0_g@00000001d
_ol01@3oool00`3ogi`0B4RL0=ooo`050?ooo`030?oOW0180000M;oo00L0oooo00<0omnL04Q8W03O
ool0403oool00`2oM4P0W=oo0?ooo`080?ooo`070?oOW01807@0_ooo0?ooo`2oM000000007Boo`02
0?ooo`070?oOW01807@0_ooo0?ooo`2oM000000007Boo`050?ooo`030?oOW018B9`0gooo00D0oooo
00<0omnL04P0001d_ol0103oool01`3ogi`0B01d0;ooo`3oool0_g@00000001d_ol03P3oool00`2o
M4P0W=oo0?ooo`080?ooo`070?oOW01807@0_ooo0?ooo`2oM000000007Boo`020?ooo`070?oOW018
07@0_ooo0?ooo`2oM000000007Boo`050?ooo`030?oOW018B9`0gooo00D0oooo00<0omnL04P0001d
_ol0103oool00`3ooml0W4P00000000300000003001d_`3oool0oooo0180oooo00<0_gA809cOo`3o
ool0203oool01`3ogi`0B01d0;ooo`3oool0_g@00000001d_ol00P3oool01`3ogi`0B01d0;ooo`3o
ool0_g@00000001d_ol01@3oool00`3ogi`0B4RL0=ooo`060?ooo`030;mdB02Lgol0oooo01/0oooo
00170?ooo`030?oog`2LB7@0_ooo00T0oooo00D0oono07A8B0180000000004RLg`040?ooo`050?oo
_`1dB4P0B00000000018W=l01P3oool00`3ooml0W4Q809cOo`050?ooo`030?oOW0180000M;oo00P0
oooo00<0gia80000001d_ol03`3oool00`3ooml0W4Qd0;ooo`090?ooo`050?oo_`1dB4P0B0000000
0018W=l0103oool01@3ookl0M4Q804P000000000B9cO00H0oooo00<0oooO09a8B02Lgol01@3oool0
0`3ogi`0B00007Boo`030?ooo`030?oOW0180000M;oo00<0oooo00<0_g@00000B02Lgol03P3oool0
0`3ooml0W4Qd0;ooo`090?ooo`050?oo_`1dB4P0B00000000018W=l0103oool01@3ookl0M4Q804P0
00000000B9cO00H0oooo00<0oooO09a8B02Lgol01@3oool00`3ogi`0B00007Boo`070?ooo`030?oo
g`2LB4P0W=oo0100oooo00<0oooO09a8M02oool02@3oool01@3ookl0M4Q804P000000000B9cO00@0
oooo00D0oono07A8B0180000000004RLg`060?ooo`030?oog`2LB4P0W=oo00D0oooo00<0omnL04P0
001d_ol01@3oool01@3ookl0M4Q804P000000000B9cO00l0oooo00<0oooO09a8M02oool02@3oool0
1@3ookl0M4Q804P000000000B9cO00@0oooo00D0oono07A8B0180000000004RLg`060?ooo`030?oo
g`2LB4P0W=oo00D0oooo00<0omnL04P0001d_ol0103oool0202oM000000007Boo`3oool0oooO09a8
000007@0_ooo4`3oool00`3ooml0W4Qd0;ooo`090?ooo`050?oo_`1dB4P0B00000000018W=l0103o
ool01@3ookl0M4Q804P000000000B9cO00H0oooo00<0oooO09a8B02Lgol01P3oool00`3ooml0W4Qd
0;ooo`0K0?ooo`00B03oool00`3o_g@007Bo0?ooo`080?ooo`040?oOW01800000000001d_`D0oooo
00@0omnL04P00000000007Bo203oool00`3OW4P007Bo0?ooo`040?ooo`030?oOW0180000M;oo00@0
oooo00L0gmoO0?ooo`3ookl0M0000000000007@0_ooo0100oooo00<0okmd001d_`3oool0203oool0
103ogi`0B00000000000M;l50?ooo`040?oOW01800000000001d_`P0oooo00<0gia8001d_`3oool0
103oool00`3ogi`0B00007Boo`030?ooo`080?oo_`1d0000000004RLg`3oool0okmd00000000M;l@
0?ooo`030?noM000M;l0oooo00P0oooo00@0omnL04P00000000007Bo1@3oool0103ogi`0B0000000
0000M;l80?ooo`030=nLB000M;l0oooo00@0oooo00<0omnL04P0001d_ol0203oool00`3OW4P007Bo
0?ooo`0@0?ooo`030?noM000M;l0oooo00P0oooo00@0omnL04P00000000007Bo1@3oool0103ogi`0
B00000000000M;l80?ooo`030=nLB000M;l0oooo00@0oooo00<0omnL04P0001d_ol01@3oool0103o
gi`0B00000000000M;lA0?ooo`030?noM000M;l0oooo00P0oooo00@0omnL04P00000000007Bo1@3o
ool0103ogi`0B00000000000M;l80?ooo`030=nLB000M;l0oooo00@0oooo00<0omnL04P0001d_ol0
0`3oool00`3ogi`0B00004RLg`030?ooo`030?noM00004P0W=oo01@0oooo00<0okmd001d_`3oool0
203oool0103ogi`0B00000000000M;l50?ooo`040?oOW01800000000001d_`P0oooo00<0gia8001d
_`3oool01P3oool00`3o_g@007Bo0?ooo`0J0?ooo`00B03oool00`3ookl0M01809cOo`070?ooo`06
0?oOW0180000001d0;ooo`3o_g@007Bo0`3oool01P3ogi`0B0000000M02oool0okmd001d_`L0oooo
00<0oono07@0M02oool0103oool00`3ogi`0B00007Boo`040?ooo`060=nLB0000000000000000000
04P0W=oo4@3oool00`3ookl0M01809cOo`070?ooo`060?oOW0180000001d0;ooo`3o_g@007Bo0`3o
ool01P3ogi`0B0000000M02oool0okmd001d_`L0oooo00<0oono07@0M02oool0103oool00`3ogi`0
B00007Boo`040?ooo`030;md0000B9`0_ia80080000000<004RL0=ooo`3oool03`3oool00`3ookl0
M01809cOo`070?ooo`060?oOW0180000001d0;ooo`3o_g@007Bo0`3oool01P3ogi`0B0000000M02o
ool0okmd001d_`L0oooo00<0oono07@0M02oool0103oool00`3ogi`0B00007Boo`080?ooo`030?oo
_`1d07@0_ooo0100oooo00<0oono07@0B02Lgol01`3oool01P3ogi`0B0000000M02oool0okmd001d
_`<0oooo00H0omnL04P0000007@0_ooo0?noM000M;l70?ooo`030?oo_`1d07@0_ooo00@0oooo00<0
omnL04P0001d_ol0103oool01P3ogi`0B0000000M02oool0okmd001d_a00oooo00<0oono07@0B02L
gol01`3oool01P3ogi`0B0000000M02oool0okmd001d_`<0oooo00H0omnL04P0000007@0_ooo0?no
M000M;l70?ooo`030?oo_`1d07@0_ooo00@0oooo00<0omnL04P0001d_ol00`3oool00`3ogi`0B000
07Boo`030?ooo`030?noM00004P0W=oo01@0oooo00<0oono07@0B02Lgol01`3oool01P3ogi`0B000
0000M02oool0okmd001d_`<0oooo00H0omnL04P0000007@0_ooo0?noM000M;l70?ooo`030?oo_`1d
07@0_ooo00H0oooo00<0oono07@0B02Lgol06P3oool004P0oooo00<0oono07@00018W=l01`3oool0
0`2oM000001d0;ooo`020?ooo`070;md0018W=l0oooo0?ooo`2oM000001d0;ooo`020?ooo`050;md
0018W=l0oooo0?oOg`3Oool0103oool00`2oM000M;oo0?ooo`030?ooo`030?oOW0180000M;oo00@0
oooo00@0oono07@000000000B9cO4`3oool00`3ookl0M00004RLg`070?ooo`030;md000007@0_ooo
0080oooo00L0_g@004RLg`3oool0oooo0;md000007@0_ooo0080oooo00D0_g@004RLg`3oool0omoO
0=ooo`040?ooo`030;md001d_ol0oooo00<0oooo00<0omnL04P0001d_ol0103oool00`3ogi`0B000
07Boo`0D0?ooo`030?oo_`1d0000B9cO00L0oooo00<0_g@00000M02oool00P3oool01`2oM000B9cO
0?ooo`3oool0_g@00000M02oool00P3oool01@2oM000B9cO0?ooo`3ogml0gooo00@0oooo00<0_g@0
07Boo`3oool00`3oool00`3ogi`0B00007Boo`030?ooo`030?oOg`3Oool0oooo00<0oooo00<0_g@0
07Boo`3oool03`3oool00`3ookl0M00004RLg`070?ooo`030;md000007@0_ooo0080oooo00L0_g@0
04RLg`3oool0oooo0;md000007@0_ooo0080oooo00D0_g@004RLg`3oool0omoO0=ooo`040?ooo`03
0;md001d_ol0oooo00<0oooo00<0omnL04P0001d_ol0103oool00`2oM000001d0;ooo`020?ooo`03
0;md0018W=l0oooo00h0oooo00<0oono07@00018W=l01`3oool00`2oM000001d0;ooo`020?ooo`07
0;md0018W=l0oooo0?ooo`2oM000001d0;ooo`020?ooo`050;md0018W=l0oooo0?oOg`3Oool0103o
ool00`2oM000M;oo0?ooo`030?ooo`030?oOW0180000M;oo00<0oooo00<0omnL04P0001d_ol00`3o
ool00`3OW4P0001d0;ooo`0D0?ooo`030?oo_`1d0000B9cO00L0oooo00<0_g@00000M02oool00P3o
ool01`2oM000B9cO0?ooo`3oool0_g@00000M02oool00P3oool01@2oM000B9cO0?ooo`3ogml0gooo
00@0oooo00<0_g@007Boo`3oool01@3oool00`3ookl0M00004RLg`0J0?ooo`00@`3oool00`3o_i`0
W=oo0?ooo`020?ooo`030?oOW0180000B9cO00L0oooo00<0_g@00018W03Oool00P3oool01`3OW4P0
07Bo0?ooo`3oool0_g@00018W03Oool00P3oool01P3OW4P007Bo0?ooo`3ookl0M4RL0=ooo`<0oooo
00<0okmd001d_`3oool00`3oool00`3ogi`0B00007Boo`050?ooo`030;md001d_ol0oooo00h0oooo
00<0oknL09cOo`3oool00P3oool00`3ogi`0B00004RLg`070?ooo`030;md0000B9`0gooo0080oooo
00L0gia8001d_`3oool0oooo0;md0000B9`0gooo0080oooo00H0gia8001d_`3oool0oono07A8W03O
ool30?ooo`030?noM000M;l0oooo00<0oooo00<0omnL04P0001d_ol01@3oool00`3OW4P0001809cO
o`0>0?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l01`3oool00`2oM00004RL0=oo
o`020?ooo`070=nLB000M;l0oooo0?ooo`2oM00004RL0=ooo`020?ooo`060=nLB000M;l0oooo0?oo
_`1dB9`0gooo0`3oool00`3o_g@007Bo0?ooo`030?ooo`030?oOW0180000M;oo00<0oooo00<0oono
07A8W03Oool00`3oool00`3o_g@007Bo0?ooo`0:0?ooo`030?noW02Lgol0oooo0080oooo00<0omnL
04P00018W=l01`3oool00`2oM00004RL0=ooo`020?ooo`070=nLB000M;l0oooo0?ooo`2oM00004RL
0=ooo`020?ooo`060=nLB000M;l0oooo0?oo_`1dB9`0gooo0`3oool00`3o_g@007Bo0?ooo`030?oo
o`030?oOW0180000M;oo00@0oooo00<0_g@00018W03Oool00P3oool00`3OW4P007Bo0?ooo`090?oo
o`030?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l01`3oool00`2oM00004RL0=ooo`020?oo
o`070=nLB000M;l0oooo0?ooo`2oM00004RL0=ooo`020?ooo`060=nLB000M;l0oooo0?oo_`1dB9`0
gooo0`3oool00`3o_g@007Bo0?ooo`030?ooo`030?oOW0180000M;oo00<0oooo00<0oooO09a8001d
_ol00P3oool0103ooml0W4P00018W03Oool?0?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P0
0018W=l01`3oool00`2oM00004RL0=ooo`020?ooo`070=nLB000M;l0oooo0?ooo`2oM00004RL0=oo
o`020?ooo`060=nLB000M;l0oooo0?oo_`1dB9`0gooo0`3oool01@3o_g@007Bo0?ooo`3o_i`0W=oo
00<0oooo00<0omnL04P00018W=l06P3oool004<0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000
001d0;ooo`L0oooo00L0okmd0000001d_ol0oooo0?oo_`1d0000M;oo0080oooo00L0okmd0000001d
_ol0oooo0?oo_`1d0000M;oo0080oooo00<0_g@00000000000000`0000001@0007@0_ooo0?ooo`3o
oml0W4P00080000000<0M;oo0?ooo`3oool00`3oool01P3ogi`0B000000000000000001809cOo`/0
oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`L0oooo00L0okmd0000001d_ol0oooo
0?oo_`1d0000M;oo0080oooo00L0okmd0000001d_ol0oooo0?oo_`1d0000M;oo0080oooo00<0_g@0
0000000000000`0000001@0007@0_ooo0?ooo`3ooml0W4P00080000000<0M;oo0?ooo`3oool0103o
ool00`3o_g@00000001d_`0=0?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02oool70?oo
o`070?noM0000000M;oo0?ooo`3ookl0M00007Boo`020?ooo`070?noM0000000M;oo0?ooo`3ookl0
M00007Boo`020?ooo`030;md00000000000000<0000000D0001d0;ooo`3oool0oooO09a800020000
000307Boo`3oool0oooo0080oooo00<0_g@00000000000000`0000000`0007@0_ooo0?ooo`090?oo
o`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02oool70?ooo`070?noM0000000M;oo0?ooo`3o
okl0M00007Boo`020?ooo`070?noM0000000M;oo0?ooo`3ookl0M00007Boo`020?ooo`030;md0000
0000000000<0000000D0001d0;ooo`3oool0oooO09a800020000000307Boo`3oool0oooo0080oooo
00L0okmd0000001d_ol0oooo0?oo_`1d0000M;oo00X0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d
0000001d0;ooo`L0oooo00L0okmd0000001d_ol0oooo0?oo_`1d0000M;oo0080oooo00L0okmd0000
001d_ol0oooo0?oo_`1d0000M;oo0080oooo00<0_g@00000000000000`0000001@0007@0_ooo0?oo
o`3ooml0W4P00080000000<0M;oo0?ooo`3oool00P3oool01`3o_g@0001d0;ooo`3oool0gia80000
B02Lgol0403oool0203ooml0W4Q809cOo`3oool0oono07@0000007@0_ooo1`3oool01`3o_g@00000
07Boo`3oool0oono07@0001d_ol00P3oool01`3o_g@0000007Boo`3oool0oono07@0001d_ol00P3o
ool00`2oM0000000000000030000000:0000M02oool0oooO09a8B02Lgol0oooo0?oo_`1d0000001d
0;oooaX0oooo00140?ooo`060?oog`2LB00000000000000007@0_ooo2@3oool01@3o_g@000000000
00000000B9cO00@0oooo00D0okmd000000000000000004RLg`030?ooo`030?noM0000000000000@0
000000L0M;oo0?ooo`3oool0oooo0?noM0000000M;oo00H0oooo00D0_g@0000000000000000004RL
g`0<0?ooo`060?oog`2LB00000000000000007@0_ooo2@3oool01@3o_g@00000000000000000B9cO
00@0oooo00D0okmd000000000000000004RLg`030?ooo`030?noM0000000000000@0000000L0M;oo
0?ooo`3oool0oooo0?noM0000000M;oo00L0oooo00D0oooO09a800000000001d0;ooo`0;0?ooo`06
0?oog`2LB00000000000000007@0_ooo2@3oool01@3o_g@00000000000000000B9cO00@0oooo00D0
okmd000000000000000004RLg`030?ooo`030?noM0000000000000@0000000L0M;oo0?ooo`3oool0
oooo0?noM0000000M;oo00@0oooo00<0okmd000000000000100000000`1d_ol0oooo0?ooo`090?oo
o`060?oog`2LB00000000000000007@0_ooo2@3oool01@3o_g@00000000000000000B9cO00@0oooo
00D0okmd000000000000000004RLg`030?ooo`030?noM0000000000000@0000000L0M;oo0?ooo`3o
ool0oooo0?noM0000000M;oo00D0oooo00D0okmd000000000000000004RLg`0<0?ooo`060?oog`2L
B00000000000000007@0_ooo2@3oool01@3o_g@00000000000000000B9cO00@0oooo00D0okmd0000
00000000000004RLg`030?ooo`030?noM0000000000000@0000000L0M;oo0?ooo`3oool0oooo0?no
M0000000M;oo00D0oooo00D0okmd000000000000001809cOo`0B0?ooo`060?oog`2LB00000000000
000007@0_ooo2@3oool01@3o_g@00000000000000000B9cO00@0oooo00D0okmd0000000000000000
04RLg`030?ooo`030?noM0000000000000@0000000@0M;oo0?ooo`3ooml0W4P00P0000000`0007@0
_ooo0?ooo`0J0?ooo`00o`3oooo/0?ooo`00o`3oooo/0?ooo`00o`3oooo/0?ooo`00o`3oooo/0?oo
o`00o`3oooo/0?ooo`00o`3oooo/0?ooo`00o`3oooo/0?ooo`00o`3oooo/0?ooo`00o`3oooo/0?oo
o`00F03ooooo000006/00000:@3oool005P0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool00`3oool2000000D0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo02L0oooo
001H0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo00<0oooo0P0000050?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0;@3oool2000001<0oooo00<00000
0?ooo`3oool0@P3oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo0440oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1j0?oo
o`800000jP3oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1j0?ooo`80
0000jP3oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1j0?ooo`800000
jP3oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1j0?ooo`800000jP3o
ool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1j0?ooo`800000jP3oool0
0`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1j0?ooo`800000jP3oool00`00
0000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1i0?ooo`030000003oool000000>X0
oooo00<000000?ooo`3oool09`3oool005P0oooo0P00001j0?ooo`030000003oool000000>T0oooo
0P00000Y0?ooo`00F03oool00`000000oooo0?ooo`1i0?ooo`030000003oool000000>X0oooo00<0
00000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0000003Z
0?ooo`030000003oool0oooo02L0oooo001H0?ooo`030000003oool0oooo07T0oooo00<000000?oo
o`000000jP3oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1i0?ooo`03
0000003oool000000>X0oooo00<000000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3oool0
N@3oool00`000000oooo0000003Z0?ooo`030000003oool0oooo02L0oooo001H0?ooo`030000003o
ool0oooo07T0oooo00<000000?ooo`000000jP3oool00`000000oooo0?ooo`0W0?ooo`00@`3oool0
103ooml0W4P00018W03Oool30?ooo`050?oOW018000000000000M02oool02@3oool00`000000oooo
0?ooo`1i0?ooo`030000003oool000000>X0oooo00<000000?ooo`3oool09`3oool004<0oooo00@0
oooO09a80000B9`0gooo0P3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool0203oool00`00
0000oooo0?ooo`1i0?ooo`030000003oool000000>X0oooo00<000000?ooo`3oool09`3oool004<0
oooo00@0oooO09a80000B9`0gooo0P3oool00`2oM00004RL0=ooo`020?ooo`030;md0000M;l0oooo
00L0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0000003Z0?ooo`030000003oool0oooo
02L0oooo000o0?ooo`030;md00000000000000D0000000@0M;oo0?oo_`1d0000B9cO0`3oool00`3O
W4P0001d0;ooo`070?ooo`030000003oool0oooo07T0oooo00<000000?ooo`000000jP3oool00`00
0000oooo0?ooo`0W0?ooo`00<P3oool00`3ooml0W4P0000000070000000504RLg`3oool0oooo0?oO
W01dW=l00P3oool0203ooml0W4P00018W03Oool0oooo0?oOW0180000M;oo0`3oool00`3o_g@00018
09cOo`070?ooo`030000003oool0oooo07T0oooo00<000000?ooo`000000jP3oool00`000000oooo
0?ooo`0W0?ooo`00@03oool02`3OW4P0M;oo0?ooo`3ooml0W4P00018W03Oool0oooo0?oOW0180000
M;oo00<0oooo00<0okmd0000B02Lgol01`3oool3000007T0oooo00<000000?ooo`000000j03oool3
000002T0oooo00100?ooo`0;0?oog`2LB7@0_ooo0?oog`2LB00004RL0=ooo`3oool0omnL04P0001d
_ol00`3oool00`3o_g@0001809cOo`070?ooo`030000003oool0oooo07T0oooo00<000000?ooo`00
0000jP3oool00`000000oooo0?ooo`0W0?ooo`00@@3oool02P3ogi`0B7Bo0?oog`2LB00004RL0=oo
o`3oool0omnL04P0001d_ol30?ooo`030?noM00004P0W=oo00L0oooo00<000000?ooo`3oool0N@3o
ool00`000000oooo0000003Z0?ooo`030000003oool0oooo02L0oooo00120?ooo`090=nLB01d_ml0
W4P00018W03Oool0oooo0?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool01`3oool00`000000
oooo0?ooo`1i0?ooo`030000003oool000000>X0oooo00<000000?ooo`3oool09`3oool00480oooo
00D0oooO09a8B018B00004RL0=ooo`020?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d_`3o
ool01`3oool00`000000oooo0?ooo`1i0?ooo`030000003oool000000>X0oooo00<000000?ooo`3o
ool09`3oool004<0oooo00@0omnL04P00000B9`0gooo0P3oool01`3ookl0M00007Boo`3oool0okmd
0000M02oool0203oool00`000000oooo0?ooo`1i0?ooo`030000003oool000000>X0oooo00<00000
0?ooo`3oool09`3oool004@0oooo00<0gia80018W03Oool00`3oool01@3ookl0M0000000000007@0
_ooo00T0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0000003Z0?ooo`030000003oool0
oooo02L0oooo001H0?ooo`030000003oool0oooo07T0oooo00<000000?ooo`000000jP3oool00`00
0000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1i0?ooo`030000003oool000000>X0
oooo00<000000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3oool0N@3oool00`000000oooo
0000003Z0?ooo`030000003oool0oooo02L0oooo001H0?ooo`030000003oool0oooo07T0oooo00<0
00000?ooo`000000jP3oool00`000000oooo0?ooo`0W0?ooo`00F03oool2000007X0oooo00<00000
0?ooo`000000j@3oool2000002T0oooo001H0?ooo`030000003oool0oooo07T0oooo00<000000?oo
o`000000jP3oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1i0?ooo`03
0000003oool000000>X0oooo00<000000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3oool0
N@3oool00`000000oooo0000003Z0?ooo`030000003oool0oooo02L0oooo001H0?ooo`030000003o
ool0oooo07T0oooo00<000000?ooo`000000jP3oool00`000000oooo0?ooo`0W0?ooo`00F03oool0
0`000000oooo0?ooo`1i0?ooo`030000003oool000000>X0oooo00<000000?ooo`3oool09`3oool0
05P0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0000003Z0?ooo`030000003oool0oooo
02L0oooo001H0?ooo`030000003oool0oooo07T0oooo00<000000?ooo`000000jP3oool00`000000
oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1i0?ooo`030000003oool000000>X0oooo
00<000000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0000
003Z0?ooo`030000003oool0oooo02L0oooo001H0?ooo`030000003oool0oooo07T0oooo00<00000
0?ooo`000000jP3oool00`000000oooo0?ooo`0W0?ooo`00F03oool2000007X0oooo00<000000?oo
o`000000j@3oool2000002T0oooo001H0?ooo`030000003oool0oooo07T0oooo00<000000?ooo`00
0000jP3oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1i0?ooo`030000
003oool000000>X0oooo00<000000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3oool0N@3o
ool00`000000oooo0000003Z0?ooo`030000003oool0oooo02L0oooo001H0?ooo`030000003oool0
oooo07T0oooo00<000000?ooo`000000jP3oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`00
0000oooo0?ooo`1i0?ooo`030000003oool000000>X0oooo00<000000?ooo`3oool09`3oool005P0
oooo00<000000?ooo`3oool0N@3oool00`000000oooo0000003Z0?ooo`030000003oool0oooo02L0
oooo001H0?ooo`030000003oool0oooo07T0oooo00<000000?ooo`000000jP3oool00`000000oooo
0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1i0?ooo`030000003oool000000>X0oooo00<0
00000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0000003Z
0?ooo`030000003oool0oooo02L0oooo001H0?ooo`030000003oool0oooo07T0oooo00<000000?oo
o`000000jP3oool00`000000oooo0?ooo`0W0?ooo`00F03oool2000007X0oooo00<000000?ooo`00
0000j@3oool2000002T0oooo001H0?ooo`030000003oool0oooo07T0oooo00<000000?ooo`000000
jP3oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1i0?ooo`030000003o
ool000000>X0oooo00<000000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3oool0N@3oool0
0`000000oooo0000003Z0?ooo`030000003oool0oooo02L0oooo001H0?ooo`030000003oool0oooo
07T0oooo00<000000?ooo`000000jP3oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000
oooo0?ooo`1i0?ooo`030000003oool000000>X0oooo00<000000?ooo`3oool09`3oool003l0oooo
00<0_g@0000000000000100000002@18W=l0oooo0?ooo`3oool0omnL04P000000000001d0;ooo`09
0?ooo`030000003oool0oooo07T0oooo00<000000?ooo`000000jP3oool00`000000oooo0?ooo`0W
0?ooo`00@03oool00`3OW4P00000000000030000000:0018W03Oool0oooo0?oo_`1d0000M;oo0?oo
o`3o_g@0001d0;ooo`P0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0000003Z0?ooo`03
0000003oool0oooo02L0oooo00110?ooo`030=nLB018W=l0oooo0080oooo00H0giad09cOo`3oool0
_g@00018W03Oool20?ooo`030;md0000M;l0oooo00L0oooo00<000000?ooo`3oool0N@3oool00`00
0000oooo0000003Z0?ooo`030000003oool0oooo02L0oooo00120?ooo`030=nLB01d_ol0oooo00<0
oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`070?ooo`030000003oool0oooo07T0
oooo00<000000?ooo`000000jP3oool00`000000oooo0?ooo`0W0?ooo`00<P3oool00`3ooml0W4P0
000000070000000304RLg`3oool0oooo00@0oooo00<0_gA809cOo`3oool00P3oool00`3ogi`0B000
07Boo`030?ooo`030?noM00004P0W=oo00L0oooo00<000000?ooo`3oool0N@3oool00`000000oooo
0000003Z0?ooo`030000003oool0oooo02L0oooo00130?ooo`030?oog`2LB7@0_ooo0080oooo00<0
omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`070?ooo`<00000N@3oool00`000000oooo0000
003X0?ooo`<00000:@3oool004@0oooo00L0okmd001d_`3oool0oooo0?oOW0180000M;oo00<0oooo
00<0okmd0000B02Lgol01`3oool00`000000oooo0?ooo`1i0?ooo`030000003oool000000>X0oooo
00<000000?ooo`3oool09`3oool004@0oooo00L0oono07@0B02Lgol0oooo0?oOW0180000M;oo00<0
oooo00<0okmd0000B02Lgol01`3oool00`000000oooo0?ooo`1i0?ooo`030000003oool000000>X0
oooo00<000000?ooo`3oool09`3oool004@0oooo00L0oono07@00018W=l0oooo0?oo_`1d0000B9cO
00<0oooo00<0gia80000M02oool01`3oool00`000000oooo0?ooo`1i0?ooo`030000003oool00000
0>X0oooo00<000000?ooo`3oool09`3oool003l0oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0
B00004RLg`020?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d_`3oool01`3oool00`000000
oooo0?ooo`1i0?ooo`030000003oool000000>X0oooo00<000000?ooo`3oool09`3oool003l0oooo
00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`80oooo00L0oono07@0001d_ol0oooo0?no
M00007@0_ooo00P0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0000003Z0?ooo`030000
003oool0oooo02L0oooo00100?ooo`060?oog`2LB00000000000000007@0_ooo103oool01@3ookl0
M0000000000007@0_ooo00T0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0000003Z0?oo
o`030000003oool0oooo02L0oooo001H0?ooo`030000003oool0oooo07P0oooo00@000000?ooo`3o
ool00000jP3oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1h0?ooo`04
0000003oool0oooo00000>X0oooo00<000000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3o
ool0N03oool010000000oooo0?ooo`00003Z0?ooo`030000003oool0oooo02L0oooo001H0?ooo`03
0000003oool0oooo07P0oooo00@000000?ooo`3oool00000jP3oool00`000000oooo0?ooo`0W0?oo
o`007`3oool00`3ookl0M7Bo0?ooo`040?ooo`030=nLW02oool0oooo02l0oooo0P00001i0?ooo`04
0000003oool0oooo00000>T0oooo0P00000Y0?ooo`007`3oool00`3o_g@000000000000400000003
0000M02oool0oooo02l0oooo00<000000?ooo`3oool0N03oool010000000oooo0?ooo`00003Z0?oo
o`030000003oool0oooo02L0oooo000O0?ooo`030;md00000000000000@0000000<0001d0;ooo`3o
ool0;`3oool00`000000oooo0?ooo`1h0?ooo`040000003oool0oooo00000>X0oooo00<000000?oo
o`3oool09`3oool00200oooo00<0_gAd0;ooo`3oool00`3oool00`3OW9`0_ooo0?ooo`0_0?ooo`03
0000003oool0oooo07P0oooo00@000000?ooo`3oool00000jP3oool00`000000oooo0?ooo`0W0?oo
o`007`3oool00`3ogi`0W;oo0?ooo`0f0?ooo`030000003oool0oooo07P0oooo00@000000?ooo`3o
ool00000jP3oool00`000000oooo0?ooo`0W0?ooo`007`3oool00`2oM00007Bo0?ooo`040?ooo`03
0=nLW02oool0oooo02l0oooo00<000000?ooo`3oool0N03oool010000000oooo0?ooo`00003Z0?oo
o`030000003oool0oooo02L0oooo000O0?ooo`030=nLB0000000000000@0000000<0001d0;ooo`3o
ool0;`3oool00`000000oooo0?ooo`1h0?ooo`040000003oool0oooo00000>X0oooo00<000000?oo
o`3oool09`3oool001l0oooo00<0oooO09a800000000100000000`0007@0_ooo0?ooo`0_0?ooo`03
0000003oool0oooo07P0oooo00@000000?ooo`3oool00000jP3oool00`000000oooo0?ooo`0W0?oo
o`00803oool00`3OW9`0_ooo0?ooo`030?ooo`030=nLW02oool0oooo02l0oooo00<000000?ooo`3o
ool0N03oool010000000oooo0?ooo`00003Z0?ooo`030000003oool0oooo02L0oooo000O0?ooo`03
0?noM018W=l0oooo03H0oooo00<000000?ooo`3oool0N03oool010000000oooo0?ooo`00003Z0?oo
o`030000003oool0oooo02L0oooo000O0?ooo`030;md0000M;l0oooo00@0oooo00<0gibL0;ooo`3o
ool0;`3oool00`000000oooo0?ooo`1h0?ooo`040000003oool0oooo00000>X0oooo00<000000?oo
o`3oool09`3oool001l0oooo00<0gia8000000000000100000000`0007@0_ooo0?ooo`0_0?ooo`03
0000003oool0oooo07P0oooo00@000000?ooo`3oool00000jP3oool00`000000oooo0?ooo`0W0?oo
o`007`3oool00`3ooml0W4P000000004000000030000M02oool0oooo02l0oooo0P00001i0?ooo`04
0000003oool0oooo00000>T0oooo0P00000Y0?ooo`009P3oool00`3OW9`0_ooo0?ooo`0_0?ooo`03
0000003oool0oooo07P0oooo00@000000?ooo`3oool00000jP3oool00`000000oooo0?ooo`0W0?oo
o`007`3oool00`3ookl0M7Bo0?ooo`040?ooo`030=nLW02oool0oooo02l0oooo00<000000?ooo`3o
ool0N03oool010000000oooo0?ooo`00003Z0?ooo`030000003oool0oooo02L0oooo000I0?ooo`03
0?oog`2LB00007Bo00<0oooo00<0okmd000000000000100000000`0007@0_ooo0?ooo`0_0?ooo`03
0000003oool0oooo07P0oooo00@000000?ooo`3oool00000jP3oool00`000000oooo0?ooo`0W0?oo
o`006@3oool00`3ooml0W4P0001d_`030?ooo`030;md00000000000000@0000000<0001d0;ooo`3o
ool0;`3oool00`000000oooo0?ooo`1h0?ooo`040000003oool0oooo00000>X0oooo00<000000?oo
o`3oool09`3oool002H0oooo00<0gibL0;ooo`3oool0;`3oool00`000000oooo0?ooo`1h0?ooo`04
0000003oool0oooo00000>X0oooo00<000000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3o
ool0N03oool010000000oooo0?ooo`00003Z0?ooo`030000003oool0oooo02L0oooo001H0?ooo`03
0000003oool0oooo07P0oooo00@000000?ooo`3oool00000jP3oool00`000000oooo0?ooo`0W0?oo
o`007`3oool02@3ooml0W4P00000000007@0_ooO09a800000000001d0;ooo`0`0?ooo`030000003o
ool0oooo07P0oooo00@000000?ooo`3oool00000jP3oool00`000000oooo0?ooo`0W0?ooo`007`3o
ool0203OW7@0W=oo0?oOW018000007Bo0?ooo`3o_g@0B9cO<@3oool00`000000oooo0?ooo`1h0?oo
o`040000003oool0oooo00000>X0oooo00<000000?ooo`3oool09`3oool001l0oooo00T0gibL0;oo
o`3oool0_g@00000B02Lgol0oooo0;mdM02oool0<03oool00`000000oooo0?ooo`1h0?ooo`040000
003oool0oooo00000>X0oooo00<000000?ooo`3oool09`3oool001l0oooo00T0omnL04Qd_`3oool0
omnL04P00000M;l0oooo0;nLW03Oool0<03oool2000007P0oooo00D000000?ooo`3oool0oooo0000
003Y0?ooo`800000:@3oool001l0oooo00D0_g@00000000004P0W=oo0=nLB0020000000307Boo`3o
ool0oooo02l0oooo00<000000?ooo`3oool0M`3oool01@000000oooo0?ooo`3oool000000>X0oooo
00<000000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3oool0M`3oool01@000000oooo0?oo
o`3oool000000>X0oooo00<000000?ooo`3oool09`3oool001l0oooo00<0oono07Ad_`3oool0103o
ool00`3OW9`0_ooo0?ooo`0_0?ooo`030000003oool0oooo07L0oooo00D000000?ooo`3oool0oooo
0000003Z0?ooo`030000003oool0oooo02L0oooo000O0?ooo`030?noM0000000000000@0000000<0
001d0;ooo`3oool0;`3oool00`000000oooo0?ooo`1g0?ooo`050000003oool0oooo0?ooo`000000
jP3oool00`000000oooo0?ooo`0W0?ooo`007`3oool00`2oM000000000000004000000030000M02o
ool0oooo02l0oooo00<000000?ooo`3oool0M`3oool01@000000oooo0?ooo`3oool000000>X0oooo
00<000000?ooo`3oool09`3oool00200oooo00<0_gAd0;ooo`3oool00`3oool00`3OW9`0_ooo0?oo
o`0Q0?ooo`050?oOW018000000000000M02oool02@3oool00`000000oooo0?ooo`1g0?ooo`050000
003oool0oooo0?ooo`000000jP3oool00`000000oooo0?ooo`0W0?ooo`007`3oool00`3ogi`0W;oo
0?ooo`0W0?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`080?ooo`030000003oool0oooo
07L0oooo00D000000?ooo`3oool0oooo0000003Z0?ooo`030000003oool0oooo02L0oooo000O0?oo
o`030;md0000M;l0oooo00@0oooo00<0gibL0;ooo`3oool0803oool00`2oM00004RL0=ooo`020?oo
o`030;md0000M;l0oooo00L0oooo00<000000?ooo`3oool0M`3oool01@000000oooo0?ooo`3oool0
00000>X0oooo00<000000?ooo`3oool09`3oool001l0oooo00<0gia8000000000000100000000`00
07@0_ooo0?ooo`0O0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool01`3oool00`00
0000oooo0?ooo`1g0?ooo`050000003oool0oooo0?ooo`000000jP3oool00`000000oooo0?ooo`0W
0?ooo`007`3oool00`3ooml0W4P000000004000000030000M02oool0oooo01l0oooo00<0omnL04P0
001d_ol00`3oool00`3o_g@0001809cOo`070?ooo`030000003oool0oooo07L0oooo00D000000?oo
o`3oool0oooo0000003Z0?ooo`030000003oool0oooo02L0oooo000P0?ooo`030=nLW02oool0oooo
00<0oooo00<0gibL0;ooo`3oool07`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo
00L0ooooo`00001[000002T0oooo000O0?ooo`030?noM018W=l0oooo02H0oooo00<0omnL04P0001d
_ol00`3oool00`3o_g@0001809cOo`070?ooo`030000003oool0oooo07L0oooo00D000000?ooo`3o
ool0oooo0000003Z0?ooo`030000003oool0oooo02L0oooo000O0?ooo`030;md0000M;l0oooo00@0
oooo00<0gibL0;ooo`3oool07`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00L0
oooo00<000000?ooo`3oool0MP3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0>P0
oooo00<000000?ooo`3oool09`3oool001l0oooo00<0gia8000000000000100000000`0007@0_ooo
0?ooo`0O0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool01`3oool00`000000oooo
0?ooo`1f0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0j03oool00`000000oooo
0?ooo`0W0?ooo`007`3oool00`3ooml0W4P000000004000000030000M02oool0oooo0200oooo00<0
gia80018W03Oool00P3oool00`2oM00007Bo0?ooo`070?ooo`030000003oool0oooo07H0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`3X0?ooo`030000003oool0oooo02L0oooo000V
0?ooo`030=nLW02oool0oooo0200oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00P0oooo
00<000000?ooo`3oool0MP3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0>P0oooo
00<000000?ooo`3oool09`3oool004X0oooo00D0oono07@000000000001d0;ooo`090?ooo`030000
003oool0oooo07H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3X0?ooo`030000
003oool0oooo02L0oooo000P0?ooo`070;md000004P0W=oo0?oo_`1d0000000004RLg`0a0?ooo`03
0000003oool0oooo07H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3X0?ooo`03
0000003oool0oooo02L0oooo000O0?ooo`050?noM0000000001809cOo`2oM0000P0000000`0007@0
_ooo0?ooo`0_0?ooo`030000003oool0oooo07H0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`3X0?ooo`030000003oool0oooo02L0oooo000O0?ooo`090;mdM02oool0oooo0?oo_`1d
B9`0gooo0?oOW018B9`0gooo0300oooo00<000000?ooo`3oool0M@3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0>P0oooo00<000000?ooo`3oool09`3oool001l0oooo00P0_gA809cO
o`3oool0giad09cOo`3oool0oknL09cOoc40oooo00<000000?ooo`3oool0M@3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo0>P0oooo00<000000?ooo`3oool09`3oool001l0oooo00<0
gia8000000000000100000000`00B9`0gooo0?ooo`0_0?ooo`800000MP3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo0>L0oooo0P00000Y0?ooo`007`3oool00`3ookl0M00000000004
000000030018W03Oool0oooo02l0oooo00<000000?ooo`3oool0M@3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0>P0oooo00<000000?ooo`3oool09`3oool002D0oooo00<0_gA809cO
o`3oool0<03oool00`000000oooo0?ooo`1e0?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool0j03oool00`000000oooo0?ooo`0W0?ooo`007`3oool00`2oM7@0_ooo0?ooo`0f0?ooo`03
0000003oool0oooo07D0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3X0?ooo`03
0000003oool0oooo02L0oooo000M0?ooo`030?oo_`1d0000000000H0000000<0B9cO0?ooo`3oool0
;`3oool00`000000oooo0?ooo`1e0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
j03oool00`000000oooo0?ooo`0W0?ooo`00703oool00`3o_g@0000000000007000000030000M02o
ool0oooo02l0oooo00<000000?ooo`3oool0M03oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo0>P0oooo00<000000?ooo`3oool09`3oool001l0oooo00<0_gAd0;ooo`3oool00`3oool0
0`3ookl0M4RL0=ooo`0`0?ooo`030000003oool0oooo07@0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`3X0?ooo`030000003oool0oooo02L0oooo000O0?ooo`030;mdM02oool0oooo
00<0oooo00<0_gA809cOo`3oool0<03oool00`000000oooo0?ooo`1c0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool0j03oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000
oooo0?ooo`1c0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0j03oool00`000000
oooo0?ooo`0W0?ooo`00803oool00`3ooml0W4P0000000020000000307Boo`3oool0oooo0300oooo
00<000000?ooo`3oool0LP3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0>P0oooo
00<000000?ooo`3oool09`3oool001l0oooo00<0oono07@000000000100000000`18W=l0oooo0?oo
o`0_0?ooo`030000003oool0oooo0780oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`3X0?ooo`030000003oool0oooo02L0oooo000O0?ooo`030=nLB018W=l0oooo0080oooo00@0okmd
0000000007@0_ooo<03oool2000007<0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`3W0?ooo`800000:@3oool001l0oooo00<0_gAd0;ooo`3oool00`3oool00`3OW4P0001d0;ooo`0`
0?ooo`030000003oool0oooo0740oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3X
0?ooo`030000003oool0oooo02L0oooo000O0?ooo`030=nLB018W=l0oooo00<0oooo00<0_g@0001d
_`3oool0<03oool00`000000oooo0?ooo`1a0?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool0j03oool00`000000oooo0?ooo`0W0?ooo`007`3oool01`3ooml0W4P00000M02oool0gia8
00000018W=l0<P3oool00`000000oooo0?ooo`1`0?ooo`030000003oool0oooo00P0oooo00<00000
0?ooo`3oool0j03oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1`0?oo
o`030000003oool0oooo00P0oooo00<000000?ooo`3oool0j03oool00`000000oooo0?ooo`0W0?oo
o`006P3oool00`3o_i`0W=oo0?ooo`090?ooo`030=nLW02oool0oooo02l0oooo00<000000?ooo`3o
ool0K`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0>P0oooo00<000000?ooo`3o
ool09`3oool001X0oooo00<0_g@00000000000002@0000000`0007@0_ooo0?ooo`0_0?ooo`030000
003oool0oooo06h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3X0?ooo`030000
003oool0oooo02L0oooo000I0?ooo`030?oo_`1d0000000000X0000000<0001d0;ooo`3oool0;`3o
ool00`000000oooo0?ooo`1/0?ooo`8000003@3oool00`000000oooo0?ooo`3X0?ooo`030000003o
ool0oooo02L0oooo000P0?ooo`030=nLW02oool0oooo00<0oooo00<0gibL0;ooo`3oool0;`3oool0
0`000000oooo0?ooo`1[0?ooo`8000003P3oool00`000000oooo0?ooo`3X0?ooo`030000003oool0
oooo02L0oooo000O0?ooo`030?noM018W=l0oooo03H0oooo00<000000?ooo`3oool0JP3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo0>P0oooo00<000000?ooo`3oool09`3oool001l0
oooo00<0_g@0001d_`3oool0103oool00`3OW9`0_ooo0?ooo`0_0?ooo`030000003oool0oooo06L0
oooo0`00000A0?ooo`030000003oool0oooo0>P0oooo00<000000?ooo`3oool09`3oool001l0oooo
00<0gia8000000000000100000000`0007@0_ooo0?ooo`0_0?ooo`800000IP3oool3000001<0oooo
00<000000?ooo`3oool0i`3oool2000002T0oooo000O0?ooo`030?oog`2LB000000000@0000000<0
001d0;ooo`3oool0;`3oool00`000000oooo0?ooo`1Q0?ooo`@000005P3oool00`000000oooo0?oo
o`3X0?ooo`030000003oool0oooo02L0oooo000V0?ooo`030=nLW02oool0oooo02l0oooo00<00000
0?ooo`3oool0G03oool6000001T0oooo00<000000?ooo`3oool0j03oool00`000000oooo0?ooo`0W
0?ooo`00F03oool00`000000oooo0?ooo`1E0?ooo`L000007`3oool00`000000oooo0?ooo`3X0?oo
o`030000003oool0oooo02L0oooo001H0?ooo`030000003oool0oooo04P0oooo3@00000V0?ooo`03
0000003oool0oooo0>P0oooo00<000000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3oool0
<@3ooolH00000380oooo00<000000?ooo`3oool0j03oool00`000000oooo0?ooo`0W0?ooo`00F03o
ool00`000000oooo0?ooo`060?ooob/00000BP3oool00`000000oooo0?ooo`3X0?ooo`030000003o
ool0oooo02L0oooo000o0?ooo`030;md00000000000000@0000000T0B9cO0?ooo`3oool0oooo0?oO
W018000000000000M02oool02@3oool00`000000oooo0?ooo`1k0?ooo`030000003oool0oooo0>P0
oooo00<000000?ooo`3oool09`3oool00400oooo00<0gia80000000000000`0000002P00B9`0gooo
0?ooo`3ookl0M00007Boo`3oool0okmd0000M02oool80?ooo`030000003oool0oooo07/0oooo00<0
00000?ooo`3oool0`03ooolP000000P0oooo00<000000?ooo`3oool09`3oool00440oooo00<0gia8
04RLg`3oool00P3oool01P3OW7@0W=oo0?ooo`2oM00004RL0=ooo`80oooo00<0_g@0001d_`3oool0
1`3oool00`000000oooo0?ooo`1k0?ooo`030000003oool0oooo05D0ooooJ`00000X0?ooo`030000
003oool0oooo02L0oooo00120?ooo`030=nLB01d_ol0oooo00<0oooo00<0oono07@00018W=l00`3o
ool00`3OW4P0001d0;ooo`070?ooo`030000003oool0oooo07/0oooo00<000000?ooo`3oool0=P3o
oolO000009<0oooo00<000000?ooo`3oool09`3oool004<0oooo00<0_gA809cOo`3oool00P3oool0
0`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00L0oooo00<000000?ooo`3oool0N`3oool0
0`000000oooo0?ooo`0W0?ooo`l00000/P3oool00`000000oooo0?ooo`0W0?ooo`00@`3oool00`3o
oml0W4Qd0;ooo`020?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01`3oool30000
07/0oooo00<000000?ooo`3oool07P3oool900000;l0oooo0`00000Y0?ooo`00A03oool01`3o_g@0
07Bo0?ooo`3oool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`070?ooo`030000003oool0
oooo07/0oooo00<000000?ooo`3oool0603oool700000<T0oooo00<000000?ooo`3oool09`3oool0
04@0oooo00L0oono07@0B02Lgol0oooo0?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01`3o
ool00`000000oooo0?ooo`1k0?ooo`030000003oool0oooo01@0oooo1@00003?0?ooo`030000003o
ool0oooo02L0oooo00140?ooo`070?oo_`1d0000B9cO0?ooo`3ookl0M00004RLg`030?ooo`030=nL
B00007@0_ooo00L0oooo00<000000?ooo`3oool0N`3oool00`000000oooo0?ooo`0A0?ooo`<00000
e03oool00`000000oooo0?ooo`0W0?ooo`00?`3oool00`3o_i`0W=oo0?ooo`020?ooo`030?oOW018
0000B9cO0080oooo00<0gia80018W03Oool00P3oool00`2oM00007Bo0?ooo`070?ooo`030000003o
ool0oooo07/0oooo00<000000?ooo`3oool03`3oool300000=H0oooo00<000000?ooo`3oool09`3o
ool003l0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`80oooo00L0oono07@0001d
_ol0oooo0?noM00007@0_ooo00P0oooo00<000000?ooo`3oool0N`3oool00`000000oooo0?ooo`0=
0?ooo`<00000f03oool00`000000oooo0?ooo`0W0?ooo`00@03oool01P3ooml0W4P0000000000000
001d0;ooo`@0oooo00D0oono07@000000000001d0;ooo`090?ooo`030000003oool0oooo07/0oooo
00<000000?ooo`3oool0303oool200000=X0oooo00<000000?ooo`3oool09`3oool005P0oooo00<0
00000?ooo`3oool0N`3oool00`000000oooo0?ooo`0:0?ooo`800000g03oool00`000000oooo0?oo
o`0W0?ooo`00F03oool00`000000oooo0?ooo`1k0?ooo`030000003oool0oooo00T0oooo0P00003M
0?ooo`030000003oool0oooo02L0oooo001H0?ooo`030000003oool0oooo07/0oooo00<000000?oo
o`3oool0203oool00`000000oooo0?ooo`3M0?ooo`030000003oool0oooo02L0oooo001H0?ooo`03
0000003oool0oooo07/0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3M0?ooo`03
0000003oool0oooo02L0oooo001H0?ooo`800000O03oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo0=d0oooo0P00000Y0?ooo`00F03oool00`000000oooo0?ooo`1k0?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool0g`3oool00`000000oooo0?ooo`0W0?ooo`00F03oool0
0`000000oooo0?ooo`1k0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0g`3oool0
0`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1k0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool0h03oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000
oooo0?ooo`1k0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0h03oool00`000000
oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1k0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0h@3oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?oo
o`1k0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0h@3oool00`000000oooo0?oo
o`0W0?ooo`00F03oool00`000000oooo0?ooo`1k0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool0h@3oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1k0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool0h@3oool00`000000oooo0?ooo`0W0?oo
o`00F03oool00`000000oooo0?ooo`1k0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0hP3oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1k0?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool0hP3oool00`000000oooo0?ooo`0W0?ooo`00F03o
ool2000007`0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3Q0?ooo`800000:@3o
ool005P0oooo00<000000?ooo`3oool0N`3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0><0oooo00<000000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3oool0N`3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo0><0oooo00<000000?ooo`3oool09`3oool005P0
oooo00<000000?ooo`3oool0N`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0><0
oooo00<000000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3oool0N`3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo0><0oooo00<000000?ooo`3oool09`3oool005P0oooo00<0
00000?ooo`3oool0N`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0><0oooo00<0
00000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3oool0N`3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0><0oooo00<000000?ooo`3oool09`3oool005P0oooo00<000000?oo
o`3oool0N`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0><0oooo00<000000?oo
o`3oool09`3oool005P0oooo00<000000?ooo`3oool0N`3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0><0oooo00<000000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3oool0
N`3oool01@000000oooo0?ooo`3oool000000>H0oooo00<000000?ooo`3oool09`3oool005P0oooo
00<000000?ooo`3oool0N`3oool01@000000oooo0?ooo`3oool000000>H0oooo00<000000?ooo`3o
ool09`3oool005P0oooo0P00001l0?ooo`050000003oool0oooo0?ooo`000000i@3oool2000002T0
oooo001H0?ooo`030000003oool0oooo07/0oooo00D000000?ooo`3oool0oooo0000003V0?ooo`03
0000003oool0oooo02L0oooo001H0?ooo`030000003oool0oooo07/0oooo00D000000?ooo`3oool0
oooo0000003V0?ooo`030000003oool0oooo02L0oooo001H0?ooo`030000003oool0oooo07/0oooo
00D000000?ooo`3oool0oooo0000003V0?ooo`030000003oool0oooo02L0oooo001H0?ooo`030000
003oool0oooo07/0oooo00D000000?ooo`3oool0oooo0000003V0?ooo`030000003oool0oooo02L0
oooo001H0?ooo`030000003oool0oooo07/0oooo00D000000?ooo`3oool0oooo0000003V0?ooo`03
0000003oool0oooo02L0oooo00130?ooo`040?oog`2LB00004RL0=ooo`<0oooo00D0omnL04P00000
0000001d0;ooo`090?ooo`030000003oool0oooo07/0oooo00@000000?ooo`3oool00000i`3oool0
0`000000oooo0?ooo`0W0?ooo`00@`3oool0103ooml0W4P00018W03Oool20?ooo`070?oo_`1d0000
M;oo0?ooo`3o_g@0001d0;ooo`080?ooo`030000003oool0oooo07/0oooo00@000000?ooo`3oool0
0000i`3oool00`000000oooo0?ooo`0W0?ooo`00@`3oool0103ooml0W4P00018W03Oool20?ooo`03
0;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool01`3oool00`000000oooo0?ooo`1k0?ooo`04
0000003oool0oooo00000>L0oooo00<000000?ooo`3oool09`3oool003l0oooo00<0_g@000000000
00001@000000101d_ol0oono07@00018W=l30?ooo`030=nLB00007@0_ooo00L0oooo00<000000?oo
o`3oool0N`3oool010000000oooo0?ooo`00003W0?ooo`030000003oool0oooo02L0oooo000o0?oo
o`0<0?oOW01dW=l0oooo0?ooo`3ooml0W4P00018W03Oool0oooo0?oOW0180000M;oo0`3oool00`3o
_g@0001809cOo`070?ooo`030000003oool0oooo07/0oooo00@000000?ooo`3oool00000i`3oool0
0`000000oooo0?ooo`0W0?ooo`00@03oool02`3OW4P0M;oo0?ooo`3ooml0W4P00018W03Oool0oooo
0?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01`3oool3000007/0oooo00@000000?ooo`3o
ool00000i@3oool3000002T0oooo00100?ooo`0;0?oog`2LB7@0_ooo0?oog`2LB00004RL0=ooo`3o
ool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`070?ooo`030000003oool0oooo07/0oooo
00@000000?ooo`3oool00000i`3oool00`000000oooo0?ooo`0W0?ooo`00@@3oool02P3ogi`0B7Bo
0?oog`2LB00004RL0=ooo`3oool0omnL04P0001d_ol30?ooo`030?noM00004P0W=oo00L0oooo00<0
00000?ooo`3oool0N`3oool010000000oooo0?ooo`00003W0?ooo`030000003oool0oooo02L0oooo
00120?ooo`090=nLB01d_ml0W4P00018W03Oool0oooo0?oo_`1d0000B9cO00<0oooo00<0gia80000
M02oool01`3oool00`000000oooo0?ooo`1k0?ooo`040000003oool0oooo00000>L0oooo00<00000
0?ooo`3oool09`3oool00480oooo00D0oooO09a8B018B00004RL0=ooo`020?ooo`030=nLB000B9`0
gooo0080oooo00<0_g@0001d_`3oool01`3oool00`000000oooo0?ooo`1k0?ooo`040000003oool0
oooo00000>L0oooo00<000000?ooo`3oool09`3oool004<0oooo00@0omnL04P00000B9`0gooo0P3o
ool01`3ookl0M00007Boo`3oool0okmd0000M02oool0203oool00`000000oooo0?ooo`1k0?ooo`04
0000003oool0oooo00000>L0oooo00<000000?ooo`3oool09`3oool004@0oooo00<0gia80018W03O
ool00`3oool01@3ookl0M0000000000007@0_ooo00T0oooo00<000000?ooo`3oool0N`3oool01000
0000oooo0?ooo`00003W0?ooo`030000003oool0oooo02L0oooo001H0?ooo`030000003oool0oooo
07/0oooo00@000000?ooo`3oool00000i`3oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`00
0000oooo0?ooo`1k0?ooo`040000003oool0oooo00000>L0oooo00<000000?ooo`3oool09`3oool0
05P0oooo00<000000?ooo`3oool0N`3oool010000000oooo0?ooo`00003W0?ooo`030000003oool0
oooo02L0oooo001H0?ooo`030000003oool0oooo07/0oooo00@000000?ooo`3oool00000i`3oool0
0`000000oooo0?ooo`0W0?ooo`00F03oool2000007`0oooo00<000000?ooo`000000i`3oool20000
02T0oooo001H0?ooo`030000003oool0oooo07/0oooo00<000000?ooo`000000j03oool00`000000
oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1k0?ooo`030000003oool000000>P0oooo
00<000000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3oool0N`3oool00`000000oooo0000
003X0?ooo`030000003oool0oooo02L0oooo001H0?ooo`030000003oool0oooo07/0oooo00<00000
0?ooo`000000j03oool00`000000oooo0?ooo`0W0?ooo`00F03oool00`000000oooo0?ooo`1k0?oo
o`030000003oool000000>P0oooo00<000000?ooo`3oool09`3oool005P0oooo00<000000?ooo`3o
ool0N`3oool00`000000oooo0000003X0?ooo`030000003oool0oooo02L0oooo001H0?ooo`030000
003oool0oooo07/0oooo00<000000?ooo`000000j03oool00`000000oooo0?ooo`0W0?ooo`00F03o
ool00`000000oooo0?ooo`1k0?ooo`030000003oool000000>P0oooo00<000000?ooo`3oool09`3o
ool005P0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`110?ooo`030000003oool0
oooo02h0oooo00<000000?ooo`0000004@3oool00`000000oooo0?ooo`120?ooo`030000003oool0
oooo0480oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo02L0oooo001H0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`0000000`3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool09`3oool005P0
ooooo`00001[000002T0oooo003o0?ooon`0oooo003o0?ooon`0oooo003o0?ooon`0oooo003o0?oo
on`0oooo003o0?ooon`0oooo003o0?ooon`0oooo003o0?ooon`0oooo003K0?ooo`030?oog`2LB000
000000<0000000<0B9cO0?ooo`3oool0o`3oool80?ooo`00f`3oool00`2oM00004RL0=ooo`030?oo
o`030=nLW02oool0oooo0?l0oooo203oool00=/0oooo00<0oooO09adW03Oool00`3oool00`3ookl0
W9cO0?ooo`3o0?ooo`P0oooo003L0?ooo`030;md00000000000000<0000000<0B9cO0?ooo`3oool0
o`3oool70?ooo`00XP3oool01P3OW4P000000000000004P0W=oO09a80080000000<0001d0;oog`2L
B0000P0000000`00M;l0oooO09a80002000000090018W02LM0000000001d_`3oool0omnL04P00000
0000B9cO0080oooo00L0_g@0000000000000001809cOo`3oool0gia80080000000<0001809cOg`2L
B0000P0000001P00M;l0gia8000000000000001d07Ad0080000000<0001d0;oog`2LB0000P000000
0`00B9`0gono07@00004000000030000M02oool0oooo00L0oooo00D0_g@0000000000000001809cO
o`030?ooo`050=nLB000000000000000001d_ol00P3oool01P2oM00000000000000007@0_ooO09a8
0080000000@004RL0=ooo`3ooml0W4P00P0000001@0004P0B4P00018W03Oool0gia80080000000L0
001d0;ooo`3ogi`0B00000000018W=l0gia80080000000@0001d0;ooo`3oool0gia80`000000101d
_ol0oooo0?ooo`2oM002000000030000M02ooml0W4P00080000000<004RL0=ooo`3oool01@3oool0
0`3ogi`0B000000000020000000407Boo`3oool0oooo0=nLB0<0000000@0M;oo0?ooo`3oool0_g@0
0P0000000`0004P0W=oo0?ooo`040?ooo`060=nLB000000000000000B02Lgml0W4P00P0000000`00
07@0_ooO09a8000200000003001d_`3oool0oooo00X0oooo00<0_g@0000000000000100000002@18
W=l0oooo0?ooo`3oool0omnL04P000000000001d0;ooo`1f0?ooo`00X`3oool00`3OW4P0001d0;oo
o`020?ooo`0U0;md0000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd0000001d_ol0oooo0?oO
W018000007Bo0?no_`2oool0gia80000M02oool0gkno0=ooo`2oM00000000018W03Oool0oooO09a8
M02oool0oooo0=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`020?ooo`0;0=nLB00007@0_ooo0?oo
o`3OW4P004RL0=ooo`3oool0oooO09a80000M;l00P3oool00`2oM4P0W=oo0?ooo`0;0?ooo`0?0;md
0000000004RL0=ooo`3ooml0W4Qd0;ooo`3oool0gia800000018W=l0oooo0?oog`2LB000B9cO0080
oooo02H0gia80018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3ooml0W4P00000001d_ol0oooo0?oo
_`1d000000000000M02oool0gia80000M02oool0oooo0=nLB00007@0_ooo0=no_`3Oool0gia80000
M02oool0oooo0=nLB0000000B9cO0?ooo`3ooml0W4P004RLg`80oooo00L0gia80018W03Oool0oooo
0?oog`2LB00007Bo00T0oooo00<0oooO09a80000M;l00`3oool01`3OW4P0000004RLg`3oool0oooO
09a80018W=l00P3oool00`3OW4P0001d0;ooo`070?ooo`030=nLB00007@0_ooo0080oooo00L0_g@0
0018W03Oool0oooo0?oog`2LB00007Bo00h0oooo00<0gia80000000000000`0000002P00B9`0gooo
0?ooo`3ookl0M00007Boo`3oool0okmd0000M02ooome0?ooo`00X`3oool00`3OW4P0001d0;ooo`02
0?ooo`0?0;md0000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd0000B02Lgol0oooo0?oog`2L
B00007Bo0080oooo00L0gia80018W03Oool0oooo0?oOW018000007Bo00<0oooo00X0omnL09boo`3o
ool0gia80000M02oool0oooo0?oog`2LB00007Bo0P3oool02`3OW4P0001d0;ooo`3oool0gia80018
W03Oool0oooo0?oog`2LB00007Bo0080oooo00D0oono07@0000000000000001d_`080?ooo`030?oO
W018000007Bo00<0oooo00D0omnL09boo`3ogi`0B000001d_`030?ooo`0?0?noM00007@0_ooo0?oo
o`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?oOW0180000B9cO0080oooo00<0oooO09a8
0000M;l00P3oool01`3OW4P0001d0;ooo`3oool0gia80018W03Oool00P3oool01P3OW4P0001d0;oo
o`3ogi`0B000001d_`<0oooo00/0okmd0000M02oool0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0
001d_`090?ooo`030?oog`2LB00007Bo0080oooo00<0omnL04P00000M;l00`3oool01`3o_g@0001d
0;ooo`3oool0gia80000M02oool01`3oool00`3OW4P0001d0;ooo`020?ooo`0:0;md0000B9`0gooo
0?ooo`3ooml0W4P0001d_`3oool0oooO09a800P0000000H0B9cO0?ooo`3oool0oooo0=nLB018W=l3
0?ooo`060=nLM02Lgol0oooo0;md0000B9`0gooo0P3oool00`2oM00007Bo0?ooo`1d0?ooo`00X`3o
ool00`3OW4P0001d0;ooo`020?ooo`070;md0000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`06
0?noM000000007Bo0?oog`2LB00007Bo0P3oool01`3OW4P004RL0=ooo`3oool0okmd0000B02Lgol0
1P3oool01`3OW4P0001d0;ooo`3oool0oooO09a80000M;l00P3oool02`3OW4P0001d0;ooo`3oool0
gia80018W03Oool0oooo0?oog`2LB00007Bo0080oooo00H0gia80000M02oool0oono07@00018W=l7
0?ooo`030?noM00004P0W=oo00D0oooo00<0okmd0000001d_ol00`3oool03`3o_g@0000007Boo`3o
ool0gia80018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3o_g@0000007Boo`020?ooo`030?oog`2L
B00007Bo0080oooo00L0gia80000M02oool0oooo0=nLB000B9`0gooo0080oooo00H0gia80000M02o
ool0okmd0000001d_ol30?ooo`0;0?noM0000000M;oo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a8
0000M;l02@3oool00`3ooml0W4P0001d_`020?ooo`030?noM0000000M;oo00<0oooo00L0okmd0000
001d_ol0oooo0=nLB00007@0_ooo00L0oooo00<0gia80000M02oool00P3oool01`2oM00004RL0=oo
o`3oool0oooO09a80000M;l0403oool00`3OW4P0M;oo0?ooo`030?ooo`030?oo_`1d0000B9cO00<0
oooo00<0gia80000M02oool0M03oool00:<0oooo00<0gia80000M02oool00P3oool01`2oM00004RL
0=ooo`3oool0oooO09a80000M;l0103oool0103o_g@0000000000000M;l20?ooo`070=nLB000B9`0
gooo0?ooo`3o_g@0001d0;ooo`060?ooo`070=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`020?oo
o`0C0=nLB00007@0_ooo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?oo_`1d0000
B9cO0?ooo`3ooml0W4P0001d_`070?ooo`030?noM00007@0_ooo00D0oooo00<0okmd0000001d_ol0
0`3oool03`3o_g@0000007Boo`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3o_g@0
000007Boo`020?ooo`030?oog`2LB00007Bo0080oooo00L0gia80000M02oool0oooo0=nLB000B9`0
gooo0080oooo00H0gia80000M02oool0okmd0000001d_ol30?ooo`0;0?noM0000000M;oo0?ooo`3O
W4P004RL0=ooo`3oool0oooO09a80000M;l02@3oool00`3ooml0W4P0001d_`020?ooo`030?noM000
0000M;oo00<0oooo00L0okmd0000001d_ol0oooo0=nLB00007@0_ooo00L0oooo00<0gia80000M02o
ool00P3oool01`2oM00004RL0=ooo`3oool0oooO09a80000M;l04@3oool00`2oM4P0W=oo0?ooo`02
0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0M03oool00:<0oooo00<0gia80000
B02Lgol00P3oool03`2oM000001d0;ooo`3oool0oooO09a80000M;l0oooo0?oo_`1d0000M;oo0?oo
o`3ooml0W4P0001d_`020?ooo`070=nLB000B9`0gooo0?ooo`3ogi`0B01809cOo`020?ooo`0;0?no
M00004P0W=oo0?ooo`3OW4P0001d0;ooo`3oool0oooO09a80000M;l00P3oool04`3OW4P0001d0;oo
o`3oool0_g@00000M02oool0oooo0?oog`2LB00007Bo0?ooo`3ookl0M00004RLg`3oool0oooO09a8
0018W=l01`3oool00`3ogi`0B01809cOo`020?ooo`060?noM00004P0W=oo0?oo_`1d0000M;oo0`3o
ool03`2oM000001809cOo`3oool0_g@00000M02oool0oooo0?oog`2LB00007Bo0?ooo`3ookl0M000
04RLg`020?ooo`030?oog`2LB00007Bo0080oooo00L0gia80000M02oool0oooo0=nLB000B9`0gooo
0080oooo00H0gia80000M02oool0oono07@0001d_ol30?ooo`0;0;md000004P0W=oo0?ooo`2oM000
001d0;ooo`3oool0oooO09a80000M;l02@3oool00`3ooml0W4P0001d_`020?ooo`030?oo_`1d0000
M;oo00<0oooo00L0_g@00000B02Lgol0oooo0=nLB00004P0W=oo00L0oooo00<0gia80000B02Lgol0
0P3oool01`2oM000001d0;ooo`3oool0oooO09a80000M;l04@3oool00`3ooml0W4Qd0;ooo`020?oo
o`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0M03oool00:40oooo01H0oooO09a80000
0000001804Q8B02Lgol0oono07@00000M7@0M01d0;ooo`3ookl0M00004RLg`3oool0oono07@00018
W=l0oooo0?oo_`1d000007Bo0P3oool00`3OW4P004RL0=ooo`020?ooo`0L0;md0000B9`0gooo0?oo
g`2LB000001d0;ooo`3oool0gia80000000004P0W=oo0?oo_`1d0000B9cO0?oog`2LB00000000000
M02ooml0W4P000000000B4P0B01809cOo`3ookl0M00004RLg`80oooo00@0_g@0001d_`3oool0okmd
0P0000000`00B9`0gooo0?ooo`050?ooo`0H0;md0000B9`0gooo0?oog`2LB000001d0;ooo`3oool0
gia80018W03Oool0oooo0?noM000000007Bo0?oog`2LB00000000018B01804P0W=oo0?oo_`1d0000
B9cO0P3oool09P3o_g@0001809cOo`3oool0okmd00000000M;l0oooO09a800000000001d0;ooo`3o
ool0gia80018W03Oool0oooO09a800000000001d0;ooo`3oool0gia80018W03Oool0oooo0?noM000
000007Bo0?oog`2LB00000000018B01804P0W=oo0?oo_`1d0000B9cO2@3oool00`3ooml0W4P0001d
_`030?ooo`0?0=nLB000B9`0gooo0?ooo`3o_g@00000001d_`3ooml0W4P00000000004P0B7BL0=no
M00007@0_ooo0080oooo0140oooO09a800000000001804Q8B02Lgol0oono07@00000M7@0M01d0;oo
o`3ookl0M00004RLg`3oool0oooO09a800080000000304RLg`3oool0oooo00@0oooo00L0okmd001d
_`3oool0oooo0?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0M03oool00:<0oooo00`0_g@0
0000M02ooml0W4P00000000004P0W=oo0?oo_`1d000000000000M02oool20?ooo`080?oo_`1d0000
00000000000007@0_ooo0?ooo`2oM003000000040018W03Oool0oooo0=nLB080000000/0001809cO
o`3oool0oooo0=nLB00007@0_ooO09a800000000001d0;ooo`020?ooo`0;0;md000007@0_ooo0?oo
o`2oM00004RL0=oog`2LB00000000000M02oool00P3oool01@3ooml0W4P0000000000000M;oo00X0
oooo00D0gia8000000000000001809cOo`030?ooo`050?noM0000000000000000018W=l00`3oool0
1`2oM00004RL0=oog`2LB00000000000M02oool00`3oool00`3o_g@000000000000200000007001d
_`3oool0oooo0;md000007@0_ooo0;md0003000000050018W03Oool0_g@00000M02oool00P3oool0
1@3o_g@00000000000000000B9cO00<0oooo00L0_g@00018W03Ooml0W4P00000000007@0_ooo00P0
oooo00<0okmd0000000000000P0000001000B9`0gooo0?ooo`3o_g@30000000:04RLg`3oool0oooo
0?ooo`2oM000001d0;oOW0180000001d0;ooo`@0oooo00`0_g@00000M02ooml0W4P00000000004P0
W=oo0?oo_`1d000000000000M02ooolB0?ooo`070?oo_`1d04P0W=oo0?ooo`3ogi`0B00007Boo`03
0?ooo`030?noM00004P0W=oo07@0oooo002i0?ooo`030;md0000B9`0gooo00X0oooo00<0gia80000
M02oool0?`3oool00`3ooml0W4P0001d_`060?ooo`030;md0000B9`0gooo01l0oooo00<0oooO09a8
0000M;l0=03oool01`3ookl0M00004RLg`3oool0oono07@00018W=l00`3oool00`3OW4P0001d0;oo
o`1d0?ooo`00^@3oool00`3ookl0M4RL0=ooo`0:0?ooo`030=nLB00007@0_ooo03l0oooo00<0oooO
09a80000M;l01P3oool00`3ookl0M4RL0=ooo`0P0?ooo`030;md0000M;l0oooo02h0oooo00<0oknL
09cOo`3oool00P3oool00`3ogi`0B00004RLg`020?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0
001d_`3oool0M03oool00;T0oooo00<0oooO09adW03Oool02P3oool00`3OW4P0001d0;ooo`0o0?oo
o`030?oog`2LB00007Bo00H0oooo00<0oooO09adW03Oool0803oool00`3OW4P007Bo0?ooo`0^0?oo
o`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02oool20?ooo`070?oo_`1d0000M;oo0?ooo`3o
_g@0001d0;ooo`1e0?ooo`00a03oool01@3ooml0W4P00000000007@0_ooo00H0oooo00<0gia80000
B02Lgol0=@3oool0103ookl0M00000000000M;l20?ooo`030=nLB00004P0W=oo00H0oooo00<0gia8
0000B02Lgol06`3oool01`3ookl0M01d0;ooo`3ooml0W4P00018W03Oool0:`3oool01P3ooml0W4P0
000000000000001d0;ooo`@0oooo00D0oono07@000000000001d0;ooo`1f0?ooo`00aP3oool00`2o
M000001d0;ooo`060?ooo`030=nLB00004P0W=oo03H0oooo00<0oooO09a80000M;l00P3oool00`3O
W4P0001809cOo`060?ooo`030=nLB00004P0W=oo01`0oooo00D0oono07@000000000001809cOo`2a
0?ooo`00o`3oooo/0?ooo`00o`3oooo/0?ooo`00o`3oooo/0?ooo`00o`3oooo/0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 490.438}, {302.75, 0}} -> {2.88714, -73.1151, \
2.18125*^-07, 0.672255}}]
}, Closed]],

Cell[TextData[{
  "Recognizing that the even and odd solutions are practically degenerate, \
one can instead locate the energies using just one side of the potential and \
choosing a match point at any convenient point.  However, in this case only \
the zeros of the matching condition represent true solutions \[LongDash] \
unless we accidentally select a match point that coincides precisely with a \
node in the wave function, the poles are spurious.  By starting the wave \
function at zero, this method finds the odd solutions only, but we know that \
for each odd solution there is an even solution at slightly smaller energy.   \
Note we need to increase ",
  StyleBox["AccuracyGoal", "InlineInput",
    FontWeight->"Bold"],
  " to obtain a clean plot."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
        mismatch[e, 20, pot, 0, 3, 2, AccuracyGoal \[Rule] 10, 
          MaxSteps \[Rule] 5000], {e, 2.85, 5}, 
        PlotRange \[Rule] {All, {\(-50\), 50}}, PlotPoints \[Rule] 100, 
        Frame \[Rule] True, 
        FrameLabel \[Rule] {"\<energy\>", "\<mismatch\>"}, 
        PlotLabel \[Rule] "\<matching condition for m=20\>"];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.23865 0.442968 0.309017 0.00618034 [
[.09025 -0.0125 -5 -15.6875 ]
[.09025 -0.0125 5 0 ]
[.31174 -0.0125 -9.5 -15.6875 ]
[.31174 -0.0125 9.5 0 ]
[.53322 -0.0125 -5 -15.6875 ]
[.53322 -0.0125 5 0 ]
[.75471 -0.0125 -9.5 -15.6875 ]
[.75471 -0.0125 9.5 0 ]
[.97619 -0.0125 -5 -15.6875 ]
[.97619 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -17.9688 -31.375 ]
[.5 -0.0125 17.9688 -15.6875 ]
[-0.0125 .0618 -24.3125 -7.84375 ]
[-0.0125 .0618 0 7.84375 ]
[-0.0125 .18541 -24.3125 -7.84375 ]
[-0.0125 .18541 0 7.84375 ]
[-0.0125 .30902 -10 -7.84375 ]
[-0.0125 .30902 0 7.84375 ]
[-0.0125 .43262 -16 -7.84375 ]
[-0.0125 .43262 0 7.84375 ]
[-0.0125 .55623 -16 -7.84375 ]
[-0.0125 .55623 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -40 -25.2813 ]
[-0.01875 .30902 -24.3125 25.2813 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -72.8125 0 ]
[.5 .63053 72.8125 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.09025 0 m
.09025 .00625 L
s
gsave
.09025 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
.31174 0 m
.31174 .00625 L
s
gsave
.31174 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3.5) show
1.000 setlinewidth
grestore
.53322 0 m
.53322 .00625 L
s
gsave
.53322 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.75471 0 m
.75471 .00625 L
s
gsave
.75471 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4.5) show
1.000 setlinewidth
grestore
.97619 0 m
.97619 .00625 L
s
gsave
.97619 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
.125 Mabswid
.13455 0 m
.13455 .00375 L
s
.17885 0 m
.17885 .00375 L
s
.22315 0 m
.22315 .00375 L
s
.26744 0 m
.26744 .00375 L
s
.35604 0 m
.35604 .00375 L
s
.40033 0 m
.40033 .00375 L
s
.44463 0 m
.44463 .00375 L
s
.48893 0 m
.48893 .00375 L
s
.57752 0 m
.57752 .00375 L
s
.62182 0 m
.62182 .00375 L
s
.66611 0 m
.66611 .00375 L
s
.71041 0 m
.71041 .00375 L
s
.799 0 m
.799 .00375 L
s
.8433 0 m
.8433 .00375 L
s
.8876 0 m
.8876 .00375 L
s
.93189 0 m
.93189 .00375 L
s
.04596 0 m
.04596 .00375 L
s
.00166 0 m
.00166 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -78.9688 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(energy) show
94.938 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .0618 m
.00625 .0618 L
s
gsave
-0.0125 .0618 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
0 .18541 m
.00625 .18541 L
s
gsave
-0.0125 .18541 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
0 .30902 m
.00625 .30902 L
s
gsave
-0.0125 .30902 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
0 .43262 m
.00625 .43262 L
s
gsave
-0.0125 .43262 -77 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
0 .55623 m
.00625 .55623 L
s
gsave
-0.0125 .55623 -77 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .09271 m
.00375 .09271 L
s
0 .12361 m
.00375 .12361 L
s
0 .15451 m
.00375 .15451 L
s
0 .21631 m
.00375 .21631 L
s
0 .24721 m
.00375 .24721 L
s
0 .27812 m
.00375 .27812 L
s
0 .33992 m
.00375 .33992 L
s
0 .37082 m
.00375 .37082 L
s
0 .40172 m
.00375 .40172 L
s
0 .46353 m
.00375 .46353 L
s
0 .49443 m
.00375 .49443 L
s
0 .52533 m
.00375 .52533 L
s
0 .0309 m
.00375 .0309 L
s
0 .58713 m
.00375 .58713 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -118.438 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
86.2813 11.8438 translate
90 rotate
-86.2813 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(mismatch) show
109.563 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.09025 .61178 m
.09025 .61803 L
s
.31174 .61178 m
.31174 .61803 L
s
.53322 .61178 m
.53322 .61803 L
s
.75471 .61178 m
.75471 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.13455 .61428 m
.13455 .61803 L
s
.17885 .61428 m
.17885 .61803 L
s
.22315 .61428 m
.22315 .61803 L
s
.26744 .61428 m
.26744 .61803 L
s
.35604 .61428 m
.35604 .61803 L
s
.40033 .61428 m
.40033 .61803 L
s
.44463 .61428 m
.44463 .61803 L
s
.48893 .61428 m
.48893 .61803 L
s
.57752 .61428 m
.57752 .61803 L
s
.62182 .61428 m
.62182 .61803 L
s
.66611 .61428 m
.66611 .61803 L
s
.71041 .61428 m
.71041 .61803 L
s
.799 .61428 m
.799 .61803 L
s
.8433 .61428 m
.8433 .61803 L
s
.8876 .61428 m
.8876 .61803 L
s
.93189 .61428 m
.93189 .61803 L
s
.04596 .61428 m
.04596 .61803 L
s
.00166 .61428 m
.00166 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .0618 m
1 .0618 L
s
.99375 .18541 m
1 .18541 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .43262 m
1 .43262 L
s
.99375 .55623 m
1 .55623 L
s
.125 Mabswid
.99625 .09271 m
1 .09271 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .15451 m
1 .15451 L
s
.99625 .21631 m
1 .21631 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .27812 m
1 .27812 L
s
.99625 .33992 m
1 .33992 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .40172 m
1 .40172 L
s
.99625 .46353 m
1 .46353 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .52533 m
1 .52533 L
s
.99625 .0309 m
1 .0309 L
s
.99625 .58713 m
1 .58713 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .30902 m
1 .30902 L
s
gsave
.5 .63053 -133.813 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(matching) show
110.375 15.813 moveto
(condition) show
158.563 15.813 moveto
(for) show
175.438 15.813 moveto
(m) show
184.813 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
192.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
204.625 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .39029 m
.02599 .38945 L
.02837 .38827 L
.02957 .38752 L
.03086 .38656 L
.03206 .38543 L
.03318 .38413 L
.03449 .38207 L
.03519 .38064 L
.03593 .37873 L
.03656 .37661 L
.03716 .37403 L
.03782 .37017 L
.03819 .36728 L
.03852 .36396 L
.03882 .36023 L
.03914 .35508 L
.03944 .34853 L
.03972 .34033 L
.03988 .33364 L
.04004 .3258 L
.04021 .31445 L
.04039 .29745 L
.04069 .24301 L
.04086 .17161 L
s
.04086 .17161 m
.04101 0 L
s
.04102 0 m
.04119 .61803 L
s
.0416 .61803 m
.04162 .59378 L
.04188 .5143 L
.04216 .47782 L
.04247 .45693 L
.04263 .44987 L
.0428 .44368 L
.0431 .43551 L
.04339 .42992 L
.04368 .42562 L
.04398 .42201 L
.04453 .41717 L
.04486 .41491 L
.04518 .41313 L
.04577 .41046 L
.0464 .40828 L
.04698 .40668 L
.04828 .404 L
.04932 .40246 L
.05046 .40114 L
.05277 .39921 L
.05403 .39841 L
.05523 .39776 L
.05748 .39677 L
.06255 .39513 L
.06736 .39399 L
.07182 .39313 L
.08132 .3916 L
.09141 .39021 L
.10114 .38897 L
.1105 .38782 L
.12045 .38662 L
.13003 .38547 L
.13925 .38434 L
.14906 .38311 L
.15851 .38189 L
.16855 .38054 L
.17822 .37918 L
.18753 .37778 L
.19742 .37619 L
.20696 .37451 L
.21612 .3727 L
.22588 .3705 L
.23527 .36796 L
.24526 .36453 L
.25024 .36236 L
.25488 .35992 L
.25935 .357 L
.26413 .35295 L
.26662 .35025 L
Mistroke
.26931 .34667 L
.27186 .34237 L
.2742 .33724 L
.27656 .33033 L
.27876 .32127 L
.28002 .31425 L
.28117 .30606 L
.28181 .30052 L
.2825 .29337 L
.28311 .28589 L
.28376 .2761 L
.28438 .26477 L
.28494 .25179 L
.28547 .23614 L
.28604 .2146 L
.28634 .20022 L
.28666 .18161 L
.28694 .16197 L
.28724 .13553 L
.28759 .09454 L
.28776 .0706 L
.28793 .04028 L
.28808 .00725 L
Mfstroke
.28808 .00725 m
.28811 0 L
s
.28967 0 m
.2897 .61803 L
s
.29197 .61803 m
.29213 .60024 L
.29245 .57423 L
.29302 .5401 L
.29335 .52545 L
.29364 .514 L
.29419 .49683 L
.29477 .4827 L
.2954 .47044 L
.29598 .46134 L
.29703 .44848 L
.29817 .438 L
.29942 .4293 L
.30076 .42214 L
.30317 .41274 L
.30565 .4059 L
.30835 .40044 L
.31089 .39648 L
.31323 .39352 L
.31778 .38903 L
.32279 .38533 L
.33257 .38011 L
.34199 .37643 L
.352 .37325 L
.36165 .3706 L
.37092 .36826 L
.38079 .3659 L
.3903 .36368 L
.40039 .36133 L
.41012 .35903 L
.41949 .35674 L
.42945 .35417 L
.43904 .35151 L
.44826 .34872 L
.45808 .34541 L
.46753 .34174 L
.47757 .33709 L
.48725 .3315 L
.49656 .32444 L
.50136 .31977 L
.50646 .31365 L
.51119 .30639 L
.51563 .29748 L
.51814 .29108 L
.52054 .2837 L
.52289 .27474 L
.52503 .26458 L
.52735 .25038 L
.52854 .24126 L
.52982 .2295 L
Mistroke
.53102 .21622 L
.53212 .2013 L
.53325 .18221 L
.53428 .16018 L
.53542 .12819 L
.536 .10732 L
.53664 .0797 L
.53723 .04811 L
.53777 .01202 L
Mfstroke
.53777 .01202 m
.53791 0 L
s
.54236 0 m
.54237 .61803 L
s
.54812 .61803 m
.54863 .59438 L
.54928 .57115 L
.55 .55079 L
.55129 .52153 L
.55255 .50014 L
.55371 .48455 L
.55605 .4612 L
.55825 .44543 L
.56065 .4323 L
.56324 .42144 L
.56811 .4067 L
.57055 .40111 L
.57323 .39591 L
.57839 .38785 L
.58307 .3821 L
.59254 .37321 L
.60225 .36637 L
.61158 .36103 L
.62151 .35615 L
.63107 .35194 L
.64027 .34817 L
.65006 .34431 L
.65948 .34066 L
.66949 .33675 L
.67914 .33287 L
.68842 .32894 L
.6983 .32445 L
.70781 .3197 L
.71695 .31454 L
.72668 .30816 L
.73605 .30072 L
.74601 .29057 L
.75098 .28422 L
.75561 .27717 L
.76035 .26832 L
.76484 .25789 L
.76979 .24269 L
.77429 .22371 L
.77664 .21047 L
.77914 .19304 L
.78126 .17392 L
.78361 .14596 L
.78487 .12686 L
.78606 .10445 L
.78715 .07962 L
.78831 .04669 L
s
.78831 .04669 m
.78957 0 L
s
.79735 0 m
.79735 .61803 L
s
.80599 .61803 m
.807 .58628 L
.8082 .55796 L
.81036 .51955 L
.81268 .48971 L
.81509 .46706 L
.81731 .45077 L
.81991 .43568 L
.82235 .42432 L
.82689 .40773 L
.83188 .39416 L
.83679 .38385 L
.84128 .37621 L
.8509 .3634 L
.86015 .35394 L
.87 .34576 L
.87948 .33894 L
.88955 .33242 L
.89926 .32661 L
.9086 .32121 L
.91853 .3156 L
.9281 .31013 L
.9373 .30476 L
.94709 .29871 L
.95652 .29247 L
.96654 .28517 L
.97619 .27718 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{349.563, 216.063},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40008<0001A2000`400?l00000o`00003oo`3ooooo
0?ooo`h0oooo003o0?ooool0oooo3P3oool00?l0ooooo`3oool>0?ooo`00o`3oool_0?ooo`030?oo
g`2LB000000000<0000000<0B9cO0?ooo`3OW4P00P0000000`1d_ol0oooo0?ooo`3@0?ooo`00o`3o
ool_0?ooo`030;md0000B9`0gooo00<0oooo00<0gibL0;ooo`3oool00P3oool00`2oM7@0_ooo0?oo
o`3@0?ooo`00o`3oool_0?ooo`030?oog`2LM9`0gooo00<0oooo00<0oono09bLg`3oool00P3oool0
0`3ogi`0M9cO0?ooo`3@0?ooo`00o`3oool`0?ooo`030;md00000000000000<0000000L0B9cO0?oo
o`3oool0oooo0?oog`2LB7@0_ooo0=00oooo003o0?oooa80oooo00L0_g@0000000000000001d0;oo
o`3oool0_g@00080000000<0001d0;oog`2LB0000P0000001000B9`0gooo0?ooo`2oM00200000004
0000M02oool0oooo0;md0080000000D0001809cOo`3oool0oono07@00004000000030000M02oool0
oooo00<0oooo00<0_g@007Boo`3oool0c`3oool00?l0oooo4@3oool03`2oM0000000001d_`3oool0
omnL04P0M02oool0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`0;0;md00000000
07Bo0?ooo`3ogi`0B01d0;ooo`3oool0gia80000M02oool00`3oool00`2oM4P0W=oo0?ooo`070?oo
o`040?oOW018000004RL0=oooll0oooo003o0?oooa00oooo00<0omnL04P00018W=l00`3oool03P3o
gi`0M9cO0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?oOW0180000B9cO0`3oool0
1P3ogi`0M9cO0?ooo`3OW4P0001d0;ooo`<0oooo00D0oono07@0000000000000001d_`050?ooo`04
0;md0000B9`0W7@007Booll0oooo003o0?oooa00oooo00<0okmd0000B02Lgol01P3oool02`3OW4P0
04RL0=ooo`3oool0oooO09a80000M;l0oooo0?noM00004P0W=oo00H0oooo00<0gia80000M02oool0
0`3oool01P3OW4P0001d0;ooo`3ookl0M00004RLg`<0oooo00D0okmd0000001d_ol0omnL04Qd_`3?
0?ooo`00o`3oool@0?ooo`030?noM00007@0_ooo00H0oooo00/0gia80018W03Oool0oooo0?oog`2L
B00007Bo0?ooo`3o_g@0001d0;ooo`060?ooo`030=nLB00007@0_ooo0080oooo00L0oono07@00018
W=l0oooo0?oog`2LB00007Bo0080oooo00L0oooO09a80000B9`0gooo0?ooo`2oM4P0W=oo0<h0oooo
003o0?oooa00oooo00<0omnL04P000000000100000003018W=l0oooo0;md000007@0_ooo0?ooo`3o
oml0W4P0001d_`3oool0omnL04P000D0000000D0B9cO0?ooo`3OW4P0001809cOo`020?ooo`070?oo
_`1d0000B9cO0?ooo`3ooml0W4P004RLg`020?ooo`030?noM00004P0W=oo0080oooo00<0okmd04RL
g`3oool0c@3oool00?l0oooo4@3oool03`2oM000B9cO0?ooo`3oool0gia80000B02Lgml0W4P00000
0000B4P0B01809cOo`3ookl0M00004RLg`020?ooo`0B0;md0018W=l0oooo0?ooo`3OW4P0001809cO
g`2LB00000000000B018M9`0gkmd0000M02oool0_g@0001d_`3oool0okmd0P0000001@00B9`0gono
07@00000B9`0gooo0080oooo00<0oono07@0M02oool0c@3oool00?l0oooo4P3oool01@3OW4P00000
0000000004P0W=oo0080oooo00L0_g@00018W03Ooml0W4P00000000007@0_ooo00<0oooo00D0gia8
000000000000001809cOo`020?ooo`080;md000007@0_mnL04P0000007@0_ooo0?oog`2LB0020000
000407Boo`3oool0oooO09a80080000000L004RL0=ooo`3oool0_g@000000000B9`0gooo0<`0oooo
003o0?ooool0oooo3P3oool00?l0ooooo`3oool>0?ooo`00o`3ooooo0?ooo`h0oooo003o0?ooool0
oooo3P3oool00?l0ooooo`3oool>0?ooo`00o`3ooooo0?ooo`h0oooo003o0?ooool0oooo3P3oool0
0?l0ooooo`3oool>0?ooo`00o`3ooooo0?ooo`h0oooo003o0?ooool0oooo3P3oool00?l0ooooo`3o
ool>0?ooo`00o`3ooooo0?ooo`h0oooo003o0?ooool0oooo3P3oool00?l0ooooo`3oool>0?ooo`00
o`3ooooo0?ooo`h0oooo003o0?ooool0oooo3P3oool00840oooo00<0oono07@0000000000P000000
0`1d_ol0oooo0?ooo`190?ooo`030?oo_`1d000000000080000000<0M;oo0?ooo`3oool00P3oool0
1P3o_g@0000007Boo`3oool0oooO09a800<0000000<0M;oo0?ooo`3oool0B`3oool0103ooml0W4P0
0018W03Ooom<0?ooo`040?oog`2LB00004RL0=ooo`80oooo00H0okmd0000001d_ol0oooo0?oog`2L
B0030000000307Boo`3oool0oooo04P0oooo00<0oooO09a8000000000P0000000`1d_ol0oooo0?oo
o`0W0?ooo`00P@3oool00`3ogi`0B00004RLg`020?ooo`030;md0018W=l0oooo04T0oooo00<0omnL
04P00018W=l00P3oool00`2oM000B9cO0?ooo`020?ooo`070?noM0000000M;oo0?ooo`3ooml0W4Qd
0;ooo`020?ooo`030;md001d_ol0oooo04/0oooo00@0oooO09a80000B9`0goooC03oool0103ooml0
W4P00018W03Oool20?ooo`070?noM0000000M;oo0?ooo`3ooml0W4Qd0;ooo`020?ooo`030;md001d
_ol0oooo04P0oooo00<0oooO09a8M02oool00P3oool00`2oM000M;oo0?ooo`0W0?ooo`00QP3oool0
0`3ookl0M01809cOo`1>0?ooo`030?oo_`1d04P0W=oo00/0oooo00<0oooO09a8M02oool0B`3oool0
103ooml0W4P00018W03Ooom<0?ooo`040?oog`2LB00004RL0=ooo`/0oooo00<0oooO09a8M02oool0
C@3oool00`3ooml0W4Qd0;ooo`0W0?ooo`00QP3oool00`3ookl0M00007Boo`1>0?ooo`030?oo_`1d
0000M;oo00`0oooo00<0_g@007Boo`3oool0AP3oool00`2oM0000000000000050000000307Boo`3o
ool0oooo04D0oooo00<0_g@00000000000001@0000000`1d_ol0oooo0?ooo`090?ooo`030;md001d
_ol0oooo04d0oooo00<0_g@007Boo`3oool09P3oool008H0oooo00<0okmd0000001d_ol0CP3oool0
0`3o_g@0000007Boo`0;0?ooo`030?oog`2LB000B9cO04L0oooo00P0omnL07BLg`3oool0oooo0?oo
g`2LB00004RL0=ooodP0oooo00P0omnL07BLg`3oool0oooo0?oog`2LB00004RL0=ooo`/0oooo00<0
oooO09a80018W=l0C@3oool00`3ooml0W4P004RLg`0W0?ooo`00Q@3oool0103ogi`0B0000000B02L
gom=0?ooo`040?oOW0180000001809cOo`/0oooo00<0gia80000001d_ol0B03oool01`3OW4P0M;oo
0?ooo`3ooml0W4P00018W03Oool0B@3oool01`3OW4P0M;oo0?ooo`3ooml0W4P00018W03Oool02`3o
ool00`3OW4P0000007Boo`1=0?ooo`030=nLB0000000M;oo02L0oooo00230?ooo`050?noM0000000
00000000B02Lgol0C03oool01@3o_g@000000000000004P0W=oo00P0oooo00L0gmoO0?ooo`3ookl0
M0000000000007@0_ooo04P0oooo00L0oooO09a8M02oool0oooO09a80000B9`0gooo04T0oooo00L0
oooO09a8M02oool0oooO09a80000B9`0gooo00L0oooo00L0gmoO0?ooo`3ookl0M0000000000007@0
_ooo04T0oooo00L0gmoO0?ooo`3ookl0M0000000000007@0_ooo02L0oooo00250?ooo`030;md0000
M;l0oooo04h0oooo00<0_g@0001d_`3oool0203oool01P3OW4P00000000000000000001809cOodX0
oooo00H0omnL04Qd_`3ooml0W4P00018W03Ooom:0?ooo`060?oOW018M;l0oooO09a80000B9`0gooo
1`3oool01P3OW4P00000000000000000001809cOodX0oooo00H0gia800000000000000000000B02L
golX0?ooo`00QP3oool00`2oM000B9cO0?ooo`1>0?ooo`030;md0018W=l0oooo00L0oooo00@0oono
07@000000000B9cOC@3oool01@3OW4P0M;oO09a80000B9`0gooo04/0oooo00D0gia807Bog`2LB000
04RL0=ooo`070?ooo`040?oo_`1d0000000004RLgd`0oooo00@0oono07@000000000B9cO:P3oool0
0840oooo00<0oooO09bL_`3oool00P3oool00`3OW4P0001d0;ooo`190?ooo`030?oog`2LW;l0oooo
0080oooo00<0gia80000M02oool0203oool00`2oM000M;oo0?ooo`1=0?ooo`050?oog`2LB4P0B4P0
0018W03Oool0B`3oool01@3ooml0W4Q804Q80000B9`0gooo00P0oooo00<0_g@007Boo`3oool0C@3o
ool00`2oM000M;oo0?ooo`0Z0?ooo`00PP3oool01P3o_g@0B9cO0?ooo`3ookl0M000001d_d/0oooo
00H0okmd04RLg`3oool0oono07@00000M;l90?ooo`060?oOW018000000000000000004P0W=ooB`3o
ool0103ogi`0B0000018W03Ooom<0?ooo`040?oOW018000004RL0=ooo`P0oooo00H0omnL04P00000
000000000000B02Lgom:0?ooo`060?oOW018000000000000000004P0W=oo9`3oool008<0oooo00D0
gia8000000000000001d0;ooo`1<0?ooo`050=nLB000000000000000M02oool02P3oool01@2oM000
0000000000000000B9cO04`0oooo00<0gia80018W03Oool0C@3oool00`3OW4P004RL0=ooo`090?oo
o`050;md00000000000000000018W=l0B`3oool01@2oM0000000000000000000B9cO02L0oooo003o
0?ooool0oooo3P3oool00?l0ooooo`3oool>0?ooo`00o`3ooooo0?ooo`h0oooo003o0?ooool0oooo
3P3oool00?l0ooooo`3oool>0?ooo`00o`3ooooo0?ooo`h0oooo003o0?ooool0oooo3P3oool00?l0
ooooo`3oool>0?ooo`00o`3ooooo0?ooo`h0oooo001R0?ooool00000R`00000P0?ooo`00HP3oool2
000000l0oooo0P00000A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03`3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03`3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03`3oool0
10000000oooo00000000000=0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0200000140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0N0?ooo`00HP3oool2000000l0oooo0P00000A0?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool03`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool03`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool03`3oool010000000oooo00000000000=0?ooo`030000003oool0oooo00l0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00l0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`0200000140oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool0B`3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo05@0oooo00@000000?ooo`0000000000D`3oool00`000000oooo0?ooo`0:0?ooo`040000003o
ool0oooo000004H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0N0?ooo`00HP3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0G`3o
ool200000600oooo00@000000?ooo`3oool00000C`3oool00`000000oooo0?ooo`0N0?ooo`00HP3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0G`3o
ool200000600oooo00@000000?ooo`3oool00000C`3oool00`000000oooo0?ooo`0N0?ooo`00HP3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0G`3o
ool200000600oooo00@000000?ooo`3oool00000C`3oool00`000000oooo0?ooo`0N0?ooo`00HP3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0G`3o
ool200000600oooo00@000000?ooo`3oool00000C`3oool00`000000oooo0?ooo`0N0?ooo`00HP3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0G`3o
ool200000600oooo00@000000?ooo`3oool00000C`3oool00`000000oooo0?ooo`0N0?ooo`00HP3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0G`3o
ool200000600oooo00@000000?ooo`3oool00000C`3oool00`000000oooo0?ooo`0N0?ooo`00HP3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05d0oooo0P00001Q0?ooo`800000H03o
ool010000000oooo0?ooo`00001?0?ooo`030000003oool0oooo01h0oooo001R0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0G@3oool200000640oooo0P00001P0?ooo`040000003o
ool0oooo000004l0oooo00<000000?ooo`3oool07P3oool00680oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`1M0?ooo`800000H03oool00`000000oooo0000001P0?ooo`040000003o
ool0oooo000004l0oooo00<000000?ooo`3oool07P3oool00680oooo0`00000>0?ooo`030000003o
ool0oooo05d0oooo0P00001P0?ooo`030000003oool000000600oooo00@000000?ooo`3oool00000
CP3oool200000200oooo001R0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0G@3o
ool200000600oooo00<000000?ooo`000000H03oool010000000oooo0?ooo`00001?0?ooo`030000
003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0G@3o
ool200000600oooo00<000000?ooo`000000H03oool010000000oooo0?ooo`00001?0?ooo`030000
003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0G@3o
ool200000600oooo00<000000?ooo`000000H03oool010000000oooo0?ooo`00001?0?ooo`030000
003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0G@3o
ool200000600oooo00<000000?ooo`000000H03oool010000000oooo0?ooo`00001?0?ooo`030000
003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0G@3o
ool200000600oooo00<000000?ooo`000000H03oool010000000oooo0?ooo`00001?0?ooo`030000
003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0G@3o
ool200000600oooo00<000000?ooo`000000H03oool010000000oooo0?ooo`00001?0?ooo`030000
003oool0oooo01h0oooo001=0?ooo`040?oog`2LB00004RL0=ooo`<0oooo00D0omnL04P000000000
001d0;ooo`090?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0G@3oool200000600
oooo00<000000?ooo`000000H03oool010000000oooo0?ooo`00001?0?ooo`030000003oool0oooo
01h0oooo001=0?ooo`040?oog`2LB00004RL0=ooo`80oooo00L0oono07@0001d_ol0oooo0?noM000
07@0_ooo00P0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1M0?ooo`800000H03o
ool00`000000oooo0000001P0?ooo`040000003oool0oooo000004l0oooo00<000000?ooo`3oool0
7P3oool004d0oooo00@0oooO09a80000B9`0gooo0P3oool00`2oM00004RL0=ooo`020?ooo`030;md
0000M;l0oooo00L0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1M0?ooo`800000
H03oool00`000000oooo0000001P0?ooo`040000003oool0oooo000004l0oooo00<000000?ooo`3o
ool07P3oool004T0oooo00<0_g@00000000000001@000000101d_ol0oono07@00018W=l30?ooo`03
0=nLB00007@0_ooo00L0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1M0?ooo`80
0000H03oool00`000000oooo0000001P0?ooo`040000003oool0oooo000004l0oooo00<000000?oo
o`3oool07P3oool003`0oooo00<0oooO09a8000000001`0000001@18W=l0oooo0?ooo`3ogi`0M9cO
0080oooo00P0oooO09a80000B9`0gooo0?ooo`3ogi`0B00007Boo`<0oooo00<0okmd0000B02Lgol0
1`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05d0oooo0P00001P0?ooo`030000
003oool000000600oooo00@000000?ooo`3oool00000C`3oool00`000000oooo0?ooo`0N0?ooo`00
BP3oool02`3OW4P0M;oo0?ooo`3ooml0W4P00018W03Oool0oooo0?oOW0180000M;oo00<0oooo00<0
okmd0000B02Lgol01`3oool4000000d0oooo00<000000?ooo`3oool0G@3oool200000600oooo00<0
00000?ooo`000000H03oool010000000oooo0?ooo`00001=0?ooo`<00000803oool004X0oooo00/0
oooO09a8M02oool0oooO09a80000B9`0gooo0?ooo`3ogi`0B00007Boo`030?ooo`030?noM00004P0
W=oo00L0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1M0?ooo`800000H03oool0
0`000000oooo0000001O0?ooo`050000003oool0oooo0?ooo`000000C`3oool00`000000oooo0?oo
o`0N0?ooo`00B`3oool02P3ogi`0B7Bo0?oog`2LB00004RL0=ooo`3oool0omnL04P0001d_ol30?oo
o`030?noM00004P0W=oo00L0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1M0?oo
o`800000H03oool00`000000oooo0000001O0?ooo`050000003oool0oooo0?ooo`000000C`3oool0
0`000000oooo0?ooo`0N0?ooo`00C03oool02@3OW4P0M;oO09a80000B9`0gooo0?ooo`3ookl0M000
04RLg`030?ooo`030=nLB00007@0_ooo00L0oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`1M0?ooo`800000H03oool00`000000oooo0000001O0?ooo`050000003oool0oooo0?ooo`00
0000C`3oool00`000000oooo0?ooo`0N0?ooo`00C03oool01@3ooml0W4Q804Q80000B9`0gooo0080
oooo00<0gia80018W03Oool00P3oool00`2oM00007Bo0?ooo`070?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool0G@3oool200000600oooo00<000000?ooo`000000G`3oool01@000000
oooo0?ooo`3oool0000004l0oooo00<000000?ooo`3oool07P3oool004d0oooo00@0omnL04P00000
B9`0gooo0P3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool0203oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo05d0oooo0P00001P0?ooo`030000003oool0000005l0oooo00D0
00000?ooo`3oool0oooo0000001?0?ooo`030000003oool0oooo01h0oooo001>0?ooo`030=nLB000
B9`0gooo00<0oooo00D0oono07@000000000001d0;ooo`090?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool0G@3oool200000600oooo00<000000?ooo`000000G`3oool01@000000oooo
0?ooo`3oool0000004l0oooo00<000000?ooo`3oool07P3oool00680oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`1M0?ooo`800000H03oool00`000000oooo0000001O0?ooo`050000
003oool0oooo0?ooo`000000C`3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo05d0oooo0P00001P0?ooo`030000003oool0000005l0oooo
00D000000?ooo`3oool0oooo0000001?0?ooo`030000003oool0oooo01h0oooo001R0?ooo`030000
003oool0oooo00d0oooo0P00001O0?ooo`800000H03oool00`000000oooo0000001O0?ooo`050000
003oool0oooo0?ooo`000000C`3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo
0?ooo`0=0?ooo`800000G`3oool200000600oooo00<000000?ooo`000000G`3oool01@000000oooo
0?ooo`3oool0000004l0oooo00<000000?ooo`3oool07P3oool00680oooo00<000000?ooo`3oool0
3@3oool2000005l0oooo0P00001P0?ooo`030000003oool0000005l0oooo00D000000?ooo`3oool0
oooo0000001?0?ooo`030000003oool0oooo01h0oooo001R0?ooo`<000003@3oool2000005l0oooo
0P00001P0?ooo`030000003oool0000005l0oooo00D000000?ooo`3oool0oooo0000001>0?ooo`80
0000803oool00680oooo00<000000?ooo`3oool03@3oool2000005l0oooo0P00001P0?ooo`030000
003oool0000005l0oooo00D000000?ooo`3oool0oooo0000001?0?ooo`030000003oool0oooo01h0
oooo001R0?ooo`030000003oool0oooo00d0oooo0P00001O0?ooo`800000H03oool00`000000oooo
0000001O0?ooo`050000003oool0oooo0?ooo`000000C`3oool00`000000oooo0?ooo`0N0?ooo`00
HP3oool00`000000oooo0?ooo`0=0?ooo`800000G`3oool200000600oooo00<000000?ooo`000000
G`3oool01@000000oooo0?ooo`3oool0000004l0oooo00<000000?ooo`3oool07P3oool00680oooo
00<000000?ooo`3oool03@3oool2000005l0oooo0P00001P0?ooo`030000003oool0000005l0oooo
00D000000?ooo`3oool0oooo0000001?0?ooo`030000003oool0oooo01h0oooo001R0?ooo`030000
003oool0oooo00d0oooo0P00001O0?ooo`800000H03oool00`000000oooo0000001O0?ooo`050000
003oool0oooo0?ooo`000000C`3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo
0?ooo`0=0?ooo`800000G`3oool200000600oooo00<000000?ooo`000000G`3oool01@000000oooo
0?ooo`3oool0000004l0oooo00<000000?ooo`3oool07P3oool00680oooo00<000000?ooo`3oool0
3@3oool2000005l0oooo0P00001P0?ooo`030000003oool0000005l0oooo00D000000?ooo`3oool0
oooo0000001?0?ooo`030000003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo00d0oooo
0P00001O0?ooo`800000H03oool00`000000oooo0000001O0?ooo`050000003oool0oooo0?ooo`00
0000C`3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo0?ooo`0=0?ooo`800000
G`3oool200000600oooo00<000000?ooo`000000G`3oool01@000000oooo0?ooo`3oool0000004l0
oooo00<000000?ooo`3oool07P3oool00680oooo00<000000?ooo`3oool03@3oool2000005l0oooo
0P00001P0?ooo`030000003oool0000005l0oooo00D000000?ooo`3oool0oooo0000001?0?ooo`03
0000003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo00d0oooo0P00001O0?ooo`800000
H03oool00`000000oooo0000001O0?ooo`050000003oool0oooo0?ooo`000000C`3oool00`000000
oooo0?ooo`0N0?ooo`00HP3oool3000000d0oooo0P00001O0?ooo`800000H03oool00`000000oooo
0000001O0?ooo`050000003oool0oooo0?ooo`000000CP3oool200000200oooo001R0?ooo`030000
003oool0oooo00d0oooo0P00001O0?ooo`800000H03oool00`000000oooo0000001O0?ooo`050000
003oool0oooo0?ooo`000000C`3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo
0?ooo`0=0?ooo`800000G`3oool200000600oooo00<000000?ooo`000000G`3oool01@000000oooo
0?ooo`3oool0000004l0oooo00<000000?ooo`3oool07P3oool00680oooo00<000000?ooo`3oool0
3@3oool2000005l0oooo0P00001P0?ooo`030000003oool0000005l0oooo00D000000?ooo`3oool0
oooo0000001?0?ooo`030000003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo00d0oooo
0P00001O0?ooo`800000H03oool00`000000oooo0000001O0?ooo`050000003oool0oooo0?ooo`00
0000C`3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo0?ooo`0=0?ooo`800000
G`3oool200000600oooo00<000000?ooo`000000GP3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo04d0oooo00<000000?ooo`3oool07P3oool00680oooo00<000000?ooo`3oool03@3o
ool2000005l0oooo0P00001P0?ooo`030000003oool0000005h0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`1=0?ooo`030000003oool0oooo01h0oooo001R0?ooo`030000003oool0
oooo00d0oooo0P00001O0?ooo`800000H03oool00`000000oooo0000001N0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`00
0000oooo0?ooo`0=0?ooo`800000G`3oool2000005l0oooo00@000000?ooo`3oool00000GP3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool07P3oool0
0680oooo00<000000?ooo`3oool03@3oool2000005l0oooo0P00001O0?ooo`040000003oool0oooo
000005h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1=0?ooo`030000003oool0
oooo01h0oooo001R0?ooo`030000003oool0oooo00d0oooo0P00001O0?ooo`800000G`3oool01000
0000oooo0?ooo`00001N0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0C@3oool0
0`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo0?ooo`0=0?ooo`800000G`3oool20000
05l0oooo00@000000?ooo`3oool00000GP3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo04d0oooo00<000000?ooo`3oool07P3oool00680oooo0`00000=0?ooo`800000G`3oool20000
05l0oooo00@000000?ooo`3oool00000GP3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo04`0oooo0P00000P0?ooo`00HP3oool00`000000oooo0?ooo`0=0?ooo`800000G`3oool20000
05l0oooo00@000000?ooo`3oool00000GP3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo04d0oooo00<000000?ooo`3oool07P3oool00680oooo00<000000?ooo`3oool03@3oool20000
05l0oooo0P00001O0?ooo`040000003oool0oooo000005h0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`1=0?ooo`030000003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo
00d0oooo0P00001O0?ooo`800000G`3oool010000000oooo0?ooo`00001M0?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`00
0000oooo0?ooo`0=0?ooo`800000G`3oool2000005l0oooo00@000000?ooo`3oool00000G@3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool07P3oool0
0680oooo00<000000?ooo`3oool03@3oool2000005l0oooo0P00001O0?ooo`040000003oool0oooo
000005d0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1=0?ooo`030000003oool0
oooo01h0oooo001R0?ooo`030000003oool0oooo00d0oooo0P00001O0?ooo`800000G`3oool01000
0000oooo0?ooo`00001M0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0C@3oool0
0`000000oooo0?ooo`0N0?ooo`00B@3oool00`2oM0000000000000040000000904RLg`3oool0oooo
0?ooo`3ogi`0B0000000000007@0_ooo00T0oooo00<000000?ooo`3oool03@3oool2000005l0oooo
0P00001O0?ooo`040000003oool0oooo000005d0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`1=0?ooo`030000003oool0oooo01h0oooo001:0?ooo`030=nLB0000000000000<00000
00X004RL0=ooo`3oool0oono07@0001d_ol0oooo0?noM00007@0_ooo203oool00`000000oooo0?oo
o`0=0?ooo`800000G`3oool2000005l0oooo00@000000?ooo`3oool00000G@3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool07P3oool004/0oooo00<0
gia804RLg`3oool00P3oool01P3OW7@0W=oo0?ooo`2oM00004RL0=ooo`80oooo00<0_g@0001d_`3o
ool01`3oool00`000000oooo0?ooo`0=0?ooo`800000G`3oool2000005l0oooo00@000000?ooo`3o
ool00000G@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo04d0oooo00<000000?oo
o`3oool07P3oool004`0oooo00<0gia807Boo`3oool00`3oool00`3ookl0M00004RLg`030?ooo`03
0=nLB00007@0_ooo00L0oooo00<000000?ooo`3oool03@3oool2000005l0oooo0P00001O0?ooo`04
0000003oool0oooo000005d0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1=0?oo
o`030000003oool0oooo01h0oooo000l0?ooo`030?oog`2LB000000000L0000000<0B9cO0?ooo`3o
ool0103oool00`2oM4P0W=oo0?ooo`020?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02L
gol01`3oool00`000000oooo0?ooo`0=0?ooo`800000G`3oool2000005l0oooo00@000000?ooo`3o
ool00000G@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo04d0oooo00<000000?oo
o`3oool07P3oool004d0oooo00<0oooO09a8M02oool00P3oool00`3ogi`0B00007Boo`030?ooo`03
0?noM00004P0W=oo00L0oooo1000000<0?ooo`800000G`3oool2000005l0oooo00@000000?ooo`3o
ool00000G03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo04/0oooo0`00000P0?oo
o`00CP3oool01`3o_g@007Bo0?ooo`3oool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`07
0?ooo`030000003oool0oooo00d0oooo0P00001O0?ooo`800000G`3oool010000000oooo0?ooo`00
001L0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?oo
o`0N0?ooo`00CP3oool01`3ookl0M01809cOo`3oool0omnL04P0001d_ol00`3oool00`3o_g@00018
09cOo`070?ooo`030000003oool0oooo00d0oooo0P00001O0?ooo`800000G`3oool010000000oooo
0?ooo`00001L0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0C@3oool00`000000
oooo0?ooo`0N0?ooo`00CP3oool01`3ookl0M00004RLg`3oool0oono07@00018W=l00`3oool00`3O
W4P0001d0;ooo`070?ooo`030000003oool0oooo00d0oooo0P00001O0?ooo`800000GP3oool01@00
0000oooo0?ooo`3oool0000005`0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`1=
0?ooo`030000003oool0oooo01h0oooo00190?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P0
0018W=l00P3oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00L0oooo00<000000?oo
o`3oool03@3oool2000005l0oooo0P00001N0?ooo`050000003oool0oooo0?ooo`000000G03oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool07P3oool0
04T0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`80oooo00L0oono07@0001d_ol0
oooo0?noM00007@0_ooo00P0oooo00<000000?ooo`3oool03@3oool2000005l0oooo0P00001N0?oo
o`050000003oool0oooo0?ooo`000000G03oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo04d0oooo00<000000?ooo`3oool07P3oool004X0oooo00H0oooO09a80000000000000000M02o
ool40?ooo`050?oo_`1d000000000000M02oool02@3oool00`000000oooo0?ooo`0=0?ooo`800000
G`3oool2000005h0oooo00D000000?ooo`3oool0oooo0000001L0?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo
0?ooo`0=0?ooo`800000G`3oool2000005h0oooo00D000000?ooo`3oool0oooo0000001K0?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`0N0?ooo`00
HP3oool00`000000oooo0?ooo`0=0?ooo`800000G`3oool2000005h0oooo00D000000?ooo`3oool0
oooo0000001K0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0C@3oool00`000000
oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo0?ooo`0=0?ooo`800000G`3oool2000005h0oooo
00D000000?ooo`3oool0oooo0000001K0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0C@3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo0?ooo`0=0?ooo`800000
G`3oool2000005h0oooo00D000000?ooo`3oool0oooo0000001K0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo
0?ooo`0=0?ooo`800000G`3oool2000005h0oooo00D000000?ooo`3oool0oooo0000001K0?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`0N0?ooo`00
HP3oool3000000d0oooo0P00001O0?ooo`800000GP3oool01@000000oooo0?ooo`3oool0000005/0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`1<0?ooo`800000803oool00680oooo
00<000000?ooo`3oool03@3oool2000005l0oooo0P00001N0?ooo`050000003oool0oooo0?ooo`00
0000FP3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3o
ool07P3oool00680oooo00<000000?ooo`3oool03@3oool2000005l0oooo0P00001N0?ooo`050000
003oool0oooo0?ooo`000000FP3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo04d0
oooo00<000000?ooo`3oool07P3oool002T0oooo00<0oono07Ad_`3oool0103oool00`3OW9`0_ooo
0?ooo`0_0?ooo`030000003oool0oooo00d0oooo0P00001O0?ooo`800000G@3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo05P0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`1=0?ooo`030000003oool0oooo01h0oooo000Y0?ooo`030?noM0000000000000@0000000<0
001d0;ooo`3oool0;`3oool00`000000oooo0?ooo`0=0?ooo`800000G`3oool2000005d0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool0C@3oool00`000000oooo0?ooo`0N0?ooo`00:@3oool00`2oM000000000000004
000000030000M02oool0oooo02l0oooo00<000000?ooo`3oool03@3oool2000005l0oooo0P00001M
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0F03oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool07P3oool002X0oooo00<0_gAd0;oo
o`3oool00`3oool00`3OW9`0_ooo0?ooo`0_0?ooo`030000003oool0oooo00d0oooo0P00001O0?oo
o`800000G@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo05P0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`1=0?ooo`030000003oool0oooo01h0oooo000Y0?ooo`03
0?oOW02L_ol0oooo03H0oooo00<000000?ooo`3oool03@3oool2000005l0oooo0P00001M0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0E`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo04d0oooo00<000000?ooo`3oool07P3oool002T0oooo00<0_g@0001d_`3oool0
103oool00`3OW9`0_ooo0?ooo`0_0?ooo`030000003oool0oooo00d0oooo0P00001O0?ooo`800000
G@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo05L0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`1=0?ooo`030000003oool0oooo01h0oooo000Y0?ooo`030=nLB000
0000000000@0000000<0001d0;ooo`3oool0;`3oool00`000000oooo0?ooo`0=0?ooo`800000G`3o
ool2000005d0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1G0?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`0N0?ooo`00:@3oool0
0`3ooml0W4P000000004000000030000M02oool0oooo02l0oooo00<000000?ooo`3oool03@3oool2
000005l0oooo0P00001M0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0E`3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool07P3oool0
02X0oooo00<0gibL0;ooo`3oool00`3oool00`3OW9`0_ooo0?ooo`0_0?ooo`030000003oool0oooo
00d0oooo0P00001N0?ooo`030000003oool0000005d0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`1G0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0C@3oool00`00
0000oooo0?ooo`0N0?ooo`00:@3oool00`3o_g@0B9cO0?ooo`0f0?ooo`<000003@3oool2000005h0
oooo00<000000?ooo`000000G03oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo05L0
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`1<0?ooo`800000803oool002T0oooo
00<0_g@0001d_`3oool0103oool00`3OW9`0_ooo0?ooo`0_0?ooo`030000003oool0oooo00d0oooo
0P00001N0?ooo`030000003oool0000005`0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`1F0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0C@3oool00`000000oooo
0?ooo`0N0?ooo`00:@3oool00`3OW4P0000000000004000000030000M02oool0oooo02l0oooo00<0
00000?ooo`3oool03@3oool2000005h0oooo00<000000?ooo`000000G03oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo05H0oooo00<000000?ooo`3oool0203oool00`000000oooo0?oo
o`1=0?ooo`030000003oool0oooo01h0oooo000Y0?ooo`030?oog`2LB000000000@0000000<0001d
0;ooo`3oool0;`3oool00`000000oooo0?ooo`0=0?ooo`800000GP3oool00`000000oooo0000001L
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0EP3oool00`000000oooo0?ooo`08
0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool07P3oool00300oooo00<0gibL0;oo
o`3oool0;`3oool00`000000oooo0?ooo`0=0?ooo`800000GP3oool00`000000oooo0000001L0?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool0E@3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo04d0oooo00<000000?ooo`3oool07P3oool002T0oooo00<0oono07Ad_`3o
ool0103oool00`3OW9`0_ooo0?ooo`0_0?ooo`030000003oool0oooo00d0oooo0P00001N0?ooo`03
0000003oool0000005`0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1E0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`0N0?ooo`00
8`3oool00`3ooml0W4P0001d_`030?ooo`030?noM0000000000000@0000000<0001d0;ooo`3oool0
;`3oool00`000000oooo0?ooo`0=0?ooo`800000GP3oool00`000000oooo0000001L0?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool0E03oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo04d0oooo00<000000?ooo`3oool07P3oool002<0oooo00<0oooO09a80000M;l00`3o
ool00`2oM000000000000004000000030000M02oool0oooo02l0oooo00<000000?ooo`3oool03@3o
ool2000005h0oooo00<000000?ooo`000000G03oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo05@0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`1=0?ooo`030000003o
ool0oooo01h0oooo000`0?ooo`030=nLW02oool0oooo02l0oooo00<000000?ooo`3oool03@3oool2
000005h0oooo00<000000?ooo`000000G03oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo05<0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`1=0?ooo`030000003oool0
oooo01h0oooo001R0?ooo`030000003oool0oooo00d0oooo0P00001N0?ooo`030000003oool00000
05/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`1C0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000
oooo0?ooo`0=0?ooo`800000GP3oool00`000000oooo0000001K0?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo04d0
oooo00<000000?ooo`3oool07P3oool002T0oooo00T0oooO09a800000000001d0;oog`2LB0000000
0000M02oool0<03oool00`000000oooo0?ooo`0=0?ooo`800000GP3oool00`000000oooo0000001K
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool07P3oool002T0oooo00P0giad09cO
o`3ogi`0B000001d_`3oool0okmd04RLgc40oooo00<000000?ooo`3oool03@3oool2000005h0oooo
00<000000?ooo`000000F`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0540oooo
00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool07P3oool002T0oooo00T0gibL0;ooo`3oool0_g@00000B02Lgol0oooo0;md
M02oool0<03oool3000000d0oooo0P00001N0?ooo`030000003oool0000005X0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`1@0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool0@`3oool00`000000oooo0?ooo`060?ooo`800000803oool002T0oooo00T0omnL04Qd_`3o
ool0omnL04P00000M;l0oooo0;nLW03Oool0<03oool00`000000oooo0?ooo`0=0?ooo`800000GP3o
ool00`000000oooo0000001J0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0D03o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0440oooo0P00000:0?ooo`030000003o
ool0oooo01h0oooo000Y0?ooo`050;md00000000001809cOo`3OW4P00P0000000`1d_ol0oooo0?oo
o`0_0?ooo`030000003oool0oooo00d0oooo0P00001N0?ooo`030000003oool0000005X0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`1?0?ooo`030000003oool0oooo00l0oooo00<0
00000?ooo`3oool0@03oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo01h0oooo001R
0?ooo`030000003oool0oooo00d0oooo0P00001N0?ooo`030000003oool0000005X0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0?`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo01h0oooo000Y0?oo
o`030?oo_`1dM;l0oooo00@0oooo00<0gibL0;ooo`3oool0;`3oool00`000000oooo0?ooo`0=0?oo
o`800000GP3oool00`000000oooo0000001I0?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool0C@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo03d0oooo0P00000>0?oo
o`030000003oool0oooo01h0oooo000Y0?ooo`030?noM0000000000000@0000000<0001d0;ooo`3o
ool0;`3oool00`000000oooo0?ooo`0=0?ooo`800000G@3oool010000000oooo0?ooo`00001I0?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool0C03oool00`000000oooo0?ooo`0B0?oo
o`030000003oool0oooo03`0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0N0?oo
o`00:@3oool00`2oM000000000000004000000030000M02oool0oooo02l0oooo00<000000?ooo`3o
ool03@3oool2000005d0oooo00@000000?ooo`3oool00000F03oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo04/0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0k0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool07P3oool002X0oooo00<0_gAd0;ooo`3o
ool00`3oool00`3OW9`0_ooo0?ooo`0Q0?ooo`050?oOW018000000000000M02oool02@3oool00`00
0000oooo0?ooo`0=0?ooo`800000G@3oool010000000oooo0?ooo`00001H0?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo03T0oooo0P00000B0?ooo`030000003oool0oooo01h0oooo000Y0?ooo`030?oOW02L_ol0oooo
02L0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00P0oooo00<000000?ooo`3oool03@3o
ool2000005d0oooo00@000000?ooo`3oool00000E`3oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo04T0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0h0?ooo`030000
003oool0oooo0180oooo00<000000?ooo`3oool07P3oool002T0oooo00<0_g@0001d_`3oool0103o
ool00`3OW9`0_ooo0?ooo`0P0?ooo`030;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool01`3o
ool00`000000oooo0?ooo`0=0?ooo`800000G@3oool010000000oooo0?ooo`00001G0?ooo`030000
003oool0oooo00P0oooo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`0F0?ooo`030000
003oool0oooo03H0oooo0P00000E0?ooo`030000003oool0oooo01h0oooo000Y0?ooo`030=nLB000
0000000000@0000000<0001d0;ooo`3oool07`3oool00`3ookl0M00004RLg`030?ooo`030=nLB000
07@0_ooo00L0oooo00<000000?ooo`3oool03@3oool2000005d0oooo00@000000?ooo`3oool00000
EP3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0
5`3oool00`000000oooo0?ooo`0d0?ooo`8000005`3oool00`000000oooo0?ooo`0N0?ooo`00:@3o
ool00`3ooml0W4P000000004000000030000M02oool0oooo01l0oooo00<0omnL04P0001d_ol00`3o
ool00`3o_g@0001809cOo`070?ooo`030000003oool0oooo00d0oooo0P00001M0?ooo`040000003o
ool0oooo000005H0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`160?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool0<P3oool2000001T0oooo00<000000?ooo`3oool0
7P3oool002X0oooo00<0gibL0;ooo`3oool00`3oool00`3OW9`0_ooo0?ooo`0O0?ooo`030?oOW018
0000M;oo00<0oooo00<0okmd0000B02Lgol01`3ooooo000008/00000803oool002T0oooo00<0okmd
04RLg`3oool09P3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00L0oooo00<00000
0?ooo`3oool03@3oool2000005`0oooo00D000000?ooo`3oool0oooo0000001E0?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool0@`3oool2000001`0oooo00<000000?ooo`3oool0;`3o
ool2000001`0oooo00<000000?ooo`3oool07P3oool002T0oooo00<0_g@0001d_`3oool0103oool0
0`3OW9`0_ooo0?ooo`0O0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01`3oool0
0`000000oooo0?ooo`0=0?ooo`800000G03oool01@000000oooo0?ooo`3oool0000005@0oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo01`0oooo00<0
00000?ooo`3oool0;@3oool2000001h0oooo00<000000?ooo`3oool07P3oool002T0oooo00<0gia8
000000000000100000000`0007@0_ooo0?ooo`0O0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia8
0000M02oool01`3oool00`000000oooo0?ooo`0=0?ooo`800000G03oool01@000000oooo0?ooo`3o
ool0000005<0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`100?ooo`8000007`3o
ool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool07P3o
ool002T0oooo00<0oooO09a800000000100000000`0007@0_ooo0?ooo`0P0?ooo`030=nLB000B9`0
gooo0080oooo00<0_g@0001d_`3oool01`3oool00`000000oooo0?ooo`0=0?ooo`800000G03oool0
1@000000oooo0?ooo`3oool000000580oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`0n0?ooo`8000008@3oool00`000000oooo0?ooo`0Z0?ooo`8000008@3oool00`000000oooo0?oo
o`0N0?ooo`00<03oool00`3OW9`0_ooo0?ooo`0P0?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d
0;ooo`080?ooo`030000003oool0oooo00d0oooo0P00001L0?ooo`050000003oool0oooo0?ooo`00
0000D@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo03`0oooo0P00000S0?ooo`03
0000003oool0oooo02T0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0N0?ooo`00
E03oool01@3ookl0M0000000000007@0_ooo00T0oooo00<000000?ooo`3oool03@3oool2000005`0
oooo00D000000?ooo`3oool0oooo0000001@0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool0>`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo02L0oooo0P00000T0?oo
o`030000003oool0oooo01h0oooo000Z0?ooo`070;md000004P0W=oo0?oo_`1d0000000004RLg`0a
0?ooo`030000003oool0oooo00d0oooo0P00001K0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0C@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo03T0oooo0P00000V
0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0N
0?ooo`00:@3oool01@3o_g@000000000B02Lgol0_g@00080000000<0001d0;ooo`3oool0;`3oool0
0`000000oooo0?ooo`0=0?ooo`800000F`3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo04`0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0f0?ooo`<00000:03oool0
0`000000oooo0?ooo`0T0?ooo`8000009`3oool00`000000oooo0?ooo`0N0?ooo`00:@3oool02@2o
M7@0_ooo0?ooo`3ookl0M4RL0=ooo`3ogi`0B4RL0=ooo`0`0?ooo`030000003oool0oooo00d0oooo
0P00001K0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0B`3oool00`000000oooo
0?ooo`0B0?ooo`030000003oool0oooo03<0oooo0`00000[0?ooo`030000003oool0oooo0280oooo
0P00000Y0?ooo`030000003oool0oooo01h0oooo000Y0?ooo`080;mdB02Lgol0oooo0=nLM02Lgol0
oooo0?noW02Lgola0?ooo`030000003oool0oooo00d0oooo0P00001J0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0B@3oool2000001D0oooo00<000000?ooo`3oool0<@3oool20000
02h0oooo00<000000?ooo`3oool0803oool2000002/0oooo00<000000?ooo`3oool07P3oool002T0
oooo00<0gia8000000000000100000000`00B9`0gooo0?ooo`0_0?ooo`030000003oool0oooo00d0
oooo0P00001J0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0A`3oool2000001L0
oooo00<000000?ooo`3oool0;P3oool300000300oooo00<000000?ooo`3oool07P3oool2000002d0
oooo00<000000?ooo`3oool07P3oool002T0oooo00<0oono07@000000000100000000`00B9`0gooo
0?ooo`0_0?ooo`<000003@3oool2000005T0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`150?ooo`8000006@3oool00`000000oooo0?ooo`0[0?ooo`<00000<`3oool00`000000oooo
0?ooo`0M0?ooo`030000003oool0oooo02`0oooo0P00000P0?ooo`00;`3oool00`2oM4P0W=oo0?oo
o`0`0?ooo`030000003oool0oooo00d0oooo0P00001I0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool0@P3oool3000001/0oooo00<000000?ooo`3oool0:@3oool2000003H0oooo00<0
00000?ooo`3oool06`3oool200000300oooo00<000000?ooo`3oool07P3oool002T0oooo00<0_gAd
0;ooo`3oool0=P3oool00`000000oooo0?ooo`0=0?ooo`800000F03oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo03h0oooo1000000N0?ooo`030000003oool0oooo02L0oooo0P00000h
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`0N
0?ooo`009`3oool00`3ookl0M000000000060000000304RLg`3oool0oooo02l0oooo00<000000?oo
o`3oool03@3oool2000005P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0j0?oo
o`@000008P3oool00`000000oooo0?ooo`0T0?ooo`<00000>P3oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo0340oooo00<000000?ooo`3oool07P3oool002H0oooo00<0okmd00000000
00001`0000000`0007@0_ooo0?ooo`0_0?ooo`030000003oool0oooo00d0oooo0P00001G0?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool0=`3oool3000002H0oooo00<000000?ooo`3o
ool08P3oool2000003d0oooo00<000000?ooo`3oool05`3oool2000003@0oooo00<000000?ooo`3o
ool07P3oool002T0oooo00<0_gAd0;ooo`3oool00`3oool00`3ookl0M4RL0=ooo`0`0?ooo`030000
003oool0oooo00d0oooo0P00001F0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
<`3oool4000002T0oooo00<000000?ooo`3oool07`3oool3000003l0oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo01h0oooo000Y0?ooo`030;mdM02o
ool0oooo00<0oooo00<0_gA809cOo`3oool0<03oool00`000000oooo0?ooo`0=0?ooo`800000E@3o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo02l0oooo1000000]0?ooo`030000003o
ool0oooo01`0oooo0`0000120?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0=@3o
ool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo0?ooo`0=0?ooo`800000E03oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo02/0oooo1000000a0?ooo`030000003oool0
oooo01X0oooo0P0000150?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0=P3oool0
0`000000oooo0?ooo`0N0?ooo`00:P3oool00`3ooml0W4P0000000020000000307Boo`3oool0oooo
0300oooo00<000000?ooo`3oool03@3oool2000005<0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0X0?ooo`<00000=@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
04D0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo
01h0oooo000Y0?ooo`030?oo_`1d0000000000@0000000<0B9cO0?ooo`3oool0;`3oool00`000000
oooo0?ooo`0=0?ooo`800000D@3oool2000000d0oooo00<000000?ooo`3oool0903oool4000003P0
oooo00<000000?ooo`3oool05`3oool2000004P0oooo00<000000?ooo`3oool04`3oool00`000000
oooo0?ooo`0g0?ooo`030000003oool0oooo01h0oooo000Y0?ooo`030=nLB018W=l0oooo0080oooo
00@0okmd0000000007@0_ooo<03oool00`000000oooo0?ooo`0<0?ooo`030000003oool0000004h0
oooo0`00000?0?ooo`030000003oool0oooo0200oooo1000000l0?ooo`030000003oool0oooo01D0
oooo0P00001:0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0>03oool00`000000
oooo0?ooo`0N0?ooo`00:@3oool00`2oM7@0_ooo0?ooo`030?ooo`030=nLB00007@0_ooo0300oooo
00<000000?ooo`3oool0303oool00`000000oooo0000001:0?ooo`@000004P3oool00`000000oooo
0?ooo`0L0?ooo`@00000@03oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo04X0oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo01h0oooo
000Y0?ooo`030=nLB018W=l0oooo00<0oooo00<0_g@0001d_`3oool0<03oool3000000`0oooo00<0
00000?ooo`000000AP3oool4000001H0oooo00<000000?ooo`3oool0603oool4000004@0oooo00<0
00000?ooo`3oool04P3oool2000004d0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?oo
o`0h0?ooo`800000803oool002T0oooo00L0oooO09a8000007@0_ooo0=nLB0000000B9cO0380oooo
00<000000?ooo`3oool0303oool00`000000oooo0000000o0?ooo`L000006P3oool00`000000oooo
0?ooo`0E0?ooo`<00000B03oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo04d0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo01h0oooo
001R0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`000000>`3oool400000240oooo00<0
00000?ooo`3oool04P3oool3000004/0oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`1>0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?oo
o`0N0?ooo`00903oool00`3o_i`0W=oo0?ooo`090?ooo`030=nLW02oool0oooo02l0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0000000c0?ooo`P000009@3oool00`000000oooo0?ooo`0@
0?ooo`800000CP3oool00`000000oooo0?ooo`0>0?ooo`800000D@3oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool07P3oool002@0oooo00<0_g@00000
000000002@0000000`0007@0_ooo0?ooo`0_0?ooo`030000003oool0oooo00/0oooo00@000000?oo
o`3oool00000;03oool7000002d0oooo00<000000?ooo`3oool03P3oool200000500oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`1A0?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool0?03oool00`000000oooo0?ooo`0N0?ooo`008`3oool00`3ookl0M0000000000:0000
00030000M02oool0oooo02l0oooo00<000000?ooo`3oool02P3oool01@000000oooo0?ooo`3oool0
000002@0oooo2000000d0?ooo`030000003oool0oooo00`0oooo0P00001B0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool0D@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo03d0oooo00<000000?ooo`3oool07P3oool002X0oooo00<0gibL0;ooo`3oool00`3oool00`3O
W9`0_ooo0?ooo`0_0?ooo`030000003oool0oooo00X0oooo00D000000?ooo`3oool0oooo0000000M
0?ooo`L00000?03oool00`000000oooo0?ooo`0:0?ooo`800000E03oool00`000000oooo0?ooo`0<
0?ooo`030000003oool0oooo0580oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0n
0?ooo`030000003oool0oooo01h0oooo000Y0?ooo`030?noM018W=l0oooo03H0oooo00<000000?oo
o`3oool0203oool2000000@0oooo00<000000?ooo`3oool04`3oool8000004<0oooo00<000000?oo
o`3oool0203oool2000005H0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`1B0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0N0?oo
o`00:@3oool00`2oM00007Bo0?ooo`040?ooo`030=nLW02oool0oooo02l0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00/0oooo2000001;0?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool0EP3oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo05<0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0o0?ooo`03
0000003oool0oooo01h0oooo000Y0?ooo`030=nLB0000000000000@0000000<0001d0;ooo`3oool0
;`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00L0oooo1000001C0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool0E`3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo05@0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0o0?ooo`030000
003oool0oooo01h0oooo000Y0?ooo`030?oog`2LB000000000@0000000<0001d0;ooo`3oool0;`3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00<0oooo1000001G0?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0F03oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo05D0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0o0?ooo`030000003o
ool0oooo01h0oooo000`0?ooo`030=nLW02oool0oooo02l0oooo00<000000?ooo`3oool03P3oool0
10000000oooo0?ooo`3oool2000005/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`1I0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0EP3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool07P3oool00680oooo0`00000>
0?ooo`040000003oool0oooo000005d0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`1I0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0EP3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo03l0oooo0P00000P0?ooo`00HP3oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0000005h0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1J
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0E`3oool00`000000oooo0?ooo`09
0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool07P3oool00680oooo00<000000?oo
o`3oool03P3oool00`000000oooo0000001N0?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool0FP3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo05L0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo01h0oooo001R0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`000000GP3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo05/0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`1G0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`0N0?ooo`00
HP3oool00`000000oooo0?ooo`0>0?ooo`800000G`3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo05/0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`1G0?ooo`030000
003oool0oooo00P0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0N0?ooo`00HP3o
ool00`000000oooo0?ooo`0>0?ooo`800000G`3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo05/0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`1H0?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool0
0`000000oooo0?ooo`0>0?ooo`800000G`3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo05/0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`1H0?ooo`030000003oool0
oooo00P0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0N0?ooo`00B@3oool00`2o
M0000000000000040000000904RLg`3oool0oooo0?ooo`3ogi`0B0000000000007@0_ooo00T0oooo
00<000000?ooo`3oool03P3oool2000005l0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`1K0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0F03oool00`000000oooo
0?ooo`080?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool07P3oool004X0oooo00<0
gia80000000000000`0000002P00B9`0gooo0?ooo`3ookl0M00007Boo`3oool0okmd0000M02oool8
0?ooo`030000003oool0oooo00h0oooo0P00001O0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0F`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo05P0oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo01h0oooo001;0?oo
o`030=nLB018W=l0oooo0080oooo00H0giad09cOo`3oool0_g@00018W03Oool20?ooo`030;md0000
M;l0oooo00L0oooo00<000000?ooo`3oool03P3oool2000005l0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`1K0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0F03o
ool00`000000oooo0?ooo`070?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool07P3o
ool004`0oooo00<0gia807Boo`3oool00`3oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0
_ooo00L0oooo00<000000?ooo`3oool03P3oool2000005l0oooo00D000000?ooo`3oool0oooo0000
001N0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0F@3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool07P3oool004d0oooo00<0_gA8
09cOo`3oool00P3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00L0oooo00<00000
0?ooo`3oool03P3oool00`000000oooo0?ooo`1N0?ooo`050000003oool0oooo0?ooo`000000GP3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo05T0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`140?ooo`030000003oool0oooo01h0oooo001=0?ooo`030?oog`2LB7@0
_ooo0080oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`070?ooo`@000003@3oool0
0`000000oooo0?ooo`1N0?ooo`050000003oool0oooo0?ooo`000000GP3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo05T0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`120?ooo`<00000803oool004h0oooo00L0okmd001d_`3oool0oooo0?oOW0180000M;oo00<0oooo
00<0okmd0000B02Lgol01`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo
00D000000?ooo`3oool0oooo0000001N0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0F@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3o
ool07P3oool004h0oooo00L0oono07@0B02Lgol0oooo0?oOW0180000M;oo00<0oooo00<0okmd0000
B02Lgol01`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo00D000000?oo
o`3oool0oooo0000001N0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0F@3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool07P3oool0
04h0oooo00L0oono07@00018W=l0oooo0?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool01`3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo00D000000?ooo`3oool0oooo
0000001N0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0FP3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool07P3oool004T0oooo00<0
oknL09cOo`3oool00P3oool00`3ogi`0B00004RLg`020?ooo`030=nLB000B9`0gooo0080oooo00<0
_g@0001d_`3oool01`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo00@0
00000?ooo`3oool00000G`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo05X0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo01h0oooo
00190?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02oool20?ooo`070?oo_`1d0000M;oo
0?ooo`3o_g@0001d0;ooo`080?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0GP3o
ool010000000oooo0?ooo`00001O0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
FP3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool0
7P3oool004X0oooo00H0oooO09a80000000000000000M02oool40?ooo`050?oo_`1d000000000000
M02oool02@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo00@000000?oo
o`3oool00000G`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo05X0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo01h0oooo001R0?oo
o`030000003oool0oooo00h0oooo00<000000?ooo`3oool0GP3oool010000000oooo0?ooo`00001O
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0FP3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool07P3oool00680oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`1N0?ooo`040000003oool0oooo000005l0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`1K0?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool0A@3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo05h0oooo00@000000?ooo`3oool00000G`3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo05/0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`15
0?ooo`030000003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool0GP3oool010000000oooo0?ooo`00001O0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0F`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo04D0oooo00<00000
0?ooo`3oool07P3oool00680oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1N0?oo
o`040000003oool0oooo000005l0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1K
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`0N
0?ooo`00HP3oool3000000h0oooo00<000000?ooo`3oool0GP3oool010000000oooo0?ooo`00001O
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0F`3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo04D0oooo0P00000P0?ooo`00HP3oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo05h0oooo00@000000?ooo`3oool00000G`3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo05/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`16
0?ooo`030000003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool0GP3oool00`000000oooo0000001P0?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool0F`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo04H0oooo00<000000?oo
o`3oool07P3oool00680oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1N0?ooo`03
0000003oool000000600oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1K0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`0N0?ooo`00
HP3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`000000
H03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo05`0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`160?ooo`030000003oool0oooo01h0oooo001R0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0GP3oool00`000000oooo0000001P0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool0G03oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo04H0oooo00<000000?ooo`3oool07P3oool00680oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`1N0?ooo`030000003oool000000600oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`1L0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0AP3oool0
0`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
05h0oooo00<000000?ooo`000000H03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
05`0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`160?ooo`030000003oool0oooo
01h0oooo001R0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0GP3oool00`000000
oooo0000001P0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0G03oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool07P3oool00680oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1N0?ooo`030000003oool000000600oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1L0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`000000H03oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo05`0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`160?ooo`030000003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool0GP3oool00`000000oooo0000001P0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0G03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo04H0oooo00<00000
0?ooo`3oool07P3oool00680oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1N0?oo
o`030000003oool000000600oooo00D000000?ooo`3oool0oooo0000001O0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool30000
00h0oooo00<000000?ooo`3oool0GP3oool00`000000oooo0000001P0?ooo`050000003oool0oooo
0?ooo`000000G`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo04D0oooo0P00000P
0?ooo`00HP3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo00<000000?oo
o`000000H03oool01@000000oooo0?ooo`3oool0000005l0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`160?ooo`030000003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool0GP3oool00`000000oooo0000001P0?ooo`050000003oool0oooo
0?ooo`000000G`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo04H0oooo00<00000
0?ooo`3oool07P3oool00680oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1N0?oo
o`030000003oool000000600oooo00D000000?ooo`3oool0oooo0000001O0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo00<000000?ooo`000000H03oool01@00
0000oooo0?ooo`3oool0000005l0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`17
0?ooo`030000003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool0GP3oool00`000000oooo0000001P0?ooo`050000003oool0oooo0?ooo`000000G`3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool07P3oool0
0680oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1N0?ooo`030000003oool00000
0600oooo00D000000?ooo`3oool0oooo0000001O0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0A`3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo05h0oooo00<000000?ooo`000000H03oool01@000000oooo0?ooo`3oool0
000005l0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`170?ooo`030000003oool0
oooo01h0oooo001R0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0GP3oool00`00
0000oooo0000001P0?ooo`050000003oool0oooo0?ooo`000000G`3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool07P3oool00680oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`1N0?ooo`030000003oool000000600oooo00D000000?oo
o`3oool0oooo0000001O0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0A`3oool0
0`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
05h0oooo00<000000?ooo`000000H03oool01@000000oooo0?ooo`3oool0000005l0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo01h0oooo001R0?oo
o`030000003oool0oooo00h0oooo00<000000?ooo`3oool0GP3oool00`000000oooo0000001P0?oo
o`050000003oool0oooo0?ooo`000000G`3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo04L0oooo00<000000?ooo`3oool07P3oool00680oooo0`00000>0?ooo`030000003oool0oooo
05h0oooo00<000000?ooo`000000H03oool01@000000oooo0?ooo`3oool0000005l0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`160?ooo`800000803oool00680oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`1N0?ooo`030000003oool000000600oooo00D000000?oo
o`3oool0oooo0000001O0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0A`3oool0
0`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
05h0oooo00<000000?ooo`000000H03oool01@000000oooo0?ooo`3oool0000005l0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo01h0oooo001R0?oo
o`030000003oool0oooo00h0oooo00<000000?ooo`3oool0GP3oool200000640oooo00D000000?oo
o`3oool0oooo0000001O0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0A`3oool0
0`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
05h0oooo0P00001Q0?ooo`040000003oool0oooo00000600oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`170?ooo`030000003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool0GP3oool200000640oooo00@000000?ooo`3oool00000H03oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3oool07P3oool0
0680oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1N0?ooo`800000H@3oool01000
0000oooo0?ooo`00001P0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0B03oool0
0`000000oooo0?ooo`0N0?ooo`00C@3oool0103ooml0W4P00018W03Oool30?ooo`050?oOW0180000
00000000M02oool02@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo0P00
001Q0?ooo`040000003oool0oooo00000600oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`180?ooo`030000003oool0oooo01h0oooo001=0?ooo`040?oog`2LB00004RL0=ooo`80oooo
00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00P0oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0?ooo`1N0?ooo`800000H@3oool010000000oooo0?ooo`00001P0?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`0N0?ooo`00C@3oool0103o
oml0W4P00018W03Oool20?ooo`030;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool01`3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo0P00001Q0?ooo`040000003oool0
oooo00000600oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`180?ooo`030000003o
ool0oooo01h0oooo00190?ooo`030;md00000000000000D0000000@0M;oo0?oo_`1d0000B9cO0`3o
ool00`3OW4P0001d0;ooo`070?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0GP3o
ool200000640oooo00@000000?ooo`3oool00000H03oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo04P0oooo00<000000?ooo`3oool07P3oool004T0oooo00`0omnL07BLg`3oool0oooo
0?oog`2LB00004RL0=ooo`3oool0omnL04P0001d_ol30?ooo`030?noM00004P0W=oo00L0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`1N0?ooo`800000H@3oool010000000oooo0?oo
o`00001P0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0B03oool00`000000oooo
0?ooo`0N0?ooo`00BP3oool02`3OW4P0M;oo0?ooo`3ooml0W4P00018W03Oool0oooo0?oOW0180000
M;oo00<0oooo00<0okmd0000B02Lgol01`3oool4000000d0oooo00<000000?ooo`3oool0GP3oool2
00000640oooo00@000000?ooo`3oool00000H03oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo04H0oooo0`00000P0?ooo`00BP3oool02`3ooml0W4Qd0;ooo`3ooml0W4P00018W03Oool0
oooo0?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01`3oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo05h0oooo0P00001Q0?ooo`040000003oool0oooo00000600oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`180?ooo`030000003oool0oooo01h0oooo001;0?oo
o`0:0?oOW018M;l0oooO09a80000B9`0gooo0?ooo`3ogi`0B00007Boo`<0oooo00<0okmd0000B02L
gol01`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo0P00001Q0?ooo`04
0000003oool0oooo00000600oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`180?oo
o`030000003oool0oooo01h0oooo001<0?ooo`090=nLB01d_ml0W4P00018W03Oool0oooo0?oo_`1d
0000B9cO00<0oooo00<0gia80000M02oool01`3oool00`000000oooo0?ooo`0>0?ooo`030000003o
ool0oooo05h0oooo0P00001Q0?ooo`040000003oool0oooo00000600oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`180?ooo`030000003oool0oooo01h0oooo001<0?ooo`050?oog`2L
B4P0B4P00018W03Oool00P3oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00L0oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1N0?ooo`800000H@3oool010000000oooo
0?ooo`00001P0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0B03oool00`000000
oooo0?ooo`0N0?ooo`00C@3oool0103ogi`0B0000018W03Oool20?ooo`070?oo_`1d0000M;oo0?oo
o`3o_g@0001d0;ooo`080?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0GP3oool2
00000640oooo00@000000?ooo`3oool00000H03oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo04P0oooo00<000000?ooo`3oool07P3oool004h0oooo00<0gia80018W03Oool00`3oool0
1@3ookl0M0000000000007@0_ooo00T0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`1N0?ooo`800000H@3oool010000000oooo0?ooo`00001P0?ooo`050000003oool0oooo0?ooo`00
0000B`3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo0?ooo`0>0?ooo`030000
003oool0oooo05h0oooo0P00001Q0?ooo`040000003oool0oooo00000600oooo00D000000?ooo`3o
ool0oooo0000001;0?ooo`030000003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool0GP3oool200000640oooo00@000000?ooo`3oool00000H03oool01@00
0000oooo0?ooo`3oool0000004/0oooo00<000000?ooo`3oool07P3oool00680oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`1N0?ooo`800000H@3oool010000000oooo0?ooo`00001P
0?ooo`050000003oool0oooo0?ooo`000000B`3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo0P00001Q0?ooo`040000003oool0
oooo00000600oooo00D000000?ooo`3oool0oooo0000001;0?ooo`030000003oool0oooo01h0oooo
001R0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0GP3oool200000640oooo00@0
00000?ooo`3oool00000H03oool01@000000oooo0?ooo`3oool0000004/0oooo00<000000?ooo`3o
ool07P3oool00680oooo0`00000>0?ooo`030000003oool0oooo05h0oooo0P00001Q0?ooo`040000
003oool0oooo00000600oooo00D000000?ooo`3oool0oooo0000001:0?ooo`800000803oool00680
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1N0?ooo`800000H@3oool010000000
oooo0?ooo`00001P0?ooo`050000003oool0oooo0?ooo`000000B`3oool00`000000oooo0?ooo`0N
0?ooo`00HP3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo0P00001Q0?oo
o`040000003oool0oooo00000600oooo00D000000?ooo`3oool0oooo0000001;0?ooo`030000003o
ool0oooo01h0oooo001R0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0GP3oool2
00000640oooo00@000000?ooo`3oool00000H03oool01@000000oooo0?ooo`3oool0000004/0oooo
00<000000?ooo`3oool07P3oool00680oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`1N0?ooo`800000H@3oool010000000oooo0?ooo`00001P0?ooo`050000003oool0oooo0?ooo`00
0000B`3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool00`000000oooo0?ooo`0>0?ooo`030000
003oool0oooo05h0oooo0P00001Q0?ooo`040000003oool0oooo00000600oooo00D000000?ooo`3o
ool0oooo0000001;0?ooo`030000003oool0oooo01h0oooo001R0?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool0GP3oool200000640oooo00@000000?ooo`3oool00000H03oool01@00
0000oooo0?ooo`3oool0000004/0oooo00<000000?ooo`3oool07P3oool00680oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`1N0?ooo`800000H@3oool010000000oooo0?ooo`00001P
0?ooo`050000003oool0oooo0?ooo`000000B`3oool00`000000oooo0?ooo`0N0?ooo`00HP3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo05h0oooo0P00001Q0?ooo`040000003oool0
oooo00000600oooo00D000000?ooo`3oool0oooo0000001;0?ooo`030000003oool0oooo01h0oooo
001R0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0GP3oool200000640oooo00@0
00000?ooo`3oool00000H03oool01@000000oooo0?ooo`3oool0000004/0oooo00<000000?ooo`3o
ool07P3oool00680oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo04/0oooo0P0000070?ooo`030000003oool0oooo05@0oooo00@000000?ooo`3oool0
00000P3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo00d0oooo00D000000?ooo`3o
ool0oooo000000120?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool07P3oool00680
oooo0P00000?0?ooo`8000004@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00H0
oooo0P0000070?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03`3oool010000000
oooo0?ooo`0000020?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`0=0?ooo`8000000P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool07P3oool00680ooooo`00002;00000200oooo003o0?ooool0oooo
3P3oool00?l0ooooo`3oool>0?ooo`00o`3ooooo0?ooo`h0oooo003o0?ooool0oooo3P3oool00?l0
ooooo`3oool>0?ooo`00o`3ooooo0?ooo`h0oooo003o0?ooool0oooo3P3oool00?H0oooo00<0oooO
09a8000000000`0000000`18W=l0oooo0?ooo`3o0?ooo`h0oooo003f0?ooo`030;md0000B9`0gooo
00<0oooo00<0gibL0;ooo`3oool0o`3oool>0?ooo`00mP3oool00`3ooml0W7BL0=ooo`030?ooo`03
0?oo_`2LW=l0oooo0?l0oooo3P3oool00?L0oooo00<0_g@00000000000000`0000000`18W=l0oooo
0?ooo`3o0?ooo`d0oooo002m0?ooo`060=nLB000000000000000B02Lgml0W4P00P0000000`0007@0
_ooO09a8000200000003001d_`3ooml0W4P00080000000T004RL09ad0000000007Bo0?ooo`3ogi`0
B00000000018W=l00P3oool01`2oM00000000000000004P0W=oo0?ooo`3OW4P00P0000000`0004P0
W=oO09a8000200000006001d_`3OW4P000000000000007@0M7@00P0000000`0007@0_ooO09a80002
000000030018W03Ookl0M00000@0000000<0001d0;ooo`3oool01`3oool01@2oM000000000000000
04P0W=oo00<0oooo00D0gia8000000000000000007Boo`020?ooo`060;md0000000000000000M02o
oml0W4P00P0000001000B9`0gooo0?oog`2LB002000000050000B018B00004RL0=ooo`3OW4P00P00
00001`0007@0_ooo0?oOW0180000000004RLg`3OW4P00P000000100007@0_ooo0?ooo`3OW4P30000
000407Boo`3oool0oooo0;md0080000000<0001d0;oog`2LB0000P0000000`00B9`0gooo0?ooo`04
0?ooo`030?oOW018000000000080000000@0M;oo0?ooo`3oool0gia80`000000101d_ol0oooo0?oo
o`2oM002000000030000B02Lgol0oooo00@0oooo00H0gia8000000000000001809cOg`2LB0020000
00030000M02ooml0W4P00080000000<007Bo0?ooo`3oool02P3oool00`2oM0000000000000040000
000904RLg`3oool0oooo0?ooo`3ogi`0B0000000000007@0_ooo07d0oooo002n0?ooo`030=nLB000
07@0_ooo0080oooo02D0_g@00018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3o_g@0000007Boo`3o
ool0omnL04P00000M;l0okno0;ooo`3OW4P0001d0;ooo`3O_kl0gooo0;md0000000004RL0=ooo`3o
oml0W4Qd0;ooo`3oool0gia80000M02oool0oooo0?oog`2LB00007Bo0080oooo00/0gia80000M02o
ool0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`030;mdB02Lgol0oooo00/0oooo
00l0_g@000000000B9`0gooo0?oog`2LB7@0_ooo0?ooo`3OW4P0000004RLg`3oool0oooO09a80018
W=l00P3oool09P3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?oog`2LB000000007Boo`3o
ool0oono07@000000000001d0;ooo`3OW4P0001d0;ooo`3oool0gia80000M02oool0gkno0=ooo`3O
W4P0001d0;ooo`3oool0gia800000018W=l0oooo0?oog`2LB000B9cO0P3oool01`3OW4P004RL0=oo
o`3oool0oooO09a80000M;l0203oool00`3ooml0W4P0001d_`030?ooo`070=nLB0000000B9cO0?oo
o`3ooml0W4P004RLg`020?ooo`030=nLB00007@0_ooo00L0oooo00<0gia80000M02oool00P3oool0
1`2oM00004RL0=ooo`3oool0oooO09a80000M;l03P3oool00`3OW4P00000000000030000000:0018
W03Oool0oooo0?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooog`0oooo002n0?ooo`030=nLB00007@0
_ooo0080oooo00l0_g@00018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3o_g@0001809cOo`3oool0
oooO09a80000M;l00P3oool01`3OW4P004RL0=ooo`3oool0omnL04P00000M;l00`3oool02P3ogi`0
W;oo0?ooo`3OW4P0001d0;ooo`3oool0oooO09a80000M;l20?ooo`0;0=nLB00007@0_ooo0?ooo`3O
W4P004RL0=ooo`3oool0oooO09a80000M;l00P3oool01@3ookl0M00000000000000007Bo00P0oooo
00<0omnL04P00000M;l00`3oool01@3ogi`0W;oo0?oOW018000007Bo00<0oooo00l0okmd0000M02o
ool0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0omnL04P00018W=l00P3oool00`3o
oml0W4P0001d_`020?ooo`070=nLB00007@0_ooo0?ooo`3OW4P004RL0=ooo`020?ooo`060=nLB000
07@0_ooo0?oOW018000007Bo0`3oool02`3o_g@0001d0;ooo`3oool0gia80018W03Oool0oooo0?oo
g`2LB00007Bo00P0oooo00<0oooO09a80000M;l00P3oool00`3ogi`0B000001d_`030?ooo`070?no
M00007@0_ooo0?ooo`3OW4P0001d0;ooo`070?ooo`030=nLB00007@0_ooo0080oooo00X0_g@00018
W03Oool0oooo0?oog`2LB00007Bo0?ooo`3ooml0W4P0200000001P18W=l0oooo0?ooo`3oool0gia8
04RLg`<0oooo00H0giad09cOo`3oool0_g@00018W03Oool20?ooo`030;md0000M;l0oooo07/0oooo
002n0?ooo`030=nLB00007@0_ooo0080oooo00L0_g@00018W03Oool0oooo0?oog`2LB00007Bo0080
oooo00H0okmd00000000M;l0oooO09a80000M;l20?ooo`070=nLB000B9`0gooo0?ooo`3o_g@00018
09cOo`060?ooo`070=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`020?ooo`0;0=nLB00007@0_ooo
0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l00P3oool01P3OW4P0001d0;ooo`3ookl0M000
04RLg`L0oooo00<0okmd0000B02Lgol01@3oool00`3o_g@0000007Boo`030?ooo`0?0?noM0000000
M;oo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?noM0000000M;oo0080oooo00<0
oooO09a80000M;l00P3oool01`3OW4P0001d0;ooo`3oool0gia80018W03Oool00P3oool01P3OW4P0
001d0;ooo`3o_g@0000007Boo`<0oooo00/0okmd0000001d_ol0oooo0=nLB000B9`0gooo0?ooo`3o
oml0W4P0001d_`080?ooo`030?oog`2LB00007Bo0080oooo00<0okmd0000001d_ol00`3oool01`3o
_g@0000007Boo`3oool0gia80000M02oool01`3oool00`3OW4P0001d0;ooo`020?ooo`070;md0000
B9`0gooo0?ooo`3ooml0W4P0001d_`0@0?ooo`030=nLB01d_ol0oooo00<0oooo00<0oono07@00018
W=l00`3oool00`3OW4P0001d0;ooo`1k0?ooo`00_P3oool00`3OW4P0001d0;ooo`020?ooo`070;md
0000B9`0gooo0?ooo`3ooml0W4P0001d_`040?ooo`040?noM00000000000001d_`80oooo00L0gia8
0018W03Oool0oooo0?noM00007@0_ooo00H0oooo00L0gia80000M02oool0oooo0?oog`2LB00007Bo
0080oooo01<0gia80000M02oool0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0oono
07@00018W=l0oooo0?oog`2LB00007Bo00L0oooo00<0okmd0000M02oool01@3oool00`3o_g@00000
07Boo`030?ooo`0?0?noM0000000M;oo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo
0?noM0000000M;oo0080oooo00<0oooO09a80000M;l00P3oool01`3OW4P0001d0;ooo`3oool0gia8
0018W03Oool00P3oool01P3OW4P0001d0;ooo`3o_g@0000007Boo`<0oooo00/0okmd0000001d_ol0
oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`080?ooo`030?oog`2LB00007Bo0080oooo00<0
okmd0000001d_ol00`3oool01`3o_g@0000007Boo`3oool0gia80000M02oool01`3oool00`3OW4P0
001d0;ooo`020?ooo`070;md0000B9`0gooo0?ooo`3ooml0W4P0001d_`0A0?ooo`030;mdB02Lgol0
oooo0080oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`1k0?ooo`00_P3oool00`3O
W4P0001809cOo`020?ooo`0?0;md000007@0_ooo0?ooo`3ooml0W4P0001d_`3oool0oono07@0001d
_ol0oooo0?oog`2LB00007Bo0080oooo00L0gia80018W03Oool0oooo0?oOW01804P0W=oo0080oooo
00/0okmd0000B02Lgol0oooo0=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`020?ooo`0C0=nLB000
07@0_ooo0?ooo`2oM000001d0;ooo`3oool0oooO09a80000M;l0oooo0?oo_`1d0000B9cO0?ooo`3o
oml0W4P004RLg`070?ooo`030?oOW01804P0W=oo0080oooo00H0okmd0000B02Lgol0oono07@0001d
_ol30?ooo`0?0;md000004P0W=oo0?ooo`2oM000001d0;ooo`3oool0oooO09a80000M;l0oooo0?oo
_`1d0000B9cO0080oooo00<0oooO09a80000M;l00P3oool01`3OW4P0001d0;ooo`3oool0gia80018
W03Oool00P3oool01P3OW4P0001d0;ooo`3ookl0M00007Boo`<0oooo00/0_g@00000B02Lgol0oooo
0;md000007@0_ooo0?ooo`3ooml0W4P0001d_`080?ooo`030?oog`2LB00007Bo0080oooo00<0oono
07@0001d_ol00`3oool01`2oM000001809cOo`3oool0gia80000B02Lgol01`3oool00`3OW4P00018
09cOo`020?ooo`070;md000007@0_ooo0?ooo`3ooml0W4P0001d_`0A0?ooo`030?oog`2LB7@0_ooo
0080oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`1k0?ooo`00_03oool05P3ooml0
W4P00000000004P0B4Q809cOo`3ookl0M000001dM01d07@0_ooo0?oo_`1d0000B9cO0?ooo`3ookl0
M00004RLg`3oool0oono07@00000M;l20?ooo`030=nLB000B9`0gooo0080oooo01`0_g@00018W03O
ool0oooO09a8000007@0_ooo0?ooo`3OW4P000000000B02Lgol0oono07@00018W=l0oooO09a80000
0000001d0;oog`2LB00000000018B01804P0W=oo0?oo_`1d0000B9cO0P3oool0102oM00007Bo0?oo
o`3o_g@2000000030018W03Oool0oooo00D0oooo01P0_g@00018W03Oool0oooO09a8000007@0_ooo
0?ooo`3OW4P004RL0=ooo`3oool0okmd00000000M;l0oooO09a80000000004Q804P0B02Lgol0oono
07@00018W=l20?ooo`0V0?noM00004P0W=oo0?ooo`3o_g@00000001d_`3ooml0W4P00000000007@0
_ooo0?ooo`3OW4P004RL0=ooo`3ooml0W4P00000000007@0_ooo0?ooo`3OW4P004RL0=ooo`3oool0
okmd00000000M;l0oooO09a80000000004Q804P0B02Lgol0oono07@00018W=l80?ooo`030?oog`2L
B00007Bo00<0oooo00l0gia80018W03Oool0oooo0?noM000000007Bo0?oog`2LB00000000000B018
M9`0gkmd0000M02oool00P3oool04@3ooml0W4P00000000004P0B4Q809cOo`3ookl0M000001dM01d
07@0_ooo0?oo_`1d0000B9cO0?ooo`3ooml0W4P000P0000000<0B9cO0?ooo`3oool0103oool01`3o
_g@007Bo0?ooo`3oool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`1k0?ooo`00_P3oool0
302oM000001d0;oog`2LB00000000000B02Lgol0oono07@000000000001d0;ooo`80oooo00P0oono
07@00000000000000000M02oool0oooo0;md00<0000000@004RL0=ooo`3oool0gia80P0000002`00
04P0W=oo0?ooo`3oool0gia80000M02ooml0W4P00000000007@0_ooo0080oooo00/0_g@00000M02o
ool0oooo0;md0000B9`0gooO09a800000000001d0;ooo`020?ooo`050?oog`2LB00000000000001d
_ol02P3oool01@3OW4P000000000000004P0W=oo00<0oooo00D0okmd000000000000000004RLg`03
0?ooo`070;md0000B9`0gooO09a800000000001d0;ooo`030?ooo`030?noM0000000000000800000
00L007Bo0?ooo`3oool0_g@00000M02oool0_g@000<0000000D004RL0=ooo`2oM000001d0;ooo`02
0?ooo`050?noM0000000000000000018W=l00`3oool01`2oM00004RL0=oog`2LB00000000000M02o
ool01`3oool00`3o_g@0000000000002000000040018W03Oool0oooo0?noM0<0000000X0B9cO0?oo
o`3oool0oooo0;md000007@0_mnL04P0000007@0_ooo103oool0302oM000001d0;oog`2LB0000000
0000B02Lgol0oono07@000000000001d0;oooa80oooo00L0oono07@0B02Lgol0oooo0?oOW0180000
M;oo00<0oooo00<0okmd0000B02Lgol0N`3oool00=@0oooo00<0_g@00018W03Oool02P3oool00`3O
W4P0001d0;ooo`0o0?ooo`030?oog`2LB00007Bo00H0oooo00<0_g@00018W03Oool07P3oool00`3o
oml0W4P0001d_`0d0?ooo`070?oo_`1d0000B9cO0?ooo`3ookl0M00004RLg`030?ooo`030=nLB000
07@0_ooo07/0oooo003D0?ooo`030?oo_`1dB9`0gooo00X0oooo00<0gia80000M02oool0?`3oool0
0`3ooml0W4P0001d_`060?ooo`030?oo_`1dB9`0gooo01l0oooo00<0_g@0001d_`3oool0;P3oool0
0`3o_i`0W=oo0?ooo`020?ooo`030?oOW0180000B9cO0080oooo00<0gia80018W03Oool00P3oool0
0`2oM00007Bo0?ooo`1k0?ooo`00e03oool00`3ooml0W7BL0=ooo`0:0?ooo`030=nLB00007@0_ooo
03l0oooo00<0oooO09a80000M;l01P3oool00`3ooml0W7BL0=ooo`0O0?ooo`030=nLB000M;l0oooo
02h0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`80oooo00L0oono07@0001d_ol0
oooo0?noM00007@0_ooo07`0oooo003O0?ooo`050?oog`2LB00000000000M02oool01P3oool00`3O
W4P0001809cOo`0e0?ooo`040?oo_`1d00000000001d_`80oooo00<0gia80000B02Lgol01P3oool0
0`3OW4P0001809cOo`0J0?ooo`070?oo_`1d07@0_ooo0?oog`2LB00004RL0=ooo`0[0?ooo`060?oo
g`2LB00000000000000007@0_ooo103oool01@3ookl0M0000000000007@0_ooo07d0oooo003Q0?oo
o`030;md000007@0_ooo00H0oooo00<0gia80000B02Lgol0=P3oool00`3ooml0W4P0001d_`020?oo
o`030=nLB00004P0W=oo00H0oooo00<0gia80000B02Lgol06`3oool01@3ookl0M0000000000004P0
W=oo0;P0oooo003o0?ooool0oooo3P3oool00?l0ooooo`3oool>0?ooo`00o`3ooooo0?ooo`h0oooo
003o0?ooool0oooo3P3oool00001\
\>"],
  ImageRangeCache->{{{0, 523.313}, {323.063, 0}} -> {2.2274, -71.4181, \
0.00862476, 0.618168}}]
}, Closed]],

Cell[TextData[{
  "The eigenvalues for the odd solutions are now readily apparent.  We locate \
the lowest odd solution using ",
  StyleBox["ShootBSWF1", "InlineInput",
    FontWeight->"Bold"],
  " for one side of the potential."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(sol1 = 
      ShootBSWF1[pot, 20, 2.8871, 2.88715, 0, 3, 3, AccuracyGoal \[Rule] 10, 
        WorkingPrecision \[Rule] 32, MaxSteps \[Rule] 10000, 
        MaxIterations \[Rule] 200]\)], "Input"],

Cell[BoxData[
    \({energy \[Rule] 2.88716643663534669831246507059643101959499164782`32. , 
      wavefunction \[Rule] \((norm$108446\ \[Psi]$108446[#1] &)\), 
      normalization \[Rule] 17.00655391192598`}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(PlotBSWF[sol1, 0, 3];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 -0.401355 0.159448 [
[.02381 -0.0125 -5 -15.6875 ]
[.02381 -0.0125 5 0 ]
[.18254 -0.0125 -9.5 -15.6875 ]
[.18254 -0.0125 9.5 0 ]
[.34127 -0.0125 -5 -15.6875 ]
[.34127 -0.0125 5 0 ]
[.5 -0.0125 -9.5 -15.6875 ]
[.5 -0.0125 9.5 0 ]
[.65873 -0.0125 -5 -15.6875 ]
[.65873 -0.0125 5 0 ]
[.81746 -0.0125 -9.5 -15.6875 ]
[.81746 -0.0125 9.5 0 ]
[.97619 -0.0125 -5 -15.6875 ]
[.97619 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -5.0625 -31.375 ]
[.5 -0.0125 5.0625 -15.6875 ]
[-0.0125 .07699 -10 -7.84375 ]
[-0.0125 .07699 0 7.84375 ]
[-0.0125 .15671 -19 -7.84375 ]
[-0.0125 .15671 0 7.84375 ]
[-0.0125 .23644 -10 -7.84375 ]
[-0.0125 .23644 0 7.84375 ]
[-0.0125 .31616 -19 -7.84375 ]
[-0.0125 .31616 0 7.84375 ]
[-0.0125 .39589 -10 -7.84375 ]
[-0.0125 .39589 0 7.84375 ]
[-0.0125 .47561 -19 -7.84375 ]
[-0.0125 .47561 0 7.84375 ]
[-0.0125 .55533 -10 -7.84375 ]
[-0.0125 .55533 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -34.6875 -13.0313 ]
[-0.01875 .30902 -19 13.0313 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -89.6875 0 ]
[.5 .63053 89.6875 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
gsave
.02381 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.18254 0 m
.18254 .00625 L
s
gsave
.18254 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.34127 0 m
.34127 .00625 L
s
gsave
.34127 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.5 0 m
.5 .00625 L
s
gsave
.5 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.5) show
1.000 setlinewidth
grestore
.65873 0 m
.65873 .00625 L
s
gsave
.65873 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.81746 0 m
.81746 .00625 L
s
gsave
.81746 -0.0125 -70.5 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.5) show
1.000 setlinewidth
grestore
.97619 0 m
.97619 .00625 L
s
gsave
.97619 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
.125 Mabswid
.05556 0 m
.05556 .00375 L
s
.0873 0 m
.0873 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.15079 0 m
.15079 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.24603 0 m
.24603 .00375 L
s
.27778 0 m
.27778 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.37302 0 m
.37302 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.43651 0 m
.43651 .00375 L
s
.46825 0 m
.46825 .00375 L
s
.53175 0 m
.53175 .00375 L
s
.56349 0 m
.56349 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62698 0 m
.62698 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.72222 0 m
.72222 .00375 L
s
.75397 0 m
.75397 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.84921 0 m
.84921 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.9127 0 m
.9127 .00375 L
s
.94444 0 m
.94444 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -66.0625 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
69.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .07699 m
.00625 .07699 L
s
gsave
-0.0125 .07699 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
0 .15671 m
.00625 .15671 L
s
gsave
-0.0125 .15671 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3.5) show
1.000 setlinewidth
grestore
0 .23644 m
.00625 .23644 L
s
gsave
-0.0125 .23644 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
0 .31616 m
.00625 .31616 L
s
gsave
-0.0125 .31616 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4.5) show
1.000 setlinewidth
grestore
0 .39589 m
.00625 .39589 L
s
gsave
-0.0125 .39589 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
0 .47561 m
.00625 .47561 L
s
gsave
-0.0125 .47561 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5.5) show
1.000 setlinewidth
grestore
0 .55533 m
.00625 .55533 L
s
gsave
-0.0125 .55533 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .01321 m
.00375 .01321 L
s
0 .02915 m
.00375 .02915 L
s
0 .0451 m
.00375 .0451 L
s
0 .06104 m
.00375 .06104 L
s
0 .09293 m
.00375 .09293 L
s
0 .10888 m
.00375 .10888 L
s
0 .12482 m
.00375 .12482 L
s
0 .14077 m
.00375 .14077 L
s
0 .17266 m
.00375 .17266 L
s
0 .1886 m
.00375 .1886 L
s
0 .20455 m
.00375 .20455 L
s
0 .22049 m
.00375 .22049 L
s
0 .25238 m
.00375 .25238 L
s
0 .26833 m
.00375 .26833 L
s
0 .28427 m
.00375 .28427 L
s
0 .30022 m
.00375 .30022 L
s
0 .33211 m
.00375 .33211 L
s
0 .34805 m
.00375 .34805 L
s
0 .364 m
.00375 .364 L
s
0 .37994 m
.00375 .37994 L
s
0 .41183 m
.00375 .41183 L
s
0 .42777 m
.00375 .42777 L
s
0 .44372 m
.00375 .44372 L
s
0 .45966 m
.00375 .45966 L
s
0 .49155 m
.00375 .49155 L
s
0 .5075 m
.00375 .5075 L
s
0 .52344 m
.00375 .52344 L
s
0 .53939 m
.00375 .53939 L
s
0 .57128 m
.00375 .57128 L
s
0 .58722 m
.00375 .58722 L
s
0 .60317 m
.00375 .60317 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -100.875 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
74.0313 11.8438 translate
90 rotate
-74.0313 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(V) show
71.625 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(,) show
77.625 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(y) show
85.063 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .61178 m
.02381 .61803 L
s
.18254 .61178 m
.18254 .61803 L
s
.34127 .61178 m
.34127 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.65873 .61178 m
.65873 .61803 L
s
.81746 .61178 m
.81746 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.05556 .61428 m
.05556 .61803 L
s
.0873 .61428 m
.0873 .61803 L
s
.11905 .61428 m
.11905 .61803 L
s
.15079 .61428 m
.15079 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.24603 .61428 m
.24603 .61803 L
s
.27778 .61428 m
.27778 .61803 L
s
.30952 .61428 m
.30952 .61803 L
s
.37302 .61428 m
.37302 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.43651 .61428 m
.43651 .61803 L
s
.46825 .61428 m
.46825 .61803 L
s
.53175 .61428 m
.53175 .61803 L
s
.56349 .61428 m
.56349 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.62698 .61428 m
.62698 .61803 L
s
.69048 .61428 m
.69048 .61803 L
s
.72222 .61428 m
.72222 .61803 L
s
.75397 .61428 m
.75397 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.84921 .61428 m
.84921 .61803 L
s
.88095 .61428 m
.88095 .61803 L
s
.9127 .61428 m
.9127 .61803 L
s
.94444 .61428 m
.94444 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .07699 m
1 .07699 L
s
.99375 .15671 m
1 .15671 L
s
.99375 .23644 m
1 .23644 L
s
.99375 .31616 m
1 .31616 L
s
.99375 .39589 m
1 .39589 L
s
.99375 .47561 m
1 .47561 L
s
.99375 .55533 m
1 .55533 L
s
.125 Mabswid
.99625 .01321 m
1 .01321 L
s
.99625 .02915 m
1 .02915 L
s
.99625 .0451 m
1 .0451 L
s
.99625 .06104 m
1 .06104 L
s
.99625 .09293 m
1 .09293 L
s
.99625 .10888 m
1 .10888 L
s
.99625 .12482 m
1 .12482 L
s
.99625 .14077 m
1 .14077 L
s
.99625 .17266 m
1 .17266 L
s
.99625 .1886 m
1 .1886 L
s
.99625 .20455 m
1 .20455 L
s
.99625 .22049 m
1 .22049 L
s
.99625 .25238 m
1 .25238 L
s
.99625 .26833 m
1 .26833 L
s
.99625 .28427 m
1 .28427 L
s
.99625 .30022 m
1 .30022 L
s
.99625 .33211 m
1 .33211 L
s
.99625 .34805 m
1 .34805 L
s
.99625 .364 m
1 .364 L
s
.99625 .37994 m
1 .37994 L
s
.99625 .41183 m
1 .41183 L
s
.99625 .42777 m
1 .42777 L
s
.99625 .44372 m
1 .44372 L
s
.99625 .45966 m
1 .45966 L
s
.99625 .49155 m
1 .49155 L
s
.99625 .5075 m
1 .5075 L
s
.99625 .52344 m
1 .52344 L
s
.99625 .53939 m
1 .53939 L
s
.99625 .57128 m
1 .57128 L
s
.99625 .58722 m
1 .58722 L
s
.99625 .60317 m
1 .60317 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
gsave
.5 .63053 -150.688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Wave) show
93.875 15.813 moveto
(Function) show
139.313 15.813 moveto
(with) show
163.563 15.813 moveto
(Energy) show
200.563 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
211.375 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.887) show
238.375 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.02381 .39589 m
.02499 .39588 L
.02605 .39585 L
.02729 .39581 L
.02846 .39575 L
.03053 .3956 L
.03279 .39538 L
.03527 .39505 L
.0379 .39463 L
.04262 .39365 L
.04749 .39235 L
.05205 .39086 L
.06244 .38653 L
.07305 .38077 L
.08274 .37437 L
.10458 .35623 L
.14429 .31191 L
.18248 .25948 L
.22313 .19902 L
.26226 .1421 L
.30384 .08887 L
.32471 .06655 L
.34391 .04919 L
.36384 .0347 L
.38246 .02463 L
.39154 .02097 L
.40128 .01798 L
.40683 .0167 L
.41209 .01578 L
.41438 .01547 L
.41681 .0152 L
.41808 .01508 L
.41948 .01497 L
.42075 .01489 L
.42194 .01482 L
.42308 .01478 L
.42429 .01474 L
.42498 .01473 L
.42564 .01472 L
.42686 .01472 L
.42815 .01473 L
.42934 .01475 L
.43065 .0148 L
.43204 .01487 L
.43431 .01502 L
.43644 .01521 L
.44129 .01582 L
.44654 .01674 L
.45134 .01783 L
.46215 .0211 L
Mistroke
.47202 .02509 L
.48117 .02963 L
.50191 .04281 L
.54109 .07803 L
.58272 .1288 L
.62284 .18903 L
.66145 .25601 L
.7025 .33566 L
.74204 .41956 L
.78006 .50623 L
.82054 .60441 L
Mfstroke
.82054 .60441 m
.82585 .61803 L
s
0 g
.02381 .05925 m
.02499 .05925 L
.02605 .05925 L
.02729 .05925 L
.02846 .05925 L
.03053 .05925 L
.03279 .05926 L
.03527 .05926 L
.0379 .05927 L
.04262 .05929 L
.04749 .05931 L
.05205 .05934 L
.06244 .05942 L
.06788 .05948 L
.07305 .05955 L
.08274 .05971 L
.09318 .05994 L
.10458 .06028 L
.1149 .0607 L
.12425 .06118 L
.13505 .06191 L
.14523 .06281 L
.15499 .06389 L
.1653 .06535 L
.18378 .06895 L
.19325 .07142 L
.20341 .07464 L
.22175 .08223 L
.24299 .09451 L
.26311 .11017 L
.30144 .15133 L
.34222 .20598 L
.36247 .23243 L
.38149 .25356 L
.39034 .2615 L
.39965 .26826 L
.40453 .27108 L
.40987 .27356 L
.41222 .27444 L
.41472 .27525 L
.41687 .27582 L
.41924 .27633 L
.42049 .27654 L
.42183 .27673 L
.42257 .27681 L
.42324 .27688 L
.42393 .27693 L
.42457 .27697 L
.42577 .27703 L
.42709 .27704 L
Mistroke
.42825 .27702 L
.42948 .27697 L
.43023 .27692 L
.43092 .27686 L
.43248 .27668 L
.43386 .27647 L
.43531 .27621 L
.43782 .27563 L
.44058 .27483 L
.44562 .27291 L
.45101 .27021 L
.46074 .26379 L
.47093 .25507 L
.48202 .24363 L
.50215 .219 L
.54298 .16383 L
.56328 .13863 L
.5823 .11824 L
.60073 .10197 L
.6201 .08857 L
.64033 .07825 L
.65884 .07157 L
.66841 .06895 L
.67862 .06669 L
.697 .06372 L
.70727 .06254 L
.71839 .06157 L
.72889 .06088 L
.73855 .0604 L
.74827 .06003 L
.75736 .05977 L
.76732 .05955 L
.77801 .05938 L
.78817 .05927 L
.79353 .05922 L
.79922 .05918 L
.80967 .05912 L
.81934 .05908 L
.82869 .05906 L
.83396 .05905 L
.83879 .05904 L
.84768 .05903 L
.85706 .05902 L
.86205 .05901 L
.86659 .05901 L
.87147 .05901 L
.87677 .05901 L
.88168 .059 L
.88621 .059 L
.89513 .059 L
Mistroke
.8998 .059 L
.90481 .059 L
.91387 .059 L
.9194 .059 L
.92465 .059 L
.92982 .059 L
.9345 .059 L
.9441 .059 L
.94937 .059 L
.95432 .059 L
.96316 .059 L
.97272 .059 L
.97619 .059 L
Mfstroke
[ .02 .04 ] 0 setdash
.02381 .059 m
.06244 .059 L
.10458 .059 L
.14415 .059 L
.18221 .059 L
.22272 .059 L
.26171 .059 L
.30316 .059 L
.34309 .059 L
.3815 .059 L
.42237 .059 L
.46172 .059 L
.49955 .059 L
.53984 .059 L
.57861 .059 L
.61984 .059 L
.65954 .059 L
.69774 .059 L
.73838 .059 L
.77751 .059 L
.81909 .059 L
.85916 .059 L
.89771 .059 L
.93871 .059 L
.97619 .059 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00k`3oool01@2oM0000000001d_`3oool0gia80080000000<004RL0=ooo`3oool0]`3oool0
0>l0oooo00P0oooO09a8M02oool0oooo0?oog`2LB00004RL0=oookT0oooo003`0?ooo`060?oo_`1d
B9`0gooo0;md000007@0_ooo^P3oool00?40oooo00@0omnL07A8B0000000M;oo^`3oool00?40oooo
00<0oono07@00018W=l0_03oool00?00oooo00D0oooO09a80000B9`0W7A809cOo`2k0?ooo`00l03o
ool01P2oM000001809cOo`3ooml0W4Qd0;oookX0oooo003^0?ooo`090?oog`2LB000000000000000
M;l0okmd0000000007@0_ooo0;T0oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo001Q0?ooo`050?oOW018000000000000
M02oool0903oool01@3ogi`0B0000000000007@0_ooo00<0oooo00H0okmd0000001d_ol0oooo0?oo
g`2LB0030000000307Boo`3oool0oooo02<0oooo00H0_g@000000000000000000018W03OoolS0?oo
o`060;md00000000000000000000B9`0gooo0P3oool01P3o_g@0000007Boo`3oool0oooO09a800<0
000000<0M;oo0?ooo`3oool08@3oool00`2oM0000000000000040000000304RLg`3oool0oooo0200
oooo00<0_g@0000000000000100000002@18W=l0oooo0?ooo`3o_g@0000007Boo`3oool0oooO09a8
00030000000307Boo`3oool0oooo0240oooo00<0oono07@0000000000P0000000`1d_ol0oooo0?oo
o`0X0?ooo`00H03oool01`3ookl0M00007Boo`3oool0okmd0000M02oool08P3oool01`3ookl0M000
07Boo`3oool0okmd0000M02oool00P3oool01`3o_g@0000007Boo`3oool0oooO09a8M02oool00P3o
ool00`2oM000M;oo0?ooo`0T0?ooo`030?oOW0180000M;oo02H0oooo00<0omnL04P0001d_ol0103o
ool01`3o_g@0000007Boo`3oool0oooO09a8M02oool00P3oool00`2oM000M;oo0?ooo`0R0?ooo`03
0=nLB0000000000000<0000000<004RL0=ooo`3oool08@3oool00`3OW4P00000000000030000000:
0018W03Oool0oooo0?noM0000000M;oo0?ooo`3ooml0W4Qd0;ooo`80oooo00<0_g@007Boo`3oool0
8@3oool00`3ogi`0B00004RLg`020?ooo`030;md0018W=l0oooo02P0oooo001P0?ooo`030;md0000
B9`0gooo0080oooo00<0_g@0001d_`3oool08@3oool00`2oM00004RL0=ooo`020?ooo`030;md0000
M;l0oooo00X0oooo00<0oooO09a8M02oool0903oool00`3ogi`0B00007Boo`0V0?ooo`030?oOW018
0000M;oo00d0oooo00<0oooO09a8M02oool08`3oool00`3OW4P0B9cO0?ooo`020?ooo`030=nLM02L
gol0oooo0280oooo00<0gia804RLg`3oool00P3oool00`3OW7@0W=oo0?ooo`090?ooo`030?oog`2L
B7@0_ooo02H0oooo00<0oono07@0B02Lgol0:03oool005l0oooo00<0oono07@00018W=l00`3oool0
0`3OW4P0001d0;ooo`0P0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool02`3oool0
0`2oM000M;oo0?ooo`0S0?ooo`030?oOW0180000M;oo02H0oooo00<0omnL04P0001d_ol03P3oool0
0`2oM000M;oo0?ooo`0S0?ooo`030=nLB01d_ol0oooo02L0oooo00<0gia807Boo`3oool03P3oool0
0`2oM000M;oo0?ooo`0U0?ooo`030?oo_`1d0000M;oo02P0oooo001O0?ooo`030?oOW0180000M;oo
00<0oooo00<0okmd0000B02Lgol0803oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo
00X0oooo00<0oooO09a80018W=l0903oool00`3ogi`0B00007Boo`0V0?ooo`030?oOW0180000M;oo
00d0oooo00<0oooO09a80018W=l09@3oool00`2oM4P0W=oo0?ooo`0W0?ooo`030;mdB02Lgol0oooo
00`0oooo00<0oooO09a80018W=l09P3oool00`3o_g@0000007Boo`0X0?ooo`00G`3oool00`3ogi`0
B00007Boo`030?ooo`030?noM00004P0W=oo0200oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0
001809cOo`0:0?ooo`030=nLB0000000M;oo02@0oooo00<0omnL04P0001d_ol09P3oool00`3ogi`0
B00007Boo`0=0?ooo`030=nLB0000000M;oo02D0oooo00<0oooO09a8M02oool09`3oool00`3ooml0
W4Qd0;ooo`0<0?ooo`030=nLB0000000M;oo02D0oooo00@0omnL04P0000004P0W=oo:03oool005l0
oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0P0?ooo`030?oOW0180000M;oo00<0
oooo00<0okmd0000B02Lgol01P3oool01`3Ogml0oooo0?oo_`1d000000000000M02oool0903oool0
0`3ogi`0B00007Boo`0V0?ooo`030?oOW0180000M;oo00T0oooo00L0gmoO0?ooo`3ookl0M0000000
000007@0_ooo02H0oooo00<0okmd001d_`3oool09`3oool00`3o_g@007Bo0?ooo`070?ooo`070=oO
g`3oool0oono07@000000000001d0;ooo`0S0?ooo`050?noM000000000000000B02Lgol0:@3oool0
05l0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0P0?ooo`030?oOW0180000M;oo
00<0oooo00<0okmd0000B02Lgol01P3oool01P3OW4P00000000000000000001809cOobD0oooo00<0
omnL04P0001d_ol09P3oool00`3ogi`0B00007Boo`090?ooo`060=nLB000000000000000000004P0
W=oo9`3oool00`3ookl0M01809cOo`0W0?ooo`030?oo_`1d04P0W=oo00L0oooo00H0gia800000000
000000000000B02LgolV0?ooo`030;md0000M;l0oooo02T0oooo001O0?ooo`030?oo_`1d0000B9cO
00<0oooo00<0gia80000M02oool0803oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo
00H0oooo00@0oono07@000000000B9cO9`3oool00`3ogi`0B00007Boo`0V0?ooo`030?oOW0180000
M;oo00T0oooo00@0oono07@000000000B9cO:@3oool00`3ookl0M00004RLg`0W0?ooo`030?oo_`1d
0000B9cO00L0oooo00@0oono07@000000000B9cO:@3oool00`2oM000B9cO0?ooo`0X0?ooo`00H03o
ool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo0240oooo00<0gia80018W03Oool00P3o
ool00`2oM00007Bo0?ooo`070?ooo`030;md001d_ol0oooo02L0oooo00<0omnL04P0001d_ol09P3o
ool00`3ogi`0B00007Boo`0:0?ooo`030;md001d_ol0oooo02@0oooo00<0oknL09cOo`3oool00P3o
ool00`3ogi`0B00004RLg`0R0?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l0203o
ool00`2oM000M;oo0?ooo`0T0?ooo`030?oog`2LW;l0oooo0080oooo00<0gia80000M02oool0:03o
ool00600oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo0280oooo00L0oono07@0001d_ol0
oooo0?noM00007@0_ooo00P0oooo00H0omnL04P00000000000000000B02LgolR0?ooo`050?oog`2L
B00000000000001d_ol0903oool01@3ooml0W4P0000000000000M;oo00X0oooo00H0omnL04P00000
000000000000B02LgolQ0?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02ooolR0?ooo`08
0?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02oool80?ooo`060?oOW018000000000000000004P0
W=oo8P3oool01P3o_g@0B9cO0?ooo`3ookl0M000001d_bT0oooo001Q0?ooo`050?oo_`1d00000000
0000M02oool0903oool01@3ookl0M0000000000007@0_ooo00X0oooo00D0_g@00000000000000000
04RLg`0T0?ooo`030?noM0000000M;oo02H0oooo00<0okmd0000001d_ol02`3oool01@2oM0000000
000000000000B9cO0280oooo00H0oooO09a80000000000000000M02ooolT0?ooo`060?oog`2LB000
00000000000007@0_ooo2P3oool01@2oM0000000000000000000B9cO02<0oooo00D0gia800000000
0000001d0;ooo`0Y0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00G03ooooo
0000038000008`3oool005`0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`3o0?ooobh0oooo
00<000000?ooo`3oool08@3oool005`0oooo0P00003o0?ooobd0oooo0`00000S0?ooo`00G03oool0
0`000000oooo0?ooo`1j0?ooo`X0o`00Z@3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`00
0000oooo0?ooo`1f0?ooo`@0o`002P3oool40?l00:D0oooo00<000000?ooo`3oool08@3oool005`0
oooo00<000000?ooo`3oool0L`3oool30?l00180oooo0`3o002R0?ooo`030000003oool0oooo0240
oooo001L0?ooo`030000003oool0oooo0740oooo0P3o000H0?ooo`<0o`00W`3oool00`000000oooo
0?ooo`0Q0?ooo`00G03oool200000700oooo0P3o000M0?ooo`80o`00V`3oool3000002<0oooo001L
0?ooo`030000003oool0oooo06d0oooo0P3o000Q0?ooo`80o`00V`3oool00`000000oooo0?ooo`0Q
0?ooo`00G03oool00`000000oooo0?ooo`1[0?ooo`80o`009@3oool00`3o0000oooo0?ooo`2H0?oo
o`030000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo06T0oooo0P3o000X0?ooo`80
o`00V03oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`1X0?ooo`030?l0
003oool0oooo02X0oooo00<0o`000?ooo`3oool0U@3oool00`000000oooo0?ooo`0Q0?ooo`00G03o
ool2000006L0oooo0P3o000^0?ooo`030?l0003oool0oooo0980oooo0`00000S0?ooo`00G03oool0
0`000000oooo0?ooo`1U0?ooo`030?l0003oool0oooo02l0oooo00<0o`000?ooo`3oool0T`3oool0
0`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`1T0?ooo`030?l0003oool0oooo
0340oooo00<0o`000?ooo`3oool0TP3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000
oooo0?ooo`1S0?ooo`030?l0003oool0oooo03<0oooo00<0o`000?ooo`3oool0T@3oool00`000000
oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`040?ooo`P000002P3oool7000000`0oooo
1`00000;0?ooo`L000002`3oool7000000/0oooo1`00000;0?ooo`L00000303oool7000000/0oooo
1`00000;0?ooo`L000002`3oool7000000/0oooo2000000;0?ooo`L000001@3ooolk000000L0oooo
00<000000?ooo`3oool08@3oool004`0oooo00<0oono07@0000000000P0000000`1d_ol0oooo0?oo
o`080?ooo`8000003@3ooolL000003T0oooo00<0o`000?ooo`3oool0=`3oool20?l003H0oooo6000
00100?ooo`<000008`3oool004`0oooo00<0omnL04P00018W=l00P3oool00`2oM000B9cO0?ooo`08
0?ooo`030000003oool0oooo02P0oooo2P00000^0?ooo`030?l0003oool0oooo03X0oooo00<0o`00
0?ooo`3oool0;`3oool4000005X0oooo00<000000?ooo`3oool08@3oool00540oooo00<0oono07@0
B02Lgol0203oool00`000000oooo0?ooo`0b0?ooo`D00000:03oool00`3o0000oooo0?ooo`0l0?oo
o`030?l0003oool0oooo02H0oooo2000001N0?ooo`030000003oool0oooo0240oooo001A0?ooo`03
0?oo_`1d0000M;oo00P0oooo00<000000?ooo`3oool0=`3oool3000002@0oooo00<0o`000?ooo`3o
ool0?P3oool00`3o0000oooo0?ooo`0R0?ooo`<00000IP3oool00`000000oooo0?ooo`0Q0?ooo`00
D@3oool00`3o_g@0000007Boo`080?ooo`030000003oool0oooo03X0oooo0`00000P0?ooo`030?l0
003oool0oooo0400oooo00<0o`000?ooo`3oool07P3oool3000006T0oooo00<000000?ooo`3oool0
8@3oool00500oooo00@0omnL04P0000004P0W=oo203oool3000003d0oooo0P00000M0?ooo`030?l0
003oool0oooo0480oooo00<0o`000?ooo`3oool06`3oool2000006X0oooo0`00000S0?ooo`00CP3o
ool01@3o_g@000000000000004P0W=oo00T0oooo00<000000?ooo`3oool0?`3oool2000001X0oooo
00<0o`000?ooo`3oool0A03oool00`3o0000oooo0?ooo`0H0?ooo`800000KP3oool00`000000oooo
0?ooo`0Q0?ooo`00D03oool00`2oM00007Bo0?ooo`090?ooo`030000003oool0oooo0440oooo0P00
000G0?ooo`030?l0003oool0oooo04H0oooo00<0o`000?ooo`3oool05@3oool200000700oooo00<0
00000?ooo`3oool08@3oool00540oooo00<0_g@004RLg`3oool0203oool00`000000oooo0?ooo`13
0?ooo`800000503oool00`3o0000oooo0?ooo`170?ooo`030?l0003oool0oooo01<0oooo0P00001b
0?ooo`030000003oool0oooo0240oooo001<0?ooo`030?oog`2LW;l0oooo0080oooo00<0gia80000
M02oool0203oool00`000000oooo0?ooo`150?ooo`8000004@3oool00`3o0000oooo0?ooo`190?oo
o`030?l0003oool0oooo0100oooo0P00001d0?ooo`030000003oool0oooo0240oooo001=0?ooo`06
0?noM018W=l0oooo0?oo_`1d000007Bo2@3oool2000004P0oooo00<000000?ooo`3oool03P3oool0
0`3o0000oooo0?ooo`1:0?ooo`030?l0003oool0oooo00h0oooo00<000000?ooo`3oool0LP3oool3
000002<0oooo001>0?ooo`050=nLB000000000000000M02oool02@3oool00`000000oooo0?ooo`18
0?ooo`030000003oool0oooo00`0oooo00<0o`000?ooo`3oool0C03oool00`3o0000oooo0?ooo`0;
0?ooo`800000M`3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`190?oo
o`030000003oool0oooo00X0oooo00<0o`000?ooo`3oool0CP3oool00`3o0000oooo0?ooo`090?oo
o`030000003oool0oooo07L0oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3o
ool0BP3oool2000000T0oooo00<0o`000?ooo`3oool0C`3oool00`3o0000oooo0?ooo`080?ooo`03
0000003oool0oooo07P0oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0
C03oool00`000000oooo0?ooo`060?ooo`030?l0003oool0oooo0500oooo00<0o`000?ooo`3oool0
1P3oool00`000000oooo0?ooo`1i0?ooo`030000003oool0oooo0240oooo001L0?ooo`800000CP3o
ool00`000000oooo0?ooo`040?ooo`030?l0003oool0oooo0580oooo00<0o`000?ooo`3oool0103o
ool00`000000oooo0?ooo`1h0?ooo`<000008`3oool005`0oooo00<000000?ooo`3oool0CP3oool0
0`000000oooo0?ooo`020?ooo`030?l0003oool0oooo05@0oooo00<0o`000?ooo`3oool00P3oool0
0`000000oooo0?ooo`1k0?ooo`030000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo
04l0oooo00D000000?ooo`3oool0oooo0?l0001G0?ooo`040?l0003oool0oooo000007h0oooo00<0
00000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0D03oool00`000000oooo0?l0001I
0?ooo`030?l000000000oooo07h0oooo00<000000?ooo`3oool08@3oool005`0oooo0P00001B0?oo
o`030000003oool0oooo05P0oooo00<000000?ooo`3oool0O03oool3000002<0oooo001L0?ooo`03
0000003oool0oooo0540oooo00<0o`000000003oool0E`3oool00`000000oooo0?l0001o0?ooo`03
0000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo0500oooo00<0o`000?ooo`000000
F03oool010000000oooo0?ooo`3o001n0?ooo`030000003oool0oooo0240oooo001L0?ooo`030000
003oool0oooo04l0oooo00D0o`000?ooo`3oool0oooo0000001F0?ooo`030000003oool0oooo0080
oooo00<0o`000?ooo`3oool0N`3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo
0?ooo`1>0?ooo`030?l0003oool0oooo00<0oooo00<000000?ooo`3oool0DP3oool00`000000oooo
0?ooo`030?ooo`030?l0003oool0oooo07/0oooo00<000000?ooo`3oool08@3oool003l0oooo00<0
oono07@0000000000P0000000`1d_ol0oooo0?ooo`020?ooo`060?noM0000000M;oo0?ooo`3ooml0
W4P00`0000000`1d_ol0oooo0?ooo`070?ooo`800000C`3oool00`3o0000oooo0?ooo`040?ooo`03
0000003oool0oooo0500oooo00<000000?ooo`3oool01@3oool00`3o0000oooo0?ooo`1h0?ooo`<0
00008`3oool003l0oooo00<0omnL04P00018W=l00P3oool00`2oM000B9cO0?ooo`020?ooo`070?no
M0000000M;oo0?ooo`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0oooo00L0oooo00<000000?oo
o`3oool0C@3oool00`3o0000oooo0?ooo`060?ooo`030000003oool0oooo04h0oooo00<000000?oo
o`3oool01`3oool00`3o0000oooo0?ooo`1i0?ooo`030000003oool0oooo0240oooo00140?ooo`03
0?oo_`1d04P0W=oo00/0oooo00<0oooO09a8M02oool01`3oool00`000000oooo0?ooo`1<0?ooo`03
0?l0003oool0oooo00P0oooo00<000000?ooo`3oool0C03oool00`000000oooo0?ooo`080?ooo`03
0?l0003oool0oooo07T0oooo00<000000?ooo`3oool08@3oool004@0oooo00<0oono07@0001d_ol0
303oool00`2oM000M;oo0?ooo`060?ooo`030000003oool0oooo04`0oooo00<0o`000?ooo`3oool0
2@3oool00`000000oooo0?ooo`1:0?ooo`030000003oool0oooo00X0oooo00<0o`000?ooo`3oool0
N03oool00`000000oooo0?ooo`0Q0?ooo`00A03oool00`3o_g@0000007Boo`0;0?ooo`030?oog`2L
B000B9cO00L0oooo00<000000?ooo`3oool0B`3oool00`3o0000oooo0?ooo`0;0?ooo`030000003o
ool0oooo04T0oooo00<000000?ooo`3oool02`3oool00`3o0000oooo0?ooo`1g0?ooo`030000003o
ool0oooo0240oooo00130?ooo`040?oOW0180000001809cOo`/0oooo00<0gia80000001d_ol01`3o
ool3000004X0oooo00<0o`000?ooo`3oool03@3oool00`000000oooo0?ooo`170?ooo`030000003o
ool0oooo00`0oooo00<0o`000?ooo`3oool0M@3oool3000002<0oooo00110?ooo`050?noM0000000
00000000B02Lgol0203oool01`3Ogml0oooo0?oo_`1d000000000000M02oool01`3oool00`000000
oooo0?ooo`190?ooo`030?l0003oool0oooo00h0oooo00<000000?ooo`3oool0AP3oool00`000000
oooo0?ooo`0>0?ooo`030?l0003oool0oooo07H0oooo00<000000?ooo`3oool08@3oool004<0oooo
00<0_g@0001d_`3oool0203oool01P3OW4P00000000000000000001809cOo`P0oooo00<000000?oo
o`3oool0B@3oool00`3o0000oooo0?ooo`0?0?ooo`030000003oool0oooo04@0oooo00<000000?oo
o`3oool0403oool00`3o0000oooo0?ooo`1e0?ooo`030000003oool0oooo0240oooo00140?ooo`03
0;md0018W=l0oooo00L0oooo00@0oono07@000000000B9cO2P3oool00`000000oooo0?ooo`180?oo
o`030?l0003oool0oooo0140oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0A0?oo
o`030?l0003oool0oooo07D0oooo00<000000?ooo`3oool08@3oool003l0oooo00<0oooO09bL_`3o
ool00P3oool00`3OW4P0001d0;ooo`080?ooo`030;md001d_ol0oooo00X0oooo00<000000?ooo`3o
ool0A`3oool00`3o0000oooo0?ooo`0C0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3o
ool04`3oool00`3o0000oooo0?ooo`1d0?ooo`030000003oool0oooo0240oooo00100?ooo`060?no
M018W=l0oooo0?oo_`1d000007Bo2@3oool01P3ogi`0B000000000000000001809cOo`L0oooo0P00
00180?ooo`030?l0003oool0oooo01<0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?oo
o`0D0?ooo`030?l0003oool0oooo0740oooo0`00000S0?ooo`00@@3oool01@3OW4P0000000000000
07@0_ooo00X0oooo00D0_g@0000000000000000004RLg`070?ooo`030000003oool0oooo04H0oooo
00<0o`000?ooo`3oool05@3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo01D0oooo
00<0o`000?ooo`3oool0L`3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?oo
o`150?ooo`030?l0003oool0oooo01L0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?oo
o`0G0?ooo`030?l0003oool0oooo0780oooo00<000000?ooo`3oool08@3oool005`0oooo00<00000
0?ooo`3oool0A@3oool00`3o0000oooo0?ooo`0H0?ooo`030000003oool0oooo03X0oooo00<00000
0?ooo`3oool06@3oool00`3o0000oooo0?ooo`1a0?ooo`030000003oool0oooo0240oooo001L0?oo
o`030000003oool0oooo04@0oooo00<0o`000?ooo`3oool06@3oool00`000000oooo0?ooo`0j0?oo
o`030000003oool0oooo01T0oooo00<0o`000?ooo`3oool0L@3oool00`000000oooo0?ooo`0Q0?oo
o`00G03oool2000004@0oooo00<0o`000?ooo`3oool06`3oool00`000000oooo0?ooo`0h0?ooo`03
0000003oool0oooo01/0oooo00<0o`000?ooo`3oool0KP3oool3000002<0oooo001L0?ooo`030000
003oool0oooo0480oooo00<0o`000?ooo`3oool07@3oool00`000000oooo0?ooo`0f0?ooo`030000
003oool0oooo01d0oooo00<0o`000?ooo`3oool0K`3oool00`000000oooo0?ooo`0Q0?ooo`00G03o
ool00`000000oooo0?ooo`120?ooo`030?l0003oool0oooo01h0oooo00<000000?ooo`3oool0=03o
ool00`000000oooo0?ooo`0N0?ooo`030?l0003oool0oooo06l0oooo00<000000?ooo`3oool08@3o
ool005`0oooo00<000000?ooo`3oool0@@3oool00`3o0000oooo0?ooo`0O0?ooo`030000003oool0
oooo03@0oooo00<000000?ooo`3oool07`3oool00`3o0000oooo0?ooo`1^0?ooo`030000003oool0
oooo0240oooo001L0?ooo`030000003oool0oooo0400oooo00<0o`000?ooo`3oool08@3oool00`00
0000oooo0?ooo`0b0?ooo`030000003oool0oooo0200oooo00<0o`000?ooo`3oool0KP3oool00`00
0000oooo0?ooo`0Q0?ooo`00G03oool200000440oooo00<0o`000?ooo`3oool08P3oool00`000000
oooo0?ooo`0`0?ooo`030000003oool0oooo0280oooo00<0o`000?ooo`3oool0J`3oool3000002<0
oooo001L0?ooo`030000003oool0oooo03l0oooo00<0o`000?ooo`3oool0903oool00`000000oooo
0?ooo`0^0?ooo`030000003oool0oooo02@0oooo00<0o`000?ooo`3oool0K03oool00`000000oooo
0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`0n0?ooo`030?l0003oool0oooo02D0oooo00<0
00000?ooo`3oool0;P3oool00`000000oooo0?ooo`0T0?ooo`030?l0003oool0oooo06`0oooo00<0
00000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0?@3oool00`3o0000oooo0?ooo`0W
0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool09P3oool00`3o0000oooo0?ooo`1[
0?ooo`030000003oool0oooo0240oooo001@0?ooo`040?oog`2LB00004RL0=ooo`P0oooo0P00000n
0?ooo`030?l0003oool0oooo02P0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0W
0?ooo`030?l0003oool0oooo06T0oooo0`00000S0?ooo`00D03oool0103ooml0W4P00018W03Oool8
0?ooo`030000003oool0oooo03`0oooo00<0o`000?ooo`3oool0:P3oool00`000000oooo0?ooo`0X
0?ooo`030000003oool0oooo02T0oooo00<0o`000?ooo`3oool0JP3oool00`000000oooo0?ooo`0Q
0?ooo`00D03oool0103ooml0W4P00018W03Oool80?ooo`030000003oool0oooo03/0oooo00<0o`00
0?ooo`3oool0:`3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02/0oooo00<0o`00
0?ooo`3oool0J@3oool00`000000oooo0?ooo`0Q0?ooo`00C03oool00`2oM0000000000000050000
000307Boo`3oool0oooo00D0oooo00<000000?ooo`3oool0>P3oool00`3o0000oooo0?ooo`0]0?oo
o`030000003oool0oooo02D0oooo00<000000?ooo`3oool0;03oool00`3o0000oooo0?ooo`1Y0?oo
o`030000003oool0oooo0240oooo001<0?ooo`080?oOW01dW=l0oooo0?ooo`3ooml0W4P00018W03O
ool80?ooo`030000003oool0oooo03X0oooo00<0o`000?ooo`3oool0;P3oool00`000000oooo0?oo
o`0T0?ooo`030000003oool0oooo02d0oooo00<0o`000?ooo`3oool0J03oool00`000000oooo0?oo
o`0Q0?ooo`00C@3oool01`3OW4P0M;oo0?ooo`3ooml0W4P00018W03Oool0203oool3000003T0oooo
00<0o`000?ooo`3oool0<03oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo02h0oooo
00<0o`000?ooo`3oool0IP3oool3000002<0oooo001=0?ooo`070?oog`2LB7@0_ooo0?oog`2LB000
04RL0=ooo`080?ooo`030000003oool0oooo03P0oooo00<0o`000?ooo`3oool0<P3oool00`000000
oooo0?ooo`0P0?ooo`030000003oool0oooo0300oooo00<0o`000?ooo`3oool0I`3oool00`000000
oooo0?ooo`0Q0?ooo`00CP3oool01P3ogi`0B7Bo0?oog`2LB00004RL0=ooo`P0oooo00<000000?oo
o`3oool0>03oool00`3o0000oooo0?ooo`0c0?ooo`030000003oool0oooo01h0oooo00<000000?oo
o`3oool0<P3oool00`3o0000oooo0?ooo`1V0?ooo`030000003oool0oooo0240oooo001?0?ooo`05
0=nLB01d_ml0W4P00018W03Oool0203oool00`000000oooo0?ooo`0g0?ooo`030?l0003oool0oooo
03D0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0c0?ooo`030?l0003oool0oooo
06H0oooo00<000000?ooo`3oool08@3oool004l0oooo00D0oooO09a8B018B00004RL0=ooo`080?oo
o`030000003oool0oooo03H0oooo00<0o`000?ooo`3oool0=`3oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo03D0oooo00<0o`000?ooo`3oool0I@3oool00`000000oooo0?ooo`0Q0?oo
o`00D03oool0103ogi`0B0000018W03Oool80?ooo`800000=P3oool00`3o0000oooo0?ooo`0i0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool0=@3oool00`3o0000oooo0?ooo`1S0?oo
o`<000008`3oool00540oooo00<0gia80018W03Oool0203oool00`000000oooo0?ooo`0e0?ooo`03
0?l0003oool0oooo03X0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0g0?ooo`03
0?l0003oool0oooo06@0oooo00<000000?ooo`3oool08@3oool002L0oooo00<0giad09cOo`3oool0
<P3oool00`000000oooo0?ooo`0d0?ooo`030?l0003oool0oooo03/0oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`0h0?ooo`030?l0003oool0oooo06@0oooo00<000000?ooo`3oool0
8@3oool002L0oooo00<0gia8001d_`3oool0<P3oool00`000000oooo0?ooo`0c0?ooo`030?l0003o
ool0oooo03d0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0j0?ooo`030?l0003o
ool0oooo06<0oooo00<000000?ooo`3oool08@3oool002L0oooo00D0gia8000000000000000007Bo
o`0`0?ooo`030000003oool0oooo03<0oooo00<0o`000?ooo`3oool0?P3oool2000001<0oooo00<0
00000?ooo`3oool0>`3oool00`3o0000oooo0?ooo`1S0?ooo`030000003oool0oooo0240oooo000W
0?ooo`030=nLB0000000000000<0000000<0001d0;ooo`3oool0;03oool2000003<0oooo00<0o`00
0?ooo`3oool0@@3oool00`000000oooo0?ooo`0>0?ooo`800000?`3oool00`3o0000oooo0?ooo`1P
0?ooo`<000008`3oool002L0oooo00H0giad09cOo`3oool0oooo0?oOW0180003000000030018W03O
ool0oooo02T0oooo00<000000?ooo`3oool0<@3oool00`3o0000oooo0?ooo`130?ooo`8000002`3o
ool300000440oooo00<0o`000?ooo`3oool0HP3oool00`000000oooo0?ooo`0Q0?ooo`00;P3oool0
0`3o_g@000000000000200000003001d_`3oool0oooo02H0oooo00<000000?ooo`3oool0<@3oool0
0`3o0000oooo0?ooo`150?ooo`D000000`3oool3000004D0oooo00<0o`000?ooo`3oool0H@3oool0
0`000000oooo0?ooo`0Q0?ooo`00<@3oool0103o_g@000000000M02ooolW0?ooo`030000003oool0
oooo0300oooo00<0o`000?ooo`3oool0BP3oool4000004P0oooo00<0o`000?ooo`3oool0H@3oool0
0`000000oooo0?ooo`0Q0?ooo`00;P3oool0103o_g@0000000000000M;lZ0?ooo`030000003oool0
oooo02l0oooo00<0o`000?ooo`3oool0V03oool00`3o0000oooo0?ooo`1P0?ooo`030000003oool0
oooo0240oooo000W0?ooo`090=nLM02Lgol0oooo0?ooo`3ogi`0B00000000000B9`0gooo02`0oooo
0P00000_0?ooo`030?l0003oool0oooo09T0oooo00<0o`000?ooo`3oool0GP3oool3000002<0oooo
000W0?ooo`060=nLB000000000000000000007@0_ooo;`3oool00`000000oooo0?ooo`0^0?ooo`03
0?l0003oool0oooo09X0oooo00<0o`000?ooo`3oool0G`3oool00`000000oooo0?ooo`0Q0?ooo`00
9`3oool00`3OW4P0000007Boo`0b0?ooo`030000003oool0oooo02d0oooo00<0o`000?ooo`3oool0
V`3oool00`3o0000oooo0?ooo`1O0?ooo`030000003oool0oooo0240oooo000W0?ooo`030=nLM02L
gol0oooo0380oooo00<000000?ooo`3oool0;03oool00`3o0000oooo0?ooo`2M0?ooo`030?l0003o
ool0oooo05h0oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0;03oool0
0`3o0000oooo0?ooo`2M0?ooo`030?l0003oool0oooo05h0oooo00<000000?ooo`3oool08@3oool0
04<0oooo00@0oooO09a80000B9`0gooo0P3oool01P3o_g@0000007Boo`3oool0oooO09a800<00000
00<0M;oo0?ooo`3oool01`3oool2000002`0oooo00<0o`000?ooo`3oool0W`3oool00`3o0000oooo
0?ooo`1K0?ooo`<000008`3oool00340oooo00H0oooO09a80000B9`0gooo0?oog`2LW;l<0?ooo`04
0?oog`2LB00004RL0=ooo`80oooo00L0okmd0000001d_ol0oooo0?oog`2LB7@0_ooo0080oooo00<0
_g@007Boo`3oool01`3oool00`000000oooo0?ooo`0Z0?ooo`030?l0003oool0oooo0:00oooo00<0
o`000?ooo`3oool0G@3oool00`000000oooo0?ooo`0Q0?ooo`00<@3oool01P3ookl0M0000018W03O
ool0_gA809cOo``0oooo00@0oooO09a80000B9`0gooo2`3oool00`3ooml0W4Qd0;ooo`070?ooo`03
0000003oool0oooo02X0oooo00<0o`000?ooo`3oool0X@3oool00`3o0000oooo0?ooo`1L0?ooo`03
0000003oool0oooo0240oooo000b0?ooo`040=nLB0000000000007Boo`T0oooo00<0_g@000000000
00001@0000000`1d_ol0oooo0?ooo`090?ooo`030;md001d_ol0oooo00H0oooo00<000000?ooo`3o
ool0:@3oool00`3o0000oooo0?ooo`2R0?ooo`030?l0003oool0oooo05`0oooo00<000000?ooo`3o
ool08@3oool003l0oooo00P0omnL07BLg`3oool0oooo0?oog`2LB00004RL0=ooo`/0oooo00<0oooO
09a80018W=l01`3oool3000002P0oooo00<0o`000?ooo`3oool0Y03oool00`3o0000oooo0?ooo`1I
0?ooo`<000008`3oool00400oooo00L0gia807Boo`3oool0oooO09a80000B9`0gooo00/0oooo00<0
gia80000001d_ol01`3oool00`000000oooo0?ooo`0W0?ooo`030?l0003oool0oooo0:D0oooo00<0
o`000?ooo`3oool0F`3oool00`000000oooo0?ooo`0Q0?ooo`00@03oool01`3ooml0W4Qd0;ooo`3o
oml0W4P00018W03Oool01`3oool01`3Ogml0oooo0?oo_`1d000000000000M02oool01`3oool00`00
0000oooo0?ooo`0V0?ooo`030?l0003oool0oooo0:L0oooo00<0o`000?ooo`3oool0FP3oool00`00
0000oooo0?ooo`0Q0?ooo`00@@3oool01P3ogi`0B7Bo0?oog`2LB00004RL0=ooo`L0oooo00H0gia8
00000000000000000000B02Lgol80?ooo`030000003oool0oooo02H0oooo00<0o`000?ooo`3oool0
Y`3oool00`3o0000oooo0?ooo`1J0?ooo`030000003oool0oooo0240oooo00120?ooo`050=nLB01d
_ml0W4P00018W03Oool01`3oool0103ookl0M00000000018W=l:0?ooo`030000003oool0oooo02D0
oooo00<0o`000?ooo`3oool0Z@3oool00`3o0000oooo0?ooo`1I0?ooo`030000003oool0oooo0240
oooo00120?ooo`050?oog`2LB4P0B4P00018W03Oool0203oool00`2oM000M;oo0?ooo`0:0?ooo`80
00009@3oool00`3o0000oooo0?ooo`2Z0?ooo`030?l0003oool0oooo05L0oooo0`00000S0?ooo`00
:`3oool01@3ogi`0M9cO0?ooo`3ookl0M0000080000000<0001d0;ooo`3oool03P3oool0103ogi`0
B0000018W03Oool80?ooo`060?oOW018000000000000000004P0W=oo1`3oool00`000000oooo0?oo
o`0S0?ooo`030?l0003oool0oooo0:`0oooo00<0o`000?ooo`3oool0F03oool00`000000oooo0?oo
o`0Q0?ooo`00:`3oool00`3ooml0W4P0000000030000000407Bo_`1d000004RL0=ooo`l0oooo00<0
gia80018W03Oool02@3oool01@2oM0000000000000000000B9cO00L0oooo00<000000?ooo`3oool0
8P3oool00`3o0000oooo0?ooo`2]0?ooo`030?l0003oool0oooo05P0oooo00<000000?ooo`3oool0
8@3oool003<0oooo00H0_gA809cO_`1d000000000018W03OoolS0?ooo`030000003oool0oooo0240
oooo00<0o`000?ooo`3oool0[`3oool00`3o0000oooo0?ooo`1G0?ooo`030000003oool0oooo0240
oooo000a0?ooo`050=nLB000000000000000M02oool09P3oool00`000000oooo0?ooo`0P0?ooo`03
0?l0003oool0oooo0;00oooo00<0o`000?ooo`3oool0E`3oool00`000000oooo0?ooo`0Q0?ooo`00
;@3oool0203o_g@00000000000000000B9cO0?ooo`2oM4P0W=oo9`3oool200000240oooo00<0o`00
0?ooo`3oool0/@3oool00`3o0000oooo0?ooo`1D0?ooo`<000008`3oool002T0oooo00H0_g@00000
0000000000000000M02oool30?ooo`030?oo_`1dB9`0gooo02L0oooo00<000000?ooo`3oool07`3o
ool00`3o0000oooo0?ooo`2b0?ooo`030?l0003oool0oooo05H0oooo00<000000?ooo`3oool08@3o
ool002L0oooo00@0oooO09a80000000007Bo1P3oool00`3ooml0W4P004RLg`0X0?ooo`030000003o
ool0oooo01h0oooo00<0o`000?ooo`3oool0]03oool00`3o0000oooo0?ooo`1E0?ooo`030000003o
ool0oooo0240oooo000_0?ooo`040;md000000000000001d_bT0oooo00<000000?ooo`3oool07@3o
ool00`3o0000oooo0?ooo`2e0?ooo`030?l0003oool0oooo05D0oooo00<000000?ooo`3oool08@3o
ool002`0oooo00H0oooO09a80000000000000000B02LgolZ0?ooo`030000003oool0oooo01/0oooo
0P3o002i0?ooo`030?l0003oool0oooo05@0oooo00<000000?ooo`3oool08@3oool002/0oooo00@0
oooO09a80000B9`0gooo;@3oool2000001/0oooo00<0o`000?ooo`3oool0^@3oool00`3o0000oooo
0?ooo`1B0?ooo`<000008`3oool002/0oooo00<0omnL07BLg`3oool0;P3oool00`000000oooo0?oo
o`0I0?ooo`030?l0003oool0oooo0;/0oooo00<0o`000?ooo`3oool0D`3oool00`000000oooo0?oo
o`0Q0?ooo`00G03oool00`000000oooo0?ooo`0G0?ooo`80o`00_P3oool00`3o0000oooo0?ooo`1C
0?ooo`030000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo01H0oooo00<0o`000?oo
o`3oool0_`3oool00`3o0000oooo0?ooo`1B0?ooo`030000003oool0oooo0240oooo001L0?ooo`03
0000003oool0oooo01@0oooo0P3o00320?ooo`030?l0003oool0oooo0580oooo00<000000?ooo`3o
ool08@3oool004`0oooo00<0oooO09a8000000000P0000000`1d_ol0oooo0?ooo`080?ooo`800000
503oool00`3o0000oooo0?ooo`320?ooo`030?l0003oool0oooo0500oooo0`00000S0?ooo`00C03o
ool00`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0oooo00P0oooo00<000000?ooo`3oool04@3o
ool20?l00<H0oooo00<0o`000?ooo`3oool0D@3oool00`000000oooo0?ooo`0Q0?ooo`00D@3oool0
0`3ooml0W4Qd0;ooo`080?ooo`030000003oool0oooo00l0oooo0P3o00380?ooo`030?l0003oool0
oooo0540oooo00<000000?ooo`3oool08@3oool00580oooo00<0_g@007Boo`3oool01`3oool00`00
0000oooo0?ooo`0=0?ooo`80o`00b`3oool00`3o0000oooo0?ooo`1@0?ooo`030000003oool0oooo
0240oooo001A0?ooo`030?oog`2LB000B9cO00P0oooo00<000000?ooo`3oool0203oool50?l00<d0
oooo00<0o`000?ooo`3oool0D03oool00`000000oooo0?ooo`0Q0?ooo`00D@3oool00`3OW4P00000
07Boo`080?ooo`<00000103oool40?l00=<0oooo00<0o`000?ooo`3oool0C@3oool3000002<0oooo
001=0?ooo`070=oOg`3oool0oono07@000000000001d0;ooo`080?ooo`030000003oool0oooo0=/0
oooo00<0o`000?ooo`3oool0C`3oool00`000000oooo0?ooo`0Q0?ooo`00C@3oool01P3OW4P00000
000000000000001809cOo`T0oooo00<000000?ooo`3oool0g03oool00`3o0000oooo0?ooo`1>0?oo
o`030000003oool0oooo0240oooo001=0?ooo`040?oo_`1d0000000004RLg`/0oooo00<000000?oo
o`3oool0g03oool00`3o0000oooo0?ooo`1>0?ooo`030000003oool0oooo0240oooo001>0?ooo`03
0;md001d_ol0oooo00/0oooo00<000000?ooo`3oool0g@3oool00`3o0000oooo0?ooo`1=0?ooo`03
0000003oool0oooo0240oooo001>0?ooo`060?oOW018000000000000000004P0W=oo203oool20000
0=h0oooo00<0o`000?ooo`3oool0B`3oool3000002<0oooo001?0?ooo`050;md0000000000000000
0018W=l0203oool00`000000oooo0?ooo`3N0?ooo`030?l0003oool0oooo04`0oooo00<000000?oo
o`3oool08@3oool005`0oooo00<000000?ooo`3oool0gP3oool00`3o0000oooo0?ooo`1<0?ooo`03
0000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo0=h0oooo00<0o`000?ooo`3oool0
C03oool00`000000oooo0?ooo`0Q0?ooo`00G03oool200000>00oooo00<0o`000?ooo`3oool0B@3o
ool3000002<0oooo001L0?ooo`030000003oool0oooo0=l0oooo00<0o`000?ooo`3oool0B`3oool0
0`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`3P0?ooo`030?l0003oool0oooo
04X0oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0h03oool00`3o0000
oooo0?ooo`1:0?ooo`030000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo0>40oooo
00<0o`000?ooo`3oool0B@3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool200000>80oooo00<0
o`000?ooo`3oool0A`3oool3000002<0oooo001L0?ooo`030000003oool0oooo0>80oooo00<0o`00
0?ooo`3oool0B03oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`3R0?oo
o`030?l0003oool0oooo04P0oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3o
ool0h`3oool00`3o0000oooo0?ooo`170?ooo`030000003oool0oooo0240oooo001L0?ooo`030000
003oool0oooo0><0oooo00<0o`000?ooo`3oool0A`3oool00`000000oooo0?ooo`0Q0?ooo`00?`3o
ool00`3ooml0W4P0000000020000000307Boo`3oool0oooo0080oooo00H0okmd0000001d_ol0oooo
0?oog`2LB0030000000307Boo`3oool0oooo00L0oooo0P00003U0?ooo`030?l0003oool0oooo04@0
oooo0`00000S0?ooo`00?`3oool00`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0oooo0080oooo
00L0okmd0000001d_ol0oooo0?oog`2LB7@0_ooo0080oooo00<0_g@007Boo`3oool01`3oool00`00
0000oooo0?ooo`3T0?ooo`030?l0003oool0oooo04H0oooo00<000000?ooo`3oool08@3oool004@0
oooo00<0oooO09a8M02oool02`3oool00`3ooml0W4Qd0;ooo`070?ooo`030000003oool0oooo0>@0
oooo00<0o`000?ooo`3oool0AP3oool00`000000oooo0?ooo`0Q0?ooo`00A@3oool00`2oM000M;oo
0?ooo`0;0?ooo`030;md001d_ol0oooo00H0oooo00<000000?ooo`3oool0i@3oool00`3o0000oooo
0?ooo`150?ooo`030000003oool0oooo0240oooo00140?ooo`030?oog`2LB000B9cO00/0oooo00<0
oooO09a80018W=l01`3oool00`000000oooo0?ooo`3U0?ooo`030?l0003oool0oooo04D0oooo00<0
00000?ooo`3oool08@3oool004@0oooo00<0gia80000001d_ol02`3oool00`3OW4P0000007Boo`07
0?ooo`<00000iP3oool00`3o0000oooo0?ooo`120?ooo`<000008`3oool00400oooo00L0gmoO0?oo
o`3ookl0M0000000000007@0_ooo00L0oooo00L0gmoO0?ooo`3ookl0M0000000000007@0_ooo00L0
oooo00<000000?ooo`3oool0iP3oool00`3o0000oooo0?ooo`140?ooo`030000003oool0oooo0240
oooo00100?ooo`060=nLB000000000000000000004P0W=oo203oool01P3OW4P00000000000000000
001809cOo`P0oooo00<000000?ooo`3oool0i`3oool00`3o0000oooo0?ooo`130?ooo`030000003o
ool0oooo0240oooo00100?ooo`040?oo_`1d0000000004RLg`X0oooo00@0oono07@000000000B9cO
2P3oool00`000000oooo0?ooo`3W0?ooo`030?l0003oool0oooo04<0oooo00<000000?ooo`3oool0
8@3oool00440oooo00<0_g@007Boo`3oool02`3oool00`2oM000M;oo0?ooo`0:0?ooo`030000003o
ool0oooo0>P0oooo00<0o`000?ooo`3oool0@P3oool00`000000oooo0?ooo`0Q0?ooo`00@@3oool0
1P3ogi`0B000000000000000001809cOo`P0oooo00H0omnL04P00000000000000000B02Lgol70?oo
o`800000j@3oool00`3o0000oooo0?ooo`100?ooo`<000008`3oool00480oooo00D0_g@000000000
0000000004RLg`090?ooo`050;md00000000000000000018W=l01`3oool00`000000oooo0?ooo`3Y
0?ooo`030?l0003oool0oooo0440oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?oo
o`3oool0j@3oool00`3o0000oooo0?ooo`110?ooo`030000003oool0oooo0240oooo001L0?ooo`03
0000003oool0oooo0>X0oooo00<0o`000?ooo`3oool0@03oool00`000000oooo0?ooo`0Q0?ooo`00
G03oool00`000000oooo0?ooo`3Z0?ooo`030?l0003oool0oooo0400oooo00<000000?ooo`3oool0
8@3oool005`0oooo0P00003[0?ooo`030?l0003oool0oooo03h0oooo0`00000S0?ooo`00G03oool0
0`000000oooo0?ooo`3[0?ooo`030?l0003oool0oooo03l0oooo00<000000?ooo`3oool08@3oool0
05`0oooo00<000000?ooo`3oool0j`3oool00`3o0000oooo0?ooo`0o0?ooo`030000003oool0oooo
0240oooo001L0?ooo`030000003oool0oooo0>`0oooo00<0o`000?ooo`3oool0?P3oool00`000000
oooo0?ooo`0Q0?ooo`00G03oool200000>d0oooo00<0o`000?ooo`3oool0?03oool3000002<0oooo
001L0?ooo`030000003oool0oooo0>`0oooo00<0o`000?ooo`3oool0?P3oool00`000000oooo0?oo
o`0Q0?ooo`00G03oool00`000000oooo0?ooo`3]0?ooo`030?l0003oool0oooo03d0oooo00<00000
0?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0k@3oool00`3o0000oooo0?ooo`0m0?oo
o`030000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo0>h0oooo00<0o`000?ooo`3o
ool0?03oool00`000000oooo0?ooo`0Q0?ooo`00CP3oool01@3ogi`0B000000000000000M;oo00T0
oooo0P00003_0?ooo`030?l0003oool0oooo03X0oooo0`00000S0?ooo`00C@3oool01`3ogi`0B000
04RLg`3oool0oono07@0001d_ol0203oool00`000000oooo0?ooo`3^0?ooo`030?l0003oool0oooo
03`0oooo00<000000?ooo`3oool08@3oool004`0oooo00<0oooO09a80000M;l00`3oool00`3o_g@0
04RL0=ooo`070?ooo`030000003oool0oooo0>l0oooo00<0o`000?ooo`3oool0>`3oool00`000000
oooo0?ooo`0Q0?ooo`00C03oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00L0oooo
00<000000?ooo`3oool0k`3oool00`3o0000oooo0?ooo`0k0?ooo`030000003oool0oooo0240oooo
001<0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01`3oool00`000000oooo0?oo
o`3`0?ooo`030?l0003oool0oooo03X0oooo00<000000?ooo`3oool08@3oool004`0oooo00<0omnL
04P0001d_ol00`3oool00`2oM000001809cOo`070?ooo`<00000l03oool00`3o0000oooo0?ooo`0h
0?ooo`<000008`3oool004`0oooo00P0oono07@000000000B9cO0?ooo`3o_g@00000001d_`P0oooo
00<000000?ooo`3oool0l03oool00`3o0000oooo0?ooo`0j0?ooo`030000003oool0oooo0240oooo
001=0?ooo`030;md0000B9`0_ia80080000000<004RL0=ooo`3oool01`3oool00`000000oooo0?oo
o`3a0?ooo`030?l0003oool0oooo03T0oooo00<000000?ooo`3oool08@3oool004d0oooo00<0omnL
04P0001d_ol0303oool00`000000oooo0?ooo`3a0?ooo`030?l0003oool0oooo03T0oooo00<00000
0?ooo`3oool08@3oool004h0oooo00<0gia80000B02Lgol02`3oool00`000000oooo0?ooo`3b0?oo
o`030?l0003oool0oooo03P0oooo00<000000?ooo`3oool08@3oool004l0oooo00<0okmd00000000
M;l02P3oool200000?<0oooo00<0o`000?ooo`3oool0=P3oool3000002<0oooo001@0?ooo`050?oo
g`2LB00000000000M02oool01`3oool00`000000oooo0?ooo`3b0?ooo`030?l0003oool0oooo03P0
oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0l`3oool00`3o0000oooo
0?ooo`0g0?ooo`030000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo0?<0oooo00<0
o`000?ooo`3oool0=`3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`3d
0?ooo`030?l0003oool0oooo03H0oooo00<000000?ooo`3oool08@3oool005`0oooo0P00003e0?oo
o`030?l0003oool0oooo03@0oooo0`00000S0?ooo`00G03oool00`000000oooo0?ooo`3d0?ooo`03
0?l0003oool0oooo03H0oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0
m@3oool00`3o0000oooo0?ooo`0e0?ooo`030000003oool0oooo0240oooo001L0?ooo`030000003o
ool0oooo0?D0oooo00<0o`000?ooo`3oool0=@3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool0
0`000000oooo0?ooo`3f0?ooo`030?l0003oool0oooo03@0oooo00<000000?ooo`3oool08@3oool0
05`0oooo0P00003g0?ooo`030?l0003oool0oooo0380oooo0`00000S0?ooo`00G03oool00`000000
oooo0?ooo`3f0?ooo`030?l0003oool0oooo03@0oooo00<000000?ooo`3oool08@3oool005`0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo02h0oooo
00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo02d0oooo
00<000000?ooo`3o0000;@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0240oooo
001L0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?l0
00070?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`0Q0?ooo`00G03ooook000000030?l000000000000003<000008`3oool00?l0oooo/@3oool00?l0
oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0
oooo:P3oool00`3ooml0W4P0000000030000000304RLg`3oool0gia80080000000<0M;oo0?ooo`3o
ool0N@3oool00?l0oooo:P3oool00`2oM00004RL0=ooo`030?ooo`030=nLW02oool0oooo0080oooo
00<0_gAd0;ooo`3oool0N@3oool00?l0oooo:P3oool00`3ooml0W7BL0=ooo`030?ooo`030?oo_`2L
W=l0oooo0080oooo00<0omnL07BLg`3oool0N@3oool00?l0oooo:`3oool00`2oM000000000000003
0000000704RLg`3oool0oooo0?ooo`3ooml0W4Qd0;ooo`1i0?ooo`00M@3oool00`2oM4P0W=oo0?oo
o`020?ooo`030?oog`2LM9`0gooo00@0oooo00P0oooO09a80000000000000018W02LM0000000001d
_`<0oooo00<0oooO09a8M02oool01@3oool01@2oM00000000000000007@0_ooo00D0oooo00<0oooO
09a8000000000P0000000`0004P0W=oo0?ooo`040?ooo`090?oo_`1d0000000000000018W;l0W4P0
0018W03Oool0_g@00080000000<0001d0;oog`2LB0000P0000001000B9`0gooo0?ooo`2oM0020000
00090000B02Lgol0oooo0?ooo`3ogi`0B00000000018W=l0gia80080000000@0001d0;ooo`3oool0
gia80`000000101d_ol0oooo0?ooo`2oM002000000030000M02ooml0W4P00080000000<004RL0=oo
o`3oool01P3oool00`3ogi`0M9cO0?ooo`020?ooo`030=nLM02Lgol0oooo0080oooo00X0gia80000
00000000001d0;ooo`3ogi`0B00000000018W=l0gia80P0000000`0004P0W=oO09a8000200000003
001d_`3oool0oooo0080oooo00<0oooO09a8000000001`0000000`00M;l0oooo0;md000200000003
0000M02ooml0W4P00080000000@004RL0=ooo`3oool0_g@00P000000100007@0_ooo0?ooo`2oM002
000000050000B02Lgol0oooo0?oo_`1d0000100000000`0007@0_ooo0?ooo`030?ooo`030;md001d
_ol0oooo01L0oooo00<0_g@0000000000000100000001P18W=l0oooo0?ooo`3o_g@0000007Boo`<0
oooo00D0gia8000000000000000004RLg`040?ooo`050=nLB0000000000000000018W=l01@3oool0
0`3OW4P0B9cO0?ooo`0k0?ooo`00M03oool00`3ookl0M00004RLg`030?ooo`030?noM00004P0W=oo
00@0oooo00T0okmd0000001d_ol0oooo0?oOW018000007Bo0?no_`2oool00P3oool00`3o_g@00000
04RLg`040?ooo`070;md0000000007Bo0?ooo`3ogi`0B01d0;ooo`060?ooo`030?oOW0180000B9cO
00L0oooo00D0gia80000M02oool0oooo0=nLB002000000090000M02oool0gia80018W03Oool0oooo
0?oog`2LB00007Bo0080oooo01P0_g@000000000B9`0gooo0?oog`2LB7@0_ooo0?ooo`3OW4P0001d
0;ooo`3O_kl0gooo0=nLB00007@0_ooo0?ooo`3OW4P0000004RLg`3oool0oooO09a80018W=l20?oo
o`070=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`090?ooo`070;md000007@0_ooo0?ooo`3ookl0
M000001d_`040?ooo`0@0=nLB00007@0_ooo0?ooo`3OW4P0001d0;ooo`3O_kl0gooo0=nLB00007@0
_ooo0?ooo`3ooml0W4P0001d_`L0oooo00<0omnL04P00018W=l0103oool02`3ogi`0B01d0;ooo`3o
ool0gia80018W03Oool0oooo0?oog`2LB00007Bo0080oooo00/0_g@000000000M;l0oooo0?oOW018
07@0_ooo0?ooo`3OW4P0001d0;ooo`030?ooo`030;mdB02Lgol0oooo00L0oooo00@0omnL04P00000
B9`0gooo603oool00`3OW4P0000000000003000000060018W03Oool0oooo0?noM0000000M;oo0P3o
ool01`3OW4P0000007Boo`3oool0oooO09a80018W=l00P3oool01`3OW4P0000007Boo`3oool0oooO
09a80018W=l0103oool00`3ogi`0B4RL0=ooo`0k0?ooo`00M03oool0103o_g@000000018W03Oool2
0?ooo`030;md0000000007Bo00@0oooo00L0okmd0000B02Lgol0oooo0?oog`2LB00007Bo00<0oooo
00D0oooO09a80000B9`0W7Ad0;ooo`020?ooo`030?oOW0180000B9cO00<0oooo00<0omnL07BLg`3o
ool01@3oool00`3ookl0M00004RLg`070?ooo`070;md0000B9`0gooo0?ooo`3ooml0W4P0001d_`02
0?ooo`0;0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0omnL04P00000M;l00`3oool01P3o
gi`0W;oo0?ooo`3OW4P004RL0=ooo`80oooo00H0gia80000M02oool0omnL04P00000M;l30?ooo`0;
0?noM00007@0_ooo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0203oool02@3ogi`0B000
04QdM01dW=l0oooo0?noM00004P0B4Q809cOo`030?ooo`070=nLB00007@0_ooo0?ooo`3OW4P004RL
0=ooo`020?ooo`070=nLB00007@0_ooo0?ooo`3ooml0W4P0001d_`070?ooo`030?oo_`1d0000B9cO
00D0oooo00h0okmd07Boo`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3ogi`0B000
04RLg`<0oooo00H0omnL07BLg`3oool0gia80000M02oool30?ooo`050?oo_`1d0000000000000000
M;l01@3oool0102oM00004RL09ad001d_ol70?ooo`030?oog`2LB000000000L0000000<0B9cO0?oo
o`3oool01@3oool00`3OW4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo00@0oooo00<0oooO09a8
0000M;l00`3oool01P3o_g@004RL0=ooo`3ooml0W4P0001d_`<0oooo00<0okmd0018W03Oool00`3o
ool00`3ooml0W4Q809cOo`0k0?ooo`00L`3oool02`3ooml0W4P00000M02LW7@0W=oo0?ooo`3ookl0
M000001dW02LB7@0_ooo00@0oooo00H0okmd00000000M;l0oooO09a80000M;l30?ooo`050?oOW018
0000M;oo0?noM018W=l00P3oool00`3o_g@0001809cOo`0;0?ooo`030?oo_`1d0000B9cO00L0oooo
00L0_g@00018W03Oool0oooo0?oog`2LB00007Bo0080oooo00/0gia80018W03Oool0oooo0?oog`2L
B00007Bo0?ooo`3o_g@0001809cOo`060?ooo`030=nLB000B9`0gooo0080oooo00H0gia80000M02o
ool0okmd0000001d_ol30?ooo`0;0?noM0000000M;oo0?ooo`3OW4P004RL0=ooo`3oool0oooO09a8
0000M;l0203oool02@3OW4P0001d0;ooo`2oM7@0_ooO09a80000M;l0omnL07BLg`030?ooo`070=nL
B00007@0_ooo0?ooo`3OW4P004RL0=ooo`020?ooo`070=nLB00007@0_ooo0?ooo`3ooml0W4P0001d
_`070?ooo`030?oo_`1d0000B9cO00P0oooo00/0gia80018W03Oool0oooo0?oog`2LB00007Bo0?oo
o`3o_g@0001809cOo`060?ooo`030=nLB00007@0_ooo00<0oooo00H0gia80000M02oool0oono07@0
0018W=l30?ooo`050?noM0000000M;oo0?oOW018M;l06P3oool00`3OW4P0M;oo0?ooo`090?ooo`03
0;md0000M;l0oooo0080oooo00H0_g@00018W03Oool0oooo0;md0000M;l30?ooo`030;md0000B9`0
gooo00@0oooo00<0gia804RLg`3oool0>P3oool007<0oooo00/0omnL04P00018W=l0omnL04Qd_`3o
ool0gia80000001d_ol0okmd07Boo`060?ooo`040?noM00000000000001d_`<0oooo00X0_g@00018
W03Oool0oooO09a8M02oool0oooo0?noM00007@0_ooo2`3oool00`3ookl0M00004RLg`020?ooo`03
0?oOW01dW=l0oooo0080oooo00L0_g@00018W03Oool0oooo0?oog`2LB00007Bo0080oooo00/0gia8
0018W03Oool0oooo0?oog`2LB00007Bo0?ooo`3o_g@0001d0;ooo`060?ooo`030=nLB000B9`0gooo
0080oooo00H0gia80000M02oool0okmd0000001d_ol30?ooo`0;0?noM0000000M;oo0?ooo`3OW4P0
04RL0=ooo`3oool0oooO09a80000M;l01`3oool02`3ookl0M00004RLg`3oool0omnL07AdM01804P0
W=oo0?ooo`2oM7@0_ooo0080oooo00L0gia80000M02oool0oooo0=nLB000B9`0gooo0080oooo00L0
gia80000M02oool0oooo0?oog`2LB00007Bo00L0oooo00<0oono07@00018W=l00`3oool00`3o_g@0
M;oo0?ooo`020?ooo`0;0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd0000M02oool0
1P3oool00`3OW4P0001d0;ooo`020?ooo`070?oo_`1d0000B9cO0?ooo`3ooml0W4P0001d_`020?oo
o`070?oog`2LB00004RL0=ooo`3oool0_gA809cOo`0J0?ooo`030;mdB02Lgol0oooo00P0oooo00L0
omnL04P0M02oool0oooo0;md00000000M;oo0080oooo00L0omnL04P0M02oool0oooo0;md00000000
M;oo00D0oooo00<0omnL04Q8W03Oool0>P3oool007<0oooo00/0gia80000B02Lgol0oooO09a8M02o
oml0W4P00000M02oool0oono07Ad_`030?ooo`070?oo_`1d0000M;oo0?ooo`3ooml0W4P0001d_`02
0?ooo`030?oOW0180000M;oo0080oooo00D0okmd04RLg`3oool0omnL04P000050000000304RLg`3o
ool0oooo00@0oooo00<0oono07@00018W=l00P3oool00`3o_g@0B9cO0?ooo`020?ooo`070;md0000
B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`0;0;md000007@0_ooo0?ooo`3ooml0W4P0001d_`3o
ool0omnL04P0B02Lgol00P3oool01`3o_g@0001809cOo`3oool0gia80018W03Oool00P3oool01P3O
W4P0001d0;ooo`3ookl0M00007Boo`<0oooo00/0_g@00000B02Lgol0oooo0;md000007@0_ooo0?oo
o`3ooml0W4P0001d_`070?ooo`070=nLB00004P0W=oo0?ooo`3ooml0W4P0001d_`020?ooo`0;0?no
M018W=l0oooo0?ooo`3OW4P0001d0;ooo`3oool0gia80018W03Oool00P3oool01`3OW4P0001d0;oo
o`3oool0oooO09a80000M;l01`3oool00`3ookl0M00004RLg`030?ooo`030;md001d_ol0oooo0080
oooo00X0_g@00000M02oool0oooo0?oog`2LB00007Bo0?ooo`3ogi`0B0001@0000001@18W=l0oooo
0=nLB00004P0W=oo0080oooo00L0oono07@00018W=l0oooo0?oog`2LB000B9cO0080oooo00<0okmd
0000B02Lgol00P3oool00`3o_g@0B9cO0?ooo`0I0?ooo`030?oog`2LB7@0_ooo00T0oooo00D0oono
07A8B0180000000004RLg`040?ooo`050?oo_`1dB4P0B00000000018W=l01P3oool00`3ooml0W4Q8
09cOo`0j0?ooo`00LP3oool02@3ookl0M0000018W03Oool0oooo0?noM01dW9`0B00004RLg`020?oo
o`0@0;mdB02Lgol0oooo0?ooo`3ookl0M00004RLg`3oool0oono07@00000M;l0oooo0?oog`2LB000
001d0;ooo`80oooo00H0oooO09a8M02oool0oooo0;md0018W=l20?ooo`030=nLB00004P0W=oo00H0
oooo00<0oono07@0000000000`0000005018W=l0oooo0?ooo`3oool0_g@00018W03Oool0oooo0?oo
g`2LB00007Bo0?oog`2LB00000000018B01804P0W=oo0?oo_`1d0000B9cO0P3oool08@2oM00004RL
0=ooo`3ooml0W4P00000M02oool0oooo0=nLB000B9`0gooo0?oog`2LB00000000000M02oool0oooo
0=nLB000B9`0gooo0?ooo`3o_g@00000001d_`3ooml0W4P000000000B4P0B01809cOo`3ookl0M000
04RLg`060?ooo`030?oo_`1d000007Bo0080oooo00<0omnL04P0001d_ol00P3oool02`3ooml0W4Qd
0;oog`2LB00000000000M02oool0oooo0=nLB000B9`0gooo0080oooo00L0gia80000000004P0W=oo
0?oo_`1d0000B9cO00L0oooo00<0oono07@000000000100000002`1d_ol0oooo0?oog`2LB0000000
0018B01804P0W=oo0?oo_`1d0000B9cO0080oooo0180_g@004RLg`3oool0oooo0=nLB00004P0W=oO
09a800000000001804QdW03O_g@0001d0;ooo`2oM00007Bo0?ooo`3o_g@2000000050018W03Ookl0
M0000018W03Oool00P3oool00`3ookl0M01d0;ooo`050?ooo`030?oog`2LB000000000L0000000<0
B9cO0?ooo`3oool0203oool00`3o_g@007Bo0?ooo`080?ooo`040?oOW01800000000001d_`D0oooo
00@0omnL04P00000000007Bo203oool00`3OW4P007Bo0?ooo`0i0?ooo`00LP3oool00`3o_g@00000
04RLg`020?ooo`040?oo_`1d0000001809cOo`80oooo00<0okmd04RLg`3oool00P3oool0203ookl0
M000000000000000001d0;ooo`3ooml0W4P00P0000002@0007@0_ooo0?oog`2LB00000000018W03O
ool0oooo0=nLB002000000030000B02Lgol0oooo00H0oooo00<0oono07@00018W=l00P3oool0303o
_g@0B9cO0?ooo`3ookl0M00000000000B9`0gooo0?oOW01800000000001d_`80oooo00L0_g@00018
W03Ooml0W4P00000000007@0_ooo00<0oooo00L0gia8000000000000001809cOo`3oool0_g@000<0
000000D004RL0=ooo`2oM000001d0;ooo`020?ooo`050?noM0000000000000000018W=l00`3oool0
1`2oM00004RL0=oog`2LB00000000000M02oool01@3oool01P3ooml0W4P0000000000000B9cO0?no
M0<0000000X0B9cO0?ooo`2oM00000000018W03Oool0_g@00000M02oool0_g@00`0000002@00B9`0
gooo0=nLB00007@0_ooO09a800000000001d0;ooo`070?ooo`030?oo_`1d0000B9cO00<0oooo00<0
_g@007Boo`3oool00P3oool01`2oM00004RL0=oog`2LB00000000000M02oool00`3oool01@3OW4P0
00000000000004P0W=oo0080oooo00P0_g@00000M02ogi`0B0000000M02oool0oooO09a800800000
00@0M;oo0?ooo`3ooml0W4P00P0000001`00B9`0gooo0?ooo`2oM00000000018W03Oool06@3oool0
0`3ookl0M01809cOo`070?ooo`060?oOW0180000001d0;ooo`3o_g@007Bo0`3oool01P3ogi`0B000
0000M02oool0okmd001d_`L0oooo00<0oono07@0M02oool0>@3oool00780oooo00<0_g@00000M02o
ool00P3oool0103ookl0M0000018W03Oool20?ooo`030?oo_`1dB9`0gooo0200oooo00<0oono07@0
0018W=l00P3oool00`3ogi`0M9cO0?ooo`0L0?ooo`030;md0000B9`0gooo02h0oooo00<0_g@00018
W03Oool00P3oool00`3OW4P0001d0;ooo`0;0?ooo`030?oo_`1d0000B9cO00<0oooo00<0okmd07Bo
o`3oool0@`3oool00`3ookl0M00004RLg`070?ooo`030;md000007@0_ooo0080oooo00L0_g@004RL
g`3oool0oooo0;md000007@0_ooo0080oooo00D0_g@004RLg`3oool0omoO0=ooo`040?ooo`030;md
001d_ol0oooo03P0oooo001a0?ooo`030?oOW018000007Bo00<0oooo00<0okmd00000018W=l0103o
ool00`3OW7@0W=oo0?ooo`0O0?ooo`030?oo_`1d0000B9cO00@0oooo00<0giad09cOo`3oool06P3o
ool00`3ookl0M4RL0=ooo`0^0?ooo`030?oo_`1dB9`0gooo0080oooo00<0gia80000M02oool02`3o
ool00`3ookl0M00004RLg`040?ooo`030?oo_`1dM;l0oooo03d0oooo00<0oknL09cOo`3oool00P3o
ool00`3ogi`0B00004RLg`070?ooo`030;md0000B9`0gooo0080oooo00L0gia8001d_`3oool0oooo
0;md0000B9`0gooo0080oooo00H0gia8001d_`3oool0oono07A8W03Oool30?ooo`030?noM000M;l0
oooo03P0oooo001a0?ooo`030=nLB0000000B9cO0080oooo00@0oooO09a800000000M;oo103oool0
0`3OW4P007Bo0?ooo`0O0?ooo`030?oOW0180000B9cO00<0oooo00<0oono07@0M02oool06`3oool0
0`3ooml0W7BL0=ooo`0^0?ooo`030?oog`2LM9`0gooo0080oooo00<0gia80000M02oool02`3oool0
0`3ogi`0B00004RLg`040?ooo`030=nLB018W=l0oooo03d0oooo00P0oooO09a8B02Lgol0oooo0?oo
_`1d0000001d0;ooo`L0oooo00L0okmd0000001d_ol0oooo0?oo_`1d0000M;oo0080oooo00L0okmd
0000001d_ol0oooo0?oo_`1d0000M;oo0080oooo00<0_g@00000000000000`0000000`0007@0_ooo
0?ooo`0g0?ooo`00K`3oool00`3ooml0W4P00000000200000003001d_`3ogi`0B00000<0000000@0
B9cO0?ooo`3oool0_g@00P0000000`00B9`0gooo0?ooo`0K0?ooo`030?oog`2LB000000000H00000
00<0001d0;ooo`3oool07`3oool00`3OW4P0001809cOo`0U0?ooo`030=nLB00004P0W=oo00@0oooo
00D0oooO09a800000000001d0;ooo`090?ooo`030?oog`2LB000000000L0000000<0B9cO0?ooo`3o
ool0?@3oool01P3ooml0W4P0000000000000001d0;ooo`T0oooo00D0okmd000000000000000004RL
g`040?ooo`050?noM0000000000000000018W=l00`3oool00`3o_g@00000000000040000000307Bo
o`3oool0oooo03H0oooo00390?ooo`030=nLB00004P0W=oo02D0oooo00<0gia80000B02Lgol01P3o
ool00`2oM000001d0;ooo`2c0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00
o`3ooona0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {-1.03327, 1.46682, \
0.0155588, 0.0309776}}]
}, Closed]],

Cell["\<\
Then searching for a slightly lower eigenvalue, we obtain the ground-state \
solution below.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(sol = 
      ShootBSWF2[pot, 20, 2.8871, 2.88715, \(-3\), 3, 0, 
        AccuracyGoal \[Rule] 10, WorkingPrecision \[Rule] 32, 
        MaxIterations \[Rule] 200]\)], "Input"],

Cell[BoxData[
    \({energy \[Rule] 
        2.887166436580411194900809123059136588795441408181`32. , 
      wavefunction \[Rule] \((\[Psi]$109480[#1] &)\), 
      normalization \[Rule] 12.0231391037152`}\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(PlotBSWF[sol, \(-3\), 3];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.15873 -0.557467 0.219273 [
[.02381 -0.0125 -9.15625 -15.6875 ]
[.02381 -0.0125 9.15625 0 ]
[.18254 -0.0125 -9.15625 -15.6875 ]
[.18254 -0.0125 9.15625 0 ]
[.34127 -0.0125 -9.15625 -15.6875 ]
[.34127 -0.0125 9.15625 0 ]
[.5 -0.0125 -5 -15.6875 ]
[.5 -0.0125 5 0 ]
[.65873 -0.0125 -5 -15.6875 ]
[.65873 -0.0125 5 0 ]
[.81746 -0.0125 -5 -15.6875 ]
[.81746 -0.0125 5 0 ]
[.97619 -0.0125 -5 -15.6875 ]
[.97619 -0.0125 5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -5.0625 -31.375 ]
[.5 -0.0125 5.0625 -15.6875 ]
[-0.0125 .10035 -10 -7.84375 ]
[-0.0125 .10035 0 7.84375 ]
[-0.0125 .20999 -19 -7.84375 ]
[-0.0125 .20999 0 7.84375 ]
[-0.0125 .31963 -10 -7.84375 ]
[-0.0125 .31963 0 7.84375 ]
[-0.0125 .42926 -19 -7.84375 ]
[-0.0125 .42926 0 7.84375 ]
[-0.0125 .5389 -10 -7.84375 ]
[-0.0125 .5389 0 7.84375 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -34.6875 -13.0313 ]
[-0.01875 .30902 -19 13.0313 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -89.6875 0 ]
[.5 .63053 89.6875 15.6875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
gsave
.02381 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
.18254 0 m
.18254 .00625 L
s
gsave
.18254 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.34127 0 m
.34127 .00625 L
s
gsave
.34127 -0.0125 -70.1563 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.5 0 m
.5 .00625 L
s
gsave
.5 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.65873 0 m
.65873 .00625 L
s
gsave
.65873 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.81746 0 m
.81746 .00625 L
s
gsave
.81746 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.97619 0 m
.97619 .00625 L
s
gsave
.97619 -0.0125 -66 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
.125 Mabswid
.05556 0 m
.05556 .00375 L
s
.0873 0 m
.0873 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.15079 0 m
.15079 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.24603 0 m
.24603 .00375 L
s
.27778 0 m
.27778 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.37302 0 m
.37302 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.43651 0 m
.43651 .00375 L
s
.46825 0 m
.46825 .00375 L
s
.53175 0 m
.53175 .00375 L
s
.56349 0 m
.56349 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62698 0 m
.62698 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.72222 0 m
.72222 .00375 L
s
.75397 0 m
.75397 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.84921 0 m
.84921 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.9127 0 m
.9127 .00375 L
s
.94444 0 m
.94444 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -66.0625 -35.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
69.125 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .10035 m
.00625 .10035 L
s
gsave
-0.0125 .10035 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
0 .20999 m
.00625 .20999 L
s
gsave
-0.0125 .20999 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3.5) show
1.000 setlinewidth
grestore
0 .31963 m
.00625 .31963 L
s
gsave
-0.0125 .31963 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
0 .42926 m
.00625 .42926 L
s
gsave
-0.0125 .42926 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4.5) show
1.000 setlinewidth
grestore
0 .5389 m
.00625 .5389 L
s
gsave
-0.0125 .5389 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .12228 m
.00375 .12228 L
s
0 .14421 m
.00375 .14421 L
s
0 .16614 m
.00375 .16614 L
s
0 .18806 m
.00375 .18806 L
s
0 .23192 m
.00375 .23192 L
s
0 .25384 m
.00375 .25384 L
s
0 .27577 m
.00375 .27577 L
s
0 .2977 m
.00375 .2977 L
s
0 .34155 m
.00375 .34155 L
s
0 .36348 m
.00375 .36348 L
s
0 .38541 m
.00375 .38541 L
s
0 .40734 m
.00375 .40734 L
s
0 .45119 m
.00375 .45119 L
s
0 .47312 m
.00375 .47312 L
s
0 .49505 m
.00375 .49505 L
s
0 .51697 m
.00375 .51697 L
s
0 .07843 m
.00375 .07843 L
s
0 .0565 m
.00375 .0565 L
s
0 .03457 m
.00375 .03457 L
s
0 .01264 m
.00375 .01264 L
s
0 .56083 m
.00375 .56083 L
s
0 .58275 m
.00375 .58275 L
s
0 .60468 m
.00375 .60468 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -100.875 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
74.0313 11.8438 translate
90 rotate
-74.0313 -11.8438 translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(V) show
71.625 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(,) show
77.625 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(y) show
85.063 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .61178 m
.02381 .61803 L
s
.18254 .61178 m
.18254 .61803 L
s
.34127 .61178 m
.34127 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.65873 .61178 m
.65873 .61803 L
s
.81746 .61178 m
.81746 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.05556 .61428 m
.05556 .61803 L
s
.0873 .61428 m
.0873 .61803 L
s
.11905 .61428 m
.11905 .61803 L
s
.15079 .61428 m
.15079 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.24603 .61428 m
.24603 .61803 L
s
.27778 .61428 m
.27778 .61803 L
s
.30952 .61428 m
.30952 .61803 L
s
.37302 .61428 m
.37302 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.43651 .61428 m
.43651 .61803 L
s
.46825 .61428 m
.46825 .61803 L
s
.53175 .61428 m
.53175 .61803 L
s
.56349 .61428 m
.56349 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.62698 .61428 m
.62698 .61803 L
s
.69048 .61428 m
.69048 .61803 L
s
.72222 .61428 m
.72222 .61803 L
s
.75397 .61428 m
.75397 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.84921 .61428 m
.84921 .61803 L
s
.88095 .61428 m
.88095 .61803 L
s
.9127 .61428 m
.9127 .61803 L
s
.94444 .61428 m
.94444 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .10035 m
1 .10035 L
s
.99375 .20999 m
1 .20999 L
s
.99375 .31963 m
1 .31963 L
s
.99375 .42926 m
1 .42926 L
s
.99375 .5389 m
1 .5389 L
s
.125 Mabswid
.99625 .12228 m
1 .12228 L
s
.99625 .14421 m
1 .14421 L
s
.99625 .16614 m
1 .16614 L
s
.99625 .18806 m
1 .18806 L
s
.99625 .23192 m
1 .23192 L
s
.99625 .25384 m
1 .25384 L
s
.99625 .27577 m
1 .27577 L
s
.99625 .2977 m
1 .2977 L
s
.99625 .34155 m
1 .34155 L
s
.99625 .36348 m
1 .36348 L
s
.99625 .38541 m
1 .38541 L
s
.99625 .40734 m
1 .40734 L
s
.99625 .45119 m
1 .45119 L
s
.99625 .47312 m
1 .47312 L
s
.99625 .49505 m
1 .49505 L
s
.99625 .51697 m
1 .51697 L
s
.99625 .07843 m
1 .07843 L
s
.99625 .0565 m
1 .0565 L
s
.99625 .03457 m
1 .03457 L
s
.99625 .01264 m
1 .01264 L
s
.99625 .56083 m
1 .56083 L
s
.99625 .58275 m
1 .58275 L
s
.99625 .60468 m
1 .60468 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .10035 m
1 .10035 L
s
.5 0 m
.5 .61803 L
s
gsave
.5 .63053 -150.688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Wave) show
93.875 15.813 moveto
(Function) show
139.313 15.813 moveto
(with) show
163.563 15.813 moveto
(Energy) show
200.563 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
211.375 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.887) show
238.375 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.13382 .61803 m
.14415 .55177 L
.18221 .34137 L
.20342 .24145 L
.22272 .16351 L
.24299 .09708 L
.26171 .05184 L
.271 .0357 L
.27575 .02917 L
.28086 .02347 L
.28352 .02105 L
.28596 .01918 L
.28844 .01761 L
.29075 .01645 L
.29215 .01589 L
.29342 .01547 L
.29483 .01512 L
.2956 .01497 L
.29633 .01486 L
.29698 .01479 L
.29766 .01474 L
.29888 .01472 L
.3002 .01478 L
.30094 .01486 L
.30164 .01497 L
.3029 .01523 L
.30424 .0156 L
.30665 .01652 L
.30923 .01787 L
.31209 .0198 L
.3172 .02439 L
.32199 .03002 L
.33092 .04385 L
.34055 .06348 L
.36059 .1186 L
.38169 .19339 L
.4198 .34784 L
.44105 .42788 L
.46036 .48628 L
.46999 .50825 L
.47507 .51759 L
.48044 .52571 L
.48514 .53125 L
.48763 .53359 L
.49031 .53564 L
.49276 .53708 L
.49501 .53803 L
.49609 .53837 L
.49726 .53864 L
.49837 .53881 L
Mistroke
.4994 .53889 L
.50069 .53888 L
.50134 .53884 L
.50206 .53875 L
.50332 .53852 L
.5045 .5382 L
.50589 .53769 L
.50715 .53712 L
.51 .53543 L
.51245 .53352 L
.51512 .53099 L
.51997 .52516 L
.52964 .50897 L
.53858 .48895 L
.55729 .43357 L
.5779 .35703 L
.61816 .19398 L
.63667 .12747 L
.64716 .09517 L
.6569 .06947 L
.66742 .04688 L
.67698 .0314 L
.68224 .02499 L
.68719 .02036 L
.68992 .0184 L
.69242 .01696 L
.69518 .01579 L
.69657 .01536 L
.6973 .01518 L
.69809 .01502 L
.69939 .01482 L
.70059 .01473 L
.70189 .01472 L
.70263 .01476 L
.70332 .01482 L
.70456 .015 L
.70571 .01524 L
.70696 .01559 L
.70828 .01605 L
.71058 .01708 L
.71272 .0183 L
.71758 .022 L
.72287 .02749 L
.72768 .03377 L
.73848 .05222 L
.74845 .07435 L
.75764 .0989 L
.77843 .16777 L
.8178 .34143 L
.85566 .55063 L
Mistroke
Mfstroke
.85566 .55063 m
.86616 .61803 L
s
0 g
.02381 .07561 m
.02846 .07561 L
.03279 .07561 L
.03754 .07561 L
.0402 .07561 L
.04262 .07561 L
.0475 .07561 L
.05026 .07561 L
.05281 .07561 L
.05749 .07561 L
.05985 .07561 L
.06244 .07562 L
.06525 .07562 L
.06792 .07562 L
.07293 .07562 L
.07572 .07562 L
.07832 .07562 L
.08135 .07563 L
.0842 .07563 L
.08668 .07564 L
.08937 .07564 L
.09406 .07566 L
.09673 .07567 L
.09954 .07568 L
.10213 .0757 L
.10458 .07571 L
.10954 .07576 L
.11406 .07581 L
.11892 .0759 L
.12422 .07602 L
.12884 .07617 L
.13374 .07639 L
.13793 .07662 L
.14255 .07697 L
.14747 .07744 L
.15264 .0781 L
.15706 .07883 L
.1619 .07984 L
.16712 .08123 L
.1727 .08313 L
.18275 .08799 L
.18808 .09149 L
.193 .0954 L
.2041 .10707 L
.21389 .1211 L
.22425 .14011 L
.26214 .23452 L
.27212 .25775 L
.27703 .26735 L
.28159 .27483 L
Mistroke
.28681 .28144 L
.28957 .28401 L
.29098 .28507 L
.2925 .28601 L
.29391 .28669 L
.2952 .28716 L
.29644 .28746 L
.29774 .28762 L
.29891 .28763 L
.3 .28753 L
.3012 .28728 L
.30248 .28687 L
.3037 .28634 L
.30505 .2856 L
.30778 .28359 L
.31031 .28114 L
.31268 .27835 L
.31802 .27043 L
.32376 .25967 L
.34389 .21029 L
.36358 .15968 L
.37398 .13712 L
.38508 .11771 L
.39587 .10351 L
.40565 .09418 L
.41037 .0907 L
.41549 .08757 L
.42476 .08332 L
.42967 .08166 L
.43425 .08041 L
.4397 .07924 L
.44468 .0784 L
.44989 .07773 L
.45483 .07723 L
.46005 .07683 L
.46573 .07651 L
.47057 .0763 L
.47512 .07616 L
.4801 .07604 L
.48291 .07599 L
.48548 .07595 L
.48784 .07592 L
.49036 .0759 L
.49181 .07589 L
.49313 .07588 L
.49438 .07587 L
.49571 .07586 L
.49637 .07586 L
.49709 .07586 L
.49837 .07586 L
Mistroke
.49908 .07586 L
.49985 .07586 L
.50058 .07586 L
.50125 .07586 L
.50253 .07586 L
.50326 .07586 L
.50393 .07586 L
.50517 .07587 L
.50648 .07587 L
.50909 .07589 L
.51185 .07592 L
.5142 .07594 L
.51678 .07598 L
.51956 .07603 L
.52254 .0761 L
.52791 .07625 L
.53333 .07646 L
.53846 .07674 L
.54307 .07706 L
.54808 .07751 L
.55294 .07807 L
.55749 .07874 L
.56249 .07967 L
.56787 .08096 L
.57322 .08259 L
.57884 .08478 L
.5841 .08735 L
.58889 .09021 L
.59844 .09771 L
.60864 .10892 L
.6181 .12276 L
.62701 .13911 L
.6655 .23485 L
.67578 .25872 L
.6808 .2684 L
.68539 .27574 L
.69023 .2817 L
.69289 .28414 L
.69542 .28587 L
.69674 .28655 L
.69747 .28686 L
.69815 .2871 L
.6988 .28728 L
.69949 .28744 L
.70069 .28761 L
.70139 .28764 L
.70215 .28763 L
.70295 .28756 L
.70369 .28744 L
.70438 .28728 L
Mistroke
.70502 .28709 L
.70644 .28654 L
.70763 .28594 L
.70874 .28526 L
.71127 .2833 L
.71373 .28086 L
.71639 .27765 L
.72558 .26244 L
.73643 .23809 L
.74636 .2124 L
.76694 .15949 L
.77696 .13766 L
.78605 .1212 L
.79557 .10749 L
.80453 .09765 L
.81399 .09004 L
.81933 .08681 L
.82423 .0844 L
.82893 .08252 L
.83396 .08091 L
.83831 .07979 L
.84305 .07881 L
.8486 .07793 L
.85385 .0773 L
.85897 .07684 L
.86371 .07652 L
.86834 .07629 L
.87333 .0761 L
.87804 .07596 L
.88236 .07587 L
.88716 .0758 L
.8924 .07574 L
.89504 .07571 L
.89787 .0757 L
.90296 .07567 L
.90555 .07566 L
.90829 .07565 L
.9132 .07564 L
.91593 .07563 L
.91848 .07563 L
.92147 .07562 L
.9243 .07562 L
.92673 .07562 L
.92937 .07562 L
.93401 .07562 L
.93663 .07562 L
.93943 .07561 L
.94187 .07561 L
.94445 .07561 L
.94854 .07561 L
Mistroke
.95231 .07561 L
.95636 .07561 L
.95866 .07561 L
.9608 .07561 L
.96469 .07561 L
.96823 .07561 L
.97619 .07561 L
Mfstroke
[ .02 .04 ] 0 setdash
.02381 .07561 m
.06244 .07561 L
.10458 .07561 L
.14415 .07561 L
.18221 .07561 L
.22272 .07561 L
.26171 .07561 L
.30316 .07561 L
.34309 .07561 L
.3815 .07561 L
.42237 .07561 L
.46172 .07561 L
.49955 .07561 L
.53984 .07561 L
.57861 .07561 L
.61984 .07561 L
.65954 .07561 L
.69774 .07561 L
.73838 .07561 L
.77751 .07561 L
.81909 .07561 L
.85916 .07561 L
.89771 .07561 L
.93871 .07561 L
.97619 .07561 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00k`3oool01@2oM0000000001d_`3oool0gia80080000000<004RL0=ooo`3oool0]`3oool0
0>l0oooo00P0oooO09a8M02oool0oooo0?oog`2LB00004RL0=oookT0oooo003`0?ooo`060?oo_`1d
B9`0gooo0;md000007@0_ooo^P3oool00?40oooo00@0omnL07A8B0000000M;oo^`3oool00?40oooo
00<0oono07@00018W=l0_03oool00?00oooo00D0oooO09a80000B9`0W7A809cOo`2k0?ooo`00l03o
ool01P2oM000001809cOo`3ooml0W4Qd0;oookX0oooo003^0?ooo`090?oog`2LB000000000000000
M;l0okmd0000000007@0_ooo0;T0oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo001U0?ooo`030?oo_`1d000000000080
000000<0M;oo0?ooo`3oool0:03oool00`2oM0000000000000040000000304RLg`3oool0oooo02P0
oooo00H0_g@000000000000000000018W03OoolT0?ooo`050?oOW018000000000000M02oool0:`3o
ool01P2oM000000000000000000004RL0=ooobT0oooo00<0_g@0000000000000100000000`18W=l0
oooo0?ooo`0V0?ooo`030?oo_`1d000000000080000000<0M;oo0?ooo`3oool0:03oool006D0oooo
00<0omnL04P00018W=l00P3oool00`2oM000B9cO0?ooo`0Y0?ooo`030=nLB0000000000000<00000
00<004RL0=ooo`3oool0:@3oool00`3ogi`0B00007Boo`0U0?ooo`070?oo_`1d0000M;oo0?ooo`3o
_g@0001d0;ooo`0[0?ooo`030?oOW0180000M;oo02`0oooo00<0gia80000000000000`0000000`00
B9`0gooo0?ooo`0V0?ooo`030?oOW0180000B9cO0080oooo00<0_g@004RLg`3oool0:03oool006X0
oooo00<0oono07@0B02Lgol0:P3oool00`3OW4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo02T0
oooo00<0omnL04P0001d_ol09@3oool00`2oM00004RL0=ooo`020?ooo`030;md0000M;l0oooo02X0
oooo00<0omnL04P0001d_ol0;@3oool00`3OW4P0B9cO0?ooo`020?ooo`030=nLM02Lgol0oooo02/0
oooo00<0oono07@0B02Lgol0:03oool006X0oooo00<0oono07@0001d_ol0:`3oool00`3OW4P0M;oo
0?ooo`0]0?ooo`030?oOW0180000M;oo02@0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d
0;ooo`0Z0?ooo`030?oOW0180000M;oo02h0oooo00<0gia807Boo`3oool0;`3oool00`3ookl0M000
07Boo`0X0?ooo`00F03oool00`3ooml0W4P0000000070000000304RLg`3oool0oooo00D0oooo00<0
okmd0000001d_ol0703oool00`3ooml0W4P0000000070000000304RLg`3oool0oooo00<0oooo00<0
_gA809cOo`3oool07@3oool00`3ooml0W4P0000000070000000304RLg`3oool0oooo0080oooo00<0
omnL04P0001d_ol0903oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo02X0oooo00<0
omnL04P0001d_ol0;`3oool00`2oM4P0W=oo0?ooo`0^0?ooo`030?noM0000000M;oo02P0oooo001Y
0?ooo`040?oOW0180000001809cOob`0oooo00<0oooO09a8M02oool0;03oool00`3ogi`0B00007Bo
o`0T0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0:P3oool00`3ogi`0B00007Bo
o`0_0?ooo`030?oog`2LB7@0_ooo02d0oooo00@0omnL04P0000004P0W=oo:03oool006L0oooo00D0
okmd000000000000001809cOo`0^0?ooo`030?noM000M;l0oooo02/0oooo00<0omnL04P0001d_ol0
903oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo02X0oooo00<0omnL04P0001d_ol0
<03oool00`3o_g@007Bo0?ooo`0Z0?ooo`050?noM000000000000000B02Lgol0:@3oool006T0oooo
00<0_g@0001d_`3oool0;P3oool00`3ookl0M01809cOo`0[0?ooo`030?oOW0180000M;oo02@0oooo
00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0Z0?ooo`030?oOW0180000M;oo0300oooo
00<0oono07@0B02Lgol0;03oool00`2oM00007Bo0?ooo`0Y0?ooo`00JP3oool00`2oM000B9cO0?oo
o`0]0?ooo`030?oo_`1d0000B9cO02/0oooo00<0omnL04P0001d_ol0903oool00`3ookl0M00004RL
g`030?ooo`030=nLB00007@0_ooo02X0oooo00<0omnL04P0001d_ol0<03oool00`3ookl0M00004RL
g`0]0?ooo`030;md0018W=l0oooo02P0oooo001U0?ooo`030?oog`2LW;l0oooo0080oooo00<0gia8
0000M02oool0:03oool00`3o_i`0W=oo0?ooo`020?ooo`030?oOW0180000B9cO02/0oooo00<0omnL
04P0001d_ol09@3oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo02X0oooo00<0omnL
04P0001d_ol0:`3oool00`3o_i`0W=oo0?ooo`020?ooo`030?oOW0180000B9cO02P0oooo00<0oooO
09bL_`3oool00P3oool00`3OW4P0001d0;ooo`0X0?ooo`00IP3oool01P3o_g@0B9cO0?ooo`3ookl0
M000001d_bT0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooobT0oooo00D0oooO09a8
00000000000007Boo`0U0?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`0Y0?ooo`050?oo
g`2LB00000000000001d_ol0:`3oool0203ooml0W4Q809cOo`3oool0oono07@0000007@0_ooo:@3o
ool01P3o_g@0B9cO0?ooo`3ookl0M000001d_bT0oooo001W0?ooo`050=nLB000000000000000M02o
ool0:P3oool01P3ooml0W4P0000000000000001d0;ooob`0oooo00<0okmd0000001d_ol09P3oool0
1@3ookl0M0000000000007@0_ooo02`0oooo00<0okmd0000001d_ol0;03oool01P3ooml0W4P00000
00000000001d0;ooob/0oooo00D0gia8000000000000001d0;ooo`0Y0?ooo`00o`3ooona0?ooo`00
o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00
o`3ooona0?ooo`00o`3ooona0?ooo`00G03ooooo0000038000008`3oool005`0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0240oooo001L0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0Q0?ooo`00
G03oool00`000000oooo0?ooo`2E0?ooo`030000003oool0oooo09D0oooo00<000000?ooo`3oool0
8@3oool005`0oooo0P00002F0?ooo`030000003oool0oooo09<0oooo0`00000S0?ooo`00G03oool0
0`000000oooo0?ooo`1E0?ooo`H0o`00>P3oool00`000000oooo0?ooo`0h0?ooo`D0o`00F03oool0
0`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`1D0?ooo`030?l0003oool0oooo
00@0oooo0P3o000h0?ooo`030000003oool0oooo03H0oooo0P3o00050?ooo`030?l0003oool0oooo
05D0oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0D`3oool00`3o0000
oooo0?ooo`060?ooo`030?l0003oool0oooo03H0oooo00<000000?ooo`3oool0=P3oool00`3o0000
oooo0?ooo`050?ooo`80o`00E@3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo
0?ooo`1B0?ooo`030?l0003oool0oooo00P0oooo00<0o`000?ooo`3oool0=@3oool00`000000oooo
0?ooo`0d0?ooo`80o`002@3oool00`3o0000oooo0?ooo`1C0?ooo`030000003oool0oooo0240oooo
001L0?ooo`030000003oool0oooo0540oooo00<0o`000?ooo`3oool02P3oool00`3o0000oooo0?oo
o`0d0?ooo`030000003oool0oooo03<0oooo00<0o`000?ooo`3oool02P3oool00`3o0000oooo0?oo
o`1B0?ooo`030000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo0500oooo00<0o`00
0?ooo`3oool0303oool00`3o0000oooo0?ooo`0c0?ooo`030000003oool0oooo0380oooo00<0o`00
0?ooo`3oool0303oool00`3o0000oooo0?ooo`1A0?ooo`030000003oool0oooo0240oooo001L0?oo
o`800000D03oool00`3o0000oooo0?ooo`0=0?ooo`030?l0003oool0oooo03<0oooo00<000000?oo
o`3oool0<P3oool00`3o0000oooo0?ooo`0=0?ooo`030?l0003oool0oooo04h0oooo0`00000S0?oo
o`00G03oool00`000000oooo0?ooo`1>0?ooo`030?l0003oool0oooo00l0oooo00<0o`000?ooo`3o
ool0<P3oool00`000000oooo0?ooo`0a0?ooo`030?l0003oool0oooo00l0oooo00<0o`000?ooo`3o
ool0C`3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`1>0?ooo`030?l0
003oool0oooo00l0oooo00<0o`000?ooo`3oool0<P3oool00`000000oooo0?ooo`0a0?ooo`030?l0
003oool0oooo00l0oooo00<0o`000?ooo`3oool0C`3oool00`000000oooo0?ooo`0Q0?ooo`00G03o
ool00`000000oooo0?ooo`1=0?ooo`030?l0003oool0oooo0140oooo00<0o`000?ooo`3oool0<@3o
ool00`000000oooo0?ooo`0`0?ooo`030?l0003oool0oooo0140oooo00<0o`000?ooo`3oool0CP3o
ool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`1=0?ooo`030?l0003oool0
oooo0140oooo00<0o`000?ooo`3oool0<@3oool00`000000oooo0?ooo`0`0?ooo`030?l0003oool0
oooo0140oooo00<0o`000?ooo`3oool0CP3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`00
0000oooo0?ooo`1<0?ooo`030?l0003oool0oooo01<0oooo00<0o`000?ooo`3oool0<03oool00`00
0000oooo0?ooo`0_0?ooo`030?l0003oool0oooo01<0oooo00<0o`000?ooo`3oool0C@3oool00`00
0000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`1<0?ooo`030?l0003oool0oooo01<0
oooo00<0o`000?ooo`3oool0<03oool00`000000oooo0?ooo`0_0?ooo`030?l0003oool0oooo01<0
oooo00<0o`000?ooo`3oool0C@3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool2000004`0oooo
00<0o`000?ooo`3oool05@3oool00`3o0000oooo0?ooo`0_0?ooo`030000003oool0oooo02h0oooo
00<0o`000?ooo`3oool05@3oool00`3o0000oooo0?ooo`1:0?ooo`<000008`3oool005`0oooo00<0
00000?ooo`3oool0B`3oool00`3o0000oooo0?ooo`0E0?ooo`030?l0003oool0oooo02l0oooo00<0
00000?ooo`3oool0;P3oool00`3o0000oooo0?ooo`0E0?ooo`030?l0003oool0oooo04`0oooo00<0
00000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0B`3oool00`3o0000oooo0?ooo`0F
0?ooo`030?l0003oool0oooo02h0oooo00<000000?ooo`3oool0;@3oool00`3o0000oooo0?ooo`0G
0?ooo`030?l0003oool0oooo04/0oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?oo
o`3oool0BP3oool00`3o0000oooo0?ooo`0G0?ooo`030?l0003oool0oooo02h0oooo00<000000?oo
o`3oool0;@3oool00`3o0000oooo0?ooo`0G0?ooo`030?l0003oool0oooo04/0oooo00<000000?oo
o`3oool08@3oool005`0oooo00<000000?ooo`3oool0BP3oool00`3o0000oooo0?ooo`0H0?ooo`03
0?l0003oool0oooo02d0oooo00<000000?ooo`3oool0;03oool00`3o0000oooo0?ooo`0I0?ooo`03
0?l0003oool0oooo04X0oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0
103ooolK000000X0oooo1`00000;0?ooo`L000001`3oool0103o0000oooo0?ooo`3oool7000000/0
oooo1`00000;0?ooo`L00000303oool7000000X0oooo2000000;0?ooo`L000002`3oool7000000<0
oooo00<0o`000?ooo`3oool01@3oool8000000/0oooo1`00000;0?ooo`L000002`3oool700000080
oooo6P0000070?ooo`030000003oool0oooo0240oooo001L0?ooo`800000803oool?000001/0oooo
00<0o`000?ooo`3oool06@3oool00`3o0000oooo0?ooo`0L0?ooob<00000703oool00`3o0000oooo
0?ooo`0J0?ooo`030?l0003oool0oooo01X0oooo3`00000O0?ooo`<000008`3oool005`0oooo00<0
00000?ooo`3oool0;P3oool3000001P0oooo00<0o`000?ooo`3oool06P3oool00`3o0000oooo0?oo
o`0F0?ooo`D000004@3oool00`000000oooo0?ooo`0?0?ooo`<000006@3oool00`3o0000oooo0?oo
o`0K0?ooo`030?l0003oool0oooo01H0oooo0`00000`0?ooo`030000003oool0oooo0240oooo001<
0?ooo`030?oo_`1d000000000080000000<0M;oo0?ooo`3oool0203oool00`000000oooo0?ooo`0a
0?ooo`8000005@3oool00`3o0000oooo0?ooo`0K0?ooo`030?l0003oool0oooo01@0oooo0P00000F
0?ooo`030000003oool0oooo0180oooo1000000D0?ooo`030?l0003oool0oooo01`0oooo00<0o`00
0?ooo`3oool04`3oool3000003<0oooo00<000000?ooo`3oool08@3oool004`0oooo00<0omnL04P0
0018W=l00P3oool00`2oM000B9cO0?ooo`080?ooo`030000003oool0oooo03<0oooo0P00000C0?oo
o`030?l0003oool0oooo01`0oooo00<0o`000?ooo`3oool04@3oool2000001P0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`0A0?ooo`030?l0003oool0oooo01`0oooo00<0o`000?oo
o`3oool04P3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo0240oooo001A0?ooo`03
0?oo_`1d04P0W=oo00P0oooo00<000000?ooo`3oool0=@3oool200000140oooo00<0o`000?ooo`3o
ool0703oool00`3o0000oooo0?ooo`0@0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3o
ool05`3oool200000100oooo00<0o`000?ooo`3oool07P3oool00`3o0000oooo0?ooo`0?0?ooo`80
0000=`3oool00`000000oooo0?ooo`0Q0?ooo`00D@3oool00`3ookl0M00007Boo`080?ooo`030000
003oool0oooo03L0oooo00<000000?ooo`3oool03@3oool00`3o0000oooo0?ooo`0M0?ooo`030?l0
003oool0oooo00h0oooo0P00000K0?ooo`030000003oool0oooo01T0oooo0P00000>0?ooo`030?l0
003oool0oooo01h0oooo00<0o`000?ooo`3oool03@3oool2000003T0oooo00<000000?ooo`3oool0
8@3oool00540oooo00<0okmd0000001d_ol0203oool00`000000oooo0?ooo`0h0?ooo`030000003o
ool0oooo00`0oooo00<0o`000?ooo`3oool07P3oool00`3o0000oooo0?ooo`0<0?ooo`030000003o
ool0oooo01/0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0;0?ooo`030?l0003o
ool0oooo01l0oooo00<0o`000?ooo`3oool02`3oool00`000000oooo0?ooo`0i0?ooo`030000003o
ool0oooo0240oooo001@0?ooo`040?oOW0180000001809cOo`P0ooooBP0000000`3o000000000000
000N000000030?l000000000000005D0000000<0o`00000000000000800000000`3o000000000000
0018000002<0oooo001>0?ooo`050?noM000000000000000B02Lgol02@3oool00`000000oooo0?oo
o`0j0?ooo`030000003oool0oooo00T0oooo00<0o`000?ooo`3oool07`3oool00`3o0000oooo0?oo
o`0:0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0703oool00`000000oooo0?oo
o`090?ooo`030?l0003oool0oooo0200oooo00<0o`000?ooo`3oool02@3oool00`000000oooo0?oo
o`0k0?ooo`030000003oool0oooo0240oooo001@0?ooo`030;md0000M;l0oooo00T0oooo00<00000
0?ooo`3oool0>`3oool00`000000oooo0?ooo`080?ooo`030?l0003oool0oooo0200oooo00<0o`00
0?ooo`3oool0203oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01d0oooo00<00000
0?ooo`3oool0203oool00`3o0000oooo0?ooo`0Q0?ooo`030?l0003oool0oooo00L0oooo00<00000
0?ooo`3oool0?03oool00`000000oooo0?ooo`0Q0?ooo`00D@3oool00`2oM000B9cO0?ooo`080?oo
o`030000003oool0oooo03`0oooo00<000000?ooo`3oool01`3oool00`3o0000oooo0?ooo`0P0?oo
o`030?l0003oool0oooo00L0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0N0?oo
o`030000003oool0oooo00H0oooo00<0o`000?ooo`3oool08P3oool00`3o0000oooo0?ooo`060?oo
o`030000003oool0oooo03d0oooo00<000000?ooo`3oool08@3oool004`0oooo00<0oooO09bL_`3o
ool00P3oool00`3OW4P0001d0;ooo`080?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3o
ool01P3oool00`3o0000oooo0?ooo`0R0?ooo`030?l0003oool0oooo00H0oooo00<000000?ooo`3o
ool07`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo00D0oooo00<0o`000?ooo`3o
ool08P3oool00`3o0000oooo0?ooo`060?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3o
ool08@3oool004d0oooo00H0okmd04RLg`3oool0oono07@00000M;l90?ooo`030000003oool0oooo
03d0oooo00<000000?ooo`3oool01@3oool00`3o0000oooo0?ooo`0R0?ooo`030?l0003oool0oooo
00D0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo
00D0oooo00<0o`000?ooo`3oool08P3oool00`3o0000oooo0?ooo`050?ooo`030000003oool0oooo
03h0oooo00<000000?ooo`3oool08@3oool004h0oooo00D0gia8000000000000001d0;ooo`090?oo
o`030000003oool0oooo03h0oooo00<000000?ooo`3oool0103oool00`3o0000oooo0?ooo`0R0?oo
o`030?l0003oool0oooo00@0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0P0?oo
o`030000003oool0oooo00@0oooo00<0o`000?ooo`3oool08`3oool00`3o0000oooo0?ooo`040?oo
o`030000003oool0oooo03h0oooo00<000000?ooo`3oool08@3oool005`0oooo0P00000o0?ooo`03
0000003oool0oooo00@0oooo00<0o`000?ooo`3oool08`3oool00`3o0000oooo0?ooo`030?ooo`03
0000003oool0oooo0240oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`020?ooo`03
0?l0003oool0oooo02@0oooo00<0o`000?ooo`3oool00`3oool00`000000oooo0?ooo`0m0?ooo`<0
00008`3oool005`0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`020?ooo`030?l0
003oool0oooo02@0oooo00<0o`000?ooo`3oool00P3oool00`000000oooo0?ooo`0R0?ooo`030000
003oool0oooo0240oooo00<000000?ooo`3oool00P3oool00`3o0000oooo0?ooo`0T0?ooo`030?l0
003oool0oooo00<0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0Q0?ooo`00G03o
ool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo0080oooo00<0o`000?ooo`3oool0903o
ool01@3o0000oooo0?ooo`3oool0000002D0oooo00<000000?ooo`3oool08P3oool01@000000oooo
0?ooo`3oool0o`0002L0oooo00D0o`000?ooo`3oool0oooo000000120?ooo`030000003oool0oooo
0240oooo001L0?ooo`030000003oool0oooo0400oooo00D000000?ooo`3oool0oooo0?l0000W0?oo
o`040?l0003oool0oooo000002D0oooo00<000000?ooo`3oool08P3oool010000000oooo0?ooo`3o
000X0?ooo`050?l0003oool0oooo0?ooo`000000@P3oool00`000000oooo0?ooo`0Q0?ooo`00G03o
ool00`000000oooo0?ooo`100?ooo`040000003oool0oooo0?l002P0oooo00<0o`000?ooo`000000
9P3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0o`0002P0oooo00@0o`000?ooo`3oool0
0000@`3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`110?ooo`030000
003oool0o`0002P0oooo00<0o`000?ooo`0000009P3oool00`000000oooo0?ooo`0S0?ooo`030000
003oool0o`0002T0oooo00<0o`000?ooo`000000@`3oool00`000000oooo0?ooo`0Q0?ooo`00G03o
ool200000480oooo00<000000?ooo`3o0000:03oool00`3o000000000?ooo`0V0?ooo`030000003o
ool0oooo02<0oooo00<000000?ooo`3o0000:@3oool00`3o000000000?ooo`110?ooo`<000008`3o
ool005`0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0
oooo02D0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0X0?ooo`030?l000000000
oooo04<0oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0@P3oool00`00
0000oooo0?ooo`0W0?ooo`030000003o0000oooo02H0oooo00<000000?ooo`3oool0903oool00`00
0000oooo0?ooo`0X0?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool08@3oool005`0
oooo00<000000?ooo`3oool0@P3oool00`3o000000000?ooo`0W0?ooo`030000003o0000oooo02H0
oooo00<000000?ooo`3oool0903oool00`3o000000000?ooo`0X0?ooo`030000003o0000oooo04<0
oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0@P3oool00`3o00000000
0?ooo`0V0?ooo`040000003oool0oooo0?l002H0oooo00<000000?ooo`3oool0903oool00`3o0000
00000?ooo`0W0?ooo`030000003oool0o`0004@0oooo00<000000?ooo`3oool08@3oool005`0oooo
00<000000?ooo`3oool0@@3oool00`3o0000oooo0000000W0?ooo`040000003oool0oooo0?l002H0
oooo00<000000?ooo`3oool08`3oool00`3o0000oooo0000000X0?ooo`030000003oool0o`0004@0
oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0@@3oool0103o0000oooo
0?ooo`00000V0?ooo`040000003oool0oooo0?l002H0oooo00<000000?ooo`3oool08`3oool0103o
0000oooo0?ooo`00000W0?ooo`040000003oool0oooo0?l004<0oooo00<000000?ooo`3oool08@3o
ool005`0oooo0P0000120?ooo`040?l0003oool0oooo000002D0oooo00<000000?ooo`3oool00P3o
ool00`3o0000oooo0?ooo`0S0?ooo`030000003oool0oooo02<0oooo00@0o`000?ooo`3oool00000
9P3oool01@000000oooo0?ooo`3oool0o`000440oooo0`00000S0?ooo`00G03oool00`000000oooo
0?ooo`100?ooo`050?l0003oool0oooo0?ooo`0000009@3oool00`000000oooo0?ooo`020?ooo`03
0?l0003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool01@3o0000oooo0?ooo`3oool00000
02D0oooo00D000000?ooo`3oool0oooo0?l000130?ooo`030000003oool0oooo0240oooo001L0?oo
o`030000003oool0oooo0400oooo00<0o`000?ooo`3oool00P3oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo00<0oooo00<0o`000?ooo`3oool08`3oool00`000000oooo0?ooo`0R0?oo
o`030?l0003oool0oooo0080oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`020?oo
o`030?l0003oool0oooo0440oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3o
ool0@03oool00`3o0000oooo0?ooo`020?ooo`030000003oool0oooo0240oooo00<000000?ooo`3o
ool00`3oool00`3o0000oooo0?ooo`0S0?ooo`030000003oool0oooo0280oooo00<0o`000?ooo`3o
ool00`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo00<0oooo00<0o`000?ooo`3o
ool0@03oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`100?ooo`030?l0
003oool0oooo00<0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`040?ooo`030?l0
003oool0oooo0280oooo00<000000?ooo`3oool08P3oool00`3o0000oooo0?ooo`030?ooo`030000
003oool0oooo0240oooo00<000000?ooo`3oool00`3oool00`3o0000oooo0?ooo`100?ooo`030000
003oool0oooo0240oooo001L0?ooo`030000003oool0oooo03l0oooo00<0o`000?ooo`3oool0103o
ool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo00D0oooo00<0o`000?ooo`3oool08P3o
ool00`000000oooo0?ooo`0R0?ooo`030?l0003oool0oooo00<0oooo00<000000?ooo`3oool0803o
ool00`000000oooo0?ooo`040?ooo`030?l0003oool0oooo0400oooo00<000000?ooo`3oool08@3o
ool005`0oooo00<000000?ooo`3oool0?`3oool00`3o0000oooo0?ooo`040?ooo`030000003oool0
oooo01l0oooo00<000000?ooo`3oool01@3oool00`3o0000oooo0?ooo`0R0?ooo`030000003oool0
oooo0240oooo00<0o`000?ooo`3oool01@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0
oooo00@0oooo00<0o`000?ooo`3oool0@03oool00`000000oooo0?ooo`0Q0?ooo`00G03oool20000
0400oooo00<0o`000?ooo`3oool01@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo
00L0oooo00<0o`000?ooo`3oool08@3oool00`000000oooo0?ooo`0Q0?ooo`030?l0003oool0oooo
00D0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`050?ooo`030?l0003oool0oooo
03d0oooo0`00000S0?ooo`00?`3oool00`3ookl0M000000000020000000307Boo`3oool0oooo0080
oooo00H0okmd0000001d_ol0oooo0?oog`2LB0030000000307Boo`3oool0oooo00L0oooo00<00000
0?ooo`3oool0?`3oool00`3o0000oooo0?ooo`050?ooo`030000003oool0oooo01d0oooo00<00000
0?ooo`3oool01`3oool00`3o0000oooo0?ooo`0Q0?ooo`030000003oool0oooo0240oooo00<0o`00
0?ooo`3oool01P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo00H0oooo00<0o`00
0?ooo`3oool0?`3oool00`000000oooo0?ooo`0Q0?ooo`00?`3oool00`3ogi`0B00004RLg`020?oo
o`030;md0018W=l0oooo0080oooo00L0okmd0000001d_ol0oooo0?oog`2LB7@0_ooo0080oooo00<0
_g@007Boo`3oool01`3oool00`000000oooo0?ooo`0n0?ooo`030?l0003oool0oooo00H0oooo00<0
00000?ooo`3oool07@3oool00`000000oooo0?ooo`070?ooo`030?l0003oool0oooo0240oooo00<0
00000?ooo`3oool08@3oool00`3o0000oooo0?ooo`060?ooo`030000003oool0oooo01d0oooo00<0
00000?ooo`3oool01P3oool00`3o0000oooo0?ooo`0o0?ooo`030000003oool0oooo0240oooo0014
0?ooo`030?oo_`1d04P0W=oo00/0oooo00<0oooO09a8M02oool01`3oool00`000000oooo0?ooo`0n
0?ooo`030?l0003oool0oooo00L0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`08
0?ooo`030?l0003oool0oooo0240oooo00<000000?ooo`3oool0803oool00`3o0000oooo0?ooo`07
0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool01`3oool00`3o0000oooo0?ooo`0o
0?ooo`030000003oool0oooo0240oooo00140?ooo`030?oo_`1d0000M;oo00`0oooo00<0_g@007Bo
o`3oool01P3oool00`000000oooo0?ooo`0n0?ooo`030?l0003oool0oooo00L0oooo00<000000?oo
o`3oool06`3oool00`000000oooo0?ooo`090?ooo`030?l0003oool0oooo0200oooo00<000000?oo
o`3oool0803oool00`3o0000oooo0?ooo`080?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool01`3oool00`3o0000oooo0?ooo`0o0?ooo`030000003oool0oooo0240oooo00140?ooo`03
0?noM0000000M;oo00/0oooo00<0oooO09a80018W=l01`3oool00`000000oooo0?ooo`0n0?ooo`03
0?l0003oool0oooo00P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0:0?ooo`03
0?l0003oool0oooo0200oooo00<000000?ooo`3oool0803oool00`3o0000oooo0?ooo`080?ooo`03
0000003oool0oooo01/0oooo00<000000?ooo`3oool0203oool00`3o0000oooo0?ooo`0n0?ooo`03
0000003oool0oooo0240oooo00130?ooo`040?oOW0180000001809cOo`/0oooo00<0gia80000001d
_ol01`3oool3000003d0oooo00<0o`000?ooo`3oool02@3oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo00X0oooo00<0o`000?ooo`3oool0803oool00`000000oooo0?ooo`0P0?ooo`03
0?l0003oool0oooo00T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`090?ooo`03
0?l0003oool0oooo03`0oooo0`00000S0?ooo`00@@3oool01@3o_g@000000000000004P0W=oo00P0
oooo00L0gmoO0?ooo`3ookl0M0000000000007@0_ooo00L0oooo00<000000?ooo`3oool0?@3oool0
0`3o0000oooo0?ooo`090?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool02P3oool0
0`3o0000oooo0?ooo`0P0?ooo`030000003oool0oooo01l0oooo00<0o`000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00T0oooo00<0o`000?ooo`3oool0?P3oool0
0`000000oooo0?ooo`0Q0?ooo`00@`3oool00`2oM00007Bo0?ooo`080?ooo`060=nLB00000000000
0000000004P0W=oo203oool00`000000oooo0?ooo`0m0?ooo`030?l0003oool0oooo00X0oooo00<0
00000?ooo`3oool05`3oool00`000000oooo0?ooo`0<0?ooo`030?l0003oool0oooo01l0oooo00<0
00000?ooo`3oool07`3oool00`3o0000oooo0?ooo`0;0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool02@3oool00`3o0000oooo0?ooo`0n0?ooo`030000003oool0oooo0240oooo0014
0?ooo`030;md0018W=l0oooo00L0oooo00@0oono07@000000000B9cO2P3oool00`000000oooo0?oo
o`0m0?ooo`030?l0003oool0oooo00X0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?oo
o`0<0?ooo`030?l0003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool00`3o0000oooo0?oo
o`0;0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool02`3oool00`3o0000oooo0?oo
o`0m0?ooo`030000003oool0oooo0240oooo000o0?ooo`030?oog`2LW;l0oooo0080oooo00<0gia8
0000M02oool0203oool00`2oM000M;oo0?ooo`0:0?ooo`030000003oool0oooo03`0oooo00<0o`00
0?ooo`3oool02`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo00d0oooo00<0o`00
0?ooo`3oool07`3oool00`000000oooo0?ooo`0O0?ooo`030?l0003oool0oooo00/0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`0;0?ooo`030?l0003oool0oooo03d0oooo00<00000
0?ooo`3oool08@3oool00400oooo00H0okmd04RLg`3oool0oono07@00000M;l90?ooo`060?oOW018
000000000000000004P0W=oo1`3oool00`000000oooo0?ooo`0l0?ooo`030?l0003oool0oooo00`0
oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0=0?ooo`030?l0003oool0oooo01l0
oooo00<000000?ooo`3oool07P3oool00`3o0000oooo0?ooo`0=0?ooo`030000003oool0oooo01D0
oooo00<000000?ooo`3oool0303oool00`3o0000oooo0?ooo`0m0?ooo`030000003oool0oooo0240
oooo00110?ooo`050=nLB000000000000000M02oool02P3oool01@2oM0000000000000000000B9cO
00L0oooo00<000000?ooo`3oool0?03oool00`3o0000oooo0?ooo`0<0?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool03P3oool00`3o0000oooo0?ooo`0N0?ooo`030000003oool0oooo
01h0oooo00<0o`000?ooo`3oool03@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo
00`0oooo00<0o`000?ooo`3oool0?@3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool2000003d0
oooo00<0o`000?ooo`3oool03@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00l0
oooo00<0o`000?ooo`3oool07P3oool00`000000oooo0?ooo`0N0?ooo`030?l0003oool0oooo00h0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0=0?ooo`030?l0003oool0oooo03X0
oooo0`00000S0?ooo`00G03oool00`000000oooo0?ooo`0k0?ooo`030?l0003oool0oooo00h0oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0?0?ooo`030?l0003oool0oooo01h0oooo
00<000000?ooo`3oool07P3oool00`3o0000oooo0?ooo`0>0?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool03P3oool00`3o0000oooo0?ooo`0l0?ooo`030000003oool0oooo0240oooo
001L0?ooo`030000003oool0oooo03/0oooo00<0o`000?ooo`3oool03`3oool00`000000oooo0?oo
o`0A0?ooo`030000003oool0oooo0100oooo00<0o`000?ooo`3oool07P3oool00`000000oooo0?oo
o`0M0?ooo`030?l0003oool0oooo00l0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`0>0?ooo`030?l0003oool0oooo03`0oooo00<000000?ooo`3oool08@3oool005`0oooo00<00000
0?ooo`3oool0>`3oool00`3o0000oooo0?ooo`0?0?ooo`030000003oool0oooo0140oooo00<00000
0?ooo`3oool04@3oool00`3o0000oooo0?ooo`0M0?ooo`030000003oool0oooo01d0oooo00<0o`00
0?ooo`3oool0403oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<0o`00
0?ooo`3oool0?03oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`0k0?oo
o`030?l0003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0A0?oo
o`030?l0003oool0oooo01d0oooo00<000000?ooo`3oool07@3oool00`3o0000oooo0?ooo`0@0?oo
o`030000003oool0oooo0140oooo00<000000?ooo`3oool03`3oool00`3o0000oooo0?ooo`0l0?oo
o`030000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo03/0oooo00<0o`000?ooo`3o
ool0403oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0180oooo00<0o`000?ooo`3o
ool07@3oool00`000000oooo0?ooo`0M0?ooo`030?l0003oool0oooo0140oooo00<000000?ooo`3o
ool03`3oool00`000000oooo0?ooo`0A0?ooo`030?l0003oool0oooo03/0oooo00<000000?ooo`3o
ool08@3oool005`0oooo00<000000?ooo`3oool0>P3oool00`3o0000oooo0?ooo`0B0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool04P3oool00`3o0000oooo0?ooo`0M0?ooo`030000
003oool0oooo01`0oooo00<0o`000?ooo`3oool04P3oool00`000000oooo0?ooo`0?0?ooo`030000
003oool0oooo0140oooo00<0o`000?ooo`3oool0>`3oool00`000000oooo0?ooo`0Q0?ooo`00G03o
ool2000003/0oooo00<0o`000?ooo`3oool04P3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo01<0oooo00<0o`000?ooo`3oool07@3oool00`000000oooo0?ooo`0L0?ooo`030?l0003o
ool0oooo01<0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0B0?ooo`030?l0003o
ool0oooo03T0oooo0`00000S0?ooo`009`3oool00`3OW7@0W=oo0?ooo`0b0?ooo`030000003oool0
oooo03X0oooo00<0o`000?ooo`3oool04`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo01@0oooo00<0o`000?ooo`3oool0703oool00`000000oooo0?ooo`0L0?ooo`030?l0003oool0
oooo01<0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0B0?ooo`030?l0003oool0
oooo03/0oooo00<000000?ooo`3oool08@3oool002L0oooo00<0gia8001d_`3oool0<P3oool00`00
0000oooo0?ooo`0j0?ooo`030?l0003oool0oooo01<0oooo00<000000?ooo`3oool02`3oool00`00
0000oooo0?ooo`0E0?ooo`030?l0003oool0oooo01`0oooo00<000000?ooo`3oool0703oool00`3o
0000oooo0?ooo`0D0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0503oool00`3o
0000oooo0?ooo`0j0?ooo`030000003oool0oooo0240oooo000W0?ooo`050=nLB000000000000000
001d_ol0<03oool00`000000oooo0?ooo`0i0?ooo`030?l0003oool0oooo01D0oooo00<000000?oo
o`3oool02P3oool00`000000oooo0?ooo`0E0?ooo`030?l0003oool0oooo01`0oooo00<000000?oo
o`3oool06`3oool00`3o0000oooo0?ooo`0E0?ooo`030000003oool0oooo00X0oooo00<000000?oo
o`3oool05@3oool00`3o0000oooo0?ooo`0j0?ooo`030000003oool0oooo0240oooo000W0?ooo`03
0=nLB0000000000000<0000000<0001d0;ooo`3oool0;03oool00`000000oooo0?ooo`0i0?ooo`03
0?l0003oool0oooo01D0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0F0?ooo`03
0?l0003oool0oooo01`0oooo00<000000?ooo`3oool06`3oool00`3o0000oooo0?ooo`0F0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool05@3oool00`3o0000oooo0?ooo`0j0?ooo`03
0000003oool0oooo0240oooo000W0?ooo`060=nLM02Lgol0oooo0?ooo`3ogi`0B0000`0000000`00
B9`0gooo0?ooo`0Y0?ooo`030000003oool0oooo03T0oooo00<0o`000?ooo`3oool05@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo01L0oooo00<0o`000?ooo`3oool06`3oool00`00
0000oooo0?ooo`0K0?ooo`030?l0003oool0oooo01H0oooo00<000000?ooo`3oool0203oool00`00
0000oooo0?ooo`0F0?ooo`030?l0003oool0oooo03X0oooo00<000000?ooo`3oool08@3oool002h0
oooo00<0okmd0000000000000P0000000`00M;l0oooo0?ooo`0V0?ooo`800000>P3oool00`3o0000
oooo0?ooo`0F0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0603oool00`3o0000
oooo0?ooo`0K0?ooo`030000003oool0oooo01X0oooo00<0o`000?ooo`3oool0603oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo01H0oooo00<0o`000?ooo`3oool0>03oool3000002<0
oooo000a0?ooo`040?noM0000000001d0;ooobL0oooo00<000000?ooo`3oool0>03oool00`3o0000
oooo0?ooo`0H0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0603oool00`3o0000
oooo0?ooo`0K0?ooo`030000003oool0oooo01X0oooo00<0o`000?ooo`3oool06@3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo01P0oooo00<0o`000?ooo`3oool0>@3oool00`000000
oooo0?ooo`0Q0?ooo`00;P3oool0103o_g@0000000000000M;lZ0?ooo`030000003oool0oooo03P0
oooo00<0o`000?ooo`3oool06@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01T0
oooo00<0o`000?ooo`3oool06`3oool00`000000oooo0?ooo`0J0?ooo`030?l0003oool0oooo01X0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0H0?ooo`030?l0003oool0oooo03T0
oooo00<000000?ooo`3oool08@3oool002L0oooo00T0giad09cOo`3oool0oooo0?oOW01800000000
0018W03Oool0;03oool00`000000oooo0?ooo`0h0?ooo`030?l0003oool0oooo01X0oooo0P000002
0?ooo`8000007@3oool00`3o0000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<0o`00
0?ooo`3oool06P3oool2000000<0oooo0P00000K0?ooo`030?l0003oool0oooo03T0oooo00<00000
0?ooo`3oool08@3oool002L0oooo00H0gia800000000000000000000M02oool_0?ooo`030000003o
ool0oooo03P0oooo00<0o`000?ooo`3oool06`3oool4000001h0oooo00<0o`000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0I0?ooo`030?l0003oool0oooo01`0oooo1000000M0?ooo`030?l0003o
ool0oooo03T0oooo00<000000?ooo`3oool08@3oool002L0oooo00<0gia80000001d_ol0<P3oool0
0`000000oooo0?ooo`0g0?ooo`030?l0003oool0oooo03h0oooo00<0o`000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0I0?ooo`030?l0003oool0oooo03h0oooo00<0o`000?ooo`3oool0>03oool0
0`000000oooo0?ooo`0Q0?ooo`009`3oool00`3OW7@0W=oo0?ooo`0b0?ooo`030000003oool0oooo
03L0oooo00<0o`000?ooo`3oool0?P3oool00`3o0000oooo0?ooo`0J0?ooo`030000003oool0oooo
01T0oooo00<0o`000?ooo`3oool0?P3oool00`3o0000oooo0?ooo`0h0?ooo`030000003oool0oooo
0240oooo001L0?ooo`800000>03oool00`3o0000oooo0?ooo`0n0?ooo`030?l0003oool0oooo01X0
oooo00<000000?ooo`3oool06@3oool00`3o0000oooo0?ooo`0n0?ooo`030?l0003oool0oooo03H0
oooo0`00000S0?ooo`00G03oool00`000000oooo0?ooo`0g0?ooo`030?l0003oool0oooo03l0oooo
00<0o`000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030?l0003oool0oooo03l0oooo
00<0o`000?ooo`3oool0>03oool00`000000oooo0?ooo`0Q0?ooo`00<@3oool01P3ooml0W4P00018
W03Oool0oooO09bL_aT0oooo00@0oooO09a80000B9`0gooo203oool00`000000oooo0?ooo`0g0?oo
o`030?l0003oool0oooo03l0oooo00<0o`000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?oo
o`030?l0003oool0oooo0400oooo00<0o`000?ooo`3oool0=`3oool00`000000oooo0?ooo`0Q0?oo
o`00<@3oool01P3ookl0M0000018W03Oool0_gA809cOoaT0oooo00@0oooO09a80000B9`0gooo203o
ool00`000000oooo0?ooo`0f0?ooo`030?l0003oool0oooo0400oooo00<0o`000?ooo`3oool06@3o
ool00`000000oooo0?ooo`0H0?ooo`030?l0003oool0oooo0400oooo00<0o`000?ooo`3oool0=`3o
ool00`000000oooo0?ooo`0Q0?ooo`00<P3oool0103OW4P000000000001d_olJ0?ooo`040?oog`2L
B00004RL0=ooo`P0oooo00<000000?ooo`3oool0=P3oool00`3o0000oooo0?ooo`100?ooo`030?l0
003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`3o0000oooo0?ooo`100?ooo`030?l0
003oool0oooo03L0oooo00<000000?ooo`3oool08@3oool004`0oooo00<0_g@00000000000001@00
00000`1d_ol0oooo0?ooo`050?ooo`030000003oool0oooo03H0oooo00<0o`000?ooo`3oool0@@3o
ool00`3o0000oooo0?ooo`0H0?ooo`030000003oool0oooo01L0oooo00<0o`000?ooo`3oool0@@3o
ool00`3o0000oooo0?ooo`0g0?ooo`030000003oool0oooo0240oooo001<0?ooo`080?oOW01dW=l0
oooo0?ooo`3ooml0W4P00018W03Oool80?ooo`030000003oool0oooo03H0oooo00<0o`000?ooo`3o
ool0@@3oool00`3o0000oooo0?ooo`0H0?ooo`030000003oool0oooo01L0oooo00<0o`000?ooo`3o
ool0@@3oool00`3o0000oooo0?ooo`0g0?ooo`030000003oool0oooo0240oooo001=0?ooo`070=nL
B01d_ol0oooo0?oog`2LB00004RL0=ooo`080?ooo`<00000=@3oool00`3o0000oooo0?ooo`120?oo
o`030?l0003oool0oooo01P0oooo00<000000?ooo`3oool05`3oool00`3o0000oooo0?ooo`120?oo
o`030?l0003oool0oooo03@0oooo0`00000S0?ooo`00C@3oool01`3ooml0W4Qd0;ooo`3ooml0W4P0
0018W03Oool0203oool00`000000oooo0?ooo`0e0?ooo`030?l0003oool0oooo0480oooo00<0o`00
0?ooo`3oool0603oool00`000000oooo0?ooo`0G0?ooo`030?l0003oool0oooo0480oooo00<0o`00
0?ooo`3oool0=P3oool00`000000oooo0?ooo`0Q0?ooo`00CP3oool01P3ogi`0B7Bo0?oog`2LB000
04RL0=ooo`P0oooo00<000000?ooo`3oool0=@3oool00`3o0000oooo0?ooo`130?ooo`030?l0003o
ool0oooo01L0oooo00<000000?ooo`3oool05P3oool00`3o0000oooo0?ooo`130?ooo`030?l0003o
ool0oooo03H0oooo00<000000?ooo`3oool08@3oool004l0oooo00D0gia807Bog`2LB00004RL0=oo
o`080?ooo`030000003oool0oooo03D0oooo00<0o`000?ooo`3oool0@`3oool00`3o0000oooo0?oo
o`0G0?ooo`030000003oool0oooo01H0oooo00<0o`000?ooo`3oool0@`3oool00`3o0000oooo0?oo
o`0f0?ooo`030000003oool0oooo0240oooo000[0?ooo`050?oOW01dW=l0oooo0?oo_`1d00000P00
00000`0007@0_ooo0?ooo`0J0?ooo`050?oog`2LB4P0B4P00018W03Oool0203oool00`000000oooo
0?ooo`0d0?ooo`030?l0003oool0oooo04@0oooo00<0o`000?ooo`3oool05`3oool00`000000oooo
0?ooo`0F0?ooo`030?l0003oool0oooo04@0oooo00<0o`000?ooo`3oool0=@3oool00`000000oooo
0?ooo`0Q0?ooo`00:`3oool00`3ooml0W4P0000000030000000407Bo_`1d000004RL0=oooa/0oooo
00@0omnL04P00000B9`0gooo203oool00`000000oooo0?ooo`0d0?ooo`030?l0003oool0oooo04@0
oooo00<0o`000?ooo`3oool05`3oool00`000000oooo0?ooo`0F0?ooo`030?l0003oool0oooo04@0
oooo00<0o`000?ooo`3oool0=@3oool00`000000oooo0?ooo`0Q0?ooo`00<`3oool01P2oM4P0W=no
07@00000000004RL0=oooaP0oooo00<0gia80018W03Oool0203oool2000003D0oooo00<0o`000?oo
o`3oool0A@3oool00`3o0000oooo0?ooo`0F0?ooo`030000003oool0oooo01D0oooo00<0o`000?oo
o`3oool0A@3oool00`3o0000oooo0?ooo`0c0?ooo`<000008`3oool00340oooo00D0gia800000000
0000001d0;ooo`0V0?ooo`030000003oool0oooo03@0oooo00<0o`000?ooo`3oool0A@3oool00`3o
0000oooo0?ooo`0F0?ooo`030000003oool0oooo01D0oooo00<0o`000?ooo`3oool0A@3oool00`3o
0000oooo0?ooo`0e0?ooo`030000003oool0oooo0240oooo000]0?ooo`080?noM000000000000000
0018W=l0oooo0;mdB02LgolW0?ooo`030000003oool0oooo03@0oooo00<0o`000?ooo`3oool0A@3o
ool00`3o0000oooo0?ooo`0F0?ooo`030000003oool0oooo01D0oooo00<0o`000?ooo`3oool0A@3o
ool00`3o0000oooo0?ooo`0e0?ooo`030000003oool0oooo0240oooo000Y0?ooo`060;md00000000
00000000000007@0_ooo0`3oool00`3ookl0M4RL0=ooo`0W0?ooo`030000003oool0oooo03<0oooo
00<0o`000?ooo`3oool0AP3oool00`3o0000oooo0?ooo`0F0?ooo`030000003oool0oooo01D0oooo
00<0o`000?ooo`3oool0AP3oool00`3o0000oooo0?ooo`0d0?ooo`030000003oool0oooo0240oooo
000W0?ooo`040?oog`2LB0000000001d_`H0oooo00<0oooO09a80018W=l0:03oool00`000000oooo
0?ooo`0c0?ooo`030?l0003oool0oooo04L0oooo00<0o`000?ooo`3oool05@3oool00`000000oooo
0?ooo`0D0?ooo`030?l0003oool0oooo04L0oooo00<0o`000?ooo`3oool0=03oool00`000000oooo
0?ooo`0Q0?ooo`00;`3oool0102oM000000000000000M;lY0?ooo`030000003oool0oooo03<0oooo
00<0o`000?ooo`3oool0A`3oool00`3o0000oooo0?ooo`0E0?ooo`030000003oool0oooo01@0oooo
00<0o`000?ooo`3oool0A`3oool00`3o0000oooo0?ooo`0d0?ooo`030000003oool0oooo0240oooo
000/0?ooo`060?oog`2LB00000000000000004P0W=oo:P3oool00`000000oooo0?ooo`0c0?ooo`03
0?l0003oool0oooo04L0oooo00<0o`000?ooo`3oool05@3oool00`000000oooo0?ooo`0D0?ooo`03
0?l0003oool0oooo04L0oooo00<0o`000?ooo`3oool0=03oool00`000000oooo0?ooo`0Q0?ooo`00
:`3oool0103ooml0W4P00018W03Oool]0?ooo`800000=03oool00`3o0000oooo0?ooo`170?ooo`03
0?l0003oool0oooo01D0oooo00<000000?ooo`3oool04`3oool00`3o0000oooo0?ooo`180?ooo`03
0?l0003oool0oooo0380oooo0`00000S0?ooo`00:`3oool00`3ogi`0M9cO0?ooo`0^0?ooo`030000
003oool0oooo03<0oooo00<0o`000?ooo`3oool0B03oool00`3o0000oooo0?ooo`0D0?ooo`030000
003oool0oooo01<0oooo00<0o`000?ooo`3oool0B03oool00`3o0000oooo0?ooo`0d0?ooo`030000
003oool0oooo0240oooo001L0?ooo`030000003oool0oooo0380oooo00<0o`000?ooo`3oool0B@3o
ool00`3o0000oooo0?ooo`0D0?ooo`030000003oool0oooo01<0oooo00<0o`000?ooo`3oool0B@3o
ool00`3o0000oooo0?ooo`0c0?ooo`030000003oool0oooo0240oooo001L0?ooo`030000003oool0
oooo0380oooo00<0o`000?ooo`3oool0B@3oool00`3o0000oooo0?ooo`0D0?ooo`030000003oool0
oooo01<0oooo00<0o`000?ooo`3oool0B@3oool00`3o0000oooo0?ooo`0c0?ooo`030000003oool0
oooo0240oooo001L0?ooo`030000003oool0oooo0380oooo00<0o`000?ooo`3oool0BP3oool00`3o
0000oooo0?ooo`0C0?ooo`030000003oool0oooo0180oooo00<0o`000?ooo`3oool0BP3oool00`3o
0000oooo0?ooo`0c0?ooo`030000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo0380
oooo00<0o`000?ooo`3oool0BP3oool00`3o0000oooo0?ooo`0C0?ooo`030000003oool0oooo0180
oooo00<0o`000?ooo`3oool0BP3oool00`3o0000oooo0?ooo`0c0?ooo`030000003oool0oooo0240
oooo001L0?ooo`030000003oool0oooo0380oooo00<0o`000?ooo`3oool0BP3oool00`3o0000oooo
0?ooo`0C0?ooo`030000003oool0oooo0180oooo00<0o`000?ooo`3oool0BP3oool00`3o0000oooo
0?ooo`0c0?ooo`030000003oool0oooo0240oooo001L0?ooo`800000<`3oool00`3o0000oooo0?oo
o`1:0?ooo`030?l0003oool0oooo01<0oooo00<000000?ooo`3oool04P3oool00`3o0000oooo0?oo
o`1;0?ooo`030?l0003oool0oooo0300oooo0`00000S0?ooo`00G03oool00`000000oooo0?ooo`0a
0?ooo`030?l0003oool0oooo04`0oooo00<0o`000?ooo`3oool04P3oool00`000000oooo0?ooo`0A
0?ooo`030?l0003oool0oooo04`0oooo00<0o`000?ooo`3oool0<P3oool00`000000oooo0?ooo`0Q
0?ooo`00G03oool00`000000oooo0?ooo`0a0?ooo`030?l0003oool0oooo04`0oooo00<0o`000?oo
o`3oool04P3oool00`000000oooo0?ooo`0A0?ooo`030?l0003oool0oooo04`0oooo00<0o`000?oo
o`3oool0<P3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`0a0?ooo`03
0?l0003oool0oooo04`0oooo00<0o`000?ooo`3oool04P3oool00`000000oooo0?ooo`0A0?ooo`03
0?l0003oool0oooo04`0oooo00<0o`000?ooo`3oool0<P3oool00`000000oooo0?ooo`0Q0?ooo`00
G03oool00`000000oooo0?ooo`0a0?ooo`030?l0003oool0oooo04d0oooo00<0o`000?ooo`3oool0
4@3oool00`000000oooo0?ooo`0A0?ooo`030?l0003oool0oooo04`0oooo00<0o`000?ooo`3oool0
<P3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`0a0?ooo`030?l0003o
ool0oooo04d0oooo00<0o`000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`030?l0003o
ool0oooo04h0oooo00<0o`000?ooo`3oool0<@3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool2
00000380oooo00<0o`000?ooo`3oool0C@3oool00`3o0000oooo0?ooo`0A0?ooo`030000003oool0
oooo0100oooo00<0o`000?ooo`3oool0CP3oool00`3o0000oooo0?ooo`0_0?ooo`<000008`3oool0
05`0oooo00<000000?ooo`3oool0<03oool00`3o0000oooo0?ooo`1?0?ooo`030?l0003oool0oooo
0100oooo00<000000?ooo`3oool0403oool00`3o0000oooo0?ooo`1>0?ooo`030?l0003oool0oooo
0340oooo00<000000?ooo`3oool08@3oool004<0oooo00@0oooO09a80000B9`0gooo0P3oool01P3o
_g@0000007Boo`3oool0oooO09a800<0000000<0M;oo0?ooo`3oool01`3oool00`000000oooo0?oo
o`0`0?ooo`030?l0003oool0oooo04l0oooo00<0o`000?ooo`3oool0403oool00`000000oooo0?oo
o`0@0?ooo`030?l0003oool0oooo04h0oooo00<0o`000?ooo`3oool0<@3oool00`000000oooo0?oo
o`0Q0?ooo`00@`3oool0103ooml0W4P00018W03Oool20?ooo`070?noM0000000M;oo0?ooo`3ooml0
W4Qd0;ooo`020?ooo`030;md001d_ol0oooo00L0oooo00<000000?ooo`3oool0<03oool00`3o0000
oooo0?ooo`1?0?ooo`030?l0003oool0oooo0100oooo00<000000?ooo`3oool03`3oool00`3o0000
oooo0?ooo`1?0?ooo`030?l0003oool0oooo0340oooo00<000000?ooo`3oool08@3oool004<0oooo
00@0oooO09a80000B9`0gooo2`3oool00`3ooml0W4Qd0;ooo`070?ooo`030000003oool0oooo0300
oooo00<0o`000?ooo`3oool0C`3oool00`3o0000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0
oooo00<0o`000?ooo`3oool0C`3oool00`3o0000oooo0?ooo`0a0?ooo`030000003oool0oooo0240
oooo000o0?ooo`030;md00000000000000D0000000<0M;oo0?ooo`3oool02@3oool00`2oM000M;oo
0?ooo`060?ooo`030000003oool0oooo0300oooo00<0o`000?ooo`3oool0D03oool00`3o0000oooo
0?ooo`0?0?ooo`030000003oool0oooo00l0oooo00<0o`000?ooo`3oool0D03oool00`3o0000oooo
0?ooo`0`0?ooo`030000003oool0oooo0240oooo000o0?ooo`080?oOW01dW=l0oooo0?ooo`3ooml0
W4P00018W03Oool;0?ooo`030?oog`2LB000B9cO00L0oooo00<000000?ooo`3oool0<03oool00`3o
0000oooo0?ooo`1@0?ooo`030?l0003oool0oooo00l0oooo00<000000?ooo`3oool03`3oool00`3o
0000oooo0?ooo`1@0?ooo`030?l0003oool0oooo0300oooo00<000000?ooo`3oool08@3oool00400
oooo00L0gia807Boo`3oool0oooO09a80000B9`0gooo00/0oooo00<0gia80000001d_ol01`3oool3
000002l0oooo00<0o`000?ooo`3oool0D@3oool00`3o0000oooo0?ooo`0?0?ooo`030000003oool0
oooo00h0oooo00<0o`000?ooo`3oool0D@3oool00`3o0000oooo0?ooo`0^0?ooo`<000008`3oool0
0400oooo00L0oooO09a8M02oool0oooO09a80000B9`0gooo00L0oooo00L0gmoO0?ooo`3ookl0M000
0000000007@0_ooo00L0oooo00<000000?ooo`3oool0;`3oool00`3o0000oooo0?ooo`1B0?ooo`03
0?l0003oool0oooo00h0oooo00<000000?ooo`3oool03P3oool00`3o0000oooo0?ooo`1A0?ooo`03
0?l0003oool0oooo0300oooo00<000000?ooo`3oool08@3oool00440oooo00H0omnL04Qd_`3ooml0
W4P00018W03Oool70?ooo`060=nLB000000000000000000004P0W=oo203oool00`000000oooo0?oo
o`0_0?ooo`030?l0003oool0oooo0580oooo00<0o`000?ooo`3oool03P3oool00`000000oooo0?oo
o`0>0?ooo`030?l0003oool0oooo0540oooo00<0o`000?ooo`3oool0<03oool00`000000oooo0?oo
o`0Q0?ooo`00@P3oool01@3OW4P0M;oO09a80000B9`0gooo00L0oooo00@0oono07@000000000B9cO
2P3oool00`000000oooo0?ooo`0_0?ooo`030?l0003oool0oooo0580oooo00<0o`000?ooo`3oool0
3P3oool00`000000oooo0?ooo`0=0?ooo`030?l0003oool0oooo05<0oooo00<0o`000?ooo`3oool0
;`3oool00`000000oooo0?ooo`0Q0?ooo`00@P3oool01@3ooml0W4Q804Q80000B9`0gooo00P0oooo
00<0_g@007Boo`3oool02P3oool00`000000oooo0?ooo`0_0?ooo`030?l0003oool0oooo05<0oooo
00<0o`000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030?l0003oool0oooo05<0oooo
00<0o`000?ooo`3oool0;`3oool00`000000oooo0?ooo`0Q0?ooo`00@`3oool0103ogi`0B0000018
W03Oool80?ooo`060?oOW018000000000000000004P0W=oo1`3oool00`000000oooo0?ooo`0^0?oo
o`030?l0003oool0oooo05@0oooo00<0o`000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?oo
o`030?l0003oool0oooo05<0oooo00<0o`000?ooo`3oool0;`3oool00`000000oooo0?ooo`0Q0?oo
o`00A03oool00`3OW4P004RL0=ooo`090?ooo`050;md00000000000000000018W=l01`3oool00`00
0000oooo0?ooo`0^0?ooo`030?l0003oool0oooo05@0oooo00<0o`000?ooo`3oool03@3oool00`00
0000oooo0?ooo`0<0?ooo`030?l0003oool0oooo05@0oooo00<0o`000?ooo`3oool0;`3oool00`00
0000oooo0?ooo`0Q0?ooo`00G03oool2000002l0oooo00<0o`000?ooo`3oool0E@3oool00`3o0000
oooo0?ooo`0<0?ooo`030000003oool0oooo00`0oooo00<0o`000?ooo`3oool0E03oool00`3o0000
oooo0?ooo`0]0?ooo`<000008`3oool005`0oooo00<000000?ooo`3oool0;P3oool00`3o0000oooo
0?ooo`1E0?ooo`030?l0003oool0oooo00`0oooo00<000000?ooo`3oool0303oool00`3o0000oooo
0?ooo`1E0?ooo`030?l0003oool0oooo02h0oooo00<000000?ooo`3oool08@3oool005`0oooo00<0
00000?ooo`3oool0;P3oool00`3o0000oooo0?ooo`1E0?ooo`030?l0003oool0oooo00`0oooo00<0
00000?ooo`3oool02`3oool00`3o0000oooo0?ooo`1F0?ooo`030?l0003oool0oooo02h0oooo00<0
00000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0;P3oool00`3o0000oooo0?ooo`1F
0?ooo`030?l0003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`3o0000oooo0?ooo`1F
0?ooo`030?l0003oool0oooo02h0oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?oo
o`3oool0;@3oool00`3o0000oooo0?ooo`1G0?ooo`030?l0003oool0oooo00/0oooo00<000000?oo
o`3oool02P3oool00`3o0000oooo0?ooo`1G0?ooo`030?l0003oool0oooo02h0oooo00<000000?oo
o`3oool08@3oool005`0oooo00<000000?ooo`3oool0;@3oool00`3o0000oooo0?ooo`1G0?ooo`03
0?l0003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`3o0000oooo0?ooo`1G0?ooo`03
0?l0003oool0oooo02h0oooo00<000000?ooo`3oool08@3oool005`0oooo0P00000^0?ooo`030?l0
003oool0oooo05P0oooo00<0o`000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030?l0
003oool0oooo05L0oooo00<0o`000?ooo`3oool0;03oool3000002<0oooo001L0?ooo`030000003o
ool0oooo02d0oooo00<0o`000?ooo`3oool0F03oool00`3o0000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00T0oooo00<0o`000?ooo`3oool0F@3oool00`3o0000oooo0?ooo`0]0?ooo`030000003o
ool0oooo0240oooo001L0?ooo`030000003oool0oooo02d0oooo00<0o`000?ooo`3oool0F03oool0
0`3o0000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<0o`000?ooo`3oool0F@3oool0
0`3o0000oooo0?ooo`0]0?ooo`030000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo
02d0oooo00<0o`000?ooo`3oool0F@3oool00`3o0000oooo0?ooo`090?ooo`030000003oool0oooo
00T0oooo00<0o`000?ooo`3oool0F@3oool00`3o0000oooo0?ooo`0]0?ooo`030000003oool0oooo
0240oooo001L0?ooo`030000003oool0oooo02`0oooo00<0o`000?ooo`3oool0FP3oool00`3o0000
oooo0?ooo`090?ooo`030000003oool0oooo00P0oooo00<0o`000?ooo`3oool0FP3oool00`3o0000
oooo0?ooo`0]0?ooo`030000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo02`0oooo
00<0o`000?ooo`3oool0FP3oool00`3o0000oooo0?ooo`090?ooo`030000003oool0oooo00P0oooo
00<0o`000?ooo`3oool0FP3oool00`3o0000oooo0?ooo`0]0?ooo`030000003oool0oooo0240oooo
001L0?ooo`030000003oool0oooo02`0oooo00<0o`000?ooo`3oool0F`3oool00`3o0000oooo0?oo
o`080?ooo`030000003oool0oooo00P0oooo00<0o`000?ooo`3oool0F`3oool00`3o0000oooo0?oo
o`0/0?ooo`030000003oool0oooo0240oooo001L0?ooo`800000;@3oool00`3o0000oooo0?ooo`1K
0?ooo`030?l0003oool0oooo00P0oooo00<000000?ooo`3oool01`3oool00`3o0000oooo0?ooo`1L
0?ooo`030?l0003oool0oooo02X0oooo0`00000S0?ooo`00G03oool00`000000oooo0?ooo`0/0?oo
o`030?l0003oool0oooo05`0oooo00<0o`000?ooo`3oool01`3oool00`000000oooo0?ooo`070?oo
o`030?l0003oool0oooo05`0oooo00<0o`000?ooo`3oool0;03oool00`000000oooo0?ooo`0Q0?oo
o`00G03oool00`000000oooo0?ooo`0/0?ooo`030?l0003oool0oooo05`0oooo00<0o`000?ooo`3o
ool01`3oool00`000000oooo0?ooo`070?ooo`030?l0003oool0oooo05`0oooo00<0o`000?ooo`3o
ool0;03oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`0[0?ooo`030?l0
003oool0oooo05h0oooo00<0o`000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030?l0
003oool0oooo05d0oooo00<0o`000?ooo`3oool0;03oool00`000000oooo0?ooo`0Q0?ooo`00G03o
ool00`000000oooo0?ooo`0[0?ooo`030?l0003oool0oooo05h0oooo00<0o`000?ooo`3oool01P3o
ool00`000000oooo0?ooo`060?ooo`030?l0003oool0oooo05h0oooo00<0o`000?ooo`3oool0:`3o
ool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`0[0?ooo`030?l0003oool0
oooo05h0oooo00<0o`000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030?l0003oool0
oooo05l0oooo00<0o`000?ooo`3oool0:`3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool20000
02`0oooo00<0o`000?ooo`3oool0G`3oool00`3o0000oooo0?ooo`050?ooo`030000003oool0oooo
00D0oooo00<0o`000?ooo`3oool0G`3oool00`3o0000oooo0?ooo`0Y0?ooo`<000008`3oool005`0
oooo00<000000?ooo`3oool0:`3oool00`3o0000oooo0?ooo`1O0?ooo`030?l0003oool0oooo00D0
oooo00<000000?ooo`3oool0103oool00`3o0000oooo0?ooo`1P0?ooo`030?l0003oool0oooo02/0
oooo00<000000?ooo`3oool08@3oool004`0oooo00<0oooO09a8000000000P0000000`1d_ol0oooo
0?ooo`080?ooo`030000003oool0oooo02/0oooo00<0o`000?ooo`3oool0H03oool00`3o0000oooo
0?ooo`040?ooo`030000003oool0oooo00@0oooo00<0o`000?ooo`3oool0H03oool00`3o0000oooo
0?ooo`0[0?ooo`030000003oool0oooo0240oooo001<0?ooo`030?oog`2LB7@0_ooo0080oooo00<0
_g@007Boo`3oool0203oool00`000000oooo0?ooo`0Z0?ooo`030?l0003oool0oooo0680oooo00<0
o`000?ooo`3oool00`3oool00`000000oooo0?ooo`030?ooo`030?l0003oool0oooo0640oooo00<0
o`000?ooo`3oool0:`3oool00`000000oooo0?ooo`0Q0?ooo`00D@3oool00`3ooml0W4Qd0;ooo`08
0?ooo`030000003oool0oooo02X0oooo00<0o`000?ooo`3oool0HP3oool00`3o0000oooo0?ooo`03
0?ooo`030000003oool0oooo0080oooo00<0o`000?ooo`3oool0H`3oool00`3o0000oooo0?ooo`0Z
0?ooo`030000003oool0oooo0240oooo001B0?ooo`030;md001d_ol0oooo00L0oooo00<000000?oo
o`3oool0:P3oool00`3o0000oooo0?ooo`1S0?ooo`030?l0003oool0oooo0080oooo00D000000?oo
o`3oool0oooo0?l0001V0?ooo`030?l0003oool0oooo02X0oooo00<000000?ooo`3oool08@3oool0
0540oooo00<0oooO09a80018W=l0203oool00`000000oooo0?ooo`0Z0?ooo`030?l0003oool0oooo
06@0oooo0P3o00020?ooo`030000003oool0oooo0080o`00IP3oool00`3o0000oooo0?ooo`0Z0?oo
o`030000003oool0oooo0240oooo001A0?ooo`030=nLB0000000M;oo00P0oooo0`00000Z0?ooo`03
0?l0003oool0oooo06H0oooo1@3o001X0?ooo`030?l0003oool0oooo02P0oooo0`00000S0?ooo`00
C@3oool01`3Ogml0oooo0?oo_`1d000000000000M02oool0203oool00`000000oooo0?ooo`0Z0?oo
o`030?l0003oool0oooo06P0oooo00<000000?ooo`3oool0J03oool00`3o0000oooo0?ooo`0Z0?oo
o`030000003oool0oooo0240oooo001=0?ooo`060=nLB000000000000000000004P0W=oo2@3oool0
0`000000oooo0?ooo`0Y0?ooo`030?l0003oool0oooo06T0oooo00<000000?ooo`3oool0J@3oool0
0`3o0000oooo0?ooo`0Y0?ooo`030000003oool0oooo0240oooo001=0?ooo`040?oo_`1d00000000
04RLg`/0oooo00<000000?ooo`3oool0:@3oool00`3o0000oooo0?ooo`1Y0?ooo`030000003oool0
oooo06T0oooo00<0o`000?ooo`3oool0:@3oool00`000000oooo0?ooo`0Q0?ooo`00CP3oool00`2o
M000M;oo0?ooo`0;0?ooo`030000003oool0oooo02T0oooo00<0o`000?ooo`3oool0J@3oool00`00
0000oooo0?ooo`1Y0?ooo`030?l0003oool0oooo02T0oooo00<000000?ooo`3oool08@3oool004h0
oooo00H0omnL04P00000000000000000B02Lgol80?ooo`030000003oool0oooo02T0oooo00<0o`00
0?ooo`3oool0J@3oool00`000000oooo0?ooo`1Y0?ooo`030?l0003oool0oooo02T0oooo00<00000
0?ooo`3oool08@3oool004l0oooo00D0_g@0000000000000000004RLg`080?ooo`030000003oool0
oooo02T0oooo00<0o`000?ooo`3oool0J@3oool00`000000oooo0?ooo`1Y0?ooo`030?l0003oool0
oooo02T0oooo00<000000?ooo`3oool08@3oool005`0oooo0P00000Z0?ooo`030?l0003oool0oooo
06T0oooo00<000000?ooo`3oool0J@3oool00`3o0000oooo0?ooo`0W0?ooo`<000008`3oool005`0
oooo00<000000?ooo`3oool0:03oool00`3o0000oooo0?ooo`1Z0?ooo`030000003oool0oooo06X0
oooo00<0o`000?ooo`3oool0:03oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo
0?ooo`0X0?ooo`030?l0003oool0oooo06X0oooo00<000000?ooo`3oool0JP3oool00`3o0000oooo
0?ooo`0X0?ooo`030000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo02P0oooo00<0
o`000?ooo`3oool0JP3oool00`000000oooo0?ooo`1Z0?ooo`030?l0003oool0oooo02P0oooo00<0
00000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0:03oool00`3o0000oooo0?ooo`1Z
0?ooo`030000003oool0oooo06X0oooo00<0o`000?ooo`3oool0:03oool00`000000oooo0?ooo`0Q
0?ooo`00G03oool00`000000oooo0?ooo`0X0?ooo`030?l0003oool0oooo06X0oooo00<000000?oo
o`3oool0JP3oool00`3o0000oooo0?ooo`0X0?ooo`030000003oool0oooo0240oooo001L0?ooo`80
0000:@3oool00`3o0000oooo0?ooo`1Z0?ooo`030000003oool0oooo06X0oooo00<0o`000?ooo`3o
ool09P3oool3000002<0oooo001L0?ooo`030000003oool0oooo02L0oooo00<0o`000?ooo`3oool0
J`3oool00`000000oooo0?ooo`1Z0?ooo`030?l0003oool0oooo02P0oooo00<000000?ooo`3oool0
8@3oool005`0oooo00<000000?ooo`3oool09`3oool00`3o0000oooo0?ooo`1[0?ooo`030000003o
ool0oooo06/0oooo00<0o`000?ooo`3oool09`3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool0
0`000000oooo0?ooo`0W0?ooo`030?l0003oool0oooo06/0oooo00<000000?ooo`3oool0J`3oool0
0`3o0000oooo0?ooo`0W0?ooo`030000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo
02L0oooo00<0o`000?ooo`3oool0J`3oool00`000000oooo0?ooo`1[0?ooo`030?l0003oool0oooo
02L0oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool09`3oool00`3o0000
oooo0?ooo`1[0?ooo`030000003oool0oooo06/0oooo00<0o`000?ooo`3oool09`3oool00`000000
oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`0W0?ooo`030?l0003oool0oooo06/0oooo
00<000000?ooo`3oool0J`3oool00`3o0000oooo0?ooo`0W0?ooo`030000003oool0oooo0240oooo
001L0?ooo`800000:03oool00`3o0000oooo0?ooo`1[0?ooo`030000003oool0oooo06/0oooo00<0
o`000?ooo`3oool09@3oool3000002<0oooo001L0?ooo`030000003oool0oooo02H0oooo00<0o`00
0?ooo`3oool0K03oool00`000000oooo0?ooo`1/0?ooo`030?l0003oool0oooo02H0oooo00<00000
0?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0O0?oo
o`030?l0003oool0oooo00/0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0]0?oo
o`030000003oool0oooo02d0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0<0?oo
o`030?l0003oool0oooo01h0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0Q0?oo
o`00G03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0080oooo00<0o`000?ooo`3o
ool00P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0080oooo00D0o`000?ooo`3o
ool0oooo000000090?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo0240oooo001L0?ooobT0000000<0o`0000000000
0000f`0000000`3o000000000000000W000002<0oooo003o0?oook40oooo003o0?oook40oooo003o
0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?ooobX0oooo00<0
oooO09a8000000000`0000000`18W=l0oooo0=nLB0020000000307Boo`3oool0oooo07T0oooo003o
0?ooobX0oooo00<0_g@00018W03Oool00`3oool00`3OW9`0_ooo0?ooo`020?ooo`030;mdM02oool0
oooo07T0oooo003o0?ooobX0oooo00<0oooO09adW03Oool00`3oool00`3ookl0W9cO0?ooo`020?oo
o`030?oOW01dW=l0oooo07T0oooo003o0?ooob/0oooo00<0_g@00000000000000`0000001`18W=l0
oooo0?ooo`3oool0oooO09a8M02oool0N@3oool007D0oooo00<0_gA809cOo`3oool00P3oool00`3o
oml0W7BL0=ooo`040?ooo`080?oog`2LB000000000000000B9`0W7@000000000M;l30?ooo`030?oo
g`2LB7@0_ooo00D0oooo00D0_g@0000000000000001d0;ooo`050?ooo`030?oog`2LB00000000080
000000<0001809cOo`3oool0103oool02@3ookl0M000000000000000B9bo09a80000B9`0gooo0;md
0002000000030000M02ooml0W4P00080000000@004RL0=ooo`3oool0_g@00P0000002@0004P0W=oo
0?ooo`3oool0omnL04P000000000B9cO0=nLB002000000040000M02oool0oooo0=nLB0<0000000@0
M;oo0?ooo`3oool0_g@00P0000000`0007@0_ooO09a80002000000030018W03Oool0oooo00H0oooo
00<0omnL07BLg`3oool00P3oool00`3OW7@0W=oo0?ooo`020?ooo`0:0=nLB000000000000000M02o
ool0omnL04P000000000B9cO0=nLB080000000<0001809cOg`2LB0000P0000000`00M;l0oooo0?oo
o`020?ooo`030?oog`2LB000000000L0000000<007Bo0?ooo`2oM0000P0000000`0007@0_ooO09a8
0002000000040018W03Oool0oooo0;md0080000000@0001d0;ooo`3oool0_g@00P0000001@0004P0
W=oo0?ooo`3ookl0M00000@0000000<0001d0;ooo`3oool00`3oool00`2oM000M;oo0?ooo`0G0?oo
o`030;md00000000000000@0000000H0B9cO0?ooo`3oool0okmd0000001d_ol30?ooo`050=nLB000
0000000000000018W=l0103oool01@3OW4P00000000000000000B9cO00D0oooo00<0gia804RLg`3o
ool0>`3oool007@0oooo00<0oono07@00018W=l00`3oool00`3o_g@0001809cOo`040?ooo`090?no
M0000000M;oo0?ooo`3ogi`0B000001d_`3o_kl0_ooo0080oooo00<0okmd00000018W=l0103oool0
1`2oM0000000001d_`3oool0omnL04P0M02oool01P3oool00`3ogi`0B00004RLg`070?ooo`050=nL
B00007@0_ooo0?ooo`3OW4P00P0000002@0007@0_ooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d
_`020?ooo`0H0;md0000000004RL0=ooo`3ooml0W4Qd0;ooo`3oool0gia80000M02oool0gkno0=oo
o`3OW4P0001d0;ooo`3oool0gia800000018W=l0oooo0?oog`2LB000B9cO0P3oool01`3OW4P004RL
0=ooo`3oool0oooO09a80000M;l02@3oool01`2oM000001d0;ooo`3oool0oono07@00000M;l0103o
ool0403OW4P0001d0;ooo`3oool0gia80000M02oool0gkno0=ooo`3OW4P0001d0;ooo`3oool0oooO
09a80000M;l70?ooo`030?oOW0180000B9cO00@0oooo00/0omnL04P0M02oool0oooo0=nLB000B9`0
gooo0?ooo`3ooml0W4P0001d_`020?ooo`0;0;md0000000007Bo0?ooo`3ogi`0B01d0;ooo`3oool0
gia80000M02oool00`3oool00`2oM4P0W=oo0?ooo`070?ooo`040?oOW018000004RL0=oooaP0oooo
00<0gia80000000000000`0000001P00B9`0gooo0?ooo`3o_g@0000007Boo`80oooo00L0gia80000
001d_ol0oooo0?oog`2LB000B9cO0080oooo00L0gia80000001d_ol0oooo0?oog`2LB000B9cO00@0
oooo00<0omnL04Q8W03Oool0>`3oool007@0oooo00@0okmd00000000B9`0gooo0P3oool00`2oM000
0000001d_`040?ooo`070?noM00004P0W=oo0?ooo`3ooml0W4P0001d_`030?ooo`050?oog`2LB000
04RL09adM02oool00P3oool00`3ogi`0B00004RLg`030?ooo`030?oOW01dW=l0oooo00D0oooo00<0
oono07@00018W=l01`3oool01`2oM00004RL0=ooo`3oool0oooO09a80000M;l00P3oool02`3OW4P0
04RL0=ooo`3oool0oooO09a80000M;l0oooo0?oOW018000007Bo00<0oooo00H0omnL09boo`3oool0
gia80018W03Oool20?ooo`060=nLB00007@0_ooo0?oOW018000007Bo0`3oool02`3o_g@0001d0;oo
o`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo00P0oooo00T0omnL04P00018M7@0M9cO0?oo
o`3o_g@0001804Q8B02Lgol00`3oool01`3OW4P0001d0;ooo`3oool0gia80018W03Oool00P3oool0
1`3OW4P0001d0;ooo`3oool0oooO09a80000M;l01`3oool00`3ookl0M00004RLg`050?ooo`0>0?no
M01d_ol0oooo0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0omnL04P00018W=l30?ooo`06
0?oOW01dW=l0oooo0=nLB00007@0_ooo0`3oool01@3ookl0M00000000000000007Bo00D0oooo00@0
_g@00018W02LM000M;oo1`3oool00`3ooml0W4P0000000070000000304RLg`3oool0oooo00D0oooo
00<0gia804RLg`3oool00P3oool00`3OW7@0W=oo0?ooo`040?ooo`030?oog`2LB00007Bo00<0oooo
00H0okmd0018W03Oool0oooO09a80000M;l30?ooo`030?noM000B9`0gooo00<0oooo00<0oooO09a8
B02Lgol0>`3oool007<0oooo00/0oooO09a8000007@0W9ad09cOo`3oool0oono07@00000M9`0W4Qd
0;ooo`040?ooo`060?noM000000007Bo0?oog`2LB00007Bo0`3oool01@3ogi`0B00007Boo`3o_g@0
B9cO0080oooo00<0okmd0000B02Lgol02`3oool00`3ookl0M00004RLg`070?ooo`070;md0000B9`0
gooo0?ooo`3ooml0W4P0001d_`020?ooo`0;0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0
okmd0000B02Lgol01P3oool00`3OW4P004RL0=ooo`020?ooo`060=nLB00007@0_ooo0?noM0000000
M;oo0`3oool02`3o_g@0000007Boo`3oool0gia80018W03Oool0oooo0?oog`2LB00007Bo00P0oooo
00T0gia80000M02oool0_gAd0;oog`2LB00007Bo0?oOW01dW=l00`3oool01`3OW4P0001d0;ooo`3o
ool0gia80018W03Oool00P3oool01`3OW4P0001d0;ooo`3oool0oooO09a80000M;l01`3oool00`3o
okl0M00004RLg`080?ooo`0;0=nLB000B9`0gooo0?ooo`3ooml0W4P0001d_`3oool0okmd0000B02L
gol01P3oool00`3OW4P0001d0;ooo`030?ooo`060=nLB00007@0_ooo0?oo_`1d0000B9cO0`3oool0
1@3o_g@0000007Boo`3ogi`0B7Bo01X0oooo00<0gia807Boo`3oool02@3oool00`2oM00007Bo0?oo
o`020?ooo`060;md0000B9`0gooo0?ooo`2oM00007Bo0`3oool00`2oM00004RL0=ooo`040?ooo`03
0=nLB018W=l0oooo03X0oooo001c0?ooo`0;0?oOW0180000B9cO0?oOW018M;l0oooo0=nLB0000000
M;oo0?noM01d_ol01P3oool0103o_g@0000000000000M;l30?ooo`0:0;md0000B9`0gooo0?oog`2L
B7@0_ooo0?ooo`3o_g@0001d0;ooo`/0oooo00<0oono07@00018W=l00P3oool00`3ogi`0M9cO0?oo
o`020?ooo`070;md0000B9`0gooo0?ooo`3ooml0W4P0001d_`020?ooo`0;0=nLB000B9`0gooo0?oo
o`3ooml0W4P0001d_`3oool0okmd0000M02oool01P3oool00`3OW4P004RL0=ooo`020?ooo`060=nL
B00007@0_ooo0?noM0000000M;oo0`3oool02`3o_g@0000007Boo`3oool0gia80018W03Oool0oooo
0?oog`2LB00007Bo00L0oooo00/0oono07@00018W=l0oooo0?oOW01dM7@0B01809cOo`3oool0_gAd
0;ooo`020?ooo`070=nLB00007@0_ooo0?ooo`3OW4P004RL0=ooo`020?ooo`070=nLB00007@0_ooo
0?ooo`3ooml0W4P0001d_`070?ooo`030?oo_`1d0000B9cO00<0oooo00<0okmd07Boo`3oool00P3o
ool02`3OW4P004RL0=ooo`3oool0oooO09a80000M;l0oooo0?noM00007@0_ooo00H0oooo00<0gia8
0000M02oool00P3oool01`3ookl0M00004RLg`3oool0oooO09a80000M;l00P3oool01`3ooml0W4P0
0018W03Oool0oooo0;mdB02Lgol06P3oool00`2oM4P0W=oo0?ooo`080?ooo`070?oOW01807@0_ooo
0?ooo`2oM000000007Boo`020?ooo`070?oOW01807@0_ooo0?ooo`2oM000000007Boo`050?ooo`03
0?oOW018B9`0gooo03X0oooo001c0?ooo`0;0=nLB00004P0W=oo0?oog`2LB7@0_ooO09a8000007@0
_ooo0?oo_`1dM;l00`3oool01`3ookl0M00007Boo`3oool0oooO09a80000M;l00P3oool00`3ogi`0
B00007Boo`020?ooo`050?noM018W=l0oooo0?oOW01800001@0000000`18W=l0oooo0?ooo`040?oo
o`030?oo_`1d0000B9cO0080oooo00<0okmd04RLg`3oool00P3oool01`2oM00004RL0=ooo`3oool0
oooO09a80000M;l00P3oool02`2oM000001d0;ooo`3oool0oooO09a80000M;l0oooo0?oOW01804P0
W=oo0080oooo00L0okmd0000B02Lgol0oooo0=nLB000B9`0gooo0080oooo00H0gia80000M02oool0
oono07@0001d_ol30?ooo`0;0;md000004P0W=oo0?ooo`2oM000001d0;ooo`3oool0oooO09a80000
M;l01`3oool01`3OW4P0001809cOo`3oool0oooO09a80000M;l00P3oool02`3o_g@0B9cO0?ooo`3o
ool0gia80000M02oool0oooo0=nLB000B9`0gooo0080oooo00L0gia80000M02oool0oooo0?oog`2L
B00007Bo00L0oooo00<0oono07@00018W=l00`3oool00`2oM000M;oo0?ooo`020?ooo`0:0;md0000
07@0_ooo0?ooo`3ooml0W4P0001d_`3oool0omnL04P000D0000000D0B9cO0?ooo`3OW4P0001809cO
o`020?ooo`070?oo_`1d0000B9cO0?ooo`3ooml0W4P004RLg`020?ooo`030?noM00004P0W=oo0080
oooo00<0okmd04RLg`3oool06@3oool00`3ooml0W4Qd0;ooo`090?ooo`050?oo_`1dB4P0B0000000
0018W=l0103oool01@3ookl0M4Q804P000000000B9cO00H0oooo00<0oooO09a8B02Lgol0>P3oool0
0780oooo00T0oono07@00000B9`0gooo0?ooo`3o_g@0M9bL04P00018W=l00P3oool0402oM4P0W=oo
0?ooo`3oool0oono07@00018W=l0oooo0?oo_`1d000007Bo0?ooo`3ooml0W4P00000M02oool20?oo
o`060?oog`2LB7@0_ooo0?ooo`2oM000B9cO0P3oool00`3OW4P0001809cOo`060?ooo`030?oo_`1d
0000000000<0000001@0B9cO0?ooo`3oool0oooo0;md0000B9`0gooo0?ooo`3ooml0W4P0001d_`3o
oml0W4P000000000B4P0B01809cOo`3ookl0M00004RLg`80oooo0240_g@00018W03Oool0oooO09a8
000007@0_ooo0?ooo`3OW4P004RL0=ooo`3ooml0W4P00000000007@0_ooo0?ooo`3OW4P004RL0=oo
o`3oool0okmd00000000M;l0oooO09a80000000004Q804P0B02Lgol0oono07@00018W=l01P3oool0
0`3ookl0M000001d_`020?ooo`030?oOW0180000M;oo0080oooo00/0oooO09a8M02ooml0W4P00000
000007@0_ooo0?ooo`3OW4P004RL0=ooo`020?ooo`070=nLB0000000001809cOo`3ookl0M00004RL
g`070?ooo`030?oo_`1d0000000000@0000000/0M;oo0?ooo`3ooml0W4P000000000B4P0B01809cO
o`3ookl0M00004RLg`020?ooo`0B0;md0018W=l0oooo0?ooo`3OW4P0001809cOg`2LB00000000000
B018M9`0gkmd0000M02oool0_g@0001d_`3oool0okmd0P0000001@00B9`0gono07@00000B9`0gooo
0080oooo00<0oono07@0M02oool01@3oool00`3ooml0W4P0000000070000000304RLg`3oool0oooo
00P0oooo00<0okmd001d_`3oool0203oool0103ogi`0B00000000000M;l50?ooo`040?oOW0180000
0000001d_`P0oooo00<0gia8001d_`3oool0>@3oool00780oooo00<0okmd00000018W=l00P3oool0
103ookl0M0000000B02Lgol20?ooo`030?noM018W=l0oooo0080oooo00P0oono07@0000000000000
0000M02oool0oooO09a80080000000T0001d0;ooo`3ooml0W4P000000000B9`0gooo0?ooo`3OW4P0
0P0000000`0004P0W=oo0?ooo`060?ooo`030?oo_`1d0000B9cO0080oooo00`0okmd04RLg`3oool0
oono07@00000000004RL0=ooo`3ogi`0B00000000000M;l20?ooo`070;md0000B9`0gooO09a80000
0000001d0;ooo`030?ooo`070=nLB000000000000000B02Lgol0oooo0;md0003000000050018W03O
ool0_g@00000M02oool00P3oool01@3o_g@00000000000000000B9cO00<0oooo00L0_g@00018W03O
oml0W4P00000000007@0_ooo00D0oooo00H0oooO09a800000000000004RLg`3o_g@30000000:04RL
g`3oool0_g@000000000B9`0gooo0;md000007@0_ooo0;md00<0000000T004RL0=ooo`3OW4P0001d
0;oog`2LB00000000000M02oool01`3oool00`3ookl0M00004RLg`030?ooo`030;md001d_ol0oooo
0080oooo00L0_g@00018W03Ooml0W4P00000000007@0_ooo00<0oooo00D0gia80000000000000018
09cOo`020?ooo`080;md000007@0_mnL04P0000007@0_ooo0?oog`2LB0020000000407Boo`3oool0
oooO09a80080000000L004RL0=ooo`3oool0_g@000000000B9`0gooo01T0oooo00<0oono07@0B02L
gol01`3oool01P3ogi`0B0000000M02oool0okmd001d_`<0oooo00H0omnL04P0000007@0_ooo0?no
M000M;l70?ooo`030?oo_`1d07@0_ooo03T0oooo001b0?ooo`030;md000007@0_ooo0080oooo00@0
oono07@00000B9`0gooo0P3oool00`3ookl0M4RL0=ooo`0P0?ooo`030?oo_`1d0000B9cO0080oooo
00<0omnL07BLg`3oool0703oool00`2oM00004RL0=ooo`0^0?ooo`030;md0000B9`0gooo0080oooo
00<0gia80000M02oool02`3oool00`3ookl0M00004RLg`030?ooo`030?noM01d_ol0oooo04<0oooo
00<0oono07@00018W=l01`3oool00`2oM000001d0;ooo`020?ooo`070;md0018W=l0oooo0?ooo`2o
M000001d0;ooo`020?ooo`050;md0018W=l0oooo0?oOg`3Oool0103oool00`2oM000M;oo0?ooo`0h
0?ooo`00L@3oool00`3ogi`0B000001d_`030?ooo`030?noM0000000B9cO00@0oooo00<0giad09cO
o`3oool07`3oool00`3ookl0M00004RLg`040?ooo`030=nLM02Lgol0oooo01X0oooo00<0oono07A8
W03Oool0;P3oool00`3ookl0M4RL0=ooo`020?ooo`030=nLB00007@0_ooo00/0oooo00<0oono07@0
0018W=l0103oool00`3ookl0M7Bo0?ooo`0m0?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P0
0018W=l01`3oool00`2oM00004RL0=ooo`020?ooo`070=nLB000M;l0oooo0?ooo`2oM00004RL0=oo
o`020?ooo`060=nLB000M;l0oooo0?oo_`1dB9`0gooo0`3oool00`3o_g@007Bo0?ooo`0h0?ooo`00
L@3oool00`3OW4P0000004RLg`020?ooo`040?oog`2LB000000007Boo`@0oooo00<0gia8001d_`3o
ool07`3oool00`3ogi`0B00004RLg`030?ooo`030?oo_`1d07@0_ooo01/0oooo00<0oooO09adW03O
ool0;P3oool00`3ooml0W7BL0=ooo`020?ooo`030=nLB00007@0_ooo00/0oooo00<0omnL04P00018
W=l0103oool00`3OW4P0B9cO0?ooo`0m0?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02o
ool70?ooo`070?noM0000000M;oo0?ooo`3ookl0M00007Boo`020?ooo`070?noM0000000M;oo0?oo
o`3ookl0M00007Boo`020?ooo`030;md00000000000000<0000000<0001d0;ooo`3oool0=`3oool0
06l0oooo00<0oooO09a8000000000P0000000`00M;l0omnL04P000030000000404RLg`3oool0oooo
0;md0080000000<004RL0=ooo`3oool06`3oool00`3ooml0W4P000000006000000030000M02oool0
oooo01l0oooo00<0gia80000B02Lgol09@3oool00`3OW4P0001809cOo`040?ooo`050?oog`2LB000
00000000M02oool02@3oool00`3ooml0W4P0000000070000000304RLg`3oool0oooo03d0oooo00H0
oooO09a80000000000000000M02oool90?ooo`050?noM0000000000000000018W=l0103oool01@3o
_g@00000000000000000B9cO00<0oooo00<0okmd000000000000100000000`1d_ol0oooo0?ooo`0f
0?ooo`00b@3oool00`3OW4P0001809cOo`0U0?ooo`030=nLB00004P0W=oo00H0oooo00<0_g@00000
M02oool0/`3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool0
0001\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {-5.06654, 1.77857, \
0.0311177, 0.0225258}}]
}, Closed]],

Cell["\<\
The splitting between the even and odd solutions is extremely small and \
becomes even smaller as the mass increases.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\((energy /. sol1)\)\  - \((energy /. sol)\)\)], "Input"],

Cell[BoxData[
    \(5.4935503411655947537294430799550239639`20.978351292548886*^-11\)], \
"Output"]
}, Closed]],

Cell["\<\
Clearly, it becomes rather difficult with this method to obtain an accurate \
value for the ground-state splitting as the mass becomes large.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "ExerciseMain"],

Cell[CellGroupData[{

Cell["Central potentials", "ExerciseTitle"],

Cell["\<\
Recall that solutions to the three-dimensional Schr\[ODoubleDot]dinger \
equation\
\>", "ExerciseText"],

Cell[BoxData[
    \(TraditionalForm\`\(-\(\[Del]\^2 \[Psi][\(r\& \[RightVector] \)]\/\(2\ m\
\)\)\)\  + \ V[r]\ \[Psi][\(r\& \[RightVector] \)] \[Equal] 
      e\ \[Psi][\(r\& \[RightVector] \)]\)], "DisplayFormula"],

Cell["with a central potential take the form", "ExerciseText"],

Cell[BoxData[
    \(TraditionalForm\`\[Psi][r\&\[RightVector]] \[LongEqual] \ 
      R\_\(n, \[ScriptL]\)[r] Y\_\(\[ScriptL], m\)[r\&^]\)], "DisplayFormula"],

Cell["where the radial equation is", "ExerciseText"],

Cell[BoxData[
    \(TraditionalForm\`R\_\(n, \[ScriptL]\)\%\[DoublePrime][r] + \(2\/r\) 
          R\_\(n, \[ScriptL]\)\%\[Prime][
            r]\  - \(\(\[ScriptL]\ \((\[ScriptL] + 
                    1)\)\)\/\(\(\ \)\(r\^2\)\)\) 
          R\_\(n, \[ScriptL]\)[
            r] + \(2  m\)\/\[HBar]\^2\ \((E\_\(n, \[ScriptL]\) - 
              V[r])\)\ R\_\(n, \[ScriptL]\)[r] \[Equal] 0\)], "DisplayFormula"],

Cell[TextData[{
  "Provided that ",
  Cell[BoxData[
      \(TraditionalForm\`\(r\^2\) V[r] \[Rule] 0\)]],
  " when ",
  Cell[BoxData[
      \(TraditionalForm\`r \[Rule] 0\)]],
  ", the radial solutions satisfy"
}], "ExerciseText"],

Cell[BoxData[
    \(TraditionalForm\`r \[Rule] 
      0\ \ \[DoubleLongRightArrow]\ \ R\_\(n, \[ScriptL]\)\  \[Proportional] \
\ r\^\[ScriptL]\)], "DisplayFormula"],

Cell["\<\
near the origin.  Thus, we can initiate the outward integration using\
\>", "ExerciseText"],

Cell[BoxData[{
    \(TraditionalForm\`\[ScriptL] = \(0\ \ \[DoubleLongRightArrow]\ \ R\_\(n, \
\[ScriptL]\)[r\_0] = 1\), \ \ R\_\(n, \[ScriptL]\)\%\[Prime][
        0] = \(-\[CurlyEpsilon]\)\), "\[IndentingNewLine]", 
    \(TraditionalForm\`\[ScriptL] > 
        0\ \ \[DoubleLongRightArrow]\ \ R\_\(n, \[ScriptL]\)[r\_0] = 
      0, \ \ R\_\(n, \[ScriptL]\)\%\[Prime][
        0] = \(+\[CurlyEpsilon]\)\)}], "DisplayFormula"],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`r\_0\)]],
  " is a small radius and ",
  Cell[BoxData[
      \(TraditionalForm\`\[CurlyEpsilon]\)]],
  " is a small positive number.  Similarly, we can initiate the inward \
integration using ",
  Cell[BoxData[
      \(TraditionalForm\`R\_\(n, \[ScriptL]\)[r\_max] = 0\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`R\_\(n, \[ScriptL]\)\%\[Prime][
          r\_max] = \(-\[CurlyEpsilon]\)\)]],
  " with relatively little error if ",
  Cell[BoxData[
      \(TraditionalForm\`r\_max\)]],
  " is large enough.  Experience shows that one should choose ",
  Cell[BoxData[
      \(TraditionalForm\`r\_match\)]],
  " near the classical turning point.  Finally, note that the normalization \
condition now takes the form"
}], "ExerciseText"],

Cell[BoxData[
    \(TraditionalForm\`\[Integral]\_0\%\[Infinity]\( R\_\(n, \[ScriptL]\)[
                r]\^2\) \(r\^2\) \[DifferentialD]r\  = \ 
      1\)], "DisplayFormula"],

Cell[TextData[{
  "a) Modify ",
  StyleBox["ShootBSWF2", "InlineInput"],
  " to determine eigenfunctions for a central potential.  Take care with the \
initial conditions for the interior solution when ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptL] = 0\)]],
  "."
}], "ExerciseText"],

Cell[TextData[{
  "b) Test case: determine the lowest couple of energies and wave functions \
for a harmonic oscillator with ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptL] = 0, 1, 2\)]],
  " and compare with exact solutions.  Recall that"
}], "ExerciseText"],

Cell[BoxData[
    \(TraditionalForm\`\[Psi]\_\(n, \[ScriptL]\)[
        r]\  = \ \(\@\(\(2  \[CapitalGamma][
                    n - \[ScriptL] + 1]\)\/\[CapitalGamma][n + 3\/2]\)\) 
        Exp[\(-m\)\ \[Omega]\ r\^2/2] \(r\^\[ScriptL]\) 
        L\_\(n - \[ScriptL]\)\%\(\((2  \[ScriptL] + 1)\)/2\)[
          m\ \[Omega]\ r\^2]\ , \ \ \ \ E\_\(n, \[ScriptL]\) = \((2  
              n - \[ScriptL] + 3\/2)\)\ \[HBar]\ \[Omega]\)], "DisplayFormula"],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`\[Omega] = \@\(k/m\)\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`L\_n\%a[x]\)]],
  " is a generalized Laguerre polynomial of degree ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  ".  We require ",
  Cell[BoxData[
      \(TraditionalForm\`n \[GreaterEqual] \[ScriptL] \[GreaterEqual] 0\)]],
  " where the number of radial nodes, excluding the origin, is ",
  Cell[BoxData[
      \(TraditionalForm\`n - \[ScriptL]\)]],
  "."
}], "ExerciseText"],

Cell["\<\
c) Test case: determine the lowest few solutions for the hydrogen atom.  \
Recall that nonrelativistic wave functions for a one-electron atom take the \
form \
\>", "ExerciseText"],

Cell[BoxData[
    \(TraditionalForm\`\[Psi]\_\(n, \[ScriptL]\)[
        r]\  = \ \(\(2  \[Kappa]\^\(3/2\)\)\/n\^2\) \(\@\(\(\((n - \[ScriptL] \
+ 1)\)!\)\/\(\((n + \[ScriptL])\)!\)\)\) 
        Exp[\(-\(\(\[Kappa]\ r\)\/n\)\)] \(\((\(2  \[Kappa]\ r\)\/n)\)\^\
\[ScriptL]\) 
        L\_\(n - \[ScriptL] - 1\)\%\(2  \[ScriptL] + 1\)[\(2  \[Kappa]\ \
r\)\/n]\ , \ \ \ \ E\_\(n, \[ScriptL]\) = \(-\(E\_0\/n\^2\)\)\)], \
"DisplayFormula"],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`\[Kappa] = Z\ \[Mu]\ e\^2/\[HBar]\^2\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`E\_0 = \(\[HBar]\^2\) \[Kappa]\^2/2  \[Mu]\)]],
  " are expressed in terms of the atomic number ",
  Cell[BoxData[
      \(TraditionalForm\`Z\)]],
  ", elementary charge ",
  Cell[BoxData[
      \(TraditionalForm\`e\)]],
  " in electrostatic (cgs) units, and the reduced mass of the \
electron-nucleus system.  The number of radial nodes, excluding the origin, \
is ",
  Cell[BoxData[
      \(TraditionalForm\`n - \[ScriptL] - 1\)]],
  "."
}], "ExerciseText"]
}, Closed]],

Cell[CellGroupData[{

Cell["Nuclear shell model", "ExerciseTitle"],

Cell["\<\
The nuclear shell model stipulates that protons and neutrons move within a \
central potential with spin-orbit coupling.  This potential is often \
parametrized in the Woods-Saxon form\
\>", "ExerciseText"],

Cell[BoxData[
    \(TraditionalForm\`V[r]\  = \ 
      V\_C\/\(Exp[\((r - r\_C)\)/a\_C] + 1\)\ \  + \ \(\[HBar]\^2\/\(\(m\_\
\[Pi]\^2\) 
                c\^2\)\) \(1\/r\) \(\[PartialD]\/\[PartialD]r\) \
\(V\_LS\/\(Exp[\((r - r\_LS)\)/a\_LS] + 
                1\)\) \(L\& \[RightVector] \)\[CenterDot]\(\[Sigma]\& \
\[RightVector] \)\)], "DisplayFormula"],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`L\&\[RightVector]\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\&\[RightVector]\)]],
  " are orbital and spin angular momentum operators, ",
  Cell[BoxData[
      \(TraditionalForm\`V\_C\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`V\_LS\)]],
  " are strength parameters,  ",
  Cell[BoxData[
      \(TraditionalForm\`r\_C\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`r\_LS\)]],
  " are size parameters, and ",
  Cell[BoxData[
      \(TraditionalForm\`a\_C\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`a\_LS\)]],
  " are surface diffuseness parameters.  The pion Compton wave length, with \
",
  Cell[BoxData[
      \(TraditionalForm\`\(m\_\[Pi]\) c\^2 = 139\)]],
  " MeV, is used to set the scale of the spin-orbit term.  Typical parameters \
for the lead nucleus are ",
  Cell[BoxData[
      \(TraditionalForm\`V\_C = \(-44\)\)]],
  " MeV, ",
  Cell[BoxData[
      \(TraditionalForm\`V\_LS = 6\)]],
  " MeV, ",
  Cell[BoxData[
      \(TraditionalForm\`r\_C = \(r\_LS = 7.52\)\)]],
  " fm, and ",
  Cell[BoxData[
      \(TraditionalForm\`a\_C = \(a\_LS = 0.67\)\)]],
  ".  The radial wave functions ",
  Cell[BoxData[
      \(TraditionalForm\`R\_\(n, \[ScriptL], j\)[r]\)]],
  " are labeled by three quantum numbers where ",
  Cell[BoxData[
      \(TraditionalForm\`n - 1\)]],
  " is the number of radial nodes (excluding the origin), ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptL]\)]],
  " is the orbital angular momentum, and ",
  Cell[BoxData[
      \(TraditionalForm\`j\)]],
  " is the total angular momentum.  The spin-orbit coupling is given by"
}], "ExerciseText"],

Cell[BoxData[
    FormBox[
      RowBox[{\(\[LeftAngleBracket]\(L\& \[RightVector] \)\[CenterDot]\(\
\[Sigma]\& \[RightVector] \)\[RightAngleBracket]\), " ", "=", " ", 
        TagBox[
          StyleBox[
            RowBox[{"{", GridBox[{
                  {"\[ScriptL]", \(j = \[ScriptL] + 1\/2\)},
                  {\(\(-\[ScriptL]\) - 1\), \(j = \[ScriptL] - 1\/2\)}
                  }]}],
            ShowAutoStyles->False],
          (#&)]}], TraditionalForm]], "DisplayFormula"],

Cell[TextData[{
  "Determine the energies for all bound states of neutrons and construct an \
energy level diagram similar to that shown below.  The first column shows all \
energy levels while the columns to the right show spectra for specific \
angular momenta, with ",
  Cell[BoxData[
      \(TraditionalForm\`j = \[ScriptL] + 1\/2\)]],
  " in blue and ",
  Cell[BoxData[
      \(TraditionalForm\`j = \[ScriptL] - 1\/2\)]],
  " in green.  Assuming that levels are filled in the order prescribed by the \
shell model and neglecting residual interactions between nucleons, verify \
that the configuration for ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[InvisiblePrefixScriptBase]\^208\)Pb\)]],
  ", with 82 protons and 126 neutrons, has all  levels filled through ",
  Cell[BoxData[
      \(TraditionalForm\`3  p\_\(1/2\)\)]],
  " for neutrons and ",
  Cell[BoxData[
      \(TraditionalForm\`1  g\_\(11/2\)\)]],
  " for protons.  (Note: the Coulomb potential reduces the binding energy for \
protons but has little effect upon the ordering of levels.)   This nucleus is \
described as \"doubly magic\" and is especially stable because both the \
proton and neutron numbers correspond to filled shells immediately below \
relatively large gaps in the single-particle energy spectra.  The spin-orbit \
coupling is crucial to explaining the level ordering and the sequence of \
magic numbers: 2, 8, 20, 28, 50, 82, 126."
}], "ExerciseText"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.10582 0.575246 0.0140365 [
[.01131 .01378 -24.3125 -7.84375 ]
[.01131 .01378 0 7.84375 ]
[.01131 .15415 -24.3125 -7.84375 ]
[.01131 .15415 0 7.84375 ]
[.01131 .29452 -24.3125 -7.84375 ]
[.01131 .29452 0 7.84375 ]
[.01131 .43488 -24.3125 -7.84375 ]
[.01131 .43488 0 7.84375 ]
[.01131 .57525 -10 -7.84375 ]
[.01131 .57525 0 7.84375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .01378 m
.03006 .01378 L
s
gsave
.01131 .01378 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
.02381 .15415 m
.03006 .15415 L
s
gsave
.01131 .15415 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(30) show
1.000 setlinewidth
grestore
.02381 .29452 m
.03006 .29452 L
s
gsave
.01131 .29452 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
.02381 .43488 m
.03006 .43488 L
s
gsave
.01131 .43488 -85.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
1.000 setlinewidth
grestore
.02381 .57525 m
.03006 .57525 L
s
gsave
.01131 .57525 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.125 Mabswid
.02381 .04186 m
.02756 .04186 L
s
.02381 .06993 m
.02756 .06993 L
s
.02381 .098 m
.02756 .098 L
s
.02381 .12608 m
.02756 .12608 L
s
.02381 .18222 m
.02756 .18222 L
s
.02381 .2103 m
.02756 .2103 L
s
.02381 .23837 m
.02756 .23837 L
s
.02381 .26644 m
.02756 .26644 L
s
.02381 .32259 m
.02756 .32259 L
s
.02381 .35066 m
.02756 .35066 L
s
.02381 .37873 m
.02756 .37873 L
s
.02381 .40681 m
.02756 .40681 L
s
.02381 .46295 m
.02756 .46295 L
s
.02381 .49103 m
.02756 .49103 L
s
.02381 .5191 m
.02756 .5191 L
s
.02381 .54717 m
.02756 .54717 L
s
.02381 .60332 m
.02756 .60332 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.11905 .01472 L
s
.02381 .16198 m
.11905 .16198 L
s
.02381 .36283 m
.11905 .36283 L
s
.02381 .55791 m
.11905 .55791 L
s
.02381 .06906 m
.11905 .06906 L
s
.02381 .24888 m
.11905 .24888 L
s
.02381 .46183 m
.11905 .46183 L
s
.02381 .13345 m
.11905 .13345 L
s
.02381 .34008 m
.11905 .34008 L
s
.02381 .55107 m
.11905 .55107 L
s
.02381 .20633 m
.11905 .20633 L
s
.02381 .43379 m
.11905 .43379 L
s
.02381 .28654 m
.11905 .28654 L
s
.02381 .52743 m
.11905 .52743 L
s
.02381 .37307 m
.11905 .37307 L
s
.02381 .46503 m
.11905 .46503 L
s
.02381 .56144 m
.11905 .56144 L
s
.02381 .07307 m
.11905 .07307 L
s
.02381 .25686 m
.11905 .25686 L
s
.02381 .47165 m
.11905 .47165 L
s
.02381 .14338 m
.11905 .14338 L
s
.02381 .35603 m
.11905 .35603 L
s
.02381 .56449 m
.11905 .56449 L
s
.02381 .22496 m
.11905 .22496 L
s
.02381 .45883 m
.11905 .45883 L
s
.02381 .3166 m
.11905 .3166 L
s
.02381 .56039 m
.11905 .56039 L
s
.02381 .41707 m
.11905 .41707 L
s
.02381 .52485 m
.11905 .52485 L
s
[ .02 .02 ] 0 setdash
.02381 .57525 m
.97619 .57525 L
s
gsave
.07672 .60332 -68.9375 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(all) show
74.875 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.18254 .60332 -72.1875 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Mathematica5
%%IncludeFont: Mathematica5
/Mathematica5 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
({) show
67.563 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
81.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.28836 .60332 -72.1875 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Mathematica5
%%IncludeFont: Mathematica5
/Mathematica5 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
({) show
67.563 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
81.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.39418 .60332 -72.1875 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Mathematica5
%%IncludeFont: Mathematica5
/Mathematica5 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
({) show
67.563 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
81.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.5 .60332 -72.1875 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Mathematica5
%%IncludeFont: Mathematica5
/Mathematica5 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
({) show
67.563 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
81.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.60582 .60332 -72.1875 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Mathematica5
%%IncludeFont: Mathematica5
/Mathematica5 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
({) show
67.563 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
81.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.71164 .60332 -72.1875 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Mathematica5
%%IncludeFont: Mathematica5
/Mathematica5 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
({) show
67.563 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
81.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.81746 .60332 -72.1875 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Mathematica5
%%IncludeFont: Mathematica5
/Mathematica5 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
({) show
67.563 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
81.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.92328 .60332 -72.1875 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding WindowsANSIEncoding def
        currentdict
    end

    newfontname exch definefont pop
} def

63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 15.813 moveto
%%IncludeResource: font Mathematica5
%%IncludeFont: Mathematica5
/Mathematica5 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
({) show
67.563 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7) show
81.375 15.813 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.2 0 1 r
[ ] 0 setdash
.12963 .01472 m
.23545 .01472 L
s
.12963 .16198 m
.23545 .16198 L
s
.12963 .36283 m
.23545 .36283 L
s
.12963 .55791 m
.23545 .55791 L
s
.23545 .06906 m
.34127 .06906 L
s
.23545 .24888 m
.34127 .24888 L
s
.23545 .46183 m
.34127 .46183 L
s
.34127 .13345 m
.44709 .13345 L
s
.34127 .34008 m
.44709 .34008 L
s
.34127 .55107 m
.44709 .55107 L
s
.44709 .20633 m
.55291 .20633 L
s
.44709 .43379 m
.55291 .43379 L
s
.55291 .28654 m
.65873 .28654 L
s
.55291 .52743 m
.65873 .52743 L
s
.65873 .37307 m
.76455 .37307 L
s
.76455 .46503 m
.87037 .46503 L
s
.87037 .56144 m
.97619 .56144 L
s
.2 1 0 r
.23545 .07307 m
.34127 .07307 L
s
.23545 .25686 m
.34127 .25686 L
s
.23545 .47165 m
.34127 .47165 L
s
.34127 .14338 m
.44709 .14338 L
s
.34127 .35603 m
.44709 .35603 L
s
.34127 .56449 m
.44709 .56449 L
s
.44709 .22496 m
.55291 .22496 L
s
.44709 .45883 m
.55291 .45883 L
s
.55291 .3166 m
.65873 .3166 L
s
.55291 .56039 m
.65873 .56039 L
s
.65873 .41707 m
.76455 .41707 L
s
.76455 .52485 m
.87037 .52485 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{416.625, 257.25},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40009`0001PB000`400?l00000o`00003oo`3ooooo
0?ooog80oooo003o0?ooool0ooooLP3oool00?l0ooooo`3ooomb0?ooo`00o`3ooooo0?ooog80oooo
003o0?ooool0ooooLP3oool00?l0ooooo`3ooomb0?ooo`00o`3ooooo0?ooog80oooo003o0?ooool0
ooooLP3oool00?l0ooooo`3ooomb0?ooo`00o`3ooooo0?ooog80oooo000[0?ooo`030000003oool0
oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0
oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000C0?ooo`040?oog`2LB00004RL0=ooo`<0
oooo00D0omnL04P000000000001d0;ooo`0<0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?oo
o`004`3oool0103ooml0W4P00018W03Oool20?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;oo
o`0;0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`004`3oool0103ooml0W4P00018W03O
ool20?ooo`030;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool02P3oool00`000000oooo0?oo
o`3o0?ooool0ooooA03oool000l0oooo00<0_g@00000000000001@000000101d_ol0oono07@00018
W=l30?ooo`030=nLB00007@0_ooo00X0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo0003
0?ooo`030?oog`2LB000000000L0000000@0B9cO0?ooo`3ogi`0M9cO0P3oool0203ooml0W4P00018
W03Oool0oooo0?oOW0180000M;oo0`3oool00`3o_g@0001809cOo`0:0?ooo`030000003oool0oooo
0?l0ooooo`3ooom40?ooo`00403oool02`3OW4P0M;oo0?ooo`3ooml0W4P00018W03Oool0oooo0?oO
W0180000M;oo00<0oooo00<0okmd0000B02Lgol02P3oool500000?l0ooooo`3ooom20?ooo`00403o
ool02`3ooml0W4Qd0;ooo`3ooml0W4P00018W03Oool0oooo0?oOW0180000M;oo00<0oooo00<0okmd
0000B02Lgol02P3ooolj000000D0oooo@00c0?oo0?ooolL0oooo000A0?ooo`0:0?oOW018M;l0oooO
09a80000B9`0gooo0?ooo`3ogi`0B00007Boo`<0oooo00<0okmd0000B02Lgol02P3oool00`000000
oooo0?ooo`3o0?ooool0ooooA03oool00180oooo00T0gia807Bog`2LB00004RL0=ooo`3oool0oono
07@00018W=l00`3oool00`3OW4P0001d0;ooo`0:0?ooo`030000003oool0oooo0?l0ooooo`3ooom4
0?ooo`004P3oool01@3ooml0W4Q804Q80000B9`0gooo0080oooo00<0gia80018W03Oool00P3oool0
0`2oM00007Bo0?ooo`0:0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`004`3oool0103o
gi`0B0000018W03Oool20?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`0;0?ooo`030000
003oool0oooo0?l0ooooo`3ooom40?ooo`00503oool00`3OW4P004RL0=ooo`030?ooo`050?oo_`1d
000000000000M02oool0303oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<0
00000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom4
0?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3o
ool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo
0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000
oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo
000[0?ooo`@00000o`3ooooo0?oood<0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom4
0?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3o
ool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo
0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000
oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo
000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o
0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`03
0000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0oooo
A03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooocX00000A03ooom0
03<0ool0ooooR03oool002/0oooo1000003o0?ooool0oooo@`3oool002/0oooo00<000000?ooo`3o
ool0o`3ooooo0?oood@0oooo000[0?ooocX00000A03ooom003?o0?l0ooooR03oool002/0oooo00<0
00000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom4
0?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3o
ool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo
0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000
oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo
000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o
0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`@0
0000o`3ooooo0?oood<0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo
0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000
oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo
000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o
0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`03
0000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0oooo
A03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0
oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0
oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0oooo
o`3ooom40?ooo`00:`3oool400000?l0ooooo`3ooom30?ooo`00:`3oool00`000000oooo0?ooo`3o
0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`03
0000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0oooo
A03oool002/0oooo>P0000230?oood00<`3oo`3ooom90?ooo`00:`3oool00`000000oooo0?ooo`3o
0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`03
0000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0oooo
A03oool002/0oooo>P0000230?oood00<ol0o`3ooom90?ooo`00:`3oool00`000000oooo0?ooo`3o
0?ooool0ooooA03oool000l0oooo00<0oono07@0000000000P0000000`1d_ol0oooo0?ooo`030?oo
o`050?oOW018000000000000M02oool0303oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool0
00l0oooo00<0omnL04P00018W=l00P3oool00`2oM000B9cO0?ooo`020?ooo`070?oo_`1d0000M;oo
0?ooo`3o_g@0001d0;ooo`0;0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00503oool0
0`3ookl0M01809cOo`020?ooo`030;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool02P3oool0
0`000000oooo0?ooo`3o0?ooool0ooooA03oool001@0oooo00L0oono07@0001d_ol0oooo0?oo_`1d
0000B9cO00<0oooo00<0gia80000M02oool02P3oool00`000000oooo0?ooo`3o0?ooool0ooooA03o
ool000<0oooo00<0oooO09a8000000001`0000000`18W=l0oooo0?ooo`040?ooo`070?noM0000000
M;oo0?ooo`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00X0oooo00<000000?ooo`3oool0
o`3ooooo0?oood@0oooo000C0?ooo`080?oOW0180000001809cOo`3oool0omnL04P0001d_ol30?oo
o`030?noM00004P0W=oo00X0oooo1@00003o0?ooool0oooo@P3oool00140oooo00D0okmd00000000
0000001809cOo`020?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol02P3oool00`00
0000oooo0?ooo`3o0?ooool0ooooA03oool001<0oooo00<0_g@0001d_`3oool00P3oool00`3ogi`0
B00007Boo`030?ooo`030?noM00004P0W=oo00X0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0
oooo000D0?ooo`070;md0018W=l0oooo0?ooo`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo
00X0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000?0?ooo`030?oog`2LW;l0oooo0080
oooo00<0gia80000M02oool00P3oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00X0
oooo>P0000050?oood00<`3oo`3oooo70?ooo`00403oool01P3o_g@0B9cO0?ooo`3ookl0M000001d
_`<0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00/0oooo00<000000?ooo`3oool0o`3o
oooo0?oood@0oooo000A0?ooo`050=nLB000000000000000M02oool0103oool01@3ookl0M0000000
000007@0_ooo00`0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003o
ool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool0
02/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0
ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<0
00000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom4
0?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3o
ool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3o
ool400000?l0ooooo`3ooom30?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool0
02/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0
ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<0
00000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom4
0?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3o
ool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo
0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000
oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo>P0000320?oood00<`3oo`3oool:0?ooo`00:`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo1000003o0?ooool0oooo@`3oool0
02/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0
ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<0
00000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom4
0?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3o
ool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3o
oolj00000<80oooo@00co`3o0?ooo`X0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom4
0?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3o
ool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo
0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool400000?l0
ooooo`3ooom30?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<0
00000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom4
0?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3o
ool0o`3ooooo0?oood@0oooo000[0?ooocX00000A03ooom003<0ool0ooooR03oool002/0oooo00<0
00000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom4
0?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3o
ool0o`3ooooo0?oood@0oooo000[0?ooocX00000A03ooom003?o0?l0ooooR03oool002/0oooo00<0
00000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom4
0?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3o
ool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3o
ool400000?l0ooooo`3ooom30?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool0
02/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0
ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<0
00000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom4
0?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3o
ool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooA03oool000l0oooo00<0_g@000000000000010000000
2@18W=l0oooo0?ooo`3oool0omnL04P000000000001d0;ooo`0<0?ooocX00000o`3oool20?oood00
<`3obP3oool00100oooo00<0gia80000000000000`0000002P00B9`0gooo0?ooo`3ookl0M00007Bo
o`3oool0okmd0000M02oool;0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`004@3oool0
0`3OW4P0B9cO0?ooo`020?ooo`060=nLM02Lgol0oooo0;md0000B9`0gooo0P3oool00`2oM00007Bo
0?ooo`0:0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`004P3oool00`3OW4P0M;oo0?oo
o`030?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool02P3oool00`000000oooo0?oo
o`3o0?ooool0ooooA03oool000<0oooo00<0oooO09a8000000001`0000000`18W=l0oooo0?ooo`03
0?ooo`030;mdB02Lgol0oooo0080oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0:
0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`004`3oool00`3ooml0W4Qd0;ooo`020?oo
o`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol02P3oool500000?l0ooooo`3ooom20?oo
o`00503oool01`3o_g@007Bo0?ooo`3oool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0:
0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00503oool01`3ookl0M01809cOo`3oool0
omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0:0?ooo`030000003oool0oooo0?l0ooooo`3o
oom40?ooo`00503oool01`3ookl0M00004RLg`3oool0oono07@00018W=l00`3oool00`3OW4P0001d
0;ooo`0:0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`003`3oool00`3o_i`0W=oo0?oo
o`020?ooo`030?oOW0180000B9cO0080oooo00<0gia80018W03Oool00P3oool00`2oM00007Bo0?oo
o`0:0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`003`3oool0203ooml0W4Q809cOo`3o
ool0oono07@0000007@0_ooo0P3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool02`3oool0
0`000000oooo0?ooo`3o0?ooool0ooooA03oool00100oooo00H0oooO09a80000000000000000M02o
ool40?ooo`050?oo_`1d000000000000M02oool0303oool00`000000oooo0?ooo`3o0?ooool0oooo
A03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0
oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0
oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0oooo
o`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo>P00003o
0?ooo`80oooo@00co`3:0?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0
oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0oooo
o`3ooom40?ooo`00:`3oool400000?l0ooooo`3ooom30?ooo`00:`3oool00`000000oooo0?ooo`3o
0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`03
0000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0oooo
A03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0
oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0
oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0oooo
o`3ooom40?ooo`00:`3ooolj000008<0oooo@00c0?oo0?ooodT0oooo000[0?ooo`030000003oool0
oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0
oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0oooo
o`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<00000
0?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`@00000o`3ooooo0?oood<0oooo000[0?ooo`03
0000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0oooo
A03oool002/0oooo>P0000230?oood00<ol0o`3ooom90?ooo`00:`3oool00`000000oooo0?ooo`3o
0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`03
0000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3ooolj000000D0oooo@00c0?oo0?ooolL0oooo
000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o
0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`03
0000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0oooo
A03oool002/0oooo>P00003o0?oood40oooo@00c0?n;0?ooo`00:`3oool00`000000oooo0?ooo`3o
0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`@0
0000o`3ooooo0?oood<0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo
0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000
oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo
000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o
0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`03
0000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0oooo
A03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0
oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0
oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0oooo
o`3ooom40?ooo`00:`3oool400000?l0ooooo`3ooom30?ooo`00:`3oool00`000000oooo0?ooo`3o
0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`03
0000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0oooo
A03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooocX00000o`3ooom1
0?oood00<ol0R`3oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`03
0000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0oooo
A03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0
oooo0?l0ooooo`3ooom40?ooo`004@3oool01P2oM000000000000000000004RL0=ooo`<0oooo00D0
omnL04P000000000001d0;ooo`0<0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`004P3o
ool00`3ogi`0B00007Boo`040?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`0;0?ooo`03
0000003oool0oooo0?l0ooooo`3ooom40?ooo`004P3oool00`3ogi`0B00007Boo`040?ooo`030;md
0000B9`0gooo0080oooo00<0_g@0001d_`3oool02P3oool00`000000oooo0?ooo`3o0?ooool0oooo
A03oool00180oooo00<0omnL04P0001d_ol00`3oool00`3ookl0M00004RLg`030?ooo`030=nLB000
07@0_ooo00X0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo00030?ooo`030?oog`2LB000
000000L0000000<0B9cO0?ooo`3oool00P3oool00`3ogi`0B00007Boo`030?ooo`030?oOW0180000
M;oo00<0oooo00<0okmd0000B02Lgol02P3ooolj00000<80oooo@00c0?oo0?ooo`X0oooo000B0?oo
o`030?oOW0180000M;oo00<0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0:0?oo
o`D00000o`3ooooo0?oood80oooo000B0?ooo`030?oOW0180000M;oo00<0oooo00<0omnL04P0001d
_ol00`3oool00`3o_g@0001809cOo`0:0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00
4P3oool00`3ogi`0B00007Boo`030?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0
2P3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool00180oooo00<0omnL04P0001d_ol00`3o
ool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00X0oooo00<000000?ooo`3oool0o`3o
oooo0?oood@0oooo000B0?ooo`030?oOW0180000M;oo00@0oooo00<0gia80018W03Oool00P3oool0
0`2oM00007Bo0?ooo`0:0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00403oool01@3o
oml0W4P0000000000000M;oo00@0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00/0oooo
00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000B0?ooo`030?noM0000000M;oo00D0oooo00D0
oono07@000000000001d0;ooo`0<0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo
0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000
oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo
000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o
0?ooool0ooooA03oool002/0oooo>P0000320?oood00<ol0o`3oool:0?ooo`00:`3oool00`000000
oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo>P0000140?oood00<`3oo`3ooon80?ooo`00:`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo>P00003o0?oooh00oooo?`0c0?m=
0?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3o
ool0o`3ooooo0?oood@0oooo000[0?ooocX00000A03ooom003?o0?l0ooooR03oool002/0oooo00<0
00000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom4
0?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3o
ool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo
0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000
oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo
000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool400000?l0ooooo`3ooom3
0?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3o
ool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo
0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000
oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo
000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o
0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`03
0000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0oooo
A03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0
oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0
oooo1000003o0?ooool0oooo@`3oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo
000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3ooolj00000?l0ooooP03ooolo
03?o04d0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3ooolj00000?l0
oooo0P3ooom003<0olX0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo
0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000
oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo
000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o
0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`03
0000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3oool400000?l0ooooo`3ooom30?ooo`00:`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?ooo`3oool0o`3ooooo
0?oood@0oooo000[0?ooocX00000P`3ooom003<0ool0ooooB@3oool002/0oooo00<000000?ooo`3o
ool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00:`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo>P0000050?oood00<`3oo`3oooo7
0?ooo`00:`3ooolj00000?l0oooo0P3ooom003?o0<X0oooo000[0?ooocX00000o`3ooonn0?oood00
<`3o3P3oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooocX00000P`3o
oom003?o0?l0ooooB@3oool001X0oooo00D0omnL04P000000000001d0;ooo`0<0?ooo`030000003o
ool0oooo0?l0ooooo`3ooom40?ooo`006@3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool0
2`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool001T0oooo00<0_g@00018W03Oool00P3o
ool00`2oM00007Bo0?ooo`0:0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`00603oool0
0`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00X0oooo00<000000?ooo`3oool0o`3ooooo
0?oood@0oooo000H0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol02P3oool00`00
0000oooo0?ooo`3o0?ooool0ooooA03oool001P0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0
001809cOo`0:0?ooo`d000002`3oool=000000/0oooo3@00000;0?ooo`d000002P3oool=000000/0
oooo3@00000;0?ooo`d000002`3oool=000000/0oooo3000000;0?ooo`d000002`3oool=000000/0
oooo3@00000;0?ooo``000002`3oool=000000/0oooo3@00000;0?ooo`d000002`3oool=000000X0
oooo3@00000;0?ooo`d000002`3oool=000000/0oooo3@00000:0?ooo`d000002`3oool=000000/0
oooo3@00000E0?ooo`00603oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00X0oooo
00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000H0?ooo`030?oOW0180000M;oo00<0oooo00<0
okmd0000B02Lgol02P3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool001P0oooo00<0oono
07@00018W=l00`3oool00`3OW4P0001d0;ooo`0:0?ooo`030000003oool0oooo0?l0ooooo`3ooom4
0?ooo`006@3oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00X0oooo00<000000?oo
o`3oool0o`3ooooo0?oood@0oooo000I0?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`0;
0?ooo`030000003oool0oooo0?l0ooooo`3ooom40?ooo`006P3oool01@3ookl0M0000000000007@0
_ooo00`0oooo00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo
0?l0ooooo`3ooom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo
00<000000?ooo`3oool0o`3ooooo0?oood@0oooo000[0?ooo`030000003oool0oooo0?l0ooooo`3o
oom40?ooo`00:`3oool00`000000oooo0?ooo`3o0?ooool0ooooA03oool002/0oooo00<000000?oo
o`3oool0503oool02@3ooml0W4P000000000000004RL09ad0000000007Bo0=nLB002000000060000
M02LW4P000000000000007@0_ooo:@3oool0103ookl0M00000000018W=l@0?ooo`050?oOW0180000
00000000M02oool09P3oool0103ookl0M00000000018W=l@0?ooo`060;md00000000000000000000
B9`0gooo9@3oool0103ookl0M00000000018W=l>0?ooo`030;md00000000000000@0000000<0B9cO
0?ooo`3oool08`3oool0103ookl0M00000000018W=l>0?ooo`030?oo_`1d000000000080000000<0
M;oo0?ooo`3oool0903oool0103ookl0M00000000018W=lC0?ooo`040?oog`2LB00004RL0=ooob@0
oooo00@0oono07@000000000B9cO3`3oool00`3ooml0W4P0000000020000000307Boo`3oool0oooo
02@0oooo00@0oono07@000000000B9cO4@3oool01@3ogi`0B000000000000000M;oo02D0oooo00@0
oono07@000000000B9cO4P3oool00`3OW4P0B9cO0?ooo`0S0?ooo`00:`3oool00`000000oooo0?oo
o`0D0?ooo`0@0?noM0000000M;oo0?ooo`3ogi`0B000001d_`3o_kl0_ooo0=nLB00007@0_ooo0?oo
o`3OW4P0001d0;ooobX0oooo00D0oono07@00000M;l0gkno0=ooo`0>0?ooo`070?oo_`1d0000M;oo
0?ooo`3o_g@0001d0;ooo`0U0?ooo`050?oo_`1d000007Bo0=no_`3Oool0403oool00`3ogi`0B000
07Boo`0W0?ooo`050?oo_`1d000007Bo0=no_`3Oool03P3oool00`3OW4P000000000000300000003
0018W03Oool0oooo02<0oooo00D0oono07@00000M;l0gkno0=ooo`0=0?ooo`030?oOW0180000B9cO
0080oooo00<0_g@004RLg`3oool0903oool01@3ookl0M000001d_`3O_kl0gooo0180oooo00@0oooO
09a80000B9`0gooo903oool01@3ookl0M000001d_`3O_kl0gooo00h0oooo00<0oooO09a8M02oool0
0P3oool00`2oM000M;oo0?ooo`0T0?ooo`050?oo_`1d000007Bo0=no_`3Oool03`3oool01`3ogi`0
B00004RLg`3oool0oono07@0001d_ol0903oool01@3ookl0M000001d_`3O_kl0gooo0140oooo00<0
omnL04Q8W03Oool08`3oool002/0oooo00<000000?ooo`3oool0503oool01`3o_g@0001809cOo`3o
ool0oooO09a80000M;l00P3oool01`3OW4P0001d0;ooo`3oool0gia80000M02oool0:P3oool00`3o
okl0M00004RLg`040?ooo`030?oog`2LB000000000L0000000D0B9cO0?ooo`2oM00004RL0=ooo`02
0?ooo`030;md0000M;l0oooo02@0oooo00<0oono07@00018W=l0103oool00`3ooml0W4P000000007
0000000704RLg`3oool0oooo0?ooo`3ogi`0B00007Boo`0W0?ooo`030?oo_`1d0000B9cO00<0oooo
00<0oooO09a8000000001`0000001P18W=l0oooo0?ooo`3oool0gia804RLg`<0oooo00<0giad09cO
o`3oool08`3oool00`3ookl0M00004RLg`030?ooo`030?oog`2LB000000000L0000000<0B9cO0?oo
o`3oool0103oool00`3ookl0M01809cOo`0T0?ooo`030?oo_`1d0000B9cO00@0oooo00<0oooO09a8
000000001`0000000`18W=l0oooo0?ooo`030?ooo`040?oog`2LB00004RL0=ooob@0oooo00<0oono
07@00018W=l0103oool00`3ooml0W4P0000000070000000304RLg`3oool0oooo00@0oooo00<0oooO
09a8M02oool0903oool00`3ookl0M00004RLg`040?ooo`030?oog`2LB000000000L0000000D0B9cO
0?ooo`3ooml0W4P0001d_`030?ooo`030?noM000B9`0gooo02<0oooo00<0oono07@00018W=l0103o
ool00`3ooml0W4P0000000070000000304RLg`3oool0oooo0080oooo00<0oooO09a8B02Lgol08`3o
ool002/0oooo00<000000?ooo`3oool05@3oool01P3o_g@00000001d_`3ooml0W4P0001d_`80oooo
00L0gia80000M02oool0oooo0=nLB00007@0_ooo02T0oooo00@0oooO09a80000000007Bo3`3oool0
0`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo02<0oooo00@0oooO09a80000000007Bo4P3o
ool00`3ogi`0B00007Boo`0V0?ooo`040?oog`2LB0000000001d_a80oooo00<0gia807Boo`3oool0
9P3oool0103ooml0W4P000000000M;lD0?ooo`030?oo_`1d0000M;oo02<0oooo00@0oooO09a80000
000007Bo403oool00`2oM0000000000000050000000307Boo`3oool0oooo0200oooo00@0oooO09a8
0000000007Bo5P3oool00`2oM000M;oo0?ooo`0R0?ooo`040?oog`2LB0000000001d_a00oooo00<0
omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0R0?ooo`040?oog`2LB0000000001d_a@0oooo
00<0gia804RLg`3oool08P3oool002/0oooo00<000000?ooo`3oool05`3oool0103o_g@000000000
0000M;l20?ooo`070=nLB00007@0_ooo0?ooo`3OW4P0001d0;ooo`0Z0?ooo`040?oog`2LB00004RL
0=ooo`h0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0T0?ooo`040?oog`2LB000
04RL0=oooa40oooo00<0omnL04P0001d_ol09`3oool0103ooml0W4P00018W03OoolB0?ooo`030;md
B02Lgol0oooo02H0oooo00@0oooO09a80000B9`0gooo4`3oool00`3o_g@0000007Boo`0T0?ooo`04
0?oog`2LB00004RL0=ooo`l0oooo00P0omnL07BLg`3oool0oooo0?oog`2LB00004RL0=ooob@0oooo
00@0oooO09a80000B9`0gooo503oool00`3ooml0W4P004RLg`0T0?ooo`040?oog`2LB00004RL0=oo
o`l0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0S0?ooo`040?oog`2LB00004RL
0=oooa<0oooo00<0omnL04Q8W03Oool08P3oool002/0oooo1000000C0?ooo`070?oo_`1d0000M;oo
0?ooo`3ooml0W4P0001d_`020?ooo`070=nLB00007@0_ooo0?ooo`3OW4P0001d0;ooo`0[0?ooo`03
0?noM00004P0M;oO00h0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0U0?ooo`03
0?noM00004P0M;oO0140oooo00<0omnL04P0001d_ol0:03oool00`3o_g@0001807Bog`0B0?ooo`03
0?oog`2LB7@0_ooo02L0oooo00<0okmd0000B01d_ml04P3oool0103ogi`0B0000000B02LgolU0?oo
o`030?noM00004P0M;oO0100oooo00L0gia807Boo`3oool0oooO09a80000B9`0gooo02D0oooo00<0
okmd0000B01d_ml0503oool00`3OW4P0000007Boo`0U0?ooo`030?noM00004P0M;oO00l0oooo00<0
omnL04P0001d_ol00`3oool00`2oM000001809cOo`0T0?ooo`030?noM00004P0M;oO01<0oooo00<0
oooO09a8B02Lgol08P3oool002/0oooo00<000000?ooo`3oool0503oool01`3ookl0M00004RLg`3o
ool0oono07@00000M;l00P3oool01`3OW4P0001d0;ooo`3oool0gia80000M02oool0:`3oool0103o
gi`0B01807AdB01d_ol20?ooo`030?oog`2LB000000000L0000000@0B9cO0?oOW0180000M;oo0`3o
ool00`3o_g@0001809cOo`0U0?ooo`040?oOW01804P0M7A807Boo`80oooo00<0oooO09a800000000
1`0000001`18W=l0oooo0?ooo`3oool0omnL04P0001d_ol0:03oool01`3ogi`0B01807AdB01d_ol0
oooo0?oog`2LB000200000000`18W=l0oooo0?ooo`040?ooo`030?noM000M;l0oooo02H0oooo00L0
omnL04P0B01dM4P0M;oo0?ooo`3ooml0W4P000P0000000D0B9cO0?ooo`3oool0oooo0?noM0020000
00030000B02Lgol0oooo02D0oooo00@0omnL04P0B01dM4P0M;oo0P3oool00`3ooml0W4P000000007
0000000:04RLg`3oool0oooo0?oog`2LB7@0_ooo0?oog`2LB00004RL0=ooobD0oooo00@0omnL04P0
B01dM4P0M;oo0P3oool00`3ooml0W4P0000000070000000:04RLg`3oool0oooo0=oOg`3oool0oono
07@000000000001d0;ooobD0oooo00@0omnL04P0B01dM4P0M;oo0P3oool00`3ooml0W4P000000007
0000000:04RLg`3oool0oono07@000000000B9cO0?ooo`3o_g@00000001d_bD0oooo00@0omnL04P0
B01dM4P0M;oo0P3oool00`3ooml0W4P0000000070000000304RLg`3oool0oooo00@0oooo00<0gia8
001d_`3oool08@3oool002/0oooo00<000000?ooo`3oool05@3oool01P3ookl0M000000000000000
001d0;ooo`80oooo00L0gia80000M02oool0oooo0=nLB00007@0_ooo02/0oooo00D0oooO09a80018
W=l0_gAd0;ooo`0<0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol09@3oool01@3o
oml0W4P004RLg`2oM7@0_ooo00l0oooo00<0omnL04P0001d_ol0:03oool01@3ooml0W4P004RLg`2o
M7@0_ooo0140oooo00<0oono07@0B02Lgol09P3oool01@3ooml0W4P004RLg`2oM7@0_ooo0100oooo
00<0_g@0001d_`3oool09P3oool01@3ooml0W4P004RLg`2oM7@0_ooo00l0oooo00H0omnL04Qd_`3o
oml0W4P00018W03OoolU0?ooo`050?oog`2LB000B9cO0;mdM02oool03P3oool01P3OW4P000000000
00000000001809cOobH0oooo00D0oooO09a80018W=l0_gAd0;ooo`0>0?ooo`030;md0000B9`0_ia8
0080000000<004RL0=ooo`3oool0903oool01@3ooml0W4P004RLg`2oM7@0_ooo0180oooo00<0oono
07@0M02oool08@3oool002/0oooo00<000000?ooo`3oool07@3oool01`3OW4P0001d0;ooo`3oool0
gia80000M02oool0;03oool0103OW4P004RL0=oo_`1dM;l<0?ooo`030?oo_`1d0000B9cO00<0oooo
00<0gia80000M02oool09P3oool0103OW4P004RL0=oo_`1dM;l?0?ooo`030?oOW0180000M;oo02T0
oooo00@0gia80018W03Ookl0M7Bo4@3oool00`3ookl0M00004RLg`0W0?ooo`040=nLB000B9`0gono
07Ad_a40oooo00<0_g@004RLg`3oool09P3oool0103OW4P004RL0=oo_`1dM;l@0?ooo`050=nLB01d
_ml0W4P00018W03Oool09P3oool0103OW4P004RL0=oo_`1dM;l>0?ooo`040?oo_`1d0000000004RL
gbT0oooo00@0gia80018W03Ookl0M7Bo3P3oool00`3ogi`0B00007Boo`0Z0?ooo`040=nLB000B9`0
gono07Ad_`d0oooo00<0omoO0=ooo`3oool00`3oool00`2oM000M;oo0?ooo`0P0?ooo`00:`3oool0
0`000000oooo0?ooo`0M0?ooo`070=nLB00007@0_ooo0?ooo`3OW4P0001d0;ooo`0/0?ooo`050?oo
_`1d04P0W=oo0=nLB01d_ol0303oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo02H0
oooo00D0oono07@0B02Lgol0gia807Boo`0>0?ooo`030?oOW0180000M;oo02T0oooo00D0oono07@0
B02Lgol0gia807Boo`0;0?ooo`030?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l09`3oool0
1@3ookl0M01809cOo`3OW4P0M;oo00/0oooo00<0oooO09bL_`3oool00P3oool00`3OW4P0001d0;oo
o`0V0?ooo`050?oo_`1d04P0W=oo0=nLB01d_ol03`3oool01@3ooml0W4Q804Q80000B9`0gooo02H0
oooo00D0oono07@0B02Lgol0gia807Boo`0>0?ooo`030;md001d_ol0oooo02T0oooo00D0oono07@0
B02Lgol0gia807Boo`0>0?ooo`030=nLB00004P0W=oo02T0oooo00D0oono07@0B02Lgol0gia807Bo
o`0<0?ooo`030?oo_`1dB9`0gooo00<0oooo00<0okmd001d_`3oool0803oool002/0oooo00<00000
0?ooo`3oool07@3oool01`3OW4P0001d0;ooo`3oool0gia80000M02oool0;@3oool01@3o_g@004RL
0=oOW018B9`0gooo00/0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo02P0oooo00D0okmd
0018W03Ogi`0B4RL0=ooo`0;0?ooo`050?oog`2LB00000000000001d_ol0:P3oool01@3o_g@004RL
0=oOW018B9`0gooo00X0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooobP0oooo00D0
okmd0018W03Ogi`0B4RL0=ooo`0;0?ooo`060?noM018W=l0oooo0?oo_`1d000007Bo:03oool01@3o
_g@004RL0=oOW018B9`0gooo00l0oooo00@0omnL04P00000B9`0gooo9`3oool01@3o_g@004RL0=oO
W018B9`0gooo00d0oooo00H0omnL04P00000000000000000B02LgolW0?ooo`050?noM000B9`0gmnL
04Q8W03Oool03P3oool00`3o_g@00000001d_`0Y0?ooo`050?noM000B9`0gmnL04Q8W03Oool0303o
ool00`2oM000000000000003000000030000M02oool0oooo01l0oooo000[0?ooo`030000003oool0
oooo01/0oooo00T0oooO09a800000000001d0;oog`2LB00000000000M02oool0;P3oool00`3ogi`0
B000001d_`0=0?ooo`050?oo_`1d000000000000M02oool0:P3oool00`3ogi`0B000001d_`0>0?oo
o`030?noM0000000M;oo02/0oooo00<0omnL04P00000M;l0303oool01P3ooml0W4P0000000000000
001d0;ooobX0oooo00<0omnL04P00000M;l03@3oool01@3OW4P000000000000007@0_ooo02T0oooo
00<0omnL04P00000M;l04@3oool00`3OW4P004RL0=ooo`0X0?ooo`030?oOW018000007Bo00l0oooo
00D0_g@0000000000000000004RLg`0X0?ooo`030?oOW018000007Bo0100oooo00D0oooO09a80000
0000001d0;ooo`0W0?ooo`030?oOW018000007Bo00d0oooo00<0okmd000000000000100000000`1d
_ol0oooo0?ooo`0N0?ooo`00:`3oool00`000000oooo0?ooo`0M0?ooo`070;md000007@0_ooo0?oo
o`2oM000001d0;ooo`3o0?ooool0oooo803oool002/0oooo00<000000?ooo`3oool0o`3ooooo0?oo
od@0oooo003o0?ooool0ooooLP3oool00?l0ooooo`3ooomb0?ooo`00o`3ooooo0?ooog80oooo003o
0?ooool0ooooLP3oool00?l0ooooo`3ooomb0?ooo`00o`3ooooo0?ooog80oooo003o0?ooool0oooo
LP3oool00001\
\>"],
  ImageRangeCache->{{{0, 623.938}, {384.875, 0}} -> {-0.698174, -42.2549, \
0.0238563, 0.179851}}],

Cell["\<\
Caution: energy levels that are nearly unbound can be difficult to determine \
accurately.  Be sure to use a large integration interval because weakly-bound \
wave functions possess long tails.  Some trial and error will probably be \
needed to choose the match radius.  Plot the wave functions to ensure that \
the integration routine found the desired number of nodes and that the tail \
behavior is reasonable.\
\>", "ExerciseText"]
}, Closed]],

Cell[CellGroupData[{

Cell["Mexican hat", "ExerciseTitle"],

Cell["The so-called Mexican hat potential takes the form", "ExerciseText"],

Cell[BoxData[
    \(V[r]\  = \(V\_0\) \((\ \((r\/r\_0)\)\^4 - 
            2 \((r\/r\_0)\)\^2\ )\)\)], "DisplayFormula",
  FontFamily->"Times New Roman"],

Cell["\<\
and is often used as a model for systems that exhibit spontaneous symmetry \
breaking.\
\>", "ExerciseText"],

Cell[TextData[{
  "a) Express the radial equation in terms of the dimensionless variables ",
  Cell[BoxData[
      \(TraditionalForm\`\[Rho] = r/r\_0\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`\[CurlyEpsilon]\_\(n, \[ScriptL]\) = 
        E\_\(n, \[ScriptL]\)/V\_0\)]],
  ", and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Mu] = m\ \(V\_0\) r\_0\^2/\[HBar]\^2\)]],
  "."
}], "ExerciseText"],

Cell[TextData[{
  "b) Determine ",
  Cell[BoxData[
      \(TraditionalForm\`1  S\)]],
  " wave functions and energies for several values of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Mu]\)]],
  " between about 1 and 100.  Make a log-log plot of dependence of ",
  Cell[BoxData[
      \(TraditionalForm\`s = 1 + \[CurlyEpsilon]\_\(1, 0\)\)]],
  " versus ",
  Cell[BoxData[
      \(TraditionalForm\`\[Mu]\)]],
  " and observe that the behavior for large ",
  Cell[BoxData[
      \(TraditionalForm\`\[Mu]\)]],
  " appears to be linear.  Therefore, fit the numerical results for large ",
  Cell[BoxData[
      \(TraditionalForm\`\[Mu]\)]],
  " with the function ",
  Cell[BoxData[
      \(TraditionalForm\`s = a\ \[Mu]\^b\)]],
  " and include this model on the log-log plot.  Comment upon your results."
}], "ExerciseText"],

Cell[TextData[{
  "c) Construct an energy-level diagram for ",
  Cell[BoxData[
      \(TraditionalForm\`\[Mu] = 5\)]],
  " including ",
  Cell[BoxData[
      \(TraditionalForm\`1 \[LessEqual] n \[LessEqual] 3\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`0 \[LessEqual] \[ScriptL] \[LessEqual] 2\)]],
  ".  "
}], "ExerciseText"]
}, Closed]],

Cell[CellGroupData[{

Cell["ground-state energy", "ExerciseTitle"],

Cell[TextData[{
  "Locating specific states using ",
  StyleBox["FindRoot", "InlineInput",
    FontWeight->"Bold"],
  " can be tedious because it is necessary to specify the energy window by \
hand and even then the algorithm sometimes returns a solution outside that \
window.  In this problem we develop a more reliable method for determining \
the ground-state energy that searches for the first sign change in the \
mismatch for ",
  StyleBox["ShootBSWF2", "InlineInput",
    FontWeight->"Bold"],
  "."
}], "ExerciseText"],

Cell[TextData[{
  "a) Write a module that produces a list of mismatch data over a specified \
energy range and determines the energy for the first sign change in that \
list.  The average of the two energies that surround the lowest node in the \
mismatch data provides a good approximation to the energy; the accuracy is \
determined by the energy step.  Although this procedure is normally intended \
for the ground state, it actually finds the state with the lowest energy \
within the specified range.  Test this routine by evaluating the mass \
dependence of the ground-state energy for the harmonic oscillator potential, \
which should take the form ",
  Cell[BoxData[
      \(TraditionalForm\`\(1\/2\) m\^\(\(-1\)/2\)\)]],
  "."
}], "ExerciseText"],

Cell["\<\
b) Upgrade this root-finding algorithm to subdivide the region surrounding \
the node several times and thereby improve the accuracy of the ground-state \
energy.  Also, provide an error message if no node is detected within the \
selected energy range.  This may occur either because the energy window is \
inappropriate or because the step size is too coarse.\
\>", "ExerciseText"],

Cell["\<\
c) Use your function to plot the dependence of the ground-state energy for a \
double-well potential upon mass.\
\>", "ExerciseText"]
}, Closed]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"5.2 for Microsoft Windows",
ScreenRectangle->{{0, 1400}, {0, 952}},
WindowToolbars->"EditBar",
WindowSize->{1392, 912},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowSelection->True,
ShowCellLabel->False,
Magnification->1.5,
StyleDefinitions -> Notebook[{
  
  Cell[CellGroupData[{
  Cell["Style Definitions", "Subtitle"],
  
  Cell["\<\
Modify the definitions below to change the default appearance of all cells in \
a given style.  Make modifications to any definition using commands in the \
Format menu.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell["Style Environment Names", "Section"],
  
  Cell[StyleData[All, "Working"],
    PageWidth->WindowWidth,
    ScriptMinSize->9],
  
  Cell[StyleData[All, "Presentation"],
    PageWidth->WindowWidth,
    ScriptMinSize->12,
    FontSize->16],
  
  Cell[StyleData[All, "Condensed"],
    PageWidth->PaperWidth,
    CellBracketOptions->{"Margins"->{1, 1},
    "Widths"->{0, 5}},
    ScriptMinSize->7,
    FontSize->10],
  
  Cell[StyleData[All, "Printout"],
    PageWidth->PaperWidth,
    ScriptMinSize->7,
    FontSize->10,
    PrivateFontOptions->{"FontType"->"Outline"}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Notebook Options", "Section"],
  
  Cell["\<\
The options defined for the style below will be used at the Notebook level.\
\>", "Text"],
  
  Cell[StyleData["Notebook"],
    PageHeaders->{{Cell[ 
            TextData[ {
              CounterBox[ "Page"]}], "PageNumber"], None, Cell[ 
            TextData[ {
              ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
            TextData[ {
              ValueBox[ "FileName"]}], "Header"], None, Cell[ 
            TextData[ {
              CounterBox[ "Page"]}], "PageNumber"]}},
    PageFooters->{{Cell[ 
            TextData[ {"Essential Mathematica for Students of Science"}], 
            "Footer"], None, Cell[ 
            TextData[ {"\[Copyright] James J. Kelly, 1998"}], "Footer"]}, {
        Cell[ 
            TextData[ {"Essential Mathematica for Students of Science"}], 
            "Footer"], None, Cell[ 
            TextData[ {"\[Copyright] James J. Kelly, 1998"}], "Footer"]}},
    PageHeaderLines->{True, True},
    PageFooterLines->{True, True},
    PrintingOptions->{"FirstPageHeader"->False,
    "FacingPages"->True},
    CellFrameLabelMargins->6,
    StyleMenuListing->None]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Headings", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Title"],
    ShowCellBracket->True,
    CellMargins->{{0, 0}, {0, 0}},
    CellGroupingRules->"TitleGrouping",
    PageBreakBelow->False,
    LineSpacing->{1, 0},
    CounterIncrements->"Title",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
        "Subtitle", 0}, {"Subsubtitle", 0}},
    FontSize->34,
    FontColor->GrayLevel[1],
    Background->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["Title", "Presentation"],
    CellMargins->{{0, 0}, {0, 0}},
    LineSpacing->{1, 0},
    FontSize->44],
  
  Cell[StyleData["Title", "Condensed"],
    CellMargins->{{0, 0}, {0, 0}},
    FontSize->24],
  
  Cell[StyleData["Title", "Printout"],
    CellMargins->{{0, 0}, {0, 0}},
    FontSize->24,
    FontTracking->"Plain",
    FontColor->GrayLevel[0],
    Background->None]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subtitle"],
    CellMargins->{{0, 0}, {0, 0}},
    PageBreakBelow->False,
    LineSpacing->{1, 3},
    ParagraphIndent->-96,
    CounterIncrements->"Subtitle",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
        "Subsubtitle", 0}},
    FontFamily->"Helvetica",
    FontSize->18,
    FontColor->GrayLevel[1],
    Background->RGBColor[0.2, 0.700008, 0.700008]],
  
  Cell[StyleData["Subtitle", "Presentation"],
    CellMargins->{{0, 0}, {0, 0}},
    LineSpacing->{1, 6},
    ParagraphIndent->-157,
    FontSize->30],
  
  Cell[StyleData["Subtitle", "Condensed"],
    CellMargins->{{0, 0}, {0, 0}},
    ParagraphIndent->-78,
    FontSize->14],
  
  Cell[StyleData["Subtitle", "Printout"],
    CellMargins->{{0, 0}, {0, 0}},
    ParagraphIndent->-85,
    FontSize->16,
    FontColor->GrayLevel[0],
    Background->None]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsubtitle"],
    ShowCellBracket->True,
    CellMargins->{{10, 4}, {30, 10}},
    PageBreakBelow->False,
    CounterIncrements->"Subsubtitle",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}},
    FontFamily->"Helvetica",
    FontSize->14,
    FontSlant->"Italic"],
  
  Cell[StyleData["Subsubtitle", "Presentation"],
    ShowCellBracket->True,
    CellMargins->{{8, 10}, {40, 20}},
    LineSpacing->{1, 0},
    FontSize->24],
  
  Cell[StyleData["Subsubtitle", "Condensed"],
    ShowCellBracket->True,
    CellMargins->{{8, 10}, {12, 8}},
    FontSize->12],
  
  Cell[StyleData["Subsubtitle", "Printout"],
    ShowCellBracket->True,
    CellMargins->{{9, 10}, {50, 10}},
    FontSize->14]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Section"],
    CellFrame->{{0, 0}, {0, 1}},
    CellDingbat->None,
    CellMargins->{{12, Inherited}, {4, 24}},
    CellGroupingRules->{"SectionGrouping", 30},
    PageBreakBelow->False,
    CellFrameMargins->6,
    CounterIncrements->"Section",
    CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->16,
    FontWeight->"Bold",
    FontColor->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["Section", "Presentation"],
    CellMargins->{{10, 10}, {8, 32}},
    LineSpacing->{1, 2},
    FontSize->24,
    FontTracking->"Condensed"],
  
  Cell[StyleData["Section", "Condensed"],
    CellMargins->{{9, 0}, {2, 50}},
    FontSize->14],
  
  Cell[StyleData["Section", "Printout"],
    CellMargins->{{9, 0}, {2, 50}},
    FontSize->14,
    FontTracking->"Plain",
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsection"],
    CellMargins->{{12, Inherited}, {8, 20}},
    CellGroupingRules->{"SectionGrouping", 40},
    PageBreakBelow->False,
    CounterIncrements->"Subsection",
    CounterAssignments->{{"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->13,
    FontWeight->"Bold"],
  
  Cell[StyleData["Subsection", "Presentation"],
    CellMargins->{{11, 10}, {8, 32}},
    LineSpacing->{1, 0},
    FontSize->22],
  
  Cell[StyleData["Subsection", "Condensed"],
    CellMargins->{{8, Inherited}, {2, 12}},
    FontSize->12],
  
  Cell[StyleData["Subsection", "Printout"],
    CellMargins->{{9, 0}, {4, 40}},
    FontSize->12]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsubsection"],
    CellDingbat->"\[FilledSquare]",
    CellMargins->{{25, Inherited}, {8, 12}},
    CellGroupingRules->{"SectionGrouping", 50},
    PageBreakBelow->False,
    CounterIncrements->"Subsubsection",
    FontFamily->"Times",
    FontSize->13,
    FontWeight->"Bold"],
  
  Cell[StyleData["Subsubsection", "Presentation"],
    CellMargins->{{29, 10}, {8, 26}},
    LineSpacing->{1, 0},
    FontSize->18],
  
  Cell[StyleData["Subsubsection", "Condensed"],
    CellMargins->{{21, 0}, {4, 20}},
    FontSize->11],
  
  Cell[StyleData["Subsubsection", "Printout"],
    CellMargins->{{21, 0}, {4, 20}},
    FontSize->11]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Body Text", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Text"],
    CellMargins->{{12, 10}, {5, 5}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 12},
    CounterIncrements->"Text",
    FontFamily->"Times"],
  
  Cell[StyleData["Text", "Presentation"],
    CellMargins->{{13, 10}, {8, 8}},
    LineSpacing->{1, 5},
    ParagraphSpacing->{0, 12},
    FontSize->16],
  
  Cell[StyleData["Text", "Condensed"],
    CellMargins->{{9, 0}, {4, 4}},
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 6}],
  
  Cell[StyleData["Text", "Printout"],
    CellMargins->{{9, 0}, {4, 4}},
    ParagraphSpacing->{0, 6}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["SmallText"],
    CellMargins->{{12, 10}, {5, 5}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 6},
    CounterIncrements->"SmallText",
    FontFamily->"Helvetica",
    FontSize->9],
  
  Cell[StyleData["SmallText", "Presentation"],
    CellMargins->{{13, 10}, {8, 8}},
    LineSpacing->{1, 5},
    FontSize->12],
  
  Cell[StyleData["SmallText", "Condensed"],
    CellMargins->{{8, 10}, {2, 2}},
    LineSpacing->{1, 2},
    FontSize->9],
  
  Cell[StyleData["SmallText", "Printout"],
    CellMargins->{{9, 0}, {4, 4}}]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Input/Output", "Section"],
  
  Cell["\<\
The cells in this section define styles used for input and output to the \
kernel.  Be careful when modifying, renaming, or removing these styles, \
because the front end associates special meanings with these style names.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Input"],
    CellFrame->0.5,
    CellMargins->{{25, 10}, {8, 8}},
    Evaluatable->True,
    CellGroupingRules->"InputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{5, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultInputFormatType,
    FormatType->InputForm,
    ShowStringCharacters->True,
    NumberMarks->True,
    CounterIncrements->"Input",
    FontWeight->"Bold",
    Background->RGBColor[1, 0.700008, 0.4]],
  
  Cell[StyleData["Input", "Presentation"],
    CellMargins->{{25, Inherited}, {10, 10}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Input", "Condensed"],
    CellMargins->{{25, 0}, {6, 6}}],
  
  Cell[StyleData["Input", "Printout"],
    CellFrame->0.5,
    CellMargins->{{25, 0}, {6, 6}},
    Background->None]
  }, Closed]],
  
  Cell[StyleData["InlineInput"],
    CellGroupingRules->"InputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    DefaultFormatType->DefaultInputFormatType,
    AutoItalicWords->{},
    FormatType->InputForm,
    ShowStringCharacters->True,
    NumberMarks->True,
    CounterIncrements->"Input",
    FontWeight->"Bold"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Output"],
    CellFrame->True,
    CellMargins->{{25, 10}, {8, 0}},
    CellElementSpacings->{"ClosedGroupTopMargin"->4},
    CellEditDuplicate->True,
    CellGroupingRules->"OutputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    CellLabelMargins->{{3, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Output",
    Background->RGBColor[1, 0.900008, 0.900008]],
  
  Cell[StyleData["Output", "Presentation"],
    CellMargins->{{25, Inherited}, {12, 0}},
    CellElementSpacings->{"ClosedGroupTopMargin"->4},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Output", "Condensed"],
    CellMargins->{{25, 0}, {6, 0}},
    CellElementSpacings->{"ClosedGroupTopMargin"->4}],
  
  Cell[StyleData["Output", "Printout"],
    CellMargins->{{25, 0}, {6, 0}},
    CellElementSpacings->{"ClosedGroupTopMargin"->4},
    Background->None]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Message"],
    CellMargins->{{62, Inherited}, {Inherited, Inherited}},
    CellGroupingRules->"OutputGrouping",
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Message",
    StyleMenuListing->None,
    FontColor->RGBColor[1, 0, 0]],
  
  Cell[StyleData["Message", "Presentation"],
    CellMargins->{{74, Inherited}, {Inherited, Inherited}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Message", "Condensed"],
    CellMargins->{{50, Inherited}, {Inherited, Inherited}}],
  
  Cell[StyleData["Message", "Printout"],
    CellMargins->{{54, Inherited}, {Inherited, Inherited}},
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Print"],
    CellMargins->{{62, Inherited}, {Inherited, Inherited}},
    CellGroupingRules->"OutputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Print",
    StyleMenuListing->None],
  
  Cell[StyleData["Print", "Presentation"],
    CellMargins->{{74, Inherited}, {Inherited, Inherited}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Print", "Condensed"],
    CellMargins->{{50, Inherited}, {Inherited, Inherited}}],
  
  Cell[StyleData["Print", "Printout"],
    CellMargins->{{54, Inherited}, {Inherited, Inherited}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Graphics"],
    CellMargins->{{62, Inherited}, {Inherited, Inherited}},
    CellGroupingRules->"GraphicsGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Graphics",
    StyleMenuListing->None],
  
  Cell[StyleData["Graphics", "Presentation"],
    CellMargins->{{74, Inherited}, {Inherited, Inherited}}],
  
  Cell[StyleData["Graphics", "Condensed"],
    CellMargins->{{52, Inherited}, {Inherited, Inherited}},
    ImageSize->{175, 175}],
  
  Cell[StyleData["Graphics", "Printout"],
    CellMargins->{{54, Inherited}, {Inherited, Inherited}},
    ImageSize->{250, 250}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["CellLabel"],
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->11,
    FontWeight->"Bold",
    FontColor->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["CellLabel", "Presentation"],
    FontSize->12],
  
  Cell[StyleData["CellLabel", "Condensed"],
    FontSize->8],
  
  Cell[StyleData["CellLabel", "Printout"],
    FontSize->8,
    FontColor->GrayLevel[0]]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Unique Styles", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Author"],
    CellMargins->{{10, 4}, {2, 10}},
    FontSize->12,
    FontSlant->"Italic"],
  
  Cell[StyleData["Author", "Presentation"],
    CellMargins->{{12, 10}, {2, 12}},
    LineSpacing->{1, 5},
    ParagraphSpacing->{0, 12},
    FontSize->16],
  
  Cell[StyleData["Author", "Condensed"],
    CellMargins->{{8, 10}, {1, 4}},
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 4},
    FontSize->9],
  
  Cell[StyleData["Author", "Printout"],
    CellMargins->{{9, 0}, {4, 12}},
    ParagraphSpacing->{0, 6}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Copyright"],
    CellMargins->{{10, 10}, {40, 2}},
    FontFamily->"Helvetica",
    FontSize->9],
  
  Cell[StyleData["Copyright", "Presentation"],
    CellMargins->{{12, 10}, {50, 2}},
    LineSpacing->{1, 5},
    FontSize->12],
  
  Cell[StyleData["Copyright", "Condensed"],
    CellMargins->{{8, 10}, {12, 1}},
    LineSpacing->{1, 2},
    FontSize->9],
  
  Cell[StyleData["Copyright", "Printout"],
    CellMargins->{{9, 0}, {72, 4}},
    FontSize->7]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Abstract"],
    CellMargins->{{45, 75}, {Inherited, 30}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Abstract", "Printout"],
    CellMargins->{{36, 67}, {Inherited, 50}},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Example"],
    CellMargins->{{12, 10}, {5, 12}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 12},
    CounterIncrements->"Example",
    FontFamily->"Times",
    FontWeight->"Bold"],
  
  Cell[StyleData["Example", "Presentation"],
    CellMargins->{{18, 10}, {8, 20}},
    LineSpacing->{1, 5},
    ParagraphSpacing->{0, 12}],
  
  Cell[StyleData["Example", "Condensed"],
    CellMargins->{{8, 10}, {4, 8}},
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 4}],
  
  Cell[StyleData["Example", "Printout"],
    CellMargins->{{9, 0}, {4, 10}},
    ParagraphSpacing->{0, 6}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Definition"],
    CellFrame->{{3, 0}, {0, 0}},
    CellMargins->{{52, 10}, {8, 8}},
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    ShowStringCharacters->True,
    CounterIncrements->"Definition",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->GrayLevel[1],
    Background->RGBColor[0.2, 0.700008, 0.700008]],
  
  Cell[StyleData["Definition", "Presentation"],
    CellMargins->{{62, Inherited}, {5, 12}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Definition", "Condensed"],
    CellMargins->{{40, 10}, {1, 4}}],
  
  Cell[StyleData["Definition", "Printout"],
    CellMargins->{{44, 0}, {2, 6}},
    Background->GrayLevel[0.6]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Theorem"],
    CellFrame->{{3, 0}, {0, 0}},
    CellMargins->{{52, 10}, {8, 8}},
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    ShowStringCharacters->True,
    CounterIncrements->"Theorem",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->GrayLevel[1],
    Background->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["Theorem", "Presentation"],
    CellMargins->{{62, Inherited}, {5, 12}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Theorem", "Condensed"],
    CellMargins->{{40, 10}, {1, 4}}],
  
  Cell[StyleData["Theorem", "Printout"],
    CellMargins->{{44, 0}, {2, 6}},
    Background->GrayLevel[0.4]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Caution"],
    CellFrame->False,
    CellDingbat->"\[WarningSign]",
    CellMargins->{{52, 25}, {8, 8}},
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    ShowStringCharacters->True,
    CounterIncrements->"Caution",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontSlant->"Plain",
    FontColor->GrayLevel[0],
    Background->RGBColor[0.996109, 0, 0],
    FontVariations->{"Underline"->False}],
  
  Cell[StyleData["Caution", "Presentation"],
    CellFrame->False,
    CellDingbat->"\[WarningSign]",
    CellMargins->{{62, Inherited}, {5, 12}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Caution", "Condensed"],
    CellFrame->False,
    CellDingbat->"\[WarningSign]",
    CellMargins->{{40, 25}, {1, 4}}],
  
  Cell[StyleData["Caution", "Printout"],
    CellFrame->False,
    CellDingbat->"\[WarningSign]",
    CellMargins->{{44, 25}, {2, 6}},
    Background->GrayLevel[0.900008]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Advice"],
    CellFrame->False,
    CellDingbat->"\[LightBulb]",
    CellMargins->{{52, 25}, {8, 8}},
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    ShowStringCharacters->True,
    CounterIncrements->"Advice",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontSlant->"Plain",
    FontColor->GrayLevel[0],
    Background->RGBColor[0, 1, 0],
    FontVariations->{"Underline"->False}],
  
  Cell[StyleData["Advice", "Presentation"],
    CellFrame->False,
    CellDingbat->"\[LightBulb]",
    CellMargins->{{62, Inherited}, {5, 12}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Advice", "Condensed"],
    CellFrame->False,
    CellDingbat->"\[LightBulb]",
    CellMargins->{{40, 25}, {1, 4}}],
  
  Cell[StyleData["Advice", "Printout"],
    CellFrame->False,
    CellDingbat->"\[LightBulb]",
    CellMargins->{{44, 25}, {2, 6}},
    Background->GrayLevel[0.900008]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Exercise"],
    CellDingbat->"\[FilledDownTriangle]",
    CellMargins->{{23, Inherited}, {4, 18}},
    CellGroupingRules->"NormalGrouping",
    PageBreakBelow->False,
    CounterIncrements->"Subsubsection",
    FontFamily->"Times",
    FontSize->13,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 0.996109]],
  
  Cell[StyleData["Exercise", "Presentation"],
    CellMargins->{{33, 10}, {8, 26}},
    LineSpacing->{1, 0},
    FontSize->18],
  
  Cell[StyleData["Exercise", "Condensed"],
    CellMargins->{{9, 0}, {6, 20}},
    FontSize->11],
  
  Cell[StyleData["Exercise", "Printout"],
    CellFrame->False,
    CellDingbat->"\[FilledDownTriangle]",
    CellMargins->{{9, 0}, {6, 20}},
    FontSize->11,
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ExerciseMain"],
    CellFrame->{{6, 0}, {0, 1}},
    CellMargins->{{12, Inherited}, {4, 20}},
    CellGroupingRules->{"SectionGrouping", 30},
    PageBreakBelow->False,
    CellFrameColor->RGBColor[0.571389, 0.19675, 0.570504],
    CounterIncrements->"Subsection",
    CounterAssignments->{{"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->16,
    FontWeight->"Bold",
    FontColor->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["ExerciseMain", "Presentation"],
    CellMargins->{{18, 10}, {8, 32}},
    LineSpacing->{1, 2},
    FontSize->24,
    FontTracking->"Condensed"],
  
  Cell[StyleData["ExerciseMain", "Condensed"],
    CellMargins->{{9, 0}, {2, 50}},
    FontSize->14],
  
  Cell[StyleData["ExerciseMain", "Printout"],
    CellMargins->{{9, 0}, {2, 50}},
    CellFrameColor->GrayLevel[0.500008],
    FontSize->14,
    FontTracking->"Plain",
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ExerciseTitle"],
    CellDingbat->"\[FilledDownTriangle]",
    CellMargins->{{23, Inherited}, {4, 18}},
    CellGroupingRules->{"SectionGrouping", 40},
    PageBreakBelow->False,
    CounterIncrements->"Subsection",
    CounterAssignments->{{"Subsubsection", 0}},
    FontFamily->"Times",
    FontSize->13,
    FontWeight->"Bold",
    FontColor->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["ExerciseTitle", "Presentation"],
    CellMargins->{{33, 10}, {8, 26}},
    LineSpacing->{1, 0},
    FontSize->18],
  
  Cell[StyleData["ExerciseTitle", "Condensed"],
    CellDingbat->"\[FilledDownTriangle]",
    CellMargins->{{9, 0}, {6, 20}},
    FontSize->11],
  
  Cell[StyleData["ExerciseTitle", "Printout"],
    CellDingbat->"\[FilledDownTriangle]",
    CellMargins->{{9, 0}, {6, 20}},
    FontSize->11,
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ExerciseText"],
    CellMargins->{{32, 10}, {5, 5}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 8},
    CounterIncrements->"ExcersiceText",
    FontFamily->"Times"],
  
  Cell[StyleData["ExerciseText", "Presentation"],
    CellMargins->{{32, 10}, {8, 8}},
    LineSpacing->{1, 5},
    ParagraphSpacing->{0, 12}],
  
  Cell[StyleData["ExerciseText", "Condensed"],
    CellMargins->{{32, 0}, {4, 4}},
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 6}],
  
  Cell[StyleData["ExerciseText", "Printout"],
    CellMargins->{{32, 0}, {4, 4}},
    ParagraphSpacing->{0, 6}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Problem"],
    CellDingbat->"\[FilledDownTriangle]",
    CellMargins->{{23, Inherited}, {4, 18}},
    CellGroupingRules->{"SectionGrouping", 40},
    PageBreakBelow->False,
    CounterIncrements->"Subsection",
    CounterAssignments->{{"Subsubsection", 0}},
    FontFamily->"Times",
    FontSize->13,
    FontWeight->"Plain",
    FontColor->GrayLevel[0]],
  
  Cell[StyleData["Problem", "Presentation"],
    CellMargins->{{33, 10}, {8, 26}},
    LineSpacing->{1, 0},
    FontSize->18],
  
  Cell[StyleData["Problem", "Condensed"],
    CellMargins->{{9, 0}, {6, 20}},
    FontSize->11],
  
  Cell[StyleData["Problem", "Printout"],
    CellFrame->False,
    CellDingbat->"\[FilledDownTriangle]",
    CellMargins->{{9, 0}, {6, 20}},
    FontSize->11,
    FontColor->GrayLevel[0]]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Formulas and Programming", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["DisplayFormula"],
    CellMargins->{{62, 10}, {2, 10}},
    CellHorizontalScrolling->True,
    DefaultFormatType->DefaultInputFormatType,
    ScriptLevel->0,
    SingleLetterItalics->True,
    UnderoverscriptBoxOptions->{LimitsPositioning->True}],
  
  Cell[StyleData["DisplayFormula", "Presentation"],
    CellMargins->{{74, 10}, {2, 10}}],
  
  Cell[StyleData["DisplayFormula", "Condensed"],
    CellMargins->{{52, 10}, {2, 10}}],
  
  Cell[StyleData["DisplayFormula", "Printout"],
    CellMargins->{{54, 10}, {2, 10}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ChemicalFormula"],
    CellMargins->{{62, 10}, {2, 10}},
    DefaultFormatType->DefaultInputFormatType,
    AutoSpacing->False,
    ScriptLevel->1,
    ScriptBaselineShifts->{0.6, Automatic},
    SingleLetterItalics->False,
    ZeroWidthTimes->True],
  
  Cell[StyleData["ChemicalFormula", "Presentation"],
    CellMargins->{{74, 10}, {2, 10}},
    FontSize->10],
  
  Cell[StyleData["ChemicalFormula", "Condensed"],
    CellMargins->{{52, 10}, {2, 10}},
    FontSize->10],
  
  Cell[StyleData["ChemicalFormula", "Printout"],
    CellMargins->{{54, 10}, {2, 10}},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Program"],
    CellMargins->{{12, 10}, {Inherited, 6}},
    FontFamily->"Courier"],
  
  Cell[StyleData["Program", "Presentation"],
    CellMargins->{{13, 30}, {Inherited, 4}},
    FontSize->9.5],
  
  Cell[StyleData["Program", "Condensed"],
    CellMargins->{{8, 10}, {Inherited, 4}},
    FontSize->9.5],
  
  Cell[StyleData["Program", "Printout"],
    CellMargins->{{9, 0}, {Inherited, 4}},
    FontSize->9.5]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Automatic Numbering", "Section"],
  
  Cell["\<\
The following styles are useful for numbered equations, figures, etc.  They \
automatically give the cell a FrameLabel containing a reference to a \
particular counter, and also increment that counter.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["NumberedEquation"],
    CellMargins->{{62, 10}, {Inherited, Inherited}},
    CellFrameLabels->{{None, Cell[ 
            TextData[ {"(", 
              CounterBox[ "NumberedEquation"], ")"}]]}, {None, None}},
    DefaultFormatType->DefaultInputFormatType,
    CounterIncrements->"NumberedEquation",
    FormatTypeAutoConvert->False],
  
  Cell[StyleData["NumberedEquation", "Presentation"],
    CellMargins->{{74, 10}, {Inherited, Inherited}}],
  
  Cell[StyleData["NumberedEquation", "Condensed"],
    CellMargins->{{52, 10}, {Inherited, Inherited}}],
  
  Cell[StyleData["NumberedEquation", "Printout"],
    CellMargins->{{54, 0}, {Inherited, Inherited}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["NumberedFigure"],
    CellMargins->{{62, 145}, {Inherited, Inherited}},
    CellFrameLabels->{{None, None}, {Cell[ 
            TextData[ {"Figure ", 
              CounterBox[ "NumberedFigure"]}]], None}},
    CounterIncrements->"NumberedFigure",
    ImageMargins->{{43, Inherited}, {Inherited, 0}},
    FormatTypeAutoConvert->False],
  
  Cell[StyleData["NumberedFigure", "Presentation"]],
  
  Cell[StyleData["NumberedFigure", "Condensed"]],
  
  Cell[StyleData["NumberedFigure", "Printout"]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["NumberedTable"],
    CellMargins->{{62, 145}, {Inherited, Inherited}},
    CellFrameLabels->{{None, None}, {Cell[ 
            TextData[ {"Table ", 
              CounterBox[ "NumberedTable"]}]], None}},
    TextAlignment->Center,
    CounterIncrements->"NumberedTable",
    FormatTypeAutoConvert->False],
  
  Cell[StyleData["NumberedTable", "Presentation"]],
  
  Cell[StyleData["NumberedTable", "Condensed"]],
  
  Cell[StyleData["NumberedTable", "Printout"]]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Headers and Footers", "Section"],
  
  Cell[StyleData["Header"],
    CellMargins->{{0, 0}, {4, 1}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->9,
    FontSlant->"Italic"],
  
  Cell[StyleData["Footer"],
    CellMargins->{{0, 0}, {0, 4}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->9,
    FontSlant->"Italic"],
  
  Cell[StyleData["PageNumber"],
    CellMargins->{{0, 0}, {4, 1}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->9,
    FontWeight->"Bold"]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Hyperlink Styles", "Section"],
  
  Cell["\<\
The cells below define styles useful for making hypertext ButtonBoxes.  The \
\"Hyperlink\" style is for links within the same Notebook, or between \
Notebooks.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Hyperlink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontWeight->"Bold",
    FontSlant->"Italic",
    FontColor->RGBColor[0, 0, 1],
    Background->None,
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`NotebookLocate[ #2]}]&),
    Active->True,
    ButtonFrame->"None",
    ButtonNote->ButtonData}],
  
  Cell[StyleData["Hyperlink", "Presentation"]],
  
  Cell[StyleData["Hyperlink", "Condensed"]],
  
  Cell[StyleData["Hyperlink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1],
    FontVariations->{"Underline"->True}]
  }, Closed]],
  
  Cell["\<\
The following styles are for linking automatically to the on-line help \
system.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["MainBookLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontWeight->"Bold",
    FontSlant->"Plain",
    FontColor->RGBColor[0, 0, 1],
    Background->None,
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "MainBook", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["MainBookLink", "Presentation"]],
  
  Cell[StyleData["MainBookLink", "Condensed"]],
  
  Cell[StyleData["MainBookLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1],
    FontVariations->{"Underline"->True}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["AddOnsLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontFamily->"Courier",
    FontWeight->"Bold",
    FontSlant->"Plain",
    FontColor->RGBColor[0, 0, 1],
    Background->None,
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "AddOns", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["AddOnsLink", "Presentation"]],
  
  Cell[StyleData["AddOnsLink", "Condensed"]],
  
  Cell[StyleData["AddOnLink", "Printout"],
    FontWeight->"Bold",
    FontColor->GrayLevel[0],
    Background->GrayLevel[1],
    FontVariations->{"Underline"->True}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["RefGuideLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontFamily->"Courier",
    FontWeight->"Bold",
    FontSlant->"Plain",
    FontColor->RGBColor[0, 0, 1],
    Background->None,
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "RefGuideLink", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["RefGuideLink", "Presentation"]],
  
  Cell[StyleData["RefGuideLink", "Condensed"]],
  
  Cell[StyleData["RefGuideLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1],
    FontVariations->{"Underline"->True}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["GettingStartedLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontWeight->"Bold",
    FontSlant->"Italic",
    FontColor->RGBColor[0, 0, 1],
    Background->None,
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "GettingStarted", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["GettingStartedLink", "Presentation"]],
  
  Cell[StyleData["GettingStartedLink", "Condensed"]],
  
  Cell[StyleData["GettingStartedLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1],
    FontVariations->{"Underline"->True}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["OtherInformationLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontWeight->"Bold",
    FontSlant->"Italic",
    FontColor->RGBColor[0, 0, 1],
    Background->None,
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "OtherInformation", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["OtherInformationLink", "Presentation"]],
  
  Cell[StyleData["OtherInformationLink", "Condensed"]],
  
  Cell[StyleData["OtherInformationLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1],
    FontVariations->{"Underline"->True}]
  }, Closed]]
  }, Closed]]
  }, Open  ]]
  }]
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 112, 2, 81, "Title"],
Cell[1891, 57, 1085, 17, 191, "Abstract"],

Cell[CellGroupData[{
Cell[3001, 78, 33, 0, 83, "Section"],

Cell[CellGroupData[{
Cell[3059, 82, 43, 0, 68, "Subsection"],
Cell[3105, 84, 160, 3, 125, "Input"],
Cell[3268, 89, 115, 2, 73, "Input"],
Cell[3386, 93, 116, 2, 99, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[3551, 101, 31, 0, 51, "Section"],
Cell[3585, 103, 606, 10, 93, "Text"],
Cell[4194, 115, 403, 11, 62, "DisplayFormula"],
Cell[4600, 128, 393, 15, 41, "Text"],
Cell[4996, 145, 211, 4, 90, "DisplayFormula"],
Cell[5210, 151, 373, 10, 67, "Text"],
Cell[5586, 163, 224, 4, 62, "DisplayFormula"],
Cell[5813, 169, 358, 8, 67, "Text"],
Cell[6174, 179, 124, 2, 135, "Input"],

Cell[CellGroupData[{
Cell[6323, 185, 241, 4, 125, "Input"],
Cell[6567, 191, 76, 1, 59, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[6680, 197, 228, 3, 99, "Input"],
Cell[6911, 202, 68612, 1648, 279, 21913, 1067, "GraphicsData", "PostScript", \
"Graphics"]
}, Closed]],
Cell[75538, 1853, 499, 17, 42, "Text"],
Cell[76040, 1872, 522, 10, 151, "Input"],

Cell[CellGroupData[{
Cell[76587, 1886, 143, 2, 99, "Input"],
Cell[76733, 1890, 64808, 1870, 279, 25227, 1377, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[141556, 3763, 1130, 30, 94, "Text"],
Cell[142689, 3795, 275, 8, 42, "Text"],

Cell[CellGroupData[{
Cell[142989, 3807, 636, 13, 151, "Input"],
Cell[143628, 3822, 79, 1, 59, "Output"]
}, Closed]],
Cell[143722, 3826, 302, 5, 67, "Text"],

Cell[CellGroupData[{
Cell[144049, 3835, 531, 11, 125, "Input"],
Cell[144583, 3848, 41550, 1022, 279, 13569, 672, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[186148, 4873, 226, 3, 41, "Text"],
Cell[186377, 4878, 169, 3, 41, "Text"],
Cell[186549, 4883, 209, 5, 43, "DisplayFormula"],
Cell[186761, 4890, 1390, 31, 171, "Text"],

Cell[CellGroupData[{
Cell[188176, 4925, 1338, 25, 229, "Input"],
Cell[189517, 4952, 77329, 1946, 279, 26215, 1310, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[266861, 6901, 260, 5, 56, "Caption"],
Cell[267124, 6908, 88, 2, 41, "Text"],
Cell[267215, 6912, 203, 4, 68, "DisplayFormula"],
Cell[267421, 6918, 1055, 36, 67, "Text"],
Cell[268479, 6956, 669, 10, 119, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[269185, 6971, 33, 0, 51, "Section"],

Cell[CellGroupData[{
Cell[269243, 6975, 47, 0, 68, "Subsection"],
Cell[269293, 6977, 492, 17, 41, "Text"],
Cell[269788, 6996, 1293, 24, 367, "Input"],
Cell[271084, 7022, 197, 4, 73, "Input"],

Cell[CellGroupData[{
Cell[271306, 7030, 79, 1, 73, "Input"],
Cell[271388, 7033, 90274, 1980, 279, 21478, 1126, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[361711, 9019, 49, 0, 68, "Subsection"],
Cell[361763, 9021, 671, 23, 67, "Text"],
Cell[362437, 9046, 986, 15, 286, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[363460, 9066, 50, 0, 68, "Subsection"],
Cell[363513, 9068, 620, 21, 68, "Text"],
Cell[364136, 9091, 2168, 38, 550, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[366341, 9134, 41, 0, 68, "Subsection"],
Cell[366385, 9136, 110, 3, 41, "Text"],
Cell[366498, 9141, 752, 13, 229, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[367299, 9160, 49, 0, 51, "Section"],
Cell[367351, 9162, 1344, 24, 177, "Text"],

Cell[CellGroupData[{
Cell[368720, 9190, 347, 6, 99, "Input"],
Cell[369070, 9198, 137724, 2639, 366, 25262, 1246, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[506809, 11840, 803, 18, 94, "Text"],
Cell[507615, 11860, 223, 5, 125, "Input"],
Cell[507841, 11867, 199, 4, 73, "Input"],

Cell[CellGroupData[{
Cell[508065, 11875, 478, 8, 138, "Input"],
Cell[508546, 11885, 143046, 3331, 379, 39876, 2053, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[651641, 15222, 58, 0, 51, "Section"],
Cell[651702, 15224, 125, 3, 41, "Text"],

Cell[CellGroupData[{
Cell[651852, 15231, 248, 4, 99, "Input"],
Cell[652103, 15237, 95195, 2929, 279, 46791, 2327, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[747313, 18169, 1227, 22, 171, "Text"],
Cell[748543, 18193, 141, 3, 41, "Text"],

Cell[CellGroupData[{
Cell[748709, 18200, 71, 1, 73, "Input"],
Cell[748783, 18203, 553, 12, 61, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[749373, 18220, 61, 1, 73, "Input"],
Cell[749437, 18223, 220, 3, 61, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[749694, 18231, 142, 2, 73, "Input"],
Cell[749839, 18235, 54521, 1279, 279, 16446, 804, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],

Cell[CellGroupData[{
Cell[804397, 19519, 76, 1, 73, "Input"],
Cell[804476, 19522, 52, 1, 59, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[804565, 19528, 58, 1, 78, "Input"],
Cell[804626, 19531, 64, 1, 92, "Output"]
}, Closed]],
Cell[804705, 19535, 103, 2, 41, "Text"],

Cell[CellGroupData[{
Cell[804833, 19541, 85, 1, 96, "Input"],
Cell[804921, 19544, 138, 2, 92, "Output"]
}, Closed]],
Cell[805074, 19549, 241, 8, 41, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[805352, 19562, 33, 0, 51, "Section"],

Cell[CellGroupData[{
Cell[805410, 19566, 38, 0, 68, "Subsection"],
Cell[805451, 19568, 197, 6, 46, "Text"],

Cell[CellGroupData[{
Cell[805673, 19578, 87, 1, 96, "Input"],
Cell[805763, 19581, 74, 1, 59, "Output"]
}, Closed]],
Cell[805852, 19585, 779, 16, 119, "Text"],

Cell[CellGroupData[{
Cell[806656, 19605, 318, 5, 99, "Input"],
Cell[806977, 19612, 99108, 2249, 348, 27143, 1356, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[906100, 21864, 553, 10, 94, "Text"],
Cell[906656, 21876, 480, 8, 177, "Input"],
Cell[907139, 21886, 865, 17, 177, "Input"],

Cell[CellGroupData[{
Cell[908029, 21907, 115, 2, 73, "Input"],
Cell[908147, 21911, 275371, 7197, 422, 101413, 5045, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],

Cell[CellGroupData[{
Cell[1183555, 29113, 107, 2, 73, "Input"],
Cell[1183665, 29117, 94846, 2045, 279, 21688, 1137, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1278560, 31168, 37, 0, 68, "Subsection"],
Cell[1278600, 31170, 392, 8, 67, "Text"],

Cell[CellGroupData[{
Cell[1279017, 31182, 350, 6, 99, "Input"],
Cell[1279370, 31190, 122370, 2499, 343, 25311, 1296, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],

Cell[CellGroupData[{
Cell[1401777, 33694, 228, 5, 99, "Input"],
Cell[1402008, 33701, 115587, 2501, 279, 28373, 1420, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1517656, 36209, 32, 0, 51, "Section"],
Cell[1517691, 36211, 525, 10, 93, "Text"],

Cell[CellGroupData[{
Cell[1518241, 36225, 67, 1, 73, "Input"],
Cell[1518311, 36228, 52, 1, 59, "Output"]
}, Closed]],
Cell[1518378, 36232, 204, 4, 41, "Text"],

Cell[CellGroupData[{
Cell[1518607, 36240, 350, 6, 99, "Input"],
Cell[1518960, 36248, 67027, 1908, 279, 27084, 1410, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[1586002, 38159, 146, 3, 41, "Text"],

Cell[CellGroupData[{
Cell[1586173, 38166, 358, 6, 99, "Input"],
Cell[1586534, 38174, 88894, 2200, 316, 27824, 1442, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[1675443, 40377, 772, 13, 120, "Text"],

Cell[CellGroupData[{
Cell[1676240, 40394, 379, 7, 99, "Input"],
Cell[1676622, 40403, 96222, 2044, 337, 22398, 1128, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[1772859, 42450, 241, 6, 42, "Text"],

Cell[CellGroupData[{
Cell[1773125, 42460, 215, 4, 99, "Input"],
Cell[1773343, 42466, 222, 3, 59, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1773602, 42474, 58, 1, 73, "Input"],
Cell[1773663, 42477, 75240, 1994, 279, 27679, 1402, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[1848918, 44474, 116, 3, 41, "Text"],

Cell[CellGroupData[{
Cell[1849059, 44481, 195, 4, 73, "Input"],
Cell[1849257, 44487, 218, 4, 59, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[1849512, 44496, 62, 1, 73, "Input"],
Cell[1849577, 44499, 90748, 2213, 279, 27971, 1433, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[1940340, 46715, 141, 3, 41, "Text"],

Cell[CellGroupData[{
Cell[1940506, 46722, 77, 1, 73, "Input"],
Cell[1940586, 46725, 99, 2, 61, "Output"]
}, Closed]],
Cell[1940700, 46730, 165, 3, 41, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[1940902, 46738, 33, 0, 54, "ExerciseMain"],

Cell[CellGroupData[{
Cell[1940960, 46742, 43, 0, 58, "ExerciseTitle"],
Cell[1941006, 46744, 113, 3, 41, "ExerciseText"],
Cell[1941122, 46749, 216, 3, 65, "DisplayFormula"],
Cell[1941341, 46754, 62, 0, 41, "ExerciseText"],
Cell[1941406, 46756, 157, 2, 43, "DisplayFormula"],
Cell[1941566, 46760, 52, 0, 41, "ExerciseText"],
Cell[1941621, 46762, 410, 7, 62, "DisplayFormula"],
Cell[1942034, 46771, 230, 8, 41, "ExerciseText"],
Cell[1942267, 46781, 164, 3, 44, "DisplayFormula"],
Cell[1942434, 46786, 101, 2, 41, "ExerciseText"],
Cell[1942538, 46790, 426, 7, 74, "DisplayFormula"],
Cell[1942967, 46799, 812, 23, 68, "ExerciseText"],
Cell[1943782, 46824, 176, 3, 63, "DisplayFormula"],
Cell[1943961, 46829, 291, 8, 42, "ExerciseText"],
Cell[1944255, 46839, 266, 6, 41, "ExerciseText"],
Cell[1944524, 46847, 451, 7, 93, "DisplayFormula"],
Cell[1944978, 46856, 530, 17, 41, "ExerciseText"],
Cell[1945511, 46875, 190, 4, 41, "ExerciseText"],
Cell[1945704, 46881, 433, 8, 75, "DisplayFormula"],
Cell[1946140, 46891, 622, 19, 67, "ExerciseText"]
}, Closed]],

Cell[CellGroupData[{
Cell[1946799, 46915, 44, 0, 58, "ExerciseTitle"],
Cell[1946846, 46917, 216, 4, 67, "ExerciseText"],
Cell[1947065, 46923, 355, 7, 68, "DisplayFormula"],
Cell[1947423, 46932, 1704, 55, 119, "ExerciseText"],
Cell[1949130, 46989, 487, 11, 88, "DisplayFormula"],
Cell[1949620, 47002, 1442, 28, 176, "ExerciseText"],
Cell[1951065, 47032, 60864, 1622, 398, 28690, 1220, "GraphicsData", \
"PostScript", "Graphics"],
Cell[2011932, 48656, 445, 7, 93, "ExerciseText"]
}, Closed]],

Cell[CellGroupData[{
Cell[2012414, 48668, 36, 0, 58, "ExerciseTitle"],
Cell[2012453, 48670, 74, 0, 41, "ExerciseText"],
Cell[2012530, 48672, 153, 3, 65, "DisplayFormula"],
Cell[2012686, 48677, 118, 3, 41, "ExerciseText"],
Cell[2012807, 48682, 403, 12, 41, "ExerciseText"],
Cell[2013213, 48696, 822, 23, 93, "ExerciseText"],
Cell[2014038, 48721, 336, 11, 41, "ExerciseText"]
}, Closed]],

Cell[CellGroupData[{
Cell[2014411, 48737, 44, 0, 58, "ExerciseTitle"],
Cell[2014458, 48739, 526, 12, 95, "ExerciseText"],
Cell[2014987, 48753, 755, 13, 124, "ExerciseText"],
Cell[2015745, 48768, 393, 6, 93, "ExerciseText"],
Cell[2016141, 48776, 143, 3, 41, "ExerciseText"]
}, Closed]]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

