(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.2'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[   2786764,      64649]*)
(*NotebookOutlinePosition[   2820709,      65873]*)
(*  CellTagsIndexPosition[   2820665,      65869]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["PDE Relaxation", "Title",
  TextAlignment->Center,
  TextJustification->0],

Cell[TextData[{
  "from ",
  StyleBox[ButtonBox["Essential Mathematica for Students of Science",
    Active->False,
    ButtonData:>{
      URL[ 
      "http://www.physics.umd.edu/courses/CourseWare/EssentialMathematica/"], 
      None},
    Active->False,
    ButtonStyle->"Hyperlink"],
    Active->False],
  "\n\[Copyright] James J. Kelly, 1998\nLast revised: January, 2007"
}], "Author",
  Active->True,
  TextAlignment->Center,
  TextJustification->0],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell["\<\
Partial differential equations (PDE) are usually much more difficult to solve \
than ordinary differential equations and there has accumulated a vast \
literature on many different symbolic and numerical techniques for attacking \
various classes of PDEs.  We obviously cannot review and compare these \
techniques in a course that covers other topics as well.  For our purposes it \
is sufficient to discuss one simple technique that is applicable to many of \
the PDEs encountered in physics.\
\>", "Text"],

Cell["\<\
Rather than attempt a general discussion, we consider the specific problem of \
Laplace's equation\
\>", "Text"],

Cell[BoxData[
    \(TraditionalForm\`\[PartialD]\^2 \[Phi]\/\[PartialD]x\^2 + \
\[PartialD]\^2 \[Phi]\/\[PartialD]y\^2 \[Equal] 0\)], "DisplayFormula",
  TextAlignment->Left,
  TextJustification->0],

Cell[TextData[{
  "with ",
  Cell[BoxData[
      \(TraditionalForm\`\[Phi][x, y]\)]],
  " specified on a set of internal surfaces and the boundary to a \
two-dimensional region.  We will attempt to solve the problem using a uniform \
rectangular grid"
}], "Text"],

Cell[BoxData[{
    \(TraditionalForm\`x\_i = x\_0 + i\ \[Delta]\), "\n", 
    \(TraditionalForm\`y\_j = y\_0 + j\ \[Delta]\), "\n", 
    \(TraditionalForm\`\[Phi]\_\(i\[InvisibleComma]j\) = \[Phi][x\_i, 
        y\_j]\)}], "DisplayFormula",
  TextAlignment->Left,
  TextJustification->0],

Cell["\<\
where \[Delta] is the grid spacing.  To discretize the equation we need to \
express second derivatives in terms of finite differences.  The first \
derivative is represented by the discrete rule:\
\>", "Text"],

Cell[BoxData[
    \(\(rule1 = 
        f\_i_ \[Rule] \(f\_\(i + 1\) - f\_\(i - 1\)\)\/\(2  \[Delta]\);\)\)], \
"Input"],

Cell["Applying this rule twice,", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(f\_i /. rule1\) /. rule1 // Simplify\)], "Input"],

Cell[BoxData[
    \(\(f\_\(\(-2\) + i\) - 2\ f\_i + f\_\(2 + i\)\)\/\(4\ \[Delta]\^2\)\)], \
"Output"]
}, Closed]],

Cell[TextData[{
  "we find that the second derivative is represented by a simple formula \
based upon a step size ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta] \[Rule] 2  \[Delta]\)]],
  ".  Notice that the Laplacian only relates grid points whose indices differ \
by even numbers.  Hence, the discrete Laplacian operator in two dimensions is \
represented as"
}], "Text"],

Cell[BoxData[
    \(DiscreteLaplacian2D[\[Phi]_, i_, 
        j_] := \(\[Phi]\[LeftDoubleBracket]i + 1, j\[RightDoubleBracket] + \
\[Phi]\[LeftDoubleBracket]i - 1, j\[RightDoubleBracket] + \[Phi]\
\[LeftDoubleBracket]i, j + 1\[RightDoubleBracket] + \
\[Phi]\[LeftDoubleBracket]i, j - 1\[RightDoubleBracket]\)\/4 - \[Phi]\
\[LeftDoubleBracket]i, j\[RightDoubleBracket]\)], "Input"],

Cell["\<\
where the grid has been rescaled.  Therefore, the discrete Laplacian at a \
particular grid point is represented by the difference between the value of \
the function at that point and the average of its values at the four nearest \
grid points.  In this sense one can describe a solution to Laplace's equation \
as the function which interpolates between boundary values as smoothly as \
possible.\
\>", "Text"],

Cell[TextData[{
  "A simple algorithm, called the ",
  StyleBox["relaxation method",
    FontSlant->"Italic"],
  ", now presents itself.  We start by establishing an array ",
  StyleBox["\[Phi]\[LeftDoubleBracket]i,j\[RightDoubleBracket]", 
    "InlineInput"],
  " that satisfies the boundary conditions and contains reasonable guesses \
for the function at grid points not on the boundary surfaces.  We then \
systematically replace each point outside the boundary surfaces by the \
average of its four nearest neighbors.  This process is repeated until the \
solution converges to acceptable accuracy.  As the local averaging procedure \
is iterated, errors in the initial guesses are reduced as the solution \
gradually relaxes toward the correct solution to the discretized version of \
Laplace's equation.  Obviously, the smaller the grid spacing and the greater \
the number of grid points the more accurately will the discretized solution \
represent the continuous solution, at a cost in computation time proportional \
to ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\^\(-2\)\)]],
  ".  Errors in the initial solution can be described as tension in the \
proposed solution.  As the iteration proceeds, these errors are smoothed out \
and the function relaxes toward the true solution.  Corrections diffuse \
through the grid slowly, spreading to nearest neighbors in each iteration."
}], "Text"],

Cell[TextData[{
  "Unfortunately, like most simple algorithms, the direct relaxation method \
converges too slowly to be of much practical utility.  Fortunately, a \
relatively simple modification improves the performance of the relaxation \
method considerably.  The trick is to anticipate the changes in the local \
average that will result from changes to neighboring values during the \
relaxation process and thus to ",
  StyleBox["overcorrect",
    FontSlant->"Italic"],
  " each point.  The resulting algorithm is known as ",
  StyleBox["simultaneous over-relaxation",
    FontSlant->"Italic"],
  " (SOR).  One can show that for the optimal choice of the over-relaxation \
parameter, \[Omega], the computation time becomes linear in ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\^\(-1\)\)]],
  "; hence, the SOR is faster by about a factor of 100 for a typical 100\
\[Times]100 grid.  The basic SOR algorithm is now"
}], "Text"],

Cell[BoxData[
    RowBox[{
      StyleBox[\(\[Phi]\[LeftDoubleBracket]i, j\[RightDoubleBracket]\),
        "InlineInput"], 
      StyleBox["\[Rule]",
        "InlineInput"], \(\[Phi]\[LeftDoubleBracket]i, 
          j\[RightDoubleBracket] + \[Omega]\ DiscreteLaplacian[\[Phi], i, 
            j]\)}]], "InlineInput",
  TextAlignment->Center,
  TextJustification->0],

Cell["\<\
where \[Omega] is adjusted as the iteration progresses.  The primary \
difficulty in implementing the SOR algorithm is optimization of the \
over-relaxation parameter \[LongDash] the gain in efficiency is realized only \
when \[Omega] is rather close to its optimal value.\
\>", "Text"],

Cell["The over-relaxation parameter is based upon a quantity", "Text"],

Cell[BoxData[
    \(\(\(\t\)\(\[Rho]\_Jacobi = 
      1 - \[Pi]\^2\/\(2  n\^2\)\)\)\)], "InlineInput",
  TextAlignment->Center,
  TextJustification->0],

Cell[TextData[{
  "known in PDE theory as the spectral radius of Jacobi iteration (a simpler \
algorithm), where ",
  Cell[BoxData[
      \(TraditionalForm\`n\[Times]n\)]],
  " is the dimensionality of the grid (assumed square).  Rather than process \
the grid points sequentially, it is more efficient to divide each iteration \
into two halves, adjusting the even points first followed by the odd points.  \
For the first half-iteration, we use ",
  Cell[BoxData[
      \(TraditionalForm\`\[Omega] = 1\)]],
  ".  For the second half of the first iteration, we use ",
  Cell[BoxData[
      \(TraditionalForm\`\[Omega] \[Rule] 1/\((1 - \[Rho]\^2/2)\)\)]],
  ".  For all subsequent iterations, ",
  Cell[BoxData[
      \(TraditionalForm\`\[Omega] \[Rule] 
        1/\((1 - \(\[Rho]\^2\) \[Omega]/4)\)\)]],
  ".  One can then show that as the iteration proceeds, the over-relaxation \
parameter approaches the optimal value"
}], "Text"],

Cell[BoxData[{
    \(\[Omega] \[Rule] 
      2\/\(\(1 + \@\(1 - \[Rho]\_Jacobi\%2\)\)\(\t\)\)\), \
"\[IndentingNewLine]", 
    \(\[Rho]\_SOR = \((\[Rho]\_Jacobi\/\(1 + \@\(1 - \[Rho]\_Jacobi\%2\)\))\)\
\^2\)}], "InlineInput",
  TextAlignment->Center,
  TextJustification->0],

Cell[TextData[{
  "This strategy is known as ",
  StyleBox["Chebyshev acceleration",
    FontSlant->"Italic"],
  " of the SOR algorithm.  For further details about the SOR method and \
references to original literature see Press ",
  StyleBox["et al., Numerical Recipes.",
    FontSlant->"Italic"]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Initialization", "Section"],

Cell[BoxData[{
    \(\(ClearAll["\<Global`*\>"];\)\), "\n", 
    \(\(Off[General::spell, \ General::spell1];\)\), "\[IndentingNewLine]", 
    \(\($TextStyle = {FontFamily \[Rule] "\<Times\>", 
          FontSize \[Rule] 12};\)\), "\[IndentingNewLine]", 
    \(Needs["\<Graphics`\>"]\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Implementation", "Section"],

Cell["\<\
We assume that the problem is formulated on a square array and that the \
boundary surfaces are parallel to the coordinate axes.  We must supply an \
array of initial values for the potential function and an array of True\
\[VerticalSeparator]False values which specify the points that lie on the \
boundary \[LongDash] the potential for points on the boundary does not \
change.  The relaxation algorithm is iterated until either the maximum number \
of iterations is exceeded or until the root-mean-square Laplacian \
\>", "Text"],

Cell[BoxData[
    \(\[Sqrt]\((\(1\/n\) \(\[Sum]\((\[CurlyPhi]\[LeftDoubleBracket]i, j\
\[RightDoubleBracket] - \((\[CurlyPhi]\[LeftDoubleBracket]i + 1, j\
\[RightDoubleBracket] + \[CurlyPhi]\[LeftDoubleBracket]i - 1, j\
\[RightDoubleBracket] + \[CurlyPhi]\[LeftDoubleBracket]i, j + 1\
\[RightDoubleBracket] + \[CurlyPhi]\[LeftDoubleBracket]i, j - 1\
\[RightDoubleBracket])\)/4. )\)\^2\))\) < conv\)], "InlineInput",
  TextAlignment->Center,
  TextJustification->0],

Cell["falls below a specified convergence criterion.", "Text"],

Cell[BoxData[{
    \(\(\(Clear[relax];\)\(\n\)
    \)\), "\n", 
    \(relax[\[Phi]_, boundary_, nmax_, conv_] := 
      Module[{\[CurlyPhi], n, m, \[Rho], \[Omega], \[Xi], niter, 
          test}, \[IndentingNewLine]{n, m} = Dimensions[\[Phi]]; \[Rho] = 
          1.  - \[Pi]\^2\/\(2. \ n\^2\); \[Omega] = 
          1. ; \[CurlyPhi] = \[Phi]; niter = 0; 
        test = 100. ; \[IndentingNewLine]While[
          test > conv\  && \ niter < 2  nmax, test = 0. ; 
          Do[If[Not[boundary\[LeftDoubleBracket]i, j\[RightDoubleBracket]] && 
                Mod[i + j, 2] \[Equal] 
                  Mod[niter, 2], \[Xi] = \[CurlyPhi]\[LeftDoubleBracket]i, 
                    j\[RightDoubleBracket] - \((\[CurlyPhi]\
\[LeftDoubleBracket]i + 1, 
                          j\[RightDoubleBracket] + \[CurlyPhi]\
\[LeftDoubleBracket]i - 1, 
                          j\[RightDoubleBracket] + \[CurlyPhi]\
\[LeftDoubleBracket]i, j + 
                            1\[RightDoubleBracket] + \[CurlyPhi]\
\[LeftDoubleBracket]i, j - 1\[RightDoubleBracket])\)/4. ; 
              test = test + \[Xi]\^2; \[CurlyPhi]\[LeftDoubleBracket]i, 
                  j\[RightDoubleBracket]\  = \[CurlyPhi]\[LeftDoubleBracket]
                    i, j\[RightDoubleBracket] - \[Omega]\ \[Xi]], {i, 2, 
              n - 1}, {j, 2, m - 1}]; niter = niter + 1; 
          If[niter \[Equal] 1, \[Omega] = 
              1. /\((1.  - \[Rho]\^2/2. )\), \[Omega] = 
              1. /\((1.  - \(\[Rho]\^2\) \[Omega]/4. )\)]; 
          test = \@\(test\/n\)]; 
        Print["\<number of iterations = \>", niter/2]; 
        Print["\<residual = \>", \ test]; Return[\[CurlyPhi]]]\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Example: parallel-plate capacitor in two dimensions", "Section"],

Cell["\<\
The electric field is constant inside an infinite parallel-plate capacitor \
and vanishes outside.  The field for a capacitor of finite dimensions is more \
complicated, bowing outward near the edges and connecting the outer edges \
with wide, somewhat circular, arcs.  For simplicity we consider a finite \
parallel-plate capacitor in two dimensions, assuming that it remains very \
long in the third dimension.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Initialize potential", "Subsection"],

Cell["\<\
First, we establish 100\[Times]100 grids for the initial potential and the \
boundary specification.\
\>", "Text"],

Cell[BoxData[{
    \(\(Clear[boundary, \[Phi], \[Phi]0];\)\), "\[IndentingNewLine]", 
    \(\(boundary = Table[False, {i, 100}, {j, 100}];\)\), "\n", 
    \(\(\[Phi] = Table[0.0, {i, 100}, {j, 100}];\)\)}], "Input"],

Cell["Next we declare the two plates to have constant potentials", "Text"],

Cell[BoxData[
    \(Do[If[
        i \[GreaterEqual] 31 && 
          i \[LessEqual] 70 && \((j \[Equal] 41\  || j \[Equal] 60)\), 
        boundary\[LeftDoubleBracket]i, j\[RightDoubleBracket] = True, 
        boundary\[LeftDoubleBracket]i, j\[RightDoubleBracket] = False], {i, 
        1, 100}, {j, 1, 100}]\)], "Input"],

Cell["\<\
and the initial potential to vary linearly between them.  Some care in the \
choice of coordinates is needed to center the plates within the grid.\
\>", "Text"],

Cell[BoxData[
    \(Do[If[
        i \[GreaterEqual] 31 && i \[LessEqual] 70 && j \[GreaterEqual] 41 && 
          j \[LessEqual] 60, \[Phi]\[LeftDoubleBracket]i, 
            j\[RightDoubleBracket] = \((2  j - 101)\)/
            19. , \[Phi]\[LeftDoubleBracket]i, j\[RightDoubleBracket] = 
          0.0], {i, 1, 100}, {j, 1, 100}]\)], "Input"],

Cell["\<\
A better initial guess would include the region outside the plates, but we do \
not bother guessing there.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Generate relaxed solution", "Subsection"],

Cell[TextData[{
  "Now we simply invoke the relaxation function and ask it to produce a \
fairly accurate solution for the entire region.  ",
  StyleBox["Be patient \[LongDash] this can take a lot of time!",
    FontWeight->"Bold",
    FontColor->RGBColor[1, 0, 0]]
}], "Text"],

Cell[BoxData[
    \(\(\[Phi]0 = \[Phi];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\[Phi] = relax[\[Phi]0, boundary, 1000, 10\^\(-4\)];\)\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("number of iterations = "\[InvisibleSpace]140\),
      SequenceForm[ "number of iterations = ", 140],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("residual = "\[InvisibleSpace]0.00009826238010493604`\
\),
      SequenceForm[ "residual = ", .00009826238010493604],
      Editable->False]], "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Display potential and field", "Subsection"],

Cell["\<\
It is instructive to examine orthogonal slices across the capacitor\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[
        Table[\[Phi]\[LeftDoubleBracket]50, j\[RightDoubleBracket], {j, 1, 
            100}], PlotRange \[Rule] All];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0141895 0.00962001 0.309017 0.294302 [
[.20659 .29652 -8 -15.6875 ]
[.20659 .29652 8 0 ]
[.39899 .29652 -8 -15.6875 ]
[.39899 .29652 8 0 ]
[.59139 .29652 -8 -15.6875 ]
[.59139 .29652 8 0 ]
[.78379 .29652 -8 -15.6875 ]
[.78379 .29652 8 0 ]
[.97619 .29652 -11 -15.6875 ]
[.97619 .29652 11 0 ]
[.00169 .01472 -18.3125 -7.84375 ]
[.00169 .01472 0 7.84375 ]
[.00169 .16187 -27.3125 -7.84375 ]
[.00169 .16187 0 7.84375 ]
[.00169 .45617 -19 -7.84375 ]
[.00169 .45617 0 7.84375 ]
[.00169 .60332 -10 -7.84375 ]
[.00169 .60332 0 7.84375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.20659 .30902 m
.20659 .31527 L
s
gsave
.20659 .29652 -69 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
.39899 .30902 m
.39899 .31527 L
s
gsave
.39899 .29652 -69 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
.59139 .30902 m
.59139 .31527 L
s
gsave
.59139 .29652 -69 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(60) show
1.000 setlinewidth
grestore
.78379 .30902 m
.78379 .31527 L
s
gsave
.78379 .29652 -69 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(80) show
1.000 setlinewidth
grestore
.97619 .30902 m
.97619 .31527 L
s
gsave
.97619 .29652 -72 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100) show
1.000 setlinewidth
grestore
.125 Mabswid
.06229 .30902 m
.06229 .31277 L
s
.11039 .30902 m
.11039 .31277 L
s
.15849 .30902 m
.15849 .31277 L
s
.25469 .30902 m
.25469 .31277 L
s
.30279 .30902 m
.30279 .31277 L
s
.35089 .30902 m
.35089 .31277 L
s
.44709 .30902 m
.44709 .31277 L
s
.49519 .30902 m
.49519 .31277 L
s
.54329 .30902 m
.54329 .31277 L
s
.63949 .30902 m
.63949 .31277 L
s
.68759 .30902 m
.68759 .31277 L
s
.73569 .30902 m
.73569 .31277 L
s
.83189 .30902 m
.83189 .31277 L
s
.87999 .30902 m
.87999 .31277 L
s
.92809 .30902 m
.92809 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.01419 .01472 m
.02044 .01472 L
s
gsave
.00169 .01472 -79.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.01419 .16187 m
.02044 .16187 L
s
gsave
.00169 .16187 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.01419 .45617 m
.02044 .45617 L
s
gsave
.00169 .45617 -80 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.01419 .60332 m
.02044 .60332 L
s
gsave
.00169 .60332 -71 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.125 Mabswid
.01419 .04415 m
.01794 .04415 L
s
.01419 .07358 m
.01794 .07358 L
s
.01419 .10301 m
.01794 .10301 L
s
.01419 .13244 m
.01794 .13244 L
s
.01419 .1913 m
.01794 .1913 L
s
.01419 .22073 m
.01794 .22073 L
s
.01419 .25016 m
.01794 .25016 L
s
.01419 .27959 m
.01794 .27959 L
s
.01419 .33845 m
.01794 .33845 L
s
.01419 .36788 m
.01794 .36788 L
s
.01419 .39731 m
.01794 .39731 L
s
.01419 .42674 m
.01794 .42674 L
s
.01419 .4856 m
.01794 .4856 L
s
.01419 .51503 m
.01794 .51503 L
s
.01419 .54446 m
.01794 .54446 L
s
.01419 .57389 m
.01794 .57389 L
s
.25 Mabswid
.01419 0 m
.01419 .61803 L
s
.008 w
.02381 .30902 Mdot
.03343 .30355 Mdot
.04305 .29806 Mdot
.05267 .29258 Mdot
.06229 .28705 Mdot
.07191 .28151 Mdot
.08153 .27592 Mdot
.09115 .27031 Mdot
.10077 .26464 Mdot
.11039 .25891 Mdot
.12001 .25313 Mdot
.12963 .24726 Mdot
.13925 .2413 Mdot
.14887 .23525 Mdot
.15849 .22908 Mdot
.16811 .22281 Mdot
.17773 .21644 Mdot
.18735 .20994 Mdot
.19697 .20331 Mdot
.20659 .19653 Mdot
.21621 .18958 Mdot
.22583 .18249 Mdot
.23545 .17525 Mdot
.24507 .16783 Mdot
.25469 .16022 Mdot
.26431 .15243 Mdot
.27393 .14443 Mdot
.28355 .13627 Mdot
.29317 .12791 Mdot
.30279 .11935 Mdot
.31241 .11058 Mdot
.32203 .10163 Mdot
.33165 .09251 Mdot
.34127 .08324 Mdot
.35089 .07376 Mdot
.36051 .06414 Mdot
.37013 .05442 Mdot
.37975 .04459 Mdot
.38937 .03467 Mdot
.39899 .02469 Mdot
.40861 .01472 Mdot
.41823 .04573 Mdot
.42785 .07675 Mdot
.43747 .10776 Mdot
.44709 .13875 Mdot
.45671 .16973 Mdot
.46633 .2007 Mdot
.47595 .23166 Mdot
.48557 .26261 Mdot
.49519 .29355 Mdot
.50481 .32448 Mdot
.51443 .35542 Mdot
.52405 .38637 Mdot
.53367 .41733 Mdot
.54329 .4483 Mdot
.55291 .47928 Mdot
.56253 .51028 Mdot
.57215 .54128 Mdot
.58177 .5723 Mdot
.59139 .60332 Mdot
.60101 .59334 Mdot
.61063 .58336 Mdot
.62025 .57344 Mdot
.62987 .56362 Mdot
.63949 .55389 Mdot
.64911 .54427 Mdot
.65873 .53481 Mdot
.66835 .52552 Mdot
.67797 .5164 Mdot
.68759 .50745 Mdot
.69721 .4987 Mdot
.70683 .49012 Mdot
.71645 .48176 Mdot
.72607 .47359 Mdot
.73569 .46562 Mdot
.74531 .45781 Mdot
.75493 .4502 Mdot
.76455 .44279 Mdot
.77417 .43555 Mdot
.78379 .42845 Mdot
.79341 .42151 Mdot
.80303 .41472 Mdot
.81265 .40808 Mdot
.82227 .4016 Mdot
.83189 .39522 Mdot
.84151 .38896 Mdot
.85113 .38279 Mdot
.86075 .37672 Mdot
.87037 .37077 Mdot
.87999 .3649 Mdot
.88961 .35912 Mdot
.89923 .35341 Mdot
.90885 .34773 Mdot
.91847 .34211 Mdot
.92809 .33653 Mdot
.93771 .33097 Mdot
.94733 .32546 Mdot
.95695 .31997 Mdot
.96657 .31449 Mdot
.97619 .30902 Mdot
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00;@3oool00`000000oooo0?oo
o`3o0?oooh40oooo000N0?ooo`060;md00000000000000000000B9`0gooo2@3oool00`000000oooo
0?ooo`3o0?oooh40oooo000O0?ooo`030?oOW0180000M;oo00/0oooo00<000000?ooo`3oool0o`3o
oon10?ooo`007`3oool00`3ogi`0B00007Boo`0;0?ooo`030000003oool0oooo0?l0ooooP@3oool0
01l0oooo00<0omnL04P0001d_ol02`3oool00`000000oooo0?ooo`3o0?oooh40oooo000@0?ooo`03
0?oog`2LB000000000L0000000<0B9cO0?ooo`3oool00P3oool00`3ogi`0B00007Boo`0;0?ooo`03
0000003oool0oooo09@0oooo00<000000?ooo`3oool0j@3oool001l0oooo00<0omnL04P0001d_ol0
2`3oool400000980oooo0`00003Z0?ooo`007`3oool00`3ogi`0B00007Boo`0;0?ooo`030000003o
ool0oooo09@0oooo00<000000?ooo`3oool0j@3oool001l0oooo00<0omnL04P0001d_ol02`3oool0
0`000000oooo0?ooo`3o0?oooh40oooo000O0?ooo`030?oOW0180000M;oo00/0oooo00<000000?oo
o`3oool0T@3oool00`000000oooo0?ooo`3/0?ooo`007`3oool00`3ogi`0B00007Boo`0;0?ooo`03
0000003oool0oooo0900oooo0`00003]0?ooo`007@3oool01@3ooml0W4P0000000000000M;oo00/0
oooo00<000000?ooo`3oool0T@3oool00`000000oooo0?ooo`3/0?ooo`007`3oool00`3o_g@00000
07Boo`0;0?ooo`030000003oool0oooo08d0oooo00<000000?ooo`3oool0l03oool002d0oooo00<0
00000?ooo`3oool0S03oool300000?40oooo000]0?ooo`030000003oool0oooo08d0oooo00<00000
0?ooo`3oool0l03oool002d0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00;@3oool00`000000
oooo0?ooo`290?ooo`030000003oool0oooo0?@0oooo000]0?ooo`<00000R03oool3000000d0oooo
00<000000?ooo`3oool0i@3oool002d0oooo00<000000?ooo`3oool0R@3oool00`000000oooo0?oo
o`0;0?ooo`<00000iP3oool002d0oooo00<000000?ooo`3oool0V03oool00`000000oooo0?ooo`3U
0?ooo`00;@3oool00`000000oooo0?ooo`260?ooo`030000003oool0oooo0?L0oooo000]0?ooo`03
0000003oool0oooo08D0oooo0`00003h0?ooo`00;@3oool00`000000oooo0?ooo`260?ooo`030000
003oool0oooo0?L0oooo000]0?ooo`030000003oool0oooo0?l0ooooP@3oool002d0oooo00<00000
0?ooo`3oool0PP3oool00`000000oooo0?ooo`3k0?ooo`00;@3oool00`000000oooo0?ooo`210?oo
o`<00000o03oool002d0oooo00<000000?ooo`3oool0PP3oool00`000000oooo0?ooo`3k0?ooo`00
;@3oool00`000000oooo0?ooo`1n0?ooo`030000003oool0oooo0?l0oooo000]0?ooo`<00000O@3o
ool300000?l0oooo0@3oool002d0oooo00<000000?ooo`3oool0OP3oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo0>40oooo000]0?ooo`030000003oool0oooo09/0oooo0`00003R0?oo
o`00;@3oool00`000000oooo0?ooo`1k0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3o
ool0h@3oool002d0oooo00<000000?ooo`3oool0NP3oool300000?l0oooo103oool002d0oooo00<0
00000?ooo`3oool0N`3oool00`000000oooo0?ooo`3o0?ooo`<0oooo000]0?ooo`030000003oool0
oooo0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`3o0?oo
o`L0oooo000]0?ooo`030000003oool0oooo07H0oooo0`00003o0?ooo`P0oooo000]0?ooo`030000
003oool0oooo07L0oooo00<000000?ooo`3oool0o`3oool70?ooo`00;@3oool00`000000oooo0?oo
o`1c0?ooo`030000003oool0oooo0?l0oooo2`3oool002d0oooo00<000000?ooo`3oool0LP3oool3
00000?l0oooo303oool002d0oooo0`00001c0?ooo`030000003oool0oooo02X0oooo00<000000?oo
o`3oool0g@3oool002d0oooo00<000000?ooo`3oool0L03oool00`000000oooo0?ooo`0/0?ooo`<0
0000gP3oool002d0oooo00<000000?ooo`3oool0K`3oool3000002h0oooo00<000000?ooo`3oool0
g@3oool002d0oooo00<000000?ooo`3oool0L03oool00`000000oooo0?ooo`3o0?ooo`h0oooo000]
0?ooo`030000003oool0oooo0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0K03oool00`00
0000oooo0?ooo`3o0?oooa80oooo000]0?ooo`030000003oool0oooo06/0oooo0`00003o0?oooa<0
oooo000]0?ooo`030000003oool0oooo06`0oooo00<000000?ooo`3oool0o`3ooolB0?ooo`00;@3o
ool00`000000oooo0?ooo`1X0?ooo`030000003oool0oooo0?l0oooo5P3oool002d0oooo00<00000
0?ooo`3oool0I`3oool300000?l0oooo5`3oool002d0oooo00<000000?ooo`3oool0J03oool00`00
0000oooo0?ooo`3o0?oooaH0oooo000]0?ooo`<00000I@3oool00`000000oooo0?ooo`3o0?oooaT0
oooo000]0?ooo`030000003oool0oooo06@0oooo0`00000l0?ooo`030000003oool0oooo0=X0oooo
000]0?ooo`030000003oool0oooo06D0oooo00<000000?ooo`3oool0>P3oool300000=/0oooo000]
0?ooo`030000003oool0oooo0640oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`3J
0?ooo`00;@3oool00`000000oooo0?ooo`1P0?ooo`<00000o`3ooolN0?ooo`00;@3oool00`000000
oooo0?ooo`1Q0?ooo`030000003oool0oooo0?l0oooo7@3oool00140oooo00D0omnL04P000000000
001d0;ooo`030?ooo`060?noM0000000M;oo0?ooo`3ooml0W4P00`0000000`1d_ol0oooo0?ooo`08
0?ooo`030000003oool0oooo0?l0ooooP@3oool00100oooo00L0oono07@0001d_ol0oooo0?noM000
07@0_ooo0080oooo00L0okmd0000001d_ol0oooo0?oog`2LB7@0_ooo0080oooo00<0_g@007Boo`3o
ool0203oool00`000000oooo0?ooo`1M0?ooo`030000003oool0oooo0?l0oooo8@3oool00100oooo
00<0_g@00018W03Oool00P3oool00`2oM00007Bo0?ooo`0:0?ooo`030?oog`2LB7@0_ooo00P0oooo
00<000000?ooo`3oool0G03oool300000?l0oooo8P3oool000l0oooo00<0oono07@00018W=l00`3o
ool00`3OW4P0001d0;ooo`0;0?ooo`030;md001d_ol0oooo00L0oooo00<000000?ooo`3oool0G@3o
ool00`000000oooo0?ooo`3o0?ooob40oooo00030?ooo`030?oog`2LB000000000L0000000D0B9cO
0?ooo`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00X0oooo00<0oooO09a80018W=l0203o
ool00`000000oooo0?ooo`1I0?ooo`030000003oool0oooo0?l0oooo9@3oool000l0oooo00<0omnL
04P0001d_ol00`3oool00`3o_g@0001809cOo`0:0?ooo`030=nLB0000000M;oo00P0oooo1000001G
0?ooo`<00000o`3ooolV0?ooo`003`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo
00H0oooo00L0gmoO0?ooo`3ookl0M0000000000007@0_ooo00P0oooo00<000000?ooo`3oool0EP3o
ool010000000oooo0?ooo`00003o0?ooobL0oooo000?0?ooo`030?oOW0180000M;oo00<0oooo00<0
okmd0000B02Lgol01P3oool01P3OW4P00000000000000000001809cOo`T0oooo00<000000?ooo`3o
ool0E@3oool3000004l0oooo00<000000?ooo`3oool0eP3oool000l0oooo00<0oono07@00018W=l0
0`3oool00`3OW4P0001d0;ooo`060?ooo`040?oo_`1d0000000004RLg`/0oooo00<000000?ooo`3o
ool0EP3oool00`000000oooo0?ooo`1=0?ooo`<00000e`3oool00100oooo00<0gia80018W03Oool0
0P3oool00`2oM00007Bo0?ooo`070?ooo`030;md001d_ol0oooo00/0oooo00<000000?ooo`3oool0
DP3oool00`000000oooo0?ooo`1B0?ooo`030000003oool0oooo0=H0oooo000@0?ooo`070?oo_`1d
0000M;oo0?ooo`3o_g@0001d0;ooo`080?ooo`060?oOW018000000000000000004P0W=oo203oool0
0`000000oooo0?ooo`1A0?ooo`<00000o`3oool]0?ooo`004@3oool01@3ookl0M0000000000007@0
_ooo00X0oooo00D0_g@0000000000000000004RLg`080?ooo`030000003oool0oooo0580oooo00<0
00000?ooo`3oool0o`3oool/0?ooo`00;@3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0
oooo0?l0oooo<03oool002d0oooo00<000000?ooo`3oool0C@3oool300000?l0oooo<@3oool002d0
oooo00<000000?ooo`3oool0CP3oool00`000000oooo0?ooo`3o0?oooc00oooo000]0?ooo`030000
003oool0oooo04/0oooo00<000000?ooo`3oool0o`3ooolc0?ooo`00;@3oool3000004X0oooo0`00
003o0?oooc@0oooo000]0?ooo`030000003oool0oooo04L0oooo00D000000?ooo`3oool0oooo0000
003o0?ooocD0oooo000]0?ooo`030000003oool0oooo04H0oooo0`00003o0?ooocP0oooo000]0?oo
o`030000003oool0oooo04L0oooo00<000000?ooo`3oool0H@3oool00`000000oooo0?ooo`3B0?oo
o`00;@3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo06@0oooo0`00003C0?ooo`00
;@3oool00`000000oooo0?ooo`120?ooo`<00000IP3oool00`000000oooo0?ooo`3B0?ooo`00;@3o
ool00`000000oooo0?ooo`130?ooo`030000003oool0oooo0?l0oooo>`3oool002d0oooo00<00000
0?ooo`3oool0@03oool00`000000oooo0?ooo`3o0?oooch0oooo000]0?ooo`030000003oool0oooo
03l0oooo0`00003o0?ooocl0oooo000]0?ooo`030000003oool0oooo03`0oooo00D000000?ooo`3o
ool0oooo0000003o0?oood00oooo000]0?ooo`030000003oool0oooo03/0oooo0`00003o0?oood<0
oooo000]0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0o`3ooom20?ooo`00;@3o
ool3000003P0oooo00<000000?ooo`3oool0o`3ooom60?ooo`00;@3oool00`000000oooo0?ooo`0g
0?ooo`<00000o`3ooom70?ooo`00;@3oool00`000000oooo0?ooo`0e0?ooo`040000003oool0oooo
00000?l0ooooB03oool002d0oooo00<000000?ooo`3oool0=03oool3000007L0oooo00<000000?oo
o`3oool0c`3oool002d0oooo00<000000?ooo`3oool0<@3oool01@000000oooo0?ooo`3oool00000
07L0oooo0`00003@0?ooo`00;@3oool00`000000oooo0?ooo`0`0?ooo`<00000N`3oool00`000000
oooo0?ooo`3?0?ooo`00;@3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo0?l0oooo
C@3oool002d0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`3o0?oooe40oooo000]
0?ooo`030000003oool0oooo02`0oooo0`00003o0?oooe80oooo000]0?ooo`030000003oool0oooo
02X0oooo00@000000?ooo`3oool00000o`3ooomC0?ooo`00;@3oool00`000000oooo0?ooo`0Y0?oo
o`<00000o`3ooomE0?ooo`00;@3oool3000002H0oooo00D000000?ooo`3oool0oooo0000003o0?oo
oeH0oooo000]0?ooo`030000003oool0oooo02D0oooo0`00000F0?ooo`030;md00000000000000@0
000000T0B9cO0?ooo`3oool0oooo0?oOW018000000000000M02oool0?P3oool0103ooml0W4P00018
W03Oool30?ooo`050?oOW018000000000000M02oool0?03oool01@3ogi`0B000000000000000M;oo
00@0oooo00D0omnL04P000000000001d0;ooo`0k0?ooo`050=nLB0000000000000000018W=l0103o
ool01@3ogi`0B0000000000007@0_ooo03P0oooo00H0_g@000000000000000000018W03Oool30?oo
o`050?oOW018000000000000M02oool0103oool01@3ogi`0B0000000000007@0_ooo00H0oooo000]
0?ooo`030000003oool0oooo0280oooo00D000000?ooo`3oool0oooo0000000H0?ooo`030=nLB000
0000000000<0000000X004RL0=ooo`3oool0oono07@0001d_ol0oooo0?noM00007@0_ooo?@3oool0
103ooml0W4P00018W03Oool20?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`0j0?ooo`07
0?oOW0180000B9cO0?ooo`3ookl0M00007Boo`020?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d
0;ooo`0i0?ooo`070=nLB0000000M;oo0?ooo`3ooml0W4P004RLg`020?ooo`070?oo_`1d0000M;oo
0?ooo`3o_g@0001d0;ooo`0h0?ooo`030?oOW0180000M;oo00@0oooo00L0oono07@0001d_ol0oooo
0?noM00007@0_ooo0080oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00D0oooo000]0?oo
o`030000003oool0oooo0240oooo0`00000L0?ooo`030=nLB018W=l0oooo0080oooo00H0giad09cO
o`3oool0_g@00018W03Oool20?ooo`030;md0000M;l0oooo03`0oooo00@0oooO09a80000B9`0gooo
0P3oool00`2oM00004RL0=ooo`020?ooo`030;md0000M;l0oooo03P0oooo00<0oooO09a80000M;l0
0`3oool01`3o_g@004RL0=ooo`3oool0_g@00018W03Oool00P3oool00`2oM00007Bo0?ooo`0g0?oo
o`030?oog`2LB00007Bo00<0oooo00L0okmd0018W03Oool0oooo0;md0000B9`0gooo0080oooo00<0
_g@0001d_`3oool0=`3oool00`3ogi`0B00007Boo`040?ooo`030;md0000B9`0gooo0080oooo00L0
_g@0001d_`3oool0oooo0;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool0103oool002d0oooo
00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0L0?ooo`030=nLB01d_ol0oooo00<0oooo
00<0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`0h0?ooo`030;md00000000000000D00000
00@0M;oo0?oo_`1d0000B9cO0`3oool00`3OW4P0001d0;ooo`0H0?ooo`030000003oool0oooo01d0
oooo00<0omnL04P0001d_ol00`3oool01P3o_g@0001809cOo`3ookl0M00004RLg`<0oooo00<0gia8
0000M02oool0>03oool00`2oM00007Bo0?ooo`020?ooo`060;md0000B9`0gooo0?oo_`1d0000B9cO
0`3oool00`3OW4P0001d0;ooo`0g0?ooo`030?oOW0180000M;oo00<0oooo00<0oono07@00018W=l0
0`3oool01P3OW4P0001d0;ooo`3ookl0M00004RLg`<0oooo00<0gia80000M02oool0103oool002d0
oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0Q0?ooo`030;mdB02Lgol0oooo0080
oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0h0?ooo`0<0?oOW01dW=l0oooo0?oo
o`3ooml0W4P00018W03Oool0oooo0?oOW0180000M;oo0`3oool00`3o_g@0001809cOo`0G0?ooo`<0
00007P3oool00`3ogi`0B00007Boo`030?ooo`060?noM00004P0W=oo0?oOW0180000M;oo0`3oool0
0`3o_g@0001809cOo`0h0?ooo`0;0?oOW01807@0_ooo0?ooo`2oM000000007Boo`3oool0omnL04P0
001d_ol00`3oool00`3o_g@0001809cOo`0g0?ooo`030?oOW0180000M;oo00<0oooo00<0omnL04P0
001d_ol00`3oool01P3o_g@0001809cOo`3ogi`0B00007Boo`<0oooo00<0okmd0000B02Lgol0103o
ool002d0oooo00<000000?ooo`3oool07@3oool300000280oooo00<0oooO09a8M02oool00P3oool0
0`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo03T0oooo00/0gia807Boo`3oool0oooO09a8
0000B9`0gooo0?ooo`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo01P0oooo00<000000?oo
o`3oool07@3oool00`3ogi`0B00007Boo`030?ooo`060;md000004P0W=oo0?oOW0180000M;oo0`3o
ool00`3o_g@0001809cOo`0i0?ooo`050?oo_`1dB4P0B00000000018W=l00P3oool00`3ogi`0B000
07Boo`030?ooo`030?noM00004P0W=oo03L0oooo00<0omnL04P0001d_ol00`3oool00`3ogi`0B000
07Boo`030?ooo`060?noM00004P0W=oo0?oOW0180000M;oo0`3oool00`3o_g@0001809cOo`040?oo
o`00;@3oool00`000000oooo0?ooo`0K0?ooo`040000003oool0oooo000002@0oooo00L0okmd001d
_`3oool0oooo0?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0>@3oool02`3ooml0W4Qd0;oo
o`3ooml0W4P00018W03Oool0oooo0?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0>03oool0
303ookl0M00000000018W=l0oooo0?noM000000007Bo0?ooo`3ogi`0B00007Boo`<0oooo00<0okmd
0000B02Lgol0>@3oool0103ogi`0B00000000000M;l30?ooo`030?oOW0180000M;oo00<0oooo00<0
okmd0000B02Lgol0=`3oool00`3ogi`0B00007Boo`030?ooo`030?oOW0180000M;oo00<0oooo00H0
okmd0000B02Lgol0omnL04P0001d_ol30?ooo`030?noM00004P0W=oo00@0oooo000]0?ooo`030000
003oool0oooo01X0oooo0`00000V0?ooo`070?oo_`1d04P0W=oo0?ooo`3ogi`0B00007Boo`030?oo
o`030?noM00004P0W=oo03X0oooo00X0omnL04Qd_`3ooml0W4P00018W03Oool0oooo0?oOW0180000
M;oo0`3oool00`3o_g@0001809cOo`0i0?ooo`030;md0000B9`0_ia80080000000H004RL0=ooo`3o
ool0omnL04P0001d_ol30?ooo`030?noM00004P0W=oo03P0oooo00H0omnL04P0000007@0_ooo0?no
M000M;l20?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0=`3oool00`3ogi`0B000
07Boo`030?ooo`030?oOW0180000M;oo00<0oooo00H0okmd0000B02Lgol0omnL04P0001d_ol30?oo
o`030?noM00004P0W=oo00@0oooo000]0?ooo`030000003oool0oooo01L0oooo00D000000?ooo`3o
ool0oooo0000000W0?ooo`070?oo_`1d0000B9cO0?ooo`3ookl0M00004RLg`030?ooo`030=nLB000
07@0_ooo03/0oooo00T0gia807Bog`2LB00004RL0=ooo`3oool0oono07@00018W=l00`3oool00`3O
W4P0001d0;ooo`0i0?ooo`030?oOW0180000M;oo00D0oooo00<0oono07@00018W=l00`3oool00`3O
W4P0001d0;ooo`0h0?ooo`030;md000007@0_ooo0080oooo00H0_g@004RLg`3oool0oono07@00018
W=l30?ooo`030=nLB00007@0_ooo03L0oooo00<0omnL04P0001d_ol00`3oool00`3ookl0M00004RL
g`030?ooo`060=nLB00007@0_ooo0?oo_`1d0000B9cO0`3oool00`3OW4P0001d0;ooo`040?ooo`00
;@3oool00`000000oooo0?ooo`0F0?ooo`<000009@3oool00`3o_i`0W=oo0?ooo`020?ooo`030?oO
W0180000B9cO0080oooo00<0gia80018W03Oool00P3oool00`2oM00007Bo0?ooo`0k0?ooo`050?oo
g`2LB4P0B4P00018W03Oool00P3oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo03X0
oooo00<0gia80000B02Lgol01@3oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo03P0
oooo00<0_g@00018W03Oool00P3oool01`3OW4P007Bo0?ooo`3oool0gia80018W03Oool00P3oool0
0`2oM00007Bo0?ooo`0g0?ooo`030?oOW0180000M;oo00@0oooo00<0gia80018W03Oool00P3oool0
1`2oM00007Bo0?ooo`3oool0gia80018W03Oool00P3oool00`2oM00007Bo0?ooo`040?ooo`00;@3o
ool3000001<0oooo00D000000?ooo`3oool0oooo0000000V0?ooo`080?oog`2LB4P0W=oo0?ooo`3o
okl0M0000000M02oool20?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`0m0?ooo`040?oO
W018000004RL0=ooo`80oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo03`0oooo00<0okmd
00000000M;l0103oool01`3ookl0M00007Boo`3oool0okmd0000M02oool0>@3oool01`3o_g@00000
07Boo`3oool0oono07@0001d_ol00P3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool0=P3o
ool01@3ooml0W4P0000000000000M;oo00@0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo
0080oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00D0oooo000]0?ooo`030000003oool0
oooo0180oooo0`00000Z0?ooo`060?oog`2LB00000000000000007@0_ooo103oool01@3ookl0M000
0000000007@0_ooo03l0oooo00<0gia80018W03Oool00`3oool01@3ookl0M0000000000007@0_ooo
03h0oooo00D0oooO09a800000000001d0;ooo`020?ooo`050?oo_`1d000000000000M02oool0>`3o
ool01@3o_g@00000000000000000B9cO00@0oooo00D0oono07@000000000001d0;ooo`0i0?ooo`03
0?noM0000000M;oo00D0oooo00D0oono07@000000000001d0;ooo`040?ooo`050?oo_`1d00000000
0000M02oool01P3oool002d0oooo00<000000?ooo`3oool0403oool010000000oooo0?ooo`00003o
0?ooofd0oooo000]0?ooo`030000003oool0oooo00l0oooo0`00003o0?ooofl0oooo000]0?ooo`03
0000003oool0oooo00`0oooo00D000000?ooo`3oool0oooo0000003o0?ooog00oooo000]0?ooo`03
0000003oool0oooo00/0oooo0`00002X0?ooo`030000003oool0oooo0<L0oooo000]0?ooo`030000
003oool0oooo00P0oooo00D000000?ooo`3oool0oooo0000002X0?ooo`<00000b03oool002d0oooo
00<000000?ooo`3oool01`3oool300000:`0oooo00<000000?ooo`3oool0a`3oool002d0oooo00<0
00000?ooo`3oool01@3oool010000000oooo0?ooo`00003o0?ooogP0oooo000]0?ooo`030000003o
ool0oooo00@0oooo0`00003o0?ooogX0oooo000]0?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool0o`3ooomi0?ooo`00;@3oool01@000000oooo0?ooo`3oool000000?l0ooooK@3oool0
0`000000oooo0?ooo`0?0?ooo`00:03ooooo000008400000203oool002d0oooo00D000000?ooo`3o
ool0oooo0000000>0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo00/0oooo00D000000?ooo`3oool0oooo0000000A
0?ooo`00;@3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo04L0oooo00<000000?oo
o`3oool0A`3oool00`000000oooo0?ooo`160?ooo`030000003oool0oooo0480oooo0`0000020?oo
o`030000003oool0oooo00l0oooo000]0?ooo`030000003oool0oooo0?l0ooooJ03oool010000000
oooo0?ooo`00000E0?ooo`00;@3oool00`000000oooo0?ooo`2i0?ooo`030000003oool0oooo0:X0
oooo0`00000G0?ooo`00;@3oool00`000000oooo0?ooo`2h0?ooo`<00000Z03oool01@000000oooo
0?ooo`3oool0000001P0oooo000]0?ooo`030000003oool0oooo0;T0oooo00<000000?ooo`3oool0
YP3oool3000001/0oooo000]0?ooo`030000003oool0oooo0?l0ooooH03oool01@000000oooo0?oo
o`3oool0000001`0oooo000]0?ooo`030000003oool0oooo0?l0ooooG`3oool3000001l0oooo000]
0?ooo`030000003oool0oooo0?l0ooooG@3oool010000000oooo0?ooo`00000P0?ooo`00;@3oool0
0`000000oooo0?ooo`3o0?oooe`0oooo0`00000R0?ooo`00;@3oool300000?l0ooooF@3oool01@00
0000oooo0?ooo`3oool0000002<0oooo000]0?ooo`030000003oool0oooo0?l0ooooF03oool30000
02H0oooo000]0?ooo`030000003oool0oooo0?l0ooooE@3oool01@000000oooo0?ooo`3oool00000
02L0oooo000]0?ooo`030000003oool0oooo0?l0ooooE03oool3000002X0oooo000]0?ooo`030000
003oool0oooo0?l0ooooE@3oool00`000000oooo0?ooo`0Y0?ooo`00;@3oool00`000000oooo0?oo
o`2m0?ooo`030000003oool0oooo0940oooo00<000000?ooo`3oool0;03oool002d0oooo00<00000
0?ooo`3oool0_03oool300000940oooo0`00000]0?ooo`00;@3oool00`000000oooo0?ooo`2m0?oo
o`030000003oool0oooo08d0oooo00D000000?ooo`3oool0oooo0000000^0?ooo`00;@3oool00`00
0000oooo0?ooo`3o0?ooodd0oooo0`00000a0?ooo`00;@3oool00`000000oooo0?ooo`3o0?ooodX0
oooo00D000000?ooo`3oool0oooo0000000b0?ooo`00;@3oool00`000000oooo0?ooo`3o0?ooodT0
oooo0`00000e0?ooo`00;@3oool300000?l0ooooAP3oool01@000000oooo0?ooo`3oool0000003H0
oooo000]0?ooo`030000003oool0oooo0?l0ooooA@3oool3000003T0oooo000]0?ooo`030000003o
ool0oooo0?l0ooooAP3oool00`000000oooo0?ooo`0h0?ooo`00;@3oool00`000000oooo0?ooo`3o
0?oood<0oooo00<000000?ooo`3oool0>`3oool002d0oooo00<000000?ooo`3oool0o`3ooom20?oo
o`<00000?03oool002d0oooo00<000000?ooo`3oool0o`3ooolo0?ooo`050000003oool0oooo0?oo
o`000000?@3oool002d0oooo00<000000?ooo`3oool0`@3oool00`000000oooo0?ooo`1i0?ooo`<0
0000@03oool002d0oooo00<000000?ooo`3oool0`03oool3000007L0oooo00D000000?ooo`3oool0
oooo000000110?ooo`00;@3oool00`000000oooo0?ooo`310?ooo`030000003oool0oooo07D0oooo
0`0000140?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooc/0oooo00<000000?ooo`3oool0@`3o
ool002d0oooo00<000000?ooo`3oool0o`3ooolh0?ooo`030000003oool0oooo04H0oooo000]0?oo
o`<00000o`3ooolg0?ooo`<00000A`3oool002d0oooo00<000000?ooo`3oool0o`3ooold0?ooo`05
0000003oool0oooo0?ooo`000000B03oool002d0oooo00<000000?ooo`3oool0o`3ooolc0?ooo`<0
0000B`3oool002d0oooo00<000000?ooo`3oool0o`3ooold0?ooo`030000003oool0oooo04X0oooo
000]0?ooo`030000003oool0oooo0?l0oooo<03oool00`000000oooo0?ooo`1>0?ooo`00;@3oool0
0`000000oooo0?ooo`3o0?ooobl0oooo0`00001?0?ooo`00;@3oool00`000000oooo0?ooo`3o0?oo
obd0oooo00@000000?ooo`3oool00000D03oool002d0oooo00<000000?ooo`3oool0a03oool00`00
0000oooo0?ooo`1T0?ooo`<00000DP3oool002d0oooo00<000000?ooo`3oool0``3oool3000006H0
oooo00<000000?ooo`3oool0D@3oool002d0oooo00<000000?ooo`3oool0a03oool00`000000oooo
0?ooo`1Q0?ooo`030000003oool0oooo05D0oooo000]0?ooo`030000003oool0oooo0?l0oooo:03o
ool3000005H0oooo000]0?ooo`030000003oool0oooo0?l0oooo9@3oool01@000000oooo0?ooo`3o
ool0000005L0oooo000]0?ooo`<00000o`3ooolT0?ooo`<00000FP3oool002d0oooo00<000000?oo
o`3oool0o`3ooolU0?ooo`030000003oool0oooo05T0oooo000]0?ooo`030000003oool0oooo0?l0
oooo8P3oool00`000000oooo0?ooo`1L0?ooo`00;@3oool00`000000oooo0?ooo`3o0?ooob40oooo
0`00001M0?ooo`00;@3oool00`000000oooo0?ooo`3o0?ooob80oooo00<000000?ooo`3oool0G03o
ool002d0oooo00<000000?ooo`3oool0o`3ooolN0?ooo`030000003oool0oooo0600oooo000A0?oo
o`050?oOW018000000000000M02oool00`3oool01P3o_g@0000007Boo`3oool0oooO09a800<00000
00<0M;oo0?ooo`3oool0203oool00`000000oooo0?ooo`3o0?oooad0oooo0`00001Q0?ooo`00403o
ool01`3ookl0M00007Boo`3oool0okmd0000M02oool00P3oool01`3o_g@0000007Boo`3oool0oooO
09a8M02oool00P3oool00`2oM000M;oo0?ooo`080?ooo`030000003oool0oooo0<P0oooo00<00000
0?ooo`3oool0DP3oool00`000000oooo0?ooo`1P0?ooo`00403oool00`2oM00004RL0=ooo`020?oo
o`030;md0000M;l0oooo00X0oooo00<0oooO09a8M02oool0203oool00`000000oooo0?ooo`370?oo
o`<00000C`3oool00`000000oooo0?ooo`1T0?ooo`003`3oool00`3ookl0M00004RLg`030?ooo`03
0=nLB00007@0_ooo00/0oooo00<0_g@007Boo`3oool01`3oool00`000000oooo0?ooo`380?ooo`03
0000003oool0oooo04d0oooo0`00001U0?ooo`003`3oool00`3ogi`0B00007Boo`030?ooo`030?no
M00004P0W=oo00X0oooo00<0oooO09a80018W=l0203oool00`000000oooo0?ooo`3o0?oooaX0oooo
00<000000?ooo`3oool0I03oool000l0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cO
o`0:0?ooo`030=nLB0000000M;oo00P0oooo1000003o0?oooaH0oooo00<000000?ooo`3oool0I`3o
ool000l0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`060?ooo`070=oOg`3oool0
oono07@000000000001d0;ooo`080?ooo`030000003oool0oooo0?l0oooo5P3oool3000006P0oooo
000?0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01P3oool01P3OW4P000000000
00000000001809cOo`T0oooo00<000000?ooo`3oool0o`3ooolG0?ooo`030000003oool0oooo06L0
oooo000?0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool01P3oool0103ookl0M000
00000018W=l;0?ooo`030000003oool0oooo0?l0oooo4`3oool00`000000oooo0?ooo`1[0?ooo`00
403oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00L0oooo00<0_g@007Boo`3oool0
2`3oool00`000000oooo0?ooo`3o0?oooa80oooo0`00001/0?ooo`00403oool01`3ookl0M00007Bo
o`3oool0okmd0000M02oool0203oool01P3ogi`0B000000000000000001809cOo`P0oooo00<00000
0?ooo`3oool0o`3ooolC0?ooo`030000003oool0oooo06/0oooo000A0?ooo`050?oo_`1d00000000
0000M02oool02P3oool01@2oM0000000000000000000B9cO00P0oooo00<000000?ooo`3oool0o`3o
ool?0?ooo`030000003oool0oooo06l0oooo000]0?ooo`030000003oool0oooo0?l0oooo3P3oool3
00000700oooo000]0?ooo`030000003oool0oooo0<`0oooo00<000000?ooo`3oool0?`3oool00`00
0000oooo0?ooo`1_0?ooo`00;@3oool00`000000oooo0?ooo`3;0?ooo`<00000?03oool00`000000
oooo0?ooo`1c0?ooo`00;@3oool00`000000oooo0?ooo`3<0?ooo`030000003oool0oooo03X0oooo
0`00001d0?ooo`00;@3oool300000?l0oooo2`3oool00`000000oooo0?ooo`1c0?ooo`00;@3oool0
0`000000oooo0?ooo`3o0?ooo`P0oooo00<000000?ooo`3oool0MP3oool002d0oooo00<000000?oo
o`3oool0o`3oool70?ooo`<00000M`3oool002d0oooo00<000000?ooo`3oool0o`3oool80?ooo`03
0000003oool0oooo07H0oooo000]0?ooo`030000003oool0oooo0?l0oooo103oool00`000000oooo
0?ooo`1j0?ooo`00;@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo0`00001k0?ooo`00;@3oool0
0`000000oooo0?ooo`3o0?ooo`@0oooo00<000000?ooo`3oool0NP3oool002d0oooo00<000000?oo
o`3oool0o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`3o0?ooo`030000003oool0oooo07h0
oooo000]0?ooo`030000003oool0oooo0<l0oooo00<000000?ooo`3oool0;03oool3000007l0oooo
000]0?ooo`030000003oool0oooo0<h0oooo0`00000^0?ooo`030000003oool0oooo07h0oooo000]
0?ooo`030000003oool0oooo0<l0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`21
0?ooo`00;@3oool300000?/0oooo0`0000220?ooo`00;@3oool00`000000oooo0?ooo`3l0?ooo`03
0000003oool0oooo0840oooo000]0?ooo`030000003oool0oooo0?l0ooooP@3oool002d0oooo00<0
00000?ooo`3oool0n03oool00`000000oooo0?ooo`250?ooo`00;@3oool00`000000oooo0?ooo`3g
0?ooo`<00000QP3oool002d0oooo00<000000?ooo`3oool0n03oool00`000000oooo0?ooo`250?oo
o`00;@3oool00`000000oooo0?ooo`3d0?ooo`030000003oool0oooo08T0oooo000]0?ooo`030000
003oool0oooo0?<0oooo0`00002:0?ooo`00;@3oool00`000000oooo0?ooo`3d0?ooo`030000003o
ool0oooo08T0oooo000]0?ooo`030000003oool0oooo0=<0oooo00<000000?ooo`3oool0ZP3oool0
02d0oooo00<000000?ooo`3oool0dP3oool3000001`0oooo00<000000?ooo`3oool0S03oool002d0
oooo0`00003C0?ooo`030000003oool0oooo01X0oooo0`00002=0?ooo`00;@3oool00`000000oooo
0?ooo`3a0?ooo`030000003oool0oooo08`0oooo000]0?ooo`030000003oool0oooo0>d0oooo00<0
00000?ooo`3oool0T03oool002d0oooo00<000000?ooo`3oool0k03oool300000940oooo000]0?oo
o`030000003oool0oooo0>d0oooo00<000000?ooo`3oool0T03oool002d0oooo00<000000?ooo`3o
ool0o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`3Y0?ooo`030000003oool0oooo09@0oooo
000]0?ooo`030000003oool0oooo0>P0oooo0`00002E0?ooo`00;@3oool00`000000oooo0?ooo`3Y
0?ooo`030000003oool0oooo09@0oooo000]0?ooo`030000003oool0oooo0=L0oooo00<000000?oo
o`3oool0YP3oool002d0oooo00<000000?ooo`3oool0eP3oool3000000d0oooo00<000000?ooo`3o
ool0U`3oool002d0oooo0`00003G0?ooo`030000003oool0oooo00/0oooo0`00002H0?ooo`00;@3o
ool00`000000oooo0?ooo`3V0?ooo`030000003oool0oooo09L0oooo000]0?ooo`030000003oool0
oooo0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0hP3oool00`000000oooo0?ooo`2K0?oo
o`00;@3oool00`000000oooo0?ooo`3Q0?ooo`<00000W03oool002d0oooo00<000000?ooo`3oool0
hP3oool00`000000oooo0?ooo`2K0?ooo`007`3oool01P2oM000000000000000000004RL0=ooo`P0
oooo00<000000?ooo`3oool0o`3ooon10?ooo`00803oool00`3ogi`0B00007Boo`0:0?ooo`030000
003oool0oooo0=h0oooo00<000000?ooo`3oool0W`3oool00200oooo00<0omnL04P0001d_ol02P3o
ool00`000000oooo0?ooo`3M0?ooo`<00000X03oool00200oooo00<0omnL04P0001d_ol02P3oool0
0`000000oooo0?ooo`3N0?ooo`030000003oool0oooo09l0oooo000P0?ooo`030?oOW0180000M;oo
00X0oooo00<000000?ooo`3oool0f`3oool00`000000oooo0?ooo`2R0?ooo`00803oool00`3ogi`0
B00007Boo`0:0?ooo`@00000f@3oool300000:<0oooo000P0?ooo`030?oOW0180000M;oo00X0oooo
00<000000?ooo`3oool0f`3oool00`000000oooo0?ooo`2R0?ooo`00803oool00`3ogi`0B00007Bo
o`0:0?ooo`030000003oool0oooo0?l0ooooP@3oool00200oooo00<0omnL04P0001d_ol02P3oool0
0`000000oooo0?ooo`3o0?oooh40oooo000P0?ooo`030?oOW0180000M;oo00X0oooo00<000000?oo
o`3oool0o`3ooon10?ooo`007P3oool01@3ooml0W4P0000000000000M;oo00X0oooo00<000000?oo
o`3oool0o`3ooon10?ooo`00803oool00`3o_g@0000007Boo`0:0?ooo`030000003oool0oooo0?l0
ooooP@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0
oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0
oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00001
\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {-12.4104, -1.17845, \
0.406794, 0.0132971}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[
        Table[\[Phi]\[LeftDoubleBracket]i, 45\[RightDoubleBracket], {i, 1, 
            100}]];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.00952381 0.603319 1.01739 [
[.21429 .59082 -8 -15.6875 ]
[.21429 .59082 8 0 ]
[.40476 .59082 -8 -15.6875 ]
[.40476 .59082 8 0 ]
[.59524 .59082 -8 -15.6875 ]
[.59524 .59082 8 0 ]
[.78571 .59082 -8 -15.6875 ]
[.78571 .59082 8 0 ]
[.97619 .59082 -11 -15.6875 ]
[.97619 .59082 11 0 ]
[.01131 .09462 -27.3125 -7.84375 ]
[.01131 .09462 0 7.84375 ]
[.01131 .19636 -27.3125 -7.84375 ]
[.01131 .19636 0 7.84375 ]
[.01131 .2981 -27.3125 -7.84375 ]
[.01131 .2981 0 7.84375 ]
[.01131 .39984 -27.3125 -7.84375 ]
[.01131 .39984 0 7.84375 ]
[.01131 .50158 -27.3125 -7.84375 ]
[.01131 .50158 0 7.84375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .60332 m
.21429 .60957 L
s
gsave
.21429 .59082 -69 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
.40476 .60332 m
.40476 .60957 L
s
gsave
.40476 .59082 -69 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
.59524 .60332 m
.59524 .60957 L
s
gsave
.59524 .59082 -69 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(60) show
1.000 setlinewidth
grestore
.78571 .60332 m
.78571 .60957 L
s
gsave
.78571 .59082 -69 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(80) show
1.000 setlinewidth
grestore
.97619 .60332 m
.97619 .60957 L
s
gsave
.97619 .59082 -72 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100) show
1.000 setlinewidth
grestore
.125 Mabswid
.07143 .60332 m
.07143 .60707 L
s
.11905 .60332 m
.11905 .60707 L
s
.16667 .60332 m
.16667 .60707 L
s
.2619 .60332 m
.2619 .60707 L
s
.30952 .60332 m
.30952 .60707 L
s
.35714 .60332 m
.35714 .60707 L
s
.45238 .60332 m
.45238 .60707 L
s
.5 .60332 m
.5 .60707 L
s
.54762 .60332 m
.54762 .60707 L
s
.64286 .60332 m
.64286 .60707 L
s
.69048 .60332 m
.69048 .60707 L
s
.7381 .60332 m
.7381 .60707 L
s
.83333 .60332 m
.83333 .60707 L
s
.88095 .60332 m
.88095 .60707 L
s
.92857 .60332 m
.92857 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .09462 m
.03006 .09462 L
s
gsave
.01131 .09462 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.02381 .19636 m
.03006 .19636 L
s
gsave
.01131 .19636 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.4) show
1.000 setlinewidth
grestore
.02381 .2981 m
.03006 .2981 L
s
gsave
.01131 .2981 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.3) show
1.000 setlinewidth
grestore
.02381 .39984 m
.03006 .39984 L
s
gsave
.01131 .39984 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
.02381 .50158 m
.03006 .50158 L
s
gsave
.01131 .50158 -88.3125 -11.8438 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.1) show
1.000 setlinewidth
grestore
.125 Mabswid
.02381 .11497 m
.02756 .11497 L
s
.02381 .13532 m
.02756 .13532 L
s
.02381 .15567 m
.02756 .15567 L
s
.02381 .17601 m
.02756 .17601 L
s
.02381 .21671 m
.02756 .21671 L
s
.02381 .23706 m
.02756 .23706 L
s
.02381 .25741 m
.02756 .25741 L
s
.02381 .27775 m
.02756 .27775 L
s
.02381 .31845 m
.02756 .31845 L
s
.02381 .3388 m
.02756 .3388 L
s
.02381 .35915 m
.02756 .35915 L
s
.02381 .37949 m
.02756 .37949 L
s
.02381 .42019 m
.02756 .42019 L
s
.02381 .44054 m
.02756 .44054 L
s
.02381 .46088 m
.02756 .46088 L
s
.02381 .48123 m
.02756 .48123 L
s
.02381 .52193 m
.02756 .52193 L
s
.02381 .54228 m
.02756 .54228 L
s
.02381 .56262 m
.02756 .56262 L
s
.02381 .58297 m
.02756 .58297 L
s
.02381 .07428 m
.02756 .07428 L
s
.02381 .05393 m
.02756 .05393 L
s
.02381 .03358 m
.02756 .03358 L
s
.02381 .01323 m
.02756 .01323 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
.03333 .60332 Mdot
.04286 .59866 Mdot
.05238 .59396 Mdot
.0619 .58917 Mdot
.07143 .58426 Mdot
.08095 .5792 Mdot
.09048 .57394 Mdot
.1 .56843 Mdot
.10952 .56263 Mdot
.11905 .55646 Mdot
.12857 .54989 Mdot
.1381 .54279 Mdot
.14762 .53515 Mdot
.15714 .52683 Mdot
.16667 .51775 Mdot
.17619 .50778 Mdot
.18571 .49679 Mdot
.19524 .48461 Mdot
.20476 .47105 Mdot
.21429 .45592 Mdot
.22381 .43895 Mdot
.23333 .41988 Mdot
.24286 .39841 Mdot
.25238 .37421 Mdot
.2619 .34697 Mdot
.27143 .3164 Mdot
.28095 .28243 Mdot
.29048 .24529 Mdot
.3 .20592 Mdot
.30952 .16628 Mdot
.31905 .12961 Mdot
.32857 .09922 Mdot
.3381 .07584 Mdot
.34762 .05855 Mdot
.35714 .04601 Mdot
.36667 .03701 Mdot
.37619 .03057 Mdot
.38571 .02597 Mdot
.39524 .02267 Mdot
.40476 .0203 Mdot
.41429 .0186 Mdot
.42381 .0174 Mdot
.43333 .01654 Mdot
.44286 .01594 Mdot
.45238 .01551 Mdot
.4619 .0152 Mdot
.47143 .01499 Mdot
.48095 .01485 Mdot
.49048 .01476 Mdot
.5 .01472 Mdot
.50952 .01472 Mdot
.51905 .01476 Mdot
.52857 .01485 Mdot
.5381 .01499 Mdot
.54762 .01521 Mdot
.55714 .01551 Mdot
.56667 .01595 Mdot
.57619 .01653 Mdot
.58571 .01741 Mdot
.59524 .0186 Mdot
.60476 .0203 Mdot
.61429 .02267 Mdot
.62381 .02597 Mdot
.63333 .03057 Mdot
.64286 .03701 Mdot
.65238 .04601 Mdot
.6619 .05854 Mdot
.67143 .07584 Mdot
.68095 .09923 Mdot
.69048 .12961 Mdot
.7 .16629 Mdot
.70952 .20591 Mdot
.71905 .24529 Mdot
.72857 .28243 Mdot
.7381 .31641 Mdot
.74762 .34696 Mdot
.75714 .3742 Mdot
.76667 .3984 Mdot
.77619 .41988 Mdot
.78571 .43895 Mdot
.79524 .45591 Mdot
.80476 .47105 Mdot
.81429 .48461 Mdot
.82381 .49679 Mdot
.83333 .50778 Mdot
.84286 .51775 Mdot
.85238 .52684 Mdot
.8619 .53514 Mdot
.87143 .54281 Mdot
.88095 .54988 Mdot
.89048 .55647 Mdot
.9 .56263 Mdot
.90952 .56844 Mdot
.91905 .57394 Mdot
.92857 .57921 Mdot
.9381 .58426 Mdot
.94762 .58917 Mdot
.95714 .59396 Mdot
.96667 .59866 Mdot
.97619 .60332 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooh40oooo
000]0?ooo`030000003oool0oooo0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0o`3ooon1
0?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooh40oooo000]0?ooo`030000003oool0oooo0?l0
ooooP@3oool002d0oooo0`00002L0?ooo`050000003oool0oooo0?ooo`0000000P3oool01@000000
oooo0?ooo`3oool0000000<0oooo00@000000?ooo`3oool000000`3oool01@000000oooo0?ooo`3o
ool000000080oooo00D000000?ooo`3oool0oooo000000030?ooo`040000003oool0oooo000000<0
oooo00D000000?ooo`3oool0oooo000000020?ooo`050000003oool0oooo0?ooo`000000[03oool0
02d0oooo00<000000?ooo`3oool0U03oool01@000000oooo0?ooo`3oool000000080oooo0`000000
0`3oool0000000000004000000050?ooo`00000000000000003oool01P0000001@3oool000000000
00000000oooo00H0000000D0oooo00000000000000000?ooo`06000000050?ooo`00000000000000
003oool01P000000103oool000000000000000020?ooo`050000003oool0oooo0?ooo`000000Y03o
ool002d0oooo00<000000?ooo`3oool0T@3oool00`000000oooo00000002000000040?ooo`000000
000000000080oooo00D000000?ooo`3oool0oooo000000020?ooo`050000003oool0oooo0?ooo`00
00000`3oool010000000oooo0?ooo`0000030?ooo`050000003oool0oooo0?ooo`0000000P3oool0
1@000000oooo0?ooo`3oool0000000<0oooo00@000000?ooo`3oool000000`3oool01@000000oooo
0?ooo`3oool000000080oooo00D000000?ooo`3oool0oooo000000020?ooo`<0000000H0oooo0000
0000000000000?ooo`00002Q0?ooo`00;@3oool00`000000oooo0?ooo`2=0?ooo`030000003oool0
oooo00<0000000<0oooo0000003oool00P3oool00`000000oooo0?ooo`0l0?ooo`060000003oool0
oooo0?ooo`000000oooo0`0000020?ooo`030000003oool0oooo09/0oooo000]0?ooo`030000003o
ool0oooo08T0oooo00<000000?ooo`3oool00`0000020?ooo`030000003oool0oooo04X0oooo00<0
00000?ooo`3oool00`0000020?ooo`030000003oool0oooo09L0oooo000]0?ooo`030000003oool0
oooo08P0oooo0`0000020?ooo`030000003oool0oooo0580oooo00<000000?ooo`3oool00`00002H
0?ooo`00;@3oool00`000000oooo0?ooo`260?ooo`040000003oool0oooo000005`0oooo00@00000
0?ooo`3oool00000UP3oool002d0oooo00<000000?ooo`3oool0Q@3oool300000600oooo0`00002E
0?ooo`00;@3oool300000880oooo00D000000?ooo`3oool0oooo0000001R0?ooo`050000003oool0
oooo0?ooo`000000TP3oool002d0oooo00<000000?ooo`3oool0P@3oool3000006P0oooo0`00002A
0?ooo`00;@3oool00`000000oooo0?ooo`220?ooo`030000003oool0oooo06P0oooo00<000000?oo
o`3oool0T03oool002d0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00;@3oool00`000000oooo
0?ooo`1n0?ooo`030000003oool0oooo0700oooo00<000000?ooo`3oool0S03oool002d0oooo00<0
00000?ooo`3oool0O@3oool300000700oooo0`00002=0?ooo`00;@3oool00`000000oooo0?ooo`1n
0?ooo`030000003oool0oooo0700oooo00<000000?ooo`3oool0S03oool002d0oooo00<000000?oo
o`3oool0o`3ooon10?ooo`00;@3oool300000?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0
N`3oool00`000000oooo0?ooo`1f0?ooo`030000003oool0oooo08T0oooo000]0?ooo`030000003o
ool0oooo07X0oooo0`00001f0?ooo`<00000RP3oool002d0oooo00<000000?ooo`3oool0N`3oool0
0`000000oooo0?ooo`1f0?ooo`030000003oool0oooo08T0oooo000]0?ooo`030000003oool0oooo
0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00;@3oool00`000000oooo
0?ooo`3o0?oooh40oooo000]0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool0OP3o
ool00`000000oooo0?ooo`250?ooo`00;@3oool3000007H0oooo0`00001n0?ooo`<00000QP3oool0
02d0oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`1n0?ooo`030000003oool0oooo
08D0oooo000]0?ooo`030000003oool0oooo0?l0ooooP@3oool00140oooo00D0omnL04P000000000
001d0;ooo`030?ooo`060?noM0000000M;oo0?ooo`3ooml0W4P00`0000000`1d_ol0oooo0?ooo`08
0?ooo`030000003oool0oooo0?l0ooooP@3oool00100oooo00L0oono07@0001d_ol0oooo0?noM000
07@0_ooo0080oooo00L0okmd0000001d_ol0oooo0?oog`2LB7@0_ooo0080oooo00<0_g@007Boo`3o
ool0203oool00`000000oooo0?ooo`3o0?oooh40oooo000@0?ooo`030;md0000B9`0gooo0080oooo
00<0_g@0001d_`3oool02P3oool00`3ooml0W4Qd0;ooo`080?ooo`030000003oool0oooo0?l0oooo
P@3oool000l0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`0;0?ooo`030;md001d
_ol0oooo00L0oooo00<000000?ooo`3oool0o`3ooon10?ooo`000`3oool00`3ooml0W4P000000007
0000000504RLg`3oool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0:0?ooo`030?oog`2L
B000B9cO00P0oooo00<000000?ooo`3oool0o`3ooon10?ooo`003`3oool00`3ogi`0B00007Boo`03
0?ooo`030?noM00004P0W=oo00X0oooo00<0gia80000001d_ol0203oool400000780oooo00<00000
0?ooo`3oool0QP3oool00`000000oooo0?ooo`210?ooo`003`3oool00`3ogi`0B00007Boo`030?oo
o`030?noM00004P0W=oo00H0oooo00L0gmoO0?ooo`3ookl0M0000000000007@0_ooo00P0oooo00<0
00000?ooo`3oool0LP3oool3000008H0oooo0`0000220?ooo`003`3oool00`3ogi`0B00007Boo`03
0?ooo`030?noM00004P0W=oo00H0oooo00H0gia800000000000000000000B02Lgol90?ooo`030000
003oool0oooo07<0oooo00<000000?ooo`3oool0QP3oool00`000000oooo0?ooo`210?ooo`003`3o
ool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00H0oooo00@0oono07@000000000B9cO
2`3oool00`000000oooo0?ooo`3o0?oooh40oooo000@0?ooo`030=nLB000B9`0gooo0080oooo00<0
_g@0001d_`3oool01`3oool00`2oM000M;oo0?ooo`0;0?ooo`030000003oool0oooo0?l0ooooP@3o
ool00100oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00P0oooo00H0omnL04P000000000
00000000B02Lgol80?ooo`030000003oool0oooo0?l0ooooP@3oool00140oooo00D0oono07@00000
0000001d0;ooo`0:0?ooo`050;md00000000000000000018W=l0203oool00`000000oooo0?ooo`3o
0?oooh40oooo000]0?ooo`<00000o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooh40
oooo000]0?ooo`030000003oool0oooo0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0o`3o
oon10?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooh40oooo000]0?ooo`030000003oool0oooo
0700oooo00<000000?ooo`3oool0S03oool00`000000oooo0?ooo`1n0?ooo`00;@3oool00`000000
oooo0?ooo`1_0?ooo`<00000S03oool3000007l0oooo000]0?ooo`030000003oool0oooo0700oooo
00<000000?ooo`3oool0S03oool00`000000oooo0?ooo`1n0?ooo`00;@3oool300000?l0ooooP@3o
ool002d0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`3o0?oo
oh40oooo000]0?ooo`030000003oool0oooo0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0
o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooh40oooo000]0?ooo`030000003oool0
oooo0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00;@3oool300000?l0
ooooP@3oool002d0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00;@3oool00`000000oooo0?oo
o`3o0?oooh40oooo000]0?ooo`030000003oool0oooo06`0oooo00<000000?ooo`3oool0U03oool0
0`000000oooo0?ooo`1j0?ooo`00;@3oool00`000000oooo0?ooo`1[0?ooo`<00000U03oool30000
07/0oooo000]0?ooo`030000003oool0oooo06`0oooo00<000000?ooo`3oool0U03oool00`000000
oooo0?ooo`1j0?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooh40oooo000]0?ooo`030000003o
ool0oooo0?l0ooooP@3oool002d0oooo0`00003o0?oooh40oooo000]0?ooo`030000003oool0oooo
0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0o`3ooon10?ooo`004@3oool01@3ogi`0B000
0000000007@0_ooo00<0oooo00<0okmd0000001d_ol01@3oool0103ooml0W4P00018W03Oool80?oo
o`030000003oool0oooo0?l0ooooP@3oool00100oooo00L0oono07@0001d_ol0oooo0?noM00007@0
_ooo0080oooo00<0okmd0000001d_ol01@3oool0103ooml0W4P00018W03Oool80?ooo`030000003o
ool0oooo0?l0ooooP@3oool00100oooo00<0_g@00018W03Oool00P3oool00`2oM00007Bo0?ooo`09
0?ooo`040?oog`2LB00004RL0=ooo`P0oooo00<000000?ooo`3oool0o`3ooon10?ooo`003`3oool0
0`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00D0oooo00<0_g@00000000000001@000000
0`1d_ol0oooo0?ooo`050?ooo`030000003oool0oooo0?l0ooooP@3oool000<0oooo00<0oooO09a8
000000001`0000001@18W=l0oooo0?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01@3oool0
203ogi`0M9cO0?ooo`3oool0oooO09a80000B9`0gooo203oool00`000000oooo0?ooo`3o0?oooh40
oooo000?0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01P3oool01`3OW4P0M;oo
0?ooo`3ooml0W4P00018W03Oool0203oool400000?l0ooooP03oool000l0oooo00<0omnL04P0001d
_ol00`3oool00`3o_g@0001809cOo`060?ooo`070?oog`2LB7@0_ooo0?oog`2LB00004RL0=ooo`08
0?ooo`030000003oool0oooo0?l0ooooP@3oool000l0oooo00<0omnL04P0001d_ol00`3oool00`3o
_g@0001809cOo`070?ooo`060?oOW018M;l0oooO09a80000B9`0gooo203oool00`000000oooo0?oo
o`3o0?oooh40oooo000?0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool0203oool0
1@3OW4P0M;oO09a80000B9`0gooo00P0oooo00<000000?ooo`3oool0J03oool00`000000oooo0?oo
o`2L0?ooo`030000003oool0oooo07H0oooo000@0?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0
001d_`3oool0203oool01@3ooml0W4Q804Q80000B9`0gooo00P0oooo00<000000?ooo`3oool0I`3o
ool3000009`0oooo0`00001g0?ooo`00403oool01`3ookl0M00007Boo`3oool0okmd0000M02oool0
2P3oool0103ogi`0B0000018W03Oool80?ooo`030000003oool0oooo06P0oooo00<000000?ooo`3o
ool0W03oool00`000000oooo0?ooo`1f0?ooo`004@3oool01@3ookl0M0000000000007@0_ooo00`0
oooo00<0gia80018W03Oool0203oool00`000000oooo0?ooo`3o0?oooh40oooo000]0?ooo`030000
003oool0oooo0?l0ooooP@3oool002d0oooo0`00003o0?oooh40oooo000]0?ooo`030000003oool0
oooo0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00;@3oool00`000000
oooo0?ooo`3o0?oooh40oooo000]0?ooo`030000003oool0oooo0?l0ooooP@3oool002d0oooo00<0
00000?ooo`3oool0o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooh40oooo000]0?oo
o`030000003oool0oooo0?l0ooooP@3oool002d0oooo0`00003o0?oooh40oooo000]0?ooo`030000
003oool0oooo0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0I@3oool00`000000oooo0?oo
o`2R0?ooo`030000003oool0oooo07<0oooo000]0?ooo`030000003oool0oooo06@0oooo0`00002R
0?ooo`<00000M03oool002d0oooo00<000000?ooo`3oool0I@3oool00`000000oooo0?ooo`2R0?oo
o`030000003oool0oooo07<0oooo000]0?ooo`030000003oool0oooo0?l0ooooP@3oool002d0oooo
00<000000?ooo`3oool0o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooh40oooo000]
0?ooo`<00000o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooh40oooo000]0?ooo`03
0000003oool0oooo0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00;@3o
ool00`000000oooo0?ooo`3o0?oooh40oooo000]0?ooo`030000003oool0oooo0?l0ooooP@3oool0
02d0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00;@3oool300000?l0ooooP@3oool002d0oooo
00<000000?ooo`3oool0H@3oool00`000000oooo0?ooo`2Z0?ooo`030000003oool0oooo06l0oooo
000]0?ooo`030000003oool0oooo0600oooo0`00002Z0?ooo`<00000L03oool00140oooo00D0omnL
04P000000000001d0;ooo`030?ooo`060?noM0000000M;oo0?ooo`3ookl0M0000`0000000`1d_ol0
oooo0?ooo`080?ooo`030000003oool0oooo0640oooo00<000000?ooo`3oool0ZP3oool00`000000
oooo0?ooo`1_0?ooo`00403oool01`3ookl0M00007Boo`3oool0okmd0000M02oool00P3oool01`3o
_g@0000007Boo`3oool0omnL04P00018W=l00P3oool00`2oM000B9cO0?ooo`080?ooo`030000003o
ool0oooo0?l0ooooP@3oool00100oooo00<0_g@00018W03Oool00P3oool00`2oM00007Bo0?ooo`0:
0?ooo`030?oo_`1d04P0W=oo00P0oooo00<000000?ooo`3oool0o`3ooon10?ooo`003`3oool00`3o
okl0M00004RLg`030?ooo`030=nLB00007@0_ooo00X0oooo00<0oono07@0001d_ol0203oool00`00
0000oooo0?ooo`3o0?oooh40oooo00030?ooo`030?oog`2LB000000000L0000000D0B9cO0?ooo`3o
gi`0B00007Boo`030?ooo`030?noM00004P0W=oo00X0oooo00<0okmd0000001d_ol0203oool00`00
0000oooo0?ooo`3o0?oooh40oooo000?0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02L
gol02@3oool0103ogi`0B0000000B02Lgol80?ooo`@00000o`3ooon00?ooo`003`3oool00`3ogi`0
B00007Boo`030?ooo`030?noM00004P0W=oo00L0oooo00D0okmd000000000000001809cOo`090?oo
o`030000003oool0oooo0?l0ooooP@3oool000l0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0
001809cOo`090?ooo`030;md0000M;l0oooo00T0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00
3`3oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00X0oooo00<0_g@004RLg`3oool0
203oool00`000000oooo0?ooo`3o0?oooh40oooo000@0?ooo`030=nLB000B9`0gooo0080oooo00<0
_g@0001d_`3oool01@3oool00`3ooml0W9bo0?ooo`020?ooo`030=nLB00007@0_ooo00P0oooo00<0
00000?ooo`3oool0o`3ooon10?ooo`00403oool01`3ookl0M00007Boo`3oool0okmd0000M02oool0
1`3oool01P3o_g@0B9cO0?ooo`3ookl0M000001d_`T0oooo00<000000?ooo`3oool0o`3ooon10?oo
o`004@3oool01@3ookl0M0000000000007@0_ooo00T0oooo00D0gia8000000000000001d0;ooo`09
0?ooo`030000003oool0oooo05d0oooo00<000000?ooo`3oool0/P3oool00`000000oooo0?ooo`1[
0?ooo`00;@3oool00`000000oooo0?ooo`1L0?ooo`<00000/P3oool3000006`0oooo000]0?ooo`<0
0000G@3oool00`000000oooo0?ooo`2b0?ooo`030000003oool0oooo06/0oooo000]0?ooo`030000
003oool0oooo0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00;@3oool0
0`000000oooo0?ooo`3o0?oooh40oooo000]0?ooo`030000003oool0oooo0?l0ooooP@3oool002d0
oooo00<000000?ooo`3oool0o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooh40oooo
000]0?ooo`030000003oool0oooo0?l0ooooP@3oool002d0oooo0`00003o0?oooh40oooo000]0?oo
o`030000003oool0oooo0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0F@3oool00`000000
oooo0?ooo`2j0?ooo`030000003oool0oooo06L0oooo000]0?ooo`030000003oool0oooo05P0oooo
0`00002j0?ooo`<00000J03oool002d0oooo00<000000?ooo`3oool0F@3oool00`000000oooo0?oo
o`2j0?ooo`030000003oool0oooo06L0oooo000]0?ooo`030000003oool0oooo0?l0ooooP@3oool0
02d0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooh40
oooo000]0?ooo`<00000o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooh40oooo000]
0?ooo`030000003oool0oooo0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0o`3ooon10?oo
o`00;@3oool00`000000oooo0?ooo`3o0?oooh40oooo000]0?ooo`030000003oool0oooo05H0oooo
00<000000?ooo`3oool0`03oool00`000000oooo0?ooo`1T0?ooo`00;@3oool00`000000oooo0?oo
o`1E0?ooo`<00000`03oool3000006D0oooo000]0?ooo`030000003oool0oooo05H0oooo00<00000
0?ooo`3oool0`03oool00`000000oooo0?ooo`1T0?ooo`00;@3oool300000?l0ooooP@3oool002d0
oooo00<000000?ooo`3oool0o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooh40oooo
000A0?ooo`050?oOW018000000000000M02oool00`3oool01@3o_g@0000007Boo`3oool0_g@000H0
000000<0B9cO0?ooo`3oool01P3oool00`000000oooo0?ooo`3o0?oooh40oooo000@0?ooo`070?oo
_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`020?ooo`030?noM0000000M;oo0080oooo00<0gia80000
000000000`0000000`00B9`0gooo0?ooo`060?ooo`030000003oool0oooo0?l0ooooP@3oool00100
oooo00<0_g@00018W03Oool00P3oool00`2oM00007Bo0?ooo`070?ooo`030=nLB018W=l0oooo0080
oooo00<0giad09cOo`3oool01P3oool00`000000oooo0?ooo`3o0?oooh40oooo000?0?ooo`030?oo
_`1d0000B9cO00<0oooo00<0gia80000M02oool0203oool00`3OW4P0M;oo0?ooo`0:0?ooo`030000
003oool0oooo0580oooo00<000000?ooo`3oool0b03oool00`000000oooo0?ooo`1P0?ooo`000`3o
ool00`3ooml0W4P0000000070000000504RLg`3oool0omnL04P0001d_ol00`3oool00`3o_g@00018
09cOo`090?ooo`030;mdB02Lgol0oooo00T0oooo00<000000?ooo`3oool0D@3oool300000<P0oooo
0`00001Q0?ooo`003`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00T0oooo00<0
oooO09a8M02oool02@3oool400000540oooo00<000000?ooo`3oool0b03oool00`000000oooo0?oo
o`1P0?ooo`003`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00X0oooo00<0okmd
001d_`3oool0203oool00`000000oooo0?ooo`3o0?oooh40oooo000?0?ooo`030?oOW0180000M;oo
00<0oooo00<0okmd0000B02Lgol02P3oool00`3ookl0M01809cOo`080?ooo`030000003oool0oooo
0?l0ooooP@3oool000l0oooo00<0oono07@00018W=l00`3oool00`3OW4P0001d0;ooo`0:0?ooo`03
0?oo_`1d0000B9cO00P0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00403oool00`3OW4P004RL
0=ooo`020?ooo`030;md0000M;l0oooo00D0oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0B000
04RLg`080?ooo`030000003oool0oooo0?l0ooooP@3oool00100oooo00L0oono07@0001d_ol0oooo
0?noM00007@0_ooo00H0oooo00P0oooO09a8B02Lgol0oooo0?oo_`1d0000001d0;ooo`P0oooo00<0
00000?ooo`3oool0o`3ooon10?ooo`004@3oool01@3ookl0M0000000000007@0_ooo00P0oooo00H0
oooO09a80000000000000000M02oool90?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3o
ool0d03oool00`000000oooo0?ooo`1L0?ooo`00;@3oool00`000000oooo0?ooo`1=0?ooo`<00000
d03oool3000005d0oooo000]0?ooo`<00000CP3oool00`000000oooo0?ooo`3@0?ooo`030000003o
ool0oooo05`0oooo000]0?ooo`030000003oool0oooo0?l0ooooP@3oool002d0oooo00<000000?oo
o`3oool0o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooh40oooo000]0?ooo`030000
003oool0oooo0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00;@3oool0
0`000000oooo0?ooo`1;0?ooo`030000003oool0oooo0=H0oooo00<000000?ooo`3oool0F@3oool0
02d0oooo0`00001:0?ooo`<00000eP3oool3000005X0oooo000]0?ooo`030000003oool0oooo04/0
oooo00<000000?ooo`3oool0eP3oool00`000000oooo0?ooo`1I0?ooo`00;@3oool00`000000oooo
0?ooo`3o0?oooh40oooo000]0?ooo`030000003oool0oooo0?l0ooooP@3oool002d0oooo00<00000
0?ooo`3oool0o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo
0=h0oooo00<000000?ooo`3oool0E@3oool002d0oooo00<000000?ooo`3oool0AP3oool300000=h0
oooo0`00001F0?ooo`00;@3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo0=h0oooo
00<000000?ooo`3oool0E@3oool002d0oooo0`00003o0?oooh40oooo000]0?ooo`030000003oool0
oooo0?l0ooooP@3oool002d0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00;@3oool00`000000
oooo0?ooo`130?ooo`030000003oool0oooo0>H0oooo00<000000?ooo`3oool0D@3oool002d0oooo
00<000000?ooo`3oool0@P3oool300000>H0oooo0`00001B0?ooo`00;@3oool00`000000oooo0?oo
o`130?ooo`030000003oool0oooo0>H0oooo00<000000?ooo`3oool0D@3oool002d0oooo00<00000
0?ooo`3oool0o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooh40oooo000]0?ooo`<0
0000o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo0>`0oooo
00<000000?ooo`3oool0CP3oool002d0oooo00<000000?ooo`3oool0?`3oool300000>`0oooo0`00
001?0?ooo`004@3oool01@3ogi`0B0000000000007@0_ooo00<0oooo00<0okmd0000001d_ol00`3o
ool01P2oM000000000000000000004RL0=ooo`P0oooo00<000000?ooo`3oool0@03oool00`000000
oooo0?ooo`3/0?ooo`030000003oool0oooo04h0oooo000@0?ooo`070?oo_`1d0000M;oo0?ooo`3o
_g@0001d0;ooo`020?ooo`030?noM0000000M;oo00@0oooo00<0omnL04P0001d_ol02P3oool00`00
0000oooo0?ooo`3o0?oooh40oooo000@0?ooo`030;md0000B9`0gooo0080oooo00<0_g@0001d_`3o
ool0203oool00`3ogi`0B00007Boo`0:0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3o
ool0m03oool00`000000oooo0?ooo`1:0?ooo`003`3oool00`3ookl0M00004RLg`030?ooo`030=nL
B00007@0_ooo00P0oooo00<0omnL04P0001d_ol02P3oool00`000000oooo0?ooo`0k0?ooo`<00000
m03oool3000004/0oooo00030?ooo`030?oog`2LB000000000L0000000D0B9cO0?ooo`3ogi`0B000
07Boo`030?ooo`030?noM00004P0W=oo00P0oooo00<0omnL04P0001d_ol02P3oool00`000000oooo
0?ooo`0l0?ooo`030000003oool0oooo0?@0oooo00<000000?ooo`3oool0BP3oool000l0oooo00<0
omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`080?ooo`030?oOW0180000M;oo00X0oooo1000
003o0?oooh00oooo000?0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0203oool0
0`3ogi`0B00007Boo`0:0?ooo`030000003oool0oooo0?l0ooooP@3oool000l0oooo00<0omnL04P0
001d_ol00`3oool00`3o_g@0001809cOo`080?ooo`030?oOW0180000M;oo00X0oooo00<000000?oo
o`3oool0>03oool00`000000oooo0?ooo`3l0?ooo`030000003oool0oooo04H0oooo000?0?ooo`03
0?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool0203oool00`3ogi`0B00007Boo`0:0?ooo`03
0000003oool0oooo03L0oooo0`00003l0?ooo`<00000A`3oool00100oooo00<0gia80018W03Oool0
0P3oool00`2oM00007Bo0?ooo`080?ooo`030?oOW0180000M;oo00X0oooo00<000000?ooo`3oool0
>03oool00`000000oooo0?ooo`3l0?ooo`030000003oool0oooo04H0oooo000@0?ooo`070?oo_`1d
0000M;oo0?ooo`3o_g@0001d0;ooo`070?ooo`050?oog`2LB00000000000001d_ol02P3oool00`00
0000oooo0?ooo`0e0?ooo`030000003oool0oooo0?l0oooo0`3oool00`000000oooo0?ooo`130?oo
o`004@3oool01@3ookl0M0000000000007@0_ooo00X0oooo00<0okmd0000001d_ol02P3oool00`00
0000oooo0?ooo`0d0?ooo`<00000o`3oool30?ooo`<00000A03oool002d0oooo00<000000?ooo`3o
ool0=@3oool00`000000oooo0?ooo`3o0?ooo`<0oooo00<000000?ooo`3oool0@`3oool002d0oooo
0`00003o0?oooh40oooo000]0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0o`3o
ool;0?ooo`030000003oool0oooo03l0oooo000]0?ooo`030000003oool0oooo0300oooo0`00003o
0?ooo`/0oooo0`0000100?ooo`00;@3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo
0?l0oooo2`3oool00`000000oooo0?ooo`0o0?ooo`00;@3oool00`000000oooo0?ooo`0]0?ooo`03
0000003oool0oooo0?l0oooo4`3oool00`000000oooo0?ooo`0k0?ooo`00;@3oool00`000000oooo
0?ooo`0/0?ooo`<00000o`3ooolC0?ooo`<00000?03oool002d0oooo00<000000?ooo`3oool0;@3o
ool00`000000oooo0?ooo`3o0?oooa<0oooo00<000000?ooo`3oool0>`3oool002d0oooo00<00000
0?ooo`3oool0:P3oool00`000000oooo0?ooo`3o0?oooaT0oooo00<000000?ooo`3oool0>03oool0
02d0oooo0`00000Y0?ooo`<00000o`3ooolI0?ooo`<00000>@3oool002d0oooo00<000000?ooo`3o
ool0:P3oool00`000000oooo0?ooo`3o0?oooaT0oooo00<000000?ooo`3oool0>03oool002d0oooo
00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0E0?ooo`030;md00000000000000@00000
00T0B9cO0?ooo`3oool0oooo0?oOW018000000000000M02oool0?P3oool0103ooml0W4P00018W03O
ool30?ooo`050?oOW018000000000000M02oool0?03oool01@3ogi`0B000000000000000M;oo00@0
oooo00D0omnL04P000000000001d0;ooo`0k0?ooo`050=nLB0000000000000000018W=l0103oool0
1@3ogi`0B0000000000007@0_ooo01h0oooo00<000000?ooo`3oool05`3oool01P2oM00000000000
0000000004RL0=ooo`<0oooo00D0omnL04P000000000001d0;ooo`040?ooo`050?oOW01800000000
0000M02oool01P3oool002d0oooo00<000000?ooo`3oool09@3oool3000001L0oooo00<0gia80000
000000000`0000002P00B9`0gooo0?ooo`3ookl0M00007Boo`3oool0okmd0000M02ooolm0?ooo`04
0?oog`2LB00004RL0=ooo`80oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo03X0oooo00L0
omnL04P00018W=l0oooo0?oo_`1d0000M;oo0080oooo00L0oono07@0001d_ol0oooo0?noM00007@0
_ooo03T0oooo00L0gia80000001d_ol0oooo0?oog`2LB000B9cO0080oooo00L0oono07@0001d_ol0
oooo0?noM00007@0_ooo01`0oooo0`00000I0?ooo`030?oOW0180000M;oo00@0oooo00L0oono07@0
001d_ol0oooo0?noM00007@0_ooo0080oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo00D0
oooo000]0?ooo`030000003oool0oooo0280oooo00D000000?ooo`3oool0oooo0000000I0?ooo`03
0=nLB018W=l0oooo0080oooo00H0giad09cOo`3oool0_g@00018W03Oool20?ooo`030;md0000M;l0
oooo03`0oooo00@0oooO09a80000B9`0gooo0P3oool00`2oM00004RL0=ooo`020?ooo`030;md0000
M;l0oooo03P0oooo00<0oooO09a80000M;l00`3oool01`3o_g@004RL0=ooo`3oool0_g@00018W03O
ool00P3oool00`2oM00007Bo0?ooo`0g0?ooo`030?oog`2LB00007Bo00<0oooo00L0okmd0018W03O
ool0oooo0;md0000B9`0gooo0080oooo00<0_g@0001d_`3oool0703oool01@000000oooo0?ooo`3o
ool0000001H0oooo00<0omnL04P0001d_ol0103oool00`2oM00004RL0=ooo`020?ooo`070;md0000
M;l0oooo0?ooo`2oM00004RL0=ooo`020?ooo`030;md0000M;l0oooo00@0oooo000]0?ooo`030000
003oool0oooo0240oooo0`00000M0?ooo`030=nLB01d_ol0oooo00<0oooo00<0oono07@00018W=l0
0`3oool00`3OW4P0001d0;ooo`0h0?ooo`030;md00000000000000D0000000@0M;oo0?oo_`1d0000
B9cO0`3oool00`3OW4P0001d0;ooo`0h0?ooo`030?oOW0180000M;oo00<0oooo00H0okmd0000B02L
gol0oono07@00018W=l30?ooo`030=nLB00007@0_ooo03P0oooo00<0_g@0001d_`3oool00P3oool0
1P2oM00004RL0=ooo`3ookl0M00004RLg`<0oooo00<0gia80000M02oool07`3oool3000001D0oooo
00<0omnL04P0001d_ol00`3oool00`3ookl0M00004RLg`030?ooo`060=nLB00007@0_ooo0?oo_`1d
0000B9cO0`3oool00`3OW4P0001d0;ooo`040?ooo`00;@3oool00`000000oooo0?ooo`0R0?ooo`03
0000003oool0oooo01d0oooo00<0_gA809cOo`3oool00P3oool00`3ogi`0B00007Boo`030?ooo`03
0?noM00004P0W=oo03P0oooo00`0omnL07BLg`3oool0oooo0?oog`2LB00004RL0=ooo`3oool0omnL
04P0001d_ol30?ooo`030?noM00004P0W=oo03P0oooo00<0omnL04P0001d_ol00`3oool01P3o_g@0
001809cOo`3ogi`0B00007Boo`<0oooo00<0okmd0000B02Lgol0>03oool02`3ogi`0B01d0;ooo`3o
ool0_g@00000001d_ol0oooo0?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0803oool00`00
0000oooo0?ooo`0D0?ooo`030?oOW0180000M;oo00<0oooo00<0omnL04P0001d_ol00`3oool01P3o
_g@0001809cOo`3ogi`0B00007Boo`<0oooo00<0okmd0000B02Lgol0103oool002d0oooo00<00000
0?ooo`3oool07P3oool00`000000oooo0?ooo`0Q0?ooo`030?oog`2LB7@0_ooo0080oooo00<0omnL
04P0001d_ol00`3oool00`3o_g@0001809cOo`0i0?ooo`0;0=nLB01d_ol0oooo0?oog`2LB00004RL
0=ooo`3oool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0h0?ooo`030?oOW0180000M;oo
00<0oooo00H0_g@00000B02Lgol0omnL04P0001d_ol30?ooo`030?noM00004P0W=oo03T0oooo00D0
oono07A8B0180000000004RLg`020?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0
903oool00`000000oooo0?ooo`0@0?ooo`030?oOW0180000M;oo00<0oooo00<0omnL04P0001d_ol0
0`3oool01P3o_g@0001809cOo`3ogi`0B00007Boo`<0oooo00<0okmd0000B02Lgol0103oool002d0
oooo0`00000M0?ooo`<000008`3oool01`3o_g@007Bo0?ooo`3oool0omnL04P0001d_ol00`3oool0
0`3o_g@0001809cOo`0i0?ooo`0;0?oog`2LB7@0_ooo0?oog`2LB00004RL0=ooo`3oool0omnL04P0
001d_ol00`3oool00`3o_g@0001809cOo`0h0?ooo`0<0?oo_`1d0000000004RLg`3oool0okmd0000
0000M;l0oooo0?oOW0180000M;oo0`3oool00`3o_g@0001809cOo`0i0?ooo`040?oOW01800000000
001d_`<0oooo00<0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0S0?ooo`<000004@3oool0
0`3ogi`0B00007Boo`030?ooo`030?oOW0180000M;oo00<0oooo00H0okmd0000B02Lgol0omnL04P0
001d_ol30?ooo`030?noM00004P0W=oo00@0oooo000]0?ooo`030000003oool0oooo01/0oooo00@0
00000?ooo`3oool00000903oool01`3ookl0M01809cOo`3oool0omnL04P0001d_ol00`3oool00`3o
_g@0001809cOo`0j0?ooo`0:0?oOW018M;l0oooO09a80000B9`0gooo0?ooo`3ogi`0B00007Boo`<0
oooo00<0okmd0000B02Lgol0>@3oool00`2oM00004RL0;nLB002000000060018W03Oool0oooo0?oO
W0180000M;oo0`3oool00`3o_g@0001809cOo`0h0?ooo`060?oOW0180000001d0;ooo`3o_g@007Bo
0P3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo02@0oooo00@000000?ooo`3oool0
00003`3oool00`3ogi`0B00007Boo`030?ooo`030?oOW0180000M;oo00<0oooo00H0okmd0000B02L
gol0omnL04P0001d_ol30?ooo`030?noM00004P0W=oo00@0oooo000]0?ooo`030000003oool0oooo
01X0oooo0`00000V0?ooo`070?oo_`1d0000B9cO0?ooo`3ookl0M00004RLg`030?ooo`030=nLB000
07@0_ooo03/0oooo00T0gia807Bog`2LB00004RL0=ooo`3oool0oono07@00018W=l00`3oool00`3O
W4P0001d0;ooo`0i0?ooo`030?oOW0180000M;oo00D0oooo00<0oono07@00018W=l00`3oool00`3O
W4P0001d0;ooo`0h0?ooo`030;md000007@0_ooo0080oooo00H0_g@004RLg`3oool0oono07@00018
W=l30?ooo`030=nLB00007@0_ooo02H0oooo0`00000>0?ooo`030?oOW0180000M;oo00<0oooo00<0
oono07@00018W=l00`3oool01P3OW4P0001d0;ooo`3ookl0M00004RLg`<0oooo00<0gia80000M02o
ool0103oool002d0oooo00<000000?ooo`3oool05`3oool01@000000oooo0?ooo`3oool000000280
oooo00<0oknL09cOo`3oool00P3oool00`3ogi`0B00004RLg`020?ooo`030=nLB000B9`0gooo0080
oooo00<0_g@0001d_`3oool0>`3oool01@3ooml0W4Q804Q80000B9`0gooo0080oooo00<0gia80018
W03Oool00P3oool00`2oM00007Bo0?ooo`0j0?ooo`030=nLB00004P0W=oo00D0oooo00<0gia80018
W03Oool00P3oool00`2oM00007Bo0?ooo`0h0?ooo`030;md0000B9`0gooo0080oooo00L0gia8001d
_`3oool0oooo0=nLB000B9`0gooo0080oooo00<0_g@0001d_`3oool09`3oool01@000000oooo0?oo
o`3oool0000000/0oooo00<0omnL04P0001d_ol0103oool00`3OW4P004RL0=ooo`020?ooo`070;md
0000M;l0oooo0?ooo`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00@0oooo000]0?ooo`03
0000003oool0oooo01H0oooo0`00000U0?ooo`080?oog`2LB4P0W=oo0?ooo`3ookl0M0000000M02o
ool20?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`0m0?ooo`040?oOW018000004RL0=oo
o`80oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo03`0oooo00<0okmd00000000M;l0103o
ool01`3ookl0M00007Boo`3oool0okmd0000M02oool0>@3oool01`3o_g@0000007Boo`3oool0oono
07@0001d_ol00P3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool0:`3oool3000000P0oooo
00D0oooO09a800000000000007Boo`040?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`02
0?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`050?ooo`00;@3oool00`000000oooo0?oo
o`0C0?ooo`050000003oool0oooo0?ooo`0000009`3oool01P3ooml0W4P0000000000000001d0;oo
o`@0oooo00D0oono07@000000000001d0;ooo`0o0?ooo`030=nLB000B9`0gooo00<0oooo00D0oono
07@000000000001d0;ooo`0n0?ooo`050?oog`2LB00000000000M02oool00P3oool01@3ookl0M000
0000000007@0_ooo03/0oooo00D0okmd000000000000000004RLg`040?ooo`050?oo_`1d00000000
0000M02oool0;@3oool01@000000oooo0?ooo`3oool0000000L0oooo00<0okmd0000001d_ol01@3o
ool01@3ookl0M0000000000007@0_ooo00@0oooo00D0oono07@000000000001d0;ooo`060?ooo`00
;@3oool00`000000oooo0?ooo`0B0?ooo`<00000o`3ooom70?ooo`<000008P3oool002d0oooo00<0
00000?ooo`3oool0403oool010000000oooo0?ooo`00003o0?ooodT0oooo00@000000?ooo`3oool0
0000803oool002d0oooo0`00000?0?ooo`<00000o`3ooom=0?ooo`<000007`3oool002d0oooo00<0
00000?ooo`3oool0303oool01@000000oooo0?ooo`3oool000000?l0ooooC`3oool01@000000oooo
0?ooo`3oool0000001`0oooo000]0?ooo`030000003oool0oooo00/0oooo0`00003o0?oooeD0oooo
0`00000K0?ooo`00;@3oool00`000000oooo0?ooo`080?ooo`050000003oool0oooo0?ooo`000000
o`3ooomG0?ooo`050000003oool0oooo0?ooo`000000603oool002d0oooo00<000000?ooo`3oool0
1`3oool300000?l0ooooG@3oool3000001L0oooo000]0?ooo`030000003oool0oooo00D0oooo00@0
00000?ooo`3oool00000o`3ooomO0?ooo`040000003oool0oooo000001D0oooo000]0?ooo`050000
003oool0oooo0?ooo`0000000P3oool300000?l0ooooH`3oool300000080oooo00<000000?ooo`3o
ool03`3oool002@0ooooo`000025000000P0oooo000]0?ooo`050000003oool0oooo0?ooo`000000
3P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00l0oooo000]0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0?0?ooo`00;@3oool0
0`000000oooo0?ooo`170?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0A`3oool0
0`000000oooo0?ooo`160?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool03`3oool0
02d0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00;@3oool00`000000oooo0?ooo`3o0?oooh40
oooo000]0?ooo`030000003oool0oooo0?l0ooooP@3oool00?l0oooo/@3oool00?l0oooo/@3oool0
0?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool0
0?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool0
0?l0oooo/@3oool00001\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {-12.425, -0.626802, \
0.406847, 0.0038085}}]
}, Closed]],

Cell["and to make a three-dimensional plot.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot3D[\[Phi], PlotRange \[Rule] All];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.01048e-016 1.04977 -0.0679587 1.04977 [
[.14109 .2015 -15.9256 -15.6875 ]
[.14109 .2015 .07441 0 ]
[.26628 .15225 -15.2517 -15.6875 ]
[.26628 .15225 .74832 0 ]
[.39776 .10054 -14.5778 -15.6875 ]
[.39776 .10054 1.42224 0 ]
[.53601 .04619 -13.9039 -15.6875 ]
[.53601 .04619 2.09615 0 ]
[.68156 -0.01098 -18.1912 -15.6875 ]
[.68156 -0.01098 3.80884 0 ]
[.76488 .0931 0 -10.7472 ]
[.76488 .0931 16 4.94035 ]
[.82636 .18723 0 -10.5891 ]
[.82636 .18723 16 5.09839 ]
[.88261 .27338 0 -10.4474 ]
[.88261 .27338 16 5.24011 ]
[.93428 .35251 0 -10.3196 ]
[.93428 .35251 16 5.36792 ]
[.98191 .42546 0 -10.2037 ]
[.98191 .42546 22 5.48377 ]
[.02411 .26511 -18.3125 -4.78001 ]
[.02411 .26511 0 10.9075 ]
[.01568 .31795 -27.3125 -4.85789 ]
[.01568 .31795 0 10.8296 ]
[.00696 .37264 -10 -4.93876 ]
[.00696 .37264 0 10.7487 ]
[-0.00208 .42927 -19 -5.02281 ]
[-0.00208 .42927 0 10.6647 ]
[-0.01144 .48795 -10 -5.11022 ]
[-0.01144 .48795 0 10.5773 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.15032 .21083 m
.15494 .21549 L
s
gsave
.14109 .2015 -76.9256 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
.2751 .16197 m
.2795 .16683 L
s
gsave
.26628 .15225 -76.2517 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
.4061 .11067 m
.41026 .11574 L
s
gsave
.39776 .10054 -75.5778 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(60) show
1.000 setlinewidth
grestore
.5438 .05675 m
.5477 .06203 L
s
gsave
.53601 .04619 -74.9039 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(80) show
1.000 setlinewidth
grestore
.68874 0 m
.69233 .00549 L
s
gsave
.68156 -0.01098 -79.1912 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100) show
1.000 setlinewidth
grestore
.125 Mabswid
.06057 .24597 m
.06342 .24868 L
s
.09013 .23439 m
.09296 .23713 L
s
.12005 .22268 m
.12285 .22545 L
s
.18096 .19883 m
.1837 .20166 L
s
.21196 .18669 m
.21467 .18955 L
s
.24334 .17441 m
.24601 .17729 L
s
.30724 .14938 m
.30985 .15233 L
s
.33979 .13664 m
.34236 .13962 L
s
.37274 .12374 m
.37527 .12675 L
s
.43987 .09745 m
.44233 .10052 L
s
.47408 .08405 m
.4765 .08716 L
s
.50872 .07049 m
.5111 .07363 L
s
.57933 .04284 m
.58163 .04604 L
s
.61533 .02874 m
.61758 .03198 L
s
.65179 .01447 m
.654 .01773 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.75258 .09765 m
.74642 .09993 L
s
gsave
.76488 .0931 -61 -14.7472 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
.81397 .19156 m
.80778 .19373 L
s
gsave
.82636 .18723 -61 -14.5891 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
.87016 .27751 m
.86393 .27958 L
s
gsave
.88261 .27338 -61 -14.4474 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(60) show
1.000 setlinewidth
grestore
.92177 .35646 m
.91551 .35844 L
s
gsave
.93428 .35251 -61 -14.3196 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(80) show
1.000 setlinewidth
grestore
.96935 .42924 m
.96306 .43113 L
s
gsave
.98191 .42546 -61 -14.2037 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100) show
1.000 setlinewidth
grestore
.125 Mabswid
.70266 .0213 m
.69899 .02272 L
s
.71969 .04735 m
.71602 .04876 L
s
.73633 .0728 m
.73264 .07418 L
s
.76845 .12194 m
.76475 .12329 L
s
.78397 .14567 m
.78026 .147 L
s
.79914 .16888 m
.79543 .17019 L
s
.82848 .21376 m
.82476 .21504 L
s
.84267 .23547 m
.83895 .23674 L
s
.85656 .25671 m
.85283 .25797 L
s
.88347 .29787 m
.87973 .2991 L
s
.8965 .31781 m
.89275 .31902 L
s
.90926 .33733 m
.90551 .33853 L
s
.93402 .37521 m
.93027 .37638 L
s
.94603 .39358 m
.94227 .39473 L
s
.95781 .41158 m
.95404 .41273 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03634 .26033 m
.04245 .25795 L
s
gsave
.02411 .26511 -79.3125 -8.78001 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.02795 .31328 m
.03408 .31095 L
s
gsave
.01568 .31795 -88.3125 -8.85789 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
71.313 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.01926 .36808 m
.02541 .3658 L
s
gsave
.00696 .37264 -71 -8.93876 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.01027 .42483 m
.01644 .42261 L
s
gsave
-0.00208 .42927 -80 -9.02281 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.00095 .48363 m
.00715 .48147 L
s
gsave
-0.01144 .48795 -71 -9.11022 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.125 Mabswid
.03468 .27078 m
.03835 .26935 L
s
.03301 .2813 m
.03669 .27988 L
s
.03134 .29189 m
.03501 .29047 L
s
.02965 .30255 m
.03332 .30114 L
s
.02623 .32409 m
.02991 .3227 L
s
.02451 .33497 m
.02819 .33359 L
s
.02277 .34593 m
.02646 .34455 L
s
.02102 .35697 m
.02471 .35559 L
s
.01749 .37927 m
.02118 .37791 L
s
.0157 .39054 m
.0194 .38919 L
s
.0139 .40189 m
.0176 .40054 L
s
.01209 .41332 m
.01579 .41198 L
s
.00843 .43642 m
.01214 .4351 L
s
.00658 .4481 m
.01029 .44678 L
s
.00472 .45986 m
.00843 .45855 L
s
.00284 .4717 m
.00656 .4704 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.659 .752 .919 r
.39852 .69908 .40169 .70185 .40701 .70035 .40384 .69753 Metetra
.659 .749 .916 r
.40384 .69753 .40701 .70035 .41233 .69885 .40917 .69598 Metetra
.66 .746 .914 r
.40917 .69598 .41233 .69885 .41766 .69735 .41452 .69442 Metetra
.66 .743 .911 r
.41452 .69442 .41766 .69735 .42301 .69584 .41987 .69286 Metetra
.66 .74 .909 r
.41987 .69286 .42301 .69584 .42837 .69433 .42523 .6913 Metetra
.66 .737 .907 r
.42523 .6913 .42837 .69433 .43373 .69282 .43061 .68974 Metetra
.66 .734 .904 r
.43061 .68974 .43373 .69282 .43911 .6913 .43599 .68817 Metetra
.66 .731 .902 r
.43599 .68817 .43911 .6913 .4445 .68978 .44139 .6866 Metetra
.66 .728 .9 r
.44139 .6866 .4445 .68978 .4499 .68826 .44679 .68503 Metetra
.66 .726 .898 r
.44679 .68503 .4499 .68826 .45531 .68674 .45221 .68345 Metetra
.66 .723 .895 r
.45221 .68345 .45531 .68674 .46073 .68521 .45764 .68187 Metetra
.66 .72 .893 r
.45764 .68187 .46073 .68521 .46616 .68368 .46308 .68029 Metetra
.66 .718 .891 r
.46308 .68029 .46616 .68368 .4716 .68214 .46852 .67871 Metetra
.66 .715 .889 r
.46852 .67871 .4716 .68214 .47705 .6806 .47398 .67712 Metetra
.66 .713 .887 r
.47398 .67712 .47705 .6806 .48252 .67906 .47945 .67553 Metetra
.66 .71 .885 r
.47945 .67553 .48252 .67906 .48799 .67752 .48494 .67395 Metetra
.66 .708 .883 r
.48494 .67395 .48799 .67752 .49348 .67597 .49043 .67235 Metetra
.66 .706 .881 r
.49043 .67235 .49348 .67597 .49897 .67442 .49593 .67076 Metetra
.659 .704 .88 r
.49593 .67076 .49897 .67442 .50448 .67287 .50145 .66916 Metetra
.659 .702 .878 r
.50145 .66916 .50448 .67287 .51 .67132 .50697 .66757 Metetra
.659 .7 .877 r
.50697 .66757 .51 .67132 .51553 .66976 .51251 .66597 Metetra
.659 .698 .875 r
.51251 .66597 .51553 .66976 .52107 .66819 .51805 .66437 Metetra
.659 .696 .874 r
.51805 .66437 .52107 .66819 .52662 .66663 .52361 .66278 Metetra
.658 .695 .873 r
.52361 .66278 .52662 .66663 .53219 .66506 .52918 .66118 Metetra
.658 .694 .872 r
.52918 .66118 .53219 .66506 .53776 .66349 .53476 .65958 Metetra
.658 .692 .871 r
.53476 .65958 .53776 .66349 .54335 .66191 .54035 .65798 Metetra
.658 .691 .87 r
.54035 .65798 .54335 .66191 .54895 .66034 .54596 .65638 Metetra
.657 .691 .87 r
.54596 .65638 .54895 .66034 .55455 .65875 .55157 .65478 Metetra
.657 .69 .869 r
.55157 .65478 .55455 .65875 .56017 .65717 .5572 .65318 Metetra
.657 .69 .869 r
.5572 .65318 .56017 .65717 .56581 .65558 .56284 .65159 Metetra
.656 .69 .869 r
.56284 .65159 .56581 .65558 .57145 .65399 .56848 .65 Metetra
.656 .69 .87 r
.56848 .65 .57145 .65399 .5771 .6524 .57415 .64841 Metetra
.656 .69 .87 r
.57415 .64841 .5771 .6524 .58277 .6508 .57982 .64682 Metetra
.656 .691 .871 r
.57982 .64682 .58277 .6508 .58845 .6492 .5855 .64523 Metetra
.655 .692 .872 r
.5855 .64523 .58845 .6492 .59414 .64759 .5912 .64365 Metetra
.655 .693 .873 r
.5912 .64365 .59414 .64759 .59984 .64599 .59691 .64207 Metetra
.655 .695 .875 r
.59691 .64207 .59984 .64599 .60555 .64438 .60263 .64049 Metetra
.655 .697 .877 r
.60263 .64049 .60555 .64438 .61128 .64276 .60836 .63891 Metetra
.654 .699 .879 r
.60836 .63891 .61128 .64276 .61701 .64114 .6141 .63734 Metetra
.654 .702 .882 r
.6141 .63734 .61701 .64114 .62276 .63952 .61986 .63577 Metetra
.654 .705 .884 r
.61986 .63577 .62276 .63952 .62852 .6379 .62563 .6342 Metetra
.653 .708 .887 r
.62563 .6342 .62852 .6379 .63429 .63627 .63141 .63264 Metetra
.653 .712 .891 r
.63141 .63264 .63429 .63627 .64008 .63464 .6372 .63107 Metetra
.653 .716 .894 r
.6372 .63107 .64008 .63464 .64587 .63301 .64301 .62951 Metetra
.652 .72 .898 r
.64301 .62951 .64587 .63301 .65168 .63137 .64882 .62795 Metetra
.652 .725 .902 r
.64882 .62795 .65168 .63137 .6575 .62973 .65465 .62638 Metetra
.652 .73 .907 r
.65465 .62638 .6575 .62973 .66333 .62809 .6605 .62482 Metetra
.651 .735 .911 r
.6605 .62482 .66333 .62809 .66917 .62644 .66635 .62325 Metetra
.65 .741 .915 r
.66635 .62325 .66917 .62644 .67503 .62479 .67222 .62169 Metetra
.65 .746 .92 r
.67222 .62169 .67503 .62479 .6809 .62313 .6781 .62012 Metetra
.649 .752 .925 r
.6781 .62012 .6809 .62313 .68678 .62148 .68399 .61854 Metetra
.648 .758 .929 r
.68399 .61854 .68678 .62148 .69267 .61981 .6899 .61696 Metetra
.648 .764 .934 r
.6899 .61696 .69267 .61981 .69858 .61815 .69582 .61538 Metetra
.647 .77 .939 r
.69582 .61538 .69858 .61815 .70449 .61648 .70175 .61379 Metetra
.645 .776 .943 r
.70175 .61379 .70449 .61648 .71042 .61481 .70769 .61219 Metetra
.644 .782 .947 r
.70769 .61219 .71042 .61481 .71636 .61313 .71364 .61059 Metetra
.643 .787 .951 r
.71364 .61059 .71636 .61313 .72232 .61146 .71961 .60898 Metetra
.642 .793 .955 r
.71961 .60898 .72232 .61146 .72828 .60977 .72559 .60736 Metetra
.641 .798 .958 r
.72559 .60736 .72828 .60977 .73426 .60809 .73159 .60573 Metetra
.64 .802 .961 r
.73159 .60573 .73426 .60809 .74026 .6064 .73759 .60409 Metetra
.639 .807 .963 r
.73759 .60409 .74026 .6064 .74626 .60471 .74361 .60244 Metetra
.638 .81 .966 r
.74361 .60244 .74626 .60471 .75228 .60301 .74964 .60079 Metetra
.637 .814 .968 r
.74964 .60079 .75228 .60301 .75831 .60131 .75568 .59912 Metetra
.636 .817 .969 r
.75568 .59912 .75831 .60131 .76435 .59961 .76174 .59744 Metetra
.636 .82 .971 r
.76174 .59744 .76435 .59961 .7704 .5979 .76781 .59576 Metetra
.636 .822 .972 r
.76781 .59576 .7704 .5979 .77647 .59619 .77389 .59406 Metetra
.635 .823 .973 r
.77389 .59406 .77647 .59619 .78255 .59447 .77998 .59236 Metetra
.635 .825 .973 r
.77998 .59236 .78255 .59447 .78865 .59275 .78609 .59064 Metetra
.635 .826 .974 r
.78609 .59064 .78865 .59275 .79475 .59103 .7922 .58892 Metetra
.636 .826 .974 r
.7922 .58892 .79475 .59103 .80087 .58931 .79833 .58719 Metetra
.636 .826 .974 r
.79833 .58719 .80087 .58931 .807 .58758 .80448 .58545 Metetra
.637 .826 .973 r
.80448 .58545 .807 .58758 .81315 .58585 .81063 .5837 Metetra
.638 .826 .973 r
.81063 .5837 .81315 .58585 .81931 .58411 .8168 .58194 Metetra
.638 .825 .972 r
.8168 .58194 .81931 .58411 .82548 .58237 .82298 .58018 Metetra
.639 .823 .971 r
.82298 .58018 .82548 .58237 .83167 .58063 .82918 .57841 Metetra
.64 .822 .97 r
.82918 .57841 .83167 .58063 .83786 .57888 .83539 .57663 Metetra
.641 .82 .969 r
.83539 .57663 .83786 .57888 .84407 .57713 .84161 .57484 Metetra
.642 .818 .968 r
.84161 .57484 .84407 .57713 .8503 .57537 .84784 .57305 Metetra
.643 .816 .966 r
.84784 .57305 .8503 .57537 .85654 .57361 .85408 .57125 Metetra
.645 .814 .965 r
.85408 .57125 .85654 .57361 .86279 .57185 .86034 .56945 Metetra
.646 .812 .963 r
.86034 .56945 .86279 .57185 .86905 .57009 .86662 .56764 Metetra
.647 .809 .961 r
.86662 .56764 .86905 .57009 .87533 .56832 .8729 .56582 Metetra
.648 .806 .959 r
.8729 .56582 .87533 .56832 .88162 .56654 .8792 .564 Metetra
.649 .803 .957 r
.8792 .564 .88162 .56654 .88793 .56476 .88551 .56218 Metetra
.65 .8 .955 r
.88551 .56218 .88793 .56476 .89425 .56298 .89184 .56035 Metetra
.651 .797 .953 r
.89184 .56035 .89425 .56298 .90058 .5612 .89818 .55851 Metetra
.652 .794 .951 r
.89818 .55851 .90058 .5612 .90693 .55941 .90453 .55667 Metetra
.652 .791 .949 r
.90453 .55667 .90693 .55941 .91329 .55761 .9109 .55482 Metetra
.653 .788 .946 r
.9109 .55482 .91329 .55761 .91966 .55582 .91727 .55297 Metetra
.654 .785 .944 r
.91727 .55297 .91966 .55582 .92605 .55402 .92367 .55112 Metetra
.655 .782 .941 r
.92367 .55112 .92605 .55402 .93245 .55221 .93007 .54926 Metetra
.655 .778 .939 r
.93007 .54926 .93245 .55221 .93886 .5504 .9365 .5474 Metetra
.656 .775 .937 r
.9365 .5474 .93886 .5504 .94529 .54859 .94293 .54553 Metetra
.657 .772 .934 r
.94293 .54553 .94529 .54859 .95174 .54677 .94938 .54366 Metetra
.657 .769 .932 r
.94938 .54366 .95174 .54677 .95819 .54495 .95584 .54178 Metetra
.658 .766 .929 r
.95584 .54178 .95819 .54495 .96466 .54313 .96231 .5399 Metetra
.658 .762 .927 r
.96231 .5399 .96466 .54313 .97115 .5413 .9688 .53801 Metetra
.658 .759 .924 r
.9688 .53801 .97115 .5413 .97765 .53947 .97531 .53612 Metetra
.659 .756 .922 r
.97531 .53612 .97765 .53947 .98416 .53763 .98183 .53423 Metetra
.663 .753 .917 r
.39534 .69631 .39852 .69908 .40384 .69753 .40067 .6947 Metetra
.663 .75 .915 r
.40067 .6947 .40384 .69753 .40917 .69598 .40601 .6931 Metetra
.663 .747 .912 r
.40601 .6931 .40917 .69598 .41452 .69442 .41136 .69149 Metetra
.663 .744 .91 r
.41136 .69149 .41452 .69442 .41987 .69286 .41672 .68987 Metetra
.663 .741 .908 r
.41672 .68987 .41987 .69286 .42523 .6913 .42209 .68826 Metetra
.663 .738 .905 r
.42209 .68826 .42523 .6913 .43061 .68974 .42747 .68665 Metetra
.663 .735 .903 r
.42747 .68665 .43061 .68974 .43599 .68817 .43287 .68503 Metetra
.663 .732 .901 r
.43287 .68503 .43599 .68817 .44139 .6866 .43827 .6834 Metetra
.663 .729 .898 r
.43827 .6834 .44139 .6866 .44679 .68503 .44368 .68178 Metetra
.663 .726 .896 r
.44368 .68178 .44679 .68503 .45221 .68345 .44911 .68015 Metetra
.663 .724 .894 r
.44911 .68015 .45221 .68345 .45764 .68187 .45454 .67852 Metetra
.663 .721 .892 r
.45454 .67852 .45764 .68187 .46308 .68029 .45998 .67689 Metetra
.663 .718 .89 r
.45998 .67689 .46308 .68029 .46852 .67871 .46544 .67526 Metetra
.663 .716 .888 r
.46544 .67526 .46852 .67871 .47398 .67712 .47091 .67363 Metetra
.663 .713 .886 r
.47091 .67363 .47398 .67712 .47945 .67553 .47638 .67199 Metetra
.662 .711 .884 r
.47638 .67199 .47945 .67553 .48494 .67395 .48187 .67036 Metetra
.662 .709 .882 r
.48187 .67036 .48494 .67395 .49043 .67235 .48737 .66872 Metetra
.662 .707 .88 r
.48737 .66872 .49043 .67235 .49593 .67076 .49288 .66708 Metetra
.661 .704 .879 r
.49288 .66708 .49593 .67076 .50145 .66916 .4984 .66545 Metetra
.661 .702 .877 r
.4984 .66545 .50145 .66916 .50697 .66757 .50393 .66381 Metetra
.661 .7 .876 r
.50393 .66381 .50697 .66757 .51251 .66597 .50947 .66217 Metetra
.661 .699 .874 r
.50947 .66217 .51251 .66597 .51805 .66437 .51503 .66054 Metetra
.66 .697 .873 r
.51503 .66054 .51805 .66437 .52361 .66278 .52059 .65891 Metetra
.66 .695 .872 r
.52059 .65891 .52361 .66278 .52918 .66118 .52617 .65727 Metetra
.659 .694 .871 r
.52617 .65727 .52918 .66118 .53476 .65958 .53175 .65565 Metetra
.659 .693 .87 r
.53175 .65565 .53476 .65958 .54035 .65798 .53735 .65402 Metetra
.659 .692 .87 r
.53735 .65402 .54035 .65798 .54596 .65638 .54296 .6524 Metetra
.658 .691 .869 r
.54296 .6524 .54596 .65638 .55157 .65478 .54858 .65079 Metetra
.657 .69 .869 r
.54858 .65079 .55157 .65478 .5572 .65318 .55421 .64918 Metetra
.657 .689 .869 r
.55421 .64918 .5572 .65318 .56284 .65159 .55985 .64758 Metetra
.656 .689 .869 r
.55985 .64758 .56284 .65159 .56848 .65 .56551 .64598 Metetra
.656 .689 .87 r
.56551 .64598 .56848 .65 .57415 .64841 .57118 .6444 Metetra
.655 .69 .87 r
.57118 .6444 .57415 .64841 .57982 .64682 .57686 .64282 Metetra
.654 .69 .871 r
.57686 .64282 .57982 .64682 .5855 .64523 .58255 .64125 Metetra
.654 .691 .872 r
.58255 .64125 .5855 .64523 .5912 .64365 .58825 .63968 Metetra
.653 .692 .874 r
.58825 .63968 .5912 .64365 .59691 .64207 .59396 .63813 Metetra
.653 .694 .876 r
.59396 .63813 .59691 .64207 .60263 .64049 .59969 .63658 Metetra
.652 .695 .878 r
.59969 .63658 .60263 .64049 .60836 .63891 .60543 .63505 Metetra
.651 .698 .88 r
.60543 .63505 .60836 .63891 .6141 .63734 .61118 .63352 Metetra
.651 .7 .883 r
.61118 .63352 .6141 .63734 .61986 .63577 .61695 .632 Metetra
.65 .703 .885 r
.61695 .632 .61986 .63577 .62563 .6342 .62272 .63049 Metetra
.649 .706 .889 r
.62272 .63049 .62563 .6342 .63141 .63264 .62851 .62898 Metetra
.649 .71 .892 r
.62851 .62898 .63141 .63264 .6372 .63107 .63431 .62748 Metetra
.648 .714 .896 r
.63431 .62748 .6372 .63107 .64301 .62951 .64013 .62599 Metetra
.647 .718 .9 r
.64013 .62599 .64301 .62951 .64882 .62795 .64596 .62451 Metetra
.646 .723 .904 r
.64596 .62451 .64882 .62795 .65465 .62638 .6518 .62302 Metetra
.646 .728 .908 r
.6518 .62302 .65465 .62638 .6605 .62482 .65765 .62154 Metetra
.645 .733 .913 r
.65765 .62154 .6605 .62482 .66635 .62325 .66352 .62005 Metetra
.644 .739 .918 r
.66352 .62005 .66635 .62325 .67222 .62169 .6694 .61857 Metetra
.643 .744 .922 r
.6694 .61857 .67222 .62169 .6781 .62012 .67529 .61709 Metetra
.643 .75 .927 r
.67529 .61709 .6781 .62012 .68399 .61854 .6812 .6156 Metetra
.642 .756 .932 r
.6812 .6156 .68399 .61854 .6899 .61696 .68711 .6141 Metetra
.641 .762 .936 r
.68711 .6141 .6899 .61696 .69582 .61538 .69305 .6126 Metetra
.64 .768 .941 r
.69305 .6126 .69582 .61538 .70175 .61379 .69899 .61109 Metetra
.639 .774 .945 r
.69899 .61109 .70175 .61379 .70769 .61219 .70495 .60957 Metetra
.638 .78 .949 r
.70495 .60957 .70769 .61219 .71364 .61059 .71092 .60804 Metetra
.637 .786 .953 r
.71092 .60804 .71364 .61059 .71961 .60898 .7169 .60649 Metetra
.636 .791 .957 r
.7169 .60649 .71961 .60898 .72559 .60736 .72289 .60493 Metetra
.636 .796 .96 r
.72289 .60493 .72559 .60736 .73159 .60573 .7289 .60336 Metetra
.635 .801 .963 r
.7289 .60336 .73159 .60573 .73759 .60409 .73492 .60177 Metetra
.634 .805 .965 r
.73492 .60177 .73759 .60409 .74361 .60244 .74095 .60017 Metetra
.634 .81 .967 r
.74095 .60017 .74361 .60244 .74964 .60079 .74699 .59856 Metetra
.633 .813 .969 r
.74699 .59856 .74964 .60079 .75568 .59912 .75305 .59693 Metetra
.633 .817 .971 r
.75305 .59693 .75568 .59912 .76174 .59744 .75912 .59528 Metetra
.633 .819 .972 r
.75912 .59528 .76174 .59744 .76781 .59576 .7652 .59361 Metetra
.633 .822 .973 r
.7652 .59361 .76781 .59576 .77389 .59406 .77129 .59193 Metetra
.633 .823 .973 r
.77129 .59193 .77389 .59406 .77998 .59236 .7774 .59024 Metetra
.634 .825 .974 r
.7774 .59024 .77998 .59236 .78609 .59064 .78351 .58853 Metetra
.635 .826 .974 r
.78351 .58853 .78609 .59064 .7922 .58892 .78964 .58681 Metetra
.636 .827 .974 r
.78964 .58681 .7922 .58892 .79833 .58719 .79579 .58507 Metetra
.636 .827 .974 r
.79579 .58507 .79833 .58719 .80448 .58545 .80194 .58332 Metetra
.638 .827 .973 r
.80194 .58332 .80448 .58545 .81063 .5837 .80811 .58155 Metetra
.639 .826 .973 r
.80811 .58155 .81063 .5837 .8168 .58194 .81428 .57977 Metetra
.64 .826 .972 r
.81428 .57977 .8168 .58194 .82298 .58018 .82048 .57798 Metetra
.641 .824 .971 r
.82048 .57798 .82298 .58018 .82918 .57841 .82668 .57618 Metetra
.642 .823 .97 r
.82668 .57618 .82918 .57841 .83539 .57663 .8329 .57437 Metetra
.644 .821 .968 r
.8329 .57437 .83539 .57663 .84161 .57484 .83913 .57255 Metetra
.645 .82 .967 r
.83913 .57255 .84161 .57484 .84784 .57305 .84537 .57072 Metetra
.646 .818 .966 r
.84537 .57072 .84784 .57305 .85408 .57125 .85162 .56888 Metetra
.647 .815 .964 r
.85162 .56888 .85408 .57125 .86034 .56945 .85789 .56704 Metetra
.649 .813 .962 r
.85789 .56704 .86034 .56945 .86662 .56764 .86417 .56518 Metetra
.65 .81 .96 r
.86417 .56518 .86662 .56764 .8729 .56582 .87046 .56332 Metetra
.651 .808 .958 r
.87046 .56332 .8729 .56582 .8792 .564 .87677 .56145 Metetra
.652 .805 .956 r
.87677 .56145 .8792 .564 .88551 .56218 .88309 .55958 Metetra
.653 .802 .954 r
.88309 .55958 .88551 .56218 .89184 .56035 .88942 .5577 Metetra
.654 .799 .952 r
.88942 .5577 .89184 .56035 .89818 .55851 .89576 .55581 Metetra
.655 .796 .95 r
.89576 .55581 .89818 .55851 .90453 .55667 .90212 .55392 Metetra
.656 .793 .948 r
.90212 .55392 .90453 .55667 .9109 .55482 .90849 .55202 Metetra
.657 .789 .945 r
.90849 .55202 .9109 .55482 .91727 .55297 .91488 .55012 Metetra
.658 .786 .943 r
.91488 .55012 .91727 .55297 .92367 .55112 .92128 .54821 Metetra
.658 .783 .941 r
.92128 .54821 .92367 .55112 .93007 .54926 .92769 .54629 Metetra
.659 .78 .938 r
.92769 .54629 .93007 .54926 .9365 .5474 .93412 .54438 Metetra
.659 .777 .936 r
.93412 .54438 .9365 .5474 .94293 .54553 .94056 .54245 Metetra
.66 .773 .933 r
.94056 .54245 .94293 .54553 .94938 .54366 .94701 .54052 Metetra
.66 .77 .931 r
.94701 .54052 .94938 .54366 .95584 .54178 .95348 .53859 Metetra
.661 .767 .928 r
.95348 .53859 .95584 .54178 .96231 .5399 .95996 .53666 Metetra
.661 .764 .926 r
.95996 .53666 .96231 .5399 .9688 .53801 .96645 .53471 Metetra
.661 .76 .923 r
.96645 .53471 .9688 .53801 .97531 .53612 .97296 .53277 Metetra
.662 .757 .921 r
.97296 .53277 .97531 .53612 .98183 .53423 .97948 .53082 Metetra
.666 .754 .916 r
.39214 .69352 .39534 .69631 .40067 .6947 .39748 .69186 Metetra
.666 .751 .913 r
.39748 .69186 .40067 .6947 .40601 .6931 .40283 .6902 Metetra
.666 .748 .911 r
.40283 .6902 .40601 .6931 .41136 .69149 .40819 .68854 Metetra
.666 .745 .909 r
.40819 .68854 .41136 .69149 .41672 .68987 .41356 .68688 Metetra
.666 .742 .906 r
.41356 .68688 .41672 .68987 .42209 .68826 .41894 .68521 Metetra
.666 .739 .904 r
.41894 .68521 .42209 .68826 .42747 .68665 .42433 .68354 Metetra
.666 .736 .902 r
.42433 .68354 .42747 .68665 .43287 .68503 .42973 .68187 Metetra
.666 .733 .899 r
.42973 .68187 .43287 .68503 .43827 .6834 .43514 .6802 Metetra
.666 .73 .897 r
.43514 .6802 .43827 .6834 .44368 .68178 .44056 .67852 Metetra
.666 .727 .895 r
.44056 .67852 .44368 .68178 .44911 .68015 .44599 .67684 Metetra
.666 .724 .893 r
.44599 .67684 .44911 .68015 .45454 .67852 .45143 .67516 Metetra
.666 .722 .891 r
.45143 .67516 .45454 .67852 .45998 .67689 .45688 .67348 Metetra
.666 .719 .889 r
.45688 .67348 .45998 .67689 .46544 .67526 .46235 .6718 Metetra
.665 .716 .887 r
.46235 .6718 .46544 .67526 .47091 .67363 .46782 .67012 Metetra
.665 .714 .885 r
.46782 .67012 .47091 .67363 .47638 .67199 .4733 .66843 Metetra
.665 .711 .883 r
.4733 .66843 .47638 .67199 .48187 .67036 .4788 .66675 Metetra
.664 .709 .881 r
.4788 .66675 .48187 .67036 .48737 .66872 .4843 .66507 Metetra
.664 .707 .879 r
.4843 .66507 .48737 .66872 .49288 .66708 .48982 .66339 Metetra
.664 .705 .878 r
.48982 .66339 .49288 .66708 .4984 .66545 .49534 .66171 Metetra
.663 .703 .876 r
.49534 .66171 .4984 .66545 .50393 .66381 .50088 .66003 Metetra
.663 .701 .875 r
.50088 .66003 .50393 .66381 .50947 .66217 .50643 .65836 Metetra
.662 .699 .873 r
.50643 .65836 .50947 .66217 .51503 .66054 .51199 .65668 Metetra
.662 .697 .872 r
.51199 .65668 .51503 .66054 .52059 .65891 .51756 .65502 Metetra
.661 .695 .871 r
.51756 .65502 .52059 .65891 .52617 .65727 .52314 .65336 Metetra
.66 .694 .87 r
.52314 .65336 .52617 .65727 .53175 .65565 .52873 .6517 Metetra
.66 .693 .87 r
.52873 .6517 .53175 .65565 .53735 .65402 .53434 .65005 Metetra
.659 .691 .869 r
.53434 .65005 .53735 .65402 .54296 .6524 .53995 .64841 Metetra
.659 .691 .869 r
.53995 .64841 .54296 .6524 .54858 .65079 .54558 .64678 Metetra
.658 .69 .869 r
.54558 .64678 .54858 .65079 .55421 .64918 .55121 .64516 Metetra
.657 .689 .869 r
.55121 .64516 .55421 .64918 .55985 .64758 .55686 .64355 Metetra
.656 .689 .869 r
.55686 .64355 .55985 .64758 .56551 .64598 .56253 .64195 Metetra
.655 .689 .869 r
.56253 .64195 .56551 .64598 .57118 .6444 .5682 .64037 Metetra
.654 .689 .87 r
.5682 .64037 .57118 .6444 .57686 .64282 .57388 .6388 Metetra
.653 .689 .871 r
.57388 .6388 .57686 .64282 .58255 .64125 .57958 .63724 Metetra
.652 .69 .872 r
.57958 .63724 .58255 .64125 .58825 .63968 .58529 .6357 Metetra
.651 .691 .874 r
.58529 .6357 .58825 .63968 .59396 .63813 .59101 .63416 Metetra
.65 .692 .876 r
.59101 .63416 .59396 .63813 .59969 .63658 .59674 .63265 Metetra
.649 .694 .878 r
.59674 .63265 .59969 .63658 .60543 .63505 .60249 .63116 Metetra
.648 .696 .88 r
.60249 .63116 .60543 .63505 .61118 .63352 .60825 .62967 Metetra
.647 .698 .883 r
.60825 .62967 .61118 .63352 .61695 .632 .61402 .6282 Metetra
.646 .701 .886 r
.61402 .6282 .61695 .632 .62272 .63049 .61981 .62675 Metetra
.645 .704 .89 r
.61981 .62675 .62272 .63049 .62851 .62898 .62561 .6253 Metetra
.644 .707 .893 r
.62561 .6253 .62851 .62898 .63431 .62748 .63142 .62387 Metetra
.643 .711 .897 r
.63142 .62387 .63431 .62748 .64013 .62599 .63724 .62245 Metetra
.642 .716 .901 r
.63724 .62245 .64013 .62599 .64596 .62451 .64308 .62104 Metetra
.641 .72 .906 r
.64308 .62104 .64596 .62451 .6518 .62302 .64893 .61964 Metetra
.64 .725 .91 r
.64893 .61964 .6518 .62302 .65765 .62154 .6548 .61824 Metetra
.639 .731 .915 r
.6548 .61824 .65765 .62154 .66352 .62005 .66067 .61684 Metetra
.638 .736 .92 r
.66067 .61684 .66352 .62005 .6694 .61857 .66657 .61544 Metetra
.637 .742 .924 r
.66657 .61544 .6694 .61857 .67529 .61709 .67247 .61405 Metetra
.636 .748 .929 r
.67247 .61405 .67529 .61709 .6812 .6156 .67839 .61264 Metetra
.635 .754 .934 r
.67839 .61264 .6812 .6156 .68711 .6141 .68432 .61123 Metetra
.634 .76 .939 r
.68432 .61123 .68711 .6141 .69305 .6126 .69026 .60981 Metetra
.633 .766 .943 r
.69026 .60981 .69305 .6126 .69899 .61109 .69622 .60838 Metetra
.632 .772 .948 r
.69622 .60838 .69899 .61109 .70495 .60957 .70219 .60694 Metetra
.631 .778 .952 r
.70219 .60694 .70495 .60957 .71092 .60804 .70817 .60548 Metetra
.631 .784 .955 r
.70817 .60548 .71092 .60804 .7169 .60649 .71417 .60401 Metetra
.63 .79 .959 r
.71417 .60401 .7169 .60649 .72289 .60493 .72018 .60251 Metetra
.63 .795 .962 r
.72018 .60251 .72289 .60493 .7289 .60336 .7262 .601 Metetra
.629 .8 .965 r
.7262 .601 .7289 .60336 .73492 .60177 .73223 .59947 Metetra
.629 .805 .967 r
.73223 .59947 .73492 .60177 .74095 .60017 .73828 .59791 Metetra
.629 .809 .969 r
.73828 .59791 .74095 .60017 .74699 .59856 .74433 .59633 Metetra
.629 .813 .971 r
.74433 .59633 .74699 .59856 .75305 .59693 .7504 .59474 Metetra
.63 .816 .972 r
.7504 .59474 .75305 .59693 .75912 .59528 .75648 .59311 Metetra
.63 .819 .973 r
.75648 .59311 .75912 .59528 .7652 .59361 .76258 .59147 Metetra
.631 .822 .974 r
.76258 .59147 .7652 .59361 .77129 .59193 .76868 .5898 Metetra
.632 .824 .974 r
.76868 .5898 .77129 .59193 .7774 .59024 .7748 .58812 Metetra
.633 .825 .975 r
.7748 .58812 .7774 .59024 .78351 .58853 .78093 .58642 Metetra
.634 .827 .975 r
.78093 .58642 .78351 .58853 .78964 .58681 .78707 .58469 Metetra
.635 .827 .974 r
.78707 .58469 .78964 .58681 .79579 .58507 .79322 .58294 Metetra
.637 .828 .974 r
.79322 .58294 .79579 .58507 .80194 .58332 .79939 .58118 Metetra
.638 .828 .973 r
.79939 .58118 .80194 .58332 .80811 .58155 .80557 .5794 Metetra
.64 .828 .973 r
.80557 .5794 .80811 .58155 .81428 .57977 .81176 .57759 Metetra
.641 .826 .972 r
.81176 .57759 .81428 .57977 .82048 .57798 .81796 .57578 Metetra
.643 .826 .971 r
.81796 .57578 .82048 .57798 .82668 .57618 .82417 .57395 Metetra
.644 .824 .969 r
.82417 .57395 .82668 .57618 .8329 .57437 .8304 .57211 Metetra
.646 .823 .968 r
.8304 .57211 .8329 .57437 .83913 .57255 .83663 .57025 Metetra
.647 .821 .967 r
.83663 .57025 .83913 .57255 .84537 .57072 .84288 .56838 Metetra
.649 .819 .965 r
.84288 .56838 .84537 .57072 .85162 .56888 .84915 .56651 Metetra
.65 .817 .963 r
.84915 .56651 .85162 .56888 .85789 .56704 .85542 .56462 Metetra
.652 .814 .961 r
.85542 .56462 .85789 .56704 .86417 .56518 .86171 .56272 Metetra
.653 .812 .96 r
.86171 .56272 .86417 .56518 .87046 .56332 .86801 .56081 Metetra
.654 .809 .958 r
.86801 .56081 .87046 .56332 .87677 .56145 .87432 .5589 Metetra
.655 .806 .956 r
.87432 .5589 .87677 .56145 .88309 .55958 .88065 .55697 Metetra
.656 .803 .953 r
.88065 .55697 .88309 .55958 .88942 .5577 .88699 .55504 Metetra
.658 .8 .951 r
.88699 .55504 .88942 .5577 .89576 .55581 .89334 .5531 Metetra
.658 .797 .949 r
.89334 .5531 .89576 .55581 .90212 .55392 .89971 .55116 Metetra
.659 .794 .947 r
.89971 .55116 .90212 .55392 .90849 .55202 .90608 .54921 Metetra
.66 .791 .944 r
.90608 .54921 .90849 .55202 .91488 .55012 .91248 .54725 Metetra
.661 .788 .942 r
.91248 .54725 .91488 .55012 .92128 .54821 .91888 .54528 Metetra
.661 .784 .94 r
.91888 .54528 .92128 .54821 .92769 .54629 .9253 .54332 Metetra
.662 .781 .937 r
.9253 .54332 .92769 .54629 .93412 .54438 .93173 .54134 Metetra
.663 .778 .935 r
.93173 .54134 .93412 .54438 .94056 .54245 .93817 .53936 Metetra
.663 .775 .932 r
.93817 .53936 .94056 .54245 .94701 .54052 .94463 .53738 Metetra
.663 .771 .93 r
.94463 .53738 .94701 .54052 .95348 .53859 .9511 .53539 Metetra
.664 .768 .927 r
.9511 .53539 .95348 .53859 .95996 .53666 .95759 .5334 Metetra
.664 .765 .925 r
.95759 .5334 .95996 .53666 .96645 .53471 .96409 .5314 Metetra
.665 .762 .922 r
.96409 .5314 .96645 .53471 .97296 .53277 .9706 .5294 Metetra
.665 .758 .92 r
.9706 .5294 .97296 .53277 .97948 .53082 .97712 .52739 Metetra
.669 .755 .914 r
.38894 .69072 .39214 .69352 .39748 .69186 .39428 .68901 Metetra
.669 .751 .912 r
.39428 .68901 .39748 .69186 .40283 .6902 .39964 .6873 Metetra
.67 .748 .91 r
.39964 .6873 .40283 .6902 .40819 .68854 .40501 .68558 Metetra
.669 .745 .907 r
.40501 .68558 .40819 .68854 .41356 .68688 .41038 .68387 Metetra
.67 .742 .905 r
.41038 .68387 .41356 .68688 .41894 .68521 .41577 .68215 Metetra
.669 .739 .903 r
.41577 .68215 .41894 .68521 .42433 .68354 .42117 .68043 Metetra
.669 .736 .9 r
.42117 .68043 .42433 .68354 .42973 .68187 .42658 .6787 Metetra
.669 .733 .898 r
.42658 .6787 .42973 .68187 .43514 .6802 .432 .67698 Metetra
.669 .731 .896 r
.432 .67698 .43514 .6802 .44056 .67852 .43742 .67525 Metetra
.669 .728 .894 r
.43742 .67525 .44056 .67852 .44599 .67684 .44286 .67352 Metetra
.669 .725 .892 r
.44286 .67352 .44599 .67684 .45143 .67516 .44831 .67179 Metetra
.669 .722 .889 r
.44831 .67179 .45143 .67516 .45688 .67348 .45377 .67006 Metetra
.668 .72 .887 r
.45377 .67006 .45688 .67348 .46235 .6718 .45924 .66833 Metetra
.668 .717 .885 r
.45924 .66833 .46235 .6718 .46782 .67012 .46472 .66659 Metetra
.667 .715 .884 r
.46472 .66659 .46782 .67012 .4733 .66843 .47021 .66486 Metetra
.667 .712 .882 r
.47021 .66486 .4733 .66843 .4788 .66675 .47571 .66313 Metetra
.667 .71 .88 r
.47571 .66313 .4788 .66675 .4843 .66507 .48122 .6614 Metetra
.666 .707 .878 r
.48122 .6614 .4843 .66507 .48982 .66339 .48675 .65968 Metetra
.666 .705 .877 r
.48675 .65968 .48982 .66339 .49534 .66171 .49228 .65795 Metetra
.665 .703 .875 r
.49228 .65795 .49534 .66171 .50088 .66003 .49782 .65623 Metetra
.665 .701 .874 r
.49782 .65623 .50088 .66003 .50643 .65836 .50338 .65452 Metetra
.664 .699 .872 r
.50338 .65452 .50643 .65836 .51199 .65668 .50894 .65281 Metetra
.663 .697 .871 r
.50894 .65281 .51199 .65668 .51756 .65502 .51452 .65111 Metetra
.663 .695 .87 r
.51452 .65111 .51756 .65502 .52314 .65336 .52011 .64942 Metetra
.662 .694 .869 r
.52011 .64942 .52314 .65336 .52873 .6517 .5257 .64773 Metetra
.661 .692 .869 r
.5257 .64773 .52873 .6517 .53434 .65005 .53131 .64606 Metetra
.66 .691 .868 r
.53131 .64606 .53434 .65005 .53995 .64841 .53693 .64439 Metetra
.659 .69 .868 r
.53693 .64439 .53995 .64841 .54558 .64678 .54257 .64274 Metetra
.658 .689 .868 r
.54257 .64274 .54558 .64678 .55121 .64516 .54821 .64111 Metetra
.657 .688 .868 r
.54821 .64111 .55121 .64516 .55686 .64355 .55386 .63949 Metetra
.656 .688 .868 r
.55386 .63949 .55686 .64355 .56253 .64195 .55953 .63789 Metetra
.654 .687 .869 r
.55953 .63789 .56253 .64195 .5682 .64037 .56521 .63631 Metetra
.653 .688 .87 r
.56521 .63631 .5682 .64037 .57388 .6388 .5709 .63474 Metetra
.652 .688 .871 r
.5709 .63474 .57388 .6388 .57958 .63724 .5766 .6332 Metetra
.651 .689 .872 r
.5766 .6332 .57958 .63724 .58529 .6357 .58232 .63167 Metetra
.649 .689 .874 r
.58232 .63167 .58529 .6357 .59101 .63416 .58805 .63017 Metetra
.648 .69 .876 r
.58805 .63017 .59101 .63416 .59674 .63265 .59379 .62869 Metetra
.647 .692 .878 r
.59379 .62869 .59674 .63265 .60249 .63116 .59954 .62723 Metetra
.645 .694 .881 r
.59954 .62723 .60249 .63116 .60825 .62967 .60531 .62579 Metetra
.644 .696 .883 r
.60531 .62579 .60825 .62967 .61402 .6282 .61109 .62438 Metetra
.642 .699 .887 r
.61109 .62438 .61402 .6282 .61981 .62675 .61688 .62298 Metetra
.641 .702 .89 r
.61688 .62298 .61981 .62675 .62561 .6253 .62269 .6216 Metetra
.639 .705 .894 r
.62269 .6216 .62561 .6253 .63142 .62387 .62851 .62024 Metetra
.638 .709 .898 r
.62851 .62024 .63142 .62387 .63724 .62245 .63435 .61889 Metetra
.636 .713 .902 r
.63435 .61889 .63724 .62245 .64308 .62104 .64019 .61756 Metetra
.635 .717 .907 r
.64019 .61756 .64308 .62104 .64893 .61964 .64605 .61624 Metetra
.634 .723 .912 r
.64605 .61624 .64893 .61964 .6548 .61824 .65193 .61492 Metetra
.632 .728 .916 r
.65193 .61492 .6548 .61824 .66067 .61684 .65782 .61361 Metetra
.631 .733 .922 r
.65782 .61361 .66067 .61684 .66657 .61544 .66372 .6123 Metetra
.629 .739 .927 r
.66372 .6123 .66657 .61544 .67247 .61405 .66964 .61099 Metetra
.628 .745 .932 r
.66964 .61099 .67247 .61405 .67839 .61264 .67557 .60968 Metetra
.628 .752 .936 r
.67557 .60968 .67839 .61264 .68432 .61123 .68151 .60836 Metetra
.626 .758 .941 r
.68151 .60836 .68432 .61123 .69026 .60981 .68747 .60702 Metetra
.626 .764 .946 r
.68747 .60702 .69026 .60981 .69622 .60838 .69344 .60568 Metetra
.625 .771 .95 r
.69344 .60568 .69622 .60838 .70219 .60694 .69943 .60431 Metetra
.624 .777 .954 r
.69943 .60431 .70219 .60694 .70817 .60548 .70542 .60293 Metetra
.624 .783 .958 r
.70542 .60293 .70817 .60548 .71417 .60401 .71143 .60153 Metetra
.623 .789 .961 r
.71143 .60153 .71417 .60401 .72018 .60251 .71745 .6001 Metetra
.623 .795 .964 r
.71745 .6001 .72018 .60251 .7262 .601 .72349 .59864 Metetra
.624 .8 .967 r
.72349 .59864 .7262 .601 .73223 .59947 .72953 .59716 Metetra
.624 .804 .969 r
.72953 .59716 .73223 .59947 .73828 .59791 .73559 .59565 Metetra
.624 .809 .971 r
.73559 .59565 .73828 .59791 .74433 .59633 .74166 .59412 Metetra
.625 .813 .972 r
.74166 .59412 .74433 .59633 .7504 .59474 .74775 .59255 Metetra
.626 .817 .974 r
.74775 .59255 .7504 .59474 .75648 .59311 .75384 .59096 Metetra
.627 .82 .974 r
.75384 .59096 .75648 .59311 .76258 .59147 .75995 .58934 Metetra
.628 .822 .975 r
.75995 .58934 .76258 .59147 .76868 .5898 .76607 .58769 Metetra
.63 .824 .975 r
.76607 .58769 .76868 .5898 .7748 .58812 .7722 .58601 Metetra
.631 .826 .975 r
.7722 .58601 .7748 .58812 .78093 .58642 .77834 .58431 Metetra
.633 .827 .975 r
.77834 .58431 .78093 .58642 .78707 .58469 .78449 .58258 Metetra
.635 .829 .975 r
.78449 .58258 .78707 .58469 .79322 .58294 .79065 .58082 Metetra
.637 .829 .974 r
.79065 .58082 .79322 .58294 .79939 .58118 .79683 .57904 Metetra
.639 .829 .974 r
.79683 .57904 .79939 .58118 .80557 .5794 .80302 .57724 Metetra
.641 .829 .973 r
.80302 .57724 .80557 .5794 .81176 .57759 .80922 .57542 Metetra
.643 .828 .972 r
.80922 .57542 .81176 .57759 .81796 .57578 .81543 .57358 Metetra
.644 .827 .97 r
.81543 .57358 .81796 .57578 .82417 .57395 .82165 .57172 Metetra
.646 .826 .969 r
.82165 .57172 .82417 .57395 .8304 .57211 .82789 .56984 Metetra
.648 .824 .968 r
.82789 .56984 .8304 .57211 .83663 .57025 .83413 .56795 Metetra
.65 .822 .966 r
.83413 .56795 .83663 .57025 .84288 .56838 .84039 .56605 Metetra
.652 .82 .964 r
.84039 .56605 .84288 .56838 .84915 .56651 .84666 .56413 Metetra
.653 .818 .963 r
.84666 .56413 .84915 .56651 .85542 .56462 .85295 .5622 Metetra
.655 .816 .961 r
.85295 .5622 .85542 .56462 .86171 .56272 .85924 .56025 Metetra
.656 .813 .959 r
.85924 .56025 .86171 .56272 .86801 .56081 .86555 .5583 Metetra
.657 .81 .957 r
.86555 .5583 .86801 .56081 .87432 .5589 .87187 .55633 Metetra
.658 .807 .955 r
.87187 .55633 .87432 .5589 .88065 .55697 .8782 .55436 Metetra
.66 .805 .953 r
.8782 .55436 .88065 .55697 .88699 .55504 .88455 .55238 Metetra
.661 .802 .95 r
.88455 .55238 .88699 .55504 .89334 .5531 .89091 .55039 Metetra
.662 .799 .948 r
.89091 .55039 .89334 .5531 .89971 .55116 .89728 .54839 Metetra
.662 .795 .946 r
.89728 .54839 .89971 .55116 .90608 .54921 .90366 .54638 Metetra
.663 .792 .943 r
.90366 .54638 .90608 .54921 .91248 .54725 .91006 .54437 Metetra
.664 .789 .941 r
.91006 .54437 .91248 .54725 .91888 .54528 .91647 .54235 Metetra
.665 .786 .939 r
.91647 .54235 .91888 .54528 .9253 .54332 .92289 .54033 Metetra
.665 .782 .936 r
.92289 .54033 .9253 .54332 .93173 .54134 .92933 .5383 Metetra
.666 .779 .934 r
.92933 .5383 .93173 .54134 .93817 .53936 .93578 .53626 Metetra
.666 .776 .931 r
.93578 .53626 .93817 .53936 .94463 .53738 .94224 .53422 Metetra
.667 .773 .929 r
.94224 .53422 .94463 .53738 .9511 .53539 .94872 .53218 Metetra
.667 .77 .926 r
.94872 .53218 .9511 .53539 .95759 .5334 .95521 .53012 Metetra
.667 .766 .924 r
.95521 .53012 .95759 .5334 .96409 .5314 .96171 .52807 Metetra
.668 .763 .921 r
.96171 .52807 .96409 .5314 .9706 .5294 .96823 .52601 Metetra
.668 .76 .919 r
.96823 .52601 .9706 .5294 .97712 .52739 .97476 .52394 Metetra
.673 .755 .913 r
.38572 .68791 .38894 .69072 .39428 .68901 .39107 .68615 Metetra
.673 .752 .91 r
.39107 .68615 .39428 .68901 .39964 .6873 .39644 .68438 Metetra
.673 .749 .908 r
.39644 .68438 .39964 .6873 .40501 .68558 .40181 .68262 Metetra
.673 .746 .906 r
.40181 .68262 .40501 .68558 .41038 .68387 .4072 .68085 Metetra
.673 .743 .903 r
.4072 .68085 .41038 .68387 .41577 .68215 .41259 .67907 Metetra
.672 .74 .901 r
.41259 .67907 .41577 .68215 .42117 .68043 .418 .6773 Metetra
.672 .737 .899 r
.418 .6773 .42117 .68043 .42658 .6787 .42342 .67552 Metetra
.672 .734 .897 r
.42342 .67552 .42658 .6787 .432 .67698 .42884 .67374 Metetra
.672 .731 .894 r
.42884 .67374 .432 .67698 .43742 .67525 .43428 .67197 Metetra
.672 .728 .892 r
.43428 .67197 .43742 .67525 .44286 .67352 .43972 .67018 Metetra
.671 .726 .89 r
.43972 .67018 .44286 .67352 .44831 .67179 .44518 .6684 Metetra
.671 .723 .888 r
.44518 .6684 .44831 .67179 .45377 .67006 .45065 .66662 Metetra
.671 .72 .886 r
.45065 .66662 .45377 .67006 .45924 .66833 .45612 .66484 Metetra
.67 .718 .884 r
.45612 .66484 .45924 .66833 .46472 .66659 .46161 .66305 Metetra
.67 .715 .882 r
.46161 .66305 .46472 .66659 .47021 .66486 .46711 .66127 Metetra
.669 .713 .88 r
.46711 .66127 .47021 .66486 .47571 .66313 .47262 .6595 Metetra
.669 .71 .879 r
.47262 .6595 .47571 .66313 .48122 .6614 .47814 .65772 Metetra
.669 .708 .877 r
.47814 .65772 .48122 .6614 .48675 .65968 .48366 .65595 Metetra
.668 .706 .875 r
.48366 .65595 .48675 .65968 .49228 .65795 .4892 .65418 Metetra
.667 .703 .874 r
.4892 .65418 .49228 .65795 .49782 .65623 .49475 .65242 Metetra
.667 .701 .873 r
.49475 .65242 .49782 .65623 .50338 .65452 .50031 .65066 Metetra
.666 .699 .871 r
.50031 .65066 .50338 .65452 .50894 .65281 .50589 .64891 Metetra
.665 .697 .87 r
.50589 .64891 .50894 .65281 .51452 .65111 .51147 .64718 Metetra
.664 .695 .869 r
.51147 .64718 .51452 .65111 .52011 .64942 .51706 .64545 Metetra
.663 .694 .868 r
.51706 .64545 .52011 .64942 .5257 .64773 .52267 .64373 Metetra
.662 .692 .868 r
.52267 .64373 .5257 .64773 .53131 .64606 .52828 .64203 Metetra
.661 .691 .867 r
.52828 .64203 .53131 .64606 .53693 .64439 .53391 .64034 Metetra
.66 .689 .867 r
.53391 .64034 .53693 .64439 .54257 .64274 .53954 .63868 Metetra
.658 .689 .867 r
.53954 .63868 .54257 .64274 .54821 .64111 .54519 .63703 Metetra
.657 .688 .867 r
.54519 .63703 .54821 .64111 .55386 .63949 .55085 .6354 Metetra
.656 .687 .867 r
.55085 .6354 .55386 .63949 .55953 .63789 .55653 .63379 Metetra
.654 .686 .868 r
.55653 .63379 .55953 .63789 .56521 .63631 .56221 .63221 Metetra
.652 .686 .869 r
.56221 .63221 .56521 .63631 .5709 .63474 .56791 .63065 Metetra
.651 .686 .87 r
.56791 .63065 .5709 .63474 .5766 .6332 .57362 .62912 Metetra
.649 .686 .871 r
.57362 .62912 .5766 .6332 .58232 .63167 .57934 .62762 Metetra
.648 .687 .873 r
.57934 .62762 .58232 .63167 .58805 .63017 .58508 .62614 Metetra
.646 .688 .875 r
.58508 .62614 .58805 .63017 .59379 .62869 .59082 .62469 Metetra
.644 .689 .878 r
.59082 .62469 .59379 .62869 .59954 .62723 .59658 .62327 Metetra
.642 .691 .881 r
.59658 .62327 .59954 .62723 .60531 .62579 .60236 .62188 Metetra
.64 .693 .884 r
.60236 .62188 .60531 .62579 .61109 .62438 .60815 .62051 Metetra
.638 .696 .887 r
.60815 .62051 .61109 .62438 .61688 .62298 .61395 .61917 Metetra
.636 .699 .891 r
.61395 .61917 .61688 .62298 .62269 .6216 .61976 .61786 Metetra
.634 .702 .895 r
.61976 .61786 .62269 .6216 .62851 .62024 .62559 .61657 Metetra
.632 .706 .899 r
.62559 .61657 .62851 .62024 .63435 .61889 .63144 .6153 Metetra
.63 .71 .904 r
.63144 .6153 .63435 .61889 .64019 .61756 .63729 .61405 Metetra
.629 .714 .908 r
.63729 .61405 .64019 .61756 .64605 .61624 .64317 .61281 Metetra
.627 .719 .913 r
.64317 .61281 .64605 .61624 .65193 .61492 .64905 .61159 Metetra
.625 .725 .918 r
.64905 .61159 .65193 .61492 .65782 .61361 .65495 .61036 Metetra
.623 .73 .923 r
.65495 .61036 .65782 .61361 .66372 .6123 .66087 .60915 Metetra
.622 .736 .929 r
.66087 .60915 .66372 .6123 .66964 .61099 .6668 .60793 Metetra
.621 .743 .934 r
.6668 .60793 .66964 .61099 .67557 .60968 .67274 .60671 Metetra
.619 .749 .939 r
.67274 .60671 .67557 .60968 .68151 .60836 .6787 .60548 Metetra
.619 .756 .944 r
.6787 .60548 .68151 .60836 .68747 .60702 .68467 .60423 Metetra
.618 .762 .948 r
.68467 .60423 .68747 .60702 .69344 .60568 .69065 .60297 Metetra
.617 .769 .953 r
.69065 .60297 .69344 .60568 .69943 .60431 .69665 .60169 Metetra
.616 .775 .957 r
.69665 .60169 .69943 .60431 .70542 .60293 .70266 .60039 Metetra
.616 .782 .961 r
.70266 .60039 .70542 .60293 .71143 .60153 .70868 .59906 Metetra
.616 .788 .964 r
.70868 .59906 .71143 .60153 .71745 .6001 .71472 .5977 Metetra
.617 .794 .967 r
.71472 .5977 .71745 .6001 .72349 .59864 .72077 .5963 Metetra
.617 .799 .969 r
.72077 .5963 .72349 .59864 .72953 .59716 .72683 .59487 Metetra
.618 .805 .971 r
.72683 .59487 .72953 .59716 .73559 .59565 .7329 .59341 Metetra
.619 .809 .973 r
.7329 .59341 .73559 .59565 .74166 .59412 .73898 .59192 Metetra
.62 .813 .974 r
.73898 .59192 .74166 .59412 .74775 .59255 .74508 .59039 Metetra
.622 .817 .975 r
.74508 .59039 .74775 .59255 .75384 .59096 .75119 .58882 Metetra
.624 .821 .976 r
.75119 .58882 .75384 .59096 .75995 .58934 .75731 .58722 Metetra
.625 .823 .977 r
.75731 .58722 .75995 .58934 .76607 .58769 .76344 .58558 Metetra
.627 .826 .977 r
.76344 .58558 .76607 .58769 .7722 .58601 .76958 .58391 Metetra
.63 .828 .977 r
.76958 .58391 .7722 .58601 .77834 .58431 .77573 .58221 Metetra
.632 .829 .976 r
.77573 .58221 .77834 .58431 .78449 .58258 .7819 .58048 Metetra
.635 .83 .976 r
.7819 .58048 .78449 .58258 .79065 .58082 .78807 .57871 Metetra
.637 .83 .975 r
.78807 .57871 .79065 .58082 .79683 .57904 .79426 .57692 Metetra
.639 .831 .974 r
.79426 .57692 .79683 .57904 .80302 .57724 .80046 .5751 Metetra
.642 .83 .973 r
.80046 .5751 .80302 .57724 .80922 .57542 .80667 .57325 Metetra
.644 .83 .972 r
.80667 .57325 .80922 .57542 .81543 .57358 .81289 .57138 Metetra
.646 .829 .97 r
.81289 .57138 .81543 .57358 .82165 .57172 .81912 .56949 Metetra
.648 .827 .969 r
.81912 .56949 .82165 .57172 .82789 .56984 .82536 .56758 Metetra
.65 .826 .967 r
.82536 .56758 .82789 .56984 .83413 .56795 .83162 .56565 Metetra
.653 .824 .966 r
.83162 .56565 .83413 .56795 .84039 .56605 .83789 .56371 Metetra
.654 .822 .964 r
.83789 .56371 .84039 .56605 .84666 .56413 .84417 .56174 Metetra
.656 .82 .962 r
.84417 .56174 .84666 .56413 .85295 .5622 .85046 .55977 Metetra
.658 .817 .96 r
.85046 .55977 .85295 .5622 .85924 .56025 .85676 .55778 Metetra
.659 .815 .958 r
.85676 .55778 .85924 .56025 .86555 .5583 .86308 .55577 Metetra
.66 .812 .956 r
.86308 .55577 .86555 .5583 .87187 .55633 .86941 .55376 Metetra
.662 .809 .954 r
.86941 .55376 .87187 .55633 .8782 .55436 .87575 .55174 Metetra
.663 .806 .952 r
.87575 .55174 .8782 .55436 .88455 .55238 .8821 .5497 Metetra
.664 .803 .949 r
.8821 .5497 .88455 .55238 .89091 .55039 .88846 .54766 Metetra
.665 .8 .947 r
.88846 .54766 .89091 .55039 .89728 .54839 .89484 .54561 Metetra
.666 .797 .945 r
.89484 .54561 .89728 .54839 .90366 .54638 .90123 .54355 Metetra
.667 .794 .942 r
.90123 .54355 .90366 .54638 .91006 .54437 .90763 .54148 Metetra
.667 .79 .94 r
.90763 .54148 .91006 .54437 .91647 .54235 .91405 .53941 Metetra
.668 .787 .938 r
.91405 .53941 .91647 .54235 .92289 .54033 .92048 .53733 Metetra
.669 .784 .935 r
.92048 .53733 .92289 .54033 .92933 .5383 .92692 .53524 Metetra
.669 .781 .933 r
.92692 .53524 .92933 .5383 .93578 .53626 .93338 .53315 Metetra
.669 .777 .93 r
.93338 .53315 .93578 .53626 .94224 .53422 .93984 .53105 Metetra
.67 .774 .928 r
.93984 .53105 .94224 .53422 .94872 .53218 .94633 .52895 Metetra
.67 .771 .925 r
.94633 .52895 .94872 .53218 .95521 .53012 .95282 .52684 Metetra
.67 .768 .923 r
.95282 .52684 .95521 .53012 .96171 .52807 .95933 .52473 Metetra
.671 .764 .921 r
.95933 .52473 .96171 .52807 .96823 .52601 .96585 .52261 Metetra
.671 .761 .918 r
.96585 .52261 .96823 .52601 .97476 .52394 .97238 .52049 Metetra
.676 .756 .911 r
.38249 .68509 .38572 .68791 .39107 .68615 .38785 .68327 Metetra
.676 .753 .909 r
.38785 .68327 .39107 .68615 .39644 .68438 .39323 .68146 Metetra
.676 .75 .907 r
.39323 .68146 .39644 .68438 .40181 .68262 .39861 .67964 Metetra
.676 .747 .904 r
.39861 .67964 .40181 .68262 .4072 .68085 .404 .67781 Metetra
.676 .744 .902 r
.404 .67781 .4072 .68085 .41259 .67907 .40941 .67599 Metetra
.675 .741 .9 r
.40941 .67599 .41259 .67907 .418 .6773 .41482 .67416 Metetra
.675 .738 .897 r
.41482 .67416 .418 .6773 .42342 .67552 .42024 .67233 Metetra
.675 .735 .895 r
.42024 .67233 .42342 .67552 .42884 .67374 .42568 .6705 Metetra
.675 .732 .893 r
.42568 .6705 .42884 .67374 .43428 .67197 .43112 .66867 Metetra
.675 .729 .891 r
.43112 .66867 .43428 .67197 .43972 .67018 .43658 .66684 Metetra
.674 .726 .889 r
.43658 .66684 .43972 .67018 .44518 .6684 .44204 .665 Metetra
.674 .724 .887 r
.44204 .665 .44518 .6684 .45065 .66662 .44751 .66317 Metetra
.673 .721 .885 r
.44751 .66317 .45065 .66662 .45612 .66484 .453 .66133 Metetra
.673 .718 .883 r
.453 .66133 .45612 .66484 .46161 .66305 .45849 .6595 Metetra
.673 .716 .881 r
.45849 .6595 .46161 .66305 .46711 .66127 .464 .65767 Metetra
.672 .713 .879 r
.464 .65767 .46711 .66127 .47262 .6595 .46951 .65584 Metetra
.671 .71 .877 r
.46951 .65584 .47262 .6595 .47814 .65772 .47504 .65402 Metetra
.671 .708 .876 r
.47504 .65402 .47814 .65772 .48366 .65595 .48057 .6522 Metetra
.67 .706 .874 r
.48057 .6522 .48366 .65595 .4892 .65418 .48612 .65038 Metetra
.669 .704 .873 r
.48612 .65038 .4892 .65418 .49475 .65242 .49168 .64858 Metetra
.668 .701 .871 r
.49168 .64858 .49475 .65242 .50031 .65066 .49724 .64678 Metetra
.668 .699 .87 r
.49724 .64678 .50031 .65066 .50589 .64891 .50282 .64499 Metetra
.667 .697 .869 r
.50282 .64499 .50589 .64891 .51147 .64718 .50841 .64322 Metetra
.665 .695 .868 r
.50841 .64322 .51147 .64718 .51706 .64545 .51401 .64146 Metetra
.664 .693 .867 r
.51401 .64146 .51706 .64545 .52267 .64373 .51962 .63971 Metetra
.663 .692 .866 r
.51962 .63971 .52267 .64373 .52828 .64203 .52524 .63798 Metetra
.662 .69 .866 r
.52524 .63798 .52828 .64203 .53391 .64034 .53087 .63627 Metetra
.661 .689 .866 r
.53087 .63627 .53391 .64034 .53954 .63868 .53651 .63457 Metetra
.659 .687 .866 r
.53651 .63457 .53954 .63868 .54519 .63703 .54217 .63291 Metetra
.657 .687 .866 r
.54217 .63291 .54519 .63703 .55085 .6354 .54784 .63127 Metetra
.656 .686 .866 r
.54784 .63127 .55085 .6354 .55653 .63379 .55351 .62965 Metetra
.654 .685 .867 r
.55351 .62965 .55653 .63379 .56221 .63221 .55921 .62807 Metetra
.652 .685 .868 r
.55921 .62807 .56221 .63221 .56791 .63065 .56491 .62652 Metetra
.65 .684 .869 r
.56491 .62652 .56791 .63065 .57362 .62912 .57062 .625 Metetra
.648 .685 .871 r
.57062 .625 .57362 .62912 .57934 .62762 .57635 .62351 Metetra
.646 .685 .872 r
.57635 .62351 .57934 .62762 .58508 .62614 .58209 .62206 Metetra
.643 .686 .875 r
.58209 .62206 .58508 .62614 .59082 .62469 .58785 .62065 Metetra
.641 .687 .877 r
.58785 .62065 .59082 .62469 .59658 .62327 .59362 .61927 Metetra
.639 .688 .88 r
.59362 .61927 .59658 .62327 .60236 .62188 .5994 .61792 Metetra
.637 .69 .883 r
.5994 .61792 .60236 .62188 .60815 .62051 .60519 .61661 Metetra
.634 .693 .887 r
.60519 .61661 .60815 .62051 .61395 .61917 .611 .61533 Metetra
.632 .695 .891 r
.611 .61533 .61395 .61917 .61976 .61786 .61683 .61408 Metetra
.629 .698 .895 r
.61683 .61408 .61976 .61786 .62559 .61657 .62266 .61287 Metetra
.627 .702 .9 r
.62266 .61287 .62559 .61657 .63144 .6153 .62852 .61168 Metetra
.624 .706 .904 r
.62852 .61168 .63144 .6153 .63729 .61405 .63439 .61051 Metetra
.622 .711 .909 r
.63439 .61051 .63729 .61405 .64317 .61281 .64027 .60936 Metetra
.62 .716 .915 r
.64027 .60936 .64317 .61281 .64905 .61159 .64617 .60822 Metetra
.618 .721 .92 r
.64617 .60822 .64905 .61159 .65495 .61036 .65208 .6071 Metetra
.616 .727 .925 r
.65208 .6071 .65495 .61036 .66087 .60915 .658 .60598 Metetra
.614 .733 .931 r
.658 .60598 .66087 .60915 .6668 .60793 .66395 .60486 Metetra
.612 .74 .936 r
.66395 .60486 .6668 .60793 .67274 .60671 .6699 .60374 Metetra
.611 .747 .941 r
.6699 .60374 .67274 .60671 .6787 .60548 .67587 .6026 Metetra
.61 .753 .946 r
.67587 .6026 .6787 .60548 .68467 .60423 .68185 .60145 Metetra
.609 .76 .951 r
.68185 .60145 .68467 .60423 .69065 .60297 .68785 .60028 Metetra
.608 .767 .956 r
.68785 .60028 .69065 .60297 .69665 .60169 .69386 .59908 Metetra
.608 .774 .96 r
.69386 .59908 .69665 .60169 .70266 .60039 .69989 .59786 Metetra
.608 .781 .964 r
.69989 .59786 .70266 .60039 .70868 .59906 .70592 .5966 Metetra
.608 .787 .967 r
.70592 .5966 .70868 .59906 .71472 .5977 .71197 .59531 Metetra
.609 .794 .97 r
.71197 .59531 .71472 .5977 .72077 .5963 .71804 .59398 Metetra
.61 .799 .972 r
.71804 .59398 .72077 .5963 .72683 .59487 .72411 .59261 Metetra
.612 .805 .974 r
.72411 .59261 .72683 .59487 .7329 .59341 .7302 .59119 Metetra
.613 .81 .976 r
.7302 .59119 .7329 .59341 .73898 .59192 .73629 .58974 Metetra
.615 .814 .977 r
.73629 .58974 .73898 .59192 .74508 .59039 .7424 .58825 Metetra
.618 .819 .978 r
.7424 .58825 .74508 .59039 .75119 .58882 .74852 .5867 Metetra
.62 .822 .978 r
.74852 .5867 .75119 .58882 .75731 .58722 .75466 .58512 Metetra
.622 .825 .978 r
.75466 .58512 .75731 .58722 .76344 .58558 .7608 .5835 Metetra
.625 .827 .978 r
.7608 .5835 .76344 .58558 .76958 .58391 .76695 .58184 Metetra
.628 .83 .978 r
.76695 .58184 .76958 .58391 .77573 .58221 .77312 .58013 Metetra
.631 .831 .977 r
.77312 .58013 .77573 .58221 .7819 .58048 .77929 .57839 Metetra
.634 .832 .977 r
.77929 .57839 .7819 .58048 .78807 .57871 .78548 .57662 Metetra
.637 .833 .976 r
.78548 .57662 .78807 .57871 .79426 .57692 .79168 .5748 Metetra
.64 .832 .975 r
.79168 .5748 .79426 .57692 .80046 .5751 .79789 .57297 Metetra
.643 .833 .973 r
.79789 .57297 .80046 .5751 .80667 .57325 .80411 .57109 Metetra
.645 .832 .972 r
.80411 .57109 .80667 .57325 .81289 .57138 .81034 .56919 Metetra
.648 .831 .971 r
.81034 .56919 .81289 .57138 .81912 .56949 .81658 .56727 Metetra
.65 .829 .969 r
.81658 .56727 .81912 .56949 .82536 .56758 .82283 .56532 Metetra
.653 .828 .967 r
.82283 .56532 .82536 .56758 .83162 .56565 .8291 .56336 Metetra
.655 .826 .965 r
.8291 .56336 .83162 .56565 .83789 .56371 .83537 .56137 Metetra
.657 .824 .964 r
.83537 .56137 .83789 .56371 .84417 .56174 .84166 .55936 Metetra
.659 .821 .962 r
.84166 .55936 .84417 .56174 .85046 .55977 .84796 .55734 Metetra
.661 .819 .96 r
.84796 .55734 .85046 .55977 .85676 .55778 .85427 .5553 Metetra
.662 .816 .958 r
.85427 .5553 .85676 .55778 .86308 .55577 .8606 .55325 Metetra
.664 .814 .955 r
.8606 .55325 .86308 .55577 .86941 .55376 .86693 .55118 Metetra
.665 .811 .953 r
.86693 .55118 .86941 .55376 .87575 .55174 .87328 .54911 Metetra
.666 .808 .951 r
.87328 .54911 .87575 .55174 .8821 .5497 .87964 .54702 Metetra
.667 .805 .949 r
.87964 .54702 .8821 .5497 .88846 .54766 .88601 .54492 Metetra
.668 .802 .946 r
.88601 .54492 .88846 .54766 .89484 .54561 .89239 .54282 Metetra
.669 .798 .944 r
.89239 .54282 .89484 .54561 .90123 .54355 .89879 .5407 Metetra
.67 .795 .941 r
.89879 .5407 .90123 .54355 .90763 .54148 .9052 .53858 Metetra
.671 .792 .939 r
.9052 .53858 .90763 .54148 .91405 .53941 .91162 .53645 Metetra
.671 .788 .937 r
.91162 .53645 .91405 .53941 .92048 .53733 .91806 .53432 Metetra
.672 .785 .934 r
.91806 .53432 .92048 .53733 .92692 .53524 .9245 .53218 Metetra
.672 .782 .932 r
.9245 .53218 .92692 .53524 .93338 .53315 .93096 .53003 Metetra
.673 .779 .929 r
.93096 .53003 .93338 .53315 .93984 .53105 .93744 .52787 Metetra
.673 .775 .927 r
.93744 .52787 .93984 .53105 .94633 .52895 .94392 .52571 Metetra
.673 .772 .924 r
.94392 .52571 .94633 .52895 .95282 .52684 .95042 .52355 Metetra
.674 .769 .922 r
.95042 .52355 .95282 .52684 .95933 .52473 .95693 .52137 Metetra
.674 .765 .92 r
.95693 .52137 .95933 .52473 .96585 .52261 .96346 .5192 Metetra
.674 .762 .917 r
.96346 .5192 .96585 .52261 .97238 .52049 .97 .51702 Metetra
.679 .757 .91 r
.37924 .68226 .38249 .68509 .38785 .68327 .38462 .68039 Metetra
.679 .754 .907 r
.38462 .68039 .38785 .68327 .39323 .68146 .39 .67852 Metetra
.679 .751 .905 r
.39 .67852 .39323 .68146 .39861 .67964 .39539 .67665 Metetra
.679 .747 .903 r
.39539 .67665 .39861 .67964 .404 .67781 .4008 .67477 Metetra
.679 .744 .901 r
.4008 .67477 .404 .67781 .40941 .67599 .40621 .67289 Metetra
.678 .741 .898 r
.40621 .67289 .40941 .67599 .41482 .67416 .41163 .67101 Metetra
.678 .739 .896 r
.41163 .67101 .41482 .67416 .42024 .67233 .41706 .66913 Metetra
.678 .736 .894 r
.41706 .66913 .42024 .67233 .42568 .6705 .4225 .66724 Metetra
.678 .733 .892 r
.4225 .66724 .42568 .6705 .43112 .66867 .42796 .66536 Metetra
.677 .73 .889 r
.42796 .66536 .43112 .66867 .43658 .66684 .43342 .66347 Metetra
.677 .727 .887 r
.43342 .66347 .43658 .66684 .44204 .665 .43889 .66158 Metetra
.676 .724 .885 r
.43889 .66158 .44204 .665 .44751 .66317 .44437 .6597 Metetra
.676 .721 .883 r
.44437 .6597 .44751 .66317 .453 .66133 .44986 .65781 Metetra
.676 .718 .881 r
.44986 .65781 .453 .66133 .45849 .6595 .45536 .65592 Metetra
.675 .716 .879 r
.45536 .65592 .45849 .6595 .464 .65767 .46088 .65404 Metetra
.674 .713 .878 r
.46088 .65404 .464 .65767 .46951 .65584 .4664 .65217 Metetra
.674 .711 .876 r
.4664 .65217 .46951 .65584 .47504 .65402 .47193 .65029 Metetra
.673 .708 .874 r
.47193 .65029 .47504 .65402 .48057 .6522 .47747 .64843 Metetra
.672 .706 .873 r
.47747 .64843 .48057 .6522 .48612 .65038 .48302 .64656 Metetra
.671 .703 .871 r
.48302 .64656 .48612 .65038 .49168 .64858 .48859 .64471 Metetra
.67 .701 .87 r
.48859 .64471 .49168 .64858 .49724 .64678 .49416 .64287 Metetra
.669 .699 .869 r
.49416 .64287 .49724 .64678 .50282 .64499 .49974 .64104 Metetra
.668 .697 .868 r
.49974 .64104 .50282 .64499 .50841 .64322 .50534 .63923 Metetra
.667 .695 .867 r
.50534 .63923 .50841 .64322 .51401 .64146 .51094 .63743 Metetra
.666 .693 .866 r
.51094 .63743 .51401 .64146 .51962 .63971 .51656 .63565 Metetra
.664 .691 .865 r
.51656 .63565 .51962 .63971 .52524 .63798 .52219 .63389 Metetra
.663 .69 .865 r
.52219 .63389 .52524 .63798 .53087 .63627 .52782 .63215 Metetra
.661 .688 .864 r
.52782 .63215 .53087 .63627 .53651 .63457 .53347 .63044 Metetra
.659 .687 .864 r
.53347 .63044 .53651 .63457 .54217 .63291 .53913 .62875 Metetra
.658 .685 .864 r
.53913 .62875 .54217 .63291 .54784 .63127 .54481 .62709 Metetra
.655 .684 .865 r
.54481 .62709 .54784 .63127 .55351 .62965 .55049 .62547 Metetra
.653 .683 .866 r
.55049 .62547 .55351 .62965 .55921 .62807 .55619 .62389 Metetra
.651 .683 .867 r
.55619 .62389 .55921 .62807 .56491 .62652 .5619 .62234 Metetra
.649 .682 .868 r
.5619 .62234 .56491 .62652 .57062 .625 .56762 .62083 Metetra
.646 .682 .87 r
.56762 .62083 .57062 .625 .57635 .62351 .57335 .61936 Metetra
.644 .683 .872 r
.57335 .61936 .57635 .62351 .58209 .62206 .5791 .61793 Metetra
.641 .683 .874 r
.5791 .61793 .58209 .62206 .58785 .62065 .58486 .61655 Metetra
.638 .684 .876 r
.58486 .61655 .58785 .62065 .59362 .61927 .59064 .61521 Metetra
.636 .685 .879 r
.59064 .61521 .59362 .61927 .5994 .61792 .59642 .61391 Metetra
.633 .687 .883 r
.59642 .61391 .5994 .61792 .60519 .61661 .60223 .61265 Metetra
.63 .689 .886 r
.60223 .61265 .60519 .61661 .611 .61533 .60805 .61144 Metetra
.627 .692 .891 r
.60805 .61144 .611 .61533 .61683 .61408 .61388 .61026 Metetra
.624 .695 .895 r
.61388 .61026 .61683 .61408 .62266 .61287 .61972 .60912 Metetra
.621 .698 .9 r
.61972 .60912 .62266 .61287 .62852 .61168 .62559 .60802 Metetra
.618 .702 .905 r
.62559 .60802 .62852 .61168 .63439 .61051 .63146 .60694 Metetra
.615 .707 .91 r
.63146 .60694 .63439 .61051 .64027 .60936 .63736 .60588 Metetra
.613 .712 .916 r
.63736 .60588 .64027 .60936 .64617 .60822 .64327 .60484 Metetra
.61 .718 .921 r
.64327 .60484 .64617 .60822 .65208 .6071 .64919 .60381 Metetra
.607 .723 .927 r
.64919 .60381 .65208 .6071 .658 .60598 .65513 .6028 Metetra
.605 .73 .933 r
.65513 .6028 .658 .60598 .66395 .60486 .66108 .60178 Metetra
.604 .737 .939 r
.66108 .60178 .66395 .60486 .6699 .60374 .66705 .60076 Metetra
.602 .743 .944 r
.66705 .60076 .6699 .60374 .67587 .6026 .67303 .59973 Metetra
.601 .751 .949 r
.67303 .59973 .67587 .6026 .68185 .60145 .67903 .59867 Metetra
.6 .758 .954 r
.67903 .59867 .68185 .60145 .68785 .60028 .68504 .59759 Metetra
.599 .765 .959 r
.68504 .59759 .68785 .60028 .69386 .59908 .69107 .59649 Metetra
.599 .773 .963 r
.69107 .59649 .69386 .59908 .69989 .59786 .6971 .59535 Metetra
.599 .78 .967 r
.6971 .59535 .69989 .59786 .70592 .5966 .70315 .59417 Metetra
.6 .787 .97 r
.70315 .59417 .70592 .5966 .71197 .59531 .70922 .59295 Metetra
.601 .793 .973 r
.70922 .59295 .71197 .59531 .71804 .59398 .71529 .59168 Metetra
.603 .8 .975 r
.71529 .59168 .71804 .59398 .72411 .59261 .72138 .59037 Metetra
.605 .806 .977 r
.72138 .59037 .72411 .59261 .7302 .59119 .72748 .58901 Metetra
.607 .811 .978 r
.72748 .58901 .7302 .59119 .73629 .58974 .73359 .58759 Metetra
.61 .816 .979 r
.73359 .58759 .73629 .58974 .7424 .58825 .73972 .58613 Metetra
.613 .82 .98 r
.73972 .58613 .7424 .58825 .74852 .5867 .74585 .58462 Metetra
.616 .824 .98 r
.74585 .58462 .74852 .5867 .75466 .58512 .752 .58305 Metetra
.619 .827 .98 r
.752 .58305 .75466 .58512 .7608 .5835 .75815 .58144 Metetra
.623 .83 .98 r
.75815 .58144 .7608 .5835 .76695 .58184 .76432 .57978 Metetra
.626 .831 .979 r
.76432 .57978 .76695 .58184 .77312 .58013 .77049 .57808 Metetra
.63 .833 .979 r
.77049 .57808 .77312 .58013 .77929 .57839 .77668 .57633 Metetra
.634 .834 .978 r
.77668 .57633 .77929 .57839 .78548 .57662 .78288 .57454 Metetra
.637 .835 .977 r
.78288 .57454 .78548 .57662 .79168 .5748 .78909 .57271 Metetra
.641 .835 .975 r
.78909 .57271 .79168 .5748 .79789 .57297 .79531 .57084 Metetra
.644 .835 .974 r
.79531 .57084 .79789 .57297 .80411 .57109 .80154 .56895 Metetra
.647 .834 .972 r
.80154 .56895 .80411 .57109 .81034 .56919 .80778 .56701 Metetra
.65 .833 .971 r
.80778 .56701 .81034 .56919 .81658 .56727 .81403 .56506 Metetra
.652 .832 .969 r
.81403 .56506 .81658 .56727 .82283 .56532 .82029 .56307 Metetra
.655 .83 .967 r
.82029 .56307 .82283 .56532 .8291 .56336 .82656 .56106 Metetra
.658 .828 .965 r
.82656 .56106 .8291 .56336 .83537 .56137 .83285 .55903 Metetra
.66 .826 .963 r
.83285 .55903 .83537 .56137 .84166 .55936 .83915 .55698 Metetra
.662 .824 .961 r
.83915 .55698 .84166 .55936 .84796 .55734 .84545 .55491 Metetra
.664 .821 .959 r
.84545 .55491 .84796 .55734 .85427 .5553 .85177 .55282 Metetra
.665 .818 .957 r
.85177 .55282 .85427 .5553 .8606 .55325 .8581 .55072 Metetra
.667 .815 .955 r
.8581 .55072 .8606 .55325 .86693 .55118 .86445 .54861 Metetra
.668 .813 .953 r
.86445 .54861 .86693 .55118 .87328 .54911 .8708 .54648 Metetra
.67 .81 .95 r
.8708 .54648 .87328 .54911 .87964 .54702 .87717 .54434 Metetra
.671 .806 .948 r
.87717 .54434 .87964 .54702 .88601 .54492 .88354 .54218 Metetra
.672 .803 .945 r
.88354 .54218 .88601 .54492 .89239 .54282 .88994 .54002 Metetra
.672 .8 .943 r
.88994 .54002 .89239 .54282 .89879 .5407 .89634 .53785 Metetra
.673 .797 .941 r
.89634 .53785 .89879 .5407 .9052 .53858 .90275 .53568 Metetra
.674 .793 .938 r
.90275 .53568 .9052 .53858 .91162 .53645 .90918 .53349 Metetra
.674 .79 .936 r
.90918 .53349 .91162 .53645 .91806 .53432 .91562 .5313 Metetra
.675 .787 .933 r
.91562 .5313 .91806 .53432 .9245 .53218 .92207 .52909 Metetra
.675 .783 .931 r
.92207 .52909 .9245 .53218 .93096 .53003 .92854 .52689 Metetra
.676 .78 .928 r
.92854 .52689 .93096 .53003 .93744 .52787 .93502 .52468 Metetra
.676 .776 .926 r
.93502 .52468 .93744 .52787 .94392 .52571 .94151 .52246 Metetra
.677 .773 .923 r
.94151 .52246 .94392 .52571 .95042 .52355 .94801 .52023 Metetra
.677 .77 .921 r
.94801 .52023 .95042 .52355 .95693 .52137 .95453 .518 Metetra
.677 .767 .919 r
.95453 .518 .95693 .52137 .96346 .5192 .96106 .51577 Metetra
.677 .763 .916 r
.96106 .51577 .96346 .5192 .97 .51702 .9676 .51353 Metetra
.682 .757 .908 r
.37599 .67942 .37924 .68226 .38462 .68039 .38137 .6775 Metetra
.682 .754 .906 r
.38137 .6775 .38462 .68039 .39 .67852 .38676 .67557 Metetra
.682 .751 .904 r
.38676 .67557 .39 .67852 .39539 .67665 .39217 .67365 Metetra
.682 .748 .901 r
.39217 .67365 .39539 .67665 .4008 .67477 .39758 .67171 Metetra
.682 .745 .899 r
.39758 .67171 .4008 .67477 .40621 .67289 .403 .66978 Metetra
.681 .742 .897 r
.403 .66978 .40621 .67289 .41163 .67101 .40843 .66785 Metetra
.681 .739 .894 r
.40843 .66785 .41163 .67101 .41706 .66913 .41387 .66591 Metetra
.681 .736 .892 r
.41387 .66591 .41706 .66913 .4225 .66724 .41932 .66397 Metetra
.68 .733 .89 r
.41932 .66397 .4225 .66724 .42796 .66536 .42478 .66203 Metetra
.68 .73 .888 r
.42478 .66203 .42796 .66536 .43342 .66347 .43025 .66009 Metetra
.68 .727 .886 r
.43025 .66009 .43342 .66347 .43889 .66158 .43573 .65815 Metetra
.679 .725 .884 r
.43573 .65815 .43889 .66158 .44437 .6597 .44122 .65621 Metetra
.679 .722 .882 r
.44122 .65621 .44437 .6597 .44986 .65781 .44672 .65427 Metetra
.678 .719 .88 r
.44672 .65427 .44986 .65781 .45536 .65592 .45222 .65233 Metetra
.678 .716 .878 r
.45222 .65233 .45536 .65592 .46088 .65404 .45774 .6504 Metetra
.677 .714 .876 r
.45774 .6504 .46088 .65404 .4664 .65217 .46327 .64847 Metetra
.676 .711 .875 r
.46327 .64847 .4664 .65217 .47193 .65029 .46881 .64655 Metetra
.675 .709 .873 r
.46881 .64655 .47193 .65029 .47747 .64843 .47436 .64463 Metetra
.674 .706 .871 r
.47436 .64463 .47747 .64843 .48302 .64656 .47992 .64272 Metetra
.673 .704 .87 r
.47992 .64272 .48302 .64656 .48859 .64471 .48549 .64082 Metetra
.672 .701 .868 r
.48549 .64082 .48859 .64471 .49416 .64287 .49107 .63894 Metetra
.671 .699 .867 r
.49107 .63894 .49416 .64287 .49974 .64104 .49666 .63707 Metetra
.67 .697 .866 r
.49666 .63707 .49974 .64104 .50534 .63923 .50226 .63521 Metetra
.669 .694 .865 r
.50226 .63521 .50534 .63923 .51094 .63743 .50787 .63337 Metetra
.667 .692 .864 r
.50787 .63337 .51094 .63743 .51656 .63565 .51349 .63155 Metetra
.666 .691 .864 r
.51349 .63155 .51656 .63565 .52219 .63389 .51913 .62976 Metetra
.664 .688 .863 r
.51913 .62976 .52219 .63389 .52782 .63215 .52477 .62799 Metetra
.662 .687 .863 r
.52477 .62799 .52782 .63215 .53347 .63044 .53042 .62625 Metetra
.66 .685 .863 r
.53042 .62625 .53347 .63044 .53913 .62875 .53609 .62454 Metetra
.658 .684 .863 r
.53609 .62454 .53913 .62875 .54481 .62709 .54177 .62287 Metetra
.656 .682 .863 r
.54177 .62287 .54481 .62709 .55049 .62547 .54746 .62124 Metetra
.653 .681 .864 r
.54746 .62124 .55049 .62547 .55619 .62389 .55316 .61965 Metetra
.651 .681 .865 r
.55316 .61965 .55619 .62389 .5619 .62234 .55888 .6181 Metetra
.648 .68 .866 r
.55888 .6181 .5619 .62234 .56762 .62083 .5646 .6166 Metetra
.645 .68 .868 r
.5646 .6166 .56762 .62083 .57335 .61936 .57034 .61515 Metetra
.642 .679 .87 r
.57034 .61515 .57335 .61936 .5791 .61793 .5761 .61374 Metetra
.639 .68 .873 r
.5761 .61374 .5791 .61793 .58486 .61655 .58186 .61239 Metetra
.636 .68 .875 r
.58186 .61239 .58486 .61655 .59064 .61521 .58765 .61109 Metetra
.632 .681 .878 r
.58765 .61109 .59064 .61521 .59642 .61391 .59344 .60984 Metetra
.629 .683 .882 r
.59344 .60984 .59642 .61391 .60223 .61265 .59925 .60864 Metetra
.626 .685 .886 r
.59925 .60864 .60223 .61265 .60805 .61144 .60508 .60749 Metetra
.622 .687 .89 r
.60508 .60749 .60805 .61144 .61388 .61026 .61092 .60639 Metetra
.618 .69 .895 r
.61092 .60639 .61388 .61026 .61972 .60912 .61677 .60533 Metetra
.615 .694 .9 r
.61677 .60533 .61972 .60912 .62559 .60802 .62265 .60431 Metetra
.611 .698 .905 r
.62265 .60431 .62559 .60802 .63146 .60694 .62853 .60332 Metetra
.608 .702 .911 r
.62853 .60332 .63146 .60694 .63736 .60588 .63444 .60237 Metetra
.605 .708 .917 r
.63444 .60237 .63736 .60588 .64327 .60484 .64036 .60143 Metetra
.601 .713 .923 r
.64036 .60143 .64327 .60484 .64919 .60381 .64629 .60051 Metetra
.599 .72 .929 r
.64629 .60051 .64919 .60381 .65513 .6028 .65224 .5996 Metetra
.596 .726 .935 r
.65224 .5996 .65513 .6028 .66108 .60178 .65821 .59869 Metetra
.594 .733 .941 r
.65821 .59869 .66108 .60178 .66705 .60076 .66419 .59778 Metetra
.592 .741 .947 r
.66419 .59778 .66705 .60076 .67303 .59973 .67018 .59685 Metetra
.59 .748 .952 r
.67018 .59685 .67303 .59973 .67903 .59867 .67619 .5959 Metetra
.59 .756 .958 r
.67619 .5959 .67903 .59867 .68504 .59759 .68222 .59492 Metetra
.589 .763 .962 r
.68222 .59492 .68504 .59759 .69107 .59649 .68826 .59391 Metetra
.589 .771 .966 r
.68826 .59391 .69107 .59649 .6971 .59535 .69431 .59286 Metetra
.589 .779 .97 r
.69431 .59286 .6971 .59535 .70315 .59417 .70038 .59177 Metetra
.591 .786 .973 r
.70038 .59177 .70315 .59417 .70922 .59295 .70645 .59062 Metetra
.592 .794 .976 r
.70645 .59062 .70922 .59295 .71529 .59168 .71254 .58942 Metetra
.594 .8 .978 r
.71254 .58942 .71529 .59168 .72138 .59037 .71865 .58816 Metetra
.597 .806 .98 r
.71865 .58816 .72138 .59037 .72748 .58901 .72476 .58685 Metetra
.6 .812 .981 r
.72476 .58685 .72748 .58901 .73359 .58759 .73089 .58548 Metetra
.603 .817 .982 r
.73089 .58548 .73359 .58759 .73972 .58613 .73702 .58406 Metetra
.608 .822 .982 r
.73702 .58406 .73972 .58613 .74585 .58462 .74317 .58256 Metetra
.611 .826 .982 r
.74317 .58256 .74585 .58462 .752 .58305 .74933 .58102 Metetra
.616 .83 .982 r
.74933 .58102 .752 .58305 .75815 .58144 .75549 .57942 Metetra
.62 .832 .982 r
.75549 .57942 .75815 .58144 .76432 .57978 .76167 .57776 Metetra
.624 .835 .981 r
.76167 .57776 .76432 .57978 .77049 .57808 .76786 .57605 Metetra
.629 .836 .98 r
.76786 .57605 .77049 .57808 .77668 .57633 .77406 .57429 Metetra
.633 .837 .979 r
.77406 .57429 .77668 .57633 .78288 .57454 .78027 .57249 Metetra
.637 .838 .978 r
.78027 .57249 .78288 .57454 .78909 .57271 .78649 .57064 Metetra
.641 .838 .976 r
.78649 .57064 .78909 .57271 .79531 .57084 .79272 .56875 Metetra
.645 .838 .975 r
.79272 .56875 .79531 .57084 .80154 .56895 .79896 .56682 Metetra
.648 .837 .973 r
.79896 .56682 .80154 .56895 .80778 .56701 .80521 .56485 Metetra
.652 .836 .971 r
.80521 .56485 .80778 .56701 .81403 .56506 .81147 .56285 Metetra
.655 .834 .969 r
.81147 .56285 .81403 .56506 .82029 .56307 .81774 .56083 Metetra
.658 .832 .967 r
.81774 .56083 .82029 .56307 .82656 .56106 .82402 .55878 Metetra
.66 .831 .965 r
.82402 .55878 .82656 .56106 .83285 .55903 .83032 .5567 Metetra
.663 .828 .963 r
.83032 .5567 .83285 .55903 .83915 .55698 .83662 .5546 Metetra
.665 .826 .961 r
.83662 .5546 .83915 .55698 .84545 .55491 .84294 .55249 Metetra
.667 .823 .959 r
.84294 .55249 .84545 .55491 .85177 .55282 .84926 .55035 Metetra
.669 .82 .956 r
.84926 .55035 .85177 .55282 .8581 .55072 .8556 .54819 Metetra
.67 .817 .954 r
.8556 .54819 .8581 .55072 .86445 .54861 .86195 .54602 Metetra
.671 .814 .952 r
.86195 .54602 .86445 .54861 .8708 .54648 .86831 .54384 Metetra
.673 .811 .949 r
.86831 .54384 .8708 .54648 .87717 .54434 .87468 .54165 Metetra
.674 .808 .947 r
.87468 .54165 .87717 .54434 .88354 .54218 .88107 .53944 Metetra
.675 .805 .945 r
.88107 .53944 .88354 .54218 .88994 .54002 .88747 .53722 Metetra
.676 .801 .942 r
.88747 .53722 .88994 .54002 .89634 .53785 .89388 .53499 Metetra
.677 .798 .94 r
.89388 .53499 .89634 .53785 .90275 .53568 .9003 .53276 Metetra
.677 .795 .937 r
.9003 .53276 .90275 .53568 .90918 .53349 .90673 .53051 Metetra
.678 .791 .935 r
.90673 .53051 .90918 .53349 .91562 .5313 .91318 .52826 Metetra
.678 .788 .932 r
.91318 .52826 .91562 .5313 .92207 .52909 .91964 .526 Metetra
.679 .785 .93 r
.91964 .526 .92207 .52909 .92854 .52689 .92611 .52374 Metetra
.679 .781 .927 r
.92611 .52374 .92854 .52689 .93502 .52468 .93259 .52147 Metetra
.679 .778 .925 r
.93259 .52147 .93502 .52468 .94151 .52246 .93909 .51919 Metetra
.68 .775 .922 r
.93909 .51919 .94151 .52246 .94801 .52023 .9456 .51691 Metetra
.68 .771 .92 r
.9456 .51691 .94801 .52023 .95453 .518 .95212 .51462 Metetra
.68 .768 .918 r
.95212 .51462 .95453 .518 .96106 .51577 .95865 .51233 Metetra
.68 .765 .915 r
.95865 .51233 .96106 .51577 .9676 .51353 .9652 .51003 Metetra
.685 .758 .907 r
.37272 .67657 .37599 .67942 .38137 .6775 .37811 .67459 Metetra
.685 .755 .904 r
.37811 .67459 .38137 .6775 .38676 .67557 .38352 .67261 Metetra
.685 .752 .902 r
.38352 .67261 .38676 .67557 .39217 .67365 .38893 .67063 Metetra
.685 .749 .9 r
.38893 .67063 .39217 .67365 .39758 .67171 .39435 .66865 Metetra
.685 .746 .898 r
.39435 .66865 .39758 .67171 .403 .66978 .39978 .66666 Metetra
.684 .742 .895 r
.39978 .66666 .403 .66978 .40843 .66785 .40521 .66467 Metetra
.684 .74 .893 r
.40521 .66467 .40843 .66785 .41387 .66591 .41066 .66268 Metetra
.684 .737 .891 r
.41066 .66268 .41387 .66591 .41932 .66397 .41612 .66068 Metetra
.683 .734 .889 r
.41612 .66068 .41932 .66397 .42478 .66203 .42159 .65869 Metetra
.683 .731 .886 r
.42159 .65869 .42478 .66203 .43025 .66009 .42707 .65669 Metetra
.682 .728 .884 r
.42707 .65669 .43025 .66009 .43573 .65815 .43255 .6547 Metetra
.682 .725 .882 r
.43255 .6547 .43573 .65815 .44122 .65621 .43805 .6527 Metetra
.681 .722 .88 r
.43805 .6527 .44122 .65621 .44672 .65427 .44356 .65071 Metetra
.681 .719 .878 r
.44356 .65071 .44672 .65427 .45222 .65233 .44907 .64872 Metetra
.68 .717 .877 r
.44907 .64872 .45222 .65233 .45774 .6504 .4546 .64673 Metetra
.679 .714 .875 r
.4546 .64673 .45774 .6504 .46327 .64847 .46014 .64475 Metetra
.678 .711 .873 r
.46014 .64475 .46327 .64847 .46881 .64655 .46568 .64278 Metetra
.678 .709 .871 r
.46568 .64278 .46881 .64655 .47436 .64463 .47124 .64081 Metetra
.677 .706 .87 r
.47124 .64081 .47436 .64463 .47992 .64272 .4768 .63885 Metetra
.675 .704 .868 r
.4768 .63885 .47992 .64272 .48549 .64082 .48238 .63691 Metetra
.674 .701 .867 r
.48238 .63691 .48549 .64082 .49107 .63894 .48797 .63498 Metetra
.673 .699 .866 r
.48797 .63498 .49107 .63894 .49666 .63707 .49356 .63306 Metetra
.672 .696 .864 r
.49356 .63306 .49666 .63707 .50226 .63521 .49917 .63116 Metetra
.67 .694 .864 r
.49917 .63116 .50226 .63521 .50787 .63337 .50479 .62928 Metetra
.669 .692 .863 r
.50479 .62928 .50787 .63337 .51349 .63155 .51042 .62742 Metetra
.667 .69 .862 r
.51042 .62742 .51349 .63155 .51913 .62976 .51605 .62559 Metetra
.665 .688 .861 r
.51605 .62559 .51913 .62976 .52477 .62799 .5217 .62378 Metetra
.663 .685 .861 r
.5217 .62378 .52477 .62799 .53042 .62625 .52736 .62202 Metetra
.661 .684 .861 r
.52736 .62202 .53042 .62625 .53609 .62454 .53304 .62029 Metetra
.658 .682 .861 r
.53304 .62029 .53609 .62454 .54177 .62287 .53872 .61859 Metetra
.656 .681 .862 r
.53872 .61859 .54177 .62287 .54746 .62124 .54442 .61695 Metetra
.653 .679 .862 r
.54442 .61695 .54746 .62124 .55316 .61965 .55012 .61535 Metetra
.65 .678 .863 r
.55012 .61535 .55316 .61965 .55888 .6181 .55584 .61381 Metetra
.647 .677 .865 r
.55584 .61381 .55888 .6181 .5646 .6166 .56158 .61231 Metetra
.644 .676 .866 r
.56158 .61231 .5646 .6166 .57034 .61515 .56732 .61087 Metetra
.641 .676 .868 r
.56732 .61087 .57034 .61515 .5761 .61374 .57308 .60949 Metetra
.637 .676 .871 r
.57308 .60949 .5761 .61374 .58186 .61239 .57886 .60817 Metetra
.633 .677 .874 r
.57886 .60817 .58186 .61239 .58765 .61109 .58464 .60691 Metetra
.629 .678 .877 r
.58464 .60691 .58765 .61109 .59344 .60984 .59045 .60571 Metetra
.625 .679 .881 r
.59045 .60571 .59344 .60984 .59925 .60864 .59627 .60456 Metetra
.621 .68 .885 r
.59627 .60456 .59925 .60864 .60508 .60749 .6021 .60348 Metetra
.617 .683 .89 r
.6021 .60348 .60508 .60749 .61092 .60639 .60795 .60245 Metetra
.612 .686 .895 r
.60795 .60245 .61092 .60639 .61677 .60533 .61381 .60148 Metetra
.608 .689 .9 r
.61381 .60148 .61677 .60533 .62265 .60431 .61969 .60055 Metetra
.604 .693 .906 r
.61969 .60055 .62265 .60431 .62853 .60332 .62559 .59967 Metetra
.6 .698 .912 r
.62559 .59967 .62853 .60332 .63444 .60237 .6315 .59881 Metetra
.596 .703 .918 r
.6315 .59881 .63444 .60237 .64036 .60143 .63743 .59799 Metetra
.593 .709 .924 r
.63743 .59799 .64036 .60143 .64629 .60051 .64338 .59718 Metetra
.589 .715 .931 r
.64338 .59718 .64629 .60051 .65224 .5996 .64934 .59639 Metetra
.586 .722 .937 r
.64934 .59639 .65224 .5996 .65821 .59869 .65532 .59559 Metetra
.583 .729 .944 r
.65532 .59559 .65821 .59869 .66419 .59778 .66131 .59479 Metetra
.581 .737 .95 r
.66131 .59479 .66419 .59778 .67018 .59685 .66732 .59398 Metetra
.579 .745 .956 r
.66732 .59398 .67018 .59685 .67619 .5959 .67335 .59314 Metetra
.578 .753 .961 r
.67335 .59314 .67619 .5959 .68222 .59492 .67939 .59227 Metetra
.577 .762 .966 r
.67939 .59227 .68222 .59492 .68826 .59391 .68544 .59136 Metetra
.577 .77 .97 r
.68544 .59136 .68826 .59391 .69431 .59286 .69151 .5904 Metetra
.578 .778 .974 r
.69151 .5904 .69431 .59286 .70038 .59177 .69759 .5894 Metetra
.58 .786 .977 r
.69759 .5894 .70038 .59177 .70645 .59062 .70368 .58833 Metetra
.582 .794 .98 r
.70368 .58833 .70645 .59062 .71254 .58942 .70978 .5872 Metetra
.585 .801 .982 r
.70978 .5872 .71254 .58942 .71865 .58816 .7159 .58601 Metetra
.588 .808 .983 r
.7159 .58601 .71865 .58816 .72476 .58685 .72203 .58475 Metetra
.592 .814 .984 r
.72203 .58475 .72476 .58685 .73089 .58548 .72817 .58342 Metetra
.597 .819 .985 r
.72817 .58342 .73089 .58548 .73702 .58406 .73432 .58202 Metetra
.602 .824 .985 r
.73432 .58202 .73702 .58406 .74317 .58256 .74048 .58056 Metetra
.607 .829 .985 r
.74048 .58056 .74317 .58256 .74933 .58102 .74665 .57903 Metetra
.611 .832 .984 r
.74665 .57903 .74933 .58102 .75549 .57942 .75283 .57743 Metetra
.617 .835 .984 r
.75283 .57743 .75549 .57942 .76167 .57776 .75902 .57578 Metetra
.622 .837 .983 r
.75902 .57578 .76167 .57776 .76786 .57605 .76522 .57406 Metetra
.627 .839 .981 r
.76522 .57406 .76786 .57605 .77406 .57429 .77143 .57229 Metetra
.632 .841 .98 r
.77143 .57229 .77406 .57429 .78027 .57249 .77765 .57047 Metetra
.637 .841 .979 r
.77765 .57047 .78027 .57249 .78649 .57064 .78388 .56859 Metetra
.642 .841 .977 r
.78388 .56859 .78649 .57064 .79272 .56875 .79012 .56667 Metetra
.646 .84 .975 r
.79012 .56667 .79272 .56875 .79896 .56682 .79637 .56471 Metetra
.65 .84 .973 r
.79637 .56471 .79896 .56682 .80521 .56485 .80263 .5627 Metetra
.654 .838 .971 r
.80263 .5627 .80521 .56485 .81147 .56285 .8089 .56067 Metetra
.657 .837 .969 r
.8089 .56067 .81147 .56285 .81774 .56083 .81518 .5586 Metetra
.66 .835 .967 r
.81518 .5586 .81774 .56083 .82402 .55878 .82147 .5565 Metetra
.663 .833 .965 r
.82147 .5565 .82402 .55878 .83032 .5567 .82777 .55438 Metetra
.666 .831 .963 r
.82777 .55438 .83032 .5567 .83662 .5546 .83408 .55223 Metetra
.668 .828 .96 r
.83408 .55223 .83662 .5546 .84294 .55249 .84041 .55006 Metetra
.67 .825 .958 r
.84041 .55006 .84294 .55249 .84926 .55035 .84674 .54787 Metetra
.672 .822 .956 r
.84674 .54787 .84926 .55035 .8556 .54819 .85309 .54566 Metetra
.673 .819 .953 r
.85309 .54566 .8556 .54819 .86195 .54602 .85944 .54344 Metetra
.675 .816 .951 r
.85944 .54344 .86195 .54602 .86831 .54384 .86581 .5412 Metetra
.676 .813 .948 r
.86581 .5412 .86831 .54384 .87468 .54165 .87219 .53895 Metetra
.677 .81 .946 r
.87219 .53895 .87468 .54165 .88107 .53944 .87858 .53669 Metetra
.678 .806 .944 r
.87858 .53669 .88107 .53944 .88747 .53722 .88499 .53441 Metetra
.679 .803 .941 r
.88499 .53441 .88747 .53722 .89388 .53499 .8914 .53213 Metetra
.68 .8 .939 r
.8914 .53213 .89388 .53499 .9003 .53276 .89783 .52983 Metetra
.681 .796 .936 r
.89783 .52983 .9003 .53276 .90673 .53051 .90427 .52753 Metetra
.681 .793 .934 r
.90427 .52753 .90673 .53051 .91318 .52826 .91072 .52522 Metetra
.682 .79 .931 r
.91072 .52522 .91318 .52826 .91964 .526 .91719 .5229 Metetra
.682 .786 .929 r
.91719 .5229 .91964 .526 .92611 .52374 .92366 .52058 Metetra
.682 .783 .926 r
.92366 .52058 .92611 .52374 .93259 .52147 .93015 .51825 Metetra
.682 .779 .924 r
.93015 .51825 .93259 .52147 .93909 .51919 .93665 .51592 Metetra
.683 .776 .921 r
.93665 .51592 .93909 .51919 .9456 .51691 .94317 .51357 Metetra
.683 .772 .919 r
.94317 .51357 .9456 .51691 .95212 .51462 .94969 .51123 Metetra
.683 .769 .917 r
.94969 .51123 .95212 .51462 .95865 .51233 .95623 .50888 Metetra
.683 .766 .914 r
.95623 .50888 .95865 .51233 .9652 .51003 .96279 .50652 Metetra
.688 .759 .905 r
.36945 .67371 .37272 .67657 .37811 .67459 .37485 .67168 Metetra
.688 .755 .903 r
.37485 .67168 .37811 .67459 .38352 .67261 .38026 .66964 Metetra
.688 .752 .901 r
.38026 .66964 .38352 .67261 .38893 .67063 .38567 .66761 Metetra
.688 .749 .898 r
.38567 .66761 .38893 .67063 .39435 .66865 .3911 .66557 Metetra
.687 .746 .896 r
.3911 .66557 .39435 .66865 .39978 .66666 .39654 .66352 Metetra
.687 .743 .894 r
.39654 .66352 .39978 .66666 .40521 .66467 .40199 .66148 Metetra
.687 .74 .891 r
.40199 .66148 .40521 .66467 .41066 .66268 .40745 .65943 Metetra
.686 .737 .889 r
.40745 .65943 .41066 .66268 .41612 .66068 .41291 .65738 Metetra
.686 .734 .887 r
.41291 .65738 .41612 .66068 .42159 .65869 .41839 .65533 Metetra
.686 .731 .885 r
.41839 .65533 .42159 .65869 .42707 .65669 .42387 .65328 Metetra
.685 .728 .883 r
.42387 .65328 .42707 .65669 .43255 .6547 .42937 .65123 Metetra
.685 .725 .881 r
.42937 .65123 .43255 .6547 .43805 .6527 .43488 .64918 Metetra
.684 .723 .879 r
.43488 .64918 .43805 .6527 .44356 .65071 .44039 .64713 Metetra
.683 .72 .877 r
.44039 .64713 .44356 .65071 .44907 .64872 .44591 .64509 Metetra
.683 .717 .875 r
.44591 .64509 .44907 .64872 .4546 .64673 .45145 .64305 Metetra
.682 .714 .873 r
.45145 .64305 .4546 .64673 .46014 .64475 .45699 .64101 Metetra
.681 .711 .871 r
.45699 .64101 .46014 .64475 .46568 .64278 .46254 .63899 Metetra
.68 .709 .87 r
.46254 .63899 .46568 .64278 .47124 .64081 .46811 .63697 Metetra
.679 .706 .868 r
.46811 .63697 .47124 .64081 .4768 .63885 .47368 .63496 Metetra
.678 .704 .867 r
.47368 .63496 .4768 .63885 .48238 .63691 .47926 .63296 Metetra
.676 .701 .865 r
.47926 .63296 .48238 .63691 .48797 .63498 .48486 .63098 Metetra
.675 .698 .864 r
.48486 .63098 .48797 .63498 .49356 .63306 .49046 .62901 Metetra
.674 .696 .863 r
.49046 .62901 .49356 .63306 .49917 .63116 .49607 .62707 Metetra
.672 .694 .862 r
.49607 .62707 .49917 .63116 .50479 .62928 .5017 .62514 Metetra
.67 .691 .861 r
.5017 .62514 .50479 .62928 .51042 .62742 .50733 .62324 Metetra
.669 .689 .86 r
.50733 .62324 .51042 .62742 .51605 .62559 .51297 .62137 Metetra
.666 .686 .859 r
.51297 .62137 .51605 .62559 .5217 .62378 .51863 .61953 Metetra
.664 .684 .859 r
.51863 .61953 .5217 .62378 .52736 .62202 .5243 .61773 Metetra
.662 .682 .859 r
.5243 .61773 .52736 .62202 .53304 .62029 .52997 .61597 Metetra
.659 .68 .859 r
.52997 .61597 .53304 .62029 .53872 .61859 .53566 .61426 Metetra
.656 .678 .859 r
.53566 .61426 .53872 .61859 .54442 .61695 .54136 .61259 Metetra
.653 .677 .86 r
.54136 .61259 .54442 .61695 .55012 .61535 .54708 .61099 Metetra
.65 .675 .861 r
.54708 .61099 .55012 .61535 .55584 .61381 .5528 .60944 Metetra
.646 .674 .862 r
.5528 .60944 .55584 .61381 .56158 .61231 .55854 .60795 Metetra
.643 .673 .864 r
.55854 .60795 .56158 .61231 .56732 .61087 .56429 .60652 Metetra
.639 .673 .866 r
.56429 .60652 .56732 .61087 .57308 .60949 .57006 .60516 Metetra
.635 .672 .869 r
.57006 .60516 .57308 .60949 .57886 .60817 .57584 .60386 Metetra
.63 .672 .872 r
.57584 .60386 .57886 .60817 .58464 .60691 .58163 .60264 Metetra
.626 .673 .875 r
.58163 .60264 .58464 .60691 .59045 .60571 .58744 .6015 Metetra
.621 .674 .879 r
.58744 .6015 .59045 .60571 .59627 .60456 .59327 .60042 Metetra
.616 .676 .884 r
.59327 .60042 .59627 .60456 .6021 .60348 .59911 .5994 Metetra
.611 .677 .888 r
.59911 .5994 .6021 .60348 .60795 .60245 .60497 .59845 Metetra
.606 .68 .894 r
.60497 .59845 .60795 .60245 .61381 .60148 .61084 .59757 Metetra
.601 .684 .9 r
.61084 .59757 .61381 .60148 .61969 .60055 .61673 .59674 Metetra
.596 .687 .905 r
.61673 .59674 .61969 .60055 .62559 .59967 .62264 .59596 Metetra
.592 .692 .912 r
.62264 .59596 .62559 .59967 .6315 .59881 .62856 .59522 Metetra
.587 .697 .919 r
.62856 .59522 .6315 .59881 .63743 .59799 .6345 .59451 Metetra
.582 .703 .925 r
.6345 .59451 .63743 .59799 .64338 .59718 .64046 .59383 Metetra
.578 .71 .933 r
.64046 .59383 .64338 .59718 .64934 .59639 .64643 .59315 Metetra
.574 .717 .939 r
.64643 .59315 .64934 .59639 .65532 .59559 .65242 .59249 Metetra
.571 .725 .946 r
.65242 .59249 .65532 .59559 .66131 .59479 .65843 .59182 Metetra
.569 .733 .953 r
.65843 .59182 .66131 .59479 .66732 .59398 .66445 .59112 Metetra
.566 .741 .959 r
.66445 .59112 .66732 .59398 .67335 .59314 .67049 .5904 Metetra
.565 .751 .965 r
.67049 .5904 .67335 .59314 .67939 .59227 .67654 .58964 Metetra
.564 .759 .97 r
.67654 .58964 .67939 .59227 .68544 .59136 .68261 .58884 Metetra
.564 .768 .974 r
.68261 .58884 .68544 .59136 .69151 .5904 .68869 .58799 Metetra
.566 .777 .978 r
.68869 .58799 .69151 .5904 .69759 .5894 .69479 .58707 Metetra
.568 .786 .981 r
.69479 .58707 .69759 .5894 .70368 .58833 .70089 .58609 Metetra
.571 .794 .984 r
.70089 .58609 .70368 .58833 .70978 .5872 .70701 .58504 Metetra
.574 .802 .986 r
.70701 .58504 .70978 .5872 .7159 .58601 .71314 .5839 Metetra
.578 .809 .987 r
.71314 .5839 .7159 .58601 .72203 .58475 .71929 .5827 Metetra
.583 .816 .988 r
.71929 .5827 .72203 .58475 .72817 .58342 .72544 .58141 Metetra
.589 .822 .988 r
.72544 .58141 .72817 .58342 .73432 .58202 .7316 .58005 Metetra
.595 .828 .988 r
.7316 .58005 .73432 .58202 .74048 .58056 .73778 .57861 Metetra
.601 .832 .987 r
.73778 .57861 .74048 .58056 .74665 .57903 .74396 .57709 Metetra
.607 .836 .987 r
.74396 .57709 .74665 .57903 .75283 .57743 .75015 .5755 Metetra
.613 .839 .986 r
.75015 .5755 .75283 .57743 .75902 .57578 .75635 .57384 Metetra
.62 .841 .985 r
.75635 .57384 .75902 .57578 .76522 .57406 .76257 .57212 Metetra
.626 .843 .983 r
.76257 .57212 .76522 .57406 .77143 .57229 .76879 .57033 Metetra
.632 .844 .981 r
.76879 .57033 .77143 .57229 .77765 .57047 .77502 .56848 Metetra
.637 .845 .98 r
.77502 .56848 .77765 .57047 .78388 .56859 .78126 .56658 Metetra
.643 .845 .978 r
.78126 .56658 .78388 .56859 .79012 .56667 .78751 .56462 Metetra
.648 .844 .976 r
.78751 .56462 .79012 .56667 .79637 .56471 .79377 .56262 Metetra
.652 .843 .974 r
.79377 .56262 .79637 .56471 .80263 .5627 .80004 .56058 Metetra
.656 .842 .972 r
.80004 .56058 .80263 .5627 .8089 .56067 .80632 .5585 Metetra
.659 .84 .969 r
.80632 .5585 .8089 .56067 .81518 .5586 .81261 .55638 Metetra
.663 .838 .967 r
.81261 .55638 .81518 .5586 .82147 .5565 .81891 .55424 Metetra
.666 .836 .965 r
.81891 .55424 .82147 .5565 .82777 .55438 .82522 .55206 Metetra
.669 .833 .962 r
.82522 .55206 .82777 .55438 .83408 .55223 .83154 .54986 Metetra
.671 .83 .96 r
.83154 .54986 .83408 .55223 .84041 .55006 .83787 .54764 Metetra
.673 .828 .958 r
.83787 .54764 .84041 .55006 .84674 .54787 .84421 .5454 Metetra
.675 .824 .955 r
.84421 .5454 .84674 .54787 .85309 .54566 .85056 .54313 Metetra
.677 .821 .953 r
.85056 .54313 .85309 .54566 .85944 .54344 .85693 .54085 Metetra
.678 .818 .95 r
.85693 .54085 .85944 .54344 .86581 .5412 .8633 .53856 Metetra
.68 .815 .948 r
.8633 .53856 .86581 .5412 .87219 .53895 .86969 .53625 Metetra
.681 .811 .945 r
.86969 .53625 .87219 .53895 .87858 .53669 .87609 .53393 Metetra
.682 .808 .943 r
.87609 .53393 .87858 .53669 .88499 .53441 .8825 .53159 Metetra
.683 .805 .94 r
.8825 .53159 .88499 .53441 .8914 .53213 .88892 .52925 Metetra
.683 .801 .938 r
.88892 .52925 .8914 .53213 .89783 .52983 .89535 .5269 Metetra
.684 .798 .935 r
.89535 .5269 .89783 .52983 .90427 .52753 .9018 .52454 Metetra
.684 .794 .933 r
.9018 .52454 .90427 .52753 .91072 .52522 .90826 .52217 Metetra
.685 .791 .93 r
.90826 .52217 .91072 .52522 .91719 .5229 .91473 .51979 Metetra
.685 .787 .928 r
.91473 .51979 .91719 .5229 .92366 .52058 .92121 .51741 Metetra
.685 .784 .925 r
.92121 .51741 .92366 .52058 .93015 .51825 .9277 .51502 Metetra
.686 .781 .923 r
.9277 .51502 .93015 .51825 .93665 .51592 .93421 .51263 Metetra
.686 .777 .92 r
.93421 .51263 .93665 .51592 .94317 .51357 .94073 .51022 Metetra
.686 .774 .918 r
.94073 .51022 .94317 .51357 .94969 .51123 .94726 .50782 Metetra
.686 .77 .916 r
.94726 .50782 .94969 .51123 .95623 .50888 .9538 .50541 Metetra
.686 .767 .913 r
.9538 .50541 .95623 .50888 .96279 .50652 .96036 .50299 Metetra
.691 .759 .904 r
.36615 .67084 .36945 .67371 .37485 .67168 .37156 .66875 Metetra
.691 .756 .901 r
.37156 .66875 .37485 .67168 .38026 .66964 .37698 .66666 Metetra
.691 .753 .899 r
.37698 .66666 .38026 .66964 .38567 .66761 .38241 .66457 Metetra
.691 .75 .897 r
.38241 .66457 .38567 .66761 .3911 .66557 .38785 .66247 Metetra
.69 .746 .894 r
.38785 .66247 .3911 .66557 .39654 .66352 .3933 .66037 Metetra
.69 .744 .892 r
.3933 .66037 .39654 .66352 .40199 .66148 .39875 .65827 Metetra
.69 .74 .89 r
.39875 .65827 .40199 .66148 .40745 .65943 .40422 .65617 Metetra
.689 .738 .888 r
.40422 .65617 .40745 .65943 .41291 .65738 .40969 .65406 Metetra
.689 .735 .886 r
.40969 .65406 .41291 .65738 .41839 .65533 .41518 .65196 Metetra
.688 .732 .884 r
.41518 .65196 .41839 .65533 .42387 .65328 .42067 .64985 Metetra
.688 .729 .881 r
.42067 .64985 .42387 .65328 .42937 .65123 .42618 .64775 Metetra
.687 .726 .879 r
.42618 .64775 .42937 .65123 .43488 .64918 .43169 .64564 Metetra
.687 .723 .877 r
.43169 .64564 .43488 .64918 .44039 .64713 .43721 .64354 Metetra
.686 .72 .875 r
.43721 .64354 .44039 .64713 .44591 .64509 .44274 .64144 Metetra
.685 .717 .874 r
.44274 .64144 .44591 .64509 .45145 .64305 .44828 .63934 Metetra
.684 .714 .872 r
.44828 .63934 .45145 .64305 .45699 .64101 .45383 .63725 Metetra
.683 .712 .87 r
.45383 .63725 .45699 .64101 .46254 .63899 .4594 .63517 Metetra
.682 .709 .868 r
.4594 .63517 .46254 .63899 .46811 .63697 .46497 .6331 Metetra
.681 .706 .867 r
.46497 .6331 .46811 .63697 .47368 .63496 .47055 .63103 Metetra
.68 .703 .865 r
.47055 .63103 .47368 .63496 .47926 .63296 .47614 .62898 Metetra
.679 .701 .863 r
.47614 .62898 .47926 .63296 .48486 .63098 .48173 .62695 Metetra
.677 .698 .862 r
.48173 .62695 .48486 .63098 .49046 .62901 .48734 .62493 Metetra
.676 .696 .861 r
.48734 .62493 .49046 .62901 .49607 .62707 .49296 .62293 Metetra
.674 .693 .86 r
.49296 .62293 .49607 .62707 .5017 .62514 .49859 .62096 Metetra
.672 .69 .859 r
.49859 .62096 .5017 .62514 .50733 .62324 .50423 .61902 Metetra
.67 .688 .858 r
.50423 .61902 .50733 .62324 .51297 .62137 .50988 .6171 Metetra
.668 .685 .857 r
.50988 .6171 .51297 .62137 .51863 .61953 .51555 .61522 Metetra
.665 .683 .857 r
.51555 .61522 .51863 .61953 .5243 .61773 .52122 .61339 Metetra
.663 .681 .857 r
.52122 .61339 .5243 .61773 .52997 .61597 .5269 .6116 Metetra
.66 .678 .857 r
.5269 .6116 .52997 .61597 .53566 .61426 .5326 .60985 Metetra
.657 .676 .857 r
.5326 .60985 .53566 .61426 .54136 .61259 .5383 .60817 Metetra
.653 .674 .858 r
.5383 .60817 .54136 .61259 .54708 .61099 .54402 .60655 Metetra
.65 .672 .859 r
.54402 .60655 .54708 .61099 .5528 .60944 .54975 .60499 Metetra
.646 .671 .86 r
.54975 .60499 .5528 .60944 .55854 .60795 .5555 .6035 Metetra
.642 .67 .862 r
.5555 .6035 .55854 .60795 .56429 .60652 .56125 .60208 Metetra
.637 .669 .864 r
.56125 .60208 .56429 .60652 .57006 .60516 .56703 .60074 Metetra
.633 .668 .866 r
.56703 .60074 .57006 .60516 .57584 .60386 .57281 .59947 Metetra
.627 .668 .869 r
.57281 .59947 .57584 .60386 .58163 .60264 .57861 .5983 Metetra
.622 .668 .873 r
.57861 .5983 .58163 .60264 .58744 .6015 .58443 .5972 Metetra
.617 .669 .877 r
.58443 .5972 .58744 .6015 .59327 .60042 .59026 .59618 Metetra
.612 .67 .882 r
.59026 .59618 .59327 .60042 .59911 .5994 .59611 .59524 Metetra
.606 .672 .887 r
.59611 .59524 .59911 .5994 .60497 .59845 .60197 .59438 Metetra
.6 .674 .893 r
.60197 .59438 .60497 .59845 .61084 .59757 .60785 .59359 Metetra
.594 .677 .898 r
.60785 .59359 .61084 .59757 .61673 .59674 .61375 .59287 Metetra
.588 .681 .905 r
.61375 .59287 .61673 .59674 .62264 .59596 .61967 .5922 Metetra
.583 .686 .912 r
.61967 .5922 .62264 .59596 .62856 .59522 .6256 .59158 Metetra
.577 .691 .919 r
.6256 .59158 .62856 .59522 .6345 .59451 .63155 .59099 Metetra
.572 .697 .927 r
.63155 .59099 .6345 .59451 .64046 .59383 .63752 .59044 Metetra
.566 .704 .934 r
.63752 .59044 .64046 .59383 .64643 .59315 .64351 .58991 Metetra
.562 .711 .941 r
.64351 .58991 .64643 .59315 .65242 .59249 .64951 .58937 Metetra
.558 .72 .949 r
.64951 .58937 .65242 .59249 .65843 .59182 .65553 .58883 Metetra
.554 .728 .956 r
.65553 .58883 .65843 .59182 .66445 .59112 .66157 .58828 Metetra
.552 .738 .962 r
.66157 .58828 .66445 .59112 .67049 .5904 .66762 .58768 Metetra
.55 .747 .968 r
.66762 .58768 .67049 .5904 .67654 .58964 .67369 .58705 Metetra
.549 .757 .974 r
.67369 .58705 .67654 .58964 .68261 .58884 .67977 .58636 Metetra
.55 .767 .978 r
.67977 .58636 .68261 .58884 .68869 .58799 .68587 .58562 Metetra
.551 .776 .982 r
.68587 .58562 .68869 .58799 .69479 .58707 .69198 .5848 Metetra
.554 .786 .985 r
.69198 .5848 .69479 .58707 .70089 .58609 .6981 .58391 Metetra
.558 .795 .988 r
.6981 .58391 .70089 .58609 .70701 .58504 .70423 .58293 Metetra
.562 .803 .989 r
.70423 .58293 .70701 .58504 .71314 .5839 .71038 .58186 Metetra
.567 .811 .99 r
.71038 .58186 .71314 .5839 .71929 .5827 .71654 .58071 Metetra
.574 .819 .991 r
.71654 .58071 .71929 .5827 .72544 .58141 .7227 .57947 Metetra
.58 .825 .991 r
.7227 .57947 .72544 .58141 .7316 .58005 .72888 .57814 Metetra
.588 .831 .991 r
.72888 .57814 .7316 .58005 .73778 .57861 .73507 .57672 Metetra
.595 .836 .99 r
.73507 .57672 .73778 .57861 .74396 .57709 .74126 .57521 Metetra
.603 .84 .989 r
.74126 .57521 .74396 .57709 .75015 .5755 .74747 .57363 Metetra
.61 .843 .988 r
.74747 .57363 .75015 .5755 .75635 .57384 .75368 .57196 Metetra
.617 .846 .986 r
.75368 .57196 .75635 .57384 .76257 .57212 .75991 .57022 Metetra
.625 .848 .985 r
.75991 .57022 .76257 .57212 .76879 .57033 .76614 .56841 Metetra
.631 .849 .983 r
.76614 .56841 .76879 .57033 .77502 .56848 .77238 .56653 Metetra
.638 .849 .981 r
.77238 .56653 .77502 .56848 .78126 .56658 .77863 .5646 Metetra
.644 .849 .979 r
.77863 .5646 .78126 .56658 .78751 .56462 .78489 .5626 Metetra
.649 .848 .976 r
.78489 .5626 .78751 .56462 .79377 .56262 .79116 .56057 Metetra
.654 .847 .974 r
.79116 .56057 .79377 .56262 .80004 .56058 .79744 .55848 Metetra
.658 .845 .972 r
.79744 .55848 .80004 .56058 .80632 .5585 .80373 .55635 Metetra
.662 .844 .97 r
.80373 .55635 .80632 .5585 .81261 .55638 .81002 .55418 Metetra
.666 .841 .967 r
.81002 .55418 .81261 .55638 .81891 .55424 .81633 .55199 Metetra
.669 .839 .965 r
.81633 .55199 .81891 .55424 .82522 .55206 .82265 .54976 Metetra
.672 .836 .962 r
.82265 .54976 .82522 .55206 .83154 .54986 .82898 .5475 Metetra
.674 .833 .96 r
.82898 .5475 .83154 .54986 .83787 .54764 .83532 .54523 Metetra
.677 .83 .957 r
.83532 .54523 .83787 .54764 .84421 .5454 .84167 .54292 Metetra
.679 .827 .954 r
.84167 .54292 .84421 .5454 .85056 .54313 .84803 .5406 Metetra
.68 .823 .952 r
.84803 .5406 .85056 .54313 .85693 .54085 .8544 .53827 Metetra
.682 .82 .949 r
.8544 .53827 .85693 .54085 .8633 .53856 .86078 .53591 Metetra
.683 .817 .947 r
.86078 .53591 .8633 .53856 .86969 .53625 .86718 .53355 Metetra
.684 .813 .944 r
.86718 .53355 .86969 .53625 .87609 .53393 .87358 .53116 Metetra
.685 .81 .942 r
.87358 .53116 .87609 .53393 .8825 .53159 .88 .52877 Metetra
.686 .806 .939 r
.88 .52877 .8825 .53159 .88892 .52925 .88643 .52637 Metetra
.687 .803 .937 r
.88643 .52637 .88892 .52925 .89535 .5269 .89287 .52396 Metetra
.687 .799 .934 r
.89287 .52396 .89535 .5269 .9018 .52454 .89932 .52154 Metetra
.688 .796 .932 r
.89932 .52154 .9018 .52454 .90826 .52217 .90578 .51911 Metetra
.688 .792 .929 r
.90578 .51911 .90826 .52217 .91473 .51979 .91226 .51667 Metetra
.688 .789 .927 r
.91226 .51667 .91473 .51979 .92121 .51741 .91875 .51423 Metetra
.689 .785 .924 r
.91875 .51423 .92121 .51741 .9277 .51502 .92525 .51178 Metetra
.689 .782 .922 r
.92525 .51178 .9277 .51502 .93421 .51263 .93176 .50932 Metetra
.689 .778 .919 r
.93176 .50932 .93421 .51263 .94073 .51022 .93828 .50686 Metetra
.689 .775 .917 r
.93828 .50686 .94073 .51022 .94726 .50782 .94482 .50439 Metetra
.689 .772 .915 r
.94482 .50439 .94726 .50782 .9538 .50541 .95137 .50192 Metetra
.689 .768 .912 r
.95137 .50192 .9538 .50541 .96036 .50299 .95793 .49945 Metetra
.694 .76 .902 r
.36285 .66795 .36615 .67084 .37156 .66875 .36827 .66581 Metetra
.694 .756 .9 r
.36827 .66581 .37156 .66875 .37698 .66666 .3737 .66367 Metetra
.694 .753 .897 r
.3737 .66367 .37698 .66666 .38241 .66457 .37914 .66152 Metetra
.694 .75 .895 r
.37914 .66152 .38241 .66457 .38785 .66247 .38458 .65936 Metetra
.693 .747 .893 r
.38458 .65936 .38785 .66247 .3933 .66037 .39004 .65721 Metetra
.693 .744 .891 r
.39004 .65721 .3933 .66037 .39875 .65827 .39551 .65505 Metetra
.692 .741 .888 r
.39551 .65505 .39875 .65827 .40422 .65617 .40098 .65289 Metetra
.692 .738 .886 r
.40098 .65289 .40422 .65617 .40969 .65406 .40646 .65073 Metetra
.692 .735 .884 r
.40646 .65073 .40969 .65406 .41518 .65196 .41196 .64857 Metetra
.691 .732 .882 r
.41196 .64857 .41518 .65196 .42067 .64985 .41746 .64641 Metetra
.69 .729 .88 r
.41746 .64641 .42067 .64985 .42618 .64775 .42297 .64424 Metetra
.69 .726 .878 r
.42297 .64424 .42618 .64775 .43169 .64564 .42849 .64208 Metetra
.689 .723 .876 r
.42849 .64208 .43169 .64564 .43721 .64354 .43402 .63992 Metetra
.688 .72 .874 r
.43402 .63992 .43721 .64354 .44274 .64144 .43956 .63777 Metetra
.688 .718 .872 r
.43956 .63777 .44274 .64144 .44828 .63934 .44511 .63561 Metetra
.687 .715 .87 r
.44511 .63561 .44828 .63934 .45383 .63725 .45067 .63347 Metetra
.686 .712 .868 r
.45067 .63347 .45383 .63725 .4594 .63517 .45624 .63133 Metetra
.685 .709 .866 r
.45624 .63133 .4594 .63517 .46497 .6331 .46181 .6292 Metetra
.684 .706 .865 r
.46181 .6292 .46497 .6331 .47055 .63103 .4674 .62708 Metetra
.682 .703 .863 r
.4674 .62708 .47055 .63103 .47614 .62898 .473 .62497 Metetra
.681 .701 .862 r
.473 .62497 .47614 .62898 .48173 .62695 .4786 .62289 Metetra
.68 .698 .86 r
.4786 .62289 .48173 .62695 .48734 .62493 .48422 .62081 Metetra
.678 .695 .859 r
.48422 .62081 .48734 .62493 .49296 .62293 .48985 .61876 Metetra
.676 .692 .858 r
.48985 .61876 .49296 .62293 .49859 .62096 .49548 .61674 Metetra
.674 .689 .856 r
.49548 .61674 .49859 .62096 .50423 .61902 .50113 .61475 Metetra
.672 .687 .856 r
.50113 .61475 .50423 .61902 .50988 .6171 .50679 .61278 Metetra
.669 .684 .855 r
.50679 .61278 .50988 .6171 .51555 .61522 .51245 .61086 Metetra
.667 .682 .854 r
.51245 .61086 .51555 .61522 .52122 .61339 .51813 .60898 Metetra
.664 .679 .854 r
.51813 .60898 .52122 .61339 .5269 .6116 .52382 .60716 Metetra
.661 .676 .854 r
.52382 .60716 .5269 .6116 .5326 .60985 .52952 .60538 Metetra
.657 .674 .854 r
.52952 .60538 .5326 .60985 .5383 .60817 .53523 .60367 Metetra
.654 .672 .855 r
.53523 .60367 .5383 .60817 .54402 .60655 .54096 .60203 Metetra
.65 .67 .856 r
.54096 .60203 .54402 .60655 .54975 .60499 .54669 .60045 Metetra
.645 .668 .857 r
.54669 .60045 .54975 .60499 .5555 .6035 .55244 .59896 Metetra
.641 .666 .859 r
.55244 .59896 .5555 .6035 .56125 .60208 .5582 .59755 Metetra
.636 .665 .861 r
.5582 .59755 .56125 .60208 .56703 .60074 .56398 .59622 Metetra
.63 .663 .863 r
.56398 .59622 .56703 .60074 .57281 .59947 .56977 .59499 Metetra
.625 .663 .867 r
.56977 .59499 .57281 .59947 .57861 .5983 .57558 .59385 Metetra
.619 .662 .87 r
.57558 .59385 .57861 .5983 .58443 .5972 .5814 .5928 Metetra
.613 .663 .875 r
.5814 .5928 .58443 .5972 .59026 .59618 .58724 .59185 Metetra
.607 .664 .879 r
.58724 .59185 .59026 .59618 .59611 .59524 .59309 .59099 Metetra
.6 .665 .885 r
.59309 .59099 .59611 .59524 .60197 .59438 .59897 .59022 Metetra
.593 .667 .891 r
.59897 .59022 .60197 .59438 .60785 .59359 .60486 .58953 Metetra
.586 .67 .897 r
.60486 .58953 .60785 .59359 .61375 .59287 .61076 .58892 Metetra
.579 .674 .904 r
.61076 .58892 .61375 .59287 .61967 .5922 .61669 .58837 Metetra
.573 .679 .912 r
.61669 .58837 .61967 .5922 .6256 .59158 .62263 .58788 Metetra
.566 .684 .919 r
.62263 .58788 .6256 .59158 .63155 .59099 .6286 .58744 Metetra
.559 .69 .927 r
.6286 .58744 .63155 .59099 .63752 .59044 .63458 .58703 Metetra
.553 .697 .936 r
.63458 .58703 .63752 .59044 .64351 .58991 .64058 .58664 Metetra
.548 .705 .943 r
.64058 .58664 .64351 .58991 .64951 .58937 .64659 .58625 Metetra
.543 .714 .951 r
.64659 .58625 .64951 .58937 .65553 .58883 .65263 .58586 Metetra
.539 .723 .959 r
.65263 .58586 .65553 .58883 .66157 .58828 .65868 .58545 Metetra
.535 .733 .966 r
.65868 .58545 .66157 .58828 .66762 .58768 .66474 .585 Metetra
.533 .743 .972 r
.66474 .585 .66762 .58768 .67369 .58705 .67083 .5845 Metetra
.532 .754 .977 r
.67083 .5845 .67369 .58705 .67977 .58636 .67692 .58393 Metetra
.532 .764 .982 r
.67692 .58393 .67977 .58636 .68587 .58562 .68304 .5833 Metetra
.534 .775 .986 r
.68304 .5833 .68587 .58562 .69198 .5848 .68916 .58259 Metetra
.537 .785 .989 r
.68916 .58259 .69198 .5848 .6981 .58391 .6953 .58179 Metetra
.542 .795 .991 r
.6953 .58179 .6981 .58391 .70423 .58293 .70145 .5809 Metetra
.548 .805 .993 r
.70145 .5809 .70423 .58293 .71038 .58186 .70761 .5799 Metetra
.554 .813 .994 r
.70761 .5799 .71038 .58186 .71654 .58071 .71378 .57881 Metetra
.562 .821 .994 r
.71378 .57881 .71654 .58071 .7227 .57947 .71996 .57761 Metetra
.571 .829 .994 r
.71996 .57761 .7227 .57947 .72888 .57814 .72615 .57631 Metetra
.58 .835 .993 r
.72615 .57631 .72888 .57814 .73507 .57672 .73235 .57491 Metetra
.588 .84 .992 r
.73235 .57491 .73507 .57672 .74126 .57521 .73856 .57341 Metetra
.598 .845 .991 r
.73856 .57341 .74126 .57521 .74747 .57363 .74478 .57182 Metetra
.606 .848 .99 r
.74478 .57182 .74747 .57363 .75368 .57196 .751 .57014 Metetra
.615 .851 .988 r
.751 .57014 .75368 .57196 .75991 .57022 .75724 .56838 Metetra
.623 .852 .986 r
.75724 .56838 .75991 .57022 .76614 .56841 .76348 .56654 Metetra
.631 .853 .984 r
.76348 .56654 .76614 .56841 .77238 .56653 .76973 .56463 Metetra
.638 .854 .982 r
.76973 .56463 .77238 .56653 .77863 .5646 .77599 .56266 Metetra
.645 .853 .979 r
.77599 .56266 .77863 .5646 .78489 .5626 .78226 .56063 Metetra
.651 .853 .977 r
.78226 .56063 .78489 .5626 .79116 .56057 .78854 .55854 Metetra
.656 .851 .975 r
.78854 .55854 .79116 .56057 .79744 .55848 .79483 .5564 Metetra
.661 .849 .972 r
.79483 .5564 .79744 .55848 .80373 .55635 .80113 .55422 Metetra
.665 .847 .969 r
.80113 .55422 .80373 .55635 .81002 .55418 .80743 .552 Metetra
.669 .845 .967 r
.80743 .552 .81002 .55418 .81633 .55199 .81375 .54975 Metetra
.672 .842 .964 r
.81375 .54975 .81633 .55199 .82265 .54976 .82008 .54746 Metetra
.675 .839 .962 r
.82008 .54746 .82265 .54976 .82898 .5475 .82641 .54515 Metetra
.678 .836 .959 r
.82641 .54515 .82898 .5475 .83532 .54523 .83276 .54282 Metetra
.68 .832 .956 r
.83276 .54282 .83532 .54523 .84167 .54292 .83912 .54046 Metetra
.682 .829 .954 r
.83912 .54046 .84167 .54292 .84803 .5406 .84549 .53808 Metetra
.684 .826 .951 r
.84549 .53808 .84803 .5406 .8544 .53827 .85186 .53568 Metetra
.685 .822 .949 r
.85186 .53568 .8544 .53827 .86078 .53591 .85825 .53327 Metetra
.687 .819 .946 r
.85825 .53327 .86078 .53591 .86718 .53355 .86465 .53084 Metetra
.688 .815 .943 r
.86465 .53084 .86718 .53355 .87358 .53116 .87107 .5284 Metetra
.689 .812 .941 r
.87107 .5284 .87358 .53116 .88 .52877 .87749 .52594 Metetra
.689 .808 .938 r
.87749 .52594 .88 .52877 .88643 .52637 .88392 .52348 Metetra
.69 .804 .936 r
.88392 .52348 .88643 .52637 .89287 .52396 .89037 .52101 Metetra
.691 .801 .933 r
.89037 .52101 .89287 .52396 .89932 .52154 .89683 .51853 Metetra
.691 .797 .931 r
.89683 .51853 .89932 .52154 .90578 .51911 .9033 .51604 Metetra
.691 .794 .928 r
.9033 .51604 .90578 .51911 .91226 .51667 .90978 .51354 Metetra
.692 .79 .926 r
.90978 .51354 .91226 .51667 .91875 .51423 .91627 .51103 Metetra
.692 .787 .923 r
.91627 .51103 .91875 .51423 .92525 .51178 .92278 .50852 Metetra
.692 .783 .921 r
.92278 .50852 .92525 .51178 .93176 .50932 .92929 .50601 Metetra
.692 .78 .918 r
.92929 .50601 .93176 .50932 .93828 .50686 .93582 .50349 Metetra
.692 .776 .916 r
.93582 .50349 .93828 .50686 .94482 .50439 .94236 .50096 Metetra
.692 .773 .914 r
.94236 .50096 .94482 .50439 .95137 .50192 .94892 .49843 Metetra
.692 .769 .911 r
.94892 .49843 .95137 .50192 .95793 .49945 .95548 .49589 Metetra
.697 .76 .9 r
.35954 .66506 .36285 .66795 .36827 .66581 .36497 .66286 Metetra
.697 .757 .898 r
.36497 .66286 .36827 .66581 .3737 .66367 .3704 .66066 Metetra
.697 .754 .896 r
.3704 .66066 .3737 .66367 .37914 .66152 .37585 .65845 Metetra
.696 .751 .894 r
.37585 .65845 .37914 .66152 .38458 .65936 .38131 .65625 Metetra
.696 .748 .891 r
.38131 .65625 .38458 .65936 .39004 .65721 .38677 .65403 Metetra
.696 .745 .889 r
.38677 .65403 .39004 .65721 .39551 .65505 .39225 .65182 Metetra
.695 .742 .887 r
.39225 .65182 .39551 .65505 .40098 .65289 .39773 .6496 Metetra
.695 .739 .885 r
.39773 .6496 .40098 .65289 .40646 .65073 .40322 .64739 Metetra
.694 .736 .883 r
.40322 .64739 .40646 .65073 .41196 .64857 .40872 .64517 Metetra
.694 .732 .88 r
.40872 .64517 .41196 .64857 .41746 .64641 .41423 .64295 Metetra
.693 .73 .878 r
.41423 .64295 .41746 .64641 .42297 .64424 .41975 .64072 Metetra
.692 .727 .876 r
.41975 .64072 .42297 .64424 .42849 .64208 .42528 .63851 Metetra
.692 .724 .874 r
.42528 .63851 .42849 .64208 .43402 .63992 .43082 .63629 Metetra
.691 .721 .872 r
.43082 .63629 .43402 .63992 .43956 .63777 .43637 .63407 Metetra
.69 .718 .87 r
.43637 .63407 .43956 .63777 .44511 .63561 .44193 .63186 Metetra
.689 .715 .868 r
.44193 .63186 .44511 .63561 .45067 .63347 .44749 .62966 Metetra
.688 .712 .867 r
.44749 .62966 .45067 .63347 .45624 .63133 .45307 .62746 Metetra
.687 .709 .865 r
.45307 .62746 .45624 .63133 .46181 .6292 .45865 .62527 Metetra
.686 .706 .863 r
.45865 .62527 .46181 .6292 .4674 .62708 .46425 .6231 Metetra
.685 .703 .861 r
.46425 .6231 .4674 .62708 .473 .62497 .46985 .62093 Metetra
.683 .7 .86 r
.46985 .62093 .473 .62497 .4786 .62289 .47546 .61879 Metetra
.682 .697 .858 r
.47546 .61879 .4786 .62289 .48422 .62081 .48109 .61666 Metetra
.68 .694 .857 r
.48109 .61666 .48422 .62081 .48985 .61876 .48672 .61455 Metetra
.678 .692 .855 r
.48672 .61455 .48985 .61876 .49548 .61674 .49236 .61247 Metetra
.676 .689 .854 r
.49236 .61247 .49548 .61674 .50113 .61475 .49801 .61042 Metetra
.674 .686 .853 r
.49801 .61042 .50113 .61475 .50679 .61278 .50368 .60841 Metetra
.672 .683 .852 r
.50368 .60841 .50679 .61278 .51245 .61086 .50935 .60643 Metetra
.668 .68 .852 r
.50935 .60643 .51245 .61086 .51813 .60898 .51503 .60451 Metetra
.666 .677 .851 r
.51503 .60451 .51813 .60898 .52382 .60716 .52073 .60264 Metetra
.662 .674 .851 r
.52073 .60264 .52382 .60716 .52952 .60538 .52643 .60083 Metetra
.658 .671 .851 r
.52643 .60083 .52952 .60538 .53523 .60367 .53215 .59909 Metetra
.654 .669 .852 r
.53215 .59909 .53523 .60367 .54096 .60203 .53788 .59742 Metetra
.65 .666 .852 r
.53788 .59742 .54096 .60203 .54669 .60045 .54362 .59583 Metetra
.645 .664 .854 r
.54362 .59583 .54669 .60045 .55244 .59896 .54938 .59432 Metetra
.64 .662 .855 r
.54938 .59432 .55244 .59896 .5582 .59755 .55514 .59291 Metetra
.635 .66 .857 r
.55514 .59291 .5582 .59755 .56398 .59622 .56093 .5916 Metetra
.629 .658 .86 r
.56093 .5916 .56398 .59622 .56977 .59499 .56672 .59039 Metetra
.622 .657 .863 r
.56672 .59039 .56977 .59499 .57558 .59385 .57254 .58929 Metetra
.616 .657 .867 r
.57254 .58929 .57558 .59385 .5814 .5928 .57836 .5883 Metetra
.609 .656 .871 r
.57836 .5883 .5814 .5928 .58724 .59185 .58421 .58741 Metetra
.601 .657 .877 r
.58421 .58741 .58724 .59185 .59309 .59099 .59007 .58663 Metetra
.594 .658 .882 r
.59007 .58663 .59309 .59099 .59897 .59022 .59595 .58596 Metetra
.586 .66 .889 r
.59595 .58596 .59897 .59022 .60486 .58953 .60185 .58538 Metetra
.578 .662 .896 r
.60185 .58538 .60486 .58953 .61076 .58892 .60776 .58489 Metetra
.57 .666 .903 r
.60776 .58489 .61076 .58892 .61669 .58837 .6137 .58448 Metetra
.562 .671 .911 r
.6137 .58448 .61669 .58837 .62263 .58788 .61965 .58413 Metetra
.554 .676 .919 r
.61965 .58413 .62263 .58788 .6286 .58744 .62563 .58384 Metetra
.546 .682 .928 r
.62563 .58384 .6286 .58744 .63458 .58703 .63162 .58358 Metetra
.538 .689 .937 r
.63162 .58358 .63458 .58703 .64058 .58664 .63763 .58335 Metetra
.532 .698 .945 r
.63763 .58335 .64058 .58664 .64659 .58625 .64366 .58313 Metetra
.526 .707 .954 r
.64366 .58313 .64659 .58625 .65263 .58586 .64971 .5829 Metetra
.52 .716 .962 r
.64971 .5829 .65263 .58586 .65868 .58545 .65577 .58264 Metetra
.516 .727 .969 r
.65577 .58264 .65868 .58545 .66474 .585 .66186 .58234 Metetra
.514 .738 .976 r
.66186 .58234 .66474 .585 .67083 .5845 .66795 .58199 Metetra
.512 .75 .981 r
.66795 .58199 .67083 .5845 .67692 .58393 .67407 .58157 Metetra
.513 .761 .986 r
.67407 .58157 .67692 .58393 .68304 .5833 .68019 .58106 Metetra
.514 .772 .99 r
.68019 .58106 .68304 .5833 .68916 .58259 .68633 .58046 Metetra
.518 .784 .993 r
.68633 .58046 .68916 .58259 .6953 .58179 .69249 .57976 Metetra
.524 .795 .995 r
.69249 .57976 .6953 .58179 .70145 .5809 .69865 .57895 Metetra
.531 .805 .996 r
.69865 .57895 .70145 .5809 .70761 .5799 .70483 .57803 Metetra
.539 .815 .997 r
.70483 .57803 .70761 .5799 .71378 .57881 .71101 .57699 Metetra
.549 .824 .997 r
.71101 .57699 .71378 .57881 .71996 .57761 .71721 .57584 Metetra
.559 .832 .997 r
.71721 .57584 .71996 .57761 .72615 .57631 .72341 .57456 Metetra
.57 .839 .996 r
.72341 .57456 .72615 .57631 .73235 .57491 .72963 .57318 Metetra
.581 .845 .995 r
.72963 .57318 .73235 .57491 .73856 .57341 .73585 .57168 Metetra
.592 .85 .993 r
.73585 .57168 .73856 .57341 .74478 .57182 .74208 .57008 Metetra
.603 .854 .991 r
.74208 .57008 .74478 .57182 .751 .57014 .74832 .56839 Metetra
.613 .856 .99 r
.74832 .56839 .751 .57014 .75724 .56838 .75456 .5666 Metetra
.622 .858 .987 r
.75456 .5666 .75724 .56838 .76348 .56654 .76082 .56473 Metetra
.631 .859 .985 r
.76082 .56473 .76348 .56654 .76973 .56463 .76708 .56278 Metetra
.639 .859 .983 r
.76708 .56278 .76973 .56463 .77599 .56266 .77335 .56077 Metetra
.647 .858 .98 r
.77335 .56077 .77599 .56266 .78226 .56063 .77963 .55868 Metetra
.653 .857 .978 r
.77963 .55868 .78226 .56063 .78854 .55854 .78592 .55655 Metetra
.659 .855 .975 r
.78592 .55655 .78854 .55854 .79483 .5564 .79221 .55435 Metetra
.664 .853 .972 r
.79221 .55435 .79483 .5564 .80113 .55422 .79852 .55212 Metetra
.669 .851 .97 r
.79852 .55212 .80113 .55422 .80743 .552 .80483 .54984 Metetra
.673 .848 .967 r
.80483 .54984 .80743 .552 .81375 .54975 .81116 .54753 Metetra
.676 .845 .964 r
.81116 .54753 .81375 .54975 .82008 .54746 .81749 .54518 Metetra
.679 .842 .961 r
.81749 .54518 .82008 .54746 .82641 .54515 .82384 .54281 Metetra
.681 .838 .959 r
.82384 .54281 .82641 .54515 .83276 .54282 .83019 .54041 Metetra
.684 .835 .956 r
.83019 .54041 .83276 .54282 .83912 .54046 .83656 .53799 Metetra
.686 .831 .953 r
.83656 .53799 .83912 .54046 .84549 .53808 .84293 .53555 Metetra
.688 .828 .95 r
.84293 .53555 .84549 .53808 .85186 .53568 .84932 .53309 Metetra
.689 .824 .948 r
.84932 .53309 .85186 .53568 .85825 .53327 .85571 .53062 Metetra
.69 .821 .945 r
.85571 .53062 .85825 .53327 .86465 .53084 .86212 .52813 Metetra
.691 .817 .942 r
.86212 .52813 .86465 .53084 .87107 .5284 .86854 .52563 Metetra
.692 .813 .94 r
.86854 .52563 .87107 .5284 .87749 .52594 .87497 .52311 Metetra
.693 .81 .937 r
.87497 .52311 .87749 .52594 .88392 .52348 .88141 .52058 Metetra
.693 .806 .935 r
.88141 .52058 .88392 .52348 .89037 .52101 .88786 .51805 Metetra
.694 .802 .932 r
.88786 .51805 .89037 .52101 .89683 .51853 .89433 .5155 Metetra
.694 .799 .93 r
.89433 .5155 .89683 .51853 .9033 .51604 .9008 .51295 Metetra
.695 .795 .927 r
.9008 .51295 .9033 .51604 .90978 .51354 .90729 .51039 Metetra
.695 .791 .925 r
.90729 .51039 .90978 .51354 .91627 .51103 .91379 .50783 Metetra
.695 .788 .922 r
.91379 .50783 .91627 .51103 .92278 .50852 .9203 .50526 Metetra
.695 .784 .92 r
.9203 .50526 .92278 .50852 .92929 .50601 .92682 .50268 Metetra
.695 .781 .917 r
.92682 .50268 .92929 .50601 .93582 .50349 .93335 .5001 Metetra
.695 .777 .915 r
.93335 .5001 .93582 .50349 .94236 .50096 .9399 .49751 Metetra
.695 .774 .912 r
.9399 .49751 .94236 .50096 .94892 .49843 .94646 .49492 Metetra
.695 .771 .91 r
.94646 .49492 .94892 .49843 .95548 .49589 .95303 .49232 Metetra
.7 .761 .899 r
.35621 .66216 .35954 .66506 .36497 .66286 .36165 .6599 Metetra
.7 .758 .897 r
.36165 .6599 .36497 .66286 .3704 .66066 .3671 .65764 Metetra
.7 .754 .894 r
.3671 .65764 .3704 .66066 .37585 .65845 .37255 .65538 Metetra
.699 .751 .892 r
.37255 .65538 .37585 .65845 .38131 .65625 .37802 .65312 Metetra
.699 .748 .89 r
.37802 .65312 .38131 .65625 .38677 .65403 .38349 .65085 Metetra
.699 .745 .888 r
.38349 .65085 .38677 .65403 .39225 .65182 .38897 .64857 Metetra
.698 .742 .885 r
.38897 .64857 .39225 .65182 .39773 .6496 .39447 .6463 Metetra
.698 .739 .883 r
.39447 .6463 .39773 .6496 .40322 .64739 .39997 .64402 Metetra
.697 .736 .881 r
.39997 .64402 .40322 .64739 .40872 .64517 .40548 .64175 Metetra
.696 .733 .879 r
.40548 .64175 .40872 .64517 .41423 .64295 .411 .63947 Metetra
.696 .73 .877 r
.411 .63947 .41423 .64295 .41975 .64072 .41653 .63719 Metetra
.695 .727 .875 r
.41653 .63719 .41975 .64072 .42528 .63851 .42206 .63491 Metetra
.694 .724 .873 r
.42206 .63491 .42528 .63851 .43082 .63629 .42761 .63264 Metetra
.694 .721 .871 r
.42761 .63264 .43082 .63629 .43637 .63407 .43317 .63036 Metetra
.693 .718 .869 r
.43317 .63036 .43637 .63407 .44193 .63186 .43873 .62809 Metetra
.692 .715 .867 r
.43873 .62809 .44193 .63186 .44749 .62966 .44431 .62583 Metetra
.691 .712 .865 r
.44431 .62583 .44749 .62966 .45307 .62746 .44989 .62357 Metetra
.69 .709 .863 r
.44989 .62357 .45307 .62746 .45865 .62527 .45548 .62132 Metetra
.689 .706 .861 r
.45548 .62132 .45865 .62527 .46425 .6231 .46108 .61908 Metetra
.687 .703 .859 r
.46108 .61908 .46425 .6231 .46985 .62093 .46669 .61686 Metetra
.686 .7 .858 r
.46669 .61686 .46985 .62093 .47546 .61879 .47231 .61465 Metetra
.684 .697 .856 r
.47231 .61465 .47546 .61879 .48109 .61666 .47794 .61246 Metetra
.682 .694 .854 r
.47794 .61246 .48109 .61666 .48672 .61455 .48358 .61029 Metetra
.681 .691 .853 r
.48358 .61029 .48672 .61455 .49236 .61247 .48923 .60815 Metetra
.678 .688 .852 r
.48923 .60815 .49236 .61247 .49801 .61042 .49489 .60605 Metetra
.676 .685 .851 r
.49489 .60605 .49801 .61042 .50368 .60841 .50056 .60397 Metetra
.673 .681 .85 r
.50056 .60397 .50368 .60841 .50935 .60643 .50624 .60195 Metetra
.67 .678 .849 r
.50624 .60195 .50935 .60643 .51503 .60451 .51193 .59997 Metetra
.667 .675 .848 r
.51193 .59997 .51503 .60451 .52073 .60264 .51763 .59805 Metetra
.664 .672 .848 r
.51763 .59805 .52073 .60264 .52643 .60083 .52334 .5962 Metetra
.66 .669 .848 r
.52334 .5962 .52643 .60083 .53215 .59909 .52906 .59441 Metetra
.655 .666 .848 r
.52906 .59441 .53215 .59909 .53788 .59742 .53479 .59271 Metetra
.651 .663 .849 r
.53479 .59271 .53788 .59742 .54362 .59583 .54054 .59109 Metetra
.645 .66 .85 r
.54054 .59109 .54362 .59583 .54938 .59432 .5463 .58958 Metetra
.64 .658 .851 r
.5463 .58958 .54938 .59432 .55514 .59291 .55207 .58816 Metetra
.634 .655 .853 r
.55207 .58816 .55514 .59291 .56093 .5916 .55786 .58685 Metetra
.627 .653 .856 r
.55786 .58685 .56093 .5916 .56672 .59039 .56366 .58567 Metetra
.62 .651 .859 r
.56366 .58567 .56672 .59039 .57254 .58929 .56948 .5846 Metetra
.612 .65 .863 r
.56948 .5846 .57254 .58929 .57836 .5883 .57531 .58366 Metetra
.604 .649 .868 r
.57531 .58366 .57836 .5883 .58421 .58741 .58117 .58285 Metetra
.596 .649 .873 r
.58117 .58285 .58421 .58741 .59007 .58663 .58703 .58216 Metetra
.587 .65 .879 r
.58703 .58216 .59007 .58663 .59595 .58596 .59292 .58158 Metetra
.578 .651 .886 r
.59292 .58158 .59595 .58596 .60185 .58538 .59883 .58113 Metetra
.569 .654 .893 r
.59883 .58113 .60185 .58538 .60776 .58489 .60475 .58077 Metetra
.56 .657 .901 r
.60475 .58077 .60776 .58489 .6137 .58448 .6107 .5805 Metetra
.55 .661 .91 r
.6107 .5805 .6137 .58448 .61965 .58413 .61666 .58031 Metetra
.54 .667 .919 r
.61666 .58031 .61965 .58413 .62563 .58384 .62265 .58018 Metetra
.531 .673 .928 r
.62265 .58018 .62563 .58384 .63162 .58358 .62865 .5801 Metetra
.522 .681 .938 r
.62865 .5801 .63162 .58358 .63763 .58335 .63468 .58004 Metetra
.513 .689 .947 r
.63468 .58004 .63763 .58335 .64366 .58313 .64072 .58 Metetra
.506 .698 .956 r
.64072 .58 .64366 .58313 .64971 .5829 .64678 .57995 Metetra
.499 .709 .965 r
.64678 .57995 .64971 .5829 .65577 .58264 .65286 .57986 Metetra
.494 .72 .972 r
.65286 .57986 .65577 .58264 .66186 .58234 .65896 .57973 Metetra
.49 .732 .979 r
.65896 .57973 .66186 .58234 .66795 .58199 .66507 .57954 Metetra
.488 .744 .985 r
.66507 .57954 .66795 .58199 .67407 .58157 .6712 .57926 Metetra
.488 .757 .99 r
.6712 .57926 .67407 .58157 .68019 .58106 .67734 .5789 Metetra
.491 .77 .993 r
.67734 .5789 .68019 .58106 .68633 .58046 .6835 .57842 Metetra
.495 .782 .996 r
.6835 .57842 .68633 .58046 .69249 .57976 .68967 .57783 Metetra
.502 .794 .998 r
.68967 .57783 .69249 .57976 .69865 .57895 .69585 .57711 Metetra
.511 .806 .999 r
.69585 .57711 .69865 .57895 .70483 .57803 .70204 .57627 Metetra
.521 .817 .999 r
.70204 .57627 .70483 .57803 .71101 .57699 .70824 .57528 Metetra
.533 .827 .999 r
.70824 .57528 .71101 .57699 .71721 .57584 .71445 .57417 Metetra
.546 .836 .998 r
.71445 .57417 .71721 .57584 .72341 .57456 .72067 .57292 Metetra
.56 .844 .997 r
.72067 .57292 .72341 .57456 .72963 .57318 .72689 .57154 Metetra
.573 .85 .996 r
.72689 .57154 .72963 .57318 .73585 .57168 .73313 .57005 Metetra
.586 .856 .995 r
.73313 .57005 .73585 .57168 .74208 .57008 .73937 .56843 Metetra
.599 .859 .993 r
.73937 .56843 .74208 .57008 .74832 .56839 .74562 .56671 Metetra
.61 .862 .991 r
.74562 .56671 .74832 .56839 .75456 .5666 .75188 .5649 Metetra
.621 .864 .988 r
.75188 .5649 .75456 .5666 .76082 .56473 .75814 .56299 Metetra
.631 .865 .986 r
.75814 .56299 .76082 .56473 .76708 .56278 .76441 .56099 Metetra
.64 .864 .983 r
.76441 .56099 .76708 .56278 .77335 .56077 .77069 .55893 Metetra
.648 .863 .981 r
.77069 .55893 .77335 .56077 .77963 .55868 .77698 .55679 Metetra
.656 .862 .978 r
.77698 .55679 .77963 .55868 .78592 .55655 .78328 .55459 Metetra
.662 .86 .975 r
.78328 .55459 .78592 .55655 .79221 .55435 .78958 .55234 Metetra
.667 .858 .972 r
.78958 .55234 .79221 .55435 .79852 .55212 .7959 .55004 Metetra
.672 .855 .969 r
.7959 .55004 .79852 .55212 .80483 .54984 .80222 .54771 Metetra
.676 .852 .966 r
.80222 .54771 .80483 .54984 .81116 .54753 .80855 .54533 Metetra
.68 .848 .964 r
.80855 .54533 .81116 .54753 .81749 .54518 .8149 .54292 Metetra
.683 .845 .961 r
.8149 .54292 .81749 .54518 .82384 .54281 .82125 .54048 Metetra
.685 .841 .958 r
.82125 .54048 .82384 .54281 .83019 .54041 .82761 .53802 Metetra
.688 .837 .955 r
.82761 .53802 .83019 .54041 .83656 .53799 .83398 .53553 Metetra
.69 .834 .952 r
.83398 .53553 .83656 .53799 .84293 .53555 .84037 .53303 Metetra
.691 .83 .95 r
.84037 .53303 .84293 .53555 .84932 .53309 .84676 .53051 Metetra
.693 .826 .947 r
.84676 .53051 .84932 .53309 .85571 .53062 .85316 .52796 Metetra
.694 .822 .944 r
.85316 .52796 .85571 .53062 .86212 .52813 .85958 .52541 Metetra
.695 .819 .941 r
.85958 .52541 .86212 .52813 .86854 .52563 .866 .52284 Metetra
.696 .815 .939 r
.866 .52284 .86854 .52563 .87497 .52311 .87244 .52027 Metetra
.696 .811 .936 r
.87244 .52027 .87497 .52311 .88141 .52058 .87889 .51768 Metetra
.697 .807 .934 r
.87889 .51768 .88141 .52058 .88786 .51805 .88534 .51508 Metetra
.697 .804 .931 r
.88534 .51508 .88786 .51805 .89433 .5155 .89181 .51247 Metetra
.698 .8 .928 r
.89181 .51247 .89433 .5155 .9008 .51295 .89829 .50986 Metetra
.698 .796 .926 r
.89829 .50986 .9008 .51295 .90729 .51039 .90479 .50724 Metetra
.698 .793 .923 r
.90479 .50724 .90729 .51039 .91379 .50783 .91129 .50461 Metetra
.698 .789 .921 r
.91129 .50461 .91379 .50783 .9203 .50526 .91781 .50198 Metetra
.698 .786 .919 r
.91781 .50198 .9203 .50526 .92682 .50268 .92433 .49934 Metetra
.698 .782 .916 r
.92433 .49934 .92682 .50268 .93335 .5001 .93087 .49669 Metetra
.698 .779 .914 r
.93087 .49669 .93335 .5001 .9399 .49751 .93742 .49404 Metetra
.698 .775 .911 r
.93742 .49404 .9399 .49751 .94646 .49492 .94399 .49139 Metetra
.698 .772 .909 r
.94399 .49139 .94646 .49492 .95303 .49232 .95056 .48873 Metetra
.703 .761 .897 r
.35287 .65924 .35621 .66216 .36165 .6599 .35832 .65693 Metetra
.703 .758 .895 r
.35832 .65693 .36165 .6599 .3671 .65764 .36377 .65461 Metetra
.702 .755 .893 r
.36377 .65461 .3671 .65764 .37255 .65538 .36924 .65229 Metetra
.702 .752 .89 r
.36924 .65229 .37255 .65538 .37802 .65312 .37472 .64997 Metetra
.702 .749 .888 r
.37472 .64997 .37802 .65312 .38349 .65085 .3802 .64765 Metetra
.701 .746 .886 r
.3802 .64765 .38349 .65085 .38897 .64857 .38569 .64532 Metetra
.701 .743 .884 r
.38569 .64532 .38897 .64857 .39447 .6463 .39119 .64299 Metetra
.7 .74 .882 r
.39119 .64299 .39447 .6463 .39997 .64402 .3967 .64065 Metetra
.7 .737 .88 r
.3967 .64065 .39997 .64402 .40548 .64175 .40222 .63831 Metetra
.699 .733 .877 r
.40222 .63831 .40548 .64175 .411 .63947 .40775 .63598 Metetra
.699 .731 .875 r
.40775 .63598 .411 .63947 .41653 .63719 .41329 .63364 Metetra
.698 .727 .873 r
.41329 .63364 .41653 .63719 .42206 .63491 .41883 .6313 Metetra
.697 .725 .871 r
.41883 .6313 .42206 .63491 .42761 .63264 .42439 .62896 Metetra
.696 .721 .869 r
.42439 .62896 .42761 .63264 .43317 .63036 .42995 .62663 Metetra
.695 .718 .867 r
.42995 .62663 .43317 .63036 .43873 .62809 .43553 .6243 Metetra
.695 .715 .865 r
.43553 .6243 .43873 .62809 .44431 .62583 .44111 .62197 Metetra
.693 .712 .863 r
.44111 .62197 .44431 .62583 .44989 .62357 .4467 .61965 Metetra
.692 .709 .861 r
.4467 .61965 .44989 .62357 .45548 .62132 .4523 .61734 Metetra
.691 .706 .859 r
.4523 .61734 .45548 .62132 .46108 .61908 .45791 .61504 Metetra
.69 .703 .857 r
.45791 .61504 .46108 .61908 .46669 .61686 .46353 .61275 Metetra
.688 .7 .856 r
.46353 .61275 .46669 .61686 .47231 .61465 .46915 .61047 Metetra
.687 .697 .854 r
.46915 .61047 .47231 .61465 .47794 .61246 .47479 .60822 Metetra
.685 .694 .852 r
.47479 .60822 .47794 .61246 .48358 .61029 .48044 .60599 Metetra
.683 .69 .851 r
.48044 .60599 .48358 .61029 .48923 .60815 .48609 .60379 Metetra
.681 .687 .849 r
.48609 .60379 .48923 .60815 .49489 .60605 .49176 .60161 Metetra
.678 .684 .848 r
.49176 .60161 .49489 .60605 .50056 .60397 .49743 .59948 Metetra
.676 .68 .847 r
.49743 .59948 .50056 .60397 .50624 .60195 .50312 .59739 Metetra
.672 .677 .846 r
.50312 .59739 .50624 .60195 .51193 .59997 .50881 .59536 Metetra
.669 .673 .845 r
.50881 .59536 .51193 .59997 .51763 .59805 .51452 .59338 Metetra
.665 .67 .845 r
.51452 .59338 .51763 .59805 .52334 .5962 .52023 .59147 Metetra
.661 .666 .844 r
.52023 .59147 .52334 .5962 .52906 .59441 .52596 .58964 Metetra
.656 .663 .844 r
.52596 .58964 .52906 .59441 .53479 .59271 .5317 .5879 Metetra
.651 .659 .845 r
.5317 .5879 .53479 .59271 .54054 .59109 .53745 .58625 Metetra
.646 .656 .846 r
.53745 .58625 .54054 .59109 .5463 .58958 .54322 .58471 Metetra
.64 .653 .847 r
.54322 .58471 .5463 .58958 .55207 .58816 .54899 .58328 Metetra
.633 .65 .849 r
.54899 .58328 .55207 .58816 .55786 .58685 .55479 .58197 Metetra
.626 .647 .852 r
.55479 .58197 .55786 .58685 .56366 .58567 .56059 .58081 Metetra
.618 .645 .855 r
.56059 .58081 .56366 .58567 .56948 .5846 .56642 .57978 Metetra
.609 .643 .859 r
.56642 .57978 .56948 .5846 .57531 .58366 .57226 .57889 Metetra
.6 .642 .864 r
.57226 .57889 .57531 .58366 .58117 .58285 .57811 .57815 Metetra
.591 .641 .869 r
.57811 .57815 .58117 .58285 .58703 .58216 .58399 .57755 Metetra
.58 .641 .876 r
.58399 .57755 .58703 .58216 .59292 .58158 .58988 .57709 Metetra
.57 .642 .883 r
.58988 .57709 .59292 .58158 .59883 .58113 .59579 .57675 Metetra
.559 .644 .891 r
.59579 .57675 .59883 .58113 .60475 .58077 .60173 .57654 Metetra
.548 .647 .899 r
.60173 .57654 .60475 .58077 .6107 .5805 .60768 .57643 Metetra
.537 .651 .909 r
.60768 .57643 .6107 .5805 .61666 .58031 .61366 .57641 Metetra
.525 .656 .918 r
.61366 .57641 .61666 .58031 .62265 .58018 .61965 .57647 Metetra
.514 .663 .928 r
.61965 .57647 .62265 .58018 .62865 .5801 .62567 .57657 Metetra
.503 .67 .938 r
.62567 .57657 .62865 .5801 .63468 .58004 .63171 .57671 Metetra
.492 .679 .948 r
.63171 .57671 .63468 .58004 .64072 .58 .63776 .57687 Metetra
.483 .689 .958 r
.63776 .57687 .64072 .58 .64678 .57995 .64384 .57701 Metetra
.474 .699 .967 r
.64384 .57701 .64678 .57995 .65286 .57986 .64994 .57712 Metetra
.468 .712 .975 r
.64994 .57712 .65286 .57986 .65896 .57973 .65605 .57717 Metetra
.463 .724 .982 r
.65605 .57717 .65896 .57973 .66507 .57954 .66218 .57715 Metetra
.46 .738 .988 r
.66218 .57715 .66507 .57954 .6712 .57926 .66832 .57704 Metetra
.459 .751 .992 r
.66832 .57704 .6712 .57926 .67734 .5789 .67448 .57683 Metetra
.462 .765 .995 r
.67448 .57683 .67734 .5789 .6835 .57842 .68066 .57649 Metetra
.467 .779 .998 r
.68066 .57649 .6835 .57842 .68967 .57783 .68684 .57601 Metetra
.476 .793 .999 r
.68684 .57601 .68967 .57783 .69585 .57711 .69304 .57539 Metetra
.487 .806 1 r
.69304 .57539 .69585 .57711 .70204 .57627 .69925 .57462 Metetra
.5 .818 1 r
.69925 .57462 .70204 .57627 .70824 .57528 .70546 .5737 Metetra
.515 .83 1 r
.70546 .5737 .70824 .57528 .71445 .57417 .71169 .57262 Metetra
.531 .84 .999 r
.71169 .57262 .71445 .57417 .72067 .57292 .71792 .57139 Metetra
.548 .849 .998 r
.71792 .57139 .72067 .57292 .72689 .57154 .72416 .57002 Metetra
.564 .856 .997 r
.72416 .57002 .72689 .57154 .73313 .57005 .7304 .56852 Metetra
.579 .862 .995 r
.7304 .56852 .73313 .57005 .73937 .56843 .73666 .56688 Metetra
.594 .866 .994 r
.73666 .56688 .73937 .56843 .74562 .56671 .74292 .56513 Metetra
.608 .869 .992 r
.74292 .56513 .74562 .56671 .75188 .5649 .74918 .56327 Metetra
.621 .871 .989 r
.74918 .56327 .75188 .5649 .75814 .56299 .75546 .56131 Metetra
.632 .871 .987 r
.75546 .56131 .75814 .56299 .76441 .56099 .76174 .55926 Metetra
.642 .871 .984 r
.76174 .55926 .76441 .56099 .77069 .55893 .76803 .55714 Metetra
.651 .869 .981 r
.76803 .55714 .77069 .55893 .77698 .55679 .77433 .55494 Metetra
.659 .867 .978 r
.77433 .55494 .77698 .55679 .78328 .55459 .78063 .55268 Metetra
.665 .865 .975 r
.78063 .55268 .78328 .55459 .78958 .55234 .78695 .55036 Metetra
.671 .862 .972 r
.78695 .55036 .78958 .55234 .7959 .55004 .79327 .548 Metetra
.676 .859 .969 r
.79327 .548 .7959 .55004 .80222 .54771 .7996 .54559 Metetra
.68 .855 .966 r
.7996 .54559 .80222 .54771 .80855 .54533 .80594 .54315 Metetra
.684 .852 .963 r
.80594 .54315 .80855 .54533 .8149 .54292 .81229 .54067 Metetra
.687 .848 .96 r
.81229 .54067 .8149 .54292 .82125 .54048 .81865 .53816 Metetra
.689 .844 .957 r
.81865 .53816 .82125 .54048 .82761 .53802 .82502 .53563 Metetra
.692 .84 .954 r
.82502 .53563 .82761 .53802 .83398 .53553 .8314 .53308 Metetra
.694 .836 .951 r
.8314 .53308 .83398 .53553 .84037 .53303 .83779 .53051 Metetra
.695 .832 .949 r
.83779 .53051 .84037 .53303 .84676 .53051 .84419 .52792 Metetra
.697 .828 .946 r
.84419 .52792 .84676 .53051 .85316 .52796 .8506 .52531 Metetra
.698 .824 .943 r
.8506 .52531 .85316 .52796 .85958 .52541 .85702 .52269 Metetra
.699 .82 .94 r
.85702 .52269 .85958 .52541 .866 .52284 .86345 .52006 Metetra
.699 .816 .938 r
.86345 .52006 .866 .52284 .87244 .52027 .8699 .51742 Metetra
.7 .813 .935 r
.8699 .51742 .87244 .52027 .87889 .51768 .87635 .51476 Metetra
.7 .809 .932 r
.87635 .51476 .87889 .51768 .88534 .51508 .88281 .5121 Metetra
.701 .805 .93 r
.88281 .5121 .88534 .51508 .89181 .51247 .88929 .50943 Metetra
.701 .801 .927 r
.88929 .50943 .89181 .51247 .89829 .50986 .89578 .50676 Metetra
.701 .798 .925 r
.89578 .50676 .89829 .50986 .90479 .50724 .90228 .50407 Metetra
.701 .794 .922 r
.90228 .50407 .90479 .50724 .91129 .50461 .90879 .50138 Metetra
.701 .79 .92 r
.90879 .50138 .91129 .50461 .91781 .50198 .91531 .49869 Metetra
.701 .787 .917 r
.91531 .49869 .91781 .50198 .92433 .49934 .92184 .49598 Metetra
.701 .783 .915 r
.92184 .49598 .92433 .49934 .93087 .49669 .92838 .49328 Metetra
.701 .78 .913 r
.92838 .49328 .93087 .49669 .93742 .49404 .93494 .49057 Metetra
.701 .776 .91 r
.93494 .49057 .93742 .49404 .94399 .49139 .94151 .48785 Metetra
.701 .773 .908 r
.94151 .48785 .94399 .49139 .95056 .48873 .94809 .48513 Metetra
.706 .762 .896 r
.34952 .65631 .35287 .65924 .35832 .65693 .35497 .65395 Metetra
.706 .759 .893 r
.35497 .65395 .35832 .65693 .36377 .65461 .36044 .65157 Metetra
.705 .755 .891 r
.36044 .65157 .36377 .65461 .36924 .65229 .36592 .6492 Metetra
.705 .752 .889 r
.36592 .6492 .36924 .65229 .37472 .64997 .3714 .64682 Metetra
.704 .749 .887 r
.3714 .64682 .37472 .64997 .3802 .64765 .3769 .64444 Metetra
.704 .746 .885 r
.3769 .64444 .3802 .64765 .38569 .64532 .3824 .64205 Metetra
.703 .743 .882 r
.3824 .64205 .38569 .64532 .39119 .64299 .38791 .63966 Metetra
.703 .74 .88 r
.38791 .63966 .39119 .64299 .3967 .64065 .39343 .63726 Metetra
.702 .737 .878 r
.39343 .63726 .3967 .64065 .40222 .63831 .39896 .63487 Metetra
.702 .734 .876 r
.39896 .63487 .40222 .63831 .40775 .63598 .40449 .63247 Metetra
.701 .731 .874 r
.40449 .63247 .40775 .63598 .41329 .63364 .41004 .63007 Metetra
.7 .728 .872 r
.41004 .63007 .41329 .63364 .41883 .6313 .41559 .62767 Metetra
.7 .725 .869 r
.41559 .62767 .41883 .6313 .42439 .62896 .42116 .62527 Metetra
.699 .722 .867 r
.42116 .62527 .42439 .62896 .42995 .62663 .42673 .62288 Metetra
.698 .719 .865 r
.42673 .62288 .42995 .62663 .43553 .6243 .43231 .62048 Metetra
.697 .716 .863 r
.43231 .62048 .43553 .6243 .44111 .62197 .4379 .61809 Metetra
.696 .713 .861 r
.4379 .61809 .44111 .62197 .4467 .61965 .4435 .61571 Metetra
.695 .71 .859 r
.4435 .61571 .4467 .61965 .4523 .61734 .44911 .61333 Metetra
.694 .706 .857 r
.44911 .61333 .4523 .61734 .45791 .61504 .45472 .61096 Metetra
.692 .703 .855 r
.45472 .61096 .45791 .61504 .46353 .61275 .46035 .60861 Metetra
.691 .7 .853 r
.46035 .60861 .46353 .61275 .46915 .61047 .46598 .60627 Metetra
.689 .697 .852 r
.46598 .60627 .46915 .61047 .47479 .60822 .47163 .60394 Metetra
.688 .693 .85 r
.47163 .60394 .47479 .60822 .48044 .60599 .47728 .60164 Metetra
.686 .69 .848 r
.47728 .60164 .48044 .60599 .48609 .60379 .48294 .59937 Metetra
.683 .686 .847 r
.48294 .59937 .48609 .60379 .49176 .60161 .48862 .59713 Metetra
.681 .683 .845 r
.48862 .59713 .49176 .60161 .49743 .59948 .4943 .59493 Metetra
.678 .679 .844 r
.4943 .59493 .49743 .59948 .50312 .59739 .49999 .59277 Metetra
.675 .675 .843 r
.49999 .59277 .50312 .59739 .50881 .59536 .50569 .59067 Metetra
.671 .671 .842 r
.50569 .59067 .50881 .59536 .51452 .59338 .5114 .58862 Metetra
.667 .668 .841 r
.5114 .58862 .51452 .59338 .52023 .59147 .51712 .58665 Metetra
.663 .664 .841 r
.51712 .58665 .52023 .59147 .52596 .58964 .52285 .58477 Metetra
.658 .66 .84 r
.52285 .58477 .52596 .58964 .5317 .5879 .5286 .58297 Metetra
.653 .656 .841 r
.5286 .58297 .5317 .5879 .53745 .58625 .53435 .58128 Metetra
.647 .652 .841 r
.53435 .58128 .53745 .58625 .54322 .58471 .54012 .5797 Metetra
.64 .648 .842 r
.54012 .5797 .54322 .58471 .54899 .58328 .54591 .57826 Metetra
.633 .645 .844 r
.54591 .57826 .54899 .58328 .55479 .58197 .5517 .57695 Metetra
.625 .641 .847 r
.5517 .57695 .55479 .58197 .56059 .58081 .55751 .57579 Metetra
.616 .638 .85 r
.55751 .57579 .56059 .58081 .56642 .57978 .56334 .57479 Metetra
.606 .635 .854 r
.56334 .57479 .56642 .57978 .57226 .57889 .56918 .57396 Metetra
.596 .633 .859 r
.56918 .57396 .57226 .57889 .57811 .57815 .57505 .57329 Metetra
.585 .632 .864 r
.57505 .57329 .57811 .57815 .58399 .57755 .58093 .57278 Metetra
.574 .631 .871 r
.58093 .57278 .58399 .57755 .58988 .57709 .58683 .57244 Metetra
.561 .632 .879 r
.58683 .57244 .58988 .57709 .59579 .57675 .59275 .57225 Metetra
.549 .633 .887 r
.59275 .57225 .59579 .57675 .60173 .57654 .59869 .57219 Metetra
.536 .636 .896 r
.59869 .57219 .60173 .57654 .60768 .57643 .60466 .57226 Metetra
.522 .639 .906 r
.60466 .57226 .60768 .57643 .61366 .57641 .61064 .57243 Metetra
.509 .645 .917 r
.61064 .57243 .61366 .57641 .61965 .57647 .61665 .57269 Metetra
.495 .65 .928 r
.61665 .57269 .61965 .57647 .62567 .57657 .62268 .57301 Metetra
.481 .658 .939 r
.62268 .57301 .62567 .57657 .63171 .57671 .62873 .57336 Metetra
.468 .667 .949 r
.62873 .57336 .63171 .57671 .63776 .57687 .6348 .57373 Metetra
.456 .677 .959 r
.6348 .57373 .63776 .57687 .64384 .57701 .64089 .57409 Metetra
.445 .688 .969 r
.64089 .57409 .64384 .57701 .64994 .57712 .647 .57441 Metetra
.436 .7 .977 r
.647 .57441 .64994 .57712 .65605 .57717 .65313 .57467 Metetra
.429 .714 .984 r
.65313 .57467 .65605 .57717 .66218 .57715 .65928 .57485 Metetra
.425 .728 .989 r
.65928 .57485 .66218 .57715 .66832 .57704 .66544 .57492 Metetra
.424 .742 .993 r
.66544 .57492 .66832 .57704 .67448 .57683 .67162 .57487 Metetra
.426 .758 .995 r
.67162 .57487 .67448 .57683 .68066 .57649 .67781 .57468 Metetra
.433 .773 .997 r
.67781 .57468 .68066 .57649 .68684 .57601 .68401 .57433 Metetra
.444 .789 .998 r
.68401 .57433 .68684 .57601 .69304 .57539 .69022 .57381 Metetra
.457 .804 .998 r
.69022 .57381 .69304 .57539 .69925 .57462 .69644 .57312 Metetra
.474 .819 .998 r
.69644 .57312 .69925 .57462 .70546 .5737 .70268 .57226 Metetra
.493 .832 .998 r
.70268 .57226 .70546 .5737 .71169 .57262 .70891 .57121 Metetra
.513 .844 .998 r
.70891 .57121 .71169 .57262 .71792 .57139 .71516 .57 Metetra
.533 .854 .997 r
.71516 .57 .71792 .57139 .72416 .57002 .72141 .56863 Metetra
.554 .862 .996 r
.72141 .56863 .72416 .57002 .7304 .56852 .72767 .5671 Metetra
.572 .869 .995 r
.72767 .5671 .7304 .56852 .73666 .56688 .73394 .56543 Metetra
.59 .873 .994 r
.73394 .56543 .73666 .56688 .74292 .56513 .74021 .56364 Metetra
.606 .876 .992 r
.74021 .56364 .74292 .56513 .74918 .56327 .74648 .56173 Metetra
.62 .877 .989 r
.74648 .56173 .74918 .56327 .75546 .56131 .75277 .55971 Metetra
.633 .878 .987 r
.75277 .55971 .75546 .56131 .76174 .55926 .75906 .5576 Metetra
.644 .877 .984 r
.75906 .5576 .76174 .55926 .76803 .55714 .76536 .5554 Metetra
.654 .875 .981 r
.76536 .5554 .76803 .55714 .77433 .55494 .77167 .55314 Metetra
.662 .873 .978 r
.77167 .55314 .77433 .55494 .78063 .55268 .77798 .5508 Metetra
.669 .87 .975 r
.77798 .5508 .78063 .55268 .78695 .55036 .7843 .54842 Metetra
.675 .867 .972 r
.7843 .54842 .78695 .55036 .79327 .548 .79063 .54598 Metetra
.68 .863 .969 r
.79063 .54598 .79327 .548 .7996 .54559 .79697 .5435 Metetra
.685 .859 .965 r
.79697 .5435 .7996 .54559 .80594 .54315 .80332 .54098 Metetra
.688 .855 .962 r
.80332 .54098 .80594 .54315 .81229 .54067 .80968 .53843 Metetra
.691 .851 .959 r
.80968 .53843 .81229 .54067 .81865 .53816 .81604 .53585 Metetra
.694 .847 .956 r
.81604 .53585 .81865 .53816 .82502 .53563 .82242 .53325 Metetra
.696 .843 .953 r
.82242 .53325 .82502 .53563 .8314 .53308 .82881 .53063 Metetra
.698 .839 .95 r
.82881 .53063 .8314 .53308 .83779 .53051 .8352 .52799 Metetra
.699 .834 .947 r
.8352 .52799 .83779 .53051 .84419 .52792 .84161 .52533 Metetra
.7 .83 .945 r
.84161 .52533 .84419 .52792 .8506 .52531 .84803 .52265 Metetra
.701 .826 .942 r
.84803 .52265 .8506 .52531 .85702 .52269 .85446 .51997 Metetra
.702 .822 .939 r
.85446 .51997 .85702 .52269 .86345 .52006 .86089 .51727 Metetra
.703 .818 .936 r
.86089 .51727 .86345 .52006 .8699 .51742 .86734 .51456 Metetra
.703 .814 .934 r
.86734 .51456 .8699 .51742 .87635 .51476 .8738 .51184 Metetra
.704 .81 .931 r
.8738 .51184 .87635 .51476 .88281 .5121 .88027 .50911 Metetra
.704 .806 .929 r
.88027 .50911 .88281 .5121 .88929 .50943 .88676 .50638 Metetra
.704 .803 .926 r
.88676 .50638 .88929 .50943 .89578 .50676 .89325 .50364 Metetra
.704 .799 .924 r
.89325 .50364 .89578 .50676 .90228 .50407 .89975 .50089 Metetra
.704 .795 .921 r
.89975 .50089 .90228 .50407 .90879 .50138 .90627 .49814 Metetra
.704 .791 .919 r
.90627 .49814 .90879 .50138 .91531 .49869 .9128 .49538 Metetra
.704 .788 .916 r
.9128 .49538 .91531 .49869 .92184 .49598 .91933 .49261 Metetra
.704 .784 .914 r
.91933 .49261 .92184 .49598 .92838 .49328 .92588 .48985 Metetra
.704 .781 .911 r
.92588 .48985 .92838 .49328 .93494 .49057 .93244 .48707 Metetra
.704 .777 .909 r
.93244 .48707 .93494 .49057 .94151 .48785 .93902 .48429 Metetra
.703 .774 .907 r
.93902 .48429 .94151 .48785 .94809 .48513 .9456 .48151 Metetra
.709 .762 .894 r
.34615 .65338 .34952 .65631 .35497 .65395 .35162 .65095 Metetra
.708 .759 .892 r
.35162 .65095 .35497 .65395 .36044 .65157 .3571 .64852 Metetra
.708 .756 .89 r
.3571 .64852 .36044 .65157 .36592 .6492 .36258 .64609 Metetra
.707 .753 .887 r
.36258 .64609 .36592 .6492 .3714 .64682 .36808 .64365 Metetra
.707 .75 .885 r
.36808 .64365 .3714 .64682 .3769 .64444 .37358 .64121 Metetra
.707 .747 .883 r
.37358 .64121 .3769 .64444 .3824 .64205 .37909 .63876 Metetra
.706 .744 .881 r
.37909 .63876 .3824 .64205 .38791 .63966 .38461 .63631 Metetra
.706 .741 .879 r
.38461 .63631 .38791 .63966 .39343 .63726 .39014 .63386 Metetra
.705 .738 .876 r
.39014 .63386 .39343 .63726 .39896 .63487 .39568 .6314 Metetra
.704 .734 .874 r
.39568 .6314 .39896 .63487 .40449 .63247 .40122 .62895 Metetra
.704 .732 .872 r
.40122 .62895 .40449 .63247 .41004 .63007 .40678 .62649 Metetra
.703 .728 .87 r
.40678 .62649 .41004 .63007 .41559 .62767 .41234 .62403 Metetra
.702 .725 .868 r
.41234 .62403 .41559 .62767 .42116 .62527 .41791 .62157 Metetra
.702 .722 .866 r
.41791 .62157 .42116 .62527 .42673 .62288 .42349 .61911 Metetra
.701 .719 .864 r
.42349 .61911 .42673 .62288 .43231 .62048 .42908 .61665 Metetra
.7 .716 .862 r
.42908 .61665 .43231 .62048 .4379 .61809 .43468 .61419 Metetra
.699 .713 .86 r
.43468 .61419 .4379 .61809 .4435 .61571 .44029 .61174 Metetra
.698 .71 .857 r
.44029 .61174 .4435 .61571 .44911 .61333 .44591 .6093 Metetra
.697 .707 .855 r
.44591 .6093 .44911 .61333 .45472 .61096 .45153 .60686 Metetra
.695 .703 .853 r
.45153 .60686 .45472 .61096 .46035 .60861 .45716 .60443 Metetra
.694 .7 .851 r
.45716 .60443 .46035 .60861 .46598 .60627 .46281 .60202 Metetra
.692 .696 .849 r
.46281 .60202 .46598 .60627 .47163 .60394 .46846 .59963 Metetra
.69 .693 .847 r
.46846 .59963 .47163 .60394 .47728 .60164 .47412 .59725 Metetra
.688 .689 .846 r
.47412 .59725 .47728 .60164 .48294 .59937 .47979 .5949 Metetra
.686 .686 .844 r
.47979 .5949 .48294 .59937 .48862 .59713 .48546 .59258 Metetra
.683 .682 .842 r
.48546 .59258 .48862 .59713 .4943 .59493 .49115 .59031 Metetra
.681 .678 .841 r
.49115 .59031 .4943 .59493 .49999 .59277 .49685 .58807 Metetra
.677 .674 .839 r
.49685 .58807 .49999 .59277 .50569 .59067 .50256 .58589 Metetra
.674 .67 .838 r
.50256 .58589 .50569 .59067 .5114 .58862 .50827 .58378 Metetra
.67 .666 .837 r
.50827 .58378 .5114 .58862 .51712 .58665 .514 .58174 Metetra
.665 .661 .836 r
.514 .58174 .51712 .58665 .52285 .58477 .51974 .57978 Metetra
.66 .657 .836 r
.51974 .57978 .52285 .58477 .5286 .58297 .52549 .57792 Metetra
.654 .652 .836 r
.52549 .57792 .5286 .58297 .53435 .58128 .53125 .57618 Metetra
.648 .648 .836 r
.53125 .57618 .53435 .58128 .54012 .5797 .53702 .57455 Metetra
.641 .643 .837 r
.53702 .57455 .54012 .5797 .54591 .57826 .54281 .57307 Metetra
.633 .639 .839 r
.54281 .57307 .54591 .57826 .5517 .57695 .54861 .57175 Metetra
.624 .635 .841 r
.54861 .57175 .5517 .57695 .55751 .57579 .55442 .5706 Metetra
.614 .631 .844 r
.55442 .5706 .55751 .57579 .56334 .57479 .56025 .56962 Metetra
.603 .627 .848 r
.56025 .56962 .56334 .57479 .56918 .57396 .5661 .56884 Metetra
.592 .624 .853 r
.5661 .56884 .56918 .57396 .57505 .57329 .57197 .56825 Metetra
.579 .622 .859 r
.57197 .56825 .57505 .57329 .58093 .57278 .57786 .56784 Metetra
.566 .621 .866 r
.57786 .56784 .58093 .57278 .58683 .57244 .58376 .56763 Metetra
.552 .62 .874 r
.58376 .56763 .58683 .57244 .59275 .57225 .58969 .56759 Metetra
.537 .621 .883 r
.58969 .56759 .59275 .57225 .59869 .57219 .59564 .56771 Metetra
.522 .623 .893 r
.59564 .56771 .59869 .57219 .60466 .57226 .60162 .56798 Metetra
.506 .626 .903 r
.60162 .56798 .60466 .57226 .61064 .57243 .60761 .56836 Metetra
.49 .63 .915 r
.60761 .56836 .61064 .57243 .61665 .57269 .61363 .56884 Metetra
.473 .636 .926 r
.61363 .56884 .61665 .57269 .62268 .57301 .61967 .56939 Metetra
.457 .643 .938 r
.61967 .56939 .62268 .57301 .62873 .57336 .62574 .56998 Metetra
.44 .652 .949 r
.62574 .56998 .62873 .57336 .6348 .57373 .63182 .5706 Metetra
.425 .662 .96 r
.63182 .5706 .6348 .57373 .64089 .57409 .63793 .57119 Metetra
.41 .673 .969 r
.63793 .57119 .64089 .57409 .647 .57441 .64406 .57175 Metetra
.398 .686 .977 r
.64406 .57175 .647 .57441 .65313 .57467 .6502 .57224 Metetra
.388 .7 .983 r
.6502 .57224 .65313 .57467 .65928 .57485 .65637 .57264 Metetra
.381 .714 .987 r
.65637 .57264 .65928 .57485 .66544 .57492 .66255 .57292 Metetra
.379 .73 .99 r
.66255 .57292 .66544 .57492 .67162 .57487 .66874 .57305 Metetra
.382 .746 .991 r
.66874 .57305 .67162 .57487 .67781 .57468 .67495 .57302 Metetra
.39 .764 .992 r
.67495 .57302 .67781 .57468 .68401 .57433 .68117 .57281 Metetra
.403 .781 .992 r
.68117 .57281 .68401 .57433 .69022 .57381 .6874 .5724 Metetra
.421 .799 .992 r
.6874 .5724 .69022 .57381 .69644 .57312 .69364 .57179 Metetra
.443 .816 .993 r
.69364 .57179 .69644 .57312 .70268 .57226 .69988 .57098 Metetra
.466 .832 .993 r
.69988 .57098 .70268 .57226 .70891 .57121 .70614 .56997 Metetra
.492 .846 .993 r
.70614 .56997 .70891 .57121 .71516 .57 .7124 .56876 Metetra
.517 .858 .994 r
.7124 .56876 .71516 .57 .72141 .56863 .71866 .56738 Metetra
.541 .868 .994 r
.71866 .56738 .72141 .56863 .72767 .5671 .72493 .56582 Metetra
.564 .876 .993 r
.72493 .56582 .72767 .5671 .73394 .56543 .73121 .56411 Metetra
.585 .881 .992 r
.73121 .56411 .73394 .56543 .74021 .56364 .73749 .56225 Metetra
.604 .884 .991 r
.73749 .56225 .74021 .56364 .74648 .56173 .74378 .56028 Metetra
.62 .885 .989 r
.74378 .56028 .74648 .56173 .75277 .55971 .75007 .55819 Metetra
.635 .885 .987 r
.75007 .55819 .75277 .55971 .75906 .5576 .75637 .55601 Metetra
.647 .884 .984 r
.75637 .55601 .75906 .5576 .76536 .5554 .76268 .55374 Metetra
.657 .882 .981 r
.76268 .55374 .76536 .5554 .77167 .55314 .76899 .55139 Metetra
.666 .879 .978 r
.76899 .55139 .77167 .55314 .77798 .5508 .77532 .54898 Metetra
.674 .876 .974 r
.77532 .54898 .77798 .5508 .7843 .54842 .78164 .5465 Metetra
.68 .871 .971 r
.78164 .5465 .7843 .54842 .79063 .54598 .78798 .54399 Metetra
.685 .868 .968 r
.78798 .54399 .79063 .54598 .79697 .5435 .79433 .54143 Metetra
.689 .863 .965 r
.79433 .54143 .79697 .5435 .80332 .54098 .80069 .53883 Metetra
.693 .859 .961 r
.80069 .53883 .80332 .54098 .80968 .53843 .80705 .5362 Metetra
.696 .854 .958 r
.80705 .5362 .80968 .53843 .81604 .53585 .81342 .53355 Metetra
.698 .85 .955 r
.81342 .53355 .81604 .53585 .82242 .53325 .81981 .53087 Metetra
.7 .845 .952 r
.81981 .53087 .82242 .53325 .82881 .53063 .8262 .52818 Metetra
.702 .841 .949 r
.8262 .52818 .82881 .53063 .8352 .52799 .83261 .52546 Metetra
.703 .836 .946 r
.83261 .52546 .8352 .52799 .84161 .52533 .83902 .52273 Metetra
.704 .832 .943 r
.83902 .52273 .84161 .52533 .84803 .52265 .84544 .51999 Metetra
.705 .828 .941 r
.84544 .51999 .84803 .52265 .85446 .51997 .85188 .51723 Metetra
.706 .824 .938 r
.85188 .51723 .85446 .51997 .86089 .51727 .85832 .51447 Metetra
.707 .82 .935 r
.85832 .51447 .86089 .51727 .86734 .51456 .86478 .51169 Metetra
.707 .816 .932 r
.86478 .51169 .86734 .51456 .8738 .51184 .87124 .50891 Metetra
.707 .812 .93 r
.87124 .50891 .8738 .51184 .88027 .50911 .87772 .50612 Metetra
.708 .808 .927 r
.87772 .50612 .88027 .50911 .88676 .50638 .88421 .50332 Metetra
.708 .804 .925 r
.88421 .50332 .88676 .50638 .89325 .50364 .89071 .50051 Metetra
.708 .8 .922 r
.89071 .50051 .89325 .50364 .89975 .50089 .89722 .4977 Metetra
.707 .796 .92 r
.89722 .4977 .89975 .50089 .90627 .49814 .90374 .49488 Metetra
.707 .793 .917 r
.90374 .49488 .90627 .49814 .9128 .49538 .91027 .49206 Metetra
.707 .789 .915 r
.91027 .49206 .9128 .49538 .91933 .49261 .91682 .48923 Metetra
.707 .785 .913 r
.91682 .48923 .91933 .49261 .92588 .48985 .92337 .4864 Metetra
.707 .782 .91 r
.92337 .4864 .92588 .48985 .93244 .48707 .92994 .48357 Metetra
.707 .779 .908 r
.92994 .48357 .93244 .48707 .93902 .48429 .93652 .48072 Metetra
.706 .775 .906 r
.93652 .48072 .93902 .48429 .9456 .48151 .94311 .47788 Metetra
.711 .763 .892 r
.34277 .65043 .34615 .65338 .35162 .65095 .34825 .64795 Metetra
.711 .759 .89 r
.34825 .64795 .35162 .65095 .3571 .64852 .35374 .64546 Metetra
.711 .756 .888 r
.35374 .64546 .3571 .64852 .36258 .64609 .35923 .64297 Metetra
.71 .753 .886 r
.35923 .64297 .36258 .64609 .36808 .64365 .36474 .64047 Metetra
.71 .75 .884 r
.36474 .64047 .36808 .64365 .37358 .64121 .37025 .63797 Metetra
.709 .747 .881 r
.37025 .63797 .37358 .64121 .37909 .63876 .37577 .63547 Metetra
.709 .744 .879 r
.37577 .63547 .37909 .63876 .38461 .63631 .3813 .63296 Metetra
.708 .741 .877 r
.3813 .63296 .38461 .63631 .39014 .63386 .38684 .63045 Metetra
.708 .738 .875 r
.38684 .63045 .39014 .63386 .39568 .6314 .39239 .62793 Metetra
.707 .735 .873 r
.39239 .62793 .39568 .6314 .40122 .62895 .39794 .62541 Metetra
.706 .732 .871 r
.39794 .62541 .40122 .62895 .40678 .62649 .40351 .62289 Metetra
.706 .729 .869 r
.40351 .62289 .40678 .62649 .41234 .62403 .40908 .62037 Metetra
.705 .726 .866 r
.40908 .62037 .41234 .62403 .41791 .62157 .41466 .61784 Metetra
.704 .723 .864 r
.41466 .61784 .41791 .62157 .42349 .61911 .42025 .61532 Metetra
.703 .72 .862 r
.42025 .61532 .42349 .61911 .42908 .61665 .42585 .61279 Metetra
.703 .717 .86 r
.42585 .61279 .42908 .61665 .43468 .61419 .43145 .61027 Metetra
.702 .713 .858 r
.43145 .61027 .43468 .61419 .44029 .61174 .43707 .60775 Metetra
.701 .71 .856 r
.43707 .60775 .44029 .61174 .44591 .6093 .44269 .60523 Metetra
.699 .707 .853 r
.44269 .60523 .44591 .6093 .45153 .60686 .44833 .60273 Metetra
.698 .703 .851 r
.44833 .60273 .45153 .60686 .45716 .60443 .45397 .60023 Metetra
.697 .7 .849 r
.45397 .60023 .45716 .60443 .46281 .60202 .45962 .59774 Metetra
.695 .696 .847 r
.45962 .59774 .46281 .60202 .46846 .59963 .46528 .59527 Metetra
.693 .693 .845 r
.46528 .59527 .46846 .59963 .47412 .59725 .47094 .59282 Metetra
.691 .689 .843 r
.47094 .59282 .47412 .59725 .47979 .5949 .47662 .59038 Metetra
.689 .685 .841 r
.47662 .59038 .47979 .5949 .48546 .59258 .4823 .58799 Metetra
.687 .681 .839 r
.4823 .58799 .48546 .59258 .49115 .59031 .488 .58562 Metetra
.684 .677 .837 r
.488 .58562 .49115 .59031 .49685 .58807 .4937 .58331 Metetra
.68 .673 .836 r
.4937 .58331 .49685 .58807 .50256 .58589 .49941 .58104 Metetra
.677 .668 .834 r
.49941 .58104 .50256 .58589 .50827 .58378 .50514 .57884 Metetra
.672 .664 .833 r
.50514 .57884 .50827 .58378 .514 .58174 .51087 .57672 Metetra
.668 .659 .832 r
.51087 .57672 .514 .58174 .51974 .57978 .51661 .57468 Metetra
.662 .654 .831 r
.51661 .57468 .51974 .57978 .52549 .57792 .52237 .57275 Metetra
.656 .649 .831 r
.52237 .57275 .52549 .57792 .53125 .57618 .52813 .57093 Metetra
.65 .644 .831 r
.52813 .57093 .53125 .57618 .53702 .57455 .53391 .56925 Metetra
.642 .639 .831 r
.53391 .56925 .53702 .57455 .54281 .57307 .5397 .56772 Metetra
.633 .633 .833 r
.5397 .56772 .54281 .57307 .54861 .57175 .5455 .56637 Metetra
.624 .628 .835 r
.5455 .56637 .54861 .57175 .55442 .5706 .55132 .56521 Metetra
.613 .623 .837 r
.55132 .56521 .55442 .5706 .56025 .56962 .55716 .56425 Metetra
.601 .619 .841 r
.55716 .56425 .56025 .56962 .5661 .56884 .56301 .56351 Metetra
.588 .615 .846 r
.56301 .56351 .5661 .56884 .57197 .56825 .56888 .56299 Metetra
.574 .611 .852 r
.56888 .56299 .57197 .56825 .57786 .56784 .57478 .5627 Metetra
.559 .609 .86 r
.57478 .5627 .57786 .56784 .58376 .56763 .58069 .56263 Metetra
.542 .607 .868 r
.58069 .56263 .58376 .56763 .58969 .56759 .58662 .56276 Metetra
.525 .607 .878 r
.58662 .56276 .58969 .56759 .59564 .56771 .59258 .56307 Metetra
.507 .608 .888 r
.59258 .56307 .59564 .56771 .60162 .56798 .59856 .56356 Metetra
.489 .611 .9 r
.59856 .56356 .60162 .56798 .60761 .56836 .60457 .56417 Metetra
.469 .615 .912 r
.60457 .56417 .60761 .56836 .61363 .56884 .6106 .5649 Metetra
.449 .62 .924 r
.6106 .5649 .61363 .56884 .61967 .56939 .61665 .56572 Metetra
.428 .626 .936 r
.61665 .56572 .61967 .56939 .62574 .56998 .62273 .56658 Metetra
.407 .634 .948 r
.62273 .56658 .62574 .56998 .63182 .5706 .62883 .56746 Metetra
.387 .644 .958 r
.62883 .56746 .63182 .5706 .63793 .57119 .63495 .56833 Metetra
.368 .655 .967 r
.63495 .56833 .63793 .57119 .64406 .57175 .6411 .56915 Metetra
.351 .667 .974 r
.6411 .56915 .64406 .57175 .6502 .57224 .64726 .5699 Metetra
.337 .68 .979 r
.64726 .5699 .6502 .57224 .65637 .57264 .65345 .57054 Metetra
.328 .695 .981 r
.65345 .57054 .65637 .57264 .66255 .57292 .65964 .57104 Metetra
.323 .711 .981 r
.65964 .57104 .66255 .57292 .66874 .57305 .66586 .57138 Metetra
.326 .729 .98 r
.66586 .57138 .66874 .57305 .67495 .57302 .67209 .57153 Metetra
.335 .748 .979 r
.67209 .57153 .67495 .57302 .68117 .57281 .67832 .57147 Metetra
.352 .769 .979 r
.67832 .57147 .68117 .57281 .6874 .5724 .68457 .57118 Metetra
.375 .789 .979 r
.68457 .57118 .6874 .5724 .69364 .57179 .69083 .57065 Metetra
.403 .81 .98 r
.69083 .57065 .69364 .57179 .69988 .57098 .69709 .5699 Metetra
.434 .83 .982 r
.69709 .5699 .69988 .57098 .70614 .56997 .70336 .56891 Metetra
.466 .847 .984 r
.70336 .56891 .70614 .56997 .7124 .56876 .70963 .5677 Metetra
.498 .862 .987 r
.70963 .5677 .7124 .56876 .71866 .56738 .71591 .56628 Metetra
.528 .874 .989 r
.71591 .56628 .71866 .56738 .72493 .56582 .72219 .56468 Metetra
.556 .883 .99 r
.72219 .56468 .72493 .56582 .73121 .56411 .72847 .56291 Metetra
.581 .889 .99 r
.72847 .56291 .73121 .56411 .73749 .56225 .73477 .56098 Metetra
.603 .892 .989 r
.73477 .56098 .73749 .56225 .74378 .56028 .74106 .55893 Metetra
.621 .893 .988 r
.74106 .55893 .74378 .56028 .75007 .55819 .74737 .55676 Metetra
.637 .893 .986 r
.74737 .55676 .75007 .55819 .75637 .55601 .75367 .55449 Metetra
.651 .891 .983 r
.75367 .55449 .75637 .55601 .76268 .55374 .75999 .55213 Metetra
.662 .888 .98 r
.75999 .55213 .76268 .55374 .76899 .55139 .76631 .54969 Metetra
.671 .885 .977 r
.76631 .54969 .76899 .55139 .77532 .54898 .77264 .54719 Metetra
.679 .881 .974 r
.77264 .54719 .77532 .54898 .78164 .5465 .77898 .54463 Metetra
.685 .877 .97 r
.77898 .54463 .78164 .5465 .78798 .54399 .78532 .54202 Metetra
.69 .872 .967 r
.78532 .54202 .78798 .54399 .79433 .54143 .79168 .53938 Metetra
.694 .867 .964 r
.79168 .53938 .79433 .54143 .80069 .53883 .79804 .5367 Metetra
.698 .862 .96 r
.79804 .5367 .80069 .53883 .80705 .5362 .80441 .53399 Metetra
.701 .857 .957 r
.80441 .53399 .80705 .5362 .81342 .53355 .81079 .53126 Metetra
.703 .852 .954 r
.81079 .53126 .81342 .53355 .81981 .53087 .81719 .5285 Metetra
.705 .848 .951 r
.81719 .5285 .81981 .53087 .8262 .52818 .82359 .52573 Metetra
.706 .843 .948 r
.82359 .52573 .8262 .52818 .83261 .52546 .83 .52294 Metetra
.708 .839 .945 r
.83 .52294 .83261 .52546 .83902 .52273 .83642 .52014 Metetra
.709 .834 .942 r
.83642 .52014 .83902 .52273 .84544 .51999 .84285 .51732 Metetra
.709 .829 .939 r
.84285 .51732 .84544 .51999 .85188 .51723 .84929 .51449 Metetra
.71 .825 .936 r
.84929 .51449 .85188 .51723 .85832 .51447 .85574 .51166 Metetra
.71 .821 .934 r
.85574 .51166 .85832 .51447 .86478 .51169 .8622 .50882 Metetra
.711 .817 .931 r
.8622 .50882 .86478 .51169 .87124 .50891 .86868 .50596 Metetra
.711 .813 .928 r
.86868 .50596 .87124 .50891 .87772 .50612 .87516 .5031 Metetra
.711 .809 .926 r
.87516 .5031 .87772 .50612 .88421 .50332 .88165 .50024 Metetra
.711 .805 .923 r
.88165 .50024 .88421 .50332 .89071 .50051 .88816 .49737 Metetra
.711 .801 .921 r
.88816 .49737 .89071 .50051 .89722 .4977 .89467 .49449 Metetra
.711 .797 .919 r
.89467 .49449 .89722 .4977 .90374 .49488 .9012 .49161 Metetra
.711 .794 .916 r
.9012 .49161 .90374 .49488 .91027 .49206 .90774 .48873 Metetra
.71 .79 .914 r
.90774 .48873 .91027 .49206 .91682 .48923 .91429 .48584 Metetra
.71 .787 .911 r
.91429 .48584 .91682 .48923 .92337 .4864 .92085 .48294 Metetra
.71 .783 .909 r
.92085 .48294 .92337 .4864 .92994 .48357 .92742 .48004 Metetra
.71 .78 .907 r
.92742 .48004 .92994 .48357 .93652 .48072 .93401 .47714 Metetra
.709 .776 .904 r
.93401 .47714 .93652 .48072 .94311 .47788 .9406 .47423 Metetra
.714 .763 .891 r
.33938 .64747 .34277 .65043 .34825 .64795 .34487 .64493 Metetra
.714 .76 .889 r
.34487 .64493 .34825 .64795 .35374 .64546 .35037 .64238 Metetra
.713 .757 .887 r
.35037 .64238 .35374 .64546 .35923 .64297 .35587 .63984 Metetra
.713 .754 .884 r
.35587 .63984 .35923 .64297 .36474 .64047 .36139 .63728 Metetra
.712 .751 .882 r
.36139 .63728 .36474 .64047 .37025 .63797 .36691 .63472 Metetra
.712 .748 .88 r
.36691 .63472 .37025 .63797 .37577 .63547 .37244 .63216 Metetra
.711 .745 .878 r
.37244 .63216 .37577 .63547 .3813 .63296 .37798 .62959 Metetra
.711 .742 .876 r
.37798 .62959 .3813 .63296 .38684 .63045 .38353 .62702 Metetra
.71 .739 .873 r
.38353 .62702 .38684 .63045 .39239 .62793 .38909 .62444 Metetra
.71 .736 .871 r
.38909 .62444 .39239 .62793 .39794 .62541 .39465 .62186 Metetra
.709 .732 .869 r
.39465 .62186 .39794 .62541 .40351 .62289 .40022 .61928 Metetra
.708 .729 .867 r
.40022 .61928 .40351 .62289 .40908 .62037 .40581 .61669 Metetra
.708 .726 .865 r
.40581 .61669 .40908 .62037 .41466 .61784 .41139 .6141 Metetra
.707 .723 .863 r
.41139 .6141 .41466 .61784 .42025 .61532 .41699 .61151 Metetra
.706 .72 .86 r
.41699 .61151 .42025 .61532 .42585 .61279 .4226 .60892 Metetra
.705 .717 .858 r
.4226 .60892 .42585 .61279 .43145 .61027 .42822 .60633 Metetra
.704 .714 .856 r
.42822 .60633 .43145 .61027 .43707 .60775 .43384 .60374 Metetra
.703 .71 .854 r
.43384 .60374 .43707 .60775 .44269 .60523 .43947 .60115 Metetra
.702 .707 .852 r
.43947 .60115 .44269 .60523 .44833 .60273 .44511 .59857 Metetra
.701 .704 .849 r
.44511 .59857 .44833 .60273 .45397 .60023 .45076 .59599 Metetra
.7 .7 .847 r
.45076 .59599 .45397 .60023 .45962 .59774 .45642 .59342 Metetra
.698 .696 .845 r
.45642 .59342 .45962 .59774 .46528 .59527 .46208 .59087 Metetra
.696 .693 .843 r
.46208 .59087 .46528 .59527 .47094 .59282 .46776 .58834 Metetra
.694 .689 .84 r
.46776 .58834 .47094 .59282 .47662 .59038 .47344 .58582 Metetra
.692 .685 .838 r
.47344 .58582 .47662 .59038 .4823 .58799 .47914 .58333 Metetra
.69 .68 .836 r
.47914 .58333 .4823 .58799 .488 .58562 .48484 .58088 Metetra
.687 .676 .834 r
.48484 .58088 .488 .58562 .4937 .58331 .49055 .57847 Metetra
.684 .672 .832 r
.49055 .57847 .4937 .58331 .49941 .58104 .49626 .57611 Metetra
.68 .667 .83 r
.49626 .57611 .49941 .58104 .50514 .57884 .50199 .57381 Metetra
.676 .662 .829 r
.50199 .57381 .50514 .57884 .51087 .57672 .50773 .57159 Metetra
.671 .657 .827 r
.50773 .57159 .51087 .57672 .51661 .57468 .51348 .56946 Metetra
.665 .651 .826 r
.51348 .56946 .51661 .57468 .52237 .57275 .51924 .56743 Metetra
.659 .646 .825 r
.51924 .56743 .52237 .57275 .52813 .57093 .52501 .56553 Metetra
.652 .64 .825 r
.52501 .56553 .52813 .57093 .53391 .56925 .53079 .56377 Metetra
.644 .634 .825 r
.53079 .56377 .53391 .56925 .5397 .56772 .53658 .56218 Metetra
.634 .628 .826 r
.53658 .56218 .5397 .56772 .5455 .56637 .54239 .56078 Metetra
.624 .621 .828 r
.54239 .56078 .5455 .56637 .55132 .56521 .54821 .5596 Metetra
.612 .615 .83 r
.54821 .5596 .55132 .56521 .55716 .56425 .55405 .55865 Metetra
.599 .61 .834 r
.55405 .55865 .55716 .56425 .56301 .56351 .55991 .55795 Metetra
.584 .604 .839 r
.55991 .55795 .56301 .56351 .56888 .56299 .56578 .55751 Metetra
.568 .6 .845 r
.56578 .55751 .56888 .56299 .57478 .5627 .57168 .55733 Metetra
.551 .596 .852 r
.57168 .55733 .57478 .5627 .58069 .56263 .5776 .55741 Metetra
.532 .593 .861 r
.5776 .55741 .58069 .56263 .58662 .56276 .58354 .55772 Metetra
.512 .592 .871 r
.58354 .55772 .58662 .56276 .59258 .56307 .58951 .55826 Metetra
.491 .592 .883 r
.58951 .55826 .59258 .56307 .59856 .56356 .5955 .55899 Metetra
.469 .594 .895 r
.5955 .55899 .59856 .56356 .60457 .56417 .60152 .55987 Metetra
.445 .596 .908 r
.60152 .55987 .60457 .56417 .6106 .5649 .60756 .56088 Metetra
.42 .601 .92 r
.60756 .56088 .6106 .5649 .61665 .56572 .61362 .56198 Metetra
.395 .606 .933 r
.61362 .56198 .61665 .56572 .62273 .56658 .61972 .56314 Metetra
.369 .613 .945 r
.61972 .56314 .62273 .56658 .62883 .56746 .62583 .56433 Metetra
.343 .621 .954 r
.62583 .56433 .62883 .56746 .63495 .56833 .63197 .5655 Metetra
.317 .631 .962 r
.63197 .5655 .63495 .56833 .6411 .56915 .63813 .56662 Metetra
.294 .641 .967 r
.63813 .56662 .6411 .56915 .64726 .5699 .64431 .56766 Metetra
.275 .653 .968 r
.64431 .56766 .64726 .5699 .65345 .57054 .65052 .56857 Metetra
.26 .667 .966 r
.65052 .56857 .65345 .57054 .65964 .57104 .65674 .56933 Metetra
.252 .682 .962 r
.65674 .56933 .65964 .57104 .66586 .57138 .66297 .5699 Metetra
.254 .701 .958 r
.66297 .5699 .66586 .57138 .67209 .57153 .66922 .57024 Metetra
.266 .722 .954 r
.66922 .57024 .67209 .57153 .67832 .57147 .67547 .57035 Metetra
.287 .745 .952 r
.67547 .57035 .67832 .57147 .68457 .57118 .68174 .57018 Metetra
.316 .771 .953 r
.68174 .57018 .68457 .57118 .69083 .57065 .68801 .56974 Metetra
.353 .797 .957 r
.68801 .56974 .69083 .57065 .69709 .5699 .69429 .56903 Metetra
.393 .822 .962 r
.69429 .56903 .69709 .5699 .70336 .56891 .70057 .56806 Metetra
.435 .845 .968 r
.70057 .56806 .70336 .56891 .70963 .5677 .70686 .56683 Metetra
.476 .864 .975 r
.70686 .56683 .70963 .5677 .71591 .56628 .71315 .56537 Metetra
.514 .879 .98 r
.71315 .56537 .71591 .56628 .72219 .56468 .71944 .5637 Metetra
.548 .89 .983 r
.71944 .5637 .72219 .56468 .72847 .56291 .72574 .56185 Metetra
.577 .897 .985 r
.72574 .56185 .72847 .56291 .73477 .56098 .73204 .55984 Metetra
.602 .901 .986 r
.73204 .55984 .73477 .56098 .74106 .55893 .73834 .55768 Metetra
.623 .902 .985 r
.73834 .55768 .74106 .55893 .74737 .55676 .74465 .55541 Metetra
.641 .901 .984 r
.74465 .55541 .74737 .55676 .75367 .55449 .75097 .55304 Metetra
.655 .899 .982 r
.75097 .55304 .75367 .55449 .75999 .55213 .75729 .55058 Metetra
.667 .895 .979 r
.75729 .55058 .75999 .55213 .76631 .54969 .76362 .54804 Metetra
.677 .891 .976 r
.76362 .54804 .76631 .54969 .77264 .54719 .76996 .54544 Metetra
.684 .886 .972 r
.76996 .54544 .77264 .54719 .77898 .54463 .7763 .54279 Metetra
.69 .881 .969 r
.7763 .54279 .77898 .54463 .78532 .54202 .78266 .54009 Metetra
.696 .876 .966 r
.78266 .54009 .78532 .54202 .79168 .53938 .78902 .53735 Metetra
.7 .871 .962 r
.78902 .53735 .79168 .53938 .79804 .5367 .79539 .53458 Metetra
.703 .866 .959 r
.79539 .53458 .79804 .5367 .80441 .53399 .80176 .53179 Metetra
.706 .86 .956 r
.80176 .53179 .80441 .53399 .81079 .53126 .80815 .52897 Metetra
.708 .855 .952 r
.80815 .52897 .81079 .53126 .81719 .5285 .81455 .52614 Metetra
.71 .85 .949 r
.81455 .52614 .81719 .5285 .82359 .52573 .82096 .52328 Metetra
.711 .845 .946 r
.82096 .52328 .82359 .52573 .83 .52294 .82738 .52042 Metetra
.712 .84 .943 r
.82738 .52042 .83 .52294 .83642 .52014 .8338 .51754 Metetra
.713 .836 .941 r
.8338 .51754 .83642 .52014 .84285 .51732 .84024 .51465 Metetra
.713 .831 .938 r
.84024 .51465 .84285 .51732 .84929 .51449 .84669 .51175 Metetra
.714 .827 .935 r
.84669 .51175 .84929 .51449 .85574 .51166 .85315 .50884 Metetra
.714 .822 .932 r
.85315 .50884 .85574 .51166 .8622 .50882 .85962 .50593 Metetra
.714 .818 .93 r
.85962 .50593 .8622 .50882 .86868 .50596 .86609 .50301 Metetra
.714 .814 .927 r
.86609 .50301 .86868 .50596 .87516 .5031 .87258 .50008 Metetra
.714 .81 .925 r
.87258 .50008 .87516 .5031 .88165 .50024 .87909 .49715 Metetra
.714 .806 .922 r
.87909 .49715 .88165 .50024 .88816 .49737 .8856 .49421 Metetra
.714 .802 .92 r
.8856 .49421 .88816 .49737 .89467 .49449 .89212 .49127 Metetra
.714 .798 .917 r
.89212 .49127 .89467 .49449 .9012 .49161 .89865 .48833 Metetra
.714 .795 .915 r
.89865 .48833 .9012 .49161 .90774 .48873 .9052 .48538 Metetra
.713 .791 .912 r
.9052 .48538 .90774 .48873 .91429 .48584 .91175 .48242 Metetra
.713 .787 .91 r
.91175 .48242 .91429 .48584 .92085 .48294 .91832 .47947 Metetra
.713 .784 .908 r
.91832 .47947 .92085 .48294 .92742 .48004 .9249 .47651 Metetra
.712 .781 .906 r
.9249 .47651 .92742 .48004 .93401 .47714 .93148 .47354 Metetra
.712 .777 .903 r
.93148 .47354 .93401 .47714 .9406 .47423 .93808 .47057 Metetra
.717 .763 .889 r
.33598 .6445 .33938 .64747 .34487 .64493 .34148 .6419 Metetra
.716 .76 .887 r
.34148 .6419 .34487 .64493 .35037 .64238 .34699 .6393 Metetra
.716 .757 .885 r
.34699 .6393 .35037 .64238 .35587 .63984 .3525 .63669 Metetra
.715 .754 .883 r
.3525 .63669 .35587 .63984 .36139 .63728 .35803 .63408 Metetra
.715 .751 .881 r
.35803 .63408 .36139 .63728 .36691 .63472 .36356 .63146 Metetra
.714 .748 .879 r
.36356 .63146 .36691 .63472 .37244 .63216 .3691 .62884 Metetra
.714 .745 .876 r
.3691 .62884 .37244 .63216 .37798 .62959 .37465 .62621 Metetra
.713 .742 .874 r
.37465 .62621 .37798 .62959 .38353 .62702 .38021 .62358 Metetra
.713 .739 .872 r
.38021 .62358 .38353 .62702 .38909 .62444 .38577 .62094 Metetra
.712 .736 .87 r
.38577 .62094 .38909 .62444 .39465 .62186 .39135 .6183 Metetra
.712 .733 .868 r
.39135 .6183 .39465 .62186 .40022 .61928 .39693 .61565 Metetra
.711 .73 .865 r
.39693 .61565 .40022 .61928 .40581 .61669 .40252 .613 Metetra
.71 .727 .863 r
.40252 .613 .40581 .61669 .41139 .6141 .40812 .61035 Metetra
.71 .724 .861 r
.40812 .61035 .41139 .6141 .41699 .61151 .41373 .60769 Metetra
.709 .721 .859 r
.41373 .60769 .41699 .61151 .4226 .60892 .41934 .60503 Metetra
.708 .717 .856 r
.41934 .60503 .4226 .60892 .42822 .60633 .42497 .60237 Metetra
.707 .714 .854 r
.42497 .60237 .42822 .60633 .43384 .60374 .4306 .59971 Metetra
.706 .711 .852 r
.4306 .59971 .43384 .60374 .43947 .60115 .43624 .59704 Metetra
.705 .707 .85 r
.43624 .59704 .43947 .60115 .44511 .59857 .44189 .59438 Metetra
.704 .704 .847 r
.44189 .59438 .44511 .59857 .45076 .59599 .44755 .59173 Metetra
.703 .7 .845 r
.44755 .59173 .45076 .59599 .45642 .59342 .45321 .58908 Metetra
.701 .697 .843 r
.45321 .58908 .45642 .59342 .46208 .59087 .45888 .58644 Metetra
.699 .693 .84 r
.45888 .58644 .46208 .59087 .46776 .58834 .46457 .58382 Metetra
.698 .689 .838 r
.46457 .58382 .46776 .58834 .47344 .58582 .47026 .58121 Metetra
.695 .684 .835 r
.47026 .58121 .47344 .58582 .47914 .58333 .47596 .57863 Metetra
.693 .68 .833 r
.47596 .57863 .47914 .58333 .48484 .58088 .48167 .57608 Metetra
.69 .675 .831 r
.48167 .57608 .48484 .58088 .49055 .57847 .48738 .57356 Metetra
.687 .671 .829 r
.48738 .57356 .49055 .57847 .49626 .57611 .49311 .5711 Metetra
.683 .666 .826 r
.49311 .5711 .49626 .57611 .50199 .57381 .49884 .56869 Metetra
.679 .66 .824 r
.49884 .56869 .50199 .57381 .50773 .57159 .50459 .56636 Metetra
.674 .655 .823 r
.50459 .56636 .50773 .57159 .51348 .56946 .51034 .56411 Metetra
.669 .649 .821 r
.51034 .56411 .51348 .56946 .51924 .56743 .5161 .56197 Metetra
.663 .642 .82 r
.5161 .56197 .51924 .56743 .52501 .56553 .52188 .55996 Metetra
.655 .636 .819 r
.52188 .55996 .52501 .56553 .53079 .56377 .52766 .55811 Metetra
.646 .629 .818 r
.52766 .55811 .53079 .56377 .53658 .56218 .53346 .55643 Metetra
.636 .622 .819 r
.53346 .55643 .53658 .56218 .54239 .56078 .53927 .55496 Metetra
.625 .615 .82 r
.53927 .55496 .54239 .56078 .54821 .5596 .54509 .55373 Metetra
.612 .607 .822 r
.54509 .55373 .54821 .5596 .55405 .55865 .55094 .55278 Metetra
.597 .6 .826 r
.55094 .55278 .55405 .55865 .55991 .55795 .5568 .55211 Metetra
.581 .593 .83 r
.5568 .55211 .55991 .55795 .56578 .55751 .56268 .55174 Metetra
.562 .587 .836 r
.56268 .55174 .56578 .55751 .57168 .55733 .56858 .55169 Metetra
.542 .582 .844 r
.56858 .55169 .57168 .55733 .5776 .55741 .5745 .55193 Metetra
.52 .578 .853 r
.5745 .55193 .5776 .55741 .58354 .55772 .58045 .55246 Metetra
.497 .575 .864 r
.58045 .55246 .58354 .55772 .58951 .55826 .58642 .55324 Metetra
.473 .574 .876 r
.58642 .55324 .58951 .55826 .5955 .55899 .59242 .55424 Metetra
.446 .574 .889 r
.59242 .55424 .5955 .55899 .60152 .55987 .59845 .55543 Metetra
.418 .575 .902 r
.59845 .55543 .60152 .55987 .60756 .56088 .6045 .55675 Metetra
.388 .578 .915 r
.6045 .55675 .60756 .56088 .61362 .56198 .61058 .55818 Metetra
.357 .582 .928 r
.61058 .55818 .61362 .56198 .61972 .56314 .61669 .55968 Metetra
.323 .587 .939 r
.61669 .55968 .61972 .56314 .62583 .56433 .62282 .5612 Metetra
.29 .593 .947 r
.62282 .5612 .62583 .56433 .63197 .5655 .62898 .56271 Metetra
.256 .599 .951 r
.62898 .56271 .63197 .5655 .63813 .56662 .63516 .56417 Metetra
.224 .607 .952 r
.63516 .56417 .63813 .56662 .64431 .56766 .64136 .56553 Metetra
.196 .616 .947 r
.64136 .56553 .64431 .56766 .65052 .56857 .64758 .56675 Metetra
.175 .626 .939 r
.64758 .56675 .65052 .56857 .65674 .56933 .65382 .5678 Metetra
.163 .639 .928 r
.65382 .5678 .65674 .56933 .66297 .5699 .66007 .56863 Metetra
.164 .657 .917 r
.66007 .56863 .66297 .5699 .66922 .57024 .66634 .5692 Metetra
.177 .68 .909 r
.66634 .5692 .66922 .57024 .67547 .57035 .67262 .56948 Metetra
.205 .708 .907 r
.67262 .56948 .67547 .57035 .68174 .57018 .6789 .56946 Metetra
.243 .739 .91 r
.6789 .56946 .68174 .57018 .68801 .56974 .68519 .56911 Metetra
.291 .773 .918 r
.68519 .56911 .68801 .56974 .69429 .56903 .69149 .56843 Metetra
.344 .807 .93 r
.69149 .56843 .69429 .56903 .70057 .56806 .69778 .56745 Metetra
.398 .837 .943 r
.69778 .56745 .70057 .56806 .70686 .56683 .70408 .56618 Metetra
.45 .863 .955 r
.70408 .56618 .70686 .56683 .71315 .56537 .71038 .56465 Metetra
.497 .883 .966 r
.71038 .56465 .71315 .56537 .71944 .5637 .71668 .5629 Metetra
.539 .897 .973 r
.71668 .5629 .71944 .5637 .72574 .56185 .72299 .56094 Metetra
.574 .906 .978 r
.72299 .56094 .72574 .56185 .73204 .55984 .7293 .55882 Metetra
.603 .91 .981 r
.7293 .55882 .73204 .55984 .73834 .55768 .73561 .55655 Metetra
.626 .911 .982 r
.73561 .55655 .73834 .55768 .74465 .55541 .74193 .55416 Metetra
.646 .91 .981 r
.74193 .55416 .74465 .55541 .75097 .55304 .74825 .55167 Metetra
.661 .907 .979 r
.74825 .55167 .75097 .55304 .75729 .55058 .75458 .54909 Metetra
.673 .902 .977 r
.75458 .54909 .75729 .55058 .76362 .54804 .76092 .54645 Metetra
.683 .897 .974 r
.76092 .54645 .76362 .54804 .76996 .54544 .76726 .54373 Metetra
.691 .892 .971 r
.76726 .54373 .76996 .54544 .7763 .54279 .77362 .54098 Metetra
.697 .886 .967 r
.77362 .54098 .7763 .54279 .78266 .54009 .77997 .53818 Metetra
.702 .88 .964 r
.77997 .53818 .78266 .54009 .78902 .53735 .78634 .53534 Metetra
.706 .875 .961 r
.78634 .53534 .78902 .53735 .79539 .53458 .79272 .53248 Metetra
.709 .869 .957 r
.79272 .53248 .79539 .53458 .80176 .53179 .79911 .5296 Metetra
.711 .863 .954 r
.79911 .5296 .80176 .53179 .80815 .52897 .8055 .52669 Metetra
.713 .858 .951 r
.8055 .52669 .80815 .52897 .81455 .52614 .8119 .52377 Metetra
.714 .852 .948 r
.8119 .52377 .81455 .52614 .82096 .52328 .81832 .52083 Metetra
.715 .847 .945 r
.81832 .52083 .82096 .52328 .82738 .52042 .82474 .51789 Metetra
.716 .842 .942 r
.82474 .51789 .82738 .52042 .8338 .51754 .83118 .51493 Metetra
.717 .837 .939 r
.83118 .51493 .8338 .51754 .84024 .51465 .83762 .51196 Metetra
.717 .833 .936 r
.83762 .51196 .84024 .51465 .84669 .51175 .84408 .50899 Metetra
.717 .828 .933 r
.84408 .50899 .84669 .51175 .85315 .50884 .85054 .50601 Metetra
.718 .824 .931 r
.85054 .50601 .85315 .50884 .85962 .50593 .85702 .50303 Metetra
.718 .819 .928 r
.85702 .50303 .85962 .50593 .86609 .50301 .8635 .50004 Metetra
.718 .815 .926 r
.8635 .50004 .86609 .50301 .87258 .50008 .87 .49705 Metetra
.718 .811 .923 r
.87 .49705 .87258 .50008 .87909 .49715 .87651 .49405 Metetra
.717 .807 .921 r
.87651 .49405 .87909 .49715 .8856 .49421 .88302 .49104 Metetra
.717 .803 .918 r
.88302 .49104 .8856 .49421 .89212 .49127 .88955 .48804 Metetra
.717 .8 .916 r
.88955 .48804 .89212 .49127 .89865 .48833 .89609 .48503 Metetra
.717 .796 .914 r
.89609 .48503 .89865 .48833 .9052 .48538 .90264 .48202 Metetra
.716 .792 .911 r
.90264 .48202 .9052 .48538 .91175 .48242 .9092 .479 Metetra
.716 .789 .909 r
.9092 .479 .91175 .48242 .91832 .47947 .91578 .47598 Metetra
.716 .785 .907 r
.91578 .47598 .91832 .47947 .9249 .47651 .92236 .47295 Metetra
.715 .782 .904 r
.92236 .47295 .9249 .47651 .93148 .47354 .92895 .46992 Metetra
.715 .778 .902 r
.92895 .46992 .93148 .47354 .93808 .47057 .93556 .46689 Metetra
.719 .764 .888 r
.33256 .64152 .33598 .6445 .34148 .6419 .33807 .63886 Metetra
.719 .761 .886 r
.33807 .63886 .34148 .6419 .34699 .6393 .34359 .6362 Metetra
.718 .758 .884 r
.34359 .6362 .34699 .6393 .3525 .63669 .34912 .63354 Metetra
.718 .755 .881 r
.34912 .63354 .3525 .63669 .35803 .63408 .35465 .63087 Metetra
.717 .752 .879 r
.35465 .63087 .35803 .63408 .36356 .63146 .36019 .62819 Metetra
.717 .749 .877 r
.36019 .62819 .36356 .63146 .3691 .62884 .36575 .62551 Metetra
.716 .746 .875 r
.36575 .62551 .3691 .62884 .37465 .62621 .3713 .62282 Metetra
.716 .743 .873 r
.3713 .62282 .37465 .62621 .38021 .62358 .37687 .62013 Metetra
.715 .74 .871 r
.37687 .62013 .38021 .62358 .38577 .62094 .38245 .61743 Metetra
.715 .737 .868 r
.38245 .61743 .38577 .62094 .39135 .6183 .38803 .61473 Metetra
.714 .734 .866 r
.38803 .61473 .39135 .6183 .39693 .61565 .39362 .61202 Metetra
.714 .731 .864 r
.39362 .61202 .39693 .61565 .40252 .613 .39922 .6093 Metetra
.713 .728 .862 r
.39922 .6093 .40252 .613 .40812 .61035 .40483 .60658 Metetra
.712 .725 .86 r
.40483 .60658 .40812 .61035 .41373 .60769 .41045 .60386 Metetra
.712 .721 .857 r
.41045 .60386 .41373 .60769 .41934 .60503 .41607 .60113 Metetra
.711 .718 .855 r
.41607 .60113 .41934 .60503 .42497 .60237 .42171 .59839 Metetra
.71 .715 .852 r
.42171 .59839 .42497 .60237 .4306 .59971 .42735 .59565 Metetra
.709 .711 .85 r
.42735 .59565 .4306 .59971 .43624 .59704 .433 .59292 Metetra
.708 .708 .848 r
.433 .59292 .43624 .59704 .44189 .59438 .43865 .59018 Metetra
.707 .704 .845 r
.43865 .59018 .44189 .59438 .44755 .59173 .44432 .58744 Metetra
.706 .701 .843 r
.44432 .58744 .44755 .59173 .45321 .58908 .44999 .58471 Metetra
.704 .697 .84 r
.44999 .58471 .45321 .58908 .45888 .58644 .45568 .58198 Metetra
.703 .693 .838 r
.45568 .58198 .45888 .58644 .46457 .58382 .46137 .57926 Metetra
.701 .689 .835 r
.46137 .57926 .46457 .58382 .47026 .58121 .46706 .57656 Metetra
.699 .685 .833 r
.46706 .57656 .47026 .58121 .47596 .57863 .47277 .57387 Metetra
.697 .68 .83 r
.47277 .57387 .47596 .57863 .48167 .57608 .47849 .57122 Metetra
.694 .675 .827 r
.47849 .57122 .48167 .57608 .48738 .57356 .48421 .56859 Metetra
.691 .67 .825 r
.48421 .56859 .48738 .57356 .49311 .5711 .48994 .56601 Metetra
.687 .665 .822 r
.48994 .56601 .49311 .5711 .49884 .56869 .49568 .56348 Metetra
.683 .659 .82 r
.49568 .56348 .49884 .56869 .50459 .56636 .50143 .56102 Metetra
.678 .653 .818 r
.50143 .56102 .50459 .56636 .51034 .56411 .50719 .55865 Metetra
.673 .647 .816 r
.50719 .55865 .51034 .56411 .5161 .56197 .51296 .55637 Metetra
.667 .64 .814 r
.51296 .55637 .5161 .56197 .52188 .55996 .51874 .55423 Metetra
.659 .632 .812 r
.51874 .55423 .52188 .55996 .52766 .55811 .52452 .55225 Metetra
.65 .625 .811 r
.52452 .55225 .52766 .55811 .53346 .55643 .53033 .55045 Metetra
.639 .616 .811 r
.53033 .55045 .53346 .55643 .53927 .55496 .53614 .54889 Metetra
.627 .608 .811 r
.53614 .54889 .53927 .55496 .54509 .55373 .54197 .54759 Metetra
.613 .599 .813 r
.54197 .54759 .54509 .55373 .55094 .55278 .54781 .5466 Metetra
.596 .59 .816 r
.54781 .5466 .55094 .55278 .5568 .55211 .55367 .54594 Metetra
.577 .582 .82 r
.55367 .54594 .5568 .55211 .56268 .55174 .55955 .54565 Metetra
.556 .574 .827 r
.55955 .54565 .56268 .55174 .56858 .55169 .56546 .54573 Metetra
.533 .566 .835 r
.56546 .54573 .56858 .55169 .5745 .55193 .57139 .54616 Metetra
.508 .561 .844 r
.57139 .54616 .5745 .55193 .58045 .55246 .57734 .54693 Metetra
.481 .556 .855 r
.57734 .54693 .58045 .55246 .58642 .55324 .58332 .548 Metetra
.452 .553 .868 r
.58332 .548 .58642 .55324 .59242 .55424 .58933 .54931 Metetra
.421 .552 .881 r
.58933 .54931 .59242 .55424 .59845 .55543 .59537 .55083 Metetra
.387 .551 .895 r
.59537 .55083 .59845 .55543 .6045 .55675 .60143 .55251 Metetra
.351 .552 .908 r
.60143 .55251 .6045 .55675 .61058 .55818 .60753 .55431 Metetra
.312 .553 .92 r
.60753 .55431 .61058 .55818 .61669 .55968 .61365 .55618 Metetra
.27 .555 .929 r
.61365 .55618 .61669 .55968 .62282 .5612 .61979 .55808 Metetra
.226 .557 .934 r
.61979 .55808 .62282 .5612 .62898 .56271 .62597 .55997 Metetra
.182 .558 .933 r
.62597 .55997 .62898 .56271 .63516 .56417 .63217 .5618 Metetra
.139 .56 .926 r
.63217 .5618 .63516 .56417 .64136 .56553 .63839 .56353 Metetra
.101 .563 .911 r
.63839 .56353 .64136 .56553 .64758 .56675 .64463 .5651 Metetra
.07 .568 .891 r
.64463 .5651 .64758 .56675 .65382 .5678 .6509 .56648 Metetra
.053 .576 .869 r
.6509 .56648 .65382 .5678 .66007 .56863 .65717 .56761 Metetra
.051 .592 .849 r
.65717 .56761 .66007 .56863 .66634 .5692 .66346 .56844 Metetra
.068 .616 .836 r
.66346 .56844 .66634 .5692 .67262 .56948 .66976 .56893 Metetra
.102 .648 .832 r
.66976 .56893 .67262 .56948 .6789 .56946 .67607 .56905 Metetra
.152 .689 .839 r
.67607 .56905 .6789 .56946 .68519 .56911 .68238 .56878 Metetra
.215 .735 .856 r
.68238 .56878 .68519 .56911 .69149 .56843 .68868 .56813 Metetra
.284 .78 .879 r
.68868 .56813 .69149 .56843 .69778 .56745 .69499 .56712 Metetra
.355 .823 .903 r
.69499 .56712 .69778 .56745 .70408 .56618 .7013 .56578 Metetra
.421 .858 .927 r
.7013 .56578 .70408 .56618 .71038 .56465 .70761 .56416 Metetra
.481 .885 .945 r
.70761 .56416 .71038 .56465 .71668 .5629 .71393 .56228 Metetra
.531 .903 .96 r
.71393 .56228 .71668 .5629 .72299 .56094 .72024 .56019 Metetra
.572 .914 .969 r
.72024 .56019 .72299 .56094 .7293 .55882 .72655 .55793 Metetra
.606 .919 .975 r
.72655 .55793 .7293 .55882 .73561 .55655 .73287 .55553 Metetra
.632 .92 .977 r
.73287 .55553 .73561 .55655 .74193 .55416 .7392 .553 Metetra
.652 .918 .978 r
.7392 .553 .74193 .55416 .74825 .55167 .74553 .55037 Metetra
.668 .914 .977 r
.74553 .55037 .74825 .55167 .75458 .54909 .75187 .54767 Metetra
.681 .909 .974 r
.75187 .54767 .75458 .54909 .76092 .54645 .75821 .5449 Metetra
.69 .903 .972 r
.75821 .5449 .76092 .54645 .76726 .54373 .76456 .54207 Metetra
.698 .897 .969 r
.76456 .54207 .76726 .54373 .77362 .54098 .77092 .53919 Metetra
.704 .89 .965 r
.77092 .53919 .77362 .54098 .77997 .53818 .77728 .53629 Metetra
.708 .884 .962 r
.77728 .53629 .77997 .53818 .78634 .53534 .78366 .53335 Metetra
.712 .878 .959 r
.78366 .53335 .78634 .53534 .79272 .53248 .79004 .53039 Metetra
.715 .872 .955 r
.79004 .53039 .79272 .53248 .79911 .5296 .79643 .5274 Metetra
.716 .866 .952 r
.79643 .5274 .79911 .5296 .8055 .52669 .80284 .52441 Metetra
.718 .86 .949 r
.80284 .52441 .8055 .52669 .8119 .52377 .80925 .5214 Metetra
.719 .854 .946 r
.80925 .5214 .8119 .52377 .81832 .52083 .81567 .51838 Metetra
.72 .849 .943 r
.81567 .51838 .81832 .52083 .82474 .51789 .8221 .51535 Metetra
.721 .844 .94 r
.8221 .51535 .82474 .51789 .83118 .51493 .82854 .51231 Metetra
.721 .839 .937 r
.82854 .51231 .83118 .51493 .83762 .51196 .83499 .50927 Metetra
.721 .834 .934 r
.83499 .50927 .83762 .51196 .84408 .50899 .84145 .50622 Metetra
.721 .829 .932 r
.84145 .50622 .84408 .50899 .85054 .50601 .84792 .50317 Metetra
.721 .825 .929 r
.84792 .50317 .85054 .50601 .85702 .50303 .85441 .50012 Metetra
.721 .82 .927 r
.85441 .50012 .85702 .50303 .8635 .50004 .8609 .49706 Metetra
.721 .816 .924 r
.8609 .49706 .8635 .50004 .87 .49705 .8674 .494 Metetra
.721 .812 .922 r
.8674 .494 .87 .49705 .87651 .49405 .87391 .49093 Metetra
.721 .808 .919 r
.87391 .49093 .87651 .49405 .88302 .49104 .88044 .48786 Metetra
.72 .804 .917 r
.88044 .48786 .88302 .49104 .88955 .48804 .88697 .48479 Metetra
.72 .8 .915 r
.88697 .48479 .88955 .48804 .89609 .48503 .89352 .48171 Metetra
.72 .797 .912 r
.89352 .48171 .89609 .48503 .90264 .48202 .90008 .47864 Metetra
.719 .793 .91 r
.90008 .47864 .90264 .48202 .9092 .479 .90664 .47555 Metetra
.719 .789 .908 r
.90664 .47555 .9092 .479 .91578 .47598 .91322 .47247 Metetra
.718 .786 .905 r
.91322 .47247 .91578 .47598 .92236 .47295 .91981 .46938 Metetra
.718 .783 .903 r
.91981 .46938 .92236 .47295 .92895 .46992 .92641 .46629 Metetra
.717 .779 .901 r
.92641 .46629 .92895 .46992 .93556 .46689 .93302 .4632 Metetra
.722 .764 .886 r
.32913 .63853 .33256 .64152 .33807 .63886 .33465 .63581 Metetra
.721 .761 .884 r
.33465 .63581 .33807 .63886 .34359 .6362 .34018 .63309 Metetra
.721 .758 .882 r
.34018 .63309 .34359 .6362 .34912 .63354 .34572 .63037 Metetra
.72 .755 .88 r
.34572 .63037 .34912 .63354 .35465 .63087 .35126 .62764 Metetra
.72 .752 .878 r
.35126 .62764 .35465 .63087 .36019 .62819 .35682 .62491 Metetra
.719 .749 .876 r
.35682 .62491 .36019 .62819 .36575 .62551 .36238 .62217 Metetra
.719 .746 .873 r
.36238 .62217 .36575 .62551 .3713 .62282 .36795 .61942 Metetra
.718 .743 .871 r
.36795 .61942 .3713 .62282 .37687 .62013 .37353 .61667 Metetra
.718 .74 .869 r
.37353 .61667 .37687 .62013 .38245 .61743 .37911 .61391 Metetra
.717 .737 .867 r
.37911 .61391 .38245 .61743 .38803 .61473 .3847 .61114 Metetra
.717 .734 .865 r
.3847 .61114 .38803 .61473 .39362 .61202 .39031 .60837 Metetra
.716 .731 .863 r
.39031 .60837 .39362 .61202 .39922 .6093 .39592 .60559 Metetra
.716 .728 .86 r
.39592 .60559 .39922 .6093 .40483 .60658 .40153 .6028 Metetra
.715 .725 .858 r
.40153 .6028 .40483 .60658 .41045 .60386 .40716 .60001 Metetra
.714 .722 .856 r
.40716 .60001 .41045 .60386 .41607 .60113 .41279 .59721 Metetra
.714 .719 .853 r
.41279 .59721 .41607 .60113 .42171 .59839 .41843 .5944 Metetra
.713 .715 .851 r
.41843 .5944 .42171 .59839 .42735 .59565 .42408 .59159 Metetra
.712 .712 .848 r
.42408 .59159 .42735 .59565 .433 .59292 .42974 .58877 Metetra
.711 .709 .846 r
.42974 .58877 .433 .59292 .43865 .59018 .43541 .58595 Metetra
.71 .705 .843 r
.43541 .58595 .43865 .59018 .44432 .58744 .44108 .58313 Metetra
.709 .701 .841 r
.44108 .58313 .44432 .58744 .44999 .58471 .44677 .58031 Metetra
.708 .698 .838 r
.44677 .58031 .44999 .58471 .45568 .58198 .45246 .57749 Metetra
.706 .693 .835 r
.45246 .57749 .45568 .58198 .46137 .57926 .45815 .57468 Metetra
.705 .689 .833 r
.45815 .57468 .46137 .57926 .46706 .57656 .46386 .57187 Metetra
.703 .685 .83 r
.46386 .57187 .46706 .57656 .47277 .57387 .46958 .56908 Metetra
.701 .68 .827 r
.46958 .56908 .47277 .57387 .47849 .57122 .4753 .5663 Metetra
.698 .675 .824 r
.4753 .5663 .47849 .57122 .48421 .56859 .48103 .56356 Metetra
.695 .67 .821 r
.48103 .56356 .48421 .56859 .48994 .56601 .48677 .56085 Metetra
.692 .664 .818 r
.48677 .56085 .48994 .56601 .49568 .56348 .49251 .55818 Metetra
.688 .658 .816 r
.49251 .55818 .49568 .56348 .50143 .56102 .49827 .55558 Metetra
.683 .652 .813 r
.49827 .55558 .50143 .56102 .50719 .55865 .50403 .55305 Metetra
.678 .645 .81 r
.50403 .55305 .50719 .55865 .51296 .55637 .50981 .55063 Metetra
.671 .638 .807 r
.50981 .55063 .51296 .55637 .51874 .55423 .51559 .54832 Metetra
.664 .629 .805 r
.51559 .54832 .51874 .55423 .52452 .55225 .52138 .54618 Metetra
.655 .621 .803 r
.52138 .54618 .52452 .55225 .53033 .55045 .52719 .54423 Metetra
.643 .611 .802 r
.52719 .54423 .53033 .55045 .53614 .54889 .533 .54253 Metetra
.63 .602 .802 r
.533 .54253 .53614 .54889 .54197 .54759 .53883 .54112 Metetra
.615 .591 .803 r
.53883 .54112 .54197 .54759 .54781 .5466 .54468 .54006 Metetra
.596 .58 .806 r
.54468 .54006 .54781 .5466 .55367 .54594 .55054 .53941 Metetra
.575 .569 .81 r
.55054 .53941 .55367 .54594 .55955 .54565 .55642 .53918 Metetra
.551 .559 .815 r
.55642 .53918 .55955 .54565 .56546 .54573 .56233 .5394 Metetra
.524 .549 .824 r
.56233 .5394 .56546 .54573 .57139 .54616 .56826 .54005 Metetra
.495 .541 .834 r
.56826 .54005 .57139 .54616 .57734 .54693 .57422 .54109 Metetra
.463 .535 .845 r
.57422 .54109 .57734 .54693 .58332 .548 .58021 .54248 Metetra
.429 .53 .858 r
.58021 .54248 .58332 .548 .58933 .54931 .58623 .54416 Metetra
.393 .526 .872 r
.58623 .54416 .58933 .54931 .59537 .55083 .59227 .54607 Metetra
.352 .523 .886 r
.59227 .54607 .59537 .55083 .60143 .55251 .59835 .54815 Metetra
.308 .521 .899 r
.59835 .54815 .60143 .55251 .60753 .55431 .60446 .55036 Metetra
.26 .519 .909 r
.60446 .55036 .60753 .55431 .61365 .55618 .61059 .55265 Metetra
.208 .516 .914 r
.61059 .55265 .61365 .55618 .61979 .55808 .61676 .55497 Metetra
.152 .512 .913 r
.61676 .55497 .61979 .55808 .62597 .55997 .62295 .55728 Metetra
.094 .506 .904 r
.62295 .55728 .62597 .55997 .63217 .5618 .62917 .55953 Metetra
.037 .499 .884 r
.62917 .55953 .63217 .5618 .63839 .56353 .63541 .56167 Metetra
0 .492 .854 r
.63541 .56167 .63839 .56353 .64463 .5651 .64168 .56365 Metetra
0 .487 .817 r
.64168 .56365 .64463 .5651 .6509 .56648 .64797 .56541 Metetra
0 .488 .779 r
.64797 .56541 .6509 .56648 .65717 .56761 .65427 .56689 Metetra
0 .498 .746 r
.65427 .56689 .65717 .56761 .66346 .56844 .66058 .56803 Metetra
0 .523 .726 r
.66058 .56803 .66346 .56844 .66976 .56893 .66691 .56875 Metetra
0 .562 .722 r
.66691 .56875 .66976 .56893 .67607 .56905 .67323 .56903 Metetra
.046 .614 .736 r
.67323 .56903 .67607 .56905 .68238 .56878 .67956 .56884 Metetra
.126 .675 .765 r
.67956 .56884 .68238 .56878 .68868 .56813 .68588 .56819 Metetra
.215 .739 .805 r
.68588 .56819 .68868 .56813 .69499 .56712 .6922 .56712 Metetra
.306 .799 .848 r
.6922 .56712 .69499 .56712 .7013 .56578 .69852 .56568 Metetra
.39 .848 .886 r
.69852 .56568 .7013 .56578 .70761 .56416 .70484 .56391 Metetra
.464 .884 .918 r
.70484 .56391 .70761 .56416 .71393 .56228 .71116 .56187 Metetra
.525 .909 .941 r
.71116 .56187 .71393 .56228 .72024 .56019 .71748 .55962 Metetra
.573 .922 .957 r
.71748 .55962 .72024 .56019 .72655 .55793 .7238 .55718 Metetra
.611 .929 .966 r
.7238 .55718 .72655 .55793 .73287 .55553 .73013 .55461 Metetra
.639 .929 .971 r
.73013 .55461 .73287 .55553 .7392 .553 .73646 .55192 Metetra
.66 .927 .973 r
.73646 .55192 .7392 .553 .74553 .55037 .7428 .54915 Metetra
.677 .922 .973 r
.7428 .54915 .74553 .55037 .75187 .54767 .74914 .5463 Metetra
.689 .916 .971 r
.74914 .5463 .75187 .54767 .75821 .5449 .75549 .54339 Metetra
.699 .909 .969 r
.75549 .54339 .75821 .5449 .76456 .54207 .76184 .54043 Metetra
.706 .902 .966 r
.76184 .54043 .76456 .54207 .77092 .53919 .76821 .53744 Metetra
.711 .895 .963 r
.76821 .53744 .77092 .53919 .77728 .53629 .77458 .53441 Metetra
.715 .888 .96 r
.77458 .53441 .77728 .53629 .78366 .53335 .78096 .53136 Metetra
.718 .881 .956 r
.78096 .53136 .78366 .53335 .79004 .53039 .78735 .5283 Metetra
.72 .874 .953 r
.78735 .5283 .79004 .53039 .79643 .5274 .79375 .52522 Metetra
.722 .868 .95 r
.79375 .52522 .79643 .5274 .80284 .52441 .80016 .52212 Metetra
.723 .862 .947 r
.80016 .52212 .80284 .52441 .80925 .5214 .80658 .51903 Metetra
.724 .856 .944 r
.80658 .51903 .80925 .5214 .81567 .51838 .813 .51592 Metetra
.725 .85 .941 r
.813 .51592 .81567 .51838 .8221 .51535 .81944 .5128 Metetra
.725 .845 .938 r
.81944 .5128 .8221 .51535 .82854 .51231 .82589 .50969 Metetra
.725 .84 .935 r
.82589 .50969 .82854 .51231 .83499 .50927 .83235 .50656 Metetra
.725 .835 .933 r
.83235 .50656 .83499 .50927 .84145 .50622 .83882 .50344 Metetra
.725 .83 .93 r
.83882 .50344 .84145 .50622 .84792 .50317 .84529 .50031 Metetra
.725 .826 .927 r
.84529 .50031 .84792 .50317 .85441 .50012 .85178 .49719 Metetra
.725 .821 .925 r
.85178 .49719 .85441 .50012 .8609 .49706 .85828 .49406 Metetra
.724 .817 .922 r
.85828 .49406 .8609 .49706 .8674 .494 .86479 .49093 Metetra
.724 .813 .92 r
.86479 .49093 .8674 .494 .87391 .49093 .87131 .4878 Metetra
.724 .809 .918 r
.87131 .4878 .87391 .49093 .88044 .48786 .87784 .48466 Metetra
.723 .805 .915 r
.87784 .48466 .88044 .48786 .88697 .48479 .88438 .48152 Metetra
.723 .801 .913 r
.88438 .48152 .88697 .48479 .89352 .48171 .89094 .47838 Metetra
.722 .798 .911 r
.89094 .47838 .89352 .48171 .90008 .47864 .8975 .47524 Metetra
.722 .794 .909 r
.8975 .47524 .90008 .47864 .90664 .47555 .90407 .4721 Metetra
.722 .791 .906 r
.90407 .4721 .90664 .47555 .91322 .47247 .91065 .46895 Metetra
.721 .787 .904 r
.91065 .46895 .91322 .47247 .91981 .46938 .91725 .4658 Metetra
.721 .784 .902 r
.91725 .4658 .91981 .46938 .92641 .46629 .92385 .46265 Metetra
.72 .78 .9 r
.92385 .46265 .92641 .46629 .93302 .4632 .93047 .45949 Metetra
.724 .764 .885 r
.32569 .63552 .32913 .63853 .33465 .63581 .33122 .63275 Metetra
.724 .762 .883 r
.33122 .63275 .33465 .63581 .34018 .63309 .33676 .62998 Metetra
.723 .759 .881 r
.33676 .62998 .34018 .63309 .34572 .63037 .34231 .6272 Metetra
.723 .756 .878 r
.34231 .6272 .34572 .63037 .35126 .62764 .34787 .62441 Metetra
.722 .753 .876 r
.34787 .62441 .35126 .62764 .35682 .62491 .35343 .62162 Metetra
.722 .75 .874 r
.35343 .62162 .35682 .62491 .36238 .62217 .359 .61882 Metetra
.721 .747 .872 r
.359 .61882 .36238 .62217 .36795 .61942 .36458 .61601 Metetra
.721 .744 .87 r
.36458 .61601 .36795 .61942 .37353 .61667 .37017 .6132 Metetra
.72 .741 .868 r
.37017 .6132 .37353 .61667 .37911 .61391 .37576 .61037 Metetra
.72 .738 .866 r
.37576 .61037 .37911 .61391 .3847 .61114 .38136 .60755 Metetra
.719 .735 .863 r
.38136 .60755 .3847 .61114 .39031 .60837 .38698 .60471 Metetra
.719 .732 .861 r
.38698 .60471 .39031 .60837 .39592 .60559 .3926 .60187 Metetra
.718 .729 .859 r
.3926 .60187 .39592 .60559 .40153 .6028 .39822 .59901 Metetra
.718 .726 .857 r
.39822 .59901 .40153 .6028 .40716 .60001 .40386 .59615 Metetra
.717 .723 .854 r
.40386 .59615 .40716 .60001 .41279 .59721 .4095 .59328 Metetra
.716 .719 .852 r
.4095 .59328 .41279 .59721 .41843 .5944 .41515 .5904 Metetra
.716 .716 .849 r
.41515 .5904 .41843 .5944 .42408 .59159 .42081 .58751 Metetra
.715 .713 .847 r
.42081 .58751 .42408 .59159 .42974 .58877 .42648 .58461 Metetra
.714 .709 .844 r
.42648 .58461 .42974 .58877 .43541 .58595 .43215 .58171 Metetra
.713 .706 .842 r
.43215 .58171 .43541 .58595 .44108 .58313 .43784 .5788 Metetra
.712 .702 .839 r
.43784 .5788 .44108 .58313 .44677 .58031 .44353 .57589 Metetra
.711 .698 .836 r
.44353 .57589 .44677 .58031 .45246 .57749 .44923 .57298 Metetra
.71 .694 .833 r
.44923 .57298 .45246 .57749 .45815 .57468 .45493 .57006 Metetra
.708 .69 .83 r
.45493 .57006 .45815 .57468 .46386 .57187 .46065 .56715 Metetra
.707 .686 .827 r
.46065 .56715 .46386 .57187 .46958 .56908 .46637 .56424 Metetra
.705 .681 .824 r
.46637 .56424 .46958 .56908 .4753 .5663 .4721 .56135 Metetra
.702 .676 .821 r
.4721 .56135 .4753 .5663 .48103 .56356 .47784 .55847 Metetra
.7 .67 .818 r
.47784 .55847 .48103 .56356 .48677 .56085 .48359 .55562 Metetra
.697 .664 .815 r
.48359 .55562 .48677 .56085 .49251 .55818 .48934 .55281 Metetra
.693 .658 .811 r
.48934 .55281 .49251 .55818 .49827 .55558 .4951 .55004 Metetra
.689 .651 .808 r
.4951 .55004 .49827 .55558 .50403 .55305 .50087 .54735 Metetra
.683 .644 .804 r
.50087 .54735 .50403 .55305 .50981 .55063 .50665 .54473 Metetra
.677 .636 .801 r
.50665 .54473 .50981 .55063 .51559 .54832 .51244 .54224 Metetra
.67 .627 .798 r
.51244 .54224 .51559 .54832 .52138 .54618 .51823 .53989 Metetra
.66 .618 .795 r
.51823 .53989 .52138 .54618 .52719 .54423 .52404 .53775 Metetra
.649 .607 .793 r
.52404 .53775 .52719 .54423 .533 .54253 .52986 .53586 Metetra
.635 .595 .792 r
.52986 .53586 .533 .54253 .53883 .54112 .53569 .53429 Metetra
.618 .583 .792 r
.53569 .53429 .53883 .54112 .54468 .54006 .54153 .53312 Metetra
.598 .57 .794 r
.54153 .53312 .54468 .54006 .55054 .53941 .5474 .53242 Metetra
.573 .556 .797 r
.5474 .53242 .55054 .53941 .55642 .53918 .55328 .53225 Metetra
.545 .543 .803 r
.55328 .53225 .55642 .53918 .56233 .5394 .55919 .53262 Metetra
.514 .531 .811 r
.55919 .53262 .56233 .5394 .56826 .54005 .56513 .53353 Metetra
.479 .52 .821 r
.56513 .53353 .56826 .54005 .57422 .54109 .57109 .53491 Metetra
.443 .511 .834 r
.57109 .53491 .57422 .54109 .58021 .54248 .57709 .53668 Metetra
.403 .504 .847 r
.57709 .53668 .58021 .54248 .58623 .54416 .58311 .53878 Metetra
.36 .497 .861 r
.58311 .53878 .58623 .54416 .59227 .54607 .58917 .54113 Metetra
.312 .491 .874 r
.58917 .54113 .59227 .54607 .59835 .54815 .59526 .54367 Metetra
.259 .485 .886 r
.59526 .54367 .59835 .54815 .60446 .55036 .60138 .54633 Metetra
.2 .478 .893 r
.60138 .54633 .60446 .55036 .61059 .55265 .60753 .54908 Metetra
.136 .469 .893 r
.60753 .54908 .61059 .55265 .61676 .55497 .61371 .55187 Metetra
.066 .456 .883 r
.61371 .55187 .61676 .55497 .62295 .55728 .61992 .55464 Metetra
0 .44 .86 r
.61992 .55464 .62295 .55728 .62917 .55953 .62616 .55736 Metetra
0 .421 .823 r
.62616 .55736 .62917 .55953 .63541 .56167 .63243 .55998 Metetra
0 .4 .772 r
.63243 .55998 .63541 .56167 .64168 .56365 .63872 .56242 Metetra
0 .383 .713 r
.63872 .56242 .64168 .56365 .64797 .56541 .64503 .56463 Metetra
.222 0 0 r
.64503 .56463 .64797 .56541 .65427 .56689 .65136 .56652 Metetra
.225 0 0 r
.65136 .56652 .65427 .56689 .66058 .56803 .6577 .56801 Metetra
.201 0 0 r
.6577 .56801 .66058 .56803 .66691 .56875 .66405 .56902 Metetra
.15 0 0 r
.66405 .56902 .66691 .56875 .67323 .56903 .6704 .56948 Metetra
.072 0 0 r
.6704 .56948 .67323 .56903 .67956 .56884 .67674 .56936 Metetra
.67674 .56936 .67956 .56884 .68588 .56819 .68308 .56868 Metetra
.139 .681 .706 r
.68308 .56868 .68588 .56819 .6922 .56712 .68942 .56749 Metetra
.254 .764 .773 r
.68942 .56749 .6922 .56712 .69852 .56568 .69574 .56588 Metetra
.36 .832 .835 r
.69574 .56588 .69852 .56568 .70484 .56391 .70207 .56392 Metetra
.45 .882 .884 r
.70207 .56392 .70484 .56391 .71116 .56187 .70839 .56167 Metetra
.522 .914 .92 r
.70839 .56167 .71116 .56187 .71748 .55962 .71472 .5592 Metetra
.577 .931 .943 r
.71472 .5592 .71748 .55962 .7238 .55718 .72105 .55657 Metetra
.619 .938 .957 r
.72105 .55657 .7238 .55718 .73013 .55461 .72738 .5538 Metetra
.649 .939 .964 r
.72738 .5538 .73013 .55461 .73646 .55192 .73371 .55093 Metetra
.671 .935 .968 r
.73371 .55093 .73646 .55192 .7428 .54915 .74005 .54798 Metetra
.687 .929 .969 r
.74005 .54798 .7428 .54915 .74914 .5463 .7464 .54497 Metetra
.699 .922 .968 r
.7464 .54497 .74914 .5463 .75549 .54339 .75275 .54191 Metetra
.708 .914 .966 r
.75275 .54191 .75549 .54339 .76184 .54043 .75912 .53881 Metetra
.714 .906 .963 r
.75912 .53881 .76184 .54043 .76821 .53744 .76549 .53569 Metetra
.719 .898 .96 r
.76549 .53569 .76821 .53744 .77458 .53441 .77186 .53255 Metetra
.723 .891 .957 r
.77186 .53255 .77458 .53441 .78096 .53136 .77825 .52938 Metetra
.725 .883 .954 r
.77825 .52938 .78096 .53136 .78735 .5283 .78465 .52621 Metetra
.727 .877 .951 r
.78465 .52621 .78735 .5283 .79375 .52522 .79105 .52302 Metetra
.728 .87 .947 r
.79105 .52302 .79375 .52522 .80016 .52212 .79747 .51984 Metetra
.729 .864 .945 r
.79747 .51984 .80016 .52212 .80658 .51903 .80389 .51664 Metetra
.729 .858 .942 r
.80389 .51664 .80658 .51903 .813 .51592 .81033 .51345 Metetra
.73 .852 .939 r
.81033 .51345 .813 .51592 .81944 .5128 .81677 .51025 Metetra
.73 .846 .936 r
.81677 .51025 .81944 .5128 .82589 .50969 .82323 .50705 Metetra
.73 .841 .933 r
.82323 .50705 .82589 .50969 .83235 .50656 .82969 .50385 Metetra
.729 .836 .931 r
.82969 .50385 .83235 .50656 .83882 .50344 .83617 .50065 Metetra
.729 .831 .928 r
.83617 .50065 .83882 .50344 .84529 .50031 .84265 .49744 Metetra
.729 .827 .926 r
.84265 .49744 .84529 .50031 .85178 .49719 .84915 .49425 Metetra
.728 .822 .923 r
.84915 .49425 .85178 .49719 .85828 .49406 .85565 .49105 Metetra
.728 .818 .921 r
.85565 .49105 .85828 .49406 .86479 .49093 .86217 .48785 Metetra
.727 .814 .918 r
.86217 .48785 .86479 .49093 .87131 .4878 .8687 .48465 Metetra
.727 .81 .916 r
.8687 .48465 .87131 .4878 .87784 .48466 .87523 .48144 Metetra
.726 .806 .914 r
.87523 .48144 .87784 .48466 .88438 .48152 .88178 .47824 Metetra
.726 .802 .912 r
.88178 .47824 .88438 .48152 .89094 .47838 .88834 .47504 Metetra
.725 .798 .909 r
.88834 .47504 .89094 .47838 .8975 .47524 .89491 .47183 Metetra
.725 .795 .907 r
.89491 .47183 .8975 .47524 .90407 .4721 .90149 .46862 Metetra
.724 .791 .905 r
.90149 .46862 .90407 .4721 .91065 .46895 .90808 .46541 Metetra
.724 .788 .903 r
.90808 .46541 .91065 .46895 .91725 .4658 .91468 .4622 Metetra
.723 .784 .901 r
.91468 .4622 .91725 .4658 .92385 .46265 .92129 .45898 Metetra
.723 .781 .899 r
.92129 .45898 .92385 .46265 .93047 .45949 .92791 .45576 Metetra
.727 .765 .883 r
.32224 .6325 .32569 .63552 .33122 .63275 .32778 .62968 Metetra
.726 .762 .881 r
.32778 .62968 .33122 .63275 .33676 .62998 .33333 .62685 Metetra
.726 .759 .879 r
.33333 .62685 .33676 .62998 .34231 .6272 .33889 .62401 Metetra
.725 .756 .877 r
.33889 .62401 .34231 .6272 .34787 .62441 .34445 .62117 Metetra
.725 .753 .875 r
.34445 .62117 .34787 .62441 .35343 .62162 .35003 .61832 Metetra
.724 .75 .873 r
.35003 .61832 .35343 .62162 .359 .61882 .35561 .61545 Metetra
.724 .747 .871 r
.35561 .61545 .359 .61882 .36458 .61601 .3612 .61259 Metetra
.723 .745 .869 r
.3612 .61259 .36458 .61601 .37017 .6132 .3668 .60972 Metetra
.723 .742 .867 r
.3668 .60972 .37017 .6132 .37576 .61037 .3724 .60683 Metetra
.722 .739 .864 r
.3724 .60683 .37576 .61037 .38136 .60755 .37801 .60394 Metetra
.722 .736 .862 r
.37801 .60394 .38136 .60755 .38698 .60471 .38363 .60104 Metetra
.721 .733 .86 r
.38363 .60104 .38698 .60471 .3926 .60187 .38926 .59813 Metetra
.721 .73 .858 r
.38926 .59813 .3926 .60187 .39822 .59901 .3949 .59521 Metetra
.72 .727 .855 r
.3949 .59521 .39822 .59901 .40386 .59615 .40055 .59228 Metetra
.72 .724 .853 r
.40055 .59228 .40386 .59615 .4095 .59328 .4062 .58934 Metetra
.719 .721 .85 r
.4062 .58934 .4095 .59328 .41515 .5904 .41186 .58638 Metetra
.719 .717 .848 r
.41186 .58638 .41515 .5904 .42081 .58751 .41753 .58342 Metetra
.718 .714 .845 r
.41753 .58342 .42081 .58751 .42648 .58461 .4232 .58044 Metetra
.717 .71 .843 r
.4232 .58044 .42648 .58461 .43215 .58171 .42889 .57746 Metetra
.717 .707 .84 r
.42889 .57746 .43215 .58171 .43784 .5788 .43458 .57446 Metetra
.716 .703 .837 r
.43458 .57446 .43784 .5788 .44353 .57589 .44028 .57146 Metetra
.715 .699 .834 r
.44028 .57146 .44353 .57589 .44923 .57298 .44599 .56844 Metetra
.713 .695 .831 r
.44599 .56844 .44923 .57298 .45493 .57006 .4517 .56542 Metetra
.712 .691 .828 r
.4517 .56542 .45493 .57006 .46065 .56715 .45743 .5624 Metetra
.711 .686 .825 r
.45743 .5624 .46065 .56715 .46637 .56424 .46316 .55938 Metetra
.709 .682 .822 r
.46316 .55938 .46637 .56424 .4721 .56135 .4689 .55635 Metetra
.707 .676 .818 r
.4689 .55635 .4721 .56135 .47784 .55847 .47464 .55334 Metetra
.704 .671 .814 r
.47464 .55334 .47784 .55847 .48359 .55562 .4804 .55034 Metetra
.702 .665 .811 r
.4804 .55034 .48359 .55562 .48934 .55281 .48616 .54736 Metetra
.698 .659 .807 r
.48616 .54736 .48934 .55281 .4951 .55004 .49192 .54442 Metetra
.694 .652 .803 r
.49192 .54442 .4951 .55004 .50087 .54735 .4977 .54153 Metetra
.69 .644 .799 r
.4977 .54153 .50087 .54735 .50665 .54473 .50348 .53871 Metetra
.684 .635 .795 r
.50348 .53871 .50665 .54473 .51244 .54224 .50928 .53598 Metetra
.677 .626 .791 r
.50928 .53598 .51244 .54224 .51823 .53989 .51508 .5334 Metetra
.668 .616 .787 r
.51508 .5334 .51823 .53989 .52404 .53775 .52089 .53099 Metetra
.656 .604 .784 r
.52089 .53099 .52404 .53775 .52986 .53586 .52671 .52885 Metetra
.642 .591 .782 r
.52671 .52885 .52986 .53586 .53569 .53429 .53254 .52705 Metetra
.624 .576 .78 r
.53254 .52705 .53569 .53429 .54153 .53312 .53839 .52569 Metetra
.601 .56 .781 r
.53839 .52569 .54153 .53312 .5474 .53242 .54425 .5249 Metetra
.573 .543 .783 r
.54425 .5249 .5474 .53242 .55328 .53225 .55013 .52475 Metetra
.539 .525 .788 r
.55013 .52475 .55328 .53225 .55919 .53262 .55604 .52531 Metetra
.502 .509 .797 r
.55604 .52531 .55919 .53262 .56513 .53353 .56198 .52653 Metetra
.462 .495 .807 r
.56198 .52653 .56513 .53353 .57109 .53491 .56795 .52831 Metetra
.419 .484 .82 r
.56795 .52831 .57109 .53491 .57709 .53668 .57395 .53055 Metetra
.372 .473 .834 r
.57395 .53055 .57709 .53668 .58311 .53878 .57998 .53314 Metetra
.321 .464 .848 r
.57998 .53314 .58311 .53878 .58917 .54113 .58605 .536 Metetra
.265 .455 .86 r
.58605 .536 .58917 .54113 .59526 .54367 .59215 .53904 Metetra
.202 .444 .869 r
.59215 .53904 .59526 .54367 .60138 .54633 .59828 .54222 Metetra
.132 .431 .871 r
.59828 .54222 .60138 .54633 .60753 .54908 .60445 .54548 Metetra
.054 .413 .863 r
.60445 .54548 .60753 .54908 .61371 .55187 .61065 .54877 Metetra
0 .39 .84 r
.61065 .54877 .61371 .55187 .61992 .55464 .61688 .55206 Metetra
0 .361 .8 r
.61688 .55206 .61992 .55464 .62616 .55736 .62314 .55531 Metetra
0 .327 .74 r
.62314 .55531 .62616 .55736 .63243 .55998 .62943 .55845 Metetra
0 .291 .664 r
.62943 .55845 .63243 .55998 .63872 .56242 .63575 .56143 Metetra
.334 0 0 r
.63575 .56143 .63872 .56242 .64503 .56463 .64209 .56417 Metetra
.364 0 0 r
.64209 .56417 .64503 .56463 .65136 .56652 .64845 .56656 Metetra
.365 0 0 r
.64845 .56656 .65136 .56652 .6577 .56801 .65482 .56849 Metetra
.337 0 0 r
.65482 .56849 .6577 .56801 .66405 .56902 .6612 .56985 Metetra
.277 0 0 r
.6612 .56985 .66405 .56902 .6704 .56948 .66757 .57051 Metetra
.187 0 0 r
.66757 .57051 .6704 .56948 .67674 .56936 .67394 .57044 Metetra
.071 0 0 r
.67394 .57044 .67674 .56936 .68308 .56868 .68029 .56967 Metetra
.68029 .56967 .68308 .56868 .68942 .56749 .68663 .56829 Metetra
.203 .72 .684 r
.68663 .56829 .68942 .56749 .69574 .56588 .69296 .56643 Metetra
.333 .812 .773 r
.69296 .56643 .69574 .56588 .70207 .56392 .69929 .5642 Metetra
.442 .878 .845 r
.69929 .5642 .70207 .56392 .70839 .56167 .70562 .56168 Metetra
.525 .919 .895 r
.70562 .56168 .70839 .56167 .71472 .5592 .71195 .55896 Metetra
.587 .94 .927 r
.71195 .55896 .71472 .5592 .72105 .55657 .71828 .55608 Metetra
.631 .948 .946 r
.71828 .55608 .72105 .55657 .72738 .5538 .72461 .55308 Metetra
.662 .947 .957 r
.72461 .55308 .72738 .5538 .73371 .55093 .73095 .55001 Metetra
.684 .943 .962 r
.73095 .55001 .73371 .55093 .74005 .54798 .7373 .54687 Metetra
.699 .935 .964 r
.7373 .54687 .74005 .54798 .7464 .54497 .74365 .54368 Metetra
.71 .927 .963 r
.74365 .54368 .7464 .54497 .75275 .54191 .75001 .54046 Metetra
.718 .918 .962 r
.75001 .54046 .75275 .54191 .75912 .53881 .75638 .53722 Metetra
.723 .91 .959 r
.75638 .53722 .75912 .53881 .76549 .53569 .76275 .53396 Metetra
.727 .902 .957 r
.76275 .53396 .76549 .53569 .77186 .53255 .76914 .53068 Metetra
.73 .893 .954 r
.76914 .53068 .77186 .53255 .77825 .52938 .77553 .5274 Metetra
.732 .886 .951 r
.77553 .5274 .77825 .52938 .78465 .52621 .78193 .52411 Metetra
.733 .878 .948 r
.78193 .52411 .78465 .52621 .79105 .52302 .78834 .52082 Metetra
.734 .871 .945 r
.78834 .52082 .79105 .52302 .79747 .51984 .79477 .51753 Metetra
.734 .865 .942 r
.79477 .51753 .79747 .51984 .80389 .51664 .8012 .51424 Metetra
.734 .859 .939 r
.8012 .51424 .80389 .51664 .81033 .51345 .80764 .51096 Metetra
.734 .853 .936 r
.80764 .51096 .81033 .51345 .81677 .51025 .81409 .50767 Metetra
.734 .847 .934 r
.81409 .50767 .81677 .51025 .82323 .50705 .82055 .50439 Metetra
.734 .842 .931 r
.82055 .50439 .82323 .50705 .82969 .50385 .82702 .50111 Metetra
.733 .837 .929 r
.82702 .50111 .82969 .50385 .83617 .50065 .8335 .49783 Metetra
.733 .832 .926 r
.8335 .49783 .83617 .50065 .84265 .49744 .84 .49455 Metetra
.732 .827 .924 r
.84 .49455 .84265 .49744 .84915 .49425 .8465 .49128 Metetra
.732 .823 .921 r
.8465 .49128 .84915 .49425 .85565 .49105 .85301 .48801 Metetra
.731 .819 .919 r
.85301 .48801 .85565 .49105 .86217 .48785 .85954 .48474 Metetra
.73 .814 .917 r
.85954 .48474 .86217 .48785 .8687 .48465 .86607 .48148 Metetra
.73 .811 .915 r
.86607 .48148 .8687 .48465 .87523 .48144 .87261 .47821 Metetra
.729 .806 .912 r
.87261 .47821 .87523 .48144 .88178 .47824 .87917 .47494 Metetra
.729 .803 .91 r
.87917 .47494 .88178 .47824 .88834 .47504 .88573 .47167 Metetra
.728 .799 .908 r
.88573 .47167 .88834 .47504 .89491 .47183 .89231 .4684 Metetra
.728 .796 .906 r
.89231 .4684 .89491 .47183 .90149 .46862 .89889 .46513 Metetra
.727 .792 .904 r
.89889 .46513 .90149 .46862 .90808 .46541 .90549 .46186 Metetra
.726 .789 .902 r
.90549 .46186 .90808 .46541 .91468 .4622 .9121 .45858 Metetra
.726 .785 .899 r
.9121 .45858 .91468 .4622 .92129 .45898 .91871 .4553 Metetra
.725 .782 .897 r
.91871 .4553 .92129 .45898 .92791 .45576 .92534 .45202 Metetra
.729 .765 .882 r
.31877 .62948 .32224 .6325 .32778 .62968 .32432 .62659 Metetra
.728 .762 .88 r
.32432 .62659 .32778 .62968 .33333 .62685 .32988 .62371 Metetra
.728 .759 .878 r
.32988 .62371 .33333 .62685 .33889 .62401 .33545 .62081 Metetra
.727 .756 .876 r
.33545 .62081 .33889 .62401 .34445 .62117 .34103 .61791 Metetra
.727 .754 .874 r
.34103 .61791 .34445 .62117 .35003 .61832 .34661 .615 Metetra
.726 .751 .872 r
.34661 .615 .35003 .61832 .35561 .61545 .3522 .61209 Metetra
.726 .748 .87 r
.3522 .61209 .35561 .61545 .3612 .61259 .3578 .60916 Metetra
.726 .745 .867 r
.3578 .60916 .3612 .61259 .3668 .60972 .36341 .60623 Metetra
.725 .742 .865 r
.36341 .60623 .3668 .60972 .3724 .60683 .36903 .60328 Metetra
.725 .739 .863 r
.36903 .60328 .3724 .60683 .37801 .60394 .37465 .60033 Metetra
.724 .737 .861 r
.37465 .60033 .37801 .60394 .38363 .60104 .38028 .59737 Metetra
.724 .734 .859 r
.38028 .59737 .38363 .60104 .38926 .59813 .38592 .5944 Metetra
.724 .731 .856 r
.38592 .5944 .38926 .59813 .3949 .59521 .39157 .59141 Metetra
.723 .728 .854 r
.39157 .59141 .3949 .59521 .40055 .59228 .39722 .58841 Metetra
.723 .725 .852 r
.39722 .58841 .40055 .59228 .4062 .58934 .40289 .58539 Metetra
.722 .721 .849 r
.40289 .58539 .4062 .58934 .41186 .58638 .40856 .58236 Metetra
.722 .718 .847 r
.40856 .58236 .41186 .58638 .41753 .58342 .41423 .57932 Metetra
.721 .715 .844 r
.41423 .57932 .41753 .58342 .4232 .58044 .41992 .57627 Metetra
.721 .712 .841 r
.41992 .57627 .4232 .58044 .42889 .57746 .42561 .5732 Metetra
.72 .708 .838 r
.42561 .5732 .42889 .57746 .43458 .57446 .43131 .57011 Metetra
.719 .704 .835 r
.43131 .57011 .43458 .57446 .44028 .57146 .43702 .56701 Metetra
.718 .7 .832 r
.43702 .56701 .44028 .57146 .44599 .56844 .44274 .5639 Metetra
.717 .696 .829 r
.44274 .5639 .44599 .56844 .4517 .56542 .44846 .56077 Metetra
.716 .692 .826 r
.44846 .56077 .4517 .56542 .45743 .5624 .4542 .55764 Metetra
.715 .688 .823 r
.4542 .55764 .45743 .5624 .46316 .55938 .45994 .55449 Metetra
.713 .683 .819 r
.45994 .55449 .46316 .55938 .4689 .55635 .46568 .55133 Metetra
.712 .678 .815 r
.46568 .55133 .4689 .55635 .47464 .55334 .47144 .54817 Metetra
.71 .672 .811 r
.47144 .54817 .47464 .55334 .4804 .55034 .4772 .54502 Metetra
.707 .666 .807 r
.4772 .54502 .4804 .55034 .48616 .54736 .48297 .54187 Metetra
.704 .66 .803 r
.48297 .54187 .48616 .54736 .49192 .54442 .48874 .53874 Metetra
.701 .652 .798 r
.48874 .53874 .49192 .54442 .4977 .54153 .49452 .53563 Metetra
.697 .645 .794 r
.49452 .53563 .4977 .54153 .50348 .53871 .50031 .53257 Metetra
.691 .636 .789 r
.50031 .53257 .50348 .53871 .50928 .53598 .50611 .52958 Metetra
.685 .626 .784 r
.50611 .52958 .50928 .53598 .51508 .5334 .51192 .52669 Metetra
.677 .615 .779 r
.51192 .52669 .51508 .5334 .52089 .53099 .51773 .52397 Metetra
.666 .602 .774 r
.51773 .52397 .52089 .53099 .52671 .52885 .52355 .52148 Metetra
.651 .587 .77 r
.52355 .52148 .52671 .52885 .53254 .52705 .52938 .51934 Metetra
.632 .571 .767 r
.52938 .51934 .53254 .52705 .53839 .52569 .53523 .51769 Metetra
.607 .551 .766 r
.53523 .51769 .53839 .52569 .54425 .5249 .54109 .51671 Metetra
.574 .529 .767 r
.54109 .51671 .54425 .5249 .55013 .52475 .54697 .51656 Metetra
.533 .506 .772 r
.54697 .51656 .55013 .52475 .55604 .52531 .55288 .51733 Metetra
.488 .485 .78 r
.55288 .51733 .55604 .52531 .56198 .52653 .55882 .51895 Metetra
.44 .467 .791 r
.55882 .51895 .56198 .52653 .56795 .52831 .56479 .52125 Metetra
.389 .452 .805 r
.56479 .52125 .56795 .52831 .57395 .53055 .5708 .52407 Metetra
.335 .439 .819 r
.5708 .52407 .57395 .53055 .57998 .53314 .57684 .52724 Metetra
.276 .427 .832 r
.57684 .52724 .57998 .53314 .58605 .536 .58292 .53068 Metetra
.211 .413 .842 r
.58292 .53068 .58605 .536 .59215 .53904 .58903 .53429 Metetra
.138 .397 .847 r
.58903 .53429 .59215 .53904 .59828 .54222 .59518 .53803 Metetra
.056 .376 .842 r
.59518 .53803 .59828 .54222 .60445 .54548 .60136 .54184 Metetra
0 .349 .823 r
.60136 .54184 .60445 .54548 .61065 .54877 .60758 .54569 Metetra
0 .314 .784 r
.60758 .54569 .61065 .54877 .61688 .55206 .61383 .54954 Metetra
0 .271 .723 r
.61383 .54954 .61688 .55206 .62314 .55531 .62011 .55336 Metetra
0 .222 .64 r
.62011 .55336 .62314 .55531 .62943 .55845 .62643 .5571 Metetra
.409 0 0 r
.62643 .5571 .62943 .55845 .63575 .56143 .63278 .56069 Metetra
.463 0 0 r
.63278 .56069 .63575 .56143 .64209 .56417 .63915 .56405 Metetra
.49 0 0 r
.63915 .56405 .64209 .56417 .64845 .56656 .64554 .56707 Metetra
.487 0 0 r
.64554 .56707 .64845 .56656 .65482 .56849 .65195 .56957 Metetra
.454 0 0 r
.65195 .56957 .65482 .56849 .6612 .56985 .65836 .57137 Metetra
.388 0 0 r
.65836 .57137 .6612 .56985 .66757 .57051 .66476 .57229 Metetra
.288 0 0 r
.66476 .57229 .66757 .57051 .67394 .57044 .67114 .57223 Metetra
.156 0 0 r
.67114 .57223 .67394 .57044 .68029 .56967 .6775 .57126 Metetra
.001 0 0 r
.6775 .57126 .68029 .56967 .68663 .56829 .68385 .56958 Metetra
.68385 .56958 .68663 .56829 .69296 .56643 .69018 .56736 Metetra
.317 .793 .708 r
.69018 .56736 .69296 .56643 .69929 .5642 .69651 .56477 Metetra
.443 .876 .804 r
.69651 .56477 .69929 .5642 .70562 .56168 .70284 .5619 Metetra
.537 .926 .87 r
.70284 .5619 .70562 .56168 .71195 .55896 .70917 .55886 Metetra
.604 .949 .912 r
.70917 .55886 .71195 .55896 .71828 .55608 .7155 .55569 Metetra
.648 .957 .936 r
.7155 .55569 .71828 .55608 .72461 .55308 .72184 .55244 Metetra
.679 .956 .949 r
.72184 .55244 .72461 .55308 .73095 .55001 .72818 .54913 Metetra
.699 .949 .956 r
.72818 .54913 .73095 .55001 .7373 .54687 .73453 .54579 Metetra
.713 .941 .958 r
.73453 .54579 .7373 .54687 .74365 .54368 .74088 .54241 Metetra
.722 .931 .959 r
.74088 .54241 .74365 .54368 .75001 .54046 .74725 .53902 Metetra
.729 .922 .957 r
.74725 .53902 .75001 .54046 .75638 .53722 .75362 .53562 Metetra
.733 .913 .955 r
.75362 .53562 .75638 .53722 .76275 .53396 .76 .53222 Metetra
.736 .903 .953 r
.76 .53222 .76275 .53396 .76914 .53068 .76639 .52881 Metetra
.738 .895 .95 r
.76639 .52881 .76914 .53068 .77553 .5274 .77279 .5254 Metetra
.739 .887 .947 r
.77279 .5254 .77553 .5274 .78193 .52411 .7792 .522 Metetra
.74 .879 .945 r
.7792 .522 .78193 .52411 .78834 .52082 .78562 .5186 Metetra
.74 .872 .942 r
.78562 .5186 .78834 .52082 .79477 .51753 .79205 .51521 Metetra
.74 .866 .939 r
.79205 .51521 .79477 .51753 .8012 .51424 .79849 .51183 Metetra
.739 .859 .936 r
.79849 .51183 .8012 .51424 .80764 .51096 .80494 .50845 Metetra
.739 .853 .934 r
.80494 .50845 .80764 .51096 .81409 .50767 .81139 .50508 Metetra
.738 .848 .931 r
.81139 .50508 .81409 .50767 .82055 .50439 .81786 .50171 Metetra
.738 .842 .929 r
.81786 .50171 .82055 .50439 .82702 .50111 .82434 .49835 Metetra
.737 .837 .926 r
.82434 .49835 .82702 .50111 .8335 .49783 .83083 .49499 Metetra
.736 .832 .924 r
.83083 .49499 .8335 .49783 .84 .49455 .83733 .49164 Metetra
.736 .828 .922 r
.83733 .49164 .84 .49455 .8465 .49128 .84384 .4883 Metetra
.735 .823 .919 r
.84384 .4883 .8465 .49128 .85301 .48801 .85036 .48496 Metetra
.734 .819 .917 r
.85036 .48496 .85301 .48801 .85954 .48474 .85689 .48162 Metetra
.734 .815 .915 r
.85689 .48162 .85954 .48474 .86607 .48148 .86343 .47829 Metetra
.733 .811 .913 r
.86343 .47829 .86607 .48148 .87261 .47821 .86998 .47495 Metetra
.732 .807 .911 r
.86998 .47495 .87261 .47821 .87917 .47494 .87654 .47162 Metetra
.731 .803 .909 r
.87654 .47162 .87917 .47494 .88573 .47167 .88311 .46829 Metetra
.731 .8 .907 r
.88311 .46829 .88573 .47167 .89231 .4684 .8897 .46495 Metetra
.73 .796 .904 r
.8897 .46495 .89231 .4684 .89889 .46513 .89629 .46162 Metetra
.73 .793 .902 r
.89629 .46162 .89889 .46513 .90549 .46186 .90289 .45829 Metetra
.729 .789 .9 r
.90289 .45829 .90549 .46186 .9121 .45858 .9095 .45495 Metetra
.729 .786 .898 r
.9095 .45495 .9121 .45858 .91871 .4553 .91613 .45161 Metetra
.728 .783 .896 r
.91613 .45161 .91871 .4553 .92534 .45202 .92276 .44826 Metetra
.731 .765 .88 r
.31528 .62644 .31877 .62948 .32432 .62659 .32085 .6235 Metetra
.731 .763 .878 r
.32085 .6235 .32432 .62659 .32988 .62371 .32642 .62055 Metetra
.73 .76 .876 r
.32642 .62055 .32988 .62371 .33545 .62081 .332 .61761 Metetra
.73 .757 .874 r
.332 .61761 .33545 .62081 .34103 .61791 .33759 .61465 Metetra
.729 .754 .872 r
.33759 .61465 .34103 .61791 .34661 .615 .34318 .61168 Metetra
.729 .751 .87 r
.34318 .61168 .34661 .615 .3522 .61209 .34879 .60871 Metetra
.728 .748 .868 r
.34879 .60871 .3522 .61209 .3578 .60916 .3544 .60573 Metetra
.728 .746 .866 r
.3544 .60573 .3578 .60916 .36341 .60623 .36002 .60273 Metetra
.728 .743 .864 r
.36002 .60273 .36341 .60623 .36903 .60328 .36564 .59973 Metetra
.727 .74 .862 r
.36564 .59973 .36903 .60328 .37465 .60033 .37128 .59672 Metetra
.727 .737 .86 r
.37128 .59672 .37465 .60033 .38028 .59737 .37692 .59369 Metetra
.726 .735 .858 r
.37692 .59369 .38028 .59737 .38592 .5944 .38257 .59065 Metetra
.726 .732 .855 r
.38257 .59065 .38592 .5944 .39157 .59141 .38822 .5876 Metetra
.726 .729 .853 r
.38822 .5876 .39157 .59141 .39722 .58841 .39389 .58453 Metetra
.725 .726 .85 r
.39389 .58453 .39722 .58841 .40289 .58539 .39956 .58144 Metetra
.725 .723 .848 r
.39956 .58144 .40289 .58539 .40856 .58236 .40524 .57834 Metetra
.724 .719 .845 r
.40524 .57834 .40856 .58236 .41423 .57932 .41093 .57522 Metetra
.724 .716 .843 r
.41093 .57522 .41423 .57932 .41992 .57627 .41662 .57208 Metetra
.724 .713 .84 r
.41662 .57208 .41992 .57627 .42561 .5732 .42233 .56893 Metetra
.723 .709 .837 r
.42233 .56893 .42561 .5732 .43131 .57011 .42804 .56576 Metetra
.722 .706 .834 r
.42804 .56576 .43131 .57011 .43702 .56701 .43376 .56256 Metetra
.722 .702 .831 r
.43376 .56256 .43702 .56701 .44274 .5639 .43948 .55935 Metetra
.721 .698 .827 r
.43948 .55935 .44274 .5639 .44846 .56077 .44521 .55612 Metetra
.72 .694 .824 r
.44521 .55612 .44846 .56077 .4542 .55764 .45096 .55286 Metetra
.719 .689 .821 r
.45096 .55286 .4542 .55764 .45994 .55449 .4567 .54959 Metetra
.718 .685 .817 r
.4567 .54959 .45994 .55449 .46568 .55133 .46246 .5463 Metetra
.717 .679 .813 r
.46246 .5463 .46568 .55133 .47144 .54817 .46822 .54299 Metetra
.715 .674 .809 r
.46822 .54299 .47144 .54817 .4772 .54502 .47399 .53967 Metetra
.713 .668 .804 r
.47399 .53967 .4772 .54502 .48297 .54187 .47977 .53634 Metetra
.711 .661 .799 r
.47977 .53634 .48297 .54187 .48874 .53874 .48555 .533 Metetra
.708 .654 .794 r
.48555 .533 .48874 .53874 .49452 .53563 .49134 .52966 Metetra
.704 .646 .789 r
.49134 .52966 .49452 .53563 .50031 .53257 .49714 .52634 Metetra
.7 .637 .783 r
.49714 .52634 .50031 .53257 .50611 .52958 .50294 .52305 Metetra
.694 .627 .777 r
.50294 .52305 .50611 .52958 .51192 .52669 .50875 .51981 Metetra
.687 .616 .771 r
.50875 .51981 .51192 .52669 .51773 .52397 .51457 .51669 Metetra
.677 .602 .765 r
.51457 .51669 .51773 .52397 .52355 .52148 .52039 .51376 Metetra
.664 .587 .759 r
.52039 .51376 .52355 .52148 .52938 .51934 .52622 .51113 Metetra
.645 .567 .753 r
.52622 .51113 .52938 .51934 .53523 .51769 .53207 .50901 Metetra
.617 .544 .75 r
.53207 .50901 .53523 .51769 .54109 .51671 .53793 .50767 Metetra
.577 .516 .749 r
.53793 .50767 .54109 .51671 .54697 .51656 .5438 .50743 Metetra
.525 .484 .753 r
.5438 .50743 .54697 .51656 .55288 .51733 .54971 .5085 Metetra
.468 .455 .761 r
.54971 .5085 .55288 .51733 .55882 .51895 .55565 .5107 Metetra
.411 .433 .773 r
.55565 .5107 .55882 .51895 .56479 .52125 .56163 .51369 Metetra
.352 .415 .787 r
.56163 .51369 .56479 .52125 .5708 .52407 .56764 .51722 Metetra
.29 .399 .802 r
.56764 .51722 .5708 .52407 .57684 .52724 .57369 .52109 Metetra
.223 .384 .814 r
.57369 .52109 .57684 .52724 .58292 .53068 .57978 .52519 Metetra
.148 .366 .821 r
.57978 .52519 .58292 .53068 .58903 .53429 .5859 .52943 Metetra
.065 .343 .819 r
.5859 .52943 .58903 .53429 .59518 .53803 .59206 .53377 Metetra
0 .315 .805 r
.59206 .53377 .59518 .53803 .60136 .54184 .59826 .53817 Metetra
0 .278 .772 r
.59826 .53817 .60136 .54184 .60758 .54569 .60449 .54261 Metetra
0 .231 .717 r
.60449 .54261 .60758 .54569 .61383 .54954 .61076 .54706 Metetra
0 .176 .636 r
.61076 .54706 .61383 .54954 .62011 .55336 .61707 .5515 Metetra
.445 0 0 r
.61707 .5515 .62011 .55336 .62643 .5571 .62341 .5559 Metetra
.521 0 0 r
.62341 .5559 .62643 .5571 .63278 .56069 .62979 .56019 Metetra
.57 0 0 r
.62979 .56019 .63278 .56069 .63915 .56405 .6362 .56431 Metetra
.591 .025 0 r
.6362 .56431 .63915 .56405 .64554 .56707 .64263 .5681 Metetra
.584 .037 0 r
.64263 .5681 .64554 .56707 .65195 .56957 .64908 .57137 Metetra
.548 .02 0 r
.64908 .57137 .65195 .56957 .65836 .57137 .65553 .57381 Metetra
.477 0 0 r
.65553 .57381 .65836 .57137 .66476 .57229 .66196 .57507 Metetra
.363 0 0 r
.66196 .57507 .66476 .57229 .67114 .57223 .66836 .57493 Metetra
.217 0 0 r
.66836 .57493 .67114 .57223 .6775 .57126 .67473 .57359 Metetra
.044 0 0 r
.67473 .57359 .6775 .57126 .68385 .56958 .68107 .57141 Metetra
.68107 .57141 .68385 .56958 .69018 .56736 .6874 .56867 Metetra
.319 .78 .646 r
.6874 .56867 .69018 .56736 .69651 .56477 .69373 .56559 Metetra
.461 .88 .769 r
.69373 .56559 .69651 .56477 .70284 .5619 .70005 .5623 Metetra
.561 .936 .85 r
.70005 .5623 .70284 .5619 .70917 .55886 .70638 .55888 Metetra
.628 .96 .899 r
.70638 .55888 .70917 .55886 .7155 .55569 .71271 .55539 Metetra
.671 .966 .927 r
.71271 .55539 .7155 .55569 .72184 .55244 .71905 .55185 Metetra
.699 .963 .942 r
.71905 .55185 .72184 .55244 .72818 .54913 .72539 .54829 Metetra
.716 .955 .949 r
.72539 .54829 .72818 .54913 .73453 .54579 .73175 .54472 Metetra
.728 .945 .953 r
.73175 .54472 .73453 .54579 .74088 .54241 .73811 .54115 Metetra
.735 .934 .953 r
.73811 .54115 .74088 .54241 .74725 .53902 .74447 .53758 Metetra
.74 .924 .952 r
.74447 .53758 .74725 .53902 .75362 .53562 .75085 .53402 Metetra
.743 .914 .951 r
.75085 .53402 .75362 .53562 .76 .53222 .75724 .53047 Metetra
.745 .905 .949 r
.75724 .53047 .76 .53222 .76639 .52881 .76364 .52692 Metetra
.746 .896 .946 r
.76364 .52692 .76639 .52881 .77279 .5254 .77004 .52339 Metetra
.746 .888 .944 r
.77004 .52339 .77279 .5254 .7792 .522 .77646 .51987 Metetra
.746 .88 .941 r
.77646 .51987 .7792 .522 .78562 .5186 .78288 .51637 Metetra
.746 .873 .939 r
.78288 .51637 .78562 .5186 .79205 .51521 .78932 .51287 Metetra
.745 .866 .936 r
.78932 .51287 .79205 .51521 .79849 .51183 .79576 .50939 Metetra
.744 .86 .934 r
.79576 .50939 .79849 .51183 .80494 .50845 .80222 .50592 Metetra
.743 .853 .931 r
.80222 .50592 .80494 .50845 .81139 .50508 .80868 .50246 Metetra
.743 .848 .929 r
.80868 .50246 .81139 .50508 .81786 .50171 .81516 .49901 Metetra
.742 .842 .926 r
.81516 .49901 .81786 .50171 .82434 .49835 .82165 .49557 Metetra
.741 .838 .924 r
.82165 .49557 .82434 .49835 .83083 .49499 .82814 .49214 Metetra
.74 .833 .922 r
.82814 .49214 .83083 .49499 .83733 .49164 .83465 .48871 Metetra
.739 .828 .92 r
.83465 .48871 .83733 .49164 .84384 .4883 .84116 .48529 Metetra
.738 .824 .917 r
.84116 .48529 .84384 .4883 .85036 .48496 .84769 .48188 Metetra
.737 .819 .915 r
.84769 .48188 .85036 .48496 .85689 .48162 .85423 .47848 Metetra
.736 .815 .913 r
.85423 .47848 .85689 .48162 .86343 .47829 .86078 .47508 Metetra
.736 .811 .911 r
.86078 .47508 .86343 .47829 .86998 .47495 .86734 .47168 Metetra
.735 .807 .909 r
.86734 .47168 .86998 .47495 .87654 .47162 .8739 .46828 Metetra
.734 .804 .907 r
.8739 .46828 .87654 .47162 .88311 .46829 .88048 .46488 Metetra
.733 .8 .905 r
.88048 .46488 .88311 .46829 .8897 .46495 .88707 .46149 Metetra
.733 .797 .903 r
.88707 .46149 .8897 .46495 .89629 .46162 .89367 .45809 Metetra
.732 .793 .901 r
.89367 .45809 .89629 .46162 .90289 .45829 .90028 .4547 Metetra
.732 .79 .899 r
.90028 .4547 .90289 .45829 .9095 .45495 .9069 .4513 Metetra
.731 .787 .897 r
.9069 .4513 .9095 .45495 .91613 .45161 .91353 .4479 Metetra
.73 .784 .895 r
.91353 .4479 .91613 .45161 .92276 .44826 .92017 .44449 Metetra
.733 .766 .879 r
.31179 .62339 .31528 .62644 .32085 .6235 .31737 .62039 Metetra
.733 .763 .877 r
.31737 .62039 .32085 .6235 .32642 .62055 .32295 .6174 Metetra
.733 .76 .875 r
.32295 .6174 .32642 .62055 .332 .61761 .32854 .61439 Metetra
.732 .757 .873 r
.32854 .61439 .332 .61761 .33759 .61465 .33414 .61138 Metetra
.732 .755 .871 r
.33414 .61138 .33759 .61465 .34318 .61168 .33974 .60835 Metetra
.731 .752 .869 r
.33974 .60835 .34318 .61168 .34879 .60871 .34536 .60533 Metetra
.731 .75 .867 r
.34536 .60533 .34879 .60871 .3544 .60573 .35098 .60228 Metetra
.73 .746 .865 r
.35098 .60228 .3544 .60573 .36002 .60273 .35661 .59923 Metetra
.73 .744 .863 r
.35661 .59923 .36002 .60273 .36564 .59973 .36224 .59617 Metetra
.729 .741 .861 r
.36224 .59617 .36564 .59973 .37128 .59672 .36789 .5931 Metetra
.729 .738 .859 r
.36789 .5931 .37128 .59672 .37692 .59369 .37354 .59001 Metetra
.729 .736 .856 r
.37354 .59001 .37692 .59369 .38257 .59065 .3792 .58691 Metetra
.729 .733 .854 r
.3792 .58691 .38257 .59065 .38822 .5876 .38487 .58379 Metetra
.728 .73 .852 r
.38487 .58379 .38822 .5876 .39389 .58453 .39054 .58065 Metetra
.728 .727 .849 r
.39054 .58065 .39389 .58453 .39956 .58144 .39622 .57749 Metetra
.728 .724 .847 r
.39622 .57749 .39956 .58144 .40524 .57834 .40191 .57432 Metetra
.727 .721 .844 r
.40191 .57432 .40524 .57834 .41093 .57522 .40761 .57112 Metetra
.727 .718 .841 r
.40761 .57112 .41093 .57522 .41662 .57208 .41332 .56791 Metetra
.727 .714 .839 r
.41332 .56791 .41662 .57208 .42233 .56893 .41903 .56467 Metetra
.726 .711 .836 r
.41903 .56467 .42233 .56893 .42804 .56576 .42475 .5614 Metetra
.726 .707 .833 r
.42475 .5614 .42804 .56576 .43376 .56256 .43048 .55811 Metetra
.725 .703 .829 r
.43048 .55811 .43376 .56256 .43948 .55935 .43621 .5548 Metetra
.725 .7 .826 r
.43621 .5548 .43948 .55935 .44521 .55612 .44195 .55146 Metetra
.724 .695 .822 r
.44195 .55146 .44521 .55612 .45096 .55286 .4477 .54809 Metetra
.723 .691 .819 r
.4477 .54809 .45096 .55286 .4567 .54959 .45346 .54469 Metetra
.722 .686 .815 r
.45346 .54469 .4567 .54959 .46246 .5463 .45923 .54127 Metetra
.722 .682 .811 r
.45923 .54127 .46246 .5463 .46822 .54299 .465 .53781 Metetra
.72 .676 .806 r
.465 .53781 .46822 .54299 .47399 .53967 .47078 .53432 Metetra
.719 .67 .802 r
.47078 .53432 .47399 .53967 .47977 .53634 .47656 .5308 Metetra
.717 .664 .796 r
.47656 .5308 .47977 .53634 .48555 .533 .48235 .52725 Metetra
.715 .657 .791 r
.48235 .52725 .48555 .533 .49134 .52966 .48815 .52367 Metetra
.713 .649 .785 r
.48815 .52367 .49134 .52966 .49714 .52634 .49395 .52006 Metetra
.709 .64 .778 r
.49395 .52006 .49714 .52634 .50294 .52305 .49976 .51644 Metetra
.705 .631 .771 r
.49976 .51644 .50294 .52305 .50875 .51981 .50558 .51281 Metetra
.699 .619 .763 r
.50558 .51281 .50875 .51981 .51457 .51669 .5114 .50921 Metetra
.691 .605 .755 r
.5114 .50921 .51457 .51669 .52039 .51376 .51723 .50571 Metetra
.68 .589 .747 r
.51723 .50571 .52039 .51376 .52622 .51113 .52306 .50241 Metetra
.662 .568 .739 r
.52306 .50241 .52622 .51113 .53207 .50901 .5289 .49955 Metetra
.634 .541 .732 r
.5289 .49955 .53207 .50901 .53793 .50767 .53476 .49753 Metetra
.587 .505 .729 r
.53476 .49753 .53793 .50767 .5438 .50743 .54063 .49699 Metetra
.512 .455 .73 r
.54063 .49699 .5438 .50743 .54971 .5085 .54653 .49857 Metetra
.44 .418 .738 r
.54653 .49857 .54971 .5085 .55565 .5107 .55247 .50164 Metetra
.371 .391 .752 r
.55247 .50164 .55565 .5107 .56163 .51369 .55845 .5056 Metetra
.303 .371 .767 r
.55845 .5056 .56163 .51369 .56764 .51722 .56447 .51003 Metetra
.232 .353 .781 r
.56447 .51003 .56764 .51722 .57369 .52109 .57053 .51472 Metetra
.157 .334 .791 r
.57053 .51472 .57369 .52109 .57978 .52519 .57662 .51956 Metetra
.074 .312 .793 r
.57662 .51956 .57978 .52519 .5859 .52943 .58276 .52448 Metetra
0 .284 .784 r
.58276 .52448 .5859 .52943 .59206 .53377 .58893 .52945 Metetra
0 .248 .759 r
.58893 .52945 .59206 .53377 .59826 .53817 .59514 .53447 Metetra
0 .202 .713 r
.59514 .53447 .59826 .53817 .60449 .54261 .60139 .53953 Metetra
0 .147 .642 r
.60139 .53953 .60449 .54261 .61076 .54706 .60769 .54461 Metetra
.448 0 0 r
.60769 .54461 .61076 .54706 .61707 .5515 .61402 .54971 Metetra
.541 0 0 r
.61402 .54971 .61707 .5515 .62341 .5559 .62039 .55482 Metetra
.609 .044 0 r
.62039 .55482 .62341 .5559 .62979 .56019 .62679 .5599 Metetra
.652 .096 0 r
.62679 .5599 .62979 .56019 .6362 .56431 .63324 .5649 Metetra
.668 .134 0 r
.63324 .5649 .6362 .56431 .64263 .5681 .63972 .56969 Metetra
.659 .151 .025 r
.63972 .56969 .64263 .5681 .64908 .57137 .64622 .57402 Metetra
.623 .141 .088 r
.64622 .57402 .64908 .57137 .65553 .57381 .65272 .57745 Metetra
.546 .086 .112 r
.65272 .57745 .65553 .57381 .66196 .57507 .6592 .57929 Metetra
.408 0 .074 r
.6592 .57929 .66196 .57507 .66836 .57493 .66561 .57885 Metetra
.244 0 0 r
.66561 .57885 .66836 .57493 .67473 .57359 .67197 .57679 Metetra
.052 0 0 r
.67197 .57679 .67473 .57359 .68107 .57141 .6783 .5738 Metetra
.6783 .5738 .68107 .57141 .6874 .56867 .68462 .57033 Metetra
.68462 .57033 .6874 .56867 .69373 .56559 .69093 .56663 Metetra
.5 .895 .746 r
.69093 .56663 .69373 .56559 .70005 .5623 .69725 .56282 Metetra
.599 .95 .837 r
.69725 .56282 .70005 .5623 .70638 .55888 .70358 .55898 Metetra
.661 .971 .89 r
.70358 .55898 .70638 .55888 .71271 .55539 .70991 .55512 Metetra
.698 .974 .919 r
.70991 .55512 .71271 .55539 .71905 .55185 .71624 .55127 Metetra
.721 .968 .935 r
.71624 .55127 .71905 .55185 .72539 .54829 .72259 .54745 Metetra
.735 .958 .943 r
.72259 .54745 .72539 .54829 .73175 .54472 .72895 .54365 Metetra
.743 .947 .947 r
.72895 .54365 .73175 .54472 .73811 .54115 .73531 .53987 Metetra
.748 .936 .948 r
.73531 .53987 .73811 .54115 .74447 .53758 .74168 .53612 Metetra
.751 .925 .947 r
.74168 .53612 .74447 .53758 .75085 .53402 .74807 .53239 Metetra
.753 .915 .946 r
.74807 .53239 .75085 .53402 .75724 .53047 .75446 .52869 Metetra
.753 .905 .944 r
.75446 .52869 .75724 .53047 .76364 .52692 .76086 .52501 Metetra
.753 .896 .942 r
.76086 .52501 .76364 .52692 .77004 .52339 .76728 .52136 Metetra
.753 .888 .94 r
.76728 .52136 .77004 .52339 .77646 .51987 .7737 .51772 Metetra
.752 .88 .937 r
.7737 .51772 .77646 .51987 .78288 .51637 .78013 .5141 Metetra
.751 .872 .935 r
.78013 .5141 .78288 .51637 .78932 .51287 .78657 .51051 Metetra
.75 .866 .933 r
.78657 .51051 .78932 .51287 .79576 .50939 .79302 .50693 Metetra
.749 .859 .93 r
.79302 .50693 .79576 .50939 .80222 .50592 .79949 .50336 Metetra
.748 .854 .928 r
.79949 .50336 .80222 .50592 .80868 .50246 .80596 .49982 Metetra
.747 .848 .926 r
.80596 .49982 .80868 .50246 .81516 .49901 .81244 .49629 Metetra
.746 .843 .924 r
.81244 .49629 .81516 .49901 .82165 .49557 .81894 .49276 Metetra
.744 .837 .922 r
.81894 .49276 .82165 .49557 .82814 .49214 .82544 .48925 Metetra
.743 .833 .92 r
.82544 .48925 .82814 .49214 .83465 .48871 .83195 .48575 Metetra
.742 .828 .917 r
.83195 .48575 .83465 .48871 .84116 .48529 .83848 .48227 Metetra
.741 .824 .916 r
.83848 .48227 .84116 .48529 .84769 .48188 .84501 .47879 Metetra
.74 .82 .913 r
.84501 .47879 .84769 .48188 .85423 .47848 .85156 .47531 Metetra
.739 .816 .911 r
.85156 .47531 .85423 .47848 .86078 .47508 .85811 .47184 Metetra
.738 .812 .909 r
.85811 .47184 .86078 .47508 .86734 .47168 .86468 .46838 Metetra
.738 .808 .907 r
.86468 .46838 .86734 .47168 .8739 .46828 .87125 .46492 Metetra
.737 .804 .905 r
.87125 .46492 .8739 .46828 .88048 .46488 .87784 .46146 Metetra
.736 .801 .904 r
.87784 .46146 .88048 .46488 .88707 .46149 .88443 .458 Metetra
.735 .797 .902 r
.88443 .458 .88707 .46149 .89367 .45809 .89104 .45455 Metetra
.735 .794 .9 r
.89104 .45455 .89367 .45809 .90028 .4547 .89766 .45109 Metetra
.734 .79 .898 r
.89766 .45109 .90028 .4547 .9069 .4513 .90428 .44763 Metetra
.733 .788 .896 r
.90428 .44763 .9069 .4513 .91353 .4479 .91092 .44417 Metetra
.733 .784 .894 r
.91092 .44417 .91353 .4479 .92017 .44449 .91756 .4407 Metetra
.736 .766 .877 r
.30828 .62033 .31179 .62339 .31737 .62039 .31387 .61728 Metetra
.735 .763 .875 r
.31387 .61728 .31737 .62039 .32295 .6174 .31946 .61422 Metetra
.734 .76 .874 r
.31946 .61422 .32295 .6174 .32854 .61439 .32506 .61117 Metetra
.734 .758 .872 r
.32506 .61117 .32854 .61439 .33414 .61138 .33067 .6081 Metetra
.733 .755 .87 r
.33067 .6081 .33414 .61138 .33974 .60835 .33629 .60502 Metetra
.733 .753 .868 r
.33629 .60502 .33974 .60835 .34536 .60533 .34191 .60193 Metetra
.733 .75 .866 r
.34191 .60193 .34536 .60533 .35098 .60228 .34755 .59884 Metetra
.733 .747 .864 r
.34755 .59884 .35098 .60228 .35661 .59923 .35319 .59573 Metetra
.732 .745 .862 r
.35319 .59573 .35661 .59923 .36224 .59617 .35883 .59261 Metetra
.732 .742 .86 r
.35883 .59261 .36224 .59617 .36789 .5931 .36449 .58947 Metetra
.732 .739 .858 r
.36449 .58947 .36789 .5931 .37354 .59001 .37015 .58632 Metetra
.731 .737 .855 r
.37015 .58632 .37354 .59001 .3792 .58691 .37582 .58316 Metetra
.731 .734 .853 r
.37582 .58316 .3792 .58691 .38487 .58379 .3815 .57998 Metetra
.731 .731 .851 r
.3815 .57998 .38487 .58379 .39054 .58065 .38718 .57677 Metetra
.731 .728 .848 r
.38718 .57677 .39054 .58065 .39622 .57749 .39287 .57354 Metetra
.731 .725 .846 r
.39287 .57354 .39622 .57749 .40191 .57432 .39857 .57029 Metetra
.73 .722 .843 r
.39857 .57029 .40191 .57432 .40761 .57112 .40428 .56702 Metetra
.73 .719 .84 r
.40428 .56702 .40761 .57112 .41332 .56791 .41 .56373 Metetra
.73 .716 .837 r
.41 .56373 .41332 .56791 .41903 .56467 .41572 .56041 Metetra
.73 .713 .834 r
.41572 .56041 .41903 .56467 .42475 .5614 .42145 .55705 Metetra
.729 .709 .831 r
.42145 .55705 .42475 .5614 .43048 .55811 .42719 .55367 Metetra
.729 .706 .828 r
.42719 .55367 .43048 .55811 .43621 .5548 .43293 .55026 Metetra
.729 .701 .825 r
.43293 .55026 .43621 .5548 .44195 .55146 .43868 .54681 Metetra
.728 .698 .821 r
.43868 .54681 .44195 .55146 .4477 .54809 .44444 .54333 Metetra
.728 .693 .817 r
.44444 .54333 .4477 .54809 .45346 .54469 .45021 .53981 Metetra
.727 .689 .813 r
.45021 .53981 .45346 .54469 .45923 .54127 .45598 .53624 Metetra
.727 .684 .809 r
.45598 .53624 .45923 .54127 .465 .53781 .46176 .53263 Metetra
.726 .679 .804 r
.46176 .53263 .465 .53781 .47078 .53432 .46755 .52898 Metetra
.725 .673 .799 r
.46755 .52898 .47078 .53432 .47656 .5308 .47334 .52527 Metetra
.724 .667 .794 r
.47334 .52527 .47656 .5308 .48235 .52725 .47914 .52151 Metetra
.723 .661 .788 r
.47914 .52151 .48235 .52725 .48815 .52367 .48495 .51768 Metetra
.721 .653 .781 r
.48495 .51768 .48815 .52367 .49395 .52006 .49076 .51378 Metetra
.719 .645 .774 r
.49076 .51378 .49395 .52006 .49976 .51644 .49658 .50981 Metetra
.716 .635 .766 r
.49658 .50981 .49976 .51644 .50558 .51281 .5024 .50576 Metetra
.713 .625 .757 r
.5024 .50576 .50558 .51281 .5114 .50921 .50823 .50163 Metetra
.708 .612 .747 r
.50823 .50163 .5114 .50921 .51723 .50571 .51406 .49744 Metetra
.7 .596 .736 r
.51406 .49744 .51723 .50571 .52306 .50241 .5199 .49325 Metetra
.687 .575 .724 r
.5199 .49325 .52306 .50241 .5289 .49955 .52574 .48924 Metetra
.662 .547 .713 r
.52574 .48924 .5289 .49955 .53476 .49753 .53159 .4859 Metetra
.609 .501 .706 r
.53159 .4859 .53476 .49753 .54063 .49699 .53745 .48447 Metetra
.486 .414 .701 r
.53745 .48447 .54063 .49699 .54653 .49857 .54334 .48713 Metetra
.391 .365 .71 r
.54334 .48713 .54653 .49857 .55247 .50164 .54928 .49173 Metetra
.311 .337 .727 r
.54928 .49173 .55247 .50164 .55845 .5056 .55527 .49704 Metetra
.234 .316 .744 r
.55527 .49704 .55845 .5056 .56447 .51003 .56129 .50261 Metetra
.157 .297 .757 r
.56129 .50261 .56447 .51003 .57053 .51472 .56736 .50823 Metetra
.076 .276 .763 r
.56736 .50823 .57053 .51472 .57662 .51956 .57346 .51386 Metetra
0 .251 .759 r
.57346 .51386 .57662 .51956 .58276 .52448 .5796 .51949 Metetra
0 .218 .741 r
.5796 .51949 .58276 .52448 .58893 .52945 .58579 .52512 Metetra
0 .177 .706 r
.58579 .52512 .58893 .52945 .59514 .53447 .59202 .53076 Metetra
0 .127 .649 r
.59202 .53076 .59514 .53447 .60139 .53953 .59828 .53643 Metetra
0 .067 .567 r
.59828 .53643 .60139 .53953 .60769 .54461 .60459 .54215 Metetra
.53 0 0 r
.60459 .54215 .60769 .54461 .61402 .54971 .61095 .54793 Metetra
.614 .064 0 r
.61095 .54793 .61402 .54971 .62039 .55482 .61734 .55379 Metetra
.675 .127 0 r
.61734 .55379 .62039 .55482 .62679 .5599 .62378 .55973 Metetra
.712 .18 0 r
.62378 .55973 .62679 .5599 .63324 .5649 .63027 .56574 Metetra
.727 .222 .028 r
.63027 .56574 .63324 .5649 .63972 .56969 .6368 .57177 Metetra
.72 .247 .125 r
.6368 .57177 .63972 .56969 .64622 .57402 .64337 .57762 Metetra
.687 .247 .199 r
.64337 .57762 .64622 .57402 .65272 .57745 .64995 .58275 Metetra
.611 .2 .239 r
.64995 .58275 .65272 .57745 .6592 .57929 .6565 .58583 Metetra
.415 .032 .202 r
.6565 .58583 .6592 .57929 .66561 .57885 .6629 .58443 Metetra
.225 0 .07 r
.6629 .58443 .66561 .57885 .67197 .57679 .66923 .58093 Metetra
.01 0 0 r
.66923 .58093 .67197 .57679 .6783 .5738 .67553 .57669 Metetra
.67553 .57669 .6783 .5738 .68462 .57033 .68182 .57224 Metetra
.423 .817 .592 r
.68182 .57224 .68462 .57033 .69093 .56663 .68813 .56778 Metetra
.565 .922 .744 r
.68813 .56778 .69093 .56663 .69725 .56282 .69444 .56339 Metetra
.651 .969 .835 r
.69444 .56339 .69725 .56282 .70358 .55898 .70076 .55907 Metetra
.701 .982 .886 r
.70076 .55907 .70358 .55898 .70991 .55512 .70708 .55483 Metetra
.729 .98 .913 r
.70708 .55483 .70991 .55512 .71624 .55127 .71342 .55067 Metetra
.745 .971 .929 r
.71342 .55067 .71624 .55127 .72259 .54745 .71977 .54657 Metetra
.754 .96 .937 r
.71977 .54657 .72259 .54745 .72895 .54365 .72613 .54254 Metetra
.759 .948 .94 r
.72613 .54254 .72895 .54365 .73531 .53987 .7325 .53856 Metetra
.761 .936 .942 r
.7325 .53856 .73531 .53987 .74168 .53612 .73888 .53462 Metetra
.762 .925 .942 r
.73888 .53462 .74168 .53612 .74807 .53239 .74527 .53073 Metetra
.762 .914 .941 r
.74527 .53073 .74807 .53239 .75446 .52869 .75167 .52688 Metetra
.762 .905 .939 r
.75167 .52688 .75446 .52869 .76086 .52501 .75807 .52306 Metetra
.76 .895 .938 r
.75807 .52306 .76086 .52501 .76728 .52136 .76449 .51928 Metetra
.759 .887 .936 r
.76449 .51928 .76728 .52136 .7737 .51772 .77092 .51553 Metetra
.758 .879 .934 r
.77092 .51553 .7737 .51772 .78013 .5141 .77736 .51181 Metetra
.757 .872 .931 r
.77736 .51181 .78013 .5141 .78657 .51051 .78381 .5081 Metetra
.755 .865 .929 r
.78381 .5081 .78657 .51051 .79302 .50693 .79027 .50443 Metetra
.754 .859 .927 r
.79027 .50443 .79302 .50693 .79949 .50336 .79674 .50078 Metetra
.752 .853 .925 r
.79674 .50078 .79949 .50336 .80596 .49982 .80322 .49714 Metetra
.751 .848 .923 r
.80322 .49714 .80596 .49982 .81244 .49629 .80971 .49353 Metetra
.749 .842 .921 r
.80971 .49353 .81244 .49629 .81894 .49276 .81621 .48993 Metetra
.748 .837 .919 r
.81621 .48993 .81894 .49276 .82544 .48925 .82272 .48634 Metetra
.747 .832 .917 r
.82272 .48634 .82544 .48925 .83195 .48575 .82924 .48277 Metetra
.745 .828 .915 r
.82924 .48277 .83195 .48575 .83848 .48227 .83578 .47921 Metetra
.744 .824 .913 r
.83578 .47921 .83848 .48227 .84501 .47879 .84232 .47566 Metetra
.743 .819 .911 r
.84232 .47566 .84501 .47879 .85156 .47531 .84887 .47212 Metetra
.742 .816 .91 r
.84887 .47212 .85156 .47531 .85811 .47184 .85543 .46859 Metetra
.741 .812 .908 r
.85543 .46859 .85811 .47184 .86468 .46838 .86201 .46506 Metetra
.74 .808 .906 r
.86201 .46506 .86468 .46838 .87125 .46492 .86859 .46154 Metetra
.739 .805 .904 r
.86859 .46154 .87125 .46492 .87784 .46146 .87518 .45801 Metetra
.738 .801 .902 r
.87518 .45801 .87784 .46146 .88443 .458 .88179 .4545 Metetra
.738 .798 .9 r
.88179 .4545 .88443 .458 .89104 .45455 .8884 .45097 Metetra
.737 .794 .898 r
.8884 .45097 .89104 .45455 .89766 .45109 .89502 .44746 Metetra
.736 .791 .897 r
.89502 .44746 .89766 .45109 .90428 .44763 .90165 .44394 Metetra
.736 .788 .895 r
.90165 .44394 .90428 .44763 .91092 .44417 .9083 .44042 Metetra
.735 .785 .893 r
.9083 .44042 .91092 .44417 .91756 .4407 .91495 .4369 Metetra
.738 .766 .876 r
.30476 .61725 .30828 .62033 .31387 .61728 .31036 .61415 Metetra
.737 .763 .874 r
.31036 .61415 .31387 .61728 .31946 .61422 .31596 .61104 Metetra
.737 .761 .872 r
.31596 .61104 .31946 .61422 .32506 .61117 .32158 .60793 Metetra
.736 .758 .871 r
.32158 .60793 .32506 .61117 .33067 .6081 .3272 .60481 Metetra
.736 .756 .869 r
.3272 .60481 .33067 .6081 .33629 .60502 .33282 .60168 Metetra
.735 .753 .867 r
.33282 .60168 .33629 .60502 .34191 .60193 .33846 .59854 Metetra
.735 .751 .865 r
.33846 .59854 .34191 .60193 .34755 .59884 .3441 .59538 Metetra
.735 .748 .863 r
.3441 .59538 .34755 .59884 .35319 .59573 .34975 .59222 Metetra
.734 .746 .861 r
.34975 .59222 .35319 .59573 .35883 .59261 .35541 .58904 Metetra
.734 .743 .859 r
.35541 .58904 .35883 .59261 .36449 .58947 .36107 .58585 Metetra
.734 .74 .857 r
.36107 .58585 .36449 .58947 .37015 .58632 .36675 .58264 Metetra
.734 .738 .855 r
.36675 .58264 .37015 .58632 .37582 .58316 .37243 .57941 Metetra
.734 .735 .852 r
.37243 .57941 .37582 .58316 .3815 .57998 .37811 .57616 Metetra
.733 .732 .85 r
.37811 .57616 .3815 .57998 .38718 .57677 .38381 .57289 Metetra
.733 .73 .847 r
.38381 .57289 .38718 .57677 .39287 .57354 .38951 .5696 Metetra
.733 .726 .845 r
.38951 .5696 .39287 .57354 .39857 .57029 .39522 .56628 Metetra
.733 .724 .842 r
.39522 .56628 .39857 .57029 .40428 .56702 .40094 .56293 Metetra
.733 .721 .839 r
.40094 .56293 .40428 .56702 .41 .56373 .40667 .55956 Metetra
.733 .718 .837 r
.40667 .55956 .41 .56373 .41572 .56041 .4124 .55615 Metetra
.733 .714 .833 r
.4124 .55615 .41572 .56041 .42145 .55705 .41814 .55272 Metetra
.733 .711 .83 r
.41814 .55272 .42145 .55705 .42719 .55367 .42389 .54924 Metetra
.732 .707 .827 r
.42389 .54924 .42719 .55367 .43293 .55026 .42964 .54573 Metetra
.732 .704 .824 r
.42964 .54573 .43293 .55026 .43868 .54681 .4354 .54218 Metetra
.732 .7 .82 r
.4354 .54218 .43868 .54681 .44444 .54333 .44117 .53859 Metetra
.732 .696 .816 r
.44117 .53859 .44444 .54333 .45021 .53981 .44695 .53494 Metetra
.732 .692 .812 r
.44695 .53494 .45021 .53981 .45598 .53624 .45273 .53125 Metetra
.732 .687 .808 r
.45273 .53125 .45598 .53624 .46176 .53263 .45852 .52749 Metetra
.731 .682 .803 r
.45852 .52749 .46176 .53263 .46755 .52898 .46432 .52368 Metetra
.731 .677 .798 r
.46432 .52368 .46755 .52898 .47334 .52527 .47012 .51979 Metetra
.731 .671 .792 r
.47012 .51979 .47334 .52527 .47914 .52151 .47593 .51581 Metetra
.73 .665 .786 r
.47593 .51581 .47914 .52151 .48495 .51768 .48174 .51175 Metetra
.73 .658 .779 r
.48174 .51175 .48495 .51768 .49076 .51378 .48756 .50756 Metetra
.729 .651 .771 r
.48756 .50756 .49076 .51378 .49658 .50981 .49339 .50325 Metetra
.728 .642 .763 r
.49339 .50325 .49658 .50981 .5024 .50576 .49922 .49878 Metetra
.727 .633 .753 r
.49922 .49878 .5024 .50576 .50823 .50163 .50505 .4941 Metetra
.726 .621 .741 r
.50505 .4941 .50823 .50163 .51406 .49744 .51089 .48916 Metetra
.723 .607 .727 r
.51089 .48916 .51406 .49744 .5199 .49325 .51673 .4839 Metetra
.718 .59 .711 r
.51673 .4839 .5199 .49325 .52574 .48924 .52258 .47825 Metetra
.705 .565 .693 r
.52258 .47825 .52574 .48924 .53159 .4859 .52841 .4724 Metetra
.664 .519 .677 r
.52841 .4724 .53159 .4859 .53745 .48447 .53426 .46788 Metetra
.414 .334 .657 r
.53426 .46788 .53745 .48447 .54334 .48713 .54015 .4738 Metetra
.302 .284 .672 r
.54015 .4738 .54334 .48713 .54928 .49173 .54609 .48111 Metetra
.215 .262 .696 r
.54609 .48111 .54928 .49173 .55527 .49704 .55208 .48828 Metetra
.135 .246 .715 r
.55208 .48828 .55527 .49704 .56129 .50261 .55811 .49514 Metetra
.057 .23 .726 r
.55811 .49514 .56129 .50261 .56736 .50823 .56418 .50175 Metetra
0 .209 .727 r
.56418 .50175 .56736 .50823 .57346 .51386 .57029 .50819 Metetra
0 .183 .716 r
.57029 .50819 .57346 .51386 .5796 .51949 .57644 .51452 Metetra
0 .149 .691 r
.57644 .51452 .5796 .51949 .58579 .52512 .58264 .52078 Metetra
0 .106 .647 r
.58264 .52078 .58579 .52512 .59202 .53076 .58888 .52704 Metetra
0 .055 .582 r
.58888 .52704 .59202 .53076 .59828 .53643 .59516 .53332 Metetra
.504 .003 0 r
.59516 .53332 .59828 .53643 .60459 .54215 .60149 .53966 Metetra
.593 .066 0 r
.60149 .53966 .60459 .54215 .61095 .54793 .60786 .54611 Metetra
.667 .129 0 r
.60786 .54611 .61095 .54793 .61734 .55379 .61428 .55271 Metetra
.721 .189 0 r
.61428 .55271 .61734 .55379 .62378 .55973 .62075 .55952 Metetra
.755 .243 0 r
.62075 .55952 .62378 .55973 .63027 .56574 .62728 .56662 Metetra
.771 .289 .087 r
.62728 .56662 .63027 .56574 .6368 .57177 .63387 .57408 Metetra
.771 .325 .19 r
.63387 .57408 .6368 .57177 .64337 .57762 .64052 .582 Metetra
.752 .345 .278 r
.64052 .582 .64337 .57762 .64995 .58275 .64724 .5902 Metetra
.694 .324 .342 r
.64724 .5902 .64995 .58275 .6565 .58583 .65395 .59697 Metetra
.352 .047 .301 r
.65395 .59697 .6565 .58583 .6629 .58443 .66025 .59216 Metetra
.135 0 .129 r
.66025 .59216 .6629 .58443 .66923 .58093 .66649 .58586 Metetra
.66649 .58586 .66923 .58093 .67553 .57669 .67274 .5798 Metetra
.67274 .5798 .67553 .57669 .68182 .57224 .67901 .57416 Metetra
.541 .881 .628 r
.67901 .57416 .68182 .57224 .68813 .56778 .6853 .56888 Metetra
.652 .959 .765 r
.6853 .56888 .68813 .56778 .69444 .56339 .6916 .56387 Metetra
.712 .987 .843 r
.6916 .56387 .69444 .56339 .70076 .55907 .69791 .55908 Metetra
.743 .99 .886 r
.69791 .55908 .70076 .55907 .70708 .55483 .70424 .55447 Metetra
.76 .982 .91 r
.70424 .55447 .70708 .55483 .71342 .55067 .71058 .54999 Metetra
.768 .971 .923 r
.71058 .54999 .71342 .55067 .71977 .54657 .71693 .54563 Metetra
.772 .958 .93 r
.71693 .54563 .71977 .54657 .72613 .54254 .72329 .54136 Metetra
.773 .945 .934 r
.72329 .54136 .72613 .54254 .7325 .53856 .72967 .53718 Metetra
.773 .934 .936 r
.72967 .53718 .7325 .53856 .73888 .53462 .73605 .53307 Metetra
.772 .923 .936 r
.73605 .53307 .73888 .53462 .74527 .53073 .74245 .52902 Metetra
.771 .912 .935 r
.74245 .52902 .74527 .53073 .75167 .52688 .74885 .52502 Metetra
.769 .902 .934 r
.74885 .52502 .75167 .52688 .75807 .52306 .75527 .52107 Metetra
.767 .894 .933 r
.75527 .52107 .75807 .52306 .76449 .51928 .76169 .51716 Metetra
.765 .885 .931 r
.76169 .51716 .76449 .51928 .77092 .51553 .76813 .5133 Metetra
.763 .878 .93 r
.76813 .5133 .77092 .51553 .77736 .51181 .77458 .50946 Metetra
.761 .871 .928 r
.77458 .50946 .77736 .51181 .78381 .5081 .78103 .50567 Metetra
.76 .865 .926 r
.78103 .50567 .78381 .5081 .79027 .50443 .7875 .50189 Metetra
.758 .858 .924 r
.7875 .50189 .79027 .50443 .79674 .50078 .79398 .49815 Metetra
.756 .852 .922 r
.79398 .49815 .79674 .50078 .80322 .49714 .80047 .49443 Metetra
.754 .847 .92 r
.80047 .49443 .80322 .49714 .80971 .49353 .80697 .49074 Metetra
.753 .842 .918 r
.80697 .49074 .80971 .49353 .81621 .48993 .81347 .48706 Metetra
.751 .837 .917 r
.81347 .48706 .81621 .48993 .82272 .48634 .81999 .4834 Metetra
.75 .832 .915 r
.81999 .4834 .82272 .48634 .82924 .48277 .82652 .47975 Metetra
.748 .828 .913 r
.82652 .47975 .82924 .48277 .83578 .47921 .83306 .47612 Metetra
.747 .823 .911 r
.83306 .47612 .83578 .47921 .84232 .47566 .83961 .47251 Metetra
.746 .819 .909 r
.83961 .47251 .84232 .47566 .84887 .47212 .84617 .4689 Metetra
.745 .815 .908 r
.84617 .4689 .84887 .47212 .85543 .46859 .85274 .4653 Metetra
.744 .812 .906 r
.85274 .4653 .85543 .46859 .86201 .46506 .85932 .46171 Metetra
.743 .808 .904 r
.85932 .46171 .86201 .46506 .86859 .46154 .86591 .45813 Metetra
.742 .805 .902 r
.86591 .45813 .86859 .46154 .87518 .45801 .87251 .45455 Metetra
.741 .802 .901 r
.87251 .45455 .87518 .45801 .88179 .4545 .87912 .45096 Metetra
.74 .798 .899 r
.87912 .45096 .88179 .4545 .8884 .45097 .88574 .44739 Metetra
.739 .795 .897 r
.88574 .44739 .8884 .45097 .89502 .44746 .89237 .44381 Metetra
.738 .792 .895 r
.89237 .44381 .89502 .44746 .90165 .44394 .89901 .44024 Metetra
.738 .789 .893 r
.89901 .44024 .90165 .44394 .9083 .44042 .90566 .43666 Metetra
.737 .786 .892 r
.90566 .43666 .9083 .44042 .91495 .4369 .91232 .43308 Metetra
.739 .766 .875 r
.30122 .61417 .30476 .61725 .31036 .61415 .30683 .61101 Metetra
.739 .764 .873 r
.30683 .61101 .31036 .61415 .31596 .61104 .31245 .60785 Metetra
.738 .761 .871 r
.31245 .60785 .31596 .61104 .32158 .60793 .31807 .60469 Metetra
.738 .759 .869 r
.31807 .60469 .32158 .60793 .3272 .60481 .3237 .60151 Metetra
.738 .756 .868 r
.3237 .60151 .3272 .60481 .33282 .60168 .32934 .59833 Metetra
.737 .754 .866 r
.32934 .59833 .33282 .60168 .33846 .59854 .33499 .59513 Metetra
.737 .751 .864 r
.33499 .59513 .33846 .59854 .3441 .59538 .34064 .59193 Metetra
.737 .749 .862 r
.34064 .59193 .3441 .59538 .34975 .59222 .3463 .58871 Metetra
.736 .746 .86 r
.3463 .58871 .34975 .59222 .35541 .58904 .35197 .58548 Metetra
.736 .744 .858 r
.35197 .58548 .35541 .58904 .36107 .58585 .35765 .58223 Metetra
.736 .741 .856 r
.35765 .58223 .36107 .58585 .36675 .58264 .36333 .57896 Metetra
.736 .739 .854 r
.36333 .57896 .36675 .58264 .37243 .57941 .36902 .57567 Metetra
.736 .736 .851 r
.36902 .57567 .37243 .57941 .37811 .57616 .37472 .57236 Metetra
.736 .734 .849 r
.37472 .57236 .37811 .57616 .38381 .57289 .38042 .56902 Metetra
.736 .731 .846 r
.38042 .56902 .38381 .57289 .38951 .5696 .38614 .56566 Metetra
.736 .728 .844 r
.38614 .56566 .38951 .5696 .39522 .56628 .39186 .56227 Metetra
.736 .725 .841 r
.39186 .56227 .39522 .56628 .40094 .56293 .39759 .55885 Metetra
.736 .722 .839 r
.39759 .55885 .40094 .56293 .40667 .55956 .40332 .5554 Metetra
.736 .719 .836 r
.40332 .5554 .40667 .55956 .4124 .55615 .40907 .55192 Metetra
.736 .716 .833 r
.40907 .55192 .4124 .55615 .41814 .55272 .41482 .5484 Metetra
.736 .713 .83 r
.41482 .5484 .41814 .55272 .42389 .54924 .42057 .54484 Metetra
.736 .71 .826 r
.42057 .54484 .42389 .54924 .42964 .54573 .42634 .54123 Metetra
.736 .706 .823 r
.42634 .54123 .42964 .54573 .4354 .54218 .43211 .53758 Metetra
.736 .703 .819 r
.43211 .53758 .4354 .54218 .44117 .53859 .43789 .53388 Metetra
.736 .699 .815 r
.43789 .53388 .44117 .53859 .44695 .53494 .44368 .53012 Metetra
.737 .695 .811 r
.44368 .53012 .44695 .53494 .45273 .53125 .44947 .52629 Metetra
.737 .691 .807 r
.44947 .52629 .45273 .53125 .45852 .52749 .45527 .5224 Metetra
.737 .686 .802 r
.45527 .5224 .45852 .52749 .46432 .52368 .46107 .51843 Metetra
.737 .681 .797 r
.46107 .51843 .46432 .52368 .47012 .51979 .46689 .51437 Metetra
.737 .676 .791 r
.46689 .51437 .47012 .51979 .47593 .51581 .4727 .5102 Metetra
.738 .671 .785 r
.4727 .5102 .47593 .51581 .48174 .51175 .47853 .5059 Metetra
.738 .665 .778 r
.47853 .5059 .48174 .51175 .48756 .50756 .48436 .50146 Metetra
.739 .658 .77 r
.48436 .50146 .48756 .50756 .49339 .50325 .49019 .49684 Metetra
.74 .651 .761 r
.49019 .49684 .49339 .50325 .49922 .49878 .49603 .49198 Metetra
.741 .643 .751 r
.49603 .49198 .49922 .49878 .50505 .4941 .50187 .4868 Metetra
.743 .634 .739 r
.50187 .4868 .50505 .4941 .51089 .48916 .50772 .4812 Metetra
.747 .624 .723 r
.50772 .4812 .51089 .48916 .51673 .4839 .51357 .47492 Metetra
.753 .613 .703 r
.51357 .47492 .51673 .4839 .52258 .47825 .51941 .46749 Metetra
.764 .6 .675 r
.51941 .46749 .52258 .47825 .52841 .4724 .52525 .4576 Metetra
.785 .588 .634 r
.52525 .4576 .52841 .4724 .53426 .46788 .53106 .44105 Metetra
.211 .148 .564 r
.53106 .44105 .53426 .46788 .54015 .4738 .53696 .45916 Metetra
.126 .15 .615 r
.53696 .45916 .54015 .4738 .54609 .48111 .5429 .47071 Metetra
.053 .155 .654 r
.5429 .47071 .54609 .48111 .55208 .48828 .54889 .47988 Metetra
0 .156 .675 r
.54889 .47988 .55208 .48828 .55811 .49514 .55492 .48797 Metetra
0 .149 .684 r
.55492 .48797 .55811 .49514 .56418 .50175 .561 .49548 Metetra
0 .133 .68 r
.561 .49548 .56418 .50175 .57029 .50819 .56711 .50266 Metetra
0 .11 .663 r
.56711 .50266 .57029 .50819 .57644 .51452 .57327 .50963 Metetra
0 .079 .632 r
.57327 .50963 .57644 .51452 .58264 .52078 .57948 .5165 Metetra
0 .039 .584 r
.57948 .5165 .58264 .52078 .58888 .52704 .58573 .52332 Metetra
0 0 .519 r
.58573 .52332 .58888 .52704 .59516 .53332 .59202 .53017 Metetra
.562 .062 0 r
.59202 .53017 .59516 .53332 .60149 .53966 .59836 .5371 Metetra
.638 .118 0 r
.59836 .5371 .60149 .53966 .60786 .54611 .60475 .54418 Metetra
.701 .176 0 r
.60475 .54418 .60786 .54611 .61428 .55271 .6112 .55147 Metetra
.75 .231 0 r
.6112 .55147 .61428 .55271 .62075 .55952 .6177 .55908 Metetra
.783 .284 0 r
.6177 .55908 .62075 .55952 .62728 .56662 .62426 .56717 Metetra
.804 .333 .105 r
.62426 .56717 .62728 .56662 .63387 .57408 .6309 .57602 Metetra
.815 .381 .211 r
.6309 .57602 .63387 .57408 .64052 .582 .63764 .5862 Metetra
.82 .43 .314 r
.63764 .5862 .64052 .582 .64724 .5902 .64454 .59922 Metetra
.825 .483 .409 r
.64454 .59922 .64724 .5902 .65395 .59697 .6518 .62002 Metetra
.153 0 .346 r
.6518 .62002 .65395 .59697 .66025 .59216 .65763 .60147 Metetra
0 0 .083 r
.65763 .60147 .66025 .59216 .66649 .58586 .66372 .5906 Metetra
.66372 .5906 .66649 .58586 .67274 .5798 .66992 .58252 Metetra
.582 .864 .539 r
.66992 .58252 .67274 .5798 .67901 .57416 .67616 .57574 Metetra
.69 .957 .706 r
.67616 .57574 .67901 .57416 .6853 .56888 .68243 .5697 Metetra
.745 .991 .802 r
.68243 .5697 .6853 .56888 .6916 .56387 .68873 .56414 Metetra
.771 .996 .856 r
.68873 .56414 .6916 .56387 .69791 .55908 .69504 .55892 Metetra
.783 .99 .888 r
.69504 .55892 .69791 .55908 .70424 .55447 .70137 .55396 Metetra
.788 .979 .906 r
.70137 .55396 .70424 .55447 .71058 .54999 .70771 .54919 Metetra
.789 .966 .918 r
.70771 .54919 .71058 .54999 .71693 .54563 .71407 .54458 Metetra
.788 .954 .924 r
.71407 .54458 .71693 .54563 .72329 .54136 .72044 .54009 Metetra
.787 .941 .928 r
.72044 .54009 .72329 .54136 .72967 .53718 .72682 .53572 Metetra
.784 .93 .929 r
.72682 .53572 .72967 .53718 .73605 .53307 .73321 .53144 Metetra
.782 .919 .93 r
.73321 .53144 .73605 .53307 .74245 .52902 .73961 .52723 Metetra
.779 .909 .93 r
.73961 .52723 .74245 .52902 .74885 .52502 .74602 .52309 Metetra
.776 .9 .929 r
.74602 .52309 .74885 .52502 .75527 .52107 .75244 .51901 Metetra
.773 .891 .928 r
.75244 .51901 .75527 .52107 .76169 .51716 .75888 .51499 Metetra
.771 .883 .927 r
.75888 .51499 .76169 .51716 .76813 .5133 .76532 .51101 Metetra
.768 .876 .925 r
.76532 .51101 .76813 .5133 .77458 .50946 .77178 .50708 Metetra
.766 .869 .924 r
.77178 .50708 .77458 .50946 .78103 .50567 .77824 .50318 Metetra
.764 .863 .922 r
.77824 .50318 .78103 .50567 .7875 .50189 .78472 .49932 Metetra
.762 .857 .921 r
.78472 .49932 .7875 .50189 .79398 .49815 .7912 .49549 Metetra
.76 .851 .919 r
.7912 .49549 .79398 .49815 .80047 .49443 .7977 .49169 Metetra
.758 .846 .917 r
.7977 .49169 .80047 .49443 .80697 .49074 .8042 .48791 Metetra
.756 .841 .916 r
.8042 .48791 .80697 .49074 .81347 .48706 .81072 .48416 Metetra
.754 .836 .914 r
.81072 .48416 .81347 .48706 .81999 .4834 .81725 .48042 Metetra
.753 .831 .912 r
.81725 .48042 .81999 .4834 .82652 .47975 .82378 .4767 Metetra
.751 .827 .911 r
.82378 .4767 .82652 .47975 .83306 .47612 .83033 .473 Metetra
.75 .823 .909 r
.83033 .473 .83306 .47612 .83961 .47251 .83689 .46932 Metetra
.748 .819 .907 r
.83689 .46932 .83961 .47251 .84617 .4689 .84346 .46565 Metetra
.747 .815 .906 r
.84346 .46565 .84617 .4689 .85274 .4653 .85004 .46199 Metetra
.746 .812 .904 r
.85004 .46199 .85274 .4653 .85932 .46171 .85663 .45834 Metetra
.745 .808 .902 r
.85663 .45834 .85932 .46171 .86591 .45813 .86322 .45469 Metetra
.744 .805 .901 r
.86322 .45469 .86591 .45813 .87251 .45455 .86983 .45105 Metetra
.743 .801 .899 r
.86983 .45105 .87251 .45455 .87912 .45096 .87645 .44742 Metetra
.743 .799 .897 r
.87645 .44742 .87912 .45096 .88574 .44739 .88308 .44378 Metetra
.741 .795 .896 r
.88308 .44378 .88574 .44739 .89237 .44381 .88972 .44015 Metetra
.741 .792 .894 r
.88972 .44015 .89237 .44381 .89901 .44024 .89636 .43651 Metetra
.74 .789 .892 r
.89636 .43651 .89901 .44024 .90566 .43666 .90302 .43288 Metetra
.739 .786 .891 r
.90302 .43288 .90566 .43666 .91232 .43308 .90969 .42924 Metetra
.741 .767 .873 r
.29767 .61107 .30122 .61417 .30683 .61101 .30329 .60786 Metetra
.741 .764 .872 r
.30329 .60786 .30683 .61101 .31245 .60785 .30892 .60466 Metetra
.74 .762 .87 r
.30892 .60466 .31245 .60785 .31807 .60469 .31456 .60144 Metetra
.74 .759 .868 r
.31456 .60144 .31807 .60469 .3237 .60151 .3202 .59822 Metetra
.74 .757 .867 r
.3202 .59822 .3237 .60151 .32934 .59833 .32585 .59498 Metetra
.739 .754 .865 r
.32585 .59498 .32934 .59833 .33499 .59513 .33151 .59173 Metetra
.739 .752 .863 r
.33151 .59173 .33499 .59513 .34064 .59193 .33717 .58847 Metetra
.739 .75 .861 r
.33717 .58847 .34064 .59193 .3463 .58871 .34284 .5852 Metetra
.738 .747 .859 r
.34284 .5852 .3463 .58871 .35197 .58548 .34852 .58191 Metetra
.738 .745 .857 r
.34852 .58191 .35197 .58548 .35765 .58223 .35421 .5786 Metetra
.738 .743 .855 r
.35421 .5786 .35765 .58223 .36333 .57896 .3599 .57528 Metetra
.738 .74 .853 r
.3599 .57528 .36333 .57896 .36902 .57567 .3656 .57193 Metetra
.738 .738 .851 r
.3656 .57193 .36902 .57567 .37472 .57236 .37131 .56856 Metetra
.738 .735 .848 r
.37131 .56856 .37472 .57236 .38042 .56902 .37703 .56516 Metetra
.738 .733 .846 r
.37703 .56516 .38042 .56902 .38614 .56566 .38275 .56173 Metetra
.738 .73 .843 r
.38275 .56173 .38614 .56566 .39186 .56227 .38848 .55827 Metetra
.738 .727 .841 r
.38848 .55827 .39186 .56227 .39759 .55885 .39422 .55479 Metetra
.739 .724 .838 r
.39422 .55479 .39759 .55885 .40332 .5554 .39997 .55126 Metetra
.739 .721 .835 r
.39997 .55126 .40332 .5554 .40907 .55192 .40572 .5477 Metetra
.739 .718 .832 r
.40572 .5477 .40907 .55192 .41482 .5484 .41148 .5441 Metetra
.739 .716 .829 r
.41148 .5441 .41482 .5484 .42057 .54484 .41725 .54045 Metetra
.739 .712 .826 r
.41725 .54045 .42057 .54484 .42634 .54123 .42302 .53676 Metetra
.74 .709 .822 r
.42302 .53676 .42634 .54123 .43211 .53758 .42881 .53301 Metetra
.74 .706 .819 r
.42881 .53301 .43211 .53758 .43789 .53388 .43459 .5292 Metetra
.741 .702 .815 r
.43459 .5292 .43789 .53388 .44368 .53012 .44039 .52533 Metetra
.741 .698 .811 r
.44039 .52533 .44368 .53012 .44947 .52629 .44619 .52139 Metetra
.742 .695 .806 r
.44619 .52139 .44947 .52629 .45527 .5224 .452 .51736 Metetra
.742 .69 .802 r
.452 .51736 .45527 .5224 .46107 .51843 .45782 .51325 Metetra
.743 .686 .796 r
.45782 .51325 .46107 .51843 .46689 .51437 .46364 .50903 Metetra
.744 .681 .791 r
.46364 .50903 .46689 .51437 .4727 .5102 .46947 .50469 Metetra
.745 .677 .785 r
.46947 .50469 .4727 .5102 .47853 .5059 .4753 .50019 Metetra
.746 .672 .778 r
.4753 .50019 .47853 .5059 .48436 .50146 .48114 .49553 Metetra
.748 .666 .77 r
.48114 .49553 .48436 .50146 .49019 .49684 .48698 .49064 Metetra
.75 .661 .762 r
.48698 .49064 .49019 .49684 .49603 .49198 .49283 .48548 Metetra
.754 .655 .752 r
.49283 .48548 .49603 .49198 .50187 .4868 .49868 .47994 Metetra
.758 .649 .741 r
.49868 .47994 .50187 .4868 .50772 .4812 .50454 .47389 Metetra
.765 .644 .727 r
.50454 .47389 .50772 .4812 .51357 .47492 .51039 .46711 Metetra
.776 .64 .711 r
.51039 .46711 .51357 .47492 .51941 .46749 .51625 .4592 Metetra
.792 .64 .693 r
.51625 .4592 .51941 .46749 .52525 .4576 .52209 .44955 Metetra
.82 .655 .679 r
.52209 .44955 .52525 .4576 .53106 .44105 .52792 .43758 Metetra
.88 .456 .051 r
.52792 .43758 .53106 .44105 .53696 .45916 .53379 .45129 Metetra
0 0 .467 r
.53379 .45129 .53696 .45916 .5429 .47071 .53973 .46277 Metetra
0 0 .575 r
.53973 .46277 .5429 .47071 .54889 .47988 .54571 .47261 Metetra
0 .044 .611 r
.54571 .47261 .54889 .47988 .55492 .48797 .55174 .48142 Metetra
0 .057 .623 r
.55174 .48142 .55492 .48797 .561 .49548 .55781 .48959 Metetra
0 .053 .619 r
.55781 .48959 .561 .49548 .56711 .50266 .56393 .49736 Metetra
0 .037 .601 r
.56393 .49736 .56711 .50266 .57327 .50963 .5701 .50489 Metetra
0 .012 .569 r
.5701 .50489 .57327 .50963 .57948 .5165 .57631 .51228 Metetra
0 0 .522 r
.57631 .51228 .57948 .5165 .58573 .52332 .58256 .51962 Metetra
0 0 .461 r
.58256 .51962 .58573 .52332 .59202 .53017 .58887 .52699 Metetra
.606 .108 0 r
.58887 .52699 .59202 .53017 .59836 .5371 .59522 .53444 Metetra
.669 .157 0 r
.59522 .53444 .59836 .5371 .60475 .54418 .60163 .54207 Metetra
.722 .207 0 r
.60163 .54207 .60475 .54418 .6112 .55147 .60809 .54995 Metetra
.765 .256 0 r
.60809 .54995 .6112 .55147 .6177 .55908 .61462 .55822 Metetra
.797 .303 0 r
.61462 .55822 .6177 .55908 .62426 .56717 .62121 .56705 Metetra
.82 .348 .08 r
.62121 .56705 .62426 .56717 .6309 .57602 .62788 .57672 Metetra
.837 .391 .163 r
.62788 .57672 .6309 .57602 .63764 .5862 .63465 .58771 Metetra
.852 .432 .23 r
.63465 .58771 .63764 .5862 .64454 .59922 .64155 .60076 Metetra
.874 .469 .258 r
.64155 .60076 .64454 .59922 .6518 .62002 .64863 .61662 Metetra
.92 .787 .697 r
.64863 .61662 .6518 .62002 .65763 .60147 .65468 .60325 Metetra
.925 .966 .667 r
.65468 .60325 .65763 .60147 .66372 .5906 .66082 .59261 Metetra
.837 .974 .666 r
.66082 .59261 .66372 .5906 .66992 .58252 .66701 .58395 Metetra
.816 .988 .727 r
.66701 .58395 .66992 .58252 .67616 .57574 .67325 .57658 Metetra
.816 .999 .789 r
.67325 .57658 .67616 .57574 .68243 .5697 .67953 .57004 Metetra
.818 .999 .836 r
.67953 .57004 .68243 .5697 .68873 .56414 .68583 .56406 Metetra
.817 .993 .868 r
.68583 .56406 .68873 .56414 .69504 .55892 .69214 .5585 Metetra
.814 .982 .889 r
.69214 .5585 .69504 .55892 .70137 .55396 .69848 .55324 Metetra
.811 .97 .903 r
.69848 .55324 .70137 .55396 .70771 .54919 .70482 .54822 Metetra
.807 .958 .912 r
.70482 .54822 .70771 .54919 .71407 .54458 .71119 .54339 Metetra
.802 .946 .917 r
.71119 .54339 .71407 .54458 .72044 .54009 .71756 .53871 Metetra
.798 .935 .921 r
.71756 .53871 .72044 .54009 .72682 .53572 .72395 .53416 Metetra
.794 .924 .923 r
.72395 .53416 .72682 .53572 .73321 .53144 .73034 .52972 Metetra
.79 .914 .924 r
.73034 .52972 .73321 .53144 .73961 .52723 .73675 .52537 Metetra
.786 .905 .924 r
.73675 .52537 .73961 .52723 .74602 .52309 .74317 .52109 Metetra
.782 .896 .924 r
.74317 .52109 .74602 .52309 .75244 .51901 .7496 .51689 Metetra
.779 .888 .923 r
.7496 .51689 .75244 .51901 .75888 .51499 .75604 .51275 Metetra
.776 .88 .922 r
.75604 .51275 .75888 .51499 .76532 .51101 .76249 .50867 Metetra
.773 .874 .921 r
.76249 .50867 .76532 .51101 .77178 .50708 .76896 .50464 Metetra
.77 .867 .92 r
.76896 .50464 .77178 .50708 .77824 .50318 .77543 .50065 Metetra
.768 .861 .919 r
.77543 .50065 .77824 .50318 .78472 .49932 .78191 .49669 Metetra
.765 .855 .917 r
.78191 .49669 .78472 .49932 .7912 .49549 .78841 .49278 Metetra
.763 .85 .916 r
.78841 .49278 .7912 .49549 .7977 .49169 .79491 .4889 Metetra
.761 .845 .914 r
.79491 .4889 .7977 .49169 .8042 .48791 .80143 .48504 Metetra
.759 .84 .913 r
.80143 .48504 .8042 .48791 .81072 .48416 .80795 .48121 Metetra
.757 .835 .911 r
.80795 .48121 .81072 .48416 .81725 .48042 .81449 .47741 Metetra
.756 .831 .91 r
.81449 .47741 .81725 .48042 .82378 .4767 .82103 .47362 Metetra
.754 .826 .908 r
.82103 .47362 .82378 .4767 .83033 .473 .82759 .46985 Metetra
.752 .822 .907 r
.82759 .46985 .83033 .473 .83689 .46932 .83416 .46611 Metetra
.751 .819 .905 r
.83416 .46611 .83689 .46932 .84346 .46565 .84073 .46237 Metetra
.749 .815 .904 r
.84073 .46237 .84346 .46565 .85004 .46199 .84732 .45865 Metetra
.748 .811 .902 r
.84732 .45865 .85004 .46199 .85663 .45834 .85392 .45494 Metetra
.747 .808 .901 r
.85392 .45494 .85663 .45834 .86322 .45469 .86052 .45123 Metetra
.746 .805 .899 r
.86052 .45123 .86322 .45469 .86983 .45105 .86714 .44754 Metetra
.745 .802 .898 r
.86714 .44754 .86983 .45105 .87645 .44742 .87376 .44384 Metetra
.744 .798 .896 r
.87376 .44384 .87645 .44742 .88308 .44378 .8804 .44015 Metetra
.744 .796 .894 r
.8804 .44015 .88308 .44378 .88972 .44015 .88704 .43646 Metetra
.743 .792 .893 r
.88704 .43646 .88972 .44015 .89636 .43651 .8937 .43277 Metetra
.742 .79 .891 r
.8937 .43277 .89636 .43651 .90302 .43288 .90036 .42908 Metetra
.741 .787 .889 r
.90036 .42908 .90302 .43288 .90969 .42924 .90704 .42538 Metetra
.743 .767 .872 r
.29411 .60796 .29767 .61107 .30329 .60786 .29974 .60471 Metetra
.743 .764 .871 r
.29974 .60471 .30329 .60786 .30892 .60466 .30538 .60145 Metetra
.742 .762 .869 r
.30538 .60145 .30892 .60466 .31456 .60144 .31103 .59818 Metetra
.742 .76 .867 r
.31103 .59818 .31456 .60144 .3202 .59822 .31668 .59491 Metetra
.741 .757 .866 r
.31668 .59491 .3202 .59822 .32585 .59498 .32234 .59162 Metetra
.741 .755 .864 r
.32234 .59162 .32585 .59498 .33151 .59173 .32801 .58832 Metetra
.741 .753 .862 r
.32801 .58832 .33151 .59173 .33717 .58847 .33369 .58501 Metetra
.741 .751 .86 r
.33369 .58501 .33717 .58847 .34284 .5852 .33937 .58168 Metetra
.74 .748 .858 r
.33937 .58168 .34284 .5852 .34852 .58191 .34506 .57834 Metetra
.74 .746 .856 r
.34506 .57834 .34852 .58191 .35421 .5786 .35075 .57498 Metetra
.74 .744 .854 r
.35075 .57498 .35421 .5786 .3599 .57528 .35646 .5716 Metetra
.74 .741 .852 r
.35646 .5716 .3599 .57528 .3656 .57193 .36217 .5682 Metetra
.74 .739 .85 r
.36217 .5682 .3656 .57193 .37131 .56856 .36789 .56477 Metetra
.741 .737 .848 r
.36789 .56477 .37131 .56856 .37703 .56516 .37362 .5613 Metetra
.741 .734 .845 r
.37362 .5613 .37703 .56516 .38275 .56173 .37935 .55781 Metetra
.741 .732 .843 r
.37935 .55781 .38275 .56173 .38848 .55827 .38509 .55429 Metetra
.741 .729 .84 r
.38509 .55429 .38848 .55827 .39422 .55479 .39084 .55074 Metetra
.741 .726 .838 r
.39084 .55074 .39422 .55479 .39997 .55126 .3966 .54714 Metetra
.742 .723 .835 r
.3966 .54714 .39997 .55126 .40572 .5477 .40236 .54351 Metetra
.742 .721 .832 r
.40236 .54351 .40572 .5477 .41148 .5441 .40813 .53983 Metetra
.742 .718 .829 r
.40813 .53983 .41148 .5441 .41725 .54045 .41391 .5361 Metetra
.743 .715 .826 r
.41391 .5361 .41725 .54045 .42302 .53676 .4197 .53232 Metetra
.743 .712 .822 r
.4197 .53232 .42302 .53676 .42881 .53301 .42549 .52848 Metetra
.744 .709 .819 r
.42549 .52848 .42881 .53301 .43459 .5292 .43129 .52458 Metetra
.744 .705 .815 r
.43129 .52458 .43459 .5292 .44039 .52533 .43709 .5206 Metetra
.745 .702 .811 r
.43709 .5206 .44039 .52533 .44619 .52139 .44291 .51655 Metetra
.746 .698 .806 r
.44291 .51655 .44619 .52139 .452 .51736 .44873 .5124 Metetra
.747 .695 .802 r
.44873 .5124 .452 .51736 .45782 .51325 .45455 .50816 Metetra
.748 .691 .797 r
.45455 .50816 .45782 .51325 .46364 .50903 .46038 .5038 Metetra
.75 .687 .791 r
.46038 .5038 .46364 .50903 .46947 .50469 .46622 .4993 Metetra
.751 .683 .786 r
.46622 .4993 .46947 .50469 .4753 .50019 .47206 .49465 Metetra
.753 .679 .779 r
.47206 .49465 .4753 .50019 .48114 .49553 .47791 .4898 Metetra
.756 .675 .772 r
.47791 .4898 .48114 .49553 .48698 .49064 .48377 .48472 Metetra
.76 .671 .764 r
.48377 .48472 .48698 .49064 .49283 .48548 .48962 .47933 Metetra
.764 .668 .756 r
.48962 .47933 .49283 .48548 .49868 .47994 .49548 .47357 Metetra
.77 .665 .746 r
.49548 .47357 .49868 .47994 .50454 .47389 .50135 .46732 Metetra
.779 .664 .736 r
.50135 .46732 .50454 .47389 .51039 .46711 .50721 .46041 Metetra
.791 .666 .725 r
.50721 .46041 .51039 .46711 .51625 .4592 .51307 .45265 Metetra
.808 .675 .716 r
.51307 .45265 .51625 .4592 .52209 .44955 .51892 .44384 Metetra
.835 .7 .715 r
.51892 .44384 .52209 .44955 .52792 .43758 .52477 .4341 Metetra
.839 .394 0 r
.52477 .4341 .52792 .43758 .53379 .45129 .53063 .44576 Metetra
.648 .246 0 r
.53063 .44576 .53379 .45129 .53973 .46277 .53655 .45657 Metetra
0 0 .459 r
.53655 .45657 .53973 .46277 .54571 .47261 .54253 .46645 Metetra
0 0 .522 r
.54253 .46645 .54571 .47261 .55174 .48142 .54855 .47557 Metetra
0 0 .546 r
.54855 .47557 .55174 .48142 .55781 .48959 .55462 .48415 Metetra
0 0 .548 r
.55462 .48415 .55781 .48959 .56393 .49736 .56074 .49236 Metetra
0 0 .534 r
.56074 .49236 .56393 .49736 .5701 .50489 .56691 .50033 Metetra
0 0 .506 r
.56691 .50033 .5701 .50489 .57631 .51228 .57313 .50816 Metetra
0 0 .465 r
.57313 .50816 .57631 .51228 .58256 .51962 .57939 .51594 Metetra
.583 .106 0 r
.57939 .51594 .58256 .51962 .58887 .52699 .5857 .52375 Metetra
.637 .144 0 r
.5857 .52375 .58887 .52699 .59522 .53444 .59207 .53166 Metetra
.688 .184 0 r
.59207 .53166 .59522 .53444 .60163 .54207 .59849 .53974 Metetra
.733 .226 0 r
.59849 .53974 .60163 .54207 .60809 .54995 .60496 .54811 Metetra
.77 .267 0 r
.60496 .54811 .60809 .54995 .61462 .55822 .6115 .55686 Metetra
.8 .307 0 r
.6115 .55686 .61462 .55822 .62121 .56705 .61811 .56616 Metetra
.823 .345 .031 r
.61811 .56616 .62121 .56705 .62788 .57672 .6248 .57622 Metetra
.842 .379 .089 r
.6248 .57622 .62788 .57672 .63465 .58771 .63157 .58729 Metetra
.857 .409 .122 r
.63157 .58729 .63465 .58771 .64155 .60076 .63846 .59964 Metetra
.869 .425 .104 r
.63846 .59964 .64155 .60076 .64863 .61662 .64545 .61321 Metetra
.901 .794 .734 r
.64545 .61321 .64863 .61662 .65468 .60325 .65162 .60236 Metetra
.939 .906 .763 r
.65162 .60236 .65468 .60325 .66082 .59261 .65781 .59264 Metetra
.924 .962 .782 r
.65781 .59264 .66082 .59261 .66701 .58395 .66404 .58413 Metetra
.895 .982 .804 r
.66404 .58413 .66701 .58395 .67325 .57658 .6703 .5766 Metetra
.873 .986 .83 r
.6703 .5766 .67325 .57658 .67953 .57004 .67658 .56982 Metetra
.857 .983 .854 r
.67658 .56982 .67953 .57004 .68583 .56406 .68288 .56358 Metetra
.845 .976 .873 r
.68288 .56358 .68583 .56406 .69214 .5585 .68921 .55777 Metetra
.835 .967 .888 r
.68921 .55777 .69214 .5585 .69848 .55324 .69555 .55228 Metetra
.827 .957 .898 r
.69555 .55228 .69848 .55324 .70482 .54822 .70191 .54706 Metetra
.819 .947 .906 r
.70191 .54706 .70482 .54822 .71119 .54339 .70828 .54203 Metetra
.813 .936 .911 r
.70828 .54203 .71119 .54339 .71756 .53871 .71466 .53718 Metetra
.807 .926 .914 r
.71466 .53718 .71756 .53871 .72395 .53416 .72105 .53247 Metetra
.801 .917 .917 r
.72105 .53247 .72395 .53416 .73034 .52972 .72746 .52789 Metetra
.796 .908 .918 r
.72746 .52789 .73034 .52972 .73675 .52537 .73387 .5234 Metetra
.792 .899 .918 r
.73387 .5234 .73675 .52537 .74317 .52109 .7403 .51901 Metetra
.788 .892 .919 r
.7403 .51901 .74317 .52109 .7496 .51689 .74674 .51469 Metetra
.784 .884 .918 r
.74674 .51469 .7496 .51689 .75604 .51275 .75319 .51045 Metetra
.78 .877 .918 r
.75319 .51045 .75604 .51275 .76249 .50867 .75965 .50626 Metetra
.777 .87 .917 r
.75965 .50626 .76249 .50867 .76896 .50464 .76612 .50214 Metetra
.774 .865 .916 r
.76612 .50214 .76896 .50464 .77543 .50065 .7726 .49805 Metetra
.771 .858 .915 r
.7726 .49805 .77543 .50065 .78191 .49669 .7791 .49402 Metetra
.769 .853 .914 r
.7791 .49402 .78191 .49669 .78841 .49278 .7856 .49002 Metetra
.766 .848 .913 r
.7856 .49002 .78841 .49278 .79491 .4889 .79211 .48606 Metetra
.764 .843 .911 r
.79211 .48606 .79491 .4889 .80143 .48504 .79863 .48213 Metetra
.762 .838 .91 r
.79863 .48213 .80143 .48504 .80795 .48121 .80517 .47823 Metetra
.76 .834 .909 r
.80517 .47823 .80795 .48121 .81449 .47741 .81171 .47435 Metetra
.758 .829 .907 r
.81171 .47435 .81449 .47741 .82103 .47362 .81827 .4705 Metetra
.756 .826 .906 r
.81827 .4705 .82103 .47362 .82759 .46985 .82483 .46667 Metetra
.754 .822 .905 r
.82483 .46667 .82759 .46985 .83416 .46611 .83141 .46286 Metetra
.753 .818 .903 r
.83141 .46286 .83416 .46611 .84073 .46237 .83799 .45906 Metetra
.752 .815 .902 r
.83799 .45906 .84073 .46237 .84732 .45865 .84459 .45528 Metetra
.75 .811 .901 r
.84459 .45528 .84732 .45865 .85392 .45494 .85119 .45151 Metetra
.749 .808 .899 r
.85119 .45151 .85392 .45494 .86052 .45123 .85781 .44775 Metetra
.748 .805 .898 r
.85781 .44775 .86052 .45123 .86714 .44754 .86443 .44399 Metetra
.747 .801 .896 r
.86443 .44399 .86714 .44754 .87376 .44384 .87107 .44024 Metetra
.747 .799 .895 r
.87107 .44024 .87376 .44384 .8804 .44015 .87771 .43649 Metetra
.745 .796 .893 r
.87771 .43649 .8804 .44015 .88704 .43646 .88436 .43275 Metetra
.745 .793 .892 r
.88436 .43275 .88704 .43646 .8937 .43277 .89102 .429 Metetra
.744 .79 .89 r
.89102 .429 .8937 .43277 .90036 .42908 .8977 .42526 Metetra
.743 .788 .888 r
.8977 .42526 .90036 .42908 .90704 .42538 .90438 .42151 Metetra
.745 .767 .871 r
.29053 .60484 .29411 .60796 .29974 .60471 .29618 .60154 Metetra
.744 .765 .869 r
.29618 .60154 .29974 .60471 .30538 .60145 .30183 .59823 Metetra
.744 .762 .868 r
.30183 .59823 .30538 .60145 .31103 .59818 .30749 .59492 Metetra
.743 .76 .866 r
.30749 .59492 .31103 .59818 .31668 .59491 .31315 .59159 Metetra
.743 .758 .865 r
.31315 .59159 .31668 .59491 .32234 .59162 .31882 .58826 Metetra
.743 .756 .863 r
.31882 .58826 .32234 .59162 .32801 .58832 .3245 .58491 Metetra
.743 .754 .861 r
.3245 .58491 .32801 .58832 .33369 .58501 .33019 .58155 Metetra
.742 .751 .859 r
.33019 .58155 .33369 .58501 .33937 .58168 .33588 .57817 Metetra
.742 .749 .858 r
.33588 .57817 .33937 .58168 .34506 .57834 .34158 .57478 Metetra
.742 .747 .856 r
.34158 .57478 .34506 .57834 .35075 .57498 .34729 .57137 Metetra
.742 .745 .854 r
.34729 .57137 .35075 .57498 .35646 .5716 .353 .56793 Metetra
.742 .743 .852 r
.353 .56793 .35646 .5716 .36217 .5682 .35873 .56447 Metetra
.743 .74 .85 r
.35873 .56447 .36217 .5682 .36789 .56477 .36446 .56098 Metetra
.743 .738 .847 r
.36446 .56098 .36789 .56477 .37362 .5613 .37019 .55746 Metetra
.743 .736 .845 r
.37019 .55746 .37362 .5613 .37935 .55781 .37594 .55391 Metetra
.743 .733 .842 r
.37594 .55391 .37935 .55781 .38509 .55429 .38169 .55033 Metetra
.744 .731 .84 r
.38169 .55033 .38509 .55429 .39084 .55074 .38745 .54671 Metetra
.744 .728 .837 r
.38745 .54671 .39084 .55074 .3966 .54714 .39322 .54305 Metetra
.744 .726 .835 r
.39322 .54305 .3966 .54714 .40236 .54351 .39899 .53934 Metetra
.745 .723 .832 r
.39899 .53934 .40236 .54351 .40813 .53983 .40477 .53559 Metetra
.746 .721 .829 r
.40477 .53559 .40813 .53983 .41391 .5361 .41056 .53178 Metetra
.746 .718 .825 r
.41056 .53178 .41391 .5361 .4197 .53232 .41636 .52792 Metetra
.747 .715 .822 r
.41636 .52792 .4197 .53232 .42549 .52848 .42216 .524 Metetra
.747 .712 .819 r
.42216 .524 .42549 .52848 .43129 .52458 .42797 .52 Metetra
.748 .709 .815 r
.42797 .52 .43129 .52458 .43709 .5206 .43379 .51593 Metetra
.749 .706 .811 r
.43379 .51593 .43709 .5206 .44291 .51655 .43961 .51178 Metetra
.751 .703 .807 r
.43961 .51178 .44291 .51655 .44873 .5124 .44544 .50753 Metetra
.752 .7 .802 r
.44544 .50753 .44873 .5124 .45455 .50816 .45127 .50317 Metetra
.753 .697 .798 r
.45127 .50317 .45455 .50816 .46038 .5038 .45712 .49869 Metetra
.755 .693 .793 r
.45712 .49869 .46038 .5038 .46622 .4993 .46296 .49407 Metetra
.757 .69 .787 r
.46296 .49407 .46622 .4993 .47206 .49465 .46882 .48927 Metetra
.76 .687 .781 r
.46882 .48927 .47206 .49465 .47791 .4898 .47467 .48429 Metetra
.763 .684 .775 r
.47467 .48429 .47791 .4898 .48377 .48472 .48054 .47907 Metetra
.767 .682 .768 r
.48054 .47907 .48377 .48472 .48962 .47933 .4864 .47355 Metetra
.773 .68 .761 r
.4864 .47355 .48962 .47933 .49548 .47357 .49227 .46769 Metetra
.78 .68 .754 r
.49227 .46769 .49548 .47357 .50135 .46732 .49814 .46139 Metetra
.789 .682 .746 r
.49814 .46139 .50135 .46732 .50721 .46041 .50401 .45457 Metetra
.801 .688 .74 r
.50401 .45457 .50721 .46041 .51307 .45265 .50988 .44714 Metetra
.817 .701 .737 r
.50988 .44714 .51307 .45265 .51892 .44384 .51575 .43909 Metetra
.838 .726 .74 r
.51575 .43909 .51892 .44384 .52477 .4341 .5216 .4306 Metetra
.806 .35 0 r
.5216 .4306 .52477 .4341 .53063 .44576 .52746 .44117 Metetra
.708 .271 0 r
.52746 .44117 .53063 .44576 .53655 .45657 .53337 .4514 Metetra
.603 .19 0 r
.53337 .4514 .53655 .45657 .54253 .46645 .53934 .46112 Metetra
0 0 .437 r
.53934 .46112 .54253 .46645 .54855 .47557 .54536 .47033 Metetra
0 0 .469 r
.54536 .47033 .54855 .47557 .55462 .48415 .55143 .47914 Metetra
0 0 .478 r
.55143 .47914 .55462 .48415 .56074 .49236 .55755 .48765 Metetra
0 0 .47 r
.55755 .48765 .56074 .49236 .56691 .50033 .56372 .49595 Metetra
0 0 .449 r
.56372 .49595 .56691 .50033 .57313 .50816 .56994 .50414 Metetra
.577 .116 0 r
.56994 .50414 .57313 .50816 .57939 .51594 .5762 .51228 Metetra
.618 .141 0 r
.5762 .51228 .57939 .51594 .5857 .52375 .58252 .52046 Metetra
.66 .17 0 r
.58252 .52046 .5857 .52375 .59207 .53166 .5889 .52874 Metetra
.701 .203 0 r
.5889 .52874 .59207 .53166 .59849 .53974 .59532 .53721 Metetra
.737 .237 0 r
.59532 .53721 .59849 .53974 .60496 .54811 .60181 .54593 Metetra
.769 .27 0 r
.60181 .54593 .60496 .54811 .6115 .55686 .60836 .55502 Metetra
.795 .303 0 r
.60836 .55502 .6115 .55686 .61811 .56616 .61498 .5646 Metetra
.815 .333 0 r
.61498 .5646 .61811 .56616 .6248 .57622 .62167 .57478 Metetra
.831 .358 .012 r
.62167 .57478 .6248 .57622 .63157 .58729 .62845 .58571 Metetra
.841 .375 .023 r
.62845 .58571 .63157 .58729 .63846 .59964 .63531 .59743 Metetra
.839 .376 0 r
.63531 .59743 .63846 .59964 .64545 .61321 .64226 .60978 Metetra
.885 .798 .758 r
.64226 .60978 .64545 .61321 .65162 .60236 .6485 .60036 Metetra
.91 .864 .782 r
.6485 .60036 .65162 .60236 .65781 .59264 .65474 .59148 Metetra
.913 .913 .805 r
.65474 .59148 .65781 .59264 .66404 .58413 .66101 .58333 Metetra
.902 .941 .824 r
.66101 .58333 .66404 .58413 .6703 .5766 .66729 .57589 Metetra
.886 .953 .843 r
.66729 .57589 .6703 .5766 .67658 .56982 .67359 .56904 Metetra
.871 .957 .859 r
.67359 .56904 .67658 .56982 .68288 .56358 .67991 .56267 Metetra
.858 .954 .873 r
.67991 .56267 .68288 .56358 .68921 .55777 .68624 .55671 Metetra
.846 .949 .884 r
.68624 .55671 .68921 .55777 .69555 .55228 .6926 .55106 Metetra
.836 .942 .893 r
.6926 .55106 .69555 .55228 .70191 .54706 .69896 .54567 Metetra
.828 .934 .9 r
.69896 .54567 .70191 .54706 .70828 .54203 .70534 .5405 Metetra
.82 .925 .904 r
.70534 .5405 .70828 .54203 .71466 .53718 .71173 .5355 Metetra
.813 .917 .908 r
.71173 .5355 .71466 .53718 .72105 .53247 .71813 .53066 Metetra
.807 .909 .91 r
.71813 .53066 .72105 .53247 .72746 .52789 .72455 .52595 Metetra
.802 .901 .912 r
.72455 .52595 .72746 .52789 .73387 .5234 .73097 .52134 Metetra
.797 .894 .913 r
.73097 .52134 .73387 .5234 .7403 .51901 .73741 .51683 Metetra
.792 .886 .913 r
.73741 .51683 .7403 .51901 .74674 .51469 .74386 .51241 Metetra
.788 .88 .913 r
.74386 .51241 .74674 .51469 .75319 .51045 .75032 .50806 Metetra
.784 .873 .913 r
.75032 .50806 .75319 .51045 .75965 .50626 .75679 .50379 Metetra
.781 .867 .913 r
.75679 .50379 .75965 .50626 .76612 .50214 .76327 .49957 Metetra
.777 .861 .912 r
.76327 .49957 .76612 .50214 .7726 .49805 .76976 .4954 Metetra
.775 .856 .911 r
.76976 .4954 .7726 .49805 .7791 .49402 .77626 .49128 Metetra
.771 .85 .91 r
.77626 .49128 .7791 .49402 .7856 .49002 .78277 .48721 Metetra
.769 .846 .909 r
.78277 .48721 .7856 .49002 .79211 .48606 .78929 .48317 Metetra
.766 .841 .908 r
.78929 .48317 .79211 .48606 .79863 .48213 .79583 .47917 Metetra
.764 .837 .907 r
.79583 .47917 .79863 .48213 .80517 .47823 .80237 .4752 Metetra
.762 .832 .906 r
.80237 .4752 .80517 .47823 .81171 .47435 .80892 .47126 Metetra
.761 .829 .905 r
.80892 .47126 .81171 .47435 .81827 .4705 .81548 .46734 Metetra
.758 .824 .904 r
.81548 .46734 .81827 .4705 .82483 .46667 .82206 .46344 Metetra
.757 .821 .903 r
.82206 .46344 .82483 .46667 .83141 .46286 .82864 .45957 Metetra
.755 .817 .901 r
.82864 .45957 .83141 .46286 .83799 .45906 .83524 .45572 Metetra
.754 .814 .9 r
.83524 .45572 .83799 .45906 .84459 .45528 .84184 .45188 Metetra
.752 .811 .899 r
.84184 .45188 .84459 .45528 .85119 .45151 .84845 .44805 Metetra
.751 .808 .897 r
.84845 .44805 .85119 .45151 .85781 .44775 .85508 .44423 Metetra
.75 .805 .896 r
.85508 .44423 .85781 .44775 .86443 .44399 .86171 .44042 Metetra
.749 .802 .895 r
.86171 .44042 .86443 .44399 .87107 .44024 .86835 .43662 Metetra
.748 .799 .893 r
.86835 .43662 .87107 .44024 .87771 .43649 .875 .43282 Metetra
.748 .796 .892 r
.875 .43282 .87771 .43649 .88436 .43275 .88167 .42902 Metetra
.747 .793 .89 r
.88167 .42902 .88436 .43275 .89102 .429 .88834 .42522 Metetra
.746 .791 .889 r
.88834 .42522 .89102 .429 .8977 .42526 .89502 .42142 Metetra
.745 .788 .887 r
.89502 .42142 .8977 .42526 .90438 .42151 .90171 .41762 Metetra
.746 .767 .87 r
.28694 .6017 .29053 .60484 .29618 .60154 .2926 .59836 Metetra
.746 .765 .868 r
.2926 .59836 .29618 .60154 .30183 .59823 .29826 .595 Metetra
.745 .763 .867 r
.29826 .595 .30183 .59823 .30749 .59492 .30393 .59164 Metetra
.745 .761 .865 r
.30393 .59164 .30749 .59492 .31315 .59159 .3096 .58827 Metetra
.745 .759 .864 r
.3096 .58827 .31315 .59159 .31882 .58826 .31529 .58489 Metetra
.745 .757 .862 r
.31529 .58489 .31882 .58826 .3245 .58491 .32098 .58149 Metetra
.744 .754 .861 r
.32098 .58149 .3245 .58491 .33019 .58155 .32667 .57809 Metetra
.744 .752 .859 r
.32667 .57809 .33019 .58155 .33588 .57817 .33238 .57466 Metetra
.744 .75 .857 r
.33238 .57466 .33588 .57817 .34158 .57478 .33809 .57122 Metetra
.744 .748 .855 r
.33809 .57122 .34158 .57478 .34729 .57137 .34381 .56776 Metetra
.744 .746 .853 r
.34381 .56776 .34729 .57137 .353 .56793 .34953 .56427 Metetra
.744 .744 .851 r
.34953 .56427 .353 .56793 .35873 .56447 .35527 .56076 Metetra
.745 .742 .849 r
.35527 .56076 .35873 .56447 .36446 .56098 .36101 .55721 Metetra
.745 .74 .847 r
.36101 .55721 .36446 .56098 .37019 .55746 .36676 .55363 Metetra
.745 .737 .845 r
.36676 .55363 .37019 .55746 .37594 .55391 .37251 .55003 Metetra
.746 .735 .842 r
.37251 .55003 .37594 .55391 .38169 .55033 .37828 .54638 Metetra
.746 .733 .84 r
.37828 .54638 .38169 .55033 .38745 .54671 .38405 .5427 Metetra
.746 .73 .837 r
.38405 .5427 .38745 .54671 .39322 .54305 .38982 .53898 Metetra
.747 .728 .834 r
.38982 .53898 .39322 .54305 .39899 .53934 .39561 .53521 Metetra
.748 .726 .832 r
.39561 .53521 .39899 .53934 .40477 .53559 .4014 .53138 Metetra
.748 .723 .829 r
.4014 .53138 .40477 .53559 .41056 .53178 .4072 .5275 Metetra
.749 .721 .826 r
.4072 .5275 .41056 .53178 .41636 .52792 .41301 .52356 Metetra
.75 .718 .822 r
.41301 .52356 .41636 .52792 .42216 .524 .41882 .51956 Metetra
.751 .716 .819 r
.41882 .51956 .42216 .524 .42797 .52 .42464 .51548 Metetra
.752 .713 .815 r
.42464 .51548 .42797 .52 .43379 .51593 .43047 .51133 Metetra
.753 .71 .812 r
.43047 .51133 .43379 .51593 .43961 .51178 .4363 .50708 Metetra
.754 .707 .808 r
.4363 .50708 .43961 .51178 .44544 .50753 .44214 .50274 Metetra
.756 .705 .803 r
.44214 .50274 .44544 .50753 .45127 .50317 .44798 .49829 Metetra
.758 .702 .799 r
.44798 .49829 .45127 .50317 .45712 .49869 .45384 .49371 Metetra
.76 .7 .794 r
.45384 .49371 .45712 .49869 .46296 .49407 .45969 .48898 Metetra
.763 .697 .789 r
.45969 .48898 .46296 .49407 .46882 .48927 .46555 .48409 Metetra
.766 .695 .784 r
.46555 .48409 .46882 .48927 .47467 .48429 .47142 .479 Metetra
.769 .693 .779 r
.47142 .479 .47467 .48429 .48054 .47907 .47729 .47369 Metetra
.774 .692 .773 r
.47729 .47369 .48054 .47907 .4864 .47355 .48317 .46811 Metetra
.78 .692 .767 r
.48317 .46811 .4864 .47355 .49227 .46769 .48905 .46222 Metetra
.786 .693 .762 r
.48905 .46222 .49227 .46769 .49814 .46139 .49493 .45597 Metetra
.795 .698 .757 r
.49493 .45597 .49814 .46139 .50401 .45457 .50081 .44932 Metetra
.806 .705 .754 r
.50081 .44932 .50401 .45457 .50988 .44714 .50668 .44224 Metetra
.82 .719 .754 r
.50668 .44224 .50988 .44714 .51575 .43909 .51255 .43478 Metetra
.836 .741 .759 r
.51255 .43478 .51575 .43909 .5216 .4306 .51842 .42708 Metetra
.782 .318 0 r
.51842 .42708 .5216 .4306 .52746 .44117 .52427 .43703 Metetra
.724 .271 0 r
.52427 .43703 .52746 .44117 .53337 .4514 .53018 .44682 Metetra
.658 .219 0 r
.53018 .44682 .53337 .4514 .53934 .46112 .53615 .45635 Metetra
.607 .176 0 r
.53615 .45635 .53934 .46112 .54536 .47033 .54216 .46555 Metetra
0 0 .405 r
.54216 .46555 .54536 .47033 .55143 .47914 .54823 .47448 Metetra
0 0 .418 r
.54823 .47448 .55143 .47914 .55755 .48765 .55435 .48319 Metetra
0 0 .416 r
.55435 .48319 .55755 .48765 .56372 .49595 .56052 .49174 Metetra
.589 .135 0 r
.56052 .49174 .56372 .49595 .56994 .50414 .56673 .50021 Metetra
.615 .148 0 r
.56673 .50021 .56994 .50414 .5762 .51228 .57301 .50865 Metetra
.645 .167 0 r
.57301 .50865 .5762 .51228 .58252 .52046 .57933 .51713 Metetra
.677 .19 0 r
.57933 .51713 .58252 .52046 .5889 .52874 .58571 .52571 Metetra
.708 .216 0 r
.58571 .52571 .5889 .52874 .59532 .53721 .59214 .53446 Metetra
.738 .242 0 r
.59214 .53446 .59532 .53721 .60181 .54593 .59863 .54346 Metetra
.763 .269 0 r
.59863 .54346 .60181 .54593 .60836 .55502 .60519 .55278 Metetra
.785 .294 0 r
.60519 .55278 .60836 .55502 .61498 .5646 .61181 .5625 Metetra
.801 .316 0 r
.61181 .5625 .61498 .5646 .62167 .57478 .61851 .57271 Metetra
.813 .334 0 r
.61851 .57271 .62167 .57478 .62845 .58571 .62528 .58344 Metetra
.817 .343 0 r
.62528 .58344 .62845 .58571 .63531 .59743 .63213 .5947 Metetra
.807 .337 0 r
.63213 .5947 .63531 .59743 .64226 .60978 .63906 .60634 Metetra
.873 .8 .776 r
.63906 .60634 .64226 .60978 .6485 .60036 .64535 .59782 Metetra
.889 .843 .793 r
.64535 .59782 .6485 .60036 .65474 .59148 .65163 .5896 Metetra
.895 .88 .812 r
.65163 .5896 .65474 .59148 .66101 .58333 .65793 .58184 Metetra
.892 .907 .829 r
.65793 .58184 .66101 .58333 .66729 .57589 .66424 .57457 Metetra
.883 .922 .845 r
.66424 .57457 .66729 .57589 .67359 .56904 .67056 .56777 Metetra
.872 .929 .858 r
.67056 .56777 .67359 .56904 .67991 .56267 .67689 .56137 Metetra
.861 .931 .87 r
.67689 .56137 .67991 .56267 .68624 .55671 .68325 .55532 Metetra
.85 .929 .88 r
.68325 .55532 .68624 .55671 .6926 .55106 .68961 .54957 Metetra
.841 .925 .887 r
.68961 .54957 .6926 .55106 .69896 .54567 .69599 .54407 Metetra
.832 .92 .893 r
.69599 .54407 .69896 .54567 .70534 .5405 .70238 .53877 Metetra
.824 .913 .898 r
.70238 .53877 .70534 .5405 .71173 .5355 .70878 .53366 Metetra
.818 .907 .902 r
.70878 .53366 .71173 .5355 .71813 .53066 .71519 .5287 Metetra
.811 .9 .904 r
.71519 .5287 .71813 .53066 .72455 .52595 .72162 .52388 Metetra
.806 .893 .906 r
.72162 .52388 .72455 .52595 .73097 .52134 .72805 .51916 Metetra
.8 .887 .907 r
.72805 .51916 .73097 .52134 .73741 .51683 .7345 .51456 Metetra
.796 .881 .908 r
.7345 .51456 .73741 .51683 .74386 .51241 .74096 .51004 Metetra
.791 .874 .908 r
.74096 .51004 .74386 .51241 .75032 .50806 .74743 .5056 Metetra
.787 .869 .909 r
.74743 .5056 .75032 .50806 .75679 .50379 .75391 .50123 Metetra
.783 .863 .909 r
.75391 .50123 .75679 .50379 .76327 .49957 .7604 .49693 Metetra
.78 .858 .908 r
.7604 .49693 .76327 .49957 .76976 .4954 .7669 .49268 Metetra
.777 .853 .907 r
.7669 .49268 .76976 .4954 .77626 .49128 .77341 .48849 Metetra
.774 .848 .907 r
.77341 .48849 .77626 .49128 .78277 .48721 .77993 .48434 Metetra
.771 .843 .906 r
.77993 .48434 .78277 .48721 .78929 .48317 .78646 .48023 Metetra
.769 .839 .905 r
.78646 .48023 .78929 .48317 .79583 .47917 .793 .47616 Metetra
.767 .835 .904 r
.793 .47616 .79583 .47917 .80237 .4752 .79955 .47213 Metetra
.765 .831 .903 r
.79955 .47213 .80237 .4752 .80892 .47126 .80611 .46812 Metetra
.762 .827 .902 r
.80611 .46812 .80892 .47126 .81548 .46734 .81269 .46414 Metetra
.761 .824 .901 r
.81269 .46414 .81548 .46734 .82206 .46344 .81927 .46018 Metetra
.759 .82 .9 r
.81927 .46018 .82206 .46344 .82864 .45957 .82586 .45625 Metetra
.757 .816 .899 r
.82586 .45625 .82864 .45957 .83524 .45572 .83247 .45234 Metetra
.756 .813 .898 r
.83247 .45234 .83524 .45572 .84184 .45188 .83908 .44844 Metetra
.754 .81 .897 r
.83908 .44844 .84184 .45188 .84845 .44805 .8457 .44456 Metetra
.753 .807 .896 r
.8457 .44456 .84845 .44805 .85508 .44423 .85233 .44069 Metetra
.752 .804 .895 r
.85233 .44069 .85508 .44423 .86171 .44042 .85898 .43683 Metetra
.751 .801 .893 r
.85898 .43683 .86171 .44042 .86835 .43662 .86563 .43297 Metetra
.75 .799 .892 r
.86563 .43297 .86835 .43662 .875 .43282 .87229 .42912 Metetra
.749 .796 .891 r
.87229 .42912 .875 .43282 .88167 .42902 .87896 .42527 Metetra
.748 .794 .889 r
.87896 .42527 .88167 .42902 .88834 .42522 .88564 .42142 Metetra
.748 .791 .888 r
.88564 .42142 .88834 .42522 .89502 .42142 .89233 .41757 Metetra
.747 .789 .886 r
.89233 .41757 .89502 .42142 .90171 .41762 .89902 .41372 Metetra
.748 .767 .869 r
.28334 .59856 .28694 .6017 .2926 .59836 .289 .59517 Metetra
.747 .765 .867 r
.289 .59517 .2926 .59836 .29826 .595 .29468 .59177 Metetra
.747 .763 .866 r
.29468 .59177 .29826 .595 .30393 .59164 .30036 .58836 Metetra
.746 .761 .865 r
.30036 .58836 .30393 .59164 .3096 .58827 .30604 .58495 Metetra
.747 .759 .863 r
.30604 .58495 .3096 .58827 .31529 .58489 .31174 .58152 Metetra
.746 .757 .862 r
.31174 .58152 .31529 .58489 .32098 .58149 .31744 .57808 Metetra
.746 .755 .86 r
.31744 .57808 .32098 .58149 .32667 .57809 .32315 .57462 Metetra
.746 .753 .858 r
.32315 .57462 .32667 .57809 .33238 .57466 .32886 .57115 Metetra
.746 .751 .857 r
.32886 .57115 .33238 .57466 .33809 .57122 .33459 .56766 Metetra
.746 .749 .855 r
.33459 .56766 .33809 .57122 .34381 .56776 .34032 .56415 Metetra
.746 .747 .853 r
.34032 .56415 .34381 .56776 .34953 .56427 .34605 .56062 Metetra
.746 .745 .851 r
.34605 .56062 .34953 .56427 .35527 .56076 .3518 .55705 Metetra
.747 .743 .849 r
.3518 .55705 .35527 .56076 .36101 .55721 .35755 .55345 Metetra
.747 .741 .847 r
.35755 .55345 .36101 .55721 .36676 .55363 .36331 .54983 Metetra
.748 .739 .844 r
.36331 .54983 .36676 .55363 .37251 .55003 .36907 .54616 Metetra
.748 .737 .842 r
.36907 .54616 .37251 .55003 .37828 .54638 .37485 .54246 Metetra
.748 .735 .84 r
.37485 .54246 .37828 .54638 .38405 .5427 .38063 .53872 Metetra
.749 .733 .837 r
.38063 .53872 .38405 .5427 .38982 .53898 .38642 .53493 Metetra
.75 .731 .835 r
.38642 .53493 .38982 .53898 .39561 .53521 .39221 .5311 Metetra
.75 .728 .832 r
.39221 .5311 .39561 .53521 .4014 .53138 .39801 .52721 Metetra
.751 .726 .829 r
.39801 .52721 .4014 .53138 .4072 .5275 .40382 .52326 Metetra
.752 .724 .826 r
.40382 .52326 .4072 .5275 .41301 .52356 .40964 .51926 Metetra
.753 .721 .823 r
.40964 .51926 .41301 .52356 .41882 .51956 .41546 .51518 Metetra
.754 .719 .819 r
.41546 .51518 .41882 .51956 .42464 .51548 .42129 .51103 Metetra
.755 .717 .816 r
.42129 .51103 .42464 .51548 .43047 .51133 .42713 .5068 Metetra
.757 .714 .812 r
.42713 .5068 .43047 .51133 .4363 .50708 .43297 .50247 Metetra
.758 .712 .809 r
.43297 .50247 .4363 .50708 .44214 .50274 .43882 .49805 Metetra
.76 .71 .805 r
.43882 .49805 .44214 .50274 .44798 .49829 .44468 .49351 Metetra
.762 .708 .801 r
.44468 .49351 .44798 .49829 .45384 .49371 .45054 .48885 Metetra
.764 .706 .796 r
.45054 .48885 .45384 .49371 .45969 .48898 .45641 .48405 Metetra
.767 .704 .792 r
.45641 .48405 .45969 .48898 .46555 .48409 .46228 .47908 Metetra
.771 .702 .787 r
.46228 .47908 .46555 .48409 .47142 .479 .46816 .47393 Metetra
.774 .702 .783 r
.46816 .47393 .47142 .479 .47729 .47369 .47404 .46857 Metetra
.779 .702 .778 r
.47404 .46857 .47729 .47369 .48317 .46811 .47992 .46297 Metetra
.785 .703 .774 r
.47992 .46297 .48317 .46811 .48905 .46222 .48581 .45711 Metetra
.791 .706 .77 r
.48581 .45711 .48905 .46222 .49493 .45597 .4917 .45095 Metetra
.799 .711 .767 r
.4917 .45095 .49493 .45597 .50081 .44932 .49758 .44449 Metetra
.809 .719 .765 r
.49758 .44449 .50081 .44932 .50668 .44224 .50347 .43772 Metetra
.82 .732 .767 r
.50347 .43772 .50668 .44224 .51255 .43478 .50935 .43071 Metetra
.833 .75 .772 r
.50935 .43071 .51255 .43478 .51842 .42708 .51523 .42356 Metetra
.764 .296 0 r
.51523 .42356 .51842 .42708 .52427 .43703 .52108 .4331 Metetra
.727 .266 0 r
.52108 .4331 .52427 .43703 .53018 .44682 .52698 .4426 Metetra
.685 .232 0 r
.52698 .4426 .53018 .44682 .53615 .45635 .53294 .45195 Metetra
.649 .202 0 r
.53294 .45195 .53615 .45635 .54216 .46555 .53895 .46112 Metetra
.625 .18 0 r
.53895 .46112 .54216 .46555 .54823 .47448 .54502 .47011 Metetra
.614 .166 0 r
.54502 .47011 .54823 .47448 .55435 .48319 .55113 .47895 Metetra
.615 .162 0 r
.55113 .47895 .55435 .48319 .56052 .49174 .5573 .48768 Metetra
.626 .164 0 r
.5573 .48768 .56052 .49174 .56673 .50021 .56352 .49636 Metetra
.643 .173 0 r
.56352 .49636 .56673 .50021 .57301 .50865 .56979 .50503 Metetra
.665 .187 0 r
.56979 .50503 .57301 .50865 .57933 .51713 .57612 .51374 Metetra
.689 .204 0 r
.57612 .51374 .57933 .51713 .58571 .52571 .5825 .52256 Metetra
.713 .224 0 r
.5825 .52256 .58571 .52571 .59214 .53446 .58894 .53154 Metetra
.735 .245 0 r
.58894 .53154 .59214 .53446 .59863 .54346 .59544 .54074 Metetra
.756 .265 0 r
.59544 .54074 .59863 .54346 .60519 .55278 .602 .55021 Metetra
.773 .284 0 r
.602 .55021 .60519 .55278 .61181 .5625 .60863 .56001 Metetra
.785 .3 0 r
.60863 .56001 .61181 .5625 .61851 .57271 .61532 .57018 Metetra
.793 .312 0 r
.61532 .57018 .61851 .57271 .62528 .58344 .62209 .58075 Metetra
.793 .316 0 r
.62209 .58075 .62528 .58344 .63213 .5947 .62893 .59168 Metetra
.783 .31 0 r
.62893 .59168 .63213 .5947 .63906 .60634 .63584 .60288 Metetra
.863 .802 .788 r
.63584 .60288 .63906 .60634 .64535 .59782 .64217 .59499 Metetra
.874 .832 .802 r
.64217 .59499 .64535 .59782 .65163 .5896 .64849 .58727 Metetra
.88 .86 .817 r
.64849 .58727 .65163 .5896 .65793 .58184 .65481 .57985 Metetra
.88 .882 .831 r
.65481 .57985 .65793 .58184 .66424 .57457 .66115 .57279 Metetra
.875 .897 .844 r
.66115 .57279 .66424 .57457 .67056 .56777 .66749 .56608 Metetra
.868 .906 .856 r
.66749 .56608 .67056 .56777 .67689 .56137 .67385 .5597 Metetra
.859 .91 .866 r
.67385 .5597 .67689 .56137 .68325 .55532 .68022 .55363 Metetra
.85 .911 .875 r
.68022 .55363 .68325 .55532 .68961 .54957 .6866 .54782 Metetra
.842 .909 .882 r
.6866 .54782 .68961 .54957 .69599 .54407 .69299 .54224 Metetra
.834 .906 .888 r
.69299 .54224 .69599 .54407 .70238 .53877 .69939 .53686 Metetra
.827 .901 .892 r
.69939 .53686 .70238 .53877 .70878 .53366 .7058 .53165 Metetra
.82 .896 .896 r
.7058 .53165 .70878 .53366 .71519 .5287 .71223 .5266 Metetra
.814 .891 .899 r
.71223 .5266 .71519 .5287 .72162 .52388 .71867 .52168 Metetra
.808 .886 .901 r
.71867 .52168 .72162 .52388 .72805 .51916 .72511 .51688 Metetra
.803 .88 .902 r
.72511 .51688 .72805 .51916 .7345 .51456 .73157 .51218 Metetra
.798 .874 .903 r
.73157 .51218 .7345 .51456 .74096 .51004 .73804 .50757 Metetra
.794 .869 .904 r
.73804 .50757 .74096 .51004 .74743 .5056 .74452 .50305 Metetra
.79 .864 .904 r
.74452 .50305 .74743 .5056 .75391 .50123 .75101 .4986 Metetra
.786 .859 .904 r
.75101 .4986 .75391 .50123 .7604 .49693 .75751 .49422 Metetra
.783 .854 .904 r
.75751 .49422 .7604 .49693 .7669 .49268 .76402 .4899 Metetra
.78 .85 .904 r
.76402 .4899 .7669 .49268 .77341 .48849 .77054 .48563 Metetra
.776 .845 .904 r
.77054 .48563 .77341 .48849 .77993 .48434 .77707 .48141 Metetra
.774 .841 .903 r
.77707 .48141 .77993 .48434 .78646 .48023 .78361 .47723 Metetra
.771 .837 .903 r
.78361 .47723 .78646 .48023 .793 .47616 .79016 .4731 Metetra
.769 .833 .902 r
.79016 .4731 .793 .47616 .79955 .47213 .79672 .469 Metetra
.766 .829 .901 r
.79672 .469 .79955 .47213 .80611 .46812 .80329 .46493 Metetra
.765 .826 .9 r
.80329 .46493 .80611 .46812 .81269 .46414 .80987 .46089 Metetra
.762 .822 .899 r
.80987 .46089 .81269 .46414 .81927 .46018 .81647 .45688 Metetra
.761 .819 .898 r
.81647 .45688 .81927 .46018 .82586 .45625 .82307 .45289 Metetra
.759 .816 .897 r
.82307 .45289 .82586 .45625 .83247 .45234 .82968 .44892 Metetra
.757 .812 .896 r
.82968 .44892 .83247 .45234 .83908 .44844 .8363 .44498 Metetra
.756 .81 .895 r
.8363 .44498 .83908 .44844 .8457 .44456 .84294 .44104 Metetra
.755 .807 .894 r
.84294 .44104 .8457 .44456 .85233 .44069 .84958 .43712 Metetra
.754 .804 .893 r
.84958 .43712 .85233 .44069 .85898 .43683 .85623 .4332 Metetra
.753 .801 .892 r
.85623 .4332 .85898 .43683 .86563 .43297 .86289 .42929 Metetra
.752 .798 .89 r
.86289 .42929 .86563 .43297 .87229 .42912 .86956 .42539 Metetra
.751 .796 .889 r
.86956 .42539 .87229 .42912 .87896 .42527 .87624 .42149 Metetra
.75 .793 .888 r
.87624 .42149 .87896 .42527 .88564 .42142 .88293 .4176 Metetra
.75 .791 .887 r
.88293 .4176 .88564 .42142 .89233 .41757 .88962 .4137 Metetra
.749 .789 .885 r
.88962 .4137 .89233 .41757 .89902 .41372 .89633 .4098 Metetra
.749 .767 .868 r
.27972 .5954 .28334 .59856 .289 .59517 .2854 .59197 Metetra
.749 .765 .866 r
.2854 .59197 .289 .59517 .29468 .59177 .29108 .58852 Metetra
.748 .763 .865 r
.29108 .58852 .29468 .59177 .30036 .58836 .29677 .58508 Metetra
.748 .762 .864 r
.29677 .58508 .30036 .58836 .30604 .58495 .30247 .58162 Metetra
.748 .76 .862 r
.30247 .58162 .30604 .58495 .31174 .58152 .30818 .57815 Metetra
.748 .758 .861 r
.30818 .57815 .31174 .58152 .31744 .57808 .31389 .57466 Metetra
.748 .756 .859 r
.31389 .57466 .31744 .57808 .32315 .57462 .31961 .57117 Metetra
.748 .754 .858 r
.31961 .57117 .32315 .57462 .32886 .57115 .32533 .56765 Metetra
.748 .753 .856 r
.32533 .56765 .32886 .57115 .33459 .56766 .33107 .56411 Metetra
.748 .75 .854 r
.33107 .56411 .33459 .56766 .34032 .56415 .33681 .56056 Metetra
.748 .749 .853 r
.33681 .56056 .34032 .56415 .34605 .56062 .34256 .55697 Metetra
.748 .747 .851 r
.34256 .55697 .34605 .56062 .3518 .55705 .34831 .55336 Metetra
.749 .745 .849 r
.34831 .55336 .3518 .55705 .35755 .55345 .35407 .54971 Metetra
.749 .743 .846 r
.35407 .54971 .35755 .55345 .36331 .54983 .35984 .54603 Metetra
.749 .741 .844 r
.35984 .54603 .36331 .54983 .36907 .54616 .36562 .54231 Metetra
.75 .739 .842 r
.36562 .54231 .36907 .54616 .37485 .54246 .3714 .53856 Metetra
.751 .737 .84 r
.3714 .53856 .37485 .54246 .38063 .53872 .3772 .53476 Metetra
.751 .735 .837 r
.3772 .53476 .38063 .53872 .38642 .53493 .38299 .53092 Metetra
.752 .733 .835 r
.38299 .53092 .38642 .53493 .39221 .5311 .3888 .52703 Metetra
.753 .731 .832 r
.3888 .52703 .39221 .5311 .39801 .52721 .39461 .52308 Metetra
.754 .729 .829 r
.39461 .52308 .39801 .52721 .40382 .52326 .40043 .51907 Metetra
.755 .727 .826 r
.40043 .51907 .40382 .52326 .40964 .51926 .40626 .515 Metetra
.756 .725 .823 r
.40626 .515 .40964 .51926 .41546 .51518 .4121 .51086 Metetra
.757 .723 .82 r
.4121 .51086 .41546 .51518 .42129 .51103 .41794 .50664 Metetra
.758 .72 .817 r
.41794 .50664 .42129 .51103 .42713 .5068 .42378 .50234 Metetra
.76 .719 .814 r
.42378 .50234 .42713 .5068 .43297 .50247 .42964 .49794 Metetra
.762 .716 .81 r
.42964 .49794 .43297 .50247 .43882 .49805 .4355 .49345 Metetra
.764 .715 .806 r
.4355 .49345 .43882 .49805 .44468 .49351 .44136 .48885 Metetra
.766 .713 .803 r
.44136 .48885 .44468 .49351 .45054 .48885 .44723 .48412 Metetra
.769 .712 .799 r
.44723 .48412 .45054 .48885 .45641 .48405 .45311 .47925 Metetra
.771 .71 .795 r
.45311 .47925 .45641 .48405 .46228 .47908 .45899 .47423 Metetra
.775 .71 .791 r
.45899 .47423 .46228 .47908 .46816 .47393 .46488 .46904 Metetra
.779 .71 .787 r
.46488 .46904 .46816 .47393 .47404 .46857 .47077 .46367 Metetra
.783 .71 .783 r
.47077 .46367 .47404 .46857 .47992 .46297 .47666 .45808 Metetra
.789 .712 .78 r
.47666 .45808 .47992 .46297 .48581 .45711 .48256 .45227 Metetra
.795 .716 .777 r
.48256 .45227 .48581 .45711 .4917 .45095 .48845 .44622 Metetra
.802 .721 .776 r
.48845 .44622 .4917 .45095 .49758 .44449 .49435 .43994 Metetra
.81 .729 .775 r
.49435 .43994 .49758 .44449 .50347 .43772 .50025 .43345 Metetra
.819 .741 .777 r
.50025 .43345 .50347 .43772 .50935 .43071 .50614 .42678 Metetra
.829 .755 .782 r
.50614 .42678 .50935 .43071 .51523 .42356 .51203 .42001 Metetra
.751 .281 0 r
.51203 .42001 .51523 .42356 .52108 .4331 .51787 .4293 Metetra
.726 .261 0 r
.51787 .4293 .52108 .4331 .52698 .4426 .52377 .43859 Metetra
.699 .238 0 r
.52377 .43859 .52698 .4426 .53294 .45195 .52973 .44781 Metetra
.674 .217 0 r
.52973 .44781 .53294 .45195 .53895 .46112 .53574 .45694 Metetra
.656 .2 0 r
.53574 .45694 .53895 .46112 .54502 .47011 .5418 .46595 Metetra
.647 .189 0 r
.5418 .46595 .54502 .47011 .55113 .47895 .54791 .47488 Metetra
.646 .185 0 r
.54791 .47488 .55113 .47895 .5573 .48768 .55408 .48374 Metetra
.652 .186 0 r
.55408 .48374 .5573 .48768 .56352 .49636 .5603 .49257 Metetra
.664 .192 0 r
.5603 .49257 .56352 .49636 .56979 .50503 .56657 .50142 Metetra
.679 .202 0 r
.56657 .50142 .56979 .50503 .57612 .51374 .5729 .51032 Metetra
.697 .215 0 r
.5729 .51032 .57612 .51374 .5825 .52256 .57928 .51932 Metetra
.715 .23 0 r
.57928 .51932 .5825 .52256 .58894 .53154 .58572 .52847 Metetra
.733 .245 0 r
.58572 .52847 .58894 .53154 .59544 .54074 .59223 .53781 Metetra
.748 .261 0 r
.59223 .53781 .59544 .54074 .602 .55021 .59879 .54738 Metetra
.761 .275 0 r
.59879 .54738 .602 .55021 .60863 .56001 .60542 .55722 Metetra
.771 .287 0 r
.60542 .55722 .60863 .56001 .61532 .57018 .61211 .56736 Metetra
.775 .294 0 r
.61211 .56736 .61532 .57018 .62209 .58075 .61887 .5778 Metetra
.774 .296 0 r
.61887 .5778 .62209 .58075 .62893 .59168 .62571 .5885 Metetra
.764 .29 0 r
.62571 .5885 .62893 .59168 .63584 .60288 .63261 .5994 Metetra
.855 .802 .798 r
.63261 .5994 .63584 .60288 .64217 .59499 .63896 .59196 Metetra
.863 .825 .808 r
.63896 .59196 .64217 .59499 .64849 .58727 .64532 .58463 Metetra
.868 .846 .82 r
.64532 .58463 .64849 .58727 .65481 .57985 .65167 .57751 Metetra
.868 .864 .832 r
.65167 .57751 .65481 .57985 .66115 .57279 .65803 .57065 Metetra
.866 .877 .843 r
.65803 .57065 .66115 .57279 .66749 .56608 .6644 .56406 Metetra
.861 .886 .853 r
.6644 .56406 .66749 .56608 .67385 .5597 .67078 .55773 Metetra
.855 .892 .862 r
.67078 .55773 .67385 .5597 .68022 .55363 .67716 .55166 Metetra
.848 .894 .87 r
.67716 .55166 .68022 .55363 .6866 .54782 .68356 .54582 Metetra
.841 .894 .877 r
.68356 .54582 .6866 .54782 .69299 .54224 .68996 .5402 Metetra
.834 .893 .882 r
.68996 .5402 .69299 .54224 .69939 .53686 .69638 .53475 Metetra
.828 .89 .887 r
.69638 .53475 .69939 .53686 .7058 .53165 .70281 .52948 Metetra
.821 .886 .89 r
.70281 .52948 .7058 .53165 .71223 .5266 .70924 .52434 Metetra
.815 .882 .893 r
.70924 .52434 .71223 .5266 .71867 .52168 .71569 .51935 Metetra
.81 .878 .895 r
.71569 .51935 .71867 .52168 .72511 .51688 .72215 .51447 Metetra
.805 .873 .897 r
.72215 .51447 .72511 .51688 .73157 .51218 .72862 .50969 Metetra
.801 .869 .898 r
.72862 .50969 .73157 .51218 .73804 .50757 .7351 .505 Metetra
.796 .864 .899 r
.7351 .505 .73804 .50757 .74452 .50305 .74159 .50041 Metetra
.792 .859 .9 r
.74159 .50041 .74452 .50305 .75101 .4986 .74809 .49589 Metetra
.788 .855 .9 r
.74809 .49589 .75101 .4986 .75751 .49422 .7546 .49143 Metetra
.785 .851 .9 r
.7546 .49143 .75751 .49422 .76402 .4899 .76112 .48704 Metetra
.781 .846 .9 r
.76112 .48704 .76402 .4899 .77054 .48563 .76765 .4827 Metetra
.778 .842 .9 r
.76765 .4827 .77054 .48563 .77707 .48141 .77419 .47841 Metetra
.775 .838 .9 r
.77419 .47841 .77707 .48141 .78361 .47723 .78074 .47418 Metetra
.773 .834 .899 r
.78074 .47418 .78361 .47723 .79016 .4731 .7873 .46998 Metetra
.77 .831 .899 r
.7873 .46998 .79016 .4731 .79672 .469 .79387 .46583 Metetra
.768 .827 .898 r
.79387 .46583 .79672 .469 .80329 .46493 .80045 .4617 Metetra
.766 .824 .898 r
.80045 .4617 .80329 .46493 .80987 .46089 .80705 .4576 Metetra
.764 .821 .897 r
.80705 .4576 .80987 .46089 .81647 .45688 .81365 .45353 Metetra
.762 .817 .896 r
.81365 .45353 .81647 .45688 .82307 .45289 .82026 .44949 Metetra
.76 .814 .895 r
.82026 .44949 .82307 .45289 .82968 .44892 .82688 .44547 Metetra
.759 .812 .895 r
.82688 .44547 .82968 .44892 .8363 .44498 .83351 .44147 Metetra
.757 .809 .894 r
.83351 .44147 .8363 .44498 .84294 .44104 .84016 .43749 Metetra
.756 .806 .893 r
.84016 .43749 .84294 .44104 .84958 .43712 .84681 .43351 Metetra
.755 .804 .891 r
.84681 .43351 .84958 .43712 .85623 .4332 .85347 .42955 Metetra
.754 .801 .89 r
.85347 .42955 .85623 .4332 .86289 .42929 .86014 .42559 Metetra
.753 .799 .889 r
.86014 .42559 .86289 .42929 .86956 .42539 .86682 .42164 Metetra
.752 .796 .888 r
.86682 .42164 .86956 .42539 .87624 .42149 .8735 .41769 Metetra
.752 .794 .887 r
.8735 .41769 .87624 .42149 .88293 .4176 .8802 .41375 Metetra
.751 .791 .886 r
.8802 .41375 .88293 .4176 .88962 .4137 .88691 .40981 Metetra
.75 .789 .885 r
.88691 .40981 .88962 .4137 .89633 .4098 .89362 .40586 Metetra
.75 .767 .867 r
.27608 .59223 .27972 .5954 .2854 .59197 .28177 .58875 Metetra
.75 .766 .866 r
.28177 .58875 .2854 .59197 .29108 .58852 .28747 .58527 Metetra
.75 .764 .864 r
.28747 .58527 .29108 .58852 .29677 .58508 .29317 .58178 Metetra
.749 .762 .863 r
.29317 .58178 .29677 .58508 .30247 .58162 .29888 .57829 Metetra
.749 .761 .862 r
.29888 .57829 .30247 .58162 .30818 .57815 .3046 .57477 Metetra
.749 .759 .86 r
.3046 .57477 .30818 .57815 .31389 .57466 .31032 .57125 Metetra
.749 .757 .859 r
.31032 .57125 .31389 .57466 .31961 .57117 .31605 .56771 Metetra
.749 .755 .857 r
.31605 .56771 .31961 .57117 .32533 .56765 .32179 .56414 Metetra
.749 .753 .856 r
.32179 .56414 .32533 .56765 .33107 .56411 .32753 .56057 Metetra
.749 .752 .854 r
.32753 .56057 .33107 .56411 .33681 .56056 .33329 .55697 Metetra
.75 .75 .852 r
.33329 .55697 .33681 .56056 .34256 .55697 .33904 .55334 Metetra
.75 .748 .851 r
.33904 .55334 .34256 .55697 .34831 .55336 .34481 .54968 Metetra
.751 .747 .848 r
.34481 .54968 .34831 .55336 .35407 .54971 .35058 .54598 Metetra
.751 .745 .846 r
.35058 .54598 .35407 .54971 .35984 .54603 .35636 .54226 Metetra
.752 .743 .844 r
.35636 .54226 .35984 .54603 .36562 .54231 .36215 .53849 Metetra
.752 .741 .842 r
.36215 .53849 .36562 .54231 .3714 .53856 .36795 .53468 Metetra
.753 .739 .84 r
.36795 .53468 .3714 .53856 .3772 .53476 .37375 .53084 Metetra
.754 .737 .838 r
.37375 .53084 .3772 .53476 .38299 .53092 .37956 .52694 Metetra
.754 .736 .835 r
.37956 .52694 .38299 .53092 .3888 .52703 .38538 .52299 Metetra
.755 .734 .832 r
.38538 .52299 .3888 .52703 .39461 .52308 .3912 .51899 Metetra
.756 .732 .83 r
.3912 .51899 .39461 .52308 .40043 .51907 .39703 .51492 Metetra
.757 .73 .827 r
.39703 .51492 .40043 .51907 .40626 .515 .40287 .51079 Metetra
.759 .728 .824 r
.40287 .51079 .40626 .515 .4121 .51086 .40871 .5066 Metetra
.76 .726 .821 r
.40871 .5066 .4121 .51086 .41794 .50664 .41456 .50232 Metetra
.762 .725 .818 r
.41456 .50232 .41794 .50664 .42378 .50234 .42042 .49795 Metetra
.763 .723 .815 r
.42042 .49795 .42378 .50234 .42964 .49794 .42629 .4935 Metetra
.765 .721 .812 r
.42629 .4935 .42964 .49794 .4355 .49345 .43216 .48895 Metetra
.767 .72 .808 r
.43216 .48895 .4355 .49345 .44136 .48885 .43803 .48429 Metetra
.769 .718 .805 r
.43803 .48429 .44136 .48885 .44723 .48412 .44391 .47951 Metetra
.772 .717 .801 r
.44391 .47951 .44723 .48412 .45311 .47925 .4498 .4746 Metetra
.775 .717 .798 r
.4498 .4746 .45311 .47925 .45899 .47423 .45569 .46954 Metetra
.778 .716 .795 r
.45569 .46954 .45899 .47423 .46488 .46904 .46159 .46434 Metetra
.782 .717 .791 r
.46159 .46434 .46488 .46904 .47077 .46367 .46749 .45896 Metetra
.786 .718 .788 r
.46749 .45896 .47077 .46367 .47666 .45808 .47339 .4534 Metetra
.791 .721 .786 r
.47339 .4534 .47666 .45808 .48256 .45227 .47929 .44766 Metetra
.797 .725 .784 r
.47929 .44766 .48256 .45227 .48845 .44622 .4852 .44172 Metetra
.803 .73 .783 r
.4852 .44172 .48845 .44622 .49435 .43994 .4911 .43561 Metetra
.81 .737 .784 r
.4911 .43561 .49435 .43994 .50025 .43345 .49701 .42933 Metetra
.818 .747 .786 r
.49701 .42933 .50025 .43345 .50614 .42678 .50291 .42292 Metetra
.825 .759 .79 r
.50291 .42292 .50614 .42678 .51203 .42001 .50881 .41646 Metetra
.742 .27 0 r
.50881 .41646 .51203 .42001 .51787 .4293 .51465 .42557 Metetra
.725 .256 0 r
.51465 .42557 .51787 .4293 .52377 .43859 .52055 .43472 Metetra
.706 .241 0 r
.52055 .43472 .52377 .43859 .52973 .44781 .5265 .44384 Metetra
.689 .226 0 r
.5265 .44384 .52973 .44781 .53574 .45694 .53251 .45292 Metetra
.677 .214 0 r
.53251 .45292 .53574 .45694 .5418 .46595 .53857 .46195 Metetra
.669 .205 0 r
.53857 .46195 .5418 .46595 .54791 .47488 .54468 .47093 Metetra
.668 .201 0 r
.54468 .47093 .54791 .47488 .55408 .48374 .55084 .47989 Metetra
.671 .202 0 r
.55084 .47989 .55408 .48374 .5603 .49257 .55706 .48884 Metetra
.679 .206 0 r
.55706 .48884 .5603 .49257 .56657 .50142 .56334 .49782 Metetra
.69 .213 0 r
.56334 .49782 .56657 .50142 .5729 .51032 .56966 .50686 Metetra
.703 .222 0 r
.56966 .50686 .5729 .51032 .57928 .51932 .57605 .516 Metetra
.716 .233 0 r
.57605 .516 .57928 .51932 .58572 .52847 .58249 .52527 Metetra
.73 .245 0 r
.58249 .52527 .58572 .52847 .59223 .53781 .58899 .53471 Metetra
.741 .257 0 r
.58899 .53471 .59223 .53781 .59879 .54738 .59556 .54435 Metetra
.751 .267 0 r
.59556 .54435 .59879 .54738 .60542 .55722 .60219 .55422 Metetra
.758 .275 0 r
.60219 .55422 .60542 .55722 .61211 .56736 .60888 .56432 Metetra
.761 .28 0 r
.60888 .56432 .61211 .56736 .61887 .5778 .61564 .57466 Metetra
.759 .281 0 r
.61564 .57466 .61887 .5778 .62571 .5885 .62247 .5852 Metetra
.751 .276 0 r
.62247 .5852 .62571 .5885 .63261 .5994 .62936 .59592 Metetra
.848 .802 .806 r
.62936 .59592 .63261 .5994 .63896 .59196 .63575 .58881 Metetra
.854 .82 .814 r
.63575 .58881 .63896 .59196 .64532 .58463 .64212 .58179 Metetra
.858 .836 .823 r
.64212 .58179 .64532 .58463 .65167 .57751 .64851 .57491 Metetra
.859 .851 .833 r
.64851 .57491 .65167 .57751 .65803 .57065 .65489 .56823 Metetra
.858 .862 .843 r
.65489 .56823 .65803 .57065 .6644 .56406 .66128 .56176 Metetra
.855 .871 .851 r
.66128 .56176 .6644 .56406 .67078 .55773 .66767 .5555 Metetra
.85 .877 .859 r
.66767 .5555 .67078 .55773 .67716 .55166 .67408 .54945 Metetra
.845 .88 .866 r
.67408 .54945 .67716 .55166 .68356 .54582 .68049 .54361 Metetra
.839 .881 .872 r
.68049 .54361 .68356 .54582 .68996 .5402 .68691 .53796 Metetra
.833 .881 .877 r
.68691 .53796 .68996 .5402 .69638 .53475 .69334 .53247 Metetra
.827 .879 .882 r
.69334 .53247 .69638 .53475 .70281 .52948 .69978 .52714 Metetra
.821 .876 .885 r
.69978 .52714 .70281 .52948 .70924 .52434 .70624 .52195 Metetra
.816 .874 .888 r
.70624 .52195 .70924 .52434 .71569 .51935 .7127 .51689 Metetra
.811 .87 .891 r
.7127 .51689 .71569 .51935 .72215 .51447 .71917 .51194 Metetra
.806 .867 .893 r
.71917 .51194 .72215 .51447 .72862 .50969 .72565 .50709 Metetra
.802 .862 .894 r
.72565 .50709 .72862 .50969 .7351 .505 .73214 .50234 Metetra
.797 .859 .895 r
.73214 .50234 .7351 .505 .74159 .50041 .73864 .49767 Metetra
.793 .854 .896 r
.73864 .49767 .74159 .50041 .74809 .49589 .74515 .49308 Metetra
.79 .85 .897 r
.74515 .49308 .74809 .49589 .7546 .49143 .75167 .48857 Metetra
.786 .846 .897 r
.75167 .48857 .7546 .49143 .76112 .48704 .7582 .48411 Metetra
.783 .843 .897 r
.7582 .48411 .76112 .48704 .76765 .4827 .76474 .4797 Metetra
.78 .839 .897 r
.76474 .4797 .76765 .4827 .77419 .47841 .77129 .47536 Metetra
.777 .835 .897 r
.77129 .47536 .77419 .47841 .78074 .47418 .77785 .47106 Metetra
.774 .832 .897 r
.77785 .47106 .78074 .47418 .7873 .46998 .78443 .46681 Metetra
.772 .829 .896 r
.78443 .46681 .7873 .46998 .79387 .46583 .79101 .46259 Metetra
.77 .825 .896 r
.79101 .46259 .79387 .46583 .80045 .4617 .7976 .45841 Metetra
.768 .822 .895 r
.7976 .45841 .80045 .4617 .80705 .4576 .8042 .45426 Metetra
.765 .819 .895 r
.8042 .45426 .80705 .4576 .81365 .45353 .81081 .45014 Metetra
.764 .816 .894 r
.81081 .45014 .81365 .45353 .82026 .44949 .81744 .44605 Metetra
.762 .813 .893 r
.81744 .44605 .82026 .44949 .82688 .44547 .82407 .44197 Metetra
.76 .81 .893 r
.82407 .44197 .82688 .44547 .83351 .44147 .83071 .43793 Metetra
.759 .808 .892 r
.83071 .43793 .83351 .44147 .84016 .43749 .83736 .4339 Metetra
.758 .805 .891 r
.83736 .4339 .84016 .43749 .84681 .43351 .84402 .42988 Metetra
.757 .803 .89 r
.84402 .42988 .84681 .43351 .85347 .42955 .85069 .42587 Metetra
.756 .801 .889 r
.85069 .42587 .85347 .42955 .86014 .42559 .85737 .42186 Metetra
.755 .798 .888 r
.85737 .42186 .86014 .42559 .86682 .42164 .86406 .41787 Metetra
.754 .796 .887 r
.86406 .41787 .86682 .42164 .8735 .41769 .87076 .41387 Metetra
.753 .794 .886 r
.87076 .41387 .8735 .41769 .8802 .41375 .87746 .40988 Metetra
.752 .792 .885 r
.87746 .40988 .8802 .41375 .88691 .40981 .88418 .40589 Metetra
.752 .79 .884 r
.88418 .40589 .88691 .40981 .89362 .40586 .8909 .4019 Metetra
.751 .767 .866 r
.27244 .58904 .27608 .59223 .28177 .58875 .27814 .58553 Metetra
.751 .766 .865 r
.27814 .58553 .28177 .58875 .28747 .58527 .28385 .58201 Metetra
.751 .764 .864 r
.28385 .58201 .28747 .58527 .29317 .58178 .28956 .57849 Metetra
.751 .763 .862 r
.28956 .57849 .29317 .58178 .29888 .57829 .29528 .57495 Metetra
.751 .761 .861 r
.29528 .57495 .29888 .57829 .3046 .57477 .30101 .5714 Metetra
.751 .759 .86 r
.30101 .5714 .3046 .57477 .31032 .57125 .30674 .56783 Metetra
.75 .758 .858 r
.30674 .56783 .31032 .57125 .31605 .56771 .31248 .56425 Metetra
.751 .756 .857 r
.31248 .56425 .31605 .56771 .32179 .56414 .31823 .56065 Metetra
.751 .755 .856 r
.31823 .56065 .32179 .56414 .32753 .56057 .32399 .55703 Metetra
.751 .753 .854 r
.32399 .55703 .32753 .56057 .33329 .55697 .32975 .55339 Metetra
.751 .752 .852 r
.32975 .55339 .33329 .55697 .33904 .55334 .33552 .54972 Metetra
.752 .75 .85 r
.33552 .54972 .33904 .55334 .34481 .54968 .34129 .54602 Metetra
.752 .748 .848 r
.34129 .54602 .34481 .54968 .35058 .54598 .34708 .54228 Metetra
.753 .747 .847 r
.34708 .54228 .35058 .54598 .35636 .54226 .35287 .5385 Metetra
.753 .745 .844 r
.35287 .5385 .35636 .54226 .36215 .53849 .35867 .53469 Metetra
.754 .743 .843 r
.35867 .53469 .36215 .53849 .36795 .53468 .36448 .53084 Metetra
.755 .742 .84 r
.36448 .53084 .36795 .53468 .37375 .53084 .37029 .52694 Metetra
.756 .74 .838 r
.37029 .52694 .37375 .53084 .37956 .52694 .37611 .523 Metetra
.757 .738 .836 r
.37611 .523 .37956 .52694 .38538 .52299 .38194 .519 Metetra
.758 .737 .833 r
.38194 .519 .38538 .52299 .3912 .51899 .38777 .51494 Metetra
.759 .735 .831 r
.38777 .51494 .3912 .51899 .39703 .51492 .39361 .51082 Metetra
.76 .733 .828 r
.39361 .51082 .39703 .51492 .40287 .51079 .39946 .50664 Metetra
.761 .731 .825 r
.39946 .50664 .40287 .51079 .40871 .5066 .40532 .50239 Metetra
.763 .73 .822 r
.40532 .50239 .40871 .5066 .41456 .50232 .41118 .49806 Metetra
.764 .728 .819 r
.41118 .49806 .41456 .50232 .42042 .49795 .41705 .49364 Metetra
.766 .727 .816 r
.41705 .49364 .42042 .49795 .42629 .4935 .42292 .48914 Metetra
.768 .725 .813 r
.42292 .48914 .42629 .4935 .43216 .48895 .4288 .48454 Metetra
.77 .724 .81 r
.4288 .48454 .43216 .48895 .43803 .48429 .43469 .47983 Metetra
.772 .723 .807 r
.43469 .47983 .43803 .48429 .44391 .47951 .44058 .47502 Metetra
.775 .723 .804 r
.44058 .47502 .44391 .47951 .4498 .4746 .44648 .47007 Metetra
.778 .723 .801 r
.44648 .47007 .4498 .4746 .45569 .46954 .45238 .465 Metetra
.781 .723 .798 r
.45238 .465 .45569 .46954 .46159 .46434 .45828 .45978 Metetra
.785 .724 .795 r
.45828 .45978 .46159 .46434 .46749 .45896 .46419 .45442 Metetra
.789 .725 .793 r
.46419 .45442 .46749 .45896 .47339 .4534 .4701 .4489 Metetra
.793 .728 .791 r
.4701 .4489 .47339 .4534 .47929 .44766 .47601 .44322 Metetra
.798 .732 .79 r
.47601 .44322 .47929 .44766 .4852 .44172 .48193 .43739 Metetra
.804 .737 .79 r
.48193 .43739 .4852 .44172 .4911 .43561 .48784 .43141 Metetra
.81 .744 .79 r
.48784 .43141 .4911 .43561 .49701 .42933 .49376 .42532 Metetra
.816 .752 .792 r
.49376 .42532 .49701 .42933 .50291 .42292 .49967 .41912 Metetra
.822 .762 .796 r
.49967 .41912 .50291 .42292 .50881 .41646 .50558 .41289 Metetra
.735 .262 0 r
.50558 .41289 .50881 .41646 .51465 .42557 .51142 .42189 Metetra
.723 .252 0 r
.51142 .42189 .51465 .42557 .52055 .43472 .51732 .43093 Metetra
.711 .242 0 r
.51732 .43093 .52055 .43472 .5265 .44384 .52327 .43998 Metetra
.699 .231 0 r
.52327 .43998 .5265 .44384 .53251 .45292 .52927 .44903 Metetra
.69 .223 0 r
.52927 .44903 .53251 .45292 .53857 .46195 .53532 .45806 Metetra
.684 .216 0 r
.53532 .45806 .53857 .46195 .54468 .47093 .54143 .46708 Metetra
.682 .213 0 r
.54143 .46708 .54468 .47093 .55084 .47989 .5476 .4761 Metetra
.684 .213 0 r
.5476 .4761 .55084 .47989 .55706 .48884 .55381 .48514 Metetra
.69 .216 0 r
.55381 .48514 .55706 .48884 .56334 .49782 .56009 .49422 Metetra
.698 .221 0 r
.56009 .49422 .56334 .49782 .56966 .50686 .56641 .50337 Metetra
.707 .228 0 r
.56641 .50337 .56966 .50686 .57605 .516 .5728 .51261 Metetra
.717 .236 0 r
.5728 .51261 .57605 .516 .58249 .52527 .57924 .52198 Metetra
.727 .245 0 r
.57924 .52198 .58249 .52527 .58899 .53471 .58575 .53149 Metetra
.736 .253 0 r
.58575 .53149 .58899 .53471 .59556 .54435 .59231 .54118 Metetra
.743 .261 0 r
.59231 .54118 .59556 .54435 .60219 .55422 .59894 .55106 Metetra
.748 .267 0 r
.59894 .55106 .60219 .55422 .60888 .56432 .60563 .56113 Metetra
.749 .27 0 r
.60563 .56113 .60888 .56432 .61564 .57466 .61239 .57139 Metetra
.747 .27 0 r
.61239 .57139 .61564 .57466 .62247 .5852 .61921 .58183 Metetra
.742 .266 0 r
.61921 .58183 .62247 .5852 .62936 .59592 .6261 .59241 Metetra
.843 .803 .812 r
.6261 .59241 .62936 .59592 .63575 .58881 .63251 .58557 Metetra
.847 .816 .819 r
.63251 .58557 .63575 .58881 .64212 .58179 .63891 .57879 Metetra
.85 .829 .826 r
.63891 .57879 .64212 .58179 .64851 .57491 .64532 .57211 Metetra
.851 .841 .834 r
.64532 .57211 .64851 .57491 .65489 .56823 .65172 .56558 Metetra
.851 .851 .842 r
.65172 .56558 .65489 .56823 .66128 .56176 .65813 .55922 Metetra
.849 .859 .849 r
.65813 .55922 .66128 .56176 .66767 .5555 .66455 .55304 Metetra
.845 .864 .856 r
.66455 .55304 .66767 .5555 .67408 .54945 .67097 .54703 Metetra
.841 .868 .863 r
.67097 .54703 .67408 .54945 .68049 .54361 .6774 .5412 Metetra
.836 .87 .868 r
.6774 .5412 .68049 .54361 .68691 .53796 .68384 .53553 Metetra
.831 .87 .873 r
.68384 .53553 .68691 .53796 .69334 .53247 .69028 .53002 Metetra
.826 .869 .877 r
.69028 .53002 .69334 .53247 .69978 .52714 .69674 .52465 Metetra
.821 .868 .881 r
.69674 .52465 .69978 .52714 .70624 .52195 .7032 .51941 Metetra
.816 .865 .884 r
.7032 .51941 .70624 .52195 .7127 .51689 .70968 .5143 Metetra
.811 .863 .886 r
.70968 .5143 .7127 .51689 .71917 .51194 .71616 .50929 Metetra
.807 .86 .888 r
.71616 .50929 .71917 .51194 .72565 .50709 .72265 .50439 Metetra
.803 .857 .89 r
.72265 .50439 .72565 .50709 .73214 .50234 .72916 .49957 Metetra
.798 .853 .891 r
.72916 .49957 .73214 .50234 .73864 .49767 .73567 .49484 Metetra
.794 .849 .892 r
.73567 .49484 .73864 .49767 .74515 .49308 .74219 .49019 Metetra
.791 .846 .893 r
.74219 .49019 .74515 .49308 .75167 .48857 .74872 .48562 Metetra
.788 .843 .893 r
.74872 .48562 .75167 .48857 .7582 .48411 .75527 .4811 Metetra
.784 .839 .894 r
.75527 .4811 .7582 .48411 .76474 .4797 .76182 .47664 Metetra
.781 .836 .894 r
.76182 .47664 .76474 .4797 .77129 .47536 .76838 .47224 Metetra
.778 .832 .894 r
.76838 .47224 .77129 .47536 .77785 .47106 .77495 .46788 Metetra
.775 .829 .894 r
.77495 .46788 .77785 .47106 .78443 .46681 .78154 .46358 Metetra
.773 .826 .894 r
.78154 .46358 .78443 .46681 .79101 .46259 .78813 .45931 Metetra
.771 .823 .893 r
.78813 .45931 .79101 .46259 .7976 .45841 .79473 .45508 Metetra
.769 .82 .893 r
.79473 .45508 .7976 .45841 .8042 .45426 .80134 .45087 Metetra
.767 .818 .892 r
.80134 .45087 .8042 .45426 .81081 .45014 .80796 .4467 Metetra
.765 .814 .892 r
.80796 .4467 .81081 .45014 .81744 .44605 .8146 .44256 Metetra
.763 .812 .891 r
.8146 .44256 .81744 .44605 .82407 .44197 .82124 .43844 Metetra
.762 .81 .891 r
.82124 .43844 .82407 .44197 .83071 .43793 .82789 .43435 Metetra
.76 .807 .89 r
.82789 .43435 .83071 .43793 .83736 .4339 .83455 .43027 Metetra
.759 .805 .89 r
.83455 .43027 .83736 .4339 .84402 .42988 .84122 .42621 Metetra
.758 .802 .889 r
.84122 .42621 .84402 .42988 .85069 .42587 .8479 .42215 Metetra
.757 .8 .888 r
.8479 .42215 .85069 .42587 .85737 .42186 .85459 .41811 Metetra
.756 .798 .887 r
.85459 .41811 .85737 .42186 .86406 .41787 .86129 .41406 Metetra
.755 .796 .886 r
.86129 .41406 .86406 .41787 .87076 .41387 .868 .41003 Metetra
.754 .794 .885 r
.868 .41003 .87076 .41387 .87746 .40988 .87472 .406 Metetra
.754 .792 .884 r
.87472 .406 .87746 .40988 .88418 .40589 .88144 .40196 Metetra
.753 .79 .883 r
.88144 .40196 .88418 .40589 .8909 .4019 .88817 .39793 Metetra
.753 .768 .865 r
.26877 .58585 .27244 .58904 .27814 .58553 .27449 .5823 Metetra
.752 .766 .864 r
.27449 .5823 .27814 .58553 .28385 .58201 .28021 .57874 Metetra
.752 .765 .863 r
.28021 .57874 .28385 .58201 .28956 .57849 .28593 .57518 Metetra
.752 .763 .862 r
.28593 .57518 .28956 .57849 .29528 .57495 .29166 .57161 Metetra
.752 .762 .861 r
.29166 .57161 .29528 .57495 .30101 .5714 .2974 .56802 Metetra
.752 .76 .859 r
.2974 .56802 .30101 .5714 .30674 .56783 .30315 .56442 Metetra
.752 .759 .858 r
.30315 .56442 .30674 .56783 .31248 .56425 .3089 .5608 Metetra
.752 .757 .857 r
.3089 .5608 .31248 .56425 .31823 .56065 .31466 .55716 Metetra
.752 .756 .855 r
.31466 .55716 .31823 .56065 .32399 .55703 .32042 .55351 Metetra
.752 .754 .854 r
.32042 .55351 .32399 .55703 .32975 .55339 .3262 .54982 Metetra
.753 .753 .852 r
.3262 .54982 .32975 .55339 .33552 .54972 .33198 .54611 Metetra
.753 .751 .851 r
.33198 .54611 .33552 .54972 .34129 .54602 .33777 .54237 Metetra
.754 .75 .849 r
.33777 .54237 .34129 .54602 .34708 .54228 .34356 .53858 Metetra
.755 .748 .847 r
.34356 .53858 .34708 .54228 .35287 .5385 .34936 .53477 Metetra
.755 .747 .845 r
.34936 .53477 .35287 .5385 .35867 .53469 .35517 .53091 Metetra
.756 .745 .843 r
.35517 .53091 .35867 .53469 .36448 .53084 .36099 .52701 Metetra
.757 .744 .841 r
.36099 .52701 .36448 .53084 .37029 .52694 .36681 .52308 Metetra
.758 .742 .839 r
.36681 .52308 .37029 .52694 .37611 .523 .37264 .51908 Metetra
.759 .741 .836 r
.37264 .51908 .37611 .523 .38194 .519 .37848 .51504 Metetra
.76 .739 .834 r
.37848 .51504 .38194 .519 .38777 .51494 .38433 .51094 Metetra
.761 .738 .831 r
.38433 .51094 .38777 .51494 .39361 .51082 .39018 .50677 Metetra
.762 .736 .829 r
.39018 .50677 .39361 .51082 .39946 .50664 .39604 .50254 Metetra
.764 .735 .826 r
.39604 .50254 .39946 .50664 .40532 .50239 .40191 .49825 Metetra
.765 .734 .824 r
.40191 .49825 .40532 .50239 .41118 .49806 .40778 .49387 Metetra
.767 .732 .821 r
.40778 .49387 .41118 .49806 .41705 .49364 .41366 .4894 Metetra
.769 .731 .818 r
.41366 .4894 .41705 .49364 .42292 .48914 .41954 .48486 Metetra
.771 .73 .815 r
.41954 .48486 .42292 .48914 .4288 .48454 .42543 .48022 Metetra
.773 .729 .812 r
.42543 .48022 .4288 .48454 .43469 .47983 .43133 .47548 Metetra
.775 .728 .81 r
.43133 .47548 .43469 .47983 .44058 .47502 .43723 .47062 Metetra
.778 .728 .807 r
.43723 .47062 .44058 .47502 .44648 .47007 .44314 .46566 Metetra
.781 .728 .804 r
.44314 .46566 .44648 .47007 .45238 .465 .44905 .46057 Metetra
.784 .729 .802 r
.44905 .46057 .45238 .465 .45828 .45978 .45496 .45536 Metetra
.787 .73 .799 r
.45496 .45536 .45828 .45978 .46419 .45442 .46088 .45002 Metetra
.791 .731 .797 r
.46088 .45002 .46419 .45442 .4701 .4489 .4668 .44454 Metetra
.795 .734 .796 r
.4668 .44454 .4701 .4489 .47601 .44322 .47272 .43892 Metetra
.799 .738 .795 r
.47272 .43892 .47601 .44322 .48193 .43739 .47865 .43319 Metetra
.804 .743 .795 r
.47865 .43319 .48193 .43739 .48784 .43141 .48457 .42733 Metetra
.809 .749 .796 r
.48457 .42733 .48784 .43141 .49376 .42532 .49049 .42138 Metetra
.814 .756 .798 r
.49049 .42138 .49376 .42532 .49967 .41912 .49642 .41536 Metetra
.819 .764 .801 r
.49642 .41536 .49967 .41912 .50558 .41289 .50234 .40931 Metetra
.73 .256 0 r
.50234 .40931 .50558 .41289 .51142 .42189 .50818 .41822 Metetra
.722 .25 0 r
.50818 .41822 .51142 .42189 .51732 .43093 .51407 .42719 Metetra
.713 .242 0 r
.51407 .42719 .51732 .43093 .52327 .43998 .52002 .43619 Metetra
.705 .235 0 r
.52002 .43619 .52327 .43998 .52927 .44903 .52602 .44521 Metetra
.698 .229 0 r
.52602 .44521 .52927 .44903 .53532 .45806 .53207 .45424 Metetra
.694 .224 0 r
.53207 .45424 .53532 .45806 .54143 .46708 .53818 .46329 Metetra
.693 .221 0 r
.53818 .46329 .54143 .46708 .5476 .4761 .54434 .47236 Metetra
.694 .221 0 r
.54434 .47236 .5476 .4761 .55381 .48514 .55055 .48146 Metetra
.698 .223 0 r
.55055 .48146 .55381 .48514 .56009 .49422 .55682 .49062 Metetra
.703 .227 0 r
.55682 .49062 .56009 .49422 .56641 .50337 .56315 .49985 Metetra
.71 .232 0 r
.56315 .49985 .56641 .50337 .5728 .51261 .56954 .50917 Metetra
.718 .238 0 r
.56954 .50917 .5728 .51261 .57924 .52198 .57598 .5186 Metetra
.725 .244 0 r
.57598 .5186 .57924 .52198 .58575 .53149 .58248 .52817 Metetra
.731 .25 0 r
.58248 .52817 .58575 .53149 .59231 .54118 .58905 .53789 Metetra
.737 .256 0 r
.58905 .53789 .59231 .54118 .59894 .55106 .59567 .54778 Metetra
.74 .26 0 r
.59567 .54778 .59894 .55106 .60563 .56113 .60236 .55783 Metetra
.741 .262 0 r
.60236 .55783 .60563 .56113 .61239 .57139 .60912 .56804 Metetra
.739 .262 0 r
.60912 .56804 .61239 .57139 .61921 .58183 .61594 .5784 Metetra
.735 .259 0 r
.61594 .5784 .61921 .58183 .6261 .59241 .62283 .5889 Metetra
.838 .803 .817 r
.62283 .5889 .6261 .59241 .63251 .58557 .62926 .58226 Metetra
.842 .813 .822 r
.62926 .58226 .63251 .58557 .63891 .57879 .63568 .57566 Metetra
.844 .824 .829 r
.63568 .57566 .63891 .57879 .64532 .57211 .64211 .56916 Metetra
.845 .834 .835 r
.64211 .56916 .64532 .57211 .65172 .56558 .64853 .56276 Metetra
.845 .842 .842 r
.64853 .56276 .65172 .56558 .65813 .55922 .65496 .55651 Metetra
.843 .849 .848 r
.65496 .55651 .65813 .55922 .66455 .55304 .6614 .55039 Metetra
.841 .854 .854 r
.6614 .55039 .66455 .55304 .67097 .54703 .66784 .54442 Metetra
.837 .857 .86 r
.66784 .54442 .67097 .54703 .6774 .5412 .67429 .53861 Metetra
.833 .86 .865 r
.67429 .53861 .6774 .5412 .68384 .53553 .68074 .53294 Metetra
.829 .86 .869 r
.68074 .53294 .68384 .53553 .69028 .53002 .6872 .52741 Metetra
.825 .86 .873 r
.6872 .52741 .69028 .53002 .69674 .52465 .69367 .52201 Metetra
.82 .859 .877 r
.69367 .52201 .69674 .52465 .7032 .51941 .70015 .51674 Metetra
.815 .858 .88 r
.70015 .51674 .7032 .51941 .70968 .5143 .70664 .51158 Metetra
.811 .856 .882 r
.70664 .51158 .70968 .5143 .71616 .50929 .71314 .50653 Metetra
.807 .853 .884 r
.71314 .50653 .71616 .50929 .72265 .50439 .71964 .50157 Metetra
.803 .85 .886 r
.71964 .50157 .72265 .50439 .72916 .49957 .72616 .4967 Metetra
.799 .848 .887 r
.72616 .4967 .72916 .49957 .73567 .49484 .73268 .49192 Metetra
.795 .844 .889 r
.73268 .49192 .73567 .49484 .74219 .49019 .73921 .48722 Metetra
.791 .841 .889 r
.73921 .48722 .74219 .49019 .74872 .48562 .74576 .48259 Metetra
.788 .838 .89 r
.74576 .48259 .74872 .48562 .75527 .4811 .75231 .47802 Metetra
.785 .836 .891 r
.75231 .47802 .75527 .4811 .76182 .47664 .75888 .4735 Metetra
.782 .832 .891 r
.75888 .4735 .76182 .47664 .76838 .47224 .76545 .46905 Metetra
.779 .829 .891 r
.76545 .46905 .76838 .47224 .77495 .46788 .77203 .46465 Metetra
.777 .827 .891 r
.77203 .46465 .77495 .46788 .78154 .46358 .77863 .46029 Metetra
.774 .824 .891 r
.77863 .46029 .78154 .46358 .78813 .45931 .78523 .45597 Metetra
.772 .821 .891 r
.78523 .45597 .78813 .45931 .79473 .45508 .79184 .45169 Metetra
.77 .818 .891 r
.79184 .45169 .79473 .45508 .80134 .45087 .79847 .44743 Metetra
.768 .815 .89 r
.79847 .44743 .80134 .45087 .80796 .4467 .8051 .44322 Metetra
.766 .813 .89 r
.8051 .44322 .80796 .4467 .8146 .44256 .81174 .43903 Metetra
.764 .811 .89 r
.81174 .43903 .8146 .44256 .82124 .43844 .81839 .43487 Metetra
.762 .808 .889 r
.81839 .43487 .82124 .43844 .82789 .43435 .82506 .43073 Metetra
.761 .806 .889 r
.82506 .43073 .82789 .43435 .83455 .43027 .83173 .42661 Metetra
.76 .803 .888 r
.83173 .42661 .83455 .43027 .84122 .42621 .83841 .4225 Metetra
.759 .802 .887 r
.83841 .4225 .84122 .42621 .8479 .42215 .8451 .41841 Metetra
.758 .8 .887 r
.8451 .41841 .8479 .42215 .85459 .41811 .8518 .41432 Metetra
.757 .798 .886 r
.8518 .41432 .85459 .41811 .86129 .41406 .85851 .41024 Metetra
.756 .796 .885 r
.85851 .41024 .86129 .41406 .868 .41003 .86523 .40616 Metetra
.755 .794 .884 r
.86523 .40616 .868 .41003 .87472 .406 .87195 .40209 Metetra
.755 .792 .883 r
.87195 .40209 .87472 .406 .88144 .40196 .87869 .39801 Metetra
.754 .79 .882 r
.87869 .39801 .88144 .40196 .88817 .39793 .88543 .39394 Metetra
.753 .768 .864 r
.2651 .58264 .26877 .58585 .27449 .5823 .27082 .57906 Metetra
.753 .766 .863 r
.27082 .57906 .27449 .5823 .28021 .57874 .27655 .57547 Metetra
.753 .765 .863 r
.27655 .57547 .28021 .57874 .28593 .57518 .28229 .57187 Metetra
.753 .764 .861 r
.28229 .57187 .28593 .57518 .29166 .57161 .28803 .56826 Metetra
.753 .762 .86 r
.28803 .56826 .29166 .57161 .2974 .56802 .29378 .56464 Metetra
.753 .761 .859 r
.29378 .56464 .2974 .56802 .30315 .56442 .29954 .561 Metetra
.753 .759 .858 r
.29954 .561 .30315 .56442 .3089 .5608 .3053 .55735 Metetra
.753 .758 .857 r
.3053 .55735 .3089 .5608 .31466 .55716 .31107 .55368 Metetra
.753 .757 .855 r
.31107 .55368 .31466 .55716 .32042 .55351 .31685 .54998 Metetra
.754 .755 .854 r
.31685 .54998 .32042 .55351 .3262 .54982 .32263 .54627 Metetra
.754 .754 .852 r
.32263 .54627 .3262 .54982 .33198 .54611 .32842 .54251 Metetra
.755 .753 .85 r
.32842 .54251 .33198 .54611 .33777 .54237 .33422 .53873 Metetra
.755 .751 .849 r
.33422 .53873 .33777 .54237 .34356 .53858 .34003 .53491 Metetra
.756 .75 .847 r
.34003 .53491 .34356 .53858 .34936 .53477 .34584 .53105 Metetra
.757 .749 .845 r
.34584 .53105 .34936 .53477 .35517 .53091 .35166 .52716 Metetra
.758 .747 .843 r
.35166 .52716 .35517 .53091 .36099 .52701 .35749 .52322 Metetra
.759 .746 .841 r
.35749 .52322 .36099 .52701 .36681 .52308 .36332 .51924 Metetra
.759 .744 .839 r
.36332 .51924 .36681 .52308 .37264 .51908 .36916 .51521 Metetra
.761 .743 .837 r
.36916 .51521 .37264 .51908 .37848 .51504 .37501 .51112 Metetra
.762 .742 .835 r
.37501 .51112 .37848 .51504 .38433 .51094 .38087 .50698 Metetra
.763 .741 .832 r
.38087 .50698 .38433 .51094 .39018 .50677 .38673 .50277 Metetra
.764 .739 .83 r
.38673 .50277 .39018 .50677 .39604 .50254 .3926 .4985 Metetra
.766 .738 .827 r
.3926 .4985 .39604 .50254 .40191 .49825 .39848 .49416 Metetra
.767 .737 .825 r
.39848 .49416 .40191 .49825 .40778 .49387 .40436 .48974 Metetra
.769 .736 .822 r
.40436 .48974 .40778 .49387 .41366 .4894 .41025 .48524 Metetra
.771 .735 .82 r
.41025 .48524 .41366 .4894 .41954 .48486 .41615 .48065 Metetra
.773 .734 .817 r
.41615 .48065 .41954 .48486 .42543 .48022 .42205 .47597 Metetra
.775 .733 .814 r
.42205 .47597 .42543 .48022 .43133 .47548 .42795 .47121 Metetra
.777 .733 .812 r
.42795 .47121 .43133 .47548 .43723 .47062 .43387 .46634 Metetra
.78 .733 .809 r
.43387 .46634 .43723 .47062 .44314 .46566 .43978 .46135 Metetra
.783 .733 .807 r
.43978 .46135 .44314 .46566 .44905 .46057 .4457 .45626 Metetra
.786 .734 .805 r
.4457 .45626 .44905 .46057 .45496 .45536 .45163 .45106 Metetra
.789 .735 .803 r
.45163 .45106 .45496 .45536 .46088 .45002 .45756 .44574 Metetra
.793 .737 .801 r
.45756 .44574 .46088 .45002 .4668 .44454 .46349 .44029 Metetra
.796 .74 .8 r
.46349 .44029 .4668 .44454 .47272 .43892 .46942 .43474 Metetra
.8 .743 .8 r
.46942 .43474 .47272 .43892 .47865 .43319 .47535 .42909 Metetra
.804 .747 .8 r
.47535 .42909 .47865 .43319 .48457 .42733 .48128 .42334 Metetra
.808 .752 .801 r
.48128 .42334 .48457 .42733 .49049 .42138 .48722 .41751 Metetra
.812 .759 .802 r
.48722 .41751 .49049 .42138 .49642 .41536 .49315 .41162 Metetra
.816 .765 .805 r
.49315 .41162 .49642 .41536 .50234 .40931 .49908 .40571 Metetra
.727 .252 0 r
.49908 .40571 .50234 .40931 .50818 .41822 .50492 .41457 Metetra
.721 .248 0 r
.50492 .41457 .50818 .41822 .51407 .42719 .51081 .42348 Metetra
.715 .242 0 r
.51081 .42348 .51407 .42719 .52002 .43619 .51675 .43245 Metetra
.709 .237 0 r
.51675 .43245 .52002 .43619 .52602 .44521 .52275 .44145 Metetra
.704 .233 0 r
.52275 .44145 .52602 .44521 .53207 .45424 .5288 .45048 Metetra
.701 .229 0 r
.5288 .45048 .53207 .45424 .53818 .46329 .5349 .45955 Metetra
.7 .227 0 r
.5349 .45955 .53818 .46329 .54434 .47236 .54106 .46865 Metetra
.701 .227 0 r
.54106 .46865 .54434 .47236 .55055 .48146 .54728 .4778 Metetra
.704 .228 0 r
.54728 .4778 .55055 .48146 .55682 .49062 .55355 .48701 Metetra
.708 .231 0 r
.55355 .48701 .55682 .49062 .56315 .49985 .55987 .4963 Metetra
.712 .235 0 r
.55987 .4963 .56315 .49985 .56954 .50917 .56626 .50568 Metetra
.718 .239 0 r
.56626 .50568 .56954 .50917 .57598 .5186 .5727 .51517 Metetra
.723 .244 0 r
.5727 .51517 .57598 .5186 .58248 .52817 .5792 .52478 Metetra
.728 .248 0 r
.5792 .52478 .58248 .52817 .58905 .53789 .58577 .53452 Metetra
.732 .252 0 r
.58577 .53452 .58905 .53789 .59567 .54778 .59239 .54441 Metetra
.734 .255 0 r
.59239 .54441 .59567 .54778 .60236 .55783 .59908 .55445 Metetra
.734 .256 0 r
.59908 .55445 .60236 .55783 .60912 .56804 .60584 .56462 Metetra
.733 .256 0 r
.60584 .56462 .60912 .56804 .61594 .5784 .61266 .57493 Metetra
.73 .254 0 r
.61266 .57493 .61594 .5784 .62283 .5889 .61954 .58536 Metetra
.834 .802 .821 r
.61954 .58536 .62283 .5889 .62926 .58226 .62599 .57889 Metetra
.837 .811 .825 r
.62599 .57889 .62926 .58226 .63568 .57566 .63243 .57245 Metetra
.839 .82 .831 r
.63243 .57245 .63568 .57566 .64211 .56916 .63888 .56608 Metetra
.84 .828 .836 r
.63888 .56608 .64211 .56916 .64853 .56276 .64533 .5598 Metetra
.839 .835 .842 r
.64533 .5598 .64853 .56276 .65496 .55651 .65178 .55363 Metetra
.838 .841 .847 r
.65178 .55363 .65496 .55651 .6614 .55039 .65823 .54757 Metetra
.836 .845 .852 r
.65823 .54757 .6614 .55039 .66784 .54442 .66469 .54165 Metetra
.833 .849 .858 r
.66469 .54165 .66784 .54442 .67429 .53861 .67115 .53585 Metetra
.83 .851 .862 r
.67115 .53585 .67429 .53861 .68074 .53294 .67762 .53019 Metetra
.826 .852 .866 r
.67762 .53019 .68074 .53294 .6872 .52741 .6841 .52465 Metetra
.822 .852 .87 r
.6841 .52465 .6872 .52741 .69367 .52201 .69058 .51923 Metetra
.819 .852 .873 r
.69058 .51923 .69367 .52201 .70015 .51674 .69708 .51393 Metetra
.814 .85 .876 r
.69708 .51393 .70015 .51674 .70664 .51158 .70358 .50874 Metetra
.81 .849 .878 r
.70358 .50874 .70664 .51158 .71314 .50653 .71009 .50364 Metetra
.806 .847 .881 r
.71009 .50364 .71314 .50653 .71964 .50157 .71661 .49865 Metetra
.803 .845 .882 r
.71661 .49865 .71964 .50157 .72616 .4967 .72314 .49373 Metetra
.799 .842 .884 r
.72314 .49373 .72616 .4967 .73268 .49192 .72967 .4889 Metetra
.795 .84 .885 r
.72967 .4889 .73268 .49192 .73921 .48722 .73622 .48415 Metetra
.792 .837 .886 r
.73622 .48415 .73921 .48722 .74576 .48259 .74277 .47947 Metetra
.789 .834 .887 r
.74277 .47947 .74576 .48259 .75231 .47802 .74934 .47485 Metetra
.786 .831 .887 r
.74934 .47485 .75231 .47802 .75888 .4735 .75592 .47029 Metetra
.783 .829 .888 r
.75592 .47029 .75888 .4735 .76545 .46905 .7625 .46579 Metetra
.78 .826 .888 r
.7625 .46579 .76545 .46905 .77203 .46465 .7691 .46133 Metetra
.777 .823 .889 r
.7691 .46133 .77203 .46465 .77863 .46029 .7757 .45693 Metetra
.775 .821 .889 r
.7757 .45693 .77863 .46029 .78523 .45597 .78231 .45257 Metetra
.773 .819 .889 r
.78231 .45257 .78523 .45597 .79184 .45169 .78894 .44824 Metetra
.771 .816 .888 r
.78894 .44824 .79184 .45169 .79847 .44743 .79557 .44395 Metetra
.769 .814 .888 r
.79557 .44395 .79847 .44743 .8051 .44322 .80222 .43968 Metetra
.767 .811 .888 r
.80222 .43968 .8051 .44322 .81174 .43903 .80887 .43545 Metetra
.765 .809 .888 r
.80887 .43545 .81174 .43903 .81839 .43487 .81553 .43125 Metetra
.764 .807 .888 r
.81553 .43125 .81839 .43487 .82506 .43073 .82221 .42707 Metetra
.762 .805 .887 r
.82221 .42707 .82506 .43073 .83173 .42661 .82889 .42291 Metetra
.761 .803 .887 r
.82889 .42291 .83173 .42661 .83841 .4225 .83558 .41876 Metetra
.76 .801 .886 r
.83558 .41876 .83841 .4225 .8451 .41841 .84228 .41463 Metetra
.759 .799 .885 r
.84228 .41463 .8451 .41841 .8518 .41432 .84899 .4105 Metetra
.758 .797 .885 r
.84899 .4105 .8518 .41432 .85851 .41024 .85571 .40638 Metetra
.757 .795 .884 r
.85571 .40638 .85851 .41024 .86523 .40616 .86244 .40226 Metetra
.757 .794 .883 r
.86244 .40226 .86523 .40616 .87195 .40209 .86918 .39815 Metetra
.756 .792 .882 r
.86918 .39815 .87195 .40209 .87869 .39801 .87592 .39404 Metetra
.755 .79 .881 r
.87592 .39404 .87869 .39801 .88543 .39394 .88268 .38993 Metetra
.754 .768 .864 r
.26141 .57942 .2651 .58264 .27082 .57906 .26714 .5758 Metetra
.754 .766 .863 r
.26714 .5758 .27082 .57906 .27655 .57547 .27288 .57218 Metetra
.754 .765 .862 r
.27288 .57218 .27655 .57547 .28229 .57187 .27863 .56855 Metetra
.754 .764 .861 r
.27863 .56855 .28229 .57187 .28803 .56826 .28438 .56492 Metetra
.754 .763 .86 r
.28438 .56492 .28803 .56826 .29378 .56464 .29015 .56126 Metetra
.754 .761 .859 r
.29015 .56126 .29378 .56464 .29954 .561 .29591 .55759 Metetra
.754 .76 .858 r
.29591 .55759 .29954 .561 .3053 .55735 .30169 .5539 Metetra
.754 .759 .856 r
.30169 .5539 .3053 .55735 .31107 .55368 .30747 .55019 Metetra
.754 .758 .855 r
.30747 .55019 .31107 .55368 .31685 .54998 .31325 .54647 Metetra
.755 .757 .854 r
.31325 .54647 .31685 .54998 .32263 .54627 .31905 .54272 Metetra
.755 .755 .852 r
.31905 .54272 .32263 .54627 .32842 .54251 .32485 .53894 Metetra
.756 .755 .851 r
.32485 .53894 .32842 .54251 .33422 .53873 .33066 .53512 Metetra
.757 .753 .849 r
.33066 .53512 .33422 .53873 .34003 .53491 .33648 .53126 Metetra
.758 .752 .847 r
.33648 .53126 .34003 .53491 .34584 .53105 .3423 .52737 Metetra
.759 .751 .846 r
.3423 .52737 .34584 .53105 .35166 .52716 .34813 .52343 Metetra
.759 .75 .844 r
.34813 .52343 .35166 .52716 .35749 .52322 .35397 .51946 Metetra
.76 .748 .842 r
.35397 .51946 .35749 .52322 .36332 .51924 .35982 .51545 Metetra
.761 .747 .84 r
.35982 .51545 .36332 .51924 .36916 .51521 .36567 .51137 Metetra
.763 .746 .838 r
.36567 .51137 .36916 .51521 .37501 .51112 .37153 .50725 Metetra
.764 .745 .836 r
.37153 .50725 .37501 .51112 .38087 .50698 .3774 .50306 Metetra
.765 .744 .833 r
.3774 .50306 .38087 .50698 .38673 .50277 .38327 .49881 Metetra
.766 .743 .831 r
.38327 .49881 .38673 .50277 .3926 .4985 .38915 .4945 Metetra
.768 .741 .829 r
.38915 .4945 .3926 .4985 .39848 .49416 .39504 .49013 Metetra
.77 .741 .826 r
.39504 .49013 .39848 .49416 .40436 .48974 .40093 .48567 Metetra
.772 .74 .824 r
.40093 .48567 .40436 .48974 .41025 .48524 .40683 .48113 Metetra
.773 .739 .821 r
.40683 .48113 .41025 .48524 .41615 .48065 .41274 .47652 Metetra
.775 .738 .819 r
.41274 .47652 .41615 .48065 .42205 .47597 .41865 .47182 Metetra
.777 .738 .817 r
.41865 .47182 .42205 .47597 .42795 .47121 .42457 .46702 Metetra
.78 .738 .814 r
.42457 .46702 .42795 .47121 .43387 .46634 .43049 .46213 Metetra
.782 .738 .812 r
.43049 .46213 .43387 .46634 .43978 .46135 .43641 .45714 Metetra
.785 .738 .81 r
.43641 .45714 .43978 .46135 .4457 .45626 .44234 .45205 Metetra
.787 .739 .808 r
.44234 .45205 .4457 .45626 .45163 .45106 .44828 .44685 Metetra
.79 .74 .806 r
.44828 .44685 .45163 .45106 .45756 .44574 .45422 .44156 Metetra
.794 .742 .805 r
.45422 .44156 .45756 .44574 .46349 .44029 .46016 .43615 Metetra
.797 .745 .804 r
.46016 .43615 .46349 .44029 .46942 .43474 .4661 .43065 Metetra
.8 .747 .804 r
.4661 .43065 .46942 .43474 .47535 .42909 .47204 .42507 Metetra
.804 .751 .804 r
.47204 .42507 .47535 .42909 .48128 .42334 .47798 .4194 Metetra
.807 .756 .805 r
.47798 .4194 .48128 .42334 .48722 .41751 .48393 .41367 Metetra
.811 .761 .806 r
.48393 .41367 .48722 .41751 .49315 .41162 .48987 .40789 Metetra
.814 .766 .809 r
.48987 .40789 .49315 .41162 .49908 .40571 .49581 .4021 Metetra
.724 .249 0 r
.49581 .4021 .49908 .40571 .50492 .41457 .50165 .41091 Metetra
.72 .246 0 r
.50165 .41091 .50492 .41457 .51081 .42348 .50754 .4198 Metetra
.716 .242 0 r
.50754 .4198 .51081 .42348 .51675 .43245 .51348 .42873 Metetra
.712 .239 0 r
.51348 .42873 .51675 .43245 .52275 .44145 .51947 .43772 Metetra
.708 .236 0 r
.51947 .43772 .52275 .44145 .5288 .45048 .52552 .44675 Metetra
.706 .233 0 r
.52552 .44675 .5288 .45048 .5349 .45955 .53162 .45583 Metetra
.705 .232 0 r
.53162 .45583 .5349 .45955 .54106 .46865 .53778 .46496 Metetra
.706 .231 0 r
.53778 .46496 .54106 .46865 .54728 .4778 .54399 .47415 Metetra
.708 .232 0 r
.54399 .47415 .54728 .4778 .55355 .48701 .55026 .4834 Metetra
.71 .234 0 r
.55026 .4834 .55355 .48701 .55987 .4963 .55658 .49273 Metetra
.714 .237 0 r
.55658 .49273 .55987 .4963 .56626 .50568 .56297 .50216 Metetra
.718 .24 0 r
.56297 .50216 .56626 .50568 .5727 .51517 .56941 .51168 Metetra
.722 .243 0 r
.56941 .51168 .5727 .51517 .5792 .52478 .57591 .52132 Metetra
.725 .246 0 r
.57591 .52132 .5792 .52478 .58577 .53452 .58247 .53108 Metetra
.728 .249 0 r
.58247 .53108 .58577 .53452 .59239 .54441 .5891 .54098 Metetra
.729 .251 0 r
.5891 .54098 .59239 .54441 .59908 .55445 .59579 .551 Metetra
.73 .252 0 r
.59579 .551 .59908 .55445 .60584 .56462 .60254 .56115 Metetra
.729 .252 0 r
.60254 .56115 .60584 .56462 .61266 .57493 .60936 .57142 Metetra
.727 .251 0 r
.60936 .57142 .61266 .57493 .61954 .58536 .61624 .58181 Metetra
.83 .802 .824 r
.61624 .58181 .61954 .58536 .62599 .57889 .6227 .57547 Metetra
.833 .81 .828 r
.6227 .57547 .62599 .57889 .63243 .57245 .62917 .56916 Metetra
.834 .817 .832 r
.62917 .56916 .63243 .57245 .63888 .56608 .63563 .56289 Metetra
.835 .823 .837 r
.63563 .56289 .63888 .56608 .64533 .5598 .6421 .55671 Metetra
.835 .829 .842 r
.6421 .55671 .64533 .5598 .65178 .55363 .64857 .55061 Metetra
.834 .834 .846 r
.64857 .55061 .65178 .55363 .65823 .54757 .65504 .54462 Metetra
.833 .838 .851 r
.65504 .54462 .65823 .54757 .66469 .54165 .66152 .53873 Metetra
.83 .841 .856 r
.66152 .53873 .66469 .54165 .67115 .53585 .668 .53296 Metetra
.827 .843 .86 r
.668 .53296 .67115 .53585 .67762 .53019 .67448 .52731 Metetra
.824 .845 .863 r
.67448 .52731 .67762 .53019 .6841 .52465 .68098 .52176 Metetra
.821 .845 .867 r
.68098 .52176 .6841 .52465 .69058 .51923 .68748 .51633 Metetra
.817 .844 .87 r
.68748 .51633 .69058 .51923 .69708 .51393 .69398 .511 Metetra
.813 .844 .873 r
.69398 .511 .69708 .51393 .70358 .50874 .7005 .50578 Metetra
.81 .843 .875 r
.7005 .50578 .70358 .50874 .71009 .50364 .70702 .50066 Metetra
.806 .841 .877 r
.70702 .50066 .71009 .50364 .71661 .49865 .71355 .49561 Metetra
.802 .839 .879 r
.71355 .49561 .71661 .49865 .72314 .49373 .7201 .49066 Metetra
.799 .837 .88 r
.7201 .49066 .72314 .49373 .72967 .4889 .72664 .48579 Metetra
.795 .835 .882 r
.72664 .48579 .72967 .4889 .73622 .48415 .7332 .481 Metetra
.792 .833 .883 r
.7332 .481 .73622 .48415 .74277 .47947 .73977 .47627 Metetra
.789 .83 .884 r
.73977 .47627 .74277 .47947 .74934 .47485 .74635 .47161 Metetra
.786 .828 .885 r
.74635 .47161 .74934 .47485 .75592 .47029 .75294 .467 Metetra
.783 .825 .885 r
.75294 .467 .75592 .47029 .7625 .46579 .75953 .46246 Metetra
.78 .823 .886 r
.75953 .46246 .7625 .46579 .7691 .46133 .76614 .45796 Metetra
.778 .821 .886 r
.76614 .45796 .7691 .46133 .7757 .45693 .77276 .45352 Metetra
.776 .818 .886 r
.77276 .45352 .7757 .45693 .78231 .45257 .77938 .44911 Metetra
.774 .816 .886 r
.77938 .44911 .78231 .45257 .78894 .44824 .78602 .44474 Metetra
.772 .814 .886 r
.78602 .44474 .78894 .44824 .79557 .44395 .79266 .44041 Metetra
.769 .812 .886 r
.79266 .44041 .79557 .44395 .80222 .43968 .79932 .43611 Metetra
.768 .81 .886 r
.79932 .43611 .80222 .43968 .80887 .43545 .80598 .43184 Metetra
.766 .808 .886 r
.80598 .43184 .80887 .43545 .81553 .43125 .81266 .42759 Metetra
.764 .805 .886 r
.81266 .42759 .81553 .43125 .82221 .42707 .81934 .42338 Metetra
.763 .804 .886 r
.81934 .42338 .82221 .42707 .82889 .42291 .82604 .41918 Metetra
.762 .802 .885 r
.82604 .41918 .82889 .42291 .83558 .41876 .83274 .41499 Metetra
.761 .8 .885 r
.83274 .41499 .83558 .41876 .84228 .41463 .83945 .41082 Metetra
.76 .799 .884 r
.83945 .41082 .84228 .41463 .84899 .4105 .84617 .40665 Metetra
.759 .797 .884 r
.84617 .40665 .84899 .4105 .85571 .40638 .8529 .4025 Metetra
.758 .795 .883 r
.8529 .4025 .85571 .40638 .86244 .40226 .85964 .39834 Metetra
.757 .793 .882 r
.85964 .39834 .86244 .40226 .86918 .39815 .86639 .3942 Metetra
.757 .792 .882 r
.86639 .3942 .86918 .39815 .87592 .39404 .87315 .39005 Metetra
.756 .791 .881 r
.87315 .39005 .87592 .39404 .88268 .38993 .87991 .3859 Metetra
.755 .768 .863 r
.2577 .57618 .26141 .57942 .26714 .5758 .26345 .57254 Metetra
.755 .767 .862 r
.26345 .57254 .26714 .5758 .27288 .57218 .2692 .56889 Metetra
.755 .765 .862 r
.2692 .56889 .27288 .57218 .27863 .56855 .27496 .56523 Metetra
.755 .765 .861 r
.27496 .56523 .27863 .56855 .28438 .56492 .28072 .56156 Metetra
.755 .763 .86 r
.28072 .56156 .28438 .56492 .29015 .56126 .28649 .55788 Metetra
.755 .762 .859 r
.28649 .55788 .29015 .56126 .29591 .55759 .29227 .55418 Metetra
.755 .761 .858 r
.29227 .55418 .29591 .55759 .30169 .5539 .29806 .55046 Metetra
.755 .76 .857 r
.29806 .55046 .30169 .5539 .30747 .55019 .30385 .54672 Metetra
.756 .759 .855 r
.30385 .54672 .30747 .55019 .31325 .54647 .30965 .54296 Metetra
.756 .758 .854 r
.30965 .54296 .31325 .54647 .31905 .54272 .31545 .53918 Metetra
.757 .757 .853 r
.31545 .53918 .31905 .54272 .32485 .53894 .32127 .53537 Metetra
.757 .756 .851 r
.32127 .53537 .32485 .53894 .33066 .53512 .32709 .53152 Metetra
.758 .755 .849 r
.32709 .53152 .33066 .53512 .33648 .53126 .33291 .52763 Metetra
.759 .754 .848 r
.33291 .52763 .33648 .53126 .3423 .52737 .33875 .5237 Metetra
.76 .753 .846 r
.33875 .5237 .3423 .52737 .34813 .52343 .34459 .51974 Metetra
.761 .752 .844 r
.34459 .51974 .34813 .52343 .35397 .51946 .35044 .51573 Metetra
.762 .751 .843 r
.35044 .51573 .35397 .51946 .35982 .51545 .3563 .51168 Metetra
.763 .749 .841 r
.3563 .51168 .35982 .51545 .36567 .51137 .36216 .50757 Metetra
.764 .749 .839 r
.36216 .50757 .36567 .51137 .37153 .50725 .36803 .50341 Metetra
.766 .748 .837 r
.36803 .50341 .37153 .50725 .3774 .50306 .37391 .49919 Metetra
.767 .747 .834 r
.37391 .49919 .3774 .50306 .38327 .49881 .37979 .49491 Metetra
.768 .746 .832 r
.37979 .49491 .38327 .49881 .38915 .4945 .38568 .49057 Metetra
.77 .745 .83 r
.38568 .49057 .38915 .4945 .39504 .49013 .39158 .48615 Metetra
.772 .744 .828 r
.39158 .48615 .39504 .49013 .40093 .48567 .39749 .48167 Metetra
.773 .743 .825 r
.39749 .48167 .40093 .48567 .40683 .48113 .4034 .4771 Metetra
.775 .743 .823 r
.4034 .4771 .40683 .48113 .41274 .47652 .40931 .47246 Metetra
.777 .742 .821 r
.40931 .47246 .41274 .47652 .41865 .47182 .41523 .46773 Metetra
.779 .742 .819 r
.41523 .46773 .41865 .47182 .42457 .46702 .42116 .46292 Metetra
.781 .742 .817 r
.42116 .46292 .42457 .46702 .43049 .46213 .42709 .45802 Metetra
.784 .742 .815 r
.42709 .45802 .43049 .46213 .43641 .45714 .43303 .45302 Metetra
.786 .743 .813 r
.43303 .45302 .43641 .45714 .44234 .45205 .43897 .44792 Metetra
.789 .743 .811 r
.43897 .44792 .44234 .45205 .44828 .44685 .44492 .44274 Metetra
.792 .745 .81 r
.44492 .44274 .44828 .44685 .45422 .44156 .45086 .43747 Metetra
.795 .747 .808 r
.45086 .43747 .45422 .44156 .46016 .43615 .45681 .43209 Metetra
.798 .748 .808 r
.45681 .43209 .46016 .43615 .4661 .43065 .46276 .42664 Metetra
.801 .751 .807 r
.46276 .42664 .4661 .43065 .47204 .42507 .46872 .42111 Metetra
.804 .755 .807 r
.46872 .42111 .47204 .42507 .47798 .4194 .47467 .41551 Metetra
.807 .758 .808 r
.47467 .41551 .47798 .4194 .48393 .41367 .48063 .40986 Metetra
.809 .763 .809 r
.48063 .40986 .48393 .41367 .48987 .40789 .48658 .40417 Metetra
.812 .767 .811 r
.48658 .40417 .48987 .40789 .49581 .4021 .49253 .39847 Metetra
.723 .247 0 r
.49253 .39847 .49581 .4021 .50165 .41091 .49837 .40726 Metetra
.72 .245 0 r
.49837 .40726 .50165 .41091 .50754 .4198 .50425 .41612 Metetra
.717 .242 0 r
.50425 .41612 .50754 .4198 .51348 .42873 .51019 .42504 Metetra
.714 .24 0 r
.51019 .42504 .51348 .42873 .51947 .43772 .51618 .43401 Metetra
.711 .237 0 r
.51618 .43401 .51947 .43772 .52552 .44675 .52223 .44305 Metetra
.71 .236 0 r
.52223 .44305 .52552 .44675 .53162 .45583 .52832 .45213 Metetra
.709 .235 0 r
.52832 .45213 .53162 .45583 .53778 .46496 .53448 .46128 Metetra
.709 .234 0 r
.53448 .46128 .53778 .46496 .54399 .47415 .54069 .47049 Metetra
.711 .235 0 r
.54069 .47049 .54399 .47415 .55026 .4834 .54696 .47978 Metetra
.713 .236 0 r
.54696 .47978 .55026 .4834 .55658 .49273 .55328 .48915 Metetra
.715 .238 0 r
.55328 .48915 .55658 .49273 .56297 .50216 .55966 .4986 Metetra
.718 .241 0 r
.55966 .4986 .56297 .50216 .56941 .51168 .5661 .50815 Metetra
.721 .243 0 r
.5661 .50815 .56941 .51168 .57591 .52132 .5726 .51781 Metetra
.723 .245 0 r
.5726 .51781 .57591 .52132 .58247 .53108 .57916 .52759 Metetra
.725 .247 0 r
.57916 .52759 .58247 .53108 .5891 .54098 .58579 .53749 Metetra
.726 .249 0 r
.58579 .53749 .5891 .54098 .59579 .551 .59247 .5475 Metetra
.726 .249 0 r
.59247 .5475 .59579 .551 .60254 .56115 .59922 .55764 Metetra
.726 .249 0 r
.59922 .55764 .60254 .56115 .60936 .57142 .60604 .56789 Metetra
.724 .248 0 r
.60604 .56789 .60936 .57142 .61624 .58181 .61292 .57825 Metetra
.828 .802 .827 r
.61292 .57825 .61624 .58181 .6227 .57547 .61941 .57201 Metetra
.83 .808 .83 r
.61941 .57201 .6227 .57547 .62917 .56916 .62589 .5658 Metetra
.831 .814 .834 r
.62589 .5658 .62917 .56916 .63563 .56289 .63237 .55962 Metetra
.831 .819 .838 r
.63237 .55962 .63563 .56289 .6421 .55671 .63885 .55352 Metetra
.831 .824 .842 r
.63885 .55352 .6421 .55671 .64857 .55061 .64534 .54749 Metetra
.831 .829 .846 r
.64534 .54749 .64857 .55061 .65504 .54462 .65183 .54154 Metetra
.829 .832 .85 r
.65183 .54154 .65504 .54462 .66152 .53873 .65832 .53568 Metetra
.827 .835 .854 r
.65832 .53568 .66152 .53873 .668 .53296 .66482 .52994 Metetra
.824 .837 .857 r
.66482 .52994 .668 .53296 .67448 .52731 .67132 .52429 Metetra
.822 .838 .861 r
.67132 .52429 .67448 .52731 .68098 .52176 .67783 .51874 Metetra
.818 .838 .864 r
.67783 .51874 .68098 .52176 .68748 .51633 .68435 .5133 Metetra
.815 .838 .867 r
.68435 .5133 .68748 .51633 .69398 .511 .69087 .50796 Metetra
.812 .837 .87 r
.69087 .50796 .69398 .511 .7005 .50578 .6974 .50271 Metetra
.809 .837 .872 r
.6974 .50271 .7005 .50578 .70702 .50066 .70394 .49755 Metetra
.805 .835 .874 r
.70394 .49755 .70702 .50066 .71355 .49561 .71048 .49249 Metetra
.802 .834 .876 r
.71048 .49249 .71355 .49561 .7201 .49066 .71704 .4875 Metetra
.799 .832 .877 r
.71704 .4875 .7201 .49066 .72664 .48579 .7236 .48259 Metetra
.795 .83 .879 r
.7236 .48259 .72664 .48579 .7332 .481 .73017 .47776 Metetra
.792 .828 .88 r
.73017 .47776 .7332 .481 .73977 .47627 .73675 .47299 Metetra
.789 .826 .881 r
.73675 .47299 .73977 .47627 .74635 .47161 .74334 .46829 Metetra
.786 .824 .882 r
.74334 .46829 .74635 .47161 .75294 .467 .74994 .46364 Metetra
.784 .822 .883 r
.74994 .46364 .75294 .467 .75953 .46246 .75655 .45906 Metetra
.781 .82 .883 r
.75655 .45906 .75953 .46246 .76614 .45796 .76317 .45452 Metetra
.778 .817 .884 r
.76317 .45452 .76614 .45796 .77276 .45352 .7698 .45004 Metetra
.776 .816 .884 r
.7698 .45004 .77276 .45352 .77938 .44911 .77643 .4456 Metetra
.774 .814 .884 r
.77643 .4456 .77938 .44911 .78602 .44474 .78308 .44119 Metetra
.772 .812 .884 r
.78308 .44119 .78602 .44474 .79266 .44041 .78974 .43682 Metetra
.77 .81 .885 r
.78974 .43682 .79266 .44041 .79932 .43611 .79641 .43247 Metetra
.768 .808 .885 r
.79641 .43247 .79932 .43611 .80598 .43184 .80308 .42817 Metetra
.766 .806 .885 r
.80308 .42817 .80598 .43184 .81266 .42759 .80977 .42389 Metetra
.765 .805 .885 r
.80977 .42389 .81266 .42759 .81934 .42338 .81646 .41964 Metetra
.763 .802 .884 r
.81646 .41964 .81934 .42338 .82604 .41918 .82317 .41541 Metetra
.763 .801 .884 r
.82317 .41541 .82604 .41918 .83274 .41499 .82988 .41118 Metetra
.762 .799 .884 r
.82988 .41118 .83274 .41499 .83945 .41082 .83661 .40698 Metetra
.76 .798 .883 r
.83661 .40698 .83945 .41082 .84617 .40665 .84334 .40278 Metetra
.76 .797 .883 r
.84334 .40278 .84617 .40665 .8529 .4025 .85008 .39859 Metetra
.759 .795 .882 r
.85008 .39859 .8529 .4025 .85964 .39834 .85683 .3944 Metetra
.758 .794 .882 r
.85683 .3944 .85964 .39834 .86639 .3942 .86359 .39022 Metetra
.758 .792 .881 r
.86359 .39022 .86639 .3942 .87315 .39005 .87036 .38604 Metetra
.757 .791 .88 r
.87036 .38604 .87315 .39005 .87991 .3859 .87713 .38186 Metetra
.756 .768 .863 r
.25398 .57294 .2577 .57618 .26345 .57254 .25974 .56927 Metetra
.756 .767 .862 r
.25974 .56927 .26345 .57254 .2692 .56889 .2655 .56559 Metetra
.756 .766 .861 r
.2655 .56559 .2692 .56889 .27496 .56523 .27127 .5619 Metetra
.755 .765 .86 r
.27127 .5619 .27496 .56523 .28072 .56156 .27705 .55821 Metetra
.756 .764 .859 r
.27705 .55821 .28072 .56156 .28649 .55788 .28283 .55449 Metetra
.756 .763 .859 r
.28283 .55449 .28649 .55788 .29227 .55418 .28862 .55077 Metetra
.756 .762 .858 r
.28862 .55077 .29227 .55418 .29806 .55046 .29441 .54702 Metetra
.756 .761 .856 r
.29441 .54702 .29806 .55046 .30385 .54672 .30022 .54325 Metetra
.756 .76 .855 r
.30022 .54325 .30385 .54672 .30965 .54296 .30602 .53947 Metetra
.757 .759 .854 r
.30602 .53947 .30965 .54296 .31545 .53918 .31184 .53566 Metetra
.758 .758 .853 r
.31184 .53566 .31545 .53918 .32127 .53537 .31766 .53182 Metetra
.759 .758 .851 r
.31766 .53182 .32127 .53537 .32709 .53152 .3235 .52794 Metetra
.76 .757 .85 r
.3235 .52794 .32709 .53152 .33291 .52763 .32933 .52402 Metetra
.76 .755 .848 r
.32933 .52402 .33291 .52763 .33875 .5237 .33518 .52006 Metetra
.761 .755 .847 r
.33518 .52006 .33875 .5237 .34459 .51974 .34103 .51606 Metetra
.762 .754 .845 r
.34103 .51606 .34459 .51974 .35044 .51573 .34689 .51203 Metetra
.763 .753 .843 r
.34689 .51203 .35044 .51573 .3563 .51168 .35276 .50795 Metetra
.765 .752 .842 r
.35276 .50795 .3563 .51168 .36216 .50757 .35863 .50381 Metetra
.766 .751 .84 r
.35863 .50381 .36216 .50757 .36803 .50341 .36452 .49961 Metetra
.767 .75 .838 r
.36452 .49961 .36803 .50341 .37391 .49919 .3704 .49536 Metetra
.769 .75 .836 r
.3704 .49536 .37391 .49919 .37979 .49491 .3763 .49105 Metetra
.77 .749 .834 r
.3763 .49105 .37979 .49491 .38568 .49057 .3822 .48668 Metetra
.772 .748 .831 r
.3822 .48668 .38568 .49057 .39158 .48615 .38811 .48224 Metetra
.773 .747 .829 r
.38811 .48224 .39158 .48615 .39749 .48167 .39402 .47772 Metetra
.775 .747 .827 r
.39402 .47772 .39749 .48167 .4034 .4771 .39995 .47313 Metetra
.777 .746 .825 r
.39995 .47313 .4034 .4771 .40931 .47246 .40587 .46847 Metetra
.779 .746 .823 r
.40587 .46847 .40931 .47246 .41523 .46773 .41181 .46372 Metetra
.781 .746 .821 r
.41181 .46372 .41523 .46773 .42116 .46292 .41774 .45889 Metetra
.783 .746 .819 r
.41774 .45889 .42116 .46292 .42709 .45802 .42369 .45397 Metetra
.785 .746 .817 r
.42369 .45397 .42709 .45802 .43303 .45302 .42963 .44897 Metetra
.788 .747 .815 r
.42963 .44897 .43303 .45302 .43897 .44792 .43558 .44388 Metetra
.79 .748 .814 r
.43558 .44388 .43897 .44792 .44492 .44274 .44154 .4387 Metetra
.793 .749 .812 r
.44154 .4387 .44492 .44274 .45086 .43747 .4475 .43345 Metetra
.795 .75 .811 r
.4475 .43345 .45086 .43747 .45681 .43209 .45346 .4281 Metetra
.798 .752 .811 r
.45346 .4281 .45681 .43209 .46276 .42664 .45942 .42268 Metetra
.801 .755 .81 r
.45942 .42268 .46276 .42664 .46872 .42111 .46538 .4172 Metetra
.803 .757 .81 r
.46538 .4172 .46872 .42111 .47467 .41551 .47135 .41167 Metetra
.806 .761 .811 r
.47135 .41167 .47467 .41551 .48063 .40986 .47731 .40608 Metetra
.808 .764 .812 r
.47731 .40608 .48063 .40986 .48658 .40417 .48327 .40046 Metetra
.81 .768 .814 r
.48327 .40046 .48658 .40417 .49253 .39847 .48924 .39483 Metetra
.721 .246 0 r
.48924 .39483 .49253 .39847 .49837 .40726 .49507 .4036 Metetra
.719 .244 0 r
.49507 .4036 .49837 .40726 .50425 .41612 .50095 .41244 Metetra
.717 .243 0 r
.50095 .41244 .50425 .41612 .51019 .42504 .50689 .42135 Metetra
.715 .241 0 r
.50689 .42135 .51019 .42504 .51618 .43401 .51288 .43032 Metetra
.713 .239 0 r
.51288 .43032 .51618 .43401 .52223 .44305 .51892 .43935 Metetra
.712 .238 0 r
.51892 .43935 .52223 .44305 .52832 .45213 .52501 .44845 Metetra
.712 .237 0 r
.52501 .44845 .52832 .45213 .53448 .46128 .53117 .45761 Metetra
.712 .237 0 r
.53117 .45761 .53448 .46128 .54069 .47049 .53737 .46684 Metetra
.713 .237 0 r
.53737 .46684 .54069 .47049 .54696 .47978 .54364 .47615 Metetra
.714 .238 0 r
.54364 .47615 .54696 .47978 .55328 .48915 .54996 .48554 Metetra
.716 .239 0 r
.54996 .48554 .55328 .48915 .55966 .4986 .55634 .49501 Metetra
.718 .241 0 r
.55634 .49501 .55966 .4986 .5661 .50815 .56278 .50459 Metetra
.72 .242 0 r
.56278 .50459 .5661 .50815 .5726 .51781 .56928 .51427 Metetra
.722 .244 0 r
.56928 .51427 .5726 .51781 .57916 .52759 .57584 .52405 Metetra
.723 .246 0 r
.57584 .52405 .57916 .52759 .58579 .53749 .58246 .53395 Metetra
.724 .247 0 r
.58246 .53395 .58579 .53749 .59247 .5475 .58915 .54397 Metetra
.724 .247 0 r
.58915 .54397 .59247 .5475 .59922 .55764 .5959 .55409 Metetra
.724 .247 0 r
.5959 .55409 .59922 .55764 .60604 .56789 .60271 .56433 Metetra
.722 .247 0 r
.60271 .56433 .60604 .56789 .61292 .57825 .60959 .57467 Metetra
.825 .802 .829 r
.60959 .57467 .61292 .57825 .61941 .57201 .61609 .56852 Metetra
.827 .807 .832 r
.61609 .56852 .61941 .57201 .62589 .5658 .62259 .56238 Metetra
.828 .812 .835 r
.62259 .56238 .62589 .5658 .63237 .55962 .62909 .55628 Metetra
.828 .816 .838 r
.62909 .55628 .63237 .55962 .63885 .55352 .63559 .55023 Metetra
.828 .82 .842 r
.63559 .55023 .63885 .55352 .64534 .54749 .6421 .54426 Metetra
.827 .824 .845 r
.6421 .54426 .64534 .54749 .65183 .54154 .6486 .53835 Metetra
.826 .827 .849 r
.6486 .53835 .65183 .54154 .65832 .53568 .65511 .53252 Metetra
.824 .829 .852 r
.65511 .53252 .65832 .53568 .66482 .52994 .66162 .52679 Metetra
.822 .83 .855 r
.66162 .52679 .66482 .52994 .67132 .52429 .66814 .52115 Metetra
.819 .832 .858 r
.66814 .52115 .67132 .52429 .67783 .51874 .67467 .51561 Metetra
.817 .832 .861 r
.67467 .51561 .67783 .51874 .68435 .5133 .6812 .51015 Metetra
.813 .832 .864 r
.6812 .51015 .68435 .5133 .69087 .50796 .68773 .5048 Metetra
.811 .832 .867 r
.68773 .5048 .69087 .50796 .6974 .50271 .69428 .49953 Metetra
.807 .831 .869 r
.69428 .49953 .6974 .50271 .70394 .49755 .70083 .49435 Metetra
.804 .83 .871 r
.70083 .49435 .70394 .49755 .71048 .49249 .70739 .48925 Metetra
.801 .829 .873 r
.70739 .48925 .71048 .49249 .71704 .4875 .71396 .48424 Metetra
.798 .828 .874 r
.71396 .48424 .71704 .4875 .7236 .48259 .72053 .47929 Metetra
.795 .826 .876 r
.72053 .47929 .7236 .48259 .73017 .47776 .72712 .47443 Metetra
.792 .824 .877 r
.72712 .47443 .73017 .47776 .73675 .47299 .73371 .46963 Metetra
.789 .822 .878 r
.73371 .46963 .73675 .47299 .74334 .46829 .74031 .46489 Metetra
.786 .821 .879 r
.74031 .46489 .74334 .46829 .74994 .46364 .74693 .46021 Metetra
.783 .818 .88 r
.74693 .46021 .74994 .46364 .75655 .45906 .75355 .45558 Metetra
.781 .817 .881 r
.75355 .45558 .75655 .45906 .76317 .45452 .76018 .45102 Metetra
.779 .815 .882 r
.76018 .45102 .76317 .45452 .7698 .45004 .76682 .44649 Metetra
.776 .813 .882 r
.76682 .44649 .7698 .45004 .77643 .4456 .77347 .44202 Metetra
.774 .811 .882 r
.77347 .44202 .77643 .4456 .78308 .44119 .78013 .43758 Metetra
.773 .81 .882 r
.78013 .43758 .78308 .44119 .78974 .43682 .7868 .43316 Metetra
.77 .808 .883 r
.7868 .43316 .78974 .43682 .79641 .43247 .79348 .4288 Metetra
.769 .806 .883 r
.79348 .4288 .79641 .43247 .80308 .42817 .80016 .42446 Metetra
.767 .805 .883 r
.80016 .42446 .80308 .42817 .80977 .42389 .80686 .42014 Metetra
.765 .803 .883 r
.80686 .42014 .80977 .42389 .81646 .41964 .81357 .41586 Metetra
.764 .802 .883 r
.81357 .41586 .81646 .41964 .82317 .41541 .82029 .41159 Metetra
.763 .8 .883 r
.82029 .41159 .82317 .41541 .82988 .41118 .82701 .40734 Metetra
.762 .799 .882 r
.82701 .40734 .82988 .41118 .83661 .40698 .83375 .4031 Metetra
.761 .797 .882 r
.83375 .4031 .83661 .40698 .84334 .40278 .84049 .39887 Metetra
.76 .796 .882 r
.84049 .39887 .84334 .40278 .85008 .39859 .84725 .39465 Metetra
.76 .795 .881 r
.84725 .39465 .85008 .39859 .85683 .3944 .85401 .39043 Metetra
.759 .793 .881 r
.85401 .39043 .85683 .3944 .86359 .39022 .86078 .38622 Metetra
.759 .792 .88 r
.86078 .38622 .86359 .39022 .87036 .38604 .86756 .38201 Metetra
.758 .791 .88 r
.86756 .38201 .87036 .38604 .87713 .38186 .87434 .3778 Metetra
.756 .768 .862 r
.25025 .56968 .25398 .57294 .25974 .56927 .25602 .56598 Metetra
.757 .767 .862 r
.25602 .56598 .25974 .56927 .2655 .56559 .26179 .56228 Metetra
.756 .766 .861 r
.26179 .56228 .2655 .56559 .27127 .5619 .26757 .55857 Metetra
.757 .766 .86 r
.26757 .55857 .27127 .5619 .27705 .55821 .27336 .55484 Metetra
.756 .764 .859 r
.27336 .55484 .27705 .55821 .28283 .55449 .27915 .55111 Metetra
.757 .764 .859 r
.27915 .55111 .28283 .55449 .28862 .55077 .28495 .54735 Metetra
.757 .763 .858 r
.28495 .54735 .28862 .55077 .29441 .54702 .29075 .54359 Metetra
.757 .762 .857 r
.29075 .54359 .29441 .54702 .30022 .54325 .29657 .5398 Metetra
.758 .762 .856 r
.29657 .5398 .30022 .54325 .30602 .53947 .30239 .53598 Metetra
.758 .76 .855 r
.30239 .53598 .30602 .53947 .31184 .53566 .30821 .53215 Metetra
.759 .76 .853 r
.30821 .53215 .31184 .53566 .31766 .53182 .31405 .52827 Metetra
.76 .759 .852 r
.31405 .52827 .31766 .53182 .3235 .52794 .31989 .52437 Metetra
.761 .758 .85 r
.31989 .52437 .3235 .52794 .32933 .52402 .32574 .52043 Metetra
.762 .758 .849 r
.32574 .52043 .32933 .52402 .33518 .52006 .33159 .51644 Metetra
.763 .757 .847 r
.33159 .51644 .33518 .52006 .34103 .51606 .33746 .51242 Metetra
.763 .756 .846 r
.33746 .51242 .34103 .51606 .34689 .51203 .34333 .50835 Metetra
.765 .755 .844 r
.34333 .50835 .34689 .51203 .35276 .50795 .34921 .50424 Metetra
.766 .754 .842 r
.34921 .50424 .35276 .50795 .35863 .50381 .35509 .50008 Metetra
.767 .754 .841 r
.35509 .50008 .35863 .50381 .36452 .49961 .36098 .49586 Metetra
.769 .753 .839 r
.36098 .49586 .36452 .49961 .3704 .49536 .36688 .49158 Metetra
.77 .752 .837 r
.36688 .49158 .3704 .49536 .3763 .49105 .37279 .48724 Metetra
.772 .752 .835 r
.37279 .48724 .3763 .49105 .3822 .48668 .3787 .48284 Metetra
.773 .751 .833 r
.3787 .48284 .3822 .48668 .38811 .48224 .38462 .47837 Metetra
.775 .751 .831 r
.38462 .47837 .38811 .48224 .39402 .47772 .39055 .47383 Metetra
.777 .75 .829 r
.39055 .47383 .39402 .47772 .39995 .47313 .39648 .46922 Metetra
.779 .75 .827 r
.39648 .46922 .39995 .47313 .40587 .46847 .40242 .46453 Metetra
.781 .75 .825 r
.40242 .46453 .40587 .46847 .41181 .46372 .40836 .45977 Metetra
.783 .75 .823 r
.40836 .45977 .41181 .46372 .41774 .45889 .41431 .45493 Metetra
.785 .75 .821 r
.41431 .45493 .41774 .45889 .42369 .45397 .42026 .45 Metetra
.787 .75 .819 r
.42026 .45 .42369 .45397 .42963 .44897 .42622 .44499 Metetra
.789 .751 .817 r
.42622 .44499 .42963 .44897 .43558 .44388 .43218 .4399 Metetra
.792 .751 .816 r
.43218 .4399 .43558 .44388 .44154 .4387 .43815 .43474 Metetra
.794 .753 .815 r
.43815 .43474 .44154 .4387 .4475 .43345 .44411 .4295 Metetra
.797 .754 .814 r
.44411 .4295 .4475 .43345 .45346 .4281 .45008 .42417 Metetra
.799 .756 .813 r
.45008 .42417 .45346 .4281 .45942 .42268 .45606 .41878 Metetra
.801 .758 .813 r
.45606 .41878 .45942 .42268 .46538 .4172 .46203 .41334 Metetra
.803 .76 .813 r
.46203 .41334 .46538 .4172 .47135 .41167 .468 .40785 Metetra
.805 .763 .813 r
.468 .40785 .47135 .41167 .47731 .40608 .47398 .40231 Metetra
.807 .766 .814 r
.47398 .40231 .47731 .40608 .48327 .40046 .47996 .39674 Metetra
.809 .769 .816 r
.47996 .39674 .48327 .40046 .48924 .39483 .48593 .39117 Metetra
.72 .245 0 r
.48593 .39117 .48924 .39483 .49507 .4036 .49176 .39993 Metetra
.719 .244 0 r
.49176 .39993 .49507 .4036 .50095 .41244 .49764 .40876 Metetra
.717 .242 0 r
.49764 .40876 .50095 .41244 .50689 .42135 .50357 .41766 Metetra
.716 .241 0 r
.50357 .41766 .50689 .42135 .51288 .43032 .50956 .42662 Metetra
.715 .24 0 r
.50956 .42662 .51288 .43032 .51892 .43935 .5156 .43566 Metetra
.714 .239 0 r
.5156 .43566 .51892 .43935 .52501 .44845 .52169 .44476 Metetra
.713 .238 0 r
.52169 .44476 .52501 .44845 .53117 .45761 .52784 .45393 Metetra
.713 .238 0 r
.52784 .45393 .53117 .45761 .53737 .46684 .53405 .46318 Metetra
.714 .238 0 r
.53405 .46318 .53737 .46684 .54364 .47615 .54031 .4725 Metetra
.715 .239 0 r
.54031 .4725 .54364 .47615 .54996 .48554 .54663 .48191 Metetra
.716 .24 0 r
.54663 .48191 .54996 .48554 .55634 .49501 .55301 .49141 Metetra
.718 .241 0 r
.55301 .49141 .55634 .49501 .56278 .50459 .55944 .501 Metetra
.719 .242 0 r
.55944 .501 .56278 .50459 .56928 .51427 .56594 .51069 Metetra
.721 .244 0 r
.56594 .51069 .56928 .51427 .57584 .52405 .5725 .52049 Metetra
.722 .245 0 r
.5725 .52049 .57584 .52405 .58246 .53395 .57912 .53039 Metetra
.722 .245 0 r
.57912 .53039 .58246 .53395 .58915 .54397 .58581 .5404 Metetra
.722 .246 0 r
.58581 .5404 .58915 .54397 .5959 .55409 .59255 .55052 Metetra
.722 .246 0 r
.59255 .55052 .5959 .55409 .60271 .56433 .59937 .56074 Metetra
.721 .245 0 r
.59937 .56074 .60271 .56433 .60959 .57467 .60624 .57107 Metetra
.823 .802 .831 r
.60624 .57107 .60959 .57467 .61609 .56852 .61276 .56499 Metetra
.825 .806 .833 r
.61276 .56499 .61609 .56852 .62259 .56238 .61928 .55891 Metetra
.825 .81 .836 r
.61928 .55891 .62259 .56238 .62909 .55628 .62579 .55287 Metetra
.825 .813 .839 r
.62579 .55287 .62909 .55628 .63559 .55023 .63231 .54687 Metetra
.825 .817 .842 r
.63231 .54687 .63559 .55023 .6421 .54426 .63883 .54093 Metetra
.825 .82 .844 r
.63883 .54093 .6421 .54426 .6486 .53835 .64536 .53505 Metetra
.823 .822 .847 r
.64536 .53505 .6486 .53835 .65511 .53252 .65188 .52925 Metetra
.821 .824 .851 r
.65188 .52925 .65511 .53252 .66162 .52679 .65841 .52354 Metetra
.82 .825 .854 r
.65841 .52354 .66162 .52679 .66814 .52115 .66495 .51791 Metetra
.817 .826 .856 r
.66495 .51791 .66814 .52115 .67467 .51561 .67148 .51236 Metetra
.815 .827 .859 r
.67148 .51236 .67467 .51561 .6812 .51015 .67803 .5069 Metetra
.812 .827 .862 r
.67803 .5069 .6812 .51015 .68773 .5048 .68458 .50153 Metetra
.809 .826 .864 r
.68458 .50153 .68773 .5048 .69428 .49953 .69114 .49624 Metetra
.806 .826 .866 r
.69114 .49624 .69428 .49953 .70083 .49435 .69771 .49104 Metetra
.803 .825 .868 r
.69771 .49104 .70083 .49435 .70739 .48925 .70428 .48593 Metetra
.801 .824 .87 r
.70428 .48593 .70739 .48925 .71396 .48424 .71086 .48088 Metetra
.797 .823 .872 r
.71086 .48088 .71396 .48424 .72053 .47929 .71745 .47591 Metetra
.794 .821 .873 r
.71745 .47591 .72053 .47929 .72712 .47443 .72405 .47101 Metetra
.791 .82 .875 r
.72405 .47101 .72712 .47443 .73371 .46963 .73065 .46618 Metetra
.789 .819 .876 r
.73065 .46618 .73371 .46963 .74031 .46489 .73727 .46141 Metetra
.786 .817 .877 r
.73727 .46141 .74031 .46489 .74693 .46021 .74389 .4567 Metetra
.784 .815 .878 r
.74389 .4567 .74693 .46021 .75355 .45558 .75053 .45204 Metetra
.781 .814 .879 r
.75053 .45204 .75355 .45558 .76018 .45102 .75717 .44744 Metetra
.778 .812 .879 r
.75717 .44744 .76018 .45102 .76682 .44649 .76382 .44289 Metetra
.776 .811 .88 r
.76382 .44289 .76682 .44649 .77347 .44202 .77049 .43838 Metetra
.775 .809 .88 r
.77049 .43838 .77347 .44202 .78013 .43758 .77716 .43391 Metetra
.773 .807 .881 r
.77716 .43391 .78013 .43758 .7868 .43316 .78384 .42947 Metetra
.771 .806 .881 r
.78384 .42947 .7868 .43316 .79348 .4288 .79053 .42506 Metetra
.769 .804 .881 r
.79053 .42506 .79348 .4288 .80016 .42446 .79723 .42069 Metetra
.767 .803 .881 r
.79723 .42069 .80016 .42446 .80686 .42014 .80394 .41635 Metetra
.766 .802 .882 r
.80394 .41635 .80686 .42014 .81357 .41586 .81066 .41204 Metetra
.764 .8 .882 r
.81066 .41204 .81357 .41586 .82029 .41159 .81739 .40775 Metetra
.764 .799 .882 r
.81739 .40775 .82029 .41159 .82701 .40734 .82413 .40346 Metetra
.763 .798 .881 r
.82413 .40346 .82701 .40734 .83375 .4031 .83088 .39919 Metetra
.762 .796 .881 r
.83088 .39919 .83375 .4031 .84049 .39887 .83763 .39494 Metetra
.761 .796 .881 r
.83763 .39494 .84049 .39887 .84725 .39465 .8444 .39068 Metetra
.76 .794 .88 r
.8444 .39068 .84725 .39465 .85401 .39043 .85117 .38644 Metetra
.76 .793 .88 r
.85117 .38644 .85401 .39043 .86078 .38622 .85795 .3822 Metetra
.759 .792 .88 r
.85795 .3822 .86078 .38622 .86756 .38201 .86474 .37796 Metetra
.759 .791 .879 r
.86474 .37796 .86756 .38201 .87434 .3778 .87154 .37372 Metetra
.757 .768 .862 r
.2465 .5664 .25025 .56968 .25602 .56598 .25228 .56269 Metetra
.757 .767 .861 r
.25228 .56269 .25602 .56598 .26179 .56228 .25806 .55896 Metetra
.757 .767 .861 r
.25806 .55896 .26179 .56228 .26757 .55857 .26385 .55523 Metetra
.757 .766 .86 r
.26385 .55523 .26757 .55857 .27336 .55484 .26965 .55149 Metetra
.757 .765 .859 r
.26965 .55149 .27336 .55484 .27915 .55111 .27545 .54772 Metetra
.757 .764 .859 r
.27545 .54772 .27915 .55111 .28495 .54735 .28126 .54395 Metetra
.758 .764 .858 r
.28126 .54395 .28495 .54735 .29075 .54359 .28708 .54016 Metetra
.758 .763 .857 r
.28708 .54016 .29075 .54359 .29657 .5398 .2929 .53634 Metetra
.758 .762 .856 r
.2929 .53634 .29657 .5398 .30239 .53598 .29873 .53251 Metetra
.759 .762 .855 r
.29873 .53251 .30239 .53598 .30821 .53215 .30457 .52865 Metetra
.76 .761 .854 r
.30457 .52865 .30821 .53215 .31405 .52827 .31041 .52476 Metetra
.761 .761 .852 r
.31041 .52476 .31405 .52827 .31989 .52437 .31627 .52083 Metetra
.762 .76 .851 r
.31627 .52083 .31989 .52437 .32574 .52043 .32213 .51686 Metetra
.763 .759 .849 r
.32213 .51686 .32574 .52043 .33159 .51644 .32799 .51285 Metetra
.764 .759 .848 r
.32799 .51285 .33159 .51644 .33746 .51242 .33387 .5088 Metetra
.765 .758 .847 r
.33387 .5088 .33746 .51242 .34333 .50835 .33975 .50471 Metetra
.766 .757 .845 r
.33975 .50471 .34333 .50835 .34921 .50424 .34564 .50058 Metetra
.767 .757 .844 r
.34564 .50058 .34921 .50424 .35509 .50008 .35153 .49639 Metetra
.769 .756 .842 r
.35153 .49639 .35509 .50008 .36098 .49586 .35744 .49214 Metetra
.77 .756 .84 r
.35744 .49214 .36098 .49586 .36688 .49158 .36335 .48784 Metetra
.772 .755 .838 r
.36335 .48784 .36688 .49158 .37279 .48724 .36926 .48347 Metetra
.773 .755 .836 r
.36926 .48347 .37279 .48724 .3787 .48284 .37519 .47905 Metetra
.775 .754 .834 r
.37519 .47905 .3787 .48284 .38462 .47837 .38112 .47456 Metetra
.777 .754 .832 r
.38112 .47456 .38462 .47837 .39055 .47383 .38705 .47 Metetra
.779 .754 .83 r
.38705 .47 .39055 .47383 .39648 .46922 .393 .46537 Metetra
.78 .753 .829 r
.393 .46537 .39648 .46922 .40242 .46453 .39895 .46066 Metetra
.782 .754 .827 r
.39895 .46066 .40242 .46453 .40836 .45977 .4049 .45589 Metetra
.784 .754 .825 r
.4049 .45589 .40836 .45977 .41431 .45493 .41086 .45103 Metetra
.787 .754 .823 r
.41086 .45103 .41431 .45493 .42026 .45 .41682 .4461 Metetra
.788 .754 .821 r
.41682 .4461 .42026 .45 .42622 .44499 .42279 .44109 Metetra
.791 .755 .82 r
.42279 .44109 .42622 .44499 .43218 .4399 .42876 .436 Metetra
.793 .755 .819 r
.42876 .436 .43218 .4399 .43815 .43474 .43474 .43083 Metetra
.795 .756 .817 r
.43474 .43083 .43815 .43474 .44411 .4295 .44072 .4256 Metetra
.797 .757 .816 r
.44072 .4256 .44411 .4295 .45008 .42417 .4467 .4203 Metetra
.799 .759 .816 r
.4467 .4203 .45008 .42417 .45606 .41878 .45268 .41493 Metetra
.801 .76 .815 r
.45268 .41493 .45606 .41878 .46203 .41334 .45866 .40952 Metetra
.803 .762 .815 r
.45866 .40952 .46203 .41334 .468 .40785 .46465 .40405 Metetra
.805 .764 .815 r
.46465 .40405 .468 .40785 .47398 .40231 .47064 .39856 Metetra
.807 .767 .816 r
.47064 .39856 .47398 .40231 .47996 .39674 .47662 .39303 Metetra
.808 .769 .817 r
.47662 .39303 .47996 .39674 .48593 .39117 .48261 .3875 Metetra
.72 .244 0 r
.48261 .3875 .48593 .39117 .49176 .39993 .48844 .39625 Metetra
.719 .243 0 r
.48844 .39625 .49176 .39993 .49764 .40876 .49431 .40507 Metetra
.718 .242 0 r
.49431 .40507 .49764 .40876 .50357 .41766 .50024 .41397 Metetra
.717 .241 0 r
.50024 .41397 .50357 .41766 .50956 .42662 .50622 .42293 Metetra
.716 .241 0 r
.50622 .42293 .50956 .42662 .5156 .43566 .51226 .43196 Metetra
.715 .24 0 r
.51226 .43196 .5156 .43566 .52169 .44476 .51835 .44107 Metetra
.715 .239 0 r
.51835 .44107 .52169 .44476 .52784 .45393 .5245 .45025 Metetra
.715 .239 0 r
.5245 .45025 .52784 .45393 .53405 .46318 .5307 .45951 Metetra
.715 .239 0 r
.5307 .45951 .53405 .46318 .54031 .4725 .53696 .46885 Metetra
.716 .24 0 r
.53696 .46885 .54031 .4725 .54663 .48191 .54328 .47827 Metetra
.717 .24 0 r
.54328 .47827 .54663 .48191 .55301 .49141 .54966 .48778 Metetra
.718 .241 0 r
.54966 .48778 .55301 .49141 .55944 .501 .55609 .49738 Metetra
.719 .242 0 r
.55609 .49738 .55944 .501 .56594 .51069 .56259 .50708 Metetra
.72 .243 0 r
.56259 .50708 .56594 .51069 .5725 .52049 .56915 .51688 Metetra
.72 .244 0 r
.56915 .51688 .5725 .52049 .57912 .53039 .57577 .52679 Metetra
.721 .244 0 r
.57577 .52679 .57912 .53039 .58581 .5404 .58245 .5368 Metetra
.721 .245 0 r
.58245 .5368 .58581 .5404 .59255 .55052 .5892 .54692 Metetra
.721 .245 0 r
.5892 .54692 .59255 .55052 .59937 .56074 .59601 .55714 Metetra
.72 .245 0 r
.59601 .55714 .59937 .56074 .60624 .57107 .60288 .56746 Metetra
.822 .802 .833 r
.60288 .56746 .60624 .57107 .61276 .56499 .60942 .56142 Metetra
.823 .805 .834 r
.60942 .56142 .61276 .56499 .61928 .55891 .61595 .55539 Metetra
.823 .808 .836 r
.61595 .55539 .61928 .55891 .62579 .55287 .62248 .54939 Metetra
.823 .811 .839 r
.62248 .54939 .62579 .55287 .63231 .54687 .62902 .54343 Metetra
.823 .814 .841 r
.62902 .54343 .63231 .54687 .63883 .54093 .63555 .53753 Metetra
.822 .816 .844 r
.63555 .53753 .63883 .54093 .64536 .53505 .64209 .53167 Metetra
.821 .818 .846 r
.64209 .53167 .64536 .53505 .65188 .52925 .64863 .52588 Metetra
.819 .819 .849 r
.64863 .52588 .65188 .52925 .65841 .52354 .65518 .52018 Metetra
.817 .82 .852 r
.65518 .52018 .65841 .52354 .66495 .51791 .66173 .51456 Metetra
.815 .821 .854 r
.66173 .51456 .66495 .51791 .67148 .51236 .66828 .50901 Metetra
.813 .822 .857 r
.66828 .50901 .67148 .51236 .67803 .5069 .67484 .50354 Metetra
.81 .821 .859 r
.67484 .50354 .67803 .5069 .68458 .50153 .68141 .49816 Metetra
.807 .821 .862 r
.68141 .49816 .68458 .50153 .69114 .49624 .68798 .49286 Metetra
.805 .821 .864 r
.68798 .49286 .69114 .49624 .69771 .49104 .69456 .48764 Metetra
.802 .82 .866 r
.69456 .48764 .69771 .49104 .70428 .48593 .70115 .4825 Metetra
.799 .819 .867 r
.70115 .4825 .70428 .48593 .71086 .48088 .70774 .47743 Metetra
.797 .818 .869 r
.70774 .47743 .71086 .48088 .71745 .47591 .71435 .47243 Metetra
.793 .817 .871 r
.71435 .47243 .71745 .47591 .72405 .47101 .72096 .46751 Metetra
.791 .816 .872 r
.72096 .46751 .72405 .47101 .73065 .46618 .72758 .46266 Metetra
.788 .815 .873 r
.72758 .46266 .73065 .46618 .73727 .46141 .73421 .45786 Metetra
.786 .814 .874 r
.73421 .45786 .73727 .46141 .74389 .4567 .74084 .45311 Metetra
.783 .812 .875 r
.74084 .45311 .74389 .4567 .75053 .45204 .74749 .44843 Metetra
.781 .811 .877 r
.74749 .44843 .75053 .45204 .75717 .44744 .75415 .4438 Metetra
.779 .809 .877 r
.75415 .4438 .75717 .44744 .76382 .44289 .76081 .43922 Metetra
.776 .808 .878 r
.76081 .43922 .76382 .44289 .77049 .43838 .76749 .43468 Metetra
.774 .806 .878 r
.76749 .43468 .77049 .43838 .77716 .43391 .77417 .43018 Metetra
.773 .805 .879 r
.77417 .43018 .77716 .43391 .78384 .42947 .78087 .42571 Metetra
.771 .804 .879 r
.78087 .42571 .78384 .42947 .79053 .42506 .78757 .42128 Metetra
.769 .803 .88 r
.78757 .42128 .79053 .42506 .79723 .42069 .79428 .41689 Metetra
.768 .801 .88 r
.79428 .41689 .79723 .42069 .80394 .41635 .801 .41251 Metetra
.766 .8 .88 r
.801 .41251 .80394 .41635 .81066 .41204 .80774 .40818 Metetra
.765 .799 .88 r
.80774 .40818 .81066 .41204 .81739 .40775 .81448 .40385 Metetra
.764 .797 .88 r
.81448 .40385 .81739 .40775 .82413 .40346 .82123 .39954 Metetra
.763 .796 .88 r
.82123 .39954 .82413 .40346 .83088 .39919 .82799 .39525 Metetra
.762 .796 .88 r
.82799 .39525 .83088 .39919 .83763 .39494 .83476 .39097 Metetra
.761 .794 .88 r
.83476 .39097 .83763 .39494 .8444 .39068 .84153 .38669 Metetra
.761 .794 .88 r
.84153 .38669 .8444 .39068 .85117 .38644 .84832 .38242 Metetra
.76 .792 .879 r
.84832 .38242 .85117 .38644 .85795 .3822 .85511 .37815 Metetra
.76 .792 .879 r
.85511 .37815 .85795 .3822 .86474 .37796 .86191 .37389 Metetra
.759 .791 .879 r
.86191 .37389 .86474 .37796 .87154 .37372 .86872 .36962 Metetra
.757 .768 .862 r
.24273 .56312 .2465 .5664 .25228 .56269 .24852 .55939 Metetra
.757 .768 .861 r
.24852 .55939 .25228 .56269 .25806 .55896 .25432 .55564 Metetra
.757 .767 .861 r
.25432 .55564 .25806 .55896 .26385 .55523 .26012 .55189 Metetra
.758 .766 .86 r
.26012 .55189 .26385 .55523 .26965 .55149 .26593 .54812 Metetra
.758 .766 .859 r
.26593 .54812 .26965 .55149 .27545 .54772 .27174 .54434 Metetra
.758 .765 .859 r
.27174 .54434 .27545 .54772 .28126 .54395 .27756 .54055 Metetra
.758 .765 .858 r
.27756 .54055 .28126 .54395 .28708 .54016 .28339 .53673 Metetra
.759 .764 .857 r
.28339 .53673 .28708 .54016 .2929 .53634 .28922 .5329 Metetra
.759 .764 .856 r
.28922 .5329 .2929 .53634 .29873 .53251 .29506 .52904 Metetra
.76 .763 .855 r
.29506 .52904 .29873 .53251 .30457 .52865 .30091 .52517 Metetra
.761 .763 .854 r
.30091 .52517 .30457 .52865 .31041 .52476 .30677 .52125 Metetra
.762 .762 .853 r
.30677 .52125 .31041 .52476 .31627 .52083 .31263 .5173 Metetra
.763 .762 .852 r
.31263 .5173 .31627 .52083 .32213 .51686 .3185 .51331 Metetra
.764 .761 .85 r
.3185 .51331 .32213 .51686 .32799 .51285 .32438 .50928 Metetra
.765 .761 .849 r
.32438 .50928 .32799 .51285 .33387 .5088 .33026 .50521 Metetra
.766 .76 .848 r
.33026 .50521 .33387 .5088 .33975 .50471 .33615 .5011 Metetra
.767 .76 .846 r
.33615 .5011 .33975 .50471 .34564 .50058 .34205 .49694 Metetra
.769 .759 .844 r
.34205 .49694 .34564 .50058 .35153 .49639 .34796 .49273 Metetra
.77 .759 .843 r
.34796 .49273 .35153 .49639 .35744 .49214 .35387 .48847 Metetra
.772 .759 .841 r
.35387 .48847 .35744 .49214 .36335 .48784 .35979 .48414 Metetra
.773 .758 .839 r
.35979 .48414 .36335 .48784 .36926 .48347 .36572 .47975 Metetra
.775 .758 .838 r
.36572 .47975 .36926 .48347 .37519 .47905 .37166 .47531 Metetra
.776 .758 .836 r
.37166 .47531 .37519 .47905 .38112 .47456 .3776 .47081 Metetra
.778 .757 .834 r
.3776 .47081 .38112 .47456 .38705 .47 .38355 .46622 Metetra
.78 .757 .832 r
.38355 .46622 .38705 .47 .393 .46537 .3895 .46157 Metetra
.782 .757 .83 r
.3895 .46157 .393 .46537 .39895 .46066 .39546 .45685 Metetra
.784 .757 .828 r
.39546 .45685 .39895 .46066 .4049 .45589 .40142 .45206 Metetra
.786 .757 .827 r
.40142 .45206 .4049 .45589 .41086 .45103 .40739 .44719 Metetra
.788 .757 .825 r
.40739 .44719 .41086 .45103 .41682 .4461 .41337 .44226 Metetra
.79 .758 .823 r
.41337 .44226 .41682 .4461 .42279 .44109 .41935 .43723 Metetra
.792 .758 .822 r
.41935 .43723 .42279 .44109 .42876 .436 .42533 .43214 Metetra
.794 .758 .821 r
.42533 .43214 .42876 .436 .43474 .43083 .43131 .42699 Metetra
.796 .759 .82 r
.43131 .42699 .43474 .43083 .44072 .4256 .4373 .42176 Metetra
.798 .761 .818 r
.4373 .42176 .44072 .4256 .4467 .4203 .4433 .41647 Metetra
.8 .761 .818 r
.4433 .41647 .4467 .4203 .45268 .41493 .44929 .41112 Metetra
.802 .763 .817 r
.44929 .41112 .45268 .41493 .45866 .40952 .45529 .40572 Metetra
.803 .764 .817 r
.45529 .40572 .45866 .40952 .46465 .40405 .46128 .40029 Metetra
.805 .766 .817 r
.46128 .40029 .46465 .40405 .47064 .39856 .46728 .39481 Metetra
.806 .768 .818 r
.46728 .39481 .47064 .39856 .47662 .39303 .47328 .38932 Metetra
.807 .77 .818 r
.47328 .38932 .47662 .39303 .48261 .3875 .47928 .38382 Metetra
.719 .244 0 r
.47928 .38382 .48261 .3875 .48844 .39625 .4851 .39256 Metetra
.719 .243 0 r
.4851 .39256 .48844 .39625 .49431 .40507 .49097 .40138 Metetra
.718 .242 0 r
.49097 .40138 .49431 .40507 .50024 .41397 .4969 .41027 Metetra
.717 .242 0 r
.4969 .41027 .50024 .41397 .50622 .42293 .50288 .41923 Metetra
.716 .241 0 r
.50288 .41923 .50622 .42293 .51226 .43196 .50891 .42827 Metetra
.716 .24 0 r
.50891 .42827 .51226 .43196 .51835 .44107 .515 .43738 Metetra
.716 .24 0 r
.515 .43738 .51835 .44107 .5245 .45025 .52115 .44657 Metetra
.716 .24 0 r
.52115 .44657 .5245 .45025 .5307 .45951 .52735 .45583 Metetra
.716 .24 0 r
.52735 .45583 .5307 .45951 .53696 .46885 .53361 .46518 Metetra
.716 .24 0 r
.53361 .46518 .53696 .46885 .54328 .47827 .53992 .47461 Metetra
.717 .241 0 r
.53992 .47461 .54328 .47827 .54966 .48778 .5463 .48413 Metetra
.718 .241 0 r
.5463 .48413 .54966 .48778 .55609 .49738 .55273 .49374 Metetra
.718 .242 0 r
.55273 .49374 .55609 .49738 .56259 .50708 .55922 .50345 Metetra
.719 .243 0 r
.55922 .50345 .56259 .50708 .56915 .51688 .56578 .51326 Metetra
.72 .243 0 r
.56578 .51326 .56915 .51688 .57577 .52679 .5724 .52317 Metetra
.72 .244 0 r
.5724 .52317 .57577 .52679 .58245 .5368 .57908 .53318 Metetra
.72 .244 0 r
.57908 .53318 .58245 .5368 .5892 .54692 .58582 .5433 Metetra
.72 .244 0 r
.58582 .5433 .5892 .54692 .59601 .55714 .59263 .55351 Metetra
.72 .244 0 r
.59263 .55351 .59601 .55714 .60288 .56746 .59951 .56383 Metetra
.82 .802 .834 r
.59951 .56383 .60288 .56746 .60942 .56142 .60606 .55783 Metetra
.821 .804 .835 r
.60606 .55783 .60942 .56142 .61595 .55539 .6126 .55183 Metetra
.821 .806 .837 r
.6126 .55183 .61595 .55539 .62248 .54939 .61915 .54586 Metetra
.821 .809 .839 r
.61915 .54586 .62248 .54939 .62902 .54343 .6257 .53993 Metetra
.821 .811 .841 r
.6257 .53993 .62902 .54343 .63555 .53753 .63226 .53404 Metetra
.82 .812 .843 r
.63226 .53404 .63555 .53753 .64209 .53167 .63881 .5282 Metetra
.819 .814 .845 r
.63881 .5282 .64209 .53167 .64863 .52588 .64537 .52243 Metetra
.817 .815 .848 r
.64537 .52243 .64863 .52588 .65518 .52018 .65193 .51673 Metetra
.815 .816 .85 r
.65193 .51673 .65518 .52018 .66173 .51456 .65849 .51111 Metetra
.813 .816 .852 r
.65849 .51111 .66173 .51456 .66828 .50901 .66506 .50555 Metetra
.811 .817 .855 r
.66506 .50555 .66828 .50901 .67484 .50354 .67164 .50008 Metetra
.809 .817 .857 r
.67164 .50008 .67484 .50354 .68141 .49816 .67822 .49469 Metetra
.806 .817 .859 r
.67822 .49469 .68141 .49816 .68798 .49286 .68481 .48937 Metetra
.803 .816 .861 r
.68481 .48937 .68798 .49286 .69456 .48764 .6914 .48414 Metetra
.801 .816 .863 r
.6914 .48414 .69456 .48764 .70115 .4825 .698 .47898 Metetra
.799 .815 .865 r
.698 .47898 .70115 .4825 .70774 .47743 .70461 .47388 Metetra
.795 .814 .867 r
.70461 .47388 .70774 .47743 .71435 .47243 .71123 .46887 Metetra
.793 .813 .868 r
.71123 .46887 .71435 .47243 .72096 .46751 .71785 .46392 Metetra
.79 .812 .87 r
.71785 .46392 .72096 .46751 .72758 .46266 .72448 .45905 Metetra
.788 .811 .871 r
.72448 .45905 .72758 .46266 .73421 .45786 .73113 .45422 Metetra
.785 .81 .872 r
.73113 .45422 .73421 .45786 .74084 .45311 .73778 .44946 Metetra
.783 .809 .873 r
.73778 .44946 .74084 .45311 .74749 .44843 .74444 .44474 Metetra
.78 .807 .874 r
.74444 .44474 .74749 .44843 .75415 .4438 .7511 .44009 Metetra
.778 .806 .875 r
.7511 .44009 .75415 .4438 .76081 .43922 .75778 .43548 Metetra
.776 .805 .876 r
.75778 .43548 .76081 .43922 .76749 .43468 .76447 .43092 Metetra
.774 .804 .877 r
.76447 .43092 .76749 .43468 .77417 .43018 .77117 .4264 Metetra
.773 .803 .877 r
.77117 .4264 .77417 .43018 .78087 .42571 .77787 .4219 Metetra
.771 .802 .878 r
.77787 .4219 .78087 .42571 .78757 .42128 .78459 .41745 Metetra
.769 .8 .878 r
.78459 .41745 .78757 .42128 .79428 .41689 .79132 .41302 Metetra
.767 .799 .879 r
.79132 .41302 .79428 .41689 .801 .41251 .79805 .40863 Metetra
.766 .799 .879 r
.79805 .40863 .801 .41251 .80774 .40818 .8048 .40426 Metetra
.765 .797 .879 r
.8048 .40426 .80774 .40818 .81448 .40385 .81155 .39993 Metetra
.764 .797 .879 r
.81155 .39993 .81448 .40385 .82123 .39954 .81831 .39559 Metetra
.763 .796 .879 r
.81831 .39559 .82123 .39954 .82799 .39525 .82508 .39127 Metetra
.762 .795 .879 r
.82508 .39127 .82799 .39525 .83476 .39097 .83187 .38697 Metetra
.762 .794 .879 r
.83187 .38697 .83476 .39097 .84153 .38669 .83865 .38266 Metetra
.761 .793 .879 r
.83865 .38266 .84153 .38669 .84832 .38242 .84545 .37837 Metetra
.76 .793 .879 r
.84545 .37837 .84832 .38242 .85511 .37815 .85226 .37408 Metetra
.76 .792 .879 r
.85226 .37408 .85511 .37815 .86191 .37389 .85907 .36979 Metetra
.76 .791 .879 r
.85907 .36979 .86191 .37389 .86872 .36962 .8659 .36551 Metetra
.758 .768 .861 r
.23895 .55982 .24273 .56312 .24852 .55939 .24475 .55607 Metetra
.758 .767 .861 r
.24475 .55607 .24852 .55939 .25432 .55564 .25056 .55231 Metetra
.758 .767 .861 r
.25056 .55231 .25432 .55564 .26012 .55189 .25637 .54854 Metetra
.758 .767 .86 r
.25637 .54854 .26012 .55189 .26593 .54812 .26219 .54476 Metetra
.758 .767 .859 r
.26219 .54476 .26593 .54812 .27174 .54434 .26801 .54096 Metetra
.758 .766 .859 r
.26801 .54096 .27174 .54434 .27756 .54055 .27384 .53715 Metetra
.759 .766 .858 r
.27384 .53715 .27756 .54055 .28339 .53673 .27968 .53331 Metetra
.76 .765 .857 r
.27968 .53331 .28339 .53673 .28922 .5329 .28552 .52946 Metetra
.76 .765 .857 r
.28552 .52946 .28922 .5329 .29506 .52904 .29138 .52559 Metetra
.761 .765 .856 r
.29138 .52559 .29506 .52904 .30091 .52517 .29723 .52169 Metetra
.761 .764 .855 r
.29723 .52169 .30091 .52517 .30677 .52125 .3031 .51776 Metetra
.763 .764 .854 r
.3031 .51776 .30677 .52125 .31263 .5173 .30897 .51379 Metetra
.764 .764 .852 r
.30897 .51379 .31263 .5173 .3185 .51331 .31485 .50978 Metetra
.765 .763 .851 r
.31485 .50978 .3185 .51331 .32438 .50928 .32074 .50573 Metetra
.766 .763 .85 r
.32074 .50573 .32438 .50928 .33026 .50521 .32664 .50165 Metetra
.767 .762 .849 r
.32664 .50165 .33026 .50521 .33615 .5011 .33254 .49752 Metetra
.768 .762 .847 r
.33254 .49752 .33615 .5011 .34205 .49694 .33845 .49334 Metetra
.77 .762 .846 r
.33845 .49334 .34205 .49694 .34796 .49273 .34437 .48911 Metetra
.771 .762 .844 r
.34437 .48911 .34796 .49273 .35387 .48847 .35029 .48482 Metetra
.773 .761 .842 r
.35029 .48482 .35387 .48847 .35979 .48414 .35623 .48048 Metetra
.775 .761 .841 r
.35623 .48048 .35979 .48414 .36572 .47975 .36216 .47608 Metetra
.776 .761 .839 r
.36216 .47608 .36572 .47975 .37166 .47531 .36811 .47162 Metetra
.778 .761 .838 r
.36811 .47162 .37166 .47531 .3776 .47081 .37406 .46709 Metetra
.78 .76 .836 r
.37406 .46709 .3776 .47081 .38355 .46622 .38002 .4625 Metetra
.782 .76 .834 r
.38002 .4625 .38355 .46622 .3895 .46157 .38598 .45783 Metetra
.783 .76 .832 r
.38598 .45783 .3895 .46157 .39546 .45685 .39195 .4531 Metetra
.785 .76 .83 r
.39195 .4531 .39546 .45685 .40142 .45206 .39793 .44829 Metetra
.787 .761 .829 r
.39793 .44829 .40142 .45206 .40739 .44719 .40391 .44342 Metetra
.789 .761 .827 r
.40391 .44342 .40739 .44719 .41337 .44226 .4099 .43846 Metetra
.791 .761 .825 r
.4099 .43846 .41337 .44226 .41935 .43723 .41589 .43344 Metetra
.793 .761 .824 r
.41589 .43344 .41935 .43723 .42533 .43214 .42188 .42835 Metetra
.795 .762 .823 r
.42188 .42835 .42533 .43214 .43131 .42699 .42788 .42319 Metetra
.797 .762 .821 r
.42788 .42319 .43131 .42699 .4373 .42176 .43388 .41797 Metetra
.799 .763 .82 r
.43388 .41797 .4373 .42176 .4433 .41647 .43988 .41268 Metetra
.801 .764 .82 r
.43988 .41268 .4433 .41647 .44929 .41112 .44589 .40734 Metetra
.802 .765 .819 r
.44589 .40734 .44929 .41112 .45529 .40572 .45189 .40196 Metetra
.804 .766 .819 r
.45189 .40196 .45529 .40572 .46128 .40029 .4579 .39653 Metetra
.805 .768 .819 r
.4579 .39653 .46128 .40029 .46728 .39481 .46391 .39108 Metetra
.806 .769 .819 r
.46391 .39108 .46728 .39481 .47328 .38932 .46992 .3856 Metetra
.807 .77 .819 r
.46992 .3856 .47328 .38932 .47928 .38382 .47593 .38012 Metetra
.719 .243 0 r
.47593 .38012 .47928 .38382 .4851 .39256 .48175 .38886 Metetra
.719 .243 0 r
.48175 .38886 .4851 .39256 .49097 .40138 .48762 .39767 Metetra
.718 .242 0 r
.48762 .39767 .49097 .40138 .4969 .41027 .49354 .40656 Metetra
.717 .242 0 r
.49354 .40656 .4969 .41027 .50288 .41923 .49952 .41552 Metetra
.717 .241 0 r
.49952 .41552 .50288 .41923 .50891 .42827 .50555 .42456 Metetra
.717 .241 0 r
.50555 .42456 .50891 .42827 .515 .43738 .51164 .43367 Metetra
.716 .241 0 r
.51164 .43367 .515 .43738 .52115 .44657 .51778 .44287 Metetra
.716 .241 0 r
.51778 .44287 .52115 .44657 .52735 .45583 .52398 .45214 Metetra
.716 .241 0 r
.52398 .45214 .52735 .45583 .53361 .46518 .53023 .46149 Metetra
.717 .241 0 r
.53023 .46149 .53361 .46518 .53992 .47461 .53655 .47093 Metetra
.717 .241 0 r
.53655 .47093 .53992 .47461 .5463 .48413 .54292 .48046 Metetra
.718 .241 0 r
.54292 .48046 .5463 .48413 .55273 .49374 .54935 .49008 Metetra
.718 .242 0 r
.54935 .49008 .55273 .49374 .55922 .50345 .55584 .49979 Metetra
.719 .243 0 r
.55584 .49979 .55922 .50345 .56578 .51326 .5624 .50961 Metetra
.719 .243 0 r
.5624 .50961 .56578 .51326 .5724 .52317 .56901 .51952 Metetra
.719 .243 0 r
.56901 .51952 .5724 .52317 .57908 .53318 .57569 .52954 Metetra
.72 .244 0 r
.57569 .52954 .57908 .53318 .58582 .5433 .58244 .53965 Metetra
.72 .244 0 r
.58244 .53965 .58582 .5433 .59263 .55351 .58924 .54987 Metetra
.719 .243 0 r
.58924 .54987 .59263 .55351 .59951 .56383 .59612 .56019 Metetra
.82 .802 .835 r
.59612 .56019 .59951 .56383 .60606 .55783 .60268 .55421 Metetra
.82 .804 .836 r
.60268 .55421 .60606 .55783 .6126 .55183 .60924 .54823 Metetra
.82 .805 .837 r
.60924 .54823 .6126 .55183 .61915 .54586 .61581 .54227 Metetra
.82 .807 .839 r
.61581 .54227 .61915 .54586 .6257 .53993 .62237 .53636 Metetra
.819 .808 .84 r
.62237 .53636 .6257 .53993 .63226 .53404 .62894 .53048 Metetra
.818 .809 .842 r
.62894 .53048 .63226 .53404 .63881 .5282 .63551 .52465 Metetra
.817 .81 .844 r
.63551 .52465 .63881 .5282 .64537 .52243 .64208 .51888 Metetra
.815 .811 .846 r
.64208 .51888 .64537 .52243 .65193 .51673 .64866 .51319 Metetra
.814 .812 .849 r
.64866 .51319 .65193 .51673 .65849 .51111 .65524 .50757 Metetra
.812 .812 .851 r
.65524 .50757 .65849 .51111 .66506 .50555 .66182 .50201 Metetra
.81 .813 .853 r
.66182 .50201 .66506 .50555 .67164 .50008 .66841 .49652 Metetra
.807 .812 .855 r
.66841 .49652 .67164 .50008 .67822 .49469 .67501 .49112 Metetra
.805 .812 .857 r
.67501 .49112 .67822 .49469 .68481 .48937 .68161 .4858 Metetra
.802 .812 .859 r
.68161 .4858 .68481 .48937 .6914 .48414 .68822 .48054 Metetra
.8 .812 .861 r
.68822 .48054 .6914 .48414 .698 .47898 .69483 .47536 Metetra
.797 .81 .863 r
.69483 .47536 .698 .47898 .70461 .47388 .70146 .47026 Metetra
.795 .81 .864 r
.70146 .47026 .70461 .47388 .71123 .46887 .70809 .46522 Metetra
.792 .809 .866 r
.70809 .46522 .71123 .46887 .71785 .46392 .71473 .46025 Metetra
.789 .808 .868 r
.71473 .46025 .71785 .46392 .72448 .45905 .72137 .45535 Metetra
.787 .807 .869 r
.72137 .45535 .72448 .45905 .73113 .45422 .72803 .45051 Metetra
.785 .807 .87 r
.72803 .45051 .73113 .45422 .73778 .44946 .73469 .44572 Metetra
.782 .805 .871 r
.73469 .44572 .73778 .44946 .74444 .44474 .74136 .44099 Metetra
.78 .805 .873 r
.74136 .44099 .74444 .44474 .7511 .44009 .74805 .43631 Metetra
.778 .804 .874 r
.74805 .43631 .7511 .44009 .75778 .43548 .75474 .43168 Metetra
.776 .802 .874 r
.75474 .43168 .75778 .43548 .76447 .43092 .76144 .4271 Metetra
.774 .801 .875 r
.76144 .4271 .76447 .43092 .77117 .4264 .76815 .42255 Metetra
.772 .801 .876 r
.76815 .42255 .77117 .4264 .77787 .4219 .77487 .41804 Metetra
.771 .799 .876 r
.77487 .41804 .77787 .4219 .78459 .41745 .7816 .41356 Metetra
.769 .799 .877 r
.7816 .41356 .78459 .41745 .79132 .41302 .78833 .40912 Metetra
.768 .798 .878 r
.78833 .40912 .79132 .41302 .79805 .40863 .79508 .4047 Metetra
.766 .796 .878 r
.79508 .4047 .79805 .40863 .8048 .40426 .80184 .40032 Metetra
.765 .796 .878 r
.80184 .40032 .8048 .40426 .81155 .39993 .80861 .39595 Metetra
.764 .795 .878 r
.80861 .39595 .81155 .39993 .81831 .39559 .81538 .3916 Metetra
.763 .795 .879 r
.81538 .3916 .81831 .39559 .82508 .39127 .82217 .38726 Metetra
.762 .794 .879 r
.82217 .38726 .82508 .39127 .83187 .38697 .82896 .38293 Metetra
.762 .793 .879 r
.82896 .38293 .83187 .38697 .83865 .38266 .83576 .37861 Metetra
.761 .793 .879 r
.83576 .37861 .83865 .38266 .84545 .37837 .84257 .3743 Metetra
.761 .792 .878 r
.84257 .3743 .84545 .37837 .85226 .37408 .84939 .36999 Metetra
.76 .791 .878 r
.84939 .36999 .85226 .37408 .85907 .36979 .85622 .36568 Metetra
.76 .791 .878 r
.85622 .36568 .85907 .36979 .8659 .36551 .86305 .36137 Metetra
.758 .768 .861 r
.23516 .55651 .23895 .55982 .24475 .55607 .24097 .55274 Metetra
.758 .768 .861 r
.24097 .55274 .24475 .55607 .25056 .55231 .24678 .54896 Metetra
.758 .767 .861 r
.24678 .54896 .25056 .55231 .25637 .54854 .25261 .54518 Metetra
.758 .767 .86 r
.25261 .54518 .25637 .54854 .26219 .54476 .25843 .54138 Metetra
.758 .767 .86 r
.25843 .54138 .26219 .54476 .26801 .54096 .26427 .53757 Metetra
.759 .767 .859 r
.26427 .53757 .26801 .54096 .27384 .53715 .27011 .53374 Metetra
.759 .766 .859 r
.27011 .53374 .27384 .53715 .27968 .53331 .27596 .5299 Metetra
.76 .766 .858 r
.27596 .5299 .27968 .53331 .28552 .52946 .28181 .52604 Metetra
.761 .766 .857 r
.28181 .52604 .28552 .52946 .29138 .52559 .28767 .52215 Metetra
.761 .766 .856 r
.28767 .52215 .29138 .52559 .29723 .52169 .29354 .51824 Metetra
.762 .766 .856 r
.29354 .51824 .29723 .52169 .3031 .51776 .29942 .51429 Metetra
.763 .765 .854 r
.29942 .51429 .3031 .51776 .30897 .51379 .3053 .5103 Metetra
.764 .765 .853 r
.3053 .5103 .30897 .51379 .31485 .50978 .31119 .50628 Metetra
.766 .765 .852 r
.31119 .50628 .31485 .50978 .32074 .50573 .31709 .50221 Metetra
.767 .765 .851 r
.31709 .50221 .32074 .50573 .32664 .50165 .323 .49811 Metetra
.768 .765 .85 r
.323 .49811 .32664 .50165 .33254 .49752 .32891 .49397 Metetra
.769 .764 .848 r
.32891 .49397 .33254 .49752 .33845 .49334 .33483 .48977 Metetra
.771 .764 .847 r
.33483 .48977 .33845 .49334 .34437 .48911 .34076 .48553 Metetra
.772 .764 .845 r
.34076 .48553 .34437 .48911 .35029 .48482 .3467 .48123 Metetra
.774 .764 .844 r
.3467 .48123 .35029 .48482 .35623 .48048 .35264 .47686 Metetra
.776 .764 .842 r
.35264 .47686 .35623 .48048 .36216 .47608 .35859 .47245 Metetra
.777 .764 .841 r
.35859 .47245 .36216 .47608 .36811 .47162 .36454 .46797 Metetra
.779 .764 .839 r
.36454 .46797 .36811 .47162 .37406 .46709 .37051 .46344 Metetra
.781 .764 .837 r
.37051 .46344 .37406 .46709 .38002 .4625 .37648 .45882 Metetra
.783 .764 .836 r
.37648 .45882 .38002 .4625 .38598 .45783 .38245 .45415 Metetra
.785 .764 .834 r
.38245 .45415 .38598 .45783 .39195 .4531 .38843 .4494 Metetra
.787 .764 .832 r
.38843 .4494 .39195 .4531 .39793 .44829 .39442 .44458 Metetra
.789 .764 .831 r
.39442 .44458 .39793 .44829 .40391 .44342 .40041 .43969 Metetra
.791 .764 .829 r
.40041 .43969 .40391 .44342 .4099 .43846 .40641 .43474 Metetra
.793 .765 .827 r
.40641 .43474 .4099 .43846 .41589 .43344 .41241 .4297 Metetra
.794 .765 .826 r
.41241 .4297 .41589 .43344 .42188 .42835 .41841 .4246 Metetra
.796 .765 .824 r
.41841 .4246 .42188 .42835 .42788 .42319 .42442 .41944 Metetra
.798 .765 .823 r
.42442 .41944 .42788 .42319 .43388 .41797 .43044 .41422 Metetra
.8 .766 .822 r
.43044 .41422 .43388 .41797 .43988 .41268 .43645 .40893 Metetra
.801 .767 .821 r
.43645 .40893 .43988 .41268 .44589 .40734 .44247 .40359 Metetra
.803 .767 .821 r
.44247 .40359 .44589 .40734 .45189 .40196 .44848 .39821 Metetra
.804 .768 .82 r
.44848 .39821 .45189 .40196 .4579 .39653 .4545 .3928 Metetra
.805 .769 .82 r
.4545 .3928 .4579 .39653 .46391 .39108 .46053 .38735 Metetra
.806 .77 .82 r
.46053 .38735 .46391 .39108 .46992 .3856 .46655 .38188 Metetra
.806 .771 .82 r
.46655 .38188 .46992 .3856 .47593 .38012 .47257 .3764 Metetra
.719 .243 0 r
.47257 .3764 .47593 .38012 .48175 .38886 .47838 .38514 Metetra
.718 .243 0 r
.47838 .38514 .48175 .38886 .48762 .39767 .48425 .39395 Metetra
.718 .242 0 r
.48425 .39395 .48762 .39767 .49354 .40656 .49017 .40284 Metetra
.718 .242 0 r
.49017 .40284 .49354 .40656 .49952 .41552 .49615 .4118 Metetra
.717 .242 0 r
.49615 .4118 .49952 .41552 .50555 .42456 .50218 .42084 Metetra
.717 .241 0 r
.50218 .42084 .50555 .42456 .51164 .43367 .50826 .42996 Metetra
.717 .241 0 r
.50826 .42996 .51164 .43367 .51778 .44287 .5144 .43916 Metetra
.717 .241 0 r
.5144 .43916 .51778 .44287 .52398 .45214 .52059 .44843 Metetra
.717 .241 0 r
.52059 .44843 .52398 .45214 .53023 .46149 .52685 .45779 Metetra
.717 .241 0 r
.52685 .45779 .53023 .46149 .53655 .47093 .53316 .46724 Metetra
.717 .241 0 r
.53316 .46724 .53655 .47093 .54292 .48046 .53953 .47677 Metetra
.718 .242 0 r
.53953 .47677 .54292 .48046 .54935 .49008 .54596 .4864 Metetra
.718 .242 0 r
.54596 .4864 .54935 .49008 .55584 .49979 .55245 .49612 Metetra
.718 .242 0 r
.55245 .49612 .55584 .49979 .5624 .50961 .559 .50594 Metetra
.719 .243 0 r
.559 .50594 .5624 .50961 .56901 .51952 .56561 .51585 Metetra
.719 .243 0 r
.56561 .51585 .56901 .51952 .57569 .52954 .57229 .52587 Metetra
.719 .243 0 r
.57229 .52587 .57569 .52954 .58244 .53965 .57903 .53599 Metetra
.719 .243 0 r
.57903 .53599 .58244 .53965 .58924 .54987 .58584 .54621 Metetra
.719 .243 0 r
.58584 .54621 .58924 .54987 .59612 .56019 .59271 .55653 Metetra
.819 .802 .835 r
.59271 .55653 .59612 .56019 .60268 .55421 .59929 .55056 Metetra
.819 .803 .836 r
.59929 .55056 .60268 .55421 .60924 .54823 .60587 .54459 Metetra
.819 .804 .837 r
.60587 .54459 .60924 .54823 .61581 .54227 .61245 .53864 Metetra
.819 .805 .838 r
.61245 .53864 .61581 .54227 .62237 .53636 .61903 .53273 Metetra
.818 .806 .84 r
.61903 .53273 .62237 .53636 .62894 .53048 .62561 .52686 Metetra
.817 .807 .841 r
.62561 .52686 .62894 .53048 .63551 .52465 .6322 .52103 Metetra
.816 .807 .843 r
.6322 .52103 .63551 .52465 .64208 .51888 .63878 .51526 Metetra
.814 .808 .845 r
.63878 .51526 .64208 .51888 .64866 .51319 .64537 .50956 Metetra
.812 .808 .847 r
.64537 .50956 .64866 .51319 .65524 .50757 .65197 .50393 Metetra
.81 .808 .849 r
.65197 .50393 .65524 .50757 .66182 .50201 .65857 .49836 Metetra
.808 .808 .851 r
.65857 .49836 .66182 .50201 .66841 .49652 .66517 .49288 Metetra
.806 .808 .853 r
.66517 .49288 .66841 .49652 .67501 .49112 .67178 .48746 Metetra
.803 .808 .855 r
.67178 .48746 .67501 .49112 .68161 .4858 .6784 .48212 Metetra
.801 .807 .857 r
.6784 .48212 .68161 .4858 .68822 .48054 .68502 .47685 Metetra
.798 .807 .859 r
.68502 .47685 .68822 .48054 .69483 .47536 .69165 .47166 Metetra
.796 .807 .86 r
.69165 .47166 .69483 .47536 .70146 .47026 .69829 .46653 Metetra
.793 .806 .862 r
.69829 .46653 .70146 .47026 .70809 .46522 .70493 .46148 Metetra
.791 .805 .864 r
.70493 .46148 .70809 .46522 .71473 .46025 .71158 .4565 Metetra
.788 .805 .866 r
.71158 .4565 .71473 .46025 .72137 .45535 .71824 .45158 Metetra
.786 .804 .867 r
.71824 .45158 .72137 .45535 .72803 .45051 .72491 .44672 Metetra
.784 .803 .868 r
.72491 .44672 .72803 .45051 .73469 .44572 .73159 .44191 Metetra
.782 .802 .869 r
.73159 .44191 .73469 .44572 .74136 .44099 .73827 .43716 Metetra
.779 .801 .87 r
.73827 .43716 .74136 .44099 .74805 .43631 .74497 .43246 Metetra
.777 .8 .872 r
.74497 .43246 .74805 .43631 .75474 .43168 .75167 .42782 Metetra
.775 .8 .873 r
.75167 .42782 .75474 .43168 .76144 .4271 .75839 .42321 Metetra
.774 .799 .873 r
.75839 .42321 .76144 .4271 .76815 .42255 .76511 .41865 Metetra
.772 .798 .874 r
.76511 .41865 .76815 .42255 .77487 .41804 .77184 .41412 Metetra
.771 .798 .875 r
.77184 .41412 .77487 .41804 .7816 .41356 .77859 .40962 Metetra
.769 .796 .876 r
.77859 .40962 .7816 .41356 .78833 .40912 .78534 .40515 Metetra
.767 .796 .876 r
.78534 .40515 .78833 .40912 .79508 .4047 .7921 .40073 Metetra
.766 .795 .877 r
.7921 .40073 .79508 .4047 .80184 .40032 .79887 .39632 Metetra
.764 .794 .877 r
.79887 .39632 .80184 .40032 .80861 .39595 .80565 .39194 Metetra
.764 .794 .878 r
.80565 .39194 .80861 .39595 .81538 .3916 .81244 .38757 Metetra
.763 .793 .878 r
.81244 .38757 .81538 .3916 .82217 .38726 .81923 .38321 Metetra
.762 .793 .878 r
.81923 .38321 .82217 .38726 .82896 .38293 .82604 .37887 Metetra
.762 .793 .878 r
.82604 .37887 .82896 .38293 .83576 .37861 .83286 .37452 Metetra
.761 .792 .878 r
.83286 .37452 .83576 .37861 .84257 .3743 .83968 .3702 Metetra
.761 .792 .878 r
.83968 .3702 .84257 .3743 .84939 .36999 .84651 .36587 Metetra
.76 .791 .878 r
.84651 .36587 .84939 .36999 .85622 .36568 .85335 .36154 Metetra
.76 .791 .878 r
.85335 .36154 .85622 .36568 .86305 .36137 .8602 .35722 Metetra
.758 .768 .861 r
.23135 .55318 .23516 .55651 .24097 .55274 .23717 .5494 Metetra
.758 .768 .861 r
.23717 .5494 .24097 .55274 .24678 .54896 .24299 .54562 Metetra
.758 .768 .861 r
.24299 .54562 .24678 .54896 .25261 .54518 .24883 .54182 Metetra
.758 .767 .86 r
.24883 .54182 .25261 .54518 .25843 .54138 .25466 .53802 Metetra
.759 .768 .86 r
.25466 .53802 .25843 .54138 .26427 .53757 .26051 .53419 Metetra
.759 .767 .86 r
.26051 .53419 .26427 .53757 .27011 .53374 .26636 .53035 Metetra
.76 .767 .859 r
.26636 .53035 .27011 .53374 .27596 .5299 .27222 .52649 Metetra
.76 .767 .858 r
.27222 .52649 .27596 .5299 .28181 .52604 .27808 .52261 Metetra
.761 .767 .858 r
.27808 .52261 .28181 .52604 .28767 .52215 .28395 .51872 Metetra
.762 .767 .857 r
.28395 .51872 .28767 .52215 .29354 .51824 .28983 .51479 Metetra
.763 .767 .856 r
.28983 .51479 .29354 .51824 .29942 .51429 .29572 .51083 Metetra
.764 .767 .855 r
.29572 .51083 .29942 .51429 .3053 .5103 .30161 .50683 Metetra
.765 .767 .854 r
.30161 .50683 .3053 .5103 .31119 .50628 .30752 .5028 Metetra
.766 .767 .853 r
.30752 .5028 .31119 .50628 .31709 .50221 .31342 .49872 Metetra
.767 .767 .852 r
.31342 .49872 .31709 .50221 .323 .49811 .31934 .4946 Metetra
.769 .767 .851 r
.31934 .4946 .323 .49811 .32891 .49397 .32527 .49044 Metetra
.77 .767 .85 r
.32527 .49044 .32891 .49397 .33483 .48977 .3312 .48624 Metetra
.772 .767 .848 r
.3312 .48624 .33483 .48977 .34076 .48553 .33714 .48198 Metetra
.773 .767 .847 r
.33714 .48198 .34076 .48553 .3467 .48123 .34308 .47766 Metetra
.775 .767 .845 r
.34308 .47766 .3467 .48123 .35264 .47686 .34904 .47329 Metetra
.777 .767 .844 r
.34904 .47329 .35264 .47686 .35859 .47245 .355 .46886 Metetra
.778 .767 .842 r
.355 .46886 .35859 .47245 .36454 .46797 .36096 .46437 Metetra
.78 .767 .841 r
.36096 .46437 .36454 .46797 .37051 .46344 .36694 .45983 Metetra
.782 .767 .839 r
.36694 .45983 .37051 .46344 .37648 .45882 .37292 .4552 Metetra
.784 .767 .837 r
.37292 .4552 .37648 .45882 .38245 .45415 .3789 .45051 Metetra
.786 .767 .836 r
.3789 .45051 .38245 .45415 .38843 .4494 .3849 .44575 Metetra
.788 .767 .834 r
.3849 .44575 .38843 .4494 .39442 .44458 .39089 .44093 Metetra
.79 .767 .833 r
.39089 .44093 .39442 .44458 .40041 .43969 .3969 .43603 Metetra
.792 .768 .831 r
.3969 .43603 .40041 .43969 .40641 .43474 .40291 .43106 Metetra
.794 .767 .829 r
.40291 .43106 .40641 .43474 .41241 .4297 .40892 .42602 Metetra
.796 .768 .828 r
.40892 .42602 .41241 .4297 .41841 .4246 .41493 .42091 Metetra
.797 .768 .826 r
.41493 .42091 .41841 .4246 .42442 .41944 .42095 .41574 Metetra
.799 .768 .825 r
.42095 .41574 .42442 .41944 .43044 .41422 .42698 .41051 Metetra
.801 .769 .823 r
.42698 .41051 .43044 .41422 .43645 .40893 .433 .40522 Metetra
.802 .769 .823 r
.433 .40522 .43645 .40893 .44247 .40359 .43903 .39987 Metetra
.803 .77 .822 r
.43903 .39987 .44247 .40359 .44848 .39821 .44506 .39449 Metetra
.804 .77 .821 r
.44506 .39449 .44848 .39821 .4545 .3928 .45109 .38907 Metetra
.805 .77 .821 r
.45109 .38907 .4545 .3928 .46053 .38735 .45713 .38362 Metetra
.806 .771 .82 r
.45713 .38362 .46053 .38735 .46655 .38188 .46316 .37815 Metetra
.806 .771 .821 r
.46316 .37815 .46655 .38188 .47257 .3764 .46919 .37267 Metetra
.719 .243 0 r
.46919 .37267 .47257 .3764 .47838 .38514 .47501 .38141 Metetra
.719 .243 0 r
.47501 .38141 .47838 .38514 .48425 .39395 .48087 .39022 Metetra
.718 .243 0 r
.48087 .39022 .48425 .39395 .49017 .40284 .48679 .39911 Metetra
.718 .242 0 r
.48679 .39911 .49017 .40284 .49615 .4118 .49276 .40807 Metetra
.718 .242 0 r
.49276 .40807 .49615 .4118 .50218 .42084 .49878 .41712 Metetra
.717 .242 0 r
.49878 .41712 .50218 .42084 .50826 .42996 .50487 .42623 Metetra
.717 .241 0 r
.50487 .42623 .50826 .42996 .5144 .43916 .511 .43543 Metetra
.717 .241 0 r
.511 .43543 .5144 .43916 .52059 .44843 .5172 .44471 Metetra
.717 .241 0 r
.5172 .44471 .52059 .44843 .52685 .45779 .52345 .45408 Metetra
.717 .241 0 r
.52345 .45408 .52685 .45779 .53316 .46724 .52976 .46353 Metetra
.717 .241 0 r
.52976 .46353 .53316 .46724 .53953 .47677 .53612 .47306 Metetra
.717 .241 0 r
.53612 .47306 .53953 .47677 .54596 .4864 .54255 .48269 Metetra
.718 .242 0 r
.54255 .48269 .54596 .4864 .55245 .49612 .54904 .49242 Metetra
.718 .242 0 r
.54904 .49242 .55245 .49612 .559 .50594 .55559 .50224 Metetra
.718 .242 0 r
.55559 .50224 .559 .50594 .56561 .51585 .5622 .51216 Metetra
.719 .243 0 r
.5622 .51216 .56561 .51585 .57229 .52587 .56888 .52219 Metetra
.719 .243 0 r
.56888 .52219 .57229 .52587 .57903 .53599 .57562 .53231 Metetra
.719 .243 0 r
.57562 .53231 .57903 .53599 .58584 .54621 .58242 .54253 Metetra
.719 .243 0 r
.58242 .54253 .58584 .54621 .59271 .55653 .58929 .55286 Metetra
.819 .802 .836 r
.58929 .55286 .59271 .55653 .59929 .55056 .59589 .54688 Metetra
.819 .802 .836 r
.59589 .54688 .59929 .55056 .60587 .54459 .60248 .54091 Metetra
.819 .803 .837 r
.60248 .54091 .60587 .54459 .61245 .53864 .60907 .53495 Metetra
.818 .803 .838 r
.60907 .53495 .61245 .53864 .61903 .53273 .61567 .52904 Metetra
.817 .804 .839 r
.61567 .52904 .61903 .53273 .62561 .52686 .62227 .52316 Metetra
.816 .804 .84 r
.62227 .52316 .62561 .52686 .6322 .52103 .62886 .51733 Metetra
.814 .804 .842 r
.62886 .51733 .6322 .52103 .63878 .51526 .63547 .51155 Metetra
.812 .804 .844 r
.63547 .51155 .63878 .51526 .64537 .50956 .64207 .50585 Metetra
.811 .804 .845 r
.64207 .50585 .64537 .50956 .65197 .50393 .64868 .50021 Metetra
.809 .804 .847 r
.64868 .50021 .65197 .50393 .65857 .49836 .65529 .49463 Metetra
.807 .804 .849 r
.65529 .49463 .65857 .49836 .66517 .49288 .66191 .48913 Metetra
.804 .804 .851 r
.66191 .48913 .66517 .49288 .67178 .48746 .66854 .4837 Metetra
.802 .804 .853 r
.66854 .4837 .67178 .48746 .6784 .48212 .67517 .47835 Metetra
.799 .803 .855 r
.67517 .47835 .6784 .48212 .68502 .47685 .6818 .47307 Metetra
.797 .803 .857 r
.6818 .47307 .68502 .47685 .69165 .47166 .68845 .46786 Metetra
.795 .802 .858 r
.68845 .46786 .69165 .47166 .69829 .46653 .6951 .46273 Metetra
.792 .802 .86 r
.6951 .46273 .69829 .46653 .70493 .46148 .70175 .45766 Metetra
.79 .801 .862 r
.70175 .45766 .70493 .46148 .71158 .4565 .70842 .45266 Metetra
.787 .801 .863 r
.70842 .45266 .71158 .4565 .71824 .45158 .71509 .44773 Metetra
.785 .8 .865 r
.71509 .44773 .71824 .45158 .72491 .44672 .72178 .44285 Metetra
.783 .8 .866 r
.72178 .44285 .72491 .44672 .73159 .44191 .72847 .43802 Metetra
.781 .799 .867 r
.72847 .43802 .73159 .44191 .73827 .43716 .73517 .43326 Metetra
.779 .798 .869 r
.73517 .43326 .73827 .43716 .74497 .43246 .74188 .42855 Metetra
.777 .798 .87 r
.74188 .42855 .74497 .43246 .75167 .42782 .74859 .42388 Metetra
.775 .797 .871 r
.74859 .42388 .75167 .42782 .75839 .42321 .75532 .41927 Metetra
.773 .796 .872 r
.75532 .41927 .75839 .42321 .76511 .41865 .76206 .41468 Metetra
.772 .796 .873 r
.76206 .41468 .76511 .41865 .77184 .41412 .7688 .41014 Metetra
.77 .795 .874 r
.7688 .41014 .77184 .41412 .77859 .40962 .77556 .40563 Metetra
.768 .795 .875 r
.77556 .40563 .77859 .40962 .78534 .40515 .78232 .40115 Metetra
.767 .794 .875 r
.78232 .40115 .78534 .40515 .7921 .40073 .7891 .3967 Metetra
.765 .793 .876 r
.7891 .3967 .7921 .40073 .79887 .39632 .79588 .39228 Metetra
.765 .793 .876 r
.79588 .39228 .79887 .39632 .80565 .39194 .80268 .38788 Metetra
.764 .792 .877 r
.80268 .38788 .80565 .39194 .81244 .38757 .80948 .3835 Metetra
.763 .792 .877 r
.80948 .3835 .81244 .38757 .81923 .38321 .81629 .37913 Metetra
.762 .792 .877 r
.81629 .37913 .81923 .38321 .82604 .37887 .82311 .37477 Metetra
.762 .791 .877 r
.82311 .37477 .82604 .37887 .83286 .37452 .82994 .37042 Metetra
.761 .792 .878 r
.82994 .37042 .83286 .37452 .83968 .3702 .83677 .36607 Metetra
.761 .791 .878 r
.83677 .36607 .83968 .3702 .84651 .36587 .84362 .36173 Metetra
.761 .791 .878 r
.84362 .36173 .84651 .36587 .85335 .36154 .85047 .35739 Metetra
.76 .791 .878 r
.85047 .35739 .85335 .36154 .8602 .35722 .85733 .35305 Metetra
.758 .768 .861 r
.22753 .54985 .23135 .55318 .23717 .5494 .23335 .54606 Metetra
.758 .768 .861 r
.23335 .54606 .23717 .5494 .24299 .54562 .23919 .54226 Metetra
.758 .768 .861 r
.23919 .54226 .24299 .54562 .24883 .54182 .24503 .53846 Metetra
.759 .768 .861 r
.24503 .53846 .24883 .54182 .25466 .53802 .25088 .53464 Metetra
.759 .768 .86 r
.25088 .53464 .25466 .53802 .26051 .53419 .25673 .53081 Metetra
.76 .768 .86 r
.25673 .53081 .26051 .53419 .26636 .53035 .26259 .52695 Metetra
.76 .768 .86 r
.26259 .52695 .26636 .53035 .27222 .52649 .26846 .52309 Metetra
.761 .768 .859 r
.26846 .52309 .27222 .52649 .27808 .52261 .27434 .5192 Metetra
.761 .768 .859 r
.27434 .5192 .27808 .52261 .28395 .51872 .28022 .51529 Metetra
.762 .768 .858 r
.28022 .51529 .28395 .51872 .28983 .51479 .28611 .51136 Metetra
.763 .768 .857 r
.28611 .51136 .28983 .51479 .29572 .51083 .292 .50739 Metetra
.764 .768 .856 r
.292 .50739 .29572 .51083 .30161 .50683 .29791 .50338 Metetra
.765 .768 .855 r
.29791 .50338 .30161 .50683 .30752 .5028 .30382 .49934 Metetra
.767 .769 .854 r
.30382 .49934 .30752 .5028 .31342 .49872 .30974 .49525 Metetra
.768 .769 .853 r
.30974 .49525 .31342 .49872 .31934 .4946 .31567 .49112 Metetra
.769 .769 .852 r
.31567 .49112 .31934 .4946 .32527 .49044 .3216 .48696 Metetra
.771 .769 .851 r
.3216 .48696 .32527 .49044 .3312 .48624 .32754 .48274 Metetra
.772 .769 .85 r
.32754 .48274 .3312 .48624 .33714 .48198 .33349 .47847 Metetra
.774 .769 .848 r
.33349 .47847 .33714 .48198 .34308 .47766 .33945 .47415 Metetra
.776 .77 .847 r
.33945 .47415 .34308 .47766 .34904 .47329 .34541 .46976 Metetra
.778 .77 .845 r
.34541 .46976 .34904 .47329 .355 .46886 .35139 .46532 Metetra
.779 .77 .844 r
.35139 .46532 .355 .46886 .36096 .46437 .35736 .46082 Metetra
.781 .77 .843 r
.35736 .46082 .36096 .46437 .36694 .45983 .36335 .45626 Metetra
.783 .77 .841 r
.36335 .45626 .36694 .45983 .37292 .4552 .36934 .45163 Metetra
.785 .77 .839 r
.36934 .45163 .37292 .4552 .3789 .45051 .37534 .44693 Metetra
.788 .771 .838 r
.37534 .44693 .3789 .45051 .3849 .44575 .38134 .44216 Metetra
.789 .77 .836 r
.38134 .44216 .3849 .44575 .39089 .44093 .38735 .43732 Metetra
.791 .771 .834 r
.38735 .43732 .39089 .44093 .3969 .43603 .39336 .43241 Metetra
.793 .771 .833 r
.39336 .43241 .3969 .43603 .40291 .43106 .39938 .42744 Metetra
.795 .771 .831 r
.39938 .42744 .40291 .43106 .40892 .42602 .40541 .42238 Metetra
.797 .771 .829 r
.40541 .42238 .40892 .42602 .41493 .42091 .41144 .41726 Metetra
.799 .771 .828 r
.41144 .41726 .41493 .42091 .42095 .41574 .41747 .41208 Metetra
.8 .771 .826 r
.41747 .41208 .42095 .41574 .42698 .41051 .4235 .40685 Metetra
.802 .772 .825 r
.4235 .40685 .42698 .41051 .433 .40522 .42954 .40154 Metetra
.803 .771 .824 r
.42954 .40154 .433 .40522 .43903 .39987 .43558 .39618 Metetra
.804 .772 .823 r
.43558 .39618 .43903 .39987 .44506 .39449 .44163 .39079 Metetra
.805 .772 .822 r
.44163 .39079 .44506 .39449 .45109 .38907 .44767 .38536 Metetra
.806 .772 .821 r
.44767 .38536 .45109 .38907 .45713 .38362 .45371 .3799 Metetra
.807 .772 .821 r
.45371 .3799 .45713 .38362 .46316 .37815 .45976 .37441 Metetra
.807 .772 .821 r
.45976 .37441 .46316 .37815 .46919 .37267 .4658 .36893 Metetra
.719 .243 0 r
.4658 .36893 .46919 .37267 .47501 .38141 .47161 .37766 Metetra
.719 .243 0 r
.47161 .37766 .47501 .38141 .48087 .39022 .47748 .38648 Metetra
.718 .243 0 r
.47748 .38648 .48087 .39022 .48679 .39911 .48339 .39537 Metetra
.718 .242 0 r
.48339 .39537 .48679 .39911 .49276 .40807 .48936 .40433 Metetra
.718 .242 0 r
.48936 .40433 .49276 .40807 .49878 .41712 .49538 .41338 Metetra
.718 .242 0 r
.49538 .41338 .49878 .41712 .50487 .42623 .50146 .4225 Metetra
.718 .242 0 r
.50146 .4225 .50487 .42623 .511 .43543 .50759 .4317 Metetra
.717 .241 0 r
.50759 .4317 .511 .43543 .5172 .44471 .51378 .44098 Metetra
.717 .241 0 r
.51378 .44098 .5172 .44471 .52345 .45408 .52003 .45035 Metetra
.717 .241 0 r
.52003 .45035 .52345 .45408 .52976 .46353 .52634 .4598 Metetra
.717 .241 0 r
.52634 .4598 .52976 .46353 .53612 .47306 .5327 .46934 Metetra
.717 .241 0 r
.5327 .46934 .53612 .47306 .54255 .48269 .53913 .47897 Metetra
.718 .242 0 r
.53913 .47897 .54255 .48269 .54904 .49242 .54561 .4887 Metetra
.718 .242 0 r
.54561 .4887 .54904 .49242 .55559 .50224 .55216 .49853 Metetra
.718 .242 0 r
.55216 .49853 .55559 .50224 .5622 .51216 .55877 .50845 Metetra
.718 .242 0 r
.55877 .50845 .5622 .51216 .56888 .52219 .56545 .51848 Metetra
.718 .243 0 r
.56545 .51848 .56888 .52219 .57562 .53231 .57218 .52861 Metetra
.719 .243 0 r
.57218 .52861 .57562 .53231 .58242 .54253 .57899 .53883 Metetra
.719 .243 0 r
.57899 .53883 .58242 .54253 .58929 .55286 .58586 .54916 Metetra
.819 .802 .835 r
.58586 .54916 .58929 .55286 .59589 .54688 .59246 .54317 Metetra
.819 .802 .836 r
.59246 .54317 .59589 .54688 .60248 .54091 .59907 .53719 Metetra
.818 .802 .836 r
.59907 .53719 .60248 .54091 .60907 .53495 .60568 .53122 Metetra
.817 .802 .837 r
.60568 .53122 .60907 .53495 .61567 .52904 .61229 .52529 Metetra
.816 .802 .838 r
.61229 .52529 .61567 .52904 .62227 .52316 .6189 .51941 Metetra
.815 .801 .839 r
.6189 .51941 .62227 .52316 .62886 .51733 .62552 .51356 Metetra
.813 .801 .84 r
.62552 .51356 .62886 .51733 .63547 .51155 .63213 .50777 Metetra
.811 .801 .842 r
.63213 .50777 .63547 .51155 .64207 .50585 .63875 .50205 Metetra
.809 .801 .844 r
.63875 .50205 .64207 .50585 .64868 .50021 .64538 .4964 Metetra
.807 .801 .845 r
.64538 .4964 .64868 .50021 .65529 .49463 .652 .49081 Metetra
.805 .8 .847 r
.652 .49081 .65529 .49463 .66191 .48913 .65864 .4853 Metetra
.803 .8 .849 r
.65864 .4853 .66191 .48913 .66854 .4837 .66527 .47985 Metetra
.8 .8 .851 r
.66527 .47985 .66854 .4837 .67517 .47835 .67192 .47449 Metetra
.798 .799 .853 r
.67192 .47449 .67517 .47835 .6818 .47307 .67857 .4692 Metetra
.795 .799 .854 r
.67857 .4692 .6818 .47307 .68845 .46786 .68522 .46398 Metetra
.794 .799 .856 r
.68522 .46398 .68845 .46786 .6951 .46273 .69189 .45883 Metetra
.791 .798 .858 r
.69189 .45883 .6951 .46273 .70175 .45766 .69856 .45374 Metetra
.788 .797 .86 r
.69856 .45374 .70175 .45766 .70842 .45266 .70524 .44874 Metetra
.786 .797 .861 r
.70524 .44874 .70842 .45266 .71509 .44773 .71193 .44379 Metetra
.784 .797 .863 r
.71193 .44379 .71509 .44773 .72178 .44285 .71862 .4389 Metetra
.782 .796 .864 r
.71862 .4389 .72178 .44285 .72847 .43802 .72533 .43407 Metetra
.78 .796 .866 r
.72533 .43407 .72847 .43802 .73517 .43326 .73204 .42929 Metetra
.778 .795 .867 r
.73204 .42929 .73517 .43326 .74188 .42855 .73876 .42456 Metetra
.775 .794 .868 r
.73876 .42456 .74188 .42855 .74859 .42388 .7455 .41989 Metetra
.774 .795 .87 r
.7455 .41989 .74859 .42388 .75532 .41927 .75224 .41526 Metetra
.772 .794 .87 r
.75224 .41526 .75532 .41927 .76206 .41468 .75899 .41066 Metetra
.771 .793 .871 r
.75899 .41066 .76206 .41468 .7688 .41014 .76575 .4061 Metetra
.769 .793 .872 r
.76575 .4061 .7688 .41014 .77556 .40563 .77252 .40158 Metetra
.768 .793 .873 r
.77252 .40158 .77556 .40563 .78232 .40115 .77929 .39709 Metetra
.766 .792 .874 r
.77929 .39709 .78232 .40115 .7891 .3967 .78608 .39263 Metetra
.765 .792 .875 r
.78608 .39263 .7891 .3967 .79588 .39228 .79288 .3882 Metetra
.764 .792 .876 r
.79288 .3882 .79588 .39228 .80268 .38788 .79969 .38379 Metetra
.763 .792 .876 r
.79969 .38379 .80268 .38788 .80948 .3835 .8065 .37939 Metetra
.763 .791 .876 r
.8065 .37939 .80948 .3835 .81629 .37913 .81333 .37501 Metetra
.762 .791 .877 r
.81333 .37501 .81629 .37913 .82311 .37477 .82016 .37064 Metetra
.762 .791 .877 r
.82016 .37064 .82311 .37477 .82994 .37042 .827 .36627 Metetra
.761 .79 .877 r
.827 .36627 .82994 .37042 .83677 .36607 .83385 .36192 Metetra
.761 .791 .878 r
.83385 .36192 .83677 .36607 .84362 .36173 .84071 .35756 Metetra
.76 .79 .878 r
.84071 .35756 .84362 .36173 .85047 .35739 .84758 .35321 Metetra
.76 .791 .878 r
.84758 .35321 .85047 .35739 .85733 .35305 .85446 .34886 Metetra
.758 .768 .861 r
.22369 .54649 .22753 .54985 .23335 .54606 .22952 .5427 Metetra
.758 .768 .861 r
.22952 .5427 .23335 .54606 .23919 .54226 .23537 .5389 Metetra
.758 .768 .861 r
.23537 .5389 .23919 .54226 .24503 .53846 .24122 .53508 Metetra
.758 .768 .861 r
.24122 .53508 .24503 .53846 .25088 .53464 .24708 .53126 Metetra
.759 .769 .861 r
.24708 .53126 .25088 .53464 .25673 .53081 .25294 .52742 Metetra
.759 .769 .86 r
.25294 .52742 .25673 .53081 .26259 .52695 .25881 .52356 Metetra
.76 .769 .86 r
.25881 .52356 .26259 .52695 .26846 .52309 .26469 .51969 Metetra
.761 .769 .86 r
.26469 .51969 .26846 .52309 .27434 .5192 .27057 .51579 Metetra
.761 .769 .859 r
.27057 .51579 .27434 .5192 .28022 .51529 .27646 .51188 Metetra
.762 .77 .859 r
.27646 .51188 .28022 .51529 .28611 .51136 .28236 .50794 Metetra
.763 .77 .858 r
.28236 .50794 .28611 .51136 .292 .50739 .28827 .50396 Metetra
.765 .77 .857 r
.28827 .50396 .292 .50739 .29791 .50338 .29419 .49995 Metetra
.766 .77 .856 r
.29419 .49995 .29791 .50338 .30382 .49934 .30011 .4959 Metetra
.767 .77 .855 r
.30011 .4959 .30382 .49934 .30974 .49525 .30604 .4918 Metetra
.769 .771 .855 r
.30604 .4918 .30974 .49525 .31567 .49112 .31198 .48767 Metetra
.77 .771 .854 r
.31198 .48767 .31567 .49112 .3216 .48696 .31792 .48349 Metetra
.771 .771 .853 r
.31792 .48349 .3216 .48696 .32754 .48274 .32387 .47928 Metetra
.773 .772 .851 r
.32387 .47928 .32754 .48274 .33349 .47847 .32983 .475 Metetra
.775 .772 .85 r
.32983 .475 .33349 .47847 .33945 .47415 .3358 .47066 Metetra
.777 .772 .849 r
.3358 .47066 .33945 .47415 .34541 .46976 .34177 .46628 Metetra
.779 .773 .847 r
.34177 .46628 .34541 .46976 .35139 .46532 .34776 .46182 Metetra
.78 .773 .846 r
.34776 .46182 .35139 .46532 .35736 .46082 .35375 .45732 Metetra
.782 .773 .844 r
.35375 .45732 .35736 .46082 .36335 .45626 .35974 .45275 Metetra
.785 .773 .843 r
.35974 .45275 .36335 .45626 .36934 .45163 .36574 .44811 Metetra
.787 .774 .841 r
.36574 .44811 .36934 .45163 .37534 .44693 .37175 .4434 Metetra
.788 .774 .839 r
.37175 .4434 .37534 .44693 .38134 .44216 .37777 .43862 Metetra
.791 .774 .838 r
.37777 .43862 .38134 .44216 .38735 .43732 .38379 .43378 Metetra
.793 .774 .836 r
.38379 .43378 .38735 .43732 .39336 .43241 .38982 .42886 Metetra
.795 .774 .834 r
.38982 .42886 .39336 .43241 .39938 .42744 .39585 .42386 Metetra
.796 .774 .833 r
.39585 .42386 .39938 .42744 .40541 .42238 .40188 .4188 Metetra
.799 .774 .831 r
.40188 .4188 .40541 .42238 .41144 .41726 .40792 .41367 Metetra
.8 .774 .829 r
.40792 .41367 .41144 .41726 .41747 .41208 .41397 .40847 Metetra
.802 .774 .828 r
.41397 .40847 .41747 .41208 .4235 .40685 .42002 .40322 Metetra
.803 .774 .826 r
.42002 .40322 .4235 .40685 .42954 .40154 .42607 .3979 Metetra
.805 .774 .825 r
.42607 .3979 .42954 .40154 .43558 .39618 .43212 .39252 Metetra
.805 .774 .824 r
.43212 .39252 .43558 .39618 .44163 .39079 .43817 .38711 Metetra
.806 .773 .823 r
.43817 .38711 .44163 .39079 .44767 .38536 .44423 .38166 Metetra
.807 .773 .822 r
.44423 .38166 .44767 .38536 .45371 .3799 .45029 .37618 Metetra
.807 .773 .821 r
.45029 .37618 .45371 .3799 .45976 .37441 .45634 .37067 Metetra
.807 .772 .821 r
.45634 .37067 .45976 .37441 .4658 .36893 .4624 .36517 Metetra
.719 .243 0 r
.4624 .36517 .4658 .36893 .47161 .37766 .46821 .37391 Metetra
.719 .243 0 r
.46821 .37391 .47161 .37766 .47748 .38648 .47407 .38272 Metetra
.719 .243 0 r
.47407 .38272 .47748 .38648 .48339 .39537 .47998 .39161 Metetra
.719 .243 0 r
.47998 .39161 .48339 .39537 .48936 .40433 .48594 .40058 Metetra
.718 .242 0 r
.48594 .40058 .48936 .40433 .49538 .41338 .49196 .40963 Metetra
.718 .242 0 r
.49196 .40963 .49538 .41338 .50146 .4225 .49804 .41875 Metetra
.718 .242 0 r
.49804 .41875 .50146 .4225 .50759 .4317 .50417 .42795 Metetra
.717 .241 0 r
.50417 .42795 .50759 .4317 .51378 .44098 .51036 .43724 Metetra
.717 .241 0 r
.51036 .43724 .51378 .44098 .52003 .45035 .5166 .4466 Metetra
.717 .241 0 r
.5166 .4466 .52003 .45035 .52634 .4598 .5229 .45606 Metetra
.717 .241 0 r
.5229 .45606 .52634 .4598 .5327 .46934 .52927 .4656 Metetra
.717 .241 0 r
.52927 .4656 .5327 .46934 .53913 .47897 .53569 .47523 Metetra
.717 .241 0 r
.53569 .47523 .53913 .47897 .54561 .4887 .54217 .48497 Metetra
.717 .242 0 r
.54217 .48497 .54561 .4887 .55216 .49853 .54872 .4948 Metetra
.718 .242 0 r
.54872 .4948 .55216 .49853 .55877 .50845 .55533 .50472 Metetra
.718 .242 0 r
.55533 .50472 .55877 .50845 .56545 .51848 .562 .51475 Metetra
.718 .243 0 r
.562 .51475 .56545 .51848 .57218 .52861 .56874 .52489 Metetra
.718 .243 0 r
.56874 .52489 .57218 .52861 .57899 .53883 .57554 .53512 Metetra
.719 .243 0 r
.57554 .53512 .57899 .53883 .58586 .54916 .58241 .54546 Metetra
.819 .802 .835 r
.58241 .54546 .58586 .54916 .59246 .54317 .58903 .53944 Metetra
.819 .801 .835 r
.58903 .53944 .59246 .54317 .59907 .53719 .59565 .53343 Metetra
.818 .801 .835 r
.59565 .53343 .59907 .53719 .60568 .53122 .60227 .52744 Metetra
.817 .8 .836 r
.60227 .52744 .60568 .53122 .61229 .52529 .6089 .52149 Metetra
.815 .8 .837 r
.6089 .52149 .61229 .52529 .6189 .51941 .61552 .51558 Metetra
.814 .799 .838 r
.61552 .51558 .6189 .51941 .62552 .51356 .62215 .50972 Metetra
.813 .799 .839 r
.62215 .50972 .62552 .51356 .63213 .50777 .62878 .5039 Metetra
.81 .798 .84 r
.62878 .5039 .63213 .50777 .63875 .50205 .63541 .49817 Metetra
.808 .797 .842 r
.63541 .49817 .63875 .50205 .64538 .4964 .64205 .4925 Metetra
.806 .797 .843 r
.64205 .4925 .64538 .4964 .652 .49081 .64869 .4869 Metetra
.804 .797 .845 r
.64869 .4869 .652 .49081 .65864 .4853 .65534 .48136 Metetra
.801 .796 .847 r
.65534 .48136 .65864 .4853 .66527 .47985 .66199 .47591 Metetra
.799 .796 .849 r
.66199 .47591 .66527 .47985 .67192 .47449 .66865 .47053 Metetra
.797 .795 .851 r
.66865 .47053 .67192 .47449 .67857 .4692 .67531 .46523 Metetra
.794 .795 .852 r
.67531 .46523 .67857 .4692 .68522 .46398 .68199 .46 Metetra
.792 .794 .854 r
.68199 .46 .68522 .46398 .69189 .45883 .68866 .45484 Metetra
.79 .794 .856 r
.68866 .45484 .69189 .45883 .69856 .45374 .69535 .44974 Metetra
.787 .794 .858 r
.69535 .44974 .69856 .45374 .70524 .44874 .70204 .44472 Metetra
.785 .793 .859 r
.70204 .44472 .70524 .44874 .71193 .44379 .70874 .43977 Metetra
.783 .793 .861 r
.70874 .43977 .71193 .44379 .71862 .4389 .71545 .43487 Metetra
.781 .793 .862 r
.71545 .43487 .71862 .4389 .72533 .43407 .72217 .43002 Metetra
.779 .792 .864 r
.72217 .43002 .72533 .43407 .73204 .42929 .7289 .42524 Metetra
.777 .792 .866 r
.7289 .42524 .73204 .42929 .73876 .42456 .73564 .42051 Metetra
.775 .792 .867 r
.73564 .42051 .73876 .42456 .7455 .41989 .74238 .41582 Metetra
.773 .791 .868 r
.74238 .41582 .7455 .41989 .75224 .41526 .74914 .41118 Metetra
.772 .791 .869 r
.74914 .41118 .75224 .41526 .75899 .41066 .7559 .40658 Metetra
.77 .791 .87 r
.7559 .40658 .75899 .41066 .76575 .4061 .76267 .40201 Metetra
.768 .791 .871 r
.76267 .40201 .76575 .4061 .77252 .40158 .76946 .39748 Metetra
.767 .791 .872 r
.76946 .39748 .77252 .40158 .77929 .39709 .77625 .39298 Metetra
.766 .791 .873 r
.77625 .39298 .77929 .39709 .78608 .39263 .78305 .38851 Metetra
.764 .79 .874 r
.78305 .38851 .78608 .39263 .79288 .3882 .78986 .38408 Metetra
.764 .79 .875 r
.78986 .38408 .79288 .3882 .79969 .38379 .79668 .37965 Metetra
.763 .79 .875 r
.79668 .37965 .79969 .38379 .8065 .37939 .80351 .37525 Metetra
.762 .79 .876 r
.80351 .37525 .8065 .37939 .81333 .37501 .81035 .37086 Metetra
.762 .79 .876 r
.81035 .37086 .81333 .37501 .82016 .37064 .8172 .36648 Metetra
.761 .79 .877 r
.8172 .36648 .82016 .37064 .827 .36627 .82405 .3621 Metetra
.761 .79 .877 r
.82405 .3621 .827 .36627 .83385 .36192 .83092 .35773 Metetra
.76 .79 .877 r
.83092 .35773 .83385 .36192 .84071 .35756 .83779 .35337 Metetra
.76 .79 .877 r
.83779 .35337 .84071 .35756 .84758 .35321 .84467 .34901 Metetra
.76 .79 .878 r
.84467 .34901 .84758 .35321 .85446 .34886 .85156 .34465 Metetra
.758 .768 .861 r
.21983 .54313 .22369 .54649 .22952 .5427 .22568 .53933 Metetra
.758 .768 .861 r
.22568 .53933 .22952 .5427 .23537 .5389 .23153 .53552 Metetra
.758 .769 .861 r
.23153 .53552 .23537 .5389 .24122 .53508 .23739 .53171 Metetra
.759 .769 .861 r
.23739 .53171 .24122 .53508 .24708 .53126 .24326 .52788 Metetra
.759 .769 .861 r
.24326 .52788 .24708 .53126 .25294 .52742 .24913 .52403 Metetra
.76 .77 .861 r
.24913 .52403 .25294 .52742 .25881 .52356 .25501 .52017 Metetra
.76 .77 .861 r
.25501 .52017 .25881 .52356 .26469 .51969 .2609 .5163 Metetra
.761 .77 .861 r
.2609 .5163 .26469 .51969 .27057 .51579 .26679 .5124 Metetra
.762 .771 .86 r
.26679 .5124 .27057 .51579 .27646 .51188 .2727 .50848 Metetra
.762 .771 .86 r
.2727 .50848 .27646 .51188 .28236 .50794 .2786 .50453 Metetra
.764 .771 .859 r
.2786 .50453 .28236 .50794 .28827 .50396 .28452 .50055 Metetra
.765 .771 .858 r
.28452 .50055 .28827 .50396 .29419 .49995 .29045 .49653 Metetra
.766 .772 .857 r
.29045 .49653 .29419 .49995 .30011 .4959 .29638 .49248 Metetra
.768 .773 .857 r
.29638 .49248 .30011 .4959 .30604 .4918 .30232 .48838 Metetra
.769 .773 .856 r
.30232 .48838 .30604 .4918 .31198 .48767 .30827 .48425 Metetra
.77 .773 .855 r
.30827 .48425 .31198 .48767 .31792 .48349 .31422 .48007 Metetra
.772 .774 .854 r
.31422 .48007 .31792 .48349 .32387 .47928 .32018 .47584 Metetra
.774 .774 .853 r
.32018 .47584 .32387 .47928 .32983 .475 .32615 .47156 Metetra
.776 .774 .852 r
.32615 .47156 .32983 .475 .3358 .47066 .33213 .46723 Metetra
.778 .775 .85 r
.33213 .46723 .3358 .47066 .34177 .46628 .33812 .46283 Metetra
.78 .775 .849 r
.33812 .46283 .34177 .46628 .34776 .46182 .34411 .45838 Metetra
.781 .776 .848 r
.34411 .45838 .34776 .46182 .35375 .45732 .35011 .45387 Metetra
.783 .776 .846 r
.35011 .45387 .35375 .45732 .35974 .45275 .35612 .44929 Metetra
.786 .777 .845 r
.35612 .44929 .35974 .45275 .36574 .44811 .36213 .44464 Metetra
.788 .777 .843 r
.36213 .44464 .36574 .44811 .37175 .4434 .36815 .43993 Metetra
.79 .777 .841 r
.36815 .43993 .37175 .4434 .37777 .43862 .37418 .43514 Metetra
.792 .777 .84 r
.37418 .43514 .37777 .43862 .38379 .43378 .38021 .43028 Metetra
.794 .777 .838 r
.38021 .43028 .38379 .43378 .38982 .42886 .38625 .42535 Metetra
.796 .778 .836 r
.38625 .42535 .38982 .42886 .39585 .42386 .39229 .42035 Metetra
.798 .778 .834 r
.39229 .42035 .39585 .42386 .40188 .4188 .39834 .41527 Metetra
.8 .778 .832 r
.39834 .41527 .40188 .4188 .40792 .41367 .40439 .41012 Metetra
.801 .777 .831 r
.40439 .41012 .40792 .41367 .41397 .40847 .41045 .40491 Metetra
.803 .777 .829 r
.41045 .40491 .41397 .40847 .42002 .40322 .41651 .39964 Metetra
.805 .777 .827 r
.41651 .39964 .42002 .40322 .42607 .3979 .42257 .39429 Metetra
.806 .776 .826 r
.42257 .39429 .42607 .3979 .43212 .39252 .42864 .38889 Metetra
.807 .776 .824 r
.42864 .38889 .43212 .39252 .43817 .38711 .43471 .38345 Metetra
.807 .775 .823 r
.43471 .38345 .43817 .38711 .44423 .38166 .44078 .37798 Metetra
.808 .774 .822 r
.44078 .37798 .44423 .38166 .45029 .37618 .44685 .37246 Metetra
.808 .774 .821 r
.44685 .37246 .45029 .37618 .45634 .37067 .45292 .36693 Metetra
.808 .773 .821 r
.45292 .36693 .45634 .37067 .4624 .36517 .45899 .36139 Metetra
.719 .243 0 r
.45899 .36139 .4624 .36517 .46821 .37391 .46479 .37014 Metetra
.719 .243 0 r
.46479 .37014 .46821 .37391 .47407 .38272 .47064 .37895 Metetra
.719 .243 0 r
.47064 .37895 .47407 .38272 .47998 .39161 .47655 .38785 Metetra
.719 .243 0 r
.47655 .38785 .47998 .39161 .48594 .40058 .48251 .39682 Metetra
.719 .243 0 r
.48251 .39682 .48594 .40058 .49196 .40963 .48853 .40587 Metetra
.718 .242 0 r
.48853 .40587 .49196 .40963 .49804 .41875 .4946 .41499 Metetra
.718 .242 0 r
.4946 .41499 .49804 .41875 .50417 .42795 .50073 .4242 Metetra
.718 .241 0 r
.50073 .4242 .50417 .42795 .51036 .43724 .50691 .43348 Metetra
.717 .241 0 r
.50691 .43348 .51036 .43724 .5166 .4466 .51316 .44284 Metetra
.717 .241 0 r
.51316 .44284 .5166 .4466 .5229 .45606 .51946 .4523 Metetra
.717 .241 0 r
.51946 .4523 .5229 .45606 .52927 .4656 .52582 .46184 Metetra
.717 .241 0 r
.52582 .46184 .52927 .4656 .53569 .47523 .53224 .47147 Metetra
.717 .241 0 r
.53224 .47147 .53569 .47523 .54217 .48497 .53872 .48121 Metetra
.717 .241 0 r
.53872 .48121 .54217 .48497 .54872 .4948 .54526 .49104 Metetra
.717 .242 0 r
.54526 .49104 .54872 .4948 .55533 .50472 .55187 .50097 Metetra
.718 .242 0 r
.55187 .50097 .55533 .50472 .562 .51475 .55854 .51101 Metetra
.718 .242 0 r
.55854 .51101 .562 .51475 .56874 .52489 .56527 .52114 Metetra
.718 .243 0 r
.56527 .52114 .56874 .52489 .57554 .53512 .57207 .53139 Metetra
.719 .243 0 r
.57207 .53139 .57554 .53512 .58241 .54546 .57894 .54173 Metetra
.82 .802 .835 r
.57894 .54173 .58241 .54546 .58903 .53944 .58558 .53568 Metetra
.819 .801 .834 r
.58558 .53568 .58903 .53944 .59565 .53343 .59221 .52963 Metetra
.818 .8 .834 r
.59221 .52963 .59565 .53343 .60227 .52744 .59885 .52361 Metetra
.817 .799 .835 r
.59885 .52361 .60227 .52744 .6089 .52149 .60549 .51763 Metetra
.815 .797 .835 r
.60549 .51763 .6089 .52149 .61552 .51558 .61213 .51169 Metetra
.814 .797 .836 r
.61213 .51169 .61552 .51558 .62215 .50972 .61877 .5058 Metetra
.812 .796 .837 r
.61877 .5058 .62215 .50972 .62878 .5039 .62541 .49996 Metetra
.809 .795 .839 r
.62541 .49996 .62878 .5039 .63541 .49817 .63206 .4942 Metetra
.807 .794 .84 r
.63206 .4942 .63541 .49817 .64205 .4925 .63871 .48851 Metetra
.805 .793 .841 r
.63871 .48851 .64205 .4925 .64869 .4869 .64537 .48289 Metetra
.802 .793 .843 r
.64537 .48289 .64869 .4869 .65534 .48136 .65203 .47734 Metetra
.8 .792 .845 r
.65203 .47734 .65534 .48136 .66199 .47591 .65869 .47187 Metetra
.798 .792 .847 r
.65869 .47187 .66199 .47591 .66865 .47053 .66537 .46648 Metetra
.795 .791 .848 r
.66537 .46648 .66865 .47053 .67531 .46523 .67204 .46117 Metetra
.793 .791 .85 r
.67204 .46117 .67531 .46523 .68199 .46 .67873 .45593 Metetra
.791 .791 .852 r
.67873 .45593 .68199 .46 .68866 .45484 .68542 .45076 Metetra
.788 .79 .854 r
.68542 .45076 .68866 .45484 .69535 .44974 .69212 .44566 Metetra
.786 .79 .856 r
.69212 .44566 .69535 .44974 .70204 .44472 .69883 .44063 Metetra
.783 .79 .858 r
.69883 .44063 .70204 .44472 .70874 .43977 .70554 .43567 Metetra
.782 .79 .859 r
.70554 .43567 .70874 .43977 .71545 .43487 .71227 .43076 Metetra
.779 .789 .861 r
.71227 .43076 .71545 .43487 .72217 .43002 .719 .42591 Metetra
.778 .789 .862 r
.719 .42591 .72217 .43002 .7289 .42524 .72574 .42112 Metetra
.775 .789 .864 r
.72574 .42112 .7289 .42524 .73564 .42051 .73249 .41638 Metetra
.773 .788 .865 r
.73249 .41638 .73564 .42051 .74238 .41582 .73925 .41169 Metetra
.772 .789 .867 r
.73925 .41169 .74238 .41582 .74914 .41118 .74602 .40705 Metetra
.771 .789 .868 r
.74602 .40705 .74914 .41118 .7559 .40658 .7528 .40244 Metetra
.769 .789 .869 r
.7528 .40244 .7559 .40658 .76267 .40201 .75958 .39787 Metetra
.768 .789 .87 r
.75958 .39787 .76267 .40201 .76946 .39748 .76638 .39333 Metetra
.766 .789 .871 r
.76638 .39333 .76946 .39748 .77625 .39298 .77319 .38882 Metetra
.765 .788 .872 r
.77319 .38882 .77625 .39298 .78305 .38851 .78 .38435 Metetra
.764 .789 .873 r
.78 .38435 .78305 .38851 .78986 .38408 .78683 .3799 Metetra
.763 .788 .874 r
.78683 .3799 .78986 .38408 .79668 .37965 .79366 .37548 Metetra
.763 .789 .875 r
.79366 .37548 .79668 .37965 .80351 .37525 .80051 .37107 Metetra
.762 .789 .875 r
.80051 .37107 .80351 .37525 .81035 .37086 .80736 .36667 Metetra
.761 .789 .876 r
.80736 .36667 .81035 .37086 .8172 .36648 .81422 .36228 Metetra
.761 .789 .876 r
.81422 .36228 .8172 .36648 .82405 .3621 .82109 .3579 Metetra
.76 .789 .877 r
.82109 .3579 .82405 .3621 .83092 .35773 .82797 .35353 Metetra
.76 .79 .877 r
.82797 .35353 .83092 .35773 .83779 .35337 .83486 .34916 Metetra
.76 .79 .877 r
.83486 .34916 .83779 .35337 .84467 .34901 .84175 .34479 Metetra
.76 .79 .878 r
.84175 .34479 .84467 .34901 .85156 .34465 .84866 .34042 Metetra
.757 .768 .862 r
.21596 .53975 .21983 .54313 .22568 .53933 .22182 .53595 Metetra
.758 .768 .862 r
.22182 .53595 .22568 .53933 .23153 .53552 .22768 .53214 Metetra
.758 .769 .862 r
.22768 .53214 .23153 .53552 .23739 .53171 .23355 .52832 Metetra
.758 .769 .862 r
.23355 .52832 .23739 .53171 .24326 .52788 .23942 .5245 Metetra
.759 .77 .862 r
.23942 .5245 .24326 .52788 .24913 .52403 .24531 .52065 Metetra
.759 .77 .862 r
.24531 .52065 .24913 .52403 .25501 .52017 .2512 .51679 Metetra
.76 .771 .862 r
.2512 .51679 .25501 .52017 .2609 .5163 .25709 .51291 Metetra
.761 .771 .861 r
.25709 .51291 .2609 .5163 .26679 .5124 .263 .509 Metetra
.761 .771 .861 r
.263 .509 .26679 .5124 .2727 .50848 .26891 .50509 Metetra
.763 .772 .861 r
.26891 .50509 .2727 .50848 .2786 .50453 .27483 .50114 Metetra
.764 .772 .86 r
.27483 .50114 .2786 .50453 .28452 .50055 .28075 .49716 Metetra
.765 .773 .86 r
.28075 .49716 .28452 .50055 .29045 .49653 .28669 .49314 Metetra
.767 .774 .859 r
.28669 .49314 .29045 .49653 .29638 .49248 .29263 .48908 Metetra
.768 .774 .858 r
.29263 .48908 .29638 .49248 .30232 .48838 .29858 .48498 Metetra
.769 .775 .857 r
.29858 .48498 .30232 .48838 .30827 .48425 .30454 .48085 Metetra
.771 .775 .857 r
.30454 .48085 .30827 .48425 .31422 .48007 .3105 .47667 Metetra
.772 .776 .856 r
.3105 .47667 .31422 .48007 .32018 .47584 .31648 .47245 Metetra
.775 .777 .855 r
.31648 .47245 .32018 .47584 .32615 .47156 .32246 .46816 Metetra
.776 .777 .853 r
.32246 .46816 .32615 .47156 .33213 .46723 .32845 .46382 Metetra
.778 .778 .852 r
.32845 .46382 .33213 .46723 .33812 .46283 .33444 .45943 Metetra
.78 .779 .851 r
.33444 .45943 .33812 .46283 .34411 .45838 .34045 .45497 Metetra
.782 .779 .85 r
.34045 .45497 .34411 .45838 .35011 .45387 .34646 .45046 Metetra
.784 .779 .848 r
.34646 .45046 .35011 .45387 .35612 .44929 .35247 .44588 Metetra
.787 .78 .847 r
.35247 .44588 .35612 .44929 .36213 .44464 .3585 .44123 Metetra
.789 .78 .845 r
.3585 .44123 .36213 .44464 .36815 .43993 .36453 .4365 Metetra
.791 .78 .843 r
.36453 .4365 .36815 .43993 .37418 .43514 .37057 .43171 Metetra
.793 .781 .842 r
.37057 .43171 .37418 .43514 .38021 .43028 .37661 .42685 Metetra
.795 .781 .84 r
.37661 .42685 .38021 .43028 .38625 .42535 .38266 .42191 Metetra
.798 .781 .838 r
.38266 .42191 .38625 .42535 .39229 .42035 .38872 .41689 Metetra
.799 .781 .836 r
.38872 .41689 .39229 .42035 .39834 .41527 .39478 .4118 Metetra
.802 .781 .834 r
.39478 .4118 .39834 .41527 .40439 .41012 .40085 .40663 Metetra
.803 .781 .832 r
.40085 .40663 .40439 .41012 .41045 .40491 .40692 .40139 Metetra
.805 .78 .83 r
.40692 .40139 .41045 .40491 .41651 .39964 .41299 .39609 Metetra
.806 .78 .828 r
.41299 .39609 .41651 .39964 .42257 .39429 .41906 .39072 Metetra
.807 .779 .827 r
.41906 .39072 .42257 .39429 .42864 .38889 .42514 .38529 Metetra
.808 .778 .825 r
.42514 .38529 .42864 .38889 .43471 .38345 .43122 .37982 Metetra
.809 .777 .824 r
.43122 .37982 .43471 .38345 .44078 .37798 .43731 .3743 Metetra
.809 .776 .822 r
.43731 .3743 .44078 .37798 .44685 .37246 .44339 .36875 Metetra
.809 .775 .821 r
.44339 .36875 .44685 .37246 .45292 .36693 .44947 .36318 Metetra
.809 .773 .82 r
.44947 .36318 .45292 .36693 .45899 .36139 .45556 .3576 Metetra
.719 .244 0 r
.45556 .3576 .45899 .36139 .46479 .37014 .46135 .36635 Metetra
.72 .244 0 r
.46135 .36635 .46479 .37014 .47064 .37895 .46721 .37517 Metetra
.72 .244 0 r
.46721 .37517 .47064 .37895 .47655 .38785 .47311 .38407 Metetra
.719 .243 0 r
.47311 .38407 .47655 .38785 .48251 .39682 .47907 .39305 Metetra
.719 .243 0 r
.47907 .39305 .48251 .39682 .48853 .40587 .48508 .4021 Metetra
.719 .243 0 r
.48508 .4021 .48853 .40587 .4946 .41499 .49115 .41122 Metetra
.718 .242 0 r
.49115 .41122 .4946 .41499 .50073 .4242 .49728 .42042 Metetra
.718 .241 0 r
.49728 .42042 .50073 .4242 .50691 .43348 .50346 .42971 Metetra
.717 .241 0 r
.50346 .42971 .50691 .43348 .51316 .44284 .5097 .43907 Metetra
.717 .241 0 r
.5097 .43907 .51316 .44284 .51946 .4523 .516 .44852 Metetra
.717 .24 0 r
.516 .44852 .51946 .4523 .52582 .46184 .52235 .45806 Metetra
.716 .24 0 r
.52235 .45806 .52582 .46184 .53224 .47147 .52877 .46769 Metetra
.716 .241 0 r
.52877 .46769 .53224 .47147 .53872 .48121 .53525 .47743 Metetra
.717 .241 0 r
.53525 .47743 .53872 .48121 .54526 .49104 .54179 .48726 Metetra
.717 .241 0 r
.54179 .48726 .54526 .49104 .55187 .50097 .54839 .4972 Metetra
.717 .242 0 r
.54839 .4972 .55187 .50097 .55854 .51101 .55506 .50724 Metetra
.718 .242 0 r
.55506 .50724 .55854 .51101 .56527 .52114 .56179 .51738 Metetra
.718 .243 0 r
.56179 .51738 .56527 .52114 .57207 .53139 .56859 .52763 Metetra
.719 .243 0 r
.56859 .52763 .57207 .53139 .57894 .54173 .57546 .53799 Metetra
.82 .802 .834 r
.57546 .53799 .57894 .54173 .58558 .53568 .58211 .53189 Metetra
.82 .8 .833 r
.58211 .53189 .58558 .53568 .59221 .52963 .58876 .5258 Metetra
.818 .799 .833 r
.58876 .5258 .59221 .52963 .59885 .52361 .59541 .51973 Metetra
.817 .797 .833 r
.59541 .51973 .59885 .52361 .60549 .51763 .60206 .51371 Metetra
.815 .795 .834 r
.60206 .51371 .60549 .51763 .61213 .51169 .60872 .50773 Metetra
.813 .794 .834 r
.60872 .50773 .61213 .51169 .61877 .5058 .61537 .5018 Metetra
.811 .793 .835 r
.61537 .5018 .61877 .5058 .62541 .49996 .62203 .49593 Metetra
.808 .791 .837 r
.62203 .49593 .62541 .49996 .63206 .4942 .62869 .49014 Metetra
.806 .79 .838 r
.62869 .49014 .63206 .4942 .63871 .48851 .63536 .48443 Metetra
.804 .79 .839 r
.63536 .48443 .63871 .48851 .64537 .48289 .64202 .47879 Metetra
.801 .789 .841 r
.64202 .47879 .64537 .48289 .65203 .47734 .6487 .47322 Metetra
.799 .788 .843 r
.6487 .47322 .65203 .47734 .65869 .47187 .65538 .46774 Metetra
.796 .788 .845 r
.65538 .46774 .65869 .47187 .66537 .46648 .66206 .46234 Metetra
.794 .787 .847 r
.66206 .46234 .66537 .46648 .67204 .46117 .66876 .45701 Metetra
.791 .787 .848 r
.66876 .45701 .67204 .46117 .67873 .45593 .67545 .45176 Metetra
.789 .786 .85 r
.67545 .45176 .67873 .45593 .68542 .45076 .68216 .44659 Metetra
.787 .786 .852 r
.68216 .44659 .68542 .45076 .69212 .44566 .68887 .44148 Metetra
.784 .786 .854 r
.68887 .44148 .69212 .44566 .69883 .44063 .69559 .43645 Metetra
.782 .786 .856 r
.69559 .43645 .69883 .44063 .70554 .43567 .70232 .43148 Metetra
.78 .786 .857 r
.70232 .43148 .70554 .43567 .71227 .43076 .70906 .42657 Metetra
.778 .786 .859 r
.70906 .42657 .71227 .43076 .719 .42591 .71581 .42172 Metetra
.776 .786 .861 r
.71581 .42172 .719 .42591 .72574 .42112 .72256 .41693 Metetra
.774 .786 .863 r
.72256 .41693 .72574 .42112 .73249 .41638 .72933 .41219 Metetra
.773 .786 .864 r
.72933 .41219 .73249 .41638 .73925 .41169 .7361 .4075 Metetra
.771 .786 .866 r
.7361 .4075 .73925 .41169 .74602 .40705 .74288 .40285 Metetra
.769 .786 .867 r
.74288 .40285 .74602 .40705 .7528 .40244 .74968 .39824 Metetra
.768 .786 .868 r
.74968 .39824 .7528 .40244 .75958 .39787 .75648 .39366 Metetra
.767 .786 .869 r
.75648 .39366 .75958 .39787 .76638 .39333 .76329 .38912 Metetra
.765 .787 .871 r
.76329 .38912 .76638 .39333 .77319 .38882 .77011 .38462 Metetra
.765 .787 .872 r
.77011 .38462 .77319 .38882 .78 .38435 .77694 .38014 Metetra
.763 .787 .872 r
.77694 .38014 .78 .38435 .78683 .3799 .78378 .37569 Metetra
.762 .787 .873 r
.78378 .37569 .78683 .3799 .79366 .37548 .79063 .37126 Metetra
.762 .787 .874 r
.79063 .37126 .79366 .37548 .80051 .37107 .79749 .36685 Metetra
.761 .788 .875 r
.79749 .36685 .80051 .37107 .80736 .36667 .80435 .36244 Metetra
.761 .788 .875 r
.80435 .36244 .80736 .36667 .81422 .36228 .81123 .35805 Metetra
.76 .788 .876 r
.81123 .35805 .81422 .36228 .82109 .3579 .81811 .35367 Metetra
.76 .789 .877 r
.81811 .35367 .82109 .3579 .82797 .35353 .82501 .34929 Metetra
.759 .789 .877 r
.82501 .34929 .82797 .35353 .83486 .34916 .83191 .34492 Metetra
.76 .789 .877 r
.83191 .34492 .83486 .34916 .84175 .34479 .83882 .34054 Metetra
.759 .79 .878 r
.83882 .34054 .84175 .34479 .84866 .34042 .84574 .33617 Metetra
.757 .768 .862 r
.21207 .53636 .21596 .53975 .22182 .53595 .21794 .53256 Metetra
.757 .769 .862 r
.21794 .53256 .22182 .53595 .22768 .53214 .22381 .52875 Metetra
.757 .769 .862 r
.22381 .52875 .22768 .53214 .23355 .52832 .22969 .52494 Metetra
.758 .77 .862 r
.22969 .52494 .23355 .52832 .23942 .5245 .23557 .5211 Metetra
.759 .77 .862 r
.23557 .5211 .23942 .5245 .24531 .52065 .24146 .51726 Metetra
.759 .771 .862 r
.24146 .51726 .24531 .52065 .2512 .51679 .24736 .5134 Metetra
.76 .771 .862 r
.24736 .5134 .2512 .51679 .25709 .51291 .25327 .50952 Metetra
.761 .772 .862 r
.25327 .50952 .25709 .51291 .263 .509 .25918 .50563 Metetra
.762 .773 .862 r
.25918 .50563 .263 .509 .26891 .50509 .2651 .50171 Metetra
.762 .773 .862 r
.2651 .50171 .26891 .50509 .27483 .50114 .27103 .49776 Metetra
.764 .774 .861 r
.27103 .49776 .27483 .50114 .28075 .49716 .27697 .49378 Metetra
.765 .774 .861 r
.27697 .49378 .28075 .49716 .28669 .49314 .28291 .48976 Metetra
.767 .775 .86 r
.28291 .48976 .28669 .49314 .29263 .48908 .28886 .48571 Metetra
.768 .776 .859 r
.28886 .48571 .29263 .48908 .29858 .48498 .29482 .48161 Metetra
.77 .777 .859 r
.29482 .48161 .29858 .48498 .30454 .48085 .30079 .47748 Metetra
.771 .778 .858 r
.30079 .47748 .30454 .48085 .3105 .47667 .30677 .4733 Metetra
.773 .778 .857 r
.30677 .4733 .3105 .47667 .31648 .47245 .31275 .46908 Metetra
.775 .779 .856 r
.31275 .46908 .31648 .47245 .32246 .46816 .31874 .4648 Metetra
.777 .78 .855 r
.31874 .4648 .32246 .46816 .32845 .46382 .32474 .46047 Metetra
.779 .781 .854 r
.32474 .46047 .32845 .46382 .33444 .45943 .33075 .45607 Metetra
.781 .781 .853 r
.33075 .45607 .33444 .45943 .34045 .45497 .33676 .45161 Metetra
.783 .782 .852 r
.33676 .45161 .34045 .45497 .34646 .45046 .34278 .4471 Metetra
.785 .783 .85 r
.34278 .4471 .34646 .45046 .35247 .44588 .34881 .44252 Metetra
.788 .783 .849 r
.34881 .44252 .35247 .44588 .3585 .44123 .35485 .43787 Metetra
.79 .784 .847 r
.35485 .43787 .3585 .44123 .36453 .4365 .36089 .43314 Metetra
.792 .784 .845 r
.36089 .43314 .36453 .4365 .37057 .43171 .36694 .42834 Metetra
.794 .785 .844 r
.36694 .42834 .37057 .43171 .37661 .42685 .373 .42346 Metetra
.797 .785 .842 r
.373 .42346 .37661 .42685 .38266 .42191 .37906 .41852 Metetra
.799 .785 .84 r
.37906 .41852 .38266 .42191 .38872 .41689 .38513 .41349 Metetra
.801 .785 .838 r
.38513 .41349 .38872 .41689 .39478 .4118 .3912 .40838 Metetra
.803 .785 .836 r
.3912 .40838 .39478 .4118 .40085 .40663 .39728 .40319 Metetra
.805 .784 .834 r
.39728 .40319 .40085 .40663 .40692 .40139 .40336 .39793 Metetra
.806 .784 .832 r
.40336 .39793 .40692 .40139 .41299 .39609 .40945 .3926 Metetra
.808 .783 .829 r
.40945 .3926 .41299 .39609 .41906 .39072 .41554 .38719 Metetra
.809 .782 .827 r
.41554 .38719 .41906 .39072 .42514 .38529 .42163 .38173 Metetra
.81 .781 .826 r
.42163 .38173 .42514 .38529 .43122 .37982 .42773 .37621 Metetra
.81 .779 .824 r
.42773 .37621 .43122 .37982 .43731 .3743 .43382 .37065 Metetra
.811 .778 .822 r
.43382 .37065 .43731 .3743 .44339 .36875 .43992 .36505 Metetra
.811 .776 .82 r
.43992 .36505 .44339 .36875 .44947 .36318 .44602 .35943 Metetra
.81 .774 .819 r
.44602 .35943 .44947 .36318 .45556 .3576 .45211 .3538 Metetra
.72 .244 0 r
.45211 .3538 .45556 .3576 .46135 .36635 .45791 .36255 Metetra
.72 .244 0 r
.45791 .36255 .46135 .36635 .46721 .37517 .46375 .37138 Metetra
.72 .244 0 r
.46375 .37138 .46721 .37517 .47311 .38407 .46966 .38029 Metetra
.72 .244 0 r
.46966 .38029 .47311 .38407 .47907 .39305 .47561 .38927 Metetra
.72 .243 0 r
.47561 .38927 .47907 .39305 .48508 .4021 .48162 .39832 Metetra
.719 .243 0 r
.48162 .39832 .48508 .4021 .49115 .41122 .48769 .40744 Metetra
.719 .242 0 r
.48769 .40744 .49115 .41122 .49728 .42042 .49381 .41664 Metetra
.718 .241 0 r
.49381 .41664 .49728 .42042 .50346 .42971 .49999 .42592 Metetra
.717 .241 0 r
.49999 .42592 .50346 .42971 .5097 .43907 .50622 .43528 Metetra
.716 .24 0 r
.50622 .43528 .5097 .43907 .516 .44852 .51252 .44472 Metetra
.716 .24 0 r
.51252 .44472 .516 .44852 .52235 .45806 .51887 .45426 Metetra
.716 .24 0 r
.51887 .45426 .52235 .45806 .52877 .46769 .52529 .46389 Metetra
.716 .24 0 r
.52529 .46389 .52877 .46769 .53525 .47743 .53177 .47362 Metetra
.716 .24 0 r
.53177 .47362 .53525 .47743 .54179 .48726 .5383 .48346 Metetra
.716 .241 0 r
.5383 .48346 .54179 .48726 .54839 .4972 .54491 .4934 Metetra
.717 .242 0 r
.54491 .4934 .54839 .4972 .55506 .50724 .55157 .50345 Metetra
.718 .242 0 r
.55157 .50345 .55506 .50724 .56179 .51738 .5583 .5136 Metetra
.718 .243 0 r
.5583 .5136 .56179 .51738 .56859 .52763 .5651 .52386 Metetra
.719 .244 0 r
.5651 .52386 .56859 .52763 .57546 .53799 .57196 .53423 Metetra
.822 .802 .833 r
.57196 .53423 .57546 .53799 .58211 .53189 .57863 .52807 Metetra
.821 .8 .832 r
.57863 .52807 .58211 .53189 .58876 .5258 .58529 .52192 Metetra
.819 .797 .831 r
.58529 .52192 .58876 .5258 .59541 .51973 .59195 .5158 Metetra
.817 .795 .832 r
.59195 .5158 .59541 .51973 .60206 .51371 .59862 .50972 Metetra
.815 .793 .832 r
.59862 .50972 .60206 .51371 .60872 .50773 .60529 .50369 Metetra
.813 .791 .832 r
.60529 .50369 .60872 .50773 .61537 .5018 .61196 .49772 Metetra
.81 .789 .833 r
.61196 .49772 .61537 .5018 .62203 .49593 .61863 .49182 Metetra
.808 .788 .834 r
.61863 .49182 .62203 .49593 .62869 .49014 .6253 .48599 Metetra
.805 .787 .836 r
.6253 .48599 .62869 .49014 .63536 .48443 .63198 .48025 Metetra
.803 .786 .837 r
.63198 .48025 .63536 .48443 .64202 .47879 .63866 .47458 Metetra
.8 .785 .839 r
.63866 .47458 .64202 .47879 .6487 .47322 .64535 .469 Metetra
.797 .784 .841 r
.64535 .469 .6487 .47322 .65538 .46774 .65204 .4635 Metetra
.795 .783 .842 r
.65204 .4635 .65538 .46774 .66206 .46234 .65874 .45809 Metetra
.792 .783 .844 r
.65874 .45809 .66206 .46234 .66876 .45701 .66545 .45276 Metetra
.79 .783 .846 r
.66545 .45276 .66876 .45701 .67545 .45176 .67216 .44751 Metetra
.788 .783 .848 r
.67216 .44751 .67545 .45176 .68216 .44659 .67888 .44232 Metetra
.785 .782 .85 r
.67888 .44232 .68216 .44659 .68887 .44148 .68561 .43721 Metetra
.783 .782 .852 r
.68561 .43721 .68887 .44148 .69559 .43645 .69234 .43218 Metetra
.781 .782 .854 r
.69234 .43218 .69559 .43645 .70232 .43148 .69909 .42722 Metetra
.779 .782 .856 r
.69909 .42722 .70232 .43148 .70906 .42657 .70584 .4223 Metetra
.777 .782 .858 r
.70584 .4223 .70906 .42657 .71581 .42172 .7126 .41745 Metetra
.775 .783 .86 r
.7126 .41745 .71581 .42172 .72256 .41693 .71937 .41266 Metetra
.773 .783 .861 r
.71937 .41266 .72256 .41693 .72933 .41219 .72615 .40792 Metetra
.771 .783 .863 r
.72615 .40792 .72933 .41219 .7361 .4075 .73294 .40323 Metetra
.77 .783 .864 r
.73294 .40323 .7361 .4075 .74288 .40285 .73973 .39859 Metetra
.768 .784 .866 r
.73973 .39859 .74288 .40285 .74968 .39824 .74654 .39398 Metetra
.767 .784 .867 r
.74654 .39398 .74968 .39824 .75648 .39366 .75335 .3894 Metetra
.766 .784 .868 r
.75335 .3894 .75648 .39366 .76329 .38912 .76018 .38486 Metetra
.764 .784 .869 r
.76018 .38486 .76329 .38912 .77011 .38462 .76701 .38035 Metetra
.763 .785 .871 r
.76701 .38035 .77011 .38462 .77694 .38014 .77386 .37588 Metetra
.762 .785 .872 r
.77386 .37588 .77694 .38014 .78378 .37569 .78071 .37143 Metetra
.761 .785 .873 r
.78071 .37143 .78378 .37569 .79063 .37126 .78758 .367 Metetra
.761 .786 .874 r
.78758 .367 .79063 .37126 .79749 .36685 .79445 .36259 Metetra
.76 .786 .874 r
.79445 .36259 .79749 .36685 .80435 .36244 .80133 .35818 Metetra
.76 .787 .875 r
.80133 .35818 .80435 .36244 .81123 .35805 .80822 .35379 Metetra
.76 .788 .876 r
.80822 .35379 .81123 .35805 .81811 .35367 .81512 .3494 Metetra
.759 .788 .876 r
.81512 .3494 .81811 .35367 .82501 .34929 .82203 .34503 Metetra
.759 .789 .877 r
.82203 .34503 .82501 .34929 .83191 .34492 .82895 .34065 Metetra
.759 .789 .878 r
.82895 .34065 .83191 .34492 .83882 .34054 .83587 .33628 Metetra
.759 .79 .878 r
.83587 .33628 .83882 .34054 .84574 .33617 .84281 .33191 Metetra
.757 .768 .862 r
.20817 .53295 .21207 .53636 .21794 .53256 .21404 .52915 Metetra
.757 .768 .862 r
.21404 .52915 .21794 .53256 .22381 .52875 .21992 .52535 Metetra
.757 .769 .863 r
.21992 .52535 .22381 .52875 .22969 .52494 .22581 .52154 Metetra
.758 .77 .863 r
.22581 .52154 .22969 .52494 .23557 .5211 .2317 .51772 Metetra
.759 .771 .863 r
.2317 .51772 .23557 .5211 .24146 .51726 .23761 .51387 Metetra
.759 .771 .863 r
.23761 .51387 .24146 .51726 .24736 .5134 .24351 .51002 Metetra
.76 .772 .863 r
.24351 .51002 .24736 .5134 .25327 .50952 .24943 .50614 Metetra
.761 .773 .863 r
.24943 .50614 .25327 .50952 .25918 .50563 .25535 .50225 Metetra
.761 .774 .863 r
.25535 .50225 .25918 .50563 .2651 .50171 .26128 .49834 Metetra
.763 .775 .863 r
.26128 .49834 .2651 .50171 .27103 .49776 .26722 .49439 Metetra
.764 .775 .863 r
.26722 .49439 .27103 .49776 .27697 .49378 .27316 .49042 Metetra
.765 .776 .862 r
.27316 .49042 .27697 .49378 .28291 .48976 .27912 .48641 Metetra
.767 .777 .862 r
.27912 .48641 .28291 .48976 .28886 .48571 .28508 .48236 Metetra
.768 .778 .861 r
.28508 .48236 .28886 .48571 .29482 .48161 .29105 .47827 Metetra
.77 .779 .86 r
.29105 .47827 .29482 .48161 .30079 .47748 .29703 .47414 Metetra
.771 .78 .86 r
.29703 .47414 .30079 .47748 .30677 .4733 .30301 .46996 Metetra
.773 .781 .859 r
.30301 .46996 .30677 .4733 .31275 .46908 .309 .46575 Metetra
.775 .782 .858 r
.309 .46575 .31275 .46908 .31874 .4648 .31501 .46147 Metetra
.777 .783 .857 r
.31501 .46147 .31874 .4648 .32474 .46047 .32102 .45714 Metetra
.779 .783 .856 r
.32102 .45714 .32474 .46047 .33075 .45607 .32703 .45275 Metetra
.782 .784 .855 r
.32703 .45275 .33075 .45607 .33676 .45161 .33306 .4483 Metetra
.784 .785 .854 r
.33306 .4483 .33676 .45161 .34278 .4471 .33909 .44379 Metetra
.786 .786 .852 r
.33909 .44379 .34278 .4471 .34881 .44252 .34513 .43921 Metetra
.788 .787 .851 r
.34513 .43921 .34881 .44252 .35485 .43787 .35118 .43456 Metetra
.791 .787 .849 r
.35118 .43456 .35485 .43787 .36089 .43314 .35724 .42983 Metetra
.793 .788 .848 r
.35724 .42983 .36089 .43314 .36694 .42834 .3633 .42503 Metetra
.796 .789 .846 r
.3633 .42503 .36694 .42834 .373 .42346 .36936 .42015 Metetra
.798 .789 .844 r
.36936 .42015 .373 .42346 .37906 .41852 .37544 .41519 Metetra
.801 .789 .842 r
.37544 .41519 .37906 .41852 .38513 .41349 .38152 .41015 Metetra
.803 .789 .84 r
.38152 .41015 .38513 .41349 .3912 .40838 .38761 .40502 Metetra
.805 .789 .837 r
.38761 .40502 .3912 .40838 .39728 .40319 .3937 .39981 Metetra
.807 .788 .835 r
.3937 .39981 .39728 .40319 .40336 .39793 .39979 .39452 Metetra
.808 .787 .833 r
.39979 .39452 .40336 .39793 .40945 .3926 .40589 .38916 Metetra
.81 .786 .83 r
.40589 .38916 .40945 .3926 .41554 .38719 .412 .38372 Metetra
.811 .785 .828 r
.412 .38372 .41554 .38719 .42163 .38173 .4181 .3782 Metetra
.812 .783 .826 r
.4181 .3782 .42163 .38173 .42773 .37621 .42421 .37264 Metetra
.813 .782 .824 r
.42421 .37264 .42773 .37621 .43382 .37065 .43032 .36702 Metetra
.813 .779 .822 r
.43032 .36702 .43382 .37065 .43992 .36505 .43643 .36136 Metetra
.812 .777 .82 r
.43643 .36136 .43992 .36505 .44602 .35943 .44254 .35567 Metetra
.812 .775 .819 r
.44254 .35567 .44602 .35943 .45211 .3538 .44865 .34997 Metetra
.72 .245 0 r
.44865 .34997 .45211 .3538 .45791 .36255 .45445 .35874 Metetra
.721 .245 0 r
.45445 .35874 .45791 .36255 .46375 .37138 .46029 .36758 Metetra
.721 .245 0 r
.46029 .36758 .46375 .37138 .46966 .38029 .46619 .3765 Metetra
.721 .245 0 r
.46619 .3765 .46966 .38029 .47561 .38927 .47214 .38548 Metetra
.72 .244 0 r
.47214 .38548 .47561 .38927 .48162 .39832 .47814 .39453 Metetra
.72 .243 0 r
.47814 .39453 .48162 .39832 .48769 .40744 .48421 .40366 Metetra
.719 .242 0 r
.48421 .40366 .48769 .40744 .49381 .41664 .49033 .41285 Metetra
.718 .241 0 r
.49033 .41285 .49381 .41664 .49999 .42592 .4965 .42213 Metetra
.717 .24 0 r
.4965 .42213 .49999 .42592 .50622 .43528 .50274 .43148 Metetra
.716 .24 0 r
.50274 .43148 .50622 .43528 .51252 .44472 .50903 .44091 Metetra
.715 .239 0 r
.50903 .44091 .51252 .44472 .51887 .45426 .51538 .45044 Metetra
.715 .239 0 r
.51538 .45044 .51887 .45426 .52529 .46389 .52179 .46006 Metetra
.715 .239 0 r
.52179 .46006 .52529 .46389 .53177 .47362 .52827 .46979 Metetra
.715 .24 0 r
.52827 .46979 .53177 .47362 .5383 .48346 .5348 .47963 Metetra
.716 .241 0 r
.5348 .47963 .5383 .48346 .54491 .4934 .5414 .48957 Metetra
.716 .241 0 r
.5414 .48957 .54491 .4934 .55157 .50345 .54806 .49963 Metetra
.717 .242 0 r
.54806 .49963 .55157 .50345 .5583 .5136 .55479 .50979 Metetra
.718 .243 0 r
.55479 .50979 .5583 .5136 .5651 .52386 .56159 .52007 Metetra
.719 .244 0 r
.56159 .52007 .5651 .52386 .57196 .53423 .56845 .53046 Metetra
.823 .802 .831 r
.56845 .53046 .57196 .53423 .57863 .52807 .57513 .52422 Metetra
.822 .799 .83 r
.57513 .52422 .57863 .52807 .58529 .52192 .5818 .518 Metetra
.82 .796 .83 r
.5818 .518 .58529 .52192 .59195 .5158 .58848 .5118 Metetra
.817 .793 .829 r
.58848 .5118 .59195 .5158 .59862 .50972 .59516 .50566 Metetra
.815 .791 .83 r
.59516 .50566 .59862 .50972 .60529 .50369 .60184 .49958 Metetra
.813 .788 .83 r
.60184 .49958 .60529 .50369 .61196 .49772 .60853 .49355 Metetra
.81 .786 .831 r
.60853 .49355 .61196 .49772 .61863 .49182 .61521 .4876 Metetra
.807 .784 .832 r
.61521 .4876 .61863 .49182 .6253 .48599 .6219 .48175 Metetra
.804 .783 .833 r
.6219 .48175 .6253 .48599 .63198 .48025 .62859 .47597 Metetra
.801 .782 .835 r
.62859 .47597 .63198 .48025 .63866 .47458 .63529 .47028 Metetra
.799 .781 .836 r
.63529 .47028 .63866 .47458 .64535 .469 .64199 .46468 Metetra
.796 .78 .838 r
.64199 .46468 .64535 .469 .65204 .4635 .64869 .45917 Metetra
.793 .779 .84 r
.64869 .45917 .65204 .4635 .65874 .45809 .65541 .45375 Metetra
.791 .779 .842 r
.65541 .45375 .65874 .45809 .66545 .45276 .66213 .44841 Metetra
.788 .779 .844 r
.66213 .44841 .66545 .45276 .67216 .44751 .66885 .44314 Metetra
.785 .778 .846 r
.66885 .44314 .67216 .44751 .67888 .44232 .67558 .43797 Metetra
.783 .779 .848 r
.67558 .43797 .67888 .44232 .68561 .43721 .68233 .43286 Metetra
.781 .779 .851 r
.68233 .43286 .68561 .43721 .69234 .43218 .68908 .42782 Metetra
.779 .779 .852 r
.68908 .42782 .69234 .43218 .69909 .42722 .69583 .42286 Metetra
.777 .779 .854 r
.69583 .42286 .69909 .42722 .70584 .4223 .7026 .41796 Metetra
.775 .779 .856 r
.7026 .41796 .70584 .4223 .7126 .41745 .70937 .41311 Metetra
.773 .779 .858 r
.70937 .41311 .7126 .41745 .71937 .41266 .71616 .40832 Metetra
.771 .78 .86 r
.71616 .40832 .71937 .41266 .72615 .40792 .72295 .40359 Metetra
.77 .78 .862 r
.72295 .40359 .72615 .40792 .73294 .40323 .72975 .3989 Metetra
.768 .78 .863 r
.72975 .3989 .73294 .40323 .73973 .39859 .73656 .39426 Metetra
.767 .781 .865 r
.73656 .39426 .73973 .39859 .74654 .39398 .74338 .38965 Metetra
.766 .782 .866 r
.74338 .38965 .74654 .39398 .75335 .3894 .75021 .38508 Metetra
.764 .782 .867 r
.75021 .38508 .75335 .3894 .76018 .38486 .75705 .38055 Metetra
.763 .783 .869 r
.75705 .38055 .76018 .38486 .76701 .38035 .7639 .37605 Metetra
.762 .783 .87 r
.7639 .37605 .76701 .38035 .77386 .37588 .77076 .37157 Metetra
.761 .783 .871 r
.77076 .37157 .77386 .37588 .78071 .37143 .77763 .36713 Metetra
.761 .784 .872 r
.77763 .36713 .78071 .37143 .78758 .367 .78451 .3627 Metetra
.76 .784 .873 r
.78451 .3627 .78758 .367 .79445 .36259 .7914 .35829 Metetra
.759 .785 .874 r
.7914 .35829 .79445 .36259 .80133 .35818 .7983 .35389 Metetra
.759 .786 .875 r
.7983 .35389 .80133 .35818 .80822 .35379 .8052 .3495 Metetra
.759 .786 .876 r
.8052 .3495 .80822 .35379 .81512 .3494 .81212 .34512 Metetra
.759 .787 .876 r
.81212 .34512 .81512 .3494 .82203 .34503 .81904 .34074 Metetra
.758 .788 .877 r
.81904 .34074 .82203 .34503 .82895 .34065 .82597 .33637 Metetra
.758 .789 .878 r
.82597 .33637 .82895 .34065 .83587 .33628 .83291 .33199 Metetra
.758 .789 .878 r
.83291 .33199 .83587 .33628 .84281 .33191 .83986 .32762 Metetra
.756 .768 .863 r
.20425 .52953 .20817 .53295 .21404 .52915 .21013 .52574 Metetra
.756 .769 .863 r
.21013 .52574 .21404 .52915 .21992 .52535 .21602 .52194 Metetra
.757 .769 .863 r
.21602 .52194 .21992 .52535 .22581 .52154 .22192 .51814 Metetra
.758 .771 .864 r
.22192 .51814 .22581 .52154 .2317 .51772 .22782 .51432 Metetra
.758 .771 .864 r
.22782 .51432 .2317 .51772 .23761 .51387 .23373 .51049 Metetra
.759 .772 .864 r
.23373 .51049 .23761 .51387 .24351 .51002 .23965 .50664 Metetra
.759 .773 .864 r
.23965 .50664 .24351 .51002 .24943 .50614 .24557 .50277 Metetra
.76 .774 .864 r
.24557 .50277 .24943 .50614 .25535 .50225 .2515 .49888 Metetra
.761 .775 .864 r
.2515 .49888 .25535 .50225 .26128 .49834 .25744 .49497 Metetra
.762 .776 .864 r
.25744 .49497 .26128 .49834 .26722 .49439 .26339 .49104 Metetra
.764 .777 .864 r
.26339 .49104 .26722 .49439 .27316 .49042 .26934 .48707 Metetra
.765 .778 .863 r
.26934 .48707 .27316 .49042 .27912 .48641 .27531 .48307 Metetra
.766 .779 .863 r
.27531 .48307 .27912 .48641 .28508 .48236 .28128 .47903 Metetra
.768 .78 .863 r
.28128 .47903 .28508 .48236 .29105 .47827 .28726 .47495 Metetra
.77 .781 .862 r
.28726 .47495 .29105 .47827 .29703 .47414 .29324 .47082 Metetra
.771 .782 .862 r
.29324 .47082 .29703 .47414 .30301 .46996 .29924 .46666 Metetra
.773 .783 .861 r
.29924 .46666 .30301 .46996 .309 .46575 .30524 .46245 Metetra
.775 .784 .86 r
.30524 .46245 .309 .46575 .31501 .46147 .31125 .45818 Metetra
.777 .785 .859 r
.31125 .45818 .31501 .46147 .32102 .45714 .31727 .45386 Metetra
.78 .786 .858 r
.31727 .45386 .32102 .45714 .32703 .45275 .3233 .44948 Metetra
.782 .787 .857 r
.3233 .44948 .32703 .45275 .33306 .4483 .32934 .44503 Metetra
.784 .788 .856 r
.32934 .44503 .33306 .4483 .33909 .44379 .33538 .44053 Metetra
.787 .789 .855 r
.33538 .44053 .33909 .44379 .34513 .43921 .34143 .43596 Metetra
.789 .79 .853 r
.34143 .43596 .34513 .43921 .35118 .43456 .34749 .43131 Metetra
.792 .791 .852 r
.34749 .43131 .35118 .43456 .35724 .42983 .35356 .42658 Metetra
.795 .792 .85 r
.35356 .42658 .35724 .42983 .3633 .42503 .35963 .42178 Metetra
.797 .792 .848 r
.35963 .42178 .3633 .42503 .36936 .42015 .36571 .41689 Metetra
.799 .793 .846 r
.36571 .41689 .36936 .42015 .37544 .41519 .3718 .41193 Metetra
.802 .793 .844 r
.3718 .41193 .37544 .41519 .38152 .41015 .37789 .40688 Metetra
.805 .793 .842 r
.37789 .40688 .38152 .41015 .38761 .40502 .38399 .40173 Metetra
.807 .793 .839 r
.38399 .40173 .38761 .40502 .3937 .39981 .3901 .3965 Metetra
.809 .792 .837 r
.3901 .3965 .3937 .39981 .39979 .39452 .3962 .39118 Metetra
.811 .791 .834 r
.3962 .39118 .39979 .39452 .40589 .38916 .40232 .38578 Metetra
.813 .79 .831 r
.40232 .38578 .40589 .38916 .412 .38372 .40844 .38029 Metetra
.814 .788 .829 r
.40844 .38029 .412 .38372 .4181 .3782 .41456 .37473 Metetra
.815 .786 .826 r
.41456 .37473 .4181 .3782 .42421 .37264 .42068 .3691 Metetra
.815 .784 .824 r
.42068 .3691 .42421 .37264 .43032 .36702 .4268 .36341 Metetra
.815 .781 .821 r
.4268 .36341 .43032 .36702 .43643 .36136 .43293 .35768 Metetra
.815 .778 .819 r
.43293 .35768 .43643 .36136 .44254 .35567 .43906 .35191 Metetra
.813 .775 .817 r
.43906 .35191 .44254 .35567 .44865 .34997 .44518 .34614 Metetra
.721 .246 0 r
.44518 .34614 .44865 .34997 .45445 .35874 .45097 .35492 Metetra
.722 .246 0 r
.45097 .35492 .45445 .35874 .46029 .36758 .45681 .36378 Metetra
.722 .246 0 r
.45681 .36378 .46029 .36758 .46619 .3765 .4627 .3727 Metetra
.722 .245 0 r
.4627 .3727 .46619 .3765 .47214 .38548 .46865 .38169 Metetra
.721 .245 0 r
.46865 .38169 .47214 .38548 .47814 .39453 .47465 .39075 Metetra
.721 .244 0 r
.47465 .39075 .47814 .39453 .48421 .40366 .48071 .39987 Metetra
.719 .242 0 r
.48071 .39987 .48421 .40366 .49033 .41285 .48683 .40906 Metetra
.718 .241 0 r
.48683 .40906 .49033 .41285 .4965 .42213 .493 .41832 Metetra
.716 .24 0 r
.493 .41832 .4965 .42213 .50274 .43148 .49923 .42766 Metetra
.715 .239 0 r
.49923 .42766 .50274 .43148 .50903 .44091 .50552 .43708 Metetra
.714 .238 0 r
.50552 .43708 .50903 .44091 .51538 .45044 .51187 .4466 Metetra
.714 .238 0 r
.51187 .4466 .51538 .45044 .52179 .46006 .51828 .45621 Metetra
.714 .238 0 r
.51828 .45621 .52179 .46006 .52827 .46979 .52475 .46593 Metetra
.714 .239 0 r
.52475 .46593 .52827 .46979 .5348 .47963 .53128 .47577 Metetra
.715 .24 0 r
.53128 .47577 .5348 .47963 .5414 .48957 .53788 .48571 Metetra
.716 .241 0 r
.53788 .48571 .5414 .48957 .54806 .49963 .54454 .49578 Metetra
.717 .242 0 r
.54454 .49578 .54806 .49963 .55479 .50979 .55127 .50596 Metetra
.718 .244 0 r
.55127 .50596 .55479 .50979 .56159 .52007 .55806 .51626 Metetra
.72 .245 0 r
.55806 .51626 .56159 .52007 .56845 .53046 .56492 .52666 Metetra
.825 .802 .829 r
.56492 .52666 .56845 .53046 .57513 .52422 .57161 .52034 Metetra
.823 .798 .828 r
.57161 .52034 .57513 .52422 .5818 .518 .5783 .51402 Metetra
.821 .795 .827 r
.5783 .51402 .5818 .518 .58848 .5118 .585 .50774 Metetra
.818 .791 .827 r
.585 .50774 .58848 .5118 .59516 .50566 .59169 .50152 Metetra
.815 .788 .827 r
.59169 .50152 .59516 .50566 .60184 .49958 .59838 .49537 Metetra
.812 .785 .827 r
.59838 .49537 .60184 .49958 .60853 .49355 .60508 .48929 Metetra
.809 .783 .828 r
.60508 .48929 .60853 .49355 .61521 .4876 .61178 .48329 Metetra
.806 .781 .829 r
.61178 .48329 .61521 .4876 .6219 .48175 .61848 .47739 Metetra
.803 .779 .831 r
.61848 .47739 .6219 .48175 .62859 .47597 .62518 .47158 Metetra
.8 .777 .832 r
.62518 .47158 .62859 .47597 .63529 .47028 .63189 .46587 Metetra
.797 .776 .834 r
.63189 .46587 .63529 .47028 .64199 .46468 .6386 .46025 Metetra
.794 .776 .836 r
.6386 .46025 .64199 .46468 .64869 .45917 .64532 .45472 Metetra
.791 .775 .838 r
.64532 .45472 .64869 .45917 .65541 .45375 .65205 .44929 Metetra
.789 .774 .84 r
.65205 .44929 .65541 .45375 .66213 .44841 .65878 .44395 Metetra
.786 .774 .842 r
.65878 .44395 .66213 .44841 .66885 .44314 .66552 .43869 Metetra
.784 .775 .844 r
.66552 .43869 .66885 .44314 .67558 .43797 .67227 .43351 Metetra
.781 .774 .847 r
.67227 .43351 .67558 .43797 .68233 .43286 .67903 .42841 Metetra
.779 .775 .849 r
.67903 .42841 .68233 .43286 .68908 .42782 .68579 .42338 Metetra
.777 .775 .851 r
.68579 .42338 .68908 .42782 .69583 .42286 .69256 .41842 Metetra
.775 .775 .853 r
.69256 .41842 .69583 .42286 .7026 .41796 .69934 .41352 Metetra
.773 .776 .855 r
.69934 .41352 .7026 .41796 .70937 .41311 .70613 .40869 Metetra
.771 .776 .857 r
.70613 .40869 .70937 .41311 .71616 .40832 .71293 .4039 Metetra
.77 .777 .859 r
.71293 .4039 .71616 .40832 .72295 .40359 .71974 .39917 Metetra
.768 .777 .86 r
.71974 .39917 .72295 .40359 .72975 .3989 .72655 .3945 Metetra
.767 .778 .862 r
.72655 .3945 .72975 .3989 .73656 .39426 .73338 .38987 Metetra
.765 .778 .864 r
.73338 .38987 .73656 .39426 .74338 .38965 .74021 .38527 Metetra
.764 .779 .865 r
.74021 .38527 .74338 .38965 .75021 .38508 .74706 .38071 Metetra
.763 .78 .867 r
.74706 .38071 .75021 .38508 .75705 .38055 .75391 .37618 Metetra
.762 .78 .868 r
.75391 .37618 .75705 .38055 .7639 .37605 .76078 .37168 Metetra
.761 .781 .869 r
.76078 .37168 .7639 .37605 .77076 .37157 .76765 .36722 Metetra
.76 .782 .871 r
.76765 .36722 .77076 .37157 .77763 .36713 .77454 .36278 Metetra
.759 .782 .872 r
.77454 .36278 .77763 .36713 .78451 .3627 .78143 .35836 Metetra
.759 .783 .873 r
.78143 .35836 .78451 .3627 .7914 .35829 .78833 .35395 Metetra
.759 .784 .874 r
.78833 .35395 .7914 .35829 .7983 .35389 .79524 .34955 Metetra
.758 .784 .875 r
.79524 .34955 .7983 .35389 .8052 .3495 .80217 .34517 Metetra
.758 .786 .876 r
.80217 .34517 .8052 .3495 .81212 .34512 .8091 .34079 Metetra
.757 .786 .876 r
.8091 .34079 .81212 .34512 .81904 .34074 .81604 .33642 Metetra
.758 .787 .877 r
.81604 .33642 .81904 .34074 .82597 .33637 .82298 .33205 Metetra
.757 .788 .878 r
.82298 .33205 .82597 .33637 .83291 .33199 .82994 .32768 Metetra
.758 .789 .879 r
.82994 .32768 .83291 .33199 .83986 .32762 .83691 .32331 Metetra
.755 .768 .863 r
.20032 .5261 .20425 .52953 .21013 .52574 .20621 .52232 Metetra
.756 .769 .864 r
.20621 .52232 .21013 .52574 .21602 .52194 .2121 .51853 Metetra
.756 .77 .864 r
.2121 .51853 .21602 .52194 .22192 .51814 .21801 .51473 Metetra
.757 .771 .864 r
.21801 .51473 .22192 .51814 .22782 .51432 .22392 .51092 Metetra
.758 .772 .865 r
.22392 .51092 .22782 .51432 .23373 .51049 .22984 .5071 Metetra
.758 .773 .865 r
.22984 .5071 .23373 .51049 .23965 .50664 .23576 .50326 Metetra
.759 .774 .865 r
.23576 .50326 .23965 .50664 .24557 .50277 .2417 .4994 Metetra
.76 .775 .865 r
.2417 .4994 .24557 .50277 .2515 .49888 .24764 .49552 Metetra
.761 .776 .866 r
.24764 .49552 .2515 .49888 .25744 .49497 .25358 .49163 Metetra
.762 .777 .866 r
.25358 .49163 .25744 .49497 .26339 .49104 .25954 .4877 Metetra
.763 .778 .865 r
.25954 .4877 .26339 .49104 .26934 .48707 .2655 .48374 Metetra
.765 .779 .865 r
.2655 .48374 .26934 .48707 .27531 .48307 .27147 .47975 Metetra
.767 .781 .865 r
.27147 .47975 .27531 .48307 .28128 .47903 .27745 .47572 Metetra
.768 .781 .864 r
.27745 .47572 .28128 .47903 .28726 .47495 .28344 .47165 Metetra
.77 .783 .864 r
.28344 .47165 .28726 .47495 .29324 .47082 .28944 .46753 Metetra
.771 .784 .863 r
.28944 .46753 .29324 .47082 .29924 .46666 .29545 .46338 Metetra
.773 .785 .863 r
.29545 .46338 .29924 .46666 .30524 .46245 .30146 .45919 Metetra
.775 .787 .862 r
.30146 .45919 .30524 .46245 .31125 .45818 .30748 .45493 Metetra
.778 .788 .861 r
.30748 .45493 .31125 .45818 .31727 .45386 .31351 .45062 Metetra
.78 .789 .86 r
.31351 .45062 .31727 .45386 .3233 .44948 .31955 .44625 Metetra
.782 .79 .859 r
.31955 .44625 .3233 .44948 .32934 .44503 .32559 .44182 Metetra
.785 .792 .858 r
.32559 .44182 .32934 .44503 .33538 .44053 .33165 .43732 Metetra
.787 .793 .857 r
.33165 .43732 .33538 .44053 .34143 .43596 .33771 .43276 Metetra
.79 .794 .856 r
.33771 .43276 .34143 .43596 .34749 .43131 .34378 .42812 Metetra
.793 .795 .854 r
.34378 .42812 .34749 .43131 .35356 .42658 .34986 .42339 Metetra
.795 .796 .852 r
.34986 .42339 .35356 .42658 .35963 .42178 .35594 .4186 Metetra
.798 .797 .851 r
.35594 .4186 .35963 .42178 .36571 .41689 .36204 .41371 Metetra
.801 .797 .849 r
.36204 .41371 .36571 .41689 .3718 .41193 .36814 .40874 Metetra
.804 .798 .846 r
.36814 .40874 .3718 .41193 .37789 .40688 .37424 .40367 Metetra
.806 .797 .844 r
.37424 .40367 .37789 .40688 .38399 .40173 .38036 .39852 Metetra
.809 .797 .841 r
.38036 .39852 .38399 .40173 .3901 .3965 .38647 .39326 Metetra
.811 .797 .838 r
.38647 .39326 .3901 .3965 .3962 .39118 .3926 .38791 Metetra
.813 .795 .835 r
.3926 .38791 .3962 .39118 .40232 .38578 .39872 .38247 Metetra
.815 .794 .832 r
.39872 .38247 .40232 .38578 .40844 .38029 .40486 .37693 Metetra
.817 .792 .829 r
.40486 .37693 .40844 .38029 .41456 .37473 .41099 .3713 Metetra
.817 .79 .826 r
.41099 .3713 .41456 .37473 .42068 .3691 .41713 .36561 Metetra
.818 .787 .823 r
.41713 .36561 .42068 .3691 .4268 .36341 .42327 .35984 Metetra
.818 .784 .82 r
.42327 .35984 .4268 .36341 .43293 .35768 .42941 .35402 Metetra
.817 .78 .818 r
.42941 .35402 .43293 .35768 .43906 .35191 .43555 .34816 Metetra
.816 .776 .816 r
.43555 .34816 .43906 .35191 .44518 .34614 .4417 .34228 Metetra
.723 .247 0 r
.4417 .34228 .44518 .34614 .45097 .35492 .44748 .35109 Metetra
.724 .248 0 r
.44748 .35109 .45097 .35492 .45681 .36378 .45331 .35996 Metetra
.724 .248 0 r
.45331 .35996 .45681 .36378 .4627 .3727 .4592 .3689 Metetra
.724 .247 0 r
.4592 .3689 .4627 .3727 .46865 .38169 .46515 .3779 Metetra
.723 .245 0 r
.46515 .3779 .46865 .38169 .47465 .39075 .47115 .38696 Metetra
.721 .244 0 r
.47115 .38696 .47465 .39075 .48071 .39987 .4772 .39608 Metetra
.72 .242 0 r
.4772 .39608 .48071 .39987 .48683 .40906 .48332 .40526 Metetra
.718 .24 0 r
.48332 .40526 .48683 .40906 .493 .41832 .48949 .41451 Metetra
.716 .239 0 r
.48949 .41451 .493 .41832 .49923 .42766 .49571 .42383 Metetra
.714 .238 0 r
.49571 .42383 .49923 .42766 .50552 .43708 .502 .43323 Metetra
.713 .237 0 r
.502 .43323 .50552 .43708 .51187 .4466 .50835 .44273 Metetra
.712 .237 0 r
.50835 .44273 .51187 .4466 .51828 .45621 .51475 .45233 Metetra
.712 .237 0 r
.51475 .45233 .51828 .45621 .52475 .46593 .52122 .46204 Metetra
.713 .238 0 r
.52122 .46204 .52475 .46593 .53128 .47577 .52775 .47187 Metetra
.714 .239 0 r
.52775 .47187 .53128 .47577 .53788 .48571 .53435 .48182 Metetra
.715 .241 0 r
.53435 .48182 .53788 .48571 .54454 .49578 .541 .4919 Metetra
.717 .243 0 r
.541 .4919 .54454 .49578 .55127 .50596 .54773 .5021 Metetra
.719 .244 0 r
.54773 .5021 .55127 .50596 .55806 .51626 .55452 .51242 Metetra
.72 .246 0 r
.55452 .51242 .55806 .51626 .56492 .52666 .56137 .52285 Metetra
.828 .802 .827 r
.56137 .52285 .56492 .52666 .57161 .52034 .56808 .51642 Metetra
.825 .798 .825 r
.56808 .51642 .57161 .52034 .5783 .51402 .57479 .50999 Metetra
.822 .793 .824 r
.57479 .50999 .5783 .51402 .585 .50774 .58149 .50361 Metetra
.819 .789 .824 r
.58149 .50361 .585 .50774 .59169 .50152 .5882 .4973 Metetra
.816 .785 .824 r
.5882 .4973 .59169 .50152 .59838 .49537 .5949 .49107 Metetra
.812 .782 .824 r
.5949 .49107 .59838 .49537 .60508 .48929 .60161 .48492 Metetra
.809 .779 .825 r
.60161 .48492 .60508 .48929 .61178 .48329 .60832 .47887 Metetra
.805 .776 .826 r
.60832 .47887 .61178 .48329 .61848 .47739 .61504 .47292 Metetra
.802 .774 .828 r
.61504 .47292 .61848 .47739 .62518 .47158 .62176 .46708 Metetra
.798 .773 .83 r
.62176 .46708 .62518 .47158 .63189 .46587 .62848 .46134 Metetra
.795 .772 .831 r
.62848 .46134 .63189 .46587 .6386 .46025 .63521 .4557 Metetra
.792 .771 .834 r
.63521 .4557 .6386 .46025 .64532 .45472 .64194 .45017 Metetra
.789 .77 .836 r
.64194 .45017 .64532 .45472 .65205 .44929 .64868 .44473 Metetra
.787 .77 .838 r
.64868 .44473 .65205 .44929 .65878 .44395 .65543 .43939 Metetra
.784 .77 .84 r
.65543 .43939 .65878 .44395 .66552 .43869 .66218 .43413 Metetra
.781 .77 .842 r
.66218 .43413 .66552 .43869 .67227 .43351 .66894 .42896 Metetra
.779 .77 .845 r
.66894 .42896 .67227 .43351 .67903 .42841 .67571 .42386 Metetra
.777 .771 .847 r
.67571 .42386 .67903 .42841 .68579 .42338 .68249 .41884 Metetra
.775 .771 .849 r
.68249 .41884 .68579 .42338 .69256 .41842 .68927 .4139 Metetra
.773 .772 .851 r
.68927 .4139 .69256 .41842 .69934 .41352 .69607 .40901 Metetra
.771 .772 .853 r
.69607 .40901 .69934 .41352 .70613 .40869 .70287 .40418 Metetra
.769 .773 .856 r
.70287 .40418 .70613 .40869 .71293 .4039 .70968 .39942 Metetra
.768 .774 .858 r
.70968 .39942 .71293 .4039 .71974 .39917 .7165 .3947 Metetra
.767 .775 .86 r
.7165 .3947 .71974 .39917 .72655 .3945 .72334 .39004 Metetra
.765 .775 .861 r
.72334 .39004 .72655 .3945 .73338 .38987 .73018 .38541 Metetra
.764 .776 .863 r
.73018 .38541 .73338 .38987 .74021 .38527 .73703 .38083 Metetra
.763 .777 .864 r
.73703 .38083 .74021 .38527 .74706 .38071 .74389 .37627 Metetra
.761 .777 .866 r
.74389 .37627 .74706 .38071 .75391 .37618 .75076 .37176 Metetra
.761 .778 .868 r
.75076 .37176 .75391 .37618 .76078 .37168 .75764 .36727 Metetra
.76 .779 .869 r
.75764 .36727 .76078 .37168 .76765 .36722 .76452 .36281 Metetra
.759 .78 .87 r
.76452 .36281 .76765 .36722 .77454 .36278 .77142 .35838 Metetra
.758 .781 .872 r
.77142 .35838 .77454 .36278 .78143 .35836 .77833 .35397 Metetra
.758 .781 .872 r
.77833 .35397 .78143 .35836 .78833 .35395 .78525 .34957 Metetra
.757 .782 .874 r
.78525 .34957 .78833 .35395 .79524 .34955 .79218 .34518 Metetra
.757 .783 .875 r
.79218 .34518 .79524 .34955 .80217 .34517 .79911 .34081 Metetra
.757 .784 .875 r
.79911 .34081 .80217 .34517 .8091 .34079 .80606 .33644 Metetra
.757 .786 .877 r
.80606 .33644 .8091 .34079 .81604 .33642 .81302 .33207 Metetra
.756 .786 .877 r
.81302 .33207 .81604 .33642 .82298 .33205 .81998 .32771 Metetra
.757 .787 .878 r
.81998 .32771 .82298 .33205 .82994 .32768 .82695 .32335 Metetra
.757 .788 .879 r
.82695 .32335 .82994 .32768 .83691 .32331 .83393 .31899 Metetra
.754 .768 .864 r
.19636 .52265 .20032 .5261 .20621 .52232 .20226 .51888 Metetra
.755 .769 .864 r
.20226 .51888 .20621 .52232 .2121 .51853 .20817 .5151 Metetra
.755 .77 .865 r
.20817 .5151 .2121 .51853 .21801 .51473 .21408 .51132 Metetra
.756 .771 .865 r
.21408 .51132 .21801 .51473 .22392 .51092 .22 .50752 Metetra
.757 .772 .866 r
.22 .50752 .22392 .51092 .22984 .5071 .22593 .50371 Metetra
.758 .773 .866 r
.22593 .50371 .22984 .5071 .23576 .50326 .23186 .49988 Metetra
.758 .774 .866 r
.23186 .49988 .23576 .50326 .2417 .4994 .2378 .49604 Metetra
.759 .776 .867 r
.2378 .49604 .2417 .4994 .24764 .49552 .24375 .49217 Metetra
.76 .777 .867 r
.24375 .49217 .24764 .49552 .25358 .49163 .24971 .48828 Metetra
.761 .778 .867 r
.24971 .48828 .25358 .49163 .25954 .4877 .25567 .48437 Metetra
.763 .78 .867 r
.25567 .48437 .25954 .4877 .2655 .48374 .26164 .48043 Metetra
.764 .781 .867 r
.26164 .48043 .2655 .48374 .27147 .47975 .26762 .47645 Metetra
.766 .782 .866 r
.26762 .47645 .27147 .47975 .27745 .47572 .27361 .47243 Metetra
.768 .784 .866 r
.27361 .47243 .27745 .47572 .28344 .47165 .27961 .46837 Metetra
.769 .785 .866 r
.27961 .46837 .28344 .47165 .28944 .46753 .28562 .46428 Metetra
.771 .786 .866 r
.28562 .46428 .28944 .46753 .29545 .46338 .29163 .46014 Metetra
.773 .788 .865 r
.29163 .46014 .29545 .46338 .30146 .45919 .29766 .45595 Metetra
.775 .789 .864 r
.29766 .45595 .30146 .45919 .30748 .45493 .30369 .45172 Metetra
.777 .791 .864 r
.30369 .45172 .30748 .45493 .31351 .45062 .30973 .44742 Metetra
.78 .792 .863 r
.30973 .44742 .31351 .45062 .31955 .44625 .31578 .44307 Metetra
.782 .793 .862 r
.31578 .44307 .31955 .44625 .32559 .44182 .32183 .43865 Metetra
.785 .795 .861 r
.32183 .43865 .32559 .44182 .33165 .43732 .3279 .43417 Metetra
.787 .796 .86 r
.3279 .43417 .33165 .43732 .33771 .43276 .33397 .42962 Metetra
.79 .798 .858 r
.33397 .42962 .33771 .43276 .34378 .42812 .34005 .42498 Metetra
.793 .799 .857 r
.34005 .42498 .34378 .42812 .34986 .42339 .34614 .42028 Metetra
.796 .8 .855 r
.34614 .42028 .34986 .42339 .35594 .4186 .35224 .41548 Metetra
.799 .801 .853 r
.35224 .41548 .35594 .4186 .36204 .41371 .35834 .4106 Metetra
.802 .802 .851 r
.35834 .4106 .36204 .41371 .36814 .40874 .36445 .40562 Metetra
.805 .802 .849 r
.36445 .40562 .36814 .40874 .37424 .40367 .37057 .40055 Metetra
.808 .802 .846 r
.37057 .40055 .37424 .40367 .38036 .39852 .3767 .39538 Metetra
.811 .802 .843 r
.3767 .39538 .38036 .39852 .38647 .39326 .38283 .3901 Metetra
.813 .801 .84 r
.38283 .3901 .38647 .39326 .3926 .38791 .38897 .38472 Metetra
.816 .8 .837 r
.38897 .38472 .3926 .38791 .39872 .38247 .39511 .37923 Metetra
.818 .799 .833 r
.39511 .37923 .39872 .38247 .40486 .37693 .40126 .37363 Metetra
.82 .796 .83 r
.40126 .37363 .40486 .37693 .41099 .3713 .40741 .36794 Metetra
.821 .794 .826 r
.40741 .36794 .41099 .3713 .41713 .36561 .41357 .36216 Metetra
.821 .79 .823 r
.41357 .36216 .41713 .36561 .42327 .35984 .41972 .3563 Metetra
.821 .786 .819 r
.41972 .3563 .42327 .35984 .42941 .35402 .42588 .35038 Metetra
.82 .782 .816 r
.42588 .35038 .42941 .35402 .43555 .34816 .43204 .3444 Metetra
.819 .777 .814 r
.43204 .3444 .43555 .34816 .4417 .34228 .43819 .33841 Metetra
.724 .249 0 r
.43819 .33841 .4417 .34228 .44748 .35109 .44397 .34725 Metetra
.726 .25 0 r
.44397 .34725 .44748 .35109 .45331 .35996 .4498 .35615 Metetra
.726 .25 0 r
.4498 .35615 .45331 .35996 .4592 .3689 .45569 .36511 Metetra
.726 .249 0 r
.45569 .36511 .4592 .3689 .46515 .3779 .46163 .37413 Metetra
.725 .247 0 r
.46163 .37413 .46515 .3779 .47115 .38696 .46762 .38319 Metetra
.723 .245 0 r
.46762 .38319 .47115 .38696 .4772 .39608 .47368 .3923 Metetra
.72 .242 0 r
.47368 .3923 .4772 .39608 .48332 .40526 .47979 .40146 Metetra
.718 .24 0 r
.47979 .40146 .48332 .40526 .48949 .41451 .48595 .41069 Metetra
.715 .238 0 r
.48595 .41069 .48949 .41451 .49571 .42383 .49218 .41998 Metetra
.713 .236 0 r
.49218 .41998 .49571 .42383 .502 .43323 .49846 .42936 Metetra
.711 .235 0 r
.49846 .42936 .502 .43323 .50835 .44273 .50481 .43883 Metetra
.71 .234 0 r
.50481 .43883 .50835 .44273 .51475 .45233 .51121 .44841 Metetra
.71 .235 0 r
.51121 .44841 .51475 .45233 .52122 .46204 .51768 .4581 Metetra
.71 .236 0 r
.51768 .4581 .52122 .46204 .52775 .47187 .5242 .46792 Metetra
.712 .238 0 r
.5242 .46792 .52775 .47187 .53435 .48182 .53079 .47788 Metetra
.713 .24 0 r
.53079 .47788 .53435 .48182 .541 .4919 .53745 .48797 Metetra
.716 .243 0 r
.53745 .48797 .541 .4919 .54773 .5021 .54417 .4982 Metetra
.719 .245 0 r
.54417 .4982 .54773 .5021 .55452 .51242 .55096 .50856 Metetra
.721 .247 0 r
.55096 .50856 .55452 .51242 .56137 .52285 .55781 .51903 Metetra
.83 .802 .824 r
.55781 .51903 .56137 .52285 .56808 .51642 .56453 .51245 Metetra
.828 .797 .822 r
.56453 .51245 .56808 .51642 .57479 .50999 .57125 .5059 Metetra
.824 .791 .821 r
.57125 .5059 .57479 .50999 .58149 .50361 .57797 .4994 Metetra
.82 .786 .821 r
.57797 .4994 .58149 .50361 .5882 .4973 .58469 .49299 Metetra
.816 .782 .821 r
.58469 .49299 .5882 .4973 .5949 .49107 .59141 .48666 Metetra
.812 .778 .821 r
.59141 .48666 .5949 .49107 .60161 .48492 .59813 .48043 Metetra
.808 .774 .822 r
.59813 .48043 .60161 .48492 .60832 .47887 .60485 .47432 Metetra
.804 .772 .823 r
.60485 .47432 .60832 .47887 .61504 .47292 .61158 .46833 Metetra
.8 .77 .825 r
.61158 .46833 .61504 .47292 .62176 .46708 .61831 .46245 Metetra
.797 .768 .827 r
.61831 .46245 .62176 .46708 .62848 .46134 .62505 .45668 Metetra
.793 .767 .829 r
.62505 .45668 .62848 .46134 .63521 .4557 .63179 .45104 Metetra
.79 .766 .831 r
.63179 .45104 .63521 .4557 .64194 .45017 .63854 .44549 Metetra
.787 .765 .833 r
.63854 .44549 .64194 .45017 .64868 .44473 .64529 .44005 Metetra
.784 .765 .836 r
.64529 .44005 .64868 .44473 .65543 .43939 .65205 .43471 Metetra
.782 .765 .838 r
.65205 .43471 .65543 .43939 .66218 .43413 .65882 .42947 Metetra
.78 .766 .841 r
.65882 .42947 .66218 .43413 .66894 .42896 .66559 .4243 Metetra
.777 .766 .843 r
.66559 .4243 .66894 .42896 .67571 .42386 .67237 .41922 Metetra
.775 .767 .846 r
.67237 .41922 .67571 .42386 .68249 .41884 .67917 .41422 Metetra
.773 .767 .848 r
.67917 .41422 .68249 .41884 .68927 .4139 .68597 .40929 Metetra
.771 .768 .85 r
.68597 .40929 .68927 .4139 .69607 .40901 .69277 .40441 Metetra
.769 .769 .852 r
.69277 .40441 .69607 .40901 .70287 .40418 .69959 .39961 Metetra
.768 .77 .854 r
.69959 .39961 .70287 .40418 .70968 .39942 .70642 .39485 Metetra
.766 .77 .856 r
.70642 .39485 .70968 .39942 .7165 .3947 .71326 .39015 Metetra
.764 .771 .858 r
.71326 .39015 .7165 .3947 .72334 .39004 .7201 .3855 Metetra
.763 .772 .86 r
.7201 .3855 .72334 .39004 .73018 .38541 .72696 .3809 Metetra
.762 .773 .862 r
.72696 .3809 .73018 .38541 .73703 .38083 .73382 .37632 Metetra
.761 .774 .864 r
.73382 .37632 .73703 .38083 .74389 .37627 .7407 .37179 Metetra
.76 .776 .865 r
.7407 .37179 .74389 .37627 .75076 .37176 .74758 .36728 Metetra
.759 .776 .867 r
.74758 .36728 .75076 .37176 .75764 .36727 .75448 .3628 Metetra
.758 .777 .868 r
.75448 .3628 .75764 .36727 .76452 .36281 .76138 .35836 Metetra
.758 .778 .87 r
.76138 .35836 .76452 .36281 .77142 .35838 .7683 .35394 Metetra
.757 .779 .871 r
.7683 .35394 .77142 .35838 .77833 .35397 .77522 .34954 Metetra
.757 .78 .873 r
.77522 .34954 .77833 .35397 .78525 .34957 .78215 .34515 Metetra
.756 .781 .874 r
.78215 .34515 .78525 .34957 .79218 .34518 .7891 .34078 Metetra
.756 .782 .875 r
.7891 .34078 .79218 .34518 .79911 .34081 .79605 .33641 Metetra
.756 .784 .876 r
.79605 .33641 .79911 .34081 .80606 .33644 .80301 .33205 Metetra
.755 .784 .877 r
.80301 .33205 .80606 .33644 .81302 .33207 .80998 .3277 Metetra
.756 .786 .878 r
.80998 .3277 .81302 .33207 .81998 .32771 .81696 .32335 Metetra
.756 .787 .878 r
.81696 .32335 .81998 .32771 .82695 .32335 .82395 .319 Metetra
.756 .788 .879 r
.82395 .319 .82695 .32335 .83393 .31899 .83095 .31464 Metetra
.754 .768 .864 r
.1924 .51919 .19636 .52265 .20226 .51888 .1983 .51543 Metetra
.754 .769 .865 r
.1983 .51543 .20226 .51888 .20817 .5151 .20422 .51167 Metetra
.755 .77 .866 r
.20422 .51167 .20817 .5151 .21408 .51132 .21014 .5079 Metetra
.755 .771 .866 r
.21014 .5079 .21408 .51132 .22 .50752 .21606 .50412 Metetra
.756 .773 .867 r
.21606 .50412 .22 .50752 .22593 .50371 .222 .50032 Metetra
.757 .774 .867 r
.222 .50032 .22593 .50371 .23186 .49988 .22794 .4965 Metetra
.758 .775 .868 r
.22794 .4965 .23186 .49988 .2378 .49604 .23389 .49267 Metetra
.759 .777 .868 r
.23389 .49267 .2378 .49604 .24375 .49217 .23985 .48882 Metetra
.759 .778 .868 r
.23985 .48882 .24375 .49217 .24971 .48828 .24581 .48495 Metetra
.761 .779 .868 r
.24581 .48495 .24971 .48828 .25567 .48437 .25179 .48105 Metetra
.762 .781 .868 r
.25179 .48105 .25567 .48437 .26164 .48043 .25777 .47713 Metetra
.764 .782 .868 r
.25777 .47713 .26164 .48043 .26762 .47645 .26376 .47316 Metetra
.766 .784 .868 r
.26376 .47316 .26762 .47645 .27361 .47243 .26975 .46916 Metetra
.767 .785 .868 r
.26975 .46916 .27361 .47243 .27961 .46837 .27576 .46512 Metetra
.769 .787 .868 r
.27576 .46512 .27961 .46837 .28562 .46428 .28178 .46104 Metetra
.771 .788 .867 r
.28178 .46104 .28562 .46428 .29163 .46014 .2878 .45692 Metetra
.772 .79 .867 r
.2878 .45692 .29163 .46014 .29766 .45595 .29383 .45276 Metetra
.775 .792 .867 r
.29383 .45276 .29766 .45595 .30369 .45172 .29987 .44854 Metetra
.777 .793 .866 r
.29987 .44854 .30369 .45172 .30973 .44742 .30592 .44426 Metetra
.78 .795 .865 r
.30592 .44426 .30973 .44742 .31578 .44307 .31198 .43993 Metetra
.782 .797 .864 r
.31198 .43993 .31578 .44307 .32183 .43865 .31805 .43553 Metetra
.785 .798 .864 r
.31805 .43553 .32183 .43865 .3279 .43417 .32412 .43106 Metetra
.788 .8 .862 r
.32412 .43106 .3279 .43417 .33397 .42962 .33021 .42653 Metetra
.791 .802 .861 r
.33021 .42653 .33397 .42962 .34005 .42498 .3363 .42192 Metetra
.794 .803 .86 r
.3363 .42192 .34005 .42498 .34614 .42028 .3424 .41722 Metetra
.797 .804 .858 r
.3424 .41722 .34614 .42028 .35224 .41548 .34851 .41244 Metetra
.8 .806 .856 r
.34851 .41244 .35224 .41548 .35834 .4106 .35463 .40756 Metetra
.803 .807 .854 r
.35463 .40756 .35834 .4106 .36445 .40562 .36075 .40259 Metetra
.807 .807 .851 r
.36075 .40259 .36445 .40562 .37057 .40055 .36688 .39751 Metetra
.81 .807 .849 r
.36688 .39751 .37057 .40055 .3767 .39538 .37302 .39233 Metetra
.813 .807 .846 r
.37302 .39233 .3767 .39538 .38283 .3901 .37917 .38703 Metetra
.816 .807 .842 r
.37917 .38703 .38283 .3901 .38897 .38472 .38532 .38162 Metetra
.819 .806 .839 r
.38532 .38162 .38897 .38472 .39511 .37923 .39148 .37608 Metetra
.821 .804 .835 r
.39148 .37608 .39511 .37923 .40126 .37363 .39764 .37043 Metetra
.823 .801 .831 r
.39764 .37043 .40126 .37363 .40741 .36794 .40381 .36466 Metetra
.824 .798 .827 r
.40381 .36466 .40741 .36794 .41357 .36216 .40998 .35879 Metetra
.825 .794 .822 r
.40998 .35879 .41357 .36216 .41972 .3563 .41615 .35282 Metetra
.825 .789 .818 r
.41615 .35282 .41972 .3563 .42588 .35038 .42233 .34677 Metetra
.824 .784 .814 r
.42233 .34677 .42588 .35038 .43204 .3444 .4285 .34066 Metetra
.822 .778 .811 r
.4285 .34066 .43204 .3444 .43819 .33841 .43468 .33453 Metetra
.727 .252 0 r
.43468 .33453 .43819 .33841 .44397 .34725 .44045 .34341 Metetra
.729 .253 0 r
.44045 .34341 .44397 .34725 .4498 .35615 .44628 .35235 Metetra
.729 .253 0 r
.44628 .35235 .4498 .35615 .45569 .36511 .45216 .36134 Metetra
.729 .251 0 r
.45216 .36134 .45569 .36511 .46163 .37413 .45809 .37037 Metetra
.727 .249 0 r
.45809 .37037 .46163 .37413 .46762 .38319 .46409 .37944 Metetra
.725 .246 0 r
.46409 .37944 .46762 .38319 .47368 .3923 .47014 .38854 Metetra
.721 .242 0 r
.47014 .38854 .47368 .3923 .47979 .40146 .47624 .39768 Metetra
.718 .239 0 r
.47624 .39768 .47979 .40146 .48595 .41069 .48241 .40687 Metetra
.714 .236 0 r
.48241 .40687 .48595 .41069 .49218 .41998 .48863 .41613 Metetra
.711 .234 0 r
.48863 .41613 .49218 .41998 .49846 .42936 .49491 .42546 Metetra
.709 .232 0 r
.49491 .42546 .49846 .42936 .50481 .43883 .50125 .4349 Metetra
.707 .232 0 r
.50125 .4349 .50481 .43883 .51121 .44841 .50765 .44444 Metetra
.707 .232 0 r
.50765 .44444 .51121 .44841 .51768 .4581 .51412 .45411 Metetra
.707 .234 0 r
.51412 .45411 .51768 .4581 .5242 .46792 .52064 .46393 Metetra
.709 .236 0 r
.52064 .46393 .5242 .46792 .53079 .47788 .52723 .47389 Metetra
.712 .239 0 r
.52723 .47389 .53079 .47788 .53745 .48797 .53388 .484 Metetra
.715 .243 0 r
.53388 .484 .53745 .48797 .54417 .4982 .5406 .49426 Metetra
.719 .246 0 r
.5406 .49426 .54417 .4982 .55096 .50856 .54738 .50466 Metetra
.723 .249 0 r
.54738 .50466 .55096 .50856 .55781 .51903 .55424 .51518 Metetra
.834 .802 .821 r
.55424 .51518 .55781 .51903 .56453 .51245 .56097 .50845 Metetra
.83 .796 .819 r
.56097 .50845 .56453 .51245 .57125 .5059 .5677 .50174 Metetra
.826 .789 .817 r
.5677 .50174 .57125 .5059 .57797 .4994 .57444 .4951 Metetra
.821 .783 .817 r
.57444 .4951 .57797 .4994 .58469 .49299 .58117 .48856 Metetra
.817 .778 .817 r
.58117 .48856 .58469 .49299 .59141 .48666 .5879 .48213 Metetra
.812 .773 .817 r
.5879 .48213 .59141 .48666 .59813 .48043 .59463 .47582 Metetra
.807 .77 .818 r
.59463 .47582 .59813 .48043 .60485 .47432 .60137 .46964 Metetra
.803 .767 .82 r
.60137 .46964 .60485 .47432 .61158 .46833 .60811 .4636 Metetra
.799 .764 .822 r
.60811 .4636 .61158 .46833 .61831 .46245 .61485 .45768 Metetra
.795 .763 .824 r
.61485 .45768 .61831 .46245 .62505 .45668 .6216 .4519 Metetra
.791 .762 .826 r
.6216 .4519 .62505 .45668 .63179 .45104 .62835 .44623 Metetra
.788 .761 .829 r
.62835 .44623 .63179 .45104 .63854 .44549 .63511 .44069 Metetra
.785 .76 .831 r
.63511 .44069 .63854 .44549 .64529 .44005 .64188 .43526 Metetra
.782 .761 .834 r
.64188 .43526 .64529 .44005 .65205 .43471 .64865 .42993 Metetra
.779 .761 .836 r
.64865 .42993 .65205 .43471 .65882 .42947 .65544 .42469 Metetra
.777 .761 .839 r
.65544 .42469 .65882 .42947 .66559 .4243 .66222 .41954 Metetra
.775 .762 .841 r
.66222 .41954 .66559 .4243 .67237 .41922 .66902 .41448 Metetra
.772 .762 .844 r
.66902 .41448 .67237 .41922 .67917 .41422 .67583 .4095 Metetra
.77 .763 .846 r
.67583 .4095 .67917 .41422 .68597 .40929 .68264 .40458 Metetra
.769 .764 .849 r
.68264 .40458 .68597 .40929 .69277 .40441 .68947 .39974 Metetra
.767 .765 .851 r
.68947 .39974 .69277 .40441 .69959 .39961 .6963 .39494 Metetra
.765 .766 .853 r
.6963 .39494 .69959 .39961 .70642 .39485 .70314 .39021 Metetra
.764 .767 .856 r
.70314 .39021 .70642 .39485 .71326 .39015 .70999 .38554 Metetra
.763 .769 .858 r
.70999 .38554 .71326 .39015 .7201 .3855 .71685 .3809 Metetra
.761 .77 .86 r
.71685 .3809 .7201 .3855 .72696 .3809 .72372 .37631 Metetra
.76 .771 .861 r
.72372 .37631 .72696 .3809 .73382 .37632 .7306 .37176 Metetra
.76 .772 .863 r
.7306 .37176 .73382 .37632 .7407 .37179 .73749 .36724 Metetra
.758 .773 .865 r
.73749 .36724 .7407 .37179 .74758 .36728 .74439 .36275 Metetra
.758 .774 .867 r
.74439 .36275 .74758 .36728 .75448 .3628 .7513 .3583 Metetra
.757 .775 .868 r
.7513 .3583 .75448 .3628 .76138 .35836 .75822 .35386 Metetra
.756 .776 .87 r
.75822 .35386 .76138 .35836 .7683 .35394 .76515 .34946 Metetra
.756 .778 .871 r
.76515 .34946 .7683 .35394 .77522 .34954 .77209 .34507 Metetra
.755 .778 .872 r
.77209 .34507 .77522 .34954 .78215 .34515 .77904 .3407 Metetra
.755 .78 .874 r
.77904 .3407 .78215 .34515 .7891 .34078 .786 .33634 Metetra
.755 .781 .875 r
.786 .33634 .7891 .34078 .79605 .33641 .79297 .33198 Metetra
.754 .782 .876 r
.79297 .33198 .79605 .33641 .80301 .33205 .79995 .32764 Metetra
.755 .784 .877 r
.79995 .32764 .80301 .33205 .80998 .3277 .80693 .32329 Metetra
.754 .785 .878 r
.80693 .32329 .80998 .3277 .81696 .32335 .81393 .31896 Metetra
.755 .786 .879 r
.81393 .31896 .81696 .32335 .82395 .319 .82093 .31462 Metetra
.755 .787 .88 r
.82093 .31462 .82395 .319 .83095 .31464 .82795 .31028 Metetra
.753 .768 .865 r
.18842 .51571 .1924 .51919 .1983 .51543 .19433 .51197 Metetra
.753 .769 .866 r
.19433 .51197 .1983 .51543 .20422 .51167 .20025 .50822 Metetra
.753 .77 .867 r
.20025 .50822 .20422 .51167 .21014 .5079 .20618 .50447 Metetra
.754 .772 .867 r
.20618 .50447 .21014 .5079 .21606 .50412 .21211 .5007 Metetra
.755 .773 .868 r
.21211 .5007 .21606 .50412 .222 .50032 .21805 .49692 Metetra
.756 .775 .868 r
.21805 .49692 .222 .50032 .22794 .4965 .224 .49312 Metetra
.757 .776 .869 r
.224 .49312 .22794 .4965 .23389 .49267 .22996 .48931 Metetra
.758 .777 .869 r
.22996 .48931 .23389 .49267 .23985 .48882 .23593 .48548 Metetra
.759 .779 .87 r
.23593 .48548 .23985 .48882 .24581 .48495 .2419 .48163 Metetra
.76 .78 .87 r
.2419 .48163 .24581 .48495 .25179 .48105 .24788 .47775 Metetra
.762 .782 .87 r
.24788 .47775 .25179 .48105 .25777 .47713 .25387 .47384 Metetra
.763 .783 .87 r
.25387 .47384 .25777 .47713 .26376 .47316 .25987 .46989 Metetra
.765 .785 .87 r
.25987 .46989 .26376 .47316 .26975 .46916 .26588 .46592 Metetra
.767 .787 .87 r
.26588 .46592 .26975 .46916 .27576 .46512 .27189 .46189 Metetra
.768 .789 .87 r
.27189 .46189 .27576 .46512 .28178 .46104 .27792 .45784 Metetra
.77 .791 .87 r
.27792 .45784 .28178 .46104 .2878 .45692 .28395 .45374 Metetra
.772 .792 .869 r
.28395 .45374 .2878 .45692 .29383 .45276 .28999 .44959 Metetra
.774 .794 .869 r
.28999 .44959 .29383 .45276 .29987 .44854 .29604 .4454 Metetra
.777 .796 .869 r
.29604 .4454 .29987 .44854 .30592 .44426 .3021 .44115 Metetra
.78 .798 .868 r
.3021 .44115 .30592 .44426 .31198 .43993 .30817 .43683 Metetra
.782 .8 .867 r
.30817 .43683 .31198 .43993 .31805 .43553 .31424 .43246 Metetra
.785 .802 .866 r
.31424 .43246 .31805 .43553 .32412 .43106 .32033 .42802 Metetra
.788 .804 .865 r
.32033 .42802 .32412 .43106 .33021 .42653 .32642 .42351 Metetra
.791 .806 .864 r
.32642 .42351 .33021 .42653 .3363 .42192 .33253 .41892 Metetra
.794 .807 .863 r
.33253 .41892 .3363 .42192 .3424 .41722 .33864 .41424 Metetra
.798 .809 .861 r
.33864 .41424 .3424 .41722 .34851 .41244 .34476 .40947 Metetra
.801 .81 .859 r
.34476 .40947 .34851 .41244 .35463 .40756 .35089 .40461 Metetra
.804 .812 .857 r
.35089 .40461 .35463 .40756 .36075 .40259 .35702 .39965 Metetra
.808 .813 .854 r
.35702 .39965 .36075 .40259 .36688 .39751 .36317 .39458 Metetra
.812 .813 .851 r
.36317 .39458 .36688 .39751 .37302 .39233 .36932 .38938 Metetra
.815 .813 .848 r
.36932 .38938 .37302 .39233 .37917 .38703 .37548 .38406 Metetra
.819 .813 .845 r
.37548 .38406 .37917 .38703 .38532 .38162 .38165 .37862 Metetra
.822 .812 .841 r
.38165 .37862 .38532 .38162 .39148 .37608 .38782 .37304 Metetra
.825 .81 .836 r
.38782 .37304 .39148 .37608 .39764 .37043 .394 .36732 Metetra
.827 .807 .831 r
.394 .36732 .39764 .37043 .40381 .36466 .40019 .36147 Metetra
.829 .803 .827 r
.40019 .36147 .40381 .36466 .40998 .35879 .40638 .35549 Metetra
.83 .798 .822 r
.40638 .35549 .40998 .35879 .41615 .35282 .41257 .34939 Metetra
.83 .793 .817 r
.41257 .34939 .41615 .35282 .42233 .34677 .41876 .3432 Metetra
.829 .787 .812 r
.41876 .3432 .42233 .34677 .4285 .34066 .42496 .33693 Metetra
.826 .78 .808 r
.42496 .33693 .4285 .34066 .43468 .33453 .43115 .33063 Metetra
.73 .256 0 r
.43115 .33063 .43468 .33453 .44045 .34341 .43691 .33956 Metetra
.733 .257 0 r
.43691 .33956 .44045 .34341 .44628 .35235 .44274 .34856 Metetra
.734 .257 0 r
.44274 .34856 .44628 .35235 .45216 .36134 .44861 .35759 Metetra
.733 .255 0 r
.44861 .35759 .45216 .36134 .45809 .37037 .45454 .36665 Metetra
.731 .251 0 r
.45454 .36665 .45809 .37037 .46409 .37944 .46053 .37572 Metetra
.727 .247 0 r
.46053 .37572 .46409 .37944 .47014 .38854 .46658 .3848 Metetra
.723 .242 0 r
.46658 .3848 .47014 .38854 .47624 .39768 .47268 .39391 Metetra
.718 .238 0 r
.47268 .39391 .47624 .39768 .48241 .40687 .47885 .40306 Metetra
.713 .234 0 r
.47885 .40306 .48241 .40687 .48863 .41613 .48507 .41226 Metetra
.709 .23 0 r
.48507 .41226 .48863 .41613 .49491 .42546 .49134 .42154 Metetra
.705 .228 0 r
.49134 .42154 .49491 .42546 .50125 .4349 .49768 .43092 Metetra
.703 .227 0 r
.49768 .43092 .50125 .4349 .50765 .44444 .50408 .44042 Metetra
.702 .228 0 r
.50408 .44042 .50765 .44444 .51412 .45411 .51054 .45006 Metetra
.703 .23 0 r
.51054 .45006 .51412 .45411 .52064 .46393 .51706 .45985 Metetra
.705 .234 0 r
.51706 .45985 .52064 .46393 .52723 .47389 .52365 .46982 Metetra
.709 .238 0 r
.52365 .46982 .52723 .47389 .53388 .484 .53029 .47996 Metetra
.714 .243 0 r
.53029 .47996 .53388 .484 .5406 .49426 .53701 .49026 Metetra
.719 .248 0 r
.53701 .49026 .5406 .49426 .54738 .50466 .54379 .50073 Metetra
.724 .252 0 r
.54379 .50073 .54738 .50466 .55424 .51518 .55064 .51132 Metetra
.838 .803 .817 r
.55064 .51132 .55424 .51518 .56097 .50845 .55739 .50439 Metetra
.833 .794 .814 r
.55739 .50439 .56097 .50845 .5677 .50174 .56414 .4975 Metetra
.828 .787 .813 r
.56414 .4975 .5677 .50174 .57444 .4951 .57088 .49069 Metetra
.822 .78 .812 r
.57088 .49069 .57444 .4951 .58117 .48856 .57763 .48401 Metetra
.817 .773 .812 r
.57763 .48401 .58117 .48856 .5879 .48213 .58437 .47746 Metetra
.811 .768 .813 r
.58437 .47746 .5879 .48213 .59463 .47582 .59112 .47105 Metetra
.806 .764 .814 r
.59112 .47105 .59463 .47582 .60137 .46964 .59787 .46481 Metetra
.801 .761 .816 r
.59787 .46481 .60137 .46964 .60811 .4636 .60462 .45871 Metetra
.797 .759 .818 r
.60462 .45871 .60811 .4636 .61485 .45768 .61137 .45277 Metetra
.792 .757 .821 r
.61137 .45277 .61485 .45768 .6216 .4519 .61813 .44697 Metetra
.789 .756 .823 r
.61813 .44697 .6216 .4519 .62835 .44623 .6249 .4413 Metetra
.785 .756 .826 r
.6249 .4413 .62835 .44623 .63511 .44069 .63168 .43576 Metetra
.782 .755 .829 r
.63168 .43576 .63511 .44069 .64188 .43526 .63846 .43034 Metetra
.779 .755 .831 r
.63846 .43034 .64188 .43526 .64865 .42993 .64524 .42502 Metetra
.776 .756 .834 r
.64524 .42502 .64865 .42993 .65544 .42469 .65204 .4198 Metetra
.774 .757 .837 r
.65204 .4198 .65544 .42469 .66222 .41954 .65884 .41468 Metetra
.772 .757 .84 r
.65884 .41468 .66222 .41954 .66902 .41448 .66565 .40964 Metetra
.77 .758 .843 r
.66565 .40964 .66902 .41448 .67583 .4095 .67247 .40468 Metetra
.768 .759 .845 r
.67247 .40468 .67583 .4095 .68264 .40458 .6793 .39979 Metetra
.766 .761 .848 r
.6793 .39979 .68264 .40458 .68947 .39974 .68614 .39497 Metetra
.765 .762 .85 r
.68614 .39497 .68947 .39974 .6963 .39494 .69299 .39021 Metetra
.763 .763 .852 r
.69299 .39021 .6963 .39494 .70314 .39021 .69984 .3855 Metetra
.762 .764 .855 r
.69984 .3855 .70314 .39021 .70999 .38554 .70671 .38084 Metetra
.76 .765 .857 r
.70671 .38084 .70999 .38554 .71685 .3809 .71359 .37624 Metetra
.759 .767 .859 r
.71359 .37624 .71685 .3809 .72372 .37631 .72047 .37167 Metetra
.759 .768 .861 r
.72047 .37167 .72372 .37631 .7306 .37176 .72737 .36714 Metetra
.758 .769 .863 r
.72737 .36714 .7306 .37176 .73749 .36724 .73427 .36264 Metetra
.757 .771 .864 r
.73427 .36264 .73749 .36724 .74439 .36275 .74119 .35817 Metetra
.756 .772 .866 r
.74119 .35817 .74439 .36275 .7513 .3583 .74811 .35373 Metetra
.755 .773 .868 r
.74811 .35373 .7513 .3583 .75822 .35386 .75505 .34932 Metetra
.755 .775 .87 r
.75505 .34932 .75822 .35386 .76515 .34946 .76199 .34493 Metetra
.754 .776 .871 r
.76199 .34493 .76515 .34946 .77209 .34507 .76895 .34057 Metetra
.754 .777 .872 r
.76895 .34057 .77209 .34507 .77904 .3407 .77591 .33621 Metetra
.753 .779 .874 r
.77591 .33621 .77904 .3407 .786 .33634 .78289 .33186 Metetra
.753 .78 .875 r
.78289 .33186 .786 .33634 .79297 .33198 .78987 .32753 Metetra
.753 .782 .876 r
.78987 .32753 .79297 .33198 .79995 .32764 .79687 .32319 Metetra
.753 .783 .877 r
.79687 .32319 .79995 .32764 .80693 .32329 .80387 .31887 Metetra
.753 .784 .878 r
.80387 .31887 .80693 .32329 .81393 .31896 .81088 .31454 Metetra
.753 .786 .879 r
.81088 .31454 .81393 .31896 .82093 .31462 .8179 .31022 Metetra
.754 .787 .88 r
.8179 .31022 .82093 .31462 .82795 .31028 .82494 .30589 Metetra
.752 .767 .866 r
.18442 .51222 .18842 .51571 .19433 .51197 .19034 .5085 Metetra
.752 .769 .867 r
.19034 .5085 .19433 .51197 .20025 .50822 .19626 .50477 Metetra
.753 .77 .868 r
.19626 .50477 .20025 .50822 .20618 .50447 .2022 .50104 Metetra
.753 .772 .868 r
.2022 .50104 .20618 .50447 .21211 .5007 .20814 .49729 Metetra
.754 .774 .869 r
.20814 .49729 .21211 .5007 .21805 .49692 .21409 .49352 Metetra
.755 .775 .869 r
.21409 .49352 .21805 .49692 .224 .49312 .22005 .48975 Metetra
.756 .777 .87 r
.22005 .48975 .224 .49312 .22996 .48931 .22601 .48595 Metetra
.757 .778 .87 r
.22601 .48595 .22996 .48931 .23593 .48548 .23199 .48214 Metetra
.758 .78 .871 r
.23199 .48214 .23593 .48548 .2419 .48163 .23797 .47831 Metetra
.759 .782 .871 r
.23797 .47831 .2419 .48163 .24788 .47775 .24396 .47445 Metetra
.76 .783 .872 r
.24396 .47445 .24788 .47775 .25387 .47384 .24996 .47057 Metetra
.762 .785 .872 r
.24996 .47057 .25387 .47384 .25987 .46989 .25596 .46665 Metetra
.764 .787 .872 r
.25596 .46665 .25987 .46989 .26588 .46592 .26198 .46269 Metetra
.766 .789 .872 r
.26198 .46269 .26588 .46592 .27189 .46189 .268 .45869 Metetra
.768 .791 .872 r
.268 .45869 .27189 .46189 .27792 .45784 .27403 .45466 Metetra
.769 .793 .872 r
.27403 .45466 .27792 .45784 .28395 .45374 .28008 .45058 Metetra
.771 .795 .872 r
.28008 .45058 .28395 .45374 .28999 .44959 .28613 .44647 Metetra
.774 .797 .872 r
.28613 .44647 .28999 .44959 .29604 .4454 .29219 .4423 Metetra
.776 .799 .871 r
.29219 .4423 .29604 .4454 .3021 .44115 .29825 .43807 Metetra
.779 .801 .871 r
.29825 .43807 .3021 .44115 .30817 .43683 .30433 .43379 Metetra
.782 .803 .87 r
.30433 .43379 .30817 .43683 .31424 .43246 .31042 .42944 Metetra
.784 .805 .869 r
.31042 .42944 .31424 .43246 .32033 .42802 .31651 .42503 Metetra
.787 .807 .869 r
.31651 .42503 .32033 .42802 .32642 .42351 .32262 .42055 Metetra
.791 .81 .867 r
.32262 .42055 .32642 .42351 .33253 .41892 .32873 .41598 Metetra
.794 .812 .866 r
.32873 .41598 .33253 .41892 .33864 .41424 .33485 .41132 Metetra
.798 .813 .864 r
.33485 .41132 .33864 .41424 .34476 .40947 .34099 .40659 Metetra
.801 .816 .863 r
.34099 .40659 .34476 .40947 .35089 .40461 .34713 .40174 Metetra
.805 .817 .86 r
.34713 .40174 .35089 .40461 .35702 .39965 .35327 .3968 Metetra
.809 .818 .858 r
.35327 .3968 .35702 .39965 .36317 .39458 .35943 .39173 Metetra
.813 .819 .855 r
.35943 .39173 .36317 .39458 .36932 .38938 .3656 .38654 Metetra
.818 .82 .851 r
.3656 .38654 .36932 .38938 .37548 .38406 .37177 .38121 Metetra
.821 .82 .847 r
.37177 .38121 .37548 .38406 .38165 .37862 .37796 .37574 Metetra
.825 .818 .843 r
.37796 .37574 .38165 .37862 .38782 .37304 .38414 .37012 Metetra
.829 .816 .838 r
.38414 .37012 .38782 .37304 .394 .36732 .39034 .36434 Metetra
.832 .814 .832 r
.39034 .36434 .394 .36732 .40019 .36147 .39654 .3584 Metetra
.834 .809 .827 r
.39654 .3584 .40019 .36147 .40638 .35549 .40275 .3523 Metetra
.835 .804 .821 r
.40275 .3523 .40638 .35549 .41257 .34939 .40896 .34606 Metetra
.835 .797 .815 r
.40896 .34606 .41257 .34939 .41876 .3432 .41518 .33968 Metetra
.834 .79 .809 r
.41518 .33968 .41876 .3432 .42496 .33693 .42139 .33322 Metetra
.831 .781 .805 r
.42139 .33322 .42496 .33693 .43115 .33063 .4276 .32671 Metetra
.735 .262 0 r
.4276 .32671 .43115 .33063 .43691 .33956 .43336 .33573 Metetra
.738 .263 0 r
.43336 .33573 .43691 .33956 .44274 .34856 .43918 .3448 Metetra
.74 .262 0 r
.43918 .3448 .44274 .34856 .44861 .35759 .44505 .35389 Metetra
.738 .259 0 r
.44505 .35389 .44861 .35759 .45454 .36665 .45098 .36298 Metetra
.735 .254 0 r
.45098 .36298 .45454 .36665 .46053 .37572 .45696 .37205 Metetra
.73 .248 0 r
.45696 .37205 .46053 .37572 .46658 .3848 .46301 .38111 Metetra
.724 .242 0 r
.46301 .38111 .46658 .3848 .47268 .39391 .46911 .39018 Metetra
.718 .236 0 r
.46911 .39018 .47268 .39391 .47885 .40306 .47527 .39926 Metetra
.711 .23 0 r
.47527 .39926 .47885 .40306 .48507 .41226 .48149 .40839 Metetra
.705 .226 0 r
.48149 .40839 .48507 .41226 .49134 .42154 .48776 .4176 Metetra
.7 .223 0 r
.48776 .4176 .49134 .42154 .49768 .43092 .4941 .4269 Metetra
.697 .222 0 r
.4941 .4269 .49768 .43092 .50408 .44042 .50049 .43634 Metetra
.696 .223 0 r
.50049 .43634 .50408 .44042 .51054 .45006 .50695 .44592 Metetra
.697 .225 0 r
.50695 .44592 .51054 .45006 .51706 .45985 .51347 .45569 Metetra
.7 .23 0 r
.51347 .45569 .51706 .45985 .52365 .46982 .52005 .46566 Metetra
.705 .236 0 r
.52005 .46566 .52365 .46982 .53029 .47996 .52669 .47583 Metetra
.712 .243 0 r
.52669 .47583 .53029 .47996 .53701 .49026 .53341 .48619 Metetra
.72 .25 0 r
.53341 .48619 .53701 .49026 .54379 .50073 .54019 .49674 Metetra
.727 .255 0 r
.54019 .49674 .54379 .50073 .55064 .51132 .54703 .50744 Metetra
.843 .803 .812 r
.54703 .50744 .55064 .51132 .55739 .50439 .5538 .50027 Metetra
.837 .793 .809 r
.5538 .50027 .55739 .50439 .56414 .4975 .56056 .49315 Metetra
.83 .783 .807 r
.56056 .49315 .56414 .4975 .57088 .49069 .56731 .48615 Metetra
.824 .775 .807 r
.56731 .48615 .57088 .49069 .57763 .48401 .57407 .4793 Metetra
.817 .768 .807 r
.57407 .4793 .57763 .48401 .58437 .47746 .58083 .47262 Metetra
.811 .762 .808 r
.58083 .47262 .58437 .47746 .59112 .47105 .58759 .46612 Metetra
.805 .758 .81 r
.58759 .46612 .59112 .47105 .59787 .46481 .59435 .4598 Metetra
.799 .754 .812 r
.59435 .4598 .59787 .46481 .60462 .45871 .60111 .45366 Metetra
.794 .752 .815 r
.60111 .45366 .60462 .45871 .61137 .45277 .60788 .44769 Metetra
.79 .751 .817 r
.60788 .44769 .61137 .45277 .61813 .44697 .61465 .44188 Metetra
.786 .75 .82 r
.61465 .44188 .61813 .44697 .6249 .4413 .62143 .43621 Metetra
.782 .749 .823 r
.62143 .43621 .6249 .4413 .63168 .43576 .62822 .43069 Metetra
.779 .75 .826 r
.62822 .43069 .63168 .43576 .63846 .43034 .63501 .42529 Metetra
.776 .75 .829 r
.63501 .42529 .63846 .43034 .64524 .42502 .64181 .41999 Metetra
.774 .751 .832 r
.64181 .41999 .64524 .42502 .65204 .4198 .64862 .4148 Metetra
.771 .752 .835 r
.64862 .4148 .65204 .4198 .65884 .41468 .65544 .40971 Metetra
.769 .753 .838 r
.65544 .40971 .65884 .41468 .66565 .40964 .66227 .40469 Metetra
.767 .754 .841 r
.66227 .40469 .66565 .40964 .67247 .40468 .6691 .39977 Metetra
.765 .755 .844 r
.6691 .39977 .67247 .40468 .6793 .39979 .67594 .39491 Metetra
.764 .757 .846 r
.67594 .39491 .6793 .39979 .68614 .39497 .6828 .39012 Metetra
.762 .758 .849 r
.6828 .39012 .68614 .39497 .69299 .39021 .68966 .38538 Metetra
.761 .759 .851 r
.68966 .38538 .69299 .39021 .69984 .3855 .69653 .38071 Metetra
.759 .761 .854 r
.69653 .38071 .69984 .3855 .70671 .38084 .70341 .37608 Metetra
.758 .763 .856 r
.70341 .37608 .70671 .38084 .71359 .37624 .7103 .3715 Metetra
.757 .764 .858 r
.7103 .3715 .71359 .37624 .72047 .37167 .7172 .36696 Metetra
.756 .765 .86 r
.7172 .36696 .72047 .37167 .72737 .36714 .72411 .36245 Metetra
.756 .767 .862 r
.72411 .36245 .72737 .36714 .73427 .36264 .73103 .35797 Metetra
.755 .768 .864 r
.73103 .35797 .73427 .36264 .74119 .35817 .73797 .35353 Metetra
.754 .77 .866 r
.73797 .35353 .74119 .35817 .74811 .35373 .74491 .34912 Metetra
.753 .772 .868 r
.74491 .34912 .74811 .35373 .75505 .34932 .75186 .34473 Metetra
.752 .773 .869 r
.75186 .34473 .75505 .34932 .76199 .34493 .75882 .34037 Metetra
.752 .774 .871 r
.75882 .34037 .76199 .34493 .76895 .34057 .76579 .33602 Metetra
.752 .776 .872 r
.76579 .33602 .76895 .34057 .77591 .33621 .77277 .33168 Metetra
.752 .777 .874 r
.77277 .33168 .77591 .33621 .78289 .33186 .77976 .32735 Metetra
.752 .779 .875 r
.77976 .32735 .78289 .33186 .78987 .32753 .78676 .32303 Metetra
.752 .78 .876 r
.78676 .32303 .78987 .32753 .79687 .32319 .79377 .31872 Metetra
.752 .782 .878 r
.79377 .31872 .79687 .32319 .80387 .31887 .80079 .31441 Metetra
.752 .783 .879 r
.80079 .31441 .80387 .31887 .81088 .31454 .80782 .3101 Metetra
.752 .785 .88 r
.80782 .3101 .81088 .31454 .8179 .31022 .81486 .3058 Metetra
.752 .786 .881 r
.81486 .3058 .8179 .31022 .82494 .30589 .82191 .30149 Metetra
.75 .767 .867 r
.1804 .50872 .18442 .51222 .19034 .5085 .18633 .50502 Metetra
.751 .769 .868 r
.18633 .50502 .19034 .5085 .19626 .50477 .19226 .50131 Metetra
.751 .771 .869 r
.19226 .50131 .19626 .50477 .2022 .50104 .1982 .4976 Metetra
.752 .772 .869 r
.1982 .4976 .2022 .50104 .20814 .49729 .20415 .49387 Metetra
.753 .774 .87 r
.20415 .49387 .20814 .49729 .21409 .49352 .21011 .49013 Metetra
.754 .776 .871 r
.21011 .49013 .21409 .49352 .22005 .48975 .21608 .48637 Metetra
.755 .777 .871 r
.21608 .48637 .22005 .48975 .22601 .48595 .22205 .4826 Metetra
.756 .779 .872 r
.22205 .4826 .22601 .48595 .23199 .48214 .22803 .47881 Metetra
.757 .781 .873 r
.22803 .47881 .23199 .48214 .23797 .47831 .23402 .475 Metetra
.758 .782 .873 r
.23402 .475 .23797 .47831 .24396 .47445 .24002 .47117 Metetra
.76 .785 .873 r
.24002 .47117 .24396 .47445 .24996 .47057 .24602 .4673 Metetra
.761 .786 .874 r
.24602 .4673 .24996 .47057 .25596 .46665 .25204 .46341 Metetra
.763 .788 .874 r
.25204 .46341 .25596 .46665 .26198 .46269 .25806 .45948 Metetra
.765 .791 .874 r
.25806 .45948 .26198 .46269 .268 .45869 .26409 .45551 Metetra
.766 .792 .874 r
.26409 .45551 .268 .45869 .27403 .45466 .27013 .4515 Metetra
.769 .795 .874 r
.27013 .4515 .27403 .45466 .28008 .45058 .27618 .44746 Metetra
.771 .797 .874 r
.27618 .44746 .28008 .45058 .28613 .44647 .28224 .44337 Metetra
.773 .799 .874 r
.28224 .44337 .28613 .44647 .29219 .4423 .28831 .43923 Metetra
.775 .802 .874 r
.28831 .43923 .29219 .4423 .29825 .43807 .29439 .43504 Metetra
.778 .804 .874 r
.29439 .43504 .29825 .43807 .30433 .43379 .30047 .43079 Metetra
.781 .806 .873 r
.30047 .43079 .30433 .43379 .31042 .42944 .30657 .42647 Metetra
.784 .809 .873 r
.30657 .42647 .31042 .42944 .31651 .42503 .31268 .42209 Metetra
.787 .811 .872 r
.31268 .42209 .31651 .42503 .32262 .42055 .31879 .41765 Metetra
.79 .814 .871 r
.31879 .41765 .32262 .42055 .32873 .41598 .32491 .41311 Metetra
.794 .816 .87 r
.32491 .41311 .32873 .41598 .33485 .41132 .33105 .40849 Metetra
.798 .819 .868 r
.33105 .40849 .33485 .41132 .34099 .40659 .33719 .40378 Metetra
.802 .821 .866 r
.33719 .40378 .34099 .40659 .34713 .40174 .34334 .39897 Metetra
.806 .823 .864 r
.34334 .39897 .34713 .40174 .35327 .3968 .3495 .39405 Metetra
.81 .825 .862 r
.3495 .39405 .35327 .3968 .35943 .39173 .35567 .389 Metetra
.815 .826 .858 r
.35567 .389 .35943 .39173 .3656 .38654 .36185 .38382 Metetra
.82 .827 .855 r
.36185 .38382 .3656 .38654 .37177 .38121 .36804 .37849 Metetra
.824 .827 .851 r
.36804 .37849 .37177 .38121 .37796 .37574 .37424 .373 Metetra
.829 .826 .846 r
.37424 .373 .37796 .37574 .38414 .37012 .38044 .36735 Metetra
.833 .825 .84 r
.38044 .36735 .38414 .37012 .39034 .36434 .38666 .3615 Metetra
.837 .821 .834 r
.38666 .3615 .39034 .36434 .39654 .3584 .39288 .35547 Metetra
.84 .817 .827 r
.39288 .35547 .39654 .3584 .40275 .3523 .3991 .34924 Metetra
.842 .811 .82 r
.3991 .34924 .40275 .3523 .40896 .34606 .40534 .34283 Metetra
.842 .803 .813 r
.40534 .34283 .40896 .34606 .41518 .33968 .41157 .33625 Metetra
.841 .794 .806 r
.41157 .33625 .41518 .33968 .42139 .33322 .41781 .32954 Metetra
.837 .783 .8 r
.41781 .32954 .42139 .33322 .4276 .32671 .42404 .32277 Metetra
.742 .27 0 r
.42404 .32277 .4276 .32671 .43336 .33573 .4298 .33192 Metetra
.746 .272 0 r
.4298 .33192 .43336 .33573 .43918 .3448 .4356 .3411 Metetra
.748 .27 0 r
.4356 .3411 .43918 .3448 .44505 .35389 .44147 .35027 Metetra
.746 .265 0 r
.44147 .35027 .44505 .35389 .45098 .36298 .44739 .35939 Metetra
.741 .258 0 r
.44739 .35939 .45098 .36298 .45696 .37205 .45337 .36846 Metetra
.734 .25 0 r
.45337 .36846 .45696 .37205 .46301 .38111 .45941 .37749 Metetra
.726 .242 0 r
.45941 .37749 .46301 .38111 .46911 .39018 .46551 .38649 Metetra
.717 .233 0 r
.46551 .38649 .46911 .39018 .47527 .39926 .47167 .39549 Metetra
.708 .226 0 r
.47167 .39549 .47527 .39926 .48149 .40839 .47789 .40452 Metetra
.7 .22 0 r
.47789 .40452 .48149 .40839 .48776 .4176 .48416 .41362 Metetra
.693 .216 0 r
.48416 .41362 .48776 .4176 .4941 .4269 .4905 .42282 Metetra
.688 .214 0 r
.4905 .42282 .4941 .4269 .50049 .43634 .49689 .43217 Metetra
.687 .215 0 r
.49689 .43217 .50049 .43634 .50695 .44592 .50334 .44168 Metetra
.688 .219 0 r
.50334 .44168 .50695 .44592 .51347 .45569 .50986 .45141 Metetra
.692 .225 0 r
.50986 .45141 .51347 .45569 .52005 .46566 .51643 .46137 Metetra
.7 .234 0 r
.51643 .46137 .52005 .46566 .52669 .47583 .52308 .47158 Metetra
.71 .243 0 r
.52308 .47158 .52669 .47583 .53341 .48619 .52979 .48203 Metetra
.72 .253 0 r
.52979 .48203 .53341 .48619 .54019 .49674 .53656 .4927 Metetra
.73 .261 0 r
.53656 .4927 .54019 .49674 .54703 .50744 .54341 .50355 Metetra
.848 .802 .806 r
.54341 .50355 .54703 .50744 .5538 .50027 .55019 .49608 Metetra
.841 .79 .802 r
.55019 .49608 .5538 .50027 .56056 .49315 .55696 .48868 Metetra
.833 .779 .801 r
.55696 .48868 .56056 .49315 .56731 .48615 .56373 .48144 Metetra
.824 .769 .8 r
.56373 .48144 .56731 .48615 .57407 .4793 .5705 .4744 Metetra
.816 .762 .801 r
.5705 .4744 .57407 .4793 .58083 .47262 .57727 .46758 Metetra
.809 .755 .803 r
.57727 .46758 .58083 .47262 .58759 .46612 .58404 .46098 Metetra
.802 .751 .805 r
.58404 .46098 .58759 .46612 .59435 .4598 .59081 .45459 Metetra
.796 .747 .808 r
.59081 .45459 .59435 .4598 .60111 .45366 .59758 .44842 Metetra
.791 .745 .811 r
.59758 .44842 .60111 .45366 .60788 .44769 .60437 .44244 Metetra
.786 .744 .814 r
.60437 .44244 .60788 .44769 .61465 .44188 .61115 .43662 Metetra
.782 .743 .817 r
.61115 .43662 .61465 .44188 .62143 .43621 .61795 .43098 Metetra
.779 .743 .821 r
.61795 .43098 .62143 .43621 .62822 .43069 .62475 .42547 Metetra
.775 .744 .824 r
.62475 .42547 .62822 .43069 .63501 .42529 .63155 .42009 Metetra
.773 .744 .827 r
.63155 .42009 .63501 .42529 .64181 .41999 .63837 .41484 Metetra
.77 .745 .83 r
.63837 .41484 .64181 .41999 .64862 .4148 .64519 .40968 Metetra
.768 .747 .834 r
.64519 .40968 .64862 .4148 .65544 .40971 .65202 .40462 Metetra
.766 .748 .837 r
.65202 .40462 .65544 .40971 .66227 .40469 .65886 .39965 Metetra
.764 .75 .84 r
.65886 .39965 .66227 .40469 .6691 .39977 .66571 .39476 Metetra
.762 .751 .843 r
.66571 .39476 .6691 .39977 .67594 .39491 .67257 .38994 Metetra
.761 .753 .845 r
.67257 .38994 .67594 .39491 .6828 .39012 .67944 .38518 Metetra
.759 .754 .848 r
.67944 .38518 .6828 .39012 .68966 .38538 .68631 .38049 Metetra
.758 .756 .851 r
.68631 .38049 .68966 .38538 .69653 .38071 .6932 .37584 Metetra
.757 .758 .853 r
.6932 .37584 .69653 .38071 .70341 .37608 .7001 .37125 Metetra
.756 .759 .855 r
.7001 .37125 .70341 .37608 .7103 .3715 .707 .3667 Metetra
.755 .761 .858 r
.707 .3667 .7103 .3715 .7172 .36696 .71392 .36219 Metetra
.754 .763 .86 r
.71392 .36219 .7172 .36696 .72411 .36245 .72084 .35771 Metetra
.753 .764 .862 r
.72084 .35771 .72411 .36245 .73103 .35797 .72778 .35327 Metetra
.753 .766 .864 r
.72778 .35327 .73103 .35797 .73797 .35353 .73473 .34885 Metetra
.752 .768 .866 r
.73473 .34885 .73797 .35353 .74491 .34912 .74168 .34446 Metetra
.751 .769 .868 r
.74168 .34446 .74491 .34912 .75186 .34473 .74865 .3401 Metetra
.751 .771 .869 r
.74865 .3401 .75186 .34473 .75882 .34037 .75563 .33575 Metetra
.75 .772 .871 r
.75563 .33575 .75882 .34037 .76579 .33602 .76262 .33143 Metetra
.751 .774 .872 r
.76262 .33143 .76579 .33602 .77277 .33168 .76961 .32711 Metetra
.75 .776 .874 r
.76961 .32711 .77277 .33168 .77976 .32735 .77662 .3228 Metetra
.75 .777 .875 r
.77662 .3228 .77976 .32735 .78676 .32303 .78364 .31851 Metetra
.75 .779 .877 r
.78364 .31851 .78676 .32303 .79377 .31872 .79066 .31421 Metetra
.75 .781 .878 r
.79066 .31421 .79377 .31872 .80079 .31441 .7977 .30993 Metetra
.75 .783 .879 r
.7977 .30993 .80079 .31441 .80782 .3101 .80475 .30564 Metetra
.751 .784 .88 r
.80475 .30564 .80782 .3101 .81486 .3058 .8118 .30135 Metetra
.751 .786 .881 r
.8118 .30135 .81486 .3058 .82191 .30149 .81887 .29706 Metetra
.749 .767 .868 r
.17637 .5052 .1804 .50872 .18633 .50502 .1823 .50152 Metetra
.75 .769 .869 r
.1823 .50152 .18633 .50502 .19226 .50131 .18824 .49784 Metetra
.75 .771 .87 r
.18824 .49784 .19226 .50131 .1982 .4976 .19419 .49414 Metetra
.751 .772 .87 r
.19419 .49414 .1982 .4976 .20415 .49387 .20015 .49044 Metetra
.752 .774 .871 r
.20015 .49044 .20415 .49387 .21011 .49013 .20611 .48672 Metetra
.753 .776 .872 r
.20611 .48672 .21011 .49013 .21608 .48637 .21208 .48299 Metetra
.754 .778 .873 r
.21208 .48299 .21608 .48637 .22205 .4826 .21806 .47925 Metetra
.755 .78 .873 r
.21806 .47925 .22205 .4826 .22803 .47881 .22405 .47548 Metetra
.756 .781 .874 r
.22405 .47548 .22803 .47881 .23402 .475 .23005 .4717 Metetra
.757 .784 .875 r
.23005 .4717 .23402 .475 .24002 .47117 .23605 .46789 Metetra
.758 .785 .875 r
.23605 .46789 .24002 .47117 .24602 .4673 .24207 .46405 Metetra
.76 .788 .876 r
.24207 .46405 .24602 .4673 .25204 .46341 .24809 .46019 Metetra
.762 .79 .876 r
.24809 .46019 .25204 .46341 .25806 .45948 .25412 .45628 Metetra
.764 .792 .876 r
.25412 .45628 .25806 .45948 .26409 .45551 .26016 .45235 Metetra
.766 .795 .877 r
.26016 .45235 .26409 .45551 .27013 .4515 .26621 .44837 Metetra
.767 .797 .877 r
.26621 .44837 .27013 .4515 .27618 .44746 .27227 .44436 Metetra
.769 .799 .877 r
.27227 .44436 .27618 .44746 .28224 .44337 .27834 .44031 Metetra
.772 .802 .877 r
.27834 .44031 .28224 .44337 .28831 .43923 .28441 .4362 Metetra
.775 .804 .877 r
.28441 .4362 .28831 .43923 .29439 .43504 .2905 .43204 Metetra
.777 .807 .877 r
.2905 .43204 .29439 .43504 .30047 .43079 .2966 .42783 Metetra
.78 .81 .876 r
.2966 .42783 .30047 .43079 .30657 .42647 .3027 .42355 Metetra
.783 .812 .876 r
.3027 .42355 .30657 .42647 .31268 .42209 .30882 .41922 Metetra
.786 .815 .875 r
.30882 .41922 .31268 .42209 .31879 .41765 .31494 .41481 Metetra
.79 .818 .875 r
.31494 .41481 .31879 .41765 .32491 .41311 .32107 .41032 Metetra
.794 .821 .873 r
.32107 .41032 .32491 .41311 .33105 .40849 .32722 .40573 Metetra
.797 .824 .872 r
.32722 .40573 .33105 .40849 .33719 .40378 .33337 .40107 Metetra
.802 .827 .871 r
.33337 .40107 .33719 .40378 .34334 .39897 .33953 .39629 Metetra
.806 .829 .869 r
.33953 .39629 .34334 .39897 .3495 .39405 .3457 .3914 Metetra
.811 .832 .866 r
.3457 .3914 .3495 .39405 .35567 .389 .35189 .38639 Metetra
.816 .833 .863 r
.35189 .38639 .35567 .389 .36185 .38382 .35808 .38123 Metetra
.821 .835 .859 r
.35808 .38123 .36185 .38382 .36804 .37849 .36428 .37591 Metetra
.827 .836 .855 r
.36428 .37591 .36804 .37849 .37424 .373 .37049 .37043 Metetra
.832 .836 .849 r
.37049 .37043 .37424 .373 .38044 .36735 .37671 .36475 Metetra
.837 .834 .843 r
.37671 .36475 .38044 .36735 .38666 .3615 .38294 .35885 Metetra
.842 .831 .836 r
.38294 .35885 .38666 .3615 .39288 .35547 .38918 .35272 Metetra
.846 .826 .828 r
.38918 .35272 .39288 .35547 .3991 .34924 .39543 .34636 Metetra
.849 .819 .819 r
.39543 .34636 .3991 .34924 .40534 .34283 .40169 .33975 Metetra
.85 .81 .81 r
.40169 .33975 .40534 .34283 .41157 .33625 .40795 .33292 Metetra
.849 .799 .802 r
.40795 .33292 .41157 .33625 .41781 .32954 .41421 .32592 Metetra
.844 .786 .795 r
.41421 .32592 .41781 .32954 .42404 .32277 .42047 .31882 Metetra
.751 .281 0 r
.42047 .31882 .42404 .32277 .4298 .33192 .42621 .32815 Metetra
.757 .283 0 r
.42621 .32815 .4298 .33192 .4356 .3411 .43201 .33748 Metetra
.758 .28 0 r
.43201 .33748 .4356 .3411 .44147 .35027 .43787 .34675 Metetra
.755 .273 0 r
.43787 .34675 .44147 .35027 .44739 .35939 .44379 .35592 Metetra
.748 .263 0 r
.44379 .35592 .44739 .35939 .45337 .36846 .44976 .36499 Metetra
.739 .252 0 r
.44976 .36499 .45337 .36846 .45941 .37749 .4558 .37396 Metetra
.728 .241 0 r
.4558 .37396 .45941 .37749 .46551 .38649 .4619 .38287 Metetra
.716 .229 0 r
.4619 .38287 .46551 .38649 .47167 .39549 .46806 .39175 Metetra
.704 .219 0 r
.46806 .39175 .47167 .39549 .47789 .40452 .47428 .40065 Metetra
.693 .211 0 r
.47428 .40065 .47789 .40452 .48416 .41362 .48055 .40961 Metetra
.684 .206 0 r
.48055 .40961 .48416 .41362 .4905 .42282 .48688 .41867 Metetra
.677 .203 0 r
.48688 .41867 .4905 .42282 .49689 .43217 .49327 .42789 Metetra
.674 .204 0 r
.49327 .42789 .49689 .43217 .50334 .44168 .49972 .43731 Metetra
.675 .209 0 r
.49972 .43731 .50334 .44168 .50986 .45141 .50623 .44697 Metetra
.681 .218 0 r
.50623 .44697 .50986 .45141 .51643 .46137 .51281 .45692 Metetra
.692 .229 0 r
.51281 .45692 .51643 .46137 .52308 .47158 .51944 .46717 Metetra
.705 .243 0 r
.51944 .46717 .52308 .47158 .52979 .48203 .52615 .47773 Metetra
.721 .257 0 r
.52615 .47773 .52979 .48203 .53656 .4927 .53292 .48858 Metetra
.735 .268 0 r
.53292 .48858 .53656 .4927 .54341 .50355 .53977 .49963 Metetra
.855 .802 .798 r
.53977 .49963 .54341 .50355 .55019 .49608 .54656 .49178 Metetra
.846 .787 .794 r
.54656 .49178 .55019 .49608 .55696 .48868 .55334 .48405 Metetra
.835 .774 .793 r
.55334 .48405 .55696 .48868 .56373 .48144 .56013 .47653 Metetra
.825 .762 .793 r
.56013 .47653 .56373 .48144 .5705 .4744 .56691 .46927 Metetra
.815 .753 .794 r
.56691 .46927 .5705 .4744 .57727 .46758 .57369 .4623 Metetra
.807 .747 .797 r
.57369 .4623 .57727 .46758 .58404 .46098 .58047 .4556 Metetra
.799 .742 .8 r
.58047 .4556 .58404 .46098 .59081 .45459 .58725 .44916 Metetra
.793 .739 .803 r
.58725 .44916 .59081 .45459 .59758 .44842 .59404 .44297 Metetra
.787 .737 .807 r
.59404 .44297 .59758 .44842 .60437 .44244 .60084 .43698 Metetra
.782 .736 .81 r
.60084 .43698 .60437 .44244 .61115 .43662 .60763 .43119 Metetra
.778 .736 .814 r
.60763 .43119 .61115 .43662 .61795 .43098 .61444 .42557 Metetra
.775 .737 .818 r
.61444 .42557 .61795 .43098 .62475 .42547 .62125 .4201 Metetra
.772 .738 .822 r
.62125 .4201 .62475 .42547 .63155 .42009 .62807 .41477 Metetra
.769 .739 .825 r
.62807 .41477 .63155 .42009 .63837 .41484 .6349 .40955 Metetra
.767 .74 .829 r
.6349 .40955 .63837 .41484 .64519 .40968 .64174 .40444 Metetra
.764 .742 .832 r
.64174 .40444 .64519 .40968 .65202 .40462 .64859 .39943 Metetra
.763 .744 .835 r
.64859 .39943 .65202 .40462 .65886 .39965 .65544 .3945 Metetra
.761 .745 .839 r
.65544 .3945 .65886 .39965 .66571 .39476 .6623 .38966 Metetra
.759 .747 .842 r
.6623 .38966 .66571 .39476 .67257 .38994 .66918 .38488 Metetra
.758 .749 .844 r
.66918 .38488 .67257 .38994 .67944 .38518 .67606 .38017 Metetra
.757 .751 .847 r
.67606 .38017 .67944 .38518 .68631 .38049 .68295 .37551 Metetra
.755 .753 .85 r
.68295 .37551 .68631 .38049 .6932 .37584 .68985 .3709 Metetra
.754 .755 .853 r
.68985 .3709 .6932 .37584 .7001 .37125 .69676 .36635 Metetra
.753 .757 .855 r
.69676 .36635 .7001 .37125 .707 .3667 .70369 .36183 Metetra
.753 .758 .857 r
.70369 .36183 .707 .3667 .71392 .36219 .71062 .35736 Metetra
.752 .76 .859 r
.71062 .35736 .71392 .36219 .72084 .35771 .71756 .35291 Metetra
.751 .762 .862 r
.71756 .35291 .72084 .35771 .72778 .35327 .72451 .34849 Metetra
.75 .764 .864 r
.72451 .34849 .72778 .35327 .73473 .34885 .73147 .34411 Metetra
.75 .766 .866 r
.73147 .34411 .73473 .34885 .74168 .34446 .73845 .33975 Metetra
.75 .768 .868 r
.73845 .33975 .74168 .34446 .74865 .3401 .74543 .33541 Metetra
.749 .769 .869 r
.74543 .33541 .74865 .3401 .75563 .33575 .75242 .3311 Metetra
.749 .771 .871 r
.75242 .3311 .75563 .33575 .76262 .33143 .75943 .3268 Metetra
.749 .772 .872 r
.75943 .3268 .76262 .33143 .76961 .32711 .76644 .32251 Metetra
.749 .774 .874 r
.76644 .32251 .76961 .32711 .77662 .3228 .77346 .31822 Metetra
.749 .776 .876 r
.77346 .31822 .77662 .3228 .78364 .31851 .7805 .31395 Metetra
.749 .778 .877 r
.7805 .31395 .78364 .31851 .79066 .31421 .78754 .30968 Metetra
.749 .78 .878 r
.78754 .30968 .79066 .31421 .7977 .30993 .79459 .30541 Metetra
.749 .781 .88 r
.79459 .30541 .7977 .30993 .80475 .30564 .80166 .30115 Metetra
.749 .784 .881 r
.80166 .30115 .80475 .30564 .8118 .30135 .80873 .29688 Metetra
.749 .785 .882 r
.80873 .29688 .8118 .30135 .81887 .29706 .81581 .29262 Metetra
.748 .767 .869 r
.17232 .50166 .17637 .5052 .1823 .50152 .17826 .49801 Metetra
.748 .769 .87 r
.17826 .49801 .1823 .50152 .18824 .49784 .18421 .49435 Metetra
.749 .771 .871 r
.18421 .49435 .18824 .49784 .19419 .49414 .19016 .49069 Metetra
.75 .773 .872 r
.19016 .49069 .19419 .49414 .20015 .49044 .19613 .48701 Metetra
.75 .774 .873 r
.19613 .48701 .20015 .49044 .20611 .48672 .2021 .48332 Metetra
.752 .777 .873 r
.2021 .48332 .20611 .48672 .21208 .48299 .20808 .47961 Metetra
.752 .778 .874 r
.20808 .47961 .21208 .48299 .21806 .47925 .21406 .47589 Metetra
.753 .78 .875 r
.21406 .47589 .21806 .47925 .22405 .47548 .22006 .47215 Metetra
.755 .783 .876 r
.22006 .47215 .22405 .47548 .23005 .4717 .22606 .4684 Metetra
.756 .785 .877 r
.22606 .4684 .23005 .4717 .23605 .46789 .23207 .46462 Metetra
.757 .787 .877 r
.23207 .46462 .23605 .46789 .24207 .46405 .23809 .46081 Metetra
.759 .789 .878 r
.23809 .46081 .24207 .46405 .24809 .46019 .24412 .45698 Metetra
.76 .791 .878 r
.24412 .45698 .24809 .46019 .25412 .45628 .25016 .45311 Metetra
.763 .794 .879 r
.25016 .45311 .25412 .45628 .26016 .45235 .25621 .4492 Metetra
.764 .796 .879 r
.25621 .4492 .26016 .45235 .26621 .44837 .26227 .44527 Metetra
.766 .799 .879 r
.26227 .44527 .26621 .44837 .27227 .44436 .26834 .44129 Metetra
.768 .801 .88 r
.26834 .44129 .27227 .44436 .27834 .44031 .27441 .43727 Metetra
.771 .804 .88 r
.27441 .43727 .27834 .44031 .28441 .4362 .2805 .4332 Metetra
.773 .807 .88 r
.2805 .4332 .28441 .4362 .2905 .43204 .28659 .42909 Metetra
.776 .81 .88 r
.28659 .42909 .2905 .43204 .2966 .42783 .29269 .42491 Metetra
.779 .813 .88 r
.29269 .42491 .2966 .42783 .3027 .42355 .29881 .42068 Metetra
.782 .816 .88 r
.29881 .42068 .3027 .42355 .30882 .41922 .30493 .41639 Metetra
.785 .819 .879 r
.30493 .41639 .30882 .41922 .31494 .41481 .31106 .41203 Metetra
.789 .822 .879 r
.31106 .41203 .31494 .41481 .32107 .41032 .31721 .40758 Metetra
.792 .826 .878 r
.31721 .40758 .32107 .41032 .32722 .40573 .32336 .40306 Metetra
.797 .829 .877 r
.32336 .40306 .32722 .40573 .33337 .40107 .32952 .39844 Metetra
.801 .832 .875 r
.32952 .39844 .33337 .40107 .33953 .39629 .3357 .39371 Metetra
.806 .835 .873 r
.3357 .39371 .33953 .39629 .3457 .3914 .34188 .38887 Metetra
.811 .839 .871 r
.34188 .38887 .3457 .3914 .35189 .38639 .34807 .3839 Metetra
.817 .842 .868 r
.34807 .3839 .35189 .38639 .35808 .38123 .35428 .37878 Metetra
.823 .844 .864 r
.35428 .37878 .35808 .38123 .36428 .37591 .36049 .3735 Metetra
.829 .845 .86 r
.36049 .3735 .36428 .37591 .37049 .37043 .36672 .36803 Metetra
.835 .846 .854 r
.36672 .36803 .37049 .37043 .37671 .36475 .37295 .36235 Metetra
.842 .845 .847 r
.37295 .36235 .37671 .36475 .38294 .35885 .3792 .35641 Metetra
.848 .843 .839 r
.3792 .35641 .38294 .35885 .38918 .35272 .38546 .35021 Metetra
.854 .838 .829 r
.38546 .35021 .38918 .35272 .39543 .34636 .39173 .34371 Metetra
.858 .83 .819 r
.39173 .34371 .39543 .34636 .40169 .33975 .39801 .33689 Metetra
.86 .819 .808 r
.39801 .33689 .40169 .33975 .40795 .33292 .4043 .32976 Metetra
.859 .805 .797 r
.4043 .32976 .40795 .33292 .41421 .32592 .41059 .32238 Metetra
.854 .79 .788 r
.41059 .32238 .41421 .32592 .42047 .31882 .41688 .31486 Metetra
.764 .296 0 r
.41688 .31486 .42047 .31882 .42621 .32815 .42261 .32445 Metetra
.772 .298 0 r
.42261 .32445 .42621 .32815 .43201 .33748 .42839 .334 Metetra
.772 .293 0 r
.42839 .334 .43201 .33748 .43787 .34675 .43424 .34341 Metetra
.766 .283 0 r
.43424 .34341 .43787 .34675 .44379 .35592 .44016 .35263 Metetra
.757 .269 0 r
.44016 .35263 .44379 .35592 .44976 .36499 .44614 .36167 Metetra
.744 .254 0 r
.44614 .36167 .44976 .36499 .4558 .37396 .45217 .37056 Metetra
.73 .239 0 r
.45217 .37056 .4558 .37396 .4619 .38287 .45827 .37934 Metetra
.714 .224 0 r
.45827 .37934 .4619 .38287 .46806 .39175 .46443 .38806 Metetra
.698 .211 0 r
.46443 .38806 .46806 .39175 .47428 .40065 .47065 .39678 Metetra
.683 .2 0 r
.47065 .39678 .47428 .40065 .48055 .40961 .47692 .40556 Metetra
.671 .192 0 r
.47692 .40556 .48055 .40961 .48688 .41867 .48325 .41444 Metetra
.661 .188 0 r
.48325 .41444 .48688 .41867 .49327 .42789 .48964 .42349 Metetra
.656 .189 0 r
.48964 .42349 .49327 .42789 .49972 .43731 .49609 .43277 Metetra
.657 .195 0 r
.49609 .43277 .49972 .43731 .50623 .44697 .5026 .44233 Metetra
.664 .206 0 r
.5026 .44233 .50623 .44697 .51281 .45692 .50917 .45224 Metetra
.679 .222 0 r
.50917 .45224 .51281 .45692 .51944 .46717 .5158 .46254 Metetra
.698 .242 0 r
.5158 .46254 .51944 .46717 .52615 .47773 .5225 .47325 Metetra
.721 .262 0 r
.5225 .47325 .52615 .47773 .53292 .48858 .52927 .48434 Metetra
.742 .279 0 r
.52927 .48434 .53292 .48858 .53977 .49963 .53611 .4957 Metetra
.863 .802 .788 r
.53611 .4957 .53977 .49963 .54656 .49178 .54291 .48735 Metetra
.851 .783 .785 r
.54291 .48735 .54656 .49178 .55334 .48405 .54971 .47919 Metetra
.837 .766 .784 r
.54971 .47919 .55334 .48405 .56013 .47653 .55651 .47134 Metetra
.824 .753 .785 r
.55651 .47134 .56013 .47653 .56691 .46927 .5633 .46385 Metetra
.813 .743 .787 r
.5633 .46385 .56691 .46927 .57369 .4623 .57009 .45672 Metetra
.803 .736 .79 r
.57009 .45672 .57369 .4623 .58047 .4556 .57688 .44994 Metetra
.795 .732 .794 r
.57688 .44994 .58047 .4556 .58725 .44916 .58368 .44347 Metetra
.788 .73 .798 r
.58368 .44347 .58725 .44916 .59404 .44297 .59048 .43727 Metetra
.782 .728 .802 r
.59048 .43727 .59404 .44297 .60084 .43698 .59729 .43132 Metetra
.778 .728 .807 r
.59729 .43132 .60084 .43698 .60763 .43119 .6041 .42556 Metetra
.774 .729 .811 r
.6041 .42556 .60763 .43119 .61444 .42557 .61092 .41999 Metetra
.77 .73 .815 r
.61092 .41999 .61444 .42557 .62125 .4201 .61775 .41457 Metetra
.767 .731 .819 r
.61775 .41457 .62125 .4201 .62807 .41477 .62458 .40929 Metetra
.765 .733 .823 r
.62458 .40929 .62807 .41477 .6349 .40955 .63142 .40413 Metetra
.762 .735 .827 r
.63142 .40413 .6349 .40955 .64174 .40444 .63827 .39909 Metetra
.761 .737 .831 r
.63827 .39909 .64174 .40444 .64859 .39943 .64513 .39413 Metetra
.759 .739 .834 r
.64513 .39413 .64859 .39943 .65544 .3945 .652 .38926 Metetra
.757 .741 .838 r
.652 .38926 .65544 .3945 .6623 .38966 .65888 .38446 Metetra
.756 .743 .841 r
.65888 .38446 .6623 .38966 .66918 .38488 .66577 .37974 Metetra
.755 .745 .844 r
.66577 .37974 .66918 .38488 .67606 .38017 .67267 .37507 Metetra
.754 .747 .847 r
.67267 .37507 .67606 .38017 .68295 .37551 .67957 .37046 Metetra
.753 .75 .85 r
.67957 .37046 .68295 .37551 .68985 .3709 .68649 .36589 Metetra
.752 .752 .852 r
.68649 .36589 .68985 .3709 .69676 .36635 .69342 .36138 Metetra
.751 .754 .855 r
.69342 .36138 .69676 .36635 .70369 .36183 .70035 .3569 Metetra
.75 .756 .857 r
.70035 .3569 .70369 .36183 .71062 .35736 .7073 .35247 Metetra
.75 .758 .859 r
.7073 .35247 .71062 .35736 .71756 .35291 .71426 .34805 Metetra
.749 .76 .861 r
.71426 .34805 .71756 .35291 .72451 .34849 .72122 .34368 Metetra
.749 .762 .864 r
.72122 .34368 .72451 .34849 .73147 .34411 .7282 .33932 Metetra
.748 .763 .866 r
.7282 .33932 .73147 .34411 .73845 .33975 .73519 .33499 Metetra
.747 .765 .868 r
.73519 .33499 .73845 .33975 .74543 .33541 .74219 .33069 Metetra
.747 .768 .87 r
.74219 .33069 .74543 .33541 .75242 .3311 .7492 .3264 Metetra
.747 .769 .871 r
.7492 .3264 .75242 .3311 .75943 .3268 .75622 .32213 Metetra
.747 .771 .873 r
.75622 .32213 .75943 .3268 .76644 .32251 .76325 .31786 Metetra
.747 .773 .874 r
.76325 .31786 .76644 .32251 .77346 .31822 .77029 .31361 Metetra
.747 .775 .876 r
.77029 .31361 .77346 .31822 .7805 .31395 .77734 .30936 Metetra
.747 .777 .877 r
.77734 .30936 .7805 .31395 .78754 .30968 .7844 .30511 Metetra
.747 .779 .879 r
.7844 .30511 .78754 .30968 .79459 .30541 .79147 .30087 Metetra
.747 .781 .88 r
.79147 .30087 .79459 .30541 .80166 .30115 .79855 .29663 Metetra
.747 .783 .882 r
.79855 .29663 .80166 .30115 .80873 .29688 .80564 .29239 Metetra
.748 .785 .883 r
.80564 .29239 .80873 .29688 .81581 .29262 .81274 .28815 Metetra
.746 .767 .87 r
.16825 .49812 .17232 .50166 .17826 .49801 .1742 .49449 Metetra
.747 .769 .871 r
.1742 .49449 .17826 .49801 .18421 .49435 .18015 .49086 Metetra
.748 .771 .872 r
.18015 .49086 .18421 .49435 .19016 .49069 .18612 .48722 Metetra
.748 .773 .873 r
.18612 .48722 .19016 .49069 .19613 .48701 .19208 .48357 Metetra
.749 .775 .874 r
.19208 .48357 .19613 .48701 .2021 .48332 .19806 .47991 Metetra
.75 .777 .875 r
.19806 .47991 .2021 .48332 .20808 .47961 .20405 .47623 Metetra
.751 .779 .876 r
.20405 .47623 .20808 .47961 .21406 .47589 .21004 .47254 Metetra
.752 .781 .877 r
.21004 .47254 .21406 .47589 .22006 .47215 .21604 .46883 Metetra
.753 .783 .878 r
.21604 .46883 .22006 .47215 .22606 .4684 .22205 .46511 Metetra
.754 .786 .878 r
.22205 .46511 .22606 .4684 .23207 .46462 .22807 .46136 Metetra
.756 .788 .879 r
.22807 .46136 .23207 .46462 .23809 .46081 .2341 .45758 Metetra
.757 .79 .88 r
.2341 .45758 .23809 .46081 .24412 .45698 .24014 .45378 Metetra
.759 .793 .88 r
.24014 .45378 .24412 .45698 .25016 .45311 .24618 .44995 Metetra
.761 .795 .881 r
.24618 .44995 .25016 .45311 .25621 .4492 .25224 .44608 Metetra
.763 .798 .881 r
.25224 .44608 .25621 .4492 .26227 .44527 .25831 .44218 Metetra
.765 .801 .882 r
.25831 .44218 .26227 .44527 .26834 .44129 .26438 .43824 Metetra
.767 .803 .883 r
.26438 .43824 .26834 .44129 .27441 .43727 .27046 .43426 Metetra
.769 .807 .883 r
.27046 .43426 .27441 .43727 .2805 .4332 .27656 .43024 Metetra
.772 .81 .883 r
.27656 .43024 .2805 .4332 .28659 .42909 .28266 .42617 Metetra
.774 .813 .883 r
.28266 .42617 .28659 .42909 .29269 .42491 .28877 .42205 Metetra
.777 .816 .883 r
.28877 .42205 .29269 .42491 .29881 .42068 .29489 .41786 Metetra
.78 .819 .883 r
.29489 .41786 .29881 .42068 .30493 .41639 .30103 .41362 Metetra
.784 .823 .883 r
.30103 .41362 .30493 .41639 .31106 .41203 .30717 .40931 Metetra
.787 .827 .883 r
.30717 .40931 .31106 .41203 .31721 .40758 .31332 .40493 Metetra
.791 .831 .882 r
.31332 .40493 .31721 .40758 .32336 .40306 .31948 .40045 Metetra
.795 .834 .881 r
.31948 .40045 .32336 .40306 .32952 .39844 .32565 .3959 Metetra
.8 .838 .88 r
.32565 .3959 .32952 .39844 .3357 .39371 .33184 .39123 Metetra
.805 .842 .879 r
.33184 .39123 .3357 .39371 .34188 .38887 .33803 .38646 Metetra
.811 .846 .877 r
.33803 .38646 .34188 .38887 .34807 .3839 .34423 .38155 Metetra
.816 .85 .874 r
.34423 .38155 .34807 .3839 .35428 .37878 .35045 .37649 Metetra
.823 .853 .87 r
.35045 .37649 .35428 .37878 .36049 .3735 .35668 .37126 Metetra
.83 .856 .866 r
.35668 .37126 .36049 .3735 .36672 .36803 .36291 .36583 Metetra
.838 .858 .86 r
.36291 .36583 .36672 .36803 .37295 .36235 .36917 .36018 Metetra
.846 .858 .852 r
.36917 .36018 .37295 .36235 .3792 .35641 .37543 .35424 Metetra
.854 .857 .843 r
.37543 .35424 .3792 .35641 .38546 .35021 .38171 .34798 Metetra
.862 .852 .832 r
.38171 .34798 .38546 .35021 .39173 .34371 .388 .34136 Metetra
.868 .844 .819 r
.388 .34136 .39173 .34371 .39801 .33689 .39431 .33432 Metetra
.872 .832 .804 r
.39431 .33432 .39801 .33689 .4043 .32976 .40062 .32685 Metetra
.872 .815 .79 r
.40062 .32685 .4043 .32976 .41059 .32238 .40695 .31898 Metetra
.866 .794 .779 r
.40695 .31898 .41059 .32238 .41688 .31486 .41328 .31087 Metetra
.782 .318 0 r
.41328 .31087 .41688 .31486 .42261 .32445 .41898 .32088 Metetra
.791 .32 0 r
.41898 .32088 .42261 .32445 .42839 .334 .42476 .33074 Metetra
.789 .31 0 r
.42476 .33074 .42839 .334 .43424 .34341 .4306 .34032 Metetra
.78 .295 0 r
.4306 .34032 .43424 .34341 .44016 .35263 .43651 .34959 Metetra
.766 .276 0 r
.43651 .34959 .44016 .35263 .44614 .36167 .44248 .35857 Metetra
.75 .255 0 r
.44248 .35857 .44614 .36167 .45217 .37056 .44852 .36733 Metetra
.731 .235 0 r
.44852 .36733 .45217 .37056 .45827 .37934 .45463 .37593 Metetra
.711 .216 0 r
.45463 .37593 .45827 .37934 .46443 .38806 .46079 .38444 Metetra
.69 .198 0 r
.46079 .38444 .46443 .38806 .47065 .39678 .467 .39293 Metetra
.67 .184 0 r
.467 .39293 .47065 .39678 .47692 .40556 .47328 .40146 Metetra
.652 .174 0 r
.47328 .40146 .47692 .40556 .48325 .41444 .47961 .4101 Metetra
.639 .168 0 r
.47961 .4101 .48325 .41444 .48964 .42349 .486 .41893 Metetra
.631 .168 0 r
.486 .41893 .48964 .42349 .49609 .43277 .49244 .428 Metetra
.63 .175 0 r
.49244 .428 .49609 .43277 .5026 .44233 .49895 .43742 Metetra
.639 .189 0 r
.49895 .43742 .5026 .44233 .50917 .45224 .50551 .44725 Metetra
.658 .211 0 r
.50551 .44725 .50917 .45224 .5158 .46254 .51214 .45759 Metetra
.686 .238 0 r
.51214 .45759 .5158 .46254 .5225 .47325 .51883 .46848 Metetra
.719 .268 0 r
.51883 .46848 .5225 .47325 .52927 .48434 .52559 .47992 Metetra
.751 .294 0 r
.52559 .47992 .52927 .48434 .53611 .4957 .53243 .49175 Metetra
.873 .8 .776 r
.53243 .49175 .53611 .4957 .54291 .48735 .53925 .48273 Metetra
.856 .776 .772 r
.53925 .48273 .54291 .48735 .54971 .47919 .54607 .47401 Metetra
.838 .756 .772 r
.54607 .47401 .54971 .47919 .55651 .47134 .55287 .46577 Metetra
.822 .741 .775 r
.55287 .46577 .55651 .47134 .5633 .46385 .55967 .45804 Metetra
.808 .73 .779 r
.55967 .45804 .5633 .46385 .57009 .45672 .56648 .45079 Metetra
.798 .724 .783 r
.56648 .45079 .57009 .45672 .57688 .44994 .57328 .44395 Metetra
.789 .72 .788 r
.57328 .44395 .57688 .44994 .58368 .44347 .58009 .43748 Metetra
.782 .719 .793 r
.58009 .43748 .58368 .44347 .59048 .43727 .5869 .43132 Metetra
.776 .719 .798 r
.5869 .43132 .59048 .43727 .59729 .43132 .59372 .42542 Metetra
.772 .719 .803 r
.59372 .42542 .59729 .43132 .6041 .42556 .60055 .41973 Metetra
.768 .721 .808 r
.60055 .41973 .6041 .42556 .61092 .41999 .60738 .41422 Metetra
.765 .722 .813 r
.60738 .41422 .61092 .41999 .61775 .41457 .61422 .40888 Metetra
.762 .725 .818 r
.61422 .40888 .61775 .41457 .62458 .40929 .62107 .40368 Metetra
.76 .727 .822 r
.62107 .40368 .62458 .40929 .63142 .40413 .62793 .39859 Metetra
.758 .729 .826 r
.62793 .39859 .63142 .40413 .63827 .39909 .63479 .3936 Metetra
.756 .732 .83 r
.63479 .3936 .63827 .39909 .64513 .39413 .64167 .38872 Metetra
.755 .734 .833 r
.64167 .38872 .64513 .39413 .652 .38926 .64855 .3839 Metetra
.754 .737 .837 r
.64855 .3839 .652 .38926 .65888 .38446 .65544 .37917 Metetra
.753 .739 .84 r
.65544 .37917 .65888 .38446 .66577 .37974 .66234 .3745 Metetra
.751 .741 .843 r
.66234 .3745 .66577 .37974 .67267 .37507 .66926 .36989 Metetra
.751 .744 .846 r
.66926 .36989 .67267 .37507 .67957 .37046 .67618 .36532 Metetra
.749 .746 .849 r
.67618 .36532 .67957 .37046 .68649 .36589 .68311 .36081 Metetra
.749 .749 .852 r
.68311 .36081 .68649 .36589 .69342 .36138 .69005 .35634 Metetra
.748 .751 .854 r
.69005 .35634 .69342 .36138 .70035 .3569 .697 .35191 Metetra
.747 .753 .857 r
.697 .35191 .70035 .3569 .7073 .35247 .70397 .34751 Metetra
.747 .755 .859 r
.70397 .34751 .7073 .35247 .71426 .34805 .71094 .34314 Metetra
.747 .758 .861 r
.71094 .34314 .71426 .34805 .72122 .34368 .71792 .3388 Metetra
.746 .759 .864 r
.71792 .3388 .72122 .34368 .7282 .33932 .72492 .33448 Metetra
.746 .762 .866 r
.72492 .33448 .7282 .33932 .73519 .33499 .73192 .33019 Metetra
.745 .764 .868 r
.73192 .33019 .73519 .33499 .74219 .33069 .73894 .32592 Metetra
.745 .766 .87 r
.73894 .32592 .74219 .33069 .7492 .3264 .74596 .32166 Metetra
.745 .768 .871 r
.74596 .32166 .7492 .3264 .75622 .32213 .753 .31742 Metetra
.745 .77 .873 r
.753 .31742 .75622 .32213 .76325 .31786 .76005 .31319 Metetra
.745 .772 .875 r
.76005 .31319 .76325 .31786 .77029 .31361 .7671 .30896 Metetra
.745 .774 .876 r
.7671 .30896 .77029 .31361 .77734 .30936 .77417 .30474 Metetra
.745 .776 .878 r
.77417 .30474 .77734 .30936 .7844 .30511 .78125 .30052 Metetra
.745 .778 .879 r
.78125 .30052 .7844 .30511 .79147 .30087 .78834 .29631 Metetra
.745 .78 .881 r
.78834 .29631 .79147 .30087 .79855 .29663 .79543 .2921 Metetra
.746 .782 .882 r
.79543 .2921 .79855 .29663 .80564 .29239 .80254 .28788 Metetra
.746 .784 .884 r
.80254 .28788 .80564 .29239 .81274 .28815 .80966 .28366 Metetra
.745 .767 .871 r
.16417 .49455 .16825 .49812 .1742 .49449 .17012 .49096 Metetra
.745 .769 .872 r
.17012 .49096 .1742 .49449 .18015 .49086 .17608 .48736 Metetra
.746 .771 .873 r
.17608 .48736 .18015 .49086 .18612 .48722 .18205 .48375 Metetra
.747 .773 .874 r
.18205 .48375 .18612 .48722 .19208 .48357 .18803 .48013 Metetra
.748 .775 .875 r
.18803 .48013 .19208 .48357 .19806 .47991 .19401 .47649 Metetra
.749 .777 .876 r
.19401 .47649 .19806 .47991 .20405 .47623 .2 .47285 Metetra
.749 .779 .877 r
.2 .47285 .20405 .47623 .21004 .47254 .206 .46919 Metetra
.751 .782 .878 r
.206 .46919 .21004 .47254 .21604 .46883 .21201 .46551 Metetra
.752 .784 .879 r
.21201 .46551 .21604 .46883 .22205 .46511 .21803 .46182 Metetra
.753 .786 .88 r
.21803 .46182 .22205 .46511 .22807 .46136 .22405 .4581 Metetra
.754 .789 .881 r
.22405 .4581 .22807 .46136 .2341 .45758 .23009 .45436 Metetra
.756 .791 .882 r
.23009 .45436 .2341 .45758 .24014 .45378 .23613 .4506 Metetra
.757 .794 .883 r
.23613 .4506 .24014 .45378 .24618 .44995 .24219 .4468 Metetra
.759 .797 .883 r
.24219 .4468 .24618 .44995 .25224 .44608 .24825 .44297 Metetra
.761 .799 .884 r
.24825 .44297 .25224 .44608 .25831 .44218 .25432 .43911 Metetra
.763 .803 .885 r
.25432 .43911 .25831 .44218 .26438 .43824 .2604 .43522 Metetra
.765 .805 .885 r
.2604 .43522 .26438 .43824 .27046 .43426 .26649 .43128 Metetra
.767 .808 .886 r
.26649 .43128 .27046 .43426 .27656 .43024 .27259 .42731 Metetra
.77 .812 .886 r
.27259 .42731 .27656 .43024 .28266 .42617 .27871 .42329 Metetra
.773 .815 .887 r
.27871 .42329 .28266 .42617 .28877 .42205 .28483 .41921 Metetra
.775 .819 .887 r
.28483 .41921 .28877 .42205 .29489 .41786 .29096 .41509 Metetra
.778 .823 .887 r
.29096 .41509 .29489 .41786 .30103 .41362 .2971 .4109 Metetra
.782 .827 .887 r
.2971 .4109 .30103 .41362 .30717 .40931 .30325 .40665 Metetra
.785 .831 .887 r
.30325 .40665 .30717 .40931 .31332 .40493 .30941 .40233 Metetra
.789 .835 .887 r
.30941 .40233 .31332 .40493 .31948 .40045 .31558 .39793 Metetra
.794 .84 .887 r
.31558 .39793 .31948 .40045 .32565 .3959 .32176 .39344 Metetra
.798 .844 .886 r
.32176 .39344 .32565 .3959 .33184 .39123 .32795 .38885 Metetra
.803 .849 .885 r
.32795 .38885 .33184 .39123 .33803 .38646 .33415 .38415 Metetra
.809 .854 .883 r
.33415 .38415 .33803 .38646 .34423 .38155 .34036 .37933 Metetra
.815 .859 .88 r
.34036 .37933 .34423 .38155 .35045 .37649 .34659 .37435 Metetra
.822 .863 .877 r
.34659 .37435 .35045 .37649 .35668 .37126 .35283 .36921 Metetra
.83 .868 .873 r
.35283 .36921 .35668 .37126 .36291 .36583 .35908 .36386 Metetra
.839 .871 .867 r
.35908 .36386 .36291 .36583 .36917 .36018 .36534 .35827 Metetra
.849 .874 .859 r
.36534 .35827 .36917 .36018 .37543 .35424 .37162 .35238 Metetra
.859 .874 .849 r
.37162 .35238 .37543 .35424 .38171 .34798 .37792 .34612 Metetra
.869 .871 .836 r
.37792 .34612 .38171 .34798 .388 .34136 .38423 .33943 Metetra
.879 .863 .82 r
.38423 .33943 .388 .34136 .39431 .33432 .39056 .33219 Metetra
.886 .849 .801 r
.39056 .33219 .39431 .33432 .40062 .32685 .39691 .32433 Metetra
.888 .828 .782 r
.39691 .32433 .40062 .32685 .40695 .31898 .40328 .31583 Metetra
.882 .801 .766 r
.40328 .31583 .40695 .31898 .41328 .31087 .40966 .30687 Metetra
.806 .35 0 r
.40966 .30687 .41328 .31087 .41898 .32088 .41533 .31754 Metetra
.815 .347 0 r
.41533 .31754 .41898 .32088 .42476 .33074 .42109 .32785 Metetra
.808 .33 0 r
.42109 .32785 .42476 .33074 .4306 .34032 .42692 .33763 Metetra
.794 .307 0 r
.42692 .33763 .4306 .34032 .43651 .34959 .43283 .34691 Metetra
.776 .281 0 r
.43283 .34691 .43651 .34959 .44248 .35857 .43881 .35577 Metetra
.754 .254 0 r
.43881 .35577 .44248 .35857 .44852 .36733 .44485 .36432 Metetra
.731 .228 0 r
.44485 .36432 .44852 .36733 .45463 .37593 .45096 .37267 Metetra
.705 .204 0 r
.45096 .37267 .45463 .37593 .46079 .38444 .45712 .3809 Metetra
.678 .182 0 r
.45712 .3809 .46079 .38444 .467 .39293 .46334 .38908 Metetra
.652 .163 0 r
.46334 .38908 .467 .39293 .47328 .40146 .46962 .39731 Metetra
.628 .149 0 r
.46962 .39731 .47328 .40146 .47961 .4101 .47595 .40565 Metetra
.608 .14 0 r
.47595 .40565 .47961 .4101 .486 .41893 .48234 .41417 Metetra
.594 .138 0 r
.48234 .41417 .486 .41893 .49244 .428 .48878 .42297 Metetra
.591 .145 0 r
.48878 .42297 .49244 .428 .49895 .43742 .49528 .43216 Metetra
.599 .162 0 r
.49528 .43216 .49895 .43742 .50551 .44725 .50184 .44184 Metetra
.623 .19 0 r
.50184 .44184 .50551 .44725 .51214 .45759 .50846 .45217 Metetra
.663 .229 0 r
.50846 .45217 .51214 .45759 .51883 .46848 .51515 .46327 Metetra
.714 .275 0 r
.51515 .46327 .51883 .46848 .52559 .47992 .52191 .47519 Metetra
.763 .316 0 r
.52191 .47519 .52559 .47992 .53243 .49175 .52874 .48778 Metetra
.885 .798 .758 r
.52874 .48778 .53243 .49175 .53925 .48273 .53558 .47779 Metetra
.86 .766 .757 r
.53558 .47779 .53925 .48273 .54607 .47401 .5424 .46836 Metetra
.836 .741 .759 r
.5424 .46836 .54607 .47401 .55287 .46577 .54922 .45968 Metetra
.816 .724 .764 r
.54922 .45968 .55287 .46577 .55967 .45804 .55603 .45174 Metetra
.801 .715 .77 r
.55603 .45174 .55967 .45804 .56648 .45079 .56285 .44441 Metetra
.789 .709 .776 r
.56285 .44441 .56648 .45079 .57328 .44395 .56966 .43759 Metetra
.781 .707 .782 r
.56966 .43759 .57328 .44395 .58009 .43748 .57648 .43117 Metetra
.774 .707 .789 r
.57648 .43117 .58009 .43748 .5869 .43132 .58331 .42509 Metetra
.769 .708 .795 r
.58331 .42509 .5869 .43132 .59372 .42542 .59014 .41928 Metetra
.765 .71 .8 r
.59014 .41928 .59372 .42542 .60055 .41973 .59698 .41369 Metetra
.762 .712 .806 r
.59698 .41369 .60055 .41973 .60738 .41422 .60383 .40829 Metetra
.759 .715 .811 r
.60383 .40829 .60738 .41422 .61422 .40888 .61068 .40303 Metetra
.757 .718 .816 r
.61068 .40303 .61422 .40888 .62107 .40368 .61754 .39792 Metetra
.755 .721 .82 r
.61754 .39792 .62107 .40368 .62793 .39859 .62441 .39291 Metetra
.753 .724 .825 r
.62441 .39291 .62793 .39859 .63479 .3936 .63129 .38801 Metetra
.752 .727 .829 r
.63129 .38801 .63479 .3936 .64167 .38872 .63818 .38319 Metetra
.751 .729 .832 r
.63818 .38319 .64167 .38872 .64855 .3839 .64508 .37846 Metetra
.75 .732 .836 r
.64508 .37846 .64855 .3839 .65544 .37917 .65199 .37379 Metetra
.749 .735 .84 r
.65199 .37379 .65544 .37917 .66234 .3745 .6589 .36918 Metetra
.748 .738 .843 r
.6589 .36918 .66234 .3745 .66926 .36989 .66583 .36463 Metetra
.747 .74 .846 r
.66583 .36463 .66926 .36989 .67618 .36532 .67277 .36012 Metetra
.747 .743 .849 r
.67277 .36012 .67618 .36532 .68311 .36081 .67971 .35566 Metetra
.746 .746 .852 r
.67971 .35566 .68311 .36081 .69005 .35634 .68667 .35124 Metetra
.745 .748 .854 r
.68667 .35124 .69005 .35634 .697 .35191 .69364 .34686 Metetra
.745 .75 .857 r
.69364 .34686 .697 .35191 .70397 .34751 .70062 .34251 Metetra
.745 .753 .859 r
.70062 .34251 .70397 .34751 .71094 .34314 .70761 .33818 Metetra
.744 .755 .861 r
.70761 .33818 .71094 .34314 .71792 .3388 .7146 .33388 Metetra
.744 .758 .864 r
.7146 .33388 .71792 .3388 .72492 .33448 .72161 .3296 Metetra
.743 .759 .866 r
.72161 .3296 .72492 .33448 .73192 .33019 .72864 .32534 Metetra
.743 .762 .868 r
.72864 .32534 .73192 .33019 .73894 .32592 .73567 .32111 Metetra
.743 .764 .87 r
.73567 .32111 .73894 .32592 .74596 .32166 .74271 .31689 Metetra
.743 .766 .872 r
.74271 .31689 .74596 .32166 .753 .31742 .74976 .31268 Metetra
.743 .768 .874 r
.74976 .31268 .753 .31742 .76005 .31319 .75683 .30847 Metetra
.743 .77 .875 r
.75683 .30847 .76005 .31319 .7671 .30896 .7639 .30428 Metetra
.743 .772 .877 r
.7639 .30428 .7671 .30896 .77417 .30474 .77098 .30009 Metetra
.743 .775 .878 r
.77098 .30009 .77417 .30474 .78125 .30052 .77808 .2959 Metetra
.743 .777 .88 r
.77808 .2959 .78125 .30052 .78834 .29631 .78518 .29171 Metetra
.744 .779 .882 r
.78518 .29171 .78834 .29631 .79543 .2921 .7923 .28753 Metetra
.744 .781 .883 r
.7923 .28753 .79543 .2921 .80254 .28788 .79943 .28334 Metetra
.744 .783 .884 r
.79943 .28334 .80254 .28788 .80966 .28366 .80656 .27915 Metetra
.743 .767 .872 r
.16007 .49097 .16417 .49455 .17012 .49096 .16603 .48741 Metetra
.744 .769 .873 r
.16603 .48741 .17012 .49096 .17608 .48736 .17199 .48384 Metetra
.744 .771 .875 r
.17199 .48384 .17608 .48736 .18205 .48375 .17797 .48026 Metetra
.745 .773 .876 r
.17797 .48026 .18205 .48375 .18803 .48013 .18395 .47668 Metetra
.746 .776 .877 r
.18395 .47668 .18803 .48013 .19401 .47649 .18994 .47307 Metetra
.747 .778 .878 r
.18994 .47307 .19401 .47649 .2 .47285 .19594 .46946 Metetra
.748 .78 .879 r
.19594 .46946 .2 .47285 .206 .46919 .20194 .46583 Metetra
.749 .782 .88 r
.20194 .46583 .206 .46919 .21201 .46551 .20796 .46219 Metetra
.75 .785 .881 r
.20796 .46219 .21201 .46551 .21803 .46182 .21398 .45853 Metetra
.751 .787 .882 r
.21398 .45853 .21803 .46182 .22405 .4581 .22001 .45485 Metetra
.753 .79 .883 r
.22001 .45485 .22405 .4581 .23009 .45436 .22606 .45115 Metetra
.754 .792 .884 r
.22606 .45115 .23009 .45436 .23613 .4506 .23211 .44742 Metetra
.756 .795 .885 r
.23211 .44742 .23613 .4506 .24219 .4468 .23817 .44366 Metetra
.757 .798 .886 r
.23817 .44366 .24219 .4468 .24825 .44297 .24424 .43988 Metetra
.759 .801 .887 r
.24424 .43988 .24825 .44297 .25432 .43911 .25032 .43606 Metetra
.761 .804 .887 r
.25032 .43606 .25432 .43911 .2604 .43522 .25641 .43221 Metetra
.763 .807 .888 r
.25641 .43221 .2604 .43522 .26649 .43128 .2625 .42833 Metetra
.765 .811 .889 r
.2625 .42833 .26649 .43128 .27259 .42731 .26861 .4244 Metetra
.768 .814 .89 r
.26861 .4244 .27259 .42731 .27871 .42329 .27473 .42043 Metetra
.77 .818 .89 r
.27473 .42043 .27871 .42329 .28483 .41921 .28086 .41642 Metetra
.773 .822 .891 r
.28086 .41642 .28483 .41921 .29096 .41509 .287 .41235 Metetra
.776 .826 .891 r
.287 .41235 .29096 .41509 .2971 .4109 .29314 .40823 Metetra
.779 .83 .892 r
.29314 .40823 .2971 .4109 .30325 .40665 .2993 .40405 Metetra
.783 .835 .892 r
.2993 .40405 .30325 .40665 .30941 .40233 .30547 .3998 Metetra
.787 .84 .892 r
.30547 .3998 .30941 .40233 .31558 .39793 .31165 .39547 Metetra
.791 .844 .892 r
.31165 .39547 .31558 .39793 .32176 .39344 .31784 .39107 Metetra
.795 .85 .892 r
.31784 .39107 .32176 .39344 .32795 .38885 .32404 .38657 Metetra
.8 .856 .891 r
.32404 .38657 .32795 .38885 .33415 .38415 .33025 .38197 Metetra
.806 .862 .89 r
.33025 .38197 .33415 .38415 .34036 .37933 .33647 .37724 Metetra
.813 .868 .888 r
.33647 .37724 .34036 .37933 .34659 .37435 .3427 .37238 Metetra
.82 .874 .885 r
.3427 .37238 .34659 .37435 .35283 .36921 .34895 .36735 Metetra
.828 .88 .881 r
.34895 .36735 .35283 .36921 .35908 .36386 .35521 .36212 Metetra
.838 .886 .876 r
.35521 .36212 .35908 .36386 .36534 .35827 .36148 .35665 Metetra
.849 .891 .868 r
.36148 .35665 .36534 .35827 .37162 .35238 .36777 .35087 Metetra
.861 .894 .857 r
.36777 .35087 .37162 .35238 .37792 .34612 .37408 .3447 Metetra
.875 .894 .843 r
.37408 .3447 .37792 .34612 .38423 .33943 .38041 .33802 Metetra
.89 .888 .823 r
.38041 .33802 .38423 .33943 .39056 .33219 .38677 .33067 Metetra
.903 .874 .799 r
.38677 .33067 .39056 .33219 .39691 .32433 .39316 .32244 Metetra
.91 .849 .771 r
.39316 .32244 .39691 .32433 .40328 .31583 .39958 .31313 Metetra
.904 .812 .746 r
.39958 .31313 .40328 .31583 .40966 .30687 .40602 .30286 Metetra
.839 .395 0 r
.40602 .30286 .40966 .30687 .41533 .31754 .41165 .31464 Metetra
.842 .381 0 r
.41165 .31464 .41533 .31754 .42109 .32785 .41738 .32557 Metetra
.826 .351 0 r
.41738 .32557 .42109 .32785 .42692 .33763 .4232 .33552 Metetra
.805 .317 0 r
.4232 .33552 .42692 .33763 .43283 .34691 .42911 .34471 Metetra
.782 .283 0 r
.42911 .34471 .43283 .34691 .43881 .35577 .4351 .35333 Metetra
.756 .249 0 r
.4351 .35333 .43881 .35577 .44485 .36432 .44115 .36158 Metetra
.727 .217 0 r
.44115 .36158 .44485 .36432 .45096 .37267 .44726 .36958 Metetra
.695 .186 0 r
.44726 .36958 .45096 .37267 .45712 .3809 .45343 .37745 Metetra
.661 .159 0 r
.45343 .37745 .45712 .3809 .46334 .38908 .45966 .38526 Metetra
.627 .135 0 r
.45966 .38526 .46334 .38908 .46962 .39731 .46594 .39311 Metetra
.594 .116 0 r
.46594 .39311 .46962 .39731 .47595 .40565 .47228 .40106 Metetra
.566 .103 0 r
.47228 .40106 .47595 .40565 .48234 .41417 .47867 .4092 Metetra
0 0 .446 r
.47867 .4092 .48234 .41417 .48878 .42297 .48512 .41763 Metetra
0 0 .451 r
.48512 .41763 .48878 .42297 .49528 .43216 .49161 .42647 Metetra
0 0 .44 r
.49161 .42647 .49528 .43216 .50184 .44184 .49817 .43588 Metetra
0 0 .409 r
.49817 .43588 .50184 .44184 .50846 .45217 .50478 .44607 Metetra
.618 .208 0 r
.50478 .44607 .50846 .45217 .51515 .46327 .51146 .45734 Metetra
.696 .278 0 r
.51146 .45734 .51515 .46327 .52191 .47519 .51821 .46993 Metetra
.778 .348 0 r
.51821 .46993 .52191 .47519 .52874 .48778 .52503 .4838 Metetra
.901 .794 .734 r
.52503 .4838 .52874 .48778 .53558 .47779 .53188 .47228 Metetra
.861 .749 .737 r
.53188 .47228 .53558 .47779 .5424 .46836 .53872 .46194 Metetra
.828 .719 .744 r
.53872 .46194 .5424 .46836 .54922 .45968 .54555 .45286 Metetra
.805 .702 .752 r
.54555 .45286 .54922 .45968 .55603 .45174 .55237 .4448 Metetra
.789 .695 .76 r
.55237 .4448 .55603 .45174 .56285 .44441 .5592 .43751 Metetra
.778 .692 .768 r
.5592 .43751 .56285 .44441 .56966 .43759 .56603 .43079 Metetra
.771 .692 .777 r
.56603 .43079 .56966 .43759 .57648 .43117 .57286 .42451 Metetra
.765 .694 .784 r
.57286 .42451 .57648 .43117 .58331 .42509 .5797 .41857 Metetra
.761 .697 .791 r
.5797 .41857 .58331 .42509 .59014 .41928 .58654 .4129 Metetra
.758 .7 .798 r
.58654 .4129 .59014 .41928 .59698 .41369 .59339 .40744 Metetra
.755 .704 .804 r
.59339 .40744 .59698 .41369 .60383 .40829 .60025 .40216 Metetra
.753 .708 .809 r
.60025 .40216 .60383 .40829 .61068 .40303 .60712 .39703 Metetra
.751 .711 .814 r
.60712 .39703 .61068 .40303 .61754 .39792 .614 .39202 Metetra
.75 .715 .819 r
.614 .39202 .61754 .39792 .62441 .39291 .62088 .38712 Metetra
.749 .719 .824 r
.62088 .38712 .62441 .39291 .63129 .38801 .62778 .3823 Metetra
.747 .722 .828 r
.62778 .3823 .63129 .38801 .63818 .38319 .63468 .37758 Metetra
.747 .725 .832 r
.63468 .37758 .63818 .38319 .64508 .37846 .64159 .37292 Metetra
.746 .728 .836 r
.64159 .37292 .64508 .37846 .65199 .37379 .64851 .36832 Metetra
.745 .731 .839 r
.64851 .36832 .65199 .37379 .6589 .36918 .65545 .36379 Metetra
.744 .734 .842 r
.65545 .36379 .6589 .36918 .66583 .36463 .66239 .3593 Metetra
.744 .737 .846 r
.66239 .3593 .66583 .36463 .67277 .36012 .66934 .35485 Metetra
.743 .74 .849 r
.66934 .35485 .67277 .36012 .67971 .35566 .6763 .35045 Metetra
.743 .743 .852 r
.6763 .35045 .67971 .35566 .68667 .35124 .68327 .34608 Metetra
.742 .745 .854 r
.68327 .34608 .68667 .35124 .69364 .34686 .69026 .34175 Metetra
.742 .748 .857 r
.69026 .34175 .69364 .34686 .70062 .34251 .69725 .33744 Metetra
.741 .75 .859 r
.69725 .33744 .70062 .34251 .70761 .33818 .70426 .33317 Metetra
.742 .753 .862 r
.70426 .33317 .70761 .33818 .7146 .33388 .71127 .3289 Metetra
.741 .755 .864 r
.71127 .3289 .7146 .33388 .72161 .3296 .7183 .32467 Metetra
.741 .758 .866 r
.7183 .32467 .72161 .3296 .72864 .32534 .72533 .32045 Metetra
.741 .76 .868 r
.72533 .32045 .72864 .32534 .73567 .32111 .73238 .31625 Metetra
.741 .762 .87 r
.73238 .31625 .73567 .32111 .74271 .31689 .73944 .31207 Metetra
.741 .765 .872 r
.73944 .31207 .74271 .31689 .74976 .31268 .74651 .30789 Metetra
.741 .767 .874 r
.74651 .30789 .74976 .31268 .75683 .30847 .75359 .30372 Metetra
.741 .769 .876 r
.75359 .30372 .75683 .30847 .7639 .30428 .76068 .29956 Metetra
.741 .771 .877 r
.76068 .29956 .7639 .30428 .77098 .30009 .76778 .2954 Metetra
.741 .773 .879 r
.76778 .2954 .77098 .30009 .77808 .2959 .7749 .29125 Metetra
.741 .776 .881 r
.7749 .29125 .77808 .2959 .78518 .29171 .78202 .28709 Metetra
.742 .778 .882 r
.78202 .28709 .78518 .29171 .7923 .28753 .78915 .28294 Metetra
.742 .78 .884 r
.78915 .28294 .7923 .28753 .79943 .28334 .7963 .27878 Metetra
.742 .782 .885 r
.7963 .27878 .79943 .28334 .80656 .27915 .80345 .27463 Metetra
.741 .767 .873 r
.15596 .48738 .16007 .49097 .16603 .48741 .16192 .48385 Metetra
.742 .769 .875 r
.16192 .48385 .16603 .48741 .17199 .48384 .16789 .48031 Metetra
.743 .771 .876 r
.16789 .48031 .17199 .48384 .17797 .48026 .17387 .47677 Metetra
.744 .773 .877 r
.17387 .47677 .17797 .48026 .18395 .47668 .17985 .47321 Metetra
.744 .775 .878 r
.17985 .47321 .18395 .47668 .18994 .47307 .18585 .46965 Metetra
.745 .778 .88 r
.18585 .46965 .18994 .47307 .19594 .46946 .19185 .46607 Metetra
.746 .78 .881 r
.19185 .46607 .19594 .46946 .20194 .46583 .19786 .46248 Metetra
.747 .783 .882 r
.19786 .46248 .20194 .46583 .20796 .46219 .20389 .45887 Metetra
.748 .785 .883 r
.20389 .45887 .20796 .46219 .21398 .45853 .20992 .45524 Metetra
.749 .788 .884 r
.20992 .45524 .21398 .45853 .22001 .45485 .21595 .4516 Metetra
.751 .791 .885 r
.21595 .4516 .22001 .45485 .22606 .45115 .222 .44794 Metetra
.752 .793 .886 r
.222 .44794 .22606 .45115 .23211 .44742 .22806 .44425 Metetra
.754 .796 .887 r
.22806 .44425 .23211 .44742 .23817 .44366 .23413 .44054 Metetra
.756 .799 .888 r
.23413 .44054 .23817 .44366 .24424 .43988 .2402 .4368 Metetra
.757 .802 .889 r
.2402 .4368 .24424 .43988 .25032 .43606 .24629 .43303 Metetra
.759 .806 .89 r
.24629 .43303 .25032 .43606 .25641 .43221 .25239 .42922 Metetra
.761 .809 .891 r
.25239 .42922 .25641 .43221 .2625 .42833 .25849 .42539 Metetra
.763 .813 .892 r
.25849 .42539 .2625 .42833 .26861 .4244 .26461 .42152 Metetra
.765 .816 .893 r
.26461 .42152 .26861 .4244 .27473 .42043 .27073 .41761 Metetra
.768 .82 .894 r
.27073 .41761 .27473 .42043 .28086 .41642 .27687 .41365 Metetra
.77 .824 .895 r
.27687 .41365 .28086 .41642 .287 .41235 .28301 .40965 Metetra
.773 .829 .895 r
.28301 .40965 .287 .41235 .29314 .40823 .28917 .4056 Metetra
.776 .833 .896 r
.28917 .4056 .29314 .40823 .2993 .40405 .29533 .40149 Metetra
.78 .838 .897 r
.29533 .40149 .2993 .40405 .30547 .3998 .30151 .39732 Metetra
.783 .844 .897 r
.30151 .39732 .30547 .3998 .31165 .39547 .30769 .39309 Metetra
.788 .85 .898 r
.30769 .39309 .31165 .39547 .31784 .39107 .31389 .38877 Metetra
.792 .855 .898 r
.31389 .38877 .31784 .39107 .32404 .38657 .32009 .38438 Metetra
.797 .862 .898 r
.32009 .38438 .32404 .38657 .33025 .38197 .32631 .37989 Metetra
.802 .869 .897 r
.32631 .37989 .33025 .38197 .33647 .37724 .33254 .37529 Metetra
.808 .877 .896 r
.33254 .37529 .33647 .37724 .3427 .37238 .33878 .37056 Metetra
.816 .884 .894 r
.33878 .37056 .3427 .37238 .34895 .36735 .34504 .36568 Metetra
.824 .893 .891 r
.34504 .36568 .34895 .36735 .35521 .36212 .3513 .36062 Metetra
.833 .901 .887 r
.3513 .36062 .35521 .36212 .36148 .35665 .35758 .35533 Metetra
.845 .91 .88 r
.35758 .35533 .36148 .35665 .36777 .35087 .36388 .34975 Metetra
.858 .917 .869 r
.36388 .34975 .36777 .35087 .37408 .3447 .3702 .34378 Metetra
.875 .923 .854 r
.3702 .34378 .37408 .3447 .38041 .33802 .37654 .33729 Metetra
.895 .923 .831 r
.37654 .33729 .38041 .33802 .38677 .33067 .38292 .33004 Metetra
.917 .912 .799 r
.38292 .33004 .38677 .33067 .39316 .32244 .38933 .32163 Metetra
.935 .882 .757 r
.38933 .32163 .39316 .32244 .39958 .31313 .39581 .31142 Metetra
.937 .83 .714 r
.39581 .31142 .39958 .31313 .40602 .30286 .40237 .29882 Metetra
.88 .456 .051 r
.40237 .29882 .40602 .30286 .41165 .31464 .40792 .31272 Metetra
.863 .414 .088 r
.40792 .31272 .41165 .31464 .41738 .32557 .41362 .32435 Metetra
.834 .364 .063 r
.41362 .32435 .41738 .32557 .4232 .33552 .41944 .33427 Metetra
.807 .319 .008 r
.41944 .33427 .4232 .33552 .42911 .34471 .42536 .34314 Metetra
.781 .277 0 r
.42536 .34314 .42911 .34471 .4351 .35333 .43136 .35135 Metetra
.752 .237 0 r
.43136 .35135 .4351 .35333 .44115 .36158 .43742 .35914 Metetra
.718 .198 0 r
.43742 .35914 .44115 .36158 .44726 .36958 .44355 .36669 Metetra
.68 .162 0 r
.44355 .36669 .44726 .36958 .45343 .37745 .44973 .3741 Metetra
.638 .128 0 r
.44973 .3741 .45343 .37745 .45966 .38526 .45597 .38146 Metetra
.593 .098 0 r
.45597 .38146 .45966 .38526 .46594 .39311 .46226 .38885 Metetra
.55 .073 0 r
.46226 .38885 .46594 .39311 .47228 .40106 .4686 .39633 Metetra
0 0 .484 r
.4686 .39633 .47228 .40106 .47867 .4092 .47499 .404 Metetra
0 0 .508 r
.47499 .404 .47867 .4092 .48512 .41763 .48144 .41193 Metetra
0 0 .519 r
.48144 .41193 .48512 .41763 .49161 .42647 .48794 .42027 Metetra
0 0 .514 r
.48794 .42027 .49161 .42647 .49817 .43588 .49449 .4292 Metetra
0 0 .487 r
.49449 .4292 .49817 .43588 .50478 .44607 .5011 .43901 Metetra
0 0 .425 r
.5011 .43901 .50478 .44607 .51146 .45734 .50776 .45018 Metetra
.644 .262 0 r
.50776 .45018 .51146 .45734 .51821 .46993 .51449 .46351 Metetra
.794 .391 0 r
.51449 .46351 .51821 .46993 .52503 .4838 .5213 .47979 Metetra
.92 .787 .697 r
.5213 .47979 .52503 .4838 .53188 .47228 .52818 .46561 Metetra
.852 .718 .715 r
.52818 .46561 .53188 .47228 .53872 .46194 .53502 .45427 Metetra
.81 .685 .727 r
.53502 .45427 .53872 .46194 .54555 .45286 .54186 .44501 Metetra
.786 .673 .74 r
.54186 .44501 .54555 .45286 .55237 .4448 .5487 .43708 Metetra
.772 .671 .751 r
.5487 .43708 .55237 .4448 .5592 .43751 .55554 .43001 Metetra
.764 .673 .762 r
.55554 .43001 .5592 .43751 .56603 .43079 .56238 .42354 Metetra
.758 .676 .772 r
.56238 .42354 .56603 .43079 .57286 .42451 .56922 .41749 Metetra
.754 .681 .781 r
.56922 .41749 .57286 .42451 .5797 .41857 .57607 .41176 Metetra
.751 .686 .788 r
.57607 .41176 .5797 .41857 .58654 .4129 .58293 .40628 Metetra
.749 .691 .795 r
.58293 .40628 .58654 .4129 .59339 .40744 .58979 .40099 Metetra
.748 .696 .802 r
.58979 .40099 .59339 .40744 .60025 .40216 .59667 .39587 Metetra
.746 .7 .808 r
.59667 .39587 .60025 .40216 .60712 .39703 .60355 .39087 Metetra
.745 .705 .813 r
.60355 .39087 .60712 .39703 .614 .39202 .61044 .38599 Metetra
.744 .709 .818 r
.61044 .38599 .614 .39202 .62088 .38712 .61734 .3812 Metetra
.743 .713 .823 r
.61734 .3812 .62088 .38712 .62778 .3823 .62425 .37649 Metetra
.743 .717 .828 r
.62425 .37649 .62778 .3823 .63468 .37758 .63116 .37186 Metetra
.742 .721 .832 r
.63116 .37186 .63468 .37758 .64159 .37292 .63809 .36729 Metetra
.742 .724 .836 r
.63809 .36729 .64159 .37292 .64851 .36832 .64503 .36277 Metetra
.741 .728 .839 r
.64503 .36277 .64851 .36832 .65545 .36379 .65197 .35831 Metetra
.741 .731 .842 r
.65197 .35831 .65545 .36379 .66239 .3593 .65893 .35389 Metetra
.74 .734 .846 r
.65893 .35389 .66239 .3593 .66934 .35485 .6659 .34951 Metetra
.74 .737 .849 r
.6659 .34951 .66934 .35485 .6763 .35045 .67287 .34517 Metetra
.74 .74 .852 r
.67287 .34517 .6763 .35045 .68327 .34608 .67986 .34086 Metetra
.739 .743 .854 r
.67986 .34086 .68327 .34608 .69026 .34175 .68686 .33658 Metetra
.739 .745 .857 r
.68686 .33658 .69026 .34175 .69725 .33744 .69387 .33233 Metetra
.739 .748 .86 r
.69387 .33233 .69725 .33744 .70426 .33317 .70089 .3281 Metetra
.739 .751 .862 r
.70089 .3281 .70426 .33317 .71127 .3289 .70792 .32389 Metetra
.739 .753 .864 r
.70792 .32389 .71127 .3289 .7183 .32467 .71496 .31969 Metetra
.738 .756 .867 r
.71496 .31969 .7183 .32467 .72533 .32045 .72202 .31552 Metetra
.738 .758 .869 r
.72202 .31552 .72533 .32045 .73238 .31625 .72908 .31136 Metetra
.738 .761 .871 r
.72908 .31136 .73238 .31625 .73944 .31207 .73616 .30721 Metetra
.738 .763 .873 r
.73616 .30721 .73944 .31207 .74651 .30789 .74324 .30307 Metetra
.738 .765 .875 r
.74324 .30307 .74651 .30789 .75359 .30372 .75034 .29894 Metetra
.739 .768 .876 r
.75034 .29894 .75359 .30372 .76068 .29956 .75745 .29481 Metetra
.739 .77 .878 r
.75745 .29481 .76068 .29956 .76778 .2954 .76457 .29069 Metetra
.739 .773 .88 r
.76457 .29069 .76778 .2954 .7749 .29125 .7717 .28657 Metetra
.739 .775 .881 r
.7717 .28657 .7749 .29125 .78202 .28709 .77884 .28245 Metetra
.74 .777 .883 r
.77884 .28245 .78202 .28709 .78915 .28294 .78599 .27832 Metetra
.74 .779 .885 r
.78599 .27832 .78915 .28294 .7963 .27878 .79315 .2742 Metetra
.74 .782 .886 r
.79315 .2742 .7963 .27878 .80345 .27463 .80032 .27007 Metetra
.739 .766 .875 r
.15182 .48377 .15596 .48738 .16192 .48385 .15779 .48028 Metetra
.74 .769 .876 r
.15779 .48028 .16192 .48385 .16789 .48031 .16376 .47678 Metetra
.741 .771 .877 r
.16376 .47678 .16789 .48031 .17387 .47677 .16975 .47326 Metetra
.741 .773 .879 r
.16975 .47326 .17387 .47677 .17985 .47321 .17574 .46975 Metetra
.743 .776 .88 r
.17574 .46975 .17985 .47321 .18585 .46965 .18174 .46621 Metetra
.743 .778 .881 r
.18174 .46621 .18585 .46965 .19185 .46607 .18775 .46267 Metetra
.744 .781 .882 r
.18775 .46267 .19185 .46607 .19786 .46248 .19377 .45911 Metetra
.745 .783 .884 r
.19377 .45911 .19786 .46248 .20389 .45887 .1998 .45555 Metetra
.746 .786 .885 r
.1998 .45555 .20389 .45887 .20992 .45524 .20583 .45196 Metetra
.747 .788 .886 r
.20583 .45196 .20992 .45524 .21595 .4516 .21188 .44836 Metetra
.749 .791 .887 r
.21188 .44836 .21595 .4516 .222 .44794 .21793 .44473 Metetra
.75 .794 .888 r
.21793 .44473 .222 .44794 .22806 .44425 .22399 .44109 Metetra
.752 .797 .89 r
.22399 .44109 .22806 .44425 .23413 .44054 .23007 .43742 Metetra
.753 .8 .891 r
.23007 .43742 .23413 .44054 .2402 .4368 .23615 .43373 Metetra
.755 .804 .892 r
.23615 .43373 .2402 .4368 .24629 .43303 .24224 .43 Metetra
.756 .807 .893 r
.24224 .43 .24629 .43303 .25239 .42922 .24835 .42625 Metetra
.758 .811 .894 r
.24835 .42625 .25239 .42922 .25849 .42539 .25446 .42247 Metetra
.76 .814 .895 r
.25446 .42247 .25849 .42539 .26461 .42152 .26058 .41866 Metetra
.763 .818 .896 r
.26058 .41866 .26461 .42152 .27073 .41761 .26671 .41481 Metetra
.765 .822 .897 r
.26671 .41481 .27073 .41761 .27687 .41365 .27285 .41092 Metetra
.768 .827 .898 r
.27285 .41092 .27687 .41365 .28301 .40965 .279 .40698 Metetra
.77 .831 .9 r
.279 .40698 .28301 .40965 .28917 .4056 .28517 .40301 Metetra
.773 .837 .901 r
.28517 .40301 .28917 .4056 .29533 .40149 .29134 .39898 Metetra
.776 .842 .902 r
.29134 .39898 .29533 .40149 .30151 .39732 .29752 .3949 Metetra
.78 .848 .903 r
.29752 .3949 .30151 .39732 .30769 .39309 .30371 .39075 Metetra
.783 .854 .904 r
.30371 .39075 .30769 .39309 .31389 .38877 .30991 .38654 Metetra
.787 .861 .904 r
.30991 .38654 .31389 .38877 .32009 .38438 .31613 .38226 Metetra
.792 .868 .905 r
.31613 .38226 .32009 .38438 .32631 .37989 .32235 .3779 Metetra
.797 .876 .905 r
.32235 .3779 .32631 .37989 .33254 .37529 .32859 .37344 Metetra
.802 .885 .905 r
.32859 .37344 .33254 .37529 .33878 .37056 .33483 .36888 Metetra
.808 .894 .904 r
.33483 .36888 .33878 .37056 .34504 .36568 .34109 .36418 Metetra
.816 .905 .903 r
.34109 .36418 .34504 .36568 .3513 .36062 .34736 .35933 Metetra
.824 .916 .899 r
.34736 .35933 .3513 .36062 .35758 .35533 .35365 .35429 Metetra
.834 .929 .894 r
.35365 .35429 .35758 .35533 .36388 .34975 .35995 .349 Metetra
.846 .942 .885 r
.35995 .349 .36388 .34975 .3702 .34378 .36627 .34339 Metetra
.861 .955 .87 r
.36627 .34339 .3702 .34378 .37654 .33729 .37261 .33733 Metetra
.881 .965 .845 r
.37261 .33733 .37654 .33729 .38292 .33004 .37898 .33058 Metetra
.91 .966 .804 r
.37898 .33058 .38292 .33004 .38933 .32163 .3854 .32263 Metetra
.95 .941 .739 r
.3854 .32263 .38933 .32163 .39581 .31142 .39193 .31214 Metetra
.978 .854 .648 r
.39193 .31214 .39581 .31142 .40237 .29882 .39871 .29477 Metetra
.923 .532 .171 r
.39871 .29477 .40237 .29882 .40792 .31272 .40406 .31324 Metetra
.85 .42 .203 r
.40406 .31324 .40792 .31272 .41362 .32435 .40976 .32493 Metetra
.813 .354 .142 r
.40976 .32493 .41362 .32435 .41944 .33427 .41561 .33417 Metetra
.79 .304 .058 r
.41561 .33417 .41944 .33427 .42536 .34314 .42156 .3423 Metetra
.768 .259 0 r
.42156 .3423 .42536 .34314 .43136 .35135 .42758 .34983 Metetra
.738 .214 0 r
.42758 .34983 .43136 .35135 .43742 .35914 .43367 .35701 Metetra
.701 .17 0 r
.43367 .35701 .43742 .35914 .44355 .36669 .43981 .36399 Metetra
.655 .127 0 r
.43981 .36399 .44355 .36669 .44973 .3741 .446 .37085 Metetra
.604 .087 0 r
.446 .37085 .44973 .3741 .45597 .38146 .45225 .37768 Metetra
.549 .051 0 r
.45225 .37768 .45597 .38146 .46226 .38885 .45855 .38454 Metetra
0 0 .505 r
.45855 .38454 .46226 .38885 .4686 .39633 .4649 .39147 Metetra
0 .006 .548 r
.4649 .39147 .4686 .39633 .47499 .404 .4713 .39857 Metetra
0 .024 .578 r
.4713 .39857 .47499 .404 .48144 .41193 .47775 .40589 Metetra
0 .033 .596 r
.47775 .40589 .48144 .41193 .48794 .42027 .48425 .41354 Metetra
0 .031 .6 r
.48425 .41354 .48794 .42027 .49449 .4292 .49081 .42171 Metetra
0 .012 .587 r
.49081 .42171 .49449 .4292 .5011 .43901 .49741 .43066 Metetra
0 0 .544 r
.49741 .43066 .5011 .43901 .50776 .45018 .50407 .44101 Metetra
0 0 .425 r
.50407 .44101 .50776 .45018 .51449 .46351 .51078 .45432 Metetra
.798 .447 0 r
.51078 .45432 .51449 .46351 .5213 .47979 .51756 .47577 Metetra
.941 .772 .643 r
.51756 .47577 .5213 .47979 .52818 .46561 .52446 .45613 Metetra
.812 .657 .69 r
.52446 .45613 .52818 .46561 .53502 .45427 .53132 .44452 Metetra
.771 .634 .71 r
.53132 .44452 .53502 .45427 .54186 .44501 .53816 .43582 Metetra
.756 .636 .728 r
.53816 .43582 .54186 .44501 .5487 .43708 .54501 .42848 Metetra
.749 .643 .744 r
.54501 .42848 .5487 .43708 .55554 .43001 .55186 .42191 Metetra
.746 .652 .757 r
.55186 .42191 .55554 .43001 .56238 .42354 .55871 .41585 Metetra
.743 .66 .768 r
.55871 .41585 .56238 .42354 .56922 .41749 .56557 .41013 Metetra
.742 .668 .778 r
.56557 .41013 .56922 .41749 .57607 .41176 .57243 .40469 Metetra
.741 .675 .787 r
.57243 .40469 .57607 .41176 .58293 .40628 .5793 .39944 Metetra
.74 .682 .794 r
.5793 .39944 .58293 .40628 .58979 .40099 .58618 .39436 Metetra
.74 .688 .801 r
.58618 .39436 .58979 .40099 .59667 .39587 .59307 .38941 Metetra
.739 .693 .807 r
.59307 .38941 .59667 .39587 .60355 .39087 .59996 .38458 Metetra
.739 .699 .813 r
.59996 .38458 .60355 .39087 .61044 .38599 .60687 .37984 Metetra
.738 .704 .818 r
.60687 .37984 .61044 .38599 .61734 .3812 .61378 .37518 Metetra
.738 .708 .823 r
.61378 .37518 .61734 .3812 .62425 .37649 .6207 .37058 Metetra
.738 .712 .827 r
.6207 .37058 .62425 .37649 .63116 .37186 .62763 .36605 Metetra
.738 .717 .832 r
.62763 .36605 .63116 .37186 .63809 .36729 .63457 .36158 Metetra
.737 .72 .836 r
.63457 .36158 .63809 .36729 .64503 .36277 .64152 .35715 Metetra
.737 .724 .839 r
.64152 .35715 .64503 .36277 .65197 .35831 .64848 .35277 Metetra
.737 .727 .843 r
.64848 .35277 .65197 .35831 .65893 .35389 .65546 .34842 Metetra
.737 .731 .846 r
.65546 .34842 .65893 .35389 .6659 .34951 .66244 .34411 Metetra
.736 .734 .849 r
.66244 .34411 .6659 .34951 .67287 .34517 .66943 .33984 Metetra
.736 .737 .852 r
.66943 .33984 .67287 .34517 .67986 .34086 .67643 .33559 Metetra
.736 .74 .855 r
.67643 .33559 .67986 .34086 .68686 .33658 .68345 .33137 Metetra
.736 .743 .857 r
.68345 .33137 .68686 .33658 .69387 .33233 .69047 .32717 Metetra
.736 .746 .86 r
.69047 .32717 .69387 .33233 .70089 .3281 .69751 .32299 Metetra
.736 .749 .862 r
.69751 .32299 .70089 .3281 .70792 .32389 .70456 .31882 Metetra
.736 .751 .865 r
.70456 .31882 .70792 .32389 .71496 .31969 .71161 .31468 Metetra
.736 .754 .867 r
.71161 .31468 .71496 .31969 .72202 .31552 .71868 .31055 Metetra
.736 .757 .869 r
.71868 .31055 .72202 .31552 .72908 .31136 .72576 .30643 Metetra
.736 .759 .871 r
.72576 .30643 .72908 .31136 .73616 .30721 .73286 .30232 Metetra
.736 .761 .873 r
.73286 .30232 .73616 .30721 .74324 .30307 .73996 .29822 Metetra
.736 .764 .875 r
.73996 .29822 .74324 .30307 .75034 .29894 .74707 .29412 Metetra
.736 .766 .877 r
.74707 .29412 .75034 .29894 .75745 .29481 .7542 .29004 Metetra
.737 .769 .879 r
.7542 .29004 .75745 .29481 .76457 .29069 .76133 .28594 Metetra
.737 .771 .88 r
.76133 .28594 .76457 .29069 .7717 .28657 .76848 .28186 Metetra
.737 .774 .882 r
.76848 .28186 .7717 .28657 .77884 .28245 .77564 .27777 Metetra
.737 .776 .884 r
.77564 .27777 .77884 .28245 .78599 .27832 .78281 .27369 Metetra
.738 .779 .886 r
.78281 .27369 .78599 .27832 .79315 .2742 .78999 .2696 Metetra
.738 .781 .887 r
.78999 .2696 .79315 .2742 .80032 .27007 .79718 .2655 Metetra
.738 .766 .876 r
.14767 .48015 .15182 .48377 .15779 .48028 .15364 .47669 Metetra
.738 .769 .877 r
.15364 .47669 .15779 .48028 .16376 .47678 .15962 .47322 Metetra
.739 .771 .879 r
.15962 .47322 .16376 .47678 .16975 .47326 .16561 .46975 Metetra
.74 .773 .88 r
.16561 .46975 .16975 .47326 .17574 .46975 .17161 .46627 Metetra
.74 .776 .881 r
.17161 .46627 .17574 .46975 .18174 .46621 .17761 .46277 Metetra
.742 .779 .883 r
.17761 .46277 .18174 .46621 .18775 .46267 .18363 .45927 Metetra
.742 .781 .884 r
.18363 .45927 .18775 .46267 .19377 .45911 .18965 .45575 Metetra
.743 .784 .886 r
.18965 .45575 .19377 .45911 .1998 .45555 .19569 .45222 Metetra
.744 .786 .887 r
.19569 .45222 .1998 .45555 .20583 .45196 .20173 .44867 Metetra
.745 .789 .888 r
.20173 .44867 .20583 .45196 .21188 .44836 .20778 .44511 Metetra
.747 .792 .889 r
.20778 .44511 .21188 .44836 .21793 .44473 .21384 .44153 Metetra
.748 .795 .891 r
.21384 .44153 .21793 .44473 .22399 .44109 .21991 .43793 Metetra
.749 .798 .892 r
.21991 .43793 .22399 .44109 .23007 .43742 .22599 .43431 Metetra
.751 .801 .893 r
.22599 .43431 .23007 .43742 .23615 .43373 .23208 .43066 Metetra
.752 .805 .894 r
.23208 .43066 .23615 .43373 .24224 .43 .23818 .42699 Metetra
.754 .808 .896 r
.23818 .42699 .24224 .43 .24835 .42625 .24428 .42329 Metetra
.756 .812 .897 r
.24428 .42329 .24835 .42625 .25446 .42247 .2504 .41957 Metetra
.758 .816 .898 r
.2504 .41957 .25446 .42247 .26058 .41866 .25653 .41582 Metetra
.76 .82 .9 r
.25653 .41582 .26058 .41866 .26671 .41481 .26267 .41203 Metetra
.762 .824 .901 r
.26267 .41203 .26671 .41481 .27285 .41092 .26882 .4082 Metetra
.764 .829 .902 r
.26882 .4082 .27285 .41092 .279 .40698 .27497 .40434 Metetra
.767 .834 .904 r
.27497 .40434 .279 .40698 .28517 .40301 .28114 .40044 Metetra
.769 .839 .905 r
.28114 .40044 .28517 .40301 .29134 .39898 .28732 .3965 Metetra
.772 .845 .907 r
.28732 .3965 .29134 .39898 .29752 .3949 .29351 .39251 Metetra
.775 .851 .908 r
.29351 .39251 .29752 .3949 .30371 .39075 .29971 .38847 Metetra
.778 .858 .909 r
.29971 .38847 .30371 .39075 .30991 .38654 .30591 .38437 Metetra
.782 .865 .911 r
.30591 .38437 .30991 .38654 .31613 .38226 .31213 .38022 Metetra
.785 .873 .912 r
.31213 .38022 .31613 .38226 .32235 .3779 .31836 .37599 Metetra
.789 .882 .913 r
.31836 .37599 .32235 .3779 .32859 .37344 .3246 .3717 Metetra
.794 .892 .914 r
.3246 .3717 .32859 .37344 .33483 .36888 .33085 .36731 Metetra
.799 .903 .915 r
.33085 .36731 .33483 .36888 .34109 .36418 .33712 .36282 Metetra
.804 .915 .914 r
.33712 .36282 .34109 .36418 .34736 .35933 .34339 .35822 Metetra
.81 .929 .913 r
.34339 .35822 .34736 .35933 .35365 .35429 .34967 .35348 Metetra
.815 .945 .909 r
.34967 .35348 .35365 .35429 .35995 .349 .35597 .34857 Metetra
.82 .962 .901 r
.35597 .34857 .35995 .349 .36627 .34339 .36229 .34345 Metetra
.822 .981 .886 r
.36229 .34345 .36627 .34339 .37261 .33733 .36862 .33806 Metetra
.814 .997 .855 r
.36862 .33806 .37261 .33733 .37898 .33058 .37496 .33234 Metetra
.771 .993 .784 r
.37496 .33234 .37898 .33058 .3854 .32263 .38133 .32622 Metetra
.57 .872 .572 r
.38133 .32622 .3854 .32263 .39193 .31214 .3877 .31988 Metetra
.088 0 .01 r
.3877 .31988 .39193 .31214 .39871 .29477 .39402 .31492 Metetra
.604 .294 .418 r
.39402 .31492 .39871 .29477 .40406 .31324 .3999 .32079 Metetra
.697 .314 .318 r
.3999 .32079 .40406 .31324 .40976 .32493 .40578 .3281 Metetra
.74 .297 .201 r
.40578 .3281 .40976 .32493 .41561 .33417 .41172 .33527 Metetra
.749 .263 .084 r
.41172 .33527 .41561 .33417 .42156 .3423 .41771 .34212 Metetra
.738 .221 0 r
.41771 .34212 .42156 .3423 .42758 .34983 .42377 .34872 Metetra
.711 .175 0 r
.42377 .34872 .42758 .34983 .43367 .35701 .42988 .35513 Metetra
.671 .128 0 r
.42988 .35513 .43367 .35701 .43981 .36399 .43604 .36143 Metetra
.619 .08 0 r
.43604 .36143 .43981 .36399 .446 .37085 .44226 .36768 Metetra
.558 .034 0 r
.44226 .36768 .446 .37085 .45225 .37768 .44852 .37391 Metetra
0 .008 .51 r
.44852 .37391 .45225 .37768 .45855 .38454 .45483 .38017 Metetra
0 .045 .57 r
.45483 .38017 .45855 .38454 .4649 .39147 .46119 .3865 Metetra
0 .076 .617 r
.46119 .3865 .4649 .39147 .4713 .39857 .4676 .39294 Metetra
0 .102 .65 r
.4676 .39294 .4713 .39857 .47775 .40589 .47406 .39954 Metetra
0 .121 .671 r
.47406 .39954 .47775 .40589 .48425 .41354 .48057 .40637 Metetra
0 .135 .682 r
.48057 .40637 .48425 .41354 .49081 .42171 .48712 .41349 Metetra
0 .146 .683 r
.48712 .41349 .49081 .42171 .49741 .43066 .49373 .421 Metetra
0 .16 .679 r
.49373 .421 .49741 .43066 .50407 .44101 .50039 .42899 Metetra
.11 .193 .671 r
.50039 .42899 .50407 .44101 .51078 .45432 .50711 .43727 Metetra
.329 .283 .655 r
.50711 .43727 .51078 .45432 .51756 .47577 .51389 .44407 Metetra
.599 .414 .602 r
.51389 .44407 .51756 .47577 .52446 .45613 .52074 .43876 Metetra
.671 .508 .657 r
.52074 .43876 .52446 .45613 .53132 .44452 .5276 .43189 Metetra
.703 .564 .694 r
.5276 .43189 .53132 .44452 .53816 .43582 .53446 .42528 Metetra
.716 .596 .72 r
.53446 .42528 .53816 .43582 .54501 .42848 .54131 .4191 Metetra
.723 .617 .739 r
.54131 .4191 .54501 .42848 .55186 .42191 .54817 .41329 Metetra
.726 .633 .754 r
.54817 .41329 .55186 .42191 .55871 .41585 .55503 .40778 Metetra
.728 .645 .767 r
.55503 .40778 .55871 .41585 .56557 .41013 .5619 .40249 Metetra
.729 .656 .777 r
.5619 .40249 .56557 .41013 .57243 .40469 .56878 .39738 Metetra
.73 .665 .786 r
.56878 .39738 .57243 .40469 .5793 .39944 .57566 .39242 Metetra
.731 .673 .794 r
.57566 .39242 .5793 .39944 .58618 .39436 .58255 .38757 Metetra
.731 .68 .801 r
.58255 .38757 .58618 .39436 .59307 .38941 .58945 .38282 Metetra
.732 .687 .807 r
.58945 .38282 .59307 .38941 .59996 .38458 .59636 .37817 Metetra
.732 .693 .813 r
.59636 .37817 .59996 .38458 .60687 .37984 .60328 .37358 Metetra
.732 .698 .818 r
.60328 .37358 .60687 .37984 .61378 .37518 .6102 .36906 Metetra
.733 .704 .823 r
.6102 .36906 .61378 .37518 .6207 .37058 .61714 .36459 Metetra
.733 .708 .828 r
.61714 .36459 .6207 .37058 .62763 .36605 .62408 .36017 Metetra
.733 .713 .832 r
.62408 .36017 .62763 .36605 .63457 .36158 .63104 .35579 Metetra
.733 .717 .836 r
.63104 .35579 .63457 .36158 .64152 .35715 .63801 .35146 Metetra
.733 .721 .84 r
.63801 .35146 .64152 .35715 .64848 .35277 .64498 .34716 Metetra
.733 .725 .843 r
.64498 .34716 .64848 .35277 .65546 .34842 .65197 .34289 Metetra
.733 .728 .846 r
.65197 .34289 .65546 .34842 .66244 .34411 .65897 .33866 Metetra
.733 .732 .849 r
.65897 .33866 .66244 .34411 .66943 .33984 .66597 .33445 Metetra
.733 .735 .852 r
.66597 .33445 .66943 .33984 .67643 .33559 .67299 .33026 Metetra
.733 .738 .855 r
.67299 .33026 .67643 .33559 .68345 .33137 .68002 .3261 Metetra
.733 .741 .858 r
.68002 .3261 .68345 .33137 .69047 .32717 .68706 .32196 Metetra
.733 .744 .86 r
.68706 .32196 .69047 .32717 .69751 .32299 .69411 .31782 Metetra
.733 .746 .863 r
.69411 .31782 .69751 .32299 .70456 .31882 .70118 .31371 Metetra
.733 .749 .865 r
.70118 .31371 .70456 .31882 .71161 .31468 .70825 .30962 Metetra
.733 .752 .868 r
.70825 .30962 .71161 .31468 .71868 .31055 .71534 .30553 Metetra
.733 .755 .87 r
.71534 .30553 .71868 .31055 .72576 .30643 .72243 .30146 Metetra
.733 .757 .872 r
.72243 .30146 .72576 .30643 .73286 .30232 .72954 .29739 Metetra
.734 .76 .874 r
.72954 .29739 .73286 .30232 .73996 .29822 .73666 .29333 Metetra
.734 .762 .876 r
.73666 .29333 .73996 .29822 .74707 .29412 .74379 .28928 Metetra
.734 .765 .878 r
.74379 .28928 .74707 .29412 .7542 .29004 .75093 .28522 Metetra
.734 .767 .879 r
.75093 .28522 .7542 .29004 .76133 .28594 .75809 .28117 Metetra
.735 .77 .881 r
.75809 .28117 .76133 .28594 .76848 .28186 .76525 .27712 Metetra
.735 .772 .883 r
.76525 .27712 .76848 .28186 .77564 .27777 .77243 .27307 Metetra
.735 .775 .885 r
.77243 .27307 .77564 .27777 .78281 .27369 .77962 .26902 Metetra
.736 .778 .886 r
.77962 .26902 .78281 .27369 .78999 .2696 .78682 .26497 Metetra
.736 .78 .888 r
.78682 .26497 .78999 .2696 .79718 .2655 .79403 .26091 Metetra
.736 .766 .877 r
.1435 .47651 .14767 .48015 .15364 .47669 .14948 .47309 Metetra
.736 .768 .879 r
.14948 .47309 .15364 .47669 .15962 .47322 .15546 .46966 Metetra
.737 .771 .88 r
.15546 .46966 .15962 .47322 .16561 .46975 .16146 .46622 Metetra
.737 .773 .882 r
.16146 .46622 .16561 .46975 .17161 .46627 .16746 .46278 Metetra
.739 .776 .883 r
.16746 .46278 .17161 .46627 .17761 .46277 .17347 .45932 Metetra
.739 .778 .885 r
.17347 .45932 .17761 .46277 .18363 .45927 .17949 .45586 Metetra
.74 .781 .886 r
.17949 .45586 .18363 .45927 .18965 .45575 .18552 .45238 Metetra
.741 .784 .887 r
.18552 .45238 .18965 .45575 .19569 .45222 .19156 .44889 Metetra
.742 .787 .889 r
.19156 .44889 .19569 .45222 .20173 .44867 .1976 .44538 Metetra
.743 .79 .89 r
.1976 .44538 .20173 .44867 .20778 .44511 .20366 .44187 Metetra
.744 .792 .892 r
.20366 .44187 .20778 .44511 .21384 .44153 .20973 .43833 Metetra
.746 .796 .893 r
.20973 .43833 .21384 .44153 .21991 .43793 .2158 .43478 Metetra
.747 .799 .894 r
.2158 .43478 .21991 .43793 .22599 .43431 .22189 .4312 Metetra
.748 .802 .896 r
.22189 .4312 .22599 .43431 .23208 .43066 .22798 .42761 Metetra
.75 .806 .897 r
.22798 .42761 .23208 .43066 .23818 .42699 .23409 .42399 Metetra
.751 .809 .898 r
.23409 .42399 .23818 .42699 .24428 .42329 .2402 .42035 Metetra
.753 .813 .9 r
.2402 .42035 .24428 .42329 .2504 .41957 .24633 .41668 Metetra
.754 .817 .902 r
.24633 .41668 .2504 .41957 .25653 .41582 .25246 .41299 Metetra
.756 .821 .903 r
.25246 .41299 .25653 .41582 .26267 .41203 .2586 .40926 Metetra
.758 .826 .905 r
.2586 .40926 .26267 .41203 .26882 .4082 .26476 .40551 Metetra
.761 .831 .906 r
.26476 .40551 .26882 .4082 .27497 .40434 .27092 .40172 Metetra
.762 .836 .908 r
.27092 .40172 .27497 .40434 .28114 .40044 .27709 .39791 Metetra
.765 .841 .91 r
.27709 .39791 .28114 .40044 .28732 .3965 .28328 .39405 Metetra
.767 .847 .912 r
.28328 .39405 .28732 .3965 .29351 .39251 .28947 .39016 Metetra
.77 .854 .913 r
.28947 .39016 .29351 .39251 .29971 .38847 .29568 .38622 Metetra
.772 .861 .915 r
.29568 .38622 .29971 .38847 .30591 .38437 .30189 .38225 Metetra
.775 .869 .917 r
.30189 .38225 .30591 .38437 .31213 .38022 .30811 .37822 Metetra
.778 .877 .92 r
.30811 .37822 .31213 .38022 .31836 .37599 .31435 .37415 Metetra
.781 .887 .922 r
.31435 .37415 .31836 .37599 .3246 .3717 .32059 .37002 Metetra
.784 .897 .923 r
.32059 .37002 .3246 .3717 .33085 .36731 .32684 .36583 Metetra
.786 .909 .925 r
.32684 .36583 .33085 .36731 .33712 .36282 .33311 .36157 Metetra
.789 .923 .926 r
.33311 .36157 .33712 .36282 .34339 .35822 .33938 .35724 Metetra
.79 .938 .926 r
.33938 .35724 .34339 .35822 .34967 .35348 .34567 .35284 Metetra
.788 .955 .924 r
.34567 .35284 .34967 .35348 .35597 .34857 .35196 .34835 Metetra
.78 .973 .916 r
.35196 .34835 .35597 .34857 .36229 .34345 .35826 .34378 Metetra
.757 .988 .896 r
.35826 .34378 .36229 .34345 .36862 .33806 .36458 .33914 Metetra
.695 .982 .844 r
.36458 .33914 .36862 .33806 .37496 .33234 .37089 .3345 Metetra
.524 .895 .699 r
.37089 .3345 .37496 .33234 .38133 .32622 .37721 .33005 Metetra
.37721 .33005 .38133 .32622 .3877 .31988 .38349 .3263 Metetra
.306 0 .066 r
.38349 .3263 .3877 .31988 .39402 .31492 .3897 .32453 Metetra
.561 .189 .293 r
.3897 .32453 .39402 .31492 .3999 .32079 .39575 .32701 Metetra
.637 .225 .247 r
.39575 .32701 .3999 .32079 .40578 .3281 .40176 .3315 Metetra
.681 .223 .158 r
.40176 .3315 .40578 .3281 .41172 .33527 .40778 .33675 Metetra
.699 .2 .051 r
.40778 .33675 .41172 .33527 .41771 .34212 .41383 .34226 Metetra
.695 .163 0 r
.41383 .34226 .41771 .34212 .42377 .34872 .41993 .34782 Metetra
.67 .118 0 r
.41993 .34782 .42377 .34872 .42988 .35513 .42607 .3534 Metetra
.626 .069 0 r
.42607 .3534 .42988 .35513 .43604 .36143 .43226 .35897 Metetra
.567 .018 0 r
.43226 .35897 .43604 .36143 .44226 .36768 .43849 .36455 Metetra
.496 0 0 r
.43849 .36455 .44226 .36768 .44852 .37391 .44477 .37014 Metetra
0 .077 .578 r
.44477 .37014 .44852 .37391 .45483 .38017 .4511 .37577 Metetra
0 .118 .639 r
.4511 .37577 .45483 .38017 .46119 .3865 .45747 .38145 Metetra
0 .153 .685 r
.45747 .38145 .46119 .3865 .4676 .39294 .46389 .38719 Metetra
0 .182 .715 r
.46389 .38719 .4676 .39294 .47406 .39954 .47036 .39301 Metetra
0 .208 .734 r
.47036 .39301 .47406 .39954 .48057 .40637 .47687 .39893 Metetra
0 .231 .741 r
.47687 .39893 .48057 .40637 .48712 .41349 .48344 .40492 Metetra
.088 .255 .741 r
.48344 .40492 .48712 .41349 .49373 .421 .49005 .41093 Metetra
.191 .285 .731 r
.49005 .41093 .49373 .421 .50039 .42899 .49671 .41675 Metetra
.313 .325 .713 r
.49671 .41675 .50039 .42899 .50711 .43727 .50343 .42183 Metetra
.451 .378 .684 r
.50343 .42183 .50711 .43727 .51389 .44407 .51021 .42475 Metetra
.575 .435 .652 r
.51021 .42475 .51389 .44407 .52074 .43876 .51704 .42299 Metetra
.631 .493 .676 r
.51704 .42299 .52074 .43876 .5276 .43189 .52389 .41903 Metetra
.665 .539 .701 r
.52389 .41903 .5276 .43189 .53446 .42528 .53074 .41429 Metetra
.686 .573 .723 r
.53074 .41429 .53446 .42528 .54131 .4191 .5376 .40933 Metetra
.699 .599 .741 r
.5376 .40933 .54131 .4191 .54817 .41329 .54447 .40437 Metetra
.707 .618 .755 r
.54447 .40437 .54817 .41329 .55503 .40778 .55134 .39947 Metetra
.712 .633 .767 r
.55134 .39947 .55503 .40778 .5619 .40249 .55822 .39465 Metetra
.716 .646 .778 r
.55822 .39465 .5619 .40249 .56878 .39738 .56511 .38991 Metetra
.719 .657 .787 r
.56511 .38991 .56878 .39738 .57566 .39242 .57201 .38525 Metetra
.721 .666 .795 r
.57201 .38525 .57566 .39242 .58255 .38757 .57891 .38066 Metetra
.723 .674 .802 r
.57891 .38066 .58255 .38757 .58945 .38282 .58582 .37613 Metetra
.724 .681 .808 r
.58582 .37613 .58945 .38282 .59636 .37817 .59274 .37165 Metetra
.725 .688 .814 r
.59274 .37165 .59636 .37817 .60328 .37358 .59967 .36723 Metetra
.726 .694 .819 r
.59967 .36723 .60328 .37358 .6102 .36906 .60661 .36285 Metetra
.727 .699 .824 r
.60661 .36285 .6102 .36906 .61714 .36459 .61356 .35851 Metetra
.727 .704 .828 r
.61356 .35851 .61714 .36459 .62408 .36017 .62052 .35421 Metetra
.728 .709 .833 r
.62052 .35421 .62408 .36017 .63104 .35579 .62749 .34994 Metetra
.728 .714 .836 r
.62749 .34994 .63104 .35579 .63801 .35146 .63447 .3457 Metetra
.729 .718 .84 r
.63447 .3457 .63801 .35146 .64498 .34716 .64146 .34149 Metetra
.729 .721 .844 r
.64146 .34149 .64498 .34716 .65197 .34289 .64846 .33731 Metetra
.729 .726 .847 r
.64846 .33731 .65197 .34289 .65897 .33866 .65548 .33314 Metetra
.729 .729 .85 r
.65548 .33314 .65897 .33866 .66597 .33445 .6625 .32901 Metetra
.73 .732 .853 r
.6625 .32901 .66597 .33445 .67299 .33026 .66953 .32489 Metetra
.73 .736 .856 r
.66953 .32489 .67299 .33026 .68002 .3261 .67658 .32079 Metetra
.73 .739 .858 r
.67658 .32079 .68002 .3261 .68706 .32196 .68363 .3167 Metetra
.73 .742 .861 r
.68363 .3167 .68706 .32196 .69411 .31782 .6907 .31263 Metetra
.73 .745 .864 r
.6907 .31263 .69411 .31782 .70118 .31371 .69778 .30856 Metetra
.73 .748 .866 r
.69778 .30856 .70118 .31371 .70825 .30962 .70487 .30452 Metetra
.73 .75 .868 r
.70487 .30452 .70825 .30962 .71534 .30553 .71197 .30048 Metetra
.731 .753 .87 r
.71197 .30048 .71534 .30553 .72243 .30146 .71909 .29645 Metetra
.731 .756 .872 r
.71909 .29645 .72243 .30146 .72954 .29739 .72621 .29243 Metetra
.731 .758 .875 r
.72621 .29243 .72954 .29739 .73666 .29333 .73335 .28841 Metetra
.732 .761 .876 r
.73335 .28841 .73666 .29333 .74379 .28928 .7405 .2844 Metetra
.732 .764 .878 r
.7405 .2844 .74379 .28928 .75093 .28522 .74765 .28039 Metetra
.732 .766 .88 r
.74765 .28039 .75093 .28522 .75809 .28117 .75483 .27637 Metetra
.732 .769 .882 r
.75483 .27637 .75809 .28117 .76525 .27712 .76201 .27236 Metetra
.733 .772 .884 r
.76201 .27236 .76525 .27712 .77243 .27307 .7692 .26835 Metetra
.733 .774 .886 r
.7692 .26835 .77243 .27307 .77962 .26902 .77641 .26434 Metetra
.734 .777 .887 r
.77641 .26434 .77962 .26902 .78682 .26497 .78363 .26032 Metetra
.734 .779 .889 r
.78363 .26032 .78682 .26497 .79403 .26091 .79086 .25629 Metetra
.733 .766 .879 r
.13932 .47286 .1435 .47651 .14948 .47309 .1453 .46948 Metetra
.734 .768 .88 r
.1453 .46948 .14948 .47309 .15546 .46966 .15129 .46608 Metetra
.735 .771 .882 r
.15129 .46608 .15546 .46966 .16146 .46622 .15728 .46269 Metetra
.736 .773 .883 r
.15728 .46269 .16146 .46622 .16746 .46278 .16329 .45928 Metetra
.736 .776 .885 r
.16329 .45928 .16746 .46278 .17347 .45932 .1693 .45587 Metetra
.737 .779 .886 r
.1693 .45587 .17347 .45932 .17949 .45586 .17533 .45244 Metetra
.738 .781 .888 r
.17533 .45244 .17949 .45586 .18552 .45238 .18136 .449 Metetra
.739 .784 .889 r
.18136 .449 .18552 .45238 .19156 .44889 .18741 .44555 Metetra
.74 .787 .891 r
.18741 .44555 .19156 .44889 .1976 .44538 .19346 .44209 Metetra
.741 .79 .892 r
.19346 .44209 .1976 .44538 .20366 .44187 .19952 .43862 Metetra
.742 .793 .894 r
.19952 .43862 .20366 .44187 .20973 .43833 .20559 .43513 Metetra
.743 .796 .895 r
.20559 .43513 .20973 .43833 .2158 .43478 .21167 .43162 Metetra
.744 .799 .897 r
.21167 .43162 .2158 .43478 .22189 .4312 .21777 .4281 Metetra
.746 .803 .898 r
.21777 .4281 .22189 .4312 .22798 .42761 .22387 .42455 Metetra
.747 .806 .9 r
.22387 .42455 .22798 .42761 .23409 .42399 .22998 .42099 Metetra
.748 .81 .901 r
.22998 .42099 .23409 .42399 .2402 .42035 .2361 .4174 Metetra
.75 .814 .903 r
.2361 .4174 .2402 .42035 .24633 .41668 .24223 .4138 Metetra
.751 .818 .905 r
.24223 .4138 .24633 .41668 .25246 .41299 .24837 .41017 Metetra
.753 .822 .906 r
.24837 .41017 .25246 .41299 .2586 .40926 .25452 .40651 Metetra
.755 .827 .908 r
.25452 .40651 .2586 .40926 .26476 .40551 .26068 .40283 Metetra
.756 .832 .91 r
.26068 .40283 .26476 .40551 .27092 .40172 .26685 .39913 Metetra
.758 .837 .912 r
.26685 .39913 .27092 .40172 .27709 .39791 .27303 .39539 Metetra
.76 .843 .914 r
.27303 .39539 .27709 .39791 .28328 .39405 .27921 .39163 Metetra
.762 .849 .916 r
.27921 .39163 .28328 .39405 .28947 .39016 .28541 .38783 Metetra
.764 .856 .919 r
.28541 .38783 .28947 .39016 .29568 .38622 .29162 .38401 Metetra
.766 .863 .921 r
.29162 .38401 .29568 .38622 .30189 .38225 .29784 .38016 Metetra
.768 .871 .924 r
.29784 .38016 .30189 .38225 .30811 .37822 .30407 .37627 Metetra
.769 .88 .927 r
.30407 .37627 .30811 .37822 .31435 .37415 .31031 .37235 Metetra
.771 .89 .929 r
.31031 .37235 .31435 .37415 .32059 .37002 .31655 .36839 Metetra
.772 .901 .932 r
.31655 .36839 .32059 .37002 .32684 .36583 .32281 .36441 Metetra
.772 .914 .935 r
.32281 .36441 .32684 .36583 .33311 .36157 .32907 .36039 Metetra
.771 .927 .937 r
.32907 .36039 .33311 .36157 .33938 .35724 .33535 .35634 Metetra
.766 .943 .938 r
.33535 .35634 .33938 .35724 .34567 .35284 .34163 .35227 Metetra
.755 .959 .937 r
.34163 .35227 .34567 .35284 .35196 .34835 .34792 .3482 Metetra
.733 .973 .928 r
.34792 .3482 .35196 .34835 .35826 .34378 .35421 .34417 Metetra
.683 .976 .902 r
.35421 .34417 .35826 .34378 .36458 .33914 .36051 .34022 Metetra
.576 .94 .83 r
.36051 .34022 .36458 .33914 .37089 .3345 .3668 .33649 Metetra
.353 .8 .655 r
.3668 .33649 .37089 .3345 .37721 .33005 .37308 .33322 Metetra
.37308 .33322 .37721 .33005 .38349 .3263 .37933 .33082 Metetra
.31 0 0 r
.37933 .33082 .38349 .3263 .3897 .32453 .38552 .32996 Metetra
.501 .059 .131 r
.38552 .32996 .3897 .32453 .39575 .32701 .39164 .3313 Metetra
.579 .119 .125 r
.39164 .3313 .39575 .32701 .40176 .3315 .39773 .33421 Metetra
.622 .13 .062 r
.39773 .33421 .40176 .3315 .40778 .33675 .40381 .33804 Metetra
.641 .115 0 r
.40381 .33804 .40778 .33675 .41383 .34226 .40992 .34236 Metetra
.637 .084 0 r
.40992 .34236 .41383 .34226 .41993 .34782 .41606 .34696 Metetra
.61 .041 0 r
.41606 .34696 .41993 .34782 .42607 .3534 .42223 .3517 Metetra
.563 0 0 r
.42223 .3517 .42607 .3534 .43226 .35897 .42845 .35653 Metetra
.497 0 0 r
.42845 .35653 .43226 .35897 .43849 .36455 .4347 .36143 Metetra
0 .109 .576 r
.4347 .36143 .43849 .36455 .44477 .37014 .441 .36637 Metetra
0 .155 .651 r
.441 .36637 .44477 .37014 .4511 .37577 .44735 .37134 Metetra
0 .195 .707 r
.44735 .37134 .4511 .37577 .45747 .38145 .45373 .37635 Metetra
0 .229 .745 r
.45373 .37635 .45747 .38145 .46389 .38719 .46017 .38138 Metetra
0 .259 .769 r
.46017 .38138 .46389 .38719 .47036 .39301 .46665 .38642 Metetra
.028 .284 .78 r
.46665 .38642 .47036 .39301 .47687 .39893 .47317 .39144 Metetra
.115 .308 .781 r
.47317 .39144 .47687 .39893 .48344 .40492 .47974 .39636 Metetra
.204 .332 .773 r
.47974 .39636 .48344 .40492 .49005 .41093 .48636 .40107 Metetra
.296 .358 .758 r
.48636 .40107 .49005 .41093 .49671 .41675 .49303 .4053 Metetra
.392 .389 .736 r
.49303 .4053 .49671 .41675 .50343 .42183 .49975 .40858 Metetra
.483 .422 .712 r
.49975 .40858 .50343 .42183 .51021 .42475 .50653 .4102 Metetra
.561 .458 .694 r
.50653 .4102 .51021 .42475 .51704 .42299 .51334 .40942 Metetra
.608 .497 .702 r
.51334 .40942 .51704 .42299 .52389 .41903 .52018 .40695 Metetra
.641 .534 .717 r
.52018 .40695 .52389 .41903 .53074 .41429 .52703 .4035 Metetra
.663 .564 .733 r
.52703 .4035 .53074 .41429 .5376 .40933 .53389 .39956 Metetra
.679 .589 .747 r
.53389 .39956 .5376 .40933 .54447 .40437 .54077 .39536 Metetra
.69 .608 .76 r
.54077 .39536 .54447 .40437 .55134 .39947 .54765 .39105 Metetra
.698 .625 .771 r
.54765 .39105 .55134 .39947 .55822 .39465 .55454 .3867 Metetra
.704 .638 .78 r
.55454 .3867 .55822 .39465 .56511 .38991 .56144 .38234 Metetra
.709 .65 .789 r
.56144 .38234 .56511 .38991 .57201 .38525 .56834 .37798 Metetra
.712 .66 .797 r
.56834 .37798 .57201 .38525 .57891 .38066 .57526 .37365 Metetra
.715 .669 .803 r
.57526 .37365 .57891 .38066 .58582 .37613 .58218 .36935 Metetra
.717 .676 .81 r
.58218 .36935 .58582 .37613 .59274 .37165 .58912 .36507 Metetra
.719 .683 .815 r
.58912 .36507 .59274 .37165 .59967 .36723 .59606 .36081 Metetra
.72 .69 .82 r
.59606 .36081 .59967 .36723 .60661 .36285 .60301 .35658 Metetra
.721 .696 .825 r
.60301 .35658 .60661 .36285 .61356 .35851 .60998 .35237 Metetra
.722 .701 .829 r
.60998 .35237 .61356 .35851 .62052 .35421 .61695 .34819 Metetra
.723 .706 .833 r
.61695 .34819 .62052 .35421 .62749 .34994 .62393 .34403 Metetra
.724 .711 .837 r
.62393 .34403 .62749 .34994 .63447 .3457 .63093 .33989 Metetra
.724 .715 .841 r
.63093 .33989 .63447 .3457 .64146 .34149 .63793 .33577 Metetra
.725 .719 .844 r
.63793 .33577 .64146 .34149 .64846 .33731 .64495 .33167 Metetra
.725 .723 .848 r
.64495 .33167 .64846 .33731 .65548 .33314 .65197 .32759 Metetra
.726 .727 .851 r
.65197 .32759 .65548 .33314 .6625 .32901 .65901 .32352 Metetra
.726 .73 .854 r
.65901 .32352 .6625 .32901 .66953 .32489 .66606 .31947 Metetra
.726 .734 .856 r
.66606 .31947 .66953 .32489 .67658 .32079 .67312 .31543 Metetra
.727 .737 .859 r
.67312 .31543 .67658 .32079 .68363 .3167 .68019 .3114 Metetra
.727 .74 .862 r
.68019 .3114 .68363 .3167 .6907 .31263 .68727 .30739 Metetra
.727 .743 .864 r
.68727 .30739 .6907 .31263 .69778 .30856 .69437 .30338 Metetra
.728 .746 .867 r
.69437 .30338 .69778 .30856 .70487 .30452 .70148 .29938 Metetra
.728 .749 .869 r
.70148 .29938 .70487 .30452 .71197 .30048 .70859 .29539 Metetra
.728 .752 .871 r
.70859 .29539 .71197 .30048 .71909 .29645 .71572 .29141 Metetra
.728 .754 .873 r
.71572 .29141 .71909 .29645 .72621 .29243 .72286 .28744 Metetra
.729 .757 .875 r
.72286 .28744 .72621 .29243 .73335 .28841 .73002 .28346 Metetra
.729 .76 .877 r
.73002 .28346 .73335 .28841 .7405 .2844 .73718 .27949 Metetra
.73 .763 .879 r
.73718 .27949 .7405 .2844 .74765 .28039 .74436 .27551 Metetra
.729 .765 .881 r
.74436 .27551 .74765 .28039 .75483 .27637 .75155 .27155 Metetra
.73 .768 .883 r
.75155 .27155 .75483 .27637 .76201 .27236 .75875 .26757 Metetra
.73 .77 .885 r
.75875 .26757 .76201 .27236 .7692 .26835 .76596 .2636 Metetra
.731 .773 .887 r
.76596 .2636 .7692 .26835 .77641 .26434 .77319 .25962 Metetra
.731 .776 .888 r
.77319 .25962 .77641 .26434 .78363 .26032 .78042 .25564 Metetra
.732 .779 .89 r
.78042 .25564 .78363 .26032 .79086 .25629 .78767 .25166 Metetra
.731 .765 .88 r
.13511 .46919 .13932 .47286 .1453 .46948 .1411 .46585 Metetra
.732 .768 .882 r
.1411 .46585 .1453 .46948 .15129 .46608 .14709 .4625 Metetra
.733 .771 .883 r
.14709 .4625 .15129 .46608 .15728 .46269 .15309 .45914 Metetra
.733 .773 .885 r
.15309 .45914 .15728 .46269 .16329 .45928 .1591 .45577 Metetra
.734 .776 .886 r
.1591 .45577 .16329 .45928 .1693 .45587 .16512 .4524 Metetra
.735 .778 .888 r
.16512 .4524 .1693 .45587 .17533 .45244 .17115 .44901 Metetra
.736 .782 .89 r
.17115 .44901 .17533 .45244 .18136 .449 .17719 .44562 Metetra
.736 .784 .891 r
.17719 .44562 .18136 .449 .18741 .44555 .18324 .44221 Metetra
.737 .787 .893 r
.18324 .44221 .18741 .44555 .19346 .44209 .1893 .43879 Metetra
.738 .79 .894 r
.1893 .43879 .19346 .44209 .19952 .43862 .19536 .43537 Metetra
.739 .793 .896 r
.19536 .43537 .19952 .43862 .20559 .43513 .20144 .43192 Metetra
.74 .797 .897 r
.20144 .43192 .20559 .43513 .21167 .43162 .20753 .42846 Metetra
.742 .8 .899 r
.20753 .42846 .21167 .43162 .21777 .4281 .21362 .42499 Metetra
.743 .803 .901 r
.21362 .42499 .21777 .4281 .22387 .42455 .21973 .4215 Metetra
.744 .807 .902 r
.21973 .4215 .22387 .42455 .22998 .42099 .22585 .41799 Metetra
.745 .811 .904 r
.22585 .41799 .22998 .42099 .2361 .4174 .23197 .41446 Metetra
.747 .815 .906 r
.23197 .41446 .2361 .4174 .24223 .4138 .23811 .41092 Metetra
.748 .819 .908 r
.23811 .41092 .24223 .4138 .24837 .41017 .24425 .40735 Metetra
.749 .823 .91 r
.24425 .40735 .24837 .41017 .25452 .40651 .25041 .40377 Metetra
.751 .828 .912 r
.25041 .40377 .25452 .40651 .26068 .40283 .25657 .40016 Metetra
.752 .833 .914 r
.25657 .40016 .26068 .40283 .26685 .39913 .26275 .39653 Metetra
.753 .838 .916 r
.26275 .39653 .26685 .39913 .27303 .39539 .26893 .39288 Metetra
.755 .844 .918 r
.26893 .39288 .27303 .39539 .27921 .39163 .27513 .38921 Metetra
.756 .85 .921 r
.27513 .38921 .27921 .39163 .28541 .38783 .28133 .38552 Metetra
.758 .857 .924 r
.28133 .38552 .28541 .38783 .29162 .38401 .28754 .38181 Metetra
.759 .865 .927 r
.28754 .38181 .29162 .38401 .29784 .38016 .29377 .37808 Metetra
.76 .873 .93 r
.29377 .37808 .29784 .38016 .30407 .37627 .3 .37433 Metetra
.76 .882 .933 r
.3 .37433 .30407 .37627 .31031 .37235 .30624 .37056 Metetra
.76 .892 .937 r
.30624 .37056 .31031 .37235 .31655 .36839 .31249 .36678 Metetra
.759 .903 .94 r
.31249 .36678 .31655 .36839 .32281 .36441 .31875 .363 Metetra
.756 .915 .944 r
.31875 .363 .32281 .36441 .32907 .36039 .32501 .35921 Metetra
.751 .928 .947 r
.32501 .35921 .32907 .36039 .33535 .35634 .33129 .35544 Metetra
.741 .942 .949 r
.33129 .35544 .33535 .35634 .34163 .35227 .33756 .3517 Metetra
.722 .956 .948 r
.33756 .3517 .34163 .35227 .34792 .3482 .34385 .34802 Metetra
.687 .964 .938 r
.34385 .34802 .34792 .3482 .35421 .34417 .35013 .34446 Metetra
.621 .956 .909 r
.35013 .34446 .35421 .34417 .36051 .34022 .35642 .34109 Metetra
.495 .905 .834 r
.35642 .34109 .36051 .34022 .3668 .33649 .36269 .33804 Metetra
.282 .766 .676 r
.36269 .33804 .3668 .33649 .37308 .33322 .36896 .33552 Metetra
.36896 .33552 .37308 .33322 .37933 .33082 .37519 .33381 Metetra
.253 0 0 r
.37519 .33381 .37933 .33082 .38552 .32996 .38139 .33325 Metetra
.421 0 0 r
.38139 .33325 .38552 .32996 .39164 .3313 .38756 .33407 Metetra
.505 0 0 r
.38756 .33407 .39164 .3313 .39773 .33421 .3937 .33604 Metetra
.55 .015 0 r
.3937 .33604 .39773 .33421 .40381 .33804 .39984 .33886 Metetra
.567 .009 0 r
.39984 .33886 .40381 .33804 .40992 .34236 .40599 .34223 Metetra
.561 0 0 r
.40599 .34223 .40992 .34236 .41606 .34696 .41217 .34597 Metetra
.53 0 0 r
.41217 .34597 .41606 .34696 .42223 .3517 .41837 .34994 Metetra
.478 0 0 r
.41837 .34994 .42223 .3517 .42845 .35653 .42462 .35406 Metetra
.407 0 0 r
.42462 .35406 .42845 .35653 .4347 .36143 .4309 .35828 Metetra
0 .196 .658 r
.4309 .35828 .4347 .36143 .441 .36637 .43722 .36257 Metetra
0 .237 .723 r
.43722 .36257 .441 .36637 .44735 .37134 .44358 .3669 Metetra
0 .273 .768 r
.44358 .3669 .44735 .37134 .45373 .37635 .44998 .37125 Metetra
0 .302 .797 r
.44998 .37125 .45373 .37635 .46017 .38138 .45643 .37558 Metetra
.037 .328 .811 r
.45643 .37558 .46017 .38138 .46665 .38642 .46292 .37987 Metetra
.121 .349 .814 r
.46292 .37987 .46665 .38642 .47317 .39144 .46946 .38405 Metetra
.202 .369 .808 r
.46946 .38405 .47317 .39144 .47974 .39636 .47604 .38804 Metetra
.281 .388 .795 r
.47604 .38804 .47974 .39636 .48636 .40107 .48267 .3917 Metetra
.357 .409 .778 r
.48267 .3917 .48636 .40107 .49303 .4053 .48934 .3948 Metetra
.431 .43 .758 r
.48934 .3948 .49303 .4053 .49975 .40858 .49606 .39704 Metetra
.498 .454 .739 r
.49606 .39704 .49975 .40858 .50653 .4102 .50283 .39804 Metetra
.554 .48 .727 r
.50283 .39804 .50653 .4102 .51334 .40942 .50963 .39757 Metetra
.595 .509 .727 r
.50963 .39757 .51334 .40942 .52018 .40695 .51646 .39584 Metetra
.625 .537 .735 r
.51646 .39584 .52018 .40695 .52703 .4035 .52331 .39321 Metetra
.647 .563 .746 r
.52331 .39321 .52703 .4035 .53389 .39956 .53018 .39001 Metetra
.664 .585 .756 r
.53018 .39001 .53389 .39956 .54077 .39536 .53706 .38644 Metetra
.676 .604 .767 r
.53706 .38644 .54077 .39536 .54765 .39105 .54395 .38265 Metetra
.686 .62 .776 r
.54395 .38265 .54765 .39105 .55454 .3867 .55084 .37873 Metetra
.693 .633 .785 r
.55084 .37873 .55454 .3867 .56144 .38234 .55775 .37473 Metetra
.699 .645 .792 r
.55775 .37473 .56144 .38234 .56834 .37798 .56467 .37068 Metetra
.703 .655 .799 r
.56467 .37068 .56834 .37798 .57526 .37365 .57159 .3666 Metetra
.707 .664 .806 r
.57159 .3666 .57526 .37365 .58218 .36935 .57853 .36251 Metetra
.71 .673 .812 r
.57853 .36251 .58218 .36935 .58912 .36507 .58548 .35843 Metetra
.712 .68 .817 r
.58548 .35843 .58912 .36507 .59606 .36081 .59243 .35434 Metetra
.714 .687 .822 r
.59243 .35434 .59606 .36081 .60301 .35658 .5994 .35026 Metetra
.716 .693 .827 r
.5994 .35026 .60301 .35658 .60998 .35237 .60637 .34618 Metetra
.717 .698 .831 r
.60637 .34618 .60998 .35237 .61695 .34819 .61336 .34212 Metetra
.719 .703 .835 r
.61336 .34212 .61695 .34819 .62393 .34403 .62036 .33807 Metetra
.719 .708 .838 r
.62036 .33807 .62393 .34403 .63093 .33989 .62737 .33403 Metetra
.72 .713 .842 r
.62737 .33403 .63093 .33989 .63793 .33577 .63438 .33 Metetra
.721 .717 .845 r
.63438 .33 .63793 .33577 .64495 .33167 .64141 .32599 Metetra
.722 .721 .849 r
.64141 .32599 .64495 .33167 .65197 .32759 .64846 .32198 Metetra
.722 .724 .852 r
.64846 .32198 .65197 .32759 .65901 .32352 .65551 .31799 Metetra
.723 .728 .855 r
.65551 .31799 .65901 .32352 .66606 .31947 .66257 .31401 Metetra
.723 .732 .857 r
.66257 .31401 .66606 .31947 .67312 .31543 .66965 .31003 Metetra
.724 .735 .86 r
.66965 .31003 .67312 .31543 .68019 .3114 .67673 .30607 Metetra
.724 .738 .862 r
.67673 .30607 .68019 .3114 .68727 .30739 .68383 .30211 Metetra
.724 .741 .865 r
.68383 .30211 .68727 .30739 .69437 .30338 .69094 .29815 Metetra
.725 .744 .867 r
.69094 .29815 .69437 .30338 .70148 .29938 .69806 .29421 Metetra
.725 .747 .87 r
.69806 .29421 .70148 .29938 .70859 .29539 .7052 .29027 Metetra
.725 .75 .872 r
.7052 .29027 .70859 .29539 .71572 .29141 .71235 .28634 Metetra
.726 .753 .874 r
.71235 .28634 .71572 .29141 .72286 .28744 .7195 .28241 Metetra
.726 .756 .876 r
.7195 .28241 .72286 .28744 .73002 .28346 .72667 .27848 Metetra
.727 .759 .878 r
.72667 .27848 .73002 .28346 .73718 .27949 .73386 .27455 Metetra
.727 .761 .88 r
.73386 .27455 .73718 .27949 .74436 .27551 .74105 .27062 Metetra
.727 .764 .882 r
.74105 .27062 .74436 .27551 .75155 .27155 .74826 .26669 Metetra
.728 .767 .884 r
.74826 .26669 .75155 .27155 .75875 .26757 .75547 .26276 Metetra
.728 .77 .886 r
.75547 .26276 .75875 .26757 .76596 .2636 .7627 .25882 Metetra
.728 .772 .888 r
.7627 .25882 .76596 .2636 .77319 .25962 .76995 .25489 Metetra
.729 .775 .89 r
.76995 .25489 .77319 .25962 .78042 .25564 .7772 .25094 Metetra
.729 .778 .891 r
.7772 .25094 .78042 .25564 .78767 .25166 .78447 .247 Metetra
.729 .765 .882 r
.13089 .46551 .13511 .46919 .1411 .46585 .13688 .4622 Metetra
.73 .768 .883 r
.13688 .4622 .1411 .46585 .14709 .4625 .14288 .45889 Metetra
.73 .77 .885 r
.14288 .45889 .14709 .4625 .15309 .45914 .14888 .45558 Metetra
.731 .773 .887 r
.14888 .45558 .15309 .45914 .1591 .45577 .1549 .45225 Metetra
.732 .776 .888 r
.1549 .45225 .1591 .45577 .16512 .4524 .16092 .44892 Metetra
.733 .779 .89 r
.16092 .44892 .16512 .4524 .17115 .44901 .16696 .44558 Metetra
.733 .781 .891 r
.16696 .44558 .17115 .44901 .17719 .44562 .173 .44223 Metetra
.734 .785 .893 r
.173 .44223 .17719 .44562 .18324 .44221 .17905 .43886 Metetra
.735 .787 .895 r
.17905 .43886 .18324 .44221 .1893 .43879 .18511 .43549 Metetra
.736 .79 .896 r
.18511 .43549 .1893 .43879 .19536 .43537 .19119 .43211 Metetra
.737 .793 .898 r
.19119 .43211 .19536 .43537 .20144 .43192 .19727 .42871 Metetra
.738 .797 .9 r
.19727 .42871 .20144 .43192 .20753 .42846 .20336 .4253 Metetra
.739 .8 .901 r
.20336 .4253 .20753 .42846 .21362 .42499 .20946 .42188 Metetra
.74 .804 .903 r
.20946 .42188 .21362 .42499 .21973 .4215 .21557 .41844 Metetra
.741 .807 .905 r
.21557 .41844 .21973 .4215 .22585 .41799 .22169 .41499 Metetra
.742 .811 .907 r
.22169 .41499 .22585 .41799 .23197 .41446 .22782 .41152 Metetra
.743 .815 .909 r
.22782 .41152 .23197 .41446 .23811 .41092 .23397 .40804 Metetra
.744 .819 .911 r
.23397 .40804 .23811 .41092 .24425 .40735 .24012 .40454 Metetra
.745 .824 .913 r
.24012 .40454 .24425 .40735 .25041 .40377 .24628 .40102 Metetra
.746 .829 .915 r
.24628 .40102 .25041 .40377 .25657 .40016 .25245 .39749 Metetra
.747 .834 .917 r
.25245 .39749 .25657 .40016 .26275 .39653 .25863 .39395 Metetra
.749 .839 .92 r
.25863 .39395 .26275 .39653 .26893 .39288 .26482 .39038 Metetra
.749 .845 .922 r
.26482 .39038 .26893 .39288 .27513 .38921 .27102 .3868 Metetra
.751 .851 .925 r
.27102 .3868 .27513 .38921 .28133 .38552 .27723 .38321 Metetra
.751 .858 .928 r
.27723 .38321 .28133 .38552 .28754 .38181 .28344 .37961 Metetra
.751 .865 .932 r
.28344 .37961 .28754 .38181 .29377 .37808 .28967 .376 Metetra
.751 .873 .935 r
.28967 .376 .29377 .37808 .3 .37433 .29591 .37239 Metetra
.75 .882 .939 r
.29591 .37239 .3 .37433 .30624 .37056 .30215 .36878 Metetra
.749 .892 .943 r
.30215 .36878 .30624 .37056 .31249 .36678 .3084 .36517 Metetra
.746 .902 .948 r
.3084 .36517 .31249 .36678 .31875 .363 .31466 .36158 Metetra
.741 .914 .952 r
.31466 .36158 .31875 .363 .32501 .35921 .32093 .35802 Metetra
.732 .926 .956 r
.32093 .35802 .32501 .35921 .33129 .35544 .3272 .3545 Metetra
.717 .938 .958 r
.3272 .3545 .33129 .35544 .33756 .3517 .33348 .35105 Metetra
.692 .948 .957 r
.33348 .35105 .33756 .3517 .34385 .34802 .33976 .34772 Metetra
.649 .951 .948 r
.33976 .34772 .34385 .34802 .35013 .34446 .34604 .34455 Metetra
.577 .937 .921 r
.34604 .34455 .35013 .34446 .35642 .34109 .35232 .34164 Metetra
.457 .886 .857 r
.35232 .34164 .35642 .34109 .36269 .33804 .35859 .33907 Metetra
.274 .773 .736 r
.35859 .33907 .36269 .33804 .36896 .33552 .36484 .33703 Metetra
.36484 .33703 .36896 .33552 .37519 .33381 .37108 .33567 Metetra
.162 0 0 r
.37108 .33567 .37519 .33381 .38139 .33325 .3773 .3352 Metetra
.315 0 0 r
.3773 .3352 .38139 .33325 .38756 .33407 .38349 .33569 Metetra
.405 0 0 r
.38349 .33569 .38756 .33407 .3937 .33604 .38967 .33706 Metetra
.453 0 0 r
.38967 .33706 .3937 .33604 .39984 .33886 .39585 .33914 Metetra
.469 0 0 r
.39585 .33914 .39984 .33886 .40599 .34223 .40205 .34176 Metetra
.46 0 0 r
.40205 .34176 .40599 .34223 .41217 .34597 .40826 .34476 Metetra
.427 0 0 r
.40826 .34476 .41217 .34597 .41837 .34994 .4145 .34803 Metetra
.371 0 0 r
.4145 .34803 .41837 .34994 .42462 .35406 .42077 .35149 Metetra
0 .247 .668 r
.42077 .35149 .42462 .35406 .4309 .35828 .42707 .35508 Metetra
0 .285 .738 r
.42707 .35508 .4309 .35828 .43722 .36257 .43342 .35874 Metetra
0 .318 .788 r
.43342 .35874 .43722 .36257 .44358 .3669 .4398 .36245 Metetra
0 .346 .82 r
.4398 .36245 .44358 .3669 .44998 .37125 .44622 .36616 Metetra
.042 .368 .837 r
.44622 .36616 .44998 .37125 .45643 .37558 .45268 .36984 Metetra
.121 .387 .842 r
.45268 .36984 .45643 .37558 .46292 .37987 .45918 .37343 Metetra
.196 .403 .839 r
.45918 .37343 .46292 .37987 .46946 .38405 .46573 .37687 Metetra
.267 .418 .828 r
.46573 .37687 .46946 .38405 .47604 .38804 .47232 .38006 Metetra
.334 .432 .813 r
.47232 .38006 .47604 .38804 .48267 .3917 .47896 .38289 Metetra
.397 .447 .796 r
.47896 .38289 .48267 .3917 .48934 .3948 .48564 .38519 Metetra
.455 .463 .778 r
.48564 .38519 .48934 .3948 .49606 .39704 .49236 .38674 Metetra
.507 .481 .762 r
.49236 .38674 .49606 .39704 .50283 .39804 .49912 .38737 Metetra
.552 .501 .752 r
.49912 .38737 .50283 .39804 .50963 .39757 .50592 .38696 Metetra
.586 .523 .75 r
.50592 .38696 .50963 .39757 .51646 .39584 .51275 .3856 Metetra
.614 .545 .753 r
.51275 .3856 .51646 .39584 .52331 .39321 .5196 .38348 Metetra
.635 .566 .76 r
.5196 .38348 .52331 .39321 .53018 .39001 .52646 .38081 Metetra
.652 .585 .767 r
.52646 .38081 .53018 .39001 .53706 .38644 .53334 .37773 Metetra
.665 .602 .775 r
.53334 .37773 .53706 .38644 .54395 .38265 .54024 .37437 Metetra
.675 .617 .783 r
.54024 .37437 .54395 .38265 .55084 .37873 .54714 .37082 Metetra
.684 .631 .79 r
.54714 .37082 .55084 .37873 .55775 .37473 .55406 .36714 Metetra
.69 .642 .797 r
.55406 .36714 .55775 .37473 .56467 .37068 .56098 .36337 Metetra
.695 .652 .803 r
.56098 .36337 .56467 .37068 .57159 .3666 .56792 .35953 Metetra
.7 .662 .809 r
.56792 .35953 .57159 .3666 .57853 .36251 .57487 .35566 Metetra
.703 .67 .814 r
.57487 .35566 .57853 .36251 .58548 .35843 .58182 .35176 Metetra
.706 .677 .819 r
.58182 .35176 .58548 .35843 .59243 .35434 .58879 .34784 Metetra
.709 .684 .824 r
.58879 .34784 .59243 .35434 .5994 .35026 .59577 .3439 Metetra
.711 .69 .828 r
.59577 .3439 .5994 .35026 .60637 .34618 .60276 .33996 Metetra
.713 .696 .832 r
.60276 .33996 .60637 .34618 .61336 .34212 .60976 .33602 Metetra
.714 .701 .836 r
.60976 .33602 .61336 .34212 .62036 .33807 .61677 .33208 Metetra
.715 .706 .84 r
.61677 .33208 .62036 .33807 .62737 .33403 .62379 .32813 Metetra
.716 .71 .843 r
.62379 .32813 .62737 .33403 .63438 .33 .63082 .3242 Metetra
.717 .715 .847 r
.63082 .3242 .63438 .33 .64141 .32599 .63787 .32027 Metetra
.718 .719 .85 r
.63787 .32027 .64141 .32599 .64846 .32198 .64492 .31634 Metetra
.719 .723 .853 r
.64492 .31634 .64846 .32198 .65551 .31799 .65199 .31242 Metetra
.719 .726 .856 r
.65199 .31242 .65551 .31799 .66257 .31401 .65907 .30851 Metetra
.72 .73 .858 r
.65907 .30851 .66257 .31401 .66965 .31003 .66616 .3046 Metetra
.72 .733 .861 r
.66616 .3046 .66965 .31003 .67673 .30607 .67326 .30069 Metetra
.721 .736 .863 r
.67326 .30069 .67673 .30607 .68383 .30211 .68038 .29679 Metetra
.721 .74 .866 r
.68038 .29679 .68383 .30211 .69094 .29815 .6875 .2929 Metetra
.722 .743 .868 r
.6875 .2929 .69094 .29815 .69806 .29421 .69464 .28901 Metetra
.722 .746 .871 r
.69464 .28901 .69806 .29421 .7052 .29027 .70179 .28512 Metetra
.723 .749 .873 r
.70179 .28512 .7052 .29027 .71235 .28634 .70895 .28123 Metetra
.723 .752 .875 r
.70895 .28123 .71235 .28634 .7195 .28241 .71613 .27735 Metetra
.724 .755 .877 r
.71613 .27735 .7195 .28241 .72667 .27848 .72331 .27346 Metetra
.724 .757 .879 r
.72331 .27346 .72667 .27848 .73386 .27455 .73051 .26958 Metetra
.724 .76 .881 r
.73051 .26958 .73386 .27455 .74105 .27062 .73772 .26569 Metetra
.725 .763 .883 r
.73772 .26569 .74105 .27062 .74826 .26669 .74495 .2618 Metetra
.725 .766 .885 r
.74495 .2618 .74826 .26669 .75547 .26276 .75218 .25791 Metetra
.725 .768 .887 r
.75218 .25791 .75547 .26276 .7627 .25882 .75943 .25402 Metetra
.726 .771 .889 r
.75943 .25402 .7627 .25882 .76995 .25489 .76669 .25012 Metetra
.726 .774 .89 r
.76669 .25012 .76995 .25489 .7772 .25094 .77397 .24622 Metetra
.727 .777 .892 r
.77397 .24622 .7772 .25094 .78447 .247 .78125 .24232 Metetra
.727 .765 .883 r
.12665 .46181 .13089 .46551 .13688 .4622 .13264 .45855 Metetra
.727 .768 .885 r
.13264 .45855 .13688 .4622 .14288 .45889 .13864 .45528 Metetra
.728 .77 .887 r
.13864 .45528 .14288 .45889 .14888 .45558 .14465 .452 Metetra
.729 .773 .888 r
.14465 .452 .14888 .45558 .1549 .45225 .15067 .44872 Metetra
.729 .776 .89 r
.15067 .44872 .1549 .45225 .16092 .44892 .1567 .44543 Metetra
.73 .778 .891 r
.1567 .44543 .16092 .44892 .16696 .44558 .16274 .44213 Metetra
.731 .782 .893 r
.16274 .44213 .16696 .44558 .173 .44223 .16879 .43882 Metetra
.732 .784 .895 r
.16879 .43882 .173 .44223 .17905 .43886 .17484 .43551 Metetra
.732 .787 .896 r
.17484 .43551 .17905 .43886 .18511 .43549 .18091 .43218 Metetra
.733 .79 .898 r
.18091 .43218 .18511 .43549 .19119 .43211 .18699 .42884 Metetra
.734 .794 .9 r
.18699 .42884 .19119 .43211 .19727 .42871 .19307 .42549 Metetra
.735 .797 .902 r
.19307 .42549 .19727 .42871 .20336 .4253 .19917 .42213 Metetra
.736 .8 .904 r
.19917 .42213 .20336 .4253 .20946 .42188 .20528 .41876 Metetra
.737 .804 .905 r
.20528 .41876 .20946 .42188 .21557 .41844 .21139 .41538 Metetra
.738 .808 .907 r
.21139 .41538 .21557 .41844 .22169 .41499 .21752 .41198 Metetra
.738 .811 .909 r
.21752 .41198 .22169 .41499 .22782 .41152 .22366 .40857 Metetra
.74 .816 .911 r
.22366 .40857 .22782 .41152 .23397 .40804 .2298 .40515 Metetra
.74 .82 .913 r
.2298 .40515 .23397 .40804 .24012 .40454 .23596 .40172 Metetra
.741 .824 .916 r
.23596 .40172 .24012 .40454 .24628 .40102 .24213 .39827 Metetra
.742 .829 .918 r
.24213 .39827 .24628 .40102 .25245 .39749 .2483 .39482 Metetra
.743 .834 .921 r
.2483 .39482 .25245 .39749 .25863 .39395 .25449 .39135 Metetra
.743 .839 .923 r
.25449 .39135 .25863 .39395 .26482 .39038 .26068 .38787 Metetra
.744 .845 .926 r
.26068 .38787 .26482 .39038 .27102 .3868 .26688 .38439 Metetra
.744 .851 .929 r
.26688 .38439 .27102 .3868 .27723 .38321 .2731 .3809 Metetra
.744 .858 .933 r
.2731 .3809 .27723 .38321 .28344 .37961 .27932 .37741 Metetra
.744 .865 .936 r
.27932 .37741 .28344 .37961 .28967 .376 .28555 .37392 Metetra
.743 .873 .94 r
.28555 .37392 .28967 .376 .29591 .37239 .29179 .37044 Metetra
.741 .881 .945 r
.29179 .37044 .29591 .37239 .30215 .36878 .29804 .36697 Metetra
.737 .891 .949 r
.29804 .36697 .30215 .36878 .3084 .36517 .30429 .36353 Metetra
.733 .9 .954 r
.30429 .36353 .3084 .36517 .31466 .36158 .31055 .36012 Metetra
.725 .911 .959 r
.31055 .36012 .31466 .36158 .32093 .35802 .31682 .35676 Metetra
.713 .921 .963 r
.31682 .35676 .32093 .35802 .3272 .3545 .3231 .35347 Metetra
.695 .931 .966 r
.3231 .35347 .3272 .3545 .33348 .35105 .32937 .35029 Metetra
.667 .938 .966 r
.32937 .35029 .33348 .35105 .33976 .34772 .33565 .34724 Metetra
.622 .938 .959 r
.33565 .34724 .33976 .34772 .34604 .34455 .34193 .3444 Metetra
.553 .923 .937 r
.34193 .3444 .34604 .34455 .35232 .34164 .34821 .34182 Metetra
.447 .88 .889 r
.34821 .34182 .35232 .34164 .35859 .33907 .35447 .3396 Metetra
.299 .795 .804 r
.35447 .3396 .35859 .33907 .36484 .33703 .36073 .33784 Metetra
.36073 .33784 .36484 .33703 .37108 .33567 .36698 .33667 Metetra
.05 0 0 r
.36698 .33667 .37108 .33567 .3773 .3352 .37321 .33619 Metetra
.188 0 0 r
.37321 .33619 .3773 .3352 .38349 .33569 .37943 .33644 Metetra
.278 0 0 r
.37943 .33644 .38349 .33569 .38967 .33706 .38565 .33737 Metetra
.328 0 0 r
.38565 .33737 .38967 .33706 .39585 .33914 .39186 .33891 Metetra
.345 0 0 r
.39186 .33891 .39585 .33914 .40205 .34176 .39809 .34092 Metetra
.336 0 0 r
.39809 .34092 .40205 .34176 .40826 .34476 .40434 .3433 Metetra
.302 0 0 r
.40434 .3433 .40826 .34476 .4145 .34803 .41061 .34595 Metetra
0 .315 .693 r
.41061 .34595 .4145 .34803 .42077 .35149 .4169 .34881 Metetra
0 .343 .758 r
.4169 .34881 .42077 .35149 .42707 .35508 .42323 .3518 Metetra
0 .37 .808 r
.42323 .3518 .42707 .35508 .43342 .35874 .4296 .35488 Metetra
0 .393 .842 r
.4296 .35488 .43342 .35874 .4398 .36245 .436 .358 Metetra
.05 .411 .861 r
.436 .358 .4398 .36245 .44622 .36616 .44244 .36111 Metetra
.124 .426 .868 r
.44244 .36111 .44622 .36616 .45268 .36984 .44891 .36418 Metetra
.193 .438 .866 r
.44891 .36418 .45268 .36984 .45918 .37343 .45543 .36713 Metetra
.257 .449 .858 r
.45543 .36713 .45918 .37343 .46573 .37687 .46199 .36992 Metetra
.317 .459 .845 r
.46199 .36992 .46573 .37687 .47232 .38006 .46859 .37245 Metetra
.374 .469 .829 r
.46859 .37245 .47232 .38006 .47896 .38289 .47523 .37462 Metetra
.426 .479 .812 r
.47523 .37462 .47896 .38289 .48564 .38519 .48192 .3763 Metetra
.473 .491 .796 r
.48192 .3763 .48564 .38519 .49236 .38674 .48864 .37737 Metetra
.515 .505 .783 r
.48864 .37737 .49236 .38674 .49912 .38737 .4954 .37771 Metetra
.551 .52 .774 r
.4954 .37771 .49912 .38737 .50592 .38696 .5022 .37728 Metetra
.582 .537 .77 r
.5022 .37728 .50592 .38696 .51275 .3856 .50902 .37611 Metetra
.606 .555 .771 r
.50902 .37611 .51275 .3856 .5196 .38348 .51587 .3743 Metetra
.626 .572 .774 r
.51587 .3743 .5196 .38348 .52646 .38081 .52274 .37198 Metetra
.642 .589 .779 r
.52274 .37198 .52646 .38081 .53334 .37773 .52962 .36927 Metetra
.656 .604 .784 r
.52962 .36927 .53334 .37773 .54024 .37437 .53652 .36625 Metetra
.666 .618 .79 r
.53652 .36625 .54024 .37437 .54714 .37082 .54343 .36302 Metetra
.675 .63 .796 r
.54343 .36302 .54714 .37082 .55406 .36714 .55035 .35962 Metetra
.682 .641 .802 r
.55035 .35962 .55406 .36714 .56098 .36337 .55729 .35609 Metetra
.688 .651 .808 r
.55729 .35609 .56098 .36337 .56792 .35953 .56424 .35248 Metetra
.693 .66 .813 r
.56424 .35248 .56792 .35953 .57487 .35566 .57119 .3488 Metetra
.697 .668 .818 r
.57119 .3488 .57487 .35566 .58182 .35176 .57816 .34508 Metetra
.7 .675 .822 r
.57816 .34508 .58182 .35176 .58879 .34784 .58514 .34131 Metetra
.703 .682 .826 r
.58514 .34131 .58879 .34784 .59577 .3439 .59213 .33752 Metetra
.706 .688 .831 r
.59213 .33752 .59577 .3439 .60276 .33996 .59913 .33371 Metetra
.707 .693 .834 r
.59913 .33371 .60276 .33996 .60976 .33602 .60614 .32989 Metetra
.71 .699 .838 r
.60614 .32989 .60976 .33602 .61677 .33208 .61317 .32605 Metetra
.711 .704 .841 r
.61317 .32605 .61677 .33208 .62379 .32813 .6202 .32221 Metetra
.712 .708 .845 r
.6202 .32221 .62379 .32813 .63082 .3242 .62725 .31836 Metetra
.713 .713 .848 r
.62725 .31836 .63082 .3242 .63787 .32027 .63431 .31452 Metetra
.714 .717 .851 r
.63431 .31452 .63787 .32027 .64492 .31634 .64138 .31067 Metetra
.715 .721 .854 r
.64138 .31067 .64492 .31634 .65199 .31242 .64846 .30683 Metetra
.716 .725 .857 r
.64846 .30683 .65199 .31242 .65907 .30851 .65555 .30298 Metetra
.717 .728 .859 r
.65555 .30298 .65907 .30851 .66616 .3046 .66266 .29914 Metetra
.717 .732 .862 r
.66266 .29914 .66616 .3046 .67326 .30069 .66977 .29529 Metetra
.718 .735 .864 r
.66977 .29529 .67326 .30069 .68038 .29679 .6769 .29145 Metetra
.718 .738 .867 r
.6769 .29145 .68038 .29679 .6875 .2929 .68404 .28761 Metetra
.719 .741 .869 r
.68404 .28761 .6875 .2929 .69464 .28901 .6912 .28377 Metetra
.719 .744 .871 r
.6912 .28377 .69464 .28901 .70179 .28512 .69837 .27993 Metetra
.72 .747 .874 r
.69837 .27993 .70179 .28512 .70895 .28123 .70554 .2761 Metetra
.72 .75 .876 r
.70554 .2761 .70895 .28123 .71613 .27735 .71273 .27226 Metetra
.721 .753 .878 r
.71273 .27226 .71613 .27735 .72331 .27346 .71994 .26842 Metetra
.721 .756 .88 r
.71994 .26842 .72331 .27346 .73051 .26958 .72715 .26458 Metetra
.722 .759 .882 r
.72715 .26458 .73051 .26958 .73772 .26569 .73438 .26074 Metetra
.722 .762 .884 r
.73438 .26074 .73772 .26569 .74495 .2618 .74162 .25689 Metetra
.723 .764 .886 r
.74162 .25689 .74495 .2618 .75218 .25791 .74888 .25304 Metetra
.723 .767 .888 r
.74888 .25304 .75218 .25791 .75943 .25402 .75614 .24919 Metetra
.723 .77 .89 r
.75614 .24919 .75943 .25402 .76669 .25012 .76342 .24534 Metetra
.724 .773 .892 r
.76342 .24534 .76669 .25012 .77397 .24622 .77072 .24148 Metetra
.724 .776 .893 r
.77072 .24148 .77397 .24622 .78125 .24232 .77802 .23761 Metetra
.724 .764 .885 r
.1224 .45809 .12665 .46181 .13264 .45855 .12839 .45487 Metetra
.725 .767 .886 r
.12839 .45487 .13264 .45855 .13864 .45528 .13439 .45165 Metetra
.725 .77 .888 r
.13439 .45165 .13864 .45528 .14465 .452 .14041 .44842 Metetra
.726 .773 .89 r
.14041 .44842 .14465 .452 .15067 .44872 .14643 .44518 Metetra
.727 .776 .891 r
.14643 .44518 .15067 .44872 .1567 .44543 .15246 .44193 Metetra
.728 .779 .893 r
.15246 .44193 .1567 .44543 .16274 .44213 .1585 .43867 Metetra
.728 .781 .895 r
.1585 .43867 .16274 .44213 .16879 .43882 .16456 .43541 Metetra
.729 .784 .897 r
.16456 .43541 .16879 .43882 .17484 .43551 .17062 .43214 Metetra
.73 .787 .898 r
.17062 .43214 .17484 .43551 .18091 .43218 .17669 .42886 Metetra
.73 .791 .9 r
.17669 .42886 .18091 .43218 .18699 .42884 .18277 .42557 Metetra
.731 .794 .902 r
.18277 .42557 .18699 .42884 .19307 .42549 .18886 .42227 Metetra
.732 .797 .904 r
.18886 .42227 .19307 .42549 .19917 .42213 .19496 .41896 Metetra
.733 .8 .906 r
.19496 .41896 .19917 .42213 .20528 .41876 .20107 .41564 Metetra
.734 .804 .908 r
.20107 .41564 .20528 .41876 .21139 .41538 .2072 .41231 Metetra
.734 .808 .91 r
.2072 .41231 .21139 .41538 .21752 .41198 .21333 .40897 Metetra
.735 .812 .912 r
.21333 .40897 .21752 .41198 .22366 .40857 .21947 .40562 Metetra
.736 .815 .914 r
.21947 .40562 .22366 .40857 .2298 .40515 .22562 .40226 Metetra
.736 .82 .916 r
.22562 .40226 .2298 .40515 .23596 .40172 .23178 .39889 Metetra
.737 .824 .919 r
.23178 .39889 .23596 .40172 .24213 .39827 .23795 .39552 Metetra
.738 .829 .921 r
.23795 .39552 .24213 .39827 .2483 .39482 .24413 .39214 Metetra
.738 .834 .924 r
.24413 .39214 .2483 .39482 .25449 .39135 .25032 .38875 Metetra
.738 .839 .927 r
.25032 .38875 .25449 .39135 .26068 .38787 .25652 .38535 Metetra
.738 .845 .93 r
.25652 .38535 .26068 .38787 .26688 .38439 .26273 .38196 Metetra
.738 .851 .933 r
.26273 .38196 .26688 .38439 .2731 .3809 .26895 .37857 Metetra
.737 .857 .937 r
.26895 .37857 .2731 .3809 .27932 .37741 .27518 .37518 Metetra
.736 .864 .941 r
.27518 .37518 .27932 .37741 .28555 .37392 .28141 .37181 Metetra
.734 .872 .945 r
.28141 .37181 .28555 .37392 .29179 .37044 .28765 .36845 Metetra
.731 .88 .949 r
.28765 .36845 .29179 .37044 .29804 .36697 .2939 .36512 Metetra
.727 .888 .954 r
.2939 .36512 .29804 .36697 .30429 .36353 .30016 .36183 Metetra
.72 .897 .959 r
.30016 .36183 .30429 .36353 .31055 .36012 .30643 .35859 Metetra
.711 .906 .964 r
.30643 .35859 .31055 .36012 .31682 .35676 .3127 .35542 Metetra
.697 .915 .969 r
.3127 .35542 .31682 .35676 .3231 .35347 .31897 .35233 Metetra
.677 .922 .973 r
.31897 .35233 .3231 .35347 .32937 .35029 .32525 .34937 Metetra
.648 .927 .973 r
.32525 .34937 .32937 .35029 .33565 .34724 .33153 .34656 Metetra
.605 .925 .969 r
.33153 .34656 .33565 .34724 .34193 .3444 .33781 .34397 Metetra
.542 .911 .953 r
.33781 .34397 .34193 .3444 .34821 .34182 .34409 .34164 Metetra
.454 .878 .921 r
.34409 .34164 .34821 .34182 .35447 .3396 .35036 .33964 Metetra
.338 .818 .868 r
.35036 .33964 .35447 .3396 .36073 .33784 .35662 .33807 Metetra
.202 .73 .793 r
.35662 .33807 .36073 .33784 .36698 .33667 .36288 .33699 Metetra
.065 .629 .712 r
.36288 .33699 .36698 .33667 .37321 .33619 .36913 .33646 Metetra
.05 0 0 r
.36913 .33646 .37321 .33619 .37943 .33644 .37538 .3365 Metetra
.134 0 0 r
.37538 .3365 .37943 .33644 .38565 .33737 .38162 .3371 Metetra
.183 0 0 r
.38162 .3371 .38565 .33737 .39186 .33891 .38787 .3382 Metetra
.202 0 0 r
.38787 .3382 .39186 .33891 .39809 .34092 .39413 .33971 Metetra
0 .394 .685 r
.39413 .33971 .39809 .34092 .40434 .3433 .4004 .34156 Metetra
0 .402 .738 r
.4004 .34156 .40434 .3433 .41061 .34595 .4067 .34368 Metetra
0 .416 .789 r
.4067 .34368 .41061 .34595 .4169 .34881 .41302 .34599 Metetra
0 .431 .833 r
.41302 .34599 .4169 .34881 .42323 .3518 .41938 .34844 Metetra
.003 .446 .865 r
.41938 .34844 .42323 .3518 .4296 .35488 .42576 .35098 Metetra
.069 .458 .884 r
.42576 .35098 .4296 .35488 .436 .358 .43218 .35355 Metetra
.133 .468 .892 r
.43218 .35355 .436 .358 .44244 .36111 .43864 .35611 Metetra
.195 .475 .891 r
.43864 .35611 .44244 .36111 .44891 .36418 .44513 .35862 Metetra
.253 .482 .884 r
.44513 .35862 .44891 .36418 .45543 .36713 .45166 .36101 Metetra
.308 .488 .873 r
.45166 .36101 .45543 .36713 .46199 .36992 .45823 .36322 Metetra
.358 .493 .858 r
.45823 .36322 .46199 .36992 .46859 .37245 .46484 .36519 Metetra
.405 .5 .843 r
.46484 .36519 .46859 .37245 .47523 .37462 .47149 .36682 Metetra
.448 .507 .827 r
.47149 .36682 .47523 .37462 .48192 .3763 .47818 .36802 Metetra
.487 .516 .813 r
.47818 .36802 .48192 .3763 .48864 .37737 .48491 .36871 Metetra
.522 .526 .801 r
.48491 .36871 .48864 .37737 .4954 .37771 .49167 .36883 Metetra
.553 .538 .792 r
.49167 .36883 .4954 .37771 .5022 .37728 .49847 .36833 Metetra
.579 .551 .788 r
.49847 .36833 .5022 .37728 .50902 .37611 .50529 .36724 Metetra
.602 .566 .786 r
.50529 .36724 .50902 .37611 .51587 .3743 .51214 .36561 Metetra
.62 .58 .787 r
.51214 .36561 .51587 .3743 .52274 .37198 .51901 .36354 Metetra
.636 .594 .79 r
.51901 .36354 .52274 .37198 .52962 .36927 .52589 .36108 Metetra
.648 .607 .794 r
.52589 .36108 .52962 .36927 .53652 .36625 .53279 .35834 Metetra
.659 .619 .798 r
.53279 .35834 .53652 .36625 .54343 .36302 .53971 .35535 Metetra
.668 .631 .803 r
.53971 .35535 .54343 .36302 .55035 .35962 .54664 .35219 Metetra
.675 .641 .808 r
.54664 .35219 .55035 .35962 .55729 .35609 .55359 .34888 Metetra
.682 .65 .812 r
.55359 .34888 .55729 .35609 .56424 .35248 .56054 .34547 Metetra
.687 .659 .817 r
.56054 .34547 .56424 .35248 .57119 .3488 .56751 .34197 Metetra
.691 .667 .821 r
.56751 .34197 .57119 .3488 .57816 .34508 .57449 .33841 Metetra
.695 .674 .825 r
.57449 .33841 .57816 .34508 .58514 .34131 .58148 .33479 Metetra
.698 .68 .829 r
.58148 .33479 .58514 .34131 .59213 .33752 .58848 .33113 Metetra
.701 .686 .833 r
.58848 .33113 .59213 .33752 .59913 .33371 .59549 .32745 Metetra
.703 .692 .837 r
.59549 .32745 .59913 .33371 .60614 .32989 .60252 .32373 Metetra
.705 .697 .84 r
.60252 .32373 .60614 .32989 .61317 .32605 .60955 .32 Metetra
.707 .702 .843 r
.60955 .32 .61317 .32605 .6202 .32221 .6166 .31626 Metetra
.708 .707 .846 r
.6166 .31626 .6202 .32221 .62725 .31836 .62366 .3125 Metetra
.71 .711 .85 r
.62366 .3125 .62725 .31836 .63431 .31452 .63073 .30874 Metetra
.711 .715 .852 r
.63073 .30874 .63431 .31452 .64138 .31067 .63782 .30497 Metetra
.712 .719 .855 r
.63782 .30497 .64138 .31067 .64846 .30683 .64491 .3012 Metetra
.713 .723 .858 r
.64491 .3012 .64846 .30683 .65555 .30298 .65202 .29742 Metetra
.713 .727 .861 r
.65202 .29742 .65555 .30298 .66266 .29914 .65914 .29364 Metetra
.714 .73 .863 r
.65914 .29364 .66266 .29914 .66977 .29529 .66627 .28986 Metetra
.715 .734 .865 r
.66627 .28986 .66977 .29529 .6769 .29145 .67342 .28607 Metetra
.715 .737 .868 r
.67342 .28607 .6769 .29145 .68404 .28761 .68057 .28229 Metetra
.716 .74 .87 r
.68057 .28229 .68404 .28761 .6912 .28377 .68774 .2785 Metetra
.717 .743 .872 r
.68774 .2785 .6912 .28377 .69837 .27993 .69493 .27472 Metetra
.717 .746 .875 r
.69493 .27472 .69837 .27993 .70554 .2761 .70212 .27093 Metetra
.718 .749 .877 r
.70212 .27093 .70554 .2761 .71273 .27226 .70933 .26714 Metetra
.718 .752 .879 r
.70933 .26714 .71273 .27226 .71994 .26842 .71655 .26335 Metetra
.719 .755 .881 r
.71655 .26335 .71994 .26842 .72715 .26458 .72378 .25955 Metetra
.719 .758 .883 r
.72378 .25955 .72715 .26458 .73438 .26074 .73103 .25576 Metetra
.719 .76 .885 r
.73103 .25576 .73438 .26074 .74162 .25689 .73829 .25196 Metetra
.72 .764 .887 r
.73829 .25196 .74162 .25689 .74888 .25304 .74556 .24815 Metetra
.72 .766 .889 r
.74556 .24815 .74888 .25304 .75614 .24919 .75284 .24434 Metetra
.721 .769 .891 r
.75284 .24434 .75614 .24919 .76342 .24534 .76014 .24053 Metetra
.721 .772 .893 r
.76014 .24053 .76342 .24534 .77072 .24148 .76745 .23671 Metetra
.722 .775 .895 r
.76745 .23671 .77072 .24148 .77802 .23761 .77477 .23289 Metetra
.722 .764 .886 r
.11812 .45436 .1224 .45809 .12839 .45487 .12412 .45119 Metetra
.722 .767 .888 r
.12412 .45119 .12839 .45487 .13439 .45165 .13012 .448 Metetra
.723 .77 .89 r
.13012 .448 .13439 .45165 .14041 .44842 .13614 .44481 Metetra
.724 .772 .891 r
.13614 .44481 .14041 .44842 .14643 .44518 .14217 .44162 Metetra
.724 .775 .893 r
.14217 .44162 .14643 .44518 .15246 .44193 .1482 .43841 Metetra
.725 .778 .895 r
.1482 .43841 .15246 .44193 .1585 .43867 .15425 .4352 Metetra
.726 .781 .897 r
.15425 .4352 .1585 .43867 .16456 .43541 .16031 .43198 Metetra
.726 .784 .898 r
.16031 .43198 .16456 .43541 .17062 .43214 .16637 .42876 Metetra
.727 .787 .9 r
.16637 .42876 .17062 .43214 .17669 .42886 .17245 .42552 Metetra
.728 .79 .902 r
.17245 .42552 .17669 .42886 .18277 .42557 .17853 .42228 Metetra
.728 .794 .904 r
.17853 .42228 .18277 .42557 .18886 .42227 .18463 .41903 Metetra
.729 .797 .906 r
.18463 .41903 .18886 .42227 .19496 .41896 .19073 .41577 Metetra
.73 .8 .908 r
.19073 .41577 .19496 .41896 .20107 .41564 .19685 .41251 Metetra
.73 .804 .91 r
.19685 .41251 .20107 .41564 .2072 .41231 .20298 .40923 Metetra
.731 .808 .912 r
.20298 .40923 .2072 .41231 .21333 .40897 .20911 .40595 Metetra
.731 .811 .914 r
.20911 .40595 .21333 .40897 .21947 .40562 .21526 .40266 Metetra
.732 .816 .916 r
.21526 .40266 .21947 .40562 .22562 .40226 .22142 .39936 Metetra
.732 .82 .919 r
.22142 .39936 .22562 .40226 .23178 .39889 .22758 .39606 Metetra
.733 .824 .921 r
.22758 .39606 .23178 .39889 .23795 .39552 .23376 .39275 Metetra
.733 .829 .924 r
.23376 .39275 .23795 .39552 .24413 .39214 .23994 .38944 Metetra
.733 .834 .927 r
.23994 .38944 .24413 .39214 .25032 .38875 .24614 .38613 Metetra
.733 .839 .93 r
.24614 .38613 .25032 .38875 .25652 .38535 .25234 .38282 Metetra
.733 .844 .933 r
.25234 .38282 .25652 .38535 .26273 .38196 .25856 .37951 Metetra
.732 .85 .936 r
.25856 .37951 .26273 .38196 .26895 .37857 .26478 .37621 Metetra
.731 .856 .94 r
.26478 .37621 .26895 .37857 .27518 .37518 .27101 .37293 Metetra
.728 .862 .944 r
.27101 .37293 .27518 .37518 .28141 .37181 .27725 .36967 Metetra
.726 .87 .948 r
.27725 .36967 .28141 .37181 .28765 .36845 .2835 .36642 Metetra
.722 .877 .953 r
.2835 .36642 .28765 .36845 .2939 .36512 .28975 .36322 Metetra
.716 .884 .958 r
.28975 .36322 .2939 .36512 .30016 .36183 .29601 .36007 Metetra
.709 .892 .963 r
.29601 .36007 .30016 .36183 .30643 .35859 .30228 .35698 Metetra
.698 .9 .969 r
.30228 .35698 .30643 .35859 .3127 .35542 .30855 .35397 Metetra
.684 .907 .974 r
.30855 .35397 .3127 .35542 .31897 .35233 .31483 .35106 Metetra
.663 .913 .978 r
.31483 .35106 .31897 .35233 .32525 .34937 .32111 .34828 Metetra
.635 .916 .98 r
.32111 .34828 .32525 .34937 .33153 .34656 .32739 .34567 Metetra
.595 .913 .977 r
.32739 .34567 .33153 .34656 .33781 .34397 .33368 .34326 Metetra
.541 .902 .968 r
.33368 .34326 .33781 .34397 .34409 .34164 .33996 .34111 Metetra
.468 .877 .949 r
.33996 .34111 .34409 .34164 .35036 .33964 .34624 .33927 Metetra
.379 .835 .917 r
.34624 .33927 .35036 .33964 .35662 .33807 .35251 .3378 Metetra
.276 .776 .873 r
.35251 .3378 .35662 .33807 .36288 .33699 .35878 .33674 Metetra
.172 .706 .825 r
.35878 .33674 .36288 .33699 .36913 .33646 .36505 .33614 Metetra
.079 .638 .784 r
.36505 .33614 .36913 .33646 .37538 .3365 .37132 .33601 Metetra
.008 .581 .761 r
.37132 .33601 .37538 .3365 .38162 .3371 .37759 .33633 Metetra
0 .54 .758 r
.37759 .33633 .38162 .3371 .38787 .3382 .38386 .33707 Metetra
0 .514 .773 r
.38386 .33707 .38787 .3382 .39413 .33971 .39015 .33817 Metetra
0 .503 .801 r
.39015 .33817 .39413 .33971 .4004 .34156 .39645 .33957 Metetra
0 .5 .834 r
.39645 .33957 .4004 .34156 .4067 .34368 .40278 .34121 Metetra
.004 .501 .865 r
.40278 .34121 .4067 .34368 .41302 .34599 .40913 .34303 Metetra
.049 .505 .89 r
.40913 .34303 .41302 .34599 .41938 .34844 .4155 .34499 Metetra
.101 .509 .906 r
.4155 .34499 .41938 .34844 .42576 .35098 .42191 .34703 Metetra
.154 .513 .914 r
.42191 .34703 .42576 .35098 .43218 .35355 .42835 .34911 Metetra
.206 .515 .914 r
.42835 .34911 .43218 .35355 .43864 .35611 .43482 .35117 Metetra
.257 .517 .908 r
.43482 .35117 .43864 .35611 .44513 .35862 .44133 .35317 Metetra
.305 .519 .898 r
.44133 .35317 .44513 .35862 .45166 .36101 .44787 .35505 Metetra
.35 .521 .885 r
.44787 .35505 .45166 .36101 .45823 .36322 .45446 .35677 Metetra
.392 .523 .87 r
.45446 .35677 .45823 .36322 .46484 .36519 .46108 .35825 Metetra
.431 .527 .855 r
.46108 .35825 .46484 .36519 .47149 .36682 .46774 .35943 Metetra
.467 .532 .84 r
.46774 .35943 .47149 .36682 .47818 .36802 .47443 .36024 Metetra
.5 .538 .827 r
.47443 .36024 .47818 .36802 .48491 .36871 .48116 .36063 Metetra
.53 .546 .816 r
.48116 .36063 .48491 .36871 .49167 .36883 .48793 .36054 Metetra
.556 .555 .808 r
.48793 .36054 .49167 .36883 .49847 .36833 .49472 .35995 Metetra
.579 .565 .803 r
.49472 .35995 .49847 .36833 .50529 .36724 .50155 .35888 Metetra
.599 .577 .8 r
.50155 .35888 .50529 .36724 .51214 .36561 .50839 .35735 Metetra
.616 .589 .8 r
.50839 .35735 .51214 .36561 .51901 .36354 .51527 .35543 Metetra
.63 .6 .801 r
.51527 .35543 .51901 .36354 .52589 .36108 .52215 .35317 Metetra
.643 .612 .803 r
.52215 .35317 .52589 .36108 .53279 .35834 .52906 .35062 Metetra
.653 .622 .806 r
.52906 .35062 .53279 .35834 .53971 .35535 .53598 .34784 Metetra
.662 .633 .81 r
.53598 .34784 .53971 .35535 .54664 .35219 .54292 .34488 Metetra
.669 .642 .814 r
.54292 .34488 .54664 .35219 .55359 .34888 .54987 .34176 Metetra
.676 .651 .818 r
.54987 .34176 .55359 .34888 .56054 .34547 .55684 .33852 Metetra
.681 .659 .821 r
.55684 .33852 .56054 .34547 .56751 .34197 .56381 .33518 Metetra
.686 .666 .825 r
.56381 .33518 .56751 .34197 .57449 .33841 .5708 .33176 Metetra
.69 .673 .829 r
.5708 .33176 .57449 .33841 .58148 .33479 .5778 .32828 Metetra
.693 .679 .832 r
.5778 .32828 .58148 .33479 .58848 .33113 .58481 .32475 Metetra
.696 .685 .836 r
.58481 .32475 .58848 .33113 .59549 .32745 .59184 .32117 Metetra
.698 .691 .839 r
.59184 .32117 .59549 .32745 .60252 .32373 .59888 .31757 Metetra
.701 .696 .842 r
.59888 .31757 .60252 .32373 .60955 .32 .60593 .31394 Metetra
.703 .701 .845 r
.60593 .31394 .60955 .32 .6166 .31626 .61299 .31029 Metetra
.704 .706 .848 r
.61299 .31029 .6166 .31626 .62366 .3125 .62006 .30662 Metetra
.706 .71 .851 r
.62006 .30662 .62366 .3125 .63073 .30874 .62714 .30294 Metetra
.707 .714 .854 r
.62714 .30294 .63073 .30874 .63782 .30497 .63424 .29924 Metetra
.708 .718 .857 r
.63424 .29924 .63782 .30497 .64491 .3012 .64135 .29554 Metetra
.709 .722 .859 r
.64135 .29554 .64491 .3012 .65202 .29742 .64847 .29183 Metetra
.71 .725 .862 r
.64847 .29183 .65202 .29742 .65914 .29364 .65561 .28812 Metetra
.711 .729 .864 r
.65561 .28812 .65914 .29364 .66627 .28986 .66275 .28439 Metetra
.712 .732 .867 r
.66275 .28439 .66627 .28986 .67342 .28607 .66991 .28067 Metetra
.713 .735 .869 r
.66991 .28067 .67342 .28607 .68057 .28229 .67709 .27694 Metetra
.713 .738 .871 r
.67709 .27694 .68057 .28229 .68774 .2785 .68427 .27321 Metetra
.714 .742 .874 r
.68427 .27321 .68774 .2785 .69493 .27472 .69147 .26948 Metetra
.714 .745 .876 r
.69147 .26948 .69493 .27472 .70212 .27093 .69868 .26574 Metetra
.715 .748 .878 r
.69868 .26574 .70212 .27093 .70933 .26714 .70591 .262 Metetra
.715 .751 .88 r
.70591 .262 .70933 .26714 .71655 .26335 .71314 .25825 Metetra
.716 .754 .882 r
.71314 .25825 .71655 .26335 .72378 .25955 .72039 .2545 Metetra
.716 .756 .884 r
.72039 .2545 .72378 .25955 .73103 .25576 .72766 .25075 Metetra
.717 .76 .886 r
.72766 .25075 .73103 .25576 .73829 .25196 .73493 .24699 Metetra
.717 .762 .888 r
.73493 .24699 .73829 .25196 .74556 .24815 .74222 .24323 Metetra
.718 .765 .89 r
.74222 .24323 .74556 .24815 .75284 .24434 .74952 .23947 Metetra
.718 .768 .892 r
.74952 .23947 .75284 .24434 .76014 .24053 .75684 .2357 Metetra
.719 .771 .894 r
.75684 .2357 .76014 .24053 .76745 .23671 .76417 .23192 Metetra
.719 .774 .896 r
.76417 .23192 .76745 .23671 .77477 .23289 .77151 .22814 Metetra
.719 .764 .888 r
.11383 .45061 .11812 .45436 .12412 .45119 .11983 .44748 Metetra
.72 .767 .889 r
.11983 .44748 .12412 .45119 .13012 .448 .12584 .44434 Metetra
.72 .769 .891 r
.12584 .44434 .13012 .448 .13614 .44481 .13186 .4412 Metetra
.721 .772 .893 r
.13186 .4412 .13614 .44481 .14217 .44162 .13789 .43805 Metetra
.722 .775 .895 r
.13789 .43805 .14217 .44162 .1482 .43841 .14393 .43489 Metetra
.722 .778 .897 r
.14393 .43489 .1482 .43841 .15425 .4352 .14998 .43172 Metetra
.723 .781 .898 r
.14998 .43172 .15425 .4352 .16031 .43198 .15604 .42855 Metetra
.724 .784 .9 r
.15604 .42855 .16031 .43198 .16637 .42876 .16211 .42537 Metetra
.724 .787 .902 r
.16211 .42537 .16637 .42876 .17245 .42552 .16819 .42218 Metetra
.725 .79 .904 r
.16819 .42218 .17245 .42552 .17853 .42228 .17428 .41898 Metetra
.725 .793 .906 r
.17428 .41898 .17853 .42228 .18463 .41903 .18038 .41578 Metetra
.726 .797 .908 r
.18038 .41578 .18463 .41903 .19073 .41577 .18649 .41257 Metetra
.726 .8 .91 r
.18649 .41257 .19073 .41577 .19685 .41251 .19261 .40936 Metetra
.727 .804 .912 r
.19261 .40936 .19685 .41251 .20298 .40923 .19874 .40614 Metetra
.727 .807 .914 r
.19874 .40614 .20298 .40923 .20911 .40595 .20488 .40292 Metetra
.728 .811 .916 r
.20488 .40292 .20911 .40595 .21526 .40266 .21103 .39968 Metetra
.728 .815 .919 r
.21103 .39968 .21526 .40266 .22142 .39936 .21719 .39645 Metetra
.728 .819 .921 r
.21719 .39645 .22142 .39936 .22758 .39606 .22336 .39321 Metetra
.728 .823 .924 r
.22336 .39321 .22758 .39606 .23376 .39275 .22954 .38997 Metetra
.728 .828 .927 r
.22954 .38997 .23376 .39275 .23994 .38944 .23573 .38673 Metetra
.728 .833 .93 r
.23573 .38673 .23994 .38944 .24614 .38613 .24193 .38349 Metetra
.728 .838 .933 r
.24193 .38349 .24614 .38613 .25234 .38282 .24814 .38026 Metetra
.727 .843 .936 r
.24814 .38026 .25234 .38282 .25856 .37951 .25436 .37704 Metetra
.726 .849 .939 r
.25436 .37704 .25856 .37951 .26478 .37621 .26059 .37383 Metetra
.724 .854 .943 r
.26059 .37383 .26478 .37621 .27101 .37293 .26682 .37064 Metetra
.721 .86 .947 r
.26682 .37064 .27101 .37293 .27725 .36967 .27307 .36747 Metetra
.718 .867 .952 r
.27307 .36747 .27725 .36967 .2835 .36642 .27932 .36434 Metetra
.713 .873 .957 r
.27932 .36434 .2835 .36642 .28975 .36322 .28558 .36126 Metetra
.707 .88 .962 r
.28558 .36126 .28975 .36322 .29601 .36007 .29184 .35823 Metetra
.699 .887 .967 r
.29184 .35823 .29601 .36007 .30228 .35698 .29811 .35527 Metetra
.688 .893 .972 r
.29811 .35527 .30228 .35698 .30855 .35397 .30439 .3524 Metetra
.673 .899 .977 r
.30439 .3524 .30855 .35397 .31483 .35106 .31067 .34964 Metetra
.653 .903 .981 r
.31067 .34964 .31483 .35106 .32111 .34828 .31696 .34701 Metetra
.626 .905 .984 r
.31696 .34701 .32111 .34828 .32739 .34567 .32325 .34455 Metetra
.59 .902 .984 r
.32325 .34455 .32739 .34567 .33368 .34326 .32953 .34229 Metetra
.544 .893 .98 r
.32953 .34229 .33368 .34326 .33996 .34111 .33582 .34027 Metetra
.486 .874 .969 r
.33582 .34027 .33996 .34111 .34624 .33927 .34211 .33853 Metetra
.417 .845 .952 r
.34211 .33853 .34624 .33927 .35251 .3378 .3484 .33711 Metetra
.34 .805 .928 r
.3484 .33711 .35251 .3378 .35878 .33674 .35468 .33604 Metetra
.262 .758 .902 r
.35468 .33604 .35878 .33674 .36505 .33614 .36097 .33536 Metetra
.19 .709 .879 r
.36097 .33536 .36505 .33614 .37132 .33601 .36726 .33506 Metetra
.133 .665 .865 r
.36726 .33506 .37132 .33601 .37759 .33633 .37355 .33514 Metetra
.094 .629 .862 r
.37355 .33514 .37759 .33633 .38386 .33707 .37985 .33558 Metetra
.075 .603 .869 r
.37985 .33558 .38386 .33707 .39015 .33817 .38616 .33632 Metetra
.074 .586 .884 r
.38616 .33632 .39015 .33817 .39645 .33957 .39249 .33733 Metetra
.088 .575 .901 r
.39249 .33733 .39645 .33957 .40278 .34121 .39884 .33855 Metetra
.114 .569 .917 r
.39884 .33855 .40278 .34121 .40913 .34303 .40521 .33994 Metetra
.148 .564 .928 r
.40521 .33994 .40913 .34303 .4155 .34499 .41161 .34146 Metetra
.187 .56 .934 r
.41161 .34146 .4155 .34499 .42191 .34703 .41804 .34305 Metetra
.228 .557 .934 r
.41804 .34305 .42191 .34703 .42835 .34911 .4245 .34467 Metetra
.269 .554 .929 r
.4245 .34467 .42835 .34911 .43482 .35117 .43099 .34627 Metetra
.31 .552 .92 r
.43099 .34627 .43482 .35117 .44133 .35317 .43751 .34782 Metetra
.349 .55 .908 r
.43751 .34782 .44133 .35317 .44787 .35505 .44407 .34925 Metetra
.386 .549 .894 r
.44407 .34925 .44787 .35505 .45446 .35677 .45067 .35053 Metetra
.421 .549 .88 r
.45067 .35053 .45446 .35677 .46108 .35825 .4573 .35159 Metetra
.453 .551 .865 r
.4573 .35159 .46108 .35825 .46774 .35943 .46396 .35239 Metetra
.483 .553 .852 r
.46396 .35239 .46774 .35943 .47443 .36024 .47066 .35288 Metetra
.511 .558 .84 r
.47066 .35288 .47443 .36024 .48116 .36063 .4774 .353 Metetra
.537 .563 .829 r
.4774 .353 .48116 .36063 .48793 .36054 .48417 .35272 Metetra
.559 .57 .821 r
.48417 .35272 .48793 .36054 .49472 .35995 .49097 .35204 Metetra
.58 .579 .816 r
.49097 .35204 .49472 .35995 .50155 .35888 .49779 .35095 Metetra
.598 .588 .813 r
.49779 .35095 .50155 .35888 .50839 .35735 .50464 .34947 Metetra
.613 .597 .811 r
.50464 .34947 .50839 .35735 .51527 .35543 .51152 .34765 Metetra
.627 .607 .811 r
.51152 .34765 .51527 .35543 .52215 .35317 .51841 .34552 Metetra
.638 .617 .812 r
.51841 .34552 .52215 .35317 .52906 .35062 .52532 .34312 Metetra
.648 .626 .814 r
.52532 .34312 .52906 .35062 .53598 .34784 .53225 .3405 Metetra
.657 .636 .817 r
.53225 .3405 .53598 .34784 .54292 .34488 .53919 .33769 Metetra
.664 .644 .82 r
.53919 .33769 .54292 .34488 .54987 .34176 .54615 .33472 Metetra
.671 .652 .823 r
.54615 .33472 .54987 .34176 .55684 .33852 .55312 .33163 Metetra
.676 .66 .826 r
.55312 .33163 .55684 .33852 .56381 .33518 .56011 .32844 Metetra
.681 .666 .829 r
.56011 .32844 .56381 .33518 .5708 .33176 .5671 .32515 Metetra
.685 .673 .832 r
.5671 .32515 .5708 .33176 .5778 .32828 .57412 .32179 Metetra
.689 .679 .835 r
.57412 .32179 .5778 .32828 .58481 .32475 .58114 .31838 Metetra
.692 .685 .839 r
.58114 .31838 .58481 .32475 .59184 .32117 .58818 .31491 Metetra
.695 .69 .842 r
.58818 .31491 .59184 .32117 .59888 .31757 .59522 .31141 Metetra
.697 .695 .844 r
.59522 .31141 .59888 .31757 .60593 .31394 .60228 .30787 Metetra
.699 .7 .847 r
.60228 .30787 .60593 .31394 .61299 .31029 .60936 .3043 Metetra
.701 .704 .85 r
.60936 .3043 .61299 .31029 .62006 .30662 .61644 .30072 Metetra
.702 .709 .853 r
.61644 .30072 .62006 .30662 .62714 .30294 .62354 .29712 Metetra
.704 .713 .856 r
.62354 .29712 .62714 .30294 .63424 .29924 .63065 .2935 Metetra
.705 .717 .858 r
.63065 .2935 .63424 .29924 .64135 .29554 .63778 .28986 Metetra
.706 .72 .861 r
.63778 .28986 .64135 .29554 .64847 .29183 .64491 .28622 Metetra
.707 .724 .863 r
.64491 .28622 .64847 .29183 .65561 .28812 .65206 .28257 Metetra
.708 .727 .866 r
.65206 .28257 .65561 .28812 .66275 .28439 .65922 .27891 Metetra
.709 .731 .868 r
.65922 .27891 .66275 .28439 .66991 .28067 .6664 .27524 Metetra
.709 .734 .87 r
.6664 .27524 .66991 .28067 .67709 .27694 .67359 .27157 Metetra
.71 .737 .872 r
.67359 .27157 .67709 .27694 .68427 .27321 .68079 .26789 Metetra
.711 .74 .875 r
.68079 .26789 .68427 .27321 .69147 .26948 .688 .26421 Metetra
.712 .744 .877 r
.688 .26421 .69147 .26948 .69868 .26574 .69523 .26052 Metetra
.712 .747 .879 r
.69523 .26052 .69868 .26574 .70591 .262 .70247 .25683 Metetra
.713 .75 .881 r
.70247 .25683 .70591 .262 .71314 .25825 .70972 .25313 Metetra
.713 .752 .883 r
.70972 .25313 .71314 .25825 .72039 .2545 .71699 .24943 Metetra
.714 .755 .885 r
.71699 .24943 .72039 .2545 .72766 .25075 .72427 .24572 Metetra
.714 .758 .887 r
.72427 .24572 .72766 .25075 .73493 .24699 .73156 .24201 Metetra
.715 .761 .889 r
.73156 .24201 .73493 .24699 .74222 .24323 .73887 .23829 Metetra
.715 .764 .891 r
.73887 .23829 .74222 .24323 .74952 .23947 .74619 .23457 Metetra
.716 .767 .893 r
.74619 .23457 .74952 .23947 .75684 .2357 .75352 .23084 Metetra
.716 .77 .895 r
.75352 .23084 .75684 .2357 .76417 .23192 .76087 .22711 Metetra
.717 .773 .897 r
.76087 .22711 .76417 .23192 .77151 .22814 .76823 .22337 Metetra
.717 .763 .889 r
.10952 .44685 .11383 .45061 .11983 .44748 .11552 .44376 Metetra
.717 .766 .891 r
.11552 .44376 .11983 .44748 .12584 .44434 .12153 .44067 Metetra
.718 .769 .893 r
.12153 .44067 .12584 .44434 .13186 .4412 .12755 .43756 Metetra
.718 .772 .895 r
.12755 .43756 .13186 .4412 .13789 .43805 .13359 .43446 Metetra
.719 .775 .896 r
.13359 .43446 .13789 .43805 .14393 .43489 .13963 .43134 Metetra
.72 .778 .898 r
.13963 .43134 .14393 .43489 .14998 .43172 .14568 .42822 Metetra
.72 .781 .9 r
.14568 .42822 .14998 .43172 .15604 .42855 .15175 .42509 Metetra
.721 .784 .902 r
.15175 .42509 .15604 .42855 .16211 .42537 .15782 .42196 Metetra
.721 .787 .904 r
.15782 .42196 .16211 .42537 .16819 .42218 .1639 .41882 Metetra
.722 .79 .906 r
.1639 .41882 .16819 .42218 .17428 .41898 .17 .41567 Metetra
.722 .793 .908 r
.17 .41567 .17428 .41898 .18038 .41578 .1761 .41252 Metetra
.723 .796 .91 r
.1761 .41252 .18038 .41578 .18649 .41257 .18222 .40936 Metetra
.723 .8 .912 r
.18222 .40936 .18649 .41257 .19261 .40936 .18834 .4062 Metetra
.723 .803 .914 r
.18834 .4062 .19261 .40936 .19874 .40614 .19448 .40304 Metetra
.724 .807 .916 r
.19448 .40304 .19874 .40614 .20488 .40292 .20062 .39987 Metetra
.724 .811 .919 r
.20062 .39987 .20488 .40292 .21103 .39968 .20678 .39669 Metetra
.724 .815 .921 r
.20678 .39669 .21103 .39968 .21719 .39645 .21294 .39351 Metetra
.724 .819 .924 r
.21294 .39351 .21719 .39645 .22336 .39321 .21912 .39034 Metetra
.724 .823 .926 r
.21912 .39034 .22336 .39321 .22954 .38997 .22531 .38716 Metetra
.724 .827 .929 r
.22531 .38716 .22954 .38997 .23573 .38673 .2315 .38399 Metetra
.723 .832 .932 r
.2315 .38399 .23573 .38673 .24193 .38349 .23771 .38083 Metetra
.722 .836 .935 r
.23771 .38083 .24193 .38349 .24814 .38026 .24392 .37767 Metetra
.722 .842 .938 r
.24392 .37767 .24814 .38026 .25436 .37704 .25014 .37453 Metetra
.72 .847 .942 r
.25014 .37453 .25436 .37704 .26059 .37383 .25638 .3714 Metetra
.718 .852 .946 r
.25638 .3714 .26059 .37383 .26682 .37064 .26262 .3683 Metetra
.714 .858 .95 r
.26262 .3683 .26682 .37064 .27307 .36747 .26886 .36524 Metetra
.711 .864 .954 r
.26886 .36524 .27307 .36747 .27932 .36434 .27512 .3622 Metetra
.705 .869 .959 r
.27512 .3622 .27932 .36434 .28558 .36126 .28138 .35922 Metetra
.699 .875 .964 r
.28138 .35922 .28558 .36126 .29184 .35823 .28766 .35631 Metetra
.69 .881 .969 r
.28766 .35631 .29184 .35823 .29811 .35527 .29393 .35346 Metetra
.678 .886 .974 r
.29393 .35346 .29811 .35527 .30439 .3524 .30021 .35071 Metetra
.664 .891 .98 r
.30021 .35071 .30439 .3524 .31067 .34964 .3065 .34807 Metetra
.645 .894 .984 r
.3065 .34807 .31067 .34964 .31696 .34701 .31279 .34557 Metetra
.621 .895 .988 r
.31279 .34557 .31696 .34701 .32325 .34455 .31908 .34322 Metetra
.59 .892 .989 r
.31908 .34322 .32325 .34455 .32953 .34229 .32538 .34107 Metetra
.551 .884 .988 r
.32538 .34107 .32953 .34229 .33582 .34027 .33168 .33913 Metetra
.504 .87 .984 r
.33168 .33913 .33582 .34027 .34211 .33853 .33798 .33745 Metetra
.451 .849 .975 r
.33798 .33745 .34211 .33853 .3484 .33711 .34428 .33605 Metetra
.393 .821 .963 r
.34428 .33605 .3484 .33711 .35468 .33604 .35058 .33495 Metetra
.333 .788 .95 r
.35058 .33495 .35468 .33604 .36097 .33536 .35688 .33418 Metetra
.28 .753 .938 r
.35688 .33418 .36097 .33536 .36726 .33506 .36319 .33373 Metetra
.235 .719 .93 r
.36319 .33373 .36726 .33506 .37355 .33514 .3695 .3336 Metetra
.203 .689 .928 r
.3695 .3336 .37355 .33514 .37985 .33558 .37582 .33376 Metetra
.186 .665 .931 r
.37582 .33376 .37985 .33558 .38616 .33632 .38216 .3342 Metetra
.182 .646 .937 r
.38216 .3342 .38616 .33632 .39249 .33733 .38851 .33486 Metetra
.19 .631 .944 r
.38851 .33486 .39249 .33733 .39884 .33855 .39489 .33572 Metetra
.207 .619 .95 r
.39489 .33572 .39884 .33855 .40521 .33994 .40128 .33672 Metetra
.231 .609 .953 r
.40128 .33672 .40521 .33994 .41161 .34146 .4077 .33784 Metetra
.259 .6 .952 r
.4077 .33784 .41161 .34146 .41804 .34305 .41415 .33902 Metetra
.291 .593 .947 r
.41415 .33902 .41804 .34305 .4245 .34467 .42063 .34023 Metetra
.323 .587 .939 r
.42063 .34023 .4245 .34467 .43099 .34627 .42713 .34142 Metetra
.355 .581 .928 r
.42713 .34142 .43099 .34627 .43751 .34782 .43367 .34256 Metetra
.387 .577 .915 r
.43367 .34256 .43751 .34782 .44407 .34925 .44025 .3436 Metetra
.417 .574 .902 r
.44025 .3436 .44407 .34925 .45067 .35053 .44686 .34449 Metetra
.446 .572 .888 r
.44686 .34449 .45067 .35053 .4573 .35159 .4535 .34519 Metetra
.473 .572 .874 r
.4535 .34519 .4573 .35159 .46396 .35239 .46017 .34566 Metetra
.498 .573 .862 r
.46017 .34566 .46396 .35239 .47066 .35288 .46688 .34585 Metetra
.522 .576 .85 r
.46688 .34585 .47066 .35288 .4774 .353 .47362 .34574 Metetra
.544 .58 .841 r
.47362 .34574 .4774 .353 .48417 .35272 .4804 .3453 Metetra
.564 .585 .833 r
.4804 .3453 .48417 .35272 .49097 .35204 .4872 .34451 Metetra
.581 .591 .828 r
.4872 .34451 .49097 .35204 .49779 .35095 .49402 .34337 Metetra
.597 .598 .824 r
.49402 .34337 .49779 .35095 .50464 .34947 .50088 .34191 Metetra
.612 .607 .822 r
.50088 .34191 .50464 .34947 .51152 .34765 .50775 .34014 Metetra
.624 .615 .821 r
.50775 .34014 .51152 .34765 .51841 .34552 .51465 .3381 Metetra
.635 .623 .821 r
.51465 .3381 .51841 .34552 .52532 .34312 .52157 .33581 Metetra
.645 .631 .822 r
.52157 .33581 .52532 .34312 .53225 .3405 .5285 .33331 Metetra
.653 .639 .824 r
.5285 .33331 .53225 .3405 .53919 .33769 .53545 .33063 Metetra
.66 .647 .826 r
.53545 .33063 .53919 .33769 .54615 .33472 .54241 .32779 Metetra
.667 .654 .828 r
.54241 .32779 .54615 .33472 .55312 .33163 .54939 .32483 Metetra
.672 .661 .831 r
.54939 .32483 .55312 .33163 .56011 .32844 .55639 .32175 Metetra
.677 .667 .833 r
.55639 .32175 .56011 .32844 .5671 .32515 .5634 .31858 Metetra
.681 .673 .836 r
.5634 .31858 .5671 .32515 .57412 .32179 .57042 .31534 Metetra
.685 .679 .839 r
.57042 .31534 .57412 .32179 .58114 .31838 .57745 .31202 Metetra
.688 .685 .841 r
.57745 .31202 .58114 .31838 .58818 .31491 .5845 .30866 Metetra
.69 .69 .844 r
.5845 .30866 .58818 .31491 .59522 .31141 .59156 .30525 Metetra
.693 .695 .847 r
.59156 .30525 .59522 .31141 .60228 .30787 .59863 .3018 Metetra
.695 .699 .85 r
.59863 .3018 .60228 .30787 .60936 .3043 .60572 .29832 Metetra
.697 .704 .852 r
.60572 .29832 .60936 .3043 .61644 .30072 .61281 .29481 Metetra
.699 .708 .855 r
.61281 .29481 .61644 .30072 .62354 .29712 .61993 .29128 Metetra
.7 .712 .857 r
.61993 .29128 .62354 .29712 .63065 .2935 .62705 .28773 Metetra
.702 .716 .86 r
.62705 .28773 .63065 .2935 .63778 .28986 .63419 .28417 Metetra
.703 .719 .862 r
.63419 .28417 .63778 .28986 .64491 .28622 .64134 .28059 Metetra
.704 .723 .864 r
.64134 .28059 .64491 .28622 .65206 .28257 .6485 .277 Metetra
.705 .726 .867 r
.6485 .277 .65206 .28257 .65922 .27891 .65568 .2734 Metetra
.706 .73 .869 r
.65568 .2734 .65922 .27891 .6664 .27524 .66287 .26979 Metetra
.707 .733 .871 r
.66287 .26979 .6664 .27524 .67359 .27157 .67007 .26617 Metetra
.707 .736 .874 r
.67007 .26617 .67359 .27157 .68079 .26789 .67729 .26255 Metetra
.708 .739 .876 r
.67729 .26255 .68079 .26789 .688 .26421 .68451 .25892 Metetra
.709 .742 .878 r
.68451 .25892 .688 .26421 .69523 .26052 .69176 .25528 Metetra
.709 .745 .88 r
.69176 .25528 .69523 .26052 .70247 .25683 .69901 .25163 Metetra
.71 .748 .882 r
.69901 .25163 .70247 .25683 .70972 .25313 .70628 .24798 Metetra
.711 .751 .884 r
.70628 .24798 .70972 .25313 .71699 .24943 .71357 .24432 Metetra
.711 .754 .886 r
.71357 .24432 .71699 .24943 .72427 .24572 .72086 .24066 Metetra
.712 .757 .888 r
.72086 .24066 .72427 .24572 .73156 .24201 .72818 .237 Metetra
.712 .76 .89 r
.72818 .237 .73156 .24201 .73887 .23829 .7355 .23332 Metetra
.712 .763 .892 r
.7355 .23332 .73887 .23829 .74619 .23457 .74284 .22965 Metetra
.713 .766 .894 r
.74284 .22965 .74619 .23457 .75352 .23084 .75019 .22596 Metetra
.714 .769 .896 r
.75019 .22596 .75352 .23084 .76087 .22711 .75756 .22227 Metetra
.714 .772 .898 r
.75756 .22227 .76087 .22711 .76823 .22337 .76494 .21857 Metetra
.714 .763 .891 r
.10519 .44307 .10952 .44685 .11552 .44376 .11119 .44003 Metetra
.715 .766 .893 r
.11119 .44003 .11552 .44376 .12153 .44067 .11721 .43697 Metetra
.715 .769 .895 r
.11721 .43697 .12153 .44067 .12755 .43756 .12323 .43392 Metetra
.716 .772 .896 r
.12323 .43392 .12755 .43756 .13359 .43446 .12927 .43085 Metetra
.716 .774 .898 r
.12927 .43085 .13359 .43446 .13963 .43134 .13531 .42778 Metetra
.717 .777 .9 r
.13531 .42778 .13963 .43134 .14568 .42822 .14137 .42471 Metetra
.717 .78 .902 r
.14137 .42471 .14568 .42822 .15175 .42509 .14744 .42163 Metetra
.718 .783 .904 r
.14744 .42163 .15175 .42509 .15782 .42196 .15351 .41854 Metetra
.718 .786 .906 r
.15351 .41854 .15782 .42196 .1639 .41882 .1596 .41545 Metetra
.719 .79 .908 r
.1596 .41545 .1639 .41882 .17 .41567 .1657 .41235 Metetra
.719 .793 .91 r
.1657 .41235 .17 .41567 .1761 .41252 .17181 .40924 Metetra
.719 .796 .912 r
.17181 .40924 .1761 .41252 .18222 .40936 .17793 .40614 Metetra
.72 .799 .914 r
.17793 .40614 .18222 .40936 .18834 .4062 .18406 .40303 Metetra
.72 .803 .916 r
.18406 .40303 .18834 .4062 .19448 .40304 .1902 .39991 Metetra
.72 .806 .918 r
.1902 .39991 .19448 .40304 .20062 .39987 .19635 .3968 Metetra
.72 .81 .921 r
.19635 .3968 .20062 .39987 .20678 .39669 .20251 .39368 Metetra
.72 .814 .923 r
.20251 .39368 .20678 .39669 .21294 .39351 .20868 .39056 Metetra
.72 .818 .926 r
.20868 .39056 .21294 .39351 .21912 .39034 .21486 .38745 Metetra
.72 .822 .928 r
.21486 .38745 .21912 .39034 .22531 .38716 .22105 .38433 Metetra
.719 .826 .931 r
.22105 .38433 .22531 .38716 .2315 .38399 .22725 .38123 Metetra
.718 .83 .934 r
.22725 .38123 .2315 .38399 .23771 .38083 .23346 .37813 Metetra
.717 .835 .937 r
.23346 .37813 .23771 .38083 .24392 .37767 .23968 .37505 Metetra
.716 .84 .941 r
.23968 .37505 .24392 .37767 .25014 .37453 .24591 .37198 Metetra
.714 .845 .944 r
.24591 .37198 .25014 .37453 .25638 .3714 .25214 .36893 Metetra
.711 .849 .948 r
.25214 .36893 .25638 .3714 .26262 .3683 .25839 .36592 Metetra
.708 .855 .952 r
.25839 .36592 .26262 .3683 .26886 .36524 .26464 .36294 Metetra
.704 .86 .956 r
.26464 .36294 .26886 .36524 .27512 .3622 .27091 .35999 Metetra
.698 .865 .961 r
.27091 .35999 .27512 .3622 .28138 .35922 .27717 .35711 Metetra
.691 .87 .966 r
.27717 .35711 .28138 .35922 .28766 .35631 .28345 .35429 Metetra
.682 .875 .971 r
.28345 .35429 .28766 .35631 .29393 .35346 .28973 .35155 Metetra
.671 .879 .976 r
.28973 .35155 .29393 .35346 .30021 .35071 .29602 .3489 Metetra
.657 .883 .981 r
.29602 .3489 .30021 .35071 .3065 .34807 .30231 .34636 Metetra
.64 .885 .986 r
.30231 .34636 .3065 .34807 .31279 .34557 .30861 .34395 Metetra
.618 .885 .989 r
.30861 .34395 .31279 .34557 .31908 .34322 .31491 .34169 Metetra
.591 .882 .992 r
.31491 .34169 .31908 .34322 .32538 .34107 .32122 .33961 Metetra
.559 .876 .993 r
.32122 .33961 .32538 .34107 .33168 .33913 .32752 .33773 Metetra
.521 .865 .992 r
.32752 .33773 .33168 .33913 .33798 .33745 .33383 .33608 Metetra
.48 .849 .989 r
.33383 .33608 .33798 .33745 .34428 .33605 .34015 .33467 Metetra
.435 .828 .983 r
.34015 .33467 .34428 .33605 .35058 .33495 .34646 .33353 Metetra
.391 .804 .977 r
.34646 .33353 .35058 .33495 .35688 .33418 .35278 .33266 Metetra
.35 .778 .972 r
.35278 .33266 .35688 .33418 .36319 .33373 .35911 .33207 Metetra
.315 .752 .968 r
.35911 .33207 .36319 .33373 .3695 .3336 .36544 .33174 Metetra
.289 .727 .966 r
.36544 .33174 .3695 .3336 .37582 .33376 .37179 .33167 Metetra
.274 .705 .966 r
.37179 .33167 .37582 .33376 .38216 .3342 .37815 .33183 Metetra
.268 .686 .968 r
.37815 .33183 .38216 .3342 .38851 .33486 .38452 .33219 Metetra
.272 .669 .969 r
.38452 .33219 .38851 .33486 .39489 .33572 .39092 .33272 Metetra
.283 .655 .969 r
.39092 .33272 .39489 .33572 .40128 .33672 .39734 .33338 Metetra
.299 .642 .967 r
.39734 .33338 .40128 .33672 .4077 .33784 .40378 .33414 Metetra
.32 .631 .962 r
.40378 .33414 .4077 .33784 .41415 .33902 .41024 .33495 Metetra
.343 .621 .954 r
.41024 .33495 .41415 .33902 .42063 .34023 .41674 .33579 Metetra
.368 .613 .945 r
.41674 .33579 .42063 .34023 .42713 .34142 .42327 .33662 Metetra
.394 .606 .933 r
.42327 .33662 .42713 .34142 .43367 .34256 .42982 .33739 Metetra
.419 .6 .921 r
.42982 .33739 .43367 .34256 .44025 .3436 .43641 .33807 Metetra
.444 .596 .908 r
.43641 .33807 .44025 .3436 .44686 .34449 .44303 .33862 Metetra
.468 .593 .894 r
.44303 .33862 .44686 .34449 .4535 .34519 .44968 .339 Metetra
.49 .591 .882 r
.44968 .339 .4535 .34519 .46017 .34566 .45636 .33918 Metetra
.512 .591 .87 r
.45636 .33918 .46017 .34566 .46688 .34585 .46308 .33912 Metetra
.532 .592 .86 r
.46308 .33912 .46688 .34585 .47362 .34574 .46983 .3388 Metetra
.551 .595 .851 r
.46983 .3388 .47362 .34574 .4804 .3453 .47661 .33819 Metetra
.568 .598 .843 r
.47661 .33819 .4804 .3453 .4872 .34451 .48341 .3373 Metetra
.584 .603 .838 r
.48341 .3373 .4872 .34451 .49402 .34337 .49024 .33611 Metetra
.598 .609 .834 r
.49024 .33611 .49402 .34337 .50088 .34191 .4971 .33463 Metetra
.611 .615 .831 r
.4971 .33463 .50088 .34191 .50775 .34014 .50398 .33289 Metetra
.622 .622 .829 r
.50398 .33289 .50775 .34014 .51465 .3381 .51088 .3309 Metetra
.633 .629 .829 r
.51088 .3309 .51465 .3381 .52157 .33581 .5178 .32869 Metetra
.642 .636 .829 r
.5178 .32869 .52157 .33581 .5285 .33331 .52474 .32628 Metetra
.65 .643 .83 r
.52474 .32628 .5285 .33331 .53545 .33063 .5317 .32369 Metetra
.657 .65 .831 r
.5317 .32369 .53545 .33063 .54241 .32779 .53867 .32096 Metetra
.663 .656 .833 r
.53867 .32096 .54241 .32779 .54939 .32483 .54566 .3181 Metetra
.668 .663 .835 r
.54566 .3181 .54939 .32483 .55639 .32175 .55266 .31513 Metetra
.673 .668 .837 r
.55266 .31513 .55639 .32175 .5634 .31858 .55968 .31206 Metetra
.677 .674 .84 r
.55968 .31206 .5634 .31858 .57042 .31534 .56671 .30891 Metetra
.681 .68 .842 r
.56671 .30891 .57042 .31534 .57745 .31202 .57375 .3057 Metetra
.684 .685 .844 r
.57375 .3057 .57745 .31202 .5845 .30866 .58081 .30242 Metetra
.687 .69 .847 r
.58081 .30242 .5845 .30866 .59156 .30525 .58788 .2991 Metetra
.689 .694 .849 r
.58788 .2991 .59156 .30525 .59863 .3018 .59496 .29573 Metetra
.692 .699 .852 r
.59496 .29573 .59863 .3018 .60572 .29832 .60206 .29233 Metetra
.694 .703 .854 r
.60206 .29233 .60572 .29832 .61281 .29481 .60917 .2889 Metetra
.695 .707 .857 r
.60917 .2889 .61281 .29481 .61993 .29128 .6163 .28544 Metetra
.697 .711 .859 r
.6163 .28544 .61993 .29128 .62705 .28773 .62343 .28196 Metetra
.698 .715 .861 r
.62343 .28196 .62705 .28773 .63419 .28417 .63058 .27846 Metetra
.7 .718 .864 r
.63058 .27846 .63419 .28417 .64134 .28059 .63775 .27494 Metetra
.701 .722 .866 r
.63775 .27494 .64134 .28059 .6485 .277 .64493 .27141 Metetra
.702 .725 .868 r
.64493 .27141 .6485 .277 .65568 .2734 .65212 .26787 Metetra
.703 .729 .87 r
.65212 .26787 .65568 .2734 .66287 .26979 .65932 .26432 Metetra
.704 .732 .873 r
.65932 .26432 .66287 .26979 .67007 .26617 .66654 .26076 Metetra
.704 .735 .875 r
.66654 .26076 .67007 .26617 .67729 .26255 .67377 .25718 Metetra
.705 .738 .877 r
.67377 .25718 .67729 .26255 .68451 .25892 .68101 .2536 Metetra
.706 .741 .879 r
.68101 .2536 .68451 .25892 .69176 .25528 .68827 .25001 Metetra
.707 .744 .881 r
.68827 .25001 .69176 .25528 .69901 .25163 .69555 .24642 Metetra
.707 .747 .883 r
.69555 .24642 .69901 .25163 .70628 .24798 .70283 .24281 Metetra
.708 .75 .885 r
.70283 .24281 .70628 .24798 .71357 .24432 .71013 .2392 Metetra
.708 .753 .887 r
.71013 .2392 .71357 .24432 .72086 .24066 .71745 .23558 Metetra
.709 .756 .889 r
.71745 .23558 .72086 .24066 .72818 .237 .72477 .23196 Metetra
.709 .759 .891 r
.72477 .23196 .72818 .237 .7355 .23332 .73212 .22833 Metetra
.71 .762 .893 r
.73212 .22833 .7355 .23332 .74284 .22965 .73947 .2247 Metetra
.71 .765 .895 r
.73947 .2247 .74284 .22965 .75019 .22596 .74684 .22106 Metetra
.711 .768 .897 r
.74684 .22106 .75019 .22596 .75756 .22227 .75423 .21741 Metetra
.711 .771 .899 r
.75423 .21741 .75756 .22227 .76494 .21857 .76163 .21376 Metetra
.711 .763 .892 r
.10084 .43928 .10519 .44307 .11119 .44003 .10685 .43628 Metetra
.712 .765 .894 r
.10685 .43628 .11119 .44003 .11721 .43697 .11286 .43327 Metetra
.712 .768 .896 r
.11286 .43327 .11721 .43697 .12323 .43392 .11889 .43025 Metetra
.713 .771 .898 r
.11889 .43025 .12323 .43392 .12927 .43085 .12493 .42724 Metetra
.713 .774 .9 r
.12493 .42724 .12927 .43085 .13531 .42778 .13098 .42421 Metetra
.714 .777 .902 r
.13098 .42421 .13531 .42778 .14137 .42471 .13704 .42118 Metetra
.714 .78 .904 r
.13704 .42118 .14137 .42471 .14744 .42163 .14311 .41815 Metetra
.715 .783 .906 r
.14311 .41815 .14744 .42163 .15351 .41854 .14919 .4151 Metetra
.715 .786 .908 r
.14919 .4151 .15351 .41854 .1596 .41545 .15528 .41206 Metetra
.716 .789 .91 r
.15528 .41206 .1596 .41545 .1657 .41235 .16138 .40901 Metetra
.716 .792 .912 r
.16138 .40901 .1657 .41235 .17181 .40924 .1675 .40595 Metetra
.716 .796 .914 r
.1675 .40595 .17181 .40924 .17793 .40614 .17362 .40289 Metetra
.716 .799 .916 r
.17362 .40289 .17793 .40614 .18406 .40303 .17975 .39983 Metetra
.716 .802 .918 r
.17975 .39983 .18406 .40303 .1902 .39991 .1859 .39677 Metetra
.717 .806 .92 r
.1859 .39677 .1902 .39991 .19635 .3968 .19205 .39371 Metetra
.716 .809 .923 r
.19205 .39371 .19635 .3968 .20251 .39368 .19822 .39065 Metetra
.716 .813 .925 r
.19822 .39065 .20251 .39368 .20868 .39056 .20439 .38759 Metetra
.716 .817 .928 r
.20439 .38759 .20868 .39056 .21486 .38745 .21058 .38453 Metetra
.716 .821 .93 r
.21058 .38453 .21486 .38745 .22105 .38433 .21677 .38148 Metetra
.715 .825 .933 r
.21677 .38148 .22105 .38433 .22725 .38123 .22298 .37844 Metetra
.714 .829 .936 r
.22298 .37844 .22725 .38123 .23346 .37813 .22919 .3754 Metetra
.712 .833 .939 r
.22919 .3754 .23346 .37813 .23968 .37505 .23542 .37239 Metetra
.711 .838 .943 r
.23542 .37239 .23968 .37505 .24591 .37198 .24165 .36939 Metetra
.708 .842 .946 r
.24165 .36939 .24591 .37198 .25214 .36893 .24789 .36642 Metetra
.706 .847 .95 r
.24789 .36642 .25214 .36893 .25839 .36592 .25414 .36348 Metetra
.702 .851 .954 r
.25414 .36348 .25839 .36592 .26464 .36294 .2604 .36058 Metetra
.698 .856 .958 r
.2604 .36058 .26464 .36294 .27091 .35999 .26667 .35771 Metetra
.692 .86 .963 r
.26667 .35771 .27091 .35999 .27717 .35711 .27295 .35491 Metetra
.685 .865 .967 r
.27295 .35491 .27717 .35711 .28345 .35429 .27923 .35218 Metetra
.676 .869 .972 r
.27923 .35218 .28345 .35429 .28973 .35155 .28552 .34952 Metetra
.665 .872 .977 r
.28552 .34952 .28973 .35155 .29602 .3489 .29181 .34695 Metetra
.652 .875 .982 r
.29181 .34695 .29602 .3489 .30231 .34636 .29811 .34449 Metetra
.636 .876 .986 r
.29811 .34449 .30231 .34636 .30861 .34395 .30441 .34216 Metetra
.617 .876 .99 r
.30441 .34216 .30861 .34395 .31491 .34169 .31072 .33997 Metetra
.594 .873 .993 r
.31072 .33997 .31491 .34169 .32122 .33961 .31704 .33794 Metetra
.567 .867 .996 r
.31704 .33794 .32122 .33961 .32752 .33773 .32336 .33609 Metetra
.537 .858 .996 r
.32336 .33609 .32752 .33773 .33383 .33608 .32968 .33445 Metetra
.504 .846 .996 r
.32968 .33445 .33383 .33608 .34015 .33467 .33601 .33302 Metetra
.469 .831 .994 r
.33601 .33302 .34015 .33467 .34646 .33353 .34234 .33182 Metetra
.435 .812 .992 r
.34234 .33182 .34646 .33353 .35278 .33266 .34867 .33086 Metetra
.404 .792 .989 r
.34867 .33086 .35278 .33266 .35911 .33207 .35502 .33012 Metetra
.377 .771 .987 r
.35502 .33012 .35911 .33207 .36544 .33174 .36138 .32963 Metetra
.356 .75 .986 r
.36138 .32963 .36544 .33174 .37179 .33167 .36774 .32934 Metetra
.343 .73 .985 r
.36774 .32934 .37179 .33167 .37815 .33183 .37412 .32926 Metetra
.337 .712 .984 r
.37412 .32926 .37815 .33183 .38452 .33219 .38052 .32935 Metetra
.338 .696 .982 r
.38052 .32935 .38452 .33219 .39092 .33272 .38694 .32958 Metetra
.344 .68 .979 r
.38694 .32958 .39092 .33272 .39734 .33338 .39338 .32993 Metetra
.356 .667 .974 r
.39338 .32993 .39734 .33338 .40378 .33414 .39984 .33036 Metetra
.37 .655 .967 r
.39984 .33036 .40378 .33414 .41024 .33495 .40632 .33085 Metetra
.388 .644 .958 r
.40632 .33085 .41024 .33495 .41674 .33579 .41284 .33136 Metetra
.407 .634 .948 r
.41284 .33136 .41674 .33579 .42327 .33662 .41938 .33185 Metetra
.427 .626 .936 r
.41938 .33185 .42327 .33662 .42982 .33739 .42595 .33229 Metetra
.447 .62 .924 r
.42595 .33229 .42982 .33739 .43641 .33807 .43255 .33265 Metetra
.468 .614 .912 r
.43255 .33265 .43641 .33807 .44303 .33862 .43918 .33289 Metetra
.487 .61 .9 r
.43918 .33289 .44303 .33862 .44968 .339 .44584 .33299 Metetra
.506 .608 .888 r
.44584 .33299 .44968 .339 .45636 .33918 .45254 .33291 Metetra
.524 .606 .877 r
.45254 .33291 .45636 .33918 .46308 .33912 .45926 .33263 Metetra
.542 .607 .868 r
.45926 .33263 .46308 .33912 .46983 .3388 .46602 .33211 Metetra
.558 .608 .859 r
.46602 .33211 .46983 .3388 .47661 .33819 .4728 .33136 Metetra
.573 .611 .852 r
.4728 .33136 .47661 .33819 .48341 .3373 .47961 .33036 Metetra
.587 .614 .846 r
.47961 .33036 .48341 .3373 .49024 .33611 .48645 .3291 Metetra
.6 .619 .842 r
.48645 .3291 .49024 .33611 .4971 .33463 .49331 .3276 Metetra
.611 .624 .839 r
.49331 .3276 .4971 .33463 .50398 .33289 .5002 .32586 Metetra
.622 .629 .837 r
.5002 .32586 .50398 .33289 .51088 .3309 .5071 .3239 Metetra
.631 .635 .836 r
.5071 .3239 .51088 .3309 .5178 .32869 .51403 .32174 Metetra
.64 .641 .836 r
.51403 .32174 .5178 .32869 .52474 .32628 .52097 .31939 Metetra
.647 .647 .836 r
.52097 .31939 .52474 .32628 .5317 .32369 .52794 .31688 Metetra
.654 .653 .837 r
.52794 .31688 .5317 .32369 .53867 .32096 .53491 .31423 Metetra
.66 .659 .838 r
.53491 .31423 .53867 .32096 .54566 .3181 .54191 .31146 Metetra
.665 .665 .84 r
.54191 .31146 .54566 .3181 .55266 .31513 .54892 .30858 Metetra
.67 .67 .842 r
.54892 .30858 .55266 .31513 .55968 .31206 .55595 .3056 Metetra
.674 .675 .843 r
.55595 .3056 .55968 .31206 .56671 .30891 .56299 .30253 Metetra
.678 .68 .845 r
.56299 .30253 .56671 .30891 .57375 .3057 .57004 .2994 Metetra
.681 .685 .848 r
.57004 .2994 .57375 .3057 .58081 .30242 .57711 .29621 Metetra
.684 .69 .85 r
.57711 .29621 .58081 .30242 .58788 .2991 .58419 .29296 Metetra
.686 .694 .852 r
.58419 .29296 .58788 .2991 .59496 .29573 .59128 .28967 Metetra
.688 .698 .854 r
.59128 .28967 .59496 .29573 .60206 .29233 .59839 .28634 Metetra
.69 .703 .856 r
.59839 .28634 .60206 .29233 .60917 .2889 .60552 .28298 Metetra
.692 .706 .859 r
.60552 .28298 .60917 .2889 .6163 .28544 .61265 .27959 Metetra
.694 .71 .861 r
.61265 .27959 .6163 .28544 .62343 .28196 .6198 .27617 Metetra
.695 .714 .863 r
.6198 .27617 .62343 .28196 .63058 .27846 .62697 .27274 Metetra
.697 .718 .865 r
.62697 .27274 .63058 .27846 .63775 .27494 .63414 .26928 Metetra
.698 .721 .868 r
.63414 .26928 .63775 .27494 .64493 .27141 .64134 .26581 Metetra
.699 .724 .87 r
.64134 .26581 .64493 .27141 .65212 .26787 .64854 .26232 Metetra
.7 .728 .872 r
.64854 .26232 .65212 .26787 .65932 .26432 .65576 .25883 Metetra
.701 .731 .874 r
.65576 .25883 .65932 .26432 .66654 .26076 .66299 .25532 Metetra
.702 .734 .876 r
.66299 .25532 .66654 .26076 .67377 .25718 .67024 .25179 Metetra
.702 .737 .878 r
.67024 .25179 .67377 .25718 .68101 .2536 .6775 .24826 Metetra
.703 .74 .88 r
.6775 .24826 .68101 .2536 .68827 .25001 .68477 .24472 Metetra
.704 .743 .882 r
.68477 .24472 .68827 .25001 .69555 .24642 .69206 .24117 Metetra
.704 .746 .884 r
.69206 .24117 .69555 .24642 .70283 .24281 .69936 .23762 Metetra
.705 .749 .886 r
.69936 .23762 .70283 .24281 .71013 .2392 .70668 .23405 Metetra
.706 .752 .888 r
.70668 .23405 .71013 .2392 .71745 .23558 .71401 .23048 Metetra
.706 .755 .89 r
.71401 .23048 .71745 .23558 .72477 .23196 .72136 .2269 Metetra
.707 .758 .892 r
.72136 .2269 .72477 .23196 .73212 .22833 .72872 .22332 Metetra
.707 .761 .894 r
.72872 .22332 .73212 .22833 .73947 .2247 .73609 .21973 Metetra
.707 .764 .896 r
.73609 .21973 .73947 .2247 .74684 .22106 .74348 .21613 Metetra
.708 .767 .898 r
.74348 .21613 .74684 .22106 .75423 .21741 .75088 .21253 Metetra
.708 .77 .9 r
.75088 .21253 .75423 .21741 .76163 .21376 .7583 .20892 Metetra
.709 .762 .894 r
.09647 .43547 .10084 .43928 .10685 .43628 .10248 .43251 Metetra
.709 .765 .896 r
.10248 .43251 .10685 .43628 .11286 .43327 .1085 .42955 Metetra
.71 .768 .898 r
.1085 .42955 .11286 .43327 .11889 .43025 .11453 .42658 Metetra
.71 .771 .9 r
.11453 .42658 .11889 .43025 .12493 .42724 .12057 .4236 Metetra
.711 .774 .902 r
.12057 .4236 .12493 .42724 .13098 .42421 .12662 .42062 Metetra
.711 .777 .903 r
.12662 .42062 .13098 .42421 .13704 .42118 .13269 .41764 Metetra
.711 .78 .905 r
.13269 .41764 .13704 .42118 .14311 .41815 .13876 .41465 Metetra
.712 .783 .907 r
.13876 .41465 .14311 .41815 .14919 .4151 .14484 .41165 Metetra
.712 .786 .909 r
.14484 .41165 .14919 .4151 .15528 .41206 .15094 .40865 Metetra
.713 .789 .911 r
.15094 .40865 .15528 .41206 .16138 .40901 .15705 .40565 Metetra
.713 .792 .913 r
.15705 .40565 .16138 .40901 .1675 .40595 .16316 .40264 Metetra
.713 .795 .915 r
.16316 .40264 .1675 .40595 .17362 .40289 .16929 .39963 Metetra
.713 .798 .918 r
.16929 .39963 .17362 .40289 .17975 .39983 .17543 .39662 Metetra
.713 .802 .92 r
.17543 .39662 .17975 .39983 .1859 .39677 .18158 .39361 Metetra
.713 .805 .922 r
.18158 .39361 .1859 .39677 .19205 .39371 .18774 .3906 Metetra
.713 .809 .925 r
.18774 .3906 .19205 .39371 .19822 .39065 .1939 .38759 Metetra
.712 .812 .927 r
.1939 .38759 .19822 .39065 .20439 .38759 .20008 .38459 Metetra
.712 .816 .93 r
.20008 .38459 .20439 .38759 .21058 .38453 .20627 .38159 Metetra
.711 .82 .932 r
.20627 .38159 .21058 .38453 .21677 .38148 .21248 .3786 Metetra
.711 .824 .935 r
.21248 .3786 .21677 .38148 .22298 .37844 .21869 .37561 Metetra
.709 .827 .938 r
.21869 .37561 .22298 .37844 .22919 .3754 .22491 .37265 Metetra
.708 .831 .941 r
.22491 .37265 .22919 .3754 .23542 .37239 .23113 .36969 Metetra
.706 .835 .944 r
.23113 .36969 .23542 .37239 .24165 .36939 .23737 .36676 Metetra
.704 .84 .948 r
.23737 .36676 .24165 .36939 .24789 .36642 .24362 .36385 Metetra
.7 .844 .951 r
.24362 .36385 .24789 .36642 .25414 .36348 .24988 .36098 Metetra
.697 .848 .955 r
.24988 .36098 .25414 .36348 .2604 .36058 .25614 .35815 Metetra
.692 .852 .959 r
.25614 .35815 .2604 .36058 .26667 .35771 .26242 .35536 Metetra
.686 .856 .964 r
.26242 .35536 .26667 .35771 .27295 .35491 .2687 .35263 Metetra
.679 .859 .968 r
.2687 .35263 .27295 .35491 .27923 .35218 .27499 .34997 Metetra
.671 .863 .972 r
.27499 .34997 .27923 .35218 .28552 .34952 .28128 .34738 Metetra
.661 .865 .977 r
.28128 .34738 .28552 .34952 .29181 .34695 .28758 .34488 Metetra
.648 .867 .981 r
.28758 .34488 .29181 .34695 .29811 .34449 .29389 .34249 Metetra
.634 .868 .986 r
.29389 .34249 .29811 .34449 .30441 .34216 .30021 .34021 Metetra
.617 .867 .99 r
.30021 .34021 .30441 .34216 .31072 .33997 .30652 .33807 Metetra
.597 .864 .993 r
.30652 .33807 .31072 .33997 .31704 .33794 .31285 .33607 Metetra
.575 .86 .996 r
.31285 .33607 .31704 .33794 .32336 .33609 .31918 .33424 Metetra
.55 .852 .998 r
.31918 .33424 .32336 .33609 .32968 .33445 .32551 .33259 Metetra
.524 .842 .999 r
.32551 .33259 .32968 .33445 .33601 .33302 .33185 .33112 Metetra
.497 .83 .999 r
.33185 .33112 .33601 .33302 .34234 .33182 .3382 .32986 Metetra
.47 .815 .998 r
.3382 .32986 .34234 .33182 .34867 .33086 .34456 .3288 Metetra
.446 .798 .997 r
.34456 .3288 .34867 .33086 .35502 .33012 .35092 .32794 Metetra
.425 .781 .996 r
.35092 .32794 .35502 .33012 .36138 .32963 .3573 .32728 Metetra
.408 .763 .995 r
.3573 .32728 .36138 .32963 .36774 .32934 .36368 .3268 Metetra
.397 .746 .993 r
.36368 .3268 .36774 .32934 .37412 .32926 .37008 .3265 Metetra
.391 .73 .99 r
.37008 .3265 .37412 .32926 .38052 .32935 .3765 .32634 Metetra
.39 .714 .987 r
.3765 .32634 .38052 .32935 .38694 .32958 .38294 .32631 Metetra
.394 .699 .982 r
.38294 .32631 .38694 .32958 .39338 .32993 .3894 .32637 Metetra
.401 .685 .976 r
.3894 .32637 .39338 .32993 .39984 .33036 .39588 .32652 Metetra
.412 .673 .969 r
.39588 .32652 .39984 .33036 .40632 .33085 .40238 .32671 Metetra
.425 .662 .959 r
.40238 .32671 .40632 .33085 .41284 .33136 .40891 .32692 Metetra
.44 .652 .949 r
.40891 .32692 .41284 .33136 .41938 .33185 .41547 .32711 Metetra
.456 .644 .938 r
.41547 .32711 .41938 .33185 .42595 .33229 .42206 .32725 Metetra
.472 .636 .927 r
.42206 .32725 .42595 .33229 .43255 .33265 .42867 .32733 Metetra
.489 .631 .915 r
.42867 .32733 .43255 .33265 .43918 .33289 .43532 .3273 Metetra
.505 .626 .904 r
.43532 .3273 .43918 .33289 .44584 .33299 .44199 .32714 Metetra
.521 .622 .893 r
.44199 .32714 .44584 .33299 .45254 .33291 .4487 .32683 Metetra
.536 .621 .883 r
.4487 .32683 .45254 .33291 .45926 .33263 .45543 .32634 Metetra
.551 .62 .874 r
.45543 .32634 .45926 .33263 .46602 .33211 .46219 .32565 Metetra
.565 .621 .866 r
.46219 .32565 .46602 .33211 .4728 .33136 .46898 .32476 Metetra
.578 .622 .86 r
.46898 .32476 .4728 .33136 .47961 .33036 .4758 .32365 Metetra
.59 .625 .854 r
.4758 .32365 .47961 .33036 .48645 .3291 .48264 .32232 Metetra
.602 .628 .85 r
.48264 .32232 .48645 .3291 .49331 .3276 .48951 .32077 Metetra
.612 .632 .846 r
.48951 .32077 .49331 .3276 .5002 .32586 .4964 .31902 Metetra
.622 .636 .844 r
.4964 .31902 .5002 .32586 .5071 .3239 .50331 .31707 Metetra
.63 .641 .843 r
.50331 .31707 .5071 .3239 .51403 .32174 .51024 .31494 Metetra
.638 .646 .842 r
.51024 .31494 .51403 .32174 .52097 .31939 .51719 .31264 Metetra
.645 .651 .842 r
.51719 .31264 .52097 .31939 .52794 .31688 .52416 .31019 Metetra
.651 .657 .842 r
.52416 .31019 .52794 .31688 .53491 .31423 .53115 .3076 Metetra
.657 .662 .843 r
.53115 .3076 .53491 .31423 .54191 .31146 .53815 .3049 Metetra
.662 .667 .844 r
.53815 .3049 .54191 .31146 .54892 .30858 .54517 .30209 Metetra
.667 .672 .845 r
.54517 .30209 .54892 .30858 .55595 .3056 .5522 .29918 Metetra
.671 .677 .847 r
.5522 .29918 .55595 .3056 .56299 .30253 .55925 .2962 Metetra
.675 .682 .849 r
.55925 .2962 .56299 .30253 .57004 .2994 .56631 .29314 Metetra
.678 .686 .851 r
.56631 .29314 .57004 .2994 .57711 .29621 .57339 .29002 Metetra
.681 .69 .853 r
.57339 .29002 .57711 .29621 .58419 .29296 .58049 .28684 Metetra
.683 .694 .854 r
.58049 .28684 .58419 .29296 .59128 .28967 .58759 .28362 Metetra
.685 .699 .857 r
.58759 .28362 .59128 .28967 .59839 .28634 .59471 .28036 Metetra
.687 .702 .859 r
.59471 .28036 .59839 .28634 .60552 .28298 .60185 .27706 Metetra
.689 .706 .861 r
.60185 .27706 .60552 .28298 .61265 .27959 .609 .27373 Metetra
.691 .71 .863 r
.609 .27373 .61265 .27959 .6198 .27617 .61616 .27038 Metetra
.692 .714 .865 r
.61616 .27038 .6198 .27617 .62697 .27274 .62334 .267 Metetra
.694 .717 .867 r
.62334 .267 .62697 .27274 .63414 .26928 .63053 .26361 Metetra
.695 .72 .869 r
.63053 .26361 .63414 .26928 .64134 .26581 .63773 .26019 Metetra
.696 .724 .871 r
.63773 .26019 .64134 .26581 .64854 .26232 .64495 .25676 Metetra
.697 .727 .873 r
.64495 .25676 .64854 .26232 .65576 .25883 .65218 .25332 Metetra
.698 .73 .875 r
.65218 .25332 .65576 .25883 .66299 .25532 .65943 .24986 Metetra
.699 .733 .877 r
.65943 .24986 .66299 .25532 .67024 .25179 .66669 .24639 Metetra
.7 .736 .879 r
.66669 .24639 .67024 .25179 .6775 .24826 .67397 .2429 Metetra
.7 .739 .881 r
.67397 .2429 .6775 .24826 .68477 .24472 .68126 .23941 Metetra
.701 .742 .883 r
.68126 .23941 .68477 .24472 .69206 .24117 .68856 .23591 Metetra
.702 .745 .885 r
.68856 .23591 .69206 .24117 .69936 .23762 .69588 .2324 Metetra
.702 .748 .887 r
.69588 .2324 .69936 .23762 .70668 .23405 .70321 .22888 Metetra
.703 .751 .889 r
.70321 .22888 .70668 .23405 .71401 .23048 .71056 .22535 Metetra
.703 .754 .891 r
.71056 .22535 .71401 .23048 .72136 .2269 .71793 .22182 Metetra
.704 .757 .893 r
.71793 .22182 .72136 .2269 .72872 .22332 .7253 .21828 Metetra
.704 .76 .895 r
.7253 .21828 .72872 .22332 .73609 .21973 .73269 .21473 Metetra
.705 .763 .897 r
.73269 .21473 .73609 .21973 .74348 .21613 .7401 .21118 Metetra
.705 .766 .899 r
.7401 .21118 .74348 .21613 .75088 .21253 .74752 .20762 Metetra
.706 .769 .901 r
.74752 .20762 .75088 .21253 .7583 .20892 .75496 .20405 Metetra
.706 .762 .896 r
.09209 .43164 .09647 .43547 .10248 .43251 .0981 .42873 Metetra
.706 .765 .898 r
.0981 .42873 .10248 .43251 .1085 .42955 .10412 .42581 Metetra
.707 .767 .899 r
.10412 .42581 .1085 .42955 .11453 .42658 .11015 .42288 Metetra
.707 .77 .901 r
.11015 .42288 .11453 .42658 .12057 .4236 .1162 .41995 Metetra
.708 .773 .903 r
.1162 .41995 .12057 .4236 .12662 .42062 .12225 .41702 Metetra
.708 .776 .905 r
.12225 .41702 .12662 .42062 .13269 .41764 .12832 .41408 Metetra
.709 .779 .907 r
.12832 .41408 .13269 .41764 .13876 .41465 .13439 .41113 Metetra
.709 .782 .909 r
.13439 .41113 .13876 .41465 .14484 .41165 .14048 .40818 Metetra
.709 .785 .911 r
.14048 .40818 .14484 .41165 .15094 .40865 .14658 .40523 Metetra
.709 .788 .913 r
.14658 .40523 .15094 .40865 .15705 .40565 .15269 .40227 Metetra
.71 .791 .915 r
.15269 .40227 .15705 .40565 .16316 .40264 .15881 .39931 Metetra
.71 .794 .917 r
.15881 .39931 .16316 .40264 .16929 .39963 .16494 .39635 Metetra
.71 .798 .919 r
.16494 .39635 .16929 .39963 .17543 .39662 .17108 .39339 Metetra
.71 .801 .922 r
.17108 .39339 .17543 .39662 .18158 .39361 .17724 .39043 Metetra
.709 .804 .924 r
.17724 .39043 .18158 .39361 .18774 .3906 .1834 .38747 Metetra
.709 .808 .926 r
.1834 .38747 .18774 .3906 .1939 .38759 .18957 .38451 Metetra
.709 .811 .929 r
.18957 .38451 .1939 .38759 .20008 .38459 .19576 .38156 Metetra
.708 .815 .931 r
.19576 .38156 .20008 .38459 .20627 .38159 .20195 .37862 Metetra
.707 .818 .934 r
.20195 .37862 .20627 .38159 .21248 .3786 .20816 .37568 Metetra
.706 .822 .937 r
.20816 .37568 .21248 .3786 .21869 .37561 .21437 .37276 Metetra
.705 .826 .94 r
.21437 .37276 .21869 .37561 .22491 .37265 .2206 .36985 Metetra
.703 .829 .943 r
.2206 .36985 .22491 .37265 .23113 .36969 .22683 .36695 Metetra
.701 .833 .946 r
.22683 .36695 .23113 .36969 .23737 .36676 .23308 .36408 Metetra
.699 .837 .949 r
.23308 .36408 .23737 .36676 .24362 .36385 .23933 .36124 Metetra
.695 .841 .953 r
.23933 .36124 .24362 .36385 .24988 .36098 .24559 .35843 Metetra
.691 .844 .956 r
.24559 .35843 .24988 .36098 .25614 .35815 .25187 .35566 Metetra
.687 .848 .96 r
.25187 .35566 .25614 .35815 .26242 .35536 .25815 .35294 Metetra
.681 .851 .964 r
.25815 .35294 .26242 .35536 .2687 .35263 .26443 .35027 Metetra
.674 .854 .968 r
.26443 .35027 .2687 .35263 .27499 .34997 .27073 .34767 Metetra
.666 .857 .972 r
.27073 .34767 .27499 .34997 .28128 .34738 .27703 .34514 Metetra
.657 .859 .977 r
.27703 .34514 .28128 .34738 .28758 .34488 .28334 .3427 Metetra
.646 .86 .981 r
.28334 .3427 .28758 .34488 .29389 .34249 .28966 .34035 Metetra
.633 .86 .985 r
.28966 .34035 .29389 .34249 .30021 .34021 .29598 .33811 Metetra
.618 .859 .989 r
.29598 .33811 .30021 .34021 .30652 .33807 .30231 .336 Metetra
.601 .857 .992 r
.30231 .336 .30652 .33807 .31285 .33607 .30865 .33402 Metetra
.582 .852 .995 r
.30865 .33402 .31285 .33607 .31918 .33424 .31499 .33219 Metetra
.562 .846 .997 r
.31499 .33219 .31918 .33424 .32551 .33259 .32134 .33052 Metetra
.541 .837 .999 r
.32134 .33052 .32551 .33259 .33185 .33112 .32769 .32901 Metetra
.52 .827 1 r
.32769 .32901 .33185 .33112 .3382 .32986 .33405 .32768 Metetra
.498 .815 1 r
.33405 .32768 .3382 .32986 .34456 .3288 .34043 .32652 Metetra
.479 .801 1 r
.34043 .32652 .34456 .3288 .35092 .32794 .34681 .32554 Metetra
.462 .786 .999 r
.34681 .32554 .35092 .32794 .3573 .32728 .3532 .32473 Metetra
.449 .772 .997 r
.3532 .32473 .3573 .32728 .36368 .3268 .35961 .32407 Metetra
.439 .756 .995 r
.35961 .32407 .36368 .3268 .37008 .3265 .36603 .32357 Metetra
.434 .741 .992 r
.36603 .32357 .37008 .3265 .3765 .32634 .37247 .32319 Metetra
.433 .727 .988 r
.37247 .32319 .3765 .32634 .38294 .32631 .37893 .32291 Metetra
.434 .713 .982 r
.37893 .32291 .38294 .32631 .3894 .32637 .3854 .32273 Metetra
.439 .7 .976 r
.3854 .32273 .3894 .32637 .39588 .32652 .3919 .32261 Metetra
.447 .687 .968 r
.3919 .32261 .39588 .32652 .40238 .32671 .39842 .32254 Metetra
.457 .677 .959 r
.39842 .32254 .40238 .32671 .40891 .32692 .40497 .32247 Metetra
.468 .667 .949 r
.40497 .32247 .40891 .32692 .41547 .32711 .41155 .32239 Metetra
.48 .658 .939 r
.41155 .32239 .41547 .32711 .42206 .32725 .41815 .32227 Metetra
.494 .651 .929 r
.41815 .32227 .42206 .32725 .42867 .32733 .42478 .32209 Metetra
.507 .644 .918 r
.42478 .32209 .42867 .32733 .43532 .3273 .43143 .32181 Metetra
.521 .64 .907 r
.43143 .32181 .43532 .3273 .44199 .32714 .43812 .32142 Metetra
.534 .636 .898 r
.43812 .32142 .44199 .32714 .4487 .32683 .44483 .32089 Metetra
.547 .633 .888 r
.44483 .32089 .4487 .32683 .45543 .32634 .45158 .32022 Metetra
.56 .632 .88 r
.45158 .32022 .45543 .32634 .46219 .32565 .45835 .31937 Metetra
.572 .632 .873 r
.45835 .31937 .46219 .32565 .46898 .32476 .46515 .31834 Metetra
.583 .633 .866 r
.46515 .31834 .46898 .32476 .4758 .32365 .47197 .31713 Metetra
.594 .634 .861 r
.47197 .31713 .4758 .32365 .48264 .32232 .47882 .31572 Metetra
.604 .637 .856 r
.47882 .31572 .48264 .32232 .48951 .32077 .48569 .31413 Metetra
.613 .64 .853 r
.48569 .31413 .48951 .32077 .4964 .31902 .49259 .31235 Metetra
.622 .643 .85 r
.49259 .31235 .4964 .31902 .50331 .31707 .4995 .3104 Metetra
.63 .647 .849 r
.4995 .3104 .50331 .31707 .51024 .31494 .50644 .30828 Metetra
.637 .651 .848 r
.50644 .30828 .51024 .31494 .51719 .31264 .5134 .30601 Metetra
.644 .656 .847 r
.5134 .30601 .51719 .31264 .52416 .31019 .52037 .3036 Metetra
.65 .66 .847 r
.52037 .3036 .52416 .31019 .53115 .3076 .52737 .30107 Metetra
.655 .665 .848 r
.52737 .30107 .53115 .3076 .53815 .3049 .53438 .29841 Metetra
.66 .669 .848 r
.53438 .29841 .53815 .3049 .54517 .30209 .5414 .29566 Metetra
.664 .674 .849 r
.5414 .29566 .54517 .30209 .5522 .29918 .54845 .29282 Metetra
.668 .678 .85 r
.54845 .29282 .5522 .29918 .55925 .2962 .5555 .2899 Metetra
.672 .683 .852 r
.5555 .2899 .55925 .2962 .56631 .29314 .56258 .2869 Metetra
.675 .687 .853 r
.56258 .2869 .56631 .29314 .57339 .29002 .56967 .28385 Metetra
.677 .691 .855 r
.56967 .28385 .57339 .29002 .58049 .28684 .57677 .28074 Metetra
.68 .695 .857 r
.57677 .28074 .58049 .28684 .58759 .28362 .58389 .27758 Metetra
.682 .699 .859 r
.58389 .27758 .58759 .28362 .59471 .28036 .59102 .27438 Metetra
.684 .702 .861 r
.59102 .27438 .59471 .28036 .60185 .27706 .59816 .27114 Metetra
.686 .706 .863 r
.59816 .27114 .60185 .27706 .609 .27373 .60533 .26788 Metetra
.688 .71 .865 r
.60533 .26788 .609 .27373 .61616 .27038 .6125 .26458 Metetra
.689 .713 .867 r
.6125 .26458 .61616 .27038 .62334 .267 .61969 .26126 Metetra
.691 .716 .869 r
.61969 .26126 .62334 .267 .63053 .26361 .62689 .25792 Metetra
.692 .72 .871 r
.62689 .25792 .63053 .26361 .63773 .26019 .63411 .25456 Metetra
.693 .723 .873 r
.63411 .25456 .63773 .26019 .64495 .25676 .64135 .25118 Metetra
.694 .726 .875 r
.64135 .25118 .64495 .25676 .65218 .25332 .64859 .24779 Metetra
.695 .729 .877 r
.64859 .24779 .65218 .25332 .65943 .24986 .65585 .24438 Metetra
.696 .732 .879 r
.65585 .24438 .65943 .24986 .66669 .24639 .66313 .24096 Metetra
.697 .735 .881 r
.66313 .24096 .66669 .24639 .67397 .2429 .67042 .23752 Metetra
.697 .738 .883 r
.67042 .23752 .67397 .2429 .68126 .23941 .67773 .23408 Metetra
.698 .741 .885 r
.67773 .23408 .68126 .23941 .68856 .23591 .68505 .23062 Metetra
.699 .744 .887 r
.68505 .23062 .68856 .23591 .69588 .2324 .69238 .22716 Metetra
.7 .747 .889 r
.69238 .22716 .69588 .2324 .70321 .22888 .69973 .22368 Metetra
.7 .75 .891 r
.69973 .22368 .70321 .22888 .71056 .22535 .7071 .22021 Metetra
.701 .753 .893 r
.7071 .22021 .71056 .22535 .71793 .22182 .71448 .21672 Metetra
.701 .756 .894 r
.71448 .21672 .71793 .22182 .7253 .21828 .72187 .21322 Metetra
.702 .759 .896 r
.72187 .21322 .7253 .21828 .73269 .21473 .72928 .20972 Metetra
.702 .762 .898 r
.72928 .20972 .73269 .21473 .7401 .21118 .73671 .20621 Metetra
.702 .765 .9 r
.73671 .20621 .7401 .21118 .74752 .20762 .74415 .20269 Metetra
.703 .768 .902 r
.74415 .20269 .74752 .20762 .75496 .20405 .7516 .19917 Metetra
.703 .761 .897 r
.08768 .42779 .09209 .43164 .0981 .42873 .09369 .42492 Metetra
.703 .764 .899 r
.09369 .42492 .0981 .42873 .10412 .42581 .09972 .42205 Metetra
.704 .767 .901 r
.09972 .42205 .10412 .42581 .11015 .42288 .10575 .41917 Metetra
.704 .77 .903 r
.10575 .41917 .11015 .42288 .1162 .41995 .1118 .41628 Metetra
.705 .773 .905 r
.1118 .41628 .1162 .41995 .12225 .41702 .11786 .41339 Metetra
.705 .776 .907 r
.11786 .41339 .12225 .41702 .12832 .41408 .12393 .4105 Metetra
.705 .778 .909 r
.12393 .4105 .12832 .41408 .13439 .41113 .13 .4076 Metetra
.706 .781 .911 r
.13 .4076 .13439 .41113 .14048 .40818 .1361 .4047 Metetra
.706 .784 .913 r
.1361 .4047 .14048 .40818 .14658 .40523 .1422 .40179 Metetra
.706 .788 .915 r
.1422 .40179 .14658 .40523 .15269 .40227 .14831 .39888 Metetra
.706 .791 .917 r
.14831 .39888 .15269 .40227 .15881 .39931 .15443 .39597 Metetra
.706 .794 .919 r
.15443 .39597 .15881 .39931 .16494 .39635 .16057 .39305 Metetra
.706 .797 .921 r
.16057 .39305 .16494 .39635 .17108 .39339 .16672 .39014 Metetra
.706 .8 .923 r
.16672 .39014 .17108 .39339 .17724 .39043 .17287 .38723 Metetra
.706 .804 .926 r
.17287 .38723 .17724 .39043 .1834 .38747 .17904 .38432 Metetra
.705 .807 .928 r
.17904 .38432 .1834 .38747 .18957 .38451 .18522 .38141 Metetra
.705 .81 .93 r
.18522 .38141 .18957 .38451 .19576 .38156 .19141 .37851 Metetra
.704 .813 .933 r
.19141 .37851 .19576 .38156 .20195 .37862 .19761 .37562 Metetra
.703 .817 .936 r
.19761 .37562 .20195 .37862 .20816 .37568 .20382 .37273 Metetra
.702 .82 .938 r
.20382 .37273 .20816 .37568 .21437 .37276 .21004 .36986 Metetra
.701 .824 .941 r
.21004 .36986 .21437 .37276 .2206 .36985 .21627 .36701 Metetra
.699 .827 .944 r
.21627 .36701 .2206 .36985 .22683 .36695 .22251 .36417 Metetra
.697 .831 .947 r
.22251 .36417 .22683 .36695 .23308 .36408 .22876 .36136 Metetra
.694 .834 .95 r
.22876 .36136 .23308 .36408 .23933 .36124 .23502 .35857 Metetra
.691 .837 .954 r
.23502 .35857 .23933 .36124 .24559 .35843 .24129 .35582 Metetra
.687 .841 .957 r
.24129 .35582 .24559 .35843 .25187 .35566 .24757 .3531 Metetra
.682 .844 .961 r
.24757 .3531 .25187 .35566 .25815 .35294 .25386 .35044 Metetra
.677 .847 .964 r
.25386 .35044 .25815 .35294 .26443 .35027 .26015 .34782 Metetra
.67 .849 .968 r
.26015 .34782 .26443 .35027 .27073 .34767 .26645 .34527 Metetra
.663 .851 .972 r
.26645 .34527 .27073 .34767 .27703 .34514 .27277 .34279 Metetra
.654 .853 .976 r
.27277 .34279 .27703 .34514 .28334 .3427 .27908 .34039 Metetra
.644 .853 .98 r
.27908 .34039 .28334 .3427 .28966 .34035 .28541 .33808 Metetra
.633 .853 .984 r
.28541 .33808 .28966 .34035 .29598 .33811 .29174 .33588 Metetra
.619 .852 .987 r
.29174 .33588 .29598 .33811 .30231 .336 .29808 .33378 Metetra
.605 .85 .99 r
.29808 .33378 .30231 .336 .30865 .33402 .30443 .33181 Metetra
.589 .846 .993 r
.30443 .33181 .30865 .33402 .31499 .33219 .31078 .32997 Metetra
.573 .84 .995 r
.31078 .32997 .31499 .33219 .32134 .33052 .31715 .32826 Metetra
.555 .832 .997 r
.31715 .32826 .32134 .33052 .32769 .32901 .32352 .32671 Metetra
.538 .824 .998 r
.32352 .32671 .32769 .32901 .33405 .32768 .3299 .32531 Metetra
.521 .813 .999 r
.3299 .32531 .33405 .32768 .34043 .32652 .33628 .32406 Metetra
.506 .802 .999 r
.33628 .32406 .34043 .32652 .34681 .32554 .34268 .32296 Metetra
.492 .789 .998 r
.34268 .32296 .34681 .32554 .3532 .32473 .3491 .322 Metetra
.481 .776 .996 r
.3491 .322 .3532 .32473 .35961 .32407 .35552 .32119 Metetra
.474 .763 .994 r
.35552 .32119 .35961 .32407 .36603 .32357 .36196 .32049 Metetra
.469 .749 .99 r
.36196 .32049 .36603 .32357 .37247 .32319 .36842 .31991 Metetra
.467 .736 .986 r
.36842 .31991 .37247 .32319 .37893 .32291 .37489 .31942 Metetra
.467 .723 .98 r
.37489 .31942 .37893 .32291 .3854 .32273 .38139 .31901 Metetra
.47 .71 .974 r
.38139 .31901 .3854 .32273 .3919 .32261 .38791 .31865 Metetra
.476 .699 .966 r
.38791 .31865 .3919 .32261 .39842 .32254 .39445 .31833 Metetra
.483 .688 .958 r
.39445 .31833 .39842 .32254 .40497 .32247 .40101 .31802 Metetra
.492 .679 .949 r
.40101 .31802 .40497 .32247 .41155 .32239 .4076 .3177 Metetra
.502 .671 .939 r
.4076 .3177 .41155 .32239 .41815 .32227 .41422 .31733 Metetra
.512 .663 .929 r
.41422 .31733 .41815 .32227 .42478 .32209 .42086 .31692 Metetra
.524 .657 .92 r
.42086 .31692 .42478 .32209 .43143 .32181 .42753 .31641 Metetra
.535 .652 .91 r
.42753 .31641 .43143 .32181 .43812 .32142 .43423 .31581 Metetra
.546 .648 .901 r
.43423 .31581 .43812 .32142 .44483 .32089 .44096 .31509 Metetra
.557 .645 .892 r
.44096 .31509 .44483 .32089 .45158 .32022 .44771 .31424 Metetra
.568 .643 .885 r
.44771 .31424 .45158 .32022 .45835 .31937 .45449 .31325 Metetra
.578 .642 .878 r
.45449 .31325 .45835 .31937 .46515 .31834 .46129 .31209 Metetra
.588 .642 .872 r
.46129 .31209 .46515 .31834 .47197 .31713 .46812 .31078 Metetra
.598 .643 .866 r
.46812 .31078 .47197 .31713 .47882 .31572 .47498 .30929 Metetra
.607 .645 .862 r
.47498 .30929 .47882 .31572 .48569 .31413 .48186 .30764 Metetra
.615 .647 .859 r
.48186 .30764 .48569 .31413 .49259 .31235 .48876 .30583 Metetra
.623 .65 .856 r
.48876 .30583 .49259 .31235 .4995 .3104 .49568 .30386 Metetra
.63 .653 .854 r
.49568 .30386 .4995 .3104 .50644 .30828 .50263 .30175 Metetra
.637 .656 .853 r
.50263 .30175 .50644 .30828 .5134 .30601 .50959 .29949 Metetra
.643 .66 .852 r
.50959 .29949 .5134 .30601 .52037 .3036 .51657 .29711 Metetra
.648 .664 .852 r
.51657 .29711 .52037 .3036 .52737 .30107 .52357 .29461 Metetra
.653 .668 .852 r
.52357 .29461 .52737 .30107 .53438 .29841 .53059 .29201 Metetra
.658 .672 .852 r
.53059 .29201 .53438 .29841 .5414 .29566 .53763 .2893 Metetra
.662 .676 .853 r
.53763 .2893 .5414 .29566 .54845 .29282 .54468 .28651 Metetra
.666 .68 .854 r
.54468 .28651 .54845 .29282 .5555 .2899 .55174 .28364 Metetra
.669 .684 .855 r
.55174 .28364 .5555 .2899 .56258 .2869 .55883 .2807 Metetra
.672 .688 .856 r
.55883 .2807 .56258 .2869 .56967 .28385 .56592 .27771 Metetra
.675 .692 .858 r
.56592 .27771 .56967 .28385 .57677 .28074 .57304 .27465 Metetra
.677 .695 .859 r
.57304 .27465 .57677 .28074 .58389 .27758 .58017 .27155 Metetra
.68 .699 .861 r
.58017 .27155 .58389 .27758 .59102 .27438 .58731 .26841 Metetra
.682 .702 .863 r
.58731 .26841 .59102 .27438 .59816 .27114 .59447 .26523 Metetra
.683 .706 .865 r
.59447 .26523 .59816 .27114 .60533 .26788 .60164 .26202 Metetra
.685 .709 .866 r
.60164 .26202 .60533 .26788 .6125 .26458 .60883 .25878 Metetra
.687 .713 .868 r
.60883 .25878 .6125 .26458 .61969 .26126 .61603 .25551 Metetra
.688 .716 .87 r
.61603 .25551 .61969 .26126 .62689 .25792 .62325 .25222 Metetra
.689 .719 .872 r
.62325 .25222 .62689 .25792 .63411 .25456 .63048 .24892 Metetra
.69 .722 .874 r
.63048 .24892 .63411 .25456 .64135 .25118 .63773 .24559 Metetra
.691 .725 .876 r
.63773 .24559 .64135 .25118 .64859 .24779 .64499 .24224 Metetra
.692 .728 .878 r
.64499 .24224 .64859 .24779 .65585 .24438 .65226 .23888 Metetra
.693 .731 .88 r
.65226 .23888 .65585 .24438 .66313 .24096 .65955 .23551 Metetra
.694 .734 .882 r
.65955 .23551 .66313 .24096 .67042 .23752 .66686 .23212 Metetra
.695 .737 .884 r
.66686 .23212 .67042 .23752 .67773 .23408 .67418 .22872 Metetra
.695 .74 .886 r
.67418 .22872 .67773 .23408 .68505 .23062 .68152 .22532 Metetra
.696 .743 .888 r
.68152 .22532 .68505 .23062 .69238 .22716 .68887 .2219 Metetra
.697 .746 .89 r
.68887 .2219 .69238 .22716 .69973 .22368 .69623 .21847 Metetra
.697 .749 .892 r
.69623 .21847 .69973 .22368 .7071 .22021 .70362 .21503 Metetra
.698 .752 .894 r
.70362 .21503 .7071 .22021 .71448 .21672 .71101 .21159 Metetra
.698 .755 .896 r
.71101 .21159 .71448 .21672 .72187 .21322 .71842 .20814 Metetra
.699 .758 .898 r
.71842 .20814 .72187 .21322 .72928 .20972 .72585 .20468 Metetra
.699 .761 .9 r
.72585 .20468 .72928 .20972 .73671 .20621 .73329 .20121 Metetra
.7 .764 .902 r
.73329 .20121 .73671 .20621 .74415 .20269 .74075 .19773 Metetra
.7 .767 .903 r
.74075 .19773 .74415 .20269 .7516 .19917 .74823 .19425 Metetra
.7 .761 .899 r
.08326 .42393 .08768 .42779 .09369 .42492 .08927 .42111 Metetra
.701 .764 .901 r
.08927 .42111 .09369 .42492 .09972 .42205 .0953 .41828 Metetra
.701 .766 .903 r
.0953 .41828 .09972 .42205 .10575 .41917 .10133 .41544 Metetra
.701 .769 .905 r
.10133 .41544 .10575 .41917 .1118 .41628 .10738 .4126 Metetra
.702 .772 .906 r
.10738 .4126 .1118 .41628 .11786 .41339 .11344 .40975 Metetra
.702 .775 .908 r
.11344 .40975 .11786 .41339 .12393 .4105 .11951 .4069 Metetra
.702 .778 .91 r
.11951 .4069 .12393 .4105 .13 .4076 .1256 .40405 Metetra
.702 .781 .912 r
.1256 .40405 .13 .4076 .1361 .4047 .13169 .40119 Metetra
.703 .784 .914 r
.13169 .40119 .1361 .4047 .1422 .40179 .1378 .39833 Metetra
.703 .787 .916 r
.1378 .39833 .1422 .40179 .14831 .39888 .14391 .39547 Metetra
.703 .79 .918 r
.14391 .39547 .14831 .39888 .15443 .39597 .15004 .3926 Metetra
.703 .793 .921 r
.15004 .3926 .15443 .39597 .16057 .39305 .15618 .38973 Metetra
.703 .796 .923 r
.15618 .38973 .16057 .39305 .16672 .39014 .16233 .38687 Metetra
.703 .799 .925 r
.16233 .38687 .16672 .39014 .17287 .38723 .16849 .384 Metetra
.702 .802 .927 r
.16849 .384 .17287 .38723 .17904 .38432 .17466 .38114 Metetra
.702 .806 .93 r
.17466 .38114 .17904 .38432 .18522 .38141 .18085 .37828 Metetra
.701 .809 .932 r
.18085 .37828 .18522 .38141 .19141 .37851 .18704 .37543 Metetra
.7 .812 .934 r
.18704 .37543 .19141 .37851 .19761 .37562 .19325 .37259 Metetra
.699 .815 .937 r
.19325 .37259 .19761 .37562 .20382 .37273 .19946 .36976 Metetra
.698 .819 .94 r
.19946 .36976 .20382 .37273 .21004 .36986 .20569 .36694 Metetra
.697 .822 .942 r
.20569 .36694 .21004 .36986 .21627 .36701 .21192 .36413 Metetra
.695 .825 .945 r
.21192 .36413 .21627 .36701 .22251 .36417 .21817 .36134 Metetra
.693 .828 .948 r
.21817 .36134 .22251 .36417 .22876 .36136 .22443 .35858 Metetra
.69 .831 .951 r
.22443 .35858 .22876 .36136 .23502 .35857 .23069 .35585 Metetra
.687 .835 .954 r
.23069 .35585 .23502 .35857 .24129 .35582 .23697 .35314 Metetra
.683 .837 .957 r
.23697 .35314 .24129 .35582 .24757 .3531 .24325 .35048 Metetra
.679 .84 .961 r
.24325 .35048 .24757 .3531 .25386 .35044 .24955 .34786 Metetra
.673 .842 .964 r
.24955 .34786 .25386 .35044 .26015 .34782 .25585 .34529 Metetra
.667 .845 .968 r
.25585 .34529 .26015 .34782 .26645 .34527 .26216 .34278 Metetra
.66 .846 .971 r
.26216 .34278 .26645 .34527 .27277 .34279 .26848 .34034 Metetra
.652 .847 .975 r
.26848 .34034 .27277 .34279 .27908 .34039 .27481 .33798 Metetra
.643 .848 .979 r
.27481 .33798 .27908 .34039 .28541 .33808 .28115 .3357 Metetra
.633 .847 .982 r
.28115 .3357 .28541 .33808 .29174 .33588 .28749 .33351 Metetra
.622 .846 .985 r
.28749 .33351 .29174 .33588 .29808 .33378 .29384 .33142 Metetra
.609 .843 .988 r
.29384 .33142 .29808 .33378 .30443 .33181 .3002 .32944 Metetra
.596 .839 .991 r
.3002 .32944 .30443 .33181 .31078 .32997 .30657 .32758 Metetra
.582 .834 .993 r
.30657 .32758 .31078 .32997 .31715 .32826 .31294 .32584 Metetra
.568 .828 .994 r
.31294 .32584 .31715 .32826 .32352 .32671 .31933 .32424 Metetra
.554 .82 .995 r
.31933 .32424 .32352 .32671 .3299 .32531 .32572 .32276 Metetra
.54 .811 .996 r
.32572 .32276 .3299 .32531 .33628 .32406 .33213 .32142 Metetra
.528 .801 .996 r
.33213 .32142 .33628 .32406 .34268 .32296 .33854 .32021 Metetra
.517 .79 .995 r
.33854 .32021 .34268 .32296 .3491 .322 .34497 .31912 Metetra
.508 .778 .993 r
.34497 .31912 .3491 .322 .35552 .32119 .35142 .31815 Metetra
.501 .766 .991 r
.35142 .31815 .35552 .32119 .36196 .32049 .35788 .31729 Metetra
.497 .754 .987 r
.35788 .31729 .36196 .32049 .36842 .31991 .36435 .31652 Metetra
.495 .743 .983 r
.36435 .31652 .36842 .31991 .37489 .31942 .37085 .31583 Metetra
.494 .73 .977 r
.37085 .31583 .37489 .31942 .38139 .31901 .37736 .31521 Metetra
.497 .719 .971 r
.37736 .31521 .38139 .31901 .38791 .31865 .3839 .31464 Metetra
.5 .708 .964 r
.3839 .31464 .38791 .31865 .39445 .31833 .39046 .3141 Metetra
.506 .698 .956 r
.39046 .3141 .39445 .31833 .40101 .31802 .39704 .31356 Metetra
.513 .689 .947 r
.39704 .31356 .40101 .31802 .4076 .3177 .40364 .31301 Metetra
.521 .681 .938 r
.40364 .31301 .4076 .3177 .41422 .31733 .41027 .31243 Metetra
.53 .674 .93 r
.41027 .31243 .41422 .31733 .42086 .31692 .41693 .31179 Metetra
.538 .667 .92 r
.41693 .31179 .42086 .31692 .42753 .31641 .42361 .31109 Metetra
.548 .662 .912 r
.42361 .31109 .42753 .31641 .43423 .31581 .43032 .3103 Metetra
.557 .658 .904 r
.43032 .3103 .43423 .31581 .44096 .31509 .43706 .3094 Metetra
.566 .655 .896 r
.43706 .3094 .44096 .31509 .44771 .31424 .44382 .30839 Metetra
.576 .653 .889 r
.44382 .30839 .44771 .31424 .45449 .31325 .45061 .30726 Metetra
.585 .651 .882 r
.45061 .30726 .45449 .31325 .46129 .31209 .45742 .30598 Metetra
.593 .651 .876 r
.45742 .30598 .46129 .31209 .46812 .31078 .46426 .30456 Metetra
.601 .651 .871 r
.46426 .30456 .46812 .31078 .47498 .30929 .47112 .303 Metetra
.609 .652 .867 r
.47112 .303 .47498 .30929 .48186 .30764 .47801 .30129 Metetra
.617 .654 .864 r
.47801 .30129 .48186 .30764 .48876 .30583 .48492 .29944 Metetra
.624 .656 .861 r
.48492 .29944 .48876 .30583 .49568 .30386 .49185 .29745 Metetra
.63 .658 .859 r
.49185 .29745 .49568 .30386 .50263 .30175 .4988 .29532 Metetra
.636 .661 .857 r
.4988 .29532 .50263 .30175 .50959 .29949 .50577 .29307 Metetra
.642 .664 .856 r
.50577 .29307 .50959 .29949 .51657 .29711 .51276 .29071 Metetra
.647 .668 .856 r
.51276 .29071 .51657 .29711 .52357 .29461 .51977 .28824 Metetra
.652 .671 .856 r
.51977 .28824 .52357 .29461 .53059 .29201 .52679 .28566 Metetra
.656 .675 .856 r
.52679 .28566 .53059 .29201 .53763 .2893 .53383 .283 Metetra
.66 .678 .856 r
.53383 .283 .53763 .2893 .54468 .28651 .54089 .28025 Metetra
.664 .682 .857 r
.54089 .28025 .54468 .28651 .55174 .28364 .54797 .27743 Metetra
.667 .685 .858 r
.54797 .27743 .55174 .28364 .55883 .2807 .55506 .27454 Metetra
.67 .689 .859 r
.55506 .27454 .55883 .2807 .56592 .27771 .56217 .27159 Metetra
.672 .692 .86 r
.56217 .27159 .56592 .27771 .57304 .27465 .56929 .26859 Metetra
.675 .696 .862 r
.56929 .26859 .57304 .27465 .58017 .27155 .57643 .26554 Metetra
.677 .699 .863 r
.57643 .26554 .58017 .27155 .58731 .26841 .58359 .26245 Metetra
.679 .703 .865 r
.58359 .26245 .58731 .26841 .59447 .26523 .59076 .25932 Metetra
.681 .706 .867 r
.59076 .25932 .59447 .26523 .60164 .26202 .59794 .25616 Metetra
.682 .709 .868 r
.59794 .25616 .60164 .26202 .60883 .25878 .60514 .25297 Metetra
.684 .712 .87 r
.60514 .25297 .60883 .25878 .61603 .25551 .61236 .24975 Metetra
.685 .715 .872 r
.61236 .24975 .61603 .25551 .62325 .25222 .61959 .24652 Metetra
.686 .719 .874 r
.61959 .24652 .62325 .25222 .63048 .24892 .62683 .24325 Metetra
.687 .722 .875 r
.62683 .24325 .63048 .24892 .63773 .24559 .63409 .23998 Metetra
.689 .725 .877 r
.63409 .23998 .63773 .24559 .64499 .24224 .64137 .23668 Metetra
.689 .728 .879 r
.64137 .23668 .64499 .24224 .65226 .23888 .64866 .23337 Metetra
.69 .731 .881 r
.64866 .23337 .65226 .23888 .65955 .23551 .65596 .23004 Metetra
.691 .734 .883 r
.65596 .23004 .65955 .23551 .66686 .23212 .66328 .2267 Metetra
.692 .737 .885 r
.66328 .2267 .66686 .23212 .67418 .22872 .67062 .22335 Metetra
.693 .74 .887 r
.67062 .22335 .67418 .22872 .68152 .22532 .67797 .21999 Metetra
.693 .742 .889 r
.67797 .21999 .68152 .22532 .68887 .2219 .68534 .21661 Metetra
.694 .745 .891 r
.68534 .21661 .68887 .2219 .69623 .21847 .69272 .21323 Metetra
.694 .748 .893 r
.69272 .21323 .69623 .21847 .70362 .21503 .70012 .20984 Metetra
.695 .751 .895 r
.70012 .20984 .70362 .21503 .71101 .21159 .70753 .20644 Metetra
.695 .754 .897 r
.70753 .20644 .71101 .21159 .71842 .20814 .71496 .20303 Metetra
.696 .757 .899 r
.71496 .20303 .71842 .20814 .72585 .20468 .72241 .19961 Metetra
.696 .76 .901 r
.72241 .19961 .72585 .20468 .73329 .20121 .72987 .19619 Metetra
.697 .763 .903 r
.72987 .19619 .73329 .20121 .74075 .19773 .73734 .19276 Metetra
.697 .766 .905 r
.73734 .19276 .74075 .19773 .74823 .19425 .74484 .18932 Metetra
.697 .76 .9 r
.07882 .42006 .08326 .42393 .08927 .42111 .08483 .41727 Metetra
.698 .763 .902 r
.08483 .41727 .08927 .42111 .0953 .41828 .09086 .41449 Metetra
.698 .766 .904 r
.09086 .41449 .0953 .41828 .10133 .41544 .0969 .4117 Metetra
.698 .769 .906 r
.0969 .4117 .10133 .41544 .10738 .4126 .10295 .4089 Metetra
.699 .772 .908 r
.10295 .4089 .10738 .4126 .11344 .40975 .10901 .4061 Metetra
.699 .774 .91 r
.10901 .4061 .11344 .40975 .11951 .4069 .11508 .40329 Metetra
.699 .777 .912 r
.11508 .40329 .11951 .4069 .1256 .40405 .12117 .40048 Metetra
.699 .78 .914 r
.12117 .40048 .1256 .40405 .13169 .40119 .12727 .39767 Metetra
.699 .783 .916 r
.12727 .39767 .13169 .40119 .1378 .39833 .13337 .39485 Metetra
.7 .786 .918 r
.13337 .39485 .1378 .39833 .14391 .39547 .1395 .39203 Metetra
.7 .789 .92 r
.1395 .39203 .14391 .39547 .15004 .3926 .14563 .38921 Metetra
.7 .792 .922 r
.14563 .38921 .15004 .3926 .15618 .38973 .15177 .38639 Metetra
.699 .795 .924 r
.15177 .38639 .15618 .38973 .16233 .38687 .15792 .38357 Metetra
.699 .798 .927 r
.15792 .38357 .16233 .38687 .16849 .384 .16409 .38075 Metetra
.699 .801 .929 r
.16409 .38075 .16849 .384 .17466 .38114 .17027 .37794 Metetra
.698 .804 .931 r
.17027 .37794 .17466 .38114 .18085 .37828 .17645 .37513 Metetra
.698 .808 .933 r
.17645 .37513 .18085 .37828 .18704 .37543 .18265 .37232 Metetra
.697 .811 .936 r
.18265 .37232 .18704 .37543 .19325 .37259 .18886 .36953 Metetra
.696 .814 .938 r
.18886 .36953 .19325 .37259 .19946 .36976 .19508 .36674 Metetra
.694 .817 .941 r
.19508 .36674 .19946 .36976 .20569 .36694 .20132 .36397 Metetra
.693 .82 .943 r
.20132 .36397 .20569 .36694 .21192 .36413 .20756 .36121 Metetra
.691 .823 .946 r
.20756 .36121 .21192 .36413 .21817 .36134 .21381 .35848 Metetra
.689 .826 .949 r
.21381 .35848 .21817 .36134 .22443 .35858 .22007 .35576 Metetra
.686 .829 .952 r
.22007 .35576 .22443 .35858 .23069 .35585 .22635 .35307 Metetra
.683 .832 .955 r
.22635 .35307 .23069 .35585 .23697 .35314 .23263 .35041 Metetra
.679 .834 .958 r
.23263 .35041 .23697 .35314 .24325 .35048 .23892 .34779 Metetra
.675 .836 .961 r
.23892 .34779 .24325 .35048 .24955 .34786 .24522 .34522 Metetra
.67 .839 .964 r
.24522 .34522 .24955 .34786 .25585 .34529 .25153 .34268 Metetra
.664 .84 .967 r
.25153 .34268 .25585 .34529 .26216 .34278 .25785 .34021 Metetra
.658 .841 .971 r
.25785 .34021 .26216 .34278 .26848 .34034 .26418 .3378 Metetra
.651 .842 .974 r
.26418 .3378 .26848 .34034 .27481 .33798 .27052 .33546 Metetra
.642 .842 .977 r
.27052 .33546 .27481 .33798 .28115 .3357 .27686 .3332 Metetra
.634 .841 .98 r
.27686 .3332 .28115 .3357 .28749 .33351 .28322 .33102 Metetra
.624 .84 .983 r
.28322 .33102 .28749 .33351 .29384 .33142 .28958 .32893 Metetra
.613 .837 .985 r
.28958 .32893 .29384 .33142 .3002 .32944 .29595 .32694 Metetra
.602 .834 .988 r
.29595 .32694 .3002 .32944 .30657 .32758 .30233 .32506 Metetra
.59 .829 .99 r
.30233 .32506 .30657 .32758 .31294 .32584 .30872 .32328 Metetra
.579 .823 .991 r
.30872 .32328 .31294 .32584 .31933 .32424 .31512 .32162 Metetra
.567 .816 .992 r
.31512 .32162 .31933 .32424 .32572 .32276 .32153 .32007 Metetra
.556 .808 .992 r
.32153 .32007 .32572 .32276 .33213 .32142 .32796 .31864 Metetra
.546 .8 .992 r
.32796 .31864 .33213 .32142 .33854 .32021 .33439 .31731 Metetra
.537 .79 .991 r
.33439 .31731 .33854 .32021 .34497 .31912 .34084 .3161 Metetra
.529 .779 .989 r
.34084 .3161 .34497 .31912 .35142 .31815 .3473 .31499 Metetra
.524 .769 .987 r
.3473 .31499 .35142 .31815 .35788 .31729 .35378 .31397 Metetra
.52 .758 .983 r
.35378 .31397 .35788 .31729 .36435 .31652 .36027 .31303 Metetra
.518 .747 .979 r
.36027 .31303 .36435 .31652 .37085 .31583 .36678 .31216 Metetra
.518 .737 .974 r
.36678 .31216 .37085 .31583 .37736 .31521 .37332 .31135 Metetra
.519 .726 .968 r
.37332 .31135 .37736 .31521 .3839 .31464 .37987 .31058 Metetra
.522 .716 .961 r
.37987 .31058 .3839 .31464 .39046 .3141 .38644 .30983 Metetra
.526 .707 .953 r
.38644 .30983 .39046 .3141 .39704 .31356 .39304 .30909 Metetra
.531 .698 .946 r
.39304 .30909 .39704 .31356 .40364 .31301 .39966 .30833 Metetra
.538 .69 .938 r
.39966 .30833 .40364 .31301 .41027 .31243 .40631 .30755 Metetra
.544 .683 .929 r
.40631 .30755 .41027 .31243 .41693 .31179 .41298 .30672 Metetra
.552 .677 .921 r
.41298 .30672 .41693 .31179 .42361 .31109 .41967 .30583 Metetra
.56 .672 .913 r
.41967 .30583 .42361 .31109 .43032 .3103 .4264 .30486 Metetra
.567 .667 .905 r
.4264 .30486 .43032 .3103 .43706 .3094 .43314 .3038 Metetra
.575 .664 .898 r
.43314 .3038 .43706 .3094 .44382 .30839 .43991 .30264 Metetra
.583 .661 .892 r
.43991 .30264 .44382 .30839 .45061 .30726 .44671 .30137 Metetra
.591 .66 .886 r
.44671 .30137 .45061 .30726 .45742 .30598 .45354 .29998 Metetra
.598 .659 .88 r
.45354 .29998 .45742 .30598 .46426 .30456 .46038 .29847 Metetra
.605 .658 .875 r
.46038 .29847 .46426 .30456 .47112 .303 .46725 .29683 Metetra
.612 .659 .871 r
.46725 .29683 .47112 .303 .47801 .30129 .47415 .29506 Metetra
.619 .66 .868 r
.47415 .29506 .47801 .30129 .48492 .29944 .48106 .29316 Metetra
.625 .662 .865 r
.48106 .29316 .48492 .29944 .49185 .29745 .488 .29114 Metetra
.631 .664 .863 r
.488 .29114 .49185 .29745 .4988 .29532 .49496 .289 Metetra
.637 .666 .861 r
.49496 .289 .4988 .29532 .50577 .29307 .50193 .28675 Metetra
.642 .668 .86 r
.50193 .28675 .50577 .29307 .51276 .29071 .50893 .28439 Metetra
.646 .671 .86 r
.50893 .28439 .51276 .29071 .51977 .28824 .51595 .28193 Metetra
.651 .674 .859 r
.51595 .28193 .51977 .28824 .52679 .28566 .52298 .27938 Metetra
.655 .677 .859 r
.52298 .27938 .52679 .28566 .53383 .283 .53003 .27674 Metetra
.659 .68 .859 r
.53003 .27674 .53383 .283 .54089 .28025 .5371 .27403 Metetra
.662 .683 .86 r
.5371 .27403 .54089 .28025 .54797 .27743 .54418 .27125 Metetra
.665 .687 .861 r
.54418 .27125 .54797 .27743 .55506 .27454 .55128 .2684 Metetra
.668 .69 .862 r
.55128 .2684 .55506 .27454 .56217 .27159 .5584 .26549 Metetra
.67 .693 .863 r
.5584 .26549 .56217 .27159 .56929 .26859 .56554 .26253 Metetra
.672 .696 .864 r
.56554 .26253 .56929 .26859 .57643 .26554 .57269 .25953 Metetra
.675 .7 .865 r
.57269 .25953 .57643 .26554 .58359 .26245 .57985 .25649 Metetra
.677 .703 .867 r
.57985 .25649 .58359 .26245 .59076 .25932 .58703 .25341 Metetra
.678 .706 .868 r
.58703 .25341 .59076 .25932 .59794 .25616 .59423 .2503 Metetra
.68 .709 .87 r
.59423 .2503 .59794 .25616 .60514 .25297 .60144 .24716 Metetra
.681 .712 .872 r
.60144 .24716 .60514 .25297 .61236 .24975 .60867 .24399 Metetra
.683 .715 .873 r
.60867 .24399 .61236 .24975 .61959 .24652 .61591 .2408 Metetra
.684 .718 .875 r
.61591 .2408 .61959 .24652 .62683 .24325 .62317 .23759 Metetra
.685 .721 .877 r
.62317 .23759 .62683 .24325 .63409 .23998 .63044 .23435 Metetra
.686 .724 .879 r
.63044 .23435 .63409 .23998 .64137 .23668 .63773 .2311 Metetra
.687 .727 .881 r
.63773 .2311 .64137 .23668 .64866 .23337 .64503 .22784 Metetra
.688 .73 .883 r
.64503 .22784 .64866 .23337 .65596 .23004 .65235 .22456 Metetra
.688 .733 .884 r
.65235 .22456 .65596 .23004 .66328 .2267 .65969 .22126 Metetra
.689 .736 .886 r
.65969 .22126 .66328 .2267 .67062 .22335 .66704 .21795 Metetra
.69 .739 .888 r
.66704 .21795 .67062 .22335 .67797 .21999 .67441 .21464 Metetra
.691 .742 .89 r
.67441 .21464 .67797 .21999 .68534 .21661 .68179 .21131 Metetra
.691 .744 .892 r
.68179 .21131 .68534 .21661 .69272 .21323 .68919 .20797 Metetra
.692 .747 .894 r
.68919 .20797 .69272 .21323 .70012 .20984 .6966 .20462 Metetra
.692 .75 .896 r
.6966 .20462 .70012 .20984 .70753 .20644 .70403 .20126 Metetra
.693 .753 .898 r
.70403 .20126 .70753 .20644 .71496 .20303 .71148 .1979 Metetra
.693 .756 .9 r
.71148 .1979 .71496 .20303 .72241 .19961 .71894 .19453 Metetra
.694 .759 .902 r
.71894 .19453 .72241 .19961 .72987 .19619 .72642 .19114 Metetra
.694 .761 .904 r
.72642 .19114 .72987 .19619 .73734 .19276 .73392 .18776 Metetra
.694 .764 .906 r
.73392 .18776 .73734 .19276 .74484 .18932 .74143 .18436 Metetra
.694 .76 .902 r
.07435 .41616 .07882 .42006 .08483 .41727 .08037 .41342 Metetra
.695 .762 .904 r
.08037 .41342 .08483 .41727 .09086 .41449 .0864 .41068 Metetra
.695 .765 .906 r
.0864 .41068 .09086 .41449 .0969 .4117 .09244 .40793 Metetra
.695 .768 .908 r
.09244 .40793 .0969 .4117 .10295 .4089 .09849 .40518 Metetra
.696 .771 .91 r
.09849 .40518 .10295 .4089 .10901 .4061 .10456 .40242 Metetra
.696 .774 .912 r
.10456 .40242 .10901 .4061 .11508 .40329 .11063 .39966 Metetra
.696 .777 .914 r
.11063 .39966 .11508 .40329 .12117 .40048 .11672 .39689 Metetra
.696 .78 .916 r
.11672 .39689 .12117 .40048 .12727 .39767 .12282 .39413 Metetra
.696 .783 .918 r
.12282 .39413 .12727 .39767 .13337 .39485 .12893 .39135 Metetra
.696 .785 .92 r
.12893 .39135 .13337 .39485 .1395 .39203 .13506 .38858 Metetra
.696 .789 .922 r
.13506 .38858 .1395 .39203 .14563 .38921 .14119 .3858 Metetra
.696 .791 .924 r
.14119 .3858 .14563 .38921 .15177 .38639 .14734 .38303 Metetra
.696 .795 .926 r
.14734 .38303 .15177 .38639 .15792 .38357 .1535 .38025 Metetra
.696 .797 .928 r
.1535 .38025 .15792 .38357 .16409 .38075 .15967 .37748 Metetra
.695 .8 .93 r
.15967 .37748 .16409 .38075 .17027 .37794 .16585 .37471 Metetra
.695 .803 .933 r
.16585 .37471 .17027 .37794 .17645 .37513 .17204 .37195 Metetra
.694 .806 .935 r
.17204 .37195 .17645 .37513 .18265 .37232 .17824 .36919 Metetra
.693 .809 .937 r
.17824 .36919 .18265 .37232 .18886 .36953 .18446 .36644 Metetra
.692 .812 .939 r
.18446 .36644 .18886 .36953 .19508 .36674 .19069 .3637 Metetra
.691 .815 .942 r
.19069 .3637 .19508 .36674 .20132 .36397 .19692 .36097 Metetra
.689 .818 .944 r
.19692 .36097 .20132 .36397 .20756 .36121 .20317 .35826 Metetra
.687 .821 .947 r
.20317 .35826 .20756 .36121 .21381 .35848 .20943 .35556 Metetra
.685 .824 .95 r
.20943 .35556 .21381 .35848 .22007 .35576 .2157 .35289 Metetra
.682 .826 .952 r
.2157 .35289 .22007 .35576 .22635 .35307 .22198 .35024 Metetra
.68 .829 .955 r
.22198 .35024 .22635 .35307 .23263 .35041 .22827 .34762 Metetra
.676 .831 .958 r
.22827 .34762 .23263 .35041 .23892 .34779 .23457 .34504 Metetra
.672 .833 .961 r
.23457 .34504 .23892 .34779 .24522 .34522 .24088 .3425 Metetra
.667 .834 .964 r
.24088 .3425 .24522 .34522 .25153 .34268 .2472 .34 Metetra
.662 .836 .967 r
.2472 .34 .25153 .34268 .25785 .34021 .25353 .33756 Metetra
.656 .837 .97 r
.25353 .33756 .25785 .34021 .26418 .3378 .25986 .33517 Metetra
.65 .837 .972 r
.25986 .33517 .26418 .3378 .27052 .33546 .26621 .33285 Metetra
.642 .837 .975 r
.26621 .33285 .27052 .33546 .27686 .3332 .27257 .3306 Metetra
.634 .836 .978 r
.27257 .3306 .27686 .3332 .28322 .33102 .27893 .32842 Metetra
.626 .835 .98 r
.27893 .32842 .28322 .33102 .28958 .32893 .28531 .32633 Metetra
.617 .832 .983 r
.28531 .32633 .28958 .32893 .29595 .32694 .29169 .32432 Metetra
.608 .829 .985 r
.29169 .32432 .29595 .32694 .30233 .32506 .29809 .3224 Metetra
.598 .825 .986 r
.29809 .3224 .30233 .32506 .30872 .32328 .30449 .32058 Metetra
.588 .819 .987 r
.30449 .32058 .30872 .32328 .31512 .32162 .3109 .31886 Metetra
.579 .813 .988 r
.3109 .31886 .31512 .32162 .32153 .32007 .31733 .31724 Metetra
.57 .806 .988 r
.31733 .31724 .32153 .32007 .32796 .31864 .32377 .31572 Metetra
.561 .798 .988 r
.32377 .31572 .32796 .31864 .33439 .31731 .33022 .31429 Metetra
.554 .789 .987 r
.33022 .31429 .33439 .31731 .34084 .3161 .33668 .31296 Metetra
.548 .78 .985 r
.33668 .31296 .34084 .3161 .3473 .31499 .34316 .31171 Metetra
.543 .771 .982 r
.34316 .31171 .3473 .31499 .35378 .31397 .34966 .31055 Metetra
.54 .761 .979 r
.34966 .31055 .35378 .31397 .36027 .31303 .35617 .30945 Metetra
.538 .751 .975 r
.35617 .30945 .36027 .31303 .36678 .31216 .3627 .30841 Metetra
.537 .741 .97 r
.3627 .30841 .36678 .31216 .37332 .31135 .36925 .30743 Metetra
.538 .732 .964 r
.36925 .30743 .37332 .31135 .37987 .31058 .37582 .30647 Metetra
.54 .722 .958 r
.37582 .30647 .37987 .31058 .38644 .30983 .38241 .30554 Metetra
.543 .714 .951 r
.38241 .30554 .38644 .30983 .39304 .30909 .38903 .3046 Metetra
.547 .705 .944 r
.38903 .3046 .39304 .30909 .39966 .30833 .39566 .30366 Metetra
.552 .698 .936 r
.39566 .30366 .39966 .30833 .40631 .30755 .40232 .3027 Metetra
.558 .691 .929 r
.40232 .3027 .40631 .30755 .41298 .30672 .40901 .30168 Metetra
.564 .685 .921 r
.40901 .30168 .41298 .30672 .41967 .30583 .41572 .30062 Metetra
.57 .68 .914 r
.41572 .30062 .41967 .30583 .4264 .30486 .42245 .29949 Metetra
.577 .676 .907 r
.42245 .29949 .4264 .30486 .43314 .3038 .42921 .29828 Metetra
.583 .672 .9 r
.42921 .29828 .43314 .3038 .43991 .30264 .43599 .29698 Metetra
.59 .669 .894 r
.43599 .29698 .43991 .30264 .44671 .30137 .4428 .29559 Metetra
.597 .667 .888 r
.4428 .29559 .44671 .30137 .45354 .29998 .44963 .29408 Metetra
.603 .666 .883 r
.44963 .29408 .45354 .29998 .46038 .29847 .45649 .29248 Metetra
.609 .665 .879 r
.45649 .29248 .46038 .29847 .46725 .29683 .46336 .29076 Metetra
.616 .665 .875 r
.46336 .29076 .46725 .29683 .47415 .29506 .47027 .28892 Metetra
.621 .666 .872 r
.47027 .28892 .47415 .29506 .48106 .29316 .47719 .28698 Metetra
.627 .667 .869 r
.47719 .28698 .48106 .29316 .488 .29114 .48413 .28492 Metetra
.632 .668 .867 r
.48413 .28492 .488 .29114 .49496 .289 .4911 .28276 Metetra
.637 .67 .865 r
.4911 .28276 .49496 .289 .50193 .28675 .49808 .2805 Metetra
.642 .672 .864 r
.49808 .2805 .50193 .28675 .50893 .28439 .50509 .27814 Metetra
.646 .674 .863 r
.50509 .27814 .50893 .28439 .51595 .28193 .51211 .27569 Metetra
.65 .677 .862 r
.51211 .27569 .51595 .28193 .52298 .27938 .51915 .27315 Metetra
.654 .68 .862 r
.51915 .27315 .52298 .27938 .53003 .27674 .52621 .27054 Metetra
.657 .682 .862 r
.52621 .27054 .53003 .27674 .5371 .27403 .53329 .26785 Metetra
.66 .685 .863 r
.53329 .26785 .5371 .27403 .54418 .27125 .54038 .2651 Metetra
.663 .688 .863 r
.54038 .2651 .54418 .27125 .55128 .2684 .54749 .26229 Metetra
.666 .691 .864 r
.54749 .26229 .55128 .2684 .5584 .26549 .55462 .25942 Metetra
.668 .694 .865 r
.55462 .25942 .5584 .26549 .56554 .26253 .56176 .2565 Metetra
.67 .697 .866 r
.56176 .2565 .56554 .26253 .57269 .25953 .56892 .25354 Metetra
.672 .7 .867 r
.56892 .25354 .57269 .25953 .57985 .25649 .5761 .25054 Metetra
.674 .703 .869 r
.5761 .25054 .57985 .25649 .58703 .25341 .58329 .2475 Metetra
.676 .706 .87 r
.58329 .2475 .58703 .25341 .59423 .2503 .5905 .24444 Metetra
.677 .709 .872 r
.5905 .24444 .59423 .2503 .60144 .24716 .59772 .24134 Metetra
.679 .712 .873 r
.59772 .24134 .60144 .24716 .60867 .24399 .60496 .23822 Metetra
.68 .715 .875 r
.60496 .23822 .60867 .24399 .61591 .2408 .61222 .23507 Metetra
.681 .718 .877 r
.61222 .23507 .61591 .2408 .62317 .23759 .61949 .2319 Metetra
.682 .72 .878 r
.61949 .2319 .62317 .23759 .63044 .23435 .62678 .22872 Metetra
.683 .723 .88 r
.62678 .22872 .63044 .23435 .63773 .2311 .63408 .22551 Metetra
.684 .726 .882 r
.63408 .22551 .63773 .2311 .64503 .22784 .6414 .22229 Metetra
.685 .729 .884 r
.6414 .22229 .64503 .22784 .65235 .22456 .64873 .21905 Metetra
.686 .732 .886 r
.64873 .21905 .65235 .22456 .65969 .22126 .65608 .2158 Metetra
.686 .735 .887 r
.65608 .2158 .65969 .22126 .66704 .21795 .66345 .21254 Metetra
.687 .738 .889 r
.66345 .21254 .66704 .21795 .67441 .21464 .67083 .20927 Metetra
.688 .74 .891 r
.67083 .20927 .67441 .21464 .68179 .21131 .67823 .20598 Metetra
.688 .743 .893 r
.67823 .20598 .68179 .21131 .68919 .20797 .68564 .20269 Metetra
.689 .746 .895 r
.68564 .20269 .68919 .20797 .6966 .20462 .69307 .19938 Metetra
.689 .749 .897 r
.69307 .19938 .6966 .20462 .70403 .20126 .70052 .19607 Metetra
.69 .752 .899 r
.70052 .19607 .70403 .20126 .71148 .1979 .70798 .19275 Metetra
.69 .755 .901 r
.70798 .19275 .71148 .1979 .71894 .19453 .71546 .18942 Metetra
.691 .758 .903 r
.71546 .18942 .71894 .19453 .72642 .19114 .72296 .18608 Metetra
.691 .76 .905 r
.72296 .18608 .72642 .19114 .73392 .18776 .73047 .18273 Metetra
.691 .763 .907 r
.73047 .18273 .73392 .18776 .74143 .18436 .738 .17938 Metetra
.691 .759 .904 r
.06987 .41225 .07435 .41616 .08037 .41342 .07589 .40956 Metetra
.692 .762 .905 r
.07589 .40956 .08037 .41342 .0864 .41068 .08192 .40686 Metetra
.692 .765 .907 r
.08192 .40686 .0864 .41068 .09244 .40793 .08796 .40415 Metetra
.692 .767 .909 r
.08796 .40415 .09244 .40793 .09849 .40518 .09402 .40144 Metetra
.693 .77 .911 r
.09402 .40144 .09849 .40518 .10456 .40242 .10008 .39873 Metetra
.693 .773 .913 r
.10008 .39873 .10456 .40242 .11063 .39966 .10616 .39601 Metetra
.693 .776 .915 r
.10616 .39601 .11063 .39966 .11672 .39689 .11225 .39329 Metetra
.693 .779 .917 r
.11225 .39329 .11672 .39689 .12282 .39413 .11836 .39056 Metetra
.693 .782 .919 r
.11836 .39056 .12282 .39413 .12893 .39135 .12447 .38784 Metetra
.693 .785 .921 r
.12447 .38784 .12893 .39135 .13506 .38858 .1306 .38511 Metetra
.693 .788 .923 r
.1306 .38511 .13506 .38858 .14119 .3858 .13674 .38238 Metetra
.693 .791 .925 r
.13674 .38238 .14119 .3858 .14734 .38303 .14289 .37964 Metetra
.693 .793 .927 r
.14289 .37964 .14734 .38303 .1535 .38025 .14905 .37691 Metetra
.692 .796 .929 r
.14905 .37691 .1535 .38025 .15967 .37748 .15522 .37418 Metetra
.692 .799 .932 r
.15522 .37418 .15967 .37748 .16585 .37471 .16141 .37146 Metetra
.691 .802 .934 r
.16141 .37146 .16585 .37471 .17204 .37195 .16761 .36874 Metetra
.69 .805 .936 r
.16761 .36874 .17204 .37195 .17824 .36919 .17382 .36602 Metetra
.689 .808 .938 r
.17382 .36602 .17824 .36919 .18446 .36644 .18004 .36331 Metetra
.689 .811 .941 r
.18004 .36331 .18446 .36644 .19069 .3637 .18627 .36061 Metetra
.687 .813 .943 r
.18627 .36061 .19069 .3637 .19692 .36097 .19251 .35793 Metetra
.685 .816 .945 r
.19251 .35793 .19692 .36097 .20317 .35826 .19876 .35526 Metetra
.684 .819 .948 r
.19876 .35526 .20317 .35826 .20943 .35556 .20503 .3526 Metetra
.681 .821 .95 r
.20503 .3526 .20943 .35556 .2157 .35289 .21131 .34997 Metetra
.679 .824 .953 r
.21131 .34997 .2157 .35289 .22198 .35024 .21759 .34736 Metetra
.676 .826 .955 r
.21759 .34736 .22198 .35024 .22827 .34762 .22389 .34478 Metetra
.673 .828 .958 r
.22389 .34478 .22827 .34762 .23457 .34504 .2302 .34223 Metetra
.669 .83 .961 r
.2302 .34223 .23457 .34504 .24088 .3425 .23651 .33972 Metetra
.665 .831 .963 r
.23651 .33972 .24088 .3425 .2472 .34 .24284 .33725 Metetra
.66 .832 .966 r
.24284 .33725 .2472 .34 .25353 .33756 .24918 .33483 Metetra
.655 .833 .968 r
.24918 .33483 .25353 .33756 .25986 .33517 .25553 .33246 Metetra
.649 .833 .971 r
.25553 .33246 .25986 .33517 .26621 .33285 .26189 .33015 Metetra
.642 .833 .974 r
.26189 .33015 .26621 .33285 .27257 .3306 .26825 .3279 Metetra
.636 .832 .976 r
.26825 .3279 .27257 .3306 .27893 .32842 .27463 .32572 Metetra
.628 .83 .978 r
.27463 .32572 .27893 .32842 .28531 .32633 .28102 .32361 Metetra
.621 .828 .98 r
.28102 .32361 .28531 .32633 .29169 .32432 .28741 .32158 Metetra
.613 .825 .981 r
.28741 .32158 .29169 .32432 .29809 .3224 .29382 .31963 Metetra
.605 .821 .983 r
.29382 .31963 .29809 .3224 .30449 .32058 .30024 .31776 Metetra
.597 .816 .984 r
.30024 .31776 .30449 .32058 .3109 .31886 .30667 .31598 Metetra
.589 .81 .984 r
.30667 .31598 .3109 .31886 .31733 .31724 .31311 .31429 Metetra
.581 .804 .984 r
.31311 .31429 .31733 .31724 .32377 .31572 .31957 .31268 Metetra
.575 .797 .983 r
.31957 .31268 .32377 .31572 .33022 .31429 .32603 .31115 Metetra
.568 .789 .982 r
.32603 .31115 .33022 .31429 .33668 .31296 .33252 .30971 Metetra
.563 .78 .98 r
.33252 .30971 .33668 .31296 .34316 .31171 .33901 .30834 Metetra
.559 .772 .978 r
.33901 .30834 .34316 .31171 .34966 .31055 .34553 .30704 Metetra
.556 .763 .974 r
.34553 .30704 .34966 .31055 .35617 .30945 .35206 .30579 Metetra
.555 .754 .97 r
.35206 .30579 .35617 .30945 .3627 .30841 .3586 .3046 Metetra
.554 .745 .966 r
.3586 .3046 .3627 .30841 .36925 .30743 .36517 .30344 Metetra
.554 .736 .96 r
.36517 .30344 .36925 .30743 .37582 .30647 .37176 .30232 Metetra
.556 .728 .955 r
.37176 .30232 .37582 .30647 .38241 .30554 .37836 .30121 Metetra
.558 .719 .948 r
.37836 .30121 .38241 .30554 .38903 .3046 .38499 .30011 Metetra
.561 .712 .942 r
.38499 .30011 .38903 .3046 .39566 .30366 .39165 .29899 Metetra
.566 .705 .935 r
.39165 .29899 .39566 .30366 .40232 .3027 .39832 .29785 Metetra
.57 .698 .928 r
.39832 .29785 .40232 .3027 .40901 .30168 .40502 .29668 Metetra
.575 .692 .921 r
.40502 .29668 .40901 .30168 .41572 .30062 .41174 .29545 Metetra
.58 .687 .914 r
.41174 .29545 .41572 .30062 .42245 .29949 .41849 .29416 Metetra
.585 .683 .908 r
.41849 .29416 .42245 .29949 .42921 .29828 .42526 .29282 Metetra
.591 .679 .902 r
.42526 .29282 .42921 .29828 .43599 .29698 .43205 .29139 Metetra
.597 .676 .896 r
.43205 .29139 .43599 .29698 .4428 .29559 .43887 .28988 Metetra
.603 .674 .891 r
.43887 .28988 .4428 .29559 .44963 .29408 .44571 .28827 Metetra
.608 .673 .886 r
.44571 .28827 .44963 .29408 .45649 .29248 .45257 .28657 Metetra
.614 .672 .882 r
.45257 .28657 .45649 .29248 .46336 .29076 .45946 .28477 Metetra
.619 .671 .878 r
.45946 .28477 .46336 .29076 .47027 .28892 .46637 .28287 Metetra
.624 .671 .875 r
.46637 .28287 .47027 .28892 .47719 .28698 .4733 .28088 Metetra
.629 .672 .872 r
.4733 .28088 .47719 .28698 .48413 .28492 .48025 .27878 Metetra
.633 .673 .87 r
.48025 .27878 .48413 .28492 .4911 .28276 .48723 .27659 Metetra
.638 .674 .868 r
.48723 .27659 .4911 .28276 .49808 .2805 .49422 .27431 Metetra
.642 .676 .867 r
.49422 .27431 .49808 .2805 .50509 .27814 .50123 .27195 Metetra
.646 .678 .866 r
.50123 .27195 .50509 .27814 .51211 .27569 .50826 .2695 Metetra
.649 .68 .865 r
.50826 .2695 .51211 .27569 .51915 .27315 .51531 .26698 Metetra
.653 .682 .865 r
.51531 .26698 .51915 .27315 .52621 .27054 .52238 .26438 Metetra
.656 .685 .865 r
.52238 .26438 .52621 .27054 .53329 .26785 .52946 .26171 Metetra
.659 .687 .865 r
.52946 .26171 .53329 .26785 .54038 .2651 .53657 .25898 Metetra
.662 .69 .866 r
.53657 .25898 .54038 .2651 .54749 .26229 .54369 .2562 Metetra
.664 .692 .866 r
.54369 .2562 .54749 .26229 .55462 .25942 .55083 .25336 Metetra
.666 .695 .867 r
.55083 .25336 .55462 .25942 .56176 .2565 .55798 .25048 Metetra
.668 .697 .868 r
.55798 .25048 .56176 .2565 .56892 .25354 .56515 .24756 Metetra
.67 .701 .869 r
.56515 .24756 .56892 .25354 .5761 .25054 .57234 .2446 Metetra
.672 .703 .871 r
.57234 .2446 .5761 .25054 .58329 .2475 .57954 .2416 Metetra
.673 .706 .872 r
.57954 .2416 .58329 .2475 .5905 .24444 .58676 .23857 Metetra
.675 .709 .873 r
.58676 .23857 .5905 .24444 .59772 .24134 .59399 .23552 Metetra
.676 .712 .875 r
.59399 .23552 .59772 .24134 .60496 .23822 .60124 .23244 Metetra
.678 .715 .877 r
.60124 .23244 .60496 .23822 .61222 .23507 .60851 .22933 Metetra
.679 .717 .878 r
.60851 .22933 .61222 .23507 .61949 .2319 .6158 .22621 Metetra
.68 .72 .88 r
.6158 .22621 .61949 .2319 .62678 .22872 .6231 .22307 Metetra
.681 .723 .882 r
.6231 .22307 .62678 .22872 .63408 .22551 .63041 .2199 Metetra
.681 .726 .883 r
.63041 .2199 .63408 .22551 .6414 .22229 .63774 .21673 Metetra
.682 .728 .885 r
.63774 .21673 .6414 .22229 .64873 .21905 .64509 .21353 Metetra
.683 .731 .887 r
.64509 .21353 .64873 .21905 .65608 .2158 .65246 .21032 Metetra
.684 .734 .889 r
.65246 .21032 .65608 .2158 .66345 .21254 .65984 .20711 Metetra
.684 .737 .89 r
.65984 .20711 .66345 .21254 .67083 .20927 .66724 .20387 Metetra
.685 .74 .892 r
.66724 .20387 .67083 .20927 .67823 .20598 .67465 .20063 Metetra
.686 .742 .894 r
.67465 .20063 .67823 .20598 .68564 .20269 .68208 .19738 Metetra
.686 .745 .896 r
.68208 .19738 .68564 .20269 .69307 .19938 .68953 .19412 Metetra
.687 .748 .898 r
.68953 .19412 .69307 .19938 .70052 .19607 .69699 .19085 Metetra
.687 .751 .9 r
.69699 .19085 .70052 .19607 .70798 .19275 .70447 .18757 Metetra
.688 .754 .902 r
.70447 .18757 .70798 .19275 .71546 .18942 .71197 .18428 Metetra
.688 .756 .904 r
.71197 .18428 .71546 .18942 .72296 .18608 .71948 .18098 Metetra
.688 .759 .906 r
.71948 .18098 .72296 .18608 .73047 .18273 .72702 .17768 Metetra
.688 .762 .908 r
.72702 .17768 .73047 .18273 .738 .17938 .73456 .17437 Metetra
.688 .759 .905 r
.06537 .40832 .06987 .41225 .07589 .40956 .07139 .40567 Metetra
.689 .761 .907 r
.07139 .40567 .07589 .40956 .08192 .40686 .07742 .40302 Metetra
.689 .764 .909 r
.07742 .40302 .08192 .40686 .08796 .40415 .08347 .40036 Metetra
.689 .767 .911 r
.08347 .40036 .08796 .40415 .09402 .40144 .08952 .39769 Metetra
.689 .77 .913 r
.08952 .39769 .09402 .40144 .10008 .39873 .09559 .39502 Metetra
.69 .773 .915 r
.09559 .39502 .10008 .39873 .10616 .39601 .10167 .39234 Metetra
.69 .775 .917 r
.10167 .39234 .10616 .39601 .11225 .39329 .10777 .38966 Metetra
.69 .778 .919 r
.10777 .38966 .11225 .39329 .11836 .39056 .11387 .38698 Metetra
.69 .781 .921 r
.11387 .38698 .11836 .39056 .12447 .38784 .11999 .3843 Metetra
.69 .784 .923 r
.11999 .3843 .12447 .38784 .1306 .38511 .12612 .38161 Metetra
.69 .787 .925 r
.12612 .38161 .1306 .38511 .13674 .38238 .13226 .37892 Metetra
.689 .79 .927 r
.13226 .37892 .13674 .38238 .14289 .37964 .13841 .37624 Metetra
.689 .793 .929 r
.13841 .37624 .14289 .37964 .14905 .37691 .14458 .37355 Metetra
.689 .795 .931 r
.14458 .37355 .14905 .37691 .15522 .37418 .15076 .37086 Metetra
.688 .798 .933 r
.15076 .37086 .15522 .37418 .16141 .37146 .15695 .36818 Metetra
.688 .801 .935 r
.15695 .36818 .16141 .37146 .16761 .36874 .16315 .3655 Metetra
.687 .804 .937 r
.16315 .3655 .16761 .36874 .17382 .36602 .16937 .36282 Metetra
.686 .806 .939 r
.16937 .36282 .17382 .36602 .18004 .36331 .17559 .36016 Metetra
.685 .809 .942 r
.17559 .36016 .18004 .36331 .18627 .36061 .18183 .3575 Metetra
.684 .812 .944 r
.18183 .3575 .18627 .36061 .19251 .35793 .18808 .35485 Metetra
.682 .814 .946 r
.18808 .35485 .19251 .35793 .19876 .35526 .19434 .35222 Metetra
.68 .817 .948 r
.19434 .35222 .19876 .35526 .20503 .3526 .20061 .34961 Metetra
.678 .819 .951 r
.20061 .34961 .20503 .3526 .21131 .34997 .20689 .34701 Metetra
.676 .821 .953 r
.20689 .34701 .21131 .34997 .21759 .34736 .21319 .34444 Metetra
.674 .824 .956 r
.21319 .34444 .21759 .34736 .22389 .34478 .21949 .34188 Metetra
.67 .825 .958 r
.21949 .34188 .22389 .34478 .2302 .34223 .22581 .33936 Metetra
.667 .827 .96 r
.22581 .33936 .2302 .34223 .23651 .33972 .23213 .33688 Metetra
.663 .828 .963 r
.23213 .33688 .23651 .33972 .24284 .33725 .23847 .33443 Metetra
.659 .829 .965 r
.23847 .33443 .24284 .33725 .24918 .33483 .24482 .33202 Metetra
.654 .829 .967 r
.24482 .33202 .24918 .33483 .25553 .33246 .25117 .32967 Metetra
.649 .829 .97 r
.25117 .32967 .25553 .33246 .26189 .33015 .25754 .32736 Metetra
.643 .829 .972 r
.25754 .32736 .26189 .33015 .26825 .3279 .26392 .32511 Metetra
.637 .828 .974 r
.26392 .32511 .26825 .3279 .27463 .32572 .27031 .32293 Metetra
.631 .826 .976 r
.27031 .32293 .27463 .32572 .28102 .32361 .27671 .3208 Metetra
.624 .824 .977 r
.27671 .3208 .28102 .32361 .28741 .32158 .28312 .31874 Metetra
.618 .821 .978 r
.28312 .31874 .28741 .32158 .29382 .31963 .28954 .31675 Metetra
.611 .817 .979 r
.28954 .31675 .29382 .31963 .30024 .31776 .29597 .31484 Metetra
.604 .813 .98 r
.29597 .31484 .30024 .31776 .30667 .31598 .30242 .313 Metetra
.598 .808 .98 r
.30242 .313 .30667 .31598 .31311 .31429 .30888 .31123 Metetra
.592 .802 .98 r
.30888 .31123 .31311 .31429 .31957 .31268 .31535 .30954 Metetra
.586 .795 .979 r
.31535 .30954 .31957 .31268 .32603 .31115 .32183 .30792 Metetra
.581 .788 .978 r
.32183 .30792 .32603 .31115 .33252 .30971 .32833 .30636 Metetra
.576 .781 .976 r
.32833 .30636 .33252 .30971 .33901 .30834 .33484 .30488 Metetra
.573 .773 .973 r
.33484 .30488 .33901 .30834 .34553 .30704 .34137 .30345 Metetra
.571 .765 .97 r
.34137 .30345 .34553 .30704 .35206 .30579 .34792 .30207 Metetra
.569 .757 .966 r
.34792 .30207 .35206 .30579 .3586 .3046 .35449 .30072 Metetra
.568 .748 .962 r
.35449 .30072 .3586 .3046 .36517 .30344 .36107 .29942 Metetra
.569 .74 .957 r
.36107 .29942 .36517 .30344 .37176 .30232 .36767 .29813 Metetra
.57 .732 .951 r
.36767 .29813 .37176 .30232 .37836 .30121 .3743 .29686 Metetra
.571 .724 .946 r
.3743 .29686 .37836 .30121 .38499 .30011 .38094 .2956 Metetra
.574 .717 .94 r
.38094 .2956 .38499 .30011 .39165 .29899 .38761 .29432 Metetra
.577 .71 .933 r
.38761 .29432 .39165 .29899 .39832 .29785 .3943 .29302 Metetra
.581 .705 .927 r
.3943 .29302 .39832 .29785 .40502 .29668 .40101 .29169 Metetra
.585 .699 .921 r
.40101 .29169 .40502 .29668 .41174 .29545 .40775 .29031 Metetra
.589 .694 .915 r
.40775 .29031 .41174 .29545 .41849 .29416 .4145 .28889 Metetra
.594 .689 .909 r
.4145 .28889 .41849 .29416 .42526 .29282 .42129 .2874 Metetra
.598 .685 .903 r
.42129 .2874 .42526 .29282 .43205 .29139 .42809 .28585 Metetra
.603 .683 .898 r
.42809 .28585 .43205 .29139 .43887 .28988 .43492 .28423 Metetra
.608 .68 .893 r
.43492 .28423 .43887 .28988 .44571 .28827 .44177 .28252 Metetra
.613 .678 .888 r
.44177 .28252 .44571 .28827 .45257 .28657 .44864 .28073 Metetra
.617 .677 .884 r
.44864 .28073 .45257 .28657 .45946 .28477 .45554 .27886 Metetra
.622 .676 .881 r
.45554 .27886 .45946 .28477 .46637 .28287 .46246 .27689 Metetra
.626 .676 .878 r
.46246 .27689 .46637 .28287 .4733 .28088 .4694 .27484 Metetra
.631 .676 .875 r
.4694 .27484 .4733 .28088 .48025 .27878 .47636 .27271 Metetra
.635 .677 .873 r
.47636 .27271 .48025 .27878 .48723 .27659 .48334 .27049 Metetra
.639 .678 .871 r
.48334 .27049 .48723 .27659 .49422 .27431 .49034 .26819 Metetra
.642 .679 .87 r
.49034 .26819 .49422 .27431 .50123 .27195 .49736 .26581 Metetra
.646 .68 .869 r
.49736 .26581 .50123 .27195 .50826 .2695 .5044 .26336 Metetra
.649 .682 .868 r
.5044 .26336 .50826 .2695 .51531 .26698 .51146 .26084 Metetra
.652 .684 .868 r
.51146 .26084 .51531 .26698 .52238 .26438 .51853 .25825 Metetra
.655 .686 .867 r
.51853 .25825 .52238 .26438 .52946 .26171 .52563 .2556 Metetra
.658 .689 .868 r
.52563 .2556 .52946 .26171 .53657 .25898 .53274 .25289 Metetra
.66 .691 .868 r
.53274 .25289 .53657 .25898 .54369 .2562 .53987 .25013 Metetra
.663 .693 .869 r
.53987 .25013 .54369 .2562 .55083 .25336 .54701 .24732 Metetra
.664 .696 .869 r
.54701 .24732 .55083 .25336 .55798 .25048 .55418 .24447 Metetra
.666 .698 .87 r
.55418 .24447 .55798 .25048 .56515 .24756 .56136 .24158 Metetra
.668 .701 .871 r
.56136 .24158 .56515 .24756 .57234 .2446 .56856 .23866 Metetra
.67 .704 .872 r
.56856 .23866 .57234 .2446 .57954 .2416 .57577 .2357 Metetra
.671 .706 .874 r
.57577 .2357 .57954 .2416 .58676 .23857 .583 .23271 Metetra
.673 .709 .875 r
.583 .23271 .58676 .23857 .59399 .23552 .59025 .22969 Metetra
.674 .712 .876 r
.59025 .22969 .59399 .23552 .60124 .23244 .59751 .22665 Metetra
.675 .714 .878 r
.59751 .22665 .60124 .23244 .60851 .22933 .60479 .22359 Metetra
.676 .717 .88 r
.60479 .22359 .60851 .22933 .6158 .22621 .61209 .22051 Metetra
.677 .72 .881 r
.61209 .22051 .6158 .22621 .6231 .22307 .6194 .2174 Metetra
.678 .722 .883 r
.6194 .2174 .6231 .22307 .63041 .2199 .62673 .21428 Metetra
.679 .725 .885 r
.62673 .21428 .63041 .2199 .63774 .21673 .63407 .21114 Metetra
.68 .728 .886 r
.63407 .21114 .63774 .21673 .64509 .21353 .64144 .20799 Metetra
.68 .731 .888 r
.64144 .20799 .64509 .21353 .65246 .21032 .64882 .20483 Metetra
.681 .733 .89 r
.64882 .20483 .65246 .21032 .65984 .20711 .65621 .20165 Metetra
.682 .736 .892 r
.65621 .20165 .65984 .20711 .66724 .20387 .66362 .19846 Metetra
.682 .739 .894 r
.66362 .19846 .66724 .20387 .67465 .20063 .67105 .19526 Metetra
.683 .741 .895 r
.67105 .19526 .67465 .20063 .68208 .19738 .6785 .19205 Metetra
.683 .744 .897 r
.6785 .19205 .68208 .19738 .68953 .19412 .68596 .18883 Metetra
.684 .747 .899 r
.68596 .18883 .68953 .19412 .69699 .19085 .69344 .1856 Metetra
.684 .75 .901 r
.69344 .1856 .69699 .19085 .70447 .18757 .70094 .18237 Metetra
.685 .752 .903 r
.70094 .18237 .70447 .18757 .71197 .18428 .70846 .17912 Metetra
.685 .755 .905 r
.70846 .17912 .71197 .18428 .71948 .18098 .71599 .17587 Metetra
.685 .758 .907 r
.71599 .17587 .71948 .18098 .72702 .17768 .72354 .1726 Metetra
.686 .761 .909 r
.72354 .1726 .72702 .17768 .73456 .17437 .73111 .16933 Metetra
.685 .758 .907 r
.06085 .40438 .06537 .40832 .07139 .40567 .06687 .40177 Metetra
.685 .761 .909 r
.06687 .40177 .07139 .40567 .07742 .40302 .0729 .39916 Metetra
.686 .763 .91 r
.0729 .39916 .07742 .40302 .08347 .40036 .07895 .39654 Metetra
.686 .766 .912 r
.07895 .39654 .08347 .40036 .08952 .39769 .08501 .39391 Metetra
.686 .769 .914 r
.08501 .39391 .08952 .39769 .09559 .39502 .09108 .39129 Metetra
.686 .772 .916 r
.09108 .39129 .09559 .39502 .10167 .39234 .09716 .38866 Metetra
.686 .775 .918 r
.09716 .38866 .10167 .39234 .10777 .38966 .10326 .38602 Metetra
.687 .777 .92 r
.10326 .38602 .10777 .38966 .11387 .38698 .10937 .38338 Metetra
.686 .78 .922 r
.10937 .38338 .11387 .38698 .11999 .3843 .11549 .38074 Metetra
.687 .783 .924 r
.11549 .38074 .11999 .3843 .12612 .38161 .12162 .3781 Metetra
.686 .786 .926 r
.12162 .3781 .12612 .38161 .13226 .37892 .12776 .37545 Metetra
.686 .789 .928 r
.12776 .37545 .13226 .37892 .13841 .37624 .13392 .3728 Metetra
.686 .791 .93 r
.13392 .3728 .13841 .37624 .14458 .37355 .14009 .37016 Metetra
.686 .794 .932 r
.14009 .37016 .14458 .37355 .15076 .37086 .14627 .36751 Metetra
.685 .797 .934 r
.14627 .36751 .15076 .37086 .15695 .36818 .15247 .36487 Metetra
.684 .8 .936 r
.15247 .36487 .15695 .36818 .16315 .3655 .15868 .36223 Metetra
.684 .802 .938 r
.15868 .36223 .16315 .3655 .16937 .36282 .16489 .3596 Metetra
.683 .805 .94 r
.16489 .3596 .16937 .36282 .17559 .36016 .17113 .35697 Metetra
.682 .808 .943 r
.17113 .35697 .17559 .36016 .18183 .3575 .17737 .35435 Metetra
.68 .81 .945 r
.17737 .35435 .18183 .3575 .18808 .35485 .18362 .35174 Metetra
.679 .813 .947 r
.18362 .35174 .18808 .35485 .19434 .35222 .18989 .34915 Metetra
.677 .815 .949 r
.18989 .34915 .19434 .35222 .20061 .34961 .19617 .34657 Metetra
.675 .817 .951 r
.19617 .34657 .20061 .34961 .20689 .34701 .20246 .344 Metetra
.673 .819 .953 r
.20246 .344 .20689 .34701 .21319 .34444 .20876 .34146 Metetra
.671 .821 .956 r
.20876 .34146 .21319 .34444 .21949 .34188 .21507 .33894 Metetra
.668 .823 .958 r
.21507 .33894 .21949 .34188 .22581 .33936 .2214 .33644 Metetra
.665 .824 .96 r
.2214 .33644 .22581 .33936 .23213 .33688 .22773 .33398 Metetra
.661 .825 .962 r
.22773 .33398 .23213 .33688 .23847 .33443 .23408 .33155 Metetra
.657 .826 .964 r
.23408 .33155 .23847 .33443 .24482 .33202 .24043 .32916 Metetra
.653 .826 .966 r
.24043 .32916 .24482 .33202 .25117 .32967 .2468 .32681 Metetra
.649 .826 .968 r
.2468 .32681 .25117 .32967 .25754 .32736 .25318 .3245 Metetra
.644 .825 .97 r
.25318 .3245 .25754 .32736 .26392 .32511 .25957 .32225 Metetra
.639 .824 .972 r
.25957 .32225 .26392 .32511 .27031 .32293 .26597 .32005 Metetra
.633 .823 .973 r
.26597 .32005 .27031 .32293 .27671 .3208 .27238 .3179 Metetra
.628 .821 .974 r
.27238 .3179 .27671 .3208 .28312 .31874 .27881 .31581 Metetra
.622 .818 .975 r
.27881 .31581 .28312 .31874 .28954 .31675 .28524 .31378 Metetra
.616 .814 .976 r
.28524 .31378 .28954 .31675 .29597 .31484 .29169 .31182 Metetra
.611 .81 .977 r
.29169 .31182 .29597 .31484 .30242 .313 .29815 .30992 Metetra
.605 .805 .977 r
.29815 .30992 .30242 .313 .30888 .31123 .30462 .30808 Metetra
.601 .8 .976 r
.30462 .30808 .30888 .31123 .31535 .30954 .31111 .30631 Metetra
.596 .794 .975 r
.31111 .30631 .31535 .30954 .32183 .30792 .31761 .30459 Metetra
.592 .788 .974 r
.31761 .30459 .32183 .30792 .32833 .30636 .32413 .30293 Metetra
.588 .781 .972 r
.32413 .30293 .32833 .30636 .33484 .30488 .33066 .30133 Metetra
.585 .774 .969 r
.33066 .30133 .33484 .30488 .34137 .30345 .3372 .29978 Metetra
.583 .766 .966 r
.3372 .29978 .34137 .30345 .34792 .30207 .34377 .29827 Metetra
.582 .759 .962 r
.34377 .29827 .34792 .30207 .35449 .30072 .35035 .29679 Metetra
.581 .751 .958 r
.35035 .29679 .35449 .30072 .36107 .29942 .35695 .29534 Metetra
.581 .743 .954 r
.35695 .29534 .36107 .29942 .36767 .29813 .36357 .29391 Metetra
.582 .736 .949 r
.36357 .29391 .36767 .29813 .3743 .29686 .37021 .29249 Metetra
.583 .729 .943 r
.37021 .29249 .3743 .29686 .38094 .2956 .37687 .29107 Metetra
.585 .722 .938 r
.37687 .29107 .38094 .2956 .38761 .29432 .38355 .28964 Metetra
.588 .716 .932 r
.38355 .28964 .38761 .29432 .3943 .29302 .39026 .28819 Metetra
.591 .71 .926 r
.39026 .28819 .3943 .29302 .40101 .29169 .39698 .28672 Metetra
.594 .704 .92 r
.39698 .28672 .40101 .29169 .40775 .29031 .40373 .2852 Metetra
.598 .699 .914 r
.40373 .2852 .40775 .29031 .4145 .28889 .4105 .28364 Metetra
.601 .695 .909 r
.4105 .28364 .4145 .28889 .42129 .2874 .4173 .28204 Metetra
.605 .692 .904 r
.4173 .28204 .42129 .2874 .42809 .28585 .42411 .28037 Metetra
.609 .688 .899 r
.42411 .28037 .42809 .28585 .43492 .28423 .43095 .27863 Metetra
.613 .686 .894 r
.43095 .27863 .43492 .28423 .44177 .28252 .43781 .27683 Metetra
.617 .684 .89 r
.43781 .27683 .44177 .28252 .44864 .28073 .4447 .27495 Metetra
.621 .682 .886 r
.4447 .27495 .44864 .28073 .45554 .27886 .4516 .273 Metetra
.625 .681 .883 r
.4516 .273 .45554 .27886 .46246 .27689 .45853 .27097 Metetra
.629 .681 .88 r
.45853 .27097 .46246 .27689 .4694 .27484 .46548 .26887 Metetra
.633 .681 .877 r
.46548 .26887 .4694 .27484 .47636 .27271 .47245 .26669 Metetra
.636 .681 .875 r
.47245 .26669 .47636 .27271 .48334 .27049 .47943 .26444 Metetra
.64 .681 .873 r
.47943 .26444 .48334 .27049 .49034 .26819 .48644 .26211 Metetra
.643 .682 .872 r
.48644 .26211 .49034 .26819 .49736 .26581 .49347 .25972 Metetra
.646 .683 .871 r
.49347 .25972 .49736 .26581 .5044 .26336 .50052 .25726 Metetra
.649 .685 .87 r
.50052 .25726 .5044 .26336 .51146 .26084 .50759 .25474 Metetra
.652 .686 .87 r
.50759 .25474 .51146 .26084 .51853 .25825 .51467 .25215 Metetra
.654 .688 .87 r
.51467 .25215 .51853 .25825 .52563 .2556 .52177 .24951 Metetra
.656 .69 .87 r
.52177 .24951 .52563 .2556 .53274 .25289 .52889 .24683 Metetra
.659 .692 .87 r
.52889 .24683 .53274 .25289 .53987 .25013 .53603 .24408 Metetra
.661 .694 .87 r
.53603 .24408 .53987 .25013 .54701 .24732 .54319 .2413 Metetra
.663 .697 .871 r
.54319 .2413 .54701 .24732 .55418 .24447 .55036 .23847 Metetra
.664 .699 .872 r
.55036 .23847 .55418 .24447 .56136 .24158 .55755 .23561 Metetra
.666 .701 .873 r
.55755 .23561 .56136 .24158 .56856 .23866 .56476 .23272 Metetra
.668 .704 .874 r
.56476 .23272 .56856 .23866 .57577 .2357 .57199 .22979 Metetra
.669 .706 .875 r
.57199 .22979 .57577 .2357 .583 .23271 .57923 .22684 Metetra
.67 .709 .877 r
.57923 .22684 .583 .23271 .59025 .22969 .58649 .22386 Metetra
.672 .711 .878 r
.58649 .22386 .59025 .22969 .59751 .22665 .59376 .22086 Metetra
.673 .714 .879 r
.59376 .22086 .59751 .22665 .60479 .22359 .60105 .21783 Metetra
.674 .716 .881 r
.60105 .21783 .60479 .22359 .61209 .22051 .60836 .21479 Metetra
.675 .719 .882 r
.60836 .21479 .61209 .22051 .6194 .2174 .61569 .21172 Metetra
.675 .722 .884 r
.61569 .21172 .6194 .2174 .62673 .21428 .62303 .20864 Metetra
.676 .724 .886 r
.62303 .20864 .62673 .21428 .63407 .21114 .63039 .20555 Metetra
.677 .727 .888 r
.63039 .20555 .63407 .21114 .64144 .20799 .63777 .20244 Metetra
.678 .73 .889 r
.63777 .20244 .64144 .20799 .64882 .20483 .64516 .19931 Metetra
.678 .732 .891 r
.64516 .19931 .64882 .20483 .65621 .20165 .65257 .19618 Metetra
.679 .735 .893 r
.65257 .19618 .65621 .20165 .66362 .19846 .66 .19303 Metetra
.679 .738 .895 r
.66 .19303 .66362 .19846 .67105 .19526 .66744 .18987 Metetra
.68 .74 .897 r
.66744 .18987 .67105 .19526 .6785 .19205 .67491 .18671 Metetra
.681 .743 .898 r
.67491 .18671 .6785 .19205 .68596 .18883 .68239 .18352 Metetra
.681 .746 .9 r
.68239 .18352 .68596 .18883 .69344 .1856 .68988 .18034 Metetra
.681 .749 .902 r
.68988 .18034 .69344 .1856 .70094 .18237 .6974 .17714 Metetra
.682 .751 .904 r
.6974 .17714 .70094 .18237 .70846 .17912 .70493 .17394 Metetra
.682 .754 .906 r
.70493 .17394 .70846 .17912 .71599 .17587 .71248 .17072 Metetra
.682 .757 .908 r
.71248 .17072 .71599 .17587 .72354 .1726 .72005 .16751 Metetra
.683 .76 .91 r
.72005 .16751 .72354 .1726 .73111 .16933 .72763 .16428 Metetra
.682 .757 .908 r
.05631 .40042 .06085 .40438 .06687 .40177 .06233 .39785 Metetra
.682 .76 .91 r
.06233 .39785 .06687 .40177 .0729 .39916 .06837 .39528 Metetra
.683 .763 .912 r
.06837 .39528 .0729 .39916 .07895 .39654 .07441 .3927 Metetra
.683 .765 .914 r
.07441 .3927 .07895 .39654 .08501 .39391 .08047 .39012 Metetra
.683 .768 .916 r
.08047 .39012 .08501 .39391 .09108 .39129 .08655 .38754 Metetra
.683 .771 .918 r
.08655 .38754 .09108 .39129 .09716 .38866 .09263 .38495 Metetra
.683 .774 .92 r
.09263 .38495 .09716 .38866 .10326 .38602 .09873 .38236 Metetra
.683 .777 .922 r
.09873 .38236 .10326 .38602 .10937 .38338 .10484 .37976 Metetra
.683 .779 .924 r
.10484 .37976 .10937 .38338 .11549 .38074 .11096 .37716 Metetra
.683 .782 .926 r
.11096 .37716 .11549 .38074 .12162 .3781 .1171 .37456 Metetra
.683 .785 .928 r
.1171 .37456 .12162 .3781 .12776 .37545 .12325 .37195 Metetra
.683 .788 .93 r
.12325 .37195 .12776 .37545 .13392 .3728 .12941 .36935 Metetra
.683 .791 .931 r
.12941 .36935 .13392 .3728 .14009 .37016 .13558 .36674 Metetra
.682 .793 .933 r
.13558 .36674 .14009 .37016 .14627 .36751 .14177 .36414 Metetra
.682 .796 .935 r
.14177 .36414 .14627 .36751 .15247 .36487 .14797 .36154 Metetra
.681 .799 .937 r
.14797 .36154 .15247 .36487 .15868 .36223 .15418 .35894 Metetra
.68 .801 .94 r
.15418 .35894 .15868 .36223 .16489 .3596 .1604 .35634 Metetra
.679 .804 .942 r
.1604 .35634 .16489 .3596 .17113 .35697 .16664 .35375 Metetra
.678 .806 .944 r
.16664 .35375 .17113 .35697 .17737 .35435 .17289 .35117 Metetra
.677 .809 .946 r
.17289 .35117 .17737 .35435 .18362 .35174 .17915 .3486 Metetra
.676 .811 .948 r
.17915 .3486 .18362 .35174 .18989 .34915 .18542 .34604 Metetra
.674 .813 .95 r
.18542 .34604 .18989 .34915 .19617 .34657 .19171 .34349 Metetra
.672 .815 .952 r
.19171 .34349 .19617 .34657 .20246 .344 .198 .34095 Metetra
.67 .817 .954 r
.198 .34095 .20246 .344 .20876 .34146 .20431 .33844 Metetra
.668 .819 .956 r
.20431 .33844 .20876 .34146 .21507 .33894 .21063 .33594 Metetra
.666 .82 .958 r
.21063 .33594 .21507 .33894 .2214 .33644 .21697 .33347 Metetra
.663 .822 .96 r
.21697 .33347 .2214 .33644 .22773 .33398 .22331 .33102 Metetra
.66 .822 .962 r
.22331 .33102 .22773 .33398 .23408 .33155 .22966 .32861 Metetra
.657 .823 .963 r
.22966 .32861 .23408 .33155 .24043 .32916 .23603 .32622 Metetra
.653 .823 .965 r
.23603 .32622 .24043 .32916 .2468 .32681 .24241 .32388 Metetra
.649 .823 .967 r
.24241 .32388 .2468 .32681 .25318 .3245 .2488 .32157 Metetra
.645 .822 .968 r
.2488 .32157 .25318 .3245 .25957 .32225 .2552 .31931 Metetra
.64 .821 .97 r
.2552 .31931 .25957 .32225 .26597 .32005 .26162 .31709 Metetra
.636 .82 .971 r
.26162 .31709 .26597 .32005 .27238 .3179 .26804 .31492 Metetra
.631 .817 .972 r
.26804 .31492 .27238 .3179 .27881 .31581 .27448 .3128 Metetra
.627 .815 .973 r
.27448 .3128 .27881 .31581 .28524 .31378 .28093 .31073 Metetra
.622 .812 .973 r
.28093 .31073 .28524 .31378 .29169 .31182 .28739 .30871 Metetra
.617 .808 .973 r
.28739 .30871 .29169 .31182 .29815 .30992 .29386 .30675 Metetra
.612 .803 .973 r
.29386 .30675 .29815 .30992 .30462 .30808 .30035 .30485 Metetra
.609 .799 .972 r
.30035 .30485 .30462 .30808 .31111 .30631 .30686 .30299 Metetra
.605 .793 .971 r
.30686 .30299 .31111 .30631 .31761 .30459 .31337 .30118 Metetra
.601 .787 .97 r
.31337 .30118 .31761 .30459 .32413 .30293 .3199 .29943 Metetra
.598 .781 .968 r
.3199 .29943 .32413 .30293 .33066 .30133 .32645 .29772 Metetra
.596 .774 .965 r
.32645 .29772 .33066 .30133 .3372 .29978 .33302 .29605 Metetra
.594 .767 .962 r
.33302 .29605 .3372 .29978 .34377 .29827 .3396 .29442 Metetra
.593 .761 .959 r
.3396 .29442 .34377 .29827 .35035 .29679 .3462 .29281 Metetra
.593 .753 .955 r
.3462 .29281 .35035 .29679 .35695 .29534 .35281 .29122 Metetra
.593 .747 .95 r
.35281 .29122 .35695 .29534 .36357 .29391 .35945 .28965 Metetra
.593 .739 .946 r
.35945 .28965 .36357 .29391 .37021 .29249 .36611 .28809 Metetra
.594 .733 .941 r
.36611 .28809 .37021 .29249 .37687 .29107 .37278 .28653 Metetra
.596 .726 .935 r
.37278 .28653 .37687 .29107 .38355 .28964 .37948 .28496 Metetra
.598 .72 .93 r
.37948 .28496 .38355 .28964 .39026 .28819 .3862 .28337 Metetra
.6 .715 .925 r
.3862 .28337 .39026 .28819 .39698 .28672 .39294 .28175 Metetra
.603 .709 .919 r
.39294 .28175 .39698 .28672 .40373 .2852 .3997 .28011 Metetra
.605 .705 .914 r
.3997 .28011 .40373 .2852 .4105 .28364 .40648 .27842 Metetra
.608 .7 .909 r
.40648 .27842 .4105 .28364 .4173 .28204 .41329 .27669 Metetra
.612 .696 .904 r
.41329 .27669 .4173 .28204 .42411 .28037 .42012 .27492 Metetra
.615 .693 .9 r
.42012 .27492 .42411 .28037 .43095 .27863 .42697 .27308 Metetra
.619 .691 .895 r
.42697 .27308 .43095 .27863 .43781 .27683 .43384 .27118 Metetra
.622 .688 .891 r
.43384 .27118 .43781 .27683 .4447 .27495 .44073 .26922 Metetra
.625 .687 .888 r
.44073 .26922 .4447 .27495 .4516 .273 .44765 .26719 Metetra
.628 .685 .885 r
.44765 .26719 .4516 .273 .45853 .27097 .45458 .2651 Metetra
.632 .685 .882 r
.45458 .2651 .45853 .27097 .46548 .26887 .46154 .26294 Metetra
.635 .684 .879 r
.46154 .26294 .46548 .26887 .47245 .26669 .46852 .26072 Metetra
.638 .684 .877 r
.46852 .26072 .47245 .26669 .47943 .26444 .47551 .25843 Metetra
.641 .684 .875 r
.47551 .25843 .47943 .26444 .48644 .26211 .48253 .25608 Metetra
.644 .685 .874 r
.48253 .25608 .48644 .26211 .49347 .25972 .48957 .25367 Metetra
.646 .686 .873 r
.48957 .25367 .49347 .25972 .50052 .25726 .49662 .2512 Metetra
.649 .687 .872 r
.49662 .2512 .50052 .25726 .50759 .25474 .5037 .24867 Metetra
.651 .688 .872 r
.5037 .24867 .50759 .25474 .51467 .25215 .51079 .24608 Metetra
.653 .69 .872 r
.51079 .24608 .51467 .25215 .52177 .24951 .51791 .24345 Metetra
.656 .692 .872 r
.51791 .24345 .52177 .24951 .52889 .24683 .52504 .24077 Metetra
.658 .693 .872 r
.52504 .24077 .52889 .24683 .53603 .24408 .53218 .23805 Metetra
.66 .695 .872 r
.53218 .23805 .53603 .24408 .54319 .2413 .53935 .23528 Metetra
.661 .697 .873 r
.53935 .23528 .54319 .2413 .55036 .23847 .54653 .23248 Metetra
.663 .7 .874 r
.54653 .23248 .55036 .23847 .55755 .23561 .55373 .22965 Metetra
.664 .702 .874 r
.55373 .22965 .55755 .23561 .56476 .23272 .56095 .22678 Metetra
.666 .704 .876 r
.56095 .22678 .56476 .23272 .57199 .22979 .56819 .22388 Metetra
.667 .706 .877 r
.56819 .22388 .57199 .22979 .57923 .22684 .57544 .22096 Metetra
.668 .709 .878 r
.57544 .22096 .57923 .22684 .58649 .22386 .58271 .21802 Metetra
.669 .711 .879 r
.58271 .21802 .58649 .22386 .59376 .22086 .59 .21505 Metetra
.67 .713 .881 r
.59 .21505 .59376 .22086 .60105 .21783 .5973 .21206 Metetra
.671 .716 .882 r
.5973 .21206 .60105 .21783 .60836 .21479 .60462 .20906 Metetra
.672 .719 .884 r
.60462 .20906 .60836 .21479 .61569 .21172 .61196 .20603 Metetra
.673 .721 .885 r
.61196 .20603 .61569 .21172 .62303 .20864 .61932 .20299 Metetra
.674 .724 .887 r
.61932 .20299 .62303 .20864 .63039 .20555 .62669 .19993 Metetra
.674 .726 .889 r
.62669 .19993 .63039 .20555 .63777 .20244 .63408 .19686 Metetra
.675 .729 .89 r
.63408 .19686 .63777 .20244 .64516 .19931 .64149 .19378 Metetra
.676 .731 .892 r
.64149 .19378 .64516 .19931 .65257 .19618 .64891 .19068 Metetra
.676 .734 .894 r
.64891 .19068 .65257 .19618 .66 .19303 .65636 .18758 Metetra
.677 .737 .896 r
.65636 .18758 .66 .19303 .66744 .18987 .66382 .18446 Metetra
.677 .739 .898 r
.66382 .18446 .66744 .18987 .67491 .18671 .67129 .18133 Metetra
.678 .742 .899 r
.67129 .18133 .67491 .18671 .68239 .18352 .67879 .1782 Metetra
.678 .745 .901 r
.67879 .1782 .68239 .18352 .68988 .18034 .6863 .17505 Metetra
.678 .748 .903 r
.6863 .17505 .68988 .18034 .6974 .17714 .69383 .1719 Metetra
.679 .751 .905 r
.69383 .1719 .6974 .17714 .70493 .17394 .70138 .16873 Metetra
.679 .753 .907 r
.70138 .16873 .70493 .17394 .71248 .17072 .70895 .16556 Metetra
.679 .756 .909 r
.70895 .16556 .71248 .17072 .72005 .16751 .71653 .16238 Metetra
.68 .759 .911 r
.71653 .16238 .72005 .16751 .72763 .16428 .72414 .15919 Metetra
.679 .757 .91 r
.05175 .39644 .05631 .40042 .06233 .39785 .05777 .39391 Metetra
.679 .759 .912 r
.05777 .39391 .06233 .39785 .06837 .39528 .06381 .39138 Metetra
.679 .762 .914 r
.06381 .39138 .06837 .39528 .07441 .3927 .06986 .38885 Metetra
.68 .765 .915 r
.06986 .38885 .07441 .3927 .08047 .39012 .07592 .38631 Metetra
.68 .767 .917 r
.07592 .38631 .08047 .39012 .08655 .38754 .08199 .38377 Metetra
.68 .77 .919 r
.08199 .38377 .08655 .38754 .09263 .38495 .08808 .38122 Metetra
.68 .773 .921 r
.08808 .38122 .09263 .38495 .09873 .38236 .09418 .37867 Metetra
.68 .776 .923 r
.09418 .37867 .09873 .38236 .10484 .37976 .10029 .37612 Metetra
.68 .779 .925 r
.10029 .37612 .10484 .37976 .11096 .37716 .10642 .37356 Metetra
.68 .781 .927 r
.10642 .37356 .11096 .37716 .1171 .37456 .11256 .371 Metetra
.68 .784 .929 r
.11256 .371 .1171 .37456 .12325 .37195 .11871 .36844 Metetra
.679 .787 .931 r
.11871 .36844 .12325 .37195 .12941 .36935 .12487 .36587 Metetra
.679 .79 .933 r
.12487 .36587 .12941 .36935 .13558 .36674 .13105 .36331 Metetra
.679 .792 .935 r
.13105 .36331 .13558 .36674 .14177 .36414 .13724 .36074 Metetra
.678 .795 .937 r
.13724 .36074 .14177 .36414 .14797 .36154 .14345 .35818 Metetra
.678 .798 .939 r
.14345 .35818 .14797 .36154 .15418 .35894 .14966 .35562 Metetra
.677 .8 .941 r
.14966 .35562 .15418 .35894 .1604 .35634 .15589 .35306 Metetra
.676 .803 .943 r
.15589 .35306 .1604 .35634 .16664 .35375 .16213 .35051 Metetra
.675 .805 .945 r
.16213 .35051 .16664 .35375 .17289 .35117 .16839 .34796 Metetra
.674 .807 .946 r
.16839 .34796 .17289 .35117 .17915 .3486 .17465 .34542 Metetra
.673 .809 .948 r
.17465 .34542 .17915 .3486 .18542 .34604 .18093 .34289 Metetra
.671 .812 .95 r
.18093 .34289 .18542 .34604 .19171 .34349 .18722 .34037 Metetra
.67 .814 .952 r
.18722 .34037 .19171 .34349 .198 .34095 .19353 .33786 Metetra
.668 .815 .954 r
.19353 .33786 .198 .34095 .20431 .33844 .19984 .33538 Metetra
.666 .817 .956 r
.19984 .33538 .20431 .33844 .21063 .33594 .20617 .3329 Metetra
.664 .818 .958 r
.20617 .3329 .21063 .33594 .21697 .33347 .21251 .33045 Metetra
.661 .819 .959 r
.21251 .33045 .21697 .33347 .22331 .33102 .21887 .32802 Metetra
.659 .82 .961 r
.21887 .32802 .22331 .33102 .22966 .32861 .22523 .32561 Metetra
.655 .82 .963 r
.22523 .32561 .22966 .32861 .23603 .32622 .23161 .32323 Metetra
.652 .821 .964 r
.23161 .32323 .23603 .32622 .24241 .32388 .238 .32089 Metetra
.649 .821 .966 r
.238 .32089 .24241 .32388 .2488 .32157 .2444 .31857 Metetra
.645 .82 .967 r
.2444 .31857 .2488 .32157 .2552 .31931 .25081 .3163 Metetra
.642 .819 .968 r
.25081 .3163 .2552 .31931 .26162 .31709 .25724 .31406 Metetra
.638 .817 .969 r
.25724 .31406 .26162 .31709 .26804 .31492 .26368 .31187 Metetra
.634 .815 .97 r
.26368 .31187 .26804 .31492 .27448 .3128 .27013 .30971 Metetra
.63 .813 .97 r
.27013 .30971 .27448 .3128 .28093 .31073 .27659 .3076 Metetra
.626 .81 .97 r
.27659 .3076 .28093 .31073 .28739 .30871 .28307 .30553 Metetra
.623 .806 .97 r
.28307 .30553 .28739 .30871 .29386 .30675 .28956 .30351 Metetra
.619 .802 .97 r
.28956 .30351 .29386 .30675 .30035 .30485 .29606 .30153 Metetra
.616 .797 .969 r
.29606 .30153 .30035 .30485 .30686 .30299 .30258 .2996 Metetra
.613 .792 .968 r
.30258 .2996 .30686 .30299 .31337 .30118 .30912 .29771 Metetra
.61 .787 .966 r
.30912 .29771 .31337 .30118 .3199 .29943 .31566 .29585 Metetra
.607 .781 .964 r
.31566 .29585 .3199 .29943 .32645 .29772 .32223 .29404 Metetra
.606 .775 .962 r
.32223 .29404 .32645 .29772 .33302 .29605 .32881 .29226 Metetra
.604 .769 .958 r
.32881 .29226 .33302 .29605 .3396 .29442 .33541 .29051 Metetra
.603 .762 .955 r
.33541 .29051 .3396 .29442 .3462 .29281 .34202 .28878 Metetra
.603 .756 .951 r
.34202 .28878 .3462 .29281 .35281 .29122 .34866 .28706 Metetra
.603 .749 .947 r
.34866 .28706 .35281 .29122 .35945 .28965 .35531 .28536 Metetra
.603 .743 .943 r
.35531 .28536 .35945 .28965 .36611 .28809 .36198 .28367 Metetra
.604 .736 .938 r
.36198 .28367 .36611 .28809 .37278 .28653 .36867 .28197 Metetra
.605 .73 .933 r
.36867 .28197 .37278 .28653 .37948 .28496 .37539 .28027 Metetra
.607 .725 .928 r
.37539 .28027 .37948 .28496 .3862 .28337 .38212 .27854 Metetra
.608 .719 .923 r
.38212 .27854 .3862 .28337 .39294 .28175 .38887 .2768 Metetra
.611 .714 .918 r
.38887 .2768 .39294 .28175 .3997 .28011 .39565 .27503 Metetra
.613 .709 .913 r
.39565 .27503 .3997 .28011 .40648 .27842 .40245 .27322 Metetra
.615 .705 .909 r
.40245 .27322 .40648 .27842 .41329 .27669 .40926 .27138 Metetra
.618 .701 .904 r
.40926 .27138 .41329 .27669 .42012 .27492 .4161 .26949 Metetra
.62 .698 .9 r
.4161 .26949 .42012 .27492 .42697 .27308 .42296 .26756 Metetra
.623 .695 .896 r
.42296 .26756 .42697 .27308 .43384 .27118 .42985 .26556 Metetra
.626 .693 .892 r
.42985 .26556 .43384 .27118 .44073 .26922 .43675 .26352 Metetra
.629 .691 .889 r
.43675 .26352 .44073 .26922 .44765 .26719 .44367 .26142 Metetra
.631 .689 .886 r
.44367 .26142 .44765 .26719 .45458 .2651 .45062 .25927 Metetra
.634 .688 .883 r
.45062 .25927 .45458 .2651 .46154 .26294 .45759 .25705 Metetra
.637 .688 .881 r
.45759 .25705 .46154 .26294 .46852 .26072 .46457 .25478 Metetra
.639 .687 .879 r
.46457 .25478 .46852 .26072 .47551 .25843 .47158 .25246 Metetra
.642 .687 .877 r
.47158 .25246 .47551 .25843 .48253 .25608 .4786 .25007 Metetra
.644 .687 .876 r
.4786 .25007 .48253 .25608 .48957 .25367 .48565 .24764 Metetra
.647 .688 .875 r
.48565 .24764 .48957 .25367 .49662 .2512 .49272 .24515 Metetra
.649 .689 .874 r
.49272 .24515 .49662 .2512 .5037 .24867 .4998 .24262 Metetra
.651 .69 .873 r
.4998 .24262 .5037 .24867 .51079 .24608 .5069 .24003 Metetra
.653 .691 .873 r
.5069 .24003 .51079 .24608 .51791 .24345 .51402 .2374 Metetra
.655 .693 .873 r
.51402 .2374 .51791 .24345 .52504 .24077 .52116 .23473 Metetra
.657 .695 .873 r
.52116 .23473 .52504 .24077 .53218 .23805 .52832 .23202 Metetra
.658 .696 .874 r
.52832 .23202 .53218 .23805 .53935 .23528 .53549 .22927 Metetra
.66 .698 .874 r
.53549 .22927 .53935 .23528 .54653 .23248 .54269 .22649 Metetra
.661 .7 .875 r
.54269 .22649 .54653 .23248 .55373 .22965 .5499 .22368 Metetra
.663 .702 .876 r
.5499 .22368 .55373 .22965 .56095 .22678 .55713 .22084 Metetra
.664 .704 .877 r
.55713 .22084 .56095 .22678 .56819 .22388 .56437 .21797 Metetra
.665 .706 .878 r
.56437 .21797 .56819 .22388 .57544 .22096 .57164 .21508 Metetra
.666 .708 .879 r
.57164 .21508 .57544 .22096 .58271 .21802 .57892 .21217 Metetra
.667 .711 .881 r
.57892 .21217 .58271 .21802 .59 .21505 .58622 .20924 Metetra
.668 .713 .882 r
.58622 .20924 .59 .21505 .5973 .21206 .59353 .20628 Metetra
.669 .716 .883 r
.59353 .20628 .5973 .21206 .60462 .20906 .60087 .20331 Metetra
.67 .718 .885 r
.60087 .20331 .60462 .20906 .61196 .20603 .60822 .20032 Metetra
.67 .72 .886 r
.60822 .20032 .61196 .20603 .61932 .20299 .61559 .19732 Metetra
.671 .723 .888 r
.61559 .19732 .61932 .20299 .62669 .19993 .62297 .1943 Metetra
.672 .726 .89 r
.62297 .1943 .62669 .19993 .63408 .19686 .63038 .19127 Metetra
.672 .728 .891 r
.63038 .19127 .63408 .19686 .64149 .19378 .6378 .18823 Metetra
.673 .731 .893 r
.6378 .18823 .64149 .19378 .64891 .19068 .64524 .18517 Metetra
.673 .733 .895 r
.64524 .18517 .64891 .19068 .65636 .18758 .6527 .18211 Metetra
.674 .736 .897 r
.6527 .18211 .65636 .18758 .66382 .18446 .66017 .17903 Metetra
.674 .738 .899 r
.66017 .17903 .66382 .18446 .67129 .18133 .66766 .17594 Metetra
.675 .741 .901 r
.66766 .17594 .67129 .18133 .67879 .1782 .67518 .17284 Metetra
.675 .744 .902 r
.67518 .17284 .67879 .1782 .6863 .17505 .68271 .16974 Metetra
.676 .747 .904 r
.68271 .16974 .6863 .17505 .69383 .1719 .69025 .16663 Metetra
.676 .749 .906 r
.69025 .16663 .69383 .1719 .70138 .16873 .69782 .1635 Metetra
.676 .752 .908 r
.69782 .1635 .70138 .16873 .70895 .16556 .7054 .16037 Metetra
.676 .755 .91 r
.7054 .16037 .70895 .16556 .71653 .16238 .71301 .15723 Metetra
.677 .757 .912 r
.71301 .15723 .71653 .16238 .72414 .15919 .72063 .15409 Metetra
.676 .756 .911 r
.04717 .39244 .05175 .39644 .05777 .39391 .05319 .38995 Metetra
.676 .759 .913 r
.05319 .38995 .05777 .39391 .06381 .39138 .05923 .38747 Metetra
.676 .761 .915 r
.05923 .38747 .06381 .39138 .06986 .38885 .06528 .38498 Metetra
.676 .764 .917 r
.06528 .38498 .06986 .38885 .07592 .38631 .07134 .38248 Metetra
.676 .767 .919 r
.07134 .38248 .07592 .38631 .08199 .38377 .07742 .37998 Metetra
.677 .769 .921 r
.07742 .37998 .08199 .38377 .08808 .38122 .08351 .37748 Metetra
.677 .772 .923 r
.08351 .37748 .08808 .38122 .09418 .37867 .08961 .37497 Metetra
.677 .775 .925 r
.08961 .37497 .09418 .37867 .10029 .37612 .09572 .37245 Metetra
.676 .778 .927 r
.09572 .37245 .10029 .37612 .10642 .37356 .10185 .36994 Metetra
.676 .78 .928 r
.10185 .36994 .10642 .37356 .11256 .371 .10799 .36742 Metetra
.676 .783 .93 r
.10799 .36742 .11256 .371 .11871 .36844 .11415 .3649 Metetra
.676 .786 .932 r
.11415 .3649 .11871 .36844 .12487 .36587 .12032 .36238 Metetra
.676 .789 .934 r
.12032 .36238 .12487 .36587 .13105 .36331 .1265 .35985 Metetra
.675 .791 .936 r
.1265 .35985 .13105 .36331 .13724 .36074 .13269 .35733 Metetra
.675 .794 .938 r
.13269 .35733 .13724 .36074 .14345 .35818 .1389 .3548 Metetra
.674 .796 .94 r
.1389 .3548 .14345 .35818 .14966 .35562 .14512 .35228 Metetra
.674 .799 .942 r
.14512 .35228 .14966 .35562 .15589 .35306 .15136 .34975 Metetra
.673 .801 .944 r
.15136 .34975 .15589 .35306 .16213 .35051 .1576 .34724 Metetra
.672 .804 .945 r
.1576 .34724 .16213 .35051 .16839 .34796 .16386 .34472 Metetra
.671 .806 .947 r
.16386 .34472 .16839 .34796 .17465 .34542 .17014 .34222 Metetra
.67 .808 .949 r
.17014 .34222 .17465 .34542 .18093 .34289 .17642 .33971 Metetra
.669 .81 .951 r
.17642 .33971 .18093 .34289 .18722 .34037 .18272 .33722 Metetra
.667 .812 .953 r
.18272 .33722 .18722 .34037 .19353 .33786 .18903 .33474 Metetra
.666 .814 .954 r
.18903 .33474 .19353 .33786 .19984 .33538 .19536 .33227 Metetra
.664 .815 .956 r
.19536 .33227 .19984 .33538 .20617 .3329 .20169 .32982 Metetra
.662 .816 .958 r
.20169 .32982 .20617 .3329 .21251 .33045 .20804 .32738 Metetra
.66 .817 .959 r
.20804 .32738 .21251 .33045 .21887 .32802 .21441 .32496 Metetra
.657 .818 .961 r
.21441 .32496 .21887 .32802 .22523 .32561 .22078 .32257 Metetra
.655 .819 .962 r
.22078 .32257 .22523 .32561 .23161 .32323 .22717 .32019 Metetra
.652 .819 .963 r
.22717 .32019 .23161 .32323 .238 .32089 .23357 .31784 Metetra
.649 .818 .964 r
.23357 .31784 .238 .32089 .2444 .31857 .23998 .31552 Metetra
.646 .818 .966 r
.23998 .31552 .2444 .31857 .25081 .3163 .24641 .31323 Metetra
.643 .816 .966 r
.24641 .31323 .25081 .3163 .25724 .31406 .25284 .31098 Metetra
.64 .815 .967 r
.25284 .31098 .25724 .31406 .26368 .31187 .2593 .30875 Metetra
.637 .813 .967 r
.2593 .30875 .26368 .31187 .27013 .30971 .26576 .30656 Metetra
.634 .811 .968 r
.26576 .30656 .27013 .30971 .27659 .3076 .27224 .3044 Metetra
.631 .808 .968 r
.27224 .3044 .27659 .3076 .28307 .30553 .27873 .30228 Metetra
.628 .805 .967 r
.27873 .30228 .28307 .30553 .28956 .30351 .28524 .3002 Metetra
.625 .801 .967 r
.28524 .3002 .28956 .30351 .29606 .30153 .29176 .29816 Metetra
.622 .797 .966 r
.29176 .29816 .29606 .30153 .30258 .2996 .29829 .29615 Metetra
.62 .792 .964 r
.29829 .29615 .30258 .2996 .30912 .29771 .30484 .29417 Metetra
.618 .787 .963 r
.30484 .29417 .30912 .29771 .31566 .29585 .31141 .29222 Metetra
.616 .782 .961 r
.31141 .29222 .31566 .29585 .32223 .29404 .31799 .29031 Metetra
.614 .776 .958 r
.31799 .29031 .32223 .29404 .32881 .29226 .32458 .28842 Metetra
.613 .77 .955 r
.32458 .28842 .32881 .29226 .33541 .29051 .3312 .28655 Metetra
.613 .764 .952 r
.3312 .28655 .33541 .29051 .34202 .28878 .33783 .28471 Metetra
.612 .758 .948 r
.33783 .28471 .34202 .28878 .34866 .28706 .34448 .28287 Metetra
.612 .752 .944 r
.34448 .28287 .34866 .28706 .35531 .28536 .35115 .28104 Metetra
.612 .745 .94 r
.35115 .28104 .35531 .28536 .36198 .28367 .35784 .27922 Metetra
.613 .739 .936 r
.35784 .27922 .36198 .28367 .36867 .28197 .36455 .2774 Metetra
.614 .734 .931 r
.36455 .2774 .36867 .28197 .37539 .28027 .37127 .27556 Metetra
.615 .728 .926 r
.37127 .27556 .37539 .28027 .38212 .27854 .37802 .27371 Metetra
.617 .723 .922 r
.37802 .27371 .38212 .27854 .38887 .2768 .38479 .27184 Metetra
.618 .718 .917 r
.38479 .27184 .38887 .2768 .39565 .27503 .39158 .26995 Metetra
.619 .713 .913 r
.39158 .26995 .39565 .27503 .40245 .27322 .39839 .26803 Metetra
.622 .709 .908 r
.39839 .26803 .40245 .27322 .40926 .27138 .40522 .26608 Metetra
.623 .705 .904 r
.40522 .26608 .40926 .27138 .4161 .26949 .41207 .26409 Metetra
.626 .702 .9 r
.41207 .26409 .4161 .26949 .42296 .26756 .41894 .26205 Metetra
.628 .699 .896 r
.41894 .26205 .42296 .26756 .42985 .26556 .42584 .25997 Metetra
.63 .696 .893 r
.42584 .25997 .42985 .26556 .43675 .26352 .43275 .25785 Metetra
.632 .694 .89 r
.43275 .25785 .43675 .26352 .44367 .26142 .43968 .25568 Metetra
.635 .693 .887 r
.43968 .25568 .44367 .26142 .45062 .25927 .44664 .25346 Metetra
.637 .691 .884 r
.44664 .25346 .45062 .25927 .45759 .25705 .45362 .25119 Metetra
.639 .691 .882 r
.45362 .25119 .45759 .25705 .46457 .25478 .46061 .24888 Metetra
.641 .69 .88 r
.46061 .24888 .46457 .25478 .47158 .25246 .46763 .24651 Metetra
.643 .69 .879 r
.46763 .24651 .47158 .25246 .4786 .25007 .47466 .2441 Metetra
.645 .69 .877 r
.47466 .2441 .4786 .25007 .48565 .24764 .48172 .24164 Metetra
.647 .69 .876 r
.48172 .24164 .48565 .24764 .49272 .24515 .48879 .23913 Metetra
.649 .691 .876 r
.48879 .23913 .49272 .24515 .4998 .24262 .49588 .23658 Metetra
.651 .692 .875 r
.49588 .23658 .4998 .24262 .5069 .24003 .50299 .23399 Metetra
.652 .693 .875 r
.50299 .23399 .5069 .24003 .51402 .2374 .51012 .23137 Metetra
.654 .694 .875 r
.51012 .23137 .51402 .2374 .52116 .23473 .51727 .2287 Metetra
.656 .695 .875 r
.51727 .2287 .52116 .23473 .52832 .23202 .52444 .226 Metetra
.657 .697 .875 r
.52444 .226 .52832 .23202 .53549 .22927 .53162 .22326 Metetra
.658 .698 .876 r
.53162 .22326 .53549 .22927 .54269 .22649 .53883 .2205 Metetra
.66 .7 .876 r
.53883 .2205 .54269 .22649 .5499 .22368 .54605 .21771 Metetra
.661 .702 .877 r
.54605 .21771 .5499 .22368 .55713 .22084 .55329 .2149 Metetra
.662 .704 .878 r
.55329 .2149 .55713 .22084 .56437 .21797 .56054 .21205 Metetra
.663 .706 .879 r
.56054 .21205 .56437 .21797 .57164 .21508 .56782 .20919 Metetra
.664 .708 .881 r
.56782 .20919 .57164 .21508 .57892 .21217 .57511 .20631 Metetra
.665 .71 .882 r
.57511 .20631 .57892 .21217 .58622 .20924 .58242 .20341 Metetra
.666 .713 .883 r
.58242 .20341 .58622 .20924 .59353 .20628 .58975 .20048 Metetra
.666 .715 .885 r
.58975 .20048 .59353 .20628 .60087 .20331 .59709 .19755 Metetra
.667 .717 .886 r
.59709 .19755 .60087 .20331 .60822 .20032 .60446 .1946 Metetra
.668 .72 .888 r
.60446 .1946 .60822 .20032 .61559 .19732 .61184 .19163 Metetra
.669 .722 .889 r
.61184 .19163 .61559 .19732 .62297 .1943 .61924 .18865 Metetra
.669 .725 .891 r
.61924 .18865 .62297 .1943 .63038 .19127 .62666 .18566 Metetra
.67 .727 .893 r
.62666 .18566 .63038 .19127 .6378 .18823 .63409 .18265 Metetra
.67 .73 .894 r
.63409 .18265 .6378 .18823 .64524 .18517 .64155 .17963 Metetra
.671 .732 .896 r
.64155 .17963 .64524 .18517 .6527 .18211 .64902 .17661 Metetra
.671 .735 .898 r
.64902 .17661 .6527 .18211 .66017 .17903 .65651 .17357 Metetra
.672 .738 .9 r
.65651 .17357 .66017 .17903 .66766 .17594 .66402 .17053 Metetra
.672 .74 .902 r
.66402 .17053 .66766 .17594 .67518 .17284 .67155 .16747 Metetra
.672 .743 .904 r
.67155 .16747 .67518 .17284 .68271 .16974 .67909 .16441 Metetra
.673 .746 .905 r
.67909 .16441 .68271 .16974 .69025 .16663 .68666 .16133 Metetra
.673 .748 .907 r
.68666 .16133 .69025 .16663 .69782 .1635 .69424 .15825 Metetra
.673 .751 .909 r
.69424 .15825 .69782 .1635 .7054 .16037 .70184 .15516 Metetra
.674 .754 .911 r
.70184 .15516 .7054 .16037 .71301 .15723 .70946 .15206 Metetra
.674 .756 .913 r
.70946 .15206 .71301 .15723 .72063 .15409 .7171 .14895 Metetra
.673 .755 .913 r
.04257 .38842 .04717 .39244 .05319 .38995 .04859 .38598 Metetra
.673 .758 .915 r
.04859 .38598 .05319 .38995 .05923 .38747 .05463 .38354 Metetra
.673 .761 .917 r
.05463 .38354 .05923 .38747 .06528 .38498 .06068 .38109 Metetra
.673 .763 .918 r
.06068 .38109 .06528 .38498 .07134 .38248 .06674 .37863 Metetra
.673 .766 .92 r
.06674 .37863 .07134 .38248 .07742 .37998 .07282 .37617 Metetra
.673 .769 .922 r
.07282 .37617 .07742 .37998 .08351 .37748 .07891 .37371 Metetra
.673 .772 .924 r
.07891 .37371 .08351 .37748 .08961 .37497 .08502 .37124 Metetra
.673 .774 .926 r
.08502 .37124 .08961 .37497 .09572 .37245 .09113 .36877 Metetra
.673 .777 .928 r
.09113 .36877 .09572 .37245 .10185 .36994 .09726 .3663 Metetra
.673 .78 .93 r
.09726 .3663 .10185 .36994 .10799 .36742 .10341 .36382 Metetra
.673 .782 .932 r
.10341 .36382 .10799 .36742 .11415 .3649 .10957 .36134 Metetra
.673 .785 .934 r
.10957 .36134 .11415 .3649 .12032 .36238 .11574 .35885 Metetra
.672 .788 .935 r
.11574 .35885 .12032 .36238 .1265 .35985 .12192 .35637 Metetra
.672 .79 .937 r
.12192 .35637 .1265 .35985 .13269 .35733 .12812 .35388 Metetra
.672 .793 .939 r
.12812 .35388 .13269 .35733 .1389 .3548 .13433 .35139 Metetra
.671 .795 .941 r
.13433 .35139 .1389 .3548 .14512 .35228 .14056 .34891 Metetra
.671 .798 .943 r
.14056 .34891 .14512 .35228 .15136 .34975 .1468 .34642 Metetra
.67 .8 .945 r
.1468 .34642 .15136 .34975 .1576 .34724 .15305 .34394 Metetra
.669 .802 .946 r
.15305 .34394 .1576 .34724 .16386 .34472 .15932 .34145 Metetra
.668 .805 .948 r
.15932 .34145 .16386 .34472 .17014 .34222 .1656 .33898 Metetra
.667 .807 .95 r
.1656 .33898 .17014 .34222 .17642 .33971 .17189 .33651 Metetra
.666 .809 .951 r
.17189 .33651 .17642 .33971 .18272 .33722 .1782 .33404 Metetra
.665 .81 .953 r
.1782 .33404 .18272 .33722 .18903 .33474 .18451 .33158 Metetra
.663 .812 .955 r
.18451 .33158 .18903 .33474 .19536 .33227 .19085 .32914 Metetra
.662 .813 .956 r
.19085 .32914 .19536 .33227 .20169 .32982 .19719 .3267 Metetra
.66 .815 .958 r
.19719 .3267 .20169 .32982 .20804 .32738 .20355 .32427 Metetra
.658 .816 .959 r
.20355 .32427 .20804 .32738 .21441 .32496 .20992 .32187 Metetra
.657 .816 .96 r
.20992 .32187 .21441 .32496 .22078 .32257 .21631 .31947 Metetra
.654 .817 .961 r
.21631 .31947 .22078 .32257 .22717 .32019 .22271 .3171 Metetra
.652 .817 .962 r
.22271 .3171 .22717 .32019 .23357 .31784 .22912 .31475 Metetra
.65 .817 .963 r
.22912 .31475 .23357 .31784 .23998 .31552 .23554 .31241 Metetra
.647 .816 .964 r
.23554 .31241 .23998 .31552 .24641 .31323 .24198 .31011 Metetra
.645 .815 .965 r
.24198 .31011 .24641 .31323 .25284 .31098 .24843 .30783 Metetra
.642 .813 .965 r
.24843 .30783 .25284 .31098 .2593 .30875 .25489 .30558 Metetra
.64 .812 .966 r
.25489 .30558 .2593 .30875 .26576 .30656 .26137 .30335 Metetra
.637 .809 .965 r
.26137 .30335 .26576 .30656 .27224 .3044 .26787 .30115 Metetra
.635 .807 .965 r
.26787 .30115 .27224 .3044 .27873 .30228 .27437 .29898 Metetra
.633 .804 .965 r
.27437 .29898 .27873 .30228 .28524 .3002 .28089 .29683 Metetra
.631 .8 .964 r
.28089 .29683 .28524 .3002 .29176 .29816 .28743 .29472 Metetra
.628 .796 .963 r
.28743 .29472 .29176 .29816 .29829 .29615 .29398 .29263 Metetra
.627 .792 .961 r
.29398 .29263 .29829 .29615 .30484 .29417 .30055 .29057 Metetra
.625 .787 .959 r
.30055 .29057 .30484 .29417 .31141 .29222 .30713 .28853 Metetra
.624 .782 .957 r
.30713 .28853 .31141 .29222 .31799 .29031 .31373 .28652 Metetra
.622 .777 .955 r
.31373 .28652 .31799 .29031 .32458 .28842 .32034 .28453 Metetra
.621 .771 .952 r
.32034 .28453 .32458 .28842 .3312 .28655 .32697 .28256 Metetra
.621 .766 .949 r
.32697 .28256 .3312 .28655 .33783 .28471 .33362 .2806 Metetra
.621 .76 .945 r
.33362 .2806 .33783 .28471 .34448 .28287 .34029 .27864 Metetra
.62 .754 .941 r
.34029 .27864 .34448 .28287 .35115 .28104 .34697 .27669 Metetra
.621 .748 .938 r
.34697 .27669 .35115 .28104 .35784 .27922 .35368 .27475 Metetra
.621 .742 .933 r
.35368 .27475 .35784 .27922 .36455 .2774 .3604 .2728 Metetra
.622 .737 .929 r
.3604 .2728 .36455 .2774 .37127 .27556 .36714 .27084 Metetra
.623 .732 .925 r
.36714 .27084 .37127 .27556 .37802 .27371 .37391 .26887 Metetra
.624 .726 .92 r
.37391 .26887 .37802 .27371 .38479 .27184 .38069 .26688 Metetra
.625 .721 .916 r
.38069 .26688 .38479 .27184 .39158 .26995 .38749 .26488 Metetra
.626 .717 .912 r
.38749 .26488 .39158 .26995 .39839 .26803 .39431 .26285 Metetra
.627 .713 .908 r
.39431 .26285 .39839 .26803 .40522 .26608 .40116 .26079 Metetra
.629 .709 .904 r
.40116 .26079 .40522 .26608 .41207 .26409 .40802 .2587 Metetra
.631 .705 .9 r
.40802 .2587 .41207 .26409 .41894 .26205 .4149 .25657 Metetra
.633 .703 .896 r
.4149 .25657 .41894 .26205 .42584 .25997 .42181 .2544 Metetra
.634 .7 .893 r
.42181 .2544 .42584 .25997 .43275 .25785 .42873 .2522 Metetra
.636 .698 .89 r
.42873 .2522 .43275 .25785 .43968 .25568 .43568 .24996 Metetra
.638 .696 .887 r
.43568 .24996 .43968 .25568 .44664 .25346 .44264 .24768 Metetra
.639 .694 .885 r
.44264 .24768 .44664 .25346 .45362 .25119 .44963 .24535 Metetra
.641 .693 .883 r
.44963 .24535 .45362 .25119 .46061 .24888 .45663 .24298 Metetra
.643 .692 .881 r
.45663 .24298 .46061 .24888 .46763 .24651 .46366 .24058 Metetra
.645 .692 .88 r
.46366 .24058 .46763 .24651 .47466 .2441 .4707 .23813 Metetra
.646 .692 .878 r
.4707 .23813 .47466 .2441 .48172 .24164 .47777 .23565 Metetra
.648 .692 .877 r
.47777 .23565 .48172 .24164 .48879 .23913 .48485 .23312 Metetra
.649 .692 .877 r
.48485 .23312 .48879 .23913 .49588 .23658 .49195 .23056 Metetra
.651 .693 .876 r
.49195 .23056 .49588 .23658 .50299 .23399 .49907 .22796 Metetra
.652 .694 .876 r
.49907 .22796 .50299 .23399 .51012 .23137 .50621 .22533 Metetra
.653 .695 .876 r
.50621 .22533 .51012 .23137 .51727 .2287 .51337 .22267 Metetra
.655 .696 .876 r
.51337 .22267 .51727 .2287 .52444 .226 .52054 .21998 Metetra
.656 .697 .876 r
.52054 .21998 .52444 .226 .53162 .22326 .52774 .21725 Metetra
.657 .699 .877 r
.52774 .21725 .53162 .22326 .53883 .2205 .53495 .21451 Metetra
.658 .7 .878 r
.53495 .21451 .53883 .2205 .54605 .21771 .54218 .21174 Metetra
.659 .702 .878 r
.54218 .21174 .54605 .21771 .55329 .2149 .54943 .20894 Metetra
.66 .704 .879 r
.54943 .20894 .55329 .2149 .56054 .21205 .5567 .20612 Metetra
.661 .706 .88 r
.5567 .20612 .56054 .21205 .56782 .20919 .56398 .20329 Metetra
.662 .708 .882 r
.56398 .20329 .56782 .20919 .57511 .20631 .57129 .20044 Metetra
.663 .71 .883 r
.57129 .20044 .57511 .20631 .58242 .20341 .57861 .19756 Metetra
.664 .712 .884 r
.57861 .19756 .58242 .20341 .58975 .20048 .58595 .19467 Metetra
.664 .714 .886 r
.58595 .19467 .58975 .20048 .59709 .19755 .59331 .19177 Metetra
.665 .717 .887 r
.59331 .19177 .59709 .19755 .60446 .1946 .60068 .18885 Metetra
.666 .719 .889 r
.60068 .18885 .60446 .1946 .61184 .19163 .60808 .18592 Metetra
.666 .722 .89 r
.60808 .18592 .61184 .19163 .61924 .18865 .61549 .18298 Metetra
.667 .724 .892 r
.61549 .18298 .61924 .18865 .62666 .18566 .62292 .18002 Metetra
.667 .726 .894 r
.62292 .18002 .62666 .18566 .63409 .18265 .63037 .17706 Metetra
.668 .729 .895 r
.63037 .17706 .63409 .18265 .64155 .17963 .63784 .17408 Metetra
.668 .731 .897 r
.63784 .17408 .64155 .17963 .64902 .17661 .64533 .17109 Metetra
.668 .734 .899 r
.64533 .17109 .64902 .17661 .65651 .17357 .65283 .1681 Metetra
.669 .736 .901 r
.65283 .1681 .65651 .17357 .66402 .17053 .66036 .16509 Metetra
.669 .739 .903 r
.66036 .16509 .66402 .17053 .67155 .16747 .6679 .16207 Metetra
.669 .742 .904 r
.6679 .16207 .67155 .16747 .67909 .16441 .67546 .15905 Metetra
.67 .744 .906 r
.67546 .15905 .67909 .16441 .68666 .16133 .68304 .15601 Metetra
.67 .747 .908 r
.68304 .15601 .68666 .16133 .69424 .15825 .69064 .15297 Metetra
.67 .75 .91 r
.69064 .15297 .69424 .15825 .70184 .15516 .69826 .14992 Metetra
.671 .752 .912 r
.69826 .14992 .70184 .15516 .70946 .15206 .7059 .14686 Metetra
.671 .755 .914 r
.7059 .14686 .70946 .15206 .7171 .14895 .71356 .14379 Metetra
.669 .755 .914 r
.03795 .38439 .04257 .38842 .04859 .38598 .04397 .38199 Metetra
.67 .757 .916 r
.04397 .38199 .04859 .38598 .05463 .38354 .05001 .37959 Metetra
.67 .76 .918 r
.05001 .37959 .05463 .38354 .06068 .38109 .05606 .37718 Metetra
.67 .763 .92 r
.05606 .37718 .06068 .38109 .06674 .37863 .06213 .37477 Metetra
.67 .765 .922 r
.06213 .37477 .06674 .37863 .07282 .37617 .06821 .37235 Metetra
.67 .768 .924 r
.06821 .37235 .07282 .37617 .07891 .37371 .0743 .36993 Metetra
.67 .771 .926 r
.0743 .36993 .07891 .37371 .08502 .37124 .0804 .3675 Metetra
.67 .774 .928 r
.0804 .3675 .08502 .37124 .09113 .36877 .08652 .36507 Metetra
.67 .776 .929 r
.08652 .36507 .09113 .36877 .09726 .3663 .09266 .36264 Metetra
.67 .779 .931 r
.09266 .36264 .09726 .3663 .10341 .36382 .0988 .3602 Metetra
.67 .782 .933 r
.0988 .3602 .10341 .36382 .10957 .36134 .10497 .35776 Metetra
.669 .784 .935 r
.10497 .35776 .10957 .36134 .11574 .35885 .11114 .35531 Metetra
.669 .787 .937 r
.11114 .35531 .11574 .35885 .12192 .35637 .11733 .35286 Metetra
.669 .789 .939 r
.11733 .35286 .12192 .35637 .12812 .35388 .12353 .35042 Metetra
.669 .792 .94 r
.12353 .35042 .12812 .35388 .13433 .35139 .12975 .34796 Metetra
.668 .794 .942 r
.12975 .34796 .13433 .35139 .14056 .34891 .13598 .34551 Metetra
.668 .797 .944 r
.13598 .34551 .14056 .34891 .1468 .34642 .14222 .34306 Metetra
.667 .799 .946 r
.14222 .34306 .1468 .34642 .15305 .34394 .14848 .34061 Metetra
.666 .801 .947 r
.14848 .34061 .15305 .34394 .15932 .34145 .15475 .33816 Metetra
.665 .803 .949 r
.15475 .33816 .15932 .34145 .1656 .33898 .16104 .33571 Metetra
.665 .806 .951 r
.16104 .33571 .1656 .33898 .17189 .33651 .16734 .33327 Metetra
.664 .807 .952 r
.16734 .33327 .17189 .33651 .1782 .33404 .17365 .33083 Metetra
.662 .809 .954 r
.17365 .33083 .1782 .33404 .18451 .33158 .17997 .32839 Metetra
.661 .811 .955 r
.17997 .32839 .18451 .33158 .19085 .32914 .18631 .32596 Metetra
.66 .812 .956 r
.18631 .32596 .19085 .32914 .19719 .3267 .19267 .32354 Metetra
.659 .813 .958 r
.19267 .32354 .19719 .3267 .20355 .32427 .19904 .32113 Metetra
.657 .814 .959 r
.19904 .32113 .20355 .32427 .20992 .32187 .20542 .31873 Metetra
.656 .815 .96 r
.20542 .31873 .20992 .32187 .21631 .31947 .21181 .31634 Metetra
.654 .815 .961 r
.21181 .31634 .21631 .31947 .22271 .3171 .21822 .31397 Metetra
.652 .815 .962 r
.21822 .31397 .22271 .3171 .22912 .31475 .22464 .31161 Metetra
.65 .815 .963 r
.22464 .31161 .22912 .31475 .23554 .31241 .23108 .30927 Metetra
.648 .815 .963 r
.23108 .30927 .23554 .31241 .24198 .31011 .23753 .30694 Metetra
.646 .813 .964 r
.23753 .30694 .24198 .31011 .24843 .30783 .24399 .30464 Metetra
.645 .812 .964 r
.24399 .30464 .24843 .30783 .25489 .30558 .25047 .30235 Metetra
.643 .81 .964 r
.25047 .30235 .25489 .30558 .26137 .30335 .25696 .30009 Metetra
.641 .809 .963 r
.25696 .30009 .26137 .30335 .26787 .30115 .26347 .29784 Metetra
.639 .806 .963 r
.26347 .29784 .26787 .30115 .27437 .29898 .26999 .29561 Metetra
.637 .803 .962 r
.26999 .29561 .27437 .29898 .28089 .29683 .27653 .29341 Metetra
.636 .8 .961 r
.27653 .29341 .28089 .29683 .28743 .29472 .28308 .29123 Metetra
.634 .796 .96 r
.28308 .29123 .28743 .29472 .29398 .29263 .28965 .28907 Metetra
.633 .792 .958 r
.28965 .28907 .29398 .29263 .30055 .29057 .29623 .28693 Metetra
.632 .787 .957 r
.29623 .28693 .30055 .29057 .30713 .28853 .30283 .2848 Metetra
.631 .783 .954 r
.30283 .2848 .30713 .28853 .31373 .28652 .30944 .28269 Metetra
.63 .778 .952 r
.30944 .28269 .31373 .28652 .32034 .28453 .31607 .2806 Metetra
.629 .772 .949 r
.31607 .2806 .32034 .28453 .32697 .28256 .32272 .27852 Metetra
.629 .767 .946 r
.32272 .27852 .32697 .28256 .33362 .2806 .32939 .27645 Metetra
.629 .762 .942 r
.32939 .27645 .33362 .2806 .34029 .27864 .33607 .27438 Metetra
.629 .756 .939 r
.33607 .27438 .34029 .27864 .34697 .27669 .34278 .27231 Metetra
.628 .75 .935 r
.34278 .27231 .34697 .27669 .35368 .27475 .3495 .27025 Metetra
.629 .745 .931 r
.3495 .27025 .35368 .27475 .3604 .2728 .35623 .26819 Metetra
.629 .74 .927 r
.35623 .26819 .3604 .2728 .36714 .27084 .36299 .26611 Metetra
.63 .734 .923 r
.36299 .26611 .36714 .27084 .37391 .26887 .36977 .26402 Metetra
.631 .73 .919 r
.36977 .26402 .37391 .26887 .38069 .26688 .37657 .26192 Metetra
.631 .725 .915 r
.37657 .26192 .38069 .26688 .38749 .26488 .38338 .2598 Metetra
.632 .72 .911 r
.38338 .2598 .38749 .26488 .39431 .26285 .39022 .25766 Metetra
.633 .716 .907 r
.39022 .25766 .39431 .26285 .40116 .26079 .39707 .2555 Metetra
.634 .712 .903 r
.39707 .2555 .40116 .26079 .40802 .2587 .40395 .25331 Metetra
.636 .709 .899 r
.40395 .25331 .40802 .2587 .4149 .25657 .41085 .25109 Metetra
.637 .706 .896 r
.41085 .25109 .4149 .25657 .42181 .2544 .41776 .24884 Metetra
.638 .703 .893 r
.41776 .24884 .42181 .2544 .42873 .2522 .4247 .24656 Metetra
.639 .701 .89 r
.4247 .24656 .42873 .2522 .43568 .24996 .43165 .24425 Metetra
.641 .699 .888 r
.43165 .24425 .43568 .24996 .44264 .24768 .43863 .2419 Metetra
.642 .697 .886 r
.43863 .2419 .44264 .24768 .44963 .24535 .44562 .23952 Metetra
.643 .696 .883 r
.44562 .23952 .44963 .24535 .45663 .24298 .45264 .23711 Metetra
.645 .695 .882 r
.45264 .23711 .45663 .24298 .46366 .24058 .45967 .23465 Metetra
.646 .694 .88 r
.45967 .23465 .46366 .24058 .4707 .23813 .46673 .23217 Metetra
.647 .694 .879 r
.46673 .23217 .4707 .23813 .47777 .23565 .4738 .22966 Metetra
.648 .693 .878 r
.4738 .22966 .47777 .23565 .48485 .23312 .48089 .22712 Metetra
.65 .694 .878 r
.48089 .22712 .48485 .23312 .49195 .23056 .488 .22454 Metetra
.651 .694 .877 r
.488 .22454 .49195 .23056 .49907 .22796 .49513 .22194 Metetra
.652 .695 .877 r
.49513 .22194 .49907 .22796 .50621 .22533 .50228 .2193 Metetra
.653 .696 .877 r
.50228 .2193 .50621 .22533 .51337 .22267 .50945 .21664 Metetra
.654 .697 .877 r
.50945 .21664 .51337 .22267 .52054 .21998 .51663 .21395 Metetra
.655 .698 .878 r
.51663 .21395 .52054 .21998 .52774 .21725 .52383 .21124 Metetra
.656 .699 .878 r
.52383 .21124 .52774 .21725 .53495 .21451 .53106 .20851 Metetra
.657 .701 .879 r
.53106 .20851 .53495 .21451 .54218 .21174 .5383 .20575 Metetra
.658 .702 .88 r
.5383 .20575 .54218 .21174 .54943 .20894 .54556 .20298 Metetra
.659 .704 .881 r
.54556 .20298 .54943 .20894 .5567 .20612 .55284 .20018 Metetra
.659 .706 .882 r
.55284 .20018 .5567 .20612 .56398 .20329 .56013 .19737 Metetra
.66 .708 .883 r
.56013 .19737 .56398 .20329 .57129 .20044 .56745 .19455 Metetra
.661 .71 .884 r
.56745 .19455 .57129 .20044 .57861 .19756 .57478 .19171 Metetra
.661 .712 .885 r
.57478 .19171 .57861 .19756 .58595 .19467 .58213 .18885 Metetra
.662 .714 .887 r
.58213 .18885 .58595 .19467 .59331 .19177 .5895 .18598 Metetra
.663 .716 .888 r
.5895 .18598 .59331 .19177 .60068 .18885 .59689 .18309 Metetra
.663 .718 .89 r
.59689 .18309 .60068 .18885 .60808 .18592 .6043 .1802 Metetra
.664 .721 .891 r
.6043 .1802 .60808 .18592 .61549 .18298 .61172 .17729 Metetra
.664 .723 .893 r
.61172 .17729 .61549 .18298 .62292 .18002 .61917 .17437 Metetra
.665 .726 .895 r
.61917 .17437 .62292 .18002 .63037 .17706 .62663 .17144 Metetra
.665 .728 .896 r
.62663 .17144 .63037 .17706 .63784 .17408 .63412 .1685 Metetra
.665 .73 .898 r
.63412 .1685 .63784 .17408 .64533 .17109 .64162 .16555 Metetra
.666 .733 .9 r
.64162 .16555 .64533 .17109 .65283 .1681 .64914 .16259 Metetra
.666 .735 .902 r
.64914 .16259 .65283 .1681 .66036 .16509 .65668 .15962 Metetra
.666 .738 .903 r
.65668 .15962 .66036 .16509 .6679 .16207 .66424 .15665 Metetra
.667 .74 .905 r
.66424 .15665 .6679 .16207 .67546 .15905 .67181 .15366 Metetra
.667 .743 .907 r
.67181 .15366 .67546 .15905 .68304 .15601 .67941 .15067 Metetra
.667 .746 .909 r
.67941 .15067 .68304 .15601 .69064 .15297 .68703 .14767 Metetra
.667 .748 .911 r
.68703 .14767 .69064 .15297 .69826 .14992 .69466 .14465 Metetra
.668 .751 .913 r
.69466 .14465 .69826 .14992 .7059 .14686 .70232 .14164 Metetra
.668 .754 .915 r
.70232 .14164 .7059 .14686 .71356 .14379 .70999 .13861 Metetra
.666 .754 .916 r
.03331 .38034 .03795 .38439 .04397 .38199 .03933 .37798 Metetra
.666 .756 .918 r
.03933 .37798 .04397 .38199 .05001 .37959 .04537 .37562 Metetra
.666 .759 .92 r
.04537 .37562 .05001 .37959 .05606 .37718 .05142 .37326 Metetra
.666 .762 .921 r
.05142 .37326 .05606 .37718 .06213 .37477 .05749 .37089 Metetra
.666 .765 .923 r
.05749 .37089 .06213 .37477 .06821 .37235 .06357 .36851 Metetra
.666 .767 .925 r
.06357 .36851 .06821 .37235 .0743 .36993 .06966 .36613 Metetra
.667 .77 .927 r
.06966 .36613 .0743 .36993 .0804 .3675 .07577 .36374 Metetra
.666 .773 .929 r
.07577 .36374 .0804 .3675 .08652 .36507 .08189 .36135 Metetra
.666 .775 .931 r
.08189 .36135 .08652 .36507 .09266 .36264 .08803 .35896 Metetra
.666 .778 .933 r
.08803 .35896 .09266 .36264 .0988 .3602 .09418 .35656 Metetra
.666 .781 .934 r
.09418 .35656 .0988 .3602 .10497 .35776 .10034 .35416 Metetra
.666 .783 .936 r
.10034 .35416 .10497 .35776 .11114 .35531 .10652 .35175 Metetra
.666 .786 .938 r
.10652 .35175 .11114 .35531 .11733 .35286 .11271 .34934 Metetra
.666 .788 .94 r
.11271 .34934 .11733 .35286 .12353 .35042 .11892 .34693 Metetra
.665 .791 .942 r
.11892 .34693 .12353 .35042 .12975 .34796 .12514 .34451 Metetra
.665 .793 .943 r
.12514 .34451 .12975 .34796 .13598 .34551 .13137 .3421 Metetra
.664 .796 .945 r
.13137 .3421 .13598 .34551 .14222 .34306 .13762 .33968 Metetra
.664 .798 .947 r
.13762 .33968 .14222 .34306 .14848 .34061 .14388 .33726 Metetra
.663 .8 .948 r
.14388 .33726 .14848 .34061 .15475 .33816 .15016 .33484 Metetra
.663 .803 .95 r
.15016 .33484 .15475 .33816 .16104 .33571 .15645 .33242 Metetra
.662 .804 .951 r
.15645 .33242 .16104 .33571 .16734 .33327 .16276 .33 Metetra
.661 .806 .953 r
.16276 .33 .16734 .33327 .17365 .33083 .16908 .32759 Metetra
.66 .808 .954 r
.16908 .32759 .17365 .33083 .17997 .32839 .17541 .32517 Metetra
.659 .81 .955 r
.17541 .32517 .17997 .32839 .18631 .32596 .18176 .32276 Metetra
.658 .811 .957 r
.18176 .32276 .18631 .32596 .19267 .32354 .18812 .32036 Metetra
.657 .812 .958 r
.18812 .32036 .19267 .32354 .19904 .32113 .1945 .31796 Metetra
.656 .813 .959 r
.1945 .31796 .19904 .32113 .20542 .31873 .20089 .31556 Metetra
.655 .814 .96 r
.20089 .31556 .20542 .31873 .21181 .31634 .2073 .31318 Metetra
.653 .814 .961 r
.2073 .31318 .21181 .31634 .21822 .31397 .21372 .3108 Metetra
.652 .814 .961 r
.21372 .3108 .21822 .31397 .22464 .31161 .22015 .30843 Metetra
.651 .814 .962 r
.22015 .30843 .22464 .31161 .23108 .30927 .2266 .30608 Metetra
.649 .813 .962 r
.2266 .30608 .23108 .30927 .23753 .30694 .23306 .30373 Metetra
.648 .812 .962 r
.23306 .30373 .23753 .30694 .24399 .30464 .23954 .3014 Metetra
.647 .811 .962 r
.23954 .3014 .24399 .30464 .25047 .30235 .24603 .29909 Metetra
.645 .81 .962 r
.24603 .29909 .25047 .30235 .25696 .30009 .25253 .29678 Metetra
.644 .808 .962 r
.25253 .29678 .25696 .30009 .26347 .29784 .25906 .29449 Metetra
.643 .805 .961 r
.25906 .29449 .26347 .29784 .26999 .29561 .26559 .29221 Metetra
.641 .803 .96 r
.26559 .29221 .26999 .29561 .27653 .29341 .27214 .28995 Metetra
.641 .799 .959 r
.27214 .28995 .27653 .29341 .28308 .29123 .27871 .2877 Metetra
.639 .796 .958 r
.27871 .2877 .28308 .29123 .28965 .28907 .2853 .28546 Metetra
.639 .792 .956 r
.2853 .28546 .28965 .28907 .29623 .28693 .29189 .28324 Metetra
.638 .788 .954 r
.29189 .28324 .29623 .28693 .30283 .2848 .29851 .28103 Metetra
.637 .784 .952 r
.29851 .28103 .30283 .2848 .30944 .28269 .30514 .27882 Metetra
.637 .779 .949 r
.30514 .27882 .30944 .28269 .31607 .2806 .31179 .27663 Metetra
.636 .774 .946 r
.31179 .27663 .31607 .2806 .32272 .27852 .31846 .27445 Metetra
.636 .769 .943 r
.31846 .27445 .32272 .27852 .32939 .27645 .32514 .27227 Metetra
.636 .764 .94 r
.32514 .27227 .32939 .27645 .33607 .27438 .33184 .27009 Metetra
.636 .758 .936 r
.33184 .27009 .33607 .27438 .34278 .27231 .33856 .26791 Metetra
.636 .753 .933 r
.33856 .26791 .34278 .27231 .3495 .27025 .34529 .26573 Metetra
.636 .747 .929 r
.34529 .26573 .3495 .27025 .35623 .26819 .35205 .26355 Metetra
.636 .742 .925 r
.35205 .26355 .35623 .26819 .36299 .26611 .35882 .26136 Metetra
.637 .737 .921 r
.35882 .26136 .36299 .26611 .36977 .26402 .36561 .25916 Metetra
.637 .732 .917 r
.36561 .25916 .36977 .26402 .37657 .26192 .37243 .25694 Metetra
.638 .728 .913 r
.37243 .25694 .37657 .26192 .38338 .2598 .37926 .25471 Metetra
.638 .723 .909 r
.37926 .25471 .38338 .2598 .39022 .25766 .38611 .25247 Metetra
.639 .719 .906 r
.38611 .25247 .39022 .25766 .39707 .2555 .39298 .25021 Metetra
.639 .715 .902 r
.39298 .25021 .39707 .2555 .40395 .25331 .39986 .24792 Metetra
.64 .712 .899 r
.39986 .24792 .40395 .25331 .41085 .25109 .40677 .24561 Metetra
.641 .708 .896 r
.40677 .24561 .41085 .25109 .41776 .24884 .4137 .24328 Metetra
.642 .706 .893 r
.4137 .24328 .41776 .24884 .4247 .24656 .42065 .24092 Metetra
.643 .703 .89 r
.42065 .24092 .4247 .24656 .43165 .24425 .42761 .23854 Metetra
.644 .701 .888 r
.42761 .23854 .43165 .24425 .43863 .2419 .4346 .23613 Metetra
.644 .699 .886 r
.4346 .23613 .43863 .2419 .44562 .23952 .4416 .2337 Metetra
.646 .698 .884 r
.4416 .2337 .44562 .23952 .45264 .23711 .44863 .23123 Metetra
.646 .696 .882 r
.44863 .23123 .45264 .23711 .45967 .23465 .45567 .22874 Metetra
.647 .696 .881 r
.45567 .22874 .45967 .23465 .46673 .23217 .46273 .22622 Metetra
.648 .695 .88 r
.46273 .22622 .46673 .23217 .4738 .22966 .46982 .22368 Metetra
.649 .695 .879 r
.46982 .22368 .4738 .22966 .48089 .22712 .47692 .22111 Metetra
.65 .695 .878 r
.47692 .22111 .48089 .22712 .488 .22454 .48404 .21852 Metetra
.651 .695 .878 r
.48404 .21852 .488 .22454 .49513 .22194 .49117 .2159 Metetra
.652 .695 .878 r
.49117 .2159 .49513 .22194 .50228 .2193 .49833 .21327 Metetra
.653 .696 .878 r
.49833 .21327 .50228 .2193 .50945 .21664 .50551 .2106 Metetra
.653 .697 .878 r
.50551 .2106 .50945 .21664 .51663 .21395 .5127 .20792 Metetra
.654 .698 .878 r
.5127 .20792 .51663 .21395 .52383 .21124 .51992 .20521 Metetra
.655 .699 .879 r
.51992 .20521 .52383 .21124 .53106 .20851 .52715 .20249 Metetra
.655 .701 .88 r
.52715 .20249 .53106 .20851 .5383 .20575 .5344 .19976 Metetra
.656 .702 .881 r
.5344 .19976 .5383 .20575 .54556 .20298 .54167 .197 Metetra
.657 .704 .881 r
.54167 .197 .54556 .20298 .55284 .20018 .54896 .19423 Metetra
.657 .705 .883 r
.54896 .19423 .55284 .20018 .56013 .19737 .55627 .19144 Metetra
.658 .707 .884 r
.55627 .19144 .56013 .19737 .56745 .19455 .56359 .18864 Metetra
.659 .709 .885 r
.56359 .18864 .56745 .19455 .57478 .19171 .57094 .18583 Metetra
.659 .711 .886 r
.57094 .18583 .57478 .19171 .58213 .18885 .5783 .183 Metetra
.66 .713 .888 r
.5783 .183 .58213 .18885 .5895 .18598 .58568 .18016 Metetra
.66 .715 .889 r
.58568 .18016 .5895 .18598 .59689 .18309 .59308 .17731 Metetra
.661 .718 .891 r
.59308 .17731 .59689 .18309 .6043 .1802 .6005 .17445 Metetra
.661 .72 .892 r
.6005 .17445 .6043 .1802 .61172 .17729 .60794 .17158 Metetra
.662 .722 .894 r
.60794 .17158 .61172 .17729 .61917 .17437 .6154 .16869 Metetra
.662 .724 .896 r
.6154 .16869 .61917 .17437 .62663 .17144 .62288 .1658 Metetra
.662 .727 .897 r
.62288 .1658 .62663 .17144 .63412 .1685 .63037 .1629 Metetra
.663 .729 .899 r
.63037 .1629 .63412 .1685 .64162 .16555 .63789 .15999 Metetra
.663 .732 .901 r
.63789 .15999 .64162 .16555 .64914 .16259 .64543 .15707 Metetra
.663 .734 .903 r
.64543 .15707 .64914 .16259 .65668 .15962 .65298 .15414 Metetra
.664 .737 .904 r
.65298 .15414 .65668 .15962 .66424 .15665 .66055 .1512 Metetra
.664 .739 .906 r
.66055 .1512 .66424 .15665 .67181 .15366 .66815 .14825 Metetra
.664 .742 .908 r
.66815 .14825 .67181 .15366 .67941 .15067 .67576 .1453 Metetra
.664 .745 .91 r
.67576 .1453 .67941 .15067 .68703 .14767 .68339 .14233 Metetra
.664 .747 .912 r
.68339 .14233 .68703 .14767 .69466 .14465 .69105 .13936 Metetra
.665 .75 .914 r
.69105 .13936 .69466 .14465 .70232 .14164 .69872 .13639 Metetra
.665 .753 .916 r
.69872 .13639 .70232 .14164 .70999 .13861 .70641 .1334 Metetra
.663 .753 .917 r
.02865 .37627 .03331 .38034 .03933 .37798 .03467 .37396 Metetra
.663 .756 .919 r
.03467 .37396 .03933 .37798 .04537 .37562 .04071 .37164 Metetra
.663 .758 .921 r
.04071 .37164 .04537 .37562 .05142 .37326 .04676 .36931 Metetra
.663 .761 .923 r
.04676 .36931 .05142 .37326 .05749 .37089 .05283 .36698 Metetra
.663 .764 .925 r
.05283 .36698 .05749 .37089 .06357 .36851 .05891 .36465 Metetra
.663 .766 .927 r
.05891 .36465 .06357 .36851 .06966 .36613 .065 .36231 Metetra
.663 .769 .928 r
.065 .36231 .06966 .36613 .07577 .36374 .07111 .35996 Metetra
.663 .772 .93 r
.07111 .35996 .07577 .36374 .08189 .36135 .07724 .35761 Metetra
.663 .774 .932 r
.07724 .35761 .08189 .36135 .08803 .35896 .08337 .35526 Metetra
.663 .777 .934 r
.08337 .35526 .08803 .35896 .09418 .35656 .08953 .3529 Metetra
.663 .78 .936 r
.08953 .3529 .09418 .35656 .10034 .35416 .09569 .35054 Metetra
.663 .782 .938 r
.09569 .35054 .10034 .35416 .10652 .35175 .10187 .34817 Metetra
.662 .785 .939 r
.10187 .34817 .10652 .35175 .11271 .34934 .10807 .3458 Metetra
.662 .788 .941 r
.10807 .3458 .11271 .34934 .11892 .34693 .11428 .34342 Metetra
.662 .79 .943 r
.11428 .34342 .11892 .34693 .12514 .34451 .12051 .34104 Metetra
.661 .792 .945 r
.12051 .34104 .12514 .34451 .13137 .3421 .12674 .33866 Metetra
.661 .795 .946 r
.12674 .33866 .13137 .3421 .13762 .33968 .133 .33628 Metetra
.661 .797 .948 r
.133 .33628 .13762 .33968 .14388 .33726 .13927 .33389 Metetra
.66 .799 .949 r
.13927 .33389 .14388 .33726 .15016 .33484 .14555 .3315 Metetra
.66 .801 .951 r
.14555 .3315 .15016 .33484 .15645 .33242 .15185 .32911 Metetra
.659 .804 .952 r
.15185 .32911 .15645 .33242 .16276 .33 .15816 .32671 Metetra
.658 .805 .953 r
.15816 .32671 .16276 .33 .16908 .32759 .16449 .32432 Metetra
.658 .807 .955 r
.16449 .32432 .16908 .32759 .17541 .32517 .17083 .32193 Metetra
.657 .809 .956 r
.17083 .32193 .17541 .32517 .18176 .32276 .17718 .31954 Metetra
.657 .81 .957 r
.17718 .31954 .18176 .32276 .18812 .32036 .18355 .31714 Metetra
.656 .811 .958 r
.18355 .31714 .18812 .32036 .1945 .31796 .18994 .31475 Metetra
.655 .812 .959 r
.18994 .31475 .1945 .31796 .20089 .31556 .19634 .31236 Metetra
.654 .813 .96 r
.19634 .31236 .20089 .31556 .2073 .31318 .20276 .30998 Metetra
.653 .813 .96 r
.20276 .30998 .2073 .31318 .21372 .3108 .20919 .3076 Metetra
.652 .813 .961 r
.20919 .3076 .21372 .3108 .22015 .30843 .21563 .30522 Metetra
.651 .813 .961 r
.21563 .30522 .22015 .30843 .2266 .30608 .22209 .30285 Metetra
.65 .813 .961 r
.22209 .30285 .2266 .30608 .23306 .30373 .22857 .30049 Metetra
.649 .812 .961 r
.22857 .30049 .23306 .30373 .23954 .3014 .23506 .29813 Metetra
.649 .811 .961 r
.23506 .29813 .23954 .3014 .24603 .29909 .24156 .29578 Metetra
.648 .809 .961 r
.24156 .29578 .24603 .29909 .25253 .29678 .24808 .29343 Metetra
.647 .807 .96 r
.24808 .29343 .25253 .29678 .25906 .29449 .25462 .29109 Metetra
.646 .805 .959 r
.25462 .29109 .25906 .29449 .26559 .29221 .26117 .28877 Metetra
.646 .803 .958 r
.26117 .28877 .26559 .29221 .27214 .28995 .26774 .28644 Metetra
.645 .8 .957 r
.26774 .28644 .27214 .28995 .27871 .2877 .27432 .28413 Metetra
.645 .796 .955 r
.27432 .28413 .27871 .2877 .2853 .28546 .28092 .28182 Metetra
.644 .793 .954 r
.28092 .28182 .2853 .28546 .29189 .28324 .28754 .27951 Metetra
.644 .789 .951 r
.28754 .27951 .29189 .28324 .29851 .28103 .29417 .27721 Metetra
.643 .784 .949 r
.29417 .27721 .29851 .28103 .30514 .27882 .30082 .27492 Metetra
.643 .78 .947 r
.30082 .27492 .30514 .27882 .31179 .27663 .30748 .27263 Metetra
.643 .775 .944 r
.30748 .27263 .31179 .27663 .31846 .27445 .31417 .27035 Metetra
.643 .77 .941 r
.31417 .27035 .31846 .27445 .32514 .27227 .32087 .26806 Metetra
.643 .765 .937 r
.32087 .26806 .32514 .27227 .33184 .27009 .32758 .26577 Metetra
.643 .76 .934 r
.32758 .26577 .33184 .27009 .33856 .26791 .33432 .26348 Metetra
.643 .755 .93 r
.33432 .26348 .33856 .26791 .34529 .26573 .34107 .26119 Metetra
.643 .75 .927 r
.34107 .26119 .34529 .26573 .35205 .26355 .34784 .2589 Metetra
.643 .745 .923 r
.34784 .2589 .35205 .26355 .35882 .26136 .35463 .25659 Metetra
.643 .739 .919 r
.35463 .25659 .35882 .26136 .36561 .25916 .36144 .25428 Metetra
.643 .735 .915 r
.36144 .25428 .36561 .25916 .37243 .25694 .36827 .25195 Metetra
.644 .73 .912 r
.36827 .25195 .37243 .25694 .37926 .25471 .37511 .24962 Metetra
.644 .726 .908 r
.37511 .24962 .37926 .25471 .38611 .25247 .38197 .24727 Metetra
.644 .721 .904 r
.38197 .24727 .38611 .25247 .39298 .25021 .38886 .24491 Metetra
.644 .718 .901 r
.38886 .24491 .39298 .25021 .39986 .24792 .39576 .24253 Metetra
.645 .714 .898 r
.39576 .24253 .39986 .24792 .40677 .24561 .40268 .24013 Metetra
.645 .711 .895 r
.40268 .24013 .40677 .24561 .4137 .24328 .40962 .23772 Metetra
.646 .708 .892 r
.40962 .23772 .4137 .24328 .42065 .24092 .41658 .23528 Metetra
.646 .705 .89 r
.41658 .23528 .42065 .24092 .42761 .23854 .42355 .23283 Metetra
.646 .703 .888 r
.42355 .23283 .42761 .23854 .4346 .23613 .43055 .23036 Metetra
.647 .701 .886 r
.43055 .23036 .4346 .23613 .4416 .2337 .43756 .22787 Metetra
.647 .699 .884 r
.43756 .22787 .4416 .2337 .44863 .23123 .4446 .22535 Metetra
.648 .698 .882 r
.4446 .22535 .44863 .23123 .45567 .22874 .45165 .22282 Metetra
.648 .697 .881 r
.45165 .22282 .45567 .22874 .46273 .22622 .45872 .22026 Metetra
.649 .696 .88 r
.45872 .22026 .46273 .22622 .46982 .22368 .46582 .21769 Metetra
.649 .696 .88 r
.46582 .21769 .46982 .22368 .47692 .22111 .47293 .2151 Metetra
.65 .696 .879 r
.47293 .2151 .47692 .22111 .48404 .21852 .48005 .21249 Metetra
.651 .696 .879 r
.48005 .21249 .48404 .21852 .49117 .2159 .4872 .20987 Metetra
.651 .696 .879 r
.4872 .20987 .49117 .2159 .49833 .21327 .49437 .20722 Metetra
.652 .696 .879 r
.49437 .20722 .49833 .21327 .50551 .2106 .50155 .20456 Metetra
.653 .697 .879 r
.50155 .20456 .50551 .2106 .5127 .20792 .50876 .20187 Metetra
.653 .698 .879 r
.50876 .20187 .5127 .20792 .51992 .20521 .51598 .19918 Metetra
.653 .699 .88 r
.51598 .19918 .51992 .20521 .52715 .20249 .52322 .19647 Metetra
.654 .7 .881 r
.52322 .19647 .52715 .20249 .5344 .19976 .53049 .19374 Metetra
.655 .702 .881 r
.53049 .19374 .5344 .19976 .54167 .197 .53777 .19101 Metetra
.655 .703 .882 r
.53777 .19101 .54167 .197 .54896 .19423 .54506 .18825 Metetra
.655 .705 .883 r
.54506 .18825 .54896 .19423 .55627 .19144 .55238 .18549 Metetra
.656 .707 .885 r
.55238 .18549 .55627 .19144 .56359 .18864 .55972 .18272 Metetra
.656 .708 .886 r
.55972 .18272 .56359 .18864 .57094 .18583 .56707 .17993 Metetra
.657 .71 .887 r
.56707 .17993 .57094 .18583 .5783 .183 .57445 .17713 Metetra
.657 .712 .889 r
.57445 .17713 .5783 .183 .58568 .18016 .58184 .17433 Metetra
.658 .715 .89 r
.58184 .17433 .58568 .18016 .59308 .17731 .58926 .17151 Metetra
.658 .717 .892 r
.58926 .17151 .59308 .17731 .6005 .17445 .59669 .16868 Metetra
.658 .719 .893 r
.59669 .16868 .6005 .17445 .60794 .17158 .60414 .16584 Metetra
.659 .721 .895 r
.60414 .16584 .60794 .17158 .6154 .16869 .61161 .16299 Metetra
.659 .723 .897 r
.61161 .16299 .6154 .16869 .62288 .1658 .61911 .16014 Metetra
.66 .726 .898 r
.61911 .16014 .62288 .1658 .63037 .1629 .62662 .15727 Metetra
.66 .728 .9 r
.62662 .15727 .63037 .1629 .63789 .15999 .63415 .1544 Metetra
.66 .731 .902 r
.63415 .1544 .63789 .15999 .64543 .15707 .6417 .15152 Metetra
.66 .733 .904 r
.6417 .15152 .64543 .15707 .65298 .15414 .64927 .14863 Metetra
.661 .736 .905 r
.64927 .14863 .65298 .15414 .66055 .1512 .65686 .14573 Metetra
.661 .738 .907 r
.65686 .14573 .66055 .1512 .66815 .14825 .66446 .14282 Metetra
.661 .741 .909 r
.66446 .14282 .66815 .14825 .67576 .1453 .67209 .13991 Metetra
.661 .744 .911 r
.67209 .13991 .67576 .1453 .68339 .14233 .67974 .13698 Metetra
.661 .746 .913 r
.67974 .13698 .68339 .14233 .69105 .13936 .68741 .13405 Metetra
.662 .749 .915 r
.68741 .13405 .69105 .13936 .69872 .13639 .6951 .13111 Metetra
.662 .751 .917 r
.6951 .13111 .69872 .13639 .70641 .1334 .70282 .12816 Metetra
.659 .752 .919 r
.02396 .37218 .02865 .37627 .03467 .37396 .02999 .36991 Metetra
.659 .755 .921 r
.02999 .36991 .03467 .37396 .04071 .37164 .03603 .36763 Metetra
.659 .758 .923 r
.03603 .36763 .04071 .37164 .04676 .36931 .04208 .36535 Metetra
.66 .76 .924 r
.04208 .36535 .04676 .36931 .05283 .36698 .04815 .36306 Metetra
.659 .763 .926 r
.04815 .36306 .05283 .36698 .05891 .36465 .05423 .36077 Metetra
.66 .766 .928 r
.05423 .36077 .05891 .36465 .065 .36231 .06033 .35847 Metetra
.66 .768 .93 r
.06033 .35847 .065 .36231 .07111 .35996 .06644 .35617 Metetra
.659 .771 .932 r
.06644 .35617 .07111 .35996 .07724 .35761 .07256 .35386 Metetra
.659 .774 .934 r
.07256 .35386 .07724 .35761 .08337 .35526 .0787 .35154 Metetra
.659 .776 .935 r
.0787 .35154 .08337 .35526 .08953 .3529 .08486 .34922 Metetra
.659 .779 .937 r
.08486 .34922 .08953 .3529 .09569 .35054 .09102 .3469 Metetra
.659 .781 .939 r
.09102 .3469 .09569 .35054 .10187 .34817 .09721 .34457 Metetra
.659 .784 .941 r
.09721 .34457 .10187 .34817 .10807 .3458 .10341 .34223 Metetra
.659 .787 .942 r
.10341 .34223 .10807 .3458 .11428 .34342 .10962 .33989 Metetra
.658 .789 .944 r
.10962 .33989 .11428 .34342 .12051 .34104 .11585 .33755 Metetra
.658 .792 .946 r
.11585 .33755 .12051 .34104 .12674 .33866 .12209 .3352 Metetra
.658 .794 .947 r
.12209 .3352 .12674 .33866 .133 .33628 .12835 .33285 Metetra
.658 .796 .949 r
.12835 .33285 .133 .33628 .13927 .33389 .13463 .33049 Metetra
.657 .798 .95 r
.13463 .33049 .13927 .33389 .14555 .3315 .14091 .32813 Metetra
.657 .801 .952 r
.14091 .32813 .14555 .3315 .15185 .32911 .14722 .32577 Metetra
.656 .802 .953 r
.14722 .32577 .15185 .32911 .15816 .32671 .15354 .3234 Metetra
.656 .804 .954 r
.15354 .3234 .15816 .32671 .16449 .32432 .15987 .32103 Metetra
.656 .806 .955 r
.15987 .32103 .16449 .32432 .17083 .32193 .16622 .31866 Metetra
.655 .808 .956 r
.16622 .31866 .17083 .32193 .17718 .31954 .17258 .31628 Metetra
.655 .809 .957 r
.17258 .31628 .17718 .31954 .18355 .31714 .17896 .3139 Metetra
.654 .81 .958 r
.17896 .3139 .18355 .31714 .18994 .31475 .18536 .31152 Metetra
.654 .811 .959 r
.18536 .31152 .18994 .31475 .19634 .31236 .19177 .30914 Metetra
.653 .812 .96 r
.19177 .30914 .19634 .31236 .20276 .30998 .19819 .30675 Metetra
.653 .813 .96 r
.19819 .30675 .20276 .30998 .20919 .3076 .20464 .30437 Metetra
.652 .813 .96 r
.20464 .30437 .20919 .3076 .21563 .30522 .21109 .30198 Metetra
.652 .813 .961 r
.21109 .30198 .21563 .30522 .22209 .30285 .21756 .2996 Metetra
.651 .812 .961 r
.21756 .2996 .22209 .30285 .22857 .30049 .22405 .29721 Metetra
.651 .812 .96 r
.22405 .29721 .22857 .30049 .23506 .29813 .23056 .29483 Metetra
.651 .81 .96 r
.23056 .29483 .23506 .29813 .24156 .29578 .23707 .29244 Metetra
.65 .809 .959 r
.23707 .29244 .24156 .29578 .24808 .29343 .24361 .29006 Metetra
.65 .807 .959 r
.24361 .29006 .24808 .29343 .25462 .29109 .25016 .28767 Metetra
.65 .805 .958 r
.25016 .28767 .25462 .29109 .26117 .28877 .25673 .28529 Metetra
.65 .803 .957 r
.25673 .28529 .26117 .28877 .26774 .28644 .26331 .28291 Metetra
.65 .8 .955 r
.26331 .28291 .26774 .28644 .27432 .28413 .26991 .28052 Metetra
.649 .797 .953 r
.26991 .28052 .27432 .28413 .28092 .28182 .27653 .27814 Metetra
.65 .794 .951 r
.27653 .27814 .28092 .28182 .28754 .27951 .28316 .27576 Metetra
.649 .79 .949 r
.28316 .27576 .28754 .27951 .29417 .27721 .28981 .27337 Metetra
.649 .786 .947 r
.28981 .27337 .29417 .27721 .30082 .27492 .29647 .27099 Metetra
.649 .781 .944 r
.29647 .27099 .30082 .27492 .30748 .27263 .30316 .2686 Metetra
.649 .777 .941 r
.30316 .2686 .30748 .27263 .31417 .27035 .30986 .26622 Metetra
.65 .772 .938 r
.30986 .26622 .31417 .27035 .32087 .26806 .31657 .26383 Metetra
.65 .767 .935 r
.31657 .26383 .32087 .26806 .32758 .26577 .32331 .26143 Metetra
.65 .762 .931 r
.32331 .26143 .32758 .26577 .33432 .26348 .33006 .25903 Metetra
.65 .757 .928 r
.33006 .25903 .33432 .26348 .34107 .26119 .33683 .25663 Metetra
.649 .752 .924 r
.33683 .25663 .34107 .26119 .34784 .2589 .34362 .25422 Metetra
.649 .747 .921 r
.34362 .25422 .34784 .2589 .35463 .25659 .35042 .25181 Metetra
.65 .742 .917 r
.35042 .25181 .35463 .25659 .36144 .25428 .35725 .24938 Metetra
.649 .737 .913 r
.35725 .24938 .36144 .25428 .36827 .25195 .36409 .24695 Metetra
.649 .732 .91 r
.36409 .24695 .36827 .25195 .37511 .24962 .37095 .24451 Metetra
.649 .728 .906 r
.37095 .24451 .37511 .24962 .38197 .24727 .37782 .24206 Metetra
.649 .724 .903 r
.37782 .24206 .38197 .24727 .38886 .24491 .38472 .2396 Metetra
.649 .72 .9 r
.38472 .2396 .38886 .24491 .39576 .24253 .39163 .23713 Metetra
.649 .716 .897 r
.39163 .23713 .39576 .24253 .40268 .24013 .39857 .23464 Metetra
.649 .713 .894 r
.39857 .23464 .40268 .24013 .40962 .23772 .40552 .23214 Metetra
.649 .71 .892 r
.40552 .23214 .40962 .23772 .41658 .23528 .41249 .22963 Metetra
.649 .707 .889 r
.41249 .22963 .41658 .23528 .42355 .23283 .41948 .22711 Metetra
.649 .705 .887 r
.41948 .22711 .42355 .23283 .43055 .23036 .42648 .22457 Metetra
.649 .703 .885 r
.42648 .22457 .43055 .23036 .43756 .22787 .43351 .22203 Metetra
.65 .701 .884 r
.43351 .22203 .43756 .22787 .4446 .22535 .44055 .21946 Metetra
.65 .699 .882 r
.44055 .21946 .4446 .22535 .45165 .22282 .44762 .21689 Metetra
.65 .698 .881 r
.44762 .21689 .45165 .22282 .45872 .22026 .4547 .2143 Metetra
.65 .697 .88 r
.4547 .2143 .45872 .22026 .46582 .21769 .4618 .21169 Metetra
.65 .697 .88 r
.4618 .21169 .46582 .21769 .47293 .2151 .46892 .20908 Metetra
.65 .696 .879 r
.46892 .20908 .47293 .2151 .48005 .21249 .47606 .20645 Metetra
.651 .696 .879 r
.47606 .20645 .48005 .21249 .4872 .20987 .48321 .20381 Metetra
.651 .696 .879 r
.48321 .20381 .4872 .20987 .49437 .20722 .49039 .20116 Metetra
.651 .697 .879 r
.49039 .20116 .49437 .20722 .50155 .20456 .49759 .19849 Metetra
.652 .697 .879 r
.49759 .19849 .50155 .20456 .50876 .20187 .5048 .19581 Metetra
.652 .698 .88 r
.5048 .19581 .50876 .20187 .51598 .19918 .51203 .19312 Metetra
.652 .699 .88 r
.51203 .19312 .51598 .19918 .52322 .19647 .51928 .19042 Metetra
.653 .7 .881 r
.51928 .19042 .52322 .19647 .53049 .19374 .52655 .18771 Metetra
.653 .701 .882 r
.52655 .18771 .53049 .19374 .53777 .19101 .53384 .18499 Metetra
.653 .703 .883 r
.53384 .18499 .53777 .19101 .54506 .18825 .54115 .18226 Metetra
.654 .704 .884 r
.54115 .18226 .54506 .18825 .55238 .18549 .54848 .17952 Metetra
.654 .706 .885 r
.54848 .17952 .55238 .18549 .55972 .18272 .55583 .17677 Metetra
.654 .708 .887 r
.55583 .17677 .55972 .18272 .56707 .17993 .5632 .17401 Metetra
.655 .71 .888 r
.5632 .17401 .56707 .17993 .57445 .17713 .57058 .17125 Metetra
.655 .712 .889 r
.57058 .17125 .57445 .17713 .58184 .17433 .57799 .16847 Metetra
.655 .714 .891 r
.57799 .16847 .58184 .17433 .58926 .17151 .58542 .16568 Metetra
.656 .716 .893 r
.58542 .16568 .58926 .17151 .59669 .16868 .59286 .16289 Metetra
.656 .718 .894 r
.59286 .16289 .59669 .16868 .60414 .16584 .60033 .16008 Metetra
.656 .72 .896 r
.60033 .16008 .60414 .16584 .61161 .16299 .60781 .15727 Metetra
.657 .722 .897 r
.60781 .15727 .61161 .16299 .61911 .16014 .61532 .15445 Metetra
.657 .725 .899 r
.61532 .15445 .61911 .16014 .62662 .15727 .62284 .15162 Metetra
.657 .727 .901 r
.62284 .15162 .62662 .15727 .63415 .1544 .63038 .14878 Metetra
.657 .73 .903 r
.63038 .14878 .63415 .1544 .6417 .15152 .63795 .14594 Metetra
.657 .732 .904 r
.63795 .14594 .6417 .15152 .64927 .14863 .64553 .14309 Metetra
.658 .734 .906 r
.64553 .14309 .64927 .14863 .65686 .14573 .65314 .14023 Metetra
.658 .737 .908 r
.65314 .14023 .65686 .14573 .66446 .14282 .66076 .13736 Metetra
.658 .74 .91 r
.66076 .13736 .66446 .14282 .67209 .13991 .66841 .13448 Metetra
.658 .742 .912 r
.66841 .13448 .67209 .13991 .67974 .13698 .67608 .1316 Metetra
.659 .745 .914 r
.67608 .1316 .67974 .13698 .68741 .13405 .68376 .12871 Metetra
.659 .747 .916 r
.68376 .12871 .68741 .13405 .6951 .13111 .69147 .12581 Metetra
.659 .75 .917 r
.69147 .12581 .6951 .13111 .70282 .12816 .6992 .1229 Metetra
.656 .751 .92 r
.01926 .36808 .02396 .37218 .02999 .36991 .02529 .36585 Metetra
.656 .754 .922 r
.02529 .36585 .02999 .36991 .03603 .36763 .03132 .36361 Metetra
.656 .757 .924 r
.03132 .36361 .03603 .36763 .04208 .36535 .03738 .36137 Metetra
.656 .759 .926 r
.03738 .36137 .04208 .36535 .04815 .36306 .04344 .35912 Metetra
.656 .762 .928 r
.04344 .35912 .04815 .36306 .05423 .36077 .04953 .35687 Metetra
.656 .765 .93 r
.04953 .35687 .05423 .36077 .06033 .35847 .05562 .35461 Metetra
.656 .767 .931 r
.05562 .35461 .06033 .35847 .06644 .35617 .06174 .35235 Metetra
.656 .77 .933 r
.06174 .35235 .06644 .35617 .07256 .35386 .06786 .35008 Metetra
.656 .773 .935 r
.06786 .35008 .07256 .35386 .0787 .35154 .074 .34781 Metetra
.656 .775 .937 r
.074 .34781 .0787 .35154 .08486 .34922 .08016 .34553 Metetra
.656 .778 .939 r
.08016 .34553 .08486 .34922 .09102 .3469 .08633 .34324 Metetra
.656 .781 .94 r
.08633 .34324 .09102 .3469 .09721 .34457 .09252 .34095 Metetra
.655 .783 .942 r
.09252 .34095 .09721 .34457 .10341 .34223 .09872 .33865 Metetra
.655 .786 .944 r
.09872 .33865 .10341 .34223 .10962 .33989 .10494 .33635 Metetra
.655 .788 .945 r
.10494 .33635 .10962 .33989 .11585 .33755 .11117 .33404 Metetra
.655 .791 .947 r
.11117 .33404 .11585 .33755 .12209 .3352 .11742 .33173 Metetra
.655 .793 .948 r
.11742 .33173 .12209 .3352 .12835 .33285 .12368 .32941 Metetra
.655 .795 .95 r
.12368 .32941 .12835 .33285 .13463 .33049 .12996 .32708 Metetra
.654 .798 .951 r
.12996 .32708 .13463 .33049 .14091 .32813 .13625 .32475 Metetra
.654 .8 .953 r
.13625 .32475 .14091 .32813 .14722 .32577 .14256 .32241 Metetra
.654 .802 .954 r
.14256 .32241 .14722 .32577 .15354 .3234 .14889 .32007 Metetra
.654 .804 .955 r
.14889 .32007 .15354 .3234 .15987 .32103 .15523 .31772 Metetra
.653 .805 .956 r
.15523 .31772 .15987 .32103 .16622 .31866 .16159 .31537 Metetra
.653 .807 .957 r
.16159 .31537 .16622 .31866 .17258 .31628 .16796 .31301 Metetra
.653 .808 .958 r
.16796 .31301 .17258 .31628 .17896 .3139 .17435 .31064 Metetra
.653 .81 .959 r
.17435 .31064 .17896 .3139 .18536 .31152 .18075 .30827 Metetra
.652 .811 .959 r
.18075 .30827 .18536 .31152 .19177 .30914 .18717 .30589 Metetra
.652 .812 .96 r
.18717 .30589 .19177 .30914 .19819 .30675 .19361 .30351 Metetra
.652 .812 .96 r
.19361 .30351 .19819 .30675 .20464 .30437 .20006 .30112 Metetra
.652 .813 .96 r
.20006 .30112 .20464 .30437 .21109 .30198 .20653 .29873 Metetra
.652 .813 .96 r
.20653 .29873 .21109 .30198 .21756 .2996 .21301 .29632 Metetra
.652 .812 .96 r
.21301 .29632 .21756 .2996 .22405 .29721 .21951 .29392 Metetra
.652 .812 .96 r
.21951 .29392 .22405 .29721 .23056 .29483 .22603 .2915 Metetra
.653 .811 .959 r
.22603 .2915 .23056 .29483 .23707 .29244 .23256 .28908 Metetra
.653 .809 .958 r
.23256 .28908 .23707 .29244 .24361 .29006 .23911 .28666 Metetra
.653 .808 .958 r
.23911 .28666 .24361 .29006 .25016 .28767 .24568 .28423 Metetra
.653 .806 .956 r
.24568 .28423 .25016 .28767 .25673 .28529 .25226 .28179 Metetra
.654 .804 .955 r
.25226 .28179 .25673 .28529 .26331 .28291 .25886 .27934 Metetra
.654 .801 .953 r
.25886 .27934 .26331 .28291 .26991 .28052 .26548 .27689 Metetra
.654 .798 .951 r
.26548 .27689 .26991 .28052 .27653 .27814 .27211 .27444 Metetra
.655 .795 .949 r
.27211 .27444 .27653 .27814 .28316 .27576 .27876 .27197 Metetra
.655 .791 .947 r
.27876 .27197 .28316 .27576 .28981 .27337 .28543 .2695 Metetra
.655 .787 .945 r
.28543 .2695 .28981 .27337 .29647 .27099 .29211 .26703 Metetra
.655 .783 .942 r
.29211 .26703 .29647 .27099 .30316 .2686 .29881 .26455 Metetra
.656 .778 .939 r
.29881 .26455 .30316 .2686 .30986 .26622 .30553 .26206 Metetra
.656 .774 .936 r
.30553 .26206 .30986 .26622 .31657 .26383 .31226 .25957 Metetra
.656 .769 .932 r
.31226 .25957 .31657 .26383 .32331 .26143 .31901 .25706 Metetra
.656 .764 .929 r
.31901 .25706 .32331 .26143 .33006 .25903 .32578 .25456 Metetra
.656 .759 .926 r
.32578 .25456 .33006 .25903 .33683 .25663 .33257 .25204 Metetra
.656 .754 .922 r
.33257 .25204 .33683 .25663 .34362 .25422 .33937 .24952 Metetra
.656 .749 .918 r
.33937 .24952 .34362 .25422 .35042 .25181 .34619 .247 Metetra
.655 .744 .915 r
.34619 .247 .35042 .25181 .35725 .24938 .35303 .24447 Metetra
.655 .739 .912 r
.35303 .24447 .35725 .24938 .36409 .24695 .35989 .24193 Metetra
.655 .735 .908 r
.35989 .24193 .36409 .24695 .37095 .24451 .36676 .23938 Metetra
.655 .73 .905 r
.36676 .23938 .37095 .24451 .37782 .24206 .37366 .23683 Metetra
.654 .726 .902 r
.37366 .23683 .37782 .24206 .38472 .2396 .38057 .23427 Metetra
.654 .722 .899 r
.38057 .23427 .38472 .2396 .39163 .23713 .38749 .2317 Metetra
.653 .718 .896 r
.38749 .2317 .39163 .23713 .39857 .23464 .39444 .22913 Metetra
.653 .715 .893 r
.39444 .22913 .39857 .23464 .40552 .23214 .4014 .22655 Metetra
.653 .711 .891 r
.4014 .22655 .40552 .23214 .41249 .22963 .40838 .22397 Metetra
.652 .709 .889 r
.40838 .22397 .41249 .22963 .41948 .22711 .41538 .22137 Metetra
.652 .706 .887 r
.41538 .22137 .41948 .22711 .42648 .22457 .4224 .21877 Metetra
.652 .704 .885 r
.4224 .21877 .42648 .22457 .43351 .22203 .42944 .21617 Metetra
.652 .702 .883 r
.42944 .21617 .43351 .22203 .44055 .21946 .43649 .21356 Metetra
.651 .7 .882 r
.43649 .21356 .44055 .21946 .44762 .21689 .44357 .21094 Metetra
.651 .699 .881 r
.44357 .21094 .44762 .21689 .4547 .2143 .45066 .20831 Metetra
.651 .698 .88 r
.45066 .20831 .4547 .2143 .4618 .21169 .45777 .20568 Metetra
.651 .697 .88 r
.45777 .20568 .4618 .21169 .46892 .20908 .4649 .20304 Metetra
.651 .697 .879 r
.4649 .20304 .46892 .20908 .47606 .20645 .47204 .20039 Metetra
.651 .696 .879 r
.47204 .20039 .47606 .20645 .48321 .20381 .47921 .19774 Metetra
.651 .696 .879 r
.47921 .19774 .48321 .20381 .49039 .20116 .4864 .19507 Metetra
.651 .697 .879 r
.4864 .19507 .49039 .20116 .49759 .19849 .4936 .19241 Metetra
.651 .697 .88 r
.4936 .19241 .49759 .19849 .5048 .19581 .50082 .18973 Metetra
.651 .698 .88 r
.50082 .18973 .5048 .19581 .51203 .19312 .50807 .18705 Metetra
.651 .699 .881 r
.50807 .18705 .51203 .19312 .51928 .19042 .51533 .18436 Metetra
.651 .7 .882 r
.51533 .18436 .51928 .19042 .52655 .18771 .52261 .18166 Metetra
.651 .701 .883 r
.52261 .18166 .52655 .18771 .53384 .18499 .52991 .17896 Metetra
.652 .702 .884 r
.52991 .17896 .53384 .18499 .54115 .18226 .53723 .17625 Metetra
.652 .704 .885 r
.53723 .17625 .54115 .18226 .54848 .17952 .54457 .17353 Metetra
.652 .705 .886 r
.54457 .17353 .54848 .17952 .55583 .17677 .55193 .1708 Metetra
.652 .707 .887 r
.55193 .1708 .55583 .17677 .5632 .17401 .5593 .16807 Metetra
.652 .709 .889 r
.5593 .16807 .5632 .17401 .57058 .17125 .5667 .16533 Metetra
.653 .711 .89 r
.5667 .16533 .57058 .17125 .57799 .16847 .57412 .16258 Metetra
.653 .713 .892 r
.57412 .16258 .57799 .16847 .58542 .16568 .58156 .15983 Metetra
.653 .715 .893 r
.58156 .15983 .58542 .16568 .59286 .16289 .58901 .15707 Metetra
.653 .717 .895 r
.58901 .15707 .59286 .16289 .60033 .16008 .59649 .1543 Metetra
.654 .719 .897 r
.59649 .1543 .60033 .16008 .60781 .15727 .60399 .15152 Metetra
.654 .721 .898 r
.60399 .15152 .60781 .15727 .61532 .15445 .61151 .14874 Metetra
.654 .724 .9 r
.61151 .14874 .61532 .15445 .62284 .15162 .61905 .14595 Metetra
.654 .726 .902 r
.61905 .14595 .62284 .15162 .63038 .14878 .62661 .14315 Metetra
.654 .728 .904 r
.62661 .14315 .63038 .14878 .63795 .14594 .63418 .14034 Metetra
.655 .731 .905 r
.63418 .14034 .63795 .14594 .64553 .14309 .64178 .13752 Metetra
.655 .733 .907 r
.64178 .13752 .64553 .14309 .65314 .14023 .6494 .1347 Metetra
.655 .736 .909 r
.6494 .1347 .65314 .14023 .66076 .13736 .65705 .13187 Metetra
.655 .738 .911 r
.65705 .13187 .66076 .13736 .66841 .13448 .66471 .12903 Metetra
.655 .741 .913 r
.66471 .12903 .66841 .13448 .67608 .1316 .67239 .12619 Metetra
.655 .743 .915 r
.67239 .12619 .67608 .1316 .68376 .12871 .68009 .12334 Metetra
.656 .746 .916 r
.68009 .12334 .68376 .12871 .69147 .12581 .68782 .12048 Metetra
.656 .749 .918 r
.68782 .12048 .69147 .12581 .6992 .1229 .69556 .11761 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.15032 .21083 m
.15494 .21549 L
s
gsave
.14109 .2015 -76.9256 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
.2751 .16197 m
.2795 .16683 L
s
gsave
.26628 .15225 -76.2517 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
.4061 .11067 m
.41026 .11574 L
s
gsave
.39776 .10054 -75.5778 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(60) show
1.000 setlinewidth
grestore
.5438 .05675 m
.5477 .06203 L
s
gsave
.53601 .04619 -74.9039 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(80) show
1.000 setlinewidth
grestore
.68874 0 m
.69233 .00549 L
s
gsave
.68156 -0.01098 -79.1912 -19.6875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 23.6875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 15.813 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
/Times-Roman findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(100) show
1.000 setlinewidth
grestore
.125 Mabswid
.06057 .24597 m
.06342 .24868 L
s
.09013 .23439 m
.09296 .23713 L
s
.12005 .22268 m
.12285 .22545 L
s
.18096 .19883 m
.1837 .20166 L
s
.21196 .18669 m
.21467 .18955 L
s
.24334 .17441 m
.24601 .17729 L
s
.30724 .14938 m
.30985 .15233 L
s
.33979 .13664 m
.34236 .13962 L
s
.37274 .12374 m
.37527 .12675 L
s
.43987 .09745 m
.44233 .10052 L
s
.47408 .08405 m
.4765 .08716 L
s
.50872 .07049 m
.5111 .07363 L
s
.57933 .04284 m
.58163 .04604 L
s
.61533 .02874 m
.61758 .03198 L
s
.65179 .01447 m
.654 .01773 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{489.875, 397.375},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000;N0002E2000`400?l00000o`00003oo`3ooooo
0?ooon00oooo003o0?ooool0ooooh03oool00?l0ooooo`3ooooP0?ooo`00o`3ooooo0?ooon00oooo
003o0?ooool0ooooh03oool00?l0ooooo`3ooooP0?ooo`00o`3ooooo0?ooon00oooo003o0?ooool0
ooooh03oool00?l0ooooo`3ooooP0?ooo`00o`3ooooo0?ooon00oooo003o0?ooool0ooooh03oool0
0?l0ooooo`3ooooP0?ooo`00o`3ooooJ0?ooo`060;md00000000000000000000B9`0gooo0`3oool0
1@3ogi`0B0000000000007@0_ooo00@0oooo00D0omnL04P000000000001d0;ooo`3^0?ooo`00o`3o
oooK0?ooo`030?oOW0180000M;oo00@0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo0080
oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo0>d0oooo003o0?ooom/0oooo00<0omnL04P0
001d_ol0103oool00`2oM00004RL0=ooo`020?ooo`070;md0000M;l0oooo0?ooo`2oM00004RL0=oo
o`020?ooo`030;md0000M;l0oooo0>`0oooo003o0?ooom/0oooo00<0omnL04P0001d_ol00`3oool0
0`3ookl0M00004RLg`030?ooo`060=nLB00007@0_ooo0?oo_`1d0000B9cO0`3oool00`3OW4P0001d
0;ooo`3/0?ooo`00o`3ooooK0?ooo`030?oOW0180000M;oo00<0oooo00<0omnL04P0001d_ol00`3o
ool01P3o_g@0001809cOo`3ogi`0B00007Boo`<0oooo00<0okmd0000B02Lgol0k03oool00?l0oooo
f`3oool00`3ogi`0B00007Boo`030?ooo`030?oOW0180000M;oo00<0oooo00H0okmd0000B02Lgol0
omnL04P0001d_ol30?ooo`030?noM00004P0W=oo0>`0oooo003o0?ooom/0oooo00<0omnL04P0001d
_ol00`3oool00`3ogi`0B00007Boo`030?ooo`060?noM00004P0W=oo0?oOW0180000M;oo0`3oool0
0`3o_g@0001809cOo`3/0?ooo`00o`3ooooK0?ooo`030?oOW0180000M;oo00<0oooo00<0omnL04P0
001d_ol00`3oool01P3o_g@0001809cOo`3ogi`0B00007Boo`<0oooo00<0okmd0000B02Lgol0k03o
ool00?l0oooof`3oool00`3ogi`0B00007Boo`030?ooo`030?oo_`1d0000B9cO00<0oooo00H0gia8
0000M02oool0oono07@00018W=l30?ooo`030=nLB00007@0_ooo0>`0oooo003o0?ooom/0oooo00<0
omnL04P0001d_ol0103oool00`3OW4P004RL0=ooo`020?ooo`070;md0000M;l0oooo0?ooo`3OW4P0
04RL0=ooo`020?ooo`030;md0000M;l0oooo0>`0oooo003o0?ooomT0oooo00D0oooO09a800000000
000007Boo`040?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`020?ooo`070?oo_`1d0000
M;oo0?ooo`3o_g@0001d0;ooo`3]0?ooo`00o`3ooooK0?ooo`030?noM0000000M;oo00D0oooo00D0
oono07@000000000001d0;ooo`040?ooo`050?oo_`1d000000000000M02oool0kP3oool00?l0oooo
o`3ooooP0?ooo`00o`3ooooo0?ooon00oooo003o0?ooool0ooooh03oool00?l0ooooo`3ooooP0?oo
o`00o`3ooooo0?ooon00oooo003o0?ooool0ooooh03oool00?l0ooooo`3ooooP0?ooo`00o`3ooooo
0?ooon00oooo003o0?ooool0ooooh03oool00?l0ooooo`3ooooP0?ooo`00o`3ooooo0?ooon00oooo
003o0?ooool0ooooh03oool00?l0ooooo`3ooooP0?ooo`00o`3ooood0?ooo`800000j@3oool00?l0
oooolP3oool2000000030?ooo`00000000000>P0oooo003o0?ooonl0oooo0`0000030?ooo`<00000
i`3oool00?l0ooook@3oool2000000H0oooo00@000000?ooo`0000000000iP3oool00?l0oooojP3o
ool3000000P0oooo00@000000?ooo`3oool00000iP3oool00?l0ooooi`3oool3000000/0oooo00@0
00000?ooo`3oool00000iP3oool00?l0ooooi@3oool2000000h0oooo00D000000?ooo`3oool0oooo
0000003U0?ooo`00o`3ooooR0?ooo`<00000403oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo0>80oooo003o0?ooon00oooo0P00000C0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0hP3oool00?l0oooog03oool4000001H0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`3Q0?ooo`00o`3ooooK0?ooo`8000006@3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo0>00oooo003o0?ooomP0oooo0`0000020?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3P0?ooo`00o`3ooooF0?ooo`8000001@3o
ool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0g`3o
ool00?l0ooood`3oool300000200oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3N
0?ooo`00o`3ooomm0?ooo`050<AO5000000000000000000DGl@0103oool01@3oa5l0500000000000
03D0Sooo04D0oooo0`00000S0?ooo`030000003oool0oooo00D0oooo0P00003O0?ooo`00o`3oooml
0?ooo`070<AO50000000=Hoo0?ooo`3ool@0Ga@001AOa0020?ooo`070?ooS`0e0000=Hoo0?ooo`3o
ScD0000e08ooo`120?ooo`8000009P3oool00`000000oooo0?ooo`030?ooo`80000000<0oooo0000
003oool0gP3oool00?l0ooooN`3oool00`3ool@0Ga@0000eS`030?ooo`070?n?=@0055l0a?oo0?oo
o`2?=@0001AO0<Coo`020?ooo`0308le0000=Hl0oooo03h0oooo0`00000X0?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool0g@3oool00?l0ooooO03oool00`2?=@0003F?0?ooo`020?oo
o`0608le000055l0a?oo0?ooS`0e000055o40`3oool00`34Ga@0000e08ooo`0l0?ooo`800000:`3o
ool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0=`0oooo003o0?ooog`0oooo00/0olAO
01@0=@2?ool0oooo08le00000000=Hoo0?ooo`3oa5l0500003F?o`030?ooo`030?n?=@0001@0GlCo
03T0oooo0`00000]0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0f`3oool00?l0
ooooO@3oool01@3oohl0=A@D01@00000000055o40080oooo00<0olAO01@0000eSol00`3oool00`3o
ScD0000D05o4o`0f0?ooo`<00000<03oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
0=/0oooo003o0?ooogd0oooo00@0olAO01@00000000003F?0`3oool00`3oa5l0500003F?o`030?oo
o`030?n?=@0001@0GlCo03@0oooo1000000a0?ooo`030000003oool0oooo00T0oooo00<000000?oo
o`3oool0fP3oool00?l0ooooO03oool01P3oa5l050000000=@2?ool0ohle000eS`80oooo00<0olAO
01@0000eSol00`3oool00`3oScD0000D05o4o`0b0?ooo`800000103oool00`000000oooo0?ooo`0^
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0f@3oool00?l0ooooO03oool00`2?
=@00000e08ooo`020?ooo`0608le000DGl@0oooo0?ooS`0e000055o40`3oool00`34Ga@0000e08oo
o`0_0?ooo`<00000=`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0=T0oooo003o
0?ooog`0oooo00<0ScD0000DG`34ool00P3oool01`34Ga@003F?0?ooo`3oool0a5lD000DG`34ool0
0P3oool00`2?=@0003F?0?ooo`0/0?ooo`<00000>P3oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo0=P0oooo003o0?ooog`0oooo00L0ohle0000000eSol0oooo0?ooS`0e0000=Hoo0080
oooo00L0oon?03D0000eSol0oooo0?n?=@0003D0Sooo02/0oooo0P00000m0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool0e`3oool00?l0ooooO@3oool01@3oScD00000000000000000
55o400@0oooo00D0oon?03D000000000000e08ooo`0Y0?ooo`<00000?`3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo0=L0oooo003o0?oook80oooo0P0000130?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool0eP3oool00?l0oooo[`3oool3000004D0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`3E0?ooo`00o`3ooon]0?ooo`800000B03oool00`000000
oooo0?ooo`0=0?ooo`030000003oool0oooo0=D0oooo003o0?ooojX0oooo0`00001:0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool0e03oool00?l0ooooY`3oool3000000030?ooo`00
0000oooo04X0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`3C0?ooo`00o`3ooonU
0?ooo`8000001@3oool00`000000oooo0?ooo`180?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool0d`3oool00?l0ooooXP3oool300000580oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`3B0?ooo`00o`3ooonP0?ooo`800000E@3oool00`000000oooo0?ooo`0>0?ooo`80
000000<0oooo0000003oool0dP3oool00?l0ooooW@3oool3000005L0oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`3A0?ooo`00o`3ooonK0?ooo`800000FP3oool00`000000oooo0?oo
o`0B0?ooo`030000003oool0oooo0=00oooo003o0?oooiP0oooo0`00001L0?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool0c`3oool00?l0ooooUP3oool2000005l0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`3?0?ooo`00o`3ooonB0?ooo`@00000H@3oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo0<h0oooo003o0?oooi00oooo1000001S0?ooo`030000
003oool0oooo01D0oooo00<000000?ooo`3oool0c@3oool00?l0ooooSP3oool2000000<0oooo00<0
00000?ooo`3oool0H@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0<d0oooo003o
0?oooh/0oooo0`0000060?ooo`030000003oool0oooo0600oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0?ooo`3<0?ooo`00o`3ooon90?ooo`8000002P3oool00`000000oooo0?ooo`1P0?oo
o`030000003oool0oooo01H0oooo00<000000?ooo`3oool0b`3oool00?l0ooooQP3oool3000006l0
oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`3;0?ooo`00o`3ooon40?ooo`800000
LP3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0<X0oooo003o0?oooh40oooo0`00
001d0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0bP3oool00?l0ooooO`3oool2
000007L0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`390?ooo`00o`3oooml0?oo
o`<00000N@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0<P0oooo003o0?ooogT0
oooo0`00001l0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0b03oool00?l0oooo
M`3oool200000080oooo00<000000?ooo`3oool0NP3oool00`000000oooo0?ooo`0J0?ooo`800000
b03oool00?l0ooooM03oool3000000D0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0?oo
o`0H0?ooo`80000000<0oooo0000003oool0a`3oool00?l0oooo7P3oool01@3oa5l0500000000000
0000=Hoo00@0oooo00D0olAO01@000000000000e08ooo`160?ooo`800000Q03oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo0<H0oooo003o0?oooad0oooo00L0olAO01@0000DGl@0oooo
0?ooS`0e0000=Hoo0080oooo00L0oon?03D0000eSol0oooo0?n?=@0003D0Sooo0480oooo0`000026
0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0a@3oool00?l0oooo703oool00`3o
ol@0Ga@0000eS`030?ooo`070?n?=@0055l0a?oo0?ooo`2?=@0001AO0<Coo`020?ooo`0308le0000
=Hl0oooo03l0oooo0P0000290?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0a03o
ool00?l0oooo703oool00`3oa5l0500003F?o`030?ooo`060?n?=@0001@0GlCo0?ooS`0e000055o4
0`3oool00`34Ga@0000e08ooo`0l0?ooo`<00000R`3oool00`000000oooo0?ooo`0M0?ooo`030000
003oool0oooo0140oooo00<0_g@0000000000000100000002@18W=l0oooo0?ooo`3oool0omnL04P0
00000000001d0;ooo`2S0?ooo`00o`3ooolL0?ooo`030?o4G`0D0000=Hoo00<0oooo00H0ohle0000
501Oa?l0olAO01@0000eSol30?ooo`030?n?=@0001@0GlCo03X0oooo0P00002>0?ooo`030000003o
ool0oooo01h0oooo00<000000?ooo`3oool04@3oool00`3OW4P00000000000030000000:0018W03O
ool0oooo0?oo_`1d0000M;oo0?ooo`3o_g@0001d0;oooj80oooo003o0?oooa`0oooo00<0olAO01@0
000eSol00`3oool01P2?=@00000D05o4o`3oa5l0500003F?o`<0oooo00<0ohle0000501Oa?l0=`3o
ool300000900oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0A0?ooo`030=nLB018
W=l0oooo0080oooo00H0giad09cOo`3oool0_g@00018W03Oool20?ooo`030;md0000M;l0oooo0:40
oooo003o0?oooa`0oooo00`0oon?03D00000000055o40?ooo`3oScD00000000eS`3oool0olAO01@0
000eSol30?ooo`030?n?=@0001@0GlCo03@0oooo0`00002C0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool04P3oool00`3OW4P0M;oo0?ooo`030?ooo`030?oo_`1d0000B9cO00<0oooo
00<0gia80000M02oool0X@3oool00?l0oooo7@3oool00`2?=@0001AO08mO500200000006000DG`34
ool0oooo0?o4G`0D0000=Hoo0`3oool00`3oScD0000D05o4o`0b0?ooo`80000000<0oooo0000003o
ool0T`3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0180oooo00<0_gA809cOo`3o
ool00P3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo0:40oooo003o0?oooad0oooo
00<0olAO01@0000eSol01@3oool00`3oohl0=@0001AOa0030?ooo`030<AO500003D0Sooo02l0oooo
0`0000040?ooo`030000003oool0oooo0980oooo00<000000?ooo`3oool0803oool00`000000oooo
0?ooo`0A0?ooo`030?oog`2LB7@0_ooo0080oooo00<0omnL04P0001d_ol00`3oool00`3o_g@00018
09cOo`2Q0?ooo`00o`3ooolN0?ooo`030<AO500001@0GlCo00D0oooo00<0a5lD000DG`34ool00P3o
ool00`2?=@0003F?0?ooo`0]0?ooo`800000W03oool00`000000oooo0?ooo`0P0?ooo`030000003o
ool0oooo0180oooo00L0okmd001d_`3oool0oooo0?oOW0180000M;oo00<0oooo00<0okmd0000B02L
gol0X@3oool00?l0oooo7`3oool00`3oScD00000000eS`040?ooo`070?ooS`0e0000=Hoo0?ooo`3o
ScD0000e08ooo`0[0?ooo`<00000WP3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo
0140oooo00L0oono07@0B02Lgol0oooo0?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol0X@3o
ool00?l0oooo803oool01@3ool@0Ga@00000000003D0Sooo0080oooo00D0oon?03D000000000000e
08ooo`0Z0?ooo`800000X@3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0100oooo
00L0oono07@00018W=l0oooo0?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool0X@3oool00?l0
ooooD`3oool300000:<0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0;0?ooo`03
0?noW02Lgol0oooo0080oooo00<0omnL04P00018W=l00P3oool00`3OW4P004RL0=ooo`020?ooo`03
0;md0000M;l0oooo0:40oooo003o0?oooe40oooo0P00002V0?ooo`030000003oool0oooo02<0oooo
00<000000?ooo`3oool02P3oool0203ooml0W4Q809cOo`3oool0oono07@0000007@0_ooo0P3oool0
1`3ookl0M00007Boo`3oool0okmd0000M02oool0XP3oool00?l0ooooCP3oool300000:P0oooo00<0
00000?ooo`3oool0903oool00`000000oooo0?ooo`0:0?ooo`060?oog`2LB00000000000000007@0
_ooo103oool01@3ookl0M0000000000007@0_ooo0:<0oooo003o0?oood/0oooo0`00002[0?ooo`03
0000003oool0oooo02@0oooo0P00002m0?ooo`00o`3ooom90?ooo`80000000<0oooo0000003oool0
Z`3oool00`000000oooo0?ooo`0R0?ooo`80000000<0oooo0000003oool0_03oool00?l0ooooAP3o
ool3000000@0oooo00<000000?ooo`3oool0Z@3oool00`000000oooo0?ooo`0P0?ooo`800000103o
ool00`000000oooo0?ooo`2j0?ooo`00o`3ooom40?ooo`800000/`3oool00`000000oooo0?ooo`0V
0?ooo`030000003oool0oooo0;X0oooo003o0?oood40oooo0`00002e0?ooo`030000003oool0oooo
02L0oooo00<000000?ooo`3oool0^@3oool00?l0oooo?`3oool200000;P0oooo00<000000?ooo`3o
ool0:03oool00`000000oooo0?ooo`2h0?ooo`00o`3oooll0?ooo`<00000^P3oool00`000000oooo
0?ooo`0X0?ooo`030000003oool0oooo0;P0oooo003o0?ooocX0oooo0P00002m0?ooo`030000003o
ool0oooo02T0oooo00<000000?ooo`3oool0]`3oool00?l0oooo=`3oool300000;l0oooo00<00000
0?ooo`3oool0:@3oool00`000000oooo0?ooo`2g0?ooo`00o`3ooold0?ooo`<00000`P3oool00`00
0000oooo0?ooo`0Z0?ooo`030000003oool0oooo0;H0oooo003o0?oooc80oooo0P0000000`3oool0
00000?ooo`330?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0]@3oool00?l0oooo
;`3oool3000000<0oooo00<000000?ooo`3oool0`P3oool00`000000oooo0?ooo`0Z0?ooo`030000
003oool0oooo0;D0oooo003o0?ooobd0oooo0P0000070?ooo`030000003oool0oooo0<40oooo00<0
00000?ooo`3oool0:`3oool00`000000oooo0?ooo`2d0?ooo`00o`3ooolZ0?ooo`<000002P3oool0
0`000000oooo0?ooo`300?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0/`3oool0
0?l0oooo:03oool200000<l0oooo0P00000^0?ooo`030000003oool0oooo0;<0oooo003o0?ooobD0
oooo0`00003>0?ooo`<0000000<0Yko[000000000000;P3oool00`000000oooo0?ooo`2b0?ooo`00
o`3ooolS0?ooo`800000cP3oool3000000@0Yko[0P00000_0?ooo`030000003oool0oooo0;40oooo
003o0?ooob00oooo0`00003>0?ooo`8000000P2W_^X00`000000Yko[0:Noj`020:Noj`<00000;P3o
ool200000;80oooo003o0?oooad0oooo0`00003>0?ooo`<000001@2W_^X00`000000Yko[0:Noj`02
00000080Z;oZ00<000000?ooo`3oool0:@3oool2000000030?ooo`000000oooo0;40oooo003o0?oo
oa/0oooo0P0000000`3oool000000?ooo`3<0?ooo`8000000P2W_^X00`000000YkkZ0:NnjP030:Nn
jP<0000000D0Z;oZ0000002X_nX0Z;oZ0000000_0?ooo`030000003oool0oooo0:l0oooo003o0?oo
oaP0oooo0`0000040?ooo`030000003oool0oooo0<L0oooo0`0000050:NnjP030000002W_^X0YkkZ
0080000000D0Z;oZ0000002X_nX0Z;oZ000000030:RojP030000003oool0oooo02`0oooo00<00000
0?ooo`3oool0[`3oool00<@0oooo00@0ooo405lD000055l0a?oo0`3oool01@3oa5l0500000000000
03D0Sooo04D0oooo0P00003?0?ooo`8000000P2W_NT00`000000YkkZ0:NnjP030:NnjP<00000102X
_nX00`000000Z;oZ000000030:RojP800000;P3oool00`000000oooo0?ooo`2^0?ooo`00a03oool0
103ool@0Ga@0000DG`34ool20?ooo`070?ooS`0e0000=Hoo0?ooo`3oScD0000e08ooo`110?ooo`<0
0000cP3oool3000000@0YkgY00<000000:NnjP2W_^X00`0000000`2X_^T000000:RojP040:RojP03
0000002X_nX0000000<0000000<0ZL3Z0000003oool0;P3oool00`000000oooo0?ooo`2]0?ooo`00
a03oool0103ool@0Ga@0000DG`34ool20?ooo`0308le000055l0a?oo0080oooo00<0ScD0000eS`3o
ool0?P3oool200000<h0oooo0`0000020:Nlj@030000002W_NT0YkgY0080YkgY0`0000050:Rnj@03
0000002X_nX0Z;oZ0080Z;oZ0`0000050:W0jP030000003oool0oooo02`0oooo00<000000?ooo`3o
ool0[@3oool00<00oooo00<0ScD00000000000001@000000100eSol0oon?03D0000DGl@30?ooo`03
0<AO500003D0Sooo03/0oooo0`00003=0?ooo`<000001@2W_>T00`000000YkgY0:Nmj@0200000080
Z;gY00<000000:Rnj@2X_^T00`2X_^T00`000000Z;oZ0:RojP0200000080ZKoZ00<000000:W0jP2Y
`>X00`2Y`>X00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo0:`0oooo00300?ooo`0<0?o4
G`0eGl@0oooo0?ooo`3ool@0Ga@0000DG`34ool0oooo0?o4G`0D0000=Hoo0`3oool00`3oScD0000D
05o4o`0h0?ooo`<00000cP3oool200000080YkcX00<000000:Nlj@2W_>T00`2W_>T3000000@0Z;gY
00<000000:Rnj@2X_^T00`2X_^T3000000@0ZKoZ00<000000:W0jP2Y`>X00`2Y`>X2000002l0oooo
00<000000?ooo`3oool0Z`3oool00<40oooo00/0a5lD03F?o`3oool0ooo405lD000055l0a?oo0?oo
o`3oa5l0500003F?o`030?ooo`030?n?=@0001@0GlCo03H0oooo0P00003=0?ooo`@000001@2W_>P0
0`000000YkcY0:Nlj@0200000080Z;gX00<000000:Rmj@2X_NT00`2X_NT00`000000Z;kY0:Rnj@02
00000080ZKkY00<000000:VojP2Y_nX00`2Y_nX00`000000ZL3Z0:W0jP0200000080Z/3Z00<00000
0?ooo`3oool0;03oool00`000000oooo0?ooo`2[0?ooo`00`@3oool02`3ool@0Ga@e08ooo`3ool@0
Ga@0000DG`34ool0oooo0?o4G`0D0000=Hoo00<0oooo00<0ohle0000501Oa?l0<`3oool300000</0
oooo100000030:Nkj0030000002W_>P0YkcX00<0YkcX0`0000040:Rmj0030000002X_NT0Z;gY00<0
Z;gY0`0000040:Vnj@030000002Y_nX0ZKoZ00<0ZKoZ0`0000050:[0jP030000003oool0oooo02`0
oooo00<000000?ooo`3oool0ZP3oool00<80oooo00X0olAO01@eS`3ool@0Ga@0000DG`34ool0oooo
0?o4G`0D0000=Hoo0`3oool00`3oScD0000D05o4o`0a0?ooo`8000000P3oool00`000000oooo0?oo
o`370?ooo`8000000P2W^nL00`000000Yk_X0:Nkj0030:Nkj0030000002W_>P0YkcX008000000P2X
_>P00`000000Z;gX0:Rmj0030:Rmj0030000002X_NT0Z;gY0080000000<0ZKkY0000002Y_^T01@2Y
_^T00`000000ZKoZ0:VojP03000000030:Zoj@000000Z/3Z00@0Z/3Z0P00000^0?ooo`030000003o
ool0oooo0:T0oooo00330?ooo`090<AO500eSl@0Ga@0000DG`34ool0oooo0?ooS`0e000055o400<0
oooo00<0a5lD0000=@2?ool0;P3oool3000000D0oooo00<000000?ooo`3oool0``3oool3000000@0
Yk_W00<000000:Nkj02W^nP00`2W^nP3000000@0Z;cX00<000000:Rmj02X_NP00`2X_NP3000000@0
ZKkY00<000000:Vnj@2Y_^T00`2Y_^T300000080Z[oY00<000000:Zoj@0000000P2Z`>X300000003
0:_1jP000000oooo02d0oooo00<000000?ooo`3oool0Z@3oool00<<0oooo00D0ooo405lD500D5000
01AO0<Coo`020?ooo`030<AO500055l0a?oo0080oooo00<0ScD0000eS`3oool0;03oool200000<`0
oooo0P0000000`2W^^L000000:Nki`050:Nki`030000002W^nP0Yk_X0080000000<0Z;_W0000002X
_>P01@2X_>P00`000000Z;gX0:Rmj002000000030:Vmj0000000ZKkY00@0ZKkY00<000000:Vnj@2Y
_^T00`0000020:Voj@050000002Z_nT0Z[oY0000002Z_nT00`0000050:_1jP030000003oool0oooo
02`0oooo00<000000?ooo`3oool0Z03oool00<@0oooo00@0olAO01@0000055l0a?oo0P3oool01`3o
ohl0=@0003F?o`3oool0ohle0000=@2?ool0:P3oool300000</0oooo0`0000040:Nji`030000002W
^nL0Yk_W00<0Yk_W0`0000040:Rki`030000002X_>P0Z;cX00<0Z;cX0`0000040:Vmj0030000002Y
_^T0ZKkY0080ZKkY0`0000050:Voj@030000002Z_nT0Z[oY008000000P2[`>T00`000000Zl7Z0:_1
jP030:_1jP030000003oool0oooo02d0oooo00<000000?ooo`3oool0Y`3oool00<D0oooo00<0a5lD
000DG`34ool00`3oool01@3oohl0=@000000000003D0Sooo02T0oooo0P00003<0?ooo`80000000<0
YkWV0000002W^^L0102W^^L00`000000Yk_W0:Nki`03000000030:Rki`000000Z;_W00@0Z;_W00<0
00000:Rlj02X_>P00`0000000`2Y_>P000000:Vmj0040:Vmj0030000002Y_^T0ZKkY008000000P2Y
_^P00`000000ZKoY0:Voj@030:Voj@@000000`2[`>T00`000000Zl7Z0:_1jP030:_1jP800000;P3o
ool00`000000oooo0?ooo`2W0?ooo`00mP3oool300000</0oooo0`0000040:NiiP030000002W^^L0
Yk[W0080Yk[W0`0000050:Rki`030000002X^nL0Z;_W0080Z;_W0`0000050:Vlj0030000002Y_NP0
ZKgX0080ZKgX0`0000050:Vnj0030000002Y_nT0ZKoY008000000P2Z_nT2000000@0Zl3Y00<00000
0:_1jP2[`NX00P0000020:_1j@030000003oool0oooo02`0oooo00<000000?ooo`3oool0YP3oool0
0?<0oooo0`00003;0?ooo`<0000000<0YkWV0000002W^NH0102W^NH00`000000Yk[W0:Nji`020000
0080Z;[W00<000000:Rki`2X^nL00`2X^nL00`000000Z;_W0:Rki`0200000080ZKcW00<000000:Vl
j02Y_>P00`2Y_>P00`000000ZKgX0:Vmj00200000080ZKgX00<000000:Vnj02Y_^P00`2Y_^P30000
00@0Z[oY0P0000040:_0j@<00000102[`NT00`000000oooo0?ooo`0Z0?ooo`80000000<0oooo0000
003oool0YP3oool00?40oooo0P00003;0?ooo`<000001@2W^NH00`000000YkWV0:NiiP020:NiiP<0
0000102X^^L00`000000Z;_W0:Rki`030:Rki`<00000102Y_>L00`000000ZKcX0:Vlj0030:Vlj0<0
00001@2Y_NP00`000000ZKkX0:Vnj002000000030:Znj0000000Z[oY00D0Z[oY00<000000:_0j@2[
`>T00P0000020:_0j@030000002[`NT0Zl7Y0080Zl7Y0P00000^0?ooo`030000003oool0oooo0:D0
oooo003^0?ooo`<00000b`3oool200000080YkSU00<000000:NiiP2W^NH00`2W^NH00`000000YkWV
0:NiiP0200000080Z;[V00<000000:Rji`2X^^L00`2X^^L00`000000Z;_W0:Rki`0200000080Z;_W
00<000000:Vli`2Y_>L00`2Y_>L00`000000ZKcX0:Vlj00200000080ZKgW00<000000:Vmj02Y_NP0
0`2Y_NP3000000@0Z[kX00<000000:Zoj@2Z_nT00`2Z_nT3000000@0Zl3Y00<000000:_1j@2[`NT0
0P0000020:c1j@030000003oool0oooo02`0oooo00<000000?ooo`3oool0Y03oool00>`0oooo0P00
00020?ooo`030000003oool0oooo0<H0oooo0`0000040:Nhi@030000002W^NH0YkWV00<0YkWV0`00
00040:RjiP030000002X^^L0Z;[W00<0Z;[W0`0000040:Rki`030000002Y_>L0ZKcW00<0ZKcW0`00
00050:Vmi`030000002Y_NP0ZKgX0080000000<0Z[kX0000002Z_^P0102Z_^P00`000000Z[oY0:Zo
j@04000000030:^oj0000000Zl3Y00<0Zl3Y0`0000040:c1j@030000003oool0oooo02`0oooo00<0
00000?ooo`3oool0Y03oool00>T0oooo0`0000050?ooo`030000003oool0oooo0<<0oooo0P000002
0:Nhi@030000002W^>D0YkSU00<0YkSU00<000000:NiiP2W^NH00P0000020:RiiP030000002X^^H0
Z;[V00<0Z;[V00<000000:Rji`2X^^L00P0000020:RkiP030000002X^nL0Z;_W00<0Z;_W00<00000
0:Vli`2Y_>L00P0000020:Vli`030000002Y_NL0ZKgW00<0ZKgW0`0000040:Znj0030000002Z_^P0
Z[kX0080Z[kX0`0000030:^oj0030000002[_nP000000080Zl3Y0P0000020:c1j@030000002/`NT0
[<7Y00<0[<7Y00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`2S0?ooo`00i`3oool20000
0</0oooo0`0000040:Nhi@030000002W^>D0YkSU00<0YkSU0`0000040:RiiP030000002X^^H0Z;[V
00<0Z;[V0`0000040:RkiP030000002X^nL0Z;_W00<0Z;_W0`0000040:Vli`030000002Y_NL0ZKgW
00<0000000<0Z[gW0000002Z_^P0102Z_^P00`000000Z[kX0:Znj00200000080ZkoX00D000000:^o
j02[_nP0ZkoX000000020:^oj0800000102/`NT00`000000[<7Y0:c1j@030:c1j@800000;P3oool0
0`000000oooo0?ooo`2R0?ooo`00i03oool300000</0oooo0P0000020:Ngi@030000002W^>D0YkSU
00<0YkSU00<000000:Nhi@2W^>D00P0000020:Rhi@030000002X^NH0Z;WV00<0Z;WV00<000000:Rj
iP2X^^H00P0000000`2X^^H000000:RkiP050:RkiP030000002X^nL0Z;_W0080000000<0ZK_W0000
002Y_>L01@2Y_>L2000000D0Z[gW00<000000:Znj02Z_^P00P2Z_^P3000000@0ZkoX00@000000:^o
j02[_nP0ZkoX100000060:c1j@030000002/`NT0[<7Y008000000P2]`NT00`000000oooo0?ooo`0[
0?ooo`030000003oool0oooo0:80oooo003R0?ooo`800000b`3oool3000000@0YkOU00<000000:Nh
i@2W^>D00`2W^>D3000000@0Z;SU00<000000:RiiP2X^NH00`2X^NH3000000@0Z;[V00<000000:Rk
iP2X^nH00`2X^nH3000000@0ZK_W00@000000:Vli`2Y_>L0ZKcW0P0000000`2Z_>L000000:Zmi`04
0:Zmi`030000002Z_^P0Z[kX008000000P2[_^L00`000000ZkoX0:^oj0030:^oj0040000002[_nP0
0000000000<0[<3X00<000000:c1j@2/`NT00`2/`NT3000000@0[L7Y00<000000?ooo`3oool0;03o
ool00`000000oooo0?ooo`2Q0?ooo`00g`3oool300000</0oooo0P0000020:Ngi0030000002W]nD0
YkOU00<0YkOU00<000000:Nhi@2W^>D00P0000000`2W^>D000000:Rhi@050:Rhi@030000002X^NH0
Z;WV0080000000<0Z;WU0000002X^^H0102X^^H00`000000Z;_V0:RkiP03000000030:VkiP000000
ZK_W00@0ZK_W00@000000:Vli`00000000000`2Z_>L00`000000Z[gW0:Zmi`030:Zmi`<00000102[
_^L00`000000ZkoX0:^oj0030:^oj080000000<0[<3X0000002/`>P00P2/`>P00`000000[<7Y0:c1
j@0300000080[L7X00<000000:g1j@2]`NT00P2]`NT2000002h0oooo00<000000?ooo`3oool0X03o
ool00=`0oooo0`00003;0?ooo`<00000102W]n@00`000000YkOU0:Ngi@030:Ngi@<00000102W^>D0
0`000000Z;SU0:Rhi@030:Rhi@<00000102X^ND00`000000Z;[V0:RjiP020:RjiP<000001@2Y^nH0
0`000000ZK_W0:Vki`020:Vki`8000001P2Z_>L00`000000Z[gW0:Zmi`0200000080ZkgW00<00000
0:^ni`2[_^L00`2[_^L00`000000ZkoX0:^oj00200000080[;oX00<000000:c0j00000000P2/`>P3
000000D0[L7X00<000000:g1j@2]`NT00P0000020:k2j@030000003oool0oooo02/0oooo00<00000
0?ooo`3oool0X03oool00=X0oooo0P00003<0?ooo`8000000P2W]^@00`000000YkOT0:Ngi0030:Ng
i0030000002W]nD0YkOU0080000000<0YkOT0000002W^>D0102W^>D00`000000Z;SU0:Rhi@030000
00030:Rii@000000Z;WU00@0Z;WU00<000000:RjiP2X^^H00P0000020:VjiP030000002Y^nH0ZK_V
00<0ZK_V00<000000:Vki`2Y^nL00P0000020:ZliP030000002Z_>L0Z[cW00<0Z[cW0`0000040:^m
i`030000002[_^L0ZkkW00<0ZkkW0`0000040:boj0030000002/`>P00000008000000P2]`>P00`00
0000[L7X0:g1j0040:g1j0800000102^`^T00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo
09l0oooo003G0?ooo`<0000000<0oooo0000003oool0a`3oool4000000@0YkKT00<000000:Ngi02W
]n@00`2W]n@3000000@0YkOT00<000000:Nhi@2W^>D00P2W^>D3000000D0Z;WU00<000000:Rii@2X
^ND00P2X^ND3000000@0ZK[V00<000000:VkiP2Y^nH00`2Y^nH3000000D0Z[cV00<000000:Zli`2Z
_>L00P0000000`2[_NL000000:^mi`050:^mi`030000002[_^L0ZkkW008000000P2/_^L00`000000
[;oX0:boj0020:boj0<00000102]`>P00`000000[L7X0:g1j0020:g1j0<000001P2^`^T00`000000
oooo0?ooo`0/0?ooo`030000003oool0oooo09h0oooo003E0?ooo`8000001@3oool00`000000oooo
0?ooo`310?ooo`@000000P2W]^<00`000000YkKT0:Nfi0040:Nfi0030000002W]n@0YkOT00800000
00<0YkOT0000002W]n@0102W]n@00`000000YkSU0:Nhi@0200000080Z;SU00<000000:Rii@2X^ND0
0`2X^ND00`000000Z;WU0:Rii@0200000080ZK[U00<000000:VjiP2Y^^H00`2Y^^H00`000000ZK_V
0:VkiP0200000080Z[_V00<000000:ZliP2Z_>H00`2Z_>H3000000@0ZkgW00<000000:^mi`2[_NL0
0`2[_NL3000000@0[;kW00<000000:boj02/_nP00P0000020:foi`030000002]`>P0[L3X00<0[L3X
00@000000:g1j000000000000`2]`NP00`000000[/;Y0:k2j@030:k2j@800000;@3oool00`000000
oooo0?ooo`2N0?ooo`00dP3oool3000000P0oooo00<000000?ooo`3oool0_P3oool2000000030:Je
h`000000YkKS00@0YkKS00<000000:Nfi02W]^@00`2W]^@3000000@0YkOT00<000000:Ngi02W]n@0
0P2W]n@3000000@0Z;SU00<000000:Rii@2X^ND00`2X^ND3000000@0ZK[U00<000000:VjiP2Y^^H0
0`2Y^^H3000000D0Z[_V00<000000:ZliP2Z_>H00P0000000`2[_>H000000:^mi`040:^mi`030000
002[_NL0ZkgW00<000000P2/_^L00`000000[;kW0:bni`020:bni`<00000102]_nL00`000000[L3X
0:g0j0030:g0j08000001@2]`NP00`000000[/;Y0:k2j@0300000080[/;X00<000000?ooo`3oool0
:`3oool00`000000oooo0?ooo`2M0?ooo`00d03oool200000<T0oooo0`0000040:Jeh`030000002W
]^<0YkKS0080YkKS00<000000:Nfi02W]^@00`0000000`2W]^@000000:Ngi0040:Ngi0030000002W
]n@0YkOT008000000P2X^>@00`000000Z;SU0:Rhi@030:Rhi@030000002X^ND0Z;WU008000000P2Y
^ND00`000000ZK[U0:Vji@030:Vji@030000002Y^^H0ZK[V008000000P2Z^^D00`000000Z[_V0:Zk
iP030:ZkiP<00000102[_>H00`000000ZkgW0:^mi`020:^mi`<000001@2/_^L00`000000[;kW0:bn
i`0200000080[KoW00<000000:foi`2]_nL00P2]_nL200000080[L3X0P0000020:g0j0030000002]
`NP0[L7X0080[L7X0`0000050:k2j0030000003oool0oooo02`0oooo00<000000?ooo`3oool0W03o
ool00<d0oooo0`0000390?ooo`80000000<0Y[CS0000002V]N<0102V]N<010000000YkKS0:Nfh`2W
]^<4000000D0YkKT00<000000:Ngi02W]n@00P2W]n@3000000@0Z;ST00<000000:Rhi@2X^>D00`2X
^>D3000000@0ZKWU00<000000:Vji@2Y^^D00`2Y^^D3000000@0Z[[U00<000000:ZkiP2Z^nH00`00
00000`2[^nH000000:^liP040:^liP030000002[_NL0ZkgW008000000P2/_NH00`000000[;kW0:bn
i`040:bni`800000102]_nL00`000000[KoW0:foi`020:foi`@00000102]`>P00`000000[L7X0:g1
j00200000080[/7X00<000000:k2j02^`^P00`2^`^P2000002d0oooo00<000000?ooo`3oool0W03o
ool00</0oooo0P0000390?ooo`<00000102V]><00`000000Y[GS0:Jeh`020:Jeh`@000000`2W]^<0
0`000000YkKT0:Nfi0030:Nfi0030000002W]n@0YkOT008000000P2X]n@00`000000Z;ST0:Rhi003
0:Rhi0030000002X^>D0Z;SU008000000P2Y^>@00`000000ZKWU0:Vii@030:Vii@030000002Y^^D0
ZK[U008000000P2Z^^D00`000000Z[[U0:Zji@020:Zji@<000001@2[^nH00`000000ZkcV0:^liP02
0:^liP<00000102/_NH00`000000[;kW0:bni`020:bni`<000001P2]_nL00`000000[KoW0:foi`02
00000080[L3W00<000000:g0j02]`>P00P2]`>P3000000D0[/7X00<000000:k2j02^`^P00P000000
0`2_`^P000000?ooo`0Z0?ooo`80000000<0oooo0000003oool0W03oool00<P0oooo0`0000390?oo
o`80000000<0Y[CR0000002V]><0102V]><00`000000Y[GS0:Jeh`0200000080YkGS00<000000:Nf
h`2W]^<00P2W]^<00`000000YkKT0:Nfi0030:Nfi0<00000102X]n@00`000000Z;ST0:Rhi0030:Rh
i0<00000102Y^>@00`000000ZKWU0:Vii@030:Vii@<00000102Z^^D00`000000Z[[U0:Zji@020000
0080Zk_U00<000000:^kiP2[^nH00`2[^nH00`000000ZkcV0:^liP0200000080[;cV00<000000:bm
iP2/_NH00`2/_NH010000000[;kW0000000000030:bniP030000002]_nL0[KoW00<0[KoW0`000000
1@2]`>L000000:g0i`2]`>L000000080[L3X0P0000020:k1i`030000002^`NP0[/7X00@0[/7X0P00
00040:o2j0030000003oool0oooo02`0oooo00<000000?ooo`3oool0VP3oool00<D0oooo0`000039
0?ooo`<00000102V]>800`000000Y[CS0:Jdh`020:Jdh`<00000102W]N<00`000000YkKS0:Nfh`03
0:Nfh`030000002W]^@0YkKT008000000P2X]n<00`000000Z;OT0:Rgi0030:Rgi0030000002X^>@0
Z;ST008000000P2Y^>@00`000000ZKST0:Vhi0030:Vhi0030000002Y^ND0ZKWU008000000P2Z^ND0
0`000000Z[[U0:Zji@020:Zji@<00000102[^nD00`000000Zk_V0:^kiP030:^kiP<00000102/_>H0
0`000000[;gV0:bmiP030:bmiP8000001@2/_^H00`000000[KoW0:foi`0300000080[KoW00<00000
0:g0i`0000000P2]`>L3000000@0[/7W00<000000:k1j02^`NP00P2^`NP3000000D0[l;X00<00000
0?ooo`3oool0;03oool00`000000oooo0?ooo`0E0?ooo`040?oog`2LB00004RL0=ooo`<0oooo00D0
omnL04P000000000001d0;ooo`1i0?ooo`00``3oool200000<X0oooo0P0000000`2V]>8000000:Jd
hP040:JdhP030000002V]><0Y[CS008000000P2W]N<00`000000YkGS0:Neh`030:Neh`040000002W
]^<0YkKS0:Nfh`@00000102X]n<00`000000Z;OT0:Rgi0030:Rgi0<00000102Y^>@00`000000ZKST
0:Vhi0030:Vhi0<00000102Z^ND00`000000Z[[U0:Zji@0200000080Zk[U00<000000:^ki@2[^nD0
0`2[^nD00`000000Zk_V0:^kiP0200000080ZkcU00<000000:bliP2/_>H00`2/_>H00`000000[;gV
0:bmiP0200000080[;gV00<000000:bniP2/_^H00P2/_^H3000000D0[KoW00<000000:g0i`000000
0P0000020:k0i`030000002^`NL0[/7W00<0[/7W00@000000:k1j000000000000`2_`^P00`000000
[l;X0:o2j0020:o2j0800000;P3oool00`000000oooo0?ooo`0D0?ooo`040?oog`2LB00004RL0=oo
o`80oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo07P0oooo001Y0?ooo`0308le00000000
000000@0000000T055o40?ooo`3oool0oooo0?o4G`0D000000000000=@2?ool0A`3oool300000080
oooo00<000000?ooo`3oool0a03oool3000000@0Y[CR00<000000:JdhP2V]>800P2V]>83000000@0
YkGS00<000000:Neh`2W]N<00`2W]N<4000000<0Z;KS00<000000:Rgh`2X]n<00`2X]n<00`000000
Z;OT0:Rgi002000000030:Vgi0000000ZKST00D0ZKST00<000000:Vhi02Y^>@00P0000020:Vii003
0000002Z^ND0Z[WU0080Z[WU0`0000040:^ji@030000002[^nD0Zk_U00<0Zk_U0`0000040:^li@03
0000002/_>H0[;cV00<0[;cV0`0000040:bmiP030000002/_^H0[;kV008000000P2]_^H00`000000
[KoW0:foi`040:foi`800000102^`>L00`000000[/7W0:k1i`030:k1i`8000001@2_`^P00`000000
[l;X0:o2j00200000080/<?X00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0C0?ooo`04
0?oog`2LB00004RL0=ooo`80oooo00<0_g@00018W03Oool00P3oool00`2oM00007Bo0?ooo`1g0?oo
o`00JP3oool00`34Ga@00000000000030000000:000DG`34ool0oooo0?ooS`0e0000=Hoo0?ooo`3o
ScD0000e08oood@0oooo0P0000060?ooo`030000003oool0oooo0<40oooo0P0000000`2V/n800000
0:JdhP040:JdhP030000002V]>80Y[CR008000000P2W]>800`000000YkGS0:Neh`030:Neh`030000
002W]N<0YkGS008000000P2X]N<00`000000Z;KS0:Rfh`020:Rfh`030000002X]n<0Z;OS00<0Z;OS
0`0000040:Vgi0030000002Y^>@0ZKST00<0ZKST0`0000040:Vii0030000002Z^ND0Z[WU00800000
0P2Z^N@00`000000Zk[U0:^ji@030:^ji@030000002[^nD0Zk_U0080000000<0Zk_U0000002[_>D0
1@2[_>D00`000000[;cV0:bliP0200000080[;cV00<000000:bmiP2/_NH00P2/_NH3000000@0[KkV
00<000000:foi`2]_nL00P2]_nL4000000D0[/3W00<000000:k1i`2^`NL00P0000020:o1i`030000
002_`^P0[l;X0080[l;X0`0000050;33j0030000003oool0oooo02/0oooo00<000000?ooo`3oool0
3`3oool00`2oM0000000000000050000000407Boo`3ookl0M00004RLg`<0oooo00<0gia80000M02o
ool0M`3oool006/0oooo00<0a5lD01AOa03oool00P3oool01P34GcD0GlCo0?ooo`2?=@0001AO0<Co
o`80oooo00<0ScD0000eS`3oool0@03oool300000<T0oooo0`0000040:JchP030000002V]>80Y[CR
0080Y[CR0`0000040:NdhP030000002W]N<0YkGS00<0YkGS0`0000040:Reh`030000002X]^<0Z;KS
00<0Z;KS00<000000:Rgh`2X]n<00P0000000`2X]n<000000:Vgi0040:Vgi0030000002Y^>@0ZKST
00<000000P2Y^>@00`000000ZKWT0:Vii0020:Vii0<00000102Z^N@00`000000Zk[U0:^ji@030:^j
i@<00000102[^nD00`000000ZkcU0:^li@030:^li@<00000102/_>H00`000000[;gV0:bmiP020000
0080[KgV00<000000:fniP2]_^H00`2]_^H010000000[KoW0000000000030:joiP030000002^`>L0
[/3W00<0[/3W0`0000040:o1i`030000002_`^P0[l;X008000000P2``^L00`000000/<?X0;33j003
0;33j0800000;@3oool00`000000oooo0?ooo`0>0?ooo`0<0?oOW01dW=l0oooo0?ooo`3ooml0W4P0
0018W03Oool0oooo0?oOW0180000M;oo0`3oool00`3o_g@0001809cOo`1g0?ooo`00K03oool00`34
Ga@0=Hoo0?ooo`030?ooo`030?ooS`0e000055o400<0oooo00<0a5lD0000=@2?ool0?P3oool20000
0<X0oooo0P0000000`2V/n8000000:JchP040:JchP030000002V]>80Y[CR008000000P2W]>800`00
0000YkCR0:NdhP030:NdhP030000002W]N<0YkGS008000000P2W]N800`000000Z;GS0:Reh`030:Re
h`040000002X]^<0Z;KS0:Rfh`@00000102X]n<00`000000ZKOT0:Vgi0020:Vgi0<000001@2Y^>@0
0`000000ZKWT0:Vii00200000080Z[WT00<000000:Zii02Z^N@00`2Z^N@00`000000Zk[U0:^ji@02
000000030:^ji0000000Zk_U00@0Zk_U00<000000:^li@2[_>D00`0000020:bli@030000002/_>H0
[;cV0080[;cV0`0000040:fmiP030000002]_^H0[KkV00<0[KkV0P0000050:joiP030000002^`>L0
[/3W00<000000P2_`>L00`000000[l7W0:o1i`020:o1i`<000001@2``^L00`000000/<?X0;33j002
00000080/<?X00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0>0?ooo`0;0=nLB01d_ol0
oooo0?oog`2LB00004RL0=ooo`3oool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`1g0?oo
o`00K@3oool00`2?=A@0GlCo0?ooo`020?ooo`030?o4G`0D0000=Hoo00<0oooo00<0ohle0000501O
a?l0>`3oool300000<P0oooo100000040:JchP030000002V/n80Y[?R0080Y[?R0`0000040:NdhP03
0000002W]>80YkCR00<0YkCR0`0000040:NehP030000002X]N<0Z;GS00<0Z;GS100000020:Rfh`03
0000002X]n<0Z;OS00<0Z;OS00<000000:Vgi02Y]n@00P0000020:Vgh`030000002Y^>@0ZKST00<0
ZKST0`0000040:Zii0030000002Z^N@0Z[WT00<0Z[WT0`0000040:^ji0030000002[^nD0Zk_U0080
Zk_U0`0000050:bli@030000002/_>H0[;cV008000000P2]_ND00`000000[KgV0:fmiP030:fmiP03
0000002]_^H0[KkV008000000P2^_^H00`000000[[oV0:joiP020:joiP<000001@2_`>L00`000000
[l7W0:o1i`02000000030;31i`000000/<;W00H0/<;W0P0000040;33j0030000003oool0oooo02/0
oooo00<000000?ooo`3oool03P3oool02`3ooml0W4Qd0;ooo`3ooml0W4P00018W03Oool0oooo0?oO
W0180000M;oo00<0oooo00<0okmd0000B02Lgol0M`3oool006d0oooo00<0ooo405lD=@2?ool00P3o
ool00`3oa5l0500003F?o`030?ooo`030?n?=@0001@0GlCo03T0oooo0P0000370?ooo`@000000P2V
/n400`000000Y[?R0:JchP030:JchP030000002V/n80Y[?R008000000P2W/n800`000000YkCR0:Nd
hP030:NdhP030000002W]>80YkCR008000000P2W]>800`000000YkGR0:NehP030:NehP030000002X
]N<0Z;GS0080000000<0Z;KR0000002X]^<0102X]^<00`000000Z;OS0:Rgh`020:Rgh`<000001@2Y
]n<00`000000ZKST0:Vhi00200000080Z[SS00<000000:Zii02Z^N@00`2Z^N@00`000000Z[WT0:Zi
i002000000030:^ji0000000Zk[T00@0Zk[T00<000000:^ki@2[^nD00P0000020:bki@030000002/
_>D0[;cU00@0[;cU0P0000040:fmi@030000002]_NH0[KgV00<0[KgV0`0000040:jniP030000002^
_nH0[[oV00<0000000<0[koV0000002_`>L01@2_`>L2000000@0/<7W00<000000;32i`2``^L00P2`
`^L3000000H0/<?X00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0>0?ooo`0:0?oOW018
M;l0oooO09a80000B9`0gooo0?ooo`3ogi`0B00007Boo`<0oooo00<0okmd0000B02Lgol0M`3oool0
06h0oooo00L0ohle000eS`3oool0oooo0?o4G`0D0000=Hoo00<0oooo00<0ohle0000501Oa?l0=P3o
ool300000<L0oooo0P0000000`2V/^4000000:Jch@040:Jch@030000002V/n80Y[?R0080Y[?R0`00
00040:NchP030000002W]>80YkCR00<0YkCR0`0000040:NdhP030000002W]N80YkGR00<0YkGR0`00
00040:RfhP030000002X]^<0Z;KS0080Z;KS00<000000:Rgh`2X]n<00P0000020:Vgh`030000002Y
]n<0ZKOS00<0ZKOS0`0000040:Zhh`030000002Z^N@0Z[WT00<0Z[WT0`0000040:^ji0030000002[
^^@0Zk[T0080Zk[T0`0000040:bki@030000002/_>D0[;cU0080[;cU0P0000000`2]_>D000000:fm
i@040:fmi@030000002]_NH0[KgV008000000P2^_^D00`000000[[kV0:jniP020:jniP<000000`2_
_nH2000000@0[l3W0`0000050;31i`030000002``^L0/<;W008000000`2``^L010000000/<?X0;33
j02``nP3000002h0oooo00<000000?ooo`3oool03P3oool02@3OW4P0M;oO09a80000B9`0gooo0?oo
o`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo07L0oooo001^0?ooo`070?ooS`0e01@0GlCo
0?ooo`3oa5l0500003F?o`030?ooo`030?n?=@0001@0GlCo03<0oooo0`0000370?ooo`<00000102V
/^400`000000Y[?Q0:Jch@020:Jch@030000002V/n80Y[?R008000000P2W/n400`000000Yk?R0:Nc
hP030:NchP030000002W]>80YkCR008000000P2W]>800`000000YkCR0:NdhP030:NdhP030000002W
]N80YkGR0080000000<0Z;GR0000002X]^80102X]^8010000000Z;KS0:Rfh`2X]^<4000000D0ZKOS
00<000000:Vgh`2Y]n<00P0000020:Zhh`030000002Z^><0Z[SS00<0Z[SS00<000000:Zii02Z^N@0
0P0000000`2[^N@000000:^ji0040:^ji0030000002[^^@0Zk[T008000000P2/^n@00`000000[;_U
0:bki@030:bki@040000002/_>D00000000000<0[KcU00<000000:fmi@2]_ND00`2]_ND3000000@0
[[kU00<000000:jniP2^_^H00P0000020:noiP080000002__nH0[koV0:noiP000000[koV0000002_
`>L2000000<0/<7V00<000000;31i`2``NL00P2``NL3000000D0/<;W00@000000;33j00000000000
0`2a`nL00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo00h0oooo00D0oooO09a8B018B000
04RL0=ooo`020?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d_`3oool0M`3oool006h0oooo
00L0oon?03D0000DGl@0oooo0?ooS`0e000055o400<0oooo00<0a5lD0000=@2?ool0<@3oool20000
00030?ooo`000000oooo0<D0oooo0P0000000`2V/^4000000:Jbh@040:Jbh@030000002V/n40Y[?Q
0080Y[?Q0`0000040:Nch@030000002W/n80Yk?R00<0Yk?R0`0000040:NdhP030000002W]>80YkCR
00<0YkCR0`0000040:RehP030000002X]^80Z;KR0080Z;KR100000030:Vfh`030000002Y]n<0ZKOS
00<0ZKOS0`0000040:Zhh`030000002Z^><0Z[SS00<0Z[SS0`0000040:^ii0030000002[^^@0Zk[T
0080Zk[T0`0000040:bki0030000002/^nD0[;_U00<0[;_U0P0000060:fli@030000002]_ND0[KgU
0080000000<0[[gU0000002^_^D01@2^_^D2000000@0[koV00H000000:noiP2__nH0[koV0000002_
_nH2000000D0/<7V00<000000;31i`2``NL00P0000020;32i`030000002``^L0/<;W00<0/<;W0P00
00050;73i`030000003oool0oooo02`0oooo00<000000?ooo`3oool03P3oool0103ogi`0B0000018
W03Oool20?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d0;ooo`1h0?ooo`00J@3oool00`3oSel0
GlCo0?ooo`020?ooo`030?o4G`0D000055o40080oooo00<0a5lD000DG`34ool00P3oool00`2?=@00
03F?0?ooo`0^0?ooo`<00000103oool00`000000oooo0?ooo`300?ooo`<00000102V/^400`000000
Y[;Q0:Jbh@030:Jbh@040000002V/n40000000000080Yk?Q00<000000:Nch@2W/n400`2W/n400`00
0000Yk?R0:NchP0200000080Yk?Q00<000000:NdhP2W]>800`2W]>800`000000YkCR0:NdhP020000
00030:RdhP000000Z;GR00@0Z;GR00<000000:RfhP2X]^800P0000020:VfhP030000002Y]^<0ZKKS
00<0ZKKS00<000000:Vgh`2Y]n<00P0000020:Zgh`030000002Z^><0Z[SS00<0Z[SS00<000000:Zh
h`2Z^><00P0000000`2[^><000000:^ii0040:^ii0030000002[^^@0Zk[T008000000P2/^^@00`00
0000[;_T0:bki0030:bki0030000002/^nD0[;_U008000000P2]^n@00`000000[KcU0:fli@030:fl
i@<00000102^_ND010000000[[kU0:jni@2^_^D2000000030:jni@000000[koV00@0[koV00<00000
0:noiP2__nH00P0000020:o0iP030000002``NH0/<7V0080/<7V0`0000050;32i`030000002``^L0
/<;W008000000P2a`^L00`000000/L?W0;73i`020;73i`800000;P3oool00`000000oooo0?ooo`0>
0?ooo`030=nLB000B9`0gooo00<0oooo00D0oono07@000000000001d0;ooo`1i0?ooo`00J@3oool0
203ool@0Ga@D05o4o`3oool0oon?03D0000003D0Sooo0P3oool01`3oohl0=@0003F?o`3oool0ohle
0000=@2?ool0;@3oool200000<P0oooo0P0000000`2V/^4000000:Jbh@040:Jbh@040000002V/^40
Y[;Q0:Jbh@@00000102W/n400`000000Yk?Q0:Nch@030:Nch@<00000102W/n400`000000YkCR0:Nd
hP030:NdhP<00000102X]>800`000000Z;GR0:RehP020:RehP<00000102Y]^800`000000ZKKS0:Vf
h`030:Vfh`<00000102Z]n<00`000000Z[SS0:Zhh`030:Zhh`<00000102[^><00`000000ZkWT0:^i
i0020:^ii0<00000102/^^@00`000000[;_T0:bki0030:bki0<00000102]^n@00`000000[KcU0:fl
i@03000000030:fli@000000[[gU00@0[[gU00@000000:jni@00000000000`2^_^D00`000000[koV
0:noiP030:noiP@000000`2_`>H00`000000/<7V0;31iP0200000080/<7V00<000000;32i`2``^L0
0`2``^L3000000@0/L;W00<000000;73i`2a`nL00P0000020;;4i`030000003oool0oooo02/0oooo
0P00002C0?ooo`00JP3oool01P3ool@0Ga@0000000000000000e08ooo`@0oooo00D0oon?03D00000
0000000e08ooo`0[0?ooo`<00000a`3oool3000000@0Y[;Q00<000000:Jbh@2V/^400P2V/^440000
00<0Y[;Q00<000000:Nch@2W/n400`2W/n400`000000Yk?Q0:Nch@0200000080Yk?Q00<000000:Nc
h@2W/n400`2W/n400`000000YkCR0:NdhP02000000030:RdhP000000Z;CR00@0Z;CR00<000000:Re
hP2X]N800P0000020:VehP030000002Y]^80ZKKR00<0ZKKR00<000000:Vfh`2Y]^<00P0000020:Vf
hP030000002Z]n<0Z[OS00<0Z[OS00<000000:Zhh`2Z^><00P0000000`2[^><000000:^hh`040:^h
h`030000002[^N@0ZkWT008000000P2/^N<00`000000[;[T0:bji0030:bji0030000002/^n@0[;_T
008000000P2]^n@00`000000[K_T0:fki0020:fki0<000001@2]_>D00`000000[[gU0:jmi@020:jm
i@8000001P2^_^D00`000000[koV0:noiP0200000080[koU0P0000030:o0iP<000001@2``NH00`00
0000/<;W0;32i`0200000080/L;V00<000000;72i`2a`^L00P2a`^L3000000D0//CW00<000000?oo
o`3oool0:03oool2000000030?ooo`000000oooo0980oooo002R0?ooo`800000b03oool200000080
Y[;Q00<000000:Jbh@2V/^400P2V/^400`000000Y[;Q0:Jbh@0200000080Y[;Q00<000000:Jbh@2V
/^400P2V/^400`000000Yk?Q0:Nch@030:Nch@<00000102W/n400`000000Yk?Q0:Nch@030:Nch@<0
0000102X]>800`000000Z;CR0:RdhP020:RdhP<00000102Y]N800`000000ZKKR0:VfhP030:VfhP<0
0000102Y]^800`000000Z[OS0:Zgh`030:Zgh`<00000102[^><00`000000ZkSS0:^hh`020:^hh`<0
0000102/^N<00`000000[;[T0:bji0030:bji0<00000102]^n@00`000000[K_T0:fki00200000080
[KcT00<000000:fli@2]_>D00`2]_>D00`000000[[gU0:jmi@0200000080[[gU00<000000:jni@2^
_^D00`2^_^D3000000@0[koU0P0000000`2_`>H00000000000020;30iP030000002``NH0/<7V00@0
/<7V0P0000040;72iP030000002a`^L0/L;W008000000P2b`nL00`000000//CW0;;4i`040;;4i`03
0000003oool0oooo02D0oooo0P0000030?ooo`030000003oool0oooo0940oooo002O0?ooo`<00000
a`3oool3000000@0Y[;Q00<000000:Jbh@2V/^400`2V/^42000000@0Y[;Q00<000000:Jbh@2V/^40
0`2V/^400`000000Yk?Q0:Nch@0200000080Yk?Q00<000000:Nch@2W/n400`2W/n400`000000Yk?Q
0:Nch@02000000030:Rdh@000000Z;CR00@0Z;CR00<000000:RdhP2X]>800P0000020:RehP030000
002Y]N80ZKGR00<0ZKGR00<000000:VfhP2Y]^800P0000020:VfhP030000002Y]^80ZKKR00<0ZKKR
00<000000:Zgh`2Z]n<00P0000000`2Z]n8000000:^hh`040:^hh`030000002[^><0ZkSS00800000
0P2[^N<00`000000[;WS0:bih`030:bih`030000002/^^@0[;[T0080000000<0[;[S0000002]^n@0
1@2]^n@2000000@0[KcT00<000000:fli@2]_>D00`2]_>D3000000D0[[gU00<000000:jni@2^_^D0
0P0000000`2__^D000000:noi@050:noi@800000102``>H00`000000/<7V0;31iP020;31iP<00000
1P2a`^H2000000@0//?W00<000000;;4i`2ba>L00P2ba>L3000002d0oooo00<000000?ooo`3oool0
T03oool009d0oooo0P0000380?ooo`8000000P2V/^400`000000Y[;Q0:Jbh@030:Jbh@030000002V
/^40Y[;Q0080000000<0Y[;Q0000002V/^40102V/^400`000000Y[;Q0:Jbh@020:Jbh@<00000102W
/n400`000000Yk?Q0:Nch@030:Nch@<00000102X]>400`000000Z;CR0:RdhP020:RdhP<00000102X
]N800`000000ZKGR0:VehP030:VehP<00000102Y]^800`000000ZKKR0:VfhP030:VfhP<00000102Z
]n800`000000ZkSS0:^hh`020:^hh`<00000102[^N<00`000000[;WS0:bih`030:bih`<00000102/
^^<010000000[K_T0:fki02]^n@2000000030:fki0000000[KcT00@0[KcT00<000000:fli@2]_>D0
0P0000000`2^_N@000000:jmi@060:jmi@800000102__^D010000000[koU0:noi@2__nD4000000D0
/<3V00@000000;31iP00000000000`2a`NH010000000/L;V0;72iP2a`^H2000000030;;2iP000000
//?W00@0//?W00@000000;;4i`00000000000`2ca>L00`000000oooo0?ooo`0[0?ooo`030000003o
ool0oooo08l0oooo002I0?ooo`@00000aP3oool4000000@0Y[;Q00<000000:Jbh@2V/^400`2V/^43
000000<0Y[;Q00<000000:Jbh@2V/^400`2V/^400`000000Y[;Q0:Jbh@0200000080Yk;Q00<00000
0:Nch@2W/n400`2W/n400`000000Yk?Q0:Nch@02000000030:Nch@000000Z;CQ00@0Z;CQ00<00000
0:RdhP2X]>800P0000020:Rdh@030000002X]N80Z;GR00<0Z;GR00<000000:VehP2Y]N800P000000
0`2Y]N8000000:VfhP050:VfhP030000002Y]^80ZKKR0080000000<0Z[OR0000002Z]n80102Z]n80
0`000000ZkSS0:^hh`0200000080ZkSR00<000000:^ih`2[^N<00`2[^N<00`000000[;WS0:bih`02
000000030:bih`000000[;[S00@0[;[S00@000000:fki000000000000`2]^n@00`000000[KcT0:fl
i0030:fli0<00000102^_N@00`000000[[gU0:jmi@020:jmi@<000001@2__^D010000000[koU0000
000000020;2oi@8000001@2``>H2000000D0/L7V00@000000;72iP00000000000`2b`^H00`000000
//?W0;;3i`030;;3i`8000001@2ca>L00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo08l0
oooo002G0?ooo`@00000a03oool4000000<0Y[;Q00<000000:Jbh@2V/^400`2V/^400`000000Y[;Q
0:Jbh@02000000030:Jbh@000000Y[;Q00@0Y[;Q00@000000:Jbh@2V/^40Y[;Q100000040:Nbh@03
0000002W/n40Yk?Q00<0Yk?Q0`0000040:Nch@030000002X]>40Z;CQ0080Z;CQ0`0000040:Rdh@03
0000002X]N80Z;GR00<0Z;GR0`0000040:VehP030000002Y]^80ZKKR00<0ZKKR0`0000040:ZghP03
0000002Z]n80Z[OR0080Z[OR0`0000040:^hhP030000002[^N<0ZkWS00<0ZkWS0`0000040:bih`03
0000002/^^<0[;[S0080[;[S0P0000060:fki0030000002]_>@0[KcT0080000000<0[[cT0000002^
_N@0102^_N@00`000000[[gU0:jmi@02000000<0[kgT00<000000:nni@2__^D00P2__^D2000000@0
/;oU00<000000;2oi@0000000P2``>H200000080/L3U00<000000;71iP2a`NH00P2a`NH2000000H0
//;V00<000000;;3i`2b`nL00P0000020;?3iP030000002ca>L0/lCW0080/lCW0P00000]0?ooo`03
0000003oool0oooo08h0oooo002E0?ooo`800000103oool00`000000oooo0?ooo`2o0?ooo`800000
0P2V/^400`000000Y[;Q0:Jbh@020:Jbh@030000002V/^40Y[;Q00<0Y[;Q0`0000040:Jbh@030000
002V/^40Y[;Q0080Y[;Q100000030:Nbh@030000002W/^40Yk;Q00<0Yk;Q00<000000:Nch@2W/n40
0P0000000`2W/n4000000:Nch@040:Nch@030000002X]>40Z;CQ008000000P2X]>400`000000Z;CQ
0:Rdh@030:Rdh@030000002X]N80Z;GR008000000P2Y]N400`000000ZKGR0:VehP020:VehP030000
002Y]^80ZKKR00<0000000<0Z[KR0000002Z]n80102Z]n800`000000Z[OR0:ZghP0200000080ZkOR
00<000000:^hhP2[^>800`2[^>800`000000ZkWS0:^ih`02000000030:bih`000000[;WS00@0[;WS
00<000000:bjh`2/^^<00P0000020:fjh`030000002]^n@0[K_T00<0[K_T0`0000040:jli0030000
002^_N@0[[gT0080[[gT0`0000050:nmi0030000002__^D0[kkU008000000P2`_^D01P000000/;oU
0;2oi@2`_nD000000;2oi@<00000102a`>D00`000000/L7V0;71iP0200000080//7V00<000000;;2
iP2b`^H00`2b`^H3000000@0/l?V00<000000;?4i`2ca>L00P0000020;?4iP030000003oool0oooo
02/0oooo00<000000?ooo`3oool0S@3oool00980oooo0`0000350?ooo`<00000102V/^400`000000
Y[;Q0:Jbh@030:Jbh@030000002V/^40Y[;Q0080000000<0Y[;Q0000002V/^40102V/^400`000000
Y[;Q0:Jbh@0200000080Yk;Q00<000000:Nbh@2W/^400P2W/^400`000000Yk;Q0:Nbh@030:Nbh@<0
0000102W/n400`000000Yk?Q0:Nch@020:Nch@<00000102X]>400`000000Z;CQ0:Rdh@030:Rdh@<0
0000102Y]N400`000000ZKGR0:VehP030:VehP8000001@2Z]^800`000000Z[OR0:ZghP020:ZghP<0
0000102[]n800`000000ZkSR0:^hhP030:^hhP<00000102/^N<00`000000[;WS0:bih`020:bih`<0
00001@2]^^<00`000000[K_T0:fki002000000030:jkh`000000[[cT00@0[[cT00<000000:jmi02^
_N@00P0000020:nmi0030000002__N@0[kgT00<0[kgT0`0000040;2ni@030000002`_nD0/;oU00<0
00000P2a_nD00`000000/L3U0;70i@020;70i@<00000102b`NH00`000000//;V0;;2iP0300000080
//;V00<000000;?3iP2c`nH00P2c`nH3000000@0/lCV00<000000?ooo`3oool0:`3oool00`000000
oooo0?ooo`2=0?ooo`00T03oool200000<H0oooo0P0000020:Jbh@030000002V/^40Y[;Q00<0Y[;Q
00<000000:Jbh@2V/^400P2V/^43000000@0Y[;Q00<000000:Jbh@2V/^400P2V/^43000000@0Yk;Q
00<000000:Nbh@2W/^400`2W/^400`000000Yk;Q0:Nbh@02000000030:Nbh@000000Yk?Q00@0Yk?Q
00<000000:Nch@2W/n400P0000020:Rch@030000002X]>40Z;CQ00<0Z;CQ00<000000:Rdh@2X]>40
0P0000020:Vdh@030000002Y]N40ZKGQ00<0ZKGQ00<000000:VehP2Y]N800P0000000`2Z]N400000
0:ZfhP040:ZfhP030000002Z]n80Z[OR008000000P2[]n800`000000ZkOR0:^ghP030:^ghP030000
002[^>80ZkSR0080000000<0[;SR0000002/^N<0102/^N<00`000000[;WS0:bih`0200000080[K[S
00<000000:fjh`2]^^<0102]^^<2000000@0[[_S00<000000:jli02^_>@00P2^_>@3000000D0[kgT
00<000000:nmi02__N@00P0000000`2`_^@000000;2ni@050;2ni@80000000<0/KoU0000002a_nD0
0P2a_nD00`000000/L3U0;70i@02000000030;71i@000000//7V00D0//7V0P0000050;;2iP030000
002c`nH0/l?V008000000P2c`nH00`000000/lCV0;?4iP020;?4iP800000;@3oool00`000000oooo
0?ooo`2<0?ooo`00S@3oool300000<H0oooo0P0000040:Jbh@030000002V/^40Y[;Q00<0Y[;Q00<0
00000:Jbh@2V/^400P0000000`2V/^4000000:Jbh@040:Jbh@030000002V/^40Y[;Q008000000P2V
/^400`000000Yk;Q0:Nbh@030:Nbh@030000002W/^40Yk;Q0080Yk;Q0`0000040:Nbh@030000002W
/n40Yk?Q0080Yk?Q0`0000040:Rch@030000002X]>40Z;CQ00<0Z;CQ0`0000040:Vdh@030000002Y
]N40ZKGQ00<0ZKGQ0`0000030:Zeh@030000002Z]^80Z[KR00<0Z[KR0`0000040:^ghP030000002[
]n80ZkOR00<0ZkOR0`0000040:bhhP030000002/^N<0[;WS0080[;WS0`0000040:fjh`030000002]
^^<0[K[S0080[K[S0`0000050:jkh`030000002^_>@0[[cT008000000P2__><00`000000[kgT0:nm
i0030:nmi0<00000102`_^@010000000/;kU0;2ni@2`_^D2000000040;6oi0000000/KoU000000@0
/KoU0P0000040;71i@040000002b`NH0//7V0;;1iP80000000<0//;U0000002b`^H01@2b`^H20000
00@0/l?V00<000000;?4iP2ca>H00P0000020;C4iP030000003oool0oooo02/0oooo00<000000?oo
o`3oool0R`3oool008/0oooo0P0000370?ooo`80000000<0Y[;Q0000002V/^40102V/^4010000000
Y[;Q0:Jbh@2V/^44000000@0Y[;Q00<000000:Jbh@2V/^400P2V/^43000000@0Y[;Q00<000000:Nb
h@2W/^400`2W/^400`000000Yk;Q0:Nbh@02000000030:Nbh0000000Yk;Q00@0Yk;Q00<000000:Nc
h@2W/n400P0000020:Rch@030000002X/n40Z;?Q00<0Z;?Q00<000000:Rdh@2X]>400P0000020:Rd
h@030000002Y]>40ZKCQ00<0ZKCQ00<000000:Veh@2Y]N400P0000000`2Y]N4000000:Zeh@040:Ze
h@030000002Z]^80Z[KR008000000P2[]^400`000000ZkOR0:^ghP030:^ghP030000002[]n80ZkOR
0080000000<0[;SR0000002/^>80102/^>800`000000[;WS0:bih`0200000080[KWR00<000000:fj
h`2]^^<00`2]^^<010000000[K[S0000000000030:jkh`030000002^^n<0[[_S0080[[_S0`000005
0:nlh`030000002__N@0[kgT0080000000<0/;gT0000002`_^@0102`_^@010000000/;kU00000000
00030;6oi0030000002a_nD000000080/KoU0`0000050;71i@040000002b`NH00000000000<0//;U
00<000000;;2iP2b`^H00P2b`^H3000000H0/l?V0P0000050;C4iP030000003oool0oooo02X0oooo
00<000000?ooo`3oool0R`3oool008P0oooo0`0000360?ooo`<000000`2V/^400`000000Y[;Q0:Jb
h@030:Jbh@@000000`2V/^400`000000Y[;Q0:Jbh@020:Jbh@030000002V/^40Y[;Q008000000P2V
/^400`000000Y[;Q0:Jbh@030:Jbh@040000002W/^40Yk;Q0:Nbh@@00000102W/^000`000000Yk;Q
0:Nbh@020:Nbh@<00000102X/n400`000000Z;?Q0:Rch@030:Rch@<00000102X]>400`000000ZKCQ
0:Vdh@030:Vdh@<00000102Y]N400`000000Z[GQ0:Zeh@020:Zeh@<00000102[]^400`000000ZkOR
0:^ghP030:^ghP<00000102/^>800`000000[;SR0:bhhP020:bhhP<00000102]^N800`000000[K[S
0:fjh`030:fjh`8000001@2^^n<00`000000[[_S0:jkh`0200000080[k_S00<000000:nlh`2__><0
102__><2000000@0/;gT00<000000;2ni02`_^@00P2`_^@2000000H0/KoT00@000000;6oi@000000
00000`2a`>D00`000000/L7U0;71i@020;71i@8000001P2b`^D010000000//;V0000000000030;?3
i@040000002c`nH0/l?V0;?3iP80000000<0]<CV0000002da>H01@2da>H00`000000oooo0?ooo`0Z
0?ooo`030000003oool0oooo08X0oooo00240?ooo`040000003oool0000000000<L0oooo0P000000
0`2V/^4000000:Jbh@040:Jbh@030000002V/^40Y[;Q008000000P2V/^400`000000Y[;Q0:Jbh@02
0:Jbh@030000002V/^40Y[;Q00<0Y[;Q0P0000040:Jbh@030000002V/^40Y[;Q00<0Y[;Q10000003
0:Nbh0030000002W/^00Yk;P0080Yk;P00<000000:Nbh@2W/^400P0000020:Nbh0030000002X/n40
Z;?Q00<0Z;?Q00<000000:Rch@2X/n400P0000020:Rch0030000002X]>40Z;CQ00<0Z;CQ00<00000
0:Vdh@2Y]>400P0000000`2Y]>4000000:Veh@040:Veh@030000002Z]N40Z[GQ008000000P2Z]^40
0`000000ZkKQ0:^fh@030:^fh@030000002[]n80ZkOR0080000000<0ZkOQ0000002/^>80102/^>80
0`000000[;SR0:bhhP0200000080[;SR00<000000:fihP2]^N800`2]^N800`000000[K[S0:fjh`02
00000080[[[R00<000000:jkh`2^^n<00P2^^n<3000000@0[k_S00<000000:nlh`2__><00P2__><3
000000D0/;gT00<000000;2ni02`_^@00P0000020;2ni0030000002a_n@0/KoT00<0/KoT0`000004
0;70i@030000002a`ND0/L7U008000000P2b`ND00`000000//;U0;;2i@030;;2i@8000001@2c`nD0
10000000/l?V0000000000030;C4iP030000002da>H0]<CV0080]<CV0P0000000`2eaNH000000?oo
o`0[0?ooo`030000003oool0oooo08T0oooo00230?ooo`<00000a@3oool4000000@0Y[;Q00<00000
0:Jbh@2V/^400P2V/^43000000@0Y[;Q00<000000:Jbh@2V/^400`2V/^400`000000Y[;Q0:Jbh@02
000000030:Jbh@000000Y[;Q00@0Y[;Q00<000000:Jbh@2V/^400P0000020:Nbh0030000002W/^00
Yk;P0080Yk;P00<000000:Nbh02W/^000`2W/^02000000@0Yk;P00<000000:Rch@2X/n400`2X/n43
000000@0Z;?P00<000000:Rdh@2X]>400`2X]>43000000@0ZKCQ00<000000:Veh@2Y]N400P2Y]N43
000000@0Z[KQ00<000000:^fh@2[]^400`2[]^43000000@0ZkOQ00<000000:bhhP2/^>800P2/^>83
000000@0[;SR00<000000:fihP2]^N800`2]^N83000000@0[[[R00<000000:jkh`2^^n<00P000002
0:jkh`030000002_^n<0[k_S00<0[k_S00@000000:nlh`00000000000`2__><00`000000/;gT0;2m
i0020;2mi0<000001@2`_^@00`000000/KoT0;6oi00200000080/KoT00<000000;70i@2a`>D00P2a
`>D3000000D0//7U00<000000;;2i@2b`^D00P0000020;?2i@030000002c`nD0/l?U0080/l?U0P00
00060;C4iP040000002da>H00000000000<0]LGV00<000000?ooo`3oool0:P3oool00`000000oooo
0?ooo`290?ooo`00P03oool3000000<0oooo00<000000?ooo`3oool0_P3oool400000080Y[?Q00<0
00000:Jbh@2V/^400`2V/^400`000000Y[;Q0:Jbh@0200000080Y[;Q00<000000:Jbh@2V/^400`2V
/^4010000000Y[;Q0:Jbh@2V/^44000000<0Y[;Q00<000000:Jbh@2V/^400`2V/^43000000@0Yk;P
00<000000:Nbh02W/^000`2W/^000`000000Yk;P0:Nbh002000000030:Nbh0000000Yk;P00@0Yk;P
00<000000:Rch@2X/n400P0000020:Rch0030000002X/n00Z;?P00<0Z;?P00<000000:Rdh@2X]>40
0P0000000`2Y]>0000000:Vdh@040:Vdh@030000002Y]N40ZKGQ008000000P2Z]N400`000000Z[KQ
0:Zfh@030:Zfh@030000002[]^40ZkKQ0080000000<0ZkKQ0000002[]n40102[]n400`000000[;SR
0:bhhP0200000080[;SQ00<000000:bhhP2/^>800`2/^>800`000000[KWR0:fihP02000000030:fi
hP000000[[[R00D0[[[R0P0000040:jkh`030000002_^n<0[k_S00<0[k_S0P0000050:nlh`030000
002`_N@0/;gT008000000P2`_N<00`000000/;kT0;2ni0030;2ni0<00000102a_n@00`000000/L3U
0;70i@0200000080//3T00<000000;;1i@2b`ND0102b`ND2000000@0/l;U00<000000;?3i@2c`nD0
0P0000020;C3i@030000002da>H0]<CV00<0]<CV0P0000060;G5iP030000003oool0oooo02X0oooo
0P0000290?ooo`00OP3oool2000000L0oooo00<000000?ooo`3oool0^`3oool200000080Y[?R00<0
00000:Jch@2V/n400P2V/n400`000000Y[;Q0:Jbh@020:Jbh@<00000102V/^400`000000Y[;Q0:Jb
h@030:Jbh@@000000P2V/^400`000000Y[;Q0:Jbh@030:Jbh@030000002V/^40Y[;Q008000000P2V
/N000`000000Yk;P0:Nbh0030:Nbh0040000002W/^00Yk;P0:Nbh0@000000`2W/^000`000000Yk;P
0:Nbh0030:Nbh0<00000102X/n000`000000Z;?P0:Rch0030:Rch0<00000102Y]>000`000000ZKCQ
0:Vdh@020:Vdh@<00000102Z]N400`000000Z[KQ0:Zfh@030:Zfh@<00000102[]^400`000000ZkOQ
0:^gh@020:^gh@<00000102/^>400`000000[;SR0:bhhP030:bhhP<00000102]^N8010000000[[[R
0:jjhP2^^^82000000030:jjhP000000[[_S00@0[[_S00<000000:nkh`2_^n<00P0000020:nlh`03
0000002__><0[kcS0080[kcS0`0000050;2mh`030000002`_^@0/;kT0080000000<0/KkT0000002a
_n@0102a_n@3000000@0//3T00<000000;;1i@2b`ND00P2b`ND3000000H0/l;U0P0000040;C3i@03
0000002da>H0]<CV00<000000P2ea>D010000000]LGV0;G5iP2eaNH3000002X0oooo0P0000000`3o
ool000000?ooo`280?ooo`00N`3oool3000000X0oooo00<000000?ooo`3oool0]`3oool3000000@0
Y[?R00<000000:Jch@2V/n400P2V/n400`000000Y[;Q0:Jbh@0200000080Y[;Q00<000000:Jbh@2V
/^400`2V/^400`000000Y[;Q0:Jbh@02000000030:Jbh@000000Y[;Q00@0Y[;Q00<000000:Jbh@2V
/^400P2V/^43000000@0Y[7P00<000000:Nbh02W/^000`2W/^0400000080Yk;P00<000000:Nbh02W
/^000`2W/^000`000000Yk;P0:Nbh00200000080Z;;P00<000000:Rch02X/n000`2X/n000`000000
Z;?P0:Rch002000000030:Vch0000000ZKCP00@0ZKCP00<000000:Vdh@2Y]>400P0000020:Zdh003
0000002Z]N40Z[GQ00<0Z[GQ00<000000:Zfh@2Z]^400P0000000`2[]^4000000:^fh@040:^fh@03
0000002[]n40ZkOQ008000000P2/]n400`000000[;SQ0:bhh@030:bhh@030000002/^>80[;SR0080
000000<0[KSR0000002]^N80102]^N8010000000[[[R0000000000030:jjhP030000002^^n<0[[_S
00<0[[_S0`0000040:nlh`030000002__><0[kcS008000000P2`_N<00`000000/;gS0;2mh`040;2m
h`800000102a_^@010000000/KoT0;6oi02a_n@2000000030;:oi0000000//3T00@0//3T00@00000
0;;1i@00000000000`2c`N@010000000/l;U0;?2i@2c`^D3000000H0]<?U0P0000050;G4i@040000
002eaNH00000000000<0]/GV00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`270?ooo`00
N@3oool200000<D0oooo0P0000020:JdhP030000002V/n80Y[?R00<0Y[?R00@000000:Jch@2V/n40
Y[?Q0`0000040:Jbh@030000002V/^40Y[;Q00<0Y[;Q0`0000040:Jbh@030000002V/^40Y[;Q0080
Y[;Q00<000000:Jbh@2V/^400P0000020:Jah0030000002V/N00Y[7P00<0Y[7P00<000000:Nbh02W
/^000P0000000`2W/^0000000:Nbh0040:Nbh0030000002W/^00Yk;P0080Yk;P0`0000040:Rbh003
0000002X/n00Z;?P00<0Z;?P0`0000040:Vch0030000002Y]>00ZKCP0080ZKCP0`0000040:Zdh003
0000002Z]N40Z[GQ00<0Z[GQ0`0000040:^fh@030000002[]^40ZkKQ0080ZkKQ0`0000040:bgh@03
0000002/^>40[;SQ00<0[;SQ0`0000040:fhhP030000002]^N80[KWR0080[KWR0P0000060:jjhP03
0000002^^n<0[[_S0080000000<0[k_R0000002__><01@2__><2000000@0/;gS00<000000;2mh`2`
_N<00P2`_N<3000000D0/KkT00P000000;6oi000000000000;:oi0000000/[oT000000D0//3T0P00
00050;?1i0040000002c`^D00000000000<0]<;U00@000000;C3i@2d`nD0]<?U0P0000000`2d`nD0
00000;G4i@040;G4i@8000001@2faNH00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo08H0
oooo001f0?ooo`<00000a03oool3000000@0Y[CR00<000000:JchP2V/n800`2V/n8400000080Y[;Q
00<000000:Jbh@2V/^400`2V/^400`000000Y[;Q0:Jbh@0200000080Y[;Q00<000000:Jbh@2V/^40
0P2V/^400`000000Y[;Q0:Jbh@020:Jbh@<00000102V/N000`000000Y[7P0:Jah0030:Jah0<00000
102W/^000`000000Yk;P0:Nbh0020:Nbh0030000002W/^00Yk;P008000000P2W/^000`000000Z;;P
0:Rbh0030:Rbh0030000002X/n00Z;?P0080000000<0Z;?P0000002Y/n00102Y/n000`000000ZKCP
0:Vdh00200000080ZKCP00<000000:Zdh02Z]>000`2Z]>000`000000Z[GQ0:Zeh@02000000030:^e
h0000000ZkKQ00@0ZkKQ00<000000:^fh@2[]^400P0000020:bfh@030000002/]n40[;OQ00<0[;OQ
00<000000:bhh@2/^>400P0000000`2]^>4000000:fhhP040:fhhP030000002]^N80[KWR00800000
0P2^^N800`000000[[[R0:jjhP030:jjhP<00000102_^n8010000000[kcS0:nlh`2__><200000003
0;2lhP000000/;gS00@0/;gS00@000000;2mh`00000000000`2a_N<00`000000/KkT0;6ni0020;6n
i08000000`2b_n@010000000/[oT0;:oi00000020;;0i08000000P2c`>@00`000000/l7T0;?1i002
0;?1i08000001@2d`^D010000000]<?U0000000000040;C3i@030000002ea>D0]LCU008000000P2f
a>D00`000000]/GV0;K5iP020;K5iP800000;@3oool00`000000oooo0?ooo`250?ooo`00M03oool2
00000<D0oooo0P0000020:Jdh`030000002V]>80Y[CR00<0Y[CR00<000000:JchP2V/n800P000000
0`2V/n8000000:Jbh@030:Jbh@030000002V/^40Y[;Q00<0Y[;Q0`0000040:Jbh@030000002V/^40
Y[;Q00<0Y[;Q00@000000:Jbh@00000000000P2V/N000`000000Y[7P0:Jah0030:Jah0030000002V
/N00Y[7P0080000000<0Y[7P0000002W/^00102W/^000`000000Yk;P0:Nbh0020:Nbh0<00000102W
/^000`000000Z;;P0:Rbh0030:Rbh0<00000102X/n000`000000ZK?P0:Vch0020:Vch0<00000102Y
]>000`000000Z[CP0:Zdh0030:Zdh0<00000102[]N000`000000ZkKQ0:^fh@020:^fh@<00000102/
]^400`000000[;OQ0:bgh@030:bgh@<00000102]^>400`000000[KSR0:fhhP020:fhhP<000001@2^
^N800`000000[[[R0:jjhP02000000030:njhP000000[k_R00@0[k_R00@000000:nlh`0000000000
0`2`_>800`000000/;gS0;2mh`030;2mh`8000001@2a_N<00`000000/KkT0;6ni00200000080/[kS
00@000000;:oi02b_n@000000P2b_n@3000000@0/l3T00<000000;?1i02c`N@00P0000020;C1i003
0000002d`^D0]<;U0080]<;U0P0000070;C3i@800000102fa>D00`000000]/GV0;K5iP0200000080
]/GU00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`250?ooo`00L@3oool300000<@0oooo
0`0000040:Jdh`030000002V]>80Y[CR00<0Y[CR0`0000040:JchP030000002V/^40Y[;Q0080Y[;Q
00<000000:Jbh@2V/^400P0000020:Fbh@030000002V/^40Y[;Q00<0Y[;Q00@000000:Jbh@2V/^40
Y[;Q0`0000040:Jah0030000002V/N00Y[7P00<0Y[7P0`0000040:Jah0030000002W/^00Yk;P00<0
Yk;P00@000000:Nbh000000000000P2W/^000`000000Yk;P0:Nbh0030:Nbh0030000002X/^00Z;;P
0080000000<0Z;;P0000002X/n00102X/n000`000000ZK?P0:Vch00200000080ZK?P00<000000:Vd
h02Y]>000`2Y]>000`000000Z[CP0:Zdh002000000030:Zdh0000000ZkGP00@0ZkGP00<000000:^f
h@2[]^400P0000020:bfh0030000002/]^40[;KQ00<0[;KQ00<000000:bgh@2/]n400P0000000`2]
]n4000000:fhh@040:fhh@030000002]^>80[KSR008000000P2^^N400`000000[[WR0:jihP040:ji
hP800000102_^^800`000000[k_R0:nkhP020:nkhP8000001P2`_>800`000000/;gS0;2mh`020000
0080/KgS00<000000;6mh`2a_N<00P2a_N<3000000D0/[kS00<000000;:oi00000000P0000020;>o
i0030000002c`>@0/l3T0080/l3T0`0000040;C1i0030000002d`^D0]<;U008000000P2d`^@00`00
0000]<?U0;C3i@020;C3i@<000001P2fa>D2000000@0]/GU00<000000?ooo`3oool0:`3oool00`00
0000oooo0?ooo`240?ooo`00KP3oool300000<<0oooo100000020:Jeh`030000002V]><0Y[CS00<0
Y[CS00<000000:JdhP2V]>800P0000020:JchP030000002V/n80Y[?R0080Y[?R00@000000:Jbh@2V
/^40Y[;Q100000040:Fbh@030000002V/^40Y[;Q00<0Y[;Q100000020:Jah@030000002V/N00Y[7P
00<0Y[7P00<000000:Jah02V/N000P0000020:Jah0030000002V/N00Y[7P0080Y[7P00@000000:Nb
h02W/^00Yk;P100000040:Nbh0030000002W/^00Yk;P00<0Yk;P0`0000040:Rbh0030000002X/n00
Z;?P0080Z;?P0`0000040:Vch0030000002Y]>00ZKCP00<0ZKCP0`0000040:Zdh0030000002[]N00
ZkGP0080ZkGP0`0000040:bfh0030000002/]^40[;KQ00<0[;KQ0`0000040:fgh@030000002]^>40
[KSQ0080[KSQ0`0000040:jih@030000002^^N80[[WR0080[[WR0`0000050:njhP030000002_^n80
[k_R008000000P2`^n800`000000/;cR0;2lhP030;2lhP<00000102a_N<00`000000/KgS0;6mh`02
00000080/[kS00<000000;:nh`2b_^<0102b_^<2000000@0/koT00<000000;?0i02c`>@00P000002
0;?1i0030000002d`N@0]<7T0080]<7T0`0000050;C2i0040000002d`nD00000000000<0]L?U00@0
00000;K4i@2fa>D0]/CU0`0000060;K5i@030000003oool0oooo02/0oooo00<000000?ooo`3oool0
P`3oool006`0oooo0P0000030?ooo`800000_`3oool2000000030:Nfi0000000Y[GS00<0Y[GS00<0
00000:Jdh`2V]><00`2V]><3000000@0Y[?R00<000000:JchP2V/n800`2V/n83000000<0YK;Q00<0
00000:Fbh@2U/^400`2U/^400`000000Y[;Q0:Jbh@02000000030:Fah@000000Y[7Q00<0Y[7Q00<0
00000:Jah02V/N000`2V/N03000000@0Y[7P00<000000:Jah02V/N000`2V/N03000000<0Yk7P00<0
00000:Nbh02W/^000`2W/^000`000000Yk;P0:Nbh002000000030:Rbg`000000Z;;P00@0Z;;P00<0
00000:Rch02X/n000P0000020:Vcg`030000002Y/n00ZK?P00<0ZK?P00<000000:Vdh02Y]>000P00
00000`2Z]>0000000:Zdh0040:Zdh0030000002[]N00ZkGP008000000P2[]N000`000000[;KP0:bf
h0030:bfh0030000002/]^40[;KQ0080000000<0[;KP0000002]]n40102]]n400`000000[KSQ0:fh
h@0200000080[[SQ00<000000:jih@2^^N400`2^^N4010000000[[WR0000000000030:nih@030000
002_^^80[k[R0080[k[R0`0000050;2khP030000002`_>80/;cR0080000000<0/KcR0000002a_N<0
1@2a_N<2000000@0/[kS00<000000;:nh`2b_^<00P2b_^<4000000D0/koT0P0000040;?1i0030000
002d`N@0]<7T008000000P2d`^@00`000000]<;T0;C2i0030;C2i08000001@2e`nD010000000]/CU
0000000000030;K4i@040000002faND0]/GU0;K5i@80000000<0]lGU0000003oool0:`3oool00`00
0000oooo0?ooo`230?ooo`00J@3oool300000<<0oooo0`0000040:Nfi0030000002V]N<0Y[GS0080
Y[GS00<000000:Jdh`2V]><00P0000020:Jdh`030000002V/n80Y[?R00<0Y[?R00<000000:JchP2V
/n800P0000000`2U/^8000000:Fbh@030:Fbh@030000002U/^40YK;Q00<0YK;Q0`0000040:Fah@03
0000002V/N40Y[7Q0080Y[7Q00<000000:Jah02V/N000P0000020:Jah0030000002V/N00Y[7P00<0
Y[7P00<000000:Jah02V/N000P0000000`2V/N0000000:Nah0030:Nah0030000002W/^00Yk;P00<0
Yk;P0`0000040:Rbg`030000002X/^00Z;;P0080Z;;P0`0000040:Vcg`030000002Y/n00ZK?P00<0
ZK?P0`0000040:Zdh0030000002Z]>00Z[CP0080Z[CP0`0000040:^eh0030000002/]^00[;KP00<0
[;KP0`0000040:bfh0030000002]]n40[KOQ0080[KOQ0`0000040:jhh@030000002^^N40[[WQ00<0
[[WQ0P0000050:nih@030000002_^^80[k[R008000000P2`^^800`000000/;_R0;2khP040;2khP80
0000102a_>8010000000/KgS0;6mh`2a_N<2000000030;6mhP000000/[kS00@0/[kS00@000000;:n
h`00000000000`2c_n<010000000/koT0;>oi02c_n@3000000H0/l7T0P0000050;C2i0030000002d
`^@0]<;T008000000P2e`n@00`000000]L?U0;G3i@020;G3i@8000001@2fa>D010000000]/GU0000
000000040;O5i@030000003oool0oooo02X0oooo00<000000?ooo`3oool0PP3oool006L0oooo0P00
00340?ooo`8000000P2W]n@00`000000YkKT0:Nfi0020:Nfi0040000002V]N<0Y[GS0:Jeh`@00000
102V]><00`000000Y[?R0:JchP030:JchP<000000`2U/^800`000000YK;Q0:Fbh@030:Fbh@030000
002U/^40YK;Q0080000000<0YK;Q0000002U/N40102U/N4010000000Y[7Q0:Jah@2V/N44000000@0
Y[7P00<000000:Jah02V/N000`2V/N03000000<0Y[7P00<000000:Nah02W/N000`2W/N000`000000
Yk;P0:Nbh00200000080Yk7O00<000000:Rbg`2X/]l00P2X/]l00`000000Z;;P0:Rbh00200000080
Z;;O00<000000:Vcg`2Y/ml00`2Y/ml00`000000ZK?P0:Vch002000000030:Vcg`000000Z[CP00@0
Z[CP00<000000:Zdh02Z]>000P0000020:^dg`030000002[]N00ZkGP00<0ZkGP00<000000:bfh02/
]^000P0000000`2/]^0000000:bfh0040:bfh0030000002]]n40[KOQ008000000P2]]n000`000000
[[SQ0:jhh@030:jhh@030000002^^N40[[WQ008000000P2^^N400`000000[kWQ0:nih@020:nih@<0
0000102`^^800`000000/;_R0;2khP020;2khP<000001@2a_>8010000000/KgS0000000000030;6m
hP030000002b_^<0/[kS00<0/[kS0P0000050;>oh`040000002c_n@00000000000<0/l3S00@00000
0;?1i02c`N@0/l7T0P0000000`2d`N<000000;C2i0050;C2i0800000102e`n@00`000000]L?U0;G3
i@0200000080]/CT00<000000;K4i@2fa>D00P2fa>D2000000L0]lGU00<000000?ooo`3oool0:P3o
ool00`000000oooo0?ooo`210?ooo`00I03oool300000<<0oooo0`0000040:Ngi0030000002W]^@0
YkKT00<0YkKT0`0000030:Jdh`030000002V]><0Y[CS00<0Y[CS00<000000:JchP2V/n800P000000
0`2U/n8000000:FbhP040:FbhP040000002U/^40YK;Q0:Fbh@@00000102U/^400`000000YK7Q0:Fa
h@020:Fah@@000000`2U/N000`000000Y[7P0:Jah0030:Jah0030000002V/N00Y[7P0080000000<0
Y[7P0000002V/N00102V/N000`000000Yk7P0:Nah0020:Nah0<00000102W/Ml00`000000Z;;O0:Rb
g`030:Rbg`800000102X/]l00`000000ZK?O0:Vcg`030:Vcg`<00000102Y/ml00`000000Z[CP0:Zd
h0020:Zdh0<00000102[]=l00`000000ZkGP0:^eh0030:^eh0<00000102/]^000`000000[;KP0:bf
h0020:bfh0<00000102]]n000`000000[[SQ0:jhh@030:jhh@<00000102^^N400`000000[kWQ0:ni
h@0200000080[k[Q00<000000;2jhP2`^^800`2`^^8010000000/;_R0000000000030;2khP030000
002a_>80/KcR0080/KcR0P0000060;6mhP030000002b_^<0/[kS008000000P2b_^<00`000000/koS
0;>oh`020;>oh`8000001@2c`><010000000/l7T0000000000030;C1h`030000002d`^@0]<;T0080
]<;T0`0000060;G3i0800000102fa>@00`000000]/CU0;K4i@0200000080]lGU00<000000;O5i@2g
aND00P2gaND3000002`0oooo00<000000?ooo`3oool0P@3oool00680oooo0P0000340?ooo`800000
0P2W]nD00`000000YkOT0:Ngi0030:Ngi0030000002W]^@0YkKT0080000000<0Y[GT0000002V]><0
0`2V]><00`000000Y[CS0:Jdh`030:Jdh`<00000102U/n800`000000YK;R0:FbhP020:FbhP@00000
0`2U/^400`000000YK;Q0:Fbh@020:Fbh@030000002U/N40YK7Q008000000P2U/N400`000000YK7P
0:Fah0020:Fah0030000002V/N00Y[7P00<0Y[7P0`0000040:Jah0030000002V/N00Y[7P0080Y[7P
00<000000:Nah02W/N000P0000020:Nag`030000002W/Ml0Yk7O00<0Yk7O00<000000:Rbg`2X/]l0
0P0000000`2X/]l000000:Rbg`040:Rbg`030000002Y/ml0ZK?O008000000P2Y/ml00`000000ZK?O
0:Vcg`020:Vcg`030000002Z]>00Z[CP008000000P2[]=l00`000000ZkCO0:^dg`030:^dg`030000
002[]N00ZkGP0080000000<0[;GO0000002/]^00102/]^000`000000[;KP0:bfh00200000080[KOP
00<000000:fgh02]]n000`2]]n000`000000[[SQ0:jhh@02000000030:jhh0000000[[WQ00D0[[WQ
0P0000040:njh@030000002`^^80/;[R00<0/;[R0P0000050;2khP030000002a_>80/KcR00800000
0P2a_>800`000000/KgR0;6mhP030;6mhP<00000102b_^<00`000000/koS0;>oh`03000000030;>o
h`000000/l3S00<0/l3S0P0000060;C1h`040000002d`^@00000000000<0]L;T00@000000;G3i02e
`n@0]L?T0`0000060;K4i08000001@2gaND010000000]lGU0000000000030;S6i@030000003oool0
oooo02X0oooo00<000000?ooo`3oool0P03oool005l0oooo0`0000340?ooo`800000102W]nD00`00
0000YkOT0:Ngi0030:Ngi0<000000`2V]N@00`000000Y[CS0:Jdh`030:Jdh`030000002V]><0Y[CS
008000000P2U/n<00`000000YK?R0:FchP020:FchP030000002U/^80YK;R008000000P2U/^800`00
0000YK;Q0:Fbh@020:Fbh@030000002U/^40YK;Q00<0YK;Q0P0000040:Fah@030000002U/N00YK7P
00<0YK7P00<000000:Jah02V/N000P0000000`2V/N0000000:Jah0040:Jah0040000002V/N00Y[7P
0:Jah0@00000102W/Ml00`000000Yk7O0:Nag`030:Nag`<000000`2X/]l00`000000Z;;O0:Rbg`03
0:Rbg`<00000102Y/ml00`000000ZK?O0:Vcg`030:Vcg`800000102[]=l00`000000ZkCO0:^dg`03
0:^dg`<00000102/]Ml00`000000[;KP0:bfh0020:bfh0<00000102]]n000`000000[KOP0:fgh003
0:fgh0<00000102^^>0010000000[[WQ0:jih@2^^N42000000030:nih@000000[k[Q00@0[k[Q00<0
00000;2jhP2`^^800P0000020;2kh@030000002`^n80/;_R0080/;_R0`0000050;6lhP030000002a
_N80/KgR0080000000<0/[gR0000002b_^<01@2b_^<200000080/koS00<000000;>oh`0000000P2c
`><200000080]<3S00<000000;C1h`2d`N<00`2d`N<2000000D0]L;T00@000000;G3i00000000000
0`2f`n@010000000]/CT0;K4i02fa>@2000000030;O4i0000000]lGU00@0]lGU0P0000050;S6i@03
0000003oool0oooo02/0oooo00<000000?ooo`3oool0O`3oool005d0oooo0P0000350?ooo`800000
00<0YkSV0000002W]nD0102W]nD00`000000YkOT0:Ngi002000000030:Jfi0000000Y[GT00@0Y[GT
00@000000:Jdh`2V]><0Y[CS100000040:Fch`030000002U/n80YK?R00<0YK?R0P0000040:FbhP03
0000002U/^40YK;Q00<0YK;Q00<000000:Fbh@2U/^400P0000000`2U/N4000000:Fah@040:Fah@04
0000002U/N00YK7P0:Fah0@00000102V/N000`000000Y[7P0:Jah0020:Jah0@000000`2V/Ml00`00
0000Yk7O0:Nag`030:Nag`030000002W/Ml0Yk7O0080000000<0Z;7O0000002X/]l0102X/]l00`00
0000Z;;O0:Rbg`0200000080ZK;O00<000000:Vcg`2Y/ml00`2Y/ml00`000000ZK?O0:Vcg`020000
00030:Zcg`000000ZkCO00@0ZkCO00<000000:^dg`2[]=l00P0000000`2[]=l000000:beg`040:be
g`030000002/]^00[;KP008000000P2]]]l00`000000[KOP0:fgh0030:fgh0030000002]]n00[KOP
0080000000<0[[OP0000002^^>00102^^>0010000000[[WQ0000000000030:nih@030000002_^^40
[k[Q00<0[k[Q0`0000040;2kh@030000002`^n80/;_R008000000P2a^n400`000000/KcR0;6lhP04
0;6lhP800000102b_N8010000000/[kS0;:nh`2b_^<3000000@0/koS00@000000;>oh`0000000000
1@2d`><00`000000]<7S0;C1h`0200000080]L7S00<000000;G2i02e`^@00P2e`^@2000000D0]/?T
00@000000;K4i00000000000102ga>@00`000000]lGU0;O5i@0200000080^<GT00<000000;S6i@2h
a^D00P2ha^D2000002`0oooo00<000000?ooo`3oool0O`3oool005X0oooo0`0000330?ooo`@00000
102W^>H010000000YkOU0:Ngi@2W]nD4000000@0Y[KT00<000000:Jei02V]N@00P2V]N@4000000<0
YKCS00<000000:Fch`2U/n<00`2U/n<00`000000YK?R0:FchP02000000030:BbhP000000YK;R00@0
YK;R00@000000:Fbh@2U/^40YK;Q100000030:Fah@030000002U/N40YK7Q00<0YK7Q100000030:Fa
h0030000002V/N00Y[7P0080Y[7P00<000000:Jah02V/N000P0000020:Jag`030000002V/Ml0Y[7O
0080Y[7O00<000000:Nag`2W/Ml00`2W/Ml3000000@0Z;7O00<000000:Rbg`2X/]l00P2X/]l30000
00@0ZK;O00<000000:Vcg`2Y/ml00`2Y/ml3000000<0Z[?O00<000000:^dg`2[]=l00`2[]=l30000
00@0ZkCO00<000000:beg`2/]Ml00P2/]Ml3000000@0[KKO00<000000:fgh02]]n000`2]]n030000
00@0[[OP00<000000:jhh02^^>000P2^^>02000000H0[kWQ00<000000:njh@2_^^400P0000000`2`
^^4000000;2kh@050;2kh@800000102a^n400`000000/KcR0;6lhP020;6lhP<000001@2b_N801000
0000/[kS0000000000030;>nhP040000002c_n<0/koS0;>oh`80000000<0];oR0000002d`><01@2d
`><2000000@0]L7S00<000000;G2i02e`^@00P0000020;K2h`030000002f`n@0]/?T0080]/?T0P00
00070;O4i0800000102haN@00`000000^<KU0;S6i@0200000080^<KU00<000000?ooo`3oool0:P3o
ool00`000000oooo0?ooo`1n0?ooo`00E`3oool300000080oooo0P00002n0?ooo`@000000`2W^NH0
0`000000YkSV0:NhiP020:NhiP@000000`2V]nD00`000000Y[KT0:Jfi0020:Jfi0030000002V]N@0
Y[GT008000000P2U]N@00`000000YKCS0:Fdh`030:Fdh`040000002U/n<0YK?S0:Fch`@000000`2T
/^800`000000YK;R0:FbhP030:FbhP@000000P2U/N400`000000YK7Q0:Fah@030:Fah@030000002U
/N40YK7Q008000000P2U/N000`000000YK7P0:Fah0020:Fah0030000002V/N00Y[7P00<0Y[7P0P00
00040:Jag`030000002V/Ml0Y[7O00<0Y[7O00<000000:Nag`2W/Ml00P0000000`2W/Ml000000:Ra
g`040:Rag`030000002X/]l0Z;;O008000000P2X/Ml00`000000ZK;O0:Vbg`030:Vbg`030000002Y
/ml0ZK?O0080000000<0ZK;O0000002Z/ml0102Z/ml00`000000ZkCO0:^dg`0200000080ZkCO00<0
00000:^dg`2[]=l00P2[]=l00`000000[;GO0:beg`0200000080[;GO00<000000:ffg`2]]]l00`2]
]]l00`000000[KOP0:fgh002000000030:jgg`000000[[OP00@0[[OP00<000000:jhh02^^>000P00
00020:nhh0030000002_^N40[kWQ00<0[kWQ0`0000040;2jh@040000002`^n40/;_Q0;2kh@800000
00<0/K_Q0000002a^n40102a^n4010000000/KcR0000000000030;:lhP030000002b_N80/[gR0080
/[gR0P0000050;>nhP030000002c_n<00000008000000P2d_n800`000000]<3S0;C0h`020;C0h`<0
00001P2e`N<2000000@0]/;S00<000000;K3i02f`n@00P0000020;O3i0030000002ga>@0]lCT0080
]lCT0`0000060;S5i0800000102ha^D00`000000oooo0?ooo`0X0?ooo`80000000<0oooo0000003o
ool0OP3oool005D0oooo0P0000330?ooo`8000000P2W^^L00`000000YkWV0:NiiP020:NiiP030000
002W^>H0YkSV008000000P2V^>H00`000000Y[OU0:Jgi@020:Jgi@030000002V]^@0Y[KT00<0Y[KT
0P0000040:Fei0030000002U]><0YKCS00<0YKCS100000020:Bch`030000002T/^80Y;;R00<0Y;;R
00<000000:FbhP2U/^800P0000020:BbhP030000002U/N40YK7Q0080YK7Q00<000000:Fah@2U/N40
0P2U/N43000000@0YK7P00<000000:Fah02U/N000`2U/N000`000000Y[7P0:Jah002000000030:J`
h0000000Y[7O00@0Y[7O00@000000:Jag`2V/Ml0Y[7O100000040:Nag`030000002X/Ml0Z;7O0080
Z;7O0`0000040:Rag`030000002Y/]l0ZK;O00<0ZK;O0`0000040:Vbg`030000002Z/ml0Z[?O0080
Z[?O0`0000040:^dg`030000002[]=l0ZkCO00<0ZkCO0P0000040:beg`030000002]]]l0[KKO00<0
[KKO0`0000040:jgg`030000002^]n00[[OP0080[[OP0`0000050:nhh0030000002_^N40[kWQ0080
000000<0/;WP0000002`^^40102`^^4010000000/;_Q0000000000030;6kh@030000002a^n40/K_Q
00<0/K_Q0P0000050;:lhP030000002b_N80/[gR008000000P2c_N800`000000/kkR0;>nhP020;>n
hP8000000P2d_n801P000000];oR0;BohP2d_n8000000;C0h`8000000`2e`><010000000]L7S0;G1
h`2e`N<3000000H0]/;S0P0000050;O3i0040000002ga>@00000000000<0]lCT00@000000;S5i02h
aN@0^<GT0P0000000`2haN@000000;S6i@030;S6i@800000;03oool00`000000oooo0?ooo`1m0?oo
o`00DP3oool300000<<0oooo0P0000040:Nji`030000002W^NH0YkWV00<0YkWV0P0000040:JhiP03
0000002V]nD0Y[OU00<0Y[OU00<000000:Jfi02V]^@00P0000000`2U]^D000000:Fei0040:Fei003
0000002U]><0YKCS008000000P2T/n<00`000000Y;?S0:Bch`020:Bch`030000002T/^80Y;;R0080
Y;;R0`0000040:BbhP030000002U/N40YK7Q0080YK7Q00<000000:Fah@2U/N400P0000020:Fah@03
0000002U/N00YK7P00<0YK7P00<000000:Fah02U/N000P2U/N03000000<0Y[3P00<000000:Jag`2V
/Ml00`2V/Ml400000080Yk3O00<000000:Nag`2W/Ml00`2W/Ml00`000000Z;7O0:Rag`0200000080
Z;7O00<000000:Rag`2X/Ml00`2X/Ml00`000000ZK;O0:Vbg`02000000030:VbgP000000ZK;O00@0
ZK;O00<000000:Zcg`2Z/ml00P0000020:^cgP030000002[]=l0ZkCO00<0ZkCO00L000000:^dg`2[
]=l0ZkCO0000002/]=l0000000D0[;GO00<000000:ffg`2]]]l00P0000000`2]]]l000000:jgg`04
0:jgg`030000002^]n00[[OP008000000P2_]ml00`000000[kSP0:nhh0040:nhh0800000102`^N00
0`000000/;[Q0;2jh@020;2jh@8000001P2a^n400`000000/K_Q0;6kh@0200000080/[cQ00<00000
0;:lhP2b_>800P2b_>83000000@0/kgR00<000000;>nhP2c_^800P0000020;BnhP030000002d_n80
000000<0];oR0P0000050;G0h`040000002e`N<00000000000<0]/7S00@000000;K2h`2f`^<0]/;S
0P0000000`2g`^<000000;O3i0050;O3i0030000002ga>@0]lCT00<0]lCT00@000000;S5i0000000
00000`2haN@00`000000^<KU0;S6i@0200000080^LKT00<000000?ooo`3oool0:P3oool00`000000
oooo0?ooo`1l0?ooo`00D03oool200000<@0oooo0P0000000`2W_>P000000:Nji`040:Nji`030000
002W^NH0YkWV0080000000<0Y[WW0000002V^>H0102V^>H010000000Y[OU0:Jgi@2V]nD4000000<0
YKKU00<000000:Fei02U]N@00`2U]N@3000000@0Y;?S00<000000:Bch`2T/n<00P2T/n<00`000000
Y;;R0:BbhP0200000080Y;;R00<000000:BbhP2T/^800`2T/^8010000000YK7Q0:Fah@2U/N430000
00@0YK7Q00<000000:Fah02U/N000`2U/N000`000000YK7P0:Fah002000000030:F`h0000000Y[3P
00@0Y[3P00<000000:Jag`2V/Ml00P0000020:J`g`030000002W/=l0Yk3O0080Yk3O00<000000:Na
g`2W/Ml00P2W/Ml3000000@0Z;7O00<000000:Rag`2X/Ml00`2X/Ml3000000@0ZK;N00<000000:Vb
g`2Y/]l00P2Y/]l3000000@0Zk?N00<000000:^dg`2[]=l00`2[]=l010000000ZkCO000000000002
0:bdg`030000002/]Ml0[;GO00<0[;GO0`0000040:ffg`030000002^]ml0[[OO0080[[OO0`000004
0:ngg`030000002_^>00[kSP0080[kSP0`0000050;2ih0030000002`^^40/;[Q008000000P2a^^00
0`000000/K_Q0;6kh@030;6kh@<00000102b_>400`000000/[cR0;:lhP02000000030;>mh@000000
/kgR00D0/kgR0P0000050;BnhP80000000<0];oR0000000000000P2e_n800`000000]L3S0;G0h`02
0;G0h`8000001@2f`N<010000000]/;S0000000000030;O2h`030000002g`n@0]l?T0080]l?T0P00
00000`2g`n<000000;O4i0030;O4i08000001P2haN@2000000@0^LKT00<000000?ooo`3oool0:P3o
ool00`000000oooo0?ooo`1l0?ooo`00C@3oool300000<80oooo100000040:Nlj0040000002W^^L0
Yk[W0:Nji`@000000`2V^NL00`000000Y[SV0:JhiP030:JhiP@000000`2U]^D00`000000YKKU0:Ff
i@020:Ffi@030000002U]N@0YKGT008000000P2T]>@00`000000Y;?S0:Bch`030:Bch`040000002T
/n<0Y;?S0:Bch`<00000102T/^800`000000Y;;R0:BbhP030:BbhP@000000P2T/N400`000000YK7Q
0:Fah@030:Fah@040000002U/N00YK7P0:Fah0@00000102U/>000`000000Y[3P0:J`h0020:J`h0<0
0000102V/=l00`000000Yk3O0:N`g`020:N`g`030000002W/Ml0Yk7O008000000P2W/Ml00`000000
Z;7O0:Rag`030:Rag`030000002X/Ml0Z;7O0080000000<0ZK7N0000002Y/]h0102Y/]h00`000000
ZK;O0:Vbg`0200000080Z[;N00<000000:^cgP2[/mh00`2[/mh00`000000ZkCO0:^dg`020:^dg`80
00001@2/]=l00`000000[;GO0:beg`02000000030:feg`000000[KKO00@0[KKO00<000000:jgg`2^
]ml00P0000020:jgg`030000002_]ml0[kOO00<0[kOO00@000000:nhh000000000000`2_^=l00`00
0000/;WP0;2ih0020;2ih0<000001@2a^^000`000000/K_Q0;6kh@02000000030;:kh@000000/[cQ
00D0/[cQ0P0000040;>mh@040000002c_N80/kgR0;>mhP80000000<0];kQ0000002d_^801@2d_^82
000000@0]KoR00<000000;G0h`2e`><00P0000020;K1hP030000002f`N<0]/7S0080]/7S0P000006
0;O2h`040000002g`n@00000000000<0]l?S00<000000;O4i02ga>@00P0000020;S4i0040000002h
aN@0^<GT0;S5i0<000001P2ia^@00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo07/0oooo
001;0?ooo`800000`@3oool4000000<0YkgY00<000000:Nlj02W_>P00P2W_>P4000000<0Y[[W00<0
00000:Jii`2V^NL00P2V^NL00`000000Y[SV0:JhiP0200000080Y;OV00<000000:Ffi@2U]^D00`2U
]^D010000000YKKU0:Ffi@2U]^D3000000@0Y;CT00<000000:Bch`2T/n<00`2T/n<400000080Y;;S
00<000000:BbhP2T/^800`2T/^800`000000Y;;R0:BbhP02000000030:BahP000000Y;7Q00<0Y;7Q
00<000000:Fah@2U/N400`2U/N4400000080YK3P00<000000:F`h02U/>000`2U/>000`000000Y[3P
0:J`h00200000080Y[3O00<000000:J`g`2V/=l00`2V/=l00`000000Yk3O0:N`g`020:N`g`800000
102W/Ml00`000000Z;7O0:Rag`030:Rag`<00000102Y/Mh00`000000ZK;N0:VbgP020:VbgP<00000
102Z/]h00`000000Zk?N0:^cgP030:^cgP030000002[]=l0ZkCO0080000000<0[;?N0000002/]=l0
1@2/]=l2000000@0[KGO00<000000:ffg`2]]]l00P2]]]l3000000@0[[OO00<000000:ngg`2_]ml0
0`2_]ml2000000D0[kSO00<000000;2ih02`^N000P0000020;2ih0030000002a^^00/K[P00@0/K[P
0P0000040;:kh@040000002b_>40/[cQ0;:lh@<000001@2c_N4010000000/kgR0000000000030;Bn
h@030000002d_^80];kR0080];kR0`0000060;FohP800000102f`N800`000000]/7S0;K1h`020000
0080]/;S00<000000;O2h`2g`^<00`2g`^<2000000H0]l?S0P0000040;S4i0040000002haN@00000
000000<0^LGT00@000000;W6i02ia^@0^LKT0P0000000`2ja^@000000?ooo`0[0?ooo`030000003o
ool0oooo07X0oooo00170?ooo`@00000`@3oool200000080YkkZ00<000000:Nmj@2W_NT00P2W_NT0
0`000000YkcX0:Nlj00200000080Y[_X00<000000:Jji`2V^^L00`2V^^L010000000Y[WW0:Jii`2V
^NL3000000@0Y;OV00<000000:Ffi@2U]^D00`2U]^D400000080Y;GU00<000000:Bdi02T]>@00`2T
]>@00`000000Y;?S0:Bch`02000000030:>ch`000000Y;;S00<0Y;;S00<000000:BbhP2T/^800`2T
/^83000000@0Y;7R00<000000:Bah@2T/N400P2T/N400`000000YK7Q0:Fah@0200000080Y;3Q00<0
00000:F`h02U/>000P2U/>000`000000YK3P0:F`h0020:F`h0<00000102V/=l00`000000Y[3O0:J`
g`030:J`g`030000002W/=l0Yk3O0080000000<0Yk3O0000002W/Ml0102W/Ml00`000000Z;7O0:Ra
g`02000000030:RagP000000ZK7N00@0ZK7N00<000000:VbgP2Y/]h00P0000020:VbgP030000002Z
/]h0Z[;N00<0Z[;N00<000000:^cgP2[/mh00P2[/mh3000000@0[;?N00@000000:bdg`2/]=l0[;CO
0P0000000`2]]=h000000:feg`030:feg`030000002]]]l0[KKO008000000P2^]]l00`000000[[OO
0:jgg`030:jgg`030000002_]ml0[kOO0080000000<0[kSO0000002_^=l01@2_^=l2000000@0/;WP
00<000000;6jh02a^^000P2a^^03000000D0/[_Q00@000000;:lh@00000000000`2c_>400`000000
/kgQ0;>mh@020;>mh@8000001P2d_^401@000000];kR000000000000];oR00<000000`2e_n830000
00H0]/7R0P0000050;K2h`030000002g`^<0]l;S008000000P2g`n<010000000]l?S0;O3h`2g`n<3
000000H0^<CT00<000000;W5i02iaN@00`2iaN@010000000^LKT0000000000030;[6i0030000003o
ool0oooo02X0oooo00<000000?ooo`3oool0NP3oool004H0oooo0`0000310?ooo`800000102W_^X0
0`000000YkgY0:Nmj@030:Nmj@800000102V^nP00`000000Y[[W0:Jji`030:Jji`@000000P2T^>L0
0`000000Y;OV0:BgiP030:BgiP030000002U]^D0YKKU0080000000<0XkKV0000002T]ND0102T]ND0
10000000Y;CT0:Bdi02T]>@4000000@0Xk?S00<000000:Bbh`2T/^<00P2T/^<00`000000Y;;R0:Bb
hP0200000080Xk7R00<000000:BahP2T/N800P2T/N8010000000Y;7Q0:Bah@2T/N44000000@0Y;3Q
00<000000:F`h02U/>000P2U/>000`000000YK3P0:F`h00200000080YK3O00<000000:J`g`2V/=l0
0`2V/=l010000000Y[3O0:J`g`2V/=l4000000<0Yk3O00<000000:Nag`2W/Ml00`2W/Ml3000000@0
Z;7N00<000000:VagP2Y/Mh00P2Y/Mh3000000@0ZK;N00<000000:ZbgP2Z/]h00`2Z/]h00`000000
Zk?N0:^cgP02000000030:^cgP000000[;?N00@0[;?N00@000000:bdg`00000000000`2]]=h00`00
0000[KGO0:feg`030:feg`800000102^]]l00`000000[[OO0:jgg`030:jgg`<00000102_^=l01000
0000[kSO0:nhg`2_^=l2000000030;2hg`000000/;WP00@0/;WP00@000000;6jh000000000000`2a
^^000`000000/[_Q0;:kh@020;:kh@8000001@2c_>400`000000/kgQ0;>mh@0200000080/kgQ00<0
00000;Bnh@2d_^400`2d_^4500000080];oR00@000000;FohP00000000000`2f`>8010000000]/7R
0;K1hP2f`N82000000030;K1hP000000]/;S00D0]/;S0P0000040;O3h`040000002g`n<000000000
00<0^<CS00@000000;S4i02ha>@0^<CT0P0000000`2iaN<000000;W5i0030;W5i08000001P2ja^@0
0`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo07T0oooo00130?ooo`<000000`3oool00`00
0000oooo0?ooo`2l0?ooo`80000000<0Yko[0000002W_^X0102W_^X00`000000YkgY0:Nmj@020000
00030:Jlj@000000Y[_X00@0Y[_X00<000000:Jji`2V^^L00P0000000`2T^NP000000:Bhi`040:Bh
i`040000002T]nH0Y;OV0:BgiP@00000102S]^H00`000000Y;GU0:Bei@020:Bei@@000000`2S/n@0
0`000000Xk?S0:>ch`020:>ch`040000002T/^<0Y;;S0:Bbh`@00000102S/N800`000000Y;7R0:Ba
hP030:BahP<000000`2T/>400`000000Y;3Q0:B`h@030:B`h@040000002U/>00YK3P0:F`h0<00000
102U/=l00`000000Y[3O0:J`g`030:J`g`@000000P2V/=l00`000000Yk3O0:N`g`030:N`g`030000
002W/Ml0Yk7O008000000P2W/=h00`000000Z;7N0:RagP020:RagP030000002Y/Mh0ZK7N00800000
0P2Y/Mh00`000000ZK;N0:VbgP030:VbgP030000002Z/]h0Z[;N0080Z[;N0`0000040:^cgP030000
002//mh0[;?N0080[;?N0P0000060:fdgP030000002]]Ml0[KGO0080000000<0[[GN0000002^]]l0
102^]]l00`000000[[OO0:jgg`02000000030:ngg`000000[kSO00@0[kSO00@000000:nhg`000000
00000`2`^=l00`000000/;WP0;2ih0030;2ih08000001@2a^^000`000000/[_Q0;:kh@0200000080
/[_P00<000000;>lh@2c_>400P2c_>43000000D0/kgQ00<000000;Bnh@2d_^401@0000040;BohP80
00001@2f`>8010000000]/7R0000000000030;K1hP030000002f`^<0]/;S0080]/;S0`0000060;O3
h`030000002ha><0^<CS00<0^<CS00@000000;S4i000000000000`2iaN<00`000000^LGT0;W5i002
00000080^LKS00<000000;[6i02ja^@00P2ja^@00`000000^/OT0000000/0?ooo`030000003oool0
oooo07P0oooo00100?ooo`<00000`P3oool3000000@0Yko[00@000000:NnjP2W_^X0YkkZ10000003
0:Jlj@030000002V^nP0Y[_X00<0Y[_X0`0000040:Bij0030000002T^>L0Y;SW0080Y;SW10000003
0:>giP030000002S]^H0XkKV0080XkKV00<000000:Bei@2T]ND00P0000020:>di@030000002S/n@0
Xk?T0080Xk?T00<000000:>ch`2S/n<00`2S/n<3000000<0Xk;S00<000000:>ahP2S/N800`2S/N80
0`000000Y;7R0:BahP02000000030:>`h@000000Y;3Q00<0Y;3Q00<000000:B`h@2T/>400`2T/>44
00000080Y:oP00<000000:F`g`2U/=l00`2U/=l00`000000Y[3O0:J`g`02000000030:J_g`000000
Y[3O00@0Y[3O00<000000:N`g`2W/=l00P2W/=l3000000@0Yk3N00<000000:RagP2X/Mh00`2X/Mh2
000000@0ZK7N00<000000:VbgP2Y/]h00`2Y/]h00`000000Z[;N0:ZbgP02000000030:^bgP000000
Zk?N00@0Zk?N00<000000:bcgP2//mh00P0000020:bcgP030000002]]=h0[KCN00@0[KCN0P000003
0:jegP030000002^]]l0[[KO00<0[[KO0`0000040:ngg`030000002_^=l0[kSO0080[kSO0P000006
0;2hg`030000002`^N00/;WP0080000000<0/KWO0000002a^^001@2a^^02000000@0/[_P00<00000
0;>lh@2c_>400P0000020;>lh0030000002c_N40/kgQ00@0/kgQ0`0000030;Bnh@030000002d_n80
];oR0080000000<0]KoQ000000000000102f`>82000000H0]/7R00@000000;K2h`00000000000`2g
`^8010000000]l?S0;O3h`2g`n<2000000030;S3h`000000^<CS00<0^<CS0P0000060;W5h`800000
1@2ia^<010000000^/KT0000000000030;[7i0030000003oool0oooo02T0oooo00<000000?ooo`3o
ool0N03oool003h0oooo0P0000330?ooo`8000000P2W`N`00`000000Yko[0:Noj`020:Noj`@00000
0`2V_NX00`000000Y[cY0:Jlj@020:Jlj@030000002V^nP0Y[_X008000000P2T^^T00`000000Y;WX
0:Bij0020:Bij0030000002T^>L0Y;SW008000000P2S^>L00`000000XkOV0:>giP020:>giP030000
002S]^H0XkKV00<0XkKV0P0000040:>di@030000002S/n@0Xk?T00<0Xk?T00<000000:>ch`2S/n<0
0P0000000`2R/^<000000:>bh`030:>bh`030000002S/N80Xk7R00<0Xk7R0`0000030:>`h@030000
002T/>40Y;3Q00<0Y;3Q00<000000:B`h@2T/>400P0000000`2T[n0000000:B_h0030:B_h0030000
002U/=l0YK3O00<0YK3O0`0000040:J_g`030000002V/=l0Y[3O0080Y[3O00<000000:N`g`2W/=l0
0P0000020:N`gP030000002W/=h0Yk3N00<0Yk3N00<000000:RagP2X/Mh00P0000000`2X/=h00000
0:VagP040:VagP030000002Y/]h0ZK;N0080ZK;N0`0000040:^bgP030000002[/mh0Zk?N0080Zk?N
0`0000040:bcgP030000002]]=h0[KCN0080[KCN0`0000050:jegP030000002^]]l0[[KO00800000
00<0[kKN0000002_]ml0102_]ml00`000000[kSO0:nhg`0200000080/;SO00<000000;2hg`2`^=l0
0`2`^=l3000000@0/KWO00@000000;6jh02a^^00/K[P0P0000000`2b^]l000000;:kh0040;:kh080
0000102c_>000`000000/kgQ0;>mh@020;>mh@<000001P2d_^42000000@0]KoQ00<000000;K0hP2f
`>800P0000020;K0hP030000002f`N80]/7R00<0]/7R0P0000050;O2hP040000002g`n<000000000
00<0^<?S00<000000;S4h`2ha><00P0000020;W4h`040000002iaN<0^LGS0;W5h`80000000<0^LGS
0000002ia^<01@2ia^<00`000000^/OT0;[7i0040;[7i0030000003oool0oooo02T0oooo00<00000
0?ooo`3oool0M`3oool003/0oooo0`0000310?ooo`@00000102W`N`00`000000Yko[0:Noj`020000
0080Y[o[00<000000:JmjP2V_NX00`2V_NX010000000Y[cY0:Jlj@2V_>T3000000D0Y;[Y00@00000
0:Bij02T^NP0Y;WX0`0000040:>hi`030000002S]nH0XkOV00<0XkOV00<000000:>fiP2S]^H00P00
00000`2R]NH000000:>di@040:>di@040000002S/n@0Xk?T0:>ci0@000000`2R/^<00`000000Xk;S
0:>bh`030:>bh`030000002S/N80Xk7R0080000000<0Xk3R0000002S/>40102S/>400`000000Y;3Q
0:B`h@020:B`h@<00000102T[n000`000000Y:oP0:B_h0020:B_h0030000002U/=l0YK3O00800000
0P2U[ml00`000000YZoO0:J_g`020:J_g`040000002V/=l0Y[3O0:J`g`@00000102W/=h00`000000
Yk3N0:N`gP030:N`gP<000000`2X/=h00`000000ZK7N0:VagP030:VagP030000002Y/]h0ZK;N0080
000000<0Z[7M0000002[/]h0102[/]h00`000000Zk?N0:^cgP0200000080[;?M00<000000:bcgP2/
/mh00`2//mh010000000[KCN0000000000030:fdgP030000002^]Mh0[[GN0080[[GN0`0000040:nf
gP030000002_]ml0[kOO0080[kOO0`0000050;2hg`030000002`^=l0/;SO0080000000<0/KWO0000
002a^Ml0102a^Ml010000000/K[P0000000000030;:jg`030000002b^n00/[_P0080/[_P00<00000
0;>kh00000001@2c_>04000000<0];gQ00@000000;Bnh@2d_^40];kQ0`0000060;Foh@8000001@2f
`>800`000000]/7R0;K1hP0200000080]l7R00<000000;O2hP2g`^800P2g`^82000000H0^<?S0P00
00040;W4h`040000002iaN<00000000000<0^LGS00<000000;W6h`2ia^<00P2ia^<2000000030;[6
h`000000^/OT0080^/OT0`00000/0?ooo`030000003oool0oooo07H0oooo000i0?ooo`800000`@3o
ool3000000<0Yl;]00<000000:O1k02W`N`00`2W`N`2000000@0Y[o[00<000000:JmjP2V_NX00`2V
_NX400000080Y;cZ00<000000:Bjj@2T^^T00`2T^^T400000080XkWX00<000000:>hi`2S^>L00`2S
^>L010000000XkOV0:>giP2S]nH4000000<0X[GV00<000000:>di@2S]>D00`2S]>D400000080X[?T
00<000000::bh`2R/^<00`2R/^<010000000Xk;S0:>bh`2S/^<4000000@0Xk3R00<000000:>`h@2S
/>400P2S/>400`000000Y;3Q0:B`h@02000000030:>_h@000000Y:oP00@0Y:oP00@000000:B_h02T
[n00Y:oP100000040:F_g`030000002V[ml0YZoO00<0YZoO0`0000030:J_gP030000002W/=h0Yk3N
00<0Yk3N00<000000:N`gP2W/=h00P0000000`2X/=h000000:R`gP040:R`gP040000002Y/Mh0ZK7N
0:VagP@00000102Z/Md00`000000Zk;N0:^bgP020:^bgP<00000102//md00`000000[;?N0:bcgP03
0:bcgP8000001@2]]=h00`000000[[GN0:jegP0200000080[[GN00<000000:nfgP2_]]h00P2_]]h0
0`000000[kOO0:ngg`0200000080/;ON00<000000;2hg`2`^=l0102`^=l2000000@0/KWO00<00000
0;6ig`2a^Ml00P2a^Ml2000000H0/[[O00@000000;:kh000000000000P2c^n0010000000/kcP0;>l
h02c_>04000000D0];gQ00@000000;Bnh@00000000000`2e_^4010000000]KoQ0;Foh@2e_n440000
00H0]/3R0P0000040;O1hP030000002g`^80]l;R008000000P2h`^8010000000^<?S0;S3h`2h`n<3
000000H0^LCS00<000000;W5h`2iaN<0102iaN<010000000^LKS0000000000030;[6h`040000002j
an@00000000000<0^lOS00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`1f0?ooo`00=`3o
ool200000<40oooo0P0000000`2W`nd000000:O2k@040:O2k@030000002W`N`0Yl7/0080000000<0
Y/3/0000002V_n/0102V_n/00`000000Y[gZ0:JmjP0200000080Y;g[00<000000:BljP2T_>X00P2T
_>X00`000000Y;[Y0:Bjj@0200000080Xk[Y00<000000:>ij02S^NP00P2S^NP010000000XkSW0:>h
i`2S^>L5000000<0X[KW00<000000::eiP2R]NH00P2R]NH00`000000XkCU0:>di@0200000080X[?U
00<000000::ci02R/n@00P2R/n@00`000000X[;S0::bh`020::bh`@000000P2R/N<00`000000Xk3R
0:>`hP030:>`hP040000002S/>40Xk3Q0:>`h@@00000102S[n400`000000Y:oP0:B_h0020:B_h0@0
00000`2T[ml00`000000YJoO0:F_g`030:F_g`030000002V[ml0YZoO0080000000<0YZoN0000002V
[mh00`2V[mh00`000000Yk3N0:N`gP030:N`gP<00000102X/=h00`000000Z;3N0:R`gP020:R`gP@0
00000`2Y/Md00`000000Z[7M0:Zag@020:Zag@030000002[/]h0Zk;N008000000P2[/]d00`000000
[;?M0:bcg@030:bcg@030000002//mh0[;?N0080000000<0[K?M0000002]]=h01@2]]=h2000000@0
[[GN00<000000:nfgP2_]]h00`2_]]h2000000@0/;ON00<000000;2hg`2`^=l00P2`^=l3000000D0
/KWO00<000000;6ig`2a^Ml00P0000020;:jg`030000002b^]l0/[[O00<0/[[O0P0000050;>kh0D0
00000P2d_>000`000000];gQ0;Bmh@020;Bmh@8000001@2e_^4010000000]KoQ0000000000030;Jo
h@030000002f`>80]/3R0080]/3R0`0000060;O1hP800000102h`^8010000000^<?S000000000003
0;W3hP040000002ia><0^LCS0;W4h`80000000<0^LCS0000002iaN<0102iaN<2000000H0^/KS00<0
00000;_7h`2kan<00`2kan<00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo07D0oooo000g
0?ooo`800000_P3oool3000000@0Yl?]00@000000:O2k@2W`^d0Yl;]100000040:K0k0040000002V
_n/0Y[o[0:Joj`@00000102T_N/010000000Y;cZ0:BljP2T_>X4000000@0Xk[Y00<000000:>ij02S
^NP00`2S^NP3000000<0X[OX00<000000::fi`2R]^L00P2R]^L010000000X[GV0::eiP2R]NH40000
00@0X[?U00<000000::ci02R/n@00P2R/n@00`000000X[;S0::bh`0200000080XK7S00<000000::a
h`2R/N<00P2R/N<00`000000Xk3R0:>`hP020:>`hP@000000`2S/>400`000000XjoQ0:>_h@020:>_
h@030000002T[n00Y:oP008000000P2T[ml00`000000Y:oO0:B_g`020:B_g`030000002U[ml0YJoO
00<0YJoO0`0000030:J_gP030000002V[mh0YZoN00<0YZoN00<000000:N`gP2W/=h00P0000000`2W
[md000000:R`gP040:R`gP030000002X/=h0Z;3N008000000P2Y/=d00`000000ZK7M0:Vag@020:Va
g@030000002Z/Md0Z[7M00<0Z[7M0P0000040:^bg@030000002//md0[;?M00<0[;?M0`0000040:fc
g@040000002]]=h0[KCN0:fdgP80000000<0[[CM0000002^]Mh0102^]Mh00`000000[kKN0:nfgP02
000000030:nfgP000000/;ON00@0/;ON00@000000;2hg`00000000000`2a^=h00`000000/KWO0;6i
g`020;6ig`<000001@2b^]l00`000000/[[O0;:jg`02000000040;>kg`000000/k_P0;>kh0<00000
00<0/k_P000000000000102d_>000`000000];gQ0;Bmh@0200000080]KgP00<000000;Fnh@2e_^40
0`2e_^400`000000][oQ0;Joh@040;Joh@040000002f`>800000000000<0]l7Q00@000000;O1hP2g
`N80]l7R0`0000060;S2hP030000002i`n80^L?R00<0^L?R00@000000;W4h`0000000000102ia><0
0`000000^LGS0;W5h`0200000080^/GS00@000000;[6h`2ja^<0^/KS0P0000000`2ka^<000000;_7
h`030;_7h`800000:`3oool2000007H0oooo000g0?ooo`030000003oool0000000800000^@3oool2
00000080YlC^00<000000:O3k@2W`nd00`2W`nd3000000<0Y/7]00<000000:K0k02V`>`00`2V`>`3
000000<0Y;k/00<000000:Bmj`2T_N/00`2T_N/3000000<0Xk_Z00<000000:>jj@2S^^T00`2S^^T0
0`000000XkWX0:>ij002000000030:6hj@000000X[OX00<0X[OX00<000000::fi`2R]^L00`2R]^L3
000000<0XKCV00<000000::ci@2R/nD00`2R/nD010000000X[?T0::ci02R/n@3000000@0XK7S00<0
00000::ah`2R/N<00P2R/N<00`000000Xk3R0:>`hP0200000080X[3R00<000000:>`h@2S/>400P2S
/>400`000000XjoQ0:>_h@030:>_h@800000102T[ml00`000000Y:oO0:B_g`030:B_g`030000002U
[ml0YJoO0080000000<0YJkN0000002V[mh0102V[mh00`000000YZoN0:J_gP020:J_gP<00000102W
[md00`000000Z;3N0:R`gP020:R`gP<00000102Y/=d00`000000ZK7M0:Vag@030:Vag@030000002Z
/Md0Z[7M0080000000<0Zk7M0000002[/]d0102[/]d00`000000[;?M0:bcg@02000000030:bcg@00
0000[K?M00@0[K?M00@000000:fdgP00000000000`2^]=d00`000000[[GN0:jegP030:jegP<00000
0`2_]]h00`000000/;ON0;2ggP030;2ggP8000001@2a^=h00`000000/KWO0;6ig`0200000080/[WN
00<000000;:jg`2b^]l0102b^]l2000000<0/k_O0`0000020;>kh0<000001P2d_>02000000D0]KgP
00<000000;Fnh@2e_^400P0000000`2f_n4000000;Joh@040;Joh@8000001@2g`N4010000000]l7R
0000000000030;S2hP040000002h`^80^<;R0;S2hP80000000<0^<?R0000002i`n800`2i`n820000
00L0^LCS0P0000040;[5h`040000002ja^<00000000000<0^lKS00<000000;_7h`2kan<00P000002
0;_7h`030000003oool0oooo02H0oooo0P0000000`3oool000000?ooo`1e0?ooo`00=`3oool40000
0;H0oooo100000050:O4kP030000002W`nd0Yl?]0080000000<0Y/?^0000002V`Nd0102V`Nd00`00
0000Y/3/0:K0k002000000030:Bok@000000Y;k/00@0Y;k/00<000000:Bmj`2T_N/00P0000000`2S
_>/000000:>kjP040:>kjP040000002S^^T0Xk[Y0:>jj@@000000`2Q^>T00`000000X[OX0::gj003
0::gj0030000002R]^L0X[KW0080000000<0XKGW0000002Q]>H0102Q]>H010000000X[?U0::ci@2R
/nD500000080XK;T00<000000:6ah`2Q/N<00`2Q/N<010000000X[7S0::ah`2R/N<3000000@0X[3R
00<000000:>`h@2S/>400`2S/>400`000000XjoQ0:>_h@02000000030:>_h0000000Y:oO00@0Y:oO
00<000000:B_g`2T[ml00P2T[ml3000000@0YJkN00<000000:J_gP2V[mh00P2V[mh00`000000YZoN
0:J_gP02000000030:J_gP000000YjoM00@0YjoM00<000000:R`gP2X/=h00P0000020:R_g@030000
002Y/=d0ZK3M00<0ZK3M00<000000:Vag@2Y/Md00P2Y/Md3000000<0Zk7M00<000000:^bg@2[/]d0
0`2[/]d3000000@0[;?M00<000000:fcg@2]/md00P2]/md2000000H0[[CM00<000000:jegP2^]Mh0
0P0000000`2_]Md000000:nfgP040:nfgP030000002`]mh0/;ON0080000000<0/;ON0000002a^=h0
1@2a^=h2000000@0/[WN00<000000;:jg`2b^]l00P2b^]l3000000070;>kg`00000000000000002c
^ml000000;>kh002000000<0];cO00@000000;Blh02d_>00];cP0P0000000`2e_N0000000;Fmh005
0;Fmh0800000102f_n400`000000][oQ0;Joh@02000000030;O0h@000000000000@0]l7Q0P000005
0;S2hP040000002h`^800000000000<0^<?R00<000000;W3hP2i`n800P0000020;W4hP030000002i
a><0^LCS0080^LCS0`0000060;[5h`030000002ka^<0^lKS00@0^lKS0P0000040;_7h`030000003o
ool0oooo02X0oooo00<000000?ooo`3oool04P3oool01@3ogi`0B000000000000000M;oo00@0oooo
00D0omnL04P000000000001d0;ooo`1C0?ooo`00=P3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo0;00oooo0`0000030:O6k`030000002Wa>h0YlC^00<0YlC^0`0000040:K3kP040000
002V`Nd0Y/7]0:K1k@@00000102T_nd010000000Y;k/0:Bnk02T_^`4000000<0Xkc[00<000000:>k
jP2S^nX00`2S^nX4000000<0XKWZ00<000000:6hj@2Q^>T00P2Q^>T010000000X[OX0::gj02R]nP4
000000<0XKGW00<000000:6diP2Q]>H00`2Q]>H400000080X;;U00<000000:6bi02Q/^@00`2Q/^@0
10000000XK7S0:6ah`2Q/N<500000080XK3R00<000000::`hP2R/>800`2R/>8010000000Xk3Q0:>`
h@2S/>44000000<0XjoP00<000000:B_g`2T[ml00`2T[ml00`000000Y:oO0:B_g`02000000030:B^
g`000000YJkN00@0YJkN00@000000:J_gP2V[mh0YZoN100000040:J_gP030000002W[md0YjoM0080
YjoM0`0000040:R_g@030000002Y/=d0ZK3M00<0ZK3M00<000000:Vag@2Y/Md00P0000000`2Z/=d0
00000:^ag@040:^ag@030000002[/]d0Zk;M0080000000<0[;;M0000002//md0102//md00`000000
[K?M0:fcg@0200000080[K?M00<000000:jdg@2^]=d0102^]=d2000000@0[kGM00<000000:nfgP2_
]]h00P2_]]h3000000@0/;ON00@000000;6hgP2a^=h0/KSN0P0000000`2a^=h000000;:igP040;:i
gP050000002b^]l000000000002c^]l00`0000040;>kg`8000001@2d_=l010000000];cP00000000
00030;Fmh0030000002e_N00]KgP0080]KgP0`0000060;Joh@800000102g`>400`000000]l7Q0;O1
h@0200000080^<7Q00<000000;S2hP2h`^800P2h`^82000000H0^<?R0P0000050;W4hP040000002i
a><00000000000<0^/GR00@000000;[5h`2jaN<0^/GS0P0000000`2kaN<000000;_6h`020;_6h`80
000000<0^lKS0000002kan<00`2kan<2000002`0oooo00<000000?ooo`3oool0403oool01`3ogi`0
B00004RLg`3oool0oono07@0001d_ol00P3oool01`3ookl0M00007Boo`3oool0okmd0000M02oool0
DP3oool003H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2]0?ooo`80000000<0
YlO`0000002Wa^l0102Wa^l00`000000YlC^0:O4kP0200000080Y/C_00<000000:K3kP2V`nh00P2V
`nh4000000<0Y<7^00<000000:Bok@2T_nd00`2T_nd3000000<0X[g/00<000000:>lj`2S_>/00P2S
_>/00`000000Xk_Z0:>kjP0200000080XK[[00<000000:6ijP2Q^NX00P2Q^NX010000000XKSY0:6h
j@2Q^>T5000000<0X;KX00<000000:6ei`2Q]NL00P2Q]NL00`000000XKCV0:6diP0200000080X;?V
00<000000:2bi@2P/^D00P2P/^D00`000000XK;T0:6bi0020:6bi0@000000P2Q/><00`000000XK3R
0:6`hP020:6`hP030000002R/>80X[3R00<0X[3R100000020::_h@030000002S[n00XjoP00<0XjoP
00@000000:B_g`2T[ml0Y:oO100000040:B^g`030000002U[]h0YJkN0080YJkN100000030:J^gP03
0000002V[mh0YZoN0080YZoN00<000000:N_g@2W[md00P0000020:N_g@030000002X[md0Z:oM00<0
Z:oM00<000000:V`g@2Y/=d00P2Y/=d3000000@0Z[3M00<000000:^ag@2[/Md00P2[/Md3000000@0
[;;M00<000000:bcg@2//md00P2//md3000000@0[K?M00<000000:jdg@2^]=d00P2^]=d3000000D0
[kGM00<000000:nfgP2_]]h00P0000000`2`]]d000000;2ggP040;2ggP040000002a^=h000000000
00<0/KSN00<000000;:igP2b^Mh00`2b^Mh2000000030;>jg`00000000000080/k[O00<000000;>k
g`2c^ml00P0000020;Bkg`030000002d_=l0];cO0080];cO0P0000060;Fmh0040000002e_N000000
000000<0][kP00@000000;Joh@2f_n40][oQ0`0000060;O0h@800000102h`N400`000000^<;R0;S2
hP0200000080^<;Q00<000000;S3hP2h`n800P2h`n800`000000^L?R000000060;W4hP030000002j
aN80^/GR00<0^/GR00@000000;[5h`00000000000`2kaN<010000000^lKS0000000000030;_6h`03
0000002kan<0^lOS008000000P2lan<00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo00l0
oooo00<0oooO09a80000M;l00`3oool01`3o_g@004RL0=ooo`3oool0_g@00018W03Oool00P3oool0
0`2oM00007Bo0?ooo`1A0?ooo`00903oool01P2oM000000000000000000004RL0=ooo``0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`2Y0?ooo`<00000102Wao0010000000YlK_0:O6
k`2Wa^l4000000@0Y/C_00<000000:K3kP2V`nh00P0000020:C2k`030000002T`Nh0Y<7^00<0Y<7^
00<000000:Bok@2T_nd00P0000000`2R_nd000000::mk0040::mk0040000002S_>/0Xkc[0:>lj`<0
0000102Q^^/00`000000XKWZ0:6ijP030:6ijP<000000`2P]nT00`000000X;KX0:2fj0020:2fj004
0000002Q]NL0XKGW0:6ei`@00000102P/nH00`000000X;;U0:2bi@020:2bi@030000002Q/^@0XK;T
008000000P2P/N@00`000000XK3S0:6`h`020:6`h`030000002Q/>80XK3R0080XK3R00<000000::`
hP2R/>800P0000020:6_h@030000002R[n40XZoQ0080XZoQ00<000000:>_h02S[n000P2S[n040000
00<0XjkO00<000000:B^g`2T[]l00P2T[]l00`000000YJkN0:F^gP0200000080YJkN00<000000:J^
gP2V[]h00P2V[]h00`000000YZoN0:J_gP030:J_gP800000102W[md00`000000Z:oM0:R_g@030:R_
g@030000002Y/=d0ZK3M0080000000<0ZK3L0000002Z/=d0102Z/=d00`000000Zk7M0:^ag@020000
0080Zk7L00<000000:bbg@2//]d00P2//]d00`000000[;?M0:bcg@0200000080[K;L00<000000:fc
g@2]/md00`2]/md010000000[[CM0000000000030:jdg@030000002_]Md0[kGM0080[kGM0`000004
0;2fg@030000002`]mh0/;ON0080/;ON0P0000060;6hgP030000002b^Mh0/[WN00D00000102c^]l3
000000@0];_O00<000000;Blg`2d_=l00P0000020;Blg`030000002e_N00]KgP00<0]KgP0P000005
0;Jnh0040000002f_n400000000000<0]koP00@000000;O0h@2g`>40]l3Q0`0000060;S1h@800000
1@2h`^4010000000^<?R0000000000020;W3hP030000002ia>80^LCR0080^LCR0P0000000`2ja>80
00000;[5hP030;[5hP8000001P2kaN<00`000000^lKS0;_6h`040;_6h`800000102lan<00`000000
oooo0?ooo`0Z0?ooo`030000003oool0oooo00h0oooo00<0omnL04P0001d_ol00`3oool01P3o_g@0
001809cOo`3ookl0M00004RLg`<0oooo00<0gia80000M02oool0D@3oool002D0oooo00<0omnL04P0
001d_ol03P3oool00`000000oooo0?ooo`050?ooo`800000Y`3oool200000080YlSa00<000000:O7
l02Wao000`2Wao03000000<0Y/G`00<000000:K4k`2Va>l00`2Va>l2000000D0Y<;_00@000000:C1
kP2T`Nh0Y<7^100000030::ok@030000002R_N`0X[g/00<0X[g/100000020:6kk0030000002Q^^/0
XK[[00<0XK[[00<000000:6ijP2Q^NX00P0000000`2O^>X000000:2gj@030:2gj@030000002P]^P0
X;KX00<0X;KX0`00000309ndi`030000002P/nH0X;?V00<0X;?V00@000000:2bi@2P/^D0X;;U0`00
00040:2ai0030000002Q/><0XK3S0080XK3S00@000000:6`hP2Q/>80XK3R100000040:6_h@030000
002R[n40XZoQ0080XZoQ00<000000:>_h02S[n000P0000020::^g`030000002S[]l0XjkO0080XjkO
00<000000:B^g`2T[]l00`2T[]l2000000@0YJkN00<000000:J^gP2V[]h00`2V[]h00`000000YZoN
0:J_gP02000000030:N^g@000000YjoM00@0YjoM00@000000:R_g@2X[md0Z:oM100000040:V`g003
0000002Z/=d0Z[3M0080Z[3M0`0000040:^ag0030000002//]d0[;;M00<0[;;M0P0000040:fbg003
0000002]/md0[K?M00<0[K?M0P0000050:jdg@030000002_]Md0[kGM008000000P2`]Md00`000000
/;KM0;2fg@020;2fg@030000002`]mh0/;ON008000000P2a]md00`000000/KSN0;6hgP030;6hgPD0
00000P2b^Mh00`000000/k[O0;>jg`02000000030;>jgP000000];_O00D0];_O0P0000050;Blg`03
0000002e_N00]KgP0080000000<0][gO0000002f_^001@2f_^000`000000]koP0;Noh0030;Noh004
0000002g`>400000000000<0]l3Q00@000000;S1h@2h`N40^<7Q0P0000000`2h`N4000000;S2h@05
0;S2h@030000002i`n80^L?R00<0^L?R00@000000;W4hP00000000000`2ja>800`000000^/GR0;[5
hP0200000080^/GR00@000000;_5h`2kaN<0^lGS0P0000000`2ka^8000000;_6h`020;_6h`<00000
1P2lan<00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo00d0oooo00<0omnL04P0001d_ol0
0`3oool01P3o_g@0001809cOo`3ogi`0B00007Boo`<0oooo00<0okmd0000B02Lgol0D@3oool002D0
oooo00<0omnL04P0001d_ol03P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0:80
oooo0P0000050:O8l@030000002Wao00YlO`0080000000<0Y/K`0000002VaO00102VaO000`000000
Y/C_0:K4k`02000000030:C3k`000000Y<;_00@0Y<;_100000030:;0kP030000002R_nd0X[o]0080
X[o]00<000000::mk02R_N`00P0000020:6mk@030000002Q^n`0XK_/0080XK_/00@000000:6jj`2Q
^^/0XK[[1000000309nhjP030000002P]nT0X;OY00<0X;OY00<000000:2fj02P]^P00P0000000`2O
]NP0000009ndi`0309ndi`030000002P/nH0X;?V00<0X;?V1000000209nai@030000002P/N@0X;7T
00<0X;7T00@000000:6`h`2Q/><0XK3S100000030:2_hP030000002Q[n40XJoQ00<0XJoQ00@00000
0::_h@2R[n40XZoQ0`0000040::^g`030000002S[]l0XjkO00<0XjkO00<000000:B^g`2T[]l00P00
00000`2T[Mh000000:F^gP040:F^gP030000002V[]h0YZkN0080YZkN0`0000030:N^g@030000002W
[md0YjoM00<0YjoM100000020:R_g0030000002Y/=`0ZK3L00<0ZK3L00<000000:Z`g@2Z/=d00P00
00020:^`g0030000002[/M`0Zk7L00<0Zk7L00<000000:bbg@2//]d00P0000000`2//]`000000:fb
g0040:fbg0030000002]/md0[K?M0080000000<0[[?L0000002^]=d01@2^]=d2000000@0/;GM00<0
00000;2fg@2`]]d00`2`]]d2000000D0/KOM00<000000;6hgP2a^=h0100000060;:igP800000102c
^]h010000000];_O0;Bkg`2d^ml2000000030;BkgP000000];cO00D0];cO0P0000040;Jmg`040000
002f_^00][kP0;Jnh0<000001P2g_n000`000000]l3Q0;O0h@030;O0h@@000000`2h`N400`000000
^<;Q0;S2h@020;S2h@80000000<0^L;Q0000002i`n800`2i`n82000000H0^/CR0P0000040;[5hP04
0000002kaN<00000000000<0^lKR00@000000;_6h`00000000000`2lan8010000000_<OS0;c7h`2l
an<2000000030;c7h`000000oooo02X0oooo00<000000?ooo`3oool03@3oool00`3ogi`0B00007Bo
o`030?ooo`060;md000004P0W=oo0?oOW0180000M;oo0`3oool00`3o_g@0001809cOo`1A0?ooo`00
9@3oool00`3ogi`0B00007Boo`0>0?ooo`<00000203oool00`000000oooo0?ooo`2O0?ooo`800000
00<0YlWa0000002Wb?40102Wb?43000000@0Y/K`00@000000:K5l02VaO00Y/G`100000040:C3k`03
0000002T`^l0Y<;_008000000P2R`^l00`000000X/3^0:;0kP020:;0kP040000002R_nd0X[o]0::o
k@@00000102Q_Nd010000000XK_/0:6kk02Q^n`5000000<0WkW[00<0000009nhjP2O^>X00P2O^>X0
10000000X;OY0:2gj@2P]nT4000000<0WkGX00<0000009ndi`2O]>L00`2O]>L00`000000X;?V0:2c
iP020000000309jbiP000000Wk7U00<0Wk7U00<000000:2ai02P/N@00`2P/N@400000080WjoS00<0
00000:2_hP2P[n800P2P[n800`000000XJoQ0:6_h@030:6_h@@000000P2R[^000`000000XZkO0::^
g`030::^g`040000002S[]l0XjkO0:>^g`@000000`2T[Mh00`000000YJkN0:F^gP030:F^gP030000
002V[]h0YZkN0080000000<0YZkM0000002W[]d0102W[]d00`000000YjoM0:N_g@0200000080Z:kL
00<000000:R_g02X[m`00P2X[m`00`000000ZK3L0:V`g0020:V`g0<00000102[/=`00`000000Zk7L
0:^ag0030:^ag0<000000`2//]`00`000000[K;L0:fbg0030:fbg0<00000102^/m`010000000[[CM
0:jdg@2^]=d2000000030:ndg0000000/;GM00@0/;GM00<000000;2fg@2`]]d00P0000000`2a]]d0
00000;6gg@050;6gg@<000000`2b^=h010000000/[WN0;:igP2b^Mh2000000030;>igP000000/k[N
00<0/k[N00@000000;Bkg`00000000000`2d^mh00`000000];cO0;Blg`020;Blg`<000001@2f_Ml0
10000000][kP0000000000030;Jnh0040000002g_n00]koP0;Noh080000000<0]koP0000002g`>40
0`2g`>43000000D0^<7Q00@000000;S2h@00000000000`2i`^400`000000^L?R0;W3hP0200000080
^/?Q00@000000;[4hP2ja>80^/CR0`0000060;[5hP030000002ka^80^lKR00@0^lKR00<000000;c7
hP2lan800`2lan8010000000_<OS0000000000030;c7h`030000003oool0oooo02X0oooo00<00000
0?ooo`3oool0303oool0303ookl0M00000000018W=l0oooo0?noM000000007Bo0?ooo`3ogi`0B000
07Boo`<0oooo00<0okmd0000B02Lgol0D@3oool001H0oooo00<0oooO09a8000000001`0000000`18
W=l0oooo0?ooo`020?ooo`030?oOW0180000M;oo00d0oooo0P00000;0?ooo`030000003oool0oooo
09X0oooo100000040:O9l@030000002Wb?40YlSa008000000P2Vao400`000000Y/K`0:K6l0030:K6
l0<000000`2TaO02000000<0Y<?_0`0000050:;2k`040000002R`>h0X/3^0:;0kP@000000`2Q_^h0
0`000000XKg]0:6mk@030:6mk@<000000`2O^n`00`000000WkW[09nij`0309nij`040000002O^>X0
WkSZ09nhjP@000000`2N]^T00`000000WkGX09nej00209nej0040000002O]>L0WkCW09ndi`@00000
102N/^H00`000000Wk7U09nai@0209nai@030000002P/N@0X;7T0080000000<0Wk3T0000002O[n<0
0`2O[n<00`000000X:oR0:2_hP030:2_hP030000002Q[n40XJoQ0080000000<0XJkQ0000002R[^00
0`2R[^000`000000XZkO0::^g`030::^g`@000000P2S[Mh00`000000Y:gN0:B]gP030:B]gP040000
002U[]h0YJkN0:F^gP@00000102V[]d00`000000YjkM0:N^g@020:N^g@<00000102X[]`00`000000
Z:oL0:R_g0020:R_g0030000002Y/=`0ZK3L008000000P2Z[m`00`000000Zk3L0:^`g0030:^`g003
0000002[/M`0Zk7L0080000000<0[;7L0000002//]`0102//]`00`000000[K;L0:fbg00200000003
0:jbg0000000[[?L00@0[[?L00@000000:jdg@00000000000`2_]=`00`000000/;GM0;2eg@030;2e
g@<000000`2a]]d010000000/KOM0;6gg@2a]md4000000D0/[SN00@000000;:igP00000000000`2c
^Mh00`000000/k[N0;>jgP030;>jgP030000002d^mh0];_N00@0];_N00@000000;Blg`0000000000
0`2e_=l00`000000][gO0;Jmg`020;Jmg`8000001@2f_^0010000000]koP0000000000030;Noh003
0000002g`>40]l3Q008000000P2h`>000`000000^<7Q0;S1h@030;S1h@8000001P2i`^42000000@0
^/?Q00@000000;[4hP00000000000`2ja>8010000000^/GR0;[5hP2jaN82000000030;_5hP000000
^lKR0080^lKR0P0000000`2la^8000000;c7hP030;c7hP8000001P2lan<00`000000oooo0?ooo`0Z
0?ooo`030000003oool0oooo00`0oooo00<0_g@00018W02oW4P00P0000001P00B9`0gooo0?ooo`3o
gi`0B00007Boo`<0oooo00<0okmd0000B02Lgol0D@3oool002D0oooo00<0omnL04P0001d_ol03@3o
ool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo09H0oooo0`0000030:O:lP8000000`2W
bO43000000D0Y/Oa00<000000:K6l02Va_000P0000000`2Ta_4000000:C5l0040:C5l0@000000P2R
`o000`000000X/;_0:;2k`030:;2k`@000000P2P_nl00`000000XKk^0:6nkP030:6nkP030000002Q
_Nd0XKg]0080000000<0Wkc]0000002O^n`0102O^n`010000000WkW[09nij`2O^N/500000080W[OZ
00<0000009jfj@2N]^T00`2N]^T010000000WkGX09nej02O]NP4000000<0W[?W00<0000009jbiP2N
/^H00P2N/^H010000000Wk7U09nai@2O/ND4000000@0Wk3T00<0000009n_h`2O[n<00P2O[n<01000
0000X:oR0:2_hP2P[n84000000@0XJkQ00<000000::^h02R[^000P2R[^000`000000XZkO0::^g`02
000000030::]g`000000XjgN00@0XjgN00<000000:B]gP2T[Mh00P2T[Mh400000080YJgM00<00000
0:J^g@2V[]d00`2V[]d00`000000YjkM0:N^g@0200000080YjkL00<000000:R^g02X[]`00`2X[]`0
0`000000Z:oL0:R_g0020:R_g0800000102Z[m`00`000000Zk3L0:^`g0030:^`g0<00000102//M`0
0`000000[;;L0:bbg0020:bbg0<00000102^/]`00`000000[[?L0:jcg0020:jcg08000001P2_]=`0
0`000000/;GM0;2eg@02000000030;2eg0000000/KKM00@0/KKM100000030;:gg@030000002b^=h0
/[SN0080/[SN0P0000060;>igP030000002c^]h0/k[N0080000000<0];[N0000002d^mh0102d^mh2
000000D0]KcO00<000000;Jmg`2f_Ml00P0000020;Jmg`030000002f_^00][kP0080][kP0P000006
0;Noh08000001@2h`>000`000000^<7Q0;S1h@0200000080^L7Q00@000000;W2h@2i`^40^L;Q0`00
00060;[3h@030000002ja>80^/CR00<0^/CR00@000000;[5hP00000000000`2kaN8010000000^lKR
0000000000030;c6hP030000002lan80_<OR008000000P2lan8010000000_<OS0;c7h`2lan<20000
00030;g8hP000000oooo02X0oooo00<000000?ooo`3oool0303oool00`3ogi`0B00007Boo`050?oo
o`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool0D@3oool002D0oooo00<0omnL04P0001d
_ol03@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo09<0oooo0P0000000`2Wbo80
00000:O:lP040:O:lP@000000P2VbO82000000<0Y/Oa100000040:C6l@030000002TaO00Y<G`0080
00000P2Ra?400`000000X/?`0:;3l0020:;3l0030000002R`^l0X/;_008000000P2Q`O000`000000
X;o_0:2ok`020:2ok`040000002Q_^h0XKk^0:6nkP@000000`2O_>d00`000000Wk_/09nkk00309nk
k0@000000P2M^N`00`000000W[OZ09jgjP0309jgjP040000002N]^T0W[KY09jfj@D000000P2M]>P0
0`000000W[?W09jci`0309jci`040000002N/^H0W[;V09jbiP@000000`2N/>D00`000000Wk3T09n`
i00209n`i0040000002O[n<0WjoS09n_h`D000000`2O[^800`000000XJkQ0:6^h@020:6^h@040000
002R[^00XZkP0::^h0@00000102R[Ml00`000000XjgN0:>]gP020:>]gP030000002T[Mh0Y:gN0080
00000P2T[Md00`000000YJgM0:F]g@020:F]g@030000002V[]d0YZkM0080YZkM0`0000040:N^g003
0000002X[]`0Z:kL00<0Z:kL00<000000:R_g02X[m`00P0000000`2Y[]`000000:Z_g0040:Z_g003
0000002[/=`0Zk3L0080000000<0Zk3K0000002//M`0102//M`00`000000[;;L0:bbg00200000080
[K7K00<000000:jbg02^/]`00P2^/]`00`000000[[?L0:jcg00200000080[k?L00<000000:ndg02_
]=`0102_]=`2000000@0/;GL00<000000;6fg@000000100000050;:gg@030000002b^=h0/[SN0080
00000P2c^=d00`000000/kWN0;>igP040;>igP800000102d^]h00`000000];_N0;BkgP0200000003
0;FkgP000000]KcO00D0]KcO0P0000040;Jmg`030000002f_^00][kP008000000P2g_]l00`000000
]koP0;Noh0020;Noh0030000002h_n00000000H0^<3P0P0000040;W1h@040000002i`^4000000000
00<0^/;Q00@000000;[3h@2j`n40^/?Q0P0000000`2j`n4000000;[4hP030;[4hP8000001P2kaN80
0`000000_<KR0;c6hP040;c6hP800000102lan8010000000_<OS0000000000030;g8hP030000003o
ool0oooo02X0oooo00<000000?ooo`3oool0303oool00`3OW4P0001809cOo`050?ooo`030=nLB000
B9`0gooo0080oooo00<0_g@0001d_`3oool0D@3oool002D0oooo00<0omnL04P0001d_ol03@3oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo08l0oooo0`0000040:O;lP030000002Wb_80
Yl[b008000000P2Vb_800`000000Y/Wb0:K9lP030:K9lP<000000`2Tao82000000<0Y<Ka0`000005
0:;4l@040000002R`o00X/?`0:;3l0<000001@2Q`O0010000000X;o_0:2ok`2P_nl4000000<0W[g^
00<0000009nlk@2O_>d00P2O_>d00`000000Wk_/09nkk00200000080WK[]00<0000009fik02M^N`0
0P2M^N`00`000000W[OZ09jgjP0209jgjP@000000P2L]NT00`000000WKCX09fdj00309fdj0040000
002N/nL0W[?W09jci`D000000P2M/NH00`000000W[3U09j`i@0209j`i@030000002O/>@0Wk3T00<0
Wk3T0`00000309j^hP030000002O[^80WjkR0080WjkR00<000000:6^h@2Q[^400`2Q[^43000000<0
XJgO00<000000::]g`2R[Ml00P2R[Ml010000000XjgN0:>]gP2S[Mh4000000@0Y:gM00<000000:F]
g@2U[Md00P2U[Md00`000000YZkM0:J^g@0200000080YZgL00<000000:N^g02W[]`00`2W[]`00`00
0000Z:kL0:R^g0020:R^g0<000000`2Y[]`00`000000ZZoL0:Z_g0030:Z_g0<00000102[/=/00`00
0000[;7L0:bag0020:bag0<00000102]/M/00`000000[[;L0:jbg0030:jbg0800000102_/m`00`00
0000[kCL0:ndg0020:ndg0<000001@2`]M`2000000050;6fg@00000000000;6fg00000001P2b]md2
000000@0/kSM00<000000;>igP2c^Mh00P2c^Mh3000000H0];[N0P0000040;FkgP040000002e_=l0
]KcO0;Flg`<000001P2f_Ml2000000D0]kkO00@000000;Noh000000000000P2h_n000`000000^<3P
0;S0h0020;S0h0@000001@2i`N400`000000^/;Q0;[2h@030;[2h@040000002j`n400000000000<0
^/?Q00<000000;[4hP2ja>800P0000020;_4h@040000002kaN80^lGR0;_5hP80000000<0_<GR0000
002la^800P2la^82000000030;c6hP000000_<OR00@0_<OR00<000000;g8hP2mb>80102mb>800`00
0000oooo0?ooo`0Z0?ooo`030000003oool0oooo00`0oooo00<0okmd00000000M;l0103oool01`3o
okl0M00007Boo`3oool0okmd0000M02oool0DP3oool002D0oooo00<0omnL04P0001d_ol03@3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo08`0oooo0P0000020:O<l`8000000`2Wbo83
000000D0Y/[b00<000000:K9lP2VbO800P0000000`2Tb?8000000:C7lP040:C7lP@000000P2Ra_80
0`000000X/Ca0:;4l@030:;4l@@000000P2Q`o400`000000XL7`0:71l0030:71l0@000000P2N_o00
0`000000W[g^09jmkP0309jmkP040000002O_>d0Wkc]09nlk@<00000102M^^d00`000000WKW/09fi
k00209fik0030000002N]nX0W[OZ008000000P2L]^/00`000000W;GY09bej@0209bej@030000002M
]>P0WKCX0080WKCX1000000209bbi`030000002M/NH0WK7V0080WK7V00<0000009j`i@2N/>D00`2N
/>D00`000000Wk3T09n`i0020000000309f_h`000000WZkR00<0WZkR00<0000009n^hP2O[^800`2O
[^800`000000XJkQ0:6^h@02000000030:2]h0000000XJgO00<0XJgO00<000000::]g`2R[Ml00`2R
[Ml3000000<0XjcN00<000000:B]g@2T[Md00`2T[Md010000000YJgM0:F]g@2U[Md3000000@0YZgL
00<000000:N^g02W[]`00`2W[]`00`000000Z:kL0:R^g002000000030:R^f`000000ZJkL00@0ZJkL
00<000000:Z_g02Z[m`00P0000000`2Z[m/000000:^`f`040:^`f`030000002//M`0[;7L00800000
0P2//M/00`000000[K7K0:faf`030:faf`030000002^/]`0[[;L0080000000<0[[;K0000002_/m`0
102_/m`010000000[kCL0000000000030;2dg0030000002`]M`0000000D00000102a]]`010000000
/[OM0;:gg@2b]md2000000030;>gg@000000/kSM00@0/kSM00@000000;>igP00000000000`2d^Md0
10000000];[N0;BjgP2d^]h3000000D0]K_N00@000000;Flg`00000000000`2f_=h010000000][gO
0;Jmg`2f_Ml2000000030;Nng`000000]kkO00D0]kkO00<000000;Roh02h_n000`2h_n0010000000
^<3P0000000000030;W1h0040000002i`N40^L7Q0;W1h@80000000<0^/;P0000002j`^400`2j`^42
000000H0^/?Q0P0000040;_4h@040000002kaN800000000000<0_<GR00@000000;c6hP0000000000
0`2la^800`000000_<OR0;c7hP020;c7hP030000002man80000000@0_LSR00<000000;g8hP000000
:`3oool00`000000oooo0?ooo`0=0?ooo`050?oog`2LB00000000000M02oool00P3oool01@3ookl0
M0000000000007@0_ooo05<0oooo000U0?ooo`030?oOW0180000M;oo00d0oooo00<000000?ooo`3o
ool03`3oool00`000000oooo0?ooo`280?ooo`<000001@2Wc?<400000080Y/_c0P0000030:K:lP@0
0000102Tb?800`000000Y<Ob0:C7lP0200000080XlOb00<000000:;6lP2Ra_800P2Ra_800`000000
X/Ca0:;4l@0200000080XLCb00<000000:73l@2Q`o400P2Q`o400`000000XL7`0:71l00200000080
W/7`00<0000009jol02N_o000P2N_o0010000000W[g^09jmkP2N_Nh500000080WK_^00<0000009fj
k@2M^^d00`2M^^d010000000WKW/09fik02M^N`3000000@0W;K[00<0000009bej@2L]NT00P2L]NT0
0`000000WKCX09fdj00200000080Vk?X00<0000009bbi`2L/^L00P2L/^L00`000000WK7V09faiP02
09faiP040000002N/>D0W[3U09j`i@@000000`2M[n<00`000000WZkR09j^hP0309j^hP040000002O
[^80WjkR09n^hP@000000`2P[N000`000000XJgO0:6]g`030:6]g`030000002R[Ml0XZgO00800000
00<0XZcN0000002S[=h00`2S[=h00`000000Y:gM0:B]g@030:B]g@@000000P2U[=`00`000000YZgL
0:J]g0030:J]g0040000002W[]`0YjkL0:N^g0@00000102X[]/00`000000ZJkL0:V^g0020:V^g0<0
0000102Z[m/00`000000Zk3K0:^`f`020:^`f`<00000102//M/00`000000[K7K0:faf`030:faf`<0
00000`2^/]/00`000000[k?L0:ncg0030:ncg0800000102`]=`3000000030;2eg000000000000080
/KGL00<000000;6fg02a]]`00P2a]]`010000000/[OM0000000000030;>gg@030000002c^=d0/kSM
00<0/kSM0P0000050;Big@040000002d^]h00000000000<0]K_N00<000000;FkgP2e^mh00P2e^mh2
000000D0][cN00@000000;Jmg`00000000000`2g_]l00`000000]kkO0;Nng`020;Nng`<000001P2h
_n000`000000^L7P0;W1h0030;W1h0@000000`2j`^000`000000^/;Q0;[2h@0200000080^/?Q00@0
00000;[3h@2j`n40^/?Q0`0000060;_4h@030000002laN80_<GR00@0_<GR00<000000;c6hP2la^80
102la^8010000000_<OR0000000000030;g7hP040000002mb>800000000000<0_LSR00<000000?oo
o`3oool0:03oool2000006d0oooo000S0?ooo`050?oog`2LB00000000000001d_ol0303oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo08<0oooo100000020:O=l`8000000P2Wc?<20000
0080Y/cc00<000000:K;l`2Vbo<00`2Vbo<3000000<0Y<Wc0P0000030:C8lP<000001@2Sao801000
0000X/Kb0:;6lP2Ra_83000000D0XLCb00@000000:73l@2Q`o40XL?a0`00000509k1l0040000002N
_o00W[o`09jol0@000000`2L_Nl00`000000WK_^09fkkP0209fkkP040000002M^^d0WK[]09fjk@D0
00000P2K^>`00`000000W;K[09bfj`0309bfj`040000002L]NT0W;GY09bej@<00000102K/nP00`00
0000W;;W09bbi`0209bbi`040000002M/NH0WK7V09faiPD000000`2L[n@00`000000WJoS09f_h`02
09f_h`040000002N[^80WZkR09j^hPD000000P2O[N400`000000X:gP0:2]h0030:2]h0040000002Q
[Ml0XJgO0:6]g`@000000`2R[=h00`000000XjcN0:>/gP030:>/gP030000002T[Md0Y:gM00800000
00<0Y:cM0000002U[=`00`2U[=`00`000000YZgL0:J]g0030:J]g0@000000P2W[M/00`000000Z:kK
0:R^f`030:R^f`030000002Y[]`0ZJkL008000000P2Y[]/00`000000ZZoK0:Z_f`020:Z_f`030000
002[/=/0Zk3K008000000P2//=/00`000000[;7K0:baf`030:baf`030000002]/M/0[K7K00800000
00<0[[7K0000002^/]/0102^/]/00`000000[k?L0:ncg002000000030:ncf`000000/;CL00<00000
0P2`]=`2000000@0/KGL00<000000;6fg02a]]`00`2a]]`00`000000/kOM0;>gg@040;>gg@030000
002c^=d0/kSM008000000P2d^Md00`000000];WM0;Big@020;Big@8000001@2e^mh00`000000]K_N
0;FkgP0200000080][cN00<000000;JlgP2f_=h00`2f_=h00`000000]kkO0;Nng`040;Nng`040000
002g_]l00000000000<0^;oO00@000000;Roh02h_n00^;oP0P0000000`2i`=l000000;W1h0030;W1
h0<000001@2j`^02000000@0^/?Q00@000000;[3h@00000000000`2ka>4010000000^lCQ0;_4h@2k
a>42000000030;c5h@000000_<GR00<0_<GR00<000000;c6h@0000001@2la^82000000H0_LOR00<0
00000;g8hP2mb>80102mb>800`000000oooo0?ooo`0T0?ooo`<000000P3oool00`000000oooo0?oo
o`1Z0?ooo`009@3oool00`3o_g@0000007Boo`0<0?ooo`040000003oool0000000000140oooo00<0
00000?ooo`3oool0P03oool200000080Ylgd00<000000:O=l`2WcO<00`2WcO<2000000D0Y/cc00<0
00000:K;l`2Vbo<00P0000000`2Tbo@000000:C9l`040:C9l`@000000P2Sb?<00`000000XlOb0:?7
lP030:?7lP@000000P2QaO82000000<0XLCb1@00000209k2l@030000002N`O00W/7`00<0W/7`1000
000209bnl0030000002L_Nl0W;g_00<0W;g_00@0000009fkkP2M^nh0WK_^1000000309^ik@030000
002K^>`0VkS/0080VkS/00@0000009bfj`2L]^/0W;K[1@00000209Zdj@030000002K/nP0Vk?X00<0
Vk?X00@0000009bbi`2L/^L0W;;W1000000309^`iP030000002L[n@0W:oT0080W:oT00@0000009f_
h`2M[n<0WJoS1@00000309f]hP030000002O[N40WjgQ0080WjgQ00<000000:2]h02P[N000P2P[N04
00000080XJcO00<000000::/gP2R[=h00`2R[=h00`000000XjcN0:>/gP020:>/gP<00000102T[=d0
0`000000YJcL0:F/g0020:F/g0030000002V[M`0YZgL008000000P2V[=/00`000000YjgK0:N]f`02
0:N]f`030000002X[]/0Z:kK0080Z:kK0`0000040:V^f`030000002Z[m/0ZZoK00<0ZZoK0P000004
0:b`f`030000002//M/0[;7K00<0[;7K0`0000040:jaf`030000002^/]/0[[;K0080[[;K0`000002
0:ncf`<000000`2`]=`2000000030;6df`000000/KGL00@0/KGL00<000000;6fg02a]]`00P000000
0`2b]m`000000;>gg@050;>gg@800000102d^Md00`000000];WM0;Big@02000000030;Fjg@000000
]K_N00D0]K_N0P0000050;JlgP030000002f_=h0][cN0080000000<0]kgN0000002g_]l0102g_]l2
000000D0^;oO00@000000;Roh000000000000`2i`=l00`000000^L7P0;W1h00200000080^L7P00@0
00000;[2h02j`^00^/;P0`0000060;[3h@030000002ka>40^lCQ00<0^lCQ00@000000;_4h@000000
0000102laN4010000000_<GR0000000000030;c6h@030000002la^80_<KR008000000P2ma^801000
0000_LOR0;g7hP2man82000000030;g7hP000000_LSR0080_LSR0P0000000`2nb>8000000?ooo`0Y
0?ooo`030000003oool0oooo06X0oooo000d0?ooo`800000503oool00`000000oooo0?ooo`1m0?oo
o`8000001@2WcO@00`000000Ylgc0:O=l`02000000030:K<m0000000Y/cc00@0Y/cc0`0000040:C;
m0030000002TbO<0Y<Wc008000000P2SbO@00`000000XlSc0:?8l`020:?8l`030000002Sao80XlOb
008000000P2Qao<00`000000XLGb0:75lP030:75lP<000000`2Na?800`000000W/;a09k2l@0209k2
l@030000002N`O00W/7`008000000P2L`?400`000000W;k`09bnl00209bnl0040000002L_Nl0W;g_
09bmk`D000000P2J^^h00`000000VkW]09^ik@0309^ik@040000002K^>`0VkS/09^hk0@000000`2J
]N/00`000000V[CY09Zdj@0209Zdj@040000002K/nP0Vk?X09^cj0D000000P2J/NL00`000000Vk3V
09^`iP0209^`iP030000002L[n@0W:oT00<0W:oT0`00000309b]h`030000002M[N80WJgR0080WJgR
00<0000009n]h@2O[N400P2O[N400`000000X:gP0:2]h00200000080Wj_O00<000000:6/g`2Q[=l0
0P2Q[=l00`000000XZcN0::/gP020::/gP030000002S[=h0XjcN008000000P2SZmd00`000000Y:cM
0:B/g@020:B/g@030000002U[=`0YJcL0080YJcL0`0000040:J/f`030000002W[M/0YjgK0080YjgK
00<000000:R^f`2X[]/00P0000020:R]f`030000002Y[]/0ZJkK00<0ZJkK00<000000:Z_f`2Z[m/0
0P0000000`2[[mX000000:b`f`040:b`f`030000002//M/0[;7K0080000000<0[K3J0000002^/M/0
102^/M/00`000000[[;K0:jbf`02000000030:nbf`0000000000008000000P2_/m/010000000/;CL
0000000000030;6df`030000002a]M`0/KGL00<0/KGL0`0000030;:gg0030000002c]md0/kOM0080
/kOM0`0000060;Big@800000102e^]d010000000]K_N0;FkgP2e^mh2000000030;Jkg@000000][cN
00D0][cN0P0000040;NmgP030000002g_]l0]kkO008000000P2h_]h00`000000^;oO0;Rog`020;Ro
g`8000001P2i`=l2000000@0^L7P100000030;[2h0040000002j`n40^/?Q0;[3h@80000000<0^l?P
0000002ka>40102ka>400`000000_<GQ0;c5h@050;c5h@030000002la^40_<KQ00@0_<KQ0P000004
0;g6hP040000002man800000000000<0_LOR00@000000;g8hP00000000000`2nb>800`000000oooo
0?ooo`0Y0?ooo`030000003oool0oooo06T0oooo000d0?ooo`030000003oool0oooo01@0oooo00<0
00000?ooo`3oool0NP3oool2000000030:O>m0000000000000<0Ylgd100000040:K<m0030000002V
c?<0Y/cc008000000P2Ubo@2000000<0Y<_d0`0000050:?9m0040000002Sb?<0XlSc0:?8l`<00000
1@2Qao<00`000000XLGb0:75lP020000000309k5l`000000W/Cb00<0W/Cb00@0000009k2l@2N`_40
W/;a1000000409c0l@040000002L__00W;k`09bnl0D000000`2J_?000`000000V[[^09ZjkP0209Zj
kP040000002K^Nd0VkW]09^ik@D000000P2I]n`00`000000V[G[09Zej`0209Zej`040000002J]>T0
V[CY09Zdj@D000000P2I/NP00`000000V[7W09Zai`0209Zai`030000002K/>H0Vk3V00<0Vk3V00<0
000009b_i02L[n@00P0000000`2K[^@0000009b]h`0309b]h`030000002M[N80WJgR00<0WJgR00@0
000009n]h@2O[N40WjgQ0`00000409n[g`030000002Q[=l0XJcO0080XJcO00@000000::/gP2R[=h0
XZcN100000040:>[g@030000002T[=d0Y:cM00<0Y:cM00@000000:F/g000000000000P2UZm`00`00
0000YZcK0:J/f`030:J/f`030000002W[M/0YjgK0080YjgK0P0000040:R]f`030000002Y[]/0ZJkK
00<0ZJkK0`0000030:^_fP030000002//=/0[;3K00<0[;3K0`0000040:f`fP030000002^/M/0[[7K
0080[[7K0`0000000`2_/]/0000000000002000000@0[k?K0P0000060;6df`030000002a]M`0/KGL
0080000000<0/[KK0000002b]m`0102b]m`010000000/kOM0000000000030;>hg0040000002d^Md0
];WM0;Big@<000001@2e^]d010000000]K_N0000000000030;Jkg@030000002f_=h0][cN0080][cN
0`0000060;NmgP800000102h_]h00`000000^;oO0;Rog`0200000080^KoO00@000000;W0g`2i`=l0
^L3O0`0000060;W1h0800000102j`^0010000000^/?Q0000000000040;_3h0030000002ka>40^lCQ
0080000000<0^lCQ0000002laN400`2laN42000000030;c5h@000000_<KQ0080_<KQ0`0000060;g6
hP030000002man80_LOR00@0_LOR00<000000;k8hP2nb>80102nb>800`000000oooo0?ooo`0X0?oo
o`030000003oool0oooo06T0oooo000d0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3o
ool0M@3oool4000000D0Ylkd0`0000030:K=m08000000`2Vc?@3000000D0YL_d100000020:?:m080
00000`2SbO@500000080XLSd0P0000030:77l`@00000102NaO<010000000W/Cb09k4lP2Na?840000
00<0W<;b00<0000009c0l@2L`?400`2L`?43000000<0V[ka00<0000009Zll02J_?000P2J_?001000
0000V[[^09ZjkP2J^^h5000000<0VKS^00<0000009Vgk02I]n`00P2I]n`010000000V[G[09Zej`2J
]N/4000000<0VK;Y00<0000009Vaj02I/NP00P2I/NP00`000000V[7W09Zai`0209Zai`040000002K
/>H0Vk3V09^`iP@000000`2K[^@00`000000W:gS09b]h`0309b]h`040000002M[N80WJgR09f]hPD0
00000P2NZn000`000000Wj_O09n[g`0309n[g`040000002Q[=l0XJcO0:6/g`@000000`2QZmd00`00
0000Xj_M0:>[g@030:>[g@040000002T[=d0Y:cM0:B/g@<00000102UZm`00`000000YZcK0:J/f`03
0:J/f`030000002W[M/0YjgK0080000000<0YjcK0000002X[M/0102X[M/00`000000ZJkK0:V^f`02
000000030:Z^fP000000ZjoJ00@0ZjoJ00<000000:b`f`2//=/00P0000000`2/[mX000000:f`fP04
0:f`fP030000002^/M/0[[7K0080000000@0[[7J00000000000000000`2_/]/00`000000[k?K0:nc
f`0200000080/;?K00<000000;6df`2a]=/0102a]=/2000000@0/[KK00<000000;:gg02b]m`00P2b
]m`2000000D0/kSL00@000000;Big@00000000000`2d^M`00`000000]K[M0;Fjg@020;Fjg@800000
1P2f^md010000000][cN0000000000030;NlgP040000002g_Mh0]kgN0;NmgP<000001P2h_]h20000
00@0^KoO00@000000;W0g`00000000000`2i`=l010000000^L7P0;W1h02i`N02000000030;[1g`00
0000000000@0^/;P00<000000;_3h02k`n001@2k`n02000000@0^lCQ00@000000;c5h@0000000000
0`2laN4010000000_<KQ0000000000030;g6h@040000002ma^80_LKR0;g6hP80000000<0_LOQ0000
002man800P2man82000000030;k7hP000000_/SR0080_/SR0P0000000`2nb>8000000?ooo`0Y0?oo
o`030000003oool0oooo06P0oooo000d0?ooo`030000003oool0oooo01H0oooo0P00001b0?ooo`<0
00000`2Wc_D200000080Ylkd0P0000000`2Vc_D000000:K=m0040:K=m0@000000P2Uc?@200000080
YL_d0P0000020:?;m@030000002Sb_@0Xl[d00<0Xl[d0`0000030:79m0030000002Qb?@0XLSd00<0
XLSd0`00000309o6m0040000002NaO<0W/Gc09k5l`D000000P2L`o<00`000000W<;b09c2lP0309c2
lP030000002L`?40W<3a0080000000<0V[ob0000002J__40102J__4010000000V[c`09Zll02J_?04
000000<0V;W_00<0000009VhkP2I^>h00P2I^>h010000000VKO/09Vgk02I]n`500000080V;C[00<0
000009Vbj@2I/^T00P2I/^T00`000000VK7X09Vaj00209Vaj0040000002J/NL0V[7W09Zai`D00000
0`2J[^D00`000000VjkT09^^i00209^^i0040000002L[N<0W:gS09b]h`D000000P2M[>400`000000
WZ_P09j[h00209j[h0030000002OZml0Wj_O00<0Wj_O100000020:2ZgP030000002QZmd0XJ_M0080
XJ_M00<000000:>[g@2SZmd00`2SZmd400000080Y:_L00<000000:F[g02UZm`00`2UZm`010000000
YZcK0:J/f`2V[=/4000000<0YjcK00<000000:R]f`2X[M/00`2X[M/3000000@0ZZkJ00<000000:^_
fP2[[mX00P2[[mX3000000@0[:oJ00<000000:f`fP2]/=X00`2]/=X2000000050:jafP0000000000
0:jafP0000001@2_/]/2000000@0/;?K00<000000;6df`2a]=/00P2a]=/3000000D0/[KK00<00000
0;:gg02b]m`00P0000020;>gg0030000002c^=`0/kSL0080/kSL0P0000050;Big0030000002e^]d0
]K[M008000000P2f^]d00`000000][_M0;Jkg@030;Jkg@8000001@2g_=h010000000]kgN00000000
00030;RmgP040000002h_]h0^;kN0;RngP<000001P2i_ml00`000000^L3O0;W0g`030;W0g`040000
002i`N000000000000@0^/7O00<000000;[2h02j`^000P0000000`2k`^0000000;_3h0030;_3h0<0
00001P2ka>400`000000_<GQ0;c5h@040;c5h@030000002ma^40_LKQ00<0_LKQ00@000000;g6hP00
000000000`2man4010000000_LOR0000000000030;k7hP040000002nb>800000000000<0_/SR00<0
00000?ooo`3oool0:@3oool00`000000oooo0?ooo`1W0?ooo`00=03oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo06d0oooo0P0000000`2WcoD000000:O>m@040:O>m@<00000102Vc_D0
0`000000Y/gd0:K=m00200000080YLge00<000000:G<m02Uc?@00`2Uc?@2000000D0Xl_e00<00000
0:?:m02Sb_@00P0000000`2Rb_D000000:79m0040:79m0030000002Qb?@0XLSd0080000000<0WlSd
0000002Oa_@0102Oa_@3000000<0W<Gd00<0000009c3l`2L`o<00P2L`o<010000000W<;b09c2lP2L
`_84000000@0V[ob00@0000009Znl@2J__40V[ka1@00000209Rkl0030000002H^Nl0V;W_00<0V;W_
00@0000009VhkP2I^>h0VKS^1000000309Nek@050000002H]>/0V;C[09Rdj`0000001@2I/^T01000
0000VK7X09Vaj02I/NP4000000<0VJoW00<0000009Z^i@2J[^D00P2J[^D010000000VjkT09^^i02K
[^@5000000<0VjcR00<0000009f/h@2M[>400P2M[>400`000000WZ_P09j[h00209j[h0030000002O
Zml0Wj_O0080000000<0WZ[O0000002PZ]h00`2PZ]h00`000000XJ_M0:6[g@030:6[g@030000002S
Zmd0Xj_M0080000000<0Xj[L0000002TZm`00`2TZm`00`000000YJ_L0:F[g0030:F[g0@000000P2V
Zm/00`000000YjcK0:N/f`030:N/f`030000002X[M/0Z:gK0080000000<0ZJgJ0000002Z[]X0102Z
[]X00`000000ZjoJ0:^_fP02000000030:b^fP000000[:oJ00@0[:oJ00@000000:f`fP2]/=X0[K3J
1@0000040:jafP030000002_/]/0[k;K0080000000<0/;;J0000002`/m/0102`/m/010000000/KCK
0000000000030;6df`030000002b]]/0/[KK0080/[KK0`0000040;>gg0030000002c^=`0/kSL0080
00000P2d^=`00`000000];WL0;Big0030;Big08000001@2f^]d00`000000][_M0;Jkg@0200000003
0;Nkg@000000]kcN00D0]kcN00<000000;RmgP2h_Mh00`2h_Mh010000000^;kN0000000000030;Rn
gP040000002i_ml0^KoO0;Vog`80000000<0^KoO0000002i`=l0102i`=l00`000000^/7O0;[1g`05
0;[1g`800000102k`^0010000000^l?P0000000000030;_3h0040000002ka>40^lCQ0;_4h@800000
00<0_<CQ0000002laN400P2laN42000000030;g5h@000000_LKQ00<0_LKQ0P0000060;g7h@030000
002nan80_/OR00@0_/OR00<000000;k8hP2nb>80102nb>800`000000oooo0?ooo`0X0?ooo`030000
003oool0oooo06L0oooo000c0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0J@3o
ool3000000@0Yloe00<000000:O>m@2Wc_D00P0000020:K>m@8000000`2Vc_D3000000D0YLge00<0
00000:G<m02Uc?@00P0000000`2Tc?D00000000000030:?;m@@00000102Rb_D010000000XLWd0:79
m02QbO@4000000@0WlSd00<0000009o6m02Oa_@00P0000000`2Ma_D0000009c5m00409c5m0040000
002L`o<0W<?c09c3l`@000000`2J`O<2000000<0V[ob1000000309NmlP030000002H^o00V;_`0080
V;_`00@0000009Rik`2H^Nl0V;W_1@00000209JgkP030000002G]Nd0UkG]0080UkG]00<0000009Rd
j`2H]>/00P2H]>/010000000VK;Y09Vbj@2I/^T500000080V;3X00<0000009V_i`2I[nL00P2I[nL0
0`000000VZkU09Z^i@0309Z^i@<000000`2J[>@00`000000VjcR09^/hP0209^/hP030000002M[>40
WJcQ0080WJcQ00@0000009j[h02NZn00WZ_P1000000409jZg`030000002PZ]h0X:[N0080X:[N00@0
00000:6[g@2QZmd0XJ_M100000040:>Zg0030000002TZm`0Y:_L0080Y:_L00<000000:F[g02UZm`0
0P0000000`2UZm/000000:J[f`040:J[f`030000002W[=/0YjcK0080YjcK0`0000040:V]fP030000
002Z[]X0ZZkJ0080ZZkJ0`0000040:b^fP030000002/[mX0[:oJ00<0[:oJ1@0000000`2^/=X00000
0:jafP050:jafP8000000`2`/]X00`000000/;?K0;2cf`030;2cf`8000001@2a]=/00`000000/[KK
0;:ff`02000000030;>ff`000000/kOL00D0/kOL0P0000040;Bhg0030000002d^M`0];WL0080];WL
00<000000;Fig00000001P2f^]d2000000@0]k_M00@000000;NlgP2g_=h0]kcN0`0000060;RmgP03
0000002h_]h0^;kN00<0^;kN00@000000;Vog`00000000000`2i_ml00`000000^L3O0;W0g`020;W0
g`030000002j`Ml0000000@0^/7O100000050;_2h0030000002k`n00^l?P00<0^l?P00@000000;_4
h@00000000000`2la>4010000000_<GQ0000000000030;g5h@030000002ma^40_LKQ008000000P2m
a^4010000000_LOQ0;g7h@2man42000000030;k7h@000000_/OR0080_/OR0P0000000`2nb>400000
0;k8hP020;k8hP80000000<0_lSR0000003oool0:@3oool00`000000oooo0?ooo`1V0?ooo`00<`3o
ool010000000oooo00000000000J0?ooo`030000003oool0oooo06H0oooo0P0000020:O?m@800000
0`2WcoD3000000D0Y/ke100000020:G>m@8000000`2UcOD4000000D0Y<ce0`0000030:;;m@800000
0`2Rb_D4000000<0WlWe00@0000009o8m02Ob?@0WlSd1000000409g6m@040000002LaO@0W<Gd09c5
m0D000000P2J`o@2000000<0V/7c1@00000209Nol`030000002G_O80Ukgb00<0Ukgb00@0000009Rk
l02H^o00V;_`1000000209Fhl0030000002F]nh0U[O^0080U[O^00<0000009Nek@2G]Nd00`2G]Nd0
10000000V;C[09Rdj`2H]>/4000000<0U[7Z00<0000009R`j02H/>P00P2H/>P00`000000VJoW09V_
i`0209V_i`030000002J[^D0VZkU0080000000<0V:gU0000002J[>@00`2J[>@00`000000VjcR09^/
hP0309^/hP040000002M[>40WJcQ09f/h@@000000`2MZ]l00`000000WZ[O09jZg`0209jZg`030000
002PZ]h0X:[N0080X:[N100000020:6Yg0030000002SZ]`0Xj[L00<0Xj[L00<000000:B[g02TZm`0
0P2TZm`3000000@0YJ_K00<000000:J[f`2VZm/00P2VZm/00`000000YjcK0:N/f`02000000030:R/
fP000000ZJgJ00@0ZJgJ00<000000:Z^fP2Z[]X00P0000020:^]f@030000002/[]X0[:kJ0080[:kJ
00<000000:b_fP2/[mX00P2/[mX3000000D0[[3J00@000000:jafP2^/MX0[[7J0`0000050;2bfP03
0000002`/m/0/;?K0080000000<0/K?J0000002a]=/01@2a]=/2000000@0/kKK00@000000;>gg02c
]m`0/kOL0P0000000`2d]m/000000;Bhg0040;Bhg0040000002d^M`0000000000080]KWL00<00000
0;Jjg@2f^]d00P2f^]d3000000D0]k_M00@000000;NlgP00000000000`2g_=d010000000^;gN0;Rm
gP2h_Mh2000000030;RngP000000^;kN00<0^;kN0P0000060;Vog`040000002i`=l00000000000<0
^/7O00@000000;[1g`00000000000`2k`]l200000080^l;P0P0000000`2k`n0000000;_3h0030;_3
h08000001P2la>400`000000_LGQ0;g5h@040;g5h@800000102ma^4010000000_LOQ000000000003
0;k7h@040000002nan800000000000<0_/SQ00@000000;k8hP00000000000`2ob>800`000000oooo
0?ooo`0Y0?ooo`030000003oool0oooo06D0oooo000c0?ooo`8000007@3oool00`000000oooo0?oo
o`1R0?ooo`<000001@2WcoD400000080Y/oe0P0000020:K>m@8000000P2Uc_D00`000000YLke0:G>
m@030:G>m@<000000`2TcOD200000080Y<ce0P0000000`2Rc?H000000:;;m@040:;;m@@000000P2P
b_H2000000<0WlWe1000000309g8mP8000000`2Ma_D4000000<0V/Ce00<0000009[3m02J`o@00`2J
`o@3000000<0Ul3d00<0000009Nol`2G_o<00P2G_o<010000000Ukgb09NmlP2G_O8500000080UK[a
00<0000009Fhl02E^?000P2E^?000`000000U[O^09JgkP0209JgkP040000002G]Nd0UkG]09Nek@D0
00000P2E/^/00`000000U[7Z09JajP0309JajP040000002H/>P0V;3X09R`j0P000000`2H[ND00`00
0000VZcT09Z/i00309Z/i0040000002K[>80VjcR09^/hPD000000P2KZ^400`000000WJ[O09fZg`02
09fZg`030000002NZ]l0WZ[O00<0WZ[O00@000000:2ZgP00000000000P2OZMd00`000000XJWL0:6Y
g0020:6Yg0030000002SZ]`0Xj[L00<0Xj[L00@000000:B[g000000000000P2TZM/00`000000YJ_K
0:F[f`020:F[f`030000002VZm/0YZ_K0080YZ_K0`0000040:R/fP030000002Y[MX0ZJgJ0080ZJgJ
0`0000040:^]f@030000002/[]X0[:kJ00<0[:kJ100000020:f_f@030000002^/=X0[[3J00<0[[3J
00@000000:jafP00000000000`2_/MX00`000000/;;J0;2bfP020;2bfP<00000102a/mX010000000
/KCK0;6df`2a]=/2000000030;:efP000000/kKK00<0/kKK00@000000;>gg000000000000`2d]m/0
0`000000];SL0;Bhg0030;Bhg08000001@2e^M`010000000][[M0000000000030;Jjg0030000002g
^md0]k_M0080]k_M0P0000050;Nlg@040000002h_Mh00000000000<0^;kN00<000000;RngP2h_]h0
0P0000020;VogP030000002i_ml0^KoO00<0^KoO0P0000060;[1g`030000002k`]l0^l;O00<0^l;O
100000030;_3h0030000002k`n00^l?P008000000P2la>0010000000_<CQ0;c4h@2la>4200000003
0;g5h0000000_LGQ0080_LGQ0`0000060;g6h@030000002nan40_/OQ00@0_/OQ00<000000;k8h@2n
b>40102nb>400`000000_lSR0;o8hP040;o8hP030000003oool0oooo02P0oooo00<000000?ooo`3o
ool0I@3oool003<0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`1M0?ooo`@00000
0P2VcoD200000080Yloe0P0000020:K?m@030000002VcoD0Y/oe00<0Y/oe0P0000050:G>m@030000
002Uc_D0YLke0080000000<0Y<kf0000000000000`2TcOD4000000@0X/cf00<000000:;;m@2RboD0
0P0000020:3;mP030000002Pb_H0X<[f0080X<[f1000000209k9mP030000002Mb?H0WLSf00<0WLSf
1000000209_6mP030000002Ja?D0V/Ce00<0V/Ce00<0000009[3m02J`o@00P0000000`2G`_D00000
09O0m00409O0m0040000002G_o<0Ukoc09Nol`@000000`2D_?800`000000UK[a09Fjl@0209Fjl@05
0000002E^?00UKS`09Fhl00000000`2F]nh5000000<0U;C]00<0000009Fbj`2E/^/00P2E/^/01000
0000U[7Z09JajP2F/NX500000080UJkX00<0000009N]iP2G[NH00P2G[NH00`000000V:gU09R]i@02
09R]i@040000002J[>@0VZcT09Z/i0D000000P2JZ^800`000000Vj[Q09^Zh@0209^Zh@030000002M
Z]l0WJ[O00<0WJ[O00@0000009jZg`2NZ]l0WZ[O0`00000409nYg@030000002QZM`0XJWL0080XJWL
00@000000:>Zg02SZ]`0Xj[L100000040:BYf`030000002UZm/0YJ_K00<0YJ_K00@000000:J[f`00
000000000P2WZmX00`000000Z:cJ0:R/fP020:R/fP030000002Y[MX0ZJgJ008000000P2Y[=T00`00
0000ZjgI0:^]f@030:^]f@030000002/[]X0[:kJ00@00000102][mT00`000000[[3J0:j`fP030:j`
fP8000001@2_/MX00`000000/;;J0;2bfP02000000030;6bfP000000/K?J00@0/K?J00@000000;6d
f`00000000000`2b]MX00`000000/kKK0;>ff`030;>ff`030000002d]m/0];OK00@0];OK00<00000
0;Bhg02d^=`00P0000000`2e^=/000000;Fig0050;Fig0030000002f^]`0][[L00<0][[L00<00000
0;Nkg@2g^md00P0000020;Nlg@030000002g_=d0]kcM0080]kcM0P0000060;RngP8000001@2i_mh0
0`000000^KoO0;Vog`0200000080^/3N00@000000;[1g`2j`Ml0^/7O0`0000060;_2g`8000001@2k
`n02000000@0_<CP00@000000;c4h@00000000000`2maN0010000000_LGQ0000000000030;g5h@04
0000002ma^40_LKQ0;g6h@80000000<0_/KQ0000002nan400P2nan42000000030;k7h@000000_/SQ
0080_/SQ0P0000000`2ob>4000000;o8hP020;o8hP80000000<0_lSQ0000003oool0:@3oool00`00
0000oooo0?ooo`1T0?ooo`00<`3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo05X0
oooo0P0000020:K?m@8000000`2VcoD3000000D0Y/oe00<000000:K?m@2VcoD00P0000000`2UcoH0
0000000000030:G>m@@000001@2Tc_H3000000<0Xlgf0P0000030:;<mP<00000102PboH00`000000
X<[f0:3:mP0200000080W/[f00<0000009k9mP2NbOH00P2NbOH00`000000WLSf09g8mP0200000080
VlOf00<0000009_6mP2Ka_H00P2Ka_H010000000V/Ce09[4m@2Ja?D4000000@0Ul;e00@0000009O0
m02G`?@0Ul3d1@00000209Bnm0030000002D_?80U;cb0080U;cb00@0000009Fjl@2E^_40UK[a2@00
000309>ekP030000002D]>d0U;C]0080U;C]00@0000009Fbj`2E/^/0UK;[1@00000209B_j@030000
002E[^P0UJkX0080UJkX00<0000009N]iP2G[NH00`2G[NH010000000V:gU09R]i@2H[ND400000080
V:[S00<0000009ZZhP2JZ^800`2JZ^800`000000Vj[Q09^Zh@0209^Zh@040000002MZ]l0WJ[O09fZ
g`D000000P2NZ=h00`000000WjWM09nYg@0309nYg@030000002QZM`0XJWL0080XJWL0`0000030::Y
f`030000002TZM/0Y:WK00<0Y:WK00@000000:F[f`2UZm/0YJ_K0`0000040:N[fP030000002X[=X0
Z:cJ00<0Z:cJ0P0000040:V/f@030000002[[MT0ZjgI00<0ZjgI1@0000000`2/[]T000000:f_f@04
0:f_f@030000002^/=X0[[3J0080000000<0[k3I0000002_/MX01@2_/MX2000000@0/K;J00<00000
0;6cfP2a/mX00P2a/mX2000000H0/[GJ00<000000;>ff`2c]]/00P0000000`2d]]/000000;Bgf`05
0;Bgf`800000102e^=/010000000]KWL0;Fig02e^M`3000000H0][[L0P0000040;Nlg@030000002g
_=d0]kcM008000000P2h_Md00`000000^;kN0;RngP020;RngP030000002i_]d0000000H0^KoN0P00
00040;[0gP040000002j`Ml00000000000<0^l7O00@000000;_2g`2k`]l0^l;O0P0000000`2k`]l0
0000000000020;_3h0<000001P2la>000`000000_LGP0;g5h0040;g5h0030000002maN40_LGQ00<0
_LGQ00@000000;g6h@00000000000`2na^4010000000_/OQ0000000000030;k7h@040000002nb>40
0000000000<0_lSQ00@000000;o8hP00000000000`2ob>400`000000oooo0?ooo`0Y0?ooo`030000
003oool0oooo06<0oooo000c0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0E`3o
ool2000000D0Y/oe100000020:K?m@8000000`2VcoD4000000D0YLof0`0000030:C>mP8000000P2T
c_H2000000030:?>mP000000Xlgf00@0Xlgf100000020:7<mP8000000`2PboH3000000D0W/[f00@0
000009k9mP2NbOH0W/Wf0`00000509_7mP040000002Ka_H0VlKf09_6mP@000000`2Ha?H2000000<0
Ul;e1000000309C0m@030000002D__@0U;kd0080U;kd00@0000009BllP2D_?80U;cb1000000309:i
lP050000002B]o00T[O`09:gl00000001@2C]Nh010000000U;C]09Bdk@2D]>d4000000<0Tk7[00<0
000009B_j@2D[nT00P2D[nT00`000000UJkX09F^j00209F^j0040000002G[NH0UjgV09N]iPD00000
0P2GZn@00`000000V:[S09RZh`0209RZh`030000002JZ^80VZ[R0080VZ[R00@0000009^Zh@2KZ^40
Vj[Q1@00000209bXgP030000002NZ=h0WZSN0080WZSN00<0000009nYg@2OZMd00`2OZMd00`000000
XJWL0:6Yg002000000030:6Xf`000000XZWK00<0XZWK00<000000:BYf`2TZM/00`2TZM/400000080
YJWJ00<000000:N[fP2WZmX00`2WZmX00`000000Z:cJ0:R/fP02000000030:R[f@000000ZJcI00@0
ZJcI00<000000:^]f@0000001@0000030:b^f@030000002][mT0[JoI00<0[JoI0`0000040:n`f@04
0000002_/MX0[k7J0:nafP80000000<0/;7I0000002a/]X00`2a/]X00`000000/K?J0;6cfP020000
0080/[CJ00<000000;:efP2b]MX0102b]MX2000000<0];KK00<000000;Bgf`2d]m/00P2d]m/30000
00D0]KSK00@000000;Fig000000000000`2f^]`010000000][[L0;Jjg02f^]`2000000030;Nkg000
0000]kcM00@0]kcM0P0000050;Rmg@040000002h_]h0000000000080^KkM00<000000;VogP2i_mh0
0P2i_mh3000000H0^/3N00<000000;_1g`2k`Ml00`2k`Ml010000000^l;O0000000000030;_2g`@0
00000`2l`ml010000000_<CP0;c4h02la>02000000030;g4h0000000_LGP0080_LGP0P0000000`2m
aN0000000;g5h@030;g5h@8000001P2na^400`000000_/OQ0;k7h@040;k7h@030000002ob>40_lSQ
00@0_lSQ00<000000;o8h@2ob>40102ob>400`000000oooo0?ooo`0X0?ooo`030000003oool0oooo
06<0oooo000b0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0E03oool200000003
0:K@m@00000000000080Y/oe0P0000020:K?mP030000002VcoD0Y/oe00<0Y/oe0`0000030:K?mP80
00000P2UcoH2000000030:G?mP000000000000<0Y<kf100000040:?>mP030000002ScOH0Xlgf0080
00000P2RcOL00`000000XLcf0:7<mP030:7<mP<000000P2Oc?L2000000<0W/[f1@00000209c9m`80
00000`2KaoH500000080V<Kg0P00000309S4mPD000000P2D`_H00`000000U<3e09C0m@0209C0m@04
0000002D__@0U;kd09Bnm0D000000P2A^o<00`000000T[Wb09:ilP0309:ilP050000002B]o00T[O`
09:gl00000000`2C]Nh500000080TK;]00<0000009>aj`2C/N/00P2C/N/00`000000U:oY09B_j@02
09B_j@040000002E[^P0UJkX09F^j0D000000P2E[>H00`000000Uj_T09N[i00209N[i0030000002H
Z^<0V:[S0080V:[S00@0000009ZZhP2JZ^80VZ[R1@00000209ZXg`030000002LZ=h0W:SN00<0W:SN
00<0000009jXgP2NZ=h00P2NZ=h010000000WjWM09nYg@2OZMd4000000<0XJSK00<000000::Yf`2R
ZM/00`2RZM/00`000000Y:WK0:BYf`02000000030:BXfP000000YJWJ00<0YJWJ00<000000:N[fP2W
ZmX00`2WZmX3000000<0Z:_I00<000000:V/f@2Y[=T00`2Y[=T2000000050:^]f@00000000000:^]
f00000001@2/[]T00`000000[JoI0:f_f@02000000030:j_f@000000[k3I00@0[k3I00@000000:na
fP00000000000`2`/MT00`000000/K;J0;6bfP030;6bfP800000102b]=X00`000000/[GJ0;:efP02
0;:efP<000001@2d]]/010000000];OK0000000000030;Fgf`030000002e^=/0]KSK0080]KSK0P00
00050;Jjg0040000002f^]`00000000000<0]k_L00<000000;Nlg@2g_=d00P2g_=d00`000000^;cL
000000060;Rmg@030000002i_]d0^KkM00<0^KkM00@000000;VogP00000000000`2j_mh010000000
^/3N0;[0gP2j`=h2000000030;_0gP000000^l7O00<0^l7O0P0000060;_2g`800000102l`ml01000
0000_<CP0000000000030;g4h0040000002maN000000000000<0_LGP00<000000;g5h@2maN400P00
00020;k6h0040000002na^40_/KQ0;k6h@80000000<0_/OQ0000002nan400P2nan42000000030;o7
h@000000_lSQ0080_lSQ0P0000000`2ob>4000000;o8h@020;o8h@80000000<0`<WQ0000003oool0
:@3oool00`000000oooo0?ooo`1R0?ooo`00<P3oool00`000000oooo0?ooo`0R0?ooo`030000003o
ool0oooo04l0oooo100000050:K@m@8000001@2VcoH00`000000Y/oe0:K?m@02000000030:K?mP00
0000000000<0Y/of100000050:G?mP<000000`2TcoH2000000<0Xlkf0`0000050:;=m`030000002Q
c?H0XLcf0080000000<0Wlgg0000000000000`2Oc?L4000000<0W<[g0P00000309c9m`@000000`2I
aoL2000000<0V<Kg1000000309G4m`030000002D`_H0U<;f0080U<;f00@0000009C0m@2D`?D0U<3e
10000003096mm0030000002A^o<0TK_c0080TK_c00@0000009:ilP2B^O80T[Wb2@000003092dk`05
0000002A/^d0TK;]096bk@0000001@2C/N/010000000U:oY09B_j@2D[nT4000000<0TjgX00<00000
09F/iP2E[>H00P2E[>H00`000000Uj_T09N[i00209N[i0040000002HZ^<0V:[S09RZh`@000000`2I
Z>400`000000VZSO09ZXg`0209ZXg`030000002LZ=h0W:SN0080W:SN00@0000009jXgP2NZ=h0WZSN
1@00000209nWg0030000002QZ=/0XJSK00<0XJSK00<000000::Yf`2RZM/00P2RZM/3000000@0Y:SJ
00<000000:FYfP2UZMX00P2UZMX00`000000Yj_J0:N[fP02000000030:NZf@000000Z:_I00@0Z:_I
00<000000:V/f@0000001@0000040:^]f0030000002/[]T0[:kI0080[:kI0`0000040:j_f@030000
002_/=T0[k3I0080[k3I0P0000060;2af@070000002a/]X0/K;J0;6bfP000000/K;I000000050;:d
fP040000002b]MX00000000000<0/kGJ00<000000;Bff`2d]]/00P2d]]/2000000D0]KOK00<00000
0;Fhf`2e^=/00P0000020;Jif`030000002f^]`0][[L0080][[L0P0000060;Nkg0040000002g_=d0
000000000080^;cL00<000000;Rmg@2h_Md00P2h_Md3000000H0^KkM00<000000;ZogP2j_mh00`2j
_mh010000000^/3N0000000000030;_0gP030000002k`Ml0^l7O008000000P2k`Ml010000000^l;O
0;_2g`2k`]l2000000030;c2g`000000000000<0_<?O0P0000060;g4h0030000002maN00_LGP00@0
_LGP0P0000040;k6h0040000002na^400000000000<0_/OQ00@000000;k7h@00000000000`2oan40
10000000_lSQ0000000000030;o8h@040000002ob>400000000000<0`<WQ00<000000?ooo`3oool0
9P3oool2000000030?ooo`000000oooo0680oooo000b0?ooo`040000003oool0000000000280oooo
00<000000?ooo`3oool0B`3oool3000000<0Y]3e0P0000020:K@m@80000000<0Y]3f000000000000
0`2VcoH4000000D0Y/of0`0000030:G?mP8000000P2UcoH2000000030:C?mP000000Y<of00@0Y<of
100000020:;>m`8000000`2RcOL4000000D0Wlgg0`00000209g<n08000000`2Lb_L500000080VLWh
0P00000309W7m`D000000P2Ea_P00`000000ULCg09G4m`0209G4m`040000002D`_H0U<;f09C2mPD0
00000P2A_oH00`000000TKgd096mm003096mm0040000002A^o<0TK_c096kl`@000000P2>^?800`00
0000SkKa08nfl@0208nfl@030000002@]>l0T;C_00<0T;C_00D00000096bk@2A/^d0TK;]00000003
09>aj`D000000P2A[^T00`000000TjgX09>]j00209>]j0030000002E[>H0UJcV0080UJcV00@00000
09N[i02GZn@0Uj_T1@00000209NXhP030000002IZ>40VJSQ0080VJSQ00<0000009ZXg`2JZ=l00P2J
Z=l010000000W:SN09bXgP2LZ=h5000000<0WJOM00<0000009nWg02OYm`00P2OYm`00`000000XJSK
0:6Xf`020:6Xf`030000002RZM/0XZWK0080000000<0XjOJ0000002TZ=X0102TZ=X00`000000YJWJ
0:FYfP020:FYfP<00000102WZ]T010000000Z:_I0:R[f@2XZmT3000000050:V/f@00000000000:Z/
f00000001@2[[MP00`000000[:kI0:b^f@0200000080[JkH00<000000:j_f@2^[mT00P2^[mT00`00
0000[k3I0:n`f@0200000080[k3H00<000000;2af@2`/MT00`2`/MT010000000/K;J000000000002
0;6bf@030000002b]=X0/[CJ00<0/[CJ0P0000050;>efP030000002d]]/0];KK0080000000<0];KJ
0000002e]m/01@2e]m/2000000@0][WK00<000000;Jjg02f^]`00P0000020;Njf`030000002g^m`0
]k_L00@0]k_L00<000000;Rlg02h_=`00`2h_=`010000000^;gM0000000000030;Vmg@040000002i
_]d0^KkM0;Vng@80000000<0^[kM0000002j_mh00`2j_mh2000000H0^l3N0P0000040;_1g`040000
002k`]l00000000000<0_<;O00<000000;c3g`0000000P0000020;c3g`040000002ma>00_LCP0;g4
h080000000<0_LCP0000002maN000P2maN03000000H0_/KP00<000000;k7h@2nan40102nan400`00
0000_lOQ0;o7h@040;o7h@030000002ob>40_lSQ00@0_lSQ00<000000<39h@30bN401030bN400`00
0000oooo0?ooo`0X0?ooo`030000003oool0oooo0640oooo000b0?ooo`8000009@3oool00`000000
oooo0?ooo`180?ooo`80000000<0Y/oe000000000000102Vd?D3000000D0Y]3f0`0000030:K@mP80
00000P2VcoH2000000030:G?mP000000YLof00@0YLof0`0000040:C?mP030000002TcoH0Y<of0080
00000P2ScoL2000000<0X/kg100000030:3>m`8000000P2OcOL20000000309k=n0000000WLch00@0
WLch0`00000309[;n0030000002IbOP0VLWh00<0VLWh0`00000309K7n0030000002Ea_P0ULKh0080
ULKh00@0000009G4m`2Ea?L0ULCg100000030971m`030000002A_oH0TKof0080TKof00@00000096m
m02A_O@0TKgd1@00000208jjm0030000002>^?80S[Sb0080S[Sb00<0000008nfl@2?]_400P2?]_40
10000000T;C_092dk`2@]>l9000000<0T:o[00D00000096^j@2A[^T0TJkY0000000509>]j0040000
002E[>H0UJcV09F/iP@000000P2EZN@00`000000UjSR09NXhP0209NXhP030000002IZ>40VJSQ00<0
VJSQ00<0000009ZXg`2JZ=l00P2JZ=l3000000<0VjKM00<0000009fWg@2MYmd00P2MYmd00`000000
WjOL09nWg00209nWg0040000002QZ=/0XJSK0:6Xf`@00000102SYmX00`000000Y:SJ0:BXfP030:BX
fP060000002UZMX000000000002VZ=P000001@2WZ]T4000000040:R[f@0000000000000000@0ZZcH
00<000000:^]f02[[MP00P2[[MP3000000@0[JkH00<000000:j_f@2^[mT00`2^[mT2000000@0[k3H
00<000000;2af@2`/MT00`2`/MT2000000D0/K;I00<000000;:dfP2b]=X00P0000000`2c]=T00000
0;>efP050;>efP800000102d]]X010000000]KOK0;Fgf`2e]m/2000000030;JhfP000000][WK00@0
][WK0P0000040;Njf`030000002g^m`0]k_L0080]k_L0`0000060;Rlg0030000002i_Md0^KgM00<0
^KgM00@000000;Vng@00000000000`2j_]d00`000000^[oN0;ZogP0200000080^[oN00@000000;_0
gP2k`=h0^l3N0`0000060;_1g`030000002l`]l0_<;O00@0_<;O0P0000040;c3g`040000002ma>00
0000000000<0_LCP00@000000;g5h000000000000`2naN0010000000_/KP0;k6h02na^0200000003
0;k6h0000000_/OQ0080_/OQ0P0000000`2oan4000000;o7h@020;o7h@80000000<0_lOQ0000002o
b>400P2ob>42000000030<38h@000000`<WQ0080`<WQ0P0000000`30bN4000000?ooo`0Y0?ooo`03
0000003oool0oooo0600oooo000b0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0
A03oool3000000D0Y/oe100000020:K@m@8000000P2Vd?H2000000030:K@mP000000Y]3f00D0Y]3f
0P0000040:G?mP030000002UcoH0YLof008000000P2UcoH2000000<0Y<of0`0000050:??m`@00000
0P2Qc_L2000000<0X<kg1000000409k=n0030000002Mc?P0WLch0080000000<0Vlch0000002JboP0
102JboP00`000000VLWh09W9n0020000000309O9n@000000U/Oh00<0U/Oh00@0000009G6n02Ea_P0
ULKh1@00000209;3n0800000102A`OL010000000TKof096omP2A_oH4000000<0SKce00<0000008jj
m02>^_@00P2>^_@01@000000S[Sb08jhlP2>^?80000000<0SkKa1@00000208fbk`030000002>/>d0
S[3]0080S[3]00<00000092_j`2@[n/00P2@[n/00`000000TJkY096^j@02096^j@040000002C[NP0
TjgX09>]j0D000000P2CZND00`000000UJWT09FYi00209FYi0030000002GZ>80UjSR0080UjSR00<0
000009VXh@2IZ>400P2IZ>400`000000VZSO09ZXg`020000000309VVgP000000VjKM00<0VjKM00<0
000009fWg@2MYmd00`2MYmd010000000WjOL09nWg02OYm`4000000<0XJOJ00<000000:>WfP2SYmX0
0P2SYmX010000000Y:SJ0:BXfP2TZ=X4000000@0YZSH00<000000:NZf@2WZ]T0100000000`2XZmT0
0000000000020:VZf0030000002Z[=P0ZZcH0080ZZcH00<000000:^]f02[[MP00P0000020:b/e`03
0000002][]P0[JkH00<0[JkH00<000000:j_f@2^[mT00P0000000`2_[mP000000:n`f0040:n`f003
0000002`/MT0/;7I0080000000<0/K7H0000002a/]T01@2a/]T2000000@0/kCI00@000000;>efP2c
]MX0/kGJ0`0000050;BffP040000002e]m/00000000000<0][SJ00<000000;Jif`2f^M/00P2f^M/0
0`000000]kWK000000050;Njf`040000002g^m`00000000000<0^;_L00@000000;Rlg02h_=`0^;cL
0P0000000`2i_=`000000;Vmg@030;Vmg@8000001P2j_]d2000000@0^[oN00@000000;_0gP000000
00000`2k`Mh010000000^l7O0;_1g`2k`Ml2000000030;c2gP000000_<;O0080_<;O0P0000000`2l
`ml00000000000040;c3g`030000002ma>00_LCP00@0_LCP00<000000;k5h02naN000`2naN001000
0000_/KP0000000000030;k6h0040000002nan400000000000<0_lOQ00@000000;o7h@0000000000
0`2oan4010000000_lSQ0000000000030<38h@0400000030bN400000000000<0`<WQ00<000000?oo
o`3oool0:@3oool00`000000oooo0?ooo`1O0?ooo`00<P3oool00`000000oooo0?ooo`0V0?ooo`03
0000003oool0oooo03l0oooo100000020:K?m@8000000P2VcoD200000080Ym3e00<000000:K@m@2V
d?D00`2Vd?D2000000@0Y]3f0P0000020:K@mP80000000<0Y]3f0000000000000`2UcoH3000000D0
YLof100000020:??m`8000000P2ScoL200000080X/og00<000000:7>m`2Qc_L00`2Qc_L3000000<0
Wlkh00@0000009k=n02NcOP0W/gh1000000409_<n0040000002JboP0V/_h09[;n0@00000102GbOT0
10000000U/Oh09K7n02FaoP4000000<0T/Gi0P00000309;3n0T000000P2=__L2000000<0SKce00@0
000008jjm02>^_@0S[[d2@00000308bdl@030000002=/^l0SK;_0080SK;_00<0000008j`k@2>/>d0
0P2>/>d010000000T:o[092_j`2@[n/900000080TJ[W00<0000009>Yi@2CZND00P2CZND00`000000
UJWT09FYi00209FYi0040000002GZ>80UjSR09NXhPT000000`2IY]h00`000000VjKM09^Vg@0309^V
g@030000002MYmd0WJOM0080WJOM1000000209nVf`030000002QYmX0XJOJ0080XJOJ00<000000:>W
fP2SYmX00`2SYmX3000000<0Y:OH00<000000:JXf02VZ=P00P2VZ=P300000080Yj[I0`0000040:VZ
f0030000002Z[=P0ZZcH00<0ZZcH0P0000040:b/e`030000002][]P0[JkH00<0[JkH0`0000030:n_
f0030000002_/=P0[k3H00<0[k3H0`0000040;6af0040000002a/]T0/K;I0;6bf@<000001@2c]=T0
10000000/kGJ0000000000030;Bef@030000002d]]X0];KJ0080];KJ0P0000060;JhfP040000002f
^M/0000000000080]kWK00<000000;Njf`2g^]/00`2g^]/2000000D0^;_L00@000000;Rlg0000000
00000`2i_=`00`000000^KgM0;Vmg@0200000080^[kM00@000000;Zng@2j_]d0^[kM0`0000060;Zo
gP030000002k`Mh0^l7N00<0^l7N00@000000;_1g`00000000000`2l`]h010000000_<;O00000000
00030;c3g`030000002l`ml000000080_<?O00<000000;g4g`000000102ma>000`000000_/GO0000
00050;k5h0030000002na^00_/KP00@0_/KP00<000000;o7h@2oan40102oan400`000000_lOQ0;o7
h@040;o7h@0300000030b>40`<SQ00@0`<SQ00<000000<39h@30bN401030bN400`000000oooo0?oo
o`0X0?ooo`030000003oool0oooo05l0oooo000b0?ooo`030000003oool0oooo02L0oooo0P00000m
0?ooo`8000000P2VcoD00`000000Y/oe0:K?m@030:K?m@8000001@2Wd?D00`000000Y]3e0:K@m@02
000000030:K@mP000000000000<0Y]3f100000050:K@mP<000000P2Ud?H200000080YLof0P000002
0:C@m`8000000`2ScoL3000000D0X/og00<000000:7>m`2Qc_L00P0000000`2PcoP0000000000003
09o>n0@000000`2LcOT2000000<0Vlch1000000309S;n@8000000`2GbOT400000080TlOi0P000003
09;5n@D000000P2>`oT00`000000S/7h08k1n00208k1n0040000002=__L0SKkg08fnm`T000000`2:
^?D01@000000RkKc08^fl`2;]_<0000000D0S;Ca00D0000008fbk`2=/^l0SK;_0000000308j`k@D0
00000P2=[N/00`000000SjcY08n/j@0208n/j@030000002AZ^L0TJ[W0080TJ[W00<0000009>Yi@2C
ZND00P2CZND010000000UJWT09FYi02EZN@4000000<0UJKQ00D0000009NVh02GY^00UjKP00000005
09VVgP030000002KY]d0VjKM0080VjKM00<0000009fWg@2MYmd00P0000000`2MYM/0000009nVf`03
09nVf`030000002QYmX0XJOJ00<0XJOJ00<000000:>WfP2SYmX00P0000000`2SY]T000000:BWf003
0:BWf0030000002VZ=P000000080000000<0YZSH0000002WZ]T00P0000020:RYe`030000002YZ]P0
ZJ[H00<0ZJ[H00<000000:Z/f02Z[=P00P0000000`2[ZmL000000:b/e`040:b/e`030000002][]P0
[JkH0080000000<0[ZkG0000002_[mP0102_[mP00`000000[k3H0:n`f002000000030;2`f0000000
/K7H00@0/K7H00@000000;6bf@00000000000`2b/mP00`000000/kCI0;>df@020;>df@8000001@2d
]MT00`000000];KJ0;BffP0200000080][OJ00<000000;JhfP2f^=X0102f^=X00`000000]kWK0;Ni
f`030;Nif`030000002g^]/0]k[K008000000P2h^]/00`000000^;_L0;Rkg0020;Rkg08000001P2i
_=`2000000@0^[kM00@000000;Zng@00000000000`2j_md010000000^[oN0;ZogP2j_mh200000003
0;_0gP000000^l7N00@0^l7N00<000000;c2gP2l`]h0102l`]h00`000000_<?O0;c3g`040;c3g`04
0000002l`ml00000000000<0_LCO00@000000;g4h000000000000P2naMl00`000000_/GP0;k5h002
0;k5h0030000002na^00000000@0_/KP00<000000;o6h0000000102oan400`000000_lOP00000004
0;o7h@030000002ob>40000000<0`<SQ0P0000000`30b>4000000<39h@020<39h@80000000<0`LWQ
0000003oool0:@3oool00`000000oooo0?ooo`1N0?ooo`00<@3oool00`000000oooo0?ooo`0Z0?oo
o`030000003oool0oooo03P0oooo0P0000050:K?m@030000002VcoD0Y/oe0080000000<0Yloe0000
000000000`2Wd?D4000000D0Y]3f0`0000030:K@mP8000000P2Vd?H2000000030:G@mP0000000000
00<0YM3f0`0000050:C@m`@000000P2Sd?L2000000<0X/og100000050:3?n0<000000P2Mc_T20000
00<0W<gi1@00000209W<n@8000000`2HboT500000080U<Wj0P00000309?7n@@000000`2?aOX00`00
0000S/?i08k3n@0208k3n@040000002>`OP0S/7h08k1n0@000000`29_OP01@000000RK_f08VkmP29
^oH0000000D0R[Se00D0000008^fl`2;]_<0RkKc0000000308bdl@T000000`2;[^d01@000000SJg[
08f]j`2=[N/0000000D0SjcY00D00000096Zi`2AZ^L0TJ[W0000000309>Yi@D000000P2CYn<00`00
0000UJKQ09FVh@0209FVh@030000002GY^00UjKP0080UjKP00<0000009VVgP2IY]h00P2IY]h01000
0000VjKM09^Vg@2KY]d4000000@0WJGK00<0000009nVf`2OY]/00P2OY]/010000000XJOJ0:6WfP2Q
YmX4000000<0XjKI00@000000:BWf02TYmP0Y:OH0`0000030:JXf0<00000102XZML00`000000ZJ[H
0:VZf0030:VZf0<000000`2[ZmL00`000000[:cG0:b/e`040:b/e`800000102^[]L00`000000[joH
0:n_f0030:n_f0800000102`/=P00`000000/K7H0;6af0020;6af08000001@2b/mP00`000000/kCI
0;>df@0200000080];CI00<000000;Bef@2d]MT00`2d]MT2000000@0][OJ00<000000;JhfP2f^=X0
0P2f^=X3000000H0]kWK0P0000040;Rjf`030000002h^m`0^;_L008000000P2i_=`010000000^KcL
0;Vlg02i_=`3000000H0^[kM00<000000;Zog@2j_md00`2j_md010000000^[oN0000000000030;_0
gP030000002k`Mh0^l7N0080^l7N00<000000;c1gP000000102l`]h00`000000_<;N000000060;c3
g`8000001@2ma=l00`000000_/GO0;k5g`030;k5g`040000002naN000000000000<0_/KP00@00000
0;k6h000000000000`2oa^0010000000_lOQ0000000000030;o7h0040000002oan40000000000080
_lSQ00@000000<38h@00000000000`30b>4010000000`<WQ0000000000030<79h@030000003oool0
oooo02T0oooo00<000000?ooo`3oool0G@3oool00340oooo00<000000?ooo`3oool0:`3oool00`00
0000oooo0?ooo`0e0?ooo`80000000<0Yloe0000000000000`2VcoD4000000D0Yloe0`0000030:O@
m@8000000P2Vd?H2000000030:K@mP000000000000<0Y]3f100000050:G@mP<000000P2Ud?L20000
0080Y=3g0P0000020:?@m`8000000`2Sd?L4000000<0XM3h0P0000020:3?n080000000<0Wloi0000
000000000`2Mc_T4000000<0V/kj0P00000309W<n@@000000`2FboX010000000U<Wj09C9nP2DbOX5
00000080T<Oj0P00000408o5nP040000002>`oT0S/?i08k3n@@000000P29`?T00`000000RKgh08Vm
n00308Vmn00500000029^oH0RK_f08VkmP0000000`2:^?D9000000<0Qk;b00D0000008V`l029/?00
RK3`0000000408^^k@030000002=[N/0SJg[0080SJg[00@0000008n/j@2?[>T0SjcY2@000003092W
i@050000002CYn<0TjOS09>Wh`0000001@2EY^401@000000UjKP09NVh02GY^00000000<0VJKN1@00
000209^Ug0030000002MYM/0WJGK00<0WJGK00<0000009nVf`2OY]/00P2OY]/400000080XJGI00<0
00000:>Vf@2SY]T00`2SY]T300000080Y:OH100000020:JXe`030000002XZML0Z:WG00<0Z:WG00<0
00000:VZf02YZ]P00P0000000`2ZZ]L000000:^[e`040:^[e`040000002/[=L0[:cG0:b/e`<00000
1@2^[]L010000000[joH0:n_f02_[mP3000000D0/;3H00<000000;6af02a/MP00P0000020;:af003
0000002b/mP0/[?H00<0/[?H0P0000040;Bdf@030000002d]MT0];GI0080];GI00<000000;Fff@00
00001@2f]mX010000000][SJ0000000000030;NhfP040000002g^M/0]kWK0;Nif`<000001P2h^]/2
000000@0^KcL00@000000;Vlg000000000000`2j_M`010000000^[kM0;Zng@2j_]d2000000030;Zn
g@000000^[oM00@0^[oM00<000000;_0gP2k`=h0102k`=h010000000^l7N0000000000030;c1gP04
0000002l`]h0000000000080_<;N00<000000;c3g`2l`ml00P2l`ml2000000030;g3g`0000000000
0080_LCO0P0000000`2na=l000000;k5g`030;k5g`8000001P2na^000`000000_lKP0;o6h0040;o6
h0030000002oan00_lOP00@0_lOP00<000000;o8h@2ob>400`2ob>400`000000`<SQ0<38h@040<38
h@0300000031bN40`LWQ00@0`LWQ00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`1M0?oo
o`00<@3oool00`000000oooo0?ooo`02000002X0oooo00<000000?ooo`3oool0<03oool4000000D0
Yloe0`0000030:O?m@8000000P2WcoD2000000030:O@m@000000000000<0Ym3e100000050:K@mP<0
00000`2Vd?H200000080YM3f0P0000000`2Ud?H00000000000030:G@m`<000001@2Sd?L400000080
X]3h0P0000030:7@n0@000001@2OcoT300000080W<oj0P00000309[>nPD000000P2GcOX2000000<0
U/_j100000030979n`8000000`2@aoX900000080R/;j0P00000308W0n@0400000029_OP0RKgh08Vm
n0T000000`25]oH01@000000Q[Cc08Jdl`26]?<0000000@0Qk;b00<0000008V`l029/?000P29/?00
10000000Rjk]08^^k@2;[^d900000080S:WY00<0000008jXi`2>Z>L00P2>Z>L00`000000T:OU092W
i@02092Wi@030000002CYn<0TjOS0080TjOS00<0000009FVh@2EY^400P2EY^4010000000UjKP09NV
h02GY^04000000<0VJCM00<0000009^Ug02KYM`00P2KYM`00`000000WJGK09fUf`0309fUf`060000
002OY]/000000000002OY=T000001@2QYMT00`000000XjKI0:>Vf@0300000080Y:OH0P0000000`2U
Y]L000000:JXe`030:JXe`030000002XZML0Z:WG00<0Z:WG0`0000040:ZZe`030000002[ZmL0Zj_G
0080Zj_G00@000000:b/e`00000000000`2][=L00`000000[ZkG0:j^e`030:j^e`<000000`2`[mL0
0`000000/;3H0;2`f0030;2`f0800000102b/MP00`000000/[?H0;:cf0020;:cf0030000002c/mP0
000000D0];CI00@000000;Bef@00000000000P2e]]T00`000000][OJ0;JgfP030;JgfP8000001@2g
^=X010000000]kWK0000000000030;Rif`040000002h^]/0^;[K0;Rjf`<000001P2i_=`00`000000
^[gL0;Zmg0030;Zmg0040000002j_]d00000000000<0^[kM00<000000;Zog@2j_md00P2j_md00`00
0000^koM000000050;_0gP8000001P2l`Mh00`000000_<;N0;c2gP030;c2gP040000002l`ml00000
000000<0_L?O00@000000;g4g`00000000000`2na=l00`000000_/GO0;k5g`0200000080_/GO00@0
00000;k6h02na^00_/KP0P0000000`2oa^0000000;o6h0020;o6h080000000<0_lOP0000002oan00
0P2oan02000000030;o7h0000000_lSQ0080_lSQ00<000000<38h@0000001030b>400`000000`<WQ
000000030<79h@80000000<0`LWQ0000003oool0:@3oool00`000000oooo0?ooo`1L0?ooo`00<@3o
ool3000002d0oooo00<000000?ooo`3oool0;03oool3000000<0Ylke0P0000020:O?m@80000000<0
Yloe0000000000000`2WcoD4000000D0Ym3e0`0000030:O@mP8000000P2Vd?H2000000030:K@mP00
0000000000<0Y]3f100000050:G@mP<000000P2Td?L200000080Xm3g0P0000020:?An0030000002R
d?P0X]3h00<0X]3h0`0000030:3@n@8000000P2OcoT20000000309g@n@000000W<oj00@0W<oj0`00
000309W>nP8000000`2GcOX400000080Tlck0P0000030979n`D000000P2<ao`00`000000RlGk08_5
n`0208_5n`040000002:`_X0R/;j08[2nPT000000`24_?T010000000Q;Wg08Bim`24^OL2000000<0
QKOf00<0000008Jdl`26]?<00P26]?<010000000Qk;b08NblP27/_8900000080Qjc^00<0000008VZ
j`29Z^/00P29Z^/00`000000S:WY08bYj@0208bYj@050000002>Z>L0SZSW08jXi`0000001@2@YnD0
10000000TjOS09>Wh`2CYn<900000080UZCO00<0000009VTg@2IY=d00P2IY=d00`000000VjGL09^U
g00209^Ug0040000002MYM/0WJGK09fUf`@00000102OY=T00`000000XJGI0:6Uf@020:6Uf@<00000
0P2SY]T3000000@0YJKG00<000000:JXe`2VZ=L00P2VZ=L00`000000Z:WG0:RYe`02000000030:RX
eP000000ZZ[G00@0ZZ[G00<000000:^[e`2[ZmL00P2[ZmL2000000D0[JcG00<000000:j^e`2^[]L0
0P2^[]L2000000D0/:oG00@000000;2`f02`/=P0/;3H0P0000000`2a/=L000000;:af0040;:af004
0000002b/mP0000000000080/k?H00<000000;Bdf@2d]=T00`2d]=T2000000D0]KKI00<000000;Jg
fP2f]mX00P0000000`2f]mT000000;NhfP050;NhfP030000002h^M/0^;WK00<0^;WK00@000000;Rj
f`00000000000`2i^m/010000000^KcL0;Vlg02i_=`2000000030;Zlf`000000^[gL00@0^[gL00<0
00000;Zng@2j_]d0102j_]d010000000^[oM0000000000030;^og@030000002k`=h0^l3N00800000
0P2l`Md010000000_<7N0;c1gP2l`Mh2000000030;c2gP000000_<;N00<0_<;N0P0000060;g3g`80
00001@2na=l2000000@0_/GO00@000000;k6h000000000000`2oa^0010000000_lKP000000000003
0;o7h0040000002oan000000000000<0_lOP00@000000;o8h@00000000000`30b>4010000000`<SQ
0000000000020<39h@0400000031bN400000000000<0`LWQ00<000000?ooo`3oool0:@3oool00`00
0000oooo0?ooo`1K0?ooo`00<@3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo02T0
oooo0P0000000`2Wc_@000000:O>m@050:O>m@8000001@2WcoD3000000<0Yloe0P0000020:O@m@80
000000<0Ym3e0000002Wd?H0102Wd?H3000000D0Y]3f0`0000030:KAmP8000000P2Ud?H200000003
0:GAm`000000Y=3g00@0Y=3g0P0000050:?An0030000002Rd?P0X]3h0080000000<0XM7h00000000
00000`2Pd?T4000000@0WM3i00<0000009c?nP2LcoX00P0000000`2Jd?X000000000000309W>nPD0
00000P2DcO/2000000<0Tlck1000000308g9o08000000`2<ao`010000000RlGk08_5n`2;aO/40000
00<0QL;k00@0000008BonP24_oX0Q;oj0P00000308Bln@0400000024^OL0Q;Wg08Bim`h000000P23
/?800`000000QJk`08F^l00208F^l00500000027[>h0Qjc^08N/kP0000001029Z^/00`000000S:WY
08bYj@0208bYj@030000002>Z>L0SZSW0080SZSW00@00000092Wi@2@YnD0T:OU1@000002096ThP05
0000002CY>00TjCP09>Th0000000102FY=l00`000000VJCM09VTg@0309VTg@040000002KYM`0VjGL
09^Ug0@000000`2LXmX00`000000WjCI09nTf@0209nTf@030000002QYMT000000080000000<0XJGI
0000002SY]T00P0000020:>Ue`030000002UY]L0YJKG0080YJKG00<000000:JXe`2VZ=L00P2VZ=L3
000000@0Z:SF00<000000:ZZe`2ZZ]L00`2ZZ]L010000000Zj_G0000000000020:b[eP030000002]
[=L0[JcG00<0[JcG00@000000:j^e`00000000000P2_[MH00`000000/:oG0;2_e`030;2_e`<00000
0`2a/=L00`000000/[7H0;:af0030;:af08000001@2c/mP00`000000];CI0;Bdf@02000000030;Fd
f0000000]KKI00D0]KKI0P0000040;Jgf@040000002g^=X0]kSJ0;NhfP<000001P2h^M/00`000000
^K_K0;Vkf`030;Vkf`040000002i_=`00000000000<0^[cK00<000000;Zmg02j_M`00P2j_M`00`00
0000^[gL000000060;Zng@030000002k_md0^koM00@0^koM0P0000040;c1g@040000002l`Mh00000
000000<0_<;N00<000000;c2gP2l`]h00P0000020;g3gP040000002m`ml0_L?O0;g3g`80000000<0
_/CO0000000000000P2na=l3000000H0_/GO00<000000;o6h02oa^00102oa^000`000000_lOP0;o7
h0040;o7h0030000002oan00_lOP00@0_lOP00<000000<38h@30b>401030b>400`000000`<WQ0<39
h@030<39h@0300000031bN40`LWQ00@0`LWQ00<000000?ooo`3oool0:03oool00`000000oooo0?oo
o`1K0?ooo`00<@3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo02D0oooo0`000004
0:O>m08000000P2Wc_D2000000030:O?m000000000000080Yloe0P0000000`2WcoD000000:O?m@05
0:O?m@800000102Wd?D00`000000Ym3f0:O@mP0200000080Ym3f0P0000020:K@mP80000000<0Y]7f
0000002VdOH0102VdOH3000000@0YM7g00<000000:C@m`2Td?L00P0000000`2TdOL0000000000003
0:?An0@000001@2QdOP3000000<0Wm7i00@0000009g@n@2Md?T0WM3i1000000509[@nP<000000`2F
co/010000000U<gk09C=n`2DcO/500000080Slcl0P00000308g9o0T000000P26a?`2000000@0QL;k
00@0000008BonP24_oX0Q;oj2000000307nhn0050000001o]OH0OkGf07nemP0000001021/_@00`00
0000Pk3b08>`lP0208>`lP0300000025[_00QJk`0080QJk`00D0000008N/kP27[>h0Qjc^00000004
08VZj`040000002<ZNT0S:WY08bYj@T000000P2>YN@00`000000TJCR096ThP02096ThP030000002C
Y>00TjCP0080TjCP00<0000009JTg`2FY=l00P2FY=l010000000VJCM09VTg@2IY=d500000080VZ?K
00<0000009bSfP2LXmX00P2LXmX010000000WjCI09nTf@2OY=T300000080XJGI100000040:>Ue`03
0000002UY]L0YJKG00<0YJKG00@000000:JXe`00000000000P2WYmH00`000000Z:SF0:RXeP020:RX
eP030000002ZZ]L0ZZ[G0080ZZ[G0`0000040:b[eP030000002][=L0[JcG00<0[JcG0P0000040:n]
eP030000002`[mL0/:oG00<0/:oG00<000000;6`e`2a/=L0102a/=L00`000000/[7H0;:af0020000
00030;>be`000000/k?H00D0/k?H0P0000040;Fdf0040000002e]]T0]KKI0;Fff@<000001@2f]mT0
10000000]kSJ0000000000030;RifP040000002h^M/0^;WK0;Rif`80000000<0^K[J0000002i^m/0
102i^m/00`000000^[cK0;Zlf`040;Zlf`040000002j_M`0000000000080^[gL00<000000;Zng@2j
_]d00P2j_]d2000000030;^ng0000000^koM0080^koM0`0000060;c1g@030000002l`]h0_<;N00@0
_<;N0P0000040;g3gP040000002m`ml00000000000<0_/CO00@000000;k4g`00000000000`2naMl0
10000000_/GO0;k5g`2naMl2000000030;o5g`000000_lKP0080_lKP0P0000000`2oa^0000000;o7
h0020;o7h080000000<0_lOP0000002oan000P2oan02000000030<38h0000000`<SQ0080`<SQ0P00
00000`30b>0000000<39h@020<39h@0300000031bN40000000<0`LWQ0P0000000`31bN4000000?oo
o`0Y0?ooo`030000003oool0oooo05X0oooo000A0?ooo`050?oOW018000000000000M02oool00`3o
ool01P3o_g@0000007Boo`3oool0oooO09a800<0000000<0M;oo0?ooo`3oool02`3oool00`000000
oooo0?ooo`0a0?ooo`030000003oool0oooo0200oooo100000020:O>m08000000`2Wc_@4000000D0
Ylod0P0000040:O?m@8000000P2WcoD2000000030:O@m@000000000000<0Ym3e0`0000050:O@mP<0
0000102VdOH00`000000Y]7f0:KAmP0200000080Y]7f0P0000030:GAm`<000001@2TdOL3000000<0
X]7h0P0000020:7An080000000<0X=;i0000000000000`2OdOT4000000<0W=7j0P00000209[@nP80
000000<0V=3k0000000000000`2Fco/4000000<0TLkl0P00000308o<o0@000000`29b_d010000000
QlOm08O7o@27aod2000000<0Q/Cl2@00000207jnn`040000001n^_X0O[[j07jjnP800000101o^?P0
1@000000OkGf07nemP1o]OH0000000@0PK;d00@0000008>`lP23/?80Pk3b4P00000208RVj0030000
002;YNH0RjGV0080RjGV00<0000008jUi02>YN@00P2>YN@00`000000TJCR096ThP02096ThP050000
002CY>00TjCP09>Th00000000`2FY=l500000080V:;L00<0000009ZSf`2JXm/00P2JXm/00`000000
W:?J09bSfP0309bSfP<0000000<0WjCI0000000000000P0000030:6Te`030000002SYML0XjGG00<0
XjGG00<000000:FVe`2UY]L00P2UY]L2000000@0YjOF00<000000:RXeP2XZ=H00`2XZ=H010000000
ZZ[G0000000000020:^Ye@030000002/ZmH0[:_F00<0[:_F00<000000:f/e`2][=L00P0000000`2^
[=H000000:n]eP040:n]eP030000002`[mL0/:oG0080000000<0/JoF0000002a/=L01@2a/=L20000
00@0/k;G00@000000;>cf02c/mP0/k?H0`0000050;Fdf0040000002e]]T00000000000<0][KI00<0
00000;Jgf@2f]mT00P2f]mT2000000D0^;WJ00@000000;Rif`00000000000`2i^]X00`000000^K_K
0;Vkf`020;Vkf`030000002i^m/0000000H0^[cK00<000000;Zmg02j_M`00`2j_M`010000000^[kM
0000000000030;^ng0040000002k_md00000000000<0_<3M00@000000;c1g@2l`Md0_<7M0P000000
0`2l`Md000000;c2gP020;c2gP<000001P2m`mh00`000000_/CO0;k4g`040;k4g`030000002naMl0
000000<0_/GO00@000000;k5g`00000000000`2oaMl010000000_lKP0000000000030;o6h0040000
002oan000000000000<0_lOP00@000000;o7h000000000000`30b>0010000000`<SQ000000000003
0<38h00400000030bN40000000000080`LWQ00@000000<79h@00000000000`31bN400`000000oooo
0?ooo`0Y0?ooo`030000003oool0oooo05T0oooo000@0?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0
001d0;ooo`020?ooo`070?noM0000000M;oo0?ooo`3ooml0W4Qd0;ooo`020?ooo`030;md001d_ol0
oooo00/0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`0M0?ooo`8000000P2WcO@0
0`000000Ylkd0:O>m0030:O>m0<000000`2Wc_@200000080Ylod0P0000000`2Xco@0000000000003
0:O?m@@000001@2Wd?D300000080Ym3e0P0000020:O@mP8000000P2WdOH2000000<0Y]7f0`000005
0:KAmP@000000P2UdOL200000080Y=7g0P0000000`2Sd_P00000000000030:;An0@000001@2Pd_T3
00000080W];j0P00000309cAnP@00000102Hd?/400000080U=3l0P000003097>o0D000000P2;c?d2
000000<0RL[m2@0000020834o@8000000`1o`O`00`000000O[kk07jnn`0207jnn`040000001n^_X0
O[[j07jjnQ8000000P1m[O801@000000P:_`082[l020Zo00000000@0PjW]00<0000008JXj`26Z>/0
0P26Z>/01@000000R:KX08RVj028Y^P0000000@0RjGV00<0000008jUi02>YN@00P2>YN@00`000000
TJCR096ThP02096ThP040000002CY>00TjCP09>Th0@000000`2EX]d00`000000V:;L09RRg00209RR
g0030000002JXm/0VZ?K0080VZ?K00<0000009bSfP0000001P00000209nRf0030000002QY=L0XJCG
0080XJCG00<000000:>Ue`2SYML00`2SYML00`000000YJKG0:FVe`02000000030:FUeP000000YjOF
00@0YjOF00<000000:RXeP2XZ=H00P2XZ=H2000000@0ZjWE00<000000:b[eP2/ZmH00`2/ZmH30000
00<0[ZcF00<000000:n]eP2_[MH00`2_[MH3000000<0/JoF00<000000;6`e`2a/=L00P2a/=L30000
00D0/k;G00@000000;>cf000000000000`2d/mL00`000000]KCH0;Fdf0030;Fdf0030000002f]]T0
][KI00<0][KI00<000000;Jgf@2f]mT00P0000020;Nhf@030000002h^MX0^;WJ00<0^;WJ00<00000
0;VjfP2i^]X0102i^]X010000000^K_K0000000000020;Vkf`030000002j_=/0^[cK0080^[cK0P00
00000`2j_M/000000;Zmg0030;Zmg08000001P2k_]`00`000000_<3M0;c0g@030;c0g@040000002l
`Md00000000000<0_<7M00@000000;c2gP00000000000`2m`]h010000000_L?N0;g3gP2m`mh20000
00030;k3gP000000_/CO0080_/CO0`0000000`2naMl000000;k5g`030;k5g`030000002oaMl0_lGO
00@0_lGO00<000000;o6h02oa^00102oa^000`000000_lOP0;o7h0040;o7h00300000030b>00`<SP
00@0`<SP00<000000<38h030b>001030b>000`000000`LWQ0<79h@030<79h@0300000031bN40`LWQ
00@0`LWQ00<000000?ooo`3oool09@3oool4000005/0oooo000@0?ooo`030;md0000B9`0gooo0080
oooo00<0_g@0001d_`3oool02P3oool00`3ooml0W4Qd0;ooo`0;0?ooo`030000003oool0oooo03<0
oooo00<000000?ooo`3oool06P3oool2000000D0Ylgd00<000000:O>m02Wc_@00P0000000`2Wc_@0
0000000000030:O>m0@000001@2Xco@3000000<0Z<oe0P0000020:O@m@80000000<0Z=3e00000000
00000`2Wd?D3000000H0Ym7f0`0000020:KAmP8000000P2VdOH200000080Y];g0P0000030:GAm`<0
00001@2Sd_P3000000<0X];i0P0000020:3Bn@80000000<0Wm;j0000000000000`2Nd_X4000000@0
V];k1000000209KAo08000000`2Dd?`4000000<0S/km0P00000308_<o@@000000P24b_h2000000<0
P/On0P0000030834o@d000000P1g^OX00`000000N;Gi07Ren@0207Ren@050000001i/_L0NK;g07Vb
m`000000101k/?D01@000000OJgb07f]lP1m[O80000000@0P:_`00<0000008>Yk@23ZNd00P23ZNd0
1@000000QZS[08JXj`26Z>/0000000@0R:KX00<0000008^UiP2;YNH00P2;YNH010000000SZGT08jU
i02>YN@900000080TZ7O00<0000009FRg@2EX]d00P2EX]d00`000000V:;L09RRg00209RRg0040000
002JXm/0VZ?K09ZSf`<0000000D0W:?J000000000000W:7H0000000409nRf0030000002QY=L0XJCG
00<0XJCG00@000000:>Ue`2SYML0XjGG100000030:FUeP030000002WYmH0YjOF00<0YjOF00<00000
0:RXeP2XZ=H00P0000000`2YYmD000000:^Ye@040:^Ye@030000002/ZmH0[:_F0080000000<0[J[E
0000002^[=H0102^[=H00`000000[jgF0:n]eP02000000030;2]e@000000/JoF00@0/JoF00@00000
0;6`e`00000000000`2b/=H00`000000/k;G0;>be`030;>be`030000002d/mL0];?G00<0];?G00@0
00000;Fdf02e]=P0]KCH0P0000000`2f]MP000000;Jff@040;Jff@8000001@2g^=T00`000000^;WJ
0;RifP02000000030;RifP000000^K[J00@0^K[J0P0000050;Vkf`040000002j_=/00000000000<0
^[gK00<000000;Zmg02j_M`00P0000020;^ng0040000002k_]`0^kkL0;^ng0<000001P2l`=d00`00
0000_<7M0;c1g@040;c1g@030000002m`]h0_L;N00<0_L;N00@000000;g3gP00000000000`2n`mh0
10000000_/CO0000000000030;k4gP040000002naMl000000;k5g`80000000<0_lGO0000002oaMl0
0P2oaMl2000000030;o6g`000000_lKP0080_lKP0P0000000`2oa]l000000;o7h0020;o7h0800000
00<0`<OP00000030b>000P30b>02000000030<38h0000000`<SP0080`<SP0P0000000`31b>000000
0<79h@020<79h@0300000031bN00000000<0`LWQ0P0000000`31bN4000000?ooo`0Y0?ooo`030000
003oool0oooo05P0oooo000?0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool02`3o
ool00`2oM000M;oo0?ooo`0:0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool05`3o
ool2000000030:O=m0000000000000@0Ylgd0`0000050:O>m0<000000`2Xco@200000080Z<od0P00
00000`2Xco@00000000000030:S?m@@000001@2Xd?D300000080Z=3e0P0000030:OAmP80000000<0
Ym7f0000000000000`2VdOH3000000D0Y];g100000020:GBm`8000000P2Sd_P2000000030:?Bn000
0000000000<0X];i1000000509oBnP<000000P2Ldo/200000080V];k0P00000209WCo08000000`2F
dO`400000080TM3m0P00000308k>o@D000000P27cOh2000000<0Q<[n2@00000207W3oP8000000P1h
_od2000000<0Mkcl0P00000307NinP050000001h]OT0N;Gi07Ren@000000101i/_L01@000000Nk3e
07^`m@1k/?D0000000@0OJgb00D00000082[l020Zo00P:_`0000000308>YkA<000000P2<X^<01@00
0000Sj7Q08nQh@2?XN40000000@0TZ7O00<0000009FRg@2EX]d00`2EX]d00`000000V:;L09RRg002
09RRg0L00000102LXMP00`000000Wj;H09nRf00209nRf0030000002QY=L0XJCG0080XJCG10000002
0:>TeP030000002UYMH0YJGF00<0YJGF00<000000:NWeP2WYmH00P2WYmH3000000<0ZJOE00<00000
0:^Ye@2[ZMD00`2[ZMD3000000@0[J[E00<000000:j/eP2^[=H00`2^[=H2000000@0/:gE00<00000
0;6_eP2a[mH00P2a[mH2000000D0/[3F00@000000;>be`2c/]L0/k;G0`0000060;Bce`<000000`2f
]MP00`000000][KI0;Jff@020;Jff@030000002g]mP0000000H0]kSI0P0000040;RifP030000002i
^]X0^K[J008000000P2i^]X00`000000^K_K0;Vkf`020;Vkf`8000001P2j_M/2000000@0^kkL00@0
00000;^ng000000000000`2l_m`010000000_<3M0;c0g@2l`=d2000000030;g0g@000000_<7M0080
_<7M0`0000060;g2gP030000002n`mh0_/?N00@0_/?N00<000000;k4gP2na=h00`2na=h010000000
_/GO0000000000030;o5g`040000002oaMl00000000000<0_lKO00@000000;o6h000000000000`2o
a]l010000000_lOP0000000000030<37h00400000030b>000000000000<0`<SP00@000000<38h000
000000000`31b>0010000000`LWQ0000000000020<79h00400000031bN400000000000<0`LWQ00<0
00000?ooo`3oool0:@3oool00`000000oooo0?ooo`1G0?ooo`000`3oool00`3ooml0W4P000000007
0000000504RLg`3oool0omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0:0?ooo`030?oog`2L
B000B9cO00/0oooo00<000000?ooo`000000=@3oool00`000000oooo0?ooo`0C0?ooo`<000001@2W
cO@400000080Ylgd0P0000020:O>m080000000<0Z<kd0000002Xco@01@2Xco@2000000D0Z<od0`00
00030:S@m@8000000P2Xd?D2000000030:S@m@000000000000<0Z=3e100000050:OAmP<000000P2V
d_H200000080Y];g0P0000020:KBm`8000000`2Ud_L3000000D0Xm;h0`0000030:7Cn@8000000P2O
d_X20000000309kCnP000000000000<0W=?k0`00000509WCo0@000000P2Dd_d2000000<0TM3m1000
000308[?oP0400000027cOh0Qlgn08O=oPD000000P1mbOl2000000<0NlKn00@0000007W3oP1i`oh0
NL?n:000000307jVk@0500000021Y>/0PJC[086Tj`0000001025XnP01@000000R:;U08RRi@28X^D0
000000@0S:;S00<0000008nQh@2?XN400P2?XN400`000000TZ7O09:Qg`0209:Qg`030000002EX]d0
UJ;M0080UJ;M00<0000009RRg02HX]`01@00000309ZPf@030000002LXMP0W:7H0080W:7H00<00000
09nRf02OX]P00P2OX]P00`000000XJCG0:6Te`02000000030:6ReP000000XjCF00@0XjCF00<00000
0:FUeP2UYMH00P2UYMH00`000000YjOF0:NWeP02000000030:RVe0000000ZJOE00@0ZJOE00<00000
0:^Ye@2[ZMD00P0000000`2/ZM@000000:fZe@040:fZe@040000002^[=H0[ZcF0:j/eP<000001@2`
[MD00`000000/JoF0;6_eP02000000030;:_eP000000/[3F00D0/[3F0`0000030;Bbe`030000002d
/mL0];?G0080];?G0P0000060;Jef0040000002f]]T0000000000080]kOH00<000000;Nhf@2g^=T0
0P2g^=T3000000H0^;WJ0P0000040;VjfP030000002i^m/0^K_K008000000P2j_=/010000000^[gK
0;Zmf`2j_M/3000000H0^kkL00<000000;bog02l_m`00`2l_m`010000000_<3M0000000000030;g0
g@040000002l`Md00000000000<0_L;M00@000000;g2gP2m`]h0_L;N0P0000000`2n`mh000000;k3
gP020;k3gP80000000<0_/CN0000002na=h00`2na=h200000080_lGO00D000000;o5g`2oaMl0_lGO
000000060;o6g`030000002oa]l0_lKO00@0_lKO00<000000<37h030an001030an000`000000`<SP
0<38h0040<38h00300000031b>00`LSP00@0`LSP00<000000<79h031bN000`31bN000`000000`LWQ
0<79h@040<79h@030000003oool0oooo02P0oooo00<000000?ooo`3oool0E`3oool000l0oooo00<0
omnL04P0001d_ol00`3oool00`3o_g@0001809cOo`0:0?ooo`030=nLB0000000M;oo00/0oooo0P00
000g0?ooo`030000003oool0oooo00l0oooo0`0000020:O<l`8000000P2WcO@200000080Ylgc00<0
00000:O=m02WcO@00`2WcO@2000000@0Z<kd0P0000020:S?m080000000<0Z<od0000000000000P2X
co@2000000030:S?m0000000Z=3e00@0Z=3e0`0000050:S@m@<000000`2XdOH200000080Ym7f0P00
00000`2WdOH00000000000030:KBmP<000001@2Vd_L400000080Y=?h0P0000020:?Bn080000000<0
Xm?i0000000000000`2QdoT4000000@0W]?j1000000209_Do08000000P2Ido`200000080UmCm00@0
000009CBo@2Dd_d0U=;m1@00000208gAoP8000000`2:coh4000000<0PLgo0P00000307g9o`P00000
0`1``Oh010000000Kkgm06nmo@1__Od2000000<0K[[l00@0000006nfn`1_]_/0KkKk0P000002072c
n@8000000`1b[oL01@000000M:ge07B]m@1d[OD0000000@0Mj[b00D0000007ZXl01jZ?00NZS`0000
000407jVk@0500000021Y>/0PJC[086Tj`0000001025XnP00`000000R:;U08RRi@0208RRi@030000
002<X^<0S:;S0080S:;S00D0000008nQh@2?XN40Sj7Q0000000309:Qg`X000000P2GWmX00`000000
VZ3I09ZPf@0209ZPf@030000002LXMP0W:7H00<0W:7H00<0000009nRf02OX]P0100000040:6ReP03
0000002SY=H0XjCF0080XjCF00@000000:FUeP2UYMH0YJGF100000040:RVe0030000002YYmD0ZJOE
0080ZJOE0`0000040:bYe0030000002]Z]D0[J[E00<0[J[E0`0000030:n/e0030000002`[MD0/:gE
00<0/:gE0P0000040;:_eP030000002b/=H0/[3F0080/[3F0P0000050;Bbe`030000002d/mL0];?G
008000000P2f]=L00`000000][GH0;Jef0040;Jef0030000002g]mP0]kOH00<0]kOH00@000000;Nh
f@00000000000`2h^=T010000000^;WJ0;RifP2h^MX3000000H0^K[J0P0000040;Zlf`040000002j
_M/00000000000<0^kgK00@000000;^ng02k_]`0^kkL0`0000060;bog0030000002m`=d0_L3M00@0
_L3M00<000000;g2g@2m`]d00`2m`]d010000000_L;N0000000000030;k3gP040000002n`mh00000
000000<0_/CN00<000000;k4gP2na=h00P0000020;o4gP040000002oaMl000000;o5g`80000000<0
_lGO0000002oa]l00P2oa]l2000000030;o6g`000000_lKO0080_lKO0P0000000`30aml000000<37
h0020<37h080000000<0`<SP00000030b>000P30b>02000000030<78h0000000`LSP0080`LSP0P00
00000`31bN0000000<79h0020<79h00300000031bN00000000@0`LWQ00<000000<;9h@000000:P3o
ool00`000000oooo0?ooo`1F0?ooo`003`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0
W=oo00H0oooo00L0gmoO0?ooo`3ookl0M0000000000007@0_ooo00X0oooo00<000000?ooo`3oool0
>03oool2000000d0oooo0P0000000`2Wc?<000000:O<l`040:O<l`8000001@2WcO<00`000000Ylgd
0:O=m002000000030:S>l`000000000000<0Z<kd100000050:S?m0800000102Xco@00`000000Z=3e
0:S@m@0200000080Z=3e0P0000020:S@m@80000000<0Z=7e0000002XdOH0102XdOH3000000D0Ym7f
0`0000020:OBm`8000000P2Vd_L200000080Y]?g0P0000030:CCn0<00000102SdoT4000000@0X=Cj
1000000209gDn`8000000`2Ke?`3000000@0UmCm10000003097CoP8000000`2=dOh4000000<0QM3o
2@00000207K9o`8000000P1baOl2000000<0L<7n5P0000000`1b[oL0000007B]m@0207B]m@050000
001gZ_80Mj[b07NZlP000000101jZ?000`000000OZK]07jVk@0207jVk@0500000021Y>/0PJC[086T
j`0000001025XnP01@000000R:;U08RRi@28X^D0000000<0S:;S2@000003092Ng@<0000000<0TioL
0000002GWmX00`2GWmX00`000000VZ3I09ZPf@0309ZPf@040000002LXMP0W:7H09bQf0@000000`2O
XMH00`000000XJ;F0:6ReP020:6ReP030000002SY=H0XjCF0080XjCF100000020:JTe0030000002X
Y]@0Z:KD00<0Z:KD00<000000:VWe@2YYmD00P0000000`2ZYm<000000:bYe0040:bYe0030000002]
Z]D0[J[E0080[J[E0P0000050:n/e0040000002`[MD0/:gE0;2]e@80000000<0/JgD0000002b[mH0
0`2b[mH00`000000/[3F0;:`eP0200000080/k7F00<000000;Bbe`2d/]L00`2d/]L2000000@0][CG
00<000000;Jef02f]MP00P2f]MP3000000H0]kOH00<000000;Rhf@2h^=T00`2h^=T010000000^;WJ
0000000000030;Vjf@040000002i^]X0^K[J0;VjfP<000001P2j_=/00`000000^kgK0;^mf`030;^m
f`040000002k_]`00000000000<0_;kL00@000000;bog02l_m`0_;oL0P0000000`2m`=`000000;g0
g@020;g0g@80000000<0_L7M0000002m`]d0102m`]d00`000000_/?N0;k3gP040;k3gP030000002n
a=h0_/CN00@0_/CN0P0000040;o4gP040000002oaMl00000000000<0_lGO00@000000;o6g`000000
00000`2oa]l010000000_lKO0000000000030<37g`0400000030an000000000000<0`<SP00@00000
0<38h000000000000`31b>0010000000`LSP0000000000030<79h00400000031bN000000000000<0
`LWP00@000000<79h@00000000000P32bN400`000000oooo0?ooo`0X0?ooo`030000003oool0oooo
05H0oooo000?0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol01P3oool01P3OW4P0
0000000000000000001809cOo`/0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`06
0?ooo`@00000102Wc?<00`000000Ylcc0:O<l`02000000030:S=l`000000000000<0Ylgc10000005
0:S>l`<000000`2Xc_@200000080Z<od0P0000000`2Yco@00000000000030:S?m0<000001@2Xd?D3
000000@0Z=7e00<000000:SAmP2XdOH00P0000020:SAmP8000000P2WdOH2000000030:SBmP000000
000000<0Ym;g0`0000050:KCm`@000000`2Te?P400000080X]Ci0P0000020:3DnP8000000P2PeOX2
000000<0WMCk1000000309[Eo0@000000P2DeOh2000000<0TM?n1@00000308WBo`@000000P1od?l2
000000<0N/go3@00000206FnoP8000000`1T^_d200000080HkKl00<0000006Bbn`1T/_/00P1T/_/0
1P000000IZoi06J_n@1V[oT0000006V/n2@000000P29W^800`000000SIkO08fNg`0208fNg`030000
002@W]d0000000<00000102CWm`00`000000UioJ09NOfP0209NOfP040000002JX=T0VZ3I09ZPf@D0
00000P2LWmH00`000000Wj7F09nQeP0209nQeP030000002QX]H0XJ;F00<0XJ;F00@000000:>TeP00
000000000P2TX]@00`000000YZCD0:JTe0020:JTe0030000002XY]@0Z:KD0080Z:KD0`0000040:ZW
d`030000002/ZM@0[:WD00<0[:WD00@000000:fZe@00000000000P2^Z]@00`000000[jcD0:n/e003
0:n/e0<000000`2a[M@00`000000/ZoF0;:_eP030;:_eP800000102c/MH00`000000];;G0;Bbe`02
0;Bbe`030000002e/]H0000000D0][CG00@000000;Jef000000000000`2g]ML010000000]kOH0;Ng
f02g]mP3000000H0^;SI00<000000;Vjf@2i^]T00`2i^]T010000000^K[J0000000000030;ZkfP04
0000002j_=/0^[cK0;Zlf`<000001P2k_M/00`000000_;kL0;bng0030;bng0040000002l_m`00000
000000<0_L3L00@000000;g0g@00000000000`2m`Md00`000000_L;M0;g2g@020;g2g@030000002n
`]d0000000@0_/?N00<000000;k3gP000000102na=h00`000000_lCN000000050;o4gP040000002o
aMl0_lGO000000<0_lGO00<000000;o6g`2oa]l0102oa]l00`000000`<OO0<37g`040<37g`030000
0030b>00`<SP00@0`<SP00<000000<78h031b>001031b>000`000000`LWP0<79h0040<79h0030000
0031bN00`LWP00@0`LWP00<000000<;9h@32bN400`32bN400`000000oooo0?ooo`0X0?ooo`030000
003oool0oooo05D0oooo000?0?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02oool01P3o
ool0103ookl0M00000000018W=l=0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0
0P3oool3000000<0Yl_c0P0000030:O<l`<000001@2XcO<3000000<0Z<gc0P0000020:S>l`800000
00<0ZLkc0000000000000`2Xc_@4000000D0ZLod0`0000020:W@m08000000P2Xd?D200000080ZM3e
0P0000030:SAm@<000001@2XdOH3000000D0Z=;f0`0000020:OCm`8000000P2VdoL200000080Y]Ch
0P0000020:CDn080000000<0Y=Gi0000000000000`2Re?T3000000D0X=Gj1000000309cFo0@00000
0P2He_d2000000<0U=Gn1000000308kDo`D000000P25dol2000000<0Om3o1000000207;<o`800000
0`1/aoh2000000<0J<?n300000000`1S]_`0000006Bbn`0206Bbn`030000001V[oT0IZoi0080IZoi
00D0000006V/n01Y[?P0JJch0000000306bYm@050000001`Y_80L:Kb072VlP000000101dY?001@00
0000N:;]07RRk@1hX^d0000000@0OJ7Z00D00000086Pi`21X>L0PJ3W0000000408FOi00300000029
W^80RIkR0080RIkR00<0000008fNg`2=W]l00`000004092Ng@050000002CWm`0TioL09>Og0000000
0`2GWmX500000080VIkG00<0000009bOeP2LWmH00P2LWmH00`000000Wj7F09nQeP0309nQeP040000
002QX]H0XJ;F0:6ReP<00000102TX]@00`000000YZCD0:JTe0020:JTe0030000002XY]@0Z:KD0080
00000P2XYM<00`000000ZZOC0:ZWd`020:ZWd`030000002/ZM@0[:WD0080[:WD0`0000040:jZe003
0000002_[=@0[jcD00<0[jcD00<000000;6]e02a[M@0102a[M@01`000000/ZoF0;:_eP2b[mH00000
0;>_e@0000001@2c/MH010000000];;G0000000000020;FbeP030000002f]=L0][CG00<0][CG0P00
00050;Nee`040000002g]mP00000000000<0^;OH00@000000;Rhf@2h^=T0^;SI0`0000060;Vjf@03
0000002j^mX0^[_J00<0^[_J00@000000;Zlf`00000000000`2k_=X010000000^kgK0;^mf`2k_M/2
000000030;bnf`000000_;kL00@0_;kL00<000000;g0g02m`=`0102m`=`00`000000_L7M0;g1g@04
0;g1g@040000002m`]d00000000000<0_/;M00@000000;k3gP00000000000`2n`mh010000000_/CN
0000000000020;o4gP030000002oa=h0_lCN0080_lCN00D000000;o5gP000000_lGO000000020;o5
g`0300000030a]l0000000<0_lKO0P0000000`30a]l000000<37g`020<37g`80000000<0`<OO0000
0030b>000P30b>02000000030<78h0000000`LSP0080`LSP0P0000000`31b>0000000<79h0020<79
h080000000<0`LWP00000031bN000P31bN02000000030<79h0000000`/WQ0080`/WQ00<000000<;9
h0000000:P3oool00`000000oooo0?ooo`1D0?ooo`00403oool00`3OW4P004RL0=ooo`020?ooo`03
0;md0000M;l0oooo00L0oooo00<0_g@007Boo`3oool03@3oool00`000000oooo0?ooo`0l0?ooo`06
0000003oool000000000002Wbo8000001@2Wbo<400000080Z<cc0P0000020:S=l`80000000<0Z<gc
0000002XcO<01@2XcO<2000000D0ZLkc0`0000030:W?m08000000P2Yco@2000000030:W?m0000000
000000<0ZM3d0`0000060:W@m@<000000P2YdOD200000080Z=7f0P0000020:SBmP8000000P2Xd_H2
000000030:SCmP000000000000<0Ym?g0`0000050:KDn0<000001@2TeOT300000080X]Kj0P000002
0:3EnP8000000P2Oeo/00`000000W=Kl09cFo00200000080VmOl0P00000309SFo@@000000`2Beoh4
000000<0R]Ko00@0000008GCo`25dol0QM?o1@00000307S@o`d000000P1K_o`200000080Ek[l0P00
000305Jfna<0000000<0K:We0000001`Y_800P1`Y_800`000000M:C`07BTl00207BTl0050000001h
X^d0N:;]07RRk@0000000`1mXNX00`000000PJ3W086Pi`02086Pi`0500000025Wn@0QIoT08FOi000
00001029W^8300000080SIkO00<00000092Ng@2@W]d00P2@W]d010000000TioL09>Og02CWm`40000
00<0UYgH00<0000009VNe`2IW]L00P2IW]L00`000000W9oF09bOeP0209bOeP030000002OXMH0Wj7F
0080Wj7F100000020::Qe0030000002TX]@0Y:;D00<0Y:;D00<000000:JTe02VY=@00P2VY=@20000
00@0Z:GC00<000000:ZWd`2ZYm<00`2ZYm<010000000[:WD0000000000020:fXd`030000002^Z]@0
[Z[D00<0[Z[D00<000000:n/e02_[=@00P0000000`2`[=@000000;6]e0040;6]e0040000002b[mH0
000000000080/joE00<000000;>aeP2c/MH00`2c/MH2000000D0]K;F00<000000;Jde`2f]=L00P00
00000`2g]=L000000;Nee`050;Nee`030000002h]mP0^;OH00<0^;OH00@000000;Rhf@0000000000
0`2i^MT010000000^K[I0;Vjf@2i^]T2000000030;Zjf@000000^[_J00@0^[_J00<000000;^lfP2k
_=X00`2k_=X010000000^kgK0000000000040;bnf`030000002l_]`0_;kL0080000000<0_KoL0000
002m`=`00`2m`=`00`000000_L3L000000060;g1g@030000002n`]d0_/;M00@0_/;M00<000000;k3
gP2n`mh0102n`mh00`000000_lCN0;o4gP030;o4gP040000002oa=h00000000000<0_lGN00@00000
0;o5g`00000000000P30a]l010000000_lKO0000000000030<36g`0400000030aml00000000000<0
`<OO00@000000<38h000000000000`31b>0010000000`LSP0000000000030<78h00400000031bN00
0000000000<0`LWP00@000000<79h000000000000`31bN0010000000`/WQ0000000000020<;9h003
0000003oool0oooo02P0oooo00<000000?ooo`3oool0E03oool00100oooo00L0oono07@0001d_ol0
oooo0?noM00007@0_ooo00P0oooo00H0omnL04P00000000000000000B02Lgol:0?ooo`030000003o
ool0oooo03`0oooo0P0000040:O;lP030000002Wbo<0Yl_c008000000P2Xbo800`000000Z<cc0:S<
l`030:S<l`800000102XcO<200000080Z<gc0P0000000`2YcO<00000000000020:W>l`80000000<0
ZLkc0000002Yco@0102Yco@3000000D0ZLod0`0000020:W@m08000000`2Yd?D2000000030:WAm@00
0000000000<0ZM7e0`0000050:SBmP<000001@2XdoH300000080YmCg0P0000020:KDn08000000P2V
eOP200000080Y=Gi0P0000000`2Te_T00000000000030:;FnP<00000102Oeo/3000000@0VmOl1@00
000309KHoP@000000P2?f?l2000000<0R]Ko1000000307oDo`D000000P1_cod2000000<0Il[m5`00
000205R^nP8000000`1KZ_T01P000000GZOg05jWm`1NYoL0000006>Tm@d0000000<0N:;]0000001m
XNX00`1mXNX01@000000PJ3W086Pi`21X>L0000000@0QIoT100000001029W^80000008fNg`2=W]l:
00000080TYcJ00<0000009JMf02FWMP00P2FWMP00`000000VIkG09VNe`0209VNe`030000002LWmH0
W9oF0080W9oF00<0000009nQeP2OXMH00P0000000`2OWm@000000::Qe0030::Qe0030000002TX]@0
Y:;D00<0Y:;D00<000000:JTe02VY=@00P0000000`2WXm8000000:RUd`040:RUd`030000002ZYm<0
ZZOC0080ZZOC0P0000040:fXd`030000002^Z]@0[Z[D00<0[Z[D0`0000030;2/e0030000002a[M@0
/JgD00<0/JgD0P0000050;>_e@030000002c/MH0/k7F0080000000<0];7E0000002e/]H01@2e/]H2
000000@0]kCG00@000000;Nee`2g]ML0]kGG0`0000060;Rgf0030000002i^MT0^KWI00<0^KWI00@0
00000;Vjf@00000000000`2j^]T00`000000^[_J0;ZkfP020;ZkfP030000002k_=X0000000D0^kcJ
00<000000;bnf`2l_]/01@2l_]/2000000@0_KoL00@000000;g0g000000000000P2m`=`00`000000
_L7M0;g1g@020;g1g@80000000<0_/;M0000002n`]d00P2n`]d2000000030;k3g@000000_/?N0080
_/?N0`0000060;o4gP030000002oaMh0_lGN00@0_lGN00<000000<36g`0000000`30a]l00`000000
`<KO0<36g`040<36g`0300000030aml0`<OO00@0`<OO00<000000<78h031b>001031b>000`000000
`LSP0<78h0040<78h00300000031bN00`LWP00@0`LWP00<000000<79h031bN001031bN000`000000
`/WP0<;9h0030<;9h0030000003oool0oooo02P0oooo00<000000?ooo`3oool0D`3oool00140oooo
00D0oono07@000000000001d0;ooo`0:0?ooo`050;md00000000000000000018W=l02P3oool00`00
0000oooo0000000j0?ooo`80000000<0Yl[b0000000000000`2Wbo83000000D0Z<_b00<000000:S<
l`2Xc?<00P0000000`2Xc?800000000000030:S=l`@000001@2YcO<2000000@0ZLkc00<000000:W?
m02Yco@00P0000020:[?l`8000000P2Yco@2000000030:[@m0000000ZM3d00@0ZM3d0`0000050:WA
m@<000000P2Yd_D200000080Z=;f0P0000020:WCmP8000000P2XdoH2000000030:SDm`0000000000
00<0YmCg0`0000040:KEn0@000001@2Te_T3000000<0X]Oj1000000309oHo0@000000`2JfOd20000
0080U]Sn0P0000000`2Df_h000000000000308oHo`@000000`26eol4000000<0N=Cn2@00000205W7
n0l000000P14[oL;0000000305^Zn@000000GZOg0080GZOg00D0000006>Tm@1SY?D0HjCe00000003
06RRlP060000001]X>l0KJ3_06fPk`000000LYk/5`00000308ZKgP050000002>Vm/0SY_K08jKf`00
0000102BW=X00`000000UYgH09JMf00309JMf0030000002IW]L0VIkG0080VIkG00<0000009bOeP2L
WmH01000000409nOe0030000002RXM@0XZ7D0080XZ7D00<000000:BRe02TX]@00P2TX]@3000000<0
Yj?B00<000000:RUd`2XYM<00`2XYM<00`000000ZZOC0:ZWd`02000000030:^VdP000000[JSC00@0
[JSC00<000000:jZe02^Z]@00P0000000`2_Z]<000000;2/e0040;2/e0030000002a[M@0/JgD0080
000000<0/ZkD0000002c[mD01@2c[mD2000000@0];7E00@000000;FbeP2e/]H0]K;F0`0000050;Nd
e`040000002g]ML00000000000<0^;KG00@000000;Rgf02h]mP0^;OH0P0000000`2i^=P000000;Vi
f@040;Vif@030000002j^]T0^[[I00@0^[[I00@000000;ZkfP00000000000P2k_=X00`000000^kcJ
0;^lfP020;^lfP030000002l_MX0000000@0_;kK0`0000060;fog0030000002m`=`0_L3L00<0_L3L
00@000000;g1g@00000000000`2n`]d010000000_/;M0000000000030;k3g@040000002n`mh00000
000000<0_l?N00@000000;o4gP2oa=h0_lCN0P0000000`2oa=h000000;o5gP020;o5gP80000000P0
_lGN00000030a]l000000<36g`000000`<KO000000@0`<KO00<000000<37g`0000001030aml00`00
0000`LOO000000030<78h080000000<0`LSP00000031b>000P31b>02000000030<79h0000000`LWP
0080`LWP0P0000000`32bN0000000<79h0020<79h080000000<0`/WP00000032bN000P32bN000`00
0000`/[P0000000Z0?ooo`030000003oool0oooo0580oooo000_0?ooo`800000=`3oool4000000D0
Yl[b0`0000020:S;lP800000102Xbo83000000D0Z<cb0`0000030:W=l`8000000P2YcO<200000003
0:[>l`000000000000<0ZLkc0`0000060:[?l`800000102Zd?@00`000000ZM3d0:W@m00200000080
Z]7d0P0000020:WAm@80000000<0Z]7e0000000000000`2Yd_D3000000D0ZM?f0`0000050:SDm`<0
00000`2WeOL4000000<0Y]Kh0P0000020:CFn@80000000<0Y=Oj0000000000000P2ReoX200000003
0:;Hn`00000000000080WmSl0P0000000`2Nf_`000000000000309[Io@@000000`2Df_h5000000<0
S=[o10000003087HoP@000000`1_do/8000000<0Al3`4P0000000`15ZoP0000004RWn00204RWn0X0
000000<0HjCe0000001XX_800`1XX_801@000000KJ3_06fPk`1]X>l0000000<0LYk/00D0000007NM
j@1gWNT0MigY0000000307bLiP<000000P21Vn<00`000000QY_P08JKh00208JKh0030000002:Vmh0
RY_N0080RY_N00<0000008jKf`2>Vm/00P2>Vm/00`000000TYcJ09:LfP0209:LfP030000002FWMP0
UYgH0080UYgH00@0000009VNe`2IW]L0VIkG1000000309bMe0030000002OWm@0WioD0080WioD00<0
00000::Qe02RXM@00P2RXM@00`000000Y:;D0:BRe002000000030:FQdP000000Yj?B00@0Yj?B00<0
00000:RUd`2XYM<00P2XYM<3000000<0ZjKB00<000000:fXd`2]Z=<0102]Z=<2000000@0[j[C00<0
00000;2/e02`[=@00`2`[=@2000000@0/ZkD00@000000;>_e@2c[mD0/joE0`0000050;Bae@040000
002e/]H00000000000<0][?F00<000000;Nde`2g]=L00P2g]=L2000000D0^;KG00@000000;Rgf000
000000000`2i^=P00`000000^KWI0;Vif@020;Vif@030000002j^MP0000000H0^[[I00<000000;^l
fP2k_=X00`2k_=X010000000^kcJ0000000000030;bmfP040000002l_]/00000000000<0_KkK00@0
00000;fog02m_m`0_KoL0P0000000`2m_m/000000;g0g0030;g0g08000001P2n`]d00`000000_/?M
0;k3g@040;k3g@030000002o`mh0_l?N00<0_l?N00@000000;o4gP00000000000`2oa=h010000000
_lGN0000000000030;o5gP0400000030a]l00000000000<0`<KO00@000000<36g`00000000000`30
aml010000000`<OO0000000000020<77g`0400000031b>000000000000<0`LSP00@000000<78h000
000000000`31bN0010000000`LWP0000000000030<;9h00400000031bN000000000000<0`/WP00@0
00000<;9h000000000000P32b^000`000000oooo0?ooo`0X0?ooo`030000003oool0oooo0580oooo
000^0?ooo`030000003oool0oooo03@0oooo0`0000030:O9l@8000000P2Wb_82000000030:S:lP00
0000Z<_b00@0Z<_b100000020:W<lP8000000P2Xc?82000000030:W<lP000000000000<0ZLgc1000
00050:[>l`<000000P2Zco<2000000<0Z/oc0P0000000`2Zco<00000000000030:[@m0<000001@2Z
dO@3000000D0Z]7e0`0000020:[Bm@8000000P2YdoH200000080ZM?f0P0000020:SDm`80000000<0
Z=Cg0000000000000P2WeOL2000000030:OFn0000000000000<0Y]Kh100000030:CGnP@00000102R
f?/3000000@0W][l1000000309WKo@@000000`2Cg?h500000080RM_n1@00000307[Io@@000000`1T
d_HS0000000304bTm`000000DJ7f0080DJ7f2P0000000`1]X>l0000007:Nk00307:Nk0040000001g
WNT0MigY07NMj@<0000000<0O9cV00000021Vn<00`21Vn<01@000000QY_P08JKh026Vn00000000D0
RY_N00D0000008jKf`2>Vm/0SY_K0000000409:LfP040000002FWMP0UYgH09JMf0D000000P2IVmD0
0`000000W9gD09bMe00209bMe0030000002OWm@0WioD00<0WioD00<000000::Qe02RXM@010000004
0:FQdP030000002WXm80Yj?B0080Yj?B00<000000:RUd`2XYM<00P0000000`2YY=4000000:^VdP04
0:^VdP040000002]Z=<0[JSC0:fXd`<000001@2_Z]<010000000/:cD0;2/e02`[=@3000000D0/ZkD
00@000000;>_e@00000000000`2d/=@00`000000];7E0;Bae@030;Bae@030000002f/mH0][?F00<0
][?F00<000000;Nde`2g]=L00P0000020;ReeP030000002h]]L0^;KG00<0^;KG00<000000;Vhf02i
^=P0102i^=P010000000^KWI0000000000020;Zif0030000002j^]T0^[[I0080^[[I0`0000060;^l
fP030000002l_MX0_;gJ00@0_;gJ00<000000;fnf`2m_]/00`2m_]/010000000_KoL000000000003
0;fof`030000002m`=`0_L3L008000000P2n`M`010000000_/;M0;k2g@2n`]d2000000030;k2g@00
0000_/?M0080_/?M0`0000060;o3gP030000002oa=h0_lCN00@0_lCN00<000000;o5gP2oaMh0102o
aMh00`000000`<KO000000040<36g`0300000030aml0`<OO00@0`<OO00<000000<77g`31aml00`31
aml00`000000`LSP0<78h0040<78h00300000031bN00`LWP00@0`LWP00<000000<;9h032bN001032
bN000`000000`/WP0<;9h0040<;9h00300000032b^00`/[P00<0`/[P00<000000?ooo`3oool0:03o
ool00`000000oooo0?ooo`1A0?ooo`00;P3oool00`000000oooo0?ooo`0b0?ooo`80000000<0YlWa
0000002WbO401@2WbO42000000@0Z<[b00<000000:S;lP2Xbo800P0000020:W;lP8000000`2Yc?83
000000D0ZLcb0`0000030:[=lP8000000P2Zc_<2000000030:[>l`000000Z/oc00@0Z/oc0`000005
0:[?l`<000000P2[d?@200000080Z]7d0P0000020:_Am08000000P2ZdOD2000000030:_Bm@000000
000000<0Z];e0`0000050:WCmP<000001@2Xe?L2000000D0YmKh0`0000030:KGn@@00000102TfOX4
000000<0X][k1000000309kKo@@000000`2HgOh4000000<0TMgn1000000308GMo@D000000P1cfOTT
000000<0=9kc1P0000001@1AXOH0000005NNm01GW_@0000000<0GYca00<0000006BKkP0000002P00
000407bLiP0500000021Vn<0PI_S086Kh`0000001026Vn000`000000RY_N08ZKgP0208ZKgP040000
002>Vm/0SY_K08jKf`T000000P2EV]H00`000000VI_E09VKe@0209VKe@030000002LWM@0W9gD00<0
W9gD00<0000009nOe02OWm@00P2OWm@3000000<0XYoB00<000000:FQdP2UXM800P2UXM800`000000
Yj?B0:NSdP020:NSdP<00000102YY=400`000000ZjKB0:^VdP020:^VdP040000002]Z=<000000000
00<0[ZSB00<000000:nZd`2_Z]<00`2_Z]<3000000<0/JcC00<000000;:^e02b[]@00`2b[]@00`00
0000];3D0;B`e0030;B`e0040000002d/MD0];7E0;Bae@80000000<0][;D0000002f/mH0102f/mH2
000000@0^;GF00<000000;Rfe`2h]]L00P2h]]L00`000000^KOG000000060;Vhf0030000002j^MP0
^[WH00<0^[WH00@000000;Zjf@00000000000`2k^mT010000000^kcJ0;^lfP2k_=X2000000030;bl
fP000000_;gJ0080_;gJ0`0000060;fnf`030000002m_m/0_KoK00@0_KoK0P0000040;k1g0040000
002n`]d00000000000<0_/;M00@000000;k3g@00000000000`2o`md010000000_l?N0;o3gP2o`mh2
000000030;o4gP000000_lCN0080_lCN0P0000000`30aMh000000;o5gP020;o5gP80000000D0`<KN
00000030a]l000000<36g`02000000030<76g`000000`<OO0080`<OO0P0000000`31aml000000<77
g`020<77g`0300000031b=l0000000<0`LSP0P0000000`31b=l000000<79h0020<79h080000000<0
`LWP00000032bN000P32bN02000000030<;9h0000000`/WP0080`/WP0P0000000`32bN0000000<;:
h0020<;:h00300000032b^00000002T0oooo0P00001B0?ooo`00;P3oool00`000000oooo0?ooo`0_
0?ooo`<00000102WbO4200000080YlWa0P0000000`2XbO400000000000030:S:lP<000001@2Ybo84
00000080ZLcb0P0000020:W<lP80000000<0Z/gb0000002ZcO801@2ZcO82000000@0Z/kc00<00000
0:[?l`2Zco<00P0000020:_?l`8000000P2Zco<2000000030:_@l`000000000000<0Zm3d0`000005
0:_Am0<000001@2[d_D300000080Z]?e0P0000020:WCmP8000000P2Ze?H200000080Z=Cg0P000000
0`2Ye_L00000000000020:OFn080000000<0Z=Oh0000000000000P2VeoT2000000030:KHn@000000
00000080Y=Wj0P0000020:CJnP8000000P2Rf_/2000000030:;Lo000000000000080W]_m0P000000
0`2MgOd000000000000209SMoP8000000P2Hgoh4000000<0Sn3n10000003087Nna<000000P3Bnn80
10000000g?CN0=cdgP0000020=o/fP040000003Oi=L0000000000080hMcA2@0000000`0dW_<00000
03^Kl`0203^Kl`0300000012VO<0000000<0000000<0Eikd0000001NW?400P1NW?4010000000I9_^
06BKkP1TVnh500000003076Hj0000000000001H000000P2=V=T01@000000TIWG096Ie`2AVML00000
00D0UI[F00<0000009VKe@2IVmD00P2IVmD00`000000W9gD09bMe00209bMe0030000002OWm@0WioD
0080000000<0WigB0000002RWm800`2RWm800`000000YJ7B0:FQdP030:FQdP040000002WXm800000
00000080Z:;A00<000000:VTd@2YY=400P2YY=400`000000ZjKB0:^VdP020:^VdP8000001@2^Z=80
0`000000[j[C0:nZd`020:nZd`8000001@2a[=<010000000/ZkD0;:^e02b[]@2000000030;>^d`00
0000];3D00@0];3D0`0000030;Jbe0030000002f/mH0][?F0080][?F00<000000;Rce@0000001@2h
]MH010000000^;KG0000000000020;Vge`030000002i^=P0^KSH0080^KSH0`0000060;Zif0030000
002k^mT0^k_I00<0^k_I00@000000;^lfP00000000000`2l_=X010000000_;gJ0000000000030;fn
fP040000002m_]/0_KkK0;fnf`80000000<0_KoK0000002m_m/00P2m_m/3000000H0_/7L00<00000
0;k2g@2n`]d0102n`]d00`000000_l?M0;o3g@030;o3g@040000002o`mh00000000000<0_lCN00@0
00000;o4gP00000000000`30aMh010000000_lGN0000000000030<36gP0400000030a]l000000000
00<0`LKO00@000000<37g`00000000000`31aml010000000`LOO0000000000020<78g`0400000031
b>000000000000<0`LSO00@000000<79h000000000000`31bN0010000000`/WP0000000000030<;9
h00400000032bN000000000000<0`/WP00@000000<;:h000000000000P32b^000`000000oooo0?oo
o`0U0?ooo`80000000<0oooo0000003oool0D@3oool002h0oooo00<000000?ooo`3oool0;03oool3
00000080YlSa0P0000030:O9l@@000001@2XbO4300000080ZL[a0P0000020:W;lP8000000P2Ybo40
0`000000ZLcb0:W<lP030:W<lP800000102ZcO8200000080Z/gb0P0000000`2[c_80000000000003
0:[>l`<000001@2[co<3000000D0Zm3c0`0000020:_Am08000000P2[dO@200000080Zm;d0P000002
0:_Bm@80000000<0Zm?e0000000000000`2ZdoD3000000@0Z]Cf100000050:WFm`8000001@2XeoP2
000000D0Y]Si0P0000050:CJnP<000000`2Rg?`4000000@0WMgm0`00000309SOoP8000000P2GhOd4
000000<0SN;l1`00000207OPlP80000000<0M>3[000000000000200000020=;khP040000003Lm=h0
g?CN00000080gncJ00D000000=oTe`3Oi=L000000=kNeP02000000030>7Ld@000000g]SA00<00000
00<0083=0000000000001@0000000`0kVo<0000004:Il`0204:Il`D0000000<0GYca0000001TVnh0
1000000306^Ij`050000001aV>P0LISX076Hj00000000`1gUn@00`000000OIOQ07fGh@0207fGh@03
00000023Umh0PiON0080PiON00D0000008RGg028Um`0R9OL0000000408fHf@030000002AVML0TIWG
0080TIWG00<0000009FJeP2EV]H00P2EV]H00`000000VI_E09VKe@0209VKe@040000002LWM@0W9gD
09bMe0@000000`2OWM800`000000XYoB0::OdP030::OdP030000002UXM80YJ7B0080YJ7B0P000004
0:RRd@030000002YY=40ZJCA00<0ZJCA00@000000:^VdP00000000000P2]Y]400`000000[ZSB0:jX
dP030:jXdP040000002_Z]<0000000000080/:[B00<000000;6/d`2a[=<00`2a[=<3000000<0/jkC
00<000000;B`e02d/=@00`2d/=@00`000000][;D0;Jbe0040;Jbe0040000002f/mH0000000000080
^;?E00<000000;ReeP2h]MH00`2h]MH2000000D0^KOG00@000000;Vhf000000000000`2j^=P01000
0000^[WH0;Zif02j^MP3000000H0^k_I00<000000;blfP2l_=X0102l_=X00`000000_KkJ0;fnfP03
0;fnfP040000002m_]/00000000000<0_KoK00@000000;fof`00000000000`2n`=/010000000_/7L
0;k1g02n`M`2000000030;o2g0000000_/;M0080_/;M0`0000060;o3g@030000002oa=h0_lCN00@0
_lCN00<000000<35gP30aMh01030aMh00`000000`<KN0<36gP040<36gP0300000031a]l0000000@0
`LKO00<000000<77g`31aml01031aml00`000000`LSO0<78g`030<78g`0300000031b=l0`LSO00@0
`LSO00<000000<79h031bN001031bN000`000000`/WP0<;9h0040<;9h00300000032bN00`/WP00@0
`/WP00<000000<;:h032b^000`32b^000`000000oooo0?ooo`0X0?ooo`030000003oool0oooo0180
oooo00D0gia8000000000000000004RLg`040?ooo`050?oOW018000000000000M02oool0;`3oool0
02h0oooo00<000000?ooo`3oool0:P3oool2000000030:O7l0000000YlSa00@0YlSa0`0000030:S9
l@8000000P2XbO42000000030:W:l@000000ZL[a00@0ZL[a0P0000050:W;l@030000002Yc?80ZLcb
0080000000<0Z/cb0000000000000`2ZcO84000000D0Zlkb0`0000020:_>lP8000000P2[co<20000
0080Zloc0P0000020:_@l`80000000<0[=3c0000000000000`2[dO@3000000D0Zm;d0`0000050:_C
m@<000000`2[e?H4000000<0Z]Gf0P0000020:WFm`80000000<0ZMOg0000000000000P2XeoP20000
00030:SHn000000000000080Y]Si0P0000000`2Wf_T00000000000020:CJnP8000000P2Ug?/40000
00@0X]kl1000000309gPo@<000000`2GhOd200000080UnCm1000000208_TnP80000000<0R^Kg0000
000000002000000209kdj080000000D0`OcU0000003Bnn80d__R000000020=cdgP040000003Ok=X0
gncJ00000080gnCG00<000000=kNeP3Ng]H00P3Ng]H010000000g]SA0=kHd@0000020=_Dd@`00000
00<0@YWc0000001:Uo800P1:Uo8:0000000306^Ij`000000LISX0080LISX00<0000007NGi01gUn@0
0P1gUn@01@000000OIOQ07fGh@1mUn40000000@0PiON00<0000008RGg028Um`00P28Um`00`000000
SISI08fHf@0208fHf@050000002AVML0TIWG096Ie`0000001@2EV]H00`000000VI_E09VKe@050000
0080W9_B00<0000009nMdP2OWM800`2OWM800`000000XYoB0::OdP020::OdP030000002UXM80YJ7B
0080000000<0YJ3@0000002XX]40102XX]400`000000ZJCA0:VTd@020:VTd@800000102]Y]400`00
0000[ZSB0:jXdP030:jXdP800000102`Z]800`000000/JcC0;6/d`030;6/d`030000002c[]<0/jkC
00@0/jkC00<000000;B`e02d/=@00P0000000`2f/=<000000;Jbe0050;Jbe0030000002h/mD0^;?E
00<0^;?E00<000000;ReeP2h]MH00P0000000`2i]]H000000;Vge`050;Vge`030000002j^=P0^[SH
00<0^[SH00@000000;Zif000000000000`2k^]P010000000^k_I0;^kf@2k^mT2000000030;bkf@00
0000_;cJ0080_;cJ0`0000060;fnfP030000002m_m/0_KoK00@0_KoK00<000000;k0f`2n`=/00`2n
`=/010000000_/7L0000000000030;o2g0040000002n`]d00000000000<0_l?M00@000000;o3g@2o
`md0_l?M0P0000000`2oa=d000000;o4gP020;o4gP80000000<0`<CN00000030aMh00P30aMh20000
00030<35gP000000`<KN0080`<KN0P0000001@31a]h000000<76g`000000`LKO0080000000<0`LOO
00000031aml00P31aml2000000030<77g`000000`LSO0080`LSO00<000000<78g`0000001031b=l0
0`000000`/WP000000030<79h080000000<0`/WP00000032bN000P32bN02000000030<;9h0000000
`/WP0080`/WP0P0000000`32bN0000000<;:h0020<;:h00300000032b^00000002X0oooo00<00000
0?ooo`3oool0403oool01`3OW4P0000007Boo`3oool0oooO09a80018W=l00P3oool01`3ookl0M000
07Boo`3oool0okmd0000M02oool0;P3oool002h0oooo00<000000?ooo`3oool09`3oool3000000@0
YlO`00<000000:O8l@2Wb?400P0000000`2Xb?0000000:S9l@050:S9l@800000102Yb_400`000000
ZL[a0:W:l@02000000030:[;l@000000000000@0ZL_a0`0000050:[<lP<000000`2[cO8200000080
Zlkb0P0000000`2[c_800000000000030:_>lP<000001@2[co<3000000D0[=3c0`0000020:cAl`80
00000P2[d_@200000080[=;d0P0000020:_Cm@80000000<0[=?e0000000000000P2[e?H200000003
0:cEmP000000000000<0Z]Gf100000030:WGm`@00000102Xf?P3000000@0Ym[i0`0000030:GLn`80
000000<0YMgk0000000000000P2Rg_`2000000<0X^3l1000000309kRo@<000000`2Gi?d200000080
UnKk1P00000208[Yl`80000000<0SNc_0000000000000P0000000`2Nm>P000000000000900000080
gncJ00D000000=oTe`3Oi=L0gnCG000000030=kNeP040000003Lf]@00000000000@0fmCA00<00000
0=WAd@3IdM403`00000205:Fl0030000001JU>d0FYC]0080FYC]1P0000000`1aV>P0000007NGi002
07NGi0030000001mUn40OIOQ0080OIOQ00<0000008>GgP23Umh00P23Umh01@000000R9OL08RGg028
Um`0000000@0SISI00<00000096Ie`2AVML00P2AVML00`000000UI[F09FJeP0500000080V9WB00<0
000009bKdP2LVm800`2LVm800`000000WigB09nMdP0209nMdP040000002RWm80XYoB0::OdP@00000
0`2UX=000`000000Z:;A0:RRd@030:RRd@030000002YY=40ZJCA0080000000<0ZjC@0000002]Y]40
102]Y]400`000000[ZSB0:jXdP02000000030:nXd@000000/:[B00@0/:[B00<000000;6/d`2a[=<0
0P0000000`2c[=8000000;>^d`050;>^d`8000000`2f/=<00`000000][;D0;Jbe0020;Jbe0<00000
1P2h/mD2000000@0^KKF00@000000;Vge`2i]mL0^KOG0`0000060;Zhf0030000002k^]P0^k[H00<0
^k[H00@000000;^kf@00000000000`2l^mT010000000_;cJ0000000000030;fmfP040000002m_]X0
_KkJ0;fnfP80000000<0_KkJ0000002m_m/00P2m_m/2000000030;jof`000000_/3K00@0_/3K00<0
00000;o2g02o`]`0102o`]`00`000000_l?M0;o3g@030;o3g@040000002o`md00000000000<0_lCM
00@000000;o4gP00000000000`30a=h010000000`<GN0000000000030<35gP0400000030a]h00000
000000<0`LKN00@000000<76g`00000000000`31aml010000000`LOO0000000000030<77g`040000
0031b=l00000000000<0`LSO00@000000<78g`00000000000P32bN0010000000`LWP000000000003
0<;9h00400000032bN000000000000<0`/WP00@000000<;9h000000000000`32bN0010000000`/[P
0000000000020<;:h0030000003oool0oooo02P0oooo00<000000?ooo`3oool03`3oool00`3ooml0
W4P0001d_`030?ooo`070?noM000B9`0gooo0?ooo`2oM00004RL0=ooo`020?ooo`030;md0000M;l0
oooo02d0oooo000]0?ooo`040000003oool00000000002<0oooo100000020:O7l08000000`2Wao03
000000@0Z<S`0P0000020:S9l@80000000<0ZLW`0000000000000`2Yb_43000000D0Z/_a10000002
0:_<l@8000000P2Zc?82000000030:_<l@000000Zlgb00@0Zlgb0`0000050:_>lP<000000P2/co82
00000080Zloc0P0000020:c@l`8000000P2/d?<2000000030:cAl`000000000000<0[=7c0`000005
0:cBm0<000001@2/doD2000000D0[=Gf0`0000030:_FmP@00000102Zf?L4000000<0ZMWh10000003
0:SKnP<00000102UgO/3000000<0X^3l0P0000020:?Ro0030000002Nh_d0W^;m008000000P2NiO`4
00000080UnKk0P0000000`2HjOX000000000000209[/n0`000000P37nNX010000000dOKV0=7fiP00
00020=SahP030000003Kj]h00000008000000P3Oi=L00`000000g]kF0=kNeP0200000080g=[D00H0
00000=[Fe000000000000=_Dd@0000040=WAd@/0000000<0>Xk_0000000000000P0000000`1BU_00
000005ZDk@0205ZDk@060000001STnX0Hi?Z06>CjP000000JY;V2@0000000`1mUn40000008>GgP03
08>GgP0300000028Um`0R9OL0080R9OL00<0000008fHf@2=V=T02@00000309BGd`030000002HVM80
V9WB0080V9WB00<0000009bKdP2LVm800P2LVm800`000000WigB09nMdP0209nMdP@000000P2SWLl0
0`000000YJ3@0:FPd0030:FPd0030000002XX]40Z:;A0080Z:;A0`0000030:^Td0030000002]Y]40
[JKA00@0[JKA0P0000030:nXd@030000002`Z]80/:[B00@0/:[B0P0000030;>/dP030000002c[]<0
/jkC0080/jkC0`0000050;J`d`040000002f/]@00000000000<0]k;D00@000000;Rce@2h/mD0^;?E
0`0000050;VfeP040000002i]mL00000000000<0^[OG00@000000;Zhf02j^=P0^[SH0P0000000`2k
^ML000000;^jf0030;^jf08000001P2l^mT00`000000_KgJ0;fmfP030;fmfP040000002m_]X00000
000000<0_KkJ00@000000;fof`00000000000`2n_m/00`000000_/3K0;k0f`020;k0f`030000002o
`M/0000000@0_l;L00<000000;o2g00000001@2o`md00`000000_lCM0;o4g@040;o4g@0300000030
a=h0`<CN00@0`<CN00<000000<35gP30aMh01030aMh00`000000`LKN0<76gP040<76gP0300000031
aml0000000@0`LOO00<000000<77g`31aml01031aml00`000000`LSO0<78g`040<78g`0300000032
bN00`/WP00<0`/WP00<000000<;9h032bN001032bN000`000000`/WP0<;9h0040<;9h00300000032
bN00`/WP00@0`/WP00<000000<;:h032b^000`32b^000`000000oooo0?ooo`0X0?ooo`030000003o
ool0oooo00l0oooo00<0_g@0001d_`3oool00P3oool01P2oM00004RL0=ooo`3ookl0M00004RLg`<0
oooo00<0gia80000M02oool0;@3oool002d0oooo0P00000S0?ooo`8000000P2Wa^l00`000000YlO`
0:O7l0030:O7l0<000000P2Xao02000000<0Z<S`100000050:W9l0<000000P2Zb_4200000080Z/_a
0P0000020:_;l@030000002[c?40Zlca00<0Zlca0P0000040:_<l@030000002[cO80Zlgb00800000
0P2/cO4200000080Zlkb0P0000000`2/c_8000000:c?lP040:c?lP8000001@2/d?<3000000D0[=7c
0`0000020:gBl`8000000P2/d_@200000080[M?d0P0000020:cCm@80000000<0[MCe000000000000
0P2/eOH2000000030:cFmP00000000000080ZmKf0P0000000`2/eoH00000000000020:[Hm`800000
0P2[fOL200000080ZMWh0P0000000`2ZfoP00000000000020:SKnP8000000P2YgOX4000000<0Y]ok
0`0000040:?Ro0L000000`2Pio/6000000<0V^ch0P000000102Ol?D000000000002Vlo8600000080
aoWZ00D000000=7fiP3Am^H000000=SahP0200000080fn[N00<000000=cTf`3Li=/00P0000020=kN
eP050000003Lf]@0g=[D0=cJe0000000103Je]@010000000fM7A0=WAd@3IdM42000000<0eloA1@00
0000100<PN<0000000000000000303Z>k`H0000000D0FYC]0000001STnX0Hi?Z0000000406ZBiP06
0000001bT^<0LY;S07:Bh`000000NI;O3P00000208^De`030000002@UMD0T9GE0080T9GE00<00000
09BGd`2DUm<00P2DUm<00`000000V9WB09RIdP0209RIdP030000002LVm80W9_B00<0W9_B00H00000
09nMdP00000000000:2Kc`0000050:>Mc`030000002UX=00YJ3@0080YJ3@00<000000:RRd@2XX]40
0P0000000`2YX/l000000:^Td0040:^Td0040000002]Y]40[JKA0:fVd@<000001@2_Z=4010000000
/:[B0;2ZdP2`Z]83000000D0/jcB00@000000;>^d`00000000000`2e[m800`000000][3C0;J`d`03
0;J`d`030000002g/]@0]k;D00<0]k;D00@000000;Rce@00000000000`2i]=D00`000000^KKF0;Vf
eP020;VfeP8000001@2j]mL010000000^[SH0000000000030;^ie`030000002k^]P0^k[H00800000
0P2l^mP00`000000_;_I0;bkf@020;bkf@030000002m_=T0000000D0_KgJ00<000000;fnfP2m_]X0
102m_]X00`000000_[oK0;jof`040;jof`040000002n`=/00000000000<0_l7K00@000000;o2g000
000000000P2o`]`00`000000_l?M0;o3g@020;o3g@030000002o`md0000000@0_lCM00<000000<34
g@0000000`30a=h2000000030<35gP000000`<GN0080`<GN0P0000000`31a]h000000<76gP020<76
gP80000000D0`LON00000031aml000000<77g`02000000030<77g`000000`LOO0080`LOO0P000000
0`31b=l000000<78g`020<78g`80000000<0`/SO00000032bN000P32bN000`000000`/WP00000003
0<;9h080000000<0`/WP00000032bN000P32bN02000000030<;9h0000000`/WP0080`/WP0P000000
0`32b^0000000<;:h0020<;:h00300000032b^00000002X0oooo00<000000?ooo`3oool03P3oool0
2`3ogi`0B01d0;ooo`3oool0_g@00000001d_ol0oooo0?oOW0180000M;oo00<0oooo00<0okmd0000
B02Lgol0;@3oool002d0oooo00<000000?ooo`3oool0803oool2000000D0YlK_00<000000:O7l02W
ao000P0000000`2Xanl000000:S7l0040:S7l0<000000`2Yb?0200000080ZLW`0P0000000`2ZbO00
00000:[:l@040:[:l@8000001@2[bo400`000000Zlca0:_<l@02000000030:_<l@000000000000<0
Zlca0`0000050:c=l@<00000102/c_800`000000[<ob0:c?lP02000000030:g?lP00000000000080
[=3c0P0000020:g@lP8000000P2/dO<2000000030:gAl`000000[M;c00@0[M;c0P0000050:gCm0<0
00001@2]e?D2000000D0[=Kf0P0000050:cGmP8000001@2[fOL3000000D0Z]_h0P0000030:WMnP80
000000<0Z]oj0000000000000P2Vgo/200000080Z>;k100000020:GTn`8000000P2Wio/400000080
X^[j0P0000020:G]n@h000000P37nNX01P000000dOKV0=7fiP000000f?7R000000<0fn[N00H00000
0=cTf`3Li=/0g>CK0000003KgmT2000000@0g=[D00T000000=[Fe03Je]@0f]KD0000003Hdm<00000
0=WAd@000000103Gcm4<0000000303Z>k`000000A8g]0080A8g]00<0000004n=j`0000000`000000
0`1STnX0000006ZBiP0206ZBiP050000001bT^<0LY;S07:Bh`000000101iT]l01@000000Oi;L07nB
g01oT]`0000000@0QI?I00<0000008^De`2;U=L00P2;U=L01@000000T9GE092Ee@2@UMD0000000D0
U9OC00<0000009RIdP2HVM800P2HVM800`000000W9_B09bKdP04000000@0X9_?00<000000:>Mc`2S
WLl00P2SWLl00`000000YJ3@0:FPd0020:FPd0<00000102YX/l00`000000ZjC@0:^Td0020:^Td004
0000002]Y]400000000000<0[ZK?00<000000:nXd@2_Z=400P2_Z=4010000000/:[B000000000003
0;:[d@030000002c[=80/jcB00<0/jcB00<000000;F_dP2e[m800`2e[m8010000000][3C0;J`d`2f
/=<3000000H0]k;D00<000000;Vde@2i]=D00`2i]=D00`000000^KKF0;VfeP02000000030;ZfeP00
0000^[OG00D0^[OG00<000000;^ie`2k^ML0102k^ML2000000D0_;_H00@000000;bkf@0000000000
0P2m_=T00`000000_KgJ0;fmfP020;fmfP030000002n_]X0000000@0_KkJ00<000000;jofP000000
1P2n_m/00`000000_l7K0;o1f`040;o1f`030000002o`]`0_l;L00<0_l;L00@000000;o3g@000000
00000`2o`md010000000_lCM0000000000020<34g@0400000030a=h00000000000<0`<GN00@00000
0<35gP00000000000`31a]h010000000`LKN0000000000030<77gP0400000031aml00000000000<0
`LOO00@000000<77g`00000000000`31b=l010000000`LSO0000000000030<;8g`0400000032bN00
000000000080`/WP00@000000<;9h000000000000`32bN0010000000`/WP0000000000030<;9h004
00000032bN000000000000<0`/[P00@000000<;:h000000000000P32b^000`000000oooo0?ooo`0X
0?ooo`030000003oool0oooo00l0oooo00D0oono07A8B0180000000004RLg`020?ooo`030?oOW018
0000M;oo00<0oooo00<0okmd0000B02Lgol0;@3oool002d0oooo00<000000?ooo`3oool07P3oool2
000000030:O5k`000000YlK_00D0YlK_0P0000040:S7k`030000002Xao00Z<O`0080000000<0ZLS`
0000002Yb?001@2Yb?02000000@0Z/W`00<000000:[:l@2Zb_400P0000000`2[b_00000000000003
0:_;l@@000001@2[c?4300000080[<ga0P0000020:c=l@8000000P2/c_42000000<0[<kb0`000005
0:g?lP8000001@2]d?83000000@0[M7c00<000000:gBl`2]d_<00P0000000`2]d_<0000000000002
0:gCm08000000P2^e?@200000080[MCe0P0000000`2]eOD00000000000020:cFmP80000000<0[MOf
0000000000000P2/eoH2000000030:gHmP00000000000080ZmWg0P0000020:cJm`8000000P2ZfoP2
00000080Zmch0`0000050:[OnP8000000`2Xh_/200000080ZNCj0`0000030:OWn`8000000P2YjOX3
000000<0YNgi100000020;7bm080000000D0^?Cb00000031mNl0`OG_000000020<Wdk0P000000P3K
j]h01@000000g>CK0=cTf`3Li=/000000080fmoI00H000000=[Ke`00000000000=cJe00000030=[F
e0050000003Hdm<0f=?C0=SCd`0000000P3Gcm43000000@0eLgB100000000`00K]D0000000000006
0000000604B=k@000000Chg[04n=j`000000FHcX1@0000000`1ZT^H0000007:Bh`0207:Bh`050000
001iT]l0NI;O07VBg`000000101oT]`01@000000QI?I08FCf@25TmT0000000@0RiCG00<00000092E
e@2@UMD00P2@UMD00`000000U9OC09BGd`0209BGd`040000002HVM80V9WB09RIdP@000000`2LV<l0
0`000000X9_?0:2Kc`020:2Kc`030000002SWLl0Xig?00<0Xig?00H000000:FPd000000000000:NO
cP0000050:VRc`030000002[Y=00ZjC@0080ZjC@0P0000050:jVc`030000002_Z=40[jSA0080[jSA
0P0000050;:[d@040000002c[=80/jcB0;>/dP<000001P2e[m83000000<0]k7C00@000000;Nbe02g
/]@0]k;D0P0000000`2h/m@000000;Vde@040;Vde@800000102j]]H010000000^[OG0;Zge`2j]mL2
000000030;^he`000000^kWG00<0^kWG00<000000;bje`0000001P2l^mP00`000000_KcI0;flf@03
0;flf@040000002m_MX00000000000<0_[kJ00@000000;fnfP00000000000P2n_mX00`000000_[oK
0;jof`020;jof`80000000<0_l3K0000002o`M/00P2o`M/3000000H0_l;L00<000000;o3g@2o`md0
102o`md00`000000`<CM0<34g@030<34g@0300000030aMh0`<GN00@0`<GN00<000000<76gP31a]h0
1031a]h00`000000`LON0<77gP040<77gP0400000031aml0`LOO000000<0`LOO00<000000<78g`31
b=l01031b=l00`000000`/SO0<;8g`040<;8g`0300000032bN00`/WP00<0`/WP00<000000<;9h032
bN001032bN000`000000`/WP0<;9h0040<;9h00300000032b^00`/[P00@0`/[P00<000000<;:h032
b^000`32b^000`000000oooo0?ooo`0X0?ooo`030000003oool0oooo00h0oooo00@0omnL04P00000
000007Bo0`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo02d0oooo000]0?ooo`03
0000003oool0oooo01/0oooo0`0000040:O5k`8000000P2Wa^l2000000030:S6k`000000Z<O_00@0
Z<O_0P0000040:W8l08000000P2Yb?02000000030:[9l0000000000000<0Z/W`0`0000050:_:l0<0
00000`2[bo0200000080Zlca0P0000000`2/c?4000000:c=l@040:c=l@8000001P2/c_4300000080
[Lob0P0000020:g?lP80000000<0[M3b0000000000000P2]d?8200000080[]7b0P0000030:gAl`<0
00001@2]d_<2000000D0[]Cd0`0000050:gEm@8000001@2]eoH2000000D0[MSf0P0000040:cJm`@0
00000P2[g?P200000080[Mkh0P0000020:[OnP8000000P2[hOT3000000<0ZNCj0P0000020:cVnP<0
00000`2YjOX200000080[Nci1@0000020;7bm080000000@0^?Cb000000000000`OG_0P0000030<Wd
k0060000003@lNP000000000003EkN@000000P3Hj>4300000080g>CK00@000000=_Of@3KgmT00000
103JfmL200000080f]KD00@000000=SCd`3Hdm<0f=?C100000000`3Gcm4000000=G=dP030=G=dP04
0000003Dc=8000000=C;d@@0000000H007;M0000000000009h?Y0000000dPnT300000080Chg[00H0
000005V<j01IS>P0FHcX0000001RS>@40000000307:Bh`000000NI;O00<0NI;O00D0000007nBg01o
T]`0Oi;L0000000408FCf@030000002;U=L0RiCG0080RiCG00D00000092Ee@2@UMD0T9GE00000003
09BGd`D0000000<0VIK@0000002LV<l0102LV<l00`000000X9_?0:2Kc`020:2Kc`030000002SWLl0
Xig?0080Xig?0`0000040:NOcP030000002YX/l0ZJ;?00<0ZJ;?00H000000:^Td000000000000:bT
cP0000060:jVc`060000002_Z=4000000000002aZ=0000001P2bZm43000000<0]:gA00<000000;F_
dP2e[m800P2e[m82000000D0]k7C00@000000;Nbe000000000000`2h/m@00`000000^KCE0;Vde@02
0;Vde@030000002j]MD0000000@0^[KF00@000000;Zge`0000000000102k^=L010000000^kWG0000
000000020;bje`030000002l^mP0_;_H0080_;_H0`0000060;flf@030000002n_]X0_[kJ00@0_[kJ
00<000000;jofP2n_mX00`2n_mX010000000_[oK0000000000030;o0f`040000002o`M/000000000
00<0_l;K00@000000;o2g02o`]`0_l;L0P0000000`30`m`000000;o3g@020;o3g@80000000<0`<CM
00000030a=d00P30a=d00`000000`LGM000000040<35gP0300000031aMh0000000@0`LKN00<00000
0<76gP0000000`31amh2000000050<77gP000000`LOO00000031aml00P0000000`32aml000000<78
g`020<78g`80000000<0`/SO00000032b=l00P32b=l2000000030<;9g`000000`/WP0080`/WP00<0
00000<;9h00000000`32bN02000000030<;9h0000000`/WP0080`/WP0P0000000`32b^0000000<;:
h0020<;:h080000000<0`/[P00000032b^000P32b^000`000000`/[P0000000Z0?ooo`030000003o
ool0oooo00`0oooo00H0omnL04P0000007@0_ooo0?noM000M;l20?ooo`030?oOW0180000M;oo00<0
oooo00<0okmd0000B02Lgol0;@3oool002d0oooo00<000000?ooo`3oool0603oool300000080YlG^
0P0000030:O5k`@00000102Xa^l00`000000Z<O_0:S7k`02000000030:W7k`000000000000<0ZLS`
100000050:[9l0<000000P2[bO0200000080Zl[`0P0000000`2/bo0000000:_;l0050:_;l0800000
102/c?400`000000[<ga0:c=l@02000000030:g=l@000000000000<0[<ka0P0000000`2]c_400000
0:g?lP040:g?lP8000001@2]d?82000000D0[]7b100000020:kBl`8000000P2]d_<2000000030:kC
l`00000000000080[]Cd0P0000020:kDm08000000P2]eOD2000000030:kFm000000000000080[MOf
0P0000000`2^f?H00000000000020:gHmP8000000P2^fOH4000000<0[M_g0`0000040:gNn0@00000
0P2[hOT2000000<0[N;i0`0000020:cVnP8000000`2_j?T300000080[Nci0P0000020;7^m`030000
002gl?D0]o3e00T000000P39m>`400000080eNgT00D000000=SXh@3Hj>40f>SQ000000020=WTgP80
00000P3KgmT00`000000f]_G0=[Ke`020=[Ke`030000003IemH0fMOF008000000P3Hdm<3000000<0
em7C00@000000=G=dP3EcM80eLgB0P0000030=C<dP030000003CbM40dlWA00<000000P00L]d30000
0080=8?Y1000000305V<j0060000001RS>@0HXcT06:<i0000000JhcQ3P0000000`25TmT0000008^D
e`0208^De`T000000P2DU=000`000000VIK@09VFd00209VFd0030000002LV<l0W9S?00<0W9S?00<0
00000:2Kc`2PVll00P2PVll010000000Xig?0000000000020:BLc@030000002WWlh0Yio>0080Yio>
00<000000:VRc`2YX/l00P2YX/l3000000@0[:C>00<000000:jVc`2^Y/l00`2^Y/l2000000@0/JS@
00<000000;:[d@2bZm400P2bZm42000000D0]:gA00<000000;F_dP2e[m800P0000020;J_dP030000
002g/M<0]k7C00<0]k7C00<000000;Rce02h/m@0102h/m@010000000^KCE0000000000020;Zee@03
0000002j]]H0^[KF00<0^[KF00<000000;^he`2k^=L01@2k^=L00`000000_;[G0;bje`030;bje`04
0000002l^mP00000000000<0_K_H00@000000;flf@2m_=T0_KcI0P0000000`2n_MT000000;jnfP02
0;jnfP<000001P2n_mX00`000000_l3K0;o0f`040;o0f`030000002o`]/0_l;K00<0_l;K00@00000
0;o2g000000000000`30`m`010000000_l?M0000000000030<34g@0400000030a=d00000000000<0
`LGM00@000000<35gP00000000000`31aMh010000000`LKN0000000000020<76gP0400000031amh0
0000000000<0`LON00@000000<77g`00000000000`32aml010000000`LSO0000000000030<;8g`04
00000032b=l00000000000<0`/WO00@000000<;9h000000000000P32bN0010000000`/WP00000000
00030<;9h00400000032bN000000000000<0`/[P00@000000<;:h000000000000`32b^0010000000
`/[P0000000000020<;:h0030000003oool0oooo02P0oooo00<000000?ooo`3oool0303oool00`2o
M000001d0;ooo`020?ooo`060;md0018W=l0oooo0?oo_`1d0000B9cO0`3oool00`3OW4P0001d0;oo
o`0]0?ooo`00;@3oool00`000000oooo0?ooo`0F0?ooo`80000000<0YlC^0000002WaNh0102WaNh3
000000<0Z<K_0P0000030:S6k`<000001@2Yanl3000000<0Z/S_0P0000020:[9l080000000<0ZlW_
0000002[bO00102[bO02000000@0[<_`0P0000020:_;l080000000<0[<_`0000000000000`2/c?43
000000D0[Lga0`0000040:g>l@030000002]co80[Lob0080000000<0[/oa0000000000000P2]d?82
000000030:k@lP00000000000080[]7b0P0000020:oAlP8000000`2^d_<3000000@0[]?c0`000005
0:kDm0<000001@2^e_@2000000@0[]Sf0`0000030:kImP80000000<0[m_f0000000000000P2]foL2
00000080[mgg100000030:kOn0<000000`2]h_T200000080/>Ch0`0000030:oXn@8000000P2bj_P3
00000080/Nkg0P0000020;O`m@8000000P2mlO8010000000a?7`0<Cal00000020<[`k@D000000P3E
kN@01@000000f>SQ0=SXh@3Hj>40000000<0fNCN00@000000=WOf`0000000000103JfmL010000000
fMOF0=WGeP3IemH5000000@0em7C00D000000=G?d`000000eLgB000000040=C<dP030000003Cbm80
00000080dlWA1@0000000`0AMN8000000000000203B3j@0300000011Q>P0000000<0000000<0FHcX
0000001RS>@00P1RS>@01@000000JhcQ06^<h@1[S>40000000<0LhgM00D0000007^=fP1kSMX0NhgJ
0000000208:>e`D0000000<0RI3D0000002?TM80102?TM800`000000U9C@09BDd00209BDd0050000
002IU]00VIK@09VFd00000001@2LV<l010000000X9_?0:2Kc`2PVll3000000@0Y9c=00<000000:NO
cP2WWlh00`2WWlh010000000ZJ;?0000000000020:^Qc@030000002/Y<h0[:C>0080[:C>00@00000
0:jVc`2^Y/l0[ZK?0P0000000`2`Y/h000000;6Xd0030;6Xd0030000002bZm40/Z_A008000000P2c
Zm000`000000]:gA0;B]d@030;B]d@800000102f[m800`000000]k7C0;Nad`020;Nad`030000002h
/]<0000000H0^;?D00<000000;Zee@2j]MD00`2j]MD00`000000^[KF0;ZfeP02000000030;^geP00
0000^kSG00<0^kSG0`0000060;bje`030000002m^mP0_K_H00<0_K_H00@000000;flf@0000000000
0`2n_MT010000000_[kJ0000000000030;jnfP040000002n_mX0_[oJ0;jofP80000000<0_l3J0000
002o`=/00P2o`=/3000000H0_l;K00<000000<33g030`m`01030`m`00`000000`<CM0<34g@040<34
g@0300000031aMd0`LGM00@0`LGM00<000000<75gP31aMh01031aMh00`000000`LKN0<76gP030<76
gP0300000031amh0`LON00@0`LON00@000000<;7g`32aml000000`32aml00`000000`/SO0<;8g`04
0<;8g`0300000032bMl0`/WO00@0`/WO00<000000<;9h032bN000`32bN000`000000`/WP0<;9h004
0<;9h00300000032b^00`/[P00@0`/[P00<000000<;:h032b^001032b^000`000000`/[P0<;:h003
0<;:h0030000003oool0oooo02P0oooo00<000000?ooo`3oool02`3oool00`2oM00004RL0=ooo`02
0?ooo`070=nLB000M;l0oooo0?ooo`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo02d0oooo
000/0?ooo`040000003oool00000000001<0oooo0`0000040:O4kP030000002WaNh0YlG^00800000
00<0Z<G^0000002Xa^l01@2Xa^l300000080ZLK_0P0000020:W7k`80000000<0Z/O_0000002Zb>l0
102Zb>l3000000@0ZlW_00<000000:_9l02[bO000P0000000`2/b^l000000:c;l0040:c;l0<00000
1@2/bo0300000080[Lc`0P0000020:g=l@8000000P2]cO42000000<0[Lka0`0000050:k?l@800000
1@2^d?82000000H0[m7b0`0000030:oBl`@000000P2_e?<200000080[]Cd0P0000020:oEm0800000
0P2^e_@200000080[mOd100000030:oIm@<000001@2_foH2000000<0[mgg0P0000000`2`g_L00000
000000020:kOn08000000P2`hOP3000000<0/>Ch0P0000020;;Vn0<000000`2bj_P200000080]^_f
1P0000020;galP80000000<0a?7`0000000000000P3:l>d2000000040<o]j@000000d^[V0=;ZiP<0
00000P3Hj>400`000000fNCN0=WTgP0200000080fMoK00H000000=SKfP00000000000=[Ke`000002
0=WGeP<000000P3Ge=H00`000000em7C0=OAd`020=OAd`050000003Ecm<0eLoC0=G?d`000000103D
c=801P000000dl_B0=?;dP000000dlWA00000080d/WA1@000000100AMN8000000000000000030464
j0030000001>Q>D0000000<0000000<0HXcT0000001[S>400P1[S>400`000000LhgM07>=g@0207>=
g@050000001kSMX0NhgJ07^=fP0000000`22S]L00`000000RI3D08V@e00208V@e0030000002?TM80
Si7B0080Si7B00<0000009BDd02DU=000P2DU=000`000000VIK@09VFd00209VFd0030000002LV<l0
W9S?0080W9S?100000020:6Ic@030000002TW<d0Y9c=00<0Y9c=00<000000:NOcP2WWlh00P2WWlh2
000000@0Zj7=00<000000:bTcP2/Y<h00`2/Y<h3000000<0/:K>00<000000;6Xd02aZ=000`2aZ=02
000000@0/j_@00<000000;B]d@2d[M400P2d[M400`000000]ZkA000000050;J_dP040000002g/M<0
000000000080^;;C00<000000;Rce02h/m@00P2h/m@3000000H0^[GE0P0000040;^geP040000002k
^=L00000000000<0_;WF00@000000;bje`2l^]L0_;[G0`0000060;fkf0030000002n_MT0_[gI00@0
_[gI00<000000;jnfP2n_]X00`2n_]X010000000_[oJ0000000000030;o0fP040000002o`=/00000
000000<0_l7K00@000000;o2f`2o`]/0_l;K0P0000000`30`]/000000<33g0020<33g080000000<0
`<?L00000030a=d00P30a=d2000000030<74g@000000`LGM0080`LGM0P0000000`31aMd000000<75
gP020<75gP80000000<0`LKN00000031a]h00P31a]h00`000000`/ON000000040<77gP0300000032
aml000000080`/OO0`0000000`32b=l000000<;8g`020<;8g`80000000<0`/SO00000032bMl00P32
bMl2000000030<;9g`000000`/WP0080`/WP00<000000<;9h00000000`32bN02000000030<;9h000
0000`/[P0080`/[P0P0000000`32b^0000000<;:h0020<;:h080000000<0`/[P00000032b^000P32
b^000`000000`/[P0000000Z0?ooo`030000003oool0oooo00X0oooo00L0okmd0000001d_ol0oooo
0?oo_`1d0000M;oo0080oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo02h0oooo000/0?oo
o`8000004@3oool400000080Yl?^0P0000030:O4kP<00000102XaNh200000080Z<K_0P0000000`2Y
a^h000000:W6k`040:W6k`800000102Zanl00`000000Z/S_0:[8k`0200000080ZlS_00<000000:_9
k`2[bNl00`2[bNl2000000@0[<[_00<000000:c;l02/bo000P0000020:c;l08000000P2/bo020000
00030:g<l0000000[Lc`00@0[Lc`0P0000050:g=l@@000000P2^c_4200000080[/oa0P0000000`2_
co400000000000020:k@lP80000000<0[m3a0000000000000`2_dO82000000030:oBlP0000000000
0080[m;c0P0000000`2`do800000000000030:oDl`<00000102_eO@400000080[mOd0P0000020;3H
m08000000P2_fOD2000000030;7Jm@00000000000080[m_f0P0000020;7LmP<000001@2`g_L20000
00<0/>7h0P0000020;;Sm`<000000`2bi_P200000080]^Of0`0000030;K[mP8000000P2kkO@00`00
0000`>gb0<3]lP0500000080b_3]0P0000020<o]j@050000003Bj^H0d^[V0=;ZiP0000000P3Ei^<2
00000080fNCN00D000000=WOf`3Igm/0fMoK000000040=SKfP030000003IemH00000008000000`3G
e=H01P000000e];E000000000000em7C000000<0eLoC00<000000=C=d`0000000`3Dc=801@000000
dl_B0=?;dP3Cbm80000000@0d/WA2`0000001P11Q>P0000004j4i@1>Q>D0000005V5hPD0000000<0
JhcQ0000001cSMd00P1cSMd01@000000NhgJ07^=fP1kSMX0000000@0PXkG00D0000008V@e029T=@0
RI3D0000000508nAdP050000002DU=00U9C@09BDd0000000102IU]000`000000W9S?09bHc`020000
000309fFc0000000XIW=00<0XIW=00<000000:BLc@2TW<d00`2TW<d00`000000Yio>0:NOcP020000
00030:RNc0000000Zj7=00@0Zj7=00<000000:bTcP2/Y<h00P2/Y<h00`000000/:K>0;2VcP040;2V
cP070000002aZ=00/JS@0;6Xd0000000/jW?000000050;>[d0040000002d[M40000000000080]ZkA
00<000000;J_dP2f[m80102f[m800`000000^;;C0;Rbd`030;Rbd`040000002h/m@00000000000<0
^[CC00@000000;Zee@2j]MD0^[GE0`0000060;^geP030000002l^MH0_;WF00<0_;WF00@000000;bj
e`00000000000`2m^mL010000000_K_H0;fkf02m^mP2000000030;jlf0000000_[gI0080_[gI0`00
00060;jnfP030000002o`=X0_l3J00@0_l3J00<000000;o1f`2o`M/00`2o`M/010000000_l;K0000
000000030<32f`0400000030`m`00000000000<0`<?L00@000000<34g@00000000000`31a=d01000
0000`LGM0000000000030<75g@0400000031aMh00000000000<0`LKN00@000000<76gP0000000000
0`32amh010000000`LON0000000000020<;7g`0400000032aml00000000000<0`/SO00@000000<;8
g`00000000000`32b=l010000000`/WO0000000000030<;9g`0400000032bN00000000000080`/WP
00@000000<;9h000000000000`32bN0010000000`/[P0000000000030<;:h00400000032b^000000
000000<0`/[P00@000000<;:h000000000000`32b^000`000000oooo0?ooo`0V0?ooo`8000003@3o
ool01@3o_g@00000000000000000B9cO00@0oooo00D0oono07@000000000001d0;ooo`0_0?ooo`00
;03oool00`000000oooo0?ooo`0>0?ooo`8000000P2W`nd00`000000Yl?^0:O3kP030:O3kP<00000
0P2Xa>h2000000<0Z<G^100000040:W6kP030000002Ya^l0ZLK_0080000000<0Z/O^000000000000
0`2Zanl3000000D0ZlS_00<000000:_9k`2[bNl00P0000000`2/bNl00000000000030:c:k`<00000
1@2/bo03000000@0[Lc`00<000000:g<l02]c?000P0000000`2^cO000000000000020:g=l@800000
0P2^c_02000000<0[/ka0`0000050:o?l@8000001@2_d?43000000D0[m;b0P0000050;3ClP<00000
0`2`e?<4000000<0/=Kd0`0000050;3Hm0<000001@2af_D2000000<0/Mcf0P0000000`2bgOH00000
000000020;3Nm`8000000P2bgoH3000000<0/^?g0P0000020;GTmP<000000`2fioH200000080^NWe
0`0000020;_]m08000000P30kO82000000040<G]k`00000000000<W/k@<000000P3?kNT01@000000
d^[V0=;ZiP3Bj^H0000000<0eNKS00<000000=KRh00000000P0000020=WOf`030000003HfmX0f=_J
0080f=_J0`0000000`3IemH000000=ODeP020=ODeP030000003Fd]D0e];E0080e];E00D000000=G?
d`3Ecm<0eLoC000000030=C=d`030000003Dc=80000000<0dl_B00H000000=;:dP00000000000=;9
d@0000020=78d@<0000000H006CG0000000000008WOT0000000aNN@300000080CXCU00H0000005V5
hP1IQN80FHGR0000001TQMh400000080LhgM00D0000007^=fP1kSMX0NhgJ0000000308:>e`030000
0029T=@0RI3D0080RI3D00<0000008nAdP2?TM800P2?TM800`000000U9C@09BDd00209BDd0030000
002IU]00VIK@00@00000102MU/`00`000000XIW=0:6Ic@020:6Ic@030000002TW<d0Y9c=0080Y9c=
0`0000030:RNc0030000002[XLd0Zj7=00<0Zj7=00<000000:bTcP2/Y<h00P0000000`2^Y<d00000
0;2VcP040;2VcP040000002aZ=00000000000080/jW?00<000000;>[d02cZm00102cZm000`000000
]ZkA0;J^d@030;J^d@040000002f[m80]ZoB0;J_dP<000001P2h/]<00`000000^[CC0;Zdd`030;Zd
d`040000002j]MD00000000000<0^kKD00@000000;^geP2k]mH0^kOF0`0000060;bieP030000002m
^mL0_K_G00<0_K_G00@000000;fkf000000000000`2n_=P010000000_[gI0000000000030;nnf@04
0000002n_]X0_[kJ0;jnfP80000000<0_koJ0000002o`=X00P2o`=X3000000H0_l7K00<000000<32
f`30`]/01030`]/00`000000`<?L0<33g0040<33g00300000031a=d0`LCM00@0`LCM00<000000<75
g@31aMd01031aMd00`000000`LKN0<76gP040<76gP0300000032amh0`/ON00@0`/ON00<000000<;7
g`32aml00`32aml010000000`/SO0<;8g`0000030<;8g`0300000032b=l0`/SO00@0`/SO00<00000
0<;9g`32bMl01032bMl00`000000`/WP0<;9h0030<;9h00300000032bN00`/WP00@0`/WP00<00000
0<;:h032b^001032b^000`000000`/[P0<;:h0040<;:h00300000032b^00`/[P0080`/[P00<00000
0<;:h0000000903oool300000080oooo00<000000?ooo`3oool0A`3oool002`0oooo00<000000?oo
o`3oool02`3oool3000000D0Yl?]00<000000:O3kP2W`nh00P0000000`2Xa>d000000:S4kP040:S4
kP<000000`2YaNh2000000<0ZLK^0`0000050:[7kP<000000P2[b>h2000000@0ZlS_0`0000050:c9
k`<000000P2/b^l200000080[<_`0P0000020:g;k`8000000`2]c?03000000D0[/g`0P0000050:k>
l0@000000P2_co4200000080[loa0P0000000`2_d?400000000000020:o@l@8000000P2`dO420000
0080[m;b0P0000000`2`d_800000000000020;3ClP80000000<0/M?b0000000000000P2`e?<20000
00030;7El`00000000000080/=Kd0P0000000`2aeo<00000000000020;3Hm08000000P2bfO@20000
0080/M[e0P0000020;;Km@<000001@2bgOH2000000<0/]of0P0000020;CQmP<00000102ei?H00`00
0000^>Ge0;SUm@03000000<0^NWe0P0000020;gYl`8000000P30kO82000000030<G]k`0000000000
00<0bNc]00<000000<gZj@3=j^T00`0000020=;ZiP030000003Ei^<0eNKS008000000P3Fh^000`00
0000e]kM0=KNg@0200000080f=_J0`0000030=OHf08000000P3Ge=H00`000000e];E0=KBe@020=KB
e@050000003Dd=@000000=G?d`0000001@3DcM<01@000000dl_B0=?;dP3Cbm80000000@0d/[B00<0
00000=78d@3Ab=400P3Ab=430000008006CG0`000002035ii0@000000`1IQN801P000000I8GN06B5
gP1TQMh0000006j6fPD0000000<0NhgJ00000022S]L00`22S]L01@000000RI3D08V@e029T=@00000
00<0Si7B2@00000309VCc0030000002MU/`0WIK<0080WIK<00<000000:6Ic@2QVLd00P2QVLd00`00
0000Y9c=0:BLc@02000000030:FKb`000000Z9k<00@0Z9k<00<000000:^Qc@2[XLd00P2[XLd30000
00<0[ZC=00<000000;2VcP2`Y/h0102`Y/h00`000000/jW?0;>Yc`030;>Yc`040000002cZm00/j_@
0;>[d0<000001@2f[]4010000000]ZoB0000000000030;R`d@040000002h/]<0^;;C0;Rbd`<00000
1P2j]=<00`000000^kKD0;^fe0030;^fe0040000002k]mH00000000000<0_;SF00@000000;bieP2l
^MH0_;WF0`0000060;fke`030000002n_=P0_[cH00@0_[cH00<000000;nnf@2o_]T00`2o_]T01000
0000_[kJ0000000000030;nofP040000002o`=X00000000000<0`<7J00@000000;o1f`2o`M/0_l7K
0P0000000`30`]/000000<32f`020<32f`80000000<0`L?L00000030`m`00P30`m`2000000030<74
g0000000`LCM0080`LCM0P0000000`31aMd000000<75g@020<75g@80000000<0`/KM00000031a]h0
0P31a]h2000000030<;6gP000000`/ON0080`/ON0P0000000`32amh000000<;7g`020<;7g`030000
0032b=l000000080`/SO00D000000<;8g`000000`/SO000000030<;8g`80000000<0`/WO00000032
bMl00P32bMl2000000030<;9g`000000`/WP0080`/WP00<000000<;9h00000001032bN000`000000
`/[P000000030<;:h080000000<0`/[P00000032b^000P32b^02000000030<;:h0000000`/[P0080
00000P32b^000`000000oooo0?ooo`0X0?ooo`030000003oool0oooo04H0oooo000/0?ooo`030000
003oool0oooo00P0oooo0`0000020:O2k@800000102W`nd3000000@0Z<C]00<000000:S4kP2Xa>h0
0P0000000`2Ya>d000000:W5kP050:W5kP<000000P2Za^h200000080Z/O^0P0000000`2[anh00000
0:_8kP040:_8kP@000000P2/b>l200000080[<W_0P0000000`2]bNl000000:c:k`040:c:k`800000
1P2]bnl300000080[/c_0P0000020:k=l080000000<0[lg`0000000000000P2^c_0200000080[lk`
00<000000:o?l@2_co400`2_co42000000D0[m3a0P0000050;3Al@<00000102`d_83000000D0/M?b
0P0000050;7El`8000001@2aeo<2000000D0/]Wd0`0000020;;Km@8000000P2dg?@200000080/]gf
0P0000020;CNm@<00000102dhOH01@000000]n;e000000000000]NCf008000000`2hiOD00`000000
^nKd0;_Vm00300000080_NWc0P0000020<7Zl@H000000P39k>d2000000<0cN[Y00<000000=3Wi`3@
inL00P0000020=GVh`050000003Fh^00e^;P0=KRh00000000`3Fg]d5000000@0emSH00<000000=KE
e`3FeML00P0000030=KBe@8000000`3Dd=@00`000000e<gC0=C=d`030=C=d`050000003Bc=<00000
0=?;dP000000103Bb]800`000000dLWB000000020=78d@030000003@am80d<OB00@0000000<036WL
0000000000000P0aNN@00`000000@7_R0000000200000080FHGR00D0000006B5gP1TQMh0I8GN0000
000306j6fP030000001gR=L0000000d000000P2>S/h00`000000U93=09B@c@0209B@c@030000002I
Tl`0VI?<0080VI?<00<0000009fFc02MU/`00P2MU/`00`000000XIW=0:6Ic@020:6Ic@<00000102U
Vl/00`000000Z9k<0:RNc0020:RNc0030000002[XLd0Zj7=0080000000<0[:7;0000002^Y<d0102^
Y<d010000000/:K>0;2VcP2`Y/h3000000D0/jW?00@000000;>[d000000000000`2e[<l00`000000
]ZkA0;J^d@030;J^d@030000002h/=40^;3A00<0^;3A00@000000;Rbd`00000000000`2j/]801000
0000^[CC0;Zdd`2j]=<3000000H0^kKD00<000000;bheP2l^=H00`2l^=H010000000_;WF00000000
00030;fjeP040000002m^mL0_K_G0;fke`<000001@2n_=P00`000000_kgH000000050;nnf@030000
002o_mX0_koJ00@0_koJ00<000000<31fP30`MX00`30`MX010000000_l7K0000000000030<32f`04
00000030`]/00000000000<0`L?L00@000000<33g000000000000`31a=`010000000`LCM00000000
00030<75g@0400000031aMd00000000000<0`/KM00@000000<76gP00000000000`32a]h010000000
`/ON0000000000030<;7gP0400000032aml00000000000<0`/SO00@000000<;8g`00000000000P32
b=l010000000`/SO0000000000030<;9g`0400000032bMl00000000000<0`/WO00@000000<;9h000
000000000`32bN0010000000`/WP0000000000020<;:h00400000032b^000000000000<0`/[P00@0
00000<;:h000000000001032b^000`000000`/[P0<;:h0030<;:h0030000003oool0oooo02L0oooo
00<000000?ooo`3oool0AP3oool002`0oooo00<000000?ooo`3oool01P3oool2000000030:O1k000
0000Yl;]00@0Yl;]100000020:S3k@030000002Xa>d0Z<C]00<0Z<C]0P0000040:W4k@8000000P2Y
aNh2000000030:[5k@000000Z/K^00@0Z/K^0P0000040:_7kP030000002[b>h0ZlS^008000000P2/
b>h00`000000[<S_0:c8k`030:c8k`800000102]bNl00`000000[<[_0:c:k`02000000030:g:k`00
0000000000<0[L__0P0000000`2^bnl000000:k<k`040:k<k`8000001@2_cO02000000D0[lk`00<0
00000:o?l@2_co400P0000000`2`co000000000000020:o@l@80000000<0/=3a0000000000000P2`
dO4200000080/M7a00<000000;3BlP2`d_800P0000020;7Cl@8000000P2ado82000000030;;DlP00
000000000080/MGc0P0000000`2be_800000000000020;7Gl`80000000<0/mSc0000000000000P2b
fO@200000080/mWd0`0000050;CLm0<000000`2dg_D2000000040;KOm000000000000;CQmP800000
0`2gh_D300000080^>Ge0P0000020;_Vm080000000@0_^Kb00000000000000000`31j_4200000080
aNW_00<000000<WXk039j>`00`0000020<gZj@8000000P3@inL2000000040=;Ti000000000000000
0080e^;P00<000000=KNg@3Fg]d00`0000020=KKg0030000003Gf=P0emSH0080emSH00T000000=KE
e`3FeML0e]GG0000003EdmL000000=KBe@000000103Dd=@00`000000dlkD000000030=C=d`050000
003Bc=<0d/cC0=;<d`000000103Bb]801P000000dLWB0=79dP000000dLSA000000@0d<OB10000000
0`0<JM`0000000000002000000<0@7_R1000000306B5gP050000001^Q]X0KXKJ06j6fP0000000`1g
R=L00`000000OhWC07n9d`0207n9d`0300000027Rm00Qh_@0080Qh_@00<0000008j>cP2>S/h00P2>
S/h01@000000U93=09B@c@2DT<d0000000D0VI?<00<0000009fFc02MU/`00P2MU/`01P000000XIW=
000000000000XYS:000000D0YI_;00<000000:RNc02XW/`00`2XW/`2000000@0[:7;00<000000:jT
c@2^Y<d00P2^Y<d010000000/:K>0000000000030;6Vc@030000002cZLl0/jW?00<0/jW?00<00000
0;F/c`2e[<l00`2e[<l010000000]ZkA0;J^d@2f[]43000000H0^;3A00<000000;ZbdP2j/]800`2j
/]8010000000^[CC0000000000030;^ed`040000002k]]@0^kKD0;^fe080000000<0_;OD0000002l
^=H0102l^=H00`000000_K[F0;fjeP030;fjeP040000002m^mL00000000000<0_[_G00<000000;jl
f02n_=P00P0000020;nmf0030000002o_]T0_kkI0080_kkI00<000000;nnf@000000102o_mX00`00
0000`<3J000000050<31fP0300000030`]/0`<;K00@0`<;K00<000000<73g031`m`01031`m`00`00
0000`LCL0<74g0040<74g00300000031aMd0`LGM00@0`LGM00<000000<;6g@32a]d01032a]d00`00
0000`/KN0<;6gP040<;6gP0300000032amh0`/ON00@0`/ON00<000000<;8g`32b=l01032b=l00`00
0000`/SO000000030<;8g`0300000032bMl0`/WO00@0`/WO00<000000<;9g`32bMl01032bMl00`00
0000`/WP0<;9h0040<;9h00300000032b^00`/[P00<0`/[P00<000000<;:h032b^001032b^000`00
0000`/[P0<;:h0020<;:h0<000000`32b^02000000030<;:h0000000oooo02P0oooo00<000000?oo
o`3oool0A@3oool002/0oooo00<000000?ooo`3oool0103oool3000000@0Yl7/0P0000020:O2k@80
000000<0Z<;/0000002X`nd00`2X`nd00`000000Z<C]0:S4k@02000000030:W4k@000000ZLC]00@0
ZLC]0`0000040:[5k@030000002Za^h0Z/K^0080000000<0ZlK]0000002[anh0102[anh2000000D0
[<S^00<000000:c8k`2/b>l00P0000000`2]bNh00000000000030:g9k`<000001@2]b^l3000000@0
[/__00<000000:k<k`2^c>l00P0000000`2_c>l00000000000020:o=l080000000<0[lg_00000000
00000`2_c_04000000D0/<o`0P0000050;3@l@8000001@2adO42000000D0/M?a0`0000050;;DlP80
00001@2be_82000000D0/mSc0P0000030;?Im080000000<0]M[c0000000000000P2dg?@2000000<0
]]gd0`0000040;KOm08000000`2gh_D2000000<0^^?d0P0000020;_Vm08000000`2ni_800`000000
`^O`0<;Wl00300000080aNW_0P0000030<WXk0030000003<inX0c>OZ008000000P3@inL2000000<0
d^CT00<000000=?Qh@3ChN401P0000030=KKg0060000003Ef=X000000000003Gf=P000000`3FeML0
1@000000eM?G0=GCe`3EdmL0000000@0e=3D00L000000=?>e03Cc]@0dlkD0000003DcM<0000000<0
d/cC00H000000=;;d`00000000000=;:dP0000040=79dP030000003@am80d<OB0080d<OB00<00000
0=37dP3@am800`0000000`0<JM`0000001m]gP0300000004041khP000000CWcO04ilg`<000000P1T
QMh01@000000KXKJ06j6fP1^Q]X0000000<0MhSG00D0000007n9d`1oRM<0OhWC0000000408N;d003
0000002>S/h0SXk>0080SXk>00<0000009B@c@2DT<d00P2DT<d00`000000VI?<09VCc00209VCc004
0000002MU/`0WIK<09fFc0<00000102RV<X00`000000YI_;0:FKb`020:FKb`040000002XW/`0Z9k<
0:RNc0<000001@2/XL/00`000000[ZC=0:jTc@020:jTc@8000001@2aY/d010000000/jW?0;>Yc`2c
ZLl3000000H0]Jc?0`0000030;R^d0040000002h/=40^;3A0;R`d@<000001P2j/]800`000000^kGC
0;^ed`030;^ed`040000002k]]@00000000000<0_;OD00<000000;bheP2l^=H00P2l^=H00`000000
_KWF000000050;fjeP030000002n^mL0_[_G00<0_[_G00<000000;jlf00000001@2o_MP010000000
_kkI0000000000030;nnf@040000002o_mX0000000000080`<3J00<000000<31fP30`MX00P30`MX0
0`000000`<7J000000040<32f`0300000031`]/0000000@0`L?L00<000000<73g00000000`31a=`2
000000030<;4g0000000`LGM0080`LGM0P0000000`32aMd000000<;6g@020<;6g@80000000<0`/KM
00000032a]h00P32a]h2000000030<;7gP000000`/ON0080`/ON0P0000000`32b=h000000<;8g`02
0<;8g`80000000P0`/SO00000032b=l000000<;8g`000000`/WO000000<0`/WO0P0000000`32bMl0
00000<;9g`020<;9g`80000000<0`/WP00000032bN000P32bN02000000030<;:h0000000`/[P0080
`/[P00<000000<;:h00000000`32b^02000000030<;:h0000000`/[P008000000`32b^03000000<0
`/[P00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`140?ooo`00:`3oool010000000oooo
0?ooo`3oool300000080Yl7/00<000000:O1k02W`N`00`2W`N`3000000@0Z<;/00@000000:S3k@2X
`nd0Z<?]0`0000040:W4k@030000002Ya>d0ZLC]008000000P2ZaNd2000000<0Z/G]0`0000040:_6
k@030000002[anh0ZlO^0080000000<0[<O^000000000000102/b>h3000000D0[LW^0`0000020:g:
kP8000000P2]b^l200000080[/_^0P0000030:k;k`<000001@2_c>l2000000D0[lg_0`0000030;3>
l08000000P2`co02000000030;7?l000000000000080/=3a0P0000000`2adO000000000000020;7A
l@80000000<0/];a0000000000000P2ado4200000080/]?a0P0000020;;DlP80000000<0/mGb0000
000000000P2be_82000000030;CGlP00000000000080/mSc0P0000020;CHl`<000001@2ef_<20000
00<0]]gd0P0000020;OMl`030000002fgo@0]]od008000000P2hh?<3000000<0^^?d0P0000020;cT
lP<000000P2ni_82000000<0`^O`00<000000<GVkP35i^h00`0000020<WXk08000000P3<inX20000
00040<kTi`000000000000000080d^CT0P0000020=?Qh@<0000000<0e=kO0000000000000`3Ffm`0
0`000000eMSJ0=GHfP020=GHfP8000000P3FeML01P000000eM?G0=GCe`3EdmL000000=C@eP800000
00<0e=3D0000003Cc]@0103Cc]@01@000000d/cC0=;<d`3Bc=<0000000@0d/_C00<000000=79dP3A
bM800P3AbM801P000000d<SB000000000000d<OB000000@0d<OB1000000301m]gP@000000P1>O=l0
0`000000FgkK000000040000000306j6fP000000MhSG0080MhSG00<0000007n9d`1oRM<00P1oRM<0
0`000000Qh_@08N;d00208N;d0050000002>S/h0SXk>08j>cP000000102DT<d010000000VI?<09VC
c02ITl`50000000309jDb@000000XYS:00@0XYS:00<000000:FKb`2UVl/00`2UVl/3000000<0ZYk:
00<000000:bQb`2/XL/00`2/XL/01P000000[ZC=000000000000/:C<000000H0/JK=0`0000030;BY
cP030000002e[<l0]Jc?00<0]Jc?00<000000;R^d02h[]000`2h[]0010000000^;3A000000000003
0;Vad@030000002j/]80^[;B0080^[;B00<000000;^ddP0000001@2k]M<00`000000_;OD0;bge004
0;bge0040000002l^=H0000000000080_KWF00<000000;fjeP2m^]H00P2m^]H00`000000_[_F0000
00050;jke`030000002o_MP0_kgH00D0_kgH00<000000;nnf@2o_]T0102o_]T00`000000`<3J0<30
fP030<30fP0400000030`MX00000000000<0`<7J00@000000<32f`00000000000`31`]/010000000
`L?L0000000000020<73g00400000031a=`00000000000<0`/CL00@000000<75g@00000000000`32
aMd010000000`/KM0000000000030<;6g@0400000032a]h00000000000<0`/ON00@000000<;7gP00
000000000`32b=h010000000`/SO0000000000030<;8g`0400000032b=l0000000000080`/WO00@0
00000<;9g`00000000000`32bMl010000000`/WO0000000000030<;9h00400000032bN0000000000
00<0`/[P00@000000<;:h000000000000P32b^0010000000`/[P0000000000040<;:h00300000032
b^00`/[P00<0`/[P00<000000<;:h032b^001032b^000`000000oooo0?ooo`0W0?ooo`030000003o
ool0oooo04@0oooo000[0?ooo`060000003oool000000000002W`>/00000102W`N`00`000000Yl7/
0:O1k00200000080Z<;/00<000000:S2k02X`^`00`2X`^`300000080ZL?/0P0000030:W4k@<00000
1P2ZaNd300000080ZlK]0P0000030:_6k@<000001@2/anh400000080[LS^0P0000020:g9kP800000
00<0[LW^0000002]b^h0102]b^h2000000D0[/_^100000020:o<k`8000000P2_c>l2000000030;3=
k`00000000000080[lg_0P0000000`2`c^l000000;3>l0040;3>l0<000001@2aco02000000D0/M7`
0P0000050;;Bl@8000001@2bdo43000000D0/mGb0P0000050;CGlP8000000`2df?<2000000030;KI
lP00000000000080]M[c0P0000020;KKl`<000001@2ggO<2000000<0^>3c0P0000020;_QlP<00000
0`2li?8200000080_nCa0`0000020<;Wl08000000`35i^h00`000000b>G/0<SUk00200000080c>OZ
0P0000030<kTi`030000003@h^@0d>;T008000000P3ChN43000000<0e=kO00D000000=CKg@000000
e]_L000000040=GHfP030000003De]T0e=KI008000000`3EdmL00`000000e=3F0=C@eP020=C@eP03
0000003Cc]@0dlkD0080dlkD0P000000103BcM@000000=;<d`0000040=;;d`030000003Ab]<00000
0080dLWB00<000000=38dP3@b=800P3@b=800`000000d<OB0=37dP020=37dP040000003?am800000
000l^@<0000000@07fgN0000000aL=d0<G3M0P00000204ilg`060000001KO]/0FgkK05]nf`000000
IgoG1000000207N8e`050000001oRM<0OhWC07n9d`0000001027Rm000`000000SXk>08j>cP0208j>
cP030000002DT<d0U93=00D000000P2JT<T00`000000WYC909jDb@0209jDb@030000002RV<X0XYS:
0080XYS:00<000000:FKb`2UVl/00P2UVl/2000000D0ZYk:00<000000:bQb`2/XL/00`2/XL/20000
00@0/:C<00<000000;6Vc@2aY/d00`2aY/d00`000000]:W>0;BYcP030;BYcP040000002e[<l0]Jc?
0;F/c`80000000<0]jg?0000002h[]00102h[]000`000000^K7A0;Vad@030;Vad@030000002j/]80
^[;B008000000P2k]=800`000000^kGC0;^ed`020;^ed`030000002l]]<0000000H0_;OD00<00000
0;fieP2m^MH00`2m^MH010000000_K[F0000000000020;jkeP030000002n^mL0_[_G0080_[_G00<0
00000;nle`000000102o_MP3000000D0_kkI0P0000060<30fP0300000030`MX0`<7J00@0`<7J00<0
00000<72f`31`]/01031`]/00`000000`L?L0<73g0030<73g00300000032a=`0`/CL00@0`/CL00<0
00000<;5g@32aMd01032aMd00`000000`/KM0<;6g@040<;6g@0300000032amh0`/ON00@0`/ON00<0
00000<;8gP32b=h01032b=h00`000000`/SO0<;8g`040<;8g`0300000032bMl0000000<0`/WO00<0
00000<;9g`32bMl01032bMl00`000000`/WP0<;9h0040<;9h00300000032b^00`/[P00@0`/[P00<0
00000<;:h032b^000`32b^000`000000`/[P0<;:h0020<;:h080000000<0`/[P00000032b^000P32
b^000`000000`/[P000000030<;:h080000000<0`/[Q0000003oool0:03oool00`000000oooo0?oo
o`130?ooo`00:`3oool2000000@0Yl3[00@000000:O1k02W`N`0Yl7/0`0000050:S2k0030000002X
`^`0Z<;/0080000000<0ZL;/0000002Y`n`0102Y`n`300000080Z/C/0P0000030:[5k@80000000<0
ZlG]0000002[a^d0102[a^d300000080[<K]0P0000020:c7kP8000000P2]and00`000000[LS^0:g8
kP030:g8kP800000102]bNh00`000000[L[^0:g:kP02000000030:k:kP00000000000080[/_^0P00
00020:o;kP030000002_c>l0[lc_00<0[lc_0P0000050;3=k`800000102`c^l00`000000/<k`0;3>
l00200000080/Lo_0P0000020;7?l080000000<0/]3`0000000000000P2adO02000000030;;Al000
000000000080/];a0P0000000`2cd_000000000000020;;Cl@8000000P2ce?4200000080/mGb0P00
00000`2deO400000000000020;CGlP8000000P2eeo83000000D0]]Wb0P0000040;KKl`040000002h
g?80000000000080]mgc0P0000020;WNlP<000000`2khO8200000080_N7a0`0000040;oTl@030000
0032i>l0`^C_00<000000P35i^h200000080b>G/0P0000000`3;i>T00000000000040<kTi`040000
003@h^@0d>;T0=3Ri0@0000000@0dn7Q0000003Dg]l0e=kO0P0000030=CKg@8000000P3Ef=X20000
00<0e=KI00D000000=CCf0000000eM?G000000040=C@eP040000003CcmH000000=?>e0800000103B
cM@010000000d/_C0=;;d`3Bbm<200000080dL[C00<000000=79dP000000103@b=800`000000d<SB
000000020=37dP060000003?am80clOB0<o7dP00000003bi1@000002035`g@@000000`1KO]/01P00
0000IgoG06Moe`1WOmL0000007:1da<000000P2DSLT00`000000VY3909Z@b@0309Z@b@050000002N
U<T0WYC909jDb@0000001@2RV<X010000000YI_;0000000000020:NJb0030000002ZW/X0ZYk:0080
ZYk:00@000000:bQb`2/XL/0[:7;0P0000000`2_XLX000000;2Tc0030;2Tc0040000002aY/d0/JK=
0;6Vc@80000000<0/jO<0000002dZLh0102dZLh3000000<0]jg?00<000000;R^d02h[]000P2h[]00
0`000000^K3@000000050;Vad@8000001@2k]=8010000000^kGC0;^ed`0000020;bfd`030000002l
]m@0_;OD0080_;OD0`0000060;fieP030000002n^mH0_[_F00<0_[_F00@000000;jke`0000000000
0`2o_=L010000000_kgH0000000000030<2nf0030000002o_]T0_kkI008000000P30_mT010000000
`<3J0<30fP30`=X2000000030<71fP000000`<7J0080`<7J0P0000000`31`]X000000<72f`020<72
f`80000000<0`L?K00000031`m`00P31`m`00`000000`/CL000000040<;4g00300000032aM`00000
00@0`/GM00<000000<;6g@0000000`32a]d2000000030<;7gP000000`/ON0080`/ON0P0000000`32
amh000000<;8gP020<;8gP80000000<0`/SN00000032b=l00P32b=l2000000080<;8g`000000`/WO
00000032bMl000000<?9g`0000030<;9g`80000000<0`lWO00000032bN000P32bN02000000030<?:
h0000000`/[P0080`/[P0P0000001033b^0000000<;:h032b^02000000030<;:h0000000`/[P0080
00000`32b^0010000000`/[P0000000000020<;:h00400000032b^000000000000<0`/[Q00<00000
0?ooo`3oool09`3oool00`000000oooo0?ooo`130?ooo`00:`3oool2000000D0Yl3[0`0000020:S1
j`800000102X`^`3000000@0ZL;/00<000000:W3k02Y`n`00P0000000`2Z`n`000000:[4k0040:[4
k0<00000102[aNd00`000000ZlK]0:_6k@02000000030:c6k@000000[<K]00@0[<K]0P0000050:g7
k@030000002]b>h0[LS^0080000000<0[/S]0000002]bNh0102]bNh2000000D0[/[^0P0000050:o;
kP030000002_c>l0[lc_0080000000<0/<c^0000000000000P2`cNl2000000030;3=k`000000/<k_
00@0/<k_0P0000050;7?k`<000001@2bd?02000000D0/]7`0P0000050;?Bl08000001@2ce?430000
00D0]=Ga0P0000040;GGlP040000002ff?40000000000080]]Wb0P000000102gf_8000000000002f
fo<2000000@0^=cb0P0000030;WNlP8000000P2kgo43000000@0_N7a00D000000;oQl00000000000
0;oTl@02000000<0`^C_00<000000<GSk@35hnd00P0000020<SUk08000000`3;i>T01P000000cN;W
000000000000c^CW00000080d>;T0`0000030=7OhP040000003Dg]l0e=kO000000@0e=_M00<00000
0=CHg03Df=`00P0000030=CFf@8000000`3DdmP00`000000e=3F0=C@eP020=C@eP050000003CcmH0
dloF0=??eP0000001@3BcM@01@000000dL_D0000003Bbm<0000000D0dL[C00@000000=38dP3@b=80
d<SB0P0000020=38dP030000003@am80000000@0clOB1@0000000`0:G]H0000000000002035`g@03
00000011L]/00000008000000P1KO]/01@000000IgoG06Moe`1WOmL0000000<0LX7C00D0000007b4
d01lQ=00O8C@0000000408F6c@050000002=RLX0SHW:08f9bP0000001@2DSLT00`000000VY3909Z@
b@0209Z@b@030000002NU<T0WYC90080WYC900<000000::HbP2RV<X00P2RV<X2000000@0Yi[800<0
00000:ZNbP2ZW/X00`2ZW/X3000000<0[j7:00<000000;2Tc02`Y<`00`2`Y<`3000000<0/jO<00<0
00000;BYcP2dZLh00`2dZLh00`000000]jg?0;N]c`040;N]c`040000002h[]00000000000080^K3@
00<000000;Vad@2i/M400`2i/M400`000000^kCB0;^ddP040;^ddP030000002k]M<0000000@0_;KC
00@000000;bge000000000000`2m^=@010000000_KWF0;fieP2m^MH3000000H0_[_F00<000000;nl
e`2o_=L0102o_=L00`000000`;kH0<2nf0030<2nf0030000002o_]T0000000@0`;oI00@000000<30
fP00000000000`31`MX010000000`<7J0000000000030<72fP0400000031`]/00000000000<0`L?K
00@000000<73g000000000000`32a=`010000000`/CL0000000000030<;5g00400000032aMd00000
00000080`/KM00@000000<;6g@00000000000`32amh010000000`/ON0000000000030<;7gP040000
0032b=h00000000000<0`/SN00@000000<;8g`00000000000`32b=l010000000`/WO000000000002
0<?9g`0400000032bMl00000000000<0`lWO00@000000<;9h000000000000`33b^0010000000`/[P
0000000000040<?:h08000001032b^000`000000`/[P0<;:h0040<;:h00300000032b^00`/[P00<0
`/[P00<000000<;:h@32b^401032b^400`000000oooo0?ooo`0W0?ooo`030000003oool0oooo0480
oooo000I0?ooo`050?oOW018000000000000M02oool03@3oool00`000000oooo000000020:O0j`80
000000<0Z<3[0000002X`N/0102X`N/400000080ZL;[0P0000030:W2k0<00000102Z`n`00`000000
Z/C/0:[4k00200000080ZlC/00<000000:_5k@2[aNd00`2[aNd2000000@0[<K]00<000000:c6k@2/
a^d00P0000000`2]and000000:g7k@040:g7k@<00000102^b>d00`000000[LW^0:g9kP0200000003
0:k9kP00000000000080[/[^0P0000000`2_b^h000000:o;kP040:o;kP<000001@2`c>h2000000@0
/<g_00<000000;3>k`2`c^l00P0000000`2ac^l00000000000020;7?k`8000000P2bcnl200000080
/]3`0P0000000`2bd>l00000000000020;;Al080000000<0/m7`0000000000000P2cd_0200000003
0;CCl000000000000080/mCa0P0000020;GDl08000000P2deO42000000040;KFl@00000000000;GG
lP800000102ff?42000000D0]m[b0P0000030;SLlP8000000P2jg?43000000@0^moa00D000000;kO
l000000000000;gQl@02000000@0_n7`0P0000020<;Tk`8000000P35hnd2000000030<OSk0000000
000000@0bnCY00<000000<gRi`3=h^L00P3=h^L5000000<0dMoR0P0000000`3Bg>00000000000004
0=CKg@090000003Df=`0e=SL0=CHg0000000dmKJ0000003De]T0000000@0e=?H00H000000=?Ae`00
000000000=C@eP0000030=??eP030000003BcMD0000000<0d/gD00D000000=7;e03Abm@0dL_D0000
00050=7:d`050000003@bM<000000=38dP0000001@3@b=8010000000clOB0<o7dP3?am8200000080
clKB0P0000000`00AL@0000000YNeP0200000003035`g@000000@G;K0080@G;K1000000306Moe`05
0000001bPM<0LX7C07:1d`000000101lQ=000`000000QHK=08F6c@0208F6c@050000002=RLX0SHW:
08f9bP000000102DSLT00`000000VY3909Z@b@0209Z@b@030000002NU<T0WYC90080WYC900<00000
0::HbP2RV<X00P0000000`2TU/L000000:NJb0040:NJb0030000002ZW/X0ZYk:0080ZYk:00<00000
0:nQbP2_XLX0102_XLX00`000000/:C<0;2Tc0020;2Tc0030000002cYl`0/jO<00@0/jO<00L00000
0;BYcP2dZLh0]:W>0000002gZld0000000H0]jg?00<000000;V`d02i/=000`2i/=001`000000^K7A
0;Vad@2i/M4000000;^bd@0000001P2k]=800`000000_;KC0;bfd`040;bfd`030000002m^=@0_KSD
00<0_KSD00@000000;fieP00000000000`2n^]H010000000_[_F0;jkeP2n^mH2000000030;nle`00
0000_kcG0080_kcG0`0000060<2nf00300000030_mT0`;oI00@0`;oI00<000000<71fP31`MX01031
`MX00`000000`L;J0<72fP040<72fP0300000031`m/0`L?K00@0`L?K00<000000<;4g032a=`01032
a=`00`000000`/GL0<;5g0040<;5g00300000032a]d0`/KM00<0`/KM00<000000<;7gP32amh01032
amh00`000000`/ON0<;7gP040<;7gP0300000032b=h0`/SN00@0`/SN00<000000<;8g`32b=l01032
b=l010000000`lWO0<?9g`0000020<?9g`0300000033bMl0`lWO00@0`lWO00<000000<?:h033b^00
1033b^000`000000`l[P0<?:h0020<?:h080000000<0`l[P00000032b^000P32b^000`000000`/[P
000000030<;:h080000000<0`/[P00000032b^000P32b^000`000000`/[Q000000030<;:h@800000
00<0`/[Q0000003oool0:03oool00`000000oooo0?ooo`110?ooo`00603oool01`3ookl0M00007Bo
o`3oool0okmd0000M02oool02`3oool010000000oooo0?ooo`3oool2000000@0Z<3[00<000000:S1
j`2X`N/00P0000020:W1j`030000002Y`^/0ZL;[00<0ZL;[0`0000020:[3j`8000000`2Z`n`30000
00D0ZlC/00<000000:_5k@2[aNd00P0000000`2/aN`000000:c6k@040:c6k@800000102]and00`00
0000[LO]0:g7k@0200000080[/O]0P0000030:k8k@<000001@2^bNh2000000@0[l[^00<000000:o;
kP2_bnh00P0000020;3;kP8000000P2`c>h2000000030;7<kP000000000000<0/<g_0`0000050;7>
k`8000001@2bcnl3000000D0/]3_0P0000050;?Al08000001@2ddo02000000D0]MC`0`0000040;KF
l@<000000`2ff?42000000030;SHl@00000000000080]m[b0P0000020;WJl@<00000102jg?401@00
0000_=g`000000000000^moa00800000102ngo02000000<0_n7`0P0000020<;QkP8000000P35hnd2
000000<0an?/00H000000<WQj@00000000000<_Tj@0000030<gRi`<0000000<0c^3U000000000000
0P3Agn82000000<0d]cP00H000000=;JgP00000000000=CKg@0000030=CHg0050000003Ce]X0dmKJ
0=?FfP000000103DdmP00`000000dm7G0=?Ae`020=?Ae`050000003CcmH0dloF0=??eP0000000`3B
cMD00`000000d/gD000000030=7;e0050000003Ab]@000000000003Ab]<00P0000030=39d`030000
003@b=80d<SB00<0d<SB00D000000<o7dP000000clOB000000050<o6dP80000000<004G40000000:
G]H010000002045bf`030000001AMMP0DGGH00<000000P1WOmL01@000000LX7C07:1d`1bPM<00000
00<0O8C@00<0000008F6c@25Q/d00P25Q/d01@000000SHW:08f9bP2=RLX0000000@0U8g900<00000
09Z@b@2JT<T00`2JT<T00`000000WYC909jDb@04000000<0Y9K700<000000:NJb02WV/P00`2WV/P0
0`000000ZYk:0:ZNbP02000000030:bMb0000000[j7:00@0[j7:00<000000;2Tc02`Y<`00P000000
0`2bYLX000000;>Wc0040;>Wc0040000002dZLh0000000000080]j_=00<000000;N]c`2g[Ll00P2g
[Ll3000000D0^K3@00@000000;Vad@00000000000P2k/]400`000000^kCB0;^ddP020;^ddP<00000
1@2l]]<2000000H0_KSD00<000000;jjeP2n^]H00`2n^]H010000000_[_F0000000000030;nle`04
0000002o_=L00000000000<0`;gG00@000000<2nf030_]P0`;kH0P0000000`31_mP000000<2of@02
0<2of@80000000<0`L3I00000031`MX00P31`MX2000000030<72fP000000`L;J0080`L;J0P000000
0`32`m/000000<73f`020<73f`80000000<0`/CK00000032a=`00P32a=`3000000D0`/GL00<00000
0<;6g@0000000`32a]d00`000000`/KM000000040<;7gP0300000033amh0000000<0`/ON0P000000
0`33b=h000000<;8gP020<;8gP80000000<0`lSO00000032b=l00P32b=l2000000080<?9g`000000
`lWO00000033bMl000000<?9g`0000030<?9g`80000000<0`l[P00000033b^000P33b^0200000003
0<?:h0000000`l[P008000000`33b^0010000000`/[P0000000000020<;:h00400000032b^000000
000000<0`/[P00@000000<;:h000000000000P32b^4010000000`/[Q0000000000030<;:h@030000
003oool0oooo02L0oooo00<000000?ooo`3oool0@@3oool001P0oooo00<0_g@00018W03Oool00P3o
ool00`2oM00007Bo0?ooo`0:0?ooo`030000003oool0oooo0080oooo0P0000030:S0j`<000001@2Y
`N/00`000000ZL;[0:W2j`02000000030:[2j`000000Z/?[00@0Z/?[0`0000020:_3k08000000`2[
a>`4000000@0[<G/00<000000:c6k@2/a^d00P0000000`2]a^`00000000000030:g7k@<000001P2^
and300000080[/S]0P0000020:k9kP80000000<0[lW]0000000000000`2_b^h3000000H0/<_^0P00
00050;7<kP<000000P2acNh200000080/Lk_0P0000000`2bc^h00000000000020;;?k`8000000P2c
cnl200000080/]3_0P0000000`2cdNl00000000000020;?Al080000000<0]=;_0000000000000P2d
do02000000030;GCl000000000000080]MC`0P0000020;KEl0030000002fe_40]]Ka008000000`2g
eo03000000@0^=Sa0`0000030;WJl@80000000@0^m_`000000000000^]ca0P0000040;cMl0800000
0`2ngo0200000080`=o_0`0000020<;QkP8000000P34hNd300000080an?/0P0000040<WQj@H00000
0`3>h>D010000000d=kS0000000000040=;Lh0030000003Bf]h0d][N0080d][N0P0000020=CHg006
0000003Ce]X0dmKJ0=?FfP000000d]?I0P0000000`3DdmP000000=?Ae`030=?Ae`050000003BcmL0
00000=??eP0000001@3BcMD01@000000dL_D0=7;e03Abm@0000000@0dL[D00<000000=39d`3@bM<0
0P3@bM<01@000000clSC000000000000d<SB008000000`3?am800`000000clKB0<o6dP030<o6dP03
0000003>a]80000000@000000P0NHmL200000080@G;K00D00000055ef01AMMP0000005mhe0050000
000307:1d`000000O8C@0080O8C@00<0000008F6c@25Q/d00P25Q/d00`000000SHW:08f9bP0208f9
bP030000002DSLT0U8g90080U8g900<0000009Z@b@2JT<T01@0000020:2BaP030000002TU/L0Y9K7
00<0Y9K700<000000:NJb02WV/P00P2WV/P3000000<0[9g800<000000:nQbP2_XLX0102_XLX20000
00<0/ZG:00<000000;>Wc02cYl`0102cYl`00`000000]j_=0;N[c@030;N[c@040000002g[Ll00000
000000<0^Jk>00<000000;V`d02i/=000`2i/=000`000000^k;A0;^bd@030;^bd@040000002k]=80
0000000000<0_;GB00<000000;bfd`2l]]<00P0000020;jgd`040000002m^=@0_KSD0;fhe0<00000
1P2n^]H00`000000_kcG0;nle`040;nle`0300000030_ML0`;gG00<0`;gG00@000000<2nf0000000
00000`31_mP010000000`;oI0000000000030<70f@0400000031`MX00000000000<0`L;J00@00000
0<72fP00000000000`32`m/010000000`L?K0000000000030<;4f`0400000032a=`00000000000<0
`/GL00<000000<;5g032aM`00P0000020<;6g@0400000032a]d00000000000<0`/KM00@000000<;7
gP00000000000P33amh010000000`/ON0000000000030<?8gP0400000032b=h00000000000<0`lSO
00@000000<;8g`00000000000`33bMl010000000`lWO0000000000020<?9g`0400000033bMl00000
000000<0`l[P00@000000<?:h000000000001033b^000`000000`l[P0<?:h0040<?:h00300000032
b^00`/[P00<0`/[P00<000000<;:h032b^001032b^000`000000`/[Q0<;:h@030<;:h@0300000032
b^40`/[Q00@0`/[Q00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`100?ooo`005`3oool0
0`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00X0oooo00<000000?ooo`3oool0103oool3
00000080ZL3Z00<000000:W1j`2Y`N/0102Y`N/2000000@0Z/;[00<000000:[3j`2Z`n/00P000000
0`2[`n/000000:_3k0040:_3k0<000000`2/a>`2000000<0[<G/0`0000050:g6k0<000000P2^an`2
000000<0[/O]0P0000000`2^b>d000000:k8k@040:k8k@8000001@2_bNd300000080/<_]0P000003
0;3;kP80000000<0/L_]0000000000000P2ac>h2000000030;7<kP000000/Lg^00@0/Lg^0P000005
0;;>kP8000001@2ccnl3000000D0/m7_0P0000050;CBk`8000001@2edo02000000D0]]G`0P000003
0;OGl08000000P2heo000`000000^=Sa0;SHl@02000000<0^MW`0`0000040;_Kl0<000000`2lgO02
00000080_]g_0`0000030<3Ok`80000000@0`]o^00000000000000001034hNd00`000000an3[0<OP
j`02000000D0bN7Y1P0000020<kPi@8000000P3@g^<01P000000d=_Q000000000000d]cP000000@0
d][N00<000000=;Hg@3Bf=d00P0000030=?FfP030000003BdmT0d]?I0080d]?I00@000000=?Ae`3C
dML0dm7G0P0000030=;?e`030000003BcMD0d/gE00<0d/gE00D000000=7<e@000000dL_D00000004
0=7:e0030000003@bM@000000080d<WC00<000000<o8d`3?b=<00P3?b=<00`000000clOB0<o7dP02
0<o7dP030000003?a]80000000<0clKB00@000000<k6dP3>a]80c/KB0`0000001000CL`0000001iS
e`0NHmL4000000<0DGGH00@0000005mhe01ON=@000000P1]N/l400000080O8C@00<0000008F6c@25
Q/d00P25Q/d01@000000SHW:08f9bP2=RLX0000000<0U8g91000000209^>a@030000002PT/H0X9;6
00<0X9;600<000000:BFa`2TU/L00P2TU/L00`000000Yi[80:NJb002000000030:ZJaP000000[9g8
00@0[9g800@000000:nQbP2_XLX0[j7:0`0000050;:UbP040000002cYl`0/jO<0;>Wc0<000001P2g
Zld00`000000^Jk>0;V^cP030;V^cP040000002i/=00^K3@0;V`d0<000001P2k/]400`000000_;GB
0;bedP030;bedP030000002l]]<0000000@0_[OC00@000000;fhe000000000000`2o^M@010000000
_[[F0;jjeP2n^]H2000000030;nkeP000000_kcG0080_kcG0`0000060<2me`0300000031_mP0`KoH
00@0`KoH00<000000<70f@31`=T01031`=T00`000000`L;J0<72fP040<72fP0300000032`m/0`/?K
00@0`/?K00<000000<;4f`32a=/01032a=/00`000000`/GL0<;5g0030<;5g00300000032aM`00000
00D0`/KM00<000000<;6g@32a]d01032a]d00`000000`lON0<?7gP030<?7gP0300000033b=h0`lSN
00@0`lSN00<000000<?8g`33b=l01033b=l00`000000`lWO0<?9g`040<?9g`0400000033bMl0`lWO
00000080`lWO00<000000<?:h033b^001033b^000`000000`l[P0<?:h0020<?:h080000000<0`l[P
00000033b^000P33b^02000000030<?;h0000000`/[P0080`/[P00<000000<;;h@0000000`32b^02
000000030<;;h@000000`/[Q0080`/[Q00<000000<;:h@0000000`32b^42000000030<;:h@000000
oooo02L0oooo0P0000110?ooo`005`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo
00X0oooo00<000000?ooo`3oool01@3oool00`000000ZL3Z0:W0jP020:W0jP8000000P2Y`N/20000
00030:[1j`000000Z/;[00@0Z/;[0P0000040:_3j`030000002[`n`0Zl?/0080000000<0[<C[0000
002/a>`01@2/a>`300000080[LG/0P0000020:g6k080000000<0[/K/0000002^an`0102^an`30000
00@0[/S]00<000000:k8k@2^b>d00P0000000`2_bNd00000000000020:o9k@80000000<0/<[]0000
002`bnd0102`bnd3000000D0/L_]0P0000040;7<kP030000002acNh0/Lg^0080000000<0//g^0000
000000000P2bc^h2000000030;??kP00000000000080/lo_0P0000020;C@kP8000000P2cdNl20000
00030;CAk`00000000000080]=;_0P0000000`2fd^l00000000000020;GCl080000000<0]]C_0000
000000000P2feO0200000080]mG_0`0000050;SGl0800000102ifO02000000040;_Ik`000000^m_`
0;_Kl08000000`2lfnl3000000<0_]g_0P0000000`30gNh000000<3Ok`02000000<0`]o^00D00000
0<CNk000000000000<CQk@02000000<0an3[00<000000<SOj@000000100000030<cPi`030000003=
g^D0cMkU008000000`3@g^<00`000000d=_Q0=3Kh@020=3Kh@8000000P3Bf]h2000000<0d]SM00D0
00000=;Ef`000000dmKJ000000040=;Cf@050000003BdMP000000=?Ae`000000103BcmL01@000000
dLkF000000000000d/gE008000000`3Ac=D00`000000dL[D0=7:e0020=7:e0060000003@bM@0d<WD
0000003@bM<00000103?b=<00`000000clOC000000020<o7dP070000003?a]80clKB0<o6dP000000
clKB000000030<k6dP030000003>aM8000000080000000@004g<00000000000000000P0bImH20000
0080DGGH00D0000005mhe01ON=@0GgSD0000000306ejc`030000001iOL/0000000h0000000<0U8W5
0000002KS/D0102KS/D00`000000X9;60:2BaP020:2BaP030000002TU/L0Y9K700<0Y9K70P000004
0:ZJaP030000002/WLP0[9g80080[9g800@000000:nQbP00000000000`2aX/P00`000000/ZG:0;:U
bP020;:UbP040000002cYl`00000000000<0]ZS<00@000000;N[c@2gZld0]j_=0`0000060;V^cP<0
00000`2k/M0010000000^k;A0;^bd@2k/]43000000H0_;GB00<000000;jgd`2n]m<0102n]m<00`00
0000_kWD0;nie0030;nie0040000002n^]H00000000000<0_k_F00@000000;nle`00000000000`30
_ML010000000`;gG0<2me`30_ML2000000030<6nf0000000`KoH0080`KoH0P0000000`31`=P00000
0<70f@020<70f@<000001@31`]X2000000D0`/?K0P0000050<;4f`8000001P32aM`00`000000`/KM
0<;6g@030<;6g@8000001@32a]d00`000000`lOM000000030<?7gP0300000033b=h0000000@0`lSN
00<000000<?8gP0000000`33b=l2000000030<?9g`000000`lWO0080`lWO0P0000002033bMl00000
0<?9g`000000`lWO00000033b]l000001033b^0010000000`l[P00000033b^02000000<0`l[P00@0
00000<?:h000000000000`33bn0010000000`/[P0000000000020<;;h@0400000032b^0000000000
00<0`/_Q00@000000<;:h@00000000000P32b^4010000000`/[Q0000000000030<;:h@030000003o
ool0oooo02@0oooo0P0000000`3oool000000?ooo`100?ooo`005`3oool00`3ogi`0B00007Boo`03
0?ooo`030?noM00004P0W=oo00X0oooo00<000000?ooo`0000001P3oool2000000<0ZL3Z0`000004
0:[1j`030000002Z`^/0Z/;[0080000000<0Zl;[0000002[`n/0102[`n/2000000@0[<C[0P000002
0:c4k080000000<0[LC[0000002]aN`0102]aN`2000000@0[/K/00<000000:k7k02^an`00P000002
0:k7k0030000002^b>d0[/S]00<0[/S]0P0000050:o9k@800000102`b^d00`000000/<_]0;3;k@02
00000080/L_]0P0000020;7;k@80000000<0//c]0000002ac>h0102ac>h2000000D0//g^0P000005
0;??kP8000001@2dd>h3000000D0]=7_0P0000050;KBk`8000001@2fe>l2000000@0]mG_00@00000
0;SFk`00000000000P2heo02000000040;[Gk`00000000000;WIl0800000102kfNl2000000@0_=__
0P0000000`2nfnh000000;kMk`02000000@0`=g^0P0000020<;OkP8000001034g^`200000080an3[
0P0000020<SOj@@000000P3<h>L200000080cMkU0P000000103>g><000000=3Nh`0000040=3Kh@03
0000003@fMl0d=WO008000000`3Bf=d2000000<0d]GK00<000000=;Cf@3BdmT00P3BdmT01@000000
d]7H0=;Af03BdMP0000000@0d/oG00<000000=7>eP3Ac]H00P3Ac]H00`000000dLcE0=7<e@020=7<
e@060000003@bm@000000000003Ab]@00000103@bM@00`000000clSC0<o8d`020<o8d`060000003?
am<0clOC0000003?am8000001@3?a]801@000000c/KB0<k6dP3>a]80000000<0c/GB100000000`09
Dll0000000000002039WeP<0000000D0DGGH0000001ON=@0GgSD0000000406ejc`040000001iOL/0
NGg;000000@0Ph7800<0000008f5aP2=QLH00P2=QLH00`000000U8W509B9a@0209B9a@030000002K
S/D0Vhk50080Vhk500<000000:2BaP2PT/H00P2PT/H010000000Y9K70:BFa`2TU/L3000000D0ZY[6
00<000000:bMb02/WLP00`2/WLP00`000000/J;80;6Rb0030;6Rb0030000002bYLX0/ZG:00<0/ZG:
00<000000;JXc02fZ<`00`2fZ<`010000000]j_=0000000000030;V/c@030000002i[/h0^Jk>00<0
^Jk>00<000000;^ad02k/M000`2k/M0010000000^k;A0000000000030;bdd@040000002l]M80_;GB
0;bedP<000001@2n]m<00`000000_kSC000000050;nie0030000002o^mH0_k_F00@0_k_F00<00000
0<2me`30_ML00`30_ML010000000`;gG0000000000030<6nf00400000031_mP00000000000<0`L3H
00@000000<70f@00000000000`32`MT00`000000`L;J0<72fP0200000080`/;J00<000000<;3f`32
`m/00P0000020<;3f`0300000032a=/0`/CK008000000P32aM`010000000`/GL0<;5g032aM`20000
00040<?5g0000000`/KM0<;6g@8000000P33a]d00`000000`/KM0<;6g@0200000080`lOM00@00000
0<?7gP00000000000`33b=h010000000`lSN0000000000020<?8gP0400000033b=l00000000000<0
`lWO00@000000<?9g`00000000000`33bMl010000000`lWO0000000000030<?:g`0400000033b^00
0000000000<0`l[P00<000000<?:h033b^001033b^000`000000`l_P0<?;h0040<?;h00300000032
bn40`/_Q00<0`/_Q00<000000<;;h@32bn401032bn400`000000`/[Q0<;:h@030<;:h@0300000032
b^40`/[Q00<0`/[Q0P00000Y0?ooo`030000003oool0oooo03h0oooo000G0?ooo`030?oOW0180000
M;oo00<0oooo00<0okmd0000B02Lgol02P3oool2000000T0oooo0`0000020:[1jP8000000`2Z`N/3
000000@0Zl;[00<000000:_3j`2[`n/00P0000000`2/`n/000000:c4j`040:c4j`<00000102]a>/0
0`000000[LG/0:g5k002000000030:k5j`000000[/K/00@0[/K/0P0000050:k7k0030000002^b>d0
[/S]0080000000<0[lS/0000000000000P2_bNd2000000030;39k0000000/<[]00@0/<[]0P000005
0;7;k@<00000102bc>d00`000000/Lc^0;7<kP02000000030;;=k@00000000000080//g^0P000000
0`2cc^h00000000000020;??kP80000000<0]<o^0000000000000P2dd>h200000080]M3^0P000002
0;CAk`80000000<0]]7^0000000000000P2fd^l2000000030;OCkP00000000000080]]C_0P000000
102ge>l000000000002geNl2000000@0^=K_0P0000050;[Gk`8000000`2kfNl2000000040;cIkP00
000000000;cKk`800000102nfnh2000000<0`=g^0P0000020<;Mk@8000001@34g^`00`000000a]kZ
0<KNjP02000000060<SOj@000000000000000038gnT000000P3:g^L200000080cMkU0P0000040<kL
h`8000000P3@fn42000000<0d=WO00D000000=7GgP000000d]SM000000040=;Ef`060000003AdmX0
00000000003BdmT000000`3BdMP01P000000dM3H000000000000d/oG000000@0dLkF00<000000=3<
eP0000000P3Ac=D00`000000d<_D0=3;e0020=3;e0030000003@bM@0d<WD0080d<WD00H000000<o8
d`00000000000<o8d`0000040<o7d`030000003?a]80clKB00<0clKB00D000000<k6dP000000c/KB
000000050<k5dP@000000P09Dll01@000000<VOF039WeP000000A6_C00<000000P1ON=@01@000000
KG[?06ejc`1]N/l0000000<0NGg;00<0000008>1b023PLP00P23PLP00`000000SHG608f5aP0208f5
aP050000002DRLD0U8W509B9a@0000001@2KS/D00`000000X9;60:2BaP020:2BaP<000000`2VUL@0
0`000000ZY[60:ZJaP020:ZJaP040000002/WLP0[9g80:bMb0<000001@2aX/P010000000/ZG:0;:U
bP2bYLX3000000H0]ZS<00<000000;V/c@2i[<d00`2i[<d010000000^Jk>0;V^cP2i[/h3000000H0
^k7@00<000000;bdd@2l]=400`2l]=4010000000_;GB0000000000030;jfdP030000002n]m<0_[OC
008000000P2o^=<00`000000_kWD0;nie0020;nie00300000030^]@0000000@0_k_F00<000000<6l
eP0000001@30_ML00`000000`KkH0<6nf0040<6nf00300000031`=P0`L3H00@0`L3H00<000000<;1
f@32`MT00`32`MT00`000000`L;J000000040<;2fP0300000032`m/0000000@0`/?K00<000000<;4
f`0000001032aM`010000000`/GL0000000000030<?5g00300000032a]d0000000@0`lKM00<00000
0<;6g@0000001@33amd00`000000`lSN0<?8gP040<?8gP0300000033b=h0`lSN00<0`lSN00<00000
0<?9g`33bMl01033bMl00`000000`lWO0<?9g`040<?9g`0400000033b]l0`l[O000000<0`l[O00<0
00000<?:h033b^000P33b^000`000000`l[P000000030<?:h080000000<0`l_P00000033bn000P33
bn03000000<0`/_Q0P0000000`32bn4000000<;;h@020<;;h@80000000<0`/[Q00000032b^400P32
b^400`000000`/[Q000000020<;:h@8000000P32b^400`000000oooo0?ooo`0W0?ooo`030000003o
ool0oooo03d0oooo000G0?ooo`030?oOW0180000M;oo00<0oooo00<0okmd0000B02Lgol02P3oool0
0`000000oooo0?ooo`090?ooo`030000002Z`NX0Z/7Z00<0Z/7Z0`0000020:_1jP8000000`2[`^/3
000000@0[<?[00<000000:c4j`2/a>/00P0000020:g4j`030000002]a>/0[LC[00<0[LC[0P000004
0:k5j`030000002^a^`0[/K/0080000000<0[/K/0000000000000`2^an`4000000D0[lS/0P000004
0;39k0030000002`b^d0/<[]0080000000<0/L[/0000000000000P2abnd200000080//_]0P000003
0;;<k@<000001@2bcNd2000000D0/lk^0P0000050;C?kP8000001@2ed>h3000000D0]]7^0P000005
0;OCkP8000001@2ge>l2000000<0^=K_0P0000000`2je^h00000000000020;[Gk`8000000P2kf>h3
000000D0_=W^0P0000030;kKkP8000000P30fnd3000000@0`]g]00D000000<CLj`00000000000<CN
k002000000<0a]kZ100000000`38gnT00000000000020<[Ni`80000000<0c=cU0000000000001@3>
g><00`000000cm[R0<oJhP02000000<0d=WO0P0000030=7GgP030000003BeM/0d]GK0080d]GK00<0
00000=7CfP3AdmX00P3AdmX01@000000d]7H0=;Af03BdMP0000000@0dM3H00@000000=7>eP3Ac]H0
dLkF0P0000020=3<eP030000003Ac=D0000000@0d<_D00<000000=3:e00000000P3@bM@00`000000
clSC0<o8d`020<o8d`030000003?am<0clOC0080clOC00D000000<k7d`00000000000<o6dP020000
00<0c/KB00<000000<k5dP3>aM800`3>aM801`000000c/GB00000000000003Zn00000009Dll00P00
00001P0bImH0000004A[d`14Jm<0000005E^d0<000000`1]N/l01@000000NGg;07Umb`1iOL/00000
00@0Ph7800<0000008f5aP2=QLH00P2=QLH00`000000U8W509B9a@0209B9a@030000002KS/D0Vhk5
0080Vhk500@000000:2BaP2PT/H0X9;60P0000050:JEa0030000002ZV/H0ZY[600<0ZY[60`000003
0:nNaP030000002aX/P0/J;800<0/J;80`0000030;FVb@030000002fZ<`0]ZS<0080]ZS<00<00000
0;RZb`0000001@2i[<d3000000<0^jo>00@000000;^ad02k/M00^k7@0P0000000`2m/]0000000;bd
d@040;bdd@030000002n]]80_[KB00<0_[KB00<000000;jgd`0000001@2o^=<010000000_kWD0000
000000020<2je0050000002o^mH0_k_F0;nkeP0000000P31_=H00`000000`;gG0<2me`020<2me`03
00000031_]L0000000@0`KkH00<000000<:of00000001031`=P00`000000`/7I000000050<;1f@03
00000032`]X0`/;J00@0`/;J00<000000<;3f`32`m/01032`m/00`000000`/GL0<;5g0040<;5g003
00000033aM`0`lGL00@0`lGL00<000000<?6g@33a]d01033a]d00`000000`lOM0<?7g@020<?7g@80
000000<0`lSM00000033b=h00P33b=h2000000030<?8gP000000`lSN0080`lSN00<000000<?9g`00
00000`33bMl2000000030<?9g`000000`lWO0080`lWO0P0000001033b]l000000<?:g`33b]l30000
00030<?:h0000000`l[P008000000P33b^0010000000`l[P0000000000030<?;h00400000033bn00
0000000000<0`l_Q0`0000030<;;h@0400000032bn400000000000<0`/[Q00@000000<;:h@000000
00000`32b^42000000D0`/[Q00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0m0?ooo`00
5`3oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00T0oooo00<000000?ooo`3oool0
2`3oool00`000000Z/7Z0:[1jP02000000030:_1j@000000Zl7Z00@0Zl7Z0`0000020:c2jP800000
0`2/`n/3000000D0[LC[00<000000:g4j`2]a>/00P0000000`2^aN/00000000000030:k5j`<00000
1@2^a^`3000000<0[lO/0P0000020:o8k080000000<0/<S/0000002`bN`0102`bN`2000000D0/L[/
0P0000050;;;k@@000000P2bc>d200000080//g]0P0000000`2ccNd00000000000020;?>kP800000
00<0]<k]0000000000000P2dcnh2000000030;G?k@00000000000080]M3^0P0000020;K@kP800000
0P2fdNh2000000030;OBkP00000000000080]m?^0P0000000`2hdnh00000000000020;ODk`800000
0P2ie>h3000000D0^]K^0P0000040;_HkP040000002lf>h0000000000080_=W^0P0000020;kIk@<0
00001030fnd01@000000`]_/000000000000`]g]008000001034g>/2000000040<KNjP0000000000
000000<0b=gY0P0000020<[Ni`8000000`3<g>D01@000000c=_T000000000000c]cS008000000`3?
f^801@000000cmSP0000003@fMl0000000@0dMON00H000000=3Eg000000000000=;Ef`0000030=7C
fP80000000@0dM7I0000003BdMP00000103Ad=P01@000000d<kG0000003Ac]H0000000D0d<cF00<0
00000=3;e03@bm@00P3@bm@01P000000d<[D0=3:e0000000d<WD000000@0clSC00<000000<o8d`00
00000P3?am<00`000000c/OC0<k7d`020<k7d`030000003>a]80c/KB0080c/KB00<000000<k5dP00
00000P3>aM82000000<0c/GB0P0000000`00>[h0000000UCc`0400000080A6_C00<0000005E^d01E
K]000P00000206ejc`030000001iOL/0NGg;0080NGg;00<0000008>1b023PLP00P23PLP01@000000
SHG608f5aP2=QLH0000000@0U8W500<0000009^>a@2KS/D00P2KS/D4000000030::@`P000000YYG4
00D0YYG400<000000:ZJaP2ZV/H00P2ZV/H00`000000[ik60:nNaP030:nNaP030000002aX/P0/J;8
00<0/J;800<000000;FVb@2eY/T00`2eY/T00`000000]ZS<0;JXc00200000080^:[;00<000000;V/
c@2i[<d00`2i[<d00`000000^jo>0;^_cP030;^_cP040000002k/M000000000000<0_K;@00<00000
0;bdd@2l]=400P2l]=400`000000_[GA000000050;jfdP030000002o^=<0_kSC00D0_kSC00<00000
0<2je030^]@00`30^]@00`000000_k_F000000040<6leP0400000030_ML00000000000<0`KkG00@0
00000<6nf000000000000P32_mP01@000000`L3H0<70f031`=P000000080`/7I00<000000<;1f@32
`MT00P32`MT00`000000`/;I000000040<;2fP0300000033`mX0000000@0`/?K00<000000<?4f`00
00001032aM`00`000000`lGL000000030<?5g080000000<0`lKL00000033a]d00P33a]d200000003
0<?7g@000000`lOM008000000`33b=d010000000`lSN0000000000030<?8gP0400000033b=h00000
00000080`lWO00@000000<?9g`00000000000`33bMl010000000`lWO0000000000030<?:g`040000
0033b]l00000000000@0`l[P00<000000<?:h033b^000`33b^000`000000`l_P0<?;h0040<?;h003
00000033bn40`l_Q00<0`l_Q00<000000<;;h@32bn401032bn400`000000`/[Q0<;:h@030<;:h@80
00001032b^400`000000`/[Q000000030<;:h@80000000<0`L[Q0000003oool09`3oool00`000000
oooo0?ooo`0l0?ooo`00603oool00`3OW4P004RL0=ooo`020?ooo`030;md0000M;l0oooo00T0oooo
00<000000?ooo`3oool0303oool2000000@0Zl7Y00<000000:_1jP2[`NX00P0000000`2/`^X00000
0:c2jP040:c2jP<000000P2]`nX2000000@0[LC[0`0000050:k5j`<000000P2^a^/200000080[/K/
0P0000000`2_a^/000000:o7k0050:o7k0800000102`b>`00`000000/<W/0;39k002000000030;79
k000000000000080/L[/0P0000000`2bb^`00000000000020;;;k@8000000P2bbn`00`000000//c]
0;;<k@030;;<k@8000001@2ccNd2000000D0]<k]0P0000050;G?k@8000001@2fd>h3000000@0]m;^
0`0000050;SCkP800000102ie>h010000000^]G^0000000000020;[FkP80000000@0^mK^00000000
0000^mS^0P0000040;cHkP800000102nfNd01@000000`=W/000000000000`=_]008000001032fn`2
000000<0a=c[1P0000030<SMj@8000000P39g>L2000000@0c=cU00<000000<cKi03<fn@00P3<fn@2
00000080cm[R0P0000030<oHh08000000P3Aemh2000000@0d=GL00<000000=7CfP3AdmX00P000004
0=7Af@030000003Ad=P0dM3H0080dM3H00D000000=3>e`3@c]L0d<kG000000050=3<eP060000003@
bmD000000000003@bm@00000103@b]@00`000000clSC0<o8d`020<o8d`060000003?b=<0clSC0000
003?am<00000103>am<00`000000c/KB000000020<k6dP060000003>aM80c/GB0000003>aM800000
103>aM800`000000c/GB00000004000000807UW@00@0000004A[d`14Jm<000000`1EK]0300000003
06ejc`000000NGg;0080NGg;00<0000008>1b023PLP00P23PLP01@000000SHG608f5aP2=QLH00000
00D0U8W500H0000009^>a@000000000009f;`00000040::@`P030000002VUL@0YYG400<0YYG400<0
00000:ZJaP2ZV/H00`0000050:nNaP040000002aX/P0/J;80;6Rb080000000<0]:?70000002eY/T0
102eY/T2000000D0^:[;00L000000;V/c@2i[<d0^Jc=0000002k[/d0000000D0^jo>00<000000;fb
d02m/]00102m/]0010000000_;CA0000000000020;jed@030000002n]]80_[KB0080_[KB00<00000
0;nhdP000000102o^=<3000000H0`;[D00<000000<6leP31_=H01031_=H00`000000`KkG0<6ne`04
0<6ne`0300000032_mP0`[oH00<0`[oH00<000000<70f00000001032`MT010000000`/7I00000000
00030<;2f@0400000032`]X00000000000<0`l?J00@000000<;3f`00000000000`33a=/010000000
`/GL0000000000020<?5g00400000033aM`00000000000<0`lKL00@000000<?6g@00000000001033
amd00`000000`lSM0<?8g@040<?8g@0300000033b=h0`lSN00@0`lSN00<000000<?9g`33bMl00`33
bMl00`000000`lWO0<?9g`040<?9g`0300000033b]l0`l[O00@0`l[O00D000000<?:h033b^000000
0<?:h002000000030<?:h0000000`l[P0080`l[P00<000000<?;h00000000`33bn02000000030<?;
h@000000`l_Q0080`l_Q00<000000<?;h@0000000`32bn42000000040<;;h@000000`/[Q0<;:h@80
00000P32bn4010000000`/[Q0000000000020<;:h@0400000032b^400000000000<0`L[Q00<00000
0?ooo`3oool09`3oool00`000000oooo0?ooo`0k0?ooo`00603oool01`3ookl0M00007Boo`3oool0
okmd0000M02oool02P3oool00`000000oooo0?ooo`0=0?ooo`030000002[`NT0Zl7Y00<0Zl7Y0P00
00040:c2jP030000002/`^X0[<;Z0080000000<0[L;Z0000002]`nX0102]`nX400000080[/CZ0P00
00020:k5j`80000000<0[/G[0000002^a^/0102^a^/2000000@0[lK[0P0000020:o7k080000000<0
/<O[0000000000000`2`b>`3000000D0/LW/0P0000050;;:k08000001@2bbn`00`000000//c]0;;<
k@02000000030;?<k000000000000080/lg]0P0000000`2dcNd00000000000020;C>k@80000000<0
]Lk]0000000000000P2ecnd2000000030;K?k@00000000000080]]3^0P0000020;OAk@030000002g
d^h0]m;^008000000P2hd^d200000080^=?^0P000000102idnd000000000002ie>h2000000@0^]G^
0P0000050;_FkP8000000`2lf>h2000000040;kHk@00000000000;kIk@8000001030fN`2000000<0
`]_/0P0000020<?Kj`D0000000@0aMcZ000000000000am_X0`0000040<WLi`060000003:f^D00000
0000003<g>D00000103<fn@00`000000cMWR0<gIhP02000000@0cmSP00<000000<oFgP3?e]h00P00
00050=3Eg0040000003@dm/00000000000D0dM7I00H000000=3@f@00000000000=7@f00000030=3>
e`050000003@cML000000000003@c=H00P0000040=3;e@030000003@b]@0d<[D0080d<[D00H00000
0<o9e000000000000<o8d`0000040<o8d`040000003>am<0c/OC0<k7d`8000000P3>a]800`000000
c/KB000000040<k5dP030000003>aM80c/GB0080c/GB00<000000<k5dP3>aM80100000001000@/@0
000001iId00NFM020000000304A[d`000000EFk@0080EFk@00<0000006Ebb`0000000`00000207Um
b`0300000023PLP0Ph780080Ph7800D0000008f5aP2=QLH0SHG60000000209B9a@@00000102MRl00
1@000000XY320::@`P2RT<80000000H0YYG40P0000030:fJa0030000002_W/H0[ik600<0[ik60`00
00030;BSa`030000002eY/T0]JK900<0]JK900<000000;RZb`2hZ//0102hZ//010000000^Jc=0000
000000020;^^c@030000002k[lh0^jo>0080^jo>00<000000;facP0000001P2m/]000`000000_[GA
0;jed@030;jed@040000002n]]800000000000<0_kSB00@000000;nhd`00000000000`30^]@01000
0000`;[D0<2je030^]@2000000030<6ke@000000`KcF0080`KcF0`0000050<6ne`8000001P32_mP0
0`000000`/7I0<;1f@040<;1f@0300000032`]T0`/;I00@0`/;I00<000000<?3fP33`mX01033`mX0
0`000000`lCK0<?4f`040<?4f`0300000033aM`0`lGL00<0`lGL00<000000<?6g033a]`01033a]`0
0`000000`lOM0<?7g@030<?7g@0300000034b=d0000000<0`lSM0P0000000`34b=h000000<?8gP02
0<?8gP80000000<0`lWN00000033bMl00P33bMl00`000000`lWO000000040<?9g`0300000033b]l0
000000<0`l[O0P0000000`33b^0000000<?:h002000000<0`l[P00@000000<?:h000000000000P33
bn0010000000`l_P0000000000030<?;h@0400000033bn40000000000080`l_Q00@000000<;;h@00
000000001032bn42000000D0`/_Q00<000000<;:h@32b^400`32b^400`000000`L[Q0<7:h@040<7:
h@030000003oool0oooo02H0oooo00<000000?ooo`3oool0>`3oool001T0oooo00D0oono07@00000
0000001d0;ooo`0;0?ooo`030000003oool0oooo00h0oooo00<000000:_1j@2[`NT00P0000000`2/
`NT000000:c2jP040:c2jP800000102]`^X00`000000[L?Z0:g3jP0200000080[/?Z00<000000:k4
jP2^a>X00`2^a>X2000000@0[/G[00<000000:k6j`2^a^/00P0000000`2_a^/000000:o6j`040:o6
j`<000001@2`an/300000080/LS[0P0000020;79k080000000<0//W/0000000000000P2bb^`20000
00030;?:k0000000//_/00@0//_/0`0000050;?<k08000001@2dcNd2000000D0]Lk]0P0000050;K?
k@8000001@2gdNd2000000D0^=;]0`0000050;WCk@8000000`2jeNh2000000030;_Ek@0000000000
0080^mK^0P0000020;cFk@<000001@2nf>d2000000<0`=W/0P0000020<7Ij`<000000`33fn/40000
00030<GLjP000000000000<0am_X00D000000<SJi`00000000000<WLi`02000000@0b][U0P000002
0<cKi08000000`3=fN8010000000c]OQ0000003?f>02000000<0cmKN00D000000<oDg@0000000000
0=3Eg00200000080d=?K00D000000=3AfP00000000000=7Af@02000000@0d=3I00D000000=3>e`3@
c]L0d<kG000000040=3=e`030000003@bmD0d<_E00<0d<_E00@000000<o:e@000000d<[D0P000004
0<o9e0030000003?b=<0clSC0080clSC00D000000<k7d`000000c/OC000000050<k6dP030000003>
aM80c/GB0080c/GB00H000000<k5dP00000000000<k5dP0000040<k5dP@0000000D004;40000000N
FM00000003=NcP02000000<0EFk@00@0000006Ebb`1UL//000000P1dM/L>00000080UXFo00<00000
09f;`02MRl000`2MRl000`000000XY320::@`P020::@`P040000002VUL@0YYG40:JEa0<00000102]
V/@00`000000[ik60:nNaP030:nNaP030000002dXlL0]:?700@0]:?700L000000;FVb@2eY/T0]JK9
0000002hZ<T0000000H0^:[;00<000000;^^c@2k[/d00`2k[/d010000000^jo>0000000000020;fa
cP030000002m/]00_K;@0080_K;@0`0000060;jed@030000002o^=80_kSB00@0_kSB00<000000<2j
e030^]@00`30^]@010000000`;[D0000000000030<6ke@0400000031_=H00000000000<0`KgF00<0
00000<6ne`31_]L00P0000020<:oe`0400000032_mP0`[oH0<:of080000000<0`/3H00000032`MT0
0P32`MT2000000030<?2f@000000`/;I0080`/;I0P0000000`33`mX000000<?3fP020<?3fP800000
00<0`lCK00000033a=/00P33a=/2000000030<?5f`000000`lGL0080`lGL00<000000<?6g0000000
1033a]`00`000000`lOM000000020<?7g@8000000P34b=d010000000`lSM0000000000030<C8gP04
00000033b=h00000000000<0`lWN00@000000<?9g`00000000000`33bMl010000000`lWO00000000
00020<?:g`0400000033b]l00000000000@0`l[P00@000000<?:h033b^0000000`33b^000`000000
`l_P0<?;h0030<?;h00300000033bn40`l_Q00@0`l_Q00<000000<?;h@33bn400`33bn400`000000
`/_Q0<;;h@020<;;h@80000000<0`/_Q00000032bn400P32bn42000000030<;;h@000000`/[Q0080
`/[Q00<000000<;:h@0000000`31b^42000000030<7:h@000000oooo02L0oooo00<000000?ooo`3o
ool0>P3oool002T0oooo0`00000?0?ooo`800000102/`NT00`000000[<;Z0:c2jP02000000030:g2
j@000000[L;Z00@0[L;Z0P0000050:k3jP030000002^a>X0[/CZ0080000000<0[/CZ0000002^aN/0
102^aN/2000000@0[lK[00<000000:o6j`2_a^/00P0000020;37j`8000000P2`an/2000000030;78
j`000000/LS[00@0/LS[0P0000050;;9k0800000102cb^`00`000000//_/0;;;k00200000080/l_/
0P0000020;?<k080000000<0]<c/0000000000000P2dcNd2000000030;G=k000000000000080]Lk]
0P0000000`2fcn`00000000000020;K?k@80000000<0]m3/0000000000000P2gdNd2000000030;SA
k@00000000000080^=;]0P0000020;WBk@8000000P2idnd200000080^]?]0`0000040;_Ek@<00000
102le^d010000000_]K/0000000000020;kHk@8000000P2of>`3000000<0`MW[0P0000000`33fNX0
000000000002000000@0aM[Y0P0000020<OKj08000001038f^L00`000000b][U0<[Ji@030<[Ji@03
0000003;fN@0bmWT008000000`3=fN82000000<0c]OQ00<000000<oFgP3?e]h00P3?e]h00`000000
cmCM0<oDg@020<oDg@050000003@dm/0d=?K0=3Cf`000000103@dMX00`000000d=3I0=3@f@030=3@
f@050000003@c]P000000=3>e`000000103@cML00`000000clcF000000020=3;e@8000000`3?b]D0
0`000000clWD0<o9e0030<o9e0040000003>b=@000000<o8d`8000000`3>am<00`000000c/KB0<k6
dP030<k6dP040000003>a]8000000<k5dP800000103>aM800`000000c/GB0<k5dP020<k5dP030000
003>a=40000000L000000P0cG/h010000000EFk@05E^d000000306Ebb`050000001dM/L0M7K707Af
a`0000001021Nl<01@000000SH3008f0`02=P<00000000D0UXFo00<0000009f;`02MRl000P2MRl00
0`000000XY320::@`P020::@`P040000002VUL@00000000000<0ZIG100<000000:fJa02]V/@00P2]
V/@01`000000[ik60:nNaP2_W/H000000;:Oa@0000001@2dXlL010000000]JK90000000000020;RX
b@030000002hZ//0^:[;00<0^:[;0P0000060;^^c@030000002m/Lh0_K7>00<0_K7>00@000000;fb
d000000000000`2n]=0010000000_[GA0;jed@2n]M43000000D0_kSB0P0000060<2je00300000031
^mD0`K_E00@0`K_E00<000000<6meP31_MH00`31_MH00`000000`KkG000000040<:oe`0400000032
_mP00000000000<0`/3H00@000000<;1f@00000000000`33`]T010000000`/;I0000000000030<?3
fP0400000033`mX00000000000<0`lCK00@000000<?4f`00000000000`33aM/010000000`lGL0000
000000030<?6g00400000033a]`0000000000080`lOM00<000000<?7g@0000001@34b=d00`000000
a<SN0<C8gP040<C8gP0300000033bMh0`lWN00@0`lWN00<000000<?9g`33bMl01033bMl00`000000
`l[O0<?:g`030<?:g`0300000033b^00`l[P0080`l[P0P0000001@33bn0000000<?:h0000000`l[P
0080000000<0`l_P00000033bn000P33bn000`000000`l_Q000000030<?;h@80000000<0`l_Q0000
0033bn400P33bn4010000000`/_Q00000032bn42000000<0`/_Q00@000000<;;h@00000000000`32
bn4010000000`/[Q0000000000020<;:h@0400000031b^400000000000<0`L[Q00<000000?ooo`3o
ool09`3oool00`000000oooo0?ooo`0i0?ooo`00:03oool200000180oooo00<000000:c1j@2/`NT0
0`2/`NT2000000@0[L;Y00<000000:g2jP2]`^X00P0000000`2]`nT000000:k3jP050:k3jP800000
102^a>X00`000000[/G[0:k5j`02000000030:o5jP000000000000<0[lK[0`0000050;37j`<00000
102ab>/00`000000/LS[0;78j`02000000030;;8j`00000000000080//W/0P0000000`2cbN/00000
0;?:k0040;?:k08000001@2cbn`3000000D0]<c/0P0000050;G=k08000001@2fcn`2000000D0]m3/
0P0000050;SAk@8000001@2id^d3000000<0^]?]0P000000102ke>`000000;_Ek@2keNd200000080
_=G/0P0000000`2le^d00000000000040;kFk08000000`2of>`2000000030<7Hj`000000`MW[0080
00000P33fNX3000000D0aM[Y00<000000<KIj036fNP00P0000050<SJi`0500000039fND000000000
003:f^D00P0000020<_Ii080000000@0c=SR0000003=fN800000103>en401P000000c]GO00000000
0000cmKN000000@0cmCM00D000000<oCg0000000d=?K000000040=3AfP050000003?d=T000000000
003@d=T00P0000030=3>f0030000003@cML0d<gG0080d<gG00H000000<o<eP3?c=H000000=3;e@00
00040<o:e@030000003>bMD000000080clWD0P0000030<k8e0030000003>am<0c/OC0080c/OC00<0
00000<k6d`0000000`3>a]801@000000c/KB0<k6dP3>a]80000000D0c/GB00<000000<g5dP000000
0P3>aM8010000000c/CA0<k4d@3>a=450000000400M8a`000000<ek>03=NcP<0000000D0EFk@0000
001UL//0IG;;0000000407Afa`0300000021Nl<0PG_30080PG_300D0000008f0`02=P<00SH300000
000409J5_`030000002MRl00WH_00080WH_000<000000::@`P2RT<800P2RT<82000000D0ZIG100<0
00000:fJa02]V/@00P2]V/@010000000[ik60000000000020;:Oa@030000002dXlL0]:?700@0]:?7
00<000000;RXb@2hZ<T00`2hZ<T00`000000^:[;0;RZb`0200000080^jc;00<000000;^^c@2k[/d0
0P2k[/d2000000H0_K7>00<000000;jdd02n]=000`2n]=0010000000_[GA0000000000030;ngd@03
0000002o^=80_kSB008000000P30^M<010000000`;[D0<2je030^]@3000000D0`K_E0P0000060<6m
eP0300000032_mL0`[oG00@0`[oG00<000000<;0f032`=P01032`=P00`000000`l;I0<?2f@040<?2
f@0300000033`mX0`l?J00@0`l?J00<000000<?4f`33a=/01033a=/00`000000`lGK0<?5f`040<?5
f`0300000033a]`0`lKL00@0`lKL00<000000<?7g@33amd00`33amd00`000000a<SM0<C8g@030<C8
g@0300000034b=d0000000<0a<SN0P0000000`34bMh000000<?9gP020<?9gP80000000<0a<WN0000
0033bMl00P33bMl2000000030<C:g`000000`l[O0080`l[O00@000000<?:g`000000`l[P0P000003
0<?;h00400000033b^000000000000<0`l_P00@000000<?;h000000000000P33bn4010000000`l_Q
0000000000030<?;h@0400000033bn400000000000<0`/_Q00<000000<;;h@32bn401032bn400`00
0000`/_Q0<;;h@040<;;h@0300000032b^40`/[Q00<0`/[Q00<000000<7:h@31b^401031b^400`00
0000oooo0?ooo`0V0?ooo`030000003oool0oooo03T0oooo000X0?ooo`030000003oool0oooo0180
oooo00<000000:c1j@2/`NT00P0000000`2]`NT000000:g2j@040:g2j@800000102]`nT200000080
[/?Z0P0000000`2^`nT000000:k4jP040:k4jP8000001@2_aNX300000080/<KZ0P0000020;37j`80
00000P2aanX00`000000/LS[0;78j`030;78j`8000001@2bb>/2000000@0/lW[00<000000;?:k02c
b^`00P0000000`2cbn/00000000000020;?;k08000000P2dc>/200000080]<c/0P0000000`2ecN`0
0000000000020;G=k080000000<0]/k/0000000000000P2fcn`2000000030;O?k000000000000080
]m3/0P0000000`2hd>`00000000000020;SAk@80000000<0^M7/0000000000000P2id^d200000080
^];/00@000000;[Ck@0000000000102ke>`2000000D0_=G/0P0000040;kFk080000000<0_mK[0000
002of>`00P0000040<7Hj`0300000033fNX000000080000000<0`mWZ00000034f>T00P0000000`35
f^T00000000000030<KIj00500000037fNL0000000000038f^L00P0000040<WIi@030000003;fN@0
bmWT00800000103<f>8200000080c]OQ0P0000040<kEg`040000003?e=d0cmCM0<oDg@8000000`3?
dm`00`000000d=7J0=3AfP020=3AfP030000003?d=T0cm3I0080cm3I00<000000=3>f03@c]P00P3@
c]P01P000000clgG000000000000d<gG000000@0clcF00<000000<o:e@3?b]D00P3?b]D01P000000
c/WE0<k9e@000000clWD000000@0c/SD00<000000<k7d`0000000P3>am<01`000000c/KC0<k6d`3>
a]<000000<k6dP0000000`3>a]800`000000cLGB000000030<k5dP060000003=aM80cLGB0000003>
aM8000001@3>a=4200000005000^]`0000001dS700M8a`0000000P0cG/h00`000000Af?;00000002
00000080IG;;00D0000007Afa`1dM/L0M7K700000004085k``050000002=P<00SH3008f0`0000000
102FQKl00`000000WH_009f;`00309f;`0060000002RT<8000000000002UT;h000001@2YUL400`00
0000[I[40:fJa0030:fJa0030000002bWlD0/Yo500<0/Yo500@000000;BSa`2dXlL0]:?70`000005
0;RXb@030000002hZ//0000000@0^jc;00<000000;^^c@2k[/d00P0000020;f`c@030000002m/Lh0
_K7>0080_K7>0P0000060;jdd0030000002o]m40_kOA00<0_kOA00<000000;nhdP0000001030^M<0
10000000`;[D0000000000030<6ke00300000031^mD0`K_E008000000P32_MD010000000`KgF0<6m
eP31_MH2000000030<:neP000000`[oG0080`[oG0P0000000`33`=L000000<;0f0020<;0f0800000
00<0`l7H00000033`]T00P33`]T2000000030<?3f@000000`l?J0080`l?J0P0000000`33a=X00000
0<?4f`020<?4f`80000000<0a<GK00000033aM/00P33aM/2000000030<C6g0000000`lKL0080`lKL
0P0000000`34am`000000<?7g@020<?7g@0300000034amd0000000<0a<SM00D000000<C8g@34b=d0
00000<C8gP02000000<0a<WN00@000000<?9gP00000000000`34bMh010000000`lWO000000000003
0<C:g`0400000033b]l00000000000<0`l[O00<000000<?;h033bn001033bn0010000000`l_P0<?;
h00000030<?;h00300000033bn40`l_Q00<0`l_Q00<000000<?;h@33bn400`33bn42000000@0`/_Q
00<000000<;;h@0000000`32bn42000000030<;;h@000000`/_Q0080`/_Q0`0000030<;:h@800000
00<0`L[Q00000031b^400P31b^43000002T0oooo00<000000?ooo`3oool0>03oool002P0oooo00<0
00000?ooo`3oool04`3oool2000000@0[L7Y00<000000:g2j@2]`^T00P0000000`2]`^T000000000
00030:g3j@@00000102^`nT00`000000[/CZ0:k4jP02000000030:o4j@00000000000080[lGZ0P00
00000`2`aNX000000;36jP040;36jP8000001@2aanX00`000000/LS[0;78j`02000000030;;8j`00
000000000080//S[0P0000000`2cbN/000000;?9j`040;?9j`8000001@2cbn/2000000D0]<c[0`00
00050;G=k08000001@2fc^`2000000D0]lo/0P0000050;S@k08000001@2idN`2000000D0^];/00<0
00000;_Dk02ke>`00`2ke>`010000000_=C[0000000000020;cEk08000000P2neN/010000000_]K/
0000000000040;oFj`0300000031f>/0`MS[00<0`MS[100000000`33fNX00000000000040<CHj@80
00000P36fNP2000000@0amWW00<000000<WIi@39fND00`39fND010000000b]ST0000000000050<cH
hP030000003<e^40c=KQ008000001@3>eMl01@000000c]CN0000003?e=d0000000@0cm?L00H00000
0<oAf`00000000000=3AfP0000040<o@f@030000003?c]T000000080d<kH00<000000<o=e`3?cML0
0P3?cML00`000000clcF0<o<eP020<o<eP060000003?b]H000000000003?b]D00000103>bMD00`00
0000c/SD0<k8e0020<k8e0060000003>am<0c/OC0000003>am<000001@3>a]<01@000000c/KB0<k6
dP3>a]80000000<0cLGB00<000000<k5dP000000103=aM800`000000c/CA0<k4d@020<k4d@800000
00H0c/CA00000000000002jg00000007B<L20000000603=NcP000000Af?;04MSb`000000FfS70`00
000407Afa`0500000021Nl<0PG_3085k``000000102=P<001@000000UXFo09J5_`2FQKl0000000@0
WH_00`0000040:F@_P030000002YUL40ZIG10080ZIG100@000000:fJa02]V/@0[I[40`0000050;:O
a@040000002dXlL00000000000<0]jG700<000000;RXb@2hZ<T00`2hZ<T00`000000^jc;0;^/b`04
0;^/b`800000102m/<d00`000000_K7>0;facP0200000080_[?>00<000000;jdd02n]=000P2n]=02
000000H0_kOA00<000000<2id`30^M<01030^M<00`000000`K_D0<6ke0030<6ke00300000031^mD0
000000@0`[gE00@000000<6meP00000000000`32_]H010000000`[oG0000000000030<?0e`040000
0032`=P00000000000<0`l7H00@000000<?2f@00000000000`33`mT010000000`l?J000000000003
0<?4fP0400000033a=/00000000000<0a<GK00@000000<?5f`00000000000`34a]`010000000`lKL
0000000000030<C7g00400000033amd00000000000<0a<OM00<000000<C8g@0000001034b=d00`00
0000a<WN0<C9gP040<C9gP0300000034bMh0a<WN00@0a<WN00<000000<C:g`34b]l01034b]l00`00
0000`l[O0<?:g`020<?:g`0300000033bn00000000<0`l_P0P0000001@33bn0000000<?;h0000000
`l_P0080000000<0`l_P00000033bn400P33bn400`000000`l_Q000000020<?;h@8000000P33bn40
10000000`/_Q0000000000020<;;h@0400000032bn400000000000<0`/_Q00@000000<;;h@000000
00000`32bn43000000<0`L[Q00@000000<7:h@00000000000`31b^400`000000oooo0?ooo`0V0?oo
o`800000>@3oool002P0oooo00<000000?ooo`3oool0503oool2000000<0[L7Y0`0000050:g2j@<0
00000`2^`nT2000000<0[/?Y0`0000050:o4j@800000102`aNX00`000000/<KZ0;36jP0200000003
0;76jP000000/LOZ00@0/LOZ0`0000050;;8j`800000102cbN/00`000000/lW[0;?9j`0200000003
0;?:j`00000000000080/l_[0P0000000`2dbn/00000000000020;C<j`8000000P2ec>/200000080
]Lg/0P0000000`2fcN/00000000000020;K>k080000000<0]lk[0000000000000P2gcn`200000003
0;S?j`00000000000080^=3/0P0000000`2id>/00000000000020;WAk080000000<0^]7[00000000
00000P2jd^`2000000040;_Bj`00000000000;_Dk0800000102le>/2000000D0_]G[0P0000030;oF
j`80000000<0`=KZ0000000000000`0000040<;Gj@8000000`34f>T200000080aMSX0P0000050<OI
i`0500000038f>D0000000000039fND00P0000020<[Hi0050000003;en<000000000003<f>800P00
00030<cFh@050000003=e=l000000000003>eMl00P0000030<kDgP8000000P3?dm`2000000@0cm7K
00@000000<o@f@3?d=T0cm3I0P0000020<o>f@030000003@c]P0000000@0clgG00@000000<o<e`00
0000clcF0P0000040<o:eP030000003>bMD0c/WE0080c/WE00H000000<k8e000000000000<k8e000
00040<k7d`030000003>a]<0c/KC00<0c/KC00D000000<k6d`000000c/KB000000050<g5dP030000
003=aM80cLGB0080cLGB00@000000<g4d@000000c/CA0P0000040<k4d@80000000<002jg00000007
B<L01000000204MSb`030000001KJ<L0FfS700<000000P1dM/L01@000000PG_3085k``21Nl<00000
00@0SH3000<0000009J5_`2FQKl00P2FQKl01P000000WH_0000000000000WhVl000000D0YI2n00<0
00000:VE`@2YUL400`2YUL43000000<0/9_200<000000;:Oa@2bWlD00`2bWlD00`000000]jG70;NU
a`030;NUa`040000002hZ<T0^:S90;RXb@<000001P2k[</00`000000_K3=0;f`c@040;f`c@800000
102n/lh00`000000_[C@0;jdd00200000080`;K@00@000000;ngd@2o]m40_kOA0P0000000`31^=40
00000<2id`030<2id`0300000032^]<0000000D0`K_D00<000000<:me@32_MD01032_MD00`000000
`[kF0<:neP040<:neP0300000033`=L0`l3G00@0`l3G00<000000<?1f033`MP01033`MP00`000000
`l?I0<?3f@040<?3f@0300000033a=X0`lCJ00@0`lCJ00<000000<C5f`34aM/01034aM/00`000000
a<KL0<C6g0040<C6g00300000034am`0a<OL00@0a<OL00<000000<C7g@34amd01034amd00`000000
a<SM0<C8g@020<C8g@0300000034bMh0000000<0a<WN0P0000000`34bMh000000<C9gP020<C9gP80
000000<0a<[O00000034b]l00P34b]l2000000030<C:g`000000`l[O008000000P33bn0010000000
`l_P0000000000030<?;h00400000033bn000000000000<0`l_P00@000000<?;h@00000000000`33
bn42000000D0`l_Q00<000000<;;h@32bn400`32bn400`000000`/_Q0<;;h@040<;;h@0300000032
bn40`/_Q00<0`/_Q00<000000<7:h@31b^400`31b^42000000D0`L[Q00<000000?ooo`3oool0903o
ool2000000030?ooo`000000oooo03P0oooo000X0?ooo`030000003oool0oooo01H0oooo0`000002
0:g1j08000000P2]`^T2000000030:k2j0000000[/?Y00D0[/?Y0`0000020:o4j@8000000P2_a>T2
000000030;35j@000000/<GZ00@0/<GZ0P0000040;76jP030000002aanX0/LOZ008000000P2banX2
00000080//S[0P0000000`2bb>X000000;?9j`040;?9j`8000001@2cb^/2000000D0]<_[0P000005
0;G<j`<000001@2fcN/2000000D0]lk[0P0000050;S?j`8000001@2id>/2000000D0^]7[0P000005
0;_Bj`8000000`2le>/2000000030;gDj`00000000000080_]G[0P0000020;oEjP<000000P30e^X5
000000<0`]OY0P0000020<?Gj0<000001035f>P01@000000a]OV000000000000amWW008000001038
f>D00`000000b]ST0<[Hi002000000@0bmOS00<000000<cFh@3<e^400P3<e^400`000000cMCO0<gD
g`020<gDg`030000003>e=h0c]CN0080c]CN00<000000<kBg@3>d]d00P0000050<oAf`050000003>
d=X000000<o@f@0000001@3?c]T00`000000clgG0<o=e`020<o=e`050000003?c=L0clcG0<o<e`00
00001@3?b]H010000000c/WE0000003>bMD2000000@0c/SD00<000000<k7d`3>am<00P3>am<01@00
0000c/KC000000000000c/KC008000000`3>a]<00`000000cLGB0<g5dP030<g5dP040000003=aM80
00000<g5dP8000000`3=a=400`000000c/CA0<k4d@030<k4d@L000000P0MC/L010000000Af?;04MS
b`00000305]Xa`060000001^K/8000000000001dM/L000000`21Nl<00`000000SH3008f0`00208f0
`0030000002FQKl0UXFo00<00000102ORK`00`000000YI2n0:F@_P020:F@_P030000002YUL40ZIG1
0080ZIG10P0000050;2K`P040000002bWlD0/Yo50;:Oa@<000001P2gYLL3000000<0^Z[900@00000
0;^/b`2k[</0^jc;0P0000000`2m[//000000;f`c@040;f`c@030000002n/lh0_[?>00@0_[?>0P00
00040<2fd0040000002o]m400000000000<0`KSA00D000000<2id`30^M<0`;WC000000020<:jd`03
00000031^m@0`K_D0080`K_D00<000000<:le00000001032_MD00`000000`kkE000000040<:neP03
00000033_mL0000000@0`l3G00<000000<?1f00000001033`MP00`000000a<;I000000040<?3f@03
00000034`mT0000000@0`lCJ00<000000<C5fP0000000`34aM/2000000030<C6f`000000a<KL0080
a<KL0P0000000`34a]`000000<C7g0020<C7g080000000<0a<OL00000034amd00P34amd200000003
0<C8g@000000a<SM008000000P34bMh010000000a<WN0000000000030<C9gP0400000034bMh00000
000000<0a<[O00@000000<C:g`00000000001034b]l00`000000`l_P0<?;h0030<?;h00300000033
bn00`l_P00@0`l_P00@000000<?;h033bn0000000`33bn000`000000`l_Q0<?;h@020<?;h@030000
0033bn40000000<0`l_Q0P0000000`33bn4000000<;;h@020<;;h@0300000032bn40000000<0`/_Q
0P0000000`32bn4000000<;;h@020<;;h@0300000031bn4000000080`L[Q0P0000020<7:h@030000
0031b^40`L[Q0080000000<0`L[Q0000003oool0:03oool00`000000oooo0?ooo`0f0?ooo`00:03o
ool00`000000oooo0?ooo`0G0?ooo`030000002]`NP0[L7X00<0[L7X0P0000040:k2j08000000P2^
`nT2000000030:o3j0000000[lCY00@0[lCY0P0000040;35j@030000002`aNX0/<GZ0080000000<0
/LGY0000000000000`2aa^X3000000H0//OZ0P0000040;;8jP030000002cbN/0/lW[0080000000<0
/lWZ0000000000000P2cb^/2000000030;C:jP00000000000080]<_[0P0000000`2ebnX000000000
00020;G<j`8000000P2fc>/00`000000]/g[0;K=j`020;K=j`030000002gcN/0000000@0]lk[00<0
00000;S>j`000000102hcn/010000000^Lo[0000000000020;W@j`80000000<0^]3[000000000000
0P2jdN/2000000030;_Aj`00000000000080^m;[0P0000020;cBj`<000001@2me>/2000000@0_mGZ
00<000000<3Ej@0000000P0000000`30e^X00000000000020<;Fj@<000001033enP01@000000a=KW
000000000000aMSX008000001036enH00`000000b=SU0<SHi@020<SHi@80000000<0bMOT00000000
00001@3;en<01P000000bmGQ000000000000c=KQ000000<0cMCO0P0000000`3=dmh000000<kDgP02
000000<0c];M00D000000<kAg000000000000<oAf`02000000<0c]3J00<000000<o>f@3?c]T00`3?
c]T01P000000c/gH000000000000clgG000000<0clcG00<000000<k;eP0000000`3?b]H01@000000
c/WE0<k9e@3>bMD0000000D0c/SD00@000000<k7e0000000c/OC0P0000040<k6d`030000003>a]<0
c/KC0080c/KC00<000000<g5dP0000000`3=aM801@000000cLGB0<g5dP3=aM80000000@0cLCA00<0
00000<g4d@0000000`3>a=400`000000c/CA0<k4d@0200000005000JYP00000000000000000MC/L0
0`0000000`17Hl/0000005]Xa`0205]Xa`030000001^K/80KVk200d0000000<0V8:j0000002ORK`0
102ORK`00`000000YI2n0:F@_P030:F@_P060000002YUL4000000000002]U[l000001P2`Vl830000
00<0]Z;400<000000;NUa`2gYLL00`2gYLL00`000000^Z[90;ZZb@030;ZZb@040000002k[</00000
000000<0_Jk;00<000000;f`c@2m/<d00P2m/<d00`000000_k;=000000050;jccP0300000030]]00
`;K@00@0`;K@00<000000<6hd@31^=401031^=400`000000`;WC000000040<:jd`0400000031^m@0
0000000000<0`[cD00@000000<:me@00000000000`33_]D010000000`[kF0000000000020<>oe`05
00000033`=L0`l3G0<?0e`0000000P33`MP01@000000`l7H0<?1f033`MP000000080a<;I00D00000
0<?3f@33`mT0`l?I000000030<C3f@0400000033a=X0000000000080a<GJ00@000000<C5f`000000
00000`34a]/010000000a<KL0000000000030<C6g00400000034am`00000000000<0a<OL00@00000
0<C7g@00000000001034b=d00`000000a<WN0<C9gP030<C9gP0300000034bMh0a<WN00@0a<WN00<0
00000<C:g`34b]l01034b]l00`000000a<[O0<C:g`020<C:g`80000000<0a<_O00000033bn000P33
bn000`000000a<_P000000030<?;h080000000@0`l_P00000033bn00`l_P0`0000000`33c>400000
0<?;h@0200000080`l_Q00@000000<?;h@00000000000`33bn4010000000`/_Q0000000000020<;;
h@0400000032bn400000000000<0`/_Q00@000000<;;h@00000000000`31bn42000000D0`L[Q0P00
00040<7:h@030000003oool0oooo02L0oooo00<000000?ooo`3oool0=@3oool002P0oooo00<00000
0?ooo`3oool0603oool00`000000[L7X0:g1j002000000030:k1j0000000[/;X00@0[/;X0`000004
0:o3j0030000002_a>T0[lCY0080000000<0/<CY0000002`aNT0102`aNT2000000D0/LGY0`000002
0;;6j@8000000`2banX2000000030;;7j@000000000000<0//SZ0`0000050;?9jP8000001@2db^X2
000000D0]L_Z0P0000050;K<j`040000002fcN/00000000000<0]lg[00@000000;O>j`0000000000
0`2hc^/010000000^<o[0000000000040;W?j`8000001@2jd>/2000000D0^m7[0P0000040;cBj`04
0000002md^X0000000000080_MC[0P000000102ne>X000000000002oeNX400000080`=GY0P000004
0<;Fj@0500000033e^P0000000000033enP00P0000040<CFi`0300000036enH0a]OV0080a]OV0P00
00000`37e^D000000<SHi@02000000<0bMOT00D000000<WEh`00000000000<_Gh`02000000@0bmGQ
00<000000<gDg`3=e=l00P0000040<gCgP030000003>d]d0c];M0080c];M00<000000<kAg03>dM`0
0P3>dM`00`000000c]3J0<k@fP020<k@fP050000003>c]T000000000003?c]T00P0000040<k=f005
0000003?c=L0clcG0<o<e`0000000`3>bmH00`000000cl[F000000030<k9e@030000003>b=D00000
00<0c/SD00D000000<k7e03>am@0c/OD000000050<k6d`030000003=a]<000000080c/KC00L00000
0<g5dP3=aM80cLGB0000003=aM80000000<0cLGB00<000000<g4dP0000000P3=a=401`000000cLCA
0<g4d@3=a=4000000<k4d@000000103>a=4200000003000JYP000000000000807Dk700@0000003AD
a@00000000000`1KJ<L01@000000KVk206i^`P1^K/80000000<0OgBm00<0000008ej^P2=N[X00P2=
N[X00`000000V8:j09R2^P0209R2^P030000002ORK`0WhVl00<0WhVl00<000000:F@_P2UT;h00P2U
T;h2000000@0[IJo00<000000;2K`P2`Vl800`2`Vl800`000000]Z;40;JRa0030;JRa0040000002g
YLL0]jG70;NUa`80000000<0^ZO60000002jZ/T0102jZ/T00`000000_Jk;0;f^b`040;f^b`040000
002m/<d0000000000080_k;=00<000000;jccP2n/lh00P2n/lh00`000000`;G?000000040<2fd003
00000031]m00000000H0`KSA00<000000<:jd`32^]<01032^]<00`000000`[cD0<:le0040<:le003
00000033_]D0`kkE00@0`kkE00<000000<>oe`33_mL00`33_mL00`000000`l3G000000040<?1f003
00000033`MP0000000@0a<;I00<000000<?3f@0000001@34`mT00`000000a<GJ0<C5fP030<C5fP03
00000034a]/0a<KK00@0a<KK00<000000<C6g034a]`01034a]`00`000000a<OL0<C7g0040<C7g003
00000034b=d0a<SM00<0a<SM00<000000<C9g@0000000`34bMh00`000000a<WN000000040<C9gP03
00000034b]h0000000<0a<[O0P0000000`34bml000000<C:g`02000000<0a<_O00@000000<?;h000
000000000P34bn0010000000`l_P0000000000030<?;h00400000033bn000000000000@0`lcQ00<0
00000<?;h@33bn400`33bn400`000000`l_Q0<?;h@040<?;h@0300000032bn40`/_Q00<0`/_Q00<0
00000<;;h@32bn400`32bn42000000@0`L_Q00<000000<7;hP0000000`31b^42000000030<7:hP00
0000`L[Q0080`L[Q00<000000<3:hP000000:03oool00`000000oooo0?ooo`0e0?ooo`00:03oool3
000001T0oooo0P0000040:k1j0030000002^`^P0[/;X008000000P2_`^P00`000000[l?X0:o3j003
0:o3j0800000102`a>T00`000000/<GY0;35j@02000000030;75j@00000000000080/LGY0P000000
0`2ba^T000000;;6j@040;;6j@<000001@2banT300000080/lSY0P0000020;?9jP80000000<0]<WZ
0000000000000P2db^X2000000030;G:jP00000000000080]L_Z0P0000000`2fbnX000000;K<j`04
0;K<j`8000001@2gcN/2000000H0^<k[00<000000;W?j`2icn/00`2icn/010000000^/oZ00000000
00020;[@j`80000000<0^m3Z0000000000000P2kdN/2000000040;cAjP00000000000;cBj`800000
102md^X2000000@0_]CZ0`0000040<3Ej@0500000031e>P0000000000032e^T00P0000040<?Fj003
00000034e^L0a=KW00<0a=KW00@000000<GFiP000000a]OV0P0000040<OFi@8000000P39en@20000
00@0bMGS00<000000<_Eh@3;eN400`3;eN4010000000c=CP0000000000050<gCgP060000003=d]d0
00000000003>d]d00000103>dM`010000000cLoK0000003>d=X2000000@0c/kI00<000000<k=f03>
cMP00P3>cMP2000000040<k<f0000000clcG000000D0c/_F00D000000<k9e@3>bMD0c/WE00000003
0<k8e@030000003>b=@0000000<0c/OD00<000000<g6e00000000`3>a]<01P000000cLKC0<g6d`00
0000c/KC000000D0cLGB00D000000<g5dP3=aM80cLGB000000020<g4dP030000003=a=40000000D0
cLCA00<000000<k4d@3>a=400P3>a=400`000000c/?@000000040000000301e>a`000000=5C50080
=5C500@0000005]Xa`1KJ<L000000`1^K/800`000000OgBm07md_@0207md_@050000002=N[X0SGZj
08ej^P000000102HP[X00`000000WhVl09n9_00309n9_0040000002UT;h0YI2n0:F@_P8000001@2]
U[l00`000000/9_20;2K`P020;2K`P8000001P2fX/@3000000<0^ZO600<000000;ZZb@2jZ/T00P2j
Z/T00`000000_Jc9000000060;f^b`030000002o//d0_k;=00<0_k;=00@000000;jccP0000000000
0P30]Ll01@000000`;K@0<2fd030]]0000000080`KO@00<000000<6hd@31^=400P31^=4200000003
0<:jdP000000`[[C0080`[[C0`0000050<:le08000001@33_]D2000000H0`koG00<000000<?1f033
`MP01033`MP00`000000a<;I0<C2f@040<C2f@0300000034`mT0a<?I00<0a<?I0P0000040<C5fP03
00000034aM/0000000@0a<KK00<000000<G6f`0000000`34a]`2000000030<G7g0000000a<OL0080
a<OL0P0000001035b=d000000<C8g@34b=d200000080a<WM00@000000<C9gP00000000000`34bMh0
10000000a<WN0000000000020<C:gP0400000034b]l00000000000@0a<_O00<000000<C;g`34bml0
1034bml00`000000a<_P0<C;h0030<C;h00300000033bn00`l_P00@0`l_P00D000000<?<h@33c>40
00000<?<h@02000000030<?<h@000000`l_Q0080`l_Q00<000000<?<h@0000000`33bn4200000003
0<;<h@000000`/_Q0080`/_Q00<000000<;<hP0000000P32bn4200000080`/_R00@000000<7;h@00
000000000P31bn8010000000`L[Q0000000000030<7:hP0400000031b^40000000000080`<[R00<0
00000?ooo`3oool09`3oool00`000000oooo0?ooo`0d0?ooo`009`3oool2000001`0oooo00<00000
0:k1j02^`NP00`2^`NP2000000D0[l;X00<000000:o3j02_`nP00P0000000`2``nP000000;34j@04
0;34j@8000001@2aaNT2000000@0//KY00<000000;;6j@2ba^T00P0000020;;6j@8000000P2banT2
000000030;?7j@000000/lSY00@0/lSY0P0000050;C9jP8000001@2eb^X2000000@0]/_Z00<00000
0;K<j`2fc>/00P0000020;O<jP030000002gcN/0]lg[008000000P2hcNX200000080^<k[0P000000
102ic^X000000000002icn/2000000@0^/oZ0P0000050;_@jP8000001@2ldNX00`000000_M;Z0;gB
jP030;gBjP050000002nd^T000000000002ne>X00`0000001030dnT0000000000030eNT2000000@0
`MCX00<000000<?Fj033e^P00`33e^P01@000000a=GW000000000000a=KW008000000`35e^H20000
00<0amKU0P0000020<SEi08000001@39eN<01@000000b]CQ000000000000bmGQ008000000P3<e>00
1@000000c=;O000000000000cM?N00800000103=d]d010000000c]7L0<kAg03>dM`2000000<0cLoK
00<000000<k>f@3>c]T00`3>c]T010000000c/gI0000003>cMP2000000@0c/cH00<000000<k;eP3>
bmH00P3>bmH2000000040<k9eP000000c/WE000000D0c/SE00D000000<k7e03>am@0c/OD00000003
0<g6e0030000003>a]<0000000@0cLKC00<000000<g5dP3=aM800P3=aM82000000040<g4dP000000
cLGB000000@0cLCB00<000000<g4d@3=a=400`3=a=4010000000cL?A0000003>a=4200000080c/?@
2000000203ADa@80000000<0FfS70000001^K/800`1^K/801@000000OgBm07md_@1oM;d0000000@0
SGZj00<0000009R2^P2HP[X00P2HP[X00`000000WhVl09n9_00309n9_0<000000P2YT;/00`000000
[IJo0:fF_`030:fF_`040000002`Vl80/9_200000080]9k100<000000;JRa02fX/@00`2fX/@00`00
0000^ZO60;ZWaP040;ZWaP040000002jZ/T0^Z[900000080_Jc900<000000;f^b`2m[//00`2m[//2
000000H0_k;=00<000000<2ec`30]Ll00`30]Ll00`000000`;K@000000040<6gd00400000031^=40
0000000000<0`[[B00@000000<:jd`00000000000`33_=<00`000000`[cD0<:le00200000080`kgD
00<000000<>ne@33_]D00P0000020<BoeP0400000033_mL0`koG0<>oe`80000000<0a<7G00000033
`MP00P33`MP2000000030<C2f0000000a<;I0080a<;I0P0000001034`mT000000<C3f@34`mT20000
0080a<CJ00@000000<C5fP34aMX000000`34aM/010000000a<KK0000000000020<G6f`0400000034
a]`00000000000<0aLOL00@000000<C7g000000000000`35b=d00`000000a<SM000000050<C9g@03
00000034bMh0a<WN00@0a<WN00<000000<C:gP34b]h00`34b]h00`000000a<_O0<C;g`030<C;g`03
00000034bml0000000<0a<_O0P0000000`34bn0000000<C;h0020<C;h00300000034bn00000000<0
`l_P0P0000000`33c>4000000<?<h@02000000<0`lcQ00@000000<?;h@00000000000P33c>401000
0000`l_Q0000000000030<;<h@0400000032bn400000000000<0`/cR0P0000050<;;hP0300000031
bn80`L_R00<0`L_R00<000000<7:hP31b^801031b^800`000000`<[R0<3:hP030<3:hP030000003o
ool0oooo02L0oooo00<000000?ooo`3oool0<`3oool002L0oooo00<000000?ooo`3oool0703oool0
0`000000[/7X0:k1j002000000030:o2j0000000[l;X00D0[l;X0P0000040;33j0030000002`a>T0
/<CY0080000000<0/LCX0000000000000P2aaNT2000000030;;5j0000000//KY00@0//KY0P000005
0;;6j@<00000102canT00`000000/lSY0;?8j@02000000030;C8j@00000000000080]<WZ0P000000
0`2ebNT000000;G:jP030;G:jP030000002fb^T0000000D0]/_Z0P0000050;O<jP8000001@2hcNX3
000000@0^LkZ0P0000050;[?jP030000002kcnT0000000@0^m3Z00@000000;c@j@00000000000P2l
dNX2000000040;gAj@00000000000;gBjP8000000P2nd^T4000000D0`=?Y0P0000030<7Dj0800000
00@0`]CW000000000000`mKX0P0000030<CEi`8000001035e^H00`000000a]GU0<KEi@03000000@0
b=GT00D000000<SDh`00000000000<WEh`02000000@0b]CQ00D000000<cDh03<e>00c=CP00000004
0<cBg`030000003=d]d0cM;M00<0cM;M00D000000<g@g0000000c]7L000000040<g?f`030000003=
c]X0000000<0c/kI00D000000<k=f@3>cMT0c/gI000000050<k<f0040000003>bmL000000<k;eP80
0000103>bMH00`000000c/SE0<k8e@030<k8e@050000003=am@000000<k7e00000001@3=a]@00`00
0000cLKC0<g6d`020<g6d`040000003=aM<000000<g5dP800000103=a=800`000000cLCB0<g4dP02
0<g4dP030000003=`m40000000<0cLCA00D000000<g3d@3=`m40cL?A000000040<k3d0H0000000<0
03^m0000000dE<D00P0dE<D4000000<0KVk200D0000007md_@1oM;d0OgBm0000000408ej^P030000
002HP[X0V8:j0080V8:j00<0000009n9_02ORK`00P2ORK`2000000@0ZI2k00<000000:fF_`2]U[l0
0`2]U[l00`000000/9_2000000030;BN`@030000002fX/@0]Z;40080]Z;40P0000060;ZWaP030000
002jZ/T0000000@0_Jc900<000000;f^b`2m[//00P0000020;n`b`030000002o//d0_k;=0080_k;=
0P0000060<2ec`0300000031]m00`KO@00@0`KO@00<000000<:jdP32^]801032^]800`000000`kcC
0<>ld`030<>ld`0300000032_=@0000000@0`kgD00<000000<>ne@0000001034_mH010000000`koG
0000000000030<C1e`0400000033`MP00000000000<0a<;H00@000000<C2f@00000000000`34`mT0
0`000000a<?I000000040<C4fP0300000034aMX0000000D0a<GK00<000000<G6f`35a]/00`35a]/0
0`000000aLOL0<G7g0040<G7g00300000035b=d0aLSM00@0aLSM00<000000<C9g@34bMd00P34bMd2
000000030<G9gP000000a<WN0080a<WN0P0000000`35b]h000000<C:gP020<C:gP0300000034bml0
00000080a<_O0P0000020<C;g`0400000034bml00000000000<0a<_P00@000000<C;h00000000000
0P34bn0010000000`l_P0000000000040<?<h@0400000033c>40`lcQ000000<0`lcQ00<000000<?<
h@33c>400`33c>400`000000`/cQ0<;<h@030<;<h@8000001032c>800`000000`/cR000000030<;;
hP80000000<0`L_R00000031bn800P31bn800`000000`L_R000000030<7:hP80000000<0`L[R0000
0030b^800P30b^800`000000`<[R0000000X0?ooo`030000003oool0oooo03<0oooo000W0?ooo`03
0000003oool0oooo01d0oooo0P0000040:o2j08000000P2_`^P2000000030;32j0000000000000<0
/<?X0`0000050;74j0800000102baNP00`000000//KY0;;6j@02000000030;;6j000000000000080
//KY0P0000020;?7j0030000002canT0/lOY00<0/lOY0P0000050;C8j@800000102ebNT010000000
]L[Z0000000000030;K:j@030000002fbnX0]/_Z0080000000<0]l_Y0000000000000P2gc>X20000
00030;S<j@00000000000080^<gZ0P0000020;W=j@030000002ic^X0^LkZ008000000P2jc^T20000
00030;[?jP000000000000<0^loY00@000000;_@jP0000000000102ld>T2000000D0_M7Y00<00000
0;kBj@0000000P0000000`2nd^T000000;oBj00200000080`=?Y0P0000020<7Cj0<000001032e>L2
000000<0a=GW0P0000020<GDiP0400000035e^H00000000000<0a]GU00D000000<ODi00000000000
0<SEi002000000@0b=CS00<000000<[Dh@3:e>400P3:e>42000000040<[Ch0000000c=CP000000@0
c=;O00D000000<cAgP00000000000<gBg@02000000<0cM3L00<000000<g?f`3=cm/00P3=cm/01`00
0000cLkJ0<g>fP3=c]X000000<k>f@0000000`3>cMT01@000000cLcH000000000000c/cH00800000
0`3>bmL00`000000c/WF0<k9eP030<k9eP050000003=b=H000000000003>b=D00P0000030<g7e003
0000003=a]@0cLKD0080cLKD0P0000000`3=a]<000000<g6d`02000000<0cLGC00<000000<g4dP3=
a=800`3=a=800`000000cLCB000000020<g4dP070000003=`m40cL?A0<g3d@000000cLCA00000003
0<g3d@030000003=`m0000000080c/?@00<000000<k3d03>`m0010000002000k_@040000000dE<D0
=5C500000080Be_200D0000006i^`P1^K/80KVk20000000407md_@050000002=N[X0SGZj08ej^P00
00001@2HP[X01P000000WhVl000000000000Y8Rg000000D0ZI2k00<000000:fF_`2]U[l00`2]U[l0
0`000000]9k10;BN`@030;BN`@040000002fX/@0]Z;400000080^ZG400<000000;ZWaP2jYlH0102j
YlH00`000000_Jc90;f/b@030;f/b@030000002m[//0000000@0_k3;00<000000;nbc@2o//d00P00
00020<6dc@0400000030]Ll0`;G?0<2ec`<000001@31]m02000000D0`[[B0P0000060<>ld`030000
0033_M@0`kgD00@0`kgD00<000000<BoeP34_mH01034_mH00`000000a<7G0<C1e`040<C1e`030000
0034`]P0a<;H00@0a<;H00<000000<C3f@34`mT01034`mT00`000000a<CJ0<C4fP040<C4fP030000
0034aM/0a<GK00<0a<GK00<000000<G6f`0000000`35a]/00`000000aLOL000000040<G7g0030000
0035b=`0000000<0aLSM0P0000000`35bMd000000<C9g@02000000<0aLWN00@000000<C9gP000000
00000`35b]h010000000a<[N0000000000020<C;g`0300000034bml0000000D0a<_O00<000000<C;
h034bn001034bn000`000000a<_P0<C;h0030<C;h00300000033c>40`lcQ0080`lcQ0P0000001@33
c>4000000<?<h@000000`lcQ0080000000<0`lcQ00000033c>400P33c>400`000000`lcQ00000002
0<;<h@8000000P32c>8010000000`/cR0000000000020<;<hP0400000032bn800000000000<0`L_R
00@000000<7;hP00000000000P31bn8010000000`L[R0000000000030<7:hP0400000030b^800000
00000080`<[R00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`0b0?ooo`009`3oool00`00
0000oooo0?ooo`0N0?ooo`8000000`2_`^P4000000D0/<;X0`0000020;73j08000000P2aa>P20000
00030;74j0000000//GX00@0//GX0P0000050;;6j08000001@2canP00`000000/lOY0;?7j@020000
00030;C7j@00000000000080]<SY0P0000000`2eb>T000000;G9j@040;G9j@030000002fb^T0]/[Y
00@0]/[Y0P0000050;O;j@8000001@2hc>T2000000D0^LgY0P0000050;[>j@8000001@2kcnT20000
00D0_=3Y00@000000;g@j@00000000000P2mdNT5000000030;kBj@000000000000@0_m;X0P000003
0<7Cj080000000@0`]?W00000032e>L0`]CW0P0000020<?DiP<000001@35e>H200000080a]GU0P00
00040<ODi00300000038e><0b=CS00<0b=CS00@000000<WChP000000b]CQ0P0000040<[Ch0800000
0P3<d]l2000000@0c=7N00<000000<g@g03=d=`00P3=d=`01P000000cLoK000000000000cLoK0000
00D0cLkJ00D000000<k=f@3>cMT0c/gI000000040<g<f0030000003>bmL0c/_G0080c/_G00D00000
0<g9e`00000000000<k9eP02000000@0cLSF00<000000<g7e03=am@00P3=am@010000000cLKD0000
003=a]@2000000@0cLKC00<000000<g5d`3=aM<00P3=aM<00`000000cLCB000000030<g4dP060000
003=a=80cLCB0000003=a=8000001@3=`m401@000000cL?A0<g3d@3=`m4000000080cL?@00<00000
0<k3d0000000103>`m0200000003000NZ`00000003^m0080000000@0=5C50000001;Fl80Be_20`00
00000`1^K/80000007md_@0307md_@050000002=N[X0SGZj08ej^P000000102HP[X3000000@0Y8Rg
00<000000:V@^`2YT;/00P2YT;/010000000[IJo0:fF_`2]U[l3000000D0]9k100<000000;JRa000
00000`2jYL@00`000000^ZO60;ZWaP030;ZWaP8000001P2m[<T00`000000_k3;0;n`b`040;n`b`80
00001031]<d010000000`;G?0000000000030<:gc`0300000031]m00`KO@008000000P33^M400`00
0000`[[B0<:jdP0200000080`k_B00@000000<>ld`33_=<0`kcC0P0000000`34_M@000000<>me002
0<>me080000000<0a;oE00000034_mH00P34_mH2000000030<C0eP000000a<7G0080a<7G0P000000
0`35`]L000000<C2f0020<C2f080000000<0aL?H00000034`mT00P34`mT2000000030<G4f@000000
a<CJ0080a<CJ0P0000001035aMX000000<C5f`34aM/200000080aLKK00@000000<G6f`0000000000
0`35am`010000000aLOL0000000000020<G8g00400000035b=d00000000000@0aLWM00<000000<G9
gP35bMh01035bMh00`000000aL[N0<G:gP040<G:gP0300000034bml0a<_O00<0a<_O00<000000<C;
g`34bml00P34bml2000000030<C;h0000000a<_P0080a<_P0P0000000`34c>0000000<C;h0020<C;
h00400000034c>4000000<?<h@8000000`33c>4010000000`lcQ0000000000030<?<h@0400000033
c>400000000000<0`lcQ0P0000050<;<hP0300000032c>80`/cR00<0`/cR00<000000<7;hP31bn80
1031bn800`000000`L_R0<7;hP030<7;hP0300000031b^80`L[R00<0`L[R0P0000040<3:hP800000
:@3oool00`000000oooo0?ooo`0a0?ooo`009`3oool00`000000oooo0?ooo`0P0?ooo`<000000`2`
`^L200000080/<;X0P0000000`2a`nL000000;73j0040;73j0800000102aa>P00`000000//GX0;;5
j002000000030;;5j000000000000080//KX0P0000000`2ca^P000000;?7j0040;?7j0<000001@2d
anT2000000@0]LSY00<000000;G9j@2ebNT00P0000000`2fbNT00000000000020;K:j@80000000<0
]l[Y0000000000000P2gbnT2000000030;S;j@00000000000080^<cY0P0000000`2ic>T000000000
00020;W=j@80000000<0^/gY0000000000000P2jc^T200000080^lkY00<000000;_?j@2kcnT00P00
00020;c?j@80000000<0_=3Y000000000000102md>T3000000<0_]7Y0P0000040;oBj080000000<0
`=;W00000031dnP00P0000040<;Ci`8000001033e>H010000000a=?U0000000000020<GDiP800000
0P35e>D2000000D0amCT00D000000<SCh`00000000000<SDh`02000000<0bM?R00<000000<[Ch03:
dn000`3:dn000`000000bm;O0<_Bg`02000000@0c=7N0P0000000`3<d=d000000<g@g002000000@0
cLoK00<000000<g>fP3=c]X00P3=c]X2000000040<g=f@000000c/gI000000@0cLcH00<000000<g;
f00000000P3>bmL00`000000cLWG0<g9e`020<g9e`030000003=b=H0cLSF00<0cLSF00<000000<g7
e@0000000P3=am@01@000000cLKD0<g6e03=a]@0000000D0cLKC00<000000<c5d`0000000P3=aM<0
1`000000cLCB0<g4dP3=a=8000000<g4dP000000103=a=800`000000cL?A0<g3d@030<g3d@050000
003=`m4000000<g3d@000000103=`m000`000000c/?@0<k3d0030<k3d0D0000000@003^m00000000
000000000`1;Fl800`000000I6>l00000002000000<0OgBm00<0000008ej^P2=N[X00P2=N[X01P00
0000V8:j000000000000Vgnc000000D0Y8Rg00<000000:V@^`2YT;/00`2YT;/3000000<0/YVm00<0
00000;BN`@2dW/400`2dW/400`000000^ZG40;ZUa0030;ZUa0030000002jYlH0^ZO6008000000P2m
ZlL00`000000_Jc90;f/b@020;f/b@8000001P2o/</00`000000`KC=0<6dc@040<6dc@0300000032
]ll0`[O?00<0`[O?00<000000<6gd00000001033^M400`000000`[[B000000040<>kdP0400000033
_=<00000000000<0a;gD00@000000<>me000000000000`34_mD010000000a;oF0000000000030<C0
eP0400000034`ML00000000000<0aL;G00@000000<C2f000000000000`35`mP010000000a<?I0000
000000030<G4f@0400000034a=X00000000000<0aLGJ00<000000<C5f`0000001@35a]/00`000000
aLOL0<G7g0040<G7g00300000035b=`0aLSL00<0aLSL00<000000<G9g@35bMd00`35bMd00`000000
aLWM000000030<G9gP80000000<0aL[N00000035b]h00P35b]h2000000030<G;gP000000a<_O0080
a<_O00@000000<G;g`000000a<_O0P0000030<C;h00400000034bn000000000000<0a<cP00@00000
0<C;h000000000000`34c>400`000000`lcQ0<?<h@040<?<h@0400000033c>40`lcQ000000<0`lcQ
00@000000<?<h@33c>40`lcQ0P0000000`33c>8000000<;<hP020<;<hP80000000<0`/cR00000032
c>800P32c>800`000000`/cR000000030<7;hP80000000<0`L_R00000031bn800P31bn800`000000
`L_R000000020<7:hP80000000@0`<[R00000030b^80`<[R0P0000020;o9hP030000003oool0oooo
02H0oooo00<000000?ooo`3oool0<@3oool002L0oooo00<000000?ooo`3oool08@3oool00`000000
/<;W0;32i`030;32i`<00000102a`nL00`000000/L?X0;73j002000000030;73i`000000/LCX00@0
/LCX0P0000050;;5j0800000102ca^P00`000000/lOX0;?7j00200000080]<OX0P0000020;C7j@80
000000<0]LSX0000002eb>T0102eb>T2000000D0]/WY0P0000050;O:j@8000001@2hbnT2000000D0
^LcY0P0000050;[=j@8000001@2kc^T2000000D0_<oY0P0000030;g@j@<0000000@0_]3X0000002n
dNT0_]7Y0P0000020;oAj0040000002od^P00000000000@0`=;W00<000000<;Ci`32dnL00`32dnL0
1@000000`m;V000000000000`mCV008000001034dnD2000000@0aMCU00D000000<KCi00000000000
0<ODi002000000@0b=?S00<000000<WChP39dn800P39dn801@000000bM;Q000000000000b]?P0080
00000`3;d]l010000000bm3N0000003<dMh2000000@0c=3M00<000000<g?f`3=cm/00`3=cm/01000
0000c<kK0000003=c]X2000000@0cLgI00<000000<g<f03=c=P00P3=c=P01P000000cL_H0<g;f000
0000c/_G000000@0cLWG00<000000<g8eP0000000P3=b=H200000080cLOE00<000000<g7e0000000
0`3=a]@00`000000c<KD000000030<g6d`060000003<aM<0c<GC0000003=aM<000001@3=a=800`00
0000cLCB0<g4dP020<g4dP050000003=`m4000000000003=`m400P0000030<g3d@030000003=`m00
cL?@0080cL?@00<000000<g2d00000000`3>`m000`000000c/;?000000050000000301I1_@000000
00000080Be_200@0000006AS_01THk`0I6>l0`00000207md_@030000002=N[X0SGZj00<00000102K
Ok<00`000000Y8Rg0:B8]`020:B8]`030000002YT;/0ZI2k00<0ZI2k00<000000;:I_@2bVKd00`2b
VKd00`000000]9k10;BN`@020;BN`@8000001@2jYL@00`000000^ZO6000000040;f[a`030000002m
[<T0_Jc9008000000P30[lT010000000_k3;0;n`b`2o/</3000000D0`KC=00<000000<>fcP000000
1@32]ll00`000000`kWA0<>id@040<>id@0300000033^m80`k_B00@0`k_B00<000000<Bme034_M@0
1034_M@00`000000a;oE0<Boe@040<Boe@0300000034`=H0a<3F00@0a<3F00<000000<G2e`35`]L0
1035`]L00`000000aL?H0<G3f0040<G3f00300000035a=T0aLCI00@0aLCI00<000000<G5fP35aMX0
1035aMX00`000000aLKK0<G6f`030<G6f`0300000035am/0000000<0aLOL0P0000000`35b=`00000
0<G8g0020<G8g00300000035bMd0000000<0aLWM00D000000<G9g@35bMd000000<G9gP02000000<0
aL[N00@000000<G:gP00000000000`35bmh010000000a<_O0000000000030<G;g`0300000034bn00
a<_P00@0a<_P00<000000<C<h034c>001034c>000`000000a<cQ0<C<h@020<C<h@0300000034c>40
000000<0`lcQ0P0000001@33c>4000000<?<h@000000`lcQ00<0000000<0`lcQ0000000000000`33
c>8010000000`/cR0000000000030<;<hP0400000032c>80000000000080`/cR00@000000<7;hP00
000000000`31bn8010000000`L_R0000000000030<7;hP8000001030b^82000000D0_lWR00<00000
0?ooo`3oool09P3oool00`000000oooo0?ooo`0`0?ooo`009P3oool00`000000oooo0?ooo`0S0?oo
o`030000002``^L0/<;W008000000P2``^L00`000000/L?W0;73i`030;73i`800000102a`nL00`00
0000/LCX0;74j002000000030;;4i`00000000000080//GX0P0000000`2caNL000000;?6j0040;?6
j08000001@2danP3000000@0]LSX00<000000;G8j@2eb>T00P0000000`2fbNP00000000000020;K9
j@80000000<0]lWX0000000000000P2gb^T2000000030;S;j000000000000080^<_Y0P0000000`2i
bnP00000000000020;W<j@80000000<0^LcX0000000000000P2jcNT2000000030;[=j00000000000
0080^lkY0P0000000`2kc^P00000000000020;c?j@8000000P2lcnP4000000@0_]3X0P0000050;oA
j00300000030d^L0`=;W00<0`=;W00D000000<7Bi`00000000000<;Ci`02000000@0`m;V00<00000
0<CCi@34dnD00`34dnD01@000000aM;U000000000000aMCU008000001036dn@00`000000b=?S0<SC
h`020<SCh`80000000<0b=;R00000039dn800P0000040<WBh@030000003;d]l0bm;O0080bm;O00D0
00000<_@gP3;d=h0bm3N000000050<c@g@050000003<cm`000000000003=cm/00P0000030<c>f`03
0000003=cMT0cLgI00<0cLgI00H000000<g<f@00000000000<g<f00000040<g;f0030000003=bML0
cLWG0080cLWG00H000000<g8eP3=b=H000000<g8eP0000050<g7e@050000003=a]@0cLKD0<g6e000
00000`3<a]@00`000000cLKC000000040<c5d`030000003=a=80cLCB00<0cLCB00@000000<c4dP00
0000cLCB0P0000040<g3d@030000003=`m40cL?A0080cL?A00<000000<g2d00000000P3=`m001`00
0000cL;@0<g2d03=`]0000000<k3d00000000`3>`/l5000000805T6m00@0000004]K`P1;Fl800000
0`1THk`00`000000Nfbf07]/]P050000000308me/@000000Vgnc00@0Vgnc00<000000:B8]`2TR;L0
0P2TR;L010000000ZI2k0:V@^`2YT;/3000000D0/YVm00<000000;BN`@2dW/400P0000020;VR`@03
0000002jYL@0^ZG400<0^ZG400<000000;f[a`2mZlL0102mZlL2000000@0`:o900@000000;n`b`00
000000000`32/l`00`000000`KC=0<6dc@0200000080`kK>00<000000<:gc`32]ll00P32]ll00`00
0000`kS@000000040<>id@0300000034^m40000000@0`k_B00<000000<Bmd`0000001034_M@00`00
0000aKkD000000040<Boe@0300000035`=D0000000@0a<3F00<000000<G1e`0000001035`]L00`00
0000aL?H000000040<G3f00300000035a=T0000000<0aLCI0P0000000`35aMT000000<G5fP020<G5
fP80000000@0aLKJ00000035a]/0aLKK0P0000020<G7f`0400000035am`00000000000<0aLSL00@0
00000<G8g000000000000`35bMd00`000000aLWM000000040<G9g@0300000035b]h0aL[N00@0aL[N
00<000000<G;gP35bmh01035bmh00`000000aL_O0<G;g`020<G;g`0300000035c=l0000000@0a<_P
00<000000<C<h00000000`34c>02000000030<C<h@000000a<cQ008000000P34c>4010000000`lcQ
0000000000030<?<h@0400000033c>400000000000<0`lcR00<000000<?<hP33c>801033c>800`00
0000`/cR0<;<hP040<;<hP0300000032c>80`/cR0080`/cR0P0000050<7;hP8000000`31bn820000
00030<3;hP000000`<[R0080`<[R00<000000<3:hP0000000`2obN82000000030;o9hP000000oooo
02H0oooo00<000000?ooo`3oool0<03oool002H0oooo00@000000?ooo`00000000008`3oool20000
00D0/<;W00<000000;73i`2a`nL00P0000000`2a`nL000000;73i`040;73i`8000001@2ba>L20000
00@0/lGW00<000000;?6j02ca^P00P0000000`2da^P00000000000020;C7j08000000P2eanP00`00
0000]LSX0;G8j0030;G8j08000001@2fbNP2000000D0]lWX0P0000050;S;j08000001@2ibnP20000
00D0^LcX0P0000050;[=j08000001@2kc^P2000000060;c?j000000000000000002lcnP000001@2n
d>P010000000_m3W0000000000020;oAj080000000@0`=7W000000000000`=;W0P0000040<7Bi`03
00000033d^H0`m;V00<0`m;V00D000000<?Bi@00000000000<CCi@02000000@0aM;U00<000000<KC
i036dn@00P36dn@2000000030<OBh`000000b=?S008000001038d^800`000000bM;Q0<WBh@030<WB
h@060000003:dMl000000000003;d]l000000`3;d=h01@000000bloM000000000000c=3M00800000
103<cm`00`000000c<kK0<c>f`020<c>f`050000003<cMX000000000003=cMT00P0000040<g<f@03
0000003=bmP0cL_H0080cL_H00H000000<g9e`00000000000<g9e`0000040<g8eP030000003=amD0
cLOE00<0cLOE00D000000<c7e0000000cLKD000000050<c6e0030000003<aM<0c<GC0080c<GC00D0
00000<c4d`00000000000<g4dP02000000<0c<CB00<000000<g3d@3=`m400`3=`m400`000000c<;A
000000020<g3d@060000003=`]00cL;@0000003=`m0000001@3=`]001P000000c/;?0<k2c`3>`/l0
00000<k2cPD0000000<05T6m0000000000000P1;Fl801@000000I6>l06AS_01THk`0000000<0Nfbf
00<0000008me/@2?MK400P2?MK400`000000Vgnc09]o/`0209]o/`030000002TR;L0Y8Rg00<0Y8Rg
0`0000030:nB^@030000002bVKd0/YVm00<0/YVm0P0000040;VR`@030000002jYL@0^ZG40080^ZG4
0P0000060;f[a`0300000030[lT0`:o900@0`:o900<000000<:cc032/l`00`32/l`00`000000`KC=
000000050<>fcP0400000032]ll0000000000080`kS@00D000000<>id@33^M40`kWA000000020<Bk
d@0500000033^m80`k_B0<>kdP0000000P34_M<01@000000a;gD0<Bme034_M@000000080aKkD00D0
00000<Boe@34_mD0a;oE000000020<G0e@0500000034`=H0a<3F0<C0eP0000000P35`ML01@000000
aL;G0<G2e`35`]L0000000<0aL?H00@000000<G3f000000000000P35a=T010000000aLCI00000000
00030<G5f@0400000035aMX00000000000<0aLKJ00<000000<G6f`0000001@35am/00`000000aLSL
0<G8g0040<G8g00300000035bMd0aLWM00@0aLWM00<000000<G9g@35bMd00P35bMd00`000000aL[N
000000030<G:gP80000000<0aL_N00000035bmh00P35bmh2000000030<G;g`000000aL_O00800000
0`35c=l010000000a<_P0000000000020<C<h00400000034c>000000000000@0a<cQ00<000000<C<
h@34c>400`34c>400`000000`lcQ0<?<h@040<?<h@0800000033c>80`lcR00000033c>8000000<?=
hP0000030<?<hP80000000<0`/cR00000032c>800P32c>8300000080`/cR0P0000020<7<hP030000
0031bn80`L_R0080000000<0`L_R00000031bn800P0000030<3;hP0400000030b^80000000000080
`<[R00@000000;o9hP00000000000`2obN800`000000oooo0?ooo`0S0?ooo`@00000<@3oool002H0
oooo0P00000V0?ooo`030000002``^L0/<;W00@0/<;W0P0000040;73i`030000002a`nL0/L?W0080
000000<0//CW0000000000000P2ba>L2000000030;?4i`000000/lGW00@0/lGW0P0000050;C6j080
00001@2eanP00`000000]LSX0;G8j002000000030;K8j000000000000080]/WX0P0000000`2gbNP0
0000000000020;O9j080000000<0^<[X0000000000000P2hbnP2000000030;S;j000000000000080
^L_X0P0000000`2ic>P00000000000020;W<j080000000<0^/cX0000000000000P2jcNP200000003
0;_=j000000000000080^lkX0P0000000`2lc^P00000000000020;c?j080000000@0_LoX00000000
0000_]3X0P0000040;o@i`8000001@30dNL2000000<0`M;W0P0000001032dNH0000000000033d^H2
000000@0`m;U00<000000<GBi@35d^D00`35d^D010000000aM;T00000036dn@2000000@0am;S00<0
00000<SBhP38d^800`38d^801@000000b=7Q000000000000bM;Q00800000103:dMl01@000000bm3N
0<_@gP3;d=h0000000@0bloM00<000000<c?g03<cm`00`3<cm`010000000c<gK0000003<c]/20000
00@0c<gJ00<000000<g<f@3=c=T00`3=c=T010000000c<_H0000003=bmP2000000@0cLWG00<00000
0<g8eP3=b=H00P3=b=H01@000000c<OF000000000000cLOE008000000`3<am@00`000000c<KD0<c6
e0030<c6e0040000003<aM<000000<c5d`800000103<a=<00`000000c<CB0<c4dP020<c4dP030000
003<`m4000000080cL?A0P0000020<c2d@030000003=`m40000000@0cL;@00<000000<g2d03=`]00
0`3=`]001@000000cL7?0000003>`/l0000000<0c/;>0P00000010008Zh0000001I1_@0F@Kd40000
00<0I6>l00D0000007]/]P1kK;H0Nfbf0000000408me/@030000002KOk<0Vgnc00<0Vgnc00<00000
0:B8]`2TR;L00P2TR;L00`000000[i:i0:nB^@030:nB^@030000002bVKd0/YVm00<0/YVm00<00000
0;VR`@2iX/400`2iX/400`000000^ZG40;ZUa00200000080_ZS400<000000;f[a`2mZlL00P2mZlL0
0`000000`Jk7000000040<2_b@0300000032//X0000000D0`[?<00<000000<>fcP33]/h01@33]/h0
0`000000`kS@0<>hd0030<>hd00300000033^M40000000@0a;_A00<000000<>kdP0000001034_M<0
0`000000a;gD000000040<Fne00300000034_mD0000000@0aL3E00<000000<C0eP0000001035`ML0
0`000000aL;G000000050<G3f00300000035a=T0aLCI00<0aLCI00<000000<G5f@35aMT01035aMT0
0`000000aLKJ0<G6fP040<G6fP0300000035am/0aLOK00<0aLOK00<000000<K8g00000000`35b=`2
000000030<K9g0000000aLWM0080aLWM0P0000000`35bMd000000<G9g@0200000080aL[N00@00000
0<G:gP00000000000`35bmh010000000aL_N0000000000040<G;g`0300000035c=l0aLcO00@0aLcO
00<000000<C<h034c>000`34c>000`000000a<cQ0<C<h@020<C<h@80000000<0a<gQ00000034c>40
0P34c>400`000000a<gQ000000030<?<h@80000000<0`lgR00000033c>800P0000020<?=hP040000
0033c>800000000000<0`/cR00@000000<;<hP00000000000`32c>82000000D0`LcR0P0000040<7;
hP0300000030bn80`<_R00@0`<_R00<000000<3:hP30b^800`30b^800`000000_lWR0;o9hP040;o9
hP030000003oool0oooo02H0oooo00<000000?ooo`3oool04@3oool01P2oM0000000000000000000
04RL0=ooo`<0oooo00D0omnL04P000000000001d0;ooo`040?ooo`050?oOW018000000000000M02o
ool01P3oool002H0oooo00<000000?ooo`3oool09P3oool200000080/<;W0P0000000`2a`^H00000
000000030;73i`<000001@2ba>L2000000@0/lCW00<000000;?5i`2caNL00P0000000`2daNL00000
000000020;C6j080000000<0]LKW0000002eanP0102eanP3000000@0]/SX0`0000050;O9j0800000
1@2hb^P2000000D0^<_X0P0000050;W<j08000001@2jc>P2000000D0^lgX1@0000020;c>j0800000
1@2mcnP00`000000_m3W0;o@i`030;o@i`040000002od>L0000000000080`=7W0P0000020<7AiP<0
00001@32dNH00`000000`m;U0<?Bi@030<?Bi@0500000034dN@0000000000035d^D00P0000030<GB
i00300000037d^<0am;S00<0am;S00D000000<OAhP00000000000<SBhP02000000@0b=7Q00<00000
0<[Ag`3:dMl00P3:dMl2000000040<[@gP000000bm3N000000@0bloM00D000000<_>g00000000000
0<c?g002000000<0c<gK00<000000<c=fP3<cMX00`3<cMX01@000000c<cI000000000000cLcI0080
00000`3<bmP00`000000cLWG0<g9e`030<g9e`040000003<b=L000000<g8eP800000103<amH00`00
0000c<OD0<c7e0020<c7e0050000003<a]@000000000003<a]@00P0000030<c5d`030000003<a=<0
c<CC00<0c<CC00<000000<c3dP0000000P3<a=801P000000c<?A0<c3d@000000cL?A000000D0c<;A
00<000000<g2d03=`]000P3=`]001@000000cL;@000000000000cL;@008000000`3=`Ll00`000000
c/;>0<k2cP020<k2cPD0000000@05T6m0000000`B;/0<4Rk0P00000206AS_0030000001kK;H0Nfbf
0080Nfbf00D0000008me/@2?MK40SgFa0000000509]o/`040000002TR;L0Y8Rg0:B8]`8000001@2_
T[T00`000000/YVm0;:I_@020;:I_@8000001P2iX/42000000@0_ZS400<000000;f[a`2mZlL00P00
00020<6^a`0500000030[lT0`:o90<2_b@0000000P32//X00`000000`[?<0<:cc0020<:cc0030000
0033]L`0000000D0`kK>0P0000060<>hd00300000034^m40a;_A00@0a;_A00<000000<Bmd`34_M<0
1034_M<00`000000aKkD0<Fne0040<Fne00300000035`=D0aL3E00@0aL3E00<000000<G1e`35`ML0
1035`ML00`000000aL?H0<G3f0030<G3f08000001035a=T00`000000a/GI000000040<G5f@030000
0036a]X0000000@0aLKJ00<000000<K7f`0000000P35am/200000080a/SL00@000000<G8g0000000
00000`36bM`010000000aLWM0000000000040<G9g@0300000035b]h0aL[N00<0aL[N00<000000<G;
gP35bmh01035bmh00`000000aL_O0<G;g`020<G;g`80000000<0aLcO00000035c=l00P35c=l20000
00030<G<h0000000a<cP0080a<cP00@000000<C<h0000000a<cQ0P0000030<C=h@0400000034c>40
000000000080a<gQ00@000000<?<h@00000000001033cN800`000000`lgR000000030<?=hP030000
0032c>80`/cR00<0`/cR0P0000040<;<hP0300000032c><0000000<0`LcR0P0000000`31c><00000
0<7;hP020<7;hP0300000031bn<0000000<0`<_R0P0000000`30bn<000000<3:hP020<3:hP030000
002ob^<0000000<0_lWR0P0000000`2obN8000000?ooo`0V0?ooo`030000003oool0oooo0180oooo
00<0omnL04P0001d_ol0103oool01`3ookl0M00007Boo`3oool0okmd0000M02oool00P3oool01`3o
okl0M00007Boo`3oool0okmd0000M02oool01@3oool002H0oooo00<000000?ooo`3oool0:03oool2
000000D0/L;V0`0000020;;3iP8000000P2ba>L2000000030;?4i`000000/lCW00@0/lCW0P000005
0;C5i`800000102ea^L00`000000]LOX0;G7j00200000080]/OW00<000000;K8j02fb>P00P000002
0;O8i`8000000P2gbNP2000000030;O9i`000000^<[X00<0^<[X00<000000;S:i`000000102hbnP0
0`000000^L_W000000040;W<j0030000002jc>L0000000@0^/cX00@000000;_<i`00000000000P2k
cNP500000080_<kX0P0000000`2mc^L00000000000020;g?j080000000@0_/oW000000000000_m3W
0P0000040;o@i`8000001031dNH010000000`]3U0000000000020<;AiP80000000@0`m7U00000000
0000`m;U0P0000040<CAi00300000035d^@0aM;T0080aM;T00D000000<KAh`00000000000<OBh`02
000000@0am7R00<000000<SAh@38dN400`38dN4010000000bM3O0000003:dMl2000000@0b]3N00<0
00000<_?g@3;cmd00P3;cmd00`000000blkL0<_>g0020<_>g0030000003<cM/0c<gK0080c<gK00D0
00000<_<fP00000000000<c=fP02000000@0c<cI00<000000<c;f03<bmP00P3<bmP01@000000c<WH
000000000000cLWG008000000`3<b=L00`000000c<OF0<c7eP030<c7eP030000003<amD000000080
c<OD00<000000<c6e03<a]@00P3<a]@00`000000c<GC0<c5d`020<c5d`030000003<a=<0000000<0
c<CC00H000000<c3dP3<`m8000000<c4dP0000040<c3d@030000003<`]40c<;A00<0c<;A00<00000
0<c2d00000000P3=`]000`000000cL;@0<g2d0020<g2d0030000003=`Ll0cL7?0080cL7?00<00000
0<g1cP0000000P3>`/h00`000000c/7>000000040000000501I1_@000000<4Rk0318^`0000000`1T
Hk`01@000000Nfbf07]/]P1kK;H0000000@0SgFa00<0000009]o/`2KOk<00P2KOk<010000000Y8Rg
0:B8]`0000020:Z9/`030000002_T[T0[i:i0080[i:i00<000000;:I_@2bVKd00P0000020;VM_@03
0000002iX/40^J;100<0^J;100<000000;jXa02nZ<@0102nZ<@2000000D0`Jk700<000000<2_b@00
00001032//X010000000`[?<0<:cc00000020<>ec00400000033]/h0`kK>0<>fcP8000000P34^<h0
10000000`kS@0<>hd033^=02000000030<Fjd0000000a;_A0080a;_A0P0000000`35_=8000000<Bm
d`020<Bmd`80000000<0aKkC00000035_]@00P35_]@2000000030<K0e0000000aL3E0080aL3E0P00
00000`36`MH000000<G1e`020<G1e`80000000@0a/?G00000035`mP0aL?H0P0000020<K4f0040000
0035a=T0aLCI000000<0a/GI00@000000<G5f@00000000000`36a]X010000000aLKJ000000000002
0<K7f`0300000035am/0000000D0a/SL00<000000<K9g036bM`01036bM`00`000000aLWM0<G9g@03
0<G9g@0300000036b]d0000000<0aL[N00<000000<K;gP0000001035bmh010000000aL_O00000035
bml2000000<0aLcO00@000000<G<g`00000000000`35c>0010000000a<cP0000000000030<C<h003
00000034cN40a<gQ00@0a<gQ00<000000<C=h@34cN400`34cN400`000000`lgR0<?=hP020<?=hP80
000000P0`lgR00000033cN8000000<?=hP000000`lgR00000080`/cR0P0000020<;=hP0400000032
c>80000000000080`/cS00@000000<7<hP00000000000`31c><010000000`L_R0000000000020<7;
h`0400000030bn800000000000<0`<_S00@000000<3:hP00000000000P2ob^<010000000_lWR0000
000000030;o9hP030000003oool0oooo02H0oooo00<000000?ooo`3oool04@3oool00`3ogi`0B000
07Boo`040?ooo`030;md0000B9`0gooo0080oooo00L0_g@0001d_`3oool0oooo0;md0000B9`0gooo
0080oooo00<0_g@0001d_`3oool0103oool002H0oooo00<000000?ooo`3oool0:@3oool200000080
/L;V0P0000000`2b`^H000000;;3iP040;;3iP800000102ca>L00`000000/lCW0;?4i`0200000003
0;C5i`00000000000080]<GW0P0000000`2eaNL000000;G6i`040;G6i`8000001@2fanL2000000D0
]lSW0`0000040;O9i`040000002hb^P00000000000<0^<[W00@000000;S;j000000000000`2ibnL0
10000000^LcX0000000000030;[<i`040000002jc>P00000000000@0^lcW0`0000040;c=i`800000
1@2mc^L2000000D0_/oW00<000000;o@i`2od>L00`2od>L01@000000`=3V000000000000`M7V0080
00001032d>D2000000@0`m7U0P0000040<CAi080000000<0aM7S00000035d^@00P0000040<KAh`03
00000037dN80am7R00<0am7R00D000000<S@h@00000000000<SAh@02000000<0bM3O00<000000<[@
gP3:d=h00`3:d=h01P000000b/oM000000000000bloM000000@0blkL00<000000<_=g00000000P3<
cM/00`000000blcJ0<_<fP020<_<fP030000003<c=T0c<cI00<0c<cI00@000000<c:f@000000c<_H
0P0000040<c9f0030000003<b=L0c<SG0080c<SG00<000000<c7eP0000000`3<amH01P000000c<OE
0<c7e@000000c<OD000000@0c<KD00<000000<c5e00000000P3<aM<01`000000c<CC0<c4d`3<a=<0
00000<c4d`000000103<`m800`000000c<?A0<c3d@020<c3d@030000003<`]40000000<0c<;A00H0
00000<c2d03<`]0000000<g2d00000040<g2d0030000003=`Ll000000080cL7?00H000000<g1cP3=
`Lh000000<k2cP0000030<k1cP<0000000<002F^0000000000000P0`B;/200000080I6>l00D00000
07]/]P1kK;H0Nfbf0000000408me/@030000002KOk<0Vgnc00<0Vgnc0P0000030:Z9/`030000002_
T[T0[i:i00<0[i:i0P0000040;VM_@030000002iX/40^J;10080^J;100<000000;jV`@0000001@2n
Z<@00`000000`Jk70<6^a`050<6^a`0300000032//X0`[;:00<0`[;:00<000000<:cc00000001033
]L`00`000000`kK>000000040<BhcP0400000033^=000000000000<0aK[@00@000000<Bkd@000000
00000`35_=8010000000a;gC0000000000030<Fnd`0400000035_]@00000000000<0a/3D00@00000
0<G0e@00000000000`36`MH010000000aL7G0000000000030<K3e`0300000035`mP0000000@0a/CH
00<000000<G4f@0000001@36aMT00`000000a/KJ0<K6fP040<K6fP0300000036am/0a/OK00<0a/OK
00<000000<K8g036b=`00`36b=`00`000000a/WL000000030<K9g080000000@0a/WM00000035bMd0
aLWM0P0000020<K:g@0400000035b]h00000000000<0a/_N00@000000<G;gP00000000000`35bml0
0`000000aLcO0<G<g`040<G<g`0300000035c>00aLcP00@0aLcP00<000000<C<h034c>000P34c>00
0`000000a<gQ000000030<C=h@80000000@0a<gQ00000034cN40a<gQ0P0000000`33cN8000000<?=
hP02000000<0`lgR00@000000<?=hP00000000000`33cN82000000D0`/gR00<000000<;<h`32c><0
0`32c><00`000000`LcS0<7<h`040<7<h`0300000031bn<0`L_S00<0`L_S00<000000<3;h`30bn<0
0`30bn<2000000D0_l[S00<000000;o9hP2obN800`2obN82000002P0oooo00<000000?ooo`3oool0
403oool00`3ogi`0B00007Boo`030?ooo`030?oo_`1d0000B9cO00<0oooo00H0gia80000M02oool0
oono07@00018W=l30?ooo`030=nLB00007@0_ooo00@0oooo000U0?ooo`030000003oool0oooo02`0
oooo0P0000040;;2iP030000002b`nH0//?V0080000000<0/l?V0000002ca>L0102ca>L2000000D0
]<GW0P0000040;G5i`030000002ea^L0]LKW0080000000<0]/KW0000000000000P2fanL200000003
0;K7i`00000000000080]lSW0P0000020;O8i`030000002gbNL0]lWW00<0]lWW00<000000;S:i`2h
b^L0102hb^L00`000000^L_W0;W;i`040;W;i`030000002jc>L0^/cW00@0^/cW00@000000;_<i`2k
c>L0^lcW0`0000000`2lc>L000000;c=i`030;c=i`040000002mcNL0000000000080_LkW0P000000
0`2nc^H00000000000020;k?i`80000000@0_loV000000000000_m3W0P0000040<3@iP0300000032
d>D0`]3U00<0`]3U00<000000<;@i@0000000P33dND200000080`m3T00@000000<CAi00000000000
1035dN<00`000000a]7S0<KAh`030<KAh`0500000036d>80000000000037dN800P0000040<S@h@03
00000039d=l0bM3O0080bM3O00D000000<W?g`00000000000<[@gP02000000@0b/oM00@000000<_>
g03;c]`0blkL0P0000020<_=g0030000003<cM/0000000@0blcJ00<000000<_;fP0000000P3<c=T2
000000<0c<[I00<000000<c9f03<bMP00`3<bMP00`000000c<SG000000020<c8e`070000003<amH0
c<OF0<c7eP000000c<OF000000040<c7e@030000003<a]@0c<KD0080c<KD00H000000<c5e03<aM@0
00000<c5d`0000050<c4d`030000003<`m80c<?B0080c<?B00<000000<c3dP0000000P3<`m401`00
0000c<;A0<c2d@3<`]4000000<c2d@000000103<`]000`000000cL;@0<g2d0020<g2d0060000003=
`Ll0cL7?0000003=`Ll00000103=`Lh00`000000c/7>0<k1cP020<k1cP@0000000H002F^00000000
0000<4Rk0000001@DkH3000000<0Nfbf00<0000008me/@2?MK400P2?MK400`000000Vgnc09]o/`02
09]o/`800000102ZRK<00`000000[i:i0:nB^@030:nB^@030000002iWKd0^Ifm00@0^Ifm00@00000
0;VR`@2iX/4000000P2nY/400`000000_ZS40;jXa0020;jXa00300000032[LD0000000D0`Jk70P00
00060<:bbP0300000033]L`0`kG<00@0`kG<00<000000<BhcP34^<h01034^<h00`000000aK[@0<Fj
d0040<Fjd00300000035_=80aKcB00@0aKcB00<000000<Fnd`35_]<01035_]<00`000000a/3D0<K0
e0040<K0e00300000036`MH0a/7F00@0a/7F00<000000<K3e`36`mL01036`mL00`000000a/CH0<K4
f0040<K4f00300000036aMT0a/GI00<0a/GI00<000000<K6f@0000000`36a]X2000000030<K7fP00
0000a/OK0080a/OK00<000000<K8f`0000000`36b=`01@000000a/WL0<K9g0000000a/WL00800000
0`36bMd00`000000aLWM000000050<K:g@0300000036bmh0a/_N00@0a/_N00<000000<G;g`35bml0
0P35bml00`000000a/cO000000030<G<g`80000000<0aLgP00000035c>000P35c>02000000030<G=
h0000000a<cP008000000P34cN4010000000a<gQ0000000000040<C=h@8000001033cN800`000000
`lgR0<?=hP040<?=hP0800000033cN80`lgR00000033cN8000000<;=h`0000030<;=hP80000000<0
`/gS00000032c><00P32c><00`000000`LcS000000030<7<h`80000000@0`LcS00000031bn<0`L_S
0P0000001030bn<000000<3;h`30bn<200000080_l_S00<000000;o:h`2ob^<00P000000102ob^<0
00000;o9hP2obN82000000030;k9h`000000oooo02L0oooo00<000000?ooo`3oool0403oool00`3o
gi`0B00007Boo`030?ooo`030?oOW0180000M;oo00<0oooo00H0okmd0000B02Lgol0omnL04P0001d
_ol30?ooo`030?noM00004P0W=oo00@0oooo000U0?ooo`030000003oool0oooo02d0oooo00<00000
0;;2iP2b`^H00`2b`^H2000000@0/l?V00<000000;?4i`2ca>L00P0000000`2da>H0000000000002
0;C5i`80000000<0]<GV0000002eaNL0102eaNL2000000D0]/KW0P0000050;K7i`8000001@2gb>L0
0`000000]lWW0;O9i`02000000030;S9i`00000000000080^<[W0P0000000`2ib^L0000000000002
0;W;i`80000000<0^/_W0000000000000P2jc>L2000000030;_<i`000000000000<000000`2lc>L0
10000000_<gW0000000000040;g=i`8000001@2nc^H2000000D0_loV00<000000<3@iP30d>H00`30
d>H01@000000`LoU000000000000`]3U008000000`32d>D2000000D0`m3T00<000000<GAh`35dN<0
0`35dN<01@000000aM3S000000000000a]7S008000001036d>800`000000b=3Q0<S@h@020<S@h@80
000000<0b<oP00000039d=l00P0000040<W?g`030000003:cmd0b/oM00<0b/oM00D000000<[>g@00
0000blkL000000050<_=g0030000003;c=X0blcJ0080blcJ00H000000<_;fP3;bmX000000<c<f@00
00040<c:f@030000003;bMP000000080c<WH0P0000020<c8e`030000003<b=L0000000D0c<OF00<0
00000<c7e@3<amD00P3<amD01P000000blKE000000000000c<KD000000@0c<GD00<000000<c4d`3<
a=<00P3<a=<2000000050<_3dP000000c<?B0<c3dP0000000P3<`m800`000000c<?A000000050<c2
d@030000003<`]00c<;@0080c<;@00H000000<c1d000000000000<g2d00000040<g1c`030000003=
`Lh0cL7>0080cL7>00<000000<g0c@0000000P3>`Lh00`000000c/3=000000020000008002F^00@0
00000318^`0`B;/000000P1@DkH200000080Nfbf00<0000008me/@2?MK400`2?MK401P000000Vgnc
000000000000XGj]000000D0ZXVc00@000000:nB^@2_T[T0[i:i0P0000060;VM_@030000002iX/40
000000@0_ZK100@000000;jXa02nZ<@000000P32[LD010000000`Jk70<6^a`31[/L200000080`k78
00<000000<:bbP32//X00P32//X2000000D0`kG<0P0000050<BhcP8000001@35^]02000000D0aKcB
0P0000050<Fnd`8000001@36`=@2000000D0a/7F00<000000<K2eP0000000`36`mL2000000030<K4
e`000000a/CH0080a/CH0P0000001036aMP000000<K5f@36aMT200000080a/KI00@000000<K6fP00
000000000`36amX010000000a/OK0000000000030<K8f`0300000036b=`0000000@0a/WL00<00000
0<K9g@36bMd01036bMd00`000000a/[M0<K:g@020<K:g@80000000<0a/_N00000036bmh00P36bmh2
000000050<K<gP000000aL_O0<G;g`0000000P36c=l010000000aLcO0000000000030<G=h0040000
0035c>000000000000@0aLgP00<000000<C=h@34cN400`34cN400`000000a<gQ0<C=h@020<C=h@80
000000<0a<kR00000033cN800P33cN800`000000`lkR000000030<?=hP80000000<0`lgR00000033
cN800P0000020<;=h`0400000032cN800000000000<0`/gS00@000000<;<h`00000000000P31c><0
10000000`LcS0000000000040<7<h`8000001030bn<2000000D0_l_S0P0000040;o:h`800000102n
bN<00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo00l0oooo00<0omnL04P0001d_ol00`3o
ool00`3ogi`0B00007Boo`030?ooo`060?noM00004P0W=oo0?oOW0180000M;oo0`3oool00`3o_g@0
001809cOo`040?ooo`009@3oool01@000000oooo0?ooo`3oool0000002`0oooo00<000000;;2iP2b
`^H00P0000000`2c`^H000000;?3iP040;?3iP8000001@2da>H2000000@0]<GV00<000000;G5i`2e
aNL00P0000000`2fa^H00000000000020;K6i`80000000<0]/KV0000002fanL00`2fanL00`000000
]lOV000000050;O8i`<00000102hbNL3000000D0^L[W0P0000050;[;i`D000000P2kc>L00`000000
_<cW0;c<i`040;c<i`030000002mcNL0_LgW00<0_LgW00@000000;k=iP00000000000P2nc^H20000
00030;o>iP00000000000080_loV0P0000001030cnD0000000000030d>H2000000@0`LoU00<00000
0<;@i@32d>D00P32d>D010000000`m3T0000000000020<?@i080000000@0a=3S000000000000aM7S
0P0000040<G@h`0300000036d>80a]3R00<0a]3R00@000000<O@h@000000b=3Q0P0000040<S?h003
00000039cml0bLoO00<0bLoO00D000000<W>gP00000000000<[?g@02000000<0b/kM00<000000<_=
g03;cM`00`3;cM`01P000000blcK000000000000blcJ000000@0bl_J00<000000<c:f@3<b]T00P3<
b]T01P000000blWH0<_9f0000000c<WH000000D0c<SG00<000000<c7eP3<amH00`3<amH010000000
blOF0000003<amD2000000@0blKE00<000000<c5e03<aM@00P3<aM@010000000blCC0000003<a=<2
000000<0bl?B00<000000<c3dP000000103<`m800`000000c<;A0<c2d@030<c2d@040000003<`M00
00000<c2d0800000103<`M000`000000cL7?0<g1c`020<g1c`030000003=`<h000000080cL7>00H0
00000<g0c@3=`<d000000<k1cP0000030<k0c@<0000000D002F^000000000000<4Rk00000003051C
]P050000001kK;H0Nfbf07]/]P0000000`2?MK43000000<0XGj]00<000000:Z9/`2ZRK<00`2ZRK<0
10000000[i:i0:nB^@0000020;NH^0030000002iWKd0^Ifm00@0^Ifm00<000000;jV`@2nY/400`2n
Y/400`000000_ZS4000000040<:]a@0300000031[/L0000000@0`k7800<000000<:bbP32//X00P00
00020<Beb`0300000033]L`0`kG<008000000P35^<d00`000000a;S>0<BhcP0200000080a[[?00<0
00000<Fjd035^]000P0000020<Jld@0300000035_=80aKcB008000000P36_]800`000000aKkC0<Fn
d`0200000080a[oD00<000000<K0e036`=@00P0000020<K1e@0300000036`MH0a/7F008000000P36
`]H010000000a/?G0000000000030<K4e`0400000036a=P00000000000<0a/GH00<000000<K5f@00
00001@36a]T00`000000a/OJ0<K7fP040<K7fP0300000036b=/0a/SK00@0a/SK00<000000<K9g036
bM`00P36bM`00`000000alWL000000040<K9g@0400000036b]d000000<K:g@8000000`36bmh01000
0000a/_N0000000000030<K<gP0300000035bml0000000@0a/cO00<000000<G=h035cN001035cN00
0`000000aLgP0<G=h0020<G=h080000000<0aLgQ00000034cN400P34cN4010000000a<gQ00000034
cN42000000<0a<kR00@000000<?=hP00000000000P33c^8010000000`lgR0000000000040<?=hP03
00000032cN<0000000<0`/gS00<000000<;=h`32cN<01032cN<00`000000`LcS0<7<h`030<7<h`03
00000031c><0`LcS0080`LcS0P0000000`30c><000000<3;h`020<3;h`0300000030bn<0000000<0
_l_S0`0000030;o:h`80000000<0_/[S0000002nbN<00P2nbN<00`000000_/WS0000000X0?ooo`03
0000003oool0oooo00h0oooo00<0omnL04P0001d_ol00`3oool00`3ogi`0B00007Boo`030?ooo`06
0?noM00004P0W=oo0?oOW0180000M;oo0`3oool00`3o_g@0001809cOo`040?ooo`009@3oool01000
0000oooo00000000000^0?ooo`800000102c`^H00`000000/l?V0;?3iP02000000030;?3iP000000
00000080]<CV0P0000000`2da>H000000;C5iP040;C5iP8000001@2fa^H2000000@0]/KV00@00000
0;K7i`00000000000`2ganH00`000000]lSW0;O8i`0200000080^<SV00<000000;S9i`2hbNL00P00
00020;W9iP8000000P2ib^L2000000030;[:iP00000000000080^/_W1@0000020;_<i`80000000<0
_<cV0000000000000P2lc>L2000000040;g<iP00000000000;g=i`800000102ncNH2000000D0_lkV
0P0000040<3?i@8000001031cnD2000000030<;?i0000000`]3U008000001033d>@2000000D0a=3S
00<000000<G@h`35d><00`35d><01@000000a]3R000000000000a]3R008000000`37d>400`000000
b<oP0<S?h0030<S?h00500000038cml0000000000039cml00P0000040<W>gP030000003:c]d0b/kM
0080b/kM00D000000<[=g000000000000<_=g002000000@0blcK00<000000<_;fP3;bmX00P3;bmX0
1P000000bl[I000000000000c<[I000000@0blWH00<000000<c8e`3<b=L00`3<b=L01@000000blOG
000000000000c<OF008000000`3;amH00`000000blKE0<_6e@030<_6e@040000003;aM@000000<c5
e08000000`3;a=<00`000000bl?B0<_3dP040<_3dP030000003<`m80c<?B0080c<?B00D000000<_2
d@00000000000<c2d@02000000<0c<7@00<000000<c1d03<`M000`3<`M000`000000c<3?00000002
0<g1c`060000003=`<h0cL3>0000003=`Lh00000103=`<d00`000000c/3=0<k0c@020<k0c@@00000
00@002F^00000000000000000`1@DkH30000000307]/]P000000SgFa0080000000<0TfnV0000002Q
OZd0102QOZd00`000000ZXVc0:Z9/`030:Z9/`8000000`2gV;P00`000000^Ifm0;VM_@030;VM_@80
00001P2nY/400`000000`Zg50<:]a@040<:]a@0300000033/LP0`k7800@0`k780P0000040<Beb`03
00000033]L`0000000@0aKS=00<000000<BhcP0000001036^/l00`000000aK[@000000040<Jld@03
00000035_=80000000@0a[kB00<000000<Fnd`0000001036_m@00`000000a/3D000000040<K1e@03
00000036`MH0000000D0a/;F00<000000<K4e`36a=L01036a=L00`000000a/GH0<K5f0040<K5f003
00000036a]T0a/KI00<0a/KI00<000000<O7fP0000000`36amX2000000030<O8f`000000a/SK0080
a/SK0P0000000`37bM/000000<K9g002000000<0alWL00@000000<K9g@00000000000`36b]d00`00
0000a/_N0<K;gP040<K;gP0300000036c=h0a/cN00@0a/cN00<000000<K<g`36c=l00P36c=l00`00
0000a/gO000000030<G=h080000000<0aLgP00000035cN000P0000030<G=h@0400000034cN400000
000000<0a<gQ00<000000<C>hP34c^801034c^800`000000`lkR0<?>hP030<?>hP0300000033cN80
`lgR0080`lgR0P0000002033c^<000000<;=h`000000`/gS00000032cN<000000`32cN<200000004
0<;=h`000000`LcS0<7<h`80000000<0`LgS00000031c><00P0000030<3<h`0400000030bn<00000
00000080`<_S00@000000;o;h`00000000000`2obn<3000000<0_/[S00@000000;k9h`0000000000
0P2nbN<00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo00h0oooo00<0omnL04P0001d_ol0
0`3oool00`3ogi`0B00007Boo`030?ooo`060?noM00004P0W=oo0?oOW0180000M;oo0`3oool00`3o
_g@0001809cOo`040?ooo`009@3oool200000340oooo00<000000;?2iP2c`^H00`2c`^H2000000D0
/l?V0P0000040;C4iP030000002daNH0]<GV0080000000<0]LGV0000000000000P2fa^H200000003
0;K6iP000000]/KV00@0]/KV00<000000;O7iP2ganH0102ganH2000000D0^<SV0P0000050;W9iP<0
0000102jb^H4000000<0^l_V0`0000040;c<iP<00000102mc>H2000000@0_/gV0P0000000`2ncND0
00000;o>iP020;o>iP80000000@0_lkU00000030cnD0`<oU0P0000020<7>i@0400000031cnD00000
000000@0`/oT00<000000<?@i033d>@00`33d>@010000000`loS0000000000020<C@h`80000000@0
aLoR000000000000aM3S0P0000040<K@hP0300000037d>40am3Q0080am3Q00D000000<O?h0000000
00000<S?h002000000@0b<oO00<000000<W>gP39c]h00`39c]h010000000bLgM0000003:c]d20000
00@0b/gL00<000000<_<f`3;c=/00`3;c=/010000000bl_J0000003;bmX2000000@0bl[I00<00000
0<_9f03;bMP00P3;bMP01@000000blSH000000000000c<SG00800000103;amL00`000000blOF0<_7
eP020<_7eP050000003;a]D000000000003;a]D00P0000030<_5e0030000003;a=<0blCC0080blCC
00<000000<_3d`000000103;`m8010000000bl?B0000003<`m82000000@0bl;A00<000000<c1d03<
`M000P3<`M000`000000c<7@000000020<c1d08000000P3<`<l00`000000cL7?000000040<g0cP03
0000003=`<d0cL3=0080cL3=00L000000<foc000000000000<k0c@000000cko;00<0000000@002F^
00000000000000000`1@DkH00`000000Lf2_07=P[`03000000<0TfnV00<000000:5n[@2QOZd00`2Q
OZd00`000000ZXVc0:Z9/`030:Z9/`030000002gV;P0]iRh00<0]iRh00<000000;VM_@2iWKd00P00
00020;nT_P030000002nY/40_ZK10080_ZK10P0000050<:]a@8000001P33/LP00`000000a;G;0<Be
b`040<Beb`0300000035^<d0aKS=00@0aKS=00<000000<Jjc`36^/l01036^/l00`000000a[cA0<Jl
d@040<Jld@0300000036_]80a[kB00@0a[kB00<000000<Joe036_m@01036_m@00`000000a/7E0<K1
e@040<K1e@0300000036`]H0a/;F00<0a/;F00<000000<O4e`0000001036a=L00`000000alGH0000
00030<K5f080000000@0alKI00000036a]T0a/KI0P0000020<O7fP0400000036amX00000000000<0
alSK00@000000<K8f`00000000001037bM/00`000000alWL0<O9g0040<O9g00300000036b]d0a/[M
0080a/[M00<000000<O;g@0000000`36bmh2000000030<K<gP000000a/cN0080a/cN0P0000000`36
c=l000000<K<g`0200000080a/gO00@000000<G=h000000000001035cN000`000000aLgQ0<G=h@04
0<G=h@0300000034cN40a<gQ0080a<gQ00<000000<C>hP0000000`34c^82000000030<C>hP000000
`lkR0080`lkR00@000000<?>hP000000`lgR0P0000030<?>h`0400000032cN<0000000000080`/gS
00@000000<;=h`00000000001032cN<2000000@0`LgS00<000000<3<h`30c><01030c><00`000000
`<_S0<3;h`030<3;h`030000002obn<0_l_S00<0_l_S00<000000;k:h`2nb^<00`2nb^<2000000D0
_/WS00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0=0?ooo`030?oOW0180000M;oo00<0
oooo00<0oono07@00018W=l00`3oool01P3OW4P0001d0;ooo`3ookl0M00004RLg`<0oooo00<0gia8
0000M02oool0103oool002D0oooo00<000000?ooo`3oool0<@3oool00`000000/l;V0;?2iP020000
00030;?2i@00000000000080/l?V0P0000000`2d`nD000000;C4iP040;C4iP8000001@2eaNH20000
00@0]/KV00<000000;K6iP2fa^H00P0000000`2ga^H00000000000020;O7iP80000000<0^<OV0000
000000000P2hb>H2000000030;W8iP00000000000080^LWV0P0000020;[9iP030000002jb^H0^/[V
00D000000P2kbnH200000080^l_V00<000000;c<iP2lc>H00P0000020;c<iP030000002mc>H0_LcV
008000000P2mc>H010000000_/gV0000000000030;k=i@040000002oc^H00000000000@0_lkU0P00
00050<7>i@0300000032cn@0`/oT00<0`/oT00D000000<;?i000000000000<?@i002000000@0`loS
0P0000050<G?hP0300000036d>80a]3R0080a]3R0P0000000`36cn4000000<O@h@02000000@0aloP
00<000000<S?g`38cml00`38cml01@000000b<kN000000000000bLkN008000000`39cMd00`000000
b/gL0<[=g0030<[=g0050000003:c=/000000000003;c=/00P0000030<_;fP030000003;b]T0bl[I
00<0bl[I00@000000<_9f@000000blWH0P0000040<_8f0030000003;amL0blOG0080blOG0P000000
0`3;a]H000000<_7eP02000000@0blKE00<000000<_5e03;aM@00P3;aM@00`000000blCD00000002
0<_4d`060000003;`m<0bl?C0<_3d`000000bl?B0P0000030<_3dP030000003;`]40bl;A00<0bl;A
00<000000<_1d@0000000P3<`M001P000000c<7@0<c1d0000000c<7@000000D0c<3?00<000000<g0
cP3=`<h00P3=`<h00`000000c;o=000000020<g0c@030000003=_l`0cKo<0080cKo<00@000000<no
b`3?_l/0cko;100000000`00::d0000000000003051C]P040000001cH:l0Lf2_000000D0TfnV00<0
00000:5n[@2QOZd00P2QOZd00`000000ZXVc0:Z9/`020:Z9/`8000001@2gV;P00`000000^Ifm0000
00040;nT_P030000002nY/40_ZK1008000000P33[<<01@000000`Zg50<:]a@32[LD000000080aK76
00<000000<>ab033/LP00P33/LP00`000000a[C9000000040<Beb`0300000036^<`0000000@0aKS=
00<000000<NjcP0000001036^/l00`000000akc@000000040<Jld@0300000037_]80000000@0a[kB
00<000000<Nod`0000001036_m@00`000000al7D000000040<K1e@0300000037`]H0000000<0a/;F
00H000000<O4e`37a=L0alCG00000036a=L200000080alGH00@000000<K5f000000000000`37a]T0
0`000000a/KI000000050<O7fP0300000037b=/0alSK00@0alSK00<000000<O9f`37bM/00`37bM/0
0`000000alWL000000030<O9g080000000D0al[M00000036b]d0a/[M000000020<O;g@0400000036
bmh00000000000<0a/cN00@000000<K<gP00000000001036c=l00`000000a/gO0<K=g`030<K=g`03
00000035cN00aLgP0080aLgP0P0000000`35c^4000000<G=h@020<G=h@80000000<0aLkQ00000034
cN400P0000020<C>hP0400000034c^800000000000<0a<kR00@000000<?>hP00000000000`33c^80
0`000000`lkS0<?>h`040<?>h`0300000032cN<0000000<0`/gS00<000000<;=h`32cN<00P32cN<2
000000030<7=h`000000`LgS0080`LgS00<000000<7=h`0000000`30c><3000000<0`<_S0P000000
102obn@000000;o;h`2obn<2000000040;o;h`000000_/[S0;k:h`8000000P2nb^<01`000000_/WS
0;k9h`2nbN<000000;g9h`000000:03oool00`000000oooo0?ooo`0<0?ooo`030?oOW0180000M;oo
00@0oooo00<0gia80018W03Oool00P3oool01`2oM00007Bo0?ooo`3oool0gia80018W03Oool00P3o
ool00`2oM00007Bo0?ooo`040?ooo`001@3oool01@3ogi`0B0000000000007@0_ooo00<0oooo00H0
okmd0000001d_ol0oooo0?oog`2LB0030000000307Boo`3oool0oooo00`0oooo00<000000?ooo`3o
ool0<P3oool2000000D0/l;U0P0000040;C3i@030000002da>H0]<CV0080000000<0]LCU00000000
00000P2eaNH2000000030;K5i@000000]/KV00@0]/KV0P0000050;O6iP8000001@2hanH2000000D0
^LSV0P0000050;[9iP<00000102jb^H2000000D0^l_V0P0000050;c<iP8000001@2mc>H00`000000
_/gU0;k=i@040;k=i@030000002oc^D0_lkU00<0_lkU00@000000<3>i000000000000P31c^D20000
00040<7>i000000000000<;?i08000001032cn@00`000000`loS0<??h`030<??h`0400000034c^80
000000000080aLoR0P0000000`35cn8000000<K@hP02000000@0a/oQ00<000000<O?h037cn000`37
cn001@000000alkO000000000000b<oO008000001038c]h00`000000bLgM0<W=g@020<W=g@050000
0039c=`000000000003:cM`00P0000040<[<f`030000003;bmX0bl_J0080bl_J00D000000<[:fP00
000000000<_:f@02000000<0blWI00<000000<_8f03;b=P00`3;b=P010000000blOG0000003;amL2
000000@0blKF00<000000<_6e@3;a]D00`3;a]D00`000000blGE000000020<_5e0060000003;a=@0
blCD0000003;a=<000001@3;`m<00`000000bl?B0<_3dP020<_3dP030000003;`]80000000<0bl;A
00H000000<_1d@3;`M4000000<c1d00000040<c1d0030000003<`<l0c<3?00<0c<3?00<000000<bo
cP0000000P3=`<h01P000000c;o=0<boc@000000cL3=000000@0cKo<00<000000<fnb`0000000P3?
_l/40000008002R]00D00000051C]P1@DkH0D5>f0000000207=P[`030000002CKjH0TfnV00<0TfnV
00<000000:5n[@2QOZd00P2QOZd00`000000ZXVc0:Z9/`0200000080/i2a00<000000;NH^02gV;P0
0`2gV;P00`000000_jBn0;nT_P040;nT_P8000001033[<<010000000`Zg50<:]a@0000030<FaaP03
00000033/LP0`k78008000000P36]<T01@000000a;G;0<Beb`34]L/000000080a[S<00D000000<Fh
c@35^<d0aKS=000000020<NjcP0500000036^/l0a[[?0<Jjc`0000000P37_=001@000000a[cA0<Jl
d@36_=4000000080akkB00D000000<JndP36_]80a[kB000000030<Nod`0400000036_m@000000000
00<0al7D00@000000<K1e@00000000000`37`]H00`000000a/;F000000050<O4e`0300000037aMP0
alGH00<0alGH00<000000<O6f@37a]T01037a]T00`000000alOJ0<O7fP030<O7fP0300000037b=X0
000000<0alSK0P0000001037bM/000000<O9f`37bM/200000080alWL00@000000<O9g00000000000
0`37b]d00`000000a/[M000000040<O;g@0300000036c=h0a/cN00@0a/cN00<000000<K<g`36c=l0
0P36c=l2000000030<K=g`000000a/gO0080a/gO00@000000<K=h0000000aLgP0P0000030<G>h@04
00000035cN400000000000@0aLkQ00<000000<C>hP34c^800`34c^800`000000a<kR0<C>hP030<C>
hP8000001033c^800`000000`lkS000000030<?>h`80000000<0`/kS00000032cN<00`0000000`32
c^<000000<;=h`02000000<0`LgS00@000000<7=h`00000000000P31cN<010000000`<cS00000000
00030<3<h`<00000102obn@2000000@0_l_S0P0000040;k:h`040000002nbN<0000000000080_LWS
00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0:0?ooo`050?oog`2LB00000000000001d
_ol0103oool01`3ookl0M00007Boo`3oool0okmd0000M02oool00P3oool01`3ookl0M00007Boo`3o
ool0okmd0000M02oool01@3oool000@0oooo00L0oono07@0001d_ol0oooo0?noM00007@0_ooo0080
oooo00L0okmd0000001d_ol0oooo0?oog`2LB7@0_ooo0080oooo00<0_g@007Boo`3oool02`3oool0
0`000000oooo0?ooo`0d0?ooo`8000000P2c`^D2000000030;C3i@000000]<?U00@0]<?U0P000005
0;G4i@800000102faND00`000000]/KV0;K6iP02000000030;O6i@00000000000080]lKV0P000000
0`2hanD00000000000020;S7iP80000000<0^LOU0000002ib>H00`2ib>H00`000000^LSU00000002
0;[9iP<0000000<0^/WU0000002jb^H00`2jb^H00`000000^l[U000000040;_;iP040000002lbnD0
000000000080_<cV0P0000000`2mc>D00000000000020;g<iP80000000<0_/cU0000000000000P2n
cND2000000040;o=i@00000000000;o>i@8000001030c^@2000000D0`LkT00<000000<;?i032cn@0
0`32cn@01@000000`lkS000000000000`loS008000001034c^82000000@0aLoR00<000000<K?h@36
cn400`36cn401@000000a/kP000000000000aloP008000001037c]l00`000000b<kN0<S>gP020<S>
gP80000000<0b<gM00000039cMd00P0000040<W<g0030000003:c=/0b/cK0080b/cK0P0000000`3:
bm/000000<_;fP02000000@0b/[J00<000000<_9f@3;bMT00P3;bMT01@000000b/SH000000000000
blSH008000000`3;amL00`000000blKF0<_6eP030<_6eP030000003;a]H000000080blKE0P000002
0<_5e@030000003;aM@0000000@0blCD00<000000<_3d`3;`m<00`3;`m<00`000000bl;B00000002
0<_3dP070000003;`]80bl;B0<_2dP000000bl;A000000040<_1d@030000003<`M00c<7@0080c<7@
00<000000<^oc`0000000`3<`<l01P000000c;o>0<bocP000000cL3>000000@0c;o=00<000000<fo
c03=_l`00P3=_l`01`000000cKk;0<fnb`000000cko;0000003?_/X0100000000`00::d000000000
0002051C]P050000001cH:l0Lf2_07=P[`000000102CKjH00`000000XGj]0:5n[@030:5n[@800000
102cT;400`000000]iRh0;NH^0020;NH^08000001P2oY;h00`000000`jc30<>/``040<>/``800000
1@35/LH2000000D0a[C900<000000<Beb`0000001036^<`00`000000aKS=000000040<NjcP030000
0036^/l0000000@0akc@00<000000<Jld@0000001037_]800`000000a[kB000000050<Nod`030000
0037`M@0al7D00@0al7D00<000000<O2eP37`]H01037`]H00`000000alCG0<O4e`030<O4e`800000
1037aMP00`000000alKH000000040<O6f@0300000037amT0000000<0alOJ00D000000<O8fP37b=X0
00000<O8f`02000000<0alWK00<000000<O9f`0000001@37bM`00`000000al[M0<O:g@040<O:g@03
00000037bmd0al_M0080al_M00<000000<O<gP0000001036c=h010000000a/cO00000036c=l20000
00<0a/gO00@000000<K=g`00000000000`36cN000`000000aLkQ0<G>h@040<G>h@0300000035c^40
aLkQ0080aLkQ0P0000000`35c^8000000<C>hP020<C>hP0300000034c^8000000080a<kR0P000002
0<C>hP0400000033c^80000000000080`lkS00@000000<?>h`00000000001032c^<3000000<0`/kS
00<000000<7=h`31cN<01031cN<00`000000`LgS0<7=h`020<7=h`8000001@30c><00`000000_l_T
0;o;i0020;o;i080000000<0_l_T0000002obn<00P2obn<00`000000_/_T000000040;k:h`800000
102mbN<2000002H0oooo0`00000=0?ooo`030?noM0000000M;oo00D0oooo00D0oono07@000000000
001d0;ooo`040?ooo`050?oo_`1d000000000000M02oool01P3oool000@0oooo00<0_g@00018W03O
ool00P3oool00`2oM00007Bo0?ooo`0:0?ooo`030?oog`2LB7@0_ooo00/0oooo00<000000?ooo`3o
ool0=P3oool2000000@0]<?U00<000000;C3i@2d`nD00P0000000`2e`nD00000000000020;G4i@80
000000<0]/CU0000002faND0102faND2000000D0]lKU0P0000050;S7i@800000102ianD010000000
^LSV0000000000020;W8i@D000000`2jbND010000000^/[V0000000000030;_:i@040000002kbnH0
0000000000@0_<_U0P0000050;g<i@8000001@2nc>D2000000D0_lgU00<000000<3>i030c^@00`30
c^@010000000`LgT0000000000020<7>i080000000@0`/kS000000000000`/oT0P0000040<?>h`03
00000034c^80a<kR0080a<kR0P0000001034c^8000000<G?hP35cn82000000040<G>h@0000000000
0<K?h@8000001036c^000`000000alkO0<O>g`030<O>g`0400000038cMh000000<S>gP8000001038
cMd00`000000bLcL0<W<g0030<W<g00400000039bm`000000<[<f`800000103:bm/00`000000b/[J
0<[:fP030<[:fP040000003:bMT000000<_9f@800000103:b=P00`000000blOG0<_7e`020<_7e`03
0000003:a]L0000000<0blKF00H000000<_6eP3;a]H000000<_6e@0000050<_5e@030000003;a=@0
blCD0080blCD00<000000<_3d`0000000`3;`m<01P000000bl;B0<_2dP000000bl?B000000D0bl;B
00<000000<_1d@3;`M400P3;`M400`000000bl3@000000020<c1d0070000003;_ll0bko?0<^oc`00
0000c<3?000000040<bocP030000003<_ld0c;o=0080c;o=00H000000<bnc000000000000<foc000
00040<fnb`040000003?_/X0ckk:0<nnbP<000000P00::d200000003051C]P000000Lf2_0080Lf2_
00<0000009=_YP2CKjH00P2CKjH00`000000XGj]0:5n[@030:5n[@030000002cT;40/i2a00<0/i2a
00D000000;NH^02gV;P0]iRh000000020<6R^P030000002oY;h0_jBn0080_jBn00<000000<F[`000
00001@33[<<00`000000aK760<FaaP040<FaaP0300000036]<T0a[C900D0a[C900<000000<Jhc036
^<`01036^<`00`000000ak[>0<NjcP040<NjcP0300000037_=00akc@00@0akc@00<000000<NndP37
_]801037_]800`000000akoC0<Nod`030<Nod`8000001@37`M@00`000000b<;E000000030<O2eP80
000000@0b<?F00000037a=L0alCG0P0000020<O5e`0400000037aMP0alGH000000<0alKH00@00000
0<O6f@00000000000`37amT00`000000alOJ000000040<O8fP0300000037bM/0alWK00@0alWK00<0
00000<O9g037bM`00P37bM`2000000030<O;g0000000al[M0080al[M0P0000000`37bmd000000<O;
g@02000000<0alcN00@000000<K<gP00000000000`36c=l00`000000a/gO0<K=g`040<K=g`030000
0036cN00a/gP0080a/gP00<000000<K>h00000000`35c^42000000030<G>h@000000aLkQ00800000
0`35c^8010000000a<kR0000000000030<C>hP8000001@34c^800`000000`lkS0<?>h`030<?>h`03
00000032c^<0`/kS0080`/kS0P0000000`32c^<00000000000020<;>h`0300000032c^@0000000<0
`LgS0`0000020<7=h`8000000P30cN@00`000000`<cS0<3<h`02000000030;o<i0000000_l_T0080
00000`2obn@010000000_l_S0000000000020;k;i0030000002nb^<0_/[S0080000000@0_L[T0000
002mbN<0_LWS0P0000020;g9i0030000003oool0oooo0200oooo100000000`3oool000000?ooo`0X
0?ooo`000`3oool00`3ookl0M00004RLg`030?ooo`030=nLB00007@0_ooo00/0oooo00<0_g@007Bo
o`3oool02P3oool00`000000oooo0?ooo`0g0?ooo`030000002d`nD0]<?U00<0]<?U0P0000050;G3
i@800000102fa>D00`000000]/GU0;K5i@02000000030;O5i@00000000000080]lKU0P0000000`2h
a^D000000;S7i@030;S7i@030000002hanD0000000D0^LOU00<000000;W8i@0000000P0000020;W8
i@030000002jbND0^/WU00@0^/WU00<000000;_:i@2kb^D0102kb^D00`000000_<_U0;c;i@030;c;
i@030000002mbnD0000000@0_LcU00@000000;k;i@00000000000P2nc>D2000000030;o<i0000000
00000080_lgU0P0000001030cN@0000000000030c^@2000000@0`LgT0P0000050<;>h`0300000033
c^<0`lkS00<0`lkS00@000000<?>hP000000a<kR0P0000040<C>hP8000001@35c^400`000000a/kP
0<K>h0030<K>h00500000037cMl0000000000037c]l00P0000030<S=gP0300000038cMd0b<gM00<0
b<gM00D000000<S<g@00000000000<W<g002000000<0bL_L00<000000<[;f`3:bm/00`3:bm/01@00
0000bL[J000000000000b/[J008000000`3:bMT00`000000b/SH0<[8f0030<[8f0030000003:amL0
00000080blOG00L000000<[6e`3:a]L0b/KG0000003;a]H0000000@0blKF00<000000<_5e@3;aMD0
0`3;aMD010000000b/CD0000003;a=@2000000<0bl?C00<000000<_3d`000000103;`]800`000000
bl;B0<_2dP030<_2dP040000003;`M4000000<_1d@8000000P3;`=000`000000c<7@000000050<^o
c`030000003<_lh0c;o>0080c;o>00@000000<bnc@000000c;o=0P0000040<bnc0030000003=_//0
cKk;0080cKk;00<000000<flbP0000000P3?_/X400000005000X[@0000000000051C]P0000000`1c
H:l00`000000TfnV09=_YP0209=_YP030000002QOZd0XGj]0080XGj]0P0000050;>@/@040000002g
V;P0]iRh000000<0`J:j00D000000;nT_P2oY;h0_jBn000000020<F[`00300000033[<<0`jc30080
`jc300<000000<Naa00000001035/LH00`000000akC8000000050<Jdb@8000001@36^<`2000000D0
ak[>0P0000050<Nld00300000038_]40000000<0akkB0P0000001038_m8000000<Nod`37_m<20000
0080b<7D00<000000<O1e037`M@00P0000020<S2e@0400000037`]H00000000000<0b<?F00<00000
0<O4e`0000001037aML00`000000alGH000000050<O6f00300000037amT0alOI00@0alOI00<00000
0<O8fP37b=X00P37b=X00`000000b<WK000000040<O9f`0400000038b]/000000<O9g08000000`37
bm`010000000al[M0000000000040<O;g@0300000037c=h0alcN00@0alcN00<000000<K<g`36c=l0
0P36c=l00`000000algO000000030<K=g`80000000<0a/kP00000036cN000P0000020<K>h0040000
0035c^400000000000@0aLkQ00<000000<G>hP35c^801035c^800`000000a<kR0<C>hP020<C>hP03
00000034cn80000000<0a<kR0P0000001033cn<000000<?>h`33c^<2000000030<?>h`000000`/kS
008000000`32c^<010000000`/kS0000000000020<;>i00400000031cN<00000000000<0`LgT0P00
00050<3=i0800000102oc>@00`000000_l_T0;o;i0040;o;i0030000002nbn@0_/_T00<0_/_T0P00
00040;g:i08000001@2mbN@00`000000oooo0?ooo`0L0?ooo`@00000103oool00`000000oooo0?oo
o`0W0?ooo`000`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00X0oooo00<0oooO
09a80018W=l02`3oool00`000000oooo0?ooo`0h0?ooo`030000002d`nD0]<?U0080000000<0]L?T
0000000000000P2e`nD2000000030;K3i0000000]/CU00@0]/CU0P0000050;O5i@800000102ha^D0
10000000^<OU0000000000030;S7i@030000002ianD0^LOU00D000000P2ib>D2000000030;[8i@00
000000000080^/WU0P0000000`2kbND00000000000020;_:i@80000000@0_<[U000000000000_<_U
0P0000030;g;i@040000002mc>D00000000000@0_/_U0P0000050;o<i08000001@30cN@00`000000
`LgT0<7=i0030<7=i00400000031cN<0000000000080`/kS0P0000001032cN<0000000000033c^<2
000000<0`lkR00<000000<C>hP34c^800`34c^8010000000aLgQ0000000000020<G>h@80000000@0
a/gP000000000000a/kP0P0000040<O=g`0300000038cMh0b<gN0080b<gN00D000000<S<g@000000
00000<S=g@02000000@0b<cM00<000000<W;g039bm`00P39bm`01@000000bL_K000000000000b/_K
008000001039b]X00`000000b/WI0<[9f@020<[9f@030000003:b=P0000000<0b/SH00H000000<[7
e`3:amL000000<_7e`0000050<[6e`030000003;a]H0blKF0080blKF00D000000<[5e@0000000000
0<_5e@02000000<0b/CD00<000000<_3d`3;`m<0103;`m<00`000000bl;B0<_2dP020<_2dP050000
003:`]8000000000003;`]800P0000030<_1d@030000003;`=00bl3@00<0bl3@00<000000<^oc`3;
_ll00`3;_ll010000000bko>0000003<_lh2000000<0c;k=00<000000<bnc03<_/`00`3<_/`00`00
0000c;c;000000020<fnb`060000003=_<X0cKc:0000003?_/X000000P3?_<P200000006000X[@00
0000736[0000001@DkH000000P1cH:l00`000000TfnV09=_YP0209=_YP050000002QOZd0XGj]0:5n
[@0000000P2[PZP00`000000/i2a0;>@/@030;>@/@8000001031X[X010000000_jBn0;nT_P000004
0<F[`00400000033[<<0`jc300000080ak7400D000000<FaaP35/LH0aK76000000020<Ndb0040000
0036]<T0a[C90<Jdb@8000000P38^</00`000000a[S<0<Jhc00200000080b;[=00<000000<NjcP37
^/h00P0000020<Rlc`0300000037_=00akc@008000000P38_]4010000000akkB0000000000030<Ro
dP0300000037_m<0000000@0b<7D00<000000<O1e00000001@38`]D00`000000b<?F0<S3eP040<S3
eP0300000037aML0alGG00@0alGG00<000000<O6f037a]P00`37a]P00`000000b<OI000000030<O7
f@80000000D0b<SJ00000037b=X0alSJ000000030<S9f`0400000037bM/00000000000<0b<[K00<0
00000<O;g037bm`01037bm`00`000000al_M0<O;g@030<O;g@0300000037c=d0000000<0alcN0P00
00000`37cMh000000<K<g`0200000080algO00@000000<K=g`00000000001036c^000`000000a/kP
0<K>h0030<K>h00300000035c^40aLkQ00<0aLkQ00<000000<G?h@0000000`35c^82000000030<G?
hP000000a<kR008000000P34cn8010000000a<kR0000000000040<??h`8000001033c^<00`000000
`/kS0<;>h`040<;>h`0300000032c^@0000000<0`/kT00<000000<7=i031cN@00P31cN@00`000000
`LgT000000030<3=i080000000<0`<gT0000002oc>@00P2oc>@00`000000_lcT000000030;o;i080
000000<0_/_T0000002nbn@00P2nbn@00`000000_/[T000000030;g:i0030000002mbN@0000000<0
_LWT0P0000000`2lbN@000000?ooo`0I0?ooo`<00000203oool00`000000oooo0?ooo`0W0?ooo`00
0`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00X0oooo00<0gia80000001d_ol0
2`3oool00`000000oooo0000000i0?ooo`8000001@2e`n@2000000@0]/?T00<000000;K4i@2fa>D0
0P0000000`2ga>D00000000000020;O5i@80000000<0]lGU0000002ha^D0102ha^D00`000000^<OU
0;S7i@040;S7i@<00000102ianD2000000D0^/SU0P0000050;_9i@8000001@2lb^D00`000000_L_U
0;g;i@040;g;i@030000002nbnD0_/_U00<0_/_U00<000000;k;i0000000102oc>@010000000_lcT
0000000000020<3=i080000000@0`<cT000000000000`LgT0P0000040<7=h`8000001032cN<20000
00@0`lkR00D000000<C=hP00000000000<C>hP02000000@0aLgQ0P0000050<K=h00300000037cMl0
algO0080algO0P0000000`37cMh000000<S=gP02000000@0b<cM00<000000<S<g@38c=d00`38c=d0
10000000b<_L00000039bm`2000000@0bL_K00<000000<W:fP39b]X00P39b]X2000000050<W9f@00
0000b/WI0<[9f@0000000`3:b=P00`000000b/SH000000040<[7e`030000003:a]L0b/KG00<0b/KG
00@000000<[6eP000000blKF0P0000040<[5e@030000003:a=@0b/CD0080b/CD00<000000<[3e000
0000103;`m<010000000b/;C0000003;`]82000000@0b/;B00<000000<_1d@3;`M400P3;`M400`00
0000b/3@000000030<_0d0050000003;_ll000000000003;_ll00P0000030<^ocP030000003<_/d0
c;k=0080c;k=00<000000<^mc00000000`3<_/`01P000000c;c;0<blb`000000cKk;000000@0cKc:
00@000000<nlb03?_<P0ckc80`0000000`00::d0000001`aZ`02000000<0Lf2_00<0000009=_YP2C
KjH00P2CKjH00`000000XGj]000000030:^2Z0030000002cT;40/i2a00<0/i2a00<000000<6R^P31
X[X01031X[X2000000D0aJ_000<000000<>/``0000001037/L@00`000000aK76000000040<Ndb003
00000036]<T0000000@0b;S;00<000000<Jhc00000001038^/d00`000000ak[>000000040<Rlc`03
00000037_=00000000D0b;kA00<000000<RodP38_m801038_m800`000000b<7D0<S1e0040<S1e003
00000038`]D0b<;E00<0b<;E0P0000050<S3eP0300000038aML0000000<0alGG0P0000001038a]P0
00000<O6f037a]P200000080b<OI00@000000<O7f@00000000000`38b=X00`000000alSJ00000005
0<S9f`0300000038b]/0b<[K0080b<[K00<000000<S;g00000001037bm`00`000000b<_M00000002
0<O;g@8000000P37c=d010000000alcN0000000000040<O=gP0300000037cMl0algO00<0algO00<0
00000<K>h036c^000P36c^02000000030<K>h0000000a/kP0080a/kP00D000000<K?h@0000000000
0<G>h@0200000080aLoQ00@000000<G>hP00000000001035cn800`000000a<oR0<C?hP030<C?hP03
00000033cn<0`loS0080`loS0P0000000`33cn<000000<?>h`020<?>h`0300000033cn@0000000<0
`/kS0P0000000`32c^@000000<;>i003000000030<7>i0000000`LgT008000000P31cN@010000000
`<gT0000000000030<3=i0040000002oc>@0000000000080_lcT00@000000;o;i000000000000`2n
bn@010000000_/_T0000000000020;k:i0040000002mb^@0000000000080_LWT00@000000;g9i000
000000000`2lbN@00`000000oooo0?ooo`0E0?ooo`<00000303oool00`000000oooo0?ooo`0V0?oo
o`000`3oool00`3ogi`0B00007Boo`030?ooo`030?noM00004P0W=oo00H0oooo00L0gmoO0?ooo`3o
okl0M0000000000007@0_ooo00/0oooo0P00000k0?ooo`030000002e`n@0]L?T0080]L?T00<00000
0;K3i00000001@2f`n@2000000D0]lCU0P0000040;O5i@030000002ha^D0^<KU0080000000<0^<KU
0000000000000P2hanD2000000030;W7i@00000000000080^LOU0P0000000`2janD0000000000002
0;[8i@80000000<0^lSU0000000000000P2kbND2000000030;c9i@00000000000080_<[U0P000000
0`2lb^@00000000000020;g;i@80000000@0_L_T000000000000_/_U0P0000030;k;i0040000002o
c>@00000000000@0_lcT0P0000050<3<i00300000031cN<0`LgS00<0`LgS00<000000<;=h`000000
0P32cN<200000080`lgR00@000000<?>hP00000000001034cN800`000000aLgQ0<G=h@030<G=h@04
00000035cN00000000000080a/gP0P0000000`36cMl000000<O=g`02000000@0algN00<000000<S<
g@38c=d00`38c=d01@000000b<cM000000000000b<cM008000000`38bm`00`000000bL_K0<W;f`03
0<W;f`0400000039bMX000000<W:fP8000000`39bMT00`000000b/WI000000050<[8f0030000003:
amL0b/OG0080b/OG00D000000<[6e`00000000000<[6e`02000000<0b/KF00<000000<[5e@3:aMD0
0`3:aMD00`000000b/CE000000020<[4e0060000003:`m@0b/?D0<[3e0000000bl?C0P0000030<[2
d`030000003:`]80b/;B00<0b/;B00<000000<[1d@0000000P3;`M401@000000b/3@0<[0d0000000
bl3@00800000103;_ll00`000000bko>0<^ocP020<^ocP030000003;_Lh000000080c;k=00L00000
0<^mc03;_L`0bkg<0000003<_/`0000000@0c;c;00<000000<flbP3=_<X00P3=_<X00`000000cK[8
000000020<nlb0D000000P0L<J/01@000000Lf2_07=P[`1cH:l0000000@0TfnV0P0000040:^2Z003
0000002cT;40/i2a0080/i2a0P0000060<6R^P0300000035Zl00aJ_000D0aJ_000<000000<Naa037
/L@01037/L@00`000000akC80<Ndb0040<Ndb00300000038^</0b;S;00@0b;S;00<000000<Rjc@38
^/d01038^/d00`000000b;c?0<Rlc`040<Rlc`0300000038_]40b;kA00<0b;kA00<000000<Vod@00
00001038_m800`000000b<7C000000030<S1e080000000@0b<;D00000038`]D0b<;E0P0000020<S3
eP0300000038`mH0b<?F008000000P38aML010000000alGG0000000000030<S6f00300000037a]P0
000000D0b<OI00<000000<S8fP38b=X01038b=X00`000000b<WK0<S9f`020<S9f`80000000D0b<[K
00000038b]/0b<[K000000030<S;g00400000037bm`0000000000080b<_M00<000000<O;g@000000
1@37c=d00`000000algN0<O=gP030<O=gP0300000037c]l0000000<0algO00@000000<O>g`000000
a/kP0P0000030<K>h00400000036c^000000000000@0a/oQ00<000000<G?h@35cn400`35cn400`00
0000aLoR0<G?hP020<G?hP80000000<0a<oR00000034cn800P34cn8010000000a<oS00000033cn<2
000000<0`loS00@000000<?>h`00000000000P33cn@010000000`/kS0000000000040<;>i0800000
1031c^@00`000000`LgT0<7=i0030<7=i00300000030cN@0`<gT00<0`<gT0P0000050;o<i0030000
002nbn@0_/_T00<0_/_T0P0000040;k:i08000001@2mbN@00`000000_<WT0;c9i0030;c9i0800000
4P3oool4000000l0oooo00<000000?ooo`3oool09P3oool000<0oooo00<0omnL04P0001d_ol00`3o
ool00`3o_g@0001809cOo`060?ooo`060=nLB000000000000000000004P0W=oo2`3oool00`000000
oooo0?ooo`0l0?ooo`040000002e`n@00000000000<0]/?T00<000000;K3i02f`n@00P0000000`2g
a>@00000000000020;O4i@80000000<0]lCT0000002gaND0102gaND2000000@0^<KU0`0000050;W7
i@8000001@2janD2000000D0^lSU0P0000050;c9i@8000001@2lb^@2000000D0_L_T00<000000;k;
i02nbn@0102nbn@00`000000_lcT0;o<i0030;o<i00400000030c><0000000000080`<cT0P000000
1031c><0000000000031cN<2000000<0`/gS0P0000050<?=hP0300000034cN80a<gR00<0a<gR00D0
00000<C=h@00000000000<G=h@02000000@0aLgP0P0000040<K=g`0300000037cMh0algN00<0algN
00D000000<O<gP00000000000<S<g@02000000@0b<cM00<000000<S;g038bm`00P38bm`01@000000
b<[K000000000000bL_K008000000`39bMX00`000000bLWI0<W9f@040<W9f@030000003:b=P0b/SH
00<0b/SH00@000000<W7f0000000b/OG0P0000040<[6e`030000003:a]H0b/KF0080b/KF00<00000
0<W5eP0000000`3:aMD01P000000b/CE0<[4e@000000b/CD000000D0b/?D00<000000<[2d`3:`]<0
0P3:`]<00`000000b/;B000000020<[2dP8000000P3:`M400`000000bl7A000000040<[0d0030000
003;_ll0bko?00<0bko?00<000000<Znc`0000000P3;_lh01P000000bkg>0<^mcP000000c;k=0000
00D0bkg<00<000000<blb`3<_</00P3<_</00`000000c;_:000000020<flbP060000003=^/P0cK[8
0000003?_<P000000P3?^/H300000080736[0P00000207=P[`030000002CKjH0TfnV0080TfnV00<0
00000:^2Z02[PZP0102[PZP01`000000/i2a0;>@/@2cT;4000000<FR]@0000001031X[X00`000000
bJbm000000050<F[`08000001@37/L@2000000D0akC80P0000050<Rhb`8000001@38^/d2000000D0
b;c?00<000000<Vnd00000000`38_]4010000000bKoA0<Vod@0000030<RodP0500000038`M<0b<7C
00000038`M@00P0000030<S2e00300000038`]D0000000@0b<?F00<000000<S3eP0000001@38aML0
0`000000b<KH0<S6f0040<S6f00300000038amT0b<OI00<0b<OI00<000000<S8f@0000000`38b=X2
000000030<S9fP000000b<WK008000000`38b]/00`000000b<[K000000050<S;g00300000038bmd0
b<_M00<0b<_M00<000000<O<g@37c=d00P37c=d2000000040<S=gP000000algN0<O=gP8000000P37
c]l010000000algO0000000000030<O>g`0300000036c^00a/kP00@0a/kP00<000000<K?h@36cn40
0P36cn42000000030<K?h@000000aLoQ0080aLoQ00@000000<G?hP000000aLoR0P0000030<C?hP04
00000034cn800000000000<0a<oS00<000000<??h`33cn<01033cn<00`000000`loT0<??i0030<??
i00300000032c^@0`/kT0080`/kT0`0000000`31c^@000000<7>i002000000040<7>i0000000`LgT
0<7=i080000000@0`<gU00000030cN@0`<gT0P0000020;o=i@030000002oc>@0_lcT0080000000@0
_lcU0000002nbn@0_/_T0P000000102nbn@000000;k:i02nb^@200000080_L[T00<000000;g9i02m
bN@00P000000102lbN@000000;c9i02lbN@2000000030;c8i0000000oooo00h0oooo0`00000C0?oo
o`030000003oool0oooo02H0oooo00030?ooo`030?oo_`1d0000B9cO00<0oooo00<0gia80000M02o
ool01P3oool0103ookl0M00000000018W=l=0?ooo`030000003oool0oooo03d0oooo00<000000;K3
i02f`n@0102f`n@2000000D0]lCT0P0000040;O4i0030000002gaND0]lGU0080000000<0^<GT0000
00000000100000000`2ia^@00000000000020;W7i@80000000<0^/OT0000000000000P2janD20000
00030;_8i0000000^lSU00<0^lSU00<000000;_8i0000000102lbND010000000_<WT000000000002
0;c:i080000000<0_L[T0000000000000P2mbn@2000000030;k:i000000000000080_/_T0P000000
102obn@000000000002oc>@2000000@0`<cS0P0000040<7<h`8000001032cN<010000000`/cR0000
000000020<?=hP80000000@0`lgQ000000000000a<gR0P0000040<C=h@0300000035cN00aLgP00<0
aLgP00<000000<G<g`0000000`36cMl01@000000a/cN000000000000algN008000001037c=h00`00
0000b<cM0<S<g@020<S<g@80000000<0b<_L00000038bm`00P0000040<S:f`0300000039bMX0bLWJ
0080bLWJ00<000000<W9fP0000001039bMT01@000000bLSI000000000000b/SH008000000`39amP0
0`000000b/KG0<[6e`030<[6e`0400000039a]H000000<[6eP8000000`39aMH00`000000b/GE0000
00040<[4e@030000003:`m@0b/?D00<0b/?D00<000000<W2d`0000000P3:`]<01P000000b/;B0<[2
dP000000b/;B000000D0b/7A00<000000<[0d03:`=000P3:`=000`000000b[o@000000030<^oc`06
0000003:_/l0b[k?0000003;_lh00000103;_Lh00`000000bkg<0<^mc0030<^mc0030000003;^l/0
00000080c;c;00H000000<bkbP3<^lX000000<flbP0000040<fjb0040000003?^/H0ck[60<njaP@0
000000H0736[0000001DB:L0000007=P[`00000309=_YP8000001P2[PZP010000000/i2a0;>@/@00
00020<FR]@0500000031X[X0`J:j0<6R^P0000000P39[;d00`000000aJ_00<F[`0020<F[`0040000
0039/L80bK72000000<0ak7400@000000<VeaP39]LH000000`37]<P010000000bKS90<Vhb@000002
0<Rhb`8000000P39^/`00`000000b;[=0<Rjc@0200000080bKc>00<000000<Rlc`38_<l00P000003
0<Vnd00300000038_]40000000@0bKoA00<000000<RodP0000001038`M<00`000000b<;D0<S2e004
0<S2e00300000038`mH0b<?F00@0b<?F00<000000<S5e`38aML00`38aML00`000000bLKG00000004
0<S6f00300000038amP000000080b<OI0P0000020<S8f@0400000038b=X00000000000@0b<WJ00<0
00000<S:f`38b]/01038b]/00`000000b<_L0<S;g0030<S;g00300000038c=`0000000<0b<_M00@0
00000<S<g@000000alcM0P0000030<S=gP0300000037cMh0000000D0alkO00<000000<O>g`37c]l0
0P37c]l00`000000aloP000000030<K>h080000000<0a/oQ00000036cn400P0000030<K?h@040000
0035cn400000000000<0aLoR00<000000<C?hP34cn801034cn800`000000a<oS0<C?h`020<C?h`03
00000034d><0000000<0`loS0P0000001033cn@000000<??i033cn@2000000030<;?i0000000`/kT
008000000`31cn@3000000@0`LkT0P0000040<3=i@8000001@2ocND2000000@0_lcU0P0000040;k;
i08000001@2mb^@2000000@0_<WT0P0000040;c8i0030000003oool0oooo00T0oooo0`00000F0?oo
o`030000003oool0oooo02H0oooo00040?ooo`030=nLB000B9`0gooo0080oooo00<0_g@0001d_`3o
ool01`3oool00`2oM000M;oo0?ooo`0=0?ooo`030000003oool0oooo03h0oooo0P0000020;K3i080
000000<0]/?T0000002ga>@00`2ga>@00`000000]lCT000000050;O4i08000000P2haN@5000000D0
^LKT0P0000050;[7i0800000102kb>@010000000^lSU0000000000030;_8i0040000002lbND00000
000000@0_<WT0P0000050;g:i08000001@2nb^@2000000D0_l_T00<000000<3<h`30c><00`30c><0
0`000000`L_S000000020<7<h`8000000P31c>8010000000`/gS0000000000040<;<hP8000001@33
cN400`000000a<gQ0<C=h@030<C=h@0500000034c>00000000000035cN000P0000020<G<g`040000
0036cMl00000000000@0a/cN00<000000<O<gP37c=h00`37c=h010000000al_M00000038c=d20000
00@0b<_L00<000000<S:f`38b]/00`38b]/00`000000b<WK000000020<W9fP0600000039bMX0bLWJ
0<W9fP000000bLWI0P0000040<W8f@0300000039amP0bLOH0080bLOH00D000000<W6e`0000000000
0<[6e`02000000<0bLKF00<000000<W5eP39aMH01039aMH00`000000b/CE0<[4e@020<[4e@050000
0039`m@000000000003:`m@00P0000020<W2d`030000003:`]<0000000@0b/;B00<000000<[1d@3:
`M400`3:`M400`000000bL3A000000020<[0d0070000003:_m00b[o@0<Zod0000000bko?00000004
0<Znc`030000003;_Lh0bkg>0080bkg>00<000000<^lc@0000000`3;_L`01P000000bk_;0<^kb`00
0000c;c;000000@0c;_:00<000000<fjb03=^/P00P3=^/P00`000000cKS6000000030<njaP<00000
0P0L<J/02@000000E4RW0000001cH:l0000009=_YP000000VFVJ000000060:^2Z0030000002cT;40
000000@0aJ:e00@000000<6R^P31X[X000000`39[;d010000000aJ_00<F[`00000030<Va`P040000
0037/L@0ak74000000<0bKG600@000000<Ndb037]<P000000`39^<T00`000000b;S;000000040<Vj
c00300000038^/d0000000@0bKc>00<000000<Rlc`0000001P39_]000`000000bKoA0<Vod@040<Vo
d@0300000038`M<0b<7C0080b<7C00<000000<W2e00000001038`]@00`000000bL?E000000040<S3
eP0300000039aMH0000000<0b<GG00H000000<W6e`39a]L0bLKG00000038a]P200000080b<OH00<0
00000<S7f@0000001@38b=T00`000000b<WJ0<S9fP030<S9fP0300000039b]/0000000<0b<[K0P00
00001038bm/000000<S;g038bm`200000080b<cL00@000000<S;g@00000000000`38c=d00`000000
b<gN0<S=gP040<S=gP0300000037c]l0alkO0080alkO0P0000001@37c]l000000<O>g`37c]l00000
0080aloP00@000000<K>h000000000001036cn400`000000a/oQ0<K?h@040<K?h@0300000035cn80
aLoR0080aLoR00<000000<G@hP0000000`34cn82000000030<C@h`000000a<oS008000000P34d><0
10000000`loS0000000000040<??i08000001032cn@00`000000`LoT0<7?i0030<7?i00300000031
c^@000000080`LkT0P0000000`30c^D000000<3=i@020<3=i@030000002oc^D0000000<0_lgU0`00
00030;o<i@80000000<0_/cU0000002nbn@00P2nbn@00`000000_L_U000000030;g:i080000000<0
_<[U0000002lbN@00P2lbN@00`000000_<WU000000030;c8i0030000002kb>@0000000H0oooo1000
000I0?ooo`030000003oool0oooo02H0oooo00040?ooo`070?oo_`1d0000M;oo0?ooo`3o_g@0001d
0;ooo`080?ooo`060?oOW018000000000000000004P0W=oo2P3oool00`000000oooo0?ooo`100?oo
o`800000102f`n@010000000]lCT0000000000030;O4i0030000002ga>@0]lCT0080000000D0^<CT
00000000000000000;S5i002000000030;W5i0000000^LKT00<0^LKT00<000000;[6i0000000102j
an@00`000000^/OT000000050;_8i0030000002kb>@0^lST00@0^lST00<000000;c9i02lbN@00`2l
bN@00`000000_LWT000000040;g:i0040000002nb^<0000000000080_/[T0P0000000`2ob^<00000
000000020;o;i080000000@0_l_S000000000000`<cS0P0000030<7;h`8000001@31c>800`000000
`/cR0<;<hP030<;<hP0300000033c>40000000@0`lgQ00D000000<C<h@00000000000<C=h@020000
00@0a<cP00<000000<G<g`35c=l00`35c=l00`000000a/cN0<K<gP030<K<gP0500000036bmh00000
00000037c=h00P0000030<O;g@0300000038bm`0b<_L00<0b<_L00D000000<S:f`00000000000<S:
f`0200000080b<WK00<000000<W9fP0000001@39bMX00`000000bLSI0<W8f@020<W8f@80000000<0
bLOH00000039amP00P0000040<W6e`0300000039a]H0bLKF0080bLKF00<000000<W5eP0000001039
aMH010000000bLCE0000003:a=D2000000@0bL?D00<000000<W2d`39`]<00`39`]<00`000000b/;B
0<[2dP020<[2dP0500000039`M8000000000003:`M400P0000020<W0d@030000003:`=00000000D0
b[o@00<000000<Znc`3:_/l00P3:_/l00`000000b[g>000000020<^mcP070000003;_<d0bkc=0<^l
c@000000bkg<000000040<^kb`030000003<^lX0c;_:0080c;_:00<000000<bib00000000P3=^/P0
20000000cKS60<fhaP3=^<H000000<njaP000000ckS30`000000100L<J/0000005A8Y`1DB:L30000
00<0VFVJ00<000000:^2Z02[PZP0102[PZP2000000@0aJ:e00<000000<6R^P0000001039[;d00`00
0000aJ_0000000050<Va`P8000001@39]LH2000000D0bKS900<000000<Vjc039^/`01039^/`00`00
0000bKc>0<VlcP040<VlcP0300000039_]00bKk@00@0bKk@0P0000050<Vod@0300000039`M800000
0080b<7C00@000000<W2e039`]@000000`38`]@01P000000bL?E0<W3e@39`mD000000<S3eP800000
0`39aMH00`000000b<GG000000050<W6e`0300000038amP0b<OH00<0b<OH00<000000<S8f@38b=T0
0`38b=T00`000000bLWJ000000020<S9fP8000000P39b]/010000000b<[K0000000000030<S;f`03
00000038bm`0000000D0b<cL00<000000<S<g@38c=d00P38c=d00`000000b<gN000000030<S=gP80
000000<0b<kN00000037c]l00P0000030<O>g`0300000037c]l0000000@0aloP00<000000<K?h@36
cn400P36cn42000000030<K@h@000000a/oQ0080a/oQ0P0000000`36d>8000000<G?hP0200000080
aM3R00@000000<C?hP00000000001034d><00`000000a=3S0<C@h`030<C@h`0300000033cn@0`loT
0080`loT0`0000030<;?i080000000@0`/oU00000031cn@0`LoT0P0000000`31cnD000000<7>i002
000000<0`<kU00@000000<3=i@00000000000P2oc^D010000000_lgU0000000000030;o=i@<00000
0`2nc>D010000000_/_T0000000000020;g;i@040000002mb^@00000000000<0_<[U00@000000;c9
i000000000000P2lbND010000000_<ST0000000000020;_8i0040000003oool0oooo0?ooo`<00000
7@3oool00`000000oooo0?ooo`0V0?ooo`001@3oool01@3ookl0M0000000000007@0_ooo00X0oooo
00D0_g@0000000000000000004RLg`0:0?ooo`030000003oool0oooo0440oooo00<000000;K3i02f
`n@00`2f`n@00`000000]lCT0;O4i0040;O4i0D000000P2ha>@2000000@0^LGT00@000000;W6i000
000000000`2ja^@010000000^/OT0000000000030;[7i0030000002kb>@0^lST0080000000<0^lOT
0000000000000P2kb>@2000000040;c8i000000000000;c9i08000000`2mbN@010000000_L[T0000
000000040;k:h`8000001@2ob^<2000000@0_l_S0P0000040<7;h`0300000031bn80000000@0`LcR
00D000000<;<hP00000000000<;<hP02000000<0`lcQ00@000000<?=h@00000000001034c>400`00
0000a<cP0<C<h0020<C<h080000000@0aLcO00000035c=l0aLcO0P0000001036c=l0000000000036
c=h2000000@0a/_N00<000000<O;g@37bmd00P37bmd01@000000al[L000000000000b<_L00800000
1038b]/00`000000b<WK0<S9f`030<S9f`0300000039bMX0bLWJ00<0bLWJ00@000000<S8f@000000
bLSI0P0000040<W7f00300000039a]L0bLKG00<0bLKG00<000000<W6e`0000000P39a]H01P000000
bLGF0<W5eP39aMH000000<W5eP8000000`39a=D00`000000bL?D0<W3e0030<W3e00300000039`]@0
000000<0bL;C00@000000<W1d`000000b/;B0P0000040<W1dP0300000039`=40bL3A00<0bL3A00<0
00000<Zod03:_m000`3:_m000`000000bKk?000000020<Znc`060000003:_Lh0b[g>0000003;_Lh0
00001@3;_<d00`000000bk_;0<^kb`020<^kb`030000003;^/X000000080c;_:00H000000<bib03<
^LP000000<fjb00000050<fhaP030000003?^<<0ckS300<0000000<0736[0000001DB:L00P1DB:L0
0`000000VFVJ09UYVP0209UYVP030000002[PZP0Zh:X00@0Zh:X00<000000<FR]@35X[D01035X[D0
0`000000bJbm0<V/_@040<V/_@0300000039/L80bK7200@0bK7200<000000<VeaP39]LH01039]LH0
0`000000bKS90<Vhb@030<Vhb@030000003;^l/0000000@0bK[<00<000000<^mc@0000001039_<h0
0`000000b[k?000000030<Vnd08000000P3:`=400`000000bKoA0<Vod@0200000080bL7B00<00000
0<S1d`0000001039`]@00`000000b<;D000000050<W3e@0300000039aMH0bLGF00@0bLGF00<00000
0<W6e`39a]L00`39a]L00`000000bLOH000000030<S7f00500000039b=T0000000000038b=T00P00
00020<W9fP0300000038bMX0000000D0bL[K00<000000<S;f`38bm/01038bm/00`000000b<cL0<S<
g0020<S<g080000000D0b<cM00000038c=d0b<cM000000020<S=gP0400000038cMh00000000000@0
b<kN00<000000<O>g`37c]l01037c]l00`000000aloP0<O?h0020<O?h00400000037cn4000000<K?
h@8000000`36d>4010000000a/oQ0000000000040<K@hP0300000035d>80aM3R00<0aM3R00<00000
0<C@h`34d><00P34d><2000000030<C@h`000000a=3S0080a=3S00@000000<?@i0000000`loT0P00
00030<?@i0<000001032cnD2000000@0`LoU00<000000<3>i@0000000`30c^D2000000D0_lkU00<0
00000;o=i@2ocND00`2ocND00`000000_/cU0;k<i@030;k<i@8000001@2mbnD00`000000_<[U0;c:
i@030;c:i@800000102lbND2000000D0^lST0`00000P0?ooo`030000003oool0oooo02H0oooo000S
0?ooo`030000003oool0oooo0480oooo00<000000;K3i02f`n@00P0000000`2g`n<0000000000002
0;O4i0D000000P2ha>@2000000030;W5h`000000^LGT00@0^LGT00<000000;[6i02ja^@0102ja^@0
0`000000^/OT0;[7i0040;[7i08000001@2kan@2000000D0_<ST00<000000;g9i02mbN@0102mbN@0
0`000000_/[S0;k:h`030;k:h`030000002nbN<0000000<0_l[S0P000000102ob^<000000;o;h`2o
bn<200000080`<_R00@000000<7;h`00000000000`31bn8010000000`LcR0000000000040<;<hP03
00000033c>40`lcQ00@0`lcQ00<000000<C<h@34c>400`34c>4010000000a<cP00000034c>020000
00@0aLcO0P0000050<K<g`0300000036bmh0a/_N00<0a/_N00@000000<K;g@000000al_M0P000004
0<O:g00300000038b]/0b<[K00<0b<[K00<000000<S9f`0000000`38bM/01@000000b<SJ00000000
0000bLWJ008000000`38b=T00`000000bLOH0<W7f0030<W7f00300000039a]P0000000<0bLKG00H0
00000<W6e`39a]L000000<W6eP0000050<W5eP0300000039a=D0bLCE0080bLCE00<000000<W3e000
00000`39`m@01@000000bL;D0<W2e0000000bL;C008000000`39`M<00`000000bL7B0<W1dP030<W1
dP0300000039_m40000000<0bL3A00D000000<Vod000000000000<Zod00200000080bKk?00<00000
0<Znc`000000103:_Lh00`000000bkc=0<^lc@030<^lc@030000003:^l`000000080bk_;00H00000
0<^jbP3;^/X000000<bkbP0000040<bib0030000003=^<H0cKS600<0cKS600<000000<bfa0000000
0P3?^<<4000000<0E4RW00<0000009UYVP2IJIX00P2IJIX00`000000Zh:X0:^2Z0030:^2Z0800000
1@35X[D2000000D0bJbm0P0000050<Va`P030000003=]lD0000000@0bKG600<000000<bib0000000
0`39^<T010000000bk_;0<^kb`0000030<Vjc0040000003;_Ld0bkg=000000<0bKc>00L000000<Zn
c`3:_/l0b[k?00000039_]00000000@0b/3A00<000000<Vod@0000001@39`M800`000000bL;D0<W2
e0040<W2e00300000039`mD0bL?E00<0bL?E00<000000<W5eP0000000`39aMH2000000040<W6e`00
0000bLKG0<W6e`8000000P39amP010000000b<OH0000000000040<W8f@0300000039bMX0bLWJ00<0
bLWJ00<000000<W:f`39b]/00`39b]/00`000000bL_K000000030<S;f`80000000<0bLcL00000038
c=`00P0000030<S<g@0300000038c=d0000000@0b<gN00<000000<S>gP38c]h00`38c]h00`000000
b<oO000000030<O>g`80000000<0aloP00000037cn000P0000030<O?h@0300000036d>40a]3Q00@0
a]3Q00<000000<K@hP36d>800P36d>82000000030<K@hP000000aM3R0080aM3R00@000000<G@h`00
0000a=3S0P0000030<C@h`0400000034d><00000000000<0`m3T00<000000<?@i033d>@00`33d>@0
0`000000`/oU0<;?i@020<;?i@80000000<0`LoU00000031cnD00P31cnD010000000`LoU00000030
c^D300000080`<kU00<000000;o>i@2oc^D00P000000102oc^D000000;o=i@2ocND2000000040;k=
i@000000_/cU0;k<i@8000000P2nc>D00`000000_L_U0;g;i@02000000040;g;i@000000_<[U0;c:
i@80000000@0_<[U0000002lbND0_<WU0P0000020;_9i@030000002kb>@0^lST0080000000<0^/SU
0000003oool08@3oool00`000000oooo0?ooo`0U0?ooo`008`3oool3000004<0oooo0P0000050;O3
h`<00000102ha><2000000@0^LGS00<000000;W5i02iaN@00P0000000`2iaN<00000000000020;[6
i080000000<0^/KS0000000000000P2jan@2000000030;_7h`00000000000080^lOT0P0000000`2l
b><00000000000020;c8i080000000<0_LSS0000000000000P2mbN@2000000040;g9h`0000000000
0;k:h`8000000P2nbN<010000000_l[S0000000000040;o:h`8000001@30bn800`000000`L_R0<7;
hP040<7;hP0300000032c>80`/cR00<0`/cR00@000000<;;h@00000000000P33c>42000000040<?;
h@00000000000<C<h@8000000`34c>000`000000aLcO0<G<g`030<G<g`0300000035bml0000000@0
a/cO00D000000<K;gP00000000000<K;gP02000000<0a/_M00<000000<O:g037b]`00`37b]`01@00
0000alWL000000000000b<[K008000000P38bM/010000000b<WK0000000000040<S8fP0300000038
b=T0b<SI0080b<SI00D000000<S7f@00000000000<W7f002000000<0bLKH00<000000<W6e`000000
1039a]L00`000000bLGF0<W5eP030<W5eP0400000039a=H000000<W4e@8000000`39`m@00`000000
bL?D000000040<W2e00300000039`M<0bL7C0080bL7C00<000000<W1dP0000000`39`M801@000000
bKoA0<Vod@000000bL3A008000001039_m000`000000bKk?0<Vnc`030<Vnc`030000003:_Lh0b[g>
0080b[g>00<000000<Vlc@0000000`3;_<d01P000000b[_<0<Zkc0000000bk_;000000@0bk[:00<0
00000<bib03<^LP00P3<^LP00`000000bkO6000000030<fhaP070000003<]/@0c;K40000003?^<<0
00000<jd`@020000000501`aZ`000000E4RW05A8Y`000000102IJIX00`000000Zh:X0:^2Z0030:^2
Z0030000003?Z;00000000@0aJ:e00<000000<n_^P0000000`39[;d010000000c[?00<jc`0000003
0<Va`P040000003=]lD0cKO5000000<0bKG600@000000<bib03<^LP000000`39^<T01`000000bk_;
0<^kb`3;^l/000000<Vjc0000000103;_Ld00`000000bKc>000000050<Znc`030000003:`=40b/3A
00@0b/3A00<000000<W1dP39`M800`39`M82000000D0bL;D00<000000<[4e00000000P39`mD20000
0080bLGF00@000000<W5eP00000000000`39a]L00`000000bLKG000000050<W7f00300000039b=T0
bLSI00<0bLSI00<000000<W9f@0000000`39bMX00`000000bL[J000000030<W:f`0500000039bm/0
bL_K00000038bm/00P0000040<W<g00300000038c=d0b<cM00@0b<cM00<000000<S=gP38cMh00P38
cMh01@000000b<kN000000000000b<kN008000000P38cml010000000alkO0000000000040<O?h003
00000037cn40aloQ0080aloQ00<000000<O@h@0000000`36d>42000000030<K@hP000000a]3R0080
00000`36d>8010000000aM3R0000000000030<G@h`0300000034d><0a=3S00<0a=3S0P0000030<?@
i080000000@0`m3T00000033d>@0`m3T0P0000000`32d>D000000<;?i@02000000<0`LoU00@00000
0<7?i@00000000000`31cnD2000000D0`<kU0P0000040;o>i@800000102ncND2000000@0_/cU0`00
00030;g;i@<00000102lb^D2000000D0^lWU0P0000030;[8i@030000003oool0oooo0200oooo00<0
00000?ooo`3oool09@3oool00280oooo0P0000160?ooo`8000000P2g`n<2000000030;S3h`000000
^<CS00<0^<CS00<000000;S4h`0000001@2iaN<2000000D0^LGS0P0000050;[6h`8000001@2kan<2
000000@0_<SS0`0000040;g8h`<00000102mbN<2000000D0_/WS00<000000;o:h`2ob^<00`2ob^<0
0`000000`<[R000000040<3;hP0400000031b^80000000000080`L_R0P0000001031bn8000000000
0032c>82000000@0`/_Q0P0000040<?;h@8000001034c>001@000000a<_O000000000000aLcO0080
00000`35bml010000000a/cO0000000000040<K;gP0300000036bmd0a/_M0080a/_M00D000000<K:
g@00000000000<O:g002000000@0alWL00<000000<S9f`38bM/00`38bM/00`000000b<SJ0<S8fP03
0<S8fP0400000038b=X000000<S8f@8000001038amT00`000000bLKH0<W6f0040<W6f00300000039
a]L0bLKG0080bLKG00D000000<S5eP00000000000<W5eP02000000<0bLCF00<000000<W3e039`m@0
1039`m@00`000000bL;D0<W2e0020<W2e00300000039`M<000000080bL7C00L000000<W1dP39`M80
bL7B00000039`M80000000@0bKoA00<000000<Vod039_m000`39_m000`000000bKk@000000030<Vn
c`0300000039_Lh000000080b[g>00L000000<Vlc@39_<d0bKc=0000003;_<d0000000@0b[_<00<0
00000<^jbP3;^/X00P3;^/X00`000000b[S9000000020<bib0070000003;]lH0bkO60<^gaP000000
cKS6000000040<bfa0030000003>]<40c[C100@000000P1DB:L00`000000VFVJ09UYVP0309UYVP05
0000002[PZP0Zh:X0:^2Z00000000P3?Z;001@000000aJ:e0<FR]@35X[D000000080cjnj00@00000
0<V/_@39[;d000000`3>/l0010000000bK720<Va`P0000040<fga@0300000039]LH0000000@0c;W8
00<000000<Vhb@0000001@3;^l/00`000000bkg=0<^mc@040<^mc@030000003:_/l0b[k?00<0b[k?
0P0000050<[0d@030000003:`M8000000080bL7B0P0000020<[3d`0300000039`]@0bL;D00800000
0P3:a=@00`000000bL?E000000050<W5eP0300000039a]L0bLKG00@0bLKG00<000000<W7f039amP0
0`39amP00`000000b/SH000000020<W8f@8000000P39bMT010000000bLWJ0000000000030<W:fP03
00000039b]/0000000@0bL_K00<000000<W<g039c=`00`39c=`00`000000bLgL000000030<S<g@80
000000<0bLgM00000038cMh00P0000040<S>gP0300000038cml0b<oO00<0b<oO00<000000<O?h037
cn000P37cn02000000050<O@h@000000aloQ0<O?h@0000000P37d>4010000000a]3Q000000000004
0<K@hP0300000036d>80a]3R00@0a]3R00<000000<G@h`35d><00P35d><00`000000aM7T00000002
0<C@h`80000000<0a=7T00000033d>@00P0000040<?@i08000001032d>D00`000000`LoU0<7?i@03
0<7?i@8000001031cnD010000000`<oV0000000000020<3>i@<000000`2oc^D2000000030;k>iP00
0000_/gU0080_/gU00<000000;k=iP0000000P2nc>D2000000040;g<iP000000_L_U0;g;i@800000
0P2lbnD01`000000_<[U0;c:i@2lb^D000000;_:i@0000000`2kbND3000000@0^/SU0P00000Q0?oo
o`030000003oool0oooo02D0oooo000R0?ooo`030000003oool0oooo04H0oooo0`0000040;S3h`04
0000002ha><00000000000<0^<CS00<000000;W5h`2iaN<00P0000000`2iaN<00000000000020;W5
h`80000000<0^/GS0000000000000P2ja^<2000000030;_6h`00000000000080^lOS0P000000102l
an<000000;c8h`2lb><200000080_<OS00<000000;g8h`2mb><00P0000020;g8h`030000002mbN<0
_LWS008000000P2nbN<00`000000_/WS0;k9h`02000000040;o9hP00000000000;o:h`8000000`30
b^8010000000`<_R0000000000040<7:hP8000001@31bn800`000000`/_Q0<;;h@030<;;h@030000
0033bn4000000080`l_Q0P0000020<C;h00400000034c>000000000000@0a<_O00<000000<G;g`35
bml01035bml00`000000a/_N0<K;gP020<K;gP80000000<0a/[M00000036bmd00P0000040<K:g@03
00000037bM`0alWL0080alWL0P0000000`37bM/000000<S9f`020<S9f`0500000038b=X000000000
0038b=X00P0000030<S8fP0300000038amT0b<OI00<0b<OI00<000000<S6f00000000`39a]P20000
00030<S5e`000000bLKG008000001038aMH00`000000bLCF0<W4eP020<W4eP0300000038`mD00000
00<0bL?D0P0000000`38`]@000000<W2e00200000080bL7C00<000000<W1d`0000001@39`M800`00
0000bKoA0<Vod@020<Vod@0300000038_]4000000080bKo@0P0000020<Vnd00400000039_/l00000
00000080bKg>00<000000<ZmcP0000001@39_<d00`000000b[_<0<Zkc0020<Zkc00300000039^L/0
00000080bk[:00H000000<Zhb@3:^<T000000<bib00000050<^gaP030000003<]/@0c;K40080c;K4
0P0000030<jd`@<000000P1DB:L00`000000VFVJ09UYVP0309UYVP040000002[PZP0Zh:X000000<0
cjR`00@000000<FR]@35X[D000000`3?[kX010000000bJbm0<V/_@0000040<jc`08000001P3=]lD0
0`000000c;W80<bib0040<bib0030000003;^l/0bk_;00<0bk_;0P0000050<^mc@030000003;_lh0
00000080b[k?0P0000020<_0d0030000003:`=40b/3A008000000P3:`M800`000000bL7B00000004
0<[3d`0300000039`]@0000000D0b/CD00<000000<W5eP39aMH00`39aMH00`000000b/KF00000003
0<W6e`80000000@0b/OG00000039amP0bLOH0P0000020<[8f00300000039b=T0000000D0bLWI00<0
00000<W:fP39b]X01039b]X00`000000bL_K0<W;f`020<W;f`0300000039c=`0000000<0bLcL00D0
00000<W=g039cM`000000<S<g@02000000@0bLgM00<000000<S>gP38c]h00`38c]h00`000000b<oO
000000030<S?g`0400000038cml000000<O?h08000000`37d>400`000000aloQ000000040<O@h@03
00000036d>80a]3R0080a]3R0P0000000`36dN8000000<K@hP020<K@hP80000000<0aM7S00000035
d><00P0000030<GAi08000001034dN@00`000000`m3T0<?@i0020<?@i080000000<0`m7U00000032
d>D00P32d>D00`000000`]3U000000020<7?i@8000000P31d>D010000000`LoU0000000000020<3?
iP0400000030c^D00000000000<0_lkV0`0000030;k>iP040000002ncND00000000000<0_/gV0P00
00040;g<iP800000102lbnD010000000_<[U0000000000020;_:i@040000002kbND00000000000<0
^lWU00@000000;[8i@00000000000P2jb>D00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo
02D0oooo000R0?ooo`030000003oool0oooo04<0oooo0`0000020?ooo`030000002h`n<0^<?S0080
^<?S0P0000050;S4h`<00000102iaN<3000000D0^/GS0P0000050;_6h`800000102lan<2000000D0
_<OS0P0000050;g8h`8000001@2nbN<2000000D0_lWR00<000000<3:hP30b^801030b^800`000000
`L[R0<7:hP030<7:hP0400000031b^40000000000080`L_R0P0000001032bn40000000000032bn42
000000<0`l_Q0P0000050<C;h00300000034bml0a<_O00<0a<_O00@000000<C;g`00000000000P35
bml2000000030<G:gP000000a/_N008000001036b]d00`000000a/[M0<K:g@030<K:g@0400000037
bM`000000<O9g08000000`37bM/010000000b<WK0000000000040<S8fP0300000038b=X0b<SJ0080
b<SJ00D000000<S7f@00000000000<S7f@0200000080b<KH00@000000<W6f000000000001038aML0
0`000000b<GF0<S5eP020<S5eP80000000D0b<CF00000039a=H0bLCF000000020<S3e@0400000039
`m@00000000000@0b<;D00<000000<W1d`39`M<00`39`M<00`000000bL7B0<W1dP030<W1dP030000
0038_m8000000080bKoA00H000000<Rnd@38_]4000000<Vod00000050<Vnd00300000039_Lh0bKg>
00<0bKg>00<000000<Vlc@39_<d00P39_<d2000000050<Vjc0000000b[_<0<Zkc00000000P39^L/0
0`000000bk[:000000040<Zhb@030000003;]lH0bkO600<0bkO60P0000030<bfa0040000003;//40
bk;100000080c[C10`00000205A8Y`030000002IJIX0VFVJ00<0VFVJ00@000000:^2Z02[PZP00000
0`3?Z;000`000000aJ:e000000050<n_^P8000001@3>/l000`000000cKO50<fga@040<fga@800000
1@3<^LP2000000@0bk_;00@000000<bmc03<_L`000000P3;_Ld200000080bko>00<000000<Znc`00
0000103;`=000`000000b/3A000000050<[1dP030000003:`m<0b/?C00@0b/?C00<000000<[4e03:
a=@00`3:a=@00`000000b/GE000000020<W5eP8000000P3:a]H010000000bLKG0000000000030<[7
e`0300000039amP0000000D0b/SH00<000000<W9f@39bMT00`39bMT00`000000b/[J000000030<W:
fP80000000<0b/_K00000039bm/00P0000030<W<g00300000039c=`0000000@0bLgL00<000000<W=
g@39cMd00P39cMd2000000040<W>gP000000b<kN0<S>gP8000000P38cml010000000b<oO00000000
00030<S?g`0300000037d>40am3Q00@0am3Q00<000000<O@h@37d>400P37d>4010000000am7R0000
0036d>82000000<0a]7R00@000000<K@hP00000000001035dN<010000000aM7T0<GAi035dN@20000
00030<CAi0000000a=7T0080a=7T00@000000<?Ai@000000`m3T0P0000030<?Ai@0400000032d>D0
0000000000<0`]3U0P0000050<7@i@0300000030cnH0`<oV00<0`<oV00<000000;o>iP0000000`2o
c^H00`000000_/kV0;k>iP030;k>iP800000102ncNH00`000000_LgV000000030;g<iP030000002l
c>H0000000@0_<_U0P0000050;_:i@030000002kbND0^lWU00<0^lWU0P0000040;[8i@8000007`3o
ool00`000000oooo0?ooo`0U0?ooo`008P3oool00`000000oooo0?ooo`100?ooo`<000001@3oool0
0`000000^<?S0;S3h`0200000080^<?R00<000000;S4h`2ha><00P0000020;W4hP030000002iaN<0
^LGS008000000P2jaN<010000000^/GS0;[5h`2jaN<2000000030;_5h`000000^lKS0080^lKS0P00
00000`2ka^8000000;c7h`020;c7h`030000002lan80000000@0_<OS00<000000;g7hP000000102m
b><00`000000_/SR000000040;k9h`040000002nbN80000000000080_lWR0P0000000`2obN800000
000000020<3:hP80000000@0`<[R000000000000`L[R0P0000040<7:h@8000001@32bn400`000000
`l_Q0<?;h@020<?;h@0300000033b^00000000@0a<_P00D000000<C:g`00000000000<C;g`020000
00@0a<_O0P0000040<G:gP0300000036b]d0a/[M00<0a/[M00D000000<K9g@00000000000<K:g@02
000000<0alWL00<000000<O9f`37bM/01037bM/00`000000b<SJ0<S8fP030<S8fP0400000037amX0
00000<S8fP8000001038amT00`000000b<KH0<S6f0030<S6f00300000038aML0b<GG00<0b<GG00@0
00000<S5e`000000b<GF0P0000030<S4eP0300000039a=H0000000@0b<?E00<000000<S2e038`]@0
0`38`]@00`000000b<7C000000030<W1d`0500000038`M<0000000000039`M800P0000020<RodP03
00000039_m40000000@0b;kA00<000000<Vnd039_]000`39_]000`000000b;c?000000030<VmcP04
00000039^lh000000<Vlc@8000000`39^/`00`000000b[_<000000040<Vib`030000003:^<T0b[S9
0080b[S900<000000<Vfa`0000000`3;]lH010000000b[C50<Zda@0000020<bfa0070000003;//40
bk;10000003>]<4000000<f`_@0300000080E4RW00<0000009UYVP2IJIX00P2IJIX00`000000Zh:X
000000050<nX/08000001@3?[kX00`000000c[?00<jc`0030<jc`0030000003@^L<0000000@0cKO5
00@000000<jja`3>^/L000000P3<^LP200000080cKc:00@000000<^kb`3;^l/000000`3<_L`00`00
0000bkg=000000050<^ocP030000003;`=00bl3@00@0bl3@00<000000<[1dP3:`M800`3:`M800`00
0000bl?B000000040<[3d`030000003;a=@0000000<0b/CD00H000000<[5e@3:aMD000000<W5eP00
00050<[6eP030000003:amL0b/OG00@0b/OG00<000000<[8f03:b=P00`3:b=P00`000000b/WI0000
00020<W9f@8000000P3:b]X010000000bL[J0000000000040<[;f`0300000039c=`0bLcL00@0bLcL
00<000000<W=g039cM`00P39cM`010000000bLkM00000039cMd2000000<0bLkN00<000000<S>gP00
00001@38cml00`000000b<oO0<S?g`020<S?g`0300000038d>00000000<0am3Q0P0000000`37dN40
00000<O@h@02000000<0am7R00<000000<KAhP36dN801036dN800`000000aM7S0<GAh`020<GAh`<0
000000<0aM7T0000000000000`34dN@010000000a=7T0000000000030<?Ai@0300000033dND0`m7U
00@0`m7U00<000000<;@i@32d>D00P32d>D00`000000`M3V000000030<7@i@80000000@0`M3V0000
0030cnH0`<oV0P000000102ocnH000000000002oc^H2000000040;o>iP000000_/kV0;k>iP800000
0P2nc^H010000000_/gV0000000000020;g=iP040000002mc>H0000000000080_<cV00<000000;c;
i@2lbnD00P0000020;c;iP030000002kb^D0^l[U0080000000@0^l[V0000002kbND0^lWU0P000000
102jbND000000;[8i@2jb>D2000000030;W8i@000000oooo01h0oooo00<000000?ooo`3oool09@3o
ool00280oooo00<000000?ooo`3oool0?@3oool3000000T0oooo0P0000050;S3hP8000001@2ia>82
000000@0^/GS00@000000;[5h`00000000000`2kaN<010000000^lKS0000000000030;_6hP040000
002lan<00000000000<0_<OR00@000000;c7h`00000000000`2man8010000000_LSS000000000003
0;k8hP040000002nbN<00000000000@0_/WR0P0000050;o9hP8000001@30b^800`000000`L[Q0<7:
h@030<7:h@0300000031b^40000000<0`/_Q0P0000000`32b^0000000<?;h@02000000<0`l[P00@0
00000<C;h000000000001034b]l00`000000a<_O0<C;g`030<C;g`0300000035b]h0000000<0aL[N
00D000000<G:g@00000000000<K:g@02000000@0a/WM00<000000<O9g037bM`00P37bM`00`000000
a/SK000000040<O9f`0500000037b=/0000000000038b=X00P0000030<O7fP0300000038amT0b<OI
00<0b<OI00<000000<O6f@0000000`38a]P01@000000b<GH000000000000b<GG008000000`38aML0
0`000000b<CF0<S4eP040<S4eP0300000038`mD0b<?E0080b<?E00<000000<S2e00000000`38`]@0
1@000000b<7C0<S1d`000000bL7C008000001038`M<00`000000b;oB0<RodP030<RodP0300000038
_]40b;kA0080b;kA00<000000<Rmd00000000`39_]001@000000b;c?0<Rlc`000000bKg>00800000
0`39^lh00`000000bK[<0<Vjc0040<Vjc00300000039^L/0bKW;0080bKW;00<000000<Vgb@000000
0P3:^<T010000000bKK70<Vfa`0000030<^gaP060000003:]<D0b[C50000003<]/@00000103;//40
10000000cK2m0<f`_@3=/;d200000080E4RW00<0000009UYVP2IJIX00`2IJIX2000000D0cjR`00<0
00000<n_^P3?[kX00`3?[kX00`000000dkRn000000030<jc`0040000003@^L<0d;W3000000<0cKO5
00L000000<jja`3>^/L0c[[70000003<^LP0000000@0cKc:00<000000<^kb`0000001@3<_L`00`00
0000bko>0<^ocP030<^ocP030000003<`Ll0000000@0bl3@00<000000<c2d@0000000`3:`M801P00
0000bl?B0<_3dP3;`m8000000<[3d`8000000`3;a=@00`000000b/CD000000040<[5e@030000003:
a]H0b/KF00<0b/KF00<000000<_7e`000000103:amL00`000000b/SH000000020<[8f08000000P3:
bMT00`000000bLWI000000050<[:fP030000003:bm/0b/_K00<0b/_K00<000000<[<f`0000000`39
c=`2000000030<[=g0000000bLgL008000000`39c]d00`000000bLkN0<W>gP040<W>gP0300000038
cml0b<oO0080b<oO0P0000000`39d=l000000<S?g`0200000080b=3P00@000000<O@h@0000000000
1037dN400`000000am7R0<OAhP020<OAhP0300000037d^80000000<0a]7R0P0000000`36d^<00000
0<GAh`02000000<0aM;T00<000000<CAi034dN@01034dN@00`000000`m7U0<?Ai@020<?Ai@030000
0033d^D0000000<0`m7U0P0000000`32dND000000<;@i@0200000080`M3V00@000000<7@i@000000
00001031d>H2000000@0_loV0P0000040;o>iP800000102nc^H2000000@0_LgV0P0000050;c<iP80
0000102lbnH3000000<0^l[V0`0000040;[9i@800000102ib>D00`000000oooo0?ooo`0L0?ooo`03
0000003oool0oooo02D0oooo000R0?ooo`030000003oool0oooo03X0oooo0`00000=0?ooo`800000
0P2h`n82000000030;W3hP000000^LCR00<0^LCR00<000000;[4hP0000001@2jaN<00`000000^lGS
0;_5h`040;_5h`030000002ka^80^lKR00@0^lKR00<000000;c7hP2lan80102lan800`000000_LOR
0;g7hP040;g7hP030000002nb>80_/SR00@0_/SR00<000000;k9hP2nbN800`2nbN800`000000_lSR
000000040;o9hP0400000030bN40000000000080`<[R0P0000001031bN40000000000031b^420000
0080`L[Q00@000000<;;h@00000000001032b^000`000000`l[P0<?:h0040<?:h00300000034b]l0
a<[O00<0a<[O00D000000<C:g`00000000000<C;g`0200000080aL[N00@000000<G:gP0000000000
1035b]d00`000000a/WM0<K9g@030<K9g@0400000036bM`000000<O9g08000000`36b=/010000000
alWK0000000000040<O8f`0300000037amX0alOJ0080alOJ00<000000<O7f@0000000`38amT01@00
0000alKI0<O6f@000000b<KH008000001038aMP00`000000b<GG0<S5e`020<S5e`0300000038a=H0
000000@0b<CF00@000000<S3eP000000b<?E0P0000030<S2e00300000038`]@0000000@0b<7C00<0
00000<S1d`38`M<00`38`M<00`000000b;oB000000030<RodP0400000038_]4000000<Rnd@800000
0`38_M000`000000bKk@000000040<Rlc`0300000039^lh0bK_>0080bK_>00<000000<Rjc@000000
1039^/`00`000000b;W;000000020<Vib`0600000039]lT0bKO90000003:^<T000001039]/L00`00
0000bkO6000000040<Zda@030000003;//40bk;10080bk;10P0000030<f`_@8000000P1DB:L00`00
0000VFVJ09UYVP0309UYVP030000003?Z;00cjR`00<0cjR`00<000000=Zg]`000000103?[kX00`00
0000dkRn000000030<jc`0070000003@^L<0d;W30=2i``000000cKO5000000050<jja`030000003=
_<X0cKc:00@0cKc:00<000000<bmc03<_L`00`3<_L`00`000000cL3>000000030<^ocP040000003<
`Ll0c<7?000000<0bl3@00L000000<c2d@3<`]40c<;A0000003:`M80000000D0bl?B00<000000<_4
e03;a=@0103;a=@00`000000b/GE0<[5e@020<[5e@030000003;a]H0000000<0b/KF00H000000<_7
e`3;amL0blOG0000003:amL200000080b/SH00<000000<[8f00000001@3:bMT00`000000b/[J0<[:
fP030<[:fP030000003:bmX000000080b/_K0P0000020<[<f`0400000039c=`00000000000@0b/gL
00<000000<W>g@39c]d00P39c]d00`000000bLkN000000040<W>gP0400000039cmh000000<S?g`80
00001039d=l00`000000b=3P0<S@h0030<S@h00300000037dN40am7Q0080am7Q0P0000000`37dN80
00000<OAhP0200000080am;R00@000000<KAhP00000000001036d^<00`000000aM;T0<GBi0020<GB
i00300000035d^@0000000<0a=7T0P0000000`34d^D000000<?Ai@0200000080`m;U00@000000<?A
i@00000000001032dND00`000000`M3V0<7@iP030<7@iP0300000031d>H0`M3V0080`M3V0`000003
0;o?iP80000000<0_loV0000000000000P2oc^H00`000000_/kV000000020;k>iP80000000@0_LkV
0000002mcNH0_LgV0P0000020;c=iP030000002lc>H0_<cV0080000000@0_<cV0000002lbnH0_<_V
0P000000102kbnH000000;_:iP2kb^H200000080^/WV00L000000;[9i@2jbND0^/WU0000002ib>H0
000000@0^LSU00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0U0?ooo`008@3oool00`00
0000oooo0?ooo`0h0?ooo`<000004P3oool2000000@0^L?R00@000000;W4hP00000000000`2ja>80
0`000000^/GS0;[5h`02000000030;[5hP00000000000080^lGS0P0000000`2kaN80000000000002
0;_6hP80000000<0_<KR0000000000000P2lan82000000030;g7hP00000000000080_LOR0P000000
0`2man800000000000020;k8hP80000000@0_/SR000000000000_/WR0P0000030;o8hP040000002o
bN800000000000@0`<WQ0P0000050<79h@0300000031b^40`L[Q00<0`L[Q00<000000<;:h032b^00
0`32b^0010000000`l[P0000000000020<?:h080000000@0`l[O000000000000a<[O0P0000040<C:
g`0300000035b]h0aL[N00<0aL[N00<000000<G:g@35b]d00`35b]d01@000000aLWM000000000000
a/WM008000000`36bM`00`000000a/SK0<K8f`040<K8f`0300000037b=/0alSK00<0alSK00@00000
0<O7fP000000alOJ0P0000030<O7f@0300000038amT0000000@0alKI00<000000<S5f038aMP00`38
aMP00`000000alGG000000020<S5e`0600000038a=H0b<CF0<S4eP000000b<CF0P0000030<S3eP03
00000038`]@0b<;D00@0b<;D00<000000<S1d`38`M<00P38`M<01@000000b<7C000000000000b<7C
008000000P38_m8010000000b;oB0000000000030<Rnd@0300000038_M00b;g@00@0b;g@00<00000
0<Rlc`38_<l00P38_<l00`000000b;_>000000020<VkcP0400000038^/d0b;[=000000<0bK[<00H0
00000<Rib`38^L/000000<Vib`0000040<Vgb@0300000039]/L0bKK700@0bKK700<000000<Zda@3:
]<D00P3:]<D2000000<0bk;100@000000<V^_P39[[h000000P3=/;d30000000305A8Y`000000VFVJ
00@0VFVJ00<000000<nX/03?Z;000`3?Z;000`000000f[Ng000000030<n_^P070000003C^;h0dkRn
0=>h_P000000c[?0000000050=2i``030000003>^/L0c[[700<0c[[700<000000=2nb@000000103=
_<X00`000000c[o<000000030<bmc0040000003=`<h0cL3>000000<0bko>00L000000<c1c`3<`Ll0
c<7?0000003;`=00000000D0c<;A00<000000<_3dP3;`m800`3;`m82000000D0blCD00<000000<_5
e00000000P3:aMD01`000000blKF0<_6eP3;a]H000000<[6eP0000001@3;amL00`000000b/SH0<[8
f0030<[8f0030000003:bMT0b/WI00<0b/WI00<000000<_:f@0000000P3:b]X200000080b/_J00<0
00000<[;f`0000001@3:c=/00`000000b/gL0<[=g0030<[=g0030000003:c]d000000080bLkM00H0
00000<W>gP39c]h0bLkN00000039c]h2000000<0bLoN00<000000<W@g`39d=l00`39d=l00`000000
bM7P000000030<S@h00400000038dN4000000<OAh@8000001037dN800`000000am;R0<OBhP030<OB
hP0300000036d^<0a];S0080a];S0P0000000`36d^@000000<GBi00200000080aM;T00@000000<CA
i000000000001034d^D00`000000`m;U0<?Bi@030<?Bi@0300000032dND0`]7U0080`]7U0P000000
1032dNH000000<7@iP31d>H2000000030<7AiP000000`M3V008000000`30d>H3000000<0_loV00@0
00000;o>iP00000000000`2nc^H2000000@0_LkV0P0000050;c=iP800000102lc>H2000000@0^l_V
0P0000040;[9iP040000002jbND0000000000080^LSV00<000000;W8i@2ib>D00P0000000`2ianH0
00000?ooo`0M0?ooo`030000003oool0oooo02@0oooo000Q0?ooo`040000003oool00000000003@0
oooo0`00000F0?ooo`030000002i`n80^L?R00<0^L?R00<000000;[4hP2ja>80102ja>82000000D0
^/GR0P0000050;_5hP8000001@2la^82000000D0_LOR0P0000050;g7hP8000001@2nb>800`000000
_lSR0;o8hP040;o8hP0300000030bN40`<WQ00<0`<WQ00<000000<39h@0000001031bN400`000000
`LWQ000000020<7:h@80000000@0`/WP000000000000`/[P0P0000040<?:h08000001@33b]l00`00
0000a<[O0<C:g`020<C:g`80000000@0a<WN00000035b]h0aL[N0P0000001035bMd0000000000035
b]d2000000@0aLWM00<000000<K9g036bM`00P36bM`00`000000a/SL000000040<K8f`0500000036
b=/0000000000037b=/00P0000030<O7fP0300000037amT0alOI00@0alOI00<000000<O6f@37a]T0
0P37a]T01@000000alGH000000000000b<GH008000000P37aML00`000000b<GG000000050<S4eP03
00000038`mH0b<?F0080b<?F00<000000<S2e@0000001038`]@010000000b<7D00000038`M<20000
00@0b<7C00<000000<RodP38_m800`38_m800`000000b;kA0<Rnd@020<Rnd@0300000037_M400000
00@0b;g@00<000000<Rlc`0000000P38_<l01P000000b;_>0<RkcP000000bK_>000000@0b;[=00<0
00000<Vjc00000001038^L/00`000000bKO90<Vgb@020<Vgb@0300000038]LP0000000@0bKK700<0
00000<RcaP0000000P3:]<D010000000bK730<Va``0000020<^b`@0600000039[[h0bJjn0000003=
/;d000000P3:Z[T00`000000E4RW0000000409UYVP030000003`aJ@0000000<0cjR`00L000000=Zg
]`3J]kL0f[Ng0000003?[kX0000000D0dkRn00<000000=2i``3@^L<00`3@^L<2000000@0c[[700@0
00000=2nb@3@_/T000000`3=_<X01`000000c[o<0<joc03>_l`000000<bmc0000000103=`<h00`00
0000bko>000000050<c1c`030000003<`]40c<;A00<0c<;A00<000000<c3dP0000000P3;`m820000
0080c<CC00<000000<_4e03;a=@00P0000020<_5e0030000003:aMD0000000D0blKF00<000000<_7
e`3;amL00`3;amL00`000000blSG000000030<[8f0050000003;bMP000000000003:bMT00P000002
0<_:f@030000003:b]X0000000D0b/_J00<000000<[<f`3:c=/00`3:c=/00`000000b/gL00000002
0<[=g08000000P3:c]d00`000000bLkM000000050<W>gP0300000039cmh0bLoN0080bLoN00<00000
0<W@g`0000000P39d=l200000080bM7P00@000000<S@h000000000000`38dN400`000000am7R0<OA
hP020<OAhP80000000<0am;R00000037d^800P37d^8010000000a];S00000036d^<2000000@0a];T
00<000000<GBi035d^@00`35d^@00`000000a=;U0<CBi@020<CBi@80000000@0a=;U00000033d^D0
`m;U0P0000000`33d^H000000<;Ai@02000000@0`]7V0P0000040<7AiP0300000030d>H0`=3V00<0
`=3V00<000000;o?iP2ocnH00`2ocnH3000000<0_/kV00<000000;k>i`0000000`2mc^H00`000000
_LkW000000030;c=iP80000000<0_<gW0000002lc>H00P2lc>H00`000000^lcW000000030;_;iP03
0000002jbnH0000000@0^/WV0P0000040;W8iP<00000102ianH00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo02@0oooo000Q0?ooo`800000=03oool2000001X0oooo00<000000;W3hP2i`n80
0P0000000`2j`n800000000000020;[4hP80000000<0^/CR0000000000000P2jaN82000000030;_5
hP00000000000080^lGR0P0000000`2laN8000000;c6hP030;c6hP030000002ma^80000000@0_LOR
00@000000;g7hP00000000000P2man82000000030;k7h@00000000000080_/SR0P0000000`2ob>40
0000000000020;o8hP80000000@0_lSQ000000000000`<WQ0P0000030<39h@0400000031bN400000
000000<0`LWQ0P0000050<;9h00300000033b^00`l[P00<0`l[P00@000000<?9g`00000000000P33
b]l2000000030<C9g`000000a<[O008000001034bMh2000000D0aLWM00<000000<G9g@35bMd00`35
bMd010000000aLSL00000036bM`2000000<0a/SL00@000000<K8f`00000000001036b=/00`000000
alOJ0<O7fP020<O7fP0300000037amX0000000@0alOI00@000000<O6f@000000alKI0P0000040<O5
f00300000037aML0alGG00<0alGG00<000000<S4eP38a=H00`38a=H010000000al?F00000038`mH2
000000<0b<;E00@000000<S2e000000000000`38`M@00`000000b<7C0<S1d`030<S1d`0300000037
_m<0000000<0b;oB00<000000<NndP0000000P38_]401P000000akgA0<Nmd@37_M4000000<Rmd080
00000P38_<l00`000000b;c?000000040<RkcP0300000038^/d0b;[=00@0b;[=00<000000<Rib`38
^L/00P38^L/00`000000b;O:000000020<Vgb@0400000038]LP0b;G8000000<0bKK700H000000<Rc
aP38/lH000000<Zda@0000030<Va``030000003;//40000000@0bJjn00L000000<ZZ^@3:Z[T0bZZi
0000001DB:L0000000@0VFVJ00<000000?35Y00000000P3?Z;000`000000f[Ng0=Zg]`020=Zg]`03
0000003?[kX0000000@0dkRn0P0000040=2i``040000003B_LH0d[g600000080c[[700<000000=2n
b@3@_/T00P3@_/T00`000000cKc:000000050<joc0030000003=`<h0cL3>00@0cL3>00<000000<c1
c`3<`Ll00`3<`Ll00`000000cL;@000000020<c2d@8000000P3<`m800`000000bl?B000000040<c4
d`030000003;a=@0000000D0blGD00<000000<_6eP3;a]H00`3;a]H00`000000blOF000000020<_7
e`8000000P3;b=L010000000b/SH0000000000040<_9f0030000003;b]T0bl[I00<0bl[I00<00000
0<[;fP3:bmX00`3:bmX00`000000blcK000000020<[<f`8000000P3:cM`00`000000b/gL00000005
0<[>g@0300000039c]h0bLkN0080bLkN0P0000000`3:cmh000000<W?gP0200000080bM3O00<00000
0<W@g`0000001@39dN000`000000b=7Q0<SAh@020<SAh@0400000038d^8000000<OAhP8000000`37
d^8010000000am;R0000000000030<KBh`0300000036d^@0a];T0080a];T0P0000001035dn@00000
0<GBi035d^@2000000030<GCi@000000a=;U008000001034d^D2000000@0`m;V00<000000<;AiP32
dNH00P32dNH2000000030<7Bi`000000`M7V0080`M7V00<000000<3Ai`0000000P30d>H200000004
0;o@i`000000_loV0;o?iP8000000P2ncnL400000080_/kW00@000000;g>iP00000000000P2mc^L0
10000000_<gV0000000000030;c=i`040000002lc>H0000000000080^lcW00@000000;_;iP000000
00000P2jbnH00`000000^/WV0;[9iP02000000040;W9iP000000^LSV0;W8iP8000000P2ib>H00`00
0000^LOV0;W7iP02000000030;S7iP000000oooo01/0oooo00<000000?ooo`3oool0903oool00240
oooo00<000000?ooo`3oool0<03oool3000001d0oooo0P0000050;[3hP8000001@2ja>82000000D0
^lGR0P0000040;c5hP040000002la^800000000000<0_LKR00@000000;g7hP0000000000102man82
000000D0_/OQ0P0000050;o8h@8000001@2ob>400`000000`<WQ0<39h@030<39h@8000001031bN40
10000000`LWP0000000000020<;9h080000000@0`/WP000000000000`l[P0P0000040<?9g`800000
1034bMl00`000000a<WN0<C9gP030<C9gP0300000034bMh0000000@0aLWM00D000000<G9g@000000
00000<G9g@02000000<0aLSL00<000000<K8g036b=`01036b=`00`000000a/SK0<K8f`020<K8f`80
000000<0a/OJ00000037amX00P0000030<O7fP0400000037amT00000000000<0alKI00<000000<O5
f037aMP00`37aMP00`000000alGH000000030<O5e`0500000037a=L0000000000038a=H00P000003
0<O3eP0300000038`]D0b<;E00@0b<;E00<000000<S1e038`M@00P38`M@00`000000al3D00000003
0<S1d`0500000037_m<0akoC00000038_m800P0000020<NndP0300000038_]40000000D0akgA00<0
00000<Rlc`38_<l00`38_<l00`000000b;_>0<RkcP020<RkcP0300000037^/d0000000@0b;[=00<0
00000<Nic00000000P38^L/01P000000b;O:0<RgbP000000bKO9000000@0b;G800<000000<Vfa`00
00001038/lH00`000000bK730<Va``030<Va``0300000039[[h0bJjn0080bJjn0P0000030<ZZ^@03
0000001DB:L0000000<0VFVJ00H000000?35Y03`aJ@000000<nX/00000060=Zg]`030000003C^;h0
dkRn0080dkRn00@000000=Jm`P3F_L8000000P3@^L<01`000000d[g60=:maP3B_LH000000<jja`00
00001P3@_/T00`000000c[o<0<joc0030<joc08000001@3=`<h00`000000c/;?000000020<c1c`80
00000P3=`]000`000000c<;A000000050<c3dP030000003<a=<0c<CC00@0c<CC00<000000<_5e03;
aM@00`3;aM@00`000000c<KE000000020<_6eP8000000P3;amH00`000000blOG000000050<_8e`03
0000003;bMP0blWH00<0blWH00<000000<_:f@0000000`3;b]T01@000000bl_J000000000000b/_J
008000000P3;c=/00`000000b/cK000000050<[=g0030000003:c]d0b/kM0080b/kM0P0000000`3:
cmd000000<W>gP02000000@0b/oN00<000000<W@g`39d=l00`39d=l00`000000bM7P0<WAh0020<WA
h080000000<0bM;Q00000038dN400P0000030<SBhP0300000037d^80am;R00@0am;R00<000000<KB
h`36d^<00P36d^<010000000a]?T00000036d^@2000000@0aM?T0P0000040<GCi@0300000034d^D0
a=;U0080a=;U0P0000001033dnH000000<?BiP33d^H2000000030<;BiP000000`]7V008000000`31
d^L010000000`M7V0000000000030<3Ai`800000102od>L2000000@0_/oW0`0000030;k>i`800000
1@2mc^L00`000000_<gW0;c=i`030;c=i`800000102kc>L2000000D0^/_V0P0000040;W9iP800000
1@2ib>H2000000<0^<OV00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0T0?ooo`008@3o
ool00`000000oooo0?ooo`0]0?ooo`<000008@3oool200000080^/?R0P0000000`2j`n4000000;[4
hP030;[4hP030000002ka>40000000@0^lGR00<000000;c5h@0000001@2laN800`000000_LKR0;g6
hP040;g6hP030000002man80_LOR00<0_LOR00<000000;k7h@000000102nan400`000000_/OQ0000
00040;o8h@040000002ob>40000000000080_lSQ0P0000001030b>4000000<39h@30bN4200000080
`LWP00@000000<79h@00000000001031bN02000000D0`/WP00<000000<?9g`33bMl00`33bMl00`00
0000`lWO000000030<C9g`0500000034bMh0000000000034bMh00P0000030<C9gP0400000035bMd0
0000000000@0aLWM00<000000<G8g035b=`00P35b=`00`000000aLSL000000040<K8g00400000036
b=/000000<K8f`8000001036amX00`000000alOJ0<O7fP040<O7fP0300000037a]T0alKI0080alKI
00D000000<O5f000000000000<O5f00200000080alGH00@000000<O5e`00000000001037a=L00`00
0000al?F0<O3eP020<O3eP0300000037`]H0000000@0b<;E00@000000<O1e0000000b<7D0P000003
0<O0e00300000038`M<0000000@0akoC00<000000<NndP37_]800`37_]800`000000akgA0<Nmd@03
0<Nmd@0300000037_=00000000<0b;c?00<000000<Nkc`0000000P38^lh01P000000ak[=0<Njc@37
^/d000000<Rjc@8000000P37^L`00`000000b;W;000000040<RgbP0300000038]LP0b;G800@0b;G8
00<000000<RcaP38/lH00P38/lH2000000@0bK730P0000030<V^_P0400000037ZK`0ajVl00000080
bZZi0`00000309UYVP040000003`aJ@0l<FT0?35Y08000001@3J]kL2000000<0dkRn00D000000=Jm
`P3F_L80e[g2000000020=2i``030000003B_LH0d[g60080d[g600<000000=2nb@3@_/T0103@_/T2
000000@0c[o<00@000000<o1c@3?`Ld000000P3=`<h200000080c/;?00<000000<c1c`0000001@3=
`]000`000000c<?B0<c3dP030<c3dP030000003<aM<0000000<0c<CC0P000000103<aM@000000<_5
e03;aM@200000080c<KE00<000000<_6eP0000001@3;amH00`000000blSG0<_8e`030<_8e`030000
003;bMP000000080blWH0P0000020<_:f@040000003;b]T00000000000@0bl_J00<000000<_<f`3;
c=/00`3;c=/00`000000b/gL0<[=g0030<[=g0040000003;c]`000000<[>g@800000103:cmd00`00
0000b/oN0<[?gP030<[?gP030000003:d=l0000000<0bM3O00@000000<WAh0000000bM7P0P000004
0<WBh@0300000038d^80b=;R0080b=;R00<000000<SChP0000000`37d^82000000030<OCh`000000
a];S008000000`36dn@00`000000aM?T0<GCi0020<GCi080000000<0aMCU00000035dnD00P35dnD0
10000000a=?U00000034d^D2000000@0`m?V0P0000040<;BiP0300000031d^L0`M;W00<0`M;W0P00
00030<3Ai`80000000<0`=7W0000002od>L00P2od>L00`000000_m3W000000020;k?i`80000000@0
_/oW000000000000_/kW0P0000020;g>i`030000002mc^L0_LkW0080000000@0_<gW0000002lcNL0
_<gW0P000000102kc>L000000;_<i`2kc>L200000080^/_W00<000000;[;iP2jbnH00P000000102j
b^L000000;W9iP2ibNH2000000030;W9i`000000^LSV0080^LSV0`0000040;S7iP8000006`3oool0
0`000000oooo0?ooo`0T0?ooo`008@3oool00`000000oooo0?ooo`0Z0?ooo`<000009P3oool20000
00@0^/?Q00@000000;[4hP00000000000`2ka>4010000000^lGR0000000000030;c5h@030000002l
aN80_<GR0080000000<0_<GQ0000000000000P2ma^82000000040;g6h@00000000000;g7hP800000
0`2nan4010000000_/OQ0000000000030;k7h@040000002ob>400000000000@0_lSQ0P0000040<38
h@8000001@31bN000`000000`LWP0<79h0030<79h00300000032b=l0000000@0`/WP00D000000<;9
g`00000000000<?9g`0200000080`lWO00@000000<C9g`00000000001034bMh00`000000a<WN0<C9
gP040<C9gP0300000035bMd0aLWM0080aLWM0P0000000`35b=`000000<G8g002000000<0aLSL00@0
00000<K8g000000000000`36b=/00`000000a/OJ0<K7fP030<K7fP0300000036a]X0000000@0alOJ
00@000000<O6f@000000alKI0P0000040<O5f00300000037aMP0alGH00<0alGH00<000000<O4e`37
a=L00`37a=L010000000al?F00000037`mH2000000<0al;F00@000000<S2e@00000000000`37`M@0
0`000000al3D0<O0e0040<O0e00300000037_m<0akoC0080akoC00<000000<NndP0000000`37_]80
1@000000akgA000000000000akgA008000000P37_=0010000000b;c?0000000000020<Nkc`030000
0038^lh0000000D0ak[=00<000000<Nic037^L`00`37^L`00`000000b;O:0<RgbP020<RgbP800000
1@38]LP00`000000ak?7000000020<RcaP0400000037/L@0ak7400000080bK7300@000000<N^`@37
[/4000000P39[[h01`000000ajVl0<NY_0000000bZZi00000036XkH00P00000309UYVP050000003`
aJ@0l<FT0?35Y00000001P3J]kL00`000000g;nl000000020=>h_P030000003F_L80e[g20080e[g2
0P0000040=:maP030000003B`<P0000000@0d;k900@000000=30b`3@`</000000P3>_l`01`000000
cl7=0<o1c@3?`Ld000000<g0cP0000001@3>`/l00`000000cL;@0<g2d0030<g2d0030000003=a=40
00000080c<?B0P0000020<c5d`040000003<a=<00000000000<0c<GD00<000000<_5e00000001@3<
a]D00`000000blOF0<_7eP030<_7eP030000003<b=L000000080blSG0P0000020<_9f0030000003;
bMP0000000D0bl[I00<000000<_;fP3;bmX00`3;bmX00`000000blcJ000000030<_<f`050000003;
cM/000000000003:cM`00P0000030<_>g0030000003:cmd0b/oM00<0b/oM00<000000<_@gP000000
0P3:cmh200000080b]3O00@000000<W@g`00000000000`39dN000`000000bM;Q0<WBh@020<WBh@80
000000<0bM;Q00000038d^800P0000020<SChP0400000037d^800000000000@0am?S00<000000<KC
i036dn@00P36dn@010000000a]CT00000035dn@2000000<0aMCU00@000000<GCi@00000000000`34
dnD00`000000`m?V0<?CiP020<?CiP80000000<0`m?V00000032d^H00P32d^H00`000000`];W0000
00020<7Bi`80000000<0`M;W00000030dNL00P0000030<3Ai`040000002od>L00000000000<0_m3W
0P0000040;k?i`800000102mc^L3000000@0_<gW0P0000040;_<i`800000102jbnL3000000@0^/[W
0P0000030;W9i`040000002ib>H00000000000<0^<SV00@000000;S7iP00000000000P2ganH00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo02@0oooo000P0?ooo`030000003oool0oooo02P0
oooo0`00000Z0?ooo`030000002j`n40^/?Q00<0^/?Q00<000000;_4h@2ka>40102ka>400`000000
_<GQ0;c5h@040;c5h@8000001@2laN42000000D0_LKQ00<000000;k7h@2nan40102nan400`000000
_/OQ0;k7h@040;k7h@030000002ob>40_lSQ0080_lSQ0P0000000`30b>0000000<38h@020<38h@04
00000030b>00000000000080`LWP0P0000001031b>00000000000031bN02000000<0`/SO00@00000
0<;9h000000000001032bMl00`000000`lWO0<?9g`030<?9g`0300000034bMh0a<WN00<0a<WN00@0
00000<C8gP00000000000P34bMh2000000030<C8g@000000aLWM008000001035b=`00`000000aLSL
0<G8g0040<G8g00300000036b=/0a/SK0080a/SK00D000000<K7f`00000000000<K7fP02000000<0
a/KJ00@000000<O7fP00000000000`37a]T00`000000alGH0<O5f0030<O5f00300000037aMP00000
00<0alGH00D000000<O4e`00000000000<O4e`02000000<0al?F00<000000<O2eP37`]H01037`]H0
0`000000al7D0<O1e0020<O1e00300000037`M@0000000@0al3D00@000000<Nod`000000akoC0P00
00020<NndP0400000037_]800000000000@0akgA00<000000<Nld037_=000`37_=000`000000ak_?
0<Nkc`030<Nkc`0300000037^/d0ak[=00<0ak[=00<000000<Nic@0000000`37^L`00`000000akO;
000000020<RgbP0400000037]LT0akG900000080b;G80P0000020<Nca`0300000038/lH0000000<0
ak7400<000000<Va``0000000`37[/400`000000bJjn000000040<NY_080000000<0aZ>f00000000
00000P2IJIX00`000000l<FT0?35Y0020?35Y0800000103J]kL01P000000g;nl0=bo_0000000dkRn
000000D0e[g200<000000=:maP3B_LH00P3B_LH010000000d/380=;0b00000030=2nb@070000003@
`</0d<3;0=30b`000000c[o<000000050<o1c@030000003>`/l0c/;?00<0c/;?00<000000<k3d000
00000P3=`]0200000080cLCA00<000000<c3dP0000001@3<aM<00`000000c<GD0<c5e0040<c5e003
0000003<a]D0c<KE00<0c<KE00<000000<c7eP0000000P3;amH200000080c<SG00<000000<_8e`00
00001@3;bMP00`000000bl[I0<_:f@030<_:f@030000003<bmT000000080bl_J0P0000020<_<fP04
0000003;c=/00000000000@0blgK00<000000<_>g03;c]`00P3;c]`00`000000bloM000000020<[?
g@8000000P3;d=h00`000000b/oN000000050<[@g`0300000039dN00bM7P0080bM7P00@000000<WB
h0000000bM;Q0P0000040<WBh@0300000038dn80b=?R00<0b=?R00<000000<OCh`37dn<00P37dn<2
000000030<ODi0000000a]?T008000000`36e>@00`000000aMCU0<GDi@030<GDi@8000001034dnD0
10000000a=CV00000033dnH2000000<0`m?V00@000000<;BiP00000000000`32d^L2000000@0`M;W
00<000000<3Ai`30dNL00`30dNL2000000@0_m3W00<000000;k@j00000000`2ncnL01@000000_LoX
000000000000_LkW008000000P2lc^P00`000000_<gW0;c=i`02000000040;_=j0000000^lcW0;_<
i`80000000@0^lcX0000002jbnL0^/_W0P0000000`2jbnL000000;[:i`020;[:i`80000000<0^L[W
0000002ibNL00`2ibNL00`000000^<SV0;S8iP030;S8iP800000102ganH2000001T0oooo00<00000
0?ooo`3oool0903oool00200oooo00<000000?ooo`3oool09@3oool3000002h0oooo00<000000;[3
h@2j`n400P0000000`2k`n400000000000020;_4h@80000000<0^lCQ0000000000000P2laN420000
00030;c5h@00000000000080_<GQ0P0000000`2maN400000000000020;g6h@80000000<0_/KQ0000
000000000P2nan42000000030;k7h@00000000000080_/OQ0P0000000`2oan4000000;o8h@020000
00<0`<SP00@000000<38h@00000000001030b>02000000D0`LSP00<000000<;8g`32b=l01032b=l0
0`000000`/WO0<;9g`020<;9g`80000000@0`lSO00000033bMl0`lWO0P0000001033b=h000000000
0034bMh2000000@0a<SN0P0000040<C8g@0300000035b=`0aLSL00<0aLSL00@000000<G8g0000000
00000P35b=`2000000030<G7f`000000a/SK008000001036am/00`000000a/KJ0<K6fP040<K6fP03
00000037a]T0alKI0080alKI00D000000<K5f@00000000000<O5f00200000080alGH00@000000<O5
f000000000001037a=L00`000000al?F0<O3eP020<O3eP0300000037`]H0000000@0al;F00@00000
0<O1e@000000al7D0P0000030<O1e00400000037`=@00000000000<0akoC00<000000<NndP37_]80
0`37_]800`000000akgA0<Nmd@030<Nmd@0300000036_=40000000<0akc@00<000000<Jkc`000000
0`37^ll00`000000ak[>000000030<Njc@0500000037^Ld0akW=00000037^L`00P0000020<Ngb`03
00000038]lX0000000<0akG900<000000<Reb00000001@37/lL00`000000ak740<Naa0030<Naa003
00000037[/40ajk100<0ajk100<000000<NY_037ZK`00`37ZK`010000000aZ>f0<JS]P36XkH20000
000309UYVP000000l<FT0080l<FT00<000000>_9/P0000000`3J]kL00`000000g;nl0=bo_0020=bo
_0030000003F_L80e[g200<0e[g200<000000=K1a@0000000`3B_LH01`000000d/380=;0b03B`<P0
00000=2nb@0000001@3@`</00`000000cl7=0<o1c@030<o1c@030000003?`lh000000080c/;?0P00
00020<k3d0030000003=`]00000000D0cLCA00<000000<c5d`3<aM<00`3<aM<00`000000cLKC0000
00040<c5e0030000003<a]@0000000<0c<KE00H000000<c7eP3<amH000000<_7eP0000050<c8e`03
0000003;bMP0blWH00<0blWH00<000000<c:f00000000P3;b]T200000080c<_I00<000000<_;fP00
00001@3;c=X00`000000blgK0<_=f`030<_=f`030000003;c]`000000080blkL00H000000<_?g@3;
cmd000000<[?g@0000050<_@gP030000003:d=l0b]3O0080b]3O0P0000000`3:dMl000000<WAh002
000000<0bM;P00<000000<WBh@39d^400`39d^400`000000bM?R000000030<SChP0400000038e><0
00000<OCh`8000001037e>@00`000000a]CT0<KDi0020<KDi00300000036e>D000000080aMCU0P00
00020<GDiP0400000034dnD00000000000<0a=CV00<000000<?CiP33dnH00`33dnH2000000@0`];W
00<000000<7Ci`0000000P31d^L2000000040<3Bj0000000`=7W0<3Ai`8000000P2odNP010000000
_m3W0000000000020;k@j0040000002ncnL00000000000<0_LoX0P0000050;c>j0800000102kcNP2
000000@0^lcX0P0000040;[;i`<000000`2ib^L00`000000^LWW0;W9i`02000000040;S9i`000000
^<SV0;S8iP80000000@0]lSW0000002ganH0]lOV0P0000000`2ganH000000?ooo`0I0?ooo`030000
003oool0oooo02<0oooo000P0?ooo`030000003oool0oooo0280oooo0`00000b0?ooo`8000001@2k
`n42000000D0^lCQ0P0000050;c5h@8000001@2maN42000000D0_/KQ0P0000050;k7h@800000102o
an400`000000`<SP0<38h0040<38h00300000030b>00`<SP00<0`<SP00<000000<78h00000001031
b>0010000000`LSO0000000000020<;8g`80000000<0`/SO00000032bMl00P0000040<?8g`800000
1@33b=h00`000000a<SN0<C8gP030<C8gP0300000034b=d0000000<0a<SM00D000000<G8g@000000
00000<G8g002000000@0aLSL0P0000040<G7f`0300000036am/0a/OK00<0a/OK00<000000<K6fP00
00000`36a]X2000000030<K6fP000000alKI008000001036aMT00`000000alGH0<O5f0030<O5f003
00000037a=L0alCG00<0alCG00<000000<K3e`0000000P37`mH01P000000al;F0<O2eP37`]H00000
0<O2eP8000000`37`MD00`000000al7D0<O1e0040<O1e00300000037_m<0akoC0080akoC00<00000
0<Jnd`0000000`37_]800`000000a[gB000000030<Nmd@0500000036_=40a[cA00000037_=000P00
00020<Jkc`0400000037^ll00000000000<0ak[>00<000000<Njc@0000001037^Ld00`000000akO;
0<Ngb`030<Ngb`0300000037]LT0akG900<0akG900<000000<Nca`37/lL00`37/lL2000000@0ak74
0P0000040<N^`@8000001037ZK`2000000<0aZ>f0P0000000`2IJIX000000?35Y0020?35Y0060000
003[bK80jlVb0000003J]kL00000103L_k`00`000000g<C1000000030=Jm`P070000003F`LD0e/75
0=K1a@000000d[g6000000050=;0b0030000003@`</0d<3;00<0d<3;00<000000=33c@0000000P3?
`Ld200000080cl?>00<000000<k2c`0000001@3>`m000`000000cLCA0<g4d@030<g4d@030000003=
aM80000000<0c<GC00@000000<g6d`3=a]<000000`3<aM@01`000000c<KD0<c6e03<a]@000000<c6
e@000000103<amH00`000000c<SG0<c8e`030<c8e`030000003<bML0000000<0blWH00H000000<c:
f03<b]P000000<_:f@0000050<c;f@030000003;c=X0blcJ00<0blcJ00<000000<c=f`0000000P3;
cM/200000080blkL00<000000<_>g0000000103;cmd00`000000bm3N0<_@gP030<_@gP040000003;
dMh000000<[@g`800000103:dMl00`000000bM;P0<WBh0020<WBh00300000039dn4000000080bM;Q
0P0000020<WChP0400000038dn800000000000<0b=CS00<000000<ODi037e>@00P37e>@200000003
0<ODi0000000a]CT008000000`36e>D2000000D0aMCV00<000000<CDiP34e>H00P34e>H00`000000
`mCW000000020<?CiP8000000P32e>L010000000`];W0000000000030<7Ci`8000001030d^P20000
00@0_m7X0P0000040;k@j0800000102mcnP00`000000_<oX000000030;c>j0<000000`2kcNP20000
00030;_=j0000000^lcX0080^lcX00<000000;[<j0000000102jbnL00`000000^L[W0;W:i`030;W:
i`<000000`2hbNL3000000@0]lSW0P0000040;O7iP030000003oool0oooo01L0oooo00<000000?oo
o`3oool08`3oool00200oooo00<000000?ooo`0000007`3oool3000003H0oooo0P0000020;_3h@80
000000<0^lCQ0000002ka>400`2ka>400`000000_<CQ000000040;c5h@030000002maN40000000@0
_LGQ00<000000;g5h@000000102na^400`000000_/KP000000030;k7h@80000000@0_lOP0000002o
an40_lOQ0P0000000`2oan000000000000020<38h080000000@0`<OP000000000000`<SP0P000003
0<78h00400000031b>000000000000@0`LSO0P0000040<;8g`0300000033b=l0`lSO00<0`lSO00<0
00000<?8gP0000001033b=h01@000000a<SN000000000000a<SN008000000P34b=d010000000a<SM
0000000000040<G8g@0300000035b=`0aLSL00<0aLSL00<000000<G7g00000000`35am/01@000000
aLOK000000000000a/OK008000000P36a]X010000000a/KJ0000000000040<K6fP0300000036aMT0
a/GI00<0a/GI00<000000<K5f00000000`37aMP01@000000a/CH000000000000alCG008000000P36
`mL00`000000al?F000000050<O2eP0300000037`MD0al7E0080al7E00<000000<K1e00000000`37
`M@2000000050<Joe0000000akoC0<Nod`0000000P36_]<010000000akkB0000000000030<JmdP03
00000037_M40000000@0a[cA00<000000<Jkc`36^ll00`36^ll00`000000ak[>0<NjcP040<NjcP03
00000037^Ld0akW=0080akW=00<000000<Jgc00000000`37]l/00`000000a[G:000000030<Neb@03
00000036/lP0000000<0ak?700@000000<Jaa@36/LD000000P37/L@010000000aJk20<F^`P000002
0<N^`@0400000035Z[h0aJZn00000080ajVl00@000000<>T^P33Y;X000000P36XkH3000000<0l<FT
00L000000>_9/P3[bK80jlVb0000003J]kL0000000@0g;nl00<000000=c4`@0000000`3F_L800`00
0000e/750=K1a@020=K1a@030000003B`<P0d/3800<0d/3800<000000=?3b`0000000P3@`</20000
0080d<?=00<000000<o1c@0000001@3?`lh00`000000c/?@0<k3d0030<k3d0030000003>aM400000
00<0cLCA00L000000<g5dP3=aM80cLGB0000003<aM<0000000@0cLKC00<000000<c5e00000001@3<
a]@00`000000c<OF0<c7eP020<c7eP030000003<b=H0000000<0c<SG00L000000<c9e`3<bML0c<WG
0000003;bMP0000000@0c<[H00<000000<c;f@3<bmT00`3<bmT00`000000c<cJ000000020<_<fP80
00000P3<cM/00`000000blgK000000050<_>g0030000003;cmd0bloM0080bloM00D000000<_@g@00
000000000<_@gP02000000<0bm7N00<000000<[Ag`3:dMl00`3:dMl00`000000b];P000000020<WB
h00600000039dn40bM?Q00000039d^4000001@39dn800`000000b=CS0<SDh`020<SDh`0400000038
e>@000000<ODi08000001037e>@00`000000a]CU0<KDi@020<KDi@0300000035eNH0000000<0aMCV
0P0000000`34eNH000000<CDiP02000000<0`mCW0P0000040<;Di`8000000`31dnL2000000030<7C
j0000000`=;X0080`=;X00<000000;oBj00000000P2odNP2000000040;kAj0000000_]3X0;k@j080
00000P2md>P010000000_LoX0000000000020;c?j0@000000`2lc^P3000000<0^lgX00@000000;_<
j000000000000P2jc>P00`000000^/_W0;[;i`02000000040;W;j0000000^L[W0;W:i`80000000@0
^<[W0000002hbNL0^<WW0P0000020;O9i`030000002gb>L0]lSW0080000000<0]lSW0000002ganH0
0`2ganH00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo02<0oooo000P0?ooo`8000007P3o
ool2000003/0oooo0P0000040;_4h@040000002ka>400000000000<0_<CQ00@000000;c5h@000000
00000`2maN4010000000_LGQ0000000000030;g5h@040000002na^40000000000080_/KP00@00000
0;k7h@0000000000102oan02000000D0_lOP0P0000050<37h00300000031b>00`LSP00@0`LSP00<0
00000<78g`31b=l00`31b=l00`000000`/SO000000030<;8g`0500000032b=l0000000000033b=l0
0P0000030<?8gP0400000033b=h00000000000@0a<SN00<000000<C8g@34b=d00`34b=d00`000000
aLSM0<G8g@030<G8g@0500000035am`0000000000035b=`00P0000020<G7g00400000035am/00000
000000@0aLOK00<000000<K6fP36a]X00`36a]X00`000000a/KJ0<K6fP030<K6fP0500000036aMT0
000000000036aMT00P0000020<K5f00400000037aMP00000000000@0a/CH00<000000<K3e`36`mL0
0`36`mL00`000000al;F0<O2eP030<O2eP0300000036`]H000000080al7E00D000000<K1e036`M@0
00000<O1e002000000<0a[oD00<000000<Nod`0000001036_]<00`000000a[gB0<JmdP040<JmdP03
00000036_=40a[cA0080a[cA00<000000<Jkd00000000`36^ll00`000000a[[?000000040<NjcP03
00000036^Ld000000080akW=00D000000<Jgc036]l`000000<Ngb`0200000080a[G:00@000000<Ne
b@00000000000`36/lP00`000000ak?7000000030<Jaa@0300000037/L@0000000<0aJk200<00000
0<N^`@0000000`35Z[h00`000000ajVl000000030<>T^P0300000036XkH0aZ>f00<000000`3`aJ@0
0`000000jlVb0>_9/P020>_9/P030000003L_k`0g;nl0080g;nl00L000000=c4`@3La<40g<C10000
003F_L80000000@0e/750P0000040=;0b0060000003C`l/0dl?;0000003@`</000001@3@`ld00`00
0000cl?>0<o3cP030<o3cP030000003?a=00000000<0c/?@00L000000<k5d@3>aM40c/GA0000003=
a=40000000D0cLGB00<000000<g6d`3=a]<0103=a]<00`000000c<KD0<c6e0030<c6e0030000003=
b=D000000080c<OF00L000000<c8eP3<b=H0c<SF0000003<b=L0000000D0c<WG00<000000<c:f03<
b]P00P3<b]P00`000000c<_I000000030<c;f@060000003<c=X0c<cJ0000003;c=X000001@3<cM/0
0`000000blkL0<_>g0020<_>g080000000<0c<oL0000003;cmd00P0000040<_@g@030000003;dMh0
bm7N0080bm7N00<000000<_Bg`0000000P3:dMl200000080b];P00<000000<WBh00000001039dn40
0`000000bM?R0<WChP020<WChP80000000<0bMCS00000038e><00P0000030<SDi00300000037e>@0
amCT0080amCT0P0000000`36eND000000<KDi@0200000080aMGV00@000000<GDiP00000000001034
eNH00`000000`mCW0<?Di`020<?Di`0300000033e>L000000080`]CW0P0000000`32e>P000000<7C
i`02000000<0`M?X00@000000<3Bj000000000000`2od^P2000000@0_]7X0P0000040;g@j0800000
1@2lcnP2000000@0_<kX00<000000;_=j02kcNP00`2kcNP2000000@0^/cX0`0000040;W;j0800000
102hb^L2000000D0]lWW0P0000030;O8i`030000002ganH0]lOV0080000000<0]/KW0000003oool0
5`3oool00`000000oooo0?ooo`0S0?ooo`00803oool00`000000oooo0?ooo`0J0?ooo`<00000?P3o
ool00`000000^lCQ0;_4h@030;_4h@030000002la>40_<CQ00@0_<CQ00<000000;g5h@2maN40102m
aN400`000000_LGQ0;g5h@040;g5h@030000002na^00_/KP00<0_/KP00<000000;o7h02oan000`2o
an000`000000_lKP000000040;o7h00300000030an00000000@0`<OP00@000000<37g`0000000000
0P31b>02000000040<77g`00000000000<78g`8000000P32b=l010000000`/SO0000000000040<;8
g`0300000033b=h0`lSN00@0`lSN00<000000<C8gP34b=h00`34b=h00`000000a<SM000000020<C8
g@80000000@0a<OM000000000000aLSM0P0000040<G7g00300000035am`0aLOL00<0aLOL00<00000
0<G7f`35am/00`35am/00`000000aLKK000000020<K6fP80000000@0a/KJ000000000000a/KJ0P00
00040<K5f@0300000036aMP0a/GH00<0a/GH00<000000<K4f036a=P00`36a=P00`000000a/?G0000
00030<K3e`0500000036`]H0000000000037`]H00P0000020<K2eP0300000037`MD0000000@0a/7D
00<000000<Joe036_m@01036_m@00`000000a[kC0<Jnd`020<Jnd`0300000036_]80000000@0a[gB
00<000000<Jld@0000000P36_=401@000000a[_@0<Jkd0000000a[_?008000000`36^/l010000000
ak[>0000000000020<Jic@0300000037^Ld0000000@0a[O<00<000000<JebP36]LX00`36]LX00`00
0000a[?80<Jcb0040<Jcb00300000036/LD0a[7500<0a[7500<000000<F^`P35[/800`35[/800`00
0000aJZn0<FZ_P030<FZ_P0300000033Y;X0`jBj00<0`jBj0P0000000`30W;<000000?35Y0030?35
Y0070000003[bK80jlVb0>_9/P000000i/^k000000030=bo_0030000003La<40g<C10080g<C100<0
00000=K1a@3F`LD00P3F`LD010000000e/C80=K4b00000020=;0b0030000003C`l/0dl?;0080dl?;
00<000000=33c@3@`ld00`3@`ld00`000000dLC>000000030<o3cP070000003?a=00clC@0<o4d000
0000c/?@000000050<k5d@030000003=aM80cLGB00<0cLGB0P0000050<g6d`030000003=am@00000
0080c<KD0P0000020<g8e@030000003<amH0000000D0c<SF00<000000<c9e`3<bML00P3<bML20000
00050<c:f0000000c<[H0<c:f00000000`3<bmT00`000000c<_I000000040<c<fP030000003<cM/0
c<gK00<0c<gK00@000000<c>f`000000blkL0P0000040<c?g0030000003;d=d0bm3M00<0bm3M00<0
00000<cAgP0000000P3;dMh01P000000bm;O0<_Bg`000000b]7O000000D0b];P00<000000<WCh@39
dn400P39dn4010000000bMCR00000039dn82000000@0bMCS00<000000<SDi038e>@00P38e>@01000
0000amKT00000037e>@2000000@0a]GU00<000000<GEiP35eNH00`35eNH00`000000a=GV0<CEiP02
0<CEiP80000000<0a=GW00000033e>L00P0000030<?Di`8000001032e>P00`000000`M?X0<7Cj003
0<7Cj0800000102od^P00`000000_m;Y000000020;kAj080000000@0_]7Y0000002md>P0_M3X0P00
00020;g@j@030000002lcnP0_<oX0080000000@0_<oY000000000000_<kX0P000000102kc^T00000
0;_=j02kcNP2000000040;[=j0000000^/cX0;[<j08000000P2ic>P00`000000^L_X0;W;j0020000
00040;S;j0000000^<[W0;S:i`80000000<0^<WX0000002gbNL00P2gbNL3000000@0]lSW0`000004
0;K6i`030000003oool0oooo01D0oooo00<000000?ooo`3oool08`3oool00200oooo00<000000?oo
o`3oool05`3oool300000480oooo00<000000;_4h@2ka>400P0000000`2la>000000000000020;c4
h@80000000<0_<CP0000000000000P2maN42000000030;g5h000000000000080_LGQ0P000000102n
aN0000000;k6h02na^02000000040;k6h000000000000;o7h08000000`2oa^0010000000_lOP0000
000000030<37h00400000030an000000000000@0`<OO0P0000040<77g`8000001@32b=l00`000000
`/SO0<;8g`030<;8g`0400000033b=h0000000000080`lSN0P0000001033b=h0000000000034b=h2
000000<0a<SM0P0000050<C7g@0300000035am`0aLOL00<0aLOL00<000000<G7g00000000`35am`0
1@000000aLOK000000000000aLOK008000000`35a]/2000000D0a/KJ00<000000<K5f@36aMT00`36
aMT00`000000a/CI000000030<K5f00500000036a=P0000000000036a=P00P0000020<K3e`040000
0036`mL00000000000@0a/;F00<000000<K2eP36`]H00`36`]H00`000000a/7D0<K1e0020<K1e003
00000036`=@0000000@0a[oD00@000000<Jod`000000a[kC0P0000030<JndP0400000036_M800000
00000080a[cA00<000000<Jld@0000001036^m000`000000a[[?0<Jjc`040<Jjc`0300000036^Ld0
a[W=00<0a[W=00<000000<Jgc036]l`00P36]l`2000000@0a[G:00<000000<Fcb@0000001036/lP2
000000@0a[750P0000040<F^`P8000001035Z[h2000000D0`jBj00@000000<2L/`30W;<000000`3`
aJ@00`000000jlVb0>_9/P020>_9/P030000003Vbk/000000080g;nl00<000000=c4`@3La<400P3L
a<42000000<0e/7500L000000=K4b03Fa<P0e/C80000003B`<P0000000@0dl?;00<000000=;4c000
00000`3@`ld010000000dLC>0=74cP0000030<o3cP030000003?a=00clC@0080clC@00<000000<k5
d@3>aM400`3>aM42000000@0cLGB00@000000<k6d`3>a]<000000P3=a]<200000080cLOD00<00000
0<c6e00000001@3=b=D00`000000c<SF0<c8eP020<c8eP80000000<0cLWG0000003<bML00P000003
0<c:f0030000003<b]P0000000D0c<_I00<000000<c<fP3<c=X00P3<c=X01@000000c<gJ00000000
0000c<gK008000000`3<c]/00`000000c<oL0<c?g0030<c?g0030000003<d=d000000080bm3M0P00
00020<cAgP030000003;dMh0000000@0bm;O00<000000<[Bh03:d^000P3:d^02000000030<[Ch@00
0000bM?Q008000000`39e>800`000000bMCS0<WDh`030<WDh`0400000038e^@000000<SDi0800000
0`37e^@00`000000a]GU0<KEi@020<KEi@80000000@0a]KV00000035eNH0aMGV0P0000000`35e^L0
00000<CEiP02000000@0a=GW00<000000<?Di`33e>L00P33e>L00`000000`]GX000000030<;Dj003
00000031e>P000000080`M?X0P0000020<3Cj@040000002od^P00000000000<0_m;Y0P0000040;kA
j@800000102md>T3000000<0_<oY0`0000040;_>j@800000102jcNP2000000@0^LcX0`0000030;S;
j0<000000`2hbNP010000000]lWW0000000000030;O8j0040000002gb>L0000000000080]/OW00<0
00000;K6i`2fa^L00P0000000`2ea^L000000?ooo`0E0?ooo`030000003oool0oooo02<0oooo000O
0?ooo`030000003oool0oooo01D0oooo0`0000160?ooo`8000001@2la>02000000D0_<CP0P000005
0;g5h0800000102naN02000000D0_/KP00<000000;o6h02oa^00102oa^000`000000`<OP0<37h004
0<37h00300000030aml0`<OO00<0`<OO00<000000<77g`0000000P31aml200000080`LOO00<00000
0<;8g`32b=l00P0000001032b=h0000000000032b=l2000000@0`lSN0P0000050<?8gP0300000034
b=d0a<SM0080a<SM00<000000<C7g@0000001034amd01@000000a<OL000000000000aLOL00800000
0P35am`010000000aLOL0000000000040<G7f`0300000035a]/0aLKK0080aLKK00<000000<G6fP00
00001036a]X01@000000aLGI000000000000a/GI008000000P36a=T010000000a/GH000000000004
0<K4f00300000036`mL0a/?G00<0a/?G00<000000<K2eP36`]H00`36`]H00`000000a/;F00000003
0<K2eP0400000036`MD000000<K1e08000000`36`=@010000000a[oD0000000000030<Jod`030000
0036_]80a[kB00@0a[kB00<000000<Jld@36_=400`36_=400`000000a[_@0<Jkd0020<Jkd0030000
0035^/l0000000@0a[[?00<000000<FicP0000000`36^Ld00`000000aKO=000000020<Jgc0040000
0035]L/0aKG;00000080a[G:00@000000<Fcb@35/lT000000`36/lP010000000a;770<Baa`000002
0<Jaa@0400000034[/@0a:k400000080aJk200@000000<>[`@33Zl4000000P35Z[h010000000`ZJm
0<:V_@0000030<>T^P8000000`30W;<010000000l<FT0?35Y00000030>_9/P070000003Vbk/0i/^k
0>K;^`000000g;nl000000030=c4`@040000003Ka/D0flK500000080e/7500<000000=K4b03Fa<P0
0P3Fa<P00`000000dl?;0=?3b`020=?3b`040000003Ba<`0d/C<000000<0d<?=00L000000=74cP3A
a<h0dLC>0000003?`lh0000000@0clC@0P0000040<k5d@040000003>a]80c/KB00000080cLGB00L0
00000<k6d`3>a]<0c/KC0000003=a]<0000000D0cLOD00<000000<g8e@3=b=D00P3=b=D200000003
0<g9eP000000c<SF00800000103=bML00`000000c<[H0<c:f0040<c:f0030000003<bmT0c<_I0080
c<_I0P0000000`3<c=T000000<c<fP02000000@0c<gJ00<000000<c>f`3<c]/00P3<c]/00`000000
c<oL000000020<c?g08000000P3<d=d00`000000bm3M000000050<cAgP030000003;d]l0bm;O0080
bm;O00@000000<_Ch0000000b];P0P0000040<[Ch@0300000039e>80bMCR0080bMCR00D000000<WE
h`00000000000<WDh`02000000<0b=KT00<000000<OFi037e^@00P37e^@010000000amKU00000036
eND2000000@0a]KV0P0000040<GFi`0300000034eNL0a=GW0080a=GW0P0000000`33e^P000000<?D
i`0200000080`]GX00@000000<;Dj000000000000`31e>P2000000@0`=?Y0P0000040;oBj@030000
002nd^T000000080_]7Y0P000000102mdNT000000;g@j@2md>T2000000040;c@j@000000_<oY0;c?
j@8000000P2kcnT00`000000^lkY0;_>j@02000000040;[>j@000000^/gX0;[=j080000000@0^LgY
0000002ic>P0^LcX0P000000102hc>T000000;S;j02hbnP200000080^<_X00<000000;S9j02hbNP0
0P2hbNP00`000000]lSX0;O8j0030;O8j0800000102fanL3000000<0]LKW00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`0S0?ooo`007`3oool00`000000oooo0?ooo`0B0?ooo`<00000BP3o
ool200000080_<CP0P0000000`2la>0000000;c4h0030;c4h0030000002ma>00000000<0_LGP0P00
0000102naN0000000;k5h02naN02000000030;k5h000000000000080_/KP0P0000000`2oa]l00000
000000020;o6h080000000<0_lKO0000000000000P30an02000000040<37g`00000000000<37g`80
00000`31aml2000000D0`LOO0P0000050<;8gP0300000033b=h0`lSN00<0`lSN00<000000<?7gP00
00000`33b=h2000000030<?7g@000000a<SM008000000`34amd010000000a<OM0000000000040<C7
g00300000035am`0aLOL00<0aLOL00<000000<G7f`35am/00`35am/010000000aLKK00000035a]/2
000000<0aLKJ00@000000<K6fP00000000001035aMT00`000000a/CI0<K4f@030<K4f@0300000036
a=P0a/CH00<0a/CH00<000000<K3f00000000`36`mL01@000000a/;G000000000000a/;F00800000
0P36`]H010000000a/;F0000000000030<K1e@0300000036`=@0a/3D00@0a/3D00<000000<Jod`36
_m<00P36_m<00`000000aKkC000000040<JndP0300000035_M80000000<0a[cA00@000000<Fkd@00
0000a[_@0P0000030<Fjc`0400000036^/l0000000000080aKW>00@000000<Jic@00000000000P35
]ld00`000000a[O<000000030<Feb`0300000036]LX0000000@0aK?900<000000<Jcb00000000`34
/LL00`000000a[75000000030<B^a00300000035[/80000000<0`j_100<000000<FZ_P0000000`32
Y[d010000000`jBj0<>T^P0000020;nO^@0400000030W;<0`9bc0<2L/`8000000`3[bK800`000000
i/^k0>K;^`030>K;^`050000003La<40g<C10=c4`@0000000`3Ka/D00`000000e/75000000040=K4
b0030000003Ea//0000000<0dl?;00L000000=;4c03Ba<`0d/C<0000003@`ld0000000D0dLC>00<0
00000<o4d03?a=000P3?a=0010000000clGA0<o5d@0000020<k5d@070000003>a]80c/KB0<k6dP00
0000cLGB000000050<k6d`030000003=am@0cLOD00<0cLOD00@000000<g8e@000000cLSE0P000004
0<g9eP030000003=bML0cLWG00<0cLWG00<000000<g;e`0000000`3<b]P2000000030<g;f0000000
c<_I00800000103<c=T00`000000c<gJ0<c=fP030<c=fP030000003=c]/000000080c<kK00H00000
0<c?g03<cm`000000<c?g00000050<c@g@030000003<dMh0c=7N0080c=7N0P0000000`3<d]l00000
0<_Bg`02000000<0bm?P00<000000<[Ch@3:dn400P3:dn42000000030<[EhP000000bMCR00800000
1039eN<00`000000b=KT0<SFi0020<SFi00400000038e^@000000<OFi08000000`37e^D00`000000
a]KV0<KFiP020<KFiP80000000@0a]OW00000035e^L0aMKW0P0000000`34e^L000000<CEi`020000
00@0`mKX00<000000<;Ej032eNP00`32eNP00`000000`MCX0<7Dj0020<7Dj00300000030e>T00000
0080`=?Y0P0000020;oCj@040000002od^T00000000000<0_];Y0P0000040;gAj@800000102ld>T2
000000@0^loY0`0000030;[>j@<00000102icNT2000000@0^<cY0P0000040;S;j0030000002hbNP0
^<WX0080000000@0]lWX0000002gb>P0]lSX0P000000102fb>P000000;K7i`2fanL2000000030;G7
j0000000]LKW00<0]LKW0P00000F0?ooo`030000003oool0oooo0280oooo000O0?ooo`030000003o
ool0oooo00l0oooo0`00001?0?ooo`800000102la>0010000000_<CP0000000000020;g4h0040000
002maN000000000000@0_/GP0P0000050;k5h08000001@2oa]l2000000D0_lKO0P0000050<37g`03
00000031aml0`LOO0080`LOO00<000000<77g`0000001031aml010000000`/ON0000000000020<;8
gP80000000@0`/ON000000000000`lSN0P0000020<?7gP0400000033b=h00000000000@0`lOM00<0
00000<C7g@34amd01034amd00`000000a<OL0<C7g0020<C7g080000000@0a<OL00000035am`0aLOL
0P0000001035a]/0000000000035am/2000000<0aLKK00<000000<G6fP35a]X01035a]X00`000000
aLGI0<G5f@020<G5f@80000000<0aLCI00000036a=T00P36a=T01@000000aLCH000000000000a/CH
008000000P36`mP010000000a/?G0000000000040<K2e`0300000036`]H0a/;F00<0a/;F00<00000
0<K1e@36`MD00P36`MD00`000000aL3E000000040<K0e00400000035_m@000000<Jod`8000000`35
_]<010000000a[kB0000000000020<FmdP0400000036_=400000000000<0aK_A00<000000<Fjc`35
^/l01035^/l00`000000aKW>0<FicP030<FicP0300000035]ld0aKO=00<0aKO=00<000000<Feb`35
]L/00`35]L/00`000000aK?90<Fcb@040<Fcb@0300000034/LL0a;7700<0a;7700<000000<B^a034
[/@00`34[/@00`000000`j_10<>[`@030<>[`@0300000032Y[d0`ZJm00<0`ZJm0P0000030;nO^@03
00000030W;<0`9bc0080`9bc00D000000>_9/P3[bK80jlVb000000040>K;^`030000003Rc<800000
0080g<C100<000000=_6a@3Ka/D00P3Ka/D00`000000e/C80=K4b0020=K4b0070000003Ea//0eLK;
0=G6b`000000dl?;000000050=;4c0030000003Aa<h0dLC>00<0dLC>00<000000=75c`0000000P3?
a=001`000000clGA0<o5d@3?aM4000000<k5d@0000001@3>a]800`000000c/KC0<k6d`030<k6d`03
0000003>am@000000080cLOD0P0000030<g8e@030000003=bMH0cLWF00<0cLWF00<000000<g:eP00
00000P3=bML200000080cL_G00@000000<c:f00000000000103=bmP00`000000c<cI0<c<f@030<c<
f@030000003=cMX000000080c<gJ0P0000020<g>f`030000003<c]/0000000@0c<oL00<000000<c@
g@3<d=d00`3<d=d010000000c=7M0000003<dMh2000000@0c=;O00<000000<_Ch03;dn000P3;dn00
10000000bmCQ0000003:dn42000000@0b]GR00<000000<WEh`39eN<00P39eN<2000000030<WFh`00
0000b=KT008000000`38e^@00`000000amKU0<OFi@020<OFi@0400000037enH000000<KFiP800000
1036enL2000000@0a=KW00<000000<?Fj033e^P00P33e^P2000000040<?Fj0000000`]GX0<;Ej080
000000<0`MGY00000031e>P00P0000030<3Dj@800000102odnT2000000<0_];Y0P0000000`2md^X0
00000;gAj@020;gAj@030000002ldNX000000080_=3Y0P000000102kd>X000000;_?j@2kcnT20000
00040;[?jP000000^/kY0;[>j@8000000P2ic^T00`000000^LgY0;W=j@02000000040;W=j@000000
^<cY0;S<j@80000000<0^<cY0000002hbnP00`2hbnP3000000<0]lWX0`0000040;K8j0800000102e
anP010000000]LKW0000000000020;C6i`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
8P3oool001l0oooo00<000000?ooo`3oool00P00000:0?ooo`<00000D`3oool00`000000_<CP0;c4
h0020;c4h08000001@2ma>000`000000_/GP0;k5h0030;k5h0030000002naMl0000000@0_/GP00<0
00000;o5g`000000102oa]l00`000000_lKO000000040;o6g`0400000030a]l0000000000080`<OO
0P0000000`31aml000000<77g`02000000<0`LOO00@000000<77g`00000000001032amh2000000D0
`/ON00<000000<?7gP33amh00`33amh00`000000`lOM0<?7g@030<?7g@0400000033amd000000000
0080a<OM0P0000000`34am`000000<C7g002000000@0a<OL0P0000050<G6f`0300000035a]/0aLKK
0080aLKK00<000000<G5fP0000001035a]X010000000aLGJ00000035aMT2000000<0aLCI00@00000
0<K4f@00000000001035a=P00`000000a/?H0<K3f0030<K3f00300000036`]L0a/;G0080a/;G0P00
00000`35`]H000000<K2eP020<K2eP0400000035`MH000000<K1e@8000000`35`=D010000000a/3D
0000000000030<Foe00300000035_]<0aKkC00@0aKkC00<000000<FmdP35_M800`35_M800`000000
aK_A0<Fkd@020<Fkd@0300000035^]00000000@0aK[?00<000000<Fic`0000000`35^Lh00`000000
a;O>000000030<Fgc@0300000034]/`0000000<0aKG;00<000000<BdbP0000001035/lT00`000000
a;78000000030<Baa`8000001034[/@2000000@0`j_10P0000050<:V_@030000002oWkT0_ini00<0
_ini00P000000<2L/`30W;<000000;NG]P000000jlVb000000D0i/^k00D000000>;<`P000000g<C1
000000040=_6a@030000003Ib<P0000000<0e/C800<000000=G6b`3Ea//00P3Ea//00`000000d/C<
0=;4c0030=;4c0030000003Ba/h000000080dLC>0P0000020=75c`030000003?a=00000000D0clGA
00<000000<k6dP3>a]800`3>a]800`000000clOC000000030<k6d`060000003>am@0c/OD0000003=
am@00000103=b=D00`000000c/WF000000030<g9eP060000003=b]H0cL[F0000003=bML000001@3=
bmL00`000000cL_H0<g;f0030<g;f0030000003=c=T000000080c<cI0P0000020<g=fP030000003<
cMX0000000D0cLkK00<000000<c?g03<cm`00P3<cm`01@000000cM3L000000000000c=3M00800000
0`3<dMd00`000000c=;O0<cBg`020<cBg`80000000<0c=?O0000003;dn000P0000030<_Dh@030000
003:eN80b]GR00<0b]GR00@000000<[FhP000000bMGS0P0000040<WFh`0300000038e^@0b=KT0080
b=KT00@000000<SGi@000000amKU0P0000030<OGiP0300000036enL0a]OW0080a]OW0P0000001035
enL000000<CFi`34e^L2000000030<CGj0000000`mKX008000001033e^P2000000<0`MGY0P000004
0<3Dj@030000002oe>X000000080_m?Y0P0000000`2ndnX000000;kBj@02000000<0_M;Z00@00000
0;gAj@00000000000`2ldNX2000000@0^m3Z0P0000040;[?jP800000102ic^T3000000@0^LgY0P00
00030;S<j@030000002hbnP0^<_X0080000000@0]l[Y0000002gbNP0]lWX0P0000020;K9j0030000
002fb>P0]/SX0080000000<0]LSX0000002eanP00`2eanP2000000@0]<KW0P00000D0?ooo`030000
003oool0oooo0280oooo000O0?ooo`<000002@3oool3000005H0oooo00<000000;c4h02la>000P00
00020;g4g`030000002ma>00_LCP0080000000@0_LCO000000000000_/GP0P0000030;k5g`040000
002naN000000000000<0_lGO00@000000;o6g`00000000000`2oa]l010000000_lKO000000000004
0<36g`8000001031aml00`000000`LOO0<77g`040<77g`0300000032amh0`/ON00<0`/ON00<00000
0<;7gP0000001032amh00`000000`/ON000000020<?7gP80000000@0`lOM000000000000`lOM0P00
00040<?7g@8000001034am`00`000000a<OL0<C7g0030<C7g00300000034a]/0000000@0aLKK00@0
00000<C6f`000000aLKK0P0000030<G5fP0400000035a]X00000000000<0aLGJ00<000000<G4f@35
a=T01035a=T00`000000aLCH0<G4f0020<G4f080000000<0aL?H00000036`mP00P36`mP010000000
aL;G00000036`]L2000000<0aL;F00@000000<K2eP00000000000`35`MH00`000000aL3E0<G0e@04
0<G0e@0300000035_m@0aKoD0080aKoD00<000000<Fnd`0000001035_]<00`000000aKgB00000003
0<FmdP0300000035_=4000000080aK_A00H000000<Fjd035^]00aK[@00000035^/l200000080aKW?
00@000000<FicP00000000000P34]lh010000000aKO=0000000000020<Bfc00400000035]L/00000
000000<0a;C:00@000000<Fcb@00000000000P34/LP010000000a;770000000000020<>_aP040000
0034[/@0a:k400000080`Z_300@000000<>[`@33Zl4000000P31Yl0010000000`ZJm0<:V_@32Y[d2
000000D0_ini00<000000<2L/`0000000P2gUkH00`000000jlVb000000040>K;^`050000003Rc<80
h/c20>;<`P000000103Ka/D01`000000fLS80=W8b03Ib<P000000=K4b0000000103Ea//2000000@0
d/C<00H000000=;6cP3Ba/h000000=74cP0000050=75c`030000003?aM40clGA00<0clGA00<00000
0<o7dP0000000`3>a]801`000000clOC0<o7d`3?am<000000<k6d`000000103>am@00`000000cLSE
0<g8e@020<g8e@070000003>bMH0c/WF0<k9eP000000cLWF000000040<g:eP030000003=bmL0cL_G
00<0cL_G00<000000<g<f00000000`3=bmP01P000000cLcI0<g<f@000000c<cI000000D0cLgJ00<0
00000<g>f`3=c]/00P3=c]/2000000030<g?f`000000c<oL00800000103=d=`00`000000c=7M0<cA
g@020<cAg@040000003=d]h000000<cBg`800000103<dml00`000000bmCQ0<_Dh@020<_Dh@050000
003;eN4000000000003:eN800P0000030<[FhP0300000039e^<0bMKS0080bMKS0P0000000`39en@0
00000<SFi002000000<0b=OU00<000000<OGiP37enH00P37enH010000000a]SW00000036enL20000
00@0aMOW0P0000040<CGj00300000033e^P0`mKX0080`mKX0`0000020<7Ej@8000000P31e^T01000
0000`=CY0000000000030;oDjP800000102ndnX00`000000_M;Z0;gBjP030;gBjP8000000`2ldNX2
000000030;_AjP000000^m3Z0080^m3Z00<000000;[@jP0000000P2jcnX2000000040;[?jP000000
^LkY0;W>j@80000000<0^LkZ0000002icNT00P2icNT2000000030;S<jP000000^<cY00<0^<cY0P00
00040;O:j@800000102fbNP3000000<0]LSX00<000000;G7j02eanP00P000000102danP000000;C6
i`2da^L2000000030;?6j0000000oooo01<0oooo00<000000?ooo`3oool08P3oool001h0oooo0P00
00090?ooo`800000FP3oool2000000D0_LCO0P0000050;g4g`030000002naMl0_/GO00@0_/GO00<0
00000;o5g`2oaMl0102oaMl00`000000_lKO0;o6g`040;o6g`0300000030a]l0`<KO00<0`<KO00<0
00000<36g`0000000`31aml010000000`LKN0000000000020<77g`80000000@0`LON000000000000
`/ON0P0000030<;7gP0400000032amh00000000000<0`/ON0P0000050<?7g@0300000033amd0`lOM
00<0`lOM00<000000<?7g00000000`34am`01@000000a<KL000000000000a<OL008000000`34a]/0
10000000aLKK0000000000030<C6f`0300000035aMX0aLGJ00@0aLGJ00<000000<G5fP35aMX00P35
aMX00`000000aLCI000000040<G4f@0400000035a=T000000<G4f08000000`35`mP010000000a/?H
0000000000030<G2e`0300000035`]H0aL;F00@0aL;F00<000000<G1eP35`MH00P35`MH00`000000
aL3E000000040<G0e@0300000035_m@000000080aKoD00H000000<Fnd`35_]<0aKkC00000035_]<2
00000080aKgB00@000000<FmdP00000000000P35_=400`000000aK_A000000050<Fjd00300000035
^Ll0aKW?00<0aKW?00<000000<BgcP34]lh00`34]lh00`000000a;K<0<Bfc0030<Bfc00300000034
]<X0a;C:00@0a;C:00<000000<Bab034/LP00`34/LP00`000000`jo60<>_aP020<>_aP0300000034
[/@0000000<0`Z_300<000000<>[`@0000000`31Yl0010000000`ZJm0<:V_@0000020;jR_@030000
002oWkT0_ini00<0_ini00<000000;NG]P2gUkH00P2gUkH00`000000i/^k0>K;^`020>K;^`070000
003Rc<80h/c20>;<`P000000glc6000000030=_6a@030000003Ib<P0fLS80080fLS800<000000=G6
b`3Ea//00P3Ea//010000000eLO=0=G7c@0000020=;4c0030000003Ba/h0d/K>0080d/K>00<00000
0=75c`3AaLl00`3AaLl00`000000dLOA000000030<o5d@070000003?am80clOB0<o7dP000000c/KB
000000050<o7d`030000003>am@0c/OD0080c/OD00H000000<k8e000000000000<g8e@0000050<k9
eP030000003=b]H0cL[F0080cL[F00<000000<k;e`0000000`3=bmL01`000000cLcH0<g<f03=c=P0
00000<g;f0000000103=c=T00`000000cLgJ0<g=fP030<g=fP040000003=c]X000000<g>f`800000
103=cm/00`000000cM3L0<g@g0030<g@g0040000003=dMd000000<cAg@8000000`3=d]h00`000000
c=?O0<cCg`020<cCg`80000000<0c=CP0000003;e>400P0000040<_Eh@030000003:e^80b]KR0080
b]KR00@000000<[Gh`000000bMKS0P0000040<WGi00300000038enD0b=OU0080b=OU00@000000<OH
iP000000amOV0P0000030<KHi`0300000035enL0aMOW0080aMOW0P0000001034f>P000000<CGj034
enP2000000030<?Gj@000000`mKX008000000`32enT2000000@0`MKY0P0000040;oDjP030000002o
e>X000000080_]?Z0P000000102mdnX000000;gBjP2md^X2000000030;cBjP000000_=7Z00800000
0`2kdNX010000000^m3Z0000000000030;[@jP800000102jcnX2000000@0^LkZ0`0000030;S<jP03
0000002hc>T0^<cY0080000000@0]l_Y0000002gb^T0]l[Y0P000000102fb^T000000;K9j02fbNP2
000000030;G9j@000000]LSX00<0]LSX0`0000040;C7j0800000102ca^P00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo0280oooo000N0?ooo`030000003oool0oooo00D0oooo0`00001M0?oo
o`8000000P2ma=l2000000030;g4g`00000000000080_LCO0P0000000`2na=l00000000000020;k5
g`80000000<0_/GO0000000000000P2oaMl2000000030;o5g`00000000000080_lKO0P000000102o
a]l0000000000030a]l200000080`<KO00@000000<77g`00000000001031a]h2000000D0`LON00<0
00000<;7gP32amh01032amh00`000000`/ON0<;7gP020<;7gP0300000032amd0000000@0`lOM00D0
00000<?7g@00000000000<?7g@0200000080`lOL00@000000<C7g000000000001034a]`00`000000
a<KK0<C6f`040<C6f`0300000034a]/0a<KK0080a<KK00<000000<C5f`0000001035aMX010000000
aLGJ00000035aMX2000000<0aLCI00@000000<G4f@00000000000`35a=T00`000000aL?H0<G3f004
0<G3f00300000035`]L0aL;G0080aL;G00<000000<G2e`0000001035`]H010000000aL7F00000035
`MH2000000<0aL3E00@000000<G0e@00000000000P35_m@00`000000aKoD000000050<Fnd`030000
0035_M80aKgB00<0aKgB00<000000<Fld@35_=400`35_=400`000000aK[@0<Fjd0030<Fjd0030000
0034^M00000000<0aKW?00<000000<BhcP0000000`34]lh00`000000a;K=000000030<Bfc0030000
0033]</0000000@0a;C:0P0000040<Bab08000001@33[lH00`000000`Z_30<:[``030<:[``030000
0031Yl00`JO000<0`JO00P0000030;jR_@030000002oWkT0_ini0080_ini0P0000040;NG]P800000
0P3Vbk/01@000000h/c20>;<`P3Rc<80000000<0glc600<000000=_6a@000000103Ib<P00`000000
f<W;000000020=G6b`070000003Eald0eLO=0=G7c@000000d/C<000000040=;6cP030000003Ball0
000000<0dLG?00L000000=77d@3Aam40dLOA0000003?aM40000000D0clOB00<000000<o7d`3?am<0
0`3?am<010000000clSC0000003>am@2000000@0c/SD00<000000<k9eP3>bMH00`3>bMH00`000000
c/[F000000020<g:eP070000003>bmL0c/_G0<k;e`000000cL_G000000050<g<f0030000003=c=T0
cLcI0080cLcI00D000000<g=f@00000000000<g=fP02000000<0cLkJ00<000000<g?f`3=cm/00`3=
cm/00`000000cM3L000000020<g@g08000000`3=dMd00`000000cM;N0<gBgP020<gBgP040000003=
e=l000000<cCg`800000103<e>000`000000bmGQ0<_Eh@020<_Eh@80000000<0bmOR0000003:e^80
0P0000030<[Gh`0300000039en@0bMOT0080bMOT0P0000000`39f>D000000<SGi@02000000<0amSV
00<000000<KHi`36f>L00P36f>L010000000a]WX00000035enL2000000@0a=SX0P0000040<?Gj@03
00000032enT0`]OY0080`]OY00<000000<7GjP0000000P31e^T200000080`=KZ00@000000;oDjP00
000000000`2oe>X2000000@0_M?Z0P0000040;cBjP030000002kdNX0^m7Z00<0^m7Z0P0000040;[@
jP030000002jd>/0000000<0^/oZ0`0000040;W>jP030000002hc>X0^<cZ00<0^<cZ0`0000030;O;
j@<00000102fb^T2000000@0]LWY00@000000;G8j000000000000P2db>P00`000000]<OX0;C7j002
000000030;C6j0000000/lKX00<0/lKX00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0R
0?ooo`00303oool01P2oM000000000000000000004RL0=ooo``0oooo00<000000?ooo`3oool00P3o
ool300000680oooo0P0000050;g4g`8000001@2na=l2000000D0_/GO0P0000050;o5g`800000102o
a]l2000000D0`<KO00<000000<76gP31a]h00`31a]h00`000000`LKN000000040<77gP0400000032
a]h0000000000080`/ON0P0000000`32amh000000<;7gP02000000<0`/OM00@000000<?7g@000000
00001033amd00`000000`lOL0<?7g0030<?7g00300000034a]`0a<KL00<0a<KL00<000000<C6g000
00001034a]/010000000a<KK00000034a]/2000000<0a<GK00@000000<G5fP00000000000`35aMX0
0`000000aLCI0<G4f@040<G4f@0300000035a=T0aLCI0080aLCI00<000000<G3f00000001035`mP0
10000000aL?H00000035`]L2000000<0aL;G00@000000<G2eP00000000000`35`MH00`000000aL3E
0<G0e@040<G0e@0300000035_m@0aKoD00<0aKoD00<000000<Fnd`35_]<00`35_]<00`000000aKgC
000000030<FmdP0300000034_=80000000<0aKcA00<000000<Bkd@0000000`35^]001@000000a;W@
0<Bid0000000aKW?008000000P34^<h010000000a;O>0000000000020<Bfc@0400000034]/`00000
000000<0`kC;00@000000<BdbP00000000000P33//T010000000a;780<Bab00000020<:_a`040000
0033[lH0`jo60<>_aP8000001032Zl<00`000000_jW3000000040<6W`0030000002nX[d0_Z:m00<0
_Z:m00@000000;nO^@2oWkT000000P2iVkX02@000000]iNf0;NG]P2gUkH000000;B@/@000000i/^k
000000020>;<`P030000003Oc<H0glc600<0glc600<000000=W8b03Ib<P00P3Ib<P01P000000f<W;
0=S9b`000000eLK;000000D0eLO=00<000000=;6cP3Ba/h00P3Ba/h01`000000d/O?0=;7c`3Ball0
00000=75c`0000001@3Aam400`000000clOB0<o7dP030<o7dP030000003?b=8000000080clOC0P00
00030<o8d`030000003>b=@0c/SD00<0c/SD00<000000<k9e@0000000P3>bMH200000080c/[F00<0
00000<g:eP0000001@3>bmL00`000000cLcH0<g<f0030<g<f0030000003>c=P000000080cLcI00<0
00000<g=f@3=cMT00P3=cMT00`000000cLkJ0<g>fP020<g>fP030000003>cm/000000080cLoK0P00
00020<g@g0030000003=d=`0000000@0cM7M00<000000<gCgP0000000P3=d]h01@000000cMCO0<gD
g`3=e=l0000000D0c=CP00@000000<cFh@000000bmGQ0P0000040<_GhP030000003:en<0b]OS0080
b]OS00@000000<[Hi0000000bMOT0P0000040<WHi@0300000037f>H0amSV0080amSV00@000000<OI
i`000000a]SW0P0000030<KIj00300000034f>P0a=SX0080a=SX0P0000001034fNT000000<?Gj@33
enT2000000030<;Hj@000000`]OY008000000`31enX2000000@0`=KZ0P0000030;oDjP80000000@0
_]G[0000002mdnX0_M?Z0P000000102mdn/000000;cBjP2ld^X2000000040;cBj`000000^m7Z0;_A
jP8000000P2kdN/010000000^]3Z0000000000020;[@j`040000002jcnX0000000000080^Lo[00<0
00000;W>jP2ic^X00P000000102hcNX000000;S<jP2hc>X2000000040;O<jP000000]l_Y0;O;j@80
00000P2fbnX00`000000]/[Y0;K:j@02000000030;G:j@000000]LWY00<0]LWY0P0000040;C8j0<0
00000`2da^P00`000000/lKX0;?6j002000000030;?5j0000000oooo0140oooo00<000000?ooo`3o
ool08P3oool000d0oooo00<0omnL04P0001d_ol03P3oool00`000000oooo00000002000006H0oooo
00<000000;g4g`2ma=l00P2ma=l00`000000_/CO000000040;k4g`030000002na=l0000000@0_/GO
00<000000;o5g`000000102oaMl00`000000_lGO000000020;o6g`8000000P30a]h00`000000`<KO
0<36g`02000000040<76gP00000000000<76gP8000000`31a]h010000000`LON0000000000040<;6
gP8000001032amh00`000000`/OM0<;7g@040<;7g@0300000033amd0`lOM00<0`lOM00<000000<?6
g@0000000P33am`2000000040<?6g000000000000<C6g08000000`34a]`010000000a<KK00000000
00030<C6f`0300000034aM/0a<GK00@0a<GK00<000000<G5fP35aMX00P35aMX010000000a<CJ0000
000000020<G4f@80000000<0aLCI00000035a=T00P0000030<G3f00400000035`mP00000000000<0
aL?H00<000000<G2e`35`]L01035`]L00`000000aL7F0<G1eP020<G1eP0300000035`=H0000000@0
aL3E00<000000<Foe@0000000`35_m@01@000000aKkD000000000000aKkC008000000P35_M<01000
0000aKgB0000000000020<BldP0400000035_=400000000000<0a;_A00<000000<Fjd00000001034
^M000`000000a;S>0<BhcP030<BhcP0300000034]/d0a;K=00<0a;K=00<000000<>db`33]</01033
]</00`000000`k;90<>bb@020<>bb@0300000034/LP0000000<0`Zo700@000000<>_aP0000000000
0P31[<D010000000`Z_30<:[``0000020;nY``0700000031Yl00`JO00<6W`0000000_JC000000004
0;jR_@030000002oWkT0000000<0^I^j00<000000;NG]P2gUkH00P2gUkH3000000<0h/c200<00000
0=o<aP3Oc<H00P3Oc<H00`000000g<g9000000020=W8b0030000003HbL/0f<W;0080f<W;00<00000
0=G7c@3Eald00`3Eald00`000000e<S>000000030=;6cP030000003Ball0d/O?0080d/O?00<00000
0=77d@3Aam400`3Aam400`000000d<SB000000020<o7dP8000000P3?b=800`000000clOC00000004
0<o8d`030000003?bM@000000080c/SD0P0000020<k9e@030000003>bMH0000000D0c/[F00<00000
0<k;e`3>bmL00`3>bmL00`000000c/cG000000020<g<f08000000P3>c=P00`000000cLcI00000004
0<g=f@030000003>c]X000000080cLkJ00H000000<k?f`3>cm/000000<g?f`0000050<g@g0040000
003=dMd0cM7M0<gAg@8000000P3=dmh00`000000cM;N000000030<gDg`030000003=eN00000000<0
c=CP00D000000<cFh@3<e^40c=KQ000000040<_GhP80000000<0bmSS0000003:en<00P0000030<[H
i00300000039f>D0bMSU0080bMSU0P0000000`38fNH000000<OHiP02000000<0amWW00<000000<KI
j036fNP00P36fNP010000000aMWX00000034f>P2000000@0a=WY0P0000040<;Hj@0300000031enX0
`MOZ0080`MOZ00<000000<3Gj`0000000P30e^X2000000030;oFj`000000_mCZ00800000102neN/2
000000@0_M?[0P0000040;cBj`800000102kdN/2000000@0^]3[0P0000050;W?j`800000102hcNX2
000000@0]lcZ0P0000040;K;jP<000000`2eb^T00`000000]LWY0;G9j@02000000040;C8j@000000
]<SX0;C8j080000000<0]<OY0000002da^P00`2da^P3000000<0/lGX0P00000A0?ooo`030000003o
ool0oooo0280oooo000=0?ooo`030?oOW0180000M;oo00h0oooo0P00001Z0?ooo`040000002ma=l0
0000000000<0_/CO00@000000;k4g`00000000000`2na=l010000000_/GO0000000000030;o5g`04
0000002oaMl00000000000<0_lGO0P0000050<36gP8000001@31a]h00`000000`LKN0<76gP040<76
gP0300000032a]h0`/KN0080`/KN0P0000000`32a]h000000<;7gP020<;7gP0400000032a]d00000
00000080`/OM0P0000001033a]d0000000000033amd2000000<0`lKM0P0000050<?6g00300000034
a]`0a<KL00@0a<KL00<000000<C6f`34a]/00P34a]/010000000a<GK0000000000020<C5f`800000
00<0a<GJ00000035aMX00P0000040<C4fP8000001035a=T00`000000aL?H0<G3f0040<G3f0030000
0035`mP0aL?H0080aL?H00<000000<G2e`0000001035`]L010000000aL7G00000035`MH2000000<0
aL3F00@000000<G0e@00000000000P35_mD010000000aKoD0000000000040<Fne00300000035_M<0
aKgC00<0aKgC00<000000<BldP34_=800`34_=800`000000a;_A0<Bkd@040<Bkd@0300000034^M00
a;W@0080a;W@00<000000<>hc`0000000`34^<h2000000@0a;K=00<000000<>ec@0000001033]</2
000000D0`k;900<000000<:_a`32[lL00`32[lL00`000000`Jc50<6/a@020<6/a@0300000032Zl<0
000000@0_jW300<000000<6W`00000000P2mY<000`000000_Z:m0;jR_@030;jR_@030000002iVkX0
^I^j0080^I^j00<000000;NG]P2gUkH00P2gUkH01P000000ZHbf0:V<]P000000h/c2000000@0glc6
00H000000=c=b@3LcLT000000=W8b00000040=S9b`030000003Gb/d0000000<0eLO=00L000000=C8
cP3Db<h0e<S>0000003Ba/h0000000@0d/O?00<000000=;8d@0000000P3Aam4200000080d<SB00<0
00000<o7dP0000001@3?b=800`000000clSC0<o8d`020<o8d`060000003?bM@0clWD0000003>b=@0
00001@3>bMD00`000000c/[F0<k:eP030<k:eP030000003>bmL000000080c/_G0P0000020<k<e`03
0000003=c=P0000000D0c/cH00@000000<g=f@3=cMT0cLgI0P0000020<k>fP030000003=c]X00000
00@0c/oK00<000000<g@g03=d=`00`3=d=`01@000000c];M0000003=dMd0000000D0cM?N00D00000
0<gDg`3=e=l0cMCO000000030<gEh0030000003<e>00000000<0c=KQ00@000000<cGhP000000bmOR
0P0000040<_Hh`030000003:f>@0b]ST0080b]ST00@000000<[Ii@000000bMSU0P0000040<SIiP04
00000037fNL0amWW0<OIi`80000000<0a][X00000036fNP00P0000030<GIj00300000034fNT0a=WY
0080a=WY0P0000001033fNX000000<;Hj@32f>T2000000030<;HjP000000`MOZ008000000`30en/2
000000@0_mK[00<000000;kEj`2neN/00P2neN/2000000040;gEk0000000_M?[0;gCj`80000000@0
_=?/0000002ld^/0_=;[0P000000102kd^/000000;_Aj`2kdN/2000000040;[Aj`000000^]3[0;[@
j`8000000P2id>/00`000000^Lo[0;W?j`02000000040;S>j`000000^<gZ0;S=jP80000000@0]lg[
0000002gc>X0]lcZ0P000000102fc>X000000;K;jP2fbnX2000000030;K;jP000000]L[Y00<0]L[Y
0`0000040;C8j@800000102danT01P000000]<KX000000000000/lKX00000080/lGX0P0000020;;5
j0030000003oool0oooo00l0oooo00<000000?ooo`3oool08@3oool000d0oooo00<0omnL04P0001d
_ol03`3oool00`000000oooo0?ooo`1Y0?ooo`030000002na=l0_/CO00@0_/CO00<000000;k4g`2n
a=l0102na=l00`000000_lGO0;o5g`040;o5g`030000002oaMl0_lGO0080_lGO00<000000<35gP00
00001030a]h00`000000`<KN000000040<76gP0400000031a]h0000000000080`LKN0P0000000`31
a]h000000<;6gP02000000<0`/KN00@000000<;7gP00000000001032a]d2000000D0`lKM00<00000
0<?6g@33a]d00P33a]d00`000000`lKL000000040<?6g00400000033a]`0000000000080a<KL0P00
00000`34a]/000000<C6f`02000000@0a<GK0P0000040<C5fP0300000034a=X0a<CJ00<0a<CJ00<0
00000<C4f@0000000`35a=T010000000a<?I0000000000020<G3f080000000<0aL?H00000035`mP0
0P0000030<G2e`0400000035`]L00000000000<0aL7G00<000000<G0eP35`=H01035`=H00`000000
aKoE0<Foe@030<Foe@0300000035_]@0aKkD00<0aKkD00<000000<Bme00000000`35_M<00`000000
a;cC000000030<BldP0300000034^m80000000@0a;_A00<000000<Bjd00000000P34^M001@000000
`kS?0<>hc`000000a;S>008000000P33]lh010000000a;K=0<Bfc@0000020<>ec@0500000033]</0
`kC;0<>db`0000000P32/l/01`000000`k;90<>bb@33//T000000<6`b@0000000`32[lL2000000D0
`Jc500<000000;nY``2oZL<0102oZL<00`000000_JC00;fT`0020;fT`0040000002nX[d0_Z:m0;jR
_@8000001@2iVkX00`000000]iNf00000002000000<0ZHbf00<000000=o<aP3Oc<H00P3Oc<H00`00
0000g<g90=c=b@020=c=b@030000003HbL/0f<W;0080f<W;00L000000=O:c@3Gb/d0el[=0000003E
ald0000000D0e<S>00<000000=;7c`3Ball00P3Ball01P000000d/SA0=;8d@000000dLOA000000D0
d<SB00<000000<o8dP3?b=800`3?b=800`000000clSC000000020<o8d`030000003?bM@0clWD0080
clWD00<000000<k9e@3>bMD00`3>bMD00`000000c/[F000000020<k:eP8000000P3>bmL00`000000
c/_G000000050<k<e`030000003>c=P0c/cH0080c/cH0P000000103>cMT000000<g=f@0000050<k>
fP030000003>cm/0c/oK0080c/oK00D000000<kAf`00000000000<g@g002000000<0c];M00<00000
0<gCgP3=dmh00P3=dmh2000000040<gDgP000000cMCO000000D0cMGP00D000000<cFh@3<e^40c=KQ
000000030<cGhP030000003;f><0bmSS0080bmSS0P0000000`3;fN@000000<[Hi002000000<0b]WU
00<000000<SIiP38fNH00P38fNH3000000030<OIi`000000000000@0a][X00<000000<GIj035fNP0
0P35fNP010000000a=[Y00000034fNT2000000@0`mWZ0P0000030<;HjP8000001030en/00`000000
_mO[000000020;oFj`80000000<0_]K/0000002neN/00P0000040;gEk08000000`2ldn`3000000@0
^m;[0P0000040;[Aj`800000102id>/3000000<0^<k[0`0000040;O=j`800000102fc>X2000000@0
]/_Z00@000000;G:j@00000000000P2dbNX00`000000]<SY0;C8j@02000000030;C8j@000000]<OY
0080]<OY0`0000040;?6j0800000102baNP00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
0240oooo000=0?ooo`030?oOW0180000M;oo0100oooo0P00001Z0?ooo`8000000P2na=l200000003
0;k4gP00000000000080_/CO0P0000000`2oa=h00000000000020;o5g`80000000<0_lGN0000002o
aMl00P0000030<35gP0400000030a]h00000000000<0`<KN00@000000<76gP00000000001031a]h2
000000@0`LKN00<000000<;6gP32a]h01032a]h00`000000`/KM0<;6g@030<;6g@0300000032a]d0
000000@0`lKM00@000000<?6g@000000`lKM0P0000030<?6g00400000033a]`00000000000@0`lKL
0P0000040<C6f`0300000034aM/0a<GK00<0a<GK00<000000<C5fP0000000`34aMX01@000000a<CJ
000000000000a<CJ008000000P34a=T010000000aLCI0000000000040<C3f@8000001035`mP00`00
0000aL;G0<G2e`040<G2e`0300000035`ML0aL7G0080aL7G00<000000<C0eP0000001035`=H00`00
0000a;oF000000020<Foe@80000000@0a;kD000000000000aKkD0P0000020<Bme00400000035_M<0
000000000080a;cC00@000000<BldP00000000000`34^m8010000000a;_A0000000000020<Bjd003
00000034^M00000000@0`kS?00<000000<>gcP33]lh00P33]lh00`000000a;K=000000040<>ec@03
00000033]</0000000<0`[?;00@000000<>bb@00000000000P31/<T010000000`Zo7000000000002
0<2]a`0700000031[<D0`Jc50<6/a@000000_j[5000000040;nY``800000102mY<0010000000_Z:m
0000000000020;VN_@030000002iVkX0^I^j0080^I^j0P0000030:nC^@050000002YS;H0ZHbf0:V<
]P0000000`3Oc<H01`000000g<g90=c=b@3LcLT000000=[=c00000000P3HbL/00`000000el[=0=O:
c@030=O:c@030000003Db<h0e<S>00<0e<S>00<000000=?9d00000000P3Ball00`000000d/SA0=;8
d@020=;8d@030000003@b=80d<SB00<0d<SB00<000000=38dP0000000P3?b=8200000080clSC00<0
00000<o8d`000000103?bM@00`000000clWE000000030<k9e@060000003>b]H0c/[F0000003>b]H0
00001@3>bmL00`000000c/cG0<k<e`030<k<e`040000003>cMP000000<k<f0800000103>cMT00`00
0000c/kJ0<k>fP030<k>fP040000003>cmX000000<k?f`800000103>dM/00`000000c];M0<kBg@02
0<kBg@040000003>dmd000000<gCgP800000103=e=h00`000000cMGP0<gEh0020<gEh080000000@0
cMKQ0000003<e^4000000`3<en8010000000c=WS0000003;f><2000000@0bmWT00<000000<[Ii@3:
fND00P3:fND010000000bM[V00000038fNH2000000<0b=[W00<000000<KJj036f^P00P36f^P20000
00030<KKj@000000aMWX008000000`34f^T00`000000`mWZ0<?IjP020<?IjP<000000P32f>X20000
0080`MS[00@000000<3Gj`00000000000`2oen/2000000@0_]K/00<000000;gEk02meN`00P2meN`3
00000080_=?/0P0000020;_Ck0030000002kd^/0^m;[0080000000@0^];/0000002jdN/0^]7[0P00
0000102idN`000000;W@j`2id>/2000000040;S?k0000000^<k[0;S>j`80000000<0]lk[00000000
00000P2gcN/2000000040;K=j`000000]/cZ0;K<jP80000000<0]/c[0000002fbnX00P2fbnX30000
00@0]<WZ0`0000030;C8j@@000000P2canT00`000000/lKX0;?6j002000000030;;6j@000000//GX
0080//GX0`00000@0?ooo`030000003oool0oooo0240oooo000=0?ooo`030?oOW0180000M;oo0180
oooo00<000000?ooo`3oool0J@3oool2000000D0_/CN0P0000050;o4gP800000102oaMh00`000000
`<GN0<35gP040<35gP0300000030a]h0`<KN00@0`<KN00<000000<76gP31a]h00`31a]h00`000000
`LKN000000030<76gP0400000032a]h0000000000080`/KN0P0000001032a]d0000000000032a]d2
000000<0`/KM00@000000<?6g@00000000000`33a]d00`000000`lKL0<?6g0040<?6g00300000033
a]`0`lKL0080`lKL0P0000000`33aM`000000<C6f`020<C6f`0500000034aM/0000000000034aM/0
0P0000020<C5fP0400000034aMX00000000000@0a<CJ00<000000<C4f@34a=T00`34a=T00`000000
a<?I0<C3f@030<C3f@0300000034`mP0000000<0aL?H00<000000<C2f00000001035`]L010000000
a<7G00000035`ML2000000<0a<3F00@000000<G0eP00000000000`34_mH2000000D0a;kD00<00000
0<Bme034_M@00`34_M@00`000000a;cC0<Bld`030<Bld`0300000034^m80a;_B00@0a;_B00<00000
0<Bjd034^]000`34^]000`000000`kS?0<>hc`020<>hc`8000001@33]lh00`000000`kG=0<>ec@04
0<>ec@0300000032/l/0`[?;00<0`[?;00<000000<6`b@31/<T00`31/<T00`000000`:g70<2]a`02
0<2]a`0400000031[<D0000000000080_j[500D000000;nY``2oZL<0_jW3000000020;fV`P030000
002mY<00_JC00080_JC000<000000;VN_@2iW[d00P2iW[d010000000^I^j0;VK^P2iVkX2000000@0
[i>i00@000000:V<]P2YS;H0ZHbf0P0000000`3Oc<H000000=c=b@020=c=b@060000003JcL`0f/g<
0000003HbL/000001@3Gb/d00`000000e/_?000000020=C8cP8000000P3CbM000`000000d/O?0000
00040=;8d@030000003AbM8000000080d<SB0P0000020=38dP030000003?b=80000000D0clSC00<0
00000<o9e03?bM@00P3?bM@01`000000clWE0<o9e@3?bMD000000<k9e@000000103>b]H00`000000
c/_G0<k;e`030<k;e`030000003>c=L000000080c/cG0P0000030<k=f0030000003>cMT0c/gI00<0
c/gI00<000000<k>f@0000000P3>c]X2000000<0c/oJ00<000000<kAf`3>dM/00`3>dM/010000000
c];L0000003>d]d2000000<0c]?M00<000000<gDgP3=e=h00P3=e=h2000000030<kEg`000000cMGP
00800000103=e^401@000000c=OR0<cGhP3<en80000000<0c=WS00<000000<_Ii03;fN@00P3;fN@2
000000030<_Ji@000000b]WU008000000`39f^H00`000000b=[W0<SJi`020<SJi`0400000037fnP0
00000<KJj08000001036fnT00`000000a=[Y0<CJj@020<CJj@0400000033f^X000000<?IjP800000
0`32fN/2000000@0`MS[0P0000030;oGj`80000000@0_]O/0000002ne^`0_]K/0P0000000`2me^`0
00000;gEk002000000<0_=C/0P0000040;_Ck0<000000`2jd^`3000000@0^M7/0P0000040;S?k080
0000102gc^/3000000<0]/g[0`0000030;K<j`@000000P2dbnX00`000000]<WZ0;C9jP0200000004
0;C9jP000000]<SY0;C8j@80000000<0/lSZ0000002canT00`2canT3000000@0//KY0`0000020;75
j0030000003oool0oooo00h0oooo00<000000?ooo`3oool08@3oool000d0oooo00<0omnL04P0001d
_ol04`3oool00`000000oooo0?ooo`1Y0?ooo`030000002na=h0_/CN0080_/CN00<000000;k4gP00
00000`2oa=h2000000040;o4gP000000_lGN0;o5gP80000000<0_lGN0000000000000P30aMh20000
00030<35gP00000000000080`<KN0P0000001031a]h0000000000031a]h200000080`LKN00@00000
0<76gP00000000001032a]h2000000D0`/KM00<000000<;6g@32a]d01032a]d00`000000`lKM0<?6
g@020<?6g@0400000033a]`0000000000080`lKL0P0000000`33a]`000000<?6g002000000<0`lGL
00@000000<C6f`00000000001034aM/00`000000a<GJ0<C5fP030<C5fP0300000034a=X0a<CJ00<0
a<CJ00<000000<C4fP0000000`34a=T01@000000a<?I000000000000a<?I008000000P34`mP01000
0000aL?H0000000000030<C2f00400000035`]L00000000000<0a<7G00<000000<C0eP34`=H01034
`=H00`000000a;oF0<BoeP020<BoeP0300000034_mD0000000@0a;kD00<000000<Bne00000000`34
_M@00`000000a;cC000000030<Bld`0300000033^m80000000@0a;_B00<000000<>jd@0000000`34
^]000`000000`kW@000000020<>hc`0400000033]ll0`kO?000000<0`kO>00<000000<:ecP000000
1033]Ld00`000000`[?<000000030<:cb`0300000031/L/0000000<0`K390P0000050<2]a`030000
002oZ/D0_j[500<0_j[500<000000;nY``0000000`2mY/8010000000_JC00;fT`02mY<02000000D0
^Ijm0`0000020;>H_0050000002_TkT0[i>i0:nC^@000000102YS;H2000000<0g<g900<000000=[=
c03JcL`00P3JcL`00`000000el[=0=O:c@030=O:c@060000003Fbll0e/_?0000003Db<h000001@3C
bM000`000000d/SA0=;8d@020=;8d@060000003AbM80dLWB0000003@b=8000001@3@b=800`000000
clSC0<o8d`030<o8d`060000003@bM@000000000003?bM@000001@3?bMD00`000000c/[F0<k:eP02
0<k:eP030000003?bmH0000000<0c/_G00H000000<k<e`3>c=L000000<k<e`0000040<k=f0030000
003?cMP0000000<0c/gI00H000000<k>f@3>c]T000000<k>fP0000040<k?fP030000003?dM/00000
0080c]7K0P0000030<kBg0030000003>dmd0c]?M0080c]?M00@000000<kDgP000000cMCN0P000004
0<kEg`030000003=e^40cMKQ0080cMKQ0P000000103=f>4000000<cGhP0000030<cIh`040000003<
f^@000000<_Ii0800000103;f^D00`000000bM[V0<WJiP020<WJiP0400000039fnL000000<SJi`80
00000`37fnP00`000000a]_Y0<KKj@020<KKj@80000000<0aM_Z00000034f^T00P0000030<?JjP03
00000032fN/0`]W[0080`]W[00<000000<7Jk00000000P31f>/200000080`=W/0`0000030;kGk0<0
0000102me^`010000000_=C/0;cDk02le>`2000000040;_Dk@000000^m?/0;_Ck080000000@0^]?]
0000002jd^`0^];/0P0000020;WBk0030000002idN`0^M7/0080000000@0^=3/0000002hcn`0^<o/
0P000000102gcn`000000;O>j`2gc^/400000080]/g[0P000000102fcN/000000;K<j`2fc>/20000
0080]<_[00<000000;C;jP2dbnX00P2dbnX2000000@0]<WZ0P0000040;?8jP040000002canT00000
00000080//OY00<000000;;6j@2ba^T00P0000000`2aa^T000000;75j0030;75j0030000003oool0
oooo00d0oooo00<000000?ooo`3oool08@3oool000d0oooo00<0omnL04P0001d_ol0503oool00`00
0000oooo0?ooo`1Y0?ooo`040000002na=h0000000000080_/CN00@000000;o4gP0000000000102o
a=h2000000D0_lGN0P0000050<35gP8000001@31a]h00`000000`LKN0<76gP030<76gP0300000032
a]h0`/KN00<0`/KN00<000000<;6g@0000001032a]d010000000`/KM0000000000020<;6g@800000
00<0`/KM00000033a]d00P0000040<?6g08000001033a]`00`000000`lGL0<?5g0040<?5g0030000
0034aM/0a<GK00<0a<GK00<000000<C5f`0000000`34aMX01@000000a<CJ000000000000a<CJ0080
00000P34a=X010000000a<CI0000000000040<C3f@0300000034`mP0a<?H00<0a<?H00<000000<C2
f034`]P01034`]P00`000000a<7G0<C1e`020<C1e`0300000034`ML0000000@0a<3F00@000000<Bo
eP000000a;oF0P0000030<Boe@0400000034_]@0000000000080a;kD00@000000<Bme00000000000
0P34_=<010000000a;cC0000000000030<>kdP0400000034^m80000000000080`k[A00@000000<Bj
d000000000000P33^M000`000000`kS?000000030<>gc`0400000033]lh00000000000<0`[G>00@0
00000<>ec@00000000000P32/l`010000000`[?;0000000000020<6ab`0400000031/<T000000000
0080`:k900L000000<2]a`30[LL0`:g70000002n[<L0000000D0_j[500<000000;fV`P2mY/800`2m
Y/8300000080^Z;000<000000;VN_@2iW[d00P2iW[d00`000000/iRl0;>H_0020;>H_0050000002_
TkT0[i>i0:nC^@000000102YS;H300000080g<g900L000000=[=c03JcL`0f/g<0000003IcLh00000
00<0el[=00<000000=K;c`3Fbll00P3Fbll00`000000dlW@0=?9d0030=?9d0030000003CbM400000
0080d/SA00<000000=79dP3AbM800P3AbM800`000000d<SB0=38dP030=38dP030000003@bM<00000
00<0clSC00<000000=39e03@bM@00P3@bM@00`000000clWE0<o9e@030<o9e@030000003?bmD00000
0080c/[F00L000000<o;eP3?bmH0cl_F0000003>bmL0000000@0c/cG00<000000<k=f03>cMP00P3>
cMP01`000000clgH0<o=f03?cMP000000<k=f@000000103>c]T010000000c/oJ0<k?fP3>cmX20000
0080cm7K00<000000<kAf`000000103>d]`00`000000cm?M000000020<kCg@050000003>e=h0c]CN
0<kDgP0000001@3>eMl010000000c]OP0000003=e^42000000@0cMSQ00D000000<cIh`3<fN<0c=WS
000000030<cJi0030000003;f^D0bm[U0080bm[U0P0000000`3:fnH000000<WJiP02000000<0bM_W
00<000000<OKj037fnP00P37fnP010000000a]cY00000036fnT2000000<0aM_Z0P0000040<?JjP04
00000033fn/000000<;Ij`8000000`31f^`2000000<0`=W/0`0000020;kGk08000000P2mend00`00
0000_MK/0;gFk002000000040;cFk@0000000000000000@0^mC]0P0000040;[Ck@800000102id^`3
000000<0^=3/0`0000040;O?k08000000`2fc^`3000000@0]/g[0P0000040;C;j`030000002dbnX0
]<_Z0080000000@0]<[[0000002dbNX0]<WZ0P0000000`2cbNX000000;?8jP030;?8jP800000102b
anT3000000<0/LKY00<000000;75j02aaNP00P0000000`2aa>T000000?ooo`0=0?ooo`030000003o
ool0oooo0240oooo000=0?ooo`030?oOW0180000M;oo01D0oooo0P00001Z0?ooo`030000002na=h0
_/CN00<0_/CN00<000000;o4gP2oa=h00`2oa=h00`000000_lCN000000040;o5gP0300000030aMh0
000000@0`<GN00@000000<35gP00000000000P31a]h2000000040<75gP000000`LKN0<76gP800000
00@0`LKM000000000000`/KN0P0000030<;6g@0400000032a]d00000000000@0`/KM0P0000040<;6
g@0300000033a]`0`lKL00<0`lKL00<000000<?5g00000000`33a]`010000000`lGL000000000002
0<?5g080000000@0`lGK000000000000a<GK0P0000020<C5f`0400000034aMX00000000000@0a<CJ
00<000000<C4fP34a=X00`34a=X00`000000a<?I0<C3f@030<C3f@0300000034`mT0000000<0a<?H
00@000000<C2f000000000000P34`]P2000000030<C1f0000000a<7G008000000`34`ML010000000
a<3F0000000000030<BoeP0300000034_mD0a;oE00@0a;oE00<000000<Bne034_]@00`34_]@00`00
0000a;cC0<Bld`030<Bld`0300000033^m80`k_B00@0`k_B00<000000<>jd@33^]400`33^]400`00
0000`kW@0<>id0030<>id00300000033]ll0`kO?00<0`kO?00<000000<:ecP32]Lh01032]Lh00`00
0000`[?<0<:cc0030<:cc00300000031/L/0`K7;00<0`K7;00<000000<2^b@30[/T00P30[/T01000
0000`:g70000000000020;j/a`030000002oZ/D0_j[50080_j[50P0000050;fV`P030000002jX/00
^Z;00080^Z;000@000000;VN_@2iW[d0^Ijm0P0000040;>H_0030000002_TkT0[i>i0080[i>i00<0
00000:V<]P0000000P000000102FPKX000000=c=b@0000030=[=c0070000003IcLh0fLg>0=W=cP00
0000el[=000000040=K;c`030000003Ebm0000000080dlW@0P0000020=?9d@030000003Bb=400000
00@0dLWB00<000000=79dP0000000`3@b=801`000000d<WC0=39d`3@bM<000000<o8d`000000103@
bM@00`000000d<[D000000020<o9e@8000000P3?bmD00`000000c/[F000000050<o;eP030000003>
c=L0c/cG0080c/cG00H000000<o=f000000000000<k=f00000050<o=f0030000003>c]T0c/kI0080
c/kI00D000000<o@fP000000c/oJ000000050<oAf`040000003>d]`0c];L0<kBg08000000P3?dmd0
0`000000c]?M000000030<kDgP050000003?e]l000000000003>eMl00P0000030<kGh0030000003=
f>40cMSQ0080cMSQ0P000000103=fN8000000<cIh`0000030<cJi0040000003<fnD000000<_Ji@80
0000103:fnH00`000000bM_W0<WKi`020<WKi`0400000038g>P000000<OKj08000000`36g>T01000
0000aM_Z0<GKjP35fnX200000080a=c[00@000000<?JjP00000000000`33fn/00`000000`M[/0<7J
k0020<7Jk00400000030f^`000000<3Ik08000000`2ofNd2000000@0_MO]0`0000030;cFk@030000
002ke^d000000080^mC]0P000000102je>d000000;[Ck@2jdnd2000000040;WCk@000000^M;/0;WB
k080000000@0^=7]0000002hd>`0^=3/0P0000020;O@k@030000002gcn`0]lo/0080000000@0]/o/
0000002fc^`0]/k/100000020;K=j`80000000<0]Lc/0000002dbn/00`2dbn/3000000<0]<[[0`00
00030;?9jP030000002cb>X0/lSZ0080000000@0//SZ0000002banT0//OY0P0000000`2aa^X00000
0;76j@030;76j@<000000`2aa>T00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0240oooo
000=0?ooo`030?oOW0180000M;oo01L0oooo00<000000?ooo`3oool0J03oool00`000000_/CN0;k4
gP02000000040;o4gP00000000000;o4gP8000000`2oa=h010000000_lGN0000000000030<35gP04
00000030aMh00000000000@0`<GN0P0000040<75gP8000001@31a]d00`000000`/KM0<;6g@040<;6
g@0300000032a]d0`/KM0080`/KM0P0000000`32a]d000000<;6g@020<;6g@0500000032a]`00000
00000033a]`00P0000020<?5g00400000033a]`00000000000@0`lGL0P0000050<?5f`0300000034
aM/0a<GK00<0a<GK00<000000<C4fP34a=X00`34a=X00`000000a<CJ000000020<C4fP80000000@0
a<?I000000000000a<?I0P0000020<C3f@0400000034`mP00000000000@0a<;H0P0000040<C1f003
00000034`ML0a<7G00@0a<7G00<000000<BoeP34_mH00P34_mH00`000000a;oF000000030<Boe@80
000000<0`kkE00000034_]@00P34_]@00`000000`kgD000000030<Bld`0300000033_=<0000000@0
`k_B00<000000<>jdP0000000`33^]400`000000`kWA000000030<>id00300000032^=00000000<0
`kO?00<000000<:fc`0000001032]Lh00`000000`KC=000000030<:cc08000001031/L/2000000D0
`:k900<000000;j/a`2n[<L00`2n[<L010000000_j[50000000000020;fXa@040000002mY/80_JK2
0;fV`P8000001@2jX/0300000080]Ibo00D000000;>H_02cV;`0/iRl000000040:nC^@8000000`2R
R[/00`000000UX6j000000040=[=c0030000003IcLh0fLg>0080fLg>00<000000=K;c`3Fbll00P3F
bll01P000000eL_@0=G;d0000000dlW@000000D0dlWA00<000000=79dP3AbM800P3AbM801`000000
dLWB0=79dP3AbM8000000=38dP0000001@3@bM<010000000d<WD0=39e03@bM@200000080d<[D00<0
00000<o9e@0000001@3?bmD00`000000cl_F0<o;eP030<o;eP040000003?c=L000000<k<e`800000
103?cMP00`000000clgH0<o=f0030<o=f0040000003?c]T000000<k>f@8000000`3?d=X00`000000
cm7K0<oAf`030<oAf`050000003?d]/000000<kBg00000001@3?dmd01@000000c]CN0<kDgP3>e=h0
000000@0cmKO00<000000<kGh03>en000P3>en0010000000c]SQ0000003=f>42000000@0cMWR00D0
00000<cJi03<f^@0c=[T000000030<cKi@030000003:fnH0b]_V0080b]_V0P0000000`3:gNL00000
0<WKi`02000000<0b=cX00@000000<KLj@36g>T0a]cY0P0000001036gNX000000000000000040<CL
j`8000000`33fn/2000000030<;Kk0000000`M[/008000000`30f^`00`000000_mW]0;oIk@020;oI
k@030000002nfNd000000080_MO]0P0000020;cGk@040000002le^d00000000000<0^mK]0P000004
0;[Dk@800000102idnd2000000@0^=7]0P0000040;O@k@<000000`2fcn`3000000<0]/k/0`000003
0;G<k0030000002dbn/0]<_[0080000000@0]<_[0000002db^/0]<[[0P0000020;?:j`040000002c
bNX0/lWZ0;?9jP<00000102bb>X2000000@0/LKZ00H000000;76j@00000000000;75j@0000030;74
j@<000003P3oool00`000000oooo0?ooo`0P0?ooo`002`3oool01@3ooml0W4P0000000000000M;oo
01P0oooo00<000000?ooo`3oool0J03oool2000000D0_lCN00<000000;o4gP2oa=h0102oa=h00`00
0000`<GN0<35gP040<35gP0300000030aMh0`<GN00<0`<GN00<000000<75g@0000000`31aMh00`00
0000`LGM000000040<76g@0400000032aMd0000000000080`/KM0P0000000`32a]d000000<;6g@02
000000<0`/KM00@000000<;6g@00000000001032a]`00`000000`lGL0<?5g0030<?5g00300000033
aM`0`lGL00<0`lGL00<000000<?5f`0000001033aM/00`000000`lGK000000020<C5f`80000000@0
`lCJ000000000000a<CJ0P0000030<C4fP8000001@34`mT00`000000a<?I0<C3f@030<C3f@030000
0034`]P0a<;H00<0a<;H00<000000<C1f00000000`34`MP00`000000a<7G000000040<C1e`040000
0034`=L000000<BoeP8000000P34_mH010000000a;oE0000000000030<>ne@0400000034_]@00000
00000080`kgD00@000000<Bld`00000000000`33_=<010000000`k_B0000000000020<>jdP040000
0033^]40000000000080`kWA00@000000<>id000000000000P32^=0010000000`kO?000000000003
0<:fc`0400000032]Lh0000000000080`KC=00@000000<:cc000000000000P30//`010000000`K7;
0<6ab`0000020;n_bP0700000030[/T0`:k90<2^b@000000_Zg8000000050;j/a`030000002mZ<D0
_JS50080_JS500@000000;fV`P00000000000P2jY<<00`000000^Z;00;ZR`0020;ZR`0030000002e
W;l0]Ibo0080]Ibo00<000000;>H_02cV;`00P2cV;`00`000000[i>i00000002000000@0XXZk00<0
000009J1^P2FPKX00P0000000`3JcL`000000=W=cP030=W=cP030000003GcLl000000080e/_?00<0
00000=G;d03Ebm000P3Ebm000`000000dlWA0=?9d@030=?9d@060000003Bb]8000000000003AbM80
00001@3AbM800`000000d<WC0=39d`030=39d`050000003@b]@000000=39e00000001@3@b]@00`00
0000cl_E0<o;e@030<o;e@030000003?bmH000000080cl_F0P0000030<o<e`030000003?cMP0clgH
00<0clgH00<000000<o=f00000000P3?cMP2000000<0clkI00<000000<o@fP3?d=X00P3?d=X01@00
0000cm7J000000000000cm7K008000000`3?d]/00`000000cm?M0<oCg@020<oCg@80000000@0cmCM
0000003>e=h00000103?e]l010000000cmOO0000003>en02000000<0c]SQ00<000000<gIhP3=fN80
0P3=fN82000000040<gKh`000000c=[T000000<0c=_U00@000000<_MiP000000b]_V0P0000040<[M
i`0300000038g>P0b=cX0080b=cX00<000000<OMj@0000000P0000030<KMjP0300000035gN/00000
0080a=c[0P0000020<?Lk0<000000`32fn`2000000<0`=[/0P0000000`2of^d000000;oIk@020000
00<0_]W]0P0000040;cGk@8000000`2ke^d2000000040;[FkP000000^]C]0;[Dk@80000000@0^MC^
0000002idnd0^M?]0P000000102hdnd000000;SAk@2hdNd2000000040;OAk@000000]m3]0;O@k@80
000000@0]]3]0000002fcn`0]/o/0P0000020;K?k@030000002fc^`0]/k/0080000000<0]Lg/0000
002ec>`00P2ec>`3000000@0]<_[0P0000040;?:j`@000000P2bbN/00`000000//SZ0;;8jP020000
00030;77jP000000/LKZ0080/LKZ0`0000040;75j@<000000`2`a>T00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo0200oooo000=0?ooo`030?noM0000000M;oo01T0oooo00<000000?ooo`3o
ool0J03oool00`000000_lCN0;o4gP020;o4gP040000002oa=h0000000000080_lCN0P0000000`30
a=h00000000000020<35gP80000000@0`<GN000000000000`<GN0P0000020<75g@0400000031aMh0
0000000000<0`LGM00@000000<76g@00000000001032aMd2000000@0`/KM00<000000<;6g@32a]d0
1032a]d00`000000`/KL0<;6g0030<;6g00300000033aM`000000080`lGL0P0000001033aM`00000
00000033aM`2000000<0`lGK00@000000<?5f`00000000000`33aM/2000000D0`lCJ00<000000<C4
fP34a=X00P34a=X00`000000a<?J000000040<C3f@0300000034`mT0000000<0a<?I00D000000<C2
f@00000000000<C2f00200000080a<7H00@000000<C1f000000000000`34`ML010000000a<7G0000
000000030<C0e`0300000034_mH0a;oF00<0a;oF00<000000<>ne@33_]D01033_]D00`000000`kgD
0<>me0030<>me00300000033_=<0`kcC00@0`kcC00<000000<>jdP33^]800`33^]800`000000`kWA
0<>id@030<>id@0300000032^=00`[S@00<0`[S@00<000000<:fc`32]/l01032]/l00`000000`KC=
0<6dc@030<6dc@0300000030//`0`;;<0080`;;<00<000000<6ab`0000000`2o[lX010000000`:k9
0000000000020;j]b0030000002n[<L0_Zc70080_Zc70P0000050;fXa@030000002jY<<0^ZC30080
^ZC300@000000;ZR`02jX/00^Z;00P0000040;FL_`030000002cV;`0/iRl0080/iRl0P0000030:VA
_@050000002RR[/0XXZk0:::^`0000000`2FPKX2000000@0fLg>00H000000=O=c`3GcLl000000=K;
c`0000040=G;d0030000003Dbm40000000<0dlWA00<000000=;:dP3Bb]800P3Bb]800`000000dLWB
0=79dP030=79dP030000003Ab]<000000080d<WC0P0000030=3:e0030000003@b]@0d<[D00<0d<[D
00<000000=3;e@0000000P3?bmD200000080cl_F00<000000<o;eP000000103?c=L00`000000clgG
000000020<o=f08000000P3?cMP00`000000clgH000000040<o>f@030000003?cmT000000080cm3J
00<000000<oAfP3?dMX00P3?dMX00`000000cm;K0<oBf`020<oBf`040000003?dm`000000<oCg@80
0000103?e=d010000000cmKO0<oFg`3?e]l2000000<0cmOO00<000000<kHh@3>f>400P3>f>401000
0000c][R0000003=fN82000000@0cM_S00<000000<cKi@3<fnD00P0000030<_MiP030000003:gNL0
b]gW0080b]gW0P0000000`39g^P000000<SLj00200000080amgY00@000000<KNjP000000a]gZ0P00
00030<GMj`8000000`33g>`300000080`]_/0P0000020<7Kk@<00000102of^d010000000_]W]0;kI
k@2nfNd2000000040;cIkP000000_=O]0;cGk@80000000<0^mO^0000002ke^d00P0000030;[FkP<0
00000`2ie>h3000000@0^=?]0P0000040;OAk@800000102fd>d2000000@0]/o]0`0000020;G=k0D0
00000P2dc>`00`000000]<_[0;C;j`02000000040;?;k0000000/l[[0;?:j`80000000<0//W[0000
002bbN/00`2bbN/3000000<0/LOZ100000020;76jP030000002aaNT0/LGY0080000000<0/<GZ0000
002`a>T00`2`a>T2000000`0oooo00<000000?ooo`3oool0803oool002X0oooo00<000000?ooo`3o
ool0J03oool010000000_lCN0000000000040;o4gP8000001@30a=h2000000D0`<GN00<000000<75
g@31aMd00`31aMd00`000000`LGM0<75g@040<75g@0300000032aMd0`/GM00<0`/GM00<000000<;5
g@0000000`32a]d010000000`/GM0000000000020<;6g@80000000@0`/GL000000000000`/KL0P00
00030<?5g08000001@33aM`00`000000`lGK0<?5f`040<?5f`0300000033aM/0`lGK0080`lGK00<0
00000<?4f`0000001033a=X010000000`lCJ00000034a=X2000000<0a<?J00@000000<C3f@000000
00000P34`mT010000000a<?I0000000000040<C2f@0300000034`MP0a<7H00<0a<7H00<000000<C1
e`34`ML01034`ML00`000000a<3G0<C0e`020<C0e`0300000033_mH0000000<0a;oF00<000000<>n
eP0000001033_]D00`000000`kgD000000030<>me00300000033_=@0000000@0`kcC00<000000<:k
d`0000000P33^]82000000030<:idP000000`kWA0080`kWA00<000000<:hd@0000000`32^=000`00
0000`KK?000000040<:fc`0300000031]<h0000000<0`KC=0P0000050<2bc0030000002o[lX0_jo:
00<0_jo:00<000000;j]b02n[LP00`2n[LP010000000_Zc70000000000020;bZa`040000002mZ<D0
_JS50;fXa@8000001@2jY<<300000080]Yo100<000000;FL_`2eW;l00P2eW;l00`000000/iRl0000
0002000000@0ZI6m00D000000:::^`2RR[/0XXZk0000000309J1^P0300000023M;X000000080fLg>
00<000000=O=c`3GcLl00P3GcLl00`000000eL_@0=G;d0020=G;d0070000003Dbm40e<_A0=C;d@00
0000dlWA000000040=;:dP030000003Bbm8000000080dLWB0P0000020=7:d`030000003@bM<00000
00@0d<[D00<000000=3;e00000000P3@b]@200000080d<_E00<000000<o;e@0000001@3?bmH01000
0000clcG0<o<e`3?c=L200000080clgG00<000000<o=f00000001@3?cMP010000000clkI0<o>f@3?
c]T200000080cloI00<000000<o@fP000000103?dMX00`000000d=;K000000020<oBf`050000003?
dm`0cm?L0<oCg00000001@3?e=d01@000000cmKN0000003?e]l0000000@0cmOO00@000000<oIh000
0000c]SQ0P0000030<kJhP030000003=fn<0cM_S0080cM_S0P0000000`3=g>@00000000000040<_M
iP040000003;g^L000000<[Mi`8000000`39g^P200000080amgY0P0000030<KNjP0300000035gN/0
aMg[0080aMg[00@000000<CNk0000000`mc/0P0000030<;Mk@8000000`31fnd2000000040;oKkP00
0000_m[]0;oJk@80000000@0_][^0000000000000000102lfNh2000000@0^mO^00@000000;[FkP2j
e^h0^]K^0P000000102ieNh000000;WDkP2ie>h200000080^=C^00<000000;SCk@2hdnd00P000000
102gd^h000000;OAk@2gdNd2000000040;KAkP000000]]3]0;K@k@80000000@0]]3]0000002fcnd0
]/o]0P000000102ec^d000000;G=k02ecN`4000000@0]<c/0`0000040;?;k0800000102bbN/01P00
0000//W[000000000000/LS[00000080/LOZ0P0000020;77j`030000002aa^X0/LKZ0080/LKZ0P00
00030;35jP030000002`a>T0/<CY0080000000<0[lCY0000003oool02`3oool00`000000oooo0?oo
o`0P0?ooo`00:`3oool2000006T0oooo00<000000;o4gP2oa=h00`2oa=h00`000000_lCM00000004
0<34gP0300000030a=d0000000@0`<GN00<000000<35g@0000000P31aMd2000000030<75g@000000
00000080`LGM0P0000001031aMd0000000000032aMd200000080`/GM00@000000<;6g@0000000000
1032aMd2000000D0`/GL00<000000<?5g033aM`00P33aM`00`000000`/GL000000040<?5g0040000
0033aM`0000000000080`lGK0P0000000`33aM/000000<?5f`02000000<0`lCK00@000000<?4fP00
000000000`33a=X00`000000a<?J0<C3fP040<C3fP0300000034`mT0a<?I00<0a<?I00<000000<C2
f@34`]T00`34`]T00`000000`l;H000000030<C1f00400000033`MP0000000000080a<7G0P000000
0`33`=L000000<C0e`0200000080`koF00@000000<BoeP00000000000`33_]H010000000`kkE0000
000000020<>me00400000033_M@00000000000<0`kcD00@000000<>ld`00000000000`32^m<20000
00<0`[WB00@000000<>id@00000000000P32^=4010000000`[S@0000000000030<6fc`0400000032
]/l0000000000080`KC>00@000000<6dc@00000000000P30/ld01`000000`;;<0<2bc030//`00000
0;n`c00000000`2o[lX00`000000_Zk:000000050;j]b0030000002lZ/L0_:[70080_:[700@00000
0;fXa@00000000000P2jY/D00`000000^ZC30;ZT``020;ZT``030000002fWl40]Yo10080]Yo100<0
00000;FL_`2eW;l00P2eW;l2000000<0[YJo00D000000:VA_@2YTKd0ZI6m000000040:::^`040000
002FPKX0UX6j09J1^P80000000<0fLg>0000003GcLl00`3GcLl00`000000e/gA000000030=G;d003
0000003Dbm40e<_A0080e<_A00<000000=;:dP3Bb]800P3Bb]801P000000d/_B0=;;dP000000dLWB
000000D0dL[C00<000000=3:e03@b]@00P3@b]@01P000000d<_D0=3;e0000000d<[D000000D0d<_E
00<000000<o;eP3?bmH00`3?bmH01@000000d<cF0000003?c=L0000000D0clgG00<000000<o=f03?
cMP00P3?cMP2000000040<o>f0000000clkI000000D0cloI00@000000<oAfP3?dMX0cm7J0P000002
0=3Bf`030000003?d]/0000000<0cm?L00D000000=3Dg@00000000000<oDg@02000000<0cmKN00@0
00000<oGg`3?eml0cmOO0P0000030<oIh0030000003>f^80c][R0080c][R00@000000<kLh`000000
cM_S0P0000030<gLi0040000003=g^D000000<_MiP8000000`3;g^L010000000bMkX0<WNj039g^P2
00000080b=oY0P0000030<KNjP80000000<0aMo[00000035gN/00P0000030<CNk00400000032gNd0
`]g]0<;Mk@80000000<0`Mg]00000031fnd00P0000030;oKkP<000000P2nf^h2000000040;gJk`00
0000_=W^0;cIkP80000000@0^mS_0000002kenh0^mO^0P000000102jenh000000000000000040;WE
kP800000102he>h3000000<0]m;^0`0000040;KAkP800000102fd>d2000000@0]Lk]0P0000030;C=
k@80000000<0]<c/0000000000000P2cc>`00`000000/l_/0;?;k002000000030;;;k0000000//W[
0080//W[0`0000040;78j`800000102aan/00`000000/LKZ0;76jP02000000030;36jP000000/<GZ
0080/<GZ0`0000040:o4j@030000003oool0oooo00T0oooo00<000000?ooo`3oool0803oool002d0
oooo00<000000?ooo`3oool0I`3oool2000000030;o4gP000000000000<0_lCM00@000000<34gP00
000000000`30a=d010000000`<GN0000000000030<35g@8000001@31aMd2000000D0`LGM00<00000
0<;5g@32aMd00`32aMd00`000000`/GM0<;5g@030<;5g@0400000032aM`0000000000080`/GL0P00
00000`32aM`000000<?5g002000000<0`/GL00@000000<?5g000000000001033aM`2000000@0`lGK
00<000000<?4f`33a=/01033a=/00`000000`lCJ0<?4fP020<?4fP0300000033`mX0000000@0a<?J
00<000000<?3f@0000000P34`mT2000000040<?2f@00000000000<C2f@8000000P33`]P010000000
a<7H0000000000040<?1f08000001033`=L00`000000`koF0<>oeP030<>oeP0300000033_]H0`kkF
00@0`kkF00<000000<>me033_M@00`33_M@00`000000`kcD0<>le0040<>le00300000032^m<0`[_C
0080`[_C00<000000<:jdP0000001032^M800`000000`[SA0<:hd@030<:hd@0300000031]/l0`KK?
00@0`KK?00<000000<6dcP31]<h00`31]<h00`000000`;?=0<2cc@020<2cc@0400000030//`00000
00000080_k3<00@000000;n_bP00000000000P2n[/X00`000000_Zg80;j]b0020;j]b08000001@2l
Z/L00`000000^ZK50;ZVa@020;ZVa@040000002jY<<0^ZC30;ZT``8000001@2fWl400`000000]Ibo
00000002000000@0[YJo00<000000:VA_@2YTKd00P2YTKd00`000000XXZk0:::^`08000000@0elg?
00L000000=K=d@3FcM40e/gA0000003Ebm00000000@0e<_A00<000000=?<dP0000000P3Bb]800`00
0000d/_B0=;;dP020=;;dP030000003Ab]<0dL[C00<0dL[C00<000000=7;e00000000`3@b]@01@00
0000d<_D0=3;e03@bm@0000000D0d<_E00<000000=3<eP0000000`3?bmH01@000000d<cF0=3<eP3@
c=H0000000D0clgG00@000000<o=f0000000clgH0P0000040<o>f0030000003?cmT0cloI00<0cloI
00D000000=3@fP000000cm7J000000050=3Bf`050000003?dm`0cm?L0<oCg0000000103@e=d00`00
0000cmKN0<oFgP020<oFgP050000003@f=l000000<oGg`000000103?fN0010000000cm_Q0000003>
f^82000000<0c]cS00@000000<gLi03=g>@0cMcT0P0000030<gNi@030000003;g^L0bmkW0080bmkW
00<000000<[Oj00000000P0000030<SOj@<0000000<0a]kZ0000000000000`35gn/2000000<0a=k/
0P0000001032g^d000000000000000040<7Mk@040000002ofnh0_m_^0;oKkP80000000<0_]c_0000
002nf^h00P0000030;gJk`<00000102kf>l2000000<0^]O^00<000000;WGk`0000000P2ieNh20000
00040;SEk`000000^=C^0;SDkP80000000@0]m?^0000002gd^h0]m;^0P0000020;KBkP030000002f
dNh0]]7^0080000000@0]]7^0000002fd>d0]]3]0P000000102dcnh000000;G>k@2ec^d200000003
0;C>k@000000]<g]0080]<g]0`0000040;?<k0<000000`2bbn`400000080/LW/00<000000;78j`2a
b>/00P0000000`2ab>/000000;77j`020;77j`@000000`2`a^X400000080[lGZ00<000000:o4j@2_
a>T00P0000000`2^a>T000000?ooo`090?ooo`030000003oool0oooo0200oooo000^0?ooo`030000
003oool0oooo06P0oooo00<000000;o4g@2oa=d0102oa=d00`000000`<CM0<34g@030<34g@800000
1030aMd00`000000`LGM000000040<75g@0400000031aMd0000000000080`LGM0P0000001032aMd0
00000<;5g@32aMd2000000040<;5g@00000000000<;5g@8000001032aM`2000000@0`/GL00<00000
0<;5g032aM`01032aM`00`000000`lGL0<?5g0030<?5g00300000033aM/0000000<0`lGK00@00000
0<?4f`00000000000P33a=/2000000030<?4fP000000`lCJ008000000`33`mX010000000a<?J0000
000000030<?3f@8000001@33`]T00`000000`l;H0<?2f0030<?2f00300000033`MP0`l7H00<0`l7H
00<000000<?0e`0000000`33`=L00`000000`koG000000030<>oeP0300000033_]H0000000@0`kkF
00<000000<>me@0000000P33_M@2000000030<:le000000000000080`kcD0P0000000`32^m<00000
0<:kd`0200000080`[[B00<000000<:idP32^M800P0000000`31^M4000000<:hd@020<:hd@030000
0031]m00000000@0`KK?00<000000<2ec`0000000`31]<h00`000000`;?>000000040<2cc@030000
002o/<`0_k3<00<0_k3<00<000000;j^bP2n[/X00`2n[/X010000000_Zg80000000000020;b[b004
0000002lZ/L0_:[70;bZa`8000001@2jY/D300000080]j;400<000000;JO`@2fWl400P2fWl420000
00<0/9[100D000000:jF_`2^U[l0[YJo000000040:VA_@<0000000<0WH^o0000002CQ;l00P2CQ;l0
10000000QW_00000000000030=O=c`030000003FcM40e/gA0080e/gA00@000000=C;d@3Dbm40e<_A
0P0000020=?<dP030000003Bb]80000000@0d/_B00<000000=;;d`0000000`3Ab]<01`000000dL_D
0=7;e03Abm@000000=3:e00000000`3@bm@00`000000d<_E000000030=3;e@070000003@c=H0d<cF
0=3<eP000000cl_F000000030=3<eP030000003@cML0000000<0clgG00D000000<o=f03?cMP0clgH
000000050<o>f0030000003@cmT000000080cloI0P0000030=3@fP030000003@d]/0d=;K0080d=;K
0P000000103@dm/000000<oCg00000040=3Dg@040000003@e]d000000<oFgP8000000`3@f=l01000
0000cmWP0<oIh03?fN02000000<0cm_Q00<000000<kLh`3>g><00P3>g><01@000000c]gT0000003=
g>@0000000<0cMkU0P0000000`3<gnL000000<_Ni`0200000080b]oX00@000000<WQj@000000b=oY
0P0000030<OPj`0400000035gn/0aMo[0<GOj`8000000P34h>`3000000<0`]k]00<000000<7OkP00
00000P31gNd2000000040;oMkP0000000000000000@0_]c_00@000000;gJk`2mf^l0_M[_0P000002
0;_Jk`030000002kf>l0^mS_0080000000<0^]S_0000002jenh00P0000030;WGk`800000102heNl2
000000@0]m?^0P0000040;KBkP<000000`2fdNh3000000<0]<o^0`0000040;C>k@<0000000<0/lg]
0000000000000P2cc>`2000000040;;;k@000000//_/0;;;k080000000<0/L[/0000002abN`00`2a
bN`3000000@0/LS[0`0000020;37j`030000002`a^X0/<KZ0080000000<0[lK[0000002_aNX00`2_
aNX3000000<0[/CY00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0P0?ooo`00;`3oool0
0`000000oooo0?ooo`1X0?ooo`8000000P2oa=d2000000040<34g@000000`<CM0<34g@8000000P30
a=d010000000`<GM0000000000030<75g@0400000031aMd00000000000@0`LGM0P0000040<;5g@80
00001@32aMd00`000000`/GL0<;5g0030<;5g00300000032aM`0000000<0`/GL00@000000<;5g000
000000000P32aM`2000000040<?5g000000000000<?5g08000000P33aM/010000000`lGK00000000
00040<?4f`8000001033a=X00`000000`l?J0<?3fP040<?3fP0300000033`mT0`l?I0080`l?I00<0
00000<?2f@0000001033`]T00`000000`l;I000000020<?2f080000000@0`l7H000000000000`l7H
0P0000020<?0e`0400000033`=L0000000000080`koG00@000000<>oeP00000000000`33_]H01000
0000`kkF0000000000030<>me@8000001@32_=@2000000@0`[_C00<000000<:jdP32^]800`32^]82
000000<0`KWA00@000000<:hd@00000000000`31]m0010000000`KK?0000000000020<2ec`040000
0031]<h0000000000080`;?>00<000000<2cc@30/ld00P0000000`2o/Ld000000;n`c0020;n`c003
0000002n[l/0000000D0_Zk:00<000000;b[b02lZlP00P2lZlP010000000_:[70000000000020;ZX
a`030000002jY/D0^ZK50080^ZK500<000000;NRa02gX/@00P2gX/@010000000]Yo10;JO`@2fWl42
000000@0/9[100<000000:jF_`2^U[l00P2^U[l00`000000ZI6m00000002000000@0WH^o00D00000
09>4_`2CQ;l0ThBo0000000208Ik`0030000003GcLl0000000@0e/gA00D000000=G=dP000000e<_A
000000050=?<dP030000003Bbm80d/_B0080d/_B00L000000=;;d`3Bbm<0d/_C0000003Ab]<00000
00D0dL_D00D000000=3;e03@bm@0d<_D000000030=3;e@030000003@bmD0000000D0d<cF00D00000
0=3<eP3@c=H0d<cF000000030=3=e`030000003?cML0000000<0clgH00<000000=3>f00000000`3?
c]P01P000000d<oI0=3?f@000000cloI000000@0d=3J00@000000=3BfP000000d=;K0P0000040=3C
f`040000003@e=d0d=CM0=3Dg@8000000`3@e]d00`000000d=SO0=3Hg`020=3Hg`050000003@fN00
00000<oIh0000000103?fn4010000000cmcR0000003>g><2000000<0c]gT00<000000<gNi@3=g^D0
0P0000040<cOi`030000003:gnP0b]oX008000000`39hNT00`000000an3[0<OPj`020<OPj`030000
0036hN`00000008000000`34h>`2000000030<?PkP000000`]k]008000000`31gnh2000000<0_mg^
00<000000;kMk`0000000P2ng>l2000000040;gKk`0000000000000000@0^m[_0`0000030;[Hk`80
00000`2ienl2000000040;SFk`000000^=G_0;SEk`80000000@0]mG_0000002gdnh0]m?^0P000000
102fdnl000000;KBkP2fd^h2000000040;GBk`000000]]7^0;KAkP80000000@0]=3^0000002dcnh0
]<o^0P0000020;C?kP030000002dc^d0]<k]0080000000<0/lk^0000002ccNd00P2ccNd3000000@0
//_]0P0000040;7:k0040000002abN`0000000000080/LW/00<000000;78j`2ab>/00P0000000`2`
b>`000000;37j`030;37j`800000102_a^/01P000000[lGZ000000000000[/CZ000000<0[/CY0`00
000:0?ooo`030000003oool0oooo01l0oooo000`0?ooo`800000JP3oool2000000@0`<CM0P000005
0<34g@0300000031aMd0`LGM00@0`LGM00<000000<75g@31aMd00`31aMd00`000000`LGM00000003
0<;5g@0400000032aMd0000000000080`/GM0P0000001032aM`0000000000032aM`200000080`/GL
00@000000<;5g000000000001032aM`2000000D0`lGL00<000000<?5f`33aM/00`33aM/00`000000
`lCK0<?4f`030<?4f`0300000033a=/0000000<0`lCJ00@000000<?3fP00000000000P33`mX20000
00030<?3fP000000`l?I008000000`33`]T010000000`l;I0000000000030<?2f@8000001@33`MP0
0`000000`l3G0<?0e`030<?0e`0300000033_mL0`koG00<0`koG00<000000<>neP33_]H01033_]H0
0`000000`kgE0<>me@020<>me@0300000032_MD0000000@0`[cD00<000000<:le00000000`32^m<0
0`000000`K[C000000030<:jdP0300000031^M80000000@0`KWA00<000000<6gd031]m001031]m00
0`000000`;G?0<2ec`030<2ec`0300000030/lh0`;?>00<0`;?>0P0000030;nac@040000002o/<`0
000000000080_Zo;00<000000;j^bP2n[/X00P2n[/X2000000D0_:_800<000000;ZXa`2jZ<L00P2j
Z<L00`000000^ZK50;ZVa@03000000D0]j;40`0000020;:N``030000002`V/40/9[10080/9[100<0
00000:jF_`2^U[l00`0000030:FA`0050000002MRkl0WH^o09f;_`0000000`2CQ;l01@000000QW_0
08Ik`026Nl00000000<0e/gA0P0000030=G=dP030000003Cc=80dlcB00<0dlcB00<000000=?<d`00
00000`3Bbm800`000000d/_C0=;;d`020=;;d`030000003Abm@0dL_D00<0dL_D00D000000=7;e000
0000d<_D000000050=3;e@030000003@c=H0d<cF00<0d<cF00D000000=3<eP000000d<cF00000005
0=3=e`050000003?cMP0clgH0<o=f00000000`3@c]P00`000000clkH000000040=3?f@030000003@
d=X0d=3J0080d=3J00D000000=3BfP3@d]X0d=;J000000050=3Cf`050000003Ae=`000000=3Dg@00
0000103@e]d010000000dMSN0000003@f=l2000000<0d=WP00@000000<oKh@3?fn40cm_Q0P000003
0<oLhP030000003>gN@0c]gT0080c]gT00@000000<kOi@0000000000103<gnL2000000030<_Qj000
0000000000@0bN7Y00@000000<SRj`000000an3[0P0000020<KQk00400000035h^d000000<CPk080
00001033h>h00`000000`Mo^0<7OkP020<7OkP0400000030gnl000000;oMkP8000000`2ngNl20000
00<0_M__00<000000;_Kl00000000P2kf^l2000000040;[Jl0000000^]S_0;[Hk`8000000P2if?03
000000<0^=K_0`0000040;OEk`800000102fdnl2000000@0]M;_0P0000040;C@kP800000102dcnh3
000000<0/lk^1@000000102bc>d000000;;;k@2bbnd2000000030;7;k@000000/L[/0080/L[/0`00
00040;79k0<000000`2`b>`00`000000/<O[0;37j`02000000030:o6j`000000[lK[0080[lK[0`00
00040:k4jP<000000`2^`nX00`000000oooo0?ooo`070?ooo`030000003oool0oooo01l0oooo000b
0?ooo`030000003oool0oooo06P0oooo00L000000<34g@30a=d0`<CM00000030a=d0000000@0`<CM
00@000000<74g@00000000000P31aMd2000000040<75g@00000000000<75g@8000000P31aMd01000
0000`/GM0000000000040<;5g@8000001@32aM`00`000000`/GL0<;5g0030<;5g00300000032aM`0
`/GL00<0`/GL00<000000<;5g00000001033aM`00`000000`/CK000000020<?5f`80000000@0`lCK
000000000000`lCK0P0000020<?4f`0400000033a=X00000000000@0`l?J0P0000040<?3fP030000
0033`]T0`l;I00@0`l;I00<000000<?2f@33`]T00P33`]T00`000000`l7H000000040<?1f0030000
0033`MP0000000<0`l3G00<000000<?0e`0000000`33_mL010000000`[oG0000000000020<>neP80
000000<0`[kF00000033_MD00P0000030<:me@0400000032_=@0000000000080`[cD00@000000<:k
d`00000000000P31^]<010000000`[[B0000000000020<6idP0300000031^M40`KWA0080000000<0
`KSA00000031]m000`31]m000`000000`;K@000000030<2ec`030000002o]<l0000000<0`;?>0P00
00050;nac@030000002n[l/0_Zo;00<0_Zo;00@000000;j^bP00000000000P2l[LX010000000_:_8
0;b[b02lZlP2000000D0^ZS70P0000030;RUaP030000002gX/@0]j;40080]j;400<000000;:N``2b
W/<00P2bW/<00`000000/9[10;2J`@020;2J`@<0000000<0ZIK20000002UTL000`2UTL001@000000
WH^o09f;_`2MRkl0000000<0ThBo00@0000008Ik`026Nl00QW_00P0000000`3FcM4000000=G=dP03
0=G=dP030000003DcM80000000<0dlcB00L000000=?<d`3Cc=<0dlcC0000003Bbm80000000@0d/_C
00<000000=7;e00000000P3Abm@2000000<0dL_D00<000000=3;e@3@bmD00`3@bmD00`000000d<cE
000000020=3<eP8000000`3@c=H00`000000d<gG0=3=e`030=3=e`050000003@cML000000<o=f000
00001@3@c]P00`000000d<oI0=3?f@020=3?f@060000003@d=T000000000003@d=X000000`3@d]X0
0`000000dM?K000000020=3Cf`8000000`3Ae=`010000000d=KM0=3Fg@3@e]d2000000<0dMSN00<0
00000=3Ih03@fN000P3@fN001@000000d=_Q0000003?fn40000000@0cmcR00@000000<oNi0000000
c]gT0P0000020<kOi@040000003=hNH000000<cOi`8000000P3;hNP2000000030<[Rj@000000bN7Y
008000000`38h^/00`000000a^7/0<KQk002000000<0aN;]00<000000<?PkP33h>h00P33h>h20000
00030<;Qk`000000`Mo^008000000P30gnl2000000<0_]g_0P0000000`2mgO0000000;gKk`020000
00<0^m_`0P0000040;[Jl08000000P2if?02000000040;SHl0000000^=K_0;SFk`8000000P2ge_00
0`000000]mG_0;OEk`02000000040;KDl0000000]]?_0;KCk`80000000@0]M?_0000002ed^l0]M;_
0P000000102ddNl000000;C@kP2dd>h2000000040;?@k`000000]<o^0;C?kP80000000@0/lo^0000
002cc^h0/lk^0P0000020;;=kP050000002bc>d0//c]0000002bc>d00P0000040;7;k@<000000P2a
b^d00`000000/LW/0;79k002000000030;38k0000000/<S/0080/<S/100000030:o6j`@000000P2^
aN/00`000000[/CZ0:k4jP02000000030:k4j`000000[/?Z00<0[/?Z0P0000080?ooo`030000003o
ool0oooo01l0oooo000c0?ooo`030000003oool0oooo06L0oooo00@000000<34g@00000000000`30
a=d010000000`<CM0000000000040<74g@8000001@31aMd00`000000`LGM0<75g@030<75g@030000
0032aMd0`/GM00<0`/GM00<000000<;5g00000001032aM`00`000000`/GL000000020<;5g0800000
00@0`/GL000000000000`/GL0P0000030<;5g00400000033aM`00000000000<0`/CK0P0000050<?4
f`0300000033a=/0`lCK00<0`lCK00<000000<?3fP33`mX00`33`mX00`000000`l?J000000030<?3
fP0400000033`mX0000000000080`l;I0P0000000`33`]T000000<?2f@02000000<0`l7H00@00000
0<?1f000000000000P33`MP010000000`l3G0000000000020<?0e`0400000033_mL00000000000@0
`[oG0P0000040<:neP0300000032_MD0`[gE00@0`[gE00<000000<:le032_=@00`32_=@00`000000
`K[C0<6jd`030<6jd`0300000031^M80`KWB00<0`KWB0P0000040<6hd@0400000031]m0000000000
0080`;K@00@000000<2ec`00000000000P2o]<l010000000`;?>0000000000020;jbcP070000002o
/Ld0_k7=0;nac@000000_K3=000000050;j_b`030000002l[LX0_:g:0080_:g:00@000000;b[b000
000000000P2jZ/T010000000^ZS70;ZXa`2jZ<L2000000@0^:G600<000000;NRa02gX/@00`000005
0;:N``030000002`V/40000000800000102YU/801@000000YI700:FA`02UTL00000000@0WH^o00D0
000009>4_`2CQ;l0ThBo0000000208Ik`0030000001gM<80000000@0eLgB00L000000=C=dP3DcM80
e<gB0000003Cc=80000000D0dlcC00@000000=;;d`3Bbm<0d/_C0P0000020=7;e0030000003Abm@0
000000@0dL_D00<000000=7<e@0000000P3@bmD200000080d<cE00<000000=3<eP000000103@c=H0
0`000000d<gF000000020=3=e`8000000`3@cML00`000000d<kH0=3>f0030=3>f0040000003@cmP0
00000=3?f@800000103@d=T01@000000d=;J0=3BfP3@d]X000000080dM?K00<000000=3Cf`000000
103Ae=`01@000000dMKM0000003@e]d0000000@0dMSN00@000000=7Jg`000000d=WP0P0000030=3K
h@040000003?g>80cmcR0<oLhP8000000`3?g^@00`000000c]oU0<kOi@02000000<0cN7V00<00000
0<_Qj03;hNP00P0000040<[Rj@0300000038h^/0b>;[0080b>;[00@000000<OSk000000000001035
h^d010000000`n;^00000033h>h2000000<0`^7_0P0000020<3Ok`8000000P2ngo03000000@0_Mg`
00@000000;_Kl02kfo00^m_`0P000000102jfo4000000;[Jl02jf_02000000040;WIl00000000000
000000@0^=S`0P0000040;OFl0<000000`2fe?03000000@0]M?_0P0000040;CAk`800000102cd>l2
000000@0/lo^0P0000040;;=kP030000002bc>d0//c]0080000000@0/Lc^0000002abnd0/L_]0P00
00000`2`bnd000000;7:k@020;7:k@<00000102`b>`300000080[lO/00<000000:o6j`2_a^/00P00
00020:k6j`030000002^aN/0[/G[0080[/G[0P0000030:k4j`030000002^`nX0[/?Z0080000000<0
[L?Z0000003oool01`3oool00`000000oooo0?ooo`0O0?ooo`00=03oool00`000000oooo0?ooo`1W
0?ooo`0300000030a=d0`<CM00@0`<CM00<000000<74g@31a=d00`31a=d00`000000`LCM00000004
0<75g@0300000031aMd000000080`LGM0P0000001032aMd0000000000032aMd2000000<0`/GL00@0
00000<;5g000000000000`32aM`2000000D0`/GL00<000000<;5g032aM`01032aM`00`000000`/CK
0<;4f`020<;4f`0300000032a=/0000000@0`lCK00<000000<?4f`0000000P33a=/2000000040<?4
f`00000000000<?3fP8000000P33`mX010000000`l?J0000000000040<?3fP8000001033`]T00`00
0000`l7H0<?1f0040<?1f00300000033`MP0`l7H00<0`l7H00<000000<?0e`33`=L00`33`=L00`00
0000`[oG0<:oe`030<:oe`0300000032_]H0000000<0`[kF00<000000<:meP0000001032_MD00`00
0000`[cE000000030<:le00300000031^m@0000000<0`K[C00<000000<6jd`0000000`31^M800`00
0000`;SB000000050<6hd@0300000030]]00`;K@00<0`;K@00<000000;ndc`2o]<l00`2o]<l00`00
0000_[;>0;jbcP020;jbcP040000002o/Ld0000000000080_K3=00<000000;j_b`2n[l/00P2n[l/0
0`000000_:k<000000040;b]bP030000002jZ/T0^Z[90080^Z[900@000000;ZXa`00000000000P2h
YlL00`000000^:G60;RUaP020;RUaP8000000`2dXLD00`000000/Yk30;:N``020;:N``8000000`2]
V/<01@000000ZIK20:VF`P2YU/80000000@0YI7000<0000009f;_`2MRkl01`00000208Ik`0030000
001gM<80000000<0eLgB00<000000=C=dP3DcM800P3DcM800`000000dlcC0=?<d`030=?<d`050000
003Bc=<000000=;;d`0000001@3Abm@010000000dL_D0=7;e03Abm@200000080dLcE00<000000=3;
e@0000001@3@c=D010000000d<cF0=3<eP3@c=H200000080d<gF00<000000=3=e`000000103@cML0
0`000000d<kG000000020=3>f08000000`3@cmP00`000000d=3I0=3@f@030=3@f@050000003AdMX0
00000=3BfP000000103Adm/00`000000dMCL0=7Dg0020=7Dg0050000003Ae]d0dMKM0=7Fg@000000
0`3Af=h2000000<0dM[O00<000000=3Kh@3@fn400P3@fn401@000000d=gR0000003?g>80000000@0
cmkT00@000000<oPi@0000000000103=hNH010000000c>?X0000000000040<[Rj@80000000<0bNC[
00000038h^/00P0000020<OSk00400000035i>d000000<GRk@8000000`33h^h010000000`^7_0<;Q
k`32hNl200000080`>7_0P0000030;kOl080000000@0_Moa0000002mgO00_Mg`0P000000102kgO40
00000000000000040;[Kl@8000000`2ifO000`000000^=Wa000000020;SHl080000000@0]mOa0000
002ge_00]mK`0P000000102fe_0000000;KDl02fe?0200000080]MC`00<000000;GCk`2ednl00P00
0000102dd_0000000;CAk`2ddNl2000000040;?Ak`000000/m3_0;?@k`80000000@0/m3_0000002c
cnh0/lo^0P0000000`2bc^l000000;;=kP020;;=kP@000000P2ac>h4000000<0/<_]100000020;39
k@030000002`b>`0/<S/0080000000<0[lS/0000002_an`00P2_an`3000000@0[/K[00<000000:k5
j`2^aN/00P0000000`2^aN/000000:k4j`020:k4j`<00000102]`nX00`000000oooo0?ooo`050?oo
o`030000003oool0oooo01l0oooo000e0?ooo`800000J03oool200000080`<CM0P0000001031a=d0
000000000031a=d2000000<0`LCM00@000000<75g@00000000000`31aMd2000000D0`/GM00<00000
0<;5g032aM`01032aM`00`000000`/GL0<;5g0020<;5g00300000032aM`0000000@0`/GL00@00000
0<;5g000000000000P32aM`2000000030<;4g0000000`/CK008000000`32a=/010000000`lCK0000
000000030<?4f`8000001@33a=/00`000000`l?J0<?3fP030<?3fP0300000033`mX0`l?J00<0`l?J
00<000000<?2f@0000000`33`]T00`000000`l;I000000040<?1f00300000032`MP000000080`l7H
0P0000001032`=P000000<?0e`33`=L2000000040<:oe`00000000000<:oe`8000000P32_]H01000
0000`[kF0000000000030<:meP0400000032_MD0000000000080`[cE00@000000<:le00000000000
0P31^m@010000000`K[C0000000000030<6jd`0300000031^M80000000<0`;SB00<000000<6hd@31
^=400P0000000`30]m4000000<2fd0020<2fd0030000002o]M00000000<0_kC?0P0000050;jbcP03
0000002m/<d0_K3=00<0_K3=00@000000;j_b`00000000000P2l[/`00`000000_:g:0;b]bP030000
00D0^Z[900<000000;RWa`2hYlL00P2hYlL010000000^:G60;RUaP2hYLH2000000@0]:7500<00000
0;:N``2bW/<00`0000040:fJ``030000002YU/80ZIK20080ZIK200<000000:FA`02UTL0010000000
0`2KSL@0000009:7a00209:7a00300000028PLD0000000<0000000@0MgC20000003EcM800000103D
cM800`000000dlgC000000020=?<d`8000000`3Bc=<00`000000dL_D0=7;e0030=7;e0050000003A
c=@000000=7;e00000001@3Ac=D00`000000d<cE0=3<e@030=3<e@050000003@c=H000000=3<eP00
00001@3@cMH010000000d<gG0=3=e`3@cML200000080d<kG00<000000=3>f0000000103@cmP00`00
0000dM3I000000020=3@f@8000000`3AdMX00`000000dM?K0=7Cf`020=7Cf`060000003Ae=/00000
0000003Ae=`000000`3Ae]d01@000000d]SM0000003Af=h0000000@0dM[O00@000000=7Kh0000000
d=_Q0P0000030=3MhP8000000P3?g^@200000080cn3U00@000000<kRiP000000cN7V0P0000020<cS
j0040000003<i>T000000<[Rj@8000000`39i>/200000080an?/0P0000030<GTk@0400000033h^h0
`n;^0<?RkP80000000@0`^?_00000000000000000`30hNl010000000_^7`0000002ngo02000000<0
_Moa0`0000020;_Ml@80000000@0^]ga0000002jfo40^]_a0P0000000`2ifo4000000;WIl0020000
00<0^=Wa0P0000040;OGl@800000102fe_02000000@0]MC`0`0000030;CBl0<000000`2cdNl30000
00@0/m3_0P0000040;;>k`<000000P2acNh00`000000/Lc^0;7<kP02000000040;3<kP000000/<_]
0;3;k@80000000<0/<[]0000002`bNd00P2`bNd4000000@0[lS/0`0000020:k7k0040000002^a^/0
[/K[0:k6j`@000000`2^aN/400000080[LC[00@000000:g3jP2]`nX0[L?Z0P0000070?ooo`030000
003oool0oooo01l0oooo000g0?ooo`030000003oool0oooo06L0oooo0P0000050<74g@0300000031
a=d0`LCM00@0`LCM00<000000<75g@31aMd00P31aMd00`000000`LCL000000040<;5g@0400000032
aM`0000000000080`/GL0P0000000`32aM`000000<;5g002000000<0`/GL00@000000<;5g0000000
00001032aM`2000000@0`/CL00<000000<;4f`32a=/01032a=/00`000000`lCK0<?4f`020<?4f`03
00000032a=/0000000@0`lCK00<000000<;3fP0000000P33`mX2000000040<?3fP00000000000<?3
fP8000000P33`]T010000000`l;I0000000000030<?2f@0400000033`MP00000000000<0`/7H0P00
00040<;0f08000001@32_mL00`000000`[kF0<:neP030<:neP0300000032_MH0`[gF00@0`[gF00<0
00000<:le@32_=D00`32_=D00`000000`K_D0<6ke0030<6ke00300000031^]<0`K[C00@0`K[C00<0
00000<2hdP30^=800`30^=82000000<0`;OA00@000000<2fd000000000000P2o]M0010000000_kC?
0000000000020;jcc`070000002n//h0_[;>0;jbcP000000_K7>000000050;f`c@030000002l[/`0
_:k<00<0_:k<0P0000030;^[bP040000002jZ/T0^Z[90;ZZb@8000001@2hYlL300000080]J?700<0
00000;BQa@2dXLD00P2dXLD2000000<0[ig500D000000:fJ``2]V/<0[I[3000000040:VF`P<00000
00<0XI;40000002KSL@00`2KSL@01@000000TXO409:7a02BQl@000000080R87500<0000007ala`00
00000P0000040=C=dP060000003CcM<0dlgC0000003Cc=<00000103Bc=<00`000000d/cD00000002
0=7;e08000000`3Ac=@00`000000dLcE0=7<e@030=7<e@030000003Ac=D0000000<0d<cE00D00000
0=3<eP3@c=H0d<cF000000050=3=eP050000003@cML000000=3=e`0000001@3@c]L010000000d<oH
0=3?f03@cmP200000080dM3I00<000000=3@f@000000103AdMX010000000dM?J0000003Adm/20000
00@0dMCK00D000000=7Fg@3Ae]d0dMKM000000030=;Hg@040000003Af]l0dM[O0=7Jg`8000000`3A
fn000`000000d=gR0=3MhP020=3MhP030000003@gn<0d=oS008000000`3?h>D01@000000c^;V0<kR
iP3>h^H0000000<0c>?X00D000000<cTj@3<i>T0c>CY000000030<WTj`8000000P38iN`2000000<0
aNC]0P0000001034i>h000000000000000030<;Sk`0400000030ho0000000<3Qk`8000000`2nhO00
10000000_Moa0;gOl@2mgo42000000030;_Nl@000000^mga008000000`2jgO43000000<0^M_a0P00
00030;SIl@80000000@0]mSa0000002geo40]mOa0P000000102feo4000000;KFl02fe_0200000004
0;GEl@000000]MC`0;GDl080000000@0]=?`0000002dd_00]=;`0P000000102cd_0000000;?Ak`2c
dNl200000080/]3`00<000000;?@k`2cd>l00P000000102bcnl000000;;>k`2bc^l2000000030;7>
k`000000/Lg^0080/Lg^0`0000020;3<kP040000002`c>h00000000000@0/<[]0`0000030:o9k@03
0000002_b>`0[lS/0080000000<0[/S]0000002^an`00`2^an`300000080[/K/00<000000:k5j`2^
aN/00P0000000`2]aN/000000:g4j`030:g4j`@000000P2/`nX00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo01l0oooo000h0?ooo`030000003oool0oooo06L0oooo0P0000020<74g@800000
00<0`LCM0000000000000P31a=d2000000030<74g@000000`LGM008000000`31a=`010000000`/GM
0000000000040<;5g08000001032aM`00`000000`/GL0<;5g0040<;5g00300000032aM`0`/GL00<0
`/GL00<000000<;4g00000000`32a=`010000000`/CK0000000000020<;4f`80000000<0`/CK0000
0033a=/00P0000030<;4f`0400000033a=/00000000000<0`/?J0P0000050<?3fP0300000033`]T0
`l;I00<0`l;I00<000000<?2f@33`]T01033`]T00`000000`/7H0<;1f0020<;1f00300000032`=P0
000000<0`/3H00<000000<:of00000001032_mL00`000000`[kG000000030<:neP0300000032_MH0
000000@0`[gF00<000000<6leP0000000`32_=D00`000000`K_D000000030<6ke00300000031^]@0
000000@0`K[C00<000000<2id`0000000`30^=800`000000_kOB000000040<2gd@030000002o]M00
_kG@00<0_kG@00<000000;jcc`2n/ll00P2n/ll010000000_[;>0000000000030;facP030000002m
/<d0_K3=0080000000<0_:o=0000002l[/`00P2l[/`2000000@0^j_:00@000000;ZZb@0000000000
0P2iZLT00`000000^:O70;RWa`020;RWa`030000002eXlL0]J?70080]J?700@000000;BQa@2dXLD0
]:750P0000040:nMa@030000002]V/<0[I[30080[I[300<000000:VF`P0000000P0000040:6Ba005
0000002KSL@0Vhg409^=a00000000`2BQl@01@000000R87508R1a@28PLD0000000<0O7c700@00000
0=C=dP3DcM800000103CcM<00`000000d/cC0=;<d`020=;<d`060000003Bc=@0d/cD0000003Abm@0
0000103Ac=@00`000000dLcE000000030=7<e@070000003Ac=D0dLcE0=7<e@000000d<cE00000003
0=3<eP030000003@cMH0000000<0d<gF00D000000=3=e`3@cML0d<gG000000050=3>e`050000003A
cmP000000=3?f00000001@3Ad=T010000000dM7J0=7AfP3AdMX2000000<0dM?J00<000000=7Df`3A
e=/00`3Ae=/01@000000d]KL0000003Ae]d0000000<0d]SM00D000000=;JgP000000dM[O00000004
0=7Kh0040000003Bg^4000000=3MhP8000000`3@gn<01@000000d>7T0000003?h>D0000000<0c^;V
00D000000<kTi`000000c>?X000000030<cTj@030000003:i^/00000008000000`38iN`200000004
0<KVk@000000000000000080a>C^0P0000000`32iO0000000<;Sk`02000000<0`>?`00@000000;kQ
l02nhO00_^7`0P000000102mhO4000000000000000040;_Nl@040000002jgO40^]ga0;[Ml@800000
00@0^Mcb0000002ifo40^M_a0P0000020;SJlP<000000`2gf?43000000@0]]Oa0P0000030;GEl@<0
00000`2ddo03000000<0/m;`0`0000040;;@l0<000000`2bcnl3000000<0/Lk_100000020;3=kP03
0000002`c>h0/<c^0080000000@0/<_^0000002`b^d0/<[]0P0000000`2_b^h000000:o9k@030:o9
k@<000000`2^b>d00`000000[/O/0:k7k002000000030:g7k0000000[/K/0080[/K/0`0000040:g5
j`060000002]a>/000000000002/`n/000000`2/`nX3000000H0oooo00<000000?ooo`3oool07P3o
ool003T0oooo00<000000?ooo`3oool0J03oool2000000D0`LCM0P0000040<74g@0300000031a=`0
`LCL00@0`LCL00<000000<;5g032aM`00`32aM`00`000000`/GL000000030<;5g00400000032aM`0
000000000080`/GL0P0000001032aM`0000000000032aM`200000080`/CL00@000000<;4g0000000
00001032a=/2000000@0`/CK00<000000<;4f`32a=/01032a=/00`000000`/?J0<;3fP020<;3fP03
00000032`mX0000000@0`l?J00<000000<;2fP0000000`33`]T010000000`/;I0000000000020<?2
f@80000000<0`/7I00000032`MP00P0000030<;0f00300000032`=P0000000<0`[oH00@000000<:o
e`00000000000P32_]L010000000`[kF0000000000030<:meP0400000032_MH0000000000080`KcF
00@000000<:le@00000000000P31^m@010000000`K_D0000000000030<6je00400000031^]<00000
00000080`;WC00@000000<2hdP00000000000P2o]m800`000000`;OA0<2gd@02000000030;nfd@00
0000_kG@0080_kG@00<000000;jdd0000000102n/ll00`000000_K7>0;facP040;facP8000000`2l
[ld010000000_:k<0000000000020;^]c0030000002kZlX0^j_:0080^j_:00<000000;VYb@2iZLT0
0P2iZLT00`000000^:O70;RWa`03000000D0]J?70`0000020;6PaP030000002_WLD0[ig50080[ig5
00<000000:fJ``2]V/<00`0000030:JFa@050000002QT/@0XI;40:6Ba00000000`2KSL@01@000000
TXO409:7a02BQl@0000000<0R87500L0000007ala`1lO<L0O7c70000003DcM80000000@0dlgC00<0
00000=?=e00000000`3Bc=<01@000000d/cD0=;<e03Bc=@0000000@0dLcD00L000000=7<e@3Ac=D0
dLcE0000003Ac=D0000000D0dLcE00D000000=3<eP3@c=H0d<cF000000030=3=eP030000003@cMH0
000000<0d<gG00<000000=7>e`0000000`3@c]L01@000000dLoH0=7?f03AcmP0000000D0dM3I00D0
00000=7Af@000000dM7J000000040=7CfP030000003Be=X000000080dMCK0P0000030=;Fg0050000
003Bf=d0d]SM0=;Hg@0000000`3Bf]h00`000000dM_P0=7Kh0020=7Kh0050000003Bg^40d]kQ0=;N
h@0000000`3@gn<2000000<0d>7T00<000000<kRiP3>h^H00P0000030<kTi`030000003<i>T0c>CY
008000000P3:i^/010000000bNO/00000038iN`200000080a^K]0`0000000`34i>h0000000000003
0<;Ul08000000`30ho02000000040;kSl@000000000000000080_N7a0P000000102kh?8000000;_N
l@2kg_42000000040;[NlP0000000000000000@0^Mcb0P0000020;SJlP80000000@0]][b0000002g
f?40]mSa0P0000020;KHlP030000002feo40]]Oa0080000000<0]MKa0000000000000P0000020;CD
l@@000000P2cdo4400000080/]7`00<000000;;@l02bd?000P000000102bd?0000000;;?k`2bcnl2
000000040;7?k`000000/Lk_0;7>k`8000000P2`cNl010000000/<g^0;3=kP2`cNh300000080/<_^
00@000000;3;kP00000000000`2_b^h00`000000[lW]0:o9k@02000000030:k9k@000000[/S]0080
[/S]100000030:g7k0@000000P2]aN`010000000[LG[0:g5j`2]aN/3000000@0[<?[0`0000020:_2
j`030000003oool0oooo00@0oooo00<000000?ooo`3oool07P3oool003X0oooo0P00001Y0?ooo`03
00000031a=d0`LCM0080`LCM00<000000<74g00000000`31a=d010000000`LCL0000000000020<74
g080000000@0`LCL000000000000`/GL0P0000020<;5g00400000032aM`00000000000@0`/GL0P00
00050<;5g00300000032a=`0`/CL00<0`/CL00<000000<;4f`32a=/00`32a=/00`000000`/CK0000
00030<;4f`0400000032a=/0000000000080`/CK0P0000000`32`m/000000<;3fP02000000<0`/?J
00@000000<?3fP00000000000P32`]X010000000`l;I0000000000040<;2f@8000001032`MT00`00
0000`/3H0<;0f0040<;0f00300000032_mP0`[oH00<0`[oH00<000000<:ne`32_]L00`32_]L00`00
0000`[gF0<:meP040<:meP0300000031_=H0`KcF00<0`KcF00<000000<6ke031^m@00`31^m@00`00
0000`K[D0<6je0040<6je00300000030^M<0`;WC00<0`;WC00<000000;ngdP2o]m800`2o]m820000
00<0_kKA00@000000;ned000000000000`2n]=001P000000_[??000000000000_K;?000000@0_K7>
0P0000050;b_c@030000002k[L`0^jg<0080^jg<00<000000;^[bP2kZlX00`0000050;VYb@800000
0`2fY/P00`000000]J?70;FSa`020;FSa`030000002aX<H0/J360080/J3600<000000:nMa@2_WLD0
0P2_WLD3000000030:ZJaP000000YYK500<0YYK500D000000:6Ba02QT/@0XI;40000000409^=a005
0000002BQl@0TXO409:7a00000000`28PLD01@000000O7c707ala`1lO<L0000000@0dlgC00L00000
0=?=e03CcM@0dlgD0000003Bc=<0000000<0d/cD00<000000=;<e00000000`3Ac=@00`000000dLcE
0=7<e@020=7<e@030000003Ac=D0dLcE00<0dLcE00D000000=7<eP000000d<cF000000050=3=eP05
0000003@cML0d<gG0=3=e`0000000`3Ac]L00`000000d<kG000000030=7?f0030000003Ad=P00000
00<0dM3I00D000000=7Af@3AdMT0dM7I000000040=7CfP060000003Be=X0d]CJ0000003Ae=/00000
103Be]`01@000000d]SL0000003Bf=d0000000<0d][N00H000000=?Lg`00000000000=7Kh0000003
0=;Nh@050000003Bh>8000000=3Oh`0000000`3@hN@2000000030=3TiP000000000000@0c^CW00@0
00000<gVj@00000000000P3:i^/2000000<0bNO/00<000000<KVk@36i^d00P0000030<CVk`040000
0032iO00`^G`0<;Ul08000000P30iO4300000080_^?a0P000000102mho8000000000000000040;_P
lP8000000P2jg_82000000040;SNl`000000^Mcb0;WLlP80000000@0]mcb0000000000000000102f
f_82000000@0]]Sb0`0000000`2ee_400000000000020;?FlP030000002de?40]=Ca0080000000@0
/mCa0000002cdo40/m?a0P0000000`2bd_4000000;;Al0020;;Al0<000000`2bd?03000000@0/Lo_
0P0000040;3=k`@000000P2_c>l00`000000/<_^0;3;kP02000000030:o;kP000000[l[^0080[l[^
0`0000040:k9k@<000000P2]and00`000000[LO/0:g7k002000000030:g6k@000000[LG/00<0[LG/
100000020:c4k0030000002/`n/0[<?[0080000000<0Zl?[0000002[`^/00`2[`^/00`000000oooo
0?ooo`030?ooo`030000003oool0oooo01h0oooo000l0?ooo`030000003oool0oooo06L0oooo00@0
00000<74g@00000000000P31a=`010000000`LCM0000000000040<74g08000001@31a=`00`000000
`/GL0<;5g0030<;5g00300000032aM`0`/GL00<0`/GL00<000000<;4g00000000`32aM`200000004
0<;4g0000000`/CL0<;4g080000000@0`/CL000000000000`/CK0P0000020<;4f`0400000032a=/0
0000000000@0`/CK0P0000040<;3f`0300000032`mX0`/?J00@0`/?J00<000000<;2fP32`]X00`32
`]X00`000000`/;I0<;2f@030<;2f@0300000032`MT0000000<0`/7I00<000000<;1f@0000001032
`=P00`000000`/3H000000030<:of00300000032_mL000000080`[kG0P0000000`31_]L000000000
00020<:meP80000000@0`KgF00000031_=H0`KcF0P0000001031_=D000000<6ke031^m@200000003
0<2ke0000000`K[D00<0`K[D00<000000<2ie00000000`30^M<00`000000_kSC000000030;ngdP03
0000002o]m80000000@0_kKA00<000000;jdd02n]=00102n]=000`000000_K;?0;fbc`020;fbc`04
0000002m/Lh0000000000080_;3>00@000000;b_c@2l[ld0_:o=0P0000050;^]c08000000`2iZ//0
10000000^JW90;VYb@2iZLT2000000@0]ZK800<000000;FSa`2eXlL00`0000050;6PaP030000002_
WLD0000000800000102ZV/H00`000000YYK50:JFa@020:JFa@050000002QT/@0XI;40:6Ba0000000
0P2KSL@<0000000607ala`00000000000000003CcM<000001@3CcM@01@000000d/cD0=;<e03Bc=@0
000000<0d/cD00<000000=7<e0000000103Ac=D00`000000dLcE000000020=7<e@8000000`3Ac=H0
0`000000d<gF0=3=eP030=3=eP050000003AcMH000000=3=e`0000001@3Ac]L01@000000dLoH0=7?
f03AcmP0000000<0dM3H00<000000=7@f@0000000`3AdMT01P000000d];I000000000000dM?J0000
00@0d]CJ00@000000=;Fg03Be]`0d]KL0P0000030=;Hg0050000003Bf]h0d][N0=;JgP000000103C
g=l01@000000d]kQ0=;Nh@3Bg^40000000<0d^3R00<000000=3Qi03@hN@00P0000030=3TiP040000
003?i^L000000<kTi`8000000P3=i^T010000000bnS[0000000000040<WWk00400000037j>h00000
000000<0a>K_0P0000001033io0000000000000000020<3Ul@8000000P2niO8300000080_N?b0P00
0000102kh_<000000;_PlP2kh?82000000040;[Pl`0000000000000000@0^=kc0P0000020;OLlP80
000000@0]]_c0000002ff_80]][b0P000000102efO8000000;KHlP2ff?8200000080]=Ob0P000004
0;?FlP<000000`2ce?43000000<0/];a100000020;7Al0@000000P2ad?000`000000/Lo_0;7?k`02
000000040;3>l0000000/<g_0;3=k`80000000<0[lg_0000002_c>l00P2_c>l4000000030:o;kP00
0000[l_^00@000000P2^b^h00`000000[/W]0:k9k@02000000030:g8kP000000[LO]0080[LO]0`00
00040:g6k@060000002]aN`000000000002/aN`000000`2/a>`4000000<0Zl?[00<000000:_2j`2[
`^/00P0000000`2[`^/000000?ooo`030?ooo`030000003oool0oooo01h0oooo000m0?ooo`030000
003oool0oooo06L0oooo00<000000<74g031a=`00`31a=`00`000000`LCL0<74g0030<74g0030000
0031a=`0000000@0`LCL00<000000<74g00000000P32aM`2000000040<;4g000000000000<;5g080
00000P32a=`010000000`/GL0000000000040<;4g08000001@32a=`00`000000`/CK0<;4f`030<;4
f`0300000032a=/0`/CK00<0`/CK00<000000<;3f`0000000`32`m/00`000000`/?J000000040<;3
fP0300000032`]X000000080`/;J0P0000001032`]X0000000000032`]T200000080`/7I00@00000
0<;1f@00000000000`32`MT010000000`/3H0000000000020<;0f00400000032_mP00000000000<0
`[oG0P0000050<6ne`8000001031_MH2000000@0`KcE0P0000040<2ke00400000031^]@000000000
0080`;WD00@000000<2id`00000000000P2o^=<010000000_kOB0000000000020;ngdP030000002o
]]40_kKA0080000000<0_[GA0000002n]=000`2n]=02000000D0_K;?00<000000;b`cP2l/<h00P2l
/<h010000000_:o=0000000000020;^^c@040000002k[L`0^jg<0;^]c0800000102iZ//010000000
^JW90000000000020;NXbP030000002fY/P0]ZK80080]ZK80P0000030;>Sb0030000002aX<H0/J36
0080/J360P0000030:fMa`050000002ZV/H0ZY[60:ZJaP000000102VU/D700000080VHo700D00000
09:;b02BRlP0TX_80000000308V6b@0300000020P/`0000000@000001@3CcM@01@000000d/gD0000
003Bc=@0000000D0d/cD00@000000=7<e@3Ac=D0dLcE0P0000020=7<e@030000003Ac=D0000000@0
dLcF00<000000=7=eP0000000P3@cMH2000000<0dLgF00<000000=7>e`3Ac]L00`3Ac]L01@000000
dLoG0000003AcmP0000000D0dM3H00D000000=7Af@3AdMT0dM7I000000040=;Bf@030000003Be=X0
d]CJ0080d]CJ00D000000=;Ff`000000d]KL000000040=;Hg0050000003Cf]d000000=;JgP000000
0`3Cg=l2000000040=?Nh0000000d]kQ000000<0d^3R00@000000=;Rh`00000000000`3@i>H20000
00<0cnKW00<000000<gVj@3=i^T00P0000020<_Xj`040000003:jN`000000<WWk08000000P37j>h0
0`000000aNW_00000002000000<0`nO`00<000000<3Wl@0000000P0000030;kUlP<0000000<0_N?b
0000000000000`2kh_<3000000030;[Pl`00000000000080^=oc00<000000;SNl`2hg_<00P000000
102ggO<000000000000000040;KKl`8000000`2efO8300000080]=Ob0P000000102ceo8000000;?F
lP2ce_82000000040;;ElP000000/mCa0;?Dl@80000000@0/]?b0000002bd_40/];a0P0000020;7B
l@030000002adO00/M7`0080000000<0/=7a0000002ad?000P2ad?03000000@0/<k`0P0000040:o=
k`<000000P2_c>l00`000000[l_^0:o;kP02000000030:k;k`000000[/[^0080[/[^100000030:g8
kP@000000P2]and010000000[LK]0:g6k@2]a^d3000000@0[<G/0`0000030:_4k0040000002[`n/0
Zl?[0:_3j`<000000`2[`^/00`000000oooo0?ooo`020?ooo`030000003oool0oooo01h0oooo000n
0?ooo`030000003oool0oooo06L0oooo00<000000<74g031a=`00P0000001031a=`0000000000031
a=`2000000<0`LCL00@000000<74g000000000000`31a=`2000000D0`/CL00<000000<;4g032a=`0
0`32a=`00`000000`/CL0<;4g0030<;4g00300000032a=`0000000@0`/CL00<000000<;4f`000000
0P32a=/2000000040<;4f`00000000000<;4f`8000000P32`m/010000000`/?K0000000000030<;3
fP0400000032`mX00000000000<0`/;J0P0000050<;2fP0300000032`MT0`/7I00<0`/7I00<00000
0<;1f@32`MT01032`MT00`000000`/3H0<;0f0030<;0f00300000032_mL0`[oG0080`[oG00<00000
0<6ne`0000001031_]L00`000000`KgG000000030<6meP0300000031_=H0000000<0`KcE00<00000
0<2ke@0000001@30^m@00`000000`;WD0<2ie0030<2ie0030000002o^=<0_kSC00<0_kSC00<00000
0;ngdP2o]m800`2o]m82000000@0_[GA00@000000;jdd000000000000P2m/m001`000000_K;?0;fb
c`2m//l000000;bac`000000102l/<h00`000000^jk=0;^^c@020;^^c@040000002k[L`000000000
0080^Jc<00<000000;VZb`2iZ//00P2iZ//00`000000]jS:0;NXbP020;NXbP030000002fY/P0]ZK8
00<00000102cXlP00`000000/J360;6PaP02000000030:nPb0000000[Ig70080[Ig700<000000:ZJ
aP2ZV/H00P2ZV/H01P000000YYK5000000000000Y9O8000000<0Wi?700D0000009V?a`2ISlL0VHo7
0000000309:;b00500000029Q/T0RHK908V6b@0000000`20P/`010000000MWk>0000000000020=?=
e08000000`3BcM@00`000000d/cD0=;<e0030=;<e0050000003Ac=D000000=7<e@0000001@3Ac=D0
10000000dLcF0=7<eP3Ac=H200000080dLgF00<000000=3=eP000000103AcMH00`000000dLkG0000
00020=7>e`8000000`3AcmL00`000000dM3H0=7@f0030=7@f0050000003BdMP000000=7Af@000000
103Bd]T010000000d]CJ0000003Be=X2000000<0d]KK00<000000=;Hg03Bf=`00P3Bf=`01@000000
dm[M0=?Jg@3Cf]d000000080dmcO0P0000040=?Nh0030000003Bh>80d^3R008000000P3Bh^<01@00
0000dNGU0000003@i>H0000000<0cnKW0P0000000`3>j>P00000000000020<_Xj`8000000P3:jN`2
00000080anS^0P0000020<GYk`0400000033jO0000000<?Wl08000000P30io4010000000_^Ob0000
002niO82000000<0_>Gc00@000000;_Rl`2kh_<0^n;c0P0000020;WRm0800000102hgo<300000003
0;OMl`00000000000080]]cd00<000000;KKl`2ffo<00P0000000`2ef_<000000000000200000080
]=Wc0`0000040;?GlP800000102beO82000000@0/]?b0P0000040;7Bl@<000000`2`dO4400000080
/<o`00<000000;3>l02`c_000P000000102_c_0000000:o=k`2_cNl2000000030:k=l0000000[lc_
0080[lc_0`000000102^bnl000000:k;k`2^bnl300000080[LW^00<000000:g8kP2]b>h00P000002
0:c8kP040000002]and0[LO]0:g7k@@000000P2/a^d00`000000[<G/0:c5k002000000030:_5k000
0000ZlC/00<0ZlC/100000020:_3j`040000002[`^/0Zl;[0:_2j`8000000`3oool00`000000oooo
0?ooo`0N0?ooo`00?`3oool2000006P0oooo0P0000050<74g00300000031a=`0`LCL00@0`LCL00<0
00000<74g031a=`00P31a=`00`000000`/CL000000040<;4g00300000032a=`000000080`/CL0P00
00001032a=`0000000000032a=`2000000<0`/CL00@000000<;4g000000000000`32a=/2000000D0
`/CK00<000000<;3f`32`m/00`32`m/00`000000`/?J0<;3fP040<;3fP0300000032`]X0`/;J0080
`/;J00<000000<;2fP0000001032`]X00`000000`/;I000000020<;1f@80000000<0`/7I00000000
00000P32`MT2000000040<;0f0000000`/3H0<;0f080000000<0`KoH00000032_mL00P0000030<6n
e`0400000031_]L0000000000080`KgG00@000000<6meP00000000000P31_=H010000000`KcE0000
000000030<2ke@0300000030^m@0`;_D0080000000@0_k[D00000030^M@0`;WD0P0000000`2o^M@0
00000;nhd`020;nhd`030000002n]m<0000000<0_kOB00<000000;jfdP0000001@2n]M400`000000
_K?@0;fcd0020;fcd0040000002m//l0000000000080_;7?00<000000;b`cP2l/<h00P0000000`2k
[lh000000;^^c@030;^^c@030000002i[<`0^Jc<0080^Jc<00<000000;VZb`2iZ//00`0000050;NX
bP8000000`2dYLT00`000000/j?80;>Sb0020;>Sb08000000`2_X<P00`000000[Ig70:fMa`020:fM
a`030000002ZV/H0ZY[600<000000`2TUlP01@000000Wi?709nCa`2OTlL0000000@0VHo700D00000
09:;b02BRlP0TX_80000000308V6b@0900000020P/`0P8;<0822c0000000MWk>0000003CcM@00000
00@0d/gD00<000000=;=e@0000000P3Bc=@2000000<0dLcE00<000000=7<e@3Ac=D00`3Ac=D01@00
0000dLgE0000003Ac=H0000000D0dLgF00<000000=7=eP3AcMH00P3AcMH01P000000dLkG0=7>e`00
0000dLkG000000@0dLoG00<000000=7?e`0000000P3Ad=P2000000<0d]7H00@000000=;Bf@3Bd]T0
d];I0P0000030=;DfP030000003Be]/0d]KK0080d]KK00H000000=?Gg000000000000=;Hg0000003
0=?Jg@040000003Dg=h00000000000@0dmkP0P0000000`3ChN400000000000030=;Rh`050000003A
iND0dNGU0=7Ui@0000000P3?i^L200000080c^SX0P0000000`3=j^X00000000000020<[Yk0800000
0P38jnh200000080aNW_0P0000030<?Yl00300000030io40`>Oa008000000`2nio8010000000_>Gc
0;cUl`2liO<2000000040;[Tm0000000000000000080^N;d0P000000102ghO@000000;SOl`2hgo<2
00000080]mod0P0000040;KLm0<0000000<0]M[c0000000000000P2cf_@00`000000]=Wc0;CIl`02
000000040;?Hl`000000/mOb0;?GlP80000000@0/]Kc0000002beO80/]Gb0P000000102ae?800000
0;;ClP2bdo82000000040;7ClP000000/M;a0;7Bl@80000000@0/=;a0000002`dO40/=7a0P000000
0`2`d?4000000;3?l0020;3?l0@000000`2_c_03000000@0[/g`0`0000020:k;k`030000002^bnl0
[/__0080000000<0[L[_0000002]bNh00P2]bNh3000000@0[<S^100000020:c7k@040000002/a^d0
[<K]0:c6k@<000000`2[aN`00`000000ZlC/0:_4k002000000030:[4k0000000Zl?[0080Zl?[1@00
00020:[2j`040000003oool0oooo00000200oooo00110?ooo`030000003oool0oooo06H0oooo0P00
00020<74g080000000<0`LCL0000000000000P31a=`2000000030<74g0000000`LCL008000000`32
a=`010000000`/CL0000000000030<;4g08000001@32a=`00`000000`/CL0<;4g0040<;4g0030000
0032a=/0`/CK0080`/CK00<000000<;4f`0000001032a=/00`000000`/?K000000020<;3f`800000
00<0`/?K0000000000000P32`mX2000000030<;3fP000000`/;J008000000`32`]X010000000`/;J
0000000000030<;2f@8000001@32`MT2000000@0`/3H0P0000040<6of00300000031_]L0`KkG00@0
`KkG00<000000<6me`31_ML00`31_ML00`000000`KcF0<6leP030<6leP0300000030^mD0`;_E00@0
`;_E0P0000040;nje08000000`2o^M@010000000_kSC0000000000020;jgd`040000002o]m800000
00000080_[KB00<000000;jed@2n]M400P2n]M400`000000_KCA000000040;fcd0030000002l/Ll0
_;7?00<0_;7?0P0000030;^_cP030000002k[/d0^jk=0080000000<0^Jg=0000002i[<`00`2i[<`2
000000<0]jW;00@000000;NXbP2gZ<X0]jS:0P0000040;BUb@030000002cXlP0/j?80080000000<0
/:;90000002_X<P00`2_X<P010000000[Ig70:fMa`2]WLL300000080Yi[800<000000:BGb02TUlP0
0P2TUlP01@000000Wi?709nCa`2OTlL0000000<0VHo700D0000009:;b02BRlP0TX_80000000308V6
b@0500000020P/`0P8;<0822c00000000P1fO/h010000000d/gD0=;=e03BcM@200000080d/gE00<0
00000=;<e0000000103Ac=D00`000000dLgE000000030=7<e@050000003AcMD0dLgE0=7=e@000000
1@3AcMH01P000000dLgF000000000000dLgF000000@0dLkG00<000000=7?e`3AcmL00P3AcmL01P00
0000dLoG0=7?e`000000dM3H000000@0d]7H00D000000=;Bf0000000d];I000000040=;DfP040000
003CeMX000000=;Ff`800000103Cem`00`000000dm[M0=?Jg@0200000080e=cN00@000000=CNg`00
0000dmkP0P0000030=?Qh@050000003Chn<000000=;Rh`0000000`3AiND010000000dNSV00000000
00020<kXj08000000P3=j^X2000000030<_/k0000000000000<0b>_^0P0000000`35k>l000000000
00030<?Yl080000000<0`>[b0000000000000`2nio82000000040;cWl`000000000000000080^^Cd
0P000000102ii?@000000000000000040;OQm08000000P2ggo@2000000040;GNm0000000]]cd0;KL
m08000000P2dg?@2000000@0/m[d0`0000030;?Hl`<00000102be_<2000000@0/MCb0P0000040;7C
lP800000102`d_42000000@0/=3a0`0000020:o?l@030000002_c_00[lk`008000000P2^c_000`00
0000[/g`0:k=l002000000030:k<l0000000[/__0080[/__0`000000102]b^l000000:g:k`2]b^l3
00000080[<W^00<000000:c8kP2/b>h00P0000000`2/b>h000000:c7k@030:c7k@@000000P2[a^d0
10000000ZlG/0:_5k02[aN`3000000@0Z/C/0`0000020:[3k0040000002Z`^/0Z/;[0:[2j`<00000
00<0oooo0000003oool07`3oool00480oooo00<000000?ooo`3oool0I`3oool2000000D0`LCL0P00
00040<74g00300000032a=`0`/CL00@0`/CL00<000000<;4g032a=`00P32a=`00`000000`/CL0000
00040<;4g00400000032a=`0000000000080`/CL0P0000000`32a=`000000<;4f`02000000<0`/CK
00@000000<;4f`00000000000`32`m/2000000D0`/?K0P0000040<;3fP0300000032`]X0`/;J00@0
`/;J00<000000<;2f@32`]T00P32`]T00`000000`L7I000000040<;1f@0300000031`=T0000000<0
`/3H00<000000<6of00000000`31_mP00`000000`KoH000000040<6ne`0300000031_]L0000000<0
`KgG00<000000<2me`0000000`31_=H00`000000`;cF000000040<2ke@030000002o^mD0000000<0
_k[D00<000000;nie0000000102o^M@00`000000_[OC0;jgd`030;jgd`030000002n]]80_[KB00<0
_[KB00@000000;jed@00000000000P2m]=400`000000_K?@0;fcd002000000030;bbd0000000_;7?
0080_;7?0P0000050;^_cP8000000`2i[Ld00`000000^Jc<0;V/c003000000@0]jW;00@000000;NX
bP00000000000P2eYl/00`000000]:G90;BUb@020;BUb@8000000`2`X/T00`000000[j380:nPb002
0:nPb0060000002]WLL000000000002[WLT000000`2WV/P00`000000Y9O80:BGb0020:BGb0050000
002OTlL0Wi?709nCa`0000000`2ISlL;00000080P8;<00H0000007IncP1fO/h000000=;=e0000005
0=;=e@030000003Ac=D0dLcE0080dLcE00L000000=7=e@3AcMD0dLgE0000003Ac=D0000000<0dLgE
00<000000=7=eP0000000`3AcMH00`000000dLgF0=7=eP020=7=eP030000003Ac]L0dLkG0080dLkG
00H000000=7>e`00000000000=7?e`0000040=7?e`040000003BdMP0d]7H0=;Af08000000`3Bd]P0
10000000d]CJ0=;DfP3Be=X2000000<0dmGJ00<000000=?Gg03Cem`00P3Cem`2000000030=CIg000
0000000000<0e=cN00D000000=CNg`3Dg]l0e=kO000000030=?Qh@8000000`3Chn<00`000000dNGU
0=7Ui@0200000080dNSV00@000000=3Zj000000000000P3=j^X200000080bnc/00@000000<S^k@00
0000b>_^0P0000020<G/k`80000000@0`nca00000000000000000P30j_82000000040;kYl`000000
000000000080_>Oc0P000000102ji_@000000000000000020;WTm080000000@0]n?e0000002ghO@0
]n7d0P000000102fh?D000000000000000040;GNm08000000P2dg?@2000000040;?Km0000000/m[d
0;?Jm080000000@0/]Wd0000002cf?<0/mSc0P0000020;;Gl`030000002be_<0/]Kc0080000000@0
/MKc0000002ae?80/MCb0P000000102`e?<000000;7ClP2ado82000000040;3BlP000000/=;a0;3B
l@80000000@0[m7b0000002`d?40/=3a0P0000000`2_d?4000000:o?l@020:o?l@<000000`2^c_04
000000<0[/c`100000020:g;k`030000002]b^l0[L[_0080000000<0[<[_0000002/bNh00P2/bNh3
000000@0[<S^00H000000:c7k@00000000000:_6kP0000030:_6k@D000000P2ZaNd00`000000Z/C/
0:[4k002000000030:W3k0000000Z/?/00<0Z/?/0`0000020:W1j`040000003oool0oooo000001l0
oooo00130?ooo`030000003oool0oooo06L0oooo00<000000<74g031a=`00P31a=`00`000000`LCL
000000030<74g00400000032a=`0000000000080`/CL0P0000000`32a=`000000<;4g002000000<0
`/CL00@000000<;4g000000000001032a=`2000000@0`/CL00<000000<;4f`32a=/01032a=/00`00
0000`/?K0<;3f`020<;3f`0300000032`m/0000000@0`/?K00<000000<;3f`0000000`32`mX01000
0000`/;J0000000000020<;2fP80000000<0`/;J00000032`]T00P0000030<71f@0400000032`MT0
000000000080`L3I00@000000<;0f000000000000P31_mP010000000`KoH0000000000030<6of004
00000031_]L0000000000080`KkG00@000000<6me`00000000000P30_ML010000000`KcF00000000
00030<2leP0400000030^mD0000000000080_k_E00@000000;nje000000000000P2o^M@00`000000
_kWD0;nie002000000030;jhe0000000_[OC0080_[OC00<000000;jgd`0000001@2n]]800`000000
_KCA0;fdd@030;fdd@8000000`2l/]0010000000_;7?0000000000020;^ac`040000002k[lh0^jo>
0;^_cP8000001@2i[Ld2000000<0^:_=00<000000;NYb`2gZL/00P2gZL/00`000000]JO;0;FWb`02
0;FWb`030000002dYLT0]:G90080000000<0/ZG;0000002`X/T00`2`X/T00`000000[j380:nPb002
0:nPb0050000002[WLT0Zig90:^Mb@000000102WV/P010000000Y9O80:BGb02TUlP80000000309VA
b`000000TXk<0080TXk<00H0000008^;c@2;Rld0Rh_=00000023Qll400000080MWk>00<000000=;=
e@3BcMD00`3BcMD01P000000dLgE000000000000dLcE000000D0dLgE00D000000=7=e@3AcMD0dLgE
000000030=7=eP030000003AcMH0000000@0dLgF00<000000=7>eP0000000P3Ac]L00`000000dLkG
0=7>e`020=7>e`030000003AcmL0dLoG0080dLoG00D000000=;@f0000000d]7H000000040=;Bf005
0000003CdmT000000=;DfP000000103CeMX010000000e=OK0000003Cem`2000000<0e=WL00D00000
0=GLg@000000e=cN000000030=CNg`050000003EhN0000000=?Qh@0000000`3Chn<2000000030=?V
i000000000000080dNSV0P0000020=3Zj0040000003>kNT0000000000080bnc/0P0000030<S^k@03
00000035k>l0aNc_008000000P33k?42000000050<3/lP00000000000000002njO<00P0000020;_Y
m0<000000P2ji_@2000000040;SUm@0000000000000000@0]n?e0P0000020;KPm@80000000@0]=of
0000002eg_@0]Mkd0P000000102cgOD000000000000000040;?Km08000000`2bfO@3000000<0/]Oc
100000020;7Fl`@000000P2`e?<4000000<0/=;b0`0000030:oAlP<00000102_d?4300000080[/ka
100000020:g=l0030000002^c?00[/c`0080000000<0[Lc`0000002]bnl00P2]bnl500000080[<[_
100000020:c8k`040000002/b>h0[<S^0:c8kP<00000102[a^h300000080Z/G]00@000000:[5k@2Z
aNd0Z/G]100000030:W3k0030000002Z`n`0Z/?/0080000000<0ZL;/0000002Y`N/00`2Y`N/00`00
0000oooo0000000O0?ooo`00A03oool00`000000oooo0?ooo`1W0?ooo`0400000031a=`000000000
0080`LCL00@000000<74g000000000001032a=`2000000@0`/CL00<000000<;4g032a=`01032a=`0
0`000000`/CL0<;4g0030<;4g00300000032a=`0000000<0`/CL00@000000<;4f`00000000000P32
a=/2000000030<;4f`000000`/?K008000000`32`m/010000000`/?K0000000000020<;3f`040000
0032`mX00000000000@0`/;J0P0000040<;2fP0300000031`MT0`L7I00@0`L7I00<000000<70f@31
`=T00`31`=T00`000000`KoH0<6of0030<6of00300000031_mP0`KoH00@0`KoH00<000000<6ne`31
_]L00`31_]L00`000000`;gG0<2me`030<2me`0300000030_=H0`;cF00@0`;cF00<000000;nke@2o
^mD00`2o^mD00`000000_kWD0;nie0030;nie08000000`2n^=@010000000_[OC0000000000030;jg
d`030000002n]]80_[KB0080000000<0_KGB0000002m]=400P2m]=400`000000_;?A000000040;bb
d0030000002k/Ll0^k7?0080^k7?00@000000;^_cP00000000000P2j[ll010000000^Jg=0;V]c@2i
[Ld2000000@0^:_=00<000000;NYb`2gZL/00P0000000`2fZL`000000;FWb`030;FWb`8000000`2b
YL/00`000000/:;90;2Rb@020;2Rb@030000002_X<P0[j3800<00000102[WLT00`000000Yi[80:NJ
b0020:NJb0<0000000<0XYS:0000002NULX00P2NULX00`000000VI7;09VAb`0209VAb`050000002B
S/`0TXk<09:>c00000000`2;Rld01@000000PhO?08>7c`000000NXGB00@000000`3BcMD00`000000
dLgE0=7=e@020=7=e@030000003AcMD0dLgE00<0dLgE00D000000=7=e@000000dLgE000000050=7=
eP040000003AcMH0dLgF0=7=eP8000000P3Ac]H00`000000dLkG000000040=7>e`030000003BcmL0
00000080dLoG00D000000=;@f03Bd=P0d]3H000000040=;Bf0050000003CdmT0dm?I0=?Cf@000000
103CeMX01@000000e=OK0=CGf`3Dem/0000000<0e=WL0P0000030=GLg@050000003Dg]l0e=kO0=CN
g`0000000`3EhN000`000000dn?S0=?Sh`0200000080dnKT0P0000000`3BjND00000000000020=3Z
j08000000P3>kNT010000000c>o[0000000000030<S^k@80000000<0aNo_0000000000000P33k?42
00000080`>cb0P0000000`2mk?<00000000000020;_Ym08000000P2ij?D300000080^>Ge0P000000
102fiOH000000;OSm@2ghoD2000000040;GRmP0000000000000000<0]=of0`0000020;?Mm@800000
00@0/]ce0000002cfo@0/m_d0P0000000`2bf_D000000000000200000080/MSd100000030;7Gm0<0
00000`2`eO<3000000<0/=?c100000020:oBlP@000000P2^dO800`000000[m3a0:o@l@0200000004
0:k?l@000000[/ka0:k>l@80000000<0[Lka0000002]cO000P2]cO03000000@0[Lc`0`0000020:c;
l0030000002/b^l0[<[_0080000000<0ZlW_0000002/b>l00P2/b>l500000080ZlO^00<000000:_6
kP2[a^h00P0000000`2Za^h000000:[5k@020:[5k@@000000P2Ya>d010000000ZL?/0:W3k02Y`n`4
000000<0ZL;/00<000000:W1j`2Y`N/00P0000000`2X`N`000000000000O0?ooo`00A@3oool20000
06P0oooo00<000000<74g031a=`00`31a=`00`000000`/CL0<;4g0030<;4g00300000032a=`00000
00<0`/CL00@000000<;4g000000000000P32a=`2000000040<;4g000000000000<;4g08000000P32
a=`010000000`/CL0000000000040<;4f`8000001032a=/00`000000`/?K0<;3f`040<;3f`030000
0032`m/0`/?K00<0`/?K00<000000<;2fP32`]X00`32`]X00`000000`L;J000000030<;2fP030000
0031`MX0000000@0`L7I00<000000<71f@0000000P31`=T2000000040<70f@000000`KoH0<6of080
000000<0`KoH0000000000000P31_mP2000000040<2nf0000000`KkG0<6ne`80000000@0`;gG0000
0030_ML0`;gG0P0000000`30_=L00000000000020<2leP80000000<0_k_F0000002o^mD00P2o^mD0
0`000000_k[E000000030;nie0030000002n^M@0000000@0_[SD00<000000;jgd`2n]m<0102n]m<2
000000<0_KGB00@000000;fdd@00000000000P2l/m400`000000_;;@0;bbd002000000030;^bd000
0000^k7?00<0^k7?00<000000;Z_c`2j[ll00P2j[ll010000000^Jg=0000000000020;R]cP030000
002hZld0^:_=0080^:_=0P0000040;JYc0060000002eYl/000000000002cY/`00000102bYL/01000
0000/:;90;2Rb@2`X/T3000000<0[J3:00D000000:^Mb@2[WLT0Zig9000000020:NJb0<00000102R
V<X01@000000WYG:09jEbP2NULX0000000<0VI7;00D0000009:>c02BS/`0TXk<0000000308^;c@05
00000023Qll0PhO?08>7c`0000000P1jQM801@000000LH;E0000003BcMD0000000@0dLgE00<00000
0=7=e@0000000P3AcMD2000000<0dLgE00<000000=7=eP3AcMH00`3AcMH01@000000dLgF0000003A
cMH0000000D0dLkF00@000000=7>e`3Ac]L0dLkG0P0000020=;?e`030000003AcmL0000000<0d]3H
00H000000=?Af000000000000=;Bf00000030=?Cf@060000003De=T000000000003CeMX000000`3D
em/01@000000eMWK0000003DfM`0000000@0eMcM00D000000=KNgP000000e=kO000000030=GQh004
0000003Ei>40000000000080dnKT0P0000030=;Yi@040000003Ak>L0000000000080c^gY0P000002
0<c_j`0300000039l>d00000008000000P35knl2000000030<;_l@00000000000080`>cb0P000002
0;g/l`030000002jk?@00000008000000P2ij?D2000000030;KWmP000000^>Ge008000000`2fiOH3
000000030;GRmP00000000000080/n7f100000020;?NmP<000000`2bg?D300000080/][e0P000000
102`fOD000000;7Hm02af?@2000000040;3Hm0000000/MOd0;7Gm080000000@0/=Kd0000002`eO<0
/=Gc0P000000102_e?<000000;3Cl`2`do<2000000040:oCl`000000[m;b0:oBlP80000000<0[]7b
0000002^dO800P2^dO84000000<0[/oa0`0000030:g>l@@000000P2]cO400`000000[Lc`0:g<l002
000000030:c<l0000000[<_`0080[<_`0`000000102[bNl000000:_9k`2[bNl300000080ZlS_00@0
00000:_7kP2[anh0ZlO^100000030:[6kP@000000P2YaNh010000000ZLC]0:W4k@2Ya>d400000080
ZL?]00@000000:W2k02Y`^`0ZL;/100000030:S1k08000007`3oool004L0oooo00<000000?ooo`3o
ool0IP3oool00`000000`LCL0<74g002000000040<74g000000000000<;4g08000000`32a=`00`00
0000`/CL000000040<;4g08000001@32a=`00`000000`/CL0<;4g0030<;4g00300000032a=/0`/CK
00<0`/CK00<000000<;4f`0000000`32a=/010000000`/?K0000000000020<;3f`80000000@0`/?K
00000032`m/0`/?K0P0000001032`m/0000000000032`]X200000080`L;J00@000000<;2fP000000
00000`31`MX010000000`L7I0000000000030<71f@8000001031`=T2000000D0`KoH0P0000040<2n
f08000001030_ML2000000D0`;cG0P0000030;nkeP040000002o^mD0000000000080_k[E00@00000
0;nie000000000000P2n^M@00`000000_[SD0;jhe002000000030;fge0000000_[OC00<0_[OC00<0
00000;ffd`000000102m]M800`000000_;?A0;bcd@030;bcd@8000000`2k/]000`000000^k7?0;^a
c`02000000030;Z`d0000000^Zo?00<0^Zo?00<000000;R]cP2h[Lh00P2h[Lh00`000000^:_=0;R[
c@02000000030;J[c@000000]ZW<00@0]ZW<00D000000;>Vc02cY/`0/jK<000000050;:Ub`<00000
00<0[j;;0000002]X<X00`2]X<X010000000Zig90:^Mb@2[WLT3000000<0YY_;00D000000::HbP2R
V<X0XYS:0000000309jEbP030000002ITL/0VI7;0080VI7;00D0000009:>c02BS/`0TXk<00000003
08^;c@0400000023Qll0PhO?000000<0NXGB00<000000762e@0000000`3AcMD200000080dLgE00<0
00000=7=e@000000103AcMD00`000000dLgF000000020=7=eP8000000`3AcMH00`000000dLkF0=7>
eP030=7>eP050000003Ac]H000000=7>e`0000001@3BcmL01@000000d]3H0=;@f03Bd=P0000000@0
dm7H00D000000=?Cf@3CdmT0dm?I000000040=CDf@050000003Dem/0e=OK0=CGf`0000000`3EfM/0
10000000eMcM0=GLg@3Eg=d2000000<0e]kN00<000000=GQh03EhN000P0000020=GTh@040000003E
in<00000000000<0d^WU0P0000020=7/i`040000003@knT0000000000080c>o[0P0000020<W`k@03
00000035lNl00000008000000P32ko42000000030;o_lP00000000000080_Ncc0P0000020;[/m003
0000002gjoH0000000800000102fioH010000000]^Gf0;KUmP2fiOH200000080]>?g0P0000020;?Q
mP80000000@0/^3g0000002cg_H0/mkf0P0000020;7MmP@000000P2afoH3000000<0/=We0`000004
0;3Hm0800000102`e_@2000000@0[mCc0P0000040:oCl`800000102^dO8300000080[]3b00<00000
0:k?l@2^co400P000000102]co8000000:g>l@2]c_4200000080[<ka00@000000:g=l@2]cO40[Lga
0`0000040:c<l0<000000P2[b_000`000000ZlW_0:_9k`02000000030:_9l0000000ZlS_00<0ZlS_
0`0000020:[7k`030000002Za^h0Z/K^0080000000<0ZLK^0000002YaNh00`2YaNh400000080ZLC]
00@000000:W3k@2Y`nd0ZL?]100000020:S2k0040000002X`N`0Z<7/0:S1k0<000007`3oool004P0
oooo00<000000?ooo`3oool0IP3oool2000000D0`LCL00<000000<;4g032a=`01032a=`00`000000
`/CL0<;4g0020<;4g00300000032a=`0000000@0`/CL00<000000<;4g00000000P32a=`200000004
0<;4g000000000000<;4f`8000000P32a=/010000000`/CK0000000000040<;3f`8000001032`m/2
000000D0`/?K00<000000<72fP31`]X00`31`]X00`000000`L7J0<71fP040<71fP0300000031`MT0
`L7I0080`L7I00<000000<70f@0000000`31`=T00`000000`KoI000000040<6of00300000030_mP0
000000<0`;kH00<000000<2nf00000000`30_ML00`000000_kgG000000040<2le`030000002o_=L0
000000@0_k_F00<000000;nje@2o^]D00`2o^]D00`000000_[WD0;jie0030;jie0800000102m]m@0
10000000_[OC0000000000020;ffd`030000002m]M80_KGB0080000000<0_;CB0000002l/m400P2l
/m42000000D0^k;@0P0000030;Z`d0030000002j[ll0^Zo?0080000000<0^:k?0000002h[Lh00`2h
[Lh2000000<0]Z_=00<000000;JYc02fZL`00P2fZL`2000000@0/jK<00<000000;:Ub`2bYL/00`00
00040:nRb`030000002]X<X0[J3:0080[J3:0`0000000`2XW//000000:JKb`030:JKb`030000002R
V<X0XYS:0080XYS:00<0000009jEbP2NULX04`0000001`1jQM8000000762e@1aP]D000000=7=e@00
00001@3AcMD010000000dLgE0=7=e@3AcMD200000080dLgF00<000000=7=eP000000103AcMH00`00
0000dLgF000000020=7>eP8000000`3Ac]H00`000000d/oG0=;?e`030=;?e`050000003Bd=L00000
0=;@f0000000103CdMP01@000000dm;H0000003CdmT0000000@0e=CI00D000000=GFf@000000e=OK
000000030=GIf`050000003Ffm`000000=GLg@000000103Fg]h010000000e^7O0000000000030=GT
h@080000003Ein<0eNOS0000003Djn@000000=;Yi@0000020=7/i`8000000P3@knT010000000cO;Z
0000000000020<W`k@8000000P35lNl00`000000`_;`0000000200000080_nob0P0000000`2lko<0
0000000000020;[/m08000000P2gjoH00`000000]NWg000000020;KWmP80000000@0]>Kg00000000
000000000P2dhoL2000000040;;Rm`0000000000000000@0/^3g0P0000020;7MmP80000000@0/=cg
0000002afoH0/M_f0P000000102`f_H000000;3Im@2`fOD200000080[mWe00<000000;3Hm02`f?@0
0P000000102_eoD000000;3Fm02`e_@2000000040:oEm0000000[mCc0:oDl`80000000@0[]Cd0000
002_do<0[m?c0P000000102^d_<000000:kAlP2^dO82000000030:gAl`000000[]3b0080[]3b0`00
00030:g?lP<00000102/c_4400000080[<ca00<000000:c<l02/c?000P0000000`2[bo0000000:_:
l0020:_:l0@000000P2[bO000`000000ZlS_0:_8k`02000000030:[8k`000000Z/O_0080Z/O_0`00
00040:W6kP040000002YaNh0000000000080Z<G^00@000000:W4k@2Ya>d0ZLC]100000020:S3k@04
0000002X`^`0Z<;/0:S2k0@000000P3oool00`000000oooo0?ooo`0M0?ooo`00B@3oool00`000000
oooo0?ooo`1V0?ooo`8000000P31a=`2000000030<;4g000000000000080`/CL0P0000000`32a=`0
00000<;4g002000000<0`/CL00@000000<;4g000000000000`32a=`2000000D0`/CL00<000000<;4
f`32a=/00`32a=/00`000000`/?K0<;3f`030<;3f`0300000032`m/0000000<0`/?K00<000000<73
f`0000001032`m/00`000000`L;J000000020<72fP80000000<0`L;J0000000000000P31`MX20000
00030<71fP000000`L7I008000000P31`=T010000000`L3I0000000000030<6of@0400000031_mP0
000000000080`;oH00@000000<2nf000000000000P30_]P010000000`;gG0000000000030;nme`04
00000030_=L0000000000080_kcG00<000000;nkeP2o^mH00P000000102n^]H000000;nje@2o^]D2
000000030;jie@000000_[WD0080_[WD00<000000;fhe00000001@2m]m@00`000000_KKC0;ffd`03
0;ffd`8000000`2l]=8010000000_;?A0000000000020;^cd@070000002k/]00^k;@0;^bd0000000
^[7A000000040;Z`d0800000102h[/l01P000000^:g>000000000000]jc?000000@0]Z_=00@00000
0;JYc000000000000P2dZ<d00`000000/jK<0;>Vc0020;>Vc08000000`2`Y<`01@000000[j;;0:nR
b`2_X//000000080[J3:0`0000040:RNb`030000002VVl/0YY_;0080YY_;00<000000::HbP2RV<X0
100000000`2MUld0000009RDcP0209RDcP050000002CTLl0Ti7?09>Ac`0000000`2=Sm001P000000
QXcB08J<dP26S=80000007j:e@@0000000<0LH;E0000003AcMD0103AcMD01@000000dLgE0000003A
cMD0000000D0dLgF00<000000=7=eP3AcMH00P3AcMH01P000000dLgF0=7=eP000000dLkF000000@0
dLkF00<000000=;?eP0000000P3BcmL2000000<0d]3G00@000000=?Af03CdMP0dm7H0P0000030=?B
f0040000003De=T0e=CI0=CDf@8000000`3Ee]T01@000000eMWK0=GIf`3EfM/0000000<0e]_L00@0
00000=KNgP3Fg]h0e]kN0P0000020=KQg`050000003Gi>0000000=GTh@0000000P3Ein<01@000000
e>_T0=C[i03Djn@000000080dNcW00<000000=3_j@3@knT00P000000103=l^X0000000000039m>`2
00000080aO7_0P0000020<;bl0030000002nl_800000008000000P2lko<2000000030;S^m@000000
00000080]n_f0P0000030;GYm`8000000P2di_L2000000040;;Un0000000000000000080/^;g0P00
0000102`hOP000000;;Pm`2bh?L2000000040;3Om`0000000000000000@0/=cg0P0000030;3JmP<0
00000`2_fOD400000080[mOe100000020:oEm0@000000`2^e?@3000000<0[];c0`0000040:gAl`<0
00000P2]d?8400000080[<kb00<000000:c>l@2/c_400P0000000`2/cO4000000:c<l@020:c<l@@0
00000`2[bo0400000080Z/[`00@000000:_9l0000000ZlW`0`0000040:[8k`<000000P2Yanl01000
0000ZLK^0:W6kP2Ya^h3000000@0Z<G^100000020:S4k@040000002X`nd0Z<?]0:S3k@@000001P3o
ool00`000000oooo0?ooo`0M0?ooo`00BP3oool2000006P0oooo0P0000050<;4g08000001032a=`0
0`000000`/CL0<;4g0040<;4g00300000032a=`0`/CL0080`/CL00<000000<;4g00000001032a=`0
0`000000`/CK000000020<;4f`80000000@0`/?K000000000000`/?K0P0000020<;3f`0400000032
`m/00000000000<0`L?K00@000000<;3f`00000000000`31`]X2000000D0`L;J0P0000040<71fP03
00000031`=T0`L3I00<0`L3I00<000000<6of@31_mT01031_mT00`000000`;oH0<2of0030<2of003
00000030_]P0`;kH00<0`;kH00<000000;nme`2o_ML0102o_ML00`000000_kcG0;nle`030;nle`80
0000102n^]H2000000@0_[WE00<000000;jie00000000`2m^=@00`000000_KOD0;fge00200000003
0;fge0000000_KKC0080_KKC00<000000;bed`000000102l]=800`000000^k?A0;^cd@020;^cd@04
0000002k/]00000000000080^[7A00<000000;Z`d02j/=000P0000000`2h[m0000000;R^c`040;R^
c`050000002g[<l0]jc?0;N/c`0000001@2fZld00`000000]:S=0;BXc@020;BXc@030000002cY/`0
/jK<00<00000102`Y<`00`000000[j;;0:nRb`020:nRb`8000000`2[X<`01@000000Z9k;0:RNb`2X
W//0000000<0YY_;100000000`2QV/d0000009fGc@0309fGc@050000002HU<h0V9C>09RDcP000000
0`2CTLl01@000000SHo@08f?d02=Sm00000000<0QXcB00D0000007j:e@1nR]D0000007J8f0030000
0080dLgE0P0000030=7=e@030000003AcMH0dLgF00<0dLgF00H000000=7=eP00000000000=7=eP00
00040=7=eP030000003Ac]H0dLkF0080dLkF00H000000=;?eP3BcmH000000=;?e`0000040=;@e`05
0000003CdML000000=?Af0000000103Cd]P01@000000e=CH0000003De=T0000000@0eMKI00D00000
0=KHfP000000eMWK000000030=KKg0050000003Gg]`000000=KNgP0000000`3FhMl01@000000enCP
0=OTh03Gi>0000000080eNOS00H000000=C[i03Djn@0e>_T0000003CknH200000080d>oY00@00000
0<gbjP00000000000P39m>`2000000030<GekP00000000000080`_;`0P000000102nl_8000000000
002jlO<300000080^>ke0P0000000`2ekOH00000000000030;GYm`80000000@0/^Sh000000000000
00000P2biOP2000000040;3Tn00000000000000000<0/>7h0`0000020;3Om`80000000@0[mkg0000
002`g?L0/=cg0P0000000`2_foL000000000000200000080[m[f100000030:kHmP<000000`2^e_D3
000000<0[]Cd100000020:gCm0@000000P2]d_<00`000000[M7c0:gAl`02000000040:c@l`000000
[M3b0:g@lP80000000<0[<ob0000002/c_800P2/c_83000000@0[<ga0`0000020:_<l@030000002[
bo00Zl_`0080000000<0Z/_a0000002Zb_000P2Zb_0500000080Z/W`00<000000:[8k`2Zb>l00P00
00000`2Yanl000000:W7k`020:W7k`@000000P2XaNh00`000000Z<G^0:S5kP02000000030:S4kP00
0000Z<C]00<0Z<C]1000000;0?ooo`030000003oool0oooo01`0oooo001<0?ooo`030000003oool0
oooo06H0oooo00<000000<;4g032a=`00P32a=`00`000000`/CL000000020<;4g080000000<0`/CL
0000000000000P32a=`2000000030<;4g0000000`/CL008000000`32a=`010000000`/CL00000000
00030<;4f`8000001@32`m/00`000000`/?K0<;3f`030<;3f`0300000031`m/0`L?K00@0`L?K00<0
00000<72fP31`]X00P31`]X00`000000`L;J000000040<72fP0300000031`MX0000000<0`L7J00<0
00000<71fP0000000P31`=T2000000030<30f@00000000000080`KoI0P0000001030_mT000000<2o
f030_mP2000000040<2nf000000000000<2nf080000000<0_kgH0000000000000P2o_ML200000003
0;nle`000000_kcG0080_kcG00<000000;jke`0000000`2n^]H00`000000_[[F000000050;jie@03
0000002m^=@0_KSD00<0_KSD0P0000030;fge0040000002m]]<0000000000080_;GC00<000000;bd
dP2l]=800P0000000`2k]=8000000;^cd@030;^cd@030000002j/M40^[7A00<0^[7A0P0000030;R_
d0030000002h[/l0^:k?0080^:k?0P0000040;N/c`030000002fZld0]Z_=0080000000<0]:[>0000
002dZ<d00`2dZ<d2000000<0/JK=00<000000;2Tc02`Y<`00P2`Y<`00`000000[j;;000000020000
00@0Zj3<00<000000:RNb`2XW//00P2XW//3000000030:BLc@000000XI[=00<0XI[=00D0000009fG
c@2MUld0WIO=0000000309RDcP050000002CTLl0Ti7?09>Ac`0000000`2=Sm001@000000QXcB08J<
dP26S=80000000<0OX[E00H0000007J8f01fR=P000000=7=e@0000040=7=e@030000003AcMD00000
00<0dLgF00<000000=7=eP3AcMH00P3AcMH00`000000dLgF0=7=eP020=7=eP060000003Ac]H00000
0000003Ac]H00000103BcmH00`000000d]3G0=;@e`020=;@e`050000003CdML0dm7G0=?Ae`000000
0`3Cd]P2000000<0e=CH00@000000=GFf@3Ee]T0eMKI0P0000030=KHfP050000003Ffm`0e]_L0=KK
g00000000`3Gg]`200000080e^7O00H000000=OTh03Gi>00enCP0000003Gj>4200000080e>_T0P00
00020=?_iP030000003Al^L0000000<000000P39m>`200000080aOG^0P0000001031mNl000000000
002nl_82000000030;[al`00000000000080]_3e0`0000000`2ekOH00000000000020;;/m`<00000
0P2bj?P2000000040;3Wn0000000000000000080/>Ch0P0000000`2_h_P000000000000200000080
[n3h0`0000030:oNm`<000000P2_foL2000000040:kJm`000000[m[f0:oJmP80000000@0[]Wf0000
002^f?H0[]Sf0P000000102]eoH000000:kFm@2^e_D2000000040:gFm@000000[]Cd0:kDm0800000
00@0[MCd0000002]do@0[M?d0P0000000`2/d_@000000:gBl`020:gBl`@000000P2/d?<4000000<0
[<ob100000020:_>lP030000002/cO40[<ga0080000000<0Zlgb0000002[c?400P2[c?43000000@0
Z/_a0`0000020:W9l0040000002ZbO0000000:[9l0@000000`2Yanl400000080Z<K_00@000000:S5
kP2XaNh0Z<G^0`0000040:S4kP040000002Xa>d00000000000l0oooo00<000000?ooo`3oool0703o
ool004d0oooo00<000000?ooo`3oool0IP3oool010000000`/CL0000000000030<;4g08000001@32
a=`2000000@0`/CL00<000000<;4g032a=`01032a=`00`000000`/CK0<;4f`020<;4f`0300000032
a=/0000000@0`/?K00<000000<73f`0000000P32`m/2000000030<73f`00000000000080`L?K0P00
00000`31`]/000000<72fP02000000<0`L;J00@000000<72fP00000000000P31`MX010000000`L7J
0000000000030<71fP8000001@30`=T2000000@0`;oI0P0000050<2nf0030000002o_MP0_kgH00<0
_kgH0P0000030;nle`040000002o_=L0000000000080_[_G00@000000;jjeP00000000000`2n^]H0
0`000000_[WE0;jie@02000000030;fie@000000_KSD0080_KSD00<000000;fge0000000102m]m@0
0`000000_;GC0;bed`030;bed`8000000`2k]=800`000000^k?A0;^cd@02000000030;ZbdP000000
^[7A0080^[7A00<000000;V`d@000000102h[m0010000000^:k?0000000000020;N]d0030000002g
[<l0]jc?0080]jc?0P0000030;BZcP030000002dZ<d0]:S=00<00000102aY/d010000000/:C<0;2T
c02`Y<`3000000<0[J;=00<000000:^Pc02[X<`00P2[X<`00`000000Z9k;00000002000000@0Y9c=
00D000000:6Jc@2QV/d0XI[=0000000409fGc@030000002HU<h0V9C>00l0000000<0OX[E0000001f
R=P00P1fR=P00`000000dLgE0=7=e@020=7=e@070000003AcMD0dLgE0=7=e@000000dLgF00000004
0=7=eP030000003AcMH000000080dLgF00<000000=7>eP3Ac]H00P3Ac]H00`000000d/oF0=;?eP02
0=;?eP060000003BcmH000000000003Bd=L000000`3CdML01@000000dm7G0000003Cd]P0000000@0
e=CH00D000000=GEf0000000eMKI000000040=KHfP050000003Gf]X000000=KKg00000000`3Gg]`0
0`000000f>7M0=SQg@02000000<0enCP00D000000=OXh@3Gj>4000000=O/h`0200000080dnoV0P00
00020=7bi`040000003>mNT000000<Wgj`8000000P35mNh2000000030<7ek`00000000000080_?Ga
100000020;K`m@8000000P2bkoH200000080/^cg0P0000020;3Zn0<0000000<0/>Oh000000000000
0P2^iOT3000000030:oRn000000000000080[N7i00<000000:oPn02_h?P00P0000000`2^g_P00000
0000000200000080[]ch0`0000040:kJm`8000000`2^fOH3000000<0[MOf0`0000030:gFm@<00000
102]e?@2000000@0[=;d0`0000030:cAl`<000000P2[d?<00`000000[<ob0:c?lP02000000030:_?
l`000000Zlkb0080Zlkb100000030:_=lP@000000P2Zc?400`000000Z/_a0:[;l@02000000030:W:
l@000000ZLW`0080ZLW`100000020:W8l0030000002Yanl0ZLO_0080000000<0Z<O`0000002Xa^l0
0P2Xa^l500000080Z<G_00@000000:S4kP2Xa>h0Z<C^0`00000A0?ooo`030000003oool0oooo01`0
oooo001>0?ooo`030000003oool0oooo06H0oooo00<000000<;4g032a=`00P32a=`00`000000`/CL
000000040<;4g00300000032a=`000000080`/CL0P0000000`32a=`00000000000020<;4g0800000
00<0`/CL00000032a=/00P0000030<;4f`0400000032`m/00000000000<0`L?K0P0000050<73f`80
00001031`]/00`000000`L;J0<72fP040<72fP0300000031`MX0`L7J00<0`L7J00<000000<71fP31
`MX00P31`MX00`000000`<3J000000040<30f@0300000030_mT0000000<0`;oI00<000000<2of@00
00001030_]P00`000000_kkH000000030;nmf0030000002o_MP0000000@0_kcG00<000000;jke`2n
^mL00`2n^mL00`000000_[[F0;jjeP040;jjeP8000000`2m^MD010000000_KSD0000000000020;fg
e0030000002m]m@0_KOD0080000000<0_;KD0000002l]M<00P2l]M<00`000000^kCC000000040;^d
dP8000000`2j/]8010000000^[7A0000000000020;V`d@030000002h[m00^:o@00<0^:o@00<00000
0;N]d02g[M000P2g[M000`000000]jc?0;N/c`02000000030;F/c`000000]:[>00<0]:[>0P000003
0;:XcP030000002aY/d0/JK=0080/JK=0`0000000`2^Y<d000000:fRc@030:fRc@040000002[X<`0
Zj3<0:^Pc0<000000`2WW/d01@000000Y9c=0:BLc@2TW<d0000000<0XI[=200000000`2HUm000000
09>Dd@0209>Dd@050000002>T]<0SY;C08jBd`0000000`28T=D00`000000PXoH0000000300000003
07J8f0000000000000<0dLgE00<000000=7=e@3AcMD00P3AcMD010000000dLgF0=7=eP3AcMH20000
0080dLgF00<000000=7=eP000000103Ac]H00`000000d/kF000000020=;?eP030000003BcmH0d/oF
0080d/oF00D000000=?Ae`3CdML0dm7G000000030=?Ae`030000003De=P0e=CH0080e=CH00D00000
0=GEf03EeMP0eMGH000000030=KHfP8000000`3Gf]X01@000000emkL0=ONg03Gg]`0000000<0f>7M
00D000000=WTgP000000enCP000000020=OXh@050000003Gk><0encS0000003FlN@00P0000020=7b
i`80000000H0c_GY00000039mn/0bOO[00000034n>`500000080_?Ga0P0000000`2gm?8000000;;b
m00300000080/^of0P000000102_kOP000000000000000020;3Zn080000000@0[^Si000000000000
00000P2^iOT200000080[NCi0P0000030:gQn@@0000000D0[]kh000000000000[=gh000000020:kL
n08000000P2]foP00`000000[][g0:kJm`02000000030:gJm`0000000000008000000P2]f?H40000
0080[=Kf100000020:cEm@030000002]e?@0[MCd0080000000@0[=?e0000002/d_@0[=;d0P000000
102/d_@000000:cAl`2/dO<2000000030:_Am0000000Zm3c0080Zm3c0`0000040:_?l`<000000P2Z
c_800`000000Zlgb0:_=lP0200000080Z/cb00@000000:[<l@2Zc?40Z/ca0`0000040:W:l@<00000
0P2YbO4010000000ZLS`0000002Yb?03000000@0Z<O`0`0000020:O6k`040000002XaNl0Z<G_0:S5
k`D00000503oool00`000000oooo0?ooo`0L0?ooo`00C`3oool2000006L0oooo00@000000<;4g000
000000000`32a=`010000000`/CL0000000000030<;4g08000001@32a=`2000000@0`/CL00<00000
0<;4f`32a=/01032a=/00`000000`L?K0<73f`020<73f`0300000031`m/0000000@0`L?K00<00000
0<73f`0000000P31`]/2000000030<72f`00000000000080`L;J0P0000001031`]X000000<71fP31
`MX2000000050<71fP00000000000<71fP0000000`30`=X010000000`<3I0000000000020<2of@04
00000030_mT00000000000<0`;oI00@000000<2nf000000000000P2o_]P010000000_kgH00000000
00020;nmf0030000002o_=L0_kcG0080000000@0_[cG0000002n^mL0_[_G0P0000000`2n^mL00000
0;jjeP030;jjeP030000002m^MH0000000@0_KWE00<000000;fge02m]m@00`2m]m@2000000<0_;KD
00@000000;bed`00000000000P2k]=<00`000000^kCB0;^ddP02000000030;Zcd`000000^[;B00<0
^[;B00<000000;V`d@2i/=400`2i/=400`000000^:o@0;R_d002000000030;N_d@000000]jg@00<0
]jg@0P0000030;F/c`030000002dZ/h0]:[>00<00000102bZ<h00`000000/JK=0;6Vc@03000000@0
[ZC=00<000000:fRc@2]X/d00P2]X/d3000000030:VQcP000000Yik=00<0Yik=00@000000:BLc@2T
W<d0Y9c=100000000`2PVll0000009bId00209bId0030000002HUm00V9O@0080V9O@00D0000009>D
d@2CU=40TiCA0000000308jBd`0500000028T=D0R93E08R@e@0000000P22SmP00`000000NhkJ0000
0003000000030=7=e@000000dLgE00<0dLgE00D000000=7=eP000000dLgF000000050=7=eP040000
003Ac]H0dLkF0=7>eP8000000P3Bc]H00`000000d/oF000000040=;?eP050000003Cd=H000000=?A
e`0000000`3CdML00`000000e=;G000000030=CDf0080000003EeMP0eMGH0000003FemP000000=KH
fP0000040=OJfP050000003HgM/000000=ONg00000000P3HhMd2000000<0fNCN00@000000=OXh@3G
j>4000000P3Gk><01@000000e_7T0=Kai0000000doCU008000000P3>mNT010000000bOO[00000000
00020<Chk0040000002nn>d000000;Sgk`<0000000@0]oCb000000000000/_;d0P000000102_lOH0
00000000000000020:o]n080000000<0[Nci000000000000100000020:cVnP<000000P2]i?T20000
00030:_RnP0000000000008000000`2/h?T2000000<0[=gh0`0000030:gKn0@0000000D0[M[g0000
00000000[=Wg000000020:gHmP8000000P2/eoH00`000000[=Kf0:cFmP02000000030:cFmP000000
[=Ge0080[=Ge0`0000030:cCm@<000000`2/d_@3000000@0Zm7d0`0000020:_@l`030000002[co<0
Zloc0080000000<0Z/kc0000002Zc_800P2Zc_83000000<0Z/cb1@0000020:W;lP030000002Yb_40
ZL[a0080000000<0Z<[a0000002YbO400P2YbO4400000080Z<S`00<000000:S7l02Xao000P000000
0`2Wao0000000:O6k`020:O6k`@000006@3oool00`000000oooo0?ooo`0L0?ooo`00D@3oool00`00
0000oooo0?ooo`1U0?ooo`0300000032a=`0`/CL00@0`/CL00<000000<;4g032a=`00P32a=`00`00
0000`/CL000000040<;4g00300000032a=`000000080`/CL0P0000000`32a=`00000000000020<;4
f`80000000<0`L?K00000031`m/00P0000030<73f`0400000031`m/00000000000<0`L?K0P000005
0<72f`8000001031`]X2000000D0`L7J00<000000<30fP30`=X00`30`=X00`000000`;oI0<2of@03
0<2of@0300000030_mT0`;oI00@0`;oI00<000000;nnf02o_]P00`2o_]P00`000000_kgH0;nmf003
0;nmf0800000102n_=L2000000@0_[_G00@000000;jjeP00000000000P2m^MH00`000000_KWE0;fi
e@02000000040;fhe@000000_KOD0;fge080000000<0_;OE0000002l]]@00`2l]]@00`000000^kCC
0;^dd`030;^dd`800000102j/m<01P000000^[;B000000000000^K7B000000D0^K3A0P0000030;N_
d@030000002g[M00]jg@0080000000<0]Zg@0000002e[<l00`2e[<l2000000<0/j[?00<000000;:X
cP2bZ<h00P2bZ<h2000000<0/:K>00D000000:jTc@2^Y<d0[ZC=000000020:fRc@<00000102YXLh0
0`000000Yik=0:NNc@020:NNc@<0000000<0Xig?0000002PVll00`2PVll01@000000W9W@09bId02L
VM00000000<0V9O@00D0000009>Dd@2CU=40TiCA0000000308jBd`0500000028T=D0R93E08R@e@00
00000`22SmP010000000NhkJ07^>fP1kS]X2000000<0dLgE0P0000030=7=eP030000003AcMH0dLgF
00<0dLgF00D000000=7=eP000000dLkF000000050=;>eP040000003BcmH0d/oF0=;?eP8000000`3C
d=H01@000000dm7G0=?Ae`3CdML0000000<0e=;G00<000000=CDf00000000P3EeMP01@000000e]OH
0=KGf03FemP0000000@0em[J00D000000=SMf`3HgM/0f=gK000000020=SQg@030000003Ii=h0fNCN
0080fNCN00@000000=WYg`00000000000P3Gk><010000000e_7T0=Kai00000020=?di@030000003?
n>L0000000<000000P34n>`4000000040;Sgk`00000000000;;fl@D000000P2_lOH2000000030:__
m`000000000000@0000000D0ZnWj000000000000ZNSj000000020:cVnP80000000@0Z^Gj00000000
000000000P2[h_X2000000040:[QnP000000[>3i0:cPn@80000000<0Zmki0000000000000P000002
0:_Ln0@000000`2/f_P2000000<0[=Wg0`0000030:cGmP@000000`2/e_H400000080ZmCe10000002
0:_Cm@@000000P2[dO@00`000000Zm7d0:_Am002000000030:[@m0000000Zm3c0080Zm3c0`000003
0:[>l`@000000P2YcO8400000080ZLcb00@000000:W;lP2Ybo80ZL_b100000020:S:l@D000000P2X
bO4200000080Z<S`0`0000040:O7l0<000007@3oool00`000000oooo0?ooo`0L0?ooo`00DP3oool0
0`000000oooo0?ooo`1U0?ooo`8000000P32a=`2000000030<;4g0000000`/CL008000000`32a=`0
10000000`/CL0000000000030<;4g08000001@32a=`2000000@0`L?K00<000000<73f`31`m/01031
`m/00`000000`L?K0<73f`020<73f`0300000031`]/0000000@0`L;K00<000000<72f`0000000`31
`]X00`000000`<7J000000040<71fP0300000030`MX000000080`<3J0P0000001030`=X000000<2o
f@30_mT2000000030;nof@00000000000080`;oI0P000000102o_]T000000;nnf02o_]P200000003
0;nmf0000000_kgH0080_kgH00<000000;jlf00000000`2n_=L00`000000_[_G000000050;jke`03
0000002m^MH0_KWF00<0_KWF0P0000040;fhe@8000000`2l]mD00`000000_;KD0;bfe00200000003
0;^fe0000000^kCC0080^kCC00<000000;Zdd`0000001@2j/m<00`000000^K7B0;VadP020;VadP03
0000002i/=40^K3A0080000000<0^;3B0000002g[m400`2g[m42000000<0]Zg@00<000000;F/c`2e
[<l00`0000040;>Zc`030000002bZ<h0/ZS>00<00000102`Y/h010000000[ZC=0:jTc@2^Y<d30000
00<0Zj?>00D000000:VQcP2YXLh0ZJ7>000000020:NNc@<00000102SWLl01@000000X9_?0:2Kc`2P
Vll0000000<0W9W@00<0000009RGd02HUm000P2HUm000`000000TiCA09>Dd@0;0000000308:?f000
0000NhkJ0080NhkJ00D0000007>=gP000000dLgE000000040=7=eP030000003AcMD000000080dLgF
0P0000030=7=eP030000003Bc]H0d/kF00<0d/kF00D000000=;>eP000000d/oF000000040=?@eP03
0000003Cd=H000000080dm7G00<000000=CBe`3Dd]L00P3Dd]L01@000000eMGH0=GEf03EeMP00000
0080e]OH00H000000=OIf@00000000000=OJfP0000030=SMf`040000003Ih=/00000000000<0fNCN
0P0000020=WYg`040000003IkN40000000000080e_7T00@000000=?di@00000000000P3?n>L01000
0000bO[W00000032nnP700000080/_Ka100000000`2Yl_@0000000000004000000040:S]n@000000
00000:K[nP<000000`2Yj?X300000080Z^Gj0P000000102YhoX000000000000000030:[QnP<00000
0P2[g_T2000000040:[Mn@000000Zmch0:_Ln080000000@0Z]_h0000002/f_P0[=[h0P0000000`2[
fOP000000000000200000080ZmSg100000020:_Fm`030000002/e_H0[=Kf0080000000@0ZmGf0000
002[e?D0ZmCe0P000000102[doD000000:_Cm@2[doD2000000030:[Bm@000000Zm7d0080Zm7d1000
00020:[@m0D000000P2Zco<400000080ZLkc00<000000:W=lP2YcO800P0000000`2YcO<000000:W<
lP020:W<lP@000000`2Xbo8300000080Z<Wa00@000000:S9l@2XbO40Z<Wa1@0000020:O8l0030000
002Wao00YlO`00800000803oool00`000000oooo0?ooo`0L0?ooo`00D`3oool00`000000oooo0?oo
o`1V0?ooo`8000001032a=`00`000000`/CL0<;4g0040<;4g00300000032a=`0`/CL0080`/CL00<0
00000<74g00000001032a=`00`000000`LCL000000020<73f`80000000<0`L?K0000000000000P31
`m/2000000030<73f`000000`L?K008000000`31`]/010000000`L;K0000000000020<72f`040000
0031`]X00000000000<0`<7J00@000000<71fP00000000000`30`MX2000000@0`<3J0P0000050;no
f@800000102o_]T2000000<0_kgH00@000000;nmf000000000000P2n_=P010000000_[cG00000000
00030;jke`030000002n^mL0_[_G0080000000@0_K[G0000002m^MH0_KWF0P0000000`2m^MH00000
0;fhe@020;fhe@030000002l^=H0000000@0_;OE0P0000030;^fe0040000002k]=<00000000000<0
^[CC00<000000;Zcd`2j/m<00P0000000`2i/m<000000;VadP030;VadP8000000`2h/=800`000000
]joA0;N_d@02000000030;J^d@000000]Zg@00<0]Zg@0P0000030;B[d0030000002cZ/l0/j[?0080
/j[?0P0000030;6Xc`030000002`Y/h0/:K>0080/:K>0`0000020:fUc`050000002[Xlh0Zj?>0:^S
cP0000000`2YXLh3000000<0YYo?00D000000:>Mc`2SWLl0Xig?000000040:2Kc`030000002LVM00
W9W@00P0000000<0U9OD0000002?U]H00P2?U]H01P000000RYCH08ZDf02:U=P0000008FCfPT00000
103AcMH01P000000dLgE0=7=e@000000dLgF000000@0dLgF00<000000=;=e@0000000P3Bc]H20000
00<0d/kF00@000000=?@eP3Cd=H0dm3F0P0000020=?@eP030000003CdML0000000@0e=;G00D00000
0=GDe`000000eMGH000000020=KGf0030000003GfMT0emWI0080emWI00D000000=SMf`3HgM/0f=gK
000000020=WPf`050000003Ji=`000000=WTgP0000000P3IjMl200000080fNgQ00<000000=SbhP00
00000`0000020<ohi`@0000000@0`__X000000000000^_[Y0`0000000`2[mnd00000000000030000
00030:Wbm00000000000008000000P2SkOP3000000030:K[nP00000000000080YNSk100000000`2W
i_/000000000000200000080ZN?j0P0000000`2XhO/000000000000200000080ZMoj0`0000030:[M
n@<000000`2ZfoP300000080ZmWh0P000000102Zf?P000000:_Hm`2[f?L2000000030:[Gm`000000
ZmKg0080ZmKg0`0000030:_EmP<000000`2[doD3000000<0Z];e100000030:[Am0<000000P2Yd?@0
0`000000Z/oc0:[?l`02000000030:W?m0000000ZLkc0080ZLkc100000030:W=l`@000000P2Xc?80
0`000000Z<_b0:S;lP02000000030:O;lP000000Z<Wa0080Z<Wa100000020:O8l@040000002Wb?00
00000:O8l0@000008`3oool00`000000oooo0?ooo`0K0?ooo`00E03oool2000006L0oooo00H00000
0<;4g032a=`0`/CL00000032a=`200000080`/CL0P0000000`32a=`000000<;4g002000000<0`LCL
00@000000<;4g000000000000`31a=`2000000D0`L?K0P0000040<73f`0300000031`]/0`L;K00@0
`L;K00<000000<72f`31`]/00`31`]/00`000000`<7J0<31fP040<31fP0300000030`MX0`<7J0080
`<7J00<000000<30fP0000000`30`=X00`000000_koJ000000040;nof@030000002o_mT0000000<0
_kkI00<000000;nnf@000000102o_MP00`000000_[cH0;jlf0030;jlf0030000002n^mL0_[_G00@0
_[_G0P0000040;fje`8000000`2m^MH010000000_KSE0000000000020;bheP030000002l]mD0_;OE
0080000000<0^kKE0000002k]]@00`2k]]@00`000000^[CC0;Zdd`040;Zdd`8000000`2i/m<00`00
0000^K7B0;VadP02000000030;RadP000000^;3B00<0^;3B0P0000030;J^d@030000002f[M00]Zg@
00<00000102dZm000`000000/j[?0;>Zc`03000000@0/JS?00<000000;2VcP2`Y/h00`0000040:fU
c`040000002[Xlh0Zj?>0:^ScP@0000000<0Z:7?0000002VWll00`2VWll00`000000Xig?0:>Mc`02
0:>Mc`L0000000<0W9_B0000002HVM<00P2HVM<00`000000U9OD09BGe00209BGe0050000002?U]H0
SiKF08nFeP0000000`2:U=P01@000000QI?J08FCfP25TmX000000080Oi?M00@0000007VBh0000000
00000`3AcMH01@000000dLgE0=7=e@3AcMD0000000@0dLgF00H000000=;=e@3BcMD000000=;>eP00
00040=;>eP050000003CcmH000000=?@eP0000001@3Cd=H010000000e=;G0=CBe`3Dd]L2000000<0
eMCG00D000000=KGf03FemP0e]OH000000030=OIf@050000003IfmT000000=SMf`0000000P3Ih=/0
1@000000f^CL0=[Tg03Ji=`000000080fNWO00<000000=W]h@3IkN400P0000001P3Hl^8000000000
003Emn<000000=3kh`L0000000D0^_[Y00000000000000000:_gk@0400000080XO;b100000020:?]
n08000000P2QjoX200000080YNSk0P0000020:CVn`<000000P2Vi?/4000000030:SQn`0000000000
0080Ymok100000020:SMnP@000000P2Yg?T400000080ZM[h0`0000030:[Hn0<000000`2ZeoL40000
0080Z]Kg100000020:[DmP@000000P2ZdoH400000080ZM;e00<000000:[Am02ZdO@00P0000000`2Y
d?D000000:W@m0020:W@m0<000000`2Yco@400000080Z<kc00<000000:W=l`2YcO<00P0000000`2X
c?<000000:S<lP020:S<lP@000000`2Wbo8400000080YlWb00@000000:O8l@2Wb?40YlSa1000000W
0?ooo`030000003oool0oooo01/0oooo001F0?ooo`030000003oool0oooo06@0oooo00@000000<;4
g000000000001032a=`2000000@0`/CL00<000000<74g031a=`01031a=`00`000000`LCL0<74g002
0<74g00300000031a=`0000000@0`L?K00<000000<73f`0000000P31`m/2000000030<73f`000000
00000080`L;K0P0000001031`]/000000<72f`31`]/2000000030<32f`00000000000080`<7J0P00
00000`30`MX000000<31fP0200000080`<3J00@000000<30fP00000000000`2o_mX010000000_koI
0000000000020;nof@040000002o_]T0000000000080_kkI00<000000;nmf02o_MP00P000000102n
_MP000000000002n_=P2000000030;jlf000000000000080_[_G0P0000000`2m^mL000000;fje`02
0;fje`030000002l^]L0000000@0_KWF00<000000;bheP2l^=H00`2l^=H2000000@0^kKE00H00000
0;^fe000000000000;Zee00000040;Zdd`030000002i/m@0000000@0^K?C0P0000030;RadP030000
002h/=80^;3B00<000001@2f[]42000000<0]:gA00<000000;B[d02dZm000P2dZm02000000<0/ZW@
00<000000;6Xc`2aZ<l00P2aZ<l2000000<0[ZK@00<000000:fUc`2]YLl00P2]YLl3000000030:ZS
d0000000Z:7?00<0Z:7?00<000000:JOc`2VWll00P2VWll00`000000Xig?00000002000000<0WigA
00<0000009bKdP2LVm800P2LVm801@000000V9WC09RId`2HVM<0000000<0U9OD00D0000008nFeP2?
U]H0SiKF0000000308ZDf00500000025TmX0QI?J08FCfP0000000`1oTmd01P000000NI;P07VBh000
0000dLgF000000<0dLgE00H000000=7=e@00000000000=7=eP0000040=;=e@030000003Bc]H0d/kF
0080d/kF00D000000=??eP3CcmH0dloF000000050=?@eP050000003DdMH000000=CBe`000000103E
e=L01@000000e]GG0000003FemP000000080emWI0P0000030=WKf@040000003Ih=/0fN3K000000<0
f^CL00@000000=_Xg@00000000000P3IkN401@000000f?;R000000000000eOOS0080000000<0d?_S
000000000000100000000`2cnn8000000000000600000080XO;b10000000102LkOL0000000000000
00020:7[nP80000000@0X>Wk00000000000000000P2Ti_/2000000030:?To0000000000000800000
0P2Uh_/300000080Ymok0P000000102Vg_/000000:SMnP2XgOX2000000040:OLnP000000ZMci0:WL
n@8000000P2Xf_T400000080ZMWh100000020:WHn0@000000P2Ye_L00`000000Z]Kg0:[Fm`020000
00040:WEm`000000Z]Cf0:[DmP80000000@0ZM?f0000002ZdoH0Z]?f0P0000000`2Yd_H000000:WB
m@020:WBm@<000000`2Yd?D400000080Z<od100000020:S>m0040000002Xc_<0Z<kc0:S>l`<00000
0`2Xc?<400000080Yl_c00<000000:O;lP2Wbo800P0000000`2Vb_8000000:O9lP020:O9lPD00000
0`3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo01/0oooo001G0?ooo`030000003o
ool0oooo06@0oooo00<000000<;4g032a=`00`32a=`00`000000`/CL000000030<;4g00400000031
a=`0000000000080`LCL0P0000000`31a=`000000<74g002000000<0`LCL00@000000<73f`000000
00000`31`m/2000000D0`L?K0P0000040<72f`8000001@30`]/2000000@0`<7J00<000000<30fP30
`=X00`30`=X00`000000_koJ0;nofP040;nofP030000002o_mT0_koI00<0_koI00<000000;nnf@2o
_]T00`2o_]T2000000D0_[gH00<000000;jlf02n_=P00`2n_=P2000000<0_K_G00@000000;fje`00
000000000P2l^]L00`000000_KWF0;fieP02000000030;bheP000000_;SF0080_;SF00<000000;^g
eP0000001@2k]]D00`000000^[GD0;Zee0020;Zee0040000002j]=<0000000000080^K?D00<00000
0;Vcd`2i/m<00P0000000`2h/]<000000;RadP030;RadP8000000`2g/=8010000000]ZkA0;J^d@2f
[]42000000@0]:gA00<000000;B[d02dZm000`0000040;:Yd0030000002aZ<l0/JS?00<00000102^
Y]000`000000[JG?0:fUc`03000000@0ZZ?@00D000000:RQc`2XXLl0Z:7?000000030:JOc`<00000
0`2RWm401@000000WigA09nMd@2OWM40000000<0W9_B00D0000009RId`2HVM<0V9WC0000000309BG
e0l0000000<0Oi?M0000001iT^000P1iT^001@000000dLgE0=7=e@3AcMD0000000@0dLgE00<00000
0=;=e@3BcMD00P3BcMD01P000000d/kE000000000000d/kF000000<0dloF00<000000=??e@000000
0`3Cd=H01@000000e=7F0=CAeP3DdMH0000000<0eMCG0P0000030=KEe`040000003GfMT0emWI0000
00@0fM_I00@000000=[Nf@00000000000`3Ji=`01@000000fnSM0=_Xg@000000g>kN00@000000P3E
mn<00`000000d?_S00000005000000030;?khP000000000000H0000000<0U_3]0000000000000`00
000209c]m`80000000<0Vn_i0000000000001000000209oVo0<000000P2Si?`2000000030:;Ro000
00000000008000000P2Th?/3000000<0Y]kk0`0000030:OLnP<000000P2Xf_T2000000040:OIn@00
0000ZMWh0:WIn080000000@0Z=Sh0000002Yf?P0ZMSh0P0000000`2XeoP000000:WFm`020:WFm`@0
00000P2YeOL4000000<0ZM?f0`0000040:WBmP<000000P2YdOD400000080Z=3e00<000000:S?m02X
co@00P0000000`2Xco@000000:S>m0020:S>m0@000000P2WcO<400000080Ylcc00@000000:O;l`2W
bo<0Yl_c0`0000040:K:lP<000002@3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo
01/0oooo001H0?ooo`030000003oool0oooo06@0oooo0P0000000`32a=`00000000000020<;4g004
00000032a=`00000000000@0`LCL0P0000040<74g00300000031a=`0`LCL00@0`LCL00<000000<73
f`31`m/00P31`m/00`000000`L?L000000040<73f`0300000031`]/0000000<0`L;K00@000000<32
f`00000000000P30`]/2000000040<31f`000000`<7J0<31fP80000000@0`<7K00000030`=X0`<3J
0P0000000`2o`=X00000000000020;nofP80000000@0_koJ0000002o_mT0_koI0P000000102n_]T0
00000;nnf@2o_]T2000000030;jmf@000000_[gH00<0_[gH00<000000;jlf00000000`2n_=P00`00
0000_K_H000000040;fke`030000002l^]L0_;[G00<0_;[G0P0000030;bheP040000002l^=H00000
000000<0^kOF00<000000;^fe@2k]]D00P0000000`2j]]D000000;Zee0030;Zee0030000002i/m@0
^K?D00<0^K?D0P0000030;Rbd`030000002h/M80^;7B00<00000102g/=8010000000]ZkA00000000
00020;F^dP030000002d[M40]:gA0080]:gA0P0000030;>[d@030000002bZM00/ZW@0080/ZW@0P00
00030;2Xd0030000002^Y]00[ZK@0080[ZK@0P0000030:bUd0050000002ZXm00ZZ?@0:ZSd0000000
0`2XXLl400000080YJ7A00D000000::Od@2RWm40XYoA0000000309nMd@040000002LVm80W9_B09bK
dPP0000000<0UI[F0000002AVMP00P2AVMP01@000000SISJ08fHfP2=V=X0000000<0R9OL00<00000
08>GgP0000001P0000000`3AcMD000000=7=e@030=7=e@030000003AcMD000000080d/gE00<00000
0=;>e@3Bc]D00P3Bc]D01@000000dloF0=??eP3CcmH0000000<0dloE00<000000=?@eP0000000`3D
dMH01@000000eM;F0000003Ee=L0000000@0e]GG00@000000=SGe`00000000000`3IfmT200000080
f]kI00D000000=cRfP000000f^CL000000020=_Xg@070000003Lk]h0g>kN0000003Km=l000000=Si
g`0D0000000409?^l@000000000009;/m0H0000000@0V^Sk000000000000V^Kl1000000209kTo0<0
00000P2Rh_`2000000030:;Po00000000000008000000P2Tg_/400000080YMck100000020:OKnP<0
00000`2WfOT3000000<0Z=Sh0`0000030:SGn0@000000`2XeOL3000000<0Z=Cg100000020:SCmP03
0000002Yd_H0ZM;f0080000000@0Z=;f0000002YdOD0ZM7e0P0000000`2XdOD000000:S@m@020:S@
m@<000000`2Xco@400000080Ylkd00<000000:O=l`2WcO<00P0000000`2WcO@000000:O<l`020:O<
l`@000000P2Vbo<00`000000Y/[b0:K:lP02000000d0oooo00<000000?ooo`3oool08`3oool00`00
0000oooo0?ooo`0K0?ooo`00F@3oool2000006H0oooo00<000000<;4g032a=`00`32a=`00`000000
`LCL0<74g0030<74g00500000031a=`0000000000031a=`00P0000000`31a=`00000000000020<74
g080000000<0`L?L00000031`m/00P0000030<73g00400000031`m/0000000000080`L;K00@00000
0<72f`00000000001030`]/2000000@0`<7K0P0000040<31f`8000001@2o`=X2000000@0_koJ0P00
00040;jnf@800000102n_MT010000000_[gH0000000000020;jlf0040000002n_=P0000000000080
_K_H00<000000;fke`2m^mL00P000000102l^]L000000;bje`2l^]L2000000030;bie`000000_;SF
00<0_;SF00<000000;^geP2k]mH0102k]mH2000000<0^[KE00<000000;Zee02j]M@00P0000000`2i
]=D000000;Vce0020;Vce08000001@2h/]<2000000<0]k7C00<000000;N`dP2g/=800P2g/=800`00
0000]JkB0;F^dP020;F^dP030000002d[M40]:gA0080000000<0/jgB0000002cZm400P2cZm400`00
0000/ZW@0;:Yd002000000030;6Yd@000000/:S@0080/:S@00<000000:jVd02^Y]000`0000040:bU
d0040000002ZXm00ZZ?@0:ZSd0@0000000<0Yj;A0000002UXM400`2UXM4010000000XYoA0::Od@2R
Wm480000000309bMe0000000VIcE00<0VIcE00D0000009FJeP2EV]H0UI[F00000003096If0050000
002=V=X0SISJ08fHfP0000000`28Um`01@000000PiON08>GgP23Umh0000000<0OIOR00<000000=7=
e@3AcMD00P3AcMD01P000000dLgE0=7=e@000000d/gE000000@0d/kE00<000000=?>e@0000000P3C
cmH00`000000dloE0=??e@020=??e@050000003DdMH0e=7F0=CAeP0000000`3Ed]H00`000000e]GG
0=KEe`020=KEe`080000003HemL0f=OG0000003IfML000000=WKf@0000030=[Nf@050000003Lh]X0
g>;J0=cRfP0000000P3Kj=d01P000000g>kN0=c^gP000000foCO00000080f?WO00X000000=7ngP00
0000aOkK0000002fo=L000000:Khe@000000VO?F400000000`2BioX000000000000200000080V^Kl
1`00000209kRo@<000000P2Rh?`2000000040:7No0000000Y=kk0:CNn`80000000<0Xmgk00000000
00000P0000020:GKnP@000000P2Vf_X400000080YmSi100000020:OGn0@000000P2Xe_P00`000000
Z=Gg0:SEm`02000000040:SDm`000000Z=Cg0:SDm`80000000<0Z=?g0000002XdoH00P2XdoH40000
0080Z=;f100000020:SAm@D000000P2Wd?D400000080Yloe00@000000:O>m02Wc_@0Ylkd0`000002
0:O=m0D000000P2Vc?<010000000Y/_c0:K;l`2Vbo<3000000l0oooo00<000000?ooo`3oool08`3o
ool00`000000oooo0?ooo`0K0?ooo`00F`3oool00`000000oooo0?ooo`1T0?ooo`80000000@0`/CL
000000000000`LCL0P0000000`31a=`00000000000040<74g00300000031a=`0`LCL00<0`LCL0P00
00040<73g00300000031`m`0`L?L00@0`L?L00<000000<72f`31`]/00`31`]/00`000000`<;K0<32
f`030<32f`0300000030`]/0000000<0`<7K00<000000<31f`0000000`30`M/00`000000_l3K0000
00040;o0fP030000002o_mX0000000<0_koJ00<000000;jofP0000000`2n_]T00`000000_[kI0000
00050;jmf@030000002n_=P0_[cH00<0_[cH00<000000;fkf02m^mP00`2m^mP2000000@0_;[G0P00
00040;bie`050000002l^=H000000000002k^=H00P0000020;^geP80000000<0^[OF0000002j]]D0
0`2j]]D2000000<0^KCE00@000000;Vce000000000000P2h/m@010000000^;;C0;Rbd`2h/]<20000
00@0]k7C00<000000;N`dP2g/=800P0000000`2f[m<000000;F^dP030;F^dP8000000`2c[M801000
0000/j_A0;>[d@2cZm43000000<0/JWA00@000000;2Xd02`Z=00/:S@0`0000030:fWd@030000002/
YM00[:G@0080[:G@0`0000000`2YY=4000000:NRd@030:NRd@030000002UXM40YJ7A0080YJ7A0`00
00000`2RX=<0000009nNd`0309nNd`050000002LWM@0W9gD09bMe00000000`2IW=D01@000000UI[F
09FJeP2EV]H0000000<0TIWH00D0000008fHfP2=V=X0SISJ0000000308RGg00500000023Umh0PiON
08>GgP000000101mUn8010000000dLgE0=7=e@0000040=7=e@040000003Bc]D0d/kE0=;>e@800000
0P3Cc]D00`000000dloF000000040=??e@050000003Dd=D000000=CAeP0000000`3Ed]H01P000000
em?F000000000000e]GG00000080f=OG00D000000=WIe`3IfML0fMWG000000030=[Nf@060000003L
h]X0g>;J0=cRfP000000g^OK0P0000020=c^gP030000003Km=l000000080f?WO00@000000=7ngP00
0000aOkK4000000208OWm@H000000P2CiO`600000080V^;m1@0000000`2Nh?d00000000000020000
0080XMkl100000020:?Mn`80000000@0Xm_k0000002UfoX0YM_j0P000000102Uf_X000000:KJnP2V
f_X2000000040:KIn@000000YmSi0:OHn@80000000@0Y]Oi0000002WeoP0YmOh0P0000000`2We_P0
00000:SFn0020:SFn0<000000`2Xe?L3000000<0Z=?g100000020:SBmP@000000`2WdOH300000080
Ym3f00<000000:O@m@2Wd?D00P0000000`2WcoD000000:O?m@020:O?m@D000000P2Vc_@300000080
Y/cd00@000000:K<l`2Vc?<0Y/cc1@00000C0?ooo`030000003oool0oooo0280oooo00<000000?oo
o`3oool06`3oool005`0oooo00<000000?ooo`3oool0I@3oool00`000000`LCL0<74g0030<74g003
00000031a=`0`LCL00<0`LCL00<000000<74g00000000`31a=`00`000000`LCL000000030<73g004
00000031`m`0000000000080`L?L0P0000001031`m`000000<72f`31`]/2000000040<32f`000000
00000<32f`8000000P30`]/010000000`<7K0000000000030<31f`0300000030`M/0000000<0_l3K
00@000000;o0fP00000000000P2o_mX010000000_koJ0000000000020;jofP040000002n_]T00000
000000<0_[kI00<000000;jmf@2n_MT00P000000102n_MT000000;jlf02n_=P2000000040;flf@00
0000_K_H0;fkf080000000<0_;_H0000002l^]L00P2l^]L00`000000_;[H000000050;bie`030000
002k^=H0^kSF00<0^kSF0P0000030;ZgeP030000002j]]D0^[KE0080000000<0^KKF0000002i]=D0
0`2i]=D00`000000^;?D0;Rce0020;Rce0040000002h/]<0000000000080]k;D00<000000;Nad`2g
/M<00P2g/M<2000000<0]ZoC00<000000;F^dP2e[]800P0000000`2d[]8000000;>]dP030;>]dP<0
00000P2aZm800`000000/JWA0;6Yd@020;6Yd@<000000P2^Z=401@000000[JOA0:fWd@2]Ym400000
0080[:G@0`0000040:VTd@050000002WX]40Yj;A0:NRd@0000000P2UXM43000000@0XZ3C00D00000
09nNd`2OW]<0WikC0000000309bMe0040000002IW=D0VIcE09VLeAX0000000<0dLgE0000003AcMD0
0`3AcMD01@000000d/gE0000003Bc]D0000000D0dlkE00@000000=??e@3CcmD0dloE0P0000030=C@
e@050000003Ed]H0eM;F0=GBeP000000103GdmH01@000000f=OG0=SGe`3HemL000000080fMWG00D0
00000=_Le`000000f]kI000000030=cRfP050000003Nim/0g^OK0000003OkM/0100000020=Sig`04
0000003Ao]h000000<Gnf`h0000000<0Nn;a0000000000001000000208cSn`@0000000@0U>?m0000
00000000UN7m1000000209_Po@@000000P2Ng_d400000080XMgl100000020:?Kn`@000000P2Uf_X4
000000<0Y]Wi0`0000030:KGn@<000000`2We_P400000080YmGh100000020:ODm`@000000P2WdoL4
00000080Ym;f00<000000:OAmP2WdOH00P0000000`2WdOH000000:O@mP020:O@mP<000000`2WcoD4
00000080Y/ke00<000000:K>m02Vc_@00P0000000`2VcO@000000:K<m0020:K<m0@000006@3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo01/0oooo001M0?ooo`030000003oool0oooo
06D0oooo00<000000<74g031a=`00P0000001031a=`0000000000031a=`200000080`LCL00@00000
0<74g000000000000`31a=`00`000000`L?L000000040<73g08000001031`m`2000000D0`<;K00<0
00000<32f`30`]/00`30`]/00`000000`<7K0<31f`040<31f`030000002o`=/0_l3K00<0_l3K00<0
00000;nofP2o_mX00`2o_mX00`000000_[oJ0;jofP030;jofP030000002n_]T0_[kI00@0_[kI0P00
00040;jmf@800000102m_=T2000000<0_;_H00@000000;bje`00000000000`2l^]P00`000000_;WG
0;bie`02000000030;^ie`000000^kSF0080^kSF00<000000;Zhe`000000102j]mH2000000<0^KKF
00<000000;Vde@2i]=D00P0000000`2h]=D000000;Rce0030;Rce0030000002g/]@0]k;D0080]k;D
00<000000;Nad`2g/M<00P0000000`2f/=<000000;J_d`030;J_d`8000000`2d[]800`000000/jgB
0;>]dP03000000@0/J_B00<000000;6Yd@2aZM400`0000040:jXd@040000002]Ym40[JOA0:fWd@<0
00000`2[Y]801@000000ZJCA0:VTd@2YY=40000000<0Yj;A0`0000030:BRd`050000002RX=<0XZ3C
0::Pd`0000000`2OW]<80000000309VNe`000000UYgH0080UYgH00<0000009:LfP2BW=X00P2BW=X0
1@000000Si_L08nKg02?Vm`0000000<0RY_N00D0000008JKh026Vn00QY_P00000003086Kh`040000
003AcMD0dLgE0=7=e@8000000`3BcMD00`000000dlkE0=?>e@030=?>e@050000003Cc]@000000=??
e@000000103Dd=D01@000000eM7D0000003Ed]H0000000@0em?F00D000000=SEe@000000f=OG0000
00020=WIe`040000003Kg=L0fmcG0=_Le`8000000P3Lh]X010000000g^OK0=kWf`0000020=o]f`04
0000003Nm=/000000=_kfQD0000000@0On;g000000000000Pn7j1000000208kRo@<000000P2EhOd2
0000000309OOoP0000000000008000000P2Kg_d400000080W]gl100000020:7Ko0@000000P2Sf_/4
00000080Y=Wj1@0000000`2Uf?T000000000000300000080Y]Oi100000020:KFn0@000000P2Ve?P0
0`000000YmCg0:ODm`02000000040:OCm`000000Ym?g0:OCm`80000000<0Ym;g0000002Wd_H00P2W
d_H3000000<0Ym7f100000020:K@mP@000000P2VcoD010000000Y/ke0:K>m@2Vc_D3000000<0Y/gd
1000000N0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool06P3oool005h0oooo00<0
00000?ooo`3oool0I@3oool2000000D0`LCL00<000000<74g031a=`00`31a=`00`000000`LCL0<74
g0040<74g00300000031`m`0`L?L0080`L?L00<000000<73g00000000`31`m`010000000`<?L0000
000000020<32f`80000000@0`<;K00000030`]/0`<;K0P0000000`2o`]/00000000000020<31f`80
000000@0_l7K0000002o`=/0_l3K0P000000102o`=/000000;nofP2o_mX2000000040;jofP000000
00000;jofP80000000<0_[kJ0000000000000P2n_]T2000000030;fnfP000000_[gI0080_[gI00<0
00000;fmf@0000000`2m_=T00`000000_;cI000000040;bkf0030000002l^]P0_;[H00@0_;[H0P00
00030;^ie`040000002k^=H0000000000080^[SG00<000000;ZgeP2j]mH00P0000000`2j]]H00000
0;VfeP030;VfeP8000000`2h]=D00`000000^;?D0;Rce002000000030;Nce0000000]k;D00<0]k;D
0P0000030;J`d`030000002f[m<0]ZoC0080000000<0]:oC0000002d[]800`2d[]82000000<0/ZcB
00<000000;6[dP2aZm800P2aZm82000000<0[j[B00<000000:jXd@2^Z=400P2^Z=4300000080[:OB
00D000000:^VdP2[Y]80ZjKB000000030:VTd@@000000P2VXm<01@000000Y:;C0:BRd`2TX]<00000
00<0XZ3C100000000`2OX=D0000009bOeP0209bOeP030000002IW]L0VIkG0080VIkG00D0000009JM
f02FWMP0UYgH0000000309:LfP050000002?Vm`0Si_L08nKg00000000`2:Vmh00`000000QY_P08JK
h00208JKh00700000021Vn<0PI_S086Kh`000000dLgE000000040=;=e@030000003BcM@000000080
dlkE0P0000030=?>e0040000003Dd=D0e=3E0=C@e@8000000`3EdM@010000000em?F0=OCeP3GdmH2
000000<0f=GE00D000000=WIe`3IfML0fMWG000000020=_Le`030000003Mh=L0gN3G008000000P3N
im/01P000000gngK0=o]f`000000g_CK00000080fo_J00@000000=7oeP000000__g=3@0000000`1c
g?@000000000000500000080Qmom10000002093PoP<000000P2Ggoh20000000409SMoP000000Vmkm
09_No@80000000<0W=cm0000000000000P00000209oKo0@000000P2Qf_/400000080XmWk10000003
0:CHnP<000000P2UeoT00`000000Y]Oi0:KGn@02000000040:GFn@000000Y]Kh0:KFn080000000<0
Y]Gh0000002Ve?P00P2Ve?P400000080Ym?g100000020:OBm`D000000P2VdOH400000080Y]7f00<0
00000:K@mP2Vd?H00P0000000`2Vd?H000000:K?m@020:K?m@@000000P2Uc_D400000280oooo00<0
00000?ooo`3oool08@3oool00`000000oooo0?ooo`0J0?ooo`00G`3oool2000006H0oooo00<00000
0<74g031a=`00P31a=`00`000000`LCL000000020<74g080000000<0`LCL0000000000000P31a=`2
000000030<73g0000000`L?L008000000P31`m`010000000`L?L0000000000040<33g08000001030
`]/2000000D0_l;K0P0000040;o1f`800000102o`=/2000000D0_[oJ00<000000;jnfP2n_]X00`2n
_]X2000000<0_KkJ00@000000;jmf@00000000000P2m_MT010000000_KcI0000000000030;blf@05
0000002l^mP000000000002l^mP00P0000020;bjf080000000<0^kWH0000002k^ML00`2k^ML00`00
0000^[SG0;Zhe`030;Zhe`8000000`2j]]H00`000000^KKF0;VfeP02000000030;VeeP000000^;CE
00<0^;CE0P0000030;Nce0030000002g/]@0]k;D0080000000<0][7D0000002f/=<00`2f/=<20000
00<0]:oC00<000000;B^dP2d[]800P0000000`2c[]<000000;:/dP020;:/dP030000002aZm80/J_B
0080000000<0/:_C0000002_Z]800P2_Z]800`000000[ZSA0:jXd@03000000@0[:OB00@000000:^V
dP2[Y]80ZjKB100000020:RUd`050000002VXm<0YZ?C0:JSd`0000000`2TX]<4000000030::Re@00
0000Wj3E00<0Wj3E00D0000009bOeP2LWmH0W9oF0000000309VNe`050000002FWMP0UYgH09JMf000
00000`2BW=XC000000@0d/gE00H000000=;=e03BcM@000000=?>e@0000040=?>e0050000003Dc]@0
00000=C@e@000000103EdM@01@000000e]7D0000003GdmH0000000@0f=GE00D000000=WGe@000000
fMWG000000020=_Le`060000003Mh=L0gN3G0=gPe`000000gnGH0P0000020=o]f`070000003Nm=/0
g_CK0000003KnmX000000=7oeP0C00000080Omcl1@0000000`2:g_h000000000000300000080T]kn
0`00000309SMoP<000000P2Lg?d20000000409cKo@000000Wm_l09oKo080000000@0Wm[l0000002Q
f_/0XM[k0P000000102QfO/000000:?In`2SfO/2000000040:?HnP000000Y=Sj0:CHnP80000000<0
Y=Oj0000002UeoT00P2UeoT3000000<0YMKi0`0000030:KEn0@000000P2Ve?P4000000<0Y]?g0`00
00020:KBm`030000002VdOH0Y]7f0080000000<0Y]7g0000002VdOH00P2VdOH400000080Y]3f1@00
00020:G?mP030000002Uc_D0YLke008000009P3oool00`000000oooo0?ooo`0P0?ooo`030000003o
ool0oooo01X0oooo001Q0?ooo`030000003oool0oooo06@0oooo00@000000<74g000000000000`31
a=`2000000D0`LCL0P0000040<73g00300000031`m`0`L?L00<0`L?L00<000000<33g030`m`00`30
`m`00`000000`<?L000000030<32f`0300000030`]`0000000@0_l;K00<000000;o1f`0000000`2o
`M/00`000000_l7K000000030;o0f`030000002n`=/0000000@0_[oJ00<000000;jofP0000000`2n
_]X00`000000_KkJ000000040;fnfP030000002m_MT0_KgI00<0_KgI00<000000;blf@2l_=T0102l
_=T00`000000_;_H0;bkf0030;bkf08000000`2k^MP00`000000^kWG0;^ie`02000000030;Zhe`00
0000^[SG0080^[SG00<000000;Zge`000000102j]]H2000000<0^KGF00<000000;Rde@2h]=D00P00
00000`2h]=D000000;Nce0030;Nce08000000`2f/M@00`000000][3C0;J`d`02000000030;F`e000
0000]:oC00<0]:oC0P0000030;>^d`030000002b[=80/ZcB0080/ZcB0P0000030;2[d`030000002_
Z]80[j[B0080[j[B0P0000030:fXdP030000002/Ym80[:OB0080[:OB0`0000000`2ZY]<000000:RU
d`030:RUd`040000002VXm<0YZ?C0:JSd`@0000000<0Y:?D0000002RX]D00`2RX]D01@000000Wj3E
09nPe@2OX=D0000000<0W9oF00@0000009VNe`2IW]L0VIkG200000000`2DWm/00000092Ng@03092N
g@050000002=W]l0SIkO08fNg`0000001029W^801@000000QIoT08FOi025Wn@0000000<0d/gE00D0
00000=;=e03BcM@0d/gD000000040=?>e0050000003Dc]@0e<kD0=C>e0000000103EdM@01@000000
e]7D0=KAe03FdM@0000000<0f=GE0P0000030=WGe@040000003Kg=L0fmcG000000<0gN3G00D00000
0=oUf03OiMP000000>;/f00200000080g_CK00@000000=_kfP000000dOoF2P00000206CBmP@00000
0P1gf?/400000080Q=_n1000000208gLoPD0000000<0U=cn0000000000000`00000209WKoP<00000
0`2Lfod3000000<0Wm[l0`0000030:7In`<000000`2Sf?X3000000<0Y=Oj100000020:GFn@@00000
0P2UeOP400000080YMCh100000020:GCn0030000002VdoL0Y]?g0080000000<0YM;g0000002Vd_L0
0P2Vd_L3000000<0Y]7g100000030:G@mP<000000P2UcoH010000000YLof0:G?mP2UcoH4000002T0
oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0J0?ooo`00HP3oool00`000000oooo
0?ooo`1T0?ooo`0300000031a=`0`LCL0080`LCL00<000000<74g00000001031a=`00`000000`LCL
000000020<73g080000000@0`L?L000000000000`L?L0P0000001030`m`0000000000030`m`20000
0080`<?L00@000000<32f`00000000000`30`]`010000000_l;K0000000000020;o1f`040000002o
`M/0000000000080_l7K00@000000;o0f`00000000000`2n`=/010000000_[oJ0000000000020;jo
fP040000002n_]X0000000000080_KkJ00<000000;fnfP2m_]X00P000000102m_MX000000;fmf@2m
_MT2000000030;blf@00000000000080_;cI0P0000000`2l^mT000000;bkf0020;bkf0030000002k
^]P0000000@0^kWH0P0000030;Zhe`040000002j^=L0000000000080^[OG00<000000;ZfeP2j]]H0
0P0000000`2i]]L000000;VeeP030;VeeP8000000`2h]=D00`000000]k?D0;Nce002000000030;Nb
e@000000][7D00<0][7D0P0000040;F`e0060000002d[m<000000000002c[m@00000102c[]<01P00
0000/ZcB000000000000/JcC000000@0/:_C00H000000:nZdP00000000000:jZd`0000030:fXdP03
0000002/Ym80[:OB0080000000<0ZjSC0000002ZY]<00P2ZY]<00`000000Z:GC0:RUd`020:RUd`<0
000000<0YZCD0000002TXm@00`2TXm@01@000000XZ;E0::Re@2RX]D0000000<0Wj3E2000000209ZP
f@050000002GWmX0UioJ09NOfP0000000`2DWm/00`000000T9kM092Ng@02092Ng@050000002=W]l0
SIkO08fNg`0000000`29W^8010000000QIoT08FOi025Wn@2000000030=;=e@000000d/gD0080d/gD
00H000000=?=e000000000000=?>e00000030=C>e0060000003Ecm<000000000003EdM@000000`3F
dM@01@000000f=;D0000003HeMD0000000@0fMOE00@000000=_Ie@00000000000`3Mh=L010000000
gnGH0=oUf00000020>;/f0030000003RlmL0000001L000000P1mf?h400000080R=[o10000002093K
o`@000000P2Ffoh50000000309[Jo@000000000000<0000000<0WMWl0000000000000`00000209oI
o0@000000P2Qf?/400000080XmOj100000020:CFn@@000000P2TeOT00`000000YMGh0:GEn0020000
00040:GDn0000000YMCh0:GDn080000000<0YMCh0000002UdoP00P2UdoP3000000<0YM;g10000002
0:GBm`@000000P2UdOL00`000000YM3f0:G@mP02000000030:G@m`000000YLof0080YLof1000000^
0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool06P3oool006<0oooo00<000000?oo
o`3oool0I03oool010000000`LCL0000000000030<74g00400000031a=`00000000000<0`LCL0P00
00050<73g00300000030`m`0`<?L00<0`<?L00<000000<33g030`m`00`30`m`00`000000`<;L0<32
g0040<32g0030000002o`M/0_l7K00<0_l7K00<000000;o1f`2o`M/00`2o`M/00`000000_/3K0;k0
f`040;k0f`030000002n_mX0_[oJ00<0_[oJ00<000000;fnfP2m_]X00`2m_]X2000000@0_KgJ0P00
00050;blf@8000000`2l^mT010000000_;_H0000000000020;^jf0030000002k^MP0^kWH00800000
00<0^kWH0000002j^=L00`2j^=L00`000000^[OG0;Zge`030;Zge`8000000`2i]]L00`000000^KGF
0;VeeP02000000030;ReeP000000^;CE00<0^;CE0P0000040;Nbe@060000002f/M@000000000002e
/MD000001@2e/=@00`000000/joD0;>_e0020;>_e0040000002c[]<0/jkC0;>^d`800000102a[=<0
10000000/:_C0;2[d`2`Zm<2000000@0[Z[C00<000000:fXdP2]Z=800`0000030:^Xd`050000002Z
Y]<0ZZKC0:ZVd`0000000P2XYM<2000000030:NVe0000000YZCD0080YZCD00D000000:BSe02TXm@0
Y:?D000000030::Re@@0000000<0Wj;G0000002MXMP00`2MXMP01@000000VZ3I09ZPf@2JX=T00000
00<0UioJ00D0000009BOf`2DWm/0U9oK00000003092NgA0000000`3BcM@00`000000dlgD0=?=e002
0=?=e0050000003Dc]@0e<kD0=C>e0000000103Ecm<01@000000e]7D0=KAe03FdM@0000000<0f=;D
00<000000=WGe@3IemD00P3IemD020000000fmWE0=_Ie@000000g]gD0000003Mh=L000000P3OiMP0
10000000h^cH0>;/f00000020>;ce`040000003Oo=@000000=3ob@/000000P1ad?h900000080PmSo
1000000208cIo`@000000P2BfOh400000080UmWn1000000209_Io@@000000P2Mf?`500000080X=Ok
100000020:7GnP@000000P2Se_X00`000000Y=Ki0:CFn@02000000030:CEn@000000Y=Gi0080Y=Gi
0`0000030:GDn0<000000`2Ue?P400000080YM?h100000020:GBm`030000002Ud_L0YM;g00800000
00<0YM7g0000002UdOL00P2UdOL3000000<0YM3g1000000b0?ooo`030000003oool0oooo01h0oooo
00<000000?ooo`3oool06P3oool006@0oooo0P00001U0?ooo`0300000031a=`0`LCL00@0`LCL00<0
00000<74g031a=`00P31a=`00`000000`LCL000000040<73g00300000030`m`000000080`<?L0P00
00001030`m`000000<33g030`m`2000000030<32g000000000000080`<;L0P000000102o`]`00000
0;o1f`2o`M/2000000040;o1f`000000_l7K0;o1f`80000000<0_/3K0000000000000P2n`=/20000
00040;jof`000000_[oJ0;jofP80000000@0_KoK0000002m_]X0_KkJ0P000000102m_]X000000000
002m_MX2000000030;bmfP000000_;cI00<0_;cI00<000000;blf@000000102l^mT00`000000^k[H
0;^jf0030;^jf08000000`2k^MP00`000000^[SG0;Zhe`02000000040;Zhf0000000^[OG0;Zge`80
000000<0^KOG0000002i]]L00`2i]]L2000000@0^;GF00H000000;Rde@00000000000;NceP000005
0;Nbe@030000002e/MD0]K7E0080]K7E00<000000;F`e02e/=@00P0000000`2d/=D000000;>_e003
0;>_e0<000000P2b[]@00`000000/JcC0;6/d`020;6/d`<000000P2_Zm@00`000000[Z[C0:jZd`02
0:jZd`8000000P2]ZM@00`000000ZjSC0:^Xd`020:^Xd`030000002ZY]<0ZZKC00<000000`2WY]@0
1@000000YZCD0:JTe02VY=@0000000<0Y:?D100000020::SeP050000002OX]L0Wj;G09nRe`000000
0`2MXMP010000000VZ3I09ZPf@2JX=T<00000080TZ7N00<0000008nQh@2?XN400P2?XN401@000000
S:;S08bRh`2<X^<0000000<0RJ;U0P0000000`3BcM@000000=?=e0030=?=e0030000003CcM<00000
0080e<kD00L000000=G?d`3Ecm<0eLoC0000003Fcm<000000080e]7D00D000000=SBe03Hd]@00000
0=WDd`02000000050=WGe@000000fmWE0=_Ie@0000000`3NgM@010000000gnGH0=oUf00000020>;/
f0070000003RlmL0h_?G0000003Oo=@000000=3ob@0400000080IlOm2000000207WAo`T000000P27
eol400000080SmSo1000000209CHoP@000000P2If?d400000080W=Ol1000000209kGo0030000002P
eo/0X=Ok0080000000@0X=Kk0000002QeoX0XMOj0P0000000`2Re_X000000:?FnP020:?FnP@00000
0P2TeOT5000000030:CEn@000000000000<0000000<0Y=Ch0000000000000`0000020:CCn0@00000
0P2Td_P010000000YM;g0:GBm`2Ud_L3000000<0YM7g100000000`2Td?L0000000000003000003L0
oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0J0?ooo`00IP3oool00`000000oooo
0?ooo`1S0?ooo`8000000P31a=`2000000030<74g0000000`LCL008000000`31a=`010000000`L?L
0000000000030<33g08000001030`m`2000000D0`<;L0P0000040;o2g0800000102o`M/2000000D0
_/3K0P0000040;jof`800000102m_m/2000000D0_KkJ00<000000;bmfP2l_MX00P2l_MX010000000
_;cI0000000000020;blf@030000002l^mT0_;_I0080000000@0^k_I0000002k^]P0^k[H0P000000
0`2k^]T000000;^if0030;^if0800000102j^=P2000000@0^KOG00H000000;Vfe`00000000000;Rf
e`0000050;ReeP030000002g/mH0]k?F0080]k?F00<000000;Nbe@2g/]D00P0000000`2f/]H00000
0;Fae@030;Fae@8000000`2d/=D00`000000/joD0;>_e003000000@0/ZkD00<000000;6/d`2a[=<0
0`0000040:n[e0030000002^Z]<0[Z[C00<00000102]ZM@010000000ZjSC0:^Xd`2[Z=<300000080
ZJOD00<000000:NVe02WY]@00P2WY]@00`000000YZCD0:JTe00400000080Y:CF00D000000::SeP2R
XmH0XZ?F0000000309nRe`040000002MXMP0WJ7H09fQf0@000000P2KX]X01@000000V:;K09RRf`2H
X]/0000000<0UJ7M00<0000009:QgP2BXMh00P2BXMh01@000000Sj7Q08nQh@2?XN40000000<0S:;S
00L0000008VRi@29X^D0RJ;U00000025XnL0000000<0dlgD0P0000020=?=d`030000003Dc]@00000
00<0eLoC00H000000=K?d`3Fcm<000000=KAe00000020=SBe0030000003Ie=<0fMCC0080fMCC00D0
00000=_Ie@3KfMD0fmWE000000020=kMe0040000003QhM@0000000000080h^cH00L000000>;ce`3R
lmL000000=ole0000000d?o900L000000P1abOl800000080Om7o2@00000208_Fo`@000000P2Ae_h4
00000080U]Om1000000209[Go@030000002Leo`0W=Ol0080000000<0W=Kl0000002Neo`00P2Neo`3
000000<0X=Kk0`0000030:;FnP@000000P2SeOX400000080XmCi100000030:CDn@<000000P2TdoP0
0`000000Y=?h0:CCn002000000030:CCn0000000Y=;h0080Y=;h1@0000000`2Td_L0000000000003
00000080Y=7g0`00000m0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool06@3oool0
06L0oooo00<000000?ooo`3oool0I03oool2000000@0`LCL00<000000<74g031a=`01031a=`00`00
0000`<?L0<33g0020<33g00300000030`m`0000000<0`<?L00@000000<33g000000000000P30`]`2
000000040;o2g0000000_l;L0;o2g080000000@0_l;L000000000000_l7K0P0000000`2n`M`00000
000000020;k0f`80000000@0_/3K0000002n_m/0_[oK0P000000102m_m/000000;fof`2m_m/20000
00030;fnf`000000_KkJ00<0_KkJ00<000000;bnfP000000102l_MX00`000000_;cI0;blf@030;bl
f@800000102k^mT2000000<0^k[I00<000000;^if02k^MP00P000000102j^MP000000000002j^=P2
000000030;Vhf0000000^KOG00@0^KOG00<000000;Rfe`2h]]L00P2h]]L00`000000^;GF0;ReeP02
000000030;Nde`000000]k?F00<0]k?F0P0000030;JbeP030000002e/MD0]K7E0080000000<0];7E
0000002d/=D00`2d/=D2000000<0/ZoE00<000000;:^e02b[]@00P2b[]@2000000<0/:cD00<00000
0:n[e02_Zm@00P2_Zm@2000000<0[Z[D00<000000:fYe02]ZM@00P2]ZM@300000080ZjSD00D00000
0:VWe02YYm@0ZJOD000000030:NVe0<000000P2UYMH00`000000Y:CF0:BTeP020:BTeP050000002R
XmH0XZ?F0::SeP0000000P2OX]L400000080WJ?I00D0000009^RfP2KX]X0Vj;J0000000409RRf`05
0000002EXMd0UJ7M09FQg@0000000`2BXMh010000000Sj7Q08nQh@2?XN4<000000030=?=e0000000
dlgC00@0dlgC00D000000=G?d`3Ecm<0eLoC000000040=K?d`050000003Hd]@0f=;D0=SBe0000000
0`3Ie=<01@000000fmKB0000003KfMD000000080g]gD00D000000>7Qe03QhM@000000>CXe00B0000
0080NL_o00@0000007c>o`1lc_l0O<ko1000000208GAo`0400000028dol0R=?o08SCo`D0000000<0
SmCo0000000000000`00000209CEoP@000000P2He_d010000000V]Om09[Go@2Jeod3000000<0W=Kl
1000000209oFn`@000000P2Qe_X400000080X]Gj100000020:?En@@000000P2Se?T00`000000Y=Ci
0:CDn@02000000030:CDn@000000Y=?h0080Y=?h0`0000030:CCn0@000000`2Td_P300000080Y=7h
00<000000:CAm`2TdOL00P0000100?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0
6@3oool006P0oooo00<000000?ooo`3oool0I03oool00`000000`LCL0<74g002000000030<74g@00
000000000080`LCL0P0000000`30a=`000000<33g002000000<0`<?L00<000000<33g00000001030
`m`2000000@0_l;L0P0000050;o2g0030000002n`M`0_/7L00<0_/7L0P0000040;k0f`800000102m
_m/2000000@0_KkK00@000000;fnfP00000000000P2l_]X00`000000_;gJ0;bmfP02000000040;bm
fP000000_;cI0;blf@80000000<0^kcJ0000002k^mT00P2k^mT00`000000^k_I000000040;^jf@80
00001@2j^MP00`000000^KSH0;Vhf0020;Vhf0030000002i]mL0^KOG0080000000<0^;OH0000002h
]]L00`2h]]L2000000<0]kCG00<000000;NceP2g/mH00P0000000`2f/mH000000;JbeP030;JbeP80
00000`2d/MD00`000000];3E0;B`e@02000000030;>`e@000000/ZoE0080/ZoE00<000000;:^e02b
[]@00P0000000`2a[]D000000;2/e0020;2/e0030000002_Zm@0[j_D0080000000<0[jcD0000002^
Z]@00P2^Z]@00`000000[JWD0:fYe003000000@0ZjSD00@000000:VWe02YYm@0ZJOD100000020:NV
eP050000002UYMH0YJGF0:FUeP0000000`2TY=H700000080WjCH00<0000009fSf@2MXmT00P2MXmT0
1@000000Vj;J09^RfP2KX]X0000000<0V:;K30000002096ThP050000002?Y>@0SjCT08nTi0000000
0`2<YNH00`000000RJKX000000050=?=d`050000003DcM8000000=G?d`000000103Fcm<01@000000
em3B0000003Hd]@0000000<0fMCC00D000000=_FdP3Ke]80fmKB000000020=kMe0040000003QhM@0
hN7D00000080i>SD00P000000>K`dP3Vl=8000000>Gkc@000000d?bj000000@0M/?n00@0000007O7
o`1gaol0MlOo2000000208;>o`P000000P2<d_l400000080T]Cn1@00000209KDo@040000002He_d0
V=Km09SFo@@000000P2Ke_`400000080WMGk1000000209oEn`@000000P2QeOX00`000000X]Gj0:;E
nP02000000040:;DnP000000XmGi0:?En@80000000<0XmCi0000002Se?T00P2Se?T3000000<0Y=Ci
100000020:CCn0@000000P2Td_P00`000000Y=;h0:CBn002000000030:?Bn0000000Y=7h0080Y=7h
0`0000130?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool06@3oool006T0oooo0P00
001U0?ooo`8000001@31a=d2000000@0`<CL00<000000<33g030`m`01030`m`00`000000`<?L0<33
g0020<33g0030000002o`m`0000000<0_l;L00<000000;o2g0000000102o`]`00`000000_/7L0000
00030;k1g0030000002n`M`0000000<0_/3K00<000000;g0f`0000000`2m_m/00`000000_KoK0000
00050;fnf`030000002l_]X0_;kJ00<0_;kJ0P0000040;bmfP8000000`2k_=X010000000^k_I0000
000000020;^kf@030000002k^]T0^k[I0080000000<0^[[I0000002j^MP00`2j^MP00`000000^KWI
000000040;Vhf08000000`2h]mP00`000000^;KG0;Rfe`02000000030;Nee`000000]kCG00<0]kCG
0P0000030;JceP030000002f/]H0][;F0080000000<0]K;F0000002d/MD00`2d/MD2000000<0/k3E
00@000000;:_e@2b[mD0/ZoE0`0000030;6^e@040000002`[=@0/:cD0;2/e0<000000`2_[=@01000
0000[Z[D0:jZe02^Z]@3000000<0[:WE00<000000:^Xe02[Z=@00P2[Z=@3000000030:RXeP000000
YjKF00<0YjKF00@000000:FUeP2UYMH0YJGF100000000`2SYML000000::Uf0030::Uf0050000002O
Y=P0WjCH09nTf00000000`2MXmT80000000309VTg@000000UjCN00<0UjCN00D0000009BTh02DY>00
U:CP00000003096ThP050000002?Y>@0SjCT08nTi00000000`2<YNH010000000RJKX08VVj0000003
0=?=d`050000003DcM80e<gB0=C=dP0000000`3Fcm<2000000<0em3B00D000000=WDd`3Ie=<0fMCC
000000030=_FdP040000003Nf=400000000000<0hN7D00<000000>CXe00000000P3Vl=800`000000
iO_=0000000;0000000307k8oP000000000000L000000P27c_h010000000RM3o08W@o`29d?l40000
0080Sm;n1000000209CCo@040000002Fe?d0U]Cm09KDo@D0000000<0VMCl0000000000000`000000
0`2Le?/000000000000300000080W]Ck100000020:3DnP040000002QeOX0XMGj0:7EnP<000000`2R
e?X3000000<0XmCi100000020:?Dn@@000000P2SdoT400000080Xm?h00@000000:CBn02Td_P0Y=;h
0`0000030:?Bn0@00000A`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01T0oooo
001[0?ooo`030000003oool0oooo06<0oooo0P0000020<74g@80000000@0`<CM00000030a=`0`<CL
0P0000000`30a=d00000000000020<33g080000000<0`<?M00000030`m`00P0000020;o3g0040000
002o`]`00000000000<0_l;L00@000000;o2g000000000000P2n`M`010000000_/7L000000000002
0;k1g0040000002n`=/00000000000<0_L3K00<000000;fof`0000000`2m_m/00`000000_KkK0;fn
f`02000000040;bnf`000000_;kJ0;bnfP80000000@0_;gK0000002l_MX0_;gJ0P0000000`2k_=X0
00000;^lfP030;^lfP030000002k^mT0^k_I00<0^k_I0P0000040;Zjf@040000002j^MP000000000
0080^KWI00<000000;Vhf02i^=P00P0000000`2h]mP000000;Rgf0030;Rgf08000000`2g]ML00`00
0000]kCG0;Nde`02000000030;Jde`000000][?F00<0][?F0P0000030;FbeP030000002d/MD0];7E
0080000000<0/k7F0000002c/=D00`2c/=D300000080/JoE00<000000;6^e@2a[]D00P2a[]D30000
0080[jgE00<000000:n/e02_[=@00P2_[=@300000080[J_E00D000000:bYe@2/ZMD0[:WE00000002
0:^Xe0<00000102XZ=H00`000000YjKF0:NVeP020:NVeP<0000000<0YJKG0000002SYML00`2SYML0
1@000000XZGH0::Uf02RYMP0000000<0WjCH100000000`2NYMX0000009^Tf`0309^Tf`050000002I
Y=d0VJCM09VTg@0000000`2GY=h01@000000U:CP09BTh02DY>00000000<0TJCR3@0000000`3CcM<0
00000=C=dP020=C=dP050000003EcM8000000=K?d`000000103Gd=801@000000f=3A0000003Ie=<0
000000<0fmKB00P000000=kHd@3Nf=4000000>7Ld@000000hN7D00000080i>SD00@000000>K`dP00
0000iO_=0`00000307c1o@P000000`24b?h010000000Q/_n08K;oP26boh800000080SM3n10000002
09;Ao@040000002Ddod0U=?m09CCo@@000000P2Hdo`400000080VmCl1000000309gDn`030000002N
e?/0W]Ck0080000000<0WmCj0000002Pe?X00P2Pe?X400000080XMCj100000000`2Re?T000000000
000300000080XmCi100000020:?Cn@030000002SdoT0Xm?i0080000000<0Xm?i0000002SdoP00P2S
doP4000000030:?Bn0000000000000<00000C03oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo01T0oooo001/0?ooo`030000003oool0oooo06@0oooo0P0000040<34g@8000001@30a=d2
000000@0`<?M00<000000;o3g02o`m`00`2o`m`00`000000_l;L0;o2g0040;o2g0030000002n`M`0
_/7L00<0_/7L00<000000;k1g02n`M`00`2n`M`00`000000_L3K0;g0f`040;g0f`030000002m_m/0
_KoK00<0_KoK0P0000040;bnf`800000102l_M/2000000<0^kcJ00<000000;^lfP2k_=X00P000000
102k^mX000000000002k^mT2000000030;ZjfP000000^[[I00@0^[[I00<000000;Vif@2i^MT00`2i
^MT2000000<0^;OH00<000000;Rgf02h]mP00P0000000`2g]]P000000;Nee`030;Nee`8000000`2f
]=L00`000000][?F0;JceP02000000030;Fce`000000]K;F00<0]K;F0P0000030;>aeP030000002c
/=D0/k3E00<00000102a[mD00`000000/JkE0;6^e@02000000030;2^eP000000[jgE0080[jgE00<0
00000:n/e02_[=@00`0000040:f[e@040000002/ZMD0[:WE0:bYe@<000000`2ZZMH01@000000Z:SF
0:RXeP2XZ=H000000080YjKF0`0000040:FVe`050000002SYML0XjGG0:>Ue`0000000`2RYMP40000
0080X:GJ00D0000009jUfP2NYMX0WZGJ0000000309^Tf`050000002IY=d0VJCM09VTg@0000000`2G
Y=h800000080TjKR00D00000096Wi02AYn@0TJOT0000000308nXiP040000002<ZNP0S:WX000000<0
e<gB00D000000=G=dP3EcM80eLgB000000040=O@dP050000003Hd=40f=3A0=S@d@0000000`3Ke]80
1@000000g]SA0=kHd@3Nf=4000000080hMcA00L000000>CXe03Tj=@000000>K`dP000000iO_=00L0
00000P23`_`010000000PlGm08?5o@23aOd900000080R/_m00@0000008c>oP2<c_h0S<kn10000002
097?o@P000000P2Fd_`400000080V];l1000000209cCn`040000002Me?/0WMCk09gDn`@000000P2O
e?X5000000030:3DnP000000000000<000000P2Qe?T300000080X]Ci00<000000:?Dn@2Se?T00P00
00000`2SdoT000000:?Cn@020:?Cn@@000000P2SdoT5000000030:?Cn0000000000000800000D@3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0oooo001]0?ooo`030000003oool0
oooo06@0oooo00H000000<34g@30a=d0`<CM00000030a=d200000080`<CM0P0000001030a=d00000
0<33g@30`md2000000040;o3g@000000_l?L0;o3g080000000<0_l?M0000000000000P2o`]`20000
00040;o2g0000000_/7L0;k1g080000000@0_/7L0000002n`M`0_/7L0P0000000`2n`M`000000000
00020;g0f`80000000@0_L3L0000002m_m/0_KoK0P000000102l_m/000000;bnf`2l_]/200000003
0;bnf`000000_;gK0080_;gK00<000000;^mf`000000102k_=X2000000D0^k_J00<000000;ZjfP2j
^]X00P2j^]X00`000000^[[I0;Zjf@02000000040;Vif@000000^KWI0;Vif@80000000<0^;SI0000
002h]mP00`2h]mP2000000<0]kKH00<000000;Nee`2g]ML00P0000000`2f]MP000000;Jde`030;Jd
e`8000000`2e/mL00`000000]K;F0;FbeP02000000030;Bbe`000000/k7F00<0/k7F0P0000030;:`
eP030000002a[mD0/JoE0080/JoE0P0000030;2^eP040000002_[MD0[jgE0:n]e@<000000`2^[=H0
0`000000[J_E0:f[e@020:f[e@<000000P2[Z]H01@000000ZZWF0:ZYeP2ZZMH0000000<0Z:SF0`00
00030:NWe`050000002UY]L0YJKG0:FVe`0000000`2SYML400000080XZKI00D000000:2UfP2PYMX0
X:GJ0000000309jUfP040000002KY=/0VjCK09^Tf`P000000P2HY]l01@000000UZKQ09JVh@2FY^40
000000<0TjKR00D00000096Wi02AYn@0TJOT0000000308nXiP070000002<ZNP0S:WX08bYj0000000
e<gB000000030=G=dP060000003FcM4000000000003Gd=8000000`3Hd=401@000000f]7@0000003K
e]80000000<0g]SA00D000000>7Ld@3Qg=4000000>GQc`08000000<0Pkok2000000208S6o0040000
0029b?d0RLSm08W8o@T000000P2?cOd400000080Tlol00@0000009G@o02Ed?`0UM3l1000000209SA
o0@000000P2Kd_/010000000W=?k09cCn`2Ldo/400000080W]?j1000000209oCnP@000000P2QdoX0
0`000000XMCi0:7Dn@02000000030:;Cn@000000X]Ci0080X]Ci0`0000030:?Cn@@000000P2SdoT4
000000<0Xm?i0`00001F0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool0
06h0oooo0P00001T0?ooo`0400000030a=d00000000000@0`<CM0P0000040<34g@800000102o`md2
000000D0_l?M0P0000040;o2g0800000102n`M`2000000D0_/7L0P0000040;g0g0800000102l_m/2
000000<0_;kK00@000000;bmf`00000000000`2k_M/01@000000^kcJ000000000000^kcJ00800000
0P2k^mX2000000030;ZkfP000000^[[J00<0^[[J0P0000040;Vif@8000000`2h^=T00`000000^;OH
0;Rgf002000000030;Rgf@000000]kKH00<0]kKH0P0000030;Jef0030000002f]=L0][CG00800000
00<0]KCG0000002e/mL00`2e/mL2000000<0];;G00<000000;>aeP2c/MH00P0000000`2b/ML00000
0;:`eP020;:`eP030000002a[mD0/JoE0080000000<0/JoF0000002`[]H00`2`[]H300000080[jgF
00D000000:j/eP2^[=H0[ZcF000000020:f[e@80000000<0[:_F0000002[Z]H00P2[Z]H010000000
ZZWF0:ZYeP2ZZMH400000080Z:SG00D000000:NWe`2WYmL0YjOG000000030:FVe`@000000P2SYmP0
1@000000XZKI0::Vf@2RY]T0000000<0X:GJ2000000209bVg0050000002JY]h0VZKN09ZVgP000000
0`2HY]l01@000000UZKQ09JVh@2FY^40000000<0TjKR300000000`2?ZnT000000=G=dP020=G=dP03
0000003FcM40e/gA0080e/gA00D000000=S@d@3Hd=40f=3A000000030=[Ad0050000003Nf=40g]SA
0=kHd@0000000P3Qg=402@000000iN7?0>GQc`000000jNW>0000003/m/P000000=KiZP0500000080
R<7j00@0000008S3n`28`o/0R<?k2000000208g9o0040000002>bo`0S/_l08k;o0D000000P2BcO`0
10000000Tlol09??o02Cco`400000080Um3k1000000209[An`040000002Kd_/0Vm;k09_Bn`@00000
0P2Md_X400000080Wm?j100000020:3CnP040000002QdoX0XM?j0:7CnP<000000`2RdoT400000080
X]?i100000020:;Cn@@000000P2SdoT00`000000Xm?i0:?Cn@02000005X0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0H0?ooo`00L03oool00`000000oooo0?ooo`1R0?ooo`030000
0030a=d0`<CM00<0`<CM00<000000<34g@0000000`30a=d01@000000_lCM000000000000_l?M0080
000000<0_l?M0000000000000P2o`md2000000040;k2g@000000_l;L0;o2g080000000@0_/;M0000
002n`M`0_/7L0P0000000`2n`M`00000000000020;k1g080000000@0_L3L0000002m`=`0_L3L0P00
00000`2m_m`000000;bof`020;bof`030000002l_]`0000000@0_;kK00<000000;^mf`2k_M/0102k
_M/00`000000^kcJ0;^lfP030;^lfP8000000`2j^mX00`000000^[[J0;ZjfP02000000040;VjfP00
0000^KWI0;Vif@80000000<0^KWJ0000002h^=T00`2h^=T2000000<0^;OI00<000000;Nff02g]]P0
0P0000000`2g]]P000000;Jef0030;Jef08000000`2e]=L00`000000]K?G0;Fce`02000000030;Bc
e`000000];;G00<0];;G0P0000030;:ae`040000002b/=H0/[3F0;:`eP<000000`2a[mH00`000000
/:kF0;2^eP03000000@0[jgF00@000000:j/eP2^[=H0[ZcF0`0000030:b[eP040000002[Z]H0Zj[F
0:^ZeP@000000P2YZML01@000000Z:SG0:RXe`2XZ=L0000000<0YjOG100000020:FWf0050000002S
YmP0XjOH0:>Wf00000000`2RY]T4000000030:2Wf`000000WZKK00<0WZKK00D0000009bVg02LY]`0
W:KL0000000309ZVgP040000002HY]l0V:KO09RVg`P000000P2FZ><01@000000TjWU09>Yi@2CZND0
000000<0TJ[W00D0000008n[j@2?ZnT0Sj_Y000000020=G=dP070000003FcM40e/gA0=K=d@000000
elg?000000020=S@d@050000003JdM00f]7@0000003Ldlh00P0000001`3Nf=4000000>7Ld@3Qg=40
00000>GQc`0000000P3YjLh010000000k?K800000000000308Rnn0P000000P2<a?X010000000SLKk
08g6n`2=a_/800000080T/_k00@0000009;=o02BcO`0T/gl1@00000209K>n`@000000P2Ico/01000
0000V]7k09[An`2JdO/400000080W=7j1000000209kBnP030000002OdoX0Wm?j0080000000<0Wm;j
0000002PdoX00P2PdoX400000080XM?i100000020:;Cn@@000000P2RdoT00`000000X]?i0:;Cn@02
000000030:;Cn@000000Xm?i0080Xm?i0`00001M0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool0603oool00740oooo00<000000?ooo`3oool0HP3oool2000000030<34g@0000000000
0080`<CM00@000000<34g@0000000000102oa=d00`000000_l?M0;o3g@030;o3g@800000102n`]d2
000000@0_/;M0P0000050;k1g0800000102m`=`2000000<0_KoL00@000000;bof`00000000000P2l
_]`00`000000_;kK0;bnf`02000000030;^nf`00000000000080^kgK0P000000102k_M/000000;^l
fP2k_=X2000000030;Zlf`000000^[_J00<0^[_J0P0000040;VjfP8000000`2i^MX00`000000^;SI
0;Rhf@02000000030;Rhf@000000^;OI00<0^;OI0P0000030;Nff0030000002f]MP0][GH00800000
00<0][GH0000002e]=L00`2e]=L2000000<0];?G00<000000;Bbe`2d/]L00P0000000`2c/]L00000
0;:ae`030;:ae`<000000P2a/=L00`000000/JoF0;6_eP020;6_eP8000000`2_[]H00`000000[jgF
0:n]eP020:n]eP<000000P2][=H00`000000[:_F0:b[eP020:b[eP<000000P2ZZmL01@000000ZJWG
0:VYe`2YZML0000000<0Z:SG100000020:JXf0050000002UYmP0YJOH0:FWf00000000`2SYmP40000
00030::WfP000000X:OK00<0X:OK00D0000009jVf`2NY]/0WZKK0000000309bVg0P000000P2IZ>00
1@000000V:SQ09RXh@2HZ>40000000<0UZSS00D0000009>Yi@2CZND0TjWU00000003096Zi`D00000
00<0eLgB0000003FcM400P3FcM401P000000elg?0=O=c`000000f=3A00000080f]7@00<000000=cC
cP3Ldlh00P3Ldlh010000000hMcA0>7Ld@0000020>GQc`040000003YjLh000000>cfb0H000000P2<
_oL01@000000S<7i08c1n@2<`OT0000000<0S<Cj100000020936nP040000002AbO/0TLWk0979n`P0
00000P2EcO/500000080V<kj00@0000009W?n`2Ico/0VLok1000000209c@nP@000000P2NdOX01000
0000W];j09kBnP2Nd_X3000000<0Wm;j100000020:7Bn@@000000P2RdoT400000080X]?i00@00000
0:;Cn@2RdoT0X]?i0`0000030:;Cn@@00000H03oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo01P0oooo001b0?ooo`030000003oool0oooo06<0oooo00<000000<34g@30a=d00`30a=d0
0`000000_lCM0;o4g@030;o4g@030000002o`md0000000<0_l?M00<000000;o3g@0000000`2n`]d0
0`000000_/;M000000030;k2g@030000002n`]d0000000@0_/7L00<000000;g1g@0000000`2m`=`0
0`000000_L3L000000040;fog0030000002l_]`0_;kL00<0_;kL0P0000050;^nf`800000102k_M/2
000000<0^[cK00<000000;ZkfP2j^mX00P000000102i^mX000000;VjfP2i^]X2000000030;VjfP00
0000^KWJ00<0^KWJ0P0000030;Rhf@030000002h]mT0^;OI0080000000<0]kOI0000002g]]P00`2g
]]P2000000<0][GH00<000000;Fde`2e]=L00P0000000`2d]=P000000;Bce`030;Bce`8000000`2c
/]L00`000000/[7G0;:ae`02000000030;:ae`000000/K3G0080/K3G00<000000;6_eP2a[mH00P00
00000`2`[mL000000:n^eP020:n^eP030000002_[MH0[jgF0080000000<0[ZgG0000002][=H00P2]
[=H00`000000[:_F0:b[eP03000000@0ZZ_G00@000000:VYe`2YZML0ZJWG100000020:NYf0050000
002VZ=P0YZSH0:JXf00000000`2UYmP400000080XjSI00D000000::WfP2RYmX0XZOJ000000030:2W
f`040000002NY]/0WZKK09jVf`@000000P2MZ=d01@000000VjON09^WgP2KYmh0000000<0VJSP00D0
000009RXh@2HZ>40V:SQ0000000309JXh`P000000P2D[>L010000000TZgY09:]j@0000030=K=d@03
0000003GcLl0elg?0080elg?00D000000=[Ad03JdM00f]7@000000030=cCcP040000003Oe<d00000
00000080iN7?00L000000>WYcP000000k?K80000003FnJX000000080SKcf00@0000008bom`2<_oL0
S;og200000020934n@040000002@a_X0T<Kj0936nP@000000P2DbOX010000000U<_j09C;nP2DboX4
00000080V<cj00@0000009S>nP2Hc_X0V<kj1@0000000`2KcoX000000000000300000080WM3j00@0
000009kAnP2NdOX0W]7j1000000209oBn@@000000P2Pd_T400000080XM;i00<000000:;Cn@2RdoT0
0P0000000`2RdoT000000:;Cn@020:;Cn@@0000000<0X]?i0000000000000`00001U0?ooo`030000
003oool0oooo01L0oooo00<000000?ooo`3oool0603oool007<0oooo0P00001T0?ooo`0300000030
a=d0`<CM0080000000@0_lCM000000000000_lCM0P0000020;o3g@040000002o`md0000000000080
_l?M00@000000;k2g@00000000000`2n`]d00`000000_/;M000000030;k2g@040000002n`M`00000
00000080_L7M00@000000;g0g000000000000P2m`=`00`000000_KoL0;fog002000000040;bog000
000000000;bng080000000<0^kkL0000000000000P2k_]/2000000030;^mg0000000^kgK0080^kgK
00<000000;Zlf`000000102j_=/2000000@0^K_J0P0000030;VjfP030000002i^MX0^KWJ00800000
00<0^;WJ0000002h^=T00`2h^=T2000000<0]kOI00<000000;Nff02g]]P00P0000000`2f]]T00000
0;Jef0020;Jef0<000000`2d]=P00`000000];?G0;Bce`02000000030;>cf0000000/k;G00<0/k;G
0P0000030;:ae`040000002a/=L0/K3G0;6`e`<000000`2`[mL010000000[jkF0:n^eP2_[]H30000
00<0[ZgG00@000000:f/eP2][=H0[JcF0`0000030:^[e`030000002ZZmL0ZZ_G0080ZZ_G0`000002
0:VZf0050000002WZMP0YjWH0:NYf00000000`2VZ=P400000080YJSI00D000000:>Xf@2SZ=T0XjSI
000000030::WfP040000002PYm/0X:OK0:2Wf`@000000P2OZ=`01@000000WJSM09fXg@2MZ=d00000
00<0VjON00@0000009VXh02IZ>00VJSP2000000209NZi0050000002FZnD0UZ_U09J[i@0000000`2D
[>L01`000000TZgY09:]j@2B[NT000000=K=d@000000103GcLl01@000000fLg>0000003JdM000000
00<0g=?>00D000000=oDc@3Oe<d000000>?Gb`0200000080jNW>00<000000>cfb000000020000002
0930m`040000002@`_P0T<;h0932n0P000000P2CaoT010000000U<Wj09C9nP2DbOX400000080Ul_i
00@0000009S<nP2Hc?X0V<cj1000000209[>nP@000000P2LcoT010000000WM3j09g@nP2Md?X50000
000309oAn@000000000000<000000P2PdOT4000000030:7Bn@000000XM;i0080XM;i100000020:;C
n@D0000000<0X]?i0000000000000P00001[0?ooo`030000003oool0oooo01H0oooo00<000000?oo
o`3oool0603oool007D0oooo00<000000?ooo`3oool0HP3oool2000000D0_lCM00<000000;o3g@2o
`md00`2o`md00`000000_l?M0;o3g@030;o3g@030000002n`]d0_/;M00@0_/;M00<000000;k2g@2n
`]d00`2n`]d00`000000_L7M0;g1g@030;g1g@030000002m`=`0_L3L00<0_L3L0P0000050;bog003
0000002k_]`0^kkL00<0^kkL0P0000030;^mg0040000002k_M/0000000000080^[cK00<000000;Zl
f`2j_=/00P000000102i_=/000000;VkfP2i^mX2000000030;Vkf`000000^K[J00<0^K[J0P000003
0;RifP030000002h^=T0^;SI0080000000<0]kSJ0000000000000P2g]mT3000000@0][KI0`000002
0;Fef@8000000P2d]=P3000000<0/k?H00<000000;>be`2c/]L00P0000000`2b/]P000000;:ae`03
0;:ae`<000000P2`/=L00`000000/:oG0;2_e`020;2_e`<000000P2^[]L00`000000[ZgG0:j]e`02
0:j]e`<000000P2/[=L01@000000Zj_G0:^[e`2[ZmL000000080ZZ_G0`0000040:VZf0040000002W
ZMP0YjWH0:NYf0@000000P2VZMT01@000000YJSI0:FXf@2UZ=T0000000<0XjSI200000020:6Xg005
0000002OZ=`0WjSL09nXg00000000`2MZ=d800000080VjWQ00D0000009VZhP2IZ^80VJ[R00000003
09NZi0040000002FZnD0UZ_U09J[i@X00000103GcLl01@000000fLg>0=W=cP3IcLh0000000<0g=?>
00@000000=oDc@3Oe<d000000P3Sel/00`000000j=g80000000400000003096kl`000000T;ge00<0
T;ge00@000000930m`2@`?L0T<3g1000000209?3m`040000002CaOP0TlGh09?5n0P000000P2GbOT8
00000080V/ci1000000209c>n@040000002LcoT0W<oi09c?n@@000000P2Nd?T400000080X=7i00<0
00000:3An@2PdOT00P0000000`2QdOT000000:7Bn@020:7Bn@@000000P2Rd_T400000080X]?i1000
001_0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0603oool007H0oooo00<00000
0?ooo`3oool0HP3oool200000080_lCM0P000000102oa=h000000;o3g@2o`md2000000040;o3g@00
0000_l?M0;o3g@80000000<0_/?M0000000000000P2n`]d2000000040;k2g@000000_/;M0;k2g@80
000000@0_L7M0000002m`Md0_L7M0P000000102m`Md000000000002m`=`2000000030;c0g@000000
00000080_;oL0P000000102l_m`000000;^ng02k_]`2000000030;^ng0000000^kgL00<0^kgL00<0
00000;Zlf`2j_=/00`2j_=/2000000@0^KcK0P0000030;Vkf`030000002i^]X0^K[J0080000000<0
^;[J0000002h^MX00`2h^MX2000000D0]kSJ0P0000020;Jgf@80000000D0][KI000000000000]KKI
000000050;Fef@8000000P2d]=P00`000000/k?H0;>cf0020;>cf0800000102b/]P01P000000/[7G
000000000000/K7H000000<0/;3G00<000000;2_e`2`[mL00P0000000`2_[mP000000:j^e`020:j^
e`030000002^[ML0[ZgG0080000000<0[JgH0000002/[=L00P2/[=L010000000Zj_G0:^[e`2[ZmL3
000000<0ZZ_H00<000000:VZf02YZ]P00P2YZ]P300000080Yj[I00D000000:JYf@2VZMT0YZWI0000
00030:FXf@@0000000<0XjWJ0000002RZ=/00`2RZ=/01@000000XJSL0:6Xg02QZ=`0000000<0WjSL
1000000209jYgP050000002LZMl0W:WO09bYg`0000000`2KZN4010000000VJ[R09VZhP2IZ^880000
0080V:gU00D0000009J^i`2F[^L0UZkW0000000309F_j@80000000<0elg?0000003IcLh00P3IcLh0
1@000000f/g<0000003Ldlh000000080gmC=00`000000>?Gb`3Sel/0hmO;0000003XgLP000000?3W
``000000k?NZ0000002A^o<900000080Tl3f00@0000009?3m`2C`oL0Tl?g1000000209K5m`040000
002FaoP0U/Oh09K7n0@000000P2IbOP010000000VL_i09W;n@2IboT400000080W<ch00@0000009c>
n@2Lc_T0W<ki1000000209k?n@@000000P2Od?P010000000X=7i0:3An@2PdOT3000000<0XM7i1000
00020:;Bn@@000000P2Rd_T4000007@0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?oo
o`0H0?ooo`00M`3oool00`000000oooo0?ooo`1S0?ooo`800000102oa=h2000000@0_l?M0P000005
0;k3g@800000102n`]d2000000@0_L7M0P0000050;g1g@030000002l`=d0_<3M00<0_<3M0P000004
0;bog08000000`2k_]`00`000000^kgL0;^mg002000000040;Zmg0000000^[cK0;Zlf`80000000@0
^[cL0000002i_=/0^KcK0P0000000`2i^m/000000;Vkf`030;Vkf`800000102h^]X01@000000^;WJ
000000000000]kWJ008000000P2g^=X2000000030;JhfP000000][OI00<0][OI00<000000;Fff@2e
]]T00P2e]]T00`000000]KGI0;Fef@02000000030;Bdf@000000];CH0080];CH00H000000;>cf000
000000000;>cf00000050;:bf0030000002a/MP0/K7H0080/K7H00<000000;2`e`2`/=L00`000003
0:n_f0040000002^[]L0[ZkG0:j^e`<000000`2][MP010000000[:cG0:b/e`2/[=L400000080ZjcH
00D000000:Z[f02ZZmP0ZZ_H000000020:VZf0<00000102WZ]T010000000YZWI0:JYf@2VZMT40000
00030:FYfP000000XjWJ00<0XjWJ00D000000::Xf`2RZ=/0XZSK000000030:6Xg0@000000P2OZMd0
1@000000WZWN09jYgP2NZMh0000000<0W:WO00@0000009^Yh@2KZN40VjWQ1000000209^/h`050000
002I[>@0VJcT09V/i00000000`2H[ND010000000UZkW09J^i`2F[^L7000000<0fLg>00D000000=[=
c03JcL`0f/g<000000030=oDc@040000003Sel/0hmO;00000080j=g800<000000?3W``0000000`00
000209Bll`040000002D__@0U;kd09Bnm0T000000P2F`oH010000000U/Gg09K5m`2FaOL400000080
VLOg00@0000009W9n02IbOP0VLWh1000000209_;n0040000002Lc?P0W<ch09c<n0@000000P2Nc_P4
00000080Wloh00@0000009o@n02Od?P0Wm3h100000020:7@n0@000000P2RdOP400000080X];h00<0
00000:;Bn@2Rd_T00P00001h0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool05`3o
ool007P0oooo00<000000?ooo`3oool0H`3oool00`000000_lCN0;o4gP02000000040;o4gP000000
00000;o3g@80000000<0_/?N0000000000000P2n`md2000000040;k3gP000000_/;M0;k2g@800000
00@0_L;M0000002m`Md0_L7M0P0000000`2m`Md00000000000020;g1g@80000000@0_<7M0000002l
`=d0_<3M0P000000102l`=d000000;bog02l_m`2000000030;^og@000000^kkL00<0^kkL0P000004
0;Zmg0800000102j_=`2000000@0^K_K00H000000;Vkf`00000000000;Rjf`0000050;RjfP030000
002g^MX0]kWJ00<0]kWJ0P0000030;JhfP030000002f]mT0][OI0080000000<0]KOJ0000002e]]T0
0`2e]]T2000000<0];CI00@000000;Bdf02d]=P0];CH0P0000040;>cf0030000002b/]P0/[;H0080
000000<0/K;H0000002a/MP00`2a/MP200000080/;3H00<000000:n_f02_[mP00P2_[mP300000080
[ZkH00<000000:f]f02][MP00P2][MP300000080[:gH00D000000:^/f02[[=P0ZjcH000000030:Z[
f0<000000`2XZmT00`000000Yj[I0:NZf@020:NZf@<0000000<0YZ[J0000002UZMX00`2UZMX01@00
0000XjWJ0:>YfP2SZMX0000000<0XZSK100000020:6Yg0050000002OZMd0WjWM09nYg@0000000`2N
ZMh800000080W:_Q00D0000009^/h`2K[><0VjcS0000000309V/i0040000002H[ND0V:gU09R]i@@0
00000P2H[nL010000000Uk7Y09Naj@2G/NT200000080fLg>00P000000=[=c03JcL`000000=c=b@00
0000gmC=000000<0hmO;00H000000>SMb0000000l>O30000003/mjX2000000<0U;cc1000000209No
l`030000002G`OD0Ul7e0080Ul7e00@0000009K3mP2F`oH0U/?f1000000209W5mP040000002IaoL0
VLOg09W7m`@000000P2KbOL010000000Vl_h09_;n02KboP400000080WLch1000000209o=n0040000
002OcoP0Wloh09o?n0@000000P2Qd?P400000080X]7h00<000000:;An02RdOP00P0000000`2RdOP0
00000:;Bn0020:;Bn0<00000N`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01L0
oooo001i0?ooo`800000I03oool2000000D0_lCN00<000000;k3gP2n`mh00`2n`mh2000000@0_/?N
0P0000040;g2g@8000001@2m`Md2000000@0_<7M0P0000040;c0g@8000000`2k_md00`000000^kkL
0;^ng002000000040;Zng0000000^[gL0;Zmg080000000@0^[gL000000000000^[cL0P0000000`2i
_=`000000;Vkf`040;Vkf`030000002h^]/0^;[K0080^;[K00<000000;RjfP2h^]X00P000000102g
^M/000000;NifP2g^MX2000000030;Jhf`000000][SJ00<0][SJ0P0000030;FgfP030000002e]]T0
]KKI0080000000<0];KJ0000002d]=T00`2d]=T300000080/kCI00@000000;>cf02c/mP0/k?H0`00
00030;6bf0030000002a/MP0/K7H0080000000<0/;7H0000002`/=P00P2`/=P01P000000[joH0000
00000000[ZoH000000<0[ZkH00<000000:f]f02][MP00`0000040:b]f0040000002[[=P0ZjcH0:^/
f0@000000P2Y[=T01@000000Z:_I0:R[f@2XZmT000000080Yj[I0`0000040:JZfP050000002UZMX0
YJWJ0:FYfP0000000`2SZMX400000080XZ[L00D000000:6Yg02QZM`0XJWL0000000309nYg@@00000
0P2OZml01@000000WJ_P09f[h02MZn00000000<0W:_Q00@0000009^/h`2K[><0VjcS2000000209V_
iP050000002H[nL0V:oW09R_i`0000000`2G/NT2000000050=W=cP000000f/g<0=[=c00000000`3L
cLT01@000000hmO;0>?Gb`3Sel/000000080j=g800<000000?3W``0000001P00000209NmlP040000
002G_o<0Ukoc09Nol`T000000P2I`oD010000000VLGf09W5mP2IaOH400000080VlOf00@0000009_9
m`2KbOL0VlWg1000000209g;m`@000000P2Oc?L010000000Wlgh09o=n02OcOP400000080XLoh1000
00020:;@n0040000002RdOP0X]7h0:;An0<000000`2RdOP4000007h0oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`0G0?ooo`00N`3oool00`000000oooo0?ooo`1R0?ooo`8000000P2o
a=h2000000040;k4gP000000_/?N0;k3gP80000000@0_/?N0000002n`mh0_/?N0P000000102m`mh0
00000000002m`]d2000000030;g2gP00000000000080_L7M0P000000102l`Mh000000;c1g@2l`Md2
000000030;c0g@000000_<3M0080_<3M00<000000;^og@000000102k_md2000000@0^[kL0P000005
0;Zmg0030000002i_=`0^KcL0080^KcL00<000000;Vkf`2i^m/00P0000000`2h^m`000000;Rjf`03
0;Rjf`800000102g^M/2000000<0][SK00<000000;JhfP2f^=X00P0000000`2f]mX000000;FgfP03
0;FgfP8000000`2d]]X00`000000];CI0;Bdf@02000000030;>dfP000000/kCI00<0/kCI0`000002
0;:cf@030000002a/]P0/K;H0080/K;H0P0000030;2af0040000002`/=P0/;3H0;2`f0800000102^
[mP00`000000[ZkH0:j^f003000000<0[JkH00<000000:b]f02/[MP00P2/[MP300000080ZZcI00D0
00000:V/f@2Y[=T0ZJcI000000030:R[f@<000000`2WZmX01@000000YZ[J0:JZfP2VZ]X0000000<0
YJWJ100000020:BZf`050000002RZ]`0XZ[L0::Zg00000000`2QZM`400000080X:_N00D0000009n[
g`2OZml0Wj_O0000000309f[h0040000002LZn40W:_Q09b[h@@000000P2L[N<01@000000VjkT09^^
i02K[^@0000000<0VJoV00@0000009R_i`2H[nL0V:oW1000000209Vbj@050000003JcL`0f/g<0=[=
c00000000P3LcLT030000000glc6000000000000hmO;0000003XgLP000000?3W``000000k?NZ0000
00<0V;_`00@0000009NmlP2G_O80Ukgb1000000209W0lP040000002I`_@0VL;d09W2m0T000000P2K
a_D010000000VlOf09_7mP2KaoH400000080WLWf1000000209o;mP040000002Oc?L0Wlcg09o<m`@0
00000P2Qc_L400000080X/og00@000000:;@n02Rd?P0X]3h100000020:;An0@00000P`3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo01L0oooo001l0?ooo`030000003oool0oooo06<0
oooo0P0000040;k4gP800000102n`mh2000000D0_L?N00<000000;g2gP2m`]h00`2m`]h2000000@0
_<7N0P0000030;c0g@040000002l`=d0000000000080^koM00<000000;^og@2k_md00P000000102j
_]d000000000002j_]`2000000030;Zng@00000000000080^[gL0P0000000`2i_M`000000;Vlg003
0;Vlg08000000`2h^m`00`000000^;[K0;Rjf`02000000040;Njf`000000]kWK0;Nif`80000000<0
]kWK0000002f^=/00`2f^=/2000000<0][OJ00<000000;FgfP2e]mX00P0000000`2d]]X000000;Bf
fP030;BffP8000000`2c]=X00`000000/kCI0;>df@02000000030;:cf@000000/[?I0080/[?I00<0
00000;6bf02a/]P00P0000000`2a/]T000000;2af0030;2af0<000000P2_/=T00`000000[ZoH0:j_
f0020:j_f08000000P2][mT01@000000[JkH0:f^f02][]P000000080[:gH0P0000000`2[[MT00000
0:Z/f@020:Z/f@040000002Y[=T0ZJcI0:V/f@@000000P2XZmT01@000000Yj_J0:N[fP2WZmX00000
00<0YZ[J100000020:F[f`050000002TZ]/0Y:[K0:BZf`0000000`2RZ]`400000080XZ_M00D00000
0:2[gP2PZmh0X:_N0000000309n[g`P000000P2M[N801@000000W:gS09b]h`2L[N<0000000<0VjkT
2000000209Zai`040000002I/^T0VK;Y09Vbj@80000000<0f/g<0000003LcLT00P3LcLT01@000000
glc60=o<aP3Oc<H000000080j=g800<000000?3W``0000002@00000209Znl@040000002I`?80VL3b
09W0lP@000000P2K`_<010000000VlCd09_4m02Ka?@90000000309g8mP000000000000<000000P2O
bOH010000000Wl_f09o;mP2OboH400000080XLcg100000020:;>m`040000002RcoL0X/og0:;?m`@0
000000<0Xm3g0000000000000`0000280?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool05`3oool007d0oooo00<000000?ooo`3oool0H`3oool01`000000_/CN0;k4gP2na=h000000;k4
gP0000000P2n`mh2000000030;k3gP00000000000080_L?N0P000000102m`mh000000;g2gP2m`]h2
000000030;c2gP000000_<7N0080_<7N00<000000;c1gP000000102l`=d00`000000^koM0;^og@03
0;^og@8000001@2j_]d00`000000^[kM0;Zng@030;Zng@8000000`2i_M`00`000000^KcL0;Vlg002
000000030;Rlg0000000^;_L0080^;_L0`0000040;Njf`8000000`2g^M/00`000000][SK0;Jhf`02
000000030;Jhf`000000][OJ00<0][OJ0P0000030;BffP030000002d]]X0];KJ0080000000<0/kGJ
0000002c]=X00`2c]=X2000000<0/[?I00@000000;:cf@2b/mT0/[?I0`0000030;6bf@030000002`
/MP0/;7H0080000000<0/;7I0000002_/=T00P2_/=T00`000000[ZoH0:j_f002000000030:j_f@00
0000[JoI0080[JoI00<000000:f^f02][]P00`0000030:^]f@040000002Z[=T0ZZcI0:Z/f@@00000
0P2Y[=T01@000000Z:_I0:R[f@2XZmT0000000<0Yj_J100000020:J[f`050000002UZm/0YJ_K0:F[
f`0000000`2TZ]/400000080Xj_M00D000000::[g@2RZmd0XZ_M000000030:2[gP@000000P2P[=l0
1@000000WZcQ09j/h@2N[>40000000<0WJgR00@0000009b]h`2L[N<0W:gS1000000209b_i@050000
002K/>H0Vk3V09^`iP0000000`2J/NL010000000VK;Y09Vbj@2I/^T3000000<0g<g900H000000=o<
aP3Oc<H0glc60000003Rc<8600000080V[[^00D0000009Zll02J_?00V[c`0000000309Znl@@00000
0P2L`?8010000000Vl;c09_2l`2K`_<400000080WLCc00@0000009g6m02Ma_@0WLKd1000000209o8
m@040000002ObOH0WlWf09o9mPD000000P2QboH00`000000XLcg0:7<m`03000000<0X/kg1@000000
0`2ScoL0000000000002000008h0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0G
0?ooo`00OP3oool2000006<0oooo00@000000;k4gP00000000000`2na=h2000000D0_/?N0P000004
0;g3gP8000000`2l`]h010000000_<7N0000000000020;c1gP030000002l`=d0_<3M0080000000@0
^l3N000000000000^koM0P0000000`2k_md00000000000020;Zng@80000000<0^[kM0000002j_]d0
0P2j_]d00`000000^KgM000000040;Vmg0800000102h_=`300000080^;_L00<000000;Njf`2g^]/0
0P0000000`2g^]`000000;Nif`030;Nif`8000000`2f^=/00`000000][OJ0;JgfP02000000030;Fg
f`000000];KJ00<0];KJ0P0000030;>efP030000002c]=X0/kCJ0080000000<0/[CJ0000002b/mT0
0`2b/mT300000080/K?I00<000000;6bf@2a/]T00P2a/]T2000000<0/;7I00@000000:n`f@2_/=T0
[k3I0`0000030:j_f@040000002][mT0[JoI0:f_f@<000000P2/[]T00`000000ZjgI0:^]f@020:^]
f@<000000P2Z[MT01@000000ZJcI0:V/f@2Y[=T0000000<0Z:_I100000020:N/fP050000002VZm/0
YZ_K0:J[f`0000000`2UZm/400000080Y:_L00D000000:>[g@2SZmd0Xj_M000000030::[g@@00000
0P2Q[=l01@000000X:cO0:2/g`2P[=l0000000<0WZcQ2000000209f^h`050000002L[nD0W:oU09b_
i@0000000`2K/>H800000080Vk?Y0P0000000`3LcLT000000=o<aP020=o<aP040000003Rc<80h/c2
00000080i/^k0P00000309ZjkP040000002J_?00V[c`09Zll0@000000P2L_o0010000000W<3b09c0
lP2L`?8400000080W/?b00@0000009g4l`2Ma?<0WLCc1000000209o6m0040000002Ob?D0WlSe09o8
m@@000000P2Qb_D010000000XL_f0:7;mP2QboH400000080X/gf100000030:?>mP<00000TP3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01L0oooo00200?ooo`030000003oool0oooo
0640oooo00<000000;k4gP2na=h00P2na=h00`000000_/CO000000040;k3gP030000002m`ml00000
00<0_L?N00<000000;c2gP000000102l`]h00`000000_<7N0;c1gP030;c1gP8000001@2k`=h00`00
0000^koM0;^og@030;^og@8000000`2j_]d010000000^[kM0000000000020;Vmg@030000002i_M`0
^KgL0080000000@0^;gM0000002h_=`0^;cL0P0000000`2h_=`000000;Rkg0030;Rkg08000000`2g
^]`00`000000]kWK0;Nif`02000000030;Jig0000000][SK00<0][SK0P0000030;Fgf`030000002d
]]X0];KJ0080000000<0];KK0000002c]MX00`2c]MX2000000<0/[CJ00<000000;:cf@2b/mT00P00
00000`2a/mX000000;6cf@020;6cf@030000002a/]T0/K;I0080000000<0/;;J0000002`/MT00`2`
/MT300000080[[3I00<000000:j_f@2^[mT00P2^[mT300000080[JoI00H000000:b^f@2/[]T0[:kI
0000002[[MT2000000030:^^fP000000ZZgI0080ZZgI00@000000:V/f@2Y[=T0ZJcI100000020:R/
fP050000002W[=X0YjcJ0:N/fP0000000`2VZm/400000080YJ_L00D000000:B[g02TZm`0Y:_L0000
00030:>[g@@000000P2R[=h01@000000XJcO0:6/g`2Q[=l0000000<0X:cO1000000209n]h@050000
002N[^80WZkR09j^hP0000000`2M[^<010000000W:oU09b_i@2L[nD400000080WK7V00D0000009bb
i`2L/^L0W;;W0000000309^cj@80000000D0g<g90000003Oc<H0glc6000000030>;<`P040000003V
bk/000000>_9/PP000000P2L_Nl010000000W;o`09bol02L_o0400000080W/7a00@0000009k3lP2N
`o80W/?b1000000209o5l`040000002Oa_@0WlKd09o6m0@000000P2Qb?@010000000XL[e0:7:m@2Q
b_D400000080X/_e100000020:?=mP030000002Sc_H0Xlkf00800000UP3oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo01L0oooo00210?ooo`030000003oool0oooo0640oooo0P000000
0`2na=h000000;k4g`020;k4g`040000002n`mh0000000000080_L?O00@000000;g3gP0000000000
0P2l`]h00`000000_<;N0;c2gP02000000040;c2gP00000000000;c1gP80000000<0^l7N00000000
00000P2k`=h2000000040;_0gP000000^koM0;^og@80000000<0^[oN0000002j_]d00`2j_]d00`00
0000^KgM0;Vmg@030;Vmg@800000102h_Md2000000<0^;cL00<000000;Rkg02h^m`00P0000000`2g
^m`000000;Njg0030;Njg08000000`2f^M`00`000000][SK0;Jhf`02000000030;Fhf`000000]KOK
0080]KOK0`0000030;Bff`030000002c]MX0/kGJ0080000000<0/kGK0000002b]=X00`2b]=X20000
00<0/K?J00@000000;6cf@2a/mT0/K?I0`0000030;2bfP030000002`/MT0/;7I0080000000<0[k7J
0000002^/=T00P2^/=T00`000000[ZoI0:j_f@02000000030:f`fP000000[JoI0080[JoI00@00000
0:b^f@2/[]T0[:kI0P0000040:^^fP030000002Z[MT0ZZgI00@000000P2Y[MX01@000000Z:cJ0:R/
fP2X[=X0000000<0YjcJ100000020:J/f`050000002UZm`0YJ_L0:F[g00000000`2TZm`400000080
XjcM00D000000::/gP2R[=h0XZcN000000030:6/g`@000000P2Q[N001@000000WjgQ09n]h@2O[N40
000000<0WZkR2000000209j`i@050000002M/NH0WK7V09faiP0000000`2L/^L400000080WKCX00D0
00000=o<aP3Oc<H0glc6000000020>;<`P080000003Vbk/0i/^k0000003[bK80000009fjk0000003
09fkkP040000002L_Nl0W;g_09bmk`@000000P2N_o0010000000W/7a09k1l@2N`O4400000080X<?b
00@0000009o5l`2OaO<0WlGc100000020:77l`040000002Qb?@0XLSd0:78m0@000000P2Rb_@40000
0080Y<ce00@000000:?=mP2ScOH0Xlgf0`00002I0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool05P3oool00880oooo00<000000?ooo`3oool0HP3oool00`000000_/CO0;k4g`030;k4
g`030000002m`ml0_L?O00<0_L?O00<000000;c2gP2l`]h00`2l`]h2000000D0_<;N00<000000;_1
gP2k`Mh00`2k`Mh2000000@0^l3N0P0000030;ZogP030000002j_]d0^[kM0080000000@0^KkM0000
002i_Md0^KgM0P000000102i_Md000000;Rmg@2h_Md2000000030;Rlg@000000^;cL00<0^;cL0P00
00030;Nkg0030000002g^]`0]k[L0080000000<0][[L0000002f^M`00P2f^M`3000000@0]KSK0`00
00020;Bgf`040000002d]]/0];KK0;Bff`<000000`2c]M/00`000000/[CJ0;:dfP02000000030;:d
fP000000/K?J00<0/K?J0`0000020;2cfP040000002`/]X0/;;J0;2bfP<000000`2_/MX010000000
[[3I0:j`f@2^/=T3000000<0[K3J00@000000:f_f@2][mT0[JoI100000020:b^fP030000002[[]X0
ZjkJ0080ZjkJ0P0000020:V]fP050000002Y[MX0ZJgJ0:V]fP0000000`2X[=X400000080YjcK00D0
00000:J/f`2V[=/0YZcK000000030:F[g0@000000P2T[=d01@000000XjcM0:>/g@2S[=d0000000<0
XZcN100000020::]g`050000002Q[N00XJgP0:6]h00000000`2O[N4400000080X:oR00D0000009n_
h`2O[n<0WjoS0000000309j`i@P000000P2N/nL010000000WKCX09fdj02M]>P2000000030=o<aP00
0000h/c20080h/c200D000000>K;^`000000jlVb0000000209fjk0040000002M^nh0WK_^09fkkP@0
00000P2N_^l010000000W[o`09jol02N_o0400000080X<;`00@000000:33lP2P`o80X<?b10000002
0:75lP040000002Qao<0XLOc0:77l`@000000P2SbO<400000080Y<_d00@000000:C<m@2Tc?D0Y<ce
1000002L0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool05P3oool008<0oooo0P00
001S0?ooo`030000002na=l0_/CO0080000000@0_LCO0000002m`ml0_L?O0P000000102m`ml00000
0000002l`]h2000000030;c2g`00000000000080_<;N0P000000102k`]l000000;_1gP2k`Mh20000
00040;_1gP000000^l3N0;_0gP80000000<0^/3N0000002j_mh00`2j_mh2000000@0^KkM0P000004
0;Vmg@8000000`2h_=d00`000000^;cL0;Rlg002000000030;Nkg@00000000000080]k_L0`000004
0;Jjg0<000000P2e^M`2000000050;Fhf`00000000000;Bhg0000000102d]m/300000080/kKK00@0
00000;>ef`2c]M/0/kGK0`0000030;:dfP030000002a/mX0/K?J0080000000<0/K?J0000002`/mX0
0`2`/mX300000080[k;J00<000000:nafP2_/MX00P2_/MX300000080[[3J00<000000:f`fP2]/=X0
0P2]/=X300000080[:oJ00D000000:b^fP2/[]X0[:kJ000000020:^^fP80000000<0ZZkJ0000002Y
[MX00P2Y[MX00`000000ZJgJ0:V]fP0400000080Z:gK00D000000:N/f`2W[=/0YjcK000000030:J/
f`@000000P2U[=`01@000000Y:cM0:B/g@2T[=d0000000<0XjcM100000020:>]gP050000002R[Ml0
XZgO0::]g`0000000`2Q[N0400000080XJkQ00D000000:2_hP2P[n80X:oR0000000309n_h`@00000
0P2O/ND01@000000W[;V09jbiP2N/^H0000000<0W[?W1`0000030>;<`P050000003Vbk/0i/^k0000
003[bK801`00000209nlk@040000002N_^l0W[k_09jnk`@000000P2P`>l010000000X<;`0:32l02P
`_0400000080X/Ca00@000000:75lP2QaO80XLGb100000020:?7lP@000000P2TbO<010000000Y<_d
0:C;m02Tbo@400000:40oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0F0?ooo`00
Q@3oool00`000000oooo0?ooo`1Q0?ooo`800000102ma=l2000000D0_L?O00<000000;c2g`2l`]l0
0`2l`]l2000000@0^l;O0P0000040;_1gP8000000`2j`=h00`000000^[oN0;ZogP02000000040;Vo
gP000000^KkM0;Vng@80000000@0^KkN000000000000^KgM0P0000000`2h_Mh000000;Rlg@030;Rl
g@8000001@2g^md200000080][_M0P0000001@2f^]`000000000002e^]`0000000D0]KWL00<00000
0;Bhg02d^=`00P2d^=`01P000000];OK000000000000/kOL000000@0/kKK0`0000020;:ef`040000
002b]=X0/[CJ0;:dfP<000000`2a/mX00`000000/;?J0;2cfP02000000030;2cf`000000[k;J0080
[k;J00<000000:nafP2_/MX00P0000000`2^/MX000000:j`fP020:j`fP030000002]/=X0[K3J0080
000000<0[K3J0000002/[mX00P2/[mX010000000[:kJ0:b^fP2/[]X3000000<0ZZkJ00@000000:V]
fP2Y[MX0ZJgJ0`0000020:R]f`050000002X[M/0Z:gK0:R]f`0000000`2W[=/400000080YZcL00D0
00000:F/g02U[=`0YJcL000000030:B/g@@000000P2T[Mh01@000000XjgN0:>]gP2S[Mh0000000<0
XZgO100000020:6^h0050000002Q[^40XJkQ0:6^h@0000000`2P[n8400000080X;7T00D0000009na
i@2O/ND0Wk7U0000000309jbiP@000000P2O]>L010000000WkKX09nfj02O]^P3000000060>;<`P00
0000i/^k0>K;^`000000jlVb0P00000309njk0040000002O_>d0Wkc]09nlk@@000000P2P_^h01000
0000X<3_0:30k`2P`>l400000080X/;`00@000000:;4l@2Ra?40X/Ca100000020:?6l@030000002S
ao80XlOb0080000000<0Y<Sb0000002TbO<00P2TbO<400000:H0oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`0F0?ooo`00QP3oool00`000000oooo0?ooo`1Q0?ooo`030000002ma=l0
_LCO0080000000<0_LCO0000000000000P2m`ml2000000040;c3g`000000_<;O0;c2g`80000000@0
^l;O0000002k`]l0^l;O0P000000102k`Ml000000;_1gP2k`Mh2000000030;[1g`000000^/3N00<0
^/3N0P0000040;VogP8000001@2i_]h00`000000^;gN0;RmgP020;RmgP050000002h_=d000000000
002g_=d00P0000020;Nkg@80000000<0][_M0000002f^md00`2f^md00`000000]K[L0;Fjg0020;Fj
g0030000002e^M`0]KWL0080000000<0];WL0000002d^=`00`2d^=`00`000000/kOL0;>gg0020;>g
g0060000002c]]/000000000002b]]`00000102b]M/300000080/KCK00<000000;6cfP2a/mX00P2a
/mX2000000<0/;?K00@000000:nbfP2_/]X0[k;J0`0000030:jafP040000002^/=X0[[3J0:j`fP<0
00000`2]/=X010000000[:oJ0:b_fP2/[mX400000080ZjoJ00<000000:Z^fP2Z[]X00P2Z[]X30000
0080ZJkK00D000000:R]f`2X[M/0Z:gK000000020:R]f`@000000P2W[M`01@000000YZcL0:J/g02V
[=`0000000<0YJcL100000020:B]g@050000002T[Mh0Y:gN0:B]gP0000000`2S[Mh400000080XZkO
00D000000:6^h02Q[^00XJkP000000030:6^h@@000000P2Q/>801@000000X;7T0:2ai02P/N@00000
00<0Wk7U100000020:2ciP050000002O]>L0WkCW09ndi`0000000`2O]^P400000080i/^k00<00000
0>_9/P3[bK800P00000309njk0@000000P2Q_Nd010000000X;k^0:2nkP2P_^h400000080X/7_00@0
00000:;2l02R`_00X/;`100000020:?4l0040000002Sa_40XlKa0:?6l@<000000`2Tb?8400000:/0
oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0F0?ooo`00Q`3oool00`000000oooo
0?ooo`1Q0?ooo`8000001@2ma=l2000000@0_<?O0P0000040;_2g`800000102k`Ml2000000<0^/7O
00<000000;[0gP2j`=h00P000000102i`=l000000000002i_mh2000000030;VogP00000000000080
^KkN0P0000000`2h_]h000000;RmgP030;RmgP030000002g_=d0]kcM00<0]kcM0P0000030;Jkg@03
0000002f^md0][_M0080000000<0][[M0000002e^]`00`2e^]`2000000<0];WL00<000000;Bhg02d
^=`00P0000000`2c^=`000000;>gg0030;>gg0030000002b]]`0/[KL0080/[KL00H000000;:ef`00
000000000;6ef`0000030;6df`030000002a/mX0/K?J0080000000<0/;?K0000002`/m/00`2`/m/3
00000080[k;K00<000000:jafP2^/MX00P2^/MX300000080[K7J00<000000:f`fP2]/=X00P2]/=X3
00000080[:oK00D000000:^_fP2[[mX0ZjoJ000000020:Z^fP80000000<0ZZkK0000002Y[]/00P2Y
[]/010000000Z:gK0:R]f`2X[M/300000080YjgK00D000000:N]g02W[M`0YjgL000000030:J/g0@0
00000P2U[Md01@000000Y:gM0:B]g@2T[Md0000000<0Y:gN100000020:>^g`050000002R[]l0XZkO
0::^g`0000000`2Q[^0400000080XZoR00D000000:6`hP2Q/>80XK3R000000030:2ai0@000000P2Q
/^D01@000000X;?V0:2ciP2P/nH0000000<0WkCW100000020:6fj0060000002P^>T000000>K;^`00
0000jlVb1P0000020:6kk0040000002Q_Nd0XKg]0:6mk@@000000P2R_nd010000000X/7_0:;1k`2R
`Nl400000080Y<?_00@000000:?4l02Sa?00XlC`100000020:G6l@@00000[`3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo01H0oooo00280?ooo`800000HP3oool200000080_LCO0P00
0000102la>0000000;c3g`2l`ml2000000040;c3g`000000^l;O0;_2g`80000000@0^l;O0000002k
`Ml0^l7O0P0000000`2j`Ml00000000000020;[1g`<000001@2i`=l00`000000^KoN0;VogP030;Vo
gP8000000`2h_]h00`000000^;gN0;RmgP02000000040;NmgP000000]kcM0;Nlg@80000000<0]kcN
0000002f^md00`2f^md2000000<0][[M00<000000;Fjg02e^]`00P0000000`2e^Md000000;Big002
0;Big0<000000`2c^=`00`000000/kOL0;>gg002000000030;>gg0000000/[KL00<0/[KL00<00000
0;6ef`2a]M/00P2a]M/00`000000/KCK0;6df`03000000<0/;?K00<000000;2cf`2`/m/00P000000
0`2_/m/000000:nbf`020:nbf`030000002^/MX0[[7J0080000000<0[[7K0000002]/MX00P2]/MX0
0`000000[K3J0:f`fP02000000030:b`f`000000[:oK0080[:oK00@000000:^_fP2[[mX0ZjoJ0`00
00030:Z^f`040000002Y[]/0ZJkK0:V^f`@000000P2X[]/01@000000YjgK0:N]f`2W[M/000000080
YjgL100000020:J]g0050000002U[Md0YJgM0:F]g@0000000`2T[Md400000080Y:gN00D000000:>^
g`2S[]l0XjkO000000030::^g`@000000P2R[n401@000000XZoR0::_hP2R[n80000000<0XK3R1000
00020:6ai0050000002Q/^D0XK;U0:6bi@0000000`2P/nH400000080XKGW00@000000:6fj02Q]^P0
XKKX100000020>_9/P050000002Q^^X0XK[Z0:6jjP0000000`2Q^n`400000080Xkg/00@000000::o
k@2R_nd0X[o]100000020:C1kP040000002T`nl0Y<?_0:C3k`@000000P2UaO000`000000YLKa0:G6
l@0200000;<0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0F0?ooo`00RP3oool0
0`000000oooo0?ooo`1Q0?ooo`800000102la>02000000@0_<?O0P0000040;_2g`8000001@2j`Ml2
00000080^/7O0P0000020;W0g`80000000@0^KoO0000002i_mh0^KoN0P0000000`2h_ml000000;Rn
gP030;RngP800000102g_Mh2000000<0]kcN00<000000;Jkg@2f^md00P0000000`2f^mh000000;Jj
g@020;Jjg@<00000102e^Md300000080];WM00@000000;>hg02c^=`0/kSL0`0000030;>gg0030000
002b]]`0/[KL0080000000<0/[KL0000002a]M/00`2a]M/200000080/;CK0P0000020;2cf`<00000
0`2_/m/010000000[k;K0:nbf`2_/]/3000000<0[[7K00@000000:fafP2]/MX0[K7J0`0000030:b`
f`040000002/[m/0[:oK0:b_f`@000000P2[[m/00`000000ZZkK0:Z^f`020:Z^f`<000000P2Y[]/0
1@000000Z:kK0:R^f`2X[]/0000000<0YjgK0`0000020:N]g0050000002V[M`0YZgL0:J]g0000000
0`2U[Md400000080YJkN00D000000:B]gP2T[Mh0Y:gN000000030:>^g`@000000P2S[n0010000000
XZoQ0::_h@2R[n4800000080X[7S00@000000:6ai02Q/N@0XK7T200000020:6diP040000002Q]NL0
XKGW0:6ei`@000000P2R]nP200000080jlVb00@000000:6jjP2Q^^X0XK[Z100000020:>lj`040000
002S_N`0Xkg/0:>mk0@000000P2T_nd010000000Y<7^0:C1kP2T`Nh400000080YL?_00@000000:G5
l02UaO00YLG`0`00002f0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool05@3oool0
08/0oooo00<000000?ooo`3oool0H@3oool00`000000_<CP0;c4h002000000040;c4h0000000_<?O
0;c3g`80000000@0^l?P000000000000^l;O0P0000000`2j`^000000000000020;[1g`80000000<0
^/7P0000002j`Ml00`2j`Ml2000000@0^KoO0P0000030;Rog`030000002h_]h0^;kN0080000000@0
]kkN0000002g_Mh0]kgN0P0000000`2g_Mh000000;NlgP030;NlgP800000102f^mh300000080]K[M
00<000000;Fig@2e^Md00P0000000`2d^Md000000;Big@030;Big@<000000P2c^=d010000000/kOL
0;>gg02c]m`3000000<0/[KL00<000000;6ef`2a]M/00P0000000`2a]M`000000;2df`030;2df`80
00000P2_/m/00`000000[k?K0:ncf`020:ncf`<000000P2^/]/010000000[[7K0:jaf`2^/M/40000
0080[K7K00<000000:b`f`2//=/00P2//=/300000080Zk3K00D000000:^_f`2[[m/0ZjoK00000002
0:Z^f`80000000<0ZJoK0000002Y[]/00P2Y[]/010000000Z:kK0:R^f`2X[]/400000080YjkL00D0
00000:N]g02W[M`0YjgL000000020:J]g0@000000P2U[]d010000000YJkN0:F^gP2U[]h800000080
Y:kO00@000000:>_h02S[n00XjoP100000020:>`h@050000002S/>80Xk3R0:>`hP0000000`2R/N<4
00000080X[;T00D000000::ci@2R/nD0X[?U000000030:6diP@000000P2R]^H01P000000X[OX0::g
j02R]nP000000::ij@H000000P2S^nX010000000Xkc[0:>lj`2S_>/400000080Y;k/00@000000:Bo
k@2T_nd0Y;o]100000020:G2k@040000002U`nl0YL?_0:G3k`@00000^P3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo01D0oooo002<0?ooo`030000003oool0oooo0640oooo0P000004
0;c4h08000001@2k`n000`000000^/;P0;[2h0030;[2h08000000`2j`N000`000000^/7O0;[1g`02
000000040;W0g`000000^KoO0;Vog`80000000<0^;oO0000002h_ml00P2h_ml3000000@0]kkN0P00
00030;NmgP030000002g_=h0]kcN0080000000@0][cN0000002f^mh0][_N0P0000000`2e^mh00000
0;Fjg@030;Fjg@8000000`2d^Md00`000000];WM0;Big@02000000030;>hg@000000/kSM00<0/kSM
0`0000020;:gg0040000002b]]`0/[KL0;:fg0<000000`2a]M`00`000000/;CK0;2df`0200000003
0;2dg0000000[k?K0080[k?K00H000000:ncf`00000000000:jcg00000040:jbf`<000000P2]/M/0
1@000000[K7K0:faf`2]/M/000000080[;3K0P0000000`2//=/000000:^`f`020:^`f`040000002[
[m/0ZjoK0:^_f`<000000`2Y[m/010000000ZJkK0:V^f`2Y[]/400000080Z:kL00D000000:N^g02W
[]`0YjkL000000030:N]g0<000000P2V[]d00`000000YJkM0:F^g@020:F^g@<000000P2U[]h01@00
0000Y:kO0:B^g`2T[]l0000000<0Y:kO100000020:B_h0050000002S/>40Xk3Q0:>`h@0000000`2S
/>8400000080Xk;S00D000000::bi02R/^@0X[;T000000030::ci@@000000P2S]ND01@000000X[KV
0::fiP2R]^H0000000<0X[OX100000020:>ij0040000002S^nX0Xk_Z0:>kjP@000000P2T_N/01000
0000Y;k/0:Bnk02T_^`400000080Y/3/00@000000:G2k@2U`^d0YL;]1000002n0?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool05@3oool008d0oooo00<000000?ooo`3oool0H@3oool2
000000040;c4h000000000000;_4h@8000000P2k`n02000000040;_3h0000000^/;P0;[2h0800000
00<0^/;P0000002j`N000`2j`N02000000@0^L3O0P0000040;Rog`<000000P2h_]l00`000000]kkN
0;NngP02000000030;Nng`000000]kgN0080]kgN0`0000040;JlgP8000000`2e^mh00`000000]K[M
0;Fjg@02000000030;BjgP000000];WM00<0];WM0P0000030;>hg@030000002c^=d0/kSM00800000
00<0/[SM0000002b]m`00`2b]m`300000080/KKL00@000000;6eg02a]M`0/KGL0`0000030;2dg004
0000002_/m/0[k?K0:ncf`800000102^/m`01P000000[[;K000000000000[[;L000000<0[K7K00@0
00000:faf`2]/M/0[K7K0`0000030:b`f`040000002[/=/0Zk3K0:^`f`@000000P2Z[m/010000000
ZJoK0:V_f`2Y[m/400000080ZJoL00D000000:R^g02X[]`0Z:kL000000030:N^g0@000000P2W[]d0
10000000YZkM0:J^g@0000020:F^g@80000000<0YZkN0000002U[]h00P2U[]h010000000Y:kO0:B^
g`2T[]l400000080Y:oO00D000000:B_h02T[n00Y:oP000000030:>`h@@000000P2S/N801@000000
Xk;S0:>bh`2S/^<0000000<0X[;T100000020:>di0050000002S]ND0XkGU0:>ei@0000000`2R]^H4
00000080Y;SW00@000000:>ij02S^NP0XkWX100000020:Fkj@040000002T_N/0Y;g[0:Bmj`@00000
0P2V_^/010000000Y/3/0:K0k02V`>`400000<<0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0E0?ooo`00SP3oool2000006<0oooo00<000000;_4h@2ka>400`2ka>42000000@0^l?P
0P0000030;[2h0030000002j`N00^/7P0080000000@0^L7P0000002i`=l0^L3O0P000000102h`>00
00000;Rog`2h_ml2000000030;Roh0000000^;kO00<0^;kO0P0000040;Nng`<000000P2f_Ml00`00
0000][cN0;JlgP02000000030;FlgP000000]K_N00<0]K_N0P0000030;BjgP030000002d^Md0];WM
0080000000<0/kWN0000002c^=d00`2c^=d2000000<0/[SM00<000000;:gg02b]m`00P0000000`2a
]md000000;6fg0030;6fg0<000000P2`]M`00`000000/;CL0;2dg0020;2dg0<000000P2_/m`01000
0000[[?L0:jcg02^/m`2000000@0[[;L00<000000:faf`2]/M/0100000020:bag0040000002//=/0
[;3K0:b`f`@000000P2[/=`00`000000ZZoK0:Z_f`020:Z_f`<000000P2Y[m`01@000000ZJoL0:V_
g02Y[m`0000000<0Z:kL100000020:N^g0050000002W[]d0YjkM0:N^g@0000000P2V[]d3000000<0
YZkN00@000000:F^gP2U[]h0YJkN100000020:F_g`050000002T[ml0Y:oO0:B_g`0000000`2T[n04
00000080Y;3Q00D000000:>ahP2S/N80Xk7R000000030:>bh`@000000P2T/n<01@000000XkCT0:>d
i02S]>@0000000<0XkGU100000020:BgiP040000002T^>L0Y;SW0:Bhi`@000000P2U^^P010000000
YK_Y0:Fkj@2U^nT400000080Y[gZ00@000000:Jnj`2V_^/0Y[k[100000380?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool05@3oool00900oooo00<000000?ooo`3oool0H@3oool00`00
0000^lCQ0;_4h@02000000040;_3h@000000^l?P0;_3h080000000<0^/?Q0000002j`^000`2j`^02
000000@0^L7P0P0000040;S0h0800000102h_n001@000000^;kO000000000000]kkO0080000000D0
]kkO000000000000][gO000000040;Jmg`800000102e_=h01@000000]K_N000000000000];_N0080
00000P2d^]h3000000<0/kWN00<000000;>hg@2c^=d00P0000000`2b^=d000000;:hg@020;:hg@<0
00000`2a]md00`000000/KKL0;6fg002000000030;2fg@000000/;GL0080/;GL00<000000;2dg02`
]=`00P0000000`2_]=`000000:ncg0030:ncg0<000000P2^/m`00`000000[[;L0:jbg0020:jbg080
00000P2]/]`00`000000[;7L0:bag0020:bag0<000000P2[/=`01@000000Zk3L0:^`g02[/=`00000
0080ZZoK0P0000000`2Z/=`000000:V_g0020:V_g0040000002Y[m`0ZJoL0:V_g0@000000P2X[m`0
1@000000YjkL0:N^g02W[]`0000000<0YjkM0`0000020:J^g@8000000P2V[]h400000080YJoN00D0
00000:F_g`2U[ml0YJoO000000030:B_g`@000000P2T/>001@000000Y;3Q0:B`h@2T/>40000000<0
Xk7R100000020:Bbh`050000002T/n<0Y;?S0:Bch`0000000`2S]>@400000080Y;KU00@000000:Bg
iP2T]nH0Y;OV100000020:Fii`040000002U^^P0YK[X0:Fjj0@000000P2V_>T010000000Y[gZ0:Jm
jP2V_NX400000<`0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0E0?ooo`00T@3o
ool00`000000oooo0?ooo`1Q0?ooo`800000102k`n42000000<0^/?Q00<000000;[2h02j`^000P00
0000102i`^4000000000002i`N02000000040;W1h000000000000;S0h080000000<0^<3P0000002h
_n00102h_n000`000000]kkO0;Nng`030;Nng`030000002f_Ml0][gO0080][gO00D000000;Jmg`00
000000000;Fmg`02000000<0]KcN0P0000050;BkgP8000000P2c^]h200000080/kWN0`0000040;:h
g@<000000P2a]md010000000/KOM0;6gg@2a]md3000000<0/;KM00@000000;2eg02`]M`0/;GL0`00
00030:ndg0030000002_/m`0[k?L0080000000<0[[?L0000002^/m`00P2^/m`00`000000[[;L0:jb
g002000000030:fbg0000000[K;L0080[K;L00H000000:bag000000000000:bag00000030:^`g004
0000002[/=`0Zk3L0:^`g0<000000`2Z/=`010000000ZJoL0:V_g02Y[m`400000080Z:oL00D00000
0:R_g02X[m`0Z:oL000000030:N^g0@000000P2W[]d010000000YZkM0:J^g@2V[]d300000080YZoN
00D000000:F_gP2U[mh0YJoN000000030:F_g`@000000P2U/=l01@000000Y;3P0:B`h02T/>000000
00<0Y;3Q100000020:BbhP050000002T/^<0Y;;S0:Bbh`0000000`2T/n<400000080Y;CT00@00000
0:Bfi@2T]^D0Y;KU100000020:FgiP040000002U^NL0YKWW0:Fii`@000000P2V^^P010000000Y[cY
0:Jlj@2V_>T400000=40oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0E0?ooo`00
TP3oool00`000000oooo0?ooo`1Q0?ooo`030000002k`n40^l?Q0080000000<0^/?Q0000002j`n40
0`2j`n42000000D0^L;Q00<000000;W1h02i`N000`2i`N000`000000^<3P0;S0h0020;S0h0030000
002h_n00^;oP0080000000@0]koP0000002g_]l0]kkO0P0000000`2f_^0000000;Jmg`020;Jmg`80
00001@2e_Ml300000080];cO00<000000;BkgP2d^mh00P0000000`2d^ml000000;>jgP030;>jgP80
00000P2c^Mh2000000050;:hg@00000000000;:hgP000000102a]md300000080/KOM00<000000;2f
g@2`]]d00P2`]]d300000080[kGM00@000000:ndg02_]=`0[kCL0`0000030:jcg0040000002^/m`0
[[?L0:jcg0<000000`2]/]`010000000[K;L0:fbg02]/]`2000000<0[;7L00@000000:^`g02[/=`0
Zk3L100000020:Z`g0030000002Z/=`0Z[3L0080Z[3L0`0000020:V_g0050000002X[m`0Z:oL0:R_
g00000000`2X[m`400000080YjoM00<000000:N^g@2W[]d00P2W[]d300000080YZoN00@000000:J_
gP2V[mh000000`2U[mh400000080YJoO00D000000:F`g`2U/=l0YK3O000000030:B`h0@000000P2U
/N401@000000Y;;R0:BbhP2T/^80000000<0Y;;S100000020:Fdh`040000002T]>@0Y;CT0:Bdi0@0
00000P2U]^D010000000YKOV0:FgiP2U]nH400000080Y[WV00<000000:Jjj02V^^P00P2V^^P30000
0=H0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0D0?ooo`00T`3oool200000680
oooo0P0000040;[3h@050000002j`n4000000000002i`n400P0000020;W2h@80000000@0^L;Q0000
002i`N00^L7P0P0000000`2h`N4000000;S0h0020;S0h0<00000102g_n02000000@0][kP0`000002
0;Jmh0030000002e_Ml0]KgO0080000000<0]KcO0000002d_=l00`2d_=l2000000<0];_O00<00000
0;>jgP2c^]h00P0000000`2c^]l000000;>igP030;>igP030000002b^=h0/[SN0080/[SN00H00000
0;6gg@00000000000;6ggP0000030;6gg@030000002`]]d0/;KM0080000000<0/;KM0000002_]Md0
0`2_]Md300000080[kCM00@000000:jcg02^/m`0[[?L100000020:fcg0040000002]/]`0[K;L0:fb
g0@000000P2//]`010000000[;7L0:bag02//M`300000080Zk7L00D000000:Z`g02Z/=`0Z[3L0000
00020:Z`g080000000<0ZK3L0000002Y[m`00P2Y[m`010000000Z:oL0:R_g02X[m`400000080Z:oM
00D000000:N_g@2W[md0YjoM000000020:N^g@80000000<0YjoM0000002V[mh00P2V[mh00`000000
YZoN0:J_gP0400000080YZoN00D000000:F_g`2U[ml0YJoO000000030:F`g`@000000P2U/N001@00
0000YK7Q0:Fah@2U/N40000000<0Y;;R100000020:FchP040000002U]><0YKCS0:Fdh`@000000P2V
]N@010000000YKKU0:Ffi@2U]^D400000080Y[SU00D000000:JiiP2V^NH0Y[WV000000020:Jjj080
0000fP3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01@0oooo002E0?ooo`030000
003oool0oooo0600oooo0P0000030;[3h@8000001@2i`n42000000@0^L;Q0P0000040;S1h@<00000
0P2g`>400`000000]koP0;Noh002000000040;Joh@000000][kP0;Jnh080000000<0][kP0000002f
_N000`2f_N02000000<0]KcO00<000000;Blg`2d_=l00P0000000`2d^ml000000;Bkg`020;Bkg`<0
00000`2c^]l00`000000/kWN0;>igP02000000030;:igP000000/[SN00<0/[SN00<000000;6ggP2a
]mh00P2a]mh00`000000/KOM0;6gg@03000000<0/;KM00<000000:neg@2_]Md00P0000000`2_]Md0
00000:ndg@030:ndg@<000000P2^/md00`000000[K?L0:fcg0020:fcg0<000000P2//]d01P000000
[;;L0:bbg02//]`000000:bag080000000<0Zk7M0000002[/M`00P2[/M`010000000Z[3L0:Z`g02Z
/=`3000000<0ZK3L00@000000:V_g02Y[m`0ZJoL100000020:R_g@050000002X[md0Z:oM0:R_g@00
00000`2W[md3000000<0YjoM00@000000:J_gP2V[mh0YZoN0`0000020:J_gP050000002V[mh0YZoN
0:J_gP0000000`2U[ml400000080YK3P00D000000:Fah02U/N00YK7P000000030:Fah@@000000P2U
/^8010000000YK?R0:FchP2U/n8400000080Y[CS00<000000:Jei02V]N@00P2V]N@300000080YkOT
00D000000:Jhi@2V^>D0Y[SU000000030:JiiP<00000g03oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo01@0oooo002F0?ooo`030000003oool0oooo0640oooo0`0000020;[3hP030000
002i`n40^L?Q0080000000@0^L?R0000002i`^40^L;Q0P000000102h`^4000000;S1h@2h`N420000
00030;O1h@000000]l3Q00<0]l3Q0P0000040;Joh@8000000`2f_^000`000000][gP0;Jmh0020000
00030;Fmh0000000]KcO0080]KcO0`0000040;Bkg`<000000P2c^ml010000000/k[O0;>jg`2c^]l3
000000<0/[WN00<000000;:hgP2b^=h00P0000000`2a^=h000000;6ggP030;6ggP8000000P2`]mh2
00000080/;KM0`0000030:neg@030000002_]=d0[kCM0080000000<0[[CM0000002^/md00P2^/md0
0`000000[K?L0:fcg002000000030:fcg@000000[;;M0080[;;M00@000000:bbg02//]`0[;;L0P00
00040:^ag@030000002[/M`0Zk7L00@000000P2Z/=d010000000ZK3L0:V`g02Y/=`400000080ZK3M
00D000000:R_g@2X[md0Z:oM000000030:R_g@@000000P2W[md010000000YjoM0:N_g@2W[md40000
0080YZoN00D000000:J_gP2V[mh0YZoN000000020:J_gP@000000P2V/=l01@000000YK3P0:F`h02U
/>00000000<0YK7P100000020:Fbh@040000002U/^80YK;R0:FbhP@000000P2V/n801@000000Y[CS
0:Jdh`2V]><000000080Y[GT0P0000000`2W]^@000000:Ngi0020:Ngi0040000002V^>D0Y[SU0:Jh
i@@00000h03oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo01@0oooo002G0?ooo`03
0000003oool0oooo0640oooo00<000000;[3hP2j`n800P2j`n82000000@0^L?R0P0000040;S2h@80
00000`2g`N400`000000]l3Q0;O0h@02000000040;O0h@000000][oQ0;Joh@80000000<0][oQ0000
002f_^000P2f_^03000000@0]KgP0`0000020;Blh0030000002d^ml0];_O0080000000<0/k_P0000
002c^ml00`2c^ml300000080/[[O00@000000;:igP2b^Mh0/[WN0`0000030;6hgP030000002a]mh0
/KON0080000000<0/;ON0000002`]mh00`2`]mh200000080[kKN00@000000:neg@2_]Md0[kGM0`00
00030:jdg@040000002^/md0[[?M0:jcg@<000000`2]/md010000000[;;M0:bbg@2//]d400000080
[;;M00@000000:^ag@2[/Md0Zk7M0`0000020:Zag@030000002Z/=d0Z[3M0080Z[3M0`0000020:V`
g@050000002Y/=d0ZK3M0:V`g@0000000`2X[md400000080Z:oM00@000000:N_g@2W[md0YjoM1000
00020:N_gP050000002V[mh0YZoN0:J_gP0000000`2V[mh300000080Y[3O00D000000:J`g`2V/=l0
Y[3O000000030:F`h0@000000P2V/N0010000000YK;Q0:Fbh@2U/^4400000080Y[?R00D000000:Jc
hP2V/n80Y[?R000000030:Jdh`<000000`2W]^@010000000YkOT0:Ngi02W]n@400000>D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`0D0?ooo`00V03oool200000640oooo00<00000
0;[3hP2j`n800P000000102i`n8000000;W3hP2i`n82000000040;S2hP000000^<;Q0;S2h@800000
00<0]l;R0000002g`N400P2g`N43000000@0]l3Q0P0000040;Joh@<000000P2e_^400`000000]KgP
0;Fmh002000000030;Bmh@000000];cP00<0];cP0P0000030;>kh0030000002c^ml0/k_O00800000
00<0/[_O0000002b^]l00`2b^]l300000080/KWO00@000000;6hgP2a^=h0/KSN0`0000030;2ggP03
0000002`]mh0/;ON0080000000<0[kON0000002_]]h00P2_]]h300000080[[GN00<000000:jdg@2^
]=d00P2^]=d300000080[KCM00@000000:fcg@2]/md0[K?M100000020:bbg@030000002//]d0[;;M
0080[;;M0`0000020:^ag@060000002Z/Md0Z[7M0:Zag@000000Z[3M0P0000000`2Y/=d000000:V`
g@020:V`g@040000002Y/=d0ZK3M0:V`g@@000000P2X[md00`000000Z:oM0:R_g@020:R_g@<00000
0P2W[md01@000000YjoN0:N_gP2W[mh0000000<0YZoN100000020:J`gP040000002V/=l0Y[3O0000
00<0Y[3O100000020:Jah0040000002V/N00Y[7P0:Jah0@000000P2V/^401@000000Y[?R0:JchP2V
/n80000000<0Y[?R100000020:Neh`040000002W]^@0YkKT0:Nfi0@00000jP3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo01@0oooo002J0?ooo`030000003oool0oooo05l0oooo0P00
00040;W3hP800000102h`^82000000@0]l;R0`0000020;O1hP80000000@0]l3Q000000000000]/3Q
0P0000001@2f_n4000000000002e_n40000000@0]KkQ0P0000030;Bmh@030000002d_>00];cP0080
000000<0/kcP0000002c^n000P2c^n03000000<0/[_O00<000000;:jg`2b^]l00P0000000`2a^]l0
00000;6ig`030;6ig`<000000P2`^=l010000000/;ON0;2ggP2`]mh3000000<0[kON00H000000:nf
gP00000000000:jfgP0000030:jegP030000002^]=d0[[CM0080000000<0[KCN0000002]]=d00`2]
]=d300000080[;?M00D000000:bbg@2//]d0[;;M000000020:bbg@80000000<0Zk;M0000002[/Md0
0P2[/Md010000000Z[7M0:Zag@2Z/Md2000000<0ZK3M00@000000:V`g@2Y/=d0ZK3M100000020:V`
g@050000002X[md0Z:oM0:R_g@0000000P2X[md2000000030:R`g@000000YjoM0080YjoM00@00000
0:N_gP2W[mh0YjoN100000020:N`gP050000002V/=h0Y[3N0:J`gP0000000P2V/=l400000080Y[3O
00@000000:Jah02V/N00Y[7P100000020:Jah0050000002V/^40Y[;Q0:Jbh@0000000`2V/n840000
0080YkCR00@000000:Neh`2W]N<0YkGS1000003^0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool0503oool009/0oooo00<000000?ooo`3oool0G`3oool2000000040;W3hP0000000000
0;S3hP80000000@0^<;R000000000000^<;R0P000000102g`^8000000000002g`N82000000<0]l7R
0P0000050;K0h@030000002e_n40]KoQ0080]KoQ00D000000;Fnh@00000000000;Bnh@0200000080
];gQ0`0000040;>lh0<000000P2b^n0010000000/[_O0;:kg`2b^ml3000000<0/K[O00<000000;6i
g`2a^Ml00P0000000`2a^Ml000000;2hg`030;2hg`<000000P2`]ml010000000[kON0:nggP2_]mh2
000000@0[[KN00<000000:jegP2^]Mh00`0000030:fdgP030000002]]=d0[KCM0080000000<0[;CN
0000002//md00P2//md010000000[;;M0:bbg@2//]d3000000<0Zk;M00@000000:^ag@2[/Md0Zk7M
100000020:Zag@030000002Y/=d0ZK3M00@000000P2Y/=d01@000000ZK3M0:V`g@2Y/=d0000000<0
Z:oM0`0000030:R`g@040000002W[md0YjoM0:N_g@@000000P2W[mh01@000000Yk3N0:N`gP2W/=h0
000000<0Y[3N0`0000020:J`g`040000002V/=l0Y[3O0:J`g`@000000P2W/N001@000000Y[7P0:Ja
h02V/N00000000<0Y[;Q100000020:Nch@040000002W]>80YkCR0:NdhP@00000l`3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo01@0oooo002L0?ooo`030000003oool0oooo0600oooo
00<000000;S3hP2h`n800`2h`n800`000000^<;R0;S2hP030;S2hP030000002g`N80]l7R00<0]l7R
0`0000020;K1hP030000002f`>40]/3Q0080000000<0]KoR0000002e_n400P2e_n42000000D0];kQ
0P0000020;>mh@80000000D0/kcP000000000000/kcQ000000040;:kh0<000000P2b^^0010000000
/K[O0;6jg`2a^]l3000000<0/KWO00<000000;2hg`2`^=l00P0000000`2`^=l000000;2gg`030;2g
g`<000000P2_]]l010000000[[KN0:jfgP2^]]h300000080[[GN0P0000020:fdgP<000000`2/]=h0
10000000[;?M0:bcg@2//md400000080Zk;M00@000000:^bg@2[/]d0Zk;M100000020:Zag@030000
002Z/Md0Z[7M0080Z[7M0P0000020:Vag@050000002Y/=d0ZK3M0:V`g@0000000`2Y/=d400000080
Z;3M00@000000:R`g@2X/=d0Z;3M100000020:N`gP050000002W[mh0YjoN0:N_gP0000000`2W/=h4
00000080Yk3N00<000000:J`g`2V/=l0100000020:Nag`050000002W/N00Yk7P0:Nah00000000`2V
/N0400000080Yk;Q00@000000:Nch@2W/n40Yk?Q1000003h0?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool04`3oool009d0oooo0P00001Q0?ooo`030000002h`n80^<?R0080000000@0
^<?S0000002h`^80^<;R0P000000102g`^<000000;O1hP2g`N82000000030;K1h`000000]/7R00<0
]/7R0P0000040;FohP<000000P2d_n800`000000];kQ0;Bnh@02000000030;>nh@000000/kgQ00<0
/kgQ00<000000;>lh@2c_>400P2c_>401P000000/[_P000000000000/[_Q000000@0/[[P0`000002
0;6jh0040000002a^Ml0/KWO0;6ig`<000000`2`^=l00`000000/;OO0;2gg`02000000030:ngg`00
0000[kKO00<0[kKO0`0000020:jfgP040000002^]Mh0[[GN0:jegP<000000P2]]=h200000080[;CN
100000020:bcgP030000002[/]d0Zk;M0080Zk;M0`0000020:^bgP050000002Z/Md0Z[7M0:Zag@00
00000P2Z/Md2000000030:Vag@000000ZK7M0080ZK7M00<000000:V`g@2Y/=d0100000020:R`g@04
0000002X/=d0Z;3M0:R`g@@000000P2X/=h01@000000Yk3N0:N`gP2W/=h0000000<0YjoN10000002
0:N`gP040000002W/=h0Yk3N0:N`gP<000000P2W/=l01@000000Yk7O0:Nag`2W/Ml0000000<0Yk7P
100000020:Nbh0040000002W/^40Yk;Q0:Nbh@@00000o03oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo01<0oooo002O0?ooo`030000003oool0oooo05l0oooo0P0000040;S3h`800000
102g`^<2000000<0]/7S00<000000;K1hP2f`N800P000000102e`>8000000;FohP2e_n8200000003
0;BohP000000];oR00<0];oR0P0000030;>nh@030000002c_N40/kgQ0080000000<0/kgQ0000002c
_>400P2c_>42000000@0/[_Q00D000000;:jh000000000000;6jh00200000080/K[P100000020;2i
h0040000002`^=l0/;SO0;2hg`<000000`2_]ml00`000000[kKO0:nfg`02000000030:jfg`000000
[[KN00<0[[KN0`0000020:fegP030000002]]=h0[KCN0080[KCN0P0000020:bdgP050000002//mh0
[;?N0:bcgP0000000P2[/]d2000000030:^bgP000000Zk;N0080Zk;N00@000000:Zag@2Z/Md0Z[7M
0`0000030:Vag@040000002Y/Md0ZK7M0:Vag@<000000P2Y/=d00`000000Z;3M0:R`g@020:R`g@<0
00000P2X/=h01@000000Z;3N0:R`gP2X/=h0000000<0Yk3N100000020:N`gP030000002W/=h0Yk3N
0080Yk3N0`0000020:N`gP040000002W/=l0Yk3O000000<0Yk7O100000020:Nag`030000002W/^00
Yk;P0080Yk;P0`00003o0?ooo`80oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0C
0?ooo`00X03oool00`000000oooo0?ooo`1O0?ooo`030000002h`n<0^<?S0080000000@0]l?S0000
002g`^<0]l;S0P0000000`2f`^<000000;K1h`020;K1h`<00000102e`>82000000<0];oR00<00000
0;BohP2d_n800P0000000`2d_^8000000;>nh@020;>nh@<00000102c_N4300000080/[cQ00@00000
0;:kh@2b^n40/[_Q0P0000050;6jh08000000P2`^N0200000080/;WP100000020:nhg`040000002_
]ml0[kOO0:ngg`<000000`2^]]l00`000000[[KN0:jfgP02000000030:ffg`000000[KGN0080[KGN
00<000000:fdgP2]]=h00P0000001@2/]=h000000:bdgP2/]=h0000000<0[;?N0`0000030:^bgP04
0000002[/]h0Zk;N0:^bgP@000000P2Z/Mh010000000ZK7M0:Vag@2Y/Md400000080ZK7N00@00000
0:V`g@2Y/=d000000P2X/=d2000000030:R`gP000000Z;3N0080Z;3N00@000000:R`gP2X/=h0Z;3N
100000020:N`gP050000002W/=h0Yk3N0:N`gP0000000P2W/=h2000000030:N`gP000000Yk3N0080
Yk3N00<000000:N`g`2W/=l0100000020:Nag`050000002W/Ml0Yk7O0:Nag`0000000P2W/^020000
0?l0oooo1P3oool01@000000oooo0?ooo`3oool0000001D0oooo002Q0?ooo`030000003oool0oooo
05l0oooo0P0000040;O3h`800000102f`^<300000080]/7S00<000000;G0hP2e`>800P0000000`2d
`><000000;BohP020;BohP<00000102d_^8300000080/kkR00<000000;>mh@2c_N400P0000000`2b
_N8000000;:lh@030;:lh@<000000P2a^n400`000000/K[P0;6jh002000000030;2jh@000000/;WP
00<0/;WP0P0000020:nih08000000P2_^=l400000080[[OO00@000000:jfg`2^]]l0[[KO0`000003
0:ffg`040000002]]Mh0[KGN0:fegP<000000`2/]=h00`000000[;CN0:bdgP0400000080Zk?N00@0
00000:^bgP2[/]h0Zk;N100000020:ZbgP040000002Z/Mh0Z[7N0:ZagP@000000P2Y/Mh01@000000
ZK7N0:VagP2Y/Mh000000080ZK3M0`0000030:R`gP040000002X/=h0Z;3N0:R`gP@000000P2X/=h0
1@000000Yk3N0:N`gP2W/=h0000000<0Yk3N0`0000030:N`gP040000002W/=h0Yk3N0:N`gP<00000
0P2W/Ml01@000000Yk7O0:Nag`2W/Ml0000000<0Yk7O0`00003o0?ooo`T0oooo00@000000?ooo`3o
ool000005@3oool00:80oooo0P00001P0?ooo`80000000@0]l?S000000000000]/?T0P000000102f
`^<000000000002f`^<200000080]/7S0`0000040;C0h`<000000P2d_n<00`000000];kR0;BnhP02
000000030;>nhP000000/kkR0080/kkR0`0000030;:mhP030000002b_>40/[cQ0080000000<0/KcR
0000002a^n400P2a^n43000000<0/;[Q00<000000;2ih02`^N000P0000000`2_^N4000000:nih003
0:nih08000000P2^^>000`000000[[OO0:jgg`020:jgg`<000000P2]]]l010000000[KKO0:ffg`2]
]]l400000080[;GO00@000000:bdgP2/]=h0[;CN0`0000020:^cgP8000000P2[/mh400000080Zk;N
00<000000:ZbgP2Z/]h00P2Z/]h300000080ZK7N00D000000:VagP2Y/Mh0ZK7N000000030:VagP<0
00000P2X/=h010000000Z;3N0:R`gP2X/=h400000080Z;3N00D000000:R`gP2X/=h0Z;3N00000003
0:N`gP@000000P2W/=h010000000Yk3N0:N`gP2W/=h400000080Yk3N00D000000:Nag`2W/Ml0Yk7O
000000020:Nag`@00000o`3oool<0?ooo`040000003oool0oooo000001D0oooo002T0?ooo`030000
003oool0oooo05l0oooo00<000000;K3i02f`n@00`2f`n@00`000000]/;S0;K2h`030;K2h`800000
0P2e`N<2000000040;C0h`00000000000;C0h`8000000P2d_n<3000000@0/kkR0`0000020;:mhP04
0000002b_N80/[gR0;:mhP<00000102a_>8300000080/;_Q00@000000;2jh@2`^^40/;[Q0`000003
0:nih@030000002_^N00[kWP0080000000D0[kWQ0000002^^>00[[SP000000020:jgg`80000000<0
[[OP0000002]]]l00`2]]]l300000080[KKO00<000000:beg`2/]Ml00P2/]Ml300000080[;CO00<0
00000:^cgP2[/mh00P2[/mh200000080Zk?N00D000000:^bgP2[/]h0Zk;N000000020:ZbgP800000
00<0Z[;N0000002Y/Mh00P2Y/Mh010000000ZK7N0:VagP2Y/Mh400000080ZK7N00<000000:R`gP2X
/=h0100000020:R`gP050000002X/=h0Z;3N0:R`gP0000000`2X/=h400000080Yk3N00@000000:N`
gP2W/=h0Yk3N100000020:N`gP050000002W/=h0Yk3N0:N`gP0000000`2W/Ml300000?l0oooo4@3o
ool00`000000oooo0000000E0?ooo`00Y@3oool00`000000oooo0?ooo`1O0?ooo`030000002f`n@0
]/?T0080000000@0]/?T0000002f`^<0]/;S0P0000000`2e`^@000000;G1h`020;G1h`8000001@2d
`><200000080/koS0P0000001@2c_^8000000000002b_^<0000000@0/[gR0`0000020;6mhP030000
002a_>80/KcR0080000000<0/KcR0000002`^n400`2`^n4300000080/;[Q00@000000:nih@2_^N40
[kWQ0`0000030:nih@030000002^^>00[[SP00<000000`2^]n000`000000[KKO0:ffg`0200000003
0:ffh0000000[KKO0080[KKO00<000000:beg`2/]Ml00P0000000`2/]Ml000000:bdg`020:bdg`03
0000002[/mh0Zk?N0080000000D0ZkCO0000002[/mh0Zk?N000000030:^bgP<000000`2Z/]h01000
0000ZK7N0:VagP2Y/Mh400000080ZK7N00<000000:VagP2Y/Mh00P2Y/Mh200000080Z;3N00D00000
0:R`gP2X/=h0Z;3N000000030:R`gP@000000P2X/=h010000000Yk3N0:N`gP2W/=h400000080Yk3N
00D000000:N`gP2W/=h0Yk3N000000030:N`gP@00000o`3ooolE0?ooo`030000003oool0000001@0
oooo002V0?ooo`030000003oool0oooo05l0oooo0P0000040;K3i0800000102e`^@300000080]<7T
00<000000;C0h`2d`><00P0000000`2c`><000000;>oh`030;>oh`030000002b_^<0/[kS0080/[kS
00D000000;:mhP00000000000;6mh`0200000080/KgR0`0000030;6lhP030000002`^n40/;_Q0080
000000<0/;_R0000002`^^400`2`^^4300000080[kWQ00@000000:nih@2_^N40[kWQ0`0000020:jh
h08000000P2^]n03000000<0[KKP00@000000:ffg`2]]]l0[KKO0`0000030:beg`040000002/]=l0
[;CO0:bdg`<000000`2[]=l00`000000Zk?N0:^cgP0400000080Z[;N00@000000:ZbgP2Z/]h0Z[;N
100000020:VagP050000002Y/Mh0ZK7N0:VagP0000000P2Y/Mh2000000030:RagP000000Z;3N0080
Z;3N00<000000:R`gP2X/=h0100000020:R`gP030000002X/=h0Z;3N0080Z;3N0`0000020:N`gP05
0000002W/=h0Yk3N0:N`gP0000000`2W/=h400000?l0oooo6P3oool2000001@0oooo002W0?ooo`03
0000003oool0oooo05l0oooo00<000000;K3i02f`n@00P000000102e`n@000000;G2i02e`^@20000
00030;C2i0000000]<7T00<0]<7T0P0000030;?0h`030000002c_n<0/koS0080000000<0/[oS0000
002b_^<00P2b_^<2000000D0/KgS0P0000020;6lhP8000000P2a_>83000000<0/;_R00<000000;2j
h@2`^^400P0000000`2_^^8000000:nih@030:nih@<000000P2^^>4010000000[[SP0:jhh02^^>03
00000080[KOP0P0000020:ffh0@000000P2/]^0010000000[;GO0:beg`2/]Ml400000080ZkCO00@0
00000:^dg`2[]=l0ZkCO0`0000020:Zcg`8000000P2Z/]h400000080ZK;N00D000000:VagP2Y/Mh0
ZK7N000000030:VagP<000000`2X/Mh010000000Z;3N0:R`gP2X/=h300000080Z;3N00D000000:R`
gP2X/=h0Z;3N000000020:R`gP80000000<0Z;3N0000002W/=h00P2W/=h010000000Yk3N0:N`gP2W
/=h400000?l0oooo7P3oool2000001@0oooo002X0?ooo`800000H03oool2000000@0]L?T0P000003
0;C2i0030000002d`N@0]<7T0080000000<0/l7T0000002c`><00P2c`><3000000@0/[oS0`000002
0;:nh`030000002a_N<0/KgS0080000000<0/KgS0000002a_>800`2a_>8200000080/;cR00@00000
0;2khP2`^n80/;_R0`0000030:njhP030000002_^N40[kWQ0080000000<0[[WQ0000002^^>400`2^
^>4300000080[KSQ00@000000:fgh02]]n00[KOP0`0000020:bfh0040000002/]^00[;KP0:bfh0@0
00000P2[]N0010000000ZkCO0:^dg`2[]=l400000080Z[?O00<000000:Zcg`2Z/ml00P2Z/ml20000
0080ZK;O00D000000:VbgP2Y/]h0ZK;N000000030:VagP@000000P2X/Mh010000000Z;7N0:RagP2X
/Mh400000080Z;7N00@000000:R`gP2X/=h000000`2X/=h3000000<0Z;3N00@000000:N`gP2W/=h0
Yk3N1000003o0?ooob80oooo0P00000D0?ooo`00ZP3oool00`000000oooo0?ooo`1N0?ooo`030000
002e`n@0]L?T0080000000<0]<?U0000002d`^@00P2d`^@3000000@0/l7T0`0000020;;0i0030000
002b_n<0/[oS0080000000<0/[oT0000002b_^<00P2b_^<3000000<0/KgS00<000000;6lhP2a_>80
0P000000102`_><000000;2lhP2`_>8400000080[k_R00@000000:njhP2_^^80[k[R0`0000030:ji
h@030000002^^>40[[SQ0080000000<0[KSQ0000002]^>400`2]^>4300000080[;OQ00@000000:bf
h02/]^00[;KP0`0000020:^eh0030000002[]N00ZkGP0080ZkGP0`0000020:^dg`050000002Z/ml0
Z[?O0:Zcg`0000000P2Z/ml2000000050:Zcg`000000ZK;O0:Vbg`0000000`2Y/]h400000080ZK7N
00@000000:RagP2X/Mh0Z;7N100000020:RagP050000002X/Mh0Z;7N0:RagP0000000P2X/=h40000
0080Z;3N00@000000:R`gP2X/=h0Z;3N1000003o0?ooob80oooo1000000F0?ooo`00Z`3oool00`00
0000oooo0?ooo`1N0?ooo`800000102d`nD300000080/l;U00<000000;?1i02c`N@00P0000000`2c
`N@000000;;0i0020;;0i0<00000102b_n@300000080/KkT00@000000;6mh`2a_N<0/KgS0`000004
0;2lh`8000000P2_^n<200000080[k_R100000020:jjhP040000002^^N40[[WQ0:jih@<000000`2]
^>400`000000[KSQ0:fhh@02000000030:fgh@000000[;OQ0080[;OQ00H000000:bfh00000000000
0:bfh@0000030:^eh0030000002[]N00ZkGP0080000000<0ZkGP0000002[]=l00P2[]=l010000000
Z[?O0:Zcg`2Z/ml3000000<0Z[?O00<000000:Vbg`2Y/]l0100000020:Vbg`040000002Y/Mh0ZK7N
0:VagP@000000P2X/Mh01@000000Z;7N0:RagP2X/Mh0000000<0Z;7N0`0000020:R`gP040000002X
/=h0Z;3N0:R`gP@00000o`3ooolR0?ooo`@000006P3oool00:`0oooo00<000000?ooo`3oool0GP3o
ool2000000040;C3i@00000000000;?2i@8000000P2c`^D3000000@0/l7T0`0000020;;0i0030000
002b_n@0/[oT0080000000<0/KoT0000002a_^@00P2a_^@400000080/;gS00<000000;2lh`2`_><0
0P0000000`2__><000000:nkh`030:nkh`8000000P2^^n8200000080[[[R100000020:fihP040000
002]^>40[KSQ0:fhh@<000000`2]]n4010000000[;OQ0:bgh@2/]n42000000@0[;KQ00<000000:^e
h02[]N000`0000030:^eh0040000002[]=l0ZkCO0:^dg`@000000P2Z/ml010000000Z[?O0:Zcg`2Z
/ml300000080ZK;O00<000000:Vbg`2Y/]l00P2Y/]l300000080Z;7O00D000000:RagP2X/Mh0Z;7N
000000030:RagP@000000P2X/=h00`000000Z;3N0:R`gP0400000?l0oooo8P3oool4000001h0oooo
002]0?ooo`800000H03oool00`000000/l;U0;?2i@030;?2i@8000000P2c`^D2000000040;?1i000
000000000;;1i@8000000P2b`>@3000000@0/KoT0`0000020;2ni0040000002`_N<0/;gS0;2mh`<0
00000`2__><00`000000[k_S0:nkh`02000000030:jkh`000000[[_R00<0[[_R0P0000020:jjhP04
0000002]^N80[KWR0:fihP@000000P2]^>8010000000[KOQ0:fgh@2]]n4400000080[;OQ00@00000
0:bfh@2/]^40[;KQ0`0000020:^eh@040000002[]N00ZkGP0:^eh0@000000P2Z]>0010000000Z[?O
0:Zcg`2Z/ml400000080ZK?O00@000000:Vbg`2Y/]l000000P2Y/]l2000000030:Rbg`000000Z;7O
0080Z;7O00@000000:RagP2X/Mh0Z;7N100000020:RagP030000002X/=h0Z;3N0080Z;3N0P00003o
0?ooob80oooo1000000R0?ooo`00[`3oool00`000000oooo0?ooo`1N0?ooo`030000002c`^D0/l;U
0080000000<0/l;V0000002c`^D00P2c`^D2000000D0//7U0P0000020;6oi@80000000D0/KoT0000
00000000/;kT000000040;2ni0<000000P2__N@010000000[kcS0:nlh`2__><3000000<0[[_S00<0
00000:jkhP2^^n800P0000000`2^^^<000000:jjhP020:jjhP<000000P2]^N8010000000[KSR0:fh
hP2]^>8400000080[;OR00@000000:bgh@2/]n40[;OQ100000020:^fh@030000002[]N40ZkGQ00@0
00000P2Z]>000`000000Z[CP0:Zdh0020:Zdh0<000000P2Y/n001@000000ZK?O0:Vcg`2Y/ml00000
0080ZK;O0`0000030:Rbg`040000002X/Ml0Z;7O0:Rag`@000000P2X/Ml01@000000Z;7N0:RagP2X
/Mh000000080Z;3N0P00003o0?ooob00oooo1000000V0?ooo`00/03oool00`000000oooo0?ooo`1N
0?ooo`800000102c`^H300000080//7U00<000000;;1i@2b`ND00P0000000`2a`>D000000;6oi@02
0;6oi@800000102`_^@01P000000/;kT000000000000[kkT000000<0[kgT100000020:nli0040000
002^^n<0[[_S0:jkh`<000000`2^^^<01P000000[[[R000000000000[K[S000000@0[KWR0`000002
0:bhhP030000002/]n80[;OR0080[;OR0`0000020:^gh@030000002[]^40ZkKQ0080ZkKQ0P000002
0:Zeh@050000002Z]>00Z[CP0:Zdh00000000P2Z]>02000000030:Vdh0000000ZK?P0080ZK?P00@0
00000:Vcg`2Y/ml0ZK?O0`0000020:Vbg`8000000P2X/]l400000080Z;7O00D000000:Rag`2X/Ml0
Z;7O000000030:RagP<00000o`3ooolN0?ooo`@00000:P3oool00;40oooo00<000000?ooo`3oool0
GP3oool00`000000/l;V0;?2iP02000000030;;2iP000000//7U0080//7U0`0000040;70i@<00000
0P2`_nD010000000/;kT0;2ni02`_^@2000000@0[kkT0`0000020:nmi0040000002__>@0[kcT0:nl
i0@000000P2^^n<010000000[[[S0:jjh`2^^^<2000000@0[K[S00H000000:fihP00000000000:bi
h`0000030:bhhP030000002/]n80[;OR0080000000<0ZkOR0000002[]n400P2[]n400`000000ZkKQ
0:^fh@02000000050:Zfh@000000Z[GQ0:Zeh@0000000`2Z]>03000000<0ZKCP00@000000:Vch02Y
/n00ZK?P100000020:Vch0040000002Y/]l0ZK;O0:Vbg`<000000P2X/]l01@000000Z;7O0:Rag`2X
/Ml0000000<0Z;7O1000003o0?oooah0oooo0`00000^0?ooo`00/P3oool2000005l0oooo0P000004
0;;2iP<000000P2a`NH00`000000/L3U0;70i@02000000030;30iP000000/;oU0080/;oU10000002
0:nni@060000002__^@000000000002__ND00000102__N@300000080[[cT00@000000:jkh`2^^n<0
[[_S100000020:fjh`040000002]^^<0[K[S0:fjh`800000102/^N<00`000000[;SR0:bhhP030000
00<0ZkOR00@000000:^gh@2[]n40ZkOQ0`0000030:Zfh@030000002Z]N40Z[GQ00@000000P2Y]>40
10000000ZKCP0:Vdh02Y]>0400000080ZK?P00@000000:Vch02Y/n00ZK?P100000020:Rbg`040000
002X/]l0Z;;O000000<0Z;7O1000003o0?oooah0oooo1000000a0?ooo`00]03oool00`000000oooo
0?ooo`1M0?ooo`80000000@0//;V000000000000/L;V0P0000020;71iP<00000102``>H300000080
/;oU00@000000:nni@2__^D0[kkU0P0000040:nmi@050000002__N@000000000002^_N@00P000002
0:jli0@000000P2]^n@010000000[K[S0:fjh`2]^^<400000080[;WS00@000000:bih`2/^N<0[;WS
0`0000020:^hhP040000002[]n80ZkOR0:^ghP@000000P2Z]n8010000000Z[KQ0:Zfh@2Z]^430000
0080ZKGQ0P0000020:Vdh@@000000P2Y]>4010000000ZK?P0:Vch02Y/n0400000080Z;?P00D00000
0:Rbg`2X/]l0Z;;O000000020:Rbg`@00000o`3ooolN0?ooo`@00000=@3oool00;D0oooo00<00000
0?ooo`3oool0GP3oool00`000000/L;V0;72iP030;72iP8000000P2``NH00`000000/<3V0;30iP02
000000030;30iP000000/;oU0080/;oU100000020:nni@040000002__ND0[kgU0:nmi@8000001@2^
_N@200000080[KcT00@000000:fki02]^n@0[K_T100000020:bji0040000002/^N<0[;WS0:bih`@0
00000P2[^N<00`000000ZkSR0:^hhP0400000080ZkOR00@000000:ZghP2Z]n80Z[OR100000020:Zf
hP040000002Y]N40ZKGQ0:Veh@<000000P2Y]>400`000000ZKCQ0:Vdh@020:Vdh@<000000P2X/n00
1@000000Z;?P0:Rch02X/n00000000<0Z;;O0`00003o0?oooah0oooo1000000i0?ooo`00]P3oool0
0`000000oooo0?ooo`1N0?ooo`030000002a`^H0/L;V0080000000@0/<;W0000002``NH0/<7V0`00
00040;30iP<000000P2__nH010000000[kkU0:nni@2__^D400000080[[gU00<000000:jmi02^_N@0
0P0000000`2]_ND000000:fli0020:fli0<000000P2/^n@010000000[;[T0:bji02/^^@400000080
ZkWS00@000000:^ih`2[^N<0ZkWS0`0000020:^hh`8000000P2[]n8400000080Z[KR00@000000:Zf
hP2Z]^80Z[KR100000020:Veh@040000002Y]>40ZKCQ00000080ZKCQ0P0000000`2X]>4000000:Rc
h0020:Rch0040000002X/n00Z;?P0:Rch0@00000o`3ooolM0?ooo`@00000?@3oool00;L0oooo0P00
001O0?ooo`800000102``^L200000080/<7W0P0000001@2``>H000000000002_`>H0000000@0[koV
0`0000020:jniP040000002^_ND0[[gU0:jmi@<000000`2]_ND01P000000[KcT000000000000[;cU
000000@0[;_T0`0000020:bji0040000002[^N<0ZkWS0:^ih`@000000P2[^><010000000ZkSS0:^h
h`2[^><300000080Z[OS00<000000:ZfhP2Z]^800P2Z]^8300000080ZKKR00D000000:Veh@2Y]N40
ZKGQ000000020:Vdh@<000000`2X]>4010000000Z;?P0:Rch02X/n0400000?l0oooo7@3oool40000
0440oooo002i0?ooo`030000003oool0oooo05d0oooo00<000000;32i`2``^L00P0000000`2``^L0
00000;31i`020;31i`800000102_`>H01P000000[koV000000000000[[oV000000<0[[kV10000002
0:fmi@040000002]_ND0[KgU0:fmi@800000102/_>D01P000000[;_T000000000000[;_T000000@0
[;[T0`0000020:^ii0030000002[^><0ZkSS0080ZkSS0`0000020:Zhh`040000002Z]n<0Z[OS0000
0080Z[KR0P0000000`2Y]^8000000:VfhP020:VfhP040000002Y]N40ZKGQ0:Veh@<000000P2X]>42
00000080Z;CQ1000003o0?oooah0oooo0`0000150?ooo`00^P3oool00`000000oooo0?ooo`1M0?oo
o`800000102``^L300000080[l7W00@000000:o0iP2_`>H0[l3V0P0000040:joiP<000000P2]_^H0
10000000[KgU0:fmi@2]_ND400000080[;cU00@000000:bli@2/_>D0[;cU0P0000030:bki0030000
002/^^@0[;[T0080000000<0Zk[T0000002[^N@00P2[^N@00`000000ZkSS0:^hh`02000000030:Zh
i0000000Z[SS0080Z[SS00<000000:Zgh`2Z]n<00`0000030:VfhP040000002Y]^80ZKKR0:VfhP@0
00000P2X]N8010000000Z;CQ0:Rdh@2X]>4300000?l0oooo7P3oool4000004P0oooo002k0?ooo`03
0000003oool0oooo05d0oooo00<000000;32i`2``^L00P0000000`2_`^P000000:o1i`020:o1i`@0
00000P2^`>L01@000000[[oV000000000000[KoW008000000P2]_^H400000080[;gV00@000000:bl
i@2/_>D0[;cU100000020:bki@030000002/^n@0[;_T00<000000`2[^^@010000000ZkWT0:^ii02[
^N@3000000<0Z[ST00@000000:Zhh`2Z^><0Z[SS0`0000020:Vgh`040000002Y]^80ZKKR0:VfhP@0
00000P2X]N8010000000Z;GR0:RehP2X]N8400000?l0oooo7@3oool4000004`0oooo002l0?ooo`80
0000GP3oool2000000<0[l;X100000020:k1i`040000002^`>L0[/3W0:k0i`8000001@2]_nL20000
0080[KkV0P0000020:bmiP@000000P2/_>H010000000[;_U0:bki@2/^nD300000080Zk_U0P000002
0:^ji0@000000P2Z^N@010000000Z[ST0:Zhi02Z^>@400000080ZKOS00<000000:Vgh`2Y]n<01000
00020:Rfh`040000002X]N80Z;GR0:RehP@00000o`3ooolM0?ooo`@00000D03oool00;h0oooo00<0
00000?ooo`3oool0G@3oool300000080[/7X00@000000:k1i`2^`NL0[/7W100000020:g0i`030000
002]_nL0[KoW0080000000<0[KoW0000002]_^H00P2]_^H300000080[;gV00@000000:bliP2/_>H0
[;cV100000020:^ki@040000002[^nD0Zk_U0:^ki@<000000P2Z^^D010000000Z[WT0:Zii02Z^N@4
00000080ZKST00@000000:Vgh`2Y]n<0ZKOS0`0000020:Rgh`8000000P2X]^<400000?l0oooo7@3o
ool4000005@0oooo002o0?ooo`030000003oool0oooo05h0oooo00<000000:k1j02^`NP010000002
0:g1j0040000002]`>L0[L3W0:g0i`<00000102]_nL300000080[;kW00<000000:bmiP2/_NH01000
00020:^liP040000002[^nD0Zk_U0:^ki@@000000P2Z^^D00`000000Z[[U0:Zji@0400000080ZKWT
00@000000:Vhi02Y^>@0ZKST100000020:Rgi0040000002X]n<0Z;OS0:Rgh`<00000o`3ooolM0?oo
o`@00000F03oool00<00oooo00<000000?ooo`3oool0GP3oool200000080[L7X0P0000020:g1j0@0
00000P2]_nP00`000000[KoW0:foi`02000000030:bni`000000[;kW00<0[;kW0P0000020:^mi`80
00000P2[_>H400000080Z[_V00@000000:Zji@2Z^^D0Z[[U0`0000020:Vii@8000000P2Y^N@40000
0080Z;ST00@000000:Rgi02X]n@0Z;OT1000003o0?oooa`0oooo1000001L0?ooo`00`@3oool00`00
0000oooo0?ooo`1N0?ooo`040000002]`NP0[L7X0:g1j0<000000P2]`>P010000000[KoX0:foj02]
_nP3000000<0[;kW00<000000:bni`2/_^L00P0000000`2[_NL000000:^mi`020:^mi`<000000P2Z
_>H010000000Z[_V0:ZkiP2Z^nH400000080ZK[U00@000000:Vii@2Y^ND0ZKWU0`0000020:Rii@04
0000002X^>@0Z;ST0:Rhi0@00000o`3ooolM0?ooo`<00000H03oool00<80oooo0P00001O0?ooo`<0
00000P2]`NT00`000000[L3X0:g0j00400000080[;oX00@000000:bni`2/_^L0[;kW0`0000040:^m
i`<000000P2Z_NL00`000000Z[cV0:ZliP0400000080ZK_V00@000000:Vji@2Y^^D0ZK[U10000002
0:Rii@030000002X^ND0Z;WU00@00000o`3ooolM0?ooo`@00000H`3oool00<@0oooo00<000000?oo
o`3oool0G@3oool010000000[L7Y0:g1j@2]`NT300000080[<3X0P0000020:boj0@000000P2[_^P0
0`000000ZkgW0:^mi`02000000030:Zmi`000000Z[gW00<0Z[gW0P0000020:Vli`8000000P2Y^nH4
00000080Z;[V00<000000:Rii@2X^ND00P2X^ND200000?l0oooo7@3oool4000006L0oooo00350?oo
o`030000003oool0oooo05d0oooo0`0000020:c1j@040000002/`>P0[<3X0:c0j0<000000P2[_nP2
00000080ZkkX0`0000030:Zmi`030000002Z_NL0Z[gW0080000000<0ZKcW0000002Y_>L00`2Y_>L2
00000080Z;_V00D000000:RjiP2X^^H0Z;[V000000020:Rii@800000o`3ooolK0?ooo`@00000J`3o
ool00<H0oooo00<000000?ooo`3oool0G@3oool010000000[<7Y0:c1j@2/`NT400000080Zl3Y00@0
00000:^oj02[_nP0ZkoX0`0000020:Znj0040000002Z_NL0Z[gW0:Zmi`<000000`2Y_>L00`000000
ZKcW0:Vli`02000000050:Rki`000000Z;_V0:RkiP0000000`2X^^H300000?l0oooo6@3oool40000
06l0oooo00370?ooo`800000GP3oool300000080Zl7Y00@000000:_0j@2[`>T0Zl3Y100000020:Zo
j@030000002Z_^P0Z[kX00@000000P2Y_NP010000000ZKcW0:Vli`2Y_>L3000000<0Z;_W00<00000
0:RkiP2X^nH01000003o0?oooaP0oooo1000001c0?ooo`00b@3oool00`000000oooo0?ooo`1L0?oo
o`8000000P2[`NT400000080Z/3Y00@000000:Zoj@2Z_nT0Z[oY0`0000020:Vnj0040000002Y_NP0
ZKgX0:Vmj0@000000P2X_>P010000000Z;_W0:Rki`2X^nL300000?l0oooo603oool4000007L0oooo
003:0?ooo`030000003oool0oooo05d0oooo0P0000020:[1jP040000002Z`>T0Z/3Y0:[0j@@00000
0P2Y_nT00`000000ZKkX0:Vnj00400000080Z;gX00@000000:Rlj02X_>P0Z;cX1000003o0?oooaP0
oooo0`00001k0?ooo`00b`3oool00`000000oooo0?ooo`1M0?ooo`030000002Z`NX0Z/7Z00@00000
0P2Y_nX010000000ZKoY0:Voj@2Y_nT300000080Z;kY0P0000020:Rmj0@00000o`3ooolH0?ooo`@0
0000OP3oool00<`0oooo0P00001N0?ooo`8000000P2Y`>X200000080ZKoZ100000020:Rnj@040000
002X_^T0Z;kY0:Rnj@<00000o`3ooolH0?ooo`@00000PP3oool00<h0oooo00<000000?ooo`3oool0
G03oool010000000ZL3Z0:W0jP2Y`>X300000080Z;oZ00@000000:Rnj@2X_^T0Z;kY1000003o0?oo
oaL0oooo100000260?ooo`00c`3oool00`000000oooo0?ooo`1L0?ooo`<000000P2X`>/00`000000
Z;oZ0:RojP0400000?l0oooo5`3oool4000008X0oooo003@0?ooo`030000003oool0oooo05`0oooo
00@000000:S0j`2X`>/0Z<3[0`00003o0?oooaL0oooo1000002>0?ooo`00d@3oool2000005d0oooo
0`00003o0?oooaL0oooo0`00002B0?ooo`00d`3oool00`000000oooo0?ooo`1J0?ooo`030000003o
ool0oooo0?l0oooo4`3oool4000009D0oooo003D0?ooo`030000003oool0oooo05T0oooo00<00000
0?ooo`3oool0o`3oool?0?ooo`@00000V@3oool00=D0oooo00<000000?ooo`3oool0F03oool00`00
0000oooo0?ooo`3o0?ooo`/0oooo1000002M0?ooo`00eP3oool00`000000oooo0?ooo`1G0?ooo`03
0000003oool0oooo0?l0oooo1`3oool400000:40oooo003G0?ooo`800000E`3oool00`000000oooo
0?ooo`3o0?ooo`<0oooo1000002U0?ooo`00f@3oool00`000000oooo0?ooo`1D0?ooo`030000003o
ool0oooo0?h0oooo1000002Y0?ooo`00fP3oool00`000000oooo0?ooo`1C0?ooo`030000003oool0
oooo0?/0oooo0`00002]0?ooo`00f`3oool00`000000oooo0?ooo`1B0?ooo`030000003oool0oooo
0?L0oooo1000002`0?ooo`00g03oool200000580oooo00<000000?ooo`3oool0l`3oool400000;@0
oooo003N0?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0k`3oool400000;P0oooo
003O0?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0j`3oool400000;`0oooo003P
0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0i`3oool400000<00oooo003Q0?oo
o`800000C@3oool00`000000oooo0?ooo`3S0?ooo`@00000a03oool00><0oooo00<000000?ooo`3o
ool0B@3oool00`000000oooo0?ooo`3Q0?ooo`<00000b03oool00>@0oooo00<000000?ooo`3oool0
B03oool00`000000oooo0?ooo`3M0?ooo`@00000b`3oool00>D0oooo00<000000?ooo`3oool0A`3o
ool00`000000oooo0?ooo`3I0?ooo`@00000c`3oool00>H0oooo0P0000170?ooo`030000003oool0
oooo0=D0oooo1000003C0?ooo`00j03oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo
0=40oooo1000003G0?ooo`00j@3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo0<d0
oooo1000003K0?ooo`00jP3oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo0<X0oooo
0`00003O0?ooo`00j`3oool200000480oooo00<000000?ooo`3oool0aP3oool400000>80oooo003]
0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0`P3oool400000>H0oooo003^0?oo
o`030000003oool0oooo03h0oooo00<000000?ooo`3oool0_P3oool400000>X0oooo003_0?ooo`03
0000003oool0oooo03d0oooo00<000000?ooo`3oool0^P3oool400000>h0oooo003`0?ooo`030000
003oool0oooo03`0oooo00<000000?ooo`3oool0]P3oool400000?80oooo003a0?ooo`800000?03o
ool00`000000oooo0?ooo`2b0?ooo`@00000mP3oool00?<0oooo00<000000?ooo`3oool0>@3oool0
0`000000oooo0?ooo`2_0?ooo`<00000nP3oool00?@0oooo00<000000?ooo`3oool0>03oool00`00
0000oooo0?ooo`2[0?ooo`@00000o@3oool00?D0oooo00<000000?ooo`3oool0=`3oool00`000000
oooo0?ooo`2W0?ooo`@00000o`3oool20?ooo`00mP3oool2000003L0oooo00<000000?ooo`3oool0
X`3oool400000?l0oooo1P3oool00?P0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?oo
o`2O0?ooo`@00000o`3oool:0?ooo`00n@3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0
oooo09/0oooo1000003o0?ooo`h0oooo003j0?ooo`030000003oool0oooo0380oooo00<000000?oo
o`3oool0U`3oool400000?l0oooo4P3oool00?/0oooo0P00000b0?ooo`030000003oool0oooo09@0
oooo0`00003o0?oooaH0oooo003m0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0
T03oool400000?l0oooo6@3oool00?h0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?oo
o`2<0?ooo`@00000o`3ooolM0?ooo`00o`3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0
oooo08P0oooo1000003o0?ooob40oooo003o0?ooo`40oooo0P00000/0?ooo`030000003oool0oooo
08D0oooo1000003o0?ooobD0oooo003o0?ooo`<0oooo00<000000?ooo`3oool0:@3oool00`000000
oooo0?ooo`210?ooo`@00000o`3ooolY0?ooo`00o`3oool40?ooo`030000003oool0oooo02P0oooo
00<000000?ooo`3oool0O@3oool400000?l0oooo;@3oool00?l0oooo1@3oool00`000000oooo0?oo
o`0W0?ooo`030000003oool0oooo07X0oooo0`00003o0?oooc40oooo003o0?ooo`H0oooo0P00000W
0?ooo`030000003oool0oooo07H0oooo1000003o0?oooc@0oooo003o0?ooo`P0oooo00<000000?oo
o`3oool0903oool00`000000oooo0?ooo`1b0?ooo`@00000o`3ooolh0?ooo`00o`3oool90?ooo`03
0000003oool0oooo02<0oooo00<000000?ooo`3oool0KP3oool400000?l0oooo?03oool00?l0oooo
2P3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo06X0oooo1000003o0?oood00oooo
003o0?ooo`/0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`1V0?ooo`@00000o`3o
oom40?ooo`00o`3oool<0?ooo`8000008@3oool00`000000oooo0?ooo`1S0?ooo`<00000o`3ooom8
0?ooo`00o`3oool>0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0G`3oool40000
0?l0ooooB`3oool00?l0oooo3`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo05/0
oooo1000003o0?ooodl0oooo003o0?oooa00oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`1G0?ooo`@00000o`3ooomC0?ooo`00o`3ooolA0?ooo`800000703oool00`000000oooo0?oo
o`1C0?ooo`@00000o`3ooomG0?ooo`00o`3ooolC0?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool0C`3oool400000?l0ooooF`3oool00?l0oooo503oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo04/0oooo1000003o0?oooel0oooo003o0?oooaD0oooo00<000000?ooo`3o
ool05`3oool00`000000oooo0?ooo`180?ooo`<00000o`3ooomS0?ooo`00o`3ooolF0?ooo`800000
5`3oool00`000000oooo0?ooo`140?ooo`@00000o`3ooomV0?ooo`00o`3ooolH0?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool0@03oool400000?l0ooooJP3oool00?l0oooo6@3oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo03`0oooo1000003o0?ooofh0oooo003o0?oo
oaX0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0h0?ooo`@00000o`3ooomb0?oo
o`00o`3ooolK0?ooo`8000004P3oool00`000000oooo0?ooo`0d0?ooo`@00000o`3ooomf0?ooo`00
o`3ooolM0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0<03oool400000?l0oooo
NP3oool00?l0oooo7P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo02d0oooo0`00
003o0?ooogh0oooo003o0?oooal0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0Z
0?ooo`@00000o`3ooon10?ooo`00o`3ooolP0?ooo`800000303oool00`000000oooo0?ooo`0V0?oo
o`@00000o`3ooon50?ooo`00o`3ooolR0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool08P3oool400000?l0ooooR@3oool00?l0oooo8`3oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo01h0oooo1000003o0?ooohd0oooo003o0?ooob@0oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`0J0?ooo`@00000o`3ooonA0?ooo`00o`3ooolU0?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool05`3oool300000?l0ooooU@3oool00?l0oooo9P3oool20000
00H0oooo00<000000?ooo`3oool04`3oool400000?l0ooooV03oool00?l0oooo:03oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo00l0oooo1000003o0?oooi`0oooo003o0?ooobT0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0;0?ooo`@00000o`3ooonP0?ooo`00o`3o
oolZ0?ooo`050000003oool0oooo0?ooo`0000002@3oool400000?l0ooooY03oool00?l0oooo:`3o
ool2000000030?ooo`000000oooo00@0oooo1000003o0?ooojP0oooo003o0?ooobd0oooo0P000000
0`3oool000000000000200000?l0oooo[03oool00?l0oooo;P3oool200000?l0oooo/03oool00?l0
ooooo`3ooooP0?ooo`00o`3ooooo0?ooon00oooo003o0?ooool0ooooh03oool00?l0ooooo`3ooooP
0?ooo`00o`3ooooo0?ooon00oooo0000\
\>"],
  ImageRangeCache->{{{0, 733.813}, {595.063, 0}} -> {-0.0422162, 0.0144041, \
0.00209238, 0.00209238}}]
}, Closed]],

Cell["\<\
It is also useful to construct an interpolation function which can be used to \
produce a contour plot.\
\>", "Text"],

Cell[BoxData[
    \(\(f = ListInterpolation[\[Phi]];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ContourPlot[f[x, y], {x, 2, 99}, {y, 2, 99}, PlotPoints \[Rule] 100, 
        Contours \[Rule] Range[\(-0.9\), 0.9, 0.2], 
        ContourShading \[Rule] False, Frame \[Rule] False, 
        Epilog \[Rule] {RGBColor[0, 0, 1], Line[{{31, 41}, {70, 41}}], 
            RGBColor[1, 0, 0], Line[{{31, 60}, {70, 60}}]}];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% ContourGraphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.000594766 0.00991277 -0.000594766 0.00991277 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.31061 .58706 m
.32032 .58637 L
.33003 .58573 L
.33974 .5853 L
.34946 .58501 L
.35917 .58481 L
.36888 .58468 L
.37859 .58458 L
.38831 .58451 L
.39802 .58446 L
.40773 .58443 L
.41744 .5844 L
.42716 .58439 L
.43687 .58437 L
.44658 .58437 L
.45629 .58436 L
.46601 .58436 L
.47572 .58435 L
.48543 .58435 L
.49514 .58435 L
.50486 .58435 L
.51457 .58435 L
.52428 .58435 L
.53399 .58436 L
.54371 .58436 L
.55342 .58437 L
.56313 .58437 L
.57284 .58439 L
.58256 .5844 L
.59227 .58443 L
.60198 .58446 L
.61169 .58451 L
.62141 .58458 L
.63112 .58468 L
.64083 .58481 L
.65054 .58501 L
.66026 .5853 L
.66997 .58573 L
.67968 .58637 L
.68939 .58706 L
.69649 .59227 L
.69126 .60198 L
.68939 .60332 L
.67968 .60662 L
.66997 .60909 L
.66026 .61079 L
.65341 .61169 L
.65054 .61222 L
.64083 .61389 L
.63112 .61536 L
Mistroke
.62141 .61667 L
.61169 .61783 L
.60198 .61885 L
.59227 .61976 L
.58256 .62054 L
.57284 .62123 L
.57014 .62141 L
.56313 .62181 L
.55342 .6223 L
.54371 .6227 L
.53399 .62301 L
.52428 .62324 L
.51457 .62339 L
.50486 .62346 L
.49514 .62346 L
.48543 .62339 L
.47572 .62324 L
.46601 .62301 L
.45629 .6227 L
.44658 .6223 L
.43687 .62181 L
.42987 .62141 L
.42716 .62123 L
.41744 .62055 L
.40773 .61976 L
.39802 .61885 L
.38831 .61783 L
.37859 .61667 L
.36888 .61536 L
.35917 .61389 L
.34946 .61222 L
.34659 .61169 L
.33974 .6108 L
.33003 .60909 L
.32032 .60662 L
.31061 .60332 L
.30874 .60198 L
.30351 .59227 L
.31061 .58706 L
Mfstroke
.32032 .57227 m
.33003 .57036 L
.33974 .56907 L
.34946 .56816 L
.35917 .56753 L
.36888 .56707 L
.37859 .56675 L
.38831 .56652 L
.39802 .56635 L
.40773 .56623 L
.41744 .56615 L
.42716 .56609 L
.43687 .56604 L
.44658 .56601 L
.45629 .56599 L
.46601 .56597 L
.47572 .56596 L
.48543 .56595 L
.49514 .56595 L
.50486 .56595 L
.51457 .56595 L
.52428 .56596 L
.53399 .56597 L
.54371 .56599 L
.55342 .56601 L
.56313 .56604 L
.57284 .56609 L
.58256 .56615 L
.59227 .56623 L
.60198 .56635 L
.61169 .56652 L
.62141 .56675 L
.63112 .56707 L
.64083 .56753 L
.65054 .56816 L
.66026 .56907 L
.66997 .57036 L
.67968 .57227 L
.68202 .57284 L
.68939 .57523 L
.69911 .58081 L
.70118 .58256 L
.70736 .59227 L
.70772 .60198 L
.70477 .61169 L
.69911 .62117 L
.69893 .62141 L
.69027 .63112 L
.68939 .63193 L
.67968 .63992 L
Mistroke
.67843 .64083 L
.66997 .64644 L
.66294 .65054 L
.66026 .65199 L
.65054 .65679 L
.6426 .66026 L
.64083 .66098 L
.63112 .66468 L
.62141 .66794 L
.61472 .66997 L
.61169 .67083 L
.60198 .67337 L
.59227 .67561 L
.58256 .67757 L
.57284 .67927 L
.57026 .67968 L
.56313 .68072 L
.55342 .68194 L
.54371 .68294 L
.53399 .68373 L
.52428 .68432 L
.51457 .6847 L
.50486 .6849 L
.49514 .68489 L
.48543 .6847 L
.47572 .68431 L
.46601 .68373 L
.45629 .68294 L
.44658 .68194 L
.43687 .68072 L
.42973 .67968 L
.42716 .67927 L
.41744 .67757 L
.40773 .67561 L
.39802 .67338 L
.38831 .67083 L
.38528 .66997 L
.37859 .66794 L
.36888 .66468 L
.35917 .66098 L
.35739 .66026 L
.34946 .65678 L
.33974 .65199 L
.33706 .65054 L
.33003 .64644 L
.32157 .64083 L
.32032 .63992 L
.31061 .63193 L
.30973 .63112 L
.30107 .62141 L
Mistroke
.30089 .62116 L
.29523 .61169 L
.29228 .60198 L
.29264 .59227 L
.29882 .58256 L
.30089 .58081 L
.31061 .57523 L
.31798 .57284 L
.32032 .57227 L
Mfstroke
.33003 .55212 m
.33974 .55073 L
.34946 .54973 L
.35917 .549 L
.36888 .54848 L
.37859 .54809 L
.38831 .54782 L
.39802 .54762 L
.40773 .54747 L
.41744 .54736 L
.42716 .54729 L
.43687 .54724 L
.44658 .5472 L
.45629 .54717 L
.46601 .54715 L
.47572 .54714 L
.48543 .54713 L
.49514 .54713 L
.50486 .54712 L
.51457 .54713 L
.52428 .54714 L
.53399 .54715 L
.54371 .54717 L
.55342 .5472 L
.56313 .54724 L
.57284 .54729 L
.58256 .54736 L
.59227 .54747 L
.60198 .54762 L
.61169 .54782 L
.62141 .54809 L
.63112 .54848 L
.64083 .549 L
.65054 .54973 L
.66026 .55073 L
.66997 .55212 L
.67689 .55342 L
.67968 .55405 L
.68939 .55676 L
.69911 .56067 L
.70377 .56313 L
.70882 .56639 L
.71648 .57284 L
.71853 .5749 L
.72431 .58256 L
.72824 .58966 L
.72918 .59227 L
.73189 .60198 L
.73297 .61169 L
.73271 .62141 L
Mistroke
.73124 .63112 L
.72863 .64083 L
.72824 .64198 L
.72487 .65054 L
.71995 .66026 L
.71853 .66269 L
.7138 .66997 L
.70882 .67662 L
.7063 .67968 L
.69911 .68757 L
.69729 .68939 L
.68939 .69668 L
.68652 .69911 L
.67968 .70448 L
.67362 .70882 L
.66997 .71126 L
.66026 .71724 L
.65798 .71853 L
.65054 .72252 L
.64083 .72723 L
.63858 .72824 L
.63112 .73141 L
.62141 .73515 L
.61327 .73796 L
.61169 .73847 L
.60198 .74141 L
.59227 .74401 L
.58256 .74629 L
.57597 .74767 L
.57284 .74827 L
.56313 .74997 L
.55342 .75141 L
.54371 .75258 L
.53399 .75351 L
.52428 .75421 L
.51457 .75466 L
.50486 .7549 L
.49514 .7549 L
.48543 .75466 L
.47572 .75421 L
.46601 .75351 L
.45629 .75258 L
.44658 .75141 L
.43687 .74997 L
.42716 .74827 L
.42405 .74767 L
.41744 .74629 L
.40773 .74401 L
.39802 .74141 L
.38831 .73847 L
Mistroke
.38673 .73796 L
.37859 .73515 L
.36888 .73142 L
.36143 .72824 L
.35917 .72722 L
.34946 .72252 L
.34202 .71853 L
.33974 .71723 L
.33003 .71126 L
.32638 .70882 L
.32032 .70448 L
.31348 .69911 L
.31061 .69668 L
.30271 .68939 L
.30089 .68757 L
.2937 .67968 L
.29118 .67662 L
.2862 .66997 L
.28147 .66269 L
.28004 .66026 L
.27513 .65054 L
.27176 .64198 L
.27137 .64083 L
.26876 .63112 L
.26729 .62141 L
.26703 .61169 L
.26811 .60198 L
.27082 .59227 L
.27176 .58966 L
.27569 .58256 L
.28147 .5749 L
.28352 .57284 L
.29118 .56639 L
.29622 .56313 L
.30089 .56067 L
.31061 .55676 L
.32032 .55405 L
.32311 .55342 L
.33003 .55212 L
Mfstroke
.32032 .53337 m
.33003 .53204 L
.33974 .53104 L
.34946 .5303 L
.35917 .52975 L
.36888 .52935 L
.37859 .52905 L
.38831 .52883 L
.39802 .52867 L
.40773 .52856 L
.41744 .52848 L
.42716 .52841 L
.43687 .52837 L
.44658 .52834 L
.45629 .52832 L
.46601 .5283 L
.47572 .52829 L
.48543 .52829 L
.49514 .52828 L
.50486 .52828 L
.51457 .52829 L
.52428 .52829 L
.53399 .5283 L
.54371 .52832 L
.55342 .52834 L
.56313 .52837 L
.57284 .52841 L
.58256 .52847 L
.59227 .52856 L
.60198 .52867 L
.61169 .52883 L
.62141 .52905 L
.63112 .52935 L
.64083 .52975 L
.65054 .5303 L
.66026 .53104 L
.66997 .53204 L
.67968 .53337 L
.68346 .53399 L
.68939 .53513 L
.69911 .53744 L
.70882 .54042 L
.71736 .54371 L
.71853 .54421 L
.72824 .54896 L
.73573 .55342 L
.73796 .55489 L
.74767 .56229 L
.74864 .56313 L
.75738 .57167 L
Mistroke
.75843 .57284 L
.7661 .58256 L
.76709 .58398 L
.77216 .59227 L
.77681 .60174 L
.77691 .60198 L
.78056 .61169 L
.78324 .62141 L
.78505 .63112 L
.78606 .64083 L
.78632 .65054 L
.78585 .66026 L
.78467 .66997 L
.7828 .67968 L
.78024 .68939 L
.77697 .69911 L
.77681 .69955 L
.77297 .70882 L
.76818 .71853 L
.76709 .72053 L
.76256 .72824 L
.75738 .73607 L
.75604 .73796 L
.74853 .74767 L
.74767 .7487 L
.73988 .75738 L
.73796 .75937 L
.72992 .76709 L
.72824 .7686 L
.71853 .77672 L
.71842 .77681 L
.70882 .78392 L
.70502 .78652 L
.69911 .79035 L
.68939 .79611 L
.68918 .79623 L
.67968 .80131 L
.67011 .80594 L
.66997 .80601 L
.66026 .81024 L
.65054 .81408 L
.64625 .81566 L
.64083 .81755 L
.63112 .82067 L
.62141 .82348 L
.61423 .82537 L
.61169 .826 L
.60198 .82826 L
.59227 .83026 L
.58256 .83201 L
Mistroke
.57284 .83355 L
.56313 .83487 L
.56144 .83508 L
.55342 .836 L
.54371 .83692 L
.53399 .83765 L
.52428 .83818 L
.51457 .83855 L
.50486 .83872 L
.49514 .83873 L
.48543 .83854 L
.47572 .83819 L
.46601 .83764 L
.45629 .83692 L
.44658 .83599 L
.43853 .83508 L
.43687 .83488 L
.42716 .83355 L
.41744 .83202 L
.40773 .83026 L
.39802 .82826 L
.38831 .826 L
.38576 .82537 L
.37859 .82348 L
.36888 .82067 L
.35917 .81754 L
.35375 .81566 L
.34946 .81408 L
.33974 .81025 L
.33003 .80601 L
.32989 .80594 L
.32032 .80131 L
.31082 .79623 L
.31061 .79611 L
.30089 .79035 L
.29498 .78652 L
.29118 .78392 L
.28158 .77681 L
.28147 .77672 L
.27176 .7686 L
.27008 .76709 L
.26204 .75937 L
.26012 .75738 L
.25233 .7487 L
.25147 .74767 L
.24396 .73796 L
.24262 .73607 L
.23744 .72824 L
.23291 .72053 L
.23182 .71853 L
Mistroke
.22703 .70882 L
.22319 .69955 L
.22303 .69911 L
.21976 .68939 L
.2172 .67968 L
.21533 .66997 L
.21415 .66026 L
.21368 .65054 L
.21393 .64083 L
.21495 .63112 L
.21676 .62141 L
.21944 .61169 L
.22309 .60198 L
.22319 .60174 L
.22784 .59227 L
.23291 .58398 L
.2339 .58256 L
.24157 .57284 L
.24262 .57167 L
.25136 .56313 L
.25233 .56229 L
.26204 .55489 L
.26427 .55342 L
.27176 .54896 L
.28147 .54421 L
.28264 .54371 L
.29118 .54042 L
.30089 .53744 L
.31061 .53513 L
.31654 .53399 L
.32032 .53337 L
Mfstroke
.29118 .51358 m
.30089 .51264 L
.31061 .51187 L
.32032 .51127 L
.33003 .51081 L
.33974 .51045 L
.34946 .51018 L
.35917 .50998 L
.36888 .50983 L
.37859 .50972 L
.38831 .50964 L
.39802 .50958 L
.40773 .50953 L
.41744 .5095 L
.42716 .50948 L
.43687 .50946 L
.44658 .50945 L
.45629 .50944 L
.46601 .50944 L
.47572 .50943 L
.48543 .50943 L
.49514 .50943 L
.50486 .50943 L
.51457 .50943 L
.52428 .50943 L
.53399 .50944 L
.54371 .50944 L
.55342 .50945 L
.56313 .50946 L
.57284 .50948 L
.58256 .5095 L
.59227 .50953 L
.60198 .50958 L
.61169 .50964 L
.62141 .50972 L
.63112 .50983 L
.64083 .50998 L
.65054 .51018 L
.66026 .51045 L
.66997 .51081 L
.67968 .51127 L
.68939 .51187 L
.69911 .51264 L
.70882 .51358 L
.71722 .51457 L
.71853 .51474 L
.72824 .51612 L
.73796 .51774 L
.74767 .51963 L
.75738 .52181 L
Mistroke
.76709 .52428 L
.7671 .52428 L
.77681 .52709 L
.78652 .53026 L
.79623 .53385 L
.7966 .53399 L
.80594 .53789 L
.81566 .54248 L
.81804 .54371 L
.82537 .54771 L
.83463 .55342 L
.83508 .55372 L
.84479 .56071 L
.84782 .56313 L
.85451 .56896 L
.85851 .57284 L
.86422 .57897 L
.86723 .58256 L
.87393 .5916 L
.87438 .59227 L
.88024 .60198 L
.88364 .60862 L
.88506 .61169 L
.889 .62141 L
.89218 .63112 L
.89336 .63535 L
.89469 .64083 L
.89664 .65054 L
.8981 .66026 L
.8991 .66997 L
.89969 .67968 L
.89987 .68939 L
.89968 .69911 L
.89913 .70882 L
.89822 .71853 L
.89697 .72824 L
.89535 .73796 L
.89337 .74767 L
.89336 .74773 L
.89103 .75738 L
.88831 .76709 L
.88516 .77681 L
.88364 .78102 L
.88153 .78652 L
.87742 .79623 L
.87393 .80357 L
.87273 .80594 L
.86739 .81566 L
.86422 .82089 L
.86132 .82537 L
Mistroke
.85451 .83493 L
.85439 .83508 L
.84643 .84479 L
.84479 .84663 L
.83724 .85451 L
.83508 .8566 L
.82656 .86422 L
.82537 .86521 L
.81566 .87272 L
.81397 .87393 L
.80594 .87935 L
.79896 .88364 L
.79623 .88523 L
.78652 .89049 L
.78073 .89336 L
.77681 .8952 L
.76709 .89946 L
.75802 .90307 L
.75738 .90331 L
.74767 .9068 L
.73796 .90997 L
.72853 .91278 L
.72824 .91286 L
.71853 .9155 L
.70882 .9179 L
.69911 .9201 L
.68939 .92211 L
.6874 .92249 L
.67968 .92394 L
.66997 .92561 L
.66026 .92715 L
.65054 .92854 L
.64083 .9298 L
.63112 .93095 L
.62141 .93199 L
.61926 .93221 L
.61169 .93293 L
.60198 .93376 L
.59227 .9345 L
.58256 .93517 L
.57284 .93575 L
.56313 .93626 L
.55342 .93668 L
.54371 .93702 L
.53399 .9373 L
.52428 .9375 L
.51457 .93764 L
.50486 .9377 L
.49514 .93771 L
.48543 .93764 L
Mistroke
.47572 .9375 L
.46601 .9373 L
.45629 .93702 L
.44658 .93668 L
.43687 .93626 L
.42716 .93575 L
.41744 .93517 L
.40773 .93451 L
.39802 .93376 L
.38831 .93293 L
.38073 .93221 L
.37859 .93199 L
.36888 .93095 L
.35917 .9298 L
.34946 .92854 L
.33974 .92715 L
.33003 .92561 L
.32032 .92394 L
.31261 .92249 L
.31061 .9221 L
.30089 .9201 L
.29118 .9179 L
.28147 .9155 L
.27176 .91286 L
.27148 .91278 L
.26204 .90997 L
.25233 .9068 L
.24262 .90331 L
.24199 .90307 L
.23291 .89946 L
.22319 .8952 L
.21928 .89336 L
.21348 .89048 L
.20377 .88523 L
.20104 .88364 L
.19406 .87934 L
.18603 .87393 L
.18434 .87272 L
.17463 .86521 L
.17345 .86422 L
.16492 .8566 L
.16276 .85451 L
.15521 .84662 L
.15358 .84479 L
.14562 .83508 L
.14549 .83492 L
.13868 .82537 L
.13578 .82088 L
.13262 .81566 L
.12727 .80594 L
Mistroke
.12607 .80357 L
.12258 .79623 L
.11846 .78652 L
.11636 .78102 L
.11484 .77681 L
.11169 .76709 L
.10897 .75738 L
.10664 .74775 L
.10663 .74767 L
.10465 .73796 L
.10304 .72824 L
.10178 .71853 L
.10087 .70882 L
.10032 .69911 L
.10013 .68939 L
.10031 .67968 L
.1009 .66997 L
.1019 .66026 L
.10336 .65054 L
.1053 .64083 L
.10664 .63534 L
.10782 .63112 L
.111 .62141 L
.11495 .61169 L
.11636 .60864 L
.11976 .60198 L
.12563 .59227 L
.12607 .59161 L
.13276 .58256 L
.13578 .57897 L
.14149 .57284 L
.14549 .56896 L
.15218 .56313 L
.15521 .5607 L
.16492 .55372 L
.16537 .55342 L
.17463 .54771 L
.18196 .54371 L
.18434 .54249 L
.19406 .53789 L
.20339 .53399 L
.20377 .53385 L
.21348 .53026 L
.22319 .52709 L
.2329 .52428 L
.23291 .52428 L
.24262 .52181 L
.25233 .51963 L
.26204 .51774 L
.27176 .51612 L
Mistroke
.28147 .51474 L
.28278 .51457 L
.29118 .51358 L
Mfstroke
.38831 .06707 m
.39802 .06624 L
.40773 .0655 L
.41744 .06483 L
.42716 .06425 L
.43687 .06374 L
.44658 .06332 L
.45629 .06298 L
.46601 .0627 L
.47572 .0625 L
.48543 .06236 L
.49514 .0623 L
.50486 .06229 L
.51457 .06236 L
.52428 .0625 L
.53399 .0627 L
.54371 .06298 L
.55342 .06332 L
.56313 .06374 L
.57284 .06425 L
.58256 .06483 L
.59227 .06549 L
.60198 .06624 L
.61169 .06707 L
.61927 .06779 L
.62141 .06801 L
.63112 .06905 L
.64083 .0702 L
.65054 .07146 L
.66026 .07285 L
.66997 .07439 L
.67968 .07606 L
.68739 .07751 L
.68939 .0779 L
.69911 .0799 L
.70882 .0821 L
.71853 .0845 L
.72824 .08714 L
.72852 .08722 L
.73796 .09003 L
.74767 .0932 L
.75738 .09669 L
.75801 .09693 L
.76709 .10054 L
.77681 .1048 L
.78072 .10664 L
.78652 .10952 L
.79623 .11477 L
.79896 .11636 L
.80594 .12066 L
Mistroke
.81397 .12607 L
.81566 .12728 L
.82537 .13479 L
.82655 .13578 L
.83508 .1434 L
.83724 .14549 L
.84479 .15338 L
.84642 .15521 L
.85438 .16492 L
.85451 .16508 L
.86132 .17463 L
.86422 .17912 L
.86738 .18434 L
.87273 .19406 L
.87393 .19643 L
.87742 .20377 L
.88154 .21348 L
.88364 .21898 L
.88516 .22319 L
.88831 .23291 L
.89103 .24262 L
.89336 .25225 L
.89337 .25233 L
.89535 .26204 L
.89696 .27176 L
.89822 .28147 L
.89913 .29118 L
.89968 .30089 L
.89987 .31061 L
.89969 .32032 L
.8991 .33003 L
.8981 .33974 L
.89664 .34946 L
.8947 .35917 L
.89336 .36466 L
.89218 .36888 L
.889 .37859 L
.88505 .38831 L
.88364 .39136 L
.88024 .39802 L
.87437 .40773 L
.87393 .40839 L
.86724 .41744 L
.86422 .42103 L
.85851 .42716 L
.85451 .43104 L
.84782 .43687 L
.84479 .4393 L
.83508 .44628 L
.83463 .44658 L
Mistroke
.82537 .45229 L
.81804 .45629 L
.81566 .45751 L
.80594 .46211 L
.79661 .46601 L
.79623 .46615 L
.78652 .46974 L
.77681 .47291 L
.7671 .47572 L
.76709 .47572 L
.75738 .47819 L
.74767 .48037 L
.73796 .48226 L
.72824 .48388 L
.71853 .48526 L
.71722 .48543 L
.70882 .48642 L
.69911 .48736 L
.68939 .48813 L
.67968 .48873 L
.66997 .48919 L
.66026 .48955 L
.65054 .48982 L
.64083 .49002 L
.63112 .49017 L
.62141 .49028 L
.61169 .49036 L
.60198 .49042 L
.59227 .49047 L
.58256 .4905 L
.57284 .49052 L
.56313 .49054 L
.55342 .49055 L
.54371 .49056 L
.53399 .49056 L
.52428 .49057 L
.51457 .49057 L
.50486 .49057 L
.49514 .49057 L
.48543 .49057 L
.47572 .49057 L
.46601 .49056 L
.45629 .49056 L
.44658 .49055 L
.43687 .49054 L
.42716 .49052 L
.41744 .4905 L
.40773 .49047 L
.39802 .49042 L
.38831 .49036 L
Mistroke
.37859 .49028 L
.36888 .49017 L
.35917 .49002 L
.34946 .48982 L
.33974 .48955 L
.33003 .48919 L
.32032 .48873 L
.31061 .48813 L
.30089 .48736 L
.29118 .48642 L
.28278 .48543 L
.28147 .48526 L
.27176 .48388 L
.26204 .48226 L
.25233 .48037 L
.24262 .47819 L
.23291 .47572 L
.2329 .47572 L
.22319 .47291 L
.21348 .46974 L
.20377 .46615 L
.2034 .46601 L
.19406 .46211 L
.18434 .45752 L
.18196 .45629 L
.17463 .45229 L
.16537 .44658 L
.16492 .44628 L
.15521 .43929 L
.15218 .43687 L
.14549 .43104 L
.14149 .42716 L
.13578 .42103 L
.13277 .41744 L
.12607 .4084 L
.12562 .40773 L
.11976 .39802 L
.11636 .39138 L
.11494 .38831 L
.111 .37859 L
.10782 .36888 L
.10664 .36465 L
.10531 .35917 L
.10336 .34946 L
.1019 .33974 L
.1009 .33003 L
.10031 .32032 L
.10013 .31061 L
.10032 .30089 L
.10087 .29118 L
Mistroke
.10178 .28147 L
.10303 .27176 L
.10465 .26204 L
.10663 .25233 L
.10664 .25227 L
.10897 .24262 L
.11169 .23291 L
.11484 .22319 L
.11636 .21898 L
.11847 .21348 L
.12258 .20377 L
.12607 .19643 L
.12727 .19406 L
.13261 .18434 L
.13578 .17911 L
.13868 .17463 L
.14549 .16507 L
.14561 .16492 L
.15357 .15521 L
.15521 .15337 L
.16276 .14549 L
.16492 .1434 L
.17344 .13578 L
.17463 .13479 L
.18434 .12728 L
.18603 .12607 L
.19406 .12065 L
.20104 .11636 L
.20377 .11477 L
.21348 .10951 L
.21927 .10664 L
.22319 .1048 L
.23291 .10054 L
.24198 .09693 L
.24262 .09669 L
.25233 .0932 L
.26204 .09003 L
.27147 .08722 L
.27176 .08714 L
.28147 .0845 L
.29118 .0821 L
.30089 .0799 L
.31061 .07789 L
.3126 .07751 L
.32032 .07606 L
.33003 .07439 L
.33974 .07285 L
.34946 .07146 L
.35917 .0702 L
.36888 .06905 L
Mistroke
.37859 .06801 L
.38074 .06779 L
.38831 .06707 L
Mfstroke
.44658 .164 m
.45629 .16308 L
.46601 .16235 L
.47572 .16182 L
.48543 .16145 L
.49514 .16128 L
.50486 .16127 L
.51457 .16146 L
.52428 .16181 L
.53399 .16236 L
.54371 .16308 L
.55342 .16401 L
.56147 .16492 L
.56313 .16512 L
.57284 .16645 L
.58256 .16798 L
.59227 .16974 L
.60198 .17174 L
.61169 .174 L
.61424 .17463 L
.62141 .17652 L
.63112 .17933 L
.64083 .18246 L
.64625 .18434 L
.65054 .18592 L
.66026 .18975 L
.66997 .19399 L
.67011 .19406 L
.67968 .19869 L
.68918 .20377 L
.68939 .20389 L
.69911 .20965 L
.70502 .21348 L
.70882 .21608 L
.71842 .22319 L
.71853 .22328 L
.72824 .2314 L
.72992 .23291 L
.73796 .24063 L
.73988 .24262 L
.74767 .2513 L
.74853 .25233 L
.75604 .26204 L
.75738 .26393 L
.76256 .27176 L
.76709 .27947 L
.76818 .28147 L
.77297 .29118 L
.77681 .30045 L
.77697 .30089 L
Mistroke
.78024 .31061 L
.7828 .32032 L
.78467 .33003 L
.78585 .33974 L
.78632 .34946 L
.78607 .35917 L
.78505 .36888 L
.78324 .37859 L
.78056 .38831 L
.77691 .39802 L
.77681 .39826 L
.77216 .40773 L
.76709 .41602 L
.7661 .41744 L
.75843 .42716 L
.75738 .42833 L
.74864 .43687 L
.74767 .43771 L
.73796 .44511 L
.73573 .44658 L
.72824 .45104 L
.71853 .45579 L
.71736 .45629 L
.70882 .45958 L
.69911 .46256 L
.68939 .46487 L
.68346 .46601 L
.67968 .46663 L
.66997 .46796 L
.66026 .46896 L
.65054 .4697 L
.64083 .47025 L
.63112 .47065 L
.62141 .47095 L
.61169 .47117 L
.60198 .47133 L
.59227 .47144 L
.58256 .47152 L
.57284 .47159 L
.56313 .47163 L
.55342 .47166 L
.54371 .47168 L
.53399 .4717 L
.52428 .47171 L
.51457 .47171 L
.50486 .47172 L
.49514 .47172 L
.48543 .47171 L
.47572 .47171 L
.46601 .4717 L
Mistroke
.45629 .47168 L
.44658 .47166 L
.43687 .47163 L
.42716 .47159 L
.41744 .47153 L
.40773 .47144 L
.39802 .47133 L
.38831 .47117 L
.37859 .47095 L
.36888 .47065 L
.35917 .47025 L
.34946 .4697 L
.33974 .46896 L
.33003 .46796 L
.32032 .46663 L
.31654 .46601 L
.31061 .46487 L
.30089 .46256 L
.29118 .45958 L
.28264 .45629 L
.28147 .45579 L
.27176 .45104 L
.26427 .44658 L
.26204 .44511 L
.25233 .43771 L
.25136 .43687 L
.24262 .42833 L
.24157 .42716 L
.2339 .41744 L
.23291 .41602 L
.22784 .40773 L
.22319 .39826 L
.22309 .39802 L
.21944 .38831 L
.21676 .37859 L
.21495 .36888 L
.21394 .35917 L
.21368 .34946 L
.21415 .33974 L
.21533 .33003 L
.2172 .32032 L
.21976 .31061 L
.22303 .30089 L
.22319 .30045 L
.22703 .29118 L
.23182 .28147 L
.23291 .27947 L
.23744 .27176 L
.24262 .26393 L
.24396 .26204 L
Mistroke
.25147 .25233 L
.25233 .2513 L
.26012 .24262 L
.26204 .24063 L
.27008 .23291 L
.27176 .2314 L
.28147 .22328 L
.28158 .22319 L
.29118 .21608 L
.29498 .21348 L
.30089 .20965 L
.31061 .20389 L
.31082 .20377 L
.32032 .19869 L
.32989 .19406 L
.33003 .19399 L
.33974 .18976 L
.34946 .18592 L
.35375 .18434 L
.35917 .18245 L
.36888 .17933 L
.37859 .17652 L
.38577 .17463 L
.38831 .174 L
.39802 .17174 L
.40773 .16974 L
.41744 .16799 L
.42716 .16645 L
.43687 .16513 L
.43856 .16492 L
.44658 .164 L
Mfstroke
.42716 .25173 m
.43687 .25003 L
.44658 .24859 L
.45629 .24742 L
.46601 .24649 L
.47572 .24579 L
.48543 .24534 L
.49514 .2451 L
.50486 .2451 L
.51457 .24534 L
.52428 .24579 L
.53399 .24649 L
.54371 .24742 L
.55342 .24859 L
.56313 .25003 L
.57284 .25173 L
.57595 .25233 L
.58256 .25371 L
.59227 .25599 L
.60198 .25859 L
.61169 .26153 L
.61327 .26204 L
.62141 .26485 L
.63112 .26858 L
.63857 .27176 L
.64083 .27278 L
.65054 .27748 L
.65798 .28147 L
.66026 .28277 L
.66997 .28874 L
.67362 .29118 L
.67968 .29552 L
.68652 .30089 L
.68939 .30332 L
.69729 .31061 L
.69911 .31243 L
.7063 .32032 L
.70882 .32338 L
.7138 .33003 L
.71853 .33731 L
.71996 .33974 L
.72487 .34946 L
.72824 .35802 L
.72863 .35917 L
.73124 .36888 L
.73271 .37859 L
.73297 .38831 L
.73189 .39802 L
.72918 .40773 L
.72824 .41034 L
Mistroke
.72431 .41744 L
.71853 .4251 L
.71648 .42716 L
.70882 .43361 L
.70378 .43687 L
.69911 .43933 L
.68939 .44324 L
.67968 .44595 L
.67689 .44658 L
.66997 .44788 L
.66026 .44927 L
.65054 .45027 L
.64083 .451 L
.63112 .45152 L
.62141 .45191 L
.61169 .45218 L
.60198 .45238 L
.59227 .45253 L
.58256 .45264 L
.57284 .45271 L
.56313 .45276 L
.55342 .4528 L
.54371 .45283 L
.53399 .45285 L
.52428 .45286 L
.51457 .45287 L
.50486 .45287 L
.49514 .45288 L
.48543 .45287 L
.47572 .45286 L
.46601 .45285 L
.45629 .45283 L
.44658 .4528 L
.43687 .45276 L
.42716 .45271 L
.41744 .45264 L
.40773 .45253 L
.39802 .45238 L
.38831 .45218 L
.37859 .45191 L
.36888 .45152 L
.35917 .451 L
.34946 .45027 L
.33974 .44927 L
.33003 .44788 L
.32311 .44658 L
.32032 .44595 L
.31061 .44324 L
.30089 .43933 L
.29623 .43687 L
Mistroke
.29118 .43361 L
.28352 .42716 L
.28147 .4251 L
.27569 .41744 L
.27176 .41034 L
.27082 .40773 L
.26811 .39802 L
.26703 .38831 L
.26729 .37859 L
.26876 .36888 L
.27137 .35917 L
.27176 .35802 L
.27513 .34946 L
.28005 .33974 L
.28147 .33731 L
.2862 .33003 L
.29118 .32338 L
.2937 .32032 L
.30089 .31243 L
.30271 .31061 L
.31061 .30332 L
.31348 .30089 L
.32032 .29552 L
.32638 .29118 L
.33003 .28874 L
.33974 .28276 L
.34202 .28147 L
.34946 .27748 L
.35917 .27277 L
.36142 .27176 L
.36888 .26859 L
.37859 .26485 L
.38673 .26204 L
.38831 .26153 L
.39802 .25859 L
.40773 .25599 L
.41744 .25371 L
.42403 .25233 L
.42716 .25173 L
Mfstroke
.43687 .31928 m
.44658 .31806 L
.45629 .31706 L
.46601 .31627 L
.47572 .31568 L
.48543 .3153 L
.49514 .3151 L
.50486 .31511 L
.51457 .3153 L
.52428 .31569 L
.53399 .31627 L
.54371 .31706 L
.55342 .31806 L
.56313 .31928 L
.57027 .32032 L
.57284 .32073 L
.58256 .32243 L
.59227 .32439 L
.60198 .32662 L
.61169 .32917 L
.61472 .33003 L
.62141 .33206 L
.63112 .33532 L
.64083 .33902 L
.64261 .33974 L
.65054 .34322 L
.66026 .34801 L
.66294 .34946 L
.66997 .35356 L
.67843 .35917 L
.67968 .36008 L
.68939 .36807 L
.69027 .36888 L
.69893 .37859 L
.69911 .37884 L
.70477 .38831 L
.70772 .39802 L
.70736 .40773 L
.70118 .41744 L
.69911 .41919 L
.68939 .42477 L
.68202 .42716 L
.67968 .42773 L
.66997 .42964 L
.66026 .43093 L
.65054 .43184 L
.64083 .43247 L
.63112 .43293 L
.62141 .43325 L
.61169 .43348 L
Mistroke
.60198 .43365 L
.59227 .43377 L
.58256 .43385 L
.57284 .43391 L
.56313 .43396 L
.55342 .43399 L
.54371 .43401 L
.53399 .43403 L
.52428 .43404 L
.51457 .43405 L
.50486 .43405 L
.49514 .43405 L
.48543 .43405 L
.47572 .43404 L
.46601 .43403 L
.45629 .43401 L
.44658 .43399 L
.43687 .43396 L
.42716 .43391 L
.41744 .43385 L
.40773 .43377 L
.39802 .43365 L
.38831 .43348 L
.37859 .43325 L
.36888 .43293 L
.35917 .43247 L
.34946 .43184 L
.33974 .43093 L
.33003 .42964 L
.32032 .42773 L
.31798 .42716 L
.31061 .42477 L
.30089 .41919 L
.29882 .41744 L
.29264 .40773 L
.29228 .39802 L
.29523 .38831 L
.30089 .37883 L
.30107 .37859 L
.30973 .36888 L
.31061 .36807 L
.32032 .36008 L
.32157 .35917 L
.33003 .35356 L
.33706 .34946 L
.33974 .34801 L
.34946 .34321 L
.3574 .33974 L
.35917 .33902 L
.36888 .33532 L
Mistroke
.37859 .33206 L
.38528 .33003 L
.38831 .32917 L
.39802 .32663 L
.40773 .32439 L
.41744 .32243 L
.42716 .32073 L
.42974 .32032 L
.43687 .31928 L
Mfstroke
.43687 .37819 m
.44658 .3777 L
.45629 .3773 L
.46601 .37699 L
.47572 .37676 L
.48543 .37661 L
.49514 .37654 L
.50486 .37654 L
.51457 .37661 L
.52428 .37676 L
.53399 .37699 L
.54371 .3773 L
.55342 .3777 L
.56313 .37819 L
.57013 .37859 L
.57284 .37877 L
.58256 .37945 L
.59227 .38024 L
.60198 .38115 L
.61169 .38217 L
.62141 .38333 L
.63112 .38464 L
.64083 .38611 L
.65054 .38778 L
.65341 .38831 L
.66026 .3892 L
.66997 .39091 L
.67968 .39338 L
.68939 .39668 L
.69126 .39802 L
.69649 .40773 L
.68939 .41294 L
.67968 .41363 L
.66997 .41427 L
.66026 .4147 L
.65054 .41499 L
.64083 .41519 L
.63112 .41532 L
.62141 .41542 L
.61169 .41549 L
.60198 .41554 L
.59227 .41557 L
.58256 .4156 L
.57284 .41561 L
.56313 .41563 L
.55342 .41563 L
.54371 .41564 L
.53399 .41564 L
.52428 .41565 L
.51457 .41565 L
Mistroke
.50486 .41565 L
.49514 .41565 L
.48543 .41565 L
.47572 .41565 L
.46601 .41564 L
.45629 .41564 L
.44658 .41563 L
.43687 .41563 L
.42716 .41561 L
.41744 .4156 L
.40773 .41557 L
.39802 .41554 L
.38831 .41549 L
.37859 .41542 L
.36888 .41532 L
.35917 .41519 L
.34946 .41499 L
.33974 .4147 L
.33003 .41427 L
.32032 .41363 L
.31061 .41294 L
.30351 .40773 L
.30874 .39802 L
.31061 .39668 L
.32032 .39338 L
.33003 .39091 L
.33974 .38921 L
.34659 .38831 L
.34946 .38778 L
.35917 .38611 L
.36888 .38464 L
.37859 .38333 L
.38831 .38217 L
.39802 .38115 L
.40773 .38024 L
.41744 .37946 L
.42716 .37877 L
.42986 .37859 L
.43687 .37819 L
Mfstroke
0 0 1 r
.3067 .40583 m
.6933 .40583 L
s
1 0 0 r
.3067 .59417 m
.6933 .59417 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 288},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`0001/2000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00b`3ooolI00000<`0oooo002b0?oooaT000006@3ooolI00000;<0oooo
002V0?ooo``00000B`3oool<00000:L0oooo002M0?ooo`T00000H`3oool9000009h0oooo002E0?oo
o`P00000M@3oool8000009H0oooo002A0?ooo`@00000Q@3oool400000980oooo00290?ooo`P00000
S@3oool8000008X0oooo00240?ooo`D00000W@3oool5000008D0oooo00200?ooo`@00000Y`3oool4
00000840oooo001l0?ooo`@00000[`3oool4000007d0oooo001h0?ooo`@00000]`3oool4000007T0
oooo001c0?ooo`D00000_`3oool5000007@0oooo001`0?ooo`<00000b@3oool300000740oooo001^
0?ooo`800000c`3oool2000006l0oooo001[0?ooo`<00000d`3oool3000006`0oooo001X0?ooo`<0
0000f@3oool3000006T0oooo001V0?ooo`800000g`3oool2000006L0oooo001T0?ooo`800000h`3o
ool2000006D0oooo001R0?ooo`800000i`3oool2000006<0oooo001N0?ooo`@00000j`3oool40000
05l0oooo001M0?ooo`030000003oool0oooo0?40oooo00<000000?ooo`3oool0G03oool005/0oooo
0P00003e0?ooo`800000G03oool005T0oooo0P00003i0?ooo`800000FP3oool005L0oooo0P00003m
0?ooo`800000F03oool005D0oooo0P00003o0?ooo`80oooo00<000000?ooo`3oool0E@3oool005<0
oooo0P00003o0?ooo`D0oooo0P00001E0?ooo`00DP3oool00`000000oooo0?ooo`3o0?ooo`L0oooo
00<000000?ooo`3oool0DP3oool00540oooo00<000000?ooo`3oool0o`3oool90?ooo`800000DP3o
ool004l0oooo0P00003o0?ooo`h0oooo0P00001@0?ooo`00CP3oool00`000000oooo0?ooo`3o0?oo
oa00oooo00<000000?ooo`3oool0C@3oool004d0oooo00<000000?ooo`3oool0o`3ooolB0?ooo`03
0000003oool0oooo04`0oooo001<0?ooo`030000003oool0oooo0?l0oooo503oool00`000000oooo
0?ooo`1;0?ooo`00BP3oool200000?l0oooo603oool2000004/0oooo00190?ooo`030000003oool0
oooo0?l0oooo6P3oool00`000000oooo0?ooo`180?ooo`00B03oool00`000000oooo0?ooo`3o0?oo
oa`0oooo00<000000?ooo`3oool0A`3oool004L0oooo00<000000?ooo`3oool0o`3ooolN0?ooo`03
0000003oool0oooo04H0oooo00160?ooo`030000003oool0oooo0?l0oooo803oool00`000000oooo
0?ooo`150?ooo`00A@3oool00`000000oooo0?ooo`3o0?ooob80oooo00<000000?ooo`3oool0A03o
ool004@0oooo00<000000?ooo`3oool0o`3ooolT0?ooo`030000003oool0oooo04<0oooo00130?oo
o`030000003oool0oooo0?l0oooo9P3oool00`000000oooo0?ooo`120?ooo`00@P3oool00`000000
oooo0?ooo`3o0?ooobP0oooo00<000000?ooo`3oool0@@3oool00440oooo00<000000?ooo`3oool0
o`3ooolZ0?ooo`030000003oool0oooo0400oooo00100?ooo`030000003oool0oooo0?l0oooo;03o
ool00`000000oooo0?ooo`0o0?ooo`00@03oool00`000000oooo0?ooo`240?ooob400000QP3oool0
0`000000oooo0?ooo`0o0?ooo`00?`3oool00`000000oooo0?ooo`1m0?ooo`P000008@3oool80000
07l0oooo00<000000?ooo`3oool0?P3oool003h0oooo00<000000?ooo`3oool0M@3oool900000340
oooo2@00001g0?ooo`030000003oool0oooo03d0oooo000m0?ooo`030000003oool0oooo0780oooo
100000130?ooo`@00000M03oool00`000000oooo0?ooo`0l0?ooo`00?03oool00`000000oooo0?oo
o`1_0?ooo`@00000B`3oool400000700oooo00<000000?ooo`3oool0?03oool003`0oooo00<00000
0?ooo`3oool0J03oool7000005<0oooo1`00001Z0?ooo`030000003oool0oooo03/0oooo000k0?oo
o`030000003oool0oooo06L0oooo0P00001Q0?ooo`800000J@3oool00`000000oooo0?ooo`0j0?oo
o`00>P3oool00`000000oooo0?ooo`1T0?ooo`@00000I@3oool4000006D0oooo00<000000?ooo`3o
ool0>P3oool003X0oooo00<000000?ooo`3oool0H03oool4000006d0oooo1000001R0?ooo`030000
003oool0oooo03T0oooo000i0?ooo`030000003oool0oooo05h0oooo0`00001e0?ooo`<00000G`3o
ool00`000000oooo0?ooo`0i0?ooo`00>@3oool00`000000oooo0?ooo`1L0?ooo`800000N`3oool2
000005h0oooo00<000000?ooo`3oool0>03oool003P0oooo00<000000?ooo`3oool0FP3oool30000
07l0oooo0`00001L0?ooo`030000003oool0oooo03L0oooo000g0?ooo`030000003oool0oooo05P0
oooo0`0000250?ooo`<00000F@3oool00`000000oooo0?ooo`0g0?ooo`00=`3oool00`000000oooo
0?ooo`1F0?ooo`800000R`3oool2000005P0oooo00<000000?ooo`3oool0=P3oool003H0oooo00<0
00000?ooo`3oool0E@3oool2000008l0oooo0P00001G0?ooo`030000003oool0oooo03D0oooo000f
0?ooo`030000003oool0oooo05<0oooo0P00002C0?ooo`800000E@3oool00`000000oooo0?ooo`0e
0?ooo`00=@3oool00`000000oooo0?ooo`1A0?ooo`<00000U`3oool3000005<0oooo00<000000?oo
o`3oool0=03oool003D0oooo00<000000?ooo`3oool0C`3oool2000009d0oooo0P00001A0?ooo`03
0000003oool0oooo03@0oooo000d0?ooo`030000003oool0oooo04h0oooo0P00002Q0?ooo`800000
D03oool00`000000oooo0?ooo`0c0?ooo`00=03oool00`000000oooo0?ooo`1=0?ooo`030000003o
ool0oooo0:<0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`0c0?ooo`00=03oool0
0`000000oooo0?ooo`1;0?ooo`800000Y`3oool2000004d0oooo00<000000?ooo`3oool0<`3oool0
03<0oooo00<000000?ooo`3oool0B`3oool00`000000oooo0?ooo`2Y0?ooo`030000003oool0oooo
04/0oooo00<000000?ooo`3oool0<P3oool003<0oooo00<000000?ooo`3oool0BP3oool00`000000
oooo0?ooo`2[0?ooo`030000003oool0oooo04X0oooo00<000000?ooo`3oool0<P3oool003<0oooo
00<000000?ooo`3oool0B@3oool00`000000oooo0?ooo`2]0?ooo`030000003oool0oooo04T0oooo
00<000000?ooo`3oool0<P3oool00380oooo00<000000?ooo`3oool0B03oool200000;40oooo0P00
00190?ooo`030000003oool0oooo0380oooo000b0?ooo`030000003oool0oooo04L0oooo00<00000
0?ooo`3oool0/`3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo0340oooo000a0?oo
o`030000003oool0oooo04H0oooo0P00002g0?ooo`800000A`3oool00`000000oooo0?ooo`0a0?oo
o`00<@3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo0;T0oooo00<000000?ooo`3o
ool0A@3oool00`000000oooo0?ooo`0`0?ooo`00<@3oool00`000000oooo0?ooo`140?ooo`030000
003oool0oooo0;/0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`0`0?ooo`00<03o
ool00`000000oooo0?ooo`130?ooo`800000_`3oool2000004@0oooo00<000000?ooo`3oool0<03o
ool00300oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`310?ooo`030000003oool0
oooo0480oooo00<000000?ooo`3oool0;`3oool00300oooo00<000000?ooo`3oool0@@3oool00`00
0000oooo0?ooo`330?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0;`3oool002l0
oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`350?ooo`030000003oool0oooo0400
oooo00<000000?ooo`3oool0;`3oool002l0oooo00<000000?ooo`3oool0?`3oool200000<T0oooo
0P0000110?ooo`030000003oool0oooo02h0oooo000^0?ooo`030000003oool0oooo03l0oooo00<0
00000?ooo`3oool0b`3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo02h0oooo000^
0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0b`3oool00`000000oooo0?ooo`0o
0?ooo`030000003oool0oooo02d0oooo000^0?ooo`030000003oool0oooo03h0oooo00<000000?oo
o`3oool0F@3ooolI000005/0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0]0?oo
o`00;@3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo0580oooo2000000I0?ooo`P0
0000E03oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo02d0oooo000]0?ooo`030000
003oool0oooo03d0oooo00<000000?ooo`3oool0B`3oool8000002T0oooo2000001=0?ooo`030000
003oool0oooo03d0oooo00<000000?ooo`3oool0;03oool002d0oooo00<000000?ooo`3oool0?03o
ool00`000000oooo0?ooo`160?ooo`H00000>@3oool6000004P0oooo00<000000?ooo`3oool0?03o
ool00`000000oooo0?ooo`0/0?ooo`00;@3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0
oooo04@0oooo0`0000150?ooo`<00000AP3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0
oooo02`0oooo000]0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0@@3oool40000
04/0oooo100000120?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0;03oool002d0
oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0m0?ooo`@00000D`3oool4000003l0
oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0/0?ooo`00;@3oool00`000000oooo
0?ooo`0i0?ooo`030000003oool0oooo03X0oooo1000001K0?ooo`@00000?03oool00`000000oooo
0?ooo`0i0?ooo`030000003oool0oooo02`0oooo000]0?ooo`030000003oool0oooo03P0oooo00<0
00000?ooo`3oool0>@3oool2000006<0oooo0P00000k0?ooo`030000003oool0oooo03P0oooo00<0
00000?ooo`3oool0;03oool002d0oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`0g
0?ooo`<00000I`3oool3000003P0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`0/
0?ooo`00;03oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo03@0oooo0`00001]0?oo
o`<00000=P3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo02/0oooo000/0?ooo`03
0000003oool0oooo03L0oooo00<000000?ooo`3oool0<`3oool2000007<0oooo0P00000d0?ooo`03
0000003oool0oooo03P0oooo00<000000?ooo`3oool0:`3oool002`0oooo00<000000?ooo`3oool0
=`3oool00`000000oooo0?ooo`0a0?ooo`800000M`3oool2000003<0oooo00<000000?ooo`3oool0
=`3oool00`000000oooo0?ooo`0[0?ooo`00;03oool00`000000oooo0?ooo`0f0?ooo`030000003o
ool0oooo0300oooo0P00001k0?ooo`800000<P3oool00`000000oooo0?ooo`0f0?ooo`030000003o
ool0oooo02/0oooo000[0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0;P3oool2
000007l0oooo0P00000`0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool0:`3oool0
02/0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0]0?ooo`800000P`3oool20000
02l0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000
oooo0?ooo`0e0?ooo`030000003oool0oooo02`0oooo0P0000270?ooo`800000;P3oool00`000000
oooo0?ooo`0e0?ooo`030000003oool0oooo02X0oooo000[0?ooo`030000003oool0oooo03@0oooo
00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`290?ooo`030000003oool0oooo02/0oooo
00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?oo
o`0d0?ooo`030000003oool0oooo02X0oooo0P00002=0?ooo`800000;03oool00`000000oooo0?oo
o`0d0?ooo`030000003oool0oooo02X0oooo000[0?ooo`030000003oool0oooo03<0oooo00<00000
0?ooo`3oool0:P3oool00`000000oooo0?ooo`2?0?ooo`030000003oool0oooo02T0oooo00<00000
0?ooo`3oool0=03oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo02P0oooo0P00002C0?ooo`800000:P3oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo02X0oooo000[0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3o
ool09`3oool00`000000oooo0?ooo`2E0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3o
ool0<`3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0b0?ooo`030000
003oool0oooo02L0oooo00<000000?ooo`3oool0U`3oool00`000000oooo0?ooo`0V0?ooo`030000
003oool0oooo03<0oooo00<000000?ooo`3oool0:P3oool002/0oooo00<000000?ooo`3oool0<P3o
ool00`000000oooo0?ooo`0U0?ooo`800000V`3oool2000002L0oooo00<000000?ooo`3oool0<P3o
ool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0
oooo02@0oooo00<000000?ooo`3oool0W@3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0
oooo0380oooo00<000000?ooo`3oool0:P3oool002/0oooo00<000000?ooo`3oool0<@3oool00`00
0000oooo0?ooo`0S0?ooo`800000X@3oool2000002@0oooo00<000000?ooo`3oool0<P3oool00`00
0000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo0280
oooo00<000000?ooo`3oool0X`3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0340
oooo00<000000?ooo`3oool0:P3oool002/0oooo00<000000?ooo`3oool0<03oool00`000000oooo
0?ooo`0R0?ooo`030000003oool0oooo0:D0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`0a0?ooo`030000003oool0oooo02X0oooo000[0?ooo`030000003oool0oooo0300oooo00<0
00000?ooo`3oool08@3oool00`000000oooo0?ooo`2W0?ooo`030000003oool0oooo0240oooo00<0
00000?ooo`3oool0<03oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0`
0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0Z@3oool00`000000oooo0?ooo`0P
0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0:P3oool002/0oooo00<000000?oo
o`3oool0<03oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo04d0oooo3`00001?0?oo
o`030000003oool0oooo01l0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`0Z0?oo
o`00:`3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3o
ool0@@3oool=000000l0oooo3@0000120?ooo`030000003oool0oooo0200oooo00<000000?ooo`3o
ool0;`3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0_0?ooo`030000
003oool0oooo01l0oooo00<000000?ooo`3oool0>@3oool8000002T0oooo2000000k0?ooo`030000
003oool0oooo01l0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0Z0?ooo`00:`3o
ool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0<@3o
ool9000003T0oooo2@00000c0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0;`3o
ool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0
oooo01d0oooo00<000000?ooo`3oool0;P3oool4000004/0oooo1000000`0?ooo`030000003oool0
oooo01d0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`00
0000oooo0?ooo`0^0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0:`3oool40000
05<0oooo1000000]0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0;`3oool00`00
0000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo01d0
oooo00<000000?ooo`3oool0:03oool3000005/0oooo0`00000Z0?ooo`030000003oool0oooo01d0
oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo
0?ooo`0^0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool09`3oool200000640oooo
0P00000Y0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0;P3oool00`000000oooo
0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool0903oool4000006D0oooo1000000V0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool0;P3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0^
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool08@3oool3000006d0oooo0`00000S
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0Z
0?ooo`00:`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool0803oool2000007<0oooo0P00000R0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool0;P3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0^0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool07P3oool3000007L0oooo0`00000P0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0Z0?ooo`00
:`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
703oool3000007d0oooo0`00000M0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0
;P3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0^0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool06P3oool2000008<0oooo0P00000L0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0[0?ooo`00;03oool0
0`000000oooo0?ooo`0]0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool06@3oool2
000008L0oooo0P00000J0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0;@3oool0
0`000000oooo0?ooo`0[0?ooo`00;03oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo
01L0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`290?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo
02/0oooo000/0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`0F0?ooo`800000S@3oool2000001P0oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`0]0?ooo`030000003oool0oooo02/0oooo000/0?ooo`030000003oool0oooo02d0oooo
00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo08l0oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo02`0oooo
00<000000?ooo`3oool0;03oool002d0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?oo
o`0F0?ooo`030000003oool0oooo01@0oooo0P00002C0?ooo`8000005P3oool00`000000oooo0?oo
o`0F0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0;03oool002d0oooo00<00000
0?ooo`3oool0;03oool00`000000oooo0?ooo`0E0?ooo`800000503oool2000009L0oooo0P00000D
0?ooo`8000005`3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo02`0oooo000]0?oo
o`030000003oool0oooo02`0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo09L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0E0?oo
o`030000003oool0oooo02`0oooo00<000000?ooo`3oool0;03oool002d0oooo00<000000?ooo`3o
ool0;03oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0180oooo0P00002K0?ooo`80
00004`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3o
ool0;03oool002d0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo0100oooo0P00002O0?ooo`8000004P3oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo02`0oooo00<000000?ooo`3oool0;03oool002h0oooo00<000000?ooo`3oool0:`3o
ool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0X@3o
ool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0:`3o
ool00`000000oooo0?ooo`0]0?ooo`00;P3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0
oooo01@0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`2S0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0[0?ooo`030000003oool0
oooo02d0oooo000^0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo0:D0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0;@3oool002l0
oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00d0
oooo00<000000?ooo`3oool0Y@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0^0?ooo`00;`3oool00`000000oooo
0?ooo`0[0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`0f0?ooocT00000>03oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0^0?ooo`00;`3oool00`000000oooo0?oo
o`0[0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0Z0?ooo`d00000>@3oool=000002`0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0C0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0;P3oool002l0oooo00<00000
0?ooo`3oool0:`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool08`3oool8000005<0oooo2000000T0?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool04`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02l0oooo000`0?oo
o`030000003oool0oooo02X0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo01X0oooo2@00001S0?ooo`T00000703oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo0180oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0_0?oo
o`00<03oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`0G0?ooo`@00000M@3oool4000001P0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3o
ool0<03oool00340oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0B0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool03`3oool8000007d0oooo2000000A0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0Z0?ooo`030000
003oool0oooo0300oooo000b0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool04P3o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00`0oooo1000002=0?ooo`@000003@3o
ool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0:P3o
ool00`000000oooo0?ooo`0a0?ooo`00<P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`D00000U@3oool5
000000T0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo02X0oooo00<000000?ooo`3oool0<@3oool003<0oooo00<000000?ooo`3oool0:@3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01@3oool30000
09l0oooo0`0000060?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0Z0?ooo`030000003oool0oooo0380oooo000c0?ooo`030000003oool0oooo02X0
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0X@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo0380
oooo000d0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0X`3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`0Z0?ooo`030000003oool0oooo03<0oooo000d0?ooo`030000003oool0oooo02X0oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0Y03oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo03@0oooo000e
0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo0080oooo00<000000000o`000?l0Y@000?l00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0Z
0?ooo`030000003oool0oooo03@0oooo000f0?ooo`030000003oool0oooo02T0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`070?ooo`050000003oool0oooo0?ooo`000000Z@3oool0
1@000000oooo0?ooo`3oool0000000T0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?oo
o`0Y0?ooo`030000003oool0oooo03D0oooo000f0?ooo`030000003oool0oooo02T0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0080oooo0P00002U
0?ooo`8000000`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool0:P3oool00`000000oooo0?ooo`0e0?ooo`00=`3oool00`000000oooo0?ooo`0Y0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`030?oo
o`<00000W`3oool3000000D0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0?0?oo
o`030000003oool0oooo02T0oooo00<000000?ooo`3oool0=P3oool003P0oooo00<000000?ooo`3o
ool0:03oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool01P3ooolD000007L0oooo500000070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool03`3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo03L0oooo000i0?ooo`030000
003oool0oooo02P0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo01T0ooooM`00000K0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo03P0oooo000i0?ooo`030000003o
ool0oooo02T0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo0:T0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo02P0oooo00<000000?ooo`3oool0>@3oool003X0oooo00<000000?ooo`3oool0:@3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0Y`3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0:@3oool0
0`000000oooo0?ooo`0i0?ooo`00>`3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo
00l0oooo00<000000?ooo`3oool0203oool200000:D0oooo0P00000:0?ooo`030000003oool0oooo
00l0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`0j0?ooo`00>`3oool00`000000
oooo0?ooo`0Y0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool02@3oool200000:40
oooo0P00000;0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0:@3oool00`000000
oooo0?ooo`0j0?ooo`00?03oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo00l0oooo
00<000000?ooo`3oool02P3oool5000009L0oooo1@00000<0?ooo`030000003oool0oooo00l0oooo
00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0k0?ooo`00?@3oool2000002X0oooo00<0
00000?ooo`3oool03`3oool2000000l0oooo200000270?ooo`P000003`3oool200000140oooo00<0
00000?ooo`3oool0:03oool2000003h0oooo000o0?ooo`030000003oool0oooo02P0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0D0?ooob800000@`3ooolR000001H0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo03h0oooo00100?oo
o`030000003oool0oooo02P0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0e0?oo
od<00000=`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo02P0oooo00<000000?oo
o`3oool0?`3oool00440oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo0:d0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0X0?ooo`03
0000003oool0oooo0400oooo00120?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0
403oool200000:/0oooo0P00000B0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0
@@3oool004<0oooo0P00000Y0?ooo`8000004P3oool200000:L0oooo0P00000B0?ooo`800000:@3o
ool2000004@0oooo00150?ooo`800000:@3oool00`000000oooo0?ooo`0A0?ooo`@00000W`3oool4
000001<0oooo00<000000?ooo`3oool09`3oool2000004H0oooo00170?ooo`030000003oool0oooo
02L0oooo00<000000?ooo`3oool0503oool5000009D0oooo1@00000F0?ooo`030000003oool0oooo
02L0oooo00<000000?ooo`3oool0AP3oool004P0oooo0P00000X0?ooo`8000006@3oool7000008L0
oooo1`00000I0?ooo`800000:03oool2000004T0oooo001:0?ooo`800000:03oool2000001h0oooo
6@00001E0?oooaT000007P3oool2000002P0oooo0P00001;0?ooo`00C03oool2000002P0oooo0P00
000e0?oooeD00000=@3oool2000002P0oooo0P00001=0?ooo`00CP3oool00`000000oooo0?ooo`0W
0?ooo`800000^`3oool2000002T0oooo00<000000?ooo`3oool0C@3oool004l0oooo00<000000?oo
o`3oool0:03oool200000;L0oooo0P00000Z0?ooo`030000003oool0oooo04h0oooo001@0?ooo`80
0000:P3oool300000;40oooo0`00000Z0?ooo`800000D@3oool00580oooo0`00000Z0?ooo`@00000
Z@3oool4000002X0oooo0`00001C0?ooo`00E@3oool4000002X0oooo1P00002M0?ooo`H00000:P3o
ool4000005H0oooo001I0?ooo`800000;P3oool7000008l0oooo1`00000^0?ooo`800000FP3oool0
05/0oooo0`00000b0?oooaD00000I@3ooolE00000380oooo0`00001L0?ooo`00GP3oool3000004@0
ooooI@0000140?ooo`<00000G`3oool00640oooo0P0000000`3oool000000?ooo`3T0?ooo`040000
003oool0000000000680oooo001S0?ooo`<00000h`3oool3000006@0oooo001V0?ooo`@00000f`3o
ool4000006L0oooo001Z0?ooo`T00000b@3oool9000006/0oooo001c0?ooo`@00000`@3oool40000
07@0oooo001g0?ooo`P00000/@3oool8000007P0oooo001o0?oooaD00000Q`3ooolE00000800oooo
002D0?ooohL00000U@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo
/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool009D0ooooQ@00002F0?ooo`00
P03ooolE000008D0oooo5@0000210?ooo`00N03oool800000:l0oooo2000001i0?ooo`00L`3oool5
00000;l0oooo1@00001d0?ooo`00J`3oool800000<T0oooo2000001/0?ooo`00I`3oool400000=T0
oooo1000001X0?ooo`00I03oool300000>40oooo0`00001U0?ooo`00HP3oool200000>L0oooo0P00
001S0?ooo`00G`3oool3000004@0ooooH`0000140?ooo`<00000H03oool005/0oooo1000000b0?oo
oaD00000H`3ooolE00000380oooo1000001L0?ooo`00F@3oool2000002l0oooo1`00002=0?ooo`L0
0000;`3oool2000005X0oooo001F0?ooo`<00000:`3oool6000009/0oooo1P00000[0?ooo`<00000
E`3oool005<0oooo0`00000Z0?ooo`@00000Y`3oool4000002X0oooo0`00001D0?ooo`00D@3oool2
000002X0oooo0`00002_0?ooo`<00000:P3oool200000580oooo001?0?ooo`800000:P3oool20000
0;D0oooo0P00000Z0?ooo`800000D03oool004h0oooo00<000000?ooo`3oool0:03oool200000;T0
oooo0P00000Z0?ooo`030000003oool0oooo04d0oooo001<0?ooo`800000:@3oool2000003D0oooo
D`00000e0?ooo`800000:@3oool2000004d0oooo001;0?ooo`030000003oool0oooo02L0oooo0P00
000N0?oooaT00000D`3ooolI000001h0oooo0P00000Y0?ooo`030000003oool0oooo04X0oooo0019
0?ooo`800000:03oool2000001P0oooo200000250?ooo`P00000603oool2000002P0oooo0P00001:
0?ooo`00A`3oool2000002P0oooo0P00000E0?ooo`D00000U@3oool5000001D0oooo0P00000X0?oo
o`800000B03oool004D0oooo0P00000Y0?ooo`030000003oool0oooo0180oooo0`00002O0?ooo`<0
0000503oool00`000000oooo0?ooo`0W0?ooo`800000AP3oool004<0oooo0P00000Z0?ooo`030000
003oool0oooo0140oooo0P00002U0?ooo`8000004`3oool00`000000oooo0?ooo`0X0?ooo`800000
A03oool00480oooo00<000000?ooo`3oool0:03oool200000180oooo0P00002Y0?ooo`8000004P3o
ool2000002X0oooo00<000000?ooo`3oool0@@3oool00440oooo00<000000?ooo`3oool0:03oool0
0`000000oooo0?ooo`0@0?ooo`800000[@3oool200000180oooo00<000000?ooo`3oool0:03oool0
0`000000oooo0?ooo`100?ooo`00@03oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0=@3ooom3000003L0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0X0?ooo`030000003oool0oooo03l0oooo000o0?ooo`030000003oool0oooo02P0
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0E0?ooob400000@`3ooolQ000001L0
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo03h0
oooo000n0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool03`3oool200000100oooo
200000250?ooo`P00000403oool200000140oooo00<000000?ooo`3oool0:03oool00`000000oooo
0?ooo`0m0?ooo`00?03oool2000002X0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?oo
o`0;0?ooo`D00000U@3oool5000000d0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?oo
o`0X0?ooo`800000?@3oool003/0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo00T0oooo0`00002O0?ooo`<000002`3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0>P3oool003/0oooo00<000000?oo
o`3oool0:03oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00T0oooo00<000000?oo
o`3oool0X`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool0:03oool00`000000oooo0?ooo`0j0?ooo`00>P3oool00`000000oooo0?ooo`0Y0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool0203oool200000:L0oooo0P0000090?ooo`03
0000003oool0oooo00l0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0i0?ooo`00
>@3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`2Y0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo03T0oooo000i0?ooo`030000003o
ool0oooo02P0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo01X0ooooM@00000L0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo03P0oooo000h0?ooo`030000003oool0
oooo02P0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo00H0oooo5@00001e0?oooaD000001`3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00l0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0g0?ooo`00=`3oool00`00
0000oooo0?ooo`0Y0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`030?ooo`<00000W`3oool3000000D0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0=P3oool003H0
oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool00P3oool200000:D0oooo0P0000030?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo03D0
oooo000f0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`070?ooo`050000003oool0oooo0?ooo`000000Z@3oool01@000000oooo0?ooo`3oool00000
00T0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo
03D0oooo000e0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo0080oooo00<000000?l0003o0000Y@3o00000`000000
oooo0?ooo`020?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`0Z0?ooo`030000003oool0oooo03@0oooo000d0?ooo`030000003oool0oooo02X0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0Y03oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo03@0oooo
000d0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0X`3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?oo
o`0Z0?ooo`030000003oool0oooo03<0oooo000c0?ooo`030000003oool0oooo02X0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool0X@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo0380oooo000c0?oo
o`030000003oool0oooo02T0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo00D0oooo0`00002O0?ooo`<000001P3oool00`000000oooo0?ooo`080?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0b0?oo
o`00<P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool0203oool00`000000oooo0?ooo`070?ooo`@00000U`3oool4000000T0oooo00<000000?ooo`3o
ool0203oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3o
ool0<@3oool00380oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0B0?ooo`030000
003oool0oooo00P0oooo00<000000?ooo`3oool02`3oool4000008l0oooo1000000<0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0Z0?ooo`030000
003oool0oooo0340oooo000a0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool04P3o
ool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00h0oooo2000001o0?ooo`P00000403o
ool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0:P3o
ool00`000000oooo0?ooo`0`0?ooo`00<03oool00`000000oooo0?ooo`0[0?ooo`030000003oool0
oooo0180oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0F0?ooo`@00000M`3oool4
000001L0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo02X0oooo00<000000?ooo`3oool0<03oool00300oooo00<000000?ooo`3oool0:P3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool06@3oool90000
06D0oooo2@00000K0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`0[0?ooo`030000003oool0oooo02l0oooo000_0?ooo`030000003oool0oooo02/0
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0280
oooo2000001E0?ooo`P000008`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0_0?ooo`00;`3oool00`000000oooo
0?ooo`0[0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`0Y0?oooa400000<`3ooolA000002/0oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`0C0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0;P3oool002l0oooo00<0
00000?ooo`3oool0:`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool0>@3ooolc000003/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0C0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0;P3oool002l0oooo00<00000
0?ooo`3oool0:`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool0Y@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01<0oooo00<00000
0?ooo`3oool0:`3oool00`000000oooo0?ooo`0^0?ooo`00;P3oool00`000000oooo0?ooo`0[0?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`2U0?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0[0?oo
o`030000003oool0oooo02d0oooo000^0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0:<0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3o
ool0;@3oool002h0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo00l0oooo00<000000?ooo`3oool0X@3oool00`000000oooo0?ooo`0?0?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0]0?ooo`00;@3o
ool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0403o
ool2000009l0oooo0P00000B0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0;03o
ool00`000000oooo0?ooo`0/0?ooo`00;@3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0
oooo01@0oooo00<000000?ooo`3oool04P3oool2000009/0oooo0P00000C0?ooo`030000003oool0
oooo01D0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0/0?ooo`00;@3oool00`00
0000oooo0?ooo`0/0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool04`3oool00`00
0000oooo0?ooo`2G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool05@3oool00`00
0000oooo0?ooo`0/0?ooo`030000003oool0oooo02`0oooo000]0?ooo`030000003oool0oooo02`0
oooo00<000000?ooo`3oool05@3oool2000001@0oooo00<000000?ooo`3oool0U`3oool00`000000
oooo0?ooo`0B0?ooo`8000005`3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo02`0
oooo000]0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool05P3oool00`000000oooo
0?ooo`0C0?ooo`800000U@3oool2000001D0oooo00<000000?ooo`3oool05P3oool00`000000oooo
0?ooo`0/0?ooo`030000003oool0oooo02`0oooo000/0?ooo`030000003oool0oooo02d0oooo00<0
00000?ooo`3oool05P3oool00`000000oooo0?ooo`0E0?ooo`800000T@3oool2000001H0oooo00<0
00000?ooo`3oool05`3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo02`0oooo000/
0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0F
0?ooo`800000S@3oool2000001P0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0]
0?ooo`030000003oool0oooo02/0oooo000/0?ooo`030000003oool0oooo02d0oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo08T0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo02d0oooo00<000000?oo
o`3oool0:`3oool002`0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool0Q`3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0[0?ooo`00
:`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
6@3oool2000008D0oooo0P00000K0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
;@3oool00`000000oooo0?ooo`0[0?ooo`00:`3oool00`000000oooo0?ooo`0^0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool06`3oool3000007l0oooo0`00000L0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool0
0`000000oooo0?ooo`0^0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool07@3oool3
000007T0oooo0`00000O0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0;P3oool0
0`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool07`3oool2000007D0oooo0P00000Q0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000
oooo0?ooo`0^0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0803oool4000006d0
oooo1000000R0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0;P3oool00`000000
oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo01/0oooo
00<000000?ooo`3oool0903oool4000006D0oooo1000000V0?ooo`030000003oool0oooo01/0oooo
00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?oo
o`0^0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool09`3oool200000640oooo0P00
000Y0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?oo
o`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo01d0oooo00<00000
0?ooo`3oool0:03oool2000005d0oooo0P00000Z0?ooo`030000003oool0oooo01d0oooo00<00000
0?ooo`3oool0;P3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0^0?oo
o`030000003oool0oooo01d0oooo00<000000?ooo`3oool0:P3oool4000005D0oooo1000000/0?oo
o`030000003oool0oooo01`0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0Z0?oo
o`00:`3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3o
ool0;@3oool4000004d0oooo1000000_0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3o
ool0;`3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0_0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool0<03oool:000003T0oooo2P00000b0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0Z0?ooo`00:`3o
ool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0>@3o
ool7000002/0oooo1`00000k0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0;`3o
ool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0
oooo01l0oooo00<000000?ooo`3oool0@03oool=00000140oooo3@0000110?ooo`030000003oool0
oooo0200oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`00
0000oooo0?ooo`0`0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0C03ooolA0000
04h0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo
02X0oooo000[0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0803oool00`000000
oooo0?ooo`2Y0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0<03oool00`000000
oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo0240oooo
00<000000?ooo`3oool0Y`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0300oooo
00<000000?ooo`3oool0:P3oool002/0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?oo
o`0R0?ooo`030000003oool0oooo0:D0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`0a0?ooo`030000003oool0oooo02X0oooo000[0?ooo`030000003oool0oooo0340oooo00<00000
0?ooo`3oool08P3oool00`000000oooo0?ooo`2S0?ooo`030000003oool0oooo0280oooo00<00000
0?ooo`3oool0<@3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0a0?oo
o`030000003oool0oooo02<0oooo00<000000?ooo`3oool0X@3oool00`000000oooo0?ooo`0R0?oo
o`030000003oool0oooo0380oooo00<000000?ooo`3oool0:P3oool002/0oooo00<000000?ooo`3o
ool0<P3oool00`000000oooo0?ooo`0S0?ooo`800000W`3oool2000002D0oooo00<000000?ooo`3o
ool0<P3oool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0b0?ooo`030000
003oool0oooo02D0oooo0P00002K0?ooo`8000009`3oool00`000000oooo0?ooo`0b0?ooo`030000
003oool0oooo02X0oooo000[0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool09`3o
ool00`000000oooo0?ooo`2G0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0<`3o
ool00`000000oooo0?ooo`0Z0?ooo`00:`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0
oooo02L0oooo00<000000?ooo`3oool0U@3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0
oooo03<0oooo00<000000?ooo`3oool0:P3oool002/0oooo00<000000?ooo`3oool0<`3oool00`00
0000oooo0?ooo`0X0?ooo`030000003oool0oooo09<0oooo00<000000?ooo`3oool0:03oool00`00
0000oooo0?ooo`0c0?ooo`030000003oool0oooo02X0oooo000[0?ooo`030000003oool0oooo03<0
oooo00<000000?ooo`3oool0:@3oool200000940oooo0P00000Z0?ooo`030000003oool0oooo03@0
oooo00<000000?ooo`3oool0:P3oool002/0oooo00<000000?ooo`3oool0=03oool00`000000oooo
0?ooo`0Z0?ooo`030000003oool0oooo08d0oooo00<000000?ooo`3oool0:P3oool00`000000oooo
0?ooo`0d0?ooo`030000003oool0oooo02X0oooo000[0?ooo`030000003oool0oooo03@0oooo00<0
00000?ooo`3oool0:`3oool2000008/0oooo0P00000/0?ooo`030000003oool0oooo03D0oooo00<0
00000?ooo`3oool0:P3oool002/0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`0/
0?ooo`800000Q`3oool2000002h0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`0Z
0?ooo`00:`3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo02d0oooo0P0000230?oo
o`800000;`3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo02X0oooo000[0?ooo`03
0000003oool0oooo03L0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`1o0?ooo`03
0000003oool0oooo02h0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0[0?ooo`00
;03oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo02l0oooo0P00001m0?ooo`800000
<@3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo02/0oooo000/0?ooo`030000003o
ool0oooo03L0oooo00<000000?ooo`3oool0<03oool3000007L0oooo0`00000b0?ooo`030000003o
ool0oooo03L0oooo00<000000?ooo`3oool0:`3oool002`0oooo00<000000?ooo`3oool0=`3oool0
0`000000oooo0?ooo`0c0?ooo`800000L`3oool2000003@0oooo00<000000?ooo`3oool0>03oool0
0`000000oooo0?ooo`0[0?ooo`00;03oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo
03@0oooo0`00001]0?ooo`<00000=P3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo
02/0oooo000]0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0=`3oool3000006L0
oooo0`00000h0?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3oool0;03oool002d0oooo
00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`0i0?ooo`800000H`3oool2000003/0oooo
00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`0/0?ooo`00;@3oool00`000000oooo0?oo
o`0i0?ooo`030000003oool0oooo03X0oooo0`00001M0?ooo`<00000?03oool00`000000oooo0?oo
o`0i0?ooo`030000003oool0oooo02`0oooo000]0?ooo`030000003oool0oooo03X0oooo00<00000
0?ooo`3oool0?03oool4000005D0oooo1000000n0?ooo`030000003oool0oooo03X0oooo00<00000
0?ooo`3oool0;03oool002d0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`100?oo
o`@00000C@3oool400000440oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`0/0?oo
o`00;@3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo04<0oooo100000150?ooo`@0
0000A@3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo02`0oooo000]0?ooo`030000
003oool0oooo03`0oooo00<000000?ooo`3oool0AP3oool5000003/0oooo1@0000180?ooo`030000
003oool0oooo03`0oooo00<000000?ooo`3oool0;03oool002d0oooo00<000000?ooo`3oool0?@3o
ool00`000000oooo0?ooo`1:0?ooo`P00000:`3oool8000004`0oooo00<000000?ooo`3oool0?@3o
ool00`000000oooo0?ooo`0/0?ooo`00;@3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0
oooo0540oooo2@00000I0?ooo`T00000D`3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0
oooo02d0oooo000^0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0F@3ooolI0000
05/0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0]0?ooo`00;P3oool00`000000
oooo0?ooo`0o0?ooo`030000003oool0oooo0</0oooo00<000000?ooo`3oool0?`3oool00`000000
oooo0?ooo`0]0?ooo`00;P3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo0</0oooo
00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0^0?ooo`00;`3oool00`000000oooo0?oo
o`0o0?ooo`030000003oool0oooo0<T0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?oo
o`0^0?ooo`00;`3oool00`000000oooo0?ooo`100?ooo`800000a`3oool200000440oooo00<00000
0?ooo`3oool0;`3oool00300oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`330?oo
o`030000003oool0oooo0440oooo00<000000?ooo`3oool0;`3oool00300oooo00<000000?ooo`3o
ool0@P3oool00`000000oooo0?ooo`310?ooo`030000003oool0oooo0480oooo00<000000?ooo`3o
ool0;`3oool00300oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`2o0?ooo`030000
003oool0oooo0480oooo00<000000?ooo`3oool0<03oool00340oooo00<000000?ooo`3oool0@`3o
ool200000;d0oooo0P0000150?ooo`030000003oool0oooo0300oooo000a0?ooo`030000003oool0
oooo04D0oooo00<000000?ooo`3oool0^@3oool00`000000oooo0?ooo`150?ooo`030000003oool0
oooo0300oooo000a0?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool0]`3oool00`00
0000oooo0?ooo`150?ooo`030000003oool0oooo0340oooo000b0?ooo`030000003oool0oooo04H0
oooo0P00002e0?ooo`800000B03oool00`000000oooo0?ooo`0a0?ooo`00<P3oool00`000000oooo
0?ooo`180?ooo`030000003oool0oooo0;40oooo00<000000?ooo`3oool0A`3oool00`000000oooo
0?ooo`0b0?ooo`00<`3oool00`000000oooo0?ooo`180?ooo`800000[`3oool2000004X0oooo00<0
00000?ooo`3oool0<P3oool003<0oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`2[
0?ooo`030000003oool0oooo04X0oooo00<000000?ooo`3oool0<P3oool003<0oooo00<000000?oo
o`3oool0B`3oool00`000000oooo0?ooo`2Y0?ooo`030000003oool0oooo04/0oooo00<000000?oo
o`3oool0<P3oool003@0oooo00<000000?ooo`3oool0B`3oool00`000000oooo0?ooo`2W0?ooo`03
0000003oool0oooo04/0oooo00<000000?ooo`3oool0<`3oool003@0oooo00<000000?ooo`3oool0
C03oool200000:D0oooo0P00001>0?ooo`030000003oool0oooo03<0oooo000d0?ooo`030000003o
ool0oooo04h0oooo0P00002Q0?ooo`800000D03oool00`000000oooo0?ooo`0c0?ooo`00=@3oool0
0`000000oooo0?ooo`1?0?ooo`800000W@3oool200000540oooo00<000000?ooo`3oool0=03oool0
03D0oooo00<000000?ooo`3oool0D@3oool2000009T0oooo0P00001C0?ooo`030000003oool0oooo
03@0oooo000f0?ooo`030000003oool0oooo0580oooo0P00002E0?ooo`800000E03oool00`000000
oooo0?ooo`0e0?ooo`00=P3oool00`000000oooo0?ooo`1D0?ooo`800000T@3oool2000005H0oooo
00<000000?ooo`3oool0=@3oool003L0oooo00<000000?ooo`3oool0E@3oool2000008d0oooo0P00
001G0?ooo`030000003oool0oooo03H0oooo000g0?ooo`030000003oool0oooo05L0oooo0`000027
0?ooo`<00000F03oool00`000000oooo0?ooo`0g0?ooo`00>03oool00`000000oooo0?ooo`1I0?oo
o`<00000P@3oool3000005/0oooo00<000000?ooo`3oool0=`3oool003T0oooo00<000000?ooo`3o
ool0F`3oool2000007d0oooo0P00001M0?ooo`030000003oool0oooo03P0oooo000i0?ooo`030000
003oool0oooo05d0oooo0`00001g0?ooo`<00000GP3oool00`000000oooo0?ooo`0i0?ooo`00>P3o
ool00`000000oooo0?ooo`1O0?ooo`D00000K@3oool500000640oooo00<000000?ooo`3oool0>@3o
ool003X0oooo00<000000?ooo`3oool0I03oool4000006D0oooo1000001U0?ooo`030000003oool0
oooo03X0oooo000k0?ooo`030000003oool0oooo06L0oooo0P00001Q0?ooo`800000J@3oool00`00
0000oooo0?ooo`0j0?ooo`00?03oool00`000000oooo0?ooo`1X0?ooo`H00000E@3oool6000006X0
oooo00<000000?ooo`3oool0>`3oool003`0oooo00<000000?ooo`3oool0KP3oool4000004d0oooo
1000001_0?ooo`030000003oool0oooo03`0oooo000m0?ooo`030000003oool0oooo0740oooo1@00
00130?ooo`D00000L`3oool00`000000oooo0?ooo`0l0?ooo`00?P3oool00`000000oooo0?ooo`1e
0?ooo`T00000<@3oool9000007L0oooo00<000000?ooo`3oool0?@3oool003l0oooo00<000000?oo
o`3oool0O@3oool7000002<0oooo1`00001o0?ooo`030000003oool0oooo03h0oooo00100?ooo`03
0000003oool0oooo08<0oooo8`0000250?ooo`030000003oool0oooo03l0oooo00100?ooo`030000
003oool0oooo0?l0oooo;03oool00`000000oooo0?ooo`0o0?ooo`00@@3oool00`000000oooo0?oo
o`3o0?ooobX0oooo00<000000?ooo`3oool0@03oool00480oooo00<000000?ooo`3oool0o`3ooolX
0?ooo`030000003oool0oooo0440oooo00130?ooo`030000003oool0oooo0?l0oooo9P3oool00`00
0000oooo0?ooo`120?ooo`00A03oool00`000000oooo0?ooo`3o0?ooob@0oooo00<000000?ooo`3o
ool0@`3oool004D0oooo00<000000?ooo`3oool0o`3ooolR0?ooo`030000003oool0oooo04@0oooo
00160?ooo`030000003oool0oooo0?l0oooo803oool00`000000oooo0?ooo`150?ooo`00A`3oool0
0`000000oooo0?ooo`3o0?oooah0oooo00<000000?ooo`3oool0AP3oool004P0oooo00<000000?oo
o`3oool0o`3ooolL0?ooo`030000003oool0oooo04L0oooo00190?ooo`030000003oool0oooo0?l0
oooo6P3oool00`000000oooo0?ooo`180?ooo`00BP3oool200000?l0oooo603oool2000004/0oooo
001<0?ooo`030000003oool0oooo0?l0oooo503oool00`000000oooo0?ooo`1;0?ooo`00C@3oool0
0`000000oooo0?ooo`3o0?oooa80oooo00<000000?ooo`3oool0C03oool004h0oooo00<000000?oo
o`3oool0o`3oool@0?ooo`030000003oool0oooo04d0oooo001?0?ooo`800000o`3oool>0?ooo`80
0000D03oool00540oooo0P00003o0?ooo`X0oooo0P00001B0?ooo`00D`3oool00`000000oooo0?oo
o`3o0?ooo`H0oooo00<000000?ooo`3oool0DP3oool005@0oooo0P00003o0?ooo`@0oooo0P00001E
0?ooo`00EP3oool00`000000oooo0?ooo`3o0?ooo`030000003oool0oooo05D0oooo001G0?ooo`80
0000o@3oool2000005P0oooo001I0?ooo`800000n@3oool2000005X0oooo001K0?ooo`800000m@3o
ool2000005`0oooo001M0?ooo`030000003oool0oooo0?40oooo00<000000?ooo`3oool0G03oool0
05h0oooo0`00003]0?ooo`<00000G`3oool00640oooo0P00003Y0?ooo`800000HP3oool006<0oooo
0P00003U0?ooo`800000I03oool006D0oooo0P00003Q0?ooo`800000IP3oool006L0oooo0`00003K
0?ooo`<00000J03oool006X0oooo0`00003E0?ooo`<00000J`3oool006d0oooo0P00003A0?ooo`80
0000KP3oool006l0oooo100000390?ooo`@00000L03oool007<0oooo100000310?ooo`@00000M03o
ool007L0oooo1000002i0?ooo`@00000N03oool007/0oooo1000002a0?ooo`@00000O03oool007l0
oooo1000002Y0?ooo`@00000P03oool008<0oooo1@00002O0?ooo`D00000Q03oool008P0oooo2000
002?0?ooo`P00000R@3oool00900oooo100000270?ooo`@00000T@3oool009@0oooo2000001g0?oo
o`P00000U@3oool009`0oooo2P00001S0?ooo`X00000W@3oool00:H0oooo2`00001=0?ooo`/00000
Y`3oool00;40oooo6P00000I0?oooaX00000/P3oool00</0oooo6@00003<0?ooo`00o`3ooona0?oo
o`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00\
\>"],
  ImageRangeCache->{{{0, 431}, {431, 0}} -> {0.0594956, 0.0594956, 0.351094, \
0.351094}}]
}, Closed]],

Cell[TextData[{
  "Finally, we use ",
  StyleBox["PlotGradientField", "InlineInput",
    FontWeight->"Bold"],
  " to display the electric field."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(PlotGradientField[\(-f[x, y]\), {x, 1, 100}, {y, 5, 100}, 
        PlotPoints \[Rule] 15, ScaleFactor \[Rule] 5, 
        Epilog \[Rule] {RGBColor[1, 0, 0], Line[{{31, 41}, {70, 41}}], 
            Line[{{31, 60}, {70, 60}}]}];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .95927 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0178805 0.00954672 -0.0248939 0.00954672 [
[ 0 0 0 0 ]
[ 1 .95927 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
% Start of user PostScript
/mathtops {
gsave
MBeginOrig
moveto
MEndOrig
currentpoint
grestore
} bind def
/MAtocoords {
mathtops 4 2 roll mathtops
4 copy pop pop
3 -1 roll sub
/arry exch def
exch sub
/arrx exch def
arrx dup mul
arry dup mul
add sqrt
/arrl exch def
translate
} bind def
/MAarrowhead1 {
gsave
MAtocoords
arrl 0. eq
{ 0 0 Mdot }
{
[ arrx arrl div
arry arrl div
-1 arry mul arrl div
arrx arrl div
0 0 ]
concat
-0.02 0.005 moveto 0 0 lineto
-0.02 -0.005 lineto fill
-0.02 0.005 moveto 0 0 lineto
-0.02 -0.005 lineto
-0.02 0.005 lineto stroke
} ifelse
grestore
} def
/MAarrowzero1 {
gsave
mathtops translate
0 0 Mdot
grestore
} def
% End of user PostScript
0 g
.0001 w
[ ] 0 setdash
.02743 .02284 m
.0281 .02284 L
s
% Start of user PostScript
1. 5. 1.0701088 5. MAarrowhead1
% End of user PostScript
.02743 .08762 m
.02919 .08762 L
s
% Start of user PostScript
1. 11.785714 1.184255 11.785714 MAarrowhead1
% End of user PostScript
.02743 .1524 m
.03017 .1524 L
s
% Start of user PostScript
1. 18.571429 1.2871421 18.571429 MAarrowhead1
% End of user PostScript
.02743 .21718 m
.03089 .21718 L
s
% Start of user PostScript
1. 25.357143 1.3629783 25.357143 MAarrowhead1
% End of user PostScript
.02743 .28196 m
.03109 .28196 L
s
% Start of user PostScript
1. 32.142857 1.3832754 32.142857 MAarrowhead1
% End of user PostScript
.02743 .34675 m
.03042 .34675 L
s
% Start of user PostScript
1. 38.928571 1.3135282 38.928571 MAarrowhead1
% End of user PostScript
.02743 .41153 m
.02887 .41153 L
s
% Start of user PostScript
1. 45.714286 1.1508526 45.714286 MAarrowhead1
% End of user PostScript
.02743 .47631 m
.02681 .47631 L
s
% Start of user PostScript
1. 52.5 0.93496001 52.5 MAarrowhead1
% End of user PostScript
.02743 .54109 m
.02497 .54109 L
s
% Start of user PostScript
1. 59.285714 0.7425454 59.285714 MAarrowhead1
% End of user PostScript
.02743 .60587 m
.02393 .60587 L
s
% Start of user PostScript
1. 66.071429 0.63347304 66.071429 MAarrowhead1
% End of user PostScript
.02743 .67065 m
.02381 .67065 L
s
% Start of user PostScript
1. 72.857143 0.62105144 72.857143 MAarrowhead1
% End of user PostScript
.02743 .73543 m
.02435 .73543 L
s
% Start of user PostScript
1. 79.642857 0.67775583 79.642857 MAarrowhead1
% End of user PostScript
.02743 .80022 m
.02525 .80022 L
s
% Start of user PostScript
1. 86.428571 0.77147576 86.428571 MAarrowhead1
% End of user PostScript
.02743 .865 m
.02632 .865 L
s
% Start of user PostScript
1. 93.214286 0.88354199 93.214286 MAarrowhead1
% End of user PostScript
.02743 .92978 m
.02743 .92978 L
s
% Start of user PostScript
1. 100. MAarrowzero1
% End of user PostScript
.09494 .02284 m
.09561 .02402 L
s
% Start of user PostScript
8.0714286 5. 8.1416168 5.1231677 MAarrowhead1
% End of user PostScript
.09494 .08762 m
.09674 .08874 L
s
% Start of user PostScript
8.0714286 11.785714 8.2606219 11.903225 MAarrowhead1
% End of user PostScript
.09494 .1524 m
.09781 .15337 L
s
% Start of user PostScript
8.0714286 18.571429 8.3728035 18.67289 MAarrowhead1
% End of user PostScript
.09494 .21718 m
.09867 .21777 L
s
% Start of user PostScript
8.0714286 25.357143 8.4630109 25.418737 MAarrowhead1
% End of user PostScript
.09494 .28196 m
.09905 .28181 L
s
% Start of user PostScript
8.0714286 32.142857 8.5022606 32.127126 MAarrowhead1
% End of user PostScript
.09494 .34675 m
.09848 .34555 L
s
% Start of user PostScript
8.0714286 38.928571 8.4428044 38.803552 MAarrowhead1
% End of user PostScript
.09494 .41153 m
.09669 .40939 L
s
% Start of user PostScript
8.0714286 45.714286 8.2554987 45.490618 MAarrowhead1
% End of user PostScript
.09494 .47631 m
.09418 .474 L
s
% Start of user PostScript
8.0714286 52.5 7.9918267 52.257896 MAarrowhead1
% End of user PostScript
.09494 .54109 m
.09198 .53947 L
s
% Start of user PostScript
8.0714286 59.285714 7.7622082 59.116019 MAarrowhead1
% End of user PostScript
.09494 .60587 m
.09092 .60532 L
s
% Start of user PostScript
8.0714286 66.071429 7.650447 66.013824 MAarrowhead1
% End of user PostScript
.09494 .67065 m
.09096 .67099 L
s
% Start of user PostScript
8.0714286 72.857143 7.6552737 72.892246 MAarrowhead1
% End of user PostScript
.09494 .73543 m
.09167 .73628 L
s
% Start of user PostScript
8.0714286 79.642857 7.7291725 79.731113 MAarrowhead1
% End of user PostScript
.09494 .80022 m
.09268 .80129 L
s
% Start of user PostScript
8.0714286 86.428571 7.8348885 86.541535 MAarrowhead1
% End of user PostScript
.09494 .865 m
.0938 .86616 L
s
% Start of user PostScript
8.0714286 93.214286 7.952066 93.336428 MAarrowhead1
% End of user PostScript
.09494 .92978 m
.09494 .93097 L
s
% Start of user PostScript
8.0714286 100. 8.0714286 100.12459 MAarrowhead1
% End of user PostScript
.16245 .02284 m
.16315 .02523 L
s
% Start of user PostScript
15.142857 5. 15.216378 5.2508935 MAarrowhead1
% End of user PostScript
.16245 .08762 m
.16436 .08997 L
s
% Start of user PostScript
15.142857 11.785714 15.343512 12.031262 MAarrowhead1
% End of user PostScript
.16245 .1524 m
.16564 .15455 L
s
% Start of user PostScript
15.142857 18.571429 15.477681 18.796499 MAarrowhead1
% End of user PostScript
.16245 .21718 m
.16696 .21877 L
s
% Start of user PostScript
15.142857 25.357143 15.615861 25.523537 MAarrowhead1
% End of user PostScript
.16245 .28196 m
.16808 .28213 L
s
% Start of user PostScript
15.142857 32.142857 15.733585 32.160069 MAarrowhead1
% End of user PostScript
.16245 .34675 m
.16802 .34429 L
s
% Start of user PostScript
15.142857 38.928571 15.727237 38.671185 MAarrowhead1
% End of user PostScript
.16245 .41153 m
.16546 .40636 L
s
% Start of user PostScript
15.142857 45.714286 15.458138 45.172935 MAarrowhead1
% End of user PostScript
.16245 .47631 m
.16113 .47053 L
s
% Start of user PostScript
15.142857 52.5 15.005372 51.894839 MAarrowhead1
% End of user PostScript
.16245 .54109 m
.15759 .5374 L
s
% Start of user PostScript
15.142857 59.285714 14.634781 58.898872 MAarrowhead1
% End of user PostScript
.16245 .60587 m
.1566 .6051 L
s
% Start of user PostScript
15.142857 66.071429 14.530595 65.990657 MAarrowhead1
% End of user PostScript
.16245 .67065 m
.15741 .6718 L
s
% Start of user PostScript
15.142857 72.857143 14.615651 72.976802 MAarrowhead1
% End of user PostScript
.16245 .73543 m
.15871 .73741 L
s
% Start of user PostScript
15.142857 79.642857 14.751307 79.850358 MAarrowhead1
% End of user PostScript
.16245 .80022 m
.16001 .80251 L
s
% Start of user PostScript
15.142857 86.428571 14.887699 86.66848 MAarrowhead1
% End of user PostScript
.16245 .865 m
.16125 .86738 L
s
% Start of user PostScript
15.142857 93.214286 15.017415 93.463464 MAarrowhead1
% End of user PostScript
.16245 .92978 m
.16245 .93217 L
s
% Start of user PostScript
15.142857 100. 15.142857 100.25083 MAarrowhead1
% End of user PostScript
.22995 .02284 m
.23066 .02649 L
s
% Start of user PostScript
22.214286 5. 22.288453 5.3824361 MAarrowhead1
% End of user PostScript
.22995 .08762 m
.23191 .09132 L
s
% Start of user PostScript
22.214286 11.785714 22.41934 12.172798 MAarrowhead1
% End of user PostScript
.22995 .1524 m
.23339 .15612 L
s
% Start of user PostScript
22.214286 18.571429 22.573893 18.960547 MAarrowhead1
% End of user PostScript
.22995 .21718 m
.23534 .22063 L
s
% Start of user PostScript
22.214286 25.357143 22.778268 25.718652 MAarrowhead1
% End of user PostScript
.22995 .28196 m
.23815 .28405 L
s
% Start of user PostScript
22.214286 32.142857 23.072953 32.361028 MAarrowhead1
% End of user PostScript
.22995 .34675 m
.24089 .34358 L
s
% Start of user PostScript
22.214286 38.928571 23.359612 38.596835 MAarrowhead1
% End of user PostScript
.22995 .41153 m
.23641 .40046 L
s
% Start of user PostScript
22.214286 45.714286 22.890794 44.555458 MAarrowhead1
% End of user PostScript
.22995 .47631 m
.22718 .4638 L
s
% Start of user PostScript
22.214286 52.5 21.924037 51.189311 MAarrowhead1
% End of user PostScript
.22995 .54109 m
.21967 .5343 L
s
% Start of user PostScript
22.214286 59.285714 21.136775 58.574589 MAarrowhead1
% End of user PostScript
.22995 .60587 m
.22037 .60651 L
s
% Start of user PostScript
22.214286 66.071429 21.210094 66.138094 MAarrowhead1
% End of user PostScript
.22995 .67065 m
.22354 .67378 L
s
% Start of user PostScript
22.214286 72.857143 21.542159 73.184367 MAarrowhead1
% End of user PostScript
.22995 .73543 m
.2258 .7391 L
s
% Start of user PostScript
22.214286 79.642857 21.778825 80.02656 MAarrowhead1
% End of user PostScript
.22995 .80022 m
.22743 .80392 L
s
% Start of user PostScript
22.214286 86.428571 21.94979 86.816897 MAarrowhead1
% End of user PostScript
.22995 .865 m
.22875 .86866 L
s
% Start of user PostScript
22.214286 93.214286 22.087872 93.598449 MAarrowhead1
% End of user PostScript
.22995 .92978 m
.22995 .93341 L
s
% Start of user PostScript
22.214286 100. 22.214286 100.38043 MAarrowhead1
% End of user PostScript
.29746 .02284 m
.2981 .0277 L
s
% Start of user PostScript
29.285714 5. 29.352767 5.5088455 MAarrowhead1
% End of user PostScript
.29746 .08762 m
.29924 .09268 L
s
% Start of user PostScript
29.285714 11.785714 29.472083 12.315398 MAarrowhead1
% End of user PostScript
.29746 .1524 m
.30067 .15788 L
s
% Start of user PostScript
29.285714 18.571429 29.621876 19.145257 MAarrowhead1
% End of user PostScript
.29746 .21718 m
.30275 .22338 L
s
% Start of user PostScript
29.285714 25.357143 29.839713 26.006501 MAarrowhead1
% End of user PostScript
.29746 .28196 m
.30676 .28947 L
s
% Start of user PostScript
29.285714 32.142857 30.259526 32.928695 MAarrowhead1
% End of user PostScript
.29746 .34675 m
.32359 .35539 L
s
% Start of user PostScript
29.285714 38.928571 32.022498 39.834275 MAarrowhead1
% End of user PostScript
.29746 .41153 m
.30899 .38485 L
s
% Start of user PostScript
29.285714 45.714286 30.493086 42.919845 MAarrowhead1
% End of user PostScript
.29746 .47631 m
.29325 .4503 L
s
% Start of user PostScript
29.285714 52.5 28.844698 49.775956 MAarrowhead1
% End of user PostScript
.29746 .54109 m
.26122 .52099 L
s
% Start of user PostScript
29.285714 59.285714 25.488819 57.180068 MAarrowhead1
% End of user PostScript
.29746 .60587 m
.28476 .61429 L
s
% Start of user PostScript
29.285714 66.071429 27.955378 66.953592 MAarrowhead1
% End of user PostScript
.29746 .67065 m
.29092 .67727 L
s
% Start of user PostScript
29.285714 72.857143 28.600006 73.54995 MAarrowhead1
% End of user PostScript
.29746 .73543 m
.29352 .74116 L
s
% Start of user PostScript
29.285714 79.642857 28.872587 80.24313 MAarrowhead1
% End of user PostScript
.29746 .80022 m
.29515 .80543 L
s
% Start of user PostScript
29.285714 86.428571 29.043512 86.974627 MAarrowhead1
% End of user PostScript
.29746 .865 m
.29637 .86992 L
s
% Start of user PostScript
29.285714 93.214286 29.171607 93.729515 MAarrowhead1
% End of user PostScript
.29746 .92978 m
.29746 .93459 L
s
% Start of user PostScript
29.285714 100. 29.285714 100.50451 MAarrowhead1
% End of user PostScript
.36497 .02284 m
.36544 .02871 L
s
% Start of user PostScript
36.357143 5. 36.405821 5.6144934 MAarrowhead1
% End of user PostScript
.36497 .08762 m
.3663 .09383 L
s
% Start of user PostScript
36.357143 11.785714 36.496008 12.435874 MAarrowhead1
% End of user PostScript
.36497 .1524 m
.36732 .15938 L
s
% Start of user PostScript
36.357143 18.571429 36.603421 19.302181 MAarrowhead1
% End of user PostScript
.36497 .21718 m
.36863 .2256 L
s
% Start of user PostScript
36.357143 25.357143 36.740314 26.238289 MAarrowhead1
% End of user PostScript
.36497 .28196 m
.37 .29331 L
s
% Start of user PostScript
36.357143 32.142857 36.883672 33.331038 MAarrowhead1
% End of user PostScript
.36497 .34675 m
.36786 .36401 L
s
% Start of user PostScript
36.357143 38.928571 36.659275 40.736696 MAarrowhead1
% End of user PostScript
.36497 .41153 m
.36729 .374 L
s
% Start of user PostScript
36.357143 45.714286 36.599925 41.783428 MAarrowhead1
% End of user PostScript
.36497 .47631 m
.36376 .44032 L
s
% Start of user PostScript
36.357143 52.5 36.229987 48.730562 MAarrowhead1
% End of user PostScript
.36497 .54109 m
.36437 .49336 L
s
% Start of user PostScript
36.357143 59.285714 36.294053 54.286116 MAarrowhead1
% End of user PostScript
.36497 .60587 m
.35985 .61933 L
s
% Start of user PostScript
36.357143 66.071429 35.820765 67.481184 MAarrowhead1
% End of user PostScript
.36497 .67065 m
.3607 .67999 L
s
% Start of user PostScript
36.357143 72.857143 35.910076 73.835293 MAarrowhead1
% End of user PostScript
.36497 .73543 m
.36211 .74291 L
s
% Start of user PostScript
36.357143 79.642857 36.057337 80.425516 MAarrowhead1
% End of user PostScript
.36497 .80022 m
.36324 .80669 L
s
% Start of user PostScript
36.357143 86.428571 36.175901 87.107121 MAarrowhead1
% End of user PostScript
.36497 .865 m
.36416 .87096 L
s
% Start of user PostScript
36.357143 93.214286 36.272063 93.839376 MAarrowhead1
% End of user PostScript
.36497 .92978 m
.36497 .93557 L
s
% Start of user PostScript
36.357143 100. 36.357143 100.60706 MAarrowhead1
% End of user PostScript
.43248 .02284 m
.43275 .02936 L
s
% Start of user PostScript
43.428571 5. 43.457193 5.6833711 MAarrowhead1
% End of user PostScript
.43248 .08762 m
.43319 .09456 L
s
% Start of user PostScript
43.428571 11.785714 43.502723 12.512787 MAarrowhead1
% End of user PostScript
.43248 .1524 m
.43369 .16025 L
s
% Start of user PostScript
43.428571 18.571429 43.554879 19.392962 MAarrowhead1
% End of user PostScript
.43248 .21718 m
.43416 .22648 L
s
% Start of user PostScript
43.428571 25.357143 43.604708 26.331124 MAarrowhead1
% End of user PostScript
.43248 .28196 m
.43426 .29326 L
s
% Start of user PostScript
43.428571 32.142857 43.614713 33.325618 MAarrowhead1
% End of user PostScript
.43248 .34675 m
.43309 .35976 L
s
% Start of user PostScript
43.428571 38.928571 43.492598 40.291535 MAarrowhead1
% End of user PostScript
.43248 .41153 m
.4327 .37376 L
s
% Start of user PostScript
43.428571 45.714286 43.451492 41.758023 MAarrowhead1
% End of user PostScript
.43248 .47631 m
.43235 .43872 L
s
% Start of user PostScript
43.428571 52.5 43.415188 48.562993 MAarrowhead1
% End of user PostScript
.43248 .54109 m
.43244 .49665 L
s
% Start of user PostScript
43.428571 59.285714 43.424512 54.630726 MAarrowhead1
% End of user PostScript
.43248 .60587 m
.431 .61802 L
s
% Start of user PostScript
43.428571 66.071429 43.273494 67.343964 MAarrowhead1
% End of user PostScript
.43248 .67065 m
.43068 .68071 L
s
% Start of user PostScript
43.428571 72.857143 43.24022 73.910299 MAarrowhead1
% End of user PostScript
.43248 .73543 m
.43108 .74381 L
s
% Start of user PostScript
43.428571 79.642857 43.281725 80.520501 MAarrowhead1
% End of user PostScript
.43248 .80022 m
.43158 .80749 L
s
% Start of user PostScript
43.428571 86.428571 43.333931 87.190878 MAarrowhead1
% End of user PostScript
.43248 .865 m
.43204 .87164 L
s
% Start of user PostScript
43.428571 93.214286 43.382732 93.91013 MAarrowhead1
% End of user PostScript
.43248 .92978 m
.43248 .93621 L
s
% Start of user PostScript
43.428571 100. 43.428571 100.67387 MAarrowhead1
% End of user PostScript
.49999 .02284 m
.49997 .02959 L
s
% Start of user PostScript
50.5 5. 50.498323 5.7068278 MAarrowhead1
% End of user PostScript
.49999 .08762 m
.49998 .09481 L
s
% Start of user PostScript
50.5 11.785714 50.499273 12.538462 MAarrowhead1
% End of user PostScript
.49999 .1524 m
.49997 .16051 L
s
% Start of user PostScript
50.5 18.571429 50.498243 19.420478 MAarrowhead1
% End of user PostScript
.49999 .21718 m
.5 .22666 L
s
% Start of user PostScript
50.5 25.357143 50.500774 26.350246 MAarrowhead1
% End of user PostScript
.49999 .28196 m
.49999 .29301 L
s
% Start of user PostScript
50.5 32.142857 50.500172 33.29953 MAarrowhead1
% End of user PostScript
.49999 .34675 m
.49999 .35887 L
s
% Start of user PostScript
50.5 38.928571 50.500002 40.198897 MAarrowhead1
% End of user PostScript
.49999 .41153 m
.49999 .37376 L
s
% Start of user PostScript
50.5 45.714286 50.49978 41.75771 MAarrowhead1
% End of user PostScript
.49999 .47631 m
.49999 .43858 L
s
% Start of user PostScript
50.5 52.5 50.500187 48.5475 MAarrowhead1
% End of user PostScript
.49999 .54109 m
.49999 .49698 L
s
% Start of user PostScript
50.5 59.285714 50.500199 54.665417 MAarrowhead1
% End of user PostScript
.49999 .60587 m
.49999 .6175 L
s
% Start of user PostScript
50.5 66.071429 50.499734 67.289786 MAarrowhead1
% End of user PostScript
.49999 .67065 m
.49998 .68076 L
s
% Start of user PostScript
50.5 72.857143 50.499035 73.916008 MAarrowhead1
% End of user PostScript
.49999 .73543 m
.49999 .74406 L
s
% Start of user PostScript
50.5 79.642857 50.49966 80.546923 MAarrowhead1
% End of user PostScript
.49999 .80022 m
.5 .80774 L
s
% Start of user PostScript
50.5 86.428571 50.500707 87.21711 MAarrowhead1
% End of user PostScript
.49999 .865 m
.49999 .87187 L
s
% Start of user PostScript
50.5 93.214286 50.500229 93.933968 MAarrowhead1
% End of user PostScript
.49999 .92978 m
.49999 .93643 L
s
% Start of user PostScript
50.5 100. 50.5 100.69654 MAarrowhead1
% End of user PostScript
.5675 .02284 m
.56726 .02936 L
s
% Start of user PostScript
57.571429 5. 57.546188 5.6831982 MAarrowhead1
% End of user PostScript
.5675 .08762 m
.5668 .09456 L
s
% Start of user PostScript
57.571429 11.785714 57.498648 12.51286 MAarrowhead1
% End of user PostScript
.5675 .1524 m
.56632 .16024 L
s
% Start of user PostScript
57.571429 18.571429 57.447597 19.392845 MAarrowhead1
% End of user PostScript
.5675 .21718 m
.56581 .22649 L
s
% Start of user PostScript
57.571429 25.357143 57.394621 26.331588 MAarrowhead1
% End of user PostScript
.5675 .28196 m
.56573 .29326 L
s
% Start of user PostScript
57.571429 32.142857 57.386099 33.325498 MAarrowhead1
% End of user PostScript
.5675 .34675 m
.56689 .35976 L
s
% Start of user PostScript
57.571429 38.928571 57.507245 40.291557 MAarrowhead1
% End of user PostScript
.5675 .41153 m
.56728 .37375 L
s
% Start of user PostScript
57.571429 45.714286 57.548242 41.757637 MAarrowhead1
% End of user PostScript
.5675 .47631 m
.56763 .43872 L
s
% Start of user PostScript
57.571429 52.5 57.585125 48.562555 MAarrowhead1
% End of user PostScript
.5675 .54109 m
.56753 .49665 L
s
% Start of user PostScript
57.571429 59.285714 57.575022 54.630923 MAarrowhead1
% End of user PostScript
.5675 .60587 m
.56899 .61802 L
s
% Start of user PostScript
57.571429 66.071429 57.72803 67.343676 MAarrowhead1
% End of user PostScript
.5675 .67065 m
.56932 .68071 L
s
% Start of user PostScript
57.571429 72.857143 57.761845 73.910117 MAarrowhead1
% End of user PostScript
.5675 .73543 m
.56891 .74381 L
s
% Start of user PostScript
57.571429 79.642857 57.719252 80.520663 MAarrowhead1
% End of user PostScript
.5675 .80022 m
.56839 .80748 L
s
% Start of user PostScript
57.571429 86.428571 57.664836 87.189798 MAarrowhead1
% End of user PostScript
.5675 .865 m
.56793 .87163 L
s
% Start of user PostScript
57.571429 93.214286 57.616943 93.90957 MAarrowhead1
% End of user PostScript
.5675 .92978 m
.5675 .93621 L
s
% Start of user PostScript
57.571429 100. 57.571429 100.67345 MAarrowhead1
% End of user PostScript
.63501 .02284 m
.63452 .0287 L
s
% Start of user PostScript
64.642857 5. 64.592284 5.6142605 MAarrowhead1
% End of user PostScript
.63501 .08762 m
.63367 .09383 L
s
% Start of user PostScript
64.642857 11.785714 64.50303 12.435739 MAarrowhead1
% End of user PostScript
.63501 .1524 m
.63264 .15938 L
s
% Start of user PostScript
64.642857 18.571429 64.394925 19.302029 MAarrowhead1
% End of user PostScript
.63501 .21718 m
.63135 .2256 L
s
% Start of user PostScript
64.642857 25.357143 64.259845 26.238827 MAarrowhead1
% End of user PostScript
.63501 .28196 m
.62999 .29331 L
s
% Start of user PostScript
64.642857 32.142857 64.11689 33.330956 MAarrowhead1
% End of user PostScript
.63501 .34675 m
.63212 .36401 L
s
% Start of user PostScript
64.642857 38.928571 64.340502 40.736892 MAarrowhead1
% End of user PostScript
.63501 .41153 m
.63269 .374 L
s
% Start of user PostScript
64.642857 45.714286 64.399852 41.783527 MAarrowhead1
% End of user PostScript
.63501 .47631 m
.63622 .44032 L
s
% Start of user PostScript
64.642857 52.5 64.769991 48.730626 MAarrowhead1
% End of user PostScript
.63501 .54109 m
.63561 .49336 L
s
% Start of user PostScript
64.642857 59.285714 64.705687 54.286109 MAarrowhead1
% End of user PostScript
.63501 .60587 m
.64013 .61933 L
s
% Start of user PostScript
64.642857 66.071429 65.179327 67.481258 MAarrowhead1
% End of user PostScript
.63501 .67065 m
.63926 .67999 L
s
% Start of user PostScript
64.642857 72.857143 65.088412 73.8349 MAarrowhead1
% End of user PostScript
.63501 .73543 m
.63786 .7429 L
s
% Start of user PostScript
64.642857 79.642857 64.942145 80.424913 MAarrowhead1
% End of user PostScript
.63501 .80022 m
.63674 .8067 L
s
% Start of user PostScript
64.642857 86.428571 64.824018 87.10785 MAarrowhead1
% End of user PostScript
.63501 .865 m
.63582 .87096 L
s
% Start of user PostScript
64.642857 93.214286 64.727834 93.83852 MAarrowhead1
% End of user PostScript
.63501 .92978 m
.63501 .93557 L
s
% Start of user PostScript
64.642857 100. 64.642857 100.60631 MAarrowhead1
% End of user PostScript
.70252 .02284 m
.70189 .0277 L
s
% Start of user PostScript
71.714286 5. 71.649073 5.5095436 MAarrowhead1
% End of user PostScript
.70252 .08762 m
.70074 .09268 L
s
% Start of user PostScript
71.714286 11.785714 71.528093 12.315742 MAarrowhead1
% End of user PostScript
.70252 .1524 m
.69931 .15788 L
s
% Start of user PostScript
71.714286 18.571429 71.378641 19.145722 MAarrowhead1
% End of user PostScript
.70252 .21718 m
.69723 .22337 L
s
% Start of user PostScript
71.714286 25.357143 71.160722 26.005437 MAarrowhead1
% End of user PostScript
.70252 .28196 m
.69322 .28946 L
s
% Start of user PostScript
71.714286 32.142857 70.740884 32.928449 MAarrowhead1
% End of user PostScript
.70252 .34675 m
.67638 .35538 L
s
% Start of user PostScript
71.714286 38.928571 68.976806 39.83335 MAarrowhead1
% End of user PostScript
.70252 .41153 m
.69098 .38485 L
s
% Start of user PostScript
71.714286 45.714286 70.506147 42.919613 MAarrowhead1
% End of user PostScript
.70252 .47631 m
.70673 .45031 L
s
% Start of user PostScript
71.714286 52.5 72.155325 49.776233 MAarrowhead1
% End of user PostScript
.70252 .54109 m
.73877 .52098 L
s
% Start of user PostScript
71.714286 59.285714 75.511909 57.178862 MAarrowhead1
% End of user PostScript
.70252 .60587 m
.71522 .61429 L
s
% Start of user PostScript
71.714286 66.071429 73.044544 66.953771 MAarrowhead1
% End of user PostScript
.70252 .67065 m
.70907 .67727 L
s
% Start of user PostScript
71.714286 72.857143 72.400502 73.550036 MAarrowhead1
% End of user PostScript
.70252 .73543 m
.70647 .74118 L
s
% Start of user PostScript
71.714286 79.642857 72.128126 80.244459 MAarrowhead1
% End of user PostScript
.70252 .80022 m
.70483 .80543 L
s
% Start of user PostScript
71.714286 86.428571 71.95709 86.974421 MAarrowhead1
% End of user PostScript
.70252 .865 m
.7036 .86992 L
s
% Start of user PostScript
71.714286 93.214286 71.827763 93.730445 MAarrowhead1
% End of user PostScript
.70252 .92978 m
.70252 .93461 L
s
% Start of user PostScript
71.714286 100. 71.714286 100.50583 MAarrowhead1
% End of user PostScript
.77003 .02284 m
.76932 .02649 L
s
% Start of user PostScript
78.785714 5. 78.712082 5.3825368 MAarrowhead1
% End of user PostScript
.77003 .08762 m
.76807 .09132 L
s
% Start of user PostScript
78.785714 11.785714 78.580863 12.173234 MAarrowhead1
% End of user PostScript
.77003 .1524 m
.76659 .15611 L
s
% Start of user PostScript
78.785714 18.571429 78.425832 18.959461 MAarrowhead1
% End of user PostScript
.77003 .21718 m
.76464 .22064 L
s
% Start of user PostScript
78.785714 25.357143 78.22166 25.719131 MAarrowhead1
% End of user PostScript
.77003 .28196 m
.76182 .28405 L
s
% Start of user PostScript
78.785714 32.142857 77.925843 32.361201 MAarrowhead1
% End of user PostScript
.77003 .34675 m
.7591 .34359 L
s
% Start of user PostScript
78.785714 38.928571 77.641014 38.597613 MAarrowhead1
% End of user PostScript
.77003 .41153 m
.76357 .40047 L
s
% Start of user PostScript
78.785714 45.714286 78.109842 44.555533 MAarrowhead1
% End of user PostScript
.77003 .47631 m
.77279 .4638 L
s
% Start of user PostScript
78.785714 52.5 79.075722 51.189246 MAarrowhead1
% End of user PostScript
.77003 .54109 m
.78031 .53429 L
s
% Start of user PostScript
78.785714 59.285714 79.86249 58.573915 MAarrowhead1
% End of user PostScript
.77003 .60587 m
.77961 .60651 L
s
% Start of user PostScript
78.785714 66.071429 79.789913 66.138344 MAarrowhead1
% End of user PostScript
.77003 .67065 m
.77644 .67378 L
s
% Start of user PostScript
78.785714 72.857143 79.45793 73.184262 MAarrowhead1
% End of user PostScript
.77003 .73543 m
.77418 .73909 L
s
% Start of user PostScript
78.785714 79.642857 79.221242 80.025876 MAarrowhead1
% End of user PostScript
.77003 .80022 m
.77255 .80393 L
s
% Start of user PostScript
78.785714 86.428571 79.049801 86.817357 MAarrowhead1
% End of user PostScript
.77003 .865 m
.77123 .86867 L
s
% Start of user PostScript
78.785714 93.214286 78.912184 93.598913 MAarrowhead1
% End of user PostScript
.77003 .92978 m
.77003 .93341 L
s
% Start of user PostScript
78.785714 100. 78.785714 100.38067 MAarrowhead1
% End of user PostScript
.83753 .02284 m
.83683 .02523 L
s
% Start of user PostScript
85.857143 5. 85.783047 5.2507494 MAarrowhead1
% End of user PostScript
.83753 .08762 m
.83561 .08997 L
s
% Start of user PostScript
85.857143 11.785714 85.655891 12.031598 MAarrowhead1
% End of user PostScript
.83753 .1524 m
.83434 .15455 L
s
% Start of user PostScript
85.857143 18.571429 85.522623 18.796553 MAarrowhead1
% End of user PostScript
.83753 .21718 m
.83302 .21876 L
s
% Start of user PostScript
85.857143 25.357143 85.384396 25.522693 MAarrowhead1
% End of user PostScript
.83753 .28196 m
.8319 .28213 L
s
% Start of user PostScript
85.857143 32.142857 85.267225 32.159899 MAarrowhead1
% End of user PostScript
.83753 .34675 m
.83196 .34429 L
s
% Start of user PostScript
85.857143 38.928571 85.273 38.671595 MAarrowhead1
% End of user PostScript
.83753 .41153 m
.83453 .40636 L
s
% Start of user PostScript
85.857143 45.714286 85.542603 45.172816 MAarrowhead1
% End of user PostScript
.83753 .47631 m
.83885 .47055 L
s
% Start of user PostScript
85.857143 52.5 85.994804 51.896484 MAarrowhead1
% End of user PostScript
.83753 .54109 m
.84238 .5374 L
s
% Start of user PostScript
85.857143 59.285714 86.364953 58.898932 MAarrowhead1
% End of user PostScript
.83753 .60587 m
.84338 .6051 L
s
% Start of user PostScript
85.857143 66.071429 86.469033 65.990848 MAarrowhead1
% End of user PostScript
.83753 .67065 m
.84256 .67179 L
s
% Start of user PostScript
85.857143 72.857143 86.38369 72.976439 MAarrowhead1
% End of user PostScript
.83753 .73543 m
.84127 .73742 L
s
% Start of user PostScript
85.857143 79.642857 86.248715 79.851253 MAarrowhead1
% End of user PostScript
.83753 .80022 m
.83997 .80251 L
s
% Start of user PostScript
85.857143 86.428571 86.112131 86.668518 MAarrowhead1
% End of user PostScript
.83753 .865 m
.83873 .86738 L
s
% Start of user PostScript
85.857143 93.214286 85.982428 93.464346 MAarrowhead1
% End of user PostScript
.83753 .92978 m
.83753 .93218 L
s
% Start of user PostScript
85.857143 100. 85.857143 100.25122 MAarrowhead1
% End of user PostScript
.90504 .02284 m
.90437 .02401 L
s
% Start of user PostScript
92.928571 5. 92.858092 5.1227005 MAarrowhead1
% End of user PostScript
.90504 .08762 m
.90324 .08875 L
s
% Start of user PostScript
92.928571 11.785714 92.739171 11.903458 MAarrowhead1
% End of user PostScript
.90504 .1524 m
.90217 .15337 L
s
% Start of user PostScript
92.928571 18.571429 92.627464 18.672533 MAarrowhead1
% End of user PostScript
.90504 .21718 m
.9013 .21778 L
s
% Start of user PostScript
92.928571 25.357143 92.536713 25.419149 MAarrowhead1
% End of user PostScript
.90504 .28196 m
.90092 .28182 L
s
% Start of user PostScript
92.928571 32.142857 92.49709 32.12793 MAarrowhead1
% End of user PostScript
.90504 .34675 m
.90149 .34555 L
s
% Start of user PostScript
92.928571 38.928571 92.55688 38.803323 MAarrowhead1
% End of user PostScript
.90504 .41153 m
.90328 .4094 L
s
% Start of user PostScript
92.928571 45.714286 92.743648 45.491044 MAarrowhead1
% End of user PostScript
.90504 .47631 m
.9058 .47397 L
s
% Start of user PostScript
92.928571 52.5 93.008095 52.2555 MAarrowhead1
% End of user PostScript
.90504 .54109 m
.908 .53947 L
s
% Start of user PostScript
92.928571 59.285714 93.238148 59.115853 MAarrowhead1
% End of user PostScript
.90504 .60587 m
.90907 .60532 L
s
% Start of user PostScript
92.928571 66.071429 93.350537 66.013416 MAarrowhead1
% End of user PostScript
.90504 .67065 m
.90903 .671 L
s
% Start of user PostScript
92.928571 72.857143 93.346021 72.893569 MAarrowhead1
% End of user PostScript
.90504 .73543 m
.90831 .73628 L
s
% Start of user PostScript
92.928571 79.642857 93.270238 79.731017 MAarrowhead1
% End of user PostScript
.90504 .80022 m
.90731 .80129 L
s
% Start of user PostScript
92.928571 86.428571 93.16578 86.541655 MAarrowhead1
% End of user PostScript
.90504 .865 m
.90618 .86615 L
s
% Start of user PostScript
92.928571 93.214286 93.048044 93.335547 MAarrowhead1
% End of user PostScript
.90504 .92978 m
.90504 .93096 L
s
% Start of user PostScript
92.928571 100. 92.928571 100.12402 MAarrowhead1
% End of user PostScript
.97255 .02284 m
.9719 .02284 L
s
% Start of user PostScript
100. 5. 99.931747 5. MAarrowhead1
% End of user PostScript
.97255 .08762 m
.9708 .08762 L
s
% Start of user PostScript
100. 11.785714 99.816142 11.785714 MAarrowhead1
% End of user PostScript
.97255 .1524 m
.96981 .1524 L
s
% Start of user PostScript
100. 18.571429 99.71313 18.571429 MAarrowhead1
% End of user PostScript
.97255 .21718 m
.96909 .21718 L
s
% Start of user PostScript
100. 25.357143 99.637685 25.357143 MAarrowhead1
% End of user PostScript
.97255 .28196 m
.9689 .28196 L
s
% Start of user PostScript
100. 32.142857 99.617591 32.142857 MAarrowhead1
% End of user PostScript
.97255 .34675 m
.96955 .34675 L
s
% Start of user PostScript
100. 38.928571 99.685311 38.928571 MAarrowhead1
% End of user PostScript
.97255 .41153 m
.97111 .41153 L
s
% Start of user PostScript
100. 45.714286 99.849199 45.714286 MAarrowhead1
% End of user PostScript
.97255 .47631 m
.97317 .47631 L
s
% Start of user PostScript
100. 52.5 100.06501 52.5 MAarrowhead1
% End of user PostScript
.97255 .54109 m
.97501 .54109 L
s
% Start of user PostScript
100. 59.285714 100.25748 59.285714 MAarrowhead1
% End of user PostScript
.97255 .60587 m
.97604 .60587 L
s
% Start of user PostScript
100. 66.071429 100.36534 66.071429 MAarrowhead1
% End of user PostScript
.97255 .67065 m
.97619 .67065 L
s
% Start of user PostScript
100. 72.857143 100.38109 72.857143 MAarrowhead1
% End of user PostScript
.97255 .73543 m
.97563 .73543 L
s
% Start of user PostScript
100. 79.642857 100.3228 79.642857 MAarrowhead1
% End of user PostScript
.97255 .80022 m
.97474 .80022 L
s
% Start of user PostScript
100. 86.428571 100.22871 86.428571 MAarrowhead1
% End of user PostScript
.97255 .865 m
.97367 .865 L
s
% Start of user PostScript
100. 93.214286 100.11684 93.214286 MAarrowhead1
% End of user PostScript
.97255 .92978 m
.97255 .92978 L
s
% Start of user PostScript
100. 100. MAarrowzero1
% End of user PostScript
1 0 0 r
.5 Mabswid
.31383 .36652 m
.68615 .36652 L
s
.31383 .54791 m
.68615 .54791 L
s
0 0 m
1 0 L
1 .95927 L
0 .95927 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 276.25},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`0001WR000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`009`3oool00`000000oooo0?ooo`0J0?ooo`800000o`3ooolT0?ooo`80
0000703oool00`000000oooo0?ooo`0V0?ooo`009@3oool3000001T0oooo1@00000L0?ooo`800000
703oool200000:/0oooo0P00000L0?ooo`800000703oool5000001T0oooo0`00000V0?ooo`008`3o
ool5000001X0oooo1000000I0?ooo`D000006@3oool5000001P0oooo1P00000G0?ooo`H00000603o
ool5000001P0oooo1@00000H0?ooo`H00000603oool5000001T0oooo1@00000I0?ooo`@000006@3o
ool6000002@0oooo000T0?ooo`D000006@3oool4000001T0oooo1@00000I0?ooo`D00000603oool6
000001L0oooo1P00000H0?ooo`D00000603oool5000001P0oooo1P00000H0?ooo`D000006@3oool4
000001X0oooo1000000I0?ooo`D000009@3oool002D0oooo1000000J0?ooo`<000006P3oool40000
01X0oooo1000000I0?ooo`@000006@3oool5000001P0oooo1@00000H0?ooo`D00000603oool50000
01T0oooo1000000J0?ooo`@000006P3oool3000001X0oooo1000000V0?ooo`009P3oool3000001X0
oooo1000000I0?ooo`@000006P3oool4000001T0oooo1000000I0?ooo`@000006P3oool3000001X0
oooo0`00000J0?ooo`@000006@3oool4000001X0oooo0`00000J0?ooo`<000006`3oool3000002L0
oooo00030?ooo`<000008@3oool2000001/0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo
1000000I0?ooo`@000006P3oool3000001X0oooo0`00000J0?ooo`<000006`3oool2000001/0oooo
0`00000J0?ooo`<000006P3oool300000240oooo0`0000040?ooo`000`3oool7000001h0oooo0P00
000K0?ooo`8000006P3oool3000001/0oooo0P00000K0?ooo`<000006P3oool3000001X0oooo0`00
000J0?ooo`<000006P3oool3000001/0oooo0P00000K0?ooo`8000006`3oool2000001/0oooo0P00
000N0?ooo`L00000103oool000<0oooo2P00000K0?ooo`8000006`3oool2000001/0oooo0P00000K
0?ooo`8000006`3oool2000001/0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo0`00000K
0?ooo`8000006`3oool2000001/0oooo0P00000K0?ooo`8000006`3oool:000000@0oooo00030?oo
o`L00000?03oool00`000000oooo0?ooo`0I0?ooo`800000703oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo01T0oooo0P00000L0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0j0?ooo`L00000103oool000<0oooo0`00001M0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`1K0?ooo`<0
0000103oool00;X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo0;T0oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40
oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40
oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40
oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo00130?ooo`030000003oool0oooo
01/0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`2[0?ooo`030000003oool0oooo
01`0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`120?ooo`008`3oool00`000000
oooo0?ooo`0L0?ooo`8000006`3oool3000001`0oooo0`00000K0?ooo`<000006`3oool2000001T0
oooo1@00000I0?ooo`8000006`3oool2000001`0oooo0`00000L0?ooo`<000006`3oool2000001h0
oooo00<000000?ooo`3oool08P3oool00280oooo0`00000K0?ooo`D000006@3oool4000001X0oooo
1@00000H0?ooo`H00000603oool5000001T0oooo1@00000I0?ooo`D00000603oool5000001T0oooo
1@00000I0?ooo`D000006@3oool5000001/0oooo0P00000T0?ooo`008P3oool4000001T0oooo1P00
000J0?ooo`@000006P3oool4000001P0oooo1P00000H0?ooo`D000006@3oool5000001T0oooo1@00
000H0?ooo`D000006@3oool4000001X0oooo1000000J0?ooo`H000006@3oool4000002<0oooo000Q
0?ooo`H000006@3oool5000001/0oooo0`00000J0?ooo`@000006@3oool5000001T0oooo1000000J
0?ooo`<000006P3oool4000001T0oooo1000000J0?ooo`@000006P3oool3000001X0oooo1P00000I
0?ooo`D000008`3oool000<0oooo0`00000K0?ooo`L000006P3oool4000001X0oooo0`00000K0?oo
o`<000006@3oool5000001T0oooo1000000J0?ooo`<000006P3oool4000001T0oooo1000000J0?oo
o`<000006`3oool2000001/0oooo1000000J0?ooo`L000006`3oool2000000D0oooo00030?ooo`L0
00006@3oool6000001X0oooo0`00000K0?ooo`8000006`3oool3000001X0oooo0`00000J0?ooo`<0
00006`3oool3000001/0oooo0P00000K0?ooo`8000006`3oool3000001X0oooo0`00000K0?ooo`<0
00006P3oool6000001T0oooo1P0000050?ooo`000`3oool:000001X0oooo0`00000J0?ooo`<00000
6`3oool2000001/0oooo0P00000J0?ooo`<000006`3oool2000001/0oooo0`00000K0?ooo`800000
6`3oool2000001/0oooo0P00000K0?ooo`8000006`3oool3000001X0oooo0`00000J0?ooo`T00000
1@3oool000<0oooo1`00000l0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06@3o
ool2000001/0oooo0P00000K0?ooo`800000703oool00`000000oooo0?ooo`0J0?ooo`8000006`3o
ool2000001/0oooo0P00000K0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0>@3o
ool7000000D0oooo00030?ooo`<00000NP3oool00`000000oooo0?ooo`0I0?ooo`800000703oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo07L0oooo0`0000050?ooo`00W@3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo09`0oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo00110?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0703oool0
0`000000oooo0?ooo`0J0?ooo`800000703oool2000003L0oooo0P00000L0?ooo`800000703oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0@03oool0
0280oooo00<000000?ooo`3oool06`3oool3000001/0oooo1000000K0?ooo`<000006@3oool50000
01T0oooo1@00000I0?ooo`D000006@3oool5000001T0oooo1@00000I0?ooo`<000006`3oool40000
01/0oooo0P00000N0?ooo`030000003oool0oooo0240oooo000R0?ooo`<000006`3oool4000001T0
oooo1@00000I0?ooo`D000006P3oool4000001T0oooo1@00000I0?ooo`D000006@3oool5000001T0
oooo1000000I0?ooo`H000006@3oool5000001T0oooo1000000K0?ooo`<000008`3oool00240oooo
1@00000I0?ooo`H000006@3oool5000001T0oooo1@00000I0?ooo`@000006P3oool4000001T0oooo
1000000J0?ooo`@000006P3oool4000001T0oooo1@00000I0?ooo`D000006@3oool5000001X0oooo
1000000S0?ooo`00103oool3000001X0oooo1`00000G0?ooo`H000006`3oool3000001X0oooo1000
000J0?ooo`<000006P3oool4000001X0oooo0`00000J0?ooo`@000006P3oool3000001X0oooo1000
000J0?ooo`<000006`3oool6000001L0oooo1`00000J0?ooo`<000001@3oool000@0oooo1`00000F
0?ooo`T000005`3oool5000001/0oooo0`00000J0?ooo`<000006P3oool4000001T0oooo0`00000K
0?ooo`<000006`3oool2000001X0oooo0`00000K0?ooo`<000006P3oool3000001/0oooo1@00000G
0?ooo`T000005P3oool7000000D0oooo00040?ooo`X00000603oool5000001P0oooo1000000K0?oo
o`8000006`3oool2000001/0oooo0`00000J0?ooo`8000006`3oool3000001/0oooo0P00000J0?oo
o`<000006P3oool3000001/0oooo0P00000K0?ooo`@00000603oool5000001P0oooo2P0000050?oo
o`00103oool7000003/0oooo0P00000L0?ooo`030000003oool0oooo01T0oooo0P00000K0?ooo`80
00006P3oool2000001/0oooo0P00000L0?ooo`8000006P3oool2000001/0oooo0P00000K0?ooo`03
0000003oool0oooo01X0oooo0P00000k0?ooo`L000001@3oool000@0oooo0`00001j0?ooo`030000
003oool0oooo01T0oooo0P00000K0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0I0?ooo`8000006`3oool00`000000oooo0?ooo`1h0?ooo`<00000
1@3oool009h0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`2M0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00OP3oool00`000000oooo0?ooo`2_0?ooo`030000
003oool0oooo07d0oooo001O0?ooo`030000003oool0oooo01/0oooo0P00000M0?ooo`800000703o
ool2000001T0oooo1@00000I0?ooo`800000703oool2000001`0oooo0`00000M0?ooo`030000003o
ool0oooo05h0oooo00100?ooo`030000003oool0oooo01/0oooo0`00000K0?ooo`@000006@3oool5
000001T0oooo1@00000I0?ooo`D000006@3oool5000001T0oooo1@00000I0?ooo`@000006`3oool3
000001d0oooo00<000000?ooo`3oool0?`3oool00280oooo0P00000L0?ooo`<000006`3oool40000
01T0oooo1@00000J0?ooo`@000006@3oool5000001T0oooo1@00000I0?ooo`@000006P3oool40000
01T0oooo1P00000I0?ooo`@000006`3oool3000001`0oooo0P00000S0?ooo`00103oool3000001/0
oooo1000000I0?ooo`H00000603oool6000001T0oooo1@00000I0?ooo`@000006P3oool4000001X0
oooo0`00000J0?ooo`@000006P3oool4000001T0oooo1@00000I0?ooo`H00000603oool5000001X0
oooo1000000K0?ooo`<000001@3oool000@0oooo1`00000F0?ooo`P000005P3oool8000001P0oooo
1@00000J0?ooo`<000006P3oool3000001X0oooo1000000J0?ooo`<000006P3oool3000001X0oooo
1000000I0?ooo`@000006P3oool5000001P0oooo2000000F0?ooo`L000005`3oool7000000D0oooo
00040?ooo`X000004`3oool:000001@0oooo2@00000I0?ooo`@000006P3oool3000001X0oooo0`00
000J0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo1000
000I0?ooo`T00000503oool:000001<0oooo2P0000050?ooo`00103oool7000001H0oooo1@00000n
0?ooo`8000006P3oool2000001/0oooo0P00000J0?ooo`<000006P3oool3000001X0oooo0P00000K
0?ooo`8000006P3oool3000001X0oooo0P00000n0?ooo`D000005P3oool7000000D0oooo00040?oo
o`<00000N`3oool00`000000oooo0?ooo`0H0?ooo`8000006`3oool2000001/0oooo00<000000?oo
o`3oool06@3oool2000001/0oooo0P00000J0?ooo`030000003oool0oooo07T0oooo0`0000050?oo
o`00WP3oool00`000000oooo0?ooo`0I0?ooo`8000006`3oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool0W@3oool00;/0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0;X0oooo003o0?oook40oooo003o
0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o
0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o
0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o
0?oook40oooo003o0?oook40oooo003o0?oook40oooo001N0?ooo`030000003oool0oooo01d0oooo
00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0G0?ooo`D000006@3oool5000001T0oooo
1@00000H0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool07@3oool00`000000oooo
0?ooo`1M0?ooo`00?`3oool2000001d0oooo0`00000L0?ooo`<000006`3oool3000001T0oooo1@00
000I0?ooo`D000006@3oool4000001T0oooo0`00000L0?ooo`<00000703oool3000001d0oooo0P00
00100?ooo`00103oool3000001X0oooo0`00000K0?ooo`D000006P3oool4000001/0oooo1000000I
0?ooo`@000006P3oool4000001T0oooo1000000J0?ooo`@000006@3oool5000001T0oooo1000000K
0?ooo`@000006P3oool5000001/0oooo0P00000K0?ooo`<000001@3oool000@0oooo1`00000F0?oo
o`L000005`3oool8000001H0oooo1`00000H0?ooo`H000006@3oool4000001T0oooo1000000J0?oo
o`<000006P3oool3000001X0oooo1000000I0?ooo`H00000603oool7000001H0oooo2000000G0?oo
o`H000005`3oool7000000D0oooo00040?ooo`X000004`3oool:000001@0oooo2P00000D0?ooo`T0
00005`3oool5000001T0oooo1000000J0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`@0
00006@3oool5000001L0oooo2@00000D0?ooo`X00000503oool9000001@0oooo2P0000050?ooo`00
103oool7000001L0oooo1`00000F0?ooo`L000005`3oool:000001P0oooo1000000I0?ooo`<00000
6P3oool3000001X0oooo0`00000J0?ooo`8000006P3oool4000001T0oooo1000000H0?ooo`X00000
5`3oool7000001H0oooo1`00000G0?ooo`L000001@3oool000@0oooo0`00000K0?ooo`<000006P3o
ool3000003l0oooo0`00000I0?ooo`8000006`3oool2000001X0oooo0P00000K0?ooo`8000006P3o
ool3000001T0oooo0`00000o0?ooo`<000006P3oool3000001/0oooo0`0000050?ooo`00P`3oool2
000001T0oooo0P00000J0?ooo`8000006`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0
oooo01T0oooo0P00000I0?ooo`800000Q03oool009h0oooo0P00000K0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`800000W`3oool009l0
oooo00<000000?ooo`3oool0K@3oool00`000000oooo0?ooo`2N0?ooo`00o`3ooona0?ooo`00o`3o
oona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3o
oona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3o
oona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3o
oona0?ooo`00o`3ooona0?ooo`00^`3oool2000001P0oooo1@00000H0?ooo`800000_03oool000@0
oooo0P00002b0?ooo`D00000603oool5000001P0oooo1@00002b0?ooo`8000001@3oool000@0oooo
1@00000m0?ooo`<000006P3oool5000003H0oooo0P00000H0?ooo`D00000603oool5000001P0oooo
1@00000H0?ooo`800000=P3oool5000001X0oooo0`00000m0?ooo`D000001@3oool000@0oooo2000
000J0?ooo`D000005`3oool6000001H0oooo2@00000L0?ooo`030000003oool0oooo01D0oooo1@00
000H0?ooo`@000006P3oool3000001X0oooo0`00000I0?ooo`D000005P3oool00`000000oooo0?oo
o`0K0?ooo`T000005P3oool6000001L0oooo1@00000J0?ooo`P000001@3oool000@0oooo2P00000C
0?ooo`T000005@3oool8000001L0oooo1`00000K0?ooo`80000000@0oooo00000000000000005@3o
ool5000001T0oooo0`00000J0?ooo`<000006P3oool3000001T0oooo1@00000D0?ooo`<000000P3o
ool2000001/0oooo1P00000H0?ooo`P000005@3oool9000001<0oooo2P0000050?ooo`00103oool7
000001H0oooo1`00000G0?ooo`L000006@3oool5000001h0oooo1P00000D0?ooo`@000006@3oool3
000001X0oooo0`00000J0?ooo`<000006@3oool4000001<0oooo1`00000N0?ooo`D000006@3oool6
000001P0oooo1P00000G0?ooo`L000001@3oool000@0oooo0`00000J0?ooo`H000006@3oool40000
01/0oooo1000000O0?ooo`P000004P3oool4000001T0oooo0`00000J0?ooo`<000006P3oool30000
01T0oooo1000000A0?ooo`P00000803oool3000001`0oooo1000000I0?ooo`D000006`3oool30000
00D0oooo000R0?ooo`<000006`3oool3000001d0oooo00<000000?ooo`3oool07`3oool:00000100
oooo0`00000J0?ooo`800000703oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0
oooo0P00000A0?ooo`X000008@3oool00`000000oooo0?ooo`0K0?ooo`8000006`3oool3000002@0
oooo000R0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0F@3oool2000001/0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0oooo
0P00001K0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool08P3oool009d0oooo0P00
000K0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`0I0?ooo`800000WP3oool009h0oooo00<000000?ooo`3oool0K`3oool00`000000oooo0?ooo`2M
0?ooo`00WP3oool00`000000oooo0?ooo`1_0?ooo`030000003oool0oooo09d0oooo00270?oooj40
o`00R03oool00?l0oooo/@3oool00?l0oooo/@3oool009h0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`2M0?ooo`00WP3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
09d0oooo002M0?ooo`8000006`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool06@3oool2000009h0oooo002M0?ooo`8000006P3oool3000001X0oooo
0`00000J0?ooo`<000006P3oool2000009h0oooo002M0?ooo`8000006P3oool3000001X0oooo0`00
000J0?ooo`<000006P3oool2000009h0oooo00250?ooo`030000003oool0oooo01@0oooo1000000I
0?ooo`<000006P3oool3000001X0oooo0`00000I0?ooo`@000005P3oool00`000000oooo0?ooo`24
0?ooo`00Q03oool2000001H0oooo1000000I0?ooo`<000006P3oool3000001X0oooo0`00000I0?oo
o`@000005P3oool2000008D0oooo00230?ooo`<000005P3oool4000001P0oooo1@00000H0?ooo`D0
0000603oool5000001P0oooo1000000F0?ooo`<00000Q03oool00880oooo0`00000F0?ooo`D00000
603oool5000001P0oooo1@00000H0?ooo`D00000603oool5000001H0oooo0`0000230?ooo`00PP3o
ool3000001H0oooo1@00000H0?ooo`D00000603oool5000001P0oooo1@00000H0?ooo`D000005P3o
ool3000008<0oooo00210?ooo`@00000603oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01H0oooo100000220?ooo`00P03oool5000001P0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`0F0?ooo`D00000P@3oool006D0oooo00<000000?ooo`3oool05`3oool5000001T0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0G0?ooo`D000006@3oool00`000000
oooo0?ooo`1T0?ooo`00I03oool2000001X0oooo1000000I0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool05`3oool4000001X0oooo0P00001U0?ooo`000`3oool300000440oooo00<0
00000?ooo`3oool06@3oool2000001`0oooo0`00000I0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool05`3oool3000001/0oooo0`00000K0?ooo`030000003oool0oooo03l0oooo0`00
00040?ooo`000`3oool7000003`0oooo0P00000J0?ooo`<000006`3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I
0?ooo`<000006P3oool2000003`0oooo1`0000040?ooo`000`3oool:000001/0oooo0P00000K0?oo
o`8000006P3oool4000001/0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool06@3oool4000001X0oooo0P00000K0?oo
o`8000006`3oool:000000@0oooo00030?ooo`P00000703oool2000001/0oooo0`00000I0?ooo`D0
0000i@3oool5000001T0oooo0`00000J0?ooo`<00000703oool8000000@0oooo00030?ooo`D00000
7P3oool3000001/0oooo0`00000H0?ooo`D00000iP3oool6000001P0oooo0`00000K0?ooo`<00000
7P3oool5000000@0oooo00030?ooo`800000803oool3000001/0oooo0`00000J0?ooo`@00000i`3o
ool4000001X0oooo0`00000J0?ooo`@00000803oool2000000@0oooo000S0?ooo`D000006P3oool4
000001`0oooo0P00003W0?ooo`800000703oool4000001T0oooo1P00000T0?ooo`008P3oool50000
01X0oooo1@00003o0?ooob@0oooo1@00000I0?ooo`H000008`3oool002<0oooo1000000K0?ooo`<0
0000o`3ooolV0?ooo`<000006P3oool5000002@0oooo000T0?ooo`8000007P3oool00`000000oooo
0?ooo`3o0?ooob@0oooo00<000000?ooo`3oool06`3oool3000002D0oooo000U0?ooo`030000003o
ool0oooo0?l0ooooH@3oool00`000000oooo0?ooo`0U0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00o`3ooona0?ooo`00^P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool0^@3oool009`0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`2K0?ooo`00W03oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool06P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo09/0oooo002L0?oo
o`8000006`3oool3000001X0oooo0`00000J0?ooo`<000006`3oool2000009d0oooo002K0?ooo`<0
00006`3oool3000001X0oooo0`00000J0?ooo`<000006`3oool3000009`0oooo001n0?ooo`030000
003oool0oooo01X0oooo0`00000K0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`@00000
703oool00`000000oooo0?ooo`1m0?ooo`00OP3oool00`000000oooo0?ooo`0J0?ooo`@000006P3o
ool3000001X0oooo0`00000J0?ooo`<000006P3oool4000001`0oooo00<000000?ooo`3oool0O@3o
ool007h0oooo0P00000K0?ooo`@000006@3oool5000001P0oooo1@00000H0?ooo`D000006@3oool4
000001/0oooo0P00001o0?ooo`00OP3oool2000001X0oooo1P00000H0?ooo`D00000603oool50000
01P0oooo1@00000H0?ooo`H000006P3oool2000007l0oooo001n0?ooo`8000006P3oool6000001P0
oooo1@00000H0?ooo`D00000603oool5000001P0oooo1P00000I0?ooo`<00000O`3oool007d0oooo
1000000I0?ooo`@00000703oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool06P3oool4000001T0oooo1000001n0?ooo`00H@3oool00`000000oooo0?oo
o`0I0?ooo`@00000703oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo1000
000K0?ooo`030000003oool0oooo0600oooo001Q0?ooo`030000003oool0oooo01T0oooo1@00000K
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06@3oool5000001X0oooo0P00001R
0?ooo`00H@3oool2000001X0oooo1@00000K0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool06@3oool5000001X0oooo0P00001R0?ooo`004P3oool300000300oooo00<000000?ooo`3o
ool06@3oool2000001X0oooo1000000L0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool06P3oool4000001T0oooo0`00000K0?ooo`030000003oool0oooo02h0oooo0`00000C0?ooo`00
3P3oool7000001<0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`<00000
703oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0J0?ooo`<000006P3oool2000001`0oooo00<000000?ooo`3oool0
4@3oool7000000l0oooo000;0?ooo`X000004`3oool2000001/0oooo0P00000J0?ooo`<00000703o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0I0?ooo`@000006P3oool2000001/0oooo0P00000C0?ooo`X00000303o
ool000d0oooo2000000C0?ooo`8000006`3oool2000001X0oooo1000003U0?ooo`@000006@3oool3
000001/0oooo0P00000C0?ooo`P000003P3oool00100oooo1@00000C0?ooo`<000006P3oool30000
01T0oooo1000003T0?ooo`D000006@3oool3000001X0oooo0`00000C0?ooo`D000004@3oool001<0
oooo0P00000C0?ooo`<000006P3oool3000001T0oooo1@00003S0?ooo`D00000603oool4000001T0
oooo1000000C0?ooo`800000503oool002T0oooo0`00000I0?ooo`@00000603oool300000>L0oooo
0`00000H0?ooo`@000006@3oool3000002X0oooo000Y0?ooo`@00000603oool400000?l0oooo7@3o
ool5000001P0oooo1000000Z0?ooo`00:@3oool4000001P0oooo1@00003o0?oooa`0oooo1@00000H
0?ooo`@00000:P3oool002T0oooo1@00000G0?ooo`<00000o`3ooolP0?ooo`<000005`3oool50000
02X0oooo000Y0?ooo`<00000L@3oool00`000000oooo0?ooo`1a0?ooo`030000003oool0oooo06l0
oooo0`00000Z0?ooo`00W@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo09`0oooo
002L0?ooo`800000703oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool06@3oool2000009h0oooo002L0?ooo`<000006`3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06@3oool3000009d0oooo002K0?oo
o`@000006P3oool3000001X0oooo0`00000J0?ooo`<000006P3oool3000009d0oooo002K0?ooo`@0
00006P3oool3000001X0oooo0`00000J0?ooo`<000006P3oool3000009d0oooo002K0?ooo`@00000
6P3oool3000001X0oooo0`00000J0?ooo`<000006@3oool4000009d0oooo002J0?ooo`H000006@3o
ool3000001X0oooo0`00000J0?ooo`<000006@3oool5000009`0oooo002J0?ooo`H00000603oool5
000001P0oooo1@00000H0?ooo`D00000603oool5000009`0oooo002M0?ooo`030000003oool0oooo
01P0oooo1@00000H0?ooo`D00000603oool5000001X0oooo00<000000?ooo`3oool0W03oool009d0
oooo00<000000?ooo`3oool0603oool5000001P0oooo1@00000H0?ooo`D000006P3oool00`000000
oooo0?ooo`2L0?ooo`00W@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo09`0oooo
001`0?ooo`<00000:P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo02P0oooo0`00
001a0?ooo`00L@3oool6000002H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0T
0?ooo`H00000LP3oool00780oooo2000000S0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool08@3oool8000007<0oooo001c0?ooo`H00000903oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo0240oooo1`00001d0?ooo`00M@3oool4000002@0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0R0?ooo`@00000MP3oool005h0oooo0P00000F0?ooo`80000000<0
oooo0000000000008P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0200oooo1@00
000F0?ooo`800000G`3oool00140oooo0`00001;0?ooo`@00000503oool010000000oooo0?ooo`3o
ool200000200oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0N0?ooo`8000000`3o
ool00`000000oooo0?ooo`0B0?ooo`@00000B`3oool300000180oooo000=0?ooo`L00000;`3oool2
000001/0oooo1P00000G0?ooo`8000007P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01`0oooo0P00000G0?ooo`H000006`3oool2000002l0oooo1`00000>0?ooo`002P3oool:0000
01<0oooo0`00000J0?ooo`@000006@3oool7000001L0oooo0P00000L0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool06P3oool2000001L0oooo1`00000I0?ooo`@000006P3oool30000
01<0oooo2P00000;0?ooo`003@3oool7000001@0oooo1P00000G0?ooo`H000005P3oool600000>40
oooo1P00000F0?ooo`H000005`3oool6000001@0oooo1`00000>0?ooo`004@3oool3000001D0oooo
2000000E0?ooo`L000005@3oool400000>80oooo1@00000E0?ooo`L000005@3oool8000001D0oooo
0`00000B0?ooo`00:P3oool6000001H0oooo1P00000G0?ooo`<000008@3ooonQ0?l00240oooo0`00
000G0?ooo`H000005@3oool7000002/0oooo000/0?ooo`@000005`3oool4000001T0oooo00<00000
0?ooo`3oool0h`3oool00`000000oooo0?ooo`0F0?ooo`D000005`3oool4000002d0oooo000]0?oo
o`8000006@3oool300000?l0oooo6P3oool3000001P0oooo0`00000^0?ooo`00;P3oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo0?l0oooo6P3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo02d0oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oo
ok40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oo
ok40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oo
ok40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo00200?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool5000001P0oooo1@00000H0?oo
o`D000006`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo07l0oooo000A0?ooo`80
0000D`3oool2000001L0oooo0`00000J0?ooo`<000006@3oool5000001P0oooo1@00000H0?ooo`D0
00006@3oool3000001X0oooo0`00000G0?ooo`800000D`3oool200000180oooo000=0?ooo`H00000
6@3oool5000001L0oooo1@00000E0?ooo`H000005P3oool4000001X0oooo1@00000G0?ooo`@00000
6@3oool4000001X0oooo1000000G0?ooo`D000006@3oool5000001H0oooo1P00000E0?ooo`D00000
5`3oool5000001T0oooo1P00000>0?ooo`002P3oool9000001@0oooo2P00000B0?ooo`X000004P3o
ool9000001D0oooo1P00000I0?ooo`@00000603oool4000001X0oooo0`00000J0?ooo`@00000603o
ool4000001T0oooo1P00000E0?ooo`T000004P3oool:00000180oooo2P00000D0?ooo`T000002`3o
ool000`0oooo1`00000F0?ooo`L000005@3oool7000001D0oooo1`00000E0?ooo`L00000603oool4
000001P0oooo0`00000K0?ooo`<000006P3oool3000001T0oooo1000000H0?ooo`L000005@3oool7
000001@0oooo2000000D0?ooo`P000005P3oool7000000d0oooo000@0?ooo`<00000603oool50000
01L0oooo1@00000I0?ooo`<00000503oool6000001T0oooo1000000J0?ooo`8000006`3oool30000
01/0oooo0P00000J0?ooo`@000006@3oool6000001@0oooo0`00000I0?ooo`D000005`3oool50000
01P0oooo0`00000A0?ooo`00;P3oool2000001X0oooo0P00000_0?ooo`@00000703oool3000001/0
oooo0P00000K0?ooo`800000703oool2000001/0oooo0`00000L0?ooo`@00000;`3oool2000001X0
oooo0P00000_0?ooo`00NP3oool2000001l0oooo0P00000L0?ooo`030000003oool0oooo01/0oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0J0?ooo`8000007`3oool2000007/0oooo
002K0?ooo`800000703oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool06P3oool2000009`0oooo002K0?ooo`030000003oool0oooo07D0oooo00<00000
0?ooo`3oool0VP3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3o
ool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3o
ool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3o
ool00?l0oooo/@3oool00?l0oooo/@3oool00840oooo00<000000?ooo`3oool06@3oool00`000000
oooo0?ooo`1a0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0P03oool006L0oooo
00<000000?ooo`3oool05P3oool3000001X0oooo0`00000H0?ooo`H000005`3oool5000001L0oooo
1@00000I0?ooo`<000006P3oool2000001T0oooo00<000000?ooo`3oool0IP3oool002h0oooo0P00
000K0?ooo`030000003oool0oooo01L0oooo0`00000H0?ooo`@000006@3oool5000001H0oooo1@00
000H0?ooo`D000005`3oool5000001L0oooo1@00000H0?ooo`D000005`3oool4000001T0oooo00<0
00000?ooo`3oool06@3oool2000002l0oooo000A0?ooo`8000006@3oool4000001T0oooo0`00000H
0?ooo`D000005P3oool6000001L0oooo1@00000G0?ooo`D00000603oool4000001T0oooo1000000H
0?ooo`D000005P3oool6000001L0oooo1@00000H0?ooo`<000006@3oool4000001T0oooo0P00000B
0?ooo`003@3oool6000001H0oooo1`00000G0?ooo`D000005`3oool6000001H0oooo1@00000H0?oo
o`@00000603oool4000001X0oooo0`00000I0?ooo`@000006@3oool4000001L0oooo1P00000E0?oo
o`L000005P3oool6000001L0oooo1`00000F0?ooo`H000003P3oool000X0oooo2@00000D0?ooo`T0
00005@3oool8000001@0oooo2@00000D0?ooo`@000006P3oool3000001T0oooo1000000J0?ooo`<0
00006P3oool3000001X0oooo0`00000I0?ooo`@000005@3oool9000001@0oooo2000000D0?ooo`X0
0000503oool9000000/0oooo000<0?ooo`L000005P3oool7000001<0oooo2P00000C0?ooo`D00000
6@3oool3000001/0oooo0`00000J0?ooo`8000006`3oool3000001X0oooo0`00000J0?ooo`<00000
6P3oool4000001T0oooo1@00000C0?ooo`X000004P3oool7000001L0oooo1`00000=0?ooo`00403o
ool3000001X0oooo0`00001=0?ooo`<000006`3oool3000001/0oooo0P00000K0?ooo`800000703o
ool2000001/0oooo0`00000K0?ooo`800000C@3oool3000001/0oooo0`00000A0?ooo`00O@3oool0
0`000000oooo0?ooo`0K0?ooo`800000703oool00`000000oooo0?ooo`0K0?ooo`030000003oool0
oooo01X0oooo0P00000L0?ooo`8000007@3oool00`000000oooo0?ooo`1l0?ooo`00V`3oool00`00
0000oooo0?ooo`0K0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool06`3oool00`00
0000oooo0?ooo`0K0?ooo`030000003oool0oooo09X0oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
001V0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?oo
o`1a0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`1U0?ooo`00BP3oool00`000000oooo0?ooo`0H0?ooo`<000006@3oool2000001X0oooo0`00000I
0?ooo`8000006P3oool5000001X0oooo0P00000I0?ooo`<000006P3oool2000001T0oooo0`00000I
0?ooo`030000003oool0oooo04X0oooo000_0?ooo`030000003oool0oooo01L0oooo0P00000J0?oo
o`<00000603oool5000001P0oooo1@00000G0?ooo`D000005`3oool5000001L0oooo1@00000G0?oo
o`D00000603oool5000001L0oooo1000000H0?ooo`<000006P3oool00`000000oooo0?ooo`0^0?oo
o`00;@3oool3000001P0oooo1000000H0?ooo`D000005`3oool6000001L0oooo1000000H0?ooo`@0
0000603oool5000001L0oooo1@00000H0?ooo`@000005`3oool6000001L0oooo1@00000G0?ooo`@0
00006@3oool3000002h0oooo000A0?ooo`<000005`3oool5000001L0oooo1@00000G0?ooo`L00000
5P3oool5000001P0oooo1000000H0?ooo`@000006@3oool3000001T0oooo1000000H0?ooo`@00000
603oool5000001H0oooo1P00000F0?ooo`L000005P3oool6000001L0oooo0`00000B0?ooo`003@3o
ool7000001D0oooo2000000E0?ooo`L000005P3oool6000001H0oooo1@00000H0?ooo`@000006@3o
ool3000001X0oooo0`00000I0?ooo`@000006@3oool4000001P0oooo1@00000F0?ooo`H000005@3o
ool8000001D0oooo2000000E0?ooo`L000003P3oool000X0oooo2P00000C0?ooo`X00000503oool6
000001L0oooo1@00000H0?ooo`@000006@3oool3000001X0oooo0`00000J0?ooo`<000006P3oool3
000001X0oooo0`00000I0?ooo`@00000603oool5000001L0oooo1P00000D0?ooo`X000004`3oool:
000000/0oooo000=0?ooo`L00000<03oool3000001/0oooo0`00000J0?ooo`8000006`3oool20000
01/0oooo0P00000K0?ooo`<000006P3oool3000001/0oooo0P00000K0?ooo`8000006P3oool30000
01/0oooo0`00000`0?ooo`L000003P3oool00140oooo0`00001=0?ooo`8000006`3oool2000001`0
oooo0P00000K0?ooo`800000703oool00`000000oooo0?ooo`0J0?ooo`8000006`3oool2000001`0
oooo0P00000K0?ooo`800000C@3oool300000180oooo001n0?ooo`030000003oool0oooo01/0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01X0oooo
0P00000L0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0O@3oool00;T0oooo00<0
00000?ooo`3oool06`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0;P0oooo003o
0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o
0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o
0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o
0?oook40oooo001U0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0Z`3oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo06@0oooo00190?ooo`030000003oool0oooo01T0
oooo0P00000I0?ooo`<000006@3oool2000001X0oooo0`00000i0?ooo`<000006P3oool2000001T0
oooo0`00000I0?ooo`8000006`3oool00`000000oooo0?ooo`180?ooo`00;P3oool00`000000oooo
0?ooo`0G0?ooo`<000006@3oool4000001P0oooo1@00000G0?ooo`D000005`3oool6000001L0oooo
1@00000G0?ooo`H000005`3oool5000001L0oooo1@00000H0?ooo`@000006@3oool3000001P0oooo
00<000000?ooo`3oool0;P3oool002`0oooo0`00000I0?ooo`@00000603oool5000001L0oooo1000
000H0?ooo`@00000603oool6000001L0oooo1@00000G0?ooo`H000005`3oool5000001P0oooo1000
000G0?ooo`D00000603oool4000001P0oooo0`00000^0?ooo`00:`3oool5000001L0oooo1P00000G
0?ooo`@00000603oool4000001P0oooo1000000I0?ooo`@00000603oool4000001T0oooo1000000I
0?ooo`@00000603oool4000001P0oooo1000000G0?ooo`H000005P3oool5000002d0oooo000A0?oo
o`<000005P3oool6000001H0oooo1P00000G0?ooo`@00000603oool4000001T0oooo0`00000J0?oo
o`@000006@3oool3000001T0oooo1000000I0?ooo`@000006@3oool4000001P0oooo1000000G0?oo
o`H000005@3oool7000001H0oooo0`00000B0?ooo`003@3oool7000001@0oooo2@00000D0?ooo`D0
00006@3oool3000001T0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo0`00000I0?ooo`@0
00006P3oool3000001X0oooo0`00000I0?ooo`<000006@3oool5000001@0oooo2@00000D0?ooo`L0
00003P3oool000X0oooo2P00000C0?ooo`D000006@3oool3000001/0oooo0P00000J0?ooo`800000
6`3oool2000001/0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo0`00000K0?ooo`800000
6P3oool2000001/0oooo0`00000I0?ooo`D000004`3oool:000000/0oooo000=0?ooo`L00000<03o
ool2000001`0oooo0P00000K0?ooo`8000006`3oool2000001`0oooo0P00000J0?ooo`8000006`3o
ool2000001`0oooo0P00000K0?ooo`8000006`3oool2000001`0oooo0P00000`0?ooo`L000003P3o
ool00140oooo0`00001>0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0J0?ooo`800000703oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo04`0oooo0`00000B0?ooo`00W03oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
09/0oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oo
ok40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oo
ok40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oo
ok40oooo003o0?oook40oooo00170?ooo`030000003oool0oooo0?l0oooo7P3oool00`000000oooo
0?ooo`160?ooo`00;@3oool00`000000oooo0?ooo`0G0?ooo`<000006@3oool2000001X0oooo0P00
002]0?ooo`8000006P3oool2000001T0oooo0`00000I0?ooo`030000003oool0oooo02`0oooo000/
0?ooo`<00000603oool5000001L0oooo1@00000G0?ooo`D00000603oool5000001L0oooo1@00000H
0?ooo`D00000603oool5000001L0oooo1@00000H0?ooo`D000005`3oool5000001L0oooo1@00000H
0?ooo`<00000;@3oool002`0oooo1000000F0?ooo`D00000603oool4000001P0oooo1000000I0?oo
o`@00000603oool5000001P0oooo1@00000H0?ooo`D000005`3oool5000001P0oooo1@00000H0?oo
o`@00000603oool5000001H0oooo1000000]0?ooo`00:`3oool6000001D0oooo1000000I0?ooo`@0
0000603oool4000001T0oooo1000000H0?ooo`D00000603oool5000001P0oooo1@00000H0?ooo`@0
00006@3oool4000001P0oooo1000000I0?ooo`@000005@3oool6000002`0oooo000Z0?ooo`H00000
5P3oool3000001T0oooo1000000I0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`<00000
6P3oool3000001T0oooo1000000I0?ooo`@000006@3oool3000001X0oooo0`00000F0?ooo`H00000
:`3oool00180oooo0P00000E0?ooo`D00000603oool3000001T0oooo0`00000J0?ooo`<000006P3o
ool3000001X0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`<000006P3o
ool3000001T0oooo0`00000H0?ooo`D00000503oool3000001<0oooo000>0?ooo`H000005@3oool3
000001T0oooo0`00000J0?ooo`8000006`3oool2000001/0oooo0P00000K0?ooo`<000006P3oool3
000001X0oooo0`00000J0?ooo`<000006P3oool3000001/0oooo0P00000J0?ooo`<000006@3oool3
000001@0oooo1`00000?0?ooo`002`3oool9000001@0oooo0P00000K0?ooo`8000006`3oool20000
01/0oooo0P00000K0?ooo`8000006`3oool3000001X0oooo0`00000J0?ooo`<000006`3oool20000
01/0oooo0P00000K0?ooo`8000006`3oool2000001/0oooo0P00000C0?ooo`X00000303oool000d0
oooo1`00000a0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo0P00000K0?ooo`800000703oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo02h0oooo1`00000?0?ooo`004@3oool3000006/0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool0J03oool3000001<0oooo002j0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`2i0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`009P3oool60000
01L0oooo1P00003o0?oooah0oooo1P00000G0?ooo`H000009`3oool002H0oooo1P00000G0?ooo`H0
00005`3oool6000001P0oooo1@00002Y0?ooo`D00000603oool5000001P0oooo1P00000G0?ooo`H0
00009`3oool002H0oooo1@00000I0?ooo`D000005`3oool6000001P0oooo1@00000H0?ooo`D00000
603oool5000001P0oooo1@00000H0?ooo`D00000603oool5000001P0oooo1@00000H0?ooo`D00000
603oool5000001T0oooo1000000X0?ooo`009`3oool4000001T0oooo1000000I0?ooo`@000006@3o
ool5000001P0oooo1@00000H0?ooo`D00000603oool5000001P0oooo1@00000H0?ooo`D00000603o
ool5000001P0oooo1000000J0?ooo`@000006@3oool4000002P0oooo000W0?ooo`<000006P3oool4
000001T0oooo1000000J0?ooo`<000006@3oool4000001T0oooo1@00000H0?ooo`D00000603oool5
000001P0oooo1000000J0?ooo`<000006P3oool3000001X0oooo1000000I0?ooo`@00000:03oool0
02L0oooo0`00000K0?ooo`<000006@3oool4000001X0oooo0`00000J0?ooo`<000006P3oool30000
01X0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`<000006P3oool30000
01/0oooo0`00000X0?ooo`009`3oool3000001/0oooo0`00000J0?ooo`<000006P3oool3000001X0
oooo0`00000J0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`<000006P3oool3000001X0
oooo0`00000J0?ooo`<000006`3oool2000002T0oooo000X0?ooo`030000003oool0oooo01X0oooo
0P00000K0?ooo`8000006`3oool3000001X0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo
0`00000J0?ooo`<000006P3oool3000001X0oooo0P00000L0?ooo`800000703oool00`000000oooo
0?ooo`0W0?ooo`002`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0oooo0P00
000K0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`800000703oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo00X0oooo00160?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool0A@3oool00800oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool0O`3oool00;X0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0;T0oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo0000\
\>"],
  ImageRangeCache->{{{0, 431}, {413.375, 0}} -> {-1.88239, 2.60708, 0.364618, \
0.364618}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Capacitance", "Subsection"],

Cell["\<\
According to Gauss' law, specialized to two-dimensions, the total charge per \
unit transverse length on an electrode is\
\>", "Text"],

Cell[BoxData[
    \(Q\  = \ \(\[CurlyEpsilon]\_0\) \
\(\[ContourIntegral]\[VeryThinSpace]\(E\& \[RightVector] \)\[CenterDot]\
\[DifferentialD]\(S\& \[RightVector] \)\)\)], "DisplayFormula",
  FontFamily->"Times New Roman"],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`\[CurlyEpsilon]\_0\)]],
  " is the electric permittivity, ",
  Cell[BoxData[
      \(TraditionalForm\`E\&\[RightVector] = \(-\[Del]\&\[RightVector] \[Phi]\
\)\)]],
  " is the electric field, ",
  Cell[BoxData[
      \(TraditionalForm\`\[DifferentialD]S\&\[RightVector] = \(n\&^\) \
\[DifferentialD]S\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`\[DifferentialD]S\)]],
  " is a differential length for any closed curve that encompasses the \
electrode (and none other), and ",
  Cell[BoxData[
      \(TraditionalForm\`n\&^\)]],
  " is the outward normal unit vector for that curve.  Assuming that there \
are only two electrodes, the capacitance (per unit length) for this system is \
then "
}], "Text"],

Cell[BoxData[{
    \(C = Q/V\ \), "\[IndentingNewLine]", 
    \(V\  = \ \(\[Integral]\_2\%1\[VeryThinSpace]\(E\& \[RightVector] \)\
\[CenterDot]\[DifferentialD]\(s\& \[RightVector] \)\  = \ \[CapitalDelta]\
\[VeryThinSpace]\[Phi]\_1 - \[CapitalDelta]\[VeryThinSpace]\[Phi]\_2\)\)}], \
"DisplayFormula",
  FontFamily->"Times New Roman"],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`V\)]],
  " is the potential difference between the two electrodes."
}], "Text"],

Cell["\<\
For the present problem, we choose to enclose the upper electrode by a \
rectangle, as sketched below, and we know from the chosen boundary conditions \
that the potential difference between the two electrodes is 2 V.  Note that \
we prefer to use the exact voltage rather than compute the potential \
difference using a line integral because the discontinuities across the \
plates cause numerical instabilities in that integration.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(PlotGradientField[\(-f[x, y]\), {x, 1, 100}, {y, 5, 100}, 
        PlotPoints \[Rule] 15, ScaleFactor \[Rule] 5, 
        Epilog \[Rule] {RGBColor[1, 0, 0], Line[{{31, 41}, {70, 41}}], 
            Line[{{31, 60}, {70, 60}}], GrayLevel[0], Dashing[{0.02, 0.02}], 
            Line[{{20, 50}, {80, 50}, {80, 70}, {20, 70}, {20, 
                  50}}]}];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .95927 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0178805 0.00954672 -0.0248939 0.00954672 [
[ 0 0 0 0 ]
[ 1 .95927 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
% Start of user PostScript
/mathtops {
gsave
MBeginOrig
moveto
MEndOrig
currentpoint
grestore
} bind def
/MAtocoords {
mathtops 4 2 roll mathtops
4 copy pop pop
3 -1 roll sub
/arry exch def
exch sub
/arrx exch def
arrx dup mul
arry dup mul
add sqrt
/arrl exch def
translate
} bind def
/MAarrowhead1 {
gsave
MAtocoords
arrl 0. eq
{ 0 0 Mdot }
{
[ arrx arrl div
arry arrl div
-1 arry mul arrl div
arrx arrl div
0 0 ]
concat
-0.02 0.005 moveto 0 0 lineto
-0.02 -0.005 lineto fill
-0.02 0.005 moveto 0 0 lineto
-0.02 -0.005 lineto
-0.02 0.005 lineto stroke
} ifelse
grestore
} def
/MAarrowzero1 {
gsave
mathtops translate
0 0 Mdot
grestore
} def
% End of user PostScript
0 g
.0001 w
[ ] 0 setdash
.02743 .02284 m
.0281 .02284 L
s
% Start of user PostScript
1. 5. 1.0701088 5. MAarrowhead1
% End of user PostScript
.02743 .08762 m
.02919 .08762 L
s
% Start of user PostScript
1. 11.785714 1.184255 11.785714 MAarrowhead1
% End of user PostScript
.02743 .1524 m
.03017 .1524 L
s
% Start of user PostScript
1. 18.571429 1.2871421 18.571429 MAarrowhead1
% End of user PostScript
.02743 .21718 m
.03089 .21718 L
s
% Start of user PostScript
1. 25.357143 1.3629783 25.357143 MAarrowhead1
% End of user PostScript
.02743 .28196 m
.03109 .28196 L
s
% Start of user PostScript
1. 32.142857 1.3832754 32.142857 MAarrowhead1
% End of user PostScript
.02743 .34675 m
.03042 .34675 L
s
% Start of user PostScript
1. 38.928571 1.3135282 38.928571 MAarrowhead1
% End of user PostScript
.02743 .41153 m
.02887 .41153 L
s
% Start of user PostScript
1. 45.714286 1.1508526 45.714286 MAarrowhead1
% End of user PostScript
.02743 .47631 m
.02681 .47631 L
s
% Start of user PostScript
1. 52.5 0.93496001 52.5 MAarrowhead1
% End of user PostScript
.02743 .54109 m
.02497 .54109 L
s
% Start of user PostScript
1. 59.285714 0.7425454 59.285714 MAarrowhead1
% End of user PostScript
.02743 .60587 m
.02393 .60587 L
s
% Start of user PostScript
1. 66.071429 0.63347304 66.071429 MAarrowhead1
% End of user PostScript
.02743 .67065 m
.02381 .67065 L
s
% Start of user PostScript
1. 72.857143 0.62105144 72.857143 MAarrowhead1
% End of user PostScript
.02743 .73543 m
.02435 .73543 L
s
% Start of user PostScript
1. 79.642857 0.67775583 79.642857 MAarrowhead1
% End of user PostScript
.02743 .80022 m
.02525 .80022 L
s
% Start of user PostScript
1. 86.428571 0.77147576 86.428571 MAarrowhead1
% End of user PostScript
.02743 .865 m
.02632 .865 L
s
% Start of user PostScript
1. 93.214286 0.88354199 93.214286 MAarrowhead1
% End of user PostScript
.02743 .92978 m
.02743 .92978 L
s
% Start of user PostScript
1. 100. MAarrowzero1
% End of user PostScript
.09494 .02284 m
.09561 .02402 L
s
% Start of user PostScript
8.0714286 5. 8.1416168 5.1231677 MAarrowhead1
% End of user PostScript
.09494 .08762 m
.09674 .08874 L
s
% Start of user PostScript
8.0714286 11.785714 8.2606219 11.903225 MAarrowhead1
% End of user PostScript
.09494 .1524 m
.09781 .15337 L
s
% Start of user PostScript
8.0714286 18.571429 8.3728035 18.67289 MAarrowhead1
% End of user PostScript
.09494 .21718 m
.09867 .21777 L
s
% Start of user PostScript
8.0714286 25.357143 8.4630109 25.418737 MAarrowhead1
% End of user PostScript
.09494 .28196 m
.09905 .28181 L
s
% Start of user PostScript
8.0714286 32.142857 8.5022606 32.127126 MAarrowhead1
% End of user PostScript
.09494 .34675 m
.09848 .34555 L
s
% Start of user PostScript
8.0714286 38.928571 8.4428044 38.803552 MAarrowhead1
% End of user PostScript
.09494 .41153 m
.09669 .40939 L
s
% Start of user PostScript
8.0714286 45.714286 8.2554987 45.490618 MAarrowhead1
% End of user PostScript
.09494 .47631 m
.09418 .474 L
s
% Start of user PostScript
8.0714286 52.5 7.9918267 52.257896 MAarrowhead1
% End of user PostScript
.09494 .54109 m
.09198 .53947 L
s
% Start of user PostScript
8.0714286 59.285714 7.7622082 59.116019 MAarrowhead1
% End of user PostScript
.09494 .60587 m
.09092 .60532 L
s
% Start of user PostScript
8.0714286 66.071429 7.650447 66.013824 MAarrowhead1
% End of user PostScript
.09494 .67065 m
.09096 .67099 L
s
% Start of user PostScript
8.0714286 72.857143 7.6552737 72.892246 MAarrowhead1
% End of user PostScript
.09494 .73543 m
.09167 .73628 L
s
% Start of user PostScript
8.0714286 79.642857 7.7291725 79.731113 MAarrowhead1
% End of user PostScript
.09494 .80022 m
.09268 .80129 L
s
% Start of user PostScript
8.0714286 86.428571 7.8348885 86.541535 MAarrowhead1
% End of user PostScript
.09494 .865 m
.0938 .86616 L
s
% Start of user PostScript
8.0714286 93.214286 7.952066 93.336428 MAarrowhead1
% End of user PostScript
.09494 .92978 m
.09494 .93097 L
s
% Start of user PostScript
8.0714286 100. 8.0714286 100.12459 MAarrowhead1
% End of user PostScript
.16245 .02284 m
.16315 .02523 L
s
% Start of user PostScript
15.142857 5. 15.216378 5.2508935 MAarrowhead1
% End of user PostScript
.16245 .08762 m
.16436 .08997 L
s
% Start of user PostScript
15.142857 11.785714 15.343512 12.031262 MAarrowhead1
% End of user PostScript
.16245 .1524 m
.16564 .15455 L
s
% Start of user PostScript
15.142857 18.571429 15.477681 18.796499 MAarrowhead1
% End of user PostScript
.16245 .21718 m
.16696 .21877 L
s
% Start of user PostScript
15.142857 25.357143 15.615861 25.523537 MAarrowhead1
% End of user PostScript
.16245 .28196 m
.16808 .28213 L
s
% Start of user PostScript
15.142857 32.142857 15.733585 32.160069 MAarrowhead1
% End of user PostScript
.16245 .34675 m
.16802 .34429 L
s
% Start of user PostScript
15.142857 38.928571 15.727237 38.671185 MAarrowhead1
% End of user PostScript
.16245 .41153 m
.16546 .40636 L
s
% Start of user PostScript
15.142857 45.714286 15.458138 45.172935 MAarrowhead1
% End of user PostScript
.16245 .47631 m
.16113 .47053 L
s
% Start of user PostScript
15.142857 52.5 15.005372 51.894839 MAarrowhead1
% End of user PostScript
.16245 .54109 m
.15759 .5374 L
s
% Start of user PostScript
15.142857 59.285714 14.634781 58.898872 MAarrowhead1
% End of user PostScript
.16245 .60587 m
.1566 .6051 L
s
% Start of user PostScript
15.142857 66.071429 14.530595 65.990657 MAarrowhead1
% End of user PostScript
.16245 .67065 m
.15741 .6718 L
s
% Start of user PostScript
15.142857 72.857143 14.615651 72.976802 MAarrowhead1
% End of user PostScript
.16245 .73543 m
.15871 .73741 L
s
% Start of user PostScript
15.142857 79.642857 14.751307 79.850358 MAarrowhead1
% End of user PostScript
.16245 .80022 m
.16001 .80251 L
s
% Start of user PostScript
15.142857 86.428571 14.887699 86.66848 MAarrowhead1
% End of user PostScript
.16245 .865 m
.16125 .86738 L
s
% Start of user PostScript
15.142857 93.214286 15.017415 93.463464 MAarrowhead1
% End of user PostScript
.16245 .92978 m
.16245 .93217 L
s
% Start of user PostScript
15.142857 100. 15.142857 100.25083 MAarrowhead1
% End of user PostScript
.22995 .02284 m
.23066 .02649 L
s
% Start of user PostScript
22.214286 5. 22.288453 5.3824361 MAarrowhead1
% End of user PostScript
.22995 .08762 m
.23191 .09132 L
s
% Start of user PostScript
22.214286 11.785714 22.41934 12.172798 MAarrowhead1
% End of user PostScript
.22995 .1524 m
.23339 .15612 L
s
% Start of user PostScript
22.214286 18.571429 22.573893 18.960547 MAarrowhead1
% End of user PostScript
.22995 .21718 m
.23534 .22063 L
s
% Start of user PostScript
22.214286 25.357143 22.778268 25.718652 MAarrowhead1
% End of user PostScript
.22995 .28196 m
.23815 .28405 L
s
% Start of user PostScript
22.214286 32.142857 23.072953 32.361028 MAarrowhead1
% End of user PostScript
.22995 .34675 m
.24089 .34358 L
s
% Start of user PostScript
22.214286 38.928571 23.359612 38.596835 MAarrowhead1
% End of user PostScript
.22995 .41153 m
.23641 .40046 L
s
% Start of user PostScript
22.214286 45.714286 22.890794 44.555458 MAarrowhead1
% End of user PostScript
.22995 .47631 m
.22718 .4638 L
s
% Start of user PostScript
22.214286 52.5 21.924037 51.189311 MAarrowhead1
% End of user PostScript
.22995 .54109 m
.21967 .5343 L
s
% Start of user PostScript
22.214286 59.285714 21.136775 58.574589 MAarrowhead1
% End of user PostScript
.22995 .60587 m
.22037 .60651 L
s
% Start of user PostScript
22.214286 66.071429 21.210094 66.138094 MAarrowhead1
% End of user PostScript
.22995 .67065 m
.22354 .67378 L
s
% Start of user PostScript
22.214286 72.857143 21.542159 73.184367 MAarrowhead1
% End of user PostScript
.22995 .73543 m
.2258 .7391 L
s
% Start of user PostScript
22.214286 79.642857 21.778825 80.02656 MAarrowhead1
% End of user PostScript
.22995 .80022 m
.22743 .80392 L
s
% Start of user PostScript
22.214286 86.428571 21.94979 86.816897 MAarrowhead1
% End of user PostScript
.22995 .865 m
.22875 .86866 L
s
% Start of user PostScript
22.214286 93.214286 22.087872 93.598449 MAarrowhead1
% End of user PostScript
.22995 .92978 m
.22995 .93341 L
s
% Start of user PostScript
22.214286 100. 22.214286 100.38043 MAarrowhead1
% End of user PostScript
.29746 .02284 m
.2981 .0277 L
s
% Start of user PostScript
29.285714 5. 29.352767 5.5088455 MAarrowhead1
% End of user PostScript
.29746 .08762 m
.29924 .09268 L
s
% Start of user PostScript
29.285714 11.785714 29.472083 12.315398 MAarrowhead1
% End of user PostScript
.29746 .1524 m
.30067 .15788 L
s
% Start of user PostScript
29.285714 18.571429 29.621876 19.145257 MAarrowhead1
% End of user PostScript
.29746 .21718 m
.30275 .22338 L
s
% Start of user PostScript
29.285714 25.357143 29.839713 26.006501 MAarrowhead1
% End of user PostScript
.29746 .28196 m
.30676 .28947 L
s
% Start of user PostScript
29.285714 32.142857 30.259526 32.928695 MAarrowhead1
% End of user PostScript
.29746 .34675 m
.32359 .35539 L
s
% Start of user PostScript
29.285714 38.928571 32.022498 39.834275 MAarrowhead1
% End of user PostScript
.29746 .41153 m
.30899 .38485 L
s
% Start of user PostScript
29.285714 45.714286 30.493086 42.919845 MAarrowhead1
% End of user PostScript
.29746 .47631 m
.29325 .4503 L
s
% Start of user PostScript
29.285714 52.5 28.844698 49.775956 MAarrowhead1
% End of user PostScript
.29746 .54109 m
.26122 .52099 L
s
% Start of user PostScript
29.285714 59.285714 25.488819 57.180068 MAarrowhead1
% End of user PostScript
.29746 .60587 m
.28476 .61429 L
s
% Start of user PostScript
29.285714 66.071429 27.955378 66.953592 MAarrowhead1
% End of user PostScript
.29746 .67065 m
.29092 .67727 L
s
% Start of user PostScript
29.285714 72.857143 28.600006 73.54995 MAarrowhead1
% End of user PostScript
.29746 .73543 m
.29352 .74116 L
s
% Start of user PostScript
29.285714 79.642857 28.872587 80.24313 MAarrowhead1
% End of user PostScript
.29746 .80022 m
.29515 .80543 L
s
% Start of user PostScript
29.285714 86.428571 29.043512 86.974627 MAarrowhead1
% End of user PostScript
.29746 .865 m
.29637 .86992 L
s
% Start of user PostScript
29.285714 93.214286 29.171607 93.729515 MAarrowhead1
% End of user PostScript
.29746 .92978 m
.29746 .93459 L
s
% Start of user PostScript
29.285714 100. 29.285714 100.50451 MAarrowhead1
% End of user PostScript
.36497 .02284 m
.36544 .02871 L
s
% Start of user PostScript
36.357143 5. 36.405821 5.6144934 MAarrowhead1
% End of user PostScript
.36497 .08762 m
.3663 .09383 L
s
% Start of user PostScript
36.357143 11.785714 36.496008 12.435874 MAarrowhead1
% End of user PostScript
.36497 .1524 m
.36732 .15938 L
s
% Start of user PostScript
36.357143 18.571429 36.603421 19.302181 MAarrowhead1
% End of user PostScript
.36497 .21718 m
.36863 .2256 L
s
% Start of user PostScript
36.357143 25.357143 36.740314 26.238289 MAarrowhead1
% End of user PostScript
.36497 .28196 m
.37 .29331 L
s
% Start of user PostScript
36.357143 32.142857 36.883672 33.331038 MAarrowhead1
% End of user PostScript
.36497 .34675 m
.36786 .36401 L
s
% Start of user PostScript
36.357143 38.928571 36.659275 40.736696 MAarrowhead1
% End of user PostScript
.36497 .41153 m
.36729 .374 L
s
% Start of user PostScript
36.357143 45.714286 36.599925 41.783428 MAarrowhead1
% End of user PostScript
.36497 .47631 m
.36376 .44032 L
s
% Start of user PostScript
36.357143 52.5 36.229987 48.730562 MAarrowhead1
% End of user PostScript
.36497 .54109 m
.36437 .49336 L
s
% Start of user PostScript
36.357143 59.285714 36.294053 54.286116 MAarrowhead1
% End of user PostScript
.36497 .60587 m
.35985 .61933 L
s
% Start of user PostScript
36.357143 66.071429 35.820765 67.481184 MAarrowhead1
% End of user PostScript
.36497 .67065 m
.3607 .67999 L
s
% Start of user PostScript
36.357143 72.857143 35.910076 73.835293 MAarrowhead1
% End of user PostScript
.36497 .73543 m
.36211 .74291 L
s
% Start of user PostScript
36.357143 79.642857 36.057337 80.425516 MAarrowhead1
% End of user PostScript
.36497 .80022 m
.36324 .80669 L
s
% Start of user PostScript
36.357143 86.428571 36.175901 87.107121 MAarrowhead1
% End of user PostScript
.36497 .865 m
.36416 .87096 L
s
% Start of user PostScript
36.357143 93.214286 36.272063 93.839376 MAarrowhead1
% End of user PostScript
.36497 .92978 m
.36497 .93557 L
s
% Start of user PostScript
36.357143 100. 36.357143 100.60706 MAarrowhead1
% End of user PostScript
.43248 .02284 m
.43275 .02936 L
s
% Start of user PostScript
43.428571 5. 43.457193 5.6833711 MAarrowhead1
% End of user PostScript
.43248 .08762 m
.43319 .09456 L
s
% Start of user PostScript
43.428571 11.785714 43.502723 12.512787 MAarrowhead1
% End of user PostScript
.43248 .1524 m
.43369 .16025 L
s
% Start of user PostScript
43.428571 18.571429 43.554879 19.392962 MAarrowhead1
% End of user PostScript
.43248 .21718 m
.43416 .22648 L
s
% Start of user PostScript
43.428571 25.357143 43.604708 26.331124 MAarrowhead1
% End of user PostScript
.43248 .28196 m
.43426 .29326 L
s
% Start of user PostScript
43.428571 32.142857 43.614713 33.325618 MAarrowhead1
% End of user PostScript
.43248 .34675 m
.43309 .35976 L
s
% Start of user PostScript
43.428571 38.928571 43.492598 40.291535 MAarrowhead1
% End of user PostScript
.43248 .41153 m
.4327 .37376 L
s
% Start of user PostScript
43.428571 45.714286 43.451492 41.758023 MAarrowhead1
% End of user PostScript
.43248 .47631 m
.43235 .43872 L
s
% Start of user PostScript
43.428571 52.5 43.415188 48.562993 MAarrowhead1
% End of user PostScript
.43248 .54109 m
.43244 .49665 L
s
% Start of user PostScript
43.428571 59.285714 43.424512 54.630726 MAarrowhead1
% End of user PostScript
.43248 .60587 m
.431 .61802 L
s
% Start of user PostScript
43.428571 66.071429 43.273494 67.343964 MAarrowhead1
% End of user PostScript
.43248 .67065 m
.43068 .68071 L
s
% Start of user PostScript
43.428571 72.857143 43.24022 73.910299 MAarrowhead1
% End of user PostScript
.43248 .73543 m
.43108 .74381 L
s
% Start of user PostScript
43.428571 79.642857 43.281725 80.520501 MAarrowhead1
% End of user PostScript
.43248 .80022 m
.43158 .80749 L
s
% Start of user PostScript
43.428571 86.428571 43.333931 87.190878 MAarrowhead1
% End of user PostScript
.43248 .865 m
.43204 .87164 L
s
% Start of user PostScript
43.428571 93.214286 43.382732 93.91013 MAarrowhead1
% End of user PostScript
.43248 .92978 m
.43248 .93621 L
s
% Start of user PostScript
43.428571 100. 43.428571 100.67387 MAarrowhead1
% End of user PostScript
.49999 .02284 m
.49997 .02959 L
s
% Start of user PostScript
50.5 5. 50.498323 5.7068278 MAarrowhead1
% End of user PostScript
.49999 .08762 m
.49998 .09481 L
s
% Start of user PostScript
50.5 11.785714 50.499273 12.538462 MAarrowhead1
% End of user PostScript
.49999 .1524 m
.49997 .16051 L
s
% Start of user PostScript
50.5 18.571429 50.498243 19.420478 MAarrowhead1
% End of user PostScript
.49999 .21718 m
.5 .22666 L
s
% Start of user PostScript
50.5 25.357143 50.500774 26.350246 MAarrowhead1
% End of user PostScript
.49999 .28196 m
.49999 .29301 L
s
% Start of user PostScript
50.5 32.142857 50.500172 33.29953 MAarrowhead1
% End of user PostScript
.49999 .34675 m
.49999 .35887 L
s
% Start of user PostScript
50.5 38.928571 50.500002 40.198897 MAarrowhead1
% End of user PostScript
.49999 .41153 m
.49999 .37376 L
s
% Start of user PostScript
50.5 45.714286 50.49978 41.75771 MAarrowhead1
% End of user PostScript
.49999 .47631 m
.49999 .43858 L
s
% Start of user PostScript
50.5 52.5 50.500187 48.5475 MAarrowhead1
% End of user PostScript
.49999 .54109 m
.49999 .49698 L
s
% Start of user PostScript
50.5 59.285714 50.500199 54.665417 MAarrowhead1
% End of user PostScript
.49999 .60587 m
.49999 .6175 L
s
% Start of user PostScript
50.5 66.071429 50.499734 67.289786 MAarrowhead1
% End of user PostScript
.49999 .67065 m
.49998 .68076 L
s
% Start of user PostScript
50.5 72.857143 50.499035 73.916008 MAarrowhead1
% End of user PostScript
.49999 .73543 m
.49999 .74406 L
s
% Start of user PostScript
50.5 79.642857 50.49966 80.546923 MAarrowhead1
% End of user PostScript
.49999 .80022 m
.5 .80774 L
s
% Start of user PostScript
50.5 86.428571 50.500707 87.21711 MAarrowhead1
% End of user PostScript
.49999 .865 m
.49999 .87187 L
s
% Start of user PostScript
50.5 93.214286 50.500229 93.933968 MAarrowhead1
% End of user PostScript
.49999 .92978 m
.49999 .93643 L
s
% Start of user PostScript
50.5 100. 50.5 100.69654 MAarrowhead1
% End of user PostScript
.5675 .02284 m
.56726 .02936 L
s
% Start of user PostScript
57.571429 5. 57.546188 5.6831982 MAarrowhead1
% End of user PostScript
.5675 .08762 m
.5668 .09456 L
s
% Start of user PostScript
57.571429 11.785714 57.498648 12.51286 MAarrowhead1
% End of user PostScript
.5675 .1524 m
.56632 .16024 L
s
% Start of user PostScript
57.571429 18.571429 57.447597 19.392845 MAarrowhead1
% End of user PostScript
.5675 .21718 m
.56581 .22649 L
s
% Start of user PostScript
57.571429 25.357143 57.394621 26.331588 MAarrowhead1
% End of user PostScript
.5675 .28196 m
.56573 .29326 L
s
% Start of user PostScript
57.571429 32.142857 57.386099 33.325498 MAarrowhead1
% End of user PostScript
.5675 .34675 m
.56689 .35976 L
s
% Start of user PostScript
57.571429 38.928571 57.507245 40.291557 MAarrowhead1
% End of user PostScript
.5675 .41153 m
.56728 .37375 L
s
% Start of user PostScript
57.571429 45.714286 57.548242 41.757637 MAarrowhead1
% End of user PostScript
.5675 .47631 m
.56763 .43872 L
s
% Start of user PostScript
57.571429 52.5 57.585125 48.562555 MAarrowhead1
% End of user PostScript
.5675 .54109 m
.56753 .49665 L
s
% Start of user PostScript
57.571429 59.285714 57.575022 54.630923 MAarrowhead1
% End of user PostScript
.5675 .60587 m
.56899 .61802 L
s
% Start of user PostScript
57.571429 66.071429 57.72803 67.343676 MAarrowhead1
% End of user PostScript
.5675 .67065 m
.56932 .68071 L
s
% Start of user PostScript
57.571429 72.857143 57.761845 73.910117 MAarrowhead1
% End of user PostScript
.5675 .73543 m
.56891 .74381 L
s
% Start of user PostScript
57.571429 79.642857 57.719252 80.520663 MAarrowhead1
% End of user PostScript
.5675 .80022 m
.56839 .80748 L
s
% Start of user PostScript
57.571429 86.428571 57.664836 87.189798 MAarrowhead1
% End of user PostScript
.5675 .865 m
.56793 .87163 L
s
% Start of user PostScript
57.571429 93.214286 57.616943 93.90957 MAarrowhead1
% End of user PostScript
.5675 .92978 m
.5675 .93621 L
s
% Start of user PostScript
57.571429 100. 57.571429 100.67345 MAarrowhead1
% End of user PostScript
.63501 .02284 m
.63452 .0287 L
s
% Start of user PostScript
64.642857 5. 64.592284 5.6142605 MAarrowhead1
% End of user PostScript
.63501 .08762 m
.63367 .09383 L
s
% Start of user PostScript
64.642857 11.785714 64.50303 12.435739 MAarrowhead1
% End of user PostScript
.63501 .1524 m
.63264 .15938 L
s
% Start of user PostScript
64.642857 18.571429 64.394925 19.302029 MAarrowhead1
% End of user PostScript
.63501 .21718 m
.63135 .2256 L
s
% Start of user PostScript
64.642857 25.357143 64.259845 26.238827 MAarrowhead1
% End of user PostScript
.63501 .28196 m
.62999 .29331 L
s
% Start of user PostScript
64.642857 32.142857 64.11689 33.330956 MAarrowhead1
% End of user PostScript
.63501 .34675 m
.63212 .36401 L
s
% Start of user PostScript
64.642857 38.928571 64.340502 40.736892 MAarrowhead1
% End of user PostScript
.63501 .41153 m
.63269 .374 L
s
% Start of user PostScript
64.642857 45.714286 64.399852 41.783527 MAarrowhead1
% End of user PostScript
.63501 .47631 m
.63622 .44032 L
s
% Start of user PostScript
64.642857 52.5 64.769991 48.730626 MAarrowhead1
% End of user PostScript
.63501 .54109 m
.63561 .49336 L
s
% Start of user PostScript
64.642857 59.285714 64.705687 54.286109 MAarrowhead1
% End of user PostScript
.63501 .60587 m
.64013 .61933 L
s
% Start of user PostScript
64.642857 66.071429 65.179327 67.481258 MAarrowhead1
% End of user PostScript
.63501 .67065 m
.63926 .67999 L
s
% Start of user PostScript
64.642857 72.857143 65.088412 73.8349 MAarrowhead1
% End of user PostScript
.63501 .73543 m
.63786 .7429 L
s
% Start of user PostScript
64.642857 79.642857 64.942145 80.424913 MAarrowhead1
% End of user PostScript
.63501 .80022 m
.63674 .8067 L
s
% Start of user PostScript
64.642857 86.428571 64.824018 87.10785 MAarrowhead1
% End of user PostScript
.63501 .865 m
.63582 .87096 L
s
% Start of user PostScript
64.642857 93.214286 64.727834 93.83852 MAarrowhead1
% End of user PostScript
.63501 .92978 m
.63501 .93557 L
s
% Start of user PostScript
64.642857 100. 64.642857 100.60631 MAarrowhead1
% End of user PostScript
.70252 .02284 m
.70189 .0277 L
s
% Start of user PostScript
71.714286 5. 71.649073 5.5095436 MAarrowhead1
% End of user PostScript
.70252 .08762 m
.70074 .09268 L
s
% Start of user PostScript
71.714286 11.785714 71.528093 12.315742 MAarrowhead1
% End of user PostScript
.70252 .1524 m
.69931 .15788 L
s
% Start of user PostScript
71.714286 18.571429 71.378641 19.145722 MAarrowhead1
% End of user PostScript
.70252 .21718 m
.69723 .22337 L
s
% Start of user PostScript
71.714286 25.357143 71.160722 26.005437 MAarrowhead1
% End of user PostScript
.70252 .28196 m
.69322 .28946 L
s
% Start of user PostScript
71.714286 32.142857 70.740884 32.928449 MAarrowhead1
% End of user PostScript
.70252 .34675 m
.67638 .35538 L
s
% Start of user PostScript
71.714286 38.928571 68.976806 39.83335 MAarrowhead1
% End of user PostScript
.70252 .41153 m
.69098 .38485 L
s
% Start of user PostScript
71.714286 45.714286 70.506147 42.919613 MAarrowhead1
% End of user PostScript
.70252 .47631 m
.70673 .45031 L
s
% Start of user PostScript
71.714286 52.5 72.155325 49.776233 MAarrowhead1
% End of user PostScript
.70252 .54109 m
.73877 .52098 L
s
% Start of user PostScript
71.714286 59.285714 75.511909 57.178862 MAarrowhead1
% End of user PostScript
.70252 .60587 m
.71522 .61429 L
s
% Start of user PostScript
71.714286 66.071429 73.044544 66.953771 MAarrowhead1
% End of user PostScript
.70252 .67065 m
.70907 .67727 L
s
% Start of user PostScript
71.714286 72.857143 72.400502 73.550036 MAarrowhead1
% End of user PostScript
.70252 .73543 m
.70647 .74118 L
s
% Start of user PostScript
71.714286 79.642857 72.128126 80.244459 MAarrowhead1
% End of user PostScript
.70252 .80022 m
.70483 .80543 L
s
% Start of user PostScript
71.714286 86.428571 71.95709 86.974421 MAarrowhead1
% End of user PostScript
.70252 .865 m
.7036 .86992 L
s
% Start of user PostScript
71.714286 93.214286 71.827763 93.730445 MAarrowhead1
% End of user PostScript
.70252 .92978 m
.70252 .93461 L
s
% Start of user PostScript
71.714286 100. 71.714286 100.50583 MAarrowhead1
% End of user PostScript
.77003 .02284 m
.76932 .02649 L
s
% Start of user PostScript
78.785714 5. 78.712082 5.3825368 MAarrowhead1
% End of user PostScript
.77003 .08762 m
.76807 .09132 L
s
% Start of user PostScript
78.785714 11.785714 78.580863 12.173234 MAarrowhead1
% End of user PostScript
.77003 .1524 m
.76659 .15611 L
s
% Start of user PostScript
78.785714 18.571429 78.425832 18.959461 MAarrowhead1
% End of user PostScript
.77003 .21718 m
.76464 .22064 L
s
% Start of user PostScript
78.785714 25.357143 78.22166 25.719131 MAarrowhead1
% End of user PostScript
.77003 .28196 m
.76182 .28405 L
s
% Start of user PostScript
78.785714 32.142857 77.925843 32.361201 MAarrowhead1
% End of user PostScript
.77003 .34675 m
.7591 .34359 L
s
% Start of user PostScript
78.785714 38.928571 77.641014 38.597613 MAarrowhead1
% End of user PostScript
.77003 .41153 m
.76357 .40047 L
s
% Start of user PostScript
78.785714 45.714286 78.109842 44.555533 MAarrowhead1
% End of user PostScript
.77003 .47631 m
.77279 .4638 L
s
% Start of user PostScript
78.785714 52.5 79.075722 51.189246 MAarrowhead1
% End of user PostScript
.77003 .54109 m
.78031 .53429 L
s
% Start of user PostScript
78.785714 59.285714 79.86249 58.573915 MAarrowhead1
% End of user PostScript
.77003 .60587 m
.77961 .60651 L
s
% Start of user PostScript
78.785714 66.071429 79.789913 66.138344 MAarrowhead1
% End of user PostScript
.77003 .67065 m
.77644 .67378 L
s
% Start of user PostScript
78.785714 72.857143 79.45793 73.184262 MAarrowhead1
% End of user PostScript
.77003 .73543 m
.77418 .73909 L
s
% Start of user PostScript
78.785714 79.642857 79.221242 80.025876 MAarrowhead1
% End of user PostScript
.77003 .80022 m
.77255 .80393 L
s
% Start of user PostScript
78.785714 86.428571 79.049801 86.817357 MAarrowhead1
% End of user PostScript
.77003 .865 m
.77123 .86867 L
s
% Start of user PostScript
78.785714 93.214286 78.912184 93.598913 MAarrowhead1
% End of user PostScript
.77003 .92978 m
.77003 .93341 L
s
% Start of user PostScript
78.785714 100. 78.785714 100.38067 MAarrowhead1
% End of user PostScript
.83753 .02284 m
.83683 .02523 L
s
% Start of user PostScript
85.857143 5. 85.783047 5.2507494 MAarrowhead1
% End of user PostScript
.83753 .08762 m
.83561 .08997 L
s
% Start of user PostScript
85.857143 11.785714 85.655891 12.031598 MAarrowhead1
% End of user PostScript
.83753 .1524 m
.83434 .15455 L
s
% Start of user PostScript
85.857143 18.571429 85.522623 18.796553 MAarrowhead1
% End of user PostScript
.83753 .21718 m
.83302 .21876 L
s
% Start of user PostScript
85.857143 25.357143 85.384396 25.522693 MAarrowhead1
% End of user PostScript
.83753 .28196 m
.8319 .28213 L
s
% Start of user PostScript
85.857143 32.142857 85.267225 32.159899 MAarrowhead1
% End of user PostScript
.83753 .34675 m
.83196 .34429 L
s
% Start of user PostScript
85.857143 38.928571 85.273 38.671595 MAarrowhead1
% End of user PostScript
.83753 .41153 m
.83453 .40636 L
s
% Start of user PostScript
85.857143 45.714286 85.542603 45.172816 MAarrowhead1
% End of user PostScript
.83753 .47631 m
.83885 .47055 L
s
% Start of user PostScript
85.857143 52.5 85.994804 51.896484 MAarrowhead1
% End of user PostScript
.83753 .54109 m
.84238 .5374 L
s
% Start of user PostScript
85.857143 59.285714 86.364953 58.898932 MAarrowhead1
% End of user PostScript
.83753 .60587 m
.84338 .6051 L
s
% Start of user PostScript
85.857143 66.071429 86.469033 65.990848 MAarrowhead1
% End of user PostScript
.83753 .67065 m
.84256 .67179 L
s
% Start of user PostScript
85.857143 72.857143 86.38369 72.976439 MAarrowhead1
% End of user PostScript
.83753 .73543 m
.84127 .73742 L
s
% Start of user PostScript
85.857143 79.642857 86.248715 79.851253 MAarrowhead1
% End of user PostScript
.83753 .80022 m
.83997 .80251 L
s
% Start of user PostScript
85.857143 86.428571 86.112131 86.668518 MAarrowhead1
% End of user PostScript
.83753 .865 m
.83873 .86738 L
s
% Start of user PostScript
85.857143 93.214286 85.982428 93.464346 MAarrowhead1
% End of user PostScript
.83753 .92978 m
.83753 .93218 L
s
% Start of user PostScript
85.857143 100. 85.857143 100.25122 MAarrowhead1
% End of user PostScript
.90504 .02284 m
.90437 .02401 L
s
% Start of user PostScript
92.928571 5. 92.858092 5.1227005 MAarrowhead1
% End of user PostScript
.90504 .08762 m
.90324 .08875 L
s
% Start of user PostScript
92.928571 11.785714 92.739171 11.903458 MAarrowhead1
% End of user PostScript
.90504 .1524 m
.90217 .15337 L
s
% Start of user PostScript
92.928571 18.571429 92.627464 18.672533 MAarrowhead1
% End of user PostScript
.90504 .21718 m
.9013 .21778 L
s
% Start of user PostScript
92.928571 25.357143 92.536713 25.419149 MAarrowhead1
% End of user PostScript
.90504 .28196 m
.90092 .28182 L
s
% Start of user PostScript
92.928571 32.142857 92.49709 32.12793 MAarrowhead1
% End of user PostScript
.90504 .34675 m
.90149 .34555 L
s
% Start of user PostScript
92.928571 38.928571 92.55688 38.803323 MAarrowhead1
% End of user PostScript
.90504 .41153 m
.90328 .4094 L
s
% Start of user PostScript
92.928571 45.714286 92.743648 45.491044 MAarrowhead1
% End of user PostScript
.90504 .47631 m
.9058 .47397 L
s
% Start of user PostScript
92.928571 52.5 93.008095 52.2555 MAarrowhead1
% End of user PostScript
.90504 .54109 m
.908 .53947 L
s
% Start of user PostScript
92.928571 59.285714 93.238148 59.115853 MAarrowhead1
% End of user PostScript
.90504 .60587 m
.90907 .60532 L
s
% Start of user PostScript
92.928571 66.071429 93.350537 66.013416 MAarrowhead1
% End of user PostScript
.90504 .67065 m
.90903 .671 L
s
% Start of user PostScript
92.928571 72.857143 93.346021 72.893569 MAarrowhead1
% End of user PostScript
.90504 .73543 m
.90831 .73628 L
s
% Start of user PostScript
92.928571 79.642857 93.270238 79.731017 MAarrowhead1
% End of user PostScript
.90504 .80022 m
.90731 .80129 L
s
% Start of user PostScript
92.928571 86.428571 93.16578 86.541655 MAarrowhead1
% End of user PostScript
.90504 .865 m
.90618 .86615 L
s
% Start of user PostScript
92.928571 93.214286 93.048044 93.335547 MAarrowhead1
% End of user PostScript
.90504 .92978 m
.90504 .93096 L
s
% Start of user PostScript
92.928571 100. 92.928571 100.12402 MAarrowhead1
% End of user PostScript
.97255 .02284 m
.9719 .02284 L
s
% Start of user PostScript
100. 5. 99.931747 5. MAarrowhead1
% End of user PostScript
.97255 .08762 m
.9708 .08762 L
s
% Start of user PostScript
100. 11.785714 99.816142 11.785714 MAarrowhead1
% End of user PostScript
.97255 .1524 m
.96981 .1524 L
s
% Start of user PostScript
100. 18.571429 99.71313 18.571429 MAarrowhead1
% End of user PostScript
.97255 .21718 m
.96909 .21718 L
s
% Start of user PostScript
100. 25.357143 99.637685 25.357143 MAarrowhead1
% End of user PostScript
.97255 .28196 m
.9689 .28196 L
s
% Start of user PostScript
100. 32.142857 99.617591 32.142857 MAarrowhead1
% End of user PostScript
.97255 .34675 m
.96955 .34675 L
s
% Start of user PostScript
100. 38.928571 99.685311 38.928571 MAarrowhead1
% End of user PostScript
.97255 .41153 m
.97111 .41153 L
s
% Start of user PostScript
100. 45.714286 99.849199 45.714286 MAarrowhead1
% End of user PostScript
.97255 .47631 m
.97317 .47631 L
s
% Start of user PostScript
100. 52.5 100.06501 52.5 MAarrowhead1
% End of user PostScript
.97255 .54109 m
.97501 .54109 L
s
% Start of user PostScript
100. 59.285714 100.25748 59.285714 MAarrowhead1
% End of user PostScript
.97255 .60587 m
.97604 .60587 L
s
% Start of user PostScript
100. 66.071429 100.36534 66.071429 MAarrowhead1
% End of user PostScript
.97255 .67065 m
.97619 .67065 L
s
% Start of user PostScript
100. 72.857143 100.38109 72.857143 MAarrowhead1
% End of user PostScript
.97255 .73543 m
.97563 .73543 L
s
% Start of user PostScript
100. 79.642857 100.3228 79.642857 MAarrowhead1
% End of user PostScript
.97255 .80022 m
.97474 .80022 L
s
% Start of user PostScript
100. 86.428571 100.22871 86.428571 MAarrowhead1
% End of user PostScript
.97255 .865 m
.97367 .865 L
s
% Start of user PostScript
100. 93.214286 100.11684 93.214286 MAarrowhead1
% End of user PostScript
.97255 .92978 m
.97255 .92978 L
s
% Start of user PostScript
100. 100. MAarrowzero1
% End of user PostScript
1 0 0 r
.5 Mabswid
.31383 .36652 m
.68615 .36652 L
s
.31383 .54791 m
.68615 .54791 L
s
0 g
[ .02 .02 ] 0 setdash
.20881 .45244 m
.78162 .45244 L
.78162 .64338 L
.20881 .64338 L
.20881 .45244 L
s
0 0 m
1 0 L
1 .95927 L
0 .95927 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 276.25},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`0001WR000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`009`3oool00`000000oooo0?ooo`0J0?ooo`800000o`3ooolT0?ooo`80
0000703oool00`000000oooo0?ooo`0V0?ooo`009@3oool3000001T0oooo1@00000L0?ooo`800000
703oool200000:/0oooo0P00000L0?ooo`800000703oool5000001T0oooo0`00000V0?ooo`008`3o
ool5000001X0oooo1000000I0?ooo`D000006@3oool5000001P0oooo1P00000G0?ooo`H00000603o
ool5000001P0oooo1@00000H0?ooo`H00000603oool5000001T0oooo1@00000I0?ooo`@000006@3o
ool6000002@0oooo000T0?ooo`D000006@3oool4000001T0oooo1@00000I0?ooo`D00000603oool6
000001L0oooo1P00000H0?ooo`D00000603oool5000001P0oooo1P00000H0?ooo`D000006@3oool4
000001X0oooo1000000I0?ooo`D000009@3oool002D0oooo1000000J0?ooo`<000006P3oool40000
01X0oooo1000000I0?ooo`@000006@3oool5000001P0oooo1@00000H0?ooo`D00000603oool50000
01T0oooo1000000J0?ooo`@000006P3oool3000001X0oooo1000000V0?ooo`009P3oool3000001X0
oooo1000000I0?ooo`@000006P3oool4000001T0oooo1000000I0?ooo`@000006P3oool3000001X0
oooo0`00000J0?ooo`@000006@3oool4000001X0oooo0`00000J0?ooo`<000006`3oool3000002L0
oooo00030?ooo`<000008@3oool2000001/0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo
1000000I0?ooo`@000006P3oool3000001X0oooo0`00000J0?ooo`<000006`3oool2000001/0oooo
0`00000J0?ooo`<000006P3oool300000240oooo0`0000040?ooo`000`3oool7000001h0oooo0P00
000K0?ooo`8000006P3oool3000001/0oooo0P00000K0?ooo`<000006P3oool3000001X0oooo0`00
000J0?ooo`<000006P3oool3000001/0oooo0P00000K0?ooo`8000006`3oool2000001/0oooo0P00
000N0?ooo`L00000103oool000<0oooo2P00000K0?ooo`8000006`3oool2000001/0oooo0P00000K
0?ooo`8000006`3oool2000001/0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo0`00000K
0?ooo`8000006`3oool2000001/0oooo0P00000K0?ooo`8000006`3oool:000000@0oooo00030?oo
o`L00000?03oool00`000000oooo0?ooo`0I0?ooo`800000703oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo01T0oooo0P00000L0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0j0?ooo`L00000103oool000<0oooo0`00001M0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`1K0?ooo`<0
0000103oool00;X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo0;T0oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40
oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40
oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40
oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo00130?ooo`030000003oool0oooo
01/0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`2[0?ooo`030000003oool0oooo
01`0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`120?ooo`008`3oool00`000000
oooo0?ooo`0L0?ooo`8000006`3oool3000001`0oooo0`00000K0?ooo`<000006`3oool2000001T0
oooo1@00000I0?ooo`8000006`3oool2000001`0oooo0`00000L0?ooo`<000006`3oool2000001h0
oooo00<000000?ooo`3oool08P3oool00280oooo0`00000K0?ooo`D000006@3oool4000001X0oooo
1@00000H0?ooo`H00000603oool5000001T0oooo1@00000I0?ooo`D00000603oool5000001T0oooo
1@00000I0?ooo`D000006@3oool5000001/0oooo0P00000T0?ooo`008P3oool4000001T0oooo1P00
000J0?ooo`@000006P3oool4000001P0oooo1P00000H0?ooo`D000006@3oool5000001T0oooo1@00
000H0?ooo`D000006@3oool4000001X0oooo1000000J0?ooo`H000006@3oool4000002<0oooo000Q
0?ooo`H000006@3oool5000001/0oooo0`00000J0?ooo`@000006@3oool5000001T0oooo1000000J
0?ooo`<000006P3oool4000001T0oooo1000000J0?ooo`@000006P3oool3000001X0oooo1P00000I
0?ooo`D000008`3oool000<0oooo0`00000K0?ooo`L000006P3oool4000001X0oooo0`00000K0?oo
o`<000006@3oool5000001T0oooo1000000J0?ooo`<000006P3oool4000001T0oooo1000000J0?oo
o`<000006`3oool2000001/0oooo1000000J0?ooo`L000006`3oool2000000D0oooo00030?ooo`L0
00006@3oool6000001X0oooo0`00000K0?ooo`8000006`3oool3000001X0oooo0`00000J0?ooo`<0
00006`3oool3000001/0oooo0P00000K0?ooo`8000006`3oool3000001X0oooo0`00000K0?ooo`<0
00006P3oool6000001T0oooo1P0000050?ooo`000`3oool:000001X0oooo0`00000J0?ooo`<00000
6`3oool2000001/0oooo0P00000J0?ooo`<000006`3oool2000001/0oooo0`00000K0?ooo`800000
6`3oool2000001/0oooo0P00000K0?ooo`8000006`3oool3000001X0oooo0`00000J0?ooo`T00000
1@3oool000<0oooo1`00000l0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06@3o
ool2000001/0oooo0P00000K0?ooo`800000703oool00`000000oooo0?ooo`0J0?ooo`8000006`3o
ool2000001/0oooo0P00000K0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0>@3o
ool7000000D0oooo00030?ooo`<00000NP3oool00`000000oooo0?ooo`0I0?ooo`800000703oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo07L0oooo0`0000050?ooo`00W@3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo09`0oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo00110?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0703oool0
0`000000oooo0?ooo`0J0?ooo`800000703oool2000003L0oooo0P00000L0?ooo`800000703oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0@03oool0
0280oooo00<000000?ooo`3oool06`3oool3000001/0oooo1000000K0?ooo`<000006@3oool50000
01T0oooo1@00000I0?ooo`D000006@3oool5000001T0oooo1@00000I0?ooo`<000006`3oool40000
01/0oooo0P00000N0?ooo`030000003oool0oooo0240oooo000R0?ooo`<000006`3oool4000001T0
oooo1@00000I0?ooo`D000006P3oool4000001T0oooo1@00000I0?ooo`D000006@3oool5000001T0
oooo1000000I0?ooo`H000006@3oool5000001T0oooo1000000K0?ooo`<000008`3oool00240oooo
1@00000I0?ooo`H000006@3oool5000001T0oooo1@00000I0?ooo`@000006P3oool4000001T0oooo
1000000J0?ooo`@000006P3oool4000001T0oooo1@00000I0?ooo`D000006@3oool5000001X0oooo
1000000S0?ooo`00103oool3000001X0oooo1`00000G0?ooo`H000006`3oool3000001X0oooo1000
000J0?ooo`<000006P3oool4000001X0oooo0`00000J0?ooo`@000006P3oool3000001X0oooo1000
000J0?ooo`<000006`3oool6000001L0oooo1`00000J0?ooo`<000001@3oool000@0oooo1`00000F
0?ooo`T000005`3oool5000001/0oooo0`00000J0?ooo`<000006P3oool4000001T0oooo0`00000K
0?ooo`<000006`3oool2000001X0oooo0`00000K0?ooo`<000006P3oool3000001/0oooo1@00000G
0?ooo`T000005P3oool7000000D0oooo00040?ooo`X00000603oool5000001P0oooo1000000K0?oo
o`8000006`3oool2000001/0oooo0`00000J0?ooo`8000006`3oool3000001/0oooo0P00000J0?oo
o`<000006P3oool3000001/0oooo0P00000K0?ooo`@00000603oool5000001P0oooo2P0000050?oo
o`00103oool7000003/0oooo0P00000L0?ooo`030000003oool0oooo01T0oooo0P00000K0?ooo`80
00006P3oool2000001/0oooo0P00000L0?ooo`8000006P3oool2000001/0oooo0P00000K0?ooo`03
0000003oool0oooo01X0oooo0P00000k0?ooo`L000001@3oool000@0oooo0`00001j0?ooo`030000
003oool0oooo01T0oooo0P00000K0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0I0?ooo`8000006`3oool00`000000oooo0?ooo`1h0?ooo`<00000
1@3oool009h0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`2M0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00OP3oool00`000000oooo0?ooo`2_0?ooo`030000
003oool0oooo07d0oooo001O0?ooo`030000003oool0oooo01/0oooo0P00000M0?ooo`800000703o
ool2000001T0oooo1@00000I0?ooo`800000703oool2000001`0oooo0`00000M0?ooo`030000003o
ool0oooo05h0oooo00100?ooo`030000003oool0oooo01/0oooo0`00000K0?ooo`@000006@3oool5
000001T0oooo1@00000I0?ooo`D000006@3oool5000001T0oooo1@00000I0?ooo`@000006`3oool3
000001d0oooo00<000000?ooo`3oool0?`3oool00280oooo0P00000L0?ooo`<000006`3oool40000
01T0oooo1@00000J0?ooo`@000006@3oool5000001T0oooo1@00000I0?ooo`@000006P3oool40000
01T0oooo1P00000I0?ooo`@000006`3oool3000001`0oooo0P00000S0?ooo`00103oool3000001/0
oooo1000000I0?ooo`H00000603oool6000001T0oooo1@00000I0?ooo`@000006P3oool4000001X0
oooo0`00000J0?ooo`@000006P3oool4000001T0oooo1@00000I0?ooo`H00000603oool5000001X0
oooo1000000K0?ooo`<000001@3oool000@0oooo1`00000F0?ooo`P000005P3oool8000001P0oooo
1@00000J0?ooo`<000006P3oool3000001X0oooo1000000J0?ooo`<000006P3oool3000001X0oooo
1000000I0?ooo`@000006P3oool5000001P0oooo2000000F0?ooo`L000005`3oool7000000D0oooo
00040?ooo`X000004`3oool:000001@0oooo2@00000I0?ooo`@000006P3oool3000001X0oooo0`00
000J0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo1000
000I0?ooo`T00000503oool:000001<0oooo2P0000050?ooo`00103oool7000001H0oooo1@00000n
0?ooo`8000006P3oool2000001/0oooo0P00000J0?ooo`<000006P3oool3000001X0oooo0P00000K
0?ooo`8000006P3oool3000001X0oooo0P00000n0?ooo`D000005P3oool7000000D0oooo00040?oo
o`<00000N`3oool00`000000oooo0?ooo`0H0?ooo`8000006`3oool2000001/0oooo00<000000?oo
o`3oool06@3oool2000001/0oooo0P00000J0?ooo`030000003oool0oooo07T0oooo0`0000050?oo
o`00WP3oool00`000000oooo0?ooo`0I0?ooo`8000006`3oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool0W@3oool00;/0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0;X0oooo003o0?oook40oooo003o
0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o
0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o
0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o
0?oook40oooo003o0?oook40oooo003o0?oook40oooo001N0?ooo`030000003oool0oooo01d0oooo
00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0G0?ooo`D000006@3oool5000001T0oooo
1@00000H0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool07@3oool00`000000oooo
0?ooo`1M0?ooo`00?`3oool2000001d0oooo0`00000L0?ooo`<000006`3oool3000001T0oooo1@00
000I0?ooo`D000006@3oool4000001T0oooo0`00000L0?ooo`<00000703oool3000001d0oooo0P00
00100?ooo`00103oool3000001X0oooo0`00000K0?ooo`D000006P3oool4000001/0oooo1000000I
0?ooo`@000006P3oool4000001T0oooo1000000J0?ooo`@000006@3oool5000001T0oooo1000000K
0?ooo`@000006P3oool5000001/0oooo0P00000K0?ooo`<000001@3oool000@0oooo1`00000F0?oo
o`L000005`3oool8000001H0oooo1`00000H0?ooo`H000006@3oool4000001T0oooo1000000J0?oo
o`<000006P3oool3000001X0oooo1000000I0?ooo`H00000603oool7000001H0oooo2000000G0?oo
o`H000005`3oool7000000D0oooo00040?ooo`X000004`3oool:000001@0oooo2P00000D0?ooo`T0
00005`3oool5000001T0oooo1000000J0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`@0
00006@3oool5000001L0oooo2@00000D0?ooo`X00000503oool9000001@0oooo2P0000050?ooo`00
103oool7000001L0oooo1`00000F0?ooo`L000005`3oool:000001P0oooo1000000I0?ooo`<00000
6P3oool3000001X0oooo0`00000J0?ooo`8000006P3oool4000001T0oooo1000000H0?ooo`X00000
5`3oool7000001H0oooo1`00000G0?ooo`L000001@3oool000@0oooo0`00000K0?ooo`<000006P3o
ool3000003l0oooo0`00000I0?ooo`8000006`3oool2000001X0oooo0P00000K0?ooo`8000006P3o
ool3000001T0oooo0`00000o0?ooo`<000006P3oool3000001/0oooo0`0000050?ooo`00P`3oool2
000001T0oooo0P00000J0?ooo`8000006`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0
oooo01T0oooo0P00000I0?ooo`800000Q03oool009h0oooo0P00000K0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`800000W`3oool009l0
oooo00<000000?ooo`3oool0K@3oool00`000000oooo0?ooo`2N0?ooo`00o`3ooona0?ooo`00o`3o
oona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3o
oona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3o
oona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3o
oona0?ooo`00o`3ooona0?ooo`00^`3oool2000001P0oooo1@00000H0?ooo`800000_03oool000@0
oooo0P00002b0?ooo`D00000603oool5000001P0oooo1@00002b0?ooo`8000001@3oool000@0oooo
1@00000m0?ooo`<000006P3oool5000003H0oooo0P00000H0?ooo`D00000603oool5000001P0oooo
1@00000H0?ooo`800000=P3oool5000001X0oooo0`00000m0?ooo`D000001@3oool000@0oooo2000
000J0?ooo`D000005`3oool6000001H0oooo2@00000L0?ooo`030000003oool0oooo01D0oooo1@00
000H0?ooo`@000006P3oool3000001X0oooo0`00000I0?ooo`D000005P3oool00`000000oooo0?oo
o`0K0?ooo`T000005P3oool6000001L0oooo1@00000J0?ooo`P000001@3oool000@0oooo2P00000C
0?ooo`T000005@3oool8000001L0oooo1`00000K0?ooo`80000000@0oooo00000000000000005@3o
ool5000001T0oooo0`00000J0?ooo`<000006P3oool3000001T0oooo1@00000D0?ooo`<000000P3o
ool2000001/0oooo1P00000H0?ooo`P000005@3oool9000001<0oooo2P0000050?ooo`00103oool7
000001H0oooo1`00000G0?ooo`L000006@3oool5000001h0oooo1P00000D0?ooo`@000006@3oool3
000001X0oooo0`00000J0?ooo`<000006@3oool4000001<0oooo1`00000N0?ooo`D000006@3oool6
000001P0oooo1P00000G0?ooo`L000001@3oool000@0oooo0`00000J0?ooo`H000006@3oool40000
01/0oooo1000000O0?ooo`P000004P3oool4000001T0oooo0`00000J0?ooo`<000006P3oool30000
01T0oooo1000000A0?ooo`P00000803oool3000001`0oooo1000000I0?ooo`D000006`3oool30000
00D0oooo000R0?ooo`<000006`3oool3000001d0oooo00<000000?ooo`3oool07`3oool:00000100
oooo0`00000J0?ooo`800000703oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0
oooo0P00000A0?ooo`X000008@3oool00`000000oooo0?ooo`0K0?ooo`8000006`3oool3000002@0
oooo000R0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0F@3oool2000001/0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0oooo
0P00001K0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool08P3oool009d0oooo0P00
000K0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`0I0?ooo`800000WP3oool009h0oooo00<000000?ooo`3oool0K`3oool00`000000oooo0?ooo`2M
0?ooo`00WP3oool00`000000oooo0?ooo`1_0?ooo`030000003oool0oooo09d0oooo00270?oooj40
o`00R03oool00?l0oooo/@3oool00?l0oooo/@3oool009h0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`2M0?ooo`00WP3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
09d0oooo002M0?ooo`8000006`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool06@3oool2000009h0oooo002M0?ooo`8000006P3oool3000001X0oooo
0`00000J0?ooo`<000006P3oool2000009h0oooo002M0?ooo`8000006P3oool3000001X0oooo0`00
000J0?ooo`<000006P3oool2000009h0oooo00250?ooo`030000003oool0oooo01@0oooo1000000I
0?ooo`<000006P3oool3000001X0oooo0`00000I0?ooo`@000005P3oool00`000000oooo0?ooo`24
0?ooo`00Q03oool2000001H0oooo1000000I0?ooo`<000006P3oool3000001X0oooo0`00000I0?oo
o`@000005P3oool2000008D0oooo00230?ooo`<000005P3oool4000001P0oooo1@00000H0?ooo`D0
0000603oool5000001P0oooo1000000F0?ooo`<00000Q03oool00880oooo0`00000F0?ooo`D00000
603oool5000001P0oooo1@00000H0?ooo`D00000603oool5000001H0oooo0`0000230?ooo`00PP3o
ool3000001H0oooo1@00000H0?ooo`D00000603oool5000001P0oooo1@00000H0?ooo`D000005P3o
ool3000008<0oooo00210?ooo`@00000603oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01H0oooo100000220?ooo`00P03oool5000001P0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`0F0?ooo`D00000P@3oool006D0oooo00<000000?ooo`3oool05`3oool5000001T0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0G0?ooo`D000006@3oool00`000000
oooo0?ooo`1T0?ooo`00I03oool2000001X0oooo1000000I0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool05`3oool4000001X0oooo0P00001U0?ooo`000`3oool300000440oooo00<0
00000?ooo`3oool06@3oool2000001`0oooo0`00000I0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool05`3oool3000001/0oooo0`00000K0?ooo`030000003oool0oooo03l0oooo0`00
00040?ooo`000`3oool7000003`0oooo0P00000J0?ooo`<000006`3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I
0?ooo`<000006P3oool2000003`0oooo1`0000040?ooo`000`3oool:000001/0oooo0P00000K0?oo
o`8000006P3oool4000001/0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool06@3oool4000001X0oooo0P00000K0?oo
o`8000006`3oool:000000@0oooo00030?ooo`P00000703oool2000001/0oooo0`00000I0?ooo`D0
0000i@3oool5000001T0oooo0`00000J0?ooo`<00000703oool8000000@0oooo00030?ooo`D00000
7P3oool3000001/0oooo0`00000H0?ooo`D00000iP3oool6000001P0oooo0`00000K0?ooo`<00000
7P3oool5000000@0oooo00030?ooo`800000803oool3000001/0oooo0`00000J0?ooo`@00000i`3o
ool4000001X0oooo0`00000J0?ooo`@00000803oool2000000@0oooo000S0?ooo`D000006P3oool4
000001`0oooo0P00003W0?ooo`800000703oool4000001T0oooo1P00000T0?ooo`008P3oool50000
01X0oooo1@00003o0?ooob@0oooo1@00000I0?ooo`H000008`3oool002<0oooo1000000K0?ooo`<0
0000o`3ooolV0?ooo`<000006P3oool5000002@0oooo000T0?ooo`8000007P3oool00`000000oooo
0?ooo`3o0?ooob@0oooo00<000000?ooo`3oool06`3oool3000002D0oooo000U0?ooo`030000003o
ool0oooo0?l0ooooH@3oool00`000000oooo0?ooo`0U0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00o`3ooona0?ooo`00^P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool0^@3oool009`0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`2K0?ooo`00W03oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool06P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo09/0oooo002L0?oo
o`8000006`3oool3000001X0oooo0`00000J0?ooo`<000006`3oool2000009d0oooo002K0?ooo`<0
00006`3oool3000001X0oooo0`00000J0?ooo`<000006`3oool3000009`0oooo001n0?ooo`030000
003oool0oooo01X0oooo0`00000K0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`@00000
703oool00`000000oooo0?ooo`1m0?ooo`00FP3oool9000000P0oooo2@0000080?ooo`X000001`3o
ool:000000@0oooo3@0000080?ooo``000001@3oool:000000L0oooo2P0000070?ooo`X000001P3o
ool;000000P0oooo2@0000000`3oool0000000000002000000<0oooo2P0000070?ooo`X00000203o
ool9000000P0oooo1P00001O0?ooo`00FP3oool00`000000oooo0?ooo`0Q0?ooo`8000006`3oool4
000001T0oooo1@00000H0?ooo`D00000603oool5000001T0oooo1000000K0?ooo`8000007`3oool0
0`000000oooo0?ooo`1M0?ooo`00FP3oool00`000000oooo0?ooo`0Q0?ooo`8000006P3oool60000
01P0oooo1@00000H0?ooo`D00000603oool5000001P0oooo1P00000J0?ooo`8000007`3oool00`00
0000oooo0?ooo`1M0?ooo`00FP3oool00`000000oooo0?ooo`0Q0?ooo`8000006P3oool6000001P0
oooo1@00000H0?ooo`D00000603oool5000001P0oooo1P00000I0?ooo`<000007`3oool00`000000
oooo0?ooo`1M0?ooo`00O@3oool4000001T0oooo1000000L0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`@000006@3oool4000007h0oooo
001Q0?ooo`030000003oool0oooo01T0oooo1000000L0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool06P3oool4000001/0oooo00<000000?ooo`3oool0H03oool00640oooo00<00000
0?ooo`3oool06@3oool5000001/0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I
0?ooo`D000006P3oool200000680oooo001Q0?ooo`8000006P3oool5000001/0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`D000006P3oool200000680oooo000B0?ooo`<0
0000<03oool00`000000oooo0?ooo`0I0?ooo`8000006P3oool4000001`0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0J0?ooo`@000006@3oool3000001/0oooo00<000000?ooo`3o
ool0;P3oool3000001<0oooo000>0?ooo`L000004`3oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo01T0oooo0`00000L0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo0`00000J0?ooo`800000
703oool00`000000oooo0?ooo`0A0?ooo`L000003`3oool000/0oooo2P00000C0?ooo`8000006`3o
ool2000001X0oooo0`00000L0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0oooo1000000J0?ooo`8000006`3o
ool2000001<0oooo2P00000<0?ooo`003@3oool8000001<0oooo0P00000K0?ooo`8000006P3oool4
00000>D0oooo1000000I0?ooo`<000006`3oool2000001<0oooo2000000>0?ooo`00403oool50000
01<0oooo0`00000J0?ooo`<000004P3oool00`000000oooo0?ooo`040?ooo`@00000i03oool50000
0080oooo00<000000?ooo`3oool0503oool3000001X0oooo0`00000C0?ooo`D000004@3oool001<0
oooo0P00000C0?ooo`<000006P3oool300000180oooo00<000000?ooo`3oool0103oool500000><0
oooo1@0000020?ooo`030000003oool0oooo01<0oooo1000000I0?ooo`@000004`3oool2000001@0
oooo000Y0?ooo`<000006@3oool400000140oooo00<000000?ooo`3oool0103oool300000>L0oooo
0`0000020?ooo`030000003oool0oooo01<0oooo1000000I0?ooo`<00000:P3oool002T0oooo1000
000H0?ooo`@000004@3oool00`000000oooo0?ooo`3c0?ooo`030000003oool0oooo0180oooo1@00
000H0?ooo`@00000:P3oool002T0oooo1000000H0?ooo`D00000403oool00`000000oooo0?ooo`3c
0?ooo`030000003oool0oooo0180oooo1@00000H0?ooo`@00000:P3oool002T0oooo1@00000G0?oo
o`<000004P3oool00`000000oooo0?ooo`3c0?ooo`030000003oool0oooo01@0oooo0`00000G0?oo
o`D00000:P3oool002T0oooo0`00000^0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3o
ool0L@3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo0300oooo0`00000Z0?ooo`00
FP3oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
?03oool00`000000oooo0?ooo`1M0?ooo`00FP3oool00`000000oooo0?ooo`0o0?ooo`800000703o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06@3o
ool2000003h0oooo00<000000?ooo`3oool0G@3oool009`0oooo0`00000K0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`<00000W@3oool0
09/0oooo1000000J0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`<00000W@3oool009/0
oooo1000000J0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`<00000W@3oool009/0oooo
1000000J0?ooo`<000006P3oool3000001X0oooo0`00000I0?ooo`@00000W@3oool009X0oooo1P00
000I0?ooo`<000006P3oool3000001X0oooo0`00000I0?ooo`D00000W03oool009X0oooo1P00000H
0?ooo`D00000603oool5000001P0oooo1@00000H0?ooo`D00000W03oool009d0oooo00<000000?oo
o`3oool0603oool5000001P0oooo1@00000H0?ooo`D000006P3oool00`000000oooo0?ooo`2L0?oo
o`00W@3oool00`000000oooo0?ooo`0H0?ooo`D00000603oool5000001P0oooo1@00000J0?ooo`03
0000003oool0oooo09`0oooo001J0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo05d0oooo001J0?ooo`030000003o
ool0oooo01<0oooo0`00000Z0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0:03o
ool300000140oooo00<000000?ooo`3oool0G@3oool005X0oooo00<000000?ooo`3oool0503oool6
000002H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0T0?ooo`H000004P3oool0
0`000000oooo0?ooo`1M0?ooo`00FP3oool00`000000oooo0?ooo`0E0?ooo`P000008`3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo0240oooo2000000C0?ooo`030000003oool0oooo
05d0oooo001J0?ooo`030000003oool0oooo01H0oooo1P00000T0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool08@3oool7000001@0oooo00<000000?ooo`3oool0G@3oool005X0oooo
00<000000?ooo`3oool0603oool4000002@0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`0R0?ooo`@000005P3oool00`000000oooo0?ooo`1M0?ooo`00FP3oool010000000oooo0?oo
o`3oool2000001H0oooo0P0000000`3oool000000000000R0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool0803oool5000001H0oooo0P00001O0?ooo`004@3oool3000004H0oooo00<0
00000?ooo`3oool00P3oool4000001@0oooo00@000000?ooo`3oool0oooo0P00000P0?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool07P3oool2000000<0oooo00<000000?ooo`3oool0
4P3oool5000004X0oooo0`00000B0?ooo`003@3oool7000002l0oooo0P00000E0?ooo`030000003o
ool0oooo00<0oooo1P00000G0?ooo`8000007P3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo01`0oooo0P00000G0?ooo`H0000000<0oooo0000003oool0603oool2000002l0oooo1`00
000>0?ooo`002P3oool:000001<0oooo0`00000J0?ooo`@000004P3oool00`000000oooo0?ooo`04
0?ooo`L000005`3oool2000001`0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J
0?ooo`8000005`3oool700000080oooo00<000000?ooo`3oool0503oool4000001X0oooo0`00000C
0?ooo`X000002`3oool000d0oooo1`00000D0?ooo`H000005`3oool6000001H0oooo1P00003Q0?oo
o`H000005P3oool6000001L0oooo1P00000D0?ooo`L000003P3oool00140oooo0`00000E0?ooo`P0
00005@3oool7000001D0oooo1000003R0?ooo`D000005@3oool7000001D0oooo2000000E0?ooo`<0
00004P3oool002X0oooo1P00000F0?ooo`H000005`3oool300000240ooooX@3o000Q0?ooo`<00000
5`3oool6000001D0oooo1`00000[0?ooo`00;03oool4000001L0oooo1000000I0?ooo`030000003o
ool0oooo0><0oooo00<000000?ooo`3oool05P3oool5000001L0oooo1000000]0?ooo`00;@3oool2
000001T0oooo0`00003o0?oooaX0oooo0`00000H0?ooo`<00000;P3oool002h0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`3o0?oooaX0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0]0?ooo`00o`3ooona0?ooo`00FP3oool00`000000oooo0?ooo`3c0?ooo`030000
003oool0oooo05d0oooo001J0?ooo`030000003oool0oooo0?<0oooo00<000000?ooo`3oool0G@3o
ool005X0oooo00<000000?ooo`3oool0l`3oool00`000000oooo0?ooo`1M0?ooo`00FP3oool00`00
0000oooo0?ooo`3c0?ooo`030000003oool0oooo05d0oooo001J0?ooo`030000003oool0oooo0?<0
oooo00<000000?ooo`3oool0G@3oool005X0oooo00<000000?ooo`3oool0l`3oool00`000000oooo
0?ooo`1M0?ooo`00FP3oool00`000000oooo0?ooo`3c0?ooo`030000003oool0oooo05d0oooo001J
0?ooo`030000003oool0oooo0?<0oooo00<000000?ooo`3oool0G@3oool005X0oooo00<000000?oo
o`3oool0l`3oool00`000000oooo0?ooo`1M0?ooo`00FP3oool00`000000oooo0?ooo`3c0?ooo`03
0000003oool0oooo05d0oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oo
ok40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo00200?ooo`030000003oool0
oooo01T0oooo00<000000?ooo`3oool06@3oool5000001P0oooo1@00000H0?ooo`D000006`3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo07l0oooo000A0?ooo`800000A`3oool00`00
0000oooo0?ooo`090?ooo`8000005`3oool3000001X0oooo0`00000I0?ooo`D00000603oool50000
01P0oooo1@00000I0?ooo`<000006P3oool3000001L0oooo0P0000070?ooo`030000003oool0oooo
04T0oooo0P00000B0?ooo`003@3oool6000001T0oooo1@00000G0?ooo`D000003@3oool00`000000
oooo0?ooo`050?ooo`H000005P3oool4000001X0oooo1@00000G0?ooo`@000006@3oool4000001X0
oooo1000000G0?ooo`D000006@3oool5000001H0oooo1P0000030?ooo`030000003oool0oooo00l0
oooo1@00000G0?ooo`D000006@3oool6000000h0oooo000:0?ooo`T00000503oool:00000180oooo
2P00000=0?ooo`030000003oool0oooo0080oooo2@00000E0?ooo`H000006@3oool4000001P0oooo
1000000J0?ooo`<000006P3oool4000001P0oooo1000000I0?ooo`H000005@3oool:00000140oooo
2P00000B0?ooo`X00000503oool9000000/0oooo000<0?ooo`L000005P3oool7000001D0oooo1`00
000>0?ooo`030000003oool0oooo00@0oooo1`00000E0?ooo`L00000603oool4000001P0oooo0`00
000K0?ooo`<000006P3oool3000001T0oooo1000000H0?ooo`L000005@3oool700000080oooo00<0
00000?ooo`3oool03`3oool8000001@0oooo2000000F0?ooo`L000003@3oool00100oooo0`00000H
0?ooo`D000005`3oool5000000h0oooo00<000000?ooo`3oool0203oool3000001@0oooo1P00000I
0?ooo`@000006P3oool2000001/0oooo0`00000K0?ooo`8000006P3oool4000001T0oooo1P00000D
0?ooo`<000001P3oool00`000000oooo0?ooo`0@0?ooo`D000005`3oool5000001P0oooo0`00000A
0?ooo`00;P3oool2000001X0oooo0P00000>0?ooo`030000003oool0oooo01h0oooo1000000L0?oo
o`<000006`3oool2000001/0oooo0P00000L0?ooo`8000006`3oool3000001`0oooo1000000L0?oo
o`030000003oool0oooo0100oooo0P00000J0?ooo`800000;`3oool005X0oooo00<000000?ooo`3o
ool07@3oool2000001l0oooo0P00000L0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0J0?ooo`8000007`3oool2000001/0oooo00<000000?ooo`3o
ool0G@3oool005X0oooo00<000000?ooo`3oool0?P3oool2000001`0oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01X0oooo0P00000l0?ooo`030000
003oool0oooo05d0oooo001J0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0M@3o
ool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo05d0oooo003o0?oook40oooo003o0?oo
ok40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oo
ok40oooo003o0?oook40oooo001J0?ooo`030000003oool0oooo0?<0oooo00<000000?ooo`3oool0
G@3oool005X0oooo200000070?ooo`X000001`3oool:000000P0oooo2@0000080?ooo`X000001`3o
ool:000000L0oooo2P0000080?ooo`T00000203oool9000000P0oooo2P0000070?ooo`X00000203o
ool9000000P0oooo2@0000080?ooo`X000001`3oool8000005l0oooo003o0?oook40oooo003o0?oo
ok40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo00210?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool0L@3oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo0800oooo001W0?ooo`030000003oool0oooo01H0oooo0`00000J0?ooo`<0
0000603oool6000001L0oooo1@00000G0?ooo`D000006@3oool3000001X0oooo0P00000I0?ooo`03
0000003oool0oooo06H0oooo000^0?ooo`8000006`3oool00`000000oooo0?ooo`0G0?ooo`<00000
603oool4000001T0oooo1@00000F0?ooo`D00000603oool5000001L0oooo1@00000G0?ooo`D00000
603oool5000001L0oooo1000000I0?ooo`030000003oool0oooo01T0oooo0P00000_0?ooo`004@3o
ool2000001T0oooo1000000I0?ooo`<00000603oool5000001H0oooo1P00000G0?ooo`D000005`3o
ool5000001P0oooo1000000I0?ooo`@00000603oool5000001H0oooo1P00000G0?ooo`D00000603o
ool3000001T0oooo1000000I0?ooo`8000004P3oool000d0oooo1P00000F0?ooo`L000005`3oool5
000001L0oooo1P00000F0?ooo`D00000603oool4000001P0oooo1000000J0?ooo`<000006@3oool4
000001T0oooo1000000G0?ooo`H000005@3oool7000001H0oooo1P00000G0?ooo`L000005P3oool6
000000h0oooo000:0?ooo`T00000503oool9000001D0oooo2000000D0?ooo`T00000503oool40000
01X0oooo0`00000I0?ooo`@000006P3oool3000001X0oooo0`00000J0?ooo`<000006@3oool40000
01D0oooo2@00000D0?ooo`P00000503oool:000001@0oooo2@00000;0?ooo`00303oool7000001H0
oooo1`00000C0?ooo`X000004`3oool5000001T0oooo0`00000K0?ooo`<000006P3oool2000001/0
oooo0`00000J0?ooo`<000006P3oool3000001X0oooo1000000I0?ooo`D000004`3oool:00000180
oooo1`00000G0?ooo`L000003@3oool00100oooo0`00000J0?ooo`<00000C@3oool3000001/0oooo
0`00000K0?ooo`8000006`3oool2000001`0oooo0P00000K0?ooo`<000006`3oool2000004d0oooo
0`00000K0?ooo`<000004@3oool007d0oooo00<000000?ooo`3oool06`3oool2000001`0oooo00<0
00000?ooo`3oool06`3oool00`000000oooo0?ooo`0J0?ooo`800000703oool2000001d0oooo00<0
00000?ooo`3oool0O03oool009/0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`2J
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00IP3oool00`000000oooo0?ooo`0H0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool0L@3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool0I@3oool004X0oooo00<000000?ooo`3oool0603o
ool3000001T0oooo0P00000J0?ooo`<000006@3oool2000001X0oooo1@00000J0?ooo`8000006@3o
ool3000001X0oooo0P00000I0?ooo`<000006@3oool00`000000oooo0?ooo`1:0?ooo`00;`3oool0
0`000000oooo0?ooo`0G0?ooo`8000006P3oool3000001P0oooo1@00000H0?ooo`D000005`3oool5
000001L0oooo1@00000G0?ooo`D000005`3oool5000001P0oooo1@00000G0?ooo`@00000603oool3
000001X0oooo00<000000?ooo`3oool0;P3oool002d0oooo0`00000H0?ooo`@00000603oool50000
01L0oooo1P00000G0?ooo`@00000603oool4000001P0oooo1@00000G0?ooo`D00000603oool40000
01L0oooo1P00000G0?ooo`D000005`3oool4000001T0oooo0`00000^0?ooo`004@3oool3000001L0
oooo1@00000G0?ooo`D000005`3oool7000001H0oooo1@00000H0?ooo`@00000603oool4000001T0
oooo0`00000I0?ooo`@00000603oool4000001P0oooo1@00000F0?ooo`H000005P3oool7000001H0
oooo1P00000G0?ooo`<000004P3oool000d0oooo1`00000E0?ooo`P000005@3oool7000001H0oooo
1P00000F0?ooo`D00000603oool4000001T0oooo0`00000J0?ooo`<000006@3oool4000001T0oooo
1000000H0?ooo`D000005P3oool6000001D0oooo2000000E0?ooo`P000005@3oool7000000h0oooo
000:0?ooo`X000004`3oool:000001@0oooo1P00000G0?ooo`D00000603oool4000001T0oooo0`00
000J0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`<000006@3oool4000001P0oooo1@00
000G0?ooo`H00000503oool:000001<0oooo2P00000;0?ooo`003@3oool700000300oooo0`00000K
0?ooo`<000006P3oool2000001/0oooo0P00000K0?ooo`8000006`3oool3000001X0oooo0`00000K
0?ooo`8000006`3oool2000001X0oooo0`00000K0?ooo`<00000<03oool7000000h0oooo000A0?oo
o`<00000C@3oool2000001/0oooo0P00000L0?ooo`8000006`3oool2000001`0oooo00<000000?oo
o`3oool06P3oool2000001/0oooo0P00000L0?ooo`8000006`3oool2000004d0oooo0`00000B0?oo
o`00OP3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0J0?ooo`800000703oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo07d0oooo002i0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`2h0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00I@3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo0:/0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`1T
0?ooo`00B@3oool00`000000oooo0?ooo`0I0?ooo`8000006@3oool3000001T0oooo0P00000J0?oo
o`<00000>@3oool3000001X0oooo0P00000I0?ooo`<000006@3oool2000001/0oooo00<000000?oo
o`3oool0B03oool002h0oooo00<000000?ooo`3oool05`3oool3000001T0oooo1000000H0?ooo`D0
00005`3oool5000001L0oooo1P00000G0?ooo`D000005`3oool6000001L0oooo1@00000G0?ooo`D0
0000603oool4000001T0oooo0`00000H0?ooo`030000003oool0oooo02h0oooo000/0?ooo`<00000
6@3oool4000001P0oooo1@00000G0?ooo`@00000603oool4000001P0oooo1P00000G0?ooo`D00000
5`3oool6000001L0oooo1@00000H0?ooo`@000005`3oool5000001P0oooo1000000H0?ooo`<00000
;P3oool002/0oooo1@00000G0?ooo`H000005`3oool4000001P0oooo1000000H0?ooo`@000006@3o
ool4000001P0oooo1000000I0?ooo`@000006@3oool4000001P0oooo1000000H0?ooo`@000005`3o
ool6000001H0oooo1@00000]0?ooo`004@3oool3000001H0oooo1P00000F0?ooo`H000005`3oool4
000001P0oooo1000000I0?ooo`<000006P3oool4000001T0oooo0`00000I0?ooo`@000006@3oool4
000001T0oooo1000000H0?ooo`@000005`3oool6000001D0oooo1`00000F0?ooo`<000004P3oool0
00d0oooo1`00000D0?ooo`T00000503oool5000001T0oooo0`00000I0?ooo`<000006P3oool30000
01X0oooo0`00000J0?ooo`<000006@3oool4000001X0oooo0`00000J0?ooo`<000006@3oool30000
01T0oooo1@00000D0?ooo`T00000503oool7000000h0oooo000:0?ooo`X000004`3oool5000001T0
oooo0`00000K0?ooo`8000006P3oool2000001/0oooo0P00000K0?ooo`<000006P3oool3000001X0
oooo0`00000J0?ooo`<000006`3oool2000001X0oooo0P00000K0?ooo`<000006@3oool5000001<0
oooo2P00000;0?ooo`003@3oool700000300oooo0P00000L0?ooo`8000006`3oool2000001/0oooo
0P00000L0?ooo`8000006P3oool2000001/0oooo0P00000L0?ooo`8000006`3oool2000001/0oooo
0P00000L0?ooo`800000<03oool7000000h0oooo000A0?ooo`<00000CP3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool2000001`0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`1<0?ooo`<000004P3oool009`0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`2K0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00
o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00
o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00
o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00A`3oool00`000000
oooo0?ooo`3o0?oooah0oooo00<000000?ooo`3oool0AP3oool002d0oooo00<000000?ooo`3oool0
5`3oool3000001T0oooo0P00000J0?ooo`800000[@3oool2000001X0oooo0P00000I0?ooo`<00000
6@3oool00`000000oooo0?ooo`0/0?ooo`00;03oool3000001P0oooo1@00000G0?ooo`D000005`3o
ool5000001P0oooo1@00000G0?ooo`D00000603oool5000001P0oooo1@00000G0?ooo`D00000603o
ool5000001L0oooo1@00000G0?ooo`D00000603oool3000002d0oooo000/0?ooo`@000005P3oool5
000001P0oooo1000000H0?ooo`@000006@3oool4000001P0oooo1@00000H0?ooo`D00000603oool5
000001L0oooo1@00000H0?ooo`D00000603oool4000001P0oooo1@00000F0?ooo`@00000;@3oool0
02/0oooo1P00000E0?ooo`@000006@3oool4000001P0oooo1000000I0?ooo`@00000603oool50000
01P0oooo1@00000H0?ooo`D00000603oool4000001T0oooo1000000H0?ooo`@000006@3oool40000
01D0oooo1P00000/0?ooo`00:P3oool6000001H0oooo0`00000I0?ooo`@000006@3oool3000001X0
oooo0`00000J0?ooo`<000006P3oool3000001X0oooo0`00000I0?ooo`@000006@3oool4000001T0
oooo0`00000J0?ooo`<000005P3oool6000002/0oooo000B0?ooo`8000005@3oool5000001P0oooo
0`00000I0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo
0`00000J0?ooo`<000006P3oool3000001X0oooo0`00000I0?ooo`<00000603oool5000001@0oooo
0`00000C0?ooo`003P3oool6000001D0oooo0`00000I0?ooo`<000006P3oool2000001/0oooo0P00
000K0?ooo`8000006`3oool3000001X0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo0`00
000K0?ooo`8000006P3oool3000001T0oooo0`00000D0?ooo`L000003`3oool000/0oooo2@00000D
0?ooo`8000006`3oool2000001/0oooo0P00000K0?ooo`8000006`3oool2000001/0oooo0`00000J
0?ooo`<000006P3oool3000001/0oooo0P00000K0?ooo`8000006`3oool2000001/0oooo0P00000K
0?ooo`8000004`3oool:000000`0oooo000=0?ooo`L00000<@3oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?oo
o`8000006`3oool2000001`0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0^0?oo
o`L000003`3oool00140oooo0`00001[0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo06P0oooo0`00000C0?ooo`00
^P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
^@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo
/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo
/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo
/@3oool00?l0oooo/@3oool002H0oooo1P00000G0?ooo`H00000o`3ooolN0?ooo`H000005`3oool6
000002L0oooo000V0?ooo`H000005`3oool6000001L0oooo1P00000H0?ooo`D00000Z@3oool50000
01P0oooo1@00000H0?ooo`H000005`3oool6000002L0oooo000V0?ooo`D000006@3oool5000001L0
oooo1P00000H0?ooo`D00000603oool5000001P0oooo1@00000H0?ooo`D00000603oool5000001P0
oooo1@00000H0?ooo`D00000603oool5000001P0oooo1@00000I0?ooo`@00000:03oool002L0oooo
1000000I0?ooo`@000006@3oool4000001T0oooo1@00000H0?ooo`D00000603oool5000001P0oooo
1@00000H0?ooo`D00000603oool5000001P0oooo1@00000H0?ooo`@000006P3oool4000001T0oooo
1000000X0?ooo`009`3oool3000001X0oooo1000000I0?ooo`@000006P3oool3000001T0oooo1000
000I0?ooo`D00000603oool5000001P0oooo1@00000H0?ooo`@000006P3oool3000001X0oooo0`00
000J0?ooo`@000006@3oool4000002P0oooo000W0?ooo`<000006`3oool3000001T0oooo1000000J
0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo0`00000J
0?ooo`<000006P3oool3000001X0oooo0`00000K0?ooo`<00000:03oool002L0oooo0`00000K0?oo
o`<000006P3oool3000001X0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo0`00000J0?oo
o`<000006P3oool3000001X0oooo0`00000J0?ooo`<000006P3oool3000001/0oooo0P00000Y0?oo
o`00:03oool00`000000oooo0?ooo`0J0?ooo`8000006`3oool2000001/0oooo0`00000J0?ooo`<0
00006P3oool3000001X0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo0`00000J0?ooo`80
0000703oool2000001`0oooo00<000000?ooo`3oool09`3oool000/0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0I0?ooo`8000006`3oool3000001X0oooo0`00000J0?ooo`<00000
6P3oool2000001`0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0:0?ooo`00AP3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo04D0oooo00200?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo07l0oooo002j0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo0?ooo`2i0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 431}, {413.375, 0}} -> {-1.88239, 2.60708, 0.364618, \
0.364618}}]
}, Closed]],

Cell[BoxData[{
    \(\(Ex[x_, y_] = \(-D[f[x, y], x]\);\)\), "\[IndentingNewLine]", 
    \(\(Ey[x_, y_] = \(-D[f[x, y], y]\);\)\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Cap = \ \(\[CurlyEpsilon]\_0\/2\) \((Integrate[
              Ey[x, 70] - Ey[x, 50], {x, 20, 80}] + 
            Integrate[Ex[80, y] - Ex[20, y], {y, 50, 70}])\)\)], "Input"],

Cell[BoxData[
    \(3.6034873474991493`\ \[CurlyEpsilon]\_0\)], "Output"]
}, Closed]],

Cell[TextData[{
  "It is instructive to compare this result with the textbook analysis of an \
ideal capacitor without fringe fields.  We imagine that the electric field is \
uniform between the plates and vanishes outside the capacitor.  The voltage \
is then ",
  Cell[BoxData[
      \(TraditionalForm\`V = E\ d\)]],
  ", where ",
  Cell[BoxData[
      \(TraditionalForm\`d\)]],
  " is the separation between the plates, and the charge on each plate is ",
  Cell[BoxData[
      \(TraditionalForm\`Q = \(\[CurlyEpsilon]\_0\) E\ w\)]],
  ", where ",
  Cell[BoxData[
      \(TraditionalForm\`w\)]],
  " is the width of the plate.  Thus, the ideal capacitance is ",
  Cell[BoxData[
      \(TraditionalForm\`C = \[CurlyEpsilon]\_0\ w/d\)]],
  ", which takes the value ",
  Cell[BoxData[
      \(TraditionalForm\`2  \[CurlyEpsilon]\_0\)]],
  " for the present parameters.  Therefore, the actual capacitance is about \
1.8 times the ideal capacitance without edge effects and fringe fields.  With \
the ratio of width to separation being only 2, this is a relatively open \
geometry that permits significant fringe fields.  Instead of being \
distributed uniformly on inside surfaces only, the charge density is largest \
near the edges and is also present on the outside surfaces, such that the \
charged storage for given voltage is substantially larger than the simple \
textbook estimate.  This estimate only becomes reasonable when ",
  Cell[BoxData[
      \(TraditionalForm\`w/d\)]],
  " becomes very large.  Numerical methods are obviously required for many \
problems!  These issues are explored at greater depth in the exercises."
}], "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Comments about speed and efficiency", "Section"],

Cell["\<\
This method is best suited for computation of the potential and field \
relatively close to the electrodes where edge effects caused by artificial \
confinement to a rectangular grid at zero potential are small.  The edge \
effects can become important near the boundary of the computational grid.  \
Furthermore, if the artficial box compresses the equipotential surfaces, we \
may overestimate the fringe fields and, in the case of the parallel-plate \
capacitor, the capacitance of the system.  If we do require accurate fringe \
fields, we must expand the grid and endure longer computations.  If we are \
more interested in far field solutions, we should probably evaluate multipole \
amplitudes and employ a multipole expansion in that region instead.\
\>", "Text"],

Cell["\<\
Generating an accurate solution to even a simple two-dimensional pde \
obviously requires a considerable amount of cpu time; imagine how much longer \
it would take using the direct relaxation method instead of the more \
efficient simultaneous over-relaxation method.  For problems exhibiting \
symmetries in their boundary configuration and values, one can often reduce \
the amount of computation time by performing the evaluations within a \
suitable subregion only and using symmetry to map that portion of the \
solution onto the entire region.  Thus, for the parallel-plate capacitor we \
need only compute one quadrant to obtain the entire solution.  However, this \
strategy generally requires a specialized program for every problem.  One \
then needs to determine whether the human time needed to specialize the \
algorithm is more valuable than the computer time needed to execute the more \
general but less efficient program.  Unless the computation times become \
prohibitively long, I tend to value human resources over computer resources \
and recommend using well-tested general programs instead of custom programs \
for each problem.  After all, computers are getting faster all the time but \
humans are not!  Nor does the computer require as many coffee breaks.\
\>", "Text"],

Cell[TextData[{
  "Interpreted languages, like ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", are not necessarily the best choice for numerical algorithms that \
require many iterations \[LongDash] compiled languages, like Fortran or C, \
will generally perform such calculations much more rapidly.  Although the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " ",
  StyleBox["Compile", "InlineInput",
    FontWeight->"Bold"],
  " function can often improve the performance of numerical computations, the \
present implementation of the SOR algorithm is not particularly amenable to \
",
  StyleBox["Compile", "InlineInput",
    FontWeight->"Bold"],
  ".  Perhaps a better approach would be to write the relaxation algorithm in \
C and to link that program to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " using ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  ".  We could then use ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " for display but a faster compiled language for the cpu-intensive \
computations."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Electrodes outlined as polygons", "Section"],

Cell[TextData[{
  "Suppose that we wish to compute the electrostatic potential outside a \
system of electrodes, given as closed polygons represented by a list of \
points in which the first and last coincide.  To initialize the potential \
array, we need a function that determines a point is enclosed by an \
electrode.  The query ",
  StyleBox["InPolygonQ", "InlineInput"],
  " was developed in a separate notebook and will serve this purpose nicely."
}], "Text"],

Cell[BoxData[
    \(InPolygonQ[pt_, poly_] := 
      Module[{t, cpoly, n, k}, \[IndentingNewLine] (*\ 
          Prepare\ closed\ \(\(figure\)\(.\)\)\ *) \[IndentingNewLine]cpoly = 
          If[Last[poly] \[NotEqual] First[poly], Append[poly, First[poly]], 
            poly]; \[IndentingNewLine]n = 
          Length[cpoly] - 1; \[IndentingNewLine]If[\[IndentingNewLine] (*\ 
            Check\ whether\ point\ is\ on\ the\ \(\(boundary\)\(.\)\)\ \
*) \[IndentingNewLine]MemberQ[\[IndentingNewLine]Table[
              If[0 \[LessEqual] \((t /. 
                      Flatten[
                        Solve[pt \[Equal] 
                            t*cpoly\[LeftDoubleBracket]
                                  k\[RightDoubleBracket] + \((1 - t)\)*
                                cpoly\[LeftDoubleBracket]
                                  k + 1\[RightDoubleBracket], 
                          t]])\) \[LessEqual] 1, True, False, False], {k, 1, 
                n}], True], 
          True, \[IndentingNewLine] (*\ 
            Use\ winding\ number\ to\ test\ whether\ the\ point\ is\ \
\(\(inside\)\(.\)\)\ *) \[IndentingNewLine]Chop[\(1\/\(2  \[Pi]\)\) 
                Sum[Arg\ [\((cpoly\[LeftDoubleBracket]k + 1\
\[RightDoubleBracket] - pt)\) . {1. , \[ImaginaryI]}\/\((cpoly\
\[LeftDoubleBracket]k\[RightDoubleBracket] - pt)\) . {1. , \[ImaginaryI]}], \
{k, 1, n}]] \[NotEqual] 0]\[IndentingNewLine]]\)], "Input"],

Cell[CellGroupData[{

Cell["example: right triangle", "Subsection"],

Cell["First we construct the boundary.", "Text"],

Cell[BoxData[{
    \(\(Clear[boundary, \[Phi], \[Phi]0];\)\), "\[IndentingNewLine]", 
    \(\(boundary = Table[False, {i, 1, 100}, {j, 1, 100}];\)\), "\n", 
    \(\(\[Phi] = Table[0.0, {i, 1, 100}, {j, 1, 100}];\)\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(poly = {{41, 41}, {60, 41}, {41, 60}, {41, 
            41}};\)\), "\[IndentingNewLine]", 
    \(\(Show[Graphics[Line[poly]], 
        AspectRatio \[Rule] Automatic];\)\)}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.03133 0.0501253 -2.03133 0.0501253 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.02381 .02381 m
.97619 .02381 L
.02381 .97619 L
.02381 .02381 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{185.25, 185.25},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004E00015B000`400?l00000o`00003oo`3ooolF
0?ooo`00o`3ooolF0?ooo`00o`3ooolF0?ooo`00o`3ooolF0?ooo`00o`3ooolF0?ooo`00o`3ooolF
0?ooo`001P3ooooo000000X000001P3oool000H0oooo00<000000?ooo`3oool0o`3oool50?ooo`03
0000003oool0oooo00D0oooo00060?ooo`030000003oool0oooo0?l0oooo103oool00`000000oooo
0?ooo`060?ooo`001P3oool00`000000oooo0?ooo`3o0?ooo`<0oooo00<000000?ooo`3oool01`3o
ool000H0oooo00<000000?ooo`3oool0o`3oool20?ooo`030000003oool0oooo00P0oooo00060?oo
o`030000003oool0oooo0?l0oooo0@3oool00`000000oooo0?ooo`090?ooo`001P3oool00`000000
oooo0?ooo`3o0?ooo`030000003oool0oooo00X0oooo00060?ooo`030000003oool0oooo0?h0oooo
00<000000?ooo`3oool02`3oool000H0oooo00<000000?ooo`3oool0o@3oool00`000000oooo0?oo
o`0<0?ooo`001P3oool00`000000oooo0?ooo`3l0?ooo`030000003oool0oooo00d0oooo00060?oo
o`030000003oool0oooo0?/0oooo00<000000?ooo`3oool03P3oool000H0oooo00<000000?ooo`3o
ool0nP3oool00`000000oooo0?ooo`0?0?ooo`001P3oool00`000000oooo0?ooo`3i0?ooo`030000
003oool0oooo0100oooo00060?ooo`030000003oool0oooo0?P0oooo00<000000?ooo`3oool04@3o
ool000H0oooo00<000000?ooo`3oool0m`3oool00`000000oooo0?ooo`0B0?ooo`001P3oool00`00
0000oooo0?ooo`3f0?ooo`030000003oool0oooo01<0oooo00060?ooo`030000003oool0oooo0?D0
oooo00<000000?ooo`3oool0503oool000H0oooo00<000000?ooo`3oool0m03oool00`000000oooo
0?ooo`0E0?ooo`001P3oool00`000000oooo0?ooo`3c0?ooo`030000003oool0oooo01H0oooo0006
0?ooo`030000003oool0oooo0?80oooo00<000000?ooo`3oool05`3oool000H0oooo00<000000?oo
o`3oool0l@3oool00`000000oooo0?ooo`0H0?ooo`001P3oool00`000000oooo0?ooo`3`0?ooo`03
0000003oool0oooo01T0oooo00060?ooo`030000003oool0oooo0>l0oooo00<000000?ooo`3oool0
6P3oool000H0oooo00<000000?ooo`3oool0kP3oool00`000000oooo0?ooo`0K0?ooo`001P3oool0
0`000000oooo0?ooo`3]0?ooo`030000003oool0oooo01`0oooo00060?ooo`030000003oool0oooo
0>`0oooo00<000000?ooo`3oool07@3oool000H0oooo00<000000?ooo`3oool0j`3oool00`000000
oooo0?ooo`0N0?ooo`001P3oool00`000000oooo0?ooo`3Z0?ooo`030000003oool0oooo01l0oooo
00060?ooo`030000003oool0oooo0>T0oooo00<000000?ooo`3oool0803oool000H0oooo00<00000
0?ooo`3oool0j03oool00`000000oooo0?ooo`0Q0?ooo`001P3oool00`000000oooo0?ooo`3W0?oo
o`030000003oool0oooo0280oooo00060?ooo`030000003oool0oooo0>H0oooo00<000000?ooo`3o
ool08`3oool000H0oooo00<000000?ooo`3oool0i@3oool00`000000oooo0?ooo`0T0?ooo`001P3o
ool00`000000oooo0?ooo`3T0?ooo`030000003oool0oooo02D0oooo00060?ooo`030000003oool0
oooo0><0oooo00<000000?ooo`3oool09P3oool000H0oooo00<000000?ooo`3oool0hP3oool00`00
0000oooo0?ooo`0W0?ooo`001P3oool00`000000oooo0?ooo`3Q0?ooo`030000003oool0oooo02P0
oooo00060?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool0:@3oool000H0oooo00<0
00000?ooo`3oool0g`3oool00`000000oooo0?ooo`0Z0?ooo`001P3oool00`000000oooo0?ooo`3N
0?ooo`030000003oool0oooo02/0oooo00060?ooo`030000003oool0oooo0=d0oooo00<000000?oo
o`3oool0;03oool000H0oooo00<000000?ooo`3oool0g03oool00`000000oooo0?ooo`0]0?ooo`00
1P3oool00`000000oooo0?ooo`3K0?ooo`030000003oool0oooo02h0oooo00060?ooo`030000003o
ool0oooo0=X0oooo00<000000?ooo`3oool0;`3oool000H0oooo00<000000?ooo`3oool0f@3oool0
0`000000oooo0?ooo`0`0?ooo`001P3oool00`000000oooo0?ooo`3H0?ooo`030000003oool0oooo
0340oooo00060?ooo`030000003oool0oooo0=L0oooo00<000000?ooo`3oool0<P3oool000H0oooo
00<000000?ooo`3oool0eP3oool00`000000oooo0?ooo`0c0?ooo`001P3oool00`000000oooo0?oo
o`3E0?ooo`030000003oool0oooo03@0oooo00060?ooo`030000003oool0oooo0=@0oooo00<00000
0?ooo`3oool0=@3oool000H0oooo00<000000?ooo`3oool0d`3oool00`000000oooo0?ooo`0f0?oo
o`001P3oool00`000000oooo0?ooo`3B0?ooo`030000003oool0oooo03L0oooo00060?ooo`030000
003oool0oooo0=40oooo00<000000?ooo`3oool0>03oool000H0oooo00<000000?ooo`3oool0d03o
ool00`000000oooo0?ooo`0i0?ooo`001P3oool00`000000oooo0?ooo`3?0?ooo`030000003oool0
oooo03X0oooo00060?ooo`030000003oool0oooo0<h0oooo00<000000?ooo`3oool0>`3oool000H0
oooo00<000000?ooo`3oool0c@3oool00`000000oooo0?ooo`0l0?ooo`001P3oool00`000000oooo
0?ooo`3<0?ooo`030000003oool0oooo03d0oooo00060?ooo`030000003oool0oooo0</0oooo00<0
00000?ooo`3oool0?P3oool000H0oooo00<000000?ooo`3oool0bP3oool00`000000oooo0?ooo`0o
0?ooo`001P3oool00`000000oooo0?ooo`390?ooo`030000003oool0oooo0400oooo00060?ooo`03
0000003oool0oooo0<P0oooo00<000000?ooo`3oool0@@3oool000H0oooo00<000000?ooo`3oool0
a`3oool00`000000oooo0?ooo`120?ooo`001P3oool00`000000oooo0?ooo`360?ooo`030000003o
ool0oooo04<0oooo00060?ooo`030000003oool0oooo0<D0oooo00<000000?ooo`3oool0A03oool0
00H0oooo00<000000?ooo`3oool0a03oool00`000000oooo0?ooo`150?ooo`001P3oool00`000000
oooo0?ooo`330?ooo`030000003oool0oooo04H0oooo00060?ooo`030000003oool0oooo0<80oooo
00<000000?ooo`3oool0A`3oool000H0oooo00<000000?ooo`3oool0`@3oool00`000000oooo0?oo
o`180?ooo`001P3oool00`000000oooo0?ooo`300?ooo`030000003oool0oooo04T0oooo00060?oo
o`030000003oool0oooo0;l0oooo00<000000?ooo`3oool0BP3oool000H0oooo00<000000?ooo`3o
ool0_P3oool00`000000oooo0?ooo`1;0?ooo`001P3oool00`000000oooo0?ooo`2m0?ooo`030000
003oool0oooo04`0oooo00060?ooo`030000003oool0oooo0;`0oooo00<000000?ooo`3oool0C@3o
ool000H0oooo00<000000?ooo`3oool0^`3oool00`000000oooo0?ooo`1>0?ooo`001P3oool00`00
0000oooo0?ooo`2j0?ooo`030000003oool0oooo04l0oooo00060?ooo`030000003oool0oooo0;T0
oooo00<000000?ooo`3oool0D03oool000H0oooo00<000000?ooo`3oool0^03oool00`000000oooo
0?ooo`1A0?ooo`001P3oool00`000000oooo0?ooo`2g0?ooo`030000003oool0oooo0580oooo0006
0?ooo`030000003oool0oooo0;H0oooo00<000000?ooo`3oool0D`3oool000H0oooo00<000000?oo
o`3oool0]@3oool00`000000oooo0?ooo`1D0?ooo`001P3oool00`000000oooo0?ooo`2d0?ooo`03
0000003oool0oooo05D0oooo00060?ooo`030000003oool0oooo0;<0oooo00<000000?ooo`3oool0
EP3oool000H0oooo00<000000?ooo`3oool0/P3oool00`000000oooo0?ooo`1G0?ooo`001P3oool0
0`000000oooo0?ooo`2a0?ooo`030000003oool0oooo05P0oooo00060?ooo`030000003oool0oooo
0;00oooo00<000000?ooo`3oool0F@3oool000H0oooo00<000000?ooo`3oool0[`3oool00`000000
oooo0?ooo`1J0?ooo`001P3oool00`000000oooo0?ooo`2^0?ooo`030000003oool0oooo05/0oooo
00060?ooo`030000003oool0oooo0:d0oooo00<000000?ooo`3oool0G03oool000H0oooo00<00000
0?ooo`3oool0[03oool00`000000oooo0?ooo`1M0?ooo`001P3oool00`000000oooo0?ooo`2[0?oo
o`030000003oool0oooo05h0oooo00060?ooo`030000003oool0oooo0:X0oooo00<000000?ooo`3o
ool0G`3oool000H0oooo00<000000?ooo`3oool0Z@3oool00`000000oooo0?ooo`1P0?ooo`001P3o
ool00`000000oooo0?ooo`2X0?ooo`030000003oool0oooo0640oooo00060?ooo`030000003oool0
oooo0:L0oooo00<000000?ooo`3oool0HP3oool000H0oooo00<000000?ooo`3oool0YP3oool00`00
0000oooo0?ooo`1S0?ooo`001P3oool00`000000oooo0?ooo`2U0?ooo`030000003oool0oooo06@0
oooo00060?ooo`030000003oool0oooo0:@0oooo00<000000?ooo`3oool0I@3oool000H0oooo00<0
00000?ooo`3oool0X`3oool00`000000oooo0?ooo`1V0?ooo`001P3oool00`000000oooo0?ooo`2R
0?ooo`030000003oool0oooo06L0oooo00060?ooo`030000003oool0oooo0:40oooo00<000000?oo
o`3oool0J03oool000H0oooo00<000000?ooo`3oool0X03oool00`000000oooo0?ooo`1Y0?ooo`00
1P3oool00`000000oooo0?ooo`2O0?ooo`030000003oool0oooo06X0oooo00060?ooo`030000003o
ool0oooo09h0oooo00<000000?ooo`3oool0J`3oool000H0oooo00<000000?ooo`3oool0W@3oool0
0`000000oooo0?ooo`1/0?ooo`001P3oool00`000000oooo0?ooo`2L0?ooo`030000003oool0oooo
06d0oooo00060?ooo`030000003oool0oooo09/0oooo00<000000?ooo`3oool0KP3oool000H0oooo
00<000000?ooo`3oool0VP3oool00`000000oooo0?ooo`1_0?ooo`001P3oool00`000000oooo0?oo
o`2I0?ooo`030000003oool0oooo0700oooo00060?ooo`030000003oool0oooo09P0oooo00<00000
0?ooo`3oool0L@3oool000H0oooo00<000000?ooo`3oool0U`3oool00`000000oooo0?ooo`1b0?oo
o`001P3oool00`000000oooo0?ooo`2F0?ooo`030000003oool0oooo07<0oooo00060?ooo`030000
003oool0oooo09D0oooo00<000000?ooo`3oool0M03oool000H0oooo00<000000?ooo`3oool0U03o
ool00`000000oooo0?ooo`1e0?ooo`001P3oool00`000000oooo0?ooo`2C0?ooo`030000003oool0
oooo07H0oooo00060?ooo`030000003oool0oooo0980oooo00<000000?ooo`3oool0M`3oool000H0
oooo00<000000?ooo`3oool0T@3oool00`000000oooo0?ooo`1h0?ooo`001P3oool00`000000oooo
0?ooo`2@0?ooo`030000003oool0oooo07T0oooo00060?ooo`030000003oool0oooo08l0oooo00<0
00000?ooo`3oool0NP3oool000H0oooo00<000000?ooo`3oool0SP3oool00`000000oooo0?ooo`1k
0?ooo`001P3oool00`000000oooo0?ooo`2=0?ooo`030000003oool0oooo07`0oooo00060?ooo`03
0000003oool0oooo08`0oooo00<000000?ooo`3oool0O@3oool000H0oooo00<000000?ooo`3oool0
R`3oool00`000000oooo0?ooo`1n0?ooo`001P3oool00`000000oooo0?ooo`2:0?ooo`030000003o
ool0oooo07l0oooo00060?ooo`030000003oool0oooo08T0oooo00<000000?ooo`3oool0P03oool0
00H0oooo00<000000?ooo`3oool0R03oool00`000000oooo0?ooo`210?ooo`001P3oool00`000000
oooo0?ooo`270?ooo`030000003oool0oooo0880oooo00060?ooo`030000003oool0oooo08H0oooo
00<000000?ooo`3oool0P`3oool000H0oooo00<000000?ooo`3oool0Q@3oool00`000000oooo0?oo
o`240?ooo`001P3oool00`000000oooo0?ooo`240?ooo`030000003oool0oooo08D0oooo00060?oo
o`030000003oool0oooo08<0oooo00<000000?ooo`3oool0QP3oool000H0oooo00<000000?ooo`3o
ool0PP3oool00`000000oooo0?ooo`270?ooo`001P3oool00`000000oooo0?ooo`210?ooo`030000
003oool0oooo08P0oooo00060?ooo`030000003oool0oooo0800oooo00<000000?ooo`3oool0R@3o
ool000H0oooo00<000000?ooo`3oool0O`3oool00`000000oooo0?ooo`2:0?ooo`001P3oool00`00
0000oooo0?ooo`1n0?ooo`030000003oool0oooo08/0oooo00060?ooo`030000003oool0oooo07d0
oooo00<000000?ooo`3oool0S03oool000H0oooo00<000000?ooo`3oool0O03oool00`000000oooo
0?ooo`2=0?ooo`001P3oool00`000000oooo0?ooo`1k0?ooo`030000003oool0oooo08h0oooo0006
0?ooo`030000003oool0oooo07X0oooo00<000000?ooo`3oool0S`3oool000H0oooo00<000000?oo
o`3oool0N@3oool00`000000oooo0?ooo`2@0?ooo`001P3oool00`000000oooo0?ooo`1h0?ooo`03
0000003oool0oooo0940oooo00060?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool0
TP3oool000H0oooo00<000000?ooo`3oool0MP3oool00`000000oooo0?ooo`2C0?ooo`001P3oool0
0`000000oooo0?ooo`1e0?ooo`030000003oool0oooo09@0oooo00060?ooo`030000003oool0oooo
07@0oooo00<000000?ooo`3oool0U@3oool000H0oooo00<000000?ooo`3oool0L`3oool00`000000
oooo0?ooo`2F0?ooo`001P3oool00`000000oooo0?ooo`1b0?ooo`030000003oool0oooo09L0oooo
00060?ooo`030000003oool0oooo0740oooo00<000000?ooo`3oool0V03oool000H0oooo00<00000
0?ooo`3oool0L03oool00`000000oooo0?ooo`2I0?ooo`001P3oool00`000000oooo0?ooo`1_0?oo
o`030000003oool0oooo09X0oooo00060?ooo`030000003oool0oooo06h0oooo00<000000?ooo`3o
ool0V`3oool000H0oooo00<000000?ooo`3oool0K@3oool00`000000oooo0?ooo`2L0?ooo`001P3o
ool00`000000oooo0?ooo`1/0?ooo`030000003oool0oooo09d0oooo00060?ooo`030000003oool0
oooo06/0oooo00<000000?ooo`3oool0WP3oool000H0oooo00<000000?ooo`3oool0JP3oool00`00
0000oooo0?ooo`2O0?ooo`001P3oool00`000000oooo0?ooo`1Y0?ooo`030000003oool0oooo0:00
oooo00060?ooo`030000003oool0oooo06P0oooo00<000000?ooo`3oool0X@3oool000H0oooo00<0
00000?ooo`3oool0I`3oool00`000000oooo0?ooo`2R0?ooo`001P3oool00`000000oooo0?ooo`1V
0?ooo`030000003oool0oooo0:<0oooo00060?ooo`030000003oool0oooo06D0oooo00<000000?oo
o`3oool0Y03oool000H0oooo00<000000?ooo`3oool0I03oool00`000000oooo0?ooo`2U0?ooo`00
1P3oool00`000000oooo0?ooo`1S0?ooo`030000003oool0oooo0:H0oooo00060?ooo`030000003o
ool0oooo0680oooo00<000000?ooo`3oool0Y`3oool000H0oooo00<000000?ooo`3oool0H@3oool0
0`000000oooo0?ooo`2X0?ooo`001P3oool00`000000oooo0?ooo`1P0?ooo`030000003oool0oooo
0:T0oooo00060?ooo`030000003oool0oooo05l0oooo00<000000?ooo`3oool0ZP3oool000H0oooo
00<000000?ooo`3oool0GP3oool00`000000oooo0?ooo`2[0?ooo`001P3oool00`000000oooo0?oo
o`1M0?ooo`030000003oool0oooo0:`0oooo00060?ooo`030000003oool0oooo05`0oooo00<00000
0?ooo`3oool0[@3oool000H0oooo00<000000?ooo`3oool0F`3oool00`000000oooo0?ooo`2^0?oo
o`001P3oool00`000000oooo0?ooo`1J0?ooo`030000003oool0oooo0:l0oooo00060?ooo`030000
003oool0oooo05T0oooo00<000000?ooo`3oool0/03oool000H0oooo00<000000?ooo`3oool0F03o
ool00`000000oooo0?ooo`2a0?ooo`001P3oool00`000000oooo0?ooo`1G0?ooo`030000003oool0
oooo0;80oooo00060?ooo`030000003oool0oooo05H0oooo00<000000?ooo`3oool0/`3oool000H0
oooo00<000000?ooo`3oool0E@3oool00`000000oooo0?ooo`2d0?ooo`001P3oool00`000000oooo
0?ooo`1D0?ooo`030000003oool0oooo0;D0oooo00060?ooo`030000003oool0oooo05<0oooo00<0
00000?ooo`3oool0]P3oool000H0oooo00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`2g
0?ooo`001P3oool00`000000oooo0?ooo`1A0?ooo`030000003oool0oooo0;P0oooo00060?ooo`03
0000003oool0oooo0500oooo00<000000?ooo`3oool0^@3oool000H0oooo00<000000?ooo`3oool0
C`3oool00`000000oooo0?ooo`2j0?ooo`001P3oool00`000000oooo0?ooo`1>0?ooo`030000003o
ool0oooo0;/0oooo00060?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0_03oool0
00H0oooo00<000000?ooo`3oool0C03oool00`000000oooo0?ooo`2m0?ooo`001P3oool00`000000
oooo0?ooo`1;0?ooo`030000003oool0oooo0;h0oooo00060?ooo`030000003oool0oooo04X0oooo
00<000000?ooo`3oool0_`3oool000H0oooo00<000000?ooo`3oool0B@3oool00`000000oooo0?oo
o`300?ooo`001P3oool00`000000oooo0?ooo`180?ooo`030000003oool0oooo0<40oooo00060?oo
o`030000003oool0oooo04L0oooo00<000000?ooo`3oool0`P3oool000H0oooo00<000000?ooo`3o
ool0AP3oool00`000000oooo0?ooo`330?ooo`001P3oool00`000000oooo0?ooo`150?ooo`030000
003oool0oooo0<@0oooo00060?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool0a@3o
ool000H0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`360?ooo`001P3oool00`00
0000oooo0?ooo`120?ooo`030000003oool0oooo0<L0oooo00060?ooo`030000003oool0oooo0440
oooo00<000000?ooo`3oool0b03oool000H0oooo00<000000?ooo`3oool0@03oool00`000000oooo
0?ooo`390?ooo`001P3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo0<X0oooo0006
0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0b`3oool000H0oooo00<000000?oo
o`3oool0?@3oool00`000000oooo0?ooo`3<0?ooo`001P3oool00`000000oooo0?ooo`0l0?ooo`03
0000003oool0oooo0<d0oooo00060?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0
cP3oool000H0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`3?0?ooo`001P3oool0
0`000000oooo0?ooo`0i0?ooo`030000003oool0oooo0=00oooo00060?ooo`030000003oool0oooo
03P0oooo00<000000?ooo`3oool0d@3oool000H0oooo00<000000?ooo`3oool0=`3oool00`000000
oooo0?ooo`3B0?ooo`001P3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo0=<0oooo
00060?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool0e03oool000H0oooo00<00000
0?ooo`3oool0=03oool00`000000oooo0?ooo`3E0?ooo`001P3oool00`000000oooo0?ooo`0c0?oo
o`030000003oool0oooo0=H0oooo00060?ooo`030000003oool0oooo0380oooo00<000000?ooo`3o
ool0e`3oool000H0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`3H0?ooo`001P3o
ool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo0=T0oooo00060?ooo`030000003oool0
oooo02l0oooo00<000000?ooo`3oool0fP3oool000H0oooo00<000000?ooo`3oool0;P3oool00`00
0000oooo0?ooo`3K0?ooo`001P3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo0=`0
oooo00060?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0g@3oool000H0oooo00<0
00000?ooo`3oool0:`3oool00`000000oooo0?ooo`3N0?ooo`001P3oool00`000000oooo0?ooo`0Z
0?ooo`030000003oool0oooo0=l0oooo00060?ooo`030000003oool0oooo02T0oooo00<000000?oo
o`3oool0h03oool000H0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`3Q0?ooo`00
1P3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo0>80oooo00060?ooo`030000003o
ool0oooo02H0oooo00<000000?ooo`3oool0h`3oool000H0oooo00<000000?ooo`3oool09@3oool0
0`000000oooo0?ooo`3T0?ooo`001P3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo
0>D0oooo00060?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0iP3oool000H0oooo
00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`3W0?ooo`001P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo0>P0oooo00060?ooo`030000003oool0oooo0200oooo00<00000
0?ooo`3oool0j@3oool000H0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`3Z0?oo
o`001P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0>/0oooo00060?ooo`030000
003oool0oooo01d0oooo00<000000?ooo`3oool0k03oool000H0oooo00<000000?ooo`3oool0703o
ool00`000000oooo0?ooo`3]0?ooo`001P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0
oooo0>h0oooo00060?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0k`3oool000H0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`3`0?ooo`001P3oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0oooo0?40oooo00060?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool0lP3oool000H0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`3c
0?ooo`001P3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0?@0oooo00060?ooo`03
0000003oool0oooo01@0oooo00<000000?ooo`3oool0m@3oool000H0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`3f0?ooo`001P3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo0?L0oooo00060?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0n03oool0
00H0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3i0?ooo`001P3oool00`000000
oooo0?ooo`0?0?ooo`030000003oool0oooo0?X0oooo00060?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool0n`3oool000H0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`3l0?ooo`001P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0?d0oooo00060?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool0oP3oool000H0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`3o0?ooo`001P3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo0?l0oooo0@3oool000H0oooo00<000000?ooo`3oool0203oool00`000000oooo0?oo
o`3o0?ooo`80oooo00060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0o`3oool3
0?ooo`001P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0?l0oooo103oool000H0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`D0oooo00060?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool0o`3oool60?ooo`001P3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo0?l0oooo1`3oool000H0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`3o0?ooo`P0oooo00060?ooo`050000003oool0oooo0?ooo`000000o`3oool;
0?ooo`001P3oool010000000oooo0?ooo`00003o0?ooo``0oooo00060?ooo`030000003oool00000
0?l0oooo3@3oool000H0oooo0P00003o0?ooo`h0oooo00060?ooo`030000003oool0oooo0?l0oooo
3@3oool00?l0oooo5P3oool00?l0oooo5P3oool00?l0oooo5P3oool00?l0oooo5P3oool00?l0oooo
5P3oool00?l0oooo5P3oool00001\
\>"],
  ImageRangeCache->{{{0, 276.875}, {276.875, 0}} -> {40.5249, 40.5249, \
0.108082, 0.108082}}]
}, Closed]],

Cell["\<\
Then we evaluate and display the potential and field using standard steps.\
\>", "Text"],

Cell[BoxData[{
    \(\(Do[
        boundary\[LeftDoubleBracket]i, j\[RightDoubleBracket] = 
          InPolygonQ[{i, j}, poly], {i, 1, 100}, {j, 1, 100}];\)\), "\n", 
    \(\(Do[\[Phi]\[LeftDoubleBracket]i, j\[RightDoubleBracket] = 
          If[boundary\[LeftDoubleBracket]i, j\[RightDoubleBracket], 1. , 
            0. ], {i, 1, 100}, {j, 1, 100}];\)\), "\n", 
    \(\(\[Phi]0 = \[Phi];\)\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\[Phi] = relax[\[Phi]0, boundary, 1000, 10\^\(-4\)];\)\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("number of iterations = "\[InvisibleSpace]137\),
      SequenceForm[ "number of iterations = ", 137],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("residual = "\[InvisibleSpace]0.00009966974430927659`\
\),
      SequenceForm[ "residual = ", .00009966974430927659],
      Editable->False]], "Print"]
}, Closed]],

Cell[BoxData[
    \(\(f = ListInterpolation[\[Phi]];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ContourPlot[f[x, y], {x, 20, 80}, {y, 20, 80}, 
        PlotPoints \[Rule] 100, AspectRatio \[Rule] Automatic, 
        ContourShading \[Rule] False, 
        Contours \[Rule] Range[0.35, 0.95, 0.1], Frame \[Rule] False, 
        Epilog \[Rule] {GrayLevel[0.75], Polygon[poly]}];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% ContourGraphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.301282 0.0160256 -0.301282 0.0160256 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.40773 .33948 m
.41744 .33881 L
.42716 .33825 L
.43687 .33778 L
.44658 .33739 L
.45629 .33708 L
.46601 .33683 L
.47572 .33665 L
.48543 .33653 L
.49514 .33647 L
.50486 .33647 L
.51457 .33654 L
.52428 .33665 L
.53399 .33682 L
.54371 .33706 L
.55342 .33736 L
.56313 .33772 L
.57284 .33816 L
.58256 .33868 L
.59227 .33929 L
.59873 .33974 L
.60198 .33994 L
.61169 .34061 L
.62141 .34138 L
.63112 .34237 L
.64083 .34362 L
.65054 .34422 L
.66026 .34553 L
.66398 .34946 L
.6673 .35917 L
.66253 .36888 L
.66026 .37163 L
.65538 .37859 L
.65054 .38469 L
.64762 .38831 L
.64083 .39738 L
.64037 .39802 L
.63209 .40773 L
.63112 .40893 L
.62426 .41744 L
.62141 .42073 L
.61572 .42716 L
.61169 .43187 L
.60725 .43687 L
.60198 .44276 L
.5987 .44658 L
.59227 .45369 L
.58977 .45629 L
.58256 .46419 L
.58094 .46601 L
Mistroke
.57284 .47462 L
.5718 .47572 L
.56313 .48495 L
.56267 .48543 L
.55342 .49505 L
.55333 .49514 L
.54392 .50486 L
.54371 .50507 L
.53441 .51457 L
.53399 .51499 L
.52471 .52428 L
.52428 .52471 L
.51499 .53399 L
.51457 .53441 L
.50507 .54371 L
.50486 .54392 L
.49514 .55333 L
.49505 .55342 L
.48543 .56267 L
.48495 .56313 L
.47572 .5718 L
.47462 .57284 L
.46601 .58094 L
.46419 .58256 L
.45629 .58977 L
.45369 .59227 L
.44658 .5987 L
.44276 .60198 L
.43687 .60725 L
.43187 .61169 L
.42716 .61572 L
.42073 .62141 L
.41744 .62426 L
.40893 .63112 L
.40773 .63209 L
.39802 .64037 L
.39738 .64083 L
.38831 .64762 L
.38469 .65054 L
.37859 .65538 L
.37163 .66026 L
.36888 .66253 L
.35917 .6673 L
.34946 .66398 L
.34553 .66026 L
.34422 .65054 L
.34362 .64083 L
.34237 .63112 L
.34138 .62141 L
.34061 .61169 L
Mistroke
.33994 .60198 L
.33974 .59873 L
.33929 .59227 L
.33868 .58256 L
.33816 .57284 L
.33772 .56313 L
.33736 .55342 L
.33706 .54371 L
.33682 .53399 L
.33665 .52428 L
.33654 .51457 L
.33647 .50486 L
.33647 .49514 L
.33653 .48543 L
.33665 .47572 L
.33683 .46601 L
.33708 .45629 L
.33739 .44658 L
.33778 .43687 L
.33825 .42716 L
.33881 .41744 L
.33948 .40773 L
.33974 .40434 L
.34018 .39802 L
.34097 .38831 L
.34191 .37859 L
.34305 .36888 L
.34454 .35917 L
.34946 .35034 L
.35034 .34946 L
.35917 .34454 L
.36888 .34305 L
.37859 .34191 L
.38831 .34097 L
.39802 .34018 L
.40434 .33974 L
.40773 .33948 L
Mfstroke
.43687 .30025 m
.44658 .29923 L
.45629 .2984 L
.46601 .29775 L
.47572 .29727 L
.48543 .29696 L
.49514 .29681 L
.50486 .29682 L
.51457 .29698 L
.52428 .2973 L
.53399 .29779 L
.54371 .29843 L
.55342 .29924 L
.56313 .30025 L
.56862 .30089 L
.57284 .30144 L
.58256 .30284 L
.59227 .30448 L
.60198 .30636 L
.61169 .30854 L
.61977 .31061 L
.62141 .31106 L
.63112 .314 L
.64083 .31746 L
.6477 .32032 L
.65054 .32165 L
.66026 .32692 L
.66474 .33003 L
.66997 .33488 L
.67425 .33974 L
.67968 .3487 L
.67994 .34946 L
.68238 .35917 L
.68225 .36888 L
.68084 .37859 L
.67968 .38358 L
.67831 .38831 L
.67503 .39802 L
.67102 .40773 L
.66997 .41005 L
.66643 .41744 L
.66132 .42716 L
.66026 .42907 L
.65575 .43687 L
.65054 .44538 L
.64979 .44658 L
.64346 .45629 L
.64083 .46017 L
.63678 .46601 L
.63112 .47389 L
Mistroke
.62978 .47572 L
.62247 .48543 L
.62141 .48681 L
.61485 .49514 L
.61169 .49907 L
.60695 .50486 L
.60198 .51078 L
.59874 .51457 L
.59227 .52201 L
.59026 .52428 L
.58256 .53281 L
.58147 .53399 L
.57284 .54322 L
.57239 .54371 L
.56313 .55328 L
.56299 .55342 L
.55342 .56299 L
.55328 .56313 L
.54371 .57239 L
.54322 .57284 L
.53399 .58147 L
.53281 .58256 L
.52428 .59026 L
.52201 .59227 L
.51457 .59874 L
.51078 .60198 L
.50486 .60695 L
.49907 .61169 L
.49514 .61485 L
.48681 .62141 L
.48543 .62247 L
.47572 .62978 L
.47389 .63112 L
.46601 .63678 L
.46017 .64083 L
.45629 .64346 L
.44658 .64979 L
.44538 .65054 L
.43687 .65575 L
.42907 .66026 L
.42716 .66132 L
.41744 .66643 L
.41005 .66997 L
.40773 .67102 L
.39802 .67503 L
.38831 .67831 L
.38358 .67968 L
.37859 .68084 L
.36888 .68225 L
.35917 .68238 L
Mistroke
.34946 .67994 L
.3487 .67968 L
.33974 .67425 L
.33488 .66997 L
.33003 .66474 L
.32692 .66026 L
.32165 .65054 L
.32032 .6477 L
.31746 .64083 L
.314 .63112 L
.31106 .62141 L
.31061 .61977 L
.30854 .61169 L
.30636 .60198 L
.30448 .59227 L
.30284 .58256 L
.30144 .57284 L
.30089 .56862 L
.30025 .56313 L
.29924 .55342 L
.29843 .54371 L
.29779 .53399 L
.2973 .52428 L
.29698 .51457 L
.29682 .50486 L
.29681 .49514 L
.29696 .48543 L
.29727 .47572 L
.29775 .46601 L
.2984 .45629 L
.29923 .44658 L
.30025 .43687 L
.30089 .43155 L
.30147 .42716 L
.30292 .41744 L
.30464 .40773 L
.30666 .39802 L
.30902 .38831 L
.31061 .38255 L
.3118 .37859 L
.31513 .36888 L
.31918 .35917 L
.32032 .35682 L
.3245 .34946 L
.33003 .34174 L
.33177 .33974 L
.33974 .33177 L
.34174 .33003 L
.34946 .3245 L
.35682 .32032 L
Mistroke
.35917 .31918 L
.36888 .31513 L
.37859 .3118 L
.38255 .31061 L
.38831 .30902 L
.39802 .30666 L
.40773 .30464 L
.41744 .30292 L
.42716 .30147 L
.43155 .30089 L
.43687 .30025 L
Mfstroke
.42716 .26053 m
.43687 .25893 L
.44658 .25758 L
.45629 .25649 L
.46601 .25563 L
.47572 .25499 L
.48543 .25458 L
.49514 .2544 L
.50486 .25442 L
.51457 .25467 L
.52428 .25512 L
.53399 .2558 L
.54371 .2567 L
.55342 .25783 L
.56313 .25922 L
.57284 .26086 L
.57905 .26204 L
.58256 .26277 L
.59227 .26496 L
.60198 .26747 L
.61169 .27034 L
.61609 .27176 L
.62141 .27359 L
.63112 .27729 L
.64071 .28147 L
.64083 .28152 L
.65054 .28639 L
.6589 .29118 L
.66026 .29203 L
.66997 .29879 L
.67266 .30089 L
.67968 .30699 L
.68335 .31061 L
.68939 .31742 L
.69168 .32032 L
.69818 .33003 L
.69911 .33164 L
.7031 .33974 L
.70667 .34946 L
.70882 .35809 L
.70899 .35917 L
.71019 .36888 L
.71052 .37859 L
.7101 .38831 L
.70904 .39802 L
.70882 .39961 L
.70738 .40773 L
.70518 .41744 L
.70248 .42716 L
.69931 .43687 L
Mistroke
.69911 .43745 L
.6957 .44658 L
.69167 .45629 L
.68939 .4614 L
.68724 .46601 L
.68243 .47572 L
.67968 .48095 L
.67724 .48543 L
.67168 .49514 L
.66997 .49801 L
.66575 .50486 L
.66026 .51338 L
.65947 .51457 L
.65283 .52428 L
.65054 .5275 L
.64583 .53399 L
.64083 .54063 L
.63846 .54371 L
.63112 .55292 L
.63071 .55342 L
.62258 .56313 L
.62141 .5645 L
.61405 .57284 L
.61169 .57544 L
.60509 .58256 L
.60198 .58582 L
.59569 .59227 L
.59227 .59569 L
.58582 .60198 L
.58256 .60509 L
.57544 .61169 L
.57284 .61405 L
.5645 .62141 L
.56313 .62258 L
.55342 .63071 L
.55292 .63112 L
.54371 .63846 L
.54063 .64083 L
.53399 .64583 L
.5275 .65054 L
.52428 .65283 L
.51457 .65947 L
.51338 .66026 L
.50486 .66575 L
.49801 .66997 L
.49514 .67168 L
.48543 .67724 L
.48095 .67968 L
.47572 .68243 L
.46601 .68724 L
Mistroke
.4614 .68939 L
.45629 .69167 L
.44658 .6957 L
.43745 .69911 L
.43687 .69931 L
.42716 .70248 L
.41744 .70518 L
.40773 .70738 L
.39961 .70882 L
.39802 .70904 L
.38831 .7101 L
.37859 .71052 L
.36888 .71019 L
.35917 .70899 L
.35809 .70882 L
.34946 .70667 L
.33974 .7031 L
.33164 .69911 L
.33003 .69818 L
.32032 .69168 L
.31742 .68939 L
.31061 .68335 L
.30699 .67968 L
.30089 .67266 L
.29879 .66997 L
.29203 .66026 L
.29118 .6589 L
.28639 .65054 L
.28152 .64083 L
.28147 .64071 L
.27729 .63112 L
.27359 .62141 L
.27176 .61609 L
.27034 .61169 L
.26747 .60198 L
.26496 .59227 L
.26277 .58256 L
.26204 .57905 L
.26086 .57284 L
.25922 .56313 L
.25783 .55342 L
.2567 .54371 L
.2558 .53399 L
.25512 .52428 L
.25467 .51457 L
.25442 .50486 L
.2544 .49514 L
.25458 .48543 L
.25499 .47572 L
.25563 .46601 L
Mistroke
.25649 .45629 L
.25758 .44658 L
.25893 .43687 L
.26053 .42716 L
.26204 .41923 L
.26241 .41744 L
.26458 .40773 L
.26709 .39802 L
.26996 .38831 L
.27176 .38283 L
.27324 .37859 L
.27699 .36888 L
.28128 .35917 L
.28147 .35876 L
.28621 .34946 L
.29118 .34094 L
.29193 .33974 L
.29862 .33003 L
.30089 .32705 L
.30648 .32032 L
.31061 .31581 L
.31581 .31061 L
.32032 .30648 L
.32705 .30089 L
.33003 .29862 L
.33974 .29193 L
.34094 .29118 L
.34946 .28621 L
.35876 .28147 L
.35917 .28128 L
.36888 .27699 L
.37859 .27324 L
.38283 .27176 L
.38831 .26996 L
.39802 .26709 L
.40773 .26458 L
.41744 .26241 L
.41923 .26204 L
.42716 .26053 L
Mfstroke
.43687 .21219 m
.44658 .21079 L
.45629 .20964 L
.46601 .20873 L
.47572 .20807 L
.48543 .20765 L
.49514 .20747 L
.50486 .20752 L
.51457 .20781 L
.52428 .20831 L
.53399 .20906 L
.54371 .21008 L
.55342 .21134 L
.56313 .21285 L
.56676 .21348 L
.57284 .21464 L
.58256 .21671 L
.59227 .21906 L
.60198 .22172 L
.60688 .22319 L
.61169 .22473 L
.62141 .22808 L
.63112 .23184 L
.63369 .23291 L
.64083 .23603 L
.65054 .24071 L
.65421 .24262 L
.66026 .24594 L
.66997 .25181 L
.67079 .25233 L
.67968 .25841 L
.68456 .26204 L
.68939 .26589 L
.69619 .27176 L
.69911 .27446 L
.70609 .28147 L
.70882 .28444 L
.71455 .29118 L
.71853 .29631 L
.72181 .30089 L
.728 .31061 L
.72824 .31103 L
.73325 .32032 L
.73764 .33003 L
.73796 .3308 L
.74125 .33974 L
.74413 .34946 L
.74635 .35917 L
.74767 .36675 L
.74796 .36888 L
Mistroke
.749 .37859 L
.74952 .38831 L
.74955 .39802 L
.74912 .40773 L
.74826 .41744 L
.74767 .42238 L
.74698 .42716 L
.7453 .43687 L
.74325 .44658 L
.74082 .45629 L
.73803 .46601 L
.73796 .46626 L
.73489 .47572 L
.7314 .48543 L
.72824 .49347 L
.72755 .49514 L
.72337 .50486 L
.71884 .51457 L
.71853 .5152 L
.71397 .52428 L
.70882 .53387 L
.70875 .53399 L
.70319 .54371 L
.69911 .55046 L
.69726 .55342 L
.69097 .56313 L
.68939 .56548 L
.68431 .57284 L
.67968 .57927 L
.67725 .58256 L
.66997 .59204 L
.66979 .59227 L
.66191 .60198 L
.66026 .60395 L
.65359 .61169 L
.65054 .61511 L
.64478 .62141 L
.64083 .6256 L
.63547 .63112 L
.63112 .63547 L
.6256 .64083 L
.62141 .64478 L
.61511 .65054 L
.61169 .65359 L
.60395 .66026 L
.60198 .66191 L
.59227 .66979 L
.59204 .66997 L
.58256 .67725 L
.57927 .67968 L
Mistroke
.57284 .68431 L
.56548 .68939 L
.56313 .69097 L
.55342 .69726 L
.55046 .69911 L
.54371 .70319 L
.53399 .70875 L
.53387 .70882 L
.52428 .71397 L
.5152 .71853 L
.51457 .71884 L
.50486 .72337 L
.49514 .72755 L
.49347 .72824 L
.48543 .7314 L
.47572 .73489 L
.46626 .73796 L
.46601 .73803 L
.45629 .74082 L
.44658 .74325 L
.43687 .7453 L
.42716 .74698 L
.42238 .74767 L
.41744 .74826 L
.40773 .74912 L
.39802 .74955 L
.38831 .74952 L
.37859 .749 L
.36888 .74796 L
.36675 .74767 L
.35917 .74635 L
.34946 .74413 L
.33974 .74125 L
.3308 .73796 L
.33003 .73764 L
.32032 .73325 L
.31103 .72824 L
.31061 .728 L
.30089 .72181 L
.29631 .71853 L
.29118 .71455 L
.28444 .70882 L
.28147 .70609 L
.27446 .69911 L
.27176 .69619 L
.26589 .68939 L
.26204 .68456 L
.25841 .67968 L
.25233 .67079 L
.25181 .66997 L
Mistroke
.24594 .66026 L
.24262 .65421 L
.24071 .65054 L
.23603 .64083 L
.23291 .63369 L
.23184 .63112 L
.22808 .62141 L
.22473 .61169 L
.22319 .60688 L
.22172 .60198 L
.21906 .59227 L
.21671 .58256 L
.21464 .57284 L
.21348 .56676 L
.21285 .56313 L
.21134 .55342 L
.21008 .54371 L
.20906 .53399 L
.20831 .52428 L
.20781 .51457 L
.20752 .50486 L
.20747 .49514 L
.20765 .48543 L
.20807 .47572 L
.20873 .46601 L
.20964 .45629 L
.21079 .44658 L
.21219 .43687 L
.21348 .42922 L
.21386 .42716 L
.21581 .41744 L
.21803 .40773 L
.22056 .39802 L
.22319 .38906 L
.22343 .38831 L
.22663 .37859 L
.23021 .36888 L
.23291 .3622 L
.2342 .35917 L
.23862 .34946 L
.24262 .34146 L
.24353 .33974 L
.24898 .33003 L
.25233 .32452 L
.25503 .32032 L
.26177 .31061 L
.26204 .31024 L
.26928 .30089 L
.27176 .29792 L
.2777 .29118 L
Mistroke
.28147 .28718 L
.28718 .28147 L
.29118 .2777 L
.29792 .27176 L
.30089 .26928 L
.31024 .26204 L
.31061 .26177 L
.32032 .25503 L
.32452 .25233 L
.33003 .24898 L
.33974 .24353 L
.34146 .24262 L
.34946 .23862 L
.35917 .2342 L
.3622 .23291 L
.36888 .23021 L
.37859 .22663 L
.38831 .22343 L
.38906 .22319 L
.39802 .22056 L
.40773 .21803 L
.41744 .21581 L
.42716 .21386 L
.42922 .21348 L
.43687 .21219 L
Mfstroke
.47572 .15491 m
.48543 .15453 L
.49514 .15437 L
.50486 .15444 L
.51457 .15474 L
.52359 .15521 L
.52428 .15526 L
.53399 .15601 L
.54371 .157 L
.55342 .15822 L
.56313 .15968 L
.57284 .1614 L
.58256 .16337 L
.58945 .16492 L
.59227 .16559 L
.60198 .1681 L
.61169 .17089 L
.62141 .174 L
.62327 .17463 L
.63112 .17744 L
.64083 .18121 L
.64822 .18434 L
.65054 .18537 L
.66026 .18993 L
.66835 .19406 L
.66997 .19492 L
.67968 .20037 L
.68532 .20377 L
.68939 .20633 L
.69911 .21288 L
.69996 .21348 L
.70882 .22008 L
.71275 .22319 L
.71853 .22802 L
.72404 .23291 L
.72824 .23684 L
.73408 .24262 L
.73796 .24669 L
.74305 .25233 L
.74767 .25779 L
.75107 .26204 L
.75738 .27052 L
.75825 .27176 L
.76468 .28147 L
.76709 .28542 L
.77042 .29118 L
.77553 .30089 L
.77681 .30351 L
.78006 .31061 L
.78405 .32032 L
Mistroke
.78652 .32708 L
.78752 .33003 L
.79051 .33974 L
.79304 .34946 L
.79514 .35917 L
.79623 .36517 L
.79682 .36888 L
.79812 .37859 L
.79904 .38831 L
.79959 .39802 L
.79979 .40773 L
.79965 .41744 L
.79919 .42716 L
.79841 .43687 L
.79731 .44658 L
.79623 .45425 L
.79591 .45629 L
.7942 .46601 L
.7922 .47572 L
.78989 .48543 L
.78729 .49514 L
.78652 .49782 L
.78439 .50486 L
.78121 .51457 L
.77773 .52428 L
.77681 .52673 L
.77395 .53399 L
.76988 .54371 L
.76709 .54996 L
.76549 .55342 L
.7608 .56313 L
.75738 .56981 L
.75578 .57284 L
.75044 .58256 L
.74767 .58737 L
.74476 .59227 L
.73875 .60198 L
.73796 .60322 L
.73237 .61169 L
.72824 .61769 L
.72562 .62141 L
.71853 .63103 L
.71847 .63112 L
.7109 .64083 L
.70882 .6434 L
.70288 .65054 L
.69911 .65493 L
.69439 .66026 L
.68939 .66572 L
.68539 .66997 L
Mistroke
.67968 .67584 L
.67584 .67968 L
.66997 .68539 L
.66572 .68939 L
.66026 .69439 L
.65493 .69911 L
.65054 .70288 L
.6434 .70882 L
.64083 .7109 L
.63112 .71847 L
.63103 .71853 L
.62141 .72562 L
.61769 .72824 L
.61169 .73237 L
.60322 .73796 L
.60198 .73875 L
.59227 .74476 L
.58737 .74767 L
.58256 .75044 L
.57284 .75578 L
.56981 .75738 L
.56313 .7608 L
.55342 .76549 L
.54996 .76709 L
.54371 .76988 L
.53399 .77395 L
.52673 .77681 L
.52428 .77773 L
.51457 .78121 L
.50486 .78439 L
.49782 .78652 L
.49514 .78729 L
.48543 .78989 L
.47572 .7922 L
.46601 .7942 L
.45629 .79591 L
.45425 .79623 L
.44658 .79731 L
.43687 .79841 L
.42716 .79919 L
.41744 .79965 L
.40773 .79979 L
.39802 .79959 L
.38831 .79904 L
.37859 .79812 L
.36888 .79682 L
.36517 .79623 L
.35917 .79514 L
.34946 .79304 L
.33974 .79051 L
Mistroke
.33003 .78752 L
.32708 .78652 L
.32032 .78405 L
.31061 .78006 L
.30351 .77681 L
.30089 .77553 L
.29118 .77042 L
.28542 .76709 L
.28147 .76468 L
.27176 .75825 L
.27052 .75738 L
.26204 .75107 L
.25779 .74767 L
.25233 .74305 L
.24669 .73796 L
.24262 .73408 L
.23684 .72824 L
.23291 .72404 L
.22802 .71853 L
.22319 .71275 L
.22008 .70882 L
.21348 .69996 L
.21288 .69911 L
.20633 .68939 L
.20377 .68532 L
.20037 .67968 L
.19492 .66997 L
.19406 .66835 L
.18993 .66026 L
.18537 .65054 L
.18434 .64822 L
.18121 .64083 L
.17744 .63112 L
.17463 .62327 L
.174 .62141 L
.17089 .61169 L
.1681 .60198 L
.16559 .59227 L
.16492 .58945 L
.16337 .58256 L
.1614 .57284 L
.15968 .56313 L
.15822 .55342 L
.157 .54371 L
.15601 .53399 L
.15526 .52428 L
.15521 .52359 L
.15474 .51457 L
.15444 .50486 L
.15437 .49514 L
Mistroke
.15453 .48543 L
.15491 .47572 L
.15521 .47024 L
.15551 .46601 L
.15635 .45629 L
.15742 .44658 L
.15873 .43687 L
.16028 .42716 L
.16208 .41744 L
.16413 .40773 L
.16492 .40427 L
.16644 .39802 L
.16903 .38831 L
.17192 .37859 L
.17463 .37025 L
.1751 .36888 L
.17859 .35917 L
.18242 .34946 L
.18434 .34487 L
.1866 .33974 L
.19114 .33003 L
.19406 .3242 L
.19609 .32032 L
.20147 .31061 L
.20377 .3067 L
.20732 .30089 L
.21348 .29147 L
.21368 .29118 L
.22059 .28147 L
.22319 .27801 L
.22811 .27176 L
.23291 .26599 L
.23633 .26204 L
.24262 .25514 L
.2453 .25233 L
.25233 .2453 L
.25514 .24262 L
.26204 .23633 L
.26599 .23291 L
.27176 .22811 L
.27801 .22319 L
.28147 .22059 L
.29118 .21368 L
.29147 .21348 L
.30089 .20732 L
.3067 .20377 L
.31061 .20147 L
.32032 .19609 L
.3242 .19406 L
.33003 .19114 L
Mistroke
.33974 .1866 L
.34487 .18434 L
.34946 .18242 L
.35917 .17859 L
.36888 .1751 L
.37025 .17463 L
.37859 .17192 L
.38831 .16903 L
.39802 .16644 L
.40427 .16492 L
.40773 .16413 L
.41744 .16208 L
.42716 .16028 L
.43687 .15873 L
.44658 .15742 L
.45629 .15635 L
.46601 .15551 L
.47024 .15521 L
.47572 .15491 L
Mfstroke
.44658 .09621 m
.45629 .09528 L
.46601 .09456 L
.47572 .09404 L
.48543 .09372 L
.49514 .09359 L
.50486 .09366 L
.51457 .09394 L
.52428 .09442 L
.53399 .0951 L
.54371 .09599 L
.55209 .09693 L
.55342 .09709 L
.56313 .0984 L
.57284 .09992 L
.58256 .10166 L
.59227 .10362 L
.60198 .1058 L
.60545 .10664 L
.61169 .10823 L
.62141 .11091 L
.63112 .11386 L
.63873 .11636 L
.64083 .11708 L
.65054 .12057 L
.66026 .12437 L
.66436 .12607 L
.66997 .12847 L
.67968 .13289 L
.68562 .13578 L
.68939 .13768 L
.69911 .14281 L
.70389 .14549 L
.70882 .14837 L
.71853 .15433 L
.71989 .15521 L
.72824 .16078 L
.73411 .16492 L
.73796 .16774 L
.74685 .17463 L
.74767 .17529 L
.75738 .18346 L
.75839 .18434 L
.76709 .19233 L
.76889 .19406 L
.77681 .202 L
.77849 .20377 L
.78652 .21261 L
.78727 .21348 L
.79533 .22319 L
Mistroke
.79623 .22433 L
.80273 .23291 L
.80594 .2374 L
.80953 .24262 L
.81566 .25213 L
.81578 .25233 L
.82151 .26204 L
.82537 .26909 L
.82676 .27176 L
.83157 .28147 L
.83508 .28918 L
.83595 .29118 L
.83993 .30089 L
.84353 .31061 L
.84479 .31426 L
.84677 .32032 L
.84966 .33003 L
.85221 .33974 L
.85443 .34946 L
.85451 .3498 L
.85635 .35917 L
.85797 .36888 L
.8593 .37859 L
.86035 .38831 L
.86112 .39802 L
.86162 .40773 L
.86186 .41744 L
.86184 .42716 L
.86157 .43687 L
.86105 .44658 L
.86028 .45629 L
.85927 .46601 L
.85801 .47572 L
.85651 .48543 L
.85477 .49514 L
.85451 .49652 L
.85279 .50486 L
.85058 .51457 L
.84813 .52428 L
.84543 .53399 L
.84479 .53616 L
.84249 .54371 L
.83929 .55342 L
.83584 .56313 L
.83508 .56517 L
.83214 .57284 L
.82818 .58256 L
.82537 .58908 L
.82396 .59227 L
.81946 .60198 L
Mistroke
.81566 .60978 L
.81469 .61169 L
.80963 .62141 L
.80594 .62812 L
.80425 .63112 L
.79856 .64083 L
.79623 .64466 L
.79254 .65054 L
.78652 .65975 L
.78618 .66026 L
.77945 .66997 L
.77681 .67365 L
.77235 .67968 L
.76709 .68655 L
.76486 .68939 L
.75738 .6986 L
.75696 .69911 L
.74861 .70882 L
.74767 .70988 L
.73976 .71853 L
.73796 .72044 L
.7304 .72824 L
.72824 .7304 L
.72044 .73796 L
.71853 .73976 L
.70988 .74767 L
.70882 .74861 L
.69911 .75696 L
.6986 .75738 L
.68939 .76486 L
.68655 .76709 L
.67968 .77235 L
.67365 .77681 L
.66997 .77945 L
.66026 .78618 L
.65975 .78652 L
.65054 .79254 L
.64466 .79623 L
.64083 .79856 L
.63112 .80425 L
.62812 .80594 L
.62141 .80963 L
.61169 .81469 L
.60978 .81566 L
.60198 .81946 L
.59227 .82396 L
.58908 .82537 L
.58256 .82818 L
.57284 .83214 L
.56517 .83508 L
Mistroke
.56313 .83584 L
.55342 .83929 L
.54371 .84249 L
.53616 .84479 L
.53399 .84543 L
.52428 .84813 L
.51457 .85058 L
.50486 .85279 L
.49652 .85451 L
.49514 .85477 L
.48543 .85651 L
.47572 .85801 L
.46601 .85927 L
.45629 .86028 L
.44658 .86105 L
.43687 .86157 L
.42716 .86184 L
.41744 .86186 L
.40773 .86162 L
.39802 .86112 L
.38831 .86035 L
.37859 .8593 L
.36888 .85797 L
.35917 .85635 L
.3498 .85451 L
.34946 .85443 L
.33974 .85221 L
.33003 .84966 L
.32032 .84677 L
.31426 .84479 L
.31061 .84353 L
.30089 .83993 L
.29118 .83595 L
.28918 .83508 L
.28147 .83157 L
.27176 .82676 L
.26909 .82537 L
.26204 .82151 L
.25233 .81578 L
.25213 .81566 L
.24262 .80953 L
.2374 .80594 L
.23291 .80273 L
.22433 .79623 L
.22319 .79533 L
.21348 .78727 L
.21261 .78652 L
.20377 .77849 L
.202 .77681 L
.19406 .76889 L
Mistroke
.19233 .76709 L
.18434 .75839 L
.18346 .75738 L
.17529 .74767 L
.17463 .74685 L
.16774 .73796 L
.16492 .73411 L
.16078 .72824 L
.15521 .71989 L
.15433 .71853 L
.14837 .70882 L
.14549 .70389 L
.14281 .69911 L
.13768 .68939 L
.13578 .68562 L
.13289 .67968 L
.12847 .66997 L
.12607 .66436 L
.12437 .66026 L
.12057 .65054 L
.11708 .64083 L
.11636 .63873 L
.11386 .63112 L
.11091 .62141 L
.10823 .61169 L
.10664 .60545 L
.1058 .60198 L
.10362 .59227 L
.10166 .58256 L
.09992 .57284 L
.0984 .56313 L
.09709 .55342 L
.09693 .55209 L
.09599 .54371 L
.0951 .53399 L
.09442 .52428 L
.09394 .51457 L
.09366 .50486 L
.09359 .49514 L
.09372 .48543 L
.09404 .47572 L
.09456 .46601 L
.09528 .45629 L
.09621 .44658 L
.09693 .4402 L
.09734 .43687 L
.09869 .42716 L
.10024 .41744 L
.10199 .40773 L
.10398 .39802 L
Mistroke
.10619 .38831 L
.10664 .38643 L
.10863 .37859 L
.11132 .36888 L
.11426 .35917 L
.11636 .3527 L
.11745 .34946 L
.12092 .33974 L
.12465 .33003 L
.12607 .32654 L
.12869 .32032 L
.13304 .31061 L
.13578 .30482 L
.1377 .30089 L
.14271 .29118 L
.14549 .28607 L
.14808 .28147 L
.15383 .27176 L
.15521 .26954 L
.16 .26204 L
.16492 .25477 L
.16662 .25233 L
.17372 .24262 L
.17463 .24141 L
.18133 .23291 L
.18434 .22925 L
.18951 .22319 L
.19406 .21808 L
.19831 .21348 L
.20377 .2078 L
.2078 .20377 L
.21348 .19831 L
.21808 .19406 L
.22319 .18951 L
.22925 .18434 L
.23291 .18133 L
.24141 .17463 L
.24262 .17372 L
.25233 .16662 L
.25477 .16492 L
.26204 .16 L
.26954 .15521 L
.27176 .15383 L
.28147 .14808 L
.28607 .14549 L
.29118 .14271 L
.30089 .1377 L
.30482 .13578 L
.31061 .13304 L
.32032 .12869 L
Mistroke
.32654 .12607 L
.33003 .12465 L
.33974 .12092 L
.34946 .11745 L
.3527 .11636 L
.35917 .11426 L
.36888 .11132 L
.37859 .10863 L
.38643 .10664 L
.38831 .10619 L
.39802 .10398 L
.40773 .10199 L
.41744 .10024 L
.42716 .09869 L
.43687 .09734 L
.4402 .09693 L
.44658 .09621 L
Mfstroke
.42716 .0281 m
.43687 .02702 L
.44658 .0261 L
.45629 .02535 L
.46601 .02478 L
.47572 .02436 L
.48543 .0241 L
.49514 .02401 L
.50486 .02408 L
.51457 .02432 L
.52428 .02474 L
.53399 .02532 L
.54371 .02606 L
.55342 .02698 L
.56313 .02808 L
.56996 .02894 L
.57284 .02933 L
.58256 .03075 L
.59227 .03237 L
.60198 .03416 L
.61169 .03612 L
.62141 .0383 L
.62291 .03866 L
.63112 .04067 L
.64083 .04324 L
.65054 .04602 L
.65821 .04837 L
.66026 .04902 L
.66997 .05226 L
.67968 .05573 L
.68593 .05808 L
.68939 .05943 L
.69911 .06339 L
.70882 .06762 L
.70921 .06779 L
.71853 .07212 L
.72824 .07693 L
.72936 .07751 L
.73796 .08207 L
.7471 .08722 L
.74767 .08755 L
.75738 .09339 L
.76297 .09693 L
.76709 .09963 L
.77681 .10627 L
.77733 .10664 L
.78652 .11337 L
.79042 .11636 L
.79623 .12094 L
.80247 .12607 L
Mistroke
.80594 .12903 L
.81356 .13578 L
.81566 .13771 L
.82381 .14549 L
.82537 .14703 L
.83331 .15521 L
.83508 .1571 L
.84214 .16492 L
.84479 .16799 L
.85035 .17463 L
.85451 .17981 L
.85802 .18434 L
.86422 .19272 L
.86517 .19406 L
.87184 .20377 L
.87393 .20697 L
.87805 .21348 L
.88364 .22285 L
.88384 .22319 L
.88926 .23291 L
.89336 .24076 L
.89429 .24262 L
.89898 .25233 L
.90307 .26144 L
.90333 .26204 L
.90736 .27176 L
.91108 .28147 L
.91278 .28619 L
.91451 .29118 L
.91767 .30089 L
.92056 .31061 L
.92249 .31766 L
.92319 .32032 L
.92556 .33003 L
.92769 .33974 L
.92959 .34946 L
.93126 .35917 L
.93221 .36535 L
.93271 .36888 L
.93393 .37859 L
.93495 .38831 L
.93576 .39802 L
.93636 .40773 L
.93675 .41744 L
.93696 .42716 L
.93697 .43687 L
.93679 .44658 L
.93642 .45629 L
.93586 .46601 L
.9351 .47572 L
Mistroke
.93416 .48543 L
.93303 .49514 L
.93221 .50135 L
.93171 .50486 L
.93021 .51457 L
.92851 .52428 L
.92661 .53399 L
.92452 .54371 L
.92249 .55235 L
.92223 .55342 L
.91976 .56313 L
.91707 .57284 L
.91419 .58256 L
.91278 .5871 L
.91112 .59227 L
.90785 .60198 L
.90437 .61169 L
.90307 .61517 L
.90065 .62141 L
.89671 .63112 L
.89336 .63898 L
.89254 .64083 L
.88811 .65054 L
.88364 .65987 L
.88345 .66026 L
.8785 .66997 L
.87393 .67854 L
.87331 .67968 L
.86783 .68939 L
.86422 .69554 L
.86207 .69911 L
.85601 .70882 L
.85451 .71115 L
.84964 .71853 L
.84479 .72558 L
.84292 .72824 L
.83585 .73796 L
.83508 .73899 L
.82839 .74767 L
.82537 .75145 L
.82053 .75738 L
.81566 .76315 L
.81223 .76709 L
.80594 .77413 L
.80349 .77681 L
.79623 .78449 L
.79425 .78652 L
.78652 .79425 L
.78449 .79623 L
.77681 .80349 L
Mistroke
.77413 .80594 L
.76709 .81223 L
.76315 .81566 L
.75738 .82053 L
.75145 .82537 L
.74767 .82839 L
.73899 .83508 L
.73796 .83585 L
.72824 .84292 L
.72558 .84479 L
.71853 .84964 L
.71115 .85451 L
.70882 .85601 L
.69911 .86207 L
.69554 .86422 L
.68939 .86783 L
.67968 .87331 L
.67854 .87393 L
.66997 .8785 L
.66026 .88345 L
.65987 .88364 L
.65054 .88811 L
.64083 .89254 L
.63898 .89336 L
.63112 .89671 L
.62141 .90065 L
.61517 .90307 L
.61169 .90437 L
.60198 .90785 L
.59227 .91112 L
.5871 .91278 L
.58256 .91419 L
.57284 .91707 L
.56313 .91976 L
.55342 .92223 L
.55235 .92249 L
.54371 .92452 L
.53399 .92661 L
.52428 .92851 L
.51457 .93021 L
.50486 .93171 L
.50135 .93221 L
.49514 .93303 L
.48543 .93416 L
.47572 .9351 L
.46601 .93586 L
.45629 .93642 L
.44658 .93679 L
.43687 .93697 L
.42716 .93696 L
Mistroke
.41744 .93675 L
.40773 .93636 L
.39802 .93576 L
.38831 .93495 L
.37859 .93393 L
.36888 .93271 L
.36535 .93221 L
.35917 .93126 L
.34946 .92959 L
.33974 .92769 L
.33003 .92556 L
.32032 .92319 L
.31766 .92249 L
.31061 .92056 L
.30089 .91767 L
.29118 .91451 L
.28619 .91278 L
.28147 .91108 L
.27176 .90736 L
.26204 .90333 L
.26144 .90307 L
.25233 .89898 L
.24262 .89429 L
.24076 .89336 L
.23291 .88926 L
.22319 .88384 L
.22285 .88364 L
.21348 .87805 L
.20697 .87393 L
.20377 .87184 L
.19406 .86517 L
.19272 .86422 L
.18434 .85802 L
.17981 .85451 L
.17463 .85035 L
.16799 .84479 L
.16492 .84214 L
.1571 .83508 L
.15521 .83331 L
.14703 .82537 L
.14549 .82381 L
.13771 .81566 L
.13578 .81356 L
.12903 .80594 L
.12607 .80247 L
.12094 .79623 L
.11636 .79042 L
.11337 .78652 L
.10664 .77733 L
.10627 .77681 L
Mistroke
.09963 .76709 L
.09693 .76297 L
.09339 .75738 L
.08755 .74767 L
.08722 .7471 L
.08207 .73796 L
.07751 .72936 L
.07693 .72824 L
.07212 .71853 L
.06779 .70921 L
.06762 .70882 L
.06339 .69911 L
.05943 .68939 L
.05808 .68593 L
.05573 .67968 L
.05226 .66997 L
.04902 .66026 L
.04837 .65821 L
.04602 .65054 L
.04324 .64083 L
.04067 .63112 L
.03866 .62291 L
.0383 .62141 L
.03612 .61169 L
.03416 .60198 L
.03237 .59227 L
.03075 .58256 L
.02933 .57284 L
.02894 .56996 L
.02808 .56313 L
.02698 .55342 L
.02606 .54371 L
.02532 .53399 L
.02474 .52428 L
.02432 .51457 L
.02408 .50486 L
.02401 .49514 L
.0241 .48543 L
.02436 .47572 L
.02478 .46601 L
.02535 .45629 L
.0261 .44658 L
.02702 .43687 L
.0281 .42716 L
.02894 .42043 L
.02935 .41744 L
.03079 .40773 L
.03239 .39802 L
.03416 .38831 L
.03614 .37859 L
Mistroke
.0383 .36888 L
.03866 .36734 L
.04064 .35917 L
.04319 .34946 L
.04594 .33974 L
.04837 .33171 L
.0489 .33003 L
.05207 .32032 L
.05547 .31061 L
.05808 .30359 L
.05912 .30089 L
.063 .29118 L
.06715 .28147 L
.06779 .28001 L
.07156 .27176 L
.07625 .26204 L
.07751 .25954 L
.08123 .25233 L
.08653 .24262 L
.08722 .2414 L
.09217 .23291 L
.09693 .22512 L
.09815 .22319 L
.1045 .21348 L
.10664 .21034 L
.11126 .20377 L
.11636 .19681 L
.11843 .19406 L
.12607 .18435 L
.12607 .18434 L
.13422 .17463 L
.13578 .17284 L
.1429 .16492 L
.14549 .16214 L
.15219 .15521 L
.15521 .15219 L
.16214 .14549 L
.16492 .1429 L
.17284 .13578 L
.17463 .13422 L
.18434 .12607 L
.18435 .12607 L
.19406 .11843 L
.19681 .11636 L
.20377 .11126 L
.21034 .10664 L
.21348 .1045 L
.22319 .09815 L
.22512 .09693 L
.23291 .09217 L
Mistroke
.2414 .08722 L
.24262 .08653 L
.25233 .08123 L
.25954 .07751 L
.26204 .07625 L
.27176 .07156 L
.28001 .06779 L
.28147 .06715 L
.29118 .063 L
.30089 .05912 L
.30359 .05808 L
.31061 .05547 L
.32032 .05207 L
.33003 .0489 L
.33171 .04837 L
.33974 .04594 L
.34946 .04319 L
.35917 .04064 L
.36734 .03866 L
.36888 .0383 L
.37859 .03614 L
.38831 .03416 L
.39802 .03239 L
.40773 .03079 L
.41744 .02935 L
.42043 .02894 L
.42716 .0281 L
Mfstroke
.75 g
.35577 .35577 m
.66026 .35577 L
.35577 .66026 L
.35577 .35577 L
F
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 288},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`0001/2000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00``3ooolU00000<P0oooo002k0?ooo`P000009@3oool800000<00oooo002b0?ooo`T00000
=@3oool900000;L0oooo002Z0?ooo`P00000A`3oool800000:l0oooo002V0?ooo`@00000E`3oool4
00000:/0oooo002Q0?ooo`D00000G`3oool400000:L0oooo002M0?ooo`@00000J03oool500000:80
oooo002I0?ooo`@00000L@3oool4000009h0oooo002E0?ooo`@00000N@3oool4000009X0oooo002A
0?ooo`@00000P@3oool4000009H0oooo002>0?ooo`<00000R@3oool2000009@0oooo002=0?ooo`80
0000S@3oool200000980oooo00280?ooo`D00000T@3oool3000008l0oooo00250?ooo`<00000V@3o
ool2000008d0oooo00230?ooo`800000WP3oool3000008X0oooo00210?ooo`800000X`3oool40000
08H0oooo001o0?ooo`800000Z@3oool2000008@0oooo001l0?ooo`<00000[@3oool200000880oooo
001i0?ooo`<00000/P3oool200000800oooo001g0?ooo`800000]`3oool2000007h0oooo001d0?oo
o`<00000^`3oool2000007`0oooo001b0?ooo`800000`03oool2000007X0oooo001`0?ooo`800000
a03oool2000007P0oooo001_0?ooo`030000003oool0oooo0<H0oooo0`00001e0?ooo`00K@3oool2
00000<`0oooo0P00001c0?ooo`00K03oool00`000000oooo0?ooo`3>0?ooo`800000L@3oool006X0
oooo0P00003C0?ooo`800000K`3oool006P0oooo0P00003G0?ooo`800000K@3oool006H0oooo0P00
003K0?ooo`030000003oool0oooo06X0oooo001T0?ooo`800000gP3oool2000006X0oooo001R0?oo
o`800000H`3ooolQ000005h0oooo00<000000?ooo`3oool0I`3oool00640oooo00<000000?ooo`3o
ool0E`3oool<00000240oooo2000001G0?ooo`800000I`3oool00600oooo00<000000?ooo`3oool0
D`3oool5000003D0oooo1000001E0?ooo`800000I@3oool005l0oooo00<000000?ooo`3oool0C03o
ool8000003h0oooo2@00001>0?ooo`030000003oool0oooo0680oooo001M0?ooo`800000B`3oool4
000004l0oooo1000001;0?ooo`800000HP3oool005X0oooo0`0000180?ooo`D00000E`3oool50000
04P0oooo0P00001P0?ooo`00F03oool2000004L0oooo1000001Q0?ooo`<00000A`3oool00`000000
oooo0?ooo`1M0?ooo`00E`3oool00`000000oooo0?ooo`140?ooo`<00000J03oool4000004@0oooo
00<000000?ooo`3oool0G03oool005H0oooo00<000000?ooo`3oool0@`3oool2000006l0oooo0P00
00130?ooo`030000003oool0oooo05/0oooo001E0?ooo`030000003oool0oooo0480oooo0P00001c
0?ooo`<00000@@3oool00`000000oooo0?ooo`1J0?ooo`00E03oool00`000000oooo0?ooo`0o0?oo
o`@00000N03oool4000003h0oooo00<000000?ooo`3oool0F@3oool005<0oooo00<000000?ooo`3o
ool0?P3oool200000800oooo0`00000l0?ooo`800000F@3oool00580oooo00<000000?ooo`3oool0
?03oool3000008D0oooo0P00000l0?ooo`030000003oool0oooo05H0oooo001@0?ooo`800000?03o
ool3000008X0oooo0P00000k0?ooo`030000003oool0oooo05D0oooo001?0?ooo`030000003oool0
oooo03X0oooo0P00002?0?ooo`800000>P3oool00`000000oooo0?ooo`1D0?ooo`00CP3oool00`00
0000oooo0?ooo`0h0?ooo`<00000T`3oool2000003T0oooo0P00001D0?ooo`00C03oool2000003T0
oooo0P00002H0?ooo`800000>@3oool00`000000oooo0?ooo`1A0?ooo`00B`3oool00`000000oooo
0?ooo`0g0?ooo`800000W03oool3000003L0oooo00<000000?ooo`3oool0D03oool004X0oooo00<0
00000?ooo`3oool0=P3oool200000:40oooo0P00000f0?ooo`030000003oool0oooo04l0oooo0019
0?ooo`030000003oool0oooo03D0oooo0P00002U0?ooo`800000=@3oool00`000000oooo0?ooo`1>
0?ooo`00B03oool00`000000oooo0?ooo`0d0?ooo`800000Z@3oool2000003@0oooo00<000000?oo
o`3oool0C@3oool004L0oooo00<000000?ooo`3oool0<`3oool200000:d0oooo0P00000c0?ooo`80
0000C@3oool004D0oooo0P00000d0?ooo`800000/@3oool2000003<0oooo00<000000?ooo`3oool0
BP3oool004@0oooo00<000000?ooo`3oool0<P3oool200000;D0oooo0P00000a0?ooo`030000003o
ool0oooo04X0oooo00130?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool0]`3oool0
0`000000oooo0?ooo`0_0?ooo`030000003oool0oooo04T0oooo00120?ooo`030000003oool0oooo
0340oooo0P00002k0?ooo`800000<03oool2000004T0oooo00110?ooo`030000003oool0oooo0300
oooo0P00001D0?oooaX00000D@3oool200000300oooo00<000000?ooo`3oool0AP3oool00400oooo
00<000000?ooo`3oool0;`3oool2000004/0oooo2`00000J0?ooo`P00000B`3oool00`000000oooo
0?ooo`0]0?ooo`030000003oool0oooo04H0oooo000o0?ooo`030000003oool0oooo02h0oooo0P00
00190?ooo`@00000;@3oool8000004@0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?oo
o`150?ooo`00?P3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0400oooo2@00000i
0?ooo`D00000@03oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo04D0oooo000m0?oo
o`030000003oool0oooo02h0oooo00<000000?ooo`3oool0?P3oool3000004L0oooo1000000m0?oo
o`800000;@3oool00`000000oooo0?ooo`140?ooo`00?@3oool00`000000oooo0?ooo`0/0?ooo`80
0000?03oool5000004h0oooo1000000k0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3o
ool0@`3oool003`0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0i0?ooo`<00000
E`3oool4000003P0oooo0P00000/0?ooo`800000@`3oool003/0oooo00<000000?ooo`3oool0:`3o
ool2000003X0oooo0P00001N0?ooo`@00000=P3oool00`000000oooo0?ooo`0[0?ooo`030000003o
ool0oooo0400oooo000j0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0=`3oool3
000006@0oooo0`00000d0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0@03oool0
03T0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0b0?ooo`D00000JP3oool20000
03<0oooo0P00000[0?ooo`030000003oool0oooo03l0oooo000h0?ooo`030000003oool0oooo02`0
oooo00<000000?ooo`3oool0<03oool300000740oooo0`00000b0?ooo`030000003oool0oooo02T0
oooo00<000000?ooo`3oool0?P3oool003L0oooo00<000000?ooo`3oool0:`3oool200000340oooo
0P00001g0?ooo`800000<@3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo03d0oooo
000g0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0;`3oool2000007/0oooo0`00
000_0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0?@3oool003H0oooo00<00000
0?ooo`3oool0:@3oool200000300oooo0P0000200?ooo`<00000;@3oool00`000000oooo0?ooo`0X
0?ooo`030000003oool0oooo03`0oooo000e0?ooo`030000003oool0oooo02T0oooo00<000000?oo
o`3oool0;P3oool2000008D0oooo0P00000/0?ooo`030000003oool0oooo02P0oooo00<000000?oo
o`3oool0>`3oool003@0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0]0?ooo`80
0000R@3oool2000002/0oooo0P00000X0?ooo`030000003oool0oooo03/0oooo000d0?ooo`030000
003oool0oooo02L0oooo0P00000]0?ooo`<00000S@3oool2000002/0oooo00<000000?ooo`3oool0
9P3oool00`000000oooo0?ooo`0j0?ooo`00<`3oool00`000000oooo0?ooo`0W0?ooo`030000003o
ool0oooo02/0oooo0P00002B0?ooo`800000:P3oool00`000000oooo0?ooo`0V0?ooo`030000003o
ool0oooo03T0oooo000b0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0:P3oool2
000009D0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0
oooo03P0oooo000a0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0:03oool20000
09P0oooo0P00000Y0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0>03oool00300
oooo00<000000?ooo`3oool09`3oool2000002T0oooo0P00002L0?ooo`<000009`3oool00`000000
oooo0?ooo`0V0?ooo`030000003oool0oooo03L0oooo000_0?ooo`030000003oool0oooo02L0oooo
00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`2O0?ooo`8000009P3oool2000002L0oooo
00<000000?ooo`3oool0=P3oool002h0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?oo
o`0W0?ooo`800000Y03oool2000002H0oooo00<000000?ooo`3oool0903oool00`000000oooo0?oo
o`0f0?ooo`00;P3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo02L0oooo00<00000
0?ooo`3oool0@`3ooolM000004H0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0U
0?ooo`030000003oool0oooo03D0oooo000]0?ooo`030000003oool0oooo02H0oooo00<000000?oo
o`3oool09`3oool00`000000oooo0?ooo`0l0?ooo`P000007@3oool8000003l0oooo0P00000T0?oo
o`030000003oool0oooo02D0oooo00<000000?ooo`3oool0=03oool002d0oooo00<000000?ooo`3o
ool09P3oool00`000000oooo0?ooo`0U0?ooo`800000>`3oool4000002d0oooo1`00000j0?ooo`03
0000003oool0oooo0280oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0d0?ooo`00
;@3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0
=P3oool5000003P0oooo1P00000d0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0
903oool00`000000oooo0?ooo`0c0?ooo`00;03oool00`000000oooo0?ooo`0T0?ooo`8000009`3o
ool00`000000oooo0?ooo`0`0?ooo`L00000@`3oool400000340oooo0P00000T0?ooo`030000003o
ool0oooo02<0oooo00<000000?ooo`3oool0<`3oool002/0oooo00<000000?ooo`3oool0903oool0
0`000000oooo0?ooo`0U0?ooo`800000<@3oool2000004h0oooo1000000_0?ooo`8000008P3oool0
0`000000oooo0?ooo`0T0?ooo`030000003oool0oooo0380oooo000[0?ooo`030000003oool0oooo
02@0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0^0?ooo`<00000E03oool30000
02h0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo
0380oooo000Z0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool08`3oool2000002d0
oooo1000001J0?ooo`@00000:`3oool00`000000oooo0?ooo`0P0?ooo`800000903oool00`000000
oooo0?ooo`0a0?ooo`00:@3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo02<0oooo
00<000000?ooo`3oool0:@3oool400000680oooo0P00000Z0?ooo`030000003oool0oooo0200oooo
00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0`0?ooo`00:03oool00`000000oooo0?oo
o`0T0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0:03oool2000006P0oooo1000
000W0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool08P3oool00`000000oooo0?oo
o`0`0?ooo`00:03oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0280oooo0P00000Y
0?ooo`800000KP3oool3000002D0oooo0P00000P0?ooo`030000003oool0oooo02<0oooo00<00000
0?ooo`3oool0;`3oool002L0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo02D0oooo1000001c0?ooo`8000009@3oool00`000000oooo0?ooo`0N0?oo
o`030000003oool0oooo02<0oooo00<000000?ooo`3oool0;P3oool002H0oooo00<000000?ooo`3o
ool0903oool00`000000oooo0?ooo`0P0?ooo`8000009P3oool2000007T0oooo0P00000S0?ooo`03
0000003oool0oooo01l0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0]0?ooo`00
9P3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0
903oool2000007d0oooo0P00000R0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0
8P3oool00`000000oooo0?ooo`0]0?ooo`009@3oool00`000000oooo0?ooo`0T0?ooo`030000003o
ool0oooo01l0oooo00<000000?ooo`3oool08`3oool200000840oooo0P00000Q0?ooo`8000007`3o
ool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo02`0oooo000U0?ooo`030000003oool0
oooo0280oooo0P00000Q0?ooo`030000003oool0oooo0280oooo0P0000250?ooo`8000008@3oool0
0`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0;03oool0
02@0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo
0280oooo00<000000?ooo`3oool0Q`3oool200000200oooo00<000000?ooo`3oool07@3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo02`0oooo000T0?ooo`030000003oool0oooo0240
oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0Q0?ooo`800000S03oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool08P3oool00`000000
oooo0?ooo`0[0?ooo`008`3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool0803oool2000008l0oooo0P00000O0?ooo`030000003oool0oooo01`0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0[0?ooo`008`3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool2000009<0oooo0P00
000N0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`0Z0?ooo`008P3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo01h0oooo00<00000
0?ooo`3oool07`3oool00`000000oooo0?ooo`0n0?oooaT00000?P3oool00`000000oooo0?ooo`0L
0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0Z
0?ooo`008P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo01h0oooo00<000000?oo
o`3oool07P3oool2000003T0oooo2000000I0?ooo`P00000=`3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo01`0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0Y0?oo
o`008@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3o
ool07@3oool2000003<0oooo2000000Y0?ooo`P00000<03oool2000001`0oooo00<000000?ooo`3o
ool0703oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo02T0oooo000P0?ooo`030000
003oool0oooo0240oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0M0?ooo`030000
003oool0oooo02`0oooo1`00000i0?ooo`H00000;03oool2000001/0oooo00<000000?ooo`3oool0
703oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo02P0oooo000P0?ooo`030000003o
ool0oooo0200oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo02/0oooo0P0000160?ooo`<00000:`3oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01/0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0X0?ooo`007`3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0703oool0
0`000000oooo0?ooo`0X0?ooo`@00000B`3oool4000002P0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool09`3oool0
01l0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo
01`0oooo00<000000?ooo`3oool09@3oool4000005<0oooo1000000U0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo
02L0oooo000N0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool07P3oool00`000000
oooo0?ooo`0K0?ooo`030000003oool0oooo02<0oooo0`00001K0?ooo`<000008`3oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool08@3oool00`000000
oooo0?ooo`0V0?ooo`007P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo01d0oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0O0?ooo`D00000H@3oool5000001l0oooo
00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0200oooo
00<000000?ooo`3oool09P3oool001h0oooo00<000000?ooo`3oool0803oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo01X0oooo0P00000P0?ooo`800000J`3oool2000001h0oooo00<0
00000?ooo`3oool05`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0200oooo00<0
00000?ooo`3oool09P3oool001d0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0M
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool07P3oool2000006l0oooo0P00000M
0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0O
0?ooo`030000003oool0oooo02H0oooo000M0?ooo`030000003oool0oooo0200oooo00<000000?oo
o`3oool0703oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01d0oooo0P00001c0?oo
o`800000703oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool0803oool00`000000oooo0?ooo`0U0?ooo`00703oool00`000000oooo0?ooo`0P0?ooo`03
0000003oool0oooo01d0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0L0?ooo`80
0000M`3oool2000001/0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo01l0oooo00<000000?ooo`3oool09@3oool001`0oooo00<000000?ooo`3oool0
7`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0
6`3oool2000007/0oooo0P00000J0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo02D0oooo000K0?ooo`030000003o
ool0oooo0200oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01X0oooo0P00001o0?ooo`8000006@3oool00`000000oooo0?ooo`0E0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0T0?ooo`006`3oool0
0`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0I0?ooo`800000P`3oool2000001L0oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0903oool0
01/0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool0603oool2000008L0oooo0P00000F0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo
02<0oooo000J0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0703oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0R@3oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool06@3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo02<0oooo000J0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01H0oooo
0P00000o0?ooo`l00000?`3oool2000001D0oooo00<000000?ooo`3oool05@3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool08`3oool001T0oooo00<0
00000?ooo`3oool07`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool05P3oool00`000000oooo0?ooo`0^0?oooa4000003`3ooolA00000300oooo00<0
00000?ooo`3oool04`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`3oool07`3oool00`000000oooo0?ooo`0R0?ooo`006@3oool00`000000oooo0?ooo`0O
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0E
0?ooo`800000:03oool900000340oooo2@00000X0?ooo`030000003oool0oooo01<0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01h0oooo00<00000
0?ooo`3oool08P3oool001T0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0T0?oo
o`@00000@`3oool4000002D0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0D0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0R0?oo
o`00603oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool05`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0200oooo1000001;0?ooo`@0
00008P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0280oooo000H0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0G0?ooo`030000
003oool0oooo01H0oooo00<000000?ooo`3oool07@3oool4000005<0oooo2000000K0?ooo`030000
003oool0oooo0140oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool08@3oool001L0oooo00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool05@3o
ool2000001X0oooo1P00001O0?ooo`<000006@3oool200000180oooo00<000000?ooo`3oool0503o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool08@3o
ool001L0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0G0?ooo`<00000I`3oool3
000001T0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0Q0?ooo`005`3oool00`00
0000oooo0?ooo`0N0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo01D0oooo0`00001]0?ooo`@000005P3oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`0N0?ooo`030000003oool0oooo0200oooo000F0?ooo`030000003oool0oooo01l0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool0503oool2000007@0oooo0`00000D0?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01h0
oooo00<000000?ooo`3oool0803oool001H0oooo00<000000?ooo`3oool07P3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo
0?ooo`0C0?ooo`800000N@3oool200000180oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`0C0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07P3oool00`000000oooo
0?ooo`0P0?ooo`005P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool05P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo0P00
001m0?ooo`8000004@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0200oooo000F
0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0F
0?ooo`030000003oool0oooo01<0oooo0P00000B0?ooo`800000P@3oool2000000l0oooo00<00000
0?ooo`3oool03`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool07P3oool00`000000oooo0?ooo`0P0?ooo`005@3oool00`000000oooo0?ooo`0N0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0B0?oo
o`030000003oool0oooo0140oooo00<000000?ooo`3oool0P`3oool2000000h0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`0P0?ooo`005@3oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo0140oooo00<000000?ooo`3oool0QP3oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo00l0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01h0oooo00<000000?ooo`3oool07`3oool001@0oooo0P00000O0?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0B0?ooo`030000
003oool0oooo00l0oooo0P00002:0?ooo`030000003oool0oooo00/0oooo0P00000?0?ooo`800000
503oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0
7`3oool001@0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo08/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0>0?ooo`030000003o
ool0oooo0180oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo01l0oooo000C0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool03`3oool0
0`000000oooo0?ooo`2=0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool0
0`000000oooo0?ooo`0O0?ooo`004`3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool0903ooom7000002@0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0M0?ooo`030000003oool0oooo01h0oooo000C0?ooo`030000003oool0oooo01h0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0180
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0E0?oooa000000A`3oool<000001T0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0180
oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01h0
oooo000C0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`0F0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`0<0?ooo`T00000H`3oool900000100oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`0M0?ooo`030000003oool0oooo01h0oooo000B0?ooo`030000003oool0oooo01h0oooo00<0
00000?ooo`3oool06@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`0:0?ooo`<00000M@3oool;000000H0oooo00<0
00000?ooo`3oool02@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01h0oooo000B
0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0F
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`09
0?ooo`800000P`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool07@3oool00`000000oooo0?ooo`0M0?ooo`004P3oool00`000000oooo0?ooo`0M0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0B0?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool0203oool2000008H0oooo00<000000?oo
o`3oool00`3oool2000000X0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0B0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0M0?oo
o`004P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3o
ool05@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`260?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool0203oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01d0oooo000A0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0E0?ooo`030000
003oool0oooo0180oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo08H0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0H0?ooo`030000
003oool0oooo01d0oooo00<000000?ooo`3oool07@3oool00140oooo00<000000?ooo`3oool07@3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool04P3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0R03o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3o
ool00`000000oooo0?ooo`0L0?ooo`004@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`020?oooh80_kno103oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool0
0`000000oooo0?ooo`0L0?ooo`004@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`020?oooh40_kno103oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`00
0000oooo0?ooo`0L0?ooo`00403oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00d0
oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`030?oooh00_kno1@3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool03@3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool07@3oool00`000000
oooo0?ooo`0L0?ooo`00403oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01T0oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`030?ooogl0_kno1@3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`0B0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool07@3oool00`000000oooo
0?ooo`0L0?ooo`00403oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`3oool05@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool02@3oool00`000000oooo0?ooo`030?ooogh0_kno1P3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`0B0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?oo
o`0L0?ooo`00403oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool05@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`030?ooogd0_kno1P3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0B
0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0L
0?ooo`00403oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`030?ooog`0_kno1`3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0B0?oo
o`030000003oool0oooo01L0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0L0?oo
o`003`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool05@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`030?ooog/0_kno1`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0K0?ooo`00
3`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`030?ooogX0_kno203oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0B0?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0K0?ooo`003`3o
ool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0503o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`030?ooogT0_kno203oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0K0?ooo`003`3oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool05@3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`030?ooogP0_kno203oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0K0?ooo`003P3oool00`00
0000oooo0?ooo`0L0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`040?ooogL0_kno203oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0K0?ooo`003P3oool00`000000
oooo0?ooo`0L0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool05@3oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`040?ooogH0_kno203oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01L0
oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0K0?ooo`003P3oool00`000000oooo
0?ooo`0L0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool05@3oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`040?ooogD0_kno2@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0K0?ooo`003P3oool00`000000oooo0?oo
o`0L0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?oo
o`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`040?ooog@0_kno2@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool07@3oool00`000000oooo0?ooo`0K0?ooo`003@3oool00`000000oooo0?ooo`0L
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`04
0?ooog<0_kno2@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool03`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01L0oooo00<00000
0?ooo`3oool07@3oool00`000000oooo0?ooo`0K0?ooo`003@3oool00`000000oooo0?ooo`0L0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo00h0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`040?oo
og80_kno2P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`0J0?ooo`003@3oool00`000000oooo0?ooo`0L0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`040?ooog40
_kno2P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3o
ool03`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3o
ool07@3oool00`000000oooo0?ooo`0J0?ooo`003@3oool00`000000oooo0?ooo`0L0?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0B0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`040?ooog00_kno
2P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
7@3oool00`000000oooo0?ooo`0J0?ooo`003@3oool00`000000oooo0?ooo`0K0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooofl0_kno2P3o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3o
ool00`000000oooo0?ooo`0J0?ooo`003@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0
oooo01T0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooofh0_kno2P3oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool03`3oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool0
0`000000oooo0?ooo`0J0?ooo`003@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooofd0_kno2P3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`00
0000oooo0?ooo`0J0?ooo`003@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooof`0_kno2`3oool00`000000
oooo0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000
oooo0?ooo`0J0?ooo`00303oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`040?ooof/0_kno2`3oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0C0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo
0?ooo`0J0?ooo`00303oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool05@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool03@3oool00`000000oooo0?ooo`040?ooofX0_kno2`3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`0C0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?oo
o`0J0?ooo`00303oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01L0oooo00<00000
0?ooo`3oool05@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`040?ooofT0_kno2`3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0C
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0J
0?ooo`00303oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool05@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`040?ooofP0_kno2`3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0J0?oo
o`00303oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3o
ool05@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`040?ooofL0_kno2`3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0J0?ooo`00
303oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`050?ooofH0_kno2`3oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0J0?ooo`00303o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0503o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`050?ooofD0_kno303oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0D0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0J0?ooo`00303oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`050?ooof@0_kno303oool00`000000oooo0?ooo`0@0?ooo`800000403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01X0oooo000<0?ooo`030000003oool0oooo
01/0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
00D0ooooH`2o_kl<0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0M0?ooo`030000003oool0oooo01X0oooo000;0?ooo`030000003oool0oooo01/0
oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00D0
ooooHP2o_kl<0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo01X0oooo000;0?ooo`030000003oool0oooo01/0oooo
00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00D0oooo
H@2o_kl<0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`0M0?ooo`030000003oool0oooo01X0oooo000;0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00D0ooooH02o
_kl<0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`0A0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo01X0oooo000;0?ooo`030000003oool0oooo01/0oooo00<00000
0?ooo`3oool0603oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<00000
0?ooo`3oool03`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00D0ooooG`2o_kl<
0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0B
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0M
0?ooo`030000003oool0oooo01X0oooo000;0?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0180oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00D0ooooGP2o_kl<0?oo
o`030000003oool0oooo01<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0M0?oo
o`030000003oool0oooo01X0oooo000;0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool05`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00D0ooooG@2o_kl<0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo01@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo01X0oooo000;0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
5`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00D0ooooG02o_kl=0?ooo`030000
003oool0oooo01<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0B0?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0M0?ooo`030000
003oool0oooo01X0oooo000;0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool05`3o
ool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00D0ooooF`2o_kl=0?ooo`030000003o
ool0oooo01<0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo01X0oooo000:0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03`3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00D0ooooFP2o_kl=0?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo01D0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0N0?ooo`030000003oool0
oooo01X0oooo000:0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo00D0ooooF@2o_kl=0?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo
01X0oooo000:0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo00D0ooooF02o_kl=0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01D0
oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01X0
oooo000:0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool05`3oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo00D0ooooE`2o_kl=0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01D0oooo
00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01X0oooo
000:0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0C0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool03`3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo00D0ooooEP2o_kl=0?ooo`030000003oool0oooo01D0oooo00<0
00000?ooo`3oool04P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01D0oooo00<0
00000?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01X0oooo000:
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0C
0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo00D0ooooE@2o_kl>0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool04P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01X0oooo000:0?oo
o`030000003oool0oooo01/0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo0180oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo00D0ooooE02o_kl=0?ooo`8000005P3oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo01<0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo01h0oooo00<000000?ooo`3oool06P3oool000X0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool04P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool01@3ooomC0;no_`d0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0D0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01h0oooo00<000000?ooo`3oool06P3oool000X0oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
1@3ooomB0;no_`d0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo01<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool06P3oool000X0oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool01@3o
oomA0;no_`d0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0D0?ooo`030000003o
ool0oooo01<0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01l0oooo00<000000?ooo`3oool06P3oool000X0oooo00<000000?ooo`3oool06`3oool0
0`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool01@3ooom@
0;no_`d0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0
oooo01l0oooo00<000000?ooo`3oool06P3oool000X0oooo00<000000?ooo`3oool06`3oool00`00
0000oooo0?ooo`0G0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool01@3ooom?0;no
_`h0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
01<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
01l0oooo00<000000?ooo`3oool06P3oool000X0oooo00<000000?ooo`3oool06`3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool01@3ooom>0;no_`h0
oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01l0
oooo00<000000?ooo`3oool06P3oool000X0oooo00<000000?ooo`3oool06`3oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`0?0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01@3ooom=0;no_`h0oooo
00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01h0oooo
00<000000?ooo`3oool06`3oool000X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`0G0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?oo
o`0?0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01@3ooom<0;no_`h0oooo00<0
00000?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01D0oooo00<0
00000?ooo`3oool05P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01h0oooo00<0
00000?ooo`3oool06`3oool000X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0G
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool01@3ooom;0;no_`h0oooo00<00000
0?ooo`3oool05P3oool2000001H0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0F
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0K
0?ooo`002P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`050?ooodX0_kno3P3oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo01D0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0K0?oo
o`002P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3o
ool03`3oool00`000000oooo0?ooo`050?ooodT0_kno3P3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo01D0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0F0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0K0?ooo`00
2P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`050?ooodP0_kno3P3oool00`000000oooo0?ooo`0G0?ooo`030000
003oool0oooo01D0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0G0?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0K0?ooo`002P3o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`050?ooodL0_kno3P3oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo01D0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0G0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0K0?ooo`002P3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03`3oool0
0`000000oooo0?ooo`050?ooodH0_kno3P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
oooo01D0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0K0?ooo`002P3oool00`00
0000oooo0?ooo`0K0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`050?ooodD0_kno3P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0L0?ooo`002P3oool00`000000
oooo0?ooo`0K0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`050?oood@0_kno3P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01H0
oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0L0?ooo`002P3oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`050?oood<0_kno3P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0L0?ooo`002P3oool00`000000oooo0?oo
o`0K0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool00`000000oooo0?oo
o`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?oo
o`050?oood80_kno3P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01H0oooo00<0
00000?ooo`3oool05P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool07`3oool00`000000oooo0?ooo`0L0?ooo`002P3oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`05
0?oood40_kno3P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01H0oooo00<00000
0?ooo`3oool05P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool07`3oool00`000000oooo0?ooo`0M0?ooo`002P3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`050?oo
od00_kno3P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01H0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool07`3oool00`000000oooo0?ooo`0M0?ooo`002P3oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`050?ooocl0
_kno3P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3o
ool05P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool07`3oool00`000000oooo0?ooo`0M0?ooo`002P3oool00`000000oooo0?ooo`0K0?ooo`030000
003oool0oooo01L0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo0100oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`050?oooch0_kno
3P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
5`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
7`3oool00`000000oooo0?ooo`0M0?ooo`002P3oool00`000000oooo0?ooo`0K0?ooo`030000003o
ool0oooo01L0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`050?ooocd0_kno3P3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool05`3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0803o
ool00`000000oooo0?ooo`0M0?ooo`002P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0
oooo01L0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`050?oooc`0_kno3P3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0803oool0
0`000000oooo0?ooo`0M0?ooo`002P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
01L0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`050?oooc/0_kno3P3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0803oool00`00
0000oooo0?ooo`0M0?ooo`002P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01L0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`050?ooocX0_kno3P3oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0803oool00`000000
oooo0?ooo`0M0?ooo`002P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`050?ooocT0_kno3P3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool05`3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0803oool00`000000oooo
0?ooo`0M0?ooo`002P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`050?ooocP0_kno3P3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0803oool00`000000oooo0?oo
o`0N0?ooo`002P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool03P3oool00`000000oooo0?ooo`050?ooocL0_kno3P3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0I
0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0N
0?ooo`002P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`050?ooocH0_kno3P3oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo01`0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0N0?oo
o`002P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`050?ooocD0_kno3P3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01d0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0N0?ooo`00
2P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`050?oooc@0_kno3P3oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo01d0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0O0?ooo`002`3o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`050?oooc<0_kno3P3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01d0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0O0?ooo`002`3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`050?oooc80_kno3P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0
oooo01d0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0O0?ooo`002`3oool00`00
0000oooo0?ooo`0K0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0?ooo`050?oooc40_kno3P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
01`0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0O0?ooo`002`3oool00`000000
oooo0?ooo`0K0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`050?oooc00_kno3P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01d0
oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0P0?ooo`002`3oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`050?ooobl0_kno3P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01/0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01`0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0P0?ooo`002`3oool00`000000oooo0?oo
o`0K0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0503oool00`000000oooo0?oo
o`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`050?ooobh0_kno3P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo0P00
000K0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo0200oooo000;0?ooo`030000003oool0oooo01/0oooo00<00000
0?ooo`3oool0603oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<00000
0?ooo`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00D0oooo;@2o_kl>
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo0200oooo000;0?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00D0oooo;02o_kl>0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo0240oooo000<0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool05`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool03`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00D0oooo:`2o_kl>0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0240oooo000<0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
5`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00D0oooo:P2o_kl>0?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo0240oooo000<0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool05`3o
ool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00D0oooo:@2o_kl>0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0K0?ooo`030000003o
ool0oooo01/0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0R0?ooo`030000003o
ool0oooo0240oooo000<0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03`3oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00D0oooo:02o_kl>0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo0280oooo000<0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo00D0oooo9`2o_kl>0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo
01`0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo
0280oooo000<0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`0=0?ooo`030000003oool0oooo00D0oooo9P2o_kl=0?ooo`800000703oool00`000000
oooo0?ooo`0K0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0703oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool08P3oool000`0oooo
00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool01@3ooolU0;no_`d0oooo00<000000?ooo`3oool06`3oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool00`000000oooo
0?ooo`0N0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool08P3oool000`0oooo00<0
00000?ooo`3oool0703oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01@0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool01@3ooolT0;no_`d0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`0L0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0703oool00`000000oooo0?oo
o`0N0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool08P3oool000d0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool01@3ooolS0;no_`d0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0L
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0N
0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool08`3oool000d0oooo00<000000?oo
o`3oool06`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01@0oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool01@3ooolR0;no_`d0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0L0?oo
o`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0N0?oo
o`030000003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool000d0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool04@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3o
ool01@3ooolQ0;no_`h0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0L0?ooo`03
0000003oool0oooo01`0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0N0?ooo`03
0000003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool000d0oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
1@3ooolP0;no_`h0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0L0?ooo`030000
003oool0oooo01`0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0N0?ooo`030000
003oool0oooo02<0oooo00<000000?ooo`3oool0903oool000d0oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool01@3o
oolO0;no_`d0oooo0P00000K0?ooo`8000007P3oool00`000000oooo0?ooo`0L0?ooo`030000003o
ool0oooo01h0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0S0?ooo`030000003o
ool0oooo02@0oooo000=0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00@0oooo7P2o_kl=0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0
oooo01d0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0
oooo02D0oooo000=0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0?ooo`0<0?ooo`030000003oool0oooo00@0oooo7@2o_kl=0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo
01d0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo
02D0oooo000=0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo00@0oooo702o_kl=0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01d0
oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo02D0
oooo000=0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo00@0oooo6`2o_kl=0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01h0oooo
00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo02H0oooo
000>0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo00@0oooo6P2o_kl=0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool07@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01d0oooo00<0
00000?ooo`3oool0803oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo02H0oooo000>
0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo00@0oooo6@2o_kl<0?ooo`800000703oool00`000000oooo0?ooo`0M
0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0P
0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool09P3oool000h0oooo00<000000?oo
o`3oool0703oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01@0oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool0103ooolH0;no_``0oooo00<000000?ooo`3oool06P3oool2000001l0oooo00<000000?oo
o`3oool07P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0200oooo00<000000?oo
o`3oool08`3oool00`000000oooo0?ooo`0W0?ooo`003P3oool00`000000oooo0?ooo`0L0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`040?oooaL0
_kno303oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3o
ool07P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3o
ool08`3oool00`000000oooo0?ooo`0W0?ooo`003`3oool00`000000oooo0?ooo`0L0?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`040?oooaH0_kno
303oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0
7P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0
8`3oool00`000000oooo0?ooo`0X0?ooo`003`3oool00`000000oooo0?ooo`0L0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`040?oooaD0_kno303o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool07P3o
ool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool08`3o
ool00`000000oooo0?ooo`0X0?ooo`003`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`040?oooa@0_kno303oool0
0`000000oooo0?ooo`0I0?ooo`8000007`3oool200000200oooo00<000000?ooo`3oool07`3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0:03oool0
00l0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool0103ooolC0;no_``0oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`0N0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool07`3oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0:03oool00100
oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool00`3ooolB0;no_`/0oooo0P00000J0?ooo`800000803oool00`000000
oooo0?ooo`0O0?ooo`8000008@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo02@0
oooo00<000000?ooo`3oool0:@3oool00100oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0=0?ooo`800000303oool00`000000oooo0?ooo`030?oooa40_kno2`3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool00`000000oooo
0?ooo`0P0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool09@3oool00`000000oooo
0?ooo`0Y0?ooo`00403oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`3oool0503oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`030?oooa00_kno2`3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`0P0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool09@3oool00`000000oooo0?oo
o`0Y0?ooo`00403oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`030?ooo`l0_kno2`3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0O
0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0Z
0?ooo`00403oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool05@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`030?ooo`h0_kno2`3oool00`000000oooo0?ooo`0I0?oo
o`8000007`3oool200000280oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0R0?oo
o`030000003oool0oooo02D0oooo00<000000?ooo`3oool0:P3oool00140oooo00<000000?ooo`3o
ool07@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool04@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool00`3oool=0;no_`/0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?ooo`03
0000003oool0oooo0240oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0R0?ooo`03
0000003oool0oooo02D0oooo00<000000?ooo`3oool0:`3oool00140oooo00<000000?ooo`3oool0
7@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
0`3oool<0;no_`X0oooo0P00000K0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0
803oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0
9@3oool00`000000oooo0?ooo`0[0?ooo`004@3oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`030?ooo`/0_kno2P3o
ool00`000000oooo0?ooo`0I0?ooo`8000008@3oool00`000000oooo0?ooo`0P0?ooo`030000003o
ool0oooo0240oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0V0?ooo`030000003o
ool0oooo02/0oooo000A0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo00<0oooo2P2o_kl90?ooo`8000006P3oool2
00000240oooo0P00000R0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool08`3oool0
0`000000oooo0?ooo`0U0?ooo`030000003oool0oooo02`0oooo000B0?ooo`030000003oool0oooo
01d0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
0080oooo2@2o_kl90?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0803oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool08P3oool00`00
0000oooo0?ooo`0V0?ooo`030000003oool0oooo02`0oooo000B0?ooo`030000003oool0oooo01d0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0080
oooo202o_kl:0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0803oool00`000000
oooo0?ooo`0Q0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool08`3oool00`000000
oooo0?ooo`0U0?ooo`030000003oool0oooo02d0oooo000B0?ooo`030000003oool0oooo01d0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo
00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0080oooo
1`2o_kl:0?ooo`030000003oool0oooo01L0oooo0P00000Q0?ooo`8000008P3oool2000002@0oooo
00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo02h0oooo
000B0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo0080oooo1P2o_kl90?ooo`8000006@3oool00`000000oooo0?oo
o`0P0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool08`3oool00`000000oooo0?oo
o`0S0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0;P3oool001<0oooo00<00000
0?ooo`3oool07@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool04@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool00P3oool50;no_`T0oooo00<000000?ooo`3oool05`3oool200000240oooo0P00000T
0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0U
0?ooo`030000003oool0oooo02l0oooo000C0?ooo`030000003oool0oooo01d0oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0140oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0080oooo102o_kl80?oo
o`800000603oool200000280oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0R0?oo
o`030000003oool0oooo02@0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0_0?oo
o`004`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool05P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`020?ooo`<0_kno203oool00`000000oooo0?ooo`0F0?ooo`80
00008P3oool2000002@0oooo0P00000T0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3o
ool09P3oool00`000000oooo0?ooo`0`0?ooo`00503oool00`000000oooo0?ooo`0M0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`020?ooo`80_kno
203oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0
8`3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0
9`3oool00`000000oooo0?ooo`0`0?ooo`00503oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`020?ooo`030;no_`3o
ool0oooo00D0oooo0P00000G0?ooo`8000008P3oool2000002D0oooo00<000000?ooo`3oool0903o
ool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0<@3o
ool001D0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0E0?ooo`8000008`3oool0
0`000000oooo0?ooo`0T0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0903oool0
0`000000oooo0?ooo`0W0?ooo`030000003oool0oooo0340oooo000E0?ooo`030000003oool0oooo
01d0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00H0oooo0P00000G0?ooo`8000008P3oool2000002H0oooo00<000000?ooo`3oool0903oool00`00
0000oooo0?ooo`0T0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0<P3oool001D0
oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01H0
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00D0
oooo0P0000040?ooo`8000005`3oool200000280oooo0P00000W0?ooo`030000003oool0oooo02@0
oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo0380
oooo000F0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`0F0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`060?ooo`@000005P3oool300000280oooo0P00000W0?ooo`8000009P3oool00`000000oooo
0?ooo`0U0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0<`3oool001H0oooo00<0
00000?ooo`3oool07P3oool00`000000oooo0?ooo`0I0?ooo`8000005`3oool00`000000oooo0?oo
o`0A0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool07@3oool2000002@0oooo00<0
00000?ooo`3oool09P3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo02D0oooo00<0
00000?ooo`3oool0:03oool00`000000oooo0?ooo`0c0?ooo`005P3oool00`000000oooo0?ooo`0N
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0B
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool06P3oool2000002D0oooo00<00000
0?ooo`3oool09P3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo02D0oooo00<00000
0?ooo`3oool0:03oool00`000000oooo0?ooo`0d0?ooo`005`3oool00`000000oooo0?ooo`0N0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0B0?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool05P3oool3000002D0oooo0P00000W0?oo
o`8000009`3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo02P0oooo00<000000?oo
o`3oool0=03oool001L0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo01H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0=0?ooo`80
0000503oool2000002H0oooo0P00000X0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3o
ool09P3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo03D0oooo000H0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0F0?ooo`030000
003oool0oooo0180oooo00<000000?ooo`3oool03P3oool2000000h0oooo1000000V0?ooo`800000
:03oool2000002P0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0Y0?ooo`030000
003oool0oooo03D0oooo000H0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool03`3o
ool2000000T0oooo1000000Y0?ooo`030000003oool0oooo02H0oooo0P00000Y0?ooo`030000003o
ool0oooo02H0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0e0?ooo`006@3oool0
0`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0100oooo2@00000[0?ooo`800000:03oool0
0`000000oooo0?ooo`0X0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0:@3oool0
0`000000oooo0?ooo`0f0?ooo`006@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo
0440oooo0P00000Y0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool09`3oool00`00
0000oooo0?ooo`0Z0?ooo`030000003oool0oooo03H0oooo000I0?ooo`030000003oool0oooo01h0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool0?P3oool2000002X0oooo00<000000?ooo`3oool0:03oool00`000000
oooo0?ooo`0W0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0=`3oool001T0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0k0?ooo`800000:P3oool2000002X0oooo
00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo03L0oooo
000I0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`0G0?ooo`030000003oool0oooo01<0oooo0P00000h0?ooo`<00000:`3oool00`000000oooo0?oo
o`0Y0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?oo
o`0h0?ooo`006P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool0603oool00`000000oooo0?ooo`0D0?ooo`800000<`3oool3000002`0oooo0P00000[
0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0i
0?ooo`006P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0E0?ooo`800000;`3oool2000002h0oooo00<000000?oo
o`3oool0:P3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo02T0oooo00<000000?oo
o`3oool0>P3oool001/0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0K0?ooo`80
00006@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo02X0oooo0`00000_0?ooo`03
0000003oool0oooo02X0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0Z0?ooo`03
0000003oool0oooo03X0oooo000K0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0
703oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01D0oooo0P00000U0?ooo`D00000
<03oool2000002/0oooo0P00000[0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0
>`3oool001`0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0L0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool05P3oool3000001h0oooo1000000d0?ooo`030000003o
ool0oooo02X0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0Z0?ooo`030000003o
ool0oooo03`0oooo000L0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0703oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo1000000F0?ooo`@00000=P3oool2
000002`0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0
oooo03`0oooo000M0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0703oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo01/0oooo1000000>0?ooo`@00000=`3oool30000
02d0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo
03d0oooo000M0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0703oool00`000000
oooo0?ooo`0I0?ooo`8000007`3oool>000003T0oooo0P00000_0?ooo`030000003oool0oooo02X0
oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0m0?ooo`007P3oool00`000000oooo
0?ooo`0O0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`1Q0?ooo`800000<03oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02`0oooo
00<000000?ooo`3oool0?P3oool001h0oooo00<000000?ooo`3oool0803oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0GP3oool200000300oooo0P00
000/0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0?P3oool001l0oooo00<00000
0?ooo`3oool0803oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool0F`3oool200000300oooo0P00000]0?ooo`030000003oool0oooo02d0oooo00<00000
0?ooo`3oool0?`3oool001l0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0N0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool0F03oool200000340oooo00<000000?oo
o`3oool0;@3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo0400oooo000P0?ooo`03
0000003oool0oooo0200oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo05D0oooo0P00000a0?ooo`800000;`3oool00`000000oooo0?ooo`0]0?ooo`03
0000003oool0oooo0400oooo000P0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0
7@3oool00`000000oooo0?ooo`0K0?ooo`800000D`3oool200000380oooo00<000000?ooo`3oool0
;@3oool2000002l0oooo00<000000?ooo`3oool0@@3oool00240oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`0M0?ooo`8000007@3oool3000004d0oooo0`00000c0?ooo`030000003o
ool0oooo02d0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`120?ooo`008@3oool0
0`000000oooo0?ooo`0R0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool07@3oool0
0`000000oooo0?ooo`180?ooo`800000=03oool2000002l0oooo00<000000?ooo`3oool0;P3oool0
0`000000oooo0?ooo`130?ooo`008@3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo
01h0oooo00<000000?ooo`3oool07P3oool2000004H0oooo0P00000e0?ooo`030000003oool0oooo
02h0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`130?ooo`008P3oool00`000000
oooo0?ooo`0R0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool07`3oool200000400
oooo1000000f0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0;`3oool00`000000
oooo0?ooo`140?ooo`008P3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool0803oool3000003/0oooo0P00000h0?ooo`800000<@3oool00`000000oooo
0?ooo`0_0?ooo`030000003oool0oooo04@0oooo000S0?ooo`030000003oool0oooo0280oooo00<0
00000?ooo`3oool07`3oool00`000000oooo0?ooo`0Q0?ooo`800000=`3oool3000003T0oooo00<0
00000?ooo`3oool0;`3oool200000300oooo0P0000170?ooo`00903oool00`000000oooo0?ooo`0R
0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool08P3oool4000002l0oooo1000000j
0?ooo`800000<@3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo04L0oooo000T0?oo
o`030000003oool0oooo02<0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0U0?oo
o`@000009`3oool4000003`0oooo0P00000b0?ooo`030000003oool0oooo0300oooo00<000000?oo
o`3oool0A`3oool002D0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0O0?ooo`80
0000:@3oool4000001`0oooo1`00000o0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3o
ool0<03oool00`000000oooo0?ooo`180?ooo`009@3oool00`000000oooo0?ooo`0S0?ooo`030000
003oool0oooo0240oooo00<000000?ooo`3oool0:P3oool9000000T0oooo2P0000150?ooo`030000
003oool0oooo0340oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`180?ooo`009P3o
ool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0<P3o
ool9000004`0oooo0`00000c0?ooo`030000003oool0oooo0300oooo0P00001;0?ooo`009P3oool0
0`000000oooo0?ooo`0T0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0Q03oool2
000003D0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`1;0?ooo`009`3oool00`00
0000oooo0?ooo`0T0?ooo`030000003oool0oooo0240oooo0P0000220?ooo`800000=P3oool00`00
0000oooo0?ooo`0a0?ooo`030000003oool0oooo04/0oooo000X0?ooo`030000003oool0oooo02@0
oooo00<000000?ooo`3oool08P3oool2000007h0oooo0P00000g0?ooo`030000003oool0oooo0340
oooo00<000000?ooo`3oool0C03oool002P0oooo00<000000?ooo`3oool09@3oool00`000000oooo
0?ooo`0S0?ooo`030000003oool0oooo07T0oooo0P00000h0?ooo`030000003oool0oooo0340oooo
00<000000?ooo`3oool0C@3oool002T0oooo00<000000?ooo`3oool0903oool00`000000oooo0?oo
o`0T0?ooo`800000M`3oool2000003P0oooo0P00000c0?ooo`030000003oool0oooo04h0oooo000Y
0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`1b
0?ooo`800000>@3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo04l0oooo000Z0?oo
o`030000003oool0oooo02D0oooo00<000000?ooo`3oool09@3oool200000700oooo0P00000i0?oo
o`800000=@3oool00`000000oooo0?ooo`1?0?ooo`00:`3oool00`000000oooo0?ooo`0U0?ooo`03
0000003oool0oooo02H0oooo00<000000?ooo`3oool0J`3oool2000003T0oooo0P00000f0?ooo`03
0000003oool0oooo0500oooo000[0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0
9P3oool00`000000oooo0?ooo`1X0?ooo`800000>P3oool00`000000oooo0?ooo`0e0?ooo`030000
003oool0oooo0540oooo000/0?ooo`030000003oool0oooo02H0oooo0P00000W0?ooo`800000I@3o
ool3000003/0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`1B0?ooo`00;@3oool0
0`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02H0oooo0`00001P0?ooo`800000?@3oool0
0`000000oooo0?ooo`0e0?ooo`030000003oool0oooo05<0oooo000]0?ooo`030000003oool0oooo
02L0oooo00<000000?ooo`3oool0:@3oool2000005/0oooo0`00000m0?ooo`800000=`3oool00`00
0000oooo0?ooo`1D0?ooo`00;P3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02X0
oooo0`00001E0?ooo`<00000?`3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo05D0
oooo000_0?ooo`030000003oool0oooo02L0oooo0P00000]0?ooo`<00000D03oool200000400oooo
0P00000i0?ooo`030000003oool0oooo05D0oooo000`0?ooo`030000003oool0oooo02P0oooo00<0
00000?ooo`3oool0;@3oool2000004T0oooo1@0000110?ooo`030000003oool0oooo03L0oooo0P00
001H0?ooo`00<@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo02h0oooo0`000014
0?ooo`800000A@3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo05P0oooo000b0?oo
o`030000003oool0oooo02P0oooo00<000000?ooo`3oool0<03oool4000003`0oooo100000150?oo
o`800000>@3oool00`000000oooo0?ooo`1I0?ooo`00<`3oool00`000000oooo0?ooo`0X0?ooo`80
0000=03oool4000003<0oooo1@0000180?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3o
ool0FP3oool003<0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0e0?ooo`@00000
:`3oool4000004/0oooo0P00000j0?ooo`030000003oool0oooo05/0oooo000d0?ooo`030000003o
ool0oooo02T0oooo00<000000?ooo`3oool0>@3oool9000001X0oooo2000001>0?ooo`030000003o
ool0oooo03X0oooo00<000000?ooo`3oool0F`3oool003D0oooo00<000000?ooo`3oool0:@3oool2
00000480oooo6P00001E0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0G03oool0
03H0oooo00<000000?ooo`3oool0:P3oool200000:d0oooo0P00000k0?ooo`800000G`3oool003H0
oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`2X0?ooo`800000?03oool00`000000
oooo0?ooo`1O0?ooo`00=`3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo0:@0oooo
0`00000m0?ooo`030000003oool0oooo0600oooo000h0?ooo`030000003oool0oooo02`0oooo0P00
002R0?ooo`800000?`3oool00`000000oooo0?ooo`1Q0?ooo`00>@3oool00`000000oooo0?ooo`0]
0?ooo`030000003oool0oooo09h0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`1R
0?ooo`00>P3oool00`000000oooo0?ooo`0]0?ooo`800000W03oool200000400oooo00<000000?oo
o`3oool0H`3oool003/0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`2G0?ooo`80
0000@03oool2000006H0oooo000l0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0
U03oool200000440oooo00<000000?ooo`3oool0IP3oool003`0oooo00<000000?ooo`3oool0;`3o
ool2000009<0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`1W0?ooo`00?@3oool0
0`000000oooo0?ooo`0`0?ooo`800000S`3oool200000440oooo0P00001Z0?ooo`00?P3oool20000
0380oooo0P0000290?ooo`@00000@P3oool00`000000oooo0?ooo`1Z0?ooo`00@03oool00`000000
oooo0?ooo`0a0?ooo`800000Q03oool3000004D0oooo00<000000?ooo`3oool0J`3oool00400oooo
00<000000?ooo`3oool0<`3oool200000800oooo0P0000180?ooo`030000003oool0oooo06/0oooo
00110?ooo`030000003oool0oooo03@0oooo0`00001j0?ooo`<00000B03oool2000006h0oooo0012
0?ooo`800000=`3oool2000007H0oooo0P0000190?ooo`800000L03oool004@0oooo00<000000?oo
o`3oool0=P3oool00`000000oooo0?ooo`1a0?ooo`800000B`3oool00`000000oooo0?ooo`1_0?oo
o`00A@3oool2000003L0oooo0P00001^0?ooo`<00000B`3oool200000780oooo00170?ooo`030000
003oool0oooo03H0oooo0`00001Y0?ooo`800000C03oool2000007@0oooo00170?ooo`030000003o
ool0oooo03T0oooo0`00001S0?ooo`<00000C@3oool00`000000oooo0?ooo`1d0?ooo`00B03oool0
0`000000oooo0?ooo`0k0?ooo`D00000F@3oool5000004l0oooo00<000000?ooo`3oool0M@3oool0
04T0oooo0P0000100?ooo`800000D`3oool400000580oooo0P00001h0?ooo`00B`3oool00`000000
oooo0?ooo`0o0?ooo`<00000C03oool4000005D0oooo00<000000?ooo`3oool0N03oool004`0oooo
00<000000?ooo`3oool0@@3oool4000004@0oooo1000001G0?ooo`800000N`3oool004d0oooo00<0
00000?ooo`3oool0A03oool4000003`0oooo1000001I0?ooo`800000O@3oool004h0oooo0P000018
0?ooo`T00000:P3oool9000005`0oooo00<000000?ooo`3oool0O@3oool00500oooo00<000000?oo
o`3oool0CP3oool8000001X0oooo2000001S0?ooo`800000P03oool00540oooo0P00001F0?oooaX0
0000JP3oool00`000000oooo0?ooo`200?ooo`00D`3oool00`000000oooo0?ooo`3E0?ooo`800000
P`3oool005@0oooo00<000000?ooo`3oool0d`3oool00`000000oooo0?ooo`230?ooo`00E@3oool2
00000=40oooo0P0000260?ooo`00E`3oool00`000000oooo0?ooo`3<0?ooo`800000R03oool005P0
oooo0P00003:0?ooo`800000RP3oool005X0oooo0P0000360?ooo`800000S03oool005`0oooo00<0
00000?ooo`3oool0`@3oool2000008h0oooo001M0?ooo`800000_`3oool200000900oooo001O0?oo
o`800000^`3oool200000980oooo001Q0?ooo`030000003oool0oooo0;H0oooo0P00002D0?ooo`00
HP3oool200000;@0oooo0P00002F0?ooo`00I03oool00`000000oooo0?ooo`2_0?ooo`800000V03o
ool006D0oooo0P00002]0?ooo`800000VP3oool006L0oooo0P00002Y0?ooo`800000W03oool006T0
oooo0P00002T0?ooo`<00000WP3oool006/0oooo0P00002P0?ooo`800000X@3oool006d0oooo0P00
002L0?ooo`800000X`3oool006l0oooo0`00002E0?ooo`@00000Y@3oool00780oooo0P00002A0?oo
o`800000Z@3oool007@0oooo0`00002;0?ooo`<00000Z`3oool007L0oooo0`0000250?ooo`<00000
[P3oool007X0oooo0P0000210?ooo`800000/@3oool007`0oooo0`00001j0?ooo`@00000/`3oool0
07l0oooo1000001b0?ooo`@00000]`3oool008<0oooo1000001Z0?ooo`@00000^`3oool008L0oooo
1@00001Q0?ooo`@00000_`3oool008`0oooo1000001I0?ooo`@00000``3oool00900oooo1000001@
0?ooo`D00000a`3oool009@0oooo100000140?ooo`P00000c03oool009P0oooo2@00000f0?ooo`D0
0000e03oool00:40oooo2000000W0?ooo`L00000f@3oool00:T0oooo9`00003P0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 431}, {431, 0}} -> {18.7997, 18.7997, 0.217172, \
0.217172}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(PlotGradientField[Evaluate[\(-f[x, y]\)], {x, 30, 70}, {y, 32, 70}, 
        PlotPoints \[Rule] 15, ScaleFactor \[Rule] 5, 
        Epilog \[Rule] {GrayLevel[0.75], Polygon[poly]}];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .95677 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.599815 0.0220362 -0.641797 0.0220362 [
[ 0 0 0 0 ]
[ 1 .95677 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
% Start of user PostScript
/mathtops {
gsave
MBeginOrig
moveto
MEndOrig
currentpoint
grestore
} bind def
/MAtocoords {
mathtops 4 2 roll mathtops
4 copy pop pop
3 -1 roll sub
/arry exch def
exch sub
/arrx exch def
arrx dup mul
arry dup mul
add sqrt
/arrl exch def
translate
} bind def
/MAarrowhead1 {
gsave
MAtocoords
arrl 0. eq
{ 0 0 Mdot }
{
[ arrx arrl div
arry arrl div
-1 arry mul arrl div
arrx arrl div
0 0 ]
concat
-0.02 0.005 moveto 0 0 lineto
-0.02 -0.005 lineto fill
-0.02 0.005 moveto 0 0 lineto
-0.02 -0.005 lineto
-0.02 0.005 lineto stroke
} ifelse
grestore
} def
/MAarrowzero1 {
gsave
mathtops translate
0 0 Mdot
grestore
} def
% End of user PostScript
0 g
.0001 w
[ ] 0 setdash
.06127 .06336 m
.03977 .04438 L
s
% Start of user PostScript
30. 32. 29.024286 31.138503 MAarrowhead1
% End of user PostScript
.06127 .12317 m
.03619 .10491 L
s
% Start of user PostScript
30. 34.714286 28.862058 33.885464 MAarrowhead1
% End of user PostScript
.06127 .18299 m
.03245 .16634 L
s
% Start of user PostScript
30. 37.428571 28.692182 36.673048 MAarrowhead1
% End of user PostScript
.06127 .2428 m
.029 .22883 L
s
% Start of user PostScript
30. 40.142857 28.535526 39.509193 MAarrowhead1
% End of user PostScript
.06127 .30261 m
.02632 .29223 L
s
% Start of user PostScript
30. 42.857143 28.414083 42.386271 MAarrowhead1
% End of user PostScript
.06127 .36242 m
.02471 .35613 L
s
% Start of user PostScript
30. 45.571429 28.340761 45.286047 MAarrowhead1
% End of user PostScript
.06127 .42224 m
.02396 .41988 L
s
% Start of user PostScript
30. 48.285714 28.306788 48.179028 MAarrowhead1
% End of user PostScript
.06127 .48205 m
.02381 .48372 L
s
% Start of user PostScript
30. 51. 28.30002 51.076077 MAarrowhead1
% End of user PostScript
.06127 .54186 m
.02434 .54774 L
s
% Start of user PostScript
30. 53.714286 28.324012 53.981209 MAarrowhead1
% End of user PostScript
.06127 .60167 m
.0257 .61165 L
s
% Start of user PostScript
30. 56.428571 28.385905 56.881342 MAarrowhead1
% End of user PostScript
.06127 .66148 m
.0282 .67563 L
s
% Start of user PostScript
30. 59.142857 28.499316 59.784541 MAarrowhead1
% End of user PostScript
.06127 .7213 m
.03184 .73859 L
s
% Start of user PostScript
30. 61.857143 28.66446 62.642017 MAarrowhead1
% End of user PostScript
.06127 .78111 m
.03608 .80034 L
s
% Start of user PostScript
30. 64.571429 28.856954 65.444123 MAarrowhead1
% End of user PostScript
.06127 .84092 m
.04018 .86079 L
s
% Start of user PostScript
30. 67.285714 29.043116 68.187309 MAarrowhead1
% End of user PostScript
.06127 .90073 m
.04377 .92039 L
s
% Start of user PostScript
30. 70. 29.205778 70.892078 MAarrowhead1
% End of user PostScript
.12423 .06336 m
.1026 .04044 L
s
% Start of user PostScript
32.857143 32. 31.875372 30.959978 MAarrowhead1
% End of user PostScript
.12423 .12317 m
.09816 .10043 L
s
% Start of user PostScript
32.857143 34.714286 31.673947 33.682297 MAarrowhead1
% End of user PostScript
.12423 .18299 m
.09306 .16171 L
s
% Start of user PostScript
32.857143 37.428571 31.442609 36.463148 MAarrowhead1
% End of user PostScript
.12423 .2428 m
.08804 .22494 L
s
% Start of user PostScript
32.857143 40.142857 31.215018 39.332699 MAarrowhead1
% End of user PostScript
.12423 .30261 m
.08451 .2898 L
s
% Start of user PostScript
32.857143 42.857143 31.054719 42.276032 MAarrowhead1
% End of user PostScript
.12423 .36242 m
.08291 .3549 L
s
% Start of user PostScript
32.857143 45.571429 30.981858 45.230264 MAarrowhead1
% End of user PostScript
.12423 .42224 m
.08236 .41959 L
s
% Start of user PostScript
32.857143 48.285714 30.956932 48.16591 MAarrowhead1
% End of user PostScript
.12423 .48205 m
.08222 .4839 L
s
% Start of user PostScript
32.857143 51. 30.950592 51.083863 MAarrowhead1
% End of user PostScript
.12423 .54186 m
.08244 .54844 L
s
% Start of user PostScript
32.857143 53.714286 30.960823 54.012854 MAarrowhead1
% End of user PostScript
.12423 .60167 m
.08349 .61371 L
s
% Start of user PostScript
32.857143 56.428571 31.008533 56.975004 MAarrowhead1
% End of user PostScript
.12423 .66148 m
.08643 .67939 L
s
% Start of user PostScript
32.857143 59.142857 31.141595 59.955461 MAarrowhead1
% End of user PostScript
.12423 .7213 m
.09176 .74381 L
s
% Start of user PostScript
32.857143 61.857143 31.38375 62.878814 MAarrowhead1
% End of user PostScript
.12423 .78111 m
.09796 .80562 L
s
% Start of user PostScript
32.857143 64.571429 31.664852 65.683645 MAarrowhead1
% End of user PostScript
.12423 .84092 m
.1034 .86536 L
s
% Start of user PostScript
32.857143 67.285714 31.911832 68.394704 MAarrowhead1
% End of user PostScript
.12423 .90073 m
.10766 .9241 L
s
% Start of user PostScript
32.857143 70. 32.105341 71.060522 MAarrowhead1
% End of user PostScript
.18719 .06336 m
.16634 .03584 L
s
% Start of user PostScript
35.714286 32. 34.767935 30.751114 MAarrowhead1
% End of user PostScript
.18719 .12317 m
.16094 .09458 L
s
% Start of user PostScript
35.714286 34.714286 34.522939 33.416537 MAarrowhead1
% End of user PostScript
.18719 .18299 m
.15357 .15475 L
s
% Start of user PostScript
35.714286 37.428571 34.188322 36.147247 MAarrowhead1
% End of user PostScript
.18719 .2428 m
.14517 .2189 L
s
% Start of user PostScript
35.714286 40.142857 33.80728 39.058582 MAarrowhead1
% End of user PostScript
.18719 .30261 m
.14035 .28725 L
s
% Start of user PostScript
35.714286 42.857143 33.588485 42.159913 MAarrowhead1
% End of user PostScript
.18719 .36242 m
.13986 .35453 L
s
% Start of user PostScript
35.714286 45.571429 33.566433 45.213342 MAarrowhead1
% End of user PostScript
.18719 .42224 m
.14038 .41957 L
s
% Start of user PostScript
35.714286 48.285714 33.590075 48.164687 MAarrowhead1
% End of user PostScript
.18719 .48205 m
.1404 .48373 L
s
% Start of user PostScript
35.714286 51. 33.590925 51.076227 MAarrowhead1
% End of user PostScript
.18719 .54186 m
.1397 .54834 L
s
% Start of user PostScript
35.714286 53.714286 33.559316 54.008224 MAarrowhead1
% End of user PostScript
.18719 .60167 m
.13909 .61499 L
s
% Start of user PostScript
35.714286 56.428571 33.531455 57.033053 MAarrowhead1
% End of user PostScript
.18719 .66148 m
.14152 .68461 L
s
% Start of user PostScript
35.714286 59.142857 33.641804 60.192498 MAarrowhead1
% End of user PostScript
.18719 .7213 m
.1506 .75243 L
s
% Start of user PostScript
35.714286 61.857143 34.053923 63.26983 MAarrowhead1
% End of user PostScript
.18719 .78111 m
.16084 .81333 L
s
% Start of user PostScript
35.714286 64.571429 34.518562 66.033607 MAarrowhead1
% End of user PostScript
.18719 .84092 m
.168 .87098 L
s
% Start of user PostScript
35.714286 67.285714 34.843473 68.649594 MAarrowhead1
% End of user PostScript
.18719 .90073 m
.17264 .92809 L
s
% Start of user PostScript
35.714286 70. 35.053818 71.241591 MAarrowhead1
% End of user PostScript
.25015 .06336 m
.23159 .03095 L
s
% Start of user PostScript
38.571429 32. 37.728987 30.529301 MAarrowhead1
% End of user PostScript
.25015 .12317 m
.22592 .08735 L
s
% Start of user PostScript
38.571429 34.714286 37.471662 33.0886 MAarrowhead1
% End of user PostScript
.25015 .18299 m
.21632 .14351 L
s
% Start of user PostScript
38.571429 37.428571 37.036224 35.637197 MAarrowhead1
% End of user PostScript
.25015 .2428 m
.19757 .20552 L
s
% Start of user PostScript
38.571429 40.142857 36.185135 38.451078 MAarrowhead1
% End of user PostScript
.25015 .30261 m
.19085 .28759 L
s
% Start of user PostScript
38.571429 42.857143 35.880451 42.175492 MAarrowhead1
% End of user PostScript
.25015 .36242 m
.19624 .35684 L
s
% Start of user PostScript
38.571429 45.571429 36.124851 45.318026 MAarrowhead1
% End of user PostScript
.25015 .42224 m
.19914 .42049 L
s
% Start of user PostScript
38.571429 48.285714 36.256678 48.2064 MAarrowhead1
% End of user PostScript
.25015 .48205 m
.19945 .48302 L
s
% Start of user PostScript
38.571429 51. 36.2707 51.044033 MAarrowhead1
% End of user PostScript
.25015 .54186 m
.19718 .54606 L
s
% Start of user PostScript
38.571429 53.714286 36.167757 53.904801 MAarrowhead1
% End of user PostScript
.25015 .60167 m
.19187 .61212 L
s
% Start of user PostScript
38.571429 56.428571 35.926533 56.90256 MAarrowhead1
% End of user PostScript
.25015 .66148 m
.18634 .6904 L
s
% Start of user PostScript
38.571429 59.142857 35.675583 60.454833 MAarrowhead1
% End of user PostScript
.25015 .7213 m
.20986 .77118 L
s
% Start of user PostScript
38.571429 61.857143 36.743035 64.120792 MAarrowhead1
% End of user PostScript
.25015 .78111 m
.22786 .82401 L
s
% Start of user PostScript
38.571429 64.571429 37.559954 66.518217 MAarrowhead1
% End of user PostScript
.25015 .84092 m
.23532 .8769 L
s
% Start of user PostScript
38.571429 67.285714 37.898514 68.918243 MAarrowhead1
% End of user PostScript
.25015 .90073 m
.23921 .93171 L
s
% Start of user PostScript
38.571429 70. 38.074686 71.405484 MAarrowhead1
% End of user PostScript
.31311 .06336 m
.29855 .02675 L
s
% Start of user PostScript
41.428571 32. 40.767502 30.338558 MAarrowhead1
% End of user PostScript
.31311 .12317 m
.29479 .08097 L
s
% Start of user PostScript
41.428571 34.714286 40.597145 32.799304 MAarrowhead1
% End of user PostScript
.31311 .18299 m
.28961 .13093 L
s
% Start of user PostScript
41.428571 37.428571 40.361807 35.066237 MAarrowhead1
% End of user PostScript
.31311 .2428 m
.29244 .13457 L
s
% Start of user PostScript
41.428571 40.142857 40.490605 35.231623 MAarrowhead1
% End of user PostScript
.31311 .30261 m
.31335 .30317 L
s
% Start of user PostScript
41.428571 42.857143 41.439433 42.882667 MAarrowhead1
% End of user PostScript
.31311 .36242 m
.31331 .36259 L
s
% Start of user PostScript
41.428571 45.571429 41.437311 45.579185 MAarrowhead1
% End of user PostScript
.31311 .42224 m
.31329 .42228 L
s
% Start of user PostScript
41.428571 48.285714 41.436642 48.287925 MAarrowhead1
% End of user PostScript
.31311 .48205 m
.31329 .48202 L
s
% Start of user PostScript
41.428571 51. 41.436577 50.99872 MAarrowhead1
% End of user PostScript
.31311 .54186 m
.3133 .54174 L
s
% Start of user PostScript
41.428571 53.714286 41.437034 53.708826 MAarrowhead1
% End of user PostScript
.31311 .60167 m
.31333 .60133 L
s
% Start of user PostScript
41.428571 56.428571 41.438399 56.413249 MAarrowhead1
% End of user PostScript
.31311 .66148 m
.32032 .66571 L
s
% Start of user PostScript
41.428571 59.142857 41.755504 59.334403 MAarrowhead1
% End of user PostScript
.31311 .7213 m
.30785 .79978 L
s
% Start of user PostScript
41.428571 61.857143 41.189861 65.418631 MAarrowhead1
% End of user PostScript
.31311 .78111 m
.30465 .83188 L
s
% Start of user PostScript
41.428571 64.571429 41.044367 66.875219 MAarrowhead1
% End of user PostScript
.31311 .84092 m
.30601 .88053 L
s
% Start of user PostScript
41.428571 67.285714 41.106452 69.083276 MAarrowhead1
% End of user PostScript
.31311 .90073 m
.30723 .9339 L
s
% Start of user PostScript
41.428571 70. 41.161609 71.504967 MAarrowhead1
% End of user PostScript
.37607 .06336 m
.36666 .0242 L
s
% Start of user PostScript
44.285714 32. 43.858656 30.222707 MAarrowhead1
% End of user PostScript
.37607 .12317 m
.36524 .07832 L
s
% Start of user PostScript
44.285714 34.714286 43.794036 32.678984 MAarrowhead1
% End of user PostScript
.37607 .18299 m
.36552 .13051 L
s
% Start of user PostScript
44.285714 37.428571 43.806829 35.047227 MAarrowhead1
% End of user PostScript
.37607 .2428 m
.37221 .17252 L
s
% Start of user PostScript
44.285714 40.142857 44.110495 36.95354 MAarrowhead1
% End of user PostScript
.37607 .30261 m
.37607 .30261 L
s
% Start of user PostScript
44.285714 42.857143 MAarrowzero1
% End of user PostScript
.37607 .36242 m
.37607 .36242 L
s
% Start of user PostScript
44.285714 45.571429 MAarrowzero1
% End of user PostScript
.37607 .42224 m
.37607 .42224 L
s
% Start of user PostScript
44.285714 48.285714 MAarrowzero1
% End of user PostScript
.37607 .48205 m
.37607 .48205 L
s
% Start of user PostScript
44.285714 51. MAarrowzero1
% End of user PostScript
.37607 .54186 m
.37607 .54186 L
s
% Start of user PostScript
44.285714 53.714286 MAarrowzero1
% End of user PostScript
.37607 .60167 m
.38777 .60939 L
s
% Start of user PostScript
44.285714 56.428571 44.816496 56.77893 MAarrowhead1
% End of user PostScript
.37607 .66148 m
.41036 .71155 L
s
% Start of user PostScript
44.285714 59.142857 45.841731 61.414913 MAarrowhead1
% End of user PostScript
.37607 .7213 m
.39242 .77637 L
s
% Start of user PostScript
44.285714 61.857143 45.027383 64.356133 MAarrowhead1
% End of user PostScript
.37607 .78111 m
.3811 .82774 L
s
% Start of user PostScript
44.285714 64.571429 44.513852 66.687585 MAarrowhead1
% End of user PostScript
.37607 .84092 m
.37701 .87979 L
s
% Start of user PostScript
44.285714 67.285714 44.328289 69.04953 MAarrowhead1
% End of user PostScript
.37607 .90073 m
.37552 .93399 L
s
% Start of user PostScript
44.285714 70. 44.26079 71.509055 MAarrowhead1
% End of user PostScript
.43903 .06336 m
.43453 .02312 L
s
% Start of user PostScript
47.142857 32. 46.938425 30.17371 MAarrowhead1
% End of user PostScript
.43903 .12317 m
.43423 .07804 L
s
% Start of user PostScript
47.142857 34.714286 46.924729 32.666285 MAarrowhead1
% End of user PostScript
.43903 .18299 m
.43504 .13291 L
s
% Start of user PostScript
47.142857 37.428571 46.961529 35.156012 MAarrowhead1
% End of user PostScript
.43903 .2428 m
.43784 .18103 L
s
% Start of user PostScript
47.142857 40.142857 47.088763 37.33967 MAarrowhead1
% End of user PostScript
.43903 .30261 m
.43903 .30261 L
s
% Start of user PostScript
47.142857 42.857143 MAarrowzero1
% End of user PostScript
.43903 .36242 m
.43903 .36242 L
s
% Start of user PostScript
47.142857 45.571429 MAarrowzero1
% End of user PostScript
.43903 .42224 m
.43903 .42224 L
s
% Start of user PostScript
47.142857 48.285714 MAarrowzero1
% End of user PostScript
.43903 .48205 m
.43903 .48205 L
s
% Start of user PostScript
47.142857 51. MAarrowzero1
% End of user PostScript
.43903 .54186 m
.45573 .5492 L
s
% Start of user PostScript
47.142857 53.714286 47.900368 54.047377 MAarrowhead1
% End of user PostScript
.43903 .60167 m
.47013 .63762 L
s
% Start of user PostScript
47.142857 56.428571 48.553861 58.059769 MAarrowhead1
% End of user PostScript
.43903 .66148 m
.46559 .70211 L
s
% Start of user PostScript
47.142857 59.142857 48.347807 60.986301 MAarrowhead1
% End of user PostScript
.43903 .7213 m
.45757 .76365 L
s
% Start of user PostScript
47.142857 61.857143 47.983964 63.778991 MAarrowhead1
% End of user PostScript
.43903 .78111 m
.45023 .82086 L
s
% Start of user PostScript
47.142857 64.571429 47.650954 66.375315 MAarrowhead1
% End of user PostScript
.43903 .84092 m
.44558 .87655 L
s
% Start of user PostScript
47.142857 67.285714 47.439893 68.902394 MAarrowhead1
% End of user PostScript
.43903 .90073 m
.44292 .93243 L
s
% Start of user PostScript
47.142857 70. 47.319432 71.43827 MAarrowhead1
% End of user PostScript
.50199 .06336 m
.50219 .02278 L
s
% Start of user PostScript
50. 32. 50.008967 30.158482 MAarrowhead1
% End of user PostScript
.50199 .12317 m
.50213 .07803 L
s
% Start of user PostScript
50. 34.714286 50.006369 32.665823 MAarrowhead1
% End of user PostScript
.50199 .18299 m
.50205 .13384 L
s
% Start of user PostScript
50. 37.428571 50.002699 35.198313 MAarrowhead1
% End of user PostScript
.50199 .2428 m
.50199 .18316 L
s
% Start of user PostScript
50. 40.142857 49.999797 37.436297 MAarrowhead1
% End of user PostScript
.50199 .30261 m
.50199 .30261 L
s
% Start of user PostScript
50. 42.857143 MAarrowzero1
% End of user PostScript
.50199 .36242 m
.50199 .36242 L
s
% Start of user PostScript
50. 45.571429 MAarrowzero1
% End of user PostScript
.50199 .42224 m
.50199 .42224 L
s
% Start of user PostScript
50. 48.285714 MAarrowzero1
% End of user PostScript
.50199 .48205 m
.51209 .49219 L
s
% Start of user PostScript
50. 51. 50.458128 51.460347 MAarrowhead1
% End of user PostScript
.50199 .54186 m
.53152 .57313 L
s
% Start of user PostScript
50. 53.714286 51.339712 55.133435 MAarrowhead1
% End of user PostScript
.50199 .60167 m
.52975 .63473 L
s
% Start of user PostScript
50. 56.428571 51.259394 57.928521 MAarrowhead1
% End of user PostScript
.50199 .66148 m
.52603 .69636 L
s
% Start of user PostScript
50. 59.142857 51.090733 60.725273 MAarrowhead1
% End of user PostScript
.50199 .7213 m
.52104 .75673 L
s
% Start of user PostScript
50. 61.857143 50.864125 63.464972 MAarrowhead1
% End of user PostScript
.50199 .78111 m
.51603 .8153 L
s
% Start of user PostScript
50. 64.571429 50.637023 66.12301 MAarrowhead1
% End of user PostScript
.50199 .84092 m
.51203 .87288 L
s
% Start of user PostScript
50. 67.285714 50.455295 68.735945 MAarrowhead1
% End of user PostScript
.50199 .90073 m
.50916 .93009 L
s
% Start of user PostScript
50. 70. 50.325167 71.332344 MAarrowhead1
% End of user PostScript
.56495 .06336 m
.56992 .02297 L
s
% Start of user PostScript
52.857143 32. 53.082499 30.166949 MAarrowhead1
% End of user PostScript
.56495 .12317 m
.56997 .07783 L
s
% Start of user PostScript
52.857143 34.714286 53.084892 32.656748 MAarrowhead1
% End of user PostScript
.56495 .18299 m
.56891 .13275 L
s
% Start of user PostScript
52.857143 37.428571 53.036731 35.148858 MAarrowhead1
% End of user PostScript
.56495 .2428 m
.56609 .18108 L
s
% Start of user PostScript
52.857143 40.142857 52.908707 37.342303 MAarrowhead1
% End of user PostScript
.56495 .30261 m
.56495 .30261 L
s
% Start of user PostScript
52.857143 42.857143 MAarrowzero1
% End of user PostScript
.56495 .36242 m
.56495 .36242 L
s
% Start of user PostScript
52.857143 45.571429 MAarrowzero1
% End of user PostScript
.56495 .42224 m
.5799 .44617 L
s
% Start of user PostScript
52.857143 48.285714 53.535508 49.371863 MAarrowhead1
% End of user PostScript
.56495 .48205 m
.59546 .51166 L
s
% Start of user PostScript
52.857143 51. 54.241362 52.343763 MAarrowhead1
% End of user PostScript
.56495 .54186 m
.59389 .57143 L
s
% Start of user PostScript
52.857143 53.714286 54.170149 55.05636 MAarrowhead1
% End of user PostScript
.56495 .60167 m
.59144 .63173 L
s
% Start of user PostScript
52.857143 56.428571 54.059149 57.792537 MAarrowhead1
% End of user PostScript
.56495 .66148 m
.58816 .69212 L
s
% Start of user PostScript
52.857143 59.142857 53.910157 60.533232 MAarrowhead1
% End of user PostScript
.56495 .7213 m
.58434 .75198 L
s
% Start of user PostScript
52.857143 61.857143 53.736906 63.249364 MAarrowhead1
% End of user PostScript
.56495 .78111 m
.5805 .81103 L
s
% Start of user PostScript
52.857143 64.571429 53.562535 65.929232 MAarrowhead1
% End of user PostScript
.56495 .84092 m
.57712 .86947 L
s
% Start of user PostScript
52.857143 67.285714 53.409432 68.58098 MAarrowhead1
% End of user PostScript
.56495 .90073 m
.57445 .92757 L
s
% Start of user PostScript
52.857143 70. 53.287976 71.217695 MAarrowhead1
% End of user PostScript
.62792 .06336 m
.63797 .0239 L
s
% Start of user PostScript
55.714286 32. 56.170578 30.209162 MAarrowhead1
% End of user PostScript
.62792 .12317 m
.63913 .07772 L
s
% Start of user PostScript
55.714286 34.714286 56.223196 32.651782 MAarrowhead1
% End of user PostScript
.62792 .18299 m
.63812 .12992 L
s
% Start of user PostScript
55.714286 37.428571 56.177537 35.020301 MAarrowhead1
% End of user PostScript
.62792 .2428 m
.63127 .17324 L
s
% Start of user PostScript
55.714286 40.142857 55.86634 36.986293 MAarrowhead1
% End of user PostScript
.62792 .30261 m
.62792 .30261 L
s
% Start of user PostScript
55.714286 42.857143 MAarrowzero1
% End of user PostScript
.62792 .36242 m
.65256 .38988 L
s
% Start of user PostScript
55.714286 45.571429 56.832869 46.817645 MAarrowhead1
% End of user PostScript
.62792 .42224 m
.66188 .45226 L
s
% Start of user PostScript
55.714286 48.285714 57.255468 49.648326 MAarrowhead1
% End of user PostScript
.62792 .48205 m
.65958 .50989 L
s
% Start of user PostScript
55.714286 51. 57.151027 52.263326 MAarrowhead1
% End of user PostScript
.62792 .54186 m
.65695 .56893 L
s
% Start of user PostScript
55.714286 53.714286 57.032106 54.942777 MAarrowhead1
% End of user PostScript
.62792 .60167 m
.65409 .62863 L
s
% Start of user PostScript
55.714286 56.428571 56.902131 57.65203 MAarrowhead1
% End of user PostScript
.62792 .66148 m
.65095 .68854 L
s
% Start of user PostScript
55.714286 59.142857 56.759632 60.370495 MAarrowhead1
% End of user PostScript
.62792 .7213 m
.64768 .74821 L
s
% Start of user PostScript
55.714286 61.857143 56.611019 63.078472 MAarrowhead1
% End of user PostScript
.62792 .78111 m
.64443 .80749 L
s
% Start of user PostScript
55.714286 64.571429 56.463872 65.768607 MAarrowhead1
% End of user PostScript
.62792 .84092 m
.64148 .8664 L
s
% Start of user PostScript
55.714286 67.285714 56.329825 68.442077 MAarrowhead1
% End of user PostScript
.62792 .90073 m
.63892 .92505 L
s
% Start of user PostScript
55.714286 70. 56.213507 71.10337 MAarrowhead1
% End of user PostScript
.69088 .06336 m
.7065 .02644 L
s
% Start of user PostScript
58.571429 32. 59.280588 30.324359 MAarrowhead1
% End of user PostScript
.69088 .12317 m
.71023 .07985 L
s
% Start of user PostScript
58.571429 34.714286 59.449812 32.748491 MAarrowhead1
% End of user PostScript
.69088 .18299 m
.71392 .12805 L
s
% Start of user PostScript
58.571429 37.428571 59.617094 34.935497 MAarrowhead1
% End of user PostScript
.69088 .2428 m
.7044 .1493 L
s
% Start of user PostScript
58.571429 40.142857 59.18539 35.899786 MAarrowhead1
% End of user PostScript
.69088 .30261 m
.73752 .33984 L
s
% Start of user PostScript
58.571429 42.857143 60.688313 44.546521 MAarrowhead1
% End of user PostScript
.69088 .36242 m
.7343 .39372 L
s
% Start of user PostScript
58.571429 45.571429 60.542202 46.991531 MAarrowhead1
% End of user PostScript
.69088 .42224 m
.72831 .44859 L
s
% Start of user PostScript
58.571429 48.285714 60.270226 49.481537 MAarrowhead1
% End of user PostScript
.69088 .48205 m
.72393 .50652 L
s
% Start of user PostScript
58.571429 51. 60.071283 52.110646 MAarrowhead1
% End of user PostScript
.69088 .54186 m
.72032 .5657 L
s
% Start of user PostScript
58.571429 53.714286 59.9078 54.796172 MAarrowhead1
% End of user PostScript
.69088 .60167 m
.71707 .62537 L
s
% Start of user PostScript
58.571429 56.428571 59.760122 57.50419 MAarrowhead1
% End of user PostScript
.69088 .66148 m
.71394 .68521 L
s
% Start of user PostScript
58.571429 59.142857 59.618233 60.219445 MAarrowhead1
% End of user PostScript
.69088 .7213 m
.7109 .74491 L
s
% Start of user PostScript
58.571429 61.857143 59.480338 62.928804 MAarrowhead1
% End of user PostScript
.69088 .78111 m
.70801 .80439 L
s
% Start of user PostScript
58.571429 64.571429 59.349023 65.627727 MAarrowhead1
% End of user PostScript
.69088 .84092 m
.70536 .86364 L
s
% Start of user PostScript
58.571429 67.285714 59.228678 68.316617 MAarrowhead1
% End of user PostScript
.69088 .90073 m
.70302 .92266 L
s
% Start of user PostScript
58.571429 70. 59.12273 70.99513 MAarrowhead1
% End of user PostScript
.75384 .06336 m
.77402 .03103 L
s
% Start of user PostScript
61.428571 32. 62.344695 30.532954 MAarrowhead1
% End of user PostScript
.75384 .12317 m
.78065 .08678 L
s
% Start of user PostScript
61.428571 34.714286 62.645269 33.062856 MAarrowhead1
% End of user PostScript
.75384 .18299 m
.79314 .14073 L
s
% Start of user PostScript
61.428571 37.428571 63.21221 35.510853 MAarrowhead1
% End of user PostScript
.75384 .2428 m
.82943 .20008 L
s
% Start of user PostScript
61.428571 40.142857 64.859231 38.204379 MAarrowhead1
% End of user PostScript
.75384 .30261 m
.82253 .31944 L
s
% Start of user PostScript
61.428571 42.857143 64.54571 43.620769 MAarrowhead1
% End of user PostScript
.75384 .36242 m
.8022 .38226 L
s
% Start of user PostScript
61.428571 45.571429 63.623337 46.471488 MAarrowhead1
% End of user PostScript
.75384 .42224 m
.79305 .44208 L
s
% Start of user PostScript
61.428571 48.285714 63.208124 49.186063 MAarrowhead1
% End of user PostScript
.75384 .48205 m
.78747 .50194 L
s
% Start of user PostScript
61.428571 51. 62.954817 51.902611 MAarrowhead1
% End of user PostScript
.75384 .54186 m
.78336 .56194 L
s
% Start of user PostScript
61.428571 53.714286 62.768475 54.625578 MAarrowhead1
% End of user PostScript
.75384 .60167 m
.77994 .622 L
s
% Start of user PostScript
61.428571 56.428571 62.61319 57.351023 MAarrowhead1
% End of user PostScript
.75384 .66148 m
.77686 .68203 L
s
% Start of user PostScript
61.428571 59.142857 62.473267 60.07498 MAarrowhead1
% End of user PostScript
.75384 .7213 m
.77402 .7419 L
s
% Start of user PostScript
61.428571 61.857143 62.344407 62.79221 MAarrowhead1
% End of user PostScript
.75384 .78111 m
.77138 .80161 L
s
% Start of user PostScript
61.428571 64.571429 62.22471 65.501686 MAarrowhead1
% End of user PostScript
.75384 .84092 m
.76894 .86109 L
s
% Start of user PostScript
61.428571 67.285714 62.114147 68.20093 MAarrowhead1
% End of user PostScript
.75384 .90073 m
.76669 .92041 L
s
% Start of user PostScript
61.428571 70. 62.012052 70.892675 MAarrowhead1
% End of user PostScript
.8168 .06336 m
.83943 .03676 L
s
% Start of user PostScript
64.285714 32. 65.312938 30.792979 MAarrowhead1
% End of user PostScript
.8168 .12317 m
.84596 .09579 L
s
% Start of user PostScript
64.285714 34.714286 65.609376 33.471851 MAarrowhead1
% End of user PostScript
.8168 .18299 m
.85559 .15706 L
s
% Start of user PostScript
64.285714 37.428571 66.0462 36.251993 MAarrowhead1
% End of user PostScript
.8168 .2428 m
.86691 .22606 L
s
% Start of user PostScript
64.285714 40.142857 66.560019 39.38333 MAarrowhead1
% End of user PostScript
.8168 .30261 m
.86789 .30246 L
s
% Start of user PostScript
64.285714 42.857143 66.604417 42.850289 MAarrowhead1
% End of user PostScript
.8168 .36242 m
.86086 .37166 L
s
% Start of user PostScript
64.285714 45.571429 66.285244 45.990543 MAarrowhead1
% End of user PostScript
.8168 .42224 m
.85444 .43536 L
s
% Start of user PostScript
64.285714 48.285714 65.993831 48.881371 MAarrowhead1
% End of user PostScript
.8168 .48205 m
.84956 .49714 L
s
% Start of user PostScript
64.285714 51. 65.772735 51.685021 MAarrowhead1
% End of user PostScript
.8168 .54186 m
.84569 .55811 L
s
% Start of user PostScript
64.285714 53.714286 65.59674 54.451871 MAarrowhead1
% End of user PostScript
.8168 .60167 m
.84242 .61867 L
s
% Start of user PostScript
64.285714 56.428571 65.448672 57.199891 MAarrowhead1
% End of user PostScript
.8168 .66148 m
.83953 .67902 L
s
% Start of user PostScript
64.285714 59.142857 65.317185 59.938373 MAarrowhead1
% End of user PostScript
.8168 .7213 m
.83693 .73912 L
s
% Start of user PostScript
64.285714 61.857143 65.199527 62.666071 MAarrowhead1
% End of user PostScript
.8168 .78111 m
.83445 .79905 L
s
% Start of user PostScript
64.285714 64.571429 65.086695 65.385516 MAarrowhead1
% End of user PostScript
.8168 .84092 m
.83218 .85871 L
s
% Start of user PostScript
64.285714 67.285714 64.983905 68.093011 MAarrowhead1
% End of user PostScript
.8168 .90073 m
.83016 .91827 L
s
% Start of user PostScript
64.285714 70. 64.892041 70.795732 MAarrowhead1
% End of user PostScript
.87976 .06336 m
.90276 .04209 L
s
% Start of user PostScript
67.142857 32. 68.186582 31.034807 MAarrowhead1
% End of user PostScript
.87976 .12317 m
.90783 .10288 L
s
% Start of user PostScript
67.142857 34.714286 68.416638 33.793533 MAarrowhead1
% End of user PostScript
.87976 .18299 m
.91366 .1657 L
s
% Start of user PostScript
67.142857 37.428571 68.681506 36.644281 MAarrowhead1
% End of user PostScript
.87976 .2428 m
.91865 .23176 L
s
% Start of user PostScript
67.142857 40.142857 68.907849 39.641982 MAarrowhead1
% End of user PostScript
.87976 .30261 m
.91992 .29975 L
s
% Start of user PostScript
67.142857 42.857143 68.965274 42.727542 MAarrowhead1
% End of user PostScript
.87976 .36242 m
.91759 .36637 L
s
% Start of user PostScript
67.142857 45.571429 68.859707 45.750722 MAarrowhead1
% End of user PostScript
.87976 .42224 m
.91407 .43058 L
s
% Start of user PostScript
67.142857 48.285714 68.700044 48.664221 MAarrowhead1
% End of user PostScript
.87976 .48205 m
.91059 .49312 L
s
% Start of user PostScript
67.142857 51. 68.54219 51.502438 MAarrowhead1
% End of user PostScript
.87976 .54186 m
.90742 .55469 L
s
% Start of user PostScript
67.142857 53.714286 68.398336 54.296721 MAarrowhead1
% End of user PostScript
.87976 .60167 m
.90458 .61562 L
s
% Start of user PostScript
67.142857 56.428571 68.269498 57.06152 MAarrowhead1
% End of user PostScript
.87976 .66148 m
.90199 .67627 L
s
% Start of user PostScript
67.142857 59.142857 68.151829 59.813866 MAarrowhead1
% End of user PostScript
.87976 .7213 m
.89957 .73658 L
s
% Start of user PostScript
67.142857 61.857143 68.042135 62.550723 MAarrowhead1
% End of user PostScript
.87976 .78111 m
.89734 .79662 L
s
% Start of user PostScript
67.142857 64.571429 67.940841 65.275438 MAarrowhead1
% End of user PostScript
.87976 .84092 m
.89526 .85652 L
s
% Start of user PostScript
67.142857 67.285714 67.846256 67.993453 MAarrowhead1
% End of user PostScript
.87976 .90073 m
.89335 .91627 L
s
% Start of user PostScript
67.142857 70. 67.759517 70.704773 MAarrowhead1
% End of user PostScript
.94272 .06336 m
.96493 .04638 L
s
% Start of user PostScript
70. 32. 71.008113 31.229527 MAarrowhead1
% End of user PostScript
.94272 .12317 m
.96867 .10776 L
s
% Start of user PostScript
70. 34.714286 71.177714 34.014741 MAarrowhead1
% End of user PostScript
.94272 .18299 m
.97234 .17041 L
s
% Start of user PostScript
70. 37.428571 71.34419 36.857878 MAarrowhead1
% End of user PostScript
.94272 .2428 m
.97515 .23453 L
s
% Start of user PostScript
70. 40.142857 71.471795 39.767581 MAarrowhead1
% End of user PostScript
.94272 .30261 m
.97619 .29944 L
s
% Start of user PostScript
70. 42.857143 71.519001 42.713398 MAarrowhead1
% End of user PostScript
.94272 .36242 m
.97541 .36403 L
s
% Start of user PostScript
70. 45.571429 71.483631 45.644212 MAarrowhead1
% End of user PostScript
.94272 .42224 m
.97353 .42758 L
s
% Start of user PostScript
70. 48.285714 71.398278 48.528295 MAarrowhead1
% End of user PostScript
.94272 .48205 m
.97121 .4901 L
s
% Start of user PostScript
70. 51. 71.293119 51.36546 MAarrowhead1
% End of user PostScript
.94272 .54186 m
.96879 .55183 L
s
% Start of user PostScript
70. 53.714286 71.183071 54.166937 MAarrowhead1
% End of user PostScript
.94272 .60167 m
.96643 .613 L
s
% Start of user PostScript
70. 56.428571 71.075915 56.942469 MAarrowhead1
% End of user PostScript
.94272 .66148 m
.96419 .6738 L
s
% Start of user PostScript
70. 59.142857 70.974404 59.701904 MAarrowhead1
% End of user PostScript
.94272 .7213 m
.96206 .73424 L
s
% Start of user PostScript
70. 61.857143 70.877548 62.444538 MAarrowhead1
% End of user PostScript
.94272 .78111 m
.96005 .79451 L
s
% Start of user PostScript
70. 64.571429 70.786696 65.179635 MAarrowhead1
% End of user PostScript
.94272 .84092 m
.95815 .85452 L
s
% Start of user PostScript
70. 67.285714 70.700522 67.902589 MAarrowhead1
% End of user PostScript
.94272 .90073 m
.95635 .91437 L
s
% Start of user PostScript
70. 70. 70.61867 70.61867 MAarrowhead1
% End of user PostScript
.75 g
.30367 .26169 m
.72236 .26169 L
.30367 .68037 L
.30367 .26169 L
F
0 0 m
1 0 L
1 .95677 L
0 .95677 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 275.5},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`0001WB000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00^`3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0^03oool0
09h0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`2K0?ooo`00P03oool00`000000
oooo0?ooo`0K0?ooo`030000003oool0oooo01X0oooo0P00000K0?ooo`030000003oool0oooo01T0
oooo0P00000K0?ooo`8000007@3oool00`000000oooo0?ooo`1m0?ooo`00P03oool2000001`0oooo
0P00000K0?ooo`8000006P3oool3000001X0oooo0P00000K0?ooo`800000703oool2000007l0oooo
001S0?ooo`030000003oool0oooo01X0oooo0P00000L0?ooo`8000006`3oool2000001X0oooo0`00
000I0?ooo`<000006P3oool3000001`0oooo0P00000L0?ooo`030000003oool0oooo0600oooo001S
0?ooo`8000006`3oool3000001/0oooo0`00000I0?ooo`@000006@3oool3000001T0oooo1000000I
0?ooo`<000006`3oool3000001/0oooo0P00001R0?ooo`00A`3oool00`000000oooo0?ooo`0J0?oo
o`8000006P3oool4000001T0oooo1000000I0?ooo`@000006@3oool3000001T0oooo1000000H0?oo
o`@000006P3oool3000001/0oooo0P00000L0?ooo`030000003oool0oooo04@0oooo00170?ooo`<0
00006P3oool3000001X0oooo1000000H0?ooo`D00000603oool4000001P0oooo1@00000G0?ooo`D0
0000603oool4000001T0oooo1000000J0?ooo`<000006P3oool2000004L0oooo000/0?ooo`030000
003oool0oooo01T0oooo0`00000I0?ooo`@000006@3oool4000001P0oooo1@00000H0?ooo`D00000
5`3oool5000001L0oooo1@00000G0?ooo`D000006@3oool4000001T0oooo0`00000J0?ooo`<00000
703oool00`000000oooo0?ooo`0X0?ooo`004@3oool00`000000oooo0?ooo`0I0?ooo`8000006@3o
ool4000001T0oooo1000000H0?ooo`D000005`3oool6000001L0oooo1@00000G0?ooo`D000006@3o
ool3000001T0oooo0`00000H0?ooo`D00000603oool4000001T0oooo0`00000K0?ooo`800000:`3o
ool00180oooo0P00000I0?ooo`@00000603oool5000001L0oooo1@00000G0?ooo`@00000603oool4
000001/0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0
oooo01T0oooo00<000000?ooo`3oool06@3oool4000001L0oooo1@00000G0?ooo`D000006P3oool3
000001X0oooo00<000000?ooo`3oool03P3oool00180oooo1000000H0?ooo`D000005P3oool60000
01L0oooo0`00000H0?ooo`<00000703oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool06@3oool3000001L0oooo1@00000I0?ooo`@000006@3oool20000
0140oooo000C0?ooo`D000005P3oool6000001H0oooo1000000H0?ooo`030000003oool0000001/0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0000000H0?ooo`@000005`3oool5000001P0
oooo1000000A0?ooo`004`3oool7000001D0oooo1000000G0?ooo`<000006`3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06P3oool2000001L0oooo
1P00000F0?ooo`D000004P3oool001@0oooo1@00000F0?ooo`<000006@3oool00`000000oooo0000
000K0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0000001P0oooo1000000E0?ooo`H000004`3oool001D0oooo0`00000H
0?ooo`030000003oool0000001/0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I
0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I
0?ooo`@000005P3oool4000001@0oooo000E0?ooo`@000006P3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool010000000oooo0?ooo`00000G
0?ooo`@00000503oool001H0oooo00@000000?ooo`3oool000006P3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I
0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H
0?ooo`040000003oool0oooo000001D0oooo000J0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool0603oool00`000000oooo0?ooo`0G0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00
o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00W@3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo09X0oooo001o0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool06P3oool2000001h0oooo00<000000?ooo`3oool0N`3oool007l0oooo0P00000L0?ooo`800000
703oool2000001X0oooo0P00000K0?ooo`800000703oool2000001d0oooo0P00001m0?ooo`00O`3o
ool2000001`0oooo0P00000K0?ooo`<000006P3oool3000001X0oooo0P00000K0?ooo`<00000703o
ool2000007h0oooo001Q0?ooo`030000003oool0oooo01/0oooo0`00000K0?ooo`<000006P3oool3
000001X0oooo0`00000I0?ooo`<000006`3oool3000001/0oooo0`00000N0?ooo`030000003oool0
oooo05d0oooo001Q0?ooo`800000703oool4000001X0oooo0`00000J0?ooo`@000006@3oool30000
01T0oooo1000000I0?ooo`@000006`3oool3000001`0oooo0P00001P0?ooo`00HP3oool2000001`0
oooo1000000I0?ooo`@000006@3oool4000001P0oooo1000000I0?ooo`@000006@3oool4000001X0
oooo1000000K0?ooo`<00000H03oool004D0oooo00<000000?ooo`3oool06P3oool3000001/0oooo
1000000I0?ooo`@00000603oool6000001L0oooo1@00000G0?ooo`D00000603oool5000001T0oooo
1000000K0?ooo`<00000H@3oool004D0oooo0`00000J0?ooo`@000006P3oool5000001P0oooo1@00
000G0?ooo`H000005`3oool5000001L0oooo1@00000H0?ooo`D00000603oool5000001T0oooo1@00
000M0?ooo`030000003oool0oooo0440oooo00160?ooo`<000006P3oool4000001T0oooo1000000I
0?ooo`<000006@3oool4000001/0oooo00<000000?ooo`3oool06@3oool3000001X0oooo0`00000J
0?ooo`<00000603oool5000001`0oooo0P0000140?ooo`00:P3oool00`000000oooo0?ooo`0I0?oo
o`@000006@3oool5000001P0oooo0`00000M0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`@000006P3oool4000004@0oooo000?0?ooo`03
0000003oool0oooo01T0oooo0P00000J0?ooo`D00000603oool3000001`0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool06P3oool3000001T0oooo1@00000K0?ooo`800000:03oool00100oooo0P00000I0?ooo`@00000
603oool6000001L0oooo00<000000?ooo`000000703oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0K0?ooo`030000
003oool0000001L0oooo1P00000I0?ooo`@00000:@3oool00140oooo0`00000H0?ooo`D000005`3o
ool4000001/0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo01T0oooo1000000G0?ooo`H000006@3oool3000000h0oooo000B0?ooo`@000005`3oool6
000001D0oooo1000000L0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0J0?ooo`@000005@3oool7000001H0oooo1P00000?0?ooo`004P3oool60000
01H0oooo1000000G0?ooo`040000003oool0oooo000001/0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01X0oooo00@000000?ooo`3oool000005`3o
ool5000001@0oooo2000000@0?ooo`004`3oool4000001L0oooo1000000K0?ooo`030000003oool0
oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0
oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool06@3oool4000001H0oooo1P00000A0?ooo`00503oool40000
01L0oooo00<000000?ooo`3oool00P00000J0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool06@3oool010000000oooo0?ooo`00000G0?ooo`D000004P3oool001D0oooo00<000000?oo
o`3oool00P00000J0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`0H0?ooo`8000006@3oool2000000040?ooo`0000000000000001<0oooo000J0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0180oooo003o0?oook40oooo003o0?oooc@0oooo00<000000?ooo`3oool0
NP3oool009d0oooo00<000000?ooo`3oool0L`3oool00`000000oooo0?ooo`0L0?ooo`800000O03o
ool007`0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`1E0?ooo`030000003oool0
oooo01X0oooo0P00000N0?ooo`800000O03oool007`0oooo0`00000N0?ooo`800000703oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool20000
01d0oooo0`00001l0?ooo`00O03oool3000001h0oooo0P00000L0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool06@3oool2000001/0oooo0`00000L0?ooo`@00000O03oool007d0oooo
0`00000M0?ooo`<000006`3oool2000001/0oooo00<000000?ooo`3oool06@3oool2000001/0oooo
0`00000L0?ooo`<00000O@3oool007d0oooo0`00000M0?ooo`<000006`3oool2000001X0oooo0`00
000J0?ooo`8000006P3oool4000001/0oooo1000000R0?ooo`030000003oool0oooo05P0oooo001M
0?ooo`030000003oool0oooo01d0oooo1000000L0?ooo`@000006@3oool3000001X0oooo0`00000I
0?ooo`@000006@3oool4000001/0oooo1000000P0?ooo`800000F`3oool005d0oooo0`00000M0?oo
o`D000006`3oool4000001T0oooo1000000I0?ooo`<000006@3oool4000001P0oooo1@00000J0?oo
o`D000007`3oool3000005/0oooo001N0?ooo`<00000703oool6000001X0oooo1@00000H0?ooo`@0
00006@3oool3000001T0oooo1000000H0?ooo`D000006`3oool4000001h0oooo0`00001L0?ooo`00
GP3oool4000001/0oooo1@00000K0?ooo`D00000603oool5000001L0oooo1@00000G0?ooo`D00000
6P3oool3000001/0oooo00<000000?ooo`3oool07@3oool5000005`0oooo001O0?ooo`D000006@3o
ool5000001d0oooo00<000000?ooo`3oool0603oool5000001L0oooo1@00000G0?ooo`D000006P3o
ool00`000000oooo0?ooo`0J0?ooo`8000007P3oool5000005d0oooo00120?ooo`030000003oool0
oooo01X0oooo1P00000H0?ooo`D000007@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01L0oooo1@00000I0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool2
000001l0oooo1000001M0?ooo`00@`3oool2000001/0oooo1000000L0?ooo`8000007P3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo01/0oooo0P00000N0?ooo`040000003oool00000
000001d0oooo0P00000o0?ooo`00@`3oool4000001T0oooo1000000L0?ooo`8000007P3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo01X0oooo0`00000M0?ooo`050000003oool0oooo
0?ooo`0000006`3oool300000400oooo000X0?ooo`8000006P3oool5000001P0oooo00@000000?oo
o`3oool000006`3oool00`000000oooo0000000M0?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool06P3oool4000001/0oooo00<000000?ooo`3oool06`3oool500000440oooo000Y0?oo
o`<000006@3oool6000001T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0000000M0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool5000001/0oooo00<000000?oo
o`3oool06@3oool600000480oooo000>0?ooo`8000006P3oool5000001L0oooo1000000K0?ooo`03
0000003oool0oooo01P0oooo00@000000?ooo`3oool00000703oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo01T0oooo1@00000J0?ooo`030000003oool0oooo01X0oooo1P00000J0?oo
o`8000009P3oool000l0oooo1@00000G0?ooo`H000005@3oool4000001`0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0000000M0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3o
ool06@3oool6000001T0oooo00<000000?ooo`3oool0703oool4000001L0oooo1@00000W0?ooo`00
403oool7000001@0oooo1@00000G0?ooo`040000003oool0oooo000001`0oooo00<000000?ooo`3o
ool05`3oool010000000oooo0?ooo`00000L0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool06@3oool010000000oooo0?ooo`00000J0?ooo`030000003oool0oooo01/0oooo1@00000E
0?ooo`L000005`3oool5000000`0oooo000A0?ooo`D000005P3oool4000001/0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0F0?ooo`040000003oool0oooo000001/0oooo0P00000K
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I
0?ooo`8000006P3oool00`000000oooo0000000J0?ooo`030000003oool0oooo01/0oooo00D00000
0?ooo`3oool0oooo0000000F0?ooo`H000004`3oool9000000d0oooo000B0?ooo`@000005`3oool2
000000030?ooo`000000000001X0oooo0P00000K0?ooo`030000003oool0oooo01D0oooo00D00000
0?ooo`3oool0oooo0000000J0?ooo`8000006`3oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool06@3oool2000001T0oooo00@000000?ooo`3oool00000
6@3oool00`000000oooo0?ooo`0J0?ooo`8000006`3oool4000001D0oooo1`00000?0?ooo`004`3o
ool2000000030?ooo`000000000001H0oooo00D000000?ooo`3oool0oooo0000000K0?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool05@3oool01@000000oooo0?ooo`3oool0000001T0
oooo1000000J0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0H0?ooo`<000006@3oool010000000oooo0?ooo`00000H0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool06@3oool2000000030?ooo`000000000001H0oooo1P00000@0?oo
o`00503oool010000000oooo0?ooo`3oool2000001T0oooo0P00000J0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
01H0oooo1000000I0?ooo`8000006P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
01P0oooo0`00000H0?ooo`050000003oool0oooo0?ooo`0000005`3oool00`000000oooo0?ooo`0I
0?ooo`8000006P3oool2000000<0oooo00<000000?ooo`3oool04`3oool2000000040?ooo`000000
000000000180oooo000J0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01@0oooo00D000000?ooo`3oool0oooo0000
000H0?ooo`@000006@3oool2000001X0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0H0?ooo`<00000603oool01@000000oooo0?ooo`3oool0000001H0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`0H0?ooo`8000006@3oool2000000<0oooo00<000000?ooo`3oool0
4P3oool00880oooo00<000000?ooo`3oool06P3oool4000001T0oooo0P00000J0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool0603oool4000001/0oooo00<000000?ooo`3oool0P03o
ool00880oooo00<000000?ooo`3oool06@3oool5000001P0oooo0`00000J0?ooo`030000003oool0
oooo01L0oooo0`00000I0?ooo`@000006`3oool00`000000oooo0?ooo`200?ooo`00PP3oool00`00
0000oooo0?ooo`0I0?ooo`H000005`3oool3000001T0oooo0`00000H0?ooo`<00000603oool50000
01X0oooo00<000000?ooo`3oool0P@3oool00880oooo00<000000?ooo`3oool06@3oool6000001L0
oooo1000000H0?ooo`<00000603oool3000001P0oooo1@00000J0?ooo`030000003oool0oooo0840
oooo00220?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool05`3oool4000001P0oooo
0`00000H0?ooo`<000006P3oool3000001X0oooo00<000000?ooo`3oool0P@3oool008<0oooo00<0
00000?ooo`3oool06`3oool00`000000oooo0?ooo`0F0?ooo`D00000603oool3000001L0oooo1@00
000I0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0P@3oool008<0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`0F0?ooo`H000005P3oool5000001H0oooo1@00000I
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0=@3oool2000004X0oooo00230?oo
o`030000003oool0oooo01/0oooo00<000000?ooo`3oool05P3oool6000001H0oooo1@00000F0?oo
o`D000006@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0380oooo1000001;0?oo
o`00P`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3o
ool05`3oool5000001P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo02l0oooo1P00001<0?ooo`00E@3oool2000002`0oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0
;P3oool7000004d0oooo001F0?ooo`<00000:P3oool00`000000oooo0?ooo`0K0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0^0?ooo`H00000CP3o
ool005H0oooo1P00000X0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo02h0oooo1@00001?0?ooo`00E`3oool70000
02H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool0;03oool2000000040?ooo`000000000000000500oooo001H0?oo
o`D000009`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`0Z0?ooo`800000103oool00`000000oooo0?ooo`1?0?oo
o`00F@3oool4000002L0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`1F0?ooo`00
?P3oool3000001P0oooo0`0000000`3oool000000?ooo`0U0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo02P0oooo
0P00001I0?ooo`00?`3oool6000001D0oooo00@000000?ooo`3oool0oooo0P00000T0?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000
003oool0oooo02H0oooo0P00001K0?ooo`009@3oool3000001P0oooo2000000H0?ooo`030000003o
ool0oooo0280oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`1K0?ooo`009P3oool6
000001D0oooo1P00000J0?ooo`8000008P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0S0?ooo`8000006P3oool:
000003X0oooo000<0?ooo`D000005`3oool7000001@0oooo1000000L0?ooo`030000003oool0oooo
01l0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool08@3oool2000001`0oooo2@00000k0?ooo`003@3oool9000001<0
oooo1@00000F0?ooo`80000000<0oooo0000003oool06`3oool00`000000oooo0?ooo`0N0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01l0oooo0P00000O0?ooo`H000003`3oool:000002@0oooo000?0?ooo`H00000
5@3oool4000001L0oooo00<000000?ooo`3oool00P00000K0?ooo`8000007P3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`0N0?ooo`030000003oool0oooo01d0oooo1P00000A0?ooo`T00000403oool:000000/0oooo
000@0?ooo`D000005P3oool2000000030?ooo`000000000001X0oooo0P00000K0?ooo`030000003o
ool0oooo01/0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool07@3oool2000001d0oooo0`0000030?ooo`030000003o
ool0oooo0180oooo1P00000B0?ooo`T00000303oool00140oooo0`0000000`3oool000000000000E
0?ooo`040000003oool0oooo0?ooo`8000006P3oool2000001X0oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603o
ool00`000000oooo0?ooo`0K0?ooo`800000703oool3000001P0oooo2000000C0?ooo`L000003P3o
ool001<0oooo00@000000?ooo`3oool0oooo0P00000I0?ooo`8000006P3oool2000001T0oooo0P00
000K0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo01T0oooo0P00000K0?ooo`<00000603oool3000000@0oooo0`00
000A0?ooo`T000003`3oool001T0oooo0P00000I0?ooo`8000006P3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I
0?ooo`8000006@3oool2000000L0oooo00<000000?ooo`3oool03`3oool2000000@0oooo0`00000A
0?ooo`00o`3ooonM0?ooo`030000003oool0oooo0140oooo003o0?oook40oooo003o0?oook40oooo
003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo
00220?oook@0_knoNP3oool00880oooo/`2o_kmk0?ooo`00PP3ooonb0;no_g`0oooo00220?oook40
_knoO@3oool00880oooo/02o_kmn0?ooo`00PP3ooon_0;no_gl0oooo00220?ooojh0_knoP03oool0
0880oooo[@2o_kn10?ooo`00PP3ooon/0;no_h80oooo00220?oooj/0_knoP`3oool003`0oooo1@00
000A0?ooo`X000009@3oool00`000000_kno0;no_`2X0;no_h@0oooo000T0?ooo`D00000503oool9
000000d0oooo2@00000V0?ooojT0_knoQ@3oool000/0oooo1@00000E0?ooo`T000004@3oool70000
00l0oooo1P00000W0?ooojP0_knoQP3oool000`0oooo2@00000B0?ooo`H000004`3oool600000140
oooo2000000S0?ooojL0_knoQ`3oool000h0oooo1`00000C0?ooo`H00000503oool3000000040?oo
o`000000000000000100oooo0P0000040?ooo`@000007`3ooonV0;no_dD0oooo0P00000D0?ooo`D0
00004`3oool500000100oooo000?0?ooo`L000004`3oool300000080oooo0`00000C0?ooo`030000
003oool0oooo0080oooo0`00000G0?ooo`<00000703ooonU0;no_dH0oooo1@00000A0?ooo`X00000
3P3oool:000000/0oooo000A0?ooo`<000000P3oool300000180oooo00<000000?ooo`3oool00`3o
ool3000001P0oooo0`00000G0?ooo`@00000603ooonT0;no_dL0oooo2000000;0?ooo`/000003`3o
ool9000000d0oooo000C0?ooo`030000003oool0oooo00<0oooo0P00000I0?ooo`8000006@3oool2
000001T0oooo0P00000F0?oooj<0_kno103oool00`000000oooo0?ooo`0H0?ooo`<000006@3ooolG
000000@0oooo1@0000030?ooo`D000003P3oool400000080oooo1@00000?0?ooo`00PP3ooonR0;no
_`H0oooo00<000000?ooo`3oool06P3oool400000280oooo1`00000?0?ooo`8000005`3oool20000
0180oooo00220?oooj40_kno203oool2000001h0oooo1000000N0?ooo`<00000@03oool00880oooo
X02o_kl;0?ooo`030000003oool0oooo01l0oooo100000070?ooo`030000003oool0oooo05<0oooo
00220?oooil0_kno3@3oool00`000000oooo0?ooo`0R0?ooo`D000000P3oool3000005<0oooo0022
0?oooih0_kno3`3oool00`000000oooo0?ooo`0V0?ooo`L00000D@3oool00880ooooW@2o_klA0?oo
o`8000009`3oool8000004l0oooo00220?oooi`0_kno503oool00`000000oooo0?ooo`0T0?ooo`X0
0000C@3oool00880ooooV`2o_klF0?ooo`030000003oool0oooo07X0oooo00220?oooiX0_kno603o
ool01@000000oooo0?ooo`3oool0000007L0oooo00220?oooiT0_kno6P3oool5000007H0oooo0022
0?oooiP0_kno703oool4000007H0oooo00220?oooiL0_kno703oool6000007D0oooo00220?oooiH0
_kno7P3oool6000007@0oooo00220?oooiD0_kno8@3oool5000007<0oooo00220?oooi@0_kno903o
ool3000007<0oooo00220?oooi<0_kno9`3oool200000780oooo00220?oooi80_knoW03oool00880
ooooT@2o_knM0?ooo`00P@3oool00`000000_kno0;no_`2>0;no_ih0oooo00200?ooo`800000S`2o
_knO0?ooo`00@@3oool5000003T0oooo0`00002>0;no_j00oooo000?0?ooo`D000003`3oool:0000
00l0oooo2P00000C0?ooo`D000008P3oool2000008d0_knoX@3oool000X0oooo2P00000@0?ooo`T0
0000403oool9000000h0oooo2P00000T0?oooh`0_knoXP3oool000`0oooo2P00000@0?ooo``00000
3@3oool6000000030?ooo`000000000000D000002@3oool@000001`0ooooR`2o_kmf0?ooo`800000
5P3oool300000180oooo000>0?ooo`D000000`3oool5000000d0oooo100000060?ooo`@000002`3o
ool4000000P0oooo100000070?ooo`D000002@3oool6000001H0ooooRP2o_kl20?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool0603oool2000001X0oooo0`00000H0?ooo`@00000103o
ool4000000l0oooo3000000>0?ooo`004@3oool2000001L0oooo0P00000G0?ooo`8000005P3oool2
000002D0ooooR@2o_kl40?ooo`030000003oool0oooo01P0oooo0P00000J0?ooo`8000006`3oool4
000000<0oooo0P00000C0?ooo`/000004@3oool:000000/0oooo00220?ooohP0_kno1P3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo0P00000M0?ooo`L00000503oool:0000
00l0oooo1`00000>0?ooo`00PP3ooon70;no_`P0oooo00<000000?ooo`3oool06@3oool00`000000
oooo0?ooo`0J0?ooo`<000007@3oool700000140oooo1@00000D0?ooo`<000004P3oool00880oooo
QP2o_kl:0?ooo`040000003oool0oooo000001P0oooo0P00000M0?ooo`8000000`3oool00`000000
oooo0?ooo`0E0?ooo`T00000?@3oool00880ooooQ@2o_kl<0?ooo`<000006P3oool00`000000oooo
0?ooo`0L0?ooo`H000005@3oool500000440oooo00220?oooh@0_kno303oool5000001X0oooo00<0
00000?ooo`3oool07@3oool5000005X0oooo00220?oooh<0_kno303oool6000001/0oooo0P000002
0?ooo`030000003oool0oooo01L0oooo1`00001I0?ooo`00PP3ooon20;no_`h0oooo1P00000L0?oo
o`@00000603oool9000005L0oooo00220?oooh40_kno4@3oool4000001d0oooo1000000L0?ooo`D0
0000EP3oool00880ooooP02o_klC0?ooo`@000006`3oool6000007H0oooo00220?ooogl0_kno5P3o
ool2000001/0oooo1P00001f0?ooo`00PP3ooomn0;no_aT0oooo00<000000?ooo`3oool06P3oool5
000007D0oooo00220?ooogd0_kno>@3oool4000007@0oooo00220?ooog`0_kno?@3oool2000007<0
oooo00220?ooog/0_kno/`3oool00880ooooNP2o_knd0?ooo`00PP3ooomi0;no_kD0oooo00220?oo
ogP0_kno]P3oool00880ooooM`2o_kng0?ooo`00P@3oool00`000000_kno0;no_`1d0;no_kP0oooo
00200?ooo`800000M@2o_kni0?ooo`00O`3oool3000007@0_kno^P3oool002X0oooo0`00000F0?oo
o`<00000>P3oool2000007<0_kno^`3oool000l0oooo1@00000B0?ooo`L000004P3oool7000001@0
oooo1@00000S0?ooog80_kno_03oool000X0oooo2P00000?0?ooo`X000003`3oool:000000l0oooo
2`00000R0?ooog40_knoZ03oool2000001<0oooo000<0?ooo`l000002P3ooolA000000P0oooo1`00
00000`3oool0000000000009000000H0oooo200000000`3oool000000000000:000001H0ooooL02o
_kl00`3oool000000?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603o
ool00`000000oooo0?ooo`0I0?ooo`8000006@3oool2000000D0oooo00<000000?ooo`3oool04@3o
ool4000000030?ooo`0000000000008000004@3oool000h0oooo1@00000D0?ooo`D00000503oool5
000001D0oooo1@00000S0?ooofl0_kno0`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool0603oool2000001/0oooo0P0000040?ooo`030000003oool0
oooo0180oooo2000000E0?ooo`P000003P3oool00140oooo0P00000G0?ooo`8000005`3oool20000
01P0oooo0P00000S0?ooofh0_kno103oool010000000oooo0?ooo`00000H0?ooo`030000003oool0
oooo01P0oooo0P00000J0?ooo`8000006`3oool3000000040?ooo`0000000000000001H0oooo1P00
000C0?ooo`X00000303oool00880ooooK@2o_kl60?ooo`<000006@3oool2000001X0oooo00<00000
0?ooo`3oool06@3oool2000000@0oooo00<000000?ooo`3oool05@3oool5000001@0oooo2@00000A
0?ooo`D000004@3oool00880ooooK02o_kl50?ooo`D000006`3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01X0oooo0P000000103oool0000000000000000F0?ooo`L000004P3oool:
000002H0oooo00220?ooof/0_kno1`3oool5000001/0oooo00<000000?ooo`3oool0603oool20000
00<0oooo00<000000?ooo`3oool05P3oool5000001D0oooo2@0000100?ooo`00PP3ooomZ0;no_`T0
oooo1000000L0?ooo`040000003oool0oooo000001P0oooo00<000000?ooo`0000000P00000G0?oo
o`H000006@3oool5000003l0oooo00220?ooofT0_kno2`3oool3000001d0oooo1000000H0?ooo`@0
00005`3oool7000005`0oooo00220?ooofP0_kno3@3oool2000001d0oooo1000000G0?ooo`H00000
603oool6000005/0oooo00220?ooofL0_kno3`3oool2000001/0oooo1P00000G0?ooo`H000006`3o
ool3000005X0oooo00220?ooofH0_kno4@3oool00`000000oooo0?ooo`0J0?ooo`H00000603oool5
000007L0oooo00220?ooofD0_kno<@3oool5000001T0oooo0`00001g0?ooo`00PP3ooomT0;no_c@0
oooo0`00000K0?ooo`800000MP3oool00880ooooH`2o_klg0?ooo`800000TP3oool00880ooooHP2o
_ko<0?ooo`00PP3ooomQ0;no_ld0oooo00220?ooof00_knocP3oool00880ooooG`2o_ko?0?ooo`00
PP3ooomN0;no_m00oooo00220?oooed0_knod@3oool00880ooooG02o_koB0?ooo`00PP3ooomK0;no
_m<0oooo00220?oooeX0_knoe03oool00880ooooF@2o_koE0?ooo`004@3oool2000001L0oooo0P00
000H0?ooo`8000005`3oool2000001l0oooo0`0000000`3oool0_kno0;no_`1F0;no_mH0oooo000?
0?ooo`@000005@3oool4000001D0oooo1@00000C0?ooo`H000007`3oool4000005L0_kno00<00000
0?ooo`3oool0_P3oool00`000000oooo0?ooo`0C0?ooo`00303oool?000000X0oooo1`0000000`3o
ool0000000000007000000P0oooo200000000`3oool0000000000008000000D0oooo5P00000B0?oo
o`@00000EP2o_kl3000001P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool2000001T0oooo0P0000030?oo
o`030000003oool0oooo01<0oooo0P0000020?ooo`<000004`3oool000X0oooo2P00000?0?ooo`X0
00003`3oool;00000140oooo1`00000O0?ooo`@00000E@2o_kl4000001T0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0H0?ooo`8000006@3oool2000001/0oooo0P0000030?ooo`03
0000003oool0oooo01<0oooo0P000000103oool0000000000000000E0?ooo`L000004@3oool000l0
oooo1@00000B0?ooo`L000004P3oool7000001H0oooo0`00000O0?ooo`<0000000<0oooo0;no_`2o
_kl0DP2o_kl20?ooo`@000006@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool0603oool2000000<0oooo00<000000?ooo`3oool05@3oool6000001D0
oooo1@00000E0?ooo`P000003`3oool002X0oooo0`00000F0?ooo`<00000?03ooomC0;no_`D0oooo
0P00000J0?ooo`030000003oool0oooo01P0oooo0P00000J0?ooo`030000003oool0oooo01T0oooo
00<000000?ooo`0000000P00000H0?ooo`D00000503oool7000001<0oooo2P00000=0?ooo`00PP3o
oomB0;no_`P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01P0oooo0P0000020?ooo`030000003oool0oooo01D0oooo1@00000F0?ooo`L000004`3o
ool9000002P0oooo00220?oooe40_kno9@3oool010000000oooo0?ooo`00000H0?ooo`040000003o
ool0oooo000001P0oooo1000000F0?ooo`H000005@3oool9000001H0oooo1@00000W0?ooo`00PP3o
oom@0;no_bL0oooo1000000H0?ooo`@000005`3oool5000001D0oooo1`00000I0?ooo`D00000@@3o
ool00880ooooC`2o_klX0?ooo`@00000603oool4000001H0oooo1P00000G0?ooo`H00000GP3oool0
0880ooooCP2o_klX0?ooo`H000005P3oool6000001L0oooo1@00000I0?ooo`@00000G@3oool00880
ooooC@2o_klZ0?ooo`D000005`3oool6000001P0oooo1000000K0?ooo`800000G03oool00880oooo
C02o_kl]0?ooo`@00000603oool5000001X0oooo0P00001h0?ooo`00PP3ooom;0;no_bl0oooo1000
000I0?ooo`<00000U03oool00880ooooBP2o_klb0?ooo`8000006`3oool2000009<0oooo00220?oo
odT0_kno=@3oool00`000000oooo0?ooo`2]0?ooo`00PP3ooom80;no_nH0oooo00220?ooodL0_kno
i`3oool00880ooooAP2o_koX0?ooo`00PP3ooom50;no_nT0oooo00220?oood@0_knojP3oool00880
oooo@`2o_ko[0?ooo`00PP3ooom20;no_n`0oooo00220?oood40_knok@3oool00880oooo@02o_ko^
0?ooo`00PP3ooolo0;no_nl0oooo00220?oooch0_knol03oool00140oooo0P00000H0?ooo`030000
003oool0oooo01H0oooo00<000000?ooo`3oool05@3oool2000002@0oooo?@2o_kl00`000000oooo
0?ooo`3H0?ooo`030000003oool0oooo01<0oooo000?0?ooo`@00000103oool4000000h0oooo0`00
00070?ooo`<00000303oool3000000P0oooo100000090?ooo`@00000203oool6000001H0oooo?02o
_kl4000001H0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01P0oooo0P0000030?ooo`030000003oool0oooo01<0oooo0P0000000`3oool000000000
0002000001<0oooo000<0?ooo`/00000403oool5000000030?ooo`000000000000@000003@3oool5
00000080oooo1P00000:0?ooo`l00000703ooolk0;no_`H000005P3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H
0?ooo`8000006P3oool200000080oooo00<000000?ooo`3oool05@3oool6000001D0oooo1P00000B
0?ooo`002P3oool:00000140oooo2000000A0?ooo`T000003P3oool9000002@0oooo>P2o_kl20?oo
o`H000005P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool0603oool2000001X0oooo0P0000020?ooo`030000003oool0oooo01D0oooo1000000H0?oo
o`D00000503oool700000140oooo000?0?ooo`D000003`3oool:000000l0oooo2P00000C0?ooo`D0
00008`3oool00`000000_kno0;no_`0g0;no_`L0oooo0`00000F0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool0603oool2000001X0oooo00D000000?ooo`3oool0oooo0000000G0?oo
o`@000005`3oool4000001H0oooo1`00000C0?ooo`T000003`3oool00440oooo1@00000i0?ooo`<0
0000>02o_klR0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool010000000
oooo0?ooo`00000G0?ooo`030000003oool00000008000005`3oool4000001D0oooo1`00000D0?oo
o`T000005P3oool5000000h0oooo00200?ooo`800000=`2o_klT0?ooo`030000003oool0oooo01P0
oooo00@000000?ooo`3oool00000603oool4000001L0oooo1000000F0?ooo`H000005P3oool60000
01P0oooo1@00000X0?ooo`00P@3oool00`000000_kno0;no_`0d0;no_bH0oooo00<000000?ooo`00
0000603oool4000001L0oooo1000000F0?ooo`H000005`3oool5000001P0oooo100000140?ooo`00
PP3ooole0;no_bL0oooo1000000G0?ooo`@000005P3oool6000001H0oooo1P00000H0?ooo`@00000
6P3oool2000004<0oooo00220?oooc@0_kno9`3oool5000001H0oooo1P00000F0?ooo`H000005`3o
ool5000001X0oooo0P00001N0?ooo`00PP3ooolc0;no_bT0oooo1@00000F0?ooo`D00000603oool5
000001P0oooo0`00001j0?ooo`00PP3ooolb0;no_b`0oooo1000000G0?ooo`@000006@3oool30000
01X0oooo0P00001i0?ooo`00PP3ooola0;no_bh0oooo1000000H0?ooo`8000006`3oool2000009@0
oooo00220?oooc00_kno<@3oool2000001X0oooo00<000000?ooo`3oool0[P3oool00880oooo;`2o
_kld0?ooo`030000003oool0oooo0<P0oooo00220?ooobh0_knoo`3oool10?ooo`00PP3oool]0;no
_ol0oooo0P3oool00880oooo;02o_koo0?ooo`<0oooo00220?ooob/0_knoo`3oool40?ooo`00PP3o
oolZ0;no_ol0oooo1@3oool00880oooo:@2o_koo0?ooo`H0oooo00220?ooobP0_knoo`3oool70?oo
o`00PP3ooolW0;no_ol0oooo203oool00880oooo9P2o_koo0?ooo`T0oooo00220?ooobD0_knoo`3o
ool:0?ooo`00PP3ooolT0;no_ol0oooo2`3oool00180oooo00<000000?ooo`3oool0103oool20000
01T0oooo0P00000I0?ooo`800000603oool3000001H0oooo8`2o_klH0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
01P0oooo00D000000?ooo`3oool0oooo0000000F0?ooo`040000003oool0oooo000001H0oooo000@
0?ooo`<000000`3oool300000180oooo00<000000?ooo`3oool00`3oool3000001L0oooo1000000;
0?ooo`030000003oool0oooo00L0oooo1@00000I0?ooob80_kno00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool2
00000080oooo00<000000?ooo`3oool0503oool5000001H0oooo1@00000D0?ooo`003`3oool70000
01<0oooo0`0000020?ooo`<000004P3oool00`000000oooo0?ooo`020?ooo`<000003@3oool30000
00@0oooo1@00000N0?ooob40_kno0`00000I0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool0603oool200000080oooo00<000000?ooo`3oool05P3oool4000001L0oooo1000000F0?oo
o`D000004`3oool000h0oooo1P00000D0?ooo`H000004`3oool7000000h0oooo2@00000S0?ooob00
_kno00<0oooo0000000000000P00000H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00D000000?ooo`3o
ool0oooo0000000G0?ooo`@000005`3oool5000001D0oooo1P00000D0?ooo`L000004P3oool000`0
oooo2@00000B0?ooo`H000004P3oool500000100oooo2000000T0?ooo`8000007`2o_kl50?ooo`80
0000603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3o
ool06@3oool010000000oooo0?ooo`00000G0?ooo`030000003oool00000008000005P3oool50000
01H0oooo1P00000D0?ooo`H00000503oool900000100oooo000;0?ooo`X00000403oool800000100
oooo2000000=0?ooo`X000008`3oool3000001h0_kno8@3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool3000001P0oooo1000000F0?ooo`H0
0000503oool7000001H0oooo1@00000H0?ooo`D000003`3oool002@0oooo1@00000B0?ooo`X00000
4P3oool5000002@0oooo0P00000M0;no_b<0oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0000001P0oooo1000000G0?ooo`D00000503oool7000001H0oooo
1P00000G0?ooo`@00000:`3oool00840oooo00<000000;no_`2o_kl06P2o_klU0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`0000005`3oool4000001H0oooo1@00000F0?ooo`H000005P3o
ool6000001P0oooo1000000I0?ooo`800000:P3oool00880oooo6`2o_klV0?ooo`040000003oool0
oooo000001L0oooo0`00000F0?ooo`D000005`3oool5000001H0oooo1P00000H0?ooo`@000006P3o
ool2000004@0oooo00220?oooaX0_kno:03oool3000001L0oooo1000000F0?ooo`D00000603oool3
000001P0oooo1@00000J0?ooo`800000G`3oool00880oooo6@2o_klY0?ooo`@000005@3oool50000
01P0oooo0`00000I0?ooo`<000006@3oool3000007/0oooo00220?oooaP0_kno:@3oool5000001H0
oooo1@00000H0?ooo`<000006@3oool2000001/0oooo0P00001j0?ooo`00PP3ooolG0;no_b/0oooo
1@00000G0?ooo`<000006@3oool2000001/0oooo00<000000?ooo`3oool0U03oool00880oooo5P2o
_kl^0?ooo`<00000603oool3000001X0oooo00<000000?ooo`3oool0[`3oool00880oooo5@2o_kl`
0?ooo`<00000603oool200000<`0oooo00220?oooa@0_kno<P3oool2000001X0oooo00<000000?oo
o`3oool0b@3oool00880oooo4`2o_kle0?ooo`030000003oool0oooo0><0oooo00220?oooa80_kno
o`3ooolM0?ooo`00PP3ooolA0;no_ol0oooo7P3oool00880oooo402o_koo0?oooal0oooo00220?oo
o`l0_knoo`3ooolP0?ooo`00PP3oool>0;no_ol0oooo8@3oool00880oooo3@2o_koo0?ooob80oooo
00220?ooo``0_knoo`3ooolS0?ooo`00PP3oool;0;no_ol0oooo903oool00840oooo00<000000;no
_`2o_kl0202o_koo0?ooo`h0oooo00<000000?ooo`3oool0503oool001T0oooo0P00000J0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool05`3oool2000001D0oooo00<000000;no_`2o
_kl01`2o_klG0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool010000000
oooo0?ooo`00000G0?ooo`030000003oool00000008000005@3oool001<0oooo00@000000?ooo`3o
ool0oooo0P00000J0?ooo`8000006@3oool2000001P0oooo0P00000H0?ooo`P0_kno6@3oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0I0?ooo`040000003oool0oooo000001L0oooo1000000G0?ooo`D00000503oool0
0140oooo0`0000000`3oool000000000000E0?ooo`030000003oool0oooo0080oooo0P00000I0?oo
o`800000603oool2000001X0oooo1`2o_kl2000001P0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00@000000?ooo`3oool00000603oool4
000001L0oooo1000000F0?ooo`H000004`3oool00100oooo1@00000F0?ooo`8000000P3oool20000
01T0oooo0P00000H0?ooo`800000703oool60;no_a`0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0I0?ooo`040000003oool0oooo000001L0oooo1000000G0?ooo`@000005P3oool6
000001@0oooo1`00000C0?ooo`003`3oool6000001D0oooo1@00000D0?ooo`030000003oool0oooo
0080oooo0P00000H0?ooo`8000007P3oool50;no_ah0oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool01000
0000oooo0?ooo`00000H0?ooo`@000005P3oool4000001H0oooo1P00000E0?ooo`H000005P3oool6
00000180oooo000=0?ooo`P000004`3oool6000001@0oooo0P0000020?ooo`800000603oool20000
0200oooo102o_klO0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0000001P0oooo0`00000H0?ooo`@000005@3oool60000
01H0oooo1P00000F0?ooo`D00000603oool400000140oooo000<0?ooo`D000005P3oool7000001<0
oooo1@00000H0?ooo`8000008P3oool30;no_b40oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo0P00000H0?ooo`@000005P3oool6000001D0
oooo1P00000G0?ooo`D000005`3oool4000001X0oooo0P00000@0?ooo`009P3oool6000001@0oooo
1@00000B0?ooo`030000003oool0oooo0080oooo0P00000T0?ooo`80_kno8P3oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`0000005`3oool4000001H0oooo
1@00000G0?ooo`D000005`3oool5000001P0oooo0`00000J0?ooo`800000:`3oool002D0oooo0`00
000G0?ooo`L00000403oool200000080oooo0P00001;0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool0603oool3000001H0oooo1@00000G0?ooo`D00000603oool4000001P0oooo0`00
000J0?ooo`800000AP3oool003h0oooo1P00000A0?ooo`D00000CP3oool00`000000oooo0?ooo`0G
0?ooo`040000003oool0oooo000001H0oooo1@00000F0?ooo`D00000603oool3000001X0oooo0P00
000J0?ooo`800000H@3oool003d0oooo0`00000C0?ooo`H00000C`3oool00`000000oooo0?ooo`0H
0?ooo`<000005`3oool4000001P0oooo0`00000I0?ooo`<000006`3oool00`000000oooo0?ooo`1j
0?ooo`00DP3oool8000004l0oooo00<000000?ooo`3oool05`3oool4000001L0oooo0`00000I0?oo
o`<000006@3oool2000009P0oooo001A0?ooo`H00000D`3oool00`000000oooo0?ooo`0E0?ooo`D0
0000603oool3000001T0oooo0P00000K0?ooo`030000003oool0oooo09D0oooo001@0?ooo`<00000
E`3oool00`000000oooo0?ooo`0F0?ooo`@000006@3oool2000001/0oooo00<000000?ooo`3oool0
/03oool00:/0oooo00<000000?ooo`0000005P3oool4000001T0oooo0P00003=0?ooo`00Z`3oool3
000001L0oooo0`00000J0?ooo`030000003oool0oooo0</0oooo002Z0?ooo`D000005`3oool20000
0>P0oooo002[0?ooo`@00000603oool200000>L0oooo002/0?ooo`<000006@3oool00`000000oooo
0?ooo`3U0?ooo`00[@3oool300000?l0oooo0@3oool00:h0oooo0P00003o0?ooo`40oooo002_0?oo
o`800000o`3oool00;00oooo00<000000?ooo`3oool0o@3oool00?l0oooo/@3oool00?l0oooo/@3o
ool00?l0ooooV@3oool00`000000oooo0?ooo`0E0?ooo`006P3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool010000000oooo0?ooo`00000G
0?ooo`040000003oool0oooo000001L0oooo1000000F0?ooo`00503oool010000000oooo0?ooo`3o
ool2000001T0oooo0P00000J0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603o
ool00`000000oooo0?ooo`0H0?ooo`050000003oool0oooo0?ooo`0000005`3oool4000001L0oooo
1000000G0?ooo`@000005@3oool001<0oooo0P0000000`3oool000000000000J0?ooo`030000003o
ool0oooo01P0oooo0P00000J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00@000000?ooo`3oool00000
5`3oool010000000oooo00000000000G0?ooo`@000005`3oool4000001H0oooo1P00000D0?ooo`00
4P3oool4000001L0oooo00D000000?ooo`3oool0oooo0000000J0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0
oooo00@000000?ooo`3oool00000603oool3000001L0oooo1@00000F0?ooo`D000005@3oool60000
01D0oooo1P00000D0?ooo`00403oool6000001H0oooo0P0000000`3oool000000000000J0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01T0oooo0`00000H0?ooo`@000005@3oool6000001D0oooo1P00000F0?ooo`H0
00005P3oool5000001<0oooo000?0?ooo`P00000503oool4000001/0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00@000000?ooo`3oool0
00005`3oool4000001H0oooo1@00000F0?ooo`H000005@3oool6000001L0oooo1@00000G0?ooo`@0
00004P3oool000h0oooo1P00000F0?ooo`D000005`3oool010000000oooo0?ooo`00000L0?ooo`03
0000003oool0oooo01d0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool3000001H0oooo1@00000G0?ooo`D0
00005`3oool4000001L0oooo1@00000H0?ooo`<000006P3oool200000140oooo000=0?ooo`<00000
6P3oool6000001D0oooo1000000M0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`000000
5P3oool5000001L0oooo1@00000H0?ooo`<00000603oool4000001P0oooo0`00000J0?ooo`800000
;03oool002T0oooo1@00000F0?ooo`@000007@3oool00`000000oooo0?ooo`0N0?ooo`030000003o
ool0oooo01/0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0H0?ooo`<000005`3o
ool5000001L0oooo1000000I0?ooo`<000006@3oool2000001X0oooo0P0000170?ooo`00:03oool4
000001L0oooo1@00000L0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool07@3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01H0oooo1@00000H0?ooo`@00000603oool4
000001T0oooo0P00000K0?ooo`030000003oool0oooo0600oooo000W0?ooo`8000006P3oool60000
01X0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo
01T0oooo0`00000G0?ooo`D00000603oool3000001X0oooo0P00000K0?ooo`030000003oool0oooo
07/0oooo00120?ooo`D000006`3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo01d0
oooo00<000000?ooo`3oool05`3oool5000001P0oooo1000000I0?ooo`8000006`3oool2000009P0
oooo00110?ooo`@000007@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo01d0oooo
00<000000?ooo`3oool0603oool4000001T0oooo0`00000J0?ooo`8000006`3oool00`000000oooo
0?ooo`2F0?ooo`00@03oool2000001l0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`0N0?ooo`030000003oool0oooo01P0oooo1000000I0?ooo`8000006`3oool00`000000oooo0?oo
o`2b0?ooo`00GP3oool00`000000oooo0000000T0?ooo`030000003oool0oooo01h0oooo00<00000
0?ooo`3oool06@3oool3000001X0oooo0P00003?0?ooo`00G@3oool4000002@0oooo00<000000?oo
o`3oool07P3oool3000001T0oooo0`00000K0?ooo`030000003oool0oooo0<d0oooo001M0?ooo`D0
00008`3oool00`000000oooo0?ooo`0L0?ooo`D000006P3oool200000>/0oooo001L0?ooo`D00000
903oool00`000000oooo0?ooo`0M0?ooo`@000006`3oool200000>X0oooo001L0?ooo`@000009@3o
ool00`000000oooo0?ooo`0M0?ooo`@00000703oool00`000000oooo0?ooo`3X0?ooo`00F`3oool4
000002H0oooo00<000000?ooo`3oool07P3oool300000?l0oooo203oool005/0oooo0P00000X0?oo
o`030000003oool0oooo01h0oooo1000003o0?ooo`L0oooo001J0?ooo`800000:@3oool00`000000
oooo0?ooo`0O0?ooo`<00000o`3oool70?ooo`00FP3oool00`000000oooo0?ooo`0X0?ooo`030000
003oool0oooo0200oooo0P00003o0?ooo`L0oooo00250?ooo`030000003oool0oooo0200oooo0P00
003o0?ooo`L0oooo00250?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0o`3oool5
0?ooo`00P`3oool500000?D0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0E0?oo
o`006P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0E0?ooo`D000006P3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`04
0000003oool0oooo000001L0oooo1000000G0?ooo`@000005P3oool001D0oooo00<000000?ooo`3o
ool00P00000J0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0F0?ooo`@000006`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool0603oool010000000oooo0?ooo`00000H0?ooo`<000005`3oool40000
01L0oooo1000000F0?ooo`00503oool2000000030?ooo`000000oooo01X0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01H0oooo1000000K0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0000000H0?ooo`<0
0000603oool4000001D0oooo1P00000E0?ooo`H000005@3oool001<0oooo1000000H0?ooo`040000
003oool0oooo000001/0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0G0?ooo`@0
00006`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0000001P0oooo0`00000H0?ooo`@0
00005P3oool5000001H0oooo1P00000E0?ooo`H00000503oool00180oooo1@00000G0?ooo`@00000
6`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo0P00000L0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`040000
003oool0oooo000001L0oooo0`00000G0?ooo`D000005P3oool5000001L0oooo1@00000G0?ooo`D0
00005P3oool5000001<0oooo000B0?ooo`H000005P3oool4000001X0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`0I0?ooo`800000703oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool3000001H0oooo1@00000G0?ooo`@00000
5`3oool5000001P0oooo0`00000I0?ooo`<00000603oool3000001<0oooo000A0?ooo`D000005`3o
ool6000001H0oooo00<000000?ooo`000000703oool00`000000oooo0?ooo`0I0?ooo`8000007@3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00@000000?ooo`0000000000
5P3oool5000001L0oooo1000000H0?ooo`@00000603oool3000001T0oooo0`00000J0?ooo`800000
6@3oool200000180oooo000@0?ooo`@00000603oool6000001H0oooo1000000K0?ooo`030000003o
ool0oooo01X0oooo0P00000M0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool05`3o
ool5000001L0oooo1@00000G0?ooo`<000006@3oool3000001T0oooo0`00000I0?ooo`800000B@3o
ool000l0oooo0P00000J0?ooo`D00000603oool5000001X0oooo00<000000?ooo`3oool06P3oool2
000001d0oooo00<000000?ooo`3oool06@3oool3000001P0oooo1000000H0?ooo`@00000603oool3
000001X0oooo0P00000I0?ooo`8000006`3oool00`000000oooo0?ooo`160?ooo`00:`3oool30000
01T0oooo1@00000J0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool06`3oool00`00
0000oooo0?ooo`0G0?ooo`D00000603oool4000001T0oooo0`00000I0?ooo`8000006`3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo0640oooo000Z0?ooo`8000006`3oool4000001T0
oooo00<000000?ooo`0000007P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01P0
oooo1000000I0?ooo`<000006@3oool3000001X0oooo0P00000K0?ooo`030000003oool0oooo07`0
oooo00160?ooo`@000006P3oool3000001d0oooo00<000000?ooo`3oool0703oool3000001P0oooo
1000000I0?ooo`@000006@3oool3000001X0oooo00<000000?ooo`3oool0V03oool004H0oooo0P00
000K0?ooo`H000006`3oool00`000000oooo0?ooo`0J0?ooo`D000006@3oool3000001X0oooo0`00
000J0?ooo`800000]@3oool004D0oooo0P00000L0?ooo`D000006P3oool3000001`0oooo1@00000I
0?ooo`@000006P3oool2000001/0oooo00<000000?ooo`3oool0/`3oool004D0oooo00<000000?oo
o`3oool06`3oool4000001/0oooo1@00000K0?ooo`@000006P3oool3000001X0oooo0P00003A0?oo
o`00H`3oool3000001`0oooo1@00000K0?ooo`@000006`3oool2000001/0oooo00<000000?ooo`3o
ool0c`3oool00680oooo0`00000M0?ooo`@00000703oool3000001`0oooo0P00003]0?ooo`00HP3o
ool2000001h0oooo1000000M0?ooo`8000007@3oool00`000000oooo0?ooo`3[0?ooo`00HP3oool0
0`000000oooo0?ooo`0N0?ooo`8000007P3oool200000?l0oooo303oool008<0oooo0P00000O0?oo
o`030000003oool0oooo0?l0oooo2P3oool008<0oooo0P00000O0?ooo`030000003oool0oooo0?l0
oooo2P3oool008<0oooo00<000000?ooo`3oool0o`3oool[0?ooo`00P`3oool00`000000oooo0?oo
o`3o0?ooob/0oooo003o0?oook40oooo003o0?oooiP0oooo00<000000?ooo`3oool05P3oool00?l0
ooooH`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01H0oooo0`00000G0?ooo`00
6P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00@000000?ooo`3oool0
0000603oool3000001P0oooo1000000G0?ooo`<000005`3oool001H0oooo00@000000?ooo`3oool0
00006P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool0603oool010000000oooo0?ooo`00000G0?ooo`<00000603oool4000001L0oooo1000000F0?oo
o`D000005P3oool001D0oooo1000000G0?ooo`040000003oool0oooo000001/0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool0603oool010000000oooo0?ooo`00000G0?ooo`@000005`3oool4000001H0oooo1@00000F
0?ooo`H000005P3oool5000001D0oooo000E0?ooo`@000005P3oool4000001/0oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00@000000?oo
o`3oool00000603oool3000001H0oooo1@00000F0?ooo`D000005`3oool5000001H0oooo1@00000H
0?ooo`@00000503oool001@0oooo1P00000E0?ooo`@00000603oool010000000oooo0?ooo`00000J
0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool3000001H0oooo1@00000G
0?ooo`D000005P3oool5000001P0oooo0`00000H0?ooo`@000006@3oool2000001@0oooo000C0?oo
o`H000005@3oool6000001L0oooo0`00000K0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo0`00000F0?oo
o`D000005`3oool5000001L0oooo1000000H0?ooo`<000006@3oool3000001T0oooo0P00000K0?oo
o`030000003oool0oooo0140oooo000B0?ooo`D000005`3oool5000001L0oooo1@00000G0?ooo`04
0000003oool0oooo000001/0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I0?oo
o`<000005P3oool6000001L0oooo1000000H0?ooo`@00000603oool3000001T0oooo0`00000I0?oo
o`8000006`3oool00`000000oooo0?ooo`0/0?ooo`004P3oool3000001P0oooo1000000I0?ooo`@0
0000603oool3000001`0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0G0?ooo`D0
00005`3oool5000001L0oooo1000000I0?ooo`<000006@3oool2000001X0oooo0P00000K0?ooo`03
0000003oool0oooo04L0oooo000A0?ooo`8000006P3oool3000001X0oooo0`00000I0?ooo`D00000
603oool3000001X0oooo1@00000G0?ooo`D000005`3oool5000001P0oooo1000000H0?ooo`<00000
6P3oool2000001/0oooo00<000000?ooo`3oool0HP3oool002`0oooo0`00000J0?ooo`<000006P3o
ool4000001T0oooo1@00000H0?ooo`D00000603oool4000001P0oooo1000000I0?ooo`<000006@3o
ool3000001X0oooo00<000000?ooo`3oool0OP3oool002`0oooo00<000000?ooo`3oool06P3oool3
000001X0oooo1000000I0?ooo`@000006@3oool4000001T0oooo1000000I0?ooo`<000006P3oool2
000001X0oooo0P00002K0?ooo`00B@3oool2000001X0oooo1000000J0?ooo`@000006P3oool30000
01X0oooo0`00000J0?ooo`8000006P3oool2000001/0oooo00<000000?ooo`3oool0V@3oool004P0
oooo0P00000K0?ooo`<000006`3oool3000001/0oooo0`00000J0?ooo`<000006P3oool2000001/0
oooo00<000000?ooo`3oool0]@3oool004P0oooo00<000000?ooo`3oool06P3oool2000001`0oooo
0`00000K0?ooo`<000006`3oool2000001/0oooo00<000000?ooo`3oool0d@3oool006D0oooo0P00
000L0?ooo`800000703oool2000001`0oooo0P00003_0?ooo`00I@3oool00`000000oooo0?ooo`0K
0?ooo`8000007@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0>d0oooo00230?oo
o`030000003oool0oooo01`0oooo00<000000?ooo`3oool0o`3oool<0?ooo`00P`3oool00`000000
oooo0?ooo`3o0?ooob/0oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oo
ok40oooo003o0?oook40oooo003o0?oook40oooo003o0?ooogd0oooo00<000000?ooo`3oool0603o
ool00`000000oooo0?ooo`0F0?ooo`006P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
oooo01P0oooo00@000000?ooo`3oool00000603oool00`000000oooo0000000H0?ooo`8000006@3o
ool3000001L0oooo000F0?ooo`040000003oool0oooo000001X0oooo00<000000?ooo`3oool0603o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00@000000?ooo`3oool00000
603oool3000001P0oooo0`00000G0?ooo`@000005`3oool4000001L0oooo000E0?ooo`@00000603o
ool010000000oooo0?ooo`00000J0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`000000
603oool3000001L0oooo1@00000E0?ooo`H000005@3oool5000001H0oooo1P00000F0?ooo`005@3o
ool5000001L0oooo0`00000J0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0000001P0oooo0`00000F0?ooo`D000005P3o
ool6000001H0oooo1@00000F0?ooo`D000005P3oool6000001D0oooo000D0?ooo`L000005@3oool5
000001L0oooo00<000000?ooo`0000006`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`<00000603oool3
000001H0oooo1@00000G0?ooo`D000005P3oool5000001L0oooo1000000H0?ooo`<00000603oool5
000001@0oooo000D0?ooo`H000005P3oool6000001H0oooo0`00000H0?ooo`@000006P3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo01P0oooo0`00000G0?ooo`D000005P3oool50000
01L0oooo1@00000G0?ooo`@000005`3oool5000001L0oooo1000000H0?ooo`<000006@3oool30000
01@0oooo000C0?ooo`D000005`3oool6000001H0oooo1P00000F0?ooo`H000005P3oool3000001T0
oooo1P00000F0?ooo`D000005`3oool5000001L0oooo1000000H0?ooo`@00000603oool3000001P0
oooo1000000I0?ooo`8000006@3oool2000001/0oooo0P00000C0?ooo`004`3oool4000001P0oooo
1000000H0?ooo`D000005`3oool5000001L0oooo1P00000F0?ooo`H000005P3oool5000001P0oooo
1000000G0?ooo`@000006@3oool3000001P0oooo0`00000J0?ooo`8000006P3oool2000001X0oooo
00<000000?ooo`3oool0;@3oool00180oooo0`00000J0?ooo`<000006@3oool4000001P0oooo1@00
000G0?ooo`D000005`3oool5000001P0oooo1000000H0?ooo`@00000603oool4000001P0oooo0`00
000I0?ooo`<000006P3oool2000001X0oooo00<000000?ooo`3oool0B03oool00180oooo00<00000
0?ooo`3oool06@3oool3000001X0oooo0`00000I0?ooo`@00000603oool5000001P0oooo1000000H
0?ooo`@000006@3oool3000001T0oooo0`00000I0?ooo`<000006@3oool2000001/0oooo00<00000
0?ooo`3oool0H`3oool002h0oooo00<000000?ooo`3oool06@3oool3000001X0oooo0`00000I0?oo
o`@000006@3oool4000001P0oooo0`00000J0?ooo`<000006P3oool2000001X0oooo0P00000J0?oo
o`030000003oool0oooo07l0oooo001:0?ooo`8000006`3oool3000001X0oooo0`00000I0?ooo`<0
00006P3oool2000001/0oooo0P00000J0?ooo`8000006`3oool00`000000oooo0?ooo`2J0?ooo`00
BP3oool00`000000oooo0?ooo`0J0?ooo`8000006`3oool2000001X0oooo0`00000J0?ooo`800000
6`3oool2000001/0oooo00<000000?ooo`3oool0]P3oool006L0oooo0P00000K0?ooo`8000006`3o
ool2000001/0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`3B0?ooo`00I`3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`3_0?ooo`00Q03oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
0?l0oooo3@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool0
0?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool0
0001\
\>"],
  ImageRangeCache->{{{0, 431}, {412.25, 0}} -> {27.2128, 29.1245, 0.157981, \
0.157981}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(DisplayTogether[\[IndentingNewLine]ContourPlot[
          f[x, y], {x, 22, 80}, {y, 22, 80}, PlotPoints \[Rule] 100, 
          AspectRatio \[Rule] Automatic, ContourShading \[Rule] False, 
          Contours \[Rule] Range[0.35, 0.95, 0.1], Frame \[Rule] False, 
          Epilog \[Rule] {GrayLevel[0.75], 
              Polygon[poly]}], \[IndentingNewLine]PlotGradientField[
          Evaluate[\(-f[x, y]\)], {x, 22, 80}, {y, 22, 80}, 
          PlotPoints \[Rule] 15, ScaleFactor \[Rule] 5]];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.292369 0.0155913 -0.292369 0.0155913 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of user PostScript
/mathtops {
gsave
MBeginOrig
moveto
MEndOrig
currentpoint
grestore
} bind def
/MAtocoords {
mathtops 4 2 roll mathtops
4 copy pop pop
3 -1 roll sub
/arry exch def
exch sub
/arrx exch def
arrx dup mul
arry dup mul
add sqrt
/arrl exch def
translate
} bind def
/MAarrowhead1 {
gsave
MAtocoords
arrl 0. eq
{ 0 0 Mdot }
{
[ arrx arrl div
arry arrl div
-1 arry mul arrl div
arrx arrl div
0 0 ]
concat
-0.02 0.005 moveto 0 0 lineto
-0.02 -0.005 lineto fill
-0.02 0.005 moveto 0 0 lineto
-0.02 -0.005 lineto
-0.02 0.005 lineto stroke
} ifelse
grestore
} def
/MAarrowzero1 {
gsave
mathtops translate
0 0 Mdot
grestore
} def
% End of user PostScript
0 g
.5 Mabswid
[ ] 0 setdash
.37034 .33338 m
.37947 .33235 L
.38861 .33147 L
.39774 .33074 L
.40688 .33013 L
.41601 .32962 L
.42515 .32919 L
.43428 .32883 L
.44341 .32854 L
.45255 .3283 L
.46168 .32813 L
.47082 .32801 L
.47995 .32794 L
.48909 .32793 L
.49822 .32797 L
.50735 .32805 L
.51649 .32819 L
.52562 .32838 L
.53476 .32863 L
.54389 .32893 L
.55303 .32929 L
.56216 .32973 L
.57129 .33023 L
.58043 .33082 L
.58956 .3315 L
.5987 .3323 L
.60783 .33323 L
.61283 .3338 L
.61697 .33417 L
.6261 .33517 L
.63523 .33446 L
.64437 .33868 L
.64846 .34294 L
.6511 .35207 L
.64437 .36044 L
.64371 .36121 L
.63739 .37034 L
.63523 .37314 L
.63004 .37947 L
.6261 .38472 L
.62328 .38861 L
.61697 .39605 L
.61554 .39774 L
.60807 .40688 L
.60783 .40716 L
.60003 .41601 L
.5987 .41758 L
.59213 .42515 L
.58956 .42798 L
.58396 .43428 L
Mistroke
.58043 .43828 L
.5757 .44341 L
.57129 .4482 L
.56741 .45255 L
.56216 .45821 L
.5588 .46168 L
.55303 .46786 L
.55028 .47082 L
.54389 .47746 L
.54147 .47995 L
.53476 .48698 L
.5327 .48909 L
.52562 .49628 L
.52376 .49822 L
.51649 .50561 L
.5147 .50735 L
.50735 .5147 L
.50561 .51649 L
.49822 .52376 L
.49628 .52562 L
.48909 .5327 L
.48698 .53476 L
.47995 .54147 L
.47746 .54389 L
.47082 .55028 L
.46786 .55303 L
.46168 .5588 L
.45821 .56216 L
.45255 .56741 L
.4482 .57129 L
.44341 .5757 L
.43828 .58043 L
.43428 .58396 L
.42798 .58956 L
.42515 .59213 L
.41758 .5987 L
.41601 .60003 L
.40716 .60783 L
.40688 .60807 L
.39774 .61554 L
.39605 .61697 L
.38861 .62328 L
.38472 .6261 L
.37947 .63004 L
.37314 .63523 L
.37034 .63739 L
.36121 .64371 L
.36044 .64437 L
.35207 .6511 L
.34294 .64846 L
Mistroke
.33868 .64437 L
.33446 .63523 L
.33517 .6261 L
.33417 .61697 L
.3338 .61283 L
.33323 .60783 L
.3323 .5987 L
.3315 .58956 L
.33082 .58043 L
.33023 .57129 L
.32973 .56216 L
.32929 .55303 L
.32893 .54389 L
.32863 .53476 L
.32838 .52562 L
.32819 .51649 L
.32805 .50735 L
.32797 .49822 L
.32793 .48909 L
.32794 .47995 L
.32801 .47082 L
.32813 .46168 L
.3283 .45255 L
.32854 .44341 L
.32883 .43428 L
.32919 .42515 L
.32962 .41601 L
.33013 .40688 L
.33074 .39774 L
.33147 .38861 L
.33235 .37947 L
.33338 .37034 L
.3338 .36706 L
.33434 .36121 L
.33524 .35207 L
.33875 .34294 L
.34294 .33875 L
.35207 .33524 L
.36121 .33434 L
.36706 .3338 L
.37034 .33338 L
Mfstroke
.39774 .29707 m
.40688 .29546 L
.41601 .29409 L
.42515 .29294 L
.43428 .29195 L
.44341 .29115 L
.45255 .29052 L
.46168 .29004 L
.47082 .28971 L
.47995 .28953 L
.48909 .2895 L
.49822 .28961 L
.50735 .28986 L
.51649 .29026 L
.52562 .2908 L
.53476 .29149 L
.54389 .29235 L
.55303 .29337 L
.56216 .29458 L
.57129 .29599 L
.57856 .29727 L
.58043 .29762 L
.58956 .29948 L
.5987 .30162 L
.60783 .3041 L
.61527 .3064 L
.61697 .30697 L
.6261 .31035 L
.63523 .31442 L
.63746 .31553 L
.64437 .31964 L
.6508 .32467 L
.6535 .32733 L
.65873 .3338 L
.66264 .34108 L
.6631 .34294 L
.66473 .35207 L
.66434 .36121 L
.66287 .37034 L
.66264 .37139 L
.66039 .37947 L
.65724 .38861 L
.6535 .3976 L
.65344 .39774 L
.64911 .40688 L
.64437 .41591 L
.64431 .41601 L
.6391 .42515 L
.63523 .43153 L
.63352 .43428 L
Mistroke
.62761 .44341 L
.6261 .44566 L
.62137 .45255 L
.61697 .45876 L
.61484 .46168 L
.60803 .47082 L
.60783 .47108 L
.60094 .47995 L
.5987 .48277 L
.59358 .48909 L
.58956 .49394 L
.58595 .49822 L
.58043 .50465 L
.57807 .50735 L
.57129 .51496 L
.56991 .51649 L
.56216 .5249 L
.56149 .52562 L
.55303 .5345 L
.55278 .53476 L
.54389 .5438 L
.5438 .54389 L
.53476 .55278 L
.5345 .55303 L
.52562 .56149 L
.5249 .56216 L
.51649 .56991 L
.51496 .57129 L
.50735 .57807 L
.50465 .58043 L
.49822 .58595 L
.49394 .58956 L
.48909 .59358 L
.48277 .5987 L
.47995 .60094 L
.47108 .60783 L
.47082 .60803 L
.46168 .61484 L
.45876 .61697 L
.45255 .62137 L
.44566 .6261 L
.44341 .62761 L
.43428 .63352 L
.43153 .63523 L
.42515 .6391 L
.41601 .64431 L
.41591 .64437 L
.40688 .64911 L
.39774 .65344 L
.3976 .6535 L
Mistroke
.38861 .65724 L
.37947 .66039 L
.37139 .66264 L
.37034 .66287 L
.36121 .66434 L
.35207 .66473 L
.34294 .6631 L
.34108 .66264 L
.3338 .65873 L
.32733 .6535 L
.32467 .6508 L
.31964 .64437 L
.31553 .63746 L
.31442 .63523 L
.31035 .6261 L
.30697 .61697 L
.3064 .61527 L
.3041 .60783 L
.30162 .5987 L
.29948 .58956 L
.29762 .58043 L
.29727 .57856 L
.29599 .57129 L
.29458 .56216 L
.29337 .55303 L
.29235 .54389 L
.29149 .53476 L
.2908 .52562 L
.29026 .51649 L
.28986 .50735 L
.28961 .49822 L
.2895 .48909 L
.28953 .47995 L
.28971 .47082 L
.29004 .46168 L
.29052 .45255 L
.29115 .44341 L
.29195 .43428 L
.29294 .42515 L
.29409 .41601 L
.29546 .40688 L
.29707 .39774 L
.29727 .39675 L
.29895 .38861 L
.30114 .37947 L
.30371 .37034 L
.3064 .36219 L
.30675 .36121 L
.31042 .35207 L
.31506 .34294 L
Mistroke
.31553 .34214 L
.32141 .3338 L
.32467 .32991 L
.32991 .32467 L
.3338 .32141 L
.34214 .31553 L
.34294 .31506 L
.35207 .31042 L
.36121 .30675 L
.36219 .3064 L
.37034 .30371 L
.37947 .30114 L
.38861 .29895 L
.39675 .29727 L
.39774 .29707 L
Mfstroke
.43428 .25147 m
.44341 .25041 L
.45255 .24957 L
.46168 .24893 L
.47082 .2485 L
.47995 .24828 L
.48909 .24825 L
.49822 .24841 L
.50735 .24877 L
.51649 .24933 L
.52562 .25008 L
.53476 .25105 L
.53919 .25159 L
.54389 .25223 L
.55303 .25365 L
.56216 .25531 L
.57129 .25721 L
.58043 .25939 L
.58553 .26073 L
.58956 .26186 L
.5987 .26467 L
.60783 .26784 L
.61312 .26986 L
.61697 .27143 L
.6261 .27553 L
.63296 .279 L
.63523 .28023 L
.64437 .28568 L
.64803 .28813 L
.6535 .29216 L
.65965 .29727 L
.66264 .30002 L
.66879 .3064 L
.67177 .30995 L
.67595 .31553 L
.68091 .3234 L
.68159 .32467 L
.68584 .3338 L
.68891 .34294 L
.69004 .34713 L
.69083 .35207 L
.6918 .36121 L
.692 .37034 L
.69153 .37947 L
.69048 .38861 L
.69004 .39154 L
.68889 .39774 L
.68681 .40688 L
.68428 .41601 L
.68132 .42515 L
Mistroke
.68091 .42634 L
.67796 .43428 L
.67422 .44341 L
.67177 .44895 L
.67011 .45255 L
.66565 .46168 L
.66264 .46748 L
.66084 .47082 L
.6557 .47995 L
.6535 .48368 L
.65022 .48909 L
.64442 .49822 L
.64437 .49829 L
.63829 .50735 L
.63523 .51174 L
.63184 .51649 L
.6261 .52424 L
.62505 .52562 L
.61793 .53476 L
.61697 .53596 L
.61046 .54389 L
.60783 .547 L
.60263 .55303 L
.5987 .55746 L
.59443 .56216 L
.58956 .56738 L
.58584 .57129 L
.58043 .57683 L
.57683 .58043 L
.57129 .58584 L
.56738 .58956 L
.56216 .59443 L
.55746 .5987 L
.55303 .60263 L
.547 .60783 L
.54389 .61046 L
.53596 .61697 L
.53476 .61793 L
.52562 .62505 L
.52424 .6261 L
.51649 .63184 L
.51174 .63523 L
.50735 .63829 L
.49829 .64437 L
.49822 .64442 L
.48909 .65022 L
.48368 .6535 L
.47995 .6557 L
.47082 .66084 L
.46748 .66264 L
Mistroke
.46168 .66565 L
.45255 .67011 L
.44895 .67177 L
.44341 .67422 L
.43428 .67796 L
.42634 .68091 L
.42515 .68132 L
.41601 .68428 L
.40688 .68681 L
.39774 .68889 L
.39154 .69004 L
.38861 .69048 L
.37947 .69153 L
.37034 .692 L
.36121 .6918 L
.35207 .69083 L
.34713 .69004 L
.34294 .68891 L
.3338 .68584 L
.32467 .68159 L
.3234 .68091 L
.31553 .67595 L
.30995 .67177 L
.3064 .66879 L
.30002 .66264 L
.29727 .65965 L
.29216 .6535 L
.28813 .64803 L
.28568 .64437 L
.28023 .63523 L
.279 .63296 L
.27553 .6261 L
.27143 .61697 L
.26986 .61312 L
.26784 .60783 L
.26467 .5987 L
.26186 .58956 L
.26073 .58553 L
.25939 .58043 L
.25721 .57129 L
.25531 .56216 L
.25365 .55303 L
.25223 .54389 L
.25159 .53919 L
.25105 .53476 L
.25008 .52562 L
.24933 .51649 L
.24877 .50735 L
.24841 .49822 L
.24825 .48909 L
Mistroke
.24828 .47995 L
.2485 .47082 L
.24893 .46168 L
.24957 .45255 L
.25041 .44341 L
.25147 .43428 L
.25159 .43332 L
.25276 .42515 L
.25428 .41601 L
.25605 .40688 L
.25808 .39774 L
.26042 .38861 L
.26073 .3875 L
.2631 .37947 L
.26613 .37034 L
.26958 .36121 L
.26986 .36051 L
.27352 .35207 L
.27802 .34294 L
.279 .34111 L
.2832 .3338 L
.28813 .32621 L
.28921 .32467 L
.29624 .31553 L
.29727 .31431 L
.30447 .3064 L
.3064 .30447 L
.31431 .29727 L
.31553 .29624 L
.32467 .28921 L
.32621 .28813 L
.3338 .2832 L
.34111 .279 L
.34294 .27802 L
.35207 .27352 L
.36051 .26986 L
.36121 .26958 L
.37034 .26613 L
.37947 .2631 L
.3875 .26073 L
.38861 .26042 L
.39774 .25808 L
.40688 .25605 L
.41601 .25428 L
.42515 .25276 L
.43332 .25159 L
.43428 .25147 L
Mfstroke
.44341 .20484 m
.45255 .20395 L
.46168 .20328 L
.47082 .20285 L
.47995 .20262 L
.48909 .2026 L
.49822 .2028 L
.50735 .20321 L
.51649 .20382 L
.52562 .20467 L
.53476 .20576 L
.53602 .20592 L
.54389 .20706 L
.55303 .2086 L
.56216 .21041 L
.57129 .21246 L
.58043 .21478 L
.58145 .21506 L
.58956 .2174 L
.5987 .22032 L
.60783 .22358 L
.60943 .22419 L
.61697 .22722 L
.6261 .23125 L
.63043 .23333 L
.63523 .23575 L
.64437 .24075 L
.64728 .24246 L
.6535 .24634 L
.66123 .25159 L
.66264 .25261 L
.67177 .25969 L
.67302 .26073 L
.68091 .26777 L
.68307 .26986 L
.69004 .27713 L
.6917 .279 L
.69913 .28813 L
.69917 .28819 L
.70552 .29727 L
.70831 .30174 L
.71099 .3064 L
.71564 .31553 L
.71744 .31955 L
.71953 .32467 L
.72272 .3338 L
.72527 .34294 L
.72658 .34867 L
.72723 .35207 L
.72864 .36121 L
Mistroke
.72954 .37034 L
.72997 .37947 L
.72997 .38861 L
.72954 .39774 L
.72872 .40688 L
.72753 .41601 L
.72658 .42185 L
.72597 .42515 L
.72407 .43428 L
.72183 .44341 L
.71926 .45255 L
.71744 .45838 L
.71636 .46168 L
.71315 .47082 L
.70961 .47995 L
.70831 .48313 L
.70576 .48909 L
.70161 .49822 L
.69917 .50328 L
.69714 .50735 L
.69236 .51649 L
.69004 .52072 L
.68726 .52562 L
.68184 .53476 L
.68091 .53629 L
.6761 .54389 L
.67177 .55044 L
.67001 .55303 L
.66358 .56216 L
.66264 .56345 L
.65678 .57129 L
.6535 .57554 L
.64962 .58043 L
.64437 .58683 L
.64207 .58956 L
.63523 .59743 L
.6341 .5987 L
.6261 .6074 L
.62569 .60783 L
.61697 .6168 L
.6168 .61697 L
.60783 .62569 L
.6074 .6261 L
.5987 .6341 L
.59743 .63523 L
.58956 .64207 L
.58683 .64437 L
.58043 .64962 L
.57554 .6535 L
.57129 .65678 L
Mistroke
.56345 .66264 L
.56216 .66358 L
.55303 .67001 L
.55044 .67177 L
.54389 .6761 L
.53629 .68091 L
.53476 .68184 L
.52562 .68726 L
.52072 .69004 L
.51649 .69236 L
.50735 .69714 L
.50328 .69917 L
.49822 .70161 L
.48909 .70576 L
.48313 .70831 L
.47995 .70961 L
.47082 .71315 L
.46168 .71636 L
.45838 .71744 L
.45255 .71926 L
.44341 .72183 L
.43428 .72407 L
.42515 .72597 L
.42185 .72658 L
.41601 .72753 L
.40688 .72872 L
.39774 .72954 L
.38861 .72997 L
.37947 .72997 L
.37034 .72954 L
.36121 .72864 L
.35207 .72723 L
.34867 .72658 L
.34294 .72527 L
.3338 .72272 L
.32467 .71953 L
.31955 .71744 L
.31553 .71564 L
.3064 .71099 L
.30174 .70831 L
.29727 .70552 L
.28819 .69917 L
.28813 .69913 L
.279 .6917 L
.27713 .69004 L
.26986 .68307 L
.26777 .68091 L
.26073 .67302 L
.25969 .67177 L
.25261 .66264 L
Mistroke
.25159 .66123 L
.24634 .6535 L
.24246 .64728 L
.24075 .64437 L
.23575 .63523 L
.23333 .63043 L
.23125 .6261 L
.22722 .61697 L
.22419 .60943 L
.22358 .60783 L
.22032 .5987 L
.2174 .58956 L
.21506 .58145 L
.21478 .58043 L
.21246 .57129 L
.21041 .56216 L
.2086 .55303 L
.20706 .54389 L
.20592 .53602 L
.20576 .53476 L
.20467 .52562 L
.20382 .51649 L
.20321 .50735 L
.2028 .49822 L
.2026 .48909 L
.20262 .47995 L
.20285 .47082 L
.20328 .46168 L
.20395 .45255 L
.20484 .44341 L
.20592 .43446 L
.20595 .43428 L
.20728 .42515 L
.20887 .41601 L
.21071 .40688 L
.21279 .39774 L
.21506 .38898 L
.21516 .38861 L
.21783 .37947 L
.2208 .37034 L
.22411 .36121 L
.22419 .361 L
.22779 .35207 L
.23185 .34294 L
.23333 .33984 L
.23635 .3338 L
.24133 .32467 L
.24246 .32271 L
.24683 .31553 L
.25159 .30832 L
Mistroke
.25294 .3064 L
.25971 .29727 L
.26073 .29598 L
.26726 .28813 L
.26986 .2852 L
.2757 .279 L
.279 .2757 L
.2852 .26986 L
.28813 .26726 L
.29598 .26073 L
.29727 .25971 L
.3064 .25294 L
.30832 .25159 L
.31553 .24683 L
.32271 .24246 L
.32467 .24133 L
.3338 .23635 L
.33984 .23333 L
.34294 .23185 L
.35207 .22779 L
.361 .22419 L
.36121 .22411 L
.37034 .2208 L
.37947 .21783 L
.38861 .21516 L
.38898 .21506 L
.39774 .21279 L
.40688 .21071 L
.41601 .20887 L
.42515 .20728 L
.43428 .20595 L
.43446 .20592 L
.44341 .20484 L
Mfstroke
.47995 .15096 m
.48909 .15096 L
.49647 .15112 L
.49822 .15117 L
.50735 .15159 L
.51649 .15221 L
.52562 .15304 L
.53476 .1541 L
.54389 .15536 L
.55303 .15685 L
.56216 .15857 L
.5701 .16025 L
.57129 .16052 L
.58043 .16271 L
.58956 .16516 L
.5987 .16788 L
.60339 .16938 L
.60783 .17089 L
.61697 .1742 L
.6261 .17783 L
.62775 .17852 L
.63523 .18181 L
.64437 .18615 L
.64735 .18765 L
.6535 .19089 L
.66264 .19606 L
.66387 .19679 L
.67177 .20169 L
.67813 .20592 L
.68091 .20785 L
.69004 .21462 L
.69061 .21506 L
.69917 .22205 L
.70164 .22419 L
.70831 .23027 L
.71148 .23333 L
.71744 .23941 L
.72028 .24246 L
.72658 .24965 L
.72818 .25159 L
.73528 .26073 L
.73571 .26131 L
.74166 .26986 L
.74485 .27482 L
.74739 .279 L
.75251 .28813 L
.75398 .29095 L
.75709 .29727 L
.76115 .3064 L
.76311 .31126 L
Mistroke
.76473 .31553 L
.76784 .32467 L
.77053 .3338 L
.77225 .34054 L
.7728 .34294 L
.77469 .35207 L
.77621 .36121 L
.77737 .37034 L
.7782 .37947 L
.77869 .38861 L
.77886 .39774 L
.77873 .40688 L
.77829 .41601 L
.77757 .42515 L
.77657 .43428 L
.77528 .44341 L
.77372 .45255 L
.77225 .45998 L
.77189 .46168 L
.76978 .47082 L
.7674 .47995 L
.76476 .48909 L
.76311 .49434 L
.76185 .49822 L
.75867 .50735 L
.75523 .51649 L
.75398 .51963 L
.75151 .52562 L
.74752 .53476 L
.74485 .54055 L
.74325 .54389 L
.73869 .55303 L
.73571 .55869 L
.73383 .56216 L
.72868 .57129 L
.72658 .57487 L
.72322 .58043 L
.71744 .58956 L
.71744 .58957 L
.71134 .5987 L
.70831 .60304 L
.70488 .60783 L
.69917 .61551 L
.69806 .61697 L
.69085 .6261 L
.69004 .6271 L
.68323 .63523 L
.68091 .63792 L
.67517 .64437 L
.67177 .64808 L
Mistroke
.66665 .6535 L
.66264 .65763 L
.65763 .66264 L
.6535 .66665 L
.64808 .67177 L
.64437 .67517 L
.63792 .68091 L
.63523 .68323 L
.6271 .69004 L
.6261 .69085 L
.61697 .69806 L
.61551 .69917 L
.60783 .70488 L
.60304 .70831 L
.5987 .71134 L
.58957 .71744 L
.58956 .71744 L
.58043 .72322 L
.57487 .72658 L
.57129 .72868 L
.56216 .73383 L
.55869 .73571 L
.55303 .73869 L
.54389 .74325 L
.54055 .74485 L
.53476 .74752 L
.52562 .75151 L
.51963 .75398 L
.51649 .75523 L
.50735 .75867 L
.49822 .76185 L
.49434 .76311 L
.48909 .76476 L
.47995 .7674 L
.47082 .76978 L
.46168 .77189 L
.45998 .77225 L
.45255 .77372 L
.44341 .77528 L
.43428 .77657 L
.42515 .77757 L
.41601 .77829 L
.40688 .77873 L
.39774 .77886 L
.38861 .77869 L
.37947 .7782 L
.37034 .77737 L
.36121 .77621 L
.35207 .77469 L
.34294 .7728 L
Mistroke
.34054 .77225 L
.3338 .77053 L
.32467 .76784 L
.31553 .76473 L
.31126 .76311 L
.3064 .76115 L
.29727 .75709 L
.29095 .75398 L
.28813 .75251 L
.279 .74739 L
.27482 .74485 L
.26986 .74166 L
.26131 .73571 L
.26073 .73528 L
.25159 .72818 L
.24965 .72658 L
.24246 .72028 L
.23941 .71744 L
.23333 .71148 L
.23027 .70831 L
.22419 .70164 L
.22205 .69917 L
.21506 .69061 L
.21462 .69004 L
.20785 .68091 L
.20592 .67813 L
.20169 .67177 L
.19679 .66387 L
.19606 .66264 L
.19089 .6535 L
.18765 .64735 L
.18615 .64437 L
.18181 .63523 L
.17852 .62775 L
.17783 .6261 L
.1742 .61697 L
.17089 .60783 L
.16938 .60339 L
.16788 .5987 L
.16516 .58956 L
.16271 .58043 L
.16052 .57129 L
.16025 .5701 L
.15857 .56216 L
.15685 .55303 L
.15536 .54389 L
.1541 .53476 L
.15304 .52562 L
.15221 .51649 L
.15159 .50735 L
Mistroke
.15117 .49822 L
.15112 .49647 L
.15096 .48909 L
.15096 .47995 L
.15112 .47197 L
.15115 .47082 L
.15155 .46168 L
.15216 .45255 L
.15299 .44341 L
.15402 .43428 L
.15527 .42515 L
.15674 .41601 L
.15844 .40688 L
.16025 .39821 L
.16035 .39774 L
.16252 .38861 L
.16493 .37947 L
.16761 .37034 L
.16938 .36475 L
.17056 .36121 L
.17379 .35207 L
.17732 .34294 L
.17852 .34 L
.18117 .3338 L
.18534 .32467 L
.18765 .3199 L
.18987 .31553 L
.19479 .3064 L
.19679 .30289 L
.20012 .29727 L
.20589 .28813 L
.20592 .28808 L
.21215 .279 L
.21506 .27499 L
.21894 .26986 L
.22419 .26329 L
.22632 .26073 L
.23333 .25272 L
.23435 .25159 L
.24246 .24312 L
.24312 .24246 L
.25159 .23435 L
.25272 .23333 L
.26073 .22632 L
.26329 .22419 L
.26986 .21894 L
.27499 .21506 L
.279 .21215 L
.28808 .20592 L
.28813 .20589 L
Mistroke
.29727 .20012 L
.30289 .19679 L
.3064 .19479 L
.31553 .18987 L
.3199 .18765 L
.32467 .18534 L
.3338 .18117 L
.34 .17852 L
.34294 .17732 L
.35207 .17379 L
.36121 .17056 L
.36475 .16938 L
.37034 .16761 L
.37947 .16493 L
.38861 .16252 L
.39774 .16035 L
.39821 .16025 L
.40688 .15844 L
.41601 .15674 L
.42515 .15527 L
.43428 .15402 L
.44341 .15299 L
.45255 .15216 L
.46168 .15155 L
.47082 .15115 L
.47197 .15112 L
.47995 .15096 L
Mfstroke
.42515 .09553 m
.43428 .09445 L
.44341 .09356 L
.45255 .09284 L
.46168 .09232 L
.47082 .09198 L
.47995 .09181 L
.48909 .09183 L
.49822 .09203 L
.50735 .09241 L
.51649 .09298 L
.52562 .09374 L
.53476 .09469 L
.54389 .09582 L
.54745 .09631 L
.55303 .09714 L
.56216 .09867 L
.57129 .10039 L
.58043 .10231 L
.58956 .10444 L
.59354 .10544 L
.5987 .1068 L
.60783 .10939 L
.61697 .11222 L
.624 .11458 L
.6261 .11531 L
.63523 .11865 L
.64437 .12227 L
.64785 .12371 L
.6535 .12615 L
.66264 .13034 L
.66779 .13285 L
.67177 .13485 L
.68091 .13969 L
.685 .14198 L
.69004 .14491 L
.69917 .15049 L
.70015 .15112 L
.70831 .15652 L
.71364 .16025 L
.71744 .16301 L
.72578 .16938 L
.72658 .17002 L
.73571 .17759 L
.73679 .17852 L
.74485 .18578 L
.74683 .18765 L
.75398 .19468 L
.75603 .19679 L
.76311 .2044 L
Mistroke
.76448 .20592 L
.77224 .21506 L
.77225 .21507 L
.77939 .22419 L
.78138 .22688 L
.78597 .23333 L
.79052 .24009 L
.79205 .24246 L
.79763 .25159 L
.79965 .25509 L
.80277 .26073 L
.80749 .26986 L
.80879 .27251 L
.81182 .279 L
.81576 .28813 L
.81792 .2935 L
.81936 .29727 L
.82261 .3064 L
.82554 .31553 L
.82705 .32069 L
.82815 .32467 L
.83046 .3338 L
.83249 .34294 L
.83423 .35207 L
.8357 .36121 L
.83619 .36462 L
.83691 .37034 L
.83787 .37947 L
.83857 .38861 L
.83903 .39774 L
.83925 .40688 L
.83923 .41601 L
.83899 .42515 L
.83852 .43428 L
.83783 .44341 L
.83691 .45255 L
.83619 .45853 L
.83577 .46168 L
.8344 .47082 L
.83283 .47995 L
.83103 .48909 L
.82901 .49822 L
.82705 .50631 L
.82679 .50735 L
.82434 .51649 L
.82167 .52562 L
.81877 .53476 L
.81792 .53732 L
.81564 .54389 L
.81228 .55303 L
Mistroke
.80879 .56193 L
.80869 .56216 L
.80487 .57129 L
.8008 .58043 L
.79965 .58292 L
.79648 .58956 L
.79191 .5987 L
.79052 .60138 L
.78707 .60783 L
.78194 .61697 L
.78138 .61793 L
.77652 .6261 L
.77225 .63297 L
.7708 .63523 L
.76476 .64437 L
.76311 .64678 L
.7584 .6535 L
.75398 .65957 L
.75169 .66264 L
.74485 .6715 L
.74463 .67177 L
.73719 .68091 L
.73571 .68266 L
.72934 .69004 L
.72658 .69315 L
.72104 .69917 L
.71744 .70298 L
.71228 .70831 L
.70831 .71228 L
.70298 .71744 L
.69917 .72104 L
.69315 .72658 L
.69004 .72934 L
.68266 .73571 L
.68091 .73719 L
.67177 .74463 L
.6715 .74485 L
.66264 .75169 L
.65957 .75398 L
.6535 .7584 L
.64678 .76311 L
.64437 .76476 L
.63523 .7708 L
.63297 .77225 L
.6261 .77652 L
.61793 .78138 L
.61697 .78194 L
.60783 .78707 L
.60138 .79052 L
.5987 .79191 L
Mistroke
.58956 .79648 L
.58292 .79965 L
.58043 .8008 L
.57129 .80487 L
.56216 .80869 L
.56193 .80879 L
.55303 .81228 L
.54389 .81564 L
.53732 .81792 L
.53476 .81877 L
.52562 .82167 L
.51649 .82434 L
.50735 .82679 L
.50631 .82705 L
.49822 .82901 L
.48909 .83103 L
.47995 .83283 L
.47082 .8344 L
.46168 .83577 L
.45853 .83619 L
.45255 .83691 L
.44341 .83783 L
.43428 .83852 L
.42515 .83899 L
.41601 .83923 L
.40688 .83925 L
.39774 .83903 L
.38861 .83857 L
.37947 .83787 L
.37034 .83691 L
.36462 .83619 L
.36121 .8357 L
.35207 .83423 L
.34294 .83249 L
.3338 .83046 L
.32467 .82815 L
.32069 .82705 L
.31553 .82554 L
.3064 .82261 L
.29727 .81936 L
.2935 .81792 L
.28813 .81576 L
.279 .81182 L
.27251 .80879 L
.26986 .80749 L
.26073 .80277 L
.25509 .79965 L
.25159 .79763 L
.24246 .79205 L
.24009 .79052 L
Mistroke
.23333 .78597 L
.22688 .78138 L
.22419 .77939 L
.21507 .77225 L
.21506 .77224 L
.20592 .76448 L
.2044 .76311 L
.19679 .75603 L
.19468 .75398 L
.18765 .74683 L
.18578 .74485 L
.17852 .73679 L
.17759 .73571 L
.17002 .72658 L
.16938 .72578 L
.16301 .71744 L
.16025 .71364 L
.15652 .70831 L
.15112 .70015 L
.15049 .69917 L
.14491 .69004 L
.14198 .685 L
.13969 .68091 L
.13485 .67177 L
.13285 .66779 L
.13034 .66264 L
.12615 .6535 L
.12371 .64785 L
.12227 .64437 L
.11865 .63523 L
.11531 .6261 L
.11458 .624 L
.11222 .61697 L
.10939 .60783 L
.1068 .5987 L
.10544 .59354 L
.10444 .58956 L
.10231 .58043 L
.10039 .57129 L
.09867 .56216 L
.09714 .55303 L
.09631 .54745 L
.09582 .54389 L
.09469 .53476 L
.09374 .52562 L
.09298 .51649 L
.09241 .50735 L
.09203 .49822 L
.09183 .48909 L
.09181 .47995 L
Mistroke
.09198 .47082 L
.09232 .46168 L
.09284 .45255 L
.09356 .44341 L
.09445 .43428 L
.09553 .42515 L
.09631 .41939 L
.09681 .41601 L
.09827 .40688 L
.09991 .39774 L
.10177 .38861 L
.10383 .37947 L
.10544 .3729 L
.1061 .37034 L
.1086 .36121 L
.11132 .35207 L
.11427 .34294 L
.11458 .34201 L
.11746 .3338 L
.1209 .32467 L
.12371 .31767 L
.1246 .31553 L
.12859 .3064 L
.13285 .2973 L
.13287 .29727 L
.13744 .28813 L
.14198 .27963 L
.14233 .279 L
.14756 .26986 L
.15112 .26399 L
.15316 .26073 L
.15915 .25159 L
.16025 .24998 L
.16556 .24246 L
.16938 .23728 L
.17241 .23333 L
.17852 .22568 L
.17975 .22419 L
.18762 .21506 L
.18765 .21502 L
.19608 .20592 L
.19679 .20518 L
.20518 .19679 L
.20592 .19608 L
.21502 .18765 L
.21506 .18762 L
.22419 .17975 L
.22568 .17852 L
.23333 .17241 L
.23728 .16938 L
Mistroke
.24246 .16556 L
.24998 .16025 L
.25159 .15915 L
.26073 .15316 L
.26399 .15112 L
.26986 .14756 L
.279 .14233 L
.27963 .14198 L
.28813 .13744 L
.29727 .13287 L
.2973 .13285 L
.3064 .12859 L
.31553 .1246 L
.31767 .12371 L
.32467 .1209 L
.3338 .11746 L
.34201 .11458 L
.34294 .11427 L
.35207 .11132 L
.36121 .1086 L
.37034 .1061 L
.3729 .10544 L
.37947 .10383 L
.38861 .10177 L
.39774 .09991 L
.40688 .09827 L
.41601 .09681 L
.41939 .09631 L
.42515 .09553 L
Mfstroke
.05064 .64977 m
.05192 .6535 L
.0552 .66264 L
.05869 .67177 L
.05977 .67448 L
.06242 .68091 L
.06639 .69004 L
.06891 .69555 L
.07061 .69917 L
.07511 .70831 L
.07804 .71396 L
.07991 .71744 L
.08501 .72658 L
.08718 .73029 L
.09044 .73571 L
.09622 .74485 L
.09631 .74498 L
.10238 .75398 L
.10544 .75833 L
.10893 .76311 L
.11458 .77056 L
.1159 .77225 L
.12333 .78138 L
.12371 .78184 L
.13127 .79052 L
.13285 .79226 L
.13978 .79965 L
.14198 .80191 L
.14893 .80879 L
.15112 .81088 L
.15878 .81792 L
.16025 .81923 L
.16938 .82702 L
.16943 .82705 L
.17852 .8343 L
.18099 .83619 L
.18765 .84112 L
.19362 .84532 L
.19679 .84748 L
.20592 .85343 L
.20756 .85446 L
.21506 .85899 L
.22312 .86359 L
.22419 .86419 L
.23333 .86905 L
.2407 .87273 L
.24246 .87357 L
.25159 .8778 L
.26073 .88172 L
.26107 .88186 L
Mistroke
.26986 .88536 L
.279 .88873 L
.28559 .89099 L
.28813 .89184 L
.29727 .8947 L
.3064 .89732 L
.31553 .89971 L
.31724 .90013 L
.32467 .90187 L
.3338 .9038 L
.34294 .90554 L
.35207 .90706 L
.36121 .90839 L
.36817 .90926 L
.37034 .90951 L
.37947 .91044 L
.38861 .91119 L
.39774 .91174 L
.40688 .91211 L
.41601 .91231 L
.42515 .91233 L
.43428 .91218 L
.44341 .91185 L
.45255 .91134 L
.46168 .91067 L
.47082 .90982 L
.47599 .90926 L
.47995 .9088 L
.48909 .90761 L
.49822 .90625 L
.50735 .90472 L
.51649 .90301 L
.52562 .90111 L
.53007 .90013 L
.53476 .89904 L
.54389 .8968 L
.55303 .89438 L
.56216 .89177 L
.56476 .89099 L
.57129 .88899 L
.58043 .88603 L
.58956 .88288 L
.59243 .88186 L
.5987 .87954 L
.60783 .87599 L
.61579 .87273 L
.61697 .87223 L
.6261 .86825 L
.63523 .86405 L
.63621 .86359 L
Mistroke
.64437 .85963 L
.6535 .85494 L
.65443 .85446 L
.66264 .85004 L
.67098 .84532 L
.67177 .84487 L
.68091 .83945 L
.6862 .83619 L
.69004 .83376 L
.69917 .82778 L
.70025 .82705 L
.70831 .82149 L
.7133 .81792 L
.71744 .81488 L
.72547 .80879 L
.72658 .80792 L
.73571 .80059 L
.73686 .79965 L
.74485 .79288 L
.74756 .79052 L
.75398 .78477 L
.75766 .78138 L
.76311 .77623 L
.7672 .77225 L
.77225 .7672 L
.77623 .76311 L
.78138 .75766 L
.78477 .75398 L
.79052 .74756 L
.79288 .74485 L
.79965 .73686 L
.80059 .73571 L
.80792 .72658 L
.80879 .72547 L
.81488 .71744 L
.81792 .7133 L
.82149 .70831 L
.82705 .70025 L
.82778 .69917 L
.83376 .69004 L
.83619 .6862 L
.83945 .68091 L
.84487 .67177 L
.84532 .67098 L
.85004 .66264 L
.85446 .65443 L
.85494 .6535 L
.85963 .64437 L
.86359 .63621 L
.86405 .63523 L
Mistroke
.86825 .6261 L
.87223 .61697 L
.87273 .61579 L
.87599 .60783 L
.87954 .5987 L
.88186 .59243 L
.88288 .58956 L
.88603 .58043 L
.88899 .57129 L
.89099 .56476 L
.89177 .56216 L
.89438 .55303 L
.8968 .54389 L
.89904 .53476 L
.90013 .53007 L
.90111 .52562 L
.90301 .51649 L
.90472 .50735 L
.90625 .49822 L
.90761 .48909 L
.9088 .47995 L
.90926 .47599 L
.90982 .47082 L
.91067 .46168 L
.91134 .45255 L
.91185 .44341 L
.91218 .43428 L
.91233 .42515 L
.91231 .41601 L
.91211 .40688 L
.91174 .39774 L
.91119 .38861 L
.91044 .37947 L
.90951 .37034 L
.90926 .36817 L
.90839 .36121 L
.90706 .35207 L
.90554 .34294 L
.9038 .3338 L
.90187 .32467 L
.90013 .31724 L
.89971 .31553 L
.89732 .3064 L
.8947 .29727 L
.89184 .28813 L
.89099 .28559 L
.88873 .279 L
.88536 .26986 L
.88186 .26107 L
.88172 .26073 L
Mistroke
.8778 .25159 L
.87357 .24246 L
.87273 .2407 L
.86905 .23333 L
.86419 .22419 L
.86359 .22312 L
.85899 .21506 L
.85446 .20756 L
.85343 .20592 L
.84748 .19679 L
.84532 .19362 L
.84112 .18765 L
.83619 .18099 L
.8343 .17852 L
.82705 .16943 L
.82702 .16938 L
.81923 .16025 L
.81792 .15878 L
.81088 .15112 L
.80879 .14893 L
.80191 .14198 L
.79965 .13978 L
.79226 .13285 L
.79052 .13127 L
.78184 .12371 L
.78138 .12333 L
.77225 .1159 L
.77056 .11458 L
.76311 .10893 L
.75833 .10544 L
.75398 .10238 L
.74498 .09631 L
.74485 .09622 L
.73571 .09044 L
.73029 .08718 L
.72658 .08501 L
.71744 .07991 L
.71396 .07804 L
.70831 .07511 L
.69917 .07061 L
.69555 .06891 L
.69004 .06639 L
.68091 .06242 L
.67448 .05977 L
.67177 .05869 L
.66264 .0552 L
.6535 .05192 L
.64977 .05064 L
Mfstroke
.05064 .31467 m
.05348 .3064 L
.05681 .29727 L
.05977 .28963 L
.06037 .28813 L
.06416 .279 L
.06819 .26986 L
.06891 .26828 L
.07246 .26073 L
.07699 .25159 L
.07804 .24956 L
.0818 .24246 L
.0869 .23333 L
.08718 .23285 L
.09232 .22419 L
.09631 .21777 L
.09805 .21506 L
.10413 .20592 L
.10544 .20402 L
.11058 .19679 L
.11458 .19138 L
.11742 .18765 L
.12371 .17971 L
.12469 .17852 L
.13242 .16938 L
.13285 .1689 L
.14065 .16025 L
.14198 .15883 L
.14944 .15112 L
.15112 .14944 L
.15883 .14198 L
.16025 .14065 L
.1689 .13285 L
.16938 .13242 L
.17852 .12469 L
.17971 .12371 L
.18765 .11742 L
.19138 .11458 L
.19679 .11058 L
.20402 .10544 L
.20592 .10413 L
.21506 .09805 L
.21777 .09631 L
.22419 .09232 L
.23285 .08718 L
.23333 .0869 L
.24246 .0818 L
.24956 .07804 L
.25159 .07699 L
.26073 .07246 L
Mistroke
.26828 .06891 L
.26986 .06819 L
.279 .06416 L
.28813 .06037 L
.28963 .05977 L
.29727 .05681 L
.3064 .05348 L
.31467 .05064 L
Mfstroke
.0001 w
.05064 .05064 m
.03901 .03901 L
s
% Start of user PostScript
22. 22. 21.254054 21.254054 MAarrowhead1
% End of user PostScript
.05064 .11523 m
.03619 .10371 L
s
% Start of user PostScript
22. 26.142857 21.073398 25.403727 MAarrowhead1
% End of user PostScript
.05064 .17982 m
.03321 .16883 L
s
% Start of user PostScript
22. 30.285714 20.882286 29.580642 MAarrowhead1
% End of user PostScript
.05064 .24442 m
.03022 .23456 L
s
% Start of user PostScript
22. 34.428571 20.690437 33.796688 MAarrowhead1
% End of user PostScript
.05064 .30901 m
.02746 .30103 L
s
% Start of user PostScript
22. 38.571429 20.51333 38.059571 MAarrowhead1
% End of user PostScript
.05064 .3736 m
.02529 .3682 L
s
% Start of user PostScript
22. 42.714286 20.37433 42.367719 MAarrowhead1
% End of user PostScript
.05064 .43819 m
.02404 .43587 L
s
% Start of user PostScript
22. 46.857143 20.293779 46.708355 MAarrowhead1
% End of user PostScript
.05064 .50279 m
.02381 .50383 L
s
% Start of user PostScript
22. 51. 20.279192 51.067157 MAarrowhead1
% End of user PostScript
.05064 .56738 m
.02463 .57167 L
s
% Start of user PostScript
22. 55.142857 20.331731 55.418144 MAarrowhead1
% End of user PostScript
.05064 .63197 m
.0265 .63916 L
s
% Start of user PostScript
22. 59.285714 20.451704 59.746966 MAarrowhead1
% End of user PostScript
.05064 .69656 m
.02919 .70598 L
s
% Start of user PostScript
22. 63.428571 20.624113 64.032683 MAarrowhead1
% End of user PostScript
.05064 .76116 m
.03231 .77192 L
s
% Start of user PostScript
22. 67.571429 20.824227 68.261912 MAarrowhead1
% End of user PostScript
.05064 .82575 m
.03545 .83709 L
s
% Start of user PostScript
22. 71.714286 21.025885 72.441542 MAarrowhead1
% End of user PostScript
.05064 .89034 m
.03837 .90174 L
s
% Start of user PostScript
22. 75.857143 21.213098 76.588439 MAarrowhead1
% End of user PostScript
.05064 .95493 m
.04097 .96608 L
s
% Start of user PostScript
22. 80. 21.379922 80.714814 MAarrowhead1
% End of user PostScript
.11523 .05064 m
.10371 .03619 L
s
% Start of user PostScript
26.142857 22. 25.403727 21.073398 MAarrowhead1
% End of user PostScript
.11523 .11523 m
.10061 .10061 L
s
% Start of user PostScript
26.142857 26.142857 25.205175 25.205175 MAarrowhead1
% End of user PostScript
.11523 .17982 m
.09708 .16549 L
s
% Start of user PostScript
26.142857 30.285714 24.978609 29.366294 MAarrowhead1
% End of user PostScript
.11523 .24442 m
.09321 .23114 L
s
% Start of user PostScript
26.142857 34.428571 24.73033 33.577234 MAarrowhead1
% End of user PostScript
.11523 .30901 m
.08945 .29802 L
s
% Start of user PostScript
26.142857 38.571429 24.489412 37.8665 MAarrowhead1
% End of user PostScript
.11523 .3736 m
.08648 .36613 L
s
% Start of user PostScript
26.142857 42.714286 24.299012 42.234922 MAarrowhead1
% End of user PostScript
.11523 .43819 m
.0848 .43498 L
s
% Start of user PostScript
26.142857 46.857143 24.191254 46.650874 MAarrowhead1
% End of user PostScript
.11523 .50279 m
.08449 .50411 L
s
% Start of user PostScript
26.142857 51. 24.171326 51.085064 MAarrowhead1
% End of user PostScript
.11523 .56738 m
.08548 .57323 L
s
% Start of user PostScript
26.142857 55.142857 24.234894 55.518331 MAarrowhead1
% End of user PostScript
.11523 .63197 m
.08797 .64196 L
s
% Start of user PostScript
26.142857 59.285714 24.394123 59.926634 MAarrowhead1
% End of user PostScript
.11523 .69656 m
.09174 .70951 L
s
% Start of user PostScript
26.142857 63.428571 24.636157 64.258885 MAarrowhead1
% End of user PostScript
.11523 .76116 m
.09599 .77562 L
s
% Start of user PostScript
26.142857 67.571429 24.908972 68.499189 MAarrowhead1
% End of user PostScript
.11523 .82575 m
.09997 .84042 L
s
% Start of user PostScript
26.142857 71.714286 25.16371 72.655098 MAarrowhead1
% End of user PostScript
.11523 .89034 m
.10328 .90463 L
s
% Start of user PostScript
26.142857 75.857143 25.37636 76.773342 MAarrowhead1
% End of user PostScript
.11523 .95493 m
.10605 .96857 L
s
% Start of user PostScript
26.142857 80. 25.553971 80.874734 MAarrowhead1
% End of user PostScript
.17982 .05064 m
.16883 .03321 L
s
% Start of user PostScript
30.285714 22. 29.580642 20.882286 MAarrowhead1
% End of user PostScript
.17982 .11523 m
.16549 .09708 L
s
% Start of user PostScript
30.285714 26.142857 29.366294 24.978609 MAarrowhead1
% End of user PostScript
.17982 .17982 m
.1613 .1613 L
s
% Start of user PostScript
30.285714 30.285714 29.097327 29.097327 MAarrowhead1
% End of user PostScript
.17982 .24442 m
.1562 .22653 L
s
% Start of user PostScript
30.285714 34.428571 28.770583 33.281201 MAarrowhead1
% End of user PostScript
.17982 .30901 m
.15065 .29373 L
s
% Start of user PostScript
30.285714 38.571429 28.414614 37.5912 MAarrowhead1
% End of user PostScript
.17982 .3736 m
.14626 .36329 L
s
% Start of user PostScript
30.285714 42.714286 28.132915 42.052949 MAarrowhead1
% End of user PostScript
.17982 .43819 m
.14417 .43387 L
s
% Start of user PostScript
30.285714 46.857143 27.999084 46.579607 MAarrowhead1
% End of user PostScript
.17982 .50279 m
.14374 .5044 L
s
% Start of user PostScript
30.285714 51. 27.971497 51.103584 MAarrowhead1
% End of user PostScript
.17982 .56738 m
.1448 .57519 L
s
% Start of user PostScript
30.285714 55.142857 28.039497 55.643519 MAarrowhead1
% End of user PostScript
.17982 .63197 m
.14811 .64596 L
s
% Start of user PostScript
30.285714 59.285714 28.251538 60.182592 MAarrowhead1
% End of user PostScript
.17982 .69656 m
.15393 .71468 L
s
% Start of user PostScript
30.285714 63.428571 28.624978 64.590203 MAarrowhead1
% End of user PostScript
.17982 .76116 m
.16011 .78048 L
s
% Start of user PostScript
30.285714 67.571429 29.021171 68.810832 MAarrowhead1
% End of user PostScript
.17982 .82575 m
.16509 .84441 L
s
% Start of user PostScript
30.285714 71.714286 29.340413 72.91106 MAarrowhead1
% End of user PostScript
.17982 .89034 m
.16881 .90775 L
s
% Start of user PostScript
30.285714 75.857143 29.579145 76.97342 MAarrowhead1
% End of user PostScript
.17982 .95493 m
.17159 .97109 L
s
% Start of user PostScript
30.285714 80. 29.757807 81.036028 MAarrowhead1
% End of user PostScript
.24442 .05064 m
.23456 .03022 L
s
% Start of user PostScript
34.428571 22. 33.796688 20.690437 MAarrowhead1
% End of user PostScript
.24442 .11523 m
.23114 .09321 L
s
% Start of user PostScript
34.428571 26.142857 33.577234 24.73033 MAarrowhead1
% End of user PostScript
.24442 .17982 m
.22653 .1562 L
s
% Start of user PostScript
34.428571 30.285714 33.281201 28.770583 MAarrowhead1
% End of user PostScript
.24442 .24442 m
.21986 .21986 L
s
% Start of user PostScript
34.428571 34.428571 32.853541 32.853541 MAarrowhead1
% End of user PostScript
.24442 .30901 m
.21066 .28663 L
s
% Start of user PostScript
34.428571 38.571429 32.263683 37.136056 MAarrowhead1
% End of user PostScript
.24442 .3736 m
.20336 .35966 L
s
% Start of user PostScript
34.428571 42.714286 31.795519 41.819997 MAarrowhead1
% End of user PostScript
.24442 .43819 m
.20194 .43309 L
s
% Start of user PostScript
34.428571 46.857143 31.704021 46.52959 MAarrowhead1
% End of user PostScript
.24442 .50279 m
.20187 .50449 L
s
% Start of user PostScript
34.428571 51. 31.699503 51.108934 MAarrowhead1
% End of user PostScript
.24442 .56738 m
.20174 .57673 L
s
% Start of user PostScript
34.428571 55.142857 31.691393 55.742787 MAarrowhead1
% End of user PostScript
.24442 .63197 m
.20512 .65196 L
s
% Start of user PostScript
34.428571 59.285714 31.908255 60.567452 MAarrowhead1
% End of user PostScript
.24442 .69656 m
.21598 .72333 L
s
% Start of user PostScript
34.428571 63.428571 32.604894 65.145548 MAarrowhead1
% End of user PostScript
.24442 .76116 m
.22571 .7871 L
s
% Start of user PostScript
34.428571 67.571429 33.228843 69.235439 MAarrowhead1
% End of user PostScript
.24442 .82575 m
.23158 .84886 L
s
% Start of user PostScript
34.428571 71.714286 33.605509 73.196718 MAarrowhead1
% End of user PostScript
.24442 .89034 m
.23523 .91086 L
s
% Start of user PostScript
34.428571 75.857143 33.839376 77.172863 MAarrowhead1
% End of user PostScript
.24442 .95493 m
.23773 .97339 L
s
% Start of user PostScript
34.428571 80. 33.999565 81.183613 MAarrowhead1
% End of user PostScript
.30901 .05064 m
.30103 .02746 L
s
% Start of user PostScript
38.571429 22. 38.059571 20.51333 MAarrowhead1
% End of user PostScript
.30901 .11523 m
.29802 .08945 L
s
% Start of user PostScript
38.571429 26.142857 37.8665 24.489412 MAarrowhead1
% End of user PostScript
.30901 .17982 m
.29373 .15065 L
s
% Start of user PostScript
38.571429 30.285714 37.5912 28.414614 MAarrowhead1
% End of user PostScript
.30901 .24442 m
.28663 .21066 L
s
% Start of user PostScript
38.571429 34.428571 37.136056 32.263683 MAarrowhead1
% End of user PostScript
.30901 .30901 m
.27059 .27059 L
s
% Start of user PostScript
38.571429 38.571429 36.107012 36.107012 MAarrowhead1
% End of user PostScript
.30901 .3736 m
.25226 .35848 L
s
% Start of user PostScript
38.571429 42.714286 34.931456 41.744221 MAarrowhead1
% End of user PostScript
.30901 .43819 m
.25931 .43488 L
s
% Start of user PostScript
38.571429 46.857143 35.384097 46.644669 MAarrowhead1
% End of user PostScript
.30901 .50279 m
.26072 .50371 L
s
% Start of user PostScript
38.571429 51. 35.474043 51.059281 MAarrowhead1
% End of user PostScript
.30901 .56738 m
.25632 .57385 L
s
% Start of user PostScript
38.571429 55.142857 35.191875 55.558116 MAarrowhead1
% End of user PostScript
.30901 .63197 m
.24832 .66098 L
s
% Start of user PostScript
38.571429 59.285714 34.678976 61.146148 MAarrowhead1
% End of user PostScript
.30901 .69656 m
.28246 .74065 L
s
% Start of user PostScript
38.571429 63.428571 36.868852 66.256222 MAarrowhead1
% End of user PostScript
.30901 .76116 m
.29538 .79484 L
s
% Start of user PostScript
38.571429 67.571429 37.697399 69.731935 MAarrowhead1
% End of user PostScript
.30901 .82575 m
.30014 .85294 L
s
% Start of user PostScript
38.571429 71.714286 38.002868 73.458347 MAarrowhead1
% End of user PostScript
.30901 .89034 m
.30273 .9134 L
s
% Start of user PostScript
38.571429 75.857143 38.168959 77.335741 MAarrowhead1
% End of user PostScript
.30901 .95493 m
.30442 .97515 L
s
% Start of user PostScript
38.571429 80. 38.277314 81.296629 MAarrowhead1
% End of user PostScript
.3736 .05064 m
.3682 .02529 L
s
% Start of user PostScript
42.714286 22. 42.367719 20.37433 MAarrowhead1
% End of user PostScript
.3736 .11523 m
.36613 .08648 L
s
% Start of user PostScript
42.714286 26.142857 42.234922 24.299012 MAarrowhead1
% End of user PostScript
.3736 .17982 m
.36329 .14626 L
s
% Start of user PostScript
42.714286 30.285714 42.052949 28.132915 MAarrowhead1
% End of user PostScript
.3736 .24442 m
.35966 .20336 L
s
% Start of user PostScript
42.714286 34.428571 41.819997 31.795519 MAarrowhead1
% End of user PostScript
.3736 .30901 m
.35848 .25226 L
s
% Start of user PostScript
42.714286 38.571429 41.744221 34.931456 MAarrowhead1
% End of user PostScript
.3736 .3736 m
.3736 .3736 L
s
% Start of user PostScript
42.714286 42.714286 MAarrowzero1
% End of user PostScript
.3736 .43819 m
.3736 .43819 L
s
% Start of user PostScript
42.714286 46.857143 MAarrowzero1
% End of user PostScript
.3736 .50279 m
.3736 .50279 L
s
% Start of user PostScript
42.714286 51. MAarrowzero1
% End of user PostScript
.3736 .56738 m
.3736 .56738 L
s
% Start of user PostScript
42.714286 55.142857 MAarrowzero1
% End of user PostScript
.3736 .63197 m
.4253 .69032 L
s
% Start of user PostScript
42.714286 59.285714 46.03016 63.028037 MAarrowhead1
% End of user PostScript
.3736 .69656 m
.37484 .75019 L
s
% Start of user PostScript
42.714286 63.428571 42.793998 66.867913 MAarrowhead1
% End of user PostScript
.3736 .76116 m
.37041 .79828 L
s
% Start of user PostScript
42.714286 67.571429 42.509304 69.952605 MAarrowhead1
% End of user PostScript
.3736 .82575 m
.37048 .85489 L
s
% Start of user PostScript
42.714286 71.714286 42.514016 73.583117 MAarrowhead1
% End of user PostScript
.3736 .89034 m
.37099 .91471 L
s
% Start of user PostScript
42.714286 75.857143 42.547056 77.420253 MAarrowhead1
% End of user PostScript
.3736 .95493 m
.37152 .97616 L
s
% Start of user PostScript
42.714286 80. 42.580995 81.361108 MAarrowhead1
% End of user PostScript
.43819 .05064 m
.43587 .02404 L
s
% Start of user PostScript
46.857143 22. 46.708355 20.293779 MAarrowhead1
% End of user PostScript
.43819 .11523 m
.43498 .0848 L
s
% Start of user PostScript
46.857143 26.142857 46.650874 24.191254 MAarrowhead1
% End of user PostScript
.43819 .17982 m
.43387 .14417 L
s
% Start of user PostScript
46.857143 30.285714 46.579607 27.999084 MAarrowhead1
% End of user PostScript
.43819 .24442 m
.43309 .20194 L
s
% Start of user PostScript
46.857143 34.428571 46.52959 31.704021 MAarrowhead1
% End of user PostScript
.43819 .30901 m
.43488 .25931 L
s
% Start of user PostScript
46.857143 38.571429 46.644669 35.384097 MAarrowhead1
% End of user PostScript
.43819 .3736 m
.43819 .3736 L
s
% Start of user PostScript
46.857143 42.714286 MAarrowzero1
% End of user PostScript
.43819 .43819 m
.43819 .43819 L
s
% Start of user PostScript
46.857143 46.857143 MAarrowzero1
% End of user PostScript
.43819 .50279 m
.43819 .50279 L
s
% Start of user PostScript
46.857143 51. MAarrowzero1
% End of user PostScript
.43819 .56738 m
.47466 .59964 L
s
% Start of user PostScript
46.857143 55.142857 49.196042 57.212123 MAarrowhead1
% End of user PostScript
.43819 .63197 m
.46351 .67157 L
s
% Start of user PostScript
46.857143 59.285714 48.480758 61.825232 MAarrowhead1
% End of user PostScript
.43819 .69656 m
.45122 .73653 L
s
% Start of user PostScript
46.857143 63.428571 47.692779 65.992077 MAarrowhead1
% End of user PostScript
.43819 .76116 m
.44366 .795 L
s
% Start of user PostScript
46.857143 67.571429 47.207912 69.742207 MAarrowhead1
% End of user PostScript
.43819 .82575 m
.44058 .85401 L
s
% Start of user PostScript
46.857143 71.714286 47.009912 73.526988 MAarrowhead1
% End of user PostScript
.43819 .89034 m
.43925 .91452 L
s
% Start of user PostScript
46.857143 75.857143 46.924576 77.40802 MAarrowhead1
% End of user PostScript
.43819 .95493 m
.43866 .97619 L
s
% Start of user PostScript
46.857143 80. 46.887311 81.363336 MAarrowhead1
% End of user PostScript
.50279 .05064 m
.50383 .02381 L
s
% Start of user PostScript
51. 22. 51.067157 20.279192 MAarrowhead1
% End of user PostScript
.50279 .11523 m
.50411 .08449 L
s
% Start of user PostScript
51. 26.142857 51.085064 24.171326 MAarrowhead1
% End of user PostScript
.50279 .17982 m
.5044 .14374 L
s
% Start of user PostScript
51. 30.285714 51.103584 27.971497 MAarrowhead1
% End of user PostScript
.50279 .24442 m
.50449 .20187 L
s
% Start of user PostScript
51. 34.428571 51.108934 31.699503 MAarrowhead1
% End of user PostScript
.50279 .30901 m
.50371 .26072 L
s
% Start of user PostScript
51. 38.571429 51.059281 35.474043 MAarrowhead1
% End of user PostScript
.50279 .3736 m
.50279 .3736 L
s
% Start of user PostScript
51. 42.714286 MAarrowzero1
% End of user PostScript
.50279 .43819 m
.50279 .43819 L
s
% Start of user PostScript
51. 46.857143 MAarrowzero1
% End of user PostScript
.50279 .50279 m
.53644 .53644 L
s
% Start of user PostScript
51. 51. 53.158721 53.158721 MAarrowhead1
% End of user PostScript
.50279 .56738 m
.52977 .59727 L
s
% Start of user PostScript
51. 55.142857 52.730871 57.060085 MAarrowhead1
% End of user PostScript
.50279 .63197 m
.52509 .66371 L
s
% Start of user PostScript
51. 59.285714 52.430559 61.321272 MAarrowhead1
% End of user PostScript
.50279 .69656 m
.5185 .72815 L
s
% Start of user PostScript
51. 63.428571 52.007734 65.454266 MAarrowhead1
% End of user PostScript
.50279 .76116 m
.51283 .79021 L
s
% Start of user PostScript
51. 67.571429 51.644353 69.434579 MAarrowhead1
% End of user PostScript
.50279 .82575 m
.50914 .85157 L
s
% Start of user PostScript
51. 71.714286 51.407594 73.370354 MAarrowhead1
% End of user PostScript
.50279 .89034 m
.50691 .91318 L
s
% Start of user PostScript
51. 75.857143 51.264444 77.32167 MAarrowhead1
% End of user PostScript
.50279 .95493 m
.50551 .97541 L
s
% Start of user PostScript
51. 80. 51.174505 81.313132 MAarrowhead1
% End of user PostScript
.56738 .05064 m
.57167 .02463 L
s
% Start of user PostScript
55.142857 22. 55.418144 20.331731 MAarrowhead1
% End of user PostScript
.56738 .11523 m
.57323 .08548 L
s
% Start of user PostScript
55.142857 26.142857 55.518331 24.234894 MAarrowhead1
% End of user PostScript
.56738 .17982 m
.57519 .1448 L
s
% Start of user PostScript
55.142857 30.285714 55.643519 28.039497 MAarrowhead1
% End of user PostScript
.56738 .24442 m
.57673 .20174 L
s
% Start of user PostScript
55.142857 34.428571 55.742787 31.691393 MAarrowhead1
% End of user PostScript
.56738 .30901 m
.57385 .25632 L
s
% Start of user PostScript
55.142857 38.571429 55.558116 35.191875 MAarrowhead1
% End of user PostScript
.56738 .3736 m
.56738 .3736 L
s
% Start of user PostScript
55.142857 42.714286 MAarrowzero1
% End of user PostScript
.56738 .43819 m
.59964 .47466 L
s
% Start of user PostScript
55.142857 46.857143 57.212123 49.196042 MAarrowhead1
% End of user PostScript
.56738 .50279 m
.59727 .52977 L
s
% Start of user PostScript
55.142857 51. 57.060085 52.730871 MAarrowhead1
% End of user PostScript
.56738 .56738 m
.59364 .59364 L
s
% Start of user PostScript
55.142857 55.142857 56.827099 56.827099 MAarrowhead1
% End of user PostScript
.56738 .63197 m
.58917 .65839 L
s
% Start of user PostScript
55.142857 59.285714 56.540474 60.979996 MAarrowhead1
% End of user PostScript
.56738 .69656 m
.58427 .72259 L
s
% Start of user PostScript
55.142857 63.428571 56.226179 65.097788 MAarrowhead1
% End of user PostScript
.56738 .76116 m
.57981 .78586 L
s
% Start of user PostScript
55.142857 67.571429 55.940066 69.15614 MAarrowhead1
% End of user PostScript
.56738 .82575 m
.57627 .84858 L
s
% Start of user PostScript
55.142857 71.714286 55.713077 73.178441 MAarrowhead1
% End of user PostScript
.56738 .89034 m
.57374 .91122 L
s
% Start of user PostScript
55.142857 75.857143 55.550659 77.196364 MAarrowhead1
% End of user PostScript
.56738 .95493 m
.5719 .97408 L
s
% Start of user PostScript
55.142857 80. 55.432901 81.227749 MAarrowhead1
% End of user PostScript
.63197 .05064 m
.63916 .0265 L
s
% Start of user PostScript
59.285714 22. 59.746966 20.451704 MAarrowhead1
% End of user PostScript
.63197 .11523 m
.64196 .08797 L
s
% Start of user PostScript
59.285714 26.142857 59.926634 24.394123 MAarrowhead1
% End of user PostScript
.63197 .17982 m
.64596 .14811 L
s
% Start of user PostScript
59.285714 30.285714 60.182592 28.251538 MAarrowhead1
% End of user PostScript
.63197 .24442 m
.65196 .20512 L
s
% Start of user PostScript
59.285714 34.428571 60.567452 31.908255 MAarrowhead1
% End of user PostScript
.63197 .30901 m
.66098 .24832 L
s
% Start of user PostScript
59.285714 38.571429 61.146148 34.678976 MAarrowhead1
% End of user PostScript
.63197 .3736 m
.69032 .4253 L
s
% Start of user PostScript
59.285714 42.714286 63.028037 46.03016 MAarrowhead1
% End of user PostScript
.63197 .43819 m
.67157 .46351 L
s
% Start of user PostScript
59.285714 46.857143 61.825232 48.480758 MAarrowhead1
% End of user PostScript
.63197 .50279 m
.66371 .52509 L
s
% Start of user PostScript
59.285714 51. 61.321272 52.430559 MAarrowhead1
% End of user PostScript
.63197 .56738 m
.65839 .58917 L
s
% Start of user PostScript
59.285714 55.142857 60.979996 56.540474 MAarrowhead1
% End of user PostScript
.63197 .63197 m
.6538 .6538 L
s
% Start of user PostScript
59.285714 59.285714 60.685759 60.685759 MAarrowhead1
% End of user PostScript
.63197 .69656 m
.64952 .7182 L
s
% Start of user PostScript
59.285714 63.428571 60.411437 64.816105 MAarrowhead1
% End of user PostScript
.63197 .76116 m
.64572 .78212 L
s
% Start of user PostScript
59.285714 67.571429 60.167304 68.915808 MAarrowhead1
% End of user PostScript
.63197 .82575 m
.6425 .84562 L
s
% Start of user PostScript
59.285714 71.714286 59.960832 72.989027 MAarrowhead1
% End of user PostScript
.63197 .89034 m
.63988 .90899 L
s
% Start of user PostScript
59.285714 75.857143 59.793232 77.053504 MAarrowhead1
% End of user PostScript
.63197 .95493 m
.63785 .97238 L
s
% Start of user PostScript
59.285714 80. 59.662752 81.119145 MAarrowhead1
% End of user PostScript
.69656 .05064 m
.70598 .02919 L
s
% Start of user PostScript
63.428571 22. 64.032683 20.624113 MAarrowhead1
% End of user PostScript
.69656 .11523 m
.70951 .09174 L
s
% Start of user PostScript
63.428571 26.142857 64.258885 24.636157 MAarrowhead1
% End of user PostScript
.69656 .17982 m
.71468 .15393 L
s
% Start of user PostScript
63.428571 30.285714 64.590203 28.624978 MAarrowhead1
% End of user PostScript
.69656 .24442 m
.72333 .21598 L
s
% Start of user PostScript
63.428571 34.428571 65.145548 32.604894 MAarrowhead1
% End of user PostScript
.69656 .30901 m
.74065 .28246 L
s
% Start of user PostScript
63.428571 38.571429 66.256222 36.868852 MAarrowhead1
% End of user PostScript
.69656 .3736 m
.75019 .37484 L
s
% Start of user PostScript
63.428571 42.714286 66.867913 42.793998 MAarrowhead1
% End of user PostScript
.69656 .43819 m
.73653 .45122 L
s
% Start of user PostScript
63.428571 46.857143 65.992077 47.692779 MAarrowhead1
% End of user PostScript
.69656 .50279 m
.72815 .5185 L
s
% Start of user PostScript
63.428571 51. 65.454266 52.007734 MAarrowhead1
% End of user PostScript
.69656 .56738 m
.72259 .58427 L
s
% Start of user PostScript
63.428571 55.142857 65.097788 56.226179 MAarrowhead1
% End of user PostScript
.69656 .63197 m
.7182 .64952 L
s
% Start of user PostScript
63.428571 59.285714 64.816105 60.411437 MAarrowhead1
% End of user PostScript
.69656 .69656 m
.71435 .71435 L
s
% Start of user PostScript
63.428571 63.428571 64.569193 64.569193 MAarrowhead1
% End of user PostScript
.69656 .76116 m
.71094 .77875 L
s
% Start of user PostScript
63.428571 67.571429 64.350801 68.699581 MAarrowhead1
% End of user PostScript
.69656 .82575 m
.70797 .84281 L
s
% Start of user PostScript
63.428571 71.714286 64.160195 72.808257 MAarrowhead1
% End of user PostScript
.69656 .89034 m
.70544 .90669 L
s
% Start of user PostScript
63.428571 75.857143 63.997926 76.905628 MAarrowhead1
% End of user PostScript
.69656 .95493 m
.70339 .9705 L
s
% Start of user PostScript
63.428571 80. 63.86647 80.998617 MAarrowhead1
% End of user PostScript
.76116 .05064 m
.77192 .03231 L
s
% Start of user PostScript
67.571429 22. 68.261912 20.824227 MAarrowhead1
% End of user PostScript
.76116 .11523 m
.77562 .09599 L
s
% Start of user PostScript
67.571429 26.142857 68.499189 24.908972 MAarrowhead1
% End of user PostScript
.76116 .17982 m
.78048 .16011 L
s
% Start of user PostScript
67.571429 30.285714 68.810832 29.021171 MAarrowhead1
% End of user PostScript
.76116 .24442 m
.7871 .22571 L
s
% Start of user PostScript
67.571429 34.428571 69.235439 33.228843 MAarrowhead1
% End of user PostScript
.76116 .30901 m
.79484 .29538 L
s
% Start of user PostScript
67.571429 38.571429 69.731935 37.697399 MAarrowhead1
% End of user PostScript
.76116 .3736 m
.79828 .37041 L
s
% Start of user PostScript
67.571429 42.714286 69.952605 42.509304 MAarrowhead1
% End of user PostScript
.76116 .43819 m
.795 .44366 L
s
% Start of user PostScript
67.571429 46.857143 69.742207 47.207912 MAarrowhead1
% End of user PostScript
.76116 .50279 m
.79021 .51283 L
s
% Start of user PostScript
67.571429 51. 69.434579 51.644353 MAarrowhead1
% End of user PostScript
.76116 .56738 m
.78586 .57981 L
s
% Start of user PostScript
67.571429 55.142857 69.15614 55.940066 MAarrowhead1
% End of user PostScript
.76116 .63197 m
.78212 .64572 L
s
% Start of user PostScript
67.571429 59.285714 68.915808 60.167304 MAarrowhead1
% End of user PostScript
.76116 .69656 m
.77875 .71094 L
s
% Start of user PostScript
67.571429 63.428571 68.699581 64.350801 MAarrowhead1
% End of user PostScript
.76116 .76116 m
.77574 .77574 L
s
% Start of user PostScript
67.571429 67.571429 68.506524 68.506524 MAarrowhead1
% End of user PostScript
.76116 .82575 m
.77305 .84017 L
s
% Start of user PostScript
67.571429 71.714286 68.33431 72.639233 MAarrowhead1
% End of user PostScript
.76116 .89034 m
.77066 .90442 L
s
% Start of user PostScript
67.571429 75.857143 68.180948 76.76007 MAarrowhead1
% End of user PostScript
.76116 .95493 m
.76855 .96858 L
s
% Start of user PostScript
67.571429 80. 68.045536 80.875419 MAarrowhead1
% End of user PostScript
.82575 .05064 m
.83709 .03545 L
s
% Start of user PostScript
71.714286 22. 72.441542 21.025885 MAarrowhead1
% End of user PostScript
.82575 .11523 m
.84042 .09997 L
s
% Start of user PostScript
71.714286 26.142857 72.655098 25.16371 MAarrowhead1
% End of user PostScript
.82575 .17982 m
.84441 .16509 L
s
% Start of user PostScript
71.714286 30.285714 72.91106 29.340413 MAarrowhead1
% End of user PostScript
.82575 .24442 m
.84886 .23158 L
s
% Start of user PostScript
71.714286 34.428571 73.196718 33.605509 MAarrowhead1
% End of user PostScript
.82575 .30901 m
.85294 .30014 L
s
% Start of user PostScript
71.714286 38.571429 73.458347 38.002868 MAarrowhead1
% End of user PostScript
.82575 .3736 m
.85489 .37048 L
s
% Start of user PostScript
71.714286 42.714286 73.583117 42.514016 MAarrowhead1
% End of user PostScript
.82575 .43819 m
.85401 .44058 L
s
% Start of user PostScript
71.714286 46.857143 73.526988 47.009912 MAarrowhead1
% End of user PostScript
.82575 .50279 m
.85157 .50914 L
s
% Start of user PostScript
71.714286 51. 73.370354 51.407594 MAarrowhead1
% End of user PostScript
.82575 .56738 m
.84858 .57627 L
s
% Start of user PostScript
71.714286 55.142857 73.178441 55.713077 MAarrowhead1
% End of user PostScript
.82575 .63197 m
.84562 .6425 L
s
% Start of user PostScript
71.714286 59.285714 72.989027 59.960832 MAarrowhead1
% End of user PostScript
.82575 .69656 m
.84281 .70797 L
s
% Start of user PostScript
71.714286 63.428571 72.808257 64.160195 MAarrowhead1
% End of user PostScript
.82575 .76116 m
.84017 .77305 L
s
% Start of user PostScript
71.714286 67.571429 72.639233 68.33431 MAarrowhead1
% End of user PostScript
.82575 .82575 m
.83772 .83772 L
s
% Start of user PostScript
71.714286 71.714286 72.482125 72.482125 MAarrowhead1
% End of user PostScript
.82575 .89034 m
.83552 .90223 L
s
% Start of user PostScript
71.714286 75.857143 72.340674 76.619737 MAarrowhead1
% End of user PostScript
.82575 .95493 m
.8335 .96664 L
s
% Start of user PostScript
71.714286 80. 72.21134 80.750771 MAarrowhead1
% End of user PostScript
.89034 .05064 m
.90174 .03837 L
s
% Start of user PostScript
75.857143 22. 76.588439 21.213098 MAarrowhead1
% End of user PostScript
.89034 .11523 m
.90463 .10328 L
s
% Start of user PostScript
75.857143 26.142857 76.773342 25.37636 MAarrowhead1
% End of user PostScript
.89034 .17982 m
.90775 .16881 L
s
% Start of user PostScript
75.857143 30.285714 76.97342 29.579145 MAarrowhead1
% End of user PostScript
.89034 .24442 m
.91086 .23523 L
s
% Start of user PostScript
75.857143 34.428571 77.172863 33.839376 MAarrowhead1
% End of user PostScript
.89034 .30901 m
.9134 .30273 L
s
% Start of user PostScript
75.857143 38.571429 77.335741 38.168959 MAarrowhead1
% End of user PostScript
.89034 .3736 m
.91471 .37099 L
s
% Start of user PostScript
75.857143 42.714286 77.420253 42.547056 MAarrowhead1
% End of user PostScript
.89034 .43819 m
.91452 .43925 L
s
% Start of user PostScript
75.857143 46.857143 77.40802 46.924576 MAarrowhead1
% End of user PostScript
.89034 .50279 m
.91318 .50691 L
s
% Start of user PostScript
75.857143 51. 77.32167 51.264444 MAarrowhead1
% End of user PostScript
.89034 .56738 m
.91122 .57374 L
s
% Start of user PostScript
75.857143 55.142857 77.196364 55.550659 MAarrowhead1
% End of user PostScript
.89034 .63197 m
.90899 .63988 L
s
% Start of user PostScript
75.857143 59.285714 77.053504 59.793232 MAarrowhead1
% End of user PostScript
.89034 .69656 m
.90669 .70544 L
s
% Start of user PostScript
75.857143 63.428571 76.905628 63.997926 MAarrowhead1
% End of user PostScript
.89034 .76116 m
.90442 .77066 L
s
% Start of user PostScript
75.857143 67.571429 76.76007 68.180948 MAarrowhead1
% End of user PostScript
.89034 .82575 m
.90223 .83552 L
s
% Start of user PostScript
75.857143 71.714286 76.619737 72.340674 MAarrowhead1
% End of user PostScript
.89034 .89034 m
.9002 .9002 L
s
% Start of user PostScript
75.857143 75.857143 76.489165 76.489165 MAarrowhead1
% End of user PostScript
.89034 .95493 m
.89831 .96476 L
s
% Start of user PostScript
75.857143 80. 76.368133 80.630013 MAarrowhead1
% End of user PostScript
.95493 .05064 m
.96608 .04097 L
s
% Start of user PostScript
80. 22. 80.714814 21.379922 MAarrowhead1
% End of user PostScript
.95493 .11523 m
.96857 .10605 L
s
% Start of user PostScript
80. 26.142857 80.874734 25.553971 MAarrowhead1
% End of user PostScript
.95493 .17982 m
.97109 .17159 L
s
% Start of user PostScript
80. 30.285714 81.036028 29.757807 MAarrowhead1
% End of user PostScript
.95493 .24442 m
.97339 .23773 L
s
% Start of user PostScript
80. 34.428571 81.183613 33.999565 MAarrowhead1
% End of user PostScript
.95493 .30901 m
.97515 .30442 L
s
% Start of user PostScript
80. 38.571429 81.296629 38.277314 MAarrowhead1
% End of user PostScript
.95493 .3736 m
.97616 .37152 L
s
% Start of user PostScript
80. 42.714286 81.361108 42.580995 MAarrowhead1
% End of user PostScript
.95493 .43819 m
.97619 .43866 L
s
% Start of user PostScript
80. 46.857143 81.363336 46.887311 MAarrowhead1
% End of user PostScript
.95493 .50279 m
.97541 .50551 L
s
% Start of user PostScript
80. 51. 81.313132 51.174505 MAarrowhead1
% End of user PostScript
.95493 .56738 m
.97408 .5719 L
s
% Start of user PostScript
80. 55.142857 81.227749 55.432901 MAarrowhead1
% End of user PostScript
.95493 .63197 m
.97238 .63785 L
s
% Start of user PostScript
80. 59.285714 81.119145 59.662752 MAarrowhead1
% End of user PostScript
.95493 .69656 m
.9705 .70339 L
s
% Start of user PostScript
80. 63.428571 80.998617 63.86647 MAarrowhead1
% End of user PostScript
.95493 .76116 m
.96858 .76855 L
s
% Start of user PostScript
80. 67.571429 80.875419 68.045536 MAarrowhead1
% End of user PostScript
.95493 .82575 m
.96664 .8335 L
s
% Start of user PostScript
80. 71.714286 80.750771 72.21134 MAarrowhead1
% End of user PostScript
.95493 .89034 m
.96476 .89831 L
s
% Start of user PostScript
80. 75.857143 80.630013 76.368133 MAarrowhead1
% End of user PostScript
.95493 .95493 m
.96294 .96294 L
s
% Start of user PostScript
80. 80. 80.513534 80.513534 MAarrowhead1
% End of user PostScript
.75 g
.34687 .34687 m
.64311 .34687 L
.34687 .64311 L
.34687 .34687 L
F
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 288},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`0001/2000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00WP3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`2g0?ooo`00P@3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0I0?ooo`80
0000703oool00`000000oooo0?ooo`2J0?ooo`00P@3oool2000001/0oooo0P00000K0?ooo`800000
6`3oool2000001/0oooo0P00000K0?ooo`800000703oool00`000000oooo0?ooo`1m0?ooo`00I@3o
ool00`000000oooo0?ooo`0I0?ooo`8000006`3oool2000001/0oooo0P00000K0?ooo`8000006P3o
ool3000001/0oooo0P00000K0?ooo`8000006`3oool00`000000oooo0?ooo`1Q0?ooo`00B03oool0
0`000000oooo0?ooo`0J0?ooo`8000006P3oool3000001X0oooo0`00000J0?ooo`8000006P3oool3
000001X0oooo0`00000J0?ooo`<000006P3oool3000001X0oooo0P00001S0?ooo`00;03oool00`00
0000oooo0?ooo`0I0?ooo`8000006`3oool3000001T0oooo0`00000J0?ooo`<000006@3oool40000
01T0oooo1000000H0?ooo`D00000603oool4000001T0oooo0`00000J0?ooo`<000006`3oool00`00
0000oooo0?ooo`150?ooo`00403oool00`000000oooo0?ooo`0I0?ooo`8000006`3oool2000001X0
oooo0`00000J0?ooo`<000006@3oool4000001P0oooo1000000I0?ooo`@00000603oool5000001P0
oooo1000000I0?ooo`<000006@3oool3000001X0oooo0`00000K0?ooo`030000003oool0oooo02T0
oooo000A0?ooo`8000006P3oool2000001X0oooo1000000H0?ooo`@000006@3oool4000001P0oooo
1000000H0?ooo`D000005`3oool5000001L0oooo1P00000G0?ooo`D00000603oool4000001T0oooo
0`00000I0?ooo`<000006P3oool2000001`0oooo00<000000?ooo`3oool03@3oool00140oooo0`00
000I0?ooo`@00000603oool5000001P0oooo1000000H0?ooo`@00000603oool5000001L0oooo1@00
000G0?ooo`D000005`3oool6000001L0oooo1@00000G0?ooo`D00000603oool4000001P0oooo1000
000I0?ooo`<000006P3oool200000100oooo000B0?ooo`@00000603oool4000001P0oooo1@00000G
0?ooo`D000005`3oool5000001L0oooo0`00000I0?ooo`<000006`3oool3000001T0oooo1000000F
0?ooo`H000005P3oool5000001P0oooo1@00000F0?ooo`D000006@3oool3000001T0oooo1000000@
0?ooo`004P3oool6000001H0oooo1@00000G0?ooo`H000005P3oool4000001P0oooo1000000K0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo01T0oooo1000000G0?ooo`@000005`3oool5000001H0oooo1P00000G0?oo
o`D000005`3oool500000140oooo000C0?ooo`H000005P3oool5000001L0oooo1000000G0?ooo`80
000000<0oooo0000003oool06P3oool00`000000oooo0000000I0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0A0?ooo`<00000603oool4000001L0
oooo1@00000F0?ooo`D000005P3oool600000180oooo000D0?ooo`@000005`3oool4000001P0oooo
0P00000h0?ooo`800000TP3oool2000002h0oooo0P00000H0?ooo`<00000603oool4000001L0oooo
1000000C0?ooo`00503oool3000001T0oooo00<000000?ooo`3oool0D03oool2000009H0oooo0`00
00170?ooo`030000003oool0oooo01L0oooo0P00000I0?ooo`<000004`3oool001D0oooo00<00000
0?ooo`3oool0I`3oool4000009/0oooo0P00001P0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool04P3oool007`0oooo0`00002Q0?ooo`@00000S03oool007/0oooo0P00002W0?ooo`80
0000RP3oool007P0oooo0`00002[0?ooo`800000R03oool007H0oooo0P00002`0?ooo`800000QP3o
ool007<0oooo0`00002d0?ooo`800000Q03oool00740oooo0P00002i0?ooo`800000PP3oool00700
oooo00<000000?ooo`3oool0^`3oool200000800oooo001^0?ooo`800000`03oool2000007h0oooo
001[0?ooo`<00000a03oool2000007`0oooo001Y0?ooo`800000b@3oool2000007X0oooo001X0?oo
o`030000003oool0oooo0</0oooo0P00001h0?ooo`00IP3oool2000005<0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo03h0oooo0P00001f0?ooo`00
I03oool2000003L0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0K0?ooo`030000
003oool0oooo01X0oooo0P00000L0?ooo`030000003oool0oooo02<0oooo0P00001d0?ooo`00HP3o
ool2000001`0oooo00<000000?ooo`3oool06P3oool2000001`0oooo0P00000K0?ooo`800000703o
ool2000001/0oooo0P00000W0?ooo`800000LP3oool00600oooo0P00000N0?ooo`8000006`3oool2
000001`0oooo0P0000080?oooaL000006@3oool3000001/0oooo0P00000L0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool0K`3oool005h0oooo0P0000030?ooo`030000003oool0oooo
01X0oooo0P00000K0?ooo`<000006P3oool;00000180oooo100000000`3oool000000000000:0000
00d0oooo0`00000J0?ooo`<000006`3oool2000000d0oooo0P00001_0?ooo`00A`3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo00<0oooo0P00000K0?ooo`<000006P3oool300000180
oooo3000000I0?ooo`@000003@3oool8000000@0oooo1000000I0?ooo`@000006P3oool3000000l0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`1O0?ooo`00A`3oool2000001<0oooo
00<000000?ooo`3oool01@3oool2000001X0oooo1000000I0?ooo`@000003@3oool4000000P0oooo
1000000I0?ooo`@000005@3oool8000001T0oooo0`00000J0?ooo`<000004@3oool00`000000oooo
0?ooo`070?ooo`800000HP3oool002/0oooo00<000000?ooo`3oool06P3oool200000100oooo0P00
00080?ooo`<000006P3oool4000001P0oooo1@0000040?ooo`P00000303oool5000001L0oooo1P00
000G0?ooo`D00000603oool4000001X0oooo0`00000B0?ooo`030000003oool0oooo00D0oooo0`00
000L0?ooo`030000003oool0oooo04<0oooo000?0?ooo`030000003oool0oooo01X0oooo0P00000J
0?ooo`@00000303oool2000000X0oooo1000000I0?ooo`@00000603oool9000001@0oooo1@00000G
0?ooo`H000005`3oool9000001@0oooo1000000I0?ooo`@000004`3oool00`000000oooo0?ooo`03
0?ooo`<000006`3oool2000001/0oooo00<000000?ooo`3oool0:03oool00100oooo0P00000J0?oo
o`@00000603oool5000000X0oooo00<000000?ooo`3oool02`3oool4000001P0oooo1@00000G0?oo
o`H000005`3oool3000001/0oooo1000000I0?ooo`<00000103oool5000000h0oooo1@00000H0?oo
o`D00000503oool2000000030?ooo`000000000000<000006@3oool4000001T0oooo0P00000K0?oo
o`8000003P3oool00100oooo1000000I0?ooo`D000005`3oool5000000L0oooo0P00000>0?ooo`D0
00005`3oool4000001D0oooo1`00000K0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`090?ooo`<000003@3oool3000001L0oooo1@00000G0?ooo`D0
0000603oool5000001P0oooo1000000I0?ooo`<000003`3oool00140oooo1@00000G0?ooo`H00000
5P3oool6000000D0oooo00<000000?ooo`3oool03`3oool3000001P0oooo0P0000000`3oool00000
0?ooo`0A0?ooo`<000001P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0=0?ooo`<000002P3oool00`000000oooo
0?ooo`0H0?ooo`@00000603oool4000001H0oooo1P00000G0?ooo`D000005`3oool500000100oooo
000B0?ooo`H000005P3oool4000001P0oooo100000050?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`000000703oool00`000000oooo0?ooo`0=0?ooo`8000002P3oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?oo
o`0@0?ooo`@000001@3oool00`000000oooo0?ooo`0I0?ooo`040000003oool00000000001T0oooo
0`00000G0?ooo`@000005P3oool6000001H0oooo1P00000A0?ooo`004`3oool4000001L0oooo1000
000H0?ooo`80000000<0oooo0000003oool00`3oool00`000000oooo0?ooo`0D0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool02P3oool3000000`0oooo00<000000?ooo`3oool0603o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0503o
ool300000080oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`050000003o
ool000000?ooo`0000005P3oool4000001L0oooo1000000G0?ooo`H000004@3oool001<0oooo0`00
000I0?ooo`030000003oool0oooo01h0oooo0P00000m0?ooo`<00000O03oool2000003d0oooo00<0
00000?ooo`3oool05@3oool00`000000oooo0?ooo`0G0?ooo`<00000603oool400000180oooo000D
0?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3oool0>`3oool200000840oooo0`00000k
0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool05`3oool3000001<0oooo001>0?oo
o`030000003oool0oooo03T0oooo0`0000260?ooo`<00000>@3oool00`000000oooo0?ooo`190?oo
o`030000003oool0oooo0180oooo001<0?ooo`800000>@3oool3000008`0oooo0P00000h0?ooo`03
0000003oool0oooo05d0oooo001;0?ooo`030000003oool0oooo03L0oooo0P00002A0?ooo`800000
=`3oool00`000000oooo0?ooo`1L0?ooo`00BP3oool00`000000oooo0?ooo`0g0?ooo`030000003o
ool0oooo09<0oooo0P00000f0?ooo`800000G03oool004X0oooo00<000000?ooo`3oool0=@3oool2
000009P0oooo0P00000f0?ooo`030000003oool0oooo05T0oooo00180?ooo`800000=P3oool20000
09`0oooo0P00000e0?ooo`030000003oool0oooo05P0oooo00170?ooo`030000003oool0oooo03@0
oooo0P00002P0?ooo`800000=03oool00`000000oooo0?ooo`1G0?ooo`00AP3oool00`000000oooo
0?ooo`0c0?ooo`800000Y03oool2000003<0oooo00<000000?ooo`3oool0EP3oool004D0oooo00<0
00000?ooo`3oool0<P3oool200000:P0oooo0P00000b0?ooo`030000003oool0oooo05D0oooo0014
0?ooo`030000003oool0oooo0340oooo0P00001O0?ooo`030000003oool0oooo04X0oooo0P00000a
0?ooo`030000003oool0oooo05@0oooo00130?ooo`030000003oool0oooo0300oooo0P0000120?oo
o`8000007@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo02h0oooo0P00000`0?oo
o`030000003oool0oooo05<0oooo00120?ooo`030000003oool0oooo02l0oooo0P00000V0?ooo`03
0000003oool0oooo01/0oooo0P00000L0?ooo`800000703oool2000001h0oooo00<000000?ooo`3o
ool04@3oool2000002l0oooo00<000000?ooo`3oool0DP3oool00440oooo00<000000?ooo`3oool0
;P3oool2000000X0oooo00<000000?ooo`3oool06`3oool2000001`0oooo0P00000L0?ooo`<00000
6`3oool2000001d0oooo0P00000E0?ooo`800000;P3oool00`000000oooo0?ooo`1A0?ooo`00@03o
ool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo00X0oooo0P00000L0?ooo`800000703o
ool200000080oooo9P00000A0?ooo`<00000703oool3000001H0oooo00<000000?ooo`3oool0;@3o
ool00`000000oooo0?ooo`1@0?ooo`00?`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0
oooo00/0oooo0`00000K0?ooo`<000005`3oool8000001T0oooo100000090?ooo`P000002@3oool3
000001/0oooo0`00000H0?ooo`800000103oool00`000000oooo0?ooo`0W0?ooo`030000003oool0
oooo04l0oooo000n0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool02P3oool20000
00l0oooo0`00000J0?ooo`@000004P3oool4000000<0oooo1000000J0?ooo`@000004@3oool40000
00@0oooo1@00000J0?ooo`<000006P3oool2000000030?ooo`000000oooo02X0oooo00<000000?oo
o`3oool0CP3oool003d0oooo00<000000?ooo`3oool08@3oool3000000P0oooo0P00000A0?ooo`<0
00006`3oool4000000d0oooo100000070?ooo`D000006@3oool5000001@0oooo2@00000I0?ooo`@0
0000703oool2000002`0oooo00<000000?ooo`3oool0C@3oool003`0oooo00<000000?ooo`3oool0
1P3oool00`000000oooo0?ooo`0J0?ooo`<000001P3oool00`000000oooo0?ooo`0A0?ooo`@00000
6P3oool4000000T0oooo1000000;0?ooo`D00000603oool6000001L0oooo1P00000H0?ooo`D00000
6P3oool4000001`0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1<0?ooo`00>`3o
ool00`000000oooo0?ooo`080?ooo`8000006P3oool4000000<0oooo0P00000D0?ooo`D000006@3o
ool5000000@0oooo1000000?0?ooo`H000005`3oool6000001L0oooo2@00000D0?ooo`D000006P3o
ool4000000030?ooo`000000000001P0oooo0P00000B0?ooo`030000003oool0oooo04/0oooo000Y
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02@3oool3000001X0oooo1P00000G
0?ooo`D00000603oool300000080oooo1000000C0?ooo`@000006`3oool00`000000oooo0?ooo`0J
0?ooo`@000000`3oool400000140oooo1000000I0?ooo`@00000103oool00`000000oooo0?ooo`0C
0?ooo`@000004P3oool00`000000oooo0?ooo`050?ooo`800000A03oool000h0oooo0P00000J0?oo
o`8000003@3oool00`000000oooo0?ooo`0;0?ooo`@00000603oool6000001L0oooo1000000K0?oo
o`<000006P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool0203oool3000000l0oooo0`00000H0?ooo`D000001@3oool00`000000oooo0?ooo`0@0?oo
o`D00000503oool2000000<0oooo0`00000K0?ooo`800000:03oool000l0oooo0`00000I0?ooo`<0
00002`3oool00`000000oooo0?ooo`0;0?ooo`H000005`3oool4000001P0oooo0P0000000`3oool0
00000?ooo`0E0?ooo`D0000000<0oooo0000003oool06P3oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool02`3oool3000000`0oooo00<000000?ooo`3o
ool06@3oool3000000L0oooo0P00000>0?ooo`H000005P3oool7000001P0oooo1000000J0?ooo`80
00003@3oool00100oooo1@00000G0?ooo`@00000203oool00`000000oooo0?ooo`0=0?ooo`H00000
5P3oool4000001/0oooo00<000000?ooo`3oool04@3oool3000000H0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
3P3oool2000000T0oooo00<000000?ooo`3oool06P3oool00`000000oooo000000090?ooo`030000
003oool0oooo00`0oooo1000000F0?ooo`L000005P3oool6000001P0oooo1000000>0?ooo`004@3o
ool6000001D0oooo1P0000040?ooo`8000004@3oool4000001D0oooo0P000000103oool000000?oo
o`00000L0?ooo`030000003oool0oooo00h0oooo0P0000090?ooo`030000003oool0oooo01T0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0140oooo
0`0000060?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool02`3oool2000000d0oooo
0`00000G0?ooo`D000005@3oool7000001H0oooo1P00000?0?ooo`004@3oool5000001L0oooo1000
00040?ooo`030000003oool0oooo0140oooo1000000D0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool06@3oool00`000000oooo0?ooo`0;0?ooo`<00000303oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?oo
o`0D0?ooo`@0000000<0oooo0000003oool06`3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`000000603oool4000001L0oooo1P00000D0?ooo`L00000403o
ool00180oooo1000000H0?ooo`@000000`3oool00`000000oooo0?ooo`0B0?ooo`040000003oool0
oooo00000180oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo00P0oooo0P00000?0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo0P00000K0?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool00000008000005`3oool5000001H0oooo1000000B0?ooo`004`3oool2000001X0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo02`0oooo0P000021
0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0903oool00`000000oooo0000000G
0?ooo`<000005`3oool3000001<0oooo000D0?ooo`030000003oool0oooo01`0oooo00<000000?oo
o`3oool09`3oool2000002`0oooo0`0000240?ooo`800000;03oool00`000000oooo0?ooo`0V0?oo
o`030000003oool0oooo01D0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0B0?oo
o`00<P3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02X0oooo0P0000290?ooo`@0
0000:@3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo04<0oooo000a0?ooo`030000
003oool0oooo02L0oooo00<000000?ooo`3oool0:@3oool2000008l0oooo00<000000?ooo`3oool0
9`3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo0480oooo000a0?ooo`030000003o
ool0oooo02H0oooo00<000000?ooo`3oool0:03oool200000980oooo00<000000?ooo`3oool09`3o
ool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo0480oooo000`0?ooo`030000003oool0
oooo02H0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`2C0?ooo`800000:03oool0
0`000000oooo0?ooo`0U0?ooo`030000003oool0oooo0440oooo000_0?ooo`030000003oool0oooo
02H0oooo00<000000?ooo`3oool09`3oool2000009P0oooo0`00000V0?ooo`030000003oool0oooo
02D0oooo00<000000?ooo`3oool0@03oool002h0oooo00<000000?ooo`3oool09P3oool00`000000
oooo0?ooo`0W0?ooo`030000003oool0oooo09/0oooo0P00000U0?ooo`030000003oool0oooo02@0
oooo00<000000?ooo`3oool0@03oool002h0oooo00<000000?ooo`3oool09@3oool00`000000oooo
0?ooo`0W0?ooo`030000003oool0oooo07D0oooo00<000000?ooo`3oool09P3oool00`000000oooo
0?ooo`0R0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0?`3oool002d0oooo00<0
00000?ooo`3oool09@3oool00`000000oooo0?ooo`0V0?ooo`8000006`3oool00`000000oooo0?oo
o`0L0?ooo`030000003oool0oooo00D0oooo7P00000G0?ooo`800000:@3oool2000002<0oooo00<0
00000?ooo`3oool0903oool00`000000oooo0?ooo`0o0?ooo`00;03oool00`000000oooo0?ooo`0U
0?ooo`030000003oool0oooo02D0oooo0P00000M0?ooo`030000003oool0oooo01`0oooo2000000G
0?ooo`030000003oool0oooo00@0oooo2@00000>0?ooo`8000007`3oool00`000000oooo0?ooo`09
0?ooo`030000003oool0oooo0240oooo0P00000U0?ooo`030000003oool0oooo03h0oooo000[0?oo
o`030000003oool0oooo02D0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0M0?oo
o`8000006@3oool6000001`0oooo0P00000?0?ooo`L000001P3oool3000001h0oooo0P00000<0?oo
o`030000003oool0oooo0280oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0m0?oo
o`00:`3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo02@0oooo0`00000O0?ooo`80
00004@3oool8000000@0oooo0P00000L0?ooo`<000005@3oool400000080oooo0`00000N0?ooo`80
00003@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3o
ool0?03oool002X0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0T0?ooo`030000
003oool0oooo008000007P3oool3000000`0oooo1000000;0?ooo`<000006`3oool4000001T0oooo
1@00000M0?ooo`8000003`3oool200000280oooo00<000000?ooo`3oool08`3oool00`000000oooo
0?ooo`0l0?ooo`00:P3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo02<0oooo0P00
00040?ooo`8000007@3oool3000000P0oooo1000000?0?ooo`@000006P3oool4000001X0oooo1`00
000I0?ooo`<000004@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo02<0oooo00<0
00000?ooo`3oool0>`3oool002T0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0S
0?ooo`030000003oool0oooo00@0oooo1000000K0?ooo`@00000103oool3000001<0oooo1000000J
0?ooo`@000006@3oool5000000<0oooo1000000D0?ooo`@000004P3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0k
0?ooo`00:03oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`050?ooo`D000006P3oool400000080oooo0P00000F0?oo
o`D00000603oool6000001P0oooo1@0000070?ooo`<000004@3oool4000001<0oooo0P0000080?oo
o`8000005P3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo03X0oooo000X0?ooo`03
0000003oool0oooo02<0oooo00<000000?ooo`3oool03P3oool200000140oooo0P0000090?ooo`D0
00006@3oool6000001P0oooo1@00000H0?ooo`H000006P3oool3000000X0oooo0P00000>0?ooo`@0
00005P3oool00`000000oooo0?ooo`030?ooo`@000005`3oool00`000000oooo0?ooo`0R0?ooo`03
0000003oool0oooo03X0oooo000W0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0
3`3oool3000000l0oooo00<000000?ooo`3oool02@3oool6000001P0oooo1000000L0?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0<0?ooo`<00000
303oool3000001L0oooo00<000000?ooo`3oool01@00000I0?ooo`030000003oool0oooo0280oooo
00<000000?ooo`3oool0>@3oool002H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo0140oooo1000000<0?ooo`030000003oool0oooo00X0oooo1@00
000G0?ooo`8000000`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool06`3oool00`000000oooo0?ooo`0?0?ooo`8000002P3oool00`000000oooo0000
000H0?ooo`L000006P3oool010000000oooo00000000000P0?ooo`030000003oool0oooo03T0oooo
000V0?ooo`030000003oool0oooo01/0oooo0P0000050?ooo`030000003oool0oooo0180oooo1P00
00090?ooo`030000003oool0oooo00`0oooo0`00000E0?ooo`<000001P3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`0B0?ooo`8000001`3oool00`000000oooo0?ooo`0J0?ooo`D000006P3oool400000280oooo00<0
00000?ooo`3oool0>03oool002D0oooo00<000000?ooo`3oool00P00000J0?ooo`@000000`3oool0
0`000000oooo0?ooo`0C0?ooo`H000001P3oool2000000l0oooo00@000000?ooo`3oool000004@3o
ool3000000T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool0503oool2000000D0oooo00<000000?ooo`3oool06`3o
ool4000001L0oooo1P00000J0?ooo`8000001`3oool00`000000oooo0?ooo`0h0?ooo`009@3oool0
10000000oooo0?ooo`3oool3000001T0oooo1P00000G0?ooo`@000001P3oool00`000000oooo0?oo
o`0C0?ooo`030000003oool0oooo00`0oooo0P00000<0?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01H0oooo0P00
00020?ooo`030000003oool0oooo01`0oooo0`00000F0?ooo`L0000000<0oooo0000003oool05P3o
ool3000000T0oooo00<000000?ooo`3oool0=`3oool000d0oooo1000000C0?ooo`030000003oool0
oooo00<0oooo1@00000G0?ooo`H000005P3oool4000000H0oooo00<000000?ooo`3oool04`3oool0
0`000000oooo0?ooo`0:0?ooo`8000003`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0
oooo01T0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0H0?ooo`<000007@3oool0
10000000oooo0?ooo`00000G0?ooo`H000000P3oool00`000000oooo0?ooo`0A0?ooo`D000002`3o
ool00`000000oooo0?ooo`0:0?ooo`D000009`3oool000h0oooo2000000>0?ooo`030000003oool0
oooo00@0oooo1P00000F0?ooo`@00000603oool010000000oooo0?ooo`0000040?ooo`030000003o
ool0oooo01D0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo01T0oooo0`00000K0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0503o
ool5000000<0oooo00<000000?ooo`3oool03`3oool6000000`0oooo00<000000?ooo`3oool01@3o
ool9000001L0oooo1@00000<0?ooo`003`3oool6000000h0oooo00<000000?ooo`3oool01P3oool4
000001L0oooo1@00000K0?ooo`040000003oool00000000001T0oooo00<000000?ooo`3oool01@3o
ool200000180oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`02000001P0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`0C0?ooo`@000001@3oool00`000000oooo0?oo
o`0?0?ooo`@000003P3oool00`000000oooo0?ooo`040?ooo`L00000503oool9000000d0oooo000@
0?ooo`D000003P3oool00`000000oooo0?ooo`070?ooo`@000005P3oool2000000<0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool04`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0100oooo0P000002
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0303oool5000000l0oooo00<00000
0?ooo`3oool01@3oool5000001D0oooo1`00000?0?ooo`004P3oool4000000`0oooo00<000000?oo
o`3oool02@3oool010000000oooo0?ooo`00000D0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0000000K0?ooo`040000003oool0oooo0?ooo`8000005P3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00@0oooo0P00000C0?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00/0oooo00D000000?ooo`3oool0oooo0000000@0?ooo`030000003oool0oooo00@0oooo
1@00000G0?ooo`D00000403oool001<0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool07P3oool2000004P0oooo0`00
00150?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`0P0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool05`3oool400000140oooo000Q
0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0L
0?ooo`800000=`3ooolC000000<0oooo4000000f0?ooo`<00000703oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0A
0?ooo`008@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo01d0oooo0P00000N0?oo
o`030000003oool0oooo01<0oooo00<000000?ooo`3oool06@3oool8000002H0oooo2000000_0?oo
o`<000006`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0200oooo00<000000?oo
o`3oool0<`3oool00200oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo01h0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0E0?ooo`@0
0000=P3oool4000002X0oooo0`0000000`3oool000000?ooo`0J0?ooo`030000003oool0oooo01/0
oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0c0?ooo`00803oool00`000000oooo
0?ooo`0P0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool07@3oool2000001H0oooo
0P00000B0?ooo`@00000?P3oool800000240oooo100000020?ooo`8000006@3oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0<P3oool001l0oooo00<0
00000?ooo`3oool0803oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01/0oooo0P00
000H0?ooo`8000003P3oool4000000d0oooo00<000000?ooo`3oool0>@3oool4000001h0oooo0`00
00050?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo0380oooo000O0?ooo`030000003oool0oooo0200oooo00<00000
0?ooo`3oool0703oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01P0oooo0`000009
0?ooo`@000004@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01X0oooo0P000003
0?ooo`@000006@3oool4000000H0oooo00<000000?ooo`3oool05`3oool2000001/0oooo00<00000
0?ooo`3oool0803oool00`000000oooo0?ooo`0a0?ooo`007P3oool00`000000oooo0?ooo`0P0?oo
o`030000003oool0oooo01`0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0I0?oo
o`<000001@3oool4000001D0oooo0P00000L0?ooo`030000003oool0oooo01X0oooo0P0000070?oo
o`8000005P3oool5000000L0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo01l0oooo00<000000?ooo`3oool0<@3oool001h0oooo00<000000?ooo`3o
ool07`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool06P3oool4000000040?ooo`0000000000000001P0oooo0`00000K0?ooo`8000006`3oool30000
00T0oooo0`00000D0?ooo`@00000203oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0a0?ooo`007@3oool00`000000
oooo0?ooo`0P0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`0K0?ooo`D000006`3oool3000001/0oooo0`00000J0?ooo`@000002`3oool300000140
oooo00@000000?ooo`00000000002@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0`0?ooo`007@3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool04`3oool00`000000
oooo0?ooo`030?ooo`8000007P3oool5000001/0oooo1000000I0?ooo`@000006P3oool4000000h0
oooo0`00000>0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool05P3oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0<03oool001`0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01D0oooo
0P0000020?ooo`030000003oool0oooo01/0oooo1`00000L0?ooo`@000006@3oool4000001T0oooo
1@00000A0?ooo`8000002`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0300oooo
000L0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`0F0?ooo`@00000703oool2000000H0oooo00<000000?ooo`3oool06@3oool5000001T0oooo1000
000I0?ooo`D000004`3oool2000000T0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0F0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`0_0?ooo`006`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01/0oooo00<00000
0?ooo`3oool0603oool4000001T0oooo0P0000080?ooo`030000003oool0oooo01T0oooo1P00000G
0?ooo`H000006P3oool3000001D0oooo0P0000060?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool05@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01h0oooo00<00000
0?ooo`3oool0;`3oool001/0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo01P0oooo1P00000G0?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool06@3oool6000001L0oooo1P00000J0?ooo`030000003oool0oooo01L0oooo0P0000040?oo
o`030000003oool0oooo0140oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01l0oooo00<000000?ooo`3oool0;P3oool001/0oooo00<000000?ooo`3o
ool07P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`00
00001@00000D0?ooo`8000002`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01/0
oooo0`00000J0?ooo`030000003oool0oooo01T0oooo00D000000?ooo`3oool0oooo0000000D0?oo
o`030000003oool0oooo0100oooo0P0000020?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool07`3oool00`000000oooo0?ooo`0^0?ooo`006P3oool00`000000oooo0?ooo`0O0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool0603oool010000000oooo0?ooo`3oool40000
01<0oooo0P00000=0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06`3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo01/0oooo0`00000E0?ooo`030000003oool0oooo
00d0oooo100000040?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07`3oool00`00
0000oooo0?ooo`0]0?ooo`006P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01/0
oooo0`00000G0?ooo`030000003oool0oooo0080oooo1000000A0?ooo`8000003`3oool00`000000
oooo0?ooo`0K0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool02@3oool6000000D0
oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo02d0
oooo000I0?ooo`800000803oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00@00000
503oool00`000000oooo0?ooo`030?ooo`040000003oool0oooo000000h0oooo0P00000A0?ooo`03
0000003oool0oooo01/0oooo00<000000?ooo`3oool02P3ooolC000001T0oooo00<000000?ooo`3o
ool0703oool010000000oooo00000000000E0?ooo`030000003oool0oooo00H0oooo1`0000070?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0]0?oo
o`006@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool00P3oool600000100oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool0603oool?000000l0oooo00@000000?ooo`3o
ool0oooo3000000=0?ooo`030000003oool0oooo01`0oooo00@000000?ooo`3oool0oooo0P00000D
0?ooo`030000003oool0oooo00D0oooo1P0000080?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool07P3oool00`000000oooo0?ooo`0]0?ooo`00603oool00`000000oooo0?ooo`0O0?oo
o`030000003oool0oooo00<0oooo0`00000D0?ooo`030000003oool0oooo00<0oooo1`00000=0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool01`3oool2000001D0oooo00<000000?oo
o`3oool0403oool800000080oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0=0?oo
o``0000000<0oooo0000003oool0703oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool01P3oool4000000X0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0M0?ooo`030000003oool0oooo02d0oooo000H0?ooo`030000003oool0oooo01h0
oooo00<000000?ooo`3oool01@3oool6000000l0oooo00<000000?ooo`3oool0103oool6000000d0
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01D0
oooo00<000000?ooo`3oool0303oool4000000X0oooo00<000000?ooo`3oool06@3oool00`000000
oooo0?ooo`0I0?ooo`@000006`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool00`3oool5000000/0oooo00<000000?ooo`3oool02P3oool5000000T0
oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0/0?ooo`00603oool00`000000oooo
0?ooo`0;0?ooo`D000003P3oool00`000000oooo0?ooo`060?ooo`P00000303oool00`000000oooo
0?ooo`050?ooo`D00000303oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`030?ooo`P000003P3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool01000000F0?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool04@3oool01@000000oooo0?ooo`3oool0000000<0
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`040?ooo`T000002`3oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo02`0oooo000;0?ooo`D00000203oool00`000000oooo
0?ooo`0<0?ooo`T00000203oool00`000000oooo0?ooo`080?ooo`H00000303oool00`000000oooo
0?ooo`070?ooo`<0000000<0oooo0000003oool02@3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool05`3oool00`000000oooo00000003000001H0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00@0oooo
1`00000?0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0403oool010000000oooo
00000000000B0?ooo`030000003oool0oooo00@0oooo1`00000=0?ooo`030000003oool0oooo0080
oooo2P00000A0?ooo`030000003oool0oooo02`0oooo000<0?ooo`T000000P3oool00`000000oooo
0?ooo`0?0?ooo`H000002@3oool00`000000oooo0?ooo`0:0?ooo`@000002`3oool00`000000oooo
0?ooo`090?ooo`040000003oool0oooo0?ooo`800000203oool00`000000oooo0?ooo`0A0?ooo`04
0000003oool0oooo000001T0oooo0`00000J0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`0<0?ooo`800000303oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo00l0oooo0P00000E0?ooo`030000003oool0oooo00@0oooo1@00000>0?oo
o`030000003oool0oooo0080oooo2@00000A0?ooo`X000009P3oool000h0oooo1`0000020?ooo`03
0000003oool0oooo0100oooo1@0000080?ooo`030000003oool0oooo00`0oooo0P0000000`3oool0
0000000000080?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0C0?ooo`800000603oool2000000030?ooo`000000oooo01X0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00h0oooo0P00000:0?ooo`03
0000003oool0oooo00/0oooo00<000000?ooo`3oool03@3oool00`000000oooo0000000E0?ooo`03
0000003oool0oooo0080oooo1P00000@0?ooo`030000003oool0oooo0080oooo1P00000C0?ooo`T0
00004P3oool:000000/0oooo000?0?ooo`D000000`3oool00`000000oooo0?ooo`0A0?ooo`D00000
1`3oool00`000000oooo0?ooo`0=0?ooo`040000003oool0oooo0?ooo`8000001P3oool00`000000
oooo0?ooo`0A0?ooo`8000000`3oool2000001D0oooo0P00000E0?ooo`<00000103oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000
oooo0?ooo`0@0?ooo`@000001@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00X0
oooo0P0000030?ooo`030000003oool0oooo01<0oooo00@000000?ooo`00000000000`3oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo00L00000503oool7000001@0oooo2@00000<0?oo
o`004@3oool6000001H0oooo00<000000?ooo`3oool00P0000040?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0000001H0
oooo00@000000?ooo`3oool000004@3oool3000000L0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0503oool30000
0080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool04P3oool2000001P0oooo0`0000030?ooo`<00000503oool60000
01H0oooo1`00000>0?ooo`004`3oool010000000oooo0?ooo`00000O0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo
0140oooo0P00001g0?ooo`8000004P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
01H0oooo0`00000H0?ooo`@00000403oool001D0oooo00<000000?ooo`3oool07@3oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04P3oool200000180
oooo0P00001k0?ooo`<00000403oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0180
oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01H0
oooo0P00000B0?ooo`00=@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo07h0oooo
0P00000>0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0:P3oool003D0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool0403oool2000008<0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0L0?ooo`030000003oool0oooo02X0oooo000d0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool05P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool0Q03oool2000000`0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0B
0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0Y
0?ooo`00=03oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01H0oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo08L0oooo00<000000?oo
o`3oool02P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0140oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02T0oooo000c0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool0R03oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo01d0oooo00<000000?ooo`3oool0:@3oool003<0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`0W0?ooocX00000:@3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0F0?ooo`030000
003oool0oooo01d0oooo00<000000?ooo`3oool0:@3oool00380oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0H0?oooa000000>P3oool@000001X0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool05`3o
ool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02P0oooo000b0?ooo`030000003oool0
oooo01T0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool0403oool9000005X0oooo2@00000B0?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0X0?ooo`00<P3oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo00X0oooo1`00001/0?ooo`X00000203oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`0F0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0:03oool00340
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0180
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`080?ooo`<00000O@3oool3000000H0
oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0180
oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02P0
oooo000a0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool05@3oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`210?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool05`3oool00`000000oooo
0?ooo`0M0?ooo`030000003oool0oooo02L0oooo000a0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`8000003P3oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo0880oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`0F0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool09`3oool00300oooo00<00000
0?ooo`3oool0603oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo08<0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool04P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01d0oooo00<00000
0?ooo`3oool09`3oool00300oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0D0?oo
o`030000003oool0oooo0140oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`080?oo
o`030000003oool0oooo08<0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo01d0oooo00<000000?ooo`3oool09`3oool002l0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0080ooooO`2o_kl40?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo0140oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0L0?ooo`03
0000003oool0oooo02L0oooo000_0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
5@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`020?ooogh0_kno1@3oool00`000000oooo0?ooo`030?ooo`030000
003oool0oooo00P0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo01L0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0W0?ooo`00;`3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool00P3o
oomm0;no_`D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0G0?ooo`030000003o
ool0oooo01`0oooo00<000000?ooo`3oool09`3oool002h0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo0080ooooO02o_kl50?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0
oooo02L0oooo000^0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`090?ooo`X000003P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
0080ooooN`2o_kl60?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`0L0?ooo`030000003oool0oooo02H0oooo000^0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool02`3oool:000000`0oooo2@00000>0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool00P3ooomj0;no_`H0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool09P3oool002h0oooo
00<000000?ooo`3oool03P3oool4000000H0oooo00<000000?ooo`3oool0303oool9000000h0oooo
1P00000?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool00P3ooomi0;no_`H0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01`0oooo
00<000000?ooo`3oool09P3oool002D0oooo2P00000A0?ooo`P0000000<0oooo0000003oool03`3o
ool600000140oooo2000000:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool00`3o
oomh0;no_`H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0G0?ooo`030000003o
ool0oooo01`0oooo00<000000?ooo`3oool09P3oool000X0oooo2P00000B0?ooo`T000004`3oool7
000001<0oooo1P00000@0?ooo`<00000103oool4000000H0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`030?ooogL0_kno1`3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0B0?ooo`D000004`3oool5
000001D0oooo1@0000000`3oool000000?ooo`0B0?ooo`<000004P3oool000/0oooo2@00000D0?oo
o`L00000503oool6000001D0oooo0P0000020?ooo`<000003@3oool00`000000oooo0?ooo`080?oo
o`<000000P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00<0ooooMP2o_kl70?oo
o`030000003oool0oooo00P0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0=0?oo
o`<000004P3oool:000000h0oooo2P00000@0?ooo`X00000403oool7000000h0oooo000=0?ooo`L0
00005@3oool7000001@0oooo0`0000000`3oool000000000000200000180oooo00<000000?ooo`3o
ool0103oool3000000X0oooo00<000000?ooo`3oool02`3oool4000000/0oooo00<000000?ooo`3o
ool00`3ooome0;no_`L0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo00d0oooo1`00000<0?ooo`X00000403oool800000180oooo2000000B0?ooo`X0
00002`3oool000l0oooo100000000`3oool000000000000E0?ooo`<000000P3oool2000001@0oooo
0P0000040?ooo`800000403oool00`000000oooo0?ooo`070?ooo`8000001`3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo008000002@3oool00`000000oooo0?ooo`030?ooog@0_kno
1`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`000000
5`0000040?ooo`D000000P3oool5000000l0oooo100000000`3oool000000000000300000180oooo
1`0000000`3oool000000?ooo`0B0?ooo`P000003@3oool00140oooo0P00000I0?ooo`8000006@3o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00<0ooooL`2o_kl80?ooo`8000002P3o
ool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00d0oooo1`00000>0?ooo`8000005`3o
ool2000001L0oooo0P0000050?ooo`030000003oool0oooo0140oooo1@00000?0?ooo`00;03oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool00`3ooomb
0;no_`P0oooo00@000000?ooo`3oool000002@3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo00d0oooo0`00000B0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0703o
ool00`000000oooo0?ooo`0A0?ooo`8000004P3oool002`0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00<0ooooL@2o_kl80?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0
oooo01`0oooo00<000000?ooo`3oool09@3oool002/0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo00@0ooooL02o_kl80?ooo`030000003oool0oooo
00@0oooo0P0000050?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0703oool00`00
0000oooo0?ooo`0U0?ooo`00:`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool0103ooom_0;no_`P0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`0B0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0703oool00`000000
oooo0?ooo`0U0?ooo`00:`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0103ooom^0;no_`T0oooo00<000000?ooo`3oool0203oool01@000000oooo
0?ooo`3oool0000000d0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo01L0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0U0?ooo`00
:`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0
103ooom]0;no_`T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0000000>0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0G0?ooo`030000
003oool0oooo01`0oooo00<000000?ooo`3oool09@3oool002/0oooo00<000000?ooo`3oool05`3o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00@0ooooK02o_kl90?ooo`030000003o
ool0oooo00`0oooo0P00000>0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool04P3o
ool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool09@3o
ool002/0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo00@0ooooJ`2o_kl90?ooo`030000003oool0oooo00d0oooo0P00000=0?ooo`030000003oool0
oooo00l0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0
oooo01`0oooo00<000000?ooo`3oool09@3oool002/0oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo00@0ooooJP2o_kl:0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`000000303oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo
02D0oooo000[0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02`3oool00`000000
oooo0?ooo`040?ooofT0_kno2P3oool00`000000oooo0?ooo`0=0?ooo`040000003oool0oooo0?oo
o`8000002P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0180oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo02D0oooo000Z0?ooo`03
0000003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooofP0
_kno2P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool05`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo02D0oooo000Z0?ooo`030000
003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooofL0_kno
2`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
1P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
5`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02@0oooo000Z0?ooo`030000003o
ool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooofH0_kno2P3o
ool200000100oooo00<000000?ooo`3oool01P3oool01@000000oooo0?ooo`3oool0000000<0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0T0?ooo`00:P3oool00`000000oooo0?oo
o`0G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0103ooomU0;no_`X0oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`080?ooo`030000003oool000000080000000<0
oooo0000003oool04@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool07@3oool00`000000oooo0?ooo`0T0?ooo`00:@3oool00`000000oooo0?ooo`0G
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0103ooomT0;no_`X0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`090?ooo`@0000000<0oooo0000003oool0403oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool07@3oool0
0`000000oooo0?ooo`0T0?ooo`00:@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool0103ooomS0;no_`/0oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`0:0?ooo`D000004P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo
01L0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0T0?ooo`00:@3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0103ooomR0;no_`/0
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`090?ooo`H000004P3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool07@3oool00`000000
oooo0?ooo`0T0?ooo`00:@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool0103ooomQ0;no_`/0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0;0?ooo`H000004@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0T0?ooo`00:@3oool00`000000oooo0?oo
o`0F0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`0?0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool01@3ooomP0;no_`/0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0=0?ooo`D00000403oool00`000000oooo0?oo
o`0C0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?oo
o`0T0?ooo`00:@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool01@3ooomO0;no_``0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0?
0?ooo`@000003`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01L0oooo00<00000
0?ooo`3oool07@3oool00`000000oooo0?ooo`0T0?ooo`00:@3oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0?0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool01@3ooomN0;no_``0oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo008000003P3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool07@3oool00`00
0000oooo0?ooo`0T0?ooo`00:@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool01@3ooomM0;no_``0oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0903oool002P0oooo
00<000000?ooo`3oool05`3oool500000140oooo00@000000?ooo`3oool0oooo1000000<0?ooo`03
0000003oool0oooo00L0oooo0`0000040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3o
ool01@3ooomL0;no_``0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo0140oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01d0oooo00<000000?ooo`3oool0903oool000l0oooo1@00000D0?ooo`H00000
403oool900000100oooo2@00000<0?ooo`030000003oool0oooo00<0oooo1`0000040?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool01@3ooomK0;no_``0oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01D0oooo0P0000060?ooo`030000
003oool0oooo0100oooo0P00000B0?ooo`002P3oool:00000100oooo2P00000A0?ooo`/000003P3o
ool=000000L0oooo00<000000?ooo`3oool02`0000030?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool01@3ooomJ0;no_``0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`0F0?ooo`<000005`3oool5000000<0oooo00<000000?ooo`3oool0403oool400000100oooo000<
0?ooo`X00000403oool<000000l0oooo1P0000030?ooo`@000002P3oool00`000000oooo0?ooo`05
000000D0oooo1@0000020?ooo`030000003oool0oooo0080oooo200000000`3oool000000000000:
000000L0oooo00<000000?ooo`3oool01@3ooomI0;no_`H0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool01P3oool2
000000X0oooo00<000000?ooo`3oool03@3oool3000000<0oooo0`00000B0?ooo`/000004@3oool;
00000140oooo2000000=0?ooo`003P3oool5000001D0oooo1@00000D0?ooo`030000003oool00000
00<000004@3oool00`000000oooo0?ooo`030?ooo`8000002`3oool00`000000oooo0?ooo`060?oo
o`D00000103oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00D0ooooF02o_kl80?oo
o`050000003oool0oooo0?ooo`0000005@3oool010000000oooo00000000000@0?ooo`030000003o
ool0oooo00P0oooo0`0000040?ooo`030000003oool0000001<0oooo2000000D0?ooo`X00000403o
ool:00000140oooo2@00000;0?ooo`004@3oool2000001D0oooo00<000000?ooo`3oool00P00000D
0?ooo`040000003oool0oooo0?ooo`8000004@3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo00T0oooo0P0000040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool01@3o
oomG0;no_`X0oooo00@000000?ooo`3oool000005@3oool01@000000oooo0?ooo`3oool0000000h0
oooo00<000000?ooo`3oool0303oool3000000040?ooo`0000000000000001D0oooo2000000B0?oo
o`L000004`3oool7000000030?ooo`000000oooo0140oooo1`00000=0?ooo`00:03oool00`000000
oooo0?ooo`0F0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`0?0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool01@3ooomF0;no_``0
oooo0P00000E0?ooo`030000003oool0oooo00<0oooo0P00000<0?ooo`030000003oool0oooo00l0
oooo1@00000D0?ooo`X00000403oool3000001L0oooo0`0000050?ooo`030000003oool0oooo0100
oooo0`00000A0?ooo`00:03oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool01@3ooomE0;no_`d0oooo00<000000?ooo`3oool04`3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0403oool7000001<0oooo
00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02@0oooo
000X0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`050?oooe@0_kno3@3oool00`000000oooo0000000C0?ooo`030000003oool0oooo00P0oooo0P00
00020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0403oool900000100oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02D0oooo000X
0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`05
0?oooe<0_kno3@3oool01@000000oooo0?ooo`3oool000000180oooo00<000000?ooo`3oool02P3o
ool4000000<0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`05000000l0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02D0oooo000X
0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`05
0?oooe80_kno3@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool0303oool400000080oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0U
0?ooo`00:03oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01<0oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool01@3ooomA0;no_`d0oooo00<000000?ooo`3oool01@3oool010000000oooo0?ooo`00000;
0?ooo`030000003oool0oooo00`0oooo1`00000E0?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool0603oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02D0oooo000W0?oo
o`030000003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`050?oo
oe00_kno3@3oool00`000000oooo0?ooo`070?ooo`<000002P3oool00`000000oooo0?ooo`0?0?oo
o`D000005@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`0U0?ooo`009`3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0?0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool01@3ooom?0;no_``0oooo0P00000:0?ooo`@0
0000203oool00`000000oooo0?ooo`0B0?ooo`@000004`3oool00`000000oooo0?ooo`0E0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0U0?ooo`00
9`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
1@3ooom>0;no_``0oooo00<000000?ooo`3oool02@3oool5000000L0oooo00<000000?ooo`3oool0
503oool010000000oooo00000000000B0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02D0oooo000W0?ooo`030000
003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`050?ooodd0_kno
3@3oool00`000000oooo0?ooo`0:0?ooo`D000001P3oool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo01<0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool09@3oool002L0oooo00<000000?ooo`3oool05`3o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00D0ooooC02o_kl=0?ooo`030000003o
ool0oooo00d0oooo0`0000050?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04`3o
ool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool07P3o
ool00`000000oooo0?ooo`0U0?ooo`009`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool01@3ooom;0;no_`d0oooo00<000000?ooo`3oool03`3oool3
000000@0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo01D0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0
oooo02D0oooo000W0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0?ooo`050?ooodX0_kno3@3oool00`000000oooo0?ooo`0A0?ooo`8000000`3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool05P3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool09@3oool002L0
oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00D0
ooooB@2o_kl=0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`0000005P3oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool06@3oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo02H0oooo000W0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`050?ooodP0_kno3@3oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`0F0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool07P3oool00`000000oooo
0?ooo`0V0?ooo`009`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool01@3ooom70;no_`d0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool09P3oool002L0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0140oooo00<00000
0?ooo`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00D0ooooAP2o_kl=
0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0E
0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0N
0?ooo`030000003oool0oooo02H0oooo000W0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`050?ooodD0_kno3@3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0E0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0V0?oo
o`009`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3o
ool0403oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool01@3ooom40;no_`d0oooo00<000000?ooo`3oool05`3oool2000001H0oooo00<000000?ooo`3o
ool05@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool07P3oool00`000000oooo0?ooo`0V0?ooo`009`3oool00`000000oooo0?ooo`0G0?ooo`030000
003oool0oooo01<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0?0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool01@3ooom30;no_`d0oooo00<000000?ooo`3oool0
5`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0
9P3oool002L0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0>0?ooo`030000003o
ool0oooo00D0oooo@P2o_kl=0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool05P3o
ool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo02H0oooo000W0?ooo`030000003oool0
oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo00l0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`050?oood40_kno3@3oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool07P3oool0
0`000000oooo0?ooo`0W0?ooo`009`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
01<0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool01@3ooom00;no_`d0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0F0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool09`3oool00140
oooo0P00000D0?ooo`040000003oool0oooo0?ooo`800000503oool010000000oooo0?ooo`3oool2
00000100oooo00<000000?ooo`3oool0103oool2000000/0oooo00<000000?ooo`3oool02@3oool2
000000<0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`050?ooocl0_kno3@3oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo01<0oooo0P00000B0?ooo`003`3oool40000
01@0oooo00<000000?ooo`0000000`00000D0?ooo`H00000403oool00`000000oooo0?ooo`020?oo
o`@000002`3oool00`000000oooo0?ooo`070?ooo`@000000`3oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo00D0oooo?P2o_kl=0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool06`3oool00`000000oooo0000000I0?ooo`<000000P3oool00`000000oooo0?ooo`0C0?oo
o`@00000403oool000`0oooo2P00000@0?ooo``000003@3oool?000000T0oooo00<000000?ooo`00
00001P0000000`3oool0000000000007000000030?ooo`000000oooo00D0oooo1`0000020?ooo`/0
00001`3oool00`000000oooo0?ooo`050?ooocd0_kno1P3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0B0?ooo`05
0000003oool0oooo0?ooo`0000005`3oool2000001T0oooo0`0000030?ooo`030000003oool0oooo
01<0oooo0P0000020?ooo`<000005@3oool7000001D0oooo2000000=0?ooo`002P3oool900000140
oooo2P00000?0?ooo`X00000403oool:000000X0oooo00<000000?ooo`3oool00P3oool;00000003
0?ooo`000000oooo00l0oooo00<000000?ooo`3oool01@3oooll0;no_`P0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool0503oool3000001P0oooo0`0000040?ooo`030000003oool0oooo0100oooo00@000000?ooo`3o
ool0oooo1P00000D0?ooo`P00000503oool8000001<0oooo2P00000;0?ooo`00303oool7000001@0
oooo1`00000B0?ooo`L00000403oool00`000000oooo00000006000000/0oooo00<000000?ooo`3o
ool0103oool7000000<0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`050?oooc/0
_kno2P3oool010000000oooo0?ooo`00000H0?ooo`040000003oool0oooo000001T0oooo00<00000
0?ooo`3oool05P3oool00`000000oooo0?ooo`02000000040?ooo`000000000000000140oooo00<0
00000?ooo`3oool00`3oool5000001<0oooo00<000000?ooo`0000001`00000B0?ooo`X000004@3o
ool500000100oooo000@0?ooo`<00000503oool010000000oooo0?ooo`3oool3000001<0oooo00<0
00000?ooo`3oool00`00000@0?ooo`030000003oool0oooo00<0oooo0`00000;0?ooo`030000003o
ool0oooo00P0oooo0`0000030?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool01@3o
oolj0;no_``0oooo0P00000J0?ooo`8000006@3oool00`000000oooo000000030?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool00`3oool500000100oooo00<000000?ooo`3oool00P3o
ool700000180oooo00<000000?ooo`0000002@00000@0?ooo`H00000:P3oool002L0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0140oooo00<00000
0?ooo`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00D0oooo>@2o_kl=
0?ooo`8000006P3oool2000000<0oooo00<000000?ooo`3oool04P3oool010000000oooo0?ooo`3o
ool500000140oooo00<000000?ooo`3oool00`3oool6000000h0oooo00<000000?ooo`3oool00`3o
ool9000000l0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0X0?ooo`00:03oool0
0`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool01@3ooolh
0;no_`d0oooo00@000000?ooo`3oool00000603oool01@000000oooo0?ooo`000000oooo00<00000
4P3oool00`000000oooo0?ooo`030?ooo`D000003`3oool00`000000oooo0?ooo`040?ooo`L00000
3@3oool00`000000oooo0?ooo`080?ooo`D000003P3oool00`000000oooo0?ooo`0O0?ooo`030000
003oool0oooo02P0oooo000X0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04`3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`050?ooocL0_kno3@3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool00P3oool400000180oooo00<000000?ooo`3oool00`3o
ool6000000h0oooo00<000000?ooo`3oool01P3oool6000000`0oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo02P0oooo000X0?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo00l0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`050?ooocH0_kno3@3oool0
0`000000oooo0?ooo`040?ooo`040000003oool0oooo00000140oooo00<000000?ooo`3oool00`3o
ool500000100oooo00<000000?ooo`3oool00`3oool7000000d0oooo00<000000?ooo`3oool02`3o
ool3000000X0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0O0?ooo`030000003o
ool0oooo02P0oooo000X0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04`3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`050?ooocD0_kno3@3oool00`000000oooo0?ooo`060?ooo`@000003`3oool0
0`000000oooo0?ooo`030?ooo`H000003`3oool00`000000oooo0?ooo`060?ooo`H00000303oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07`3oool0
0`000000oooo0?ooo`0Y0?ooo`00:03oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool01@3ooold0;no_`d0oooo00<000000?ooo`3oool01`3oool40000
00l0oooo00<000000?ooo`3oool0103oool6000000d0oooo00<000000?ooo`3oool02P3oool40000
00X0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo
01l0oooo00<000000?ooo`3oool0:@3oool002P0oooo00<000000?ooo`3oool05P3oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`0=0?ooo`030000003oool0oooo00D0oooo<`2o_kl=0?ooo`030000003oool0oooo00L0
oooo1P00000=0?ooo`030000003oool0oooo00L0oooo1@00000<0?ooo`030000003oool0oooo00d0
oooo0P0000090?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0703oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo02T0oooo000X0?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0oooo
00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`050?oooc80_kno3@3oool00`000000oooo
0?ooo`090?ooo`D00000303oool00`000000oooo0?ooo`0:0?ooo`<000002`3oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06`3oool00`000000oooo
0?ooo`0O0?ooo`030000003oool0oooo02X0oooo000X0?ooo`030000003oool0oooo01H0oooo00<0
00000?ooo`3oool0503oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0oooo00<0
00000?ooo`3oool03@3oool00`000000oooo0?ooo`050?oooc40_kno3@3oool00`000000oooo0?oo
o`0<0?ooo`@000002P3oool00`000000oooo0?ooo`0=0?ooo`8000002@3oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0703oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo02X0oooo000X0?ooo`030000003oool0oooo01L0oooo00<00000
0?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`050?oooc00_kno3@3oool00`000000oooo0?ooo`0>
0?ooo`@00000203oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0200oooo00<00000
0?ooo`3oool0:P3oool002P0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo00D0oooo;`2o_kl=0?ooo`030000003oool0oooo0140oooo0P0000070?oo
o`030000003oool0oooo01P0oooo0P00000J0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool06`3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo02X0oooo000X0?ooo`03
0000003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`050?ooobh0
_kno3@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool0703oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo02X0oooo000X0?ooo`030000
003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`050?ooobd0_kno
3@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0
7`3oool00`000000oooo0?ooo`0[0?ooo`00:@3oool00`000000oooo0?ooo`0F0?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool01@3oool/0;no_`d0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0:`3o
ool002T0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo00D0oooo:`2o_kl=0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0703oool0
0`000000oooo0?ooo`0P0?ooo`030000003oool0oooo02/0oooo000Y0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
00l0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`050?ooobX0_kno3@3oool00`00
0000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0oooo0P00000K0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo
02/0oooo000Y0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`050?ooobT0_kno3@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01`0
oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0/0?ooo`00:@3oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool01@3ooolX0;no_`d0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0200oooo
00<000000?ooo`3oool0;03oool002T0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?oo
o`0C0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00D0oooo9`2o_kl=0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool07@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo02`0oooo000Y
0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`05
0?ooobH0_kno303oool2000001`0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo02`0oooo000Y0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`050?ooobD0_kno303oool00`000000oooo0?ooo`0J0?oo
o`8000007@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo02d0oooo000Z0?ooo`03
0000003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`040?ooob@0
_kno303oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`0]0?ooo`004P3oool00`000000oooo0?ooo`0E0?ooo`030000
003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`040?ooob<0_kno
3@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
6P3oool2000001d0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0Q0?ooo`800000
6@3oool00`000000oooo0?ooo`0B0?ooo`00403oool3000001L0oooo00<000000?ooo`0000005`3o
ool00`000000oooo0000000C0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00@0oooo8P2o_kl=0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo01`0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo01L0oooo1000000G0?ooo`<000004P3oool000h0oooo2000000D0?ooo`<000000P3oool3
00000180oooo0`0000040?ooo`<00000303oool01@000000oooo0?ooo`3oool0000000P0oooo0`00
00040?ooo`030000003oool0oooo00@0oooo0P0000070?ooo`040000003oool0oooo0?ooo`@00000
203oool00`000000oooo0?ooo`040?ooob40_kno1P3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo0180oooo00<000000?ooo`3oool0103oool2000001<0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00X0oooo00@000000?ooo`3oool0
00003`3oool00`000000oooo0?ooo`050?ooo`H000005P3oool6000001D0oooo1P00000@0?ooo`00
303oool8000001@0oooo1`00000C0?ooo`T00000403oool4000000<0oooo1@0000070?ooo`030000
003oool0oooo0080oooo100000030?ooo`P00000303oool00`000000oooo0?ooo`040?ooob00_kno
203oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00l0oooo0P0000020?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool0303oool5000000`0oooo00<000000?ooo`3oool0
203oool5000001D0oooo1`00000D0?ooo`P000003P3oool000X0oooo2P00000B0?ooo`P000004P3o
ool800000180oooo2000000<0?ooo`030000003oool0000000T000001@3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00@0oooo7`2o_kl:0?ooo`040000003oool0oooo000001P0oooo
00@000000?ooo`3oool00000603oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01<0
oooo0P0000020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04@3oool4000000L0
oooo00<000000?ooo`3oool03@3oool5000000/0oooo00<000000?ooo`3oool01`3oool7000001@0
oooo2@00000B0?ooo`X00000303oool000l0oooo1@00000@0?ooo`X00000403oool:00000100oooo
2000000>0?ooo`X000001`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00@0oooo
7P2o_kl<0?ooo`8000006P3oool2000001X0oooo00D000000?ooo`3oool0oooo0000000H0?ooo`@0
00000P3oool00`000000oooo0?ooo`0B0?ooo`D000001@3oool00`000000oooo0?ooo`0>0?ooo`H0
00002P3oool00`000000oooo0?ooo`070?ooo`T000004@3oool00`000000oooo0?ooo`030?ooo`D0
00009`3oool002/0oooo00<000000?ooo`3oool05@3oool5000000h0oooo2P00000>0?ooo`030000
003oool0oooo0080oooo1@0000080?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0
103ooolM0;no_`d0oooo0`00000I0?ooo`8000006`3oool010000000oooo00000000000H0?ooo`@0
000000<0oooo0000003oool04`3oool7000000<0oooo00<000000?ooo`3oool04@3oool6000000L0
oooo00<000000?ooo`3oool03@3oool500000100oooo00<000000?ooo`3oool0;`3oool002`0oooo
00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00@0oooo
702o_kl<0?ooo`8000000`3oool00`000000oooo0?ooo`0E0?ooo`040000003oool0oooo00000080
oooo00<000000?ooo`3oool05P3oool4000001L0oooo1@00000G0?ooo`H000000P3oool00`000000
oooo0?ooo`0E0?ooo`<000001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0300
oooo000/0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`040?oooa/0_kno303oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool00P3oool3000001L0oooo1@00000F0?ooo`L000006@3oool500000240oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0`0?ooo`00;03oool00`000000oooo0?oo
o`0G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0103ooolJ0;no_``0oooo00<0
00000?ooo`3oool01P3oool010000000oooo0?ooo`00000?0?ooo`800000103oool5000001D0oooo
1`00000G0?ooo`H000006`3oool200000200oooo00<000000?ooo`3oool08P3oool00`000000oooo
0?ooo`0`0?ooo`00;03oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool0103ooolI0;no_``0oooo00<000000?ooo`3oool0203oool4000000d0oooo00<0
00000?ooo`3oool00`3oool6000001@0oooo00<000000?ooo`3oool01@00000G0?ooo`030000003o
ool0000000<000006`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0240oooo00<0
00000?ooo`3oool0<@3oool002d0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0C
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo00@0oooo602o_kl<0?ooo`030000003oool0oooo00T0oooo1000000<
0?ooo`030000003oool0oooo00D0oooo1P00000B0?ooo`030000003oool0oooo00<0oooo1000000E
0?ooo`030000003oool0oooo00<0oooo0P00000I0?ooo`030000003oool0oooo01l0oooo00<00000
0?ooo`3oool08P3oool00`000000oooo0?ooo`0a0?ooo`00;@3oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo01<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool0103ooolG0;no_``0oooo00<000000?oo
o`3oool02@3oool6000000X0oooo00<000000?ooo`3oool0203oool400000140oooo00<000000?oo
o`3oool01P3oool2000001D0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0O0?oo
o`030000003oool0oooo0280oooo00<000000?ooo`3oool0<@3oool002d0oooo00<000000?ooo`3o
ool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00@0oooo5P2o_kl<0?ooo`03
0000003oool0oooo00/0oooo1P0000070?ooo`800000303oool4000000l0oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3o
ool07`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0340oooo000]0?ooo`030000
003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`040?oooaD0_kno
303oool00`000000oooo0?ooo`0>0?ooo`D000001@3oool00`000000oooo0?ooo`0>0?ooo`800000
3P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0
803oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0340oooo000^0?ooo`030000003o
ool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`030?oooa@0_kno2`3o
ool2000001<0oooo0`0000050?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0380oooo000^0?ooo`030000003oool0
oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`030?oooa<0_kno2`3oool0
0`000000oooo0?ooo`0E0?ooo`8000000`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0
oooo01h0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0
oooo02<0oooo00<000000?ooo`3oool0<P3oool002h0oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo00<0oooo4P2o_kl;0?ooo`030000003oool0oooo
01T0oooo0P00000N0?ooo`800000803oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo
0200oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0c0?ooo`00;`3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0=0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool00`3ooolA0;no_`X0
oooo0P00000J0?ooo`8000007P3oool200000200oooo0P00000P0?ooo`030000003oool0oooo0200
oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0c0?ooo`00;`3oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool00`3oool@0;no_`X0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo02<0oooo
00<000000?ooo`3oool0=03oool002l0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`090?ooo`030000003oool0oooo00<0oooo3`2o_kl;0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool07@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo00<0
00000?ooo`3oool08@3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo03@0oooo000_
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`03
0?ooo`h0_kno2`3oool00`000000oooo0?ooo`0G0?ooo`800000803oool00`000000oooo0?ooo`0N
0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0S
0?ooo`030000003oool0oooo03D0oooo000`0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool0203oool00`000000oooo0?ooo`030?ooo`d0_kno2P3oool2000001P0oooo0P00000P0?oo
o`800000803oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0240oooo00<000000?oo
o`3oool0903oool00`000000oooo0?ooo`0e0?ooo`00<03oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool00P3oool<0;no_`T0oooo0P00000I0?ooo`03
0000003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0P0?ooo`80
00008`3oool00`000000oooo0?ooo`0S0?ooo`800000=`3oool00300oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0080oooo2`2o_kl90?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0O0?ooo`030000
003oool0oooo0200oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0T0?ooo`030000
003oool0oooo03H0oooo000a0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`020?ooo`X0_kno2@3oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo01l0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0P0?ooo`030000003o
ool0oooo0280oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0g0?ooo`00<@3oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool00P3oool9
0;no_`T0oooo00<000000?ooo`3oool05`3oool200000200oooo0P00000Q0?ooo`030000003oool0
oooo0240oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0
oooo03L0oooo000a0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0203oool00`00
0000oooo0?ooo`020?ooo`P0_kno203oool2000001T0oooo00<000000?ooo`3oool07P3oool20000
0280oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo
02D0oooo00<000000?ooo`3oool0=`3oool00340oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool02`3oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo0080oooo1`2o_kl80?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0280
oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo00P0
oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0B0?ooo`004`3oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo0080oooo1P2o_kl80?ooo`030000003oool0oooo01P0oooo
00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0280oooo
00<000000?ooo`3oool0803oool00`000000oooo0000000G0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool01`3oool4000001P0oooo0`00000B0?ooo`004@3oool5000001D0oooo0`00
00020?ooo`<000006@3oool2000001H0oooo00@000000?ooo`3oool0oooo0P00000>0?ooo`030000
003oool0oooo00T0oooo0`0000090?ooo`030000003oool0oooo0080oooo1@2o_kl70?ooo`800000
6@3oool2000001X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo0080oooo00D000000?ooo`3oool0oooo0000000G0?ooo`030000003oool000000080
00005P3oool5000000`0oooo00<000000?ooo`3oool0203oool5000001H0oooo1P00000@0?ooo`00
3`3oool5000001H0oooo1P0000030?ooo`030000003oool0oooo0100oooo00@000000?ooo`3oool0
oooo0P0000020?ooo`030000003oool0oooo01@0oooo0`00000@0?ooo`030000003oool0oooo00L0
oooo0P0000030?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool00P3oool40;no_`L0
oooo00<000000?ooo`0000005P3oool3000000030?ooo`000000oooo01X0oooo00<000000?ooo`3o
ool00P00000G0?ooo`030000003oool0oooo00T0oooo0P00000>0?ooo`030000003oool0oooo0180
oooo00<000000?ooo`3oool0103oool00`000000oooo00000002000001L0oooo1@00000F0?ooo`D0
00002`3oool00`000000oooo0?ooo`070?ooo`L000005@3oool8000000h0oooo000=0?ooo`P00000
4`3oool7000000@0oooo00<000000?ooo`3oool03P3oool6000000@0oooo00<000000?ooo`3oool0
4`3oool2000001<0oooo00<000000?ooo`3oool0103oool2000000D0oooo00<000000?ooo`3oool0
1P3oool00`000000oooo0?ooo`020?ooo`<0_kno1`3oool01@000000oooo0?ooo`3oool0000001@0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0000001X0
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0?0?ooo`040000003oool0oooo0000
00l0oooo00<000000?ooo`3oool01P3oool4000001L0oooo1P00000D0?ooo`L000002P3oool00`00
0000oooo0?ooo`070?ooo`T000004`3oool:000000`0oooo000;0?ooo`X000004@3oool9000000@0
oooo00<000000?ooo`3oool03@3oool5000000L0oooo00<000000?ooo`3oool03@3oool00`000000
oooo0?ooo`0200000080oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`020?ooo`80
0000203oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0080oooo0P2o_kl70?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool01@000000oooo0?ooo`3oool00000
00@0oooo00<000000?ooo`3oool04@3oool4000000d0oooo00<000000?ooo`3oool01`3oool50000
01D0oooo1`00000G0?ooo`H000001`3oool00`000000oooo0?ooo`0=0?ooo`D00000:03oool002D0
oooo1@00000:0?ooo`030000003oool0oooo00/0oooo1`0000060?ooo`030000003oool0oooo00/0
oooo1@0000040?ooo`030000003oool0oooo0140oooo00@000000?ooo`00000000002P3oool00`00
0000oooo0?ooo`060?ooo`050000003oool0oooo0?ooo`2o_kl01P3oool2000000L0oooo00<00000
0?ooo`3oool03@3oool2000000P0oooo00<000000?ooo`3oool05P3oool2000000030?ooo`000000
oooo01X0oooo00@000000?ooo`00000000000`3oool00`000000oooo0?ooo`0B0?ooo`@00000303o
ool00`000000oooo0?ooo`070?ooo`L000005P3oool6000001X0oooo0`0000060?ooo`030000003o
ool0oooo03X0oooo000d0?ooo`030000003oool0oooo00T0oooo1P00000:0?ooo`030000003oool0
oooo00T0oooo1@0000060?ooo`030000003oool0oooo0100oooo0P00000=0?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo00/0oooo0P00000:0?ooo`030000003oool0oooo01@0oooo0P0000040?ooo`040000003oool0
oooo000001L0oooo1`00000D0?ooo`H000002P3oool00`000000oooo0?ooo`0:0?ooo`H00000503o
ool010000000oooo0?ooo`3oool400000240oooo00<000000?ooo`3oool0>`3oool003@0oooo00<0
00000?ooo`3oool0203oool3000000h0oooo00<000000?ooo`3oool0203oool6000000L0oooo00<0
00000?ooo`3oool0303oool3000000030?ooo`000000oooo00d0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0203o
ool2000000d0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`040?ooo`040000003o
ool00000000001H0oooo1P00000G0?ooo`D000002@3oool00`000000oooo0?ooo`0=0?ooo`@00000
4`3oool00`000000oooo0?ooo`050?ooo`8000007`3oool00`000000oooo0?ooo`0l0?ooo`00=@3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00H0oooo2@0000070?ooo`030000003o
ool0oooo00D0oooo00@000000?ooo`3oool0oooo0P0000050?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool01P3oool2000000<0oooo0P00000=0?ooo`030000003oool0oooo00D0oooo
0P00000@0?ooo`030000003oool0oooo00l0oooo0P0000070?ooo`D000005P3oool5000001T0oooo
100000080?ooo`030000003oool0oooo0100oooo0P00000A0?ooo`030000003oool0oooo02L0oooo
00<000000?ooo`3oool0?03oool001D0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo00@0oooo1@00000<0?ooo`030000003oool0oooo00@0oooo0P00
00000`3oool00000000000070?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool01`3o
ool300000100oooo00<000000?ooo`3oool00P3oool200000180oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`060?ooo`H00000603oool4000001X0oooo0P0000070?ooo`030000003o
ool0oooo02@0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0m0?ooo`005P3oool0
0`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool05@3oool0
0`000000oooo0?ooo`020?ooo`@000002P3oool00`000000oooo0?ooo`0<0?ooo`8000006`3oool0
10000000oooo00000000000E0?ooo`030000003oool0000000/0oooo0P00000:0?ooo`D000005`3o
ool00`000000oooo00000002000001`0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`0T0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0?@3oool001H0oooo00<00000
0?ooo`3oool07@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01H0oooo00<00000
0?ooo`0000001@00000;0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0603oool2
000001L0oooo0`0000090?ooo`8000003@3oool5000001D0oooo00@000000?ooo`3oool0oooo0`00
000P0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?oo
o`0m0?ooo`005P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool05@3oool9000000/0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0D
0?ooo`<000000P3oool00`000000oooo0?ooo`0C0?ooo`D000001P3oool200000100oooo1000000D
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0U
0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0?P3oool001L0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01D0oooo1@00000@0?oo
o`030000003oool0oooo00d0oooo0P00000@0?ooo`@000001P3oool00`000000oooo0?ooo`0C0?oo
o`@00000103oool2000001@0oooo0P00000B0?ooo`8000009P3oool00`000000oooo0?ooo`0U0?oo
o`030000003oool0oooo02P0oooo00<000000?ooo`3oool0?P3oool001L0oooo00<000000?ooo`3o
ool07P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3o
ool04P3oool00`000000oooo0?ooo`0>0?ooo`<000002@3oool4000000X0oooo00<000000?ooo`3o
ool0503oool300000080oooo0P00000G0?ooo`8000003`3oool2000002L0oooo00<000000?ooo`3o
ool09@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo03l0oooo000G0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0F0?ooo`030000
003oool0oooo0180oooo00<000000?ooo`3oool0403oool9000000l0oooo00<000000?ooo`3oool0
503oool4000001X0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0U0?ooo`030000
003oool0oooo02D0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`100?ooo`00603o
ool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool05@3o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool04`3o
ool3000002T0oooo0P00000W0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0:03o
ool00`000000oooo0?ooo`100?ooo`00603oool00`000000oooo0?ooo`0N0?ooo`030000003oool0
oooo01T0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0
oooo02P0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`02000002H0oooo0P00000X
0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`11
0?ooo`006@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo02P0oooo00@000000?oo
o`3oool000003@3oool2000000@0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0W
0?ooo`030000003oool0oooo02H0oooo0P00000Z0?ooo`030000003oool0oooo0440oooo000I0?oo
o`030000003oool0oooo01h0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo01<0oooo00<000000?ooo`3oool0:03oool3000000/0oooo0P00000Z0?oo
o`800000:03oool2000002P0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`120?oo
o`006@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo02H0oooo1@0000080?ooo`80
0000:`3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3o
ool0:@3oool00`000000oooo0?ooo`130?ooo`006P3oool00`000000oooo0?ooo`0N0?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`800000
9@3oool6000000D0oooo0`00000[0?ooo`800000:@3oool00`000000oooo0?ooo`0X0?ooo`030000
003oool0oooo02T0oooo00<000000?ooo`3oool0@`3oool001X0oooo00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503o
ool2000002@0oooo2P00000/0?ooo`800000:P3oool00`000000oooo0?ooo`0X0?ooo`030000003o
ool0oooo02T0oooo00<000000?ooo`3oool0A03oool001X0oooo00<000000?ooo`3oool07`3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool05@3oool2
000002@0oooo1000000_0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0:@3oool0
0`000000oooo0?ooo`0X0?ooo`030000003oool0oooo04D0oooo000K0?ooo`030000003oool0oooo
01l0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
01H0oooo0`00000P0?ooo`H00000;03oool2000002/0oooo00<000000?ooo`3oool0:@3oool00`00
0000oooo0?ooo`0Y0?ooo`030000003oool0oooo04D0oooo000K0?ooo`030000003oool0oooo01l0
oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0
oooo0`00000F0?ooo`L00000103oool2000002/0oooo00<000000?ooo`3oool0:P3oool00`000000
oooo0?ooo`0Y0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0AP3oool001`0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01P0oooo
00<000000?ooo`3oool06P3oool5000000d0oooo1000000=0?ooo`030000003oool0oooo02H0oooo
0P00000[0?ooo`800000:`3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo0340oooo
00<000000?ooo`3oool04P3oool001<0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0N0?ooo`d00000>03oool2000002`0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?oo
o`0Y0?ooo`8000006@3oool00`000000oooo0?ooo`0G0?ooo`<000004`3oool00180oooo0P000009
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0H03oool2000002`0oooo0P00000/
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool05`3oool3000001P0oooo0`00000H
0?ooo`@000004P3oool00140oooo1@0000070?ooo`030000003oool0oooo00d0oooo0P0000020?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo00P0oooo00<000000?ooo`3oool03P3oool2000000T0oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00@000000?oo
o`0000000000603oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool06@3oool010000000oooo0?ooo`0000070?ooo`030000003oool0oooo00h0oooo1000
000G0?ooo`D000005`3oool4000001H0oooo1`00000@0?ooo`003`3oool6000000T0oooo00<00000
0?ooo`3oool02`3oool5000000l0oooo0P0000060?ooo`030000003oool0oooo008000004P3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool2
000001/0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`050?ooo`040000003oool0
oooo000001P0oooo100000050?ooo`030000003oool0oooo00l0oooo1000000G0?ooo`H000005@3o
ool6000001D0oooo2000000?0?ooo`003P3oool8000000P0oooo00<000000?ooo`3oool02@3oool6
00000140oooo00<000000?ooo`3oool00`3oool4000001D0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01H0oooo0P0000000`3oool000000?ooo`0J
0?ooo`030000003oool0oooo00l0oooo0P0000090?ooo`<00000603oool4000000@0oooo00<00000
0?ooo`3oool03`3oool6000001D0oooo1`00000G0?ooo`D000005`3oool6000000h0oooo000=0?oo
o`H00000303oool00`000000oooo0?ooo`070?ooo`P000004@3oool010000000oooo0?ooo`3oool4
000001L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0E0?ooo`8000000`3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0503oool2
000000@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0000000<0?ooo`8000002P3oool5
000001H0oooo1P0000020?ooo`030000003oool0oooo0140oooo1P00000E0?ooo`L00000603oool4
000001X0oooo0`00000=0?ooo`00303oool300000100oooo00<000000?ooo`3oool01P3oool60000
01@0oooo00<000000?ooo`3oool01P00000F0?ooo`8000000P3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0000001h0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0B0?oo
o`8000001P3oool00`000000oooo0?ooo`0I0?ooo`<000002`3oool00`000000oooo0?ooo`090?oo
o`H000005`3oool5000000030?ooo`000000oooo01D0oooo1@00000C0?ooo`030000003oool0oooo
0080oooo1000000J0?ooo`800000:@3oool00200oooo00<000000?ooo`3oool0103oool3000001T0
oooo00<000000?ooo`0000001000000F0?ooo`D00000703oool2000001h0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0=0?ooo`@000002@3oool00`000000oooo0000000G0?ooo`D0
00002@3oool00`000000oooo0?ooo`0;0?ooo`D000006@3oool4000001T0oooo0`00000C0?ooo`03
0000003oool0oooo00D0oooo0P0000140?ooo`00803oool00`000000oooo0?ooo`0Q0?ooo`@00000
603oool5000001`0oooo00@000000?ooo`0000000000703oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo00/0oooo0P00000=0?ooo`<00000603oool4000000P0oooo00<000000?ooo`3o
ool03P3oool4000001X0oooo0P00000K0?ooo`8000004@3oool00`000000oooo0?ooo`1<0?ooo`00
8@3oool00`000000oooo0?ooo`0O0?ooo`<000006@3oool7000001/0oooo00@000000?ooo`3oool0
oooo0P00000J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0203oool3000000d0
oooo1@00000I0?ooo`<000001`3oool00`000000oooo0?ooo`0@0?ooo`<000006@3oool010000000
oooo0?ooo`00000/0?ooo`030000003oool0oooo04d0oooo000Q0?ooo`030000003oool0oooo01h0
oooo0P0000000`3oool000000?ooo`0H0?ooo`D000000P3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0K0?ooo`<0
0000103oool300000140oooo1@00000I0?ooo`<00000103oool2000001@0oooo0`00000G0?ooo`03
0000003oool0oooo02d0oooo00<000000?ooo`3oool0CP3oool00280oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`0E0?ooo`@000001@3oool00`000000oooo0?ooo`0E0?ooo`030000
003oool0oooo00H0oooo0P00000G0?ooo`030000003oool0oooo01T0oooo1@000000103oool00000
00000000000E0?ooo`@000006P3oool2000000<0oooo00<000000?ooo`3oool05P3oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0CP3oool00280oooo
00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0D0?ooo`800000203oool00`000000oooo
0?ooo`0A0?ooo`040000003oool0oooo000000X0oooo0`00000D0?ooo`030000003oool0oooo01X0
oooo1@00000I0?ooo`<000006`3oool4000002l0oooo00<000000?ooo`3oool0;P3oool00`000000
oooo0?ooo`1?0?ooo`008`3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0180oooo
00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0@0?ooo`<000003P3oool400000100oooo
00<000000?ooo`3oool06P3oool4000001/0oooo0P00000K0?ooo`800000<03oool00`000000oooo
0?ooo`0^0?ooo`030000003oool0oooo0500oooo000S0?ooo`030000003oool0oooo02<0oooo00<0
00000?ooo`3oool07P3oool00`000000oooo0?ooo`0>0?ooo`D000004@3oool4000000`0oooo00<0
00000?ooo`3oool04`3oool7000000040?ooo`0000000000000001`0oooo0P00000H0?ooo`800000
<@3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0540oooo000T0?ooo`030000003o
ool0oooo0280oooo00<000000?ooo`3oool07`3oool2000000h0oooo1000000F0?ooo`@00000203o
ool00`000000oooo0?ooo`0>0?ooo`D00000203oool4000001`0oooo00<000000?ooo`3oool05@3o
ool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0D@3o
ool002D0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0
oooo00/0oooo0`00000K0?oooaT000003P3oool3000003<0oooo00<000000?ooo`3oool0<03oool0
0`000000oooo0?ooo`0_0?ooo`030000003oool0oooo0580oooo000U0?ooo`030000003oool0oooo
0280oooo00<000000?ooo`3oool08@3oool2000000X0oooo0`00000R0?ooo`D000007@3oool20000
0340oooo0P00000b0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0D`3oool002H0
oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo00L0
oooo0`00000R0?ooo`D000007@3oool2000002l0oooo0P00000c0?ooo`030000003oool0oooo0300
oooo00<000000?ooo`3oool0D`3oool002H0oooo00<000000?ooo`3oool08`3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo00L0oooo0P00000T0?ooo`<000007`3oool00`000000oooo
0?ooo`0[0?ooo`800000<`3oool200000340oooo0P00001F0?ooo`009`3oool00`000000oooo0?oo
o`0S0?ooo`030000003oool0oooo0240oooo0P0000060?ooo`8000009@3oool3000004/0oooo0P00
000d0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0EP3oool002L0oooo00<00000
0?ooo`3oool0903oool00`000000oooo0?ooo`0R0?ooo`800000103oool00`000000oooo0?ooo`0T
0?ooo`<00000B@3oool2000003@0oooo0P00000b0?ooo`030000003oool0oooo05L0oooo000X0?oo
o`030000003oool0oooo02@0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0X0?oo
o`<00000A`3oool2000003D0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`1G0?oo
o`00:@3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3o
ool0:03oool00`000000oooo0?ooo`140?ooo`800000=`3oool00`000000oooo0?ooo`0a0?ooo`03
0000003oool0oooo05P0oooo000Z0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0
8`3oool2000002P0oooo00<000000?ooo`3oool0@@3oool3000003L0oooo0P00000c0?ooo`030000
003oool0oooo05T0oooo000[0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0903o
ool2000002H0oooo00<000000?ooo`3oool0?@3oool4000003P0oooo0P00000d0?ooo`030000003o
ool0oooo05X0oooo000/0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool09@3oool3
00000640oooo0P00000k0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0F`3oool0
02`0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0W0?ooo`800000G@3oool20000
03/0oooo0P00000e0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0C0?ooo`00503oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo02D0
oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`1G0?ooo`<00000?03oool00`000000
oooo0?ooo`0]0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0G0?ooo`<00000603oool3000001@0oooo000C0?ooo`800000603oool2000002L0oooo
0P00000Y0?ooo`800000E@3oool2000003d0oooo0P00000E0?ooo`030000003oool0oooo01L0oooo
0`0000040?ooo`030000003oool0oooo0140oooo0`00000H0?ooo`@000005`3oool4000001<0oooo
000B0?ooo`@000005`3oool3000000030?ooo`000000oooo01H0oooo00D000000?ooo`3oool0oooo
0000000:0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool05`3oool4000001X0oooo
00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0E0?ooo`D000006`3oool00`000000oooo
0?ooo`0I0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool04`3oool3000001P0
oooo100000030?ooo`030000003oool0oooo0180oooo1000000F0?ooo`H000005@3oool700000140
oooo000@0?ooo`H000005`3oool4000001L0oooo0`0000000`3oool000000?ooo`0:0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool06P3oool010000000oooo00000000000H0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool04P3oool3000000@0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0J0?ooo`<0000000<0oooo0000003oool05@3oool4000001H0
oooo1P0000000`3oool000000?ooo`0C0?ooo`H000005@3oool6000001D0oooo2000000@0?ooo`00
3`3oool8000001D0oooo1P00000F0?ooo`@000003@3oool00`000000oooo0?ooo`080?ooo`050000
003oool0oooo0?ooo`0000006`3oool00`000000oooo0?ooo`020?ooo`<000005@3oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo00h0oooo100000080?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`000000603oool5000001H0oooo1@00000G0?ooo`H000005P3oool5000001L0
oooo1@00000G0?ooo`H000003`3oool000h0oooo1P00000G0?ooo`H000005P3oool6000000d0oooo
0P0000080?ooo`80000000<0oooo0000003oool06`3oool00`000000oooo0?ooo`050?ooo`800000
4`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00T0oooo1000000<0?ooo`040000
003oool0oooo000001P0oooo0`00000G0?ooo`H000005`3oool5000001P0oooo1000000H0?ooo`@0
0000603oool4000001X0oooo0`00000>0?ooo`003@3oool3000001X0oooo1@0000020?ooo`030000
003oool0oooo01<0oooo1@00000@0?ooo`030000003oool0oooo00@0oooo1@00000K0?ooo`030000
003oool0oooo00P0oooo1@00000=0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
1@3oool400000140oooo0`00000G0?ooo`@00000603oool5000001T0oooo0`00000I0?ooo`@00000
6@3oool3000001X0oooo0P00000Z0?ooo`00:P3oool3000000D0oooo00<000000?ooo`3oool04@3o
ool4000001<0oooo00<000000?ooo`3oool00`3oool6000001L0oooo00@000000?ooo`3oool00000
3`3oool6000000L0oooo00<000000?ooo`3oool06P3oool300000080oooo0`00000C0?ooo`D00000
603oool4000001D0oooo0P0000000`3oool0000000000003000001T0oooo0`00000G0?ooo`030000
003oool0oooo008000006`3oool2000004D0oooo000Y0?ooo`800000203oool00`000000oooo0?oo
o`0@0?ooo`<000005@3oool2000000<0oooo1@00000H0?ooo`<000005P3oool4000000<0oooo00<0
00000?ooo`3oool05`3oool8000001L0oooo1000000I0?ooo`<00000503oool00`000000oooo0?oo
o`020?ooo`@000006P3oool2000001H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`1O0?ooo`00=03oool00`000000oooo0?ooo`0>0?ooo`<00000603oool00`000000oooo00000004
000001T0oooo1@00000H0?oooad0000000<0oooo0000000000000`00000I0?ooo`@000006@3oool3
00000180oooo0P0000070?ooo`800000703oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo06H0oooo000e0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool06@3oool50000
01X0oooo1000000J0?ooo`D000006@3oool4000001X0oooo1000000I0?ooo`8000004@3oool00`00
0000oooo0?ooo`080?ooo`800000;`3oool00`000000oooo0?ooo`1W0?ooo`00=P3oool00`000000
oooo0?ooo`0Y0?ooo`800000703oool4000001X0oooo1000000J0?ooo`@000006`3oool3000001X0
oooo0P00000?0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0;03oool00`000000
oooo0?ooo`1X0?ooo`00=P3oool00`000000oooo0?ooo`0X0?ooo`80000000<0oooo0000003oool0
6@3oool4000001/0oooo1000000K0?ooo`<00000703oool2000001/0oooo00<000000?ooo`3oool0
2`3oool2000003/0oooo00<000000?ooo`3oool0J@3oool003L0oooo00<000000?ooo`3oool09`3o
ool010000000oooo0?ooo`3oool2000001P0oooo0`00000L0?ooo`@000006`3oool3000001`0oooo
0P00000W0?ooo`800000?03oool00`000000oooo0?ooo`1Z0?ooo`00>03oool00`000000oooo0?oo
o`0/0?ooo`030000003oool0oooo01D0oooo0P00000N0?ooo`8000007@3oool2000001d0oooo00<0
00000?ooo`3oool08`3oool2000003d0oooo00<000000?ooo`3oool0J`3oool003T0oooo00<00000
0?ooo`3oool0;03oool00`000000oooo0?ooo`0D0?ooo`8000007P3oool2000001d0oooo0P000012
0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0K03oool003X0oooo00<000000?oo
o`3oool0;03oool2000001@0oooo00<000000?ooo`3oool07@3oool2000001h0oooo00<000000?oo
o`3oool0?P3oool2000003h0oooo00<000000?ooo`3oool0K@3oool003/0oooo00<000000?ooo`3o
ool0;@3oool200000380oooo00<000000?ooo`3oool0F`3oool3000003h0oooo0P00001`0?ooo`00
?03oool00`000000oooo0?ooo`0^0?ooo`800000<03oool00`000000oooo0?ooo`1I0?ooo`800000
@03oool00`000000oooo0?ooo`1`0?ooo`00?@3oool00`000000oooo0?ooo`0_0?ooo`800000R03o
ool200000480oooo00<000000?ooo`3oool0L03oool003h0oooo00<000000?ooo`3oool0<03oool0
0`000000oooo0?ooo`220?ooo`<00000@`3oool00`000000oooo0?ooo`1a0?ooo`00?`3oool00`00
0000oooo0?ooo`0`0?ooo`800000P03oool2000004@0oooo0P00001d0?ooo`00@03oool00`000000
oooo0?ooo`0a0?ooo`<00000N`3oool2000004@0oooo0P00001f0?ooo`00@@3oool00`000000oooo
0?ooo`0c0?ooo`800000M`3oool2000004D0oooo00<000000?ooo`3oool0MP3oool00480oooo00<0
00000?ooo`3oool0=03oool200000780oooo0`0000160?ooo`030000003oool0oooo07L0oooo0013
0?ooo`030000003oool0oooo03D0oooo0P00001/0?ooo`@00000B03oool00`000000oooo0?ooo`1h
0?ooo`00A03oool00`000000oooo0?ooo`0f0?ooo`800000I`3oool3000004X0oooo0P00001U0?oo
o`030000003oool0oooo01<0oooo000D0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3o
ool0=`3oool300000680oooo0P00001<0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0G0?ooo`<00000503oool001<0oooo0`00000I0?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool0>@3oool4000005/0oooo0`00001:0?ooo`040000
003oool0oooo000001L0oooo00<000000?ooo`3oool05`3oool3000001P0oooo0`00000H0?ooo`@0
00004`3oool001<0oooo0`00000H0?ooo`@000005@3oool2000000050?ooo`000000oooo0?ooo`00
00006`3oool00`000000oooo0?ooo`0I0?ooo`@00000603oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool00P3oool4000001<0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0000000I0?ooo`D000005P3oool3000001P0oooo1000000G0?ooo`@0
00005`3oool600000180oooo000B0?ooo`D000005`3oool4000001L0oooo1000000K0?ooo`030000
003oool0oooo01X0oooo00@000000?ooo`3oool0oooo1000000D0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool06@3oool5000001L0oooo00@000000?ooo`3oool00000603oool30000
01P0oooo1@00000E0?ooo`H000005P3oool6000001D0oooo1P00000F0?ooo`H000004P3oool00140
oooo1`00000E0?ooo`H000005P3oool5000001P0oooo00<000000?ooo`0000006`3oool00`000000
oooo0?ooo`060?ooo`@00000403oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01H0
oooo1000000L0?ooo`<000005P3oool5000001L0oooo1P00000F0?ooo`D000005`3oool5000001H0
oooo1P00000G0?ooo`D000004@3oool00100oooo1P00000G0?ooo`D000005`3oool6000001L0oooo
0`00000H0?ooo`80000000<0oooo0000003oool02`3oool4000000`0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0>0?ooo`P00000103oool3000001L0oooo1@00000G0?ooo`D00000
5P3oool6000001L0oooo1@00000H0?ooo`@000005`3oool5000001P0oooo1000000@0?ooo`00403o
ool4000001P0oooo1000000H0?ooo`H000005`3oool5000001L0oooo1000000@0?ooo`@000001@3o
ool3000001T0oooo1P0000090?ooo`@000002P3oool5000001P0oooo1000000H0?ooo`@000005@3o
ool00`000000oooo0?ooo`05000001P0oooo0`00000I0?ooo`@00000603oool3000001/0oooo0P00
000?0?ooo`003`3oool2000001/0oooo0`00000I0?ooo`@0000000<0oooo0000000000005P3oool4
000001P0oooo1P00000B0?ooo`X000005`3oool600000080oooo1`00000>0?ooo`D00000603oool4
000001T0oooo0`00000C0?ooo`800000103oool4000001T0oooo0`00000J0?ooo`8000006P3oool2
000002/0oooo000[0?ooo`<000006P3oool3000000@0oooo00<000000?ooo`3oool04P3oool40000
01T0oooo1@00000H0?ooob@000005P3oool4000001T0oooo1000000H0?ooo`<000004@3oool20000
00P0oooo0P00000J0?ooo`800000703oool00`000000oooo0?ooo`140?ooo`00:`3oool00`000000
oooo0?ooo`0I0?ooo`<000001P3oool00`000000oooo0?ooo`0A0?ooo`<000006P3oool5000001P0
oooo1000000I0?ooo`@000006@3oool4000001X0oooo0`00000I0?ooo`<000003`3oool00`000000
oooo0?ooo`090?ooo`8000006`3oool00`000000oooo0?ooo`1P0?ooo`00A`3oool00`000000oooo
0?ooo`070?ooo`8000004@3oool2000001/0oooo1000000I0?ooo`@000006@3oool4000001X0oooo
0`00000K0?ooo`8000006P3oool2000000d0oooo0P00000=0?ooo`030000003oool0oooo07`0oooo
001C0?ooo`030000003oool0oooo00d0oooo0P00000L0?ooo`<000006`3oool2000001/0oooo0`00
000J0?ooo`<000006`3oool2000001/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`2<0?ooo`00E03oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01/0oooo0P00000L
0?ooo`8000006`3oool3000001/0oooo0P00000L0?ooo`030000003oool0oooo02@0oooo0P00002?
0?ooo`00E@3oool2000002X0oooo0P00000L0?ooo`8000006`3oool2000001`0oooo0P0000110?oo
o`800000T@3oool005L0oooo0P00000X0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo03d0oooo0P00002C0?ooo`00
F@3oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
F@3oool2000009D0oooo001J0?ooo`800000^`3oool2000009L0oooo001L0?ooo`030000003oool0
oooo0;H0oooo0P00002I0?ooo`00G@3oool200000;@0oooo0P00002K0?ooo`00G`3oool200000;00
oooo0P00002M0?ooo`00H@3oool200000:`0oooo0P00002O0?ooo`00H`3oool200000:L0oooo0`00
002Q0?ooo`00I@3oool200000:80oooo0`00002T0?ooo`00I`3oool2000009l0oooo0P00002V0?oo
o`00J@3oool2000009X0oooo0`00002X0?ooo`00J`3oool2000009H0oooo0P00002[0?ooo`00K@3o
ool200000980oooo0P00002]0?ooo`00K`3oool3000008/0oooo1000001S0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0C0?ooo`005@3oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo03l0oooo0`0000250?ooo`<00000C03oool00`00
0000oooo0?ooo`0G0?ooo`8000006@3oool3000001P0oooo0`00000D0?ooo`00503oool3000001P0
oooo0`00000I0?ooo`030000003oool0oooo02L0oooo0P0000210?ooo`800000<`3oool00`000000
oooo0?ooo`0G0?ooo`<000005`3oool5000001L0oooo1000000G0?ooo`@00000503oool001@0oooo
1000000G0?ooo`@00000603oool3000001P0oooo00@000000?ooo`3oool000003@3oool3000000/0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool06@3oool4000001P0oooo0`00000H0?ooo`<000005`3oool6000001D0
oooo1P00000F0?ooo`H000005@3oool6000001<0oooo000C0?ooo`H000005@3oool6000001H0oooo
1@00000G0?ooo`@00000403oool4000000@0oooo0P0000000`3oool000000?ooo`0J0?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool4000001D0oooo1P00000G0?ooo`D00000
5`3oool5000001L0oooo1@00000F0?ooo`H000005P3oool5000001H0oooo1P00000B0?ooo`004P3o
ool6000001H0oooo1@00000G0?ooo`H000005P3oool5000001<0oooo2@00000G0?ooo`D000005`3o
ool6000001H0oooo1P00000A0?ooo`@0000000<0oooo0000000000000`00000H0?ooo`@00000603o
ool4000001P0oooo1000000H0?ooo`@00000603oool4000001L0oooo1@00000A0?ooo`004@3oool5
000001L0oooo1000000I0?ooo`D000005`3oool4000001P0oooo1000000H0?ooo`D000005`3oool6
000001H0oooo1P00000=0?ooo`@000001P3oool4000001P0oooo1000000I0?ooo`<000006@3oool4
000001P0oooo1000000I0?ooo`8000006@3oool300000140oooo000A0?ooo`<000006@3oool30000
01T0oooo1@00000H0?ooo`@00000603oool8000001@0oooo1000000I0?ooo`D000005`3oool50000
00P0oooo1@00000:0?ooo`@000006@3oool3000001X0oooo0`00000J0?ooo`8000006P3oool20000
01/0oooo00<000000?ooo`3oool0603oool200000100oooo000@0?ooo`8000006P3oool3000001X0
oooo1000000H0?ooo`@000006@3oool3000000D0oooo0`00000A0?ooo`@000006@3oool4000001P0
oooo1@0000050?ooo`<00000403oool4000001T0oooo0`00000J0?ooo`8000006`3oool2000001/0
oooo00<000000?ooo`3oool0A@3oool002`0oooo00<000000?ooo`3oool06P3oool2000001X0oooo
0`00000J0?ooo`<00000203oool4000000d0oooo1000000I0?ooo`@000006@3oool9000001@0oooo
0`00000J0?ooo`8000006`3oool2000001/0oooo00<000000?ooo`3oool0H@3oool004P0oooo0P00
000K0?ooo`8000006`3oool2000000d0oooo200000060?ooo`8000006`3oool3000001H0oooo1P00
000K0?ooo`8000006P3oool2000001`0oooo00<000000?ooo`3oool0O@3oool004P0oooo00<00000
0?ooo`3oool06P3oool2000001/0oooo0P00000E0?ooo`P000006`3oool3000000d0oooo2@000004
0?ooo`8000006`3oool2000001/0oooo00<000000?ooo`3oool0VP3oool006D0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`h000003`3oool2000000@0oooo2P00000=0?oo
o`800000703oool00`000000oooo0?ooo`2f0?ooo`00PP3oool00`000000oooo0?ooo`0J0?ooo`03
0000003oool0oooo00/0oooo5@00000H0?ooo`030000003oool0oooo0=<0oooo002O0?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool0l03oool00?l0oooo/@3oool00?l0oooo/@3oool0
0?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool0
0?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool0
0?l0ooooP03oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01<0oooo000E0?ooo`03
0000003oool0oooo0?l0oooo<@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01L0
oooo0P00000I0?ooo`<00000503oool001@0oooo0`00000I0?ooo`030000003oool0oooo01P0oooo
00<000000?ooo`3oool0g`3oool00`000000oooo0?ooo`0F0?ooo`<000006@3oool3000001L0oooo
1@00000G0?ooo`@00000503oool001@0oooo1000000H0?ooo`<00000603oool3000001T0oooo00<0
00000?ooo`3oool0R`3oool2000001X0oooo0P00000H0?ooo`<000005`3oool5000001H0oooo1P00
000E0?ooo`H000005P3oool6000001<0oooo000C0?ooo`H000005P3oool6000001H0oooo1@00000G
0?ooo`<00000603oool4000001P0oooo1000000J0?ooo`030000003oool0oooo01T0oooo0`00000G
0?ooo`D000005`3oool5000001H0oooo1@00000H0?ooo`@000005`3oool5000001H0oooo1P00000F
0?ooo`D000004`3oool001<0oooo1@00000G0?ooo`D000005P3oool5000001P0oooo1@00000F0?oo
o`H000005P3oool6000001H0oooo1@00000G0?ooo`D000005`3oool5000001L0oooo1@00000G0?oo
o`D00000603oool3000001P0oooo1000000H0?ooo`@00000603oool400000180oooo000B0?ooo`@0
0000603oool5000001L0oooo1000000I0?ooo`@000005`3oool5000001L0oooo1P00000F0?ooo`D0
00005`3oool5000001P0oooo1000000H0?ooo`@00000603oool4000001T0oooo0`00000H0?ooo`@0
0000603oool4000001P0oooo1000000A0?ooo`004P3oool3000001T0oooo1000000H0?ooo`<00000
6P3oool4000001L0oooo1@00000H0?ooo`@00000603oool4000001P0oooo1000000H0?ooo`@00000
603oool4000001T0oooo0`00000I0?ooo`<000006P3oool2000001X0oooo0P00000J0?ooo`800000
4@3oool00140oooo0`00000J0?ooo`8000006P3oool3000001T0oooo1000000H0?ooo`@000006@3o
ool4000001P0oooo1000000H0?ooo`@000006@3oool3000001T0oooo0`00000I0?ooo`<000006P3o
ool2000001/0oooo0P00000K0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool03P3o
ool00140oooo00<000000?ooo`3oool06@3oool2000001X0oooo0`00000J0?ooo`<000006P3oool3
000001T0oooo0`00000I0?ooo`<000006@3oool3000001X0oooo0`00000I0?ooo`<000006P3oool3
000001X0oooo0P00000K0?ooo`030000003oool0oooo04H0oooo000]0?ooo`030000003oool0oooo
01T0oooo0P00000K0?ooo`8000006`3oool2000001X0oooo0`00000J0?ooo`8000006P3oool20000
01/0oooo0P00000J0?ooo`8000006`3oool2000001/0oooo00<000000?ooo`3oool0HP3oool004T0
oooo00<000000?ooo`3oool06P3oool2000001/0oooo0P00000K0?ooo`8000006P3oool2000001X0
oooo0P00000K0?ooo`8000006P3oool2000001`0oooo00<000000?ooo`3oool0OP3oool006H0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0oooo
00<000000?ooo`3oool0V`3oool008<0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0e03oool00?l0oooo/@3oool0
0?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool0
0?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00001\
\>"],
  ImageRangeCache->{{{0, 431}, {431, 0}} -> {18.7517, 18.7517, 0.223221, \
0.223221}}]
}, Closed]],

Cell["\<\
Notice that the innermost equipotentials tend to follow the shape of the \
electrode,while the outer ones tend to be circular until one approaches the \
artificial square edge of the computational mesh,which distorts the ideal \
circular shape in the far field.\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Electrode defined by parametric curve", "Section"],

Cell[TextData[{
  "Consider an electrode that can be described by a closed parametric curve \
of the form ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalC][t] = {x[t], y[t]}\)]],
  " where ",
  Cell[BoxData[
      \(TraditionalForm\`\[ScriptCapitalC][
          t + 2  \[Pi]\ n] = \[ScriptCapitalC][t]\)]],
  " for integer ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  ".  We can approximate the potential surrounding this electrode by \
discreting the boundary and applying our present tools.  For example, the \
polygon sketched below approximates an electrode in the shape of a \
lemniscate.  It is defined by 50 points, but looks smooth enough for our \
purposes.  (The more points, the longer the calculation.)"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(poly = 
        Table[10.  {Sin[t], Sin[2  t]} + {50.5, 50.5}, {t, 0, 
            2  \[Pi], \(2  \[Pi]\)\/50}];\)\), "\[IndentingNewLine]", 
    \(\(ListPlot[poly, PlotJoined \[Rule] True];\)\)}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.90952 0.0477132 -1.18015 0.0294884 [
[.23758 -0.01311 -6 -9 ]
[.23758 -0.01311 6 0 ]
[.47614 -0.01311 -6 -9 ]
[.47614 -0.01311 6 0 ]
[.71471 -0.01311 -6 -9 ]
[.71471 -0.01311 6 0 ]
[.95328 -0.01311 -6 -9 ]
[.95328 -0.01311 6 0 ]
[-0.01349 .14683 -12 -4.5 ]
[-0.01349 .14683 0 4.5 ]
[-0.01349 .29427 -12 -4.5 ]
[-0.01349 .29427 0 4.5 ]
[-0.01349 .44171 -12 -4.5 ]
[-0.01349 .44171 0 4.5 ]
[-0.01349 .58916 -12 -4.5 ]
[-0.01349 .58916 0 4.5 ]
[ -0.00099 -0.00061 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.23758 -0.00061 m
.23758 .00564 L
s
[(45)] .23758 -0.01311 0 1 Mshowa
.47614 -0.00061 m
.47614 .00564 L
s
[(50)] .47614 -0.01311 0 1 Mshowa
.71471 -0.00061 m
.71471 .00564 L
s
[(55)] .71471 -0.01311 0 1 Mshowa
.95328 -0.00061 m
.95328 .00564 L
s
[(60)] .95328 -0.01311 0 1 Mshowa
.125 Mabswid
.04672 -0.00061 m
.04672 .00314 L
s
.09444 -0.00061 m
.09444 .00314 L
s
.14215 -0.00061 m
.14215 .00314 L
s
.18986 -0.00061 m
.18986 .00314 L
s
.28529 -0.00061 m
.28529 .00314 L
s
.333 -0.00061 m
.333 .00314 L
s
.38072 -0.00061 m
.38072 .00314 L
s
.42843 -0.00061 m
.42843 .00314 L
s
.52386 -0.00061 m
.52386 .00314 L
s
.57157 -0.00061 m
.57157 .00314 L
s
.61928 -0.00061 m
.61928 .00314 L
s
.667 -0.00061 m
.667 .00314 L
s
.76242 -0.00061 m
.76242 .00314 L
s
.81014 -0.00061 m
.81014 .00314 L
s
.85785 -0.00061 m
.85785 .00314 L
s
.90556 -0.00061 m
.90556 .00314 L
s
.25 Mabswid
0 -0.00061 m
1 -0.00061 L
s
-0.00099 .14683 m
.00526 .14683 L
s
[(45)] -0.01349 .14683 1 0 Mshowa
-0.00099 .29427 m
.00526 .29427 L
s
[(50)] -0.01349 .29427 1 0 Mshowa
-0.00099 .44171 m
.00526 .44171 L
s
[(55)] -0.01349 .44171 1 0 Mshowa
-0.00099 .58916 m
.00526 .58916 L
s
[(60)] -0.01349 .58916 1 0 Mshowa
.125 Mabswid
-0.00099 .02888 m
.00276 .02888 L
s
-0.00099 .05837 m
.00276 .05837 L
s
-0.00099 .08785 m
.00276 .08785 L
s
-0.00099 .11734 m
.00276 .11734 L
s
-0.00099 .17632 m
.00276 .17632 L
s
-0.00099 .20581 m
.00276 .20581 L
s
-0.00099 .2353 m
.00276 .2353 L
s
-0.00099 .26478 m
.00276 .26478 L
s
-0.00099 .32376 m
.00276 .32376 L
s
-0.00099 .35325 m
.00276 .35325 L
s
-0.00099 .38274 m
.00276 .38274 L
s
-0.00099 .41223 m
.00276 .41223 L
s
-0.00099 .4712 m
.00276 .4712 L
s
-0.00099 .50069 m
.00276 .50069 L
s
-0.00099 .53018 m
.00276 .53018 L
s
-0.00099 .55967 m
.00276 .55967 L
s
.25 Mabswid
-0.00099 0 m
-0.00099 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.5 .30902 m
.5598 .38235 L
.61866 .45108 L
.67564 .51088 L
.72986 .558 L
.78045 .58947 L
.82662 .60332 L
.86764 .59868 L
.90286 .57584 L
.93172 .53623 L
.95378 .48235 L
.96868 .41757 L
.97619 .34598 L
.97619 .27206 L
.96868 .20046 L
.95378 .13569 L
.93172 .08181 L
.90286 .0422 L
.86764 .01936 L
.82662 .01472 L
.78045 .02857 L
.72986 .06004 L
.67564 .10716 L
.61866 .16696 L
.5598 .23568 L
.5 .30902 L
.4402 .38235 L
.38134 .45108 L
.32436 .51088 L
.27014 .558 L
.21955 .58947 L
.17338 .60332 L
.13236 .59868 L
.09714 .57584 L
.06828 .53623 L
.04622 .48235 L
.03132 .41757 L
.02381 .34598 L
.02381 .27206 L
.03132 .20046 L
.04622 .13569 L
.06828 .08181 L
.09714 .0422 L
.13236 .01936 L
.17338 .01472 L
.21955 .02857 L
.27014 .06004 L
.32436 .10716 L
.38134 .16696 L
.4402 .23568 L
Mistroke
.5 .30902 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3ooona
0?ooo`00M03oool01@3ogi`0B000000000000000M;oo00<0oooo00D0_g@0000000000000000007Bo
o`1A0?ooo`050;md0000000000000000001d_ol0103oool01@3o_g@00000000000000000M;oo0580
oooo00D0_g@0000000000000000007Boo`040?ooo`050;md0000000000000000001d_ol0DP3oool0
1@2oM00000000000000007@0_ooo00<0oooo00D0okmd000000000000000007Boo`0A0?ooo`00MP3o
ool00`2oM000M;oo0?ooo`020?ooo`070;md0000M;l0oooo0?ooo`3ogi`0B00007Boo`1?0?ooo`07
0;md0000M;l0oooo0?ooo`3ogi`0B00007Boo`030?ooo`0609a8M02oool0oooo0?oo_`1d07@0_ooo
D03oool01`2oM00007Bo0?ooo`3oool0omnL04P0001d_ol00P3oool01`2oM00007Bo0?ooo`3oool0
omnL04P0001d_ol0D03oool01P3OW4P007Bo0?ooo`3oool0okmd001d_`<0oooo00H0W4Qd0;ooo`3o
ool0oono07@0M02oool@0?ooo`00MP3oool00`2oM000M;oo0?ooo`070?ooo`030?noM000M;l0oooo
05<0oooo00H0okmd001d_`3oool0oooo0?noM000M;l30?ooo`030=nLB018W=l0oooo05@0oooo00<0
okmd001d_`3oool01P3oool00`3o_g@007Bo0?ooo`1>0?ooo`030?oo_`1d0000M;oo0080oooo00H0
oono07A8W03oool0oooo0?noM000M;l30?ooo`030=nLB018W=l0oooo00l0oooo001b0?ooo`030=nL
B0000000000000<0000000<0M;oo0?ooo`3oool01@3oool00`3o_g@007Bo0?ooo`1C0?ooo`060?no
M000M;l0oooo0?ooo`3OW4P0B9cO0`3oool00`3o_g@007Bo0?ooo`1D0?ooo`030?noM000M;l0oooo
00H0oooo00<0okmd001d_`3oool0CP3oool01`3o_g@004Q807@0M02oool0oooo0=nLB000M;l00P3o
ool00`3OW4P0B9cO0?ooo`020?ooo`030?noM000M;l0oooo00l0oooo001b0?ooo`060?noM00007@0
_ooo0?ooo`2oM000M;oo203oool00`2LB000M;oo0?ooo`1C0?ooo`0609a8001d_ol0oooo0?ooo`3O
W4P0B9cO0`3oool00`3o_g@007Bo0?ooo`1D0?ooo`0309a8001d_ol0oooo00H0oooo00<0W4P007Bo
o`3oool0CP3oool00`3ogi`0B4RL0?noM002000000060018W03oool0oooo0?ooo`3OW4P0B9cO0`3o
ool00`3o_g@007Bo0?ooo`0?0?ooo`00L`3oool01@3OW4P007Bo0?ooo`2oM000M;oo00<0oooo00<0
oono07@0000000000P0000000`18W=l0oooo0?ooo`1>0?ooo`030?oo_`1d000000000080000000H0
B9cO0?ooo`3oool0oooo0=nLB018W=l30?ooo`030?noM000M;l0oooo04l0oooo00<0oono07@00000
00000P0000001P18W=l0oooo0?ooo`3oool0oono07@000<0000000<0B9cO0?ooo`3oool0CP3oool0
0`3ookl0M01d0;ooo`060?ooo`030=nLB018W=l0oooo0080oooo00<0okmd001d_`3oool03`3oool0
07@0oooo00@0W4P004RLg`2oM000M;oo0`3oool00`3ookl0M01d0;ooo`1C0?ooo`030?oo_`1d07@0
_ooo00H0oooo00<0okmd04RLg`3oool00P3oool00`3OW4P0B9cO0?ooo`1?0?ooo`030?oo_`1d07@0
_ooo00H0oooo00<0oono07@0M02oool0E03oool00`2oM000B9cO0?ooo`050?ooo`030?noM018W=l0
oooo0080oooo00<0gia804RLg`3oool03`3oool007@0oooo00@0omnL04P0B018B000M;oo0`3oool0
0`3ookl0M01d0;ooo`1C0?ooo`030?oo_`1d07@0_ooo00L0oooo00H0W4P007Boo`3oool0omnL04P0
M02ooom@0?ooo`030?oo_`1d07@0_ooo00H0oooo00<0oono07@0M02oool0E03oool00`3ookl0M000
001d_`060?ooo`0609a8001d_ol0oooo0?oOW01807@0_ooo403oool007D0oooo00<0okmd0000001d
_ol00`3oool00`3ookl0M00000000002000000030000M02oool0oooo04h0oooo00<0oono07@00000
00000P0000000`0007@0_ooo0?ooo`020?ooo`050?oo_`1d000000000000B02Lgol0D@3oool00`3o
okl0M00000000002000000060000M02oool0oooo0?ooo`3ookl0M0000`0000000`0007@0_ooo0?oo
o`1A0?ooo`040;md0000000000000018W0<0oooo00D0oono07@000000000001809cOo`0A0?ooo`00
o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00
o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`006P3ooooo000009<00000103oool001X0
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01D0
oooo000J0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0E0?ooo`006P3oool00`000000oooo0?ooo`1M0?ooo`030000003oool0oooo05`0oooo00<0
00000?ooo`3oool0G@3oool00`000000oooo0?ooo`1L0?ooo`030000003oool0oooo01D0oooo000J
0?ooo`030000003oool0oooo0?l0ooooU03oool001X0oooo00<000000?ooo`3oool0o`3ooonD0?oo
o`006P3oool00`000000oooo0?ooo`100?ooo`D00000o`3oool30?ooo`H00000AP3oool001X0oooo
00<000000?ooo`3oool0>03oool8000000D0oooo0`00003l0?ooo`<000001P3oool8000003h0oooo
000J0?ooo`030000003oool0oooo03<0oooo1@00000@0?ooo`<00000mP3oool300000140oooo1@00
000i0?ooo`006P3oool00`000000oooo0?ooo`0a0?ooo`800000603oool400000>l0oooo0`00000I
0?ooo`800000=`3oool001X0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`0L0?oo
o`<00000j@3oool3000001h0oooo00<000000?ooo`3oool0=03oool001X0oooo00<000000?ooo`3o
ool0;P3oool200000280oooo0`00003S0?ooo`<000008P3oool2000003@0oooo000J0?ooo`<00000
;@3oool00`000000oooo0?ooo`0U0?ooo`800000g`3oool2000002L0oooo00<000000?ooo`3oool0
<@3oool001X0oooo00<000000?ooo`3oool0:`3oool2000002X0oooo0P00003K0?ooo`800000:P3o
ool200000340oooo000J0?ooo`030000003oool0oooo02T0oooo0P00000^0?ooo`800000e`3oool2
000002h0oooo0P00000_0?ooo`006P3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo
0300oooo00<000000?ooo`3oool0d`3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo
02`0oooo000J0?ooo`030000003oool0oooo02H0oooo0P00000d0?ooo`800000d@3oool2000003@0
oooo0P00000/0?ooo`006P3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo03H0oooo
0P00003=0?ooo`800000>03oool00`000000oooo0?ooo`0Y0?ooo`006P3oool00`000000oooo0?oo
o`0T0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool0b@3oool00`000000oooo0?oo
o`0i0?ooo`030000003oool0oooo02P0oooo000J0?ooo`030000003oool0oooo02<0oooo00<00000
0?ooo`3oool0>`3oool200000<L0oooo0P00000l0?ooo`030000003oool0oooo02P0oooo000J0?oo
o`030000003oool0oooo02<0oooo00<000000?ooo`3oool0?@3oool200000<<0oooo0P00000o0?oo
o`030000003oool0oooo02L0oooo000J0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3o
ool0@03oool00`000000oooo0?ooo`2o0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3o
ool09P3oool001X0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`120?ooo`800000
_@3oool2000004<0oooo00<000000?ooo`3oool09P3oool001X0oooo00<000000?ooo`3oool0803o
ool00`000000oooo0?ooo`150?ooo`800000^@3oool2000004H0oooo00<000000?ooo`3oool09@3o
ool001X0oooo0`00000P0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0]@3oool0
0`000000oooo0?ooo`170?ooo`030000003oool0oooo02@0oooo000J0?ooo`030000003oool0oooo
01l0oooo00<000000?ooo`3oool0B@3oool00`000000oooo0?ooo`2c0?ooo`030000003oool0oooo
04P0oooo00<000000?ooo`3oool0903oool001X0oooo00<000000?ooo`3oool07P3oool00`000000
oooo0?ooo`1;0?ooo`030000003oool0oooo0;40oooo00<000000?ooo`3oool0BP3oool00`000000
oooo0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo04d0oooo
00<000000?ooo`3oool0[P3oool2000004h0oooo00<000000?ooo`3oool08P3oool001X0oooo00<0
00000?ooo`3oool07@3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo0:`0oooo00<0
00000?ooo`3oool0C`3oool00`000000oooo0?ooo`0Q0?ooo`006P3oool00`000000oooo0?ooo`0L
0?ooo`030000003oool0oooo0500oooo0P00002[0?ooo`030000003oool0oooo0500oooo00<00000
0?ooo`3oool08@3oool001X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`1C0?oo
o`030000003oool0oooo0:L0oooo00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`0P0?oo
o`006P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo05D0oooo00<000000?ooo`3o
ool0Y@3oool00`000000oooo0?ooo`1D0?ooo`030000003oool0oooo01l0oooo000J0?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool0EP3oool00`000000oooo0?ooo`2S0?ooo`030000
003oool0oooo05D0oooo00<000000?ooo`3oool07`3oool001X0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo0:40oooo00<000000?ooo`3oool0E`3o
ool00`000000oooo0?ooo`0N0?ooo`006P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0
oooo05T0oooo00<000000?ooo`3oool0WP3oool2000005X0oooo00<000000?ooo`3oool07P3oool0
01X0oooo0`00000H0?ooo`030000003oool0oooo05/0oooo00<000000?ooo`3oool0W03oool00`00
0000oooo0?ooo`1K0?ooo`030000003oool0oooo01d0oooo000J0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool0G03oool00`000000oooo0?ooo`2J0?ooo`030000003oool0oooo05`0
oooo00<000000?ooo`3oool07@3oool001X0oooo00<000000?ooo`3oool05`3oool00`000000oooo
0?ooo`1N0?ooo`030000003oool0oooo09P0oooo00<000000?ooo`3oool0GP3oool00`000000oooo
0?ooo`0L0?ooo`006P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo05l0oooo0P00
002G0?ooo`030000003oool0oooo05l0oooo00<000000?ooo`3oool0703oool001X0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`1Q0?ooo`030000003oool0oooo09<0oooo00<00000
0?ooo`3oool0H03oool00`000000oooo0?ooo`0L0?ooo`006P3oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo06<0oooo00<000000?ooo`3oool0T03oool2000006@0oooo00<000000?oo
o`3oool06`3oool001X0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`1T0?ooo`03
0000003oool0oooo08h0oooo00<000000?ooo`3oool0I03oool00`000000oooo0?ooo`0K0?ooo`00
6P3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo06H0oooo00<000000?ooo`3oool0
S03oool00`000000oooo0?ooo`1V0?ooo`030000003oool0oooo01X0oooo000J0?ooo`030000003o
ool0oooo01D0oooo00<000000?ooo`3oool0I`3oool00`000000oooo0?ooo`2:0?ooo`030000003o
ool0oooo06L0oooo00<000000?ooo`3oool06P3oool001X0oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`1Y0?ooo`030000003oool0oooo08P0oooo00<000000?ooo`3oool0J03oool0
0`000000oooo0?ooo`0J0?ooo`006P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
06X0oooo00<000000?ooo`3oool0QP3oool00`000000oooo0?ooo`1Z0?ooo`030000003oool0oooo
01T0oooo000J0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0J`3oool00`000000
oooo0?ooo`240?ooo`030000003oool0oooo06/0oooo00<000000?ooo`3oool06@3oool001X0oooo
0`00000C0?ooo`030000003oool0oooo06d0oooo00<000000?ooo`3oool0PP3oool00`000000oooo
0?ooo`1]0?ooo`030000003oool0oooo01P0oooo000J0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool0KP3oool00`000000oooo0?ooo`200?ooo`030000003oool0oooo06h0oooo00<0
00000?ooo`3oool0603oool001X0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`1`
0?ooo`030000003oool0oooo07h0oooo00<000000?ooo`3oool0L03oool00`000000oooo0?ooo`0G
0?ooo`006P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0740oooo00<000000?oo
o`3oool0O03oool00`000000oooo0?ooo`1a0?ooo`030000003oool0oooo01L0oooo000J0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool0LP3oool00`000000oooo0?ooo`1j0?ooo`03
0000003oool0oooo0780oooo00<000000?ooo`3oool05`3oool001X0oooo00<000000?ooo`3oool0
4@3oool00`000000oooo0?ooo`1d0?ooo`030000003oool0oooo07P0oooo00<000000?ooo`3oool0
M03oool00`000000oooo0?ooo`0F0?ooo`006P3oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo07D0oooo00<000000?ooo`3oool0MP3oool00`000000oooo0?ooo`1e0?ooo`030000003o
ool0oooo01H0oooo000J0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0M`3oool0
0`000000oooo0?ooo`1d0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool05@3oool0
01X0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo
07<0oooo00<000000?ooo`3oool0N03oool00`000000oooo0?ooo`0E0?ooo`001P3oool01@3ogi`0
B000000000000000M;oo00<0oooo00D0_g@0000000000000000007Boo`070?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool0N03oool00`000000oooo0?ooo`1b0?ooo`030000003oool0
oooo07P0oooo00<000000?ooo`3oool05@3oool000P0oooo00<0_g@007Boo`3oool00P3oool01`2o
M00007Bo0?ooo`3oool0omnL04P0001d_ol01P3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo07T0oooo00<000000?ooo`3oool0L03oool00`000000oooo0?ooo`1i0?ooo`030000003o
ool0oooo01D0oooo00080?ooo`030;md001d_ol0oooo00L0oooo00<0okmd001d_`3oool01@3oool0
0`000000oooo0?ooo`0?0?ooo`030000003oool0oooo07/0oooo00<000000?ooo`3oool0KP3oool0
0`000000oooo0?ooo`1k0?ooo`030000003oool0oooo01@0oooo00040?ooo`030=nLB00000000000
00<0000000<0M;oo0?ooo`3oool01@3oool00`3o_g@007Bo0?ooo`050?ooo`@000003P3oool00`00
0000oooo0?ooo`1l0?ooo`030000003oool0oooo06`0oooo00<000000?ooo`3oool0O03oool00`00
0000oooo0?ooo`0D0?ooo`00103oool01P3o_g@0001d0;ooo`3oool0_g@007Boo`P0oooo00<0W4P0
07Boo`3oool01@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo07d0oooo00<00000
0?ooo`3oool0JP3oool00`000000oooo0?ooo`1m0?ooo`030000003oool0oooo01@0oooo00050?oo
o`050=nLB000M;l0oooo0;md001d_ol00`3oool00`3ookl0M000000000020000000304RLg`3oool0
oooo00D0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`1n0?ooo`030000003oool0
oooo06P0oooo00<000000?ooo`3oool0OP3oool00`000000oooo0?ooo`0D0?ooo`001P3oool0102L
B000B9cO0;md001d_ol30?ooo`030?oo_`1d07@0_ooo00X0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`200?ooo`030000003oool0oooo06H0oooo00<000000?ooo`3oool0P03oool0
0`000000oooo0?ooo`0C0?ooo`001P3oool0103ogi`0B01804Q8001d_ol30?ooo`030?oo_`1d07@0
_ooo00X0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`210?ooo`030000003oool0
oooo06@0oooo00<000000?ooo`3oool0P@3oool00`000000oooo0?ooo`0C0?ooo`001`3oool00`3o
_g@0000007Boo`030?ooo`030?oo_`1d000000000080000000<0001d0;ooo`3oool01@3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo0880oooo00<000000?ooo`3oool0HP3oool00`00
0000oooo0?ooo`220?ooo`030000003oool0oooo01<0oooo000J0?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool0P`3oool00`000000oooo0?ooo`1P0?ooo`030000003oool0oooo08<0
oooo00<000000?ooo`3oool04`3oool001X0oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`250?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0Q@3oool00`000000oooo
0?ooo`0B0?ooo`006P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo08H0oooo00<0
00000?ooo`3oool0G03oool00`000000oooo0?ooo`260?ooo`030000003oool0oooo0180oooo000J
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0Q`3oool00`000000oooo0?ooo`1J
0?ooo`030000003oool0oooo08L0oooo00<000000?ooo`3oool04P3oool001X0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`280?ooo`030000003oool0oooo05P0oooo00<000000?oo
o`3oool0R03oool00`000000oooo0?ooo`0B0?ooo`006P3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo08T0oooo00<000000?ooo`3oool0E`3oool00`000000oooo0?ooo`280?ooo`03
0000003oool0oooo0180oooo000J0?ooo`<00000303oool00`000000oooo0?ooo`2:0?ooo`030000
003oool0oooo05H0oooo00<000000?ooo`3oool0RP3oool00`000000oooo0?ooo`0A0?ooo`006P3o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo08/0oooo00<000000?ooo`3oool0E03o
ool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo0140oooo000J0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool0S03oool00`000000oooo0?ooo`1B0?ooo`030000003oool0
oooo08`0oooo00<000000?ooo`3oool04@3oool001X0oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`2=0?ooo`030000003oool0oooo0500oooo00<000000?ooo`3oool0S@3oool00`00
0000oooo0?ooo`0A0?ooo`006P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo08l0
oooo00<000000?ooo`3oool0CP3oool00`000000oooo0?ooo`2?0?ooo`030000003oool0oooo0100
oooo000J0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0T03oool00`000000oooo
0?ooo`1=0?ooo`030000003oool0oooo08l0oooo00<000000?ooo`3oool0403oool001X0oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`2A0?ooo`030000003oool0oooo04/0oooo00<0
00000?ooo`3oool0T03oool00`000000oooo0?ooo`0@0?ooo`006P3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo0940oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`2A
0?ooo`030000003oool0oooo0100oooo000J0?ooo`030000003oool0oooo00X0oooo00<000000?oo
o`3oool0T`3oool00`000000oooo0?ooo`180?ooo`030000003oool0oooo09<0oooo00<000000?oo
o`3oool03`3oool001X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`2D0?ooo`03
0000003oool0oooo04H0oooo00<000000?ooo`3oool0U03oool00`000000oooo0?ooo`0?0?ooo`00
6P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo09D0oooo00<000000?ooo`3oool0
A03oool00`000000oooo0?ooo`2E0?ooo`030000003oool0oooo00l0oooo000J0?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool0UP3oool00`000000oooo0?ooo`130?ooo`030000003o
ool0oooo09D0oooo00<000000?ooo`3oool03`3oool001X0oooo0`00000:0?ooo`030000003oool0
oooo09L0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`2F0?ooo`030000003oool0
oooo00l0oooo000J0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0V03oool00`00
0000oooo0?ooo`0o0?ooo`030000003oool0oooo09L0oooo00<000000?ooo`3oool03`3oool001X0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`2H0?ooo`030000003oool0oooo03h0
oooo00<000000?ooo`3oool0V03oool00`000000oooo0?ooo`0?0?ooo`006P3oool00`000000oooo
0?ooo`090?ooo`030000003oool0oooo09X0oooo00<000000?ooo`3oool0?03oool00`000000oooo
0?ooo`2J0?ooo`030000003oool0oooo00h0oooo000J0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool0V`3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo09X0oooo00<0
00000?ooo`3oool03P3oool001X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`2L
0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool0V`3oool00`000000oooo0?ooo`0>
0?ooo`006P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo09d0oooo00<000000?oo
o`3oool0=`3oool00`000000oooo0?ooo`2L0?ooo`030000003oool0oooo00h0oooo000J0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0WP3oool00`000000oooo0?ooo`0e0?ooo`03
0000003oool0oooo09d0oooo00<000000?ooo`3oool03P3oool001X0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`2O0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
WP3oool00`000000oooo0?ooo`0>0?ooo`006P3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo09l0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`2O0?ooo`030000003o
ool0oooo00h0oooo000J0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0X03oool0
0`000000oooo0?ooo`0a0?ooo`030000003oool0oooo09l0oooo00<000000?ooo`3oool03P3oool0
01X0oooo0`0000090?ooo`030000003oool0oooo0:40oooo00<000000?ooo`3oool0;`3oool00`00
0000oooo0?ooo`2P0?ooo`030000003oool0oooo00h0oooo000J0?ooo`030000003oool0oooo00P0
oooo00<000000?ooo`3oool0X`3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo0:40
oooo00<000000?ooo`3oool03P3oool001X0oooo00<000000?ooo`3oool0203oool00`000000oooo
0?ooo`2T0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0X`3oool00`000000oooo
0?ooo`0=0?ooo`006P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0:D0oooo00<0
00000?ooo`3oool0:@3oool00`000000oooo0?ooo`2T0?ooo`030000003oool0oooo00d0oooo000J
0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0Y@3oool00`000000oooo0?ooo`0Y
0?ooo`030000003oool0oooo0:@0oooo00<000000?ooo`3oool03@3oool001X0oooo00<000000?oo
o`3oool0203oool00`000000oooo0?ooo`2V0?ooo`030000003oool0oooo02L0oooo00<000000?oo
o`3oool0Y@3oool00`000000oooo0?ooo`0=0?ooo`006P3oool00`000000oooo0?ooo`080?ooo`03
0000003oool0oooo0:L0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`2V0?ooo`03
0000003oool0oooo00d0oooo000J0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
Z03oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0:L0oooo00<000000?ooo`3oool0
3@3oool001X0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`2Y0?ooo`030000003o
ool0oooo0240oooo00<000000?ooo`3oool0Z03oool00`000000oooo0?ooo`0=0?ooo`006P3oool0
0`000000oooo0?ooo`080?ooo`030000003oool0oooo0:X0oooo00<000000?ooo`3oool07`3oool0
0`000000oooo0?ooo`2Y0?ooo`030000003oool0oooo00d0oooo000J0?ooo`030000003oool0oooo
00P0oooo00<000000?ooo`3oool0ZP3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo
0:T0oooo00<000000?ooo`3oool03@3oool001X0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`2/0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0Z`3oool00`000000
oooo0?ooo`0<0?ooo`006P3oool3000000L0oooo00<000000?ooo`3oool0[@3oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo0:`0oooo00<000000?ooo`3oool0303oool001X0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`2^0?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`3oool0[@3oool00`000000oooo0?ooo`0<0?ooo`006P3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo0:l0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`2^
0?ooo`030000003oool0oooo00`0oooo000J0?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool0/03oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0:l0oooo00<000000?oo
o`3oool0303oool001X0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`2`0?ooo`03
0000003oool0oooo01D0oooo00<000000?ooo`3oool0[`3oool00`000000oooo0?ooo`0<0?ooo`00
6P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0;40oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`2`0?ooo`030000003oool0oooo00`0oooo000J0?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0/P3oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo0;40oooo00<000000?ooo`3oool0303oool001X0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`2c0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0/P3oool0
0`000000oooo0?ooo`0<0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
0;@0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`2c0?ooo`030000003oool0oooo
00`0oooo00050?ooo`050;md0000000000000000001d_ol0103oool01@3o_g@00000000000000000
M;oo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`2e0?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`3oool0]03oool00`000000oooo0?ooo`0<0?ooo`00103oool01`2o
M00007Bo0?ooo`3oool0omnL04P0001d_ol00`3oool01P2LB7@0_ooo0?ooo`3ookl0M01d0;ooo`H0
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`2e0?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool0]03oool00`000000oooo0?ooo`0<0?ooo`002@3oool01P3o_g@007Bo
0?ooo`3oool0okmd001d_`<0oooo00<0gia804RLg`3oool01@3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo0;H0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`2e0?oo
o`030000003oool0oooo00`0oooo00090?ooo`060?noM000M;l0oooo0?ooo`3OW4P0B9cO0`3oool0
0`3o_g@007Bo0?ooo`050?ooo`@000001P3oool00`000000oooo0?ooo`2g0?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool0]P3oool00`000000oooo0?ooo`0<0?ooo`002@3oool01P2L
B000M;oo0?ooo`3oool0gia804RLg`<0oooo00<0okmd001d_`3oool01@3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo0;P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`2g0?ooo`030000003oool0oooo00`0oooo00040?ooo`030?oo_`1d000000000080000000H0B9cO
0?ooo`3oool0oooo0=nLB018W=l30?ooo`030?noM000M;l0oooo00D0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`2i0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
^03oool00`000000oooo0?ooo`0<0?ooo`00103oool00`3ookl0M01d0;ooo`060?ooo`030?noM018
W=l0oooo0080oooo00<0gia804RLg`3oool01@3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo0;X0oooo00D000000?ooo`3oool0oooo0000002k0?ooo`030000003oool0oooo00`0oooo
00040?ooo`030?oo_`1d07@0_ooo00L0oooo00H0W4P007Boo`3oool0omnL04P0M02oool60?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool0^P3oool01@000000oooo0?ooo`3oool00000
0;/0oooo00<000000?ooo`3oool0303oool000@0oooo00<0oono07@0000000000P0000000`0007@0
_ooo0?ooo`020?ooo`050?oo_`1d000000000000B02Lgol01`3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo0;/0oooo00<000000?ooo`000000_03oool00`000000oooo0?ooo`0<0?oo
o`006P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0;`0oooo00<000000?ooo`3o
ool0^`3oool00`000000oooo0?ooo`0<0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo0;/0oooo00<000000?ooo`000000_03oool00`000000oooo0?ooo`0<0?ooo`006P3o
ool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0;X0oooo00D000000?ooo`3oool0oooo
0000002k0?ooo`030000003oool0oooo00`0oooo000J0?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool0^P3oool01@000000oooo0?ooo`3oool000000;/0oooo00<000000?ooo`3oool0
303oool001X0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`2i0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool0^03oool00`000000oooo0?ooo`0<0?ooo`006P3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo0;P0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`2g0?ooo`030000003oool0oooo00`0oooo000J0?ooo`<000001`3oool00`00
0000oooo0?ooo`2g0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0]P3oool00`00
0000oooo0?ooo`0<0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0;H0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`2e0?ooo`030000003oool0oooo00`0
oooo000J0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0]@3oool00`000000oooo
0?ooo`0;0?ooo`030000003oool0oooo0;@0oooo00<000000?ooo`3oool0303oool001X0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`2e0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool0]03oool00`000000oooo0?ooo`0<0?ooo`006P3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo0;@0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`2c
0?ooo`030000003oool0oooo00`0oooo000J0?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool0/`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0;80oooo00<000000?oo
o`3oool0303oool001X0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`2b0?ooo`03
0000003oool0oooo0140oooo00<000000?ooo`3oool0/@3oool00`000000oooo0?ooo`0<0?ooo`00
6P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0;40oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`2`0?ooo`030000003oool0oooo00`0oooo000J0?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool0/03oool00`000000oooo0?ooo`0E0?ooo`030000003o
ool0oooo0:l0oooo00<000000?ooo`3oool0303oool001X0oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`2`0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0[`3oool0
0`000000oooo0?ooo`0<0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
0:l0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`2^0?ooo`030000003oool0oooo
00`0oooo000J0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0[P3oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo0:d0oooo00<000000?ooo`3oool0303oool001X0oooo
0`0000070?ooo`030000003oool0oooo0:d0oooo00<000000?ooo`3oool06`3oool00`000000oooo
0?ooo`2/0?ooo`030000003oool0oooo00`0oooo000J0?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool0[03oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0:/0oooo00<0
00000?ooo`3oool0303oool001X0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`2Z
0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0Z@3oool00`000000oooo0?ooo`0=
0?ooo`006P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0:X0oooo00<000000?oo
o`3oool07`3oool00`000000oooo0?ooo`2Y0?ooo`030000003oool0oooo00d0oooo000J0?ooo`03
0000003oool0oooo00P0oooo00<000000?ooo`3oool0Z@3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0:P0oooo00<000000?ooo`3oool03@3oool001X0oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`2X0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0
Y`3oool00`000000oooo0?ooo`0=0?ooo`006P3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo0:L0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`2V0?ooo`030000003o
ool0oooo00d0oooo000J0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0YP3oool0
0`000000oooo0?ooo`0W0?ooo`030000003oool0oooo0:D0oooo00<000000?ooo`3oool03@3oool0
01X0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`2U0?ooo`030000003oool0oooo
02T0oooo00<000000?ooo`3oool0Y03oool00`000000oooo0?ooo`0=0?ooo`006P3oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo0:D0oooo00<000000?ooo`3oool0:@3oool00`000000
oooo0?ooo`2T0?ooo`030000003oool0oooo00d0oooo000J0?ooo`030000003oool0oooo00P0oooo
00<000000?ooo`3oool0Y03oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo0:<0oooo
00<000000?ooo`3oool03@3oool001X0oooo0`0000080?ooo`030000003oool0oooo0:<0oooo00<0
00000?ooo`3oool0;@3oool00`000000oooo0?ooo`2Q0?ooo`030000003oool0oooo00h0oooo000J
0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0X@3oool00`000000oooo0?ooo`0_
0?ooo`030000003oool0oooo0:00oooo00<000000?ooo`3oool03P3oool001X0oooo00<000000?oo
o`3oool02@3oool00`000000oooo0?ooo`2P0?ooo`030000003oool0oooo0340oooo00<000000?oo
o`3oool0W`3oool00`000000oooo0?ooo`0>0?ooo`006P3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo09l0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`2O0?ooo`03
0000003oool0oooo00h0oooo000J0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
W`3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo09h0oooo00<000000?ooo`3oool0
3P3oool001X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`2N0?ooo`030000003o
ool0oooo03D0oooo00<000000?ooo`3oool0W@3oool00`000000oooo0?ooo`0>0?ooo`006P3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo09d0oooo00<000000?ooo`3oool0=`3oool0
0`000000oooo0?ooo`2L0?ooo`030000003oool0oooo00h0oooo000J0?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool0W03oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo
09/0oooo00<000000?ooo`3oool03P3oool001X0oooo00<000000?ooo`3oool02@3oool00`000000
oooo0?ooo`2K0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0VP3oool00`000000
oooo0?ooo`0>0?ooo`006P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo09X0oooo
00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`2J0?ooo`030000003oool0oooo00h0oooo
000J0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0V03oool00`000000oooo0?oo
o`0n0?ooo`030000003oool0oooo09P0oooo00<000000?ooo`3oool03`3oool001X0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`2H0?ooo`030000003oool0oooo03l0oooo00<00000
0?ooo`3oool0U`3oool00`000000oooo0?ooo`0?0?ooo`006P3oool3000000X0oooo00<000000?oo
o`3oool0U`3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo09H0oooo00<000000?oo
o`3oool03`3oool001X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`2F0?ooo`03
0000003oool0oooo04<0oooo00<000000?ooo`3oool0U@3oool00`000000oooo0?ooo`0?0?ooo`00
6P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo09D0oooo00<000000?ooo`3oool0
A03oool00`000000oooo0?ooo`2E0?ooo`030000003oool0oooo00l0oooo000J0?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool0U03oool00`000000oooo0?ooo`160?ooo`030000003o
ool0oooo09@0oooo00<000000?ooo`3oool03`3oool001X0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`2C0?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3oool0T`3oool0
0`000000oooo0?ooo`0?0?ooo`006P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo
0940oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`2A0?ooo`030000003oool0oooo
0100oooo000J0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0T@3oool00`000000
oooo0?ooo`1;0?ooo`030000003oool0oooo0900oooo00<000000?ooo`3oool0403oool001X0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`2@0?ooo`030000003oool0oooo04d0oooo
00<000000?ooo`3oool0S`3oool00`000000oooo0?ooo`0@0?ooo`001@3oool01@2oM00000000000
00000000M;oo00@0oooo00D0_g@0000000000000000007Boo`070?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool0S`3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo08l0
oooo00<000000?ooo`3oool0403oool000@0oooo00L0_g@0001d_`3oool0oooo0?oOW0180000M;oo
0080oooo00L0_g@0001d_`3oool0oooo0?oOW0180000M;oo00H0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`2=0?ooo`030000003oool0oooo0500oooo00<000000?ooo`3oool0S@3o
ool00`000000oooo0?ooo`0A0?ooo`002@3oool00`3o_g@007Bo0?ooo`060?ooo`030?noM000M;l0
oooo00D0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`2<0?ooo`030000003oool0
oooo0580oooo00<000000?ooo`3oool0S03oool00`000000oooo0?ooo`0A0?ooo`002@3oool00`3o
_g@007Bo0?ooo`060?ooo`030?noM000M;l0oooo00D0oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`2;0?ooo`030000003oool0oooo05@0oooo00<000000?ooo`3oool0R`3oool00`00
0000oooo0?ooo`0A0?ooo`002@3oool00`2LB000M;oo0?ooo`060?ooo`0309a8001d_ol0oooo00D0
oooo1000000;0?ooo`030000003oool0oooo08X0oooo00<000000?ooo`3oool0EP3oool00`000000
oooo0?ooo`2:0?ooo`030000003oool0oooo0140oooo00040?ooo`030?oo_`1d0000000000800000
00H0B9cO0?ooo`3oool0oooo0?oo_`1d00030000000304RLg`3oool0oooo00D0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`290?ooo`030000003oool0oooo05L0oooo00<000000?oo
o`3oool0R03oool00`000000oooo0?ooo`0B0?ooo`00103oool00`3ookl0M01d0;ooo`060?ooo`03
0?oo_`1d07@0_ooo00X0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`280?ooo`03
0000003oool0oooo05P0oooo00<000000?ooo`3oool0R03oool00`000000oooo0?ooo`0B0?ooo`00
103oool00`3ookl0M01d0;ooo`060?ooo`030?oo_`1d07@0_ooo00X0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`270?ooo`030000003oool0oooo05X0oooo00<000000?ooo`3oool0
Q`3oool00`000000oooo0?ooo`0B0?ooo`00103oool00`3ookl0M00000000002000000060000M02o
ool0oooo0?ooo`3ookl0M0000`0000000`0007@0_ooo0?ooo`050?ooo`030000003oool0oooo00d0
oooo00<000000?ooo`3oool0QP3oool00`000000oooo0?ooo`1L0?ooo`030000003oool0oooo08H0
oooo00<000000?ooo`3oool04P3oool001X0oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`250?ooo`030000003oool0oooo05h0oooo00<000000?ooo`3oool0Q@3oool00`000000oooo
0?ooo`0B0?ooo`006P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo08<0oooo00<0
00000?ooo`3oool0H03oool00`000000oooo0?ooo`230?ooo`030000003oool0oooo01<0oooo000J
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0PP3oool00`000000oooo0?ooo`1R
0?ooo`030000003oool0oooo0880oooo00<000000?ooo`3oool04`3oool001X0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`210?ooo`030000003oool0oooo06@0oooo00<000000?oo
o`3oool0P@3oool00`000000oooo0?ooo`0C0?ooo`006P3oool00`000000oooo0?ooo`0>0?ooo`03
0000003oool0oooo0800oooo00<000000?ooo`3oool0IP3oool00`000000oooo0?ooo`200?ooo`03
0000003oool0oooo01<0oooo000J0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
OP3oool00`000000oooo0?ooo`1X0?ooo`030000003oool0oooo07h0oooo00<000000?ooo`3oool0
503oool001X0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`1m0?ooo`030000003o
ool0oooo06X0oooo00<000000?ooo`3oool0O@3oool00`000000oooo0?ooo`0D0?ooo`006P3oool3
000000l0oooo00<000000?ooo`3oool0O03oool00`000000oooo0?ooo`1/0?ooo`030000003oool0
oooo07`0oooo00<000000?ooo`3oool0503oool001X0oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`1k0?ooo`030000003oool0oooo06h0oooo00<000000?ooo`3oool0N`3oool00`00
0000oooo0?ooo`0D0?ooo`006P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo07T0
oooo00<000000?ooo`3oool0L03oool00`000000oooo0?ooo`1i0?ooo`030000003oool0oooo01D0
oooo000J0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0N03oool00`000000oooo
0?ooo`1b0?ooo`030000003oool0oooo07P0oooo00<000000?ooo`3oool05@3oool001X0oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo07<0oooo00<0
00000?ooo`3oool0N03oool00`000000oooo0?ooo`0E0?ooo`006P3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool0M03oool00`000000oooo0?ooo`1g
0?ooo`030000003oool0oooo01D0oooo000J0?ooo`030000003oool0oooo0140oooo00<000000?oo
o`3oool0M@3oool00`000000oooo0?ooo`1f0?ooo`030000003oool0oooo07D0oooo00<000000?oo
o`3oool05P3oool001X0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`1d0?ooo`03
0000003oool0oooo07P0oooo00<000000?ooo`3oool0M03oool00`000000oooo0?ooo`0F0?ooo`00
6P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0780oooo00<000000?ooo`3oool0
NP3oool00`000000oooo0?ooo`1b0?ooo`030000003oool0oooo01L0oooo000J0?ooo`030000003o
ool0oooo0180oooo00<000000?ooo`3oool0L@3oool00`000000oooo0?ooo`1l0?ooo`030000003o
ool0oooo0740oooo00<000000?ooo`3oool05`3oool001X0oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`1`0?ooo`030000003oool0oooo07h0oooo00<000000?ooo`3oool0L03oool0
0`000000oooo0?ooo`0G0?ooo`006P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo
06h0oooo00<000000?ooo`3oool0P03oool00`000000oooo0?ooo`1^0?ooo`030000003oool0oooo
01P0oooo000J0?ooo`<000004`3oool00`000000oooo0?ooo`1]0?ooo`030000003oool0oooo0880
oooo00<000000?ooo`3oool0K@3oool00`000000oooo0?ooo`0H0?ooo`006P3oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo06/0oooo00<000000?ooo`3oool0Q03oool00`000000oooo
0?ooo`1[0?ooo`030000003oool0oooo01T0oooo000J0?ooo`030000003oool0oooo01@0oooo00<0
00000?ooo`3oool0JP3oool00`000000oooo0?ooo`260?ooo`030000003oool0oooo06X0oooo00<0
00000?ooo`3oool06@3oool001X0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`1Y
0?ooo`030000003oool0oooo08P0oooo00<000000?ooo`3oool0J03oool00`000000oooo0?ooo`0J
0?ooo`006P3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo06L0oooo00<000000?oo
o`3oool0RP3oool00`000000oooo0?ooo`1W0?ooo`030000003oool0oooo01X0oooo000J0?ooo`03
0000003oool0oooo01D0oooo00<000000?ooo`3oool0IP3oool00`000000oooo0?ooo`2<0?ooo`03
0000003oool0oooo06H0oooo00<000000?ooo`3oool06P3oool001X0oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`1T0?ooo`030000003oool0oooo08h0oooo0P00001U0?ooo`030000
003oool0oooo01/0oooo000J0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0HP3o
ool2000009<0oooo00<000000?ooo`3oool0HP3oool00`000000oooo0?ooo`0K0?ooo`006P3oool0
0`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0600oooo00<000000?ooo`3oool0U03oool0
0`000000oooo0?ooo`1P0?ooo`030000003oool0oooo01`0oooo000J0?ooo`030000003oool0oooo
01L0oooo00<000000?ooo`3oool0G`3oool00`000000oooo0?ooo`2F0?ooo`030000003oool0oooo
05l0oooo00<000000?ooo`3oool0703oool001X0oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`1N0?ooo`030000003oool0oooo09P0oooo00<000000?ooo`3oool0GP3oool00`000000
oooo0?ooo`0L0?ooo`006P3oool3000001P0oooo00<000000?ooo`3oool0G03oool00`000000oooo
0?ooo`2J0?ooo`800000G@3oool00`000000oooo0?ooo`0M0?ooo`006P3oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo05/0oooo00<000000?ooo`3oool0W@3oool00`000000oooo0?oo
o`1J0?ooo`030000003oool0oooo01d0oooo000J0?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool0F03oool200000:40oooo00<000000?ooo`3oool0F03oool00`000000oooo0?ooo`0N
0?ooo`006P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo05L0oooo00<000000?oo
o`3oool0XP3oool00`000000oooo0?ooo`1G0?ooo`030000003oool0oooo01h0oooo000J0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool0E@3oool00`000000oooo0?ooo`2T0?ooo`80
0000EP3oool00`000000oooo0?ooo`0O0?ooo`006P3oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo05@0oooo00<000000?ooo`3oool0Y`3oool00`000000oooo0?ooo`1C0?ooo`030000
003oool0oooo01l0oooo000J0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0DP3o
ool00`000000oooo0?ooo`2Y0?ooo`030000003oool0oooo0540oooo00<000000?ooo`3oool0803o
ool001X0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`1@0?ooo`030000003oool0
oooo0:/0oooo00<000000?ooo`3oool0C`3oool00`000000oooo0?ooo`0Q0?ooo`006P3oool00`00
0000oooo0?ooo`0M0?ooo`030000003oool0oooo04d0oooo0P00002_0?ooo`030000003oool0oooo
04h0oooo00<000000?ooo`3oool08@3oool001X0oooo00<000000?ooo`3oool07@3oool00`000000
oooo0?ooo`1<0?ooo`030000003oool0oooo0;00oooo0P00001=0?ooo`030000003oool0oooo0280
oooo000J0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0BP3oool00`000000oooo
0?ooo`2c0?ooo`030000003oool0oooo04T0oooo00<000000?ooo`3oool08`3oool001X0oooo00<0
00000?ooo`3oool07`3oool00`000000oooo0?ooo`180?ooo`030000003oool0oooo0;D0oooo00<0
00000?ooo`3oool0A`3oool00`000000oooo0?ooo`0T0?ooo`006P3oool300000200oooo00<00000
0?ooo`3oool0A@3oool200000;T0oooo0P0000170?ooo`030000003oool0oooo02@0oooo000J0?oo
o`030000003oool0oooo0200oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`2k0?oo
o`030000003oool0oooo04<0oooo00<000000?ooo`3oool09@3oool001X0oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`110?ooo`800000_`3oool200000480oooo00<000000?ooo`3o
ool09P3oool001X0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0o0?ooo`030000
003oool0oooo0<40oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0V0?ooo`006P3o
ool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo03`0oooo0P0000350?ooo`800000?P3o
ool00`000000oooo0?ooo`0W0?ooo`006P3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0
oooo03/0oooo00<000000?ooo`3oool0a`3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0
oooo02P0oooo000J0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0>03oool20000
0</0oooo0P00000j0?ooo`030000003oool0oooo02P0oooo000J0?ooo`030000003oool0oooo02D0
oooo00<000000?ooo`3oool0=@3oool200000<l0oooo0P00000g0?ooo`030000003oool0oooo02T0
oooo00060?ooo`050;md0000000000000000M02oool00`3oool01@3o_g@00000000000000000M;oo
00L0oooo00<000000?ooo`3oool09P3oool2000003@0oooo00<000000?ooo`3oool0d@3oool00`00
0000oooo0?ooo`0b0?ooo`800000;03oool000D0oooo00H0gia8001d_`3oool0oooo0?noM000M;l3
0?ooo`0609a8M02oool0oooo0?oo_`1d07@0_ooo1P3oool00`000000oooo0?ooo`0X0?ooo`030000
003oool0oooo02l0oooo0P00003E0?ooo`800000<@3oool00`000000oooo0?ooo`0/0?ooo`00103o
ool00`3ookl0M00007Boo`020?ooo`060?oo_`1dB9`0oooo0?ooo`3o_g@007Bo0`3oool00`3OW4P0
B9cO0?ooo`050?ooo`030000003oool0oooo02T0oooo0P00000^0?ooo`030000003oool0oooo0=L0
oooo00<000000?ooo`3oool0;03oool2000002l0oooo00040?ooo`070?noM000B4P0M01d0;ooo`3o
ool0gia8001d_`020?ooo`030=nLB018W=l0oooo0080oooo00<0okmd001d_`3oool01@3oool00`00
0000oooo0?ooo`0[0?ooo`030000003oool0oooo02T0oooo0P00003K0?ooo`800000:`3oool00`00
0000oooo0?ooo`0_0?ooo`00103oool00`3ogi`0B4RL0?noM002000000060018W03oool0oooo0?oo
o`3OW4P0B9cO0`3oool00`3o_g@007Bo0?ooo`050?ooo`@00000:`3oool2000002L0oooo0P00003O
0?ooo`8000009`3oool200000380oooo00040?ooo`030?oo_`1d07@0_ooo00H0oooo00<0gia804RL
g`3oool00P3oool00`3o_g@007Bo0?ooo`050?ooo`030000003oool0oooo02h0oooo0P00000R0?oo
o`<00000h`3oool300000280oooo0P00000d0?ooo`001@3oool00`2oM000B9cO0?ooo`050?ooo`03
0?noM018W=l0oooo0080oooo00<0gia804RLg`3oool01@3oool00`000000oooo0?ooo`0`0?ooo`03
0000003oool0oooo01`0oooo0`00003Y0?ooo`<000007P3oool00`000000oooo0?ooo`0d0?ooo`00
1@3oool00`3ookl0M000001d_`060?ooo`0609a8001d_ol0oooo0?oOW01807@0_ooo1P3oool00`00
0000oooo0?ooo`0a0?ooo`800000603oool400000>l0oooo0`00000I0?ooo`800000=`3oool000L0
oooo00@0_g@000000000000004RL0`3oool01@3ookl0M0000000000004P0W=oo00L0oooo00<00000
0?ooo`3oool0<`3oool400000140oooo0`00003f0?ooo`<000004@3oool5000003T0oooo000J0?oo
o`030000003oool0oooo03L0oooo200000060?ooo`<00000o03oool3000000H0oooo2000000n0?oo
o`006P3oool00`000000oooo0?ooo`0o0?ooo`H00000o`3oool30?ooo`H00000AP3oool001X0oooo
00<000000?ooo`3oool0o`3ooonD0?ooo`006P3oool00`000000oooo0?ooo`3o0?oooi@0oooo000J
0?ooo`030000003oool0oooo0?l0ooooU03oool001X0oooo00<000000?ooo`3oool0o`3ooonD0?oo
o`006P3oool00`000000oooo0?ooo`3o0?oooi@0oooo0000\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {38.5918, 38.4312, 0.0786206, \
0.127211}}]
}, Closed]],

Cell["\<\
The construction of the boundary is now somewhat time-consuming\
\>", "Text"],

Cell[BoxData[{
    \(\(Clear[boundary, \[Phi], \[Phi]0];\)\), "\[IndentingNewLine]", 
    \(\(boundary = Table[False, {i, 1, 120}, {j, 1, 120}];\)\), "\n", 
    \(\(\[Phi] = Table[0.0, {i, 1, 120}, {j, 1, 120}];\)\)}], "Input"],

Cell[BoxData[
    \(\(Do[
        boundary\[LeftDoubleBracket]i, j\[RightDoubleBracket] = 
          InPolygonQ[{i, j}, poly], {i, 1, 120}, {j, 1, 120}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(DisplayTogether[ListPlot[Position[boundary, True]], 
        ListPlot[poly, PlotJoined \[Rule] True]];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.90952 0.0477132 -1.18015 0.0294884 [
[.23758 -0.01311 -6 -9 ]
[.23758 -0.01311 6 0 ]
[.47614 -0.01311 -6 -9 ]
[.47614 -0.01311 6 0 ]
[.71471 -0.01311 -6 -9 ]
[.71471 -0.01311 6 0 ]
[.95328 -0.01311 -6 -9 ]
[.95328 -0.01311 6 0 ]
[-0.01349 .14683 -12 -4.5 ]
[-0.01349 .14683 0 4.5 ]
[-0.01349 .29427 -12 -4.5 ]
[-0.01349 .29427 0 4.5 ]
[-0.01349 .44171 -12 -4.5 ]
[-0.01349 .44171 0 4.5 ]
[-0.01349 .58916 -12 -4.5 ]
[-0.01349 .58916 0 4.5 ]
[ -0.00099 -0.00061 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.23758 -0.00061 m
.23758 .00564 L
s
[(45)] .23758 -0.01311 0 1 Mshowa
.47614 -0.00061 m
.47614 .00564 L
s
[(50)] .47614 -0.01311 0 1 Mshowa
.71471 -0.00061 m
.71471 .00564 L
s
[(55)] .71471 -0.01311 0 1 Mshowa
.95328 -0.00061 m
.95328 .00564 L
s
[(60)] .95328 -0.01311 0 1 Mshowa
.125 Mabswid
.04672 -0.00061 m
.04672 .00314 L
s
.09444 -0.00061 m
.09444 .00314 L
s
.14215 -0.00061 m
.14215 .00314 L
s
.18986 -0.00061 m
.18986 .00314 L
s
.28529 -0.00061 m
.28529 .00314 L
s
.333 -0.00061 m
.333 .00314 L
s
.38072 -0.00061 m
.38072 .00314 L
s
.42843 -0.00061 m
.42843 .00314 L
s
.52386 -0.00061 m
.52386 .00314 L
s
.57157 -0.00061 m
.57157 .00314 L
s
.61928 -0.00061 m
.61928 .00314 L
s
.667 -0.00061 m
.667 .00314 L
s
.76242 -0.00061 m
.76242 .00314 L
s
.81014 -0.00061 m
.81014 .00314 L
s
.85785 -0.00061 m
.85785 .00314 L
s
.90556 -0.00061 m
.90556 .00314 L
s
.25 Mabswid
0 -0.00061 m
1 -0.00061 L
s
-0.00099 .14683 m
.00526 .14683 L
s
[(45)] -0.01349 .14683 1 0 Mshowa
-0.00099 .29427 m
.00526 .29427 L
s
[(50)] -0.01349 .29427 1 0 Mshowa
-0.00099 .44171 m
.00526 .44171 L
s
[(55)] -0.01349 .44171 1 0 Mshowa
-0.00099 .58916 m
.00526 .58916 L
s
[(60)] -0.01349 .58916 1 0 Mshowa
.125 Mabswid
-0.00099 .02888 m
.00276 .02888 L
s
-0.00099 .05837 m
.00276 .05837 L
s
-0.00099 .08785 m
.00276 .08785 L
s
-0.00099 .11734 m
.00276 .11734 L
s
-0.00099 .17632 m
.00276 .17632 L
s
-0.00099 .20581 m
.00276 .20581 L
s
-0.00099 .2353 m
.00276 .2353 L
s
-0.00099 .26478 m
.00276 .26478 L
s
-0.00099 .32376 m
.00276 .32376 L
s
-0.00099 .35325 m
.00276 .35325 L
s
-0.00099 .38274 m
.00276 .38274 L
s
-0.00099 .41223 m
.00276 .41223 L
s
-0.00099 .4712 m
.00276 .4712 L
s
-0.00099 .50069 m
.00276 .50069 L
s
-0.00099 .53018 m
.00276 .53018 L
s
-0.00099 .55967 m
.00276 .55967 L
s
.25 Mabswid
-0.00099 0 m
-0.00099 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
.04672 .14683 Mdot
.04672 .17632 Mdot
.04672 .20581 Mdot
.04672 .2353 Mdot
.04672 .26478 Mdot
.04672 .29427 Mdot
.04672 .32376 Mdot
.04672 .35325 Mdot
.04672 .38274 Mdot
.04672 .41223 Mdot
.04672 .44171 Mdot
.04672 .4712 Mdot
.09444 .05837 Mdot
.09444 .08785 Mdot
.09444 .11734 Mdot
.09444 .14683 Mdot
.09444 .17632 Mdot
.09444 .20581 Mdot
.09444 .2353 Mdot
.09444 .26478 Mdot
.09444 .29427 Mdot
.09444 .32376 Mdot
.09444 .35325 Mdot
.09444 .38274 Mdot
.09444 .41223 Mdot
.09444 .44171 Mdot
.09444 .4712 Mdot
.09444 .50069 Mdot
.09444 .53018 Mdot
.09444 .55967 Mdot
.14215 .02888 Mdot
.14215 .05837 Mdot
.14215 .08785 Mdot
.14215 .11734 Mdot
.14215 .14683 Mdot
.14215 .17632 Mdot
.14215 .20581 Mdot
.14215 .2353 Mdot
.14215 .26478 Mdot
.14215 .29427 Mdot
.14215 .32376 Mdot
.14215 .35325 Mdot
.14215 .38274 Mdot
.14215 .41223 Mdot
.14215 .44171 Mdot
.14215 .4712 Mdot
.14215 .50069 Mdot
.14215 .53018 Mdot
.14215 .55967 Mdot
.14215 .58916 Mdot
.18986 .02888 Mdot
.18986 .05837 Mdot
.18986 .08785 Mdot
.18986 .11734 Mdot
.18986 .14683 Mdot
.18986 .17632 Mdot
.18986 .20581 Mdot
.18986 .2353 Mdot
.18986 .26478 Mdot
.18986 .29427 Mdot
.18986 .32376 Mdot
.18986 .35325 Mdot
.18986 .38274 Mdot
.18986 .41223 Mdot
.18986 .44171 Mdot
.18986 .4712 Mdot
.18986 .50069 Mdot
.18986 .53018 Mdot
.18986 .55967 Mdot
.18986 .58916 Mdot
.23758 .05837 Mdot
.23758 .08785 Mdot
.23758 .11734 Mdot
.23758 .14683 Mdot
.23758 .17632 Mdot
.23758 .20581 Mdot
.23758 .2353 Mdot
.23758 .26478 Mdot
.23758 .29427 Mdot
.23758 .32376 Mdot
.23758 .35325 Mdot
.23758 .38274 Mdot
.23758 .41223 Mdot
.23758 .44171 Mdot
.23758 .4712 Mdot
.23758 .50069 Mdot
.23758 .53018 Mdot
.23758 .55967 Mdot
.28529 .08785 Mdot
.28529 .11734 Mdot
.28529 .14683 Mdot
.28529 .17632 Mdot
.28529 .20581 Mdot
.28529 .2353 Mdot
.28529 .26478 Mdot
.28529 .29427 Mdot
.28529 .32376 Mdot
.28529 .35325 Mdot
.28529 .38274 Mdot
.28529 .41223 Mdot
.28529 .44171 Mdot
.28529 .4712 Mdot
.28529 .50069 Mdot
.28529 .53018 Mdot
.333 .11734 Mdot
.333 .14683 Mdot
.333 .17632 Mdot
.333 .20581 Mdot
.333 .2353 Mdot
.333 .26478 Mdot
.333 .29427 Mdot
.333 .32376 Mdot
.333 .35325 Mdot
.333 .38274 Mdot
.333 .41223 Mdot
.333 .44171 Mdot
.333 .4712 Mdot
.333 .50069 Mdot
.38072 .17632 Mdot
.38072 .20581 Mdot
.38072 .2353 Mdot
.38072 .26478 Mdot
.38072 .29427 Mdot
.38072 .32376 Mdot
.38072 .35325 Mdot
.38072 .38274 Mdot
.38072 .41223 Mdot
.38072 .44171 Mdot
.42843 .2353 Mdot
.42843 .26478 Mdot
.42843 .29427 Mdot
.42843 .32376 Mdot
.42843 .35325 Mdot
.42843 .38274 Mdot
.47614 .29427 Mdot
.47614 .32376 Mdot
.52386 .29427 Mdot
.52386 .32376 Mdot
.57157 .2353 Mdot
.57157 .26478 Mdot
.57157 .29427 Mdot
.57157 .32376 Mdot
.57157 .35325 Mdot
.57157 .38274 Mdot
.61928 .17632 Mdot
.61928 .20581 Mdot
.61928 .2353 Mdot
.61928 .26478 Mdot
.61928 .29427 Mdot
.61928 .32376 Mdot
.61928 .35325 Mdot
.61928 .38274 Mdot
.61928 .41223 Mdot
.61928 .44171 Mdot
.667 .11734 Mdot
.667 .14683 Mdot
.667 .17632 Mdot
.667 .20581 Mdot
.667 .2353 Mdot
.667 .26478 Mdot
.667 .29427 Mdot
.667 .32376 Mdot
.667 .35325 Mdot
.667 .38274 Mdot
.667 .41223 Mdot
.667 .44171 Mdot
.667 .4712 Mdot
.667 .50069 Mdot
.71471 .08785 Mdot
.71471 .11734 Mdot
.71471 .14683 Mdot
.71471 .17632 Mdot
.71471 .20581 Mdot
.71471 .2353 Mdot
.71471 .26478 Mdot
.71471 .29427 Mdot
.71471 .32376 Mdot
.71471 .35325 Mdot
.71471 .38274 Mdot
.71471 .41223 Mdot
.71471 .44171 Mdot
.71471 .4712 Mdot
.71471 .50069 Mdot
.71471 .53018 Mdot
.76242 .05837 Mdot
.76242 .08785 Mdot
.76242 .11734 Mdot
.76242 .14683 Mdot
.76242 .17632 Mdot
.76242 .20581 Mdot
.76242 .2353 Mdot
.76242 .26478 Mdot
.76242 .29427 Mdot
.76242 .32376 Mdot
.76242 .35325 Mdot
.76242 .38274 Mdot
.76242 .41223 Mdot
.76242 .44171 Mdot
.76242 .4712 Mdot
.76242 .50069 Mdot
.76242 .53018 Mdot
.76242 .55967 Mdot
.81014 .02888 Mdot
.81014 .05837 Mdot
.81014 .08785 Mdot
.81014 .11734 Mdot
.81014 .14683 Mdot
.81014 .17632 Mdot
.81014 .20581 Mdot
.81014 .2353 Mdot
.81014 .26478 Mdot
.81014 .29427 Mdot
.81014 .32376 Mdot
.81014 .35325 Mdot
.81014 .38274 Mdot
.81014 .41223 Mdot
.81014 .44171 Mdot
.81014 .4712 Mdot
.81014 .50069 Mdot
.81014 .53018 Mdot
.81014 .55967 Mdot
.81014 .58916 Mdot
.85785 .02888 Mdot
.85785 .05837 Mdot
.85785 .08785 Mdot
.85785 .11734 Mdot
.85785 .14683 Mdot
.85785 .17632 Mdot
.85785 .20581 Mdot
.85785 .2353 Mdot
.85785 .26478 Mdot
.85785 .29427 Mdot
.85785 .32376 Mdot
.85785 .35325 Mdot
.85785 .38274 Mdot
.85785 .41223 Mdot
.85785 .44171 Mdot
.85785 .4712 Mdot
.85785 .50069 Mdot
.85785 .53018 Mdot
.85785 .55967 Mdot
.85785 .58916 Mdot
.90556 .05837 Mdot
.90556 .08785 Mdot
.90556 .11734 Mdot
.90556 .14683 Mdot
.90556 .17632 Mdot
.90556 .20581 Mdot
.90556 .2353 Mdot
.90556 .26478 Mdot
.90556 .29427 Mdot
.90556 .32376 Mdot
.90556 .35325 Mdot
.90556 .38274 Mdot
.90556 .41223 Mdot
.90556 .44171 Mdot
.90556 .4712 Mdot
.90556 .50069 Mdot
.90556 .53018 Mdot
.90556 .55967 Mdot
.95328 .14683 Mdot
.95328 .17632 Mdot
.95328 .20581 Mdot
.95328 .2353 Mdot
.95328 .26478 Mdot
.95328 .29427 Mdot
.95328 .32376 Mdot
.95328 .35325 Mdot
.95328 .38274 Mdot
.95328 .41223 Mdot
.95328 .44171 Mdot
.95328 .4712 Mdot
.5 Mabswid
.5 .30902 m
.5598 .38235 L
.61866 .45108 L
.67564 .51088 L
.72986 .558 L
.78045 .58947 L
.82662 .60332 L
.86764 .59868 L
.90286 .57584 L
.93172 .53623 L
.95378 .48235 L
.96868 .41757 L
.97619 .34598 L
.97619 .27206 L
.96868 .20046 L
.95378 .13569 L
.93172 .08181 L
.90286 .0422 L
.86764 .01936 L
.82662 .01472 L
.78045 .02857 L
.72986 .06004 L
.67564 .10716 L
.61866 .16696 L
.5598 .23568 L
.5 .30902 L
.4402 .38235 L
.38134 .45108 L
.32436 .51088 L
.27014 .558 L
.21955 .58947 L
.17338 .60332 L
.13236 .59868 L
.09714 .57584 L
.06828 .53623 L
.04622 .48235 L
.03132 .41757 L
.02381 .34598 L
.02381 .27206 L
.03132 .20046 L
.04622 .13569 L
.06828 .08181 L
.09714 .0422 L
.13236 .01936 L
.17338 .01472 L
.21955 .02857 L
.27014 .06004 L
.32436 .10716 L
.38134 .16696 L
.4402 .23568 L
Mistroke
.5 .30902 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3ooona
0?ooo`00M03oool01@3ogi`0B000000000000000M;oo00<0oooo00D0_g@0000000000000000007Bo
o`1A0?ooo`050;md0000000000000000001d_ol0103oool01@3o_g@00000000000000000M;oo0580
oooo00D0_g@0000000000000000007Boo`040?ooo`050;md0000000000000000001d_ol0DP3oool0
1@2oM00000000000000007@0_ooo00<0oooo00D0okmd000000000000000007Boo`0A0?ooo`00MP3o
ool00`2oM000M;oo0?ooo`020?ooo`070;md0000M;l0oooo0?ooo`3ogi`0B00007Boo`1?0?ooo`07
0;md0000M;l0oooo0?ooo`3ogi`0B00007Boo`030?ooo`0609a8M02oool0oooo0?oo_`1d07@0_ooo
D03oool01`2oM00007Bo0?ooo`3oool0omnL04P0001d_ol00P3oool01`2oM00007Bo0?ooo`3oool0
omnL04P0001d_ol0D03oool01P3OW4P007Bo0?ooo`3oool0okmd001d_`<0oooo00H0W4Qd0;ooo`3o
ool0oono07@0M02oool@0?ooo`00MP3oool00`2oM000M;oo0?ooo`070?ooo`030?noM000M;l0oooo
05<0oooo00H0okmd001d_`3oool0oooo0?noM000M;l30?ooo`030=nLB018W=l0oooo05@0oooo00<0
okmd001d_`3oool01P3oool00`3o_g@007Bo0?ooo`1>0?ooo`030?oo_`1d0000M;oo0080oooo00H0
oono07A8W03oool0oooo0?noM000M;l30?ooo`030=nLB018W=l0oooo00l0oooo001b0?ooo`030=nL
B0000000000000<0000000<0M;oo0?ooo`3oool01@3oool00`3o_g@007Bo0?ooo`1C0?ooo`060?no
M000M;l0oooo0?ooo`3OW4P0B9cO0`3oool00`3o_g@007Bo0?ooo`1D0?ooo`030?noM000M;l0oooo
00H0oooo00<0okmd001d_`3oool0CP3oool01`3o_g@004Q807@0M02oool0oooo0=nLB000M;l00P3o
ool00`3OW4P0B9cO0?ooo`020?ooo`030?noM000M;l0oooo00l0oooo001b0?ooo`060?noM00007@0
_ooo0?ooo`2oM000M;oo203oool00`2LB000M;oo0?ooo`1C0?ooo`0609a8001d_ol0oooo0?ooo`3O
W4P0B9cO0`3oool00`3o_g@007Bo0?ooo`1D0?ooo`0309a8001d_ol0oooo00H0oooo00<0W4P007Bo
o`3oool0CP3oool00`3ogi`0B4RL0?noM002000000060018W03oool0oooo0?ooo`3OW4P0B9cO0`3o
ool00`3o_g@007Bo0?ooo`0?0?ooo`00L`3oool01@3OW4P007Bo0?ooo`2oM000M;oo00<0oooo00<0
oono07@0000000000P0000000`18W=l0oooo0?ooo`1>0?ooo`030?oo_`1d000000000080000000H0
B9cO0?ooo`3oool0oooo0=nLB018W=l30?ooo`030?noM000M;l0oooo04l0oooo00<0oono07@00000
00000P0000001P18W=l0oooo0?ooo`3oool0oono07@000<0000000<0B9cO0?ooo`3oool0CP3oool0
0`3ookl0M01d0;ooo`060?ooo`030=nLB018W=l0oooo0080oooo00<0okmd001d_`3oool03`3oool0
07@0oooo00@0W4P004RLg`2oM000M;oo0`3oool00`3ookl0M01d0;ooo`1C0?ooo`030?oo_`1d07@0
_ooo00H0oooo00<0okmd04RLg`3oool00P3oool00`3OW4P0B9cO0?ooo`1?0?ooo`030?oo_`1d07@0
_ooo00H0oooo00<0oono07@0M02oool0E03oool00`2oM000B9cO0?ooo`050?ooo`030?noM018W=l0
oooo0080oooo00<0gia804RLg`3oool03`3oool007@0oooo00@0omnL04P0B018B000M;oo0`3oool0
0`3ookl0M01d0;ooo`1C0?ooo`030?oo_`1d07@0_ooo00L0oooo00H0W4P007Boo`3oool0omnL04P0
M02ooom@0?ooo`030?oo_`1d07@0_ooo00H0oooo00<0oono07@0M02oool0E03oool00`3ookl0M000
001d_`060?ooo`0609a8001d_ol0oooo0?oOW01807@0_ooo403oool007D0oooo00<0okmd0000001d
_ol00`3oool00`3ookl0M00000000002000000030000M02oool0oooo04h0oooo00<0oono07@00000
00000P0000000`0007@0_ooo0?ooo`020?ooo`050?oo_`1d000000000000B02Lgol0D@3oool00`3o
okl0M00000000002000000060000M02oool0oooo0?ooo`3ookl0M0000`0000000`0007@0_ooo0?oo
o`1A0?ooo`040;md0000000000000018W0<0oooo00D0oono07@000000000001809cOo`0A0?ooo`00
o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00
o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`006P3ooooo000009<00000103oool001X0
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01D0
oooo000J0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0E0?ooo`006P3oool00`000000oooo0?ooo`1M0?ooo`030000003oool0oooo05`0oooo00<0
00000?ooo`3oool0G@3oool00`000000oooo0?ooo`1L0?ooo`030000003oool0oooo01D0oooo000J
0?ooo`030000003oool0oooo0?l0ooooU03oool001X0oooo00<000000?ooo`3oool0o`3ooonD0?oo
o`006P3oool00`000000oooo0?ooo`100?ooo`D00000o`3oool30?ooo`H00000AP3oool001X0oooo
00<000000?ooo`3oool0>03oool8000000D0oooo0`00003l0?ooo`<000001P3oool8000003h0oooo
000J0?ooo`030000003oool0oooo03<0oooo1@00000@0?ooo`<00000mP3oool300000140oooo1@00
000i0?ooo`006P3oool00`000000oooo0?ooo`0a0?ooo`800000603oool400000>l0oooo0`00000I
0?ooo`800000=`3oool001X0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`0L0?oo
o`<00000j@3oool3000001h0oooo00<000000?ooo`3oool0=03oool001X0oooo00<000000?ooo`3o
ool0;P3oool2000000L0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`050?ooo`<0
0000h`3oool3000000L0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`050?ooo`80
0000=03oool001X0oooo0`00000]0?ooo`030000003oool0oooo00H0oooo0`00000@0?ooo`<00000
2@3oool200000=l0oooo0P0000090?ooo`<00000403oool3000000P0oooo00<000000?ooo`3oool0
<@3oool001X0oooo00<000000?ooo`3oool0:`3oool2000000X0oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0:0?ooo`800000f`3oool2000000`0oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`080?ooo`800000<@3oool001X0oooo00<000000?ooo`3oool0:@3oool2
000002h0oooo0P00003G0?ooo`800000;P3oool2000002l0oooo000J0?ooo`030000003oool0oooo
02P0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`3C0?ooo`030000003oool0oooo
0300oooo00<000000?ooo`3oool0;03oool001X0oooo00<000000?ooo`3oool09P3oool2000003@0
oooo0P00003A0?ooo`800000=03oool2000002`0oooo000J0?ooo`030000003oool0oooo02D0oooo
00<000000?ooo`3oool0=P3oool200000<d0oooo0P00000h0?ooo`030000003oool0oooo02T0oooo
000J0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?oo
o`390?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool0:03oool001X0oooo00<00000
0?ooo`3oool08`3oool00`000000oooo0?ooo`0k0?ooo`800000a`3oool2000003`0oooo00<00000
0?ooo`3oool0:03oool001X0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0m0?oo
o`800000``3oool2000003l0oooo00<000000?ooo`3oool09`3oool001X0oooo00<000000?ooo`3o
ool08P3oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo0;l0oooo00<000000?ooo`3o
ool0@03oool00`000000oooo0?ooo`0V0?ooo`006P3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo0480oooo0P00002m0?ooo`800000@`3oool00`000000oooo0?ooo`0V0?ooo`006P3o
ool00`000000oooo0?ooo`0P0?ooo`040000003oool0oooo000001<0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00P0oooo0P00002i0?ooo`800000
2P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
403oool010000000oooo0?ooo`00000W0?ooo`006P3oool300000200oooo00<000000?ooo`000000
0P00000A0?ooo`<00000403oool300000100oooo0`00000;0?ooo`030000003oool0oooo0;D0oooo
00<000000?ooo`3oool02@3oool300000100oooo0`00000@0?ooo`<00000403oool300000080oooo
00<000000?ooo`3oool0903oool001X0oooo00<000000?ooo`3oool07`3oool01@000000oooo0?oo
o`3oool0000001<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool0/`3oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`050000
003oool0oooo0?ooo`0000009P3oool001X0oooo00<000000?ooo`3oool07P3oool00`000000oooo
0?ooo`1;0?ooo`030000003oool0oooo0;40oooo00<000000?ooo`3oool0BP3oool00`000000oooo
0?ooo`0S0?ooo`006P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo04d0oooo00<0
00000?ooo`3oool0[P3oool2000004h0oooo00<000000?ooo`3oool08P3oool001X0oooo00<00000
0?ooo`3oool07@3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo0:`0oooo00<00000
0?ooo`3oool0C`3oool00`000000oooo0?ooo`0Q0?ooo`006P3oool00`000000oooo0?ooo`0L0?oo
o`030000003oool0oooo0500oooo0P00002[0?ooo`030000003oool0oooo0500oooo00<000000?oo
o`3oool08@3oool001X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`1C0?ooo`03
0000003oool0oooo0:L0oooo00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`0P0?ooo`00
6P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo05D0oooo00<000000?ooo`3oool0
Y@3oool00`000000oooo0?ooo`1D0?ooo`030000003oool0oooo01l0oooo000J0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool0EP3oool00`000000oooo0?ooo`2S0?ooo`030000003o
ool0oooo05D0oooo00<000000?ooo`3oool07`3oool001X0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`1H0?ooo`030000003oool0oooo0:40oooo00<000000?ooo`3oool0E`3oool0
0`000000oooo0?ooo`0N0?ooo`006P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
05T0oooo00<000000?ooo`3oool0WP3oool2000005X0oooo00<000000?ooo`3oool07P3oool001X0
oooo0`00000H0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo09`0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool07@3oool001X0oooo
00<000000?ooo`3oool0603oool00`000000oooo0?ooo`070?ooo`<000004@3oool300000100oooo
0`00000@0?ooo`<00000403oool3000000D0oooo00<000000?ooo`3oool0VP3oool00`000000oooo
0?ooo`030?ooo`<000004@3oool300000100oooo0`00000@0?ooo`<00000403oool3000000T0oooo
00<000000?ooo`3oool07@3oool001X0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?oo
o`090?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`2H0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`090?ooo`030000003oool0oooo01`0oooo000J0?ooo`030000003oool0oooo01L0oooo00<00000
0?ooo`3oool0G`3oool2000009L0oooo00<000000?ooo`3oool0G`3oool00`000000oooo0?ooo`0L
0?ooo`006P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0640oooo00<000000?oo
o`3oool0T`3oool00`000000oooo0?ooo`1P0?ooo`030000003oool0oooo01`0oooo000J0?ooo`03
0000003oool0oooo01H0oooo00<000000?ooo`3oool0H`3oool00`000000oooo0?ooo`2@0?ooo`80
0000I03oool00`000000oooo0?ooo`0K0?ooo`006P3oool00`000000oooo0?ooo`0F0?ooo`030000
003oool0oooo06@0oooo00<000000?ooo`3oool0SP3oool00`000000oooo0?ooo`1T0?ooo`030000
003oool0oooo01/0oooo000J0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0IP3o
ool00`000000oooo0?ooo`2<0?ooo`030000003oool0oooo06H0oooo00<000000?ooo`3oool06P3o
ool001X0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`1W0?ooo`030000003oool0
oooo08X0oooo00<000000?ooo`3oool0I`3oool00`000000oooo0?ooo`0J0?ooo`006P3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo06T0oooo00<000000?ooo`3oool0R03oool00`00
0000oooo0?ooo`1X0?ooo`030000003oool0oooo01X0oooo000J0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool0JP3oool00`000000oooo0?ooo`260?ooo`030000003oool0oooo06X0
oooo00<000000?ooo`3oool06@3oool001X0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03`3oool2000008@0oooo
0P00000A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`0I0?ooo`006P3oool3000001<0oooo00<000000?ooo`3oool0303oool300000140oooo0`00
000@0?ooo`<00000403oool300000100oooo0`00000@0?ooo`<00000PP3oool300000100oooo0`00
000A0?ooo`<00000403oool300000100oooo0`00000@0?ooo`<000003P3oool00`000000oooo0?oo
o`0H0?ooo`006P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`800000PP3oool200000180oooo00<00000
0?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01P0oooo000J0?oo
o`030000003oool0oooo0180oooo00<000000?ooo`3oool0L03oool00`000000oooo0?ooo`1n0?oo
o`030000003oool0oooo0700oooo00<000000?ooo`3oool05`3oool001X0oooo00<000000?ooo`3o
ool04P3oool00`000000oooo0?ooo`1a0?ooo`030000003oool0oooo07`0oooo00<000000?ooo`3o
ool0L@3oool00`000000oooo0?ooo`0G0?ooo`006P3oool00`000000oooo0?ooo`0B0?ooo`030000
003oool0oooo0780oooo00<000000?ooo`3oool0NP3oool00`000000oooo0?ooo`1b0?ooo`030000
003oool0oooo01L0oooo000J0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0M03o
ool00`000000oooo0?ooo`1h0?ooo`030000003oool0oooo07@0oooo00<000000?ooo`3oool05P3o
ool001X0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`1e0?ooo`030000003oool0
oooo07H0oooo00<000000?ooo`3oool0M@3oool00`000000oooo0?ooo`0F0?ooo`006P3oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool0M03oool00`00
0000oooo0?ooo`1g0?ooo`030000003oool0oooo01D0oooo000J0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0M`3oool00`000000oooo0?ooo`1c0?ooo`030000003oool0oooo07P0
oooo00<000000?ooo`3oool05@3oool000H0oooo00D0omnL04P000000000000007Boo`030?ooo`05
0;md0000000000000000001d_ol01`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
07P0oooo00<000000?ooo`3oool0LP3oool00`000000oooo0?ooo`1h0?ooo`030000003oool0oooo
01D0oooo00080?ooo`030;md001d_ol0oooo0080oooo00L0_g@0001d_`3oool0oooo0?oOW0180000
M;oo00H0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`1i0?ooo`030000003oool0
oooo0700oooo00<000000?ooo`3oool0N@3oool00`000000oooo0?ooo`0E0?ooo`00203oool00`2o
M000M;oo0?ooo`070?ooo`030?noM000M;l0oooo00D0oooo00<000000?ooo`3oool03`3oool20000
0180oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool0KP3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`8000005P3oool000@0
oooo00<0gia80000000000000`0000000`1d_ol0oooo0?ooo`050?ooo`030?noM000M;l0oooo00D0
oooo1000000>0?ooo`<00000403oool300000140oooo0`00000@0?ooo`<00000403oool300000100
oooo0`00000@0?ooo`<000002@3oool00`000000oooo0?ooo`1/0?ooo`030000003oool0oooo00L0
oooo0`00000@0?ooo`<000004@3oool300000100oooo0`00000@0?ooo`<00000403oool300000100
oooo0`00000F0?ooo`00103oool01P3o_g@0001d0;ooo`3oool0_g@007Boo`P0oooo00<0W4P007Bo
o`3oool01@3oool00`000000oooo0?ooo`0?0?ooo`8000004P3oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`1Z0?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0100oooo0P00000F0?ooo`001@3oool01@3OW4P007Bo0?ooo`2oM000M;oo
00<0oooo00<0oono07@0000000000P0000000`18W=l0oooo0?ooo`050?ooo`030000003oool0oooo
00l0oooo00<000000?ooo`3oool0OP3oool00`000000oooo0?ooo`1X0?ooo`030000003oool0oooo
07h0oooo00<000000?ooo`3oool0503oool000H0oooo00@0W4P004RLg`2oM000M;oo0`3oool00`3o
okl0M01d0;ooo`0:0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0P03oool00`00
0000oooo0?ooo`1V0?ooo`030000003oool0oooo0800oooo00<000000?ooo`3oool04`3oool000H0
oooo00@0omnL04P0B018B000M;oo0`3oool00`3ookl0M01d0;ooo`0:0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool0P@3oool00`000000oooo0?ooo`1T0?ooo`030000003oool0oooo
0840oooo00<000000?ooo`3oool04`3oool000L0oooo00<0okmd0000001d_ol00`3oool00`3ookl0
M00000000002000000030000M02oool0oooo00D0oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`220?ooo`030000003oool0oooo0680oooo00<000000?ooo`3oool0PP3oool00`000000
oooo0?ooo`0C0?ooo`006P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo08<0oooo
00<000000?ooo`3oool0H03oool00`000000oooo0?ooo`230?ooo`030000003oool0oooo01<0oooo
000J0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0Q@3oool00`000000oooo0?oo
o`1N0?ooo`030000003oool0oooo08D0oooo00<000000?ooo`3oool04P3oool001X0oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`260?ooo`030000003oool0oooo05`0oooo00<00000
0?ooo`3oool0QP3oool00`000000oooo0?ooo`0B0?ooo`006P3oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo08L0oooo00<000000?ooo`3oool0FP3oool00`000000oooo0?ooo`270?oo
o`030000003oool0oooo0180oooo000J0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3o
ool0R03oool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo08P0oooo00<000000?ooo`3o
ool04P3oool001X0oooo00<000000?ooo`3oool03@3oool010000000oooo0?ooo`00000B0?ooo`03
0000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`04
0000003oool0oooo000005T0oooo00<000000?ooo`0000004P3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool010000000oooo0?ooo`00000D
0?ooo`006P3oool3000000`0oooo00<000000?ooo`3oool00`00000@0?ooo`<000004@3oool30000
0100oooo0`00000@0?ooo`<00000403oool300000100oooo0`00000@0?ooo`<0000000<0oooo0000
003oool0E`3oool00`000000oooo0000000200000100oooo0`00000@0?ooo`<000004@3oool30000
0100oooo0`00000@0?ooo`<00000403oool300000100oooo0`0000020?ooo`030000003oool0oooo
0140oooo000J0?ooo`030000003oool0oooo00`0oooo00D000000?ooo`3oool0oooo0000000B0?oo
o`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?oo
o`050000003oool0oooo0?ooo`000000EP3oool01@000000oooo0?ooo`3oool000000180oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00D0
00000?ooo`3oool0oooo0000000C0?ooo`006P3oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo08`0oooo00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`2<0?ooo`030000003o
ool0oooo0140oooo000J0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0S@3oool0
0`000000oooo0?ooo`1@0?ooo`030000003oool0oooo08d0oooo00<000000?ooo`3oool04@3oool0
01X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`2?0?ooo`030000003oool0oooo
04h0oooo00<000000?ooo`3oool0S`3oool00`000000oooo0?ooo`0@0?ooo`006P3oool00`000000
oooo0?ooo`0;0?ooo`030000003oool0oooo0900oooo00<000000?ooo`3oool0C@3oool00`000000
oooo0?ooo`2?0?ooo`030000003oool0oooo0100oooo000J0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0T@3oool00`000000oooo0?ooo`1;0?ooo`030000003oool0oooo0900oooo
00<000000?ooo`3oool0403oool001X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`2A0?ooo`030000003oool0oooo04X0oooo00<000000?ooo`3oool0T@3oool00`000000oooo0?oo
o`0@0?ooo`006P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo09<0oooo00<00000
0?ooo`3oool0B03oool00`000000oooo0?ooo`2C0?ooo`030000003oool0oooo00l0oooo000J0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool0U03oool00`000000oooo0?ooo`160?oo
o`030000003oool0oooo09@0oooo00<000000?ooo`3oool03`3oool001X0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`2E0?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3o
ool0U@3oool00`000000oooo0?ooo`0?0?ooo`006P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo04<0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo0100oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool03`3oool001X0oooo0`00000:0?ooo`030000003o
ool0oooo0080oooo0`00000@0?ooo`<000004@3oool300000100oooo0`00000@0?ooo`<00000403o
ool300000100oooo0`00000@0?ooo`<00000303oool00`000000oooo0?ooo`110?ooo`030000003o
ool0oooo00T0oooo0`00000@0?ooo`<00000403oool300000140oooo0`00000@0?ooo`<00000403o
ool300000100oooo0`00000@0?ooo`<00000103oool00`000000oooo0?ooo`0?0?ooo`006P3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool03`3oool0
01X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`2H0?ooo`030000003oool0oooo
03h0oooo00<000000?ooo`3oool0V03oool00`000000oooo0?ooo`0?0?ooo`006P3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo09X0oooo00<000000?ooo`3oool0?03oool00`000000
oooo0?ooo`2J0?ooo`030000003oool0oooo00h0oooo000J0?ooo`030000003oool0oooo00T0oooo
00<000000?ooo`3oool0V`3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo09X0oooo
00<000000?ooo`3oool03P3oool001X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?oo
o`2L0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool0V`3oool00`000000oooo0?oo
o`0>0?ooo`006P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo09d0oooo00<00000
0?ooo`3oool0=`3oool00`000000oooo0?ooo`2L0?ooo`030000003oool0oooo00h0oooo000J0?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool0WP3oool00`000000oooo0?ooo`0e0?oo
o`030000003oool0oooo09d0oooo00<000000?ooo`3oool03P3oool001X0oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`2O0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3o
ool0WP3oool00`000000oooo0?ooo`0>0?ooo`006P3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo09l0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`2O0?ooo`030000
003oool0oooo00h0oooo000J0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0X03o
ool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo09l0oooo00<000000?ooo`3oool03P3o
ool001X0oooo0`0000090?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`050000003oool0oooo0?ooo`000000<@3oool010000000oooo0?oo
o`00000B0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0>0?ooo`006P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00@0oooo0`00
000@0?ooo`<000004@3oool300000100oooo0`00000@0?ooo`<00000403oool300000100oooo0`00
000@0?ooo`<00000403oool3000000<0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?oo
o`0300000100oooo0`00000@0?ooo`<00000403oool300000140oooo0`00000@0?ooo`<00000403o
ool300000100oooo0`00000@0?ooo`<000001@3oool00`000000oooo0?ooo`0>0?ooo`006P3oool0
0`000000oooo0?ooo`080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0:`3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00d0oooo000J0?ooo`030000003oool0oooo
00P0oooo00<000000?ooo`3oool0Y@3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo
0:@0oooo00<000000?ooo`3oool03@3oool001X0oooo00<000000?ooo`3oool0203oool00`000000
oooo0?ooo`2U0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0Y03oool00`000000
oooo0?ooo`0=0?ooo`006P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0:H0oooo
00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`2U0?ooo`030000003oool0oooo00d0oooo
000J0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0Y`3oool00`000000oooo0?oo
o`0U0?ooo`030000003oool0oooo0:H0oooo00<000000?ooo`3oool03@3oool001X0oooo00<00000
0?ooo`3oool0203oool00`000000oooo0?ooo`2X0?ooo`030000003oool0oooo02<0oooo00<00000
0?ooo`3oool0Y`3oool00`000000oooo0?ooo`0=0?ooo`006P3oool00`000000oooo0?ooo`080?oo
o`030000003oool0oooo0:T0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`2X0?oo
o`030000003oool0oooo00d0oooo000J0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3o
ool0ZP3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0:T0oooo00<000000?ooo`3o
ool03@3oool001X0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`2Z0?ooo`030000
003oool0oooo01l0oooo00<000000?ooo`3oool0Z@3oool00`000000oooo0?ooo`0=0?ooo`006P3o
ool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool07@3o
ool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00`0oooo000J0?ooo`<000001`3oool0
0`000000oooo0?ooo`050?ooo`<00000403oool300000140oooo0`00000@0?ooo`<00000403oool3
00000100oooo0`00000@0?ooo`<00000403oool300000100oooo0`00000<0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool02@3oool300000100oooo0`00000@0?ooo`<00000403oool3
00000140oooo0`00000@0?ooo`<00000403oool300000100oooo0`00000@0?ooo`<000001`3oool0
0`000000oooo0?ooo`0<0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
00`0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
00`0oooo000J0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0[`3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo0:h0oooo00<000000?ooo`3oool0303oool001X0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`2`0?ooo`030000003oool0oooo01D0oooo
00<000000?ooo`3oool0[`3oool00`000000oooo0?ooo`0<0?ooo`006P3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo0;00oooo00<000000?ooo`3oool05@3oool00`000000oooo0?oo
o`2_0?ooo`030000003oool0oooo00`0oooo000J0?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool0/@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0;00oooo00<00000
0?ooo`3oool0303oool001X0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`2b0?oo
o`030000003oool0oooo0140oooo00<000000?ooo`3oool0/@3oool00`000000oooo0?ooo`0<0?oo
o`006P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0;<0oooo00<000000?ooo`3o
ool03`3oool00`000000oooo0?ooo`2b0?ooo`030000003oool0oooo00`0oooo000J0?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool0]03oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo0;<0oooo00<000000?ooo`3oool0303oool000D0oooo00D0_g@00000000000000000
07Boo`040?ooo`050?noM000000000000000001d_ol01`3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo0;D0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`2d0?ooo`03
0000003oool0oooo00`0oooo00040?ooo`070;md0000M;l0oooo0?ooo`3ogi`0B00007Boo`030?oo
o`0609a8M02oool0oooo0?oo_`1d07@0_ooo1P3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo0;D0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`2d0?ooo`030000003o
ool0oooo00`0oooo00090?ooo`060?noM000M;l0oooo0?ooo`3o_g@007Bo0`3oool00`3OW4P0B9cO
0?ooo`050?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool01@000000oooo
0?ooo`3oool0000000/0oooo00@000000?ooo`3oool000004P3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0<0?oo
o`002@3oool01P3o_g@007Bo0?ooo`3oool0gia804RLg`<0oooo00<0okmd001d_`3oool01@3oool4
000000H0oooo00<000000?ooo`3oool01@3oool300000100oooo0`00000A0?ooo`<00000403oool3
00000100oooo0`00000@0?ooo`<00000403oool300000100oooo0`00000@0?ooo`<00000403oool3
000000<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0300000100oooo0`00000@
0?ooo`<00000403oool300000100oooo0`00000A0?ooo`<00000403oool300000100oooo0`00000@
0?ooo`<00000403oool3000000L0oooo00<000000?ooo`3oool0303oool000T0oooo00H0W4P007Bo
o`3oool0oooo0=nLB018W=l30?ooo`030?noM000M;l0oooo00D0oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0303o
ool000@0oooo00<0oono07@0000000000P0000001P18W=l0oooo0?ooo`3oool0gia804RLg`<0oooo
00<0okmd001d_`3oool01@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0;T0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2h0?ooo`030000003oool0oooo00`0oooo
00040?ooo`030?oo_`1d07@0_ooo00H0oooo00<0okmd04RLg`3oool00P3oool00`3OW4P0B9cO0?oo
o`050?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0^P3oool01@000000oooo0?oo
o`3oool000000;/0oooo00<000000?ooo`3oool0303oool000@0oooo00<0oono07@0M02oool01`3o
ool01P2LB000M;oo0?ooo`3ogi`0B01d0;ooo`H0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`2j0?ooo`050000003oool0oooo0?ooo`000000^`3oool00`000000oooo0?ooo`0<0?oo
o`00103oool00`3ookl0M00000000002000000030000M02oool0oooo0080oooo00D0oono07@00000
0000001809cOo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0^`3oool00`00
0000oooo0000002l0?ooo`030000003oool0oooo00`0oooo000J0?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool0_03oool00`000000oooo0?ooo`2k0?ooo`030000003oool0oooo00`0
oooo000J0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0^`3oool00`000000oooo
0000002l0?ooo`030000003oool0oooo00`0oooo000J0?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool0^P3oool01@000000oooo0?ooo`3oool000000;/0oooo00<000000?ooo`3oool0
303oool001X0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`2j0?ooo`050000003o
ool0oooo0?ooo`000000^`3oool00`000000oooo0?ooo`0<0?ooo`006P3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo0;T0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`2h0?ooo`030000003oool0oooo00`0oooo000J0?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0140oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0140oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00`0oooo000J0?oo
o`<000001`3oool00`000000oooo0?ooo`050?ooo`<00000403oool300000140oooo0`00000@0?oo
o`<00000403oool300000100oooo0`00000@0?ooo`<00000403oool300000100oooo0`00000@0?oo
o`<000000`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00<00000403oool30000
0100oooo0`00000@0?ooo`<00000403oool300000140oooo0`00000@0?ooo`<00000403oool30000
0100oooo0`00000@0?ooo`<000001`3oool00`000000oooo0?ooo`0<0?ooo`006P3oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00D000000?ooo`3oool0oooo0000000;0?oo
o`040000003oool0oooo00000180oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0303oool001X0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`2e0?ooo`030000003oool0oooo00/0oooo00<000000?oo
o`3oool0]03oool00`000000oooo0?ooo`0<0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo0;D0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`2d0?ooo`03
0000003oool0oooo00`0oooo000J0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
]03oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0;<0oooo00<000000?ooo`3oool0
303oool001X0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`2c0?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool0/P3oool00`000000oooo0?ooo`0<0?ooo`006P3oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo0;80oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`2a0?ooo`030000003oool0oooo00`0oooo000J0?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`3oool0/@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo
0;00oooo00<000000?ooo`3oool0303oool001X0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`2`0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0[`3oool00`000000
oooo0?ooo`0<0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0;00oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`2_0?ooo`030000003oool0oooo00`0oooo
000J0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0[`3oool00`000000oooo0?oo
o`0G0?ooo`030000003oool0oooo0:h0oooo00<000000?ooo`3oool0303oool001X0oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool02`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`0<0?ooo`006P3oool3000000L0oooo00<000000?oo
o`3oool01@3oool300000100oooo0`00000A0?ooo`<00000403oool300000100oooo0`00000@0?oo
o`<00000403oool300000100oooo0`00000@0?ooo`<00000303oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo00T0oooo0`00000@0?ooo`<00000403oool300000100oooo0`00000A0?oo
o`<00000403oool300000100oooo0`00000@0?ooo`<00000403oool3000000L0oooo00<000000?oo
o`3oool0303oool001X0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo01d0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0<0?ooo`00
6P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0:X0oooo00<000000?ooo`3oool0
7`3oool00`000000oooo0?ooo`2Y0?ooo`030000003oool0oooo00d0oooo000J0?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool0ZP3oool00`000000oooo0?ooo`0O0?ooo`030000003o
ool0oooo0:T0oooo00<000000?ooo`3oool03@3oool001X0oooo00<000000?ooo`3oool0203oool0
0`000000oooo0?ooo`2Y0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0Z03oool0
0`000000oooo0?ooo`0=0?ooo`006P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
0:P0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`2W0?ooo`030000003oool0oooo
00d0oooo000J0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0Y`3oool00`000000
oooo0?ooo`0U0?ooo`030000003oool0oooo0:H0oooo00<000000?ooo`3oool03@3oool001X0oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`2V0?ooo`030000003oool0oooo02L0oooo
00<000000?ooo`3oool0Y@3oool00`000000oooo0?ooo`0=0?ooo`006P3oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo0:D0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?oo
o`2T0?ooo`030000003oool0oooo00d0oooo000J0?ooo`030000003oool0oooo00P0oooo00<00000
0?ooo`3oool0Y@3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo0:@0oooo00<00000
0?ooo`3oool03@3oool001X0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo02/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0=0?oo
o`006P3oool3000000P0oooo00<000000?ooo`3oool0103oool300000100oooo0`00000A0?ooo`<0
0000403oool300000100oooo0`00000@0?ooo`<00000403oool300000100oooo0`00000@0?ooo`<0
00000`3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo00<00000403oool300000100
oooo0`00000@0?ooo`<000004@3oool300000100oooo0`00000@0?ooo`<00000403oool300000100
oooo0`0000050?ooo`030000003oool0oooo00h0oooo000J0?ooo`030000003oool0oooo00T0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0140oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo
00D000000?ooo`3oool0oooo0000000a0?ooo`040000003oool0oooo00000180oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0140oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00h0oooo000J0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0X03oool00`000000oooo0?ooo`0a0?ooo`03
0000003oool0oooo09l0oooo00<000000?ooo`3oool03P3oool001X0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`2O0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool0
W`3oool00`000000oooo0?ooo`0>0?ooo`006P3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo09l0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`2N0?ooo`030000003o
ool0oooo00h0oooo000J0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0WP3oool0
0`000000oooo0?ooo`0e0?ooo`030000003oool0oooo09d0oooo00<000000?ooo`3oool03P3oool0
01X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`2M0?ooo`030000003oool0oooo
03L0oooo00<000000?ooo`3oool0W03oool00`000000oooo0?ooo`0>0?ooo`006P3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo09`0oooo00<000000?ooo`3oool0>@3oool00`000000
oooo0?ooo`2K0?ooo`030000003oool0oooo00h0oooo000J0?ooo`030000003oool0oooo00T0oooo
00<000000?ooo`3oool0V`3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo09X0oooo
00<000000?ooo`3oool03P3oool001X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?oo
o`2J0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0VP3oool00`000000oooo0?oo
o`0>0?ooo`006P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo09P0oooo00<00000
0?ooo`3oool0?P3oool00`000000oooo0?ooo`2H0?ooo`030000003oool0oooo00l0oooo000J0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0?0?oo
o`006P3oool3000000X0oooo00<000000?ooo`3oool00P3oool300000100oooo0`00000A0?ooo`<0
0000403oool300000100oooo0`00000@0?ooo`<00000403oool300000100oooo0`00000<0?ooo`03
0000003oool0oooo0440oooo00<000000?ooo`3oool02@3oool300000100oooo0`00000@0?ooo`<0
00004@3oool300000100oooo0`00000@0?ooo`<00000403oool300000100oooo0`0000040?ooo`03
0000003oool0oooo00l0oooo000J0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
@`3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`0?0?ooo`006P3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo09D0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`2E0?ooo`030000003o
ool0oooo00l0oooo000J0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0U03oool0
0`000000oooo0?ooo`160?ooo`030000003oool0oooo09@0oooo00<000000?ooo`3oool03`3oool0
01X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`2C0?ooo`030000003oool0oooo
04P0oooo00<000000?ooo`3oool0T`3oool00`000000oooo0?ooo`0?0?ooo`006P3oool00`000000
oooo0?ooo`0;0?ooo`030000003oool0oooo0940oooo00<000000?ooo`3oool0BP3oool00`000000
oooo0?ooo`2A0?ooo`030000003oool0oooo0100oooo000J0?ooo`030000003oool0oooo00/0oooo
00<000000?ooo`3oool0T@3oool00`000000oooo0?ooo`1;0?ooo`030000003oool0oooo0900oooo
00<000000?ooo`3oool0403oool001X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`2@0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0S`3oool00`000000oooo0?oo
o`0@0?ooo`001@3oool01@2oM0000000000000000000M;oo00@0oooo00D0_g@00000000000000000
07Boo`070?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0S`3oool00`000000oooo
0?ooo`1>0?ooo`030000003oool0oooo08l0oooo00<000000?ooo`3oool0403oool000@0oooo00L0
_g@0001d_`3oool0oooo0?oOW0180000M;oo0080oooo00L0_g@0001d_`3oool0oooo0?oOW0180000
M;oo00H0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`2=0?ooo`030000003oool0
oooo0500oooo00<000000?ooo`3oool0S@3oool00`000000oooo0?ooo`0A0?ooo`002@3oool00`3o
_g@007Bo0?ooo`060?ooo`030?noM000M;l0oooo00D0oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`2<0?ooo`030000003oool0oooo0580oooo00<000000?ooo`3oool0S03oool00`00
0000oooo0?ooo`0A0?ooo`002@3oool00`3o_g@007Bo0?ooo`060?ooo`030?noM000M;l0oooo00D0
oooo00<000000?ooo`3oool0303oool01@000000oooo0?ooo`3oool000000180oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00D000000?oo
o`3oool0oooo0000001F0?ooo`050000003oool0oooo0?ooo`0000004P3oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool01@000000oooo0?oo
o`3oool0000001<0oooo00090?ooo`0309a8001d_ol0oooo00H0oooo00<0W4P007Boo`3oool01@3o
ool4000000/0oooo00<000000?ooo`3oool00`00000@0?ooo`<000004@3oool300000100oooo0`00
000@0?ooo`<00000403oool300000100oooo0`00000@0?ooo`<0000000<0oooo0000003oool0E`3o
ool00`000000oooo0000000200000100oooo0`00000@0?ooo`<000004@3oool300000100oooo0`00
000@0?ooo`<00000403oool300000100oooo0`0000020?ooo`030000003oool0oooo0140oooo0004
0?ooo`030?oo_`1d000000000080000000H0B9cO0?ooo`3oool0oooo0?oo_`1d00030000000304RL
g`3oool0oooo00D0oooo00<000000?ooo`3oool03@3oool010000000oooo0?ooo`00000B0?ooo`03
0000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`04
0000003oool0oooo000005T0oooo00<000000?ooo`0000004P3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool010000000oooo0?ooo`00000D
0?ooo`00103oool00`3ookl0M01d0;ooo`060?ooo`030?oo_`1d07@0_ooo00X0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`280?ooo`030000003oool0oooo05P0oooo00<000000?oo
o`3oool0R03oool00`000000oooo0?ooo`0B0?ooo`00103oool00`3ookl0M01d0;ooo`060?ooo`03
0?oo_`1d07@0_ooo00X0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`270?ooo`03
0000003oool0oooo05X0oooo00<000000?ooo`3oool0Q`3oool00`000000oooo0?ooo`0B0?ooo`00
103oool00`3ookl0M00000000002000000060000M02oool0oooo0?ooo`3ookl0M0000`0000000`00
07@0_ooo0?ooo`050?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0QP3oool00`00
0000oooo0?ooo`1L0?ooo`030000003oool0oooo08H0oooo00<000000?ooo`3oool04P3oool001X0
oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`250?ooo`030000003oool0oooo05h0
oooo00<000000?ooo`3oool0Q@3oool00`000000oooo0?ooo`0B0?ooo`006P3oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo08<0oooo00<000000?ooo`3oool0H03oool00`000000oooo
0?ooo`230?ooo`030000003oool0oooo01<0oooo000J0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool0PP3oool00`000000oooo0?ooo`1R0?ooo`030000003oool0oooo0880oooo00<0
00000?ooo`3oool04`3oool001X0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`21
0?ooo`030000003oool0oooo06@0oooo00<000000?ooo`3oool0P@3oool00`000000oooo0?ooo`0C
0?ooo`006P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0800oooo00<000000?oo
o`3oool0IP3oool00`000000oooo0?ooo`200?ooo`030000003oool0oooo01<0oooo000J0?ooo`03
0000003oool0oooo00l0oooo00<000000?ooo`3oool0OP3oool00`000000oooo0?ooo`1X0?ooo`03
0000003oool0oooo07h0oooo00<000000?ooo`3oool0503oool001X0oooo00<000000?ooo`3oool0
3`3oool200000180oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool0JP3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo0100oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`800000
5P3oool001X0oooo0`00000?0?ooo`<00000403oool300000140oooo0`00000@0?ooo`<00000403o
ool300000100oooo0`00000@0?ooo`<000002@3oool00`000000oooo0?ooo`1/0?ooo`030000003o
ool0oooo00L0oooo0`00000@0?ooo`<000004@3oool300000100oooo0`00000@0?ooo`<00000403o
ool300000100oooo0`00000F0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`8000004P3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool01`3oool0
0`000000oooo0?ooo`1^0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo0P00000F0?ooo`006P3oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo07T0oooo00<000000?ooo`3oool0L03oool00`00
0000oooo0?ooo`1i0?ooo`030000003oool0oooo01D0oooo000J0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0N03oool00`000000oooo0?ooo`1b0?ooo`030000003oool0oooo07P0
oooo00<000000?ooo`3oool05@3oool001X0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`1g0?ooo`030000003oool0oooo07<0oooo00<000000?ooo`3oool0N03oool00`000000oooo
0?ooo`0E0?ooo`006P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo07L0oooo00<0
00000?ooo`3oool0M03oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo01D0oooo000J
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0M@3oool00`000000oooo0?ooo`1f
0?ooo`030000003oool0oooo07D0oooo00<000000?ooo`3oool05P3oool001X0oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`1d0?ooo`030000003oool0oooo07P0oooo00<000000?oo
o`3oool0M03oool00`000000oooo0?ooo`0F0?ooo`006P3oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo0780oooo00<000000?ooo`3oool0NP3oool00`000000oooo0?ooo`1b0?ooo`03
0000003oool0oooo01L0oooo000J0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
L@3oool00`000000oooo0?ooo`1l0?ooo`030000003oool0oooo0740oooo00<000000?ooo`3oool0
5`3oool001X0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`1`0?ooo`030000003o
ool0oooo07h0oooo00<000000?ooo`3oool0L03oool00`000000oooo0?ooo`0G0?ooo`006P3oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`800000PP3oool200000180oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01P0oooo000J0?ooo`<000004`3oool00`00
0000oooo0?ooo`0<0?ooo`<000004@3oool300000100oooo0`00000@0?ooo`<00000403oool30000
0100oooo0`0000220?ooo`<00000403oool300000140oooo0`00000@0?ooo`<00000403oool30000
0100oooo0`00000>0?ooo`030000003oool0oooo01P0oooo000J0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0
oooo0P0000240?ooo`8000004@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool06@3oool001X0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`1Z0?ooo`030000003oool0oooo08H0oooo00<000000?ooo`3oool0JP3oool00`000000oooo
0?ooo`0I0?ooo`006P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo06T0oooo00<0
00000?ooo`3oool0R03oool00`000000oooo0?ooo`1X0?ooo`030000003oool0oooo01X0oooo000J
0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0I`3oool00`000000oooo0?ooo`2:
0?ooo`030000003oool0oooo06L0oooo00<000000?ooo`3oool06P3oool001X0oooo00<000000?oo
o`3oool05@3oool00`000000oooo0?ooo`1V0?ooo`030000003oool0oooo08`0oooo00<000000?oo
o`3oool0IP3oool00`000000oooo0?ooo`0J0?ooo`006P3oool00`000000oooo0?ooo`0F0?ooo`03
0000003oool0oooo06@0oooo00<000000?ooo`3oool0SP3oool2000006D0oooo00<000000?ooo`3o
ool06`3oool001X0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`1R0?ooo`800000
T`3oool00`000000oooo0?ooo`1R0?ooo`030000003oool0oooo01/0oooo000J0?ooo`030000003o
ool0oooo01L0oooo00<000000?ooo`3oool0H03oool00`000000oooo0?ooo`2D0?ooo`030000003o
ool0oooo0600oooo00<000000?ooo`3oool0703oool001X0oooo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`1O0?ooo`030000003oool0oooo09H0oooo00<000000?ooo`3oool0G`3oool0
0`000000oooo0?ooo`0L0?ooo`006P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
09P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool0703oool001X0oooo0`00000H0?ooo`030000003oool0oooo00L0
oooo0`00000A0?ooo`<00000403oool300000100oooo0`00000@0?ooo`<000001@3oool00`000000
oooo0?ooo`2J0?ooo`800000103oool300000140oooo0`00000@0?ooo`<00000403oool300000100
oooo0`0000090?ooo`030000003oool0oooo01d0oooo000J0?ooo`030000003oool0oooo01P0oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0W@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0140oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0M0?ooo`006P3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo05P0oooo0P00002Q0?ooo`030000003oool0oooo05P0oooo00<0
00000?ooo`3oool07P3oool001X0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`1G
0?ooo`030000003oool0oooo0:80oooo00<000000?ooo`3oool0E`3oool00`000000oooo0?ooo`0N
0?ooo`006P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo05D0oooo00<000000?oo
o`3oool0Y03oool2000005H0oooo00<000000?ooo`3oool07`3oool001X0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`1D0?ooo`030000003oool0oooo0:L0oooo00<000000?ooo`3o
ool0D`3oool00`000000oooo0?ooo`0O0?ooo`006P3oool00`000000oooo0?ooo`0K0?ooo`030000
003oool0oooo0580oooo00<000000?ooo`3oool0Z@3oool00`000000oooo0?ooo`1A0?ooo`030000
003oool0oooo0200oooo000J0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0D03o
ool00`000000oooo0?ooo`2[0?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool08@3o
ool001X0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`1=0?ooo`800000[`3oool0
0`000000oooo0?ooo`1>0?ooo`030000003oool0oooo0240oooo000J0?ooo`030000003oool0oooo
01d0oooo00<000000?ooo`3oool0C03oool00`000000oooo0?ooo`2`0?ooo`800000C@3oool00`00
0000oooo0?ooo`0R0?ooo`006P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo04X0
oooo00<000000?ooo`3oool0/`3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo02<0
oooo000J0?ooo`030000003oool0oooo01l0oooo00D000000?ooo`3oool0oooo0000000C0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0:0?ooo`03
0000003oool0oooo0;D0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool0403oool01@000000oooo0?ooo`3oool00000
02H0oooo000J0?ooo`<00000803oool00`000000oooo0000000200000140oooo0`00000@0?ooo`<0
0000403oool3000000T0oooo0P00002i0?ooo`8000002@3oool300000100oooo0`00000@0?ooo`<0
0000403oool300000080oooo00<000000?ooo`3oool0903oool001X0oooo00<000000?ooo`3oool0
803oool010000000oooo0?ooo`00000C0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool0403oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0;/0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool0403oool010000000oooo0?ooo`00000W0?ooo`006P3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0440oooo0P00002o0?ooo`800000@P3oool00`000000oooo0?ooo`0V0?ooo`00
6P3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0
`@3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo02H0oooo000J0?ooo`030000003o
ool0oooo02<0oooo00<000000?ooo`3oool0?03oool200000<D0oooo0P00000n0?ooo`030000003o
ool0oooo02L0oooo000J0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0>`3oool0
0`000000oooo0?ooo`370?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0:03oool0
01X0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0h0?ooo`800000b`3oool20000
03X0oooo00<000000?ooo`3oool0:03oool001X0oooo00<000000?ooo`3oool09@3oool00`000000
oooo0?ooo`0e0?ooo`800000c`3oool2000003L0oooo00<000000?ooo`3oool0:@3oool000H0oooo
00D0_g@0000000000000001d0;ooo`030?ooo`050?noM000000000000000001d_ol01`3oool00`00
0000oooo0?ooo`0V0?ooo`800000=03oool00`000000oooo0?ooo`3A0?ooo`030000003oool0oooo
0380oooo0P00000/0?ooo`001@3oool01P3OW4P007Bo0?ooo`3oool0okmd001d_`<0oooo00H0W4Qd
0;ooo`3oool0oono07@0M02oool60?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0
;`3oool200000=D0oooo0P00000a0?ooo`030000003oool0oooo02`0oooo00040?ooo`030?oo_`1d
0000M;oo0080oooo00H0oono07A8W03oool0oooo0?noM000M;l30?ooo`030=nLB018W=l0oooo00D0
oooo00<000000?ooo`3oool0:@3oool2000002h0oooo00<000000?ooo`3oool0e`3oool00`000000
oooo0?ooo`0/0?ooo`800000;`3oool000@0oooo00L0okmd0018B01d07@0_ooo0?ooo`3OW4P007Bo
0080oooo00<0gia804RLg`3oool00P3oool00`3o_g@007Bo0?ooo`050?ooo`030000003oool0oooo
02/0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
00X0oooo0P00003K0?ooo`800000303oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool0;`3oool000@0oooo00<0omnL04Q8W03o_g@00P0000001P00B9`0
oooo0?ooo`3oool0gia804RLg`<0oooo00<0okmd001d_`3oool01@3oool4000002/0oooo0P000008
0?ooo`<00000403oool3000000T0oooo0P00003O0?ooo`8000002@3oool300000100oooo0`000008
0?ooo`800000<P3oool000@0oooo00<0oono07@0M02oool01P3oool00`3OW4P0B9cO0?ooo`020?oo
o`030?noM000M;l0oooo00D0oooo00<000000?ooo`3oool0;P3oool2000000L0oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`050?ooo`<00000h`3oool3000000L0oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`050?ooo`800000=03oool000D0oooo00<0_g@004RLg`3o
ool01@3oool00`3o_g@0B9cO0?ooo`020?ooo`030=nLB018W=l0oooo00D0oooo00<000000?ooo`3o
ool0<03oool00`000000oooo0?ooo`0L0?ooo`<00000j@3oool3000001h0oooo00<000000?ooo`3o
ool0=03oool000D0oooo00<0oono07@00000M;l01P3oool01P2LB000M;oo0?ooo`3ogi`0B01d0;oo
o`H0oooo00<000000?ooo`3oool0<@3oool2000001P0oooo1000003_0?ooo`<000006@3oool20000
03L0oooo00070?ooo`040;md0000000000000018W0<0oooo00D0oono07@000000000001809cOo`07
0?ooo`030000003oool0oooo03<0oooo1000000A0?ooo`<00000mP3oool300000140oooo1@00000i
0?ooo`006P3oool00`000000oooo0?ooo`0g0?ooo`P000001P3oool300000?`0oooo0`0000060?oo
o`P00000?P3oool001X0oooo00<000000?ooo`3oool0?`3oool600000?l0oooo0`3oool6000004H0
oooo000J0?ooo`030000003oool0oooo0?l0ooooU03oool001X0oooo00<000000?ooo`3oool0o`3o
oonD0?ooo`006P3oool00`000000oooo0?ooo`3o0?oooi@0oooo000J0?ooo`030000003oool0oooo
0?l0ooooU03oool001X0oooo00<000000?ooo`3oool0o`3ooonD0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {38.5918, 38.4312, 0.0786206, \
0.127211}}]
}, Closed]],

Cell["\<\
Notice that the result can be inaccurate if the mesh does not align well with \
the polygon vertex points.  If that occurs, we must use a finer mesh that is \
better able to describe curved surfaces.  Here the mesh does not reproduce \
the left and right sides of the electrode particulary well and is somewhat \
ragged elsewhere.  However, we will not spend the time now that would be \
required for a better approximation to this electrode.\
\>", "Text"],

Cell["We now compute the potential and field using familiar steps.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(Do[\[Phi]\[LeftDoubleBracket]i, j\[RightDoubleBracket] = 
          If[boundary\[LeftDoubleBracket]i, j\[RightDoubleBracket], 1. , 
            0. ], {i, 1, 120}, {j, 1, 120}];\)\), "\n", 
    \(\(\[Phi]0 = \[Phi];\)\), "\[IndentingNewLine]", 
    \(\(\[Phi] = 
        relax[\[Phi]0, boundary, 1000, 
          10\^\(-4\)];\)\), "\[IndentingNewLine]", 
    \(\(f = ListInterpolation[\[Phi]];\)\)}], "Input"],

Cell[BoxData[
    InterpretationBox[\("number of iterations = "\[InvisibleSpace]168\),
      SequenceForm[ "number of iterations = ", 168],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("residual = "\[InvisibleSpace]0.00009925370076314031`\
\),
      SequenceForm[ "residual = ", .000099253700763140312],
      Editable->False]], "Print"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ContourPlot[f[x, y], {x, 20, 80}, {y, 20, 80}, 
        PlotPoints \[Rule] 200, AspectRatio \[Rule] Automatic, 
        ContourShading \[Rule] False, 
        Contours \[Rule] Range[0.49, 0.99, 0.1], Frame \[Rule] False, 
        Epilog \[Rule] {GrayLevel[0.75], Polygon[poly]}];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% ContourGraphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.301282 0.0160256 -0.301282 0.0160256 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.39128 .35234 m
.39612 .35184 L
.40095 .35231 L
.4022 .35263 L
.40578 .35397 L
.41057 .35746 L
.41061 .3575 L
.41505 .36229 L
.41544 .36263 L
.42027 .36615 L
.4217 .36712 L
.42511 .3696 L
.42726 .37196 L
.42994 .37498 L
.43173 .37679 L
.43477 .37923 L
.43806 .38162 L
.4396 .38266 L
.44372 .38645 L
.44443 .38721 L
.44839 .39128 L
.44927 .39206 L
.4541 .39588 L
.45439 .39612 L
.45893 .40021 L
.45955 .40095 L
.46292 .40578 L
.46376 .40745 L
.46522 .41061 L
.46813 .41544 L
.46859 .41604 L
.47301 .42027 L
.47342 .42058 L
.47826 .42412 L
.4795 .42511 L
.48309 .42795 L
.48589 .42994 L
.48792 .43117 L
.49275 .43365 L
.49544 .43477 L
.49758 .43544 L
.50242 .43655 L
.50725 .437 L
.51208 .43676 L
.51691 .43584 L
.52054 .43477 L
.52174 .43428 L
.52658 .43199 L
.53015 .42994 L
.53141 .42909 L
Mistroke
.53624 .42524 L
.53642 .42511 L
.54107 .42167 L
.54296 .42027 L
.5459 .41761 L
.54771 .41544 L
.55068 .41061 L
.55073 .41049 L
.55309 .40578 L
.55557 .40177 L
.55638 .40095 L
.5604 .39699 L
.56155 .39612 L
.56523 .39326 L
.56757 .39128 L
.57006 .38875 L
.57224 .38645 L
.57489 .38385 L
.5778 .38162 L
.57973 .38024 L
.58418 .37679 L
.58456 .37643 L
.58869 .37196 L
.58939 .37116 L
.59422 .36715 L
.59433 .36712 L
.59905 .36374 L
.60084 .36229 L
.60388 .35903 L
.6053 .35746 L
.60872 .35379 L
.61334 .35263 L
.61355 .35258 L
.61838 .3518 L
.62321 .35197 L
.62649 .35263 L
.62804 .35317 L
.63288 .3567 L
.63345 .35746 L
.6374 .36229 L
.63771 .3626 L
.64254 .36653 L
.64334 .36712 L
.64737 .37179 L
.64742 .37196 L
.64866 .37679 L
.64903 .38162 L
.64886 .38645 L
.64909 .39128 L
.64968 .39612 L
Mistroke
.65051 .40095 L
.65215 .40578 L
.6522 .40589 L
.65645 .41061 L
.65704 .41114 L
.66165 .41544 L
.66187 .41606 L
.66455 .42027 L
.66524 .42511 L
.66535 .42994 L
.66512 .43477 L
.66513 .4396 L
.66524 .44443 L
.66533 .44927 L
.66541 .4541 L
.66548 .45893 L
.66554 .46376 L
.66559 .46859 L
.66563 .47342 L
.66567 .47826 L
.6657 .48309 L
.66572 .48792 L
.66574 .49275 L
.66576 .49758 L
.66577 .50242 L
.66578 .50725 L
.66578 .51208 L
.66578 .51691 L
.66577 .52174 L
.66576 .52658 L
.66574 .53141 L
.66572 .53624 L
.6657 .54107 L
.66566 .5459 L
.66563 .55073 L
.66558 .55557 L
.66552 .5604 L
.66546 .56523 L
.66538 .57006 L
.66529 .57489 L
.66518 .57973 L
.66542 .58456 L
.66544 .58939 L
.665 .59422 L
.66282 .59905 L
.66187 .60042 L
.65862 .60388 L
.65704 .60521 L
.65364 .60872 L
.6522 .61071 L
Mistroke
.65116 .61355 L
.65019 .61838 L
.64954 .62321 L
.64904 .62804 L
.64925 .63288 L
.64907 .63771 L
.64817 .64254 L
.64737 .64416 L
.64577 .64737 L
.64254 .64999 L
.63948 .6522 L
.63771 .65386 L
.63496 .65704 L
.63288 .65975 L
.63115 .66187 L
.62804 .6636 L
.62321 .6649 L
.61838 .66508 L
.61355 .66429 L
.60882 .66187 L
.60872 .66175 L
.60388 .65753 L
.60345 .65704 L
.59905 .65285 L
.59823 .6522 L
.59422 .64953 L
.59147 .64737 L
.58939 .64552 L
.5866 .64254 L
.58456 .64045 L
.58138 .63771 L
.57973 .63653 L
.57499 .63288 L
.57489 .63281 L
.57006 .6283 L
.56983 .62804 L
.56523 .62366 L
.56466 .62321 L
.5604 .61991 L
.55862 .61838 L
.55557 .61551 L
.55413 .61355 L
.55122 .60872 L
.55073 .60783 L
.54841 .60388 L
.5459 .60063 L
.5443 .59905 L
.54107 .59638 L
.53822 .59422 L
.53624 .5928 L
Mistroke
.53158 .58939 L
.53141 .58928 L
.52658 .58653 L
.52228 .58456 L
.52174 .58436 L
.51691 .58287 L
.51208 .58199 L
.50725 .58175 L
.50242 .58219 L
.49758 .58325 L
.49363 .58456 L
.49275 .58494 L
.48792 .58731 L
.48447 .58939 L
.48309 .59033 L
.47826 .59386 L
.47775 .59422 L
.47342 .59753 L
.47163 .59905 L
.46859 .6022 L
.4674 .60388 L
.46469 .60872 L
.46376 .61057 L
.46186 .61355 L
.45893 .61706 L
.45734 .61838 L
.4541 .62105 L
.45127 .62321 L
.44927 .62488 L
.44607 .62804 L
.44443 .6298 L
.44091 .63288 L
.4396 .63392 L
.43477 .63748 L
.43446 .63771 L
.42994 .64184 L
.42927 .64254 L
.42511 .64678 L
.42439 .64737 L
.42027 .65051 L
.41759 .6522 L
.41544 .654 L
.41243 .65704 L
.41061 .65909 L
.40704 .66187 L
.40578 .66263 L
.40095 .66454 L
.39612 .66502 L
.39128 .66451 L
.38645 .66228 L
Mistroke
.38584 .66187 L
.38162 .65734 L
.38139 .65704 L
.37695 .6522 L
.37679 .65208 L
.37196 .64857 L
.37049 .64737 L
.36816 .64254 L
.3674 .63771 L
.36726 .63288 L
.36748 .62804 L
.36712 .62374 L
.36705 .62321 L
.36643 .61838 L
.36547 .61355 L
.36301 .60872 L
.36229 .60777 L
.35798 .60388 L
.35746 .60345 L
.35388 .59905 L
.35263 .5965 L
.35185 .59422 L
.35136 .58939 L
.35137 .58456 L
.35165 .57973 L
.35152 .57489 L
.35141 .57006 L
.35133 .56523 L
.35125 .5604 L
.35119 .55557 L
.35113 .55073 L
.35109 .5459 L
.35105 .54107 L
.35102 .53624 L
.35099 .53141 L
.35097 .52658 L
.35096 .52174 L
.35095 .51691 L
.35094 .51208 L
.35095 .50725 L
.35095 .50242 L
.35096 .49758 L
.35098 .49275 L
.351 .48792 L
.35103 .48309 L
.35107 .47826 L
.35111 .47342 L
.35116 .46859 L
.35122 .46376 L
.35128 .45893 L
Mistroke
.35136 .4541 L
.35144 .44927 L
.35154 .44443 L
.35166 .4396 L
.35167 .43477 L
.35142 .42994 L
.35155 .42511 L
.35227 .42027 L
.35263 .41946 L
.35439 .41544 L
.35746 .41283 L
.36011 .41061 L
.36229 .40867 L
.36432 .40578 L
.36606 .40095 L
.36687 .39612 L
.36712 .3945 L
.36739 .39128 L
.36757 .38645 L
.3674 .38162 L
.3677 .37679 L
.36873 .37196 L
.37196 .3678 L
.37284 .36712 L
.37679 .36421 L
.37886 .36229 L
.38162 .35891 L
.38254 .35746 L
.38645 .3542 L
.39019 .35263 L
.39128 .35234 L
Mfstroke
.38645 .33324 m
.39128 .33198 L
.39612 .33111 L
.40095 .33055 L
.40578 .3303 L
.41061 .33038 L
.41544 .33071 L
.42027 .33125 L
.42511 .3319 L
.42994 .33268 L
.43326 .3333 L
.43477 .33359 L
.4396 .33454 L
.44443 .33553 L
.44927 .33655 L
.4541 .33757 L
.45678 .33813 L
.45893 .33857 L
.46376 .33954 L
.46859 .34046 L
.47342 .3413 L
.47826 .34206 L
.48309 .34272 L
.48505 .34296 L
.48792 .34328 L
.49275 .34372 L
.49758 .34404 L
.50242 .34423 L
.50725 .34428 L
.51208 .3442 L
.51691 .34399 L
.52174 .34364 L
.52658 .34317 L
.52835 .34296 L
.53141 .34258 L
.53624 .34188 L
.54107 .34109 L
.5459 .34021 L
.55073 .33926 L
.55557 .33825 L
.55613 .33813 L
.5604 .33721 L
.56523 .33615 L
.57006 .33508 L
.57489 .33404 L
.57844 .3333 L
.57973 .33304 L
.58456 .33207 L
.58939 .33122 L
.59422 .3305 L
Mistroke
.59905 .3299 L
.60388 .32947 L
.60872 .32928 L
.61355 .32937 L
.61838 .3298 L
.62321 .33052 L
.62804 .33156 L
.63288 .33309 L
.63344 .3333 L
.63771 .33508 L
.64254 .33753 L
.64361 .33813 L
.64737 .34046 L
.65093 .34296 L
.6522 .34394 L
.65679 .3478 L
.65704 .34802 L
.66172 .35263 L
.66187 .35278 L
.66591 .35746 L
.6667 .35844 L
.66957 .36229 L
.67153 .36514 L
.67283 .36712 L
.6758 .37196 L
.67636 .37297 L
.67838 .37679 L
.6808 .38162 L
.68119 .38243 L
.68312 .38645 L
.68535 .39128 L
.68603 .3928 L
.68747 .39612 L
.68946 .40095 L
.69086 .40451 L
.69134 .40578 L
.69312 .41061 L
.69474 .41544 L
.69569 .41849 L
.69621 .42027 L
.69749 .42511 L
.69865 .42994 L
.69971 .43477 L
.70052 .43888 L
.70066 .4396 L
.70152 .44443 L
.70229 .44927 L
.703 .4541 L
.70363 .45893 L
.70419 .46376 L
Mistroke
.7047 .46859 L
.70515 .47342 L
.70535 .47587 L
.70554 .47826 L
.70588 .48309 L
.70616 .48792 L
.7064 .49275 L
.70658 .49758 L
.70672 .50242 L
.70681 .50725 L
.70685 .51208 L
.70683 .51691 L
.70677 .52174 L
.70666 .52658 L
.70649 .53141 L
.70628 .53624 L
.70601 .54107 L
.70569 .5459 L
.70535 .55015 L
.7053 .55073 L
.70486 .55557 L
.70434 .5604 L
.70376 .56523 L
.70311 .57006 L
.70237 .57489 L
.70155 .57973 L
.70062 .58456 L
.70052 .58503 L
.69958 .58939 L
.69844 .59422 L
.69714 .59905 L
.69569 .60386 L
.69568 .60388 L
.69407 .60872 L
.69233 .61355 L
.69086 .61744 L
.69049 .61838 L
.68853 .62321 L
.68644 .62804 L
.68603 .62898 L
.68424 .63288 L
.68193 .63771 L
.68119 .63918 L
.67948 .64254 L
.67678 .64737 L
.67636 .64807 L
.67374 .6522 L
.67153 .65542 L
.67037 .65704 L
.6667 .66173 L
Mistroke
.66659 .66187 L
.66225 .6667 L
.66187 .66709 L
.65715 .67153 L
.65704 .67164 L
.6522 .67557 L
.65113 .67636 L
.64737 .67893 L
.64363 .68119 L
.64254 .6818 L
.63771 .68421 L
.63329 .68603 L
.63288 .68618 L
.62804 .68777 L
.62321 .68889 L
.61838 .68972 L
.61355 .69027 L
.60872 .69052 L
.60388 .6905 L
.59905 .69028 L
.59422 .68991 L
.58939 .68942 L
.58456 .6888 L
.57973 .68808 L
.57489 .68733 L
.57006 .68651 L
.56728 .68603 L
.56523 .68567 L
.5604 .68481 L
.55557 .68397 L
.55073 .68315 L
.5459 .68237 L
.54107 .68165 L
.53783 .68119 L
.53624 .68099 L
.53141 .6804 L
.52658 .6799 L
.52174 .67949 L
.51691 .67919 L
.51208 .67898 L
.50725 .67889 L
.50242 .67891 L
.49758 .67903 L
.49275 .67926 L
.48792 .6796 L
.48309 .68002 L
.47826 .68053 L
.47342 .68112 L
.47285 .68119 L
.46859 .68177 L
Mistroke
.46376 .68248 L
.45893 .68322 L
.4541 .68399 L
.44927 .68477 L
.44443 .68554 L
.4413 .68603 L
.4396 .68628 L
.43477 .68697 L
.42994 .68762 L
.42511 .68816 L
.42027 .68856 L
.41544 .68886 L
.41061 .68898 L
.40578 .68887 L
.40095 .68847 L
.39612 .68777 L
.39128 .68677 L
.3884 .68603 L
.38645 .68542 L
.38162 .6836 L
.37679 .68133 L
.37652 .68119 L
.37196 .67854 L
.36863 .67636 L
.36712 .67529 L
.36241 .67153 L
.36229 .67143 L
.35746 .6669 L
.35726 .6667 L
.3529 .66187 L
.35263 .66154 L
.34915 .65704 L
.3478 .65512 L
.34586 .6522 L
.34296 .64745 L
.34292 .64737 L
.3403 .64254 L
.33813 .63807 L
.33797 .63771 L
.33578 .63288 L
.33368 .62804 L
.3333 .62715 L
.33166 .62321 L
.32974 .61838 L
.32847 .61498 L
.32795 .61355 L
.32624 .60872 L
.32466 .60388 L
.32364 .60051 L
.32322 .59905 L
Mistroke
.32197 .59422 L
.3209 .58939 L
.31994 .58456 L
.31908 .57973 L
.31881 .57805 L
.31832 .57489 L
.31764 .57006 L
.31704 .56523 L
.31651 .5604 L
.31605 .55557 L
.31564 .55073 L
.31529 .5459 L
.31499 .54107 L
.31474 .53624 L
.31454 .53141 L
.31439 .52658 L
.31428 .52174 L
.31422 .51691 L
.31419 .51208 L
.31421 .50725 L
.31427 .50242 L
.31438 .49758 L
.31453 .49275 L
.31472 .48792 L
.31496 .48309 L
.31524 .47826 L
.31557 .47342 L
.31595 .46859 L
.31639 .46376 L
.31688 .45893 L
.31743 .4541 L
.31805 .44927 L
.31873 .44443 L
.31881 .44396 L
.3195 .4396 L
.32036 .43477 L
.32131 .42994 L
.32237 .42511 L
.32356 .42027 L
.32364 .41998 L
.32496 .41544 L
.32653 .41061 L
.32825 .40578 L
.32847 .40519 L
.33008 .40095 L
.332 .39612 L
.3333 .393 L
.33404 .39128 L
.33616 .38645 L
.33813 .3821 L
Mistroke
.33835 .38162 L
.34066 .37679 L
.34296 .37224 L
.34312 .37196 L
.34594 .36712 L
.3478 .36421 L
.3491 .36229 L
.35263 .35751 L
.35267 .35746 L
.35683 .35263 L
.35746 .35197 L
.36177 .3478 L
.36229 .34733 L
.36712 .34338 L
.36769 .34296 L
.37196 .34007 L
.3752 .33813 L
.37679 .33728 L
.38162 .33504 L
.38625 .3333 L
.38645 .33324 L
Mfstroke
.42027 .29427 m
.42511 .29378 L
.42994 .29338 L
.43477 .29306 L
.4396 .29279 L
.44443 .29258 L
.44927 .29242 L
.4541 .2923 L
.45893 .2922 L
.46376 .29213 L
.46859 .29208 L
.47342 .29204 L
.47826 .292 L
.48309 .29197 L
.48792 .29194 L
.49275 .2919 L
.49758 .29186 L
.50242 .29181 L
.50725 .29175 L
.51208 .29168 L
.51691 .2916 L
.52174 .29152 L
.52658 .29143 L
.53141 .29133 L
.53624 .29123 L
.54107 .29114 L
.5459 .29105 L
.55073 .29098 L
.55557 .29092 L
.5604 .29089 L
.56523 .29089 L
.57006 .29093 L
.57489 .29101 L
.57973 .29115 L
.58456 .29135 L
.58939 .29162 L
.59422 .29197 L
.59905 .29242 L
.60388 .29297 L
.60872 .29363 L
.61355 .29441 L
.61482 .29465 L
.61838 .29535 L
.62321 .29644 L
.62804 .29768 L
.63288 .29912 L
.63401 .29948 L
.63771 .30074 L
.64254 .30255 L
.64674 .30431 L
Mistroke
.64737 .30459 L
.6522 .30684 L
.65666 .30914 L
.65704 .30934 L
.66187 .31209 L
.66493 .31397 L
.6667 .31511 L
.67153 .31842 L
.67207 .31881 L
.67636 .32205 L
.67836 .32364 L
.68119 .32598 L
.68405 .32847 L
.68603 .33028 L
.68918 .3333 L
.69086 .33498 L
.69387 .33813 L
.69569 .34012 L
.6982 .34296 L
.70052 .34572 L
.70221 .3478 L
.70535 .35186 L
.70593 .35263 L
.7094 .35746 L
.71019 .3586 L
.71265 .36229 L
.71502 .36603 L
.71569 .36712 L
.71856 .37196 L
.71985 .37423 L
.72126 .37679 L
.7238 .38162 L
.72468 .38337 L
.72619 .38645 L
.72845 .39128 L
.72951 .39365 L
.73058 .39612 L
.73259 .40095 L
.73434 .40542 L
.73448 .40578 L
.73626 .41061 L
.73792 .41544 L
.73918 .41928 L
.73949 .42027 L
.74095 .42511 L
.74231 .42994 L
.74358 .43477 L
.74401 .43647 L
.74476 .4396 L
.74586 .44443 L
Mistroke
.74687 .44927 L
.7478 .4541 L
.74866 .45893 L
.74884 .46004 L
.74943 .46376 L
.75013 .46859 L
.75077 .47342 L
.75133 .47826 L
.75182 .48309 L
.75224 .48792 L
.75259 .49275 L
.75287 .49758 L
.75309 .50242 L
.75323 .50725 L
.75331 .51208 L
.75332 .51691 L
.75327 .52174 L
.75315 .52658 L
.75296 .53141 L
.75269 .53624 L
.75236 .54107 L
.75196 .5459 L
.75149 .55073 L
.75094 .55557 L
.75032 .5604 L
.74962 .56523 L
.74884 .57006 L
.74884 .57008 L
.74798 .57489 L
.74704 .57973 L
.74601 .58456 L
.74488 .58939 L
.74401 .5929 L
.74367 .59422 L
.74235 .59905 L
.74094 .60388 L
.73943 .60872 L
.73918 .60949 L
.73781 .61355 L
.73607 .61838 L
.73434 .62288 L
.73421 .62321 L
.73224 .62804 L
.73014 .63288 L
.72951 .63426 L
.7279 .63771 L
.72553 .64254 L
.72468 .64419 L
.723 .64737 L
.7203 .6522 L
Mistroke
.71985 .65299 L
.71744 .65704 L
.71502 .66088 L
.71438 .66187 L
.71111 .6667 L
.71019 .668 L
.7076 .67153 L
.70535 .67446 L
.70384 .67636 L
.70052 .68035 L
.6998 .68119 L
.69569 .68574 L
.69542 .68603 L
.69086 .69067 L
.69067 .69086 L
.68603 .69522 L
.6855 .69569 L
.68119 .69939 L
.67981 .70052 L
.67636 .70322 L
.6735 .70535 L
.67153 .70676 L
.6667 .71 L
.66642 .71019 L
.66187 .71298 L
.65832 .71502 L
.65704 .71572 L
.6522 .71822 L
.64882 .71985 L
.64737 .72051 L
.64254 .72259 L
.63771 .72447 L
.63715 .72468 L
.63288 .72618 L
.62804 .72771 L
.62321 .72907 L
.62154 .72951 L
.61838 .73029 L
.61355 .73138 L
.60872 .73233 L
.60388 .73317 L
.59905 .7339 L
.59569 .73434 L
.59422 .73453 L
.58939 .73506 L
.58456 .73552 L
.57973 .7359 L
.57489 .73622 L
.57006 .73647 L
.56523 .73667 L
Mistroke
.5604 .73682 L
.55557 .73693 L
.55073 .73701 L
.5459 .73705 L
.54107 .73706 L
.53624 .73705 L
.53141 .73702 L
.52658 .73697 L
.52174 .73691 L
.51691 .73683 L
.51208 .73674 L
.50725 .73664 L
.50242 .73653 L
.49758 .7364 L
.49275 .73627 L
.48792 .73613 L
.48309 .73596 L
.47826 .73579 L
.47342 .73559 L
.46859 .73537 L
.46376 .73513 L
.45893 .73485 L
.4541 .73454 L
.45127 .73434 L
.44927 .73419 L
.44443 .7338 L
.4396 .73335 L
.43477 .73284 L
.42994 .73226 L
.42511 .73162 L
.42027 .73088 L
.41544 .73006 L
.41254 .72951 L
.41061 .72913 L
.40578 .72809 L
.40095 .72693 L
.39612 .72562 L
.39292 .72468 L
.39128 .72417 L
.38645 .72255 L
.38162 .72075 L
.37939 .71985 L
.37679 .71875 L
.37196 .71656 L
.36882 .71502 L
.36712 .71414 L
.36229 .71148 L
.36009 .71019 L
.35746 .70857 L
.35263 .70538 L
Mistroke
.35259 .70535 L
.3478 .70189 L
.34601 .70052 L
.34296 .69807 L
.34016 .69569 L
.33813 .69389 L
.33488 .69086 L
.3333 .68933 L
.33006 .68603 L
.32847 .68432 L
.32567 .68119 L
.32364 .67881 L
.32163 .67636 L
.31881 .67274 L
.3179 .67153 L
.31443 .6667 L
.31397 .66603 L
.31121 .66187 L
.30914 .65856 L
.30822 .65704 L
.30542 .6522 L
.30431 .65019 L
.30281 .64737 L
.30035 .64254 L
.29948 .64073 L
.29806 .63771 L
.2959 .63288 L
.29465 .62992 L
.29388 .62804 L
.29197 .62321 L
.29018 .61838 L
.28981 .61734 L
.28851 .61355 L
.28695 .60872 L
.2855 .60388 L
.28498 .60207 L
.28415 .59905 L
.2829 .59422 L
.28174 .58939 L
.28067 .58456 L
.28015 .58203 L
.27969 .57973 L
.2788 .57489 L
.27798 .57006 L
.27723 .56523 L
.27657 .5604 L
.27598 .55557 L
.27546 .55073 L
.27532 .54926 L
.27501 .5459 L
Mistroke
.27462 .54107 L
.27429 .53624 L
.27402 .53141 L
.27382 .52658 L
.27369 .52174 L
.27362 .51691 L
.2736 .51208 L
.27364 .50725 L
.27374 .50242 L
.2739 .49758 L
.27413 .49275 L
.27442 .48792 L
.27477 .48309 L
.27518 .47826 L
.27532 .4768 L
.27566 .47342 L
.27619 .46859 L
.2768 .46376 L
.27748 .45893 L
.27823 .4541 L
.27905 .44927 L
.27995 .44443 L
.28015 .44341 L
.28093 .4396 L
.28199 .43477 L
.28313 .42994 L
.28437 .42511 L
.28498 .42284 L
.2857 .42027 L
.28714 .41544 L
.28868 .41061 L
.28981 .40723 L
.29032 .40578 L
.29208 .40095 L
.29395 .39612 L
.29465 .39439 L
.29594 .39128 L
.29807 .38645 L
.29948 .38339 L
.30032 .38162 L
.30272 .37679 L
.30431 .37373 L
.30526 .37196 L
.30799 .36712 L
.30914 .36517 L
.3109 .36229 L
.31397 .35752 L
.31401 .35746 L
.31736 .35263 L
.31881 .35064 L
Mistroke
.32096 .3478 L
.32364 .34445 L
.32487 .34296 L
.32847 .33884 L
.32912 .33813 L
.3333 .33375 L
.33376 .3333 L
.33813 .32914 L
.33888 .32847 L
.34296 .32495 L
.34457 .32364 L
.3478 .32112 L
.35096 .31881 L
.35263 .31765 L
.35746 .31449 L
.3583 .31397 L
.36229 .31163 L
.36692 .30914 L
.36712 .30904 L
.37196 .30671 L
.37679 .30463 L
.37758 .30431 L
.38162 .30277 L
.38645 .30112 L
.39128 .29967 L
.39199 .29948 L
.39612 .29841 L
.40095 .29731 L
.40578 .29635 L
.41061 .29554 L
.41544 .29485 L
.41704 .29465 L
.42027 .29427 L
Mfstroke
.49275 .24147 m
.49758 .24128 L
.50242 .24113 L
.50725 .24101 L
.51208 .24093 L
.51691 .24089 L
.52174 .24087 L
.52658 .2409 L
.53141 .24096 L
.53624 .24105 L
.54107 .24119 L
.5459 .24137 L
.54889 .2415 L
.55073 .24158 L
.55557 .24184 L
.5604 .24215 L
.56523 .2425 L
.57006 .24291 L
.57489 .24337 L
.57973 .24389 L
.58456 .24447 L
.58939 .24511 L
.59422 .24582 L
.59744 .24633 L
.59905 .2466 L
.60388 .24745 L
.60872 .24839 L
.61355 .24941 L
.61838 .25051 L
.62106 .25116 L
.62321 .2517 L
.62804 .25299 L
.63288 .25438 L
.63771 .25588 L
.63806 .25599 L
.64254 .25748 L
.64737 .2592 L
.65163 .26082 L
.6522 .26105 L
.65704 .26302 L
.66187 .26512 L
.66306 .26566 L
.6667 .26735 L
.67153 .26973 L
.673 .27049 L
.67636 .27226 L
.68119 .27494 L
.68185 .27532 L
.68603 .27779 L
.68982 .28015 L
Mistroke
.69086 .28081 L
.69569 .28401 L
.6971 .28498 L
.70052 .2874 L
.70379 .28981 L
.70535 .291 L
.70999 .29465 L
.71019 .2948 L
.71502 .29883 L
.71577 .29948 L
.71985 .3031 L
.72117 .30431 L
.72468 .30763 L
.72623 .30914 L
.72951 .31243 L
.73101 .31397 L
.73434 .31752 L
.73552 .31881 L
.73918 .32295 L
.73977 .32364 L
.74381 .32847 L
.74401 .32872 L
.74764 .3333 L
.74884 .33487 L
.75127 .33813 L
.75367 .34146 L
.75473 .34296 L
.75802 .3478 L
.7585 .34852 L
.76116 .35263 L
.76334 .35612 L
.76415 .35746 L
.767 .36229 L
.76817 .36435 L
.76971 .36712 L
.7723 .37196 L
.773 .3733 L
.77476 .37679 L
.77711 .38162 L
.77783 .38316 L
.77934 .38645 L
.78147 .39128 L
.78266 .3941 L
.7835 .39612 L
.78542 .40095 L
.78725 .40578 L
.7875 .40645 L
.78898 .41061 L
.79063 .41544 L
.79218 .42027 L
Mistroke
.79233 .42073 L
.79365 .42511 L
.79503 .42994 L
.79633 .43477 L
.79716 .43803 L
.79755 .4396 L
.79868 .44443 L
.79974 .44927 L
.80072 .4541 L
.80164 .45893 L
.80199 .46091 L
.80248 .46376 L
.80324 .46859 L
.80393 .47342 L
.80455 .47826 L
.8051 .48309 L
.80557 .48792 L
.80598 .49275 L
.80632 .49758 L
.80659 .50242 L
.80678 .50725 L
.80682 .50822 L
.80691 .51208 L
.80697 .51691 L
.80696 .52174 L
.80688 .52658 L
.80682 .529 L
.80673 .53141 L
.80652 .53624 L
.80623 .54107 L
.80587 .5459 L
.80545 .55073 L
.80495 .55557 L
.80438 .5604 L
.80374 .56523 L
.80302 .57006 L
.80222 .57489 L
.80199 .57621 L
.80135 .57973 L
.8004 .58456 L
.79938 .58939 L
.79827 .59422 L
.79716 .59875 L
.79708 .59905 L
.79581 .60388 L
.79446 .60872 L
.79301 .61355 L
.79233 .61575 L
.79148 .61838 L
.78986 .62321 L
Mistroke
.78815 .62804 L
.7875 .6298 L
.78633 .63288 L
.78441 .63771 L
.78266 .64191 L
.7824 .64254 L
.78027 .64737 L
.77803 .6522 L
.77783 .65262 L
.77568 .65704 L
.7732 .66187 L
.773 .66226 L
.7706 .6667 L
.76817 .67101 L
.76787 .67153 L
.765 .67636 L
.76334 .67906 L
.76198 .68119 L
.75881 .68603 L
.7585 .68649 L
.75548 .69086 L
.75367 .69338 L
.75198 .69569 L
.74884 .69982 L
.74829 .70052 L
.74441 .70535 L
.74401 .70584 L
.74031 .71019 L
.73918 .71148 L
.73599 .71502 L
.73434 .71679 L
.73142 .71985 L
.72951 .72179 L
.72658 .72468 L
.72468 .7265 L
.72144 .72951 L
.71985 .73095 L
.71598 .73434 L
.71502 .73516 L
.71019 .73914 L
.71014 .73918 L
.70535 .74291 L
.70389 .74401 L
.70052 .74648 L
.69717 .74884 L
.69569 .74985 L
.69086 .75305 L
.68989 .75367 L
.68603 .75608 L
.68195 .7585 L
Mistroke
.68119 .75894 L
.67636 .76165 L
.67322 .76334 L
.67153 .76422 L
.6667 .76664 L
.66351 .76817 L
.66187 .76893 L
.65704 .77109 L
.65252 .773 L
.6522 .77313 L
.64737 .77505 L
.64254 .77686 L
.63981 .77783 L
.63771 .77856 L
.63288 .78015 L
.62804 .78165 L
.62458 .78266 L
.62321 .78305 L
.61838 .78435 L
.61355 .78557 L
.60872 .7867 L
.60507 .7875 L
.60388 .78774 L
.59905 .78871 L
.59422 .78961 L
.58939 .79043 L
.58456 .79118 L
.57973 .79186 L
.57614 .79233 L
.57489 .79248 L
.57006 .79303 L
.56523 .79352 L
.5604 .79395 L
.55557 .79433 L
.55073 .79465 L
.5459 .79491 L
.54107 .79512 L
.53624 .79527 L
.53141 .79538 L
.52658 .79543 L
.52174 .79544 L
.51691 .79539 L
.51208 .7953 L
.50725 .79516 L
.50242 .79497 L
.49758 .79474 L
.49275 .79445 L
.48792 .79411 L
.48309 .79373 L
.47826 .7933 L
Mistroke
.47342 .79282 L
.46898 .79233 L
.46859 .79228 L
.46376 .79169 L
.45893 .79105 L
.4541 .79034 L
.44927 .78959 L
.44443 .78878 L
.4396 .7879 L
.43747 .7875 L
.43477 .78696 L
.42994 .78595 L
.42511 .78487 L
.42027 .78371 L
.41613 .78266 L
.41544 .78248 L
.41061 .78117 L
.40578 .77978 L
.40095 .7783 L
.39948 .77783 L
.39612 .77672 L
.39128 .77506 L
.38645 .77329 L
.38568 .773 L
.38162 .77142 L
.37679 .76943 L
.37385 .76817 L
.37196 .76733 L
.36712 .76511 L
.36347 .76334 L
.36229 .76275 L
.35746 .76026 L
.35421 .7585 L
.35263 .75763 L
.3478 .75483 L
.34586 .75367 L
.34296 .75188 L
.33825 .74884 L
.33813 .74877 L
.3333 .74547 L
.33125 .74401 L
.32847 .74198 L
.32478 .73918 L
.32364 .73828 L
.31881 .73437 L
.31878 .73434 L
.31397 .73022 L
.31318 .72951 L
.30914 .7258 L
.30796 .72468 L
Mistroke
.30431 .72112 L
.30305 .71985 L
.29948 .71614 L
.29843 .71502 L
.29465 .71083 L
.29408 .71019 L
.28997 .70535 L
.28981 .70516 L
.28609 .70052 L
.28498 .6991 L
.28241 .69569 L
.28015 .69258 L
.27893 .69086 L
.27563 .68603 L
.27532 .68556 L
.2725 .68119 L
.27049 .67795 L
.26952 .67636 L
.2667 .67153 L
.26566 .66968 L
.26401 .6667 L
.26147 .66187 L
.26082 .66061 L
.25904 .65704 L
.25674 .6522 L
.25599 .65057 L
.25456 .64737 L
.25249 .64254 L
.25116 .6393 L
.25052 .63771 L
.24866 .63288 L
.24689 .62804 L
.24633 .62643 L
.24523 .62321 L
.24366 .61838 L
.24217 .61355 L
.2415 .61124 L
.24077 .60872 L
.23946 .60388 L
.23822 .59905 L
.23707 .59422 L
.23666 .59245 L
.23599 .58939 L
.235 .58456 L
.23407 .57973 L
.23322 .57489 L
.23244 .57006 L
.23183 .56597 L
.23173 .56523 L
.23109 .5604 L
Mistroke
.23052 .55557 L
.23001 .55073 L
.22957 .5459 L
.2292 .54107 L
.22889 .53624 L
.22864 .53141 L
.22846 .52658 L
.22834 .52174 L
.22828 .51691 L
.22829 .51208 L
.22836 .50725 L
.22849 .50242 L
.22868 .49758 L
.22894 .49275 L
.22927 .48792 L
.22966 .48309 L
.23011 .47826 L
.23062 .47342 L
.2312 .46859 L
.23183 .46393 L
.23186 .46376 L
.23258 .45893 L
.23337 .4541 L
.23423 .44927 L
.23516 .44443 L
.23616 .4396 L
.23666 .4373 L
.23724 .43477 L
.23839 .42994 L
.23962 .42511 L
.24094 .42027 L
.2415 .41831 L
.24234 .41544 L
.24382 .41061 L
.2454 .40578 L
.24633 .40305 L
.24706 .40095 L
.24882 .39612 L
.25068 .39128 L
.25116 .39007 L
.25263 .38645 L
.2547 .38162 L
.25599 .37872 L
.25687 .37679 L
.25916 .37196 L
.26082 .3686 L
.26157 .36712 L
.26411 .36229 L
.26566 .35947 L
.26678 .35746 L
Mistroke
.2696 .35263 L
.27049 .35115 L
.27256 .3478 L
.27532 .34352 L
.27568 .34296 L
.27898 .33813 L
.28015 .33648 L
.28246 .3333 L
.28498 .32996 L
.28614 .32847 L
.28981 .32391 L
.29004 .32364 L
.29417 .31881 L
.29465 .31826 L
.29855 .31397 L
.29948 .31299 L
.30322 .30914 L
.30431 .30805 L
.3082 .30431 L
.30914 .30343 L
.31354 .29948 L
.31397 .2991 L
.31881 .29503 L
.31927 .29465 L
.32364 .2912 L
.32547 .28981 L
.32847 .28761 L
.3322 .28498 L
.3333 .28423 L
.33813 .28106 L
.33957 .28015 L
.34296 .27807 L
.34771 .27532 L
.3478 .27527 L
.35263 .27264 L
.35682 .27049 L
.35746 .27017 L
.36229 .26785 L
.36712 .26568 L
.36718 .26566 L
.37196 .26365 L
.37679 .26174 L
.37925 .26082 L
.38162 .25996 L
.38645 .2583 L
.39128 .25675 L
.3938 .25599 L
.39612 .25531 L
.40095 .25397 L
.40578 .25272 L
Mistroke
.41061 .25156 L
.41237 .25116 L
.41544 .25048 L
.42027 .24948 L
.42511 .24856 L
.42994 .24771 L
.43477 .24692 L
.43866 .24633 L
.4396 .24619 L
.44443 .24552 L
.44927 .24491 L
.4541 .24435 L
.45893 .24384 L
.46376 .24338 L
.46859 .24296 L
.47342 .24259 L
.47826 .24225 L
.48309 .24196 L
.48792 .2417 L
.49221 .2415 L
.49275 .24147 L
Mfstroke
.47826 .18799 m
.48309 .1876 L
.48792 .18727 L
.49275 .18698 L
.49758 .18674 L
.50242 .18655 L
.50725 .1864 L
.51208 .18629 L
.51691 .18624 L
.52174 .18622 L
.52658 .18626 L
.53141 .18633 L
.53624 .18646 L
.54107 .18664 L
.5459 .18686 L
.55073 .18713 L
.55557 .18745 L
.5604 .18782 L
.56523 .18824 L
.56636 .18835 L
.57006 .18871 L
.57489 .18924 L
.57973 .18981 L
.58456 .19045 L
.58939 .19114 L
.59422 .19189 L
.59905 .1927 L
.60175 .19318 L
.60388 .19357 L
.60872 .19449 L
.61355 .19548 L
.61838 .19653 L
.62321 .19766 L
.62467 .19801 L
.62804 .19885 L
.63288 .20011 L
.63771 .20144 L
.64253 .20284 L
.64254 .20284 L
.64737 .20432 L
.6522 .20588 L
.65704 .20751 L
.6575 .20767 L
.66187 .20923 L
.6667 .21103 L
.67048 .2125 L
.67153 .21293 L
.67636 .21491 L
.68119 .21699 L
.68199 .21734 L
Mistroke
.68603 .21916 L
.69086 .22142 L
.6924 .22217 L
.69569 .22379 L
.70052 .22627 L
.70191 .227 L
.70535 .22886 L
.71019 .23156 L
.71067 .23183 L
.71502 .23438 L
.71879 .23666 L
.71985 .23732 L
.72468 .24038 L
.72639 .2415 L
.72951 .24358 L
.73351 .24633 L
.73434 .24691 L
.73918 .25039 L
.74022 .25116 L
.74401 .25402 L
.74655 .25599 L
.74884 .25781 L
.75256 .26082 L
.75367 .26175 L
.75826 .26566 L
.7585 .26587 L
.76334 .27017 L
.76369 .27049 L
.76817 .27465 L
.76887 .27532 L
.773 .27934 L
.77382 .28015 L
.77783 .28424 L
.77855 .28498 L
.78266 .28936 L
.78308 .28981 L
.78742 .29465 L
.7875 .29473 L
.79158 .29948 L
.79233 .30036 L
.79558 .30431 L
.79716 .30628 L
.79941 .30914 L
.80199 .3125 L
.8031 .31397 L
.80664 .31881 L
.80682 .31907 L
.81004 .32364 L
.81165 .32599 L
.81332 .32847 L
Mistroke
.81647 .3333 L
.81649 .33333 L
.8195 .33813 L
.82132 .34114 L
.82241 .34296 L
.82521 .3478 L
.82615 .34947 L
.8279 .35263 L
.83048 .35746 L
.83098 .35841 L
.83297 .36229 L
.83536 .36712 L
.83581 .36806 L
.83766 .37196 L
.83985 .37679 L
.84065 .37858 L
.84196 .38162 L
.84398 .38645 L
.84548 .39016 L
.84592 .39128 L
.84778 .39612 L
.84956 .40095 L
.85031 .40307 L
.85125 .40578 L
.85287 .41061 L
.85441 .41544 L
.85514 .41784 L
.85587 .42027 L
.85725 .42511 L
.85856 .42994 L
.8598 .43477 L
.85997 .43546 L
.86097 .4396 L
.86208 .44443 L
.86311 .44927 L
.86407 .4541 L
.8648 .45805 L
.86496 .45893 L
.86579 .46376 L
.86656 .46859 L
.86725 .47342 L
.86788 .47826 L
.86845 .48309 L
.86895 .48792 L
.86939 .49275 L
.86964 .49583 L
.86976 .49758 L
.87007 .50242 L
.87032 .50725 L
.8705 .51208 L
Mistroke
.87062 .51691 L
.87068 .52174 L
.87067 .52658 L
.87061 .53141 L
.87047 .53624 L
.87028 .54107 L
.87002 .5459 L
.8697 .55073 L
.86964 .55152 L
.86931 .55557 L
.86886 .5604 L
.86834 .56523 L
.86776 .57006 L
.86711 .57489 L
.8664 .57973 L
.86562 .58456 L
.8648 .58924 L
.86478 .58939 L
.86386 .59422 L
.86288 .59905 L
.86184 .60388 L
.86072 .60872 L
.85997 .61178 L
.85953 .61355 L
.85826 .61838 L
.85692 .62321 L
.8555 .62804 L
.85514 .62925 L
.85402 .63288 L
.85246 .63771 L
.85082 .64254 L
.85031 .64398 L
.84909 .64737 L
.84728 .6522 L
.84548 .6568 L
.84538 .65704 L
.84341 .66187 L
.84134 .6667 L
.84065 .66829 L
.83919 .67153 L
.83694 .67636 L
.83581 .67871 L
.8346 .68119 L
.83216 .68603 L
.83098 .68831 L
.82963 .69086 L
.82699 .69569 L
.82615 .69719 L
.82425 .70052 L
.82139 .70535 L
Mistroke
.82132 .70548 L
.81842 .71019 L
.81649 .71324 L
.81534 .71502 L
.81213 .71985 L
.81165 .72055 L
.80879 .72468 L
.80682 .72744 L
.80532 .72951 L
.80199 .73397 L
.80171 .73434 L
.79795 .73918 L
.79716 .74017 L
.79404 .74401 L
.79233 .74606 L
.78996 .74884 L
.7875 .75167 L
.78572 .75367 L
.78266 .75703 L
.78129 .7585 L
.77783 .76214 L
.77667 .76334 L
.773 .76702 L
.77184 .76817 L
.76817 .7717 L
.76679 .773 L
.76334 .77619 L
.76152 .77783 L
.7585 .7805 L
.756 .78266 L
.75367 .78462 L
.75019 .7875 L
.74884 .78858 L
.74407 .79233 L
.74401 .79238 L
.73918 .79602 L
.73763 .79716 L
.73434 .79953 L
.73083 .80199 L
.72951 .80289 L
.72468 .80612 L
.72361 .80682 L
.71985 .80923 L
.71595 .81165 L
.71502 .81222 L
.71019 .81508 L
.70775 .81649 L
.70535 .81783 L
.70052 .82047 L
.69893 .82132 L
Mistroke
.69569 .82301 L
.69086 .82544 L
.6894 .82615 L
.68603 .82777 L
.68119 .83 L
.679 .83098 L
.67636 .83214 L
.67153 .83419 L
.66754 .83581 L
.6667 .83615 L
.66187 .83802 L
.65704 .83981 L
.65471 .84065 L
.6522 .84152 L
.64737 .84315 L
.64254 .8447 L
.64004 .84548 L
.63771 .84618 L
.63288 .84758 L
.62804 .84891 L
.62321 .85017 L
.62265 .85031 L
.61838 .85135 L
.61355 .85247 L
.60872 .85352 L
.60388 .8545 L
.60053 .85514 L
.59905 .85541 L
.59422 .85626 L
.58939 .85705 L
.58456 .85777 L
.57973 .85843 L
.57489 .85904 L
.57006 .85958 L
.56624 .85997 L
.56523 .86007 L
.5604 .8605 L
.55557 .86086 L
.55073 .86117 L
.5459 .86143 L
.54107 .86163 L
.53624 .86177 L
.53141 .86186 L
.52658 .86189 L
.52174 .86186 L
.51691 .86178 L
.51208 .86165 L
.50725 .86146 L
.50242 .86122 L
.49758 .86092 L
Mistroke
.49275 .86057 L
.48792 .86015 L
.48598 .85997 L
.48309 .85969 L
.47826 .85917 L
.47342 .85859 L
.46859 .85795 L
.46376 .85726 L
.45893 .85651 L
.4541 .85571 L
.4509 .85514 L
.44927 .85484 L
.44443 .8539 L
.4396 .8529 L
.43477 .85184 L
.42994 .85073 L
.42818 .85031 L
.42511 .84955 L
.42027 .8483 L
.41544 .84698 L
.41061 .84559 L
.41023 .84548 L
.40578 .84413 L
.40095 .8426 L
.39612 .84099 L
.39511 .84065 L
.39128 .83931 L
.38645 .83754 L
.3819 .83581 L
.38162 .83571 L
.37679 .83379 L
.37196 .83178 L
.37009 .83098 L
.36712 .82969 L
.36229 .82751 L
.35938 .82615 L
.35746 .82524 L
.35263 .82287 L
.34957 .82132 L
.3478 .8204 L
.34296 .81782 L
.34054 .81649 L
.33813 .81514 L
.3333 .81235 L
.33213 .81165 L
.32847 .80944 L
.32429 .80682 L
.32364 .80641 L
.31881 .80326 L
.31692 .80199 L
Mistroke
.31397 .79998 L
.30998 .79716 L
.30914 .79656 L
.30431 .79299 L
.30343 .79233 L
.29948 .78928 L
.29722 .7875 L
.29465 .78541 L
.29134 .78266 L
.28981 .78136 L
.28575 .77783 L
.28498 .77715 L
.28042 .773 L
.28015 .77275 L
.27534 .76817 L
.27532 .76815 L
.27049 .76334 L
.27049 .76333 L
.26585 .7585 L
.26566 .7583 L
.26141 .75367 L
.26082 .75302 L
.25717 .74884 L
.25599 .74746 L
.2531 .74401 L
.25116 .74162 L
.24921 .73918 L
.24633 .73548 L
.24547 .73434 L
.24187 .72951 L
.2415 .72899 L
.23843 .72468 L
.23666 .72213 L
.23512 .71985 L
.23194 .71502 L
.23183 .71486 L
.22888 .71019 L
.227 .70711 L
.22595 .70535 L
.22313 .70052 L
.22217 .69882 L
.22043 .69569 L
.21783 .69086 L
.21734 .68992 L
.21534 .68603 L
.21294 .68119 L
.2125 .68029 L
.21065 .67636 L
.20844 .67153 L
.20767 .6698 L
Mistroke
.20633 .6667 L
.2043 .66187 L
.20284 .65824 L
.20237 .65704 L
.20052 .6522 L
.19875 .64737 L
.19801 .64528 L
.19706 .64254 L
.19544 .63771 L
.1939 .63288 L
.19318 .63054 L
.19243 .62804 L
.19103 .62321 L
.18971 .61838 L
.18845 .61355 L
.18835 .61312 L
.18727 .60872 L
.18615 .60388 L
.1851 .59905 L
.18412 .59422 L
.18351 .59109 L
.1832 .58939 L
.18234 .58456 L
.18154 .57973 L
.18081 .57489 L
.18015 .57006 L
.17954 .56523 L
.17899 .5604 L
.17868 .55737 L
.17851 .55557 L
.17808 .55073 L
.17771 .5459 L
.1774 .54107 L
.17715 .53624 L
.17696 .53141 L
.17683 .52658 L
.17675 .52174 L
.17674 .51691 L
.17678 .51208 L
.17687 .50725 L
.17702 .50242 L
.17724 .49758 L
.1775 .49275 L
.17783 .48792 L
.17822 .48309 L
.17867 .47826 L
.17868 .47809 L
.17917 .47342 L
.17974 .46859 L
.18037 .46376 L
Mistroke
.18106 .45893 L
.18181 .4541 L
.18263 .44927 L
.18351 .44443 L
.18351 .44439 L
.18445 .4396 L
.18545 .43477 L
.18653 .42994 L
.18767 .42511 L
.18835 .42238 L
.18888 .42027 L
.19016 .41544 L
.19152 .41061 L
.19294 .40578 L
.19318 .405 L
.19444 .40095 L
.19602 .39612 L
.19767 .39128 L
.19801 .39033 L
.19941 .38645 L
.20122 .38162 L
.20284 .37749 L
.20312 .37679 L
.20511 .37196 L
.20719 .36712 L
.20767 .36603 L
.20936 .36229 L
.21163 .35746 L
.2125 .35565 L
.214 .35263 L
.21646 .3478 L
.21734 .34614 L
.21904 .34296 L
.22173 .33813 L
.22217 .33736 L
.22453 .3333 L
.227 .3292 L
.22745 .32847 L
.23049 .32364 L
.23183 .32158 L
.23367 .31881 L
.23666 .31444 L
.23699 .31397 L
.24045 .30914 L
.2415 .30772 L
.24407 .30431 L
.24633 .3014 L
.24785 .29948 L
.25116 .29542 L
.2518 .29465 L
Mistroke
.25593 .28981 L
.25599 .28974 L
.26025 .28498 L
.26082 .28436 L
.26479 .28015 L
.26566 .27926 L
.26955 .27532 L
.27049 .27439 L
.27455 .27049 L
.27532 .26976 L
.27981 .26566 L
.28015 .26535 L
.28498 .26115 L
.28536 .26082 L
.28981 .25713 L
.29123 .25599 L
.29465 .2533 L
.29744 .25116 L
.29948 .24963 L
.30404 .24633 L
.30431 .24614 L
.30914 .24279 L
.31107 .2415 L
.31397 .23959 L
.3186 .23666 L
.31881 .23654 L
.32364 .23362 L
.3267 .23183 L
.32847 .23083 L
.3333 .22816 L
.33546 .227 L
.33813 .22561 L
.34296 .22317 L
.34502 .22217 L
.3478 .22084 L
.35263 .21862 L
.35552 .21734 L
.35746 .2165 L
.36229 .21447 L
.36712 .21254 L
.36721 .2125 L
.37196 .2107 L
.37679 .20894 L
.38044 .20767 L
.38162 .20727 L
.38645 .20568 L
.39128 .20417 L
.39575 .20284 L
.39612 .20274 L
.40095 .20137 L
Mistroke
.40578 .20008 L
.41061 .19886 L
.41414 .19801 L
.41544 .1977 L
.42027 .19661 L
.42511 .19558 L
.42994 .19462 L
.43477 .19371 L
.43775 .19318 L
.4396 .19286 L
.44443 .19206 L
.44927 .19132 L
.4541 .19064 L
.45893 .19001 L
.46376 .18943 L
.46859 .1889 L
.47342 .18842 L
.47418 .18835 L
.47826 .18799 L
Mfstroke
.50242 .12545 m
.50725 .12529 L
.51208 .12517 L
.51691 .1251 L
.52174 .12507 L
.52658 .12508 L
.53141 .12514 L
.53624 .12524 L
.54107 .12539 L
.54476 .12553 L
.5459 .12558 L
.55073 .12582 L
.55557 .1261 L
.5604 .12642 L
.56523 .12679 L
.57006 .1272 L
.57489 .12767 L
.57973 .12818 L
.58456 .12874 L
.58939 .12935 L
.59422 .13001 L
.59672 .13036 L
.59905 .13071 L
.60388 .13146 L
.60872 .13226 L
.61355 .13312 L
.61838 .13402 L
.62321 .13497 L
.62432 .1352 L
.62804 .13597 L
.63288 .13702 L
.63771 .13813 L
.64254 .13929 L
.64551 .14003 L
.64737 .1405 L
.6522 .14177 L
.65704 .1431 L
.66187 .14449 L
.66312 .14486 L
.6667 .14594 L
.67153 .14744 L
.67636 .14901 L
.67839 .14969 L
.68119 .15065 L
.68603 .15235 L
.69086 .15412 L
.69194 .15452 L
.69569 .15595 L
.70052 .15786 L
.7042 .15935 L
Mistroke
.70535 .15983 L
.71019 .16188 L
.71502 .16401 L
.71542 .16419 L
.71985 .1662 L
.72468 .16847 L
.72582 .16902 L
.72951 .17082 L
.73434 .17325 L
.7355 .17385 L
.73918 .17577 L
.74401 .17837 L
.74458 .17868 L
.74884 .18105 L
.75316 .18351 L
.75367 .18381 L
.7585 .18668 L
.76125 .18835 L
.76334 .18964 L
.76817 .19269 L
.76892 .19318 L
.773 .19585 L
.77621 .19801 L
.77783 .19911 L
.78266 .20248 L
.78317 .20284 L
.7875 .20596 L
.78982 .20767 L
.79233 .20956 L
.79617 .2125 L
.79716 .21328 L
.80199 .21712 L
.80226 .21734 L
.80682 .22108 L
.80812 .22217 L
.81165 .22518 L
.81375 .227 L
.81649 .22942 L
.81916 .23183 L
.82132 .23382 L
.82436 .23666 L
.82615 .23837 L
.82938 .2415 L
.83098 .24308 L
.83421 .24633 L
.83581 .24796 L
.83888 .25116 L
.84065 .25303 L
.84339 .25599 L
.84548 .25828 L
Mistroke
.84775 .26082 L
.85031 .26373 L
.85197 .26566 L
.85514 .26941 L
.85604 .27049 L
.85997 .27531 L
.85998 .27532 L
.86379 .28015 L
.8648 .28147 L
.86748 .28498 L
.86964 .28788 L
.87105 .28981 L
.87447 .29459 L
.8745 .29465 L
.87785 .29948 L
.8793 .30163 L
.88108 .30431 L
.88413 .30902 L
.88421 .30914 L
.88724 .31397 L
.88896 .31679 L
.89017 .31881 L
.89301 .32364 L
.8938 .32499 L
.89577 .32847 L
.89844 .3333 L
.89863 .33365 L
.90101 .33813 L
.90346 .34289 L
.9035 .34296 L
.9059 .3478 L
.90822 .35263 L
.90829 .35278 L
.91046 .35746 L
.91262 .36229 L
.91312 .36346 L
.91469 .36712 L
.9167 .37196 L
.91796 .37506 L
.91865 .37679 L
.92053 .38162 L
.92233 .38645 L
.92279 .3877 L
.92406 .39128 L
.92573 .39612 L
.92733 .40095 L
.92762 .40185 L
.92886 .40578 L
.93032 .41061 L
.93172 .41544 L
Mistroke
.93245 .41806 L
.93306 .42027 L
.93434 .42511 L
.93556 .42994 L
.93672 .43477 L
.93728 .43723 L
.93781 .4396 L
.93884 .44443 L
.93982 .44927 L
.94074 .4541 L
.94159 .45893 L
.94211 .46206 L
.94239 .46376 L
.94313 .46859 L
.94381 .47342 L
.94444 .47826 L
.94502 .48309 L
.94553 .48792 L
.94599 .49275 L
.94639 .49758 L
.94673 .50242 L
.94695 .50582 L
.94703 .50725 L
.94727 .51208 L
.94745 .51691 L
.94758 .52174 L
.94765 .52658 L
.94768 .53141 L
.94764 .53624 L
.94755 .54107 L
.9474 .5459 L
.9472 .55073 L
.94695 .55557 L
.94695 .55559 L
.94664 .5604 L
.94627 .56523 L
.94585 .57006 L
.94538 .57489 L
.94484 .57973 L
.94426 .58456 L
.94361 .58939 L
.94292 .59422 L
.94216 .59905 L
.94211 .59933 L
.94134 .60388 L
.94047 .60872 L
.93953 .61355 L
.93855 .61838 L
.9375 .62321 L
.93728 .62418 L
Mistroke
.93639 .62804 L
.93522 .63288 L
.93398 .63771 L
.93269 .64254 L
.93245 .6434 L
.93134 .64737 L
.92994 .6522 L
.92847 .65704 L
.92762 .65974 L
.92693 .66187 L
.92533 .6667 L
.92365 .67153 L
.92279 .67397 L
.92192 .67636 L
.92012 .68119 L
.91825 .68603 L
.91796 .68678 L
.91632 .69086 L
.91431 .69569 L
.91312 .69845 L
.91222 .70052 L
.91006 .70535 L
.90829 .70919 L
.90782 .71019 L
.90551 .71502 L
.90346 .71915 L
.90311 .71985 L
.90063 .72468 L
.89863 .72847 L
.89807 .72951 L
.89543 .73434 L
.8938 .73725 L
.8927 .73918 L
.88988 .74401 L
.88896 .74554 L
.88696 .74884 L
.88413 .75339 L
.88395 .75367 L
.88085 .7585 L
.8793 .76086 L
.87765 .76334 L
.87447 .76799 L
.87435 .76817 L
.87094 .773 L
.86964 .77481 L
.86742 .77783 L
.8648 .78132 L
.86378 .78266 L
.86003 .7875 L
.85997 .78756 L
Mistroke
.85615 .79233 L
.85514 .79356 L
.85214 .79716 L
.85031 .79931 L
.84799 .80199 L
.84548 .80486 L
.84372 .80682 L
.84065 .8102 L
.8393 .81165 L
.83581 .81535 L
.83472 .81649 L
.83098 .82031 L
.82998 .82132 L
.82615 .82511 L
.82508 .82615 L
.82132 .82975 L
.82 .83098 L
.81649 .83423 L
.81473 .83581 L
.81165 .83856 L
.80927 .84065 L
.80682 .84275 L
.80359 .84548 L
.80199 .84681 L
.79769 .85031 L
.79716 .85074 L
.79233 .85455 L
.79156 .85514 L
.7875 .85823 L
.78516 .85997 L
.78266 .8618 L
.77848 .8648 L
.77783 .86526 L
.773 .86861 L
.77149 .86964 L
.76817 .87185 L
.76415 .87447 L
.76334 .87499 L
.7585 .87803 L
.75644 .8793 L
.75367 .88098 L
.74884 .88384 L
.74833 .88413 L
.74401 .8866 L
.73975 .88896 L
.73918 .88928 L
.73434 .89187 L
.73065 .8938 L
.72951 .89438 L
.72468 .8968 L
Mistroke
.72093 .89863 L
.71985 .89915 L
.71502 .90142 L
.71053 .90346 L
.71019 .90361 L
.70535 .90573 L
.70052 .90777 L
.69926 .90829 L
.69569 .90974 L
.69086 .91164 L
.68695 .91312 L
.68603 .91347 L
.68119 .91523 L
.67636 .91692 L
.67333 .91796 L
.67153 .91856 L
.6667 .92012 L
.66187 .92163 L
.658 .92279 L
.65704 .92307 L
.6522 .92444 L
.64737 .92575 L
.64254 .92701 L
.64014 .92762 L
.63771 .92822 L
.63288 .92938 L
.62804 .93047 L
.62321 .9315 L
.61847 .93245 L
.61838 .93247 L
.61355 .93338 L
.60872 .93425 L
.60388 .93507 L
.59905 .93583 L
.59422 .93653 L
.58939 .93716 L
.58844 .93728 L
.58456 .93775 L
.57973 .93828 L
.57489 .93877 L
.57006 .9392 L
.56523 .93958 L
.5604 .93991 L
.55557 .94018 L
.55073 .94039 L
.5459 .94056 L
.54107 .94068 L
.53624 .94075 L
.53141 .94077 L
.52658 .94073 L
Mistroke
.52174 .94064 L
.51691 .9405 L
.51208 .94031 L
.50725 .94007 L
.50242 .93977 L
.49758 .93943 L
.49275 .93903 L
.48792 .93858 L
.48309 .93808 L
.47826 .93753 L
.47622 .93728 L
.47342 .93693 L
.46859 .93628 L
.46376 .93556 L
.45893 .93479 L
.4541 .93397 L
.44927 .9331 L
.44585 .93245 L
.44443 .93217 L
.4396 .9312 L
.43477 .93016 L
.42994 .92905 L
.42511 .92789 L
.42402 .92762 L
.42027 .92667 L
.41544 .92539 L
.41061 .92406 L
.40621 .92279 L
.40578 .92266 L
.40095 .9212 L
.39612 .91968 L
.39128 .91809 L
.39088 .91796 L
.38645 .91644 L
.38162 .91472 L
.3773 .91312 L
.37679 .91293 L
.37196 .91108 L
.36712 .90916 L
.36499 .90829 L
.36229 .90717 L
.35746 .90511 L
.3537 .90346 L
.35263 .90298 L
.3478 .90078 L
.34323 .89863 L
.34296 .8985 L
.33813 .89614 L
.33351 .8938 L
.3333 .89369 L
Mistroke
.32847 .89115 L
.32442 .88896 L
.32364 .88854 L
.31881 .88583 L
.31585 .88413 L
.31397 .88304 L
.30914 .88015 L
.30775 .8793 L
.30431 .87717 L
.30007 .87447 L
.29948 .87409 L
.29465 .8709 L
.29277 .86964 L
.28981 .86761 L
.28581 .8648 L
.28498 .86422 L
.28015 .86071 L
.27916 .85997 L
.27532 .85708 L
.27279 .85514 L
.27049 .85334 L
.26669 .85031 L
.26566 .84947 L
.26083 .84548 L
.26082 .84548 L
.25599 .84134 L
.25519 .84065 L
.25116 .83707 L
.24977 .83581 L
.24633 .83265 L
.24454 .83098 L
.2415 .82807 L
.23952 .82615 L
.23666 .82332 L
.23468 .82132 L
.23183 .8184 L
.23 .81649 L
.227 .81329 L
.22549 .81165 L
.22217 .80799 L
.22113 .80682 L
.21734 .80248 L
.21691 .80199 L
.21283 .79716 L
.2125 .79676 L
.20889 .79233 L
.20767 .7908 L
.20508 .7875 L
.20284 .78458 L
.2014 .78266 L
Mistroke
.19801 .77807 L
.19784 .77783 L
.19439 .773 L
.19318 .77127 L
.19104 .76817 L
.18835 .76415 L
.18781 .76334 L
.18467 .7585 L
.18351 .75669 L
.18163 .75367 L
.17868 .74884 L
.17868 .74884 L
.17583 .74401 L
.17385 .74057 L
.17307 .73918 L
.1704 .73434 L
.16902 .73179 L
.16781 .72951 L
.1653 .72468 L
.16419 .72248 L
.16288 .71985 L
.16054 .71502 L
.15935 .71252 L
.15827 .71019 L
.15608 .70535 L
.15452 .70182 L
.15396 .70052 L
.15191 .69569 L
.14994 .69086 L
.14969 .69025 L
.14803 .68603 L
.1462 .68119 L
.14486 .67756 L
.14443 .67636 L
.14272 .67153 L
.14108 .6667 L
.14003 .6635 L
.1395 .66187 L
.13798 .65704 L
.13652 .6522 L
.1352 .64763 L
.13512 .64737 L
.13378 .64254 L
.13251 .63771 L
.13128 .63288 L
.13036 .62909 L
.13012 .62804 L
.129 .62321 L
.12794 .61838 L
.12694 .61355 L
Mistroke
.12599 .60872 L
.12553 .60627 L
.1251 .60388 L
.12426 .59905 L
.12347 .59422 L
.12274 .58939 L
.12205 .58456 L
.12142 .57973 L
.12083 .57489 L
.1207 .57372 L
.1203 .57006 L
.11982 .56523 L
.11939 .5604 L
.119 .55557 L
.11867 .55073 L
.11839 .5459 L
.11815 .54107 L
.11797 .53624 L
.11783 .53141 L
.11775 .52658 L
.11772 .52174 L
.11774 .51691 L
.11781 .51208 L
.11793 .50725 L
.11809 .50242 L
.11831 .49758 L
.11858 .49275 L
.11889 .48792 L
.11926 .48309 L
.11968 .47826 L
.12015 .47342 L
.12067 .46859 L
.1207 .4683 L
.12124 .46376 L
.12186 .45893 L
.12254 .4541 L
.12327 .44927 L
.12405 .44443 L
.12489 .4396 L
.12553 .43611 L
.12578 .43477 L
.12673 .42994 L
.12773 .42511 L
.12879 .42027 L
.12991 .41544 L
.13036 .41358 L
.13109 .41061 L
.13233 .40578 L
.13363 .40095 L
.13499 .39612 L
Mistroke
.1352 .39539 L
.1364 .39128 L
.13789 .38645 L
.13944 .38162 L
.14003 .37984 L
.14106 .37679 L
.14274 .37196 L
.14449 .36712 L
.14486 .36613 L
.14631 .36229 L
.1482 .35746 L
.14969 .35378 L
.15017 .35263 L
.1522 .3478 L
.15432 .34296 L
.15452 .34252 L
.15652 .33813 L
.1588 .3333 L
.15935 .33215 L
.16116 .32847 L
.16361 .32364 L
.16419 .32252 L
.16615 .31881 L
.16878 .31397 L
.16902 .31354 L
.1715 .30914 L
.17385 .3051 L
.17432 .30431 L
.17724 .29948 L
.17868 .29715 L
.18026 .29465 L
.1834 .28981 L
.18351 .28964 L
.18665 .28498 L
.18835 .28252 L
.19001 .28015 L
.19318 .27575 L
.1935 .27532 L
.19711 .27049 L
.19801 .26931 L
.20085 .26566 L
.20284 .26316 L
.20473 .26082 L
.20767 .25727 L
.20875 .25599 L
.2125 .25164 L
.21292 .25116 L
.21726 .24633 L
.21734 .24624 L
.22176 .2415 L
Mistroke
.22217 .24106 L
.22644 .23666 L
.227 .2361 L
.23131 .23183 L
.23183 .23133 L
.23639 .227 L
.23666 .22675 L
.2415 .22234 L
.24169 .22217 L
.24633 .2181 L
.24722 .21734 L
.25116 .21402 L
.25299 .2125 L
.25599 .21008 L
.25903 .20767 L
.26082 .20628 L
.26536 .20284 L
.26566 .20263 L
.27049 .1991 L
.27202 .19801 L
.27532 .1957 L
.27901 .19318 L
.28015 .19242 L
.28498 .18925 L
.28639 .18835 L
.28981 .18619 L
.2942 .18351 L
.29465 .18324 L
.29948 .1804 L
.30248 .17868 L
.30431 .17765 L
.30914 .175 L
.31131 .17385 L
.31397 .17245 L
.31881 .16999 L
.32077 .16902 L
.32364 .16762 L
.32847 .16533 L
.33095 .16419 L
.3333 .16312 L
.33813 .16099 L
.342 .15935 L
.34296 .15895 L
.3478 .15698 L
.35263 .15509 L
.35411 .15452 L
.35746 .15327 L
.36229 .15151 L
.36712 .14983 L
.36754 .14969 L
Mistroke
.37196 .14822 L
.37679 .14666 L
.38162 .14516 L
.38263 .14486 L
.38645 .14373 L
.39128 .14236 L
.39612 .14105 L
.40007 .14003 L
.40095 .1398 L
.40578 .13861 L
.41061 .13747 L
.41544 .13639 L
.42027 .13535 L
.42105 .1352 L
.42511 .13438 L
.42994 .13345 L
.43477 .13258 L
.4396 .13176 L
.44443 .13099 L
.44863 .13036 L
.44927 .13027 L
.4541 .1296 L
.45893 .12897 L
.46376 .12839 L
.46859 .12786 L
.47342 .12738 L
.47826 .12695 L
.48309 .12656 L
.48792 .12621 L
.49275 .12591 L
.49758 .12566 L
.50038 .12553 L
.50242 .12545 L
Mfstroke
.75 g
.50801 .50801 m
.5281 .54787 L
.54787 .58522 L
.56701 .61772 L
.58522 .64332 L
.60221 .66043 L
.61772 .66795 L
.63149 .66543 L
.64332 .65302 L
.65302 .63149 L
.66043 .60221 L
.66543 .56701 L
.66795 .5281 L
.66795 .48793 L
.66543 .44902 L
.66043 .41382 L
.65302 .38453 L
.64332 .36301 L
.63149 .35059 L
.61772 .34807 L
.60221 .3556 L
.58522 .3727 L
.56701 .39831 L
.54787 .43081 L
.5281 .46816 L
.50801 .50801 L
.48793 .54787 L
.46816 .58522 L
.44902 .61772 L
.43081 .64332 L
.41382 .66043 L
.39831 .66795 L
.38453 .66543 L
.3727 .65302 L
.36301 .63149 L
.3556 .60221 L
.35059 .56701 L
.34807 .5281 L
.34807 .48793 L
.35059 .44902 L
.3556 .41382 L
.36301 .38453 L
.3727 .36301 L
.38453 .35059 L
.39831 .34807 L
.41382 .3556 L
.43081 .3727 L
.44902 .39831 L
.46816 .43081 L
.48793 .46816 L
.50801 .50801 L
F
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 288},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`0001/2000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00fP3oool>00000<P0oooo003;
0?ooo`l000003P3oool>00000;X0oooo00330?ooo`P00000:`3oool900000;40oooo002m0?ooo`H0
0000?03oool600000:/0oooo002g0?ooo`H00000B03oool600000:D0oooo002b0?ooo`D00000E03o
ool400000:40oooo002^0?ooo`@00000G@3oool4000009d0oooo002Z0?ooo`@00000I@3oool50000
09P0oooo002V0?ooo`@00000KP3oool4000009@0oooo002T0?ooo`800000MP3oool400000900oooo
002P0?ooo`@00000O03oool2000008h0oooo002N0?ooo`800000PP3oool4000008X0oooo002L0?oo
o`800000R03oool2000008P0oooo002G0?ooo`D00000S03oool3000008D0oooo002E0?ooo`800000
U03oool00`000000oooo0?ooo`220?ooo`00T`3oool2000009L0oooo1@00001o0?ooo`00T@3oool2
000009h0oooo0P00001m0?ooo`00SP3oool300000:80oooo0P00001k0?ooo`00S@3oool00`000000
oooo0?ooo`2U0?ooo`800000N@3oool008X0oooo0`00002Z0?ooo`800000M`3oool008T0oooo00<0
00000?ooo`3oool0[@3oool2000007D0oooo00260?ooo`<00000/P3oool300000780oooo00250?oo
o`030000003oool0oooo0;H0oooo00<000000?ooo`3oool0K`3oool008<0oooo0P00002j0?ooo`80
0000K`3oool00800oooo0`00002m0?ooo`030000003oool0oooo06d0oooo001n0?ooo`800000`@3o
ool2000006d0oooo001n0?ooo`030000003oool0oooo0<80oooo0P00001[0?ooo`00O03oool20000
0580oooo8P00001C0?ooo`800000J@3oool007X0oooo0P0000190?ooo`/000008P3oool8000004d0
oooo0`00001V0?ooo`00N03oool2000004D0oooo1P00000e0?ooo`L00000B@3oool00`000000oooo
0?ooo`1S0?ooo`00N03oool00`000000oooo0?ooo`0n0?ooo`H00000@P3oool6000004@0oooo00<0
00000?ooo`3oool0HP3oool007H0oooo0P00000m0?ooo`@00000CP3oool400000440oooo0P00001R
0?ooo`00M03oool2000003X0oooo1@00001F0?ooo`@00000?`3oool200000600oooo001c0?ooo`03
0000003oool0oooo03H0oooo1000001O0?ooo`@00000?@3oool00`000000oooo0?ooo`1M0?ooo`00
LP3oool00`000000oooo0?ooo`0c0?ooo`@00000I`3oool3000003/0oooo00<000000?ooo`3oool0
G03oool00700oooo0P00000d0?ooo`800000KP3oool4000003P0oooo0P00001L0?ooo`00K`3oool0
0`000000oooo0?ooo`0a0?ooo`<00000M03oool2000003P0oooo00<000000?ooo`3oool0F@3oool0
06h0oooo00<000000?ooo`3oool0;`3oool3000007T0oooo1000000e0?ooo`030000003oool0oooo
05P0oooo001]0?ooo`030000003oool0oooo02h0oooo0P0000200?ooo`800000=03oool2000005P0
oooo001/0?ooo`030000003oool0oooo02d0oooo0P0000240?ooo`800000=03oool00`000000oooo
0?ooo`1E0?ooo`00JP3oool2000002d0oooo0`0000280?ooo`800000<`3oool00`000000oooo0?oo
o`1D0?ooo`00JP3oool00`000000oooo0?ooo`0Z0?ooo`800000S@3oool200000380oooo00<00000
0?ooo`3oool0D`3oool006P0oooo0P00000Z0?ooo`<00000T@3oool300000300oooo0P00001C0?oo
o`00I`3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo09D0oooo00<000000?ooo`3o
ool0;P3oool00`000000oooo0?ooo`1A0?ooo`00I@3oool2000002T0oooo0`00002I0?ooo`<00000
;P3oool200000540oooo001T0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0W@3o
ool2000002h0oooo00<000000?ooo`3oool0CP3oool006<0oooo00<000000?ooo`3oool09P3oool3
00000:80oooo0P00000]0?ooo`800000CP3oool00680oooo00<000000?ooo`3oool09P3oool00`00
0000oooo0?ooo`2U0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0C03oool00640
oooo00<000000?ooo`3oool09@3oool200000:T0oooo0P00000/0?ooo`800000C03oool00600oooo
00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`2[0?ooo`030000003oool0oooo02/0oooo
00<000000?ooo`3oool0B@3oool005l0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?oo
o`1<0?oooa@00000C@3oool2000002/0oooo00<000000?ooo`3oool0B@3oool005l0oooo00<00000
0?ooo`3oool08`3oool200000400oooo3`00000D0?ooo`h00000@@3oool2000002X0oooo0P000019
0?ooo`00GP3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo03P0oooo2000000a0?oo
o`T00000>P3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo04L0oooo001M0?ooo`03
0000003oool0oooo02<0oooo00<000000?ooo`3oool0<`3oool600000480oooo1000000g0?ooo`03
0000003oool0oooo02P0oooo0P0000170?ooo`00G03oool00`000000oooo0?ooo`0R0?ooo`800000
<03oool6000004`0oooo1P00000b0?ooo`800000:@3oool00`000000oooo0?ooo`150?ooo`00F`3o
ool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo02/0oooo1@00001H0?ooo`800000<P3o
ool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo04@0oooo001J0?ooo`030000003oool0
oooo0240oooo0P00000/0?ooo`800000G`3oool4000002l0oooo00<000000?ooo`3oool09`3oool0
0`000000oooo0?ooo`130?ooo`00F@3oool00`000000oooo0?ooo`0P0?ooo`800000:P3oool40000
06D0oooo1@00000[0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0@P3oool005P0
oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0X0?ooo`800000KP3oool2000002X0
oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`110?ooo`00F03oool00`000000oooo
0?ooo`0O0?ooo`030000003oool0oooo02D0oooo1000001b0?ooo`800000:@3oool2000002P0oooo
00<000000?ooo`3oool0@03oool005L0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`0T0?ooo`800000N03oool2000002T0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?oo
o`0o0?ooo`00EP3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo02<0oooo0P00001l
0?ooo`800000:03oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo03h0oooo001E0?oo
o`030000003oool0oooo01l0oooo00<000000?ooo`3oool08P3oool200000800oooo0P00000W0?oo
o`8000009P3oool00`000000oooo0?ooo`0n0?ooo`00E03oool200000200oooo00<000000?ooo`3o
ool0803oool3000008@0oooo0`00000V0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3o
ool0?@3oool005<0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0O0?ooo`800000
RP3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0
?03oool005<0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0N0?ooo`800000S@3o
ool2000002D0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0k0?ooo`00DP3oool0
0`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01d0oooo0P00002A0?ooo`800000903oool0
0`000000oooo0?ooo`0T0?ooo`030000003oool0oooo03X0oooo001A0?ooo`030000003oool0oooo
01h0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`2C0?ooo`8000008`3oool00`00
0000oooo0?ooo`0T0?ooo`030000003oool0oooo03T0oooo001A0?ooo`030000003oool0oooo01d0
oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`2E0?ooo`030000003oool0oooo0280
oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0i0?ooo`00D03oool00`000000oooo
0?ooo`0M0?ooo`030000003oool0oooo01`0oooo0P00002I0?ooo`<000008P3oool00`000000oooo
0?ooo`0S0?ooo`030000003oool0oooo03P0oooo001?0?ooo`030000003oool0oooo01d0oooo00<0
00000?ooo`3oool06`3oool2000009h0oooo00<000000?ooo`3oool0803oool00`000000oooo0?oo
o`0R0?ooo`030000003oool0oooo03P0oooo001?0?ooo`030000003oool0oooo01`0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`2O0?ooo`8000008@3oool00`000000oooo0?ooo`0R
0?ooo`800000>03oool004h0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo03D0oooo?P00000_0?ooo`030000003oool0oooo01l0oooo00<000000?oo
o`3oool08P3oool00`000000oooo0?ooo`0f0?ooo`00C@3oool00`000000oooo0?ooo`0L0?ooo`03
0000003oool0oooo01X0oooo0P00000X0?oooa000000?P3oool8000002P0oooo0P00000P0?ooo`03
0000003oool0oooo0280oooo0P00000f0?ooo`00C@3oool00`000000oooo0?ooo`0L0?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool08P3oool6000005D0oooo1000000U0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0d0?ooo`00C@3o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01T0oooo0P00000Q0?ooo`@00000G`3o
ool400000280oooo0P00000O0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0=03o
ool004`0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01`0oooo1000001W0?ooo`8000008@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0
oooo0280oooo00<000000?ooo`3oool0<`3oool004/0oooo00<000000?ooo`3oool06`3oool00`00
0000oooo0?ooo`0I0?ooo`8000007@3oool2000006d0oooo0P00000P0?ooo`8000007P3oool00`00
0000oooo0?ooo`0R0?ooo`030000003oool0oooo03<0oooo001:0?ooo`8000007@3oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo01/0oooo0P00001a0?ooo`<000007P3oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0<P3oool004T0oooo
00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01X0oooo
0P00001f0?ooo`8000007@3oool2000001h0oooo00<000000?ooo`3oool08P3oool00`000000oooo
0?ooo`0a0?ooo`00B@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool06@3oool2000007X0oooo0P00000M0?ooo`030000003oool0oooo01`0oooo00<0
00000?ooo`3oool08P3oool00`000000oooo0?ooo`0`0?ooo`00B03oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool2000007h0oooo0P00000L
0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0`
0?ooo`00B03oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool05P3oool300000880oooo0P00000K0?ooo`030000003oool0oooo01`0oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`0_0?ooo`00A`3oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo01L0oooo00<000000?ooo`3oool05@3oool2000008L0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3o
ool0;P3oool004L0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0G0?ooo`030000
003oool0oooo01H0oooo00<000000?ooo`3oool0Q`3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0^0?ooo`00AP3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool05@3o
ool2000008/0oooo0P00000I0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0803o
ool00`000000oooo0?ooo`0^0?ooo`00AP3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01H0oooo0P00000F0?ooo`030000003oool0oooo08d0oooo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0;@3oool0
04D0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool0S`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
01/0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0/0?ooo`00A@3oool00`000000
oooo0?ooo`0I0?ooo`800000603oool00`000000oooo0?ooo`0C0?ooo`800000L`3oool3000001d0
oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0240
oooo00<000000?ooo`3oool0:`3oool004@0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool05`3oool<000004L0oooo
2@0000030?ooo`H00000603oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0[0?ooo`00A03oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`0D0?ooo`@00000303oool6000003/0oooo1P00000B0?ooo`<000005P3oool2000001P0oooo00<0
00000?ooo`3oool06P3oool200000280oooo00<000000?ooo`3oool0:P3oool004<0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01<0oooo00<00000
0?ooo`3oool04`3oool2000001H0oooo1000000c0?ooo`@000006`3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0P
0?ooo`030000003oool0oooo02X0oooo00130?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0140oooo0`00000L0?oo
o`H00000:@3oool400000200oooo0`00000D0?ooo`8000005`3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo0200oooo00<000000?ooo`3oool0:@3oool00480oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3o
ool0403oool00`000000oooo0?ooo`0S0?ooo`D00000803oool4000002L0oooo0P00000C0?ooo`03
0000003oool0oooo01D0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0P0?ooo`03
0000003oool0oooo02T0oooo00120?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0
5@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00l0oooo0P00000[0?ooo`P00000
3`3oool9000002d0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0E0?ooo`800000
6`3oool200000280oooo00<000000?ooo`3oool0:03oool00440oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`0c0?ooo`l00000=`3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0P0?ooo`030000003o
ool0oooo02P0oooo00110?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool05@3oool0
0`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00h0oooo0P00001m0?ooo`8000004@3oool2
000001L0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0
oooo02L0oooo00100?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool05@3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0O`3oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool06`3oool00`00
0000oooo0?ooo`0O0?ooo`030000003oool0oooo02L0oooo00100?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0A0?ooo`8000003`3oool2000000X0
oooo2@2o_kmJ0?ooo`P0_kno3@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01@0
oooo0P00000K0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool09P3oool00400oooo
00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0180oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`080?ooo``0_knoEP3oool;0;no_``0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01T0oooo
00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0V0?ooo`00?`3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo00P0oooo3`2o_kmA0?ooo`l0_kno303oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo02H0oooo000o0?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00d0oooo0P000009
0?oooa40_knoC`3ooolA0;no_``0oooo00<000000?ooo`3oool03`3oool2000001H0oooo00<00000
0?ooo`3oool06@3oool200000240oooo00<000000?ooo`3oool09@3oool003l0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0180oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`080?oooa<0_knoC@3ooolC0;no_`/0oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool07`3oool00`000000oooo0?ooo`0U0?ooo`00?`3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01@0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0<0?ooo`80
00002@3ooolE0;no_d/0oooo5@2o_kl;0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3o
ool0903oool003h0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo0180oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`080?oooaH0_kno
B@3ooolG0;no_`/0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0T0?ooo`00?P3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00P0oooo5`2o_km70?oooaP0_kno2`3o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo02<0oooo000m0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool0203ooolI0;no_dD0oooo6P2o_kl;0?ooo`030000003oool0
oooo00l0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo01l0oooo00<000000?ooo`3oool08`3oool003d0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`080?ooo`030000002o_kl0_kno01P0_kno@`3ooolK0;no_`030000003oool0oooo
00T0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0D0?ooo`8000006`3oool00`00
0000oooo0?ooo`0O0?ooo`030000003oool0oooo0280oooo000l0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool02@3ooolL0;no_d80oooo702o_kl;0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01l0
oooo00<000000?ooo`3oool08P3oool003`0oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`090?oooad0_kno@03ooolM0;no_``0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool07`3oool00`000000oooo
0?ooo`0R0?ooo`00?03oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01D0oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00T0oooo7`2o
_klm0?ooo`030000002o_kl0_kno01d0_kno2`3oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0N0?ooo`030000003o
ool0oooo0280oooo000l0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02@3ooolO
0;no_`800000>`3oool00`000000_kno0;no_`0M0;no_``0oooo00<000000?ooo`3oool03`3oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool07`3oool0
0`000000oooo0?ooo`0Q0?ooo`00>`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
00T0oooo8@2o_kl00`3oool000000?ooo`0g0?ooo`800000802o_kl<0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
01l0oooo00<000000?ooo`3oool08@3oool003/0oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03@3oool00`000000
oooo0?ooo`090?ooob80_kno00<0oooo0000003oool0=@3oool00`000000oooo0;no_`0Q0;no_``0
oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0P0?ooo`00>P3oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo00X0oooo8`2o_kl00`3oool000000?ooo`0c0?ooo`030000
003oool0_kno0280_kno303oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0200oooo
000j0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0503oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02P3ooolS0;no_`80oooo00<0
00000?ooo`3oool0<03oool00`000000oooo0?ooo`0S0;no_`d0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`0O0?ooo`00>P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0
oooo01@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo00X0oooo902o_kl20?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0902o_kl=
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool003X0oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`090?ooobH0_kno0P3oool00`000000oooo0?ooo`0/0?oo
o`030000003oool0oooo02H0_kno3@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo
01l0oooo000i0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02P3ooolV0;no_`<0
oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0W0;no_`d0oooo00<000000?ooo`3o
ool03`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool07P3oool00`000000oooo0?ooo`0O0?ooo`00>@3oool00`000000oooo0?ooo`0G0?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0>0?ooo`030000
003oool0oooo00X0oooo9`2o_kl30?ooo`800000:03oool2000000<0oooo9`2o_kl>0?ooo`030000
003oool0oooo00l0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo01h0oooo00<000000?ooo`3oool07P3oool003T0oooo00<000000?ooo`3oool05`3o
ool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0;0?ooobP0_kno0`3oool00`000000oooo0?ooo`0V0?ooo`040000003o
ool0oooo0?ooobP0_kno3P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01h0oooo
000i0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`0A0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02P3ooolY0;no_`@0oooo00<0
00000?ooo`3oool0903oool00`000000oooo0?ooo`020?ooobT0_kno3@3oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo01d0oooo000h0?ooo`030000003oool0oooo01L0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool02`3ooolZ0;no_`<0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`02
0?ooobX0_kno3@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool0603oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01d0oooo000h0?oo
o`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool02`3ooolZ0;no_`@0oooo00<000000?oo
o`3oool08P3oool00`000000oooo0?ooo`020?ooobX0_kno3P3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0N0?oo
o`030000003oool0oooo01d0oooo000h0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3o
ool02`3oool[0;no_`<0oooo00<000000?ooo`3oool08P3oool010000000oooo0?ooo`3oool[0;no
_`h0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0M0?ooo`00>03oool00`000000
oooo0?ooo`0F0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo00X0oooo;02o_kl40?ooo`030000003oool0oooo0200
oooo00@000000?ooo`3oool0oooo;02o_kl00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
01h0oooo00<000000?ooo`3oool0703oool003L0oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`090?ooo`030000002o_kl0_kno02/0_kno0`3oool00`000000oooo0?ooo`0O0?ooo`80
00000`3oool]0;no_`030000003oool0oooo00/0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool07P3oool00`000000
oooo0?ooo`0L0?ooo`00=`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00T0oooo
00<000000?ooo`2o_kl0;@2o_kl30?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0
0P3oool^0;no_`030000003oool0oooo00/0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0C0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool07P3oool00`000000oooo
0?ooo`0L0?ooo`00=`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`2o_kl0;@2o_kl40?ooo`800000703oool2000000@0oooo;P2o_kl00`3oool000000?oo
o`0<0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0503oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool06`3oool003L0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0_kno02h0_kno1@3oool2
000001T0oooo00<000000?ooo`3oool00`3oool_0;no_`030?ooo`000000oooo00`0oooo00<00000
0?ooo`3oool03`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool07`3oool00`000000oooo0?ooo`0K0?ooo`00=`3oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo00X0oooo00<000000?ooo`2o_kl0;P2o_kl60?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool0103oool_0;no_`030?ooo`000000oooo00`0oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool07P3oool00`000000oooo0?ooo`0K0?ooo`00=P3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`2o_kl0;`2o_kl60?ooo`800000503oool2000000H0
oooo<02o_kl00`3oool000000?ooo`0<0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3o
ool06`3oool003H0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo0140oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0_kno02l0_kno203oool200000140oooo00<000000?ooo`3oool01@3ooola0;no_`030?oo
o`000000oooo00d0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0J0?ooo`00=P3o
ool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00X0oooo00<000000;no_`2o_kl0<02o
_kl90?ooo`8000003@3oool2000000P0oooo<P2o_kl00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01h0oooo00<000000?ooo`3oool06P3oool003H0oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0:0?ooo`030000002o_kl0_kno0340_kno2P3oool2000000L0oooo10000009
0?oooc<0_kno00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo01@0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0N0?ooo`030000003oool0
oooo01X0oooo000e0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02`3oool00`00
0000_kno0;no_`0a0;no_``0oooo1`00000=0?oooc<0_kno00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0N0?ooo`030000003oool0oooo01X0oooo000e0?ooo`030000003oool0oooo01L0
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00d0
oooo00<000000?ooo`3oool02`3oool00`000000_kno0;no_`0b0;no_ah0oooo=02o_kl00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool06@3oool003D0oooo
00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0;0?ooo`030000002o_kl0_kno0380_kno
7P3ooold0;no_`030000003oool0oooo00`0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool07P3oool00`000000oooo
0?ooo`0I0?ooo`00=@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00/0oooo00<0
00000;no_`2o_kl0<`2o_klL0?ooocD0_kno00<000000?ooo`3oool03@3oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0N0?ooo`030000003oool0oooo01T0oooo000e0?ooo`030000003oool0oooo01H0oooo00<00000
0?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool02`3oool00`000000_kno0;no_`0c0;no_a`0oooo=@2o_kl00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool06@3oool003D0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0140oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0;0?ooocL0_kno6P3ooolf0;no_`030000003oool0oooo
00d0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0H0?ooo`00=03oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo00`0oooo>02o_klH0?ooocP0_kno3`3oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool06@3oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo01P0oooo000d0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool0303ooolh0;no_aP0oooo>02o_kl?0?ooo`030000003oool0oooo0140oooo
00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01h0oooo
00<000000?ooo`3oool0603oool003@0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?oo
o`0C0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0<0?ooocT0_kno5P3oooli0;no_`l0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?oo
o`0H0?ooo`00=03oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00`0oooo>@2o_klF
0?ooocT0_kno403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool0603oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01P0oooo000d0?oo
o`030000003oool0oooo01H0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo00h0oooo00<000000?ooo`3oool0303ooolj0;no_a@0oooo>P2o_kl@0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01h0oooo00<000000?ooo`3oool0603oool003@0oooo00<000000?ooo`3o
ool05P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`0<0?ooocX0_kno503ooolj0;no_a00oooo00<000000?ooo`3o
ool04@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool07P3oool00`000000oooo0?ooo`0G0?ooo`00=03oool00`000000oooo0?ooo`0F0?ooo`030000
003oool0oooo01<0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`030000
003oool0oooo00`0oooo>`2o_klB0?oooc/0_kno403oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo01<0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?ooo`030000
003oool0oooo01L0oooo000d0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04`3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0303o
oolk0;no_a40oooo?02o_kl@0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0503o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool05`3o
ool003@0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0<0?oooc`0_kno403oooll
0;no_a00oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0G0?ooo`00<`3oool00`00
0000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo00`0oooo?02o_kl?0?ooocd0_kno403oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0N0?ooo`030000003oool0oooo01L0oooo000c0?ooo`030000003oool0oooo01L0
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool0303ooolm0;no_`h0oooo?@2o_kl@0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01h0
oooo00<000000?ooo`3oool05`3oool003<0oooo00<000000?ooo`3oool05`3oool00`000000oooo
0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`0=0?ooocd0_kno3@3oooln0;no_a40oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07P3oool00`000000oooo
0?ooo`0G0?ooo`00<`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00d0oooo?P2o
_kl<0?oooch0_kno4@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01@0oooo00<0
00000?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01H0oooo000c
0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03@3oooln0;no_`/0oooo?`2o_klA
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0I
0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool05P3oool003<0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0140oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0=0?ooocl0_kno2P3ooolo0;no_a40oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool07P3oool00`000000oooo0?ooo`0F0?ooo`00<`3oool00`000000oooo0?ooo`0F0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`03
0000003oool0oooo00d0oooo?`2o_kl90?oood00_kno4@3oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?ooo`03
0000003oool0oooo01H0oooo000c0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
303ooom10;no_`P0oooo@02o_klA0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0
5P3oool003<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0<0?oood40_kno1`3o
oom10;no_a40oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0D0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0F0?ooo`00<`3oool0
0`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00`0oooo@P2o_kl60?oood40_kno4@3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01H0oooo000b0?ooo`030000003oool0oooo
01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool0303ooom20;no_`D0oooo@P2o_klA0?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
01h0oooo00<000000?ooo`3oool05P3oool00380oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`0<0?oood<0_kno103ooom20;no_a40oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07P3oool00`000000
oooo0?ooo`0F0?ooo`00<P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00`0oooo
@`2o_kl30?oood<0_kno4@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01H0oooo
000b0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0303ooom40;no_`80oooo@`2o
_klA0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0503oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool05@3oool00380oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool03`3oool00`000000oooo0?ooo`0<0?oood@0_kno00<0oooo0;no_`2o_kl0@P2o_klA
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0I
0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool05@3oool00380oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`0<0?ooohT0_kno4@3oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?oo
o`030000003oool0oooo01D0oooo000b0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3o
ool04`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3o
ool0303ooom40;no_`030?ooo`2o_kl0_kno0480_kno4@3oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo01@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?ooo`03
0000003oool0oooo01D0oooo000b0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
303ooom40;no_`030?ooo`2o_kl0_kno0480_kno4@3oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?ooo`030000
003oool0oooo01D0oooo000b0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool04`3o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0303o
oom30;no_`<0oooo@`2o_klA0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0503o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool05@3o
ool00380oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0<0?oood<0_kno0`3ooom3
0;no_a40oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo01T0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0E0?ooo`00<P3oool00`00
0000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo00`0oooo@P2o_kl50?oood80_kno4@3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`0N0?ooo`030000003oool0oooo01D0oooo000b0?ooo`030000003oool0oooo01L0
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool0303ooom20;no_`D0oooo@P2o_klA0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01h0
oooo00<000000?ooo`3oool05@3oool00380oooo00<000000?ooo`3oool05`3oool00`000000oooo
0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`0<0?oood40_kno1`3ooom10;no_a40oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool07P3oool00`000000oooo
0?ooo`0E0?ooo`00<P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00`0oooo@@2o
_kl70?oood40_kno4@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01@0oooo00<0
00000?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01D0oooo000b
0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0303ooom00;no_`T0oooo@02o_klA
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0I
0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool05@3oool00380oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0140oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0=0?ooocl0_kno2@3ooom00;no_a40oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool07P3oool00`000000oooo0?ooo`0E0?ooo`00<P3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`03
0000003oool0oooo00d0oooo?`2o_kl:0?ooocl0_kno4@3oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo01@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?ooo`03
0000003oool0oooo01D0oooo000b0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
3@3oooln0;no_`/0oooo?`2o_klA0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0
5@3oool00380oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0=0?oooch0_kno303o
ooln0;no_a40oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0D0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0E0?ooo`00<P3oool0
0`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00d0oooo?@2o_kl=0?oooch0_kno4@3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01D0oooo000b0?ooo`030000003oool0oooo
01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool03@3ooolm0;no_`h0oooo?@2o_klA0?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
01h0oooo00<000000?ooo`3oool05@3oool00380oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`0=0?oooc`0_kno3`3ooolm0;no_a40oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07`3oool00`000000
oooo0?ooo`0E0?ooo`00<`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00d0oooo
?02o_kl@0?oooc`0_kno4@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01D0oooo
000c0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03@3ooolk0;no_a40oooo?02o
_klA0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool05@3oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool05@3oool003<0oooo00<00000
0?ooo`3oool05P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool03`3oool00`000000oooo0?ooo`0<0?oooc/0_kno4P3ooolk0;no_a00oooo0P00000B
0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0O
0?ooo`030000003oool0oooo01D0oooo000c0?ooo`030000003oool0oooo01H0oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool0303ooolj0;no_a@0oooo>P2o_kl@0?ooo`030000003oool0oooo0140oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01l0oooo00<000000?oo
o`3oool05@3oool003<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0D0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0<0?ooocX0
_kno503ooolj0;no_a00oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0D0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0E0?ooo`00
<`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00`0oooo>@2o_klF0?ooocT0_kno
403oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01D0oooo000c0?ooo`030000003o
ool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0303oooli0;no_aH0oooo>@2o_kl@0?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01l0oooo00<000000?ooo`3oool05@3oool003<0oooo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0<0?ooocP0_kno603ooolh0;no_a00oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool07P3oool0
0`000000oooo0?ooo`0F0?ooo`00<`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
01<0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
00`0oooo>02o_klH0?ooocP0_kno403oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo
01H0oooo000c0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0303ooolg0;no_aX0
oooo=P2o_kl00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool05@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3o
ool05P3oool003<0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo0100oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0<0?ooo`030000
002o_kl0_kno03<0_kno703ooole0;no_`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0
7P3oool00`000000oooo0?ooo`0F0?ooo`00<`3oool00`000000oooo0?ooo`0G0?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0>0?ooo`030000003o
ool0oooo00`0oooo00<000000;no_`2o_kl0<`2o_klL0?ooocD0_kno00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01H0oooo000d0?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool02`3oool00`000000_kno0;no_`0b0;no_ah0oooo=02o_kl0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool05P3oool0
03@0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0;0?ooo`030000002o_kl0_kno
0380_kno7P3ooold0;no_`030000003oool0oooo00d0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0E0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool07`3oool00`00
0000oooo0?ooo`0F0?ooo`00=03oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00/0
oooo00<000000;no_`2o_kl0<@2o_klP0?oooc<0_kno00<000000?ooo`3oool03@3oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool06@3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo01H0oooo000d0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool02`3oool00`000000_kno0;no_`0a0;no_`d0oooo1@00000>0?oooc<0_kno
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01L0oooo
000d0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02`3oool00`000000_kno0;no
_`0`0;no_`X0oooo100000050?ooo`D000002P3ooolb0;no_`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool07P3oool00`000000oooo0?ooo`0G0?ooo`00=03oool00`000000oooo0?ooo`0G
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`2o_kl0;`2o_kl;0?ooo`030000003oool0
oooo00/0oooo0P0000080?oooc40_kno00<0oooo0000003oool03@3oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0O
0?ooo`030000003oool0oooo01L0oooo000d0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool02P3oool00`000000oooo0;no_`0_0;no_`T0oooo0P00000@0?ooo`8000001`3oool`0;no
_`030?ooo`000000oooo00d0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0D0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0G0?oo
o`00=03oool00`000000oooo0?ooo`0G0?ooo`8000005@3oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0;no_`0^0;no_`P0
oooo0P00000C0?ooo`030000003oool0oooo00H0oooo;`2o_kl00`3oool000000?ooo`0=0?ooo`03
0000003oool0oooo0140oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01l0oooo00<000000?ooo`3oool05`3oool003@0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0_kno02h0_kno1P3oool2000001H0oooo
0P0000060?ooobl0_kno00<0oooo0000003oool03@3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo01D0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0O0?ooo`030000
003oool0oooo01L0oooo000e0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0;no_`0]0;no_`H0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?oo
o`050?ooobh0_kno00<0oooo0000003oool03@3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo01D0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0O0?ooo`030000003o
ool0oooo01L0oooo000e0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0;no_`0]0;no_`D0oooo00<000000?ooo`3oool06@3oool2000000D0oooo;P2o_kl0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool05@3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool05`3oool0
03D0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0:0?ooo`030000002o_kl0_kno
02/0_kno1@3oool00`000000oooo0?ooo`0L0?ooo`800000103oool]0;no_`030000003oool0oooo
00`0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo
01T0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0G0?ooo`00=@3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo00/0oooo;02o_kl40?ooo`800000803oool00`000000
oooo0?ooo`020?ooob`0_kno00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo01@0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0O0?ooo`03
0000003oool0oooo01P0oooo000e0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
303oool[0;no_`@0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`020?ooob/0_kno
3`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01P0oooo000f0?ooo`030000003o
ool0oooo01L0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool0303ooolZ0;no_`@0oooo00<000000?ooo`3oool08P3o
ool00`000000oooo0?ooo`020?ooobX0_kno3`3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo01D0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0P0?ooo`030000003o
ool0oooo01P0oooo000f0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02`3ooolZ
0;no_`@0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`020?ooobX0_kno3`3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`0P0?ooo`030000003oool0oooo01P0oooo000f0?ooo`030000003oool0oooo
01L0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool02`3ooolY0;no_`@0oooo00<000000?ooo`3oool08`3oool00`00
0000oooo0?ooo`030?ooobT0_kno3P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo
01T0oooo000f0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02`3ooolX0;no_`@0
oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`020?ooobP0_kno3`3oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo01T0oooo000f0?ooo`030000003oool0oooo01P0oooo
00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool02`3ooolW0;no_`<0oooo0P00000X0?ooo`030000003oool0oooo0080oooo
9`2o_kl>0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool05@3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool06@3oool003L0oooo00<0
00000?ooo`3oool05`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0100oooo0P00
000@0?ooo`030000003oool0oooo00X0oooo9P2o_kl40?ooo`030000003oool0oooo02P0oooo00@0
00000?ooo`3oool0oooo9`2o_kl>0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
5@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0
6@3oool003L0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0E0?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0:0?ooobH0_kno0P3o
ool2000002`0oooo00@000000?ooo`3oool0oooo9P2o_kl=0?ooo`030000003oool0oooo0140oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0200oooo
00<000000?ooo`3oool06@3oool003L0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0;0?ooob@0_kno0P3oool00`000000oooo0?ooo`0]0?ooo`8000000P3ooolT0;no_`h0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool0803oool00`000000oooo0?ooo`0I0?ooo`00=`3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo00/0oooo8`2o_kl20?ooo`030000003oool0oooo0300oooo00<00000
0?ooo`3oool08`2o_kl=0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool05@3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool06P3oool0
03P0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0:0?ooob<0_kno00<0oooo0000
003oool0<`3oool00`000000oooo0;no_`0R0;no_`d0oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0E0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0803oool00`00
0000oooo0?ooo`0J0?ooo`00>03oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00X0
oooo8P2o_kl00`3oool000000?ooo`0e0?ooo`030000003oool0_kno0240_kno3@3oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`0P0?ooo`030000003oool0oooo01X0oooo000h0?ooo`030000003oool0oooo01P0oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool02@3ooolQ0;no_`030?ooo`000000oooo03L0oooo00<000000?ooo`2o_kl0
7`2o_kl=0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool05@3oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool06P3oool003P0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`0:0?oooal0_kno00<0oooo0000003oool0>@3o
ool00`000000oooo0;no_`0N0;no_``0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`0E0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0803oool00`000000oooo0?oo
o`0K0?ooo`00>@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool04P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00T0oooo7`2o_kl0
0`000000oooo0?ooo`0j0?ooo`030000002o_kl0_kno01d0_kno303oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0P
0?ooo`030000003oool0oooo01/0oooo000i0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool05@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool02P3ooolM0;no_cl0oooo00<000000;no_`2o_kl06`2o_kl=0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
0200oooo00<000000?ooo`3oool0703oool003T0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0E0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`090?oooa`0_kno@@3oool00`000000_kno0;no_`0J0;no_``0oooo00<000000?ooo`3o
ool04@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool0803oool00`000000oooo0?ooo`0L0?ooo`00>@3oool00`000000oooo0?ooo`0H0?ooo`030000
003oool0oooo01D0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0>0?ooo`030000
003oool0oooo00T0oooo00<000000;no_`2o_kl0602o_km30?oooa/0_kno00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo01`0oooo000j0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool02@3ooolI0;no_dD0oooo6P2o_kl<0?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo0240oooo00<000000?ooo`3oool0703oool003X0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`090?ooo`030000002o_kl0_kno01D0_knoA`3ooolH0;no_`030000003oool0
oooo00X0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
oooo01X0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0L0?ooo`00>P3oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo00T0oooo5P2o_km90?oooaL0_kno303oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool06`3oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo01d0oooo000j0?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00l0
oooo00<000000?ooo`3oool0203ooolE0;no_d/0oooo5@2o_kl=0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0200
oooo00<000000?ooo`3oool07@3oool003X0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`090?oooa<0_knoC@3ooolC0;no_`d0oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`0F0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`0M0?ooo`00>`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01D0oooo00<0
00000?ooo`3oool04P3oool200000100oooo00<000000?ooo`3oool02@3ooolA0;no_dl0oooo4@2o
_kl=0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool05P3oool00`000000oooo0?oo
o`0K0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool07@3oool003/0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0180oooo00<00000
0?ooo`3oool03P3oool00`000000oooo0?ooo`0:0?ooo`l0_knoD@3oool?0;no_`d0oooo00<00000
0?ooo`3oool04P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01/0oooo00<00000
0?ooo`3oool08@3oool00`000000oooo0?ooo`0N0?ooo`00?03oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo01D0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0>0?oo
o`8000002`3oool<0;no_eH0oooo2`2o_kl>0?ooo`030000003oool0oooo0140oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0240oooo00<000000?oo
o`3oool07P3oool003`0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0E0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0:0?ooo`T0
_knoF@3oool00`000000_kno0;no_`060;no_`h0oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool08@3oool00`000000
oooo0?ooo`0O0?ooo`00?03oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0?0?ooo`800000PP3oool00`000000oooo
0?ooo`0B0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool06`3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo01l0oooo000l0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`3oool0OP3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool0703oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo01l0oooo000m
0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0O03oool00`000000oooo0?ooo`0B
0?ooo`030000003oool0oooo01L0oooo0P00000M0?ooo`030000003oool0oooo0280oooo00<00000
0?ooo`3oool07`3oool003d0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo01<0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`1j0?oo
o`030000003oool0oooo01<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0L0?oo
o`030000003oool0oooo0280oooo00<000000?ooo`3oool0803oool003h0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3o
ool04@3oool00`000000oooo0?ooo`1g0?ooo`8000005@3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo01`0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0P0?ooo`00
?P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`0A0?ooo`800000<@3oool?000003H0oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`0Q0?ooo`00?`3oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01L0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0B0?ooo`8000009P3o
ool9000000l0oooo2@00000Z0?ooo`<000005P3oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo01`0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0Q0?ooo`00?`3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool05@3oool0
0`000000oooo0?ooo`0C0?ooo`8000007P3oool600000240oooo1P00000R0?ooo`800000603oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool08P3oool0
0`000000oooo0?ooo`0R0?ooo`00?`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
01L0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0D0?ooo`@00000503oool60000
02d0oooo1P00000J0?ooo`8000006@3oool00`000000oooo0?ooo`0H0?ooo`8000007P3oool00`00
0000oooo0?ooo`0R0?ooo`030000003oool0oooo0280oooo000o0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01P0
oooo5000000i0?ooo`L000003`3oool4000001X0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool08`3oool00400oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01@0oooo
0P00001/0?ooo`l000007@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01d0oooo
00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0S0?ooo`00@03oool00`000000oooo0?oo
o`0K0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?oo
o`2E0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?oo
o`0S0?ooo`030000003oool0oooo02<0oooo00110?ooo`030000003oool0oooo01/0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`0E0?ooo`800000U03oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0S
0?ooo`00@@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`2@0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool07P3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo02@0oooo00120?ooo`03
0000003oool0oooo01X0oooo0P00000I0?ooo`800000603oool2000008h0oooo0P00000L0?ooo`03
0000003oool0oooo01h0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0T0?ooo`00
@P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
5`3oool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
7`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo02@0oooo00120?ooo`030000003o
ool0oooo01`0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0G0?ooo`030000003o
ool0000008X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0N0?ooo`030000003o
ool0oooo02<0oooo00<000000?ooo`3oool09@3oool004<0oooo00<000000?ooo`3oool06`3oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo0P0000280?ooo`8000007@3oool0
0`000000oooo0?ooo`0N0?ooo`8000009@3oool00`000000oooo0?ooo`0U0?ooo`00@`3oool00`00
0000oooo0?ooo`0K0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool20000
08D0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo
02@0oooo00<000000?ooo`3oool09P3oool004@0oooo00<000000?ooo`3oool06`3oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo01X0oooo0P0000220?ooo`030000003oool0oooo01`0
oooo0P00000P0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool09P3oool004@0oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01/0oooo
0P00001n0?ooo`8000007P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo02@0oooo
00<000000?ooo`3oool09`3oool004D0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`0J0?ooo`800000703oool3000007X0oooo0P00000O0?ooo`8000008@3oool00`000000oooo0?oo
o`0T0?ooo`030000003oool0oooo02L0oooo00150?ooo`030000003oool0oooo01`0oooo00<00000
0?ooo`3oool06P3oool00`000000oooo0?ooo`0M0?ooo`800000MP3oool200000200oooo00<00000
0?ooo`3oool0803oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo02P0oooo00160?oo
o`030000003oool0oooo01`0oooo00<000000?ooo`3oool06P3oool2000001l0oooo0P00001a0?oo
o`<000008P3oool00`000000oooo0?ooo`0O0?ooo`8000009P3oool00`000000oooo0?ooo`0X0?oo
o`00AP3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool07`3oool2000006d0oooo0P00000T0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3o
ool09@3oool00`000000oooo0?ooo`0Y0?ooo`00A`3oool00`000000oooo0?ooo`0L0?ooo`030000
003oool0oooo01/0oooo0P00000Q0?ooo`@00000I@3oool4000002D0oooo00<000000?ooo`3oool0
803oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo02T0oooo00170?ooo`030000003o
ool0oooo01d0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0S0?ooo`@00000G`3o
ool2000002P0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0U0?ooo`030000003o
ool0oooo02X0oooo00180?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool06`3oool2
000002L0oooo1@00001F0?ooo`@00000:03oool2000002<0oooo00<000000?ooo`3oool09@3oool0
0`000000oooo0?ooo`0Z0?ooo`00B03oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo
01d0oooo00<000000?ooo`3oool0:@3oool8000004P0oooo1P00000/0?ooo`030000003oool0oooo
0240oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0[0?ooo`00B@3oool00`000000
oooo0?ooo`0M0?ooo`8000007P3oool00`000000oooo0?ooo`0`0?ooo`l00000<P3oool700000300
oooo0P00000S0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0;03oool004T0oooo
00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo03h0oooo
<P00000f0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool09P3oool00`000000oooo
0?ooo`0/0?ooo`00BP3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo01d0oooo00<0
00000?ooo`3oool0Y03oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo02H0oooo00<0
00000?ooo`3oool0;@3oool004X0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0M
0?ooo`800000X`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo02D0oooo00<00000
0?ooo`3oool0;P3oool004/0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0N0?oo
o`030000003oool0oooo09h0oooo0P00000U0?ooo`030000003oool0oooo02H0oooo00<000000?oo
o`3oool0;P3oool004`0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0O0?ooo`03
0000003oool0oooo09/0oooo0P00000V0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3o
ool0;P3oool004d0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0O0?ooo`800000
VP3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0
;`3oool004d0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0P0?ooo`030000003o
ool0oooo09H0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0W0?ooo`030000003o
ool0oooo0300oooo001>0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0803oool3
000009<0oooo0P00000W0?ooo`030000003oool0oooo02L0oooo0P00000b0?ooo`00CP3oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0S`3oool20000
02P0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`0b0?ooo`00C`3oool00`000000
oooo0?ooo`0P0?ooo`030000003oool0oooo0240oooo0`00002<0?ooo`800000:@3oool2000002T0
oooo00<000000?ooo`3oool0<P3oool004l0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`0S0?ooo`800000RP3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02P0oooo
00<000000?ooo`3oool0<`3oool00500oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`0T0?ooo`800000Q@3oool3000002T0oooo0P00000Z0?ooo`030000003oool0oooo03<0oooo001A
0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool09@3oool200000880oooo00<00000
0?ooo`3oool0:@3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo03@0oooo001A0?oo
o`030000003oool0oooo0240oooo0P00000X0?ooo`800000OP3oool2000002`0oooo00<000000?oo
o`3oool0:@3oool00`000000oooo0?ooo`0d0?ooo`00DP3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo02P0oooo0P00001j0?ooo`800000;@3oool00`000000oooo0?ooo`0Y0?ooo`03
0000003oool0oooo03D0oooo001C0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0
:@3oool2000007H0oooo0P00000^0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0
=P3oool005<0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0Z0?ooo`800000LP3o
ool2000002l0oooo00<000000?ooo`3oool0:@3oool2000003P0oooo001D0?ooo`030000003oool0
oooo0280oooo0P00000/0?ooo`@00000J@3oool500000300oooo00<000000?ooo`3oool0:@3oool0
0`000000oooo0?ooo`0h0?ooo`00E@3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo
02d0oooo0P00001U0?ooo`800000=03oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo
03P0oooo001E0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0;P3oool5000005`0
oooo1000000d0?ooo`800000;03oool00`000000oooo0?ooo`0i0?ooo`00EP3oool00`000000oooo
0?ooo`0T0?ooo`030000003oool0oooo0380oooo1000001F0?ooo`800000=`3oool00`000000oooo
0?ooo`0/0?ooo`030000003oool0oooo03T0oooo001G0?ooo`030000003oool0oooo02@0oooo00<0
00000?ooo`3oool0=@3oool4000004h0oooo1000000h0?ooo`030000003oool0oooo02`0oooo00<0
00000?ooo`3oool0>P3oool005P0oooo00<000000?ooo`3oool0903oool2000003T0oooo1P000012
0?ooo`H00000>P3oool2000002h0oooo00<000000?ooo`3oool0>`3oool005T0oooo00<000000?oo
o`3oool09@3oool00`000000oooo0?ooo`0l0?ooo`H00000=@3oool7000003l0oooo00<000000?oo
o`3oool0;@3oool00`000000oooo0?ooo`0l0?ooo`00FP3oool00`000000oooo0?ooo`0U0?ooo`80
0000@P3oool9000002H0oooo1P0000150?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3o
ool0?03oool005X0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`180?ooobH00000
BP3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo03d0oooo001K0?ooo`030000003o
ool0oooo02L0oooo00<000000?ooo`3oool0]@3oool200000300oooo00<000000?ooo`3oool0?P3o
ool005`0oooo00<000000?ooo`3oool09`3oool200000;@0oooo00<000000?ooo`3oool0;`3oool0
0`000000oooo0?ooo`0o0?ooo`00G@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo
0:l0oooo0P00000a0?ooo`030000003oool0oooo0400oooo001N0?ooo`030000003oool0oooo02P0
oooo00<000000?ooo`3oool0[P3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo0440
oooo001O0?ooo`030000003oool0oooo02P0oooo0P00002[0?ooo`<00000<@3oool00`000000oooo
0?ooo`120?ooo`00G`3oool00`000000oooo0?ooo`0Z0?ooo`800000Z03oool00`000000oooo0?oo
o`0a0?ooo`030000003oool0oooo04<0oooo001P0?ooo`800000;03oool200000:@0oooo0P00000c
0?ooo`800000A@3oool00640oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`2R0?oo
o`030000003oool0oooo0340oooo00<000000?ooo`3oool0A@3oool00680oooo0P00000/0?ooo`<0
0000WP3oool3000003<0oooo0P0000170?ooo`00H`3oool00`000000oooo0?ooo`0]0?ooo`030000
003oool0oooo09T0oooo0P00000e0?ooo`030000003oool0oooo04L0oooo001T0?ooo`800000;P3o
ool3000009H0oooo0P00000f0?ooo`800000B@3oool006H0oooo00<000000?ooo`3oool0;P3oool2
00000980oooo0P00000g0?ooo`030000003oool0oooo04T0oooo001V0?ooo`030000003oool0oooo
0300oooo0P00002>0?ooo`800000>@3oool00`000000oooo0?ooo`190?ooo`00I`3oool200000340
oooo0`00002:0?ooo`<00000>@3oool00`000000oooo0?ooo`1:0?ooo`00J03oool00`000000oooo
0?ooo`0b0?ooo`800000QP3oool2000003X0oooo0P00001=0?ooo`00J@3oool2000003@0oooo0P00
00220?ooo`800000?03oool00`000000oooo0?ooo`1<0?ooo`00JP3oool00`000000oooo0?ooo`0d
0?ooo`800000OP3oool2000003`0oooo0P00001?0?ooo`00J`3oool2000003H0oooo1000001h0?oo
o`800000?P3oool00`000000oooo0?ooo`1>0?ooo`00K03oool00`000000oooo0?ooo`0h0?ooo`80
0000LP3oool4000003h0oooo0P00001A0?ooo`00K@3oool2000003X0oooo0P00001^0?ooo`800000
@@3oool00`000000oooo0?ooo`1A0?ooo`00K`3oool00`000000oooo0?ooo`0i0?ooo`@00000I`3o
ool300000480oooo00<000000?ooo`3oool0DP3oool00700oooo00<000000?ooo`3oool0?03oool3
00000600oooo100000130?ooo`800000E@3oool00740oooo00<000000?ooo`3oool0?P3oool40000
05P0oooo100000160?ooo`030000003oool0oooo05D0oooo001b0?ooo`030000003oool0oooo0440
oooo1000001@0?ooo`@00000B@3oool00`000000oooo0?ooo`1F0?ooo`00L`3oool2000004D0oooo
100000180?ooo`@00000C03oool00`000000oooo0?ooo`1G0?ooo`00M@3oool00`000000oooo0?oo
o`160?ooo`H00000?P3oool4000004l0oooo00<000000?ooo`3oool0F03oool007H0oooo00<00000
0?ooo`3oool0B`3oool7000002h0oooo2@00001A0?ooo`800000F`3oool007L0oooo0P00001B0?oo
o`X00000703oool8000005T0oooo00<000000?ooo`3oool0F`3oool007T0oooo00<000000?ooo`3o
ool0F@3ooolL00000600oooo00<000000?ooo`3oool0G03oool007X0oooo0P00003C0?ooo`800000
G`3oool007`0oooo00<000000?ooo`3oool0c`3oool00`000000oooo0?ooo`1O0?ooo`00O@3oool2
00000<d0oooo0P00001R0?ooo`00OP3oool200000</0oooo00<000000?ooo`3oool0HP3oool00800
oooo0P0000370?ooo`800000I@3oool00880oooo00<000000?ooo`3oool0``3oool2000006H0oooo
00230?ooo`800000`@3oool2000006P0oooo00250?ooo`030000003oool0oooo0;`0oooo0P00001Z
0?ooo`00QP3oool200000;X0oooo0P00001/0?ooo`00R03oool200000;P0oooo00<000000?ooo`3o
ool0J`3oool008X0oooo0P00002d0?ooo`800000KP3oool008/0oooo00<000000?ooo`3oool0/03o
ool200000700oooo002<0?ooo`800000[P3oool200000780oooo002>0?ooo`800000ZP3oool20000
07@0oooo002@0?ooo`800000YP3oool2000007H0oooo002B0?ooo`800000XP3oool2000007P0oooo
002D0?ooo`800000WP3oool2000007X0oooo002F0?ooo`<00000V@3oool2000007`0oooo002I0?oo
o`@00000TP3oool3000007h0oooo002M0?ooo`800000SP3oool200000840oooo002O0?ooo`800000
R03oool4000008<0oooo002Q0?ooo`800000Q03oool2000008L0oooo002S0?ooo`@00000OP3oool2
000008T0oooo002W0?ooo`800000N03oool4000008/0oooo002Y0?ooo`800000M03oool2000008l0
oooo002[0?ooo`D00000J`3oool400000940oooo002`0?ooo`@00000HP3oool5000009D0oooo002d
0?ooo`@00000F`3oool3000009X0oooo002h0?ooo`@00000E03oool3000009d0oooo002l0?ooo`D0
0000B@3oool600000:00oooo00310?ooo`D00000@03oool400000:H0oooo00360?ooo`P00000;`3o
ool900000:X0oooo003>0?ooo`T00000703oool:00000;<0oooo003G0?oooa`00000_@3oool00?l0
oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0
oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0
oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0
oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0
oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00001
\
\>"],
  ImageRangeCache->{{{0, 431}, {431, 0}} -> {18.7997, 18.7997, 0.217172, \
0.217172}}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(DisplayTogether[\[IndentingNewLine]ContourPlot[
          f[x, y], {x, 22, 80}, {y, 20, 80}, PlotPoints \[Rule] 200, 
          AspectRatio \[Rule] Automatic, ContourShading \[Rule] False, 
          Contours \[Rule] Range[0.49, 0.99, 0.1], Frame \[Rule] False, 
          Epilog \[Rule] {GrayLevel[0.75], 
              Polygon[poly]}], \[IndentingNewLine]PlotGradientField[
          Evaluate[\(-f[x, y]\)], {x, 22, 80}, {y, 20, 80}, 
          PlotPoints \[Rule] 15, ScaleFactor \[Rule] 8]];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.02852 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.277737 0.0153246 -0.249087 0.0153246 [
[ 0 0 0 0 ]
[ 1 1.02852 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.02852 L
0 1.02852 L
closepath
clip
newpath
% Start of user PostScript
/mathtops {
gsave
MBeginOrig
moveto
MEndOrig
currentpoint
grestore
} bind def
/MAtocoords {
mathtops 4 2 roll mathtops
4 copy pop pop
3 -1 roll sub
/arry exch def
exch sub
/arrx exch def
arrx dup mul
arry dup mul
add sqrt
/arrl exch def
translate
} bind def
/MAarrowhead1 {
gsave
MAtocoords
arrl 0. eq
{ 0 0 Mdot }
{
[ arrx arrl div
arry arrl div
-1 arry mul arrl div
arrx arrl div
0 0 ]
concat
-0.02 0.005 moveto 0 0 lineto
-0.02 -0.005 lineto fill
-0.02 0.005 moveto 0 0 lineto
-0.02 -0.005 lineto
-0.02 0.005 lineto stroke
} ifelse
grestore
} def
/MAarrowzero1 {
gsave
mathtops translate
0 0 Mdot
grestore
} def
% End of user PostScript
0 g
.5 Mabswid
[ ] 0 setdash
.38546 .37577 m
.38992 .37547 L
.39439 .37605 L
.39527 .37622 L
.39886 .37775 L
.403 .38084 L
.40332 .38117 L
.40726 .38546 L
.40779 .38591 L
.41226 .38915 L
.41361 .39008 L
.41672 .39229 L
.41893 .3947 L
.42119 .39726 L
.42321 .39932 L
.42566 .40131 L
.42924 .40394 L
.43012 .40449 L
.43459 .4085 L
.43465 .40856 L
.43906 .4131 L
.43915 .41318 L
.44352 .41673 L
.44492 .4178 L
.44799 .42041 L
.44982 .42242 L
.45245 .42595 L
.453 .42704 L
.4552 .43167 L
.45692 .43468 L
.4581 .43629 L
.46139 .4398 L
.46268 .44091 L
.46585 .44323 L
.46892 .44553 L
.47032 .44661 L
.47479 .45003 L
.47496 .45015 L
.47925 .45259 L
.48372 .45459 L
.48417 .45477 L
.48819 .45593 L
.49265 .45672 L
.49712 .45689 L
.50159 .45645 L
.50605 .45542 L
.50811 .45477 L
.51052 .45378 L
.51499 .45151 L
.51729 .45015 L
Mistroke
.51945 .44867 L
.52335 .44553 L
.52392 .44507 L
.52838 .44179 L
.52957 .44091 L
.53285 .43788 L
.53414 .43629 L
.53695 .43167 L
.53732 .43093 L
.53925 .42704 L
.54178 .42304 L
.54242 .42242 L
.54625 .41864 L
.54735 .4178 L
.55072 .4152 L
.55311 .41318 L
.55518 .41109 L
.55758 .40856 L
.55965 .4065 L
.56292 .40394 L
.56412 .40308 L
.56858 .39968 L
.56899 .39932 L
.57305 .39499 L
.57328 .3947 L
.57752 .39059 L
.57868 .39008 L
.58198 .38775 L
.58493 .38546 L
.58645 .38391 L
.58922 .38084 L
.59092 .37892 L
.59538 .37671 L
.59691 .37622 L
.59985 .37561 L
.60431 .37541 L
.60878 .37608 L
.6092 .37622 L
.61325 .37821 L
.61626 .38084 L
.61771 .38288 L
.6199 .38546 L
.62218 .38752 L
.62563 .39008 L
.62665 .39088 L
.62955 .3947 L
.63054 .39932 L
.63085 .40394 L
.63072 .40856 L
.63092 .41318 L
Mistroke
.63111 .41549 L
.63146 .4178 L
.63228 .42242 L
.63401 .42704 L
.63558 .42932 L
.63813 .43167 L
.64005 .4333 L
.64334 .43629 L
.64451 .439 L
.64543 .44091 L
.64611 .44553 L
.64625 .45015 L
.64604 .45477 L
.64607 .45939 L
.64619 .46401 L
.64629 .46863 L
.64638 .47325 L
.64646 .47787 L
.64653 .48249 L
.64659 .48711 L
.64664 .49173 L
.64669 .49635 L
.64673 .50097 L
.64676 .50559 L
.64678 .51021 L
.6468 .51483 L
.64681 .51945 L
.64682 .52408 L
.64683 .5287 L
.64682 .53332 L
.64682 .53794 L
.6468 .54256 L
.64678 .54718 L
.64676 .5518 L
.64673 .55642 L
.64669 .56104 L
.64664 .56566 L
.64658 .57028 L
.64652 .5749 L
.64644 .57952 L
.64635 .58414 L
.64624 .58876 L
.64612 .59338 L
.64634 .598 L
.64634 .60262 L
.64587 .60724 L
.64451 .61079 L
.64394 .61186 L
.64018 .61649 L
.64005 .61659 L
Mistroke
.63558 .62086 L
.63539 .62111 L
.63295 .62573 L
.63196 .63035 L
.63133 .63497 L
.63111 .63661 L
.63087 .63959 L
.63104 .64421 L
.63088 .64883 L
.63015 .65345 L
.62783 .65807 L
.62665 .65911 L
.62218 .66244 L
.62185 .66269 L
.61771 .66709 L
.61754 .66731 L
.61325 .67181 L
.61308 .67193 L
.60878 .67434 L
.60431 .67502 L
.59985 .67482 L
.59538 .67363 L
.59235 .67193 L
.59092 .67108 L
.5874 .66731 L
.58645 .66633 L
.58242 .66269 L
.58198 .66241 L
.57752 .65928 L
.57605 .65807 L
.57305 .65533 L
.57132 .65345 L
.56858 .65072 L
.56631 .64883 L
.56412 .64726 L
.56017 .64421 L
.55965 .64382 L
.55526 .63959 L
.55518 .6395 L
.55072 .63527 L
.55033 .63497 L
.54625 .63181 L
.54455 .63035 L
.54178 .62779 L
.54024 .62573 L
.53746 .62111 L
.53732 .62085 L
.53481 .61649 L
.53285 .61386 L
.53086 .61186 L
Mistroke
.52838 .60979 L
.52504 .60724 L
.52392 .60644 L
.51945 .60314 L
.51868 .60262 L
.51499 .60045 L
.51052 .5983 L
.50982 .598 L
.50605 .59675 L
.50159 .59576 L
.49712 .59533 L
.49265 .59549 L
.48819 .59625 L
.48372 .59754 L
.48242 .598 L
.47925 .59943 L
.47479 .60188 L
.47364 .60262 L
.47032 .60501 L
.4672 .60724 L
.46585 .60825 L
.46139 .61186 L
.46139 .61187 L
.45738 .61649 L
.45692 .61716 L
.45468 .62111 L
.45245 .62515 L
.45203 .62573 L
.44799 .6301 L
.44767 .63035 L
.44352 .63373 L
.44188 .63497 L
.43906 .63743 L
.43694 .63959 L
.43459 .64204 L
.43201 .64421 L
.43012 .6457 L
.42582 .64883 L
.42566 .64896 L
.42119 .6531 L
.42086 .65345 L
.41672 .65762 L
.41617 .65807 L
.41226 .66107 L
.40969 .66269 L
.40779 .66427 L
.40475 .66731 L
.40332 .66893 L
.39954 .67193 L
.39886 .67237 L
Mistroke
.39439 .67435 L
.38992 .67493 L
.38546 .67464 L
.38099 .67318 L
.37886 .67193 L
.37652 .66928 L
.37505 .66731 L
.37206 .66385 L
.37081 .66269 L
.36759 .66036 L
.36475 .65807 L
.36313 .65499 L
.36236 .65345 L
.36152 .64883 L
.36138 .64421 L
.36163 .63959 L
.36119 .63497 L
.36061 .63035 L
.35975 .62573 L
.35866 .62271 L
.35751 .62111 L
.35419 .61765 L
.35274 .61649 L
.34973 .6137 L
.34825 .61186 L
.3461 .60724 L
.34576 .60262 L
.34581 .598 L
.34605 .59338 L
.34598 .58876 L
.34592 .58414 L
.34587 .57952 L
.34583 .5749 L
.34579 .57028 L
.34576 .56566 L
.34574 .56104 L
.34572 .55642 L
.3457 .5518 L
.34569 .54718 L
.34568 .54256 L
.34567 .53794 L
.34567 .53332 L
.34567 .5287 L
.34567 .52408 L
.34567 .51945 L
.34568 .51483 L
.34569 .51021 L
.3457 .50559 L
.34571 .50097 L
.34573 .49635 L
Mistroke
.34575 .49173 L
.34578 .48711 L
.34581 .48249 L
.34584 .47787 L
.34589 .47325 L
.34593 .46863 L
.34599 .46401 L
.34606 .45939 L
.34604 .45477 L
.34581 .45015 L
.34586 .44553 L
.34645 .44091 L
.34973 .43631 L
.34973 .43629 L
.35419 .43218 L
.35476 .43167 L
.35866 .4273 L
.35878 .42704 L
.36027 .42242 L
.36101 .4178 L
.36154 .41318 L
.36172 .40856 L
.36152 .40394 L
.36185 .39932 L
.36302 .3947 L
.36313 .39438 L
.36703 .39008 L
.36759 .38967 L
.37206 .38607 L
.37267 .38546 L
.37641 .38084 L
.37652 .38069 L
.38099 .37703 L
.38329 .37622 L
.38546 .37577 L
Mfstroke
.38099 .35734 m
.38546 .35629 L
.38992 .35554 L
.39439 .35505 L
.39886 .35487 L
.40332 .35496 L
.40779 .35528 L
.41226 .35578 L
.41672 .35637 L
.42119 .35709 L
.42468 .35774 L
.42566 .35793 L
.43012 .3588 L
.43459 .35971 L
.43906 .36065 L
.44352 .3616 L
.44716 .36236 L
.44799 .36253 L
.45245 .36343 L
.45692 .3643 L
.46139 .3651 L
.46585 .36583 L
.47032 .36648 L
.47421 .36698 L
.47479 .36704 L
.47925 .36751 L
.48372 .36786 L
.48819 .3681 L
.49265 .36822 L
.49712 .36822 L
.50159 .36811 L
.50605 .36787 L
.51052 .36751 L
.51499 .36705 L
.5156 .36698 L
.51945 .36648 L
.52392 .36582 L
.52838 .36507 L
.53285 .36425 L
.53732 .36337 L
.54178 .36244 L
.54217 .36236 L
.54625 .36148 L
.55072 .3605 L
.55518 .35951 L
.55965 .35854 L
.5635 .35774 L
.56412 .35761 L
.56858 .35671 L
.57305 .3559 L
Mistroke
.57752 .3552 L
.58198 .35462 L
.58645 .35417 L
.59092 .35392 L
.59538 .3539 L
.59985 .35419 L
.60431 .35475 L
.60878 .35557 L
.61325 .35675 L
.61614 .35774 L
.61771 .35837 L
.62218 .3604 L
.6258 .36236 L
.62665 .36286 L
.63111 .36573 L
.63283 .36698 L
.63558 .36915 L
.63843 .3716 L
.64005 .3731 L
.64314 .37622 L
.64451 .37772 L
.64715 .38084 L
.64898 .38318 L
.65064 .38546 L
.65345 .38959 L
.65376 .39008 L
.65659 .3947 L
.65791 .39711 L
.65907 .39932 L
.66139 .40394 L
.66238 .40599 L
.6636 .40856 L
.66574 .41318 L
.66685 .41568 L
.66776 .4178 L
.66966 .42242 L
.67131 .42664 L
.67147 .42704 L
.67317 .43167 L
.67472 .43629 L
.67578 .43972 L
.67612 .44091 L
.67735 .44553 L
.67846 .45015 L
.67946 .45477 L
.68024 .4587 L
.68037 .45939 L
.6812 .46401 L
.68194 .46863 L
.68261 .47325 L
Mistroke
.68322 .47787 L
.68376 .48249 L
.68424 .48711 L
.68467 .49173 L
.68471 .49223 L
.68504 .49635 L
.68537 .50097 L
.68564 .50559 L
.68586 .51021 L
.68604 .51483 L
.68617 .51945 L
.68625 .52408 L
.68629 .5287 L
.68628 .53332 L
.68622 .53794 L
.68611 .54256 L
.68595 .54718 L
.68575 .5518 L
.68549 .55642 L
.68518 .56104 L
.68482 .56566 L
.68471 .56688 L
.68439 .57028 L
.6839 .5749 L
.68334 .57952 L
.68272 .58414 L
.68201 .58876 L
.68122 .59338 L
.68034 .598 L
.68024 .59845 L
.67935 .60262 L
.67825 .60724 L
.67701 .61186 L
.67578 .61598 L
.67562 .61649 L
.67408 .62111 L
.67241 .62573 L
.67131 .62865 L
.67065 .63035 L
.66878 .63497 L
.66685 .63944 L
.66678 .63959 L
.66468 .64421 L
.66246 .64883 L
.66238 .649 L
.66012 .65345 L
.65791 .65743 L
.65753 .65807 L
.65463 .66269 L
.65345 .66444 L
Mistroke
.65141 .66731 L
.64898 .67048 L
.64779 .67193 L
.64451 .67564 L
.64364 .67655 L
.64005 .68004 L
.63877 .68117 L
.63558 .68384 L
.63302 .68579 L
.63111 .68713 L
.62665 .68994 L
.62583 .69041 L
.62218 .69235 L
.61771 .69435 L
.61599 .69503 L
.61325 .69598 L
.60878 .69724 L
.60431 .69815 L
.59985 .69881 L
.59538 .69922 L
.59092 .69935 L
.58645 .69927 L
.58198 .69902 L
.57752 .69865 L
.57305 .69817 L
.56858 .69758 L
.56412 .69691 L
.55965 .6962 L
.55518 .69545 L
.55283 .69503 L
.55072 .69466 L
.54625 .69387 L
.54178 .69309 L
.53732 .69233 L
.53285 .69161 L
.52838 .69093 L
.52467 .69041 L
.52392 .69031 L
.51945 .68976 L
.51499 .68928 L
.51052 .68888 L
.50605 .68857 L
.50159 .68835 L
.49712 .68823 L
.49265 .68821 L
.48819 .68828 L
.48372 .68845 L
.47925 .68871 L
.47479 .68906 L
.47032 .68949 L
Mistroke
.46585 .69 L
.46253 .69041 L
.46139 .69056 L
.45692 .69118 L
.45245 .69185 L
.44799 .69255 L
.44352 .69326 L
.43906 .69398 L
.43459 .69469 L
.43236 .69503 L
.43012 .69537 L
.42566 .69601 L
.42119 .6966 L
.41672 .69708 L
.41226 .69745 L
.40779 .69774 L
.40332 .69785 L
.39886 .69777 L
.39439 .69744 L
.38992 .69683 L
.38546 .69596 L
.38175 .69503 L
.38099 .69481 L
.37652 .69321 L
.37206 .69124 L
.37042 .69041 L
.36759 .68882 L
.36313 .68597 L
.36287 .68579 L
.35866 .68263 L
.35693 .68117 L
.35419 .6787 L
.352 .67655 L
.34973 .67412 L
.34783 .67193 L
.34526 .6687 L
.34424 .66731 L
.3411 .66269 L
.34079 .66221 L
.33828 .65807 L
.33633 .65452 L
.33577 .65345 L
.33355 .64883 L
.33186 .64512 L
.33145 .64421 L
.32945 .63959 L
.32752 .63497 L
.32739 .63467 L
.32569 .63035 L
.32397 .62573 L
Mistroke
.32293 .62281 L
.32234 .62111 L
.32083 .61649 L
.31946 .61186 L
.31846 .60808 L
.31826 .60724 L
.31723 .60262 L
.31631 .598 L
.31549 .59338 L
.31476 .58876 L
.31411 .58414 L
.31399 .58323 L
.31354 .57952 L
.31303 .5749 L
.31259 .57028 L
.3122 .56566 L
.31187 .56104 L
.31158 .55642 L
.31134 .5518 L
.31115 .54718 L
.311 .54256 L
.3109 .53794 L
.31084 .53332 L
.31081 .5287 L
.31083 .52408 L
.31089 .51945 L
.31099 .51483 L
.31113 .51021 L
.31132 .50559 L
.31155 .50097 L
.31182 .49635 L
.31213 .49173 L
.3125 .48711 L
.31291 .48249 L
.31338 .47787 L
.31391 .47325 L
.31399 .47257 L
.3145 .46863 L
.31516 .46401 L
.31589 .45939 L
.31671 .45477 L
.31762 .45015 L
.31846 .44629 L
.31864 .44553 L
.31978 .44091 L
.32112 .43629 L
.32262 .43167 L
.32293 .43076 L
.32426 .42704 L
.32601 .42242 L
Mistroke
.32739 .41891 L
.32784 .4178 L
.32979 .41318 L
.33183 .40856 L
.33186 .40849 L
.33392 .40394 L
.33612 .39932 L
.33633 .39889 L
.33847 .3947 L
.34079 .39071 L
.34118 .39008 L
.34419 .38546 L
.34526 .38394 L
.34762 .38084 L
.34973 .37829 L
.35159 .37622 L
.35419 .37358 L
.35631 .3716 L
.35866 .36957 L
.36197 .36698 L
.36313 .36614 L
.36759 .36324 L
.36915 .36236 L
.37206 .36085 L
.37652 .3589 L
.37965 .35774 L
.38099 .35734 L
Mfstroke
.41226 .32041 m
.41672 .31996 L
.42119 .31959 L
.42566 .31928 L
.43012 .31903 L
.43459 .31883 L
.43906 .31867 L
.44352 .31855 L
.44799 .31846 L
.45245 .31839 L
.45692 .31834 L
.46139 .31829 L
.46585 .31826 L
.47032 .31823 L
.47479 .3182 L
.47925 .31817 L
.48372 .31813 L
.48819 .31809 L
.49265 .31804 L
.49712 .31798 L
.50159 .31792 L
.50605 .31784 L
.51052 .31776 L
.51499 .31767 L
.51945 .31759 L
.52392 .3175 L
.52838 .31741 L
.53285 .31733 L
.53732 .31726 L
.54178 .31721 L
.54625 .31718 L
.55072 .31718 L
.55518 .31721 L
.55965 .31728 L
.56412 .31741 L
.56858 .31758 L
.57305 .31782 L
.57752 .31813 L
.58198 .31852 L
.58645 .319 L
.59092 .31958 L
.59538 .32026 L
.59829 .32077 L
.59985 .32108 L
.60431 .32202 L
.60878 .3231 L
.61325 .32434 L
.61664 .32539 L
.61771 .32575 L
.62218 .32733 L
Mistroke
.62665 .32909 L
.62882 .33001 L
.63111 .33105 L
.63558 .33321 L
.6383 .33464 L
.64005 .33559 L
.64451 .3382 L
.64621 .33926 L
.64898 .34105 L
.65304 .34388 L
.65345 .34417 L
.65791 .34758 L
.65906 .3485 L
.66238 .35126 L
.66449 .35312 L
.66685 .35528 L
.6694 .35774 L
.67131 .35966 L
.67389 .36236 L
.67578 .36443 L
.67802 .36698 L
.68024 .36961 L
.68185 .3716 L
.68471 .37528 L
.68541 .37622 L
.68874 .38084 L
.68918 .38148 L
.69184 .38546 L
.69364 .38829 L
.69475 .39008 L
.69749 .3947 L
.69811 .39578 L
.70007 .39932 L
.7025 .40394 L
.70258 .40409 L
.70479 .40856 L
.70695 .41318 L
.70704 .41338 L
.70899 .4178 L
.71091 .42242 L
.71151 .42393 L
.71272 .42704 L
.71442 .43167 L
.71598 .43618 L
.71601 .43629 L
.71751 .44091 L
.7189 .44553 L
.72021 .45015 L
.72044 .45102 L
.72142 .45477 L
Mistroke
.72255 .45939 L
.7236 .46401 L
.72457 .46863 L
.72491 .47036 L
.72546 .47325 L
.72627 .47787 L
.72701 .48249 L
.72769 .48711 L
.72829 .49173 L
.72883 .49635 L
.72929 .50097 L
.72938 .50184 L
.7297 .50559 L
.73003 .51021 L
.73031 .51483 L
.73051 .51945 L
.73065 .52408 L
.73073 .5287 L
.73074 .53332 L
.73068 .53794 L
.73057 .54256 L
.73038 .54718 L
.73013 .5518 L
.72982 .55642 L
.72943 .56104 L
.72938 .56168 L
.72898 .56566 L
.72846 .57028 L
.72787 .5749 L
.7272 .57952 L
.72645 .58414 L
.72563 .58876 L
.72491 .59248 L
.72473 .59338 L
.72374 .598 L
.72266 .60262 L
.7215 .60724 L
.72044 .61117 L
.72025 .61186 L
.7189 .61649 L
.71745 .62111 L
.71598 .6255 L
.7159 .62573 L
.71424 .63035 L
.71246 .63497 L
.71151 .63734 L
.71058 .63959 L
.70857 .64421 L
.70704 .64753 L
.70643 .64883 L
Mistroke
.70416 .65345 L
.70258 .6565 L
.70174 .65807 L
.69916 .66269 L
.69811 .6645 L
.69642 .66731 L
.69364 .6717 L
.69349 .67193 L
.69037 .67655 L
.68918 .67823 L
.68701 .68117 L
.68471 .68417 L
.68342 .68579 L
.68024 .68961 L
.67955 .69041 L
.67578 .6946 L
.67537 .69503 L
.67131 .69917 L
.67082 .69965 L
.66685 .7034 L
.66588 .70427 L
.66238 .7073 L
.66044 .70889 L
.65791 .71089 L
.6544 .71352 L
.65345 .7142 L
.64898 .71726 L
.64763 .71814 L
.64451 .72008 L
.64005 .72267 L
.63989 .72276 L
.63558 .72505 L
.63111 .72723 L
.6308 .72738 L
.62665 .72923 L
.62218 .73104 L
.61965 .732 L
.61771 .73269 L
.61325 .73418 L
.60878 .73552 L
.60472 .73662 L
.60431 .73672 L
.59985 .7378 L
.59538 .73875 L
.59092 .73959 L
.58645 .74033 L
.58198 .74098 L
.58 .74124 L
.57752 .74154 L
.57305 .74202 L
Mistroke
.56858 .74243 L
.56412 .74277 L
.55965 .74305 L
.55518 .74328 L
.55072 .74346 L
.54625 .74361 L
.54178 .74371 L
.53732 .74378 L
.53285 .74382 L
.52838 .74384 L
.52392 .74383 L
.51945 .7438 L
.51499 .74376 L
.51052 .74371 L
.50605 .74364 L
.50159 .74356 L
.49712 .74347 L
.49265 .74337 L
.48819 .74326 L
.48372 .74314 L
.47925 .74301 L
.47479 .74287 L
.47032 .74272 L
.46585 .74254 L
.46139 .74235 L
.45692 .74214 L
.45245 .74191 L
.44799 .74165 L
.44352 .74135 L
.4419 .74124 L
.43906 .74102 L
.43459 .74065 L
.43012 .74022 L
.42566 .73974 L
.42119 .73921 L
.41672 .73861 L
.41226 .73793 L
.40779 .73717 L
.40486 .73662 L
.40332 .73631 L
.39886 .73536 L
.39439 .7343 L
.38992 .73311 L
.3861 .732 L
.38546 .7318 L
.38099 .73034 L
.37652 .72871 L
.37316 .72738 L
.37206 .72692 L
.36759 .72495 L
Mistroke
.36313 .72279 L
.36305 .72276 L
.35866 .72042 L
.3547 .71814 L
.35419 .71783 L
.34973 .71499 L
.34754 .71352 L
.34526 .71191 L
.34124 .70889 L
.34079 .70854 L
.33633 .70486 L
.33565 .70427 L
.33186 .70085 L
.33059 .69965 L
.32739 .69649 L
.32599 .69503 L
.32293 .69171 L
.32179 .69041 L
.31846 .68645 L
.31793 .68579 L
.31436 .68117 L
.31399 .68068 L
.31105 .67655 L
.30953 .67431 L
.30797 .67193 L
.3051 .66731 L
.30506 .66724 L
.30243 .66269 L
.30059 .65933 L
.29993 .65807 L
.29758 .65345 L
.29613 .65042 L
.29539 .64883 L
.29332 .64421 L
.29166 .64026 L
.29139 .63959 L
.28956 .63497 L
.28786 .63035 L
.2872 .62847 L
.28626 .62573 L
.28477 .62111 L
.28338 .61649 L
.28273 .6142 L
.28209 .61186 L
.28089 .60724 L
.27978 .60262 L
.27876 .598 L
.27826 .59559 L
.27783 .59338 L
.27697 .58876 L
Mistroke
.27618 .58414 L
.27547 .57952 L
.27484 .5749 L
.27428 .57028 L
.2738 .56581 L
.27378 .56566 L
.27335 .56104 L
.27297 .55642 L
.27265 .5518 L
.2724 .54718 L
.27221 .54256 L
.27208 .53794 L
.27201 .53332 L
.27199 .5287 L
.27203 .52408 L
.27213 .51945 L
.27229 .51483 L
.27251 .51021 L
.27278 .50559 L
.27312 .50097 L
.27351 .49635 L
.2738 .49337 L
.27396 .49173 L
.27448 .48711 L
.27506 .48249 L
.27571 .47787 L
.27643 .47325 L
.27721 .46863 L
.27807 .46401 L
.27826 .46303 L
.279 .45939 L
.28002 .45477 L
.28112 .45015 L
.2823 .44553 L
.28273 .44393 L
.28357 .44091 L
.28494 .43629 L
.28641 .43167 L
.2872 .42933 L
.28799 .42704 L
.28967 .42242 L
.29146 .4178 L
.29166 .4173 L
.29336 .41318 L
.29539 .40856 L
.29613 .40696 L
.29755 .40394 L
.29984 .39932 L
.30059 .39786 L
.30228 .3947 L
Mistroke
.30488 .39008 L
.30506 .38978 L
.30767 .38546 L
.30953 .38254 L
.31064 .38084 L
.31384 .37622 L
.31399 .37601 L
.31729 .3716 L
.31846 .37011 L
.32103 .36698 L
.32293 .36477 L
.32509 .36236 L
.32739 .3599 L
.32952 .35774 L
.33186 .35547 L
.33442 .35312 L
.33633 .35144 L
.33986 .3485 L
.34079 .34776 L
.34526 .34439 L
.34598 .34388 L
.34973 .34133 L
.35299 .33926 L
.35419 .33853 L
.35866 .33599 L
.36124 .33464 L
.36313 .33369 L
.36759 .33162 L
.37143 .33001 L
.37206 .32976 L
.37652 .3281 L
.38099 .32662 L
.38521 .32539 L
.38546 .32533 L
.38992 .32419 L
.39439 .32319 L
.39886 .32232 L
.40332 .32158 L
.40779 .32095 L
.40916 .32077 L
.41226 .32041 L
Mfstroke
.48372 .26984 m
.48819 .26968 L
.49265 .26955 L
.49712 .26945 L
.50159 .26939 L
.50605 .26936 L
.51052 .26936 L
.51499 .26939 L
.51945 .26945 L
.52392 .26954 L
.52838 .26968 L
.53285 .26984 L
.53525 .26995 L
.53732 .27005 L
.54178 .27029 L
.54625 .27057 L
.55072 .2709 L
.55518 .27127 L
.55965 .2717 L
.56412 .27217 L
.56858 .2727 L
.57305 .27328 L
.57752 .27392 L
.58167 .27457 L
.58198 .27462 L
.58645 .27539 L
.59092 .27623 L
.59538 .27715 L
.59985 .27814 L
.60425 .27919 L
.60431 .2792 L
.60878 .28036 L
.61325 .2816 L
.61771 .28293 L
.62051 .28381 L
.62218 .28435 L
.62665 .28588 L
.63111 .28752 L
.63349 .28843 L
.63558 .28926 L
.64005 .29112 L
.64442 .29305 L
.64451 .29309 L
.64898 .29519 L
.65345 .29742 L
.65393 .29767 L
.65791 .29978 L
.66238 .30229 L
.66239 .30229 L
.66685 .30494 L
Mistroke
.67001 .30691 L
.67131 .30775 L
.67578 .31071 L
.67697 .31153 L
.68024 .31385 L
.68337 .31615 L
.68471 .31717 L
.68918 .32067 L
.6893 .32077 L
.69364 .32438 L
.69483 .32539 L
.69811 .3283 L
.69999 .33001 L
.70258 .33245 L
.70483 .33464 L
.70704 .33684 L
.7094 .33926 L
.71151 .34148 L
.71371 .34388 L
.71598 .34642 L
.71778 .3485 L
.72044 .35166 L
.72164 .35312 L
.72491 .35723 L
.7253 .35774 L
.72877 .36236 L
.72938 .36318 L
.73208 .36698 L
.73384 .36953 L
.73523 .3716 L
.73823 .37622 L
.73831 .37634 L
.74109 .38084 L
.74278 .38368 L
.74381 .38546 L
.74641 .39008 L
.74724 .39161 L
.74888 .3947 L
.75124 .39932 L
.75171 .40027 L
.75348 .40394 L
.75562 .40856 L
.75617 .4098 L
.75765 .41318 L
.75959 .4178 L
.76064 .42042 L
.76143 .42242 L
.76318 .42704 L
.76484 .43167 L
.76511 .43244 L
Mistroke
.76641 .43629 L
.7679 .44091 L
.7693 .44553 L
.76957 .44647 L
.77062 .45015 L
.77186 .45477 L
.77302 .45939 L
.77404 .4637 L
.77411 .46401 L
.77512 .46863 L
.77606 .47325 L
.77694 .47787 L
.77774 .48249 L
.77847 .48711 L
.77851 .48733 L
.77913 .49173 L
.77972 .49635 L
.78024 .50097 L
.7807 .50559 L
.78109 .51021 L
.78141 .51483 L
.78167 .51945 L
.78186 .52408 L
.78198 .5287 L
.78203 .53332 L
.78202 .53794 L
.78195 .54256 L
.78181 .54718 L
.7816 .5518 L
.78133 .55642 L
.78099 .56104 L
.78058 .56566 L
.78011 .57028 L
.77956 .5749 L
.77895 .57952 L
.77851 .58254 L
.77826 .58414 L
.7775 .58876 L
.77666 .59338 L
.77575 .598 L
.77477 .60262 L
.77404 .60587 L
.77372 .60724 L
.77258 .61186 L
.77137 .61649 L
.77007 .62111 L
.76957 .6228 L
.76869 .62573 L
.76723 .63035 L
.76568 .63497 L
Mistroke
.76511 .6366 L
.76403 .63959 L
.7623 .64421 L
.76064 .6484 L
.76047 .64883 L
.75854 .65345 L
.7565 .65807 L
.75617 .6588 L
.75436 .66269 L
.75211 .66731 L
.75171 .66812 L
.74975 .67193 L
.74726 .67655 L
.74724 .67658 L
.74465 .68117 L
.74278 .68434 L
.7419 .68579 L
.73902 .69041 L
.73831 .69151 L
.73599 .69503 L
.73384 .69817 L
.7328 .69965 L
.72945 .70427 L
.72938 .70438 L
.72593 .70889 L
.72491 .71018 L
.72221 .71352 L
.72044 .71563 L
.7183 .71814 L
.71598 .72076 L
.71416 .72276 L
.71151 .72559 L
.70979 .72738 L
.70704 .73015 L
.70516 .732 L
.70258 .73446 L
.70025 .73662 L
.69811 .73855 L
.69502 .74124 L
.69364 .74241 L
.68944 .74586 L
.68918 .74607 L
.68471 .74954 L
.68347 .75048 L
.68024 .75284 L
.67704 .7551 L
.67578 .75596 L
.67131 .75893 L
.67007 .75972 L
.66685 .76174 L
Mistroke
.66249 .76434 L
.66238 .7644 L
.65791 .76693 L
.65414 .76896 L
.65345 .76932 L
.64898 .77159 L
.64485 .77358 L
.64451 .77374 L
.64005 .77577 L
.63558 .77769 L
.63435 .7782 L
.63111 .7795 L
.62665 .78122 L
.62219 .78282 L
.62218 .78283 L
.61771 .78434 L
.61325 .78577 L
.60878 .78711 L
.60762 .78744 L
.60431 .78836 L
.59985 .78953 L
.59538 .79063 L
.59092 .79164 L
.58897 .79206 L
.58645 .79259 L
.58198 .79347 L
.57752 .79427 L
.57305 .79502 L
.56858 .7957 L
.56412 .79632 L
.56131 .79668 L
.55965 .79689 L
.55518 .79739 L
.55072 .79784 L
.54625 .79824 L
.54178 .79859 L
.53732 .79888 L
.53285 .79913 L
.52838 .79933 L
.52392 .79948 L
.51945 .79958 L
.51499 .79964 L
.51052 .79966 L
.50605 .79963 L
.50159 .79956 L
.49712 .79945 L
.49265 .79929 L
.48819 .79909 L
.48372 .79885 L
.47925 .79856 L
Mistroke
.47479 .79823 L
.47032 .79785 L
.46585 .79743 L
.46139 .79697 L
.45883 .79668 L
.45692 .79646 L
.45245 .7959 L
.44799 .79529 L
.44352 .79462 L
.43906 .79391 L
.43459 .79315 L
.43012 .79234 L
.4287 .79206 L
.42566 .79146 L
.42119 .79052 L
.41672 .78951 L
.41226 .78845 L
.40829 .78744 L
.40779 .78731 L
.40332 .78611 L
.39886 .78482 L
.39439 .78346 L
.39237 .78282 L
.38992 .78202 L
.38546 .7805 L
.38099 .77888 L
.37918 .7782 L
.37652 .77718 L
.37206 .77537 L
.36786 .77358 L
.36759 .77346 L
.36313 .77145 L
.35866 .76932 L
.35793 .76896 L
.35419 .76707 L
.34973 .7647 L
.34908 .76434 L
.34526 .76218 L
.3411 .75972 L
.34079 .75953 L
.33633 .75674 L
.33382 .7551 L
.33186 .75379 L
.32739 .75067 L
.32712 .75048 L
.32293 .74738 L
.32094 .74586 L
.31846 .7439 L
.3152 .74124 L
.31399 .74022 L
Mistroke
.30985 .73662 L
.30953 .73633 L
.30506 .7322 L
.30485 .732 L
.30059 .72782 L
.30016 .72738 L
.29613 .72317 L
.29574 .72276 L
.29166 .71823 L
.29158 .71814 L
.28765 .71352 L
.2872 .71296 L
.28394 .70889 L
.28273 .70733 L
.28043 .70427 L
.27826 .7013 L
.2771 .69965 L
.27394 .69503 L
.2738 .69482 L
.27095 .69041 L
.26933 .68782 L
.2681 .68579 L
.2654 .68117 L
.26486 .68023 L
.26283 .67655 L
.2604 .67193 L
.26039 .67193 L
.25808 .66731 L
.25593 .6628 L
.25588 .66269 L
.25379 .65807 L
.25181 .65345 L
.25146 .65262 L
.24993 .64883 L
.24815 .64421 L
.247 .64108 L
.24646 .63959 L
.24487 .63497 L
.24337 .63035 L
.24253 .62766 L
.24195 .62573 L
.24061 .62111 L
.23935 .61649 L
.23817 .61186 L
.23806 .61145 L
.23706 .60724 L
.23604 .60262 L
.23508 .598 L
.2342 .59338 L
.2336 .59001 L
Mistroke
.23338 .58876 L
.23264 .58414 L
.23196 .57952 L
.23134 .5749 L
.2308 .57028 L
.23031 .56566 L
.2299 .56104 L
.22954 .55642 L
.22924 .5518 L
.22913 .54979 L
.22901 .54718 L
.22883 .54256 L
.22872 .53794 L
.22866 .53332 L
.22867 .5287 L
.22873 .52408 L
.22886 .51945 L
.22905 .51483 L
.22913 .51312 L
.2293 .51021 L
.22961 .50559 L
.22998 .50097 L
.23041 .49635 L
.2309 .49173 L
.23146 .48711 L
.23208 .48249 L
.23277 .47787 L
.23353 .47325 L
.2336 .47283 L
.23435 .46863 L
.23524 .46401 L
.2362 .45939 L
.23723 .45477 L
.23806 .45123 L
.23833 .45015 L
.23951 .44553 L
.24076 .44091 L
.2421 .43629 L
.24253 .43487 L
.24352 .43167 L
.24503 .42704 L
.24662 .42242 L
.247 .42137 L
.2483 .4178 L
.25008 .41318 L
.25146 .40974 L
.25195 .40856 L
.25392 .40394 L
.25593 .39948 L
.256 .39932 L
Mistroke
.25819 .3947 L
.2604 .39028 L
.2605 .39008 L
.26292 .38546 L
.26486 .38193 L
.26548 .38084 L
.26817 .37622 L
.26933 .3743 L
.27101 .3716 L
.2738 .36727 L
.27399 .36698 L
.27714 .36236 L
.27826 .36078 L
.28047 .35774 L
.28273 .35475 L
.28399 .35312 L
.2872 .34913 L
.28772 .3485 L
.29166 .34388 L
.29166 .34388 L
.29586 .33926 L
.29613 .33897 L
.30032 .33464 L
.30059 .33436 L
.30506 .33004 L
.30509 .33001 L
.30953 .32597 L
.31019 .32539 L
.31399 .32215 L
.31567 .32077 L
.31846 .31855 L
.3216 .31615 L
.32293 .31516 L
.32739 .31197 L
.32803 .31153 L
.33186 .30897 L
.33508 .30691 L
.33633 .30614 L
.34079 .30348 L
.34287 .30229 L
.34526 .30097 L
.34973 .29861 L
.35158 .29767 L
.35419 .29639 L
.35866 .2943 L
.36148 .29305 L
.36313 .29235 L
.36759 .29051 L
.37206 .28879 L
.37302 .28843 L
Mistroke
.37652 .28717 L
.38099 .28566 L
.38546 .28426 L
.38694 .28381 L
.38992 .28294 L
.39439 .28171 L
.39886 .28057 L
.40332 .2795 L
.40469 .27919 L
.40779 .27851 L
.41226 .27759 L
.41672 .27673 L
.42119 .27594 L
.42566 .2752 L
.42983 .27457 L
.43012 .27453 L
.43459 .2739 L
.43906 .27333 L
.44352 .2728 L
.44799 .27232 L
.45245 .27188 L
.45692 .27148 L
.46139 .27112 L
.46585 .2708 L
.47032 .27051 L
.47479 .27026 L
.47925 .27003 L
.48104 .26995 L
.48372 .26984 L
Mfstroke
.46585 .21894 m
.47032 .21857 L
.47479 .21824 L
.47925 .21795 L
.48372 .21771 L
.48819 .2175 L
.49265 .21734 L
.49712 .21722 L
.50159 .21714 L
.50605 .2171 L
.51052 .2171 L
.51499 .21714 L
.51945 .21722 L
.52392 .21735 L
.52838 .21752 L
.53285 .21773 L
.53732 .21798 L
.54178 .21828 L
.54625 .21862 L
.55072 .21901 L
.55195 .21912 L
.55518 .21944 L
.55965 .21992 L
.56412 .22045 L
.56858 .22103 L
.57305 .22166 L
.57752 .22234 L
.58198 .22308 L
.58579 .22374 L
.58645 .22386 L
.59092 .2247 L
.59538 .22559 L
.59985 .22654 L
.60431 .22755 L
.60771 .22836 L
.60878 .22863 L
.61325 .22976 L
.61771 .23096 L
.62218 .23222 L
.62479 .23298 L
.62665 .23354 L
.63111 .23494 L
.63558 .2364 L
.6391 .2376 L
.64005 .23794 L
.64451 .23955 L
.64898 .24123 L
.65151 .24223 L
.65345 .243 L
.65791 .24485 L
Mistroke
.66238 .24678 L
.66253 .24685 L
.66685 .2488 L
.67131 .2509 L
.67248 .25147 L
.67578 .2531 L
.68024 .25539 L
.68157 .25609 L
.68471 .25778 L
.68918 .26027 L
.68995 .26071 L
.69364 .26287 L
.69772 .26533 L
.69811 .26557 L
.70258 .26839 L
.70498 .26995 L
.70704 .27132 L
.71151 .27437 L
.71179 .27457 L
.71598 .27755 L
.7182 .27919 L
.72044 .28087 L
.72426 .28381 L
.72491 .28432 L
.72938 .28791 L
.73 .28843 L
.73384 .29166 L
.73546 .29305 L
.73831 .29556 L
.74065 .29767 L
.74278 .29963 L
.7456 .30229 L
.74724 .30387 L
.75033 .30691 L
.75171 .3083 L
.75486 .31153 L
.75617 .31292 L
.75919 .31615 L
.76064 .31775 L
.76334 .32077 L
.76511 .3228 L
.76732 .32539 L
.76957 .32809 L
.77114 .33001 L
.77404 .33365 L
.77481 .33464 L
.77833 .33926 L
.77851 .33949 L
.78172 .34388 L
.78297 .34564 L
Mistroke
.78497 .3485 L
.78744 .35212 L
.78811 .35312 L
.79112 .35774 L
.79191 .35898 L
.79401 .36236 L
.79637 .36626 L
.7968 .36698 L
.79948 .3716 L
.80084 .37402 L
.80205 .37622 L
.80452 .38084 L
.80531 .38235 L
.8069 .38546 L
.80918 .39008 L
.80977 .3913 L
.81138 .3947 L
.81348 .39932 L
.81424 .40104 L
.81549 .40394 L
.81743 .40856 L
.81871 .41172 L
.81928 .41318 L
.82106 .4178 L
.82276 .42242 L
.82317 .42358 L
.82438 .42704 L
.82593 .43167 L
.8274 .43629 L
.82764 .43706 L
.8288 .44091 L
.83012 .44553 L
.83137 .45015 L
.8321 .45297 L
.83256 .45477 L
.83368 .45939 L
.83473 .46401 L
.83572 .46863 L
.83657 .47291 L
.83664 .47325 L
.83749 .47787 L
.83829 .48249 L
.83902 .48711 L
.83968 .49173 L
.84029 .49635 L
.84083 .50097 L
.84104 .50293 L
.84131 .50559 L
.84173 .51021 L
.84208 .51483 L
Mistroke
.84238 .51945 L
.84261 .52408 L
.84279 .5287 L
.8429 .53332 L
.84296 .53794 L
.84295 .54256 L
.84289 .54718 L
.84276 .5518 L
.84258 .55642 L
.84233 .56104 L
.84202 .56566 L
.84165 .57028 L
.84121 .5749 L
.84104 .57663 L
.84072 .57952 L
.84017 .58414 L
.83955 .58876 L
.83887 .59338 L
.83812 .598 L
.83732 .60262 L
.83657 .60658 L
.83644 .60724 L
.8355 .61186 L
.8345 .61649 L
.83343 .62111 L
.83229 .62573 L
.8321 .62647 L
.83108 .63035 L
.8298 .63497 L
.82845 .63959 L
.82764 .64225 L
.82703 .64421 L
.82553 .64883 L
.82396 .65345 L
.82317 .65569 L
.82231 .65807 L
.82058 .66269 L
.81877 .66731 L
.81871 .66747 L
.81688 .67193 L
.81491 .67655 L
.81424 .67807 L
.81284 .68117 L
.81069 .68579 L
.80977 .68771 L
.80846 .69041 L
.80613 .69503 L
.80531 .69662 L
.80371 .69965 L
.80118 .70427 L
Mistroke
.80084 .70489 L
.79856 .70889 L
.79637 .71261 L
.79583 .71352 L
.79299 .71814 L
.79191 .71985 L
.79004 .72276 L
.78744 .72668 L
.78697 .72738 L
.78378 .732 L
.78297 .73313 L
.78046 .73662 L
.77851 .73925 L
.777 .74124 L
.77404 .74506 L
.77341 .74586 L
.76967 .75048 L
.76957 .75059 L
.76577 .7551 L
.76511 .75587 L
.76171 .75972 L
.76064 .76091 L
.75748 .76434 L
.75617 .76572 L
.75306 .76896 L
.75171 .77033 L
.74844 .77358 L
.74724 .77475 L
.74362 .7782 L
.74278 .77899 L
.73858 .78282 L
.73831 .78306 L
.73384 .78697 L
.73329 .78744 L
.72938 .79072 L
.72774 .79206 L
.72491 .79433 L
.72189 .79668 L
.72044 .79779 L
.71598 .80112 L
.71573 .8013 L
.71151 .80433 L
.70922 .80593 L
.70704 .80741 L
.70258 .81038 L
.70233 .81055 L
.69811 .81324 L
.69499 .81517 L
.69364 .81598 L
.68918 .81862 L
Mistroke
.68715 .81979 L
.68471 .82116 L
.68024 .8236 L
.67872 .82441 L
.67578 .82594 L
.67131 .82819 L
.66961 .82903 L
.66685 .83035 L
.66238 .83243 L
.65967 .83365 L
.65791 .83442 L
.65345 .83633 L
.64898 .83816 L
.64871 .83827 L
.64451 .83991 L
.64005 .84159 L
.63643 .84289 L
.63558 .84319 L
.63111 .84472 L
.62665 .84618 L
.62241 .84751 L
.62218 .84758 L
.61771 .84891 L
.61325 .85017 L
.60878 .85136 L
.60578 .85213 L
.60431 .85249 L
.59985 .85356 L
.59538 .85457 L
.59092 .85552 L
.58645 .85641 L
.58463 .85675 L
.58198 .85723 L
.57752 .858 L
.57305 .85872 L
.56858 .85937 L
.56412 .85998 L
.55965 .86053 L
.55518 .86103 L
.55184 .86137 L
.55072 .86148 L
.54625 .86187 L
.54178 .86221 L
.53732 .8625 L
.53285 .86274 L
.52838 .86293 L
.52392 .86307 L
.51945 .86317 L
.51499 .8632 L
.51052 .86319 L
Mistroke
.50605 .86313 L
.50159 .86302 L
.49712 .86287 L
.49265 .86266 L
.48819 .86241 L
.48372 .8621 L
.47925 .86175 L
.47509 .86137 L
.47479 .86134 L
.47032 .86089 L
.46585 .86038 L
.46139 .85983 L
.45692 .85922 L
.45245 .85857 L
.44799 .85786 L
.44352 .8571 L
.44154 .85675 L
.43906 .85629 L
.43459 .85541 L
.43012 .85448 L
.42566 .85349 L
.42119 .85246 L
.41982 .85213 L
.41672 .85137 L
.41226 .85021 L
.40779 .84899 L
.40332 .84771 L
.40265 .84751 L
.39886 .84636 L
.39439 .84495 L
.38992 .84348 L
.38819 .84289 L
.38546 .84194 L
.38099 .84032 L
.37652 .83864 L
.37556 .83827 L
.37206 .83689 L
.36759 .83506 L
.36427 .83365 L
.36313 .83315 L
.35866 .83117 L
.35419 .8291 L
.35403 .82903 L
.34973 .82696 L
.34526 .82472 L
.34465 .82441 L
.34079 .82239 L
.33633 .81996 L
.33601 .81979 L
.33186 .81745 L
Mistroke
.32797 .81517 L
.32739 .81482 L
.32293 .81209 L
.32047 .81055 L
.31846 .80926 L
.31399 .80631 L
.31342 .80593 L
.30953 .80325 L
.30678 .8013 L
.30506 .80006 L
.30059 .79674 L
.30052 .79668 L
.29613 .79329 L
.29459 .79206 L
.29166 .78969 L
.28897 .78744 L
.2872 .78594 L
.28362 .78282 L
.28273 .78203 L
.27852 .7782 L
.27826 .77796 L
.2738 .77371 L
.27366 .77358 L
.26933 .76927 L
.26902 .76896 L
.26486 .76463 L
.26459 .76434 L
.2604 .75978 L
.26034 .75972 L
.25628 .7551 L
.25593 .75469 L
.2524 .75048 L
.25146 .74934 L
.24867 .74586 L
.247 .74372 L
.24509 .74124 L
.24253 .73781 L
.24166 .73662 L
.23836 .732 L
.23806 .73157 L
.2352 .72738 L
.2336 .72497 L
.23216 .72276 L
.22923 .71814 L
.22913 .71797 L
.22643 .71352 L
.22466 .71051 L
.22374 .70889 L
.22115 .70427 L
.2202 .70252 L
Mistroke
.21867 .69965 L
.21628 .69503 L
.21573 .69394 L
.21399 .69041 L
.2118 .68579 L
.21126 .68464 L
.20969 .68117 L
.20767 .67655 L
.2068 .6745 L
.20573 .67193 L
.20388 .66731 L
.20233 .66327 L
.20211 .66269 L
.20042 .65807 L
.19881 .65345 L
.19787 .65067 L
.19726 .64883 L
.19578 .64421 L
.19438 .63959 L
.1934 .63623 L
.19304 .63497 L
.19178 .63035 L
.19058 .62573 L
.18944 .62111 L
.18893 .61893 L
.18838 .61649 L
.18737 .61186 L
.18643 .60724 L
.18555 .60262 L
.18473 .598 L
.18447 .59646 L
.18397 .59338 L
.18327 .58876 L
.18263 .58414 L
.18205 .57952 L
.18153 .5749 L
.18106 .57028 L
.18066 .56566 L
.1803 .56104 L
.18001 .55642 L
.18 .55626 L
.17977 .5518 L
.17959 .54718 L
.17946 .54256 L
.17939 .53794 L
.17937 .53332 L
.17941 .5287 L
.1795 .52408 L
.17965 .51945 L
.17985 .51483 L
Mistroke
.18 .51199 L
.18011 .51021 L
.18042 .50559 L
.18079 .50097 L
.18122 .49635 L
.1817 .49173 L
.18224 .48711 L
.18284 .48249 L
.18351 .47787 L
.18423 .47325 L
.18447 .4718 L
.18501 .46863 L
.18585 .46401 L
.18674 .45939 L
.18771 .45477 L
.18873 .45015 L
.18893 .44929 L
.18983 .44553 L
.19099 .44091 L
.19221 .43629 L
.1934 .43203 L
.1935 .43167 L
.19487 .42704 L
.1963 .42242 L
.19781 .4178 L
.19787 .41764 L
.19939 .41318 L
.20105 .40856 L
.20233 .40514 L
.20279 .40394 L
.20461 .39932 L
.20651 .3947 L
.2068 .39401 L
.20849 .39008 L
.21057 .38546 L
.21126 .38396 L
.21274 .38084 L
.215 .37622 L
.21573 .37477 L
.21736 .3716 L
.21983 .36698 L
.2202 .3663 L
.2224 .36236 L
.22466 .35843 L
.22508 .35774 L
.22787 .35312 L
.22913 .35109 L
.23078 .3485 L
.2336 .3442 L
.23382 .34388 L
Mistroke
.23699 .33926 L
.23806 .33773 L
.2403 .33464 L
.24253 .33163 L
.24376 .33001 L
.247 .32587 L
.24737 .32539 L
.25115 .32077 L
.25146 .3204 L
.2551 .31615 L
.25593 .31521 L
.25924 .31153 L
.2604 .31028 L
.26357 .30691 L
.26486 .30558 L
.26813 .30229 L
.26933 .30111 L
.2729 .29767 L
.2738 .29684 L
.27794 .29305 L
.27826 .29276 L
.28273 .28887 L
.28325 .28843 L
.2872 .28515 L
.28886 .28381 L
.29166 .28159 L
.29479 .27919 L
.29613 .27819 L
.30059 .27493 L
.3011 .27457 L
.30506 .27181 L
.30783 .26995 L
.30953 .26883 L
.31399 .26597 L
.31503 .26533 L
.31846 .26324 L
.32278 .26071 L
.32293 .26062 L
.32739 .25812 L
.33116 .25609 L
.33186 .25572 L
.33633 .25342 L
.34029 .25147 L
.34079 .25122 L
.34526 .24912 L
.34973 .24711 L
.35033 .24685 L
.35419 .24519 L
.35866 .24336 L
.36151 .24223 L
Mistroke
.36313 .2416 L
.36759 .23993 L
.37206 .23833 L
.37417 .2376 L
.37652 .23681 L
.38099 .23536 L
.38546 .23398 L
.38881 .23298 L
.38992 .23266 L
.39439 .23141 L
.39886 .23023 L
.40332 .2291 L
.40639 .22836 L
.40779 .22804 L
.41226 .22703 L
.41672 .22608 L
.42119 .22518 L
.42566 .22434 L
.42897 .22374 L
.43012 .22354 L
.43459 .2228 L
.43906 .22211 L
.44352 .22146 L
.44799 .22087 L
.45245 .22032 L
.45692 .21981 L
.46139 .21935 L
.46381 .21912 L
.46585 .21894 L
Mfstroke
.49265 .15891 m
.49712 .15878 L
.50159 .15868 L
.50605 .15863 L
.51052 .15861 L
.51499 .15863 L
.51945 .1587 L
.52392 .1588 L
.52838 .15894 L
.5313 .15906 L
.53285 .15912 L
.53732 .15935 L
.54178 .15961 L
.54625 .1599 L
.55072 .16025 L
.55518 .16063 L
.55965 .16105 L
.56412 .16152 L
.56858 .16203 L
.57305 .16259 L
.57752 .16318 L
.58098 .16368 L
.58198 .16382 L
.58645 .16451 L
.59092 .16523 L
.59538 .16601 L
.59985 .16682 L
.60431 .16768 L
.60738 .1683 L
.60878 .16859 L
.61325 .16954 L
.61771 .17053 L
.62218 .17158 L
.62665 .17267 L
.62764 .17292 L
.63111 .17381 L
.63558 .175 L
.64005 .17625 L
.64448 .17754 L
.64451 .17755 L
.64898 .1789 L
.65345 .1803 L
.65791 .18177 L
.65908 .18216 L
.66238 .18329 L
.66685 .18487 L
.67131 .18651 L
.67204 .18678 L
.67578 .18821 L
.68024 .18997 L
Mistroke
.68376 .1914 L
.68471 .1918 L
.68918 .19369 L
.69364 .19564 L
.69449 .19602 L
.69811 .19766 L
.70258 .19975 L
.70444 .20064 L
.70704 .20191 L
.71151 .20414 L
.7137 .20526 L
.71598 .20645 L
.72044 .20883 L
.72237 .20988 L
.72491 .21128 L
.72938 .21381 L
.73058 .2145 L
.73384 .21642 L
.73831 .21912 L
.73831 .21912 L
.74278 .22191 L
.74565 .22374 L
.74724 .22478 L
.75171 .22774 L
.75263 .22836 L
.75617 .2308 L
.75928 .23298 L
.76064 .23396 L
.76511 .23722 L
.76563 .2376 L
.76957 .24058 L
.77171 .24223 L
.77404 .24405 L
.77753 .24685 L
.77851 .24764 L
.78297 .25133 L
.78313 .25147 L
.78744 .25515 L
.78852 .25609 L
.79191 .25909 L
.79369 .26071 L
.79637 .26318 L
.79867 .26533 L
.80084 .2674 L
.80346 .26995 L
.80531 .27177 L
.80809 .27457 L
.80977 .27629 L
.81255 .27919 L
.81424 .28098 L
Mistroke
.81686 .28381 L
.81871 .28583 L
.82104 .28843 L
.82317 .29086 L
.82507 .29305 L
.82764 .29608 L
.82896 .29767 L
.8321 .30152 L
.83273 .30229 L
.83637 .30691 L
.83657 .30717 L
.8399 .31153 L
.84104 .31306 L
.84331 .31615 L
.8455 .3192 L
.84662 .32077 L
.84981 .32539 L
.84997 .32563 L
.8529 .33001 L
.85444 .33236 L
.8559 .33464 L
.8588 .33926 L
.8589 .33943 L
.8616 .34388 L
.86337 .34687 L
.86432 .3485 L
.86695 .35312 L
.86784 .3547 L
.8695 .35774 L
.87197 .36236 L
.8723 .363 L
.87434 .36698 L
.87664 .3716 L
.87677 .37187 L
.87886 .37622 L
.881 .38084 L
.88124 .38136 L
.88306 .38546 L
.88505 .39008 L
.8857 .39165 L
.88696 .3947 L
.88883 .39932 L
.89017 .40275 L
.89063 .40394 L
.89235 .40856 L
.89401 .41318 L
.89464 .41498 L
.8956 .4178 L
.89713 .42242 L
.89859 .42704 L
Mistroke
.8991 .42869 L
.89999 .43167 L
.90133 .43629 L
.90261 .44091 L
.90357 .44451 L
.90383 .44553 L
.905 .45015 L
.90611 .45477 L
.90715 .45939 L
.90803 .46349 L
.90814 .46401 L
.90908 .46863 L
.90995 .47325 L
.91077 .47787 L
.91153 .48249 L
.91224 .48711 L
.9125 .48892 L
.91289 .49173 L
.9135 .49635 L
.91404 .50097 L
.91454 .50559 L
.91497 .51021 L
.91536 .51483 L
.91569 .51945 L
.91597 .52408 L
.91619 .5287 L
.91637 .53332 L
.9165 .53794 L
.91657 .54256 L
.91659 .54718 L
.91655 .5518 L
.91647 .55642 L
.91633 .56104 L
.91614 .56566 L
.91589 .57028 L
.91559 .5749 L
.91525 .57952 L
.91484 .58414 L
.91439 .58876 L
.91388 .59338 L
.91332 .598 L
.9127 .60262 L
.9125 .60406 L
.91204 .60724 L
.91131 .61186 L
.91053 .61649 L
.9097 .62111 L
.9088 .62573 L
.90803 .62949 L
.90786 .63035 L
Mistroke
.90686 .63497 L
.9058 .63959 L
.90468 .64421 L
.90357 .64855 L
.90349 .64883 L
.90226 .65345 L
.90097 .65807 L
.89962 .66269 L
.8991 .66443 L
.89822 .66731 L
.89675 .67193 L
.89522 .67655 L
.89464 .67825 L
.89362 .68117 L
.89196 .68579 L
.89024 .69041 L
.89017 .69059 L
.88845 .69503 L
.8866 .69965 L
.8857 .70183 L
.88468 .70427 L
.88268 .70889 L
.88124 .71215 L
.88062 .71352 L
.87848 .71814 L
.87677 .72172 L
.87626 .72276 L
.87397 .72738 L
.8723 .73064 L
.8716 .732 L
.86915 .73662 L
.86784 .73904 L
.86662 .74124 L
.86401 .74586 L
.86337 .74697 L
.86132 .75048 L
.8589 .75449 L
.85853 .7551 L
.85565 .75972 L
.85444 .76163 L
.85268 .76434 L
.84997 .76844 L
.84962 .76896 L
.84646 .77358 L
.8455 .77496 L
.84321 .7782 L
.84104 .7812 L
.83984 .78282 L
.83657 .78717 L
.83636 .78744 L
Mistroke
.83277 .79206 L
.8321 .79291 L
.82906 .79668 L
.82764 .79842 L
.82523 .8013 L
.82317 .80372 L
.82127 .80593 L
.81871 .80884 L
.81718 .81055 L
.81424 .81378 L
.81295 .81517 L
.80977 .81854 L
.80857 .81979 L
.80531 .82313 L
.80404 .82441 L
.80084 .82758 L
.79936 .82903 L
.79637 .83188 L
.7945 .83365 L
.79191 .83605 L
.78946 .83827 L
.78744 .84008 L
.78424 .84289 L
.78297 .84398 L
.77881 .84751 L
.77851 .84776 L
.77404 .85142 L
.77316 .85213 L
.76957 .85498 L
.7673 .85675 L
.76511 .85843 L
.76118 .86137 L
.76064 .86177 L
.75617 .86501 L
.75479 .86599 L
.75171 .86815 L
.74811 .87061 L
.74724 .8712 L
.74278 .87414 L
.74109 .87523 L
.73831 .877 L
.73384 .87978 L
.73372 .87985 L
.72938 .88247 L
.72596 .88447 L
.72491 .88508 L
.72044 .88761 L
.71776 .88909 L
.71598 .89006 L
.71151 .89244 L
Mistroke
.70905 .89371 L
.70704 .89474 L
.70258 .89697 L
.69976 .89833 L
.69811 .89913 L
.69364 .90122 L
.68982 .90296 L
.68918 .90324 L
.68471 .90519 L
.68024 .90708 L
.67904 .90758 L
.67578 .9089 L
.67131 .91066 L
.66727 .9122 L
.66685 .91235 L
.66238 .91399 L
.65791 .91556 L
.65424 .91682 L
.65345 .91708 L
.64898 .91855 L
.64451 .91995 L
.64005 .9213 L
.63958 .92144 L
.63558 .92259 L
.63111 .92382 L
.62665 .925 L
.62251 .92606 L
.62218 .92614 L
.61771 .92723 L
.61325 .92827 L
.60878 .92925 L
.60431 .93018 L
.60178 .93068 L
.59985 .93105 L
.59538 .93188 L
.59092 .93267 L
.58645 .9334 L
.58198 .93409 L
.57752 .93472 L
.57306 .9353 L
.57305 .9353 L
.56858 .93583 L
.56412 .93632 L
.55965 .93676 L
.55518 .93716 L
.55072 .93751 L
.54625 .93781 L
.54178 .93806 L
.53732 .93826 L
.53285 .93842 L
Mistroke
.52838 .93854 L
.52392 .93861 L
.51945 .93863 L
.51499 .93861 L
.51052 .93854 L
.50605 .93842 L
.50159 .93826 L
.49712 .93805 L
.49265 .9378 L
.48819 .9375 L
.48372 .93715 L
.47925 .93676 L
.47479 .93632 L
.47032 .93584 L
.46585 .93531 L
.46575 .9353 L
.46139 .93474 L
.45692 .93412 L
.45245 .93344 L
.44799 .93271 L
.44352 .93194 L
.43906 .93112 L
.43671 .93068 L
.43459 .93026 L
.43012 .92935 L
.42566 .92838 L
.42119 .92735 L
.41672 .92628 L
.41584 .92606 L
.41226 .92515 L
.40779 .92397 L
.40332 .92274 L
.39886 .92145 L
.39881 .92144 L
.39439 .92011 L
.38992 .91871 L
.38546 .91725 L
.38415 .91682 L
.38099 .91574 L
.37652 .91416 L
.37206 .91253 L
.37116 .9122 L
.36759 .91084 L
.36313 .90909 L
.35939 .90758 L
.35866 .90728 L
.35419 .9054 L
.34973 .90346 L
.3486 .90296 L
.34526 .90145 L
Mistroke
.34079 .89939 L
.33858 .89833 L
.33633 .89725 L
.33186 .89503 L
.32929 .89371 L
.32739 .89273 L
.32293 .89036 L
.32059 .88909 L
.31846 .88792 L
.31399 .88539 L
.3124 .88447 L
.30953 .88279 L
.30506 .8801 L
.30466 .87985 L
.30059 .87733 L
.29731 .87523 L
.29613 .87447 L
.29166 .87151 L
.29033 .87061 L
.2872 .86846 L
.28367 .86599 L
.28273 .86532 L
.27826 .86207 L
.27731 .86137 L
.2738 .85872 L
.27123 .85675 L
.26933 .85527 L
.26539 .85213 L
.26486 .8517 L
.2604 .84802 L
.25978 .84751 L
.25593 .84423 L
.25439 .84289 L
.25146 .8403 L
.24921 .83827 L
.247 .83625 L
.24421 .83365 L
.24253 .83205 L
.23941 .82903 L
.23806 .82771 L
.23478 .82441 L
.2336 .8232 L
.23031 .81979 L
.22913 .81854 L
.22599 .81517 L
.22466 .81371 L
.22182 .81055 L
.2202 .8087 L
.21779 .80593 L
.21573 .8035 L
Mistroke
.21389 .8013 L
.21126 .79811 L
.21012 .79668 L
.2068 .79248 L
.20647 .79206 L
.20295 .78744 L
.20233 .78661 L
.19955 .78282 L
.19787 .78048 L
.19625 .7782 L
.1934 .77409 L
.19305 .77358 L
.18996 .76896 L
.18893 .7674 L
.18696 .76434 L
.18447 .76039 L
.18405 .75972 L
.18123 .7551 L
.18 .75304 L
.1785 .75048 L
.17586 .74586 L
.17553 .74527 L
.17331 .74124 L
.17107 .73706 L
.17083 .73662 L
.16844 .732 L
.1666 .72835 L
.16612 .72738 L
.16388 .72276 L
.16213 .71904 L
.16171 .71814 L
.15962 .71352 L
.15767 .70907 L
.15759 .70889 L
.15563 .70427 L
.15374 .69965 L
.1532 .6983 L
.15192 .69503 L
.15017 .69041 L
.14873 .68651 L
.14847 .68579 L
.14684 .68117 L
.14527 .67655 L
.14427 .67349 L
.14376 .67193 L
.14231 .66731 L
.14092 .66269 L
.1398 .65886 L
.13958 .65807 L
.1383 .65345 L
Mistroke
.13708 .64883 L
.13591 .64421 L
.13533 .64187 L
.13479 .63959 L
.13372 .63497 L
.13271 .63035 L
.13175 .62573 L
.13087 .62123 L
.13085 .62111 L
.12999 .61649 L
.12919 .61186 L
.12844 .60724 L
.12773 .60262 L
.12708 .598 L
.12647 .59338 L
.1264 .59282 L
.12591 .58876 L
.1254 .58414 L
.12494 .57952 L
.12453 .5749 L
.12416 .57028 L
.12385 .56566 L
.12358 .56104 L
.12335 .55642 L
.12317 .5518 L
.12304 .54718 L
.12297 .54256 L
.12294 .53794 L
.12296 .53332 L
.12302 .5287 L
.12313 .52408 L
.12329 .51945 L
.1235 .51483 L
.12376 .51021 L
.12406 .50559 L
.12441 .50097 L
.12481 .49635 L
.12526 .49173 L
.12575 .48711 L
.1263 .48249 L
.1264 .48169 L
.1269 .47787 L
.12755 .47325 L
.12824 .46863 L
.12899 .46401 L
.12979 .45939 L
.13065 .45477 L
.13087 .45362 L
.13155 .45015 L
.13251 .44553 L
Mistroke
.13353 .44091 L
.1346 .43629 L
.13533 .43324 L
.13573 .43167 L
.13691 .42704 L
.13815 .42242 L
.13945 .4178 L
.1398 .41658 L
.1408 .41318 L
.14222 .40856 L
.14371 .40394 L
.14427 .40225 L
.14525 .39932 L
.14686 .3947 L
.14854 .39008 L
.14873 .38955 L
.15028 .38546 L
.15209 .38084 L
.1532 .37808 L
.15396 .37622 L
.15591 .3716 L
.15767 .36758 L
.15794 .36698 L
.16004 .36236 L
.16213 .35792 L
.16222 .35774 L
.16448 .35312 L
.1666 .34892 L
.16682 .3485 L
.16925 .34388 L
.17107 .34052 L
.17176 .33926 L
.17437 .33464 L
.17553 .33261 L
.17706 .33001 L
.17985 .32539 L
.18 .32515 L
.18274 .32077 L
.18447 .3181 L
.18574 .31615 L
.18885 .31153 L
.18893 .31141 L
.19206 .30691 L
.1934 .30504 L
.1954 .30229 L
.19787 .29897 L
.19885 .29767 L
.20233 .29318 L
.20243 .29305 L
.20614 .28843 L
Mistroke
.2068 .28763 L
.20998 .28381 L
.21126 .28231 L
.21398 .27919 L
.21573 .27721 L
.21812 .27457 L
.2202 .27232 L
.22242 .26995 L
.22466 .26761 L
.2269 .26533 L
.22913 .26309 L
.23156 .26071 L
.2336 .25875 L
.23642 .25609 L
.23806 .25456 L
.24148 .25147 L
.24253 .25054 L
.24677 .24685 L
.247 .24665 L
.25146 .24291 L
.25229 .24223 L
.25593 .23929 L
.25807 .2376 L
.2604 .2358 L
.26412 .23298 L
.26486 .23244 L
.26933 .22919 L
.27048 .22836 L
.2738 .22605 L
.27717 .22374 L
.27826 .22301 L
.28273 .22009 L
.28423 .21912 L
.2872 .21726 L
.29166 .21452 L
.29169 .2145 L
.29613 .21188 L
.29961 .20988 L
.30059 .20933 L
.30506 .20686 L
.30805 .20526 L
.30953 .20449 L
.31399 .20219 L
.3171 .20064 L
.31846 .19997 L
.32293 .19783 L
.32684 .19602 L
.32739 .19577 L
.33186 .19377 L
.33633 .19185 L
Mistroke
.3374 .1914 L
.34079 .19 L
.34526 .18822 L
.34899 .18678 L
.34973 .1865 L
.35419 .18484 L
.35866 .18325 L
.36183 .18216 L
.36313 .18172 L
.36759 .18024 L
.37206 .17882 L
.37626 .17754 L
.37652 .17746 L
.38099 .17615 L
.38546 .1749 L
.38992 .1737 L
.39294 .17292 L
.39439 .17255 L
.39886 .17145 L
.40332 .1704 L
.40779 .1694 L
.41226 .16845 L
.41299 .1683 L
.41672 .16754 L
.42119 .16669 L
.42566 .16587 L
.43012 .16511 L
.43459 .16439 L
.43906 .16372 L
.43937 .16368 L
.44352 .16309 L
.44799 .1625 L
.45245 .16195 L
.45692 .16145 L
.46139 .16099 L
.46585 .16057 L
.47032 .16019 L
.47479 .15986 L
.47925 .15956 L
.48372 .1593 L
.48819 .15909 L
.48886 .15906 L
.49265 .15891 L
Mfstroke
.0001 w
.0594 .05741 m
.04594 .04219 L
s
% Start of user PostScript
22. 20. 21.121673 19.007146 MAarrowhead1
% End of user PostScript
.0594 .12308 m
.0424 .10788 L
s
% Start of user PostScript
22. 24.285714 20.890593 23.29367 MAarrowhead1
% End of user PostScript
.0594 .18876 m
.03855 .17387 L
s
% Start of user PostScript
22. 28.571429 20.638977 27.60006 MAarrowhead1
% End of user PostScript
.0594 .25444 m
.03451 .24049 L
s
% Start of user PostScript
22. 32.857143 20.375192 31.946742 MAarrowhead1
% End of user PostScript
.0594 .32011 m
.03061 .30803 L
s
% Start of user PostScript
22. 37.142857 20.12124 36.354304 MAarrowhead1
% End of user PostScript
.0594 .38579 m
.0272 .37647 L
s
% Start of user PostScript
22. 41.428571 19.898707 40.820537 MAarrowhead1
% End of user PostScript
.0594 .45147 m
.02484 .44581 L
s
% Start of user PostScript
22. 45.714286 19.744461 45.345028 MAarrowhead1
% End of user PostScript
.0594 .51714 m
.02381 .51562 L
s
% Start of user PostScript
22. 50. 19.677243 49.900647 MAarrowhead1
% End of user PostScript
.0594 .58282 m
.02421 .58551 L
s
% Start of user PostScript
22. 54.285714 19.703281 54.461069 MAarrowhead1
% End of user PostScript
.0594 .6485 m
.02597 .65481 L
s
% Start of user PostScript
22. 58.571429 19.818368 58.983429 MAarrowhead1
% End of user PostScript
.0594 .71418 m
.02882 .72328 L
s
% Start of user PostScript
22. 62.857143 20.004219 63.451023 MAarrowhead1
% End of user PostScript
.0594 .77985 m
.03218 .79075 L
s
% Start of user PostScript
22. 67.142857 20.223669 67.854103 MAarrowhead1
% End of user PostScript
.0594 .84553 m
.03567 .8573 L
s
% Start of user PostScript
22. 71.428571 20.450887 72.196841 MAarrowhead1
% End of user PostScript
.0594 .91121 m
.03899 .92321 L
s
% Start of user PostScript
22. 75.714286 20.667662 76.497289 MAarrowhead1
% End of user PostScript
.0594 .97688 m
.04198 .98855 L
s
% Start of user PostScript
22. 80. 20.863045 80.761086 MAarrowhead1
% End of user PostScript
.12289 .05741 m
.10976 .03885 L
s
% Start of user PostScript
26.142857 20. 25.286213 18.789072 MAarrowhead1
% End of user PostScript
.12289 .12308 m
.10592 .1041 L
s
% Start of user PostScript
26.142857 24.285714 25.035609 23.046781 MAarrowhead1
% End of user PostScript
.12289 .18876 m
.10158 .16976 L
s
% Start of user PostScript
26.142857 28.571429 24.752088 27.331836 MAarrowhead1
% End of user PostScript
.12289 .25444 m
.09679 .23615 L
s
% Start of user PostScript
26.142857 32.857143 24.439718 31.663863 MAarrowhead1
% End of user PostScript
.12289 .32011 m
.09165 .30386 L
s
% Start of user PostScript
26.142857 37.142857 24.104155 36.082061 MAarrowhead1
% End of user PostScript
.12289 .38579 m
.087 .37312 L
s
% Start of user PostScript
26.142857 41.428571 23.800565 40.602035 MAarrowhead1
% End of user PostScript
.12289 .45147 m
.08376 .44383 L
s
% Start of user PostScript
26.142857 45.714286 23.589341 45.216192 MAarrowhead1
% End of user PostScript
.12289 .51714 m
.08245 .51533 L
s
% Start of user PostScript
26.142857 50. 23.503857 49.881778 MAarrowhead1
% End of user PostScript
.12289 .58282 m
.08314 .58685 L
s
% Start of user PostScript
26.142857 54.285714 23.548535 54.548659 MAarrowhead1
% End of user PostScript
.12289 .6485 m
.08577 .65757 L
s
% Start of user PostScript
26.142857 58.571429 23.720303 59.163693 MAarrowhead1
% End of user PostScript
.12289 .71418 m
.08979 .72693 L
s
% Start of user PostScript
26.142857 62.857143 23.982828 63.689647 MAarrowhead1
% End of user PostScript
.12289 .77985 m
.09443 .79464 L
s
% Start of user PostScript
26.142857 67.142857 24.285282 68.108141 MAarrowhead1
% End of user PostScript
.12289 .84553 m
.09883 .86098 L
s
% Start of user PostScript
26.142857 71.428571 24.572973 72.437046 MAarrowhead1
% End of user PostScript
.12289 .91121 m
.10274 .92645 L
s
% Start of user PostScript
26.142857 75.714286 24.827818 76.708919 MAarrowhead1
% End of user PostScript
.12289 .97688 m
.10603 .99139 L
s
% Start of user PostScript
26.142857 80. 25.042638 80.946699 MAarrowhead1
% End of user PostScript
.18638 .05741 m
.17396 .03538 L
s
% Start of user PostScript
30.285714 20. 29.475478 18.562438 MAarrowhead1
% End of user PostScript
.18638 .12308 m
.17001 .10004 L
s
% Start of user PostScript
30.285714 24.285714 29.217685 22.781769 MAarrowhead1
% End of user PostScript
.18638 .18876 m
.16517 .16491 L
s
% Start of user PostScript
30.285714 28.571429 28.901577 27.014978 MAarrowhead1
% End of user PostScript
.18638 .25444 m
.15923 .23056 L
s
% Start of user PostScript
30.285714 32.857143 28.513946 31.29893 MAarrowhead1
% End of user PostScript
.18638 .32011 m
.15224 .29799 L
s
% Start of user PostScript
30.285714 37.142857 28.058105 35.699209 MAarrowhead1
% End of user PostScript
.18638 .38579 m
.14542 .36833 L
s
% Start of user PostScript
30.285714 41.428571 27.612898 40.2894 MAarrowhead1
% End of user PostScript
.18638 .45147 m
.14078 .44123 L
s
% Start of user PostScript
30.285714 45.714286 27.310002 45.046012 MAarrowhead1
% End of user PostScript
.18638 .51714 m
.13919 .51509 L
s
% Start of user PostScript
30.285714 50. 27.20608 49.866022 MAarrowhead1
% End of user PostScript
.18638 .58282 m
.14028 .58882 L
s
% Start of user PostScript
30.285714 54.285714 27.277489 54.677172 MAarrowhead1
% End of user PostScript
.18638 .6485 m
.14418 .6617 L
s
% Start of user PostScript
30.285714 58.571429 27.532272 59.43319 MAarrowhead1
% End of user PostScript
.18638 .71418 m
.1503 .73221 L
s
% Start of user PostScript
30.285714 62.857143 27.931514 64.033819 MAarrowhead1
% End of user PostScript
.18638 .77985 m
.1569 .7998 L
s
% Start of user PostScript
30.285714 67.142857 28.362158 68.444321 MAarrowhead1
% End of user PostScript
.18638 .84553 m
.16255 .8655 L
s
% Start of user PostScript
30.285714 71.428571 28.730889 72.731815 MAarrowhead1
% End of user PostScript
.18638 .91121 m
.16704 .93009 L
s
% Start of user PostScript
30.285714 75.714286 29.023592 76.946638 MAarrowhead1
% End of user PostScript
.18638 .97688 m
.17059 .99445 L
s
% Start of user PostScript
30.285714 80. 29.255515 81.146077 MAarrowhead1
% End of user PostScript
.24987 .05741 m
.23869 .03195 L
s
% Start of user PostScript
34.428571 20. 33.699172 18.338638 MAarrowhead1
% End of user PostScript
.24987 .12308 m
.23491 .09584 L
s
% Start of user PostScript
34.428571 24.285714 33.452377 22.50816 MAarrowhead1
% End of user PostScript
.24987 .18876 m
.22987 .15942 L
s
% Start of user PostScript
34.428571 28.571429 33.12357 26.657045 MAarrowhead1
% End of user PostScript
.24987 .25444 m
.22281 .22331 L
s
% Start of user PostScript
34.428571 32.857143 32.66273 30.825953 MAarrowhead1
% End of user PostScript
.24987 .32011 m
.2128 .2892 L
s
% Start of user PostScript
34.428571 37.142857 32.009634 35.125566 MAarrowhead1
% End of user PostScript
.24987 .38579 m
.20155 .36086 L
s
% Start of user PostScript
34.428571 41.428571 31.275521 39.802018 MAarrowhead1
% End of user PostScript
.24987 .45147 m
.19481 .43803 L
s
% Start of user PostScript
34.428571 45.714286 30.835728 44.837444 MAarrowhead1
% End of user PostScript
.24987 .51714 m
.19363 .51502 L
s
% Start of user PostScript
34.428571 50. 30.7589 49.861325 MAarrowhead1
% End of user PostScript
.24987 .58282 m
.1947 .59136 L
s
% Start of user PostScript
34.428571 54.285714 30.828284 54.843183 MAarrowhead1
% End of user PostScript
.24987 .6485 m
.20008 .66813 L
s
% Start of user PostScript
34.428571 58.571429 31.179584 59.852575 MAarrowhead1
% End of user PostScript
.24987 .71418 m
.21044 .74056 L
s
% Start of user PostScript
34.428571 62.857143 31.855787 64.578871 MAarrowhead1
% End of user PostScript
.24987 .77985 m
.22043 .8069 L
s
% Start of user PostScript
34.428571 67.142857 32.507539 68.908117 MAarrowhead1
% End of user PostScript
.24987 .84553 m
.22746 .87074 L
s
% Start of user PostScript
34.428571 71.428571 32.9664 73.073926 MAarrowhead1
% End of user PostScript
.24987 .91121 m
.23227 .934 L
s
% Start of user PostScript
34.428571 75.714286 33.280399 77.201799 MAarrowhead1
% End of user PostScript
.24987 .97688 m
.2358 .99744 L
s
% Start of user PostScript
34.428571 80. 33.510366 81.341426 MAarrowhead1
% End of user PostScript
.31336 .05741 m
.30411 .0289 L
s
% Start of user PostScript
38.571429 20. 37.968393 18.140136 MAarrowhead1
% End of user PostScript
.31336 .12308 m
.30082 .09187 L
s
% Start of user PostScript
38.571429 24.285714 37.75372 22.248638 MAarrowhead1
% End of user PostScript
.31336 .18876 m
.29637 .1539 L
s
% Start of user PostScript
38.571429 28.571429 37.462915 26.296951 MAarrowhead1
% End of user PostScript
.31336 .25444 m
.28941 .21465 L
s
% Start of user PostScript
38.571429 32.857143 37.008859 30.260956 MAarrowhead1
% End of user PostScript
.31336 .32011 m
.27598 .27471 L
s
% Start of user PostScript
38.571429 37.142857 36.132321 34.179877 MAarrowhead1
% End of user PostScript
.31336 .38579 m
.2518 .347 L
s
% Start of user PostScript
38.571429 41.428571 34.554315 38.897626 MAarrowhead1
% End of user PostScript
.31336 .45147 m
.24209 .43644 L
s
% Start of user PostScript
38.571429 45.714286 33.921274 44.733758 MAarrowhead1
% End of user PostScript
.31336 .51714 m
.24722 .51579 L
s
% Start of user PostScript
38.571429 50. 34.255758 49.911677 MAarrowhead1
% End of user PostScript
.31336 .58282 m
.24475 .59162 L
s
% Start of user PostScript
38.571429 54.285714 34.094871 54.85999 MAarrowhead1
% End of user PostScript
.31336 .6485 m
.25006 .67874 L
s
% Start of user PostScript
38.571429 58.571429 34.441096 60.544944 MAarrowhead1
% End of user PostScript
.31336 .71418 m
.27211 .75553 L
s
% Start of user PostScript
38.571429 62.857143 35.880282 65.5556 MAarrowhead1
% End of user PostScript
.31336 .77985 m
.28705 .81603 L
s
% Start of user PostScript
38.571429 67.142857 36.854745 69.503582 MAarrowhead1
% End of user PostScript
.31336 .84553 m
.2943 .87626 L
s
% Start of user PostScript
38.571429 71.428571 37.327939 73.433923 MAarrowhead1
% End of user PostScript
.31336 .91121 m
.29871 .93777 L
s
% Start of user PostScript
38.571429 75.714286 37.615438 77.447864 MAarrowhead1
% End of user PostScript
.31336 .97688 m
.3017 1.00015 L
s
% Start of user PostScript
38.571429 80. 37.810595 81.518122 MAarrowhead1
% End of user PostScript
.37684 .05741 m
.37012 .0265 L
s
% Start of user PostScript
42.714286 20. 42.275616 17.983464 MAarrowhead1
% End of user PostScript
.37684 .12308 m
.36785 .08874 L
s
% Start of user PostScript
42.714286 24.285714 42.127177 22.044442 MAarrowhead1
% End of user PostScript
.37684 .18876 m
.36487 .14952 L
s
% Start of user PostScript
42.714286 28.571429 41.932958 26.010961 MAarrowhead1
% End of user PostScript
.37684 .25444 m
.36086 .20741 L
s
% Start of user PostScript
42.714286 32.857143 41.671285 29.788415 MAarrowhead1
% End of user PostScript
.37684 .32011 m
.35447 .2562 L
s
% Start of user PostScript
42.714286 37.142857 41.254355 32.971893 MAarrowhead1
% End of user PostScript
.37684 .38579 m
.33481 .35957 L
s
% Start of user PostScript
42.714286 41.428571 39.971236 39.717525 MAarrowhead1
% End of user PostScript
.37684 .45147 m
.37656 .45119 L
s
% Start of user PostScript
42.714286 45.714286 42.695925 45.695925 MAarrowhead1
% End of user PostScript
.37684 .51714 m
.37684 .51714 L
s
% Start of user PostScript
42.714286 50. MAarrowzero1
% End of user PostScript
.37684 .58282 m
.37684 .58282 L
s
% Start of user PostScript
42.714286 54.285714 MAarrowzero1
% End of user PostScript
.37684 .6485 m
.39411 .64718 L
s
% Start of user PostScript
42.714286 58.571429 43.840762 58.48556 MAarrowhead1
% End of user PostScript
.37684 .71418 m
.35212 .77963 L
s
% Start of user PostScript
42.714286 62.857143 41.100727 67.128077 MAarrowhead1
% End of user PostScript
.37684 .77985 m
.35963 .82381 L
s
% Start of user PostScript
42.714286 67.142857 41.591195 70.011468 MAarrowhead1
% End of user PostScript
.37684 .84553 m
.36346 .88059 L
s
% Start of user PostScript
42.714286 71.428571 41.841247 73.716555 MAarrowhead1
% End of user PostScript
.37684 .91121 m
.36624 .9407 L
s
% Start of user PostScript
42.714286 75.714286 42.022344 77.63919 MAarrowhead1
% End of user PostScript
.37684 .97688 m
.36833 1.00232 L
s
% Start of user PostScript
42.714286 80. 42.158708 81.659938 MAarrowhead1
% End of user PostScript
.44033 .05741 m
.43658 .02498 L
s
% Start of user PostScript
46.857143 20. 46.612342 17.883893 MAarrowhead1
% End of user PostScript
.44033 .12308 m
.43547 .08689 L
s
% Start of user PostScript
46.857143 24.285714 46.539986 21.924102 MAarrowhead1
% End of user PostScript
.44033 .18876 m
.43426 .14743 L
s
% Start of user PostScript
46.857143 28.571429 46.461207 25.87418 MAarrowhead1
% End of user PostScript
.44033 .25444 m
.43377 .20605 L
s
% Start of user PostScript
46.857143 32.857143 46.429271 29.699577 MAarrowhead1
% End of user PostScript
.44033 .32011 m
.43901 .26338 L
s
% Start of user PostScript
46.857143 37.142857 46.771127 33.440695 MAarrowhead1
% End of user PostScript
.44033 .38579 m
.46225 .33887 L
s
% Start of user PostScript
46.857143 41.428571 48.287661 38.366934 MAarrowhead1
% End of user PostScript
.44033 .45147 m
.44089 .45189 L
s
% Start of user PostScript
46.857143 45.714286 46.893496 45.741999 MAarrowhead1
% End of user PostScript
.44033 .51714 m
.44033 .51714 L
s
% Start of user PostScript
46.857143 50. MAarrowzero1
% End of user PostScript
.44033 .58282 m
.44023 .58288 L
s
% Start of user PostScript
46.857143 54.285714 46.850706 54.289213 MAarrowhead1
% End of user PostScript
.44033 .6485 m
.46749 .68421 L
s
% Start of user PostScript
46.857143 58.571429 48.629356 60.901772 MAarrowhead1
% End of user PostScript
.44033 .71418 m
.44187 .76636 L
s
% Start of user PostScript
46.857143 62.857143 46.957299 66.262716 MAarrowhead1
% End of user PostScript
.44033 .77985 m
.43375 .8243 L
s
% Start of user PostScript
46.857143 67.142857 46.427423 70.043518 MAarrowhead1
% End of user PostScript
.44033 .84553 m
.43355 .8824 L
s
% Start of user PostScript
46.857143 71.428571 46.414663 73.834631 MAarrowhead1
% End of user PostScript
.44033 .91121 m
.4345 .94235 L
s
% Start of user PostScript
46.857143 75.714286 46.476356 77.746294 MAarrowhead1
% End of user PostScript
.44033 .97688 m
.43543 1.00363 L
s
% Start of user PostScript
46.857143 80. 46.537294 81.745083 MAarrowhead1
% End of user PostScript
.50382 .05741 m
.50328 .02449 L
s
% Start of user PostScript
51. 20. 50.964661 17.852004 MAarrowhead1
% End of user PostScript
.50382 .12308 m
.50328 .08635 L
s
% Start of user PostScript
51. 24.285714 50.964558 21.888461 MAarrowhead1
% End of user PostScript
.50382 .18876 m
.50337 .147 L
s
% Start of user PostScript
51. 28.571429 50.970556 25.846493 MAarrowhead1
% End of user PostScript
.50382 .25444 m
.50345 .20672 L
s
% Start of user PostScript
51. 32.857143 50.976229 29.743405 MAarrowhead1
% End of user PostScript
.50382 .32011 m
.50294 .26924 L
s
% Start of user PostScript
51. 37.142857 50.942808 33.823239 MAarrowhead1
% End of user PostScript
.50382 .38579 m
.50086 .34706 L
s
% Start of user PostScript
51. 41.428571 50.806626 38.901185 MAarrowhead1
% End of user PostScript
.50382 .45147 m
.5012 .43953 L
s
% Start of user PostScript
51. 45.714286 50.828985 44.935335 MAarrowhead1
% End of user PostScript
.50382 .51714 m
.50382 .51663 L
s
% Start of user PostScript
51. 50. 51. 49.966215 MAarrowhead1
% End of user PostScript
.50382 .58282 m
.50218 .5893 L
s
% Start of user PostScript
51. 54.285714 50.892933 54.708618 MAarrowhead1
% End of user PostScript
.50382 .6485 m
.50079 .67785 L
s
% Start of user PostScript
51. 58.571429 50.802442 60.4865 MAarrowhead1
% End of user PostScript
.50382 .71418 m
.50252 .75882 L
s
% Start of user PostScript
51. 62.857143 50.915485 65.770529 MAarrowhead1
% End of user PostScript
.50382 .77985 m
.50318 .82293 L
s
% Start of user PostScript
51. 67.142857 50.958236 69.953667 MAarrowhead1
% End of user PostScript
.50382 .84553 m
.503 .88264 L
s
% Start of user PostScript
51. 71.428571 50.946404 73.849932 MAarrowhead1
% End of user PostScript
.50382 .91121 m
.50282 .94271 L
s
% Start of user PostScript
51. 75.714286 50.934665 77.770199 MAarrowhead1
% End of user PostScript
.50382 .97688 m
.50269 1.00403 L
s
% Start of user PostScript
51. 80. 50.926237 81.771597 MAarrowhead1
% End of user PostScript
.56731 .05741 m
.56983 .02489 L
s
% Start of user PostScript
55.142857 20. 55.307825 17.878038 MAarrowhead1
% End of user PostScript
.56731 .12308 m
.57098 .08707 L
s
% Start of user PostScript
55.142857 24.285714 55.382666 21.935502 MAarrowhead1
% End of user PostScript
.56731 .18876 m
.57246 .14768 L
s
% Start of user PostScript
55.142857 28.571429 55.479342 25.890878 MAarrowhead1
% End of user PostScript
.56731 .25444 m
.57355 .20631 L
s
% Start of user PostScript
55.142857 32.857143 55.550298 29.716653 MAarrowhead1
% End of user PostScript
.56731 .32011 m
.56927 .26133 L
s
% Start of user PostScript
55.142857 37.142857 55.270686 33.306865 MAarrowhead1
% End of user PostScript
.56731 .38579 m
.53009 .33267 L
s
% Start of user PostScript
55.142857 41.428571 52.714508 37.962262 MAarrowhead1
% End of user PostScript
.56731 .45147 m
.56731 .45147 L
s
% Start of user PostScript
55.142857 45.714286 MAarrowzero1
% End of user PostScript
.56731 .51714 m
.56731 .51714 L
s
% Start of user PostScript
55.142857 50. MAarrowzero1
% End of user PostScript
.56731 .58282 m
.56731 .58282 L
s
% Start of user PostScript
55.142857 54.285714 MAarrowzero1
% End of user PostScript
.56731 .6485 m
.54179 .68121 L
s
% Start of user PostScript
55.142857 58.571429 53.477491 60.705784 MAarrowhead1
% End of user PostScript
.56731 .71418 m
.56573 .76947 L
s
% Start of user PostScript
55.142857 62.857143 55.040203 66.465372 MAarrowhead1
% End of user PostScript
.56731 .77985 m
.57321 .82392 L
s
% Start of user PostScript
55.142857 67.142857 55.528217 70.018164 MAarrowhead1
% End of user PostScript
.56731 .84553 m
.57247 .88193 L
s
% Start of user PostScript
55.142857 71.428571 55.479762 73.804116 MAarrowhead1
% End of user PostScript
.56731 .91121 m
.57107 .94198 L
s
% Start of user PostScript
55.142857 75.714286 55.388522 77.722162 MAarrowhead1
% End of user PostScript
.56731 .97688 m
.5699 1.00353 L
s
% Start of user PostScript
55.142857 80. 55.312204 81.738919 MAarrowhead1
% End of user PostScript
.63079 .05741 m
.63609 .02634 L
s
% Start of user PostScript
59.285714 20. 59.631224 17.973024 MAarrowhead1
% End of user PostScript
.63079 .12308 m
.63829 .08897 L
s
% Start of user PostScript
59.285714 24.285714 59.774594 22.059453 MAarrowhead1
% End of user PostScript
.63079 .18876 m
.64137 .15024 L
s
% Start of user PostScript
59.285714 28.571429 59.975925 26.058048 MAarrowhead1
% End of user PostScript
.63079 .25444 m
.64612 .20913 L
s
% Start of user PostScript
59.285714 32.857143 60.285998 29.900789 MAarrowhead1
% End of user PostScript
.63079 .32011 m
.65616 .26113 L
s
% Start of user PostScript
59.285714 37.142857 60.941121 33.294081 MAarrowhead1
% End of user PostScript
.63079 .38579 m
.71933 .30099 L
s
% Start of user PostScript
59.285714 41.428571 65.06316 35.894936 MAarrowhead1
% End of user PostScript
.63079 .45147 m
.61864 .45377 L
s
% Start of user PostScript
59.285714 45.714286 58.492778 45.864208 MAarrowhead1
% End of user PostScript
.63079 .51714 m
.62305 .51717 L
s
% Start of user PostScript
59.285714 50. 58.780557 50.001829 MAarrowhead1
% End of user PostScript
.63079 .58282 m
.62225 .58261 L
s
% Start of user PostScript
59.285714 54.285714 58.728201 54.272118 MAarrowhead1
% End of user PostScript
.63079 .6485 m
.71157 .65521 L
s
% Start of user PostScript
59.285714 58.571429 64.556697 59.009436 MAarrowhead1
% End of user PostScript
.63079 .71418 m
.65974 .77206 L
s
% Start of user PostScript
59.285714 62.857143 61.17438 66.634613 MAarrowhead1
% End of user PostScript
.63079 .77985 m
.6471 .8214 L
s
% Start of user PostScript
59.285714 67.142857 60.350003 69.853879 MAarrowhead1
% End of user PostScript
.63079 .84553 m
.64209 .87939 L
s
% Start of user PostScript
59.285714 71.428571 60.022542 73.638008 MAarrowhead1
% End of user PostScript
.63079 .91121 m
.63887 .94013 L
s
% Start of user PostScript
59.285714 75.714286 59.812835 77.601875 MAarrowhead1
% End of user PostScript
.63079 .97688 m
.63674 1.00213 L
s
% Start of user PostScript
59.285714 80. 59.673996 81.647147 MAarrowhead1
% End of user PostScript
.69428 .05741 m
.70178 .02852 L
s
% Start of user PostScript
63.428571 20. 63.918056 18.115124 MAarrowhead1
% End of user PostScript
.69428 .12308 m
.70478 .09191 L
s
% Start of user PostScript
63.428571 24.285714 64.113665 22.251368 MAarrowhead1
% End of user PostScript
.69428 .18876 m
.70909 .1547 L
s
% Start of user PostScript
63.428571 28.571429 64.395016 26.348815 MAarrowhead1
% End of user PostScript
.69428 .25444 m
.71581 .21682 L
s
% Start of user PostScript
63.428571 32.857143 64.833469 30.40273 MAarrowhead1
% End of user PostScript
.69428 .32011 m
.72783 .27968 L
s
% Start of user PostScript
63.428571 37.142857 65.617923 34.504648 MAarrowhead1
% End of user PostScript
.69428 .38579 m
.746 .35247 L
s
% Start of user PostScript
63.428571 41.428571 66.803397 39.25433 MAarrowhead1
% End of user PostScript
.69428 .45147 m
.75388 .43672 L
s
% Start of user PostScript
63.428571 45.714286 67.317899 44.751981 MAarrowhead1
% End of user PostScript
.69428 .51714 m
.75144 .51533 L
s
% Start of user PostScript
63.428571 50. 67.158058 49.881276 MAarrowhead1
% End of user PostScript
.69428 .58282 m
.75214 .59165 L
s
% Start of user PostScript
63.428571 54.285714 67.204268 54.861914 MAarrowhead1
% End of user PostScript
.69428 .6485 m
.74693 .67424 L
s
% Start of user PostScript
63.428571 58.571429 66.864346 60.251306 MAarrowhead1
% End of user PostScript
.69428 .71418 m
.73038 .74946 L
s
% Start of user PostScript
63.428571 62.857143 65.784097 65.159922 MAarrowhead1
% End of user PostScript
.69428 .77985 m
.71739 .81293 L
s
% Start of user PostScript
63.428571 67.142857 64.93658 69.301572 MAarrowhead1
% End of user PostScript
.69428 .84553 m
.71029 .87476 L
s
% Start of user PostScript
63.428571 71.428571 64.472848 73.336121 MAarrowhead1
% End of user PostScript
.69428 .91121 m
.70591 .93714 L
s
% Start of user PostScript
63.428571 75.714286 64.187437 77.40685 MAarrowhead1
% End of user PostScript
.69428 .97688 m
.70301 1.00002 L
s
% Start of user PostScript
63.428571 80. 63.998285 81.509643 MAarrowhead1
% End of user PostScript
.75777 .05741 m
.76686 .0312 L
s
% Start of user PostScript
67.571429 20. 68.164539 18.290023 MAarrowhead1
% End of user PostScript
.75777 .12308 m
.77017 .09554 L
s
% Start of user PostScript
67.571429 24.285714 68.380699 22.488664 MAarrowhead1
% End of user PostScript
.75777 .18876 m
.77471 .15987 L
s
% Start of user PostScript
67.571429 28.571429 68.676559 26.686106 MAarrowhead1
% End of user PostScript
.75777 .25444 m
.78095 .22472 L
s
% Start of user PostScript
67.571429 32.857143 69.084107 30.918052 MAarrowhead1
% End of user PostScript
.75777 .32011 m
.78943 .29161 L
s
% Start of user PostScript
67.571429 37.142857 69.637505 35.283038 MAarrowhead1
% End of user PostScript
.75777 .38579 m
.7985 .36314 L
s
% Start of user PostScript
67.571429 41.428571 70.229279 39.950355 MAarrowhead1
% End of user PostScript
.75777 .45147 m
.804 .43883 L
s
% Start of user PostScript
67.571429 45.714286 70.58827 44.889658 MAarrowhead1
% End of user PostScript
.75777 .51714 m
.80518 .51475 L
s
% Start of user PostScript
67.571429 50. 70.665044 49.843707 MAarrowhead1
% End of user PostScript
.75777 .58282 m
.80399 .59019 L
s
% Start of user PostScript
67.571429 54.285714 70.587561 54.766695 MAarrowhead1
% End of user PostScript
.75777 .6485 m
.79928 .66547 L
s
% Start of user PostScript
67.571429 58.571429 70.280174 59.67906 MAarrowhead1
% End of user PostScript
.75777 .71418 m
.79093 .73744 L
s
% Start of user PostScript
67.571429 62.857143 69.735434 64.375533 MAarrowhead1
% End of user PostScript
.75777 .77985 m
.78246 .80467 L
s
% Start of user PostScript
67.571429 67.142857 69.18273 68.762477 MAarrowhead1
% End of user PostScript
.75777 .84553 m
.77616 .86949 L
s
% Start of user PostScript
67.571429 71.428571 68.771399 72.992368 MAarrowhead1
% End of user PostScript
.75777 .91121 m
.77167 .93355 L
s
% Start of user PostScript
67.571429 75.714286 68.478603 77.172429 MAarrowhead1
% End of user PostScript
.75777 .97688 m
.76851 .99752 L
s
% Start of user PostScript
67.571429 80. 68.272439 81.346688 MAarrowhead1
% End of user PostScript
.82126 .05741 m
.83118 .03412 L
s
% Start of user PostScript
71.714286 20. 72.361563 18.480291 MAarrowhead1
% End of user PostScript
.82126 .12308 m
.83455 .09921 L
s
% Start of user PostScript
71.714286 24.285714 72.581366 22.728168 MAarrowhead1
% End of user PostScript
.82126 .18876 m
.83865 .16465 L
s
% Start of user PostScript
71.714286 28.571429 72.848901 26.998002 MAarrowhead1
% End of user PostScript
.82126 .25444 m
.84379 .23093 L
s
% Start of user PostScript
71.714286 32.857143 73.184871 31.323543 MAarrowhead1
% End of user PostScript
.82126 .32011 m
.84966 .29884 L
s
% Start of user PostScript
71.714286 37.142857 73.567458 35.754852 MAarrowhead1
% End of user PostScript
.82126 .38579 m
.85518 .36918 L
s
% Start of user PostScript
71.714286 41.428571 73.928181 40.344623 MAarrowhead1
% End of user PostScript
.82126 .45147 m
.85899 .44158 L
s
% Start of user PostScript
71.714286 45.714286 74.176491 45.069026 MAarrowhead1
% End of user PostScript
.82126 .51714 m
.8603 .51476 L
s
% Start of user PostScript
71.714286 50. 74.262146 49.844538 MAarrowhead1
% End of user PostScript
.82126 .58282 m
.85926 .58783 L
s
% Start of user PostScript
71.714286 54.285714 74.194373 54.612796 MAarrowhead1
% End of user PostScript
.82126 .6485 m
.85593 .66013 L
s
% Start of user PostScript
71.714286 58.571429 73.977091 59.330156 MAarrowhead1
% End of user PostScript
.82126 .71418 m
.85082 .73052 L
s
% Start of user PostScript
71.714286 62.857143 73.643646 63.923878 MAarrowhead1
% End of user PostScript
.82126 .77985 m
.84528 .79853 L
s
% Start of user PostScript
71.714286 67.142857 73.281767 68.361784 MAarrowhead1
% End of user PostScript
.82126 .84553 m
.84032 .86476 L
s
% Start of user PostScript
71.714286 71.428571 72.95809 72.683477 MAarrowhead1
% End of user PostScript
.82126 .91121 m
.83636 .93 L
s
% Start of user PostScript
71.714286 75.714286 72.700112 76.94084 MAarrowhead1
% End of user PostScript
.82126 .97688 m
.8332 .99489 L
s
% Start of user PostScript
71.714286 80. 72.493399 81.175119 MAarrowhead1
% End of user PostScript
.88475 .05741 m
.89502 .03701 L
s
% Start of user PostScript
75.857143 20. 76.527436 18.669101 MAarrowhead1
% End of user PostScript
.88475 .12308 m
.89815 .10266 L
s
% Start of user PostScript
75.857143 24.285714 76.731739 22.953072 MAarrowhead1
% End of user PostScript
.88475 .18876 m
.9018 .16878 L
s
% Start of user PostScript
75.857143 28.571429 76.970009 27.26765 MAarrowhead1
% End of user PostScript
.88475 .25444 m
.90581 .23563 L
s
% Start of user PostScript
75.857143 32.857143 77.231575 31.630106 MAarrowhead1
% End of user PostScript
.88475 .32011 m
.90997 .30366 L
s
% Start of user PostScript
75.857143 37.142857 77.503057 36.069064 MAarrowhead1
% End of user PostScript
.88475 .38579 m
.91381 .37314 L
s
% Start of user PostScript
75.857143 41.428571 77.753616 40.603278 MAarrowhead1
% End of user PostScript
.88475 .45147 m
.91642 .44368 L
s
% Start of user PostScript
75.857143 45.714286 77.923752 45.20581 MAarrowhead1
% End of user PostScript
.88475 .51714 m
.91744 .51489 L
s
% Start of user PostScript
75.857143 50. 77.990317 49.853201 MAarrowhead1
% End of user PostScript
.88475 .58282 m
.91678 .58608 L
s
% Start of user PostScript
75.857143 54.285714 77.947363 54.498486 MAarrowhead1
% End of user PostScript
.88475 .6485 m
.91454 .65668 L
s
% Start of user PostScript
75.857143 58.571429 77.801587 59.105202 MAarrowhead1
% End of user PostScript
.88475 .71418 m
.91121 .72611 L
s
% Start of user PostScript
75.857143 62.857143 77.584014 63.636056 MAarrowhead1
% End of user PostScript
.88475 .77985 m
.90729 .79412 L
s
% Start of user PostScript
75.857143 67.142857 77.328502 68.073994 MAarrowhead1
% End of user PostScript
.88475 .84553 m
.90352 .86091 L
s
% Start of user PostScript
75.857143 71.428571 77.0823 72.432162 MAarrowhead1
% End of user PostScript
.88475 .91121 m
.9002 .92681 L
s
% Start of user PostScript
75.857143 75.714286 76.865578 76.732323 MAarrowhead1
% End of user PostScript
.88475 .97688 m
.8974 .99222 L
s
% Start of user PostScript
75.857143 80. 76.683238 81.001002 MAarrowhead1
% End of user PostScript
.94823 .05741 m
.95845 .03971 L
s
% Start of user PostScript
80. 20. 80.666858 18.845245 MAarrowhead1
% End of user PostScript
.94823 .12308 m
.96129 .1057 L
s
% Start of user PostScript
80. 24.285714 80.852054 23.151666 MAarrowhead1
% End of user PostScript
.94823 .18876 m
.96434 .17216 L
s
% Start of user PostScript
80. 28.571429 81.051018 27.488006 MAarrowhead1
% End of user PostScript
.94823 .25444 m
.9676 .23917 L
s
% Start of user PostScript
80. 32.857143 81.264044 31.861056 MAarrowhead1
% End of user PostScript
.94823 .32011 m
.97078 .30703 L
s
% Start of user PostScript
80. 37.142857 81.471149 36.288823 MAarrowhead1
% End of user PostScript
.94823 .38579 m
.97349 .37579 L
s
% Start of user PostScript
80. 41.428571 81.647813 40.775985 MAarrowhead1
% End of user PostScript
.94823 .45147 m
.97541 .44522 L
s
% Start of user PostScript
80. 45.714286 81.773264 45.306451 MAarrowhead1
% End of user PostScript
.94823 .51714 m
.97619 .51507 L
s
% Start of user PostScript
80. 50. 81.824351 49.864759 MAarrowhead1
% End of user PostScript
.94823 .58282 m
.97579 .58493 L
s
% Start of user PostScript
80. 54.285714 81.798497 54.423269 MAarrowhead1
% End of user PostScript
.94823 .6485 m
.97431 .65438 L
s
% Start of user PostScript
80. 58.571429 81.701961 58.955441 MAarrowhead1
% End of user PostScript
.94823 .71418 m
.97192 .72305 L
s
% Start of user PostScript
80. 62.857143 81.545607 63.436017 MAarrowhead1
% End of user PostScript
.94823 .77985 m
.96913 .79088 L
s
% Start of user PostScript
80. 67.142857 81.363903 67.86243 MAarrowhead1
% End of user PostScript
.94823 .84553 m
.96625 .85777 L
s
% Start of user PostScript
80. 71.428571 81.175652 72.22738 MAarrowhead1
% End of user PostScript
.94823 .91121 m
.96352 .92417 L
s
% Start of user PostScript
80. 75.714286 80.997469 76.559906 MAarrowhead1
% End of user PostScript
.94823 .97688 m
.96105 .98987 L
s
% Start of user PostScript
80. 80. 80.836605 80.84762 MAarrowhead1
% End of user PostScript
.75 g
.49616 .52481 m
.51536 .56292 L
.53427 .59863 L
.55257 .62971 L
.56998 .6542 L
.58623 .67055 L
.60106 .67775 L
.61423 .67534 L
.62555 .66347 L
.63482 .64289 L
.6419 .61488 L
.64669 .58122 L
.6491 .54401 L
.6491 .5056 L
.64669 .46839 L
.6419 .43473 L
.63482 .40673 L
.62555 .38615 L
.61423 .37427 L
.60106 .37186 L
.58623 .37906 L
.56998 .39542 L
.55257 .4199 L
.53427 .45098 L
.51536 .4867 L
.49616 .52481 L
.47695 .56292 L
.45805 .59863 L
.43974 .62971 L
.42233 .6542 L
.40608 .67055 L
.39125 .67775 L
.37808 .67534 L
.36677 .66347 L
.3575 .64289 L
.35041 .61488 L
.34562 .58122 L
.34321 .54401 L
.34321 .5056 L
.34562 .46839 L
.35041 .43473 L
.3575 .40673 L
.36677 .38615 L
.37808 .37427 L
.39125 .37186 L
.40608 .37906 L
.42233 .39542 L
.43974 .4199 L
.45805 .45098 L
.47695 .4867 L
.49616 .52481 L
F
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 296.188},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`0001_2000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00_03oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool0^03oool009l0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`2K0?ooo`00
P`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01X0oooo0P00000I0?ooo`<00000
6P3oool2000001/0oooo0P00000K0?ooo`030000003oool0oooo07l0oooo001V0?ooo`030000003o
ool0oooo01X0oooo0P00000J0?ooo`8000006P3oool3000001T0oooo0`00000J0?ooo`<000006P3o
ool2000001X0oooo0P00000K0?ooo`030000003oool0oooo06<0oooo001:0?ooo`030000003oool0
oooo01T0oooo0P00000K0?ooo`8000006P3oool2000001X0oooo0`00000I0?ooo`<000006@3oool4
000001T0oooo0`00000J0?ooo`8000006P3oool2000001/0oooo00<000000?ooo`3oool0A`3oool0
04X0oooo0P00000J0?ooo`<000006P3oool3000001T0oooo0`00000I0?ooo`@00000603oool40000
01P0oooo1000000I0?ooo`<000006@3oool3000001X0oooo0P00000J0?ooo`8000006P3oool00`00
0000oooo0?ooo`0/0?ooo`00;`3oool00`000000oooo0?ooo`0I0?ooo`8000006P3oool3000001T0
oooo0`00000I0?ooo`<000006@3oool4000001L0oooo1@00000H0?ooo`@00000603oool4000001T0
oooo0`00000I0?ooo`<000006@3oool3000001T0oooo0P00000K0?ooo`030000003oool0oooo0100
oooo000C0?ooo`030000003oool0oooo01T0oooo0P00000J0?ooo`<000006@3oool3000001T0oooo
1000000H0?ooo`@000005`3oool6000001H0oooo1P00000F0?ooo`H000005`3oool4000001P0oooo
1000000H0?ooo`@00000603oool3000001T0oooo0`00000J0?ooo`8000004P3oool001<0oooo0`00
000J0?ooo`8000006@3oool4000001P0oooo1000000H0?ooo`@00000603oool4000001L0oooo1P00
000F0?ooo`H000005P3oool6000001H0oooo1@00000H0?ooo`@000005`3oool4000001T0oooo0`00
000H0?ooo`<000006P3oool2000001<0oooo000D0?ooo`<000006@3oool4000001P0oooo1000000G
0?ooo`D000005`3oool5000001L0oooo1@00000F0?ooo`@000006`3oool00`000000oooo0?ooo`0H
0?ooo`@000005P3oool5000001L0oooo1@00000G0?ooo`@00000603oool4000001P0oooo0`00000I
0?ooo`<000004`3oool001@0oooo1@00000G0?ooo`D000005`3oool5000001L0oooo1@00000F0?oo
o`@00000603oool3000001/0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool3000001H0oooo1@00000G0?oo
o`D000005`3oool4000001P0oooo1000000C0?ooo`005@3oool6000001H0oooo1@00000F0?ooo`H0
00005P3oool4000001L0oooo0P0000000`3oool000000?ooo`0I0?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0G0?ooo`@000005P3oool5000001L0
oooo1@00000G0?ooo`@00000503oool001D0oooo1`00000E0?ooo`H000005P3oool4000001L0oooo
0`00000K0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo01T0oooo0`00000G0?ooo`@000005P3oool5000001L0oooo
1@00000D0?ooo`005P3oool5000001L0oooo1000000G0?ooo`<000006`3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool010000000oooo0000
0000000G0?ooo`@000005P3oool6000001@0oooo000F0?ooo`@00000603oool3000001/0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`<000005`3oool4000001D0oooo000G
0?ooo`030000003oool0oooo0?l0ooooP03oool2000001D0oooo003o0?oook40oooo003o0?oook40
oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40
oooo003o0?oook40oooo003o0?oook40oooo003o0?oook40oooo003H0?ooo`030000003oool0oooo
0=D0oooo002N0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`0K0?ooo`030000003oool0oooo0;L0oooo002N0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool06P3oool2000001`0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`2J0?ooo`00P@3oool00`000000oooo0?ooo`0J0?ooo`8000006`3oool2000001X0oooo0`00
000K0?ooo`8000006`3oool2000001/0oooo00<000000?ooo`3oool0OP3oool00840oooo0P00000K
0?ooo`8000006`3oool2000001X0oooo0`00000K0?ooo`8000006`3oool2000001X0oooo0P00000K
0?ooo`030000003oool0oooo0680oooo001U0?ooo`030000003oool0oooo01T0oooo0P00000K0?oo
o`<000006P3oool2000001X0oooo1000000I0?ooo`<000006P3oool3000001X0oooo0P00000J0?oo
o`800000I03oool004T0oooo00<000000?ooo`3oool06@3oool2000001X0oooo0`00000J0?ooo`<0
00006@3oool4000001T0oooo1000000I0?ooo`@000006@3oool3000001T0oooo0`00000J0?ooo`80
00006`3oool00`000000oooo0?ooo`160?ooo`00B@3oool2000001X0oooo0`00000I0?ooo`@00000
6@3oool4000001P0oooo1000000H0?ooo`D000006@3oool4000001P0oooo1000000H0?ooo`@00000
6@3oool3000001X0oooo0P00000K0?ooo`030000003oool0oooo02X0oooo000]0?ooo`030000003o
ool0oooo01X0oooo0P00000J0?ooo`<000006@3oool4000001P0oooo1000000H0?ooo`D000005`3o
ool6000001L0oooo1@00000H0?ooo`@000005`3oool5000001P0oooo1000000I0?ooo`<000006P3o
ool2000002`0oooo000]0?ooo`<000006P3oool4000001P0oooo1000000H0?ooo`@00000603oool5
000001L0oooo1@00000G0?ooo`H000005`3oool5000001L0oooo1@00000G0?ooo`D000005`3oool4
000001T0oooo0`00000J0?ooo`8000006`3oool00`000000oooo0?ooo`0?0?ooo`004P3oool00`00
0000oooo0?ooo`0I0?ooo`<000006@3oool5000001L0oooo1@00000G0?ooo`D000005`3oool30000
01T0oooo0`00000L0?ooo`030000003oool0oooo01T0oooo0`00000G0?ooo`D000005P3oool60000
01L0oooo1000000I0?ooo`<000006@3oool3000001T0oooo0P00000B0?ooo`004`3oool2000001T0
oooo1000000I0?ooo`D000005P3oool6000001H0oooo1000000K0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0
oooo1000000H0?ooo`@000005P3oool5000001P0oooo1000000H0?ooo`@00000603oool300000180
oooo000C0?ooo`@00000603oool5000001L0oooo1P00000F0?ooo`@000005`3oool2000000030?oo
o`000000oooo01X0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`040000
003oool00000000001L0oooo1000000G0?ooo`D000005`3oool4000001P0oooo0`00000C0?ooo`00
503oool5000001H0oooo1P00000F0?ooo`D000005`3oool3000001`0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0
603oool010000000oooo00000000000F0?ooo`D000005`3oool5000001H0oooo1@00000C0?ooo`00
5@3oool6000001D0oooo1000000H0?ooo`@000006P3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool0603oool4000001H0oooo1P00000E0?ooo`D00000
503oool001H0oooo1000000F0?ooo`@00000603oool010000000oooo0?ooo`00000J0?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0H0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`040000
003oool00000000001L0oooo1000000G0?ooo`@00000503oool001H0oooo1000000G0?ooo`040000
003oool0oooo000001/0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`04
0000003oool00000000001L0oooo0`00000E0?ooo`005`3oool00`000000oooo0?ooo`3o0?oooh40
oooo00<000000?ooo`3oool04`3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool0
0?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool0
0?l0oooo/@3oool00;/0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo0;L0oooo002M0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0
V@3oool009d0oooo0P00000L0?ooo`8000006P3oool3000001/0oooo0P00000L0?ooo`800000V`3o
ool007l0oooo00<000000?ooo`3oool06`3oool2000001`0oooo0P00000J0?ooo`<000006`3oool2
000001`0oooo0P00000L0?ooo`030000003oool0oooo07`0oooo001o0?ooo`800000703oool30000
01/0oooo0P00000J0?ooo`<000006P3oool3000001/0oooo0`00000K0?ooo`800000OP3oool006<0
oooo00<000000?ooo`3oool06@3oool3000001/0oooo0`00000J0?ooo`@000002P3ooolX000000@0
oooo1000000J0?ooo`<000006`3oool2000001/0oooo00<000000?ooo`3oool0H03oool006<0oooo
0P00000K0?ooo`<000006P3oool4000001T0oooo100000030?ooo`L000003P3oool5000001D0oooo
2000000I0?ooo`@000006P3oool3000001X0oooo0P00001R0?ooo`00A`3oool00`000000oooo0?oo
o`0J0?ooo`8000006P3oool4000001T0oooo1000000H0?ooo`P000005@3oool6000001L0oooo3000
000B0?ooo`@000006@3oool4000001T0oooo0`00000K0?ooo`030000003oool0oooo04@0oooo0017
0?ooo`<000006P3oool3000001T0oooo1@00000H0?ooo`D000004@3oool6000000030?ooo`000000
000000<000005`3oool6000001L0oooo1@0000070?ooo`@000003@3oool5000001P0oooo1000000I
0?ooo`<000006`3oool2000004H0oooo000[0?ooo`030000003oool0oooo01X0oooo0`00000I0?oo
o`@00000603oool6000001L0oooo1000000>0?ooo`@000001`3oool3000001/0oooo00<000000?oo
o`3oool06P3oool3000000/0oooo1P0000070?ooo`D00000603oool4000001T0oooo0`00000J0?oo
o`8000006`3oool00`000000oooo0?ooo`0Y0?ooo`00;03oool2000001X0oooo1000000I0?ooo`@0
0000603oool4000001P0oooo0P0000000`3oool000000?ooo`090?ooo`@000003@3oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool04@3oool4000000@0
oooo1000000G0?ooo`D00000603oool4000001P0oooo1000000I0?ooo`8000006`3oool00`000000
oooo0?ooo`0>0?ooo`00403oool2000001X0oooo0`00000J0?ooo`D000005`3oool5000001L0oooo
0`00000L0?ooo`030000003oool0oooo00D0oooo0`00000A0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0E0?ooo`D000006`3oool4000001L0oooo
1@00000G0?ooo`@000006@3oool3000001T0oooo0P00000A0?ooo`004@3oool3000001T0oooo1000
000H0?ooo`H000005`3oool3000001/0oooo00<000000?ooo`3oool06@3oool010000000oooo0?oo
o`3oool4000001@0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo01T0oooo0`00000I0?ooo`040000003oool00000000001H0oooo1@00000G0?ooo`D0
0000603oool3000001T0oooo0`00000A0?ooo`004P3oool5000001H0oooo1P00000G0?ooo`@00000
603oool00`000000oooo0000000K0?ooo`030000003oool0oooo01T0oooo1000000H0?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`040000
003oool0oooo0?ooo`@000005@3oool00`000000oooo0?ooo`0H0?ooo`@000005P3oool6000001H0
oooo1@00000H0?ooo`<000004P3oool001<0oooo1P00000E0?ooo`H000005P3oool4000001/0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0F0?ooo`80000000<0oooo0000003oool0
6P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`050?ooo`8000004`3oool00`000000oooo0?ooo`0H0?ooo`040000
003oool00000000001L0oooo1000000F0?ooo`D000005`3oool500000180oooo000C0?ooo`D00000
5`3oool4000001P0oooo00@000000?ooo`3oool000006P3oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo0180oooo100000030?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool01`3oool4000000h0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`04
0000003oool00000000001L0oooo1000000F0?ooo`D000004`3oool001@0oooo1000000G0?ooo`@0
00006`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool03`3oool2000000L0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0;0?ooo`80
0000303oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3o
ool06P3oool2000001P0oooo1000000C0?ooo`005@3oool2000000030?ooo`000000oooo01H0oooo
00@000000?ooo`3oool000006`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool02`3oool4000000X0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0>0?ooo`@000001`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0000000H0?ooo`<00000503oool001H0oooo
00@000000?ooo`3oool000006P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`080?ooo`8000003P3oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0H0?ooo`030000003oool0oooo0180oooo0P0000050?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0
oooo00@000000?ooo`3oool00000503oool00900oooo0P00002D0?ooo`800000R03oool008l0oooo
00<000000?ooo`3oool0UP3oool2000008H0oooo002<0?ooo`<00000V`3oool3000008<0oooo0029
0?ooo`<00000X@3oool200000840oooo00270?ooo`800000YP3oool2000007l0oooo00250?ooo`80
0000ZP3oool2000007d0oooo00230?ooo`800000=@3oool00`000000oooo0?ooo`0K0?ooo`030000
003oool0oooo01`0oooo00<000000?ooo`3oool0>@3oool2000007/0oooo00210?ooo`800000603o
ool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool06`3o
ool2000001h0oooo00<000000?ooo`3oool0703oool2000007T0oooo00210?ooo`030000003oool0
oooo01L0oooo0P00000M0?ooo`800000703oool2000001`0oooo0P00000M0?ooo`800000803oool2
000007L0oooo001o0?ooo`8000006P3oool2000001d0oooo0P00000K0?ooo`<000006`3oool30000
01d0oooo0P00000R0?ooo`800000M@3oool007`0oooo0`00000L0?ooo`<00000703oool3000001X0
oooo0`00000K0?ooo`<00000703oool3000002<0oooo00<000000?ooo`3oool0L`3oool007/0oooo
0`00000M0?ooo`@000006P3oool4000000d0oooo8P0000080?ooo`D000006P3oool4000001d0oooo
00<000000?ooo`3oool0103oool2000007<0oooo001i0?ooo`8000000P3oool2000001d0oooo1000
000I0?ooo`D000000`3oool9000000d0oooo1000000A0?ooo`d000006P3oool3000001d0oooo0P00
00080?ooo`800000L@3oool00600oooo00<000000?ooo`3oool0503oool2000000@0oooo0`00000L
0?ooo`@000006@3oool8000001D0oooo1@00000H0?ooo`P000005`3oool4000001`0oooo0P00000;
0?ooo`800000K`3oool00600oooo0P00000E0?ooo`030000003oool0oooo00<0oooo1000000K0?oo
o`D00000503oool:000001L0oooo1P00000G0?ooo`H000000P3oool600000140oooo1000000K0?oo
o`<000003@3oool2000000d0oooo00<000000?ooo`3oool0G@3oool00640oooo0P00000C0?ooo`03
0000003oool0oooo00D0oooo1000000J0?ooo`@000004@3oool4000000@0oooo0`00000J0?ooo`H0
00006P3oool3000000P0oooo1000000<0?ooo`D000006P3oool4000000l0oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0?ooo`1N0?ooo`00H@3oool4000000l0oooo0P0000080?ooo`D00000
6@3oool3000000`0oooo1P00000:0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`0<0?ooo`@000002P3oool3000001T0oooo1@00000@0?ooo`030000
003oool0oooo00L0oooo0P00001P0?ooo`00A03oool00`000000oooo0?ooo`0K0?ooo`@00000303o
ool2000000/0oooo0`00000L0?ooo`030000003oool0oooo00P0oooo0P00000A0?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0A0?ooo`@000001@3o
ool00`000000oooo0?ooo`0I0?ooo`D000004P3oool00`000000oooo0?ooo`040?ooo`<00000703o
ool00`000000oooo0?ooo`120?ooo`00A@3oool2000001/0oooo1@00000:0?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`0000007@3oool00`000000oooo0?ooo`040?ooo`<000004`3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool05@3oool4
000000030?ooo`000000oooo01/0oooo1000000C0?ooo`8000000`3oool4000001X0oooo0P000015
0?ooo`00:@3oool2000001X0oooo1000000J0?ooo`D00000203oool00`000000oooo0?ooo`0?0?oo
o`030000003oool0oooo01X0oooo00<000000?ooo`3oool01000000F0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`800000703oool01000
0000oooo00000000000E0?ooo`H000006P3oool3000001/0oooo00<000000?ooo`3oool09`3oool0
02X0oooo0`00000I0?ooo`D00000603oool4000000L0oooo0P00000B0?ooo`030000003oool0oooo
01/0oooo0P00000J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`0I0?ooo`D000006@3oool00`000000oooo0?ooo`0G0?ooo`D000006@3oool30000
01X0oooo0P00000J0?ooo`8000003P3oool000h0oooo0`00000J0?ooo`D000005`3oool6000001L0
oooo00<000000?ooo`0000001P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01H0
oooo1@00000K0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06@3oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo0080oooo0P00000F0?ooo`030000003oool0oooo01T0
oooo0`00000H0?ooo`D00000603oool4000001P0oooo0`00000?0?ooo`003`3oool6000001L0oooo
1P00000F0?ooo`@000006`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01D0oooo
00<000000?ooo`3oool04`3oool2000000@0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`040?ooo`800000503oool00`000000oooo0?ooo`0I0?ooo`040000003oool00000000001H0
oooo1@00000G0?ooo`D000005`3oool400000100oooo000@0?ooo`P00000503oool5000001L0oooo
1000000L0?ooo`040000003oool0oooo000001P0oooo00<000000?ooo`3oool04@3oool2000000L0
oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`070?ooo`8000004@3oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0503oool4000001D0
oooo1P00000F0?ooo`D000004@3oool00140oooo1P00000F0?ooo`@00000603oool010000000oooo
0?ooo`00000L0?ooo`8000006P3oool00`000000oooo0?ooo`0>0?ooo`8000002@3oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo00T0oooo1000000=0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0B0?ooo`040000003oool000000000
01L0oooo1000000E0?ooo`L000004@3oool001<0oooo1000000G0?ooo`80000000<0oooo0000003o
ool06P3oool2000001T0oooo0P00000L0?ooo`030000003oool0oooo00/0oooo0P00000;0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool03@3oool2000000X0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0100oooo00D000000?ooo`3o
ool0oooo0000000G0?ooo`@000005P3oool500000180oooo000D0?ooo`80000000<0oooo00000000
00005P3oool00`000000oooo0?ooo`02000001/0oooo00<000000?ooo`3oool05P3oool00`000000
oooo0000000K0?ooo`030000003oool0oooo00L0oooo1000000>0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool03`3oool3000000P0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool06@3oool010000000
oooo0?ooo`00000G0?ooo`@000004`3oool001D0oooo00D000000?ooo`3oool0oooo0000000J0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool04`3oool2000000<0oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0B0?ooo`8000001@3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo00T0oooo0P00000=0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool0603oool010000000oooo0?ooo`00000D0?ooo`00I@3oool00`000000oooo0?ooo`0U0?oo
o`8000002@3oool00`000000oooo0?ooo`2:0?ooo`<00000:`3oool00`000000oooo0?ooo`1J0?oo
o`00I03oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo00T0oooo0P00002>0?ooo`80
0000:P3oool2000005X0oooo001S0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0
2P3oool2000005/0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo02X0oooo00<000000?ooo`3oool0E`3oool00680oooo00<000000?ooo`3oool0903o
ool2000000d0oooo0`00000R0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool08@3o
ool2000000l0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`1F0?ooo`00H@3oool0
0`000000oooo0?ooo`0S0?ooo`8000003`3oool400000240oooo00<000000?ooo`3oool0=03oool2
000002<0oooo0P00000@0?ooo`800000:`3oool2000005H0oooo001P0?ooo`030000003oool0oooo
0280oooo0P00000B0?ooo`@000007`3oool2000001@0oooo4P00000@0?ooo`<000008@3oool30000
0180oooo0P00000Z0?ooo`030000003oool0oooo05@0oooo001O0?ooo`030000003oool0oooo0240
oooo0P00000D0?ooo`@000007`3oool3000000@0oooo3`0000060?ooo`030000003oool0oooo00T0
oooo3P0000000`3oool000000000000200000200oooo1000000D0?ooo`800000:@3oool2000005@0
oooo001N0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0503oool5000001d0oooo
2000000E0?ooo`800000603oool7000001d0oooo1000000G0?ooo`800000:03oool00`000000oooo
0?ooo`1B0?ooo`00GP3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00P0oooo00<0
00000?ooo`3oool05@3oool4000001P0oooo1P000000103oool0000000000000000H0?ooo`<00000
6@3oool400000080oooo1P00000G0?ooo`@00000603oool00`000000oooo0?ooo`0W0?ooo`030000
003oool0oooo0540oooo001L0?ooo`800000603oool3000000H0oooo0P00000H0?ooo`<000004`3o
ool6000000H0oooo1000000H0?ooo`<00000603oool6000000L0oooo1000000B0?ooo`D000006@3o
ool2000002P0oooo00<000000?ooo`3oool0D03oool005`0oooo00<000000?ooo`3oool0603oool3
000000D0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0<0?ooo`@00000303oool5
000001L0oooo1000000G0?ooo`H000002`3oool4000000l0oooo1000000K0?ooo`8000000P3oool0
0`000000oooo0?ooo`0R0?ooo`030000003oool0oooo04l0oooo001K0?ooo`030000003oool0oooo
01T0oooo1@0000000`3oool000000000000M0?ooo`030000003oool0oooo00T0oooo0`00000@0?oo
o`D000005`3oool4000001X0oooo0`00000?0?ooo`@000002`3oool010000000oooo00000000000M
0?ooo`8000009P3oool00`000000oooo0?ooo`1>0?ooo`00FP3oool00`000000oooo0?ooo`0K0?oo
o`H000007P3oool00`000000oooo0?ooo`050?ooo`@000005@3oool00`000000oooo0?ooo`0F0?oo
o`D000006P3oool00`000000oooo0?ooo`0C0?ooo`@000001`3oool00`000000oooo0?ooo`0M0?oo
o`<000009`3oool00`000000oooo0?ooo`1=0?ooo`00F@3oool00`000000oooo0?ooo`0L0?ooo`L0
00007P3oool00`000000oooo0?ooo`020?ooo`8000006@3oool00`000000oooo0?ooo`0F0?ooo`H0
00006@3oool00`000000oooo0?ooo`0G0?ooo`800000103oool00`000000oooo0?ooo`0M0?ooo`<0
000000<0oooo0000000000009@3oool00`000000oooo0?ooo`1=0?ooo`00F03oool010000000oooo
0?ooo`00000M0?ooo`D000007`3oool5000001/0oooo00<000000?ooo`3oool05P3oool6000001T0
oooo00<000000?ooo`3oool06@3oool01@000000oooo0?ooo`3oool0000001d0oooo1@0000020?oo
o`030000003oool0oooo02@0oooo00<000000?ooo`3oool0C03oool005L0oooo00<000000?ooo`3o
ool00P3oool2000001T0oooo1P00000O0?ooo`<000007P3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool06`3oool3000001d0oooo1@0000040?ooo`80
00005`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo04/0oooo001G0?ooo`030000
003oool0oooo0080oooo1000000G0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3o
ool06P3oool200000080oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool07@3oool4000001/0oooo100000050?ooo`03
0000003oool0oooo01<0oooo0P00000?0?ooo`030000003oool0oooo04X0oooo001F0?ooo`030000
003oool0oooo00@0oooo1@00000D0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
5`3oool2000000@0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`020?ooo`800000
6@3oool3000000L0oooo0P00000A0?ooo`@00000403oool00`000000oooo0?ooo`190?ooo`00@@3o
ool200000180oooo00<000000?ooo`3oool01P3oool600000140oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`0D0?ooo`8000001`3oool00`000000oooo0?ooo`0K0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0L0?ooo`030000003o
ool0oooo00<0oooo0`00000F0?ooo`040000003oool0oooo000000T0oooo0P00000=0?ooo`D00000
4P3oool00`000000oooo0?ooo`050?ooo`800000@@3oool00480oooo1000000>0?ooo`030000003o
ool0oooo00P0oooo1000000A0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool04@3o
ool2000000T0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`060?ooo`800000503o
ool01@000000oooo0?ooo`3oool0000000T0oooo00<000000?ooo`3oool02@3oool6000001@0oooo
00<000000?ooo`3oool00P3oool300000480oooo000W0?ooo`<000006@3oool6000000/0oooo00<0
00000?ooo`3oool0203oool400000100oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0>0?ooo`8000002`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00P0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`02000000/0oooo0P00000:0?ooo`@000005@3o
ool00`000000oooo0?ooo`04000001X0oooo0P00000W0?ooo`00:03oool6000001H0oooo1`000008
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool00P00000=0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool02`3oool2000000h0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06`3oool0
0`000000oooo0?ooo`090?ooo`800000403oool00`000000oooo00000003000000`0oooo00<00000
0?ooo`3oool0203oool4000001H0oooo1@00000H0?ooo`@00000:03oool000d0oooo1000000H0?oo
o`P00000503oool5000000P0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool02@3oool2000000l0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0<0?ooo`8000003@3oool6000000h0oooo00<000000?oo
o`3oool01P3oool010000000oooo0?ooo`00000E0?ooo`L000005@3oool6000001T0oooo0`00000=
0?ooo`003P3oool8000001@0oooo1P00000F0?ooo`D000001P3oool00`000000oooo0?ooo`0B0?oo
o`030000003oool0oooo00L0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`080?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0>0?oo
o`8000002P3oool600000100oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0H0?oo
o`D00000503oool7000001H0oooo1P00000>0?ooo`00403oool6000001H0oooo1000000G0?ooo`05
0000003oool0oooo0?ooo`0000001@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
0140oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo
00L0oooo00<000000?ooo`0000001000000A0?ooo`040000003oool0oooo0?ooo`8000006P3oool6
000001D0oooo1P00000C0?ooo`P000003`3oool00140oooo1@00000G0?ooo`80000000<0oooo0000
000000006P3oool200000080oooo00<000000?ooo`3oool05@3oool2000000<0oooo0P00000F0?oo
o`030000003oool0oooo0080oooo0`00000D0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool04@3oool3000000H0oooo00<000000?ooo`0000000`00000C0?ooo`<000006`3oool01@00
0000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool04P3oool5000001@0oooo1`00000@
0?ooo`004P3oool3000000030?ooo`000000000001H0oooo00@000000?ooo`3oool0oooo0P00000J
0?ooo`8000006P3oool00`000000oooo0000000I0?ooo`040000003oool0oooo000001P0oooo00<0
00000?ooo`3oool0603oool010000000oooo0?ooo`3ooolh000001H0oooo00<000000?ooo`3oool0
5@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00@00000503oool00`000000oooo
0?ooo`0H0?ooo`8000001`3oool00`000000oooo0?ooo`0@0?ooo`80000000@0oooo000000000000
00005P3oool400000180oooo000D0?ooo`040000003oool0oooo0?ooo`8000006P3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0000001X0oooo0P00000J0?ooo`<00000603oool00`000000
oooo0?ooo`0;0?oooa400000603oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo0080
oooo2@00000=0?ooo`030000003oool0oooo01H0oooo0P0000001@3oool000000?ooo`3oool00000
0080oooo00<000000?ooo`3oool04P3oool00`000000oooo0000000H0?ooo`030000003oool0oooo
00P0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
01<0oooo1@00000C0?ooo`00C@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool08@3oool500000580oooo1@00000R0?ooo`030000003oool0oooo0080
00006`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3o
ool04P3oool004d0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0H0?ooo`800000
7`3oool5000005`0oooo0`00000P0?ooo`8000007P3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo03l0oooo001<0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0603o
ool00`000000oooo0?ooo`0L0?ooo`800000I03oool3000001d0oooo0P00000N0?ooo`030000003o
ool0oooo0240oooo00<000000?ooo`3oool0?`3oool004`0oooo00<000000?ooo`3oool06P3oool0
0`000000oooo0?ooo`0G0?ooo`8000006`3oool4000006T0oooo0`00000I0?ooo`030000003oool0
oooo008000007@3oool200000280oooo00<000000?ooo`3oool0?P3oool004/0oooo00<000000?oo
o`3oool06P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01T0oooo0P00001`0?oo
o`030000003oool0oooo01D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0L0?oo
o`030000003oool0oooo0240oooo00<000000?ooo`3oool0?@3oool004/0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01L0oooo0`0000160?ooo`03
0000003oool0oooo02X0oooo0`00000C0?ooo`030000003oool0oooo00@0oooo0P00000M0?ooo`03
0000003oool0oooo0200oooo00<000000?ooo`3oool0?@3oool004X0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
A`3oool3000002`0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0l0?ooo`00BP3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool05P3o
ool2000002d0oooo00<000000?ooo`3oool06`3oool3000002`0oooo0`00000>0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0P0?ooo`030000003o
ool0oooo03/0oooo00190?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0?ooo`0E0?ooo`800000;P3oool2000001d0oooo1000000^0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0
oooo0200oooo00<000000?ooo`3oool0>`3oool004P0oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`0F0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0;@3oool30000
01d0oooo1P00000]0?ooo`8000002@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0j0?ooo`00A`3oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool05@3oool00`000000
oooo0?ooo`0]0?ooo`<000007`3oool6000002d0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool07`3oool00`000000
oooo0?ooo`0j0?ooo`00A`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00H0oooo
0P00000>0?ooo`030000003oool0oooo01<0oooo0P00000`0?ooo`<00000403oool00`000000oooo
0?ooo`0<0?ooo`D00000;`3oool3000000@0oooo00<000000?ooo`3oool0403oool2000001`0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0i0?ooo`00A`3oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo00P0oooo0`00000;0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool0;`3oool400000100oooo00<000000?ooo`3oool03@3oool300000380oooo00D0
00000?ooo`3oool0oooo0000000D0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0
7`3oool00`000000oooo0?ooo`0i0?ooo`00AP3oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo00T0oooo1@0000070?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0;`3o
ool500000100oooo0P00000>0?ooo`040000003oool0oooo00000380oooo0`00000F0?ooo`800000
203oool200000140oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0h0?ooo`00A@3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00/0oooo1P0000040?ooo`030000003o
ool0oooo01<0oooo00<000000?ooo`3oool05P3oool800000140oooo1@00000A0?ooo`8000004@3o
ool00`000000oooo0?ooo`0;0?ooo`l000005`3oool2000001L0oooo00<000000?ooo`3oool0103o
ool300000180oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0g0?ooo`00A@3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00`0oooo100000040?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool04P3oool5000000P0oooo2000000:0?ooo`@000004@3oool2
00000180oooo00<000000?ooo`3oool0103oool6000000l0oooo0`00000C0?ooo`040000003oool0
oooo000001L0oooo00<000000?ooo`3oool01@00000D0?ooo`030000003oool0oooo01l0oooo00<0
00000?ooo`3oool0=`3oool004@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0>
0?ooo`@000000P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00800000403oool0
0`000000oooo0?ooo`0B0?ooo`D000001`3oool300000100oooo1000000B0?ooo`030000003oool0
oooo00<00000603oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool05@3oool6000001H0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0g
0?ooo`00A03oool00`000000oooo0?ooo`0?0?ooo`<000001P3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo00<00000503oool00`000000oooo0?ooo`020?ooo`<000002`3oool30000
01X0oooo1`00000C0?ooo`@000003`3oool6000001`0oooo0`00000=0?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool0503oool6000001L0oooo00<000000?ooo`3oool07P3oool00`00
0000oooo0?ooo`0f0?ooo`00A03oool00`000000oooo0?ooo`0@0?ooo`H0000000<0oooo0000003o
ool05P3oool2000001<0oooo00<000000?ooo`3oool0103oool5000000L0oooo00<000000?ooo`3o
ool08@3oool300000140oooo1@00000:0?ooo`@00000103oool00`000000oooo0?ooo`0N0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0C0?ooo`D0
0000603oool3000001h0oooo00<000000?ooo`3oool0=P3oool004@0oooo00<000000?ooo`3oool0
4@3oool8000001H0oooo00@000000?ooo`0000000000403oool00`000000oooo0?ooo`060?ooo`H0
00000`3oool2000002<0oooo00@000000?ooo`3oool0oooo200000090?ooo`D000000P3oool80000
00T0oooo00<000000?ooo`3oool07P3oool2000000L0oooo0P00000:0?ooo`030000003oool0oooo
0140oooo00<000000?ooo`0000000P00000E0?ooo`H00000803oool00`000000oooo0?ooo`0e0?oo
o`00?P3oool6000001D0oooo1P00000G0?ooo`040000003oool0oooo0?ooo`8000003@3oool00`00
0000oooo0?ooo`070?ooo`D000000`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo
00T0oooo4000000A0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`0:0?ooo`030000003oool0oooo0100oooo00D000000?ooo`3oool0oooo0000000C
0?ooo`T000005P3oool3000000L0oooo00<000000?ooo`3oool0=@3oool003l0oooo2@00000C0?oo
o`@000005@3oool2000000H0oooo00<000000?ooo`3oool02@3oool2000000X0oooo1`00000U0?oo
o`030000003oool0oooo01D0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0P0?oo
o`8000000P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0oooo0P0000050?oo
o`030000003oool0oooo0100oooo1`0000020?ooo`030000003oool0oooo00l0oooo1P0000090?oo
o`030000003oool0oooo03@0oooo000U0?ooo`D000005`3oool6000001D0oooo0P0000000`3oool0
00000000000C0?ooo`030000003oool0oooo00H0oooo0P0000080?ooo`030000003oool0oooo00/0
oooo0P0000000`3oool000000000000V0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3o
ool05@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool000000100oooo00<000000?ooo`3o
ool02`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0100oooo100000050?ooo`03
0000003oool0oooo00/0oooo2000000:0?ooo`030000003oool0oooo00`0oooo0P00000V0?ooo`00
2`3oool5000001H0oooo2@00000C0?ooo`D000005@3oool2000000<0oooo0P00000@0?ooo`800000
2P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00`0oooo0P0000020?ooo`800000
1P3oool90;no_a@0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0F0?ooo`030000
003oool0oooo00d0oooo1`2o_kl<0?ooo`8000004@3oool00`000000oooo0?ooo`080?ooo`800000
2P3oool00`000000oooo0?ooo`0>0?ooo`@000001P3oool00`000000oooo0?ooo`0;0?ooo`L00000
303oool00`000000oooo0?ooo`070?ooo`D000005P3oool5000000`0oooo000<0?ooo`T000004`3o
ool6000001@0oooo1`00000B0?ooo`030000003oool0oooo00D0oooo0P00000=0?ooo`030000003o
ool0oooo00/0oooo0P0000040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`030?ooo`/0_kno4P3oool00`000000oooo0?ooo`0G0?ooo`030000003o
ool0oooo01D0oooo00<000000?ooo`3oool02`3oool:0;no_`X0oooo00<000000?ooo`0000004@3o
ool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0303o
ool200000080oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0<0?ooo`@000003P3o
ool00`000000oooo0?ooo`040?ooo`H000004`3oool9000000d0oooo000>0?ooo`L00000503oool5
000001<0oooo00D000000?ooo`3oool0oooo000000030?ooo`800000403oool00`000000oooo0?oo
o`070?ooo`8000002`3oool00`000000oooo0?ooo`0=0?ooo`80000000<0oooo0000003oool03P3o
ool00`000000oooo0?ooo`050?ooo`8000000P3oool>0;no_`l0oooo00<000000?ooo`3oool0603o
ool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo00L0oooo3P2o_kl80?ooo`050000003o
ool0oooo0?ooo`0000004@3oool00`000000oooo0?ooo`040?ooo`8000003P3oool00`000000oooo
0?ooo`090?ooo`8000003@3oool00`000000oooo0?ooo`090?ooo`D00000403oool00`000000oooo
0?ooo`030?ooo`D00000503oool7000000l0oooo000?0?ooo`H000005@3oool3000000040?ooo`00
00000000000000l0oooo00<000000?ooo`3oool0203oool2000000h0oooo00<000000?ooo`3oool0
2@3oool2000000T0oooo00<000000?ooo`3oool03`3oool2000000h0oooo00<000000?ooo`3oool0
203oool00`000000_kno0;no_`0>0;no_`h0oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo00D0oooo402o_kl60?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool01`3oool200000100oooo00<000000?ooo`3oool01P3oool2000000<0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`030?ooo`@000005P3oool500000100oooo
000A0?ooo`<0000000@0oooo0000000000000000503oool00`000000oooo0?ooo`020?ooo`800000
3@3oool00`000000oooo0?ooo`0:0?ooo`8000002`3oool2000000d0oooo0P0000060?ooo`030000
003oool0oooo0100oooo0`00000=0?ooo`030000003oool0oooo00P0oooo4P2o_kl<0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`040?oooa80_kno
103oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00l0oooo00@000000?ooo`000000
00004P3oool00`000000oooo0?ooo`040?ooo`8000004P3oool00`000000oooo0?ooo`040?ooo`80
0000603oool00`000000oooo0?ooo`02000000030?ooo`000000oooo01H0oooo1@00000A0?ooo`00
4`3oool00`000000oooo0?ooo`020?ooo`8000006@3oool2000000/0oooo00<000000?ooo`3oool0
303oool2000000P0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo0100oooo00D000000?ooo`3oool0oooo0000000;0?ooo`030000003oool0oooo00P0
oooo502o_kl;0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`020?oooa@0_kno0P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00l0
oooo0P00000D0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool04P3oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo01P0oooo0`00000I0?ooo`8000000P3oool00`000000
oooo0?ooo`0A0?ooo`00@03oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00P0oooo
5@2o_km50?oooaH0_kno2`3oool00`000000oooo0?ooo`0>0?ooo`8000005@3oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0<03oool003l0oooo00<0
00000?ooo`3oool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`3oool03@3oool00`000000oooo0?ooo`080?ooo`030000002o_kl0_kno01@0_kno@`3o
oolG0;no_`030000003oool0oooo00P0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?oo
o`0`0?ooo`00?`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00P0oooo00<00000
0;no_`2o_kl05P2o_km10?oooaT0_kno00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0N
0?ooo`030000003oool0oooo02l0oooo000n0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool0203oool00`000000oooo0;no_`0H0;no_cl0oooo6P2o_kl00`000000oooo0?ooo`090?oo
o`030000003oool0oooo00h0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo01h0oooo00<000000?ooo`3oool0;`3oool003h0oooo00<000000?ooo`3o
ool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool0303oool00`000000oooo0?ooo`080?ooo`030000002o_kl0_kno01T0_kno?P3ooolK0;no_``0
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0^0?ooo`00?P3oool00`000000oooo
0?ooo`0F0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo00T0oooo00<000000;no_`2o_kl06P2o_kll0?oooa`0_kno
303oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo02h0oooo000m0?ooo`800000603o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`090?oooah0_kno00<000000?ooo`3oool0=P3oool00`000000_kno0;no
_`0L0;no_``0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0^0?ooo`00?@3oool0
0`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00T0oooo7P2o_kl00`000000oooo0?ooo`0e
0?ooo`030000003oool0_kno01d0_kno303oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0M0?ooo`030000003oool0
oooo02h0oooo000m0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04`3oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0203ooolP0;no
_`800000=03oool00`000000oooo0;no_`0N0;no_`d0oooo00<000000?ooo`3oool03P3oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`00
0000oooo0?ooo`0]0?ooo`00?03oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00T0
oooo8@2o_kl00`3oool000000?ooo`0a0?ooo`030000003oool0_kno0200_kno303oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo02d0oooo000l0?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool02@3ooolQ0;no_`80oooo00<000000?ooo`3oool0;P3oool00`000000oooo
0?ooo`0Q0;no_``0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo01L0oooo0P00000O0?ooo`030000003oool0oooo02`0oooo000l0?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool02P3ooolR0;no_`80oooo00<000000?ooo`3oool0;03o
ool00`000000oooo0?ooo`0R0;no_`d0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0C0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?oo
o`0/0?ooo`00?03oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01<0oooo00<00000
0?ooo`3oool03`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00T0oooo902o_kl2
0?ooo`030000003oool0oooo02T0oooo0P0000020?ooob<0_kno3@3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0M
0?ooo`030000003oool0oooo02`0oooo000k0?ooo`030000003oool0oooo01H0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool02@3ooolT0;no_`<0oooo0P00000Y0?ooo`030000003oool0oooo02D0_kno303oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`0M0?ooo`030000003oool0oooo02`0oooo000k0?ooo`030000003oool0oooo01H0
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00d0
oooo00<000000?ooo`3oool02P3ooolU0;no_`<0oooo00<000000?ooo`3oool09@3oool2000000<0
oooo9@2o_kl=0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool04`3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0:`3oool003/0oooo
00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0;0?ooobH0_kno0`3oool00`000000oooo
0?ooo`0T0?ooo`040000003oool0oooo0?ooobH0_kno3@3oool00`000000oooo0?ooo`0?0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo02/0oooo000k0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
2P3ooolW0;no_`@0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`020?ooobH0_kno
3P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02X0oooo000j0?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool02P3ooolX0;no_`<0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`020?ooobP0_kno3@3oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo0180oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo02X0oooo000j0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02P3ooolX
0;no_`@0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`020?ooobP0_kno3P3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02X0oooo000j0?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
00d0oooo00<000000?ooo`3oool02`3ooolY0;no_`<0oooo00<000000?ooo`3oool07`3oool00`00
0000oooo0?ooo`020?ooobT0_kno3P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo
01<0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo
02X0oooo000j0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool04`3oool:000000P0
oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`090?ooo`030000002o_kl0_kno02P0
_kno0`3oool200000200oooo00@000000?ooo`3oool0oooo:P2o_kl2000000`0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`040?ooo`X000001@3oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo01d0oooo00<000000?ooo`3oool0:@3oool003T0oooo00<000000?ooo`3o
ool05P3oool;000000X0oooo00<000000?ooo`000000200000080?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0;no_`0Z0;no_`<0oooo00<000000?ooo`3o
ool07@3oool00`000000oooo0?ooo`020?ooob/0_kno00<000000?ooo`3oool02P3oool00`000000
oooo0?ooo`0?0?ooo`030000003oool0oooo00@0oooo2@0000060?ooo`030000003oool0oooo00T0
oooo1@0000090?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0:@3oool003T0oooo
00<000000?ooo`3oool02P00000<0?ooo`030000003oool0000000P000002P3oool010000000oooo
0?ooo`3oool9000000H0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`090?ooo`03
0000003oool0_kno02/0_kno0`3oool200000100oooo00<000000?ooo`3oool02@3oool00`000000
oooo0?ooo`020?ooob`0_kno00<0oooo0000003oool02`3oool00`000000oooo0?ooo`0?0?ooo`03
0000003oool0oooo00@0oooo1P0000080?ooo`030000003oool0oooo00@0oooo2@00000;0?ooo`03
0000003oool0oooo01d0oooo00<000000?ooo`3oool0:03oool002T0oooo1000000<0?ooo`040000
003oool0oooo0?ooo`T000002`3oool00`000000oooo0?ooo`020?ooo`L000002P3oool00`000000
oooo0?ooo`030?ooo`@000000`3oool4000000030?ooo`000000oooo00h0oooo00<000000?ooo`3o
ool02@3oool00`000000oooo0;no_`0[0;no_`D0oooo00<000000?ooo`3oool03@3oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo00<0oooo;02o_kl00`3oool000000?ooo`0;0?ooo`03
0000003oool0oooo00l0oooo00<000000?ooo`3oool02@0000090?ooo`030000003oool0oooo00@0
oooo1`00000=0?ooo`040000003oool0oooo0?ooo`X000003`3oool9000002D0oooo000:0?ooo`X0
0000403oool9000000`0oooo00<000000?ooo`3oool00`3oool7000000/0oooo00<000000?ooo`3o
ool0103oool4000000030?ooo`0000000000008000001P3oool00`000000oooo0?ooo`050?ooo`80
00001`3oool4000000d0oooo00<000000?ooo`3oool02@3oool00`000000oooo0;no_`0/0;no_`D0
oooo0P00000=0?ooo`8000001`3oool2000000D0oooo;@2o_kl00`3oool000000?ooo`0;0?ooo`03
0000003oool0oooo00h0oooo100000050?ooo`<000002`3oool010000000oooo0?ooo`3oool80000
00h0oooo00@000000?ooo`3oool0oooo2@00000@0?ooo`P000004@3oool:000000/0oooo000;0?oo
o`T000004P3oool7000000/0oooo00<000000?ooo`3oool01P3oool4000000030?ooo`0000000000
008000001`3oool00`000000oooo0?ooo`060?ooo`8000001@3oool4000000030?ooo`000000oooo
0140oooo00<000000?ooo`3oool01@0000080?ooo`030000003oool0oooo00T0oooo00<000000;no
_`2o_kl0;02o_kl60?ooo`030000003oool0oooo00/0oooo0P0000060?ooo`030000003oool0oooo
00D0oooo;@2o_kl00`3oool000000?ooo`0;0?ooo`030000003oool0oooo00/0oooo0`0000000`3o
ool000000?ooo`060?ooo`030000003oool0oooo00/0oooo100000030?ooo`<00000403oool00`00
0000oooo0?ooo`0800000180oooo1P00000C0?ooo`T00000303oool000d0oooo2P00000A0?ooo`X0
00001P3oool00`000000oooo0?ooo`080?ooo`8000001@3oool4000000<0oooo00<000000?ooo`3o
ool04@3oool400000100oooo00<000000?ooo`3oool01@3oool4000000<0oooo00<000000?ooo`3o
ool02P3oool00`000000_kno0;no_`0]0;no_`H0oooo0P00000;0?ooo`800000103oool2000000L0
oooo;P2o_kl00`3oool000000?ooo`0<0?ooo`030000003oool0oooo00H0oooo100000050?ooo`03
0000003oool0oooo00l0oooo1@0000050?ooo`030000003oool0oooo00l0oooo100000020?ooo`@0
00004@3oool3000000030?ooo`000000000000800000503oool7000000h0oooo000?0?ooo`@00000
103oool300000140oooo0P0000050?ooo`<000000`3oool00`000000oooo0?ooo`0C0?ooo`<00000
5@3oool00`000000oooo0?ooo`03000000`0oooo00<000000?ooo`3oool02P3oool4000000`0oooo
00<000000;no_`2o_kl0;@2o_kl80?ooo`800000203oool400000080oooo00<000000?ooo`3oool0
1P3oool_0;no_`030?ooo`000000oooo00`0oooo00<000000?ooo`3oool0103oool2000000T0oooo
00<000000?ooo`3oool03@3oool2000000@0oooo00<000000?ooo`3oool04P3oool300000080oooo
00@000000?ooo`3oool0oooo0P00000A0?ooo`800000103oool400000140oooo0`0000000`3oool0
00000000000200000100oooo000A0?ooo`8000009@3oool00`000000oooo0?ooo`0E0?ooo`030000
003oool0oooo01<0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo00X0oooo00<000000;no_`2o_kl0;P2o_kl90?ooo`8000001P3oool6000000T0oooo
<02o_kl00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01X0oooo00@000000?ooo`3oool0
00005@3oool200000180oooo000g0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
2P3oool00`000000_kno0;no_`0_0;no_`X0oooo2`0000090?oooc40_kno00<000000?ooo`3oool0
2`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
5`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02L0oooo000g0?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0@0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool02P3ooolb0;no_a00oooo1@0000090?oooc40_kno00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool05`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo02L0oooo000g
0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02`3ooolc0;no_`l0oooo0`00000:
0?oooc80_kno00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0?0?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0
oooo02L0oooo000g0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02`3ooolc0;no
_a`0oooo<P2o_kl00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01d0oooo00<000000?oo
o`3oool09P3oool003L0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0oooo00l0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0;0?oooc@0
_kno6P3ooolc0;no_`030000003oool0oooo00`0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool07@3oool00`000000
oooo0?ooo`0V0?ooo`00=`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00X0oooo
=@2o_klJ0?oooc<0_kno00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo01<0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0L0?ooo`030000
003oool0oooo02H0oooo000f0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04P3o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02P3o
oolf0;no_aP0oooo=@2o_kl>0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04`3o
ool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool09P3o
ool003H0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0:0?ooocH0_kno603ooole
0;no_`l0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0
oooo01L0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0U0?ooo`00=P3oool00`00
0000oooo0?ooo`0F0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo00X0oooo=`2o_klF0?ooocH0_kno3`3oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`0M0?ooo`030000003oool0oooo02D0oooo000f0?ooo`030000003oool0oooo01H0
oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool02P3ooolg0;no_aH0oooo=P2o_kl?0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01d0
oooo00<000000?ooo`3oool09@3oool003H0oooo00<000000?ooo`3oool05@3oool00`000000oooo
0?ooo`0C0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`0;0?ooocP0_kno503ooolg0;no_`l0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0B0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool07@3oool00`000000oooo
0?ooo`0U0?ooo`00=P3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00/0oooo>02o
_klD0?ooocL0_kno3`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool05`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo02D0oooo000f
0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02`3oooli0;no_a80oooo>02o_kl?
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0G
0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool09@3oool003H0oooo00<000000?oo
o`3oool05@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`0;0?ooocT0_kno4@3oooli0;no_`l0oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01L0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`0T0?ooo`00=P3oool00`000000oooo0?ooo`0E0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00/0oooo>P2o_kl@0?ooocT0_kno3`3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo02@0oooo000f0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
2`3ooolj0;no_`l0oooo>P2o_kl@0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0
903oool003D0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0;0?oooc/0_kno3P3o
oolj0;no_a00oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0oooo01L0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0T0?ooo`00=@3oool0
0`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool03`3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00/0oooo>`2o_kl=0?oooc/0_kno403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo02@0oooo000e0?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool02`3oooll0;no_``0oooo>`2o_kl@0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
01`0oooo00<000000?ooo`3oool0903oool003D0oooo00<000000?ooo`3oool05@3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`0;0?oooc`0_kno2`3oooll0;no_a00oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0703oool00`000000
oooo0?ooo`0T0?ooo`00=@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0180oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00/0oooo
?@2o_kl:0?oooc`0_kno403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo02@0oooo
000e0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02P3oooln0;no_`T0oooo?@2o
_kl@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`0G0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool08`3oool003D0oooo00<00000
0?ooo`3oool05@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool03P3oool00`000000oooo0?ooo`0:0?ooocl0_kno1`3oooln0;no_a00oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01L0oooo00<00000
0?ooo`3oool07@3oool00`000000oooo0?ooo`0S0?ooo`00=@3oool00`000000oooo0?ooo`0;0?oo
o`800000203oool00`000000oooo0?ooo`0:0?ooo`<000001@3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo00h0oooo00<000000?ooo`3oool02P3ooolo0;no_`L0oooo?P2o_kl@0?oo
o`030000003oool0oooo0100oooo0`00000C0?ooo`030000003oool00000008000005@3oool20000
01h0oooo00<000000?ooo`3oool08`3oool000l0oooo1000000E0?ooo`D00000203oool00`000000
oooo0?ooo`070?ooo`H00000203oool00`000000oooo0?ooo`060?ooo`L000001@3oool00`000000
oooo0?ooo`0<0?ooo`<0000000<0oooo0000003oool03`3oool00`000000oooo0?ooo`0:0?oood00
_kno1@3ooolo0;no_a00oooo00<000000?ooo`3oool0403oool7000000l0oooo00<000000?ooo`00
00001P00000A0?ooo`H000004P3oool5000000<0oooo00<000000?ooo`3oool03P3oool500000100
oooo000:0?ooo`T00000403oool:000000P0oooo00<000000?ooo`3oool0103oool:000000L0oooo
00<000000?ooo`3oool00`3oool<000000<0oooo00<000000?ooo`3oool0203oool7000000030?oo
o`000000oooo00l0oooo00<000000?ooo`3oool02P3ooom00;no_`D0oooo?`2o_kl@0?ooo`030000
003oool0oooo00h0oooo3000000<0?ooo``000003P3oool9000000l0oooo2P00000?0?ooo`X00000
2`3oool000`0oooo3P00000;0?oooa400000203oool7000000030?ooo`000000000000T00000103o
ool7000000<0oooo:@00000;0?oood40_kno0`3ooom00;no_a00oooo00@000000?ooo`3oool0oooo
3@0000020?ooo`P00000103oool;000000030?ooo`000000000000H000001`3oool@000000`0oooo
3P00000=0?ooo`/000003@3oool000h0oooo1@00000E0?ooo`D00000203oool00`000000oooo0?oo
o`080?ooo`D00000203oool00`000000oooo0?ooo`070?ooo`D000001P3oool00`000000oooo0?oo
o`080?ooo`L0000000<0oooo0000003oool03`3oool00`000000oooo0?ooo`0:0?oood40_kno0`3o
oom00;no_a00oooo00<000000?ooo`3oool0403oool600000100oooo00<000000?ooo`0000001000
000C0?ooo`D000004`3oool5000000<0oooo00<000000?ooo`3oool03`3oool5000000l0oooo000A
0?ooo`800000603oool2000000P0oooo00<000000?ooo`3oool02`3oool2000000P0oooo00<00000
0?ooo`3oool02P3oool2000000H0oooo00<000000?ooo`3oool0303oool3000000030?ooo`000000
oooo00h0oooo00<000000?ooo`3oool02`3ooom20;no_`030?ooo`2o_kl0_kno03l0_kno403oool0
0`000000oooo0?ooo`0@0?ooo`<000004`3oool010000000oooo00000000000F0?ooo`8000005P3o
ool2000000H0oooo00<000000?ooo`3oool03`3oool200000180oooo000e0?ooo`030000003oool0
oooo01D0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo00d0oooo00<000000?ooo`3oool02`3ooon40;no_a00oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0703oool0
0`000000oooo0?ooo`0S0?ooo`00=@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
00/0oooo@P2o_kl00`3oool0_kno0;no_`0o0;no_a00oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0703oool00`00
0000oooo0?ooo`0S0?ooo`00=@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0180
oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00/0
oooo@P2o_kl20?oood00_kno403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo02<0
oooo000e0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02`3ooom10;no_`<0oooo
@02o_kl@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool08`3oool003D0oooo00<0
00000?ooo`3oool05@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`3oool03@3oool00`000000oooo0?ooo`0;0?oood40_kno0`3oool00`000000_kno0;no
_`0m0;no_a00oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0D0?ooo`030000003o
ool0oooo01L0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0S0?ooo`00=@3oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00/0oooo@02o_kl30?ooo`800000?`2o_kl@
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0G
0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool08`3oool003D0oooo00<000000?oo
o`3oool05@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0:0?oood00_kno0`3oool3000003h0_kno403oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`0L0?ooo`030000003oool0oooo02<0oooo000e0?ooo`030000003oool0oooo01D0
oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool02P3ooolo0;no_`L0oooo?P2o_kl@0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01`0
oooo00<000000?ooo`3oool08`3oool003D0oooo00<000000?ooo`3oool05@3oool00`000000oooo
0?ooo`0B0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`0:0?ooocl0_kno203ooolm0;no_a00oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0D0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool07@3oool00`000000oooo
0?ooo`0R0?ooo`00=@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00X0oooo?P2o
_kl90?ooocd0_kno403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0280oooo000e
0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool02`3ooolm0;no_`X0oooo?02o_kl@
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool08P3oool003D0oooo00<000000?oo
o`3oool05@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0;0?oooc`0_kno2`3oooll0;no_a00oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`0R0?ooo`00=@3oool00`000000oooo0?ooo`0E0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0>0?ooo`03
0000003oool0oooo00/0oooo?02o_kl<0?oooc/0_kno403oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo0280oooo000e0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
2`3ooolk0;no_`d0oooo>`2o_kl@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0
8`3oool003D0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0;0?oooc/0_kno3P3o
oolj0;no_a00oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0C0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0S0?ooo`00=@3oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00/0oooo>P2o_kl?0?ooocX0_kno403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo02<0oooo000e0?ooo`030000003oool0oooo
01D0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool02`3ooolj0;no_a00oooo>@2o_kl@0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
01`0oooo00<000000?ooo`3oool08`3oool003D0oooo00<000000?ooo`3oool05@3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`0;0?ooocT0_kno4@3oooli0;no_a00oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool07@3oool00`000000
oooo0?ooo`0S0?ooo`00=@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00/0oooo
>@2o_klB0?ooocP0_kno403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02<0oooo
000e0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02`3ooolh0;no_``0oooo0P00
00060?ooocL0_kno403oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo01@0oooo00<0
00000?ooo`3oool05`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02<0oooo000e
0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02`3ooolh0;no_``0oooo1@000003
0?ooocL0_kno3`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02<0oooo000e0?oo
o`030000003oool0oooo01H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool02`3ooolg0;no_`d0oooo100000050?oo
ocH0_kno3`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01<0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02<0oooo000e0?ooo`03
0000003oool0oooo01H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool02`3ooolg0;no_`d0oooo100000050?ooocH0
_kno3`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02<0oooo000e0?ooo`030000
003oool0oooo01H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool02`3ooolf0;no_`h0oooo0`0000070?ooocD0_kno
3`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo02<0oooo000A0?ooo`800000603o
ool2000000P0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0:0?ooocH0_kno3P3o
ool3000000L0oooo=@2o_kl?0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04`3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01D0oooo0P0000060?ooo`030000003o
ool0oooo00l0oooo0P00000B0?ooo`003`3oool4000000<0oooo1000000>0?ooo`D000000`3oool6
000000d0oooo0P0000080?ooo`@000004P3oool00`000000oooo0?ooo`030?ooo`<000002P3oool0
0`000000oooo0?ooo`0;0?ooo`@00000303ooole0;no_`l0oooo0P0000090?oooc<0_kno00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`020?ooo`@000002P3oool00`000000oooo0?ooo`0:
0?ooo`<000001P3oool010000000oooo0?ooo`00000>0?ooo`<000001P3oool300000100oooo1000
00000`3oool0000000000003000000<0oooo00<000000?ooo`3oool02`3oool:000000h0oooo000<
0?ooo`X000003`3oool;000000H0oooo00<000000?ooo`3oool0203oool400000080oooo1P000000
0`3oool000000?ooo`0;0?ooo`8000001P3oool6000000L0oooo0P0000050?ooo`030000003oool0
oooo0080oooo200000030?ooo`030000003oool0oooo00/0oooo=02o_kl?0?ooo`8000002@3ooolc
0;no_`030000003oool0oooo00d0oooo00<000000?ooo`3oool01P3oool6000000@0oooo00@00000
0?ooo`0000000000303oool5000000040?ooo`000000oooo0?ooo`<000003`3oool;00000180oooo
2P00000@0?ooo`X000002`3oool000X0oooo2P00000?0?ooo`X000002@3oool00`000000oooo0?oo
o`050?ooo`T000001`3oool00`000000oooo0?ooo`070?ooo`/0000000<0oooo0000003oool0203o
ool4000000<0oooo1`00000;0?ooo`030000003oool0oooo00/0oooo<`2o_kl@0?ooo`030000003o
ool0oooo00T0oooo<P2o_kl00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00d0oooo2`00
000>0?ooo`T000004P3oool800000140oooo2P00000?0?ooo`L000003P3oool000l0oooo1@00000D
0?ooo`D000002@3oool00`000000oooo0?ooo`030?ooo`X000002@3oool00`000000oooo0?ooo`03
0?ooo`P000001`3oool00`000000oooo0?ooo`040?ooo`X000000P3oool00`000000oooo0?ooo`0=
0?ooo`030000003oool0oooo00/0oooo<`2o_kl@0?ooo`030000003oool0oooo00T0oooo<P2o_kl0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool02000000<
0?ooo`030000003oool0000000L00000403oool:000000l0oooo1@0000030?ooo`030000003oool0
oooo00h0oooo0`00000B0?ooo`00=P3oool00`000000oooo0?ooo`080?ooo`D000002@3oool01000
0000oooo0?ooo`3oool:000000L0oooo00<000000?ooo`3oool00P3oool:000000@0oooo00<00000
0?ooo`3oool03@3oool00`000000oooo0?ooo`0;0?oooc80_kno7P3ooola0;no_`030000003oool0
oooo00`0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0:000000T0oooo00<00000
0?ooo`3oool02P00000>0?ooo`D000006`3oool00`000000oooo0?ooo`0T0?ooo`00=P3oool00`00
0000oooo0?ooo`0F0?ooo`030000003oool0oooo00H0oooo1@0000070?ooo`030000003oool0oooo
00L0oooo1@0000040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02`3oool00`00
0000_kno0;no_`0_0;no_ah0oooo<@2o_kl00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool01@00000>0?ooo`030000003oool0oooo01P0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`0T0?ooo`00=P3oool00`000000oooo0?ooo`0F0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00/0oooo00<000000;no_`2o_kl0;P2o_kl<0?ooo`T000002`3oool`0;no_`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0D0?ooo`03
0000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0T0?ooo`00
=P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00X0oooo00<000000;no_`2o_kl0
;@2o_kl:0?ooo`<000002@3oool2000000T0oooo;`2o_kl00`3oool000000?ooo`0=0?ooo`030000
003oool0oooo00l0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0H0?ooo`030000
003oool0oooo01d0oooo00<000000?ooo`3oool0903oool003H0oooo00<000000?ooo`3oool05P3o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0:0?ooo`030000002o_kl0_kno02d0_kno2@3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo00L0oooo;P2o_kl00`3oool000000?ooo`0<0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo01h0oooo00<000000?ooo`3oool0903oool003L0oooo00<000000?ooo`3oool05@3oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`0:0?ooo`030000002o_kl0_kno02`0_kno203oool200000100oooo0P000008
0?ooobd0_kno00<0oooo0000003oool0303oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0N0?ooo`030000003oool0
oooo02@0oooo000g0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool04`3oool00`00
0000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0;no_`0/0;no_`H0oooo0P00000D0?ooo`030000003oool0oooo00D0oooo;@2o_kl00`3o
ool000000?ooo`0<0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool0903oool003L0
oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`090?ooo`030000003oool0_kno02/0
_kno1@3oool2000001L0oooo0P0000060?ooob`0_kno00<0oooo0000003oool0303oool00`000000
oooo0?ooo`0?0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo02@0oooo000g0?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool02@3oool00`000000oooo0;no_`0[0;no_`@0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`030?ooob`0_kno00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`0M0?ooo`030000003oool0oooo02D0oooo000g0?ooo`030000003oool0oooo01H0oooo00<0
00000?ooo`3oool04`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool02P3oool00`000000_kno0;no_`0Y0;no_`@0oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`030?ooob/0_kno00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?oo
o`0M0?ooo`030000003oool0oooo02D0oooo000g0?ooo`030000003oool0oooo01H0oooo00<00000
0?ooo`3oool04`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool02P3oool00`000000_kno0;no_`0X0;no_`@0oooo00<000000?ooo`3oool07@3oool0
0`000000oooo0?ooo`020?ooobX0_kno00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0N
0?ooo`030000003oool0oooo02D0oooo000h0?ooo`030000003oool0oooo01H0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool02P3oool00`000000_kno0;no_`0W0;no_`<0oooo0P00000P0?ooo`030000003oool0oooo
0080oooo:@2o_kl?0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool09@3oool003P0
oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0;0?ooobP0_kno0`3oool00`000000
oooo0?ooo`0P0?ooo`030000003oool0oooo00<0oooo:02o_kl>0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01h0
oooo00<000000?ooo`3oool09@3oool003P0oooo00<000000?ooo`3oool05P3oool00`000000oooo
0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`0;0?ooobP0_kno0`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0080oooo
:02o_kl>0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04`3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool09@3oool003P0oooo00<0
00000?ooo`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`0:0?ooobL0_kno103oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo00<0oooo9P2o_kl>0?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`3oool0503oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01h0oooo00<0
00000?ooo`3oool09P3oool003P0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0C
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0;
0?ooobH0_kno0`3oool2000002@0oooo00<000000?ooo`3oool00P3ooolV0;no_`h0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool07P3oool00`000000oooo0?ooo`0V0?ooo`00>03oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo01<0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo00/0oooo9@2o_kl30?ooo`030000003oool0oooo02D0oooo00<000000?oo
o`3oool00P3ooolU0;no_`h0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0D0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0V0?oo
o`00>@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3o
ool04@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00X0oooo902o_kl30?ooo`80
0000:03oool010000000oooo0?ooo`3ooolU0;no_`h0oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool07P3oool00`00
0000oooo0?ooo`0V0?ooo`00>@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00X0
oooo902o_kl20?ooo`030000003oool0oooo02T0oooo00@000000?ooo`3oool0oooo8`2o_kl>0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0I0?oo
o`030000003oool0oooo01h0oooo00<000000?ooo`3oool09`3oool003T0oooo00<000000?ooo`3o
ool05`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`0:0?ooob80_kno0P3oool00`000000oooo0?ooo`0[0?ooo`80
00000P3ooolR0;no_`h0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0W0?ooo`00
>@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00X0oooo8@2o_kl20?ooo`030000
003oool0oooo02h0oooo00<000000?ooo`3oool08@2o_kl=0?ooo`030000003oool0oooo0100oooo
00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01h0oooo
00<000000?ooo`3oool09`3oool003X0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`090?ooob40_kno00<0oooo0000003oool0<@3oool00`000000oooo0;no_`0P0;no_`d0oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`3oool07P3oool00`000000oooo0?ooo`0W0?ooo`00>P3oool00`000000oooo0?ooo`0F
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo00T0oooo802o_kl00`3oool000000?ooo`0c0?ooo`030000003oool0
_kno01h0_kno3@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo02P0oooo000j0?oo
o`030000003oool0oooo01L0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo00d0oooo0P00000;0?oooah0_kno00<0oooo0000003oool0=@3oool00`00
0000oooo0;no_`0M0;no_`d0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0D0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo0180oooo000B0?ooo`8000009P3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0>0?ooo`03
0000003oool0oooo00T0oooo7P2o_kl00`000000oooo0?ooo`0f0?ooo`030000002o_kl0_kno01`0
_kno303oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0L0?ooo`040000003oool0oooo000001D0oooo0`00000B0?oo
o`00403oool400000080oooo1000000B0?ooo`030000003oool0oooo00@0oooo0P0000060?ooo`03
0000003oool0oooo0140oooo0P0000040?ooo`030000003oool0oooo0180oooo0P00000C0?ooo`03
0000003oool0oooo00@0oooo0P0000070?ooo`030000003oool0oooo00T0oooo00<000000;no_`2o
_kl06P2o_kl00`000000oooo0?ooo`020?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool0603ooolL0;no_`D00000203oool00`000000oooo0?ooo`0:0?ooo`800000103oool00`000000
oooo0?ooo`0C0?ooo`8000006@3oool3000000D0oooo00<000000?ooo`3oool04@3oool300000080
oooo1000000B0?ooo`P00000403oool000d0oooo2@00000D0?ooo`<000000P3oool4000000P0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`030?ooo`@000001P3oool00`000000oooo
0?ooo`0@0?ooo`8000000P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0080oooo
0P0000090?ooo`030000003oool0oooo00T0oooo00<000000;no_`2o_kl06@2o_kl70?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool05`3oool00`000000oooo0;no_`0J0;no_`D0oooo
300000090?ooo`@0000000<0oooo0000003oool05@3oool010000000oooo00000000000I0?ooo`<0
00000`3oool3000001@0oooo1`00000C0?ooo`P000003P3oool000/0oooo2P00000C0?ooo`L00000
303oool00`000000oooo0?ooo`050?ooo`<000000P3oool3000000X0oooo00<000000?ooo`3oool0
3@3oool3000000@0oooo00<000000?ooo`3oool04@3oool010000000oooo00000000000<0?ooo`03
0000003oool0oooo00T0oooo00<000000;no_`2o_kl05`2o_kl90?ooo`030000003oool0oooo01L0
oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0J0;no_`030000003oool0oooo00X0
oooo00<000000?ooo`3oool00P3oool@000001D0oooo00@000000?ooo`3oool0oooo0`0000040?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool01`00000D0?ooo`P000004@3oool:0000
00`0oooo000@0?ooo`D000004@3oool8000000d0oooo00<000000?ooo`3oool00`3oool7000000d0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`020?ooo`<000001`3oool00`000000
oooo0?ooo`0A0?ooo`8000003P3oool00`000000oooo0?ooo`090?ooo`030000002o_kl0_kno01H0
_kno2P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01L0oooo00@000000?ooo`3o
ool0oooo6@2o_kl<0?ooo`030000003oool0oooo00/0oooo2P00000C0?ooo`030000003oool0oooo
00@0oooo0P0000000`3oool0000000000002000000h0oooo00<000000?ooo`3oool00`3oool60000
01<0oooo2P00000U0?ooo`00903oool:000000h0oooo00<000000?ooo`3oool02000000?0?ooo`03
0000003oool0oooo0080oooo0`0000020?ooo`8000002`3oool00`000000oooo0?ooo`0>0?ooo`80
000000<0oooo0000003oool03P3oool00`000000oooo0?ooo`090?ooo`030000002o_kl0_kno01@0
_kno303oool00`000000oooo0?ooo`0C0?ooo`D000005P3oool00`000000oooo0?ooo`030?oooaL0
_kno00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`H000000`3oool200000140
oooo0P0000080?ooo`H000003@3oool00`000000oooo0?ooo`030?ooo`P00000503oool00`000000
oooo0?ooo`0Y0?ooo`00?03oool00`000000oooo00000008000000l0oooo00@000000?ooo`3oool0
oooo1P00000=0?ooo`030000003oool0oooo00`0oooo0P0000030?ooo`030000003oool0oooo00d0
oooo00<000000?ooo`3oool02P3ooolE0;no_`h0oooo00<000000?ooo`3oool04P3oool5000001D0
oooo00<000000?ooo`3oool01@3ooolF0;no_``0oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`040?ooo`8000001@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo1`00000;0?ooo`030000003oool0oooo00P0oooo1@00000B0?ooo`030000003oool0oooo02X0
oooo000l0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`0000001@00000?0?ooo`030000
003oool0oooo00T0oooo0P0000060?ooo`030000003oool0oooo00d0oooo0P00000:0?oooa@0_kno
403oool00`000000oooo0?ooo`0A0?ooo`@000004`3oool00`000000oooo000000090?oooa@0_kno
3@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00H0oooo0P0000020?ooo`<00000
203oool00`000000oooo0?ooo`070?ooo`T000002@3oool00`000000oooo0?ooo`0O0?ooo`030000
003oool0oooo02X0oooo000l0?ooo`030000003oool0oooo01P0oooo2000000?0?ooo`050000003o
ool0oooo0?ooo`000000103oool3000000P0oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`090?oooa80_kno4P3oool00`000000oooo0000000@0?ooo`@000004P3oool4000000X0oooo
4P2o_kl=0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool02@3oool6000000H0oooo
00<000000?ooo`3oool03@3oool5000000L0oooo00<000000?ooo`3oool0803oool00`000000oooo
0?ooo`0Z0?ooo`00?@3oool00`000000oooo0?ooo`0F0?ooo`D00000503oool010000000oooo0000
000000020?ooo`800000303oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00T0oooo
402o_klC0?ooo`<00000403oool3000001<0oooo1@00000:0?oooa00_kno3@3oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo00X0oooo1P0000040?ooo`8000006`3oool00`000000oooo
0?ooo`0P0?ooo`030000003oool0oooo02X0oooo000m0?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool05@3oool6000000h0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0:0?ooo`h0_kno503oool400000100oooo0P00000B0?ooo`D00000303oool>0;no_`d0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0;0?ooo`L000000`3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0:`3oool003d0oooo0P00000I
0?ooo`030000003oool0oooo01<0oooo1P00000@0?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool02P3oool;0;no_aD0oooo1@00000@0?ooo`8000004P3oool400000100oooo2P2o_kl>
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool03P3oool600000080oooo00<00000
0?ooo`3oool06P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo02/0oooo000n0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool04@3oool800000100oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`090?ooo`T0_kno603oool5000000l0oooo00<000000?oo
o`3oool0403oool400000180oooo00<000000;no_`2o_kl01@2o_kl=0?ooo`8000004P3oool00`00
0000oooo0?ooo`0C0?ooo`@00000703oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo
02`0oooo000n0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0403oool6000001@0
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0[0?ooo`<000003`3oool00`000000
oooo0?ooo`0@0?ooo`800000:@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01H0
oooo00<000000?ooo`3oool06@3oool200000240oooo00<000000?ooo`3oool0;03oool003h0oooo
00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0?0?ooo`<00000103oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0:`3oool300000200oooo
0P00000X0?ooo`800000503oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0/0?ooo`00?`3oool00`000000oooo0?oo
o`0H0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`0?0?ooo`800000;03oool200000200oooo00<000000?ooo`3oool09`3oool00`000000oooo0?oo
o`0B0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?oo
o`0P0?ooo`030000003oool0oooo02`0oooo000o0?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0:`3oool00`000000oooo0?ooo`150?ooo`800000503oool00`000000oooo0?ooo`0F
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0]
0?ooo`00@03oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01D0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0740oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`3oool0803oool00`000000oooo0?ooo`0]0?ooo`00@03oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo01H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0A0?ooo`80
0000:`3ooolD00000340oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0F0?ooo`03
0000003oool0oooo01/0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0^0?ooo`00
@03oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0
4`3oool2000001<0oooo0P00000R0?ooo`L00000503oool8000002L0oooo0P00000E0?ooo`030000
003oool0oooo01L0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0P0?ooo`030000
003oool0oooo02h0oooo00100?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool05P3o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01<0oooo0P00000J0?ooo`H000008`3o
ool6000001h0oooo0`00000F0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo02l0oooo00110?ooo`030000003oool0
oooo01T0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0C0?ooo`8000005@3oool4
000000h0oooo2000000_0?ooo`H000005P3oool2000001T0oooo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0;`3oool0
0480oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool05`3oool>000003d0oooo2@0000070?ooo`H000006P3oool00`00
0000oooo0?ooo`0G0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`0_0?ooo`00@P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01H0
oooo00<000000?ooo`3oool0503oool2000006/0oooo1`00000O0?ooo`030000003oool0oooo01L0
oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo02l0
oooo00120?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool05P3oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo08d0oooo0P00000I0?ooo`030000003oool0oooo01`0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0`0?ooo`00@`3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?oo
o`2;0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0703oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo0300oooo00130?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo08T0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0240oooo00<00000
0?ooo`3oool0<@3oool004<0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo01D0oooo0P0000280?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0340oooo000D0?ooo`03
0000003oool0oooo02`0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo01L0oooo00<000000?ooo`3oool0Q03oool00`000000oooo0?ooo`0I0?ooo`03
0000003oool0oooo01d0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo0180oooo000B0?ooo`<000000`3oool2000001<0oooo00<000000?ooo`3oool0
0`3oool2000000l0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0>0?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo00/0oooo0P00000;0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
1`3oool200000100oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0G0?ooo`040000
003oool0000000000080oooo00<000000?ooo`3oool0503oool2000000040?ooo`00000000000000
0180oooo000A0?ooo`L00000503oool2000000<0oooo0P00000A0?ooo`030000003oool0oooo00L0
oooo0P00000A0?ooo`030000003oool0oooo00D0oooo0P00000A0?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo01P0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0
oooo00<000000?ooo`3oool0103oool2000001<0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`0>0?ooo`800000303oool00`000000oooo0?ooo`0:0?ooo`800000103oool00`000000
oooo0?ooo`0?0?ooo`030000003oool0oooo0080oooo1@00000F0?ooo`D000004@3oool000l0oooo
1`00000E0?ooo`H00000503oool00`000000oooo000000040?ooo`800000503oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`0@0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0303oool2000000040?oo
o`000000000000000100oooo00<000000?ooo`3oool00`3oool5000001D0oooo1P00000@0?ooo`00
3@3oool9000001<0oooo1P00000F0?ooo`<000000P3oool2000001L0oooo00<000000?ooo`3oool0
0P00000F0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool04P3oool2000000H0oooo
00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo
00<000000?ooo`3oool06P3oool010000000oooo0?ooo`3oool2000001L0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`0?0?ooo`D000003P3oool00`000000oooo0?ooo`030?ooo`L0
0000503oool8000000h0oooo000<0?ooo`X000004P3oool8000001D0oooo1@00000J0?ooo`800000
6@3oool01@000000oooo0?ooo`3oool0000001L0oooo0P0000030?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01X0
oooo00@000000?ooo`00000000006P3oool01@000000oooo0?ooo`3oool0000001L0oooo0P000000
103oool000000000000000030?ooo`030000003oool0oooo00l0oooo1P00000=0?ooo`030000003o
ool0oooo00<0oooo2@00000F0?ooo`D000003@3oool002L0oooo1P00000F0?ooo`H000005`3oool0
0`000000oooo0?ooo`02000001/0oooo00<000000?ooo`0000006P3oool2000000030?ooo`000000
oooo01/0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
oooo01X0oooo0P00000M0?ooo`030000003oool0000001X0oooo100000020?ooo`030000003oool0
oooo0100oooo1`00000;0?ooo`030000003oool0oooo00T0oooo1@00000W0?ooo`009P3oool30000
01T0oooo2000000E0?ooo`@000000P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
01/0oooo1000000J0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0H0?ooo`80000000<0oooo0000003oool07@3oool00`000000oooo0?ooo`0I0?oo
o`D0000000<0oooo0000003oool04`3oool6000000X0oooo00<000000?ooo`3oool0=@3oool00440
oooo1`00000F0?ooo`D000000P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0000001`0
oooo00@000000?ooo`3oool0oooo0P00000H0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0E0?ooo`<00000103oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0000001T0oooo1`00000I0?ooo`<00000203oool00`000000oooo0?ooo`0f0?oo
o`00@03oool3000000D0oooo00<000000?ooo`3oool04P3oool700000080oooo00<000000?ooo`3o
ool05P3oool01@000000oooo0?ooo`3oool0000001/0oooo00<000000?ooo`3oool00`3oool40000
01@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
01<0oooo0P0000070?ooo`030000003oool0oooo01T0oooo00D000000?ooo`3oool0oooo00000002
0?ooo`030000003oool0oooo01D0oooo1P00000S0?ooo`030000003oool0oooo03H0oooo00180?oo
o`030000003oool0oooo0140oooo1P0000050?ooo`030000003oool0oooo01@0oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00P0oooo0`00000A0?oo
o`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0?0?oo
o`@000002P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00<0oooo1000000H0?oo
o`@000008P3oool00`000000oooo0?ooo`0g0?ooo`00B@3oool00`000000oooo0?ooo`0?0?ooo`@0
00002@3oool00`000000oooo0?ooo`0>0?ooo`050000003oool0oooo0?ooo`0000001`3oool00`00
0000oooo0?ooo`0F0?ooo`030000003oool0oooo00/0oooo1P00000;0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0;0?ooo`@000003P3oool00`00
0000oooo0?ooo`0F0?ooo`030000003oool0oooo00@0oooo1000000H0?ooo`030000003oool0oooo
008000008@3oool00`000000oooo0?ooo`0g0?ooo`00B@3oool00`000000oooo0?ooo`0>0?ooo`80
0000303oool00`000000oooo0?ooo`0=0?ooo`<0000000<0oooo0000003oool0203oool2000001H0
oooo00<000000?ooo`3oool04@3oool:000000030?ooo`000000oooo01L0oooo1@00000I0?ooo`03
0000003oool0oooo00D0oooo1P00000C0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool0103oool6000001H0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0g0?ooo`00
BP3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo00`0oooo1000000;0?ooo`030000
003oool0oooo01<0oooo00<000000?ooo`3oool0703ooolC000000L0oooo1@00000F0?ooo`/00000
6@3oool00`000000oooo0?ooo`0B0?ooo`800000203oool5000001D0oooo00<000000?ooo`3oool0
903oool00`000000oooo0?ooo`0h0?ooo`00B`3oool00`000000oooo0?ooo`0L0?ooo`030000003o
ool0oooo00X0oooo1P00000;0?ooo`8000004`3oool00`000000oooo0?ooo`0N0?ooo`030000003o
ool0oooo00h0oooo8P0000020?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0403o
ool00`000000oooo0?ooo`0:0?ooo`@000004`3oool00`000000oooo0?ooo`0T0?ooo`030000003o
ool0oooo03T0oooo001;0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool02@3oool5
000000h0oooo0P00000@0?ooo`030000003oool0oooo01d0oooo1@00000F0?ooo`<000006@3oool0
0`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0303oool2
000001<0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0i0?ooo`00B`3oool00`00
0000oooo0?ooo`0N0?ooo`030000003oool0oooo00L0oooo1000000A0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool07@3oool5000001H0oooo0`00000G0?ooo`H00000803oool00`00
0000oooo0?ooo`0=0?ooo`8000004@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo
02@0oooo00<000000?ooo`3oool0>P3oool004`0oooo00<000000?ooo`3oool07@3oool00`000000
oooo0?ooo`070?ooo`<000004`3oool2000000h0oooo00<000000?ooo`3oool07@3oool5000001H0
oooo0`00000G0?ooo`H00000803oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo
0280oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0j0?ooo`00C@3oool00`000000
oooo0?ooo`0M0?ooo`8000001P3oool3000001H0oooo0P00000;0?ooo`030000003oool0oooo01l0
oooo0`00000G0?ooo`800000603oool500000280oooo0`00000:0?ooo`030000003oool0oooo0280
oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0k0?ooo`00CP3oool00`000000oooo
0?ooo`0M0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool05`3oool2000000X0oooo
00<000000?ooo`3oool07`3oool3000001P0oooo00<000000?ooo`3oool05`3oool400000200oooo
1@0000080?ooo`800000903oool2000002L0oooo00<000000?ooo`3oool0>`3oool004h0oooo00<0
00000?ooo`3oool07P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo00@0oooo00@0
00000?ooo`3oool000008@3oool3000001P0oooo00<000000?ooo`3oool05`3oool400000240oooo
1@0000050?ooo`8000009@3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo03`0oooo
001?0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool07`3oool2000000@0oooo0`00
000R0?ooo`<00000<P3oool3000002<0oooo100000040?ooo`030000003oool0oooo02@0oooo0P00
000W0?ooo`030000003oool0oooo03d0oooo001?0?ooo`030000003oool0oooo01l0oooo00<00000
0?ooo`3oool0803oool010000000oooo0?ooo`3oool500000240oooo00<000000?ooo`3oool0<@3o
ool3000002@0oooo0`0000030?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool09P3o
ool00`000000oooo0?ooo`0n0?ooo`00D03oool00`000000oooo0?ooo`0O0?ooo`030000003oool0
oooo0200oooo0P0000000`3oool000000000000200000280oooo00<000000?ooo`3oool0<P3oool2
000002@0oooo0`0000000`3oool000000000000V0?ooo`800000:03oool00`000000oooo0?ooo`0n
0?ooo`00D@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0280oooo1@00000R0?oo
o`030000003oool0oooo0380oooo00<000000?ooo`3oool0903oool3000002L0oooo00<000000?oo
o`3oool09`3oool00`000000oooo0?ooo`0o0?ooo`00D@3oool00`000000oooo0?ooo`0O0?ooo`80
00008P3oool4000005P0oooo00<000000?ooo`3oool08`3oool4000002H0oooo0P00000Y0?ooo`03
0000003oool0oooo03l0oooo001B0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0
803oool3000000030?ooo`000000000007X0oooo0P0000030?ooo`030000003oool0oooo02<0oooo
00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`100?ooo`00D`3oool00`000000oooo0?oo
o`0O0?ooo`800000803oool2000000@0oooo1000001d0?ooo`800000:`3oool00`000000oooo0?oo
o`0W0?ooo`030000003oool0oooo0440oooo001C0?ooo`030000003oool0oooo0200oooo00<00000
0?ooo`3oool07P3oool2000000P0oooo0P00001^0?ooo`@00000;03oool00`000000oooo0?ooo`0W
0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool04P3oool001D0oooo00<000000?oo
o`3oool00P00000J0?ooo`030000003oool0oooo01T0oooo00D000000?ooo`3oool0oooo0000000F
0?ooo`030000003oool0oooo00T0oooo0P00000>0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool0203oool3000001P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H
0?ooo`030000003oool0oooo01P0oooo0`00000I0?ooo`030000003oool0oooo0180oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00L0oooo00<00000
0?ooo`3oool03`3oool010000000oooo0?ooo`00000G0?ooo`030000003oool00000008000004`3o
ool001<0oooo1@00000G0?ooo`030000003oool0oooo008000006P3oool2000000D0oooo00<00000
0?ooo`3oool04P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`04000001@0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01D0oooo00@000000?ooo`0000000000703o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0603o
ool200000080oooo00D000000?ooo`3oool0oooo0000000C0?ooo`@000005`3oool500000180oooo
000B0?ooo`@00000603oool4000001L0oooo00D000000?ooo`3oool0oooo000000080?ooo`030000
003oool0oooo0100oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool01@3oool300000140oooo00<000000?ooo`3oool0
6@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01<0oooo100000020?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo01T0oooo100000000`3oool000000?ooo`0D0?ooo`@000005P3oool600000140oooo
000@0?ooo`L000005P3oool4000001L0oooo0P0000000`3oool000000000000:0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool0203oool2000000l0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00l0oooo100000060?ooo`030000003o
ool0oooo01X0oooo00<000000?ooo`3oool0303oool2000000/0oooo00@000000?ooo`3oool00000
603oool4000001D0oooo1`00000C0?ooo`P00000403oool000h0oooo2@00000D0?ooo`H000005P3o
ool4000000`0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0:0?ooo`H00000203o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0303o
ool2000000/0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo00/0oooo1000000F0?ooo`H00000503oool8000001D0oooo1P00000?0?ooo`003@3oool5
000001P0oooo2000000D0?ooo`D000003@3oool00`000000oooo0?ooo`080?ooo`040000003oool0
oooo000001H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo0140oooo100000040?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`060?ooo`H000003@3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo00T0oooo0P00000>0?ooo`@000005P3oool6000001L0oooo1P00000H0?ooo`<000003P3o
ool002T0oooo1P00000G0?ooo`H000003@3oool00`000000oooo0?ooo`060?ooo`@00000603oool2
00000080oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0E0?ooo`H000006`3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0080oooo1000000C0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`0000001`3oool00`000000oooo0?ooo`0=0?ooo`H000005`3oool5
000001X0oooo0`00000X0?ooo`00:03oool3000001X0oooo1@00000?0?ooo`030000003oool0oooo
00H0oooo1000000J0?ooo`030000003oool0000001`0oooo00<000000?ooo`3oool06P3oool00`00
0000oooo00000006000001@0oooo00<000000?ooo`3oool05`3oool7000001P0oooo00<000000?oo
o`3oool06`3oool2000000H0oooo00<000000?ooo`3oool03`3oool5000001H0oooo00<000000?oo
o`3oool0100000140?ooo`00A03oool400000180oooo0P0000050?ooo`H00000603oool00`000000
oooo0000000M0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool01P3oool:000000X0
oooo00<000000?ooo`3oool03@3oool:00000080oooo00<000000?ooo`3oool06P3oool00`000000
oooo0?ooo`0J0?ooo`@000000`3oool2000001@0oooo1000000D0?ooo`8000001@3oool2000004<0
oooo00130?ooo`8000005P3oool00`000000oooo0?ooo`030?ooo`D00000603oool6000001/0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0@0?oooaX00000303oool00`000000oooo
0?ooo`0J0?ooo`040000003oool0oooo000001P0oooo1@0000000`3oool000000000000G0?ooo`@0
00004P3oool00`000000oooo0?ooo`1:0?ooo`00G03oool010000000oooo0?ooo`3oool4000001X0
oooo1@0000000`3oool000000000000F0?ooo`030000003oool0000001`0oooo00<000000?ooo`3o
ool0603oool6000001X0oooo00<000000?ooo`3oool06P3oool3000001T0oooo1P00000J0?ooo`80
00004P3oool00`000000oooo0?ooo`1:0?ooo`00G03oool010000000oooo0?ooo`3oool2000001/0
oooo1@0000040?ooo`030000003oool0oooo01<0oooo0`00000K0?ooo`D000005`3oool6000001L0
oooo1P00000H0?ooo`D000006P3oool4000001d0oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`1;0?ooo`00G@3oool00`000000oooo0000000M0?ooo`@000001P3oool00`000000oooo
0?ooo`0B0?ooo`D000006@3oool4000001P0oooo1@00000H0?ooo`H000006@3oool4000001X0oooo
1@00000/0?ooo`030000003oool0oooo04`0oooo001N0?ooo`030000003oool0oooo01/0oooo1000
00080?ooo`8000004P3oool4000001X0oooo1000000I0?ooo`@000006@3oool5000001T0oooo1000
000H0?ooo`8000000`3oool2000002/0oooo00<000000?ooo`3oool0C@3oool005l0oooo00<00000
0?ooo`3oool06P3oool2000000`0oooo0P00000@0?ooo`@000006P3oool3000001X0oooo1000000I
0?ooo`D000006P3oool4000001L0oooo00<000000?ooo`3oool00`3oool2000002X0oooo00<00000
0?ooo`3oool0C@3oool00600oooo00<000000?ooo`3oool0603oool2000000l0oooo0P00000=0?oo
o`@000006`3oool3000001X0oooo0`00000K0?ooo`<00000703oool3000001D0oooo0P0000070?oo
o`030000003oool0oooo02H0oooo0P00001@0?ooo`00H@3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo00l0oooo00<000000?ooo`3oool02`3oool3000001`0oooo0P00000K0?ooo`<0
00006`3oool3000001d0oooo0P00000C0?ooo`800000<P3oool00`000000oooo0?ooo`1?0?ooo`00
HP3oool00`000000oooo0?ooo`0Y0?ooo`8000002`3oool2000001d0oooo0P00000L0?ooo`800000
703oool2000001d0oooo0P00000B0?ooo`030000003oool0oooo0300oooo0P00001B0?ooo`00H`3o
ool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo00P0oooo0P00000M0?ooo`030000003o
ool0oooo01/0oooo00<000000?ooo`3oool06`3oool2000001h0oooo00<000000?ooo`3oool03P3o
ool2000003<0oooo00<000000?ooo`3oool0D@3oool006@0oooo00<000000?ooo`3oool0:P3oool2
000000P0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0K0?ooo`030000003oool0
oooo01`0oooo00<000000?ooo`3oool0:`3oool2000003@0oooo00<000000?ooo`3oool0DP3oool0
06D0oooo00<000000?ooo`3oool0:`3oool2000008h0oooo0P00000e0?ooo`030000003oool0oooo
05<0oooo001V0?ooo`030000003oool0oooo02`0oooo0P00002:0?ooo`800000=P3oool00`000000
oooo0?ooo`1D0?ooo`00I`3oool00`000000oooo0?ooo`0]0?ooo`@00000Q03oool2000003L0oooo
00<000000?ooo`3oool0E@3oool006P0oooo00<000000?ooo`3oool0<03oool200000800oooo0P00
000h0?ooo`030000003oool0oooo05H0oooo001Y0?ooo`030000003oool0oooo0340oooo0P00001l
0?ooo`800000>@3oool00`000000oooo0?ooo`1G0?ooo`00JP3oool00`000000oooo0?ooo`0b0?oo
o`800000MP3oool4000003X0oooo00<000000?ooo`3oool0F03oool006/0oooo0P00000d0?ooo`@0
0000L03oool3000003/0oooo0P00001K0?ooo`00K@3oool00`000000oooo0?ooo`0e0?ooo`<00000
JP3oool3000003`0oooo0P0000170?ooo`030000003oool0oooo01<0oooo000F0?ooo`030000003o
ool0oooo05D0oooo00<000000?ooo`3oool0=`3oool2000006H0oooo0P00000o0?ooo`030000003o
ool0oooo02`0oooo00<000000?ooo`3oool05P3oool3000001@0oooo000E0?ooo`8000000P3oool0
0`000000oooo0?ooo`0D0?ooo`050000003oool0oooo0?ooo`0000006`3oool00`000000oooo0?oo
o`0H0?ooo`050000003oool0oooo0?ooo`0000005`3oool00`000000oooo0?ooo`0H0?ooo`030000
003oool0oooo00D0oooo1000000?0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0E0?ooo`@000006`3oool00`000000oooo0?ooo`0H0?ooo`030000
003oool0oooo00H0oooo0P00000@0?ooo`050000003oool0oooo0?ooo`0000005`3oool00`000000
oooo00000002000001H0oooo1@00000C0?ooo`00503oool5000001H0oooo1@00000G0?ooo`050000
003oool0oooo0?ooo`0000006P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`080?ooo`<000003@3oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool04`3oool300000080
oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0403oool00`000000oooo00000002000001L0oooo1000000F0?ooo`H0
00004P3oool00180oooo1P00000F0?ooo`@00000603oool5000001X0oooo00<000000?ooo`3oool0
1@3oool2000001<0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0<0?ooo`D00000
203oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
3P3oool5000000H0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool000000080oooo0P00000D0?ooo`@000005`3oool5000001D0oooo1P00000B0?ooo`004@3o
ool8000001@0oooo1P00000G0?ooo`@00000603oool00`000000oooo0000000:0?ooo`030000003o
ool0oooo00l0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0A0?ooo`D000000P3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool02`3o
ool3000000/0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0H0?ooo`<0000000<0
oooo0000003oool0503oool6000001D0oooo1`00000F0?ooo`D000004@3oool00100oooo1P00000F
0?ooo`D00000603oool6000001H0oooo0`00000<0?ooo`030000003oool0oooo00X0oooo00@00000
0?ooo`3oool000006`3oool00`000000oooo0?ooo`0G0?ooo`D000006P3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo00<0oooo1`00000>0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`0000005`3oool5000001L0oooo1@00000G0?ooo`H000005`3oool400000100oooo000?
0?ooo`<000006@3oool4000001T0oooo1P00000G0?ooo`@00000303oool2000000X0oooo0P000000
0`3oool000000?ooo`0J0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool01@00000E
0?ooo`030000003oool0oooo01T0oooo1P00000F0?ooo`030000003oool0000001T0oooo0`00000H
0?ooo`@000006@3oool4000001P0oooo1000000J0?ooo`8000003`3oool002X0oooo0P00000K0?oo
o`D00000603oool6000000d0oooo00<000000?ooo`3oool01P3oool5000001P0oooo0P0000000`3o
ool000000?ooo`0J0?ooo`030000003oool0oooo00D0oooo2@00000<0?ooo`030000003oool0oooo
0140oooo2@00000K0?ooo`<00000603oool5000001L0oooo1@00000I0?ooo`@000006P3oool20000
02X0oooo00170?ooo`<000006P3oool5000000h0oooo00<000000?ooo`3oool01P3oool6000001L0
oooo1@00000H0?ooo`D000003P3ooolQ000000D0oooo1@00000G0?ooo`D000006@3oool4000001H0
oooo00<000000?ooo`3oool00`00000K0?ooo`800000AP3oool004H0oooo0P00000K0?ooo`@00000
4@3oool3000000D0oooo1@00000H0?ooo`@000006@3oool5000001L0oooo1P00000G0?ooo`D00000
603oool4000001X0oooo0`00000D0?ooo`8000001@3oool2000001`0oooo00<000000?ooo`3oool0
@`3oool006<0oooo0`00000D0?ooo`030000003oool0oooo0080oooo1@00000I0?ooo`@000006@3o
ool4000001P0oooo1@00000I0?ooo`@00000603oool4000001/0oooo0`00000A0?ooo`800000203o
ool00`000000oooo0?ooo`1P0?ooo`00HP3oool3000001H0oooo0`0000000`3oool0000000000002
000001X0oooo0`00000J0?ooo`@000006@3oool4000001T0oooo1000000I0?ooo`@000006`3oool2
00000100oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`1O0?ooo`00HP3oool00`00
0000oooo0?ooo`0H0?ooo`040000003oool00000000001/0oooo0`00000K0?ooo`<000006P3oool4
000001T0oooo0`00000K0?ooo`<00000703oool2000000l0oooo00<000000?ooo`3oool0J`3oool0
07h0oooo0`00000K0?ooo`<000006`3oool3000001X0oooo0`00000K0?ooo`800000703oool20000
01d0oooo00<000000?ooo`3oool02P3oool3000006h0oooo001n0?ooo`040000003oool0oooo0000
01X0oooo0P00000L0?ooo`8000006`3oool3000001/0oooo0P00000L0?ooo`800000:@3oool00`00
0000oooo0?ooo`1_0?ooo`00PP3oool2000001P0oooo0P00000L0?ooo`800000703oool2000001`0
oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0U0?ooo`800000LP3oool008<0oooo
00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01/0oooo
00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo0780oooo
00240?ooo`<00000<`3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo05d0oooo0`00
001e0?ooo`00Q`3oool00`000000oooo0?ooo`2^0?ooo`030000003oool0oooo07D0oooo00280?oo
o`<00000ZP3oool3000007P0oooo002;0?ooo`030000003oool0oooo0:H0oooo00<000000?ooo`3o
ool0N@3oool008`0oooo0P00002S0?ooo`<00000O03oool008h0oooo0P00002P0?ooo`030000003o
ool0oooo07d0oooo002@0?ooo`800000V`3oool300000800oooo002B0?ooo`030000003oool0oooo
09L0oooo00<000000?ooo`3oool0P@3oool009<0oooo1000002B0?ooo`<00000Q03oool009H0oooo
0`00002>0?ooo`<00000QP3oool009T0oooo0P0000290?ooo`<00000L`3oool00`000000oooo0?oo
o`0C0?ooo`005`3oool00`000000oooo0?ooo`210?ooo`800000QP3oool00`000000oooo0?ooo`1I
0?ooo`030000003oool0oooo01L0oooo0`00000D0?ooo`005P3oool2000000030?ooo`000000oooo
01H0oooo00@000000?ooo`3oool000006`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
oooo01T0oooo00<000000?ooo`3oool04`3oool300000080oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0=0?ooo`@00000203oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
oooo01P0oooo00@000000?ooo`3oool00000603oool3000001P0oooo1000000D0?ooo`005@3oool4
000001L0oooo1000000H0?ooo`040000003oool0oooo000001X0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`0G0?ooo`<000006P3oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0;0?ooo`800000303o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0000001P0oooo0`00000H0?ooo`@000005`3o
ool5000001<0oooo000C0?ooo`H000005`3oool4000001L0oooo1000000H0?ooo`040000003oool0
oooo000001/0oooo00<000000?ooo`3oool0603oool00`000000oooo00000002000001L0oooo00<0
00000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`3oool01P3oool4000000l0oooo00<000000?ooo`000000603oool3000001P0oooo1000
000F0?ooo`D000005P3oool6000001<0oooo000B0?ooo`P000005@3oool6000001H0oooo1000000H
0?ooo`<000006@3oool00`000000oooo0000000K0?ooo`030000003oool0oooo0080oooo0`00000D
0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0I
0?ooo`030000003oool0oooo00@0oooo0P00000C0?ooo`<000005`3oool5000001H0oooo1@00000G
0?ooo`D000005P3oool600000180oooo000A0?ooo`H000005`3oool6000001H0oooo1P00000F0?oo
o`H000005`3oool3000001P0oooo0`0000080?ooo`@000003`3oool00`000000oooo0?ooo`0J0?oo
o`030000003oool0oooo01T0oooo0`00000H0?ooo`L000004`3oool5000001P0oooo1000000G0?oo
o`D00000603oool3000001P0oooo1@00000A0?ooo`00403oool3000001X0oooo1@00000H0?ooo`D0
00005`3oool5000001L0oooo1@00000G0?ooo`D000002@3oool4000000X0oooo1@00000G0?ooo`H0
00005P3oool6000001H0oooo1P00000G0?ooo`D00000603oool3000001T0oooo0`00000I0?ooo`<0
00006@3oool300000140oooo000]0?ooo`<000006@3oool4000001P0oooo1@00000H0?ooo`@00000
603oool4000000h0oooo100000060?ooo`D000005`3oool6000001H0oooo1P00000D0?ooo`P00000
603oool4000001T0oooo0`00000I0?ooo`<000006@3oool2000001/0oooo0P00000@0?ooo`00;03o
ool2000001/0oooo0P00000J0?ooo`@00000603oool4000001T0oooo1000000B0?ooo`@000000P3o
ool4000001P0oooo1@00000H0?ooo`D00000403oool4000000<0oooo1@00000I0?ooo`<000006@3o
ool3000001X0oooo0P00000K0?ooo`030000003oool0oooo02X0oooo00180?ooo`030000003oool0
oooo01X0oooo0P00000J0?ooo`<000006P3oool3000001L0oooo1P00000I0?ooo`@00000603oool5
000000X0oooo1P0000080?ooo`@000006@3oool3000001X0oooo0P00000L0?ooo`030000003oool0
oooo04D0oooo001T0?ooo`8000006`3oool2000001X0oooo0`00000J0?ooo`P000005@3oool40000
01T0oooo0`0000070?ooo`@000003`3oool3000001X0oooo0`00000J0?ooo`800000H`3oool006@0
oooo00<000000?ooo`3oool06@3oool2000001/0oooo0`00000K0?ooo`8000001@3oool7000000h0
oooo0`00000J0?ooo`X00000503oool2000001/0oooo0P00000K0?ooo`030000003oool0oooo0640
oooo00200?ooo`030000003oool0oooo01X0oooo0P00000L0?ooo`800000303oool<00000080oooo
0`00000A0?ooo``000006`3oool2000001`0oooo00<000000?ooo`3oool0O@3oool009d0oooo0P00
000L0?ooo`030000003oool0oooo01L0oooo5P00000:0?ooo`800000703oool00`000000oooo0?oo
o`2J0?ooo`00W@3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`2g0?ooo`00f03oool00`000000oooo0?ooo`3E0?oo
o`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?oo
o`00o`3ooona0?ooo`00o`3ooonJ0?ooo`030000003oool0oooo01@0oooo000H0?ooo`030000003o
ool0oooo01L0oooo00<000000?ooo`3oool0o`3ooom:0?ooo`030000003oool0oooo01L0oooo0`00
000E0?ooo`005`3oool3000001L0oooo1000000H0?ooo`040000003oool0oooo000001X0oooo00<0
00000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01P0oooo00@0
00000?ooo`3oool00000603oool2000001P0oooo1000000E0?ooo`005P3oool4000001L0oooo0`00
000H0?ooo`@00000603oool00`000000oooo0000000K0?ooo`030000003oool0oooo01T0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0I0?ooo`030000003oool0000001T0oooo00<0
00000?ooo`000000603oool3000001L0oooo1000000F0?ooo`H00000503oool001D0oooo1P00000E
0?ooo`D000005`3oool4000001P0oooo0`00000H0?ooo`040000003oool0oooo000001P0oooo00<0
00000?ooo`0000006`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool06@3oool3000001P0oooo0`00000I0?ooo`<000005P3oool6000001D0oooo1@00
000G0?ooo`D00000503oool001@0oooo1@00000F0?ooo`L000005@3oool6000001H0oooo1P00000F
0?ooo`<000006@3oool3000001T0oooo0P00000I0?ooo`H000005P3oool5000001L0oooo1@00000F
0?ooo`D000005`3oool5000001L0oooo1@00000F0?ooo`D00000603oool4000001<0oooo000C0?oo
o`@000005`3oool6000001L0oooo1@00000G0?ooo`D000005`3oool5000001L0oooo1@00000G0?oo
o`D000005P3oool6000001H0oooo1@00000G0?ooo`D000005`3oool4000001P0oooo1@00000G0?oo
o`@00000603oool3000001T0oooo1000000B0?ooo`004P3oool2000001X0oooo1000000H0?ooo`@0
0000603oool5000001P0oooo1000000H0?ooo`@00000603oool4000001L0oooo1@00000H0?ooo`@0
0000603oool4000001L0oooo1000000I0?ooo`@00000603oool4000001P0oooo0`00000J0?ooo`80
00004P3oool002d0oooo0P00000K0?ooo`<000006@3oool4000001P0oooo1000000I0?ooo`@00000
603oool4000001P0oooo1000000H0?ooo`@00000603oool4000001P0oooo1000000I0?ooo`<00000
6P3oool2000001T0oooo0P00000L0?ooo`030000003oool0oooo00l0oooo00190?ooo`<000006P3o
ool2000001X0oooo0`00000I0?ooo`@000006@3oool3000001T0oooo1000000H0?ooo`@000006@3o
ool3000001T0oooo0`00000J0?ooo`8000006`3oool2000001X0oooo00<000000?ooo`3oool0:`3o
ool004T0oooo00<000000?ooo`3oool06@3oool2000001/0oooo0`00000I0?ooo`<000006P3oool3
000001T0oooo0`00000J0?ooo`8000006P3oool3000001X0oooo0P00000K0?ooo`8000006`3oool0
0`000000oooo0?ooo`160?ooo`00I@3oool00`000000oooo0?ooo`0J0?ooo`8000006P3oool20000
01/0oooo0P00000J0?ooo`<000006P3oool2000001/0oooo0P00000J0?ooo`800000703oool00`00
0000oooo0?ooo`1R0?ooo`00PP3oool00`000000oooo0?ooo`0I0?ooo`8000006`3oool2000001/0
oooo0P00000J0?ooo`8000006`3oool2000001/0oooo00<000000?ooo`3oool0O`3oool009h0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`2K0?ooo`00^`3oool00`000000oooo0?oo
o`0J0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0^03oool00?l0oooo/@3oool0
0?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool0
0?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00001\
\>"],
  ImageRangeCache->{{{0, 431}, {443.25, 0}} -> {18.1202, 16.2537, 0.227128, \
0.227128}}]
}, Closed]],

Cell["\<\
The large spacing between equipotentials within the neck region shows that \
the electric field is especially small there because repulsion of like \
charges produces a relatively small surface charge density near the neck.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Interior shielding by Faraday pail (thick U)", "Section"],

Cell["\<\
The thick U-shaped electrode shown below is sometimes called the Faraday pail \
because it tends to expel electric fields from the interior of the pail even \
though it is a fairly open geometry.  This behavior is illustrated by placing \
a circular electrode over the mouth of the pail.  First, we construct and \
display the electrodes.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(poly1 = {{49, 60}, {49, 40}, {71, 40}, {71, 60}, {68, 60}, {68, 
            43}, {52, 43}, {52, 60}, {49, 60}};\)\), "\[IndentingNewLine]", 
    \(\(poly2 = 
        Table[4.  {Cos[t], Sin[t]} + {60, 70}, {t, 0, 
            2  \[Pi], \(2  \[Pi]\)\/100}];\)\), "\[IndentingNewLine]", 
    \(\(Show[Graphics[{Line[poly1], Line[poly2]}], 
        AspectRatio \[Rule] Automatic, PlotRange \[Rule] All];\)\)}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.54545 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.0974 0.04329 -1.69481 0.04329 [
[ 0 0 0 0 ]
[ 1 1.54545 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.02381 .9026 m
.02381 .0368 L
.97619 .0368 L
.97619 .9026 L
.84632 .9026 L
.84632 .16667 L
.15368 .16667 L
.15368 .9026 L
.02381 .9026 L
s
.67316 1.3355 m
.67282 1.34637 L
.67179 1.3572 L
.67009 1.36794 L
.66772 1.37856 L
.66469 1.38901 L
.661 1.39924 L
.65668 1.40923 L
.65174 1.41892 L
.6462 1.42828 L
.64009 1.43728 L
.63342 1.44587 L
.62623 1.45403 L
.61854 1.46173 L
.61038 1.46892 L
.60178 1.47559 L
.59278 1.4817 L
.58342 1.48724 L
.57373 1.49218 L
.56374 1.4965 L
.55351 1.50018 L
.54306 1.50322 L
.53245 1.50559 L
.5217 1.50729 L
.51087 1.50832 L
.5 1.50866 L
.48913 1.50832 L
.4783 1.50729 L
.46755 1.50559 L
.45694 1.50322 L
.44649 1.50018 L
.43626 1.4965 L
.42627 1.49218 L
.41658 1.48724 L
.40722 1.4817 L
.39822 1.47559 L
.38962 1.46892 L
.38146 1.46173 L
.37377 1.45403 L
.36658 1.44587 L
.35991 1.43728 L
.3538 1.42828 L
.34826 1.41892 L
.34332 1.40923 L
.339 1.39924 L
.33531 1.38901 L
.33228 1.37856 L
.32991 1.36794 L
.32821 1.3572 L
.32718 1.34637 L
Mistroke
.32684 1.3355 L
.32718 1.32463 L
.32821 1.3138 L
.32991 1.30305 L
.33228 1.29243 L
.33531 1.28199 L
.339 1.27175 L
.34332 1.26177 L
.34826 1.25208 L
.3538 1.24271 L
.35991 1.23372 L
.36658 1.22512 L
.37377 1.21696 L
.38146 1.20927 L
.38962 1.20208 L
.39822 1.19541 L
.40722 1.18929 L
.41658 1.18376 L
.42627 1.17882 L
.43626 1.1745 L
.44649 1.17081 L
.45694 1.16778 L
.46755 1.1654 L
.4783 1.1637 L
.48913 1.16268 L
.5 1.16234 L
.51087 1.16268 L
.5217 1.1637 L
.53245 1.1654 L
.54306 1.16778 L
.55351 1.17081 L
.56374 1.1745 L
.57373 1.17882 L
.58342 1.18376 L
.59278 1.18929 L
.60178 1.19541 L
.61038 1.20208 L
.61854 1.20927 L
.62623 1.21696 L
.63342 1.22512 L
.64009 1.23372 L
.6462 1.24271 L
.65174 1.25208 L
.65668 1.26177 L
.661 1.27175 L
.66469 1.28199 L
.66772 1.29243 L
.67009 1.30305 L
.67179 1.3138 L
.67282 1.32463 L
Mistroke
.67316 1.3355 L
Mfstroke
0 0 m
1 0 L
1 1.54545 L
0 1.54545 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{186.313, 287.875},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004G0001[b000`400?l00000o`00003oo`3ooolH
0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH
0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`001P3ooooo
000000/000001`3oool000H0oooo00<000000?ooo`3oool0o`3oool70?ooo`030000003oool0oooo
00D0oooo00060?ooo`030000003oool0oooo0?l0oooo1`3oool00`000000oooo0?ooo`050?ooo`00
1P3oool00`000000oooo0?ooo`3o0?ooo`L0oooo00<000000?ooo`3oool01@3oool000H0oooo00<0
00000?ooo`3oool0o`3oool70?ooo`030000003oool0oooo00D0oooo00060?ooo`030000003oool0
oooo0?l0oooo1`3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`3o0?oo
o`L0oooo00<000000?ooo`3oool01@3oool000H0oooo00<000000?ooo`3oool0o`3oool70?ooo`03
0000003oool0oooo00D0oooo00060?ooo`030000003oool0oooo0?l0oooo1`3oool00`000000oooo
0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`3o0?ooo`L0oooo00<000000?ooo`3oool01@3o
ool000H0oooo00<000000?ooo`3oool0o`3oool70?ooo`030000003oool0oooo00D0oooo00060?oo
o`030000003oool0oooo0?l0oooo1`3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000
oooo0?ooo`3o0?ooo`L0oooo00<000000?ooo`3oool01@3oool000H0oooo00<000000?ooo`3oool0
o`3oool70?ooo`030000003oool0oooo00D0oooo00060?ooo`030000003oool0oooo0?l0oooo1`3o
ool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`3o0?ooo`L0oooo00<00000
0?ooo`3oool01@3oool000H0oooo00<000000?ooo`3oool0o`3oool70?ooo`030000003oool0oooo
00D0oooo00060?ooo`030000003oool0oooo0?l0oooo1`3oool00`000000oooo0?ooo`050?ooo`00
1P3oool00`000000oooo0?ooo`3o0?ooo`L0oooo00<000000?ooo`3oool01@3oool000H0oooo00<0
00000?ooo`3oool0o`3oool70?ooo`030000003oool0oooo00D0oooo00060?ooo`030000003oool0
oooo0?l0oooo1`3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`3o0?oo
o`L0oooo00<000000?ooo`3oool01@3oool000H0oooo00<000000?ooo`3oool0o`3oool70?ooo`03
0000003oool0oooo00D0oooo00060?ooo`030000003oool0oooo0?l0oooo1`3oool00`000000oooo
0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`3o0?ooo`L0oooo00<000000?ooo`3oool01@3o
ool000H0oooo00<000000?ooo`3oool0o`3oool70?ooo`030000003oool0oooo00D0oooo00060?oo
o`030000003oool0oooo0?l0oooo1`3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000
oooo0?ooo`3o0?ooo`L0oooo00<000000?ooo`3oool01@3oool000H0oooo00<000000?ooo`3oool0
o`3oool70?ooo`030000003oool0oooo00D0oooo00060?ooo`030000003oool0oooo0?l0oooo1`3o
ool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`3o0?ooo`L0oooo00<00000
0?ooo`3oool01@3oool000H0oooo00<000000?ooo`3oool0o`3oool70?ooo`030000003oool0oooo
00D0oooo00060?ooo`030000003oool0oooo0?l0oooo1`3oool00`000000oooo0?ooo`050?ooo`00
1P3oool00`000000oooo0?ooo`3o0?ooo`L0oooo00<000000?ooo`3oool01@3oool000H0oooo00<0
00000?ooo`3oool0o`3oool70?ooo`030000003oool0oooo00D0oooo00060?ooo`030000003oool0
oooo0?l0oooo1`3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?oo
ol8000008`3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`00
1P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo
0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000
oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000
oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<00000
0?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?oo
o`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3o
ool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0
oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<0
00000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`05
0?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`00
1P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo
0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000
oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000
oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<00000
0?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?oo
o`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3o
ool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0
oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<0
00000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`05
0?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`00
1P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo
0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000
oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000
oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<00000
0?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?oo
o`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3o
ool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0
oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<0
00000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`05
0?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`00
1P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo
0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000
oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000
oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<00000
0?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?oo
o`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3o
ool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0
oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<0
00000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`05
0?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`00
1P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo
0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000
oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000
oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<00000
0?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?oo
o`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3o
ool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0
oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<0
00000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`05
0?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`00
1P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo
0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000
oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000
oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<00000
0?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?oo
o`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3o
ool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0
oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<0
00000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`05
0?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`00
1P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo
0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000
oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000
oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<00000
0?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?oo
o`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3o
ool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0
oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<0
00000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`05
0?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`00
1P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo
0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000
oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000
oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<00000
0?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?oo
o`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3o
ool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0
oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<0
00000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`05
0?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`00
1P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo
0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000
oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000
oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<00000
0?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?oo
o`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3o
ool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0
oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<0
00000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`05
0?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`00
1P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo
0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000
oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000
oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<00000
0?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?oo
o`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3o
ool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0
oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0
oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<0
00000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`05
0?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`00
1P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`050?ooo`001P3oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo0;h0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`050?ooo`001P3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0;h0oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`050?ooo`001P3ooolU00000<00oooo9@0000070?ooo`00o`3ooolH0?ooo`00
o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00
o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00
o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00
o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00
o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00
o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00
o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00
o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00
o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00
o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00
o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00
o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00
o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00
o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00o`3ooolH0?ooo`00
o`3ooolH0?ooo`00P@3ooolE00000840oooo001n0?ooo`<000005@3oool3000007h0oooo001k0?oo
o`<000006`3oool2000007`0oooo001h0?ooo`<00000803oool3000007T0oooo001f0?ooo`800000
9P3oool2000007L0oooo001d0?ooo`800000:P3oool2000007D0oooo001c0?ooo`030000003oool0
oooo02`0oooo0P00001c0?ooo`00L@3oool200000340oooo0P00001a0?ooo`00K`3oool2000003D0
oooo00<000000?ooo`3oool0KP3oool006h0oooo00<000000?ooo`3oool0=P3oool00`000000oooo
0?ooo`1]0?ooo`00K@3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo06`0oooo001/
0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0J`3oool006/0oooo00<000000?oo
o`3oool0?03oool2000006/0oooo001Z0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3o
ool0J03oool006T0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`1W0?ooo`00J03o
ool00`000000oooo0?ooo`130?ooo`030000003oool0oooo06H0oooo001W0?ooo`030000003oool0
oooo04D0oooo00<000000?ooo`3oool0I@3oool006H0oooo00<000000?ooo`3oool0A`3oool00`00
0000oooo0?ooo`1T0?ooo`00I@3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo06<0
oooo001T0?ooo`030000003oool0oooo04/0oooo00<000000?ooo`3oool0HP3oool006<0oooo00<0
00000?ooo`3oool0C03oool00`000000oooo0?ooo`1R0?ooo`00H`3oool00`000000oooo0?ooo`1=
0?ooo`030000003oool0oooo0640oooo001R0?ooo`030000003oool0oooo04h0oooo00<000000?oo
o`3oool0H@3oool00640oooo00<000000?ooo`3oool0D03oool00`000000oooo0?ooo`1P0?ooo`00
H@3oool00`000000oooo0?ooo`1@0?ooo`030000003oool0oooo0600oooo001P0?ooo`030000003o
ool0oooo0580oooo00<000000?ooo`3oool0G`3oool005l0oooo00<000000?ooo`3oool0D`3oool0
0`000000oooo0?ooo`1O0?ooo`00G`3oool00`000000oooo0?ooo`1D0?ooo`030000003oool0oooo
05h0oooo001O0?ooo`030000003oool0oooo05D0oooo00<000000?ooo`3oool0G@3oool005h0oooo
00<000000?ooo`3oool0EP3oool00`000000oooo0?ooo`1M0?ooo`00GP3oool00`000000oooo0?oo
o`1G0?ooo`030000003oool0oooo05`0oooo001N0?ooo`030000003oool0oooo05L0oooo00<00000
0?ooo`3oool0G03oool005d0oooo00<000000?ooo`3oool0F@3oool00`000000oooo0?ooo`1K0?oo
o`00G@3oool00`000000oooo0?ooo`1I0?ooo`030000003oool0oooo05/0oooo001M0?ooo`030000
003oool0oooo05T0oooo00<000000?ooo`3oool0F`3oool005`0oooo00<000000?ooo`3oool0FP3o
ool00`000000oooo0?ooo`1K0?ooo`00G03oool00`000000oooo0?ooo`1J0?ooo`030000003oool0
oooo05/0oooo001L0?ooo`030000003oool0oooo05X0oooo00<000000?ooo`3oool0F`3oool005/0
oooo00<000000?ooo`3oool0G03oool00`000000oooo0?ooo`1J0?ooo`00F`3oool00`000000oooo
0?ooo`1L0?ooo`030000003oool0oooo05X0oooo001K0?ooo`030000003oool0oooo05`0oooo00<0
00000?ooo`3oool0FP3oool005/0oooo00<000000?ooo`3oool0G@3oool00`000000oooo0?ooo`1I
0?ooo`00F`3oool00`000000oooo0?ooo`1M0?ooo`030000003oool0oooo05T0oooo001K0?ooo`03
0000003oool0oooo05d0oooo00<000000?ooo`3oool0F@3oool005/0oooo00<000000?ooo`3oool0
G@3oool00`000000oooo0?ooo`1I0?ooo`00F`3oool00`000000oooo0?ooo`1M0?ooo`030000003o
ool0oooo05T0oooo001K0?ooo`030000003oool0oooo05d0oooo00<000000?ooo`3oool0F@3oool0
05/0oooo00<000000?ooo`3oool0G@3oool00`000000oooo0?ooo`1I0?ooo`00F`3oool00`000000
oooo0?ooo`1M0?ooo`030000003oool0oooo05T0oooo001K0?ooo`030000003oool0oooo05d0oooo
00<000000?ooo`3oool0F@3oool005/0oooo00<000000?ooo`3oool0G@3oool00`000000oooo0?oo
o`1I0?ooo`00F`3oool00`000000oooo0?ooo`1M0?ooo`030000003oool0oooo05T0oooo001K0?oo
o`030000003oool0oooo05d0oooo00<000000?ooo`3oool0F@3oool005/0oooo00<000000?ooo`3o
ool0G@3oool00`000000oooo0?ooo`1I0?ooo`00F`3oool00`000000oooo0?ooo`1M0?ooo`030000
003oool0oooo05T0oooo001K0?ooo`030000003oool0oooo05d0oooo00<000000?ooo`3oool0F@3o
ool005/0oooo00<000000?ooo`3oool0G03oool00`000000oooo0?ooo`1J0?ooo`00F`3oool00`00
0000oooo0?ooo`1L0?ooo`030000003oool0oooo05X0oooo001K0?ooo`030000003oool0oooo05`0
oooo00<000000?ooo`3oool0FP3oool005`0oooo00<000000?ooo`3oool0FP3oool00`000000oooo
0?ooo`1K0?ooo`00G03oool00`000000oooo0?ooo`1J0?ooo`030000003oool0oooo05/0oooo001L
0?ooo`030000003oool0oooo05X0oooo00<000000?ooo`3oool0F`3oool005d0oooo00<000000?oo
o`3oool0F@3oool00`000000oooo0?ooo`1K0?ooo`00G@3oool00`000000oooo0?ooo`1I0?ooo`03
0000003oool0oooo05/0oooo001M0?ooo`030000003oool0oooo05T0oooo00<000000?ooo`3oool0
F`3oool005h0oooo00<000000?ooo`3oool0E`3oool00`000000oooo0?ooo`1L0?ooo`00GP3oool0
0`000000oooo0?ooo`1G0?ooo`030000003oool0oooo05`0oooo001N0?ooo`030000003oool0oooo
05H0oooo00<000000?ooo`3oool0G@3oool005l0oooo00<000000?ooo`3oool0E03oool00`000000
oooo0?ooo`1N0?ooo`00G`3oool00`000000oooo0?ooo`1D0?ooo`030000003oool0oooo05h0oooo
001P0?ooo`030000003oool0oooo0580oooo00<000000?ooo`3oool0G`3oool00600oooo00<00000
0?ooo`3oool0DP3oool00`000000oooo0?ooo`1O0?ooo`00H@3oool00`000000oooo0?ooo`1@0?oo
o`030000003oool0oooo0600oooo001Q0?ooo`030000003oool0oooo0500oooo00<000000?ooo`3o
ool0H03oool00680oooo00<000000?ooo`3oool0CP3oool00`000000oooo0?ooo`1Q0?ooo`00H`3o
ool00`000000oooo0?ooo`1<0?ooo`030000003oool0oooo0680oooo001T0?ooo`030000003oool0
oooo04/0oooo00<000000?ooo`3oool0HP3oool006D0oooo00<000000?ooo`3oool0B@3oool00`00
0000oooo0?ooo`1S0?ooo`00I@3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo06<0
oooo001V0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0I03oool006L0oooo00<0
00000?ooo`3oool0A@3oool00`000000oooo0?ooo`1U0?ooo`00J03oool00`000000oooo0?ooo`13
0?ooo`030000003oool0oooo06H0oooo001Y0?ooo`030000003oool0oooo0440oooo00<000000?oo
o`3oool0I`3oool006X0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`1X0?ooo`00
J`3oool00`000000oooo0?ooo`0l0?ooo`800000J`3oool006`0oooo00<000000?ooo`3oool0>P3o
ool00`000000oooo0?ooo`1[0?ooo`00K@3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0
oooo06`0oooo001^0?ooo`800000=`3oool00`000000oooo0?ooo`1]0?ooo`00L03oool2000003<0
oooo0P00001`0?ooo`00LP3oool00`000000oooo0?ooo`0^0?ooo`800000LP3oool007<0oooo0P00
000/0?ooo`800000M03oool007D0oooo0P00000X0?ooo`800000MP3oool007L0oooo0P00000T0?oo
o`800000N03oool007T0oooo0P00000Q0?ooo`030000003oool0oooo07P0oooo001k0?ooo`H00000
5@3oool6000007/0oooo00210?ooo`<000003`3oool300000840oooo00240?ooo`l00000Q03oool0
0?l0oooo603oool00?l0oooo603oool00?l0oooo603oool00?l0oooo603oool00?l0oooo603oool0
0?l0oooo603oool00?l0oooo603oool00?l0oooo603oool00?l0oooo603oool00?l0oooo603oool0
0001\
\>"],
  ImageRangeCache->{{{0, 278.438}, {430.813, 0}} -> {48.4499, 39.1292, \
0.124446, 0.124446}}]
}, Closed]],

Cell["\<\
Then, we evaluate the potential and field using familiar steps.  The circle \
is charged to 1 V while the pail is at ground (as is the edge of the \
computational region).\
\>", "Text"],

Cell[BoxData[{
    \(\(Clear[boundary, \[Phi], \[Phi]0];\)\), "\[IndentingNewLine]", 
    \(\(boundary = Table[False, {i, 1, 120}, {j, 1, 120}];\)\), "\n", 
    \(\(\[Phi] = Table[0.0, {i, 1, 120}, {j, 1, 120}];\)\)}], "Input"],

Cell[BoxData[{
    \(\(Do[
        boundary\[LeftDoubleBracket]i, j\[RightDoubleBracket] = 
          InPolygonQ[{i, j}, poly1], {i, 1, 120}, {j, 1, 
          65}];\)\), "\[IndentingNewLine]", 
    \(\(Do[
        boundary\[LeftDoubleBracket]i, j\[RightDoubleBracket] = 
          InPolygonQ[{i, j}, poly2], {i, 1, 120}, {j, 65, 
          120}];\)\), "\[IndentingNewLine]", 
    \(\(Do[\[Phi]\[LeftDoubleBracket]i, j\[RightDoubleBracket] = 
          If[boundary\[LeftDoubleBracket]i, j\[RightDoubleBracket], 0. , 
            0. ], {i, 1, 120}, {j, 1, 65}];\)\), "\[IndentingNewLine]", 
    \(\(Do[\[Phi]\[LeftDoubleBracket]i, j\[RightDoubleBracket] = 
          If[boundary\[LeftDoubleBracket]i, j\[RightDoubleBracket], 1. , 
            0. ], {i, 1, 120}, {j, 65, 120}];\)\), "\[IndentingNewLine]", 
    \(\(\[Phi]0 = \[Phi];\)\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(\[Phi] = 
        relax[\[Phi]0, boundary, 1000, 
          10\^\(-4\)];\)\), "\[IndentingNewLine]", 
    \(\(f = ListInterpolation[\[Phi]];\)\)}], "Input"],

Cell[BoxData[
    InterpretationBox[\("number of iterations = "\[InvisibleSpace]156\),
      SequenceForm[ "number of iterations = ", 156],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("residual = "\[InvisibleSpace]0.00009863536794881267`\
\),
      SequenceForm[ "residual = ", .000098635367948812674],
      Editable->False]], "Print"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ContourPlot[f[x, y], {x, 1, 120}, {y, 1, 120}, 
        PlotPoints \[Rule] 200, PlotRange \[Rule] All, 
        AspectRatio \[Rule] Automatic, ContourShading \[Rule] False, 
        Contours \[Rule] Range[0.05, 1. , 0.1], Frame \[Rule] False, 
        Epilog \[Rule] {RGBColor[1, 0, 0], Polygon[poly1], \ 
            RGBColor[0, 0, 1], Polygon[poly2]}];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% ContourGraphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0111506 0.00808016 0.0111506 0.00808016 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.49275 .54323 m
.49758 .54245 L
.50242 .54584 L
.5025 .5459 L
.50725 .54711 L
.51208 .54842 L
.51467 .55073 L
.51691 .5528 L
.51967 .55557 L
.52174 .55758 L
.52459 .5604 L
.52607 .56523 L
.52658 .56898 L
.5273 .57006 L
.53078 .57489 L
.53056 .57973 L
.5271 .58456 L
.52658 .58563 L
.52627 .58939 L
.5246 .59422 L
.52174 .59811 L
.52014 .59905 L
.51691 .60241 L
.51523 .60388 L
.51208 .60607 L
.50725 .60742 L
.50398 .60872 L
.50242 .60989 L
.49758 .61248 L
.49275 .6119 L
.4882 .60872 L
.48792 .60844 L
.48309 .60701 L
.47826 .60508 L
.47624 .60388 L
.47342 .60068 L
.47182 .59905 L
.46859 .59585 L
.46779 .59422 L
.46611 .58939 L
.4646 .58456 L
.46376 .58342 L
.46126 .57973 L
.46106 .57489 L
.46376 .57087 L
.46438 .57006 L
.46662 .56523 L
.4681 .5604 L
.46859 .55901 L
.47232 .55557 L
Mistroke
.47342 .55454 L
.47684 .55073 L
.47826 .54981 L
.48309 .54741 L
.48792 .54654 L
.48879 .5459 L
.49275 .54323 L
Mfstroke
.48792 .54038 m
.49275 .53832 L
.49758 .53801 L
.50242 .53949 L
.50591 .54107 L
.50725 .54168 L
.51208 .54346 L
.51664 .5459 L
.51691 .5461 L
.52174 .55008 L
.52245 .55073 L
.52658 .55495 L
.52706 .55557 L
.53017 .5604 L
.53141 .56308 L
.53246 .56523 L
.53476 .57006 L
.53623 .57489 L
.53624 .57499 L
.53643 .57973 L
.53624 .58127 L
.53553 .58456 L
.53422 .58939 L
.53251 .59422 L
.53141 .59667 L
.53012 .59905 L
.52676 .60388 L
.52658 .60411 L
.52221 .60872 L
.52174 .60913 L
.51691 .61275 L
.51564 .61355 L
.51208 .61545 L
.50725 .61743 L
.5044 .61838 L
.50242 .61889 L
.49758 .61973 L
.49275 .61955 L
.48792 .61844 L
.48771 .61838 L
.48309 .61684 L
.47826 .61464 L
.47628 .61355 L
.47342 .61169 L
.4697 .60872 L
.46859 .60765 L
.46517 .60388 L
.46376 .60205 L
.46183 .59905 L
.45941 .59422 L
Mistroke
.45893 .59298 L
.45769 .58939 L
.45635 .58456 L
.45537 .57973 L
.45555 .57489 L
.45712 .57006 L
.45893 .56623 L
.45946 .56523 L
.46183 .5604 L
.46376 .55719 L
.46491 .55557 L
.46859 .55151 L
.46942 .55073 L
.47342 .54724 L
.4752 .5459 L
.47826 .54422 L
.48309 .54228 L
.48619 .54107 L
.48792 .54038 L
Mfstroke
.48309 .53546 m
.48792 .53404 L
.49275 .53307 L
.49758 .5329 L
.50242 .53365 L
.50725 .53494 L
.51097 .53624 L
.51208 .53668 L
.51691 .53902 L
.52042 .54107 L
.52174 .54195 L
.52658 .5457 L
.5268 .5459 L
.53141 .55044 L
.53167 .55073 L
.53549 .55557 L
.53624 .55664 L
.53855 .5604 L
.54097 .56523 L
.54107 .56544 L
.54283 .57006 L
.5441 .57489 L
.54467 .57973 L
.54461 .58456 L
.54406 .58939 L
.54301 .59422 L
.54145 .59905 L
.54107 .60001 L
.53929 .60388 L
.53646 .60872 L
.53624 .60905 L
.53282 .61355 L
.53141 .61513 L
.52809 .61838 L
.52658 .61967 L
.52174 .62319 L
.52171 .62321 L
.51691 .62592 L
.51208 .628 L
.51197 .62804 L
.50725 .62952 L
.50242 .6305 L
.49758 .63098 L
.49275 .63088 L
.48792 .63023 L
.48309 .62907 L
.47993 .62804 L
.47826 .62738 L
.47342 .62509 L
.47022 .62321 L
Mistroke
.46859 .62212 L
.46385 .61838 L
.46376 .6183 L
.45911 .61355 L
.45893 .61333 L
.45547 .60872 L
.4541 .60654 L
.45264 .60388 L
.45049 .59905 L
.44927 .59547 L
.44892 .59422 L
.44787 .58939 L
.44732 .58456 L
.44726 .57973 L
.44783 .57489 L
.4491 .57006 L
.44927 .56956 L
.45096 .56523 L
.45337 .5604 L
.4541 .55914 L
.45645 .55557 L
.45893 .5523 L
.46026 .55073 L
.46376 .54713 L
.46513 .5459 L
.46859 .54309 L
.4715 .54107 L
.47342 .5399 L
.47826 .53739 L
.48095 .53624 L
.48309 .53546 L
Mfstroke
.49275 .52618 m
.49758 .52607 L
.50237 .52658 L
.50242 .52658 L
.50725 .52758 L
.51208 .52906 L
.51691 .53102 L
.51774 .53141 L
.52174 .53349 L
.5262 .53624 L
.52658 .5365 L
.53141 .54009 L
.53258 .54107 L
.53624 .54439 L
.53775 .5459 L
.54107 .54956 L
.54204 .55073 L
.54562 .55557 L
.5459 .55598 L
.54859 .5604 L
.55073 .56461 L
.55101 .56523 L
.55291 .57006 L
.55432 .57489 L
.55524 .57973 L
.55557 .58264 L
.5557 .58456 L
.55572 .58939 L
.55557 .59195 L
.55533 .59422 L
.55451 .59905 L
.55326 .60388 L
.55154 .60872 L
.55073 .61062 L
.54932 .61355 L
.54653 .61838 L
.5459 .61933 L
.54307 .62321 L
.54107 .6256 L
.53879 .62804 L
.53624 .63048 L
.5334 .63288 L
.53141 .6344 L
.52658 .63758 L
.52636 .63771 L
.52174 .64016 L
.51691 .64222 L
.51605 .64254 L
.51208 .64381 L
.50725 .64495 L
Mistroke
.50242 .64569 L
.49758 .64602 L
.49275 .64595 L
.48792 .64549 L
.48309 .64461 L
.47826 .64332 L
.47589 .64254 L
.47342 .64159 L
.46859 .63937 L
.46557 .63771 L
.46376 .6366 L
.45893 .63318 L
.45854 .63288 L
.4541 .62897 L
.45316 .62804 L
.44927 .6237 L
.44887 .62321 L
.44542 .61838 L
.44443 .6168 L
.44262 .61355 L
.4404 .60872 L
.4396 .60665 L
.43868 .60388 L
.43743 .59905 L
.43662 .59422 L
.43623 .58939 L
.43625 .58456 L
.43671 .57973 L
.43761 .57489 L
.43902 .57006 L
.4396 .56841 L
.44093 .56523 L
.44334 .5604 L
.44443 .55851 L
.44631 .55557 L
.44927 .55151 L
.44989 .55073 L
.4541 .54599 L
.45418 .5459 L
.45893 .54142 L
.45934 .54107 L
.46376 .53761 L
.46573 .53624 L
.46859 .53441 L
.47342 .53178 L
.47419 .53141 L
.47826 .52965 L
.48309 .52801 L
.48792 .52686 L
.48952 .52658 L
Mistroke
.49275 .52618 L
Mfstroke
.47826 .52104 m
.48309 .5196 L
.48792 .51861 L
.49275 .51806 L
.49758 .51798 L
.50242 .51838 L
.50725 .51923 L
.51208 .52052 L
.51564 .52174 L
.51691 .52224 L
.52174 .5244 L
.52587 .52658 L
.52658 .52698 L
.53141 .52997 L
.53351 .53141 L
.53624 .5334 L
.53985 .53624 L
.54107 .53726 L
.54532 .54107 L
.5459 .54162 L
.5501 .5459 L
.55073 .54659 L
.55427 .55073 L
.55557 .55239 L
.55788 .55557 L
.5604 .55944 L
.56097 .5604 L
.56357 .56523 L
.56523 .56884 L
.56573 .57006 L
.56746 .57489 L
.56878 .57973 L
.56971 .58456 L
.57006 .58631 L
.57028 .58939 L
.5705 .59422 L
.57037 .59905 L
.57006 .60245 L
.56988 .60388 L
.56904 .60872 L
.56783 .61355 L
.56623 .61838 L
.56523 .62093 L
.56423 .62321 L
.56178 .62804 L
.5604 .63042 L
.55883 .63288 L
.55557 .63741 L
.55533 .63771 L
.55115 .64254 L
Mistroke
.55073 .64298 L
.54614 .64737 L
.5459 .64758 L
.54107 .65144 L
.54002 .6522 L
.53624 .65471 L
.53222 .65704 L
.53141 .65747 L
.52658 .65978 L
.52174 .6617 L
.52127 .66187 L
.51691 .66325 L
.51208 .66445 L
.50725 .66533 L
.50242 .66589 L
.49758 .66615 L
.49275 .6661 L
.48792 .66574 L
.48309 .66507 L
.47826 .66408 L
.47342 .66276 L
.47069 .66187 L
.46859 .66109 L
.46376 .65904 L
.45976 .65704 L
.45893 .65658 L
.4541 .65366 L
.45196 .6522 L
.44927 .65021 L
.44584 .64737 L
.44443 .64611 L
.44083 .64254 L
.4396 .64121 L
.43665 .63771 L
.43477 .63523 L
.43314 .63288 L
.43019 .62804 L
.42994 .62758 L
.42775 .62321 L
.42574 .61838 L
.42511 .6166 L
.42415 .61355 L
.42293 .60872 L
.42209 .60388 L
.4216 .59905 L
.42146 .59422 L
.42168 .58939 L
.42224 .58456 L
.42318 .57973 L
.4245 .57489 L
Mistroke
.42511 .57304 L
.42622 .57006 L
.42837 .56523 L
.42994 .5622 L
.43097 .5604 L
.43406 .55557 L
.43477 .55456 L
.43767 .55073 L
.4396 .5484 L
.44183 .5459 L
.44443 .54318 L
.4466 .54107 L
.44927 .53864 L
.45208 .53624 L
.4541 .53462 L
.45842 .53141 L
.45893 .53105 L
.46376 .52792 L
.46605 .52658 L
.46859 .5252 L
.47342 .52291 L
.47629 .52174 L
.47826 .52104 L
Mfstroke
.47826 .5112 m
.48309 .50985 L
.48792 .50892 L
.49275 .50842 L
.49758 .50835 L
.50242 .50872 L
.50725 .5095 L
.51208 .51072 L
.51622 .51208 L
.51691 .51234 L
.52174 .51437 L
.52658 .51677 L
.52685 .51691 L
.53141 .5195 L
.53502 .52174 L
.53624 .52253 L
.54107 .52582 L
.54213 .52658 L
.5459 .52933 L
.54863 .53141 L
.55073 .53306 L
.55464 .53624 L
.55557 .53702 L
.56017 .54107 L
.5604 .54128 L
.56519 .5459 L
.56523 .54594 L
.56968 .55073 L
.57006 .55117 L
.57366 .55557 L
.57489 .55719 L
.57715 .5604 L
.57973 .56446 L
.58017 .56523 L
.58276 .57006 L
.58456 .57395 L
.58495 .57489 L
.58678 .57973 L
.58825 .58456 L
.58939 .58939 L
.58939 .58941 L
.5902 .59422 L
.59072 .59905 L
.59093 .60388 L
.59085 .60872 L
.59047 .61355 L
.5898 .61838 L
.58939 .62064 L
.58883 .62321 L
.58755 .62804 L
Mistroke
.58596 .63288 L
.58456 .63648 L
.58404 .63771 L
.58175 .64254 L
.57973 .64628 L
.57909 .64737 L
.57601 .6522 L
.57489 .65379 L
.57246 .65704 L
.57006 .65995 L
.56838 .66187 L
.56523 .66517 L
.56367 .6667 L
.5604 .66967 L
.55818 .67153 L
.55557 .67358 L
.55169 .67636 L
.55073 .67701 L
.5459 .68001 L
.54381 .68119 L
.54107 .68264 L
.53624 .68493 L
.53365 .68603 L
.53141 .68691 L
.52658 .68859 L
.52174 .69 L
.51829 .69086 L
.51691 .69116 L
.51208 .69207 L
.50725 .69274 L
.50242 .69316 L
.49758 .69336 L
.49275 .69331 L
.48792 .69304 L
.48309 .69253 L
.47826 .69179 L
.47369 .69086 L
.47342 .6908 L
.46859 .68955 L
.46376 .68804 L
.45893 .68625 L
.45838 .68603 L
.4541 .68416 L
.44927 .68176 L
.44822 .68119 L
.44443 .679 L
.44034 .67636 L
.4396 .67585 L
.43477 .67226 L
.43386 .67153 L
Mistroke
.42994 .66815 L
.42838 .6667 L
.42511 .66342 L
.42367 .66187 L
.42027 .6579 L
.41958 .65704 L
.41604 .6522 L
.41544 .65132 L
.41295 .64737 L
.41061 .64317 L
.41029 .64254 L
.408 .63771 L
.40607 .63288 L
.40578 .63207 L
.40447 .62804 L
.40319 .62321 L
.40222 .61838 L
.40154 .61355 L
.40116 .60872 L
.40107 .60388 L
.40128 .59905 L
.40178 .59422 L
.4026 .58939 L
.40374 .58456 L
.40521 .57973 L
.40578 .57808 L
.40702 .57489 L
.40921 .57006 L
.41061 .56735 L
.41181 .56523 L
.41483 .5604 L
.41544 .55949 L
.4183 .55557 L
.42027 .55307 L
.42226 .55073 L
.42511 .54761 L
.42676 .5459 L
.42994 .54277 L
.43177 .54107 L
.43477 .53839 L
.43729 .53624 L
.4396 .53434 L
.44331 .53141 L
.44443 .53054 L
.44927 .52696 L
.44981 .52658 L
.4541 .5236 L
.45692 .52174 L
.45893 .52047 L
.46376 .51763 L
Mistroke
.46507 .51691 L
.46859 .51512 L
.47342 .51296 L
.47569 .51208 L
.47826 .5112 L
Mfstroke
.48792 .49723 m
.49275 .49672 L
.49758 .49665 L
.50242 .49701 L
.50638 .49758 L
.50725 .49782 L
.51208 .49906 L
.51691 .50074 L
.52085 .50242 L
.52174 .50284 L
.52658 .50533 L
.5299 .50725 L
.53141 .50817 L
.53624 .51126 L
.53749 .51208 L
.54107 .51449 L
.54462 .51691 L
.5459 .51778 L
.55073 .52104 L
.55178 .52174 L
.55557 .52425 L
.55911 .52658 L
.5604 .52741 L
.56523 .53056 L
.56651 .53141 L
.57006 .53378 L
.57365 .53624 L
.57489 .53711 L
.57973 .54063 L
.58031 .54107 L
.58456 .54442 L
.58634 .5459 L
.58939 .54857 L
.59171 .55073 L
.59422 .55323 L
.59643 .55557 L
.59905 .55855 L
.60058 .5604 L
.60388 .56475 L
.60422 .56523 L
.6074 .57006 L
.60872 .57227 L
.61017 .57489 L
.61256 .57973 L
.61355 .58195 L
.6146 .58456 L
.61632 .58939 L
.61774 .59422 L
.61838 .59676 L
.61888 .59905 L
Mistroke
.61975 .60388 L
.62036 .60872 L
.62071 .61355 L
.62082 .61838 L
.62068 .62321 L
.62031 .62804 L
.61969 .63288 L
.61883 .63771 L
.61838 .63981 L
.61772 .64254 L
.61636 .64737 L
.61475 .6522 L
.61355 .65539 L
.61288 .65704 L
.61072 .66187 L
.60872 .66588 L
.60828 .6667 L
.60552 .67153 L
.60388 .67416 L
.60243 .67636 L
.59905 .68108 L
.59897 .68119 L
.5951 .68603 L
.59422 .68706 L
.59079 .69086 L
.58939 .69232 L
.58596 .69569 L
.58456 .697 L
.58054 .70052 L
.57973 .7012 L
.57489 .70498 L
.57439 .70535 L
.57006 .70839 L
.56732 .71019 L
.56523 .71148 L
.5604 .71429 L
.55904 .71502 L
.55557 .71679 L
.55073 .71905 L
.54889 .71985 L
.5459 .72107 L
.54107 .72288 L
.53624 .72449 L
.53561 .72468 L
.53141 .72589 L
.52658 .7271 L
.52174 .72812 L
.51691 .72896 L
.5129 .72951 L
.51208 .72961 L
Mistroke
.50725 .7301 L
.50242 .73041 L
.49758 .73055 L
.49275 .73052 L
.48792 .73032 L
.48309 .72994 L
.47921 .72951 L
.47826 .72938 L
.47342 .72866 L
.46859 .72775 L
.46376 .72666 L
.45893 .72539 L
.45651 .72468 L
.4541 .72392 L
.44927 .72225 L
.44443 .72036 L
.44323 .71985 L
.4396 .71823 L
.43477 .71586 L
.43315 .71502 L
.42994 .71325 L
.42511 .71035 L
.42485 .71019 L
.42027 .70714 L
.41778 .70535 L
.41544 .70359 L
.41163 .70052 L
.41061 .69966 L
.4062 .69569 L
.40578 .69529 L
.40137 .69086 L
.40095 .69041 L
.39705 .68603 L
.39612 .6849 L
.39319 .68119 L
.39128 .6786 L
.38973 .67636 L
.38663 .67153 L
.38645 .67123 L
.38387 .6667 L
.38162 .66228 L
.38142 .66187 L
.37926 .65704 L
.37738 .6522 L
.37679 .65052 L
.37577 .64737 L
.37441 .64254 L
.37331 .63771 L
.37244 .63288 L
.37196 .62939 L
Mistroke
.37181 .62804 L
.37143 .62321 L
.37129 .61838 L
.37139 .61355 L
.37173 .60872 L
.37196 .60651 L
.37234 .60388 L
.3732 .59905 L
.37432 .59422 L
.37574 .58939 L
.37679 .58631 L
.37745 .58456 L
.37949 .57973 L
.38162 .57537 L
.38187 .57489 L
.38463 .57006 L
.38645 .5672 L
.3878 .56523 L
.39128 .56058 L
.39143 .5604 L
.39558 .55557 L
.39612 .55499 L
.4003 .55073 L
.40095 .55012 L
.40565 .5459 L
.40578 .54579 L
.41061 .54188 L
.41166 .54107 L
.41544 .53827 L
.41831 .53624 L
.42027 .53488 L
.42511 .53163 L
.42544 .53141 L
.42994 .52846 L
.43283 .52658 L
.43477 .5253 L
.4396 .52211 L
.44015 .52174 L
.44443 .51887 L
.44731 .51691 L
.44927 .51557 L
.4541 .51231 L
.45444 .51208 L
.45893 .50916 L
.46202 .50725 L
.46376 .50623 L
.46859 .50361 L
.47106 .50242 L
.47342 .50138 L
.47826 .49956 L
Mistroke
.48309 .49818 L
.48564 .49758 L
.48792 .49723 L
Mfstroke
.48792 .48225 m
.49275 .4817 L
.49758 .48162 L
.50242 .48201 L
.50725 .48288 L
.50816 .48309 L
.51208 .48424 L
.51691 .48611 L
.52069 .48792 L
.52174 .48849 L
.52658 .49138 L
.52863 .49275 L
.53141 .49476 L
.53505 .49758 L
.53624 .49854 L
.541 .50242 L
.54107 .50247 L
.5459 .50634 L
.54709 .50725 L
.55073 .50988 L
.55404 .51208 L
.55557 .51302 L
.5604 .51575 L
.56272 .51691 L
.56523 .5181 L
.57006 .52026 L
.57361 .52174 L
.57489 .52226 L
.57973 .52417 L
.58456 .52602 L
.58604 .52658 L
.58939 .52784 L
.59422 .52971 L
.59842 .53141 L
.59905 .53167 L
.60388 .5338 L
.60872 .53612 L
.60895 .53624 L
.61355 .53869 L
.61759 .54107 L
.61838 .54156 L
.62321 .54476 L
.6248 .5459 L
.62804 .54837 L
.63092 .55073 L
.63288 .55245 L
.63618 .55557 L
.63771 .5571 L
.64078 .5604 L
.64254 .56243 L
Mistroke
.64481 .56523 L
.64737 .56864 L
.64837 .57006 L
.65151 .57489 L
.6522 .57605 L
.65429 .57973 L
.65674 .58456 L
.65704 .58519 L
.65888 .58939 L
.66075 .59422 L
.66187 .59748 L
.66236 .59905 L
.66373 .60388 L
.66487 .60872 L
.66579 .61355 L
.6665 .61838 L
.6667 .62 L
.66699 .62321 L
.6673 .62804 L
.66741 .63288 L
.66732 .63771 L
.66705 .64254 L
.6667 .6465 L
.6666 .64737 L
.66595 .6522 L
.66511 .65704 L
.66409 .66187 L
.66288 .6667 L
.66187 .67024 L
.66147 .67153 L
.65986 .67636 L
.65804 .68119 L
.65704 .68364 L
.65601 .68603 L
.65376 .69086 L
.6522 .69395 L
.65128 .69569 L
.64857 .70052 L
.64737 .70253 L
.6456 .70535 L
.64254 .70993 L
.64236 .71019 L
.63883 .71502 L
.63771 .71647 L
.635 .71985 L
.63288 .72236 L
.63083 .72468 L
.62804 .72769 L
.62628 .72951 L
.62321 .73255 L
Mistroke
.62132 .73434 L
.61838 .73702 L
.6159 .73918 L
.61355 .74114 L
.60994 .74401 L
.60872 .74495 L
.60388 .74849 L
.60338 .74884 L
.59905 .75177 L
.59608 .75367 L
.59422 .75482 L
.58939 .75764 L
.58784 .7585 L
.58456 .76026 L
.57973 .76269 L
.57837 .76334 L
.57489 .76494 L
.57006 .76702 L
.5672 .76817 L
.56523 .76893 L
.5604 .77069 L
.55557 .77231 L
.55337 .773 L
.55073 .77379 L
.5459 .77512 L
.54107 .77631 L
.53624 .77738 L
.534 .77783 L
.53141 .77832 L
.52658 .77915 L
.52174 .77985 L
.51691 .78042 L
.51208 .78086 L
.50725 .78119 L
.50242 .7814 L
.49758 .7815 L
.49275 .78147 L
.48792 .78133 L
.48309 .78106 L
.47826 .78068 L
.47342 .78017 L
.46859 .77955 L
.46376 .77881 L
.45893 .77796 L
.45827 .77783 L
.4541 .77697 L
.44927 .77585 L
.44443 .77459 L
.4396 .77319 L
.43897 .773 L
Mistroke
.43477 .77167 L
.42994 .76999 L
.42511 .76817 L
.42511 .76817 L
.42027 .76618 L
.41544 .76403 L
.41398 .76334 L
.41061 .76169 L
.40578 .75917 L
.40456 .7585 L
.40095 .75646 L
.39632 .75367 L
.39612 .75355 L
.39128 .75041 L
.389 .74884 L
.38645 .74702 L
.38245 .74401 L
.38162 .74336 L
.37679 .73942 L
.3765 .73918 L
.37196 .73516 L
.37108 .73434 L
.36712 .73053 L
.36611 .72951 L
.36229 .72548 L
.36156 .72468 L
.35746 .71993 L
.3574 .71985 L
.35357 .71502 L
.35263 .71377 L
.35005 .71019 L
.3478 .70685 L
.34682 .70535 L
.34385 .70052 L
.34296 .699 L
.34113 .69569 L
.33866 .69086 L
.33813 .68976 L
.33641 .68603 L
.33439 .68119 L
.3333 .67837 L
.33257 .67636 L
.33095 .67153 L
.32954 .6667 L
.32847 .66252 L
.32832 .66187 L
.32729 .65704 L
.32645 .6522 L
.3258 .64737 L
.32533 .64254 L
Mistroke
.32506 .63771 L
.32497 .63288 L
.32506 .62804 L
.32536 .62321 L
.32586 .61838 L
.32656 .61355 L
.32748 .60872 L
.32847 .60444 L
.32861 .60388 L
.32998 .59905 L
.33158 .59422 L
.3333 .5897 L
.33343 .58939 L
.33556 .58456 L
.33801 .57973 L
.33813 .57949 L
.34078 .57489 L
.34296 .57145 L
.3439 .57006 L
.34744 .56523 L
.3478 .56477 L
.35145 .5604 L
.35263 .5591 L
.35603 .55557 L
.35746 .55418 L
.36128 .55073 L
.36229 .54987 L
.36712 .54607 L
.36736 .5459 L
.37196 .54271 L
.37453 .54107 L
.37679 .53971 L
.38162 .53703 L
.38315 .53624 L
.38645 .53461 L
.39128 .53241 L
.39363 .53141 L
.39612 .53038 L
.40095 .52848 L
.40578 .52664 L
.40595 .52658 L
.41061 .5248 L
.41544 .52292 L
.41836 .52174 L
.42027 .52094 L
.42511 .51884 L
.42921 .51691 L
.42994 .51655 L
.43477 .51397 L
.43789 .51208 L
Mistroke
.4396 .51095 L
.44443 .50755 L
.44484 .50725 L
.44927 .50376 L
.45091 .50242 L
.4541 .49982 L
.45685 .49758 L
.45893 .49596 L
.46329 .49275 L
.46376 .49243 L
.46859 .48937 L
.47121 .48792 L
.47342 .48683 L
.47826 .4848 L
.48309 .48328 L
.48373 .48309 L
.48792 .48225 L
Mfstroke
.47826 .46335 m
.48309 .46166 L
.48792 .46052 L
.49275 .45991 L
.49758 .45983 L
.50242 .46026 L
.50725 .46121 L
.51208 .46272 L
.51469 .46376 L
.51691 .46481 L
.52174 .46754 L
.52335 .46859 L
.52658 .47097 L
.52951 .47342 L
.53141 .47518 L
.53445 .47826 L
.53624 .48024 L
.53865 .48309 L
.54107 .48617 L
.54239 .48792 L
.5459 .49273 L
.54592 .49275 L
.54963 .49758 L
.55073 .4989 L
.55431 .50242 L
.55557 .50339 L
.5604 .50614 L
.56344 .50725 L
.56523 .50783 L
.57006 .50922 L
.57489 .51032 L
.57973 .51115 L
.58456 .51167 L
.58939 .51176 L
.59422 .51156 L
.59905 .5112 L
.60388 .51077 L
.60872 .51032 L
.61355 .50994 L
.61838 .50965 L
.62321 .5095 L
.62804 .50949 L
.63288 .50966 L
.63771 .51002 L
.64254 .51058 L
.64737 .51135 L
.65106 .51208 L
.6522 .51233 L
.65704 .51355 L
.66187 .51502 L
Mistroke
.6667 .51673 L
.66717 .51691 L
.67153 .5187 L
.67636 .52097 L
.67789 .52174 L
.68119 .52354 L
.68603 .52643 L
.68625 .52658 L
.69086 .52968 L
.69321 .53141 L
.69569 .53332 L
.69922 .53624 L
.70052 .53737 L
.70452 .54107 L
.70535 .54188 L
.70922 .5459 L
.71019 .54698 L
.71342 .55073 L
.71502 .55271 L
.71721 .55557 L
.71985 .55923 L
.72065 .5604 L
.72375 .56523 L
.72468 .56677 L
.72658 .57006 L
.72916 .57489 L
.72951 .5756 L
.73149 .57973 L
.73361 .58456 L
.73434 .58636 L
.73551 .58939 L
.73723 .59422 L
.73877 .59905 L
.73918 .60041 L
.74015 .60388 L
.74135 .60872 L
.74239 .61355 L
.74327 .61838 L
.74401 .6232 L
.74401 .62321 L
.74461 .62804 L
.74506 .63288 L
.74536 .63771 L
.74552 .64254 L
.74556 .64737 L
.74546 .6522 L
.74523 .65704 L
.74488 .66187 L
.74439 .6667 L
.74401 .66982 L
Mistroke
.74377 .67153 L
.74301 .67636 L
.74211 .68119 L
.74108 .68603 L
.73992 .69086 L
.73918 .69364 L
.7386 .69569 L
.73715 .70052 L
.73557 .70535 L
.73434 .70877 L
.73382 .71019 L
.73192 .71502 L
.72986 .71985 L
.72951 .72063 L
.72764 .72468 L
.72526 .72951 L
.72468 .73062 L
.72268 .73434 L
.71993 .73918 L
.71985 .73931 L
.71698 .74401 L
.71502 .74705 L
.71382 .74884 L
.71045 .75367 L
.71019 .75404 L
.70685 .7585 L
.70535 .76042 L
.70301 .76334 L
.70052 .76633 L
.69894 .76817 L
.69569 .77181 L
.6946 .773 L
.69086 .77692 L
.68996 .77783 L
.68603 .78168 L
.68499 .78266 L
.68119 .78613 L
.67966 .7875 L
.67636 .79033 L
.67395 .79233 L
.67153 .79427 L
.66778 .79716 L
.6667 .79797 L
.66187 .80145 L
.66109 .80199 L
.65704 .80475 L
.65386 .80682 L
.6522 .80787 L
.64737 .81082 L
.64597 .81165 L
Mistroke
.64254 .81363 L
.63771 .81628 L
.63732 .81649 L
.63288 .81879 L
.62804 .82118 L
.62775 .82132 L
.62321 .82344 L
.61838 .82559 L
.61706 .82615 L
.61355 .82761 L
.60872 .82951 L
.60479 .83098 L
.60388 .83131 L
.59905 .83301 L
.59422 .83462 L
.59046 .83581 L
.58939 .83614 L
.58456 .83757 L
.57973 .8389 L
.57489 .84014 L
.57283 .84065 L
.57006 .8413 L
.56523 .84237 L
.5604 .84337 L
.55557 .84431 L
.55073 .84517 L
.54888 .84548 L
.5459 .84594 L
.54107 .84664 L
.53624 .84726 L
.53141 .84781 L
.52658 .84829 L
.52174 .84869 L
.51691 .84902 L
.51208 .84927 L
.50725 .84947 L
.50242 .8496 L
.49758 .84964 L
.49275 .84963 L
.48792 .84955 L
.48309 .84939 L
.47826 .84916 L
.47342 .84886 L
.46859 .84849 L
.46376 .84806 L
.45893 .84755 L
.4541 .84696 L
.44927 .84629 L
.44443 .84556 L
.44392 .84548 L
Mistroke
.4396 .84476 L
.43477 .84389 L
.42994 .84293 L
.42511 .8419 L
.42027 .84078 L
.41973 .84065 L
.41544 .83957 L
.41061 .83827 L
.40578 .83689 L
.40218 .83581 L
.40095 .83543 L
.39612 .83388 L
.39128 .83222 L
.38787 .83098 L
.38645 .83045 L
.38162 .82858 L
.37679 .82662 L
.37567 .82615 L
.37196 .82454 L
.36712 .82234 L
.36497 .82132 L
.36229 .82002 L
.35746 .81756 L
.35543 .81649 L
.35263 .81496 L
.3478 .81223 L
.34681 .81165 L
.34296 .80934 L
.33894 .80682 L
.33813 .8063 L
.3333 .80308 L
.33173 .80199 L
.32847 .79966 L
.3251 .79716 L
.32364 .79604 L
.31897 .79233 L
.31881 .79219 L
.31397 .7881 L
.31329 .7875 L
.30914 .78373 L
.30801 .78266 L
.30431 .77907 L
.30307 .77783 L
.29948 .77412 L
.29843 .773 L
.29465 .7688 L
.29409 .76817 L
.29001 .76334 L
.28981 .7631 L
.2862 .7585 L
Mistroke
.28498 .75689 L
.28263 .75367 L
.28015 .75012 L
.27928 .74884 L
.27614 .74401 L
.27532 .7427 L
.27319 .73918 L
.27049 .73442 L
.27044 .73434 L
.26789 .72951 L
.26566 .72498 L
.26551 .72468 L
.26331 .71985 L
.26125 .71502 L
.26082 .71396 L
.25937 .71019 L
.25765 .70535 L
.25607 .70052 L
.25599 .70028 L
.25463 .69569 L
.25333 .69086 L
.25218 .68603 L
.25116 .68119 L
.25116 .68119 L
.25027 .67636 L
.24952 .67153 L
.2489 .6667 L
.2484 .66187 L
.24804 .65704 L
.24781 .6522 L
.24771 .64737 L
.24774 .64254 L
.2479 .63771 L
.24821 .63288 L
.24866 .62804 L
.24924 .62321 L
.24997 .61838 L
.25085 .61355 L
.25116 .61199 L
.25188 .60872 L
.25306 .60388 L
.25441 .59905 L
.25594 .59422 L
.25599 .59408 L
.25766 .58939 L
.25957 .58456 L
.26082 .58162 L
.26168 .57973 L
.264 .57489 L
.26566 .57172 L
Mistroke
.26656 .57006 L
.26938 .56523 L
.27049 .56346 L
.27249 .5604 L
.27532 .55636 L
.2759 .55557 L
.27967 .55073 L
.28015 .55015 L
.28387 .5459 L
.28498 .5447 L
.28854 .54107 L
.28981 .53983 L
.29377 .53624 L
.29465 .53549 L
.29948 .53164 L
.29978 .53141 L
.30431 .52819 L
.30677 .52658 L
.30914 .52511 L
.31397 .52238 L
.31519 .52174 L
.31881 .51996 L
.32364 .51783 L
.32595 .51691 L
.32847 .51598 L
.3333 .5144 L
.33813 .51306 L
.34237 .51208 L
.34296 .51196 L
.3478 .51107 L
.35263 .5104 L
.35746 .50995 L
.36229 .50968 L
.36712 .50961 L
.37196 .50969 L
.37679 .50992 L
.38162 .51026 L
.38645 .51068 L
.39128 .5111 L
.39612 .51149 L
.40095 .51173 L
.40578 .51176 L
.41061 .51137 L
.41544 .51063 L
.42027 .50962 L
.42511 .50832 L
.4285 .50725 L
.42994 .50674 L
.43477 .50445 L
.4376 .50242 L
Mistroke
.4396 .50055 L
.44228 .49758 L
.44443 .49487 L
.446 .49275 L
.44927 .48828 L
.44953 .48792 L
.45327 .48309 L
.4541 .48209 L
.45747 .47826 L
.45893 .47674 L
.46241 .47342 L
.46376 .47227 L
.46859 .46859 L
.4686 .46859 L
.47342 .46565 L
.47727 .46376 L
.47826 .46335 L
Mfstroke
.71985 .40559 m
.72468 .40556 L
.72951 .40572 L
.73084 .40578 L
.73434 .40608 L
.73918 .40664 L
.74401 .40737 L
.74884 .40831 L
.75367 .40947 L
.75775 .41061 L
.7585 .41084 L
.76334 .41241 L
.76817 .41422 L
.77113 .41544 L
.773 .41626 L
.77783 .41855 L
.78116 .42027 L
.78266 .4211 L
.7875 .4239 L
.78944 .42511 L
.79233 .42697 L
.79668 .42994 L
.79716 .43028 L
.80199 .43389 L
.80311 .43477 L
.80682 .43784 L
.80886 .4396 L
.81165 .44213 L
.8141 .44443 L
.81649 .44677 L
.81893 .44927 L
.82132 .4518 L
.82339 .4541 L
.82615 .4573 L
.82751 .45893 L
.83098 .46326 L
.83138 .46376 L
.83499 .46859 L
.83581 .46974 L
.83839 .47342 L
.84065 .47685 L
.84154 .47826 L
.84451 .48309 L
.84548 .48472 L
.84731 .48792 L
.84996 .49275 L
.85031 .49341 L
.85245 .49758 L
.85476 .50242 L
.85514 .50324 L
Mistroke
.85696 .50725 L
.85904 .51208 L
.85997 .51433 L
.86103 .51691 L
.86293 .52174 L
.86472 .52658 L
.8648 .52681 L
.86641 .53141 L
.868 .53624 L
.86949 .54107 L
.86964 .54155 L
.87091 .5459 L
.87225 .55073 L
.87349 .55557 L
.87447 .55964 L
.87465 .5604 L
.87574 .56523 L
.87675 .57006 L
.87768 .57489 L
.87855 .57973 L
.8793 .58422 L
.87935 .58456 L
.88007 .58939 L
.88075 .59422 L
.88139 .59905 L
.88194 .60388 L
.88241 .60872 L
.88284 .61355 L
.88321 .61838 L
.88352 .62321 L
.88378 .62804 L
.88398 .63288 L
.88413 .63766 L
.88413 .63771 L
.88425 .64254 L
.88432 .64737 L
.88431 .6522 L
.88422 .65704 L
.88413 .6611 L
.88411 .66187 L
.88395 .6667 L
.8837 .67153 L
.88341 .67636 L
.88306 .68119 L
.88265 .68603 L
.88218 .69086 L
.88167 .69569 L
.88109 .70052 L
.88044 .70535 L
.87975 .71019 L
Mistroke
.8793 .71308 L
.87899 .71502 L
.87816 .71985 L
.87726 .72468 L
.87629 .72951 L
.87523 .73434 L
.87447 .73761 L
.87409 .73918 L
.87286 .74401 L
.87158 .74884 L
.87022 .75367 L
.86964 .75562 L
.86874 .7585 L
.86716 .76334 L
.86548 .76817 L
.8648 .77003 L
.8637 .773 L
.86184 .77783 L
.85997 .78232 L
.85983 .78266 L
.8577 .7875 L
.85544 .79233 L
.85514 .79294 L
.85304 .79716 L
.85049 .80199 L
.85031 .80232 L
.84775 .80682 L
.84548 .81066 L
.84487 .81165 L
.84178 .81649 L
.84065 .81819 L
.83851 .82132 L
.83581 .82504 L
.83499 .82615 L
.83127 .83098 L
.83098 .83134 L
.82726 .83581 L
.82615 .8371 L
.82301 .84065 L
.82132 .84247 L
.81843 .84548 L
.81649 .84744 L
.81354 .85031 L
.81165 .85207 L
.80821 .85514 L
.80682 .85634 L
.80244 .85997 L
.80199 .86034 L
.79716 .86416 L
.79631 .8648 L
Mistroke
.79233 .86775 L
.78966 .86964 L
.7875 .87112 L
.78266 .87433 L
.78244 .87447 L
.77783 .87733 L
.77449 .8793 L
.773 .88016 L
.76817 .88286 L
.76583 .88413 L
.76334 .88545 L
.7585 .88791 L
.75635 .88896 L
.75367 .89025 L
.74884 .89246 L
.74581 .8938 L
.74401 .89457 L
.73918 .89659 L
.73434 .89853 L
.73409 .89863 L
.72951 .90039 L
.72468 .90214 L
.72093 .90346 L
.71985 .90384 L
.71502 .90548 L
.71019 .90706 L
.70624 .90829 L
.70535 .90856 L
.70052 .91003 L
.69569 .91142 L
.69086 .91275 L
.68945 .91312 L
.68603 .91401 L
.68119 .91521 L
.67636 .91635 L
.67153 .91744 L
.66918 .91796 L
.6667 .91849 L
.66187 .91949 L
.65704 .92045 L
.6522 .92136 L
.64737 .92223 L
.64418 .92279 L
.64254 .92307 L
.63771 .92386 L
.63288 .92462 L
.62804 .92536 L
.62321 .92605 L
.61838 .92671 L
.61355 .92735 L
Mistroke
.61143 .92762 L
.60872 .92796 L
.60388 .92854 L
.59905 .92909 L
.59422 .92962 L
.58939 .93012 L
.58456 .93059 L
.57973 .93105 L
.57489 .93147 L
.57006 .93185 L
.56523 .93221 L
.56185 .93245 L
.5604 .93255 L
.55557 .93286 L
.55073 .93314 L
.5459 .93338 L
.54107 .93361 L
.53624 .93382 L
.53141 .93398 L
.52658 .93413 L
.52174 .93427 L
.51691 .93438 L
.51208 .93446 L
.50725 .93452 L
.50242 .93455 L
.49758 .93458 L
.49275 .93458 L
.48792 .93455 L
.48309 .9345 L
.47826 .93442 L
.47342 .93434 L
.46859 .93422 L
.46376 .93407 L
.45893 .93391 L
.4541 .93372 L
.44927 .9335 L
.44443 .93324 L
.4396 .93296 L
.43477 .93267 L
.43136 .93245 L
.42994 .93235 L
.42511 .93199 L
.42027 .9316 L
.41544 .9312 L
.41061 .93077 L
.40578 .93032 L
.40095 .92984 L
.39612 .92933 L
.39128 .9288 L
.38645 .92824 L
Mistroke
.38162 .92766 L
.38131 .92762 L
.37679 .92703 L
.37196 .92637 L
.36712 .92569 L
.36229 .92497 L
.35746 .92422 L
.35263 .92343 L
.34883 .92279 L
.3478 .92261 L
.34296 .92174 L
.33813 .92083 L
.3333 .91989 L
.32847 .91892 L
.3239 .91796 L
.32364 .9179 L
.31881 .91681 L
.31397 .91567 L
.30914 .91448 L
.30431 .91323 L
.3039 .91312 L
.29948 .91192 L
.29465 .91054 L
.28981 .90909 L
.28727 .90829 L
.28498 .90757 L
.28015 .90601 L
.27532 .90438 L
.27269 .90346 L
.27049 .90267 L
.26566 .90091 L
.26082 .89907 L
.2597 .89863 L
.25599 .89714 L
.25116 .89515 L
.24804 .8938 L
.24633 .89303 L
.2415 .8908 L
.2377 .88896 L
.23666 .88845 L
.23183 .88598 L
.22838 .88413 L
.227 .88338 L
.22217 .88066 L
.21983 .8793 L
.21734 .87781 L
.2125 .8748 L
.21199 .87447 L
.20767 .87161 L
.20482 .86964 L
Mistroke
.20284 .86822 L
.19827 .8648 L
.19801 .86461 L
.19318 .86076 L
.19223 .85997 L
.18835 .85666 L
.18663 .85514 L
.18351 .85228 L
.18144 .85031 L
.17868 .84759 L
.17661 .84548 L
.17385 .84258 L
.17208 .84065 L
.16902 .83717 L
.16788 .83581 L
.16419 .83126 L
.16396 .83098 L
.16033 .82615 L
.15935 .8248 L
.15692 .82132 L
.15452 .81773 L
.15372 .81649 L
.15074 .81165 L
.14969 .80991 L
.1479 .80682 L
.14524 .80199 L
.14486 .80128 L
.14272 .79716 L
.14037 .79233 L
.14003 .7916 L
.13817 .7875 L
.13611 .78266 L
.1352 .78042 L
.13416 .77783 L
.13232 .773 L
.13059 .76817 L
.13036 .76751 L
.12894 .76334 L
.12741 .7585 L
.12597 .75367 L
.12553 .75217 L
.12458 .74884 L
.12331 .74401 L
.12214 .73918 L
.12103 .73434 L
.1207 .73285 L
.11999 .72951 L
.11902 .72468 L
.11813 .71985 L
.11733 .71502 L
Mistroke
.11658 .71019 L
.1159 .70535 L
.11587 .70513 L
.11529 .70052 L
.11474 .69569 L
.11423 .69086 L
.11378 .68603 L
.11339 .68119 L
.11306 .67636 L
.11279 .67153 L
.11258 .6667 L
.1124 .66187 L
.11227 .65704 L
.11222 .6522 L
.11222 .64737 L
.11226 .64254 L
.11238 .63771 L
.11257 .63288 L
.11277 .62804 L
.11304 .62321 L
.1134 .61838 L
.11377 .61355 L
.11419 .60872 L
.11467 .60388 L
.11521 .59905 L
.11581 .59422 L
.11587 .59377 L
.11646 .58939 L
.11718 .58456 L
.11799 .57973 L
.11885 .57489 L
.11979 .57006 L
.1207 .56565 L
.12079 .56523 L
.12187 .5604 L
.12304 .55557 L
.12429 .55073 L
.12553 .5462 L
.12562 .5459 L
.12704 .54107 L
.12855 .53624 L
.13015 .53141 L
.13036 .53077 L
.13182 .52658 L
.13359 .52174 L
.1352 .51761 L
.13548 .51691 L
.13747 .51208 L
.13957 .50725 L
.14003 .50624 L
Mistroke
.14179 .50242 L
.14412 .49758 L
.14486 .49612 L
.14662 .49275 L
.14929 .48792 L
.14969 .48722 L
.15211 .48309 L
.15452 .47916 L
.15509 .47826 L
.15829 .47342 L
.15935 .47188 L
.16169 .46859 L
.16419 .46524 L
.16534 .46376 L
.16902 .45923 L
.16928 .45893 L
.1735 .4541 L
.17385 .45372 L
.17803 .44927 L
.17868 .4486 L
.18295 .44443 L
.18351 .44391 L
.18835 .4396 L
.18835 .4396 L
.19318 .43562 L
.19426 .43477 L
.19801 .43196 L
.20087 .42994 L
.20284 .42862 L
.20767 .42558 L
.20847 .42511 L
.2125 .42283 L
.21734 .42028 L
.21734 .42027 L
.22217 .41797 L
.227 .41591 L
.22819 .41544 L
.23183 .4141 L
.23666 .41252 L
.2415 .41115 L
.24362 .41061 L
.24633 .41 L
.25116 .40906 L
.25599 .40833 L
.26082 .4078 L
.26566 .40747 L
.27049 .40733 L
.27532 .40738 L
.28015 .40765 L
.28498 .40814 L
Mistroke
.28981 .40883 L
.29465 .40973 L
.29855 .41061 L
.29948 .41084 L
.30431 .41219 L
.30914 .41375 L
.31373 .41544 L
.31397 .41554 L
.31881 .41754 L
.32364 .41977 L
.32465 .42027 L
.32847 .42225 L
.3333 .42498 L
.33352 .42511 L
.33813 .42796 L
.34111 .42994 L
.34296 .43123 L
.34778 .43477 L
.3478 .43478 L
.35263 .43863 L
.35379 .4396 L
.35746 .44281 L
.35924 .44443 L
.36229 .44733 L
.36425 .44927 L
.36712 .45222 L
.36889 .4541 L
.37196 .4575 L
.37321 .45893 L
.37679 .46319 L
.37726 .46376 L
.38108 .46859 L
.38162 .46931 L
.38469 .47342 L
.38645 .47587 L
.38814 .47826 L
.39128 .48286 L
.39144 .48309 L
.39469 .48792 L
.39612 .49011 L
.39775 .49275 L
.40095 .49745 L
.40101 .49758 L
.40578 .50122 L
.41061 .50178 L
.41544 .50163 L
.42027 .50145 L
.42511 .50126 L
.42994 .5008 L
.43396 .49758 L
Mistroke
.43477 .49647 L
.4363 .49275 L
.43703 .48792 L
.43801 .48309 L
.43898 .47826 L
.4396 .47533 L
.44011 .47342 L
.44149 .46859 L
.44304 .46376 L
.44443 .45986 L
.44478 .45893 L
.44674 .4541 L
.44897 .44927 L
.44927 .44868 L
.45156 .44443 L
.4541 .44033 L
.45459 .4396 L
.45822 .43477 L
.45893 .43394 L
.4627 .42994 L
.46376 .42895 L
.4685 .42511 L
.46859 .42504 L
.47342 .42201 L
.47693 .42027 L
.47826 .41972 L
.48309 .41804 L
.48792 .41693 L
.49275 .41635 L
.49758 .41627 L
.50242 .41669 L
.50725 .41761 L
.51208 .41909 L
.51503 .42027 L
.51691 .42117 L
.52174 .42395 L
.52344 .42511 L
.52658 .42754 L
.52924 .42994 L
.53141 .43215 L
.53371 .43477 L
.53624 .43804 L
.53734 .4396 L
.54036 .44443 L
.54107 .44569 L
.54295 .44927 L
.54518 .4541 L
.5459 .45582 L
.54715 .45893 L
.54889 .46376 L
Mistroke
.55041 .46859 L
.55073 .46972 L
.5517 .47342 L
.55286 .47826 L
.55392 .48309 L
.55496 .48792 L
.55557 .49225 L
.55567 .49275 L
.55785 .49758 L
.5604 .50028 L
.56523 .5012 L
.57006 .50135 L
.57489 .50158 L
.57973 .50176 L
.58456 .50162 L
.58939 .49924 L
.59075 .49758 L
.59406 .49275 L
.59422 .49248 L
.59719 .48792 L
.59905 .48517 L
.60043 .48309 L
.60371 .47826 L
.60388 .478 L
.60713 .47342 L
.60872 .47126 L
.61071 .46859 L
.61355 .46495 L
.61449 .46376 L
.61838 .45907 L
.6185 .45893 L
.62277 .4541 L
.62321 .45361 L
.62736 .44927 L
.62804 .44858 L
.63227 .44443 L
.63288 .44387 L
.6376 .4396 L
.63771 .43951 L
.64254 .43549 L
.64344 .43477 L
.64737 .43176 L
.64988 .42994 L
.6522 .42832 L
.65704 .42515 L
.65711 .42511 L
.66187 .42226 L
.66546 .42027 L
.6667 .41962 L
.67153 .41725 L
Mistroke
.67554 .41544 L
.67636 .41509 L
.68119 .41314 L
.68603 .41146 L
.68878 .41061 L
.69086 .41001 L
.69569 .40879 L
.70052 .40775 L
.70535 .40688 L
.71019 .40624 L
.71502 .40582 L
.71564 .40578 L
.71985 .40559 L
Mfstroke
1 0 0 r
.40708 .49596 m
.40708 .33436 L
.58484 .33436 L
.58484 .49596 L
.5606 .49596 L
.5606 .3586 L
.43132 .3586 L
.43132 .49596 L
.40708 .49596 L
F
0 0 1 r
.52828 .57676 m
.52822 .57879 L
.52803 .58081 L
.52771 .58282 L
.52727 .5848 L
.5267 .58675 L
.52601 .58866 L
.5252 .59052 L
.52428 .59233 L
.52325 .59408 L
.52211 .59576 L
.52086 .59736 L
.51952 .59889 L
.51808 .60032 L
.51656 .60166 L
.51496 .60291 L
.51328 .60405 L
.51153 .60508 L
.50972 .60601 L
.50786 .60681 L
.50595 .6075 L
.504 .60807 L
.50202 .60851 L
.50001 .60883 L
.49799 .60902 L
.49596 .60908 L
.49393 .60902 L
.49191 .60883 L
.4899 .60851 L
.48792 .60807 L
.48597 .6075 L
.48406 .60681 L
.4822 .60601 L
.48039 .60508 L
.47864 .60405 L
.47696 .60291 L
.47536 .60166 L
.47383 .60032 L
.4724 .59889 L
.47106 .59736 L
.46981 .59576 L
.46867 .59408 L
.46764 .59233 L
.46672 .59052 L
.46591 .58866 L
.46522 .58675 L
.46465 .5848 L
.46421 .58282 L
.46389 .58081 L
.4637 .57879 L
.46364 .57676 L
.4637 .57473 L
.46389 .57271 L
.46421 .57071 L
.46465 .56872 L
.46522 .56677 L
.46591 .56486 L
.46672 .563 L
.46764 .56119 L
.46867 .55944 L
.46981 .55776 L
.47106 .55616 L
.4724 .55464 L
.47383 .5532 L
.47536 .55186 L
.47696 .55061 L
.47864 .54947 L
.48039 .54844 L
.4822 .54752 L
.48406 .54671 L
.48597 .54602 L
.48792 .54546 L
.4899 .54501 L
.49191 .5447 L
.49393 .5445 L
.49596 .54444 L
.49799 .5445 L
.50001 .5447 L
.50202 .54501 L
.504 .54546 L
.50595 .54602 L
.50786 .54671 L
.50972 .54752 L
.51153 .54844 L
.51328 .54947 L
.51496 .55061 L
.51656 .55186 L
.51808 .5532 L
.51952 .55464 L
.52086 .55616 L
.52211 .55776 L
.52325 .55944 L
.52428 .56119 L
.5252 .563 L
.52601 .56486 L
.5267 .56677 L
.52727 .56872 L
.52771 .57071 L
.52803 .57271 L
.52822 .57473 L
.52828 .57676 L
F
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{313.25, 313.25},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40007E0001eB000`400?l00000o`00003oo`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF
0?ooo`00o`3ooooF0?ooo`00_P3ooomD0?l00<<0oooo002n0?oooe@0o`00``3oool00;h0ooooE03o
00330?ooo`00_P3ooomD0?l00<<0oooo002n0?oooe@0o`00``3oool00;h0ooooE03o00330?ooo`00
_P3ooomD0?l00<<0oooo002n0?oooe@0o`00``3oool00;h0ooooE03o00330?ooo`00_P3ooomD0?l0
0<<0oooo002n0?oooe@0o`00``3oool00;h0ooooE03o00330?ooo`00_P3oool<0?l003`0oooo303o
00330?ooo`00_P3oool<0?l003`0oooo303o00330?ooo`00_P3oool<0?l003`0oooo303o00330?oo
o`00_P3oool<0?l003`0oooo303o00330?ooo`00_P3oool<0?l003`0oooo303o00330?ooo`00_P3o
ool<0?l003`0oooo303o00330?ooo`00_P3oool<0?l003`0oooo303o00330?ooo`00_P3oool<0?l0
03`0oooo303o00330?ooo`00_P3oool<0?l003`0oooo303o00330?ooo`00_P3oool<0?l003`0oooo
303o00330?ooo`00_P3oool<0?l003`0oooo303o00330?ooo`00_P3oool<0?l003`0oooo303o0033
0?ooo`00_P3oool<0?l003`0oooo303o00330?ooo`00_P3oool<0?l003`0oooo303o00330?ooo`00
_P3oool<0?l003`0oooo303o00330?ooo`00_P3oool<0?l003`0oooo303o00330?ooo`00_P3oool<
0?l003`0oooo303o00330?ooo`00_P3oool<0?l003`0oooo303o00330?ooo`00_P3oool<0?l003`0
oooo303o00330?ooo`00_P3oool<0?l003`0oooo303o00330?ooo`00_P3oool<0?l003`0oooo303o
00330?ooo`00_P3oool<0?l003`0oooo303o000h0?oooa400000NP3oool007D0oooo5000000e0?oo
o``0o`00?03oool<0?l003<0oooo1@00000A0?ooo`D00000M@3oool00700oooo1@00000D0?ooo`@0
0000<@3oool<0?l003`0oooo303o000`0?ooo`<000006`3oool400000740oooo001^0?ooo`800000
7@3oool2000002l0oooo303o000l0?ooo``0o`00;@3oool300000280oooo0`00001^0?ooo`00JP3o
ool400000240oooo1@00000Z0?ooo``0o`00?03oool<0?l002X0oooo0`00000X0?ooo`<00000J`3o
ool006X0oooo00<000000?ooo`3oool09`3oool2000002P0oooo303o000J0?ooo`/000005`3oool<
0?l002P0oooo0P00000^0?ooo`030000003oool0oooo06P0oooo001W0?ooo`<00000;03oool20000
02H0oooo303o000F0?ooo`@000002`3oool2000001D0oooo303o000V0?ooo`800000<@3oool20000
06P0oooo001U0?ooo`800000<@3oool00`000000oooo0?ooo`0S0?ooo``0o`004`3oool300000140
oooo0P00000C0?ooo``0o`009P3oool00`000000oooo0?ooo`0b0?ooo`<00000I@3oool006<0oooo
0P00000d0?ooo`8000008`3oool<0?l00180oooo00<000000?ooo`3oool0503oool200000140oooo
303o000T0?ooo`800000>03oool2000006<0oooo001Q0?ooo`800000>03oool200000240oooo303o
000A0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool03P3oool<0?l00240oooo0`00
000l0?ooo`030000003oool0oooo0600oooo001P0?ooo`030000003oool0oooo03X0oooo0P00000O
0?ooo``0o`00403oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00d0oooo303o000O
0?ooo`800000@03oool00`000000oooo0?ooo`1O0?ooo`00GP3oool2000003l0oooo00<000000?oo
o`3oool0703oool<0?l000h0oooo0P00000M0?ooo`8000003@3oool<0?l001h0oooo00<000000?oo
o`3oool0@@3oool2000005l0oooo001L0?ooo`800000@P3oool2000001`0oooo303o000=0?ooo`03
0000003oool0oooo01l0oooo00<000000?ooo`3oool02P3oool<0?l001d0oooo00<000000?ooo`3o
ool0A03oool2000005d0oooo001K0?ooo`030000003oool0oooo04@0oooo0P00000J0?ooo``0o`00
303oool200000280oooo00<000000?ooo`3oool02@3oool<0?l001/0oooo0P0000190?ooo`030000
003oool0oooo05X0oooo001J0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool05`3o
ool<0?l000`0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`090?ooo``0o`006`3o
ool00`000000oooo0?ooo`190?ooo`030000003oool0oooo05T0oooo001I0?ooo`030000003oool0
oooo04T0oooo00<000000?ooo`3oool05P3oool<0?l000/0oooo00<000000?ooo`3oool08`3oool0
0`000000oooo0?ooo`080?ooo``0o`006P3oool00`000000oooo0?ooo`1;0?ooo`030000003oool0
oooo05P0oooo001H0?ooo`030000003oool0oooo04X0oooo00<000000?ooo`3oool05P3oool<0?l0
00X0oooo0P00000U0?ooo`030000003oool0oooo00P0oooo303o000H0?ooo`800000C`3oool00`00
0000oooo0?ooo`1G0?ooo`00E`3oool00`000000oooo0?ooo`1<0?ooo`8000005P3oool<0?l000T0
oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`070?ooo``0o`005`3oool00`000000
oooo0?ooo`1@0?ooo`030000003oool0oooo05H0oooo001E0?ooo`800000D@3oool00`000000oooo
0?ooo`0C0?ooo``0o`002@3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo00H0oooo
303o000F0?ooo`030000003oool0oooo0580oooo00<000000?ooo`3oool0E@3oool005@0oooo00<0
00000?ooo`3oool0DP3oool00`000000oooo0?ooo`0B0?ooo``0o`00203oool00`000000oooo0?oo
o`0X0?ooo`030000003oool0oooo00H0oooo303o000E0?ooo`030000003oool0oooo05@0oooo00<0
00000?ooo`3oool0E03oool005<0oooo00<000000?ooo`3oool0E03oool00`000000oooo0?ooo`0A
0?ooo``0o`00203oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo00D0oooo303o000D
0?ooo`030000003oool0oooo05H0oooo00<000000?ooo`3oool0D`3oool00580oooo00<000000?oo
o`3oool0EP3oool00`000000oooo0?ooo`0@0?ooo``0o`00203oool00`000000oooo0?ooo`0Y0?oo
o`030000003oool0oooo00D0oooo303o000C0?ooo`030000003oool0oooo05P0oooo0P00001C0?oo
o`00D@3oool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo00l0oooo303o00070?ooo`03
0000003oool0oooo02/0oooo00<000000?ooo`3oool0103oool<0?l00180oooo00<000000?ooo`3o
ool0FP3oool00`000000oooo0?ooo`1A0?ooo`00D@3oool00`000000oooo0?ooo`1H0?ooo`030000
003oool0oooo00l0oooo303o00070?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0
103oool<0?l00180oooo00<000000?ooo`3oool0F`3oool00`000000oooo0?ooo`1@0?ooo`00D03o
ool00`000000oooo0?ooo`1J0?ooo`8000003`3oool<0?l000H0oooo00<000000?ooo`3oool0;03o
ool00`000000oooo0?ooo`040?ooo``0o`004@3oool00`000000oooo0?ooo`1M0?ooo`030000003o
ool0oooo04l0oooo001?0?ooo`030000003oool0oooo05`0oooo00<000000?ooo`3oool03@3oool<
0?l000H0oooo00<000000?ooo`3oool04`3oool6000001@0oooo00<000000?ooo`3oool00`3oool<
0?l000l0oooo0P00001P0?ooo`030000003oool0oooo04l0oooo001>0?ooo`030000003oool0oooo
05h0oooo00<000000?ooo`3oool0303oool<0?l000D0oooo00<000000?ooo`3oool0403oool40000
00H0oooo0`00000A0?ooo`030000003oool0oooo00<0oooo303o000>0?ooo`030000003oool0oooo
0640oooo00<000000?ooo`3oool0CP3oool004d0oooo00<000000?ooo`3oool0H03oool00`000000
oooo0?ooo`0;0?ooo``0o`001@3oool00`000000oooo0?ooo`0=0?ooo`<000003@3oool3000000l0
oooo00<000000?ooo`3oool00P3oool<0?l000h0oooo00<000000?ooo`3oool0HP3oool00`000000
oooo0?ooo`1=0?ooo`00C03oool00`000000oooo0?ooo`1R0?ooo`030000003oool0oooo00X0oooo
303o00050?ooo`030000003oool0oooo00/0oooo0P00000C0?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool00P3oool<0?l000d0oooo00<000000?ooo`3oool0I03oool00`000000oooo
0?ooo`1<0?ooo`00B`3oool00`000000oooo0?ooo`1T0?ooo`030000003oool0oooo00T0oooo303o
00050?ooo`030000003oool0oooo00T0oooo0P00000F0?ooo`<00000303oool010000000oooo0?oo
o`3oool<0?l000`0oooo00<000000?ooo`3oool0IP3oool00`000000oooo0?ooo`1;0?ooo`00BP3o
ool00`000000oooo0?ooo`1U0?ooo`030000003oool0oooo00T0oooo303o00050?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`090?ooo`040000003o
ool0oooo0?ooo``0o`002`3oool00`000000oooo0?ooo`1W0?ooo`030000003oool0oooo04/0oooo
001:0?ooo`030000003oool0oooo06H0oooo00<000000?ooo`3oool0203oool<0?l000@0oooo00<0
00000?ooo`3oool0203oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00P0oooo00@0
00000?ooo`3oool0oooo303o000;0?ooo`030000003oool0oooo06P0oooo00<000000?ooo`3oool0
BP3oool004X0oooo00<000000?ooo`3oool0I`3oool00`000000oooo0?ooo`070?ooo``0o`00103o
ool00`000000oooo0?ooo`070?ooo`030000003oool0oooo01d0oooo0P0000080?ooo`040000003o
ool0oooo0?ooo``0o`002P3oool00`000000oooo0?ooo`1Z0?ooo`030000003oool0oooo04T0oooo
00190?ooo`030000003oool0oooo06T0oooo00<000000?ooo`3oool01P3oool<0?l000<0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo00H0oooo00@0
00000?ooo`3oool0oooo303o00090?ooo`030000003oool0oooo06`0oooo00<000000?ooo`3oool0
B03oool004P0oooo00<000000?ooo`3oool0JP3oool00`000000oooo0?ooo`060?ooo``0o`000`3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`0<0?l000T0oooo00<000000?ooo`3oool0K03oool00`000000oooo0?oo
o`180?ooo`00A`3oool00`000000oooo0?ooo`1/0?ooo`030000003oool0oooo00D0oooo303o0003
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool03P3oool400000140oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`0<0?l000P0oooo00<000000?ooo`3oool0K@3oool0
0`000000oooo0?ooo`180?ooo`00A`3oool00`000000oooo0?ooo`1]0?ooo`030000003oool0oooo
00@0oooo303o00030?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool02`3oool40000
00@0oooo1@00000=0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0303o00070?oo
o`030000003oool0oooo06l0oooo00<000000?ooo`3oool0A`3oool004H0oooo00<000000?ooo`3o
ool0K`3oool00`000000oooo0?ooo`030?ooo``0o`000P3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00T0oooo0`00000=0?ooo`8000002`3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00`0o`001P3oool00`000000oooo0?ooo`1a0?ooo`030000003oool0oooo04H0
oooo00160?ooo`030000003oool0oooo0700oooo00<000000?ooo`3oool00P3oool<0?l00080oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`070?ooo`<000004P3oool4000000P0oooo
0P0000050?ooo`030000003oool0o`0000/0o`001P3oool00`000000oooo0?ooo`1b0?ooo`030000
003oool0oooo04D0oooo00150?ooo`030000003oool0oooo0740oooo00<000000?ooo`3oool00P3o
ool<0?l00080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`060?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0o`0000/0o`001@3oool00`000000oooo0?ooo`1c0?ooo`030000003oool0oooo04D0oooo0015
0?ooo`030000003oool0oooo0740oooo00<000000?ooo`3oool00P3oool<0?l00080oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo01P0oooo0P000008
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o00002`3o00050?ooo`030000003oool0
oooo07@0oooo00<000000?ooo`3oool0A03oool004@0oooo00<000000?ooo`3oool0L`3oool01000
0000oooo0?ooo`3oool<0?l00080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool01@3oool2000000<0oooo00<00000
0?ooo`3o00002`3o00040?ooo`030000003oool0oooo07D0oooo00<000000?ooo`3oool0A03oool0
04<0oooo00<000000?ooo`3oool0M03oool010000000oooo0?ooo`3oool<0?l000030?ooo`000000
oooo00<0oooo00<000000?ooo`3oool0103oool200000100oooo00<000000?ooo`3oool0303oool2
000000H0oooo00H000000?ooo`3oool0oooo0000003oool<0?l000<0oooo00<000000?ooo`3oool0
MP3oool00`000000oooo0?ooo`140?ooo`00@`3oool00`000000oooo0?ooo`1e0?ooo`8000003P3o
ool01@000000oooo0?ooo`3oool0000000H0oooo00<000000?ooo`3oool02@3oool7000000030?oo
o`000000000000<000002`3oool2000000D0oooo00D000000?ooo`3oool0oooo0000000>0?ooo`03
0000003oool0oooo07P0oooo00<000000?ooo`3oool0@`3oool00480oooo00<000000?ooo`3oool0
N03oool00`000000oooo0?ooo`090?ooo`8000000`3oool00`000000oooo0?ooo`030?ooo`800000
2@3oool3000000d0oooo100000090?ooo`030000003oool0oooo00<0oooo00@000000?ooo`3oool0
oooo0P00000<0?ooo`030000003oool0oooo07P0oooo00<000000?ooo`3oool0@`3oool00480oooo
00<000000?ooo`3oool0N@3oool;000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`060?ooo`<00000503oool2000000P0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`020?ooo``00000O03oool00`000000oooo0?ooo`120?ooo`00@@3oool00`000000oooo0?oo
o`270?ooo`8000001@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo08T0oooo00<0
00000?ooo`3oool0@P3oool00440oooo00<000000?ooo`3oool0Q03oool3000000D0oooo0P000006
0?ooo`<000006`3oool3000000H0oooo0P0000040?ooo`@00000R03oool00`000000oooo0?ooo`11
0?ooo`00@@3oool00`000000oooo0?ooo`230?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`0:0?ooo`P000003@3oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo00D0oooo0`0000250?ooo`030000003oool0oooo0440oooo00100?ooo`03
0000003oool0oooo05l0oooo600000060?ooo`L000001`3oool2000000H0oooo00<000000?ooo`3o
ool01`3oool4000000P0oooo1@0000090?ooo`8000001P3oool00`000000oooo0?ooo`070?ooo`D0
00001@3ooolJ00000680oooo00<000000?ooo`3oool0@03oool00400oooo00<000000?ooo`3oool0
F`3oool4000001P0oooo1P00000=0?ooo`030000003oool0oooo00@0oooo0P0000070?ooo`<00000
4@3oool2000000T0oooo00<000000?ooo`3oool0103oool2000000`0oooo1@00000J0?ooo`D00000
G@3oool00`000000oooo0?ooo`100?ooo`00?`3oool00`000000oooo0?ooo`1J0?ooo`800000;@3o
ool2000000D0oooo0P0000070?ooo`8000005P3oool2000000P0oooo00<000000?ooo`3oool01@3o
ool2000002h0oooo0P00001L0?ooo`030000003oool0oooo03l0oooo000o0?ooo`030000003oool0
oooo05D0oooo1@00000]0?ooo`8000001P3oool00`000000oooo0?ooo`050?ooo`8000003P3oool0
0`000000oooo0?ooo`090?ooo`<000001P3oool2000000L0oooo00<000000?ooo`3oool0;@3oool2
000005X0oooo00<000000?ooo`3oool0?`3oool003h0oooo00<000000?ooo`3oool0E03oool20000
0300oooo0P0000070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01`3oool70000
00030?ooo`000000000000<000002@3oool2000000H0oooo0P0000060?ooo`800000;`3oool30000
05L0oooo00<000000?ooo`3oool0?`3oool003h0oooo00<000000?ooo`3oool0DP3oool2000002l0
oooo0`0000070?ooo`8000001P3oool2000000L0oooo0`00000=0?ooo`@000001`3oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo1000000^0?ooo`800000EP3oool00`000000
oooo0?ooo`0n0?ooo`00?P3oool00`000000oooo0?ooo`1A0?ooo`800000;P3oool2000000P0oooo
0P0000060?ooo`8000001`3oool2000001@0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`050?ooo`8000002@3oool3000002d0oooo0P00001D0?ooo`030000003oool0oooo03h0oooo
000m0?ooo`030000003oool0oooo0500oooo0P00000^0?ooo`8000002@3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00D0oooo0P00000:0?ooo`<000002P3oool00`000000oooo0?oo
o`050?ooo`8000001`3oool00`000000oooo0?ooo`090?ooo`800000;@3oool2000005<0oooo00<0
00000?ooo`3oool0?@3oool003d0oooo00<000000?ooo`3oool0CP3oool2000002/0oooo1@000009
0?ooo`8000001`3oool00`000000oooo0?ooo`040?ooo`8000001`3oool5000000<0oooo1@000006
0?ooo`<000001P3oool00`000000oooo0?ooo`050?ooo`8000002P3oool3000002`0oooo00<00000
0?ooo`3oool0D@3oool00`000000oooo0?ooo`0m0?ooo`00?@3oool00`000000oooo0?ooo`1=0?oo
o`030000003oool0oooo02T0oooo0P00000=0?ooo`030000003oool0oooo00D0oooo0P0000060?oo
o`030000003oool0oooo00D0oooo0P00000=0?ooo`8000001`3oool2000000D0oooo00<000000?oo
o`3oool01P3oool2000000/0oooo0P00000[0?ooo`800000D@3oool00`000000oooo0?ooo`0m0?oo
o`00?@3oool00`000000oooo0?ooo`1<0?ooo`030000003oool0oooo02P0oooo0P00000>0?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool0103oool2000000D0oooo0`0000090?ooo`03
0000003oool0oooo00D0oooo0P0000070?ooo`030000003oool0oooo00<0oooo0P0000070?ooo`03
0000003oool0oooo00/0oooo0`00000Z0?ooo`800000D03oool00`000000oooo0?ooo`0l0?ooo`00
?03oool00`000000oooo0?ooo`1<0?ooo`030000003oool0oooo02L0oooo0P00000>0?ooo`800000
1`3oool00`000000oooo0?ooo`030?ooo`8000001@3oool2000000L0oooo1@0000000`3oool00000
00000002000000D0oooo0P0000060?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
1@3oool2000000h0oooo0P00000Z0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0
?03oool003`0oooo00<000000?ooo`3oool0BP3oool2000002L0oooo0`00000?0?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00@0oooo0`00000:0?ooo`<00000103oool2000000D0oooo0P0000050?ooo`8000001`3o
ool2000000h0oooo0P00000Y0?ooo`030000003oool0oooo04`0oooo00<000000?ooo`3oool0?03o
ool003/0oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0
oooo00h0oooo0P0000070?ooo`8000001@3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo00<0oooo0P0000050?ooo`@000001`3oool2000000@0oooo0P0000040?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0=0?ooo`800000:03oool0
0`000000oooo0?ooo`1;0?ooo`030000003oool0oooo03`0oooo000k0?ooo`030000003oool0oooo
04T0oooo00<000000?ooo`3oool09@3oool200000100oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo0P0000040?ooo`800000103oool30000
00@0oooo100000050?ooo`800000103oool00`000000oooo0?ooo`020?ooo`8000001P3oool00`00
0000oooo0?ooo`060?ooo`8000003`3oool2000002L0oooo0P00001<0?ooo`030000003oool0oooo
03/0oooo000k0?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3oool09@3oool00`000000
oooo0?ooo`0?0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`030?ooo`030000003oool0oooo0080oooo0P0000050?ooo`030000003oool0oooo0080
oooo0`0000030?ooo`@000000`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00d0
oooo0P00000W0?ooo`030000003oool0oooo04X0oooo00<000000?ooo`3oool0>`3oool003X0oooo
00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo00l0oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`800000103oool01@000000oooo
0?ooo`3oool0000000H0oooo00@000000?ooo`3oool0oooo0P0000040?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool01@3oool2000000P0oooo00<000000?ooo`3oool03P3oool20000
02H0oooo00<000000?ooo`3oool0B@3oool00`000000oooo0?ooo`0k0?ooo`00>P3oool00`000000
oooo0?ooo`180?ooo`030000003oool0oooo02<0oooo0P00000A0?ooo`030000003oool0oooo00H0
oooo0P0000060?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool200000080
oooo0`0000020?ooo`03000000000?l0003o00P0003o0P0000030?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`8000001P3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool09@3oool00`000000
oooo0?ooo`190?ooo`030000003oool0oooo03X0oooo000j0?ooo`030000003oool0oooo04L0oooo
00<000000?ooo`3oool08P3oool200000140oooo0P0000080?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`050000003oool0oooo0?ooo`00
00000`3oool00`000000003o0000o`0<0000o`8000000P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00H0oooo0P00000@0?ooo`8000009P3oool00`000000oooo0?ooo`180?ooo`03
0000003oool0oooo03X0oooo000j0?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool0
8`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool0
oooo0P0000020?oooa80003o00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo00l0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`170?oo
o`030000003oool0oooo03X0oooo000i0?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3o
ool08P3oool200000140oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool010000000oooo0?ooo`0000030?oo
oa@0003o00@000000?ooo`3oool00000103oool01@000000oooo0?ooo`3oool0000000D0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00l0oooo00<0
00000?ooo`3oool0903oool00`000000oooo0?ooo`160?ooo`030000003oool0oooo03X0oooo000i
0?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool000000`3ooolF0000o`030000003o
ool0oooo008000000`3oool01@000000oooo0?ooo`3oool0000000D0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00l0oooo0P00000U0?ooo`030000
003oool0oooo04H0oooo00<000000?ooo`3oool0>@3oool003T0oooo00<000000?ooo`3oool0A03o
ool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3o
ool01@000000oooo0?ooo`3oool000000080oooo60000?l010000000oooo0?ooo`0000030?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`050?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0R0?ooo`03
0000003oool0oooo04H0oooo00<000000?ooo`3oool0>@3oool003T0oooo00<000000?ooo`3oool0
@`3oool00`000000oooo0?ooo`0R0?ooo`8000004@3oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`050000
003oool0oooo0?ooo`0000000P3ooolJ0000o`040000003oool0oooo000000<0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`020?ooo`8000001P3oool00`000000oooo0?ooo`070?oo
o`8000004P3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo04H0oooo00<000000?oo
o`3oool0>03oool003T0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0R0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool01@000000oooo0?ooo`3oool00000
0080oooo6P000?l010000000oooo0?ooo`0000030?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo04D0oooo00<00000
0?ooo`3oool0>03oool003P0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`0R0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool01@000000oooo0?ooo`3oool0
00000080oooo70000?l010000000oooo0?ooo`0000030?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo04D0oooo00<0
00000?ooo`3oool0>03oool003P0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`0Q
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`04
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool01@000000oooo0?ooo`3o
ool000000080oooo70000?l010000000oooo0?ooo`0000030?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00P0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo04@0oooo
00<000000?ooo`3oool0>03oool003P0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0203oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo00@0oooo00D000000?ooo`3oool0oooo000000030?ooo`030000
003oool0oooo01h0003o0P3oool01@000000oooo0?ooo`3oool0000000@0oooo00<000000?ooo`3o
ool0103oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3o
ool04@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3o
ool0>03oool003P0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`0R0?ooo`030000
003oool0oooo0100oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool01@000000oooo0?ooo`3oool000000080
oooo7P000?l010000000oooo0?ooo`0000030?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool00`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00P0oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo04@0oooo00<000000?oo
o`3oool0=`3oool003L0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00<0oooo00D000000?ooo`3oool0oooo000000030?ooo`040000003oool0oooo
000001h0003o00@000000?ooo`3oool000000`3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`090?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`140?ooo`030000003o
ool0oooo03L0oooo000g0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`0@0?ooo`8000002P3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo00<0oooo00D000000?ooo`3oool0oooo000000030?ooo`040000003oool0oooo000001h0003o
00D0oooo0000003oool0oooo000000030?ooo`050000003oool0oooo0?ooo`0000001P3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool0=`3oool003L0
oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool00P3oool01@000000oooo0?ooo`3oool000000080oooo00<000000000
o`000?l070000?l01@3oool000000?ooo`3oool0000000<0oooo00D000000?ooo`3oool0oooo0000
00060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?oo
o`0g0?ooo`00=`3oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo0240oooo00<00000
0?ooo`3oool0403oool00`000000oooo0?ooo`090?ooo`8000001`3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0080oooo00D000000?ooo`3oool0oooo000000020?ooo`0300000000
0?l0003o01`0003o00D0oooo0000003oool0oooo000000030?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo
00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0480oooo
00<000000?ooo`3oool0=`3oool003L0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool02@3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool01@000000oooo0?oo
o`3oool000000080oooo00<000000000o`000?l070000?l01@3oool000000?ooo`3oool0000000<0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0240
oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0g0?ooo`00=P3oool00`000000oooo
0?ooo`100?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`020?ooo`050000003oool0oooo0?ooo`0000000`3ooolN0000o`050000003oool0oooo0?oo
o`0000000`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0140oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo03H0oooo000f0?ooo`03
0000003oool0oooo0400oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0080oooo00D000000?ooo`3oool0oooo000000030?oooah0003o00D000000?oo
o`3oool0oooo000000030?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0403oool2
00000280oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0f0?ooo`00=P3oool00`00
0000oooo0?ooo`0o0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`020?ooo`050000003oool0oooo0?ooo`0000000`3oool00`000000003o0000o`0J
0000o`050000003oool0oooo0?ooo`000000103oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`110?ooo`030000003o
ool0oooo03H0oooo000f0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool0
10000000oooo0?ooo`3ooolL0000o`050000003oool0oooo0?ooo`000000103oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`090?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`110?ooo`030000003oool0oooo03H0oooo000f0?ooo`030000003oool0oooo03l0oooo00<0
00000?ooo`3oool0803oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool00P3oool010000000oooo0?ooo`3ooolL0000o`<0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
1P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
8@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo03H0oooo000f0?ooo`030000003o
ool0oooo03l0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool00P3oool010000000oooo0?ooo`3ooolJ0000o`050000
003oool0oooo0?ooo`000000103oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo03H0
oooo000f0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0803oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3oool01@000000oooo
0?ooo`3oool0000001P0003o00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0Q0?ooo`03
0000003oool0oooo0400oooo00<000000?ooo`3oool0=P3oool003H0oooo00<000000?ooo`3oool0
?P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
2`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`030?ooo`040000003oool0oooo0?oooaP0003o00D000000?ooo`3o
ool0oooo000000040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`0Q0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool0=@3oool003H0
oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0140
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080
oooo00<000000000o`000?l04P000?l00`3oool000000?ooo`030?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0240
oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0e0?ooo`00=@3oool00`000000oooo
0?ooo`0n0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`0;0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P3ooolD0000o`030000
003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo0140oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`100?ooo`030000
003oool0oooo03D0oooo000e0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`020?ooo`03000000000?l0003o00h0003o00<0oooo0000003oool00`3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01@3o
ool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool04P3o
ool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0=@3o
ool003D0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0
oooo0180oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo00@0oooo00<000000000o`000?l02P000?l3000000@0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0203oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`0o0?ooo`030000003oool0oooo03D0oooo000e0?ooo`030000003oool0oooo
03h0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool00`3oool2000000H0oooo00<000000000o`000?l01P000?l00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0?`3oool00`00
0000oooo0?ooo`0e0?ooo`00=@3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo0240
oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool01@3oool2000000@0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool04P3oool00`000000
oooo0?ooo`0Q0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0=@3oool003D0oooo
00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0180oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`040?ooo`8000001`3oool4000000H0oooo
0P0000060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0=@3oool003D0oooo00<0
00000?ooo`3oool0?@3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00`0oooo0P00
00080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0=@3oool003D0oooo00<00000
0?ooo`3oool0?@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo01<0oooo00<00000
0?ooo`3oool02`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`8000002P3oool2000000T0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00P0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0240oooo00<00000
0?ooo`3oool0?`3oool00`000000oooo0?ooo`0d0?ooo`00=@3oool00`000000oooo0?ooo`0m0?oo
o`030000003oool0oooo0200oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo00P0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo00L0oooo2P0000090?ooo`8000001`3oool2000000P0oooo00<000000?oo
o`3oool0203oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0180oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo03@0oooo000e0?ooo`03
0000003oool0oooo03`0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0oooo00/0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo00H0oooo0P00000I0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool0203oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3o
ool04P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3o
ool0=03oool003D0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo01<0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`080?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0E0?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0Q0?ooo`030000
003oool0oooo03l0oooo00<000000?ooo`3oool0=03oool003@0oooo00<000000?ooo`3oool0?@3o
ool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool01`3o
ool2000001<0oooo0P0000090?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo03@0oooo000d0?ooo`030000003oool0
oooo03d0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo00P0oooo0`00000=0?ooo`<000002P3oool00`000000oooo0?ooo`080?ooo`030000003oool0
oooo00T0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0C0?ooo`030000003oool0
oooo0280oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0d0?ooo`00=03oool00`00
0000oooo0?ooo`0m0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool04`3oool00`00
0000oooo0?ooo`0<0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool01`3oool20000
00`0oooo3@00000<0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool08P3oool00`00
0000oooo0?ooo`0n0?ooo`030000003oool0oooo03@0oooo000d0?ooo`030000003oool0oooo03d0
oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00d0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`080?ooo`8000008P3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool03@3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0?P3oool00`000000
oooo0?ooo`0d0?ooo`00=03oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo0240oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo00T0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo03@0oooo
000d0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?oo
o`0:0?ooo`8000007@3oool2000000/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?oo
o`0n0?ooo`030000003oool0oooo03@0oooo000d0?ooo`030000003oool0oooo03d0oooo00<00000
0?ooo`3oool08@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`8000006P3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0C
0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0d
0?ooo`00=03oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo0240oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00/0oooo0P00000=0?oo
o`8000005P3oool2000000d0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0n0?oo
o`030000003oool0oooo03@0oooo000d0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool02`3oool00`000000oooo0?ooo`0=0?ooo`<000003P3oool5000000h0oooo00<000000?ooo`3o
ool02`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool08P3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo03@0oooo000d0?ooo`030000
003oool0oooo03d0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0?0?ooo`L00000
0`3oool400000180oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0>0?ooo`030000
003oool0oooo01D0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0n0?ooo`030000
003oool0oooo03@0oooo000d0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool08@3o
ool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0303o
ool2000001H0oooo0`00000D0?ooo`8000003P3oool00`000000oooo0?ooo`0>0?ooo`030000003o
ool0oooo01D0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0n0?ooo`030000003o
ool0oooo03@0oooo000d0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03`3oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0?P3oool0
0`000000oooo0?ooo`0d0?ooo`00=03oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo
0280oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0?0?ooo`8000003`3oool00`00
0000oooo0?ooo`0W0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`0E0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0?P3oool00`00
0000oooo0?ooo`0d0?ooo`00=03oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo0280
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00h0
oooo0P00000V0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`0F0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0?P3oool00`000000
oooo0?ooo`0d0?ooo`00=03oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo0280oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0oooo
0P00000S0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0?P3oool00`000000oooo
0?ooo`0d0?ooo`00=03oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo0280oooo00<0
00000?ooo`3oool05@3oool00`000000oooo0?ooo`0@0?ooo`8000004@3oool00`000000oooo0?oo
o`0N0?ooo`<000004@3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01D0oooo00<0
00000?ooo`3oool08`3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo03@0oooo000d
0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0E
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool2000001`0oooo0P00000C
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0S
0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0=03oool003@0oooo00<000000?oo
o`3oool0?@3oool00`000000oooo0?ooo`0R0?ooo`8000005`3oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo0140oooo1@00000E0?ooo`800000503oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo01H0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0n0?oo
o`030000003oool0oooo03@0oooo000d0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3o
ool08`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool05@3oool4000000`0oooo1@00000E0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool05P3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3o
ool0=03oool003@0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0S0?ooo`030000
003oool0oooo01H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0H0?ooo``00000
6@3oool2000001<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0T0?ooo`030000
003oool0oooo03h0oooo00<000000?ooo`3oool0=03oool003@0oooo00<000000?ooo`3oool0?@3o
ool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool04P3o
ool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool05`3o
ool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0=03o
ool003@0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0
oooo01L0oooo00<000000?ooo`3oool04P3oool2000003T0oooo0P00000D0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0
oooo03@0oooo000d0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool0903oool00`00
0000oooo0?ooo`0G0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0=P3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool08`3oool00`00
0000oooo0?ooo`0o0?ooo`030000003oool0oooo03@0oooo000e0?ooo`030000003oool0oooo03d0
oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01<0
oooo0P00000e0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool05`3oool00`000000
oooo0?ooo`0T0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0=03oool003D0oooo
00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo01P0oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0`0?ooo`8000005`3oool00`000000oooo
0?ooo`0H0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0?`3oool00`000000oooo
0?ooo`0d0?ooo`00=@3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo02@0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0E0?ooo`800000;P3oool2000001P0oooo00<0
00000?ooo`3oool0603oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo0400oooo00<0
00000?ooo`3oool0=03oool003D0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0T
0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool05P3oool2000002/0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo02@0oooo00<00000
0?ooo`3oool0@03oool00`000000oooo0?ooo`0d0?ooo`00=@3oool00`000000oooo0?ooo`0m0?oo
o`030000003oool0oooo02@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?oo
o`800000:03oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool0903oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo03D0oooo000e0?ooo`03
0000003oool0oooo03h0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0I0?ooo`80
00006P3oool300000280oooo0`00000J0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool09@3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo03D0oooo000e0?ooo`030000
003oool0oooo03h0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo01/0oooo0P00000L0?ooo`@00000703oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo02D0oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0e0?ooo`00=@3o
ool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0L0?ooo`800000603oool2000001l0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool0=@3o
ool003D0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool07@3oool7000000X0oooo1`00000P0?ooo`800000703oool0
0`000000oooo0?ooo`0U0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool0=@3oool0
03D0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo
01`0oooo00<000000?ooo`3oool08`3oool:000002H0oooo00<000000?ooo`3oool06`3oool00`00
0000oooo0?ooo`0V0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool0=@3oool003D0
oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo01`0
oooo00<000000?ooo`3oool0D@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo02H0
oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`0e0?ooo`00=@3oool00`000000oooo
0?ooo`0o0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`1?0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool09P3oool00`000000oooo
0?ooo`110?ooo`030000003oool0oooo03D0oooo000f0?ooo`030000003oool0oooo03l0oooo00<0
00000?ooo`3oool09P3oool00`000000oooo0?ooo`0L0?ooo`800000C@3oool2000001l0oooo00<0
00000?ooo`3oool09P3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo03D0oooo000f
0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0M
0?ooo`030000003oool0oooo04X0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0V
0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool0=P3oool003H0oooo00<000000?oo
o`3oool0?`3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo01d0oooo0P0000190?oo
o`030000003oool0oooo01h0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`110?oo
o`030000003oool0oooo03H0oooo000f0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3o
ool0:03oool00`000000oooo0?ooo`0N0?ooo`800000AP3oool00`000000oooo0?ooo`0N0?ooo`03
0000003oool0oooo02P0oooo00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`0f0?ooo`00
=P3oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool0
803oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0
:03oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo03H0oooo000f0?ooo`030000003o
ool0oooo0400oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`0P0?ooo`030000003o
ool0oooo03l0oooo0P00000R0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0@@3o
ool00`000000oooo0?ooo`0f0?ooo`00=P3oool00`000000oooo0?ooo`110?ooo`030000003oool0
oooo02P0oooo00<000000?ooo`3oool0803oool2000003d0oooo0P00000S0?ooo`030000003oool0
oooo02P0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0f0?ooo`00=`3oool00`00
0000oooo0?ooo`100?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool08P3oool30000
03T0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo
0480oooo00<000000?ooo`3oool0=P3oool003L0oooo00<000000?ooo`3oool0@03oool00`000000
oooo0?ooo`0Y0?ooo`030000003oool0oooo02@0oooo0P00000e0?ooo`8000009@3oool00`000000
oooo0?ooo`0X0?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool0=`3oool003L0oooo
00<000000?ooo`3oool0@@3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo02D0oooo
0P00000`0?ooo`<000009@3oool2000002/0oooo00<000000?ooo`3oool0@P3oool00`000000oooo
0?ooo`0g0?ooo`00=`3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo02X0oooo00<0
00000?ooo`3oool09P3oool2000002`0oooo0P00000W0?ooo`030000003oool0oooo02/0oooo00<0
00000?ooo`3oool0@P3oool00`000000oooo0?ooo`0g0?ooo`00=`3oool00`000000oooo0?ooo`12
0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool09`3oool4000002@0oooo1000000X
0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`0g
0?ooo`00=`3oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo02X0oooo00<000000?oo
o`3oool0:`3oool3000001h0oooo0`00000[0?ooo`030000003oool0oooo02`0oooo00<000000?oo
o`3oool0@`3oool00`000000oooo0?ooo`0g0?ooo`00>03oool00`000000oooo0?ooo`110?ooo`03
0000003oool0oooo02/0oooo00<000000?ooo`3oool0;@3oool5000001<0oooo1P00000^0?ooo`03
0000003oool0oooo02/0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`0h0?ooo`00
>03oool00`000000oooo0?ooo`120?ooo`030000003oool0oooo02/0oooo0P00000b0?oooa<00000
<`3oool00`000000oooo0?ooo`0[0?ooo`800000A@3oool00`000000oooo0?ooo`0h0?ooo`00>03o
ool00`000000oooo0?ooo`120?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0M@3o
ool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool0>03o
ool003P0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`0/0?ooo`800000M03oool0
0`000000oooo0?ooo`0/0?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool0>03oool0
03P0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo
06l0oooo0P00000^0?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool0>03oool003T0
oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo06d0
oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo03T0
oooo000i0?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool0;P3oool00`000000oooo
0?ooo`1[0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0AP3oool00`000000oooo
0?ooo`0i0?ooo`00>@3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo02h0oooo0P00
001Y0?ooo`800000<@3oool00`000000oooo0?ooo`160?ooo`030000003oool0oooo03T0oooo000i
0?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`1V
0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0i
0?ooo`00>P3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo0300oooo00<000000?oo
o`3oool0I03oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo04H0oooo00<000000?oo
o`3oool0>P3oool003X0oooo00<000000?ooo`3oool0A@3oool200000380oooo0P00001R0?ooo`80
0000<P3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo03X0oooo000j0?ooo`030000
003oool0oooo04H0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`1M0?ooo`800000
<`3oool00`000000oooo0?ooo`180?ooo`030000003oool0oooo03X0oooo000k0?ooo`030000003o
ool0oooo04H0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`1K0?ooo`030000003o
ool0oooo0380oooo0P00001:0?ooo`030000003oool0oooo03X0oooo000k0?ooo`030000003oool0
oooo04H0oooo00<000000?ooo`3oool0<`3oool2000005X0oooo00<000000?ooo`3oool0<P3oool0
0`000000oooo0?ooo`190?ooo`030000003oool0oooo03/0oooo000k0?ooo`030000003oool0oooo
04L0oooo0P00000e0?ooo`800000E@3oool3000003D0oooo00<000000?ooo`3oool0B@3oool00`00
0000oooo0?ooo`0k0?ooo`00>`3oool00`000000oooo0?ooo`180?ooo`030000003oool0oooo03D0
oooo00<000000?ooo`3oool0D03oool2000003L0oooo00<000000?ooo`3oool0BP3oool00`000000
oooo0?ooo`0k0?ooo`00?03oool00`000000oooo0?ooo`180?ooo`030000003oool0oooo03D0oooo
0P00001>0?ooo`800000>03oool00`000000oooo0?ooo`1:0?ooo`030000003oool0oooo03`0oooo
000l0?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3oool0=`3oool3000004T0oooo0`00
000i0?ooo`030000003oool0oooo04X0oooo00<000000?ooo`3oool0?03oool003d0oooo00<00000
0?ooo`3oool0B03oool00`000000oooo0?ooo`0i0?ooo`800000A@3oool2000003/0oooo00<00000
0?ooo`3oool0BP3oool00`000000oooo0?ooo`0m0?ooo`00?@3oool00`000000oooo0?ooo`190?oo
o`800000>`3oool200000400oooo0`00000l0?ooo`030000003oool0oooo04/0oooo00<000000?oo
o`3oool0?@3oool003d0oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`0k0?ooo`@0
0000>P3oool2000003h0oooo00<000000?ooo`3oool0C03oool00`000000oooo0?ooo`0m0?ooo`00
?P3oool00`000000oooo0?ooo`1:0?ooo`030000003oool0oooo03h0oooo0`00000b0?ooo`D00000
?`3oool2000004d0oooo00<000000?ooo`3oool0?P3oool003h0oooo00<000000?ooo`3oool0B`3o
ool00`000000oooo0?ooo`100?ooo`@00000;03oool2000004<0oooo00<000000?ooo`3oool0C@3o
ool00`000000oooo0?ooo`0n0?ooo`00?P3oool00`000000oooo0?ooo`1<0?ooo`800000A03oool5
00000280oooo1@0000140?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0?P3oool0
03h0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`170?ooo`L000005@3oool60000
04P0oooo00<000000?ooo`3oool0CP3oool00`000000oooo0?ooo`0o0?ooo`00?`3oool00`000000
oooo0?ooo`1=0?ooo`030000003oool0oooo04d0oooo5@00001=0?ooo`030000003oool0oooo04l0
oooo00<000000?ooo`3oool0?`3oool003l0oooo00<000000?ooo`3oool0CP3oool00`000000oooo
0?ooo`2^0?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0?`3oool003l0oooo00<0
00000?ooo`3oool0C`3oool00`000000oooo0?ooo`2[0?ooo`800000D@3oool00`000000oooo0?oo
o`100?ooo`00@03oool00`000000oooo0?ooo`1?0?ooo`800000ZP3oool00`000000oooo0?ooo`1A
0?ooo`030000003oool0oooo0400oooo00100?ooo`030000003oool0oooo0540oooo00<000000?oo
o`3oool0YP3oool00`000000oooo0?ooo`1A0?ooo`030000003oool0oooo0440oooo00110?ooo`03
0000003oool0oooo0540oooo00<000000?ooo`3oool0X`3oool2000005<0oooo0P0000130?ooo`00
@@3oool00`000000oooo0?ooo`1B0?ooo`030000003oool0oooo0:40oooo00<000000?ooo`3oool0
DP3oool00`000000oooo0?ooo`130?ooo`00@P3oool00`000000oooo0?ooo`1B0?ooo`800000X03o
ool00`000000oooo0?ooo`1C0?ooo`030000003oool0oooo04<0oooo00120?ooo`030000003oool0
oooo05@0oooo00<000000?ooo`3oool0W03oool00`000000oooo0?ooo`1D0?ooo`030000003oool0
oooo04<0oooo00130?ooo`030000003oool0oooo05@0oooo00<000000?ooo`3oool0V@3oool20000
05H0oooo00<000000?ooo`3oool0A03oool004@0oooo00<000000?ooo`3oool0E03oool2000009P0
oooo00<000000?ooo`3oool0EP3oool00`000000oooo0?ooo`140?ooo`00A03oool00`000000oooo
0?ooo`1F0?ooo`030000003oool0oooo09@0oooo00<000000?ooo`3oool0EP3oool00`000000oooo
0?ooo`150?ooo`00A@3oool00`000000oooo0?ooo`1F0?ooo`030000003oool0oooo0900oooo0`00
001I0?ooo`030000003oool0oooo04D0oooo00160?ooo`030000003oool0oooo05H0oooo0P00002?
0?ooo`030000003oool0oooo05T0oooo00<000000?ooo`3oool0AP3oool004H0oooo00<000000?oo
o`3oool0F03oool3000008X0oooo0P00001L0?ooo`030000003oool0oooo04H0oooo00170?ooo`03
0000003oool0oooo05X0oooo00<000000?ooo`3oool0QP3oool00`000000oooo0?ooo`1K0?ooo`03
0000003oool0oooo04L0oooo00180?ooo`030000003oool0oooo05X0oooo0P0000230?ooo`<00000
G@3oool00`000000oooo0?ooo`180?ooo`00B03oool00`000000oooo0?ooo`1L0?ooo`030000003o
ool0oooo07l0oooo00<000000?ooo`3oool0GP3oool00`000000oooo0?ooo`180?ooo`00B03oool0
0`000000oooo0?ooo`1M0?ooo`800000OP3oool00`000000oooo0?ooo`1N0?ooo`030000003oool0
oooo04T0oooo00190?ooo`030000003oool0oooo05h0oooo0`00001i0?ooo`800000H@3oool00`00
0000oooo0?ooo`190?ooo`00BP3oool00`000000oooo0?ooo`1P0?ooo`800000M03oool300000640
oooo0P00001<0?ooo`00B`3oool00`000000oooo0?ooo`1Q0?ooo`800000L03oool2000006@0oooo
00<000000?ooo`3oool0B`3oool004/0oooo00<000000?ooo`3oool0H`3oool3000006/0oooo0P00
001U0?ooo`030000003oool0oooo04`0oooo001<0?ooo`030000003oool0oooo06D0oooo0P00001T
0?ooo`D00000IP3oool00`000000oooo0?ooo`1=0?ooo`00C@3oool00`000000oooo0?ooo`1V0?oo
o`800000H03oool2000006X0oooo00<000000?ooo`3oool0CP3oool004d0oooo00<000000?ooo`3o
ool0J03oool4000005X0oooo0P00001/0?ooo`030000003oool0oooo04h0oooo001>0?ooo`800000
K03oool300000580oooo1@00001]0?ooo`030000003oool0oooo04l0oooo001?0?ooo`030000003o
ool0oooo06d0oooo1000001<0?ooo`800000L@3oool00`000000oooo0?ooo`1@0?ooo`00D03oool0
0`000000oooo0?ooo`1`0?ooo`D00000@`3oool400000780oooo00<000000?ooo`3oool0D@3oool0
0540oooo00<000000?ooo`3oool0M03oool5000003L0oooo1`00001f0?ooo`030000003oool0oooo
0540oooo001B0?ooo`030000003oool0oooo07P0oooo1P00000/0?ooo`D00000N`3oool2000005@0
oooo001C0?ooo`030000003oool0oooo07d0oooo2@00000J0?ooo`T00000O`3oool00`000000oooo
0?ooo`1D0?ooo`00E03oool2000008H0oooo6P0000270?ooo`030000003oool0oooo05D0oooo001F
0?ooo`030000003oool0oooo0?l0oooo903oool00`000000oooo0?ooo`1F0?ooo`00EP3oool00`00
0000oooo0?ooo`3o0?ooob<0oooo00<000000?ooo`3oool0E`3oool005L0oooo0P00003o0?ooob80
oooo00<000000?ooo`3oool0F03oool005T0oooo00<000000?ooo`3oool0o`3ooolN0?ooo`030000
003oool0oooo05T0oooo001J0?ooo`030000003oool0oooo0?l0oooo703oool00`000000oooo0?oo
o`1J0?ooo`00F`3oool00`000000oooo0?ooo`3o0?oooaT0oooo0P00001M0?ooo`00G03oool00`00
0000oooo0?ooo`3o0?oooaL0oooo00<000000?ooo`3oool0G@3oool005d0oooo0P00003o0?oooaD0
oooo0P00001P0?ooo`00G`3oool200000?l0oooo4P3oool00`000000oooo0?ooo`1P0?ooo`00H@3o
ool00`000000oooo0?ooo`3o0?ooo`h0oooo00<000000?ooo`3oool0H@3oool00680oooo00<00000
0?ooo`3oool0o`3oool;0?ooo`800000I03oool006<0oooo00<000000?ooo`3oool0o`3oool90?oo
o`030000003oool0oooo06@0oooo001T0?ooo`800000o`3oool70?ooo`800000I`3oool006H0oooo
0P00003o0?ooo`<0oooo0P00001Y0?ooo`00J03oool00`000000oooo0?ooo`3n0?ooo`030000003o
ool0oooo06T0oooo001Y0?ooo`<00000n`3oool2000006`0oooo001/0?ooo`030000003oool0oooo
0?H0oooo0P00001^0?ooo`00K@3oool300000?80oooo0`00001`0?ooo`00L03oool200000>l0oooo
00<000000?ooo`3oool0L@3oool00780oooo0P00003[0?ooo`800000M03oool007@0oooo0`00003U
0?ooo`<00000MP3oool007L0oooo0P00003Q0?ooo`800000N@3oool007T0oooo0P00003M0?ooo`80
0000N`3oool007/0oooo0P00003H0?ooo`<00000O@3oool007d0oooo0`00003A0?ooo`@00000P03o
ool00800oooo1000003:0?ooo`<00000Q03oool008@0oooo0`0000340?ooo`<00000Q`3oool008L0
oooo0P00002o0?ooo`<00000RP3oool008T0oooo1000002h0?ooo`<00000S@3oool008d0oooo1@00
002_0?ooo`@00000T03oool00980oooo1000002V0?ooo`D00000U03oool009H0oooo1@00002M0?oo
o`@00000V@3oool009/0oooo1000002D0?ooo`D00000W@3oool009l0oooo1@0000290?ooo`H00000
XP3oool00:@0oooo1`00001k0?ooo`L00000Z03oool00:/0oooo1P00001^0?ooo`L00000[`3oool0
0;40oooo2@00001L0?ooo`T00000]P3oool00;X0oooo300000170?ooo`T00000_`3oool00<H0oooo
4000000U0?oooa800000b03oool00=H0oooo9@00003J0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?oo
o`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?oo
o`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?oo
o`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?oo
o`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?oo
o`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?oo
o`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00o`3ooooF0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 468.875}, {468.875, 0}} -> {-1.38062, -1.38062, \
0.39593, 0.39593}}]
}, Closed]],

Cell["\<\
Notice that potential is pretty uniform and the electric field is quite small \
in the bottom of the pail, despite the charged object above it.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(PlotGradientField[\(-f[x, y]\), {x, 42, 80}, {y, 30, 83}, 
        PlotPoints \[Rule] 15, ScaleFactor \[Rule] 4, 
        Epilog \[Rule] {RGBColor[1, 0, 0], Polygon[poly1], \ 
            RGBColor[0, 0, 1], Polygon[poly2]}];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.3833 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.994043 0.0245063 -0.70186 0.0245063 [
[ 0 0 0 0 ]
[ 1 1.3833 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
% Start of user PostScript
/mathtops {
gsave
MBeginOrig
moveto
MEndOrig
currentpoint
grestore
} bind def
/MAtocoords {
mathtops 4 2 roll mathtops
4 copy pop pop
3 -1 roll sub
/arry exch def
exch sub
/arrx exch def
arrx dup mul
arry dup mul
add sqrt
/arrl exch def
translate
} bind def
/MAarrowhead1 {
gsave
MAtocoords
arrl 0. eq
{ 0 0 Mdot }
{
[ arrx arrl div
arry arrl div
-1 arry mul arrl div
arrx arrl div
0 0 ]
concat
-0.02 0.005 moveto 0 0 lineto
-0.02 -0.005 lineto fill
-0.02 0.005 moveto 0 0 lineto
-0.02 -0.005 lineto
-0.02 0.005 lineto stroke
} ifelse
grestore
} def
/MAarrowzero1 {
gsave
mathtops translate
0 0 Mdot
grestore
} def
% End of user PostScript
0 g
.0001 w
[ ] 0 setdash
.03522 .03333 m
.03559 .03303 L
s
% Start of user PostScript
42. 30. 42.015066 29.987657 MAarrowhead1
% End of user PostScript
.03522 .1261 m
.0358 .12576 L
s
% Start of user PostScript
42. 33.785714 42.023737 33.771855 MAarrowhead1
% End of user PostScript
.03522 .21888 m
.03611 .21842 L
s
% Start of user PostScript
42. 37.571429 42.036159 37.552671 MAarrowhead1
% End of user PostScript
.03522 .31165 m
.0365 .31089 L
s
% Start of user PostScript
42. 41.357143 42.052203 41.326076 MAarrowhead1
% End of user PostScript
.03522 .40442 m
.03695 .40316 L
s
% Start of user PostScript
42. 45.142857 42.070471 45.091436 MAarrowhead1
% End of user PostScript
.03522 .4972 m
.03748 .4951 L
s
% Start of user PostScript
42. 48.928571 42.092127 48.843073 MAarrowhead1
% End of user PostScript
.03522 .58997 m
.03805 .58643 L
s
% Start of user PostScript
42. 52.714286 42.115485 52.569563 MAarrowhead1
% End of user PostScript
.03522 .68275 m
.03807 .67657 L
s
% Start of user PostScript
42. 56.5 42.116067 56.248079 MAarrowhead1
% End of user PostScript
.03522 .77552 m
.03572 .76584 L
s
% Start of user PostScript
42. 60.285714 42.020412 59.890762 MAarrowhead1
% End of user PostScript
.03522 .86829 m
.03055 .85748 L
s
% Start of user PostScript
42. 64.071429 41.809216 63.630005 MAarrowhead1
% End of user PostScript
.03522 .96107 m
.02598 .95268 L
s
% Start of user PostScript
42. 67.857143 41.622782 67.514683 MAarrowhead1
% End of user PostScript
.03522 1.05384 m
.02381 1.04948 L
s
% Start of user PostScript
42. 71.642857 41.534311 71.465005 MAarrowhead1
% End of user PostScript
.03522 1.14662 m
.02387 1.14608 L
s
% Start of user PostScript
42. 75.428571 41.53681 75.406692 MAarrowhead1
% End of user PostScript
.03522 1.23939 m
.0252 1.24165 L
s
% Start of user PostScript
42. 79.214286 41.590863 79.306414 MAarrowhead1
% End of user PostScript
.03522 1.33216 m
.02697 1.33607 L
s
% Start of user PostScript
42. 83. 41.663405 83.159415 MAarrowhead1
% End of user PostScript
.10174 .03333 m
.10213 .03316 L
s
% Start of user PostScript
44.714286 30. 44.730373 29.99325 MAarrowhead1
% End of user PostScript
.10174 .1261 m
.10233 .12596 L
s
% Start of user PostScript
44.714286 33.785714 44.738282 33.779684 MAarrowhead1
% End of user PostScript
.10174 .21888 m
.10274 .21873 L
s
% Start of user PostScript
44.714286 37.571429 44.754949 37.565465 MAarrowhead1
% End of user PostScript
.10174 .31165 m
.1033 .31125 L
s
% Start of user PostScript
44.714286 41.357143 44.777926 41.34064 MAarrowhead1
% End of user PostScript
.10174 .40442 m
.10387 .40358 L
s
% Start of user PostScript
44.714286 45.142857 44.801187 45.108585 MAarrowhead1
% End of user PostScript
.10174 .4972 m
.10474 .49569 L
s
% Start of user PostScript
44.714286 48.928571 44.836752 48.866919 MAarrowhead1
% End of user PostScript
.10174 .58997 m
.10603 .58713 L
s
% Start of user PostScript
44.714286 52.714286 44.889217 52.598189 MAarrowhead1
% End of user PostScript
.10174 .68275 m
.10763 .67658 L
s
% Start of user PostScript
44.714286 56.5 44.954631 56.248184 MAarrowhead1
% End of user PostScript
.10174 .77552 m
.10565 .76194 L
s
% Start of user PostScript
44.714286 60.285714 44.873748 59.731641 MAarrowhead1
% End of user PostScript
.10174 .86829 m
.09609 .85238 L
s
% Start of user PostScript
44.714286 64.071429 44.483692 63.421991 MAarrowhead1
% End of user PostScript
.10174 .96107 m
.08913 .9498 L
s
% Start of user PostScript
44.714286 67.857143 44.199801 67.397286 MAarrowhead1
% End of user PostScript
.10174 1.05384 m
.08668 1.04895 L
s
% Start of user PostScript
44.714286 71.642857 44.099666 71.443067 MAarrowhead1
% End of user PostScript
.10174 1.14662 m
.08762 1.14707 L
s
% Start of user PostScript
44.714286 75.428571 44.138004 75.447016 MAarrowhead1
% End of user PostScript
.10174 1.23939 m
.09007 1.24317 L
s
% Start of user PostScript
44.714286 79.214286 44.238304 79.368503 MAarrowhead1
% End of user PostScript
.10174 1.33216 m
.0927 1.33758 L
s
% Start of user PostScript
44.714286 83. 44.345478 83.220962 MAarrowhead1
% End of user PostScript
.16826 .03333 m
.16859 .03327 L
s
% Start of user PostScript
47.428571 30. 47.442254 29.997596 MAarrowhead1
% End of user PostScript
.16826 .1261 m
.1688 .12616 L
s
% Start of user PostScript
47.428571 33.785714 47.45069 33.78785 MAarrowhead1
% End of user PostScript
.16826 .21888 m
.16922 .21918 L
s
% Start of user PostScript
47.428571 37.571429 47.467916 37.583633 MAarrowhead1
% End of user PostScript
.16826 .31165 m
.17011 .31162 L
s
% Start of user PostScript
47.428571 41.357143 47.504373 41.355973 MAarrowhead1
% End of user PostScript
.16826 .40442 m
.17065 .40411 L
s
% Start of user PostScript
47.428571 45.142857 47.526097 45.129872 MAarrowhead1
% End of user PostScript
.16826 .4972 m
.17169 .49659 L
s
% Start of user PostScript
47.428571 48.928571 47.568665 48.90358 MAarrowhead1
% End of user PostScript
.16826 .58997 m
.17373 .58874 L
s
% Start of user PostScript
47.428571 52.714286 47.651941 52.663896 MAarrowhead1
% End of user PostScript
.16826 .68275 m
.17819 .67921 L
s
% Start of user PostScript
47.428571 56.5 47.834113 56.355741 MAarrowhead1
% End of user PostScript
.16826 .77552 m
.18293 .75273 L
s
% Start of user PostScript
47.428571 60.285714 48.027347 59.355907 MAarrowhead1
% End of user PostScript
.16826 .86829 m
.15939 .8439 L
s
% Start of user PostScript
47.428571 64.071429 47.06695 63.075943 MAarrowhead1
% End of user PostScript
.16826 .96107 m
.14989 .94612 L
s
% Start of user PostScript
47.428571 67.857143 46.679159 67.247185 MAarrowhead1
% End of user PostScript
.16826 1.05384 m
.14777 1.04903 L
s
% Start of user PostScript
47.428571 71.642857 46.592654 71.446382 MAarrowhead1
% End of user PostScript
.16826 1.14662 m
.15063 1.14917 L
s
% Start of user PostScript
47.428571 75.428571 46.709303 75.532869 MAarrowhead1
% End of user PostScript
.16826 1.23939 m
.15489 1.2456 L
s
% Start of user PostScript
47.428571 79.214286 46.883337 79.467654 MAarrowhead1
% End of user PostScript
.16826 1.33216 m
.15865 1.3396 L
s
% Start of user PostScript
47.428571 83. 47.036575 83.303532 MAarrowhead1
% End of user PostScript
.23477 .03333 m
.23505 .03336 L
s
% Start of user PostScript
50.142857 30. 50.154294 30.001247 MAarrowhead1
% End of user PostScript
.23477 .1261 m
.23515 .1263 L
s
% Start of user PostScript
50.142857 33.785714 50.158203 33.793694 MAarrowhead1
% End of user PostScript
.23477 .21888 m
.23523 .21946 L
s
% Start of user PostScript
50.142857 37.571429 50.161534 37.595074 MAarrowhead1
% End of user PostScript
.23477 .31165 m
.23477 .31165 L
s
% Start of user PostScript
50.142857 41.357143 MAarrowzero1
% End of user PostScript
.23477 .40442 m
.23477 .40442 L
s
% Start of user PostScript
50.142857 45.142857 MAarrowzero1
% End of user PostScript
.23477 .4972 m
.23477 .4972 L
s
% Start of user PostScript
50.142857 48.928571 MAarrowzero1
% End of user PostScript
.23477 .58997 m
.23477 .58997 L
s
% Start of user PostScript
50.142857 52.714286 MAarrowzero1
% End of user PostScript
.23477 .68275 m
.23477 .68275 L
s
% Start of user PostScript
50.142857 56.5 MAarrowzero1
% End of user PostScript
.23477 .77552 m
.23376 .7239 L
s
% Start of user PostScript
50.142857 60.285714 50.10141 58.179321 MAarrowhead1
% End of user PostScript
.23477 .86829 m
.21661 .83353 L
s
% Start of user PostScript
50.142857 64.071429 49.401728 62.653005 MAarrowhead1
% End of user PostScript
.23477 .96107 m
.20694 .94205 L
s
% Start of user PostScript
50.142857 67.857143 49.007181 67.081053 MAarrowhead1
% End of user PostScript
.23477 1.05384 m
.20619 1.05075 L
s
% Start of user PostScript
50.142857 71.642857 48.976529 71.516805 MAarrowhead1
% End of user PostScript
.23477 1.14662 m
.21299 1.15332 L
s
% Start of user PostScript
50.142857 75.428571 49.253925 75.702209 MAarrowhead1
% End of user PostScript
.23477 1.23939 m
.22014 1.24926 L
s
% Start of user PostScript
50.142857 79.214286 49.545696 79.617245 MAarrowhead1
% End of user PostScript
.23477 1.33216 m
.22513 1.3422 L
s
% Start of user PostScript
50.142857 83. 49.749175 83.409348 MAarrowhead1
% End of user PostScript
.30129 .03333 m
.30149 .03342 L
s
% Start of user PostScript
52.857143 30. 52.865127 30.003858 MAarrowhead1
% End of user PostScript
.30129 .1261 m
.30153 .12632 L
s
% Start of user PostScript
52.857143 33.785714 52.86687 33.794762 MAarrowhead1
% End of user PostScript
.30129 .21888 m
.30147 .21934 L
s
% Start of user PostScript
52.857143 37.571429 52.864402 37.590354 MAarrowhead1
% End of user PostScript
.30129 .31165 m
.30129 .31165 L
s
% Start of user PostScript
52.857143 41.357143 MAarrowzero1
% End of user PostScript
.30129 .40442 m
.29967 .40384 L
s
% Start of user PostScript
52.857143 45.142857 52.790827 45.119131 MAarrowhead1
% End of user PostScript
.30129 .4972 m
.29584 .49623 L
s
% Start of user PostScript
52.857143 48.928571 52.634657 48.888964 MAarrowhead1
% End of user PostScript
.30129 .58997 m
.28873 .58805 L
s
% Start of user PostScript
52.857143 52.714286 52.344629 52.63582 MAarrowhead1
% End of user PostScript
.30129 .68275 m
.27299 .67794 L
s
% Start of user PostScript
52.857143 56.5 51.702358 56.303888 MAarrowhead1
% End of user PostScript
.30129 .77552 m
.23328 .72646 L
s
% Start of user PostScript
52.857143 60.285714 50.082088 58.283767 MAarrowhead1
% End of user PostScript
.30129 .86829 m
.27128 .82671 L
s
% Start of user PostScript
52.857143 64.071429 51.632531 62.374351 MAarrowhead1
% End of user PostScript
.30129 .96107 m
.25929 .93641 L
s
% Start of user PostScript
52.857143 67.857143 51.143333 66.850776 MAarrowhead1
% End of user PostScript
.30129 1.05384 m
.2603 1.05666 L
s
% Start of user PostScript
52.857143 71.642857 51.184663 71.757983 MAarrowhead1
% End of user PostScript
.30129 1.14662 m
.27569 1.161 L
s
% Start of user PostScript
52.857143 75.428571 51.812513 76.015583 MAarrowhead1
% End of user PostScript
.30129 1.23939 m
.28662 1.25439 L
s
% Start of user PostScript
52.857143 79.214286 52.258634 79.826256 MAarrowhead1
% End of user PostScript
.30129 1.33216 m
.29258 1.34518 L
s
% Start of user PostScript
52.857143 83. 52.501674 83.531293 MAarrowhead1
% End of user PostScript
.36781 .03333 m
.36791 .03345 L
s
% Start of user PostScript
55.571429 30. 55.575625 30.00493 MAarrowhead1
% End of user PostScript
.36781 .1261 m
.36792 .12634 L
s
% Start of user PostScript
55.571429 33.785714 55.576072 33.795509 MAarrowhead1
% End of user PostScript
.36781 .21888 m
.36788 .21923 L
s
% Start of user PostScript
55.571429 37.571429 55.574359 37.585859 MAarrowhead1
% End of user PostScript
.36781 .31165 m
.36781 .31165 L
s
% Start of user PostScript
55.571429 41.357143 MAarrowzero1
% End of user PostScript
.36781 .40442 m
.36672 .40211 L
s
% Start of user PostScript
55.571429 45.142857 55.52721 45.048239 MAarrowhead1
% End of user PostScript
.36781 .4972 m
.36419 .49346 L
s
% Start of user PostScript
55.571429 48.928571 55.423857 48.775827 MAarrowhead1
% End of user PostScript
.36781 .58997 m
.35966 .58253 L
s
% Start of user PostScript
55.571429 52.714286 55.238903 52.410782 MAarrowhead1
% End of user PostScript
.36781 .68275 m
.35124 .66657 L
s
% Start of user PostScript
55.571429 56.5 54.895284 55.839905 MAarrowhead1
% End of user PostScript
.36781 .77552 m
.34213 .7393 L
s
% Start of user PostScript
55.571429 60.285714 54.523501 58.80758 MAarrowhead1
% End of user PostScript
.36781 .86829 m
.33499 .81832 L
s
% Start of user PostScript
55.571429 64.071429 54.232082 62.032155 MAarrowhead1
% End of user PostScript
.36781 .96107 m
.30457 .92251 L
s
% Start of user PostScript
55.571429 67.857143 52.990895 66.283828 MAarrowhead1
% End of user PostScript
.36781 1.05384 m
.31183 1.07301 L
s
% Start of user PostScript
55.571429 71.642857 53.287123 72.425142 MAarrowhead1
% End of user PostScript
.36781 1.14662 m
.34167 1.17389 L
s
% Start of user PostScript
55.571429 75.428571 54.504951 76.54135 MAarrowhead1
% End of user PostScript
.36781 1.23939 m
.35571 1.26048 L
s
% Start of user PostScript
55.571429 79.214286 55.077707 80.074806 MAarrowhead1
% End of user PostScript
.36781 1.33216 m
.36137 1.3481 L
s
% Start of user PostScript
55.571429 83. 55.30873 83.650416 MAarrowhead1
% End of user PostScript
.43432 .03333 m
.43437 .03346 L
s
% Start of user PostScript
58.285714 30. 58.287558 30.005335 MAarrowhead1
% End of user PostScript
.43432 .1261 m
.43437 .12633 L
s
% Start of user PostScript
58.285714 33.785714 58.287602 33.795163 MAarrowhead1
% End of user PostScript
.43432 .21888 m
.43435 .21919 L
s
% Start of user PostScript
58.285714 37.571429 58.286769 37.584344 MAarrowhead1
% End of user PostScript
.43432 .31165 m
.43432 .31165 L
s
% Start of user PostScript
58.285714 41.357143 MAarrowzero1
% End of user PostScript
.43432 .40442 m
.43386 .40103 L
s
% Start of user PostScript
58.285714 45.142857 58.266574 45.004463 MAarrowhead1
% End of user PostScript
.43432 .4972 m
.43276 .49177 L
s
% Start of user PostScript
58.285714 48.928571 58.221824 48.707229 MAarrowhead1
% End of user PostScript
.43432 .58997 m
.43088 .57942 L
s
% Start of user PostScript
58.285714 52.714286 58.144984 52.283593 MAarrowhead1
% End of user PostScript
.43432 .68275 m
.42778 .66161 L
s
% Start of user PostScript
58.285714 56.5 58.018454 55.637524 MAarrowhead1
% End of user PostScript
.43432 .77552 m
.42405 .73592 L
s
% Start of user PostScript
58.285714 60.285714 57.866365 58.669612 MAarrowhead1
% End of user PostScript
.43432 .86829 m
.4138 .80257 L
s
% Start of user PostScript
58.285714 64.071429 57.448031 61.389373 MAarrowhead1
% End of user PostScript
.43432 .96107 m
.43415 .97754 L
s
% Start of user PostScript
58.285714 67.857143 58.278411 68.52949 MAarrowhead1
% End of user PostScript
.43432 1.05384 m
.43458 1.05374 L
s
% Start of user PostScript
58.285714 71.642857 58.295974 71.638557 MAarrowhead1
% End of user PostScript
.43432 1.14662 m
.41792 1.18976 L
s
% Start of user PostScript
58.285714 75.428571 57.616311 77.188954 MAarrowhead1
% End of user PostScript
.43432 1.23939 m
.42861 1.26557 L
s
% Start of user PostScript
58.285714 79.214286 58.052607 80.282437 MAarrowhead1
% End of user PostScript
.43432 1.33216 m
.43155 1.35013 L
s
% Start of user PostScript
58.285714 83. 58.172346 83.732996 MAarrowhead1
% End of user PostScript
.50084 .03333 m
.50082 .03346 L
s
% Start of user PostScript
61. 30. 60.999182 30.005346 MAarrowhead1
% End of user PostScript
.50084 .1261 m
.50082 .12632 L
s
% Start of user PostScript
61. 33.785714 60.999054 33.794396 MAarrowhead1
% End of user PostScript
.50084 .21888 m
.50083 .21919 L
s
% Start of user PostScript
61. 37.571429 60.999349 37.584161 MAarrowhead1
% End of user PostScript
.50084 .31165 m
.50084 .31165 L
s
% Start of user PostScript
61. 41.357143 MAarrowzero1
% End of user PostScript
.50084 .40442 m
.50111 .40091 L
s
% Start of user PostScript
61. 45.142857 61.010995 44.999573 MAarrowhead1
% End of user PostScript
.50084 .4972 m
.50176 .49154 L
s
% Start of user PostScript
61. 48.928571 61.037422 48.697634 MAarrowhead1
% End of user PostScript
.50084 .58997 m
.50285 .57906 L
s
% Start of user PostScript
61. 52.714286 61.081749 52.268872 MAarrowhead1
% End of user PostScript
.50084 .68275 m
.50465 .66107 L
s
% Start of user PostScript
61. 56.5 61.155432 55.615496 MAarrowhead1
% End of user PostScript
.50084 .77552 m
.50691 .73531 L
s
% Start of user PostScript
61. 60.285714 61.247664 58.645093 MAarrowhead1
% End of user PostScript
.50084 .86829 m
.51547 .79818 L
s
% Start of user PostScript
61. 64.071429 61.596949 61.210505 MAarrowhead1
% End of user PostScript
.50084 .96107 m
.49929 .96984 L
s
% Start of user PostScript
61. 67.857143 60.936709 68.215291 MAarrowhead1
% End of user PostScript
.50084 1.05384 m
.50084 1.05384 L
s
% Start of user PostScript
61. 71.642857 MAarrowzero1
% End of user PostScript
.50084 1.14662 m
.51351 1.19332 L
s
% Start of user PostScript
61. 75.428571 61.516853 77.334206 MAarrowhead1
% End of user PostScript
.50084 1.23939 m
.50424 1.26635 L
s
% Start of user PostScript
61. 79.214286 61.138614 80.314398 MAarrowhead1
% End of user PostScript
.50084 1.33216 m
.50246 1.35036 L
s
% Start of user PostScript
61. 83. 61.066005 83.742528 MAarrowhead1
% End of user PostScript
.56736 .03333 m
.56726 .03345 L
s
% Start of user PostScript
63.714286 30. 63.710065 30.005074 MAarrowhead1
% End of user PostScript
.56736 .1261 m
.56725 .12634 L
s
% Start of user PostScript
63.714286 33.785714 63.709827 33.795448 MAarrowhead1
% End of user PostScript
.56736 .21888 m
.56729 .21923 L
s
% Start of user PostScript
63.714286 37.571429 63.711567 37.585662 MAarrowhead1
% End of user PostScript
.56736 .31165 m
.56736 .31165 L
s
% Start of user PostScript
63.714286 41.357143 MAarrowzero1
% End of user PostScript
.56736 .40442 m
.56831 .40175 L
s
% Start of user PostScript
63.714286 45.142857 63.753181 45.033727 MAarrowhead1
% End of user PostScript
.56736 .4972 m
.57051 .49289 L
s
% Start of user PostScript
63.714286 48.928571 63.842724 48.752626 MAarrowhead1
% End of user PostScript
.56736 .58997 m
.57441 .58147 L
s
% Start of user PostScript
63.714286 52.714286 64.002044 52.367479 MAarrowhead1
% End of user PostScript
.56736 .68275 m
.58137 .66475 L
s
% Start of user PostScript
63.714286 56.5 64.286171 55.765822 MAarrowhead1
% End of user PostScript
.56736 .77552 m
.58896 .73853 L
s
% Start of user PostScript
63.714286 60.285714 64.595882 58.776131 MAarrowhead1
% End of user PostScript
.56736 .86829 m
.5987 .81467 L
s
% Start of user PostScript
63.714286 64.071429 64.993064 61.883086 MAarrowhead1
% End of user PostScript
.56736 .96107 m
.65304 .91344 L
s
% Start of user PostScript
63.714286 67.857143 67.210454 65.913742 MAarrowhead1
% End of user PostScript
.56736 1.05384 m
.63314 1.06892 L
s
% Start of user PostScript
63.714286 71.642857 66.398631 72.258055 MAarrowhead1
% End of user PostScript
.56736 1.14662 m
.59225 1.17812 L
s
% Start of user PostScript
63.714286 75.428571 64.730106 76.713995 MAarrowhead1
% End of user PostScript
.56736 1.23939 m
.57815 1.26202 L
s
% Start of user PostScript
63.714286 79.214286 64.15478 80.137742 MAarrowhead1
% End of user PostScript
.56736 1.33216 m
.57294 1.34877 L
s
% Start of user PostScript
63.714286 83. 63.94216 83.677814 MAarrowhead1
% End of user PostScript
.63388 .03333 m
.63368 .03343 L
s
% Start of user PostScript
66.428571 30. 66.420707 30.004076 MAarrowhead1
% End of user PostScript
.63388 .1261 m
.63367 .12636 L
s
% Start of user PostScript
66.428571 33.785714 66.420317 33.796007 MAarrowhead1
% End of user PostScript
.63388 .21888 m
.63374 .21931 L
s
% Start of user PostScript
66.428571 37.571429 66.422893 37.58897 MAarrowhead1
% End of user PostScript
.63388 .31165 m
.63388 .31165 L
s
% Start of user PostScript
66.428571 41.357143 MAarrowzero1
% End of user PostScript
.63388 .40442 m
.63522 .40334 L
s
% Start of user PostScript
66.428571 45.142857 66.483588 45.098458 MAarrowhead1
% End of user PostScript
.63388 .4972 m
.63841 .49544 L
s
% Start of user PostScript
66.428571 48.928571 66.613541 48.856843 MAarrowhead1
% End of user PostScript
.63388 .58997 m
.64429 .58641 L
s
% Start of user PostScript
66.428571 52.714286 66.853714 52.568777 MAarrowhead1
% End of user PostScript
.63388 .68275 m
.65678 .67414 L
s
% Start of user PostScript
66.428571 56.5 67.363291 56.148918 MAarrowhead1
% End of user PostScript
.63388 .77552 m
.6753 .73687 L
s
% Start of user PostScript
66.428571 60.285714 68.119074 58.708598 MAarrowhead1
% End of user PostScript
.63388 .86829 m
.66586 .82522 L
s
% Start of user PostScript
66.428571 64.071429 67.733554 62.313938 MAarrowhead1
% End of user PostScript
.63388 .96107 m
.68062 .93399 L
s
% Start of user PostScript
66.428571 67.857143 68.335866 66.752354 MAarrowhead1
% End of user PostScript
.63388 1.05384 m
.67903 1.05975 L
s
% Start of user PostScript
66.428571 71.642857 68.271311 71.883887 MAarrowhead1
% End of user PostScript
.63388 1.14662 m
.66013 1.16385 L
s
% Start of user PostScript
66.428571 75.428571 67.500023 76.131701 MAarrowhead1
% End of user PostScript
.63388 1.23939 m
.64814 1.25593 L
s
% Start of user PostScript
66.428571 79.214286 67.010736 79.889274 MAarrowhead1
% End of user PostScript
.63388 1.33216 m
.6421 1.34598 L
s
% Start of user PostScript
66.428571 83. 66.764157 83.563986 MAarrowhead1
% End of user PostScript
.70039 .03333 m
.70013 .03339 L
s
% Start of user PostScript
69.142857 30. 69.132249 30.00242 MAarrowhead1
% End of user PostScript
.70039 .1261 m
.70005 .12632 L
s
% Start of user PostScript
69.142857 33.785714 69.128694 33.794691 MAarrowhead1
% End of user PostScript
.70039 .21888 m
.70004 .21945 L
s
% Start of user PostScript
69.142857 37.571429 69.128292 37.594927 MAarrowhead1
% End of user PostScript
.70039 .31165 m
.70039 .31165 L
s
% Start of user PostScript
69.142857 41.357143 MAarrowzero1
% End of user PostScript
.70039 .40442 m
.70039 .40442 L
s
% Start of user PostScript
69.142857 45.142857 MAarrowzero1
% End of user PostScript
.70039 .4972 m
.70039 .4972 L
s
% Start of user PostScript
69.142857 48.928571 MAarrowzero1
% End of user PostScript
.70039 .58997 m
.70039 .58997 L
s
% Start of user PostScript
69.142857 52.714286 MAarrowzero1
% End of user PostScript
.70039 .68275 m
.70039 .68275 L
s
% Start of user PostScript
69.142857 56.5 MAarrowzero1
% End of user PostScript
.70039 .77552 m
.70296 .7189 L
s
% Start of user PostScript
69.142857 60.285714 69.247463 57.975136 MAarrowhead1
% End of user PostScript
.70039 .86829 m
.72179 .83128 L
s
% Start of user PostScript
69.142857 64.071429 70.015859 62.56087 MAarrowhead1
% End of user PostScript
.70039 .96107 m
.73137 .94081 L
s
% Start of user PostScript
69.142857 67.857143 70.406967 67.030381 MAarrowhead1
% End of user PostScript
.70039 1.05384 m
.73173 1.05171 L
s
% Start of user PostScript
69.142857 71.642857 70.421585 71.555862 MAarrowhead1
% End of user PostScript
.70039 1.14662 m
.72323 1.15493 L
s
% Start of user PostScript
69.142857 75.428571 70.074738 75.767637 MAarrowhead1
% End of user PostScript
.70039 1.23939 m
.71518 1.25048 L
s
% Start of user PostScript
69.142857 79.214286 69.746345 79.666844 MAarrowhead1
% End of user PostScript
.70039 1.33216 m
.70988 1.34295 L
s
% Start of user PostScript
69.142857 83. 69.529843 83.439988 MAarrowhead1
% End of user PostScript
.76691 .03333 m
.76656 .0333 L
s
% Start of user PostScript
71.857143 30. 71.842919 29.998839 MAarrowhead1
% End of user PostScript
.76691 .1261 m
.7664 .12622 L
s
% Start of user PostScript
71.857143 33.785714 71.8363 33.790382 MAarrowhead1
% End of user PostScript
.76691 .21888 m
.76601 .21931 L
s
% Start of user PostScript
71.857143 37.571429 71.820385 37.58914 MAarrowhead1
% End of user PostScript
.76691 .31165 m
.76458 .31166 L
s
% Start of user PostScript
71.857143 41.357143 71.762078 41.357392 MAarrowhead1
% End of user PostScript
.76691 .40442 m
.76405 .40425 L
s
% Start of user PostScript
71.857143 45.142857 71.740487 45.1356 MAarrowhead1
% End of user PostScript
.76691 .4972 m
.76281 .4969 L
s
% Start of user PostScript
71.857143 48.928571 71.689774 48.916211 MAarrowhead1
% End of user PostScript
.76691 .58997 m
.76035 .58928 L
s
% Start of user PostScript
71.857143 52.714286 71.589368 52.68618 MAarrowhead1
% End of user PostScript
.76691 .68275 m
.75439 .68071 L
s
% Start of user PostScript
71.857143 56.5 71.346285 56.416906 MAarrowhead1
% End of user PostScript
.76691 .77552 m
.73527 .74387 L
s
% Start of user PostScript
71.857143 60.285714 70.566225 58.994279 MAarrowhead1
% End of user PostScript
.76691 .86829 m
.77747 .84107 L
s
% Start of user PostScript
71.857143 64.071429 72.288187 62.960666 MAarrowhead1
% End of user PostScript
.76691 .96107 m
.78736 .9451 L
s
% Start of user PostScript
71.857143 67.857143 72.691689 67.205617 MAarrowhead1
% End of user PostScript
.76691 1.05384 m
.78919 1.04926 L
s
% Start of user PostScript
71.857143 71.642857 72.766321 71.455925 MAarrowhead1
% End of user PostScript
.76691 1.14662 m
.78558 1.15003 L
s
% Start of user PostScript
71.857143 75.428571 72.619076 75.567775 MAarrowhead1
% End of user PostScript
.76691 1.23939 m
.78063 1.24643 L
s
% Start of user PostScript
71.857143 79.214286 72.417066 79.501511 MAarrowhead1
% End of user PostScript
.76691 1.33216 m
.77659 1.34023 L
s
% Start of user PostScript
71.857143 83. 72.251992 83.329336 MAarrowhead1
% End of user PostScript
.83343 .03333 m
.83306 .03319 L
s
% Start of user PostScript
74.571429 30. 74.556418 29.994296 MAarrowhead1
% End of user PostScript
.83343 .1261 m
.83283 .12602 L
s
% Start of user PostScript
74.571429 33.785714 74.547025 33.782408 MAarrowhead1
% End of user PostScript
.83343 .21888 m
.83241 .21883 L
s
% Start of user PostScript
74.571429 37.571429 74.529912 37.569332 MAarrowhead1
% End of user PostScript
.83343 .31165 m
.83178 .31135 L
s
% Start of user PostScript
74.571429 41.357143 74.504053 41.344755 MAarrowhead1
% End of user PostScript
.83343 .40442 m
.83119 .4037 L
s
% Start of user PostScript
74.571429 45.142857 74.479991 45.113481 MAarrowhead1
% End of user PostScript
.83343 .4972 m
.83025 .49592 L
s
% Start of user PostScript
74.571429 48.928571 74.441873 48.876203 MAarrowhead1
% End of user PostScript
.83343 .58997 m
.82875 .58745 L
s
% Start of user PostScript
74.571429 52.714286 74.380416 52.611414 MAarrowhead1
% End of user PostScript
.83343 .68275 m
.82647 .67689 L
s
% Start of user PostScript
74.571429 56.5 74.287708 56.260943 MAarrowhead1
% End of user PostScript
.83343 .77552 m
.82789 .76041 L
s
% Start of user PostScript
74.571429 60.285714 74.345435 59.669064 MAarrowhead1
% End of user PostScript
.83343 .86829 m
.83955 .85052 L
s
% Start of user PostScript
74.571429 64.071429 74.821285 63.346279 MAarrowhead1
% End of user PostScript
.83343 .96107 m
.84723 .94887 L
s
% Start of user PostScript
74.571429 67.857143 75.134607 67.359334 MAarrowhead1
% End of user PostScript
.83343 1.05384 m
.84969 1.04887 L
s
% Start of user PostScript
74.571429 71.642857 75.235178 71.440046 MAarrowhead1
% End of user PostScript
.83343 1.14662 m
.84838 1.14749 L
s
% Start of user PostScript
74.571429 75.428571 75.181559 75.46404 MAarrowhead1
% End of user PostScript
.83343 1.23939 m
.84554 1.24371 L
s
% Start of user PostScript
74.571429 79.214286 75.065515 79.390625 MAarrowhead1
% End of user PostScript
.83343 1.33216 m
.84265 1.33806 L
s
% Start of user PostScript
74.571429 83. 74.947617 83.240584 MAarrowhead1
% End of user PostScript
.89994 .03333 m
.89954 .03307 L
s
% Start of user PostScript
77.285714 30. 77.269321 29.98936 MAarrowhead1
% End of user PostScript
.89994 .1261 m
.89934 .12581 L
s
% Start of user PostScript
77.285714 33.785714 77.261017 33.773832 MAarrowhead1
% End of user PostScript
.89994 .21888 m
.899 .21848 L
s
% Start of user PostScript
77.285714 37.571429 77.24718 37.555371 MAarrowhead1
% End of user PostScript
.89994 .31165 m
.89856 .31097 L
s
% Start of user PostScript
77.285714 41.357143 77.229408 41.329389 MAarrowhead1
% End of user PostScript
.89994 .40442 m
.89808 .40326 L
s
% Start of user PostScript
77.285714 45.142857 77.209815 45.095254 MAarrowhead1
% End of user PostScript
.89994 .4972 m
.89743 .49522 L
s
% Start of user PostScript
77.285714 48.928571 77.183075 48.847845 MAarrowhead1
% End of user PostScript
.89994 .58997 m
.8967 .58654 L
s
% Start of user PostScript
77.285714 52.714286 77.153281 52.574126 MAarrowhead1
% End of user PostScript
.89994 .68275 m
.8964 .67646 L
s
% Start of user PostScript
77.285714 56.5 77.141007 56.243521 MAarrowhead1
% End of user PostScript
.89994 .77552 m
.89879 .765 L
s
% Start of user PostScript
77.285714 60.285714 77.238599 59.856456 MAarrowhead1
% End of user PostScript
.89994 .86829 m
.90475 .85638 L
s
% Start of user PostScript
77.285714 64.071429 77.481982 63.585156 MAarrowhead1
% End of user PostScript
.89994 .96107 m
.90987 .952 L
s
% Start of user PostScript
77.285714 67.857143 77.69068 67.487098 MAarrowhead1
% End of user PostScript
.89994 1.05384 m
.91215 1.04931 L
s
% Start of user PostScript
77.285714 71.642857 77.783676 71.457908 MAarrowhead1
% End of user PostScript
.89994 1.14662 m
.91192 1.14625 L
s
% Start of user PostScript
77.285714 75.428571 77.774553 75.41366 MAarrowhead1
% End of user PostScript
.89994 1.23939 m
.91035 1.24197 L
s
% Start of user PostScript
77.285714 79.214286 77.710249 79.319595 MAarrowhead1
% End of user PostScript
.89994 1.33216 m
.90838 1.33642 L
s
% Start of user PostScript
77.285714 83. 77.629758 83.173552 MAarrowhead1
% End of user PostScript
.96646 .03333 m
.96611 .03294 L
s
% Start of user PostScript
80. 30. 79.985706 29.983941 MAarrowhead1
% End of user PostScript
.96646 .1261 m
.96593 .12563 L
s
% Start of user PostScript
80. 33.785714 79.978212 33.766444 MAarrowhead1
% End of user PostScript
.96646 .21888 m
.96569 .21824 L
s
% Start of user PostScript
80. 37.571429 79.96839 37.545457 MAarrowhead1
% End of user PostScript
.96646 .31165 m
.96537 .31069 L
s
% Start of user PostScript
80. 41.357143 79.955455 41.317763 MAarrowhead1
% End of user PostScript
.96646 .40442 m
.96505 .40294 L
s
% Start of user PostScript
80. 45.142857 79.942346 45.082282 MAarrowhead1
% End of user PostScript
.96646 .4972 m
.96471 .49488 L
s
% Start of user PostScript
80. 48.928571 79.928609 48.833983 MAarrowhead1
% End of user PostScript
.96646 .58997 m
.96455 .58632 L
s
% Start of user PostScript
80. 52.714286 79.921919 52.565136 MAarrowhead1
% End of user PostScript
.96646 .68275 m
.96504 .67709 L
s
% Start of user PostScript
80. 56.5 79.941803 56.268992 MAarrowhead1
% End of user PostScript
.96646 .77552 m
.96711 .76771 L
s
% Start of user PostScript
80. 60.285714 80.026402 59.967117 MAarrowhead1
% End of user PostScript
.96646 .86829 m
.97072 .85988 L
s
% Start of user PostScript
80. 64.071429 80.173572 63.727891 MAarrowhead1
% End of user PostScript
.96646 .96107 m
.97409 .9543 L
s
% Start of user PostScript
80. 67.857143 80.311259 67.58108 MAarrowhead1
% End of user PostScript
.96646 1.05384 m
.97594 1.04999 L
s
% Start of user PostScript
80. 71.642857 80.386645 71.485777 MAarrowhead1
% End of user PostScript
.96646 1.14662 m
.97619 1.14577 L
s
% Start of user PostScript
80. 75.428571 80.396993 75.393895 MAarrowhead1
% End of user PostScript
.96646 1.23939 m
.97536 1.2409 L
s
% Start of user PostScript
80. 79.214286 80.363252 79.275808 MAarrowhead1
% End of user PostScript
.96646 1.33216 m
.97406 1.33524 L
s
% Start of user PostScript
80. 83. 80.309954 83.125481 MAarrowhead1
% End of user PostScript
1 0 0 r
.20677 .76852 m
.20677 .27839 L
.7459 .27839 L
.7459 .76852 L
.67239 .76852 L
.67239 .35191 L
.28028 .35191 L
.28028 .76852 L
.20677 .76852 L
F
0 0 1 r
.57436 1.01358 m
.57417 1.01974 L
.57359 1.02587 L
.57262 1.03195 L
.57128 1.03796 L
.56956 1.04387 L
.56748 1.04967 L
.56503 1.05532 L
.56224 1.06081 L
.5591 1.06611 L
.55564 1.0712 L
.55187 1.07607 L
.54779 1.08068 L
.54344 1.08504 L
.53882 1.08911 L
.53395 1.09289 L
.52886 1.09635 L
.52356 1.09948 L
.51807 1.10228 L
.51242 1.10472 L
.50663 1.10681 L
.50071 1.10853 L
.4947 1.10987 L
.48862 1.11083 L
.48249 1.11141 L
.47634 1.11161 L
.47018 1.11141 L
.46405 1.11083 L
.45797 1.10987 L
.45196 1.10853 L
.44604 1.10681 L
.44025 1.10472 L
.4346 1.10228 L
.42911 1.09948 L
.42381 1.09635 L
.41872 1.09289 L
.41385 1.08911 L
.40923 1.08504 L
.40488 1.08068 L
.40081 1.07607 L
.39703 1.0712 L
.39357 1.06611 L
.39044 1.06081 L
.38764 1.05532 L
.38519 1.04967 L
.38311 1.04387 L
.38139 1.03796 L
.38005 1.03195 L
.37908 1.02587 L
.3785 1.01974 L
.37831 1.01358 L
.3785 1.00743 L
.37908 1.0013 L
.38005 .99521 L
.38139 .9892 L
.38311 .98329 L
.38519 .9775 L
.38764 .97184 L
.39044 .96636 L
.39357 .96106 L
.39703 .95596 L
.40081 .9511 L
.40488 .94648 L
.40923 .94212 L
.41385 .93805 L
.41872 .93428 L
.42381 .93082 L
.42911 .92768 L
.4346 .92489 L
.44025 .92244 L
.44604 .92035 L
.45196 .91864 L
.45797 .91729 L
.46405 .91633 L
.47018 .91575 L
.47634 .91556 L
.48249 .91575 L
.48862 .91633 L
.4947 .91729 L
.50071 .91864 L
.50663 .92035 L
.51242 .92244 L
.51807 .92489 L
.52356 .92768 L
.52886 .93082 L
.53395 .93428 L
.53882 .93805 L
.54344 .94212 L
.54779 .94648 L
.55187 .9511 L
.55564 .95596 L
.5591 .96106 L
.56224 .96636 L
.56503 .97184 L
.56748 .9775 L
.56956 .98329 L
.57128 .9892 L
.57262 .99521 L
.57359 1.0013 L
.57417 1.00743 L
.57436 1.01358 L
F
0 0 m
1 0 L
1 1.3833 L
0 1.3833 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{210.125, 290.625},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004k0001/b000`400?l00000o`00003oo`3oooll
0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3oooll
0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00@`3oool300000180oooo0`00
001K0?ooo`<000004P3oool300000140oooo1000001H0?ooo`002`3oool00`000000oooo0?ooo`0A
0?ooo`8000004P3oool2000000h0oooo1`00000>0?ooo`L00000503oool00`000000oooo0?ooo`0B
0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04P3oool7000000h0oooo1`00000=
0?ooo`P000003@3oool2000001<0oooo0P00000C0?ooo`030000003oool0oooo0180oooo00<00000
0?ooo`3oool02@3oool000X0oooo0P00000?0?ooo`D00000403oool4000000l0oooo1@00000@0?oo
o`D00000E`3oool500000100oooo1@00000?0?ooo`H000003P3oool500000100oooo1@00000?0?oo
o`<000004P3oool3000000T0oooo00090?ooo`8000003P3oool6000000l0oooo1@00000@0?ooo`80
00004`3oool2000005d0oooo0P00000C0?ooo`8000004`3oool2000000l0oooo1P00000?0?ooo`H0
00003P3oool400000140oooo100000070?ooo`00203oool3000000l0oooo0`00000B0?ooo`<00000
`P3oool300000180oooo0`00000?0?ooo`H000003`3oool6000000D0oooo00070?ooo`@00000403o
ool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0<80oooo00<000000?ooo`3oool04P3o
ool00`000000oooo0?ooo`0?0?ooo`@000004@3oool4000000H0oooo00060?ooo`@00000o`3ooolC
0?ooo`<000004P3oool3000000L0oooo00080?ooo`800000o`3ooolD0?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool01P3oool00?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03o
ool00?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03o
ool00?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03o
ool00?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03oool00740oooo1@00000@0?ooo`D00000
403oool4000009`0oooo001K0?ooo`030000003oool0oooo01<0oooo1@00000@0?ooo`D00000403o
ool4000001H0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo05T0oooo00140?ooo`030000003oool0oooo01<0oooo0`00000E0?ooo`<000004P3oool3
00000180oooo0`00000E0?ooo`<000004P3oool300000180oooo0`00001J0?ooo`00A03oool20000
01<0oooo1000000E0?ooo`<000004P3oool300000180oooo0P00000F0?ooo`@000004@3oool40000
0100oooo1@00000D0?ooo`030000003oool0oooo0480oooo00130?ooo`D000004P3oool4000001@0
oooo0`00000B0?ooo`<000004P3oool2000001D0oooo1000000A0?ooo`@000004@3oool4000001<0
oooo0`0000140?ooo`00;P3oool400000140oooo1P00000C0?ooo`8000005@3oool00`000000oooo
0?ooo`0B0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04`3oool2000001<0oooo
0P00000B0?ooo`<000004`3oool600000140oooo0`00000_0?ooo`002P3oool200000140oooo1000
000=0?ooo`P00000403oool4000001@0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`0B0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`0B0?ooo`030000003oool0oooo0140oooo0P00000C0?ooo`P000003@3oool7000000h0oooo1`00
000>0?ooo`8000002P3oool000L0oooo1000000>0?ooo`L000003P3oool600000=<0oooo1@00000?
0?ooo`H000003`3oool4000000L0oooo00050?ooo`D000003`3oool500000100oooo0P00003J0?oo
o`8000004@3oool400000100oooo1@0000050?ooo`001@3oool400000140oooo0`00003o0?ooo`<0
oooo0P00000@0?ooo`@000001P3oool000H0oooo0P00000B0?ooo`030000003oool0oooo0?l0oooo
5P3oool3000000H0oooo00060?ooo`030000003oool0oooo0?l0oooo:`3oool00`000000oooo0?oo
o`050?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3o
ooll0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3o
ooll0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3o
ooll0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00G@3oool200000180oooo1@00000@0?ooo`D0
0000403oool4000001@0oooo00<000000?ooo`3oool04@3oool2000001<0oooo0P00001M0?ooo`00
A`3oool00`000000oooo0?ooo`0A0?ooo`@000004P3oool500000100oooo1@00000@0?ooo`@00000
503oool300000140oooo1000000A0?ooo`@00000F`3oool004D0oooo0`00000C0?ooo`@000004`3o
ool300000180oooo0`00000B0?ooo`<00000503oool400000100oooo0`00000B0?ooo`<000005P3o
ool00`000000oooo0?ooo`130?ooo`00;`3oool00`000000oooo0?ooo`0D0?ooo`<000004`3oool3
000001<0oooo0`00000B0?ooo`<000004P3oool3000001@0oooo0`00000A0?ooo`<000004P3oool3
000001D0oooo0P0000150?ooo`00;`3oool3000001D0oooo0P00000C0?ooo`<000004`3oool30000
0180oooo0`00000B0?ooo`800000503oool300000180oooo0P00000B0?ooo`<000005@3oool40000
04@0oooo000I0?ooo`@000004@3oool5000001@0oooo0P00000D0?ooo`8000004`3oool300000180
oooo0`00000B0?ooo`800000503oool300000180oooo0P00000B0?ooo`8000005@3oool500000180
oooo0`00000_0?ooo`002P3oool2000000d0oooo2000000=0?ooo`L000004`3oool2000001<0oooo
0P00000D0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`0B0?ooo`8000004`3oool00`000000oooo0?ooo`0A0?ooo`8000004`3oool500000100oooo
1`00000>0?ooo`<000004P3oool00`000000oooo0?ooo`090?ooo`001P3oool5000000h0oooo1P00
000C0?ooo`@000004`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool04P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0180oooo00<0
00000?ooo`3oool04P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0180oooo0P00
000E0?ooo`D000003`3oool6000000l0oooo0P0000090?ooo`00103oool6000000l0oooo0P00003_
0?ooo`800000403oool500000100oooo0`0000070?ooo`001@3oool400000?l0oooo5P3oool20000
0140oooo1@0000050?ooo`001@3oool300000?l0oooo603oool00`000000oooo0?ooo`0@0?ooo`<0
00001P3oool000H0oooo00<000000?ooo`3oool0o`3oool[0?ooo`030000003oool0oooo00D0oooo
003o0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo
003o0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo
003o0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo00100?ooojX0o`00
D@3oool00400ooooZP3o001A0?ooo`00@03ooonZ0?l00540oooo00100?ooojX0o`00D@3oool00400
ooooZP3o001A0?ooo`00@03ooonZ0?l00540oooo00100?ooojX0o`00D@3oool00400ooooZP3o001A
0?ooo`00;`3oool2000000l0ooooZP3o000;0?ooo`800000A03oool000/0oooo00<000000?ooo`3o
ool08@3oool5000000`0ooooZP3o00080?ooo`D000008`3oool2000001<0oooo00<000000?ooo`3o
ool02@3oool000T0oooo0`00000=0?ooo`P000003P3oool7000000X0ooooZP3o00060?ooo`L00000
3P3oool7000000h0oooo1@00000@0?ooo`<000002@3oool000L0oooo0`00000?0?ooo`L000003`3o
ool4000000d0ooooZP3o00090?ooo`@000003`3oool600000100oooo1@00000?0?ooo`@000001`3o
ool000D0oooo1@00000@0?ooo`@000008P3ooonZ0?l001h0oooo1000000@0?ooo`@000004@3oool5
000000D0oooo00060?ooo`<000004@3oool2000002@0ooooZP3o000P0?ooo`8000004@3oool30000
0180oooo0`0000060?ooo`001`3oool00`000000oooo0?ooo`0f0?ooojX0o`00=03oool00`000000
oooo0?ooo`0A0?ooo`<000001P3oool00400ooooZP3o00190?ooo`030000003oool0oooo00D0oooo
00100?ooojX0o`00D@3oool00400ooooZP3o001A0?ooo`00@03ooonZ0?l00540oooo00100?ooojX0
o`00D@3oool00400ooooZP3o001A0?ooo`00@03ooonZ0?l00540oooo00100?ooojX0o`00D@3oool0
0400oooo603o001k0?oooaL0o`00D@3oool00400oooo603o001k0?oooaL0o`00D@3oool00400oooo
603o001k0?oooaL0o`00D@3oool00400oooo603o001k0?oooaL0o`00D@3oool00400oooo603o001k
0?oooaL0o`00D@3oool00400oooo603o001k0?oooaL0o`00D@3oool00400oooo603o001k0?oooaL0
o`00D@3oool00400oooo603o001k0?oooaL0o`00D@3oool00400oooo603o001k0?oooaL0o`00D@3o
ool00400oooo603o001k0?oooaL0o`00D@3oool00400oooo603o001k0?oooaL0o`00D@3oool00400
oooo603o001k0?oooaL0o`00D@3oool00400oooo603o001k0?oooaL0o`00D@3oool00400oooo603o
001k0?oooaL0o`00D@3oool00400oooo603o001k0?oooaL0o`00D@3oool000/0oooo00<000000?oo
o`3oool08@3oool3000000h0oooo603o000K0?ooo`030000003oool0oooo0180oooo00<000000?oo
o`3oool04P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0180oooo00<000000?oo
o`3oool02@3ooolG0?l000X0oooo0`00000S0?ooo`8000004`3oool00`000000oooo0?ooo`090?oo
o`002@3oool3000000h0oooo1`00000>0?ooo`L000002P3ooolH0?l000H0oooo1`00000>0?ooo`80
00004`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0140oooo0P00000B0?ooo`<0
00002`3ooolG0?l000H0oooo1`00000>0?ooo`L000003P3oool500000100oooo0P00000:0?ooo`00
1`3oool300000100oooo1P00000?0?ooo`D00000303ooolH0?l000L0oooo1P00000>0?ooo`<00000
4P3oool200000180oooo0P00000C0?ooo`800000403oool3000000d0oooo5`3o00080?ooo`D00000
3`3oool600000100oooo1@00000?0?ooo`<00000203oool000D0oooo1@00000A0?ooo`<000004@3o
ool2000000l0oooo603o00090?ooo`<000003`3oool300000140oooo0`00000B0?ooo`8000004P3o
ool3000000h0oooo1@00000=0?oooaL0o`002`3oool200000140oooo0`00000A0?ooo`@00000403o
ool4000000L0oooo00060?ooo`<000004P3oool00`000000oooo0?ooo`0R0?oooaP0o`002`3oool0
0`000000oooo0?ooo`0>0?ooo`<00000403oool300000180oooo0`00000A0?ooo`<000003`3oool3
000000h0oooo5`3o000P0?ooo`030000003oool0oooo0100oooo0`00000A0?ooo`@000001P3oool0
00L0oooo00<000000?ooo`3oool0=P3ooolH0?l001`0oooo1000000?0?ooo`@00000403oool40000
0100oooo1000000@0?ooo`030000003oool0oooo00d0oooo5`3o000d0?ooo`030000003oool0oooo
0100oooo0`0000070?ooo`00@03ooolH0?l001`0oooo0P00000A0?ooo`@00000403oool400000100
oooo1000000P0?oooaL0o`00B03oool00`000000oooo0?ooo`060?ooo`00@03ooolH0?l007/0oooo
5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A
0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00
@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH
0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0
oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o
001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?oo
o`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03o
oolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l0
07/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo
5`3o001A0?ooo`00@03ooolH0?l002l0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?oo
o`0c0?oooaL0o`00D@3oool000/0oooo00<000000?ooo`3oool04@3oool2000000h0oooo0`00000>
0?oooaP0o`00203oool4000000h0oooo00<000000?ooo`3oool04P3oool2000001<0oooo0P00000E
0?ooo`030000003oool0oooo0100oooo0`00000:0?oooaL0o`002@3oool3000002<0oooo0P00000D
0?ooo`030000003oool0oooo00T0oooo00090?ooo`<00000403oool4000000l0oooo1`00000:0?oo
oaP0o`00103oool8000000l0oooo0P00000B0?ooo`8000004`3oool2000001@0oooo00<000000?oo
o`3oool03@3oool6000000/0oooo5`3o00050?ooo`L000003P3oool8000000h0oooo0`00000B0?oo
o`8000002P3oool000L0oooo1000000?0?ooo`D00000403oool5000000`0oooo603o00060?ooo`D0
0000403oool400000100oooo0`00000A0?ooo`<000004P3oool3000000l0oooo1@00000<0?oooaL0
o`001`3oool5000000l0oooo1`00000?0?ooo`@000004@3oool3000000P0oooo00060?ooo`@00000
403oool400000140oooo0P00000?0?oooaP0o`002@3oool200000140oooo1@00000?0?ooo`<00000
403oool400000100oooo0`00000A0?ooo`8000003P3ooolG0?l000X0oooo0P00000A0?ooo`@00000
4@3oool5000000l0oooo100000070?ooo`001`3oool300000140oooo0P00000S0?oooaP0o`00703o
ool400000100oooo0`00000?0?ooo`D000003`3oool400000140oooo00<000000?ooo`3oool03@3o
oolG0?l001l0oooo0P00000B0?ooo`<00000403oool5000000H0oooo00070?ooo`8000004P3oool0
0`000000oooo0?ooo`0R0?oooaP0o`007@3oool00`000000oooo0?ooo`0@0?ooo`@000003P3oool5
00000100oooo0P00000R0?oooaL0o`00=03oool00`000000oooo0?ooo`0@0?ooo`<000001`3oool0
00P0oooo00<000000?ooo`3oool0=@3ooolH0?l00300oooo0P00000B0?ooo`<000004@3oool00`00
0000oooo0?ooo`0P0?oooaL0o`00A`3oool00`000000oooo0?ooo`070?ooo`00@03ooolH0?l007/0
oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o
001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?oo
o`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03o
oolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l0
07/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo
5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A
0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00
@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH
0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l002l0
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01`0
oooo5`3o001A0?ooo`00@03ooolH0?l001T0oooo00<000000?ooo`3oool04`3oool2000001@0oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0M0?oooaL0o`00D@3oool000/0oooo00<0
00000?ooo`3oool04P3oool2000001h0oooo603o00020?ooo`P00000403oool2000001<0oooo0P00
000C0?ooo`8000004`3oool3000001<0oooo100000080?oooaL0o`00203oool4000000h0oooo0P00
000C0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool02@3oool000X0oooo0P00000A
0?ooo`@000003`3oool7000000T0oooo603o00030?ooo`L00000403oool400000140oooo0`00000B
0?ooo`8000004P3oool300000100oooo1`0000090?oooaL0o`00103oool8000000l0oooo1000000A
0?ooo`8000004`3oool2000000X0oooo00080?ooo`<00000403oool500000100oooo1P00000:0?oo
oaP0o`001@3oool400000180oooo1000000@0?ooo`<000004P3oool300000100oooo1000000@0?oo
o`D000002`3ooolG0?l000H0oooo1@00000A0?ooo`D000003`3oool400000140oooo0`0000090?oo
o`001`3oool400000100oooo1000000A0?ooo`@00000303ooolH0?l000L0oooo0P00000B0?ooo`<0
00004@3oool400000100oooo1000000A0?ooo`8000004P3oool3000000`0oooo5`3o00090?ooo`80
00004P3oool400000100oooo1@00000?0?ooo`<000002@3oool000H0oooo1@00000A0?ooo`800000
4P3oool2000000h0oooo603o000L0?ooo`030000003oool0oooo0100oooo0P00000B0?ooo`@00000
4P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00`0oooo5`3o000N0?ooo`800000
4P3oool300000140oooo0`0000080?ooo`001`3oool300000180oooo00<000000?ooo`3oool08@3o
oolH0?l007/0oooo5`3o000O0?ooo`030000003oool0oooo0100oooo0`00000A0?ooo`@000001`3o
ool000T0oooo00<000000?ooo`3oool0=03ooolH0?l007/0oooo5`3o000c0?ooo`030000003oool0
oooo0100oooo0P0000090?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0
oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o
001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?oo
o`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03o
oolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l0
07/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo
5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A
0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l004H0oooo00<000000?ooo`3o
ool0<P3ooolG0?l00540oooo00100?oooaP0o`00;P3oool00`000000oooo0?ooo`0D0?ooo`800000
=03ooolG0?l00540oooo00100?oooaP0o`005P3oool00`000000oooo0?ooo`0E0?ooo`8000005@3o
ool2000001L0oooo00<000000?ooo`3oool06P3ooolG0?l00540oooo00100?oooaP0o`005P3oool3
000001D0oooo0P00000D0?ooo`<000005P3oool2000001`0oooo5`3o001A0?ooo`00@03ooolH0?l0
01L0oooo1000000C0?ooo`<000004`3oool3000001@0oooo0`00000M0?oooaL0o`00D@3oool000/0
oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0L0?oooaP0o`0000<0oooo00000000
00000P00000B0?ooo`D000004P3oool300000180oooo1000000C0?ooo`@000004@3oool8000000@0
oooo5`3o000I0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool04`3oool00`000000
oooo0?ooo`090?ooo`002P3oool2000001@0oooo00<000000?ooo`3oool04P3oool3000000P0oooo
603o0005000001<0oooo0`00000C0?ooo`@000004@3oool400000180oooo1@00000A0?ooo`L00000
1@3ooolG0?l000L0oooo0`00000@0?ooo`8000004`3oool2000001@0oooo00<000000?ooo`3oool0
2@3oool000T0oooo0`00000B0?ooo`<00000403oool6000000T0oooo603o000700000140oooo1000
000B0?ooo`<00000503oool2000001<0oooo0`00000B0?ooo`D000001`3ooolG0?l000<0oooo1`00
000@0?ooo`@000004P3oool2000001<0oooo0P00000:0?ooo`002@3oool300000140oooo0`00000A
0?ooo`D000002P3ooolH0?l00080oooo0P00000E0?ooo`030000003oool0oooo0400oooo00<00000
0?ooo`3oool04@3oool2000000T0oooo5`3o00050?ooo`@000004P3oool500000100oooo0`00000B
0?ooo`8000002P3oool000P0oooo0`00000A0?ooo`@000004P3oool2000000`0oooo603o001k0?oo
oaL0o`001`3oool2000001<0oooo0`00000A0?ooo`@000004@3oool3000000T0oooo00070?ooo`@0
00004P3oool2000001<0oooo00<000000?ooo`3oool02`3ooolH0?l007/0oooo5`3o000L0?ooo`<0
00004@3oool500000100oooo0`0000090?ooo`00203oool3000001<0oooo00<000000?ooo`3oool0
7`3ooolH0?l007/0oooo5`3o000M0?ooo`030000003oool0oooo0140oooo0`00000A0?ooo`@00000
203oool000X0oooo00<000000?ooo`3oool0<`3ooolH0?l007/0oooo5`3o000a0?ooo`030000003o
ool0oooo0140oooo0P00000:0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l0
07/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo
5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A
0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00
@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l007/0oooo5`3o001A0?ooo`00@03ooolH
0?l007/0oooo5`3o001A0?ooo`00@03ooolH0?l002d0oooo00<000000?ooo`3oool05`3oool00`00
0000oooo0?ooo`0a0?oooaL0o`00D@3oool00400oooo603o000C0?ooo`030000003oool0oooo01L0
oooo00<000000?ooo`3oool05P3oool2000001T0oooo00<000000?ooo`3oool05P3oool00`000000
o`000?l0000E0?l00540oooo00100?oooaP0o`004`3oool2000001P0oooo0P00000G0?ooo`800000
603oool2000001H0oooo0`00000G0?l00540oooo00100?oooaP0o`00503oool3000001H0oooo0P00
000F0?ooo`<000005P3oool3000001D0oooo1@00000G0?l00540oooo00100?oooaP0o`00503oool4
000001D0oooo0`00000E0?ooo`<000005@3oool4000001H0oooo0`0000000`3oool0o`000?l0000E
0?l000030000003oool0oooo04h0oooo000i0?ooo`030000003oool0oooo00@0oooo603o000D0?oo
o`D00000503oool3000001@0oooo1000000D0?ooo`D000005P3oool3000000030?ooo`3o0000o`00
01D0o`000P00001?0?ooo`00>03oool2000000H0oooo603o000E0?ooo`<000005@3oool4000001<0
oooo1000000E0?ooo`<000005@3oool2000000040?ooo`000000oooo0?oooaL0o`000`00001>0?oo
o`00=`3oool2000000L0oooo603o000E0?ooo`030000003oool0000001L0oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`0D0?ooo`030000003oool0000001@0oooo00<000000?ooo`3o
ool0103ooolG0?l000800000C`3oool00240oooo00<000000?ooo`3oool04P3oool3000000L0oooo
603o0002000001H0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`050?oooaL0o`00
00<000000?ooo`0000005`3oool00`000000oooo0?ooo`0d0?ooo`002`3oool00`000000oooo0?oo
o`0B0?ooo`8000004`3oool4000000L0oooo603o00020?ooo`030000003oool0oooo01@0oooo00<0
00000?ooo`3oool04`3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01@0oooo00<0
00000?ooo`3oool04`3oool00`000000oooo0?ooo`060?oooaL0o`000`3oool00`000000oooo0?oo
o`0D0?ooo`800000503oool00`000000oooo0?ooo`0N0?ooo`002`3oool00`000000oooo0?ooo`0B
0?ooo`8000004P3oool4000000P0oooo603o00030?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool04P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01<0oooo00<00000
0?ooo`3oool04`3oool00`000000oooo0?ooo`070?oooaL0o`00103oool00`000000oooo0?ooo`0C
0?ooo`800000503oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00T0oooo000:0?oo
o`8000004`3oool3000001<0oooo0`00000A0?ooo`030000003oool0oooo0100oooo0P00000D0?oo
o`030000003oool0oooo01<0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo0180oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0180oooo0`00000C0?ooo`8000004`3oool00`000000oooo0?ooo`090?oo
o`002P3oool200000180oooo0`00000C0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool04P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool04P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool04P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo0`00000C0?ooo`80
00004P3oool2000000/0oooo000:0?ooo`<000004@3oool300000><0oooo1000000A0?ooo`<00000
4P3oool2000000/0oooo000:0?ooo`<00000403oool400000><0oooo0P00000C0?ooo`@000004@3o
ool3000000X0oooo00090?ooo`@000004P3oool200000?P0oooo1000000@0?ooo`@000002P3oool0
00T0oooo1000003o0?ooo`d0oooo0P00000B0?ooo`@000002P3oool00?l0oooo?03oool00?l0oooo
?03oool00:40oooo00<000000?ooo`3oool0U`3oool00:40oooo00<000000?ooo`3oool0U`3oool0
0880oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`2G0?ooo`00PP3oool2000001`0
oooo0P00002I0?ooo`00PP3oool2000001`0oooo0P00002I0?ooo`00PP3oool3000001X0oooo1000
002H0?ooo`00PP3oool3000001X0oooo1000000I0?ooo`030000003oool0oooo07`0oooo001Y0?oo
o`030000003oool0oooo01H0oooo1000000J0?ooo`<00000603oool2000007h0oooo001Y0?ooo`80
00005`3oool3000001/0oooo00<000000?ooo`3oool05P3oool3000007l0oooo001E0?ooo`030000
003oool0oooo0180oooo0`00000G0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0
5P3oool4000001D0oooo00<000000?ooo`3oool0I`3oool005D0oooo0P00000C0?ooo`@000005P3o
ool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01D0oooo1@00000D0?ooo`030000003o
ool0oooo06P0oooo001F0?ooo`8000004P3oool5000001H0oooo00<000000?ooo`3oool05`3oool0
0`000000oooo0?ooo`0F0?ooo`<000004`3oool300000140oooo00<000000?ooo`3oool0EP3oool0
04@0oooo00<000000?ooo`3oool03`3oool300000180oooo0`00000G0?ooo`030000003oool0oooo
01L0oooo00<000000?ooo`3oool05P3oool00`000000oooo0000000B0?ooo`<000004@3oool20000
05P0oooo00140?ooo`800000403oool400000140oooo00<000000?ooo`0000005`3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04@3oool400000100
oooo0P00001I0?ooo`00A03oool3000000l0oooo1@00000B0?ooo`030000003oool0oooo01D0oooo
00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01<0oooo
0P00000A0?ooo`8000004P3oool00`000000oooo0?ooo`140?ooo`00<P3oool00`000000oooo0?oo
o`0?0?ooo`@000003`3oool3000001@0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?oo
o`0E0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04`3oool200000100oooo0`00
000A0?ooo`800000AP3oool00380oooo0P00000A0?ooo`@000003P3oool00`000000oooo0000000E
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0E
0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool03`3oool300000140oooo0`000016
0?ooo`00<P3oool200000140oooo0`00000B0?ooo`030000003oool0oooo0180oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01D0oooo00<000000?oo
o`3oool04P3oool00`000000oooo0?ooo`0A0?ooo`8000004@3oool2000001<0oooo00<000000?oo
o`3oool0<@3oool001h0oooo00<000000?ooo`3oool04@3oool300000100oooo00<000000?ooo`00
00004`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3o
ool0403oool00`000000oooo0?ooo`0@0?ooo`<000004P3oool2000003<0oooo00090?ooo`030000
003oool0oooo0180oooo0P00000B0?ooo`<000004P3oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo0180oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo01<0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo00l0oooo1000000B0?ooo`800000503oool00`000000oooo0?ooo`0L0?ooo`002@3o
ool2000001<0oooo0P00000B0?ooo`@000004P3oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo0180oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0D0?ooo`030000003o
ool0oooo01<0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo0140oooo0`00000A0?ooo`<000004`3oool2000001<0oooo00<000000?ooo`3oool0203o
ool000T0oooo0`00000B0?ooo`<000004@3oool2000001@0oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04`3oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo0`00000B0?ooo`<000004P3oool2
000000X0oooo00090?ooo`<000004P3oool3000001<0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04`3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04@3oool40000
0180oooo0P00000B0?ooo`<000002P3oool000X0oooo0`00000A0?ooo`@000004P3oool00`000000
oooo0?ooo`0B0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04P3oool00`000000
oooo0?ooo`0B0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04P3oool00`000000
oooo0?ooo`0B0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04@3oool00`000000
oooo0?ooo`0B0?ooo`<000004@3oool300000180oooo0P00000;0?ooo`002P3oool400000100oooo
0P00003h0?ooo`@000004@3oool3000000/0oooo000:0?ooo`<00000o`3oool>0?ooo`800000403o
ool4000000/0oooo000:0?ooo`030000003oool0oooo0?l0oooo8P3oool2000000/0oooo003o0?oo
oc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003o0?oo
oc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003:0?oo
o`@00000K@3oool00940oooo20000?l]0?ooo`L00000KP3oool008`0oooo4`000?lX0?ooo`D00000
K`3oool005l0oooo0`00000X0?oooaH0003o9`3oool300000740oooo001P0?ooo`H00000803ooolN
0000ob40oooo0P0000000`3oool000000?ooo`1a0?ooo`00H@3oool5000001l0oooo8@000?lN0?oo
o`030000003oool0oooo07@0oooo001S0?ooo`8000007P3ooolT0000oa/0oooo0P00000@0?ooo`80
0000I@3oool00540oooo0P00000A0?ooo`<000006`3ooolW0000oaL0oooo0P00000?0?ooo`@00000
IP3oool00580oooo1000000A0?ooo`800000603ooolY0000oa@0oooo0P00000?0?ooo`D00000403o
ool2000005D0oooo00110?ooo`800000403oool500000140oooo00<000000?ooo`3oool0503ooolZ
0000oa80oooo0P00000B0?ooo`@000003@3oool4000005H0oooo000_0?ooo`030000003oool0oooo
0100oooo1000000?0?ooo`8000004`3oool2000001<0oooo;@000?l?0?ooo`030000003oool0oooo
0140oooo1000000<0?ooo`D000004P3oool2000004<0oooo000L0?ooo`030000003oool0oooo0140
oooo0P00000A0?ooo`D000003P3oool3000001<0oooo0P00000@0?ooobl0003o303oool200000180
oooo0P0000020?ooo`030000003oool0oooo00`0oooo1000000?0?ooo`@000004`3oool00`000000
oooo0?ooo`0^0?ooo`00203oool00`000000oooo0?ooo`0A0?ooo`<000004@3oool4000000l0oooo
1000000B0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0303oool`0000o`X0oooo
0P00000B0?ooo`8000004`3oool3000000h0oooo1@00000B0?ooo`8000004`3oool00`000000oooo
0?ooo`0K0?ooo`002@3oool200000180oooo1000000@0?ooo`@000003`3oool3000001<0oooo0P00
000C0?ooo`8000002`3ooolb0000o`L0oooo0P00000C0?ooo`030000003oool0oooo0140oooo0P00
00000`3oool000000?ooo`0?0?ooo`@00000403oool300000180oooo0P00000E0?ooo`030000003o
ool0oooo00H0oooo00090?ooo`@00000403oool5000000l0oooo0`00000A0?ooo`040000003oool0
0000000001<0oooo0P00000C0?ooo`8000002@3ooolc0000o`@0oooo0P00000C0?ooo`8000004`3o
ool00`000000oooo0?ooo`0A0?ooo`@000003`3oool500000100oooo1000000C0?ooo`8000002@3o
ool000X0oooo1000000@0?ooo`<000004@3oool00`000000oooo0000000D0?ooo`030000003oool0
oooo0180oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`050?oooc@0003o0`3oool0
0`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04@3oool0
10000000oooo0?ooo`00000A0?ooo`<00000403oool400000180oooo0`00000:0?ooo`002P3oool3
00000140oooo0`00001I0?ooocH0003oEP3oool00`000000oooo0?ooo`0@0?ooo`<00000403oool5
000000X0oooo000;0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0F03ooolg0000
ofT0oooo00<000000?ooo`3oool0403oool3000000/0oooo001i0?ooocP0003oO@3oool00`000000
oooo0?ooo`0:0?ooo`00N@3oooli0000ohT0oooo001i0?ooocT0003oR@3oool007T0oooo>@000?n9
0?ooo`00N03ooolj0000ohT0oooo001h0?oooc/0003oR03oool007L0oooo?0000?n80?ooo`00M`3o
ooll0000ohP0oooo001g0?ooocd0003oQ`3oool007L0oooo?@000?n70?ooo`00M`3ooolm0000ohL0
oooo001f0?oooch0003oQ`3oool007H0oooo?P000?n70?ooo`00MP3oooln0000ohL0oooo001f0?oo
och0003oQ`3oool007H0oooo?P000?n70?ooo`00M`3ooolm0000ohL0oooo001g0?ooocd0003oQ`3o
ool007L0oooo?@000?n70?ooo`00M`3oooll0000ohP0oooo001g0?oooc`0003oR03oool007L0oooo
?0000?n80?ooo`00N03ooolk0000ohP0oooo001h0?ooocX0003oR@3oool000L0oooo0`00000A0?oo
o`<00000403oool7000000l0oooo1000000a0?ooocT0003o;@3oool3000000l0oooo1`00000@0?oo
o`<000004@3oool3000001`0oooo00080?ooo`H000003P3oool6000000d0oooo1P00000;0?ooo`P0
00003P3oool200000240oooo>@000?lM0?ooo`030000003oool0oooo00d0oooo1`00000;0?ooo`H0
00003@3oool6000000h0oooo1P00000B0?ooo`<00000203oool000T0oooo1@00000?0?ooo`D00000
3`3oool4000000d0oooo200000090?ooo``000004`3oool2000000D0oooo>0000?l00`3oool00000
0000000C0?ooo`@00000103oool3000000X0oooo2000000<0?ooo`D000003`3oool5000000l0oooo
1@00000?0?ooo`H000002@3oool000/0oooo0P00000B0?ooo`8000004P3oool200000100oooo0P00
000:0?ooo`P000005P3oool3000000P0oooo=`000?l30?ooo`@000004`3oool9000000/0oooo0P00
000A0?ooo`030000003oool0oooo0100oooo0P00000B0?ooo`8000004@3oool4000000/0oooo000<
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0<P3oool4000000h0oooo00<00000
0?ooo`3oool00P3oool3000000/0oooo=P000?l80?ooo`@000001@3oool00`000000oooo0?ooo`0:
0?ooo`P00000;`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<00000
0?ooo`3oool02`3oool006D0oooo0`000000103oool0000000000000000?0?ooocD0003o303oool4
000000040?ooo`0000000000000007L0oooo001T0?ooo`D000004P3ooold0000oa40oooo1P00001e
0?ooo`00HP3oool7000001<0oooo<P000?lB0?ooo`L00000M03oool00640oooo1000000H0?oooc40
003oS@3oool007d0oooo<0000?n>0?ooo`00O`3oool]0000ohl0oooo00200?ooob/0003oT03oool0
0800oooo:P000?nA0?ooo`00PP3ooolW0000oi80oooo00230?ooob@0003oU03oool008D0oooo8@00
0?nE0?ooo`00QP3ooolN0000oiL0oooo00280?oooaX0003oV@3oool008X0oooo5P000?nK0?ooo`00
SP3oool?0000oih0oooo002A0?ooo`P0003oXP3oool00?l0oooo?03oool00?l0oooo?03oool00?l0
oooo?03oool00?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03oool00?l0
oooo?03oool00200oooo0P00000C0?ooo`030000003oool0oooo0;P0oooo00<000000?ooo`3oool0
4@3oool2000003D0oooo000;0?ooo`<000003`3oool500000140oooo0`00000B0?ooo`030000003o
ool0oooo0980oooo00<000000?ooo`3oool0403oool300000140oooo1@00000?0?ooo`<000004@3o
ool3000000`0oooo00070?ooo`L000003@3oool7000000l0oooo1@00000@0?ooo`@000004@3oool0
10000000oooo0?ooo`00000D0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04P3o
ool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04P3o
ool400000100oooo1@00000?0?ooo`L000003@3oool7000000d0oooo1`0000080?ooo`002@3oool5
000000l0oooo1@00000=0?ooo`L000003P3oool600000100oooo1000000D0?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0
oooo0180oooo0`00000A0?ooo`D000003`3oool7000000d0oooo1000000@0?ooo`D000003`3oool5
000000X0oooo000<0?ooo`8000004P3oool200000200oooo2000000>0?ooo`D00000503oool00`00
0000oooo0?ooo`0C0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04`3oool00`00
0000oooo0?ooo`0B0?ooo`<00000403oool7000003D0oooo0P00000B0?ooo`8000003@3oool005L0
oooo1P00000A0?ooo`030000003oool0000001H0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo0180oooo1@00000B0?ooo`@00000E`3oool005H0oooo
1000000C0?ooo`<000005`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`0000004`3oool4000006`0oooo001]0?ooo`@000005@3oool00`000000oooo0?oo
o`0E0?ooo`030000003oool0oooo01@0oooo0`00000F0?ooo`800000J`3oool006`0oooo1000000F
0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool04P3oool500000880oooo001/0?oo
o`<000005@3oool2000001T0oooo00<000000?ooo`3oool04`3oool400000880oooo001[0?ooo`80
00005`3oool3000001P0oooo0`00000E0?ooo`<00000P@3oool006/0oooo00<000000?ooo`3oool0
5P3oool2000001P0oooo1000000F0?ooo`800000P@3oool008@0oooo0P00000H0?ooo`@00000603o
ool00`000000oooo0?ooo`1n0?ooo`00P`3oool2000001X0oooo0`00002I0?ooo`00P`3oool20000
01X0oooo0`00002I0?ooo`00P`3oool00`000000oooo0?ooo`0J0?ooo`800000V@3oool008<0oooo
00<000000?ooo`3oool06P3oool2000009T0oooo002Q0?ooo`030000003oool0oooo09L0oooo003o
0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003o
0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003o0?oooc`0oooo003o
0?oooc`0oooo000=0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo0:@0oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0303oool000/0
oooo0`00000A0?ooo`<000004P3oool200000180oooo0`00000B0?ooo`030000003oool0oooo07`0
oooo00<000000?ooo`3oool0403oool300000180oooo0`00000A0?ooo`<000004@3oool3000000`0
oooo00090?ooo`H000003P3oool6000000l0oooo1@00000@0?ooo`@000004@3oool3000001@0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0180oooo
00<000000?ooo`3oool04P3oool200000180oooo0`00000A0?ooo`D000003`3oool6000000h0oooo
1P00000>0?ooo`H000002P3oool000L0oooo2000000=0?ooo`L000003P3oool600000100oooo1@00
000@0?ooo`@000004@3oool2000001D0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`0B0?ooo`<000004P3oool300000100oooo1@00000?0?ooo`L000003P3oool7000000d0oooo2000
000<0?ooo`P00000203oool00300oooo1000000B0?ooo`@000004@3oool400000180oooo1000000A
0?ooo`8000005@3oool300000140oooo1000000A0?ooo`@000004@3oool4000001<0oooo10000015
0?ooo`00A@3oool2000001@0oooo0`00000B0?ooo`@000004P3oool400000180oooo1000000B0?oo
o`<000004P3oool400000180oooo0`00001K0?ooo`00FP3oool2000001@0oooo0`00000C0?ooo`<0
00004`3oool4000001<0oooo0P00000D0?ooo`800000503oool2000005X0oooo001J0?ooo`030000
003oool0oooo01<0oooo0P00000D0?ooo`<00000503oool3000001<0oooo0`00000E0?ooo`030000
003oool0oooo06d0oooo001_0?ooo`8000005@3oool2000001D0oooo0P00000E0?ooo`800000Q@3o
ool006l0oooo00<000000?ooo`3oool0503oool2000001D0oooo0P00000F0?ooo`030000003oool0
oooo08<0oooo00260?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0VP3oool009h0
oooo00<000000?ooo`3oool0VP3oool00?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03oool0
0?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03oool0
0?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03oool0
0?l0oooo?03oool00?l0oooo?03oool00?l0oooo?03oool000d0oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`3P0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04@3o
ool00`000000oooo0?ooo`0<0?ooo`00303oool200000180oooo0`00000B0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`1V0?ooo`030000003oool0
oooo0180oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@0?ooo`<000004P3oool2
00000180oooo0`00000<0?ooo`002P3oool500000100oooo1000000A0?ooo`<000004@3oool40000
0140oooo0`00000B0?ooo`8000004P3oool2000002X0oooo0P00000B0?ooo`8000004P3oool30000
0180oooo0`00000A0?ooo`@00000403oool400000100oooo1@00000;0?ooo`002@3oool600000100
oooo1@00000?0?ooo`D00000403oool500000100oooo1000000A0?ooo`@00000403oool500000100
oooo1000000A0?ooo`@000004@3oool400000100oooo1@00000@0?ooo`@00000403oool500000100
oooo1@00000?0?ooo`L000002@3oool000P0oooo1000000B0?ooo`@000004@3oool400000100oooo
1@00000@0?ooo`@000004P3oool300000140oooo1000000A0?ooo`@000004P3oool300000180oooo
0`00000A0?ooo`@000004@3oool400000140oooo1000000A0?ooo`@000004P3oool4000000P0oooo
000M0?ooo`8000004`3oool300000180oooo0`00000B0?ooo`<000004P3oool300000180oooo1000
000B0?ooo`<000004`3oool2000001<0oooo0P00000C0?ooo`<000004P3oool2000001@0oooo0P00
000C0?ooo`8000007@3oool00340oooo0P00000C0?ooo`8000004`3oool2000001@0oooo0`00000C
0?ooo`8000004`3oool2000001@0oooo0`00000B0?ooo`800000503oool2000001@0oooo00<00000
0?ooo`3oool0A@3oool005/0oooo00<000000?ooo`3oool04`3oool2000001@0oooo0P00000C0?oo
o`8000005@3oool2000001<0oooo0P00000E0?ooo`030000003oool0oooo05T0oooo001a0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0D0?ooo`03
0000003oool0oooo0180oooo00<000000?ooo`3oool0K`3oool008L0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`2K0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3oooll0?oo
o`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3oooll0?ooo`00o`3oooll0?oo
o`00o`3oooll0?ooo`00o`3oooll0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 314.188}, {434.938, 0}} -> {40.5625, 28.6188, \
0.194818, 0.194818}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "ExerciseMain"],

Cell[CellGroupData[{

Cell["Charge distribution on parallel-plate capacitor", "ExerciseTitle"],

Cell[TextData[{
  "Using the parallel-plate capacitor from ",
  StyleBox["pde relaxation.nb",
    FontSlant->"Italic"],
  ", estimate and plot the surface charge densities on both sides of the \
upper plate.  This will take care because the normal component of the \
electric field is discontinuous at the plates, which causes some numerical \
difficulties.  It will help to plot ",
  Cell[BoxData[
      \(TraditionalForm\`E\_y[x, y]\)]],
  " as a function of ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  " for ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " near the middle of the plate first.",
  "  Then compare the total charge density with the simple approximation that \
the field is uniform within the capacitor and vanishes elsewhere."
}], "ExerciseText"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Capacitance for parallel plates with ",
  Cell[BoxData[
      \(TraditionalForm\`w/d = 10\)]]
}], "ExerciseTitle"],

Cell[TextData[{
  "Estimate the capacitance per unit length for a very long parallel-pate \
capacitor whose ratio of width to separation is ",
  Cell[BoxData[
      \(TraditionalForm\`w/d = 10\)]],
  ".  Compare with the simple approximation that neglects edge effects and \
fringe fields. "
}], "ExerciseText"]
}, Closed]],

Cell[CellGroupData[{

Cell["Concentric squares", "ExerciseTitle"],

Cell[TextData[{
  "Suppose that a square metallic plate with sides of length 40 units is \
maintained at constant unit potential, ",
  Cell[BoxData[
      \(TraditionalForm\`\[Phi] = 1\)]],
  ".  This plate is centered within another square with sides of length 100 \
units that is cut from a metallic plate that is held at ",
  Cell[BoxData[
      \(TraditionalForm\`\[Phi] = 0\)]],
  ".  Determine and display the two-dimensional potential and field in the \
space between these conductors, which are effectively infinite in the \
transverse direction. "
}], "ExerciseText"]
}, Closed]],

Cell[CellGroupData[{

Cell["Poisson equation", "ExerciseTitle"],

Cell["\<\
Generalize the relaxation function to solve the Poisson equation\
\>", "ExerciseText"],

Cell[BoxData[
    \(TraditionalForm\`\[PartialD]\^2 \[Phi]\/\[PartialD]x\^2 + \
\[PartialD]\^2 \[Phi]\/\[PartialD]y\^2 \[Equal] \(-\[Rho]\)\)], \
"DisplayFormula",
  TextAlignment->Left,
  TextJustification->0],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`\[Rho][x, y]\)]],
  " is the charge density and we employ units with ",
  Cell[BoxData[
      \(TraditionalForm\`\[CurlyEpsilon]\_0 \[Rule] 1\)]],
  "."
}], "ExerciseText"]
}, Closed]],

Cell[CellGroupData[{

Cell["Electrostatic focusing by cylindrical anodes ", "ExerciseTitle"],

Cell[TextData[{
  "Two hollow cylindrical anodes are used for focusing an electron beam that \
passes through small holes on the central axis.  The first cylinder has \
negative potential and second an opposite positive potential; it is useful to \
take ",
  Cell[BoxData[
      \(TraditionalForm\`\[Phi]\_0 = \(\[PlusMinus]1\)\)]],
  ".  The separation between the two cylinders is negligibly small, so that \
the potential vanishes on the central cross section of the device.  Let the \
radius of each cylinder be 50 units and the length of each cylinder also be \
50 units, so that a cross section through the enclosed space is represented \
by a grid with 100\[Times]100 points.  Laplace's equation in cylindrical \
coordinates takes the form"
}], "ExerciseText"],

Cell[BoxData[
    \(TraditionalForm\`\[PartialD]\^2 \[Phi]\/\[PartialD]r\^2 + \(1\/r\) \
\[PartialD]\[Phi]\/\[PartialD]r + \[PartialD]\^2 \[Phi]\/\[PartialD]z\^2 \
\[Equal] 0\)], "DisplayFormula",
  TextAlignment->Left,
  TextJustification->0],

Cell["\<\
a) Formulate the finite-difference equation that represents Laplace's \
equation in cylindrical coordinates.\
\>", "ExerciseText"],

Cell["b) Write an SOR program to solve for the potential.", "ExerciseText"],

Cell["\<\
c) Display the potential and the electric field.  [Be careful to evaluate the \
gradient in cylindrical coordinates properly.]  Demonstrate that this device \
does tend to focus an electron beam near its symmetry axis.\
\>", "ExerciseText"]
}, Closed]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"5.2 for Microsoft Windows",
ScreenRectangle->{{0, 1400}, {0, 952}},
WindowToolbars->"EditBar",
WindowSize->{1392, 912},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowSelection->True,
ShowCellLabel->False,
Magnification->1.5,
StyleDefinitions -> Notebook[{
  
  Cell[CellGroupData[{
  Cell["Style Definitions", "Subtitle"],
  
  Cell["\<\
Modify the definitions below to change the default appearance of all cells in \
a given style.  Make modifications to any definition using commands in the \
Format menu.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell["Style Environment Names", "Section"],
  
  Cell[StyleData[All, "Working"],
    PageWidth->WindowWidth,
    ScriptMinSize->9],
  
  Cell[StyleData[All, "Presentation"],
    PageWidth->WindowWidth,
    ScriptMinSize->12,
    FontSize->16],
  
  Cell[StyleData[All, "Condensed"],
    PageWidth->PaperWidth,
    CellBracketOptions->{"Margins"->{1, 1},
    "Widths"->{0, 5}},
    ScriptMinSize->7,
    FontSize->10],
  
  Cell[StyleData[All, "Printout"],
    PageWidth->PaperWidth,
    ScriptMinSize->7,
    FontSize->10,
    PrivateFontOptions->{"FontType"->"Outline"}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Notebook Options", "Section"],
  
  Cell["\<\
The options defined for the style below will be used at the Notebook level.\
\>", "Text"],
  
  Cell[StyleData["Notebook"],
    PageHeaders->{{Cell[ 
            TextData[ {
              CounterBox[ "Page"]}], "PageNumber"], None, Cell[ 
            TextData[ {
              ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
            TextData[ {
              ValueBox[ "FileName"]}], "Header"], None, Cell[ 
            TextData[ {
              CounterBox[ "Page"]}], "PageNumber"]}},
    PageFooters->{{Cell[ 
            TextData[ {"Essential Mathematica for Students of Science"}], 
            "Footer"], None, Cell[ 
            TextData[ {"\[Copyright] James J. Kelly, 1998"}], "Footer"]}, {
        Cell[ 
            TextData[ {"Essential Mathematica for Students of Science"}], 
            "Footer"], None, Cell[ 
            TextData[ {"\[Copyright] James J. Kelly, 1998"}], "Footer"]}},
    PageHeaderLines->{True, True},
    PageFooterLines->{True, True},
    PrintingOptions->{"FirstPageHeader"->False,
    "FacingPages"->True},
    CellFrameLabelMargins->6,
    StyleMenuListing->None]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Headings", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Title"],
    ShowCellBracket->True,
    CellMargins->{{0, 0}, {0, 0}},
    CellGroupingRules->"TitleGrouping",
    PageBreakBelow->False,
    LineSpacing->{1, 0},
    CounterIncrements->"Title",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
        "Subtitle", 0}, {"Subsubtitle", 0}},
    FontSize->34,
    FontColor->GrayLevel[1],
    Background->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["Title", "Presentation"],
    CellMargins->{{0, 0}, {0, 0}},
    LineSpacing->{1, 0},
    FontSize->44],
  
  Cell[StyleData["Title", "Condensed"],
    CellMargins->{{0, 0}, {0, 0}},
    FontSize->24],
  
  Cell[StyleData["Title", "Printout"],
    CellMargins->{{0, 0}, {0, 0}},
    FontSize->24,
    FontTracking->"Plain",
    FontColor->GrayLevel[0],
    Background->None]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subtitle"],
    CellMargins->{{0, 0}, {0, 0}},
    PageBreakBelow->False,
    LineSpacing->{1, 3},
    ParagraphIndent->-96,
    CounterIncrements->"Subtitle",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
        "Subsubtitle", 0}},
    FontFamily->"Helvetica",
    FontSize->18,
    FontColor->GrayLevel[1],
    Background->RGBColor[0.2, 0.700008, 0.700008]],
  
  Cell[StyleData["Subtitle", "Presentation"],
    CellMargins->{{0, 0}, {0, 0}},
    LineSpacing->{1, 6},
    ParagraphIndent->-157,
    FontSize->30],
  
  Cell[StyleData["Subtitle", "Condensed"],
    CellMargins->{{0, 0}, {0, 0}},
    ParagraphIndent->-78,
    FontSize->14],
  
  Cell[StyleData["Subtitle", "Printout"],
    CellMargins->{{0, 0}, {0, 0}},
    ParagraphIndent->-85,
    FontSize->16,
    FontColor->GrayLevel[0],
    Background->None]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsubtitle"],
    ShowCellBracket->True,
    CellMargins->{{10, 4}, {30, 10}},
    PageBreakBelow->False,
    CounterIncrements->"Subsubtitle",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}},
    FontFamily->"Helvetica",
    FontSize->14,
    FontSlant->"Italic"],
  
  Cell[StyleData["Subsubtitle", "Presentation"],
    ShowCellBracket->True,
    CellMargins->{{8, 10}, {40, 20}},
    LineSpacing->{1, 0},
    FontSize->24],
  
  Cell[StyleData["Subsubtitle", "Condensed"],
    ShowCellBracket->True,
    CellMargins->{{8, 10}, {12, 8}},
    FontSize->12],
  
  Cell[StyleData["Subsubtitle", "Printout"],
    ShowCellBracket->True,
    CellMargins->{{9, 10}, {50, 10}},
    FontSize->14]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Section"],
    CellFrame->{{0, 0}, {0, 1}},
    CellDingbat->None,
    CellMargins->{{12, Inherited}, {4, 24}},
    CellGroupingRules->{"SectionGrouping", 30},
    PageBreakBelow->False,
    CellFrameMargins->6,
    CounterIncrements->"Section",
    CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->16,
    FontWeight->"Bold",
    FontColor->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["Section", "Presentation"],
    CellMargins->{{10, 10}, {8, 32}},
    LineSpacing->{1, 2},
    FontSize->24,
    FontTracking->"Condensed"],
  
  Cell[StyleData["Section", "Condensed"],
    CellMargins->{{9, 0}, {2, 50}},
    FontSize->14],
  
  Cell[StyleData["Section", "Printout"],
    CellMargins->{{9, 0}, {2, 50}},
    FontSize->14,
    FontTracking->"Plain",
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsection"],
    CellMargins->{{12, Inherited}, {8, 20}},
    CellGroupingRules->{"SectionGrouping", 40},
    PageBreakBelow->False,
    CounterIncrements->"Subsection",
    CounterAssignments->{{"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->13,
    FontWeight->"Bold"],
  
  Cell[StyleData["Subsection", "Presentation"],
    CellMargins->{{11, 10}, {8, 32}},
    LineSpacing->{1, 0},
    FontSize->22],
  
  Cell[StyleData["Subsection", "Condensed"],
    CellMargins->{{8, Inherited}, {2, 12}},
    FontSize->12],
  
  Cell[StyleData["Subsection", "Printout"],
    CellMargins->{{9, 0}, {4, 40}},
    FontSize->12]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsubsection"],
    CellDingbat->"\[FilledSquare]",
    CellMargins->{{25, Inherited}, {8, 12}},
    CellGroupingRules->{"SectionGrouping", 50},
    PageBreakBelow->False,
    CounterIncrements->"Subsubsection",
    FontFamily->"Times",
    FontSize->13,
    FontWeight->"Bold"],
  
  Cell[StyleData["Subsubsection", "Presentation"],
    CellMargins->{{29, 10}, {8, 26}},
    LineSpacing->{1, 0},
    FontSize->18],
  
  Cell[StyleData["Subsubsection", "Condensed"],
    CellMargins->{{21, 0}, {4, 20}},
    FontSize->11],
  
  Cell[StyleData["Subsubsection", "Printout"],
    CellMargins->{{21, 0}, {4, 20}},
    FontSize->11]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Body Text", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Text"],
    CellMargins->{{12, 10}, {5, 5}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 12},
    CounterIncrements->"Text",
    FontFamily->"Times"],
  
  Cell[StyleData["Text", "Presentation"],
    CellMargins->{{13, 10}, {8, 8}},
    LineSpacing->{1, 5},
    ParagraphSpacing->{0, 12},
    FontSize->16],
  
  Cell[StyleData["Text", "Condensed"],
    CellMargins->{{9, 0}, {4, 4}},
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 6}],
  
  Cell[StyleData["Text", "Printout"],
    CellMargins->{{9, 0}, {4, 4}},
    ParagraphSpacing->{0, 6}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["SmallText"],
    CellMargins->{{12, 10}, {5, 5}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 6},
    CounterIncrements->"SmallText",
    FontFamily->"Helvetica",
    FontSize->9],
  
  Cell[StyleData["SmallText", "Presentation"],
    CellMargins->{{13, 10}, {8, 8}},
    LineSpacing->{1, 5},
    FontSize->12],
  
  Cell[StyleData["SmallText", "Condensed"],
    CellMargins->{{8, 10}, {2, 2}},
    LineSpacing->{1, 2},
    FontSize->9],
  
  Cell[StyleData["SmallText", "Printout"],
    CellMargins->{{9, 0}, {4, 4}}]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Input/Output", "Section"],
  
  Cell["\<\
The cells in this section define styles used for input and output to the \
kernel.  Be careful when modifying, renaming, or removing these styles, \
because the front end associates special meanings with these style names.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Input"],
    CellFrame->0.5,
    CellMargins->{{25, 10}, {8, 8}},
    Evaluatable->True,
    CellGroupingRules->"InputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{5, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultInputFormatType,
    FormatType->InputForm,
    ShowStringCharacters->True,
    NumberMarks->True,
    CounterIncrements->"Input",
    FontWeight->"Bold",
    Background->RGBColor[1, 0.700008, 0.4]],
  
  Cell[StyleData["Input", "Presentation"],
    CellMargins->{{25, Inherited}, {10, 10}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Input", "Condensed"],
    CellMargins->{{25, 0}, {6, 6}}],
  
  Cell[StyleData["Input", "Printout"],
    CellFrame->0.5,
    CellMargins->{{25, 0}, {6, 6}},
    Background->None]
  }, Closed]],
  
  Cell[StyleData["InlineInput"],
    CellGroupingRules->"InputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    DefaultFormatType->DefaultInputFormatType,
    AutoItalicWords->{},
    FormatType->InputForm,
    ShowStringCharacters->True,
    NumberMarks->True,
    CounterIncrements->"Input",
    FontWeight->"Bold"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Output"],
    CellFrame->True,
    CellMargins->{{25, 10}, {8, 0}},
    CellElementSpacings->{"ClosedGroupTopMargin"->4},
    CellEditDuplicate->True,
    CellGroupingRules->"OutputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    CellLabelMargins->{{3, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Output",
    Background->RGBColor[1, 0.900008, 0.900008]],
  
  Cell[StyleData["Output", "Presentation"],
    CellMargins->{{25, Inherited}, {12, 0}},
    CellElementSpacings->{"ClosedGroupTopMargin"->4},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Output", "Condensed"],
    CellMargins->{{25, 0}, {6, 0}},
    CellElementSpacings->{"ClosedGroupTopMargin"->4}],
  
  Cell[StyleData["Output", "Printout"],
    CellMargins->{{25, 0}, {6, 0}},
    CellElementSpacings->{"ClosedGroupTopMargin"->4},
    Background->None]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Message"],
    CellMargins->{{62, Inherited}, {Inherited, Inherited}},
    CellGroupingRules->"OutputGrouping",
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Message",
    StyleMenuListing->None,
    FontColor->RGBColor[1, 0, 0]],
  
  Cell[StyleData["Message", "Presentation"],
    CellMargins->{{74, Inherited}, {Inherited, Inherited}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Message", "Condensed"],
    CellMargins->{{50, Inherited}, {Inherited, Inherited}}],
  
  Cell[StyleData["Message", "Printout"],
    CellMargins->{{54, Inherited}, {Inherited, Inherited}},
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Print"],
    CellMargins->{{62, Inherited}, {Inherited, Inherited}},
    CellGroupingRules->"OutputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Print",
    StyleMenuListing->None],
  
  Cell[StyleData["Print", "Presentation"],
    CellMargins->{{74, Inherited}, {Inherited, Inherited}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Print", "Condensed"],
    CellMargins->{{50, Inherited}, {Inherited, Inherited}}],
  
  Cell[StyleData["Print", "Printout"],
    CellMargins->{{54, Inherited}, {Inherited, Inherited}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Graphics"],
    CellMargins->{{62, Inherited}, {Inherited, Inherited}},
    CellGroupingRules->"GraphicsGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Graphics",
    StyleMenuListing->None],
  
  Cell[StyleData["Graphics", "Presentation"],
    CellMargins->{{74, Inherited}, {Inherited, Inherited}}],
  
  Cell[StyleData["Graphics", "Condensed"],
    CellMargins->{{52, Inherited}, {Inherited, Inherited}},
    ImageSize->{175, 175}],
  
  Cell[StyleData["Graphics", "Printout"],
    CellMargins->{{54, Inherited}, {Inherited, Inherited}},
    ImageSize->{250, 250}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["CellLabel"],
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->11,
    FontWeight->"Bold",
    FontColor->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["CellLabel", "Presentation"],
    FontSize->12],
  
  Cell[StyleData["CellLabel", "Condensed"],
    FontSize->8],
  
  Cell[StyleData["CellLabel", "Printout"],
    FontSize->8,
    FontColor->GrayLevel[0]]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Unique Styles", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Author"],
    CellMargins->{{10, 4}, {2, 10}},
    FontSize->12,
    FontSlant->"Italic"],
  
  Cell[StyleData["Author", "Presentation"],
    CellMargins->{{12, 10}, {2, 12}},
    LineSpacing->{1, 5},
    ParagraphSpacing->{0, 12},
    FontSize->16],
  
  Cell[StyleData["Author", "Condensed"],
    CellMargins->{{8, 10}, {1, 4}},
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 4},
    FontSize->9],
  
  Cell[StyleData["Author", "Printout"],
    CellMargins->{{9, 0}, {4, 12}},
    ParagraphSpacing->{0, 6}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Copyright"],
    CellMargins->{{10, 10}, {40, 2}},
    FontFamily->"Helvetica",
    FontSize->9],
  
  Cell[StyleData["Copyright", "Presentation"],
    CellMargins->{{12, 10}, {50, 2}},
    LineSpacing->{1, 5},
    FontSize->12],
  
  Cell[StyleData["Copyright", "Condensed"],
    CellMargins->{{8, 10}, {12, 1}},
    LineSpacing->{1, 2},
    FontSize->9],
  
  Cell[StyleData["Copyright", "Printout"],
    CellMargins->{{9, 0}, {72, 4}},
    FontSize->7]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Abstract"],
    CellMargins->{{45, 75}, {Inherited, 30}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Abstract", "Printout"],
    CellMargins->{{36, 67}, {Inherited, 50}},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Example"],
    CellMargins->{{12, 10}, {5, 12}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 12},
    CounterIncrements->"Example",
    FontFamily->"Times",
    FontWeight->"Bold"],
  
  Cell[StyleData["Example", "Presentation"],
    CellMargins->{{18, 10}, {8, 20}},
    LineSpacing->{1, 5},
    ParagraphSpacing->{0, 12}],
  
  Cell[StyleData["Example", "Condensed"],
    CellMargins->{{8, 10}, {4, 8}},
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 4}],
  
  Cell[StyleData["Example", "Printout"],
    CellMargins->{{9, 0}, {4, 10}},
    ParagraphSpacing->{0, 6}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Definition"],
    CellFrame->{{3, 0}, {0, 0}},
    CellMargins->{{52, 10}, {8, 8}},
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    ShowStringCharacters->True,
    CounterIncrements->"Definition",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->GrayLevel[1],
    Background->RGBColor[0.2, 0.700008, 0.700008]],
  
  Cell[StyleData["Definition", "Presentation"],
    CellMargins->{{62, Inherited}, {5, 12}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Definition", "Condensed"],
    CellMargins->{{40, 10}, {1, 4}}],
  
  Cell[StyleData["Definition", "Printout"],
    CellMargins->{{44, 0}, {2, 6}},
    Background->GrayLevel[0.6]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Theorem"],
    CellFrame->{{3, 0}, {0, 0}},
    CellMargins->{{52, 10}, {8, 8}},
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    ShowStringCharacters->True,
    CounterIncrements->"Theorem",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->GrayLevel[1],
    Background->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["Theorem", "Presentation"],
    CellMargins->{{62, Inherited}, {5, 12}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Theorem", "Condensed"],
    CellMargins->{{40, 10}, {1, 4}}],
  
  Cell[StyleData["Theorem", "Printout"],
    CellMargins->{{44, 0}, {2, 6}},
    Background->GrayLevel[0.4]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Caution"],
    CellFrame->False,
    CellDingbat->"\[WarningSign]",
    CellMargins->{{52, 25}, {8, 8}},
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    ShowStringCharacters->True,
    CounterIncrements->"Caution",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontSlant->"Plain",
    FontColor->GrayLevel[0],
    Background->RGBColor[0.996109, 0, 0],
    FontVariations->{"Underline"->False}],
  
  Cell[StyleData["Caution", "Presentation"],
    CellFrame->False,
    CellDingbat->"\[WarningSign]",
    CellMargins->{{62, Inherited}, {5, 12}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Caution", "Condensed"],
    CellFrame->False,
    CellDingbat->"\[WarningSign]",
    CellMargins->{{40, 25}, {1, 4}}],
  
  Cell[StyleData["Caution", "Printout"],
    CellFrame->False,
    CellDingbat->"\[WarningSign]",
    CellMargins->{{44, 25}, {2, 6}},
    Background->GrayLevel[0.900008]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Advice"],
    CellFrame->False,
    CellDingbat->"\[LightBulb]",
    CellMargins->{{52, 25}, {8, 8}},
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    ShowStringCharacters->True,
    CounterIncrements->"Advice",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontSlant->"Plain",
    FontColor->GrayLevel[0],
    Background->RGBColor[0, 1, 0],
    FontVariations->{"Underline"->False}],
  
  Cell[StyleData["Advice", "Presentation"],
    CellFrame->False,
    CellDingbat->"\[LightBulb]",
    CellMargins->{{62, Inherited}, {5, 12}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Advice", "Condensed"],
    CellFrame->False,
    CellDingbat->"\[LightBulb]",
    CellMargins->{{40, 25}, {1, 4}}],
  
  Cell[StyleData["Advice", "Printout"],
    CellFrame->False,
    CellDingbat->"\[LightBulb]",
    CellMargins->{{44, 25}, {2, 6}},
    Background->GrayLevel[0.900008]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Exercise"],
    CellDingbat->"\[FilledDownTriangle]",
    CellMargins->{{23, Inherited}, {4, 18}},
    CellGroupingRules->"NormalGrouping",
    PageBreakBelow->False,
    CounterIncrements->"Subsubsection",
    FontFamily->"Times",
    FontSize->13,
    FontWeight->"Bold",
    FontColor->RGBColor[0, 0, 0.996109]],
  
  Cell[StyleData["Exercise", "Presentation"],
    CellMargins->{{33, 10}, {8, 26}},
    LineSpacing->{1, 0},
    FontSize->18],
  
  Cell[StyleData["Exercise", "Condensed"],
    CellMargins->{{9, 0}, {6, 20}},
    FontSize->11],
  
  Cell[StyleData["Exercise", "Printout"],
    CellFrame->False,
    CellDingbat->"\[FilledDownTriangle]",
    CellMargins->{{9, 0}, {6, 20}},
    FontSize->11,
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ExerciseMain"],
    CellFrame->{{6, 0}, {0, 1}},
    CellMargins->{{12, Inherited}, {4, 20}},
    CellGroupingRules->{"SectionGrouping", 30},
    PageBreakBelow->False,
    CellFrameColor->RGBColor[0.571389, 0.19675, 0.570504],
    CounterIncrements->"Subsection",
    CounterAssignments->{{"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->16,
    FontWeight->"Bold",
    FontColor->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["ExerciseMain", "Presentation"],
    CellMargins->{{18, 10}, {8, 32}},
    LineSpacing->{1, 2},
    FontSize->24,
    FontTracking->"Condensed"],
  
  Cell[StyleData["ExerciseMain", "Condensed"],
    CellMargins->{{9, 0}, {2, 50}},
    FontSize->14],
  
  Cell[StyleData["ExerciseMain", "Printout"],
    CellMargins->{{9, 0}, {2, 50}},
    CellFrameColor->GrayLevel[0.500008],
    FontSize->14,
    FontTracking->"Plain",
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ExerciseTitle"],
    CellDingbat->"\[FilledDownTriangle]",
    CellMargins->{{23, Inherited}, {4, 18}},
    CellGroupingRules->{"SectionGrouping", 40},
    PageBreakBelow->False,
    CounterIncrements->"Subsection",
    CounterAssignments->{{"Subsubsection", 0}},
    FontFamily->"Times",
    FontSize->13,
    FontWeight->"Bold",
    FontColor->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["ExerciseTitle", "Presentation"],
    CellMargins->{{33, 10}, {8, 26}},
    LineSpacing->{1, 0},
    FontSize->18],
  
  Cell[StyleData["ExerciseTitle", "Condensed"],
    CellDingbat->"\[FilledDownTriangle]",
    CellMargins->{{9, 0}, {6, 20}},
    FontSize->11],
  
  Cell[StyleData["ExerciseTitle", "Printout"],
    CellDingbat->"\[FilledDownTriangle]",
    CellMargins->{{9, 0}, {6, 20}},
    FontSize->11,
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ExerciseText"],
    CellMargins->{{32, 10}, {5, 5}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 8},
    CounterIncrements->"ExcersiceText",
    FontFamily->"Times"],
  
  Cell[StyleData["ExerciseText", "Presentation"],
    CellMargins->{{32, 10}, {8, 8}},
    LineSpacing->{1, 5},
    ParagraphSpacing->{0, 12}],
  
  Cell[StyleData["ExerciseText", "Condensed"],
    CellMargins->{{32, 0}, {4, 4}},
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 6}],
  
  Cell[StyleData["ExerciseText", "Printout"],
    CellMargins->{{32, 0}, {4, 4}},
    ParagraphSpacing->{0, 6}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Problem"],
    CellDingbat->"\[FilledDownTriangle]",
    CellMargins->{{23, Inherited}, {4, 18}},
    CellGroupingRules->{"SectionGrouping", 40},
    PageBreakBelow->False,
    CounterIncrements->"Subsection",
    CounterAssignments->{{"Subsubsection", 0}},
    FontFamily->"Times",
    FontSize->13,
    FontWeight->"Plain",
    FontColor->GrayLevel[0]],
  
  Cell[StyleData["Problem", "Presentation"],
    CellMargins->{{33, 10}, {8, 26}},
    LineSpacing->{1, 0},
    FontSize->18],
  
  Cell[StyleData["Problem", "Condensed"],
    CellMargins->{{9, 0}, {6, 20}},
    FontSize->11],
  
  Cell[StyleData["Problem", "Printout"],
    CellFrame->False,
    CellDingbat->"\[FilledDownTriangle]",
    CellMargins->{{9, 0}, {6, 20}},
    FontSize->11,
    FontColor->GrayLevel[0]]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Formulas and Programming", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["DisplayFormula"],
    CellMargins->{{62, 10}, {2, 10}},
    CellHorizontalScrolling->True,
    DefaultFormatType->DefaultInputFormatType,
    ScriptLevel->0,
    SingleLetterItalics->True,
    UnderoverscriptBoxOptions->{LimitsPositioning->True}],
  
  Cell[StyleData["DisplayFormula", "Presentation"],
    CellMargins->{{74, 10}, {2, 10}}],
  
  Cell[StyleData["DisplayFormula", "Condensed"],
    CellMargins->{{52, 10}, {2, 10}}],
  
  Cell[StyleData["DisplayFormula", "Printout"],
    CellMargins->{{54, 10}, {2, 10}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ChemicalFormula"],
    CellMargins->{{62, 10}, {2, 10}},
    DefaultFormatType->DefaultInputFormatType,
    AutoSpacing->False,
    ScriptLevel->1,
    ScriptBaselineShifts->{0.6, Automatic},
    SingleLetterItalics->False,
    ZeroWidthTimes->True],
  
  Cell[StyleData["ChemicalFormula", "Presentation"],
    CellMargins->{{74, 10}, {2, 10}},
    FontSize->10],
  
  Cell[StyleData["ChemicalFormula", "Condensed"],
    CellMargins->{{52, 10}, {2, 10}},
    FontSize->10],
  
  Cell[StyleData["ChemicalFormula", "Printout"],
    CellMargins->{{54, 10}, {2, 10}},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Program"],
    CellMargins->{{12, 10}, {Inherited, 6}},
    FontFamily->"Courier"],
  
  Cell[StyleData["Program", "Presentation"],
    CellMargins->{{13, 30}, {Inherited, 4}},
    FontSize->9.5],
  
  Cell[StyleData["Program", "Condensed"],
    CellMargins->{{8, 10}, {Inherited, 4}},
    FontSize->9.5],
  
  Cell[StyleData["Program", "Printout"],
    CellMargins->{{9, 0}, {Inherited, 4}},
    FontSize->9.5]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Automatic Numbering", "Section"],
  
  Cell["\<\
The following styles are useful for numbered equations, figures, etc.  They \
automatically give the cell a FrameLabel containing a reference to a \
particular counter, and also increment that counter.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["NumberedEquation"],
    CellMargins->{{62, 10}, {Inherited, Inherited}},
    CellFrameLabels->{{None, Cell[ 
            TextData[ {"(", 
              CounterBox[ "NumberedEquation"], ")"}]]}, {None, None}},
    DefaultFormatType->DefaultInputFormatType,
    CounterIncrements->"NumberedEquation",
    FormatTypeAutoConvert->False],
  
  Cell[StyleData["NumberedEquation", "Presentation"],
    CellMargins->{{74, 10}, {Inherited, Inherited}}],
  
  Cell[StyleData["NumberedEquation", "Condensed"],
    CellMargins->{{52, 10}, {Inherited, Inherited}}],
  
  Cell[StyleData["NumberedEquation", "Printout"],
    CellMargins->{{54, 0}, {Inherited, Inherited}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["NumberedFigure"],
    CellMargins->{{62, 145}, {Inherited, Inherited}},
    CellFrameLabels->{{None, None}, {Cell[ 
            TextData[ {"Figure ", 
              CounterBox[ "NumberedFigure"]}]], None}},
    CounterIncrements->"NumberedFigure",
    ImageMargins->{{43, Inherited}, {Inherited, 0}},
    FormatTypeAutoConvert->False],
  
  Cell[StyleData["NumberedFigure", "Presentation"]],
  
  Cell[StyleData["NumberedFigure", "Condensed"]],
  
  Cell[StyleData["NumberedFigure", "Printout"]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["NumberedTable"],
    CellMargins->{{62, 145}, {Inherited, Inherited}},
    CellFrameLabels->{{None, None}, {Cell[ 
            TextData[ {"Table ", 
              CounterBox[ "NumberedTable"]}]], None}},
    TextAlignment->Center,
    CounterIncrements->"NumberedTable",
    FormatTypeAutoConvert->False],
  
  Cell[StyleData["NumberedTable", "Presentation"]],
  
  Cell[StyleData["NumberedTable", "Condensed"]],
  
  Cell[StyleData["NumberedTable", "Printout"]]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Headers and Footers", "Section"],
  
  Cell[StyleData["Header"],
    CellMargins->{{0, 0}, {4, 1}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->9,
    FontSlant->"Italic"],
  
  Cell[StyleData["Footer"],
    CellMargins->{{0, 0}, {0, 4}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->9,
    FontSlant->"Italic"],
  
  Cell[StyleData["PageNumber"],
    CellMargins->{{0, 0}, {4, 1}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->9,
    FontWeight->"Bold"]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Hyperlink Styles", "Section"],
  
  Cell["\<\
The cells below define styles useful for making hypertext ButtonBoxes.  The \
\"Hyperlink\" style is for links within the same Notebook, or between \
Notebooks.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Hyperlink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontWeight->"Bold",
    FontSlant->"Italic",
    FontColor->RGBColor[0, 0, 1],
    Background->None,
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`NotebookLocate[ #2]}]&),
    Active->True,
    ButtonFrame->"None",
    ButtonNote->ButtonData}],
  
  Cell[StyleData["Hyperlink", "Presentation"]],
  
  Cell[StyleData["Hyperlink", "Condensed"]],
  
  Cell[StyleData["Hyperlink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1],
    FontVariations->{"Underline"->True}]
  }, Closed]],
  
  Cell["\<\
The following styles are for linking automatically to the on-line help \
system.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["MainBookLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontWeight->"Bold",
    FontSlant->"Plain",
    FontColor->RGBColor[0, 0, 1],
    Background->None,
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "MainBook", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["MainBookLink", "Presentation"]],
  
  Cell[StyleData["MainBookLink", "Condensed"]],
  
  Cell[StyleData["MainBookLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1],
    FontVariations->{"Underline"->True}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["AddOnsLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontFamily->"Courier",
    FontWeight->"Bold",
    FontSlant->"Plain",
    FontColor->RGBColor[0, 0, 1],
    Background->None,
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "AddOns", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["AddOnsLink", "Presentation"]],
  
  Cell[StyleData["AddOnsLink", "Condensed"]],
  
  Cell[StyleData["AddOnLink", "Printout"],
    FontWeight->"Bold",
    FontColor->GrayLevel[0],
    Background->GrayLevel[1],
    FontVariations->{"Underline"->True}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["RefGuideLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontFamily->"Courier",
    FontWeight->"Bold",
    FontSlant->"Plain",
    FontColor->RGBColor[0, 0, 1],
    Background->None,
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "RefGuideLink", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["RefGuideLink", "Presentation"]],
  
  Cell[StyleData["RefGuideLink", "Condensed"]],
  
  Cell[StyleData["RefGuideLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1],
    FontVariations->{"Underline"->True}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["GettingStartedLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontWeight->"Bold",
    FontSlant->"Italic",
    FontColor->RGBColor[0, 0, 1],
    Background->None,
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "GettingStarted", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["GettingStartedLink", "Presentation"]],
  
  Cell[StyleData["GettingStartedLink", "Condensed"]],
  
  Cell[StyleData["GettingStartedLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1],
    FontVariations->{"Underline"->True}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["OtherInformationLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontWeight->"Bold",
    FontSlant->"Italic",
    FontColor->RGBColor[0, 0, 1],
    Background->None,
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "OtherInformation", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["OtherInformationLink", "Presentation"]],
  
  Cell[StyleData["OtherInformationLink", "Condensed"]],
  
  Cell[StyleData["OtherInformationLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1],
    FontVariations->{"Underline"->True}]
  }, Closed]]
  }, Closed]]
  }, Open  ]]
  }]
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 80, 2, 81, "Title"],
Cell[1859, 57, 455, 15, 85, "Author"],

Cell[CellGroupData[{
Cell[2339, 76, 31, 0, 83, "Section"],
Cell[2373, 78, 518, 8, 93, "Text"],
Cell[2894, 88, 122, 3, 41, "Text"],
Cell[3019, 93, 198, 4, 68, "DisplayFormula"],
Cell[3220, 99, 263, 7, 41, "Text"],
Cell[3486, 108, 287, 6, 97, "DisplayFormula"],
Cell[3776, 116, 220, 4, 41, "Text"],
Cell[3999, 122, 119, 3, 96, "Input"],
Cell[4121, 127, 41, 0, 41, "Text"],

Cell[CellGroupData[{
Cell[4187, 131, 71, 1, 73, "Input"],
Cell[4261, 134, 102, 2, 76, "Output"]
}, Closed]],
Cell[4378, 139, 386, 8, 67, "Text"],
Cell[4767, 149, 380, 6, 96, "Input"],
Cell[5150, 157, 422, 7, 93, "Text"],
Cell[5575, 166, 1418, 24, 198, "Text"],
Cell[6996, 192, 949, 18, 145, "Text"],
Cell[7948, 212, 365, 9, 36, "InlineInput"],
Cell[8316, 223, 296, 5, 67, "Text"],
Cell[8615, 230, 70, 0, 41, "Text"],
Cell[8688, 232, 151, 4, 64, "InlineInput"],
Cell[8842, 238, 934, 20, 119, "Text"],
Cell[9779, 260, 274, 7, 172, "InlineInput"],
Cell[10056, 269, 309, 8, 67, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[10402, 282, 33, 0, 51, "Section"],
Cell[10438, 284, 296, 5, 151, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[10771, 294, 33, 0, 51, "Section"],
Cell[10807, 296, 542, 8, 93, "Text"],
Cell[11352, 306, 464, 8, 59, "InlineInput"],
Cell[11819, 316, 62, 0, 41, "Text"],
Cell[11884, 318, 1665, 30, 334, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[13586, 353, 70, 0, 51, "Section"],
Cell[13659, 355, 436, 7, 93, "Text"],

Cell[CellGroupData[{
Cell[14120, 366, 42, 0, 68, "Subsection"],
Cell[14165, 368, 124, 3, 41, "Text"],
Cell[14292, 373, 215, 3, 125, "Input"],
Cell[14510, 378, 74, 0, 41, "Text"],
Cell[14587, 380, 322, 6, 73, "Input"],
Cell[14912, 388, 170, 3, 41, "Text"],
Cell[15085, 393, 346, 6, 73, "Input"],
Cell[15434, 401, 130, 3, 41, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[15601, 409, 47, 0, 68, "Subsection"],
Cell[15651, 411, 277, 6, 41, "Text"],
Cell[15931, 419, 54, 1, 73, "Input"],

Cell[CellGroupData[{
Cell[16010, 424, 89, 1, 74, "Input"],
Cell[16102, 427, 173, 3, 35, "Print"],
Cell[16278, 432, 189, 4, 35, "Print"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[16516, 442, 49, 0, 68, "Subsection"],
Cell[16568, 444, 91, 2, 41, "Text"],

Cell[CellGroupData[{
Cell[16684, 450, 165, 3, 73, "Input"],
Cell[16852, 455, 38747, 928, 279, 13235, 608, "GraphicsData", "PostScript", \
"Graphics"]
}, Closed]],

Cell[CellGroupData[{
Cell[55636, 1388, 142, 3, 73, "Input"],
Cell[55781, 1393, 43637, 1049, 279, 15108, 692, "GraphicsData", "PostScript", \
"Graphics"]
}, Closed]],
Cell[99433, 2445, 53, 0, 41, "Text"],

Cell[CellGroupData[{
Cell[99511, 2449, 79, 1, 73, "Input"],
Cell[99593, 2452, 1425201, 28374, 609, 802916, 20687, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[1524809, 30829, 127, 3, 41, "Text"],
Cell[1524939, 30834, 67, 1, 73, "Input"],

Cell[CellGroupData[{
Cell[1525031, 30839, 352, 5, 125, "Input"],
Cell[1525386, 30846, 74207, 2231, 444, 25532, 1625, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[1599608, 33080, 157, 5, 42, "Text"],

Cell[CellGroupData[{
Cell[1599790, 33089, 264, 4, 99, "Input"],
Cell[1600057, 33095, 70192, 1923, 427, 31914, 1446, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1670298, 35024, 33, 0, 68, "Subsection"],
Cell[1670334, 35026, 144, 3, 41, "Text"],
Cell[1670481, 35031, 222, 4, 62, "DisplayFormula"],
Cell[1670706, 35037, 780, 22, 67, "Text"],
Cell[1671489, 35061, 335, 6, 92, "DisplayFormula"],
Cell[1671827, 35069, 148, 5, 41, "Text"],
Cell[1671978, 35076, 457, 7, 93, "Text"],

Cell[CellGroupData[{
Cell[1672460, 35087, 391, 6, 125, "Input"],
Cell[1672854, 35095, 72686, 1961, 427, 32022, 1454, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[1745555, 37059, 141, 2, 99, "Input"],

Cell[CellGroupData[{
Cell[1745721, 37065, 194, 3, 92, "Input"],
Cell[1745918, 37070, 73, 1, 59, "Output"]
}, Closed]],
Cell[1746006, 37074, 1646, 34, 197, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[1747701, 37114, 54, 0, 51, "Section"],
Cell[1747758, 37116, 781, 11, 145, "Text"],
Cell[1748542, 37129, 1306, 18, 223, "Text"],
Cell[1749851, 37149, 1076, 29, 121, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[1750964, 37183, 50, 0, 51, "Section"],
Cell[1751017, 37185, 466, 8, 94, "Text"],
Cell[1751486, 37195, 1421, 24, 400, "Input"],

Cell[CellGroupData[{
Cell[1752932, 37223, 45, 0, 68, "Subsection"],
Cell[1752980, 37225, 48, 0, 41, "Text"],
Cell[1753031, 37227, 227, 3, 125, "Input"],

Cell[CellGroupData[{
Cell[1753283, 37234, 205, 4, 99, "Input"],
Cell[1753491, 37240, 17954, 262, 290, 673, 44, "GraphicsData", "PostScript", \
"Graphics"]
}, Closed]],
Cell[1771460, 37505, 98, 2, 41, "Text"],
Cell[1771561, 37509, 405, 7, 125, "Input"],

Cell[CellGroupData[{
Cell[1771991, 37520, 89, 1, 74, "Input"],
Cell[1772083, 37523, 173, 3, 35, "Print"],
Cell[1772259, 37528, 189, 4, 35, "Print"]
}, Closed]],
Cell[1772463, 37535, 67, 1, 73, "Input"],

Cell[CellGroupData[{
Cell[1772555, 37540, 316, 5, 99, "Input"],
Cell[1772874, 37547, 119844, 2900, 444, 27656, 1757, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],

Cell[CellGroupData[{
Cell[1892755, 40452, 218, 3, 99, "Input"],
Cell[1892976, 40457, 89410, 2162, 426, 31662, 1444, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],

Cell[CellGroupData[{
Cell[1982423, 42624, 531, 8, 151, "Input"],
Cell[1982957, 42634, 174815, 4574, 444, 58121, 3129, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[2157787, 47211, 285, 5, 67, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2158121, 47222, 56, 0, 51, "Section"],
Cell[2158180, 47224, 747, 17, 93, "Text"],

Cell[CellGroupData[{
Cell[2158952, 47245, 229, 4, 125, "Input"],
Cell[2159184, 47251, 43737, 746, 279, 3680, 247, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[2202936, 48000, 87, 2, 41, "Text"],
Cell[2203026, 48004, 227, 3, 125, "Input"],
Cell[2203256, 48009, 168, 3, 73, "Input"],

Cell[CellGroupData[{
Cell[2203449, 48016, 139, 2, 73, "Input"],
Cell[2203591, 48020, 65603, 1226, 279, 8767, 520, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[2269209, 49249, 466, 7, 93, "Text"],
Cell[2269678, 49258, 76, 0, 41, "Text"],

Cell[CellGroupData[{
Cell[2269779, 49262, 431, 8, 151, "Input"],
Cell[2270213, 49272, 173, 3, 35, "Print"],
Cell[2270389, 49277, 190, 4, 35, "Print"]
}, Closed]],

Cell[CellGroupData[{
Cell[2270616, 49286, 316, 5, 99, "Input"],
Cell[2270935, 49293, 118209, 3811, 444, 45485, 2908, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],

Cell[CellGroupData[{
Cell[2389181, 53109, 531, 8, 151, "Input"],
Cell[2389715, 53119, 190244, 5751, 457, 77431, 4354, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[2579974, 58873, 247, 4, 67, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[2580258, 58882, 63, 0, 51, "Section"],
Cell[2580324, 58884, 362, 6, 67, "Text"],

Cell[CellGroupData[{
Cell[2580711, 58894, 439, 7, 154, "Input"],
Cell[2581153, 58903, 36494, 578, 444, 2468, 153, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[2617662, 59484, 195, 4, 41, "Text"],
Cell[2617860, 59490, 227, 3, 125, "Input"],
Cell[2618090, 59495, 846, 15, 177, "Input"],

Cell[CellGroupData[{
Cell[2618961, 59514, 179, 4, 100, "Input"],
Cell[2619143, 59520, 173, 3, 35, "Print"],
Cell[2619319, 59525, 190, 4, 35, "Print"]
}, Closed]],

Cell[CellGroupData[{
Cell[2619546, 59534, 391, 6, 125, "Input"],
Cell[2619940, 59542, 94549, 2977, 482, 34962, 2237, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]],
Cell[2714504, 62522, 167, 3, 41, "Text"],

Cell[CellGroupData[{
Cell[2714696, 62529, 261, 4, 99, "Input"],
Cell[2714960, 62535, 67429, 1975, 448, 33533, 1552, "GraphicsData", \
"PostScript", "Graphics"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[2782438, 64516, 33, 0, 54, "ExerciseMain"],

Cell[CellGroupData[{
Cell[2782496, 64520, 72, 0, 58, "ExerciseTitle"],
Cell[2782571, 64522, 779, 19, 93, "ExerciseText"]
}, Closed]],

Cell[CellGroupData[{
Cell[2783387, 64546, 133, 4, 35, "ExerciseTitle"],
Cell[2783523, 64552, 311, 7, 67, "ExerciseText"]
}, Closed]],

Cell[CellGroupData[{
Cell[2783871, 64564, 43, 0, 35, "ExerciseTitle"],
Cell[2783917, 64566, 576, 12, 93, "ExerciseText"]
}, Closed]],

Cell[CellGroupData[{
Cell[2784530, 64583, 41, 0, 35, "ExerciseTitle"],
Cell[2784574, 64585, 96, 2, 41, "ExerciseText"],
Cell[2784673, 64589, 210, 5, 68, "DisplayFormula"],
Cell[2784886, 64596, 241, 8, 41, "ExerciseText"]
}, Closed]],

Cell[CellGroupData[{
Cell[2785164, 64609, 70, 0, 35, "ExerciseTitle"],
Cell[2785237, 64611, 767, 13, 119, "ExerciseText"],
Cell[2786007, 64626, 243, 5, 64, "DisplayFormula"],
Cell[2786253, 64633, 140, 3, 41, "ExerciseText"],
Cell[2786396, 64638, 75, 0, 41, "ExerciseText"],
Cell[2786474, 64640, 250, 4, 67, "ExerciseText"]
}, Closed]]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

